      SUBROUTINE GCR( R, U, NEQ, INNER )

      USE control_vectors
c
C**** *GCR*   - Routine to determine inverse of operator 
C                  
C
C     Purpose.
C     --------
C        Given vector r, the vector u so that OPM(u)=r is approximated 
C        using the conjugate residual method.
C
C**   Interface.
C     ----------
C        ??????
C
C        Explicit arguments :
C        --------------------
C        None
C
C        Implicit arguments :
C        --------------------
C        None
C
C     Method.
C     -------
C        See documentation
C
C     Externals.
C     ----------
C     None
C
C     Reference.
C     ----------
C        See documentation 
C        
C
C     Author.
C     -------
C        Martin van Gijzen *Utrecht University*
C
C     Modifications.
C     --------------
C        Original : 95-10-5
C
c**********************************************************************
c     
      INTEGER          NEQ, INNER
      REAL             R(NEQ), U(NEQ), P(NEQ), Q(NEQ), CK(NEQ), UK(NEQ)
C
      INTEGER          ITS, I
      REAL             ALPHA, BETA, CT
c
c...     blas routines
      REAL             DDOT, DNRM2

      TYPE (control_vector) :: ZVAZY,ZVAZH
c
c...
      DO I = 1, NEQ
        U(I) = 0.
      ENDDO
c
      DO ITS = 1, INNER
c
c...        Preconditioning can be performed here: take
C...                 q = ( I-BuuH )P( I-uuHB )r
c...        with P some approximation for the inverse of ( A - evB )
c
        CALL DCOPY( NEQ, R, 1, Q, 1 )

        CALL ALLOCATE_CTLVEC(ZVAZY)
        CALL ALLOCATE_CTLVEC(ZVAZH)

        ZVAZY = Q(:)
        CALL OPM(ZVAZY,ZVAZH)
        CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH,NEQ,P(:))

        CALL DEALLOCATE_CTLVEC(ZVAZY)
        CALL DEALLOCATE_CTLVEC(ZVAZH)

          IF ( ITS .GT. 1 ) THEN
            ALPHA = DDOT( NEQ, CK, 1, P, 1 )
            CALL DAXPY( NEQ, -ALPHA, CK, 1, P, 1 )
            CALL DAXPY( NEQ, -ALPHA, UK, 1, Q, 1 )
          END IF
c
        BETA = DNRM2( NEQ, P, 1 )
C
          IF ( BETA .LE. 0. ) THEN
            BETA = 0.
          ELSE
            BETA = 1. / BETA
          END IF
C
        CALL DSCAL( NEQ, BETA, P, 1 )
        CALL DSCAL( NEQ, BETA, Q, 1 )
C
        CALL DCOPY( NEQ, P, 1, CK, 1 )
        CALL DCOPY( NEQ, Q, 1, UK, 1 )
C
        BETA = DDOT( NEQ, R, 1, P, 1 )
        CALL DAXPY( NEQ,  BETA, Q, 1, U, 1 )
        CALL DAXPY( NEQ, -BETA, P, 1, R, 1 )
C
      ENDDO
C
      END
