SUBROUTINE MULTVDV(PVEC,PDIA,PROD)

!     Purpose.
!     --------
!       Compute A=VDV^t

!     Author.
!     -------
!       Yannick Tremolet

!     Modifications.
!     --------------
!       Original  : 12-May-2004
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN)  :: PVEC(:,:)
REAL(KIND=JPRB), INTENT(IN)  :: PDIA(:)
REAL(KIND=JPRB), INTENT(OUT) :: PROD(:,:)

INTEGER(KIND=JPIM) :: ilen,ji,jj,jk
REAL(KIND=JPRB), ALLOCATABLE :: ZZ(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('MULTVDV',0,ZHOOK_HANDLE)

ilen=SIZE(PDIA)
IF (SIZE(PVEC,1)/=ilen) CALL ABOR1('MULVDV: Error size PVEC 1')
IF (SIZE(PVEC,2)/=ilen) CALL ABOR1('MULVDV: Error size PVEC 2')
IF (SIZE(PROD,1)/=ilen) CALL ABOR1('MULVDV: Error size PROD 1')
IF (SIZE(PROD,2)/=ilen) CALL ABOR1('MULVDV: Error size PROD 2')

ALLOCATE(ZZ(ilen,ilen))
ZZ(:,:)=0.0_JPRB

DO jj=1,ilen
  DO ji=1,ilen
    ZZ(ji,jj)=ZZ(ji,jj)+PVEC(ji,jj)*PDIA(jj)
  ENDDO
ENDDO

PROD(:,:)=0.0_JPRB

! Diagonal and above
DO jk=1,ilen
  DO jj=1,ilen
    DO ji=1,jj
      PROD(ji,jj)=PROD(ji,jj)+ZZ(ji,jk)*PVEC(jj,jk)
    ENDDO
  ENDDO
ENDDO

! Below diagonal using symmetry
DO jj=1,ilen-1
  DO ji=jj+1,ilen
    PROD(ji,jj)=PROD(jj,ji)
  ENDDO
ENDDO

DEALLOCATE(ZZ)

IF (LHOOK) CALL DR_HOOK('MULTVDV',1,ZHOOK_HANDLE)
END SUBROUTINE MULTVDV
