      subroutine sgtsl(kn,pc,pd,pe,pb,kinfo)
      integer kn,kinfo
      real pc(kn),pd(kn),pe(kn),pb(kn)
c
c     dgtsl given a general tridiagonal matrix and a right hand
c     side will find the solution.
c
c     on entry
c
c        kn      integer
c                is the order of the tridiagonal matrix.
c
c        pc      real(kn)
c                is the subdiagonal of the tridiagonal matrix.
c                pc(2) through pc(kn) should contain the subdiagonal.
c                on output pc is destroyed.
c
c        pd      real(kn)
c                is the diagonal of the tridiagonal matrix.
c                on output pd is destroyed.
c
c        pe      real(kn)
c                is the superdiagonal of the tridiagonal matrix.
c                pe(1) through pe(kn-1) should contain the superdiagonal.
c                on output pe is destroyed.
c
c        pb      real(kn)
c                is the right hand side vector.
c
c     on return
c
c        pb       is the solution vector.
c
c        kinfo    integer
c                = 0 normal value.
c                = k if the k-th element of the diagonal becomes
c                    exactly zero.  the subroutine returns when
c                    this is detected.
c
c     linpack. this version dated 08/14/78 .
c     jack dongarra, argonne national laboratory.
c     95-01-11 lars isaksen , ECMWF    doctorized
c
c     no externals
c     fortran abs
c
c     internal variables
c
      integer jk,jb,ikp1,inm1,inm2
      real zt
c     begin block permitting ...exits to 100
c
      kinfo = 0
      pc(1) = pd(1)
      inm1 = kn - 1
      if (inm1 .lt. 1) go to 40
         pd(1) = pe(1)
         pe(1) = 0.
         pe(kn) = 0.
c
         do 30 jk = 1, inm1
         ikp1 = jk + 1
c
c        find the largest of the two rows
c
         if (abs(pc(ikp1)) .lt. abs(pc(jk))) go to 10
c
c           interchange row
c
            zt = pc(ikp1)
            pc(ikp1) = pc(jk)
            pc(jk) = zt
            zt = pd(ikp1)
            pd(ikp1) = pd(jk)
            pd(jk) = zt
            zt = pe(ikp1)
            pe(ikp1) = pe(jk)
            pe(jk) = zt
            zt = pb(ikp1)
            pb(ikp1) = pb(jk)
            pb(jk) = zt
   10    continue
c
c           zero elements
c
         if (pc(jk) .ne. 0.) go to 20
            kinfo = jk
c     ............exit
            go to 100
   20    continue
         zt = -pc(ikp1)/pc(jk)
         pc(ikp1) = pd(ikp1) + zt*pd(jk)
         pd(ikp1) = pe(ikp1) + zt*pe(jk)
         pe(ikp1) = 0.
         pb(ikp1) = pb(ikp1) + zt*pb(jk)
   30    continue
   40    continue
         if (pc(kn) .ne. 0.) go to 50
            kinfo = kn
            go to 90
   50    continue
c
c        back solve
c
         inm2 = kn - 2
         pb(kn) = pb(kn)/pc(kn)
         if (kn .eq. 1) go to 80
            pb(inm1) = (pb(inm1) - pd(inm1)*pb(kn))/pc(inm1)
            if (inm2 .lt. 1) go to 70
               do 60 jb = 1, inm2
               ik = inm2 - jb + 1
               pb(ik) = (pb(ik) - pd(ik)*pb(ik+1) - pe(ik)*pb(ik+2))
     s                  /pc(ik)
   60          continue
   70       continue
   80    continue
   90    continue
  100 continue
c
      return
      end
