      SUBROUTINE LANSO(N,LDOBSC,LANMAX,MAXPRS,ENDL,ENDR,J,NEIG,RITZ,
     *                 BND,R,WRK,ALF,BET,ETA,OLDETA,NQ,IERR,MSGLVL)

      USE control_vectors
C
C     .... inputs
C     .... N      dimension of the eigenproblem
C     .... LDOBSC .T. if use (OPK_OBSCOR,OPM_OBSCOR) inst. of (OPK,OPM)
C     .... LANMAX upper limit to the number of Lanczos steps
C     .... MAXPRS upper limit to the number of wanted eigenpairs
C .... ENDL   left end of the interval containing the wanted eigenvalues
C.... ENDR   right end of the interval containing the wanted eigenvalues
C
C     .... work space
C     .... R      holds 5 vectors of length n. see the text for details.
C .... NQ(4)  contains the pointers to the begining of each vector in R.
C     .... ALF    array to hold diagonal of the tridiagonal T
C     .... BET    array to hold off-diagonal of T
C     .... ETA    orthogonality estimate of lanczos vectors at step j
C     .... OLDETA orthogonality estimate of lanczos vectors at step j-1
C
C     .... outputs
C     .... J      actual number of Lanczos steps taken
C     .... NEIG   number of computed eigenpairs
C     .... RITZ   array to hold the converged Ritz values
C     .... BND    array to hold the error bounds
C     .... IERR   error flag
C
C     .... BLAS routines:    DATX,DAXPY,DCOPY,DDOT,DSCAL,IDAMAX
C     .... subroutines:      DSORT2,TQLB,ORTBND,PURGE,STARTV,STPONE
C     .... user-supplied:    OPK,OPM,STORE
C
C
C
C
C
C     .. Parameters ..
      INTEGER          STORQ, RETRQ, STORP, RETRP
      PARAMETER        (STORQ=1,RETRQ=2,STORP=3,RETRP=4)
      INTEGER          MAXLL
      REAL FOUR
      PARAMETER        (MAXLL=2,FOUR=4.0E0)
C     .. Scalar Arguments ..
      REAL ENDL, ENDR
      INTEGER          IERR, J, LANMAX, MAXPRS, MSGLVL, N, NEIG
      LOGICAL          LDOBSC
C     .. Array Arguments ..
      REAL ALF(LANMAX), BET(LANMAX+1), BND(LANMAX),
     *                 ETA(LANMAX), OLDETA(LANMAX), R(5*N),
     *                 RITZ(LANMAX), WRK(N)
      TYPE (control_vector) :: ZVAZY,ZVAZH
      INTEGER          NQ(4)
C     .. Scalars in Common ..
      REAL ANORM, EPS, EPS1, EPS34, EPSN, RCEPS1, REPS, RNM,
     *                 TOL
C     .. Local Scalars ..
      REAL GAP, GAPL, ONE, T, ZERO
      INTEGER          FIRST, I, ID1, ID2, ID3, L, LAST, LL, MID
      LOGICAL          ENOUGH
C     .. External Functions ..
      REAL DDOT, STARTV
      INTEGER          IDAMAX
      EXTERNAL         DDOT, STARTV, IDAMAX
C     .. External Subroutines ..
      EXTERNAL         DATX, DAXPY, DCOPY, DSCAL, DSORT2, OPK, OPM,
     *                 ORTBND, PURGE, STORE, STPONE, TQLB, OPK_OBSCOR,
     *                 OPM_OBSCOR
C     .. Intrinsic Functions ..
C     INTRINSIC        ABS, MAX, MIN, SQRT
C     .. Common blocks ..
      COMMON           /RDATA/RNM, ANORM, TOL, EPS, EPS1, RCEPS1, EPSN,
     *                 REPS, EPS34
C     .. Data statements ..
      DATA             ONE, ZERO/1.0E0, 0.0E0/
C     .. Executable Statements ..
C
      CALL STPONE(N,LDOBSC,R,WRK,ALF,NQ,IERR,MSGLVL)
      J = 1
      IF (N.EQ.1) THEN
         NEIG = 1
         RITZ(1) = ALF(1)
         BND(1) = ZERO
      ELSE IF (IERR.NE.0) THEN
         NEIG = 0
         RETURN
      END IF
C
      ETA(1) = EPS1
      OLDETA(1) = EPS1
C
      LL = 0
C     %bgn%
      IF (MSGLVL.GE.1) THEN
         CALL PURGE(N,LDOBSC,LL,1,R,R(NQ(1)),R(NQ(4)),R(NQ(3)),
     *    WRK,ETA,OLDETA,MSGLVL)
      END IF
C     %end%
      FIRST = 2
      LAST = MIN(MAXPRS+MAX(8,MAXPRS),LANMAX)
      ENOUGH = .FALSE.
      DO 240 ID1 = 1, MAXPRS
         IF (ENOUGH) GO TO 260
         IF (RNM.LE.TOL) RNM = ZERO
C
C        ....    lanczos loop
C
         DO 40 J = FIRST, LAST
            MID = NQ(2)
            NQ(2) = NQ(1)
            NQ(1) = MID
            MID = NQ(3)
            NQ(3) = NQ(4)
            NQ(4) = MID
            CALL STORE(N,STORQ,J-1,R(NQ(2)))
            IF (J-1.LE.MAXLL) CALL STORE(N,STORP,J-1,R(NQ(4)))
            BET(J) = RNM
C
C           ...        Restart if invariant subspace is found
C
            IF (BET(J).EQ.ZERO) THEN
               RNM = STARTV(N,LDOBSC,J,R,WRK,NQ,EPS,MSGLVL)
               ENOUGH = RNM .EQ. ZERO
               IF (ENOUGH) GO TO 60
            END IF
C
C           ....       take a Lanczos step
C
            T = ONE/RNM
            CALL DATX(N,T,R,1,R(NQ(1)),1)
            CALL DSCAL(N,T,R(NQ(3)),1)
C
            IF (.NOT.LDOBSC) THEN
              CALL ALLOCATE_CTLVEC(ZVAZY)
              CALL ALLOCATE_CTLVEC(ZVAZH)

              NQN=NQ(3)+N-1
              ZVAZY = R(NQ(3):NQN)
              CALL OPK(ZVAZY,ZVAZH)
              CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, R(1:N))
            ELSE
              CALL OPK_OBSCOR(N,R(NQ(3)),R)                       
            ENDIF
            CALL DAXPY(N,-RNM,R(NQ(2)),1,R,1)
C
            ALF(J) = DDOT(N,R,1,R(NQ(3)),1)
            CALL DAXPY(N,-ALF(J),R(NQ(1)),1,R,1)
C
C           ....       orthogonalize against initial Lanczos vectors
C
            IF (J.LE.MAXLL+1 .AND. ABS(ALF(J-1)).GT.FOUR*ABS(ALF(J)))
     *          THEN
               LL = J - 1
C              %bgn%
               IF (MSGLVL.GE.1) THEN
                  WRITE(6,99999) 'LL =', LL, ' |ALF(', J - 1,
     *            ')/ALF(', J, ')|=', ABS(ALF(J-1))/MAX(ABS(ALF(J)),
     *            EPS1)
               END IF
C              %end%
            END IF
            DO 20 I = 1, MIN(LL,J-2)
               CALL STORE(N,RETRP,I,WRK)
               T = DDOT(N,WRK,1,R,1)
               CALL STORE(N,RETRQ,I,WRK)
               CALL DAXPY(N,-T,WRK,1,R,1)
               ETA(I) = EPS1
               OLDETA(I) = EPS1
   20       CONTINUE
C
C           ....       extended local reorthogonalization
C
            T = DDOT(N,R,1,R(NQ(4)),1)
            CALL DAXPY(N,-T,R(NQ(2)),1,R,1)
            IF (BET(J).GT.ZERO) BET(J) = BET(J) + T
            T = DDOT(N,R,1,R(NQ(3)),1)
            CALL DAXPY(N,-T,R(NQ(1)),1,R,1)
            ALF(J) = ALF(J) + T
            IF (.NOT.LDOBSC) THEN
              ZVAZY = R(1:N)
              CALL OPM(ZVAZY,ZVAZH)
              NQN=NQ(4)+N-1
              CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, R(NQ(4):NQN))
              CALL DEALLOCATE_CTLVEC(ZVAZY)
              CALL DEALLOCATE_CTLVEC(ZVAZH)
            ELSE
              CALL OPM_OBSCOR(N,R,R(NQ(4)))
            ENDIF
            RNM = SQRT(DDOT(N,R,1,R(NQ(4)),1))
            ANORM = BET(J) + ABS(ALF(J)) + RNM
            TOL = EPSN*ANORM
C
C           ....       update the orthogonality bounds
C
            CALL ORTBND(N,J,ALF,BET,ETA,OLDETA)
C
C           ....       restore the orthogonality state when needed
C
C           CALL PURGE(...,ETA,OLDETA,MSGLVL)
            CALL PURGE(N,LDOBSC,LL,J,R,R(NQ(1)),R(NQ(4)),R(NQ(3)),
     *                 WRK,ETA,OLDETA,MSGLVL)
            IF (RNM.LE.TOL) RNM = ZERO
   40    CONTINUE
         J = LAST
   60    IF (ENOUGH) J = J - 1
         FIRST = J + 1
         BET(J+1) = RNM
C
C        ....    Now analyze T
C
         L = 1
         DO 140 ID2 = 1, J
            IF (L.GT.J) GO TO 160
            DO 80 I = L, J
               IF (BET(I+1).EQ.ZERO) GO TO 100
   80       CONTINUE
            I = J
C           ....       Now i is at the end of an unreduced submatrix
C
  100       CALL DCOPY(I-L+1,ALF(L),1,RITZ(L),-1)
            IF (I.GT.L) CALL DCOPY(I-L,BET(L+1),1,WRK(L+1),-1)
            CALL TQLB(I-L+1,RITZ(L),WRK(L),BND(L),IERR)
            IF (IERR.NE.0) THEN
               WRITE(6,*) ' TQLB failed to converge (ierr =', IERR, ')'
               WRITE(6,*) ' L =', L, ' I =', I
               WRITE(6,*) (ID3,RITZ(ID3),WRK(ID3),BND(ID3),ID3=L,I)
            END IF
C           %bgn%
            IF (MSGLVL.GE.1) THEN
               WRITE(6,99998) 'TQLB exited with IERR =', IERR
               T = DDOT(I-L+1,BND(L),1,BND(L),1)
               WRITE(6,99997) 'TQLB delivered...', ' J =', J,
     *           ' FIRST =', FIRST, ' LAST =', LAST, ' ID1 =', ID1,
     *           ' L =', L,' I =', I, ' |BND|^2 =', T
               WRITE(6,99996) (ID3,RITZ(ID3),BND(ID3),ALF(ID3),BET(ID3),
     *           ID3=L,I)
            END IF
C           %end%
            DO 120 ID3 = L, I
               BND(ID3) = RNM*ABS(BND(ID3))
  120       CONTINUE
            L = I + 1
  140    CONTINUE
C
C        ....    Sort eigenvalues into increasing order
C
  160    CALL DSORT2(J,RITZ,BND)
C        %bgn%
         IF (MSGLVL.GE.4) THEN
            WRITE(6,99995) 'DSORT2 delivered...'
            WRITE(6,99994) (ID3,RITZ(ID3),BND(ID3),ID3=1,J)
         END IF
C        %end%
C
C        ....    Massage error bounds for very close Ritz values
C
         MID = IDAMAX(J,BND,1)
         DO 200 L = -1, 1, 2
            DO 180 I = ((J+1)-L*(J-1))/2, MID - L, L
               IF (ABS(RITZ(I+L)-RITZ(I)).LT.EPS34*ABS(RITZ(I))) THEN
                  IF (BND(I).GT.TOL .AND. BND(I+L).GT.TOL) THEN
                     BND(I+L) = SQRT(BND(I)**2+BND(I+L)**2)
                     BND(I) = ZERO
                  END IF
               END IF
  180       CONTINUE
  200    CONTINUE
C
C        ....    Refine the error bounds
C
         NEIG = 0
         GAPL = RITZ(J) - RITZ(1)
         DO 220 I = 1, J
            GAP = GAPL
            IF (I.LT.J) GAPL = RITZ(I+1) - RITZ(I)
            GAP = MIN(GAP,GAPL)
            IF (GAP.GT.BND(I)) THEN
               BND(I) = BND(I)*(BND(I)/GAP)
            END IF
            IF (BND(I).LE.16.0E0*EPS*ABS(RITZ(I))) THEN
               NEIG = NEIG + 1
               ENOUGH = ENOUGH .OR. ENDL .LT. RITZ(I) .AND. RITZ(I)
     *                  .LT. ENDR
            END IF
  220    CONTINUE
C
C        ....    Should we stop?
C
         IF (NEIG.LT.MAXPRS) THEN
            IF (NEIG.EQ.0) THEN
               LAST = FIRST + 8
            ELSE
               LAST = FIRST + MAX(2,((J-6)*(MAXPRS-NEIG))/NEIG)
            END IF
            LAST = MIN(LAST,LANMAX)
         ELSE
            ENOUGH = .TRUE.
         END IF
         ENOUGH = ENOUGH .OR. FIRST .GT. LANMAX
  240 CONTINUE
  260 CALL STORE(N,STORQ,J,R(NQ(1)))
      RETURN
C
99999 FORMAT (/1X,A,I3,A,I1,A,I1,A,1P,D9.1)
99998 FORMAT (/1X,A,I5)
99997 FORMAT (/1X,A,3(A,I3),/20X,3(A,I3),A,1P,D22.14,/3X,'I',9X,'RITZ(',
     *       'I)',8X,'(',8X,'BND(I)',8X,')',7X,'ALF(I)',6X,'BET(I)')
99996 FORMAT (1X,I3,1X,1P,D22.14,1X,'(',D22.14,')',4X,D11.3,1X,D11.3)
99995 FORMAT (/1X,A)
99994 FORMAT (1X,I3,1X,1P,D22.14,1X,'(',D22.14,')')
      END
