      SUBROUTINE PURGE(N,LDOBSC,LL,J,R,Q,RA,QA,WRK,ETA,OLDETA,MSGLVL)

      USE control_vectors
C
C     .... This routine examines ETA to decide whether
C     .... re-orthogonalization should be performed.
C
C     .... N      dimension of the eigenproblem
C     .... LDOBSC .T. if use (OPK_OBSCOR,OPM_OBSCOR) inst. of (OPK,OPM)
C     .... LL     no. of initial Lanczos vectors in local orthog.
C     .... J      current Lanczos step
C     .... R      the residual vector to become the next Lanczos vector
C     .... Q      the current Lanczos vector
C     .... RA     the product of the mass matrix and r
C     .... QA     the product of the mass matrix and q
C    .... WRK    a temporary vector to hold the previous Lanczos vectors
C.... ETA    state of orthogonality between r and previous Lanczos vectors
C.... OLDETA state of orthogonality between q and previous Lanczos vectors
C
C     .... BLAS routines:    DAXPY,DDOT,IDAMAX
C     .... subroutines:      none
C     .... user-supplied:    OPM,STORE
C
C
C     %bgn%
C     .. Parameters ..
      INTEGER          RETRQ
      PARAMETER        (RETRQ=2)
      INTEGER          LMTPRS
      PARAMETER        (LMTPRS=250)
C     .. Scalar Arguments ..
      INTEGER          J, LL, MSGLVL, N
      LOGICAL          LDOBSC
C     .. Array Arguments ..
      REAL ETA(J), OLDETA(J), Q(N), QA(N), R(N), RA(N),
     *                 WRK(N)
      TYPE (control_vector) :: ZVAZY,ZVAZH
C     .. Scalars in Common ..
      REAL ANORM, EPS, EPS1, EPS34, EPSN, RCEPS1, REPS, RNM,
     *                 TOL
C     .. Local Scalars ..
      REAL REPS1, T, TQ, TR
      INTEGER          I, K, LOOP
C     .. Local Arrays ..
      REAL COSQR(LMTPRS)
C     .. External Functions ..
      REAL DDOT
      INTEGER          IDAMAX
      EXTERNAL         DDOT, IDAMAX
C     .. External Subroutines ..
      EXTERNAL         ANGLES, DAXPY, OPM, PRANGL, PRRULE, STORE,
     $  OPM_OBSCOR
C     .. Intrinsic Functions ..
      INTRINSIC        ABS, SQRT
C     .. Common blocks ..
      COMMON           /RDATA/RNM, ANORM, TOL, EPS, EPS1, RCEPS1, EPSN,
     *                 REPS, EPS34
C     .. Executable Statements ..
C
      IF (MSGLVL.GE.1) THEN
         IF (J.LE.LL+2) THEN
            K = IDAMAX(J,ETA(1),1)
         ELSE
            K = IDAMAX(J-(LL+2),ETA(LL+1),1) + LL
         END IF
         CALL PRRULE(J,6,RNM,TOL,K,ABS(ETA(K)),REPS)
      END IF
      IF (MSGLVL.GE.2) THEN
         CALL PRANGL(J,6,-1.0E0,TOL,EPS,0.0E0,0.0E0,ETA)
      END IF
      IF (MSGLVL.GE.1) THEN
         CALL ANGLES(ABS(N),J,RNM,RA,Q,WRK,COSQR)
         CALL PRANGL(J,6,-1.0E0,TOL,EPS,0.0E0,0.0E0,COSQR)
      END IF
C     %end%
C
      IF (J.LE.LL+1) RETURN
      K = IDAMAX(J-(LL+1),ETA(LL+1),1) + LL
C
      IF (ABS(ETA(K)).GT.REPS) THEN
         REPS1 = EPS1/REPS
         DO 40 LOOP = 1, 2
            IF (RNM.GT.TOL) THEN
C
C  ....       Bring in a Lanczos vector t and orthogonalize both r and q
C              ....       against it
C
               TQ = 0.0E0
               TR = 0.0E0
               DO 20 I = 1, J - 1
                  CALL STORE(N,RETRQ,I,WRK)
                  T = -DDOT(N,QA,1,WRK,1)
                  TQ = TQ + ABS(T)
                  CALL DAXPY(N,T,WRK,1,Q,1)
                  T = -DDOT(N,RA,1,WRK,1)
                  TR = TR + ABS(T)
                  CALL DAXPY(N,T,WRK,1,R,1)
   20          CONTINUE
               IF (.NOT.LDOBSC) THEN
                 CALL ALLOCATE_CTLVEC(ZVAZY)
                 CALL ALLOCATE_CTLVEC(ZVAZH)
 
                 ZVAZY = Q(:)
                 CALL OPM(ZVAZY,ZVAZH)
                 CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, QA(:))
               ELSE
                 CALL OPM_OBSCOR(N,Q,QA)
               ENDIF
C
C              ....          restore local orthogonality
C
               T = -DDOT(N,R,1,QA,1)
               TR = TR + ABS(T)
               CALL DAXPY(N,T,Q,1,R,1)
C
               IF (.NOT.LDOBSC) THEN
                 ZVAZY = R(:)
                 CALL OPM(ZVAZY,ZVAZH)
                 CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, RA(:))

                 CALL DEALLOCATE_CTLVEC(ZVAZY)
                 CALL DEALLOCATE_CTLVEC(ZVAZH)
               ELSE
                 CALL OPM_OBSCOR(N,R,RA)
               ENDIF
               RNM = SQRT(DDOT(N,RA,1,R,1))
C              %bgn%
               IF (MSGLVL.GE.1) THEN
                  CALL ANGLES(N,J,RNM,RA,Q,WRK,COSQR)
                  IF (RNM.EQ.0.0E0) THEN
                     CALL PRANGL(J,6,RNM,TOL,EPS,TQ/REPS1,
     *                           TR/(REPS1*EPS1),COSQR)
                  ELSE
                     CALL PRANGL(J,6,RNM,TOL,EPS,TQ/REPS1,
     *                           TR/(REPS1*RNM),COSQR)
                  END IF
               END IF
C              %end%
               IF (TQ.LE.REPS1 .AND. TR.LE.REPS1*RNM) GO TO 60
            END IF
   40    CONTINUE
   60    DO 80 I = LL + 1, J
            ETA(I) = EPS1
            OLDETA(I) = EPS1
   80    CONTINUE
      END IF
      RETURN
      END
