      REAL FUNCTION RANDOM(IY)
C
C     random is a uniform random number generator based  on  theory  and
C    suggestions  given  in  d.e.knuth (1969),  vol  2.  the integer  iy
C  should be initialized to an arbitrary integer prior to the first call
C   to random.the calling program should  not  alter  the  value  of  iy
C   between subsequent calls to random.values of random will be returned
C     in the interval (0,1).
C
C
C     .. Scalar Arguments ..
      INTEGER                          IY
C     .. Local Scalars ..
      REAL HALFM, S
      INTEGER                          IA, IC, ITWO, M, M2, MIC
C     .. Intrinsic Functions ..
      INTRINSIC                        ATAN, REAL, INT, SQRT
C     .. Data statements ..
      DATA                             M2, ITWO/0, 2/
C     .. Executable Statements ..
      IF (M2.EQ.0) THEN
C
C        if first entry, compute machine integer word length
C
         M = 1
   20    M2 = M
         M = ITWO*M2
         IF (M.GT.M2) GO TO 20
         HALFM = M2
C
C        compute multiplier and increment for linear congruential method
C
         IA = 8*INT(HALFM*ATAN(1.E0)/8.E0) + 5
         IC = 2*INT(HALFM*(0.5E0-SQRT(3.E0)/6.E0)) + 1
         MIC = (M2-IC) + M2
C
C        s is the scale factor for converting to floating point
C
         S = 0.5E0/HALFM
      END IF
C
C        compute next random number
C
      IY = IY*IA
C
C     the following statement is for computers which do not allow
C     integer overflow on addition
C
      IF (IY.GT.MIC) IY = (IY-M2) - M2
C
      IY = IY + IC
C
C     the following statement is for computers where the
C     word length for addition is greater than for multiplication
C
      IF (IY/2.GT.M2) IY = (IY-M2) - M2
C
C     the following statement is for computers where integer
C     overflow affects the sign bit
C
      IF (IY.LT.0) IY = (IY+M2) + M2
      RANDOM = REAL(IY)*S
      RETURN
      END
