      REAL FUNCTION STARTV(N,LDOBSC,J,R,WRK,NQ,EPS,MSGLVL)
#ifndef SGI

      USE control_vectors
C
C     .... This routine delivers a starting vector in R and returns |R|;
C     .... it returns ZERO if range is spanned or if no starting vector
C     .... within range of operator can be found.
C
C     .... N      dimension of the eigenproblem
C     .... LDOBSC .T. if use (OPK_OBSCOR,OPM_OBSCOR) inst. of (OPK,OPM)
C     .... J      starting index for a Lanczos run
C   .... R      an array containing [r(j),q(j),q(j-1),p(j),p(j-1)/Mr(j)]
C     .... NQ(4)  location pointers for the array R
C
C     .... BLAS routines:    DAXPY,DDOT
C     .... subroutines:      RANDOM
C     .... user-supplied:    OPK,OPM,STORE
C
C
C
C     .. Parameters ..
      INTEGER                          RETRQ
      PARAMETER                        (RETRQ=2)
C     .. Scalar Arguments ..
      REAL EPS
      REAL HARVEST
      INTEGER                          J, MSGLVL, N
      LOGICAL                          LDOBSC
C     .. Array Arguments ..
      REAL R(5*N), WRK(N)
      TYPE (control_vector) :: ZVAZY,ZVAZH
      INTEGER                          NQ(4)
C     .. Local Scalars ..
      REAL ONE, RNM2, T, ZERO
      INTEGER                          I, ID, IRAND, LOOP
C     .. External Functions ..
      REAL DDOT, RANDOM
      EXTERNAL                         DDOT, RANDOM
C     .. External Subroutines ..
      EXTERNAL                         DAXPY, OPK, OPM, STORE
      EXTERNAL                         OPK_OBSCOR, OPM_OBSCOR
C     .. Intrinsic Functions ..
      INTRINSIC                        SQRT
C     .. Data statements ..
      DATA                             ONE, ZERO/1.0E0, 0.0E0/
C     .. Executable Statements ..
C
C     .... get initial vector, default is random
C
      RNM2 = DDOT(N,R,1,R,1)
      IRAND = 918272 + J
      ISEED=J*3*N
      DO 100 ID = 1, 3
         IF (ID.GT.1 .OR. J.GT.1 .OR. RNM2.EQ.ZERO) THEN
            DO 20 I = 1, N
               ISEED=ISEED+2
               CALL RANDOM_SEED(ISEED)
               CALL RANDOM_NUMBER(HARVEST)
               R(I) = HARVEST - 0.5
   20       CONTINUE
         END IF
         IF (.NOT.LDOBSC) THEN
           CALL ALLOCATE_CTLVEC(ZVAZY)
           CALL ALLOCATE_CTLVEC(ZVAZH)

           ZVAZY = R(1:N)
           CALL OPM(ZVAZY,ZVAZH)
C
C     ....    apply operator to put R in range (essential if M singular)
C
           CALL OPK(ZVAZH,ZVAZY)
           CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZY, N, R(1:N))

           CALL OPM(ZVAZY,ZVAZH)
           NQN=NQ(3)+N-1
           CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, R(NQ(3):NQN))

           CALL DEALLOCATE_CTLVEC(ZVAZY)
           CALL DEALLOCATE_CTLVEC(ZVAZH)
         ELSE
           CALL OPM_OBSCOR(N,R,R(NQ(3)))
C
C     ....    apply operator to put R in range (essential if M singular)
C
           CALL OPK_OBSCOR(N,R(NQ(3)),R)
           CALL OPM_OBSCOR(N,R,R(NQ(3)))
         ENDIF
C
         RNM2 = DDOT(N,R,1,R(NQ(3)),1)
         IF (RNM2.EQ.ZERO) THEN
            GO TO 100
         ELSE IF (J.GT.1) THEN
            DO 60 LOOP = 1, 2
               DO 40 I = 1, J - 1
                  CALL STORE(N,RETRQ,I,WRK)
                  T = -DDOT(N,R(NQ(3)),1,WRK,1)
                  CALL DAXPY(N,T,WRK,1,R,1)
   40          CONTINUE
               IF (.NOT.LDOBSC) THEN
                 CALL ALLOCATE_CTLVEC(ZVAZY)
                 CALL ALLOCATE_CTLVEC(ZVAZH)

                 ZVAZY = R(1:N)
                 CALL OPM(ZVAZY,ZVAZH)
                 NQN=NQ(3)+N-1
                 CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, R(NQ(3):NQN))

                 CALL DEALLOCATE_CTLVEC(ZVAZY)
                 CALL DEALLOCATE_CTLVEC(ZVAZH)
               ELSE
                 CALL OPM_OBSCOR(N,R,R(NQ(3))) 
               ENDIF
               T = DDOT(N,R(NQ(3)),1,R,1)
C              %bgn%
               IF (MSGLVL.GE.1) THEN
                  WRITE(6,99999) ID, ' J =', J, ' LOOP =', LOOP, ' T =',
     *              T, ' RNM2 =', RNM2, ' T/RNM2 =', T/RNM2
               END IF
C              %end%
               IF (T.LE.EPS*RNM2) THEN
                  T = ZERO
                  GO TO 80
               END IF
   60       CONTINUE
   80       RNM2 = T
         END IF
         IF (RNM2.GT.ZERO) GO TO 120
  100 CONTINUE
  120 STARTV = SQRT(RNM2)
      RETURN
C
99999 FORMAT (/1X,'STARTV[',I1,']',A4,I3,A,I2,1P,3(A,D10.2))
#endif
      END
