      SUBROUTINE STPONE(N,LDOBSC,R,WRK,ALF,NQ,IERR,MSGLVL)

      USE control_vectors
C
C    .... This routine performs the first step of the Lanczos algorithm.
C     .... It performs a step of extended local re-orthogonalization.
C
C     .... N      dimension of the eigenproblem
C     .... LDOBSC .T. if use (OPK_OBSCOR,OPM_OBSCOR) inst. of (OPK,OPM)
C   .... R      an array containing [r(j),q(j),q(j-1),p(j),p(j-1)/Mr(j)]
C     .... ALF    diagonal elements of T
C     .... NQ(4)  location pointers for the array R
C
C     .... BLAS routines:    DATX,DAXPY,DDOT,DSCAL
C     .... subroutines:      STARTV
C     .... user-supplied:    OPK,OPM,STORE
C
C
C
C     .. Scalar Arguments ..
      INTEGER           IERR, MSGLVL, N
      LOGICAL           LDOBSC
C     .. Array Arguments ..
      REAL ALF(1), R(5*N), WRK(N)
      TYPE (control_vector) :: ZVAZY,ZVAZH
      INTEGER           NQ(4)
C     .. Scalars in Common ..
      REAL ANORM, EPS, EPS1, EPS34, EPSN, RCEPS1, REPS,
     *                  RNM, TOL
C     .. Local Scalars ..
      REAL ONE, T, ZERO
C     .. External Functions ..
      REAL DDOT, STARTV
      EXTERNAL          DDOT, STARTV
C     .. External Subroutines ..
      EXTERNAL          DATX, DAXPY, DSCAL, OPK, OPM,
     *       OPK_OBSCOR, OPM_OBSCOR
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, SQRT
C     .. Common blocks ..
      COMMON            /RDATA/RNM, ANORM, TOL, EPS, EPS1, RCEPS1, EPSN,
     *                  REPS, EPS34
C     .. Data statements ..
      DATA              ONE, ZERO/1.0E0, 0.0E0/
C     .. Executable Statements ..
C
C     .... get initial vector, default is random
C
      RNM = STARTV(N,LDOBSC,1,R,WRK,NQ,EPS,MSGLVL)
      IF (RNM.EQ.ZERO) RETURN
C
C     .... normalize starting vector
C
      T = ONE/RNM
      CALL DATX(N,T,R,1,R(NQ(1)),1)
      CALL DSCAL(N,T,R(NQ(3)),1)
C
C     .... take the first step
C
      IF (.NOT.LDOBSC) THEN
        CALL ALLOCATE_CTLVEC(ZVAZY)
        CALL ALLOCATE_CTLVEC(ZVAZH)

        NQN=NQ(3)+N-1
        ZVAZY = R(NQ(3):NQN)
        CALL OPK(ZVAZY,ZVAZH)
        CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, R(1:N))
      ELSE
        CALL OPK_OBSCOR(N,R(NQ(3)),R)
      ENDIF
      ALF(1) = DDOT(N,R,1,R(NQ(3)),1)
      CALL DAXPY(N,-ALF(1),R(NQ(1)),1,R,1)
C
C     .... restore local orthogonality
C
      T = DDOT(N,R,1,R(NQ(3)),1)
      CALL DAXPY(N,-T,R(NQ(1)),1,R,1)
      ALF(1) = ALF(1) + T
      IF (.NOT.LDOBSC) THEN
        ZVAZY = R(1:N)
        CALL OPM(ZVAZY,ZVAZH)
        NQN=NQ(4)+N-1
        CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, N, R(NQ(4):NQN))

        CALL DEALLOCATE_CTLVEC(ZVAZY)
        CALL DEALLOCATE_CTLVEC(ZVAZH)
      ELSE
        CALL OPM_OBSCOR(N,R,R(NQ(4)))
      ENDIF
      RNM = SQRT(DDOT(N,R,1,R(NQ(4)),1))
      ANORM = RNM + ABS(ALF(1))
      TOL = EPSN*ANORM
C
      RETURN
      END
