MODULE CONTROL_VECTORS_DATA_MIX

!   Purpose.
!   --------
!     Setup and store parameters from IFS not available in IFSAUX.
!
!   Author.
!   -------
!     Y. Tremolet
!
!   Modifications.
!   -------------- 
!     Original   25-Jul-2004
!     A. Deckmyn 01-Oct-2008 : LAM wavelet Jb
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM

IMPLICIT NONE
SAVE
PRIVATE
PUBLIC SETUP_CTLVEC, &
     & linitcv, ltovscv, nmoderr, lelam, lwaveletjb, &
     & nsmax, nmsmax, nserr, nflevl, nflevg, &
     & mytovoff, nctl3, nctl2, nctl1, nve3d, nve2d, nparam, nscales, &
     & nproma, n_regions, n_regions_ns, n_regions_ew, &
     & my_region_ns, my_region_ew, myproc,&
     & nlentovscv, nlentovscvg, nlentovscvp,nvatovv, nvatovoff,&
     & mylevs, nblocksl, nblocksg, ngptotl, ngptotg, ngrb, ngrbe,&
     & mytsoff, nobntv, nobntvg, ltscv,lrepro,mysetw,nprtrw,nproc

! ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: nsmax, nmsmax, nserr, nmoderr, &
           & mytovoff, nctl3, nctl2, nctl1, nparam, nscales, &
           & nproma, n_regions_ns, n_regions_ew, &
           & my_region_ns, my_region_ew, myproc,&
           & nlentovscv, nlentovscvg, nvatovv, nflevl, nflevg,&
           & mytsoff, nobntv, nobntvg, nve3d, nve2d,mysetw,nprtrw,nproc
   

INTEGER(KIND=JPIM), ALLOCATABLE :: mylevs(:), nblocksl(:), nblocksg(:), &
                                 & ngptotl(:,:,:), ngptotg(:), ngrb(:), &
                                 & nlentovscvp(:),nvatovoff(:),n_regions(:), &
                                 & ngrbe(:)

LOGICAL :: linitcv, ltovscv, lelam, lwaveletjb, ltscv,lrepro

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

SUBROUTINE SETUP_CTLVEC(kctl3,kctl2,kctl1,kgrb,ldinit,ksmax,kmsmax, &
                      & kmerr,kve3d,kve2d,kserr,kgrbe,kparam,ldlam, &
                      & ldwavelet, ldtovs,klentovscv,klentovscvg, &
                      & klentovscvp, kvatovv,kvatovoff, &
                      & kproma,kflevl,kflevg,kylevs, &
                      & knscales,kblocksl,kblocksg,kgptotl,kgptotg, &
                      & k_regions,k_regions_ns,k_regions_ew, &
                      & kseta,ksetb,kproc,kmyproc, &
                      & kobntv,kobntvg,ldts,ktsoff,ldrepro,kmysetw,kprtrw)

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: kctl3,kctl2,kctl1
INTEGER(KIND=JPIM), INTENT(IN) :: kgrb(kctl3+kctl2)
INTEGER(KIND=JPIM), INTENT(IN) :: ksmax,kmsmax,kmerr,kserr,kparam
INTEGER(KIND=JPIM), INTENT(IN) :: kve3d,kve2d
INTEGER(KIND=JPIM), INTENT(IN) :: kgrbe(kve3d+kve2d)
LOGICAL, INTENT(IN) :: ldinit, ldtovs, ldwavelet
LOGICAL, INTENT(IN),OPTIONAL :: ldts
INTEGER(KIND=JPIM), INTENT(IN) :: klentovscv, klentovscvg, kvatovv
LOGICAL, INTENT(IN) :: ldlam
INTEGER(KIND=JPIM), INTENT(IN) :: kflevl, kflevg, knscales,kproc,kmyproc,  &
                                & kproma, k_regions_ns, k_regions_ew, kseta, ksetb
INTEGER(KIND=JPIM), INTENT(IN) :: kylevs(kflevl), kblocksl(knscales), &
                                & kblocksg(knscales), kgptotg(knscales), &
                                & kgptotl(knscales,k_regions_ns,k_regions_ew)
INTEGER(KIND=JPIM), INTENT(IN) :: klentovscvp(:)
INTEGER(KIND=JPIM), INTENT(IN) :: kvatovoff(:)
INTEGER(KIND=JPIM), INTENT(IN) :: k_regions(:)
INTEGER(KIND=JPIM), INTENT(IN) :: kprtrw,kmysetw
INTEGER(KIND=JPIM),OPTIONAL :: kobntv, kobntvg
INTEGER(KIND=JPIM),OPTIONAL :: ktsoff
LOGICAL, INTENT(IN) :: ldrepro


nctl2=kctl2
nctl3=kctl3
IF (.NOT.ALLOCATED(ngrb)) ALLOCATE(ngrb(nctl3+nctl2))
ngrb(:)=kgrb(:)
nctl1=kctl1
nsmax=ksmax
nmsmax=kmsmax
nmoderr=kmerr
nve3d=kve3d
nve2d=kve2d
IF (.NOT.ALLOCATED(ngrbe)) ALLOCATE(ngrbe(nve3d+nve2d))
ngrbe(:)=kgrbe(:)
linitcv=ldinit
nserr=kserr
ltovscv=ldtovs
nlentovscv =klentovscv
nlentovscvg=klentovscvg
nvatovv=kvatovv
nproc=kproc
if (ltovscv) then
  IF (.NOT.ALLOCATED(nlentovscvp)) ALLOCATE(nlentovscvp(kproc))
  IF (.NOT.ALLOCATED(nvatovoff)) ALLOCATE(nvatovoff(kproc))
  nlentovscvp(:)=klentovscvp(:)
  nvatovoff(:)=kvatovoff(:)
  mytovoff=kvatovoff(kmyproc)
endif
IF (.NOT.ALLOCATED(n_regions)) ALLOCATE(n_regions(k_regions_ns))
lelam=ldlam
nflevl=kflevl
nflevg=kflevg
nproma=kproma
n_regions(1:k_regions_ns)=k_regions(1:k_regions_ns)
n_regions_ns=k_regions_ns
n_regions_ew=k_regions_ew
my_region_ns=kseta
my_region_ew=ksetb
myproc=kmyproc
mysetw=kmysetw
nprtrw=kprtrw
IF (.NOT.ALLOCATED(MYLEVS)) ALLOCATE(MYLEVS(nflevl))
MYLEVS(:)=kylevs(:)
nparam=kparam
lwaveletjb=ldwavelet
IF (ldwavelet) THEN
  IF (LELAM) THEN
    ! nblocks and nblocksg used for basic LAM wavelets data distribution
    ALLOCATE(nblocksl(2))
    nblocksl(:)=kblocksl(:)
    ALLOCATE(nblocksg(2))
    nblocksg(:)=kblocksg(:)
  ELSE
    nscales=knscales
    ALLOCATE (nblocksl(knscales))
    ALLOCATE (nblocksg(knscales))
    ALLOCATE (ngptotl(knscales,k_regions_ns,k_regions_ew))
    ALLOCATE (ngptotg(knscales))
    nblocksl(:)=kblocksl(:)
    nblocksg(:)=kblocksg(:)
    ngptotl(:,:,:)=kgptotl(:,:,:)
    ngptotg(:)=kgptotg(:)
  ENDIF
ENDIF

IF (PRESENT(ldts)) THEN
   ltscv=ldts
ELSE
   ltscv=.FALSE.
ENDIF
IF (PRESENT(kobntv)) nobntv=kobntv
IF (PRESENT(kobntvg)) nobntvg=kobntvg
IF (PRESENT(ktsoff)) mytsoff=ktsoff
lrepro=ldrepro
write(*,*)'SETUP_CTLVEC: CONTROL_VECTOR type has been setup.'
write(*,*)'SETUP_CTLVEC: LINITCV, LWAVELETJB, LREPRO=',LINITCV,LWAVELETJB,LREPRO
write(*,*)'SETUP_CTLVEC: NMODERR, NPARAM=',NMODERR,NPARAM
write(*,*)'SETUP_CTLVEC: LTOVSCV, LELAM =',LTOVSCV,LELAM

END SUBROUTINE setup_ctlvec

! ------------------------------------------------------------------

END MODULE CONTROL_VECTORS_DATA_MIX
