#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE CALL_SL_AD(KGPTOT,PB1,PB15,PB2,&
 & PGMVT1,PGMVT1S,PGFLT1,KGPBLKS,&
 & PGM,PGEMU,PGESLO,PGECLO,PGNORDL,&
 & PGNORDM,PGSQM2,PCOLON,PSILON,&
 & PRINDX,PRINDY )  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPROC
USE YOMSLREP , ONLY : LSLADREP ,LVECADIN
USE YOMGEM   , ONLY : NLOENG
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NDGLG    ,NDGSAG   ,NDGENG   ,&
 & NDGSAL   ,NDGENL   ,NFLSA    ,NFLEN    ,NFLSUL
USE YOMDYN   , ONLY : NITMP     ,NVLAG    ,NWLAG    ,NTLAG    ,L2TLFF
USE YOMSEP   , ONLY : NVSEPC   ,NVSEPL
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE PTRSLB1  , ONLY : RPARSL1
USE PTRSLB15 , ONLY : MSLB1UR05,MSLB1WR05,RPARSL15  
USE YOMMP    , ONLY : NASLB1    ,NSLPROCS  ,&
 & NSLRPT    ,NSLSPT    ,NSLSENDPOS,&
 & NSLRECVPOS,NSENDPTR  ,NRECVPTR  ,NSLCOMM   ,LSLCOMM ,&
 & NSLSTA    ,NSLONL    ,NSLOFF    ,NFRSTLOFF ,&
 & MYFRSTACTLAT         ,MYLSTACTLAT  
USE YOMCT3   ,ONLY : NSTEP
USE YOMVAR  , ONLY : NSIM4D
USE YOMSC2   , ONLY : NSLWIDE   ,NFLDSLB1 ,NFLDSLB15, NFLDSLB2
USE YOMMASK  , ONLY : LSLONDEM, NFIXSFLD, LSLONDEM_ACTIVE, MASK_SL1, MASK_SL2,&
 & MFIELD_TYPE15,  MASK_SL2A, MASK_SL3,  LNEW_MASK_SL, &
 & MASK_SL2T, MASK_SL2AT, MASK_SL3T
USE YOMGMV   , ONLY : YT1
USE YOM_YGFL , ONLY : YGFL   ,YGFLC
USE OML_MOD, ONLY : OML_OMP, OML_SET_EVENT

!**** *CALL_SL_AD* - Semi-Lagrangian Interpolation AD

!     Purpose.
!     --------
!           Semi-Lagrangian Interpolation AD

!**   Interface.
!     ----------
!        *CALL* *CALL_SL_AD*

!        Explicit arguments :
!        --------------------
!        INPUT/OUTPUT:
!          PB1     - SLBUF1-buffer for interpolations.
!          PB2     - SLBUF2-buf to communicate info from non lag. to lag. dyn.
!          PB15    - SLBUF1-buf traj for interpolations
!          Some other dummy arguments documented for ex in LAPINE(A,B)AD

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by GP_MODEL_AD.

!     Author.
!     -------
!        Deborah Salmond 

!     Modifications.
!     --------------
!   Modified 00-11-10 by C. Temperton: streamlining.
!   Modified 00-11-23 by C. Temperton: renamed _TRAJ and _INT routines.
!   Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!   Modified at ECMWF (24 -> 24R1): some local arrays become allocatable.
!   Modified 02-10-01 G.Mozdzynski: support for radiation on-demand comms
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   Modified 05-02-02 by C. Temperton: streamlining.
!   D.Salmond     08-Feb-2005 New versions of LAPINEAAD+LAPINEBAD
!   F.Vana        08-Jan-2007 plane geometry extension
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   F. Vana       15-Aug-2008  Weights driven interpolation.
!   K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!   K. Yessad Dec 2008: merge SLCOMM+SLCOMM1 -> SLCOMM.
!   K. Yessad Dec 2008: merge the different (E)SLEXTPOL.. -> (E)SLEXTPOL.
!   F. Vana       11-Dec-2008 OpenMP for vector platforms
!   F. Vana   13-Jan-2009: TL/AD of SLHD
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGPTOT
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPBLKS 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NASLB1,NFLDSLB1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB15(NASLB1,NFLDSLB15) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2,KGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM,KGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVT1S(NPROMA,YT1%NDIMS,KGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1,KGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDL(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDM(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRINDX(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRINDY(KGPTOT) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JSTGLO, IST, IEND, IBL, IOFF,  IDIM1, IDIM2, &
 & JINC, JROF, JLEV, ILEVELS, ISTART, ISTOP, IDIM, JGFL, JITER

INTEGER(KIND=JPIM) :: IDUMARR(2)

LOGICAL :: LLINC, LL2TLFFO, LLEXCLUDE

! * trigonometric arrays.

REAL(KIND=JPRB) :: ZR    (NPROMA,NFLEVG,KGPBLKS)
REAL(KIND=JPRB) :: ZS    (NPROMA,NFLEVG,KGPBLKS)
REAL(KIND=JPRB) :: ZR5   (NPROMA,NFLEVG,NITMP,KGPBLKS)
REAL(KIND=JPRB) :: ZS5   (NPROMA,NFLEVG,NITMP,KGPBLKS)
REAL(KIND=JPRB) :: ZUF5  (NPROMA,NFLEVG,NITMP-1,KGPBLKS)
REAL(KIND=JPRB) :: ZVF5  (NPROMA,NFLEVG,NITMP-1,KGPBLKS)
REAL(KIND=JPRB) :: ZLON  (NPROMA,NFLEVG,KGPBLKS)
REAL(KIND=JPRB) :: ZLAT  (NPROMA,NFLEVG,KGPBLKS)
REAL(KIND=JPRB) :: ZLON5 (NPROMA,NFLEVG,NITMP,KGPBLKS)
REAL(KIND=JPRB) :: ZLAT5 (NPROMA,NFLEVG,NITMP,KGPBLKS)

! * interpolated quantities arrays.

INTEGER(KIND=JPIM), ALLOCATABLE :: IL0(:,:,:,:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: ILH0(:,:,:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IDEP(:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: ZDLO(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLO(:,:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLOSLD(:,:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDLAT(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLA(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLASLD(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDVER(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVINTW(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVINTWSLD(:,:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: ZDLO5(:,:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLO5(:,:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLOSLD5(:,:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDLAT5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLA5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCLASLD5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDVER5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVINTW5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVINTWSLD5(:,:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: ZCOSCO(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSINLA(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSINCO(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCOPHI(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZLEV(:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: ZCOSCO5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSINLA5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSINCO5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCOPHI5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZLEV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZOUT5(:,:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: ZINC(:,:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: INC(:,:,:)

INTEGER(KIND=JPIM),PARAMETER :: JP_LOCKS=32
INTEGER(KIND=JPIM) :: ILOCK(JP_LOCKS)
INTEGER(KIND=JPIM) :: JII
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "lapinea5.intfb.h"
#include "lapineaad.intfb.h"
#include "lapinebad.intfb.h"
#include "slcomm.intfb.h"
#include "slcomm2.intfb.h"
#include "slcomm2a.intfb.h"
#include "slextpol.intfb.h"
#include "slextpolad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CALL_SL_AD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    PRELIMINARY ALLOCATIONS.
!              ------------------------

ALLOCATE(IL0(NPROMA,NFLEVG,0:3,NITMP,KGPBLKS))
ALLOCATE(ILH0(NPROMA,NFLEVG,0:3,KGPBLKS))
ALLOCATE(IDEP(NPROMA,NFLEVG,KGPBLKS))

ALLOCATE(ZDLO(NPROMA,NFLEVG,0:3,KGPBLKS))
ALLOCATE(ZCLO(NPROMA,NFLEVG,3,2,KGPBLKS))
ALLOCATE(ZCLOSLD(NPROMA,NFLEVG,3,2,KGPBLKS))
ALLOCATE(ZDLAT(NPROMA,NFLEVG,KGPBLKS))
ALLOCATE(ZCLA(NPROMA,NFLEVG,3,KGPBLKS))
ALLOCATE(ZCLASLD(NPROMA,NFLEVG,3,KGPBLKS))
ALLOCATE(ZDVER(NPROMA,NFLEVG,KGPBLKS))
ALLOCATE(ZVINTW(NPROMA,NFLEVG,3,KGPBLKS))
ALLOCATE(ZVINTWSLD(NPROMA,NFLEVG,3,KGPBLKS))

ALLOCATE(ZDLO5(NPROMA,NFLEVG,0:3,NITMP,KGPBLKS))
ALLOCATE(ZCLO5(NPROMA,NFLEVG,3,2,KGPBLKS))
ALLOCATE(ZCLOSLD5(NPROMA,NFLEVG,3,2,KGPBLKS))
ALLOCATE(ZDLAT5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZCLA5(NPROMA,NFLEVG,3,KGPBLKS))
ALLOCATE(ZCLASLD5(NPROMA,NFLEVG,3,KGPBLKS))
ALLOCATE(ZDVER5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZVINTW5(NPROMA,NFLEVG,3,KGPBLKS))
ALLOCATE(ZVINTWSLD5(NPROMA,NFLEVG,3,KGPBLKS))

ALLOCATE(ZCOSCO(NPROMA,NFLEVG,KGPBLKS))
ALLOCATE(ZSINLA(NPROMA,NFLEVG,KGPBLKS))
ALLOCATE(ZSINCO(NPROMA,NFLEVG,KGPBLKS))
ALLOCATE(ZCOPHI(NPROMA,NFLEVG,KGPBLKS))
ALLOCATE(ZLEV(NPROMA,NFLEVG,KGPBLKS))

ALLOCATE(ZCOSCO5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZSINLA5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZSINCO5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZCOPHI5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZLEV5(NPROMA,NFLEVG,NITMP,KGPBLKS))
ALLOCATE(ZOUT5(NPROMA,NFLEVG,NITMP,KGPBLKS))

!     ------------------------------------------------------------------

!*  Temporary initialization

!     ------------------------------------------------------------------

!*       1.    Interface to global arrays.
!              ---------------------------

IF(OML_OMP()) THEN
  DO JII=1,JP_LOCKS
    CALL OML_SET_EVENT(KGPTOT,ILOCK(JII))
  ENDDO
ENDIF

! Physics is called in CPGLAG
LLINC=.FALSE.

CALL GSTATS(35,2)
CALL GSTATS(63,0)

IF (NPROC > 1) THEN

  IF(LSLONDEM)THEN
    LNEW_MASK_SL=.TRUE.
    LSLONDEM_ACTIVE=.TRUE.
  ENDIF

  IF (LSLONDEM_ACTIVE) THEN
    IF( NSIM4D==0 .OR. LSLADREP )THEN
      NFIXSFLD(1)=MSLB1UR05
      NFIXSFLD(2)=MSLB1WR05+NFLEVG+NFLSUL
      CALL SLCOMM(NASLB1,NSLPROCS,NSLRPT,&
       & NSLSPT,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM,&
       & NFIXSFLD,NFLDSLB15,LLINC,0,PB15,LDSLCOMM=LSLCOMM(:,NSTEP))  
    ELSE
      NFIXSFLD(1)=1
      NFIXSFLD(2)=NFLDSLB1
      LLEXCLUDE=.FALSE.
      CALL SLCOMM2A(NASLB1,NSLPROCS,NSLRPT,&
       & NSLSPT,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM,&
       & NFIXSFLD,LLEXCLUDE,MASK_SL1,MASK_SL2T(:,NSTEP),MASK_SL2AT(:,NSTEP),MASK_SL3T(:,NSTEP),&
       & NFLDSLB15,LLINC,PB15,MFIELD_TYPE15,LDSLCOMM=LSLCOMM(:,NSTEP))
    ENDIF
  ELSE
    CALL SLCOMM(NASLB1,NSLPROCS,NSLRPT,&
     & NSLSPT,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM,&
     & IDUMARR,NFLDSLB15,LLINC,0,PB15)  
  ENDIF

ENDIF

IF (LSLONDEM_ACTIVE) THEN
  IF(ALLOCATED(MASK_SL2))DEALLOCATE(MASK_SL2)
  ALLOCATE(MASK_SL2(NASLB1+4))
  IF(ALLOCATED(MASK_SL2A))DEALLOCATE(MASK_SL2A)
  ALLOCATE(MASK_SL2A(NASLB1+4))
  IF(ALLOCATED(MASK_SL3))DEALLOCATE(MASK_SL3)
  ALLOCATE(MASK_SL3(NASLB1+4))
  MASK_SL2=0
  MASK_SL2A=0
  MASK_SL3=0

  CALL SLEXTPOL(NSLWIDE,NSLWIDE,&
   & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDSLB15,NSLSTA,NSLONL,NSLOFF,&
   & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,NFIXSFLD,3,&
   & RPARSL15,PB15,KMASK3=MASK_SL3)  
ELSE
  CALL SLEXTPOL(NSLWIDE,NSLWIDE,&
   & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDSLB15,NSLSTA,NSLONL,NSLOFF,&
   & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,IDUMARR,1,&
   & RPARSL15,PB15)  
ENDIF

CALL GSTATS(63,1)
CALL GSTATS(35,3)

CALL GSTATS(1017,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JSTGLO,IST,IEND,IBL,IOFF)

DO JSTGLO=1,KGPTOT,NPROMA
  IST=1
  IEND=MIN(NPROMA,KGPTOT-JSTGLO+1)
  IBL=(JSTGLO-1)/NPROMA+1
  IOFF=JSTGLO

  CALL LAPINEA5(IST,IEND,.TRUE.,&
   & PCOLON(IOFF),PSILON(IOFF),PGM(IOFF),PGEMU(IOFF),&
   & PGSQM2(IOFF),PGECLO(IOFF),PGESLO(IOFF),PGNORDL(IOFF),&
   & PGNORDM(IOFF),PRINDX(IOFF),PRINDY(IOFF),&
   & PB15(1,1),PB2(1,1,IBL),&
   & ZLON5(1,1,1,IBL),ZLAT5(1,1,1,IBL),&
   & ZR5(1,1,1,IBL),ZS5(1,1,1,IBL),&
   & ZUF5(1,1,1,IBL),ZVF5(1,1,1,IBL),&
   & IL0(1,1,0,1,IBL),ILH0(1,1,0,IBL),&
   & ZDLO5(1,1,0,1,IBL),ZCLO5(1,1,1,1,IBL),ZCLOSLD5(1,1,1,1,IBL),&
   & ZDLAT5(1,1,1,IBL),ZCLA5(1,1,1,IBL),ZCLASLD5(1,1,1,IBL),&
   & ZDVER5(1,1,1,IBL),ZVINTW5(1,1,1,IBL),ZVINTWSLD5(1,1,1,IBL),&
   & ZCOSCO5(1,1,1,IBL),ZSINCO5(1,1,1,IBL),&
   & ZCOPHI5(1,1,1,IBL),ZSINLA5(1,1,1,IBL),&
   & ZLEV5(1,1,1,IBL),ZOUT5(1,1,1,IBL),IDEP(1,1,IBL))
ENDDO

!$OMP END PARALLEL DO
CALL GSTATS(1017,1)

IF (NPROC > 1 .AND. LSLONDEM_ACTIVE) THEN

  IF( NSIM4D==0 .OR. LSLADREP )THEN
    LLINC=.FALSE.
  
    CALL GSTATS(35,2)
    CALL GSTATS(64,0)
  
    NFIXSFLD(1)=MSLB1UR05
    NFIXSFLD(2)=MSLB1WR05+NFLEVG+NFLSUL
    LLEXCLUDE=.TRUE.
  
    CALL SLCOMM2A(NASLB1,NSLPROCS,NSLRPT,&
     & NSLSPT,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM,&
     & NFIXSFLD,LLEXCLUDE,MASK_SL1,MASK_SL2,MASK_SL2A,MASK_SL3,&
     & NFLDSLB15,LLINC,PB15,MFIELD_TYPE15,LDSLCOMM=LSLCOMM(:,NSTEP))  
  
    CALL SLEXTPOL(NSLWIDE,NSLWIDE,&
     & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDSLB15,NSLSTA,NSLONL,NSLOFF,&
     & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,NFIXSFLD,2,&
     & RPARSL15,PB15)  
  
    CALL GSTATS(64,1)
    CALL GSTATS(35,3)
  ENDIF

ENDIF

CALL GSTATS(1018,0)

LL2TLFFO=L2TLFF.AND.(LMPHYS.OR.LEPHYS).AND.(.NOT.LAGPHY)

! Compute number of updates for LARCINBAD:
IDIM=0
IF (NWLAG == 3) IDIM=IDIM+8
IF (NWLAG == 3 .AND. LL2TLFFO) IDIM=IDIM+32
IDIM=IDIM+32
IF (NWLAG == 3) IDIM=IDIM+8
IF (NWLAG == 3 .AND. LL2TLFFO) IDIM=IDIM+32
IDIM=IDIM+32
IF (NTLAG == 3) IDIM=IDIM+8
IDIM=IDIM+32
DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LADV) IDIM=IDIM+32
ENDDO
IF (NVLAG == 2) IDIM=IDIM+32
IF (NVLAG == 3) IDIM=IDIM+8
IDIM=IDIM+12

IF (LVECADIN) THEN
  IDIM1=IDIM*NFLEVG+1
  IDIM2=NPROMA
ELSE
  IDIM1=IDIM
  IDIM2=1
ENDIF

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) &
!$OMP& PRIVATE(JSTGLO,IST,IEND,IBL,IOFF,JII,ZINC,INC) &
!$OMP& PRIVATE(JINC,ISTART,ISTOP,ILEVELS,JROF,JLEV)

DO JSTGLO=1,KGPTOT,NPROMA

  ALLOCATE(ZINC(IDIM1,IDIM2,1))
  ALLOCATE( INC(IDIM1,IDIM2,1))

  IST=1
  IEND=MIN(NPROMA,KGPTOT-JSTGLO+1)
  IBL=(JSTGLO-1)/NPROMA+1
  IOFF=JSTGLO
  IF(OML_OMP().AND.JSTGLO == 1) THEN
    DO JII=1,JP_LOCKS
      CALL OML_SET_EVENT(1,ILOCK(JII))
    ENDDO
  ENDIF

  CALL LAPINEBAD(IST,IEND,JSTGLO,&
   & PCOLON(IOFF),PSILON(IOFF),PGM(IOFF),PGEMU(IOFF),&
   & PGSQM2(IOFF),PGECLO(IOFF),PGESLO(IOFF),PGNORDL(IOFF),&
   & PGNORDM(IOFF),PRINDX(IOFF),PRINDY(IOFF),&
   & PB1(1,1),PB15(1,1), &
   ! -----------------------------------------------------------------------
   & PGMVT1(1,1,1,IBL),PGMVT1S(1,1,IBL),PGFLT1(1,1,1,IBL),PB2(1,1,IBL), &
   & ZINC,INC,IDIM,IDIM1,IDIM2,&
   ! -----------------------------------------------------------------------
   & ZLON(1,1,IBL),ZLAT(1,1,IBL),ZR(1,1,IBL),ZS(1,1,IBL),&
   & ZLON5(1,1,NITMP,IBL),ZLAT5(1,1,NITMP,IBL),&
   & ZR5(1,1,NITMP,IBL),ZS5(1,1,NITMP,IBL),&
   ! -----------------------------------------------------------------------
   & IL0(1,1,0,NITMP,IBL),ILH0(1,1,0,IBL),&
   & ZDLO(1,1,0,IBL),ZCLO(1,1,1,1,IBL),ZCLOSLD(1,1,1,1,IBL),&
   & ZDLAT(1,1,IBL),ZCLA(1,1,1,IBL),ZCLASLD(1,1,1,IBL),&
   & ZDVER(1,1,IBL),ZVINTW(1,1,1,IBL),ZVINTWSLD(1,1,1,IBL),&
   ! -----------------------------------------------------------------------
   & ZDLO5(1,1,0,NITMP,IBL),ZCLO5(1,1,1,1,IBL),ZCLOSLD5(1,1,1,1,IBL),&
   & ZDLAT5(1,1,NITMP,IBL),ZCLA5(1,1,1,IBL),ZCLASLD5(1,1,1,IBL),&
   & ZDVER5(1,1,NITMP,IBL),ZVINTW5(1,1,1,IBL),ZVINTWSLD5(1,1,1,IBL),&
   ! -----------------------------------------------------------------------
   & ZCOSCO(1,1,IBL),ZSINCO(1,1,IBL),ZCOPHI(1,1,IBL),ZSINLA(1,1,IBL),&
   & ZLEV(1,1,IBL),&
   ! -----------------------------------------------------------------------
   & IDEP(1,1,IBL),&
   ! -----------------------------------------------------------------------
   & ILOCK)

  IF (LVECADIN) THEN

    ! The global update (moved from LARCINBAD to here)

    !!! NOTE: On no account should the following loop ordering be changed!
    !!! (loop on JINC must be innermost for vectorization;
    !!! the chopping is done to guarantee safety of the vectorization;
    !!! outer loop must be on JROF for reproducibility)

!$OMP CRITICAL
    DO JROF=IST,IEND
      ISTOP=0
      DO JLEV=1,NVSEPC
        ILEVELS=(NFLEVG-JLEV)/NVSEPC + 1
        ISTART=ISTOP+1
        ISTOP=ISTART+ILEVELS*IDIM-1
#if defined(VPP)
!OCL NOVREC
#elif defined(NECSX)
!CDIR VWORK=STACK
!CDIR VWORKSZ=4M
!CDIR UNROLL=4
!CDIR NODEP
!CDIR GTHREORDER
!CDIR VOVERTAKE(PB1)
#endif
        DO JINC=ISTART,ISTOP
          PB1(INC(JINC,JROF,1),1)=PB1(INC(JINC,JROF,1),1)+ZINC(JINC,JROF,1)
        ENDDO
      ENDDO
    ENDDO

!$OMP END CRITICAL
  ENDIF

  DEALLOCATE(ZINC)
  DEALLOCATE(INC)

ENDDO

!$OMP END PARALLEL DO

IF(OML_OMP()) THEN
  DO JII=1,JP_LOCKS
    CALL OML_SET_EVENT(KGPTOT,ILOCK(JII))
  ENDDO
ENDIF

! Compute number of updates for LARCINAAD:
! 3D-linear interpolations, 8 each :
IDIM=24
IF (LVECADIN) THEN
  IDIM1=IDIM*NFLEVG+1
ELSE
  IDIM1=IDIM
ENDIF
  
!$OMP PARALLEL DO SCHEDULE (DYNAMIC,1) &
!$OMP& PRIVATE(JSTGLO,IST,IEND,IBL,IOFF,JII,ZINC,INC ) &
!$OMP& PRIVATE(JINC,ISTART,ISTOP,ILEVELS,JROF,JLEV,JITER)


DO JSTGLO=1,KGPTOT,NPROMA

  ALLOCATE(ZINC(IDIM1,IDIM2,NITMP))
  ALLOCATE( INC(IDIM1,IDIM2,NITMP))

  IST=1
  IEND=MIN(NPROMA,KGPTOT-JSTGLO+1)
  IBL=(JSTGLO-1)/NPROMA+1
  IOFF=JSTGLO
  IF(OML_OMP().AND.JSTGLO == 1) THEN
    DO JII=1,JP_LOCKS
      CALL OML_SET_EVENT(1,ILOCK(JII))
    ENDDO
  ENDIF

  CALL LAPINEAAD(IST,IEND,JSTGLO,&
   & PCOLON(IOFF),PSILON(IOFF),&
   & PGM(IOFF),PGEMU(IOFF),&
   & PGSQM2(IOFF),PGECLO(IOFF),PGESLO(IOFF),PGNORDL(IOFF),&
   & PGNORDM(IOFF),PRINDX(IOFF),PRINDY(IOFF),&
   & ZINC,INC,IDIM1,IDIM2,&
   & PB1(1,1),PB2(1,1,IBL),PB15(1,1),&
   ! -----------------------------------------------------------------------
   & ZLON(1,1,IBL),ZLAT(1,1,IBL),ZR(1,1,IBL),ZS(1,1,IBL),&
   & ZLON5(1,1,1,IBL),ZLAT5(1,1,1,IBL),ZR5(1,1,1,IBL),ZS5(1,1,1,IBL),&
   ! -----------------------------------------------------------------------
   & IL0(1,1,0,1,IBL),&
   & ZDLO(1,1,0,IBL),ZCLO(1,1,1,1,IBL),ZCLOSLD(1,1,1,1,IBL),&
   & ZDLAT(1,1,IBL),ZCLA(1,1,1,IBL),ZCLASLD(1,1,1,IBL),&
   & ZDVER(1,1,IBL),ZVINTW(1,1,1,IBL),ZVINTWSLD(1,1,1,IBL),&
   & ZDLO5(1,1,0,1,IBL),ZDLAT5(1,1,1,IBL),ZDVER5(1,1,1,IBL),&
   ! -----------------------------------------------------------------------
   & ZCOSCO(1,1,IBL),ZSINCO(1,1,IBL),ZCOPHI(1,1,IBL),ZSINLA(1,1,IBL),&
   & ZLEV(1,1,IBL),&
   & ZCOSCO5(1,1,1,IBL),ZSINCO5(1,1,1,IBL),&
   & ZCOPHI5(1,1,1,IBL),ZSINLA5(1,1,1,IBL),&
   & ZLEV5(1,1,1,IBL),&
   ! -----------------------------------------------------------------------
   & ZUF5(1,1,1,IBL),ZVF5(1,1,1,IBL),ZOUT5(1,1,1,IBL),&
   & ILOCK)

  IF (LVECADIN) THEN
!$OMP CRITICAL

    ! Finally, do all the updates together (if in vector mode):

    !!! NOTE: On no account should the following loop ordering be changed!
    !!! (loop on JINC must be innermost for vectorization;
    !!! the chopping is done to guarantee safety of the vectorization;
    !!! outer loop must be JITER (counted backward) and JROF for
    !!! reproducibility)

    DO JITER=NITMP-1,1,-1
      DO JROF=IST,IEND
      ISTOP=0
        DO JLEV=1,NVSEPL
          ILEVELS=(NFLEVG-JLEV)/NVSEPL + 1
          ISTART=ISTOP+1
          ISTOP=ISTART+ILEVELS*IDIM-1
#if defined(VPP)
!OCL NOVREC
#elif defined(NECSX)
!CDIR VWORK=STACK
!CDIR VWORKSZ=4M
!CDIR UNROLL=4
!CDIR NODEP
!CDIR GTHREORDER
!CDIR VOVERTAKE(PB1)
#endif
          DO JINC=ISTART,ISTOP
            PB1(INC(JINC,JROF,JITER),1)=PB1(INC(JINC,JROF,JITER),1) &
             & +ZINC(JINC,JROF,JITER)
          ENDDO
        ENDDO
      ENDDO
    ENDDO

!$OMP END CRITICAL
  ENDIF

  DEALLOCATE(ZINC)
  DEALLOCATE(INC)


ENDDO

!$OMP END PARALLEL DO

CALL GSTATS(1018,1)

IF(.NOT.LSLADREP)THEN

  CALL SLEXTPOLAD(NSLWIDE,&
    & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDSLB1,NSLSTA,NSLONL,NSLOFF,&
    & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
    & RPARSL1,PB1,MASK_SL1)

  IF(LSLONDEM_ACTIVE)THEN

    NFIXSFLD(1)=1
    NFIXSFLD(2)=0

    IF (NPROC > 1) THEN
      LLINC=.TRUE.
      CALL GSTATS(85,0)
      CALL SLCOMM2(NASLB1,NSLPROCS,NSLSPT,&
       & NSLRPT,NSLRECVPOS,NSLSENDPOS,NRECVPTR,NSENDPTR,NSLCOMM,&
       & NFIXSFLD,MASK_SL1,MASK_SL2,&
       & NFLDSLB1,LLINC,PB1,LDSLCOMM=LSLCOMM(:,NSTEP))
      CALL GSTATS(85,1)
    ENDIF

  ELSE

      IF (NPROC > 1) THEN
      LLINC=.TRUE.
      CALL GSTATS(85,0)
      CALL SLCOMM(NASLB1,NSLPROCS,NSLSPT,&
       & NSLRPT,NSLRECVPOS,NSLSENDPOS,NRECVPTR,NSENDPTR,NSLCOMM,&
       & IDUMARR,NFLDSLB1,LLINC,0,PB1)
      CALL GSTATS(85,1)
    ENDIF

  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       2.    FINAL ACTIONS AND DESALLOCATIONS.
!              ---------------------------------

DEALLOCATE(ZDLO)
DEALLOCATE(ZCLO)
DEALLOCATE(ZCLOSLD)
DEALLOCATE(IL0)
DEALLOCATE(ILH0)
DEALLOCATE(IDEP)
DEALLOCATE(ZDLAT)
DEALLOCATE(ZCLA)
DEALLOCATE(ZCLASLD)
DEALLOCATE(ZDVER)
DEALLOCATE(ZVINTW)
DEALLOCATE(ZVINTWSLD)
DEALLOCATE(ZDLO5)
DEALLOCATE(ZCLO5)
DEALLOCATE(ZCLOSLD5)
DEALLOCATE(ZDLAT5)
DEALLOCATE(ZCLA5)
DEALLOCATE(ZCLASLD5)
DEALLOCATE(ZDVER5)
DEALLOCATE(ZVINTW5)
DEALLOCATE(ZVINTWSLD5)
DEALLOCATE(ZCOSCO)
DEALLOCATE(ZSINLA)
DEALLOCATE(ZSINCO)
DEALLOCATE(ZCOPHI)
DEALLOCATE(ZLEV)
DEALLOCATE(ZCOSCO5)
DEALLOCATE(ZSINLA5)
DEALLOCATE(ZSINCO5)
DEALLOCATE(ZCOPHI5)
DEALLOCATE(ZLEV5)
DEALLOCATE(ZOUT5)

LSLONDEM_ACTIVE=.FALSE.
LNEW_MASK_SL=.FALSE.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CALL_SL_AD',1,ZHOOK_HANDLE)

END SUBROUTINE CALL_SL_AD
