SUBROUTINE CPEULDYNTL(CDCONF,&
 & KST,KPROF,PDT,&
 & PGFL,&
 & PRCORI,PGM,PRATATH,PRATATX,PSDVBC,PKENE,PEVEL,PVVEL,PSGRTL,PSGRTM,&
 & PRES0,PRDELP,PKAP,&
 & PGFLT1,PGMVT1,PGMVT1S,PGMV,PGMVS,&
 & PB2,&
 & PGMV5,PGFL5,PSDVBC5,PEVEL5,PVVEL5,PRES5,PRDELP5,PKAP5)

!**** *CPEULDYNTL* - tangent linear dynamic grid point calculations
!                    for Eulerian model.

!     Purpose.
!     --------
!     Tangent Linear Dynamic computations in grid-point space for Eulerian model.

!     The following options present in the direct code are not yet coded in TL:
!     - LRUBC=T and NDPSFI=1. 
!     - LNHDYN=T.
!     - LPC_FULL=T.
!     - LVERTFE=T.

!**   Interface.
!     ----------
!        *CALL* *CPEULDYNTL(...)

!        Explicit arguments :
!        --------------------

!        INPUT (OTHER THAN "...5" VARIABLES):
!          CDCONF  - "stepo" configuration for grid-point calculations.
!          KST     - first element of work.
!          KPROF   - depth of work.
!          PDT     - SL2TL or first timestep: timestep; otherwise 2*timestep.
!          PGFL    - GFL variables at t and t-dt.
!          PRCORI  - Coriolis parameter 2.OMEGA .
!          PGM     - mapping factor.
!          PRATATH - curvature term Cy = dm/dy (used when plane).
!          PRATATX - curvature term Cx = dm/dx (used when plane).
!          PSDVBC  - vertical integral of divergence which appears in the RHS
!                    on continuity equation, including the "lrubc" and
!                    "delta m=1" contribution of (ETADOT DP/DETA).
!          PKENE   - cinetic energy.
!          PEVEL   - ETADOT dP/dETA at t.
!          PVVEL   - omega/P a t.
!          PSGRTL  - zonal component of the pressure force
!                    (grad(phi)+RT grad(p)).
!          PSGRTM  - meridian component of the pressure force.
!          PRES0   - pressure on an interlayer at t.
!          PRDELP  - 1/(pressure depth of layers) at t.
!          PKAP    - R/CP at t.

!        INPUT/OUTPUT:
!          PGFLT1  - t+dt term for the GFL equations. 
!          PGMVT1  - GMV variables at t+dt.
!          PGMVT1S - GMVS variables at t+dt.
!          PGMV    - GMV variables at t and t-dt.
!          PGMVS   - GMVS variables at t and t-dt.

!        OUTPUT:
!          PB2     - "SLBUF2" buffer.

!        INPUT "...5" VARIABLES:
!          PGMV5   - GMV variables (trajectory).
!          PGFL5   - GFL variables (trajectory).
!          PSDVBC5 - vertical integral of divergence which appears in the RHS
!                    on continuity equation, including the "lrubc" and
!                    "delta m=1" contribution of (ETADOT DP/DETA).
!          PEVEL5  - ETADOT dP/dETA (trajectory).
!          PVVEL5  - omega/P a t (trajectory).
!          PRES5   - pressure on an interlayer (trajectory).
!          PRDELP5 - 1/(pressure depth of layers) (trajectory).
!          PKAP5   - R/CP (trajectory).

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!     Reference.
!     ----------
!        ARPEGE documentation

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!   -- name = CPDYNTL --
!   Original : 90-03-22
!   Modified : 91-11-01 F Rabier (U V Formulation)
!   Modified : 92-07-08 V.Ducrocq (introduction of a new curvature term
!    in the U equation and KGL1 KGL2 PRATATX in the call)
!   Modified : MARCH 1994 by K. YESSAD: semi-implicit scheme with
!              not reduced divergence.
!   Modified : 96-02-02 L.Isaksen Message Passing option added
!   Modified : 98-01-08 E. Holm (Ozone included)
!   Modified 99-09-17 by C. Temperton : add LIMPF option
!   Modified 99-11-15 by K. YESSAD:
!    - replace "psdiv" by "psdvbc" already containing the "lrubc"
!      and "delta m=1" contribution for term (ETADOT DP/DETA).
!    - some cleaning (comments, put dummy arguments in a more logical way,
!      remove useless dummy arguments).
!   Modified 01-01-17 by C. Temperton : add LSPRT option
!   R. El Khatib : 01-08-07 Pruning options
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   -- name = CPEULDYNTL --
!   K.Yessad     : 20-03-2006: update according to the direct code
!    routine CPEULDYN and rename it into CPEULDYNTL. 
!   M.Jidane  08-04-2006 : Reintro of R dep. on q variables under key
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (Dec 2008): remove dummy CDLOCK
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG  ,NPROMA
USE YOMSC2   , ONLY : NFLDSLB2
USE YOMCT0   , ONLY : LRFRIC  ,LSPRT   ,LRUBC    ,LNHDYN   ,LPC_FULL
USE YOMCVER  , ONLY : LVERTFE
USE YOMDYN   , ONLY : BETADT  ,RKRF    ,LIMPF    ,LSIDG    ,LDRY_ECMWF
USE YOMGEM   , ONLY : RSTRET
USE YOMCST   , ONLY : RD       ,RV      ,RETV
USE YOMPHY   , ONLY : NDPSFI
USE YOM_YGFL , ONLY : YGFL    ,YGFLC    ,YQ
USE YOMGMV   , ONLY : NDIMGMV,NDIMGMVS,YT0,YT9,YT5,YT1
USE PTRSLB2  , ONLY : MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2SPSI

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDVBC(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKENE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL5(NPROMA,NFLEVG,YGFL%NDIM5) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDVBC5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP5(NPROMA,NFLEVG) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZR0   (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZR5   (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSID  (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSIDL (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSIDM (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSISP (NPROMA       )
REAL(KIND=JPRB) :: ZSISPL(NPROMA       )
REAL(KIND=JPRB) :: ZSISPM(NPROMA       )
REAL(KIND=JPRB) :: ZSIT  (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSITL (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSITM (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTT0L (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTT0M (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTT5L (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTT5M (NPROMA,NFLEVG)

REAL(KIND=JPRB) :: ZFM, ZBDT, ZED, ZED5

INTEGER(KIND=JPIM) :: IPROFS, JLEV, JROF,JGFL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "sigam.intfb.h"
#include "sitnu.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPEULDYNTL',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (NDPSFI == 1) THEN
  CALL ABOR1(' CPEULDYNTL: NDPSFI=1 not yet coded in EUL TL model')
ENDIF
IF (LRUBC) THEN
  CALL ABOR1(' CPEULDYNTL: LRUBC=T not yet coded in EUL TL model')
ENDIF
IF (LNHDYN) THEN
  CALL ABOR1(' CPEULDYNTL: LNHDYN=T not yet coded in EUL TL model')
ENDIF
IF (LPC_FULL) THEN
  CALL ABOR1(' CPEULDYNTL: LPC_FULL=T not yet coded in EUL TL model')
ENDIF
IF (LVERTFE) THEN
  CALL ABOR1(' CPEULDYNTL: LVERTFE=T not yet coded in EUL TL model')
ENDIF

!     ------------------------------------------------------------------

!*       1.    INTERMEDIATE QUANTITIES FOR EQUATIONS RHS
!              -----------------------------------------

! ky: not fully consistent with the direct code but consistent with the
!     cy33 version of cpeuldyntl.F90.
ZFM = 0.5_JPRB

! dimension for calls to SI routines
IPROFS=KPROF-KST+1

!     ------------------------------------------------------------------

!*       2.    VERTICAL ADVECTION.
!              -------------------

! * U,V,T,GFL
!   Layers 1 to nflevg-1.

DO JLEV=1,NFLEVG-1
  DO JROF=KST,KPROF
    ZED=PDT*0.5_JPRB*PEVEL(JROF,JLEV)
    ZED5=PDT*0.5_JPRB*PEVEL5(JROF,JLEV)

    PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)+&
     & (PGMV(JROF,JLEV,YT0%MU)-PGMV(JROF,JLEV+1,YT0%MU))&
     & *ZED5*PRDELP5(JROF,JLEV)+&
     & (PGMV5(JROF,JLEV,YT5%MU)-PGMV5(JROF,JLEV+1,YT5%MU))&
     & *ZED*PRDELP5(JROF,JLEV)+&
     & (PGMV5(JROF,JLEV,YT5%MU)-PGMV5(JROF,JLEV+1,YT5%MU))&
     & *ZED5*PRDELP(JROF,JLEV)  
    PGMVT1(JROF,JLEV+1,YT1%MU)=PGMVT1(JROF,JLEV+1,YT1%MU)+&
     & (PGMV(JROF,JLEV,YT0%MU)-PGMV(JROF,JLEV+1,YT0%MU))&
     & *ZED5*PRDELP5(JROF,JLEV+1)+&
     & (PGMV5(JROF,JLEV,YT5%MU)-PGMV5(JROF,JLEV+1,YT5%MU))&
     & *ZED *PRDELP5(JROF,JLEV+1)+&
     & (PGMV5(JROF,JLEV,YT5%MU)-PGMV5(JROF,JLEV+1,YT5%MU))&
     & *ZED5*PRDELP (JROF,JLEV+1)  

    PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)+&
     & (PGMV(JROF,JLEV,YT0%MV)-PGMV(JROF,JLEV+1,YT0%MV))&
     & *ZED5*PRDELP5(JROF,JLEV)+&
     & (PGMV5(JROF,JLEV,YT5%MV)-PGMV5(JROF,JLEV+1,YT5%MV))&
     & *ZED*PRDELP5(JROF,JLEV)+&
     & (PGMV5(JROF,JLEV,YT5%MV)-PGMV5(JROF,JLEV+1,YT5%MV))&
     & *ZED5*PRDELP(JROF,JLEV)  
    PGMVT1(JROF,JLEV+1,YT1%MV)=PGMVT1(JROF,JLEV+1,YT1%MV)+&
     & (PGMV(JROF,JLEV,YT0%MV)-PGMV(JROF,JLEV+1,YT0%MV))&
     & *ZED5*PRDELP5(JROF,JLEV+1)+&
     & (PGMV5(JROF,JLEV,YT5%MV)-PGMV5(JROF,JLEV+1,YT5%MV))&
     & *ZED *PRDELP5(JROF,JLEV+1)+&
     & (PGMV5(JROF,JLEV,YT5%MV)-PGMV5(JROF,JLEV+1,YT5%MV))&
     & *ZED5*PRDELP (JROF,JLEV+1)  

    PGMVT1(JROF,JLEV,YT1%MT)=PGMVT1(JROF,JLEV,YT1%MT)+&
     & (PGMV(JROF,JLEV,YT0%MT)-PGMV(JROF,JLEV+1,YT0%MT))&
     & *ZED5*PRDELP5(JROF,JLEV)+&
     & (PGMV5(JROF,JLEV,YT5%MT)-PGMV5(JROF,JLEV+1,YT5%MT))&
     & *ZED*PRDELP5(JROF,JLEV)+&
     & (PGMV5(JROF,JLEV,YT5%MT)-PGMV5(JROF,JLEV+1,YT5%MT))&
     & *ZED5*PRDELP(JROF,JLEV)  
    PGMVT1(JROF,JLEV+1,YT1%MT)=PGMVT1(JROF,JLEV+1,YT1%MT)+&
     & (PGMV(JROF,JLEV,YT0%MT)-PGMV(JROF,JLEV+1,YT0%MT))&
     & *ZED5*PRDELP5(JROF,JLEV+1)+&
     & (PGMV5(JROF,JLEV,YT5%MT)-PGMV5(JROF,JLEV+1,YT5%MT))&
     & *ZED *PRDELP5(JROF,JLEV+1)+&
     & (PGMV5(JROF,JLEV,YT5%MT)-PGMV5(JROF,JLEV+1,YT5%MT))&
     & *ZED5*PRDELP (JROF,JLEV+1)  

  ENDDO
ENDDO

DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LADV) THEN
    DO JLEV=1,NFLEVG-1
      DO JROF=KST,KPROF
        ZED=PDT*0.5_JPRB*PEVEL(JROF,JLEV)
        ZED5=PDT*0.5_JPRB*PEVEL5(JROF,JLEV)
        PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)=PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)&
         & +(PGFL(JROF,JLEV,YGFLC(JGFL)%MP)-PGFL(JROF,JLEV+1,YGFLC(JGFL)%MP))&
         & *ZED5*PRDELP5(JROF,JLEV)+(&
         & PGFL5(JROF,JLEV,YGFLC(JGFL)%MP5)-PGFL5(JROF,JLEV+1,YGFLC(JGFL)%MP5)&
         & )*ZED*PRDELP5(JROF,JLEV)+(&
         & PGFL5(JROF,JLEV,YGFLC(JGFL)%MP5)-PGFL5(JROF,JLEV+1,YGFLC(JGFL)%MP5)&
         & )*ZED5*PRDELP(JROF,JLEV)  
        PGFLT1(JROF,JLEV+1,YGFLC(JGFL)%MP1)=PGFLT1(JROF,JLEV+1,YGFLC(JGFL)%MP1)&
         & +(PGFL(JROF,JLEV,YGFLC(JGFL)%MP)-PGFL(JROF,JLEV+1,YGFLC(JGFL)%MP))&
         & *ZED5*PRDELP5(JROF,JLEV+1)+(&
         & PGFL5(JROF,JLEV,YGFLC(JGFL)%MP5)-PGFL5(JROF,JLEV+1,YGFLC(JGFL)%MP5)&
         & )*ZED*PRDELP5(JROF,JLEV+1)+(&
         & PGFL5(JROF,JLEV,YGFLC(JGFL)%MP5)-PGFL5(JROF,JLEV+1,YGFLC(JGFL)%MP5)&
         & )*ZED5*PRDELP(JROF,JLEV+1)  
      ENDDO
    ENDDO
  ENDIF
ENDDO
  
! * U,V,T,GFL:
!   Bottom layer.
!   The vertical advection at bottom level is not yet modified when NDPSI=1,
!   since Physics outputs are needed.

!     ------------------------------------------------------------------

!*       3.    CASE "LSPRT".
!              -------------

! *  Convert gradients of 'TV' to gradients of T if necessary.
!    If no gradient of Q approximate (for EUL NMI - still needed??)

!        3.0     Trajectory code to compute ZR5, ZTT5L, ZTT5M.

IF (LSPRT) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZR5(JROF,JLEV)=RD+(RV-RD)*PGFL5(JROF,JLEV,YQ%MP5)
    ENDDO
    IF (YQ%LCDERS) THEN
      DO JROF=KST,KPROF
        ZTT5L(JROF,JLEV)= &
         & RD*(PGMV5(JROF,JLEV,YT5%MTL)-RETV*PGMV5(JROF,JLEV,YT5%MT)&
         & *PGFL5(JROF,JLEV,YQ%MP5L))/ZR5(JROF,JLEV)  
        ZTT5M(JROF,JLEV)= &
         & RD*(PGMV5(JROF,JLEV,YT5%MTM)-RETV*PGMV5(JROF,JLEV,YT5%MT)&
         & *PGFL5(JROF,JLEV,YQ%MP5M))/ZR5(JROF,JLEV)  
      ENDDO
    ELSE
      DO JROF=KST,KPROF
        ZTT5L(JROF,JLEV)=RD*PGMV5(JROF,JLEV,YT5%MTL)/ZR5(JROF,JLEV)
        ZTT5M(JROF,JLEV)=RD*PGMV5(JROF,JLEV,YT5%MTM)/ZR5(JROF,JLEV)
      ENDDO
    ENDIF
  ENDDO
ELSE
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZTT5L(JROF,JLEV)=PGMV5(JROF,JLEV,YT5%MTL)
      ZTT5M(JROF,JLEV)=PGMV5(JROF,JLEV,YT5%MTM)
    ENDDO
  ENDDO
ENDIF

!        3.1     TL code.

IF (LSPRT) THEN
  DO JLEV=1,NFLEVG
    IF (.NOT. LDRY_ECMWF) THEN
      DO JROF=KST,KPROF
        ZR0(JROF,JLEV)=(RV-RD)*PGFL(JROF,JLEV,YQ%MP)
      ENDDO
    ENDIF
  ENDDO
  DO JLEV=1,NFLEVG
    IF (YQ%LCDERS) THEN
      DO JROF=KST,KPROF
        ZTT0L(JROF,JLEV)=RD*(+((PGMV(JROF,JLEV,YT0%MTL) &
         & -RETV*(PGMV5(JROF,JLEV,YT5%MT)*PGFL(JROF,JLEV,YQ%MPL)&
         & +PGMV(JROF,JLEV,YT0%MT)*PGFL5(JROF,JLEV,YQ%MP5L)))/ZR5(JROF,JLEV)))  
        ZTT0M(JROF,JLEV)=RD*(+((PGMV(JROF,JLEV,YT0%MTM) &
         & -RETV*(PGMV5(JROF,JLEV,YT5%MT)*PGFL(JROF,JLEV,YQ%MPM)&
         & +PGMV(JROF,JLEV,YT0%MT)*PGFL5(JROF,JLEV,YQ%MP5M)))/ZR5(JROF,JLEV)))  
      ENDDO
      IF (.NOT. LDRY_ECMWF) THEN
        DO JROF=KST,KPROF
          ZTT0L(JROF,JLEV)=ZTT0L(JROF,JLEV)&
            & +RD*(-(PGMV5(JROF,JLEV,YT5%MTL)-RETV*PGMV5(JROF,JLEV,YT5%MT)&
            & *PGFL5(JROF,JLEV,YQ%MP5L))*ZR0(JROF,JLEV)&
            & /(ZR5(JROF,JLEV)*ZR5(JROF,JLEV)))
          ZTT0M(JROF,JLEV)=ZTT0M(JROF,JLEV)&
            & +RD*(-(PGMV5(JROF,JLEV,YT5%MTM)-RETV*PGMV5(JROF,JLEV,YT5%MT)&
            & *PGFL5(JROF,JLEV,YQ%MP5M))*ZR0(JROF,JLEV)&
            & /(ZR5(JROF,JLEV)*ZR5(JROF,JLEV)))
        ENDDO
      ENDIF
    ELSE
      DO JROF=KST,KPROF
        ZTT0L(JROF,JLEV)=RD*(PGMV(JROF,JLEV,YT0%MTL)/ZR5(JROF,JLEV))
        ZTT0M(JROF,JLEV)=RD*(PGMV(JROF,JLEV,YT0%MTM)/ZR5(JROF,JLEV))
      ENDDO
      IF (.NOT. LDRY_ECMWF) THEN
        DO JROF=KST,KPROF
          ZTT0L(JROF,JLEV)=ZTT0L(JROF,JLEV)+RD*(-PGMV5(JROF,JLEV,YT5%MTL) &
           & *ZR0(JROF,JLEV)/(ZR5(JROF,JLEV)*ZR5(JROF,JLEV)))
          ZTT0M(JROF,JLEV)=ZTT0M(JROF,JLEV)+RD*(-PGMV5(JROF,JLEV,YT5%MTM) &
           & *ZR0(JROF,JLEV)/(ZR5(JROF,JLEV)*ZR5(JROF,JLEV)))
        ENDDO
      ENDIF
    ENDIF
  ENDDO
ELSE
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZTT0L(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MTL)
      ZTT0M(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MTM)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*       4.    HORIZONTAL ADVECTION.
!              ---------------------

! * U,V,T,GFL:

DO JLEV=1,NFLEVG
  DO JROF=KST,KPROF

    PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)&
     & +PDT* ((PGMV5(JROF,JLEV,YT5%MVOR)+PRCORI(JROF))*PGMV(JROF,JLEV,YT0%MV)&
     & +PGMV(JROF,JLEV,YT0%MVOR)*PGMV5(JROF,JLEV,YT5%MV)&
     & -PGMV5(JROF,JLEV,YT5%MV)*PGMV(JROF,JLEV,YT0%MVL)&
     & -PGMV(JROF,JLEV,YT0%MV)*PGMV5(JROF,JLEV,YT5%MVL)&
     & -PGMV5(JROF,JLEV,YT5%MU)*PGMV(JROF,JLEV,YT0%MUL)&
     & -PGMV(JROF,JLEV,YT0%MU)*PGMV5(JROF,JLEV,YT5%MUL)&
     & +PKENE(JROF,JLEV)*PRATATX(JROF)&
     & )  

    PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)&
     & +PDT* (-PGMV5(JROF,JLEV,YT5%MDIV)*PGMV(JROF,JLEV,YT0%MV)&
     & -PGMV(JROF,JLEV,YT0%MDIV)*PGMV5(JROF,JLEV,YT5%MV)&
     & -PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)&
     & -PGMV5(JROF,JLEV,YT5%MU)*PGMV(JROF,JLEV,YT0%MVL)&
     & -PGMV(JROF,JLEV,YT0%MU)*PGMV5(JROF,JLEV,YT5%MVL)&
     & +PGMV5(JROF,JLEV,YT5%MV)*PGMV(JROF,JLEV,YT0%MUL)&
     & +PGMV(JROF,JLEV,YT0%MV)*PGMV5(JROF,JLEV,YT5%MUL)&
     & -PKENE(JROF,JLEV)*PRATATH(JROF)&
     & )  

    PGMVT1(JROF,JLEV,YT1%MT)=PGMVT1(JROF,JLEV,YT1%MT)&
     & -PDT*(ZTT0L(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MU)&
     & +ZTT5L(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MU)&
     & +ZTT0M(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MV)&
     & +ZTT5M(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MV))

  ENDDO
ENDDO

DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LCDERS .AND. YGFLC(JGFL)%LADV) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)=&
         & PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)-PDT* &
         & (PGFL(JROF,JLEV,YGFLC(JGFL)%MPL)*PGMV5(JROF,JLEV,YT5%MU)&
         & +PGFL5(JROF,JLEV,YGFLC(JGFL)%MP5L)*PGMV(JROF,JLEV,YT0%MU)&
         & +PGFL(JROF,JLEV,YGFLC(JGFL)%MPM)*PGMV5(JROF,JLEV,YT5%MV)&
         & +PGFL5(JROF,JLEV,YGFLC(JGFL)%MP5M)*PGMV(JROF,JLEV,YT0%MV))  
      ENDDO
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

!*       5.    SEMI-IMPLICIT CORRECTION.
!              -------------------------

! ky: part 5 is purely linear and normally should be identical to part 5
!     of cpeuldyn.F90, for the blocks which are not under LSPRT.

ZBDT=0.0_JPRB
IF(CDCONF == 'A')THEN
  ZBDT=BETADT
ELSEIF(CDCONF == 'E')THEN
  ZBDT=0.0_JPRB
ENDIF

!        5.1     Prepare input arrays for SI calculations

! * GMV variables:

DO JLEV=1,NFLEVG
  IF (LSIDG) THEN
    DO JROF=KST,KPROF
      ZSID(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MDIV)-2.0_JPRB*PGMV(JROF,JLEV,YT0%MDIV)
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      ZSID(JROF,JLEV)= &
       & (PGMV(JROF,JLEV,YT9%MDIV)-2.0_JPRB*PGMV(JROF,JLEV,YT0%MDIV))&
       & *RSTRET*RSTRET/(PGM(JROF)*PGM(JROF))  
    ENDDO
  ENDIF
  DO JROF=KST,KPROF
    ZSITL(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MTL)-2.0_JPRB*PGMV(JROF,JLEV,YT0%MTL)
    ZSITM(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MTM)-2.0_JPRB*PGMV(JROF,JLEV,YT0%MTM)
  ENDDO
ENDDO

! * GMVS variables:
DO JROF=KST,KPROF
  ZSISPL(JROF)=PGMVS(JROF,YT9%MSPL)-2.0_JPRB*PGMVS(JROF,YT0%MSPL)
  ZSISPM(JROF)=PGMVS(JROF,YT9%MSPM)-2.0_JPRB*PGMVS(JROF,YT0%MSPM)
ENDDO

!        5.2     Do SI calculations
!                This should match what is in LASSIE/LANHSI

CALL SITNU(NPROMA,1,ZSID(KST,1),ZSIT(KST,1),ZSISP(KST),IPROFS)
CALL SIGAM(NPROMA,1,ZSIDL(KST,1),ZSITL(KST,1),ZSISPL(KST),IPROFS)
CALL SIGAM(NPROMA,1,ZSIDM(KST,1),ZSITM(KST,1),ZSISPM(KST),IPROFS)

IF (LIMPF) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZSIDL(JROF,JLEV)=ZSIDL(JROF,JLEV) &
       & -PRCORI(JROF)*(PGMV(JROF,JLEV,YT9%MV)-2.0_JPRB*PGMV(JROF,JLEV,YT0%MV))
      ZSIDM(JROF,JLEV)=ZSIDM(JROF,JLEV) &
       & +PRCORI(JROF)*(PGMV(JROF,JLEV,YT9%MU)-2.0_JPRB*PGMV(JROF,JLEV,YT0%MU))
    ENDDO
  ENDDO
ENDIF

! * For "spectral RT" option, adjust semi-implicit term in
!   T-equation to compensate for later multiplication by R/Rd

IF (LSPRT) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZSIT(JROF,JLEV)=RD*ZSIT(JROF,JLEV)/ZR5(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!        5.3     Fill PB2(MSLB2[X]SI) arrays

DO JLEV=1,NFLEVG
  DO JROF=KST,KPROF
    PB2(JROF,MSLB2USI-1+JLEV)=-ZBDT*ZSIDL(JROF,JLEV)*PDT*ZFM
    PB2(JROF,MSLB2VSI-1+JLEV)=-ZBDT*ZSIDM(JROF,JLEV)*PDT*ZFM
    PB2(JROF,MSLB2TSI-1+JLEV)=-ZBDT*ZSIT(JROF,JLEV)*PDT*ZFM
  ENDDO
ENDDO

DO JROF=KST,KPROF
  PB2(JROF,MSLB2SPSI)= -ZBDT*ZSISP(JROF)*PDT*ZFM
ENDDO

!     ------------------------------------------------------------------

!*       6.    SEMI-IMPLICIT FOR PC SCHEME ONLY.
!              ---------------------------------

!nyc IF (LPC_FULL) THEN
!nyc   Not yet coded, but should be identical to part 6 of CPEULDYN
!nyc   (except for code under LSPRT) because this is linear code!
!nyc ENDIF

!     ------------------------------------------------------------------

!*       7+8.  Non linear tendency at time "t" and temporal advance.
!              -----------------------------------------------------

DO JLEV=1,NFLEVG
  DO JROF=KST,KPROF
    PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)-PDT*PSGRTL(JROF,JLEV)
    PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)-PDT*PSGRTM(JROF,JLEV)
    PGMVT1(JROF,JLEV,YT1%MT)=PGMVT1(JROF,JLEV,YT1%MT)&
     & +PDT*PKAP(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MT)*PVVEL5(JROF,JLEV)&
     & +PDT*PKAP5(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MT)*PVVEL5(JROF,JLEV)&
     & +PDT*PKAP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MT)*PVVEL(JROF,JLEV)  
  ENDDO
ENDDO

! * Rayleigh friction:
IF(LRFRIC) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU) &
       & -PDT*RKRF(JLEV)*PGMV(JROF,JLEV,YT0%MU)
    ENDDO
  ENDDO
ENDIF

PGMVT1S(KST:KPROF,YT1%MSP)=PGMVT1S(KST:KPROF,YT1%MSP)&
 & -PDT*PSDVBC (KST:KPROF,NFLEVG)/PRES5(KST:KPROF,NFLEVG)&
 & +PDT*PSDVBC5(KST:KPROF,NFLEVG)*PRES0(KST:KPROF,NFLEVG)&
 & /(PRES5(KST:KPROF,NFLEVG)*PRES5(KST:KPROF,NFLEVG))  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPEULDYNTL',1,ZHOOK_HANDLE)
END SUBROUTINE CPEULDYNTL

