SUBROUTINE CPFHPFS( KPROMA, KSTART, KPROF, KFLEV,&
 !  Variables 2D Input
 & PFCCQL,PFCCQN,PFCSQL,PFCSQN,&
 & PFPLCL,PFPLCN,PFPLSL,PFPLSN,&
 & PFPEVPL,PFPEVPN,&
 & PCP,PT,&
 !  Variables 1D Input
 & PCPS,PTS,&
 !  Variables 2D Out
 & PFHSCL,PFHSCN,PFHSSL,PFHSSN,&
 & PFHPCL,PFHPCN,PFHPSL,PFHPSN,&
 & PFHP  ,PFP   ,PFEPFP )  

!**** *CPFHPFS*  CALCUL DES FLUX D'ENTHALPIES ET DE CHALEURS SENSIBLES DUS
!      AUX PRECIPITATIONS ET A LA CONDENSATION .

!     ARGUMENTS D'ENTREE.
!     ------------------.
!       KSTART      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT.
!       KPROF      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
!       KPROMA       : DIMENSION HORIZONTALE DES TABLEAUX.
!       KFLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".

! - 2D (0:KFLEV).
!   -----------.
!       PFCCQL: FLUX DE CONDENSATION LIE AUX RR CONVECTIVES LIQUIDES.
!       PFCCQN: FLUX DE CONDENSATION LIE AUX RR CONVECTIVES NEIGEUSES.
!       PFCSQL: FLUX DE CONDENSATION LIE AUX RR STRATIFORMES LIQUIDES.
!       PFCSQN: FLUX DE CONDENSATION LIE AUX RR STRATIFORMES NEIGEUSES.
!       PFPEVPL:FLUX D'EVAPORATION LIE AUX RR STRATIFORMES LIQUIDES.
!       PFPEVPN:FLUX D'EVAPORATION LIE AUX RR STRATIFORMES NEIGEUSES.
!       PFPLCL: FLUX DE PRECIPITATIONS CONVECTIVES LIQUIDES.
!       PFPLCN: FLUX DE PRECIPITATIONS CONVECTIVES NEIGEUSES.
!       PFPLSL: FLUX DE PRECIPITATIONS STRATIFORMES LIQUIDES.
!       PFPLSN: FLUX DE PRECIPITATIONS STRATIFORMES NEIGEUSES.

! - 2D (1:KFLEV).
!   -----------.
!       PCP : CHALEUR SPECIFIQUE.
!       PT  : TEMPERATURE.

! - 1D.
!   --.
!       PCPS: CHALEUR SPECIFIQUE EN SURFACE.
!       PTS : TEMPERATURE DE SURFACE.

! - ARGUMENTS DE SORTIE (0:KFLEV).
!   ----------------------------.
!       PFHSCL : FLUX DE CHALEUR SENSIBLE LIE AUX RR CONVECTIVES LIQUIDES.
!       PFHSCN : FLUX DE CHALEUR SENSIBLE LIE AUX RR CONVECTIVES NEIGEUSES.
!       PFHSSL : FLUX DE CHALEUR SENSIBLE LIE AUX RR STRATIFORMES LIQUIDES.
!       PFHSSN : FLUX DE CHALEUR SENSIBLE LIE AUX RR STRATIFORMES NEIGEUSES.
!       PFHPCL : FLUX D'ENTHALPIE LIE A LA CONDENSATION CONVECTIVE LIQUIDE.
!       PFHPCN : FLUX D'ENTHALPIE LIE A LA CONDENSATION CONVECTIVE NEIGEUSE.
!       PFHPSL : FLUX D'ENTHALPIE LIE A LA CONDENSATION STRATIFORME LIQUIDE.
!       PFHPSN : FLUX D'ENTHALPIE LIE A LA CONDENSATION STRATIFORME NEIGEUSE.
!       PFHP   : FLUX TOTAL D'ENTHALPIE + FLUX DE CHALEUR SENSIBLE.
!       PFP    : FLUX TOTAL DE PRECIPITATIONS.
!       PFEPFP : PSEUDO FLUX D'ENTHALPIE LIE A L'ENERGIE PRECIPITATIONS.

! - ARGUMENTS IMPLICITES.
!   --------------------.
!       COMMON YOMCST : RCPD,RCPV,RCW,RCS,RLVZER,RLSZER
!       COMMON YOMPHY : NDPSFI

! - AUTEUR.
!   ------.
!       E.BAZILE LE 94/04/07.

!     Modifie par E.Bazile 07.03.95 : Nettoyage
!     Modifie par P. Marquet 2002-02 new PFHMLTS, PFHEVPP (for Lopez)
!     Modifie par F. Bouyssel 2006-01 introduction of PFHMLTS, PFHEVPP
!     Modifie par R. Brozkova 2007-02 introduction of LPHSPSH option
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RCPD     ,RCW      ,RCS      ,RLVZER   ,RLSZER    
USE YOMPHY   , ONLY : LPHSPSH

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHP(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFP(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEPFP(KPROMA,0:KFLEV) 

!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JLON
REAL(KIND=JPRB) :: ZCPI, ZTI, ZTS

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPFHPFS',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------

!*
!-----------------------------------------------------------------------
! --- 1. LIMITE SUPERIEURE.
!        -----------------.

DO JLON=KSTART,KPROF
  PFHSCL(JLON,0)=PFPLCL(JLON,0)*PT(JLON,1)*(RCW-RCPD)
  PFHSCN(JLON,0)=PFPLCN(JLON,0)*PT(JLON,1)*(RCS-RCPD)
  PFHSSL(JLON,0)=PFPLSL(JLON,0)*PT(JLON,1)*(RCW-RCPD)
  PFHSSN(JLON,0)=PFPLSN(JLON,0)*PT(JLON,1)*(RCS-RCPD)

  PFHPCL(JLON,0)= - RLVZER * PFCCQL(JLON,0)
  PFHPCN(JLON,0)= - RLSZER * PFCCQN(JLON,0)

  PFHPSL(JLON,0)= - RLVZER * ( PFCSQL(JLON,0) - PFPEVPL(JLON,0))
  PFHPSN(JLON,0)= - RLSZER * ( PFCSQN(JLON,0) - PFPEVPN(JLON,0))

  PFP(JLON,0)= PFPLCL(JLON,0)+PFPLCN(JLON,0)+PFPLSL(JLON,0)+PFPLSN(JLON,0)

  PFEPFP(JLON,0)= 0._JPRB

  PFHP(JLON,0) = PFHSCL(JLON,0) + PFHSCN(JLON,0) +&
   & PFHSSL(JLON,0) + PFHSSN(JLON,0) +&
   & PFHPCL(JLON,0) + PFHPCN(JLON,0) +&
   & PFHPSL(JLON,0) + PFHPSN(JLON,0) +&
   & PFEPFP(JLON,0)  
ENDDO

!*
!-----------------------------------------------------------------------
! --- 2. CALCULS NIVEAUX FLUX.
!        --------------------.

DO JLEV = 1 , KFLEV-1
  DO JLON = KSTART , KPROF
    ZTI = 0.5_JPRB * ( PT(JLON,JLEV) + PT(JLON,JLEV+1) )
    ZCPI= 0.5_JPRB * ( PCP(JLON,JLEV) + PCP(JLON,JLEV+1) )

    PFHSCL(JLON,JLEV)=PFPLCL(JLON,JLEV)*ZTI*(RCW-RCPD)
    PFHSCN(JLON,JLEV)=PFPLCN(JLON,JLEV)*ZTI*(RCS-RCPD)
    PFHSSL(JLON,JLEV)=PFPLSL(JLON,JLEV)*ZTI*(RCW-RCPD)
    PFHSSN(JLON,JLEV)=PFPLSN(JLON,JLEV)*ZTI*(RCS-RCPD)

    PFHPCL(JLON,JLEV)= - RLVZER * PFCCQL(JLON,JLEV)
    PFHPCN(JLON,JLEV)= - RLSZER * PFCCQN(JLON,JLEV)

    PFHPSL(JLON,JLEV)= - RLVZER * (PFCSQL(JLON,JLEV)-PFPEVPL(JLON,JLEV))
    PFHPSN(JLON,JLEV)= - RLSZER * (PFCSQN(JLON,JLEV)-PFPEVPN(JLON,JLEV))

    PFP(JLON,JLEV)= PFPLCL(JLON,JLEV)+PFPLCN(JLON,JLEV)+&
     & PFPLSL(JLON,JLEV)+PFPLSN(JLON,JLEV)  

    PFEPFP(JLON,JLEV)= 0._JPRB 

    PFHP(JLON,JLEV) = PFHSCL(JLON,JLEV) + PFHSCN(JLON,JLEV) +&
     & PFHSSL(JLON,JLEV) + PFHSSN(JLON,JLEV) +&
     & PFHPCL(JLON,JLEV) + PFHPCN(JLON,JLEV) +&
     & PFHPSL(JLON,JLEV) + PFHPSN(JLON,JLEV) +&
     & PFEPFP(JLON,JLEV)  
  ENDDO
ENDDO

!*
!-----------------------------------------------------------------------
! --- 3. LIMITE INFERIEURE.
!        -----------------.

DO JLON=KSTART,KPROF

  IF(LPHSPSH) THEN
    ZTS=PT(JLON,KFLEV)
  ELSE
    ZTS=PTS(JLON)
  ENDIF

  PFHSCL(JLON,KFLEV)=PFPLCL(JLON,KFLEV)*ZTS*(RCW-RCPD)
  PFHSCN(JLON,KFLEV)=PFPLCN(JLON,KFLEV)*ZTS*(RCS-RCPD)
  PFHSSL(JLON,KFLEV)=PFPLSL(JLON,KFLEV)*ZTS*(RCW-RCPD)
  PFHSSN(JLON,KFLEV)=PFPLSN(JLON,KFLEV)*ZTS*(RCS-RCPD)

  PFHPCL(JLON,KFLEV)= - RLVZER * PFCCQL(JLON,KFLEV)
  PFHPCN(JLON,KFLEV)= - RLSZER * PFCCQN(JLON,KFLEV)

  PFHPSL(JLON,KFLEV)= - RLVZER * (PFCSQL(JLON,KFLEV)-PFPEVPL(JLON,KFLEV))
  PFHPSN(JLON,KFLEV)= - RLSZER * (PFCSQN(JLON,KFLEV)-PFPEVPN(JLON,KFLEV))

  PFP(JLON,KFLEV)= PFPLCL(JLON,KFLEV)+PFPLCN(JLON,KFLEV)+&
   & PFPLSL(JLON,KFLEV)+PFPLSN(JLON,KFLEV)  

  PFEPFP(JLON,KFLEV)= 0._JPRB

  PFHP(JLON,KFLEV) = PFHSCL(JLON,KFLEV) + PFHSCN(JLON,KFLEV) +&
   & PFHSSL(JLON,KFLEV) + PFHSSN(JLON,KFLEV) +&
   & PFHPCL(JLON,KFLEV) + PFHPCN(JLON,KFLEV) +&
   & PFHPSL(JLON,KFLEV) + PFHPSN(JLON,KFLEV) +&
   & PFEPFP(JLON,KFLEV)  
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPFHPFS',1,ZHOOK_HANDLE)
END SUBROUTINE CPFHPFS
