SUBROUTINE CPG2AD(CDCONF,KNUMB,&
 & PB1,PB2,PGMV,PGMVS,&
 & PRCORI,PGM,PRATATH,PRATATX,&
 & POROG,POROGL,POROGM,&
 & PUT0,PVT0,PUT0L,PVT0L,PDIVT0,PVORT0,&
 & PSPT0,PSPT0L,PSPT0M,&
 & PUT9,PVT9,PDIVT9,&
 & PSPT9,PSPT9L,PSPT9M,&
 & PUT1,PVT1,PSPT1,&
 & PUT5,PVT5,PUT5L,PVT5L,PDIVT5,PVORT5,&
 & PSPT5,PSPT5L,PSPT5M)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPROMA
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LTWOTL
USE YOMCT2   , ONLY : NSTOP2
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : SIVP     ,TDT      ,BETADT   ,LIMPF    ,LELTRA   ,LSIDG
USE YOMGEM   , ONLY : RSTRET
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1SP9 ,MSLB1U0  ,&
 & MSLB1V0  ,MSLB1SP0 ,MSLB1UR0 ,MSLB1VR0  
USE PTRSLB2  , ONLY : MSLB2USI ,MSLB2VSI ,MSLB2URL ,MSLB2VRL
USE YOMGMV  , ONLY : NDIMGMV,NDIMGMVS

!**** *CPG2AD* - Grid point calculations, 2-D model - adjoint.

!     Purpose.
!     --------
!     adjoint grid point calculations in dynamics.

!**   Interface.
!     ----------
!        *CALL* *CPG2AD(...)

!        Explicit arguments :
!        --------------------

!        CDCONF    : configuration of work                       (I)
!        KNUMB     : number of elements of arrays for which
!                    computations are performed (MP version)     (I)
!        PB1       : Buffer for interpolations                   (I)
!        PB2       : Buffer for t+dt quantities                  (I)
!        PRCORI    : Coriolis parameter                          (I)
!        PGM       : Mapping factor                              (I)
!        PRATATH   : First curvature term                        (I)
!        PRATATX   : Second curvature term                       (I)
!        POROG     : Orography                                   (I/O)
!        POROGL    : Zonal component of orography derivative     (I/O)
!        POROGM    : Meridian component of orography derivative  (I/O)
!        PUT0      : zonal wind time t                           (I/O)
!        PVT0      : meridian wind time t                        (I/O)
!        PUT0L     : zonal derivative of zonal wind time t       (I/O)
!        PVT0L     : zonal derivative of meridian wind time t    (I/O)
!        PDIVT0    : divergence  time t                          (I/O)
!        PVORT0    : vorticity  time t                           (I/O)
!        PSPT0     : equivalent height time t                    (I/O)
!        PSPT0L    : zonal derivative of PSPT0                   (I/O)
!        PSPT0M    : meridian derivative of PSPT0                (I/O)
!        PUT5      : zonal wind time t (trajectory)              (I/O)
!        PVT5      : meridian wind time t (trajectory)           (I/O)
!        PUT5L     : zonal deriv of zonal wind time t (traj)     (I/O)
!        PVT5L     : zonal deriv of meridian wind time t (traj)  (I/O)
!        PDIVT5    : divergence  time t (trajectory)             (I/O)
!        PVORT5    : vorticity  time t (trajectory)              (I/O)
!        PSPT5     : equivalent height time t (trajectory)       (I)
!        PSPT5L    : zonal derivative of PSPT5                   (I/O)
!        PSPT5M    : meridian derivative of PSPT5                (I/O)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!      Is called by SCAN2MAD.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-02-04
!        Modified by F. Rabier : 91-07-14
!        Modified by JN Thepaut: 92-03-17 (IO scheme)
!        Modified by D.Dent    : 92-06-25   Remove event logic
!        Modified by V.Ducrocq : 92-07-08 (new curvature term)
!        Modified by L.Labbe   : 93-04-15 (orography as trajectory and
!                                          control variable)
!        Modified MAR 1994 by K. YESSAD: semi-implicit computations
!                   with not reduced divergence + harmonisation of
!                   notations between eulerian and semi-lag computations
!                   + correction of bugs.
!        Modified 95-03-01 by M.Hamrud (F90 re-write)
!        Modified (OCT 1995) by K. YESSAD: redimensioning of Z(X)NLT9 for
!           2TL SL scheme.
!        Modified 96-04-04 by L.Isaksen (Adapted for Message Passing)
!        Modified 98-01-14 by E.Holm (ozone in calls to SC2RDT9/WRT9 
!                                   and GPTF1AD/2AD)
!        Modified 98-03-04 by C. Temperton (calling sequence changed to
!                                         match that of CPG2TL)
!        Modified 98-07-29 by C. Temperton (LIMPF option added).
!        Modified 98-08-05 by K. YESSAD and C. TEMPERTON: cleanings.
!          - change NFLEVG into "1"
!          - add missing dummy arguments in CALL SC2WRT9/SC2RDT9.
!          - Use PTRSLB1 and PTRSLB2 pointers.
!          - Correction of false comments.
!          - additional cleanings consistent to the CPG2 and CPG2TL ones.
!        Modified 98-12-29 by C. Temperton (adjoint of semi-Lagrangian)
!        Modified 99-03-30 by C. Temperton (bugfix for adjoint of s-L)
!        Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!        Modified 99-06-15 by C. Temperton (bugfix in map factor scaling)
!        Modified 99-08-16 by C. Temperton (removed redundant RKROMA)
!        Modified 99-10-13 by C. Temperton (removed redundant POROG5 arrays)
!        Modified 99-11-19 by C. Temperton (bugfix, NSLB2 -> KSLB2)
!        Modified 00-03-06 by C. Temperton (more bugfixes, following CPGAD)
!        Modified 01-07-11 by K. YESSAD: relaxation of thin layer hyp for SL2TL.
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        Modified 03-2002  J.Vivoda  interface to SC2...9 routines
!        Modified 02-09-30 by P. Smolikova (iterface to SC2...9 for d4 in NH)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,1,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT0L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT0L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT0L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT0M(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT9L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT9M(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT1(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT1(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT1(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT5L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT5L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVT5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORT5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT5L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT5M(NPROMA) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSPNLT9 (NPROMA)
REAL(KIND=JPRB) :: ZBDT(NPROMA)
REAL(KIND=JPRB) :: ZDUM(NPROMA)
INTEGER(KIND=JPIM) :: IEND, IENDC, IST, ISTC, JROF  
LOGICAL :: LLFSTEP, LLLSTEP, LLSHW, LLVEQ,LLRSTEP
REAL(KIND=JPRB) :: ZBT, ZDT, ZTE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gptf2ad.intfb.h"
#include "lacdynshwad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG2AD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

IF    (CDCONF == 'A')THEN
  ZDT=TDT
  ZTE=1.0_JPRB
ELSEIF(CDCONF == 'E')THEN
  ZDT=1.0_JPRB
  ZTE=0.0_JPRB
ELSE
  ZDT=0.0_JPRB
  ZTE=0.0_JPRB
  CALL ABOR1(' CPG2AD - UNKNOWN CONFIGURATION ')
ENDIF

LLSHW=NCONF == 201.OR.NCONF == 421.OR.NCONF == 521
LLVEQ=NCONF == 202.OR.NCONF == 422.OR.NCONF == 522

IF(NSTEP > 0) THEN
  LLFSTEP=.FALSE.
ELSE
  LLFSTEP=.TRUE.
ENDIF
IF(NSTEP > 0) THEN
  LLRSTEP=.FALSE.
ELSE
  LLRSTEP=.TRUE.
ENDIF
IF(NSTEP < NSTOP2-1) THEN
  LLLSTEP=.FALSE.
ELSE
  LLLSTEP=.TRUE.
ENDIF

IST =1
IEND=KNUMB
ISTC =1
IENDC=KNUMB

!     ------------------------------------------------------------------

!*       2.    INTERFACE TO GLOBAL ARRAYS (FIRST PART)
!              ---------------------------------------

!     ------------------------------------------------------------------

!*       7.    TIME FILTER (PART 1)
!              --------------------

!     ------------------------------------------------------------------

!*       6.    MAP FACTOR
!              -----------

DO JROF=IST,IEND
  PUT0  (JROF)=PUT0  (JROF)/ PGM(JROF)
  PVT0  (JROF)=PVT0  (JROF)/ PGM(JROF)
  PUT0L (JROF)=PUT0L (JROF)/(PGM(JROF)**2)
  PVT0L (JROF)=PVT0L (JROF)/(PGM(JROF)**2)
  PDIVT0(JROF)=PDIVT0(JROF)/(PGM(JROF)**2)
  PVORT0(JROF)=PVORT0(JROF)/(PGM(JROF)**2)
  PSPT0L(JROF)=PSPT0L(JROF)/ PGM(JROF)
  PSPT0M(JROF)=PSPT0M(JROF)/ PGM(JROF)
  IF (.NOT.LELTRA) THEN
    PUT9  (JROF)=PUT9  (JROF)/ PGM(JROF)
    PVT9  (JROF)=PVT9  (JROF)/ PGM(JROF)
  ENDIF
  IF (.NOT.LTWOTL) THEN
    PDIVT9(JROF)=PDIVT9(JROF)/(PGM(JROF)**2)
    PSPT9L(JROF)=PSPT9L(JROF)/ PGM(JROF)
    PSPT9M(JROF)=PSPT9M(JROF)/ PGM(JROF)
  ENDIF
ENDDO
DO JROF=IST,IEND
  PUT5  (JROF)=PUT5  (JROF)* PGM(JROF)
  PVT5  (JROF)=PVT5  (JROF)* PGM(JROF)
  PUT5L (JROF)=PUT5L (JROF)*(PGM(JROF)**2)
  PVT5L (JROF)=PVT5L (JROF)*(PGM(JROF)**2)
  PDIVT5(JROF)=PDIVT5(JROF)*(PGM(JROF)**2)
  PVORT5(JROF)=PVORT5(JROF)*(PGM(JROF)**2)
  PSPT5L(JROF)=PSPT5L(JROF)* PGM(JROF)
  PSPT5M(JROF)=PSPT5M(JROF)* PGM(JROF)
ENDDO

!     ------------------------------------------------------------------

!*       3.    SHALLOW WATER MODEL.
!              --------------------

IF(LLSHW)THEN

  IF(.NOT.LSLAG) THEN

!*        3.1  EULERIAN SCHEME.

!*        3.1.2   SEMI-IMPLICIT ADJUSTMENT FOR EULERIAN SCHEME.

    ZBT=BETADT*0.5_JPRB*ZDT*ZTE
    DO JROF=IST,IEND
      IF (LSIDG) THEN
        ZBDT(JROF)=ZBT*SIVP(1)
      ELSE
        ZBDT(JROF)=ZBT*SIVP(1)*RSTRET*RSTRET/(PGM(JROF)*PGM(JROF))
      ENDIF
    ENDDO

    DO JROF=IST,IEND
      PDIVT0(JROF)=PDIVT0(JROF)+PSPT1(JROF)*ZBDT(JROF)*2.0_JPRB
      PDIVT9(JROF)=PDIVT9(JROF)-PSPT1(JROF)*ZBDT(JROF)
      PSPT0L(JROF)=PSPT0L(JROF)+PUT1(JROF)*ZBT*2.0_JPRB
      PSPT9L(JROF)=PSPT9L(JROF)-PUT1(JROF)*ZBT
      PSPT0M(JROF)=PSPT0M(JROF)+PVT1(JROF)*ZBT*2.0_JPRB
      PSPT9M(JROF)=PSPT9M(JROF)-PVT1(JROF)*ZBT
    ENDDO

    IF (LIMPF) THEN
      DO JROF=IST,IEND
        PVT0(JROF)=PVT0(JROF)-PRCORI(JROF)*PUT1(JROF)*ZBT*2.0_JPRB
        PVT9(JROF)=PVT9(JROF)+PRCORI(JROF)*PUT1(JROF)*ZBT
        PUT0(JROF)=PUT0(JROF)+PRCORI(JROF)*PVT1(JROF)*ZBT*2.0_JPRB
        PUT9(JROF)=PUT9(JROF)-PRCORI(JROF)*PVT1(JROF)*ZBT
      ENDDO
    ENDIF

!*        3.1.1   EXPLICIT STEP FOR EULERIAN SCHEME.

    DO JROF=IST,IEND
      PUT9  (JROF)=PUT9  (JROF)+PUT1(JROF)*ZTE
      PVORT0(JROF)=PVORT0(JROF)+PUT1(JROF)*ZDT*PVT5  (JROF)
      PVT0  (JROF)=PVT0  (JROF)+PUT1(JROF)*ZDT*PVORT5(JROF)
      PVT0L (JROF)=PVT0L (JROF)-PUT1(JROF)*ZDT*PVT5  (JROF)
      PVT0  (JROF)=PVT0  (JROF)-PUT1(JROF)*ZDT*PVT5L (JROF)
      PUT0L (JROF)=PUT0L (JROF)-PUT1(JROF)*ZDT*PUT5  (JROF)
      PUT0  (JROF)=PUT0  (JROF)-PUT1(JROF)*ZDT*PUT5L (JROF)
      PUT0  (JROF)=PUT0  (JROF)+&
       & PUT1(JROF)*ZDT*PUT5  (JROF)*PRATATX(JROF)  
      PVT0  (JROF)=PVT0  (JROF)+&
       & PUT1(JROF)*ZDT*PVT5  (JROF)*PRATATX(JROF)  
      PVT0  (JROF)=PVT0  (JROF)+PUT1(JROF)*ZDT*PRCORI(JROF)
      PSPT0L(JROF)=PSPT0L(JROF)-PUT1(JROF)*ZDT

      PVT9  (JROF)=PVT9  (JROF)+PVT1(JROF)*ZTE
      PDIVT0(JROF)=PDIVT0(JROF)-PVT1(JROF)*ZDT*PVT5  (JROF)
      PVT0  (JROF)=PVT0  (JROF)-PVT1(JROF)*ZDT*PDIVT5(JROF)
      PVT0L (JROF)=PVT0L (JROF)-PVT1(JROF)*ZDT*PUT5  (JROF)
      PUT0  (JROF)=PUT0  (JROF)-PVT1(JROF)*ZDT*PVT5L (JROF)
      PUT0L (JROF)=PUT0L (JROF)+PVT1(JROF)*ZDT*PVT5  (JROF)
      PVT0  (JROF)=PVT0  (JROF)+PVT1(JROF)*ZDT*PUT5L (JROF)
      PUT0  (JROF)=PUT0  (JROF)-&
       & PVT1(JROF)*ZDT*PUT5  (JROF)*PRATATH(JROF)  
      PVT0  (JROF)=PVT0  (JROF)-&
       & PVT1(JROF)*ZDT*PVT5  (JROF)*PRATATH(JROF)  
      PUT0  (JROF)=PUT0  (JROF)-PVT1(JROF)*ZDT*PRCORI(JROF)
      PSPT0M(JROF)=PSPT0M(JROF)-PVT1(JROF)*ZDT

      PSPT9 (JROF)=PSPT9 (JROF)+PSPT1(JROF)*ZTE
      PUT0  (JROF)=PUT0  (JROF)-&
       & PSPT1(JROF)*ZDT*PSPT5L(JROF)  
      PSPT0L(JROF)=PSPT0L(JROF)-PSPT1(JROF)*ZDT*PUT5  (JROF)
      PVT0  (JROF)=PVT0  (JROF)-&
       & PSPT1(JROF)*ZDT*PSPT5M(JROF)  
      PSPT0M(JROF)=PSPT0M(JROF)-PSPT1(JROF)*ZDT*PVT5  (JROF)
      PDIVT0(JROF)=PDIVT0(JROF)-&
       & PSPT1(JROF)*ZDT*PSPT5 (JROF)  
      PSPT0 (JROF)=PSPT0 (JROF)-PSPT1(JROF)*ZDT*PDIVT5(JROF)
    ENDDO

  ELSE

!*        3.2  SEMI-LAGRANGIAN SCHEME.

    CALL LACDYNSHWAD(CDCONF,NPROMA,IST,IEND,ZDT,&
     & PUT9,PVT9,PUT0,PVT0,PSPT0,PDIVT0,PSPT0L,PSPT0M,&
     & PSPT5,PDIVT5,&
     & PGM,PRCORI,POROG,POROGL,POROGM,&
     & ZSPNLT9,&
     & PB1(1,MSLB1U9),PB1(1,MSLB1V9),PB1(1,MSLB1SP9),&
     & PB1(1,MSLB1U0),PB1(1,MSLB1V0),PB1(1,MSLB1SP0),&
     & PSPT1,&
     & PB1(1,MSLB1UR0),PB1(1,MSLB1VR0),&
     & PB2(1,MSLB2USI),PB2(1,MSLB2VSI),&
     & PB2(1,MSLB2URL),PB2(1,MSLB2VRL))

  ENDIF

!     ------------------------------------------------------------------

!*       4.    VORTICITY EQUATION MODEL.
!              -------------------------

ELSEIF(LLVEQ)THEN
  DO JROF=IST,IEND
    PSPT0 (JROF)=0.0_JPRB
    PSPT0L(JROF)=0.0_JPRB
    PSPT0M(JROF)=0.0_JPRB
    PSPT9 (JROF)=0.0_JPRB
    PSPT9L(JROF)=0.0_JPRB
    PSPT9M(JROF)=0.0_JPRB
    PUT9  (JROF)=PUT9  (JROF)+PUT1(JROF)*ZTE
    PVORT0(JROF)=PVORT0(JROF)+PUT1(JROF)*ZDT*PVT5  (JROF)
    PVT0  (JROF)=PVT0  (JROF)+PUT1(JROF)*ZDT*PVORT5(JROF)
    PVT0L (JROF)=PVT0L (JROF)-PUT1(JROF)*ZDT*PVT5  (JROF)
    PVT0  (JROF)=PVT0  (JROF)-PUT1(JROF)*ZDT*PVT5L (JROF)
    PUT0L (JROF)=PUT0L (JROF)-PUT1(JROF)*ZDT*PUT5  (JROF)
    PUT0  (JROF)=PUT0  (JROF)-PUT1(JROF)*ZDT*PUT5L (JROF)
    PVT0  (JROF)=PVT0  (JROF)+PUT1(JROF)*ZDT*PRCORI(JROF)

    PVT9  (JROF)=PVT9  (JROF)+PVT1(JROF)*ZTE
    PDIVT0(JROF)=PDIVT0(JROF)-PVT1(JROF)*ZDT*PVT5  (JROF)
    PVT0  (JROF)=PVT0  (JROF)-PVT1(JROF)*ZDT*PDIVT5(JROF)
    PVT0L (JROF)=PVT0L (JROF)-PVT1(JROF)*ZDT*PUT5  (JROF)
    PUT0  (JROF)=PUT0  (JROF)-PVT1(JROF)*ZDT*PVT5L (JROF)
    PUT0L (JROF)=PUT0L (JROF)+PVT1(JROF)*ZDT*PVT5  (JROF)
    PVT0  (JROF)=PVT0  (JROF)+PVT1(JROF)*ZDT*PUT5L (JROF)
    PUT0  (JROF)=PUT0  (JROF)-PVT1(JROF)*ZDT*PRCORI(JROF)
    PUT0  (JROF)=PUT0  (JROF)-PVT1(JROF)*ZDT*&
     & PUT5  (JROF)&
     & *PRATATH(JROF)  
    PUT0  (JROF)=PUT0  (JROF)+PUT1(JROF)*ZDT*&
     & PUT5  (JROF)&
     & *PRATATX(JROF)  
    PVT0  (JROF)=PVT0  (JROF)-PVT1(JROF)*ZDT*&
     & PVT5  (JROF)&
     & *PRATATH(JROF)  
    PVT0  (JROF)=PVT0  (JROF)+PUT1(JROF)*ZDT*&
     & PVT5  (JROF)&
     & *PRATATX(JROF)  
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*       2.    INTERFACE TO GLOBAL ARRAYS (SECOND PART)
!              ----------------------------------------

!*       2.3   MAP FACTOR

DO JROF=IST,IEND
  PUT0  (JROF)=PUT0  (JROF)* PGM(JROF)
  PVT0  (JROF)=PVT0  (JROF)* PGM(JROF)
  PUT0L (JROF)=PUT0L (JROF)*(PGM(JROF)**2)
  PVT0L (JROF)=PVT0L (JROF)*(PGM(JROF)**2)
  PDIVT0(JROF)=PDIVT0(JROF)*(PGM(JROF)**2)
  PVORT0(JROF)=PVORT0(JROF)*(PGM(JROF)**2)
  PSPT0L(JROF)=PSPT0L(JROF)* PGM(JROF)
  PSPT0M(JROF)=PSPT0M(JROF)* PGM(JROF)
  IF (.NOT.LELTRA) THEN
    PUT9  (JROF)=PUT9  (JROF)* PGM(JROF)
    PVT9  (JROF)=PVT9  (JROF)* PGM(JROF)
  ENDIF
  IF (.NOT.LTWOTL) THEN
    PDIVT9(JROF)=PDIVT9(JROF)*(PGM(JROF)**2)
    PSPT9L(JROF)=PSPT9L(JROF)* PGM(JROF)
    PSPT9M(JROF)=PSPT9M(JROF)* PGM(JROF)
  ENDIF
ENDDO

!*       2.2   TIME FILTER (PART 2).

IF (.NOT.LTWOTL) THEN
  CALL GPTF2AD(IST,IEND,LLRSTEP,PGMV,PGMVS,ZDUM)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG2AD',1,ZHOOK_HANDLE)
END SUBROUTINE CPG2AD
