SUBROUTINE CPG2LAGAD(KNUMB,PGMV,PGMVS,PGM,PUT1,PVT1)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LTWOTL
USE YOMDIM   , ONLY : NPROMA
USE YOMCT2   , ONLY : NSTOP2
USE YOMCT3   , ONLY : NSTEP
USE YOMGMV   , ONLY : NDIMGMV,NDIMGMVS,YT0,YT9

!**** *CPG2LAGAD* - Grid point calculations 2D lagged part .(ADJ.)

!     Purpose.
!     --------
!           Grid point calculations 2D lagged part (adjoint).

!**   Interface.
!     ----------
!        *CALL* *CPG2LAGAD(...)*

!        Explicit arguments :
!        --------------------

!        ASSOCIATED WITH INPUT or INPUT-OUTPUT IN CPG2LAG:
!         KNUMB     : number of elements of arrays for which computations (I)
!                     are performed (used in message passing version).
!         PGMV      : "t-dt" and "t" upper air GMV variables.             (IO)
!         PGMVS     : "t-dt" and "t" surface GMV variables.               (IO)
!         PGM       : mapping factor.                                     (I)

!        ASSOCIATED WITH OUTPUT IN CPG2LAG:
!         PUT1      : t+dt U-wind.                                        (IO)
!         PVT1      : t+dt V-wind.                                        (IO)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!      Is called by SCAN2MAD.

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!        J.N. Thepaut  *ECMWF*

!     Modifications.
!     --------------
!      Original : 92-04-01
!      Modified 95-03-01 by M.Hamrud (F90 re-write)
!      Modified 96-04-04 by L.Isaksen (Adapted for Message Passing)
!      Modified 98-08-05 by K. YESSAD: cleanings according to the CPG2LAG ones.
!      Modified 98-12-29 by C.Temperton (Adjoint of semi-Lagrangian)
!      Modified 99-03-15 by C.Temperton (Bugfixes for LMESSP=.T.)
!      Modified 99-06-23 by C.Temperton (L2TLFF option added)
!      Modified 99-11-19 by C.Temperton (bugfix, NSLB2 -> KSLB2)
!      Modified 00-03-06 by C.Temperton: Repairs following optimization
!                                        of 3-D semi-Lagrangian adjoint.
!      Modified 00-07-06 by G.Mozdzynski:take out ladinead for reproducibility
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      K. Yessad (Dec 2008): remove useless dummy arguments.
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,1,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT1(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT1(NPROMA) 

!------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IST, IEND, JROF
LOGICAL ::  LLLSTEP
REAL(KIND=JPRB) :: ZDUMARR(1,1,1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------

#include "gptf1ad.intfb.h"

!------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPG2LAGAD',0,ZHOOK_HANDLE)
!------------------------------------------------------------------

!*       1.    PRELIMINARY COMPUTATIONS.
!              -------------------------

IST   = 1
IEND  = KNUMB
IF(NSTEP < NSTOP2-1) THEN
  LLLSTEP=.FALSE.
ELSE
  LLLSTEP=.TRUE.
ENDIF

!*       2.   LAGGED PART OF GRIDPOINT COMPUTATIONS
!             -------------------------------------
 !*       2.4  DIVIDE BY MAP FACTOR

DO JROF=IST,IEND
  PUT1(JROF)=PUT1(JROF)/PGM(JROF)
  PVT1(JROF)=PVT1(JROF)/PGM(JROF)
ENDDO

IF(LTWOTL) THEN
  IF(LLLSTEP) THEN
    PGMV(:,:,YT0%NDIM+1:) = 0.0_JPRB
    PGMVS(:,YT0%NDIMS+1:) = 0.0_JPRB
  ENDIF
  DO JROF=IST,IEND
    PGMV(JROF,1,YT0%MU)=PGMV(JROF,1,YT0%MU)+PGMV(JROF,1,YT9%MU)
    PGMV(JROF,1,YT9%MU)=0.0_JPRB
    PGMV(JROF,1,YT0%MV)=PGMV(JROF,1,YT0%MV)+PGMV(JROF,1,YT9%MV)
    PGMV(JROF,1,YT9%MV)=0.0_JPRB
  ENDDO
ELSE
  CALL GPTF1AD(IST,IEND,LLLSTEP,PGMV,PGMVS,ZDUMARR)
ENDIF

!------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPG2LAGAD',1,ZHOOK_HANDLE)
END SUBROUTINE CPG2LAGAD

