SUBROUTINE CPG5_GP( &
 !---------------------------------------------------------------------
 ! - INPUT .
 & KST,KEND,KSTGLO, &
 & LDLADNMI, &
 & PGM, &
 & POROG, &
 & POROGL_RE,POROGM_RE,POROGLL_RE,POROGMM_RE,POROGLM_RE, &
 !---------------------------------------------------------------------
 ! - INPUT/OUTPUT .
 & PGFL5,PGMV5,PGMV5S, &
 !---------------------------------------------------------------------
 ! - OUTPUT .
 & POROGL,POROGM,POROGLL,POROGMM,POROGLM, &
 & PUT95,PUT95L,PVT95,PVT95L,PTT95,PQT95, &
 & PSPDT95,PSVDT95,PVORT95,PDIVT95, &
 & PRE5,PRE5L,PRE5M,PRE5F, &
 & PDELP5,PRDELP5,PLNPR5,PALPH5, &
 & PRPRE5,PRPREF5,PRPP5, &
 & PLHU5,PLHV5, &
 & PHI5,PHIF5, &
 & PCP5,PR5,PRTGR5, &
 & PKAP5,PRRED5, &
 & PEVEL5,PVVEL5,PSDVBC5, &
 & PSGRT5L,PSGRT5M,PGPH5L,PGPH5M,&
 & PSDIV5,&
 & PRT5,PRT5L,PRT5M,PWW5, &
 & PRE95,PRE95F,&
 & PDELP95,PRDELP95,PLNPR95,PALPH95, &
 & PRPRE95,PRPP95,PRRED95, &
 & PLHU95,PLHV95, &
 & PHI95,PHIF95,PCP95,PR95, &
 & PGPPC5,&
 & PQS,PQS1,PTS,PTS1,PSNS, &
 & PSD_VF5, &
 & PQCHA5L,PQCHA5M, &
 & PALPH5L,PALPH5M,PALPHPL5L,PALPHPL5M,PLNPR5L,PLNPR5M, &
 & PRNHPPI5,PHI5FL,PHI5FM, &
 & PCOEFD5,PCOEFA5,PCOEFAPL5, &
 & PNH15L,PNH15M)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMNH  ,NPROMM   ,NFGPPC
USE YOMDIMT  , ONLY : NPROMT
USE YOMCT0   , ONLY : LRUBC    ,LNHDYN   ,LTWOTL   ,LSPRT    ,LPC_OLD  ,&
 & LPC_FULL
USE YOMIOPNH , ONLY : NG3NH95  ,LTRAJNH
USE YOMCT3   , ONLY : NSTEP
USE YOMCST   , ONLY : RD       ,RV
USE YOMDYN   , ONLY : SITLAF   ,NCURRENT_ITER  
USE YOMGEM   , ONLY : VBH      ,VAH      ,VC       ,VDELB    ,RSTRET  
USE YOMCVER  , ONLY : LVERTFE
USE YOMSIMPHL, ONLY : LSIMPH   ,LTRAJPS
USE YOMIOP   , ONLY : NG3D95   ,NG2D95   ,NG2DS5
USE YOPHLC   , ONLY : LSPHLC
USE YOM_YGFL , ONLY : YGFL     ,YQ
USE YOMGMV   , ONLY : YT5
USE PTRGPPC  , ONLY : MGPPCO_Y5,MGPPCO_GL5,MGPPCO_GM5,MGPPCO_RR5,MGPPCO_WW5
USE SURFACE_FIELDS_MIX, ONLY : YSD_VFD
USE TRAJECTORY_MOD, ONLY: GET_TRAJ_SFC

!**** *CPG5_GP* - Grid point calculations:
!                initial part of not lagged grid-point calculations
!                for the trajectory in TL and AD codes.

!     Purpose.
!     --------
!           Grid point calculations:
!           initial part of not lagged grid-point calculations,
!           for the trajectory in TL and AD codes.
!           - get data in buffers.
!           - multiply p-order horizontal derivatives by M**p.
!           - calls some GP... routines to initialise some auxiliary variables.

!           This routine has the same structure as CPG_GP, but it works on
!           trajectory arrays (the name of which ending by 5 or 95)
!           and does some calls to RDPHTR.. routines. Only a subset of CPG_GP
!           is done under CPG5_GP. The call of CPG5_GP is due to the fact that,
!           when computing the trajectory in the direct code, all the variables
!           computed by CPG_GP are not saved on buffers (too expensive in
!           memory):
!           some of them need to be recomputed by CPG5_GP when calling the TL
!           or AD code.

!           Abbreviation "vwv" stands for "vertical wind variable".

!**   Interface.
!     ----------
!        *CALL* *CPG5_GP(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        KST       : first element of work.
!        KEND      : last element of work.
!        KSTGLO    : global offset.
!        LDLADNMI  : .T. if adiabatic NNMI iteration.
!        PGM       : mapping factor.
!        POROG     : surface orography.
!        POROGL_RE : zonal component of "vnabla'(surf orography)"
!        POROGM_RE : meridian component of "vnabla'(surf orography)"
!        POROGLL_RE: second order zonal reduced der. of the orography.
!        POROGMM_RE: second order merid reduced der. of the orography.
!        POROGLM_RE: second order crossed reduced der. of the orography.

!     INPUT/OUTPUT:
!     -------------
!        PGFL5     : unified_treatment grid-point fields at t
!        PGMV5     : upper air GMV variables at time t and t-dt.
!        PGMV5S    : surface GMV variables at time t and t-dt.

!     OUTPUT:
!     -------
!        POROGL    : zonal component of "rssavnabla(surf orography)"
!        POROGM    : meridian component of "rssavnabla(surf orography)"
!        POROGLL   : second order zonal semi-reduced der. of the orography.
!        POROGMM   : second order merid semi-reduced der. of the orography.
!        POROGLM   : second order crossed semi-reduced der. of the orography.
!        PUT95     : U-wind at time t-dt on layers.
!        PUT95L    : zonal component of U-wind at time t-dt on layers.
!        PVT95     : V-wind at time t-dt on layers.
!        PVT95L    : zonal component of V-wind at time t-dt on layers.
!        PTT95     : temperature at time t-dt on layers.
!        PQT95     : humidity at time t-dt on layers.
!        PSPDT95   : pressure departure eqn prog variable at t-dt on layers.
!        PSVDT95   : vertical divergence eqn prog variable at t-dt on layers.
!        PVORT95   : vorticity at time t-dt on layers.
!        PDIVT95   : divergence at time t-dt on layers.
!        PRE5      : hydrostatic pressure "prehyd" on interlayers at time t.
!        PRE5L     : zonal component of "rssavnabla prehyds" at t.
!        PRE5M     : meridian component of "rssavnabla prehyds" at t.
!        PRE5F     : hydrostatic pressure "prehyd" on layers at time t.
!        PDELP5    : hydrostatic pressure depth of layers at t.
!        PRDELP5   : 1/(hydrostatic pressure depth of layers) at t.
!        PLNPR5    : "delta" on layers at time t.
!        PALPH5    : "alpha" on layers at time t.
!        PRPRE5    : 1/PRE5.
!        PRPREF5   : 1/PRE5F.
!        PRPP5     : 1/(PRE5(lbar-1)*PRE5(lbar)).
!        PLHU5     : zonal wind time t on interlayers.
!        PLHV5     : meridian wind time t on interlayers.
!        PHI5      : geopotential height "gz" at t on interlayers.
!        PHIF5     : geopotential height "gz" at t on layers.
!        PCP5      : "cp" at t.
!        PR5       : air constant "R" at t.
!        PRTGR5    : scalar ratio of vectors "(rssavnabla prehyd)/prehyd" at t.
!        PKAP5     : "kappa = R/cp" at t.
!        PRRED5    : reduced gas constant by (1.+Prond) in NH at t.
!        PEVEL5    : "etadot d prehyd / d eta" at t.
!                    On interlayers (resp. layers) if LVERTFE=F (resp. T).
!        PVVEL5    : "omega/prehyd" on layers at t.
!        PSDVBC5   : vertical integral of divergence computed in "gpcty",
!                    including the "lrubc" and "delta m=1" contributions
!                    of "etadot d prehyd / d eta".
!        PSGRT5L   : semi-reduced zonal component of the pressure force grad.
!        PSGRT5M   : semi-reduced merid. component of the pressure force grad.
!        PGPH5L    : semi-reduced zonal component of "gz" gradient
!                    on interlayers.
!        PGPH5M    : semi-reduced meridian component of "gz" gradient
!                    on interlayers.
!        PSDIV5    : vertical integral of divergence without
!                   "lrubc" and "delta m=1" contributions.
!        PRT5      : RT on layers at t.
!        PRT5L     : zonal component of "rssavnabla RT" on layers at t.
!        PRT5M     : meridian component of "rssavnabla RT" on layers at t.
!        PWW5      : weights to compute (U,V) on interlayers.
!        PRE95     : hydrostatic pressure "prehyd" on interlayers at t-dt.
!        PRE95F    : hydrostatic pressure "prehyd" on layers at time t-dt.
!        PDELP95   : hydrostatic pressure depth of layers at t-dt.
!        PRDELP95  : 1/(hydrostatic pressure depth of layers) at t-dt.
!        PRPRE95   : 1/PRE95.
!        PRPP95    : 1/(PRE95(lbar-1)*PRE95(lbar)).
!        PRRED95   : reduced gas constant by (1.+Prond) in NH at t-dt.
!        PLNPR95   : "delta" on layers at time t-dt.
!        PALPH95   : "alpha" on layers at time t-dt.
!        PLHU95    : zonal wind time t-dt on interlayers.
!        PLHV95    : meridian wind time t-dt on interlayers.
!        PHI95     : geopotential height "gz" at t-dt on interlayers.
!        PHIF95    : geopotential height "gz" at t-dt on layers.
!        PCP95     : "cp" at t-dt.
!        PR95      : air constant "R" at t-dt.
!        PGPPC5    : buffer containing some fields used in the PC schemes.
!        PQS       : surface specific humidity at t or t-dt accord. to ltwotl.
!        PQS1      : surface specific humidity at t+dt.
!        PTS       : surface temperature at t or t-dt accord. to ltwotl.
!        PTS1      : surface temperature at t+dt.
!        PSNS      : mass of snow per unit surf at t or t-dt accord. to ltwotl.
!        PSD_VF5   : surface fields(recovered by GET_TRAJ_SFC).
!        PQCHA5L   : zonal comp grad(log(pre/prehyd)).
!        PQCHA5M   : merid comp grad(log(pre/prehyd)).
!        PALPH5L   : cf. PALPHL in GPGRXYB (time t).
!        PALPH5M   : cf. PALPHM in GPGRXYB (time t).
!        PALPHPL5L : cf. PALPHPLL in GPGRXYB (time t)
!        PALPHPL5M : cf. PALPHPLM in GPGRXYB (time t)
!        PLNPR5L   : cf. PLNPRL in GPGRXYB (time t).
!        PLNPR5M   : cf. PLNPRM in GPGRXYB (time t).
!        PRNHPPI5  : "prehyd/pre" on layers (time t).
!        PHI5FL    : zonal comp grad(gz) on layers
!        PHI5FM    : merid comp grad(gz) on layers
!        PCOEFD5   : coefficient to compute (PLNPR5L,PLNPR5M)
!        PCOEFA5   : coefficient to compute (PALPH5L,PALPH5M)
!        PCOEFAPL5 : coefficient to compute (PALPHPL5L,PALPHPL5M)
!        PNH15L    : zonal comp of RT grad(log(prehyd/pre)) at full layers
!        PNH15M    : merid comp of RT grad(log(prehyd/pre)) at full layers

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!        K. YESSAD, after part 3 of CPGTL and CPGAD.

! Modifications
! -------------
!   Original : 08 Jul 2004
!   06-Jan-2005 K. Yessad: old GPGRP renamed into GPGRP_OLD.
!   31-Mar-2005 K. Yessad: new GPGRP.
!   M.Hamrud  15-Jan-2006  Revised GPRCP
!   S. Ivatek-S.  10-Apr-2007  Initialisation NEC
!   S. Ivatek-S.  17-Apr-2007  Over dimensioning of PGPNH5 to NFGPNH+1,
!                          boundary checking problem if NFGPNH=0 bf
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDLADNMI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL_RE(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM_RE(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGLL_RE(NPROMNH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGMM_RE(NPROMNH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGLM_RE(NPROMNH) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL5(NPROMA,NFLEVG,YGFL%NDIM5) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV5(NPROMA,NFLEVG,YT5%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV5S(NPROMA,YT5%NDIMS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POROGLL(NPROMNH) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POROGMM(NPROMNH) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POROGLM(NPROMNH) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUT95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUT95L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVT95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVT95L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTT95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQT95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPDT95(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSVDT95(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVORT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIVT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE5L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE5M(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE5F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDELP5(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRDELP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPR5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPH5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPRE5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPREF5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPP5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHU5(NPROMNH,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHV5(NPROMNH,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIF5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PR5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTGR5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKAP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRRED5(NPROMNH,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVVEL5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSDVBC5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSGRT5L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSGRT5M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPH5L(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPH5M(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSDIV5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRT5L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRT5M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWW5(NPROMNH,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE95(NPROMT,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE95F(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDELP95(NPROMT,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRDELP95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPRE95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPP95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRRED95(NPROMNH,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPR95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPH95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHU95(NPROMNH,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHV95(NPROMNH,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI95(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIF95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PR95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPPC5(NPROMA,NFGPPC+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQS1(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTS1(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSNS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSD_VF5(NPROMA,YSD_VFD%NDIM5)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCHA5L(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCHA5M(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPH5L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPH5M(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPHPL5L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPHPL5M(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPR5L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPR5M(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRNHPPI5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI5FL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI5FM(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOEFD5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOEFA5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOEFAPL5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNH15L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNH15M(NPROMA,NFLEVG)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IFLAG, IITFIELDS, INHFIELDS, IPROMVC
INTEGER(KIND=JPIM) :: IFIELDS, I_KSTEP, IBLK
INTEGER(KIND=JPIM) :: JLEV, JROF

LOGICAL :: LLVERCOR
LOGICAL :: LLSTR

REAL(KIND=JPRB) :: ZRTGR95(NPROMT,NFLEVG)     ! cf. PRTGR in GPXYB (time t-dt).
REAL(KIND=JPRB) :: ZSNS1(NPROMM)              ! cf. PSNS but at time t+dt.
REAL(KIND=JPRB) :: ZEVT0(NPROMA)
REAL(KIND=JPRB) :: ZDIVP5(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZPSPT95(NPROMT)
REAL(KIND=JPRB) :: ZDELNHPRE5(NPROMNH,NFLEVG)! "Delta pre" on layers (time t).
REAL(KIND=JPRB) :: ZLNNHPREF5L(NPROMNH,NFLEVG)! zon cmp grad(PNHPRE5F)/PNHPRE5F
REAL(KIND=JPRB) :: ZLNNHPREF5M(NPROMNH,NFLEVG)! mer cmp grad(PNHPRE5F)/PNHPRE5F
REAL(KIND=JPRB) :: ZSGRTS5L(NPROMNH)         ! cf. PSGRT5L but at the surface
REAL(KIND=JPRB) :: ZSGRTS5M(NPROMNH)         ! cf. PSGRT5M but at the surface
REAL(KIND=JPRB) :: ZNHPPI5(NPROMA,NFLEVG)    ! "pre/prehyd" on layers (time t).

REAL(KIND=JPRB) :: ZDUM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZDUMSPT(NPROMA)
REAL(KIND=JPRB) :: ZDUM_LLVERCOR(1,1)
REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gpcty.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgrgeo.intfb.h"
#include "gpgrp.intfb.h"
#include "gpgrxyb.intfb.h"
#include "gphluv.intfb.h"
#include "gphlwi.intfb.h"
#include "gpmpfc.intfb.h"
#include "gpmpfc_gmvs.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gprt.intfb.h"
#include "gp_spv.intfb.h"
#include "gpxyb.intfb.h"
#include "rdittrajm.intfb.h"
#include "rdnhtrajm.intfb.h"
#include "rdphtrajm.intfb.h"
#include "rdphtrsf.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG5_GP',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

ZEPS=100.0_JPRB*TINY(1.0_JPRB)
LLSTR=(ABS(RSTRET-1.0_JPRB)>ZEPS)
ZDUM(:,:)=0.0_JPRB
ZDUMSPT(:)=0.0_JPRB

! --- Relaxation of thin layer hypothesis not yet coded ----------------------
LLVERCOR=.FALSE.
IPROMVC=1

!     ------------------------------------------------------------------

!*       1.    INTERFACE TO GLOBAL ARRAYS/WORKFILES, AND READ TRAJECTORY.
!              ----------------------------------------------------------

!*       1.1    Geographical derivatives of orography
!               -------------------------------------

DO JROF=KST,KEND
  POROGL(JROF)=POROGL_RE(JROF)*PGM(JROF)
  POROGM(JROF)=POROGM_RE(JROF)*PGM(JROF)
ENDDO
IF(LNHDYN) THEN
  DO JROF=KST,KEND
    POROGLL(JROF)=POROGLL_RE(JROF)*PGM(JROF)**2
    POROGMM(JROF)=POROGMM_RE(JROF)*PGM(JROF)**2
    POROGLM(JROF)=POROGLM_RE(JROF)*PGM(JROF)**2
  ENDDO
ENDIF

!*       1.2    Initialization of NHS traj array (iteration)
!               --------------------------------------------

IF (LNHDYN.AND.LPC_OLD) PGPPC5(:,:)=1.0_JPRB

!*       1.2.1  Read stored model trajectory at t-dt.

! switch off most of T-DT in case where we are in the NHS iteration

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

  ! simplified physics

  IF (LTRAJPS.AND..NOT.LDLADNMI) THEN
    IFIELDS=NG3D95*NFLEVG+NG2D95
    CALL RDPHTRAJM(NSTEP,KST,KEND,KSTGLO,IFIELDS,&
     & PUT95,PVT95,PTT95,PQT95,ZPSPT95)
    CALL RDPHTRSF(NSTEP,KST,KEND,KSTGLO,NG2DS5,PQS,PTS,PSNS)
    I_KSTEP=NSTEP+1
    CALL RDPHTRSF(I_KSTEP,KST,KEND,KSTGLO,NG2DS5,PQS1,PTS1,ZSNS1)
  ENDIF

  ! NHS predictive step

  IF (LNHDYN.AND.LTRAJNH) THEN
    INHFIELDS=NG3NH95*NFLEVG+1
    CALL RDNHTRAJM(NSTEP,KST,KEND,KSTGLO,INHFIELDS,&
     & PSPDT95,PSVDT95,PDIVT95,&
     & PUT95,PVT95,PTT95,ZPSPT95)
  ENDIF

ENDIF

!*       1.2.2  NHS iteration: corrective step for LPC_OLD: read trajectory

IF ((NCURRENT_ITER > 0 .AND. LPC_OLD).AND.LTRAJNH) THEN

  IITFIELDS=13*NFLEVG+1
  CALL RDITTRAJM(NSTEP,KST,KEND,KSTGLO,IITFIELDS,&
   & PGMV5(1,1,YT5%MU),PGMV5(1,1,YT5%MV),PGMV5(1,1,YT5%MT),&
   & PGMV5(1,1,YT5%MSVD),PGPPC5(1,MGPPCO_Y5),PGPPC5(1,MGPPCO_RR5),&
   & PGPPC5(1,MGPPCO_GL5),PGPPC5(1,MGPPCO_GM5),PGPPC5(1,MGPPCO_WW5),&
   & PGMV5(1,1,YT5%MVOR),PGMV5(1,1,YT5%MDIV),&
   & PGMV5(1,1,YT5%MUL),PGMV5(1,1,YT5%MVL),PGMV5S(1,YT5%MSP))

ENDIF

!*       1.3    Get surface variables from incore traj
!               --------------------------------------

IBLK=(KSTGLO-1)/NPROMA+1
IF (((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) &
 & .AND.(LSIMPH.OR.LSPHLC)) THEN
  CALL GET_TRAJ_SFC(NSTEP,IBLK,KST,KEND,PSD_VF=PSD_VF5)
ENDIF

!*       1.4    Map factor
!               ----------

IF(LLSTR) THEN

  IFLAG=0

  CALL GPMPFC(&
   ! --- INPUT ---------------------------------------------------------
   & KST,KEND,IFLAG,PGM,&
   ! --- INPUT/OUTPUT --------------------------------------------------
   & PGMV5(1,1,YT5%MU),PGMV5(1,1,YT5%MV),PGMV5(1,1,YT5%MDIV),&
   & PGMV5(1,1,YT5%MTL),PGMV5(1,1,YT5%MTM),&
   & PGMV5(1,1,YT5%MSPDL),PGMV5(1,1,YT5%MSPDM),&
   & PGMV5(1,1,YT5%MSVDL),PGMV5(1,1,YT5%MSVDM),&
   & ZDUM,ZDUM,&
   & ZDUM,ZDUM,ZDUM,ZDUM,ZDUM,ZDUM,ZDUM,ZDUM,ZDUM,&
   & PGMV5(1,1,YT5%MUL),PGMV5(1,1,YT5%MVL),PGMV5(1,1,YT5%MVOR),&
   & ZDUM,ZDUM,ZDUM,&
   & PGFL5)

  CALL GPMPFC_GMVS(&
   ! --- INPUT ---------------------------------------------------------
   & KST,KEND,IFLAG,PGM,&
   ! --- INPUT/OUTPUT --------------------------------------------------
   & PGMV5S(1,YT5%MSPL),PGMV5S(1,YT5%MSPM),&
   & ZDUMSPT,ZDUMSPT)

ENDIF

!*       1.5    surface pressure variable and surface variables.
!               ------------------------------------------------

CALL GP_SPV(LDLADNMI,.TRUE.,KST,KEND,&
 & PGMV5S(1,YT5%MSP),PGMV5S(1,YT5%MSPL),PGMV5S(1,YT5%MSPM),&
 & ZPSPT95,ZDUMSPT,ZDUMSPT,&
 & PRE5,PRE5L,PRE5M,&
 & PRE95,ZDUMSPT,ZDUMSPT)

!     ------------------------------------------------------------------

!*       3.    INITIALISE AUXILIARY VARIABLES.
!              -------------------------------

!*       3.1   TIME t0 CALCULATIONS.
!              ---------------------

!*       3.1.1  Compute half layer hydrostatic pressure

CALL GPPREH(NPROMA,KST,KEND,NFLEVG,VAH,VBH,PRE5)

!*       3.1.2  Compute auxillary arrays related to the vertical
!               and hydrostatic pressure at full levels

CALL GPXYB(NPROMA,KST,KEND,NFLEVG,VDELB,VC,&
 & PRE5,PDELP5(1,1),PRDELP5,PLNPR5,PALPH5,PRTGR5,&
 & PRPRE5,PRPP5)

CALL GPPREF(NPROMA,KST,KEND,NFLEVG,PALPH5,PRE5,PRE5F)

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

  IF(LVERTFE) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        PRPREF5(JROF,JLEV)=1.0_JPRB/PRE5F(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

  CALL GPGRXYB(NPROMA,KST,KEND,NFLEVG,VC,VBH,&
   & PRE5L,PRE5M,PRDELP5,PRPRE5,PRPP5,PLNPR5,PALPH5,PRTGR5,&
   & PLNPR5L,PLNPR5M,PALPH5L,PALPH5M,PALPHPL5L,PALPHPL5M,&
   & PCOEFD=PCOEFD5,PCOEFA=PCOEFA5,PCOEFAPL=PCOEFAPL5)

!*       3.1.3  Compute R, Cp and Kappa

  CALL GPRCP(NPROMA,KST,KEND,NFLEVG,PQ=PGFL5(1,1,YQ%MP5),&
   & PCP=PCP5,PR=PR5,PKAP=PKAP5)

!*       3.1.4  Computations of RT and its derivatives

  CALL GPRT(LSPRT,NPROMA,KST,KEND,NFLEVG,RD,RV,PR5,&
   & PGMV5(1,1,YT5%MT),PGMV5(1,1,YT5%MTL),&
   & PGMV5(1,1,YT5%MTM),PGFL5(1,1,YQ%MP5L),PGFL5(1,1,YQ%MP5M),&
   & PRT5,PRT5L,PRT5M)

!*       3.1.5  Compute quantities for NH model.

  IF (LNHDYN) THEN
    CALL ABOR1(' CPG5_GP: NH model not yet coded for AD and TL ')
    ! not yet coded (see CPG_GP)
    ! call GNHPRE
    ! calculation of PRRED5,ZRTR5
    ! call GNHPREH
    ! call GNHGRPRE
  ELSE
    ZNHPPI5(1:NPROMA,1:NFLEVG)=1.0_JPRB
    PRNHPPI5(1:NPROMA,1:NFLEVG)=1.0_JPRB
  ENDIF

!*       3.1.6  Solve continuity equation
!               and compute some vertical velocities.

  CALL GPCTY(LLVERCOR,NPROMA,IPROMVC,KST,KEND,NFLEVG,VDELB,VBH,&
   & PGMV5(1,1,YT5%MU),PGMV5(1,1,YT5%MV),&
   & PGMV5(1,1,YT5%MDIV),ZEVT0,LRUBC,&
   & PDELP5(1,1),PRDELP5,PLNPR5,PALPH5,PRTGR5,PRE5L,PRE5M,&
   & ZDUM_LLVERCOR,ZDUM_LLVERCOR,ZDUM_LLVERCOR,&
   & ZDUM_LLVERCOR,ZDUM_LLVERCOR,ZDUM_LLVERCOR,&
   & PEVEL5,PVVEL5,PSDIV5,PSDVBC5,ZDIVP5,PRPREF5)

!*       3.1.7  Integrate Hydrostatics
!               Computes the geopotential height "gz" and its gradient.

  ! * "gz" on layers and interlayers.

  DO JROF=KST,KEND
    PHI5(JROF,NFLEVG)=POROG(JROF)
  ENDDO

  IF(LNHDYN) THEN
    CALL GPGEO(NPROMA,KST,KEND,NFLEVG,PHI5,PHIF5,&
     & PGMV5(1,1,YT5%MT),PRRED5,PLNPR5,PALPH5)
  ELSE
    CALL GPGEO(NPROMA,KST,KEND,NFLEVG,PHI5,PHIF5,&
     & PGMV5(1,1,YT5%MT),PR5,PLNPR5,PALPH5)
  ENDIF

  ! * "rssavnabla gz" on layers and interlayers.
                                                                                
  CALL GPGRGEO(LNHDYN,&
   & NPROMA,KST,KEND,NFLEVG,&
   & PRNHPPI5,PGMV5(1,1,YT5%MT),PR5,PRT5L,PRT5M,&
   & PLNPR5,PLNPR5L,PLNPR5M,PALPH5,PALPH5L,PALPH5M,&
   & PQCHA5L,PQCHA5M,&
   & POROGL,POROGM,&
   & PHI5FL,PHI5FM,PGPH5L,PGPH5M,&
   & PNH1L=PNH15L,PNH1M=PNH15M)

!*       3.1.8  Compute half-level winds for non-hydrostatic.

  IF(LNHDYN) THEN
    ! ky: if VFE, condition of call to GPHLWI+GPHLUV has to be updated later
    !     when implementing NH+VFE TL and AD codes.
    CALL GPHLWI(NPROMA,KST,KEND,PLNPR5,PALPH5,PWW5)
    CALL GPHLUV(NPROMA,KST,KEND,PWW5,PGMV5(1,1,YT5%MU),PGMV5(1,1,YT5%MV),&
     & PLHU5,PLHV5)
  ENDIF

!*       3.1.9  Computation of the pressure force for the momentum eqn RHS.

  CALL GPGRP( &
   ! --- INPUT ----------------------------------------------------------------
   & LLVERCOR, &
   & NPROMA,IPROMVC,NPROMNH,KST,KEND,NFLEVG, &
   & PRT5,PRT5L,PRT5M,PRDELP5,PRE5L,PRE5M,PRTGR5, &
   & PLNPR5,PLNPR5L,PLNPR5M,PALPH5,PALPHPL5L,PALPHPL5M, &
   & ZDELNHPRE5,ZLNNHPREF5L,ZLNNHPREF5M, &
   & PRNHPPI5,PQCHA5L,PQCHA5M, &
   & PGPH5L,PGPH5M,PHI5FL,PHI5FM, &
   & ZDUM,ZDUM,ZDUM,ZDUM,ZDUM,ZDUM, &
   ! --- OUTPUT ---------------------------------------------------------------
   & PSGRT5L,PSGRT5M,ZSGRTS5L,ZSGRTS5M)

ENDIF

!*       3.2   TIME t9 CALCULATIONS.
!              ---------------------

IF (.NOT.LTWOTL) THEN

  IF ((LSIMPH.AND..NOT.LDLADNMI).OR.&
   & (LNHDYN.AND.((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)))&
   & )THEN

!*       3.2.1  Compute half layer hydrostatic pressure

    CALL GPPREH(NPROMA,KST,KEND,NFLEVG,VAH,VBH,PRE95)

!*       3.2.2  Compute auxillary arrays related to the vertical
!               and hydrostatic pressure at full levels.

    CALL GPXYB(NPROMA,KST,KEND,NFLEVG,VDELB,VC,&
     & PRE95,PDELP95(1,1),PRDELP95,PLNPR95,PALPH95,&
     & ZRTGR95,PRPRE95,PRPP95)
    CALL GPPREF(NPROMA,KST,KEND,NFLEVG,PALPH95,PRE95,PRE95F)

  ENDIF

  IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

!*       3.2.3  Compute R, Cp and Kappa

    IF (LSIMPH.AND..NOT.LDLADNMI) THEN
      CALL GPRCP(NPROMA,KST,KEND,NFLEVG,PQ=PQT95,&
       &PCP=PCP95,PR=PR95)
    ENDIF

!*       3.2.5  Compute quantities for NH model.

    IF(LNHDYN.AND.LSIMPH) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEND
          PRRED95(JROF,JLEV)=PR95(JROF,JLEV)/(1.0_JPRB+PSPDT95(JROF,JLEV)&
           & *SITLAF(JLEV)/PRE95F(JROF,JLEV))
        ENDDO
      ENDDO
    ENDIF

!*       3.2.7  Integrate Hydrostatics
!               Computes the geopotential height "gz" and its gradient.

    IF (LSIMPH.AND..NOT.LDLADNMI) THEN
      DO JROF=KST,KEND
        PHI95(JROF,NFLEVG)=POROG(JROF)
      ENDDO
    ENDIF

    IF(LNHDYN) THEN
      IF(LSIMPH) THEN
        CALL GPGEO(NPROMA,KST,KEND,NFLEVG,&
         & PHI95,PHIF95,PTT95,PRRED95,PLNPR95,PALPH95)
      ENDIF
    ELSE
      IF (LSIMPH.AND..NOT.LDLADNMI) THEN
        CALL GPGEO(NPROMA,KST,KEND,NFLEVG,&
         & PHI95,PHIF95,PTT95,PR95,PLNPR95,PALPH95)
      ENDIF
    ENDIF

!*       3.2.8  Compute half-level winds for non-hydrostatic

    IF(LNHDYN) THEN
      ! ky: if VFE, condition of call to GPHLWI+GPHLUV has to be updated later
      !     when implementing NH+VFE TL and AD codes.
      CALL GPHLUV(NPROMA,KST,KEND,PWW5,PUT95,PVT95,PLHU95,PLHV95)
    ENDIF

  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       4.    INITIALISE AUXILIARY VARIABLES.
!              -------------------------------

!*       4.1   Compute half-level winds for non-hydrostatic

IF (LNHDYN .AND. NCURRENT_ITER > 0 .AND. LPC_OLD) THEN
  CALL GPHLUV(NPROMA,KST,KEND, &
   & PGPPC5(1,MGPPCO_WW5),PGMV5(1,1,YT5%MU),PGMV5(1,1,YT5%MV),&
   & PLHU5,PLHV5)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG5_GP',1,ZHOOK_HANDLE)
END SUBROUTINE CPG5_GP
