#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE CPG_DYN( &
 ! --- INPUT ----------------------------------------------------------
 & CDCONF, &
 & KST,KEND,KSTGLO, &
 & LDLADNMI, &
 & PDT, &
 & PSLHDA,PSLHDD0, &
 & PGM,PGNORDL,PGNORDM,PRCORI,PRCORIC,PGOMVRL,PGOMVRM,PRATATX,PRATATH, &
 & POROG,POROGL,POROGM, &
 & PRE0,PRDELP0,PLNPR0, &
 & PLHU0,PLHV0, &
 & PHI0,PHIF0,PCP0,PR0, &
 & PKAP0,PEVEL0,PVVEL0,PSDVBC0,PKENE0, &
 & PSGRTL,PSGRTM, &
 & PVCRSSA0F,PVCASRS0F,PVCASRS0H,PVCW0F,PWDLW0F, &
 & PVCRSSA9F,PRDLR0,PRDLR9, &
 & PLSM,PTSOL,PQSOL, &
 & PGFL,&
 & PTNDPD,PTNDVD,PTNDGW,P3DIVG,PD3Y9,PDBBC,PRDPHI,PGWT0,PGWT9,&
 & PNHXT0,PNHXT9,&
 ! --- INPUT/OUTPUT ---------------------------------------------------
 & PGFLPC,&
 & PGWS,&
 & PGMV,PGMVS,&
 & PGPPC,PB1,PB2,PGMVT1,PGMVT1S,PGFLT1, &
 & PD3Y0)

! -------------------------------------------------------------
!**** *CPG_DYN* - Grid point calculations: non lagged dynamics.

!     Purpose.
!     --------
!           Grid point calculations non lagged dynamics.
!           + simple physics.

!           Abbreviation "vwv" stands for "vertical wind variable".

!**   Interface.
!     ----------
!        *CALL* *CPG_DYN(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        CDCONF    : configuration.
!        KST       : first element of work.
!        KEND      : last element of work.
!        KSTGLO    : global offset.
!        LDLADNMI  : .T. if adiabatic NNMI iteration.
!        PDT       : For a leap-frog scheme (three time level scheme):
!                     'dt' at the first time-step, '2 dt' otherwise.
!                    For a 2TL SL scheme: timestep 'dt'.
!        PSLHDA    : Scaling factor of the deformation in f(d) function
!                    (including the model resolution correction)
!        PSLHDD0   :  Treshold for deformation tensor enhancement
!        PGM       : mapping factor.
!        PGNORDL   : zonal component ("Gnordl") of the unit vector
!                    directed towards the true North pole.
!        PGNORDM   : meridian component ("Gnordm") of the unit vector
!                    directed towards the true North pole.
!        PRCORI    : Coriolis parameter "f = 2 Omega sin(theta)".
!        PRCORIC   : 2 Omega cos(theta).
!        PGOMVRL   : zonal component of "2 Omega vec a k".
!        PGOMVRM   : meridian component of "2 Omega vec a k".
!        PRATATX   : spherical geometry: zero
!                    plane geometry: 2 S (sin(theta) - K_L)/(a cos(theta))
!                    Used for curvature terms.
!        PRATATH   : spherical geometry: 2 tan(theta_bne)/a
!                    (theta_bne = tilted but not stretched sphere latitude)
!                    plane geometry: 2 C (sin(theta) - K_L)/(a cos(theta))
!                    Used for curvature terms.
!        POROG     : surface orography.
!        POROGL    : zonal component of "rssavnabla(surf orography)"
!        POROGM    : meridian component of "rssavnabla(surf orography)"
!        PRE0      : hydrostatic pressure "prehyd" at half levels at time t.
!        PRDELP0   : 1/(hydrostatic pressure depth of layers) at t.
!        PLNPR0    : "delta" at full levels at time t.
!        PLHU0     : zonal wind time t at half levels.
!        PLHV0     : meridian wind time t at half levels.
!        PHI0      : geopotential height "gz" at t at half levels.
!        PHIF0     : geopotential height "gz" at t at full levels.
!        PCP0      : "cp" at t.
!        PR0       : air constant "R" at t.
!        PKAP0     : "kappa = R/cp" at t.
!        PEVEL0    : "etadot d prehyd / d eta" at t.
!                    At half (resp. full) levels if LVERTFE=F (resp. T).
!        PVVEL0    : "omega/prehyd" at full levels at t.
!        PSDVBC0   : vertical integral of divergence computed in "gpcty",
!                    including the "lrubc" and "delta m=1" contributions
!                    of "etadot d prehyd / d eta".
!        PKENE0    : kinetic energy at t.
!        PSGRTL    : semi-reduced zonal component of the pressure force grad.
!        PSGRTM    : semi-reduced merid. component of the pressure force grad.
!        PVCRSSA0F : "rs/a" at full levels at t.
!        PVCASRS0F : "a/rs" at full levels at t.
!        PVCASRS0H : "a/rs" at half levels at t.
!        PVCW0F    : pseudo-vertical velocity " W = Drs/Dt " at full levels at t.
!        PWDLW0F   : vertical velocity " w = Dr/Dt " at full levels at t.
!        PVCRSSA9F : "rs/a" at full levels at t-dt.
!        PRDLR0    : array containing "r/a", "a/r", "grad(r/a)" at t.
!        PRDLR9    : array containing "r/a", "a/r" at t-dt.
!        PLSM      : land-sea mask.
!        PTSOL     : surface temperature.
!        PQSOL     : surface specific humidity.
!        PGFL      : unified_treatment grid-point fields at t.
!        PTNDPD    : [D(pressure departure variable)/Dt]_adiab
!        PTNDVD    : [D(vertical divergence variable)/Dt]_adiab
!        PTNDGW    : [D(gw)/Dt]_adiab (LGWADV only)
!                    half level 0 to nflevg-1 values if LVFE_GW=F.
!                    full level 1 to nflevg values if LVFE_GW=T.
!        P3DIVG    : D3 at t (full levels).
!        PD3Y9     : term 'Y' at t-dt.
!        PDBBC     : [D(gw)_surf/Dt]_adiab.
!        PRDPHI    : (RT prehyd [Delta log(prehyd)])/pre at t.
!        PGWT0     : [gw] at t (LGWADV=T only; levels: see in CPG_GP).
!        PGWT9     : [gw] at t-dt (LGWADV=T only; levels: see in CPG_GP).
!        PNHXT0    : "X" at full levels at t, diagnosed in CPG_GP.
!        PNHXT9    : "X" at full levels at t-dt, diagnosed in CPG_GP.

!     INPUT/OUTPUT:
!     -------------
!        PGFLPC    : unified_treatment grid-point fields at t (3TL PC only)
!        PGWS      : [gw]_surf at t.
!        PGMV      : GMV variables at t-dt and t.
!        PGMVS     : GMVS variables at t-dt and t.
!        PGPPC     : buffer containing some fields used in the PC schemes.
!        PB1       : "SLBUF1" buffer.
!        PB2       : "SLBUF2" buffer.
!        PGMVT1    : GMV variables at t+dt.
!        PGMVT1S   : GMVS variables at t+dt.
!        PGFLT1    : t+dt term for the unified_treatment of grid-point fields
!                    (as input contains the non lagged physics).
!        PD3Y0     : term 'Y' at t.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE documentation about Eulerian and semi-Lagrangian schemes.

! Author
! ------
!   09-Aug-2001 K. YESSAD after part 7 of CPG. 

! Modifications
! -------------
!   04 JAN 2002: K.YESSAD - pruning; thin layer hyp relaxation for SL.
!   Modified 03-2002  J.Vivoda : PC schemes for NH dynamics (LPC_XXXX keys)
!   Modified 02-02-05 by C. Temperton: Changed call to 
!                        STORE_TRAJ_SLAG (for LSPRT=.T.)
!   Modified 09-2002  P.Smolikova : variable d4 in NH
!   Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!    semi-lag advection of vertical divergence in the NH model.
!   Modified 01-2003  P.Smolikova : variable d4 in 2tl PC NH
!   Modified 2003-03-16 M.Hamrud - Revised data flow (GFL)
!   Modified 2003-07-17 C. Fischer - psvdauxt0* come from pgmv
!   Modified 2003-08-25 F. Vana - SL diffusion
!   Modified 2003-08 P. Smolikova : option LRDBBC
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   M.Hortal 01-Dec-2003 Extra-fields coming from the dynamics
!      Jan-2004 K. YESSAD  remove PZ(Z)SPT.. arrays and use PGMVS instead.
!      Jan-2004 K. YESSAD  use PB1 and PB2 as dummy arguments.
!      Jan-2004 K. YESSAD  make the code valid for any value of NFLSA.
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   Modified 2004-11 F. Vana - MSLB[1,2]DEF0 replaced by MSLB2KAPPA
!   12-Jan-2005 K. Yessad: GNHPDVD replaced by GP./GNH. routines in CPG_GP.
!   20-Feb-2005 J. Vivoda  3TL PC Eulerian scheme, GWADV scheme for PC_FULL
!   06-Jun-2006 S. Malardel: add calls for large scale forcing (CALL CP_FORCING)
!   08-Jun-2006 F. Vana   PSLHDA and PSLHDD0 fields to LACDYN
!   K. Yessad 07-02-2007: Splitting alti/surf for (gw) in NH+LGWADV.
!   K. Yessad 07-03-2007: Remove useless (gw)_surf interpolations in NH+LGWADV.
!   17-Apr-2007 S. Ivatek-S Over dimensioning of PGPNH to NFGPNH+1, boundary
!                           checking problem if NFGPNH=0 bf
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   09-Sep-2008 J. Masek  Dataflow for flow deformation along pressure levels
!   K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!   F. Vana 13-Jan-2009: Storing KAPPA for TL/AD of SLHD
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOEPHY   , ONLY : LEPHYS
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL   ,LPC_OLD  ,LPC_FULL ,LSFORC
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NPROMA   ,NPROMNH  ,NPROMVC  ,NPROMDLW ,NPROMDLR ,&
 & NFLEVG   ,NFLSA    ,NFGPPC
USE YOMDYN   , ONLY : NCURRENT_ITER
USE YOM_YGFL , ONLY : YGFL , YQ, YFORC
USE YOMPHY   , ONLY : LMPHYS
USE YOMTRSL  , ONLY : NTRSLTYPE
USE YOPHLC   , ONLY : LSPHLC
USE YOMTRAJ, ONLY : LTRAJSAVE, LTRAJSLAG
USE TRAJ_SEMILAG_MOD , ONLY : STORE_TRAJ_SLAG
USE YOMGMV   , ONLY : NDIMGMV,NDIMGMVS,YT0,YT1,YT9
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1T9  ,MSLB1GFL9 ,MSLB1C9  ,&
 & MSLB1SP9 ,MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,MSLB1U0  ,MSLB1V0  ,MSLB1T0
USE PTRSLB2  , ONLY : MSLB2URL ,MSLB2VRL ,MSLB2WRL ,MSLB2KAPPA
USE PTRGPPC  , ONLY : MGPPCO_Y ,MGPPCO_GL,MGPPCO_GM,MGPPCO_RR
USE PAR_RDLR , ONLY : JP_DIMR0 ,JP_DIMR9

!    ------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDLADNMI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLHDA(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLHDD0(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORIC(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGOMVRL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGOMVRM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHU0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHV0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDVBC0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKENE0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSA0F(NPROMVC,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRS0F(NPROMVC,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRS0H(NPROMVC,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCW0F(NPROMVC,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWDLW0F(NPROMDLW,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSA9F(NPROMVC,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDLR0(NPROMDLR,0:NFLEVG,JP_DIMR0)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDLR9(NPROMDLR,0:NFLEVG,JP_DIMR9)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSOL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSOL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNDPD(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNDVD(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNDGW(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: P3DIVG(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PD3Y9(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBBC(NPROMNH)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDPHI(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGWT0(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGWT9(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHXT0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHXT9(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLPC(NPROMA,NFLEVG,YGFL%NDIMPC)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGWS(NPROMNH)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGPPC(NPROMA,NFGPPC+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PD3Y0(NPROMNH,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IBLK
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "cpeuldyn.intfb.h"
#include "cp_forcing.intfb.h"
#include "gnhsim.intfb.h"
#include "lacdyn.intfb.h"
#include "vdiflcz.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_DYN',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    NHS iteration: corrective step for LPC_OLD.
!              -------------------------------------------

IF (NCURRENT_ITER > 0 .AND. LPC_OLD) THEN

  CALL GNHSIM(KST,KEND,&
   & POROGL,POROGM,&
   & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),&
   & PLHU0,PLHV0,PLNPR0,&
   & PGPPC(1,MGPPCO_Y),PGPPC(1,MGPPCO_RR),&
   & PGPPC(1,MGPPCO_GL),PGPPC(1,MGPPCO_GM),&
   & PGMVT1(1,1,YT1%MSPD))  

ENDIF

!     ------------------------------------------------------------------

!*       2.    Semi-lagrangian scheme
!              ----------------------

!* Cases nsiter=0, predictor for lpc_old, predictor and corrector for lpc_full.

IF(LSLAG.AND.((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL))) THEN

!*       2.1   Semi-lagrangian dynamics (not lagged part).

  CALL LACDYN(&
   ! --- INPUT ----------------------------------------------------------------
   & CDCONF,KST,KEND,PDT,&
   & PVCRSSA9F,PNHXT9,PVCRSSA0F,PVCASRS0F,PVCASRS0H,PVCW0F,PWDLW0F,&
   & PRDLR0,PRDLR9,PNHXT0,&
   & PRE0,PKAP0,PRDELP0,PEVEL0,PVVEL0,PSDVBC0,PSGRTL,PSGRTM,&
   & PSLHDA,PSLHDD0,PLHU0,PLHV0,&
   & PGM,PGNORDL,PGNORDM,PRCORI,PRCORIC,PGOMVRL,PGOMVRM,&
   & POROG,POROGL,POROGM,&
   & PTNDPD,PTNDVD,PTNDGW,P3DIVG,PD3Y0,PD3Y9,PDBBC,PRDPHI,PGWT0,PGWT9,&
   & PGFL,&
   ! --- INPUT/OUTPUT ---------------------------------------------------------
   & PGMV(1,1,1),PGMVS(1,1),PB1(1,1),PB2(1,1),PGMVT1(1,1,1),PGMVT1S(1,1),PGWS)

!*       2.2   Add large scale forcing for SCUM (Single Column Unified Model)

  ! A centered scheme is used for SL3TL and an uncentered scheme is used for
  ! SL2TL (RHS at time "t" for predictor step, provisional "t+dt" step for
  ! corrector step).
  IF (LSFORC) THEN
    CALL CP_FORCING(KST,KEND,PDT,PGFL(1,1,YFORC(1)%MP), &
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMVT1(1,1,YT1%MU),&
     & PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT),PGFLT1(1,1,YQ%MP1))
  ENDIF

!*       2.3   Simple physics.

  IF ((.NOT.LDLADNMI).AND.(.NOT.(LMPHYS.OR.LEPHYS)).AND.LSPHLC) THEN
    IF (LTWOTL) THEN
      CALL VDIFLCZ (KST,KEND,NPROMA,NFLEVG,&
       & PDT,&
       & PLSM,&
       & PHI0,PHIF0,PCP0,PR0,PRDELP0,&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),PRE0,&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),&
       & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT))  
    ELSE
      CALL VDIFLCZ (KST,KEND,NPROMA,NFLEVG,&
       & PDT,&
       & PLSM,&
       & PHI0,PHIF0,PCP0,PR0,PRDELP0,&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),PRE0,&
       & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
       & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT))  
    ENDIF
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       3.    Eulerian scheme.
!              ----------------

!* Cases nsiter=0, predictor for lpc_old, predictor and corrector for lpc_full.

IF ( &
 & .NOT.LSLAG.AND.((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL))&
 & ) THEN

!*       3.1   Dynamics with Eulerian advection treatment 

  CALL CPEULDYN(&
   ! --- INPUT ----------------------------------------------------
   & CDCONF,KST,KEND,PDT,&
   & POROG,PGFL,PTSOL,PQSOL,PNHXT0,&
   & PGNORDL,PGNORDM,&
   & PRCORI,PRCORIC,PGM,PRATATH,PRATATX,PSDVBC0,PKENE0,PEVEL0,PVVEL0,&
   & PSGRTL,PSGRTM,&
   & PVCASRS0F,PVCASRS0H,PVCW0F,PWDLW0F,&
   & PRE0,PRDELP0,PKAP0,PTNDPD,PTNDVD,P3DIVG,PD3Y9,PNHXT9,&
   ! --- INPUT/OUTPUT ---------------------------------------------
   & PD3Y0,PGFLT1,PGMVT1(1,1,1),PGMVT1S(1,1),PGFLPC,PGMV(1,1,1),PGMVS(1,1),&
   ! --- OUTPUT ---------------------------------------------------
   & PB2(1,1))

!*       3.2   Add large scale forcing for SCUM (Single Column Unified Model)

  ! A centered scheme is used for EUL.
  IF (LSFORC) THEN
    CALL CP_FORCING(KST,KEND,PDT,PGFL(1,1,YFORC(1)%MP), &
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMVT1(1,1,YT1%MU), &
     & PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT),PGFLT1(1,1,YQ%MP1))
  ENDIF

!*       3.3   Simple physics.

  IF ((.NOT.LDLADNMI).AND.(.NOT.(LMPHYS.OR.LEPHYS)).AND.LSPHLC) THEN
    CALL VDIFLCZ (KST,KEND,NPROMA,NFLEVG,&
     & PDT,&
     & PLSM,&
     & PHI0,PHIF0,PCP0,PR0,PRDELP0,&
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),PRE0,&
     & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
     & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT))  
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       4.   Stores some variables for AD/TL.
!             --------------------------------

IF (LTRAJSAVE .AND. LTRAJSLAG .AND. NTRSLTYPE==2) THEN
  IBLK=(KSTGLO-1)/NPROMA+1
  CALL STORE_TRAJ_SLAG(&
    & PB1(1,MSLB1U9+1-NFLSA),PB1(1,MSLB1V9+1-NFLSA),&
    & PB1(1,MSLB1T9+1-NFLSA),PB1(1,MSLB1GFL9), &
    & PB1(1,MSLB1C9+1-NFLSA),&
    & PB1(1,MSLB1U0+1-NFLSA),PB1(1,MSLB1V0+1-NFLSA),PB1(1,MSLB1T0+1-NFLSA),&
    & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),&
    & PGMVT1(1,1,YT1%MT),PGFLT1(1,1,YQ%MP1),&
    & PB1(1,MSLB1UR0+1-NFLSA),PB1(1,MSLB1VR0+1-NFLSA),&
    & PB1(1,MSLB1WR0+1-NFLSA),&
    & PB2(1,MSLB2URL),PB2(1,MSLB2VRL),PB2(1,MSLB2WRL),PB2(1,MSLB2KAPPA),&
    & PB1(1,MSLB1SP9),&
    & NSTEP,IBLK,KST,KEND)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_DYN',1,ZHOOK_HANDLE)
END SUBROUTINE CPG_DYN
