#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE CPG_DYN_TL( &
 ! --- INPUT ----------------------------------------------------------
 & CDCONF, &
 & KST,KEND,KSTGLO, &
 & LDLADNMI, &
 & PDT, &
 & PSLHDA,PSLHDD0, &
 & PGM,PRCORI,PGOMVRL,PGOMVRM,PRATATX,PRATATH, &
 & POROG,POROGL,POROGM, &
 & PRE0,PDELP0,PRDELP0, &
 & PHI0,PHIF0,PCP0,PR0, &
 & PKAP0,PEVEL0,PVVEL0,PSDVBC0,PKENE0, &
 & PSGRTL,PSGRTM, &
 & PGFL,&
 ! --- INPUT/OUTPUT ---------------------------------------------------
 & PGMV,PGMVS,&
 & PB1,PB2,PGMVT1,PGMVT1S,PGFLT1, &
 ! --- TRAJECTORY-INPUT -----------------------------------------------
 & PRE5,PDELP5,PRDELP5, &
 & PHI5,PHIF5, &
 & PCP5,PR5,PKAP5, &
 & PEVEL5,PVVEL5,PSDVBC5, &
 & PSGRT5L,PSGRT5M, &
 & PSD_VF5,&
 & PUT95,PVT95, &
 ! --- TRAJECTORY-INOUT -----------------------------------------------
 & PEDOTT95,PUNLT95,PVNLT95,PTNLT95,PSPNLT95, &
 & PB15,PGMV5,PGMV5S,PGFL5)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMDIM   , ONLY : NFLSA    ,NFLEVG   ,NPROMA
USE YOMDIMT  , ONLY : NPROMT
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL   ,LPC_OLD  ,LPC_FULL 
USE YOMCT3   , ONLY : NSTEP
USE YOMDYNA  , ONLY : LSLHD    ,LSLHD_STATIC
USE YOMDYN   , ONLY : NCURRENT_ITER  
USE YOMSC2   , ONLY : NFLDSLB2 ,NFLDSLB1,NFLDSLB15
USE YOPHLC   , ONLY : LSPHLC
USE YOM_YGFL , ONLY : YGFL
USE YOMGMV   , ONLY : NDIMGMV,NDIMGMVS,YT0,YT9,YT5,YT1
USE YOMTRSL  , ONLY : NTRSLTYPE
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : LEPHYS
USE PTRSLB15 , ONLY : MSLB1U95 ,MSLB1V95 ,MSLB1T95 ,MSLB1GFL95,MSLB1C95 ,&
 & MSLB1U05 ,MSLB1V05 ,MSLB1T05 ,MSLB1SP95,MSLB1UR05,MSLB1VR05,MSLB1WR05
USE PTRSLB2  , ONLY : MSLB2U15 ,MSLB2V15 ,MSLB2T15 ,MSLB2Q15 ,&
 & MSLB2URL5,MSLB2VRL5,MSLB2WRL5,MSLB2KAPPA5
USE SURFACE_FIELDS_MIX, ONLY : YSD_VFD, YSD_VF
USE TRAJ_SEMILAG_MOD, ONLY : GET_TRAJ_SLAG

!**** *CPG_DYN_TL* - Tan. lin. grid point calculations: non lagged dynamics.

!     Purpose.
!     --------
!           Tan. lin. grid point calculations: non lagged dynamics
!           + simple physics.

!**   Interface.
!     ----------
!        *CALL* *CPG_DYN_TL(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        CDCONF    : configuration.
!        KST       : first element of work.
!        KEND      : last element of work.
!        KSTGLO    : global offset.
!        LDLADNMI  : .T. if adiabatic NNMI iteration.
!        PDT       : For a leap-frog scheme (three time level scheme):
!                   'dt' at the first time-step, '2 dt' otherwise.
!                    For a 2TL SL scheme: timestep 'dt'.
!        PSLHDA    : Scaling factor of the deformation in f(d) function
!                    (including the model resolution correction)
!        PSLHDD0   : Treshold for deformation tensor enhancement
!        PGM       : mapping factor.
!        PRCORI    : Coriolis parameter "f = 2 Omega sin(theta)".
!        PGOMVRL   : zonal component of "2 Omega vec a k".
!        PGOMVRM   : meridian component of "2 Omega vec a k".
!        PRATATX   : spherical geometry: zero
!                    plane geometry: 2 S (sin(theta) - K_L)/(a cos(theta))
!                    Used for curvature terms.
!        PRATATH   : spherical geometry: 2 tan(theta_bne)/a
!                    (theta_bne = tilted but not stretched sphere latitude)
!                    plane geometry: 2 C (sin(theta) - K_L)/(a cos(theta))
!                    Used for curvature terms.
!        POROG     : surface orography.
!        POROGL    : zonal component of "rssavnabla(surf orography)"
!        POROGM    : meridian component of "rssavnabla(surf orography)"
!        PRE0      : hydrostatic pressure "prehyd" at half levels at time t.
!        PDELP0    ! hydrostatic pressure depth of layers at t.
!        PRDELP0   : 1/(hydrostatic pressure depth of layers) at t.
!        PHI0      : geopotential height "gz" at t at half levels.
!        PHIF0     : geopotential height "gz" at t at full levels.
!        PCP0      : "cp" at t.
!        PR0       : air constant "R" at t.
!        PKAP0     : "kappa = R/cp" at t.
!        PEVEL0    : "etadot d prehyd / d eta" at t.
!                    At half (resp. full) levels if LVERTFE=F (resp. T).
!        PVVEL0    : "omega/prehyd" at full levels at t.
!        PSDVBC0   : vertical integral of divergence computed in "gpcty",
!                    including the "lrubc" and "delta m=1" contributions
!                    of "etadot d prehyd / d eta".
!        PKENE0    : kinetic energy at t.
!        PSGRTL    : semi-reduced zonal component of the pressure force grad.
!        PSGRTM    : semi-reduced merid. component of the pressure force grad.
!        PGFL      : unified_treatment grid-point fields at t.

!     INPUT/OUTPUT:
!     -------------
!        PGMV      : GMV variables at t-dt and t.
!        PGMVS     : GMVS variables at t-dt and t.
!        PB1       : "SLBUF1" buffer.
!        PB2       : "SLBUF2" buffer.
!        PGMVT1    : GMV variables at t+dt.
!        PGMVT1S   : GMVS variables at t+dt.
!        PGFLT1    : t+dt term for the unified_treatment of grid-point fields
!                    (as input contains the non lagged physics).

!     TRAJECTORY (INPUT):
!     -------------------
!       PRE5,PDELP5,PRDELP5,
!       PHI5,PHIF5,
!       PCP5,PR5,PKAP5,
!       PEVEL5,PVVEL5,PSDVBC5,
!       PSGRT5L,PSGRT5M,
!       PSD_VF5,
!       They are computed in CPG5_GP, see in CPG5_GP for comments.

!     TRAJECTORY (INOUT):
!     -------------------
!       PEDOTT95  : etadot at t-dt (trajectory).
!       PUNLT95   : nonlinear term in U-equn at t-dt (trajectory).
!       PVNLT95   : nonlinear term in V-equn at t-dt (trajectory).
!       PTNLT95   : nonlinear term in T-equn at t-dt (trajectory).
!       PSPNLT95  : nonlinear term in continuity equn at t-dt (trajectory).
!       PB15      : Semi-Lagrangian buffer 15 (cf. PB1 but for trajectory).
!       PGMV5     : GMV variables at t-dt and t (trajectory).
!       PGMV5S    : GMVS variables at t-dt and t (trajectory).
!       PGFL5     : GFL variables at t-dt and t (trajectory).

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad, after CPGTL (old part 4 of CPGTL)

! Modifications
! -------------
!   Original     : 12 Jul 2004.
!   Modified by K. Yessad 21-Mar-2006: CPEULDYNTL instead of CPDYNTL+GNHDYNTL.
!   17-Apr-2007 S. Ivatek-S Over dimensioning of PGPNH and PGPNH5 to NFGPNH+1,
!                          boundary checking problem if NFGPNH=0 bf
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (Dec 2008): remove dummy CDLOCK and some useless dummy arg
!   F. Vana  13-Jan-2009: TL/AD of SLHD
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)       :: CDCONF
INTEGER(KIND=JPIM),INTENT(IN)       :: KST
INTEGER(KIND=JPIM),INTENT(IN)       :: KEND
INTEGER(KIND=JPIM),INTENT(IN)       :: KSTGLO
LOGICAL           ,INTENT(IN)       :: LDLADNMI
REAL(KIND=JPRB)   ,INTENT(IN)       :: PDT
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSLHDA(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSLHDD0(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PGM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRCORI(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PGOMVRL(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PGOMVRM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRATATX(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRATATH(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: POROG(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: POROGL(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: POROGM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRE0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PDELP0(NPROMA,0:NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRDELP0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PHI0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PHIF0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PCP0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PR0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PKAP0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PEVEL0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PVVEL0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSDVBC0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PKENE0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSGRTL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSGRTM(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PGFL(NPROMA,NFLEVG,YGFL%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGMVS(NPROMA,NDIMGMVS)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGMVT1S(NPROMA,YT1%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRE5(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PDELP5(NPROMA,0:NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PRDELP5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PHI5(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PHIF5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PCP5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PR5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PKAP5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PEVEL5(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PVVEL5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSDVBC5(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSGRT5L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSGRT5M(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PSD_VF5(NPROMA,YSD_VFD%NDIM5)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PUT95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)       :: PVT95(NPROMT,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PEDOTT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PUNLT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PVNLT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PTNLT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PSPNLT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PB15(NPROMA,NFLDSLB15)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGMV5S(NPROMA,YT5%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGFL5(NPROMA,NFLEVG,YGFL%NDIM5)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IBLK
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "cpeuldyntl.intfb.h"
#include "lacdyntl.intfb.h"
#include "vdiflcztl.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_DYN_TL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    NHS iteration: corrective step for LPC_OLD.
!              -------------------------------------------

IF (NCURRENT_ITER > 0 .AND. LPC_OLD) THEN
  CALL ABOR1('CPG_DYN_TL 1: LPC_OLD not coded')
ENDIF

!     ------------------------------------------------------------------

!*       2.    Semi-lagrangian scheme 
!              -----------------------

!* Cases nsiter=0, predictor for lpc_old, predictor and corrector for lpc_full.

IF(LSLAG.AND.((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL))) THEN

!*       2.1   Semi-lagrangian dynamics (not lagged part).

  CALL LACDYNTL(CDCONF,KST,KEND,PDT,&
   & PSLHDA,PSLHDD0,&
   & PUT95,PVT95,&
   & PGFL,&
   & PRE0,PKAP0,PDELP0,PRDELP0,PEVEL0,PVVEL0,PSGRTL,PSGRTM,&
   & PGFL5,&
   & PRE5,PKAP5,PDELP5,PRDELP5,PEVEL5,PVVEL5,PSGRT5L,PSGRT5M,&
   & PGM,PRCORI,PGOMVRL,PGOMVRM,&
   & POROG,POROGL,POROGM,&
   & PGMV(1,1,1),PGMVS(1,1),PB1(1,1),PB2(1,1),PGMVT1(1,1,1),PGMVT1S(1,1),&
   & PEDOTT95,PUNLT95,PVNLT95,PTNLT95,PSPNLT95,&
   & PGMV5(1,1,1),PGMV5S(1,1),PB15(1,1))  

!*       2.2   simplified physics.

  IF ((.NOT.LDLADNMI).AND.((.NOT.LMPHYS).OR.LEPHYS).AND.LSPHLC) THEN
    IF (LTWOTL) THEN
      CALL VDIFLCZTL (KST,KEND,NPROMA,NFLEVG,&
       & PDT,&
       & PSD_VF5(1,YSD_VF%YLSM%MP5),&
       & PHI5,&
       & PHI0,PHIF0,&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),&
       & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT))
    ELSE
      CALL VDIFLCZTL (KST,KEND,NPROMA,NFLEVG,&
       & PDT,&
       & PSD_VF5(1,YSD_VF%YLSM%MP5),&
       & PHI5,&
       & PHI0,PHIF0,&
       & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
       & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT))
    ENDIF
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       3.    Eulerian scheme.
!              ----------------

!* Cases nsiter=0, predictor for lpc_old, predictor and corrector for lpc_full.

IF (&
 & .NOT.LSLAG.AND.((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL))&
 & ) THEN

!*       3.1   Dynamics with Eulerian advection treatment

  CALL CPEULDYNTL(CDCONF,&
   & KST,KEND,PDT,&
   & PGFL,&
   & PRCORI,PGM,PRATATH,PRATATX,&
   & PSDVBC0,PKENE0,PEVEL0,PVVEL0,PSGRTL,PSGRTM,&
   & PRE0,PRDELP0,PKAP0,&
   & PGFLT1,PGMVT1(1,1,1),PGMVT1S(1,1),PGMV(1,1,1),PGMVS(1,1),&
   & PB2(1,1),&
   & PGMV5(1,1,1),PGFL5,PSDVBC5,PEVEL5,PVVEL5,PRE5,PRDELP5,PKAP5)

!*       3.2   Simple physics.

  IF ((.NOT.LDLADNMI).AND.((.NOT.LMPHYS).OR.LEPHYS).AND.LSPHLC) THEN
! IF ((.NOT.LDLADNMI).AND.(.NOT.(LMPHYS.OR.LEPHYS)).AND.LSPHLC) THEN
    CALL VDIFLCZTL (KST,KEND,NPROMA,NFLEVG,&
     & PDT,&
     & PSD_VF5(1,YSD_VF%YLSM%MP5),&
     & PHI5,&
     & PHI0,PHIF0,&
     & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
     & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),PGMVT1(1,1,YT1%MT))  
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!        4.    Read elements for semi-lagrangian trajectory into PB.
!              -----------------------------------------------------

IF (NTRSLTYPE == 2) THEN
  IBLK=(KSTGLO-1)/NPROMA+1
  CALL GET_TRAJ_SLAG(&
   & PB15(1,MSLB1U95+1-NFLSA),PB15(1,MSLB1V95+1-NFLSA),&
   & PB15(1,MSLB1T95+1-NFLSA),PB15(1,MSLB1GFL95-NFLSA),&
   & PB15(1,MSLB1C95+1-NFLSA),&
   & PB15(1,MSLB1U05+1-NFLSA),PB15(1,MSLB1V05+1-NFLSA),&
   & PB15(1,MSLB1T05+1-NFLSA),&
   & PB2(1,MSLB2U15),PB2(1,MSLB2V15),&
   & PB2(1,MSLB2T15),PB2(1,MSLB2Q15),&
   & PB15(1,MSLB1UR05+1-NFLSA),PB15(1,MSLB1VR05+1-NFLSA),&
   & PB15(1,MSLB1WR05+1-NFLSA),&
   & PB2(1,MSLB2URL5),PB2(1,MSLB2VRL5),PB2(1,MSLB2WRL5),&
   & PB2(1,MSLB2KAPPA5),PB15(1,MSLB1SP95),&
   & NSTEP,IBLK,KST,KEND)  

  IF (LSLHD.AND.LSLHD_STATIC) THEN
    ! In case of static SLHD (= different two standard SL interpolations)
    ! KAPPA becomes trivial, hence no need to save it from NL model.
    PB2(KST:KEND,MSLB2KAPPA5:MSLB2KAPPA5+NFLEVG-1)=1._JPRB
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_DYN_TL',1,ZHOOK_HANDLE)
END SUBROUTINE CPG_DYN_TL
