SUBROUTINE CPGLAGAD (CDCONF,KNUMB,&
 & PB2,PGM,PGMV,PGMVS,PGFL,PGMVT1,PGMVT1S)  

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE PTRSLB2  , ONLY : MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2SPSI,MSLB2PDSI, &
 &                    MSLB2VDSI,MSLB2T15 ,MSLB2Q15
USE YOMCT0   , ONLY : LNHDYN   ,LPC_FULL ,LTWOTL   ,LSPRT
USE YOMCT2   , ONLY : NSTOP2
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMDYN   , ONLY : NCURRENT_ITER   ,LDRY_ECMWF
USE YOMGEM   , ONLY : RSTRET
USE YOMCST   , ONLY : RD       ,RV
USE YOMGMV   , ONLY : YT0      ,YT9      ,YT1      ,NDIMGMV  ,NDIMGMVS
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMSC2   , ONLY : NFLDSLB2
USE YOM_YGFL , ONLY : YGFL     ,YGFLC    ,YQ

!**** *CPGLAGAD* - Grid point calculations lagged part.adjoint.

!     Purpose.
!     --------
!           Grid point calculations lagged part.AD

!**   Interface.
!     ----------
!        *CALL* *CPGLAGAD(...)*

!        Explicit arguments :
!        --------------------

!        CDCONF    : configuration of work
!        KNUMB     : number of elements of arrays for which computations
!                    are performed (used in message passing version)
!        PB2       : "SLBUF2" buffer.
!        PGM       : mapping factor.
!        PGMV      : "t-dt" and "t" GMV upper air variables.
!        PGMVS     : "t-dt" and "t" GMV surface variables.
!        PGFL      : "t-dt" and "t" GFL variables.
!        PGMVT1    : "t+dt" GMV upper air variables.
!        PGMVT1S   : "t+dt" GMV surface variables.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by GP_MODEL_AD.

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!        Jean-Noel Thepaut  *ECMWF*

! Modifications
! -------------
!   Original : 92-04-08
!   L.Isaksen: 96-02-20 Message Passing option added
!   J.F. Mahfouf: 97-02-24 Call to E.C.M.W.F. linear physics
!   E.Holm   : 98-01-14 Ozone included
!   C.Temperton: 99-10-19 Adjoint of semi-Lagrangian
!   M.Charron: 98-11-24 Nonhydrostatic dynamics
!   C. Fischer: 00-02-04 add dummy KNUMBE
!   Y.Tremolet: 00-08-15 use module TRAJECTORY
!   G. Mozdzynski: 00-07-06 take out lapinead for reproducibility
!   C.Fischer: 00-11-14 interface of PB2 in NHS iter step
!   C.Temperton: 01-01-26 add the case LSPRT=.TRUE.
!   R. El Khatib : 01-08-07 Pruning options
!   03-2002 J.Vivoda PC schemes for NH dynamics (LPC_XXXX keys)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   09-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   09-Apr-2006 M. Jidane  R dep. on q variables under key
!   K. Yessad (Dec 2008): cleanings
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVT1S(NPROMA,YT1%NDIMS) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IEND, ILEV, IST, JLEV, JROF,JGFL

LOGICAL :: LLLSTEP, LLSTR
REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZR, ZR5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gptf1ad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPGLAGAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    Interface to global arrays.
!              ---------------------------

!*       1.1   INITIAL COMPUTATIONS

IST   = 1
IEND  = KNUMB

ZEPS=100.0_JPRB*TINY(1.0_JPRB)
LLSTR=(ABS(RSTRET-1.0_JPRB)>ZEPS)

IF(NSTEP < NSTOP2-1) THEN
  LLLSTEP=.FALSE.
ELSE
  LLLSTEP=.TRUE.
ENDIF
IF(CDCONF == 'F') THEN
  LLLSTEP=.TRUE.
ENDIF

!     ------------------------------------------------------------------

!*       5.3  DIVIDE BY MAP FACTOR

IF(LLSTR) THEN
  DO JLEV=1,NFLEVG
    DO JROF=IST,IEND
      PGMVT1(JROF,JLEV,YT1%MU) = PGMVT1(JROF,JLEV,YT1%MU)/PGM(JROF)
      PGMVT1(JROF,JLEV,YT1%MV) = PGMVT1(JROF,JLEV,YT1%MV)/PGM(JROF)
    ENDDO
  ENDDO
ENDIF

!*       5.1  TRANSFER FROM BUFFER TO T+DT ARRAYS

! Case LSPRT=.TRUE. - see comments in CPGLAGTL

IF (LSPRT) THEN

  DO JLEV=1,NFLEVG
    ILEV=JLEV-1
    DO JROF=IST,IEND
      IF (.NOT. LDRY_ECMWF) ZR=PB2(JROF,MSLB2T15+ILEV)*PGMVT1(JROF,JLEV,YT1%MT)/RD
      ZR5=RD+(RV-RD)*PB2(JROF,MSLB2Q15+ILEV)
      PGMVT1(JROF,JLEV,YT1%MT)=ZR5*PGMVT1(JROF,JLEV,YT1%MT)/RD
      IF (.NOT. LDRY_ECMWF) PGFL(JROF,JLEV,YQ%MP1)=PGFL(JROF,JLEV,YQ%MP1)+(RV-RD)*ZR
    ENDDO
  ENDDO

ENDIF

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

  DO JLEV=1,NFLEVG
    ILEV = JLEV-1
    DO JROF=IST,IEND
      PB2(JROF,MSLB2USI+ILEV)=PB2(JROF,MSLB2USI+ILEV)+PGMVT1(JROF,JLEV,YT1%MU)
      PB2(JROF,MSLB2VSI+ILEV)=PB2(JROF,MSLB2VSI+ILEV)+PGMVT1(JROF,JLEV,YT1%MV)
      PB2(JROF,MSLB2TSI+ILEV)=PB2(JROF,MSLB2TSI+ILEV)+PGMVT1(JROF,JLEV,YT1%MT)
    ENDDO
    IF(LNHDYN) THEN
      DO JROF=IST,IEND
        PB2(JROF,MSLB2PDSI+ILEV)= PB2(JROF,MSLB2PDSI+ILEV)+&
         & PGMVT1(JROF,JLEV,YT1%MSPD)  
        PB2(JROF,MSLB2VDSI+ILEV)= PB2(JROF,MSLB2VDSI+ILEV)+&
         & PGMVT1(JROF,JLEV,YT1%MSVD)  
      ENDDO
    ENDIF
  ENDDO
  DO JROF=IST,IEND
    PB2(JROF,MSLB2SPSI) = PB2(JROF,MSLB2SPSI)+PGMVT1S(JROF,YT1%MSP)
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*    4.    TIME FILTER (PART 1).
!           ---------------------

IF(NCURRENT_ITER == 0)THEN
  IF(LTWOTL) THEN
    IF(LLLSTEP) THEN
      PGMV(:,:,YT0%NDIM+1:) = 0.0_JPRB
      PGMVS(:,YT0%NDIMS+1:) = 0.0_JPRB
      DO JGFL=1,YGFL%NUMFLDS
        IF(YGFLC(JGFL)%LT9) THEN
          PGFL(:,:,YGFLC(JGFL)%MP9) = 0.0_JPRB
        ENDIF
      ENDDO
    ENDIF
    DO JLEV=1,NFLEVG
      DO JROF=IST,IEND
        PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU)+PGMV(JROF,JLEV,YT9%MU)
        PGMV(JROF,JLEV,YT9%MU)=0.0_JPRB
        PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV)+PGMV(JROF,JLEV,YT9%MV)
        PGMV(JROF,JLEV,YT9%MV)=0.0_JPRB
      ENDDO
    ENDDO
  ELSE
    CALL GPTF1AD(IST,IEND,LLLSTEP,PGMV,PGMVS,PGFL)  
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPGLAGAD',1,ZHOOK_HANDLE)
END SUBROUTINE CPGLAGAD

