SUBROUTINE CPGLAGTL (KNUMB,&
 & PB2,PGM,PGMV,PGMVS,PGFL,PGMVT1,PGMVT1S)  

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE PTRSLB2  , ONLY : MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2SPSI,MSLB2PDSI, &
 &                    MSLB2VDSI,MSLB2T15 ,MSLB2Q15
USE YOMCST   , ONLY : RD      ,RV
USE YOMCT0   , ONLY : LNHDYN   ,LPC_FULL ,LSPRT
USE YOMCT2   , ONLY : NSTOP2
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMDYN   , ONLY : NCURRENT_ITER   ,LDRY_ECMWF
USE YOMGEM   , ONLY : RSTRET
USE YOMGMV   , ONLY : YT1      ,NDIMGMV  ,NDIMGMVS
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMSC2   , ONLY : NFLDSLB2
USE YOM_YGFL , ONLY : YGFL     ,YQ

!**** *CPGLAGTL* - Grid point calculations lagged part.Tan. linear.

!     Purpose.
!     --------
!           Grid point calculations lagged part.TL

!**   Interface.
!     ----------
!        *CALL* *CPGLAGTL(...)*

!        Explicit arguments :
!        --------------------

!        KNUMB     : number of elements of arrays for which computations
!                    are performed (used in message passing version)
!        PB2       : "SLB2" second buffer.
!        PGM       : mapping factor.
!        PGMV      : "t-dt" and "t" GMV upper air variables.
!        PGMVS     : "t-dt" and "t" GMV surface variables.
!        PGFL      : "t-dt" and "t" GFL variables.
!        PGMVT1    : "t+dt" GMV upper air variables.
!        PGMVT1S   : "t+dt" GMV surface variables.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by GP_MODEL_TL.

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!        Jean-Noel Thepaut  *ECMWF*

! Modifications
! -------------
!   Original : 92-04-08
!   L.Isaksen: 96-02-02 Message Passing option added
!   J.F. Mahfouf: 97-02-24 Call to E.C.M.W.F. linear physics
!   E.Holm   : 98-01-14 Ozone included
!   M. Charron   : 98-09-22 NH variables
!   C. Fischer: 00-02-04 add dummy KNUMBE
!   C.Temperton: 99-07-19 Tangent-linear of semi-Lagrangian scheme
!   Y.Tremolet: 00-08-15 use module TRAJECTORY
!   C.Fischer: 00-11-14 interface of PB2 in NHS iter step
!   C.Temperton: 01-01-26 add the case LSPRT=.TRUE.
!   R. El Khatib : 01-08-07 Pruning options
!   03-2002 J.Vivoda PC schemes for NH dynamics (LPC_XXXX keys)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   09-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   08-Apr-2006 M. Jidane : Reintroduction of R dep on q varaibles under key
!   K. Yessad (Dec 2008): cleanings
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB2(NPROMA,NFLDSLB2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IEND, ILEV, IST, JLEV, JROF

LOGICAL :: LLLSTEP, LLSTR
REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZR5   ,ZR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gptf1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPGLAGTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    Interface to global arrays.
!              ---------------------------

!*       1.1   INITIAL COMPUTATIONS

IST   = 1
IEND  = KNUMB

ZEPS=100.0_JPRB*TINY(1.0_JPRB)
LLSTR=(ABS(RSTRET-1.0_JPRB)>ZEPS)

IF(NSTEP < NSTOP2) THEN
  LLLSTEP=.FALSE.
ELSE
  LLLSTEP=.TRUE.
ENDIF

!     ------------------------------------------------------------------

!*    4.    TIME FILTER (PART 1).
!           ---------------------

IF (NCURRENT_ITER == 0) THEN
  CALL GPTF1(IST,IEND,LLLSTEP,PGMV,PGMVS,PGFL)  
ENDIF

!     ------------------------------------------------------------------

!*       5.   FINISH
!             ------

!*       5.1  TRANSFER FROM BUFFER TO T+DT ARRAYS (USUAL CASE)

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

  DO JLEV=1,NFLEVG
    ILEV = JLEV-1
    DO JROF=IST,IEND
      PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)+PB2(JROF,MSLB2USI+ILEV)
      PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)+PB2(JROF,MSLB2VSI+ILEV)
      PGMVT1(JROF,JLEV,YT1%MT)=PGMVT1(JROF,JLEV,YT1%MT)+PB2(JROF,MSLB2TSI+ILEV)
    ENDDO
    IF (LNHDYN) THEN
      DO JROF=IST,IEND
        PGMVT1(JROF,JLEV,YT1%MSPD)=PGMVT1(JROF,JLEV,YT1%MSPD)+&
         & PB2(JROF,MSLB2PDSI+ILEV)  
        PGMVT1(JROF,JLEV,YT1%MSVD)=PGMVT1(JROF,JLEV,YT1%MSVD)+&
         & PB2(JROF,MSLB2VDSI+ILEV)  
      ENDDO
    ENDIF
  ENDDO
  DO JROF=IST,IEND
    PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)+PB2(JROF,MSLB2SPSI)
  ENDDO
ENDIF

! Case LSPRT=.TRUE. - Note that in order to prevent the adjoint from
! being impossibly complicated, the "T15" and "Q15" values used here
! are not really the correct ones!

IF (LSPRT) THEN
  DO JLEV=1,NFLEVG
    ILEV=JLEV-1
    DO JROF=IST,IEND
      ZR5=RD+(RV-RD)*PB2(JROF,MSLB2Q15+ILEV)
      IF (LDRY_ECMWF) THEN
        PGMVT1(JROF,JLEV,YT1%MT)=ZR5*PGMVT1(JROF,JLEV,YT1%MT)/RD
      ELSE
        ZR=(RV-RD)*PGFL(JROF,JLEV,YQ%MP1)
        PGMVT1(JROF,JLEV,YT1%MT)=(ZR5*PGMVT1(JROF,JLEV,YT1%MT) &
         & +ZR*PB2(JROF,MSLB2T15+ILEV))/RD
      ENDIF
    ENDDO
  ENDDO
ENDIF

!*       5.3  DIVIDE BY MAP FACTOR

IF(LLSTR) THEN
  DO JLEV=1,NFLEVG
    DO JROF=IST,IEND
      PGMVT1(JROF,JLEV,YT1%MU) = PGMVT1(JROF,JLEV,YT1%MU)/PGM(JROF)
      PGMVT1(JROF,JLEV,YT1%MV) = PGMVT1(JROF,JLEV,YT1%MV)/PGM(JROF)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPGLAGTL',1,ZHOOK_HANDLE)
END SUBROUTINE CPGLAGTL
