!OCL  NOEVAL
SUBROUTINE CPMVVPS (LDVERCOR,KPROMA,KPROMVC,KSTART,KPROF,KFLEV,PDT,&
 !  Variables 2D Input
 & PVCASRSH, PFP, &
 !  Variables 1D Input
 & PRES, PFEVL, PFEVN,&
 !  Variables 2D In/Out
 & PEVEL, PSDVBC, PSDFI, PSPT1)

!     ------------------------------------------------------------------
!     MODIFICATION DE LA VITESSE VERTICALE ET DE LA PRESSION DE SURFACE
!     DANS LE CAS NDPSFI = 1.
!     ------------------------------------------------------------------

!      VOIR DOCUMENTATION, INTERFACE PHYSICO-DYNAMIQUE
!                            -----------------

!     ARGUMENTS D ENTREE.
!     ------------------.
!       LDVERCOR: .T./.F.:
!                 pas hypothese de couche mince/hypothese de couche mince.
!                 thin layer hypothesis relaxed/applied.
!       KPROMA : DIMENSION HORIZONTALE.
!       KPROMVC: horizontal dimension for arrays used only if "lvercor=T".
!                Dimension horizontale pour les tableaux qui ne sont utilises
!                que si "lvercor=T".
!       KSTART : DEBUT DE LA BOUCLE HORIZONTALE.
!       KPROF : BORNE HORIZONTALE DES CALCULS.
!       KFLEV : DIMENSION ET BORNE VERTICALE.
!       PDT   : Delta t (SL2TL or first timestep) or 2 Delta t.

! --- INPUT 2D.
!     --------.
!       PVCASRSH: "a/rs" sur les intercouches.
!                 "a/rs" on interlayers.
!       PFP : FLUX TOTAL DE PRECIPITATIONS LIQUIDES ET NEIGEUSES.

! --- INPUT 1D.
!     --------.
!       PRES : PRESSION DE SURFACE A L'INSTANT OU EST CALCULEE LA PHYSIQUE.
!              Surface pressure at the same instant as non lagged physics.
!       PFEVL, PFEVN ( IDEM) : FLUX D'EVAPORATION.

!     ARGUMENTS IMPLICITES
!     --------------------
!       CONSTANTES UNIVERSELLES = COMMON /YOMCST/: RG
!       DECOUPAGE VERTICAL      = COMMON /YOMGEM/: VBH
!       YOMGEM UTILISE YOMDIM,PARDIM

!     SORTIES
!     -------
!       PEVEL (KPROMA,0:KFLEV) : VITESSE VERTICALE GENERALISEE.
!       PSDVBC (IDEM) : Integral of divergence term, including
!                       the "lrubc" and "delta m=1" contributions
!                       of (etadot DP/DETA), but not the
!                       "delta m=1" physics.
!       PSDFI (IDEM)  : Integral of divergence term, including
!                       the "lrubc" and "delta m=1" contributions
!                       of (etadot DP/DETA), and the
!                       "delta m=1" physics.
!       PSPT1 (KPROMA) : surface pressure or log of pressure buffer.

!     AUTEUR : E.BAZILE  JUIN 93.
!     ------

!     INSPIRE DE CPATY ECRIT EN SON TEMPS PAR A.JOLY

!     Modifications:
!     --------------
!      Modified by K. YESSAD (Nov 1999): additional dummy arguments.
!       - "psdiv" replaced by the couple "(psdvbc,psdfi)".
!       - add physics to "pspt1" is done in this routine.
!      R. El Khatib : 01-08-07 Pruning options
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      K. Yessad and Y. Seity : Apr 2005 : reformulation of delta m=1 for 
!                                          barycentric system.
!      K. Yessad (Dec 2008): remove dummy CDLOCK and useless dummy arg
!    ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMGEM   , ONLY : VBH
USE YOMCST   , ONLY : RG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMVC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
LOGICAL           ,INTENT(IN)    :: LDVERCOR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRSH(KPROMVC,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFP(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVEL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSDVBC(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSDFI(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT1(KPROMA) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZFE(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZEVELS(KPROMA)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPMVVPS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!     ------------------------------------------------------------------
!     1. - MODIFY :
!          * The divergence vertical integral term.
!          * etadot (d prehyd/d eta).
!     ------------------------------------------------------------------

! * ZFE : FLUX D EVAPORATION TOTAL
DO JROF = KSTART, KPROF
  ZFE(JROF,KFLEV)=PFEVL(JROF)+PFEVN(JROF)
  ZEVELS(JROF)=ZFE(JROF,KFLEV)+PFP(JROF,KFLEV)
ENDDO
ZFE(KSTART:KPROF,0:KFLEV-1)=0.0_JPRB

! * 1.1 Surface/bottom values:

IF (LDVERCOR) THEN
  DO JROF = KSTART, KPROF
    PEVEL(JROF,KFLEV) = PEVEL(JROF,KFLEV) &
     & + RG*PVCASRSH(JROF,KFLEV)*PVCASRSH(JROF,KFLEV)*ZEVELS(JROF)  
  ENDDO
ELSE
  DO JROF = KSTART, KPROF
    PEVEL(JROF,KFLEV) = PEVEL(JROF,KFLEV) + RG*ZEVELS(JROF)
  ENDDO
ENDIF
DO JROF = KSTART, KPROF
  PSDVBC(JROF,KFLEV) = PSDVBC(JROF,KFLEV) + RG*ZEVELS(JROF)
  PSDFI(JROF,KFLEV) = PSDFI(JROF,KFLEV) + RG*ZEVELS(JROF) &
   & + RG*(PFP(JROF,KFLEV) + ZFE(JROF,KFLEV))
ENDDO

! * 1.2 Other levels:

IF (LDVERCOR) THEN
  DO JLEV = 1, KFLEV-1
    DO JROF = KSTART, KPROF
      PEVEL(JROF,JLEV) = PEVEL(JROF,JLEV) + RG*( &
       & PVCASRSH(JROF,KFLEV)*PVCASRSH(JROF,KFLEV)*VBH(JLEV)*ZEVELS(JROF) )
    ENDDO
  ENDDO
ELSE
  DO JLEV = 1, KFLEV-1
    DO JROF = KSTART, KPROF
      PEVEL(JROF,JLEV) = PEVEL(JROF,JLEV) &
       & + RG*( VBH(JLEV)* ZEVELS(JROF) )  
    ENDDO
  ENDDO
ENDIF

DO JLEV = 1, KFLEV-1
  DO JROF = KSTART, KPROF
    PSDVBC(JROF,JLEV)= PSDVBC(JROF,JLEV) &
     & + RG*( VBH(JLEV)* ZEVELS(JROF) )
    PSDFI(JROF,JLEV) = PSDFI(JROF,JLEV) &
     & + RG*( VBH(JLEV)* ZEVELS(JROF) ) &
     & + RG*(PFP(JROF,JLEV) + ZFE(JROF,JLEV))
  ENDDO
ENDDO

!    ------------------------------------------------------------------
!     2. - ADD PHYSICS TO PSPT1.
!    ------------------------------------------------------------------

IF (LDVERCOR) THEN
  DO JROF = KSTART, KPROF
    PSPT1(JROF)=PSPT1(JROF) &
     & -PDT*RG*PVCASRSH(JROF,KFLEV)*PVCASRSH(JROF,KFLEV) &
     & *(PFP(JROF,KFLEV)+ZFE(JROF,KFLEV))/PRES(JROF)  
  ENDDO
ELSE
  DO JROF = KSTART, KPROF
    PSPT1(JROF)=PSPT1(JROF) &
     & -PDT*RG*(PFP(JROF,KFLEV)+ZFE(JROF,KFLEV))/PRES(JROF)  
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPMVVPS',1,ZHOOK_HANDLE)
END SUBROUTINE CPMVVPS
