SUBROUTINE CPPHINPAD(KST,KEND,&
 & PUT5,PVT5,PQT5,PQT5L,PQT5M,PRDELP5,PEVEL5,&
 & PUT0,PVT0,PQT0,PQT0L,PQT0M,PRDELP0,PEVEL0,&
 & PCVGQ5,&
 & PCVGQ)  

!**** *CPPHINPAD*

!    Subject.
!    --------
!    Adjoint computation of:
!    - COMPUTATION OF CONVERGENCE OF HUMIDITY  AND PHYSICAL WIND
!      COMPONENTS

!**  Interface.
!    ----------
!     *CALL* *CPPHINPAD*

! ---------------------------------------------------------------------

!    Externals.
!    ----------

!    Method.
!    -------

!    Author.
!    -------
!    96-08-08  M. Janiskova

!    Modifications.
!    --------------
!       99-01, code optimization - M. Janiskova
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
! ---------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMM

! ---------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVEL0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCVGQ5(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCVGQ(NPROMM,NFLEVG) 

! ---------------------------------------------------------------------

REAL(KIND=JPRB) :: ZED5(NPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZED
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPPHINPAD',0,ZHOOK_HANDLE)
! ---------------------------------------------------------------------

!*       1.1   ASTRONOMY.

!     This computation was already done in CPPHINPADT

!*    1.1.2 Convergence of humidity and physical wind components.

DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    PCVGQ5(JROF,JLEV)=&
     & -PQT5L(JROF,JLEV)*PUT5(JROF,JLEV)&
     & -PQT5M(JROF,JLEV)*PVT5(JROF,JLEV)  
  ENDDO
ENDDO

DO JLEV=1,NFLEVG-1
  DO JROF=KST,KEND
    ZED5(JROF,JLEV)=0.5_JPRB*PEVEL5(JROF,JLEV)
    PCVGQ5(JROF,JLEV)=PCVGQ5(JROF,JLEV)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *ZED5(JROF,JLEV)*PRDELP5(JROF,JLEV)  

    PCVGQ5(JROF,JLEV+1)=PCVGQ5(JROF,JLEV+1)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *ZED5(JROF,JLEV)*PRDELP5(JROF,JLEV+1)  
  ENDDO
ENDDO

!*
!     ---------------------------------------------------------------
!     1.2   CONVERGENCE OF HUMIDITY AND PHYSICAL WIND COMPONENTS.

DO JLEV=NFLEVG-1,1,-1
  DO JROF=KST,KEND
    ZED=0.0_JPRB
    PRDELP0(JROF,JLEV+1)=PRDELP0(JROF,JLEV+1)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *ZED5(JROF,JLEV)*PCVGQ(JROF,JLEV+1)  
    ZED=ZED+(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *PRDELP5(JROF,JLEV+1)*PCVGQ(JROF,JLEV+1)  
    PQT0(JROF,JLEV)=PQT0(JROF,JLEV)+ZED5(JROF,JLEV)&
     & *PRDELP5(JROF,JLEV+1)*PCVGQ(JROF,JLEV+1)  
    PQT0(JROF,JLEV+1)=PQT0(JROF,JLEV+1)-ZED5(JROF,JLEV)&
     & *PRDELP5(JROF,JLEV+1)*PCVGQ(JROF,JLEV+1)  
    PCVGQ(JROF,JLEV+1)=PCVGQ(JROF,JLEV+1)

    PRDELP0(JROF,JLEV)=PRDELP0(JROF,JLEV)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *ZED5(JROF,JLEV)*PCVGQ(JROF,JLEV)  
    ZED=ZED+(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *PRDELP5(JROF,JLEV)*PCVGQ(JROF,JLEV)  
    PQT0(JROF,JLEV)=PQT0(JROF,JLEV)+ZED5(JROF,JLEV)&
     & *PRDELP5(JROF,JLEV)*PCVGQ(JROF,JLEV)  
    PQT0(JROF,JLEV+1)=PQT0(JROF,JLEV+1)-ZED5(JROF,JLEV)&
     & *PRDELP5(JROF,JLEV)*PCVGQ(JROF,JLEV)  
    PCVGQ(JROF,JLEV)=PCVGQ(JROF,JLEV)

    PEVEL0(JROF,JLEV)=PEVEL0(JROF,JLEV)+0.5_JPRB*ZED
    ZED=0.0_JPRB
  ENDDO
ENDDO

DO JLEV=NFLEVG,1,-1
  DO JROF=KST,KEND
    PUT0(JROF,JLEV)=PUT0(JROF,JLEV)-PQT5L(JROF,JLEV)*PCVGQ(JROF,JLEV)
    PQT0L(JROF,JLEV)=PQT0L(JROF,JLEV)-PUT5(JROF,JLEV)*PCVGQ(JROF,JLEV)
    PVT0(JROF,JLEV)=PVT0(JROF,JLEV)-PQT5M(JROF,JLEV)*PCVGQ(JROF,JLEV)
    PQT0M(JROF,JLEV)=PQT0M(JROF,JLEV)-PVT5(JROF,JLEV)*PCVGQ(JROF,JLEV)
  ENDDO
ENDDO

! ---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPPHINPAD',1,ZHOOK_HANDLE)
END SUBROUTINE CPPHINPAD

