SUBROUTINE CPPHINPTL(KST,KEND,&
 & PGEMU,PGELAM,&
 & PUT5,PVT5,PQT5,PQT5L,PQT5M,PRDELP5,PEVEL5,&
 & PUT0,PVT0,PQT0,PQT0L,PQT0M,PRDELP0,PEVEL0,&
 & PMU0,PCVGQ5,&
 & PCVGQ)  

!**** *CPPHINPTL* 

!    Subject.
!    --------
!    Tangent linear of:
!    - COMPUTATION OF SOLAR ANGLE AND CONVERGENCE OF HUMIDITY
!      AND PHYSICAL WIND COMPONENTS

!**  Interface.
!    ----------
!     *CALL* *CPPHINPTL*

! ---------------------------------------------------------------------

!    Externals.
!    ----------

!    Method.
!    -------

!    Author.
!    -------
!    96-08-06 M. Janiskova

!    Modifications.
!    --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
! ---------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMM
USE YOMRIP   , ONLY : RCODEC   ,RSIDEC   ,RCOVSR   ,RSIVSR

! ---------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMU0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCVGQ5(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCVGQ(NPROMM,NFLEVG) 

! ---------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZED, ZED5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPPHINPTL',0,ZHOOK_HANDLE)
! ---------------------------------------------------------------------

!*       1.1   ASTRONOMY.

DO JROF=KST,KEND
  PMU0(JROF)=MAX( RSIDEC*PGEMU(JROF)&
   & -RCODEC*RCOVSR*SQRT(1.0_JPRB-PGEMU(JROF)**2)*COS(PGELAM(JROF))&
   & +RCODEC*RSIVSR*SQRT(1.0_JPRB-PGEMU(JROF)**2)*SIN(PGELAM(JROF))&
   & ,0.0_JPRB)  
ENDDO

!*    1.2   CONVERGENCE OF HUMIDITY AND PHYSICAL WIND COMPONENTS.

DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    PCVGQ5(JROF,JLEV)=&
     & -PQT5L(JROF,JLEV)*PUT5(JROF,JLEV)&
     & -PQT5M(JROF,JLEV)*PVT5(JROF,JLEV)  
    PCVGQ(JROF,JLEV)=-PQT5L(JROF,JLEV)*PUT0(JROF,JLEV)&
     & -PUT5(JROF,JLEV)*PQT0L(JROF,JLEV)&
     & -PQT5M(JROF,JLEV)*PVT0(JROF,JLEV)&
     & -PVT5(JROF,JLEV)*PQT0M(JROF,JLEV)   
  ENDDO
ENDDO

DO JLEV=1,NFLEVG-1
  DO JROF=KST,KEND
    ZED5=0.5_JPRB*PEVEL5(JROF,JLEV)
    ZED=0.5_JPRB*PEVEL0(JROF,JLEV)

    PCVGQ5(JROF,JLEV)=PCVGQ5(JROF,JLEV)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *ZED5*PRDELP5(JROF,JLEV)  
    PCVGQ(JROF,JLEV)=PCVGQ(JROF,JLEV)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))*ZED5 &
     & *PRDELP0(JROF,JLEV)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *PRDELP5(JROF,JLEV)*ZED &
     & +ZED5*PRDELP5(JROF,JLEV)*PQT0(JROF,JLEV)&
     & -ZED5*PRDELP5(JROF,JLEV)*PQT0(JROF,JLEV+1)  

    PCVGQ5(JROF,JLEV+1)=PCVGQ5(JROF,JLEV+1)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *ZED5*PRDELP5(JROF,JLEV+1)  
    PCVGQ(JROF,JLEV+1)=PCVGQ(JROF,JLEV+1)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))*ZED5 &
     & *PRDELP0(JROF,JLEV+1)&
     & +(PQT5(JROF,JLEV)-PQT5(JROF,JLEV+1))&
     & *PRDELP5(JROF,JLEV+1)*ZED &
     & +ZED5*PRDELP5(JROF,JLEV+1)*PQT0(JROF,JLEV)&
     & -ZED5*PRDELP5(JROF,JLEV+1)*PQT0(JROF,JLEV+1)  
  ENDDO
ENDDO

! ---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPPHINPTL',1,ZHOOK_HANDLE)
END SUBROUTINE CPPHINPTL
