SUBROUTINE CPQTUV ( KPROMA, KSTART, KPROF, KFLEV, PDT,&
 !  Variables 2D Input
 & PUT9 ,PVT9 ,PTT9 ,PGFL ,&
 !  Variables 2D In/Out
 & PUT1 ,PVT1 ,PTT1 ,PGFLT1 )  

!     ------------------------------------------------------------------
!     Update wind, temperature and GFL variables.
!     ------------------------------------------------------------------

!     INPUTS
!     ------
!       KPROMA : Horizontal dimension
!       KSTART : Start point
!       KPROF  : End point
!       KFLEV  : Number of levels
!       PDT    : Timestep for physics
!       PUT9   : t-dt U-wind.
!       PVT9   : t-dt V-wind.
!       PTT9   : t-dt temperature.
!       PGFL   : t-dt GFL variables. 

!     INPUT/OUTPUT
!     ------------
!       PUT1   : t+dt U-wind.
!       PVT1   : t+dt V-wind.
!       PTT1   : t+dt temperature.
!       PGFLT1 : t+dt GFL variables.

!     OUTPUTS
!     -------
!  ------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_YGFL , ONLY : YGFL   ,YGFLC

!  ------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(KPROMA,KFLEV,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(KPROMA,KFLEV,YGFL%NDIM1) 

!  ------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF, JGFL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!  ------------------------------------------------------------


!  ------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPQTUV',0,ZHOOK_HANDLE)

!  ------------------------------------------------------------

!*     1.    TIME STEPPING
!            -------------


DO JLEV = 1, KFLEV
  DO JROF = KSTART, KPROF
    PUT1(JROF,JLEV) = PUT9(JROF,JLEV) + PDT * PUT1(JROF,JLEV)
    PVT1(JROF,JLEV) = PVT9(JROF,JLEV) + PDT * PVT1(JROF,JLEV)
    PTT1(JROF,JLEV) = PTT9(JROF,JLEV) + PDT * PTT1(JROF,JLEV)
  ENDDO
ENDDO

DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LT1) THEN
    DO JLEV = 1, KFLEV
      DO JROF = KSTART, KPROF
        PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1) = PGFL(JROF,JLEV,YGFLC(JGFL)%MP9_PH) + &
         & PDT * PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)  
      ENDDO
    ENDDO
  ENDIF
ENDDO

!  ------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPQTUV',1,ZHOOK_HANDLE)
END SUBROUTINE CPQTUV

