SUBROUTINE CPTEND_NEW ( KPROMA, KSTART, KPROF, KFLEV,&
 !  Variables 2D Input
 & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS ,&
 & PDIFTQ , PDIFTQI, PDIFTQL, PDIFTS ,&
 & PFCCQL , PFCCQN , PFCSQL , PFCSQN ,&
 & PFPLSL , PFPLSN , PFPLCL,  PFPLCN, &
 & PFESL  , PFESN  , PFECL  , PFECN  ,&
 & PFASL  , PFASN  , PFACL  , PFACN  ,&
 & PFCQLNG, PFCQING, PFCQRNG, PFCQSNG,&
 & PFCQNG , PFRMH  , PFRSO  , PFRTH  ,&
 & PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
 & PSTRTU , PSTRTV , PSTRMU , PSTRMV ,&
 & PFTKE  ,&
 & PRDELP , PAPHIF , PCP    ,&
 & PU     , PV     , PT     ,&
 & PQ     , PQI    , PQL    ,&
 & PQR    , PQS    ,&
 !  Variables 1D Input
 & PCPS   , PTS    ,&
 !  Variables 2D Output
 & PFHSCL , PFHSCN , PFHSSL , PFHSSN,&
 & PFHPCL , PFHPCN , PFHPSL , PFHPSN,&
 & PFHP   , PFP    , PFEPFP , PFCMPCQ, PFCMPSN, PFCMPSL,&
 & PTENDU , PTENDV , PTENDH ,&
 & PTENDQ , PTENDQI, PTENDQL, PTENDQR, PTENDQS,&
 & PTENDTKE )

!     ------------------------------------------------------------------
!     ACTION DES PROCESSUS PHYSIQUES SUR LA QUANTITE DE MOUVEMENT ,
!     LES VARIABLES THERMODYNAMIQUES ET SUR L'EQUATION DE CONTINUITE
!     INTERFACE AVEC LES PARAMETRISATIONS PHYSIQUES  (IALPP)
!     ------------------------------------------------------------------

!     BUT
!     ---
!**** *CPTEND* CACUL DES TENDANCES PHYSIQUES SUR U, V, H, Q, QI, QL, TKE
!      SPECIFIQUE ET SUR LA TEMPERATURE.

!      VOIR DOCUMENTATION, INTERFACE PHYSICO-DYNAMIQUE.
!                            -----------------

!      POUR COMPRENDRE LA LOGIQUE DE CETTE ROUTINE, BIEN REALISER
!      QUE LA VARIABLE THERMODYNAMIQUE DE LA PHYSIQUE EST L ENTHALPIE
!                                                         -----------
!      (PLUS EXACTEMENT L ENERGIE STATIQUE HUMIDE)

!      LE TRANSPORT EVENTUEL PAR LES PLUIES EST PRIS EN COMPTE DE
!      MANIERE MIXTE :
!      - UNE PARTIE DE CE TRANSPORT EST CONTENUE DE MANIERE IMPLICITE
!        DANS LA FORMULATION EN FLUX D ENTHALPIE
!      - IL RESTE POURTANT UN TERME DE TRANSPORT D ENERGIE POTENTIELLE
!        QUI EST TRAITE ICI COMME UNE ADVECTION VERTICALE

!     ARGUMENTS D ENTREE
!     ------------------
!       KPROMA : DIMENSION HORIZONTALE
!       KSTART : DEBUT DE LA BOUCLE HORIZONTALE
!       KPROF  : BORNE HORIZONTALE DES CALCULS
!       KFLEV  : DIMENSION ET BORNE VERTICALE

! --- INPUT 2D (0:KLEV).
!     -----------------.
! PDIFCQ     : FLUX CONVECTIF D'HUMIDITE SPECIFIQUE (HORS COND. ET RR).
! PDIFCQI    : FLUX CONVECTIF D'EAU SOLIDE (HORS COND. ET RR).
! PDIFCQL    : FLUX CONVECTIF D'EAU LIQUIDE (HORS COND. ET RR).
! PDIFCS     : FLUX CONVECTIF D'ENTHALPIE .
! PDIFTQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
! PDIFTQI    : FLUX TURBULENT (ET Q NEGATIF) D'EAU GLACE.
! PDIFTQL    : FLUX TURBULENT (ET Q NEGATIF) D'EAU LIQUIDE.
! PDIFTS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
! PFCCQL     : FLUX DE CONDENSATION LIE AUX RR CONVECTIVES LIQUIDES.
! PFCCQN     : FLUX DE CONDENSATION LIE AUX RR CONVECTIVES NEIGEUSES.
! PFCSQL     : FLUX DE CONDENSATION LIE AUX RR STRATIFORMES LIQUIDES.
! PFCSQN     : FLUX DE CONDENSATION LIE AUX RR STRATIFORMES NEIGEUSES.
! PFPLCL     : PRECIPITATIONS CONVECTIVES SOUS FORME LIQUIDE.
! PFPLCN     : PRECIPITATIONS CONVECTIVES SOUS FORME NEIGE.
! PFPLSL     : PRECIPITATIONS STRATIFORMES SOUS FORME LIQUIDE.
! PFPLSN     : PRECIPITATIONS STRATIFORMES SOUS FORME NEIGE.
! PFRMH      : FLUX MESOSPHERIQUE D'ENTHALPIE.
! PFRSO      : FLUX DE RAYONNEMENT SOLAIRE.
! PFRTH      : FLUX DE RAYONNEMENT THERMIQUE.
! PSTRCU     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "U".
! PSTRCV     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "V".
! PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".
! PSTRTU     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "U".
! PSTRTV     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "V".
! PSTRMU     : FLUX MESOSPHERIQUE DE QUANTITE DE MOUVEMENT "U".
! PSTRMV     : FLUX MESOSPHERIQUE DE QUANTITE DE MOUVEMENT "V".
! PFTKE      : FLUX DE "TKE".

! --- INPUT 2D (1:KLEV).
!     -----------------.
! PRDELP     : INVERSE DE L'EPAISSEUR EN PRESSION DE LA COUCHE.
! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.
! PT         : TEMPERATURE.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PQI        : GLACE.
! PQL        : EAU LIQUIDE.
! PQR        : RAIN
! PQS        : SNOW

! - 1D.
!   --.
! PCPS       : CHALEUR SPECIFIQUE EN SURFACE.
! PTS        : TEMPERATURE DE SURFACE.

! --- ARGUMENTS IMPLICITES.
!     --------------------.
! CONSTANTES UNIVERSELLES = COMMON /YOMCST/: RG,RCPD
! INDICATEURS LOGIQUES    = COMMON /YOMPHY/: NDPSFI,LCONDWT

! --- SORTIES 2D (0:KFLEV).
!     --------------------.
! PFHSCL : FLUX DE CHALEUR SENSIBLE LIE AUX RR CONVECTIVES LIQUIDES.
! PFHSCN : FLUX DE CHALEUR SENSIBLE LIE AUX RR CONVECTIVES NEIGEUSES.
! PFHSSL : FLUX DE CHALEUR SENSIBLE LIE AUX RR STRATIFORMES LIQUIDES.
! PFHSSN : FLUX DE CHALEUR SENSIBLE LIE AUX RR STRATIFORMES NEIGEUSES.
! PFHPCL : FLUX D'ENTHALPIE LIE A LA CONDENSATION CONVECTIVE LIQUIDE.
! PFHPCN : FLUX D'ENTHALPIE LIE A LA CONDENSATION CONVECTIVE NEIGEUSE.
! PFHPSL : FLUX D'ENTHALPIE LIE A LA CONDENSATION STRATIFORME LIQUIDE.
! PFHPSN : FLUX D'ENTHALPIE LIE A LA CONDENSATION STRATIFORME NEIGEUSE.
! PFHP   : FLUX TOTAL D'ENTHALPIE + FLUX DE CHALEUR SENSIBLE.
! PFP    : FLUX TOTAL DE PRECIPITATIONS.
! PFEPFP : PSEUDO FLUX D'ENTHALPIE LIE A L'ENERGIE PRECIPITATIONS.

! --- SORTIES 2D (1:KFLEV).
!     --------------------.
! PTENDU     : TENDANCE DU VENT SUR U.
! PTENDV     : TENDANCE DU VENT SUR V.
! PTENDH     : TENDANCE DE L'ENTHALPIE.
! PTENDQ     : TENDANCE DE L'HUMIDITE.
! PTENDQI    : TENDANCE DE L'EAU GLACE.
! PTENDQL    : TENDANCE DE L'EAU LIQUIDE.
! PTENDQR    : TENDANCE DU CONTENU EN EAU PRECIPITANT LIQUIDE.
! PTENDQS    : TENDANCE DU CONTENU EN EAU PRECIPITANT SOLIDE.
! PTENDTKE   : TENDANCE DE TKE.

! --- AUTEUR.
!     ------.
!     02/01/92 E.BAZILE.
!        Reunification de Cpaty,Cpdthp,Cpdup par E.Bazile.

!        Modified 92-06-18 by C.Dreveton (ajout PFHP en argument)
!        Modified 93-06-22 by E.BAZILE (ajout de l'eau liq. et sol.)
!        Modified 04-01-16 by P. Marquet (Lopez and Gueremy schemes,
!                          they both add PFPEVPP to PTENDQ, with
!                          PFPFP and PFADVP only for Lopez) 
!                          The signs of the fluxes PFPEVPP, PFPFP and 
!                          PFADVP have changed in ADVPRC.
!        Modified 07-02-01 by M.Janousek (Catry's modifications for
!                          NDPSFI=1; tendency of pTKE)
!        Modified 07-05-07 by F.Bouyssel (tendency of TKE + Cleaning)
!        Modified 07-06-13 by T.Kovacic, PFCMPCQ, PFCMPSN, PFCMPSL added for DDH
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG, RCPD, RCW, RCS, RLVZER, RLSZER
USE YOMPHY   , ONLY : NDPSFI, LCONDWT, LPHSPSH, LECT
USE YOMLDDH  , ONLY : LSDDH

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQI (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQL (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQI (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQL (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRMH   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRMU  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRMV  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTKE   (KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP  (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF  (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFASL   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFASN   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFACL   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFACN   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFESL   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFESN   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFECL   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFECN   (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNG  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLNG (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQING (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQRNG (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQSNG (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP     (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU      (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV      (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT      (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ      (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQI     (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQL     (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQR     (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS     (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPS    (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS     (KPROMA)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSCL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSCN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSSL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSSN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPCL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPCN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSL  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSN  (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHP    (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFP     (KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEPFP  (KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDU  (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDV  (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDH  (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDQ  (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDQI (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDQL (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDQR (KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDQS (KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDTKE(KPROMA,KFLEV)

!-----------------------------------------------------------------------

REAL(KIND=JPRB) :: ZGSDP (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZJTOT (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZTI   (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZQH   (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZQLH  (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZQIH  (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZQRH  (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZQSH  (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZCHAT (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZLIFT (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPL  (KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPN  (KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPCQ(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSL(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPLSL(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPLSN(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPLCL(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPLCN(KPROMA,0:KFLEV)

INTEGER(KIND=JPIM) :: JLEV, JROF

LOGICAL :: LLDPSFI

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPTEND_NEW',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------

! ------------------------------------------------------------------
! intermediate CPTEND version made for acpluie_prog and Luc's scheme
! ------------------------------------------------------------------

! the following assumptions are made:

! * Lopez scheme uses both QR and QS as prognostic variable
! * the diffusive fluxes (for qv, qi, ql and s) have a turbulent 
!   and convective component
! * the pseudo-fluxes between the species (','',''') all have a
!   stratiform component, only condensation has a convective
!   component
! * CPFHPFS is integrated in this routine and should not be called
! ------------------------------------------------------------------

!    -------------------------------------------------------------------

LLDPSFI=(NDPSFI==1)
ZLIFT=0.0_JPRB
ZCHAT=0.0_JPRB

!  ------------------------------------------------------------
!   1. CALCUL DE L'EVOLUTION DE Q ET CALCUL PARTIEL DE dU/dT ET
!  ET DE dH/dt ( avec H = CpT + Phi + Ec )
!  ------------------------------------------------------------

! Calculation of temperatures on half levels

IF (LPHSPSH) THEN
  DO JROF = KSTART, KPROF
    ZTI(JROF,0)=PT(JROF,1)
    ZTI(JROF,KFLEV)=PT(JROF,KFLEV)
  ENDDO
ELSE
  DO JROF = KSTART, KPROF
    ZTI(JROF,0)=PT(JROF,1)
    ZTI(JROF,KFLEV)=PTS(JROF)
  ENDDO
ENDIF

DO JLEV= 1, KFLEV-1
  DO JROF = KSTART, KPROF
    ZTI(JROF,JLEV)=(PT(JROF,JLEV)+PT(JROF,JLEV+1))*0.5_JPRB
  ENDDO
ENDDO

! preparation for case delta_m=1

IF (LLDPSFI) THEN
  DO JROF = KSTART, KPROF
    ZQH  (JROF,0) = 0.0_JPRB
    ZQLH (JROF,0) = 0.0_JPRB
    ZQIH (JROF,0) = 0.0_JPRB
    ZQRH (JROF,0) = 0.0_JPRB
    ZQSH (JROF,0) = 0.0_JPRB
    ZQH  (JROF,KFLEV) = PQ (JROF,KFLEV)
    ZQLH (JROF,KFLEV) = PQL(JROF,KFLEV)
    ZQIH (JROF,KFLEV) = PQI(JROF,KFLEV)
    ZQRH (JROF,KFLEV) = PQR(JROF,KFLEV)
    ZQSH (JROF,KFLEV) = PQS(JROF,KFLEV)
  ENDDO
  DO JLEV= 1, KFLEV-1
    DO JROF = KSTART, KPROF
      ZQH (JROF,JLEV) = (PQ (JROF,JLEV)+PQ (JROF,JLEV+1))*0.5_JPRB
      ZQLH(JROF,JLEV) = (PQL(JROF,JLEV)+PQL(JROF,JLEV+1))*0.5_JPRB
      ZQIH(JROF,JLEV) = (PQI(JROF,JLEV)+PQI(JROF,JLEV+1))*0.5_JPRB
      ZQRH(JROF,JLEV) = (PQR(JROF,JLEV)+PQR(JROF,JLEV+1))*0.5_JPRB
      ZQSH(JROF,JLEV) = (PQS(JROF,JLEV)+PQS(JROF,JLEV+1))*0.5_JPRB
    ENDDO
  ENDDO
ENDIF

! Transformation between absolute and relative precip fluxes

IF (LLDPSFI) THEN
  DO JLEV= 0, KFLEV
    DO JROF = KSTART, KPROF
      ZFPL(JROF,JLEV)=(1.0_JPRB-ZQRH(JROF,JLEV))*PFPLSL(JROF,JLEV)&
        & -ZQRH(JROF,JLEV)*PFPLSN(JROF,JLEV)
      ZFPN(JROF,JLEV)=(1.0_JPRB-ZQSH(JROF,JLEV))*PFPLSN(JROF,JLEV)&
        & -ZQSH(JROF,JLEV)*PFPLSL(JROF,JLEV)
      ZFPLSL(JROF,JLEV)=ZFPL(JROF,JLEV)
      ZFPLSN(JROF,JLEV)=ZFPN(JROF,JLEV)
      ZFPL(JROF,JLEV)=(1.0_JPRB-ZQRH(JROF,JLEV))*PFPLCL(JROF,JLEV)&
        & -ZQRH(JROF,JLEV)*PFPLCN(JROF,JLEV)
      ZFPN(JROF,JLEV)=(1.0_JPRB-ZQSH(JROF,JLEV))*PFPLCN(JROF,JLEV)&
        & -ZQSH(JROF,JLEV)*PFPLCL(JROF,JLEV)
      ZFPLCL(JROF,JLEV)=ZFPL(JROF,JLEV)
      ZFPLCN(JROF,JLEV)=ZFPN(JROF,JLEV)
      ZFPL(JROF,JLEV)=ZFPLSL(JROF,JLEV)+ZFPLCL(JROF,JLEV)
      ZFPN(JROF,JLEV)=ZFPLSN(JROF,JLEV)+ZFPLCN(JROF,JLEV)
      ZCHAT(JROF,JLEV)=(PCP(JROF,JLEV)-RCW*ZQRH(JROF,JLEV)-&
       & RCS*ZQSH(JROF,JLEV))/(1.0_JPRB-ZQRH(JROF,JLEV)-ZQSH(JROF,JLEV))
      ZLIFT(JROF,JLEV)=(ZFPL(JROF,JLEV)+ZFPN(JROF,JLEV))/&
        & (1.0_JPRB-ZQRH(JROF,JLEV)-ZQSH(JROF,JLEV))
    ENDDO
  ENDDO
ELSE
  DO JLEV= 0, KFLEV
    DO JROF = KSTART, KPROF
      ZFPL(JROF,JLEV)=PFPLSL(JROF,JLEV)+PFPLCL(JROF,JLEV)
      ZFPN(JROF,JLEV)=PFPLSN(JROF,JLEV)+PFPLCN(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

! Diagnostics

DO JLEV= 0, KFLEV
  DO JROF = KSTART, KPROF

    IF (LLDPSFI) THEN

      PFHSCL(JROF,JLEV)=ZFPLCL(JROF,JLEV)*ZTI(JROF,JLEV)*RCW
      PFHSCN(JROF,JLEV)=ZFPLCN(JROF,JLEV)*ZTI(JROF,JLEV)*RCS
      PFHSSL(JROF,JLEV)=ZFPLSL(JROF,JLEV)*ZTI(JROF,JLEV)*RCW
      PFHSSN(JROF,JLEV)=ZFPLSN(JROF,JLEV)*ZTI(JROF,JLEV)*RCS

    ELSE

      PFHSCL(JROF,JLEV)=PFPLCL(JROF,JLEV)*ZTI(JROF,JLEV)*(RCW-RCPD)
      PFHSCN(JROF,JLEV)=PFPLCN(JROF,JLEV)*ZTI(JROF,JLEV)*(RCS-RCPD)
      PFHSSL(JROF,JLEV)=PFPLSL(JROF,JLEV)*ZTI(JROF,JLEV)*(RCW-RCPD)
      PFHSSN(JROF,JLEV)=PFPLSN(JROF,JLEV)*ZTI(JROF,JLEV)*(RCS-RCPD)

    ENDIF

    PFHPCL(JROF,JLEV)= - RLVZER * (PFCCQL(JROF,JLEV)-PFECL(JROF,JLEV))
    PFHPCN(JROF,JLEV)= - RLSZER * (PFCCQN(JROF,JLEV)-PFECN(JROF,JLEV))
    PFHPSL(JROF,JLEV)= - RLVZER * (PFCSQL(JROF,JLEV)-PFESL(JROF,JLEV))
    PFHPSN(JROF,JLEV)= - RLSZER * (PFCSQN(JROF,JLEV)-PFESN(JROF,JLEV))

    PFP(JROF,JLEV)= ZFPL(JROF,JLEV) +ZFPN(JROF,JLEV)

    PFEPFP(JROF,JLEV)=-NDPSFI*ZCHAT(JROF,JLEV)*ZTI(JROF,JLEV)*&
     & PFP(JROF,JLEV)

    PFHP(JROF,JLEV) = PFHSCL(JROF,JLEV) + PFHSCN(JROF,JLEV) +&
     & PFHSSL(JROF,JLEV) + PFHSSN(JROF,JLEV) +&
     & PFHPCL(JROF,JLEV) + PFHPCN(JROF,JLEV) +&
     & PFHPSL(JROF,JLEV) + PFHPSN(JROF,JLEV) +&
     & PFEPFP(JROF,JLEV)  

  ENDDO
ENDDO

! Computation of the total enthalpy flux J_tot (including NDPSFI==1)

DO JLEV= 0, KFLEV
  DO JROF = KSTART, KPROF
    ZJTOT(JROF,JLEV) = PFRSO(JROF,JLEV)+PFRTH(JROF,JLEV)+PFRMH(JROF,JLEV)+&
        & PDIFTS(JROF,JLEV)+PDIFCS(JROF,JLEV)+&
        & (RCW-RCPD)*ZFPL(JROF,JLEV)*ZTI(JROF,JLEV)+&
        & (RCS-RCPD)*ZFPN(JROF,JLEV)*ZTI(JROF,JLEV)-&
        & RLVZER*(PFCCQL(JROF,JLEV)+PFCSQL(JROF,JLEV)-PFESL(JROF,JLEV)&
        & -PFECL(JROF,JLEV))-&
        & RLSZER*(PFCCQN(JROF,JLEV)+PFCSQN(JROF,JLEV)-PFESN(JROF,JLEV)&
        & -PFECN(JROF,JLEV))
  ENDDO
ENDDO

IF (LLDPSFI) THEN
  DO JLEV= 0, KFLEV
    DO JROF = KSTART, KPROF
      ZJTOT(JROF,JLEV) = ZJTOT(JROF,JLEV)-&
          & (ZCHAT(JROF,JLEV)-RCPD)*ZTI(JROF,JLEV)*&
          &  (ZFPL(JROF,JLEV)+ZFPN(JROF,JLEV))
    ENDDO
  ENDDO
ENDIF

!CDIR UNROLL=8
DO JLEV = 1, KFLEV
  DO JROF = KSTART, KPROF

    ZGSDP(JROF,JLEV)=RG*PRDELP(JROF,JLEV)

    PTENDU(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
     & (PSTRCU (JROF,JLEV-1) - PSTRCU (JROF,JLEV)) +&
     & (PSTRDU (JROF,JLEV-1) - PSTRDU (JROF,JLEV)) +&
     & (PSTRMU (JROF,JLEV-1) - PSTRMU (JROF,JLEV)) +&
     & (PSTRTU (JROF,JLEV-1) - PSTRTU (JROF,JLEV)) )  

    PTENDV(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
     & (PSTRCV (JROF,JLEV-1) - PSTRCV (JROF,JLEV)) +&
     & (PSTRDV (JROF,JLEV-1) - PSTRDV (JROF,JLEV)) +&
     & (PSTRMV (JROF,JLEV-1) - PSTRMV (JROF,JLEV)) +&
     & (PSTRTV (JROF,JLEV-1) - PSTRTV (JROF,JLEV)) )  

    PTENDQ(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
     & (PDIFTQ (JROF,JLEV-1) - PDIFTQ (JROF,JLEV)) +&
     & (PDIFCQ (JROF,JLEV-1) - PDIFCQ (JROF,JLEV)) +&
     & (PFCCQL (JROF,JLEV-1) - PFCCQL (JROF,JLEV)) +&
     & (PFCCQN (JROF,JLEV-1) - PFCCQN (JROF,JLEV)) +&
     & (PFCSQL (JROF,JLEV-1) - PFCSQL (JROF,JLEV)) +&
     & (PFCSQN (JROF,JLEV-1) - PFCSQN (JROF,JLEV)) +&
     & (PFCQNG (JROF,JLEV-1) - PFCQNG (JROF,JLEV)) -&
     & (PFECL  (JROF,JLEV-1) - PFECL  (JROF,JLEV)) -&
     & (PFESL  (JROF,JLEV-1) - PFESL  (JROF,JLEV)) -&
     & (PFECN  (JROF,JLEV-1) - PFECN  (JROF,JLEV)) -&
     & (PFESN  (JROF,JLEV-1) - PFESN  (JROF,JLEV)))

    PTENDH(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
     &  ZJTOT  (JROF,JLEV-1) - ZJTOT  (JROF,JLEV))

  ENDDO
ENDDO

!  ------------------------------------------------------------
!   2. CALCUL DE L'EVOLUTION DE QL, QI, QR, QS
!  ------------------------------------------------------------

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Pronostic equations for liquid/solid water/precipitation 
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

IF (LCONDWT) THEN
  
  ! - - - - - - - - - - - - - - - - - -
  ! Usual equation for QL=suspended liquid water
  ! - - - - - - - - - - - - - - - - - -

!cdir unroll=8
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PTENDQL(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
       & (PDIFTQL(JROF,JLEV-1) - PDIFTQL(JROF,JLEV)) +&
       & (PDIFCQL(JROF,JLEV-1) - PDIFCQL(JROF,JLEV)) +&
       & (PFCQLNG(JROF,JLEV-1) - PFCQLNG(JROF,JLEV)) +&
       & (PFACL  (JROF,JLEV-1) - PFACL  (JROF,JLEV)) +&
       & (PFASL  (JROF,JLEV-1) - PFASL  (JROF,JLEV)) -&
       & (PFCSQL (JROF,JLEV-1) - PFCSQL (JROF,JLEV)) -&
       & (PFCCQL (JROF,JLEV-1) - PFCCQL (JROF,JLEV)))
    ENDDO
  ENDDO

  ! - - - - - - - - - - - - - - - - - -
  ! equation for QI=suspended ice water
  ! - - - - - - - - - - - - - - - - - -

!cdir unroll=8
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PTENDQI(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
       & (PDIFTQI(JROF,JLEV-1) - PDIFTQI(JROF,JLEV)) +&
       & (PDIFCQI(JROF,JLEV-1) - PDIFCQI(JROF,JLEV)) +&
       & (PFCQING(JROF,JLEV-1) - PFCQING(JROF,JLEV)) +&
       & (PFACN  (JROF,JLEV-1) - PFACN  (JROF,JLEV)) +&
       & (PFASN  (JROF,JLEV-1) - PFASN  (JROF,JLEV)) -&
       & (PFCSQN (JROF,JLEV-1) - PFCSQN (JROF,JLEV)) -&
       & (PFCCQN (JROF,JLEV-1) - PFCCQN (JROF,JLEV)))
    ENDDO
  ENDDO

  ! -----------------------------------
  ! Equation for QR=rain water
  ! -----------------------------------

!cdir unroll=8
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PTENDQR(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
       & (PFESL  (JROF,JLEV-1) - PFESL  (JROF,JLEV)) +&
       & (PFECL  (JROF,JLEV-1) - PFECL  (JROF,JLEV)) +&
       & (PFCQRNG(JROF,JLEV-1) - PFCQRNG(JROF,JLEV)) +&
       & (ZFPL   (JROF,JLEV-1) - ZFPL   (JROF,JLEV)) -&
       & (PFACL  (JROF,JLEV-1) - PFACL  (JROF,JLEV)) -&
       & (PFASL  (JROF,JLEV-1) - PFASL  (JROF,JLEV)))
    ENDDO
  ENDDO

  ! ---------------------------------
  ! Equation for QS=snow water
  ! --------------------------------

!cdir unroll=8
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PTENDQS(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
       & (PFESN  (JROF,JLEV-1) - PFESN  (JROF,JLEV)) +&
       & (PFECN  (JROF,JLEV-1) - PFECN  (JROF,JLEV)) +&
       & (PFCQSNG(JROF,JLEV-1) - PFCQSNG(JROF,JLEV)) +&
       & (ZFPN   (JROF,JLEV-1) - ZFPN   (JROF,JLEV)) -&
       & (PFACN  (JROF,JLEV-1) - PFACN  (JROF,JLEV)) -&
       & (PFASN  (JROF,JLEV-1) - PFASN  (JROF,JLEV)))
    ENDDO
  ENDDO
ELSE
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PTENDQI(JROF,JLEV) = 0.0_JPRB
      PTENDQL(JROF,JLEV) = 0.0_JPRB
      PTENDQS(JROF,JLEV) = 0.0_JPRB
      PTENDQR(JROF,JLEV) = 0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     For DDH, could be used in next loop for PTENDQ, PTENDQI,PTENDQL 
IF (LSDDH) THEN
  IF (LLDPSFI) THEN
    DO JLEV = 0, KFLEV
      DO JROF = KSTART, KPROF
        PFCMPCQ(JROF,JLEV) = ZQH(JROF,JLEV)*ZLIFT(JROF,JLEV)
        IF (LCONDWT) THEN
           PFCMPSN(JROF,JLEV) = ZQIH(JROF,JLEV)*ZLIFT(JROF,JLEV)
           PFCMPSL(JROF,JLEV) = ZQLH(JROF,JLEV)*ZLIFT(JROF,JLEV)
         ENDIF
      ENDDO
    ENDDO
  ELSE
    DO JLEV = 0, KFLEV
      DO JROF = KSTART, KPROF
        PFCMPCQ(JROF,JLEV) =  0.0_JPRB 
        IF (LCONDWT) THEN
           PFCMPSN(JROF,JLEV) =  0.0_JPRB
           PFCMPSL(JROF,JLEV) =  0.0_JPRB
         ENDIF
      ENDDO
    ENDDO
  ENDIF
ENDIF

IF (LLDPSFI) THEN
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
       PTENDQ(JROF,JLEV) = PTENDQ(JROF,JLEV)-ZGSDP(JROF,JLEV)*&
         & (ZQH(JROF,JLEV-1)*ZLIFT(JROF,JLEV-1)-&
         & ZQH(JROF,JLEV)*ZLIFT(JROF,JLEV))
       IF (LCONDWT) THEN
         PTENDQI(JROF,JLEV) = PTENDQI(JROF,JLEV)-ZGSDP(JROF,JLEV)*&
           & (ZQIH(JROF,JLEV-1)*ZLIFT(JROF,JLEV-1)-&
           & ZQIH(JROF,JLEV)*ZLIFT(JROF,JLEV))
         PTENDQL(JROF,JLEV) = PTENDQL(JROF,JLEV)-ZGSDP(JROF,JLEV)*&
           & (ZQLH(JROF,JLEV-1)*ZLIFT(JROF,JLEV-1)-&
           & ZQLH(JROF,JLEV)*ZLIFT(JROF,JLEV))
       ENDIF
    ENDDO
  ENDDO
ENDIF

!  ------------------------------------------------------------
!   3. CALCUL DE L'EVOLUTION DE TKE
!  ------------------------------------------------------------

IF (LECT) THEN
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PTENDTKE(JROF,JLEV) = ZGSDP(JROF,JLEV)*(&
       & PFTKE (JROF,JLEV-1) - PFTKE (JROF,JLEV))
    ENDDO
  ENDDO
ENDIF

!  ------------------------------------------------------------
!   4. CALCUL DE L'EVOLUTION DE P ET W (CAS NH)
!  ------------------------------------------------------------

! TO DO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPTEND_NEW',1,ZHOOK_HANDLE)
END SUBROUTINE CPTEND_NEW
