SUBROUTINE CPUTQY ( KPROMA, KSTART, KPROF, KFLEV, PDT,&
 !  Variables 2D Input
 & PTENDH, PTENDQ, PTENDQI, PTENDQL, PTENDQR, PTENDQS,&
 & PTENDPTKE, PTENDU, PTENDV,&
 & PTENDTKE, PTENDT, &
 & PCP9, PDELP9, PTT9, PUT9, PVT9,&
 !  Variables 2D In/Out
 & PFDIS,PTT1, PQT1, PQIT1, PQLT1, PQRT1, PQST1, PUT1, PVT1, PTKET1 )

!     -----------------------------------------------------------------
!**** *CPUTQY*  EVOLUTION DE T, VENT , Q , QI , QL, QR, QS
!               ET  TKE.
!     -----------------------------------------------------------------

!     ARGUMENTS D'ENTREE
!     ------------------
!       KPROMA : DIMENSION HORIZONTALE
!       KSTART : DEBUT DE LA BOUCLE HORIZONTALE
!       KPROF : BORNE HORIZONTALE DES CALCULS
!       KFLEV : DIMENSION ET BORNE VERTICALE
!       PDT : PAS DE TEMPS EFFECTIF

!       PTENDH (KPROMA,KFLEV) : TENDANCE DE L'ENTHALPIE
!       PTENDQ ( IDEM ) : TENDANCE DE L'EAU VAPEUR
!       PTENDQI ( IDEM ) : TENDANCE DE L'EAU GLACE
!       PTENDQL ( IDEM ) : TENDANCE DE L'EAU LIQUIDE
!       PTENDQR ( IDEM ) : TENDANCE DE L'EAU PRECIPITANTE LIQUIDE
!       PTENDQS ( IDEM ) : TENDANCE DE L'EAU PRECIPITANTE SOLIDE
!       PTENDPTKE ( IDEM ) : TENDANCE DU TKE
!       PTENDU, PTENDV (KPROMA,KFLEV) : TENDANCE DU VENT
!       PTENDTKE : TENDANCE DE L'ENERGIE KINETIQUE TURB.
!       PTENDT   : TENDANCE DE LA TEMPERATURE SI ON N'UTILISE PAS
!                  LES FLUXES, MAIS LES TENDANCES (HIRLAM OPTION)
!       PCP9 ( IDEM ) : CHALEUR SPECIFIQUE A T=9
!       PDELP9 ( IDEM ) : EPAISSEUR DE LA COUCHE A T=9
!       PTT9, PUT9, PVT9 (IDEM) : ETAT THERMODYNAMIQUE A T=9

!     ARGUMENTS IMPLICITES
!     --------------------
!       CONSTANTS UNIVERSELLES  = COMMON /YOMCST/:  RG,RCPD,RCPV,RCS,RCW

!     SORTIES
!     -------
!       PTT1, PQT1 (KPROMA,KFLEV): VARIABLES A FAIRE EVOLUER
!       PQIT1,PQLT1 (IDEM) : EAU SOL. ET LIQ. A FAIRE EVOLUER
!       PQST1,PQRT1 (IDEM) : EAU SOL. ET LIQ. PRECIPITANTE A FAIRE EVOLUER
!       PUT1, PVT1 (IDEM)    : VENT A FAIRE EVOLUER
!       PFDIS(KPROMA,0:KFLEV): FLUX ENTHALPIE DU A LA DISSIPATION EC
!       PTKET1               : ENERGIE KINETIQUE TURB. A FAIRE EVOLUER

!      02/01/92  Bonne Annee
!        Rediger par E.Bazile .
!      18/06/92 Modification par C.Dreveton (ajout PFDIS)
!      22/06/93 Modification par E.Bazile (ajout de l'eau liquide et solide )
!      07/03/95 Modification par E.Bazile (Optimisation des calculs)
!      07/10/04 Modification par F. Bouyssel (Lopez Ql/Qi/Qr prog. scheme)
!      25/01/06 Modification par F. Bouyssel (PTENDQS, PQST1)
!      01/06/06 Modification par B. Sass (PTENDTKE,PTKET1,PTENDT) 
!      30/10/06 Modification par F. Bouyssel (Bug correction found by M.Bellus)
!      02/10/06 Modification par M. Bellus (PTENDPTKE,LPTKE,L3MT,LSTRAPRO);
!               + corrected bug in ZTDCP comp. (missing snow, rain parts)
!      01/02/07 Modification par M.Janousek (ALARO pTKE is not *Dt, adding
!               pTKE tendency to T and enthalpy diss.flux)
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RCPD     ,RCPV     ,RCW      ,RCS
USE YOMPHY   , ONLY : LCONDWT  ,LPROCLD, LECT      ,&
 & LPTKE     ,L3MT   ,LSTRAPRO
USE YHLOPTION, ONLY : LHL, LHLTEND

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQ(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQI(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQS(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDPTKE(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDU(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDTKE(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDT(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFDIS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQIT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQRT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQST1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKET1(KPROMA,KFLEV)

!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZCPF, ZDEC, ZFLAG, ZH9, ZRDT, ZRRG, ZTDCP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

LOGICAL :: LLPREC, LLTKE

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPUTQY',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------

!  ------------------------------------------------------------
!   1. CALCUL DE L'EVOLUTION DU VENT , DE Q ET DE LA TEMPERATURE
!  ------------------------------------------------------------

LLPREC=LPROCLD.OR.L3MT.OR.LSTRAPRO
LLTKE=LECT.OR.LHL

IF (PDT <= 0.0_JPRB) THEN
  ZFLAG=0.0_JPRB
  ZRDT= 0.0_JPRB
ELSE
  ZFLAG=1.0_JPRB
  ZRDT= 1.0_JPRB/PDT
ENDIF
ZRRG   = 1.0_JPRB/RG
DO JROF=KSTART,KPROF
  PFDIS(JROF,0)=0.0_JPRB
ENDDO
DO JLEV = 1, KFLEV
  DO JROF = KSTART, KPROF

    PUT1(JROF,JLEV) = PUT1(JROF,JLEV) + PDT * PTENDU(JROF,JLEV)
    PVT1(JROF,JLEV) = PVT1(JROF,JLEV) + PDT * PTENDV(JROF,JLEV)
    PQT1(JROF,JLEV) = PQT1(JROF,JLEV) + PDT * PTENDQ(JROF,JLEV)

!     SI PAS D'EAU LIQUIDE LES TENDANCES SONT MISES A ZERO DANS CPTEND

    ZTDCP = (RCPV - RCPD) * PTENDQ(JROF,JLEV)
 
    IF (LCONDWT) THEN
      PQIT1(JROF,JLEV)=PQIT1(JROF,JLEV) + PDT*PTENDQI(JROF,JLEV)
      PQLT1(JROF,JLEV)=PQLT1(JROF,JLEV) + PDT*PTENDQL(JROF,JLEV)
      ZTDCP = ZTDCP + (RCW  - RCPD)* PTENDQL(JROF,JLEV) +&
       & (RCS  - RCPD)* PTENDQI(JROF,JLEV)  
    ENDIF

    IF (LLPREC) THEN
      PQRT1(JROF,JLEV)=PQRT1(JROF,JLEV) + PDT*PTENDQR(JROF,JLEV)
      PQST1(JROF,JLEV)=PQST1(JROF,JLEV) + PDT*PTENDQS(JROF,JLEV)
      ZTDCP = ZTDCP + (RCW  - RCPD)* PTENDQR(JROF,JLEV) +&
       & (RCS  - RCPD)* PTENDQS(JROF,JLEV)  
    ENDIF

    IF (LLTKE) THEN
      PTKET1(JROF,JLEV)=PTKET1(JROF,JLEV) + PDT*PTENDTKE(JROF,JLEV)
    ENDIF

    IF (LPTKE) THEN
      ! ALARO-0 pTKE
      PTKET1(JROF,JLEV)=PTKET1(JROF,JLEV) + PDT*PTENDPTKE(JROF,JLEV)
    ENDIF

    IF( (LHL).AND.(LHLTEND) ) THEN

!     HIRLAM CASE WHEN PROCESS ACCUMULATION OF TENDENCIES IS ACTIVE.

      PTT1(JROF,JLEV) = PTT1(JROF,JLEV) + PDT*PTENDT(JROF,JLEV)

    ELSE

!      NORMAL CASE

      ZCPF = 1.0_JPRB / ( PCP9(JROF,JLEV) + PDT * ZTDCP )

      ZDEC = PDT * ( PUT9(JROF,JLEV)*PTENDU(JROF,JLEV) +&
      & PVT9(JROF,JLEV)*PTENDV(JROF,JLEV) + 0.5_JPRB * PDT * (&
      & (PTENDU(JROF,JLEV))**2 + (PTENDV(JROF,JLEV))**2 ))  
      ZH9 = PCP9(JROF,JLEV) * PTT9(JROF,JLEV)

      PTT1(JROF,JLEV) = PTT1(JROF,JLEV) + ( ZCPF * ( ZH9 - ZDEC &
      & + PDT * PTENDH(JROF,JLEV) ) - PTT9(JROF,JLEV) ) * ZFLAG
      PFDIS(JROF,JLEV)= PFDIS(JROF,JLEV-1)+ ZRDT*ZRRG*PDELP9(JROF,JLEV)*ZDEC

      IF (LPTKE) THEN
        PTT1(JROF,JLEV) = PTT1(JROF,JLEV) - &
           & ZCPF * PDT * PTENDPTKE(JROF,JLEV) * ZFLAG
        PFDIS(JROF,JLEV)= PFDIS(JROF,JLEV)+ &
           &ZRDT*ZRRG*PDELP9(JROF,JLEV)*PDT*PTENDPTKE(JROF,JLEV)*ZFLAG
      ENDIF

    ENDIF

  ENDDO
ENDDO

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPUTQY',1,ZHOOK_HANDLE)
END SUBROUTINE CPUTQY
