SUBROUTINE CPUTQY_AROME ( KPROMA, KSTART, KPROF, KFLEV, KFLDN, KFLDX, PDT,&
 !  Variables 2D Input
 & PTENDT, PTENDR, PTENDU, PTENDV, PTENDD, PTENDTKE,PTENDEXT,&
 !  Variables 2D In/Out
 & PTT1, PQV1, PQC1, PQI1,PQR1,PQS1,PQG1,PQH1,PTKE1,PEXT1,&
 & PUT1, PVT1, PVWVT1)  

!     --------------------------------------------
!**** *CPUTQY_AROME*  EVOLUTION DE T,  et des  QI et de la TKE
!     --------------------------------------------

!     ARGUMENTS D'ENTREE
!     ------------------
!       KPROMA : DIMENSION HORIZONTALE
!       KSTART : DEBUT DE LA BOUCLE HORIZONTALE
!       KPROF : BORNE HORIZONTALE DES CALCULS
!       KFLEV : DIMENSION ET BORNE VERTICALE
!       KFLDN, KFLDX : SECONDE DIMENSION de PEXT1 (/= selon SL ou pas)
!       PDT : PAS DE TEMPS EFFECTIF

!       PTTS (KPROMA,KFLEV) : TENDANCE DE TEMPERATURE 
!       PRS ( KPROMA,KFLEV,NRR ) : TENDANCE DES Qi

!     ARGUMENTS IMPLICITES
!     --------------------
!       CONSTANTS = COMMON /YOMARAR/: NRR

!     SORTIES
!     -------
!       PTT1(NPROMA,KFLEV): VARIABLES A FAIRE EVOLUER
!       PNEBH (NVCLIN_AROME): tableau contenant les Qi  faire voluer
!       PQV1 : Qv au temps 1
!       PQL1 : Ql au temps 1
!       PQI1 : Qi au temps 1

!     Modifications
!     -------------
!   03/03/03 Creation par Y. Seity et S. Malardel
!   04/07/27 Modif par G. Hello (to run Hydrostatic case)
!   05/09/28 Modif par Y. Seity (Add PTENDEXT)
!   25-Jan-08 K. Yessad: bug correction for EXT.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!      07/12/18 Modif par Y. Seity (Add Hail PQH1)
! ----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMARAR   ,ONLY : NRR
USE YOMCT0   , ONLY : LNHDYN
USE YOM_YGFL , ONLY : NGFL_EXT,YH

! ----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDR(KPROMA,KFLEV,NRR) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDU(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDD(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDTKE(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDEXT(KPROMA,KFLEV,NGFL_EXT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQV1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQC1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQI1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQR1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQG1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQH1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEXT1(KPROMA,KFLDN:KFLDX,NGFL_EXT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVWVT1(KPROMA,KFLEV) 

! ----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF, JEXT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPUTQY_AROME',0,ZHOOK_HANDLE)

! ----------------------------------------------------------------------------

!  ------------------------------------------------------------
!   1. CALCUL DE L'EVOLUTION DE LA TEMPERATURE DU VENT ET TKE
!  ------------------------------------------------------------

DO JLEV = 1, KFLEV
  DO JROF = KSTART, KPROF
    PTT1(JROF,JLEV) = PTT1(JROF,JLEV)+PTENDT(JROF,JLEV)*PDT 
    PUT1(JROF,JLEV) = PUT1(JROF,JLEV)+PTENDU(JROF,JLEV)*PDT
    PVT1(JROF,JLEV) = PVT1(JROF,JLEV)+PTENDV(JROF,JLEV)*PDT
    PTKE1(JROF,JLEV) = PTKE1(JROF,JLEV)+PTENDTKE(JROF,JLEV)*PDT
  ENDDO
ENDDO

!  ------------------------------------------------------------
!   2. CALCUL DE L'EVOLUTION DES EXTRA-GFL 
!  ------------------------------------------------------------
DO JEXT=1,NGFL_EXT
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF
      PEXT1(JROF,JLEV,JEXT)=PEXT1(JROF,JLEV,JEXT)+PTENDEXT(JROF,JLEV,JEXT)*PDT
    ENDDO
  ENDDO
ENDDO

!  ------------------------------------------------------------
!   3. CALCUL DE L'EVOLUTION DE w
!  ------------------------------------------------------------
IF(LNHDYN) THEN
  DO JLEV=1, KFLEV
    DO JROF=KSTART,KPROF
      PVWVT1(JROF,JLEV) = PVWVT1(JROF,JLEV)+PTENDD(JROF,JLEV)*PDT
    ENDDO
  ENDDO
ENDIF
     
!  ------------------------------------------------------------
!   4. CALCUL DE L'EVOLUTION DES HYDROMETEORES
!  ------------------------------------------------------------
DO JLEV = 1, KFLEV
  DO JROF = KSTART, KPROF

    PQV1(JROF,JLEV)= PQV1(JROF,JLEV)+ PTENDR(JROF,JLEV,1)*PDT
    PQC1(JROF,JLEV)=PQC1(JROF,JLEV)+ PTENDR(JROF,JLEV,2)*PDT
    PQR1(JROF,JLEV)=PQR1(JROF,JLEV)+ PTENDR(JROF,JLEV,3)*PDT
    PQI1(JROF,JLEV)= PQI1(JROF,JLEV)+ PTENDR(JROF,JLEV,4)*PDT
    PQS1(JROF,JLEV)= PQS1(JROF,JLEV)+ PTENDR(JROF,JLEV,5)*PDT
    PQG1(JROF,JLEV)= PQG1(JROF,JLEV)+PTENDR(JROF,JLEV,6)*PDT

  ENDDO
ENDDO
IF (YH%LACTIVE) THEN
  DO JLEV = 1, KFLEV
    DO JROF = KSTART, KPROF
      PQH1(JROF,JLEV)= PQH1(JROF,JLEV)+PTENDR(JROF,JLEV,7)*PDT
    ENDDO
  ENDDO
ENDIF
! ----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPUTQY_AROME',1,ZHOOK_HANDLE)
END SUBROUTINE CPUTQY_AROME
