SUBROUTINE CTVTOT(KSTART,KPROF,PT,PGFL)

!**** *CTVTOT* - Computes T from 'TV' (TV defined as RT/Rd)

!     Purpose.
!     --------
!           COMPUTES T FROM 'TV'

!**   Interface.
!     ----------
!        *CALL* *CTVTOT(...)

!        Explicit arguments :
!        --------------------
!                              PT(NPROMA,NFLEVG)    - output: T
!                                                  - input: TV
!                              PQ(NPROMA,NFLEVG)    - Q

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Clive Temperton  *ECMWF*

!     Modifications.
!     --------------
!        Original : 94-05-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified 04-02-06: Y. Seity : new arguments to GPRCP for AROME
!        M.Hamrud  15-Jan-2006  Revised GPRCP
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMCST   , ONLY : RD
USE YOM_YGFL , ONLY : YGFL

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZR(NPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: IEND, IST, JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gprcp.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CTVTOT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTES T FROM 'TV'
!              --------------------

IST =KSTART
IEND=KPROF

CALL GPRCP(NPROMA,IST,IEND,NFLEVG,PR=ZR,PGFL=PGFL)

DO JLEV = 1, NFLEVG
  DO JROF = IST, IEND
    PT(JROF,JLEV)=RD*PT(JROF,JLEV)/ZR(JROF,JLEV)
  ENDDO
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CTVTOT',1,ZHOOK_HANDLE)
END SUBROUTINE CTVTOT
