SUBROUTINE ESTRCPL(KST,KEND,KSTC,KENDC,KSTGLO,PE3Y)

! ESTRCPL - Copy of stranger "Y" - lagged part if SL scheme

! Purpose
! -------
!   The "Y" term is ready in CPGLAG if SL, copy of it to GPPC bufr

! Interface
! ---------
!   KST    : start of work
!   KEND   : end of work
!   KSTC   : starting index of the buffer
!   KENDC  : ending index of the buffer
!   KSTGLO : global offset in message passing v.
!   PE3Y   : stranger Y

! Externals
! ---------
!   SC2RDG
!   SC2WRG

! Method
! ------
!   Read GPPC buffer, replace Y-term by PE3Y and write out buffer

! Reference
! ---------

! Author
! ------
!      Mar-1996 R. BUBNOVA, M. JANOUSEK (CNRM/GMAP/EXT)

! Modifications
! -------------
!      May-1997 R. Bubnova    adaptation to lmessp=.T.
!      Jun-1998 P. Le Moigne  call to sc2rdg and sc2wrg
!      Oct-2000 C. Fischer    gridpoint_buffer type for CY23
!      Mar-2002 J. Vivoda     PC schemes for NH dynamics (LPC_XXX keys)
!   06-Dec-2001 R. El Khatib  Cleaning sm variables
!   01-Oct-2003 M. Hamrud     CY28 Cleaning
!   10-Jun-2004 J. Masek      NH cleaning (LPC_NOTR)
!   06-Jul-2004 K. Yessad     Use the content of PTRGPPC, now computed in setup
!   07-Oct-2004 A. Bogatchev  Cleaning interfaces
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LNHDYN   ,LPC_OLD
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NFGPPC
USE YOMGPPCB  , ONLY : GPPCBUF
USE PTRGPPC  , ONLY : MGPPCO_Y
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!------------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTC
INTEGER(KIND=JPIM),INTENT(IN)    :: KENDC
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO
REAL(KIND=JPRB)   ,INTENT(IN)    :: PE3Y(NPROMA,NFLEVG) 

!------------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZGPPC(NPROMA,NFGPPC)

!------------------------------------------------------------------------------

#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ESTRCPL',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

! 1. Read in GPPC buffer

IF ( LPC_OLD.AND.LNHDYN ) THEN
  CALL SC2RDG(GPPCBUF,KSTGLO,KSTC,KENDC,ZGPPC)
ELSE
  ZGPPC = 0.0_JPRB 
ENDIF

! 2. Replace relevant part of GPPC by PE3Y

IF ( LPC_OLD.AND.LNHDYN ) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      ZGPPC(JROF,JLEV+MGPPCO_Y-1)=PE3Y(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

! 3. Write out GPPC

CALL SC2WRG(GPPCBUF,KSTGLO,KSTC,KENDC,ZGPPC)

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ESTRCPL',1,ZHOOK_HANDLE)

END SUBROUTINE ESTRCPL
