SUBROUTINE GNHY( &
 ! --- INPUT -----------------------------------------------------------------
 & KFLEV,KPROMA,KSTART,KEND, &
 & PNHX,PDVER,PR, &
 ! --- OUTPUT ----------------------------------------------------------------
 & PNHY &
 & )

! GNHY - Diagnose Y-term of the option LPC_OLD.

! Purpose
! -------
!   Diagnose Y-term of the option LPC_OLD.
!    (NPDVAR=2, NVDVAR=3): Y = X + dver ( (Rd/R) -1 )
!    (NPDVAR=2, NVDVAR=4): Y = X

!   This routine must not be called if LPC_OLD=.F.

! Interface
! ---------
!   * INPUT:
!   KFLEV   - number of levels.
!   KPROMA  - length of work.
!   KSTART  - start of work.
!   KEND    - end of work.
!   PNHX    - X-term at full levels.
!   PDVER   - vertical divergence at full levels.
!   PR      - R at full levels.

!   * OUTPUT:
!   PNHY    - Y-term at full levels.

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   06 Dec 2004 K. Yessad (after GNHPDVD).

! Modifications
! -------------
! End Modifications
!---------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCST   , ONLY : RD
USE YOMDYNA  , ONLY : NVDVAR   ,NPDVAR
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHX(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHY(KPROMA,KFLEV) 

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZHOOK_HANDLE

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHY',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

IF (NPDVAR == 2) THEN

  IF (NVDVAR == 3) THEN
    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        PNHY(JROF,JLEV)=PNHX(JROF,JLEV) &
         & +PDVER(JROF,JLEV)*(RD/PR(JROF,JLEV) - 1.0_JPRB)
      ENDDO
    ENDDO
  ELSEIF (NVDVAR == 4) THEN
    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        PNHY(JROF,JLEV)=PNHX(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHY',1,ZHOOK_HANDLE)

END SUBROUTINE GNHY

