SUBROUTINE GPADDSLPHY(LDLFSTEP,KST,KEND,PTMST, &
 & PSAVTEND,PGFLSLP, &
 & PSLB1U0,PSLB1V0,PSLB1T0,PSLB1GFL0)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOEPHY   , ONLY : LEO3CH
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NFLSA    ,NFLEN
USE YOMSLPHY , ONLY : NVTEND,MU_SAVTEND,MV_SAVTEND,MT_SAVTEND
USE YOM_YGFL , ONLY : YGFL     ,YGFLC ,YL ,YI ,YR ,YS ,YO3

!**** *GPADDSLPHY - add physics tendencies to RHS of semi-lagrangian buffers

!     Purpose. add physics tendencies to RHS 
!     -------- 

!**   Interface.
!     ----------
!        *CALL* *GPADDSLPHY(...)*

!        Explicit arguments : see CPG
!        -------------------- 

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        None

!     Author.
!     -------
!        Nils Wedi  *ECMWF*

!     Modifications.
!     --------------
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDLFSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSAVTEND(NPROMA,NFLEVG,NVTEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLSLP(NPROMA,NFLEVG,YGFL%NDIMSLP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSLB1U0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSLB1V0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSLB1T0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSLB1GFL0(NPROMA,NFLSA:NFLEN,YGFL%NUMFLDS) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF, JGFL, IGFL 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPADDSLPHY',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF(.NOT.LDLFSTEP) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PSLB1U0(JROF,JLEV)=PSLB1U0(JROF,JLEV)+PSAVTEND(JROF,JLEV,MU_SAVTEND)*PTMST
      PSLB1V0(JROF,JLEV)=PSLB1V0(JROF,JLEV)+PSAVTEND(JROF,JLEV,MV_SAVTEND)*PTMST
      PSLB1T0(JROF,JLEV)=PSLB1T0(JROF,JLEV)+PSAVTEND(JROF,JLEV,MT_SAVTEND)*PTMST
    ENDDO
  ENDDO

  DO JGFL=1,YGFL%NUMFLDS

    ! IGFL should be consistent with what is done in LARCINB and SUSLB.
    ! For the time being: IGFL=JGFL (fields in interpolation buffer counted
    !  even if useless).
    IGFL=JGFL

    IF(LEO3CH .OR. YGFLC(JGFL)%MPSLP /= YO3%MPSLP)THEN
      IF(YGFLC(JGFL)%LPHY .AND. YGFLC(JGFL)%LADV) THEN
        ! PTENL and PTENI contain the total amount of liquid water and ice
        IF (YGFLC(JGFL)%MPSLP == YL%MPSLP .OR. &
          & YGFLC(JGFL)%MPSLP == YI%MPSLP .OR. &
          & YGFLC(JGFL)%MPSLP == YR%MPSLP .OR. &
          & YGFLC(JGFL)%MPSLP == YS%MPSLP ) THEN
          DO JLEV=1,NFLEVG
            DO JROF=KST,KEND
              PSLB1GFL0(JROF,JLEV,IGFL)=PSLB1GFL0(JROF,JLEV,IGFL)+ &
               & PGFLSLP(JROF,JLEV,YGFLC(JGFL)%MPSLP)  
            ENDDO
          ENDDO
        ELSE
          DO JLEV=1,NFLEVG
            DO JROF=KST,KEND
              PSLB1GFL0(JROF,JLEV,IGFL)=PSLB1GFL0(JROF,JLEV,IGFL)+ &
               & PGFLSLP(JROF,JLEV,YGFLC(JGFL)%MPSLP)*PTMST  
            ENDDO
          ENDDO
        ENDIF
      ENDIF
    ENDIF

  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPADDSLPHY',1,ZHOOK_HANDLE)
END SUBROUTINE GPADDSLPHY
