SUBROUTINE GPHLWITL(KPROMA,KSTART,KPROF,&
 & PLNPR5,PALPH5,PLNPR0,PALPH0,PWW0)

!**** *GPHLWITL* - to half-levels interpolation weights for
!                  tangent linear model

!     Purpose.
!     --------
!           Compute weights for interpolation of winds to
!           half levels (in non-hydrostatic dynamics)

!**   Interface.
!     ----------
!        *CALL* *GPHLWITL()

!        Explicit arguments :
!        --------------------
!         See direct code.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        Interpolation from full-levels using logaritmic pressure profile
!        Then modify values on the top and bottom using boundary condition
!        (at the bottom free-slip condition)
!        Store the weights of vertical interpolation

!     Externals.
!     ----------
!     Reference.
!     ----------
!        ARPEGE/ALADIN documentation

!     Author.
!     -------
!        Martin Charron,  CNRM/GMAP - visiteur

!     Modifications.
!     --------------
!        Original : September 1998
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad      20-Mar-2006 Cleaning + optimisation
!        K. Yessad (Dec 2008): remove dummy CDLOCK
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWW0(KPROMA,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPHLWITL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

DO JLEV=1, NFLEVG - 1
  DO JROF=KSTART,KPROF
    PWW0(JROF,JLEV)= 1.0_JPRB/(PLNPR5(JROF,JLEV+1)-PALPH5(JROF,JLEV+1)&
     & +PALPH5(JROF,JLEV))**2 * ((PLNPR0(JROF,JLEV+1)-PALPH0 &
     & (JROF,JLEV+1)) *  (PLNPR5(JROF,JLEV+1)-PALPH5(JROF,&
     & JLEV+1)+PALPH5(JROF,JLEV) ) - (PLNPR5(JROF,JLEV+1)-&
     & PALPH5(JROF,JLEV+1)) *  (PLNPR0(JROF,JLEV+1)-PALPH0 &
     & (JROF,JLEV+1)+PALPH0(JROF,JLEV)) )  
  ENDDO
ENDDO

!     ------------------------------------------------------------------

DO JROF=KSTART,KPROF
  !* Bottom half level: 
  PWW0(JROF,NFLEVG) = 0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPHLWITL',1,ZHOOK_HANDLE)
END SUBROUTINE GPHLWITL
