SUBROUTINE GPINISLBAD(KST,KEN,PTE, &
 & PGFL, &
 & PUT9    ,PVT9     ,PTT9, &
 & PSPDT9  ,PSVDT9   ,PSPT9, &
 & PB2U1   ,PB2V1    ,PB2T1  ,PGFLT1, &
 & PB2PD1  ,PB2VD1   ,PB2SP1 , &
 & PQICE   ,PQLI)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL   ,LNHDYN
USE YOM_YGFL , ONLY : YGFL     ,YGFLC    ,YL       ,YI

!**** *GPINISLBAD* - 

!     Purpose.
!     --------
!      Adjoint code of GPINISLB.

!**   Interface.
!     ----------
!        *CALL* *GPINISLBAD(...)*

!        Explicit arguments :
!        --------------------

!        See direct code (GPINISLB) for comments about dummy arguments

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
       
!     Author.
!     -------
!     C. Fischer : 02-06-28  * METEO-FRANCE *

!     Modifications.
!     --------------
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   K.Yessad      05-Jul-2004 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTE 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSVDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2U1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2V1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2T1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2PD1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2VD1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2SP1(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQICE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLI(NPROMA,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF, JGFL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPINISLBAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*          1.     PB2[X]1 for prognostic variables.
!                  --------------------------------- 

!*          1.1    PB2[X]1 for GMV variables (U,V,T,NH variables).

IF (LSLAG) THEN
  PB2U1(KST:KEN,1:NFLEVG) = 0.0_JPRB
  PB2V1(KST:KEN,1:NFLEVG) = 0.0_JPRB
  PB2T1(KST:KEN,1:NFLEVG) = 0.0_JPRB
  IF(LNHDYN) THEN
    ! ky: AD of NH not yet completed (missing NHX here).
    PB2PD1(KST:KEN,1:NFLEVG) = 0.0_JPRB
    PB2VD1(KST:KEN,1:NFLEVG) = 0.0_JPRB
  ENDIF
ELSE !Eulerian
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEN
      PUT9(JROF,JLEV)  = PUT9(JROF,JLEV)+PB2U1(JROF,JLEV)*PTE
      PB2U1(JROF,JLEV) = 0.0_JPRB
      PVT9(JROF,JLEV)  = PVT9(JROF,JLEV)+PB2V1(JROF,JLEV)*PTE
      PB2V1(JROF,JLEV) = 0.0_JPRB
      PTT9(JROF,JLEV)  = PTT9(JROF,JLEV)+PB2T1(JROF,JLEV)*PTE
      PB2T1(JROF,JLEV) = 0.0_JPRB
    ENDDO
  ENDDO
  IF(LNHDYN) THEN
    ! ky: AD of NH not yet completed (missing NHX here).
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEN
        PSPDT9(JROF,JLEV) = PSPDT9(JROF,JLEV)+PB2PD1(JROF,JLEV)*PTE
        PB2PD1(JROF,JLEV) = 0.0_JPRB
        PSVDT9(JROF,JLEV) = PSVDT9(JROF,JLEV)+PB2VD1(JROF,JLEV)*PTE
        PB2VD1(JROF,JLEV) = 0.0_JPRB
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*          1.2    PB2[X]1 for GFL variables, and also PQLI, PQICE.

IF(YL%LACTIVE) THEN
  IF (LSLAG.AND.LTWOTL) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEN
        PGFL(JROF,JLEV,YL%MP) = PGFL(JROF,JLEV,YL%MP)+PQLI(JROF,JLEV)
        PQLI(JROF,JLEV) = 0.0_JPRB
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEN
        PGFL(JROF,JLEV,YL%MP9) = PGFL(JROF,JLEV,YL%MP9)+PQLI(JROF,JLEV)
        PQLI(JROF,JLEV) = 0.0_JPRB
      ENDDO
    ENDDO
  ENDIF
ELSE
  PQLI(KST:KEN,1:NFLEVG) = 0.0_JPRB
ENDIF
IF(YI%LACTIVE) THEN
  IF (LSLAG.AND.LTWOTL) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEN
        PGFL(JROF,JLEV,YI%MP) = PGFL(JROF,JLEV,YI%MP)+PQICE(JROF,JLEV)
        PQICE(JROF,JLEV) = 0.0_JPRB
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEN
        PGFL(JROF,JLEV,YI%MP9) = PGFL(JROF,JLEV,YI%MP9)+PQICE(JROF,JLEV)
        PQICE(JROF,JLEV) = 0.0_JPRB
      ENDDO
    ENDDO
  ENDIF
ELSE
  PQICE(KST:KEN,1:NFLEVG) = 0.0_JPRB
ENDIF

IF (LSLAG.AND.LTWOTL) THEN
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LT1) THEN
      PGFLT1(KST:KEN,1:NFLEVG,YGFLC(JGFL)%MP1)=0.0_JPRB
    ENDIF
  ENDDO
ELSEIF(LSLAG.AND.(.NOT.LTWOTL)) THEN
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LADV) THEN
      PGFLT1(KST:KEN,1:NFLEVG,YGFLC(JGFL)%MP1) = 0.0_JPRB
    ELSEIF(YGFLC(JGFL)%LT1) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEN
          PGFL(JROF,JLEV,YGFLC(JGFL)%MP) = PGFL(JROF,JLEV,YGFLC(JGFL)%MP)+ &
           & PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)*PTE  
          PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1) = 0.0_JPRB
        ENDDO
      ENDDO
    ENDIF
  ENDDO
ELSE !Eulerian
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LT1) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEN
          PGFL(JROF,JLEV,YGFLC(JGFL)%MP9) = PGFL(JROF,JLEV,YGFLC(JGFL)%MP9) + &
           & PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)*PTE  
          PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1) = 0.0_JPRB
        ENDDO
      ENDDO
    ENDIF
  ENDDO
ENDIF

!*          1.3    PB2[X]1 for continuity equation (GMVS).

IF (LSLAG) THEN
  PB2SP1(KST:KEN) = 0.0_JPRB
ELSE
  DO JROF=KST,KEN
    PSPT9(JROF)  = PSPT9(JROF)+PB2SP1(JROF)*PTE
    PB2SP1(JROF) = 0.0_JPRB
  ENDDO
ENDIF

!*          2.     PB2[X] for other purposes.
!                  -------------------------- 

! Nothing for the time being.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPINISLBAD',1,ZHOOK_HANDLE)
END SUBROUTINE GPINISLBAD
