SUBROUTINE GPINO3CH(KSTART,KPROF,KPROMA,KGPLAT,PKOZO)

!**** *GPINO3CH* - Ozone chemistry.
 
!     Purpose.  
!     --------
!      Memory transfer: fills PKOZO with the content of buffer TOZ2DG.

!**   Interface.
!     ----------
!        *CALL* *GPINO3CH(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!         KSTART       : start of work
!         KPROF        : depth of work
!         KPROMA       : horizontal dimension
!         KGPLAT       : DM-global number of the latitude of point jrof=KSTART

!        OUTPUT:
!         PKOZO        : fields for photochemistery of ozone

!        Implicit arguments :
!        --------------------
  
!     Method.
!     -------
!        See documentation

!     Externals.  None. 
!     ----------
!      Called by EC_PHYS.
    
!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        ????

! Modifications.
! --------------
!   Original : ????
!   K. Yessad (Sep 2008): add missing comments, cleanings.
! End Modifications
!------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   
USE YOMDPHY  , ONLY : NVCLIS   ,NTOZ2D   ,NTOZ3D
USE YOMOZO   , ONLY : TOZ2DG   

! -------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPLAT(KPROMA)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKOZO(KPROMA,NFLEVG*NVCLIS+1) 

! -------------------------------------------------------------------------

INTEGER(KIND=JPIM) ::  IGL, JJ, JVAR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! -------------------------------------------------------------------------

#include "abor1.intfb.h"

! -------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPINO3CH',0,ZHOOK_HANDLE)

! -------------------------------------------------------------------------

IF(NTOZ3D > 0) then
  CALL ABOR1('GPINIO3CH : NTOZ3D > 0 BROKEN WITH MESSAGE PASSING')
ENDIF

DO JVAR=1,NTOZ2D*NVCLIS*NFLEVG
  DO JJ=KSTART,KPROF
    IGL=KGPLAT(JJ)
    PKOZO(JJ,JVAR)=TOZ2DG(JVAR,IGL)
  ENDDO
ENDDO

! -------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPINO3CH',1,ZHOOK_HANDLE)
END SUBROUTINE GPINO3CH
