SUBROUTINE GPMKTENDAD(KST,KEND,PTSPHY,&
 & PUT1,PVT1 ,PTT1,&
 & PGFLT1,&
 & PUT9,PVT9 ,PTT9,&
 & PGFL)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOM_YGFL , ONLY : YGFL     ,YGFLC

!**** *GPMKTENDAD - Convert t+dt values to tendencies - adjoint

!     Purpose. Convert t+dt values to tendencies before calling
!     -------- ECMWF physics

!**   Interface.
!     ----------
!        *CALL* *GPMKTENDAD(...)*

!        Explicit arguments : see CPGLAG, all names the same
!        -------------------- except for DOCTOR norm modifications

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        None

!     Author.
!     -------
!        J.F. Mahfouf  *ECMWF*

!     Modifications.
!     --------------
!      Original : 96-01-08 (From GPMKTEND)
!      Modified 98-01-09 by E. Holm : Introduced ozone
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      K. Yessad (Dec 2008): remove dummy CDLOCK
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF, JGFL
REAL(KIND=JPRB) :: ZTS_R
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPMKTENDAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

ZTS_R=1.0_JPRB/PTSPHY
DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    PUT9(JROF,JLEV)=PUT9(JROF,JLEV)-PUT1(JROF,JLEV)*ZTS_R
    PUT1(JROF,JLEV)=PUT1(JROF,JLEV)*ZTS_R
    PVT9(JROF,JLEV)=PVT9(JROF,JLEV)-PVT1(JROF,JLEV)*ZTS_R
    PVT1(JROF,JLEV)=PVT1(JROF,JLEV)*ZTS_R
    PTT9(JROF,JLEV)=PTT9(JROF,JLEV)-PTT1(JROF,JLEV)*ZTS_R
    PTT1(JROF,JLEV)=PTT1(JROF,JLEV)*ZTS_R
  ENDDO
ENDDO

DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LT1) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        PGFL(JROF,JLEV,YGFLC(JGFL)%MP9)=PGFL(JROF,JLEV,YGFLC(JGFL)%MP9)-&
         & PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)*ZTS_R  
        PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)=PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)*ZTS_R
      ENDDO
    ENDDO
  ENDIF
ENDDO
        
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPMKTENDAD',1,ZHOOK_HANDLE)
END SUBROUTINE GPMKTENDAD
