SUBROUTINE GPPREHTL(KPROMA,KSTART,KPROF,KFLEV,PVBH,PRESH)

!**** *GPPREHTL* - Computes half level pressure, tangent linear

!     Purpose.
!     --------
!           Computes pressures at half model levels - T.L.

!**   Interface.
!     ----------
!        *CALL* *GPPREHTL(KPROMA,KSTART,KPROF,KFLEV,PVBH,PRESH)

!        Explicit arguments :
!        --------------------
!                              KPROMA    : DIMENSIONING
!                              KSTART    : START OF THE PACKET.
!                              KPROF     : DEPTH OF WORK
!                              KFLEV     : VERT. DIMENSIONING
!                              PRESH(KPROMA,0:KFLEV) - HALF LEVEL PRESSURE
!                              PVBH(KFLEV)- VERTICAL COORDINATE

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------
!        See documentation
!     Half level P:     PHk = Ak + Bk * Ps

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Erik Andersson and Jean-Noel THEPAUT

!     Modifications.
!     --------------
!        Original : 92-11-23
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH(0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KPROMA,0:KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPPREHTL',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    COMPUTES HALF LEVEL PRESSURES.
!              ------------------------------

DO JLEV=0,KFLEV-1
  DO JLON=KSTART,KPROF
    PRESH(JLON,JLEV)=PVBH(JLEV)*PRESH(JLON,KFLEV)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPREHTL',1,ZHOOK_HANDLE)
END SUBROUTINE GPPREHTL

