SUBROUTINE GPPWCTL(KPROMA,KSTART,KPROF,KFLEV,PWCH,PQ,PRXP,PQ5,PRXP5)

!**** *GPPWCTL* - Computes half level PWC.

!     Purpose.
!     --------
!           Computes half level PWC, tangent linear.

!**   Interface.
!     ----------
!        *CALL* *GPPWCTL(KPROMA,KSTART,KPROF,KFLEV,PWCH,PQ,PRXP
!                                              ,PQ5,PRXP5)

!        Explicit arguments :
!        --------------------
!        KPROMA                      - HORIZ. DIMENSIONING
!        KSTART                      - START OF WORK
!        KPROF                       - DEPTH OF WORK
!        KFLEV                       - NUMBER OF LEVELS
!        PWCH(KPROMA,0:KFLEV)        - HALF LEVEL PWC             (OUTPUT)
!        PQ  (KPROMA,KFLEV)          - HUMIDITY                   (INPUT)
!        PRXP(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)
!        PQ5 (KPROMA,KFLEV)         - HUMIDITY                   (INPUT-T)
!        PRXP5(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Method.
!     -------
!        See documentation.

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Erik Andersson, based on PP-routines by
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-06-08
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWCH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZDELP, ZDELP5, ZRGI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPPWCTL',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    COMPUTES HALF LEVEL PWC
!              -----------------------

ZRGI=1.0_JPRB/RG
DO JL=KSTART,KPROF
  PWCH(JL,0)=0.0_JPRB
ENDDO
DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    ZDELP5=PRXP5(JL,JLEV,1)-PRXP5(JL,JLEV-1,1)
    ZDELP=PRXP(JL,JLEV,1)-PRXP(JL,JLEV-1,1)
    PWCH(JL,JLEV)=PWCH(JL,JLEV-1)&
     & + ( PQ5(JL,JLEV)*ZDELP &
     & +PQ(JL,JLEV)*ZDELP5 )*ZRGI  
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPWCTL',1,ZHOOK_HANDLE)
END SUBROUTINE GPPWCTL
