SUBROUTINE GPRCP(KPROMA,KSTART,KPROF,KFLEV,PQ,PQI,PQL,PQR,PQS,PQG,PCP,PR,PKAP,&
 & PGFL,KGFLTYP)

!**** *GPRCP* - Computes Cp, R and R/Cp from Q

!     Purpose.
!     --------
!           COMPUTES CP AND R  AND R/CP FROM Q

!**   Interface.
!     ----------
!        *CALL* *GPRCP(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA               - dimensioning.
!          KSTART               - start of work.
!          KPROF                - depth of work.
!          KFLEV                - number of layers.
!          PQ(KPROMA,KFLEV)     - specific humidity.
!          PQI(KPROMA,KFLEV)    - ice.
!          PQL(KPROMA,KFLEV)    - liquid water.
!          PQR(KPROMA,KFLEV)    - rain.
!          PQS(KPROMA,KFLEV)    - snow.
!          PQG(KPROMA,KFLEV)    - graupel.

!        OUTPUT:
!          PCP(KPROMA,KFLEV)    - CP
!          PR(KPROMA,KFLEV)     - R
!          PKAP(KPROMA,KFLEV)   - KAPPA

!        Implicit arguments :  Physical constants from YOMCST
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-02-04
!        Modified by E.Bazile 94-05-13 (Liquid Water and Ice)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified by Y.Seity  04-02-13 (Rain, Snow and Graupel)
!        M.Hamrud  15-Jan-2006  Revised GPRCP
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD       ,RV       ,RCPD     ,RCPV     ,RCW      ,RCS
USE YOM_YGFL , ONLY : YGFL, YGFLC, YQ, YL, YI, YR, YS, YG

!     ------------------------------------------------------------------


IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PQ(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PQI(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PQL(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PQR(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PQS(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PQG(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(IN)    :: PGFL(KPROMA,KFLEV,YGFL%NDIM) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(OUT)   :: PCP(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(OUT)   :: PR(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL   ,INTENT(OUT)   :: PKAP(KPROMA,KFLEV) 
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN)    :: KGFLTYP

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JLON, JGFL
INTEGER(KIND=JPIM) :: IACT(YGFL%NUMFLDS),IPT(YGFL%NUMFLDS),INUMACT,IGFL,IGFLTYP
LOGICAL :: LLGFL,LLQ,LLQL,LLQI,LLQR,LLQS,LLQG,LLKAP,LLR,LLCP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPRCP',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTES R AND CP AND KAPPA.
!              ----------------------------
LLGFL = PRESENT(PGFL)
IF(.NOT. LLGFL) THEN
  LLQ  = PRESENT(PQ)
  LLQL = PRESENT(PQL)
  LLQI = PRESENT(PQI)
  LLQR = PRESENT(PQR)
  LLQS = PRESENT(PQS)
  LLQG = PRESENT(PQG)
ENDIF
LLR   = PRESENT(PR)
LLCP  = PRESENT(PCP)
LLKAP = PRESENT(PKAP)
IF(LLKAP) THEN
  IF(.NOT. LLR)  CALL ABOR1('GPRCP:PKAP PRESENT REQUIRES PR PRESENT')
  IF(.NOT. LLCP) CALL ABOR1('GPRCP:PKAP PRESENT REQUIRES PCP PRESENT')
ENDIF

IF(LLGFL) THEN
  IGFLTYP = 0
  IF(PRESENT(KGFLTYP)) IGFLTYP=KGFLTYP
  INUMACT = 0
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LTHERMACT) THEN
      INUMACT = INUMACT+1
      IACT(INUMACT) = JGFL
      IF(IGFLTYP == 0) THEN
        IPT(INUMACT) = YGFLC(JGFL)%MP
      ELSEIF(IGFLTYP == 1) THEN
        IPT(INUMACT) = YGFLC(JGFL)%MP1
      ELSEIF(IGFLTYP == 5) THEN
        IPT(INUMACT) = YGFLC(JGFL)%MP5
      ELSEIF(IGFLTYP == 9) THEN
        IPT(INUMACT) = YGFLC(JGFL)%MP9_PH
      ELSEIF(IGFLTYP == 101) THEN
        IPT(INUMACT) = YGFLC(JGFL)%MP_SL1
      ELSE
        CALL ABOR1('GPRCP:UNKNOWN GFL TYPE')
      ENDIF
    ENDIF
  ENDDO
ENDIF

IF(LLGFL) THEN
  IF(LLR .AND. LLCP) THEN
    DO JLEV=1,KFLEV
      DO JGFL=1,INUMACT
        IGFL = IACT(JGFL)
        IF(JGFL == 1) THEN
          DO JLON=KSTART,KPROF
            PR(JLON,JLEV) = &
             & (YGFLC(IGFL)%R-RD)*PGFL(JLON,JLEV,IPT(JGFL))
            PCP(JLON,JLEV) = &
             & (YGFLC(IGFL)%RCP-RCPD)*PGFL(JLON,JLEV,IPT(JGFL))
          ENDDO
        ELSE
          DO JLON=KSTART,KPROF
            PR(JLON,JLEV) = PR(JLON,JLEV)+&
             & (YGFLC(IGFL)%R-RD)*PGFL(JLON,JLEV,IPT(JGFL))
            PCP(JLON,JLEV) = PCP(JLON,JLEV)+&
             & (YGFLC(IGFL)%RCP-RCPD)*PGFL(JLON,JLEV,IPT(JGFL))
          ENDDO
        ENDIF
      ENDDO
      IF(INUMACT == 0) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = RD
          PCP(JLON,JLEV) = RCPD
        ENDDO
      ELSE
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = RD+PR(JLON,JLEV)
          PCP(JLON,JLEV) = RCPD+PCP(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLKAP) THEN
        DO JLON=KSTART,KPROF
          PKAP(JLON,JLEV) = PR(JLON,JLEV)/PCP(JLON,JLEV)
        ENDDO
      ENDIF
    ENDDO
  ELSEIF(LLR) THEN
    DO JLEV=1,KFLEV
      DO JGFL=1,INUMACT
        IGFL = IACT(JGFL)
        IF(JGFL == 1) THEN
          DO JLON=KSTART,KPROF
            PR(JLON,JLEV) = &
             & (YGFLC(IGFL)%R-RD)*PGFL(JLON,JLEV,IPT(JGFL))
          ENDDO
        ELSE
          DO JLON=KSTART,KPROF
            PR(JLON,JLEV) = PR(JLON,JLEV)+&
             & (YGFLC(IGFL)%R-RD)*PGFL(JLON,JLEV,IPT(JGFL))
          ENDDO
        ENDIF
      ENDDO
      IF(INUMACT == 0) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = RD
        ENDDO
      ELSE
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = RD+PR(JLON,JLEV)
        ENDDO
      ENDIF
    ENDDO
  ELSEIF(LLCP) THEN
    DO JLEV=1,KFLEV
       DO JGFL=1,INUMACT
        IGFL = IACT(JGFL)
        IF(JGFL == 1) THEN
          DO JLON=KSTART,KPROF
            PCP(JLON,JLEV) = &
             & (YGFLC(IGFL)%RCP-RCPD)*PGFL(JLON,JLEV,IPT(JGFL))
          ENDDO
        ELSE
          DO JLON=KSTART,KPROF
            PCP(JLON,JLEV) = PCP(JLON,JLEV)+&
             & (YGFLC(IGFL)%RCP-RCPD)*PGFL(JLON,JLEV,IPT(JGFL))
          ENDDO
        ENDIF
      ENDDO
      IF(INUMACT == 0) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = RCPD
        ENDDO
      ELSE
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = RCPD+PCP(JLON,JLEV)
        ENDDO
      ENDIF
    ENDDO
  ENDIF
ELSE
  IF(LLR .AND. LLCP) THEN
    DO JLEV=1,KFLEV
      IF(LLQ .AND. YQ%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV)  = RD+(RV-RD)*PQ(JLON,JLEV)
          PCP(JLON,JLEV) = RCPD+(RCPV-RCPD)*PQ(JLON,JLEV)
        ENDDO
      ELSE
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV)  = RD
          PCP(JLON,JLEV) = RCPD
        ENDDO
      ENDIF
      IF(LLQL .AND. YL%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQL(JLON,JLEV)
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCW-RCPD)*PQL(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQI .AND. YI%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQI(JLON,JLEV)
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCS-RCPD)*PQI(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQR .AND. YR%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQR(JLON,JLEV)
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCW-RCPD)*PQR(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQS .AND. YS%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQS(JLON,JLEV)
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCS-RCPD)*PQS(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQG .AND. YG%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQG(JLON,JLEV)
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCS-RCPD)*PQG(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLKAP) THEN
        DO JLON=KSTART,KPROF
          PKAP(JLON,JLEV) = PR(JLON,JLEV)/PCP(JLON,JLEV)
        ENDDO
      ENDIF
    ENDDO
  ELSEIF(LLR) THEN
    DO JLEV=1,KFLEV
      IF(LLQ .AND. YQ%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = RD+(RV-RD)*PQ(JLON,JLEV)
        ENDDO
      ELSE
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = RD
        ENDDO
      ENDIF
      IF(LLQL .AND. YL%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQL(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQI .AND. YI%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQI(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQR .AND. YR%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQR(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQS .AND. YS%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQS(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQG .AND. YG%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PR(JLON,JLEV) = PR(JLON,JLEV)-RD*PQG(JLON,JLEV)
        ENDDO
      ENDIF
    ENDDO
  ELSEIF(LLCP) THEN
    DO JLEV=1,KFLEV
      IF(LLQ .AND. YQ%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = RCPD+(RCPV-RCPD)*PQ(JLON,JLEV)
        ENDDO
      ELSE
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = RCPD
        ENDDO
      ENDIF
      IF(LLQL .AND. YL%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCW-RCPD)*PQL(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQI .AND. YI%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCS-RCPD)*PQI(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQR .AND. YR%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCW-RCPD)*PQR(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQS .AND. YS%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCS-RCPD)*PQS(JLON,JLEV)
        ENDDO
      ENDIF
      IF(LLQG .AND. YG%LTHERMACT) THEN
        DO JLON=KSTART,KPROF
          PCP(JLON,JLEV) = PCP(JLON,JLEV)+(RCS-RCPD)*PQG(JLON,JLEV)
        ENDDO
      ENDIF
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPRCP',1,ZHOOK_HANDLE)
END SUBROUTINE GPRCP
