SUBROUTINE GPTF2AD(&
 ! --- INPUT ---------------------------------------------------------
 & KST,KEN,LDLSTEP,&
 ! --- INPUT-OUTPUT --------------------------------------------------
 & PGMV,PGMVS,PGFL)

!**** *GPTF2AD* - Timefilter part 2. adjoint

!     Purpose.
!     --------
!           Performs part 2 of the time-filtering.adjoint

!**   Interface.
!     ----------
!        *CALL* *GPTF2AD(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!         KST          : start of work
!         KEN          : depth of work
!         LDLSTEP      : check on the last time step.

!        INPUT/OUTPUT:
!         PGMV         : "t" and "t-dt" upper air GMV variables.
!         PGMVS        : "t" and "t-dt" surface GMV variables.
!         PGFL         : "t" and "t-dt" GFL variables.

!        Implicit arguments :  None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel Thepaut  *ECMWF*

! Modifications
! -------------
!   Original : 93-03-17
!   Modified : 96-02-20 L.Isaksen (Adapted for Message Passing)
!   Modified : 98-01-07 E.Holm (Included ozone)
!   Modified : 99-08-16 C.Temperton (removed redundant RKROMA)
!   Modified : 98-11-20 M.Charron (Included NH variables)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   09-Jun-2004 J. Masek   NH cleaning (LVSLWBC)
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (Dec 2008): remove dummy CDLOCK
! End Modifications
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LNHDYN
USE YOMDIM   , ONLY : NFLEVG   ,NFTHER   ,NPROMA
USE YOMDYN   , ONLY : REPS2    ,REPSM2
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOM_YGFL , ONLY : YGFL     ,YGFLC

USE YOMGMV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEN 
LOGICAL           ,INTENT(IN)    :: LDLSTEP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JK, JL,JGFL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!       ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTF2AD',0,ZHOOK_HANDLE)

!       ------------------------------------------------------------------

!*       1. PERFORM TIME FILTER (PART 2)
!           ----------------------------

IF(LDLSTEP) THEN
  DO JK=1,NFLEVG
    DO JL=KST,KEN
      PGMV(JL,JK,YT0%MU)  = PGMV(JL,JK,YT0%MU)+PGMV(JL,JK,YT9%MU)
      PGMV(JL,JK,YT9%MU)  = 0.0_JPRB
      PGMV(JL,JK,YT0%MV)  = PGMV(JL,JK,YT0%MV)+PGMV(JL,JK,YT9%MV)
      PGMV(JL,JK,YT9%MV)  = 0.0_JPRB
      PGMV(JL,JK,YT0%MDIV)= PGMV(JL,JK,YT0%MDIV)+PGMV(JL,JK,YT9%MDIV)
      PGMV(JL,JK,YT9%MDIV)= 0.0_JPRB
    ENDDO
    IF(NFTHER >= 1) THEN
      DO JL=KST,KEN
        PGMV(JL,JK,YT0%MT)  = PGMV(JL,JK,YT0%MT)+PGMV(JL,JK,YT9%MT)
        PGMV(JL,JK,YT9%MT)  = 0.0_JPRB
        PGMV(JL,JK,YT0%MTL) = PGMV(JL,JK,YT0%MTL)+PGMV(JL,JK,YT9%MTL)
        PGMV(JL,JK,YT9%MTL) = 0.0_JPRB
        PGMV(JL,JK,YT0%MTM) = PGMV(JL,JK,YT0%MTM)+PGMV(JL,JK,YT9%MTM)
        PGMV(JL,JK,YT9%MTM) = 0.0_JPRB
      ENDDO
    ENDIF
    IF(LNHDYN) THEN
      DO JL=KST,KEN
        PGMV(JL,JK,YT0%MSPD) = PGMV(JL,JK,YT0%MSPD) + PGMV(JL,JK,YT9%MSPD)
        PGMV(JL,JK,YT9%MSPD) = 0.0_JPRB
        PGMV(JL,JK,YT0%MSVD) = PGMV(JL,JK,YT0%MSVD) + PGMV(JL,JK,YT9%MSVD)
        PGMV(JL,JK,YT9%MSVD) = 0.0_JPRB
        PGMV(JL,JK,YT0%MSPDL) = PGMV(JL,JK,YT0%MSPDL) + PGMV(JL,JK,YT9%MSPDL)
        PGMV(JL,JK,YT9%MSPDL) = 0.0_JPRB
        PGMV(JL,JK,YT0%MSVDL) = PGMV(JL,JK,YT0%MSVDL) + PGMV(JL,JK,YT9%MSVDL)
        PGMV(JL,JK,YT9%MSVDL) = 0.0_JPRB
        PGMV(JL,JK,YT0%MSPDM) = PGMV(JL,JK,YT0%MSPDM) + PGMV(JL,JK,YT9%MSPDM)
        PGMV(JL,JK,YT9%MSPDM) = 0.0_JPRB
        PGMV(JL,JK,YT0%MSVDM) = PGMV(JL,JK,YT0%MSVDM) + PGMV(JL,JK,YT9%MSVDM)
        PGMV(JL,JK,YT9%MSVDM) = 0.0_JPRB
      ENDDO
    ENDIF
  ENDDO

  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LT9) THEN
      DO JK=1,NFLEVG
        DO JL=KST,KEN
          PGFL(JL,JK,YGFLC(JGFL)%MP) = PGFL(JL,JK,YGFLC(JGFL)%MP)+ &
           & PGFL(JL,JK,YGFLC(JGFL)%MP9)  
          PGFL(JL,JK,YGFLC(JGFL)%MP9) = 0.0_JPRB
        ENDDO
      ENDDO
    ENDIF
  ENDDO

  DO JL=KST,KEN
    PGMVS(JL,YT0%MSP)  = PGMVS(JL,YT0%MSP)+PGMVS(JL,YT9%MSP)
    PGMVS(JL,YT9%MSP)  = 0.0_JPRB
    PGMVS(JL,YT0%MSPL) = PGMVS(JL,YT0%MSPL)+PGMVS(JL,YT9%MSPL)
    PGMVS(JL,YT9%MSPL) = 0.0_JPRB
    PGMVS(JL,YT0%MSPM) = PGMVS(JL,YT0%MSPM)+PGMVS(JL,YT9%MSPM)
    PGMVS(JL,YT9%MSPM) = 0.0_JPRB
  ENDDO

ELSE
  DO JK=1,NFLEVG
    DO JL=KST,KEN
      PGMV(JL,JK,YT0%MU)  = PGMV(JL,JK,YT0%MU)  +REPS2*PGMV(JL,JK,YT9%MU)
      PGMV(JL,JK,YT0%MV)  = PGMV(JL,JK,YT0%MV)  +REPS2*PGMV(JL,JK,YT9%MV)
      PGMV(JL,JK,YT0%MDIV)= PGMV(JL,JK,YT0%MDIV)+REPS2*PGMV(JL,JK,YT9%MDIV)
    ENDDO
    IF(NFTHER >= 1) THEN
      DO JL=KST,KEN
        PGMV(JL,JK,YT0%MT)  = PGMV(JL,JK,YT0%MT)  +REPS2*PGMV(JL,JK,YT9%MT)
        PGMV(JL,JK,YT0%MTL) = PGMV(JL,JK,YT0%MTL) +REPS2*PGMV(JL,JK,YT9%MTL)
        PGMV(JL,JK,YT0%MTM) = PGMV(JL,JK,YT0%MTM) +REPS2*PGMV(JL,JK,YT9%MTM)
      ENDDO
    ENDIF
    IF(LNHDYN) THEN
      DO JL=KST,KEN
        PGMV(JL,JK,YT0%MSPD) =PGMV(JL,JK,YT0%MSPD)+REPS2*PGMV(JL,JK,YT9%MSPD)
        PGMV(JL,JK,YT0%MSVD) =PGMV(JL,JK,YT0%MSVD)+REPS2*PGMV(JL,JK,YT9%MSVD)
        PGMV(JL,JK,YT0%MSPDL)=PGMV(JL,JK,YT0%MSPDL) &
         & +REPS2*PGMV(JL,JK,YT9%MSPDL)
        PGMV(JL,JK,YT0%MSVDL)=PGMV(JL,JK,YT0%MSVDL) &
         & +REPS2*PGMV(JL,JK,YT9%MSVDL)
        PGMV(JL,JK,YT0%MSPDM)=PGMV(JL,JK,YT0%MSPDM) &
         & +REPS2*PGMV(JL,JK,YT9%MSPDM)
        PGMV(JL,JK,YT0%MSVDM)=PGMV(JL,JK,YT0%MSVDM) &
         & +REPS2*PGMV(JL,JK,YT9%MSVDM)
      ENDDO
    ENDIF
  ENDDO

  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LT9) THEN
      DO JK=1,NFLEVG
        DO JL=KST,KEN
          PGFL(JL,JK,YGFLC(JGFL)%MP) = PGFL(JL,JK,YGFLC(JGFL)%MP)+&
           & REPSM2*PGFL(JL,JK,YGFLC(JGFL)%MP9)  
        ENDDO
      ENDDO
    ENDIF
  ENDDO

  DO JL=KST,KEN
    PGMVS(JL,YT0%MSP)  = PGMVS(JL,YT0%MSP) +REPS2*PGMVS(JL,YT9%MSP)
    PGMVS(JL,YT0%MSPL) = PGMVS(JL,YT0%MSPL)+REPS2*PGMVS(JL,YT9%MSPL)
    PGMVS(JL,YT0%MSPM) = PGMVS(JL,YT0%MSPM)+REPS2*PGMVS(JL,YT9%MSPM)
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTF2AD',1,ZHOOK_HANDLE)
END SUBROUTINE GPTF2AD
