SUBROUTINE GPTF2PC(&
 ! --- INPUT ---------------------------------------------------------
 & KST,KEN,&
 & PUT9  ,PVT9   ,PTT9  ,PSPDT9  ,PSVDT9  ,&
 & PSPT9 ,PNHXT9 ,&
 & PUT0  ,PVT0   ,PTT0  ,PSPDT0  ,PSVDT0  ,&
 & PSPT0 ,PNHXT0    ,&
 ! --- INPUT/OUTPUT --------------------------------------------------
 & PCUT9 ,PCVT9  ,PCTT9 ,PCSPDT9 ,PCSVDT9 ,&
 & PCSPT9,PCNHXT9)

!**** *GPTF2PC* - Timefilter part 2

!     Purpose.
!     --------
!           Performs part 2 of the time-filtering for the t-dt array.
!           Routine called in 3TL PC schemes.
            
!**   Interface.
!     ----------
!        *CALL* *GPTF2PC(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!         KST          : start of work
!         KEN          : depth of work
!         P(U,V,T,SPD,SVD,SP)T9  : pre-filtered quantities at time t-dt
!         PNHXT9                 : pre-filtered Xterm at time t-dt
!         P(U,V,T,SPD,SVD,SP)T0  : quantities at time t
!         PCD3YT0                : Xterm at time t

!        INPUT/OUTPUT:
!         PC(U,V,T,SPD,SVD,SP)T9 : quantities at time t-dt
!         PCNHXT9                : Xterm at time t-dt

!        Implicit arguments :  None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        LACE internal documentation

!     Author.
!     -------
!        Jozef Vivoda (based on GPTF1)  *ECMWF*

! Modifications
! -------------
!   Original : 21-Feb-2005
!   K. Yessad (Dec 2008): remove dummy CDLOCK
! End Modifications
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LTWOTL   ,LNHDYN   ,LPC_FULL
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA 
USE YOMDYN   , ONLY : REPS2    ,NCURRENT_ITER  
USE YOMDYNA  , ONLY : NVDVAR  
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMGMV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEN 
REAL(KIND=JPRB)   ,INTENT(IN) :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PTT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPDT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSVDT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PNHXT0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN) :: PUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSVDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PNHXT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCSPDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCSVDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCSPT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PCNHXT9(NPROMA,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL,JK
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTF2PC',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. PERFORM TIME FILTER (PART 1)
!           ----------------------------

IF (.NOT.LTWOTL.AND.NCURRENT_ITER==0) THEN

  IF( NSTEP <= 0 )THEN

    IF(LPC_FULL)THEN
      DO JL=1,NFLEVG
        DO JK=KST,KEN
          PCUT9(JK,JL)=PUT0(JK,JL)
          PCVT9(JK,JL)=PVT0(JK,JL)
          PCTT9(JK,JL)=PTT0(JK,JL)
        ENDDO
      ENDDO
      IF(LNHDYN)THEN
        DO JL=1,NFLEVG
          DO JK=KST,KEN
            PCSPDT9(JK,JL)=PSPDT0(JK,JL)
            PCSVDT9(JK,JL)=PSVDT0(JK,JL)
          ENDDO
        ENDDO
      ENDIF
      DO JK=KST,KEN
        PCSPT9(JK) = PSPT0(JK)
      ENDDO
    ENDIF

    IF(NVDVAR==4)THEN
      DO JL=1,NFLEVG
        DO JK=KST,KEN
          PCNHXT9(JK,JL)=PNHXT0(JK,JL)
        ENDDO
      ENDDO
    ENDIF

  ELSE

    IF(LPC_FULL)THEN
      DO JL=1,NFLEVG
        DO JK=KST,KEN
          PCUT9(JK,JL) = PUT9(JK,JL)+REPS2*PUT0(JK,JL)
          PCVT9(JK,JL) = PVT9(JK,JL)+REPS2*PVT0(JK,JL)
          PCTT9(JK,JL) = PTT9(JK,JL)+REPS2*PTT0(JK,JL)
        ENDDO
      ENDDO
      IF(LNHDYN)THEN
        DO JL=1,NFLEVG
          DO JK=KST,KEN
            PCSPDT9(JK,JL) = PSPDT9(JK,JL)+REPS2*PSPDT0(JK,JL)
            PCSVDT9(JK,JL) = PSVDT9(JK,JL)+REPS2*PSVDT0(JK,JL)
          ENDDO
        ENDDO
      ENDIF
      DO JK=KST,KEN
        PCSPT9(JK) = PSPT9(JK)+REPS2*PSPT0(JK)
      ENDDO
    ENDIF

    IF(NVDVAR==4)THEN
      DO JL=1,NFLEVG
        DO JK=KST,KEN
          PCNHXT9(JK,JL)=PNHXT9(JK,JL)+REPS2*PNHXT0(JK,JL)
        ENDDO
      ENDDO
    ENDIF

  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTF2PC',1,ZHOOK_HANDLE)
END SUBROUTINE GPTF2PC
