SUBROUTINE LADAD(KNUMB,&
 & KASLB1,PB1,PB2,PGMVT1,PGMVT1S,&
 & PGEMU,PGESLO,PGECLO,PGNORDL,&
 & PGNORDM,PGSQM2,PCOLON,PSILON,&
 & PRINDX,PRINDY)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPROMA   ,NFLEVG
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE PTRSLB2  , ONLY : &
 & MSLB2URL ,MSLB2VRL ,MSLB2URL5,MSLB2VRL5,&
 & MSLB2USI ,MSLB2VSI ,MSLB2USI5,MSLB2VSI5,&
 & MSLB2U15 ,MSLB2V15  
USE PTRSLB1  , ONLY : MSLB1UR0 ,MSLB1VR0 ,MSLB1U0  ,MSLB1V0  ,&
 & MSLB1SP0 ,MSLB1U9  ,MSLB1V9  ,MSLB1SP9   
USE PTRSLB15  , ONLY : MSLB1UR05,&
 & MSLB1VR05,MSLB1U05 ,MSLB1V05 ,MSLB1SP05,MSLB1U95 ,&
 & MSLB1V95 ,MSLB1SP95  
USE YOMGMV    , ONLY : YT1

!**** *LADAD* - call ladinead
!               AD code of 2D shallow-water model.

!     Purpose.
!     --------
!           call ladinead (maintaining bit reproducibility)

!**   Interface.
!     ----------
!        *CALL* *LADAD(...)*

!        Explicit arguments :
!        --------------------
!        ASSOCIATED WITH INPUT IN LADAD:
!         KNUMB     : number of elements of arrays for which computations (I)
!                     are performed (used in message passing version).
!         KASLB1    : horizontal dimension for semi-Lagrangian buffer.    (I)
!         PB1       : buffer for quantities to be interpolated.
!         PB2       : buffer for quantities at the arrival grid-point.    (O)
!         PGEMU     : sinus of the grid-point geographical latitude.
!         PGSQM2    : cosinus of the grid-point geographical latitude.
!         PGECLO    : cosinus of the grid-point geographical longitude.
!         PGESLO    : sinus of the grid-point geographical longitude.
!         PGNORDL   : zonal component of the normalized vector directed
!                     towards the geographical north pole.
!         PGNORDM   : meridian component of the normalized vector directed
!                     towards the geographical north pole.
!         PCOLON    : cosinus of the grid-point longitude on the
!                     computational sphere.
!         PSILON    : sinus of the grid-point longitude on the
!                     computational sphere.
!         PRINDX    : x coordinate on the plane.
!         PRINDY    : y coordinate on the plane.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!      Is called by SCAN2MAD.

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!        George Mozdzynski  *ECMWF*

!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(KASLB1,NFLDSLB1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVT1S(NPROMA,YT1%NDIMS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRINDX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRINDY(NPROMA) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IST, IEND, IFLDX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "ladinead.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LADAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IST   = 1
IEND  = KNUMB

IFLDX=NFLDSLB1

CALL LADINEAD(IST,IEND,KASLB1,NPROMA,1,IFLDX,&
 & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PGNORDL,PGNORDM,PRINDX,PRINDY,PB1,&
 & PB1(1,MSLB1UR0),PB1(1,MSLB1VR0),&
 & PB2(1,MSLB2URL)  ,PB2(1,MSLB2VRL),&
 & PB1(1,MSLB1U0) ,PB1(1,MSLB1V0),PB1(1,MSLB1SP0),&
 & PB1(1,MSLB1U9) ,PB1(1,MSLB1V9),PB1(1,MSLB1SP9),&
 & PB1(1,MSLB1UR05),PB1(1,MSLB1VR05),&
 & PB2(1,MSLB2URL5)  ,PB2(1,MSLB2VRL5),&
 & PB1(1,MSLB1U05) ,PB1(1,MSLB1V05),PB1(1,MSLB1SP05),&
 & PB1(1,MSLB1U95) ,PB1(1,MSLB1V95),PB1(1,MSLB1SP95),&
 & PB2(1,MSLB2USI),PB2(1,MSLB2VSI),&
 & PB2(1,MSLB2USI5),PB2(1,MSLB2VSI5),&
 & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),&
 & PGMVT1S(1,YT1%MSP),&
 & PB2(1,MSLB2U15),PB2(1,MSLB2V15))  

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LADAD',1,ZHOOK_HANDLE)
END SUBROUTINE LADAD

