SUBROUTINE LAIDDI_INIT(KPROMA,KPROMB,KSTART,KPROF,KFLEV,KL0,&
 & KMAP,PSIGN )  

!**** *LAIDDI_INIT - semi-LAgrangian scheme:   (adjoint version)
!                 Initialise for BI-dimensional 12 points interpolations.

!     Purpose.
!     --------
!       Initialise for bi-dimensional 12 point interpolations.

!**   Interface.
!     ----------
!        *CALL* *LAIDDI_INIT(KPROMA,KPROMB,KSTART,KPROF,KFLEV,KL0
!                      ,KMAP,PSIGN)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.
!        OUTPUT:
!          KMAP    - mapping array for bit reproducibility
!          PSIGN   - sign (+ or -) to be applied to point (symmetric/
!                    antisymmetric properties at polar latitudes)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCINAD.

!     Reference.
!     ----------

!     Author.
!     -------
!        G. Mozdzynski (ECMWF)

!     Modifications.
!     --------------
!      Original : 00/08/16
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMSLREP , ONLY : NSLMAP, RSASIGN

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,0:3) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KMAP(KPROMB,12,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSIGN(KPROMB,12,KFLEV,2) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
INTEGER(KIND=JPIM) :: I1,I2,I3,I4,I5,I6,I7,I8,I9,I10,I11,I12
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDDI_INIT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

DO JLEV=1,KFLEV

!OCL NOVREC
  DO JROF=KSTART,KPROF

    I1=KL0(JROF,JLEV,0)+1
    I2=KL0(JROF,JLEV,0)+2
    I3=KL0(JROF,JLEV,1)
    I4=KL0(JROF,JLEV,1)+1
    I5=KL0(JROF,JLEV,1)+2
    I6=KL0(JROF,JLEV,1)+3
    I7=KL0(JROF,JLEV,2)
    I8=KL0(JROF,JLEV,2)+1
    I9=KL0(JROF,JLEV,2)+2
    I10=KL0(JROF,JLEV,2)+3
    I11=KL0(JROF,JLEV,3)+1
    I12=KL0(JROF,JLEV,3)+2

    KMAP(JROF,1,JLEV)=NSLMAP(I1)
    KMAP(JROF,2,JLEV)=NSLMAP(I2)
    KMAP(JROF,3,JLEV)=NSLMAP(I3)
    KMAP(JROF,4,JLEV)=NSLMAP(I4)
    KMAP(JROF,5,JLEV)=NSLMAP(I5)
    KMAP(JROF,6,JLEV)=NSLMAP(I6)
    KMAP(JROF,7,JLEV)=NSLMAP(I7)
    KMAP(JROF,8,JLEV)=NSLMAP(I8)
    KMAP(JROF,9,JLEV)=NSLMAP(I9)
    KMAP(JROF,10,JLEV)=NSLMAP(I10)
    KMAP(JROF,11,JLEV)=NSLMAP(I11)
    KMAP(JROF,12,JLEV)=NSLMAP(I12)

    PSIGN(JROF,1,JLEV,1)=RSASIGN(MOD(I1-1,KPROMA)+1,1)
    PSIGN(JROF,1,JLEV,2)=RSASIGN(MOD(I1-1,KPROMA)+1,2)
    PSIGN(JROF,2,JLEV,1)=RSASIGN(MOD(I2-1,KPROMA)+1,1)
    PSIGN(JROF,2,JLEV,2)=RSASIGN(MOD(I2-1,KPROMA)+1,2)
    PSIGN(JROF,3,JLEV,1)=RSASIGN(MOD(I3-1,KPROMA)+1,1)
    PSIGN(JROF,3,JLEV,2)=RSASIGN(MOD(I3-1,KPROMA)+1,2)
    PSIGN(JROF,4,JLEV,1)=RSASIGN(MOD(I4-1,KPROMA)+1,1)
    PSIGN(JROF,4,JLEV,2)=RSASIGN(MOD(I4-1,KPROMA)+1,2)
    PSIGN(JROF,5,JLEV,1)=RSASIGN(MOD(I5-1,KPROMA)+1,1)
    PSIGN(JROF,5,JLEV,2)=RSASIGN(MOD(I5-1,KPROMA)+1,2)
    PSIGN(JROF,6,JLEV,1)=RSASIGN(MOD(I6-1,KPROMA)+1,1)
    PSIGN(JROF,6,JLEV,2)=RSASIGN(MOD(I6-1,KPROMA)+1,2)
    PSIGN(JROF,7,JLEV,1)=RSASIGN(MOD(I7-1,KPROMA)+1,1)
    PSIGN(JROF,7,JLEV,2)=RSASIGN(MOD(I7-1,KPROMA)+1,2)
    PSIGN(JROF,8,JLEV,1)=RSASIGN(MOD(I8-1,KPROMA)+1,1)
    PSIGN(JROF,8,JLEV,2)=RSASIGN(MOD(I8-1,KPROMA)+1,2)
    PSIGN(JROF,9,JLEV,1)=RSASIGN(MOD(I9-1,KPROMA)+1,1)
    PSIGN(JROF,9,JLEV,2)=RSASIGN(MOD(I9-1,KPROMA)+1,2)
    PSIGN(JROF,10,JLEV,1)=RSASIGN(MOD(I10-1,KPROMA)+1,1)
    PSIGN(JROF,10,JLEV,2)=RSASIGN(MOD(I10-1,KPROMA)+1,2)
    PSIGN(JROF,11,JLEV,1)=RSASIGN(MOD(I11-1,KPROMA)+1,1)
    PSIGN(JROF,11,JLEV,2)=RSASIGN(MOD(I11-1,KPROMA)+1,2)
    PSIGN(JROF,12,JLEV,1)=RSASIGN(MOD(I12-1,KPROMA)+1,1)
    PSIGN(JROF,12,JLEV,2)=RSASIGN(MOD(I12-1,KPROMA)+1,2)

  ENDDO

ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LAIDDI_INIT',1,ZHOOK_HANDLE)
END SUBROUTINE LAIDDI_INIT
