SUBROUTINE LAIDLIAD(KPROMA,KPROMB,KSTART,KPROF,KFLEV,&
 & KFLDN,KFLDX,&
 & PDLAT,PDLO,KL0,&
 & PXSL,PXF)  

!**** *LAIDLIAD-  semi-LAgrangian scheme:
!                 Bilinear horizontal interpolations for one variable.

!     Purpose.
!     --------
!       Performs bilinear horizontal interpolations for one variable.

!       The current code is valid only if the interpolation point and
!        the weights are constants (contrary to LAIDLITLAD which assumes
!        that these quantities temporally vary).

!**   Interface.
!     ----------
!        *CALL* *LAIDLIAD(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          PDLAT   - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PDLO    - weights (distances) for horizontal linear interpolation
!                    on a same longitude.
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.

!        INPUT/OUTPUT:
!          PXSL    - quantity to be interpolated.

!        INPUT:
!          PXF     - interpolated variable.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutines LAGINL3
!        written by Maurice IMBARD, Alain CRAPLET and Michel ROCHAS
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!        Original : FEBRUARY 1992.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,KFLEV,1:2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,1:2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSL(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXF(KPROMB,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZDLAT, ZDLO1, ZDLO2, ZVALL01, ZVALL02
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDLIAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

! k.y.: this piece of code is currently unused.
!   It is planned in the future to put the content of LAIDLITLAD
!   in this deck.

DO JROF=KSTART,KPROF
!DIR$ IVDEP
!OCL NOVREC
  DO JLEV=1,KFLEV

!     Computation of coordinates and distances.

    ZDLAT =PDLAT(JROF,JLEV)

    ZDLO1 =PDLO(JROF,JLEV,1)
    ZDLO2 =PDLO(JROF,JLEV,2)

!     Interpolation.

    ZVALL01=(1.0_JPRB-ZDLAT)*PXF(JROF,JLEV)
    ZVALL02=ZDLAT*PXF(JROF,JLEV)
    PXSL(KL0(JROF,JLEV,1)+1)=PXSL(KL0(JROF,JLEV,1)+1)+(1.0_JPRB-ZDLO1)*ZVALL01
    PXSL(KL0(JROF,JLEV,1)+2)=PXSL(KL0(JROF,JLEV,1)+2)+ZDLO1*ZVALL01
    PXSL(KL0(JROF,JLEV,2)+1)=PXSL(KL0(JROF,JLEV,2)+1)+(1.0_JPRB-ZDLO2)*ZVALL02
    PXSL(KL0(JROF,JLEV,2)+2)=PXSL(KL0(JROF,JLEV,2)+2)+ZDLO2*ZVALL02

  ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LAIDLIAD',1,ZHOOK_HANDLE)
END SUBROUTINE LAIDLIAD

