SUBROUTINE LAITRE_GMV_TL( &
 ! --- INPUT ---------------------------------------------------------
 & KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX, &
 & LDSLHD,LDADV5, &
 & KL0,PDLO,PCLO,PCLOSLD, &
 & PDLAT,PCLA,PCLASLD,PVINTW,PVINTWSLD, &
 & PDLO5,PCLO5,PCLOSLD5, &
 & PDLAT5,PCLA5,PCLASLD5,PVINTW5,PVINTWSLD5, &
 & PXSL,PXSL5, &
 ! --- OUTPUT --------------------------------------------------------
 & POUT,POUT5)

! Purpose :
! -------
!   LAITRE_GMV_TL - semi-Lagrangian scheme: origin point interpolations 
!   for all variables (tangent-linear version)

! Interface :
! ---------
!   INPUT:
!     KASLB1     - horizontal dimension of SLBUF1
!     KPROMA     - horizontal dimension
!     KSTART     - first element of arrays where computations are performed
!     KPROF      - depth of work
!     KFLEV      - vertical dimension
!     KFLDN      - number of the first field
!     KFLDX      - number of the last field
!     LDSLHD     - switch: .T. = diffusive interpolation (SLHD)
!                          .F. = normal interpolation
!     LDADV5     - field advected without wind increments
!     KL0        - indices of the four western points of the 16 point 
!                  interpolation grid
!     PDLO       - distances for horizontal linear interpolations
!                  in longitude (latitude rows 0, 1, 2, 3)
!     PCLO       - weights for horizontal cubic interpolations in longitude
!                  (latitude rows 1, 2)
!     PCLOSLD    - weights for horizontal cubic interpolations in longitude,
!                  SLHD case (latitude rows 1, 2)
!     PDLAT      - distance for horizontal linear interpolations in latitude
!     PCLA       - weights for horizontal cubic interpolations in latitude
!     PCLASLD    - weights for horizontal cubic interpolations in latitude,
!                  SLHD case
!     PVINTW     - weights for cubic vertical interpolation
!     PVINTWSLD  - weights for cubic vertical interpolation, SLHD case
! ----------------------------trajectory variables------------------------
!     PDLO5      - distances for horizontal linear interpolations
!                  in longitude (latitude rows 0, 1, 2, 3)
!     PCLO5      - weights for horizontal cubic interpolations in longitude
!                  (latitude rows 1, 2)
!     PCLOSLD5   - weights for horizontal cubic interpolations in longitude,
!                  SLHD case (latitude rows 1, 2)
!     PDLAT5     - distance for horizontal linear interpolations in latitude
!     PCLA5      - weights for horizontal cubic interpolations in latitude
!     PCLASLD5   - weights for horizontal cubic interpolations in latitude,
!                  SLHD case
!     PVINTW5    - weights for cubic vertical interpolation
!     PVINTWSLD5 - weights for cubic vertical interpolation, SLHD case
! ------------------------------------------------------------------------
!     PXSL       - quantity to interpolate at O
! ----------------------------trajectory variable ------------------------
!     PXSL5      - quantity to interpolate at O
! ------------------------------------------------------------------------
!   OUTPUT:
!     POUT       - interpolated quantity at O
! ----------------------------trajectory variable ------------------------
!     POUT5      - interpolated quantity at O
! ------------------------------------------------------------------------

! Externals :
! ---------
!   ABOR1
!   LAITRITL

! Method :
! ------
!   See documentation.

! Reference :
! ---------

! Author :
! ------
!   01-Jul-2008 F. Vana (CHMI/LACE) 

! Modifications :
! -------------

!-------------------------------------------------------------------------------

USE PARKIND1  , ONLY : JPIM     ,JPRB
USE YOMHOOK   , ONLY : LHOOK    ,DR_HOOK

!    -----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDADV5
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMA,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMA,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(KPROMA,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOSLD(KPROMA,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLASLD(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWSLD(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO5(KPROMA,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO5(KPROMA,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOSLD5(KPROMA,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT5(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA5(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLASLD5(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW5(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWSLD5(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KASLB1,KFLDN:KFLDX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL5(KASLB1,KFLDN:KFLDX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT5(KPROMA,KFLEV) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -----------------------------------------------------------------------

#include "laitritl.intfb.h"

!    -----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRE_GMV_TL',0,ZHOOK_HANDLE)

!    -----------------------------------------------------------------------


! 1. Origin point interpolations 
!    --------------------------------------------------------------------

IF (LDSLHD) THEN
  CALL LAITRITL(KASLB1,KPROMA,KSTART,KPROF,KFLEV, &
   & KFLDN,KFLDX,LDADV5, &
   & PDLAT,PCLASLD,PDLO,PCLOSLD,PDLAT5,PCLASLD5,PDLO5,PCLOSLD5, &
   & KL0(1,1,0),PVINTWSLD,PVINTWSLD5, &
   & PXSL,PXSL5,POUT,POUT5)  
ELSE
  CALL LAITRITL(KASLB1,KPROMA,KSTART,KPROF,KFLEV, &
   & KFLDN,KFLDX,LDADV5, &
   & PDLAT,PCLA,PDLO,PCLO,PDLAT5,PCLA5,PDLO5,PCLO5, &
   & KL0(1,1,0),PVINTW,PVINTW5, &
   & PXSL,PXSL5,POUT,POUT5)
ENDIF

!    -----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRE_GMV_TL',1,ZHOOK_HANDLE)
END SUBROUTINE LAITRE_GMV_TL
