SUBROUTINE LAPINEB( &
 ! --- INPUT ------------------------------------------------------------------
 & KFLDSLB2,KST,KPROF,KSTGLO,LDSLPHY,&
 & PCOLON,PSILON,PGM,PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PGNORDL,PGNORDM,POMVRL,POMVRM,PINDX,PINDY,&
 & POROG,POROGL_RE,POROGM_RE,&
 & KL0,KLH0,PDLO,PCLO,PCLOSLD,&
 & PDLAT,PCLA,PCLASLD,KLEV,PDVER,PVINTW,PVINTWSLD,PVINTWS,&
 & PVDERW,PHVW,&
 & KL0H,PDLOH,PCLOH,PCLOHSLD,PDLATH,PCLAH,PCLAHSLD,PDVERH,PVINTWH,PVINTWHSLD,&
 & PUF,PVF,&
 & PB1,PQ,&
 ! --- INPUT/OUTPUT -----------------------------------------------------------
 & KVSEPC,KVSEPL,&
 & PCOSCO,PSINCO,PSINLA,PCOPHI,&
 & PLON,PLAT,PQX,PQY,&
 & PB2,PGMVT1,PGMVT1S,PGFLT1,&
 & PGFLPC,&
 ! --- OUTPUT -----------------------------------------------------------------
 & PUP9,PVP9,PTP9,PGFLP9)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE YOMSTA   , ONLY : RTSUR
USE YOMCST   , ONLY : RPI      ,ROMEGA   ,RA       ,RD
USE YOMCT0   , ONLY : LNHDYN   ,LRPLANE  ,LVERCOR  ,LPC_OLD  ,LPC_CHEAP
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_GW
USE YOMDIM   , ONLY : NDGUNG   ,NDGUXG   ,NDLUNG   ,NDLUXG   ,NFLEVG   , &
 & NFGPPC   ,NPROMA   ,NPROMVC  ,NPROMDLR ,NFLSA    ,NFLEN
USE YOMDYNA  , ONLY : LGWADV   ,NGWADVSI ,LRDBBC   ,NVDVAR   ,LNH_PDVD , &
 & LRWSDLR  ,LRWSDLW  ,ND4SYS   ,LSLDIA
USE YOMDYN   , ONLY : NVLAG    ,RCMSLP0  ,&
 &                    LADVFW   ,LADVF    ,L2TLFF   ,RW2TLFF  ,LSLINLC1 , &
 &                    NSITER   ,NCURRENT_ITER      ,LSLINLC2 ,LSLINL
USE YOMGEM   , ONLY : VDELB    ,VFE_RDETAH,NSTTYP  ,RMUCEN   ,RLOCEN   , &
 &                    RSTRET   ,RC2P1    ,RC2M1    ,VBH      ,VAH      , &
 &                    VC       ,VDELA
USE YEMGEO   , ONLY : EDELX    ,EDELY
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMPHY   , ONLY : LMPHYS
USE YOMRIP   , ONLY : RTDT
USE YOM_YGFL , ONLY : YGFL     ,YGFLC,YQ,YCVGQ
USE YOMGPPCB , ONLY : GPPCBUF
USE YOMSC2   , ONLY : NFLDSLB1
USE YOMGMV   , ONLY : YT1
USE YOMMP    , ONLY : NASLB1
USE PTRGPPC  , ONLY : MGPPCF_U ,MGPPCF_V ,MGPPCF_T ,MGPPCF_SPD,MGPPCF_SVD, &
 & MGPPCF_SP,MGPPCF_UP,MGPPCF_VP,MGPPCF_TP,MGPPCF_GFLP,MGPPCF_NHX,MGPPCF_CP, &
 & MGPPCF_CPSI,MGPPCF_BBC
USE PTRSLB2  , ONLY : MSLB2Y1  ,MSLB2VCVWEI,MSLB2DBBC1,MSLB2DPHI1,&
 & MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2SPSI,MSLB2VDSI,MSLB2PDSI,&
 & MSLB2GWF ,MSLB2GWS ,MSLB2GDW
USE PAR_RDLR , ONLY : JP_RSAF  ,JP_ASRF  ,JP_ASRH  ,JP_DIMR1

!**** *LAPINEB* - semi-LAgrangian scheme(Interpolation):
!                Interface subroutine for interpolations and lagrangian
!                trends. (Programme INterface d'Ensemble).

!     Purpose.
!     --------
!       Grid point calculations in dynamics.

!       Does the interpolations at the origin point O of the SL-trajectory.
!       Then adds the interpolated quantities of the RHS of equations to
!       yield a provisional value for X(t+dt) for each prognostic variable X.

!       Abbreviation "vwv" stands for "vertical wind variable".

!**   Interface.
!     ----------
!        *CALL* *LAPINEB(.....)

!        Explicit arguments :
!        --------------------
!        INPUT:
!          KFLDSLB2    - second dimension of PB2 (must be >= NFLDSLB2)
!          KST         - first element of arrays where
!                        computations are performed.
!          KPROF       - depth of work.
!          KSTGLO      - global offset into nproma buffer.
!          LDSLPHY     - split physics in semi lagrangian mode.
!          PCOLON      - cos(computational sphere longitude).
!          PSILON      - sin(computational sphere longitude).
!          PGM         - mapping factor.
!          PGEMU       - sin(geographical latitude).
!          PGSQM2      - cos(geographical latitude).
!          PGECLO      - cos(geographical longitude).
!          PGESLO      - sin(geographical longitude).
!          PGNORDL     - zonal component of the vector directed towards the
!                        true North pole.
!          PGNORDM     - meridian component of the vector directed towards the
!                        true North pole.
!          POMVRL      - zonal component of "2 Omega vec a k".
!          POMVRM      - meridian component of "2 Omega vec a k".
!          PINDX       - x coordinate on the plane.
!          PINDY       - y coordinate on the plane.
!          POROG       - surface orography (Phi_surf).
!          POROGL_RE   - zonal component of vnabla'(orography).
!          POROGM_RE   - meridian component of vnabla'(orography).
!          KL0         - index of the four western points
!                        of the 16 points interpolation grid.
!          KLH0        - second value of index of the four western points
!                        of the 16 points interpolation grid if needed.
!          PDLO        - distances for horizontal linear interpolations
!                        in longitude (latitude rows 0, 1, 2, 3)
!          PCLO        - weights for horizontal cubic interpolations in
!                        longitude (latitude rows 1, 2)
!          PCLOSLD     - weights for horizontal cubic interpolations in
!                        longitude, SLHD case (latitude rows 1, 2)
!          PDLAT       - distance for horizontal linear interpolations
!                        in latitude
!          PCLA        - weights for horizontal cubic interpolations
!                        in latitude
!          PCLASLD     - weights for horizontal cubic interpolations
!                        in latitude, SLHD case
!          KLEV        - lower level of the vertical interpolation
!                        grid needed for vertical interpolations.
!          PDVER       - distance for vertical linear interpolation.
!          PVINTW      - Vertical cubic interpolation weights.
!          PVINTWSLD   - Vertical cubic interpolation weights, SLHD case.
!          PVINTWS     - Vertical spline interpolation weights.
!          PVDERW      - weights to compute vertical derivatives necessary for
!                        Hermite cubic vertical interpolation.
!          PHVW        - Hermite vertical cubic interpolation weights.
!          KL0H        - half-level KL0
!          PDLOH       - half-level PDLO
!          PCLOH       - half-level PCLO
!          PCLOHSLD    - half-level PCLOSLD
!          PDLATH      - half-level PDLAT
!          PCLAH       - half-level PCLA
!          PCLAHSLD    - half-level PCLASLD
!          PDVERH      - half-level PDVER
!          PVINTWH     - half-level PVINTW
!          PVINTWHSLD  - half-level PVINTWSLD
!          PUF         - U-comp of "(a/rs)*wind" at the medium point necessary
!                        to find the positions of the medium and origin points,
!                        in a local repere linked to computational sphere.
!                        Used in this routine for l2tlff=T.
!          PVF         - V-comp of "(a/rs)*wind" at the medium point necessary
!                        to find the positions of the medium and origin points,
!                        in a local repere linked to computational sphere.
!                        Used in this routine for l2tlff=T.
!          PB1         - "SLBUF1" buffer for interpolations.
!          PQ          - moisture at "t".

!        INPUT/OUTPUT:
!          KVSEPC      - vertical separation (used in SL adjoint, cubic interp)
!          KVSEPL      - vertical separation (used in SL adj, linear interp)
!          PCOSCO      - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                        of the interpolation point (geographical longitude
!                        and latitude).
!                        ! if LRPLANE: x - coordinate (fractional system).
!          PSINCO      - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                        of the interpolation point (geographical longitude
!                        and latitude).
!                        ! if LRPLANE: y - coordinate (fractional system).
!          PSINLA      - sine of the interpolation point geographical latitude.
!          PCOPHI      - cosine of the geographical angle between the
!                        interpolation point and the grid-point.
!          PLON        - computational sphere longitude of interpolation point.
!          PLAT        - computational sphere latitude of interpolation point.
!          PQX         - first element of the wind displacement matrix (pt O).
!          PQY         - second element of the wind displacement matrix (pt O).
!          PB2         - "SLBUF2" buffer.
!          PGMVT1      - t+dt GMV (upper air) variables.
!          PGMVT1S     - t+dt GMVS (surface) variables.
!          PGFLT1      - t+dt unified_treatment grid-point fields.
!          PGFLPC      - to store PC scheme quantities for GFL.

!        OUTPUT:
!          PUP9        - Interpolated quantity at O in U-wind eqn
!                        for split physics (if LDSLPHY=.T. only).
!          PVP9        - Interpolated quantity at O in V-wind eqn
!                        for split physics (if LDSLPHY=.T. only).
!          PTP9        - Interpolated quantity at O in T eqn
!                        for split physics (if LDSLPHY=.T. only).
!          PGFLP9      - Interpolated quantity at O in unified_treatment
!                        grid-point fields
!                        for split physics (if LDSLPHY=.T. only).

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------
!      Called by CALL_SL.

!     Reference.
!     ----------
!        ARPEGE documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        K. YESSAD after the subroutine CPLGDY2 written by
!        Maurice IMBARD   METEO FRANCE/EERM/CRMD

! Modifications
! -------------
!   Original : JUNE 1991.
!   Modifications:
!   K. YESSAD (METEO-FRANCE CNRM/GMAP).  22/11/1991.
!   K. YESSAD: FEBRUARY 1992.
!   K. YESSAD: MAY 1992.
!   M. HORTAL (ECMWF) and K. YESSAD (CNRM/GMAP): 04/11/1992.
!   K. YESSAD: DECEMBER 1992 (non interpolating vertical scheme and
!              some cleaning, introduction of a decentering factor
!              in semi-implicit terms and terms averaged along the
!              semi-lagrangian trajectory).
!   K. YESSAD: MARCH 1993 (non interpolating vertical scheme
!              with exact residual advections, LVADRES=.T.).
!   Modified 93-12-14 by K. YESSAD: interpolations at medium point for
!                                 semi-lagrangian temperature equation.
!   M.HAMRUD: 94-01-31 Memory savings (use of "work-space")
!   Modified 94-03-23 by M. Hortal: interpolations at medium point for
!                                 semi-lagrangian momentum equations.
!   Modified 94-10-17 M. Tolstykh: CUD interpolation, variable decentering
!   K. YESSAD (JAN 1995):
!        - Introduction of array RCUDWM for optimisations of CUD-5
!          meridian derivatives.
!        - Constant part of decentering factor becomes VESLA for
!          explicit terms and vertical residual advections.
!   R. BUBNOVA (22 JAN 1995):
!        - Plane geometry, after developments of Martin Janousek.
!   K. YESSAD (AUGUST 1995):
!        - Accurate two-time level SL scheme (after Tolstykh/Janousek).
!        - Second-order decentering in two-time level SL scheme.
!        - DO loops updated according to F90 norms.
!   K. YESSAD (MARCH 1996):
!        - New option LCORAN=.T. (analytical computation of 2*Omega*Vec*r).
!   Modified 96-03 by A. Untch : grid-point prognostic ozone (EC)
!   Modified 96-04-17 by C. Temperton:
!        - Refined treatment of Coriolis term in 2TL scheme
!   Modified by K. YESSAD (AUGUST 1996):
!        - Hermite cubic vertical interpolations for passive scalars.
!          Not yet coded for plane geometry.
!          Not yet coded for CUD-5 scheme.
!        - Removal of variable uncentering.
!   Modified 96-10-23 by C. Temperton / M. Hortal:
!        - Analytic treatment of Coriolis term in 2TL scheme for
!          option L2TLFF=T.
!   Modified 97-05-19 by C. Temperton: LCUTRA option added      
!          (only used for 2D model)
!   Modified by K. YESSAD (MAY 1998): adaptation of L2TLFF to LAGPHY=.F..
!   Modified July 1998 by K. YESSAD: cleaning
!        - removal of options NODEC=2, L2TLAW=T, L2TLAT=T
!        - VESLA is replaced by VESL.
!        - removal of useless dummy arguments.
!        - complete declaration of dummy arguments.
!        - correction of false comments.
!        - replace NDGSA,NDGEN by NDGSAG,NDGENG
!        - adaptation of L2TLFF to NSTTYP=2 (correction of residual bugs).
!        - LRPLANE version of L2TLFF.
!   Modified 98-05-27 by E. Holm: Order of ozone changed in call
!          and in calls to LARMES and LAINOR.
!   Modified 98-09-01 by N. Wedi, added parameters for SL-physics
!   Modified 98-12-02 by K. YESSAD: new option LRADVTORO.
!   Modified 98-12-18 by K. YESSAD and I. GOSPODINOV: option LELTRA.
!   Modified 99-11-16 by C. Temperton: added LLADJO in call to LAINOR;
!                                           allowed LSLAG+LSPO3.
!   Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!   Modified 00-02-07 by C. Temperton: removed redundant use of PDP0
!                                      when NVLAG=3 (contained zero!)
!   Modified 00-02-11 by C. Temperton: added KVSEP, LFINDVSEP
!   Modified 00-06-15 by C. Temperton: NVSEP -> NVSEPC,NVSEPL.
!   Modified NOV 1999 by K. YESSAD:
!    - in 'lacudw', compute PDLAT and ILA as in 'lascaw' =>
!      'nrlatx', 'nindlat' and 'rdllat' become useless.
!   K. YESSAD (Nov 1999): cleaning
!    - recode SL3TL like SL2TL and eliminate useless interpolations
!      in continuity equation.
!      Remark: (PX0,PDP0) is now useless but has to be kept to make
!      easier a test of a pure 2D formulation of continuity equation.
!   Modified April 2000 by Malgorzata Szczech:
!    - bug corrections for thermodynamical ozon.
!    - add option "lhvo3" for thermodynamical ozon.
!   Modified August 2000 by Deborah Salmond to split into _traj and _int
!   Modified 00-11-06 by C. Temperton: bugfix for ozone advection.
!   Modified 00-11-23 by C. Temperton: renamed _TRAJ and _INT routines.
!   Modified 01-06-26 by K. YESSAD:
!    - cleanings (remove useless variables),
!      update or correct some comments, especially in the context of the
!      introduction of thin layer hypothesis relaxation ("(a/rs)*winds"
!      are interpolated and no longer winds if lvercor=true).
!    - Relaxation of thin layer hypothesis.
!    - Restore options using Hermite vertical interpolations.
!   Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!   D.Salmond (15-10-01) FULLIMP mods
!   Modified 02-03-08 by K. YESSAD:
!    - Relaxation of thin layer hypothesis for l2tlff=true.
!   J.Vivoda 03-2002 PC schemes for NH dynamics (LPC_XXXX keys)
!   Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!    semi-lag advection of vertical divergence in the NH model.
!   Modified 02-09-30 by P. Smolikova : variable d4 in NH
!   Modified 2003-03-16 M.Hamrud/M.Hortal - Revised data flow (GFL)
!   Modified 03-2003 by J. Vivoda : bugfix for option LGWADV=T.
!   Modified 08-2003 by F. Vana : option LSLHD.
!   Modified 08-2003 by P. Smolikova : option LRDBBC.
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   M.Hortal 01-Dec-2003 Extra-fields coming from the dynamics
!   D.Salmond 22-Dec-2003 Karim's Larcin cleaning
!   09-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LVSLWBC)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   04-Nov-2004 F. Vana   : No more interpolation for SLHD coeff. + splines
!   20-Feb-2005 J. Vivoda : 3TL Eul PC scheme, GWADV scheme for PC_FULL 
!   11-Feb-2005 F. Vana : Clening
!   07-Sep-2005 K. Yessad : directly calls LARCINHB.
!   14-Oct-2005 Y. Bouteloup : Computation of CVGQ for French physics 
!   K. Yessad 07-02-2007: Splitting alti/surf for (gw) in NH+LGWADV.
!   K. Yessad 07-02-2007: Call GPUVS everywhere to compute (U,V)_surf.
!   K. Yessad 07-02-2007: Cleaning (indentations).
!   K. Yessad 07-03-2007: Remove useless (gw)_surf interpolations in NH+LGWADV.
!   28-Aug-2007 F. Vana : Removing splines
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   30-Jun-2008 J. Masek   Dataflow for new SLHD interpolators
!   K. Yessad Nov 2008: rationalisation of dummy argument interfaces
! End Modifications
!------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSLB2
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO
LOGICAL           ,INTENT(IN)    :: LDSLPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMVRL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMVRM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINDX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINDY(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL_RE(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM_RE(NPROMA) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(NPROMA,NFLEVG,0:3) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLH0(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOSLD(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLASLD(NPROMA,NFLEVG,3)
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWSLD(NPROMA,NFLEVG,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWS(NPROMA,NFLEVG,1:4) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVDERW(NPROMA,NFLEVG,2,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHVW(NPROMA,NFLEVG,4) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0H(NPROMA,0:NFLEVG-1,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLOH(NPROMA,0:NFLEVG-1,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOH(NPROMA,0:NFLEVG -1,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOHSLD(NPROMA,0:NFLEVG-1,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLATH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLAH(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLAHSLD(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVERH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWH(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWHSLD(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB1(NASLB1,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(NPROMA,NFLEVG)  
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVSEPC 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVSEPL 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOSCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSINCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSINLA(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOPHI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLON(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLAT(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQX(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQY(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,KFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLPC(NPROMA,NFLEVG,YGFL%NDIMPC)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGFLP9(NPROMA,NFLEVG,YGFL%NUMFLDS) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IHVIO
INTEGER(KIND=JPIM) :: JLEV
INTEGER(KIND=JPIM) :: JROF
INTEGER(KIND=JPIM) :: JGFL
INTEGER(KIND=JPIM) :: IVDVAR
INTEGER(KIND=JPIM) :: IDIMGFL

LOGICAL :: LL2TLFFO
LOGICAL :: LLCOMPUTE_OMVAK_O,LLDL
LOGICAL :: LLNEED_RS1 ! T: rs(t+dt) is needed.
LOGICAL :: LLNEED_R1  ! T: r(t+dt) is needed.
LOGICAL :: LLNEED_NHX
LOGICAL :: LLDER
LOGICAL :: LLSLINLC2

! * interpolated quantities arrays.
REAL(KIND=JPRB) :: ZUT1(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZVT1(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUZ9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZVZ9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSPT1(NPROMA,NFLEVG), ZSPTO(NPROMA,NFLEVG+1)
REAL(KIND=JPRB) :: ZRSA9(NPROMA,NFLEVG)
! * interpolated quantities arrays for NH:
REAL(KIND=JPRB),ALLOCATABLE :: ZVWV9(:,:), ZNHX9(:,:), ZSPD9(:,:), ZY9(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZU9_SI(:,:),ZV9_SI(:,:),ZT9_SI(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSVD9_SI(:,:),ZSPD9_SI(:,:)
! * interpolated quantities arrays for diagnostic BBC in NH:
REAL(KIND=JPRB),ALLOCATABLE :: ZDBBC9(:), ZDPHI9(:), ZGWS9(:)
! * interpolated quantities for physics tendencies:
REAL(KIND=JPRB) ,ALLOCATABLE :: ZUP9(:,:), ZVP9(:,:)
! ZDP9        - Interpolated 2D term at O for continuity equation.
! ZU9         - Interpolated quantity at O in the U-wind eqn.
! ZV9         - Interpolated quantity at O in the V-wind eqn.
! ZT9         - Interpolated quantity at O in the temperature eqn.
! ZGFL9       - Interpolated quantity at O in the
!               unified_treatment grid-point fields
REAL(KIND=JPRB) :: ZDP9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZU9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZV9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZT9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGFL9(NPROMA,NFLEVG,YGFL%NUMFLDS)
REAL(KIND=JPRB) :: ZC9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZC9_SI(NPROMA,NFLEVG)

! * t+dt quantities used only if lvercor=.true.:
REAL(KIND=JPRB),ALLOCATABLE :: ZPREHYD1H(:,:)    ! "prehyd" on interlayers.
REAL(KIND=JPRB),ALLOCATABLE :: ZPREHYD1F(:,:)    ! "prehyd" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZDELP1(:,:)       ! "delta prehyd" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZRDELP1(:,:)      ! "1/(delta prehyd)" on layers
REAL(KIND=JPRB),ALLOCATABLE :: ZLNPR1(:,:)       ! "delta" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZALPH1(:,:)       ! "alpha" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZRTGR1(:,:)       ! pressure gradient term
!                                                  on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZRPRE1(:,:)       ! "1/prehyd" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZRPP1(:,:)        ! "1/(prehyd[lbar]
!                                                   *prehyd[lbar-1])".
REAL(KIND=JPRB),ALLOCATABLE :: ZVCRSSA1F(:,:)    ! "rs/a" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZVCRSSA1H(:,:)    ! "rs/a" on interlayers.
REAL(KIND=JPRB),ALLOCATABLE :: ZVCASRS1F(:,:)    ! "a/rs" on layers.
REAL(KIND=JPRB),ALLOCATABLE :: ZVCASRS1H(:,:)    ! "a/rs" on interlayers.
REAL(KIND=JPRB),ALLOCATABLE :: ZRDLR1(:,:,:)     ! "a/r", "r/a". 
! * rssavnabla(orog) used only for NH model with lgwadv=true or lrdbbc=true:
REAL(KIND=JPRB),ALLOCATABLE :: ZOROGL_SR(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZOROGM_SR(:)
! * surface wind when lgwadv=true or lrdbbc=true (NH):
REAL(KIND=JPRB),ALLOCATABLE :: ZUST1(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZVST1(:)
! * used only for NH model with lgwadv=true:
REAL(KIND=JPRB),ALLOCATABLE :: ZVWVT1(:,:),ZVWVT1S(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSVDT1(:,:)
! * other quantities (declared in alphabetical order).
REAL(KIND=JPRB) :: ZGMDTX(NPROMA)
REAL(KIND=JPRB) :: ZGMDTY(NPROMA)
REAL(KIND=JPRB),ALLOCATABLE :: ZGPPC(:,:)
REAL(KIND=JPRB) :: ZINEZV(NPROMA)
REAL(KIND=JPRB),ALLOCATABLE :: ZP2(:,:),ZQ2(:,:)
REAL(KIND=JPRB) :: ZVWEI(NPROMA,NFLEVG) ! weight for vertical integrations.

REAL(KIND=JPRB) :: ZCOPHI
REAL(KIND=JPRB) :: ZDEPI
REAL(KIND=JPRB) :: ZDGUN
REAL(KIND=JPRB) :: ZDGUX
REAL(KIND=JPRB) :: ZDLUN
REAL(KIND=JPRB) :: ZDLUX
REAL(KIND=JPRB) :: ZDSTRET
REAL(KIND=JPRB) :: ZPHI
REAL(KIND=JPRB) :: ZPP
REAL(KIND=JPRB) :: ZPP1
REAL(KIND=JPRB) :: ZPP9
REAL(KIND=JPRB) :: ZQQ
REAL(KIND=JPRB) :: ZQQ1
REAL(KIND=JPRB) :: ZQQ9
REAL(KIND=JPRB) :: ZSINX
REAL(KIND=JPRB) :: ZTXO
REAL(KIND=JPRB) :: ZTYO
REAL(KIND=JPRB) :: ZCMSLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "elarche.intfb.h"
#include "gnhdlra.intfb.h"
#include "gnhgw2svd.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gpuvs.intfb.h"
#include "gpvcrs.intfb.h"
#include "gpxyb.intfb.h"
#include "lacone.intfb.h"
#include "larche.intfb.h"
#include "larcinb.intfb.h"
#include "larcinhb.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"
#include "verint.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAPINEB',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINAR ALLOCATIONS AND INITIALISATIONS:
!              -------------------------------------------

LLNEED_RS1=LVERCOR.AND.(LADVF.OR.LADVFW)

! ky: if LGWADV.AND.ND4SYS==2.AND.NCURRENT_ITER==NSITER, PGMVT1(.,YT1%MNHX)
!     is updated in CPGLAG with a diagnosed value of NHX, so no need to do here
LLNEED_NHX=(NVDVAR == 4 .AND. &
 & ((.NOT.LGWADV).OR.(LGWADV.AND.ND4SYS==2.AND.NCURRENT_ITER<NSITER)))

! The exact conditions when we actually need to compute r(t+dt) when LRWSDLR=T
! are too tricky to list: we decide to compute it always if LRWSDLR=T.
! We need it at least when: 
!  (LRWSDLW.AND.LRWSDLR).AND.(LADVF.OR.LADVFW)=T
!  LRWSDLR.AND.LGWADV=T (for the surface condition)
LLNEED_R1=LRWSDLR

ZDEPI=2.0_JPRB*RPI

ZDSTRET=2.0_JPRB*RSTRET

IF(LNHDYN) THEN
  ALLOCATE (ZSPD9(NPROMA,NFLEVG))
  ZSPD9(:,:)=0.0_JPRB
  ALLOCATE (ZVWV9(NPROMA,NFLEVG))
  ZVWV9(:,:)=0.0_JPRB
  IF (LLNEED_NHX)THEN
    ALLOCATE (ZNHX9(NPROMA,NFLEVG))
    ZNHX9(:,:)=0.0_JPRB
  ENDIF
  IF (LRDBBC) THEN
    ALLOCATE (ZDBBC9(NPROMA))
    ALLOCATE (ZDPHI9(NPROMA))
    ALLOCATE (ZGWS9(NPROMA))
  ENDIF
ENDIF

IF( LNHDYN.AND.LPC_OLD.AND.(NCURRENT_ITER == 0) )THEN
  ALLOCATE (ZY9(NPROMA,NFLEVG))
  ZY9(:,:) = 0.0_JPRB
ENDIF

IF( LDSLPHY ) THEN
  ALLOCATE (ZUP9(NPROMA,NFLEVG))
  ALLOCATE (ZVP9(NPROMA,NFLEVG))
  ZUP9(:,:)=0.0_JPRB
  ZVP9(:,:)=0.0_JPRB
ENDIF

IF(LSLINL) THEN
  ALLOCATE (ZU9_SI(NPROMA,NFLEVG))
  ALLOCATE (ZV9_SI(NPROMA,NFLEVG))
  ALLOCATE (ZT9_SI(NPROMA,NFLEVG))
  ALLOCATE (ZSVD9_SI(NPROMA,NFLEVG))
  ALLOCATE (ZSPD9_SI(NPROMA,NFLEVG))
ENDIF

IF (LLNEED_RS1) THEN
  ALLOCATE (ZPREHYD1H(NPROMVC,0:NFLEVG))
  ALLOCATE (ZPREHYD1F(NPROMVC,NFLEVG))
  ALLOCATE (ZDELP1(NPROMVC,NFLEVG))
  ALLOCATE (ZRDELP1(NPROMVC,NFLEVG))
  ALLOCATE (ZLNPR1(NPROMVC,NFLEVG))
  ALLOCATE (ZALPH1(NPROMVC,NFLEVG))
  ALLOCATE (ZRTGR1(NPROMVC,NFLEVG))
  ALLOCATE (ZRPRE1(NPROMVC,NFLEVG))
  ALLOCATE (ZRPP1(NPROMVC,NFLEVG))
  ALLOCATE (ZVCRSSA1F(NPROMVC,NFLEVG))
  ALLOCATE (ZVCRSSA1H(NPROMVC,0:NFLEVG))
  ALLOCATE (ZVCASRS1F(NPROMVC,NFLEVG))
  ALLOCATE (ZVCASRS1H(NPROMVC,0:NFLEVG))
ENDIF

IF (LLNEED_R1) ALLOCATE (ZRDLR1(NPROMDLR,0:NFLEVG,JP_DIMR1))

IF (LPC_CHEAP) THEN
  IF (.NOT.ALLOCATED(ZGPPC)) ALLOCATE(ZGPPC(NPROMA,NFGPPC))
  ZGPPC(:,:)=0.0_JPRB
ENDIF

IHVIO=0
DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%CSLINT == 'LAIHVT      ' .OR. &
     & YGFLC(JGFL)%CSLINT == 'LAIHVTQM    ' .OR. &
     & YGFLC(JGFL)%CSLINT == 'LAIHVTQMH   ' ) THEN  
    IHVIO=1
  ENDIF
ENDDO

LL2TLFFO=L2TLFF.AND.(LMPHYS.OR.LEPHYS).AND.(.NOT.LAGPHY)

IF(LRPLANE) THEN
  ZDLUN=REAL(NDLUNG,JPRB)
  ZDLUX=REAL(NDLUXG,JPRB)
  ZDGUN=REAL(NDGUNG,JPRB)
  ZDGUX=REAL(NDGUXG,JPRB)
  DO JROF = KST, KPROF
    ZGMDTX (JROF)=0.5_JPRB*PGM(JROF)*RTDT/EDELX
    ZGMDTY (JROF)=0.5_JPRB*PGM(JROF)*RTDT/EDELY
    ZINEZV(JROF)=MAX(0.0_JPRB,SIGN(1.0_JPRB,(PINDX(JROF)-ZDLUN)*&
     & (ZDLUX-PINDX(JROF))))*&
     & MAX(0.0_JPRB,SIGN(1.0_JPRB,(PINDY(JROF)-ZDGUN)*(ZDGUX-PINDY(JROF))))  
  ENDDO
  ! * Retrieve the origin point coordinates from LAPINEA
  !   for "2 Omega vectorial a k" recalculation.
  PLON(KST:KPROF,:)=PSINLA(KST:KPROF,:)
  PLAT(KST:KPROF,:)=PCOPHI(KST:KPROF,:)
ENDIF

IF (LVERCOR) THEN
  ! * [vertical weight]_l =
  !   {[delta B]_l (rs**2/a**2)_l} / {sum_k=1_to_k=L [delta B]_k (rs**2/a**2)_k}
  DO JLEV=1,NFLEVG
    ZVWEI(KST:KPROF,JLEV)=PB2(KST:KPROF,MSLB2VCVWEI-1+JLEV)
  ENDDO
ELSE
  ! * [vertical weight]_l = [delta B]_l
  DO JLEV=1,NFLEVG
    ZVWEI(KST:KPROF,JLEV)=VDELB(JLEV)
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*       2.    INTERPOLATIONS AT THE ORIGIN POINT AND UPDATE EQNS RHS
!              ------------------------------------------------------

!        2.1   INTERPOLATIONS AND/OR GET/SAVE INTERPOLATED VALUES.

!        2.1.1 INTERPOLATIONS.

IF ( .NOT. (LPC_CHEAP.AND.(NCURRENT_ITER > 0)) ) THEN

  ! * Full level quantities:

  CALL LARCINB(&
   ! --- INPUT ---------------------------------------------------------
   & KST,KPROF,IHVIO,LDSLPHY,LL2TLFFO,&
   & KL0,KLH0,PDLO,PCLO,PCLOSLD,&
   & PDLAT,PCLA,PCLASLD,KLEV,PDVER,PVINTW,PVINTWSLD,PVINTWS,PVDERW,PHVW,PB1,&
   ! --- INPUT/OUTPUT --------------------------------------------------
   & KVSEPC,KVSEPL,&
   ! --- OUTPUT --------------------------------------------------------
   & ZDP9,ZU9,ZV9,ZT9,&
   & ZGFL9,ZC9,ZC9_SI,ZVWV9,ZNHX9,ZSPD9,ZY9,&
   & ZRSA9,&
   & ZUZ9,ZVZ9,&
   & ZUP9,ZVP9,PTP9,PGFLP9,&
   & ZDBBC9,ZDPHI9,ZGWS9,&
   & ZU9_SI,ZV9_SI,ZT9_SI,ZSVD9_SI,ZSPD9_SI)

  ! * Half level quantities:
  IF (LNHDYN.AND.LNH_PDVD.AND.LGWADV.AND.(.NOT.LVFE_GW)) THEN
    ! * Remarks:
    !   - LARCINHB is called only:
    !     - in a finite difference vertical discretisation;
    !     - in a VFE discretisation if (gw) is at half levels and if the
    !       Laplacian term is discretised with FD.
    !     For the other cases of VFE, (gw) is at full levels so the ad-hoc
    !      interpolation should be done under LARCINB.
    CALL LARCINHB(KST,KPROF,&
     & KL0H,PDLOH,PCLOH,PCLOHSLD,PDLATH,PCLAH,PCLAHSLD,&
     & PDVERH,PVINTWH,PVINTWHSLD,PB1,ZVWV9)
  ENDIF

ENDIF

!        2.1.2 SAVE INTERPOLATED VALUES IN GPPCBUF AND ZGPPC.

IF ( LPC_CHEAP.AND.(NCURRENT_ITER == 0) ) THEN

  ! U,V:
  ! - includes rotation matrix.
  ! - if LADVF=T, does not yet include [2 Omega wedge vec(r)] at O
  !   (additional incrementation to be done in part 2.3.2).
  DO JLEV=1,NFLEVG 
    DO JROF=KST,KPROF 
      ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
      ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
      ZGPPC(JROF,MGPPCF_U+JLEV-1)=ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)
      ZGPPC(JROF,MGPPCF_V+JLEV-1)=-ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)
      IF( LDSLPHY ) THEN
        ZGPPC(JROF,MGPPCF_UP+JLEV-1)=ZPP*ZUP9(JROF,JLEV)+ZQQ*ZVP9(JROF,JLEV)
        ZGPPC(JROF,MGPPCF_VP+JLEV-1)=-ZQQ*ZUP9(JROF,JLEV)+ZPP*ZVP9(JROF,JLEV)
      ENDIF
    ENDDO
  ENDDO

  ! T:
  ZGPPC(KST:KPROF,MGPPCF_T:MGPPCF_T+NFLEVG-1)=ZT9(KST:KPROF,1:NFLEVG)
  IF( LDSLPHY ) THEN
    ZGPPC(KST:KPROF,MGPPCF_TP:MGPPCF_TP+NFLEVG-1)=PTP9(KST:KPROF,1:NFLEVG)
  ENDIF

  ! NH variables:
  IF( LNHDYN ) THEN
    ZGPPC(KST:KPROF,MGPPCF_SPD:MGPPCF_SPD+NFLEVG-1)=ZSPD9(KST:KPROF,1:NFLEVG)
    ZGPPC(KST:KPROF,MGPPCF_SVD:MGPPCF_SVD+NFLEVG-1)=ZVWV9(KST:KPROF,1:NFLEVG)
    IF (LLNEED_NHX)THEN
      ZGPPC(KST:KPROF,MGPPCF_NHX:MGPPCF_NHX+NFLEVG-1)=ZNHX9(KST:KPROF,1:NFLEVG)
    ENDIF
    IF( LRDBBC ) THEN
      ZGPPC(KST:KPROF,MGPPCF_BBC)=ZDBBC9(KST:KPROF)
      ZGPPC(KST:KPROF,MGPPCF_BBC+1)=ZDPHI9(KST:KPROF)
      ZGPPC(KST:KPROF,MGPPCF_BBC+2)=ZGWS9(KST:KPROF)
    ENDIF
  ENDIF

  ! Continuity equation:
  ZGPPC(KST:KPROF,MGPPCF_SP:MGPPCF_SP+NFLEVG-1)=ZDP9(KST:KPROF,1:NFLEVG)
  ZGPPC(KST:KPROF,MGPPCF_CP:MGPPCF_CP+NFLEVG-1)=ZC9(KST:KPROF,1:NFLEVG)
  IF (LSLINLC1) THEN
    ZGPPC(KST:KPROF,MGPPCF_CPSI:MGPPCF_CPSI+NFLEVG-1)=ZC9_SI(KST:KPROF,1:NFLEVG)
  ENDIF

  ! GFL:
  IF( LDSLPHY ) THEN
    ! ky: this one must be saved in GFL(PC), not in ZGPPC (change to do later).
    DO JGFL=1,YGFL%NUMFLDS
      ZGPPC(KST:KPROF,MGPPCF_GFLP+(JGFL-1)*NFLEVG:MGPPCF_GFLP+JGFL*NFLEVG-1)=&
       & PGFLP9(KST:KPROF,1:NFLEVG,JGFL)
    ENDDO
  ENDIF
  DO JGFL=1,YGFL%NUMFLDS
    IF ((YGFLC(JGFL)%LADV).AND.(YGFLC(JGFL)%LPC)) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PGFLPC(JROF,JLEV,YGFLC(JGFL)%MPPC)=ZGFL9(JROF,JLEV,JGFL)
        ENDDO
      ENDDO
    ENDIF
  ENDDO

  CALL SC2WRG(GPPCBUF,KSTGLO,KST,KPROF,ZGPPC(1:NPROMA,1:NFGPPC))

ENDIF

!        2.1.3 GET INTERPOLATED VALUES FROM GPPCBUF AND ZGPPC.

IF ( LPC_CHEAP.AND.(NCURRENT_ITER > 0) ) THEN

  ! fill interpolated quantities from buffer
  ! read gmv variables
  CALL SC2RDG(GPPCBUF,KSTGLO,KST,KPROF,ZGPPC(1:NPROMA,1:NFGPPC))

  ! U,V:
  ! - includes rotation matrix.
  ! - if LADVF=T, includes 2 Omega wedge vec(r) at O.
  ZU9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_U:MGPPCF_U+NFLEVG-1)
  ZV9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_V:MGPPCF_V+NFLEVG-1)
  IF( LDSLPHY ) THEN
    ZUP9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_UP:MGPPCF_UP+NFLEVG-1)
    ZVP9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_VP:MGPPCF_VP+NFLEVG-1)
  ENDIF

  ! T:
  ZT9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_T:MGPPCF_T+NFLEVG-1)
  IF( LDSLPHY ) THEN
    PTP9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_TP:MGPPCF_TP+NFLEVG-1)
  ENDIF

  ! NH variables:
  IF( LNHDYN ) THEN
    ZSPD9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_SPD:MGPPCF_SPD+NFLEVG-1)
    ZVWV9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_SVD:MGPPCF_SVD+NFLEVG-1)
    IF (LLNEED_NHX)THEN
      ZNHX9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_NHX:MGPPCF_NHX+NFLEVG-1)
    ENDIF
    IF( LRDBBC ) THEN
      ZDBBC9(KST:KPROF)=ZGPPC(KST:KPROF,MGPPCF_BBC)
      ZDPHI9(KST:KPROF)=ZGPPC(KST:KPROF,MGPPCF_BBC+1)
      ZGWS9(KST:KPROF)=ZGPPC(KST:KPROF,MGPPCF_BBC+2)
    ENDIF
  ENDIF

  ! Continuity equation:
  ZDP9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_SP:MGPPCF_SP+NFLEVG-1)
  ZC9(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_CP:MGPPCF_CP+NFLEVG-1)
  IF (LSLINLC1) THEN
    ZC9_SI(KST:KPROF,1:NFLEVG)=ZGPPC(KST:KPROF,MGPPCF_CPSI:MGPPCF_CPSI+NFLEVG-1)
  ENDIF

  ! GFL:
  IF( LDSLPHY ) THEN
    DO JGFL=1,YGFL%NUMFLDS
      PGFLP9(KST:KPROF,1:NFLEVG,JGFL)=&
       & ZGPPC(KST:KPROF,MGPPCF_GFLP+(JGFL-1)*NFLEVG:MGPPCF_GFLP+JGFL*NFLEVG-1)
    ENDDO
  ENDIF
  DO JGFL=1,YGFL%NUMFLDS
    IF ((YGFLC(JGFL)%LADV).AND.(YGFLC(JGFL)%LPC)) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          ZGFL9(JROF,JLEV,JGFL)=PGFLPC(JROF,JLEV,YGFLC(JGFL)%MPPC)
        ENDDO
      ENDDO
    ENDIF
  ENDDO

ENDIF

!*       2.2   ADD QUANTITIES INTERPOLATED AT THE ORIGIN POINT
!              TO ARRAYS P(X)T1 FOR CONTINUITY EQUATION.

!        2.2.1 ADD INTERPOLATED TERMS.

IF (NVLAG == 2) THEN
  IF(LVERTFE) THEN
    CALL ABOR1(' LAPINEB 2.2.1: LVERTFE=.T. NOT COMPATIBLE WITH NVLAG == 2')
  ENDIF

  DO JLEV=1,NFLEVG
    PGMVT1S(KST:KPROF,YT1%MSP)=PGMVT1S(KST:KPROF,YT1%MSP)+ &
     & ZVWEI(KST:KPROF,JLEV)*(ZDP9(KST:KPROF,JLEV)+ZC9(KST:KPROF,JLEV))
  ENDDO

ELSEIF (NVLAG == 3) THEN

  IF(LVERTFE) THEN
    DO JLEV=1,NFLEVG
      ZSPT1(KST:KPROF,JLEV)=ZVWEI(KST:KPROF,JLEV)*VFE_RDETAH(JLEV)&
       & *(ZDP9(KST:KPROF,JLEV)+ZC9(KST:KPROF,JLEV))  
    ENDDO
    CALL VERINT(NPROMA,KST,KPROF,NFLEVG,ZSPT1,ZSPTO,0)
    PGMVT1S(KST:KPROF,YT1%MSP)=PGMVT1S(KST:KPROF,YT1%MSP) &
     & +ZSPTO(KST:KPROF,NFLEVG+1)
    IF (LSLINLC1.AND.(.NOT.LSLINLC2)) THEN
      ! caution: for LSLINLC1.AND.LSLINLC2, PGMVT1S should not be incremented
      !  with any linear term (in this case the linear term is added to PB2
      !  (see below) and PB2 is added to the RHS of continuity equation in
      !  GPENDTR).
      DO JLEV=1,NFLEVG
        PGMVT1S(KST:KPROF,YT1%MSP)=PGMVT1S(KST:KPROF,YT1%MSP) &
         & +ZVWEI(KST:KPROF,JLEV)*ZC9_SI(KST:KPROF,JLEV)
      ENDDO
    ENDIF
  ELSE
    DO JLEV=1,NFLEVG
      PGMVT1S(KST:KPROF,YT1%MSP)=PGMVT1S(KST:KPROF,YT1%MSP)+ &
       &ZVWEI(KST:KPROF,JLEV)*(ZDP9(KST:KPROF,JLEV)+ZC9(KST:KPROF,JLEV))  
    ENDDO
  ENDIF

  LLSLINLC2=LSLINLC2.AND.NGWADVSI==1.AND.(NCURRENT_ITER == 0)
  IF (LLSLINLC2.AND. .NOT.(LVERTFE.AND..NOT.LSLINLC1)) THEN
    ! Add the linear term, FD vertically integrated, to PB2.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI) &
         & +ZVWEI(JROF,JLEV)*ZC9_SI(JROF,JLEV)
      ENDDO
    ENDDO
  ELSEIF (LLSLINLC2.AND.(LVERTFE.AND..NOT.LSLINLC1)) THEN
    ! Add the linear term, VFE vertically integrated, to PB2.
    DO JLEV=1,NFLEVG
      ZSPT1(KST:KPROF,JLEV)=ZVWEI(KST:KPROF,JLEV)*VFE_RDETAH(JLEV)&
       & *ZC9_SI(KST:KPROF,JLEV)
    ENDDO
    CALL VERINT(NPROMA,KST,KPROF,NFLEVG,ZSPT1,ZSPTO,0)
    PB2(KST:KPROF,MSLB2SPSI)=PB2(KST:KPROF,MSLB2SPSI)+ZSPTO(KST:KPROF,NFLEVG+1)
  ENDIF

ENDIF

!        2.2.2 COMPUTE "rs/a" AT t+dt (LVERCOR).

IF (LLNEED_RS1) THEN
  ! * compute "prehyds"(t+dt):
  DO JROF=KST,KPROF
    ZPREHYD1H(JROF,NFLEVG)=EXP(PGMVT1S(JROF,YT1%MSP))
  ENDDO
  ! * compute "prehyd"(t+dt) on interlayers:
  CALL GPPREH(NPROMVC,KST,KPROF,NFLEVG,VAH,VBH,ZPREHYD1H)
  ! * compute "prehyd"(t+dt) on layers:
  CALL GPXYB(NPROMVC,KST,KPROF,NFLEVG,VDELB,VC,&
   & ZPREHYD1H,ZDELP1(1,1),ZRDELP1,ZLNPR1,ZALPH1,ZRTGR1,ZRPRE1,ZRPP1)  
  CALL GPPREF(NPROMVC,KST,KPROF,NFLEVG,ZALPH1,ZPREHYD1H,ZPREHYD1F)
  ! * compute "rs/a"(t+dt) on layers:
  !   (actually only 'ZVCRSSA1F' and 'ZVCASRS1F' are useful).
  CALL GPVCRS(NPROMVC,KST,KPROF,NFLEVG,&
   & ZPREHYD1F,ZPREHYD1H,ZVCRSSA1F,ZVCRSSA1H,ZVCASRS1F,ZVCASRS1H)
ENDIF

!*       2.3   ADD QUANTITIES INTERPOLATED AT THE ORIGIN
!              POINTS TO ARRAYS P(X)T1 FOR EQUATIONS OTHER THAN
!              CONTINUITY EQUATION.

!        2.3.1a ADD GENERAL CONTRIBUTIONS IN P[X]T1.

! * Momentum equation.
IF( LPC_CHEAP.AND.(NCURRENT_ITER > 0) ) THEN
  ! Rotation (p,q) has already been taken into account in (ZU..9,ZV..9).
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)+ZU9(JROF,JLEV)
      PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)+ZV9(JROF,JLEV)
    ENDDO
  ENDDO
  IF( LDSLPHY ) THEN
    PUP9(KST:KPROF,1:NFLEVG)=ZUP9(KST:KPROF,1:NFLEVG)
    PVP9(KST:KPROF,1:NFLEVG)=ZVP9(KST:KPROF,1:NFLEVG)
  ENDIF
ELSE
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
      ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
      PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU) &
       & +ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)
      PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV) &
       & -ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)
      IF( LDSLPHY ) THEN
        PUP9(JROF,JLEV)=ZPP*ZUP9(JROF,JLEV)+ZQQ*ZVP9(JROF,JLEV)
        PVP9(JROF,JLEV)=-ZQQ*ZUP9(JROF,JLEV)+ZPP*ZVP9(JROF,JLEV)
      ENDIF
    ENDDO
  ENDDO
  IF (LSLINL.AND.NGWADVSI==1.AND.NCURRENT_ITER==0) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        PB2(JROF,MSLB2USI-1+JLEV)=PB2(JROF,MSLB2USI-1+JLEV) &
         & +ZPP*ZU9_SI(JROF,JLEV)+ZQQ*ZV9_SI(JROF,JLEV)
        PB2(JROF,MSLB2VSI-1+JLEV)=PB2(JROF,MSLB2VSI-1+JLEV) &
         & -ZQQ*ZU9_SI(JROF,JLEV)+ZPP*ZV9_SI(JROF,JLEV)
      ENDDO
    ENDDO
  ELSEIF (LSLINL.AND.NGWADVSI==2.AND.NCURRENT_ITER==0) THEN
    ! Simply save the version of ZPP,ZQQ not modified by L2TLFF
    ALLOCATE (ZP2(NPROMA,NFLEVG))
    ALLOCATE (ZQ2(NPROMA,NFLEVG))
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZP2(JROF,JLEV)=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQ2(JROF,JLEV)=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
      ENDDO
    ENDDO
  ENDIF
ENDIF

! * Temperature equation.
PGMVT1(KST:KPROF,1:NFLEVG,YT1%MT)=PGMVT1(KST:KPROF,1:NFLEVG,YT1%MT) &
 & +ZT9(KST:KPROF,1:NFLEVG)  
IF (LSLINL.AND.NGWADVSI==1.AND.NCURRENT_ITER==0) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PB2(JROF,MSLB2TSI-1+JLEV)=PB2(JROF,MSLB2TSI-1+JLEV)+ZT9_SI(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF
 
! * GFL variables equations.
DO JGFL=1,YGFL%NUMFLDS
  IF (LSLDIA .AND. &
   & (YGFLC(JGFL)%CNAME == 'DPDETA0         ') .AND. YGFLC(JGFL)%LGP) THEN
    ! SL dynamics diagnostics
    ZCMSLP=RCMSLP0/(RD*RTSUR)
    IF (LVERTFE) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)=VDELA(JLEV) & 
           & + EXP(ZDP9(JROF,JLEV)-ZCMSLP*POROG(JROF))*VDELB(JLEV)
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)=(VAH(JLEV)-VAH(JLEV-1)) & 
           & + EXP(ZDP9(JROF,JLEV)-ZCMSLP*POROG(JROF))*VDELB(JLEV)
        ENDDO
      ENDDO
    ENDIF    
  ENDIF
  IF (YGFLC(JGFL)%LADV) THEN
    IF ((YGFLC(JGFL)%CNAME == 'CVGQ            ') .AND. YGFLC(JGFL)%LGP) THEN
      ! Computation of SL Moisture Convergence for MF physics
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PGFLT1(JROF,JLEV,YGFLC(JGFL)%MP1)=&
           &ZGFL9(JROF,JLEV,JGFL) - PQ(JROF,JLEV)
        ENDDO
      ENDDO
    ELSE     
      PGFLT1(KST:KPROF,1:NFLEVG,YGFLC(JGFL)%MP1)=&
       & PGFLT1(KST:KPROF,1:NFLEVG,YGFLC(JGFL)%MP1)+&
       & ZGFL9(KST:KPROF,1:NFLEVG,JGFL)  
    ENDIF
  ENDIF
ENDDO

! * Computation of spectral Moisture Convergence for French physics
IF (YCVGQ%LSP) THEN
  PGFLT1(KST:KPROF,1:NFLEVG,YCVGQ%MP1)=PGFLT1(KST:KPROF,1:NFLEVG,YQ%MP1)
ENDIF   

! * NH variables equations.
IF(LNHDYN) THEN
  ! PGMVT1(.,.,YT1%MSVD): half levels lbar=0 to nflevg-1
  !  if LGWADV.AND.(.NOT.LVFE_GW); full levels otherwise.
  PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSVD)=PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSVD) &
   & +ZVWV9(KST:KPROF,1:NFLEVG)  
  IF (LLNEED_NHX) THEN
    PGMVT1(KST:KPROF,1:NFLEVG,YT1%MNHX)=PGMVT1(KST:KPROF,1:NFLEVG,YT1%MNHX) &
     & +ZNHX9(KST:KPROF,1:NFLEVG)  
  ENDIF
  PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSPD)=PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSPD) &
   & +ZSPD9(KST:KPROF,1:NFLEVG)  
  IF (LSLINL.AND.NGWADVSI==1.AND.NCURRENT_ITER==0) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB2(JROF,MSLB2VDSI-1+JLEV)=PB2(JROF,MSLB2VDSI-1+JLEV)&
         & +ZSVD9_SI(JROF,JLEV)
        PB2(JROF,MSLB2PDSI-1+JLEV)=PB2(JROF,MSLB2PDSI-1+JLEV)&
         & +ZSPD9_SI(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!        2.3.1b ADD GENERAL CONTRIBUTIONS IN "YT1" PART OF PB2.

IF(LNHDYN.AND.LPC_OLD.AND.(NCURRENT_ITER == 0))THEN
  DO JLEV=1,NFLEVG
    PB2(KST:KPROF,MSLB2Y1-1+JLEV)=&
     & PB2(KST:KPROF,MSLB2Y1-1+JLEV)+ZY9(KST:KPROF,JLEV)  
  ENDDO
ENDIF

!        2.3.1c COMPUTE "r/a" AT t+dt (LRWSDLR).

IF (LLNEED_R1) THEN
  IDIMGFL=YGFL%NDIM1
  CALL GNHDLRA(NPROMDLR,IDIMGFL,KST,KPROF,1,.FALSE.,.FALSE.,POROG, &
   & PGMVT1S(1,YT1%MSP),PGMVT1(1,1,YT1%MT),PGMVT1(1,1,YT1%MSPD),ZRDLR1, &
   & PGFL=PGFLT1)
ENDIF

!        2.3.2 SPECIAL TREATMENT OF CORIOLIS TERM:

! For the corrector step of LPC_CHEAP nothing has to be done here
!  because term [2 Omega wedge vec(r)] at O has already been taken
!  into account in the calculations done in part 2.1.3.

IF ( .NOT. (LPC_CHEAP.AND.(NCURRENT_ITER > 0)) ) THEN

  LLDL=(LVERCOR.OR.(LRWSDLW.AND.LRWSDLR))
  LLCOMPUTE_OMVAK_O= &
   & (((.NOT.LLDL).AND.LADVF).OR.(LLDL.AND.(LADVF.OR.LADVFW)))  

  IF (LLCOMPUTE_OMVAK_O) THEN
    ! * Analytical computation of "2 Omega wedge a k" at "O":
    IF (LRPLANE) THEN
      ! * calculation of LACONE done already in ELARCHE -> results are in
      !   zlon, zlat tables.
      ZU9(KST:KPROF,1:NFLEVG)=PLON(KST:KPROF,1:NFLEVG)
      ZV9(KST:KPROF,1:NFLEVG)=PLAT(KST:KPROF,1:NFLEVG)
    ELSE
      CALL LACONE(NPROMA,KST,KPROF,NFLEVG,NSTTYP,&
       & ROMEGA,ZDSTRET,RC2M1,RC2P1,RMUCEN,RA,RPI,&
       & PLON,PLAT,ZU9,ZV9)  
    ENDIF
  ENDIF

  IF ((LVERCOR.OR.(LRWSDLW.AND.LRWSDLR)).AND.LADVFW) THEN
    IF (LADVF) THEN
      ! * Replace "2 Omega wedge a k" by "2 Omega wedge rs k".
      ZU9(KST:KPROF,1:NFLEVG)= &
       & ZU9(KST:KPROF,1:NFLEVG)*ZRSA9(KST:KPROF,1:NFLEVG)  
      ZV9(KST:KPROF,1:NFLEVG)= &
       & ZV9(KST:KPROF,1:NFLEVG)*ZRSA9(KST:KPROF,1:NFLEVG)  
    ELSE
      ! * Replace "2 Omega wedge a k" by "2 Omega wedge (rs-a) k".
      ZU9(KST:KPROF,1:NFLEVG)= &
       & ZU9(KST:KPROF,1:NFLEVG)*(ZRSA9(KST:KPROF,1:NFLEVG)-1.0_JPRB)  
      ZV9(KST:KPROF,1:NFLEVG)= &
       & ZV9(KST:KPROF,1:NFLEVG)*(ZRSA9(KST:KPROF,1:NFLEVG)-1.0_JPRB)  
    ENDIF
  ENDIF

  IF (LLCOMPUTE_OMVAK_O) THEN
    ! * Add the content of "pu9,pv9" to "(U,V)(t+dt)":
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU) &
         & +ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)
        PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV) &
         & -ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

  IF ( LLCOMPUTE_OMVAK_O .AND. (.NOT.L2TLFF) &
   & .AND. LPC_CHEAP.AND.(NCURRENT_ITER == 0) ) THEN
    ! ZGPPC(wind) must be incremented with term [2 Omega wedge vec(r)] at O.
    !  If L2TLFF=T, the incrementation is not done here but in part 2.3.4
    !  in order to take account of the recomputed version of O.
    DO JLEV=1,NFLEVG 
      DO JROF=KST,KPROF 
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        ZGPPC(JROF,MGPPCF_U+JLEV-1)=ZGPPC(JROF,MGPPCF_U+JLEV-1) &
         & +ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)
        ZGPPC(JROF,MGPPCF_V+JLEV-1)=ZGPPC(JROF,MGPPCF_V+JLEV-1) &
         & -ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)
      ENDDO
    ENDDO
    CALL SC2WRG(GPPCBUF,KSTGLO,KST,KPROF,ZGPPC(1:NPROMA,1:NFGPPC))
  ENDIF

ENDIF

!        2.3.3 ADD CORIOLIS TERM AT "F" WHEN NOT DONE IN ROUTINE "LATTEX"

! * Involves the cases of lvercor=.true. when "rs/a" is needed at t+dt.
!   In this case work cannot be done in routine "LATTEX" because
!   at this stage of calculations "rs/a"(t+dt) is not yet available.
IF (LVERCOR.AND.LADVFW) THEN
  IF (LADVF) THEN
    ! * Substract "2 Omega wedge rs k" at F and t+dt
    !   to obtain (U,V)(t+dt) from ((U,V)+2 Omega wedge rs k)(t+dt):
    DO JLEV=1,NFLEVG
      PGMVT1(KST:KPROF,JLEV,YT1%MU)=PGMVT1(KST:KPROF,JLEV,YT1%MU) &
       & -POMVRL(KST:KPROF)*ZVCRSSA1F(KST:KPROF,JLEV)  
      PGMVT1(KST:KPROF,JLEV,YT1%MV)=PGMVT1(KST:KPROF,JLEV,YT1%MV) &
       & -POMVRM(KST:KPROF)*ZVCRSSA1F(KST:KPROF,JLEV)  
    ENDDO
  ELSE
    ! * Substract "2 Omega wedge (rs-a) k" at F and t+dt
    !   to obtain (U,V)(t+dt) from ((U,V)+2 Omega wedge (rs-a) k)(t+dt):
    DO JLEV=1,NFLEVG
      PGMVT1(KST:KPROF,JLEV,YT1%MU)=PGMVT1(KST:KPROF,JLEV,YT1%MU) &
       & -POMVRL(KST:KPROF)*(ZVCRSSA1F(KST:KPROF,JLEV)-1.0_JPRB)  
      PGMVT1(KST:KPROF,JLEV,YT1%MV)=PGMVT1(KST:KPROF,JLEV,YT1%MV) &
       & -POMVRM(KST:KPROF)*(ZVCRSSA1F(KST:KPROF,JLEV)-1.0_JPRB)  
    ENDDO
  ENDIF
ENDIF

! * Involves the cases of LRWSDLW.AND.LRWSDLR=.true. when "r/a" is needed at
!   t+dt.
!   In this case work cannot be done in routine "LATTEX" because
!   at this stage of calculations "r/a"(t+dt) is not yet available.
IF ((LRWSDLW.AND.LRWSDLR).AND.LADVFW) THEN
  IF (LADVF) THEN
    ! * Substract "2 Omega wedge r k" at F and t+dt
    !   to obtain (U,V)(t+dt) from ((U,V)+2 Omega wedge r k)(t+dt):
    DO JLEV=1,NFLEVG
      PGMVT1(KST:KPROF,JLEV,YT1%MU)=PGMVT1(KST:KPROF,JLEV,YT1%MU) &
       & -POMVRL(KST:KPROF)*ZRDLR1(KST:KPROF,JLEV,JP_RSAF)  
      PGMVT1(KST:KPROF,JLEV,YT1%MV)=PGMVT1(KST:KPROF,JLEV,YT1%MV) &
       & -POMVRM(KST:KPROF)*ZRDLR1(KST:KPROF,JLEV,JP_RSAF)
    ENDDO
  ELSE
    ! * Substract "2 Omega wedge (r-a) k" at F and t+dt
    !   to obtain (U,V)(t+dt) from ((U,V)+2 Omega wedge (r-a) k)(t+dt):
    DO JLEV=1,NFLEVG
      PGMVT1(KST:KPROF,JLEV,YT1%MU)=PGMVT1(KST:KPROF,JLEV,YT1%MU) &
       & -POMVRL(KST:KPROF)*(ZRDLR1(KST:KPROF,JLEV,JP_RSAF)-1.0_JPRB)  
      PGMVT1(KST:KPROF,JLEV,YT1%MV)=PGMVT1(KST:KPROF,JLEV,YT1%MV) &
       & -POMVRM(KST:KPROF)*(ZRDLR1(KST:KPROF,JLEV,JP_RSAF)-1.0_JPRB)  
    ENDDO
  ENDIF
ENDIF

!        2.3.4 SPECIAL TREATMENT OF CORIOLIS TERM:
!              CASE "L2TLFF=T".

! * Refined calculation of the origin point.
!   Refined treatment of Coriolis term in 2TL scheme (or in 3TL SLI scheme).
!   Works for the following configurations:
!    - SL3TL or SL2TL.
!    - LADVF=.T. or LADVFW=.T.
!    - NWLAG=3.
!   If LPC_CHEAP=T recalculation of O is done in the predictor step only.

IF ( L2TLFF .AND. .NOT.(LPC_CHEAP.AND.(NCURRENT_ITER > 0)) ) THEN

  IF (LRPLANE) THEN

    ! * compute new origin point:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ! * old rotation and vector deplacement:
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        ! * new estimate of trajectory (a/rs)*wind:
        !   at this stage (pgmvt1(U,V)) contains wind;
        !   (puf,pvf) and (zuz9,zvz9) contain (a/rs)*wind.
        IF (LVERCOR) THEN
          ZUT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*&
           & (ZVCASRS1F(JROF,JLEV)*PGMVT1(JROF,JLEV,YT1%MU) + ZUZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*PUF(JROF,JLEV)  
          ZVT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*&
           & (ZVCASRS1F(JROF,JLEV)*PGMVT1(JROF,JLEV,YT1%MV) + ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*PVF(JROF,JLEV)  
        ELSEIF(LRWSDLR) THEN
          ZUT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*&
           & (ZRDLR1(JROF,JLEV,JP_ASRF)*PGMVT1(JROF,JLEV,YT1%MU) &
           & + ZUZ9(JROF,JLEV))+(1.0_JPRB-RW2TLFF)*PUF(JROF,JLEV)  
          ZVT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*&
           & (ZRDLR1(JROF,JLEV,JP_ASRF)*PGMVT1(JROF,JLEV,YT1%MV) &
           & + ZVZ9(JROF,JLEV))+(1.0_JPRB-RW2TLFF)*PVF(JROF,JLEV)  
        ELSE
          ZUT1(JROF,JLEV)=(1.0_JPRB-RW2TLFF)*PUF(JROF,JLEV) &
           & +RW2TLFF*0.5_JPRB*(PGMVT1(JROF,JLEV,YT1%MU) + ZUZ9(JROF,JLEV))
          ZVT1(JROF,JLEV)=(1.0_JPRB-RW2TLFF)*PVF(JROF,JLEV) &
           & +RW2TLFF*0.5_JPRB*(PGMVT1(JROF,JLEV,YT1%MV) + ZVZ9(JROF,JLEV))
        ENDIF
        ! * subtract old estimate of advected Coriolis term:
        PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)&
         & -ZPP*ZU9(JROF,JLEV)-ZQQ*ZV9(JROF,JLEV)  
        PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)&
         & +ZQQ*ZU9(JROF,JLEV)-ZPP*ZV9(JROF,JLEV)  
        ! * compute the new origin point ON:
        ZTXO=PINDX(JROF)-2.0_JPRB*ZUT1(JROF,JLEV)*ZGMDTX(JROF)
        ZTYO=PINDY(JROF)-2.0_JPRB*ZVT1(JROF,JLEV)*ZGMDTY(JROF)
        PLON(JROF,JLEV)=ZTXO*ZINEZV(JROF)+PINDX(JROF)*(1.0_JPRB-ZINEZV(JROF))
        PLAT(JROF,JLEV)=ZTYO*ZINEZV(JROF)+PINDY(JROF)*(1.0_JPRB-ZINEZV(JROF))
        ZTXO=MIN(MAX(ZTXO,ZDLUN),ZDLUX)
        ZTYO=MIN(MAX(ZTYO,ZDGUN),ZDGUX)
        PCOSCO(JROF,JLEV)=ZTXO*ZINEZV(JROF)+PINDX(JROF)*(1.0_JPRB-ZINEZV(JROF))
        PSINCO(JROF,JLEV)=ZTYO*ZINEZV(JROF)+PINDY(JROF)*(1.0_JPRB-ZINEZV(JROF))
      ENDDO
    ENDDO
    ! * compute rotation matrix and Coriolis term "2 Omega wedge a k"
    !   at the new origin point ON.
    CALL ELARCHE(NPROMA,KST,KPROF,NFLEVG,&
     & PGEMU,PGSQM2,PGECLO,PGESLO,&
     & PCOSCO,PSINCO,PQX,PQY,PLON,PLAT)  
    ZU9(KST:KPROF,1:NFLEVG)=PLON(KST:KPROF,1:NFLEVG)
    ZV9(KST:KPROF,1:NFLEVG)=PLAT(KST:KPROF,1:NFLEVG)
    IF ((LVERCOR.OR.(LRWSDLW.AND.LRWSDLR)).AND.LADVFW) THEN
      IF (LADVF) THEN
        ! * multiply by (rs/a) at O to obtain "(rs/a)_O (2 Omega wedge a k)_ON".
        ZU9(KST:KPROF,1:NFLEVG)=&
         & ZU9(KST:KPROF,1:NFLEVG)*ZRSA9(KST:KPROF,1:NFLEVG)  
        ZV9(KST:KPROF,1:NFLEVG)=&
         & ZV9(KST:KPROF,1:NFLEVG)*ZRSA9(KST:KPROF,1:NFLEVG)  
      ELSE
        ! * multiply by (rs/a - 1) at O
        !   to obtain "(rs/a - 1)_O (2 Omega wedge a k)_ON".
        ZU9(KST:KPROF,1:NFLEVG)=ZU9(KST:KPROF,1:NFLEVG) &
         & *(ZRSA9(KST:KPROF,1:NFLEVG)-1.0_JPRB)
        ZV9(KST:KPROF,1:NFLEVG)=ZV9(KST:KPROF,1:NFLEVG) &
         & *(ZRSA9(KST:KPROF,1:NFLEVG)-1.0_JPRB)
      ENDIF
    ENDIF
    ! * add new estimate of advected Coriolis term:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)&
         & +ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)  
        PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)&
         & -ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)  
      ENDDO
    ENDDO

  ELSE

    ! * compute new origin point:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ! * old rotation and vector deplacement:
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        ZPP9=PQX(JROF,JLEV)
        ZQQ9=PQY(JROF,JLEV)
        ZPP1=PGNORDM(JROF)
        ZQQ1=-PGNORDL(JROF)
        ! * new estimate of trajectory (a/rs)*wind:
        !   at this stage (pgmvt1(U,V)) contains wind;
        !   (puf,pvf) and (zuz9,zvz9) contain (a/rs)*wind.
        IF (LVERCOR) THEN
          ZUT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*(&
           & ZPP1*ZVCASRS1F(JROF,JLEV)*PGMVT1(JROF,JLEV,YT1%MU)&
           & +ZQQ1*ZVCASRS1F(JROF,JLEV)*PGMVT1(JROF,JLEV,YT1%MV)&
           & +ZPP9*ZUZ9(JROF,JLEV)+ZQQ9*ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*(ZPP1*PUF(JROF,JLEV)+ZQQ1*PVF(JROF,JLEV))  
          ZVT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*(&
           & -ZQQ1*ZVCASRS1F(JROF,JLEV)*PGMVT1(JROF,JLEV,YT1%MU)&
           & +ZPP1*ZVCASRS1F(JROF,JLEV)*PGMVT1(JROF,JLEV,YT1%MV)&
           & -ZQQ9*ZUZ9(JROF,JLEV)+ZPP9*ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*(-ZQQ1*PUF(JROF,JLEV)+ZPP1*PVF(JROF,JLEV))  
        ELSEIF (LRWSDLR) THEN
          ZUT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*(&
           & ZPP1*ZRDLR1(JROF,JLEV,JP_ASRF)*PGMVT1(JROF,JLEV,YT1%MU)&
           & +ZQQ1*ZRDLR1(JROF,JLEV,JP_ASRF)*PGMVT1(JROF,JLEV,YT1%MV)&
           & +ZPP9*ZUZ9(JROF,JLEV)+ZQQ9*ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*(ZPP1*PUF(JROF,JLEV)+ZQQ1*PVF(JROF,JLEV))  
          ZVT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*(&
           & -ZQQ1*ZRDLR1(JROF,JLEV,JP_ASRF)*PGMVT1(JROF,JLEV,YT1%MU)&
           & +ZPP1*ZRDLR1(JROF,JLEV,JP_ASRF)*PGMVT1(JROF,JLEV,YT1%MV)&
           & -ZQQ9*ZUZ9(JROF,JLEV)+ZPP9*ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*(-ZQQ1*PUF(JROF,JLEV)+ZPP1*PVF(JROF,JLEV))  
        ELSE
          ZUT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*(&
           & ZPP1*PGMVT1(JROF,JLEV,YT1%MU)+ZQQ1*PGMVT1(JROF,JLEV,YT1%MV)&
           & +ZPP9*ZUZ9(JROF,JLEV)+ZQQ9*ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*(ZPP1*PUF(JROF,JLEV)+ZQQ1*PVF(JROF,JLEV))  
          ZVT1(JROF,JLEV)=RW2TLFF*0.5_JPRB*(&
           & -ZQQ1*PGMVT1(JROF,JLEV,YT1%MU)+ZPP1*PGMVT1(JROF,JLEV,YT1%MV)&
           & -ZQQ9*ZUZ9(JROF,JLEV)+ZPP9*ZVZ9(JROF,JLEV))&
           & +(1.0_JPRB-RW2TLFF)*(-ZQQ1*PUF(JROF,JLEV)+ZPP1*PVF(JROF,JLEV))  
        ENDIF
        ! * subtract old estimate of advected Coriolis term:
        PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)&
         & -ZPP*ZU9(JROF,JLEV)-ZQQ*ZV9(JROF,JLEV)  
        PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)&
         & +ZQQ*ZU9(JROF,JLEV)-ZPP*ZV9(JROF,JLEV)  
        ! * compute the new origin point ON:
        ZPHI=0.5_JPRB*RTDT*SQRT(ZUT1(JROF,JLEV)*ZUT1(JROF,JLEV)&
         & +ZVT1(JROF,JLEV)*ZVT1(JROF,JLEV))/RA  
        ZCOPHI=1.0_JPRB-0.5_JPRB*ZPHI*ZPHI
        ZSINX=ZCOPHI*RTDT*(1.0_JPRB-ZPHI*ZPHI/6.0_JPRB)/RA
        PSINLA(JROF,JLEV)=PGEMU(JROF)*(2.0_JPRB*ZCOPHI*ZCOPHI-1.0_JPRB)&
         & -ZVT1(JROF,JLEV)*PGSQM2(JROF)*ZSINX  
        PCOSCO(JROF,JLEV)=PGSQM2(JROF)*(2.0_JPRB*ZCOPHI*ZCOPHI-1.0_JPRB)&
         & +ZVT1(JROF,JLEV)*PGEMU(JROF)*ZSINX  
        PSINCO(JROF,JLEV)=-ZUT1(JROF,JLEV)*ZSINX
        PCOPHI(JROF,JLEV)=2.0_JPRB*ZCOPHI*ZCOPHI-1.0_JPRB
      ENDDO
    ENDDO
    ! * compute rotation matrix and Coriolis term "2 Omega wedge a k"
    !   at the new origin point ON.
    CALL LARCHE(NPROMA,KST,KPROF,NFLEVG,NSTTYP,ZDSTRET,&
     & RC2M1,RC2P1,RPI,ZDEPI,RLOCEN,RMUCEN,&
     & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
     & PCOSCO,PSINCO,PSINLA,PCOPHI,1,PLON,PLAT,PQX,PQY)  
    CALL LACONE(NPROMA,KST,KPROF,NFLEVG,NSTTYP,&
     & ROMEGA,ZDSTRET,RC2M1,RC2P1,RMUCEN,RA,RPI,&
     & PLON,PLAT,ZU9,ZV9)  
    IF ((LVERCOR.OR.(LRWSDLW.AND.LRWSDLR)).AND.LADVFW) THEN
      IF (LADVF) THEN
        ! * multiply by (rs/a) at O to obtain "(rs/a)_O (2 Omega wedge a k)_ON".
        ZU9(KST:KPROF,1:NFLEVG)=&
         & ZU9(KST:KPROF,1:NFLEVG)*ZRSA9(KST:KPROF,1:NFLEVG)  
        ZV9(KST:KPROF,1:NFLEVG)=&
         & ZV9(KST:KPROF,1:NFLEVG)*ZRSA9(KST:KPROF,1:NFLEVG)  
      ELSE
        ! * multiply by (rs/a - 1) at O
        !   to obtain "(rs/a - 1)_O (2 Omega wedge a k)_ON".
        ZU9(KST:KPROF,1:NFLEVG)=ZU9(KST:KPROF,1:NFLEVG) &
         & *(ZRSA9(KST:KPROF,1:NFLEVG)-1.0_JPRB)
        ZV9(KST:KPROF,1:NFLEVG)=ZV9(KST:KPROF,1:NFLEVG) &
         & *(ZRSA9(KST:KPROF,1:NFLEVG)-1.0_JPRB)
      ENDIF
    ENDIF
    ! * add new estimate of advected Coriolis term:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU)&
         & +ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)  
        PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV)&
         & -ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF ( LPC_CHEAP.AND.(NCURRENT_ITER == 0) ) THEN
    ! ZGPPC(wind) must be incremented with term [2 Omega wedge vec(r)] at O.
    DO JLEV=1,NFLEVG 
      DO JROF=KST,KPROF 
        ZPP=PQX(JROF,JLEV)*PGNORDM(JROF)-PQY(JROF,JLEV)*PGNORDL(JROF)
        ZQQ=PQY(JROF,JLEV)*PGNORDM(JROF)+PQX(JROF,JLEV)*PGNORDL(JROF)
        ZGPPC(JROF,MGPPCF_U+JLEV-1)=ZGPPC(JROF,MGPPCF_U+JLEV-1) &
         & +ZPP*ZU9(JROF,JLEV)+ZQQ*ZV9(JROF,JLEV)
        ZGPPC(JROF,MGPPCF_V+JLEV-1)=ZGPPC(JROF,MGPPCF_V+JLEV-1) &
         & -ZQQ*ZU9(JROF,JLEV)+ZPP*ZV9(JROF,JLEV)
      ENDDO
    ENDDO
    CALL SC2WRG(GPPCBUF,KSTGLO,KST,KPROF,ZGPPC(1:NPROMA,1:NFGPPC))
  ENDIF

ENDIF

!        2.3.5 CONVERT "gw" TOWARDS "d-hat"
!              + diagnostic BBC in NH (LNH_PDVD=T only).

IF(LGWADV.OR.LRDBBC) THEN
  ! * allocate arrays for storing rssavnabla(orog)=M*vnabla'(orog).
  ALLOCATE(ZOROGL_SR(NPROMA)) 
  ALLOCATE(ZOROGM_SR(NPROMA)) 
  ZOROGL_SR(KST:KPROF)=PGM(KST:KPROF)*POROGL_RE(KST:KPROF)
  ZOROGM_SR(KST:KPROF)=PGM(KST:KPROF)*POROGM_RE(KST:KPROF)
  ! * allocate arrays for storing the surface wind, and compute them.
  ALLOCATE(ZUST1(NPROMA)) 
  ALLOCATE(ZVST1(NPROMA)) 
  LLDER=.FALSE.
  CALL GPUVS(NFLEVG,NPROMA,KST,KPROF,LLDER, &
   & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV),ZUST1,ZVST1)
ENDIF

IF(LGWADV) THEN

  ! * allocate ZVWVT1,ZVWVT1S and ZSVDT1.
  IF (LVERTFE.AND.LVFE_GW) THEN
    ALLOCATE (ZVWVT1(NPROMA,1:NFLEVG))
    ALLOCATE (ZVWVT1S(NPROMA))
  ELSE
    ALLOCATE (ZVWVT1(NPROMA,0:NFLEVG))
  ENDIF
  ALLOCATE (ZSVDT1(NPROMA,NFLEVG))

  ! * Update ZVWVT1:
  !   "gw(surf) is assumed to be equal to vecV(l=L)*gradorog
  !   If LRWSDLR=T, take account to the fact that ZOROG(L,M)_SR contains
  !    (r/a)*nabla(orog) and that we need to use nabla(orog).
  IF (LVERTFE.AND.LVFE_GW) THEN
    ZVWVT1(KST:KPROF,1:NFLEVG)=PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSVD)
    ZVWVT1S(KST:KPROF)=&
     & ZUST1(KST:KPROF)*ZOROGL_SR(KST:KPROF)&
     & +ZVST1(KST:KPROF)*ZOROGM_SR(KST:KPROF)  
    IF (LRWSDLR) THEN
      ZVWVT1S(KST:KPROF)=ZRDLR1(KST:KPROF,NFLEVG,JP_ASRH)*ZVWVT1S(KST:KPROF)
    ENDIF
  ELSE
    ZVWVT1(KST:KPROF,0:NFLEVG-1)=PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSVD)
    ZVWVT1(KST:KPROF,NFLEVG)=&
     & ZUST1(KST:KPROF)*ZOROGL_SR(KST:KPROF)&
     & +ZVST1(KST:KPROF)*ZOROGM_SR(KST:KPROF)  
    IF (LRWSDLR) THEN
      ZVWVT1(KST:KPROF,NFLEVG)= &
       & ZRDLR1(KST:KPROF,NFLEVG,JP_ASRH)*ZVWVT1(KST:KPROF,NFLEVG)
    ENDIF
  ENDIF

  ! * Convert explicit guess of (gw) towards d3.
  !   (for d4, X-term is added in GPENDTR via the SI term).
  IF (NVDVAR==4) THEN
    IVDVAR=3
  ELSE
    IVDVAR=NVDVAR 
  ENDIF
  IF (LVERTFE.AND.LVFE_GW) THEN
    ! * RDERI is not the exact inverse of RINTE, so it is not advised at all
    !   to apply RDERI to (gw)(t+dt): the choice done here is to apply
    !   RDERI to ((gw)(t+dt)-(gw)(t)), and to use quantities saved in buffer
    !   SLBUF2 ((gw)(t), (gw_surf)(t) and (-g dw)(t)).
    IF (LRWSDLR) THEN
      CALL GNHGW2SVD(KST,KPROF,IVDVAR,&
       & PGMVT1S(1,YT1%MSP),PGMVT1(1,1,YT1%MT),PGMVT1(1,1,YT1%MSPD),ZSVDT1,&
       & LDRWSDLR=LRWSDLR,PRDLR=ZRDLR1,&
       & PGWF=ZVWVT1,PGWS=ZVWVT1S,PGWRF=PB2(1,MSLB2GWF), &
       & PGWRS=PB2(1,MSLB2GWS),PGDWR=PB2(1,MSLB2GDW))
    ELSE
      CALL GNHGW2SVD(KST,KPROF,IVDVAR,&
       & PGMVT1S(1,YT1%MSP),PGMVT1(1,1,YT1%MT),PGMVT1(1,1,YT1%MSPD),ZSVDT1,&
       & PGWF=ZVWVT1,PGWS=ZVWVT1S,PGWRF=PB2(1,MSLB2GWF), &
       & PGWRS=PB2(1,MSLB2GWS),PGDWR=PB2(1,MSLB2GDW))
    ENDIF
  ELSE
    IF (LRWSDLR) THEN
      CALL GNHGW2SVD(KST,KPROF,IVDVAR,&
       & PGMVT1S(1,YT1%MSP),PGMVT1(1,1,YT1%MT),PGMVT1(1,1,YT1%MSPD),ZSVDT1,&
       & LDRWSDLR=LRWSDLR,PRDLR=ZRDLR1,&
       & PGWH=ZVWVT1)  
    ELSE
      CALL GNHGW2SVD(KST,KPROF,IVDVAR,&
       & PGMVT1S(1,YT1%MSP),PGMVT1(1,1,YT1%MT),PGMVT1(1,1,YT1%MSPD),ZSVDT1,&
       & PGWH=ZVWVT1)  
    ENDIF
  ENDIF

  ! * Transfer ZSVDT1 into PGMVT1(.,.,YT1%MSVD).
  PGMVT1(KST:KPROF,1:NFLEVG,YT1%MSVD)=ZSVDT1(KST:KPROF,1:NFLEVG)

  ! * deallocate arrays (zvwvt1,zvwvt1s,zsvdt1).
  IF (ALLOCATED (ZVWVT1)) DEALLOCATE(ZVWVT1)
  IF (ALLOCATED (ZVWVT1S)) DEALLOCATE(ZVWVT1S)
  IF (ALLOCATED (ZSVDT1)) DEALLOCATE(ZSVDT1)

ENDIF

IF (LRDBBC) THEN
  ! * Diagnostic BBC in NH:
  DO JROF=KST,KPROF 
    PGMVT1(JROF,NFLEVG,YT1%MSVD) = PGMVT1(JROF,NFLEVG,YT1%MSVD) &
     & - 0.5_JPRB*(ZDPHI9(JROF)+PB2(JROF,MSLB2DPHI1)) &
     & * (ZUST1(JROF)*ZOROGL_SR(JROF)+ZVST1(JROF)*ZOROGM_SR(JROF) &
     & - ZGWS9(JROF))-(ZDBBC9(JROF)+PB2(JROF,MSLB2DBBC1))
  ENDDO
ENDIF

IF(LGWADV.OR.LRDBBC) THEN
  ! * deallocate arrays (zorogl_sr,zorogm_sr,zust1,zvst1).
  IF (ALLOCATED(ZOROGL_SR)) DEALLOCATE(ZOROGL_SR)
  IF (ALLOCATED(ZOROGM_SR)) DEALLOCATE(ZOROGM_SR)
  IF (ALLOCATED(ZUST1)) DEALLOCATE(ZUST1)
  IF (ALLOCATED(ZVST1)) DEALLOCATE(ZVST1)
ENDIF

!        2.3.6 Add linear contributions interpolated at O if NGWADVSI=2
!              (some specific cases of NH + LGWADV only).

! In this case the linear contributions interpolated at O are added to
! the equations RHS after the conversion (gw) -> d4, but before the
! lagged physics and the specific treatment of term NHX done in CPGLAG.
! This is done only at the predictor step (only case where the terms added
! are non-zero).
! Not coded for the obsolescent option NVLAG=2.

! * Continuity equation:
IF (NVLAG == 3 .AND. NCURRENT_ITER == 0) THEN
  IF (LSLINLC2.AND. .NOT.(LVERTFE.AND..NOT.LSLINLC1) .AND.NGWADVSI==2) THEN
    ! Add the linear term, FD vertically integrated, to PB2.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP) &
         & +ZVWEI(JROF,JLEV)*ZC9_SI(JROF,JLEV)
      ENDDO
    ENDDO
  ELSEIF (LSLINLC2.AND.(LVERTFE.AND..NOT.LSLINLC1) .AND.NGWADVSI==2) THEN
    ! Add the linear term, VFE vertically integrated, to PB2.
    DO JLEV=1,NFLEVG
      ZSPT1(KST:KPROF,JLEV)=ZVWEI(KST:KPROF,JLEV)*VFE_RDETAH(JLEV)&
       & *ZC9_SI(KST:KPROF,JLEV)
    ENDDO
    CALL VERINT(NPROMA,KST,KPROF,NFLEVG,ZSPT1,ZSPTO,0)
    PGMVT1S(KST:KPROF,YT1%MSP)=PGMVT1S(KST:KPROF,YT1%MSP) &
     & +ZSPTO(KST:KPROF,NFLEVG+1)
  ENDIF
ENDIF

! * Momentum equation (should use the versions of ZPP,ZQQ not modified by
!   L2TLFF):
IF (NCURRENT_ITER == 0) THEN
  IF (LSLINL.AND.NGWADVSI==2) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZPP=ZP2(JROF,JLEV)
        ZQQ=ZQ2(JROF,JLEV)
        PGMVT1(JROF,JLEV,YT1%MU)=PGMVT1(JROF,JLEV,YT1%MU) &
         & +ZPP*ZU9_SI(JROF,JLEV)+ZQQ*ZV9_SI(JROF,JLEV)
        PGMVT1(JROF,JLEV,YT1%MV)=PGMVT1(JROF,JLEV,YT1%MV) &
         & -ZQQ*ZU9_SI(JROF,JLEV)+ZPP*ZV9_SI(JROF,JLEV)
      ENDDO
    ENDDO
    IF( ALLOCATED(ZP2) ) DEALLOCATE(ZP2)
    IF( ALLOCATED(ZQ2) ) DEALLOCATE(ZQ2)
  ENDIF
ENDIF

! * Temperature equation:
IF (NCURRENT_ITER == 0) THEN
  IF (LSLINL.AND.NGWADVSI==2) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PGMVT1(JROF,JLEV,YT1%MT)=PGMVT1(JROF,JLEV,YT1%MT)+ZT9_SI(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
ENDIF

! * NH equations:
IF (NCURRENT_ITER == 0) THEN
  IF (LSLINL.AND.NGWADVSI==2) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PGMVT1(JROF,JLEV,YT1%MSVD)=PGMVT1(JROF,JLEV,YT1%MSVD) &
         & +ZSVD9_SI(JROF,JLEV)
        PGMVT1(JROF,JLEV,YT1%MSPD)=PGMVT1(JROF,JLEV,YT1%MSPD) &
         & +ZSPD9_SI(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       4.    DEALLOCATIONS
!              -------------

IF( LDSLPHY ) THEN
  IF( ALLOCATED(ZUP9) ) DEALLOCATE(ZUP9)
  IF( ALLOCATED(ZVP9) ) DEALLOCATE(ZVP9)
ENDIF

IF(LNHDYN) THEN
  IF (ALLOCATED (ZVWV9)) DEALLOCATE (ZVWV9)
  IF (ALLOCATED (ZNHX9)) DEALLOCATE (ZNHX9)
  IF (ALLOCATED (ZSPD9)) DEALLOCATE (ZSPD9)
  IF (LRDBBC) THEN
    IF (ALLOCATED (ZDBBC9)) DEALLOCATE (ZDBBC9)
    IF (ALLOCATED (ZDPHI9)) DEALLOCATE (ZDPHI9)
    IF (ALLOCATED (ZGWS9))  DEALLOCATE (ZGWS9)
  ENDIF
ENDIF
IF(LNHDYN.AND.LPC_OLD.AND.(NCURRENT_ITER == 0))THEN
  IF (ALLOCATED (ZY9)) DEALLOCATE (ZY9)
ENDIF

IF (ALLOCATED (ZPREHYD1H)) DEALLOCATE (ZPREHYD1H)
IF (ALLOCATED (ZPREHYD1F)) DEALLOCATE (ZPREHYD1F)
IF (ALLOCATED (ZDELP1))    DEALLOCATE (ZDELP1)
IF (ALLOCATED (ZRDELP1))   DEALLOCATE (ZRDELP1)
IF (ALLOCATED (ZLNPR1))    DEALLOCATE (ZLNPR1)
IF (ALLOCATED (ZALPH1))    DEALLOCATE (ZALPH1)
IF (ALLOCATED (ZRTGR1))    DEALLOCATE (ZRTGR1)
IF (ALLOCATED (ZRPRE1))    DEALLOCATE (ZRPRE1)
IF (ALLOCATED (ZRPP1))     DEALLOCATE (ZRPP1)
IF (ALLOCATED (ZVCRSSA1F)) DEALLOCATE (ZVCRSSA1F)
IF (ALLOCATED (ZVCRSSA1H)) DEALLOCATE (ZVCRSSA1H)
IF (ALLOCATED (ZVCASRS1F)) DEALLOCATE (ZVCASRS1F)
IF (ALLOCATED (ZVCASRS1H)) DEALLOCATE (ZVCASRS1H)
IF (ALLOCATED (ZRDLR1))    DEALLOCATE (ZRDLR1)

IF(LSLINL) THEN
  IF (ALLOCATED(ZU9_SI)) DEALLOCATE (ZU9_SI)
  IF (ALLOCATED(ZV9_SI)) DEALLOCATE (ZV9_SI)
  IF (ALLOCATED(ZT9_SI)) DEALLOCATE (ZT9_SI)
  IF (ALLOCATED(ZSVD9_SI)) DEALLOCATE (ZSVD9_SI)
  IF (ALLOCATED(ZSPD9_SI)) DEALLOCATE (ZSPD9_SI)
ENDIF

IF ( ALLOCATED( ZGPPC ) ) DEALLOCATE(ZGPPC)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAPINEB',1,ZHOOK_HANDLE)
END SUBROUTINE LAPINEB
