!ocl list_copy(32,PLSDEPI)
!ocl list_copy(32,PLATI)
!ocl list_copy(32,PIPI0)
!ocl list_copy(32,PIPI1)
!ocl list_copy(32,PIPI2)
!ocl list_copy(32,PVETAF)
!ocl list_copy(32,PVCUICO)
SUBROUTINE LASCAWAD(KPROMB,KST,KPROF,KFLEV,KGLN,KGLX,&
 & KFLDN,KWIS,KHOR,&
 & LDSLHD,LDSLHDQUAD,&
 & P4JP,PIS2,PLSDEPI,PLATI,&
 & PIPI0,PIPI1,PIPI2,PSLD1,PSLD2,PSLD3,PSLDW,&
 & PLON,PLAT,PLEV,PKAPPA,&
 & PLON5,PLAT5,PLEV5,PKAPPA5,&
 & PVETAF,KVAUTF,&
 & PVCUICO,PVSLD,PVSLDW,KRLEVX,PVRLEVX,&
 & PDLAT,PCLA,PCLASLD,PDLO,PCLO,PCLOSLD,PDVER,PVINTW,PVINTWSLD )  

!**** *LASCAWAD - semi-LAgrangian scheme:   (adjoint version)
!                 Storage of Coordinates And Weights.

!     Purpose.
!     --------
!       Determines the interpolation grid:
!       - computation of the latitude and the longitude of the
!         point situated at the upper left corner of the 16 points
!         square, and of the interpolation point.
!       - computation of weights.
!       Storage of coordinates and weights.

!**   Interface.
!     ----------
!        *CALL* *LASCAWAD(.........)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMB  - horizontal dimension for interpolation point quantities.
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KGLN    - lower dimension for latitude.
!          KGLX    - upper dimension for latitude.
!          KFLDN   - number of the first field.
!          KWIS    - kind of interpolation.
!          KHOR    - 0: Horizontal interpolation for each level
!                       of a 3D variable.
!                    1: Interpolation for all origin points corresponding
!                       to a final point of a 2D variable.
!          LDSLHD  - key activating SLHD weights precomputation.
!          LDSLHDQUAD - key activating quadratic weights precomputation.
!          P4JP    - Approximative inverse of the differences of latitudes.
!          PIS2    - PI / 2
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PLATI   - latitude on the work sphere.
!          PIPI0   - coefficients for the bicubic interpolations.
!          PIPI1   - coefficients for the bicubic interpolations.
!          PIPI2   - coefficients for the bicubic interpolations.
!          PSLD1   - auxiliary quantity for SLHD interpolation in latitude.
!          PSLD2   - auxiliary quantity for SLHD interpolation in latitude.
!          PSLD3   - auxiliary quantity for SLHD interpolation in latitude.
!          PSLDW   - weights for SLHD Laplacian smoother in latitude.
! ------------------ input variables (output in TL) ----------------
!          PDLAT     - distance for horizontal linear interpolations
!                      in latitude.
!          PCLA      - weights for horizontal cubic interpolations
!                      in latitude.
!          PCLASLD   - weights for horizontal cubic interpolations
!                      in latitude, SLHD case
!          PDLO      - distances for horizontal linear interpolations
!                      in longitude (latitude rows 0, 1, 2, 3)
!          PCLO      - weights for horizontal cubic interpolations in
!                      longitude (latitude rows 1, 2)
!          PCLOSLD   - weights for horizontal cubic interpolations in
!                      longitude, SLHD case (latitude rows 1, 2)
!          PDVER     - distance for vertical linear interpolation
!          PVINTW    - vertical cubic interpolation weights
!          PVINTWSLD - vertical cubic interpolation weights, SLHD case
! ---------------------- trajectory variables ------------------------
!          PLON5   - Interpolation point longitude on the work sphere.
!          PLAT5   - Interpolation point latitude on the work sphere.
!          PLEV5   - vertical coordinate of the interpolation point.
!          PKAPPA5 - kappa function ("coefficient of SLHD") based on the
!                    rescaled horizontal deformation of the flow evaluated
!                    at instant "t" for the final point F
! --------------------------------------------------------------------
!          PVETAF  - Values of ETA at full levels
!          KVAUTF  - Help table for vertical box search: gives the number
!                    of the layer immediately above eta.
!          PVCUICO - Denominators of the vertical cubic interpolation
!                    coefficients
!          PVSLD   - auxiliary quantities for vertical SLHD interpolation
!          PVSLDW  - weights for SLHD vertical Laplacian smoother
!          KRLEVX  - Dimension of KVAUTF
!          PVRLEVX - FLOAT(KRLEVX).

! ------------ output variables (input in TL) ----------------------
!          PLON    - Interpolation point longitude on the work sphere.
!          PLAT    - Interpolation point latitude on the work sphere.
!          PLEV    - vertical coordinate of the interpolation point.
!          PKAPPA  - kappa function ("coefficient of SLHD") based on the
!                    rescaled horizontal deformation of the flow evaluated
!                    at instant "t" for the final point F

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        C. Temperton, ECMWF

!     Modifications.
!     --------------
!        Original : 98/10/19.
!        Modified 98-12-29 by C.Temperton: Cleaning, following LASCAW.
!        Modified 99-10-01 by C.Temperton: 3D options added.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified 28-07-2006 R. El Khatib + NEC support team : optimization
!        for vector platforms (translation of VPP list_copy directives)
!        Modified 04-Aug-2008 : F. Vana - weights driven interpolation fully
!                              projected to TL (still excluding P/C and half
!                              level interpolation) + optimization for vect.
!        K. Yessad (Dec 2008): remove useless dummy arguments
!        F. Vana  13-Jan-2009: fixed bug in PKAPPA computation
!        K. Yessad + F. Vana (Feb 2009): split loops, rewrite in a shorter way.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMDYNA  , ONLY : LSLHD_OLD
USE YOMMP    , ONLY : NFRSTLOFF
USE YOMLASCAW, ONLY : KPDUP
USE YOMPLDSW , ONLY : LOPT_SCALAR

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGLN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGLX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KRLEVX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KWIS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHOR 
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDSLHDQUAD
REAL(KIND=JPRB)   ,INTENT(IN)    :: P4JP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATI(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPI0(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPI1(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPI2(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLD1(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLD2(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLD3(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLDW(3,3,KGLN:KGLX)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLON(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLAT(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLEV(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKAPPA(KPROMB,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON5(KPROMB,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT5(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV5(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPPA5(KPROMB,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVETAF(0:KFLEV+1) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVAUTF(0:KRLEVX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCUICO(4,0:KFLEV-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVSLD(3,0:KFLEV-1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVSLDW(3,3,0:KFLEV-1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRLEVX 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLASLD(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(KPROMB,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOSLD(KPROMB,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDVER(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWSLD(KPROMB,KFLEV,3)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZVETA(0:KFLEV+1)

INTEGER(KIND=JPIM) :: IFLVM2, ILA, ILEV, ILEVV,&
 & ILO1, ILO2, IZLAT, JLEV, JROF, IJ_, J_
INTEGER(KIND=JPIM) :: IILA(KPROMB,KFLEV), IILEV(KPROMB,KFLEV)

REAL(KIND=JPRB) ::  ZDA, ZDB, ZDC, ZDD, ZFAC, ZLO1, ZLO2
REAL(KIND=JPRB) ::  ZDLO5(KPROMB,KFLEV,2), ZDLAT5(KPROMB,KFLEV), ZDVER5(KPROMB,KFLEV)

LOGICAL :: LLT_SLHD(3),LLSLHD,LLSLHDQUAD,LLSLHD_OLD

! Duplicata of some dummy or local arrays for optimisation on NEC platform.
! REAL(KIND=JPRB) :: ZVETAF_(KPDUP,0:KFLEV+1)
REAL(KIND=JPRB) :: ZVCUICO_(KPDUP,4,0:KFLEV-1)
REAL(KIND=JPRB) :: ZVSLD_(KPDUP,3,0:KFLEV-1)
REAL(KIND=JPRB) :: ZVSLDW_(KPDUP,3,3,0:KFLEV-1)
REAL(KIND=JPRB) :: ZLSDEPI_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRLATI_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRIPI0_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRIPI1_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRIPI2_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLD1_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLD2_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLD3_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLDW_(KPDUP,3,3,KGLN:KGLX)

REAL(KIND=JPRB) :: ZZWH(KPROMB,3,KFLEV)
REAL(KIND=JPRB) :: ZZWH5(KPROMB,3,KFLEV)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "lascaw_cla_ad.intfb.h"
#include "lascaw_clo_ad.intfb.h"
#include "lascaw_vintw_ad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LASCAWAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

! cases relevant for SLHD scheme (switches LDSLHD, LDSLHDQUAD are
! deactivated during computation of medium points in LAPINEA)
LLSLHD=LDSLHD.AND. &
 & (KWIS==103.OR.KWIS==203)
LLSLHDQUAD=LDSLHDQUAD.AND. &
 & (KWIS==103.OR.KWIS==203)

! switch for old SLHD scheme
LLSLHD_OLD=LLSLHD.AND.LSLHD_OLD

LLT_SLHD(1)=LLSLHD
LLT_SLHD(2)=LLSLHDQUAD
LLT_SLHD(3)=LLSLHD_OLD

IFLVM2=KFLEV-2

! * matches with the case kslev=1 of lascaw (TL of kslev=0 not yet coded).
ZVETA(0:KFLEV+1)=PVETAF(0:KFLEV+1)

DO J_ = 1, KPDUP
  ZLSDEPI_(J_,KGLN:KGLX)=PLSDEPI(KGLN:KGLX)
  ZRLATI_(J_,KGLN:KGLX)=PLATI(KGLN:KGLX)
  ZRIPI0_(J_,KGLN:KGLX)=PIPI0(KGLN:KGLX)
  ZRIPI1_(J_,KGLN:KGLX)=PIPI1(KGLN:KGLX)
  ZRIPI2_(J_,KGLN:KGLX)=PIPI2(KGLN:KGLX)
  IF (LOPT_SCALAR) THEN
    ZVCUICO_(J_,1:4,0:KFLEV-1)=PVCUICO(1:4,0:KFLEV-1)
  ELSE
!CDIR NOUNROLL
    DO JLEV=1,4*KFLEV
      ZVCUICO_(J_,JLEV,0)=PVCUICO(JLEV,0)
    ENDDO
  ENDIF
ENDDO
IF (LLSLHDQUAD) THEN
  DO J_ = 1, KPDUP
    ZSLD1_(J_,KGLN:KGLX)=PSLD1(KGLN:KGLX)
    ZSLD2_(J_,KGLN:KGLX)=PSLD2(KGLN:KGLX)
    ZSLD3_(J_,KGLN:KGLX)=PSLD3(KGLN:KGLX)
    IF (LOPT_SCALAR) THEN
      ZVSLD_(J_,1:3,0:KFLEV-1)=PVSLD(1:3,0:KFLEV-1)
    ELSE
!CDIR NOUNROLL
      DO JLEV=1,3*KFLEV
        ZVSLD_(J_,JLEV,0)=PVSLD(JLEV,0)
      ENDDO
    ENDIF
  ENDDO
ENDIF
IF ( LLSLHD ) THEN
  DO J_ = 1, KPDUP
    ZSLDW_(J_,1:3,1:3,KGLN:KGLX)=PSLDW(1:3,1:3,KGLN:KGLX)
    IF (LOPT_SCALAR) THEN
      ZVSLDW_(J_,1:3,1:3,0:KFLEV-1)=PVSLDW(1:3,1:3,0:KFLEV-1)
    ELSE
!CDIR NOUNROLL
      DO JLEV=1,9*KFLEV
        ZVSLDW_(J_,JLEV,1,0)=PVSLDW(JLEV,1,0)
      ENDDO
    ENDIF
  ENDDO
ENDIF


!     ------------------------------------------------------------------

!*       1.    3D MODEL.
!              ---------

!        1.01  Coordinates and weights for trilinear interpolations.

IF (KWIS == 101) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=1,KFLEV

    ! * AD calculation of linear weights.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT5(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT5(JROF,JLEV))-1.5_JPRB)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV)+PDLAT(JROF,JLEV)&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      PLON(JROF,JLEV)=PLON(JROF,JLEV)+&
       & PDLO(JROF,JLEV,1)*ZLSDEPI_(IJ_,ILA+1)+PDLO(JROF,JLEV,2)*ZLSDEPI_(IJ_,ILA+2)  

      ILEV  =KVAUTF(INT(PLEV5(JROF,JLEV)*ZFAC))-1
      IF(ILEV < IFLVM2.AND.&
       & (PLEV5(JROF,JLEV)-ZVETA(ILEV+2)) > 0.0_JPRB) ILEV=ILEV+1  

      PLEV(JROF,JLEV)=PLEV(JROF,JLEV)+PDVER(JROF,JLEV)/&
       & (ZVETA(ILEV+2)-ZVETA(ILEV+1))  

    ENDDO
  ENDDO

ENDIF

!        1.02  Coordinates and weights for trilinear interpolations
!              + 4 points interpolations.

IF (KWIS == 102) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=1,KFLEV
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * AD calculation of linear weights.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT5(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT5(JROF,JLEV))-1.5_JPRB)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV)+PDLAT(JROF,JLEV)&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      PLON(JROF,JLEV)=PLON(JROF,JLEV)+&
       & PDLO(JROF,JLEV,1)*ZLSDEPI_(IJ_,ILA+1)+PDLO(JROF,JLEV,2)*ZLSDEPI_(IJ_,ILA+2)  

      ILEVV =KVAUTF(INT(PLEV5(JROF,JLEV)*ZFAC))-1
      IF(ILEVV < IFLVM2.AND.&
       & (PLEV5(JROF,JLEV)-ZVETA(ILEVV+2)) > 0.0_JPRB) ILEVV=ILEVV+1  

      PLEV(JROF,JLEV)=PLEV(JROF,JLEV)+PDVER(JROF,JLEV)/&
       & (ZVETA(ILEVV+2)-ZVETA(ILEVV+1))  

    ENDDO
  ENDDO

ENDIF

!        1.03  Coordinates and weights for ( horizontal 12 points
!              + vertical cubic + 32 points interpolations ) or
!              ( horizontal 12 points + 32 points interpolations ).
!              For vertical interpolations only 4 points cubic
!              interpolations are used.

IF (KWIS == 103) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=1,KFLEV
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * AD of calculation of linear weights: trajectory.
!CDIR NOLSTVAL
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT5(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT5(JROF,JLEV))-1.5_JPRB)
      ZDLAT5(JROF,JLEV)=(PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))

      ZDA   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      ! for input to LASCAW_CLA_AD.
      IILA(JROF,JLEV)=ILA
      ZZWH5(JROF,1,JLEV)=(ZDA*ZDC)*ZDD*ZRIPI0_(IJ_,ILA+1)
      ZZWH5(JROF,2,JLEV)=(ZDA*ZDB)*ZDD*ZRIPI1_(IJ_,ILA+1)
      ZZWH5(JROF,3,JLEV)=(ZDA*ZDB)*ZDC*ZRIPI2_(IJ_,ILA+1)

      ZLO1  =PLON5(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1  =INT(ZLO1)
      ZDLO5(JROF,JLEV,1)=ZLO1-REAL(ILO1,JPRB)
      ZLO2  =PLON5(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2  =INT(ZLO2)
      ZDLO5(JROF,JLEV,2)=ZLO2-REAL(ILO2,JPRB)

      ILEVV=KVAUTF(INT(PLEV5(JROF,JLEV)*ZFAC))-1
      IF(ILEVV < IFLVM2.AND.&
       & (PLEV5(JROF,JLEV)-ZVETA(ILEVV+2)) > 0.0_JPRB) ILEVV=ILEVV+1  
      IILEV(JROF,JLEV)=ILEVV

      ZDVER5(JROF,JLEV)=(PLEV5(JROF,JLEV)-ZVETA(ILEVV+1))/&
       & (ZVETA(ILEVV+2)-ZVETA(ILEVV+1)) 

    ENDDO
  ENDDO

  ! * AD of calculation of PVINTW and PVINTWSLD:
  CALL LASCAW_VINTW_AD(KPROMB,KFLEV,1,KST,KPROF,LLT_SLHD,IILEV,&
   & PLEV,PDVER,PKAPPA,PVETAF,ZVCUICO_,ZVSLD_,ZVSLDW_,&
   & PLEV5,ZDVER5,PKAPPA5,&
   & PVINTW,PVINTWSLD)

    ! * AD of calculation of PCLO and PCLOSLD:
  CALL LASCAW_CLO_AD(KFLEV,KPROMB,KST,KPROF,LLT_SLHD,&
   & PDLO(:,:,2),PKAPPA,&
   & ZDLO5(:,:,2),PKAPPA5,&
   & PCLO(:,:,:,2),PCLOSLD(:,:,:,2))
  CALL LASCAW_CLO_AD(KFLEV,KPROMB,KST,KPROF,LLT_SLHD,&
   & PDLO(:,:,1),PKAPPA,&
   & ZDLO5(:,:,1),PKAPPA5,&
   & PCLO(:,:,:,1),PCLOSLD(:,:,:,1))

  ! * AD of calculation of PCLA and PCLASLD:
  CALL LASCAW_CLA_AD(KFLEV,KPROMB,KGLN,KGLX,KST,KPROF,LLT_SLHD,&
   & IILA,ZZWH,PDLAT,PKAPPA,ZSLD1_,ZSLD2_,ZSLD3_,ZSLDW_,&
   & ZZWH5,ZDLAT5,PKAPPA5,&
   & PCLA,PCLASLD)
    
  DO JLEV=1,KFLEV
    ! * AD of calculation of linear weights: increments.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1
      ILA=IILA(JROF,JLEV)

      ZDA   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      PLEV(JROF,JLEV)=PLEV(JROF,JLEV)+PDVER(JROF,JLEV) &
       & /(ZVETA(IILEV(JROF,JLEV)+2)-ZVETA(IILEV(JROF,JLEV)+1))

      PLON(JROF,JLEV)=PLON(JROF,JLEV)&
       & +PDLO(JROF,JLEV,0)*ZLSDEPI_(IJ_,ILA  )&
       & +PDLO(JROF,JLEV,1)*ZLSDEPI_(IJ_,ILA+1)&
       & +PDLO(JROF,JLEV,2)*ZLSDEPI_(IJ_,ILA+2)&
       & +PDLO(JROF,JLEV,3)*ZLSDEPI_(IJ_,ILA+3)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV)&
       & +(ZDA*(ZDC+ZDD)+ZDC*ZDD)*ZZWH(JROF,1,JLEV)*ZRIPI0_(IJ_,ILA+1)&
       & +(ZDA*(ZDB+ZDD)+ZDB*ZDD)*ZZWH(JROF,2,JLEV)*ZRIPI1_(IJ_,ILA+1)&
       & +(ZDA*(ZDB+ZDC)+ZDB*ZDC)*ZZWH(JROF,3,JLEV)*ZRIPI2_(IJ_,ILA+1)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV)+ &
       & PDLAT(JROF,JLEV)/(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))

    ENDDO
  ENDDO

ENDIF

!        1.04  Coordinates and weights for ( horizontal 12 points
!              + vertical cubic + 32 points interpolations ) or
!              ( horizontal 12 points + 32 points interpolations ).
!              For vertical interpolations 4 points cubic
!              interpolations and Hermite cubic interpolations are used.

IF (KWIS == 104) THEN     !!! AD not supported !!!
  CALL ABOR1('LASCAWAD: The option KWIS=104 is not supported by AD code')
ENDIF

!        1.05  Coordinates and weights for ( horizontal 12 points
!              + vertical cubic + 32 points interpolations ) or
!              ( horizontal 12 points + 32 points interpolations )
!              with vertical Hermite cubic interpolations.
!              For vertical interpolation cubic B-spline
!              interpolation are used.  

IF (KWIS == 105) THEN !!! AD not supported !!!
  CALL ABOR1('LASCAWAD: The option KWIS=105 is not supported by AD code')
ENDIF

!     ----------------------------------------------------------------

!*       2.    2D MODEL AND CASES IN THE 3D MODEL WHERE ONLY
!              2D INTERPOLATIONS ARE NEEDED.
!              ---------------------------------------------

!        2.01  Coordinates and weights for bilinear interpolations.

IF (KWIS == 201) THEN

  DO JLEV=1,KFLEV
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * AD calculation of linear weights.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT5(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT5(JROF,JLEV))-1.5_JPRB)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV) +&
       & PDLAT(JROF,JLEV) / (ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      PLON(JROF,JLEV)=PLON(JROF,JLEV)&
       & +PDLO(JROF,JLEV,1)*ZLSDEPI_(IJ_,ILA+1)&
       & +PDLO(JROF,JLEV,2)*ZLSDEPI_(IJ_,ILA+2)  

    ENDDO
  ENDDO

ENDIF

!        2.03  Coordinates and weights for 12 points interpolations.

IF (KWIS == 203) THEN

  DO JLEV=1,KFLEV
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * AD of calculation of linear weights: trajectory.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT5(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT5(JROF,JLEV))-1.5_JPRB)
      ZDLAT5(JROF,JLEV)=(PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))

      ZDA   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      ! for input to LASCAW_CLA_AD.
      IILA(JROF,JLEV)=ILA
      ZZWH5(JROF,1,JLEV)=(ZDA*ZDC)*ZDD*ZRIPI0_(IJ_,ILA+1)
      ZZWH5(JROF,2,JLEV)=(ZDA*ZDB)*ZDD*ZRIPI1_(IJ_,ILA+1)
      ZZWH5(JROF,3,JLEV)=(ZDA*ZDB)*ZDC*ZRIPI2_(IJ_,ILA+1)

      ZLO1  =PLON5(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1  =INT(ZLO1)
      ZDLO5(JROF,JLEV,1)=ZLO1-REAL(ILO1,JPRB)
      ZLO2  =PLON5(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2  =INT(ZLO2)
      ZDLO5(JROF,JLEV,2)=ZLO2-REAL(ILO2,JPRB)

    ENDDO
  ENDDO

  ! * AD of calculation of PCLO and PCLOSLD:
  CALL LASCAW_CLO_AD(KFLEV,KPROMB,KST,KPROF,LLT_SLHD,&
   & PDLO(:,:,2),PKAPPA,&
   & ZDLO5(:,:,2),PKAPPA5,&
   & PCLO(:,:,:,2),PCLOSLD(:,:,:,2))
  CALL LASCAW_CLO_AD(KFLEV,KPROMB,KST,KPROF,LLT_SLHD,&
   & PDLO(:,:,1),PKAPPA,&
   & ZDLO5(:,:,1),PKAPPA5,&
   & PCLO(:,:,:,1),PCLOSLD(:,:,:,1))

  ! * AD of calculation of PCLA and PCLASLD:
  CALL LASCAW_CLA_AD(KFLEV,KPROMB,KGLN,KGLX,KST,KPROF,LLT_SLHD,&
   & IILA,ZZWH,PDLAT,PKAPPA,ZSLD1_,ZSLD2_,ZSLD3_,ZSLDW_,&
   & ZZWH5,ZDLAT5,PKAPPA5,&
   & PCLA,PCLASLD)

  DO JLEV=1,KFLEV
    ! * AD of calculation of linear weights: increments.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1
      ILA=IILA(JROF,JLEV)

      ZDA   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT5(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      PLON(JROF,JLEV)=PLON(JROF,JLEV)&
       & +PDLO(JROF,JLEV,0)*ZLSDEPI_(IJ_,ILA  )&
       & +PDLO(JROF,JLEV,1)*ZLSDEPI_(IJ_,ILA+1)&
       & +PDLO(JROF,JLEV,2)*ZLSDEPI_(IJ_,ILA+2)&
       & +PDLO(JROF,JLEV,3)*ZLSDEPI_(IJ_,ILA+3)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV)&
       & +(ZDA*(ZDC+ZDD)+ZDC*ZDD)*ZZWH(JROF,1,JLEV)*ZRIPI0_(IJ_,ILA+1)&
       & +(ZDA*(ZDB+ZDD)+ZDB*ZDD)*ZZWH(JROF,2,JLEV)*ZRIPI1_(IJ_,ILA+1)&
       & +(ZDA*(ZDB+ZDC)+ZDB*ZDC)*ZZWH(JROF,3,JLEV)*ZRIPI2_(IJ_,ILA+1)

      PLAT(JROF,JLEV)=PLAT(JROF,JLEV)+ &
       & PDLAT(JROF,JLEV)/(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))

    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LASCAWAD',1,ZHOOK_HANDLE)
END SUBROUTINE LASCAWAD
