SUBROUTINE LATTEX_DNT5(&
 ! INPUT
 & KPROMA,KSTART,KPROF,KFLEV,KFLSA,KFLEN,KSTEP,&
 & LDSETTLS,LDINCXT1,&
 & KXLAG ,PESGP , PESGM ,PXI  ,PXIGP ,PXIGM,&
 & PXT0,&
!nyc!   & PXT9,&
 & PMOY1X,&
 ! INPUT/OUTPUT
 & PXSI, PXNLT9, PXT1  ,PXL0   ,PXL9 &
!nyc!   & PXSI, PXNLT9, PXT1  ,PXL0   ,PXL9,&
!nyc!   ! INPUT/OUTPUT (for PC scheme)
!nyc!   & PCXNLT9, PCXNLT99 &
 & )  

!------------------------------------------------------------------------------
! LATTEX_DNT5 - Semi-Lagrangian scheme.
!               Computation of the t and t-dt useful quantities
!               at grid-points. Equations for tri-dimensional
!               variables for 2TL scheme.
!               Trajectory version of LATTEX_DNT

! Purpose
! -------
!       Trajectory version of LATTEX_DNT: see LATTEX_DNT for more information.
!       Remarks:
!        - This routine is called only in the TL model when NTRSLTYPE=1
!          but this option has never worked (uncomplete code).
!        - The following options are currently not coded in this routine
!          (especially "!nyc!" commented and not validated pieces of code):
!          * K[X]LAG = 2 (coded but not validated)
!          * LPC_FULL. (commented code)

!       There are small differences between LATTEX_DNT and LATTEX_DNT5,
!       especially regarding the incrementation of PXT1 and PXSI.
!       Note that in parts 2.1 and 2.2 the incrementation of PXT1 is always
!        done in the direct code LATTEX_DNT, but conditionally done (key
!        LDINCXT1) in LATTEX_DNT5.

! Interface
! ---------
!   CALL LATTEX_DNT5(..)

! Explicit arguments :
! --------------------

! * INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of work.
!          KPROF   - depth of work.
!          KFLEV   - number of layers.
!          KFLSA   - lower number of layer for arrays with extra-layers.
!          KFLEN   - upper number of layer for arrays with extra-layers.
!          KSTEP   - Number of the current time-step.
!          LDSETTLS- .T./.F.: Stable/Conventional extrapolations for SL2TL.
!          LDINCXT1- increment PXT1?
!                    - for wind equation: yes if L2TLFF, no otherwise.
!                    - for other equations: always F.
!          KXLAG   - type of SL discretisation 
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          PXI     - second o. dec. factor
!          PXIGP   - second o. dec. - first times step:  1+xidt  other:  1+xidt
!          PXIGM   - second o. dec. - first times step:  1-xidt  other:  1+xidt
!          PXT0    - prognostic variable time t (predictor or SI),
!                    preliminary t+dt (corrector)
!nyc!      PXT9    - prognostic variable time t (corrector),
!nyc!                not used for predictor or SI
!          PMOY1X  - full nonlinear model at time t [(Delta t/2) "cursive" A]

! * INPUT/OUTPUT:
!          PXSI    - semi-implicit linear model at time t
!                    [- (Delta t/2) "cursive" B]
!          PXNLT9  - buffer used during predictor resp. SI time step
!          PXT1    - t+dt term and other final point terms 
!          PXL0    - second SL quantity to be interpolated (linearly) at O
!                    if KXLAG=3
!          PXL9    - SL quantity to be interpolated at O 
!nyc!      PCXNLT9 - buffer used during corrector
!nyc!      PCXNLT99- buffer used during corrector in the case of
!nyc!                pseudo-second order decentering

! Externals
! ---------
!   none
!   Called by LATTEX5.

! Method
! ------

! Reference
! ---------
!   Arpege documentation about semi-Lagrangian scheme.

! Author
! ------
!      K. Yessad (March 2007) after LATTEX_DNT.

! Modifications
! -------------
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LPC_NESC ,LPC_FULL
USE YOMDYN   , ONLY : NCURRENT_ITER
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLSA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
LOGICAL           ,INTENT(IN)    :: LDSETTLS 
LOGICAL           ,INTENT(IN)    :: LDINCXT1
INTEGER(KIND=JPIM),INTENT(IN)    :: KXLAG 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXIGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXIGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXT0(KPROMA,KFLEV) 
!nyc!  REAL(KIND=JPRB)   ,INTENT(IN)    :: PXT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMOY1X(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSI(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXNLT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXT1(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXL0(KPROMA,KFLSA:KFLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXL9(KPROMA,KFLSA:KFLEN) 
!nyc!  REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCXNLT9(KPROMA,KFLEV) 
!nyc!  REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCXNLT99(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV,JROF

!     * ZMOY[X]NL (used only for 2TL SL scheme) is the non linear term
!       - extrapolated at t+dt/2 if lsettls=false
!       - a combination of the t and t-dt contributions for interpolation
!         at the origin point if lsettls=true
!     * Z[X]NLT0 (used only for 2TL SL scheme with lsettls=true) is the non
!       linear term at t.
REAL(KIND=JPRB) :: ZMOYXNL(KPROMA)
REAL(KIND=JPRB) :: ZXNLT0(KPROMA)
REAL(KIND=JPRB) :: ZXIDT0,ZXIDT9

LOGICAL :: LLSI_XIDT
LOGICAL :: LLCT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEX_DNT5',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*      1. AUXILIARY VARIABLES.
!       -----------------------

! true if second order decentering is applied
LLSI_XIDT = PXI > 0.0_JPRB

LLCT = (LPC_FULL .AND. NCURRENT_ITER > 0)

! factor for nonlinear residual at time 0 ( residual = A - ZXIDT0 B)

IF( LLSI_XIDT.AND.KSTEP>0.AND..NOT.LPC_NESC )THEN
  ZXIDT0 = 1.0_JPRB+PXI
ELSE
  ZXIDT0 = 1.0_JPRB
ENDIF

! factor for nonlinear residual at time 9 ( residual = A - ZXIDT9 B)

IF( LLSI_XIDT )THEN
  ZXIDT9 = 1.0_JPRB+PXI
ELSE
  ZXIDT9 = 1.0_JPRB
ENDIF

!     ------------------------------------------------------------------

!*      2. MAIN CALCULATIONS.
!       ---------------------

IF( .NOT.LLCT )THEN

  !############################################
  ! 2.1 Predictor for LPC_FULL,
  !     or predictor for LPC_OLD,
  !     or case NSITER=0.
  !############################################

  DO JLEV=1,KFLEV

    ! nonlinear residual time t
    DO JROF=KSTART,KPROF
      ZXNLT0(JROF)=PMOY1X(JROF,JLEV)+ZXIDT0*PXSI(JROF,JLEV)
    ENDDO
  
    ! nonlinear residual time t+dt/2
    IF (KSTEP <= 0) THEN
      ZMOYXNL(KSTART:KPROF)=ZXNLT0(KSTART:KPROF)
    ELSE
      IF (LDSETTLS) THEN
        IF (LLSI_XIDT) THEN
          DO JROF=KSTART,KPROF
            ZMOYXNL(JROF)=2.0_JPRB*ZXNLT0(JROF)-PXNLT9(JROF,JLEV)
          ENDDO
        ELSE
          DO JROF=KSTART,KPROF
            ZMOYXNL(JROF)=(1.0_JPRB+PESGM)*ZXNLT0(JROF)-PXNLT9(JROF,JLEV)
          ENDDO
        ENDIF
        !nyc!  ELSEIF((.NOT.LDSETTLS).AND.LPC_FULL.AND.LPC_NESC)THEN
        !nyc!    IF(LLSI_XIDT)THEN
        !nyc!      DO JROF=KSTART,KPROF
        !nyc!        ZMOYXNL(JROF)=ZXNLT0(JROF)&
        !nyc!         & +PXI*( 1.5_JPRB*PXSI(JROF,JLEV)-0.5_JPRB*PXNLT9(JROF,JLEV) )
        !nyc!      ENDDO
        !nyc!    ELSE
        !nyc!      ZMOYXNL(KSTART:KPROF)=ZXNLT0(KSTART:KPROF)
        !nyc!    ENDIF
      ELSE
        ! * remaining case: ldsettls=false, lpc_nesc=false.
        DO JROF=KSTART,KPROF
          ZMOYXNL(JROF)=1.5_JPRB*ZXNLT0(JROF)-0.5_JPRB*PXNLT9(JROF,JLEV)
        ENDDO
      ENDIF
    ENDIF
  
    !nyc!   ! save quantities for corrector step
    !nyc!   IF( LPC_FULL )THEN
  
    !nyc!     ! save quantity for pseudo-second order decentering valid at time t+dt/2
    !nyc!     IF( LLSI_XIDT )THEN
    !nyc!       IF( KSTEP <= 0 )THEN
    !nyc!         PCXNLT99(KSTART:KPROF,JLEV)=0.0_JPRB
    !nyc!       ELSE
    !nyc!         DO JROF=KSTART,KPROF
    !nyc!           PCXNLT99(JROF,JLEV)=PXI*(1.5_JPRB*PMOY1X(JROF,JLEV)&
    !nyc!            & -0.5_JPRB*PCXNLT9(JROF,JLEV))
    !nyc!         ENDDO
    !nyc!       ENDIF
    !nyc!     ENDIF
  
    !nyc!     ! save nonlinear model at time t
    !nyc!     PCXNLT9(KSTART:KPROF,JLEV)=PMOY1X(KSTART:KPROF,JLEV)
  
    !nyc!   ENDIF
  
    !nyc!   IF( LPC_FULL.AND.LPC_NESC )THEN

    !nyc!     ! save linear model t to be used as t-dt next time step
    !nyc!     IF(LLSI_XIDT)THEN
    !nyc!       PXNLT9(KSTART:KPROF,JLEV)=PXSI(KSTART:KPROF,JLEV)
    !nyc!     ENDIF
    !nyc!   ELSE
        
      ! save of nonlinear residual at time t
      ! to be used as nonlinear residual at time t-dt next time step
      DO JROF=KSTART,KPROF
        PXNLT9(JROF,JLEV)=PMOY1X(JROF,JLEV)+ZXIDT9*PXSI(JROF,JLEV)
      ENDDO
    !nyc!   ENDIF

    IF (KXLAG == 2) THEN
      ! * Remark: case ldsettls=true is not coded for kxlag=2.
      IF (LLSI_XIDT) THEN
        DO JROF=KSTART,KPROF
          PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT0(JROF,JLEV)+ZMOYXNL(JROF)&
           & -PXIGM*PXSI(JROF,JLEV)
        ENDDO
        IF (LDINCXT1) THEN
          DO JROF=KSTART,KPROF
            PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+ZMOYXNL(JROF)
          ENDDO
        ENDIF
      ELSE
        !nyc!   IF(LPC_FULL.AND.LPC_NESC)THEN
        !nyc!     DO JROF=KSTART,KPROF
        !nyc!       PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT0(JROF,JLEV)&
        !nyc!        & +PESGM*PMOY1X(JROF,JLEV)
        !nyc!     ENDDO
        !nyc!     IF (LDINCXT1) THEN
        !nyc!       DO JROF=KSTART,KPROF
        !nyc!         PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZMOYXNL(JROF)
        !nyc!       ENDDO
        !nyc!     ENDIF
        !nyc!   ELSE
          DO JROF=KSTART,KPROF
            PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT0(JROF,JLEV)&
             & +PESGM*ZMOYXNL(JROF)-PESGM*PXSI(JROF,JLEV)
          ENDDO
          IF (LDINCXT1) THEN
            DO JROF=KSTART,KPROF
              PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZMOYXNL(JROF)
            ENDDO
          ENDIF
        !nyc!   ENDIF
      ENDIF
    ELSEIF (KXLAG == 3) THEN
      IF (LLSI_XIDT) THEN
        IF (LDSETTLS) THEN
          DO JROF=KSTART,KPROF
            PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+ZMOYXNL(JROF)-PXIGM*PXSI(JROF,JLEV)
          ENDDO
          IF (LDINCXT1) THEN
            DO JROF=KSTART,KPROF
              PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+ZXNLT0(JROF)
            ENDDO
          ENDIF
        ELSE
          DO JROF=KSTART,KPROF
            PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+ZMOYXNL(JROF)-PXIGM*PXSI(JROF,JLEV)
          ENDDO
          IF (LDINCXT1) THEN
            DO JROF=KSTART,KPROF
              PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+ZMOYXNL(JROF)
            ENDDO
          ENDIF
        ENDIF
      ELSE
        IF (LDSETTLS) THEN
          DO JROF=KSTART,KPROF
            PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+ZMOYXNL(JROF)-PESGM*PXSI(JROF,JLEV)
          ENDDO
          IF (LDINCXT1) THEN
            DO JROF=KSTART,KPROF
              PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZXNLT0(JROF)
            ENDDO
          ENDIF
          !nyc!   ELSEIF((.NOT.LDSETTLS).AND.LPC_FULL.AND.LPC_NESC)THEN
          !nyc!     DO JROF=KSTART,KPROF
          !nyc!       PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+PESGM*PMOY1X(JROF,JLEV)
          !nyc!     ENDDO
          !nyc!     IF (LDINCXT1) THEN
          !nyc!       DO JROF=KSTART,KPROF
          !nyc!         PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZMOYXNL(JROF)
          !nyc!       ENDDO
          !nyc!     ENDIF
        ELSE
          ! * remaining case: ldsettls=false, lpc_nesc=false.
          DO JROF=KSTART,KPROF
            PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+PESGM*ZMOYXNL(JROF)&
             & -PESGM*PXSI(JROF,JLEV)
          ENDDO
          IF (LDINCXT1) THEN
            DO JROF=KSTART,KPROF
              PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZMOYXNL(JROF)
            ENDDO
          ENDIF
        ENDIF
      ENDIF
      DO JROF=KSTART,KPROF
        PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT0(JROF,JLEV)
      ENDDO
    ENDIF

  ENDDO

  !nyc!  ELSE

  !nyc!    !############################################
  !nyc!    ! 2.2 Corrector for LPC_FULL
  !nyc!    !############################################

  !nyc!    DO JLEV=1,KFLEV

  !nyc!      ! nonlinear residual time t+dt (preliminary state from predictor)

  !nyc!      DO JROF=KSTART,KPROF
  !nyc!        ZMOYXNL(JROF)=PMOY1X(JROF,JLEV)+PXSI(JROF,JLEV)
  !nyc!      ENDDO

  !nyc!      IF (KXLAG == 2) THEN
  !nyc!        IF (LLSI_XIDT) THEN
  !nyc!          DO JROF=KSTART,KPROF
  !nyc!            PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PXIGP*ZMOYXNL(JROF)&
  !nyc!             & -PCXNLT99(JROF,JLEV)
  !nyc!            PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT9(JROF,JLEV)&
  !nyc!             & +PXIGM*PCXNLT9 (JROF,JLEV)-PCXNLT99(JROF,JLEV)
  !nyc!          ENDDO
  !nyc!        ELSE
  !nyc!          DO JROF=KSTART,KPROF
  !nyc!            PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZMOYXNL(JROF)
  !nyc!            PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT9(JROF,JLEV)&
  !nyc!             & +PESGM*PCXNLT9(JROF,JLEV)
  !nyc!          ENDDO
  !nyc!        ENDIF
  !nyc!      ELSEIF (KXLAG == 3) THEN
  !nyc!        IF (LLSI_XIDT) THEN
  !nyc!          DO JROF=KSTART,KPROF
  !nyc!            PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+PXIGM*PCXNLT9(JROF,JLEV)&
  !nyc!             & -PCXNLT99(JROF,JLEV)
  !nyc!            PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PXIGP*ZMOYXNL(JROF)&
  !nyc!             & -PCXNLT99(JROF,JLEV)
  !nyc!          ENDDO
  !nyc!        ELSE
  !nyc!          DO JROF=KSTART,KPROF
  !nyc!            PXL0(JROF,JLEV)=PXL0(JROF,JLEV)+PESGM*PCXNLT9(JROF,JLEV)
  !nyc!            PXT1(JROF,JLEV)=PXT1(JROF,JLEV)+PESGP*ZMOYXNL(JROF)
  !nyc!          ENDDO
  !nyc!        ENDIF
  !nyc!        DO JROF=KSTART,KPROF
  !nyc!          PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT9(JROF,JLEV)
  !nyc!        ENDDO
  !nyc!      ENDIF

  !nyc!    ENDDO

ENDIF

!########################################################
! 2.3 Addition of preliminary quantity for LAGPHY physics
!########################################################

! the only difference with LATTEX_DNT in this part is that
!  the multiplication of PXSI by PXIGP or PESGP is not done.

DO JLEV=1,KFLEV
  IF(LLSI_XIDT)THEN
    DO JROF=KSTART,KPROF
      PXT1(JROF,JLEV)=PXT1(JROF,JLEV)-PXIGP*PXSI(JROF,JLEV)
    ENDDO
  ELSE
    DO JROF=KSTART,KPROF
      PXT1(JROF,JLEV)=PXT1(JROF,JLEV)-PESGP*PXSI(JROF,JLEV)
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEX_DNT5',1,ZHOOK_HANDLE)
END SUBROUTINE LATTEX_DNT5

