SUBROUTINE LAVABOTL(KST,KPROF,LD2TLFF1,PB15,PB1)

!**** *LAVABOTL* Semi-Lagrangian scheme.  (tangent-linear version)
!                VAlues at the BOundaries: Upper and lower extrapolations.

!     Purpose.
!     --------
!          This subroutine do upper and lower extrapolations of
!          semi-lagrangian quantities for extra-levels JLEV=0
!          and JLEV=NFLEVG+1.

!**   Interface.
!     ----------
!        *CALL* *LAVABOTL(......)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          LD2TLFF1- .T./.F.: Refined treatement of (2*Omega Vec r) at
!                    the origin point when there is t-dt (or t in SL2TL)
!                    physics / Other cases.

!        INPUT/OUTPUT:
!          PB15    - trajectory version of PB1.
!          PB1     - "SLBUF1" buffer for interpolations.


!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None
!        Called by LACDYNTL.

!     Reference.
!     ----------
!        Arpege documentation Part I  Chapter 3
!                             Part II Chapter 5 and 6
!        Documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 99/07/21
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA    ,NFLEN
USE YOMDYN   , ONLY : NVLAG    ,NWLAG    ,NTLAG
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB15
USE YOMSLPHY , ONLY : LSLPHY
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1T9  ,MSLB1GFL9 ,&
 & MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,MSLB1UR9 ,MSLB1VR9 ,MSLB1U0  ,MSLB1V0  ,&
 & MSLB1T0  ,MSLB1C9  ,MSLB1UP9 ,MSLB1VP9 ,MSLB1TP9 ,MSLB1GFLP9
USE PTRSLB15 , ONLY : MSLB1U95 ,MSLB1V95 ,MSLB1T95 ,MSLB1GFL95,MSLB1C95 ,&
 & MSLB1U05 ,MSLB1V05 ,MSLB1T05 ,MSLB1UR05,MSLB1VR05,MSLB1WR05,&
 & MSLB1UR95,MSLB1VR95
USE YOM_YGFL , ONLY : YGFL    ,YGFLC

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LD2TLFF1 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB15(NPROMA,NFLDSLB15)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JGFL, ISLB1GFL9, ISLB1GFL95, ISLB1GFLP9
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVABOTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    UPPER AND LOWER LATERAL BOUNDARIES CONDITIONS SET TO 0.
!              --------------------------------------------------------

!        1.1   TRAJECTORY CODE
!              This part could be put in a routine named LAVABO5.

! * P[X]RL05 and P[X]RL95 for [X]=U,V,W:
PB15(KST:KPROF,MSLB1UR05)=0.0_JPRB
PB15(KST:KPROF,MSLB1UR05+NFLEVG+1)=0.0_JPRB
PB15(KST:KPROF,MSLB1VR05)=0.0_JPRB
PB15(KST:KPROF,MSLB1VR05+NFLEVG+1)=0.0_JPRB
PB15(KST:KPROF,MSLB1WR05)=0.0_JPRB
PB15(KST:KPROF,MSLB1WR05+NFLEVG+1)=0.0_JPRB
IF (LD2TLFF1) THEN
  PB15(KST:KPROF,MSLB1UR95)=0.0_JPRB
  PB15(KST:KPROF,MSLB1UR95+NFLEVG+1)=0.0_JPRB
  PB15(KST:KPROF,MSLB1VR95)=0.0_JPRB
  PB15(KST:KPROF,MSLB1VR95+NFLEVG+1)=0.0_JPRB
ENDIF

! * P[X]L05 and P[X]L95 for [X]=U,V,T,advected GFL:
PB15(KST:KPROF,MSLB1U95)=0.0_JPRB
PB15(KST:KPROF,MSLB1U95+NFLEVG+1)=0.0_JPRB
PB15(KST:KPROF,MSLB1V95)=0.0_JPRB
PB15(KST:KPROF,MSLB1V95+NFLEVG+1)=0.0_JPRB
PB15(KST:KPROF,MSLB1T95)=0.0_JPRB
PB15(KST:KPROF,MSLB1T95+NFLEVG+1)=0.0_JPRB
DO JGFL=1,YGFL%NUMFLDS
  IF( YGFLC(JGFL)%LADV ) THEN
    ISLB1GFL95=MSLB1GFL95+(YGFLC(JGFL)%MP_SL1-1)*(NFLEN-NFLSA+1)
    PB15(KST:KPROF,ISLB1GFL95)=0.0_JPRB
    PB15(KST:KPROF,ISLB1GFL95+NFLEVG+1)=0.0_JPRB
  ENDIF
ENDDO

IF (NWLAG /= 2) THEN
  PB15(KST:KPROF,MSLB1U05)=0.0_JPRB
  PB15(KST:KPROF,MSLB1U05+NFLEVG+1)=0.0_JPRB
  PB15(KST:KPROF,MSLB1V05)=0.0_JPRB
  PB15(KST:KPROF,MSLB1V05+NFLEVG+1)=0.0_JPRB
ENDIF
IF (NTLAG /= 2) THEN
  PB15(KST:KPROF,MSLB1T05)=0.0_JPRB
  PB15(KST:KPROF,MSLB1T05+NFLEVG+1)=0.0_JPRB
ENDIF

! * P[X]L05 and P[X]L95 for [X]=SPD,VWV,NHX: not yet coded

! * P[X]L05 and P[X]L95 for [X]=C (continuity equation):
IF (NVLAG == 2.OR.NVLAG == 3) THEN
  PB15(KST:KPROF,MSLB1C95)=0.0_JPRB
  PB15(KST:KPROF,MSLB1C95+NFLEVG+1)=0.0_JPRB
  ! case (LSLINLC1.OR.LSLINLC2): not yet coded.
ENDIF

! * P[X]LP95 for [X]=U,V,T,advected GFL: not yet coded.
! * P[X]L95 for [X]=VCRSSA: not yet coded.
! * P[X]L95_SI (GMV equations, code under LSLINL): not yet coded.


!        1.2   TL CODE
!              This part must be identical to the content of LAVABO for
!              the options existing in the TL code, because LAVABO only
!              contains linear code (could be replaced by a call of LAVABO).

! * P[X]RL0 and P[X]RL9 for [X]=U,V,W:
PB1(KST:KPROF,MSLB1UR0)=0.0_JPRB
PB1(KST:KPROF,MSLB1UR0+NFLEVG+1)=0.0_JPRB
PB1(KST:KPROF,MSLB1VR0)=0.0_JPRB
PB1(KST:KPROF,MSLB1VR0+NFLEVG+1)=0.0_JPRB
PB1(KST:KPROF,MSLB1WR0)=0.0_JPRB
PB1(KST:KPROF,MSLB1WR0+NFLEVG+1)=0.0_JPRB
IF (LD2TLFF1) THEN
  PB1(KST:KPROF,MSLB1UR9)=0.0_JPRB
  PB1(KST:KPROF,MSLB1UR9+NFLEVG+1)=0.0_JPRB
  PB1(KST:KPROF,MSLB1VR9)=0.0_JPRB
  PB1(KST:KPROF,MSLB1VR9+NFLEVG+1)=0.0_JPRB
ENDIF

! * P[X]L0 and P[X]L9 for [X]=U,V,T,advected GFL:
PB1(KST:KPROF,MSLB1U9)=0.0_JPRB
PB1(KST:KPROF,MSLB1U9+NFLEVG+1)=0.0_JPRB
PB1(KST:KPROF,MSLB1V9)=0.0_JPRB
PB1(KST:KPROF,MSLB1V9+NFLEVG+1)=0.0_JPRB
PB1(KST:KPROF,MSLB1T9)=0.0_JPRB
PB1(KST:KPROF,MSLB1T9+NFLEVG+1)=0.0_JPRB
DO JGFL=1,YGFL%NUMFLDS
  IF( (YGFLC(JGFL)%CSLINT /= 'LAITVSPCQM  ').AND.YGFLC(JGFL)%LADV ) THEN
    ISLB1GFL9=MSLB1GFL9+(YGFLC(JGFL)%MP_SL1-1)*(NFLEN-NFLSA+1)
    PB1(KST:KPROF,ISLB1GFL9)=0.0_JPRB
    PB1(KST:KPROF,ISLB1GFL9+NFLEVG+1)=0.0_JPRB
  ENDIF
ENDDO

IF (NWLAG /= 2) THEN
  PB1(KST:KPROF,MSLB1U0)=0.0_JPRB
  PB1(KST:KPROF,MSLB1U0+NFLEVG+1)=0.0_JPRB
  PB1(KST:KPROF,MSLB1V0)=0.0_JPRB
  PB1(KST:KPROF,MSLB1V0+NFLEVG+1)=0.0_JPRB
ENDIF
IF (NTLAG /= 2) THEN
  PB1(KST:KPROF,MSLB1T0)=0.0_JPRB
  PB1(KST:KPROF,MSLB1T0+NFLEVG+1)=0.0_JPRB
ENDIF

! * P[X]L0 and P[X]L9 for [X]=SPD,VWV,NHX: not yet coded.

! * P[X]L0 and P[X]L9 for [X]=C (continuity equation):
IF (NVLAG == 2.OR.NVLAG == 3) THEN
  PB1(KST:KPROF,MSLB1C9)=0.0_JPRB
  PB1(KST:KPROF,MSLB1C9+NFLEVG+1)=0.0_JPRB
  ! case (LSLINLC1.OR.LSLINLC2): not yet coded.
ENDIF

! * P[X]LP9 for [X]=U,V,T,advected GFL:
IF( LSLPHY ) THEN
  ! ky: it seems to miss the zeroing of the bottom condition;
  !     is it JGFL or YGFLC(JGFL)%MP_SL1 in ISLB1GFLP9?
  PB1(KST:KPROF,MSLB1UP9+NFLEVG+1)=0.0_JPRB
  PB1(KST:KPROF,MSLB1VP9+NFLEVG+1)=0.0_JPRB
  PB1(KST:KPROF,MSLB1TP9+NFLEVG+1)=0.0_JPRB
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LADV) THEN
      ISLB1GFLP9=MSLB1GFLP9+(JGFL-1)*(NFLEN-NFLSA+1)
      PB1(KST:KPROF,ISLB1GFLP9+NFLEVG+1)=0.0_JPRB
    ENDIF
  ENDDO
ENDIF

! * P[X]L9 for [X]=VCRSSA: not yet coded.
! * P[X]L9_SI (GMV equations, code under LSLINL): not yet coded.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVABOTL',1,ZHOOK_HANDLE)
END SUBROUTINE LAVABOTL

