SUBROUTINE SIDDAD(KLEV,KLON,PDH,PDV,PRNH,PT,PSP,KNLON)

!**** * SIDDAD* - Provide 3D divergence increment in semi-implicit
!                 scheme for the case of nonhydrostatic dynamics -
!                 adjoint model

!     Purpose.
!     --------
!           Operators gamma, h, L.

!**   Interface.
!     ----------
!        *CALL* *SIDDAD(KLEV,KLON,PDH,PDV,PRNH,PT,PSP,KNLON)

!        Explicit arguments :
!        --------------------
!        KLEV   : DISTANCE IN MEMORY BETWEEN VALUES OF THE DIVERGENCE
!                OR TEMPERATURE AT THE VERTICAL
!        KLON   : DISTANCE IN MEMORY BETWEEN VALUES OF THE DIVERGENCE
!                OR TEMPERATURE AT THE SAME LEVEL

!           TYPICAL VALUES ARE  NDLSUR,1  FOR GRID POINT ARRAY
!                               1,NFLSUR  FOR SPECTRAL ARRAY

!        PDH   : HORIZONTAL DIVERGENCE
!        PDV   : VERTICAL DIVERGENCE VARIABLE
!        PRNH  : NONHYDROSTATIC PRESSURE DEPARTURE VARIABLE
!        PT    : TEMPERATURE
!        PSP   : LOG(SURFACE PRESSURE)
!        KNLON : NUMBER OF VERTICAL COLUMNS TREATED

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Arpege/Aladin Documentation

!     Author.
!     -------
!        Martin Charron   CNRM/GMAP - visiteur

!     Modifications.
!     --------------
!        Original : November 1998
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG       ,RD
USE YOMDYN   , ONLY : SITR

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLON 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDH(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDV(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRNH(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP(KNLON) 
REAL(KIND=JPRB) :: ZW3D(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON)
REAL(KIND=JPRB) :: ZW2D(KNLON)

INTEGER(KIND=JPIM) :: IAD, ILEV, ILON, INLON, JLEV, JLON

REAL(KIND=JPRB) :: ZDIM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sisevead.intfb.h"
#include "sigamad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIDDAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

! ky: caution: this piece of code is maybe not properly coded for
!     NPDVAR=2, NVDVAR=3 or 4.

ILEV = KLEV
ILON = KLON
INLON= KNLON

ZW3D=0.0_JPRB
ZW2D=0.0_JPRB

ZDIM = RG*RG/(RD*SITR)
DO JLEV = NFLEVG,1,-1
  DO JLON = INLON,1,-1
    IAD  = 1+(JLEV-1)*ILEV+(JLON-1)*ILON
    ZW3D(IAD)=ZW3D(IAD)+ZDIM*PDV(IAD)
    PDV(IAD)=0.0_JPRB
  ENDDO
ENDDO

CALL SISEVEAD(ILEV,ILON,PRNH,ZW3D,INLON)

DO JLEV = NFLEVG,1,-1
  DO JLON = INLON,1,-1
    IAD  = 1+(JLEV-1)*ILEV+(JLON-1)*ILON
    ZW3D(IAD)=ZW3D(IAD)-SITR*PDH(IAD)
    PRNH(IAD)=PRNH(IAD)+RD*SITR*PDH(IAD)
  ENDDO
ENDDO

CALL SIGAMAD(ILEV,ILON,ZW3D,PRNH,ZW2D,INLON)

DO JLON = 1, INLON
  ZW2D(JLON)  = 0.0_JPRB
ENDDO

CALL SIGAMAD(ILEV,ILON,PDH,PT,PSP,INLON)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SIDDAD',1,ZHOOK_HANDLE)
END SUBROUTINE SIDDAD
