SUBROUTINE SPAY(CDCONF)

! SPAY - Update of R.H.S. for S.-I. solver in non hydrostatic
!        Case LELAM=F.

! Purpose
! -------
!   It retrieves the temporarily saved original R.H.S.
!   from SPAxPCO tables and makes correction by Y term

!   This routine is called only for the corrector step of lpc_old=true.

! Interface
! ---------
!  * INPUT:
!   CDCONF - configuration of work.

! Externals
! ---------
!   None

! Method
! ------

! Reference
! ---------
!   Documentation of ARPEGE/ALADIN

! Author
! ------
!   K. Yessad (after routine ESPAY).
!   13-Dec-2004

! Modifications
! -------------
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMSPPCO , ONLY : SPA3PCO  ,SPA2PCO  
USE YOMCST   , ONLY : RD       ,RCPD     ,RCVD     
USE YOMDIM   , ONLY : NFLEVL   ,NFLSUR   ,NSPEC2   
USE YOMDYN   , ONLY : SITR     ,NRUBC    ,NSITER   ,NCURRENT_ITER  
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SPT      ,SPSPD    
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!------------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

!------------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZYITER(NFLSUR,NSPEC2)

INTEGER(KIND=JPIM) :: JLEV, JSE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------------------

#include "abor1.intfb.h"

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPAY',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

IF (CDCONF == 'A' .OR. CDCONF == 'I' .OR. CDCONF == 'S') THEN

!*    1. Copy "Y" to local array.
!     ---------------------------

  ! * At this stage Y is stored in SPSPD.
  ZYITER(:,:)=SPSPD(:,:)

  IF (NRUBC > 0) THEN
    ! * NRUBC>0 not yet coded in ARPEGE.
    CALL ABOR1(' SPAY: NRUBC > 0 not yet coded!')
  ENDIF

!------------------------------------------------------------------------------

!*    2. Retrieve original R.H.S.
!     ---------------------------

  SPA3(:,:,:)=SPA3PCO(:,:,:)
  SPA2(:,:)=SPA2PCO(:,:)

!------------------------------------------------------------------------------

!     3. Comput. of (m*Eta_top_dot) spectr. coef.
!     -------------------------------------------

  IF (NRUBC > 0) THEN
    ! Not yet coded.
  ENDIF

!------------------------------------------------------------------------------

!*    4. Make corrections related to RUBC.
!     ------------------------------------

  IF (NRUBC > 0) THEN
    ! Not yet coded.
  ENDIF

!------------------------------------------------------------------------------

!*    5. Make corrections for NH case + deallocations.
!     ------------------------------------------------

  DO JLEV=1,NFLEVL
    DO JSE=1,NSPEC2
      SPT(JLEV,JSE)=SPT(JLEV,JSE)-(RD*SITR/RCVD)*ZYITER(JLEV,JSE)
      SPSPD(JLEV,JSE)=SPSPD(JLEV,JSE)-(RCPD/RCVD)*ZYITER(JLEV,JSE)
    ENDDO
  ENDDO

  IF (NCURRENT_ITER == NSITER) THEN
    DEALLOCATE(SPA3PCO)
    DEALLOCATE(SPA2PCO)
  ENDIF

ENDIF

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPAY',1,ZHOOK_HANDLE)
END SUBROUTINE SPAY
