SUBROUTINE SPCIMPFPOSTAD(CDCONF,KM,KSTA,KEND,PSPVOR,PSPDIV)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MYSETV   ,NPTRSV   ,NPTRSVF
USE YOMDIM   , ONLY : NFLEVL   ,NSMAX
USE YOMCST   , ONLY : ROMEGA
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LTWOTL
USE YOMDYN   , ONLY : TDT      ,BETADT   ,VESL     ,XIDT
USE YOMGEM   , ONLY : RSTRET


!**** *SPCIMPFPOSTAD* - INCREMENT VORTICITY FOR IMPLICIT CORIOLIS CASE - ADJOINT

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SPCIMPFINITAD(..)

!        Explicit arguments :  CDCONF - configuration (see doc.)
!        --------------------  KM     - Zonal wavenumber 
!                              KSTA   - First column processed
!                              KEND   - Last columns processed
!                              PSPDIV - Distributed divergence
!                              PSPVOR - Distributed vorticity
!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Tomas Wilhelmsson *ECMWF*

!     Modifications.
!     --------------
!       30-09-2009 Original with code extracted from spcsiad.F90
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPVOR(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDIV(:,:) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZALPHA(0:NSMAX+1)
REAL(KIND=JPRB) :: ZDENIM(0:NSMAX+1)
REAL(KIND=JPRB) :: ZEPSI(0:NSMAX)
REAL(KIND=JPRB) :: ZFPLUS(-1:NSMAX+1)
REAL(KIND=JPRB) :: ZFMINUS(0:NSMAX+2)

INTEGER(KIND=JPIM) :: II, ILO, IN, JLEV, JN, IMSP, IRSP, JL  

LOGICAL :: LL3D,LLDOSI

REAL(KIND=JPRB) :: ZAL, ZBDT, ZBDT2, ZEM, ZEN, ZF, ZTEMP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCIMPFPOSTAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LL3D=(NCONF == 1.OR.NCONF == 131.OR.NCONF == 302.OR.NCONF == 401 &
   & .OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801)

LLDOSI=LL3D.AND.(CDCONF == 'A'.OR.CDCONF == 'I'.OR. &
 & CDCONF == 'S' .OR. CDCONF == 'T')

IF (LLDOSI) THEN

!*       1.    SEMI-IMPLICIT SPECTRAL COMPUTATIONS.
!              ------------------------------------

!*        1.1  Preliminary initialisations.

  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT*(1.0_JPRB+XIDT)
    ELSE
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT
    ENDIF
  ELSE
    ZBDT=0.5_JPRB*TDT*BETADT
  ENDIF
  ZBDT2=(ZBDT*RSTRET)**2

!            Set up helper arrays for implicit Coriolis case

  ZEM=REAL(KM,JPRB)
  ZAL=2.0_JPRB*ZBDT*ROMEGA*ZEM
  ILO=KM
  IF (KM == 0) THEN
    ZALPHA(0)=0.0_JPRB
    ZDENIM(0)=0.0_JPRB
    ZEPSI(0)=0.0_JPRB
    ILO=1
  ENDIF
  DO JL=ILO,NSMAX
    ZEN=REAL(JL,JPRB)
! !!! N.B. ZALPHA is set up with the negative of the value used in
!     SPCSI, in order to simplify the manipulations with complex numbers
    ZALPHA(JL)=-ZAL/(ZEN*(ZEN+1.0_JPRB))
    ZDENIM(JL)=1.0_JPRB/(1.0_JPRB+ZALPHA(JL)**2)
    ZEPSI(JL)=SQRT((ZEN*ZEN-ZEM*ZEM)/(4.0_JPRB*ZEN*ZEN-1.0_JPRB))
  ENDDO
  ZALPHA(NSMAX+1)=0.0_JPRB
  ZDENIM(NSMAX+1)=0.0_JPRB

  IF (KM == 0) THEN
    ZFPLUS(0)=0.0_JPRB
    ZFMINUS(0)=0.0_JPRB
  ENDIF
  ZF=2.0_JPRB*ZBDT*ROMEGA
  DO JL=ILO,NSMAX-1
    ZEN=REAL(JL,JPRB)
    ZFPLUS(JL)=ZF*ZEN*ZEPSI(JL+1)/(ZEN+1.0_JPRB)
    ZFMINUS(JL)=ZF*(ZEN+1.0_JPRB)*ZEPSI(JL)/ZEN
  ENDDO
  ZEN=REAL(NSMAX,JPRB)
  ZFPLUS(NSMAX)=0.0_JPRB
  ZFMINUS(NSMAX)=ZF*(ZEN+1.0_JPRB)*ZEPSI(NSMAX)/ZEN
  ZFPLUS(NSMAX+1)=0.0_JPRB
  ZFMINUS(NSMAX+1)=0.0_JPRB
  ZFMINUS(NSMAX+2)=0.0_JPRB
  
!*       1.5  Increment vorticity

  IF (KM == 0) THEN
    DO JN=2,NSMAX
      DO JLEV=1,NFLEVL
        IRSP=KSTA+(JN-KM)*2
        PSPDIV(JLEV,IRSP-2)=PSPDIV(JLEV,IRSP-2)&
          & -ZDENIM(JN)*ZFMINUS(JN)*PSPVOR(JLEV,IRSP)  
      ENDDO
    ENDDO
    DO JN=1,NSMAX-1
      DO JLEV=1,NFLEVL
        IRSP=KSTA+(JN-KM)*2
        PSPDIV(JLEV,IRSP+2)=PSPDIV(JLEV,IRSP+2)&
          & -ZDENIM(JN)*ZFPLUS(JN)*PSPVOR(JLEV,IRSP)  
      ENDDO
    ENDDO
  ELSE
    DO JN=KM+1,NSMAX
      DO JLEV=1,NFLEVL
        IRSP=KSTA+(JN-KM)*2
        IMSP=IRSP+1
        PSPDIV(JLEV,IRSP-2)=PSPDIV(JLEV,IRSP-2)&
          & -ZDENIM(JN)*ZFMINUS(JN)*(PSPVOR(JLEV,IRSP)&
          & -ZALPHA(JN)*PSPVOR(JLEV,IMSP))  
        PSPDIV(JLEV,IMSP-2)=PSPDIV(JLEV,IMSP-2)&
          & -ZDENIM(JN)*ZFMINUS(JN)*(PSPVOR(JLEV,IMSP)&
          & +ZALPHA(JN)*PSPVOR(JLEV,IRSP))  
      ENDDO
    ENDDO
    DO JN=KM,NSMAX-1
      DO JLEV=1,NFLEVL
        IRSP=KSTA+(JN-KM)*2
        IMSP=IRSP+1
        PSPDIV(JLEV,IRSP+2)=PSPDIV(JLEV,IRSP+2)&
          & -ZDENIM(JN)*ZFPLUS(JN)*(PSPVOR(JLEV,IRSP)&
          & -ZALPHA(JN)*PSPVOR(JLEV,IMSP))  
        PSPDIV(JLEV,IMSP+2)=PSPDIV(JLEV,IMSP+2)&
          & -ZDENIM(JN)*ZFPLUS(JN)*(PSPVOR(JLEV,IMSP)&
          & +ZALPHA(JN)*PSPVOR(JLEV,IRSP))  
      ENDDO
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCIMPFPOSTAD',1,ZHOOK_HANDLE)
END SUBROUTINE SPCIMPFPOSTAD
