SUBROUTINE SPCIMPFSOLVE(&
 ! --- INPUT -----------------------------------------------------------------
 & LDONEM,PSDIVP, &
 ! --- OUTPUT ----------------------------------------------------------------
 & PSPDIVP)  

USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK

USE YOMLAP   , ONLY : MYMS, NASM0
USE YOMMP    , ONLY : MYSETN, NPTRMF, NSPSTAF, MYLEVS
USE YOMDIM   , ONLY : NFLEVG, NFLEVL, NFLSUR, NSMAX, NSPEC2, NUMP
USE YOMCST   , ONLY : ROMEGA
USE YOMCT0   , ONLY : LSLAG, LTWOTL
USE YOMDYN   , ONLY : SIVP, TDT, BETADT, VESL, XIDT  
USE YOMLAP   , ONLY : RLAPDI
USE YOMGEM   , ONLY : RSTRET

!**** *SPCIMPFSOLVE* - CALL SIMPLICO FROM S-SPACE

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SPCIMPFSOLVE(..)

!        Explicit arguments :  LDONEM  - T if only one m if processed
!                              PSDIVP  - right hand side
!                              PSPDIVP - solution

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Tomas Wilhelmsson  *ECMWF*

!     Modifications.
!     --------------
!        21-09-2009 Original with extracted code from spcsi.F90
!     ------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDONEM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDIVP (:,:)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPDIVP(:,:)

!     ------------------------------------------------------------------


REAL(KIND=JPRB) :: ZALPHA (0:NSMAX+1)
REAL(KIND=JPRB) :: ZDENIM (0:NSMAX+1)
REAL(KIND=JPRB) :: ZEPSI  (0:NSMAX)
REAL(KIND=JPRB) :: ZFPLUS (0:NSMAX+1)
REAL(KIND=JPRB) :: ZFMINUS(0:NSMAX+1)
REAL(KIND=JPRB) :: ZSIVP(NFLEVL)

REAL(KIND=JPRB),ALLOCATABLE :: ZSDIVP (:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSPDIVP(:,:)

INTEGER(KIND=JPIM) :: ILO, IM, ISTA, IEND, JL, JMLOC

LOGICAL :: LL3D,LLDOSI,LLDOVERFLT

REAL(KIND=JPRB) :: ZAL, ZBDT, ZBDT2, ZEM, ZEN, ZF, ZTEMP, ZCT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "simplico.h"
#include "trmtos.intfb.h"
#include "trstom.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCIMPFSOLVE',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

ALLOCATE(ZSDIVP (NFLSUR,NSPEC2))
ALLOCATE(ZSPDIVP(NFLSUR,NSPEC2))

!     ------------------------------------------------------------------

!*       2.    SEMI-IMPLICIT SPECTRAL COMPUTATIONS.
!              ------------------------------------

!*        2.1  Preliminary initialisations.

IF (LSLAG) THEN
  IF (LTWOTL) THEN
    ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT*(1.0_JPRB+XIDT)
  ELSE
    ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT
  ENDIF
ELSE
  ZBDT=0.5_JPRB*TDT*BETADT
ENDIF
ZBDT2=(ZBDT*RSTRET)**2

DO JL=1,NFLEVL
  ZSIVP(JL) = SIVP(MYLEVS(JL))
ENDDO

CALL GSTATS(9,2)
CALL GSTATS(55,0)
CALL TRSTOM(PSPDIVG=PSDIVP,PSPDIV=ZSDIVP,LDFULLM=LDONEM)
CALL GSTATS(55,2)
CALL GSTATS(9,3)

CALL GSTATS(1028,0)
!$OMP PARALLEL DO SCHEDULE(STATIC,1) PRIVATE(JMLOC,IM,ISTA,IEND) &
!$OMP&PRIVATE(ZEM,ZAL,ILO,ZALPHA,ZDENIM,ZEPSI,ZEN,ZFPLUS,ZFMINUS)
DO JMLOC=1,NUMP

  IM=MYMS(JMLOC)
  ISTA=NASM0(IM)
  IEND=ISTA+2*(NSMAX+1-IM)-1

  !*        2.2  Set up helper arrays for implicit Coriolis.

  ZEM=REAL(IM,JPRB)
  ZAL=2.0_JPRB*ZBDT*ROMEGA*ZEM
  ILO=IM
  IF (IM == 0) THEN
    ZALPHA(0)=0.0_JPRB
    ZDENIM(0)=0.0_JPRB
    ZEPSI(0)=0.0_JPRB
    ILO=1
  ENDIF
  DO JL=ILO,NSMAX
    ZEN=REAL(JL,JPRB)
    ZALPHA(JL)=ZAL/(ZEN*(ZEN+1.0_JPRB))
    ZDENIM(JL)=1.0_JPRB/(1.0_JPRB+ZALPHA(JL)**2)
    ZEPSI(JL)=SQRT((ZEN*ZEN-ZEM*ZEM)/(4.0_JPRB*ZEN*ZEN-1.0_JPRB))
  ENDDO
  ZALPHA(NSMAX+1)=0.0_JPRB
  ZDENIM(NSMAX+1)=0.0_JPRB
  
  IF (IM == 0) THEN
    ZFPLUS(0)=0.0_JPRB
    ZFMINUS(0)=0.0_JPRB
  ENDIF
  ZF=2.0_JPRB*ZBDT*ROMEGA
  DO JL=ILO,NSMAX-1
    ZEN=REAL(JL,JPRB)
    ZFPLUS(JL)=ZF*ZEN*ZEPSI(JL+1)/(ZEN+1.0_JPRB)
    ZFMINUS(JL)=ZF*(ZEN+1.0_JPRB)*ZEPSI(JL)/ZEN
  ENDDO
  ZEN=REAL(NSMAX,JPRB)
  ZFPLUS(NSMAX)=0.0_JPRB
  ZFMINUS(NSMAX)=ZF*(ZEN+1.0_JPRB)*ZEPSI(NSMAX)/ZEN
  ZFPLUS(NSMAX+1)=0.0_JPRB
  ZFMINUS(NSMAX+1)=0.0_JPRB

!           Solve complex pentadiagonal system

  CALL SIMPLICO(IM,NSMAX,NFLEVL,NFLSUR,ZALPHA(IM),&
  & ZDENIM(IM),ZFPLUS(IM),ZFMINUS(IM),ZSIVP,RLAPDI(0),&
  & ZBDT2,ZSDIVP(1,ISTA),ZSPDIVP(1,ISTA))
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1028,1)

CALL GSTATS(9,2)
CALL GSTATS(55,3)
CALL TRMTOS(PSPDIV=ZSPDIVP,PSPDIVG=PSPDIVP,LDFULLM=LDONEM)  
CALL GSTATS(55,1)
CALL GSTATS(9,3)

!     ------------------------------------------------------------------

DEALLOCATE(ZSDIVP)
DEALLOCATE(ZSPDIVP)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCIMPFSOLVE',1,ZHOOK_HANDLE)
END SUBROUTINE SPCIMPFSOLVE
