SUBROUTINE SPCMASCOR( CDCONF )

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMP    , ONLY : MYSETW   ,NPROCM, NPSP
USE YOMLUN   , ONLY : NULERR
USE YOMCT0   , ONLY : NCONF, NPRINTLEV  
USE YOMCT3   , ONLY : NSTEP
USE YOMMASS  , ONLY : GMASSINC, GMASSI, GMASS0
USE YOMSP    , ONLY : SPSP
USE YOMLAP   , ONLY : NASM0

!**** *SPCMASCOR* - MASS CORRECTION IN SPECTRAL SPACE.
!                   "MASCOR" cheap formulation of mass corrector.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SPCMASCOR(..)

!        Explicit arguments :
!        -------------------- 
!        * INPUT
!          CDCONF  - configuration (see doc.)

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad (after old part 0.5 of SPCSI, with cleanings)

!     Modifications.
!     --------------
!        Original : 09-Dec-2004
!        simplify : Nils Wedi + Mats Hamrud 2008-02-08
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

!     ------------------------------------------------------------------

LOGICAL :: LL3D, LLMASCOR, LLDO
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCMASCOR',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LLDO=(MYSETW == NPROCM(0).AND.(NPSP==1))
LL3D=(NCONF == 1.OR.NCONF == 131.OR.NCONF == 302.OR.NCONF == 401 &
 & .OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801)
LLMASCOR=LL3D.AND.(CDCONF == 'A'.OR.CDCONF == 'I').AND.LLDO

!     ------------------------------------------------------------------

!*       1.    GLOBAL MASS CORRECTION.
!              -----------------------

IF(LLMASCOR) THEN

  IF (NPRINTLEV >= 1 ) THEN
    WRITE(NULERR,*) ' '
    WRITE(NULERR,*) '*** SPCMASCOR at step ',NSTEP,' *** START ***'
    WRITE(NULERR,*) '   Initial mass = ',GMASSI
    WRITE(NULERR,*) '   Current mass = ',GMASS0
    WRITE(NULERR,*) '   Increment    = ',GMASSINC
    WRITE(NULERR,*) '   Uncorrected value:'
    WRITE(NULERR,*) '   ln(ps) (m=0,n=0) = ',SPSP(NASM0(0))
    WRITE(NULERR,*) '   Equiv.  mass = ',EXP(SPSP(NASM0(0)))
  ENDIF

  ! * apply mass correction.
  SPSP(NASM0(0))=SPSP(NASM0(0))-GMASSINC

  IF ( NPRINTLEV >= 1 ) THEN
    WRITE(NULERR,*) '   Corrected value:'
    WRITE(NULERR,*) '   ln(ps) (m=0,n=0) = ',SPSP(NASM0(0))
    WRITE(NULERR,*) '   Equiv.  mass = ',EXP(SPSP(NASM0(0)))
    WRITE(NULERR,*)'*** SPCMASCOR at step ',NSTEP,' ***  END  ***'
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCMASCOR',1,ZHOOK_HANDLE)
END SUBROUTINE SPCMASCOR
