SUBROUTINE SPECRT(KDIR,LDINCR,LDBCKGR,PSP7Q,PSP7I,PSP7L, &
     & PGP7Q,PGP7I,PGP7L)

!**** *SPECRT* - Computes spectral coeffs of 'TV' (TV defined as RT/Rd) or
!                spectral coeffs of T from TV and Q

!     Purpose.
!     --------
!           COMPUTES SPECTRAL COEFFICIENTS OF 'TV' (i.e., RT/Rd) or T

!**   Interface.
!     ----------
!        *CALL* *SPECRT(KDIR,[LDINCR],[LDBCKGR])

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :  
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Clive Temperton  *ECMWF*

!     Modifications.
!     --------------
!        Original : 94-05-16
!        Modified : 95-12-22 M.Hamrud    Change of ZT etc. layout
!        M.Hamrud : 95-10-20 Interface block for EXTGPF
!        F.Rabier : 95-12-12 Also computes T from TV and Q
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - one dummy argument less in CALL SPEREE.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified 04-02-06: Y. Seity : new arguments to GPRCP
!        M.Hamrud  15-Jan-2006  Revised GPRCP
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMSP    , ONLY : SPT      ,SPQ      ,SPL      ,SPI
USE YOMSPJB  , ONLY : SP7T
USE YOMGEM   , ONLY : NGPTOT
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NGPBLKS   ,NSPEC2
USE YOMMP    , ONLY : NBSETLEV
USE YOMCST   , ONLY : RD
USE YOMGMV   , ONLY : GMV5,GMV5S, YT5
USE YOMGFL   , ONLY : GFL, GFL5
USE YOM_YGFL , ONLY : YGFL, YQ, YL, YI, YR, YS, YG
USE TRAJECTORY_MOD,ONLY : LTRAJGP, GET_TRAJ_GRID
USE YOMCT3   , ONLY : NSTEP
USE YOMLUN   , ONLY : NULOUT
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIR 
LOGICAL, OPTIONAL ,INTENT(IN)    :: LDINCR
LOGICAL, OPTIONAL ,INTENT(IN)    :: LDBCKGR
REAL(KIND=JPRB), OPTIONAL, INTENT(IN)  :: PSP7Q(:,:)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN)  :: PSP7I(:,:)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN)  :: PSP7L(:,:)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN)  :: PGP7Q(:,:,:)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN)  :: PGP7I(:,:,:)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN)  :: PGP7L(:,:,:)
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZT(NPROMA,NFLEVG,NGPBLKS)
REAL(KIND=JPRB) :: ZR  (NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGFL7Q(NPROMA,NFLEVG,NGPBLKS),ZGFL7I(NPROMA,NFLEVG,NGPBLKS)
REAL(KIND=JPRB) :: ZGFL7L(NPROMA,NFLEVG,NGPBLKS)
LOGICAL         :: LLINCR,LLBCKGR

INTEGER(KIND=JPIM) :: IEND, IST, JKGLO, JL, JLEV, IBL  
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

INTERFACE
#include "inv_trans.h"
#include "dir_trans.h"
END INTERFACE

#include "gprcp.intfb.h"
#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPECRT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------


!*       1.    COMPUTES SPECTRAL COEFFS OF 'TV' OR T
!              -------------------------------------
!   If the fields contain increments we use the trajectory q field to
!   calculate the convertion from Tv to T. The GFL field contains
!   the humidity increment, so it cannot be used to do the convertion
IF(PRESENT(LDINCR)) THEN
  LLINCR=LDINCR
ELSE
  LLINCR=.FALSE.
ENDIF
IF(PRESENT(LDBCKGR)) THEN
  LLBCKGR=LDBCKGR
ELSE
  LLBCKGR=.FALSE.
ENDIF 
IF (LLINCR) THEN
  IF( LTRAJGP )THEN

!   Retrieve grid-point trajectory. We only need humidity

    IF (.NOT.ALLOCATED(GMV5)) ALLOCATE (GMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
    IF (.NOT.ALLOCATED(GMV5S)) ALLOCATE (GMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
    IF (.NOT.ALLOCATED(GFL5)) ALLOCATE (GFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
    CALL GET_TRAJ_GRID(GMV5,GMV5S,GFL5,KSTEP=NSTEP)
  ELSE
    IF(.NOT.ALLOCATED(GMV5)) CALL ABOR1('SCAN2MTL:GMV5 NOT ALLOCATED')
  ENDIF
ENDIF

CALL GSTATS(25,0)
IF (.NOT. LLBCKGR) THEN
  CALL INV_TRANS(PSPSCALAR=SPT,KVSETSC=NBSETLEV,KPROMA=NPROMA,PGP=ZT)
  IF (.NOT.LLINCR) THEN
    IF(YQ%LSP) THEN
      CALL INV_TRANS(PSPSCALAR=SPQ,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
       & PGP=GFL(:,:,YQ%MP,:))  
    ENDIF
    IF(YL%LSP) THEN
      CALL INV_TRANS(PSPSCALAR=SPL,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
       & PGP=GFL(:,:,YL%MP,:))  
    ENDIF
    IF(YI%LSP) THEN
      CALL INV_TRANS(PSPSCALAR=SPI,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
       & PGP=GFL(:,:,YI%MP,:))  
    ENDIF
  ENDIF

  CALL GSTATS(1038,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) &
!$OMP&PRIVATE(JKGLO,IST,IEND,JLEV,JL,IBL,ZR)
  DO JKGLO=1,NGPTOT,NPROMA
    IST =1
    IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    
    IF (.NOT.LLINCR) THEN
      CALL GPRCP(NPROMA,IST,IEND,NFLEVG,PGFL=GFL(1,1,1,IBL),PR=ZR)
    ELSE
      CALL GPRCP(NPROMA,IST,IEND,NFLEVG,PQ=GFL5(1,1,YQ%MP5,IBL),PR=ZR)
    ENDIF
    IF (KDIR == 0) THEN
      DO JLEV=1,NFLEVG
        DO JL=IST,IEND
          ZT(JL,JLEV,IBL)=ZR(JL,JLEV)*ZT(JL,JLEV,IBL)/RD
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        DO JL=IST,IEND
          ZT(JL,JLEV,IBL)=RD*ZT(JL,JLEV,IBL)/ZR(JL,JLEV)
        ENDDO
      ENDDO
    ENDIF
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1038,1)
  CALL DIR_TRANS(PSPSCALAR=SPT,KVSETSC=NBSETLEV,KPROMA=NPROMA,PGP=ZT)
ELSE
  CALL INV_TRANS(PSPSCALAR=SP7T,KVSETSC=NBSETLEV,KPROMA=NPROMA,PGP=ZT)
  IF(PRESENT(PSP7Q).AND.UBOUND(PSP7Q,2)==NSPEC2) THEN
    CALL INV_TRANS(PSPSCALAR=PSP7Q,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
     & PGP=ZGFL7Q)
    write(nulout,*) 'background T-->Tv: Q taken from spectral'
  ELSEIF (PRESENT(PGP7Q).AND.UBOUND(PGP7Q,1)==NPROMA.AND. &
     & UBOUND(PGP7Q,2)==NFLEVG.AND.UBOUND(PGP7Q,3)==NGPBLKS) THEN
    ZGFL7Q(:,:,:)=PGP7Q(:,:,:)
    write(nulout,*) 'background T-->Tv: Q taken from gridpoint'
  ELSE
    ZGFL7Q(:,:,:)=0.0_JPRB  
    write(nulout,*) 'background T-->Tv: Q is assumed to be zero'
  ENDIF
  IF(PRESENT(PSP7L).AND.UBOUND(PSP7L,2)==NSPEC2) THEN
    CALL INV_TRANS(PSPSCALAR=PSP7L,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
     & PGP=ZGFL7L)
    write(nulout,*) 'background T-->Tv: L taken from spectral'
  ELSEIF (PRESENT(PGP7L).AND.UBOUND(PGP7L,1)==NPROMA.AND. &
     & UBOUND(PGP7L,2)==NFLEVG.AND.UBOUND(PGP7L,3)==NGPBLKS) THEN
    ZGFL7L(:,:,:)=PGP7L(:,:,:)
    write(nulout,*) 'background T-->Tv: L taken from gridpoint'
  ELSE
    ZGFL7L(:,:,:)=0.0_JPRB
    write(nulout,*) 'background T-->Tv: L is assumed to be zero'
  ENDIF
  IF(PRESENT(PSP7I).AND.UBOUND(PSP7I,2)==NSPEC2) THEN
    CALL INV_TRANS(PSPSCALAR=PSP7I,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
     & PGP=ZGFL7I)
    write(nulout,*) 'background T-->Tv: I taken from spectral'
  ELSEIF (PRESENT(PGP7I).AND.UBOUND(PGP7I,1)==NPROMA.AND. &
     & UBOUND(PGP7I,2)==NFLEVG.AND.UBOUND(PGP7I,3)==NGPBLKS) THEN
    ZGFL7I(:,:,:)=PGP7I(:,:,:)
    write(nulout,*) 'background T-->Tv: I taken from gridpoint'
  ELSE
    ZGFL7I(:,:,:)=0.0_JPRB
    write(nulout,*) 'background T-->Tv: I is assumed to be zero'
  ENDIF
  CALL GSTATS(1038,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) &
!$OMP&PRIVATE(JKGLO,IST,IEND,JLEV,JL,IBL,ZR)
  DO JKGLO=1,NGPTOT,NPROMA
    IST =1
    IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    CALL GPRCP(NPROMA,IST,IEND,NFLEVG,&
      & PQ=ZGFL7Q(1,1,IBL),PQI=ZGFL7I(1,1,IBL),PQL=ZGFL7L(1,1,IBL),PR=ZR)

    IF (KDIR == 0) THEN
      DO JLEV=1,NFLEVG
        DO JL=IST,IEND
          ZT(JL,JLEV,IBL)=ZR(JL,JLEV)*ZT(JL,JLEV,IBL)/RD
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        DO JL=IST,IEND
          ZT(JL,JLEV,IBL)=RD*ZT(JL,JLEV,IBL)/ZR(JL,JLEV)
        ENDDO
      ENDDO
    ENDIF
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1038,1)
  CALL DIR_TRANS(PSPSCALAR=SP7T,KVSETSC=NBSETLEV,KPROMA=NPROMA,PGP=ZT)
ENDIF

CALL GSTATS(25,1)
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPECRT',1,ZHOOK_HANDLE)
END SUBROUTINE SPECRT
