C     *** CIT AIRSHED MODEL ***
C     INCLUDE FILE TO DEFINE MAXIMUM ARRAY SIZES
C     WRITTEN BY ROBERT HARLEY (FEBRUARY 1991)
C
C     CAUTIONARY NOTES:  IF CHANGES ARE MADE TO THIS FILE,
C     BE SURE TO CHECK THAT THE DEFINITION OF LBIGRC IS REVISED
C     (IF NGMX OR NGMY IS CHANGED).  ALSO, IF NEMAX OR NIMAX 
C     ARE CHANGED, VERIFY THE NEW DEFINITION OF LWORK.
C
C     NSMAX  - MAXIMUM NUMBER OF CHEMICAL SPECIES
C     NIMAX  - MAXIMUM NUMBER OF INITIAL CONDITION SPECIES
C     NEMAX  - MAXIMUM NUMBER OF EMITTED SPECIES 
C     NHCMAX - MAXIMUM NUMBER OF HYDROCARBON SPECIES
C     NLUMAX - MAXIMUM NUMBER OF LAND USE TYPES
C     NRMAX  - MAXIMUM NUMBER OF CHEMICAL REACTIONS
C     NPMAX  - MAXIMUM NUMBER OF POINT SOURCES   
C     NCMAX  - MAXIMUM NUMBER OF SPECIAL COEFFICIENTS
C     LWORK  - LENGTH OF THE WORK VECTOR
C
C  AEROSOL PARAMETERS
C
      PARAMETER (NASECT   =  8)    ! aerosol sections
      PARAMETER (NASPEC   = 37)    ! aerosol species 
       PARAMETER (IAPART   =  0)    ! transport calculations
c     PARAMETER (IAPART   =  1)    ! equilibrium calculations
      PARAMETER (IAAPP    =  0)    ! 0:full rigorous calculation; 1:w/o solids 
      PARAMETER (IAOPT    =  2)    ! 1: Bromley; 2: Kusk and Meissner; 3: Pitzer
      PARAMETER (IASEC    =  1)    ! (0/1): calculating or ignoring minor solids
      PARAMETER (IATX     = 20)    ! number of maximum iterations
      PARAMETER (AEPS1    =  0.001)! error tolerance for H+ and water convr.
      PARAMETER (AEPS2    =  0.01) ! error tolerance for solids convergence
      PARAMETER (DPUP     = 10.)        ! micrometer upper limit aerosol diameter 
      PARAMETER (DPLOW    =  0.039) ! micrometer lower limit aerosol diameter
      PARAMETER (DENSP    =  1.2)   ! aerosol density (g/cm3)
      PARAMETER (ACOURANT =  0.1)   ! courant number
      PARAMETER (IWATCAL  =  1)     ! 0:only do water evap/cond 1:full evap/cond
      PARAMETER (ISALTCAL =  0)     ! Sea-salt emissions: 0:read from file 1:calculate

      PARAMETER (NGSPEC = 124)
      PARAMETER (NAMAX = NASECT*NASPEC)
      PARAMETER (NSMAX = NGSPEC + NAMAX, NIMAX = 84, NEMAX = 172)
      PARAMETER (NHCMAX = 20, NLUMAX = 32, NRMAX = 361)
      PARAMETER (NPMAX = 1200, NCMAX = 47)
C
C     MAXIMUM NUMBER OF GRID SQUARES (DO NOT SET NGMZ LARGER THAN 10)
C     NGMX   - MAXIMUM NUMBER OF MASTER GRID CELLS IN X DIRECTION
C     NGMY   - MAXIMUM NUMBER OF MASTER GRID CELLS IN Y DIRECTION
C     NGMZ   - MAXIMUM NUMBER OF VERTICAL COMPUTATIONAL CELLS
C     LBIGRC - SET TO LARGER OF NGMX AND NGMY
C
      PARAMETER (NGMX = 1, NGMY = 1, NGMZ = 1)
      PARAMETER (LBIGRC = NGMX)
C
C     DEFINE LWORK USING THE LARGER OF NEMAX AND NIMAX
C
      PARAMETER (LWORK = NGMX * NGMY * NEMAX)
C
C     DERIVED PARAMETERS FOR SCALAR AND VECTOR ARRAY DIMENSIONS
C
C     LSCAL  - ELEMENTS IN A SCALAR (SURFACE LEVEL ONLY) ARRAY
C     LWIND  - ELEMENTS IN THE 3D WIND FIELD ARRAY
C     LAREAS - ELEMENTS IN GROUND-LEVEL EMISSIONS ARRAY
C     LC     - ELEMENTS IN SPECIES CONCENTRATION ARRAY
C     LIC    - ELEMENTS IN INITIAL CONDITION CONCENTRATION ARRAY
C     LAVG   - ELEMENTS IN HOURLY AVERAGE GROUND-LEVEL CONC ARRAY
C
      PARAMETER (LSCAL  =  NGMX * NGMY)
      PARAMETER (LWIND  =  NGMX * NGMY * (NGMZ+1))
      PARAMETER (LAREAS =  NGMX * NGMY * NEMAX)
      PARAMETER (LC     =  NGMX * NGMY * NGMZ * NSMAX)
      PARAMETER (LIC    =  NGMX * NGMY * NGMZ * NIMAX)
      PARAMETER (LAVG   =  NGMX * NGMY * NSMAX)
C
C     DERIVED PARAMETERS FOR BOUNDARY CONDITION ARRAYS
C
      PARAMETER (LBR   = NGMY * NGMZ * NSMAX)
      PARAMETER (LBRIC = NGMY * NGMZ * NIMAX)
      PARAMETER (LBC   = NGMX * NGMZ * NSMAX)
      PARAMETER (LBCIC = NGMX * NGMZ * NIMAX)
C
C     TOTAL NUMBER OF STORAGE LOCATIONS REQUIRED FOR
C     A VERTICAL COLUMN OF SPECIES CONCENTRATIONS
C
      PARAMETER (LCOLSP = NSMAX * NGMZ)
C
C     TOTAL NUMBER OF STORAGE LOCATIONS REQUIRED TO SPECIFY
C     ALL CHEMICAL REACTION RATES IN A SINGLE VERTICAL COLUMN 
C
      PARAMETER (NRZ = NRMAX * NGMZ)









