SUBROUTINE APLM1G(PF,PRXMIN,PDF1,KMODE,KNITER,KRZ)

!**** *APLM1G*

!     PURPOSE.
!     --------

!       Interface to the minimisation routine *M1QN3R*.

!**   INTERFACE.
!     ----------

!     *CALL* *APLM1G(PF,PRXMIN,PDF1,KMODE,KNITER,KRZ)*

!         *PF*     - Value of the function
!         *PRXMIN* - Minimum step (input)
!         *PDF1*   - Magnitude of the first step (input)
!         *KMODE*  - Diagnostic of the minimisation
!         *KNITER* - Maximum number of iterations (input)
!         *KRZ*    - Size of the working array

!     METHOD.
!     -------

!       Initializes additional parameters and calls *M1QN3R*.

!     EXTERNALS.
!     ----------

!       M1QN3R

!     AUTHOR.
!     -------

!       Y. BOUTELOUP and M. DEQUE   1 FEB 91.

!     Modifications.
!     --------------
!       J.N THEPAUT  : 93-04-07 (change of algorithm : M1QN3)
!       R. EL Khatib : 94-03-08 comdeck PTRSPOR
!       P. Le Moigne : 98-09-29 minimization parallelized.
!       D. Giard     : 01-11-22 printing unit + misc.
!       Y. Tremollet : 02-??-?? control_vectors used (CF)
!       C. Fischer   : 03-04-02 added ks1d as optional arg
!       M. Hamrud    : 01-Oct-2003 CY28 Cleaning
!       D. Giard     : 02-11-25 cleaning for minimization
!       F. Taillefer : 05-03-25 back to an old minimization code (M1QN3 --> M1QN3R)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSEFRE
USE PTRSPOR  , ONLY : XRNM     ,XSNM
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PF
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXMIN
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDF1
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNITER
INTEGER(KIND=JPIM),INTENT(INOUT) :: KRZ

REAL(KIND=JPRB) :: ZRZ(KRZ)
REAL(KIND=JPRB) :: ZXRNM(NSEFRE),ZXSNM(NSEFRE)
REAL(KIND=JPRB) :: ZEPSR, ZEPSS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTEGER(KIND=JPIM) :: IZ(5)
INTEGER(KIND=JPIM) :: IMPR, IOUT, ISIM, ISTEP

EXTERNAL SIMREL,PRSPL2
EXTERNAL SID1,SID2

#include "grtestr.intfb.h"

IF (LHOOK) CALL DR_HOOK('APLM1G',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IOUT = NULOUT
IMPR = 4
ZEPSR = 1.E-6_JPRB
ZEPSS = 1.E-12_JPRB
ISIM = 2*KNITER
ISTEP = 10

ZXRNM(:)=XRNM(:)
ZXSNM(:)=XSNM(:)

CALL GRTESTR(SIMREL,PRSPL2,NSEFRE,ZXRNM,PF,ZXSNM,ZEPSS,ISTEP)

CALL M1QN3R(SIMREL,PRSPL2,SID1,SID2,NSEFRE,ZXRNM,PF,ZXSNM,&
 & PRXMIN,PDF1,ZEPSR,IMPR,IOUT,KMODE,KNITER,ISIM,IZ,ZRZ,KRZ)

XRNM(:)=ZXRNM(:)
XSNM(:)=ZXSNM(:)

IF (LHOOK) CALL DR_HOOK('APLM1G',1,ZHOOK_HANDLE)
END SUBROUTINE APLM1G
