SUBROUTINE GEO923(KPINT,KLEN,PBO,PLONG,PMU,PSLAC,PSLOC,PCLOC)

!**** *GEO923*

!     PURPOSE.
!     --------
!      Compute the constants (YOMDIL) and fields related to the geometry
!      which are used by configuration 923.

!     INTERFACE.
!     ----------
!      CALL GEO923(KPINT,PMU,PSLAC,PSLOC,PCLOC)
!          KPINT : size of interpolation box                        ( input)
!          KLEN  : real size of grid                                (output)
!          PBO   : sine of the boundary latitudes of the NDGLG boxes|
!          PLONG : cosine of latitude on the computation grid       |
!          PMU   : sine of latitude on the computation grid         | output
!          PSLAC : sine of the latitudes of the subgrid             | fields
!          PSLOC : sine of the longitudes of the subgrid            |
!          PCLOC : cosine of the longitudes of the subgrid          |

!     AUTHORS.
!     --------
!      D. Giard 97-03-13 from INCLIx

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDLON
USE YOMDIL   , ONLY : NPOTYP   ,SLAPO    ,GLAPO    ,SLOPO    ,GLOPO    ,FACDI
USE YOMCST   , ONLY : RPI
USE YOMLEG   , ONLY : RW       ,RMU
USE YOMGEM   , ONLY : NLOEN    ,NSTTYP   ,RMUCEN   ,RLOCEN   ,RSTRET

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPINT 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLEN 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBO(NDGLG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLONG(NDGLG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMU(NDGLG*NDLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSLAC(NDGLG*KPINT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSLOC(NDLON*KPINT,NDGLG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLOC(NDLON*KPINT,NDGLG) 
INTEGER(KIND=JPIM) :: ILEN, IND, J, JJ

REAL(KIND=JPRB) :: ZC1, ZC2, ZC3, ZCJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!  YOMDIL

IF (LHOOK) CALL DR_HOOK('GEO923',0,ZHOOK_HANDLE)
SLAPO = RMUCEN
GLAPO = SQRT(1.0_JPRB-RMUCEN**2)
SLOPO = SIN(RLOCEN)
GLOPO = COS(RLOCEN)
FACDI = RSTRET
NPOTYP= NSTTYP

!  PBO = Sine of the boundary latitudes of the NDGLG boxes.

PBO(1) = 1.0_JPRB
DO JJ=1,NDGLG-1
  IND = MIN(JJ,NDGLG+1-JJ)
  PBO(JJ+1) = PBO(JJ) - 2.0_JPRB*RW(IND)
ENDDO
PBO(NDGLG+1) = -1.0_JPRB

!  PLONG = Cosine of latitude on the computation grid.

DO JJ=1,NDGLG
  IND = MIN(JJ,NDGLG+1-JJ)
  PLONG(JJ) = SQRT(1.0_JPRB-RMU(IND)**2)
ENDDO

!  PMU   = Sine of latitude on the computation grid.
!  PSLAC = Sine of the latitudes of the subgrid.
!  PSLOC = Sine of the longitudes of the subgrid.
!  PCLOC = Cosine of the longitudes of the subgrid.

ZC1 = 1.0_JPRB/REAL(KPINT,JPRB)
ZC2 = 2.0_JPRB*RPI*ZC1
ZC3 = 0.5_JPRB*REAL(1+KPINT,JPRB)
ILEN = 0
DO JJ=1,NDGLG
  ZCJ = ZC2/REAL(NLOEN(JJ),JPRB)
  DO J=1,NLOEN(JJ)
    PMU(J+ILEN) = RMU(JJ)
  ENDDO
  DO J=1,KPINT
    PSLAC((JJ-1)*KPINT+J) = PBO(JJ) + ZC1*(PBO(JJ+1)-PBO(JJ))*(J-0.5_JPRB)
  ENDDO
  DO J=1,NLOEN(JJ)*KPINT
    PSLOC(J,JJ) = SIN( ZCJ*(J-ZC3) )
    PCLOC(J,JJ) = COS( ZCJ*(J-ZC3) )
  ENDDO
  ILEN = ILEN + NLOEN(JJ)
ENDDO
KLEN = ILEN

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GEO923',1,ZHOOK_HANDLE)
END SUBROUTINE GEO923
