SUBROUTINE INCLI0

!**** *INCLI0*

!     PURPOSE.
!     --------

!     This routine prepares the calculation of model climatological constants.

!**   INTERFACE.
!     ----------

!     CALL INCLI0

!     METHOD.
!     -------

!     The parameters are read on *NULNAM* .Then the memory is allocated.
!     Finally, the active routine is called.

!     EXTERNALS.
!     ----------

!      SUALCLIA , DEALCLIA
!      INTER0 , EINTER0
!      INCLI1 , EINCLI1
!      INCLI2 , EINCLI2
!      INCLI3 , EINCLI3
!      INCLI4 , EINCLI4
!      INCLI5 , EINCLI5
!      INCLI6 , EINCLI6
!      INCLI7 , EINCLI7
!      INCLI8 , EINCLI8
!      INCLI9 , EINCLI9
!      INCLI10, EINCLI10
!      INCLIB , EINCLIB
!      INCLIR , EINCLIR

!     AUTHORS.
!     --------

!       M. DEQUE   1 FEB 91.

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!      M. Deque     : 92-10-01 Inclusion of all climatic routines
!      R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!      R. EL Khatib : 93-05-13 Defaults values for NPNPP or when stretching
!      Y. Bouteloup : 93-08-26 New cost function controled by LNEWORO
!      R. EL Khatib : 94-03-08 comdecks YOMCLA, YOMCLB, YOMCLC,
!                              YOMCLS, YOMCLT, YOMCLP, YOMCLO
!      M. Deque     : 94-08-26 comdeck YOMCLR
!      P. Le Moigne : 95-01-18 New cost function controled by LNEWORO2
!      P. Le Moigne : 95-01-18 comdeck YOMCLA: key LKEYF to activate
!                              spectral fit of orog. in LAM case.
!      M. Deque     : 95-01-02 comdeck YOMCLW and new INCLIS/INRESS
!      P. Le Moigne : 95-10-11 comdeck YOMCLA: key LNLSM to import
!                              new land sea mask.
!      R. EL Khatib : 96-04-01 Set-up defaults controled by LECMWF+LMPHYS
!      D. Giard     : 97-03-13 New 923 (LN923=.T., calls to (E)INCLIn, n=1-5)
!                              calls to INCLICM , (E)INTER0 , VAL923
!      D. Giard     : 97-09-12 call to (E)INCLI6
!      D. Giard     : 97-10-16 modifications for ALADIN, from Luc Gerard
!      D. Giard     : 97-10-24 call to INCLIE suppressed
!                              (INCLIE suppressed  by ECMWF)
!      D. Giard     : 98-11-03 check boundaries for Part 5 + new defaults
!      D. Giard     : 98-12-08 NAMCLI read by (E)INCLIB instead of NAMCLB
!      D. Giard     : 99-07-02 VAL923 called in SUPHY/SUPHMF now
!      P.Marguinaud,Y.Moudden : 99-06-22 modified envelope 
!      D. Giard     : 99-07-21 controls for modified envelope
!      S. Kertesz   : 99-12-16 (E)INCLI7
!      R. El Khatib : 00-06-29 Remove old conf. 923
!      D. Giard     : 00-09-12 Restore INCLIR, further cleanings
!                              Suppress spectral ozone coefficients
!      D. Giard     : 00-09-15 Message passing
!      D. Giard     : 01-11-15 LVGSN should be set to .TRUE. temporarily 
!      D. Giard     : 01-11-16 add DEALCLIA to deallocate PTRSPOR if required
!      D. Giard     : 01-11-26 new default values for orography
!      D. Giard     : 01-11-29 changes in YOMCLA/NAMCLA: -NPNPP, NFLIS->NLISSZ
!      R. El Khatib : 01-08-07 Pruning options
!      Y. Bouteloup : 02-03-29 INCLI8 => add A,B and C for climatological ozone profile
!      D. Giard     : 03-06-16 aqua-planet : (E)INCLI10
!      D. Giard     : 04-09-15 update (parts 2-10)
!      D. Giard     : 02-11-25 new formulation for extension zone
!      D. Giard     : 02-12-16 LNEWORO3 -> LNEWENV
!      D. Giard     : 02-12-16 spectral smoothing for orography
!      D. Giard     : 03-01-13 spectral smoothing for orography (tunings)
!      D. Giard     : 05-04-04 semi-envelope and old ARPEGE cost functions
!                              removed, polynomial cost function added, simple 
!                              or no spectral fit for ARPEGE added 
!      M. Hortal    : 05-09-07 spectral smoothing of orography by a diffusion operator 5del16
!      F. Taillefer : 09-06-02 add LZ0THER
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NDGUNG   ,NDGUXG   ,&
 & NDLUNG   ,NDLUXG   ,NMSMAX
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMCT0   , ONLY : LELAM    ,NPROC
USE YOMPHY   , ONLY : LSOLV    ,LVGSN
USE YOMMCC   , ONLY : N923     ,LMCC12   ,LMCC19   ,LN923
USE YOMGEM   , ONLY : NSTAGP   ,NTSTAGP  ,RSTRET
USE YOMGC    , ONLY : GELAM    ,GELAT
USE YOMCST   , ONLY : RPI
USE YOMCLA   , ONLY : NLISSZ   ,NLISSR   ,FACZ0    ,FENVN    ,&
 & FENVS    ,FACE     ,QMAX     ,QMIN     ,HDIM     ,&
 & HMIN     ,QPOWER   ,QCONST   ,XINCOC   ,SCEXT    ,&
 & LNORO    ,LNLSM    ,LKEYF    ,LNEWORO  ,LNEWORO2 ,&
 & NLISSP   ,FLISA    ,FLISB    ,LSPSMORO
USE YOMCLI   , ONLY : LIEEE    ,LGLOBE   ,NPINT    ,NDATX    ,&
 & NDATY    ,NGLOBX   ,NGLOBY   ,ELONSW   ,ELATSW   ,&
 & ELONNE   ,ELATNE   ,EDLON    ,EDLAT    ,SVEG     ,&
 & LZ0THER  ,SFCZ0    ,RSTR     ,RSWR     ,NSLICE  

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IADL(NDGLG)
INTEGER(KIND=JPIM) :: IDINT, IPINT, IX, J

REAL(KIND=JPRB) :: ZCV, ZEPS, ZLATN, ZLATS, ZLONE, ZLONW, ZX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "sualclia.intfb.h"
#include "dealclia.intfb.h"
#include "eincli1.intfb.h"
#include "eincli2.intfb.h"
#include "eincli3.intfb.h"
#include "eincli4.intfb.h"
#include "eincli5.intfb.h"
#include "eincli6.intfb.h"
#include "eincli7.intfb.h"
#include "eincli8.intfb.h"
#include "eincli9.intfb.h"
#include "eincli10.intfb.h"
#include "einclib.intfb.h"
#include "einclir.intfb.h"
#include "einter0.intfb.h"
#include "incli1.intfb.h"
#include "incli2.intfb.h"
#include "incli3.intfb.h"
#include "incli4.intfb.h"
#include "incli5.intfb.h"
#include "incli6.intfb.h"
#include "incli7.intfb.h"
#include "incli8.intfb.h"
#include "incli9.intfb.h"
#include "incli10.intfb.h"
#include "inclib.intfb.h"
#include "inclir.intfb.h"
#include "inter0.intfb.h"
#include "posnam.intfb.h"

#include "namcla.h"
#include "namcli.h"

!     ------------------------------------------------------------------
!*
!     1. SET INITIAL VALUES.
!        -------------------

!        1.0 Environment

IF (LHOOK) CALL DR_HOOK('INCLI0',0,ZHOOK_HANDLE)

IF ( NPROC > 1 ) CALL ABOR1(' ONLY ONE PROC. FOR 923 !')
IF (LELAM) THEN
  IADL(1:NDGLG)=NTSTAGP(1:NDGLG)-1
ELSE
  IADL(1:NDGLG)=NSTAGP(1:NDGLG)-1
ENDIF  

ZEPS=1.E-12_JPRB

!        1.1 Set implicit default values

!*** Set values for *COMMON* *YOMCLA*
!  Parameters for orography
!  FENVN  : Factor for the envelope orography on pole of interest.
!           CV=0. OM=SQRT(2.)
!  FENVS  : Factor for the envelope orography on the antipode of the pole of
!           interest. CV=0. OM=0.
!  LNORO  : Key for reading a new orography on a separate file
!  LNLSM  : Key for reading a new land-sea mask on a separate file
!  LKEYF  : Key for spectral fit on the orography, only for ALADIN
!  NLISSR : Number of calls to ELISLAP to smooth the orography (ALADIN only)
FENVN = 0._JPRB
FENVS = 0._JPRB
LNORO = .FALSE.
LNLSM = .FALSE.
LKEYF = .TRUE.
NLISSR= 0
!  Parameters for fitted orography
!  LNEWORO : Switch to the new cost function (Bouteloup)
!  LNEWORO2: Switch to the new cost function (Jerczynski)
!  LSPSMORO: Switch for spectral smoothing of orography by a diffusion operator (linear grid)
!  QMAX    : Maximum value of weight
!  QMIN    : Min--------------------
!  HMIN    : Reference height in the formulation of weight
!  HDIM    : Scaling factor for the orography deviation
!  XINCOC  : Part of the fraction of sea in the weight
!  SCEXT   : Scaling factor for the weight of the extension zone
!  Defaults are ARPEGE ones for the constants
LNEWORO = .FALSE.
LNEWORO2= .FALSE.
LSPSMORO= .FALSE.
QMAX    = 4._JPRB
QMIN    = 2._JPRB
HMIN    = 150._JPRB
HDIM    = 1._JPRB
XINCOC  = 0.0_JPRB
SCEXT   = 0.0_JPRB
!  Additional parameters for the Jerczynski cost function
!  QPOWER  : Exponent in the additional term
!  QCONST  : Scaling factor in the additional term
!  FACE    : 1. for initial formulation , 0. for polynomial one
!  Defaults are ALADIN/LACE ones.
QPOWER = 3.5_JPRB
QCONST = 0.4_JPRB
FACE   = 1.0_JPRB
!  Parameters for the spectral smoothing of orography
!  NLISSP : type of smoothing
!   0 : no, 1 : importing a lower resolution one , 2 : in optimization
!  FLISA  : tuning parameter if NLISSP=2
!  FLISB  : threshold in spectral space if NLISSP=2
NLISSP = 0
FLISB  = 0.0_JPRB
FLISA  = 0.0_JPRB
!  Parameters for roughness length
!  NLISSZ : Number of calls to LISLAP to smooth the roughness length
!  FACZ0  : Scaling factor for the orographic part of Z0 ;CV=1. OM=0.42
NLISSZ = 3
FACZ0  = 1.0_JPRB

!*** Set values for *COMMON* *YOMCLI* 
!  NPINT  : size of the interpolation box ; >= minimum computed by (E)INTER0
!  LIEEE  : if ieee format is used
!  LGLOBE : if global dataset
!  NDATX  : x-size of the dataset (longitude) 
!  NDATY  : y-size of the dataset (latitude) 
!  ELONSW , ELATSW , ELONNE , ELATNE : for a local dataset (INCLI5)
!           latitudes and longitudes of the SW and NE corners (in degrees) 
!  EDLON , EDLAT  : resolution of the dataset (in degrees)       - local
!  NGLOBX, NGLOBY : corresponding numbers of points on the globe - local
!  SVEG   : threshold for significant vegetation cover 
!  LZ0THER: .FALSE. if no orographic part in the thermic roughness length
!  SFCZ0  : scaling factor for the secondary part of z0 (urban., veget.)
!  RSTR   : threshold for temperature-RTT in presence of snow ((E)INCLI6)
!           fixed value of SST-RTT for aqua-planet ((E)INCLI10)
!  RSWR   : threshold for mean moisture   in presence of snow ((E)INCLI6)
!  NSLICE : number of packet used to slice the domain (aladin only)
!  Variables in YOMCLI but not in NAMCLI are set in VAL923
LIEEE  = .TRUE.
LGLOBE = .TRUE.
NPINT = 1
NDATX = 1
NDATY = 1
ELONSW =   0._JPRB
ELATSW = -90._JPRB
ELONNE = 360._JPRB
ELATNE =  90._JPRB
EDLON = 360._JPRB
EDLAT = 180._JPRB
NGLOBX = 1
NGLOBY = 1
SVEG = 0.02_JPRB
LZ0THER = .TRUE.
SFCZ0 = 1._JPRB
RSTR = 5.00_JPRB
RSWR = 0.02_JPRB
NSLICE = 1
IF (.NOT.LSOLV) SVEG = 0._JPRB

!        1.2 Modify default values for YOMCLI according to YOMMCC

IF (LMCC12) THEN
  LIEEE =.TRUE.
  NDATX =  2
  NDATY = 64
ELSEIF (LMCC19) THEN
  LIEEE =.FALSE.
  NDATX = 64
  NDATY = 32
ELSEIF (N923  ==  1) THEN
  LIEEE =.TRUE.
  NDATX = 8640
  NDATY = 4320
ELSEIF (N923  ==  2) THEN
  IF (LSOLV) THEN
    LIEEE =.TRUE.
    NDATX = 360
    NDATY = 180
  ELSE
    LIEEE =.FALSE.
    NDATX = 432
    NDATY = 216
  ENDIF
ELSEIF (N923  ==  3) THEN
  LIEEE =.TRUE.
  NDATX = 432
  NDATY = 216
ELSEIF (N923  ==  4) THEN
  LIEEE =.TRUE.
  NDATX = 360
  NDATY = 180
ELSEIF (N923  ==  6) THEN
  LIEEE =.TRUE.
  NDATX = 240
  NDATY = 120
ELSEIF (N923  ==  8) THEN    
  LIEEE =.FALSE.
  NDATX = 144
  NDATY = 73                 
ELSEIF (N923  ==  9) THEN    
  LIEEE =.FALSE.
  NDATX = 72
  NDATY = 45                 
ENDIF
NGLOBX = NDATX
NGLOBY = NDATY
EDLON = 360._JPRB/REAL(NDATX,JPRB)
EDLAT = 180._JPRB/REAL(NDATY,JPRB)
IF (LELAM) THEN
  CALL EINTER0(NGLOBX,NGLOBY,NPINT)
ELSE
  CALL INTER0(NDGLG,NDLON,NGLOBY,NGLOBX,RSTRET,NPINT)
ENDIF
IF (N923  ==  5) THEN
  LGLOBE=.FALSE.
  LIEEE =.TRUE.
  NPINT = 1
  NDATX = 860
  NDATY = 420
  ELONSW = -25.0_JPRB
  ELATSW =  30.0_JPRB
  EDLON = 0.1_JPRB
  EDLAT = 0.1_JPRB
  ELONNE = ELONSW + EDLON*NDATX
  ELATNE = ELATSW + EDLAT*NDATY
  NGLOBX = NINT(360._JPRB/EDLON)
  NGLOBY = NINT(180._JPRB/EDLAT)
ENDIF

!        1.3 Read namelist NAMCLI and modify YOMCLI accordingly

CALL POSNAM(NULNAM,'NAMCLI')
READ(NULNAM,NAMCLI)

!*** Miscellaneous
RSTR=MAX(0.0_JPRB,RSTR)
RSWR=MAX(0.0_JPRB,MIN(1.0_JPRB,RSWR))

!*** Resolution
NDATX= MAX(NDATX,1)
NDATY= MAX(NDATY,1)
!  Global dataset (described by NDAT-X/Y)
IF (LGLOBE) THEN
  NGLOBX = NDATX
  NGLOBY = NDATY
  EDLON = 360._JPRB/REAL(NDATX,JPRB)
  EDLAT = 180._JPRB/REAL(NDATY,JPRB)
  ELONSW =   0._JPRB
  ELATSW = -90._JPRB
  ELONNE = 360._JPRB
  ELATNE =  90._JPRB
!  Local dataset (described by NDAT-X/Y, ELON-SW/NE, ELAT-SW/NE)
ELSE
  EDLON = (ELONNE-ELONSW)/REAL(NDATX,JPRB)
  EDLAT = (ELATNE-ELATSW)/REAL(NDATY,JPRB)
  IF (EDLON <= ZEPS .OR. EDLAT <= ZEPS) &
   & CALL ABOR1(' PROBLEM WITH LOCAL DATASET CORNERS')  
  IF (EDLON >= 360._JPRB .AND. EDLAT >= 180._JPRB) &
   & CALL ABOR1(' GLOBAL DATASET DECLARED AS LOCAL ?')  
  NGLOBX = NINT(360._JPRB/EDLON)
  NGLOBY = NINT(180._JPRB/EDLAT)
ENDIF

!*** Interpolation box or operator
IF (N923 == 5 .OR. N923 == 7) THEN
! Control in parts 5 or 7
  NPINT = MIN(2,MAX(1,NPINT))
ELSE
! Size for parts 1, 2, 3, 4, 6, 8, 9, 10 and old options B and R
!  Minimum size, as a function of model geometry and data resolution
  IPINT = 1
  IF (LELAM) THEN
    CALL EINTER0(NGLOBX,NGLOBY,IPINT)
  ELSE
    CALL INTER0(NDGLG,NDLON,NGLOBY,NGLOBX,RSTRET,IPINT)
  ENDIF
!  Control (NPINT must be odd and at least IPINT)
  NPINT = (NPINT/2)*2+1
  NPINT = MAX(IPINT,NPINT)
!  Temporary warning for interpolators
  IF (MOD(NDATY,2) == 1) WRITE(UNIT=NULOUT,&
   & FMT='(''CAUTION : INTERPOLATORS MAY NOT SUIT ODD NDATY'')')
ENDIF

!*** Control of boundaries
!  ALADIN
IF (LELAM .AND. .NOT.LGLOBE) THEN
  IDINT= NPINT/2
  ZCV = 180._JPRB/RPI
!  Latitude boundary
  ZLATN = 0.0_JPRB
  ZLATS = RPI
  DO J=NDLUNG,NDLUXG
    ZLATN = MAX(ZLATN, GELAT(J+IADL(NDGUXG)) )
    ZLATS = MIN(ZLATS, GELAT(J+IADL(NDGUNG)) )
  ENDDO
  ZLATN = ZLATN*ZCV + (IDINT+1)*EDLAT
  ZLATS = ZLATS*ZCV - (IDINT+1)*EDLAT
  IF (N923 == 5 .OR. N923 == 7) THEN
    IF (ELATSW  >=  ZLATN) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELATSW ='',F8.2,'' >'',F8.2,'' (dg)'')') ELATSW,ZLATN  
      IF (LHOOK) CALL DR_HOOK('INCLI0',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
    IF (ELATNE  <=  ZLATS) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELATNE ='',F8.2,'' <'',F8.2,'' (dg)'')') ELATNE,ZLATS  
      IF (LHOOK) CALL DR_HOOK('INCLI0',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
  ELSE
    IF (ELATSW  >  ZLATS) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELATSW ='',F8.2,'' >'',F8.2,'' (dg)'')') ELATSW,ZLATS  
      CALL ABOR1('INCLI0')
    ENDIF
    IF (ELATNE  <  ZLATN) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELATNE ='',F8.2,'' <'',F8.2,'' (dg)'')') ELATNE,ZLATN  
      CALL ABOR1('INCLI0')
    ENDIF
  ENDIF
!  Longitude boundary
  ZX = 1.0_JPRB/MAX(ZEPS, SIN(RPI-MAX(ABS(ZLATN),ABS(ZLATS))/ZCV))
  IX = MIN(NGLOBX/4,INT(ZX))
  ZLONW = GELAM(     1+IADL(NDGUXG))*ZCV
  ZLONE = GELAM(NDLUXG+IADL(NDGUXG))*ZCV
  ZLONW = ZLONW-(1.0_JPRB-SIGN(1.0_JPRB,180._JPRB-ZLONW))*180._JPRB
  ZLONE = ZLONE-(1.0_JPRB-SIGN(1.0_JPRB,180._JPRB-ZLONE))*180._JPRB
  ZLONW = ZLONW - (IX+IDINT)*EDLON
  ZLONE = ZLONE + (IX+IDINT)*EDLON
  IF (N923 == 5 .OR. N923 == 7) THEN
    IF (ELONSW  >=  ZLONE) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELONSW ='',F8.2,'' >'',F8.2,'' (dg)'')') ELONSW,ZLONE  
      IF (LHOOK) CALL DR_HOOK('INCLI0',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
    IF (ELONNE  <=  ZLONW) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELONNE ='',F8.2,'' <'',F8.2,'' (dg)'')') ELONNE,ZLONW  
      IF (LHOOK) CALL DR_HOOK('INCLI0',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
  ELSE
    IF (ELONSW  >  ZLONW) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELONSW ='',F8.2,'' >'',F8.2,'' (dg)'')') ELONSW,ZLONW  
      CALL ABOR1('INCLI0')
    ENDIF
    IF (ELONNE  <  ZLONE) THEN
      WRITE(UNIT=NULOUT,FMT='(''BAD POSITION OF DATASET :'',&
       & '' ELONNE ='',F8.2,'' <'',F8.2,'' (dg)'')') ELONNE,ZLONE  
      CALL ABOR1('INCLI0')
    ENDIF
  ENDIF
ENDIF
!  ARPEGE
IF (.NOT.LELAM .AND. .NOT.LGLOBE .AND. N923 /= 5 .AND. N923 /= 7) THEN
  CALL ABOR1(' GLOBAL DATASETS REQUIRED')
ENDIF

!*** Temporary control for the albedos of vegetation and bare-ground
IF (LELAM .AND. LN923 .AND. (N923 == 2 .OR. N923 == 4 .OR. N923 ==5) &
 & .AND. .NOT.LVGSN) CALL ABOR1(' SET LVGSN=.T. IN NAMPHY FOR SAFETY')  

!        1.4 Print dataset description

WRITE(UNIT=NULOUT,FMT='('' COMMON YOMCLI''/&
 & '' LIEEE='',L2,'' LGLOBE='',L2,'' NDATX='',I4,'' NDATY='',I4,&
 & '' NPINT='',I3,'' EDLON (dg)='',F5.2,'' EDLAT (dg)='',F5.2,/,&
 & '' SVEG = '',F5.2,'' LZ0THER = '',L2,'' SFCZ0 = '',F5.2,&
 & '' RSTR = '',F3.0,''K  RSWR = '',F3.2,'' NSLICE = '',I4)') &
 & LIEEE,LGLOBE,NDATX,NDATY,NPINT,EDLON,EDLAT,&
 & SVEG,LZ0THER,SFCZ0,RSTR,RSWR,NSLICE  
IF (.NOT.LGLOBE) THEN
  WRITE(UNIT=NULOUT,FMT='(&
   & '' ELATSW (dg)='',F8.2,'' ELONSW (dg)='',F8.2,&
   & '' ELATNE (dg)='',F8.2,'' ELONNE (dg)='',F8.2,&
   & '' NGLOBX='',I5,'' NGLOBY='',I5)')&
   & ELATSW,ELONSW,ELATNE,ELONNE,NGLOBX,NGLOBY  
ENDIF

!        1.5  Read and check topography description (if required)

IF ( LN923 .AND. N923 == 1 ) THEN

  CALL POSNAM(NULNAM,'NAMCLA')
  READ(NULNAM,NAMCLA)

  SCEXT  = MAX(0.0_JPRB,SCEXT)
  NLISSP = MAX(0,MIN(2,NLISSP))

  IF (NLISSP == 1 .AND. .NOT. LNORO) THEN
    CALL ABOR1(' IMPORTING SPECTRAL OROGRAPHY REQUIRES LNORO=.TRUE. !')
  ENDIF
  IF ((LNEWORO.OR.LNEWORO2.OR.LNORO) .AND. .NOT.LKEYF) THEN
    CALL ABOR1(' IMPORTING OR OPTIMIZING ONLY SPECTRAL OROGRAPHY !')
  ENDIF
  IF (NLISSP == 2 .AND. .NOT.(LNEWORO.OR.LNEWORO2)) THEN
    CALL ABOR1(' SPECTRAL COST FUNCTION ALONE FAILS !')
  ENDIF  

  IF (LNORO) THEN
    IF (.NOT.LELAM) THEN
      LNEWORO  = .FALSE.
      LNEWORO2 = .FALSE.
    ENDIF
  ELSE
    LNLSM = .FALSE.
  ENDIF

  WRITE(UNIT=NULOUT,FMT='('' COMMON YOMCLA''/&
   &'' LKEYF '',L2,'' LNEWORO '',L2,'' LNEWORO2 '',L2,'' LSPSMORO '',L2,&
   &'' XINCOC '',F8.3,'' SCEXT '',F8.3,/,&
   &'' QMAX '',F6.1,'' QMIN '',F6.1,'' HDIM '',F6.2,'' HMIN '',F6.0,&
   &'' QPOWER '',F6.3,'' QCONST '',F6.3,'' FACE '',F6.3,/,&
   &'' FENVN '',F6.3,'' FENVS '',F6.3,/,&
   &'' LNORO '',L2,'' LNLSM '',L2,/,&
   &'' NLISSZ '',I2,'' FACZ0'',F6.3,/,'' NLISSR '',I2,&
   &'' NLISSP '',I2,'' FLISA '',F6.3,'' FLISB '',F6.1,//)') &
   &LKEYF,LNEWORO,LNEWORO2,LSPSMORO,XINCOC,SCEXT,QMAX,QMIN,HDIM,HMIN,QPOWER,QCONST,&
   &FACE,FENVN,FENVS,LNORO,LNLSM,NLISSZ,FACZ0,NLISSR,NLISSP,FLISA,FLISB

  IF ((MOD(NDLON,NMSMAX) > 2) .AND. LSPSMORO) THEN
    CALL ABOR1(' THE USE OF LSPSMORO REQUIRES A LINEAR GRID !')
  ENDIF
  IF (LSPSMORO .AND. (LNEWORO.OR.LNEWORO2)) THEN
    CALL ABOR1('ONLY ONE TYPE OF SPECTRAL SMOOTHING AT THE TIME !')
  ENDIF
  IF (LNEWORO .AND. LNEWORO2) THEN
    CALL ABOR1('ONLY ONE TYPE OF SPECTRAL SMOOTHING AT THE TIME !')
  ENDIF

ENDIF

!     ------------------------------------------------------------------
!*
!     2  ACTIVE CALCULATION.
!        -------------------

!*       2.1   MEMORY ALLOCATION

IF ( LN923 .AND. N923 == 1 ) THEN
  CALL SUALCLIA
ENDIF

!*       2.2   NEW C923

IF (LN923) THEN
!  TOPOGRAPHY
  IF (N923 == 1) THEN
    IF (LELAM) THEN
      CALL EINCLI1
    ELSE
      CALL INCLI1
    ENDIF
  ENDIF
!  FIXED FIELDS
  IF (N923 == 2) THEN
    IF (LELAM) THEN
      CALL EINCLI2
    ELSE
      CALL INCLI2
    ENDIF
  ENDIF
!  OLD CLIMATOLOGY FOR SOIL TEMPERATURE AND MOISTURE
  IF (N923 == 3) THEN
    IF (LELAM) THEN
      CALL EINCLI3
    ELSE
      CALL INCLI3
    ENDIF
  ENDIF
!  OTHER MONTHLY VARYING FIELDS - VEGETATION
  IF (N923 == 4) THEN
    IF (LELAM) THEN
      CALL EINCLI4
    ELSE
      CALL INCLI4
    ENDIF
  ENDIF
!  LOCAL MODIFICATION OF FIELDS ON LAND
  IF (N923 == 5) THEN
    IF (LELAM) THEN
      CALL EINCLI5
    ELSE
      CALL INCLI5
    ENDIF
  ENDIF
!  NEW CLIMATOLOGY FOR SOIL TEMPERATURE AND MOISTURE
  IF (N923 == 6) THEN
    IF (LELAM) THEN
      CALL EINCLI6
    ELSE
      CALL INCLI6
    ENDIF
  ENDIF
!  LOCAL MODIFICATION OF FIELDS ON SEA/LAKES
  IF (N923 == 7) THEN
    IF (LELAM) THEN
      CALL EINCLI7
    ELSE
      CALL INCLI7
    ENDIF
  ENDIF
!  A, B, C COEFFICIENTS FOR OZONE PROFILES
  IF (N923 == 8) THEN
    IF (LELAM) THEN
      CALL EINCLI8
    ELSE
      CALL INCLI8
    ENDIF
  ENDIF
!  AEROSOLS (NEW)
  IF (N923 == 9) THEN
    IF (LELAM) THEN
      CALL EINCLI9
    ELSE
      CALL INCLI9
    ENDIF
  ENDIF
!  AQUA-PLANET
  IF (N923 == 10) THEN
    IF (LELAM) THEN
      CALL EINCLI10
    ELSE
      CALL INCLI10
    ENDIF
  ENDIF
!  SPECTRAL EMISSIVITY
  IF (N923 == 11) THEN
    CALL ABOR1('INCLI0 : PART 11 NOT YET READY')
!    IF (LELAM) THEN
!      CALL EINCLI11
!    ELSE
!      CALL INCLI11
!    ENDIF
  ENDIF

!*       2.3   OLD C923

ELSE
!  OZONE PARAMETERIZATION
  IF (LMCC12) THEN
    IF (LELAM) THEN
      CALL EINCLIB
    ELSE
      CALL INCLIB
    ENDIF
!  AEROSOLS (OLD)
  ELSEIF (LMCC19) THEN
    IF (LELAM) THEN
      CALL EINCLIR
    ELSE
      CALL INCLIR
    ENDIF
  ELSE
    CALL ABOR1('SORRY : NO MORE OLD CONF. 923 !')
  ENDIF

ENDIF

!*       2.4   MEMORY DEALLOCATION

IF ( LN923 .AND. N923 == 1 ) THEN
  CALL DEALCLIA
ENDIF

IF (LHOOK) CALL DR_HOOK('INCLI0',1,ZHOOK_HANDLE)
END SUBROUTINE INCLI0
