SUBROUTINE SIMREL(KINDIC,KN,PX,PF,PG,KITER)

!**** *SIMREL*

!     PURPOSE.
!     --------

!       External of the minimisation routine *M1QN3R* : calculates the function
!     and its gradient.

!**   INTERFACE.
!     ----------

!     *CALL* *SIMREL(KINDIC,KN,PX,PF,PG,KITER)*

!         *KINDIC* - Required by the minimization package
!         *KN*     - Size of the argument (input)
!         *PX*     - Argument (input)
!         *PF*     - Value of the function
!         *PG*     - Value of the gradient
!         *KITER*  - Required by the minimization package

!     METHOD.
!     -------

!       Transforms into grid point , calculates over the grid the quadratic
!     function and its derivatives, then returns to spectral coefficients.

!     EXTERNALS.
!     ----------

!           SPREORD
!           REESPE
!           SPEREE
!           ESPERAD
!           ESPEREE

!     AUTHOR.
!     -------

!         Y. BOUTELOUP and M. DEQUE   1 FEB 91.
!         Modified (93-08-26) by Y.Bouteloup : New cost function
!                                              Controled by LNEWORO
!         Modified by R. EL Khatib : 94-03-08 comdecks YOMCLA and PTRSPOR
!         Modified by P. Le Moigne : 95-01-18 new cost function
!                                             controled by LNEWORO2
!         Modified by L.Isaksen    : 95-06-06 Reordering of spectral arrays
!         Modified : 95-12-22 M.Hamrud    Change of XREV,XPDS layout
!         Modified by P. Le Moigne : 95-11-27 No reordering of spectral arrays
!                                    in LAM case.
!         Modified by F. Bouttier  : 96-05-30 add KITER argument
!         Modified by D. Giard     : 97-12-15 , call to EREESPESM
!         Modified by P. Le Moigne : 98-01-26 , call to ESPERAD instead
!                                    of EREESPESM
!         Modified by D. Giard     : 98-01-30 , reordering of spectral arrays
!         98-03-17: R. EL KHATIB : SPREORD
!         Modified by P. Le Moigne : 98-09-29 , minimization parallelized.
!         Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of one dummy argument in 'CALL SPEREESM'.
!         Modified : 99-06-22 P.Marguinaud,Y.Moudden, modified envelope 
!         Modified : 99-09-21 V. Ivanovici, modified envelope for ALADIN
!         Modified : 99-11-17 D. Giard, new spectral transforms
!         Modified : 00-09-15 D. Giard, message passing
!         Modified : 01-03-08 S. Alexandru, management of the extension zone
!         Modified : 01-11-23 D. Giard, clean call to ESPERAD
!         C. Fischer:03-04-07 implement control_vector type
!         Modified : 02-12-05 D. Giard, cleaner call to ESPERAD
!         Modified : 02-12-16 D. Giard, LNEWORO3-> LNEWENV
!         Modified : 03-01-13 D. Giard, spectral smoothing and cleaning
!                             (from K. Stadlbacher)
!         Modified : 05-03-25 F. Taillefer, back to an old minimization code
!         Modified : 05-04-04 D. Giard, update, semi-envelope and old ARPEGE 
!                             cost functions, adding a polynomial one
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NSEFRE   ,NSPEC2   ,NSMAX    ,NMSMAX
USE PTRSPOR  , ONLY : XREV     ,XPDS     ,XPDL     ,XEXT
USE YOMCLA   , ONLY : HDIM     ,QPOWER   ,QCONST   ,FACE     ,&
 & LNEWORO  , LNEWORO2 ,NLISSP
USE YOMLEG   , ONLY : RW
USE YOMGEM   , ONLY : NLOENG   ,NSTAGP   ,NTSTAGP  ,NGPTOT
USE YOMCT0   , ONLY : LELAM
USE YOMLAP   , ONLY : NASN0
USE YEMLAP   , ONLY : NESM0    ,NCPL4M  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(INOUT) :: KINDIC
INTEGER(KIND=JPIM),INTENT(IN)    :: KN
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PX(KN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PF 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PG(KN) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KITER

REAL(KIND=JPRB) :: ZS(NGPTOT)
REAL(KIND=JPRB) :: ZX1(NSPEC2),ZG1(NSPEC2)
REAL(KIND=JPRB) :: ZPX(NSEFRE),ZPG(NSEFRE)
REAL(KIND=JPRB),ALLOCATABLE :: ZST(:,:,:),ZGT(:,:)
REAL(KIND=JPRB) :: ZCST, ZDF, ZDS, ZW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTEGER(KIND=JPIM) :: IADL(NDGLG)
INTEGER(KIND=JPIM) :: IADR, JI, JK, JM, JN

#include "abor1.intfb.h"
#include "esperad.intfb.h"
#include "esperee.intfb.h"
#include "reespe.intfb.h"
#include "speree.intfb.h"
#include "spreord.intfb.h"

IF (LHOOK) CALL DR_HOOK('SIMREL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1. SPECTRAL - GRID POINT TRANSFORM OF THE ARGUMENT
!        -----------------------------------------------

IF (KN /= NSEFRE) CALL ABOR1('SIMREL : DIMENSION MISMATCH !')

IF (LELAM) THEN
  IADL(1:NDGLG)=NTSTAGP(1:NDGLG)-1
ELSE
  IADL(1:NDGLG)=NSTAGP(1:NDGLG)-1
ENDIF  

ZPX(:)=PX(:)
CALL SPREORD(1,ZPX,ZX1,.TRUE.)
IF (LELAM) THEN
  CALL ESPEREE(1,1,ZX1,ZS)
ELSE
  CALL SPEREE(1,1,ZX1,ZS)
ENDIF

!     ------------------------------------------------------------------

!*    2. CALCULATION OF THE FUNCTION AND ITS GRADIENT.
!        ---------------------------------------------

PF=0.0_JPRB

IF (LNEWORO) THEN
  DO JK=1,NDGLG
    ZW=RW(JK)/REAL(NLOENG(JK),JPRB)
    DO JI=1,NLOENG(JK)
      IADR=JI+IADL(JK)
      ZDF=(ABS(ZS(IADR)-XREV(IADR))/HDIM)**XPDS(IADR)
      ZDS=XPDS(IADR)*(ZS(IADR)-XREV(IADR))*&
       & (ABS(ZS(IADR)-XREV(IADR))/HDIM)**(XPDS(IADR)-2.0_JPRB)/HDIM**2  
      PF=PF+XEXT(JI,JK)*ZW*ZDF
      ZS(IADR)=XEXT(JI,JK)*ZDS
    ENDDO
  ENDDO

ELSEIF (LNEWORO2) THEN
  ZCST=QCONST**QPOWER
  DO JK=1,NDGLG
    ZW=RW(JK)/REAL(NLOENG(JK),JPRB)
    DO JI=1,NLOENG(JK)
      IADR=JI+IADL(JK)
      ZDF=FACE*XPDS(IADR)*(ZS(IADR)-XREV(IADR))**2 &
       & +ZCST*ABS(ZS(IADR)-XREV(IADR))**QPOWER  
      ZDS=2.0_JPRB*FACE*XPDS(IADR)*(ZS(IADR)-XREV(IADR))&
       & +ZCST*QPOWER*(ZS(IADR)-XREV(IADR))&
       & *ABS(ZS(IADR)-XREV(IADR))**(QPOWER-2.0_JPRB)  
      PF=PF+XEXT(JI,JK)*ZW*ZDF
      ZS(IADR)=XEXT(JI,JK)*ZDS
    ENDDO
  ENDDO
ENDIF

IF (LELAM) THEN
  DO JK=1,NDGLG
    ZW=RW(JK)/REAL(NLOENG(JK),JPRB)
    DO JI=1,NLOENG(JK)
      IADR=JI+IADL(JK)
      ZS(IADR)=ZS(IADR)*ZW
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*    3. GRID POINT - SPECTRAL TRANSFORM OF THE GRADIENT
!        -----------------------------------------------

IF (LELAM) THEN
  ALLOCATE ( ZST (NGPTOT,1,1) )
  ALLOCATE ( ZGT (1,NSPEC2) )
  ZST(:,:,:)=0.0_JPRB
  ZGT(:,:)  =0.0_JPRB
  ZST(:,1,1)=ZS(:)
  CALL ESPERAD(1,1,ZGT,ZST)
  ZG1(:)=ZGT(1,:)
  DEALLOCATE ( ZST )
  DEALLOCATE ( ZGT )
ELSE
  CALL REESPE(1,1,ZG1,ZS)
ENDIF
CALL SPREORD(1,ZPG,ZG1,.FALSE.)

!     ------------------------------------------------------------------

!*    4. ADDITION OF A SPECTRAL TERM TO THE FUNCTION AND THE GRADIENT
!        ------------------------------------------------------------

IF (NLISSP == 2) THEN
 IF (LELAM) THEN
  DO JM=0,NMSMAX
    DO JN=0,NCPL4M(JM)-1
      IADR=NESM0(JM)+JN
      PF=PF+2*XPDL(IADR)*ZPX(IADR)**2
      ZPG(IADR)=ZPG(IADR)+4*XPDL(IADR)*ZPX(IADR)
    ENDDO
  ENDDO
 ELSE
  DO JN=0,NSMAX
    DO JM=-JN,JN
      IADR=NASN0(JN)+JM
      IF (JM == 0) THEN
        PF=PF+XPDL(IADR)*ZPX(IADR)**2
        ZPG(IADR)=ZPG(IADR)+2*XPDL(IADR)*ZPX(IADR)
      ELSE
        PF=PF+2*XPDL(IADR)*ZPX(IADR)**2
        ZPG(IADR)=ZPG(IADR)+4*XPDL(IADR)*ZPX(IADR)
      ENDIF
    ENDDO
  ENDDO
 ENDIF 
ENDIF

!     ------------------------------------------------------------------

PG(:)=ZPG(:)
PX(:)=ZPX(:)

IF (LHOOK) CALL DR_HOOK('SIMREL',1,ZHOOK_HANDLE)
END SUBROUTINE SIMREL
