SUBROUTINE CABINE (PHI,PHU,PRESF,KPOS)
!****------------------------------------------------------------------------
!****   CABINE : INTERPOLATION VERTICALE DES ECART-TYPES D'ERREUR DE
!****   ------    PREVISION CLIMATOLOGIQUES.
!****------------------------------------------------------------------------

!  BUT : INTERPOLER LES ECART-TYPES D'ERREUR DE PREVISION CLIMATOLOGIQUES
!  ---   POUR LA VERTICALE KPOS A PARTIR DES VERTICALES CLIMATOLOGIQUES
!        SUR NIVEAUX PRESSION POUR PHI ET HU (RESPECTIVEMENT GEOPOTENTIEL ET
!        HUMIDITE).
!        INITIALISATION EGALEMENT DES ECART-TYPES D'ERREUR DE PREVISION POUR
!        LE CHAMP DE SST (PAS D'INTERPOLATION VERTICALE !).
!        LA METHODE UTILISEE EST L'INTERPOLATION LINEAIRE EN PRESSION
!         (...BOF... PB  03/12/90).

!  ROUTINE APPELANTE :     *CABANE*
!  -----------------

!  ARGUMENTS D'ENTREE :
!  ------------------
!     Explicite : - PHI   : ecart-type du geopotentiel sur les JPNVS niveaux
!                           donnes par XPPRS (common QACOSS),
!                 - PHU   : ecart-type de l'humidite sur les JPNVS niveaux
!                           donnes par XPPRS (common QACOSS),
!                 - PRESF : pressions des NFLEVG niveaux a interpoler,
!                 - KPOS  : indice sur l'horizontale de la verticale.

!     Implicite :  YOMDIM QAPASS QACOSS

!  ARGUMENTS DE SORTIE :
!  -------------------
!     Implicite : - QASSET : tableau ESIG (ecart-types d'erreur guess).
!                 - QAGPSF : tableaux RSGPHI et RSGHUM.
!****----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE QAPASS   , ONLY : JPNVS
USE QASSET   , ONLY : ESIG
USE QALORI   , ONLY : QCAGUE
USE QAGPSF   , ONLY : RSGHUM   ,RSGPHI
USE QACOSS   , ONLY : XPPRS
USE QAREF    , ONLY : REF_S_SST, REF_S_SN, REF_S_T2, REF_S_H2, REF_S_V1

IMPLICIT NONE

REAL(KIND=JPRB), DIMENSION(JPNVS) ,INTENT(IN)    :: PHI 
REAL(KIND=JPRB), DIMENSION(JPNVS) ,INTENT(IN)    :: PHU 
REAL(KIND=JPRB), DIMENSION(NFLEVG),INTENT(IN)    :: PRESF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPOS 

INTEGER(KIND=JPIM) :: ILEV, ILEVHU, JN

REAL(KIND=JPRB) :: ZCOEF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!**------------------------------------------------------------------------
!**   1.    Interpolation verticale des ecart-types de PHI et HU.
!**         ----------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CABINE',0,ZHOOK_HANDLE)
ILEV=JPNVS
DO JN = NFLEVG , 1 , -1
  ILEVHU=NFLEVG+JN
  120 CONTINUE
  IF (PRESF(JN) > XPPRS(ILEV).OR.ILEV == 1) THEN
    IF (PRESF(JN) < XPPRS(JPNVS).AND.PRESF(JN) > XPPRS(1))  THEN
      ZCOEF=(PRESF(JN)-XPPRS(ILEV))/(XPPRS(ILEV+1)-XPPRS(ILEV))
      ESIG(JN,KPOS)=PHI(ILEV)+ZCOEF*(PHI(ILEV+1)-PHI(ILEV))
      ESIG(ILEVHU,KPOS)=PHU(ILEV)+ZCOEF*(PHU(ILEV+1)-PHU(ILEV))
    ELSE
!           Blindage sur niveaux JN hors (1,JPNVS) :
      ESIG(JN,KPOS)=PHI(ILEV)
      ESIG(ILEVHU,KPOS)=PHU(ILEV)
    ENDIF
    ESIG(JN,KPOS)=ESIG(JN,KPOS)*QCAGUE(1,KPOS)
    ESIG(ILEVHU,KPOS)=ESIG(ILEVHU,KPOS)*QCAGUE(1,KPOS)
    RSGPHI(JN,KPOS)=ESIG(JN,KPOS)
    RSGHUM(JN,KPOS)=ESIG(ILEVHU,KPOS)
  ELSE
!  Recherche du niveau de pression immediatement superieur a JN
    ILEV=ILEV-1
    GOTO 120
  ENDIF
ENDDO

!**------------------------------------------------------------------------
!**   2.    Alimentation des ecart-types de surface
!**         ---------------------------------------

ESIG(2*NFLEVG+1,KPOS)=REF_S_SST
ESIG(2*NFLEVG+2,KPOS)=REF_S_SN * QCAGUE(1,KPOS)
ESIG(2*NFLEVG+3,KPOS)=REF_S_T2 * QCAGUE(1,KPOS)
ESIG(2*NFLEVG+4,KPOS)=REF_S_H2 * QCAGUE(1,KPOS)
ESIG(2*NFLEVG+5,KPOS)=REF_S_V1 * QCAGUE(1,KPOS) / QCAGUE(2,KPOS)

!**------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CABINE',1,ZHOOK_HANDLE)
END SUBROUTINE CABINE
