SUBROUTINE CABOTU (PLAT,PLON,KBOITE)
!****----------------------------------------------------------------
!****   CABOTU : Recherche du numero de boite geographique
!****   ------
!****   Auteur                : Ph. Caille                  31/07/89
!****   Derniere modification : 16/07/90
!****----------------------------------------------------------------
!**
!** But : calcul du numero de la boite d'observation contenant l'ob-
!** ---   servation de latitude PLAT et de longitude PLON.
!**
!**  Pour une  boite geographique  donnee, les  bords nord  et ouest
!**  appartiennent  a la boite  alors que les  bords sud et  est n'y
!**  appartiennent pas. Dans la version  actuelle (Juillet 1990), le
!**  globe  est decoupe  en boites  geographiques  de cinq degres de
!**  cote (latitude  et longitude), excepte  aux poles ou les boites
!**  correspondent aux calottes polaires de cinq degres de latitude.
!**
!** Sequence d'appel : CALL CABOTU (PLAT,PLON,KBOITE)
!** ----------------
!**
!** Arguments d'entree :
!** ------------------
!**   PLAT   :  latitude du point    ( en radian )
!**   PLON   :  longitude du point   ( en radian )
!**
!** Arguments de sortie :
!** -------------------
!**   KBOITE :  numero de la boite d'observation
!**
!** Sous-programme amont :  CARACO
!** --------------------
!** Sous-programmes aval :  aucun
!** --------------------
!** COMMON utilises :       QAPABO - YOMANCS
!** ---------------
!** Modifications :
!** -------------
!****----------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : RDEGREES
USE QAPABO   , ONLY : JPBLAN   ,JPBLAS   ,NBODLA   ,NBODLO   ,&
 & NBONLO   ,NBONBT  

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBOITE 
INTEGER(KIND=JPIM) :: ILAT, ILON

REAL(KIND=JPRB) :: ZLABPN, ZLABPS, ZLAT, ZLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!**------------------------------------------------------------------
!**  - 1 -  CALCUL DU NUMERO DE BOITE
!**         -------------------------

IF (LHOOK) CALL DR_HOOK('CABOTU',0,ZHOOK_HANDLE)
ZLABPN = REAL( JPBLAN - NBODLA ,JPRB)
ZLABPS = REAL( JPBLAS + NBODLA ,JPRB)

ZLON   = PLON * RDEGREES
ZLAT   = PLAT * RDEGREES
IF ( ZLON < 0.0_JPRB ) ZLON = ZLON + 360._JPRB
IF ( ZLON >= 360._JPRB ) ZLON = ZLON - 360._JPRB

IF ( ZLAT > ZLABPN ) THEN
  KBOITE = 1
ELSEIF ( ZLAT <= ZLABPS ) THEN
  KBOITE = NBONBT
ELSE
  ILAT   = INT( (ZLABPN-ZLAT) / REAL(NBODLA,JPRB) )
  ILON   = INT( ZLON/REAL(NBODLO,JPRB) ) + 1
  KBOITE = NBONLO*ILAT + ILON + 1
ENDIF

IF ( KBOITE < 1.OR. KBOITE > NBONBT ) KBOITE = 0

!**------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CABOTU',1,ZHOOK_HANDLE)
END SUBROUTINE CABOTU
