SUBROUTINE CACLSST
!****--------------------------------------------------------------------------
!****   CACLSST : CONSTITUTION D'UN CHAMP DE RAPPEL CLIMATOLOGIQUE 
!****   -------    POUR LA TEMPERATURE DE SURFACE DE LA MER
!****
!****   Auteur :       F. TAILLEFER  07/99
!****--------------------------------------------------------------------------
!  BUT : LECTURE DANS LE FICHIER "ICMSHxxxxSST" D'UN CHAMP REFERENCE DE SST
!  ---   (L'ANALYSE NESDIS, RECUPEREE DANS LA BDAP ET INTERPOLEE SUR LA GRILLE
!         DU MODELE PAR UNE CONFIGURATION 931, EXECUTEE AU PREALABLE).
!        MISE A JOUR AVEC NOTRE CLIM SUR LA BANQUISE.
!        LE CHAMP AINSI ELABORE SERVIRA DE RAPPEL AU CHAMP DE SST ANALYSE ICI
!        DANS CANARI.
!        CE CHAMP EST STOCKE DANS LE BUFFER DES CHAMPS POINT DE GRILLE DU
!        MODELE.
!        SI LE SWITCH QUI COMMANDE L'ELABORATION DE CE CHAMP (NSSTLIS) N'EST
!        PAS ACTIVE, ON PREND LA CLIMATOLOGIE (CONF. 923) PARTOUT.

!  ROUTINE APPELANTE :    *CANARI*
!  -----------------

!  SOUS-PROGRAMMES APPELES :  routines d'utilisation des fichiers LFI
!  -----------------------    INCGPF SC2RDG SC2WRG DISGRID
!                             FAITOU FADIES FACILE FAIRME
!                             CHIEN CCHIEN UPDCAL

!  ARGUMENT D'ENTREE :
!  -----------------

!  ARGUMENT DE SORTIE :
!  ------------------

!  MODIFICATIONS :
!  -------------
!    R. El Khatib : 99-12-28 remove poles rows for extgpf.
!    R. El Khatib : 00-11-03 Optional communications in extgpf
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        M.Hamrud      01-Jul-2006 Revised surface fields
!      ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NPROMA   ,NSMAX    , &
 & NFLEVG   ,NDGSAG   ,NDGENG  
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG  ,NLOENG   ,NMENG    , &
 & VALH     ,VBH      ,NSTTYP   ,NHTYP    ,RMUCEN   , &
 & RLOCEN   ,RSTRET   ,VP00  
USE YOMLUN   , ONLY : NULOUT   ,NULCL1   ,NULCL2
USE YOMMP    , ONLY : MYPROC
USE YOMCT0   , ONLY : CNMEXP   ,NQUAD    ,LELAM    ,NPROC
USE YOMPHY1  , ONLY : TMERGL
USE YOMRIP   , ONLY : NINDAT
USE YOMTAG   , ONLY : MTAGGSUM
USE YOMSTA   , ONLY : RDTDZ1
USE YOMCST   , ONLY : RG
USE SURFACE_FIELDS_MIX, ONLY : SD_VX, YSD_VX, SP_CI, YSP_CI
USE QACTEX   , ONLY : NSSTLIS  ,RCLISST  ,NSEAICE
USE QACLIM   , ONLY : LCLIM
USE MPL_MODULE

IMPLICIT NONE

REAL(KIND=JPRB) :: ZICE(NGPTOT), ZSST(NGPTOT), ZCLIM(NGPTOT), ZGEO(NGPTOT), &
 & ZFIELD(NGPTOT), ZHIS(NGPTOTG), ZGAUSS(NDGSAG:NDGENG), ZDATO(3), &
 & ZMASK(NGPTOT)  

INTEGER(KIND=JPIM) :: IDATEF(11), ILMO(12)
CHARACTER (LEN = 16) ::  CLNOMF
LOGICAL :: LLEX

INTEGER(KIND=JPIM) :: IEND, IHIS, INDEX, IA2, IM2, IJ2, ILDAT, ITAG, &
 & IPROLEC, IREP, IST, JL, JSTGLO, IADAT, IFDAT, IA1, IM1, IJ1, IINF,IBLK

REAL(KIND=JPRB) :: ZSL1, ZSL2, ZSL3, ZLICE, ZCAL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

#include "cchien.intfb.h"
#include "chien.intfb.h"
#include "disgrid.intfb.h"
#include "updcal.intfb.h"

!**----------------------------------------------------------------------------
!**   1.    Recuperation du champ climatologique de SST.
!**         -------------------------------------------

IF (LHOOK) CALL DR_HOOK('CACLSST',0,ZHOOK_HANDLE)
WRITE(NULOUT,'(/,''entree subroutine CACLSST '')')

IF (RCLISST == 0.0_JPRB ) THEN
  WRITE(NULOUT,*) '   on ne veut pas faire de rappel climatologique '
  WRITE(NULOUT,*) '     --> inutile d''elaborer le champ ! '
  WRITE(NULOUT,*) '  '
  IF (LHOOK) CALL DR_HOOK('CACLSST',1,ZHOOK_HANDLE)
  RETURN
ENDIF

IF (.NOT.LCLIM) THEN
  WRITE(NULOUT,*) '   la lecture des fichiers climatologiques a ete impossible'
  WRITE(NULOUT,*) '     --> aucun rappel ne sera applique !'
  WRITE(NULOUT,*) '  '
  IF (LHOOK) CALL DR_HOOK('CACLSST',1,ZHOOK_HANDLE)
  RETURN
ENDIF

IST=1
DO JSTGLO = 1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
  IBLK=(JSTGLO-1)/NPROMA+1
  DO JL = IST,IEND
    ZCLIM(JSTGLO+JL-1)=SD_VX(JL,YSD_VX%YTSC%MP,IBLK)
    ZGEO(JSTGLO+JL-1) =SD_VX(JL,YSD_VX%YORO%MP,IBLK)
  ENDDO
ENDDO

IPROLEC=1

!**----------------------------------------------------------------------------
!**   2.    Traitement de la partie banquise.
!**         --------------------------------

WRITE(NULOUT,*) ' Traitement limite banquise '

!**   2.1  on ne veut pas utiliser de limite de glace actualisee

IF (NSEAICE == 0) THEN
  WRITE(NULOUT,*) '   on ne veut pas utiliser de limite de glace actualisee '
ELSE

!**   2.2  ouverture fichier glace et controle validite

  CLNOMF='ICMSH'//CNMEXP(1:4)//'SEAICE'
  INQUIRE(FILE=CLNOMF,EXIST=LLEX)
  IF (.NOT.LLEX) THEN
    WRITE(NULOUT,*)'   fichier limite glace absent'
    WRITE(NULOUT,*)'    --> pas d''actualisation de la banquise ! '
    NSEAICE=0
  ELSE
    INDEX=10
    IF (MYPROC == IPROLEC) THEN
      WRITE(NULOUT,*)'   controle fichier fourni '
      CALL FAITOU(IREP,NULCL1,.TRUE.,CLNOMF,'UNKNOWN',.TRUE.,.FALSE.,0, &
       & INDEX,IHIS,'CADRE SEAICE')  
      IINF=-1
      IF (.NOT.LELAM) THEN
        CALL CHIEN('CADRE SEAICE    ',NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,NDGLG, &
         & NDLON,NLOENG,NMENG,ZGAUSS,NHTYP,NFLEVG,VP00,VALH,VBH,NQUAD, &
         & IINF,NDGSAG,NDGENG,1.0E-7_JPRB,LLEX,NULOUT)  
      ELSE
        CALL CCHIEN('CADRE SEAICE',NULCL1,IINF)
        LLEX=.FALSE.
      ENDIF
      CALL FADIES(IREP,NULCL1,IDATEF)
      IADAT=IDATEF(1)*10000+IDATEF(2)*100+IDATEF(3)
      CALL FALAIS(IREP,NULCL1,'DATA.DATE',ZDATO,3)
      IFDAT=NINT(ZDATO(1))*10000+NINT(ZDATO(2))*100+NINT(ZDATO(3))
      IA1=NCCAA(NINDAT)
      IM1=NMM(NINDAT)
      IJ1=NDD(NINDAT)
      CALL UPDCAL(IJ1,IM1,IA1,-NSEAICE,IJ2,IM2,IA2,ILMO,99)
      ILDAT=IA2*10000+IM2*100+IJ2
      WRITE(NULOUT,'(''    date ANALYSE : '',I8,/,''    date fichier glace : '', &
       & I8,/,'' date donnees glace : '',I8)') NINDAT,IADAT,IFDAT  
      IF ( IADAT < ILDAT .OR. IADAT > NINDAT ) THEN
        WRITE(NULOUT,'(''  retard autorise : '',I2,'' jours donc '', &
         & ''MAUVAISE DATE fichier'',/,''    --> banquise non '',&
         & ''actualisee !'')') NSEAICE  
        NSEAICE=0
        OPEN(UNIT=63,FORM='FORMATTED')
        WRITE(63,'('' PB DATE FICHIER GLACE '')')
        CLOSE(63)
      ELSE
        WRITE(NULOUT,'(''  retard autorise : '',I2,'' jours donc OK '')') NSEAICE
      ENDIF

!**   2.3  lecture du champ de glace SSM/I

      IF (NSEAICE > 0) THEN
        CALL FACILE(IREP,NULCL1,'SURF',0,'SEA.ICE.CONC',ZHIS,.FALSE.)
        ZICE(1:NGPTOT)=ZHIS(1:NGPTOT)
        CALL DISGRID(1,0,NGPTOTG,ZHIS)
      ENDIF
      CALL FAIRME(IREP,NULCL1,'UNKNOWN')
    ENDIF

    IF (NPROC > 1) THEN
      ITAG=MTAGGSUM+147
      CALL MPL_BROADCAST(NSEAICE,KROOT=IPROLEC,KTAG=ITAG,CDSTRING='CACLSST:')
      IF (MYPROC /= IPROLEC .AND. NSEAICE > 0) CALL DISGRID(1,1,NGPTOT,ZICE)
    ENDIF

  ENDIF    ! fin fichier glace present
ENDIF      ! fin NSEAICE > 0

!**----------------------------------------------------------------------------
!**   3.    Traitement de la partie SST NESDIS.
!**         ----------------------------------

WRITE(NULOUT,*) ' Traitement SST NESDIS '
ZMASK(:)=0.0_JPRB

!**   3.1  on ne veut pas utiliser de SST NESDIS

IF (NSSTLIS == 0) THEN
  WRITE(NULOUT,*) '   on ne veut pas utiliser de SST americaine '
ELSE

!**   3.2  ouverture fichier SST et controles validite

  CLNOMF='ICMSH'//CNMEXP(1:4)//'SST'

  INQUIRE(FILE=CLNOMF,EXIST=LLEX)
  IF (.NOT.LLEX) THEN
    WRITE(NULOUT,*)'   fichier SST reference (NESDIS) absent'
    WRITE(NULOUT,*)'     --> pas de rappel vers cette SST en eau libre !'
    NSSTLIS=0
  ELSE
    INDEX=10
    IF (MYPROC == IPROLEC) THEN
      WRITE(NULOUT,*)'   controle fichier fourni '
      CALL FAITOU(IREP,NULCL2,.TRUE.,CLNOMF,'UNKNOWN',.TRUE.,.FALSE.,0, &
       & INDEX,IHIS,'CADRE SST')  
      IINF=-1
      IF (.NOT.LELAM) THEN
        CALL CHIEN('CADRE SST       ',NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,NDGLG,NDLON, &
         & NLOENG,NMENG,ZGAUSS,NHTYP,NFLEVG,VP00,VALH,VBH,NQUAD,IINF, &
         & NDGSAG,NDGENG,1.0E-7_JPRB,LLEX,NULOUT)  
      ELSE
        CALL CCHIEN('CADRE SST',NULCL2,IINF)
        LLEX=.FALSE.
      ENDIF
      CALL FADIES(IREP,NULCL2,IDATEF)
      IFDAT=IDATEF(1)*10000+IDATEF(2)*100+IDATEF(3)
      IA1=NCCAA(NINDAT)
      IM1=NMM(NINDAT)
      IJ1=NDD(NINDAT)
      CALL UPDCAL(IJ1,IM1,IA1,-NSSTLIS,IJ2,IM2,IA2,ILMO,99)
      ILDAT=IA2*10000+IM2*100+IJ2
      WRITE(NULOUT,'(''    date ANALYSE : '',I8,''    date SST NESDIS : '', &
       & I8)') NINDAT,IFDAT  
      IF ( IFDAT < ILDAT .OR. IFDAT > NINDAT ) THEN
        WRITE(NULOUT,'(''  retard autorise : '',I2,'' jours donc '', &
         & ''MAUVAISE DATE fichier'',/,''    --> application '', &
         & ''rappel climatologique classique !'')') NSSTLIS  
        NSSTLIS=0
        OPEN(UNIT=64,FORM='FORMATTED')
        WRITE(64,'('' PB DATE FICHIER SST NESDIS '')')
        CLOSE(64)
      ELSE
        WRITE(NULOUT,'(''  retard autorise : '',I2,'' jours donc OK '')') NSSTLIS
      ENDIF

!**   3.3  lecture champ SST de rappel

      IF (NSSTLIS > 0) THEN
        CALL FACILE(IREP,NULCL2,'SURF',0,'SST.CLIM.',ZHIS,.FALSE.)
        ZSST(1:NGPTOT)=ZHIS(1:NGPTOT)
        CALL DISGRID(1,0,NGPTOTG,ZHIS)
      ENDIF
      CALL FAIRME(IREP,NULCL2,'UNKNOWN')
    ENDIF

    IF (NPROC > 1) THEN
      ITAG=MTAGGSUM+148
      CALL MPL_BROADCAST(NSSTLIS,KROOT=IPROLEC,KTAG=ITAG,CDSTRING='CACLSST:')
      IF (MYPROC /= IPROLEC .AND. NSSTLIS > 0) CALL DISGRID(1,1,NGPTOT,ZSST)
    ENDIF

  ENDIF             ! fin fichier present
ENDIF               ! fin NSSTLIS /= 0

!**----------------------------------------------------------------------------
!**   4.    Elaboration du champ de rappel definitif.
!**         ----------------------------------------

ZSL1=271.4_JPRB         ! limite validite SST NESDIS interpolee  (~ -1.75 degC)

ZCAL=0.05_JPRB          ! enveloppe estimee erreurs interpolations

ZSL2=TMERGL-ZCAL        ! transformation mer en banquise  (~ -1.97 degC)
ZSL3=TMERGL+ZCAL        ! transformation mer en eau       (~ -1.87 degC)

ZLICE=60._JPRB          ! seuil pourcentage occurence glace --> banquise

! on ne fait pas de distinction terre/mer car les valeurs sur terre ne
! sont pas importantes (elles ne seront jamais utilisees !)

!**   4.1  mise au niveau modele de la SST NESDIS (conservation nature)

IF (NSSTLIS > 0) THEN
  DO JL = 1,NGPTOT
    IF (ZSST(JL) > ZSL1) ZMASK(JL)=1.0_JPRB
    ZHIS(JL)=MAX(ZSL3,ZSST(JL)+RDTDZ1/RG*ZGEO(JL))
  ENDDO
ENDIF

!**   4.2  cas sans banquise actualisee

IF (NSEAICE == 0) THEN
  ZFIELD(:)=ZCLIM(:)
  DO JL = 1,NGPTOT
    IF (ZCLIM(JL) > TMERGL .AND. ZMASK(JL) > 0.5_JPRB) ZFIELD(JL)=ZHIS(JL)
  ENDDO
ELSE

!**   4.3  cas avec actualisation de la banquise

  DO JL = 1,NGPTOT
    IF (ZICE(JL) < ZLICE) THEN
      IF (ZMASK(JL) > 0.5_JPRB) THEN
        ZFIELD(JL)=ZHIS(JL)
      ELSE
        ZFIELD(JL)=MAX(ZCLIM(JL),ZSL3)
      ENDIF
    ELSE
      ZFIELD(JL)=MIN(ZCLIM(JL),ZSL2)
    ENDIF
  ENDDO

ENDIF

!**----------------------------------------------------------------------------
!**   5.    Ecriture du champ dans le buffer.
!**         --------------------------------

! position du champ de rappel SST dans le buffer
IHIS=8

DO JSTGLO = 1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
  IBLK=(JSTGLO-1)/NPROMA+1
  DO JL = IST,IEND
    SP_CI(JL,YSP_CI%YCI(IHIS)%MP0,IBLK)=ZFIELD(JSTGLO+JL-1)
  ENDDO
ENDDO

WRITE(NULOUT,'('' Ecriture champ de rappel SST terminee'',/)')

!**----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CACLSST',1,ZHOOK_HANDLE)
END SUBROUTINE CACLSST
