SUBROUTINE CAGADE (KGUESS)
!****---------------------------------------------------------------------------
!****  DK CAGADE : TEST DATE DE l'EBAUCHE et eventuellement du fichier de STATs
!****     ------
!****  Auteurs   : JH 05/90, PB 10/90,11/90, FT 01/92, VC 04/93
!****---------------------------------------------------------------------------

!  BUT : ON VERIFIE SI LE GUESS MIS A DISPOSITION  CORRESPOND A LA DATE
!  ---   EXIGEE, PUIS ON INITIALISE LA VALEUR DE KGUESS :
!                       1 --> PREVI 6H,
!                       2 --> PREVI 12H,
!                       3 --> PREVI 18H,
!                       4 --> PREVI 24H,
!                       5 --> PREVI 30H,
!                       6 --> CLIMATOLOGIE (JPNBGU).

!  ROUTINE APPELANTE : *CALIFE*
!  -----------------

!  SOUS-PROGRAMMES APPELES :    AUCUN MAIS FONCTIONS IN-LINE DE GESTION DE
!  -----------------------      DATE EN CALENDRIER JULIEN (CD FCTTIM).

!  ARGUMENTS D'ENTREE :
!  ------------------
!     Explicite : aucun
!     Implicite : YOMCST YOMRIP YOMDAG QAPASS

!  ARGUMENTS DE SORTIE :
!  -------------------
!     Explicite : KGUESS - numero du guess
!     Implicite : aucun
!****---------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RDAY
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS   ,RTIMST
USE YOMDAG   , ONLY : NDATGU   ,NRESGU   ,NECHGU   ,NTYPGU
USE YOMLUN   , ONLY : NULOUT
USE QAPASS   , ONLY : JPNBGU

!  Parameter local reliant le numero du guess a une heure d'echeance

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(OUT)   :: KGUESS 
INTEGER(KIND=JPIM) :: JPCYCL

PARAMETER (JPCYCL = 6)

CHARACTER (LEN = 13) ::  CLTYP(5)

INTEGER(KIND=JPIM) :: IDAN, IDHEUR, IDJOUR, IDMOIS, ITAN, ITHH,&
 & ITIMED, ITIMET, ITIMETL, ITJOUR, ITMN, ITMOIS, &
 & ITSEC  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcttim.h"

!**------------------------------------------------------------------------
!**   1.    Initialisations diverses.
!**         ------------------------
!**

IF (LHOOK) CALL DR_HOOK('CAGADE',0,ZHOOK_HANDLE)
CLTYP(1)='   UNKNOWN   '
CLTYP(2)='CLIMATOLOGIE '
CLTYP(3)='   ARPEGE    '
CLTYP(4)='  EMERAUDE   '
CLTYP(5)='      CEP    '

!**------------------------------------------------------------------------
!**   2.    Lecture du guess.
!**         ----------------
!**

!  Date du guess demande :
IDAN   = NCCAA(NINDAT)
IDMOIS = NMM(NINDAT)
IDJOUR = NDD(NINDAT)
IDHEUR = NCTH(NSSSSS)
!  Temps absolu en HEURES du guess demande :
ITIMED = NINT(RTIMST/3600)
!  Date du guess lu :
ITAN   = NCCAA(NDATGU)
ITMOIS = NMM(NDATGU)
ITJOUR = NDD(NDATGU)
ITHH   = NRESGU/100
ITMN   = MOD(NRESGU,100)
ITSEC  = ITHH*3600+ITMN*60

!  Temps absolu en HEURES du guess trouve (date de lancement du modele qui a
!  fourni la prevision/guess, correspondant a NDATGU*10000+NRESGU+NECHGU) :
ITIMETL = NINT(RTIME(ITAN,ITMOIS,ITJOUR,ITSEC,RDAY)/3600)
ITIMET  = ITIMETL + NECHGU

WRITE(NULOUT,*) '   '
WRITE(NULOUT,*) '   '
WRITE(NULOUT,2001) IDJOUR,IDMOIS,IDAN,IDHEUR
WRITE(NULOUT,2002) ITJOUR,ITMOIS,ITAN,ITHH,NECHGU
WRITE(NULOUT,2003) CLTYP(NTYPGU)

IF (NTYPGU == 2) THEN
!  si CLIM :
  IF (ITMOIS == IDMOIS) THEN
    KGUESS = JPNBGU
  ELSE
    WRITE(NULOUT,1001) IDAN,IDMOIS,IDJOUR,IDHEUR,&
     & ITAN,ITMOIS,ITJOUR,ITHH,NECHGU  
    CALL ABOR1('CAGADE : INCOHERENCE DE MOIS CLIM')
  ENDIF
ELSE
!  si MODELE :
  IF (ITIMET == ITIMED) THEN
    KGUESS = MIN(6,MAX(NECHGU/JPCYCL,1))
  ELSE
    WRITE(NULOUT,1001) IDAN,IDMOIS,IDJOUR,IDHEUR,&
     & ITAN,ITMOIS,ITJOUR,ITHH,NECHGU  
    CALL ABOR1('CAGADE : INCOHERENCE DATE GUESS MODELE')
  ENDIF
ENDIF

!**------------------------------------------------------------------------
1001 FORMAT(//,2X,50('*'),/,2X,'**',46X,'**',&
 & /,'  **    PROBLEME DE DATE DU GUESS PROPOSE :       **',&
 & /,'  **    DATE DU GUESS DEMANDE:',I4,3I2,'         **',&
 & /,'  **    RUN DU GUESS TROUVE  :',I4,3I2,' ECH',I3,' **',&
 & /,'  **',46X,'**',/,2X,50('*'),//)  
2001 FORMAT(2X,50('*'),/,2X,'*',48X,'*',/,2X,'* GUESS DEMANDE     : ',&
 & 2(I2,'/'),I4,' A ',I2,' H',10X,'*',/,2X,'*',48X,'*',/,2X,50('*'))  
2002 FORMAT(2X,'*',48X,'*',/,2X,'* RUN DU GUESS TROUVE: ',2(I2,'/'),I4,&
 & ' A ',I2,'H, ECH =',I2,' *',/,2X,'*',48X,'*',/,2X,50('*'))  
2003 FORMAT(2X,'*',48X,'*',/,2X,'* TYPE DE GUESS TROUVE: ',A13,12X,'*',&
 & /,2X,'*',48X,'*',/,2X,50('*'),/)  

!**------------------------------------------------------------------------
WRITE(NULOUT,*) '  '
IF (LHOOK) CALL DR_HOOK('CAGADE',1,ZHOOK_HANDLE)
END SUBROUTINE CAGADE
