SUBROUTINE CAIDGU
!****-------------------------------------------------------------------
!****   CAIDGU : Impression des moyennes, ecart-types et extrema
!****   ------
!****   Auteur :    V.Casse   03/93
!****-------------------------------------------------------------------

!**  But : Calcul final et impression des moyennes, ecart-types et
!**  ---   extrema avec leur localisation.

!**  Arguments d'entree   : neant
!**  ------------------
!**  Arguments de sortie  : neant
!**  -------------------
!**  Sous-programme amont : CANARI
!**  --------------------
!**  Sous-programmes aval : neant
!**  --------------------
!**  COMMON utilises      : YOMDIM  YOMLUN  YOMCT0  YOMMP  YOMANCS
!**  ---------------        QAPDGU

!**  Algorithme : on dispose des elements calcules en multitache, on calcule
!**  ----------   moyenne, ecart-type ainsi que les extrema en MONOTACHE !
!**               idem en memoire distribuee (globalisation des resultats)

!**  Modifications :  
!**  -------------
!**      F. Taillefer 10-03-98  add message passing case
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NTASKS   ,NPROC
USE YOMMP    , ONLY : MYPROC
USE YOMTAG   , ONLY : MTAGGSUM
USE YOMANCS  , ONLY : RDEGREES
USE QAPDGU   , ONLY : JPNUMAX  ,NBPOINT  ,RMOY1    ,RMOY2    ,&
 & RMIN     ,RMAX     ,RPLACE   ,NBNIV    ,CNAME  
USE MPL_MODULE

IMPLICIT NONE
INTEGER(KIND=JPIM) :: IPOIN(JPNUMAX), IZPOIN(JPNUMAX)
REAL(KIND=JPRB) ::    ZMOY1(NFLEVG,JPNUMAX), ZMOY2(NFLEVG,JPNUMAX)
REAL(KIND=JPRB) ::    ZMIN(NFLEVG,JPNUMAX), ZMAX(NFLEVG,JPNUMAX)
REAL(KIND=JPRB) ::    ZLACE(NFLEVG,4,JPNUMAX), ZZLACE(NFLEVG,4,JPNUMAX)
REAL(KIND=JPRB) ::    ZZMOY1(NFLEVG,JPNUMAX), ZZMOY2(NFLEVG,JPNUMAX)
REAL(KIND=JPRB) ::    ZZMIN(NFLEVG,JPNUMAX), ZZMAX(NFLEVG,JPNUMAX)

INTEGER(KIND=JPIM) :: IBF1, IBF2, IERR, ITAG1,&
 & ITAG2, ITAG3, ITAG4, ITAG5, ITAG6, &
 & JNUM, JROC, JTASK, JZ  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!**---------------------------------------------------------------------
!**  - 1 - Globalisation des resultats si memoire non distribuee.
!**        -----------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CAIDGU',0,ZHOOK_HANDLE)
IF ( NPROC == 1 ) THEN

!*     2.1   initialisations

  IPOIN = 0
  ZMOY1(:,:) = 0.0_JPRB
  ZMOY2(:,:) = 0.0_JPRB
  ZMIN (:,:) = +1.E+12_JPRB
  ZMAX (:,:) = -1.E+12_JPRB
  ZLACE(:,:,:) = 99.99_JPRB

!*     2.2   nombre de donnees

  DO JTASK = 1 , NTASKS
    DO JNUM = 1 , JPNUMAX
      IPOIN(JNUM) = IPOIN(JNUM) + NBPOINT(JNUM,JTASK)
    ENDDO
  ENDDO

!*     2.3   moments du premier et second ordre + extrema et localisation en radians

  DO JTASK = 1 , NTASKS
    DO JNUM = 1 , JPNUMAX
!DIR$ SHORTLOOP
      DO JZ = 1 , NBNIV(JNUM)
        ZMOY1(JZ,JNUM) = ZMOY1(JZ,JNUM) + RMOY1(JZ,JNUM,JTASK)
        ZMOY2(JZ,JNUM) = ZMOY2(JZ,JNUM) + RMOY2(JZ,JNUM,JTASK)

        IF (RMIN(JZ,JNUM,JTASK)  <  ZMIN(JZ,JNUM)) THEN
          ZMIN(JZ,JNUM) = RMIN(JZ,JNUM,JTASK)
          ZLACE(JZ,1,JNUM) = RPLACE(JZ,1,JNUM,JTASK)
          ZLACE(JZ,2,JNUM) = RPLACE(JZ,2,JNUM,JTASK)
        ENDIF

        IF (RMAX(JZ,JNUM,JTASK)  >  ZMAX(JZ,JNUM)) THEN
          ZMAX(JZ,JNUM) = RMAX(JZ,JNUM,JTASK)
          ZLACE(JZ,3,JNUM) = RPLACE(JZ,3,JNUM,JTASK)
          ZLACE(JZ,4,JNUM) = RPLACE(JZ,4,JNUM,JTASK)
        ENDIF
      ENDDO
    ENDDO
  ENDDO

ELSE

!**---------------------------------------------------------------------
!**  - 2 - Globalisation des resultats si memoire distribuee.
!**        -------------------------------------------------

  IBF1=JPNUMAX*NFLEVG
  IBF2=JPNUMAX*NFLEVG*4

  ITAG1=MTAGGSUM+JPNUMAX+1
  ITAG2=ITAG1+1
  ITAG3=ITAG2+1
  ITAG4=ITAG3+1
  ITAG5=ITAG4+1
  ITAG6=ITAG5+1

!*     2.1   envoi resultat local au processeur 1

  IF (MYPROC /= 1) THEN

    CALL MPL_SEND(NBPOINT(1:JPNUMAX,1),KDEST=1,KTAG=ITAG1,KERROR=IERR,CDSTRING='CAIDGU:')
    IF (IERR < 0) CALL ABOR1(' CAIDGU: pb envoi NBPOINT')
    CALL MPL_SEND(RMOY1(1:IBF1,1,1),KDEST=1,KTAG=ITAG2,KERROR=IERR,CDSTRING='CAIDGU:')
    IF (IERR < 0) CALL ABOR1(' CAIDGU: pb envoi RMOY1')
    CALL MPL_SEND(RMOY2(1:IBF1,1,1),KDEST=1,KTAG=ITAG3,KERROR=IERR,CDSTRING='CAIDGU:')
    IF (IERR < 0) CALL ABOR1(' CAIDGU: pb envoi RMOY2')
    CALL MPL_SEND(RMIN(1:IBF1,1,1),KDEST=1,KTAG=ITAG4,KERROR=IERR,CDSTRING='CAIDGU:')
    IF (IERR < 0) CALL ABOR1(' CAIDGU: pb envoi RMIN')
    CALL MPL_SEND(RMAX(1:IBF1,1,1),KDEST=1,KTAG=ITAG5,KERROR=IERR,CDSTRING='CAIDGU:')
    IF (IERR < 0) CALL ABOR1(' CAIDGU: pb envoi RMAX')
    CALL MPL_SEND(RPLACE(1:IBF2,1,1,1),KDEST=1,KTAG=ITAG6,KERROR=IERR,CDSTRING='CAIDGU:')
    IF (IERR < 0) CALL ABOR1(' CAIDGU: pb envoi RPLACE')

  ELSE

!*     2.2   regroupement resultat

    IPOIN(:) = NBPOINT(:,1)
    ZMOY1(:,:) = RMOY1(:,:,1)
    ZMOY2(:,:) = RMOY2(:,:,1)
    ZMAX(:,:) = RMAX(:,:,1)
    ZMIN(:,:) = RMIN(:,:,1)
    ZLACE(:,:,:) = RPLACE(:,:,:,1)

    DO JROC = 1 , NPROC-1
      CALL MPL_RECV(IZPOIN(1:JPNUMAX),KTAG=ITAG1,CDSTRING='CAIDGU:')
      IPOIN(:)=IPOIN(:)+IZPOIN(:)
      CALL MPL_RECV(ZZMOY1(1:IBF1,1),KTAG=ITAG2,CDSTRING='CAIDGU:')
      ZMOY1(:,:)=ZMOY1(:,:)+ZZMOY1(:,:)
      CALL MPL_RECV(ZZMOY2(1:IBF1,1),KTAG=ITAG3,CDSTRING='CAIDGU:')
      ZMOY2(:,:)=ZMOY2(:,:)+ZZMOY2(:,:)
      CALL MPL_RECV(ZZMIN(1:IBF1,1),KTAG=ITAG4,CDSTRING='CAIDGU:')
      CALL MPL_RECV(ZZMAX(1:IBF1,1),KTAG=ITAG5,CDSTRING='CAIDGU:')
      CALL MPL_RECV(ZZLACE(1:IBF2,1,1),KTAG=ITAG6,CDSTRING='CAIDGU:')
      DO JNUM = 1 , JPNUMAX
        DO JZ = 1 , NBNIV(JNUM)
          IF (ZZMIN(JZ,JNUM)  <  ZMIN(JZ,JNUM)) THEN
            ZMIN(JZ,JNUM) = ZZMIN(JZ,JNUM)
            ZLACE(JZ,1,JNUM) = ZZLACE(JZ,1,JNUM)
            ZLACE(JZ,2,JNUM) = ZZLACE(JZ,2,JNUM)
          ENDIF
          IF (ZZMAX(JZ,JNUM)  >  ZMAX(JZ,JNUM)) THEN
            ZMAX(JZ,JNUM) = ZZMAX(JZ,JNUM)
            ZLACE(JZ,3,JNUM) = ZZLACE(JZ,3,JNUM)
            ZLACE(JZ,4,JNUM) = ZZLACE(JZ,4,JNUM)
          ENDIF
        ENDDO
      ENDDO
    ENDDO
  ENDIF

ENDIF

IF (MYPROC == 1) THEN

!**---------------------------------------------------------------------
!**  - 3 - Calcul moyenne et ecart type et localisation en degres.
!**        ------------------------------------------------------

  DO JNUM = 1 , JPNUMAX
!   si il y a eu des donnees ...
    IF (IPOIN(JNUM)  >  0) THEN
!DIR$ SHORTLOOP
      DO JZ = 1 , NBNIV(JNUM)
        ZMOY1(JZ,JNUM) = ZMOY1(JZ,JNUM) / IPOIN(JNUM)
        ZMOY2(JZ,JNUM) = SQRT &
         & (MAX(ZMOY2(JZ,JNUM)/IPOIN(JNUM)-ZMOY1(JZ,JNUM)**2,0.0_JPRB))  
        ZLACE(JZ,1,JNUM) = ZLACE(JZ,1,JNUM) * RDEGREES
        ZLACE(JZ,2,JNUM) = ZLACE(JZ,2,JNUM) * RDEGREES
        ZLACE(JZ,3,JNUM) = ZLACE(JZ,3,JNUM) * RDEGREES
        ZLACE(JZ,4,JNUM) = ZLACE(JZ,4,JNUM) * RDEGREES
      ENDDO
    ENDIF
  ENDDO

!**---------------------------------------------------------------------
!**  - 4 - Impressions.
!**        -----------

  DO JNUM = 1 , JPNUMAX
!   si il y a des niveaux demandes ...
    IF (NBNIV(JNUM)  >  0) THEN
      WRITE (NULOUT,901) CNAME(JNUM), IPOIN(JNUM)
!    si il y a eu des donnees ...
      IF (IPOIN(JNUM)  /=  0) THEN
        WRITE (NULOUT,902)
!DIR$ SHORTLOOP
        DO JZ = 1 , NBNIV(JNUM)
          WRITE(NULOUT,903) JZ, ZMOY1(JZ,JNUM), ZMOY2(JZ,JNUM),&
           & ZMIN(JZ,JNUM), ZLACE(JZ,1,JNUM), ZLACE(JZ,2,JNUM),&
           & ZMAX(JZ,JNUM), ZLACE(JZ,3,JNUM), ZLACE(JZ,4,JNUM)  
        ENDDO
        WRITE (NULOUT,904)
      ENDIF
    ENDIF
  ENDDO

ENDIF

901 FORMAT(/,' Renseignements fournis par CAIDGU pour le champ ',&
 & A16,' sur ',I6, ' points')  
902 FORMAT('  ----------------------------------------------',&
 & '------------------------------------------------',/,&
 & ' | niv.     moyenne    ecart-type     minimum   ',&
 & '  lat.     lon.      maximum     lat.     lon.  |',/,&
 & '  ----------------------------------------------',&
 & '------------------------------------------------')  
903 FORMAT(' |  ',I2,'  | ',E10.4,' | ',E10.4,' | ',E10.4,' | ',&
 & F6.2,' | ',F6.2,' | ',E10.4,' | ',F6.2,' | ',F6.2,' |')  
904 FORMAT('  ----------------------------------------------',&
 & '------------------------------------------------')  

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAIDGU',1,ZHOOK_HANDLE)
END SUBROUTINE CAIDGU
