SUBROUTINE CAINSU (PLAT, PRESS, PSTRET, KLEV, KTASK)
!****---------------------------------------------------------------------------
!****  CAINSU : determiner les parametres du modele statistique en un point
!****---------------------------------------------------------------------------
!*** ENTREES:  PLAT  : LAT DU PT
!*** -------   PRESS : PRESSIONS DU PT
!***           PSTRET: Fonction de l'etirement
!***           KLEV  : NOMBRE DE NIVEAUX
!***           KTASK : NUMERO DE TACHE
!***           COMMON "QACOSS"
!***
!*** SORTIE:   COMMON "QACOST"
!*** -------
!*** COMMONS:  YOMCST - QAPASS - QACOSS - QACOST
!*** -------
!*** Modif. :  LG 03/00 parameters for the statistical model of snow analyses added
!*** -----     LG 11/00 param. for the vert.part of the stat.model of SN anal. added
!****---------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI
USE QAPASS   , ONLY : JPNVS    ,JPBLS
USE QACOSS   , ONLY : XPPRS    ,XPLRS    ,XPOHZ    ,XPOHUD   ,&
 & XPOHHU   ,XRMU     ,XRNU     ,XPOSST   ,XPOVZ    ,&
 & XPOVH    ,XPOHT2   ,XPOHH2   ,XPOVORBL ,XPODIVBL ,&
 & XNUBL    ,XKPBL    ,XPOHSN   ,XPOHPSN  
  
USE QACOST   , ONLY : QPOHZ    ,QPOHUD   ,QPOHHU   ,QRMU     ,&
 & QRNU     ,QPOSST   ,QPOVZ    ,QPOVH    ,QPOHT2   ,&
 & QPOHH2   ,QPOVORBL ,QPODIVBL ,QNUBL    ,QKPBL    ,&
 & QPOHSN   ,QPOHPSN  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESS(KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRET(2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTASK 
INTEGER(KIND=JPIM) :: ILEV(0:KLEV), ILATS, ILATN
REAL(KIND=JPRB) :: ZTPRESPS(KLEV)
REAL(KIND=JPRB) ::    ZPINF(0:KLEV), ZPSUP(0:KLEV), ZPDLP(0:KLEV), ZMPRESPS
REAL(KIND=JPRB) ::    ZLAT, ZPOS, ZPON

INTEGER(KIND=JPIM) :: ILATR, JLEV, JNIV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!**---------------------------------------------------------------------
!**  - 1 - Recherche de la localisation verticale.
!**        --------------------------------------

IF (LHOOK) CALL DR_HOOK('CAINSU',0,ZHOOK_HANDLE)
DO JLEV = 1 , KLEV
  ZTPRESPS(JLEV) = EXP(PRESS(JLEV))
  IF (PRESS(JLEV)  <  XPLRS(1)) THEN
    ILEV(JLEV) = 2
    ZPINF(JLEV) = 1.0_JPRB
    ZPSUP(JLEV) = 0.0_JPRB
    ZPDLP(JLEV) = 1.0_JPRB
  ELSEIF (PRESS(JLEV)  >=  XPLRS(JPNVS)) THEN
    ILEV(JLEV) = JPNVS
    ZPINF(JLEV) = 0.0_JPRB
    ZPSUP(JLEV) = 1.0_JPRB
    ZPDLP(JLEV) = 1.0_JPRB
  ELSE
    DO JNIV = 2 , JPNVS
      IF (PRESS(JLEV)  <  XPLRS(JNIV)) GOTO 112
    ENDDO
    CALL ABOR1('CAINSU: PB INEXPLICABLE')
    112 CONTINUE
    ILEV(JLEV) = JNIV
    ZPINF(JLEV) = XPPRS(JNIV) - ZTPRESPS(JLEV)
    ZPSUP(JLEV) = ZTPRESPS(JLEV) - XPPRS(JNIV-1)
    ZPDLP(JLEV) = 1.0_JPRB/ (XPPRS(JNIV) - XPPRS(JNIV-1))
  ENDIF
ENDDO

! pression moyenne et niveau moyen de reference
! ZMPRESPS n'est pas necessairement entre XPPRS(ILEV(0)) et XPPRS(ILEV(0)-1)

ZMPRESPS = SUM(ZTPRESPS) / REAL(KLEV,JPRB)

ILEV(0) = 0
DO JLEV = 1 , KLEV
  ILEV(0) = ILEV(0) + ILEV(JLEV)
ENDDO
ILEV(0) = ILEV(0) / KLEV

ZPINF(0) = XPPRS(ILEV(0)) - ZMPRESPS
ZPSUP(0) = ZMPRESPS - XPPRS(ILEV(0)-1)
ZPDLP(0) = 1.0_JPRB/ (ZPINF(0) + ZPSUP(0))

!**---------------------------------------------------------------------
!**  - 2 - Recherche de la localisation horizontale.
!**        ----------------------------------------

ZLAT  = PLAT / RPI * 2.0_JPRB * REAL(JPBLS+1,JPRB)
ILATS = NINT ( ZLAT )
ILATN = ILATS + 1
ZPON  = ZLAT + 0.5_JPRB - REAL(ILATS,JPRB)
ZPOS  = 1.0_JPRB - ZPON

!**---------------------------------------------------------------------
!**  - 3 - Traitement des points aux poles et des autres.
!**        ---------------------------------------------

IF ( ILATS  ==  -JPBLS-1.OR. ILATS  ==  JPBLS+1) THEN
  ILATR = ILATN
  IF (ILATS  == JPBLS+1) ILATR = ILATS

  DO JLEV = 0 , KLEV
    QPOHZ(JLEV,KTASK)  = ( XPOHZ(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XPOHZ(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) &
     & * ZPDLP(JLEV) * PSTRET(2)  
    QPOHUD(JLEV,KTASK) = ( XPOHUD(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XPOHUD(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) &
     & * ZPDLP(JLEV) * PSTRET(2)  
    QRMU(JLEV,KTASK)   = ( XRMU(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XRMU(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) * ZPDLP(JLEV)  
    QRNU(JLEV,KTASK)   = ( XRNU(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XRNU(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) &
     & * ZPDLP(JLEV) / PSTRET(1)  
    QPOHHU(JLEV,KTASK) = ( XPOHHU(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XPOHHU(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) &
     & * ZPDLP(JLEV) * PSTRET(2)  
    QPOVZ(JLEV,KTASK)  = ( XPOVZ(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XPOVZ(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) * ZPDLP(JLEV)  
    QPOVH(JLEV,KTASK)  = ( XPOVH(ILATR,ILEV(JLEV))*ZPSUP(JLEV)+&
     & XPOVH(ILATR,ILEV(JLEV)-1)*ZPINF(JLEV) ) * ZPDLP(JLEV)  
  ENDDO
  QPOSST(KTASK)   = XPOSST(ILATR) * PSTRET(2)
  QPOHT2(KTASK)   = XPOHT2(ILATR) * PSTRET(2)
  QPOHH2(KTASK)   = XPOHH2(ILATR) * PSTRET(2)
  QPOHSN(KTASK)   = XPOHSN(ILATR) * PSTRET(2)
  QPOHPSN(KTASK)  = XPOHPSN(ILATR)
  QPOVORBL(KTASK) = XPOVORBL(ILATR) * PSTRET(2)
  QPODIVBL(KTASK) = XPODIVBL(ILATR) * PSTRET(2)
  QNUBL(KTASK)    = XNUBL(ILATR)
  QKPBL(KTASK)    = XKPBL(ILATR)
ELSE
  DO JLEV = 0 , KLEV
    QPOHZ(JLEV,KTASK)  =(( (XPOHZ(ILATS,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOHZ(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XPOHZ(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOHZ(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV) * PSTRET(2)  
    QPOHUD(JLEV,KTASK) =(( (XPOHUD(ILATS,ILEV(JLEV) )*ZPSUP(JLEV)+&
     & XPOHUD(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XPOHUD(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOHUD(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV) * PSTRET(2)  
    QRMU(JLEV,KTASK)   =(( (XRMU(ILATS,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XRMU(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XRMU(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XRMU(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV)  
    QRNU(JLEV,KTASK)   =(( (XRNU(ILATS,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XRNU(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XRNU(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XRNU(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV) / PSTRET(1)  
    QPOHHU(JLEV,KTASK) =(( (XPOHHU(ILATS,ILEV(JLEV) )*ZPSUP(JLEV)+&
     & XPOHHU(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XPOHHU(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOHHU(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV) * PSTRET(2)  
    QPOVZ(JLEV,KTASK)  =(( (XPOVZ(ILATS,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOVZ(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XPOVZ(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOVZ(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV)  
    QPOVH(JLEV,KTASK)  =(( (XPOVH(ILATS,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOVH(ILATS,ILEV(JLEV)-1)*ZPINF(JLEV))) * ZPOS &
     & + ( (XPOVH(ILATN,ILEV(JLEV)  )*ZPSUP(JLEV)+&
     & XPOVH(ILATN,ILEV(JLEV)-1)*ZPINF(JLEV))&
     & ) * ZPON ) * ZPDLP(JLEV)  
  ENDDO
  QPOSST(KTASK)   = ( XPOSST(ILATS)*ZPOS + XPOSST(ILATN)*ZPON ) * PSTRET(2)
  QPOHT2(KTASK)   = ( XPOHT2(ILATS)*ZPOS + XPOHT2(ILATN)*ZPON ) * PSTRET(2) 
  QPOHH2(KTASK)   = ( XPOHH2(ILATS)*ZPOS + XPOHH2(ILATN)*ZPON ) * PSTRET(2) 
  QPOHSN(KTASK)   = ( XPOHSN(ILATS)*ZPOS + XPOHSN(ILATN)*ZPON ) * PSTRET(2)
  QPOHPSN(KTASK)  = XPOHPSN(ILATS)*ZPOS + XPOHPSN(ILATN)*ZPON
  QPOVORBL(KTASK) = ( XPOVORBL(ILATS)*ZPOS + XPOVORBL(ILATN)*ZPON ) * PSTRET(2)
  QPODIVBL(KTASK) = ( XPODIVBL(ILATS)*ZPOS + XPODIVBL(ILATN)*ZPON ) * PSTRET(2)
  QNUBL(KTASK)    = XNUBL(ILATS)*ZPOS + XNUBL(ILATN)*ZPON
  QKPBL(KTASK)    = XKPBL(ILATS)*ZPOS + XKPBL(ILATN)*ZPON
ENDIF

!**------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAINSU',1,ZHOOK_HANDLE)
END SUBROUTINE CAINSU
