SUBROUTINE CALVER(KDLEN,KDBDY,KSET,KMXCOUNT,KPNLV,PRES,KCOUNT,PRESF5,KCMXXX)
!-----------------------------------------------------------------------
! Vertical interpolation of the model errors to observation points.
!-----------------------------------------------------------------------
!**   MODIFICATIONS
!     -------------
!     P. Moll         99/10/15 : Put Apache and ACHMT in cy22 (for MF)
!     F. Taillefer    00/09/10 : CMAFOC --> ODB
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : ROMEGA   ,RD
USE YOMOBSET , ONLY : MLNSET
USE YOMVNMB  , ONLY : NVNUMB
USE YOMANCS  , ONLY : RMDI
USE YOMDB
USE QALBAR   , ONLY : LRESCPL
USE QALORI   , ONLY : QCAOBS   ,QCAOBSET
USE QACOST   , ONLY : QPOHZ    ,QRNU     ,QPOVZ
USE GOMS_MIX     , ONLY : YGOMEC   ,YGOMCAN  ,YGOMCANA
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXCOUNT 
INTEGER(KIND=JPIM)               :: KDBDY ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPNLV(KDLEN,KDBDY) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KDLEN,KMXCOUNT) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCMXXX 
!   PRESF5 :  FULL LEVEL PRESSURE VALUES AT OBS POINTS OF SET KSET
!   PRES   :  PRESSURE LEVELS OF OBS

!   KCOUNT :  NUMBER OF OBS TO TREAT IN A GIVEN SET

REAL(KIND=JPRB) :: ZPHI(0:NFLEVG) , ZHUM(0:NFLEVG) , ZSTRET(2)

INTEGER(KIND=JPIM) :: IPOSM, IBODY, ICMXXX, IGPTSK, IPOS, &
 & INUMB, IPB, JJPB, JLEV, JOBS, JWAG  

REAL(KIND=JPRB) :: Z_PRESLOG(1), ZA, ZA2, ZB, ZKZ, ZLAT, ZNU2, ZQVAL, ZRCORI
REAL(KIND=JPRB) :: ZSST, ZT2, ZSN, ZH2, ZV1
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "cainsu.intfb.h"

#include "qavari.h"
#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('CALVER',0,ZHOOK_HANDLE)
IGPTSK = 1
ZQVAL  = RMDI
ICMXXX = KCMXXX

!-----------------------------------------------------------------------
!  - 1 -  Interpolation verticale du guess aux points d'obs.
!         --------------------------------------------------

IF ( .NOT. LRESCPL  .AND. LHOOK) CALL DR_HOOK('CALVER',1,ZHOOK_HANDLE)
IF ( .NOT. LRESCPL ) RETURN

!  - 1.1 -  boucle sur les obs.

DO JOBS = 1 , MLNSET(KSET)
  IPOSM    = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  ZRCORI    = MAX( ABS(YGOMEC(IPOSM)%ECCORI) , ROMEGA )
  ZLAT      = ROBHDR(JOBS,MDBLAT)
  IPOS      = QCAOBSET(KSET,JOBS)
  ZSTRET(1) = QCAOBS(1,IPOS)
  ZSTRET(2) = QCAOBS(2,IPOS)

!  - 1.2 -  boucle sur les wagons

  DO JWAG = 1 , KCOUNT(JOBS)
    IBODY = MLNKH2B(JOBS)+KPNLV(JOBS,JWAG)-1
    Z_PRESLOG(1) = LOG(PRES(JOBS,JWAG))
    CALL CAINSU(ZLAT,Z_PRESLOG,ZSTRET,1,IGPTSK)
    ZKZ  = QPOVZ(1,IGPTSK)
    ZA2  = QPOHZ(0,IGPTSK)**2
    ZNU2 = QRNU(0,IGPTSK)**2

!  localisation verticale de l'obs.
!  --------------------------------

    IF (PRES(JOBS,JWAG) >= PRESF5(JOBS,NFLEVG)) THEN
      IPB=0
    ELSE
      DO JJPB = NFLEVG-1 , 1 , -1
        IPB = JJPB + 1
        IF (PRES(JOBS,JWAG) > PRESF5(JOBS,JJPB)) GOTO 102
      ENDDO
      IPB = 1
      102 CONTINUE
    ENDIF

!  on remplit les verticales d'erreur de hu et de phi
!  et des champs de surface
!  --------------------------------------------------

    DO JLEV = 1 , NFLEVG
      ZHUM(JLEV) = YGOMCANA(IPOSM)%EMMH(JLEV)
      ZPHI(JLEV) = YGOMCANA(IPOSM)%EMMZ(JLEV)
    ENDDO

    ZSST  = YGOMCAN(IPOSM)%EMMST
    ZSN   = YGOMCAN(IPOSM)%EMMSN
    ZT2   = YGOMCAN(IPOSM)%EMMT2
    ZH2   = YGOMCAN(IPOSM)%EMMH2
    ZV1   = YGOMCAN(IPOSM)%EMMV1

    INUMB = ROBODY(IBODY,MDBVNM)

!   on ne veut pas de Q et de pwc pour le moment

    IF ( INUMB == NVNUMB( 6)   .AND. LHOOK) CALL DR_HOOK('CALVER',1,ZHOOK_HANDLE)
    IF ( INUMB == NVNUMB( 6)  ) RETURN
    IF ( INUMB == NVNUMB( 63)  .AND. LHOOK) CALL DR_HOOK('CALVER',1,ZHOOK_HANDLE)
    IF ( INUMB == NVNUMB( 63) ) RETURN

    IF ( INUMB == NVNUMB( 5) ) THEN

!  interpolation du champ d'humidite
!  ----------------------------------

      IF ( IPB == 1 ) THEN
!  l'obs. se trouve au-dessus du dernier niveau du modele.
        ZQVAL = ZHUM(1)
      ELSEIF ( IPB == 0 ) THEN
!  l'obs. se trouve au-dessous du dernier niveau du modele.
        ZQVAL = ZHUM(NFLEVG)
      ELSE
!  cas normal : on interpole l'humidite lineairement en log(p)
        ZA    = (ZHUM(IPB-1)-ZHUM(IPB))/&
         & LOG(PRESF5(JOBS,IPB-1)/PRESF5(JOBS,IPB))  
        ZB    = ZHUM(IPB)-ZA*LOG(PRESF5(JOBS,IPB))
        ZQVAL = ZA*LOG(PRES(JOBS,JWAG)) + ZB
      ENDIF

    ELSEIF ( INUMB == NVNUMB(1) .OR. INUMB == NVNUMB(2) .OR.&
       & INUMB == NVNUMB(3) .OR. INUMB == NVNUMB(4) .OR.&
       & INUMB == NVNUMB(8)                         ) THEN  

!  interpolation du champ de geopotentiel
!  ---------------------------------------

      IF ( IPB == 1 ) THEN
! l'obs se trouve au-dessus du dernier niveau du modele
        ZQVAL = ZPHI(1)
      ELSEIF ( IPB == 0 ) THEN
! l'obs se trouve au-dessous du dernier niveau du modele
        ZQVAL = ZPHI(NFLEVG)
      ELSE
! cas normal : on interpole le geopotentiel lineairement en log(p)
        ZA    = (ZPHI(IPB-1)-ZPHI(IPB))/&
         & LOG(PRESF5(JOBS,IPB-1)/PRESF5(JOBS,IPB))  
        ZB    = ZPHI(IPB)-ZA*LOG(PRESF5(JOBS,IPB))
        ZQVAL = ZA*LOG(PRES(JOBS,JWAG)) + ZB
      ENDIF

    ENDIF

!  calcul et saisie de la valeur finale dans ROBODY
!  ------------------------------------------------
    IF ( INUMB == NVNUMB( 5) ) THEN
      ROBODY(IBODY,ICMXXX) = ZQVAL
    ELSEIF ( INUMB == NVNUMB( 1)) THEN
      ROBODY(IBODY,ICMXXX) = FSIGU(ZQVAL/ZRCORI,ZA2,ZNU2)
      ROBODY(IBODY+1,ICMXXX) = ROBODY(IBODY,ICMXXX)
    ELSEIF ( INUMB == NVNUMB(3) .OR. INUMB == NVNUMB(4)  ) THEN
      ROBODY(IBODY,ICMXXX) = ZQVAL
    ELSEIF ( INUMB == NVNUMB( 8)) THEN
      ROBODY(IBODY,ICMXXX) = FSIGT(ZQVAL/RD,ZKZ)
    ELSEIF ( INUMB == NVNUMB(12)) THEN
      ROBODY(IBODY,ICMXXX) = ZSST
    ELSEIF ( INUMB == NVNUMB(25) .OR. INUMB == NVNUMB(34) .OR.&
       & INUMB == NVNUMB(46)                          ) THEN  
      ROBODY(IBODY,ICMXXX) = ZSN
    ELSEIF ( INUMB == NVNUMB( 7)) THEN
      ROBODY(IBODY,ICMXXX) = ZH2
    ELSEIF ( INUMB == NVNUMB(10)) THEN
      ROBODY(IBODY,ICMXXX) = ZT2
    ELSEIF ( INUMB == NVNUMB(56)) THEN
      ROBODY(IBODY,ICMXXX) = ZV1
      ROBODY(IBODY+1,ICMXXX) = ROBODY(IBODY,ICMXXX)
    ELSEIF ( INUMB == NVNUMB(62)) THEN
      ROBODY(IBODY,ICMXXX) = ZV1
      ROBODY(IBODY+1,ICMXXX) = ROBODY(IBODY,ICMXXX)
    ELSE
      WRITE(NULOUT,FMT='('' Erreur dans CALVER '')')
      CALL ABOR1('CALVER')
    ENDIF

!  - 1.3 -  fin de la boucle sur les wagons

  ENDDO

!  - 1.4 -  fin de la boucle sur les obs.

ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CALVER',1,ZHOOK_HANDLE)
END SUBROUTINE CALVER
