SUBROUTINE CAMELO
!--------------------------------------------------------------------
!    CAMELO : Preparation des observations
!    ------
!    Auteur :    Ph. Caille   27/02/90
!--------------------------------------------------------------------
!  Sequence d'appel : CALL CAMELO
!  ----------------
!  Sous-programme amont : CANARI
!  --------------------   
!  Sous-programmes aval : 
!  --------------------   
!     CABIYO : initialisation des ecarts-types d'erreur d'obs.
!     GETDB  : recuperation observations
!     CARACO : mise a jour informations sur les observations
!     CALICO : saisie des ecarts-types d'erreur d'obs et mise a jour
!              informations supplementaires sur les parametres observes
!     PUTDB  : mise a jour base des observations
!     CAPITO : impressions recapitulatives

!  COMMON utilises : YOMCMA - YOMLUN - YOMDIMO - QACTEX
!  ---------------    
!  Modifications : Ph. Caille   05/93  passage du FOC au CMAFOC
!  -------------   Ph. Caille   03/96  passage du CMAFOC au CMAFOC
!                  F. Taillefer 08/00  passage du CMAFOC a ODB
!                  M.Hamrud     10/03  CY28 Cleaning
!------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NOBTOT
USE YOMLUN   , ONLY : NULOUT   ,NULERR

IMPLICIT NONE

INTEGER(KIND=JPIM) :: INFO(1), IRET
REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "cabiyo.intfb.h"
#include "calico.intfb.h"
#include "capito.intfb.h"
#include "caraco.intfb.h"

IF (LHOOK) CALL DR_HOOK('CAMELO',0,ZHOOK_HANDLE)
WRITE (NULOUT,401)

!--------------------------------------------------------------------
!  - 1 -  Initialisation des ecarts-types d'erreurs d'observation
!         -------------------------------------------------------

CALL CABIYO

!--------------------------------------------------------------------
!  - 2 -  Saisie informations supplementaires
!         -----------------------------------

!  - 2.1 -  recuperation des observations en memoire

CALL GETDB('CAMELO',0,IRET,INFO,0,ZINFO,0,-1,-1,-1,-1,-1,-1)
IF (IRET==0) THEN
  IF (LHOOK) CALL DR_HOOK('CAMELO',1,ZHOOK_HANDLE)
  RETURN
ENDIF
IF (IRET/=NOBTOT) THEN
  WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
  WRITE(NULERR,*)'  Expecting IOBTOT=',NOBTOT,', but got =',IRET
  CALL ABORTDB('CAMELO','INCONSISTENCIES FROM GETDB',IRET)
ENDIF

!  - 2.2 -  mise a jour donnees sur observations

CALL CARACO

!  - 2.3 -  saisie ecart-types d'erreur d'obs (+ autres donnees parametres)

CALL CALICO

!  - 2.4 -  mise a jour base de donnees

CALL PUTDB('CAMELO',0,IRET,INFO,0,ZINFO,0)

!--------------------------------------------------------------------
!  - 3 -  Impressions recapitulatives
!         ---------------------------

CALL CAPITO( NULOUT )

WRITE (NULOUT,402)

!------------------------------------------------------------------
!  - 4 -  Formats
!         -------

401 FORMAT(///,'            *******************************',&
 & '*************************************************',&
 & '************',/,12X,'*',90(' '),'*',/,'          ',&
 & '  *      C H A R G E M E N T   D E S   O B S    E',&
 & ' N    M E M O I R E    C E N T R A L E      *',/,&
 & 12X,'*',90(' '),'*',/,'            ***************',&
 & '*************************************************',&
 & '****************************',//)  

402 FORMAT(//,'            *******************************',&
 & '*************************************************',&
 & '************',/,12X,'*',90(' '),'*',/,'          ',&
 & '  *                                F I N   D E   ',&
 & 'C A M E L O                                 *',/,&
 & 12X,'*',90(' '),'*',/,'            ***************',&
 & '*************************************************',&
 & '****************************',//)  

!--------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAMELO',1,ZHOOK_HANDLE)
END SUBROUTINE CAMELO
