SUBROUTINE CANCER
!-----------------------------------------------------------------------
!   CANCER : Verification des residus et ecriture des flags
!   ------
!-----------------------------------------------------------------------
!  auteur        :  P. Moll 06/91
!  modifications :  VC 07/93, Ph.C 11/93, VC 04/94, Ph.C 03/96, FT 09/00
!                   MS 03/01  adds LVERAL
!                   FT 03/02  debug rejet 2e composante vent
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIMO  , ONLY : JPNOTP
USE PARCMA   , ONLY : JPMXVAR
USE YOMDIMO  , ONLY : NOBTOT
USE YOMCT0   , ONLY : LELAM
USE YOMANCS  , ONLY : RMDI
USE YOMDB
USE YOMVNMB  , ONLY : NVNUMB
USE YOMCOCTP , ONLY : NSYNOP   ,NSRSCD   ,NATSCD   ,NSYRCD   ,NRADOH
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMMP    , ONLY : MYPROC
USE YOMCST   , ONLY : RG
USE YOMOBS   , ONLY : LSLREJ
USE QALBAR   , ONLY : LRESCPL  ,NRESECR
USE QALORI   , ONLY : QCAOBS
USE QACOST   , ONLY : QPOVZ
USE QACOBS   , ONLY : OROLIM   ,ORODIF
USE QAMCOK   , ONLY : RCOEFK
USE QAKEKI   , ONLY : NCQFCO   ,NCQFPC   ,NCQFPI   ,NCQFIN   ,&
 & NCQECO   ,NCQEPC   ,NCQEPI   ,NCQEIN   ,NCLNIN  
USE QAVARA   , ONLY : NVXINV   ,NVMXINV
USE QACTEX   , ONLY : LVERAL
USE GOMS_MIX     , ONLY : YGOMEC, NGOMS1D
IMPLICIT NONE

INTEGER(KIND=JPIM) :: INBCLL(JPMXVAR,JPNOTP,0:25), ICUMCL(JPMXVAR,JPNOTP,0:22)
INTEGER(KIND=JPIM) :: INBCLG(JPMXVAR,JPNOTP,0:25)
INTEGER(KIND=JPIM) :: ITOT(JPMXVAR,JPNOTP), IREJL(JPMXVAR,JPNOTP,0:1)
INTEGER(KIND=JPIM) :: IREJG(JPMXVAR,JPNOTP,0:1), IOROL(2), IOROG(2)
INTEGER(KIND=JPIM) :: INLV(NOBTOT),ITY(NOBTOT)
REAL(KIND=JPRB) :: ZSTRET(2)
REAL(KIND=JPRB) :: ZMREL(JPMXVAR,JPNOTP,2), ZMREG(JPMXVAR,JPNOTP,2)
LOGICAL :: LLTYP(JPNOTP), LLALT, LLORO, LLAND
CHARACTER (LEN = 32) ::  CLNOM(JPMXVAR)
INTEGER(KIND=JPIM) , DIMENSION(:,:), ALLOCATABLE :: ICODW

INTEGER(KIND=JPIM) :: IADW, IADWB, IADWEC, IBAD, IBODY, INBODY, IBPCEBA, IRET, INFO(1), &
 & IPOSM, IBPFINA, ICQF, ICQF1, ICQF1O, &
 & ICQF2, ICQF2O, ICQF3, ICQF4, ICQFB, IDIM, &
 & IFLAG, IFLG, IFLGB, IFLGEC, IFLGLNB, IFLGLU, &
 & ILEN, ILENF, ILN, INM, INMB, INMS, INOW, IOT, IRG, ITASK, &
 & IVNM, IVNMB, IVNMEC, IWAG, IXNLV, IRESECR, &
 & JJ, JNM, JOBS, JTY, JW, JWB  

REAL(KIND=JPRB) :: ZALT, ZCORVE, ZDEN2, ZEPS, ZKZ, ZLAT, ZPRB, ZPRES, &
 & ZPRESB, ZPRH, ZPRM(1), ZRES, ZRG, ZSIGG, ZSIGO, ZALTDIF, ZINFO(1)  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "qastat.h"
#include "qabitu.h"

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cainsu.intfb.h"
#include "casnd1.intfb.h"
#include "casndr1.intfb.h"

#include "openmp_obs.h"

!-----------------------------------------------------------------------
! - 1 - Initialisations diverses.
!       ------------------------

IF (LHOOK) CALL DR_HOOK('CANCER',0,ZHOOK_HANDLE)
CLNOM = ' NON REPERTORIE                 '

CLNOM(1)=  ' VENT U                         '
CLNOM(2)=  ' VENT V                         '
CLNOM(3)=  ' GEOPOTENTIEL                   '
CLNOM(4)=  ' EPAISSEUR                      '
CLNOM(5)=  ' HUMIDITE RELATIVE (NIVEAU)     '
CLNOM(7)=  ' HUMIDITE RELATIVE A 2M         '
CLNOM(8)=  ' TEMPERATURE                    '
CLNOM(10)= ' TEMPERATURE A 2M               '
CLNOM(12)= ' TEMPERATURE DE SURFACE         '
CLNOM(34)= ' TAUX DE PRECIPITATIONS         '
CLNOM(46)= ' QUANTITE DE NEIGE              '
CLNOM(51)= ' RADIANCE BRUTE                 '
CLNOM(52)= ' FRACTION NUAGEUSE              '
CLNOM(53)= ' SCATTEROMETRE                  '
CLNOM(56)= ' VENT U A 10M                   '
CLNOM(57)= ' VENT V A 10M                   '
CLNOM(58)= ' HUMIDITE RELATIVE DE COUCHE    '

ITOT (:,:)    = 0
IREJL (:,:,:) = 0
ZMREL (:,:,:) = 0.0_JPRB
INBCLL(:,:,:) = 0
INBCLG(:,:,:) = 0

IOROL (:) = 0
IOROG (:) = 0

ZPRB     = RMDI
ILEN     = 1
ILENF    = 4
IWAG     = 0
LLTYP(:) = .FALSE.

! on n'est pas en multi-taches, donc ITASK doit valoir 1.
ITASK = 1

CALL GETDB('CANCER', 0, IRET, INFO, 0, ZINFO, 0, -1, -1, -1, -1, -1, -1)

IF (IRET /= NOBTOT) THEN
  WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
  WRITE(NULERR,*)' Expecting ',NOBTOT,', but got =',IRET
  CALL ABORTDB('CANCER', 'INCONSISTENCIES FROM GETDB', IRET)
ENDIF

IF (NRESECR == 1) THEN
  IRESECR=MDBOMF
ELSE
  IRESECR=MDBOMN
ENDIF

! dimensionnement de ICODW
DO JOBS = 1,NOBTOT
  INLV(JOBS) = MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  ITY(JOBS) = ROBHDR(JOBS,MDBOTP)
ENDDO
IXNLV = MAXVAL(INLV(1:NOBTOT))

ALLOCATE (ICODW(NOBTOT,IXNLV))
ICODW(:,:) = 0

! mise a jour de ECEMIS qui a servi au rejet des parametres de surface
! selon le masque terre/mer d'interpolation du guess.
IF (LSLREJ) THEN
  DO JOBS = 1 , NGOMS1D
    YGOMEC(JOBS)%ECEMIS=YGOMEC(JOBS)%ECEMIS-REAL(INT(YGOMEC(JOBS)%ECEMIS),JPRB)
  ENDDO
ENDIF

!-----------------------------------------------------------------------
!  - 2 - Verification des residus par rapport a sigma.
!        --------------------------------------------

!  - 2.1 -  Boucle sur les obs.

DO JOBS = 1 , NOBTOT

  IF ( LELAM .AND. ROBHDR(JOBS,MDBTLA) == RMDI ) CYCLE

  ZALT      = ROBHDR(JOBS,MDBALT)
  ZLAT      = ROBHDR(JOBS,MDBLAT)
  ZSTRET(1) = QCAOBS(1,JOBS)
  ZSTRET(2) = QCAOBS(2,JOBS)
  IOT       = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)
  INOW      = 0
  IPOSM    = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  IF (ZALT /= RMDI) THEN
    ZALTDIF = ABS(ZALT-YGOMEC(IPOSM)%ECOROG/RG)
  ELSE
    ZALTDIF = 0.0_JPRB
  ENDIF

  LLAND= ITY(JOBS)==NSYNOP .AND. (IOT==NSRSCD.OR.IOT==NATSCD.OR.IOT==NSYRCD.OR.IOT==NRADOH)

!  elimination des SYNOP trop eleves
  IF (LLAND .AND. ZALT > OROLIM ) THEN
    LLALT=.TRUE.
    IOROL(1)=IOROL(1)+1
  ELSE
    LLALT=.FALSE.
  ENDIF

!  elimination des SYNOP si (relief modele-altitude obs) > ORODIF
  IF (LLAND .AND. ZALTDIF > ORODIF ) THEN
    LLORO=.TRUE.
    IOROL(2)=IOROL(2)+1
  ELSE
    LLORO=.FALSE.
  ENDIF

!  - 2.2 -  Boucle sur les wagons

  INBODY = MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  DO JW = 1,INBODY

    IBODY = MLNKH2B(JOBS)+(JW-1)
    IWAG  = IWAG + 1
    INOW  = INOW + 1
    ZSIGG = ROBODY(IBODY,MDBFGE)
    ZPRH  = ROBODY(IBODY,MDBPPP)
    ZPRM(1)  = ZPRH

!  recuperation du residu (MDBOMF ou MDBOMN)
!  -----------------------------------------
    ZRES  = ROBODY(IBODY,IRESECR)
    IVNM  = ROBODY(IBODY,MDBVNM)
    INM   = NVMXINV(IVNM)
    IF ( INM < 0.OR. INM > JPMXVAR ) CALL ABOR1('CANCER')
    INMS  = NVXINV(IVNM)
    IBAD  = 0
    IF (IVNM == NVNUMB( 1).OR.IVNM == NVNUMB( 2).OR.&
     & IVNM == NVNUMB(56).OR.IVNM == NVNUMB(57)) ICODW(JOBS,INOW)=1  

!  elimination des parametres visiblement mauvais
!  ----------------------------------------------
    IF (ITY(JOBS) == NSYNOP .AND. ZALT > 8807._JPRB) IBAD = 1
    IF (ABS(ZRES) >= 99999._JPRB) IBAD = 1

!  traitement d'une donnee de couche
!  ---------------------------------
    IF ( IVNM == NVNUMB(4) .OR. IVNM == NVNUMB(58) ) THEN

      IF ( IVNM == NVNUMB(4) ) THEN
        ZPRB = EXP( ROBODY(IBODY,MDBPRL) )
        ZPRM(1) = LOG( (ZPRB+ZPRH)*0.5_JPRB )
      ELSEIF ( IVNM == NVNUMB(58) ) THEN
        ZPRB = ROBODY(IBODY,MDBPRL)
        ZPRM(1) = ROBODY(IBODY,MDBRBPIO)
      ENDIF

      CALL CAINSU(ZLAT,ZPRM,ZSTRET,1,ITASK)
      ZKZ = QPOVZ(1,ITASK)

!  couche trop petite
      IF ( ABS(ZPRH-ZPRB) <= 1.0_JPRB ) THEN
        IBAD = 1
      ELSE
        ZCORVE = 1.0_JPRB/(1.0_JPRB+ZKZ*LOG(ZPRB/ZPRH)**2)

        IF ( IVNM == NVNUMB(4) ) THEN
          ZSIGG = SQRT(ROBODY(IBODY,MDBFGE)**2 + &
           & ROBODY(IBODY,MDBRBOE)**2 - &
           & 2.0_JPRB*ROBODY(IBODY,MDBFGE)* &
           & ROBODY(IBODY,MDBRBOE)*ZCORVE)  
        ELSEIF ( IVNM == NVNUMB(58) ) THEN
          IF ( LRESCPL ) THEN
            ROBODY(IBODY,MDBFGE) = 0.5_JPRB *&
             & SQRT(ROBODY(IBODY,MDBFGE)**2 + &
             & ROBODY(IBODY,MDBRBOE)**2 + &
             & 2.0_JPRB*ROBODY(IBODY,MDBFGE)* &
             & ROBODY(IBODY,MDBRBOE)*ZCORVE)  
            ZSIGG = ROBODY(IBODY,MDBFGE)
          ENDIF
        ENDIF
      ENDIF

    ENDIF

!  recherche des parametres sur liste noire
!  ----------------------------------------
    IFLG = ROBODY(IBODY,MDBFLG)
    IFLGLNB = KBITLU(IFLG,NCLNIN,ILEN)

!  elimination des parametres des SYNOP trop eleves
!  ------------------------------------------------
    IF (LLALT) THEN
      IFLGLNB = 1
      IFLG = KBIT1OU0(IFLG,IFLGLNB,NCLNIN)
    ENDIF

!  elimination des parametres des SYNOP pour lesquels
!  relief modele-altitude obs est superieure a ORODIF
!  --------------------------------------------------
    IF (LLORO) IBAD = 1

!  repartition des observations par classe
!  ---------------------------------------
    IF ( IBAD == 0 ) THEN
      ZSIGO = ROBODY(IBODY,MDBFOE)
      ZDEN2 = ZSIGO**2 + ZSIGG**2
      ZRG = ABS(ZRES) / SQRT(ZDEN2)
      IF ( IFLGLNB == 1 ) THEN
        IRG = 25
      ELSE
        IRG = MIN(22,INT(5._JPRB*ZRG))
        ZMREL(INM,ITY(JOBS),1) = ZMREL(INM,ITY(JOBS),1) + ZRES
        ZMREL(INM,ITY(JOBS),2) = ZMREL(INM,ITY(JOBS),2) + ZRES**2
      ENDIF
    ELSE
      ZRG = 0.0_JPRB
      IRG = 23 + IFLGLNB
    ENDIF

    INBCLL(INM,ITY(JOBS),IRG) = INBCLL(INM,ITY(JOBS),IRG) + 1

    IF ( ZRG >= RCOEFK(ITY(JOBS),INMS) )&
     & IREJL(INM,ITY(JOBS),IFLGLNB) = IREJL(INM,ITY(JOBS),IFLGLNB)+1  

!  ecriture des flags de rejet
!  ---------------------------
    IF ( LRESCPL .AND. .NOT. LVERAL ) THEN
      IFLAG = 0
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQFCO)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQFPC)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQFPI)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQFIN)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQECO)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQEPC)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQEPI)
      IFLG  = KBIT1OU0(IFLG,IFLAG,NCQEIN)

      IF ( IBAD /= 0 ) THEN
        IBPCEBA = NCQEIN
        IBPFINA = NCQFIN
      ELSEIF ( ZRG >= RCOEFK(ITY(JOBS),INMS) ) THEN
        IBPCEBA = NCQEPI
        IBPFINA = NCQFPI
      ELSEIF ( ZRG >= 0.7_JPRB*RCOEFK(ITY(JOBS),INMS) ) THEN
        IBPCEBA = NCQEPC
        IBPFINA = NCQFCO
      ELSE
        IBPCEBA = NCQECO
        IBPFINA = NCQFCO
      ENDIF

      IF ( IFLGLNB == 1 ) IBPFINA = NCQFIN

      IFLAG = 1
      IFLG  = KBIT1OU0(IFLG,IFLAG,IBPCEBA)
      IFLG  = KBIT1OU0(IFLG,IFLAG,IBPFINA)

      ROBODY(IBODY,MDBFLG)  = IFLG
      ROBODY(IBODY,MDB_QC_A) = ZRG
      ROBODY(IBODY,MDB_QC_PGE) = ZRG
    ENDIF

  ENDDO

ENDDO

!  calcul de divers totaux pour impression
!  ---------------------------------------

IDIM=JPMXVAR*JPNOTP*26
CALL CASND1(INBCLL,INBCLG,IDIM,10)
IDIM=JPMXVAR*JPNOTP*2
CALL CASNDR1(ZMREL,ZMREG,IDIM,11)

CALL CASND1(IOROL,IOROG,2,12)

DO JTY = 1 , JPNOTP
  DO JNM = 1 , JPMXVAR
    DO JJ = 0 , 22
      ITOT(JNM,JTY) = ITOT(JNM,JTY) + INBCLG(JNM,JTY,JJ)
      ICUMCL(JNM,JTY,JJ) = ITOT(JNM,JTY)
    ENDDO
    IF (ITOT(JNM,JTY) /= 0) LLTYP(JTY) = .TRUE.
  ENDDO
ENDDO

IF ( .NOT.LRESCPL .AND. .NOT. LVERAL ) GOTO 400

!-----------------------------------------------------------------------
! - 3 - Rejet des vents dont une composante a ete rejetee.
!       -------------------------------------------------

DO JOBS = 1 , NOBTOT
  IF ( LELAM .AND. ROBHDR(JOBS,MDBTLA) == RMDI ) CYCLE

  DO JW = 1 , INLV(JOBS)-1
    IF ( ICODW(JOBS,JW) == 0 ) CYCLE
    IADW  = MLNKH2B(JOBS)+(JW-1)
    IFLG  = ROBODY(IADW,MDBFLG)
    ICQF  = KBITLU(IFLG,1,ILENF)
    ZPRES = ROBODY(IADW,MDBPPP)

    DO JWB = JW+1 , INLV(JOBS)
      IF ( ICODW(JOBS,JWB) == 0 ) CYCLE
      IADWB  = MLNKH2B(JOBS)+(JWB-1)
      ZPRESB = ROBODY(IADWB,MDBPPP)

      IF ( ZPRES == ZPRESB ) THEN
        IVNMB = ROBODY(IADWB,MDBVNM)
        IFLGB = ROBODY(IADWB,MDBFLG)
        ICQFB = KBITLU(IFLGB,1,ILENF)
        IF ( ICQF == ICQFB ) THEN
          CYCLE
        ELSEIF ( ICQF > ICQFB ) THEN
          IFLGLU = IFLG
          IFLGEC = IFLGB
          IADWEC = IADWB
          IVNMEC = IVNMB
        ELSEIF ( ICQF < ICQFB ) THEN
          IFLGLU = IFLGB
          IFLGEC = IFLG
          IADWEC = IADW
          IVNMEC = IVNM
        ENDIF
        ICQF1O = KBITLU(IFLGEC,1,ILEN)
        ICQF2O = KBITLU(IFLGEC,2,ILEN)
        ICQF1  = KBITLU(IFLGLU,1,ILEN)
        ICQF2  = KBITLU(IFLGLU,2,ILEN)
        ICQF3  = KBITLU(IFLGLU,3,ILEN)
        ICQF4  = KBITLU(IFLGLU,4,ILEN)
        IFLGEC = KBIT1OU0(IFLGEC,ICQF1,1)
        IFLGEC = KBIT1OU0(IFLGEC,ICQF2,2)
        IFLGEC = KBIT1OU0(IFLGEC,ICQF3,3)
        IFLGEC = KBIT1OU0(IFLGEC,ICQF4,4)
        ROBODY(IADWEC,MDBFLG) = IFLGEC
        IF ( ICQF1O+ICQF2O == 0 ) THEN
          IFLGLNB = KBITLU(IFLGB,NCLNIN,ILEN)
          INMB    = NVMXINV(IVNMEC)
          IREJL(INMB,ITY(JOBS),IFLGLNB) = IREJL(INMB,ITY(JOBS),IFLGLNB) + 1
        ENDIF
      ENDIF
    ENDDO

  ENDDO

ENDDO

!-----------------------------------------------------------------------
!  - 4 - Impression des diagnostics.
!        --------------------------

400 CONTINUE

IDIM=JPMXVAR*JPNOTP*2
CALL CASND1(IREJL,IREJG,IDIM,13)

IF (MYPROC == 1) THEN

  WRITE (NULOUT,411) IWAG

  DO JTY = 1 , JPNOTP
    IF ( .NOT.LLTYP(JTY) ) CYCLE
    WRITE (NULOUT,412) JTY

    DO JNM = 1 , JPMXVAR
      IF ( ITOT(JNM,JTY ) == 0) CYCLE
      WRITE (NULOUT,'(A32,/)') CLNOM(JNM)
      WRITE (NULOUT,421) (REAL(JJ,JPRB)*0.01_JPRB, JJ=20,440,20)
      WRITE (NULOUT,420) (INBCLG(JNM,JTY,JJ), JJ=0,21)
      WRITE (NULOUT,421) (100._JPRB*REAL(INBCLG(JNM,JTY,JJ),JPRB)/&
       & MAX(1.0_JPRB,REAL(ICUMCL(JNM,JTY,22),JPRB)), JJ=0,21)  
      WRITE (NULOUT,421) (100._JPRB*REAL(ICUMCL(JNM,JTY,JJ),JPRB)/&
       & MAX(1.0_JPRB,REAL(ICUMCL(JNM,JTY,22),JPRB)), JJ=0,21)  
      WRITE (NULOUT,413)
      ILN = REAL(INBCLG(JNM,JTY,25),JPRB)+REAL(INBCLG(JNM,JTY,24),JPRB)
      WRITE (NULOUT,419) ' WAGONS REJETES : ', IREJG(JNM,JTY,0),&
       & ' (',100._JPRB*REAL(IREJG(JNM,JTY,0),JPRB)/REAL(ITOT(JNM,JTY),JPRB),&
       & '% )  plus',INBCLG(JNM,JTY,23),' ELIMINES (',100._JPRB*&
       & REAL(INBCLG(JNM,JTY,23),JPRB)/REAL(ITOT(JNM,JTY),JPRB),&
       & '% )   -  LISTE NOIRE ( ',ILN,' ) : ',INBCLG(JNM,JTY,24),&
       & ' ELIMINES  et  ',IREJG(JNM,JTY,1),' REJETES (',100._JPRB*&
       & REAL(IREJG(JNM,JTY,1),JPRB)/MAX(1.0_JPRB,REAL(ILN,JPRB)),'% )'  
      WRITE (NULOUT,413)
    ENDDO

  ENDDO

!  Impressions de resultats statistiques supplementaires
!  -----------------------------------------------------

  WRITE (NULOUT,414) IOROG(1),IOROG(2)
  ZEPS = 0.000001_JPRB
  WRITE (NULOUT,415)

  DO JTY = 1 , JPNOTP
    IF (.NOT.LLTYP(JTY)) CYCLE
    WRITE (NULOUT,412) JTY

    DO JNM = 1 , JPMXVAR
      IF ( ITOT(JNM,JTY)  <=  0 ) CYCLE
      WRITE (NULOUT,417) CLNOM(JNM), 'OBS-MOD = ',&
       & ZMREG(JNM,JTY,1)/ITOT(JNM,JTY), ' SIGMA = ',&
       & SQRT(ABS(ZMREG(JNM,JTY,2)/ITOT(JNM,JTY)+ZEPS-&
       & (ZMREG(JNM,JTY,1)/ITOT(JNM,JTY))**2)),&
       & ' (',ITOT(JNM,JTY), ')'  
    ENDDO

  ENDDO

  WRITE (NULOUT,416)

ENDIF

DEALLOCATE(ICODW)

!-----------------------------------------------------------------------
!  - 5 - Mise a jour ODB.
!        ---------------

CALL PUTDB('CANCER', 0, IRET, INFO, 0, ZINFO, 0)

!-----------------------------------------------------------------------
!  - 6 - Formats.
!        -------

411 FORMAT(/,'   ******************************************',/,&
 & '   * Impression des diagnostics des residus *',/,&
 & '   ******************************************',//,&
 & ' nombre total de wagons : ',I8,/)  
412 FORMAT(/,12X,'Type d''observations numero ',I2,/)
413 FORMAT(1X)
414 FORMAT(/,' Elimination des SYNOP sur relief:',/,&
 & '    altitude station trop haute: ',I5,/,&
 & '    relief modele trop eloigne:  ',I5,/,1X)  
415 FORMAT(/,' Statistiques supplementaires ',/,1X,28('-'))
416 FORMAT(/' Fin de l''impression des statistiques supplementaires',&
 & ' de CANCER ',/,1X,62('-'),//)  
417 FORMAT(2X,A32,A11,F8.3,A9,F8.3,A2,I5.5,A1)
419 FORMAT(1X,A,I4,A,F5.1,A,I4,A,F5.1,A,I4,A,I4,A,I4,A,F5.1,A)
420 FORMAT(22(1X,I5))
421 FORMAT(22(1X,F5.1))

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CANCER',1,ZHOOK_HANDLE)
END SUBROUTINE CANCER
