SUBROUTINE CANEVA( KQUOI , KANTY , KTASK,  PDISTR, PDELTA, PDELPI,&
 & KMXGSL, KMXGQ , PCORMIN,PSTRET,&
 & PLATA , PLONA , PRESSS, KNBPH, KNBPV,&
 & KNXI  , KNLXI , KNYJ  , KNLYJ , KNM  ,&
 & PSIGPS, PRESIA, PSIGA,  KNPR)  
!****-------------------------------------------i--------------------------
!****   CANEVA : ROUTINE DU COEUR DE L'ANALYSE 
!****   ------
!****  Auteurs :     YD 09/89, MCP 01/90, PhC 05/92, PM 01/93, VC 05/93
!****----------------------------------------------------------------------
!**  BUT : ANALYSE D'UNE LISTE DE VARIABLES SUR UNE LISTE DE POINTS SELON
!**  ---   LES OPTIONS DEMANDEES
!**
!**  ENTREES:
!**  -------
!**     KQUOI : -3 a 3 (>=0 GEO, 1 STA+ANA VERT, 2 STA VERT ANA, 3 STA+ANA)
!**     KANTY : MODE D'UTILISATION DE CANEVA
!**                 0 : ANALYSE DE CHAMPS EN POINTS DE GRILLE
!**                 1,2,...10 : REJET DES OBS DE LA CATEGORIE = KANTY
!**     KTASK : NUMERO DE LA TACHE DEVANT EXECUTER CANEVA
!**     PDISTR: DISTANCE MAXIMALE DE RECHERCHE DES OBSERVATIONS
!**             DE CHAQUE TYPE AUTOUR D'UN POINT
!**     PDELTA: DISTANCE VERTICALE DE SELECTION EN PRESSION (EN PA)
!**             DES OBSERVATIONS DE CHAQUE TYPE
!**     PDELPI: INTERDISTANCE MINIMALE ENTRE DEUX NIVEAUX SELEC-
!**             -TIONNES A PARTIR DU MEME MESSAGE (EN PA)
!**     KMXGSL: NOMBRE MAXIMUM D'OBSERVATIONS DE CHAQUE TYPE A
!**             SELECTIONNER DANS LA 1ERE PHASE DE RECHERCHE GEO.
!**             BESTIALE (CERCLE DE RAYON PDISTR)
!**     KMXGQ : NOMBRE MAXIMUM D'OBSERVATIONS DE CHAQUE TYPE A
!**             SELECTIONNER DANS CHAQUE SECTEUR DE RECHERCHE
!**             AUTOUR D'UN POINT (EN GENERAL, PAR QUADRANT)
!**             APRES APPLICATION DES CONTRAINTES D'INTERDISTANCE
!**     PCORMIN:CORRELATION MINIMALE QU'UN PREDICTEUR DOIT
!**             PRESENTER AVEC AU MOINS UN PREDICTANT POUR ETRE
!**             SELECTIONNE
!**     PSTRET: FONCTION DE STRETCHING POUR LES STATISTIQUES HORIZONTALES
!**     PLATA : LATITUDE DES POINTS A ANALYSER
!**     PLONA : LONGITUDE DES POINTS A ANALYSER
!**     PRESSS: PRESSION DES POINTS A ANALYSER
!**     KNBPH : NOMBRE DE POINTS SUR L'HORIZONTALE
!**     KNBPV : NOMBRE DE POINTS SUR LA VERTICALE
!**     KNXI  : NUMERO DES MODELES STATISTIQUES ASSOCIES AUX PREDICTANTS
!**     KNLXI : NOMBRE DE TYPES DE PREDICTANTS (LONGUEUR DE KNXI)
!**     KNXJ  : NUMERO DES MODELES STATISTIQUES ASSOCIES AUX PREDICTEURS
!**     KNLYJ : NOMBRE DE TYPES DE PREDICTEURS (LONGUEUR DE KNYJ)
!**     KNM   : NOMBRE DE PREDICTEURS A SELECTIONNER EN UN POINT
!**             D'ANALYSE POUR CHAQUE PREDICTANT INDIQUE DANS KNXI
!**     PSIGPS: ECART-TYPES D'ERREUR DE L'EBAUCHE
!**
!**  SORTIES: PSIGPS: ECARTS-TYPES D'ERREUR DE L'EBAUCHE UTILISES PAR
!**  -------          L'ANALYSE (COHERENTS AVEC LES MODELES STATISTIQUES)
!**           PRESIA: RESIDUS ANALYSES
!**           PSIGA : ECART-TYPES D'ERREUR D'ANALYSE
!**           KNPR  : NOMBRE DE PREDICTEURS UTILISES
!**
!**  SOUS-PROGRAMMES: CANADA
!**  ---------------
!**  DESCRIPTION:     CANEVA EFFECTUE LE REPERAGE DES VARIABLES QUE L'ON
!**  -----------      SOUHAITE ANALYSER PARMI LES PREDICTANTS POSSIBLES
!**                   ET LE REPERAGE DES VARIABLES QUE L'ON SOUHAITE
!**                   UTILISER PARMI LES PREDICTEURS POSSIBLES. IL PASSE
!**                   ENSUITE LA MAIN A CANADA QUI LANCERA LES DIFFERENTES
!**                   ETAPES DE L'ANALYSE EN FONCTION DES OPTIONS VOULUES.
!****-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NTASKS
USE QAPABL   , ONLY : JPNPM    ,JPNPM2   ,JPNIV
USE PARDIMO  , ONLY : JPNOTP
USE QAPAVU   , ONLY : JPNVX    ,JPNINV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNBPH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNBPV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLXI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLYJ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KQUOI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KANTY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTASK 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDISTR(JPNOTP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELTA(JPNOTP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELPI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXGSL(JPNOTP) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXGQ(JPNOTP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCORMIN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRET(2,KNBPH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATA(KNBPH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLONA(KNBPH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESSS(KNBPH,KNBPV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNXI(KNLXI) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNYJ(KNLYJ) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNM(KNLXI) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSIGPS(KNBPH,KNLXI,KNBPV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRESIA(KNBPH,KNLXI,KNBPV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSIGA(KNBPH,KNLXI,KNBPV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNPR(KNBPH,KNBPV) 
INTEGER(KIND=JPIM) :: INM(KNLXI)
LOGICAL :: LLFLAI(0:JPNVX), LLFLAJ(0:JPNINV)

INTEGER(KIND=JPIM) :: IM, IMOY, IMPRED, INBPV, IQUOI, ITROP, JITER, JJ, JK, JX

REAL(KIND=JPRB) :: ZDEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "canada.intfb.h"

!**------------------------------------------------------------------------
!**  - 1 -  VALIDATION DES OPTIONS DEMANDEES
!**         --------------------------------

IF (LHOOK) CALL DR_HOOK('CANEVA',0,ZHOOK_HANDLE)
IF ( IABS(KQUOI)  >  3.OR.KANTY  <  0.OR. KANTY  >  JPNOTP+10 ) THEN
  WRITE (NULOUT,*) KQUOI, KANTY
  CALL ABOR1 ('** Pb dans CANEVA option impossible **')
ELSEIF ( KTASK  >  NTASKS ) THEN
  WRITE (NULOUT,*) 'Pb dans CANEVA: KTASK = ',KTASK
  CALL ABOR1 ('** Pb dans CANEVA trop de taches **')
ELSEIF ( KNLXI  >  JPNVX ) THEN
  WRITE (NULOUT,*) 'Pb dans CANEVA: KNLXI = ', KNLXI
  CALL ABOR1 ('** Pb dans CANEVA trop de predictands **')
ENDIF

IF ( IABS(KQUOI)  ==  1 ) THEN
  INBPV = 1
ELSE
  INBPV = KNBPV
ENDIF
IF ( INBPV  >  JPNIV ) THEN
  WRITE (NULOUT,*) 'Pb dans CANEVA: INBPV > JPNIV: ',INBPV,JPNIV
  CALL ABOR1 ('* Pb dans CANEVA INBPV > JPNIV *')
ENDIF

IF ( KNBPV == 1.AND. IABS(KQUOI) >= 2 ) THEN
  IQUOI = SIGN(1,KQUOI)
ELSE
  IQUOI = KQUOI
ENDIF

! on verifie que les predictands sont bien ranges
DO JX = 1 , KNLXI-1
  IF ( KNXI(JX)  >  KNXI(JX+1) ) THEN
    WRITE (NULOUT,*) ' Un peu d''ordre SVP ! ', KNXI
    CALL ABOR1('De l''ordre avant toute chose')
  ENDIF
ENDDO

!**------------------------------------------------------------------------
!**  - 2 -  REPERAGES, LIMITATIONS, INITIALISATIONS
!**         ---------------------------------------

IF ( IQUOI  /=  0 ) THEN
! reperage des predictands
  DO JK = 1 , JPNVX
    LLFLAI(JK) = .FALSE.
  ENDDO
  DO JK = 1 , KNLXI
    LLFLAI(KNXI(JK)) = .TRUE.
  ENDDO
! reperage des predicteurs
  DO JK = 1 , JPNINV
    LLFLAJ(JK) = .FALSE.
  ENDDO
  DO JK = 1 , KNLYJ
    LLFLAJ(KNYJ(JK)) = .TRUE.
  ENDDO
! Afin d'eviter la selection d'un trop grand nombre de predicteurs pour la
! constitution de la matrice a inverser
! ON LIMITE LE NOMBRE DE PREDICTEURS LA OU ON EN A DEMANDE LE PLUS
! ceci est necessaire pour le controle de qualite ou on ignore a priori le
! nombre de niveaux, DANGER on peut ne retenir aucun predicteur !!!
! IMPRED : nombre total de predicteurs
  IMPRED = 0
  DO JX = 1 , KNLXI
    INM(JX) = KNM(JX)
    IMPRED = IMPRED + KNM(JX)
  ENDDO

! attention si 1.5 change il convient de le changer dans CANADA
  ZDEN = 1.0_JPRB
  IF ( IABS(IQUOI)  ==  2 ) THEN
    ZDEN = 1.0_JPRB / 1.5_JPRB
  ELSE
    ZDEN = 1.0_JPRB
  ENDIF
  IM = MIN( JPNPM / INBPV, INT(JPNPM2 * ZDEN) )

  IF ( IMPRED  >  IM ) THEN
    WRITE (NULOUT,*) 'CANEVA : demande de trop de predicteurs ',&
     & IMPRED, INBPV, KNLXI, IM, JPNPM  
    ITROP = IMPRED - IM
    IMOY  = IM / KNLXI
    IF (IMOY  <  0) CALL ABOR1(' Pb CANEVA : IMOY = 0 ')
    DO JITER = 1,ITROP
      DO JJ = KNLXI,1,-1
        IF (INM(JJ)  >  IMOY) THEN
          INM(JJ) = INM(JJ) - 1
          IMPRED = IMPRED - 1
          IF ( IMPRED  <=  IM ) GOTO 300
        ENDIF
      ENDDO
    ENDDO
  ENDIF
ENDIF

!**------------------------------------------------------------------------
!**  - 3 -  EXECUTION DE L'ANALYSE
!**         ----------------------

300 CONTINUE

IF (KNLYJ  /=  0.OR. IABS(IQUOI)  <=  1) THEN
  CALL CANADA (IQUOI,  KANTY,  KTASK,&
   & PDISTR, PDELTA, PDELPI,&
   & KMXGSL, KMXGQ,  PCORMIN,PSTRET,&
   & PLATA,  PLONA,  PRESSS, KNBPH, KNBPV,&
   & KNXI,   KNLXI,  LLFLAI, LLFLAJ, INM,&
   & PSIGPS, PRESIA, PSIGA,  KNPR)  
ELSE
  KNPR=0
  PSIGA=PSIGPS
  PRESIA=0
ENDIF

!**------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CANEVA',1,ZHOOK_HANDLE)
END SUBROUTINE CANEVA
