SUBROUTINE CAOHIS (LDACT,KREP)
!****--------------------------------------------------------------------------
!****   CAOHIS : OUVERTURE FICHIER HISTORIQUE DE L'ANALYSE 
!****   ------   
!****   Auteur :     F. TAILLEFER  10/95
!****--------------------------------------------------------------------------
!  BUT : OUVERTURE DU FICHIER HISTORIQUE DE L'ANALYSE (STAT et INCR HU2m)
!  ---   
!        ON TRAVAILLE SOIT SUR LE FICHIER ENTREE (LDACT=.FALSE.) DONT IL FAUT
!        TESTER LA VALIDITE, SOIT SUR LE FICHIER DE SORTIE (LDACT=.TRUE.)
!        QU'IL FAUT ALORS DEFINIR.

!  ROUTINE APPELANTE :    *CALIFE*  *CANARI*
!  -----------------
!  SOUS-PROGRAMME APPELE :  CHIEN et routines d'utilisation des fichiers LFI
!  ---------------------
!  ARGUMENT D'ENTREE :
!  -----------------
!     Explicite :  LDACT : option de traitement choisi
!                  KREP  : code reponse ouverture fichier
!                           (doit etre a 0 au debut)
!  ARGUMENT DE SORTIE :
!  ------------------
!     Explicite :  KREP  : code reponse ouverture fichier
!                           1 --> OK    2 --> pb fichier

!  MODIFICATIONS:
!  --------------
!     Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!      - LRPOLE removed from YOMCT0 and 'CALL CHIEN'.
!      - LLFPOL renamed into LLFICP.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!****--------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDGSAG   ,NDGENG   ,NDLON    ,NFLEVG   ,NSMAX
USE YOMLUN   , ONLY : NULOUT   ,NULASE   ,NULASS
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS   ,RTIMTR
USE YOMCT0   , ONLY : CNMEXP   ,NQUAD    ,LELAM
USE YOMGEM   , ONLY : NLOENG   ,NMENG    ,VALH     ,VBH      ,&
 & NSTTYP   ,NHTYP    ,RMUCEN   ,RLOCEN   ,RSTRET   ,VP00  
USE YOMOP    , ONLY : CNMCA
USE YOMCST   , ONLY : RDAY
USE YOMMP    , ONLY : MYPROC   ,NSTRIN   ,NSTROUT

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDACT 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KREP 
#include "fcttim.h"

INTEGER(KIND=JPIM) :: JPCYCL

PARAMETER(JPCYCL=6)
INTEGER(KIND=JPIM) :: IDATEF(11)
LOGICAL :: LLFICP, LLFILE
CHARACTER (LEN = 16) ::  CLNOMF, CLIDEN
REAL(KIND=JPRB) :: ZGAUSS(NDGSAG:NDGENG), ZEPS

INTEGER(KIND=JPIM) :: IFILE, INBARI, INBARP, INIMES, IREP, ISS, JD, IINF

REAL(KIND=JPRB) :: ZTI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cchien.intfb.h"
#include "chien.intfb.h"

!**----------------------------------------------------------------------------
!**   1.    Initialisations.
!**         ---------------

IF (LHOOK) CALL DR_HOOK('CAOHIS',0,ZHOOK_HANDLE)
KREP=1
INIMES=1
INBARP=20

CALL FANMSG(0)

!**----------------------------------------------------------------------------
!**   2.    Traitement du fichier en entree.
!**         -------------------------------

IF ( .NOT. LDACT ) THEN

  IF (MYPROC <= NSTRIN) THEN

    CLNOMF='ICMSH'//CNMEXP(1:4)//'STAT   '
    WRITE(NULOUT,'('' OUVERTURE FICHIER STATISTIQUE GUESS'')')

!*    2.1.  Ouverture du fichier 

    INQUIRE(FILE=CLNOMF,EXIST=LLFILE)
    IF ( .NOT. LLFILE ) THEN
      WRITE(NULOUT,*) '  FICHIER STATISTIQUES ASSIMILEES ABSENT'
      WRITE(NULOUT,*) '  --> UTILISATION STATISTIQUES TABULEES'
      KREP=2
      IF (LHOOK) CALL DR_HOOK('CAOHIS',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
    CALL FAITOU(IREP,NULASE,.TRUE.,CLNOMF,'UNKNOWN',.TRUE.,.TRUE.,&
     & INIMES,INBARP,INBARI,'CADRE STAT      ')  
    CALL LFIMST(IREP,NULASE,.FALSE.)

!*    2.2.  Test de la geometrie 

    IF ( .NOT.LELAM )  THEN
      ZEPS=1.0E-7_JPRB
      CALL CHIEN('CADRE STAT      ',NSTTYP,RMUCEN,RLOCEN,RSTRET,&
       & NSMAX,NDGLG,NDLON,NLOENG,NMENG,ZGAUSS,NHTYP,&
       & NFLEVG,VP00,VALH,VBH,NQUAD,0,NDGSAG,&
       & NDGENG,ZEPS,LLFICP,NULOUT)  
    ELSE
!* Learn Geometry of ALADIN domain From the Given Namelist
      IFILE=17
      IINF=0
      CALL CCHIEN('CADRE STAT      ',IFILE,IINF)
      LLFICP=.FALSE.
    ENDIF

!*    2.3.  Impression identificateur (modele origine)

    CALL FALSIF(IREP,NULASE,CLIDEN)
    WRITE(NULOUT,'(''   IDENTIFICATEUR FICHIER : '',A16)') CLIDEN

!*    2.4.  Verification date

    CALL FADIES(IREP,NULASE,IDATEF)
    WRITE(NULOUT,*)'   DATE FICHIER : ',IDATEF
    ISS=IDATEF(4)*3600+IDATEF(5)*60+IDATEF(7)*3600+JPCYCL*3600
    ZTI=RTIME(IDATEF(1),IDATEF(2),IDATEF(3),ISS,RDAY)
    IF (ZTI /= RTIMTR) THEN
      WRITE(NULOUT,'('' ATTENTION: FICHIER HISTORIQUE CONSTITUE'',&
       & '' A UNE DATE DIFFERENTE DE '',I2,&
       & '' HEURES AUPARAVANT !'')') JPCYCL  
    ENDIF

  ENDIF

!**----------------------------------------------------------------------------
!**   3.    Traitement du fichier de sortie.
!**         -------------------------------

ELSE

  IF (MYPROC <= NSTROUT) THEN

    WRITE(NULOUT,'(/,'' OUVERTURE FICHIER STATISTIQUE ANALYSE'')')
    CLNOMF='ICMSH'//CNMEXP(1:4)//'STA2   '
    CLIDEN='ARP             '
    IF ( LELAM ) CLIDEN='UNK             '
    CALL FAITOU(IREP,NULASS ,.TRUE.,CLNOMF,'NEW',.TRUE.,.TRUE.,1,&
     & INBARP,INBARI,CNMCA)  
    CALL LFIMST(IREP,NULASS ,.FALSE.)
    CALL FAUTIF(IREP,NULASS ,CLIDEN)
    IDATEF(1)=NCCAA(NINDAT)
    IDATEF(2)=NMM(NINDAT)
    IDATEF(3)=NDD(NINDAT)
    IDATEF(4)=NCTH(NSSSSS)
    IDATEF(5)=(NSSSSS-IDATEF(4)*3600)/60
    IDATEF(6)=1
    DO JD = 7 , 11
      IDATEF(JD)=0
    ENDDO
    CALL FANDAR(IREP,NULASS ,IDATEF)

  ENDIF

ENDIF

!**----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAOHIS',1,ZHOOK_HANDLE)
END SUBROUTINE CAOHIS
