SUBROUTINE CAPDGU( KX  , KY  , KZ  ,&
 & KXD , KYD , KZD , KXF , KYF , KZF ,&
 & KOUT , CDVAR , PC , LD )  
!****----------------------------------------------------------------
!****  DK CAPDGU : Calcul et  impression des  moyenne, ecart-type  et
!****     ------   extrema d'un champ
!****
!****  Auteur                : Christian Blondin           03/06/91
!****  Refonte generale      : Philippe Caille             02/01/92
!****  Derniere modification : Vincent Casse                  05/92
!****----------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMGEM   , ONLY : NLOEN    ,NSTAGP
USE YOMGC    , ONLY : GELAM    ,GELAT
USE YOMANCS  , ONLY : RDEGREES

!***
!*** But : Calcul et impression des moyenne, ecart-type et extrema d'un champ
!*** ---
!***
!*** Sequence d'appel     : CALL CAPDGU( KX  , KY  , KZ  ,
!*** ----------------                    KXD , KYD , KZD ,
!***                                     KXF , KYF , KZF ,
!***                                     KOUT , CDVAR , PC , LD )
!*** Arguments d'entree   :
!*** ------------------
!***   KX     : dimension suivant les pseudo-longitudes de PC
!***   KY     : dimension suivant les pseudo-latitudes de PC
!***   KZ     : dimension suivant la verticale de PC
!***   KXD    : indice de debut suivant KX de la sous partie d'etude de PC
!***   KYD    : idem suivant KY
!***   KZD    : idem suivant KZ
!***   KXF    : indice de fin suivant KX de la sous partie d'etude de PC
!***   KYF    : idem suivant KY
!***   KZF    : idem suivant KZ
!***   KOUT   : unite logique pour les impressions
!***   CDVAR  : nom du champ
!***   PC     : champ a traiter
!***   LD     : = .true.  calcul sur une partie et sur tout le champ
!***            = .false. calcul sur une partie du champ uniquement
!***
!*** Arguments de sortie  :
!*** -------------------
!*** Sous-programme amont : CA..T.
!*** --------------------
!*** Sous-programmes aval :
!*** --------------------
!*** COMMON utilises      : YOMGEM - YOMGC - YOMANCS
!*** ---------------
!*** Algorithme : l'appel a un sous-programme est evite au prix de 2 "GOTO"
!*** ----------
!****----------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KZ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KXD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KYD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KZD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KXF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KYF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KZF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOUT 
CHARACTER         ,INTENT(IN)    :: CDVAR(10)*1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC(KX,KY,KZ) 
LOGICAL                          :: LD ! Argument NOT used
INTEGER(KIND=JPIM) ::           IKXD2, IKYD2, IKZD2, IKXF2, IKYF2, IKZF2
LOGICAL ::           LLD2

INTEGER(KIND=JPIM) :: ILONW, INBPT, ITOT, ITOTU, IXU, IYU, JX, JY, JZ

REAL(KIND=JPRB) :: ZECTY, ZLAMAX, ZLAMIN, ZLOMAX, ZLOMIN, ZMAX, ZMEAN, ZMIN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!**---------------------------------------------------------------------
!**  - 1 -  TRAITEMENT DU CHAMP
!**         -------------------
!**

IF (LHOOK) CALL DR_HOOK('CAPDGU',0,ZHOOK_HANDLE)
WRITE (KOUT,201) CDVAR

IXU   = KXF - KXD + 1
IYU   = KYF - KYD + 1
ITOT  = KX  * KY
ITOTU = IXU * IYU

LLD2   = LD .OR. (ITOTU  ==  ITOT)

IF ( ITOTU > 0.AND. ITOTU < ITOT ) THEN
  IKZD2 = KZD
  IKZF2 = KZF
  IKYD2 = KYD
  IKYF2 = KYF
  IKXD2 = KXD
  IKXF2 = KXF
  WRITE (KOUT,202) 'partiel ', IXU, IYU
ELSE
  GOTO 998
ENDIF

999 CONTINUE
WRITE (KOUT,204)

DO JZ = IKZD2 , IKZF2
  ZMEAN  = 0.0_JPRB
  ZECTY  = 0.0_JPRB
  ZMAX   = -1.E+12_JPRB
  ZMIN   = +1.E+12_JPRB
  INBPT  = 0

  DO JY = IKYD2 , IKYF2
    DO JX = IKXD2 , MIN(IKXF2,NLOEN(JY))
      INBPT = INBPT + 1
      ZMEAN = ZMEAN + PC(JX,JY,JZ)
      ZECTY = ZECTY + PC(JX,JY,JZ)**2

      IF ( PC(JX,JY,JZ)  >  ZMAX ) THEN
        ZMAX   = PC(JX,JY,JZ)
        ILONW  = MIN( KXD , NLOEN(JY) )
        ZLAMAX = GELAT(ILONW+JX-1+NSTAGP(JY)) * RDEGREES
        ZLOMAX = GELAM(ILONW+JX-1+NSTAGP(JY)) * RDEGREES
      ENDIF

      IF ( PC(JX,JY,JZ)  <  ZMIN ) THEN
        ZMIN   = PC(JX,JY,JZ)
        ILONW  = MIN( KXD , NLOEN(JY) )
        ZLAMIN = GELAT(ILONW+JX-1+NSTAGP(JY)) * RDEGREES
        ZLOMIN = GELAM(ILONW+JX-1+NSTAGP(JY)) * RDEGREES
      ENDIF
    ENDDO
  ENDDO

  IF ( INBPT /= 0 ) THEN
    ZMEAN = ZMEAN / INBPT
    ZECTY = SQRT( MAX(ZECTY / INBPT - ZMEAN**2, 0.0_JPRB) )
  ENDIF

  WRITE (KOUT,205) JZ, ZMEAN, ZECTY, ZMIN, ZLAMIN, ZLOMIN,&
   & ZMAX, ZLAMAX, ZLOMAX  
ENDDO

WRITE (KOUT,206)

998 CONTINUE
IF ( LLD2 ) THEN
  LLD2  = .FALSE.
  IKZD2 = 1
  IKZF2 = KZ
  IKYD2 = 1
  IKYF2 = KY
  IKXD2 = 1
  IKXF2 = KX
  WRITE (KOUT,202) 'total   ', KX, KY
  GOTO 999
ENDIF

!**------------------------------------------------------------------
!**  - 2 -  FORMATS
!**         -------
!**

201 FORMAT(//,' Renseignements fournis par CAPDGU pour le champ ',&
 & 10A1,/,&
 & ' ------------------------------------------------',&
 & '----------',/)  
202 FORMAT(' Champ ',A8,'     KX : ',I4,'   KY : ',I4)
204 FORMAT(/,'  -----------------------------------------------',&
 & '-----------------------------------------------',/,&
 & ' | niv.     moyenne    ecart-type     minimum   ',&
 & '  lat.     lon.      maximum     lat.     lon.  |',/,&
 & '  ------------------------------------------------',&
 & '----------------------------------------------')  
205 FORMAT(' |  ',I2,'  | ',E10.4,' | ',E10.4,' | ',E10.4,' | ',&
 & F6.2,' | ',F6.2,' | ',E10.4,' | ',F6.2,' | ',F6.2,' |')  
206 FORMAT('  --------------------------------------------------',&
 & '--------------------------------------------')  

!**------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAPDGU',1,ZHOOK_HANDLE)
END SUBROUTINE CAPDGU
