SUBROUTINE CAPITO( KUNIT )
!****----------------------------------------------------------------
!****  CAPITO : Impressions d'informations sur les observations
!****  ------    (called by CAMELO)
!****----------------------------------------------------------------

!  Modifications :  - LG 03/00 rms for snow added
!  -------------    - FT 08/00 CMAFOC --> ODB
!                   - FT 01/01 suppression logiques sur observations
!        O.Spaniel  : 03-04-15 cleaning a same named entity from modules
!        M.Hamrud   : 03-10-01 CY28 Cleaning

!  Argument d'entree :
!  -----------------
!    KUNIT  : unite logique du fichier pour les impressions
!--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIMO  , ONLY : JPNOTP   ,JPXSOBT  ,JPXVAR   ,JPXAREA
USE QAPASS   , ONLY : JPNVS
USE YOMDIMO  , ONLY : NOBTOTG  ,NOBTOT   ,NMXSOBT  ,NMXAREA  ,&
 & NMXSET   ,NMXLEN   ,NOBTOV   ,NOBNTV   ,NOBSCA
USE YOMCST   , ONLY : RG
USE YOMCMA   , ONLY : NMXVAR   ,NMXOTP
USE YOMOBA   , ONLY : MOBDD1   ,MNBOTPG
USE YOMCMDDR , ONLY : NCD1OT   ,NCD1OD
USE YOMCOCTP , ONLY : NSYNOP   ,NAIREP   ,NSATOB   ,NDRIBU   ,NTEMP    ,&
 & NPILOT   ,NSATEM   ,NPAOB    ,NSCATT   ,NLIMB  
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMCT0   , ONLY : NPROC
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMTAG   , ONLY : MTAGCOST
USE MPL_MODULE
USE QAREF    , ONLY : REF_STAT
USE QAETEO   , ONLY : ECTERO   ,ETEOST   ,ETEOHU

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KUNIT
LOGICAL :: LLMPTO(JPNOTP)
CHARACTER :: CLADAT*11
INTEGER(KIND=JPIM) :: IBUFG(4,NPROC),IBUF(4)

INTEGER(KIND=JPIM) :: IAN, IHEURE, IJOUR, ILREC, IMINUT, IMOIS, &
 & ITAG, J, J0, J1, JB, JPROC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "qabitu.h"

!--------------------------------------------------------------------
!  - 1 -  MISE A DISPOSITION DES INFORMATIONS NECESSAIRES.
!         -----------------------------------------------

IF (LHOOK) CALL DR_HOOK('CAPITO',0,ZHOOK_HANDLE)

IBUF(1)=NOBTOT
IBUF(2)=NOBTOV
IBUF(3)=NOBNTV
IBUF(4)=NOBSCA
ITAG = MTAGCOST
IF (MYPROC == 1) THEN
  IBUFG(:,1)=IBUF(:)
  DO JPROC = 2,NPROC
    CALL MPL_RECV(IBUF(1:4),KTAG=ITAG,KOUNT=ILREC,CDSTRING='CAPITO :')  
    IF (ILREC /= 4) CALL ABOR1('CAPITO : RECEIVED MESSAGE LENGTH OF WRONG SIZE')  
    IBUFG(:,JPROC)=IBUF(:)
  ENDDO
ELSE
  CALL MPL_SEND(IBUF(1:4),KDEST=NPRCIDS(1),KTAG=ITAG,CDSTRING='CAPITO :')
ENDIF
    
IF (MYPROC /= 1) THEN
  IF (LHOOK) CALL DR_HOOK('CAPITO',1,ZHOOK_HANDLE)
  RETURN
ENDIF

!--------------------------------------------------------------------
!  - 2 -  INITIALISATIONS IMPRESSIONS.
!         ---------------------------

CLADAT  = '       --  '

LLMPTO(:) = .FALSE.

IF ( MNBOTPG(NSYNOP) > 0 ) LLMPTO(NSYNOP) = .TRUE.
IF ( MNBOTPG(NAIREP) > 0 ) LLMPTO(NAIREP) = .TRUE.
IF ( MNBOTPG(NSATOB) > 0 ) LLMPTO(NSATOB) = .TRUE.
IF ( MNBOTPG(NDRIBU) > 0 ) LLMPTO(NDRIBU) = .TRUE.
IF ( MNBOTPG(NTEMP)  > 0 ) LLMPTO(NTEMP)  = .TRUE.
IF ( MNBOTPG(NPILOT) > 0 ) LLMPTO(NPILOT) = .TRUE.
IF ( MNBOTPG(NSATEM) > 0 ) LLMPTO(NSATEM) = .TRUE.
IF ( MNBOTPG(NPAOB)  > 0 ) LLMPTO(NPAOB)  = .TRUE.
IF ( MNBOTPG(NSCATT) > 0 ) LLMPTO(NSCATT) = .TRUE.
IF ( MNBOTPG(NLIMB)  > 0 ) LLMPTO(NLIMB)  = .TRUE.

!--------------------------------------------------------------------
!  - 3 -  IMPRESSIONS CONCERNANT LE CHARGEMENT DES OBS.
!         --------------------------------------------

IAN    = NINDAT / 10000
IMOIS  = MOD( NINDAT/100 , 100 )
IJOUR  = MOD( NINDAT , 100 )
IHEURE = NSSSSS / 3600
IMINUT = ( NSSSSS - IHEURE*3600 ) / 60

WRITE (CLADAT(2:9),'(I8)') MOBDD1(NCD1OD)
WRITE (CLADAT(10:11),'(I2)') MOBDD1(NCD1OT)/10000
IF ( CLADAT(10:10) == ' ' ) CLADAT(10:10) = '0'

WRITE (KUNIT,1101) IJOUR,IMOIS,IAN,IHEURE,IMINUT
WRITE (KUNIT,1103) CLADAT
WRITE (KUNIT,1108)
WRITE (KUNIT,1107) MNBOTPG

WRITE (KUNIT,1110) NMXOTP,JPNOTP
WRITE (KUNIT,1111) NMXSOBT,JPXSOBT,NMXVAR,JPXVAR,NMXAREA,JPXAREA
WRITE (KUNIT,1112) NMXSET,NMXLEN

WRITE (KUNIT,1113) NOBTOTG
WRITE (KUNIT,*) 'Nombre effectif d''observations par processeur             ',&
 & '          NOBTOT     ',(IBUFG(1,J),J=1,NPROC)  
WRITE (KUNIT,*) 'Nombre effectif de TOVS par processeur                     ',&
 & '         NOBTOV     ',(IBUFG(2,J),J=1,NPROC)  
WRITE (KUNIT,*) 'Nombre effectif de SCATT par processeur                    ',&
 & '         NOBSCA     ',(IBUFG(4,J),J=1,NPROC)  
WRITE (KUNIT,*) 'Nombre effectif d''observations non TOVS/SCATT par proc.   ',&
 & '          NOBNTV     ',(IBUFG(3,J),J=1,NPROC)  

!--------------------------------------------------------------------
!  - 4 -  IMPRESSION DES ECART-TYPES D'ERREUR D'OBSERVATION.
!         -------------------------------------------------

WRITE (KUNIT,1200)

!  - 4.1 -  pour la categorie NSYNOP

IF ( LLMPTO(NSYNOP) ) THEN
  WRITE (KUNIT,1211)
  WRITE (KUNIT,1201)
  WRITE (KUNIT,1212)ECTERO(NSYNOP,1,1,1)/RG,&
   & ECTERO(NSYNOP,1,2,1),ECTERO(NSYNOP,1,3,1),&
   & ECTERO(NSYNOP,1,4,1),   &
   & ECTERO(NSYNOP,1,6,1)  
  WRITE (KUNIT,1213)ECTERO(NSYNOP,2,1,1)/RG,&
   & ECTERO(NSYNOP,2,2,1),ECTERO(NSYNOP,2,3,1),&
   & ECTERO(NSYNOP,2,4,1),ECTERO(NSYNOP,2,5,1)  
ENDIF

!  - 4.2 -  pour la categorie NAIREP

IF ( LLMPTO(NAIREP) ) THEN
  WRITE (KUNIT,1221)
  WRITE (KUNIT,1222) (ECTERO(NAIREP,1,3,J1),J1=1,JPNVS)
  WRITE (KUNIT,1223) (ECTERO(NAIREP,1,2,J1),J1=1,JPNVS)
ENDIF

!  - 4.3 -  pour la categorie NSATOB

IF ( LLMPTO(NSATOB) ) THEN
  WRITE (KUNIT,1231)
  WRITE (KUNIT,1232) (ECTERO(NSATOB,1,3,J1),J1=1,JPNVS)
  WRITE (KUNIT,1233) (ECTERO(NSATOB,2,3,J1),J1=1,JPNVS)
  WRITE (KUNIT,1234) (ECTERO(NSATOB,3,3,J1),J1=1,JPNVS)
  WRITE (KUNIT,1235) (ECTERO(NSATOB,4,3,J1),J1=1,JPNVS)
  WRITE (KUNIT,1236) (ECTERO(NSATOB,5,3,J1),J1=1,JPNVS)
ENDIF

!  - 4.4 -  pour la categorie NDRIBU

IF ( LLMPTO(NDRIBU) ) THEN
  WRITE (KUNIT,1241)
  WRITE (KUNIT,1242)ECTERO(NDRIBU,1,1,1)/RG,ECTERO(NDRIBU,1,2,1),&
   & ECTERO(NDRIBU,1,3,1),   ECTERO(NDRIBU,1,5,1)  
ENDIF

!  - 4.5 -  pour la categorie NTEMP

IF ( LLMPTO(NTEMP) ) THEN
  WRITE (KUNIT,1251)
  WRITE (KUNIT,1252) (ECTERO(NTEMP,1,1,J1)/RG,J1=1,JPNVS)
  WRITE (KUNIT,1253) (ECTERO(NTEMP,1,2,J1),J1=1,JPNVS)
  WRITE (KUNIT,1254) (ECTERO(NTEMP,1,3,J1),J1=1,JPNVS)
  WRITE (KUNIT,1255) (ECTERO(NTEMP,1,4,J1),J1=1,10)
ENDIF

!  - 4.6 -  pour la categorie NPILOT

IF ( LLMPTO(NPILOT) ) THEN
  WRITE (KUNIT,1261)
  WRITE (KUNIT,1262) (ECTERO(NPILOT,1,3,J1),J1=1,JPNVS)
ENDIF

!  - 4.7 -  pour la categorie NSATEM

IF ( LLMPTO(NSATEM) ) THEN
  WRITE(KUNIT,1271)
  WRITE(KUNIT,'(4X,19I4)') (NINT(REF_STAT(J,1)*.01_JPRB),J=JPNVS,2,-1)
  J0 = JPNVS*(JPNVS-1)/2 + 1
  DO JB = JPNVS-1 , 1 , -1
    WRITE (KUNIT,'(I5,19(F4.0))') NINT(REF_STAT(JB,1)*.01_JPRB),&
     & (ETEOST(J0-J1)/RG,J1=1,JPNVS-JB)  
    J0 = J0 - JPNVS + JB
  ENDDO
  WRITE (KUNIT,1274) (ETEOHU(J1),J1=1,3)
ENDIF

!  - 4.8 -  pour la categorie NPAOB

!  - 4.9 -  pour la categorie NSCATT

!  - 4.10 - pour la categorie NLIMB

!--------------------------------------------------------------------
!  - 10 -  FORMATS.
!          -------

1101 FORMAT(/,6X,67('*'),/,6X,'*   LECTURE DU FICHIER D''OBSERVATI',&
 & 'ONS DU ',I2,'/',I2,'/',I4,' A ',I2,'H',I2,' UTC   *',/,&
 & 6X,67('*'),/)  
1103 FORMAT(' Date fichier  : ',2X,A11)
1108 FORMAT(/,24X,'SY-SH',5X,'AIREP',5X,'SATOB',5X,'DRIBU',5X,' TEMP',&
 & 5X,'PILOT',5X,'SATEM',5X,'PAOB ',5X,'SCAT.',5X,' RAD.',/)  
1107 FORMAT(' Nombre d''obs. lues ',10(1X,I9))
1110 FORMAT(//,' Nombre maximum de types d''observations          ',&
 & '                    NMXOTP   ',I10,'       JPNOTP   ',I10)  
1111 FORMAT(' Nombre maximum de sous-types d''observations        ',&
 & '                 NMXSOBT  ',I10,'       JPXSOBT  ',I10,/,&
 & ' Nombre maximum de variables                                ',&
 & '         NMXVAR   ',I10,'       JPXVAR   ',I10,/,&
 & ' Nombre maximum de zones                                    ',&
 & '         NMXAREA  ',I10,'       JPXAREA  ',I10)  
1112 FORMAT(&
 & ' Nombre maximum de paquets d''observations                  ',&
 & '          NMXSET   ',I10,/,&
 & ' Longueur maximale d''un paquet d''observations             ',&
 & '           NMXLEN   ',I10)  
1113 FORMAT(/,&
 & ' Nombre global d''observations                              ',&
 & '          NOBTOTG  ',I10,/)  

1200 FORMAT(//,38X,53('*'),/,38X,'*   ECARTS-TYPES D''ERREURS D''OBS',&
 & 'ERVATION UTILISES   *',/,38X,53('*'),//,'  Unites :   ',&
 & 'niveaux pression        en hPa',/,'  ------     geopot',&
 & 'entiel            en m',/,'             temperature   ',&
 & '          en degres',/,'             vent             ',&
 & '       en m/s',/,'             humidite relative      ',&
 & ' de 0 a 1',/,'             epaisseur de couche     en m',&
 & /,'             SST                     en degres',&
 & /,'             quantite de neige       en kg/m**2')  

1211 FORMAT(/'  Messages SYNOP, SHIP, etc...   (NSYNOP)',/,&
 & '  ----------------------------')  
1201 FORMAT(30X,'Z        T       u/v      Hu       SST      SN')
1212 FORMAT('           SYNOP       ',4F9.2,'         ',F9.2)
1213 FORMAT('           SHIP        ',5F9.2)

1221 FORMAT(/,'  Messages AIREP   (NAIREP)',/,'  --------------',/,&
 & '  pression en hPa   1000   950   900   850   800   700',&
 & '   600   500   400   300   250   200   150   100',&
 & '    70    50    30    20    10')  
1222 FORMAT('           u/v    ',19F6.2)
1223 FORMAT('            T     ',19F6.2)

1231 FORMAT(/,'  Messages SATOB   (NSATOB)   u/v',/,'  ------------',/,&
 & '  pression en hPa   1000   950   900   850   800   700',&
 & '   600   500   400   300   250   200   150   100',&
 & '    70    50    30    20    10')  
1232 FORMAT('     METEOSAT     ',19F6.2)
1233 FORMAT('       GOES       ',19F6.2)
1234 FORMAT('  sat. japonais   ',19F6.2)
1235 FORMAT('   sat. indien    ',19F6.2)
1236 FORMAT('   autres sat.    ',19F6.2)

1241 FORMAT(/,'  Messages DRIBU  (NDRIBU)',/,'  --------------')
1242 FORMAT(30X,'Z        T       u/v      SST',/,&
 & '           DRIBU       ',4F9.2)  

1251 FORMAT(/,'  Messages TEMP   (NTEMP)',/,'  -------------',/,&
 & '  pression en hPa   1000   950   900   850   800   700',&
 & '   600   500   400   300   250   200   150   100',&
 & '    70    50    30    20    10')  
1252 FORMAT('       Z          ',19F6.2)
1253 FORMAT('       T          ',19F6.2)
1254 FORMAT('      u/v         ',19F6.2)
1255 FORMAT('       Hu         ',19F6.2)
1261 FORMAT(/,'  Messages PILOT   (NPILOT)',/,'  --------------',/,&
 & '  pression en hPa   1000   950   900   850   800   700',&
 & '   600   500   400   300   250   200   150   100',&
 & '    70    50    30    20    10')  
1262 FORMAT('      u/v         ',19F6.2)
1271 FORMAT(/,'  Messages SATEM  (NSATEM)',/,'  -------------- ')
1274 FORMAT(/,'      humidite    1000/700 700/500 500/300 ',&
 & /,'                 ',3(F8.2,2X))  

!--------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAPITO',1,ZHOOK_HANDLE)
END SUBROUTINE CAPITO
