SUBROUTINE CARATK
!-------------------------------------------------------------------------
!  CARATK : Initialisation de variables specifiques au controle
!  ------   de la qualite des observations
!-------------------------------------------------------------------------
!  But : initialisations des  parametres de recherche geographique,
!  ---   des differents types  de predicteurs et  predictands  pour
!        chaque type d'observations, etc...
!        la liste des parametres initialises est presentee dans les
!        commentaires du comdeck QADORE

!  Sequence d'appel     : CALL CARATK
!  ----------------
!  Arguments de sortie  : tout se passe dans le COMMON QADORE
!  -------------------
!  Sous-programme amont : CAVODK
!  --------------------
!  COMMON utilises      : PARDIMO - QAPAVU - QADORE - YOMLUN
!-------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIMO  , ONLY : JPNOTP
USE QAPAVU   , ONLY : JPNVX    ,JPNVY
USE YOMLUN   , ONLY : NULOUT
USE QADORE   , ONLY : MINM     ,NMXGSL   ,NMXGQ    ,QDISTR   ,QDISTV

!  distance de recherche autour d'un point (en metres)

IMPLICIT NONE
REAL(KIND=JPRB)      , DIMENSION( JPNOTP * JPNOTP ) ::  ZDISTR=(/&
 & 250000._JPRB,      0.0_JPRB,      0.0_JPRB,  250000._JPRB,  500000._JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,  250000._JPRB,  250000._JPRB,      0.0_JPRB,  500000._JPRB,&
 & 500000._JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,  250000._JPRB,  250000._JPRB,      0.0_JPRB,  500000._JPRB,&
 & 500000._JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 & 250000._JPRB,      0.0_JPRB,      0.0_JPRB,  250000._JPRB,  250000._JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 & 250000._JPRB,  250000._JPRB,  250000._JPRB,  250000._JPRB,  500000._JPRB,&
 & 500000._JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,  250000._JPRB,  250000._JPRB,      0.0_JPRB,  500000._JPRB,&
 & 500000._JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,  800000._JPRB,&
 &     0.0_JPRB,  750000._JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&  
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&  
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&  
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&  
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&  
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,&  
 &     0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB,      0.0_JPRB /)

!  distance verticale maximale de recherche autour d'un niveau (en hPa) 
!  si byniv
!  Actuellement, ZDISTV est fixe a 100 hPa de facon arbitraire car byvert !

REAL(KIND=JPRB)      , DIMENSION( JPNOTP * JPNOTP ) ::  ZDISTV=(/&
 & 10000._JPRB,      0.0_JPRB,      0.0_JPRB,  10000._JPRB,  10000._JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,   10000._JPRB,   10000._JPRB,     0.0_JPRB,  10000._JPRB,&
 & 10000._JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,   10000._JPRB,   10000._JPRB,     0.0_JPRB,  10000._JPRB,&
 & 10000._JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 & 10000._JPRB,      0.0_JPRB,      0.0_JPRB,  10000._JPRB,  10000._JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 & 10000._JPRB,   10000._JPRB,   10000._JPRB,  10000._JPRB,  10000._JPRB,&
 & 10000._JPRB,   10000._JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,   10000._JPRB,   10000._JPRB,     0.0_JPRB,  10000._JPRB,&
 & 10000._JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,  10000._JPRB,&
 &    0.0_JPRB,   10000._JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB,&  
 &    0.0_JPRB,      0.0_JPRB,      0.0_JPRB,     0.0_JPRB,     0.0_JPRB /)

!  nombre maximum d'observations de chaque type a selectionner dans
!  la premiere phase de recherche geographique

INTEGER(KIND=JPIM)   , DIMENSION( JPNOTP * JPNOTP ) ::  IMXGSL=(/&
 & 700,    0,    0,  100,  200,    0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0, 1000,  500,    0,  200,  100,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0, 1000,  500,    0,  200,  100,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 & 700,    0,    0,  100,  200,    0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 & 700, 1000,  500,  100,  200,  100,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0, 1000,  500,    0,  200,  100,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,  200,    0,  500,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&  
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&  
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&  
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&  
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&  
 &   0,    0,    0,    0,    0,&
 &   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,&  
 &   0,    0,    0,    0,    0 /)

!  nombre maximum d'observations de chaque type a selectionner dans
!  chaque secteur de recherche autour d'un point

INTEGER(KIND=JPIM)   , DIMENSION( JPNOTP * JPNOTP ) ::  IMXGQ=(/&
 & 4,  0,  0,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  4,  4,  0,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  4,  4,  0,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 4,  0,  0,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 4,  4,  4,  4,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  4,  4,  0,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  4,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
 & 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 /)  

!  nombre de types de predictands retenus par type d'obs.

INTEGER(KIND=JPIM)   , DIMENSION( JPNVX * JPNOTP ) ::  IINM=(/&
 & 10, 0, 0,10,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! phi
 &  0,20, 0,20,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! T
 &  0, 6, 6, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! U
 &  0, 6, 6, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! V
 &  0, 0, 0, 0,20, 0,20, 0, 0, 0, 0, 0, 0, 0, 0,    &! E
 &  0, 0, 0, 0,20, 0,20, 0, 0, 0, 0, 0, 0, 0, 0,    &! HU
 & 20, 0, 0,20,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! SST
 & 20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! SN
 &  0, 0, 0, 0,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! Q
 & 20, 0, 0,20,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! T2m
 & 20, 0, 0,20,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! H2m
 & 20, 0, 0,20,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    &! U1m
 & 20, 0, 0,20,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0  /)  ! V1m  

INTEGER(KIND=JPIM) :: IND, IND2, J1, J2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!-------------------------------------------------------------------------
!  - 1 -  Test de coherence entre les data et jpnotp, jpnvx et jpnvy
!         ----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CARATK',0,ZHOOK_HANDLE)
IF ( JPNOTP /= 15.OR. JPNVX /= 13.OR. JPNVY /= 13 ) THEN
  WRITE (NULOUT,101) JPNOTP,JPNVX,JPNVY
  CALL ABOR1(' CARATK / jpnotp - jpnvx - jpnvy ')
ENDIF

101 FORMAT(' *** probleme PARAMETER dans CARATK *** ',/,&
 & ' on attend JPNOTP = 15 et JPNOTP vaut ',I2,/,&
 & ' on attend JPNVX  = 13 et JPNVX  vaut ',I2,/,&
 & ' on attend JPNVY  = 13 et JPNVY  vaut ',I2)  

!-------------------------------------------------------------------------
!  - 2 -  Initialisation du common qadore
!         -------------------------------

DO J2 = 1 , JPNOTP
  DO J1 = 1 , JPNVX
    IND = (J1-1)*JPNOTP + J2
    MINM(J1,J2) = IINM(IND)
  ENDDO
ENDDO

DO J2 = 1 , JPNOTP
  IND2 = (J2-1)*JPNOTP
  DO J1 = 1 , JPNOTP
    IND = IND2 + J1
    NMXGSL(J1,J2) = IMXGSL(IND)
    NMXGQ (J1,J2) = IMXGQ (IND)
    QDISTR(J1,J2) = ZDISTR(IND)
    QDISTV(J1,J2) = LOG(ZDISTV(IND)+1.0_JPRB)
  ENDDO
ENDDO

!--------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CARATK',1,ZHOOK_HANDLE)
END SUBROUTINE CARATK
