SUBROUTINE CASELA ( KTASK, LDFLAI,KNLX,  KNM,   PCORMIN,&
 & PLAT,  PLON,  PRESS, PSIG,  KNBPV,  KIV,&
 & KASELS,KOSELS,PDSELS,KNSELS,&
 & PDXMB, KNIN,  KNOUT)  
!****----------------------------------------------------------------------
!****    CASELA : Selection statistique
!****    ------
!****   Auteurs : MCP 08/90, VC 03/92
!****-------------------------------------------------------------------
!*** But : Selection statistique pour un ensemble de predictants des
!*** ---   "meilleurs" predicteurs autour d'un point pour un ou plusieurs
!***       niveaux pression et constitution du second membre du systeme lineaire
!***
!*** Arguments d'entree :
!*** ------------------
!***   KTASK  : numero de la tache executant CASELA
!***   LDFLAI : tableau indiquant les variables de la table NVX a analyser
!***   KNLX   : nombre de mots de LDFLAI egaux a .TRUE.
!***   KNM    : tableau du nombre maximum de predicteurs par predictants
!***   PCORMIN: correlation minimale pour retenir un predicteur
!***   PLAT   : latitude du point d'analyse en radians
!***   PLON   : longitude du point d'analyse en radians
!***   PRESS  : tableau des pressions de chaque niveau en Pascals
!***   PSIG   : tableau des ecart-types d'erreur du guess pour chaque predictant
!***   KNBPV  : nombre de niveau pression a traiter
!***   KASELS : tableau des adresses des predicteurs
!***   KOSELS : tableau des adresses des observations contenant les predicteurs
!***   PDSELS : tableau des distances du point d'analyse aux predicteurs
!***   KNSELS : tableau du nombre de predicteurs par type
!***   KNIN   : > somme des KNSELS; sert a dimensionner ZCOVAR et ZCOREL
!***
!*** Arguments de sortie :
!*** -------------------
!***   KASELS, KOSELS, PDSELS, KNSELS reordonnes
!***   KNOUT : somme des KNSELS en sortie, nombre de predicteurs retenus
!***           dimension utile de PDXMB
!***   PDXMB : matrice des "covariances normees" entre les erreurs des
!***           predicteurs et celles des predictants
!***           C'EST LE 2eme MEMBRE DU SYSTEME LINEAIRE
!***
!*** Sous-programme amont : CANADA
!*** --------------------
!***
!*** Sous-programmes aval : CACOVA - CAMERA
!*** --------------------
!***
!*** COMMON utilises : QAPABL  dimensions des tableaux du coeur de l'analyse
!*** ---------------   QAPAVU  dimensions des tables NVX et NVY
!***
!*** Algorithme
!*** ----------
!***   CASELA sert a gerer l'interface entre CANADA et CACOVA/CAMERA, en effet
!***   tant que la selection statistique n'a pas ete faite (CAMERA), le nombre
!***   de predicteurs est limite pour chacun des JPNVY (common QAPAVU) types a
!***   JPKZMA (common QAPABL), alors que la taille maximale du systeme lineaire
!***   est somme des KMN() * KNBPV < JPNPM (common QAPABL).
!***   Ceci fait que le tableau PDXMB est dimensionne (JPNPM,KNLX,KNBPV*2),alors
!***   que les tableaux locaux ZCOVAR et ZCOREL sont declares (KNIN,KNLX,KNBPV).
!***--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPABL   , ONLY : JPNPM    ,JPKZMA
USE QAPAVU   , ONLY : JPNVX    ,JPNVY

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNLX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNBPV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTASK 
LOGICAL           ,INTENT(IN)    :: LDFLAI(0:JPNVX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNM(KNLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCORMIN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESS(KNBPV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIG(JPNVX,KNBPV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIV 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KASELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOSELS(JPNVY,JPKZMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDSELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNSELS(JPNVY) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDXMB(JPNPM,KNLX,KNBPV*2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIN 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNOUT 
REAL(KIND=JPRB) ::    ZCOVAR(KNIN,KNLX,KNBPV), ZCOREL(KNIN,KNLX,KNBPV)

INTEGER(KIND=JPIM) :: JV, JX, JY
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cacova.intfb.h"
#include "camera.intfb.h"

!**---------------------------------------------------------------------
!**  - 1 - CALCUL DES COVARIANCES/CORRELATIONS ENTRE LES ERREURS DES
!**        PREDICTEURS PRESELECTIONNES ET CELLES DES PREDICTANTS.
!**        ---------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CASELA',0,ZHOOK_HANDLE)
CALL CACOVA( KTASK  , LDFLAI , KNLX   , PLAT   , PLON   , PRESS  ,&
 & PSIG   , KNBPV  , KIV    , KASELS , KOSELS , PDSELS ,&
 & KNSELS , KNIN   , ZCOVAR , ZCOREL)  

!**---------------------------------------------------------------------
!**  - 2 - SELECTION DES "MEILLEURS" PREDICTEURS.
!**        -------------------------------------

CALL CAMERA ( KASELS, KOSELS, PDSELS, KNSELS, ZCOVAR, ZCOREL,&
 & PCORMIN, KNM,   KNIN,   KNLX,   KNBPV,  KNOUT)  

!**---------------------------------------------------------------------
!**  - 3 - RECUPERATION DU SECOND MEMBRE DU SYSTEME LINEAIRE
!**        ET DES CORRELATIONS.
!**        -------------------------------------------------

DO JV = 1 , KNBPV
  DO JX = 1 , KNLX
    DO JY = 1 , KNOUT
      PDXMB(JY,JX,JV) = ZCOVAR(JY,JX,JV)
      PDXMB(JY,JX,JV+KNBPV) = ZCOREL(JY,JX,JV)
    ENDDO
  ENDDO
ENDDO

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASELA',1,ZHOOK_HANDLE)
END SUBROUTINE CASELA
