SUBROUTINE CASGQA (KMXGQ, KMXGSL, PLAT, PLON, KTYPO, KH, KTSK)
!****-------------------------------------------------------------------
!****    CASGQA : SELECTION D'AU PLUS KMXGQ OBSERVATIONS
!****    ------   PAR QUADRANT (OU SECTEUR)
!****
!****    Auteur : MCP 04/90; VC 08/92
!****-------------------------------------------------------------------

!  ENTREE:  KMXGQ : nb. max. d'obs. par secteurs
!  ------   KMXGSL: dimension de KASEL, KQSEL, PDSEL
!           PLAT  : latitude en radian
!           PLON  : longitude en radian
!           KTYPO : type d'obs traite
!           KH    : position horizontale du point traite
!           KTSK  : numero de la tache

!  SORTIE:  implicite dans common QANADA
!  ------     NIASELG ] -|
!             RZDSELG ] -| CORRESPONDANT AUX OBSERVATIONS SELECTIONNEES
!             NINSELG : EST ALORS LE NOMBRE D'OBSERVATION CONSERVEES

!  COMMON UTILISES: YOMOBA - YOMOAHDR - QANADA
!  ---------------

!  SOUS-PROGRAMMES: aucun
!  ---------------

!  DESCRIPTION:     ON TRIE LES OBSERVATIONS PAR QUADRANT. QUAND IL Y
!  -----------      A TROP D'OBSERVATIONS DANS UN QUADRANT, ON GARDE
!                   LES KMXGQ PLUS PROCHES DU POINT CENTRAL DE RECHERCHE

!    DETERMINATION DE QUADRANT
!    =========================                        ^    .P2
!                                                     !   .
!    LE NUMERO DE QUADRANT NE DEPEND           2      !  .   1
!    QUE DU SIGNE DE SIN(BETA1) ET                    ! .
!    DE COS(BETA1) DEFINIS DANS QADITS.  ------------ P1------------> Est
!    DE CE FAIT ON UTILISE LES FONCTIONS              !
!    DONNANT LE NUMERATEUR SEUL POUR           3      !      4
!    CES CALCULS.                                     !

!  MODIFICATIONS:   07/96 F. Taillefer : suppression NIQSELG
!  -------------    11/00 F. Taillefer : CMAFOC --> ODB
!****------------------------------------------------------------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPCAD   , ONLY : JPQUAD
USE QANADA   , ONLY : NINSELG  ,NIASELG  ,RZDSELG
USE YOMCST   , ONLY : RA
USE YOMDB

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXGQ 
INTEGER(KIND=JPIM)               :: KMXGSL ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTSK 
INTEGER(KIND=JPIM), EXTERNAL :: ISMAX

INTEGER(KIND=JPIM) :: IASEL(KMXGQ,JPQUAD), INSEL(JPQUAD), IQSEL(KMXGSL)
REAL(KIND=JPRB) ::    ZDSEL(KMXGQ,JPQUAD)

INTEGER(KIND=JPIM) :: IAD, IFTQA1, IFTQA2, II, IDEC, IQ, JTYPO, JOB, JQ

REAL(KIND=JPRB) :: ZLA1, ZLA2, ZLO1, ZLO2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "qadits.h"
#include "openmp_obs.h"

!**---------------------------------------------------------------------
!**  - 1 - INITIALISATION PREALABLE.
!**        ------------------------

IF (LHOOK) CALL DR_HOOK('CASGQA',0,ZHOOK_HANDLE)
INSEL = 0

IDEC = 0
DO JTYPO = 1 , KTYPO-1
  IDEC = IDEC + NINSELG(JTYPO,KH,KTSK)
ENDDO

!**---------------------------------------------------------------------
!**  - 2 - TRI PAR QUADRANT ET COMPTAGE AVEC LIMITATION EVENTUELLE.
!**        -------------------------------------------------------

DO JOB = 1 , NINSELG(KTYPO,KH,KTSK)
  IAD = NIASELG(IDEC+JOB,KH,KTSK)
  ZLA1 = PLAT
  ZLO1 = PLON
  ZLA2 = ROBHDR(IAD,MDBLAT)
  ZLO2 = ROBHDR(IAD,MDBLON)
  IF (CASI1N(ZLA1,ZLA2,COS(ZLO2-ZLO1)) >= 0.0_JPRB) THEN
    IFTQA1=1
    IFTQA2=2
  ELSE
    IFTQA1=4
    IFTQA2=3
  ENDIF
  IF (CACON (ZLA2,SIN(ZLO2-ZLO1)) >= 0.0_JPRB) THEN
    IQSEL(JOB)=IFTQA1
  ELSE
    IQSEL(JOB)=IFTQA2
  ENDIF
ENDDO

DO JOB = 1 , NINSELG(KTYPO,KH,KTSK)
  IQ = IQSEL(JOB)
  IF (INSEL(IQ) < KMXGQ) THEN
    INSEL(IQ)           = INSEL(IQ) + 1
    IASEL(INSEL(IQ),IQ) = NIASELG(IDEC+JOB,KH,KTSK)
    ZDSEL(INSEL(IQ),IQ) = RZDSELG(IDEC+JOB,KH,KTSK)
  ELSE
    II = ISMAX(INSEL(IQ), ZDSEL(1,IQ), 1)
    IF (RZDSELG(IDEC+JOB,KH,KTSK) < ZDSEL(II,IQ)) THEN
      IASEL(II,IQ) = NIASELG(IDEC+JOB,KH,KTSK)
      ZDSEL(II,IQ) = RZDSELG(IDEC+JOB,KH,KTSK)
    ENDIF
  ENDIF
ENDDO

!**---------------------------------------------------------------------
!**  - 3 - RANGEMENT POUR LA SORTIE.
!**        ------------------------

NINSELG(KTYPO,KH,KTSK) = 0
DO JQ  = 1 , JPQUAD
  DO JOB = 1 , INSEL(JQ)
    NINSELG(KTYPO,KH,KTSK) = NINSELG(KTYPO,KH,KTSK) + 1
    NIASELG(IDEC+NINSELG(KTYPO,KH,KTSK),KH,KTSK) = IASEL(JOB,JQ)
    RZDSELG(IDEC+NINSELG(KTYPO,KH,KTSK),KH,KTSK) = ZDSEL(JOB,JQ)
  ENDDO
ENDDO

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASGQA',1,ZHOOK_HANDLE)
END SUBROUTINE CASGQA
