SUBROUTINE CASGRA (KTYPA,KTYPO,KH,KTSK,PLAT,PLON,PDISTR,KMXGSL)
!-------------------------------------------------------------------
!    CASGRA : Extraction des observations dans le rayon PDISTR
!    ------   (sur la sphere reelle) autour du point (PLAT,PLON)

!    auteur : M.C. Pierrard 03/90
!--------------------------------------------------------------------
!   CACHBU indique les boites concernees
!   Dans chaque boite on prend les adresses des obs. du type KTYPO
!   comprises dans le cercle de rayon PDISTR sur la sphere geographique reelle

!  Arguments d'entree   :
!  ------------------
!    KTYPO  : type des observations a selectionner
!    KH     : position horizontale du point traite
!    KTSK   : numero de tache
!    PLAT   : latitude du point en radians
!    PLON   : longitude du point en radians
!    PDISTR : distance maximale de recherche des observations
!    KMXGSL : nombre maximum d'obs. du type KTYPO a selectionner

!  Arguments de sortie  : implicite dans common QANADA
!  -------------------
!  Sous-programme amont : CASGVA
!  --------------------
!  Sous-programmes aval : CACHBU
!  --------------------
!  COMMON utilises      : QABOIT QANADA
!  ---------------        YOMCST YOMLUN YOMDB YOMANCS YOMCT0

!  Modifications        : VC 02/93  PC 05/96
!  -------------          FT 11/00  CMAFOC --> ODB
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPABL   , ONLY : JP3KSE
USE QAPABO   , ONLY : NBONBT
USE YOMCST   , ONLY : RA
USE YOMLUN   , ONLY : NULOUT
USE YOMDB
USE YOMANCS  , ONLY : RDEGREES
USE QABOIT   , ONLY : MPROBO   ,MNBOBO
USE QANADA   , ONLY : NINSELG  ,NIASELG  ,RZDSELG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTSK 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDISTR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXGSL 
INTEGER(KIND=JPIM) ::   INUMB(NBONBT), IAD(4000)
REAL(KIND=JPRB) ::      ZD(4000)

INTEGER(KIND=JPIM) :: IBOITE, IDEC, IDEC1, ILNBB, INBOBS, JB, JOB, JTYPO

REAL(KIND=JPRB) :: ZLAT, ZLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cachbu.intfb.h"

#include "qadits.h"
#include "openmp_obs.h"

!-----------------------------------------------------------------------
!  - 1 -  Quelles sont les boites qui coupent le cercle de recherche ?
!         ------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CASGRA',0,ZHOOK_HANDLE)
ILNBB = 0
CALL CACHBU (PLAT, PLON, PDISTR, INUMB, ILNBB)

!-----------------------------------------------------------------------
!  - 2 -  Extraction des observations
!         ---------------------------

DO JB = 1 , ILNBB
  IBOITE = INUMB(JB)
  INBOBS = MNBOBO(IBOITE,KTYPO)
  DO JOB = 1 , INBOBS
    IAD(JOB) = MPROBO(IBOITE,KTYPO)+JOB-1
    ZLAT = ROBHDR(IAD(JOB),MDBLAT)
    ZLON = ROBHDR(IAD(JOB),MDBLON)
    ZD(JOB) = CADIT2(ZLAT,ZLON,PLAT,PLON)
  ENDDO

! selection si le critere distance est satisfait

  IDEC = 0
  DO JTYPO = 1 , KTYPO-1
    IDEC = IDEC + NINSELG(JTYPO,KH,KTSK)
  ENDDO
  DO JOB = 1, INBOBS
    IF (((KTYPA /= KTYPO .AND. KTYPA /= KTYPO+10) .OR. &
       & ZD(JOB) > 0.0_JPRB) .AND. ZD(JOB) <= PDISTR**2 ) THEN  
      IDEC1=IDEC+NINSELG(KTYPO,KH,KTSK)+1
      IF ( IDEC1 > JP3KSE ) THEN
        WRITE (NULOUT,291) KTYPO
        IF (LHOOK) CALL DR_HOOK('CASGRA',1,ZHOOK_HANDLE)
        RETURN
      ENDIF
      NINSELG(KTYPO,KH,KTSK)=NINSELG(KTYPO,KH,KTSK)+1
      NIASELG(IDEC1,KH,KTSK)=IAD(JOB)
      RZDSELG(IDEC1,KH,KTSK)=ZD(JOB)
      IF ( NINSELG(KTYPO,KH,KTSK) == KMXGSL ) THEN
        WRITE (NULOUT,292) JB,ILNBB,KTYPO,KMXGSL,PLAT*RDEGREES,PLON*RDEGREES
        IF (LHOOK) CALL DR_HOOK('CASGRA',1,ZHOOK_HANDLE)
        RETURN
      ENDIF
    ENDIF
  ENDDO
ENDDO

291 FORMAT(' debordement nb total obs selectionnees (type=',I2,')')
292 FORMAT(' no boite lue =',I4,' sur ',I4,'  KTYPO =',I2,&
 & '  KMXGSL =',I5,'  Lat. =',F6.2,' Lon. =',F7.2,&
 & '  *** Debordement dans CASGRA *** ')  

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASGRA',1,ZHOOK_HANDLE)
END SUBROUTINE CASGRA
