SUBROUTINE CASGVA (KTYPA,PDISTR,KMXGSL,KMXGQ,PLAT,PLON,KH,KTSK)
!****------------------------------------------------------------------
!****  CASGVA :  selection geographique des observations autour d'une
!****  ------    verticale
!****
!****  Auteurs : YD 06/89, 11/89, MCP 03/90, VC 08/92
!****------------------------------------------------------------------
!**
!  ENTREE : KTYPA : type d'analyse (0 OU 1-10)
!  ------   PDISTR: distance maximale de recherche des observations
!           KMXGSL: nombre maximum d'observation par type
!           KMXGQ : nombre maximum d'obs. par type et par secteur
!           PLAT  : latitude en radian
!           PLON  : longitude en radian
!           KH    : point traite --> position sur l'horizontale
!           KTSK  : numero de tache

!  SORTIE : implicite dans common QANADA
!  ------

!  DESCRIPTION:     CONSTITUTION DU TABLEAU NIASELG CONTENANT NINSELG
!  -----------      ADRESSES D'OBSERVATIONS (ET ALIMENTATION RZDSELG
!                   ET NIQSELG) SELECTIONNEES AU VOISINAGE DE LA
!                   VERTICALE DE COORDONNEES (PLAT,PLON) SELON DES
!                   CRITERES GEOGRAPHIQUES FIXES PAR (PDISTR,KMXGSL,KMXGQ).
!**
!****------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCOCTP , ONLY : NTEMP    ,NSATEM
USE PARDIMO  , ONLY : JPNOTP
USE YOMOBA   , ONLY : MNBOTPG
USE QANADA   , ONLY : NINSELG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KTYPA 
REAL(KIND=JPRB)   ,INTENT(IN) :: PDISTR(JPNOTP) 
INTEGER(KIND=JPIM),INTENT(IN) :: KMXGSL(JPNOTP) 
INTEGER(KIND=JPIM),INTENT(IN) :: KMXGQ(JPNOTP) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN) :: PLON 
INTEGER(KIND=JPIM),INTENT(IN) :: KH 
INTEGER(KIND=JPIM),INTENT(IN) :: KTSK 

!  NOMBRE MIMIMUM DE TEMP DISPONIBLES AUTOUR D'UN SATEM A CONTROLER
!  A PARTIR DUQUEL ON N'UTILISE PAS D'AUTRES SATEM POUR LE CONTROLER.

INTEGER(KIND=JPIM) :: JPSATE

INTEGER(KIND=JPIM) :: JPSEPT

PARAMETER (JPSATE=3, JPSEPT=7)

INTEGER(KIND=JPIM) :: JJ, JI

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "casgqa.intfb.h"
#include "casgra.intfb.h"

!**--------------------------------------------------------------------
!** - 1 - PREAMBULE
!**       ---------

IF (LHOOK) CALL DR_HOOK('CASGVA',0,ZHOOK_HANDLE)
DO JI = 1,JPNOTP
  NINSELG(JI,KH,KTSK)=0
ENDDO

!**--------------------------------------------------------------------
!** - 2 - SELECTION GEOGRAPHIQUE PAR TYPE D'OBS
!**       -------------------------------------

DO JJ = 1,JPSEPT
  IF ( MNBOTPG(JJ) /= 0 .AND. KMXGSL(JJ) /= 0 ) THEN
    IF ( KTYPA /= NSATEM .OR. JJ /= NSATEM .OR. &
       & NINSELG(NTEMP,KH,KTSK) <= JPSATE )     THEN  
      CALL CASGRA(KTYPA,JJ,KH,KTSK,PLAT,PLON,PDISTR(JJ),KMXGSL(JJ))
      IF ( KMXGQ(JJ) > 0 ) THEN
        CALL CASGQA(KMXGQ(JJ),KMXGSL(JJ),PLAT,PLON,JJ,KH,KTSK)
      ENDIF
    ENDIF
  ENDIF
ENDDO

!**--------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASGVA',1,ZHOOK_HANDLE)
END SUBROUTINE CASGVA
