SUBROUTINE CASPIA( KANTY  , LDVERT , LDFLAJ , PDELTA , PDELPI ,&
 & PRESS  , KH     , KTASK  ,&
 & KASELS , KOSELS , PDSELS , KNSELS , KNOUT  )  
!-----------------------------------------------------------------------
!   CASPIA : Selection de predicteurs interessants dans une couche
!   ------   de pression, parmi un ensemble d'observations selectionnees
!            autour d'un point.

!   auteur : M.-C. Pierrard 05/90
!   Mod. O.Spaniel  : 03-04-15 cleaning-a same named entity from modules
!--------------------------------------------------------------------

!  But : ut supra
!  ---
!    constitution  des tableaux  sus-mentionnes,  notamment  KASELS
!    contenant KNSELS adresses de predicteurs extraits des observa-
!    tions  selectionnees   geographiquement  au  voisinage  de  la
!    verticale. Les types de predicteur
!    a extraire  sont  indiques  par le  tableau  de flags logiques
!    LDFLAJ. Les predicteurs  doivent se trouver dans la  couche de
!    pression (PRESS-PDELTA,PRESS+PDELTA) ou (0,PRESS+PDELTA) selon
!    la valeur  de LDVERT. De  plus les  predicteurs  extraits d'un
!    meme message doivent etre distants d'au moins PDELPI; certains
!    niveaux  des  profils  sont donc  supprimes. En  revanche  les
!    niveaux les plus proches de PRESS sont privilegies.

!  Arguments d'entree   :
!  ------------------
!    KANTY   : type  d'analyse  (normale ou  controle de  coherence
!              spatiale)
!    LDVERT  : mode de selection
!              = .TRUE.   selection sur la verticale
!              = .FALSE.  selection sur un seul niveau
!    LDFLAJ  : flags indiquant les variables de la table NVY a uti-
!              liser comme predicteurs
!    PDELTA  : distance verticale de selection, en pression (en Pa)
!    PDELPI  : interdistance minimale entre deux niveaux selection-
!              nes a partir du meme message (en Pa)
!    PRESS   : si LDVERT = .FALSE.  pression du point (en Pa)
!              si LDVERT = .TRUE.   pression du niveau le plus bas
!    KH      : position du point de travail
!    KTASK   : numero de la tache d'execution

!  Arguments de sortie  :
!  -------------------
!    KASELS  : tableau  contenant, pour chaque type  de predicteurs
!              de la table NVY les adresses des predicteurs retenus
!    KOSELS  : adresse de  l'observation  contenant  le  predicteur
!              situe a l'adresse KASELS
!    PDSELS  : distance avec le predicteur situe a l'adresse KASELS
!    KNSELS  : nombre de predicteurs de chaque type selectionnes
!    KNOUT   : nombre total  de predicteurs  (somme des KNSELS); ce
!              nombre est  utile pour dimensionner  les matrices de
!              variances-covariances

!  Sous-programme amont : CASSVA
!  --------------------
!  COMMON utilises      : QAVARA   - QAKEKI
!  ---------------        YOMDDROA - YOMOAHDR - YOMOABDY
!                         YOMVNMB - YOMOCTP  - YOMOBA
!                         YOMLUN   - YOMCST   - YOMDIMO
!  Algorithme :
!  ----------
!    - initialisation des tableuax de sortie
!    - pour chaque type d'observation
!       - recherche des bornes de la couche de pression
!       - pour chaque observation selectionnee
!          - positionnement par rapport  aux bornes de la couche de
!            chaque predicteur possible
!          - selection des predicteurs effectivement dans la couche
!          - selection des predicteurs au code NCFINA souhaite
!          - application des contraintes d'interdistances en privi-
!            legiant les niveaux les plus proches du niveau analyse
!          - saisie des informations utiles (remplissage de KASELS,
!            KOSELS,etc...)
!          - si le  nombre maximum  de predicteurs  selectionnables
!            est  atteint,  impression  d'un message  et fin  de la
!            selection

!  Modifications : Ph. Caille 09/92
!  -------------   Ph. Caille 04/96  passage du CMAFOC au CMAFOC
!                  LG         03/00  SN analyses added 
!                  FT         11/00  CMAFOC --> ODB
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPABL   , ONLY : JPKZMA
USE QAPAVU   , ONLY : JPNVY    ,JPNINV   
USE PARDIMO  , ONLY : JPNOTP
USE YOMDB
USE YOMVNMB  , ONLY : NVNUMB
USE YOMLUN   , ONLY : NULOUT
USE QAVARA   , ONLY : NVXINV
USE QAKEKI   , ONLY : NCQFCO   ,NCQFIN
USE QANADA   , ONLY : NINSELG  ,NIASELG  ,RZDSELG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KANTY 
LOGICAL           ,INTENT(IN)    :: LDVERT 
LOGICAL           ,INTENT(IN)    :: LDFLAJ(0:JPNINV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELTA(JPNOTP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELPI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTASK 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KASELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KOSELS(JPNVY,JPKZMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDSELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNSELS(JPNVY) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNOUT 
INTEGER(KIND=JPIM) :: JPMXWG

PARAMETER ( JPMXWG = 5000 )

INTEGER(KIND=JPIM) :: IAWAG(JPNVY,JPMXWG),IAGARD(JPMXWG)
INTEGER(KIND=JPIM) :: INEXT(JPNVY),IPROCH(JPNVY)
INTEGER(KIND=JPIM) :: IAWAG1(JPMXWG),IAWAG2(JPMXWG),INUM1(JPMXWG)
REAL(KIND=JPRB) ::    ZPRES1(JPMXWG),ZPRES2(JPMXWG),ZINPR1(JPMXWG)
REAL(KIND=JPRB) ::    ZPRESS(JPNVY,JPMXWG),ZABDEL(JPNVY)

INTEGER(KIND=JPIM) :: IADO, IADW, ICODY, IFLG, II, ILNUM1, INBWAG, INGARD, &
 & INSEL, INWAG, IVNM, IDEC, JEXT, JGARD, JOB, ICNT, JTYP, JVAR, JWAG, J

REAL(KIND=JPRB) :: ZDEL, ZDIF, ZDTO, ZPBAS, ZPHAUT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "qabitu.h"
#include "openmp_obs.h"

!--------------------------------------------------------------------
!  - 1 -  Preambule
!         ---------

IF (LHOOK) CALL DR_HOOK('CASPIA',0,ZHOOK_HANDLE)
KNSELS(:) = 0
KNOUT     = 0

!--------------------------------------------------------------------
!  - 2 -  Recherche des predicteurs retenus dans le tableau ldflaj
!         --------------------------------------------------------

!  ne sont selectionnes que les predicteurs a un niveau de pression
!  adequat et tels que:

!    NCQFCO = 1   pour une analyse quelconque
!    NCQFCO = 1   lors du controle de coherence spatiale si le parametre
!                 est suppose bon ou douteux
!    NCQFIN <> 1  lors du controle de coherence spatiale si le parametre
!                 est suppose mauvais

!  boucle sur les types d'observations
IDEC = 0
DO JTYP = 1 , JPNOTP
  IF (PDELTA(JTYP) <= 0.0_JPRB) GOTO 254

  IF ( LDVERT ) THEN
    ZPHAUT = 0.0_JPRB
    ZPBAS  = PRESS + PDELTA(JTYP)
  ELSE
    ZPHAUT = PRESS - PDELTA(JTYP)
    ZPBAS  = PRESS + PDELTA(JTYP)
  ENDIF

!  boucle sur les observations selectionnees

  DO JOB = 1 , NINSELG(JTYP,KH,KTASK)

!   - 2.1 -  Positionnement des predicteurs retenus par rapport
!            a la couche (zphaut,zpbas)

    IADO   = NIASELG(IDEC+JOB,KH,KTASK)
    INWAG  = MLNKH2B(IADO+1)-MLNKH2B(IADO)
    INBWAG = 0

    DO JWAG = 1 , INWAG
      IADW  = MLNKH2B(IADO)+JWAG-1
      ICODY = ROBODY(IADW,MDBVNM)

      IF ( LDFLAJ(ICODY) ) THEN
        INBWAG = INBWAG + 1
        IAWAG1(INBWAG) = IADW

        IF ( ICODY == NVNUMB(4) ) THEN
! cas des epaisseurs
          ZPRES1(INBWAG) = (ROBODY(IADW,MDBRBPIO)+ROBODY(IADW,MDBPRL))*0.5_JPRB
        ELSE
          ZPRES1(INBWAG) = ROBODY(IADW,MDBRBPIO)
        ENDIF

        ZINPR1(INBWAG) = (ZPRES1(INBWAG)-ZPBAS) * (ZPRES1(INBWAG)-ZPHAUT)
      ENDIF
    ENDDO

!  - 2.2 -  Selection des predicteurs situes dans la couche

    INEXT  = 0
    IPROCH = 0

    DO JVAR = 1 , JPNVY
      ZABDEL(JVAR) = PDELTA(JTYP)
    ENDDO

    ILNUM1 = 0
    ICNT=0
    DO J=1,INBWAG
      IF (ZINPR1(J) <  0.0_JPRB) THEN
        ICNT=ICNT+1
        INUM1(ICNT)=J
      ENDIF
    ENDDO
    ILNUM1=ICNT

!  - 2.3 -  Selection des predicteurs au code final adequat pour
!           l'analyse souhaitee (fonction de l'analyse demandee)

    INBWAG = 0
    DO JWAG = 1 , ILNUM1
      IFLG = ROBODY(IAWAG1(INUM1(JWAG)),MDBFLG)
      IF (( KANTY <= 10 .AND. KBITLU(IFLG,NCQFCO,1) == 1 ) .OR. &
         & ( KANTY > 10 .AND. KBITLU(IFLG,NCQFIN,1) /= 1 )) THEN  
        INBWAG = INBWAG+1
        IAWAG2(INBWAG) = IAWAG1 ( INUM1(JWAG) )
        ZPRES2(INBWAG) = ZPRES1 ( INUM1(JWAG) )
      ENDIF
    ENDDO

    DO JWAG = 1 , INBWAG
      IVNM  = ROBODY(IAWAG2(JWAG),MDBVNM)
      ICODY = NVXINV(IVNM)
!lg      
!      IF (ICODY == JPPXSN) ICODY = JPPXHU
      INEXT(ICODY)               = INEXT(ICODY) + 1
      IAWAG(ICODY,INEXT(ICODY))  = IAWAG2(JWAG)
      ZPRESS(ICODY,INEXT(ICODY)) = ZPRES2(JWAG)
      ZDEL = ABS ( ZPRES2(JWAG) - PRESS )
      IF ( ZDEL < ZABDEL(ICODY) ) THEN
        ZABDEL(ICODY) = ZDEL
        IPROCH(ICODY) = INEXT(ICODY)
      ENDIF
    ENDDO

!  - 2.4 -  application des contraintes d'interdistances en donnant la 
!           preference aux observations les plus proches du niveau a analyser

!  ATTENTION : la demarche utilisee repose sur le fait que dans un message les 
!  ---------   differents parametres sont ranges par ordre de pression decroissante

    DO JVAR = 1 , JPNVY

      IF ( INEXT(JVAR) >= 1 ) THEN
        INGARD = 1
        IAGARD(INGARD) = IAWAG(JVAR,IPROCH(JVAR))
        II     = IPROCH(JVAR)

        DO JEXT = IPROCH(JVAR)-1 , 1 , -1
          ZDIF = ZPRESS(JVAR,JEXT) - ZPRESS(JVAR,II)
          IF ( JVAR == 1 ) THEN
            ZDTO = PDELPI / EXP(ZPRESS(JVAR,II))
          ELSE
            ZDTO = PDELPI * 1.E-5_JPRB
          ENDIF
          IF ( ZDIF >= ZDTO ) THEN
            INGARD         = INGARD + 1
            IAGARD(INGARD) = IAWAG(JVAR,JEXT)
            II             = JEXT
          ENDIF
        ENDDO

        II = IPROCH(JVAR)

        DO JEXT = IPROCH(JVAR)+1 , INEXT(JVAR)
          ZDIF = ZPRESS(JVAR,II) - ZPRESS(JVAR,JEXT)
          IF ( JVAR == 1 ) THEN
            ZDTO = PDELPI / EXP(ZPRESS(JVAR,II))
          ELSE
            ZDTO = PDELPI * 1.E-5_JPRB
          ENDIF
          IF ( ZDIF >= ZDTO ) THEN
            INGARD         = INGARD + 1
            IAGARD(INGARD) = IAWAG(JVAR,JEXT)
            II             = JEXT
          ENDIF
        ENDDO

!  - 2.5 -  stockage des adresses et autres renseignements

        DO JGARD = 1 , INGARD
          KNSELS(JVAR)       = KNSELS(JVAR) + 1
          KNOUT              = KNOUT + 1
          INSEL              = KNSELS(JVAR)
          KASELS(JVAR,INSEL) = IAGARD(JGARD)
          KOSELS(JVAR,INSEL) = IADO
          PDSELS(JVAR,INSEL) = RZDSELG(IDEC+JOB,KH,KTASK)
          IF ( INSEL == JPKZMA ) THEN
            WRITE (NULOUT,261) JPKZMA,JVAR,PRESS
            IF (LHOOK) CALL DR_HOOK('CASPIA',1,ZHOOK_HANDLE)
            RETURN
          ENDIF
        ENDDO

      ENDIF
    ENDDO
  ENDDO
  254 CONTINUE
  IDEC = IDEC + NINSELG(JTYP,KH,KTASK)
ENDDO

!  - 2.6 -  formats

261 FORMAT(' Debordement dans CASPIA: maximum atteint (',I3,&
 & ') pour JVAR=',I3,' en un point pres.:',F8.1)  

!--------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASPIA',1,ZHOOK_HANDLE)
END SUBROUTINE CASPIA
