SUBROUTINE CASSVA (KTASK,  KANTY,  KH,     LDFLAI, LDFLAJ,&
 & KNLX,   KNM,    PDELTA, PDELPI, PCORMIN,&
 & PLAT,   PLON,   PRESS,  PSIG,   KNBPV,  KIV,&
 & KASELS, KOSELS, PDSELS, KNSELS, PDXMB,  KNOUT)  
!----------------------------------------------------------------------------
!  CASSVA : Selection statistique des predicteurs sur une verticale
!  ------
!  auteur : MCP 08/90
!----------------------------------------------------------------------------
!  ENTREE :
!  ------
!           KTASK : NUMERO DE LA TACHE EXECUTANT CASSVA
!           KANTY : TYPE D'ANALYSE
!                   0 : ANALYSE DE CHAMPS EN POINTS DE GRILLE
!                   1,2,...10 : REJET DES OBS DE LA CATEGORIE = KANTY
!           KH    : POSITION DU POINT DE TRAVAIL
!           LDFLAI: FLAGS INDIQUANT QUELLES SONT LES VARIABLES DE LA
!                   TABLE NVX QU'ON SOUHAITE ANALYSER
!           LDFLAJ: FLAGS INDIQUANT QUELLES SONT LES VARIABLES DE LA
!                   TABLE NVY A UTILISER COMME PREDICTEURS
!           KNLX  : NOMBRE DE VARIABLES QU'ON SOUHAITE ANALYSER
!                   (NOMBRE DE MOTS DE LDFLAI VALANT .TRUE.)
!           KNM   : NOMBRE DE PREDICTEURS A SELECTIONNER POUR CHAQUE
!                   PREDICTANT QUAND IL Y EN A TROP.
!           PDELTA: DISTANCE VERTICALE DE SELECTION EN PRESSION (EN PA)
!                   DES OBSERVATIONS DE CHAQUE TYPE
!           PDELPI: INTERDISTANCE MINIMALE ENTRE DEUX NIVEAUX SELEC-
!                   TIONNES A PARTIR DU MEME MESSAGE (EN PA)
!           PCORMIN:CORRELATION MINIMALE QU'UN PREDICTEUR DOIT
!                   PRESENTER AVEC AU MOINS UN PREDICTANT POUR ETRE
!                   SELECTIONNE
!           PLAT  : LATITUDE  DU POINT EN REEL ET RADIAN
!           PLON  : LONGITUDE  "   "    "  "    "    "
!           PRESS : PRESSION EN REEL ET PASCAL POUR CHAQUE NIVEAU
!           PSIG  : ECART-TYPE D'ERREUR DU GUESS par predictand
!           KNBPV : NOMBRE DE NIVEAUX SUR LA VERTICALE

!  SORTIE : PSIG  : ECART-TYPE D'ERREUR DU GUESS POUR CHAQUE PREDICTANT
!  ------           Y COMPRIS POUR LES EPAISSEURS (VALEURS D'ENTREE
!                   ECRASEES DANS CE CAS)
!           KASELS: TABLEAU CONTENANT, POUR CHAQUE TYPE DE PREDICTEUR
!                   DE LA TABLE NVY, LES ADRESSES DES PREDICTEURS
!                   SELECTIONNES
!           KOSELS: ADRESSE DE L'OBSERVATION CONTENANT LE PREDICTEUR
!                   SITUE A L'ADRESSE KASELS
!           PDSELS: DISTANCE ENTRE LE PREDICTEUR SITUE A L'ADRESSE
!                   KASELS ET LE POINT (PLAT,PLON)
!           KNSELS: NOMBRE EFFECTIF DE PREDICTEURS DE CHAQUE TYPE
!                   SELECTIONNES
!           PDXMB : MATRICE DES COVARIANCES ENTRE LES ERREURS DU GUESS
!                   POUR LES PREDICTANTS ET POUR LES PREDICTEURS
!                   = 2EME MEMBRE DU SYSTEME LINEAIRE
!           KNOUT : NOMBRE TOTAL DE PREDICTEURS (SOMME DES KNSELS)
!                   --> DIMENSION UTILE DE PDXMB

!  COMMON UTILISES: QAPAVU QAPABL
!  ---------------
!  SOUS-PROGRAMMES: CASPIA CASELA
!  ---------------

!  DESCRIPTION:     A PARTIR DES OBSERVATIONS (=MESSAGES) PREALABLEMENT
!  -----------      SELECTIONNEES DANS UN VOLUME GEOGRAPHIQUE, ON VA
!                   EXTRAIRE DES PREDICTEURS (=PARAMETRES) DES TYPES
!                   SOUHAITES, SITUES DANS UNE TRANCHE DE PRESSION
!                   DEFINIE PAR PDELPI, PRESENTANT UNE CORRELATION
!                   MINIMALE PCORMIN AVEC LES PREDICTANTS. LEUR NOMBRE
!                   SERA ENSUITE LIMITE A somme des knm*knbpv EN SE BASANT
!                   SUR LES VALEURS DES CORRELATIONS ENTRE PREDICTEURS
!                   ET PREDICTANTS.
!                   LE TRAVAIL EST FAIT ESSENTIELLEMENT PAR LES SOUS-
!                   PROGRAMMES. LES CALCULS NECESSAIRES ETANT SIMILAIRES
!                   A CEUX QU'IL FAUT REALISER POUR CONSTRUIRE LE 2EME
!                   MEMBRE DU SYSTEME LINEAIRE DE L'INTERPOLATION OPTIMALE,
!                   LA MATRICE DE COVARIANCES CORRESPONDANT AU SECOND
!                   MEMBRE EST FOURNIE EN SORTIE. ON RECUPERE AUSSI
!                   D'AUTRES ELEMENTS UTILES POUR LE CALCUL DU RESULTAT
!                   FINAL DE L'ANALYSE: LES ECARTS-TYPES D'ERREUR DU
!                   GUESS POUR LES PREDICTANTS (UTILE DANS LE CAS DES
!                   EPAISSEURS DES VENTS ET DE LA TEMPERATURE).

!  MODIFICATIONS:   VC 03/92, PM 01/93, VC 10/93, Ph.C 04/96
!  -------------    FT 11/00  CMAFOC --> ODB
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!---------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPAVU   , ONLY : JPNVX    ,JPNVY    ,JPNINV
USE QAPABL   , ONLY : JPNPM    ,JPKZMA
USE PARDIMO  , ONLY : JPNOTP

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNBPV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTASK 
INTEGER(KIND=JPIM),INTENT(IN)    :: KANTY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KH 
LOGICAL           ,INTENT(IN)    :: LDFLAI(0:JPNVX) 
LOGICAL           ,INTENT(IN)    :: LDFLAJ(0:JPNINV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNM(KNLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELTA(JPNOTP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELPI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCORMIN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESS(KNBPV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIG(JPNVX,KNBPV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIV 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KASELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOSELS(JPNVY,JPKZMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDSELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNSELS(JPNVY) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDXMB(JPNPM,KNLX,KNBPV) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNOUT 
INTEGER(KIND=JPIM), EXTERNAL :: ISMAX

LOGICAL :: LLVERT

INTEGER(KIND=JPIM) :: IN

REAL(KIND=JPRB) :: ZPRES
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "casela.intfb.h"
#include "caspia.intfb.h"

!---------------------------------------------------------------------------
!  - 1 -  Selection des predicteurs dans une couche de pression donnee
!         ------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CASSVA',0,ZHOOK_HANDLE)
ZPRES  = PRESS(ISMAX(KNBPV,PRESS,1))
LLVERT = KNBPV  /=  1
CALL CASPIA (KANTY , LLVERT, LDFLAJ, PDELTA, PDELPI, ZPRES,&
 & KH, KTASK, KASELS, KOSELS, PDSELS, KNSELS, IN)  

!---------------------------------------------------------------------------
!  - 2 -  Selection statistique des predicteurs et recuperation du second
!         membre du systeme lineaire et des ecart-types d'erreur du guess
!         ---------------------------------------------------------------

IF ( IN == 0 ) THEN
  KNOUT = 0
ELSE
  CALL CASELA (KTASK,  LDFLAI, KNLX,   KNM,    PCORMIN,&
   & PLAT,   PLON,   PRESS,  PSIG,   KNBPV,  KIV ,&
   & KASELS, KOSELS, PDSELS, KNSELS,&
   & PDXMB,  IN,     KNOUT)  
ENDIF

!---------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASSVA',1,ZHOOK_HANDLE)
END SUBROUTINE CASSVA
