SUBROUTINE CASTOR
!****-------------------------------------------------------------------
!****   CASTOR : SYSTEME DE TRI DES OBSERVATIONS POUR LES RESIDUS
!****   ------
!****   auteur : PATRICK MOLL
!****-------------------------------------------------------------------
!**   BUT.
!**   ---
!**     PREPARATION DES PAQUETS D'OBSERVATION POUR LE MULTITASKING
!**     DU CALCUL DES RESIDUS.
!**     POUR CHAQUE PAQUET, ON DEFINIT 3 PARAMETRES :
!**     * MTYPOB(NSET)     TYPE D'OBSERVATION
!**     * MLNSET(NSET)     TAILLE DU PAQUET
!**     * MMXBDY(NSET)     NB MAX DE WAGONS PARMI LES OBS DU PAQUET
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LELAM
USE PARDIMO  , ONLY : JPNOTP
USE YOMANCS  , ONLY : RMDI
USE YOMDIMO  , ONLY : NMXSET   ,NMXLEN   ,NOBTOT
USE YOMOBA   , ONLY : MNBOTP
USE YOMOBSET , ONLY : MTYPOB   ,MLNSET   ,NSETOT   ,MMXBDY
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMDB
USE QALORI   , ONLY : QCAOBSET

IMPLICIT NONE

INTEGER(KIND=JPIM) :: INBOTP(10)
INTEGER(KIND=JPIM) :: ICMNLV(NMXLEN), IOTP(NOBTOT)

INTEGER(KIND=JPIM) :: JOBS, JSET, JTYP, IRET, INFO(1)

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "abor1.intfb.h"

#include "openmp_obs.h"

!**---------------------------------------------------------------------
!**    1.    INITIALISATIONS ET TESTS DE BASE.
!**          --------------------------------

IF (LHOOK) CALL DR_HOOK('CASTOR',0,ZHOOK_HANDLE)
CALL GETDB('CASTOR', 0, IRET, INFO, 0, ZINFO, 0, -1, -1, -1, -1, -1, -1)

IF (IRET == 0 .AND. LHOOK) CALL DR_HOOK('CASTOR',1,ZHOOK_HANDLE)
IF (IRET == 0) RETURN
IF (IRET /= NOBTOT) THEN
  WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
  WRITE(NULERR,*)'       expecting =',NOBTOT,', but got =',IRET
  CALL ABORTDB('CASTOR','INCONSISTENCIES FROM GETDB',IRET)
ENDIF

MLNSET(:)=0
INBOTP(:)=0
NSETOT=0

DO JOBS = 1,NOBTOT
  IOTP(JOBS) = ROBSU(JOBS,MDBOTP)
ENDDO

!  test necessaire pour la configuration E701
IF ( LELAM ) THEN
  DO JOBS = 1,NOBTOT
    IF ( ROBSU(JOBS,MDBTLA) /= RMDI ) THEN
      INBOTP(IOTP(JOBS))=INBOTP(IOTP(JOBS))+1
    ENDIF
  ENDDO
ENDIF

IF (.NOT.ALLOCATED(QCAOBSET)) ALLOCATE(QCAOBSET(NMXSET,NMXLEN))

!**---------------------------------------------------------------------
!**    2.    PREPARATION DES PAQUETS D'OBS POUR CANARI.
!**          -----------------------------------------

DO JTYP = 1 , JPNOTP

  IF ( MNBOTP(JTYP) == 0 ) CYCLE
  IF ( LELAM .AND. INBOTP(JTYP) == 0 ) CYCLE

  NSETOT = NSETOT + 1
  MTYPOB(NSETOT) = JTYP

  DO JOBS = 1,NOBTOT

    IF (IOTP(JOBS) /= JTYP ) CYCLE

!  test necessaire pour la configuration E701
    IF ( ROBSU(JOBS,MDBTLA) /= RMDI ) THEN

      IF ( MLNSET(NSETOT) == NMXLEN )   THEN
        MMXBDY(NSETOT) = MAXVAL(ICMNLV(1:NMXLEN))
        NSETOT = NSETOT + 1
        MTYPOB(NSETOT) = JTYP
      ENDIF
      MLNSET(NSETOT) = MLNSET(NSETOT) + 1
      ICMNLV(MLNSET(NSETOT)) = ROBSU(JOBS,MLNK_HDR2BODY(2))
      ROBSU(JOBS,MDB_KSET_AT_INDEX) = NSETOT
      QCAOBSET(NSETOT,MLNSET(NSETOT)) = JOBS

    ENDIF
  ENDDO
  MMXBDY(NSETOT) = MAXVAL(ICMNLV(1:MLNSET(NSETOT)))
ENDDO

WRITE(NULOUT,*) '  '
WRITE(NULOUT,*) ' RESULTAT DE LA FABRICATION DES PAQUETS D OBS'
WRITE(NULOUT,*) '  '
DO JSET = 1 , NSETOT
  WRITE(NULOUT,*) ' PAQUET NO ',JSET,',  TYPE NO ',MTYPOB(JSET),&
   & ',  CONTIENT ',MLNSET(JSET),' OBS'  
ENDDO

IF ( NSETOT > NMXSET ) THEN
  WRITE(NULERR,*) ' *** CAUTION! THE BEAVER IS DAMAGED!!! ***'
  WRITE(NULERR,*) '     NSETOT=',NSETOT,' WHILE NMXSET=',NMXSET
  CALL ABOR1('CASTOR')
ENDIF

!**---------------------------------------------------------------------

CALL PUTDB('CASTOR',0, IRET, INFO, 0, ZINFO, 0)

IF (LHOOK) CALL DR_HOOK('CASTOR',1,ZHOOK_HANDLE)
END SUBROUTINE CASTOR
