SUBROUTINE CAVEGI
!****-------------------------------------------------------------------
!****   CAVEGI : initialisation de QACVEG
!****   ------   (controle de l'analyse de surface associee a ISBA)
!****   Auteur :       D. Giard    03/94  05/96
!****      Mod :       E. Bazile   01/97 Retrait du biais moyen de T2m/Hu2m
!****                                    pour analyse de l'eau du sol
!****                  D. Giard    06/97 Differents rappels clim.
!****             S. Ivatek-Sahdan 09/02 Smoothing of SWI and changing Wp
!****-------------------------------------------------------------------
!**   ROUTINES APPELANTES: - CANAMI -  APPEL: CALL CAVEGI
!**   ENTREES: - EXPLICITE - NAMELIST NACVEG
!**   SORTIES: - *QACVEG* : constantes et cles de controle de l'analyse de
!**               surface associee a ISBA, utilisees dans CACSTS si LSOLV=.T.
!**   COMMONS: - YOMLUN  : UNITES LOGIQUES
!**            - YOMPHY1 : CONSTANTES PHYSIQUES
!**            - QACTEX  : CLEFS DE L'ANALYSE - RAPPEL CLIM
!**            - YOMDAG  : DESCRIPTION DE L'EBAUCHE
!**   EXTERNES: - ABOR1  - ROUTINE SYSTEME D'ARRET D'EXECUTION.
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LELAM
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMPHY   , ONLY : LSOLV
USE YOMPHY1  , ONLY : EWFC     ,EWWILT   ,GWFC     ,GWWILT
USE YOMDAG   , ONLY : NECHGU   ,NTYPGU
USE QACTEX   , ONLY : RCLIMCA
USE QACVEG   , ONLY : VGAT1    ,VGAT2    ,VGAT3    ,VGAH1    ,&
 & VGAH2    ,VGAH3    ,VGBT1    ,VGBT2    ,VGBT3    ,&
 & VGBH1    ,VGBH2    ,VGBH3    ,VGCT1    ,VGCT2    ,&
 & VGCH1    ,VGCH2    ,SIGT2MP  ,SIGHP1   ,SIGHP2   ,&
 & SIGT2MR  ,SIGH2MR  ,ADWR     ,NLISSEW  ,LSGOBS   ,&
 & LIMVEG   ,MINDJ    ,V10MX    ,SPRECIP  ,SWFC     ,&
 & LHUMID   ,LISSEW   ,SIGT2MO  ,SIGH2MO  ,ANEBUL   ,&
 & NNEBUL   ,SNEIGT   ,NNEIGT   ,SNEIGW   ,NNEIGW   ,&
 & SCOEFT   ,SCOEFH   ,RCLIMTS  ,RCLIMTP  ,RCLIMWS  ,&
 & RCLIMWP  ,RCLIMN   ,RCLIMV   ,SEVAP    ,SICE     ,&
 & SMU0     ,L_SM_WP  ,NR_SM_WP ,RA_SM_WP  

IMPLICIT NONE

INTEGER(KIND=JPIM) :: J

REAL(KIND=JPRB) :: ZARG, ZEPS, ZSAB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"

#include "nacveg.h"

!**---------------------------------------------------------------------
!**  0. Analyse de surface sans ISBA : initialisation minimale.
!**     ------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CAVEGI',0,ZHOOK_HANDLE)
RCLIMTS = 1.0_JPRB
RCLIMTP = 1.0_JPRB
RCLIMWS = 1.0_JPRB
RCLIMWP = 1.0_JPRB
RCLIMN = 10._JPRB
RCLIMV =  5._JPRB
LISSEW = .FALSE.
IF (.NOT. LSOLV .AND. LHOOK) CALL DR_HOOK('CAVEGI',1,ZHOOK_HANDLE)
IF (.NOT. LSOLV) RETURN

!**---------------------------------------------------------------------
!**  1. Initialisation des polynomes bruts et des champs de reference.
!**     -------------------------------------------------------------

! lecture des coefficients polynomiaux
DO J=1,24
  READ(61,'(6X,8F10.4)') VGAT1(J),VGAT2(J),VGAT3(J),&
   & VGBT1(J),VGBT2(J),VGBT3(J),&
   & VGCT1(J),VGCT2(J)  
ENDDO
DO J=1,24
  READ(61,'(6X,8F10.4)') VGAH1(J),VGAH2(J),VGAH3(J),&
   & VGBH1(J),VGBH2(J),VGBH3(J),&
   & VGCH1(J),VGCH2(J)  
ENDDO

! constantes pour le limon (reference)
ZSAB = 43._JPRB
ZARG = 18._JPRB
ADWR = GWFC*(ZARG**EWFC) - GWWILT*(ZARG**EWWILT)

! ecarts-type d'erreur de prevision
SIGHP1= 15.E-2_JPRB
DO J = 1,24
  SIGT2MP(J)= MAX(.3_JPRB , 2.7_JPRB*(1.0_JPRB-&
   & ((REAL(J,JPRB)-15._JPRB)/9._JPRB)**2))  
  SIGHP2(J)= (REAL(J,JPRB)*2.0_JPRB/3._JPRB - 15._JPRB)*1.E-2_JPRB
ENDDO

! ecarts-type d'erreur d'observation de reference
SIGT2MR = 1.0_JPRB
SIGH2MR = 10.E-2_JPRB

!**---------------------------------------------------------------------
!**  2. Lecture et controle de la namelist NACVEG.
!**     -----------------------------------------

! valeurs par defaut
LIMVEG  = .TRUE.
MINDJ   = 6
V10MX   = 10._JPRB
SPRECIP = 0.6_JPRB
SEVAP   = 0.001_JPRB
SICE    = 5._JPRB
SMU0    = 0.0_JPRB
SWFC    = 1.0_JPRB
LHUMID  = .TRUE.
SIGT2MO = 1.0_JPRB
SIGH2MO = 0.1_JPRB
ANEBUL  = 0.0_JPRB
NNEBUL  = 1
SNEIGT  = 1.0_JPRB
NNEIGT  = 0
SNEIGW  = 0.0_JPRB
NNEIGW  = 1
SCOEFT  = 0.5_JPRB
SCOEFH  = 0.0_JPRB
RCLIMTS = 0.0_JPRB
RCLIMTP = 1.0_JPRB
RCLIMWP = 0.1_JPRB
RCLIMWS = 0.0_JPRB
RCLIMN  = 0.0_JPRB
RCLIMV  = 1.0_JPRB
LISSEW  = .TRUE.
L_SM_WP = .FALSE.
NR_SM_WP= 1
IF (.NOT.LELAM) THEN
  RA_SM_WP = 10000.0_JPRB
ELSE
  RA_SM_WP = 2500.0_JPRB
ENDIF

! lecture
CALL POSNAM(NULNAM,'NACVEG')
READ (NULNAM,NACVEG, ERR=30, END=40)

! controles
V10MX  = ABS(V10MX)
SPRECIP= MAX(SPRECIP,0.0_JPRB)
SEVAP  = MAX(SEVAP,0.0_JPRB)
SWFC   = MAX(SWFC,1.0_JPRB)
ANEBUL = MAX(0.0_JPRB,MIN(1.0_JPRB,ANEBUL))
NNEBUL = ABS(NNEBUL)
SNEIGT = MAX(0.0_JPRB,MIN(1.0_JPRB,SNEIGT))
NNEIGT = ABS(NNEIGT)
SNEIGW = MAX(0.0_JPRB,MIN(1.0_JPRB,SNEIGW))
NNEIGW = ABS(NNEIGW)
SCOEFT = MAX(0.0_JPRB,MIN(1.0_JPRB,SCOEFT))
SCOEFH = MAX(0.0_JPRB,MIN(1.0_JPRB,SCOEFH))
RCLIMN = MAX(0.0_JPRB,RCLIMN)
RCLIMV = MAX(0.0_JPRB,RCLIMV)
IF (RCLIMCA > 0.0_JPRB) THEN
  RCLIMTS= MAX(0.0_JPRB,MIN(1.0_JPRB/RCLIMCA,RCLIMTS))
  RCLIMTP= MAX(0.0_JPRB,MIN(1.0_JPRB/RCLIMCA,RCLIMTP))
  RCLIMWS= MAX(0.0_JPRB,MIN(1.0_JPRB/RCLIMCA,RCLIMWS))
  RCLIMWP= MAX(0.0_JPRB,MIN(1.0_JPRB/RCLIMCA,RCLIMWP))
ENDIF

!**---------------------------------------------------------------------
!**  3. Initialisation des variables internes.
!**     -------------------------------------

ZEPS   = 1.E-13_JPRB
LSGOBS = SIGT2MO > 0.0_JPRB .AND. SIGH2MO > 0.0_JPRB .AND.&
 & (ABS(SIGH2MO-SIGH2MR) > ZEPS .OR. ABS(SIGT2MO-SIGT2MR) > ZEPS)  
NLISSEW= 0

!**---------------------------------------------------------------------
!**  4. Controle de l'ebauche.
!**     ---------------------

IF (.NOT.LELAM) THEN
  IF (NTYPGU /= 3) THEN
    WRITE (NULOUT,*) ' WARNING : TYPE DE GUESS INCORRECT'
  ENDIF
  IF (NECHGU /= 6) THEN
    WRITE (NULOUT,10)
    V10MX = 0.0_JPRB
  ENDIF
ENDIF
10 FORMAT(/,&
 & ' ATTENTION : L ANALYSE ISBA DE L HUMIDITE DU SOL, ADAPTEE A',&
 & ' DES PREVISIONS 6h, EST INVALIDEE -SEUIL SUR LE VENT- !')  

!**----------------------------------------------------------------------
!**  5. Impressions.
!**     -----------

WRITE (NULOUT,20)&
 & LIMVEG, V10MX, MINDJ, SPRECIP, SEVAP, SICE, SMU0, SWFC,&
 & LHUMID, LISSEW, LSGOBS, SIGT2MO, SIGH2MO,&
 & ANEBUL, NNEBUL, SNEIGT, NNEIGT, SNEIGW, NNEIGW,&
 & SCOEFT, SCOEFH,&
 & RCLIMTS, RCLIMTP, RCLIMWS, RCLIMWP, RCLIMN, RCLIMV,&
 & L_SM_WP, NR_SM_WP ,RA_SM_WP  
20 FORMAT(/,&
 & '     NAMELIST NACVEG (ANALYSE DE SURFACE ISBA)',/,&
 & '     -----------------------------------------',/,&
 & ' CONDITIONS D ANALYSE DU CONTENU EN EAU DU SOL :',/,3X,&
 & ' LIMVEG :',L2,' V10MX :',F7.2, ' MINDJ :',I3,&
 & ' SPRECIP :',F7.2,' SEVAP :',F10.7,' SICE :',F10.3,&
 & ' SMU0 :',F7.3,' SWFC :',F7.3,/,&
 & ' PONDERATION DES INCREMENTS :',/,3X,&
 & ' LHUMID :',L2,' LISSEW :',L2,&
 & ' LSGOBS :',L2,' SIGT2MO :',F5.2,' SIGH2MO :',F4.2,/,3X,&
 & ' ANEBUL :',F4.2,' NNEBUL :',I2,&
 & ' SNEIGT :',F4.2,' NNEIGT :',I2,&
 & ' SNEIGW :',F4.2,' NNEIGW :',I2,/,&
 & ' CALCUL DU BIAIS :',/,3X,&
 & ' SCOEFT :',F4.2,' SCOEFH :',F4.2,/,&
 & ' RAPPEL VERS LA CLIM. :',/,3X,&
 & ' RCLIMTS :',F5.2,' RCLIMTP :',F5.2,&
 & ' RCLIMWS :',F5.2,' RCLIMWP :',F5.2,&
 & ' RCLIMN :',F5.2,' RCLIMV :',F5.2,/,&
 & ' SPATIAL SMOOTHING OF SWI AND CHANGING Wp : ',/,&
 & ' L_SM_WP :',L2,' NR_SM_WP :',I2,' RA_SM_WP :',F10.3,//)  

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAVEGI',1,ZHOOK_HANDLE)
RETURN
30 CONTINUE
CALL ABOR1 (' ROUTINE *CAVEGI*: ERREUR LECTURE ')
40 CONTINUE
CALL ABOR1 (' ROUTINE *CAVEGI*: FIN DE FICHIER SUR *NULNAM* ')
IF (LHOOK) CALL DR_HOOK('CAVEGI',1,ZHOOK_HANDLE)

!**---------------------------------------------------------------------
END SUBROUTINE CAVEGI
