SUBROUTINE CAVODK
!****-------------------------------------------------------------------
!****   CAVODK : Controle de qualite des observations
!****   ------
!****   auteur : Ph.Caille 02/92
!****-------------------------------------------------------------------

!*** But : controle de la qualite des observations
!*** ---
!*** Sequence d'appel     : CALL CAVODK
!*** ----------------
!*** Arguments d'entree   : neant
!*** ------------------
!*** Arguments de sortie  : neant
!*** -------------------
!*** Sous-programme amont : CANARI
!*** --------------------
!*** Sous-programmes aval : CARATK - CARNAK - GETDB - PUTDB
!*** --------------------
!*** COMMON utilises      : YOMDIMO - YOMDB
!*** ---------------
!*** Algorithme : - CARATK initialise les commons QADOR1 et QADOR2 (ds QADORE)
!*** ----------     qui contiennent les parametres pour CANEVA
!***              - mise en memoire des obs que l'on veut controler
!***              - CARNAK effectue les analyses par appel a CANEVA
!***              - mise a jour des flags des obs controlees
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NOBTOT
USE YOMDB
USE YOMCOCTP , ONLY : NSATEM

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JPOB1,JPOB2,JPOB3

PARAMETER ( JPOB1 = 3 , JPOB2 = 2 , JPOB3 = 8 )

! ces parametres servent a definir la 2eme dimension des tableaux qui contiendront
! les informations relatives aux parametres dont on aura besoin dans CARNAK
!   pour ZOBHDR: latitude transformee TLA, latitude LAT et longitude LON;
!   pour IOBODY: numero de variable VNM, flag d'analyse FLG;
!   pour ZOBODY: FGE, RBPIO, PRL, RBOE, OER, RER, OMF, FGC2.

REAL(KIND=JPRB),ALLOCATABLE :: ZOBHDR(:,:),ZOBODY(:,:)

INTEGER(KIND=JPIM),ALLOCATABLE :: ILNKH2B(:),IOBODY(:,:)

INTEGER(KIND=JPIM) :: ITYP(NOBTOT)
INTEGER(KIND=JPIM) :: INDSIZ(3)
INTEGER(KIND=JPIM) :: IRET,INFO(1),IPRE,IWAG,JOBS

REAL(KIND=JPRB) :: ZINFO(1) 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "caratk.intfb.h"
#include "carnak.intfb.h"

#include "openmp_obs.h"

!**---------------------------------------------------------------------
!**  - 1 -  INITIALISATIONS DIVERSES
!**         ------------------------

IF (LHOOK) CALL DR_HOOK('CAVODK',0,ZHOOK_HANDLE)
CALL CARATK

!**---------------------------------------------------------------------
!**  - 2 -  RECUPERATION DES OBS DE MON PROCESSEUR
!**         --------------------------------------

CALL GETDB('CAVODK', 0, IRET, INFO, 0, ZINFO, 0, -1, -1, -1, -1, -1, -1)

INDSIZ(1)=NOBTOT
INDSIZ(2)=SIZE(ROBODY,1)
INDSIZ(3)=NOBTOT+1

ALLOCATE(ILNKH2B(INDSIZ(3)))
ALLOCATE(ZOBHDR(INDSIZ(1),JPOB1))
ALLOCATE(IOBODY(INDSIZ(2),JPOB2))
ALLOCATE(ZOBODY(INDSIZ(2),JPOB3))

ILNKH2B(:)=MLNKH2B(:)
ZOBHDR(:,1)=ROBHDR(1:INDSIZ(1),MDBTLA)
ZOBHDR(:,2)=ROBHDR(1:INDSIZ(1),MDBLAT)
ZOBHDR(:,3)=ROBHDR(1:INDSIZ(1),MDBLON)
IOBODY(:,1)=ROBODY(:,MDBVNM)
IOBODY(:,2)=ROBODY(:,MDBFLG)
ZOBODY(:,1)=ROBODY(:,MDBFGE)
ZOBODY(:,2)=ROBODY(:,MDBRBPIO)
ZOBODY(:,3)=ROBODY(:,MDBPRL)
ZOBODY(:,4)=ROBODY(:,MDBRBOE)
ZOBODY(:,5)=ROBODY(:,MDBOER)
ZOBODY(:,6)=ROBODY(:,MDBRER)
ZOBODY(:,7)=ROBODY(:,MDBOMF)
ZOBODY(:,8)=ROBODY(:,MDB_QC_L)

IPRE=0
DO JOBS = 1 , NOBTOT
  IWAG=ILNKH2B(JOBS+1)-ILNKH2B(JOBS)
  IF (ROBHDR(JOBS,MDBOTP) == NSATEM ) IWAG=IWAG*2
  IPRE=MAX(IPRE,IWAG)
  ITYP(JOBS)=ROBHDR(JOBS,MDBOTP)
ENDDO
  
CALL PUTDB('CAVODK', 0, IRET, INFO, 0, ZINFO, 0)

!**---------------------------------------------------------------------
!**  - 3 - CONTROLE DE QUALITE
!**        -------------------

CALL CARNAK(ILNKH2B,ZOBHDR,IOBODY,ZOBODY,JPOB1,JPOB2,JPOB3,INDSIZ,IPRE,ITYP)

DEALLOCATE(ILNKH2B)
DEALLOCATE(ZOBHDR)

!**---------------------------------------------------------------------
!**  - 4 - MISE A JOUR BASE DE DONNEES
!**        ---------------------------

CALL GETDB('CAVODK', 1, IRET, INFO, 0, ZINFO, 0, -1, -1, -1, -1, -1, -1)

ROBODY(:,MDB_QC_L)=ZOBODY(:,8)
ROBODY(:,MDBFLG)=IOBODY(:,2)

CALL PUTDB('CAVODK', 1, IRET, INFO, 0, ZINFO, 0)

DEALLOCATE(IOBODY)
DEALLOCATE(ZOBODY)

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAVODK',1,ZHOOK_HANDLE)
END SUBROUTINE CAVODK
