SUBROUTINE CORMASS3A

!**** *CORMASS*  - COMPUTE THE MASS CORRECTION IN ALADIN - PART A

!     Purpose.
!     --------
!     This program computes the mass in the coupling file

!**   Interface.
!     ----------
!        *CALL* *CORMASS3A

!        Explicit arguments :     NONE
!        --------------------

!        Implicit arguments :     NONE
!        --------------------

!     Method.
!     -------
!     - Go to grid point space (ESPEREE) ;
!     -   Compute the total vapor water field (Pvap),
!         then the dry-air surface pressure field (Pdry = P - Pvap)
!         pressure field.
!     Externals.
!     ----------

!     Reference.
!     ----------
!        CLIMAT-ARPEGE documentation (Meteo-France, CNRM/GMGEC/EAC)

!     Author.
!     -------
!        Jean Philippe Piedelievre and Michel Deque
!          (from CORMASS written by Pascal Marquet)  *Meteo-France*

!     Modifications.
!     --------------
!        Original : 05-06-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   
USE YOMDIM   , ONLY : NFLEVG   ,NSPEC2
USE YOMGEM   , ONLY : VBH      ,VAH      ,NGPTOT   ,NGPTOTG  
USE YOMSP    , ONLY : SPSP     ,SPQ
USE YOMMP    , ONLY : MYPROC
USE YOMMALD  , ONLY : XMALD 

IMPLICIT NONE

REAL(KIND=JPRB):: ZSPS (NSPEC2),  ZSPQ  (NSPEC2)

REAL(KIND=JPRB):: ZREE  (NGPTOT)
REAL(KIND=JPRB):: ZPSF  (NGPTOT)
REAL(KIND=JPRB):: ZPVF  (NGPTOT)
REAL(KIND=JPRB):: ZDPF  (NGPTOT)
REAL(KIND=JPRB):: ZPDRYF(NGPTOT)

REAL(KIND=JPRB):: ZPVFG  (NGPTOTG)
REAL(KIND=JPRB):: ZPDRYFG(NGPTOTG)

INTEGER(KIND=JPIM):: IOPROC,IFLD,JLEV,JROF

REAL(KIND=JPRB) :: ZAPDRYF, ZAPVF, ZDA, ZDB, ZWEIG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "diwrgrid.intfb.h"
#include "esperee.intfb.h"

! -------------------------------------------------

!*       1.    START OF ROUTINE.
!              -----------------

!*       1.1   INITIALISISATIONS
!               - - - - - - - - 
IF (LHOOK) CALL DR_HOOK('CORMASS3A',0,ZHOOK_HANDLE)
WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) '    ====================================== '
WRITE(NULOUT,*) '    =======  ENTRANCE IN CORMASS3A ======= '
WRITE(NULOUT,*) '    ====================================== '
WRITE(NULOUT,*) ' '

IOPROC=1 

IFLD=0

! One proc for IO

!     ------------------------------------------------------------------
!*       3.    COMPUTE THE TOTAL MASS OF WATER VAPOR.  (see SUSPECA*)
!              ------------------------------------------------------

WRITE(NULOUT,*) '  '
WRITE(NULOUT,*) ' COMPUTE THE TOTAL WATER VAPOR MASS '

ZSPS(1:NSPEC2)=SPSP(1:NSPEC2)
CALL ESPEREE(1,1,ZSPS,ZPSF)
ZPSF=EXP(ZPSF)
ZPVF= 0.0_JPRB
DO JLEV=1,NFLEVG

!        *** Compute pressure difference accross layers *** 

  ZDA=VAH(JLEV)-VAH(JLEV-1)
  ZDB=VBH(JLEV)-VBH(JLEV-1)
  DO JROF=1,NGPTOT
    ZDPF(JROF)=ZDA+ZDB*ZPSF(JROF)
  ENDDO

  ZSPQ(1:NSPEC2)=SPQ(JLEV,1:NSPEC2)
  CALL ESPEREE(1,1,ZSPQ,ZREE)

  DO JROF=1,NGPTOT
    ZPVF(JROF) = ZPVF(JROF) + ZREE(JROF)*ZDPF(JROF)
  ENDDO

ENDDO !JLEV
DO JROF=1,NGPTOT
  ZPDRYF(JROF) = ZPSF(JROF) -  ZPVF(JROF)
ENDDO

!     ------------------------------------------------------------------
!*       5.    COMPUTE AVERAGE CORRECTION OF MASS. (see ZODIGI, ZODIA)
!      -------------------------------------------------------

! Local version of RW

!IROF=0
!DO JGL=1,NLSTLAT(MY_REGION_NS)-NFRSTLOFF
!  IGLG=MYLATS(JGL)
!  DO JLON=1,NONL(NPTRFLOFF+JGL,MY_REGION_EW)
!    IROF=IROF+1
!    ZRW(IROF)=RW(IGLG)
!  ENDDO
!ENDDO

IF (MYPROC == IOPROC) THEN

  WRITE(NULOUT,*) ' COMPUTE AVERAGE CORRECTION OF MASS '

! Global

  ZAPVF  = 0.0_JPRB
  ZAPDRYF= 0.0_JPRB

! Receive

  IFLD=IFLD+1
  ZPVFG(1:NGPTOT)=ZPVF
  CALL DIWRGRID (IOPROC,IFLD,1,NGPTOTG,ZPVFG)  
  IFLD=IFLD+1
  ZPDRYFG(1:NGPTOT)=ZPDRYF
  CALL DIWRGRID (IOPROC,IFLD,1,NGPTOTG,ZPDRYFG)  
!  IFLD=IFLD+1
!  ZRWG(1:NGPTOT)=ZRW
!  CALL DIWRGRID (IOPROC,IFLD,1,NGPTOTG,ZRWG)
!  IFLD=IFLD+1
!  ZGMG(1:NGPTOT)=GM
!  CALL DIWRGRID (IOPROC,IFLD,1,NGPTOTG,ZGMG)

!  IROF=0
!  DO JGL=1,NDGL
!  DO JLON=1,NLOENG(JGL)
!    IROF=IROF+1
!    ZNLG(IROF)=REAL(NLOENG(JGL),JPRB)
!  ENDDO
!  ENDDO

  ZWEIG=1./REAL(NGPTOTG,JPRB)
  DO JROF=1,NGPTOTG
!   ZWEIG=ZRWG(JROF)/ZGMG(JROF)**2/ZNLG(JROF)
    ZAPVF  =ZAPVF   +   ZPVFG(JROF)*ZWEIG
    ZAPDRYF=ZAPDRYF + ZPDRYFG(JROF)*ZWEIG
  ENDDO


  WRITE (NULOUT,*) '  '
  WRITE (NULOUT,'(1X,'' MEAN DRIVER DRY PRESSURE '',f19.8)') ZAPDRYF
  WRITE (NULOUT,'(1X,'' MEAN DRIVER WATER PRESSURE '',f19.8)') ZAPVF
  WRITE (NULOUT,*) '  '
  XMALD=ZAPDRYF

ELSE

! Send

  IFLD=IFLD+1
  CALL DIWRGRID (IOPROC,IFLD,0,NGPTOT,ZPVF)  
  IFLD=IFLD+1
  CALL DIWRGRID (IOPROC,IFLD,0,NGPTOT,ZPDRYF)
! IFLD=IFLD+1
! CALL DIWRGRID (IOPROC,IFLD,0,NGPTOT,ZRW)
! IFLD=IFLD+1
! CALL DIWRGRID (IOPROC,IFLD,0,NGPTOT,GM)
  XMALD=0.0_JPRB

ENDIF ! IOPROC

WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) '    ====================================== '
WRITE(NULOUT,*) '    ==========  END OF CORMASS3A ========= '
WRITE(NULOUT,*) '    ====================================== '
WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) ' '

IF (LHOOK) CALL DR_HOOK('CORMASS3A',1,ZHOOK_HANDLE)
END SUBROUTINE CORMASS3A
