SUBROUTINE CNMI

!**** *CNMI*  - CONTROL NORMAL MODE INITIALIZATION

!     Purpose.
!     --------

!        To control the normal mode initialization.

!**   Interface.
!     ----------
!        *CALL* *CNMI

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        *CMAC*     -  NORMAL-MODE INITIALIZATION

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Bill Heckley  *ECMWF*

!     Modifications.
!     --------------
!        Original : 90-04-11
!        P. Marquet : 91-10-15 : CALLS to SUTIDE AND UPDTIDE activated.
!        P. Marquet : 91-11-14 : restore the CALL SUSPEC (diab. NNMI purpose)
!                                (it was desactivated in cycle 7...)
!        cycle 8    : 91-12-03 : CALL SUSPEC replaced by CALL SUINIF(0)
!        ????????   : 93-??-?? : spectral arrays saved on NSAVSP
!        D. Giard   : 93-05-04 : (temporary?) correction of bug in I/O on NSAVSP
!        D. Giard   : 93-05-24 : CACUMTND --> CNTACTND
!        D. Giard   : 99-07-19 LINITER moved from YOMNMIA to YOMINI
!        M.Hamrud   : 03-02-02 Remove diabatic initialization
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMINI   , ONLY : LINITER
USE YOMNMIA  , ONLY :     LSTDNMI

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cmac.intfb.h"
!      -----------------------------------------------------------

!*       1.    SET SWITCHES FOR INITIALIZATION.
!              --------------------------------

IF (LHOOK) CALL DR_HOOK('CNMI',0,ZHOOK_HANDLE)
LINITER = .TRUE.

!      -----------------------------------------------------------

!*       2.    NORMAL-MODE INITIALIZATION.
!              ---------------------------

!     Project the contents of the SP3 arrays onto normal-modes,
!     initialize and project initialized fields back into spectral space.

CALL CMAC

!      -----------------------------------------------------------

!*       3.    RESET MODEL SWITCHES TO ORIGINAL SETTINGS.
!              ------------------------------------------

LINITER = .FALSE.
LSTDNMI = .FALSE.
!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CNMI',1,ZHOOK_HANDLE)
END SUBROUTINE CNMI
