RECURSIVE SUBROUTINE CNT0

USE PARKIND1  ,ONLY : JPIM     ,JPRB     ,JPIB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF    ,NPROC    ,&
 & LFPART2  ,LOPDIS  ,LOUTPUT  ,LMPOFF  
USE YOMCT3   , ONLY : NSTEP
USE YOMLUN   , ONLY : NULOUT
USE IOSTREAM_MIX , ONLY : CLOSE_IOSTREAM, Y_IOSTREAM_FDB, IOSTREAM_STATS
USE YOMMP    , ONLY : NOUTTYPE
USE YOMTIM   , ONLY : RSTART   ,RVSTART  ,RTIMEF
USE YOMLCZ   , ONLY : LJACDAV
USE YOMFPCT0 , ONLY : NFPCT0
USE YOMVAR   , ONLY : LFDBERR
USE MPL_MODULE  , ONLY : MPL_BARRIER, MPL_END
USE YOMSIG,   ONLY : RESTART

!**** *CNT0*  - Routine which controls the job at level 0.

!     Purpose.
!     --------
!           Controls the job at level 0, the lowest level. The parameter
!       NCONF decides which path to follow.

!**   Interface.
!     ----------
!        *CALL* *CNT0

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        D. Giard     : 83-06-15 closing NMI MIO files
!        R. El Khatib : 94-03-02 prepare inplementation of NCONF=927
!        M. Hamrud    : 94-05-09  SDS statistics
!        D. Giard     : 94-07-26 closing matrix files
!        MPP Group    : 95-10-01 Support for Distributed Memory version
!        M. Deque     : 97-06-01 New config 903
!        D. Dent      : 97-10-01 add final timing reports
!        R. El Khatib : 99-03-09 LFPART2=NFPCT0.EQ.0
!        F. Taillefer : 99-07-15 new config 931
!        D. Dent      : 00-09-01  MPE to MPL conversion
!        M. Hamrud    : 09-08-01  Option removal
!        R. El Khatib : 01-08-07 Pruning options
!        F. Taillefer : 03-04-15 new config 932
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        M. Drusch    : 17-01-07  new config 302 / simplified EKF for soil moisture
!        K. Yessad    : 15-Sep-08 prune conf 951.
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIB), EXTERNAL :: GETHWM, GETRSS, GETCURHEAP, GETSTK, GETMAXSTK

INTEGER(KIND=JPIM) ::  ICONFI

LOGICAL :: LLEND
LOGICAL :: LLFIRSTCALL=.TRUE.

REAL(KIND=JPRB) :: ZCT, ZVT, ZWT, ZRATIO
CHARACTER (LEN = 9) ::  CLCONF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cad1.intfb.h"
#include "can1.intfb.h"
#include "cgr1.intfb.h"
#include "cnt1.intfb.h"
#include "csekf1.intfb.h"
#include "cormass.intfb.h"
#include "cprep1.intfb.h"
#include "cprep5.intfb.h"
#include "cseaice.intfb.h"
#include "ctl1.intfb.h"
#include "cun1.intfb.h"
#include "cun3.intfb.h"
#include "cva1.intfb.h"
#include "freemem.intfb.h"
#include "gstats_output_ifs.intfb.h"
#include "incli0.intfb.h"
#include "inclitc.intfb.h"
#include "opdis.intfb.h"
#include "ppclose.intfb.h"
#include "ppflush.intfb.h"
#include "ppstat.intfb.h"
#include "su0yoma.intfb.h"
#include "su0yomb.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CNT0',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------
!*       1.    Initialize level 0 commons.
!              ---------------------------

!*       1.0   Statistics gathering

IF(LLFIRSTCALL) THEN
  CALL GSTATS(0,0)
  LLFIRSTCALL = .FALSE.
ENDIF

!*       1.1   Control post-processing

LFPART2=NFPCT0 == 0

!*       1.2   Setups up to and including main array allocations

CALL SU0YOMA

!*       1.3   Setups after main array allocations

CALL SU0YOMB

!     ------------------------------------------------------------------
!*       2.    Decide on configuration of job.
!              -------------------------------

ICONFI=IABS(NCONF)/100

!*       2.0   Integration job.

IF(ICONFI == 0)THEN
  WRITE(UNIT=NULOUT,FMT='('' INTEGRATION JOB'',2I6)')ICONFI,NCONF
  CALL CNT1

!*       2.1   Variational job.

ELSEIF(ICONFI == 1)THEN
  IF (LJACDAV) THEN
    WRITE(UNIT=NULOUT,&
     & FMT='('' HESSIAN SINGULAR VECTOR JOB '',2I6)')&
     & ICONFI,NCONF  
    CALL CUN3
  ELSE
    WRITE(UNIT=NULOUT,FMT='('' VARIATIONAL JOB'',2I6)')ICONFI,NCONF
    CALL CVA1
  ENDIF

!*       2.2   2D integration job.

ELSEIF(ICONFI == 2)THEN
  WRITE(UNIT=NULOUT,FMT='('' 2D INTEGR.  JOB'',2I6)')ICONFI,NCONF
  CALL CNT1

!*       2.3   Kalman Filter / soil moisture analysis

ELSEIF(ICONFI == 3)THEN
  WRITE(UNIT=NULOUT,FMT='('' SEKF surface analysis'',2I6)')ICONFI,NCONF
  CALL CSEKF1

!*       2.4   Test of the adjoint.

ELSEIF(ICONFI == 4)THEN
  WRITE(UNIT=NULOUT,FMT='('' TEST OF THE ADJOINT'',2I6)')ICONFI,NCONF
  CALL CAD1

!*       2.5   Test of the tangent linear.

ELSEIF(ICONFI == 5)THEN
  WRITE(UNIT=NULOUT,FMT='('' TEST OF THE TANGENT'',2I6)')ICONFI,NCONF
  CALL CTL1

!*       2.6   Search for most unstable modes.

ELSEIF(ICONFI == 6)THEN
  WRITE(UNIT=NULOUT,FMT='('' UNSTABLE MODES     '',2I6)')ICONFI,NCONF
  CALL CUN1

!*       2.7   Optimal Interpolation job.

ELSEIF(ICONFI == 7)THEN
  WRITE(UNIT=NULOUT,FMT='('' OPTIMAL INTER JOB'',2I6)')ICONFI,NCONF
  CALL CAN1

!*       2.8   Sensitivity job

ELSEIF(ICONFI == 8)THEN
  WRITE(UNIT=NULOUT,FMT='('' SENSITIVITY JOB'',2I6)')ICONFI,NCONF
  CALL CGR1

!*       2.9   Setting up the initial conditions.

ELSEIF(ICONFI == 9)THEN
  IF (NCONF < 950) THEN
    WRITE(UNIT=NULOUT,FMT='('' SETTING UP INITIAL CONDITIONS''&
     & ,2I6)')ICONFI,NCONF  
    IF (NCONF == 940) THEN
      CALL CORMASS
    ELSEIF (NCONF == 923) THEN
      CALL INCLI0
    ELSEIF (NCONF == 903) THEN
      CALL CPREP5
    ELSEIF (NCONF == 931) THEN
      CALL INCLITC
    ELSEIF (NCONF == 932) THEN
      CALL CSEAICE
    ELSE
      CALL CPREP1
    ENDIF
  ENDIF
ELSE
  WRITE(UNIT=NULOUT,FMT='('' ERROR CNT0  JOB'',2I6)')ICONFI,NCONF
  CALL ABOR1('CALLED IN CNT0')
ENDIF
!     ------------------------------------------------------------------
!*       3.    RUN DOWN
!              --------

!*       3.4   CLOSE FDB AND OTHER POST PROCESSING FILES

IF (LFDBERR .OR. NOUTTYPE==2) THEN
  CALL CLOSE_IOSTREAM(Y_IOSTREAM_FDB)
ENDIF
CALL PPFLUSH
CALL PPCLOSE
CALL PPSTAT

!*       3.6 STATISTICS

CALL USER_CLOCK(PELAPSED_TIME=ZWT,PVECTOR_CP=ZVT,PTOTAL_CP=ZCT)
ZCT=ZCT-RSTART
ZVT=ZVT-RVSTART
ZWT=ZWT-RTIMEF
!  avoid divide by zero
IF(ABS(ZWT) <= TINY(ZWT)) THEN
  ZRATIO=1.0
ELSE
  ZRATIO=ZCT/ZWT
ENDIF
CLCONF='000000000'
WRITE(NULOUT,'('' NSTEP ='',I6,'' CNT0   '',A9,4F8.3,5(1x,I15))')&
 & NSTEP,CLCONF,ZVT,ZCT,ZWT,ZRATIO,GETHWM(),GETRSS(),GETCURHEAP(),GETSTK(),GETMAXSTK()  

IF (LOPDIS) THEN
  RSTART=RSTART+ZCT
  RVSTART=RVSTART+ZVT
  RTIMEF=RTIMEF+ZWT
  CALL OPDIS(CLCONF,'CNT0    ',ZCT,ZVT,ZWT,RSTART,RVSTART,RTIMEF)
ENDIF
CALL IOSTREAM_STATS

LLEND=.NOT.LFPART2
IF (LLEND) THEN
  CALL GSTATS(0,1)
  CALL GSTATS_OUTPUT_IFS
ENDIF

WRITE(NULOUT,*) ' *** END CNT0 *** '

IF(LLEND.AND..NOT.LMPOFF) THEN
!           free all still allocated (fortran) memory ... including NPRCIDS(:)
  CALL FREEMEM()

!           gather memory usage statistics
  CALL getmemstat(NULOUT, 'CNT0')

  IF (NPROC > 1) THEN
    IF(LOUTPUT) THEN
      CLOSE(NULOUT)
    ENDIF
    CALL MPL_BARRIER(CDSTRING='CNT0:')
  ENDIF
  CALL MPL_END()
ENDIF

IF(RESTART == 'R') THEN
  STOP 30
ENDIF

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CNT0',1,ZHOOK_HANDLE)
END SUBROUTINE CNT0
