SUBROUTINE CNT1

!**** *CNT1*  - Controls integration job at level 1.

!     Purpose.
!     --------
!           Controls the integration job at level 1.

!**   Interface.
!     ----------
!        *CALL* *CNT1

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        SU1YOM  - initialize level 1 commons
!        SUOBSC1 - set up observations
!        CNT2    - control integration job at level 2
!        SCREEN  - observation screening
!        SACMAC1 - save CMA files and evaluate cost function 
!        PERTOBS - perturb observations

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!      J.N.Thepaut: 87-10-15  comparison to observations during forecast.
!      H. Jarvinen: 95-09-15  observation screening.
!      H. Jarvinen: 95-10-02  compress CMA file for assimilation.
!      J.N.Thepaut: 95-12-18  disable setup of obs. in case of postpro.
!     R. El Khatib: 02-11-12  Pruning LREFFP
!      D. Dee     : 04-03-03  Variational bias correction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        M.Fisher : 21-10-04  Perturb observations
!        P.Burton :  29-09-04 P.Burton LVARBC cleanup
!        Dick Dee : 01-03-08  New VarBC setup
!        Y.Tremolet    17-Nov-2008 Jc-DFI diagnostics in outer loop
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LOBS, LOBSC1, LSCREEN, L_SCREEN_CALL
USE YOMVAR   , ONLY : LVARBC, LJCDFI
USE YOMTVRAD , ONLY : SATGRP_TABLE
USE YOMSCC,    ONLY : LPERTURB
USE YOMLUN,    ONLY : NULOUT
USE VARBC_SETUP, ONLY : VARBC_SETUP_TRAJ, VARBC_STATS, VARBC_PRINT, VARBC_SAVE

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cnt2.intfb.h"
#include "dealshu.intfb.h"
#include "dealxmo.intfb.h"
#include "pertobs.intfb.h"
#include "sacmac1.intfb.h"
#include "screen.intfb.h"
#include "su1yom.intfb.h"
#include "suobsc1.intfb.h"
#include "sualldfi.intfb.h"
#include "suprepjcdfi.intfb.h"
#include "evjcdfi.intfb.h"
#include "dealldfi.intfb.h"

!      -----------------------------------------------------------------

!*       1.    Initialize YOMCT1.
!              ------------------

IF (LHOOK) CALL DR_HOOK('CNT1',0,ZHOOK_HANDLE)
CALL SU1YOM
!      -----------------------------------------------------------------
!*       2.    set up observations (LOBSC1)
!              ----------------------------

IF (LOBSC1) CALL SUOBSC1

IF (LJCDFI) THEN
  CALL SUPREPJCDFI(NULOUT)
  CALL SUALLDFI(NULOUT)
ENDIF

!      -----------------------------------------------------------------
!*       2.5   set up VarBC (LOBSC1)
!              ---------------------

IF(LOBSC1 .AND. LVARBC) THEN
  CALL VARBC_SETUP_TRAJ
ENDIF

!      -----------------------------------------------------------------

!*       3.    Call level 2 control routine.
!              -----------------------------

CALL CNT2
CALL DEALXMO()


!      -----------------------------------------------------------------
!*       3.5   perturb observations (LOBSC1)
!              ----------------------------

IF (LOBSC1 .AND. LPERTURB) THEN
  CALL PERTOBS
ENDIF

!      -----------------------------------------------------------------

!*       4.    Observation screening (LSCREEN)
!              -------------------------------

IF (LSCREEN.AND.L_SCREEN_CALL) THEN
  CALL GSTATS(45,0)
  CALL SCREEN
  CALL GSTATS(45,1)
ENDIF

!      -----------------------------------------------------------------

!*       5.    Save VarBC information for cycling
!              ----------------------------------

IF (LOBS .AND. LVARBC) THEN
  CALL VARBC_STATS()
  CALL VARBC_SAVE()
  CALL VARBC_PRINT(1)
ENDIF

!      -----------------------------------------------------------------

!*       6.    save CMA files and evaluate cost function (LOBSC1)
!              --------------------------------------------------

IF(LSCREEN.OR.LOBSC1) CALL GSTATS(46,0)

IF (LJCDFI) CALL EVJCDFI(NULOUT,.true.)

IF(LOBSC1) THEN
  CALL GSTATS(1814,0)
  CALL DEALSHU()
  CALL RTTOV_DEALLOC_ALLCOEF(SIZE(SATGRP_TABLE))
  CALL DEALSUR()
  CALL GSTATS(1814,1)
  CALL SACMAC1
ENDIF

IF (LJCDFI) CALL DEALLDFI(NULOUT)

IF(LSCREEN.OR.LOBSC1) CALL GSTATS(46,1)

!      -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CNT1',1,ZHOOK_HANDLE)
END SUBROUTINE CNT1
