SUBROUTINE CNT3TL

!**** *CNT3TL*  - LEVEL 3 CONTROL ROUTINE OF THE MODEL

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *CNT3TL

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        Modified : 91-09-25 p.c and JN.T - remove useless logic
!        Modified : 93-04-13 L. Labbe - Initialize grid point orography
!        Modified : 94-03-09 L. Labbe - Use of NMI after the run (821)
!        Modified : 94-08-05 J.N. Thepaut - Filter for the variational jobs.
!        Bouttier : 00-06-12 protect against clouds
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMNMIA  , ONLY : NVMOD    ,NVMODPP  ,LNMIRQ   ,LNPROJ
USE YOMMP    , ONLY : NUMVMO
USE YOEPHY   , ONLY : LEPHYS
USE YOEPHLI , ONLY  : LPHYLIN
USE GFL_SUBS_MOD , ONLY : DEACT_CLOUD_GFL

IMPLICIT NONE

LOGICAL :: LLPHYLIN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cnmitl.intfb.h"
#include "cnt4tl.intfb.h"
#include "csta.intfb.h"
#include "fltmode.intfb.h"
#include "su3yom.intfb.h"

!      -----------------------------------------------------------

!*       1.    Initialize LEVEL 3 COMMONS, check and start.
!              --------------------------------------------

IF (LHOOK) CALL DR_HOOK('CNT3TL',0,ZHOOK_HANDLE)
WRITE(UNIT=NULOUT,FMT='('' START CNT3TL'')')
CALL SU3YOM
IF (LEPHYS) THEN
  CALL DEACT_CLOUD_GFL
ENDIF

!IF( YGFL%NUMGPFLDS>0 ) THEN
!  CALL ABOR1('ERROR CNT3TL - upper air GP fields not yet supported by TL model')
!ENDIF
CALL CSTA
LLPHYLIN=LPHYLIN
LPHYLIN=.TRUE.

!      -----------------------------------------------------------

!*       4.    FILTERING. 
!              ----------

!*       4.1   NORMAL MODE INITIALIZATION.
!              ---------------------------

IF (LNMIRQ) THEN
  CALL CNMITL
ELSEIF (LNPROJ) THEN
  CALL FLTMODE(NVMOD,NVMODPP,NUMVMO)
ENDIF

!      -----------------------------------------------------------

!*       5.    INTEGRATION.
!              ------------

CALL CNT4TL
!      -----------------------------------------------------------

LPHYLIN=LLPHYLIN
WRITE(UNIT=NULOUT,FMT='('' END CNT3TL'')')

IF (LHOOK) CALL DR_HOOK('CNT3TL',1,ZHOOK_HANDLE)
END SUBROUTINE CNT3TL
