SUBROUTINE CNT4TL

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMDIMO  , ONLY : NOBTOTG
USE YOMLUN   , ONLY : NULOUT
USE YOMVAR   , ONLY : NSIMU    ,NSIM4D   ,NFRREF   ,NFRANA   ,&
 & NFRGRA   ,NSIM4DL  ,LTWINC   ,LTWANA   ,LJCDFI   ,LTRREF   ,&
 & LAVARC   ,LMODERR  , LFCOBSTEST
USE YOMHCP   , ONLY : NHECP
USE YOMCT0   , ONLY : LFPART2   ,LNHDYN   ,NCONF    ,NSTOP    ,NFRPOS   ,NFRISP   ,&
 & NFRHIS   ,NFSRHIS  ,NFRGDI   ,NFRSDI   ,NFRDHFG  ,NFRDHFZ  ,&
 & NFRDHFD  ,NFRDHP   ,NPRINTLEV,NFRMASSCON,&
 & LFDBOP   ,LTWOTL   ,LSITRIC  ,LOBS , LELAM
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT2   , ONLY : NSTAR2   ,NSTOP2
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP, TDT, LSTRHD, LSIDG, NSITER, NCURRENT_ITER
USE YOMECTAB , ONLY : NTSLVSTST
USE YOMLDDH  , ONLY : LHDOUFG  ,LHDOUFZ  ,LHDOUFD  ,LHDOUP
USE YOMCFU   , ONLY : NFRCFU
USE YOMXFU   , ONLY : NFRXFU   ,NFRRAZ
USE YOMRES   , ONLY : NFRRES
USE YOMVRTL  , ONLY : L131TL,LOBSTL
USE YOMSKF   , ONLY : LTWSKFY
USE YOMMP    , ONLY : NOUTTYPE, NPSP
USE YOMIO    , ONLY : LPPTSF
USE YOMTIM   , ONLY : RSTART   ,RVSTART  ,RTIMEF
USE YOMTNEWT , ONLY : LIDMODEL
USE YOMTLEVOL, ONLY : LTLEVOL
USE YEMCT0   , ONLY : LE0COTA  ,NFRLSG
USE YEMJK    , ONLY : LEJK
USE YOMDFI   , ONLY : NSTDFI
USE YOMSP    , ONLY: SPSP, SPA3, SPD2D, SP5A3, SP5A2, SPA1, SP5A1
USE YOMGLOBS , ONLY : LPREMPOBS
USE YOMLCZ   , ONLY: L_SOS, L_EOFS, NSTEPS_PER_STATE, YSTATE_VECTOR_4D, LOCNORM
USE TRAJECTORY_MOD, ONLY: LTRAJGP, GET_TRAJ_SPEC
USE YOMMODEL_ERROR,ONLY: NTYPE_MODERR, NDIM_MODERR, MSTEPERR, NPRTMODERR, &
                    & SPMODERR, GPMODERR, SPMODERR_HYB, GPMODERR_HYB, &
                    & N_COUPLED_WINDOWS
USE SPECTRAL_FIELDS_MOD
USE CONTROL_VECTORS

!**** *CNT4TL*  - Controls tangent linear model at level 4

!     Purpose.
!     --------
!     Controls the temporal loop

!**   Interface.
!     ----------
!        *CALL* *CNT4TL

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by CNT3TL.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

! Modifications
! -------------
!   Original : 87-10-15
!   =04-08-91= Rationalisation of calls to STEPO Philippe Courtier
!   08/03/1993 Additional horizontal diffusion on not stretched sphere
!              (K. YESSAD).
!   Modified by K. YESSAD    : OCTOBER 1993. Semi-implicit scheme with
!                              not reduced divergence.
!   Modified by J-N.Thepaut : 10-06-94 Introduction of obs management in
!                             the TL code
!   K. YESSAD (APRIL 1994): Unified horizontal diffusion scheme.
!   M.Hamrud :95-10-30 (new form of CLCONF)
!   K. YESSAD (NOV 1995): removal of HDC horizontal diffusion scheme.
!   F.Rabier :96-10-09 (post-processing of background+increments)
!   M.Fisher :97-01-31 write increment at intermediate time
!   M. TOLSTYKH  (OCT 1996): Fourier space horizontal diffusion
!   C. Temperton 97-02-24: modified first timestep when XIDT>0
!   C. Temperton 98-07-16: two-time-level scheme
!   C. Temperton 99-08-16: removed redundant RKROMA
!   P. GAUTHIER  99-07-21: introduction of JC-DFI
!   C. Soci:00-04-05 array for large scale gradients timesteps managing
!   Y.Tremolet   00-08-11: use module TRAJECTORY
!   J.Vivoda (03-2002) PC schemes for NH dynamics (LPC_XXXX keys)
!   G. Desroziers : 02-09-16 also treat Aladin mean wind trajectory
!   01-Oct-2003 M. Hamrud   CY28 Cleaning
!   25-Feb-2004 C. Fischer  merge Aladin into Arpege/IFS cnt4tl
!      Dec-2003 K. Yessad   cleaning in horizontal diffusion.
!   10-Jun-2004 J. Masek    NH cleaning (LPC_NOTR)
!   01-Jul-2004 K. Yessad   Cleaning in PC scheme, and paragraph numbering
!      harmonisation with the direct code.
!   R. El Khatib : 05-02-21 specific frequency array to reset instant. fluxes
!   Y.Tremolet    18-Mar-2004 Model error changes
!        V.Guidard&C.Fischer: 01-Apr-2004 Introduction of Jk event
!   K. Yessad     08-Feb-2005 SI scheme for ARPEGE NH model
!   K. Yessad     21-Apr-2005 cleanings for SPNORM-printings.
!   B. Chapnik    02-May-2007 fgat : temporal loop considered only if tstep corresponds to tslot
!                                    and call to stepo only when needed
!   Y. Seity     11-Jan-2008 : add ISHISTS for surfex output files   
!   Y.Tremolet    27-Nov-2008 Jc-DFI for long windows
!   F. Vana   13-Jan-2009 : Removed special specHD setup when LSLHD
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER (LEN = 10) ::  CLTIMEOD,CLDAT(3)


CHARACTER (LEN = 9) ::  CLCONF
!   1 : configuration of WRTRA      write the trajectory
!   2 : configuration of LTINV      inverse Legendre transform
!   3 : configuration of FTINV      inverse Fourier transform
!   4 : configuration of GPC        grid point computations
!   5 : configuration of POS        post processing
!   6 : configuration of OBS        comparison to observations
!   7 : configuration of FTDIR      direct Fourier transform
!   8 : configuration of LTDIR      direct Legendre transform
!   9 : configuration of SPC        spectral space computations

! - IPOSTS : ARRAY CONTAINING POST-PROCESSING TIME STEPS
INTEGER(KIND=JPIM) :: IPOSTS(0:NSTOP/NFRPOS)
! - IPISPS : ARRAY CONTAINING ISP (Animation !) TIME STEPS
INTEGER(KIND=JPIM) :: IPISPS(0:NSTOP/NFRISP)
! - IHISTS : ARRAY CONTAINING TRAJECTORY TIME STEPS
INTEGER(KIND=JPIM) :: IHISTS(0:NSTOP/NFRHIS)
! - ISHISTS : ARRAY CONTAINING TRAJECTORY TIME STEPS FOR SURFACE
INTEGER(KIND=JPIM) :: ISHISTS(0:NSTOP/NFSRHIS)
! - IRESTS : ARRAY CONTAINING RESTART    TIME STEPS
INTEGER(KIND=JPIM) :: IRESTS(0:NSTOP/NFRRES)
! - IGDITS : GRID POINT DIAGNOSTICS TIME STEPS
INTEGER(KIND=JPIM) :: IGDITS(0:NSTOP/NFRGDI)
! - ISDITS : SPECTRAL DIAGNOSTICS TIME STEPS
INTEGER(KIND=JPIM) :: ISDITS(0:NSTOP/NFRSDI)
! - ICFUTS : CONTROLS ACCUMULATED FLUX WRITE-UP
INTEGER(KIND=JPIM) :: ICFUTS(0:NSTOP/NFRCFU)
! - IDHFGTS : WRITE OUT TIME STEPS FOR GLOBAL MEANS DDH
INTEGER(KIND=JPIM) :: IDHFGTS(0:NSTOP/NFRDHFG)
! - IDHFZTS : WRITE OUT TIME STEPS FOR ZONAL MEANS DDH
INTEGER(KIND=JPIM) :: IDHFZTS(0:NSTOP/NFRDHFZ)
! - IDHFDTS : WRITE OUT TIME STEPS FOR LIMITED AREAS DDH
INTEGER(KIND=JPIM) :: IDHFDTS(0:NSTOP/NFRDHFD)
! - IDHPTS : PAS DE TEMPS DE SORTIES IMPRIMEES DES DDH
INTEGER(KIND=JPIM) :: IDHPTS(0:NSTOP/NFRDHP)
! - IXFUTS : CONTROLS INSTANTANEOUS FLUX WRITE-UP
INTEGER(KIND=JPIM) :: IXFUTS(0:NSTOP/NFRXFU)
! - IRAZTS : CONTROLS INSTANTANEOUS FLUX RESET
INTEGER(KIND=JPIM) :: IRAZTS(0:NSTOP/NFRRAZ)
! - IANATS : ARRAY CONTAINING ITERATIONS FOR WHICH THE ANALYSIS
!        MUST BE WRITTEN ON FILE, POST-PROCESSED TO MODEL LEVELS.
INTEGER(KIND=JPIM) :: IANATS(0:NSIM4DL/NFRANA)
! - IGRATS : ARRAY CONTAINING ITERATIONS FOR WRITE OUT OF GRADIENTS
INTEGER(KIND=JPIM) :: IGRATS(0:NSIM4DL/NFRGRA)
! - IREFTS : ARRAY CONTAINING ITERATIONS FOR WRITE OUT OF REFERENCE STATE
INTEGER(KIND=JPIM) :: IREFTS(0:NSTOP/NFRREF)
! - IMASSCONS: ARRAY CONTAINING mass conservation fixup time steps
INTEGER(KIND=JPIM) :: IMASSCONS(0:NSTOP/NFRMASSCON)
! - ILSGTS : ARRAY CONTAINING ITERATIONS FOR LBC GRADIENTS I/O
INTEGER(KIND=JPIM) :: ILSGTS(0:NSTOP/NFRLSG)
! - IHECPS : ARRAY CONTAINING ITERATIONS FOR AVARC DIAGNOSTIC
INTEGER(KIND=JPIM) :: IHECPS(0:NSTOP)

REAL(KIND=JPRB), ALLOCATABLE :: ZSPA3(:,:,:), ZSPSP(:)

INTEGER(KIND=JPIM) :: IACTIM,IDIGLST,IOUTTYPE, ISTOP, J, JS, JSTEP, I, IAVARE &
                   &, JSITER, JJ, IWEAK, IACTIM2, ipersub, istep, isub
INTEGER(KIND=JPIM) :: IHOUR, IMIN, IVALUES(9)
REAL(KIND=JPRB) :: ZSEC, ZT1, ZT2

LOGICAL :: LL3DVAR, LLFDBOP, LLMLPP, LLPLPP, LLSLOT, llevs
LOGICAL :: LLSPNRM,LLGPNRM, LLSLOT2, LLFGATLAM

REAL(KIND=JPRB) :: ZCT, ZVT, ZWT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!      -----------------------------------------------------------------

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"
#include "add5to3.intfb.h"
#include "avarc.intfb.h"
#include "cain.intfb.h"
#include "cainin.intfb.h"
#include "chkobtim.intfb.h"
#include "digfil.intfb.h"
#include "elsin0ta.intfb.h"
#include "evarjk.intfb.h"
#include "gpnorm_gfl.intfb.h"
#include "iopack.intfb.h"
#include "lcnortl.intfb.h"
#include "moevar.intfb.h"
#include "monio.intfb.h"
#include "monvar.intfb.h"
#include "obsv.intfb.h"
#include "obsvtl.intfb.h"
#include "ppclose.intfb.h"
#include "ppflush.intfb.h"
#include "post_obshor.intfb.h"
#include "spnorm.intfb.h"
#include "stepo.intfb.h"
#include "stepotl.intfb.h"
#include "sbs5to3.intfb.h"
#include "suhdu.intfb.h"
#include "suheg.intfb.h"
#include "sunhsi.intfb.h"
#include "sutric.intfb.h"
#include "updtim.intfb.h"
#include "savmoderr.intfb.h"
#include "gridpoint_norm.intfb.h"
#include "add_moderr_tl.intfb.h"
#include "weak_constraint_tl.intfb.h"
!      -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CNT4TL',0,ZHOOK_HANDLE)

!      -----------------------------------------------------------------

!*       1.    Initialize.
!              -----------
CALL GSTATS(22,0)
WRITE(NULOUT,*)'START CNT4TL, NSIM4D=',NSIM4D

LL3DVAR=(NCONF == 131.AND.NSTOP == 0)

!!BC 3d-fgat-case
LLFGATLAM=(LELAM .AND. LIDMODEL)



!      -----------------------------------------------------------------

!*       2.    Prepare occurences of I/O events.
!              ---------------------------------

CALL MONIO(IPOSTS,IPISPS,IHISTS,ISHISTS,IGDITS,ISDITS,IDHFGTS,IDHFZTS,&
 & IDHFDTS,IDHPTS,ICFUTS,IXFUTS,IRESTS,IMASSCONS,IRAZTS)
CALL MONVAR(IREFTS,IANATS,IGRATS,IHECPS)

IAVARE=1
IF (LELAM) THEN
  CALL MOEVAR(ILSGTS)
ENDIF

!* events not implemented
DO JS=0,NSTOP/NFRDHFG
  IDHFGTS(JS)=0
ENDDO
DO JS=0,NSTOP/NFRDHFZ
  IDHFZTS(JS)=0
ENDDO
DO JS=0,NSTOP/NFRDHFD
  IDHFDTS(JS)=0
ENDDO
DO JS=0,NSTOP/NFRDHP
  IDHPTS(JS)=0
ENDDO
IF (NPRINTLEV >= 1) THEN
  WRITE(NULOUT,'('' POST-PROCESSING EVENTS, IPOSTS '')')
  WRITE(NULOUT,'(40I2)')IPOSTS
  WRITE(NULOUT,'('' HISTORY WRITE-UP, IHISTS '')')
  WRITE(NULOUT,'(40I2)')IHISTS
  WRITE(NULOUT,'('' AN WRITE-OUT EVENTS, IANATS '')')
  WRITE(NULOUT,'(40I2)') (IANATS(J),J=0,NSIMU/NFRANA)
  WRITE(NULOUT,'('' AVARC EVENTS  '')')
  WRITE(NULOUT,'(40I2)') (IHECPS(J),J=0,NSTOP)
ENDIF
IACTIM=1
IACTIM2=1

IF (NSIM4D<=2.OR.NSIM4D==NSIM4DL) THEN
  WRITE(NULOUT,*)'NORMS AT START CNT4TL'
  CALL SPNORM(0)
  CALL GPNORM_GFL
  llevs=(NPRTMODERR>=2)
  DO jj=1,NDIM_MODERR
    CALL SPECTRAL_NORM (SPMODERR(jj),'CNT4TL MODERR')
    CALL GRIDPOINT_NORM(GPMODERR(jj),'CNT4TL MODERR',llevs)
  ENDDO
ENDIF

!      -----------------------------------------------------------------

!*       3.    Tangent-linear integration.
!              ---------------------------

!*       3.1  Main temporal loop

IF(NSTOP2 == NSTOP)THEN
! For final time step (diagnostics, post-processing)
  ISTOP=NSTOP
ELSE
  ISTOP=NSTOP2-1
ENDIF

!     Temporal loop begins ...

TEMPORAL_LOOP: DO JSTEP=NSTAR2,ISTOP

IF (LLFGATLAM .AND. (ALL(NTSLVSTST /= JSTEP)) .AND. (JSTEP /= ISTOP)) CYCLE

  CALL USER_CLOCK(PTOTAL_CP=ZT1)

!        3.1.1  Time filtering constraint based on digital filter
!               is being computed.

  IF (LJCDFI) THEN
!   Step should be local to subwindow
    IF (N_COUPLED_WINDOWS>1) THEN
      isub=1
      istep=0
      IF (jstep>0) THEN
        ipersub=NSTOP/N_COUPLED_WINDOWS
        istep=MOD(jstep-1,ipersub)+1
        isub=(jstep-1)/ipersub+1
      ENDIF
    ELSE
      isub=1
      istep=JSTEP
    ENDIF
    IDIGLST=2*NSTDFI
    IF (NSIM4D==0.AND.N_COUPLED_WINDOWS>0) THEN
      write(nulout,*)'CNT4TL: Calling DIGFIL: jstep,isub,istep=',jstep,isub,istep
    ENDIF
    CALL DIGFIL(istep, IDIGLST, isub, SPA3, SPD2D, SPA1, .TRUE.)
  ENDIF

!*       3.2  Current value of the time step length

  IF(JSTEP == 0.OR.LTWOTL)THEN
    TDT=TSTEP
  ELSE
    TDT=2.0_JPRB*TSTEP
  ENDIF

!*       3.3  Reset of time dependant constants (clock,astronomy...)

  CALL UPDTIM(JSTEP,TDT,TSTEP,.FALSE.)

!*       3.4  Reset semi-implicit solver in the multilevel model, and the
!             additional horizontal diffusion operator.

  IF (.NOT.LELAM) THEN

    ! * SI scheme:
    IF (LNHDYN) THEN
      CALL SUNHSI(NULOUT,.FALSE.)
    ELSE
      IF (LSIDG) THEN
        CALL SUHEG
      ENDIF
      IF (LSITRIC.AND.(JSTEP == NSTAR2.OR.(JSTEP == 1.AND..NOT.LTWOTL))) THEN
        CALL SUTRIC
      ENDIF
    ENDIF

    ! * HD scheme:
    IF (LSTRHD) THEN
      ! * This call to SUHDU actually does something in a leap-frog scheme if
      !   TDT=2TSTEP for the current jstep, and TDT=TSTEP for the previous one.
      CALL SUHDU
    ENDIF

  ELSE

    ! * SI scheme:
    IF (LNHDYN .AND. (NSTOP > 0 .AND. .NOT.LFPART2)) THEN
      CALL SUNHSI(NULOUT,.FALSE.)
    ENDIF

    ! * HD scheme:

  ENDIF

!*       3.6  Reset YOMCT3

  NSTEP=JSTEP

!*       3.6.1  Nullify GT3

  IF (LELAM.AND.LE0COTA.AND.(NSTOP /= 0)) THEN
    CALL ELSIN0TA
  ENDIF

!*       3.8  Reset time dependent variables

  LLSPNRM=ISDITS(JSTEP/MAX(1,NFRSDI)) == 1.AND.MOD(JSTEP,MAX(1,NFRSDI)) == 0
  LLGPNRM=IGDITS(JSTEP/MAX(1,NFRGDI)) == 1.AND.MOD(JSTEP,MAX(1,NFRGDI)) == 0
  LLMLPP = LTWSKFY .AND. IHISTS(JSTEP/NFRHIS) == 1 &
   & .AND. MOD(JSTEP,NFRHIS) == 0

!*       3.8.2  Some diagnostics

  IF (LFCOBSTEST.AND.(NSIM4D<=4.OR.NSIM4D==NSIM4DL)) THEN
    WRITE(NULOUT,*)'LFSOBS NORMS AT START CNT4TL'
    CALL SPNORM(0)
    CALL GPNORM_GFL
  ENDIF
  IF(LLSPNRM) THEN
    WRITE(UNIT=NULOUT,FMT='('' NORMS AT NSTEP CNT4TL '',I4)')NSTEP
    CALL SPNORM(0)
  ENDIF
  IF (LLGPNRM) THEN
    CALL GPNORM_GFL
  ENDIF

  LLPLPP =IPOSTS(JSTEP/NFRPOS) == 1.AND.MOD(JSTEP,NFRPOS) == 0

  IF(IDHFGTS(JSTEP/NFRDHFG) == 1.AND.MOD(JSTEP,NFRDHFG) == 0) THEN
    LHDOUFG=.TRUE.
  ELSE
    LHDOUFG=.FALSE.
  ENDIF
  IF(IDHFZTS(JSTEP/NFRDHFZ) == 1.AND.MOD(JSTEP,NFRDHFZ) == 0) THEN
    LHDOUFZ=.TRUE.
  ELSE
    LHDOUFZ=.FALSE.
  ENDIF
  IF(IDHFDTS(JSTEP/NFRDHFD) == 1.AND.MOD(JSTEP,NFRDHFD) == 0) THEN
    LHDOUFD=.TRUE.
  ELSE
    LHDOUFD=.FALSE.
  ENDIF

  IF(IDHPTS(JSTEP/NFRDHP) == 1.AND.MOD(JSTEP,NFRDHP) == 0) THEN
    LHDOUP=.TRUE.
  ELSE
    LHDOUP=.FALSE.
  ENDIF

!*       3.8.6  Add the forcing that was saved during the adjoint integration

  IF (L_EOFS) THEN
    IF (MOD((JSTEP+1),NSTEPS_PER_STATE)==0 .AND. JSTEP+1 < NSTOP) THEN
      I = (JSTEP+1)/NSTEPS_PER_STATE
      ALLOCATE (ZSPA3(SIZE(SPA3,1),SIZE(SPA3,2),SIZE(SPA3,3)))
      ZSPA3(:,:,:) = SPA3(:,:,:)
      IF (NPSP==1) THEN
        ALLOCATE (ZSPSP(SIZE(SPSP)))
        ZSPSP(:)   = SPSP(:)
      ENDIF
      CALL CAIN (YSTATE_VECTOR_4D(I))
      SPA3(:,:,:) = SPA3(:,:,:) + ZSPA3(:,:,:)
      DEALLOCATE (ZSPA3)
      IF (NPSP==1) THEN
        SPSP(:)   = SPSP(:)   + ZSPSP(:)
        DEALLOCATE (ZSPSP)
      ENDIF
    ENDIF
  ENDIF

!*       3.8.7  Manage trajectory

  IF(.NOT.LTRAJGP) THEN
    IF (LTRREF) THEN
      CLCONF(1:1)='B'
    ELSE
      CALL GSTATS(15,0)
      CALL GET_TRAJ_SPEC(SP5A3,SP5A2,SP5A1,KSTEP=JSTEP)
      CALL GSTATS(15,1)
      CLCONF(1:1)='0'
    ENDIF
    IF(LL3DVAR) THEN
      CLCONF(2:2)='K'
    ELSE
      CLCONF(2:2)='E'
    ENDIF
    CLCONF(3:9)='0000000'
    LLSLOT=.FALSE.
    CALL CHKOBTIM(LLSLOT,IACTIM)
    IF(.NOT. LIDMODEL .OR. (LLSLOT .AND. LIDMODEL) ) THEN
    CALL STEPO(CLCONF)
    ENDIF
  ENDIF

!*       3.X  Weak Constraints 4D-Var

  IF (LMODERR) THEN
    iweak=MSTEPERR(NSTEP)
    IF (NTYPE_MODERR==1.AND.iweak>0) CALL WEAK_CONSTRAINT_TL
    IF (NTYPE_MODERR==4.AND.iweak>0) THEN
      CALL ADD_MODERR_TL(SPMODERR,GPMODERR,NSTEP)
      IF (LJCDFI.AND.N_COUPLED_WINDOWS>1) THEN
        ipersub=NSTOP/N_COUPLED_WINDOWS
        isub=NSTEP/ipersub+1
        istep=0
        IDIGLST=2*NSTDFI
        IF (NSIM4D==0) THEN
          write(nulout,*)'CNT4TL: Calling DIGFIL: estep,isub,istep=',nstep,isub,istep
        ENDIF
        CALL DIGFIL(istep, IDIGLST, isub, SPA3, SPD2D, SPA1, .TRUE.)
      ENDIF
    ELSEIF (NTYPE_MODERR==5 .AND.iweak>0) THEN
      IF (NSIM4D == NSIM4DL) THEN
        CALL ADD_MODERR_TL(SPMODERR,GPMODERR,NSTEP,SPMODERR_HYB,GPMODERR_HYB)
      ELSE
        CALL ADD_MODERR_TL(SPMODERR,GPMODERR,NSTEP)
      ENDIF
    ENDIF
  ENDIF

!*       3.9  Post-processing time step

  IF (LLPLPP .AND. (LTLEVOL .OR. NSIM4D==NSIM4DL &
   & .OR. (.NOT.LECMWF .AND. IANATS(NSIM4D/NFRANA) == 1 &
   & .AND.MOD(NSIM4D,NFRANA) == 0) )) THEN
    CLCONF(1:9)='A00000000'
    LLFDBOP=LFDBOP
    LFDBOP=.FALSE.
    IOUTTYPE=NOUTTYPE
    NOUTTYPE=1
    IF(LECMWF)THEN
      LTWINC=.TRUE.
      CALL IOPACK('A')
      LTWINC=.FALSE.
    ELSE
      LTWANA=.TRUE.
      CALL ADD5TO3
      CALL STEPOTL(CLCONF)
      LTWANA=.FALSE.
    ENDIF
    IF (JSTEP==0) THEN
      IF (LMODERR.AND.NTYPE_MODERR/=1 .AND. NTYPE_MODERR/=5) CALL SAVMODERR
    ENDIF
    IF (LTLEVOL) THEN
      CALL PPFLUSH
      CALL PPCLOSE
    ENDIF
    IF(.NOT.LECMWF) CALL SBS5TO3
    LFDBOP=LLFDBOP
    NOUTTYPE=IOUTTYPE
    LTWINC=.FALSE.
  ELSE
    LLPLPP=.FALSE.
  ENDIF

!*       3.9.4  Comparison to reference state (AVARC)
  IF(LAVARC.AND.IHECPS(JSTEP)== 1) THEN
    WRITE (NULOUT,*) 'JSTEP = ', 'avarc active: event = ',IAVARE
    IF(IAVARE > NHECP(0)) THEN
      CALL ABOR1('AVARC ERROR : IAVARE > NHECP(0) ')
    ENDIF
    CALL AVARC(IAVARE)
    IAVARE = IAVARE + 1
  ENDIF

 !* 3.9.5 Comparison to the global analysis
   ! Not managed in the 4DVAR framework
     IF (LELAM) THEN
      IF (LEJK.AND.(JSTEP==0)) CALL EVARJK
     ENDIF


!*       3.10  Ordinary time step (predictor if PC scheme).

  IF(L131TL.AND.JSTEP == 0) THEN
    CLCONF(1:1)='I'
  ELSEIF (LLMLPP) THEN
    CLCONF(1:1)='A'
  ELSE
    CLCONF(1:1)='0'
  ENDIF

  IF(JSTEP < NSTOP)THEN
    CLCONF(2:2)='A'
    CLCONF(3:3)='A'
    CLCONF(4:4)='A'
  ELSE
    CLCONF(2:2)='0'
    CLCONF(3:3)='0'
    CLCONF(4:4)='0'
  ENDIF

  CLCONF(5:5)='0'
  CLCONF(6:6)='0'

  IF(L131TL) THEN
    LLSLOT2=.FALSE.
    CALL CHKOBTIM(LLSLOT2,IACTIM2)
    IF(LLSLOT2) THEN
      IF(LL3DVAR) THEN
        CLCONF(2:2)='G'
        CLCONF(3:3)='B'
      ELSE
        CLCONF(2:2)='A'
        CLCONF(3:3)='A'
      ENDIF
      CLCONF(6:6)='V'
    ENDIF
  ENDIF

  IF(JSTEP < NSTOP)THEN
    CLCONF(7:7)='A'
    CLCONF(8:8)='A'

    NCURRENT_ITER=0

    IF (NSITER == 0) THEN
      CLCONF(9:9)='A'
    ELSEIF (NSITER > 0) THEN
      CLCONF(9:9)='T'
    ELSEIF(NSITER < 0) THEN
      CALL ABOR1('NEGATIVE NSITER NOT ALLOWED')
    ENDIF

  ELSE
    NCURRENT_ITER=0
    CLCONF(7:7)='0'
    CLCONF(8:8)='0'
    CLCONF(9:9)='0'
  ENDIF

! Identity model (I)
  IF (LIDMODEL) THEN
    IF (JSTEP/=0) THEN
      CLCONF(2:2)='0'
      CLCONF(3:3)='0'
    ENDIF
    CLCONF(4:4)='0'
    CLCONF(5:5)='0'
    CLCONF(7:7)='0'
    CLCONF(8:8)='0'
    CLCONF(9:9)='0'
  ENDIF

  IF(CLCONF(1:9)/='000000000') THEN
    CALL STEPOTL(CLCONF)
  ENDIF

!*       3.11  Corrector step.

  IF(JSTEP < NSTOP .AND. NSITER > 0) THEN

    ! Iteration step:

    DO JSITER = 1, NSITER-1
      NCURRENT_ITER=JSITER
      CLCONF(1:9)='0AAS00AAS'
      CALL STEPOTL(CLCONF)
    ENDDO

    ! Last step:

    NCURRENT_ITER=NSITER
    CLCONF(1:9)='0AAS00AAA'
    CALL STEPOTL(CLCONF)

  ENDIF

!*       3.12  MODERR

  IF (LMODERR.AND.NTYPE_MODERR==2) CALL ADD_MODERR_TL(SPMODERR,GPMODERR,NSTEP)

!*       3.13  Copy state for use as forcing during the adjoint

  IF (L_SOS) THEN
    IF (MOD((JSTEP+1),NSTEPS_PER_STATE)==0 .AND. JSTEP+1 < NSTOP) THEN
      I = (JSTEP+1)/NSTEPS_PER_STATE
      IF(LOCNORM) THEN
        ALLOCATE (ZSPA3(SIZE(SPA3,1),SIZE(SPA3,2),SIZE(SPA3,3)))
        ZSPA3(:,:,:) = SPA3(:,:,:)
        IF (NPSP==1) THEN
          ALLOCATE (ZSPSP(SIZE(SPSP)))
          ZSPSP(:)   = SPSP(:)
        ENDIF
        CALL LCNORTL
        CALL CAININ (YSTATE_VECTOR_4D(I))
        SPA3(:,:,:) = ZSPA3(:,:,:)
        DEALLOCATE (ZSPA3)
        IF (NPSP==1) THEN
          SPSP(:)   = ZSPSP(:)
          DEALLOCATE (ZSPSP)
        ENDIF
      ELSE
        CALL CAININ (YSTATE_VECTOR_4D(I))
      ENDIF
    ENDIF
  ENDIF

!*       3.15  Signal SMS event for completion of post_processing

  IF(LLPLPP.OR.LLMLPP) THEN
    IF(NOUTTYPE /= 2) THEN
      CALL PPFLUSH
      IF( LPPTSF )THEN
        CALL PPCLOSE
      ENDIF
    ENDIF
  ENDIF

  CALL USER_CLOCK(PTOTAL_CP=ZT2)
  CALL DATE_AND_TIME(CLDAT(1),CLTIMEOD,CLDAT(3),IVALUES)
  ZSEC   = REAL(NSTEP,JPRB)*TSTEP +0.5_JPRB
  IHOUR  = INT( ZSEC/3600._JPRB )
  IMIN   = INT( (ZSEC-REAL(IHOUR,JPRB)*3600._JPRB)/60._JPRB )

ENDDO TEMPORAL_LOOP
IF (LMODERR .AND. NSIM4D == NSIM4DL .AND. NTYPE_MODERR == 5 ) CALL SAVMODERR
!*       3.21  Miscellaneous.

! Computations in observation space

IF(LOBS .AND. NOBTOTG > 0) THEN
  CALL USER_CLOCK(PELAPSED_TIME=ZWT,PVECTOR_CP=ZVT,PTOTAL_CP=ZCT)
  ZCT=ZCT-RSTART
  ZVT=ZVT-RVSTART
  ZWT=ZWT-RTIMEF
  RSTART=RSTART+ZCT
  RVSTART=RVSTART+ZVT
  RTIMEF=RTIMEF+ZWT
  WRITE(NULOUT,'('' NSTEP ='',I6,'' OBSVTL  '',A9)')&
   & NSTEP,CLCONF
  IF(NCONF == 131)THEN
    IF(LOBSTL)THEN
      IF(LPREMPOBS) THEN
        IF (NSIM4D == 0) THEN
          CALL POST_OBSHOR(.TRUE.)
        ENDIF
        CALL POST_OBSHOR(.FALSE.)
      ENDIF
      CALL OBSVTL
    ENDIF
  ELSE
    IF(LPREMPOBS) THEN
      CALL POST_OBSHOR(.FALSE.)
    ENDIF
    CALL OBSV('DI')
  ENDIF
ENDIF

CALL GSTATS(22,1)
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CNT4TL',1,ZHOOK_HANDLE)
END SUBROUTINE CNT4TL
