SUBROUTINE CSEKF1

!**** *CSEKF1*  - Controls EKF job at level 1.

!     Purpose.
!     --------
!           Controls the EKF job at level 1.

!**   Interface.
!     ----------
!        *CALL* *CSEKF1

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation, tbd

!     Externals.
!     ----------
!        SU1YOM  - initialize level 1 commons
!        SUOBSC1 - set up observations
!        CSEKF2  - control EKF job at level 2

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Matthias Drusch and Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 17-01-07
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "csekf2.intfb.h"
#include "dealxmo.intfb.h"
#include "su1yom.intfb.h"

!      -----------------------------------------------------------------

!*       1.    Initialize YOMCT1.
!              ------------------

IF (LHOOK) CALL DR_HOOK('CSEKF1',0,ZHOOK_HANDLE)
CALL SU1YOM

! This part may be introduced later on ...
!      -----------------------------------------------------------------
!*       2.    set up observations (LOBSC1)
!              ----------------------------

!IF(LOBSC1) THEN
!  CALL SUOBSC1
!ENDIF

!      -----------------------------------------------------------------

!*       3.    Call level 2 control routine.
!              -----------------------------

CALL CSEKF2
CALL DEALXMO()

!      -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CSEKF1',1,ZHOOK_HANDLE)
END SUBROUTINE CSEKF1
