SUBROUTINE CVA1

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE YOMVAR   , ONLY : LAVCGL   ,L3DFGAT, LFCOBS
USE YOMCT0   , ONLY : NCONF    ,NSTOP    ,LOBS , &
 & LOBSREF    
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT1   , ONLY : N1POS    ,LRFILAF
USE YOMLUN   , ONLY : NULOUT
USE YOMECTAB , ONLY : NTSLOTNO
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE YOPHLC   , ONLY : LSPHLC   ,LVDFLC   ,LSDRLC   ,LZMCON   ,LKEXP
USE YOPHNC   , ONLY : LETRAJP  ,LERADSW2 ,LENCLD2
USE YOMVRTL  , ONLY : L131TL   ,LDRYTL
USE YOMPHY   , ONLY : LMPHYS   ,LNEIGE   ,LCVRA    ,LGWD     ,&
 & LHMTO    ,LNEBCO   ,LRAY     ,LSFHYD  ,&
 & LSTRA    ,LTHERMO  ,LVDIF    ,LNEBN    ,LCVPP
USE YOMPHY0  , ONLY : ECMNP    ,GWDCD    ,GWDSE
USE YOMPHY2  , ONLY : XMULAF
USE YOMSIMPHL, ONLY : LSIMPH   ,LTRAJPS
USE TRAJECTORY_MOD,ONLY : LTRAJSAVE, LPRTTRAJ
USE YOMCST   , ONLY : RV, RD
USE YOERAD   , ONLY : NTSW, NSW
USE YOMSKF   , ONLY : LROKF
USE GFL_SUBS_MOD , ONLY : DEACT_CLOUD_GFL
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YA, YL, YI, YR, YS


!     *CVA1*  - Controls variational assimilation job at level 1.

!     Purpose.
!     --------
!           Controls variational assimilation job at level 1.

!     Interface.
!     ----------
!        *CALL* *CVA1

!     Externals.   SU1YOM - initialize level 1 commons
!     ----------   SUSPEC - initialize initial conditions
!                  CNT2   - control integration of model at level 2
!                  SUOBS  - Initialize obs. processing
!                  CVA2   - Minimization
!                  TEST_VAR        - Test variational subroutines
!                  FORECAST_ERROR  - Forecast error computation
!                  ROKFCOVAR       - Propagate the covariance matrix for the Farrell/Ioannou
!                                    Reduced-Order Kalman Filter.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        MODIFIED BY D. VASILJEVIC : 90-11-13
!        MODIFIED BY F. RABIER AND J.N THEPAUT : 91-01-15 (diagnostics)
!        MODIFIED BY D. VASILJEVIC : 90-11-13
!        MODIFIED BY W. HECKLEY    : 91-02-20
!        MODIFIED BY D. VASILJEVIC : 91-02-18
!        MODIFIED BY D. VASILJEVIC : 91-05-23
!        MODIFIED BY J.N THEPAUT : 91-09-16 (minimisation algorithms)
!        Bill Heckley and Ross Hoffman: 92-03-06  (RXMIN defined in SUVAR)
!        P.Courtier preconditioning (for cy9).
!        P.Courtier LSIMOB cleaning (for cy10).
!        J.N Thepaut : 94-07-12 - intro of TL incremental.
!        Mike Fisher : 95-01-20 - options to minimize using CONGRAD and
!                                 to generate random background vectors
!        L. Isaksen  : 95-06-06 - Reordering of spectral arrays
!        M. Hamrud   : 95-10-01 - Interface block for SUSPEC
!        M. Fisher   : 95-09-20 - Determine convergence using RCVGE
!        F. Bouttier : 96-01-11 - New GRTEST, cleaning, rm VATRA, call SUHESS
!        P.Courtier  : 96-03-05 - Cleaning lsimob=.t.
!        F.Rabier    : 96-10-15 - Move 'read ITBMIN' from getmini to suhess
!        F.Veerse    : 97-06-04 - Call to GETMINI for cold start at DMN
!        M.Janiskova : 97-06-12 - Set-up simplified phys.param.of Meteo-France
!        J.Barkmeijer: 96-12-12 - option to compute SV's using 3D-Var Hessian
!        F.Bouttier  : 97-07-28 - Clean & Protect gradient test against varQC
!        E.Holm      : 97-12-01 - Ozone included
!        M.Hamrud    : 98-01-10 - Minimisation parallelized
!        M.Hamrud    : 98-03-06 - Moved GETMINI+ to SUVAZX
!        F.Veerse    : 97-11-13 - LFREIN=.T. for DMN eulerian reference traj.
!        M.Janiskova : 97-11-14 - changed set-up of MF low res. physics
!        L.Dragulanescu:97-11-04- 3DVAR ALADIN
!        R. El Khatib: 98-04-02 - Cleanup 3DVAR ALADIN
!        J.N. Thepaut: 98-09-02 - tidy up LFREIN
!        M. Torki    : 99-02-20 - option to minimize using N1CG1 (precond. CG)
!        M. Torki    : 99-04-16 - introduction of Multi-incremental
!        A. Horanyi  : 99-07-08 - call SUCOS before calling CNT2
!        P.Moll      : 99-08-03 - previous call of SUCOS if MF  
!        Y.Tremolet  : 00-06-16 - Split into CVA1, CVA2 and FORECAST_ERROR.
!        C. Temperton: 00-11-06 - bugfix for ozone advection.
!        C. Loo      : 03-12-03 - changed set-up of MF simplified physics 
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Bouteloup : 28-Jan-2005 - Add the call to DEACT_CLOUD_GFL when french prognostic
!                                    microphysics is used
!        M.Janiskova : 14-Dec-2005 - Set cloud related perturbations to zero 
!        C. Fischer  : 24-Nov-2005 - Do not DEACT_CLOUD_GFL if 3DVAR FGAT for ARP/ALD
!        F. Bouyssel : 30-10-06 - changed set-up of MF simplified physics for GWD
!        Y.Tremolet  : 17-Feb-2009 - Call SUVAZX inside SU1YOM
!        K. Yessad (Sep 2008): prune enhanced diffusion (lfrein).
!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "cnt2.intfb.h"
#include "cva2.intfb.h"
#include "forecast_error.intfb.h"
#include "rokfcovar.intfb.h"
#include "su1yom.intfb.h"
#include "suaersn.intfb.h"
#include "suclopn.intfb.h"
#include "spnorm.intfb.h"
#include "sucos.intfb.h"
#include "suobs.intfb.h"
#include "suswn.intfb.h"
#include "cfcsens2obs.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CVA1',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!     Note : the call to bgvecs has been moved to su0yomb
!     ------------------------------------------------------------------

!        1.    Initialize level 1 commons.
!              ---------------------------

CALL GSTATS(20,0)

!     ------------------------------------------------------------------

!        3.       REAL OBSERVATIONS
!                 -----------------

WRITE(NULOUT,*) '--- Variational job : setup observations ',&
 & '----------------------------------'  

!       Define time slot
IF((NCONF == 131.AND.NSTOP == 0))    THEN
  NTSLOTNO=1
ENDIF
!        Initialise observation processing for Jo cost-function
IF (LOBS) CALL SUOBS
!     ------------------------------------------------------------------

WRITE(NULOUT,*) '--- Variational job : setup initial fields ',&
 & '----------------------------'  

CALL SU1YOM

!     ------------------------------------------------------------------

!        4.       TRAJECTORY FOR LINEARIZATION
!                 ----------------------------

IF(NCONF == 131) THEN
  WRITE(NULOUT,*) '--- Variational job : setup linearization ',&
   & 'trajectory -----------------------'  

  IF (LOBS) THEN
    LOBSREF=.TRUE.
  ELSE
    LOBSREF=.FALSE.
  ENDIF
  LRFILAF=.FALSE.
  N1POS=0
  IF(.NOT.LECMWF) CALL SUCOS

  LPRTTRAJ=.true.
  CALL CNT2
  LPRTTRAJ=.false.

  LTRAJSAVE=.false.

!       switch off brake for adjoint whatever the setup is

  N1POS=1
  LOBSREF=.FALSE.
  IF(L131TL) THEN
    IF (LDRYTL) THEN
      WRITE(NULOUT,*) ' *** Using dry R in TL/AD model ***'
      ! This is dirty. Constants should never be modified.
      ! To be cleaned when we understand the problem.../FB
      RV=RD
      !RCPV=RCPD   ! to be tested later
    ENDIF
    IF (LECMWF) THEN
      IF (LETRAJP.AND.LSPHLC) THEN
        WRITE(NULOUT,*)&
         & ' LINEAR PHYSICS ACTIVATED ',&
         & ' ==> SIMPLIFIED VERTICAL DIFFUSION SET TO FALSE '  
      ENDIF

      IF (LETRAJP) THEN
        LEPHYS=.TRUE.
        LAGPHY=.TRUE.
        LSPHLC=.FALSE.
        
        IF (LERADSW2 .AND. NSW > 2) THEN
!       use only 2 spectral intervals for linearized sw
          WRITE(NULOUT,*) 'USING 2 SPEC.INTERVALS FOR LINEARIZED SW'
          NSW=2
          CALL SUSWN   (NTSW, NSW)
          CALL SUCLOPN (NTSW, NSW,NFLEVG)
          CALL SUAERSN (NTSW, NSW)
        ENDIF
      ENDIF

    ELSE
      LMPHYS=.FALSE.
      LCVPP=.FALSE.
      LTHERMO=.FALSE.
      LVDIF=.FALSE.
      LRAY=.FALSE.
      LGWD=.FALSE.
      LCVRA=.FALSE.
      LSTRA=.FALSE.
      LSFHYD=.FALSE.
      LNEBCO=.FALSE.
      LNEBN=.FALSE.
      LNEIGE=.FALSE.
      LHMTO=.FALSE.

      IF (LTRAJPS) THEN
        LSIMPH=.TRUE.
        LNEIGE=.TRUE.
        LSPHLC=.FALSE.
        ECMNP=7000._JPRB
        GWDCD=6._JPRB
        GWDSE=3.5E-03_JPRB
        XMULAF=0.0_JPRB
      ELSE
        LSPHLC=.TRUE.
        LVDFLC=.TRUE.
        LSDRLC=.TRUE.
        LZMCON=.TRUE.
        LKEXP =.TRUE.
      ENDIF
    ENDIF

  ENDIF
  IF(LECMWF.OR.(.NOT.L3DFGAT))THEN
    ! TL and AD models do not support cloud fields. yet.
    CALL DEACT_CLOUD_GFL
  ENDIF
  IF (LENCLD2) THEN
    ! Put perturbation of cloud variables to zero
    GFL(:,:,YA%MP,:) = 0.0_JPRB
    GFL(:,:,YL%MP,:) = 0.0_JPRB
    GFL(:,:,YI%MP,:) = 0.0_JPRB
    GFL(:,:,YR%MP,:) = 0.0_JPRB
    GFL(:,:,YS%MP,:) = 0.0_JPRB
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!      5.  Do the job
!          ----------

WRITE(NULOUT,*) '--- CVA1 call cfcsens2obs 0.0-------------------'
call spnorm(0)
IF (LAVCGL) THEN
  CALL FORECAST_ERROR
ELSEIF (LROKF) THEN
  CALL ROKFCOVAR
ELSEIF (LFCOBS) THEN
  CALL CFCSENS2OBS
ELSE
  CALL CVA2
ENDIF

!     ------------------------------------------------------------------

WRITE(NULOUT,*) '--- End of variational job.-------------------'
CALL GSTATS(20,1)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CVA1',1,ZHOOK_HANDLE)
END SUBROUTINE CVA1
