SUBROUTINE FORECAST_ERROR

!     *FORECAST_ERROR*  - Controls forecast error computation. 

!     Purpose.
!     --------
!           Controls forecast error computation.

!     Interface.
!     ----------
!        *CALL* *FORECAST_ERROR

!     Externals.   SIM4D  - simulator (for X computes J and grad(J) )
!     ----------   SCAAS  - scalar product computation
!                  GRTEST - test of gradient
!                  CTONB  - change of base (for M1QN3 only)
!                  CTCAB  - change of base inverse (for M1QN3 only)
!                  CONGRAD- minimization algorithm (X = 2 or 3)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Y. Tremolet   (Split from CVA1)

!     Modifications.
!     --------------
!        Original : 00-06-16
!        02-08-07 Y.Tremolet   Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        Y.Tremolet    18-Mar-2004 Tests of 4D-Var components
!        D.Dee         19-Mar-2004 Variational bias correction
!        G. Radnoti    03-Dec-2004 Backward tl propagation
!        G.Desroziers  28-Feb-2008 Possible computation of ensemble bg error variance
!        Y.Tremolet    17-Feb-2009 Added ADJOTEST
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARDIMO  , ONLY : JPNOTP   ,JPXVAR
USE YOMVAR   , ONLY : CFNSIGF  ,NSIM4D   ,NBGVECS  ,NITER    ,&
 & NITER_MIN, &
 & CFNSIGA  ,LWRISIGA ,CFNSIGB  ,LWRISIGB ,NSIM4DL  ,&
 & LWRISIGF ,LGCV     ,LGCVJO   ,LJC      ,LJCDFI   ,&
 & NITERGCV ,GCVJO    ,RCVGE    ,L3DFGAT  ,NPCVECS  ,&
 & R_NORM_REDUCTION_ABORT_LEVEL ,L_CHECK_CONVERGENCE,&
 & NUPTRA   ,LSKIPMIN, NDIAG, RDX, LTOVSCV   ,&
 & L_INFO_CONTENT, N_INFO_CONTENT_METHOD, N_INFO_CONTENT_SEED ,&
 & N_DIAGS_CONVERGENCE, N_DIAGS_EIGENVECS, &
 & EVBCGL, LVARBC,  LPROPTL, L_ABS_CONVERGENCE, L_INFO_CONVERGENCE ,&
 & RTOL_CHECK_GRADIENT, LTSCV, LBACKGE, LMODERR, LJBIMPACT, LJCIMPACT
USE YOMCMA   , ONLY : NRESUPD
USE YOMCVA   , ONLY : YVAZX, YVAZG, YVAZ0, YSCALPSQRT, YRSCALPSQRT
USE YOMJG    , ONLY : LCTLFG
USE YOMCT0   , ONLY : LBACKG   ,LOBS     ,LSIMOB   ,LELAM
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT1   , ONLY : N1POS    ,N1GDI    ,N1SDI
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMCOSJO , ONLY : NOTVAR   ,NGCVVAR
USE YOMTNEWT , ONLY : LIDMODEL
USE CONTROL_VECTORS
USE YOMDIM   , ONLY : NFD2D,NFLSUR,NSPEC2,NS3D,NS2D,NFLEVG
USE YOMMP    , ONLY : NPSP
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SPA1, &
 & SPA3INI, SPA2INI, SPA1INI
USE YOMSPJB  , ONLY : SP7A3    ,SP7A2, SP7A1
USE YOMTVRAD , ONLY : TOVSCVX  ,TOVSCVBG
USE YOMANCS  , ONLY : RMDI
USE YOMVCGL  , ONLY : YVCGLEV   ,RCGLEV, NVCGLEV
USE TESTVAR_MIX, ONLY: NTESTVAR, LCVTEST, LINTEST, LTLTEST, LADTEST, LGRTEST
USE RANDOM_NUMBERS_MIX
USE YOMTS    , ONLY : TSCVX, TSCVBG

IMPLICIT NONE

! * LOCAL EXTERNAL FUNCTIONS:
EXTERNAL PROSCA

INTEGER(KIND=JPIM) , ALLOCATABLE, DIMENSION(:,:) :: IGCVVAR

! * LOCAL TYPE "control_vector" VARIABLES:
TYPE(control_vector) :: YL_ZLANCZOS, YL_ZVAZXAN, YL_ZVAZGAN
TYPE(control_vector) :: YLCGLWK(niter+1)

TYPE(randomNumberStream) :: YL_RANDOM_STREAM_INFO

INTEGER(KIND=JPIM) :: INOBS, ITER, INDIC, IDUM, INVECS, J, jj

LOGICAL :: LLJC, LLJCDFI, LLGOLUB, LLFAIL

REAL(KIND=JPRB) :: ZEPS, ZF, ZFI, ZGI, ZFE, ZGE, ZGCV, ZGOLUBL_INV, ZGOLUBU_INV, &
 & ZGOLUBL_LOG, ZGOLUBU_LOG, ZVAZGNO
REAL(KIND=JPRB) :: zwork(niter+1,4)
REAL(KIND=JPRB), ALLOCATABLE :: ZRAND(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "bgvecs.intfb.h"
#include "bgevecs.intfb.h"
#include "chavarin.intfb.h"
#include "congrad.intfb.h"
#include "estsig.intfb.h"
#include "estsiga.intfb.h"
#include "grtest.intfb.h"
#include "savmini.intfb.h"
#include "prt_ctlvec_norms.intfb.h"
#include "scaas.intfb.h"
#include "setran.intfb.h"
#include "sim4d.intfb.h"
#include "svvarbc.intfb.h"
#include "writeoba.intfb.h"
#include "writesd.intfb.h"
#include "xformev.intfb.h"
#include "tltest.intfb.h"
#include "adtest.intfb.h"
#include "cvtest.intfb.h"
#include "litest.intfb.h"
#include "prt_conv_diags.intfb.h"
#include "save_evecs.intfb.h"
#include "tlprop.intfb.h"
#include "write_ctlvec_grib.intfb.h"
#include "adjotest.intfb.h"
#include "djbdy.intfb.h"
#include "djcdy.intfb.h"
!     ------------------------------------------------------------------

!        1.2 Diagnose background error setup
!            -------------------------------

IF (LHOOK) CALL DR_HOOK('FORECAST_ERROR',0,ZHOOK_HANDLE)
IF (LBACKG) THEN
  IF (LBACKGE) THEN
    WRITE(NULOUT,*) '--- Variational job : Diagnose background ',&
     & 'error variance from ensemble vectors ----------------------------'
    CALL BGEVECS
  ELSE
    WRITE(NULOUT,*) '--- Variational job : Diagnose background ',&
     & 'error setup ----------------------------'  
    CALL BGVECS
  ENDIF
  IF(LWRISIGB)  CALL WRITESD(CFNSIGB,'eb')
ENDIF

IF (.NOT. LECMWF .AND. (NITER <= 1) ) THEN
  IF (LHOOK) CALL DR_HOOK('FORECAST_ERROR',1,ZHOOK_HANDLE)
  RETURN
ENDIF

!     ------------------------------------------------------------------

!        2.  Compute initial cost-function and gradient.
!            -------------------------------------------

WRITE(NULOUT,*) '--- Variational job : initial call to ',&
 & 'simulator ----------------------------'  

IF (L3DFGAT) THEN
  LIDMODEL=.TRUE.
ENDIF

IDUM=-999
ZF=0.0_JPRB
NSIM4D=0
N1POS=1
N1SDI=1
N1GDI=1
IF (LPROPTL) THEN
  IF (.NOT. ALLOCATED(SPA3INI)) ALLOCATE(SPA3INI(NFLSUR,NSPEC2,NS3D))
  IF (.NOT. ALLOCATED(SPA2INI)) ALLOCATE(SPA2INI(NSPEC2,NS2D))
  IF (LELAM .AND. .NOT. ALLOCATED(SPA1INI)) ALLOCATE(SPA1INI(NFLEVG,2))
  SPA3INI=0.0_JPRB
  SPA2INI=0.0_JPRB
  IF (LELAM) SPA1INI=0.0_JPRB
ENDIF
CALL SIM4D(4,IDUM,YVAZX,ZF,YVAZG,NSIM4D)
ZVAZGNO=0.0_JPRB
ZFI=ZF
CALL SCAAS(IDUM,YVAZG,YVAZG,ZVAZGNO)
ZGI=ZVAZGNO
WRITE(UNIT=NULOUT,FMT='('' INITIAL GRADIENT NORM SQUARED : ''&
 & ,E14.7,'' COST FUNCTION : '',E14.7)')&
 & ZGI,ZFI  

IF (NTESTVAR==1 .OR. NTESTVAR==3) THEN
  NDIAG=0
  N1POS=0
  N1SDI=1
  N1GDI=0
  IF (LCVTEST) CALL CVTEST
  IF (LINTEST) CALL LITEST(YVAZX)
  IF (LADTEST) CALL ADTEST
  IF (LADTEST) CALL ADJOTEST
! TL test cannot be run before the minimisation without changing the
! result. This is in particular due to the switches and values which
! are changed in CVA1 after the call to CNT2. These changes cannot
! be switched on and off easily for the test and then back again. If
! you don't mind this, uncomment the following line.
! Otherwise, just dream of a future where the IFS would be modular...
! GR: reproducibility seems to be solved, but here tltest is still useless
! IF (LTLTEST) CALL TLTEST(YVAZX)
  IF (LGRTEST) CALL GRTEST(SIM4D,PROSCA,YVAZX,RDX,8,NULOUT)
  NDIAG=1
ENDIF

!     ------------------------------------------------------------------

!        3.    Minimization.
!              -------------

!        3.1   Initialize.

IF (L_ABS_CONVERGENCE) THEN
  ZEPS=SQRT(RCVGE/ZGI)
ELSE
  ZEPS=RCVGE
ENDIF
ITER=NITER
N1POS=0
N1SDI=0
N1GDI=0

CALL ALLOCATE_CTLVEC(YL_ZLANCZOS)
DO jj=1,niter+1
  CALL allocate_ctlvec(YLCGLWK(jj))
ENDDO

IF (L_INFO_CONTENT) THEN

!== For the analysis-difference method, compute and save the unperturbed analysis

  IF (N_INFO_CONTENT_METHOD==2) THEN
    ITER=NITER/2
    INVECS=ITER+1
    CALL CONGRAD (YVAZX,YVAZG,YL_ZLANCZOS,ZEPS,EVBCGL,ITER,INVECS, &
     & NITER_MIN, &
     & ZGOLUBU_INV,ZGOLUBL_INV, ZGOLUBU_LOG,ZGOLUBL_LOG, &
     & YLCGLWK,zwork, &
     & LDEVECS=.FALSE., LDSOLVE=.TRUE., LD_LGOLUB=.FALSE., &
     & LD_INFO_CONVERGENCE=L_INFO_CONVERGENCE)
    CALL ALLOCATE_CTLVEC(YL_ZVAZXAN)
    YL_ZVAZXAN=YVAZX
  ENDIF

!== generate random vector in the Euclidean inner-prouct space
  ALLOCATE (ZRAND(YL_ZLANCZOS%nsizeg))
  CALL SETRAN (N_INFO_CONTENT_SEED,YL_RANDOM_STREAM_INFO)
  CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM_INFO)
  WHERE (ZRAND(:)>0.5_JPRB)
    ZRAND(:)=1.0_JPRB
  ELSEWHERE
    ZRAND(:)=-1.0_JPRB
  ENDWHERE
  YL_ZLANCZOS = ZRAND(:)
  DEALLOCATE (ZRAND)
  YL_ZLANCZOS%data = YL_ZLANCZOS%data * YRSCALPSQRT%data

  IF (N_INFO_CONTENT_METHOD==1) THEN
    LLGOLUB = .TRUE.
  ELSEIF (N_INFO_CONTENT_METHOD==2) THEN
    ITER = NITER/2

!== save the perturbation
    CALL ALLOCATE_CTLVEC(YL_ZVAZGAN)
    YL_ZVAZGAN=YL_ZLANCZOS

!== take the perturbation to model space and perturb the background
    YL_ZLANCZOS%data = -YL_ZLANCZOS%data * YRSCALPSQRT%data
    CALL CHAVARIN (YL_ZLANCZOS)
    SP7A3(:,:,:) = SPA3(:,:,:)
    IF(NPSP == 1) SP7A2(:,1:NFD2D) = SPA2(:,1:NFD2D)
    IF (LELAM) SP7A1(:,:) = SPA1(:,:)
    IF(LTOVSCV) THEN
      WHERE(TOVSCVBG(:,:) /= RMDI) TOVSCVBG(:,:) = TOVSCVX(:,:)
    ENDIF
    IF(LTSCV) THEN
       WHERE(TSCVBG(:) /= RMDI) TSCVBG(:) = TSCVX(:)
    ENDIF


!== calculate starting gradient for the perturbed analysis
    YVAZX=0.0_JPRB
    CALL SIM4D(4,IDUM,YVAZX,ZF,YVAZG,NSIM4D)
    YL_ZLANCZOS = YVAZG
    LLGOLUB = .FALSE.
  ENDIF
ELSE
  YL_ZLANCZOS = YVAZG
  YL_ZLANCZOS%data = -YVAZG%data
  LLGOLUB = .FALSE.
ENDIF

!        3.3   Minimize.

WRITE(NULOUT,*) '--- Variational job : minimization ',&
 & '(Lanczos method)-----------'  

INVECS=ITER+1
CALL CONGRAD (YVAZX,YVAZG,YL_ZLANCZOS,ZEPS,EVBCGL,ITER,INVECS, &
 & NITER_MIN, &
 & ZGOLUBU_INV,ZGOLUBL_INV, ZGOLUBU_LOG,ZGOLUBL_LOG, &
 & YLCGLWK,zwork, &
 & LDEVECS=.TRUE., LDSOLVE=.TRUE., LD_LGOLUB=LLGOLUB, &
 & LD_INFO_CONVERGENCE=L_INFO_CONVERGENCE)  
CALL DEALLOCATE_CTLVEC(YL_ZLANCZOS)

IF (L_INFO_CONTENT) THEN
  IF (N_INFO_CONTENT_METHOD==1) THEN
    WRITE (NULOUT,*) 'Entropy reduction (upper bound) =',ZGOLUBU_LOG
    WRITE (NULOUT,*) 'Entropy reduction (lower bound) =',ZGOLUBL_LOG
    WRITE (NULOUT,*) 'Degrees of freedom for signal (upper bound) =',ZGOLUBU_INV
    WRITE (NULOUT,*) 'Degrees of freedom for signal (lower bound) =',ZGOLUBL_INV
  ELSEIF (N_INFO_CONTENT_METHOD==2) THEN
    YVAZX%data=(YVAZX%data-YL_ZVAZXAN%data) * YSCALPSQRT%data
    ZVAZGNO = -DOT_PRODUCT (YL_ZVAZGAN,YVAZX)
    WRITE (NULOUT,*) 'Degrees of freedom for signal = ',ZVAZGNO
    CALL DEALLOCATE_CTLVEC(YL_ZVAZXAN)
    CALL DEALLOCATE_CTLVEC(YL_ZVAZGAN)
  ENDIF
ENDIF

IF (LJBIMPACT) CALL DJBDY(iter,niter,YLCGLWK,zwork,YVAZX,YVAZ0)
IF (LJCIMPACT) CALL DJCDY(iter,niter,YLCGLWK,zwork,YVAZX,YVAZ0)

! Transform the eigenvectors to model space and calculate analysis error variance

INVECS = NPCVECS + NVCGLEV
IF (INVECS>0) CALL XFORMEV (INVECS,.TRUE.)

!     ------------------------------------------------------------------

!        4.    Test of the quality of the minimization.
!              ----------------------------------------

!        4.1   Diagnose decrease of cost-function and gradient norm.

CALL ALLOCATE_CTLVEC(YL_ZLANCZOS)
YL_ZLANCZOS%data(:) = YVAZG%data(:)

WRITE(NULOUT,*) '--- Variational job : final call to simulator ',&
 & '-----------------------------'  
N1SDI=1
N1GDI=1
N1POS=1
INDIC=4
ZF=0.0_JPRB
NSIM4D=NSIM4DL
CALL SIM4D(INDIC,IDUM,YVAZX,ZF,YVAZG,NSIM4D)
ZVAZGNO=0.0_JPRB
CALL SCAAS(IDUM,YVAZG,YVAZG,ZVAZGNO)
ZFE=ZF
ZGE=ZVAZGNO
WRITE(UNIT=NULOUT,FMT='(''   FINAL GRADIENT NORM SQUARED : ''&
 & ,E14.7,'' COST FUNCTION : '',E14.7)')&
 & ZGE,ZFE  
WRITE(NULOUT,*) '--- Variational job : diagnose quality of the ',&
 & 'minimization ----------------'  
IF(ZFI <= 0.0_JPRB) ZFI=1.0_JPRB
IF(ZGI <= 0.0_JPRB) ZGI=1.0_JPRB

WRITE(UNIT=NULOUT,FMT='('' RATIO FINAL/INITIAL GRADIENT SQUARED : ''&
 & ,E14.7,'' COST FUNCTION : '',E14.7)')&
 & ZGE/ZGI,ZFE/ZFI  

WRITE(NULOUT,'(A,ES10.3)')'Ratio final/initial gradient norm = ',sqrt(ZGE/ZGI)
WRITE(NULOUT,'(A,ES10.3)')'Ratio final/initial cost function = ',ZFE/ZFI

! Check CONGRAD's idea of the gradient against SIM4D's.

LLFAIL=.FALSE.

YL_ZLANCZOS%data(:) = YL_ZLANCZOS%data(:) - YVAZG%data(:)
CALL SCAAS(IDUM,YL_ZLANCZOS,YL_ZLANCZOS,ZVAZGNO)
ZVAZGNO=SQRT(ZVAZGNO/MAX(ZGE,1E-20_JPRB))
WRITE(NULOUT,*) 'Relative Error of CONGRAD''s gradient = ',ZVAZGNO
IF (ZVAZGNO>RTOL_CHECK_GRADIENT) THEN
  WRITE (NULERR,*) 'ACCURACY-OF-GRADIENT CHECK FAILS:'
  LLFAIL=.TRUE.
  CALL WRITE_CTLVEC_GRIB ('gradient_error',YL_ZLANCZOS)
ENDIF
CALL DEALLOCATE_CTLVEC(YL_ZLANCZOS)

! Before checking convergence because it is when the minimisation
! does not converge that tests are the most needed.
IF (NTESTVAR==2 .OR. NTESTVAR==3) THEN
  NDIAG=0
  N1POS=0
  IF (LCVTEST) CALL CVTEST
  IF (LINTEST) CALL LITEST(YVAZX)
  IF (LADTEST) CALL ADTEST
  IF (LADTEST) CALL ADJOTEST
  IF (LTLTEST) CALL TLTEST(YVAZX)
  IF (LGRTEST) CALL GRTEST(SIM4D,PROSCA,YVAZX,RDX,8,NULOUT)
ENDIF
IF (LPROPTL) THEN
  N1SDI=1
  N1POS=1
  CALL TLPROP(YVAZX)
ENDIF

IF (ZGE/ZGI  >  R_NORM_REDUCTION_ABORT_LEVEL .AND. &
   & .NOT. LSKIPMIN .AND. &
   & .NOT.(L_INFO_CONTENT .AND. N_INFO_CONTENT_METHOD==1)) THEN  
  WRITE (NULERR,*) 'CONVERGENCE CHECK FAILS:'
  WRITE (NULERR,*) 'REDUCTION IN GRADIENT NORM SQUARED EXCEEDS',&
   & R_NORM_REDUCTION_ABORT_LEVEL  
  LLFAIL=.TRUE.
ENDIF

IF (LLFAIL) THEN
  IF (N_DIAGS_CONVERGENCE>=1) CALL PRT_CONV_DIAGS
  IF (L_CHECK_CONVERGENCE) THEN
    IF (N_DIAGS_EIGENVECS>=1) CALL SAVE_EVECS
    CALL ABOR1 ('FORECAST_ERROR: CONVERGENCE CHECK FAILS')
  ELSE
    WRITE (NULOUT,*)'ABORT DISABLED BECAUSE L_CHECK_CONVERGENCE==.FALSE.'
  ENDIF
ELSE
  IF (N_DIAGS_CONVERGENCE>=2) CALL PRT_CONV_DIAGS
  IF (N_DIAGS_EIGENVECS  >=2) CALL SAVE_EVECS
ENDIF

if (ALLOCATED(YVCGLEV)) then
  DO j=1,SIZE(YVCGLEV)
    call deallocate_ctlvec(YVCGLEV(j))
  ENDDO
  DEALLOCATE (YVCGLEV)
endif
if (ALLOCATED(RCGLEV)) DEALLOCATE (RCGLEV)

DO jj=1,niter+1
  CALL deallocate_ctlvec(YLCGLWK(jj))
ENDDO

!     ------------------------------------------------------------------
!        5.0 Write the control vector
!        (Note: CVA2 does not call SAVMINI if NSTOP is zero. Why not?)

IF (LCTLFG) THEN
  YVAZX%data(:)=YVAZX%data(:)+YVAZ0%data(:)
ELSEIF (LMODERR) THEN
  YVAZX%moderr(:)=YVAZX%moderr(:)+YVAZ0%moderr(:)
ENDIF
IF (NUPTRA /= NRESUPD-1) CALL SAVMINI(YVAZX)
IF (LVARBC) CALL SVVARBC(YVAZX%PARAMS,YVAZX%NPARA)
CALL PRT_CTLVEC_NORMS(YVAZX,'YVAZX')

!        5.1 Estimate and write out standard dev of an/fc errors.

IF (LWRISIGA .OR. LWRISIGF) THEN
  WRITE(NULOUT,*) '--- Variational job : write estimates of ',&
   & 'analysis/forecast errors-----------------'  

  IF (NBGVECS==0) CALL ESTSIGA ('an')
  IF (LWRISIGA) CALL WRITESD (CFNSIGA,'ea')

  IF (LWRISIGF) THEN
    CALL ESTSIG
    CALL WRITESD(CFNSIGF,'ef')
  ENDIF
ENDIF

!        5.2  Calculate Generalized Cross-Validation function

IF (LGCV) THEN

!   Save the analysis and gradient for the GCV calculation
  CALL ALLOCATE_CTLVEC(YL_ZVAZXAN)
  CALL ALLOCATE_CTLVEC(YL_ZVAZGAN)
  YL_ZVAZXAN = YVAZX
  YL_ZVAZGAN = YVAZG

!         Calculate the numerator (the contribution to Jo at the
!         analysis point from the selected observations). Store in ZGCV.

  NSIM4D=0
  LLJC=LJC
  LLJCDFI=LJCDFI
  LJC=.FALSE.
  LJCDFI=.FALSE.
  ALLOCATE (IGCVVAR(JPXVAR,JPNOTP))
  IGCVVAR(:,:) = NOTVAR (:,:)
  NOTVAR (:,:) = NGCVVAR(:,:)
  ZF=0.0_JPRB
  N1POS=1
  N1SDI=1
  N1GDI=1

  CALL SIM4D(4,IDUM,YVAZX,ZF,YVAZG,NSIM4D)

  ZGCV = GCVJO
  LJC=LLJC
  LJCDFI=LLJCDFI

!         Calculate the denominator (trace(I-A))

  YVAZX = 0.0_JPRB
  LGCVJO=.TRUE.
  NSIM4D=0
  N1POS=1
  N1SDI=1
  N1GDI=1
  CALL ALLOCATE_CTLVEC(YL_ZLANCZOS)
  DO jj=1,niter+1
    CALL allocate_ctlvec(YLCGLWK(jj))
  ENDDO

  CALL SIM4D(4,IDUM,YVAZX,ZF,YL_ZLANCZOS,NSIM4D)

  IF (YL_ZLANCZOS%nmoderr>0) YL_ZLANCZOS%moderr(:) = 0.0_JPRB
  IF (YL_ZLANCZOS%ltovscv  ) YL_ZLANCZOS%tovscv(:) = 0.0_JPRB
  IF (YL_ZLANCZOS%lam1d    ) YL_ZLANCZOS%lamcv(:)  = 0.0_JPRB
  IF (YL_ZLANCZOS%npara>0  ) YL_ZLANCZOS%params(:) = 0.0_JPRB

  INOBS = NINT(ZF)

  LGCVJO=.FALSE.
  NSIM4D=0
  N1POS=1
  N1SDI=1
  N1GDI=1
  NOTVAR(:,:) = IGCVVAR(:,:)
  DEALLOCATE (IGCVVAR)

  YVAZX = YL_ZVAZXAN
  YVAZG = YL_ZVAZGAN
  CALL DEALLOCATE_CTLVEC(YL_ZVAZXAN)
  CALL DEALLOCATE_CTLVEC(YL_ZVAZGAN)

  ITER = NITERGCV
  INVECS=ITER+1

  CALL CONGRAD (YVAZX,YVAZG,YL_ZLANCZOS,EVBCGL,ZEPS,ITER,INVECS, &
   & NITER_MIN, &
   & ZGOLUBU_INV, ZGOLUBL_INV, ZGOLUBU_LOG, ZGOLUBL_LOG, &
   & YLCGLWK,zwork, &
   & LDEVECS=.FALSE., LDSOLVE=.FALSE., LD_LGOLUB=.TRUE., &
   & LD_INFO_CONVERGENCE=L_INFO_CONVERGENCE)  
  CALL DEALLOCATE_CTLVEC(YL_ZLANCZOS)
  DO jj=1,niter+1
    CALL deallocate_ctlvec(YLCGLWK(jj))
  ENDDO

  WRITE (NULOUT,'(''---- Summary of GCV calculation ----''/)')
  WRITE (NULOUT,*) 'Jo at analysis point        =',ZGCV
  WRITE (NULOUT,*) 'Number of obs. departures   =',INOBS
  WRITE (NULOUT,*) 'Bounds on estimated trace(A)=',ZGOLUBU_INV, ZGOLUBL_INV

  IF (ABS(ZGOLUBU_INV-ZGOLUBL_INV)  >  1E-2_JPRB*ABS(ZGOLUBU_INV+ZGOLUBL_INV)) THEN
    WRITE (NULOUT,*) 'WARNING: Bounds have not converged'
  ENDIF

  IF (SQRT(ZGOLUBU_INV*ZGOLUBL_INV) > REAL(INOBS,JPRB)) THEN
    WRITE (NULOUT,*) 'ERROR: Trace(A) > Number of obs'
    CALL ABOR1 ('FORECAST_ERROR: GCV has Trace(A) > Number of obs')
  ENDIF

  ZGCV = INOBS * ZGCV / (REAL(INOBS,JPRB)-SQRT(ZGOLUBU_INV*ZGOLUBL_INV))**2

  WRITE (NULOUT,*) 'Generalized Cross Validation Function = ',ZGCV
  WRITE (NULOUT,*)
ENDIF

!        5.3 Save the observation data base
!            ------------------------------

IF (LOBS .AND. (.NOT.LSIMOB) ) THEN
  CALL GSTATS(47,0)
  WRITE(NULOUT,*) '--- Variational job : write final obs ',&
   & 'files -------------------------------'  
  WRITE(NULOUT,'('' ** WRITE-OUT OBS. '')')
  CALL WRITEOBA
  CALL GSTATS(47,1)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FORECAST_ERROR',1,ZHOOK_HANDLE)
END SUBROUTINE FORECAST_ERROR
