SUBROUTINE GP_MODEL_AD(CDCONF)

! GP_MODEL_AD - Model (adjoint)

! Purpose 
! -------
!   Interface to Adjoint grid-point model

! Interface
! ---------
!   CDCONF - configuration of work (see doc.)

! Externals
! ---------
!   See the "include" topic below.

! Method
! ------

! Reference
! ---------
!   ECMWF Research Department documentation of the IFS

! Author
! ------
!   Deborah Salmond (ECMWF)

! Modifications
! -------------
!   06-Mar-2001 C. Fischer     Merge LAM version into global code
!   01-Oct-2002 G. Mozdzynski  support for radiation on-demand comms
!      Mar-2003 J. Vivoda      PC schemes for NH dynamics (LPC_XXXX keys) 
!   01-Oct-2003 M. Hamrud      CY28 Cleaning
!   24-May-2004 C. Fischer     gmv's set to zero from 1 to ngptot whatever 
!                              model (instead of igpcomp)
!   10-Jun-2004 J. Masek       NH cleaning (LPC_NOTR, LFULLIMP)
!   01-Jul-2004 K. Yessad      Make clearer the tests for PC scheme.
!      Jul-2004 K. Yessad      Modularization of CPGAD according to CPG.
!     J.Vivoda     02-03   : PC schemes for NH dynamics (LPC_XXXX keys) 
!     M.Hamrud     01-Oct-2003 CY28 Cleaning
!     C. Temperton 03-12-22: Fix for the case LSFLAG=F, LSPRT=T
!   03-03-2006  R.Brozkova set T1 arrays to zero for security (extension zone problem)
!   05-04-2006  M. Jidane  : CY31 Phasing
!   01-08-2006  D. Salmond : 4D-Var rain assimilation
!   04-Jun-2007 M. Jidane  : Fix NSLCORE bug
!   25-Apr-2008 A. Geer    : Multiple sensor 4D-Var rain assimilation
!   23-Sep-2008 A. Geer    : Allow ADTEST to work with 4D-Var rain assimilation
!   09-Mar-2009 W. Bell    : Add azimuth angle for windsat-like sensors
!   16-Dec-2009 F. Karbou  : Allow surface emissivity handling
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NGPBLKS  ,&
 & NDGSAG   ,NDGENG   ,NDGLG    ,NDGSAL   ,NDGENL  ,NSMAX
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LELAM    ,LPC_FULL ,&
 & LSPRT
USE YOMSC2   , ONLY : NSLWIDE  ,NFLDSLB2 ,NFLDSLB1 ,NFLDSLB15
USE YOMGEM   , ONLY : RCOLON   ,RSILON   ,RATATH   ,RATATX   ,&
 & NLOENG   ,NGPTOT   ,RINDX    ,RINDY    ,&
 & NGPTOT_CAP  
USE YOMGC    , ONLY : RCORI    ,GEMU     ,GSQM2    ,GELAM    ,GECLO    ,&
 & GESLO    ,GM       ,GNORDL   ,GNORDM   ,OROG     ,OROGL    ,OROGM    ,&
 & GOMVRL   ,GOMVRM
USE YOMDYN   , ONLY : NCURRENT_ITER
USE YOMMP    , ONLY : NASLB1   ,NSLSTA   ,NSLONL   ,NSLOFF   ,&
 & NSLCORE  ,NSLPAD   ,&
 & MYFRSTACTLAT ,MYLSTACTLAT ,NFRSTLOFF  
USE YOMSLREP , ONLY : LSLADREP  ,NGPTOTAD  ,NADCORE  ,&
 & GEMUAD   ,GESLOAD  ,GECLOAD  ,GMAD     ,GNORDLAD  ,&
 & GNORDMAD ,GSQM2AD  ,RCOLONAD ,RSILONAD ,RINDXAD   ,&
 & RINDYAD  
USE YOEPHY   , ONLY : LEPHYS   ,LEMWAVE
USE YOPHNC   , ONLY : LETRAJP
USE YOMGMV   , ONLY : GMV,GMVS,GMV5,GMV5S,GMVT1,GMVT1S,YT1,YT0,YT9,YT5
USE SURFACE_FIELDS_MIX , ONLY : SP_SB,SP_SG,SP_RR,&
 & SD_VF,SD_VN,SD_VD
USE YOMGFL   , ONLY : GFL,GFLT1
USE YOM_YGFL , ONLY : YGFL,YGFLC
USE PARMWAVE , ONLY : NMAX_CHAN
USE YOMMWAVE , ONLY : LEMWAVE_ACTIVE, N_SATSENS
USE YOMTVRAD , ONLY : MNSERIES 
USE TESTVAR_MIX,   ONLY : LTESTADJM

!    -------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

!    -------------------------------------------------------------------

REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUFTEMP(:), ZSLBUF2X(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMVT1X(:,:,:,:),ZGMVT1SX(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFLT1X(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF2(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF1AUX(:,:), ZSLBUF1(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF15AUX(:,:), ZSLBUF15(:,:)

INTEGER(KIND=JPIM) :: IBL, ICEND, ICENDE, IGPCOMP, &
 & IOFF, &
 & ILSB, ILFB, JGLLOC, IST, IEND, &
 & JKGLO, JROF, JLEV,&
 & ISLB2,JFLD,IGPTOT,J

LOGICAL :: LL2D

REAL(KIND=JPRB), ALLOCATABLE :: ZMWAVE_TB_BUF_AD(:,:,:), &
 & ZMWAVE_OERR_BUF(:,:,:), ZMWAVE_ZEN_BUF(:,:), ZMWAVE_AZI_BUF(:,:), &
 & ZMWAVE_PERS1_BUF(:,:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IMWAVE_BUF(:,:,:)

REAL(KIND=JPRB) :: ZZZ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

#include "call_sl_ad.intfb.h"
#include "cpg25.intfb.h"
#include "cpg2ad.intfb.h"
#include "cpg2lagad.intfb.h"
#include "cpg5.intfb.h"
#include "cpgad.intfb.h"
#include "cpglagad.intfb.h"
#include "ec_phys_ad.intfb.h"
#include "ladad.intfb.h"
#include "pre_sladrep.intfb.h"
#include "mwave_get_ad.intfb.h"

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GP_MODEL_AD',0,ZHOOK_HANDLE)

!    -------------------------------------------------------------------

!    -------------------------------------------------------------------
!*       1.    INITIALISATION
!    -------------------------------------------------------------------

IF(NCONF/100 == 2.OR.NCONF/10 == 42.OR.NCONF/10 == 52) THEN
  LL2D=.TRUE.
ELSE
  LL2D=.FALSE.
ENDIF

! Total number of grid points - ndglg*ndlon in global / ndguxg*ndlon in LAM
! used for model computations

! Changed NGPTOT into IGPCOMP in order to pass properly
! the scan of NSLCORE values for the LAM models
IF(.NOT.LELAM) THEN
  IGPCOMP=NGPTOT
ELSE
  IGPCOMP=MIN(NGPTOT,NGPTOT_CAP)
ENDIF

CALL GSTATS(1222,0)
IF(.NOT.ALLOCATED(GFLT1)) THEN
  ALLOCATE(GFLT1(NPROMA,NFLEVG,YGFL%NDIM1,NGPBLKS))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NGPBLKS
    GFLT1(:,:,:,J)=0.0_JPRB
  ENDDO
!$OMP END PARALLEL DO
ENDIF
IF (LSLAG) THEN
  IF (.NOT.ALLOCATED(ZSLBUF1)) THEN
    ALLOCATE (ZSLBUF1(NASLB1,NFLDSLB1))
    IF (NSLPAD > 0) THEN
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
      DO J=1,NFLDSLB1
        ZSLBUF1(:,J)=HUGE(ZZZ)
      ENDDO
!$OMP END PARALLEL DO
      ILSB=MAX(NDGSAG,MYFRSTACTLAT-NSLWIDE)-NFRSTLOFF
      ILFB=MIN(NDGENG,MYLSTACTLAT +NSLWIDE)-NFRSTLOFF
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JGLLOC,IST,IEND)
      DO JGLLOC=ILSB,ILFB
        IF (NLOENG(JGLLOC+NFRSTLOFF) == NSLONL(JGLLOC)) THEN
          IST =NSLOFF(JGLLOC)
          IEND=NSLOFF(JGLLOC)+NSLONL(JGLLOC)+2
        ELSE
          IST =NSLOFF(JGLLOC)+1
          IEND=NSLOFF(JGLLOC)+NSLONL(JGLLOC)
        ENDIF
        ZSLBUF1(IST:IEND,:)=0.0_JPRB
      ENDDO
!$OMP END PARALLEL DO
    ELSE
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
      DO J=1,NFLDSLB1
        ZSLBUF1(:,J)=0.0_JPRB
      ENDDO
!$OMP END PARALLEL DO
    ENDIF
  ENDIF
  ALLOCATE (ZSLBUF15(NASLB1,NFLDSLB15))
  IF (NSLPAD > 0) THEN
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
    DO J=1,NFLDSLB15
      ZSLBUF15(:,J)=HUGE(ZZZ)
    ENDDO
!$OMP END PARALLEL DO
  ENDIF
ENDIF

IF (.NOT.ALLOCATED(ZSLBUF2)) ALLOCATE (ZSLBUF2(NPROMA,NFLDSLB2,NGPBLKS))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
DO J=1,NGPBLKS
  ZSLBUF2(:,:,J)=0.0_JPRB
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1222,1)

IF(LEMWAVE .AND. CDCONF(6:6) == 'V' .AND. MNSERIES > 0 &
 & .AND. .NOT. LTESTADJM) THEN
  LEMWAVE_ACTIVE=.TRUE.
ELSE
  LEMWAVE_ACTIVE=.FALSE.
ENDIF
ALLOCATE( ZMWAVE_TB_BUF_AD(NGPTOT,NMAX_CHAN,N_SATSENS) )
ALLOCATE( ZMWAVE_OERR_BUF(NGPTOT,NMAX_CHAN,N_SATSENS) )
ALLOCATE( ZMWAVE_PERS1_BUF(NGPTOT,NMAX_CHAN,N_SATSENS) )
ALLOCATE( ZMWAVE_ZEN_BUF(NGPTOT,N_SATSENS) )
ALLOCATE( ZMWAVE_AZI_BUF(NGPTOT,N_SATSENS) )
ALLOCATE( IMWAVE_BUF(NGPTOT,NMAX_CHAN,N_SATSENS) ) 

!    -------------------------------------------------------------------
!*       2.    TRAJECTORY HANDLING.
!    -------------------------------------------------------------------

IF (LSLAG) THEN

!*     NON-LAGGED PART OF TRAJECTORY COMPUTATIONS:
!      CPG25/CPG5 will read the saved semi-Lagrangian buffer
!      information and refill the buffers directly.

  CALL GSTATS(1011,0)
!$OMP PARALLEL PRIVATE(JKGLO,ICEND,IBL,IOFF,ZSLBUF15AUX,JFLD,JROF)

  ALLOCATE (ZSLBUF15AUX(NPROMA,NFLDSLB15))

!$OMP DO SCHEDULE(DYNAMIC,1)

  DO JKGLO=1,IGPCOMP,NPROMA
    ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    IOFF=JKGLO
    IF (.NOT.LL2D) THEN
      CALL CPG5(ICEND,JKGLO,ZSLBUF15AUX,ZSLBUF2(1,1,IBL))  
    ELSE
      CALL CPG25(CDCONF(4:4),&
       & ICEND,JKGLO,ZSLBUF15AUX,ZSLBUF2(1,1,IBL),&
       & RCORI(IOFF),GM(IOFF),&
       & GOMVRL(IOFF),GOMVRM(IOFF),&
       & OROG(IOFF),OROGL(IOFF),OROGM(IOFF),&
       & GMV5(1,1,YT5%MU,IBL)  ,GMV5(1,1,YT5%MV,IBL),&
       & GMV5(1,1,YT5%MUL,IBL) ,GMV5(1,1,YT5%MVL,IBL),&
       & GMV5(1,1,YT5%MDIV,IBL),GMV5(1,1,YT5%MVOR,IBL),&
       & GMV5S(1,YT5%MSP,IBL) ,GMV5S(1,YT5%MSPL,IBL),&
       & GMV5S(1,YT5%MSPM,IBL))  
    ENDIF
!        move data from blocked form to latitude (NASLB1) form
    DO JFLD=1,NFLDSLB15
      DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IGPCOMP)
        ZSLBUF15(NSLCORE(JROF),JFLD)=ZSLBUF15AUX(JROF-JKGLO+1,JFLD)
      ENDDO
    ENDDO

  ENDDO

!$OMP END DO
  DEALLOCATE(ZSLBUF15AUX)
!$OMP END PARALLEL
  CALL GSTATS(1011,1)
ELSE
  CALL GSTATS(1222,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NGPBLKS
    ZSLBUF2(:,:,J) = 0.0_JPRB
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1222,1)
  IF (LSPRT) THEN
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,IBL,IOFF)
    DO JKGLO=1,NGPTOT,NPROMA
      ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1
      IOFF=JKGLO
      CALL CPG5(ICEND,JKGLO,ZSLBUF15AUX,ZSLBUF2(1,1,IBL))  
    ENDDO
!$OMP END PARALLEL DO
  ENDIF
ENDIF

!    -------------------------------------------------------------------
!*       2.    TRANSFER FROM T+DT ARRAYS TO SL BUFFER
!    -------------------------------------------------------------------

CALL GSTATS(1012,0)


!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,IBL,IOFF,ICENDE)
DO JKGLO=1,IGPCOMP,NPROMA
  ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
  ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBL=(JKGLO-1)/NPROMA+1
  IOFF=JKGLO
  IF(.NOT.LL2D) THEN
    CALL CPGLAGAD(CDCONF(4:4),ICEND,&
     & ZSLBUF2(1,1,IBL), &
     & GM(IOFF),&
     & GMV(1,1,1,IBL),GMVS(1,1,IBL),GFL(1,1,1,IBL),&
     & GMVT1(1,1,1,IBL),GMVT1S(1,1,IBL))  
  ELSE
    CALL CPG2LAGAD(ICEND,GMV(1,1,1,IBL),GMVS(1,1,IBL),GM(IOFF),&
     & GMVT1(1,1,YT1%MU,IBL),GMVT1(1,1,YT1%MV,IBL))  
  ENDIF
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1012,1)

IF(LEMWAVE_ACTIVE)THEN
  CALL MWAVE_GET_AD(ZMWAVE_TB_BUF_AD,ZMWAVE_ZEN_BUF, &
     & ZMWAVE_AZI_BUF,ZMWAVE_OERR_BUF,ZMWAVE_PERS1_BUF,IMWAVE_BUF)
ENDIF

!    -------------------------------------------------------------------
!*       3.    ECMWF Physics (adjoint)
!    -------------------------------------------------------------------

IF(LEPHYS .AND. LETRAJP)THEN

  CALL GSTATS(1013,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,IBL,IOFF)

  DO JKGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    IOFF=JKGLO
    IF(.NOT.LL2D) THEN
      CALL EC_PHYS_AD(CDCONF(4:4),IBL,ICEND,&
       & JKGLO,ICEND,NSMAX,ZSLBUF2(1,1,IBL),&
       & GMV(1,1,1,IBL),GMVS(1,1,IBL),GFL(1,1,1,IBL),&
       & GMVT1(1,1,1,IBL),GMVT1S(1,1,IBL),GFLT1(1,1,1,IBL),&
       & SP_SB(1,1,1,IBL),SP_SG(1,1,IBL),SP_RR(1,1,IBL),&
       & SD_VN(1,1,IBL),SD_VD(1,1,IBL),&
       & GEMU(IOFF),GELAM(IOFF),GESLO(IOFF),&
       & GECLO(IOFF),OROG(IOFF),&
       & ZMWAVE_TB_BUF_AD,ZMWAVE_ZEN_BUF,ZMWAVE_AZI_BUF, &
       & ZMWAVE_OERR_BUF,ZMWAVE_PERS1_BUF,IMWAVE_BUF)  
    ENDIF
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1013,1)

  CALL GSTATS_BARRIER(732)

ENDIF

!    -------------------------------------------------------------------
!*       4.    SEMI_LAGRANGIAN INTERPOLATION
!    -------------------------------------------------------------------

IF (LSLAG) THEN
  IF( LSLADREP )THEN
    IGPTOT=NGPTOTAD
  ELSE
    IGPTOT=NGPTOT
  ENDIF
  ISLB2=(IGPTOT-1)/NPROMA+1

  IF( LSLADREP )THEN
    ALLOCATE(ZSLBUF2X(NPROMA,NFLDSLB2,ISLB2))
    ALLOCATE(ZGMVT1X(NPROMA,NFLEVG,YT1%NDIM,ISLB2))
    ALLOCATE(ZGMVT1SX(NPROMA,YT1%NDIMS,ISLB2))
    ALLOCATE(ZGFLT1X(NPROMA,NFLEVG,YGFL%NDIM1,ISLB2))
    CALL PRE_SLADREP(ISLB2,GFLT1,ZSLBUF2,&
     & ZGMVT1X,ZGMVT1SX,ZGFLT1X,ZSLBUF2X)  
  ENDIF

  IF(.NOT.LL2D) THEN

    IF( LSLADREP )THEN
      CALL CALL_SL_AD(IGPTOT,ZSLBUF1,ZSLBUF15,ZSLBUF2X,&
       & ZGMVT1X,ZGMVT1SX,ZGFLT1X,ISLB2,&
       & GMAD,GEMUAD,GESLOAD,GECLOAD,GNORDLAD,&
       & GNORDMAD,GSQM2AD,RCOLONAD,RSILONAD,&
       & RINDXAD,RINDYAD )  
    ELSE
      CALL CALL_SL_AD(IGPTOT,ZSLBUF1,ZSLBUF15,ZSLBUF2,&
       & GMVT1,GMVT1S,GFLT1,ISLB2,&
       & GM,GEMU,GESLO,GECLO,GNORDL,&
       & GNORDM,GSQM2,RCOLON,RSILON,&
       & RINDX,RINDY )  
    ENDIF
      
  ELSE

    DO JKGLO=1,IGPTOT,NPROMA
      ICEND=MIN(NPROMA,IGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1
      IOFF=JKGLO
      IF( LSLADREP )THEN
        CALL LADAD(ICEND,&
         & NASLB1,ZSLBUF1,ZSLBUF2X(1,1,IBL),&
         & ZGMVT1X(1,1,1,IBL),ZGMVT1SX(1,1,IBL),&
         & GEMUAD(IOFF),GESLOAD(IOFF),GECLOAD(IOFF),GNORDLAD(IOFF),&
         & GNORDMAD(IOFF),GSQM2AD(IOFF),RCOLONAD(IOFF),RSILONAD(IOFF),&
         & RINDXAD(IOFF),RINDYAD(IOFF) )  
      ELSE
        CALL LADAD(ICEND,&
         & NASLB1,ZSLBUF1,ZSLBUF2(1,1,IBL),GMVT1(1,1,1,IBL),GMVT1S(1,1,IBL),&
         & GEMU(IOFF),GESLO(IOFF),GECLO(IOFF),GNORDL(IOFF),&
         & GNORDM(IOFF),GSQM2(IOFF),RCOLON(IOFF),RSILON(IOFF),&
         & RINDX(IOFF),RINDY(IOFF) )  
      ENDIF
    ENDDO

  ENDIF

  DEALLOCATE(ZSLBUF15)

  IF( LSLADREP )THEN

!       Update core points for ZSLBUF2 from ZSLBUF2X

    CALL GSTATS(1985,0)
!$OMP PARALLEL PRIVATE(JFLD,JKGLO,IBL,JROF,ZSLBUFTEMP)
    ALLOCATE (ZSLBUFTEMP(NASLB1))
!$OMP DO SCHEDULE(STATIC)
    DO JFLD=1,NFLDSLB2
      DO JKGLO=1,NGPTOTAD,NPROMA
        IBL=(JKGLO-1)/NPROMA+1
        DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOTAD)
          ZSLBUFTEMP(NADCORE(JROF))=ZSLBUF2X(JROF-JKGLO+1,JFLD,IBL)
        ENDDO
      ENDDO
      DO JKGLO=1,NGPTOT,NPROMA
        IBL=(JKGLO-1)/NPROMA+1
        DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOT)
          ZSLBUF2(JROF-JKGLO+1,JFLD,IBL)=ZSLBUFTEMP(NSLCORE(JROF))
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
    DEALLOCATE(ZSLBUFTEMP)
!$OMP END PARALLEL

!$OMP PARALLEL PRIVATE(JFLD,JLEV,JKGLO,IBL,JROF,ZSLBUFTEMP)
    ALLOCATE (ZSLBUFTEMP(NASLB1))
!$OMP DO SCHEDULE(STATIC)
    DO JFLD=1,YT1%NDIM
      DO JLEV=1,NFLEVG
        DO JKGLO=1,NGPTOTAD,NPROMA
          IBL=(JKGLO-1)/NPROMA+1
          DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOTAD)
            ZSLBUFTEMP(NADCORE(JROF))=ZGMVT1X(JROF-JKGLO+1,JLEV,JFLD,IBL)
          ENDDO
        ENDDO
        DO JKGLO=1,NGPTOT,NPROMA
          IBL=(JKGLO-1)/NPROMA+1
          DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOT)
            GMVT1(JROF-JKGLO+1,JLEV,JFLD,IBL)=ZSLBUFTEMP(NSLCORE(JROF))
          ENDDO
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
    DEALLOCATE(ZSLBUFTEMP)
!$OMP END PARALLEL

!$OMP PARALLEL PRIVATE(JFLD,JKGLO,IBL,JROF,ZSLBUFTEMP)
    ALLOCATE (ZSLBUFTEMP(NASLB1))
!$OMP DO SCHEDULE(STATIC)
    DO JFLD=1,YT1%NDIMS
      DO JKGLO=1,NGPTOTAD,NPROMA
        IBL=(JKGLO-1)/NPROMA+1
        DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOTAD)
          ZSLBUFTEMP(NADCORE(JROF))=ZGMVT1SX(JROF-JKGLO+1,JFLD,IBL)
        ENDDO
      ENDDO
      DO JKGLO=1,NGPTOT,NPROMA
        IBL=(JKGLO-1)/NPROMA+1
        DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOT)
          GMVT1S(JROF-JKGLO+1,JFLD,IBL)=ZSLBUFTEMP(NSLCORE(JROF))
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
    DEALLOCATE(ZSLBUFTEMP)
!$OMP END PARALLEL

!$OMP PARALLEL PRIVATE(JFLD,JLEV,JKGLO,IBL,JROF,ZSLBUFTEMP)
    ALLOCATE (ZSLBUFTEMP(NASLB1))
!$OMP DO SCHEDULE(STATIC)
    DO JFLD=1,YGFL%NDIM1
      DO JLEV=1,NFLEVG
        DO JKGLO=1,NGPTOTAD,NPROMA
          IBL=(JKGLO-1)/NPROMA+1
          DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOTAD)
            ZSLBUFTEMP(NADCORE(JROF))=ZGFLT1X(JROF-JKGLO+1,JLEV,JFLD,IBL)
          ENDDO
        ENDDO
        DO JKGLO=1,NGPTOT,NPROMA
          IBL=(JKGLO-1)/NPROMA+1
          DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,NGPTOT)
            GFLT1(JROF-JKGLO+1,JLEV,JFLD,IBL)=ZSLBUFTEMP(NSLCORE(JROF))
          ENDDO
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
    DEALLOCATE(ZSLBUFTEMP)
!$OMP END PARALLEL
    CALL GSTATS(1985,1)

    DEALLOCATE(ZSLBUF2X)
    DEALLOCATE(ZGMVT1X)
    DEALLOCATE(ZGMVT1SX)
    DEALLOCATE(ZGFLT1X)

  ENDIF

ENDIF

!    -------------------------------------------------------------------
!*       5.    DYNAMICS
!    -------------------------------------------------------------------

CALL GSTATS(1016,0)

IF(.NOT.LL2D) THEN

  CALL CPGAD(&
   & CDCONF(4:4),IGPCOMP,&
   & GMV,GMVS,&
   & ZSLBUF1,ZSLBUF2,GMVT1,GMVT1S,GFLT1, &
   & GMV5,GMV5S)

ELSE

!$OMP PARALLEL PRIVATE(JKGLO,ICEND,IBL,IOFF,&
!$OMP&ZSLBUF1AUX,JROF,JFLD,ICENDE)

  ALLOCATE (ZSLBUF1AUX(NPROMA,NFLDSLB1))

!$OMP DO SCHEDULE(DYNAMIC,1)

  DO JKGLO=1,IGPCOMP,NPROMA
    ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
    ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    IOFF=JKGLO

    IF(LSLAG.AND.((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)))&
     & THEN
! Move data from blocked form to latitude (NASLB1) form
      DO JFLD=1,NFLDSLB1
        DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IGPCOMP)
          ZSLBUF1AUX(JROF-JKGLO+1,JFLD)=ZSLBUF1(NSLCORE(JROF),JFLD)
        ENDDO
      ENDDO
    ENDIF

    CALL CPG2AD(CDCONF(4:4),ICEND,&
     & ZSLBUF1AUX,ZSLBUF2(1,1,IBL),GMV(1,1,1,IBL),GMVS(1,1,IBL),&
     & RCORI(IOFF),GM(IOFF),RATATH(IOFF),RATATX(IOFF),&
     & OROG(IOFF),OROGL(IOFF),OROGM(IOFF),&
     & GMV(1,1,YT0%MU,IBL) ,GMV(1,1,YT0%MV,IBL),&
     & GMV(1,1,YT0%MUL,IBL),GMV(1,1,YT0%MVL,IBL),&
     & GMV(1,1,YT0%MDIV,IBL),&
     & GMV(1,1,YT0%MVOR,IBL),GMVS(1,YT0%MSP,IBL),&
     & GMVS(1,YT0%MSPL,IBL),GMVS(1,YT0%MSPM,IBL),&
     & GMV(1,1,YT9%MU,IBL) ,GMV(1,1,YT9%MV,IBL),&
     & GMV(1,1,YT9%MDIV,IBL),&
     & GMVS(1,YT9%MSP,IBL),&
     & GMVS(1,YT9%MSPL,IBL),GMVS(1,YT9%MSPM,IBL),&
     & GMVT1(1,1,YT1%MU,IBL)  ,GMVT1(1,1,YT1%MV,IBL),&
     & GMVT1S(1,YT1%MSP,IBL),&
     & GMV5(1,1,YT5%MU,IBL)  ,GMV5(1,1,YT5%MV,IBL),&
     & GMV5(1,1,YT5%MUL,IBL) ,GMV5(1,1,YT5%MVL,IBL),&
     & GMV5(1,1,YT5%MDIV,IBL),GMV5(1,1,YT5%MVOR,IBL),&
     & GMV5S(1,YT5%MSP,IBL) ,GMV5S(1,YT5%MSPL,IBL),&
     & GMV5S(1,YT5%MSPM,IBL))  
  ENDDO

!$OMP END DO
  DEALLOCATE(ZSLBUF1AUX)
!$OMP END PARALLEL

ENDIF

CALL GSTATS(1016,1)

!    -------------------------------------------------------------------
!*       6.    DEALLOCATIONS
!    -------------------------------------------------------------------

IF (ALLOCATED(ZSLBUF1)) DEALLOCATE(ZSLBUF1)
IF (ALLOCATED(ZSLBUF15)) DEALLOCATE(ZSLBUF15)
IF (ALLOCATED(ZSLBUF2)) DEALLOCATE(ZSLBUF2)

DEALLOCATE( ZMWAVE_TB_BUF_AD, ZMWAVE_OERR_BUF, ZMWAVE_ZEN_BUF, IMWAVE_BUF ) 
DEALLOCATE( ZMWAVE_AZI_BUF,ZMWAVE_PERS1_BUF)

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GP_MODEL_AD',1,ZHOOK_HANDLE)
END SUBROUTINE GP_MODEL_AD
