SUBROUTINE RESTART_CNT3

!     Purpose:
!     --------
!       Reset all model fields and a few other things before
!       new call to CNT3. List of fields to reset from restart
!       file (WRRESF).

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       31-Oct-03: Original
!       17-12-2004: G. Radnoti: REACT/DEACT-CLOUD
!        M.Hamrud      01-Jul-2006 Revised surface fields
!       JJMorcrette 20070321 Prognostic aerosols in rad and clouds
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMSP   , ONLY: SPA2, SPA3, SPOR
USE YOMGFUB , ONLY: GFUBUF
USE YOMXFUB , ONLY: XFUBUF
USE YOMRADF , ONLY: EMTD, TRSW, EMTC, TRSC, SRSWDCS, SRSWD, EDRO, TAUAER
USE YOMSLPHY, ONLY: SAVTEND, LSLPHY
USE YOEPHY  , ONLY: LEPHYS , LEPCLD
USE YOPHNC  , ONLY: LERADSW2
USE YOERAD  , ONLY: NSW, NTSW, NSWNL, NSWTL
USE YOMDIM  , ONLY: NFLEVG
USE YOEPHLI , ONLY: LPHYLIN
USE SURFACE_FIELDS_MIX , ONLY : SURF_STORE, SURF_RESTORE, SURF_STORE_ARRAY
USE YOM_YGFL, ONLY: YGFL
USE YOMGFL  , ONLY: GFL, GFLSLP
USE YOMGMV  , ONLY: GMV, GMVS
USE YOMVRTL , ONLY: LTLINT
USE GFL_SUBS_MOD, ONLY: DEACT_CLOUD_GFL,REACT_CLOUD_GFL
IMPLICIT NONE
INTEGER(KIND=JPIM) :: isw
REAL(KIND=JPRB) :: zzz
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL :: llnull

#include "su2yom.intfb.h"
#include "suaersn.intfb.h"
#include "suclopn.intfb.h"
#include "sucos.intfb.h"
#include "suswn.intfb.h"
#include "sugridf.intfb.h"
#include "sugridu.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RESTART_CNT3',0,ZHOOK_HANDLE)
CALL SU2YOM
CALL SUCOS 

IF (LEPHYS.AND.LERADSW2) THEN
  IF (LPHYLIN) THEN
    ISW=NSWTL
  ELSE
    ISW=NSWNL
  ENDIF
  IF (NSW/=ISW) THEN
    NSW=ISW
    CALL SUSWN  (NTSW, NSW)
    CALL SUCLOPN(NTSW, NSW,NFLEVG)
    CALL SUAERSN(NTSW, NSW)
  ENDIF
ENDIF

! Spectral data
SPA2=0.0_JPRB
SPA3=0.0_JPRB

! Upper-air GFL data
GMV =0.0_JPRB
GMVS=0.0_JPRB
GFL =0.0_JPRB
IF(ALLOCATED(GFLSLP)) then
GFLSLP=0.0_JPRB
ENDIF

! Surface grid-point data
CALL SURF_STORE
SURF_STORE_ARRAY(:,:,:) = HUGE(zzz)
CALL SURF_RESTORE

! Cumulated and instantaneous fluxes grid-point data
CALL WIPE_GPBUF(GFUBUF)
CALL WIPE_GPBUF(XFUBUF)

! Additional tendencies and radiation field data
IF (LEPHYS) THEN
  EMTD=0.0_JPRB
  TRSW=0.0_JPRB
  EMTC=0.0_JPRB
  TRSC=0.0_JPRB
  SRSWDCS=0.0_JPRB
  SRSWD=0.0_JPRB
  EDRO=0.0_JPRB
  TAUAER=0.0_JPRB
  IF (LSLPHY) THEN
    SAVTEND=0.0_JPRB
  ENDIF
ENDIF
IF (.NOT.LTLINT) THEN
  IF (LEPHYS) THEN
    IF (LEPCLD) THEN
      CALL REACT_CLOUD_GFL
    ELSE
      CALL DEACT_CLOUD_GFL
    ENDIF
  ENDIF
  CALL SUGRIDF(1,SPOR,LLNULL)
  IF (YGFL%NUMGPFLDS>0) THEN
    CALL SUGRIDU(1)
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('RESTART_CNT3',1,ZHOOK_HANDLE)

! ------------------------------------------------------------------
CONTAINS

SUBROUTINE WIPE_GPBUF(YD_PGPBUF)
USE GRIDPOINT_BUFFERS_MIX, ONLY: gridpoint_buffer
TYPE (gridpoint_buffer), INTENT(INOUT) :: YD_PGPBUF
REAL(KIND=JPRB) :: zzz
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('RESTART_CNT3:WIPE_GPBUF',0,ZHOOK_HANDLE)

IF (ASSOCIATED(YD_PGPBUF%GPBUF)) THEN
  YD_PGPBUF%GPBUF(:)= HUGE(zzz)
  YD_PGPBUF%LFILLED = .FALSE.
ENDIF

IF (LHOOK) CALL DR_HOOK('RESTART_CNT3:WIPE_GPBUF',1,ZHOOK_HANDLE)
END SUBROUTINE WIPE_GPBUF

! ------------------------------------------------------------------

END SUBROUTINE RESTART_CNT3
