SUBROUTINE SCAN2HAD(CDCONF)

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK


!**** *SCAN2HAD* - Caller for SCAN2MAD

!     Purpose.
!     --------
!        Prepare for calling SCAN2MAD by setting
!        up IO-scheme etc. Call SCAN2MAD - multi-tasking interface
!        to Fourier and grid-point computations.

!**   Interface.
!     ----------
!        *CALL* *SCAN2HAD(CDCONF)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)
!        --------------------

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   SCAN2MAD  - comp. in fourier/grid-point space
!     ----------   

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-03-01
!        D. Giard : 93-05-24 CDCONF(8)='T'
!        Modified : 93-01-18 M.Hamrud (use only one Fourier file)
!        Modified : 95-10-30 M.Hamrud (Change CDCONF def.)
!        Modified : 96-02-20 L.Isaksen (Message Passing option added)
!        Modified : 97-10-02 D.Dent    (SECOND replaced by USER_CLOCK)
!        Modified : 98-10-22 C.Temperton (adjoint of semi-Lagrangian)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!        - removal of LRPOLE in YOMCT0 and code under LRPOLE.
!        Modified : 99-03-15 C.Temperton (adjoint of s-L, LMESSP=.T.)
!        Modified : 99-11-22 C.Temperton (adjoint of s-L, bugfix)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): prune enhanced diffusion (lfrein).
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "scan2mad.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SCAN2HAD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    CALL COMPUTATIONS DONE IN GRID-POINT SPACE.
!              -------------------------------------------

CALL SCAN2MAD(CDCONF)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SCAN2HAD',1,ZHOOK_HANDLE)
END SUBROUTINE SCAN2HAD
