SUBROUTINE SCAN2HTL(CDCONF)

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!**** *SCAN2HTL* - Caller for SCAN2MTL

!     Purpose.
!     --------
!        Prepare for calling SCAN2MTL by setting
!        up IO-scheme etc. Call SCAN2MTL - multi-tasking interface
!        to Fourier and grid-point computations.

!**   Interface.
!     ----------
!        *CALL* *SCAN2HTL(CDCONF)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)
!        --------------------

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   SCAN2MTL  - comp. in fourier/grid-point space
!     ---------- 

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-03-01
!        Modified : 93-01-18 M.Hamrud (use only one Fourier file)
!        Modified : 94-07-28 J.N. Thepaut (alloc obs arrays for incremental)
!        Modified : 95-10-30 M.Hamrud (Changed CDCONF form)
!        Modified : 95-12-05 F. Bouttier (alloc trajectory obs arrays)
!        Modified : 96-02-20 L.Isaksen (Message Passing option added)
!        Modified : 97-10-02 D. Dent   (replace SECOND by USER_CLOCK)
!        Modified : 98-06-16 C. Temperton (TL of semi-Lagrangian)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!        - removal of LRPOLE in YOMCT0 and code under LRPOLE.
!        Modified : 99-03-15 C. Temperton (TL of s-L, LMESSP=.T.)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): prune enhanced diffusion (lfrein).
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "scan2mtl.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SCAN2HTL',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

CALL SCAN2MTL(CDCONF)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SCAN2HTL',1,ZHOOK_HANDLE)
END SUBROUTINE SCAN2HTL

