#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE SCAN2M(CDCONF)

!****-------------------------------------------------------------------
!**** *SCAN2M* - Grid-point space computations
!****-------------------------------------------------------------------
!     Purpose.   Computations in grid-point space
!     --------   

!**   Interface.
!     ----------
!        *CALL* *SCAN2M (..)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)
!        --------------------  

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   See includes below.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud   *ECMWF*

! Modifications
! -------------
!   L. Isaksen 97-05-05 : Distributed memory version of SCAN2MSM
!   E.Andersson 97-11-17 : Introduce O3 and CIW to obs points
!   E. Holm    97-12-18  : Ozone included in assimilation/Eulerian model
!   L. Isaksen  97-05-05: Distributed memory version of SCAN2MSM
!   P.Le Moigne 97-12-02: cdconf(3:3)='2' in case of canari and Aladin
!   R.El Khatib 97-06-02: 3 NH fields
!   F.Taillefer 98-02-04: CANARI MP development (CAPOTX call)
!   K. YESSAD (JAN 1998): merge of some FULL-POS and SL routines
!      relative to the halo.
!      - FPEXTPOL and SLEXTPOL merged into SLEXTPOL.
!      - FPREAD,FPWRIT and SLCOMM merged into SLCOMM.
!      - call to FPADD3P replaced by a simple memory transfer.
!   K. YESSAD 98-07-31: Removal of useless RSQM2 in 'CALL CPG'.
!      Modified 98-08-10 by K. YESSAD : removal of LRPOLE option.
!      - removal of false comments about calls to SC2GTP.
!   R.El Khatib 98-08-27: cleanup adiabatic pp.
!   98-10-08 R.El Khatib remove useless ISTGLO while calling
!            ENDVPOS or HPOSLAG ; fullpos communications only if
!            NFPWIDE >0
!   G. Mozdzynski 98-10-27: save grid-point trajectory
!   D. Dent       99-05-06: support for log surface pressure in FULL-POS
!   D.Salmond     99-08   : Remove NSLBCT for LMESSP=.TRUE.
!   99-04-01 R.El Khatib use ZGP(1,IEVT0 ,IBL) in fullpos
!   Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!   Modified 98-12-02 by K. YESSAD: new option LRADVTORO.
!   R.El Khatib   00-04-18 Allocation of fullpos buffer HFPBUF moved from
!                          stepo saves memory
!   D. Dent & D.Salmond 99-09-27: 'On demand' SL communications
!   00-07-28 R.El Khatib fullpos buffer 2 only in new conf. 'G' ; 
!                        reorganize fullpos conf(5:5)
!   D. Dent       00-04-12: Support for computation of 
!                           Montgomery geopotential in full-pos
!   C. Temperton 00-02-11: computation of NVSEP (used in S/L adjoint)
!   C. Temperton 00-06-15: NVSEP -> NVSEPC,NVSEPL.
!   Y. Tremolet  00-08-09: use module TRAJECTORY
!   M. Fisher    00-10-26: wavelet Jb and Jb vertocal coordinate
!   M. Hamrud    00-10-25: Transform package
!   C. Fischer   01-03-06: Merge LAM version into global code
!   K. YESSAD    01-08-20: complete rewriting and cleaning of CPG.
!   C. Fischer   01-11-15: small bugfix for aladin Jo computation (AL15_op2)
!   C. Fischer   01-03-06: Merge LAM version into global code
!   C. Temperton 01-07-12: repair shallow-water code
!   D.Salmond    01-06-20: Introduce GP_MODEL
!   Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!   R. El Khatib 01-08-07: Pruning options
!   J. Vivoda  03-2002     PC schemes for NH dynamics (LPC_XXXX keys)
!   P. Smolikova 02-09-30 : variable d4 in NH (GPPAUX etc.)
!   G. Desroziers 02-09-16 : also treat Aladin mean wind trajectory
!   E. Holm      01-08-24: Move JGNR/I and EJGNRGG/I to SCAN2MTL
!                          because of new control variable for humidity
!   R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!   G.Mozdzynski: 02-10-01 support for radiation on-demand comms
!   C. Fischer   03-07-16: gppaux becomes part of gmv
!   O.Spaniel    03-04-15: cleaning-a same named entity from modules
!   R. El Khatib : 03-04-17 Fullpos improvemnts
!   C. Fischer : 03-06-03 - use igpcomp/ngptot_cap in call to cobs for LAM/Jo
!   R. El Khatib : 03-08-08 fullpos & gfl
!   G. Radnoti: 03-10-01 : update calls to eslextpol1
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   05-Jan-2004 Y. Seity   time stepping only for not coupled GFL(bug LGPQ=T)
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!   27-Feb-2004 M. Tudor   time stepping only for during predictor     
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests about PC schemes.
!   R. El Khatib : 04-10-21 Fullpos fix for extra gfl
!   14-Mar-2005 R. Engelen   Fullpos of greenhouse gases
!   14-Mar-2005 A. Untch   Fullpos of aerosols
!        Y.Tremolet    01-Aug-2005 Number sections consistently with TL and AD
!    Y.Bouteloup : 05-01-19 : Add QL QI in the call to capotx
!    Y.Bouteloup : 05-01-31 : Add YR in the call to VPOS
!   Y. Seity : 04-11-16: new arguments to CTVTOT for R and Cp calc.
!   G. Hello: 05-01-26 Fullpos fix for arome gfl
!        M.Hamrud  15-Jan-2005  Revised GPRCP
!   G. Desroziers and K. Yessad (sept 2005):
!    - split option LTRAJHR into LTRAJHR_ALTI and LTRAJHR_SURF.
!    - adapt option LTRAJHR to METEO-FRANCE configurations.
!   R. El Ouaraini & R. El Khatib : 05-07-28 Allocate CUF fields and CALL fpinvtrcuf.
!   R. El Khatib  20-May-2005 NFPWIDE moved to YOMWFPDS
!    Y.Bouteloup : 05-12-25 : Add QR QS in the call to capotx
!    O.Spaniel :   22-02-2006 : Phasing CY31
!   M. Bellus :   29-Sep-2006 : Add prognostic convection variables (ALARO-0)
!    - YDAL, YDOM, YUAL, YUOM, YUEN, YUNEBH in the call to VPOS
!   11-Apr-2005 J. Flemming Aerosol replaced with GRG (global ractive gases)
!   02-May-2005 A. Dethof  More changes for GRG fields
!   E. Holm      05-10-20: Move jbvcoord_interpolate to cvargptl
!   08-08-2006  S. Serrar diagnostic tracers (TRAC) added to GFL fields
!        M.Hamrud      01-Jul-2006  Revised surface fields
!   K. Yessad (oct 2006): bug correction for LTRAJHR in part 2.
!   J.Haseler 27-Feb-2007  Generalise read of gridpoint trajectory
!   S. Serrar 22-03-2007: changes for the post-processing of ERA40 GFL fields
!   S. Serrar 17-07-2007 post-processing methane related GFL fields
!   S. Serrar 02-05-2008 test on LERA40 removed
!   Y. Seity  11-01-2008: bf for non-coupled GFL not initialized in E zone
!   R. El Khatib  17-Oct-2008 bf for ALADIN fullpos with equal distribution
!                             of C+I+E (RDISTR_E=1.)
!   K. Yessad Dec 2008: merge SLCOMM+SLCOMM1 -> SLCOMM.
!   K. Yessad Dec 2008: merge the different (E)SLEXTPOL.. -> (E)SLEXTPOL.
!   H. Hersbach Dec 2009: Determine ZPARITY in cobs
!-----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN    ,ONLY : NULOUT 
USE YOMDIM   , ONLY : NFLEVG   ,NDGLG    ,NDGSAG   ,NDGENG   ,&
 & NPROMA   ,NGPBLKS  ,NDGSAL   ,&
 & NDGENL  
USE YOMCT0   , ONLY : NCONF    ,NPROC    ,NSTOP    ,LIFSMIN  ,LSPRT    ,&
 & LOBS     ,LOBSC1   ,LOBSREF  ,LELAM  ,LPC_FULL, LIFSTRAJ  ,LFPART2
USE YOMCT3   , ONLY : NSTEP
USE YOMDFPB  , ONLY : HFPBUF   ,NLENHFPB ,NSTAHFPB
USE YOMGEM   , ONLY : NGPTOT   ,NLOENG   ,NGPTOT_CAP
USE YOMDYN   , ONLY : NCURRENT_ITER
USE YOMGC    , ONLY : RCORI    ,GEMU     ,GELAM    ,GM       ,&
 & GNORDL   ,GNORDM   ,OROG     ,OROGL    ,OROGM    ,GELAT      
USE YOMSC2   , ONLY : NFLDOBB1 ,NSLWIDE
USE YOMWFPDS , ONLY : NFPWIDE
USE YOMFPDIM , ONLY : NFPGT1   ,NFPDYNB  ,NFPBFIN
USE YOMFPSC2 , ONLY : NFPROMA  ,NFPBLOCS
USE YOMMP    , ONLY :    NFPCORE   ,&
 & NASLB1    ,NAFPB1    ,NFPPROCS  ,&
 & NFPRPT    ,&
 & NFPSPT    ,NFPSENDPOS,&
 & NFPRECVPOS,NFPSENDPTR,&
 & NFPRECVPTR,NFPCOMM   ,&
 & NSLSTA    ,NFPSTA    ,NSLONL    ,NFPONL    ,&
 & NSLOFF    ,NFPOFF    ,&
 & NFRSTLOFF ,MYFRSTACTLAT         ,MYLSTACTLAT  
USE YOMOMPDIST , ONLY : GPP
USE YOMGFL     , ONLY : GFL,GFLT1
USE QASSET   , ONLY : ESIG
USE QAGPSF   , ONLY : RSGPHI   ,RSGHUM
USE QALORI   , ONLY : QCAGUE
USE YOMMASK  , ONLY : NFIXSFLD, LSLONDEM_ACTIVE, MASK_SL2
USE YOMSP    , ONLY : SPA3,   SPSP,   SPA1
USE TRAJECTORY_MOD, ONLY: LTRAJSAVE, LTRAJCST, LTRAJHR, LREADGPTRAJ, &
 & LTRAJHR_ALTI,LTRAJHR_SURF, &
 & GET_TRAJ_GRID, &
 & STORE_TRAJ_MAIN, STORE_TRAJ_SFC, STORE_TRAJ_CST,  FORCE_SFC_TRAJ
USE SURFACE_FIELDS_MIX, ONLY : YSP_SGD, SP_SG, YSP_RRD,SP_RR, YSD_VVD, SD_VV,&
 & YSD_VFD, SD_VF, YSP_CID,YSP_CI, SP_CI, YSP_X2D, SP_X2, YSP_SBD, SP_SB, &
 & YSD_VXD, SD_VX
USE YOMLCZ   , ONLY : LFORCEWR ,GPFORCEU ,GPFORCEV ,GPFORCET ,&
 & GPFORCEQ ,GPFORCESP  
USE TESTVAR_MIX, ONLY: LTESTINC
USE YOM_YGFL , ONLY : YGFL     ,YGFLC    ,YQ, YO3, YL, YI, YA,&
 & YS, YR, YG, YH, YTKE, YEXT, NGFL_EXT,YAERO, NAERO,&
 & YDAL, YDOM, YUAL, YUOM, YUEN, YUNEBH,&
 & YGHG, NGHG, YGRG, NGRG,&
 & YTRAC,NTRAC, YERA40 , NERA40, YLRCH4, YCH4S
USE YOMGMV , ONLY : GMV,GMVS,GMVT1,GMVT1S,YT0,YT5
USE YOMVAR, ONLY: LMODERR
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, GPMODERR
USE YOMFPG , ONLY : LFPOSBUF

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9),INTENT(IN) :: CDCONF

!     ------------------------------------------------------------------

REAL(KIND=JPRB), ALLOCATABLE :: ZOBBUF1(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZFPBUF1AUX(:,:), ZFPBUF1(:,:)
REAL(KIND=JPRB) ::  ZPARITY(NFLDOBB1)
REAL(KIND=JPRB), ALLOCATABLE :: ZPARFP1(:)

REAL(KIND=JPRB),ALLOCATABLE :: ZUT(:,:),ZVT(:,:),ZTT(:,:),ZQT(:,:),ZLT(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZIT(:,:),ZRT(:,:),ZST(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSP_SB(:,:,:),ZSP_SG(:,:),ZSP_RR(:,:),ZSP_CI(:,:),ZSP_X2(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSD_VF(:,:),ZSD_VV(:,:),ZSD_VX(:,:)
REAL(KIND=JPRB),DIMENSION(:),ALLOCATABLE :: ZPS,ZRCORI,ZGEMU
REAL(KIND=JPRB),DIMENSION(:),ALLOCATABLE :: ZGM,ZGELAT,ZGELAM,ZGNORDL,ZGNORDM
REAL(KIND=JPRB),DIMENSION(:,:),ALLOCATABLE :: ZCAGUE,ZESIG,ZSGPHI,ZSGHUM

REAL(KIND=JPRB) :: ZERA40(NPROMA,NFLEVG,NERA40)
REAL(KIND=JPRB) :: ZLRCH4(NPROMA,NFLEVG), ZCH4S(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGHG(NPROMA,NFLEVG,NGHG)
REAL(KIND=JPRB) :: ZTRAC(NPROMA,NFLEVG,NTRAC)
REAL(KIND=JPRB) :: ZAERO(NPROMA,NFLEVG,NAERO)
REAL(KIND=JPRB) :: ZGRG(NPROMA,NFLEVG,NGRG)
REAL(KIND=JPRB) :: ZGFL(NPROMA,NFLEVG,NGFL_EXT)
REAL(KIND=JPRB) :: ZGFLL(NPROMA,NFLEVG,NGFL_EXT), ZGFLM(NPROMA,NFLEVG,NGFL_EXT)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL5(:,:,:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTEGER(KIND=JPIM) :: IBL, ICEND, IEND, IEND1, INBPT, INDEX, &
 & IOFF, IOFFBL, IPCT, IST, IST1, ITASK, &
 & JKGLO, JROF, JLEV, IBLK, JL, JI, &
 & IGPCOMP, JGFL, ICENDE, IFPCOMP
INTEGER(KIND=JPIM) :: IDUMARR(2)
INTEGER(KIND=JPIM) :: IEXT_MP(NGFL_EXT), IEXT_MPL(NGFL_EXT),IEXT_MPM(NGFL_EXT)

LOGICAL :: LL2D, LLADNMI, LLDERR, LLHFP, LLJGNR, LLINC, LLVFP, LLMODEL
LOGICAL :: LLEXT_ACTIVE(NGFL_EXT), LLEXT_CDERS(NGFL_EXT)
LOGICAL :: LLGHG_ACTIVE(NGHG) , LLTRAC_ACTIVE(NTRAC)
LOGICAL :: LLAERO_ACTIVE(NAERO)
LOGICAL :: LLGRG_ACTIVE(NGRG)
LOGICAL :: LLTRAJHR
LOGICAL :: LLUD, LLDD, LL_LRCH4, LL_CH4S

!     ------------------------------------------------------------------

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"
#include "add_modbias_tl.intfb.h"
#include "suallocuf.intfb.h"
#include "capotx.intfb.h"
#include "cobs.intfb.h"
#include "cobslag.intfb.h"
#include "ctvtot.intfb.h"
#include "deallocuf.intfb.h"
#include "ebipos.intfb.h"
#include "endvpos.intfb.h"
#include "eslextpol.intfb.h"
#include "fpinvtrcuf.intfb.h"
#include "gpnorm_gfl.intfb.h"
#include "gp_model.intfb.h"
#include "hpos.intfb.h"
#include "fposhor.intfb.h"
#include "ini1scan2m.intfb.h"
#include "save_test4dinc.intfb.h"
#include "slcomm.intfb.h"
#include "slextpol.intfb.h"
#include "spnorm.intfb.h"
#include "sufpsc2b.intfb.h"
#include "sujbvcoord.intfb.h"
#include "vpos.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SCAN2M',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*       1.    INITIAL SETUP, MAIN LOOP CONTROL.
!     ------------------------------------------------------------------

!*       1.1   INITIAL SETUP

CALL INI1SCAN2M(CDCONF,NCONF,LLDERR,LL2D,LLVFP,LLHFP,LLJGNR)

LLINC=.FALSE.

! Try to guess when we are in an actual model time step
! and do it only once per time step...
LLMODEL =     (CDCONF(1:1)=='0' .OR. CDCONF(1:1)=='A' .OR. CDCONF(1:1)=='T') &
      & .AND. (CDCONF(2:2)=='A') &
      & .AND. (CDCONF(3:3)=='A') &
      & .AND. (CDCONF(4:4)=='0' .OR. CDCONF(4:4)=='A') &
      & .AND. (CDCONF(5:5)=='0' .OR. CDCONF(5:5)=='B') &
      & .AND. (CDCONF(6:6)=='0' .OR. CDCONF(6:6)=='V')

! Total number of grid points - ndglg*ndlon in global / ndguxg*ndlon in LAM
! used for model computations
IF (LELAM) THEN
  IGPCOMP=MIN(NGPTOT,NGPTOT_CAP)
ELSE
  IGPCOMP=NGPTOT
ENDIF
IF (LELAM.AND.LFPOSBUF) THEN
  IFPCOMP=MIN(NGPTOT,NGPTOT_CAP)
ELSE
  IFPCOMP=NGPTOT
ENDIF

LL_LRCH4 = YLRCH4%LACTIVE
LL_CH4S = YCH4S%LACTIVE

IPCT=1

!     ------------------------------------------------------------------
!*       2.    INITIAL GRIDPOINT COMPUTATIONS
!     ------------------------------------------------------------------

IF (.NOT. LFORCEWR) CALL GSTATS(8,0)

IF (CDCONF(3:3) == 'C' .OR. CDCONF(3:3) == 'D') THEN
  CALL ABOR1('SCAN2M: CDCONF(3:3)= C or D not supported')
ENDIF

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

  ! Forcing surface fields with interpolated trajectory

  IF (LIFSMIN.AND.(LTRAJHR.AND.LTRAJHR_SURF).AND.LTRAJSAVE) THEN

    ! k.y.: this block has been validated at least at ECMWF
    !       and has to be checked by GMAP/PROC people for
    !       MF applications validity.

    IST =1
    IBLK=0
!    if(nstep == 0) then ! Reproduce old bug
    DO JKGLO=1,NGPTOT,NPROMA
      IBLK=IBLK+1
      IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
      CALL FORCE_SFC_TRAJ(NSTEP,IBLK,IST,IEND)
    ENDDO
!    endif

  ENDIF

  ! Forcing upper air fields with interpolated trajectory
  !  in the case LREADGPTRAJ=T.

  IF (LIFSMIN.AND.(LTRAJHR.AND.LTRAJHR_ALTI).AND.LTRAJSAVE) THEN

    IF (LREADGPTRAJ) THEN
     
      ALLOCATE (ZGMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
      ALLOCATE (ZGMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
      ALLOCATE (ZGFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
      CALL GET_TRAJ_GRID(ZGMV5,ZGMV5S,ZGFL5,KSTEP=NSTEP)

      DO JGFL=1,YGFL%NDIM5
        IF (YGFLC(JGFL)%LTRAJIO) THEN
          GFL(:,:,YGFLC(JGFL)%MP,:)=ZGFL5(:,:,YGFLC(JGFL)%MP,:)
        ENDIF
      ENDDO

      DEALLOCATE(ZGMV5)
      DEALLOCATE(ZGMV5S)
      DEALLOCATE(ZGFL5)
    ENDIF
    WRITE(NULOUT,*) 'NORMS IN SCAN2M AFTER HR TRAJECTORY FORCING'
    CALL SPNORM(0)
    CALL GPNORM_GFL
  ENDIF

ENDIF
!*       2.1  FORCE

IF (LFORCEWR) THEN
  DO JKGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    DO JLEV=1,NFLEVG
      DO JL=1,ICEND
        GPFORCEU(JL,JLEV,IBL,0)=GMV(JL,JLEV,YT0%MU,IBL)
        GPFORCEV(JL,JLEV,IBL,0)=GMV(JL,JLEV,YT0%MV,IBL)
        GPFORCET(JL,JLEV,IBL,0)=GMV(JL,JLEV,YT0%MT,IBL)
        GPFORCEQ(JL,JLEV,IBL,0)=GFL(JL,JLEV,YQ%MP,IBL)
      ENDDO
    ENDDO
    DO JL=1,ICEND
      GPFORCESP(JL,IBL,0)    =GMVS(JL,YT0%MSP,IBL)
    ENDDO
  ENDDO
  IF (LHOOK) CALL DR_HOOK('SCAN2M',1,ZHOOK_HANDLE)
  RETURN
ENDIF

!*       2.2 CONVERSION OF TTx ARRAY FROM 'TV' TO T

IF (LSPRT) THEN
  CALL GSTATS(1033,0)
  IF (CDCONF(3:3)=='A'.OR.CDCONF(3:3)=='B'.OR.CDCONF(3:3)=='1') THEN
!$OMP PARALLEL DO SCHEDULE(STATIC,1) PRIVATE(JKGLO,ICEND,IBL)
    DO JKGLO=1,NGPTOT,NPROMA
      ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1
      CALL CTVTOT(1,ICEND,GMV(1,1,YT0%MT,IBL),GFL(1,1,1,IBL))
    ENDDO
!$OMP END PARALLEL DO
  ENDIF
  CALL GSTATS(1033,1)
ENDIF

!*       2.4 Save nonlinear fields for inner/outer loop tests
IF (LTESTINC .and. NCONF==1 .and. NSTOP>1 .and. CDCONF(5:5)=='0') THEN
  CALL SAVE_TEST4DINC(NSTEP)
ENDIF

!     ------------------------------------------------------------------
!*       3.    TRAJECTORY HANDLING
!     ------------------------------------------------------------------

! Store surface and grid-point trajectory
! ---------------------------------------

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN
  IF (LTRAJSAVE.AND.LTRAJCST) THEN
!   This is a dirty fix.
!   The trajectory should not contain any constants.
    IST =1
    IBLK=0
    DO JKGLO=1,NGPTOT,NPROMA
      IBLK=IBLK+1
      IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
      CALL STORE_TRAJ_CST(NSTEP,IBLK,IST,IEND)
    ENDDO
  ENDIF
ENDIF

IF (NCURRENT_ITER == 0) THEN

  ! Saving main trajectory

  LLTRAJHR=LTRAJHR.AND.LTRAJHR_ALTI

  IF (LTRAJSAVE.AND. &
     & ((LLTRAJHR.AND.LIFSTRAJ).OR.(.NOT.LLTRAJHR.AND.LIFSMIN))) THEN

    ! Write out main upper air trajectory at inner-loop resolution 
    ! in either spectral space (VOR,DIV,T,Q,(O3)) or in grid point 
    ! space (Q,(O3)). Controlled by LREADGPTRAJ
    CALL STORE_TRAJ_MAIN(SPA3,SPSP,SPA1,GMV,GMVS,GFL,NSTEP,LREADGPTRAJ)

  ENDIF

  ! Saving surface trajectory

  LLTRAJHR=LTRAJHR.AND.LTRAJHR_SURF

  IF (LTRAJSAVE.AND. &
     & ((LLTRAJHR.AND.LIFSTRAJ).OR.(.NOT.LLTRAJHR.AND.LIFSMIN))) THEN

    ! Write out surface fields. This is always done in grid point space.

    IST =1
    IBLK=0
    DO JKGLO=1,NGPTOT,NPROMA
      IBLK=IBLK+1
      IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
      CALL STORE_TRAJ_SFC(NSTEP,IBLK,IST,IEND)
    ENDDO
  ENDIF
ENDIF


IF (CDCONF(6:6) == 'V'.OR.CDCONF(6:6) == 'C'.OR.CDCONF(6:6) == 'F') THEN

!*       4.1 NON-LAGGED PART OF COMPUTATIONS

  IF (.NOT.ALLOCATED(ZOBBUF1)) ALLOCATE(ZOBBUF1(NASLB1,NFLDOBB1))
  CALL GSTATS(1034,0)
  IF ((CDCONF(6:6)=='C').OR.(LOBS.AND.(LOBSC1.OR.LOBSREF))) THEN

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JKGLO,ICEND,ICENDE,IBL,IOFF)
    DO JKGLO=1,IGPCOMP,NPROMA
      ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
      ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1
      IOFF=JKGLO
      CALL COBS(1,ICEND,JKGLO,ICENDE,&
       & GNORDL(IOFF),GNORDM(IOFF),&
       & ESIG(1,IOFF),GM(IOFF),&
       & OROG(IOFF),RCORI(IOFF),&
       & GMV(1,1,1,IBL),GMVS(1,1,IBL),GFL(1,1,1,IBL),ZPARITY,ZOBBUF1)  

      IF (LMODERR.AND.NTYPE_MODERR==3) THEN
        CALL ADD_MODBIAS_TL(ICEND,IBL,JKGLO,ZOBBUF1,GPMODERR(1))
      ENDIF
    ENDDO
!$OMP END PARALLEL DO
  ENDIF
  CALL GSTATS(1034,1)

!*       4.2 DISTRIBUTED MEMORY : COMMUNICATION BETWEEN PROCESSORS

  IF (CDCONF(6:6)  ==  'C'.OR.LOBS) THEN
    
    CALL GSTATS(8,2)

    NFIXSFLD(1)=1
    NFIXSFLD(2)=0
    
    LSLONDEM_ACTIVE=.TRUE.
    IF(.NOT.ALLOCATED(MASK_SL2))THEN
      ALLOCATE(MASK_SL2(NASLB1+4))
    ELSE
      DEALLOCATE(MASK_SL2)
      ALLOCATE(MASK_SL2(NASLB1+4))
    ENDIF
    IF (.NOT.LELAM) THEN
      CALL SLEXTPOL(NSLWIDE,NSLWIDE,&
       & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDOBB1,NSLSTA,NSLONL,NSLOFF,&
       & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,NFIXSFLD,3,&
       & ZPARITY,ZOBBUF1,KMASK2=MASK_SL2)  
    ELSE
      CALL ESLEXTPOL(NSLWIDE,NSLWIDE,&
       & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDOBB1,NSLSTA,NSLONL,NSLOFF,&
       & NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,NFIXSFLD,3,&
       & ZOBBUF1,KMASK2=MASK_SL2)  
    ENDIF
  ENDIF
ENDIF

!*       4.3 LAGGED PART OF COMPUTATIONS.

IF (CDCONF(6:6) == 'V'.OR.CDCONF(6:6) == 'C'.OR.CDCONF(6:6) == 'F') THEN
  IF (LOBS) THEN
    CALL COBSLAG(NASLB1,.FALSE.,ZPARITY,ZOBBUF1)
    CALL GSTATS(8,3)
  ENDIF
ENDIF
IF (ALLOCATED(ZOBBUF1)) DEALLOCATE(ZOBBUF1)

LSLONDEM_ACTIVE=.FALSE.

!     ------------------------------------------------------------------
!*       5.    GRIDPOINT MODEL.
!     ------------------------------------------------------------------

IF (CDCONF(4:4) /= '0') THEN
  CALL GP_MODEL(CDCONF)
ENDIF

!     ------------------------------------------------------------------
!*       6.    POST-PROCESSING.
!     ------------------------------------------------------------------

IF (CDCONF(5:5) /= '0') THEN

  IF (CDCONF(5:5) == 'I'.OR.CDCONF(5:5) == 'A'.OR.&
     & CDCONF(5:5) == 'Z'.OR.CDCONF(5:5) == 'E') THEN  
    IF (.NOT.ALLOCATED(HFPBUF)) THEN
      ALLOCATE(NSTAHFPB(NFPBLOCS))
      CALL SUFPSC2B(NFPDYNB,NSTAHFPB,NLENHFPB,NFPROMA,NFPBLOCS)
      ALLOCATE(HFPBUF(NLENHFPB))
    ENDIF
  ENDIF

!*       6.1  VERTICAL POST-PROCESSING COMPUTATIONS.

  IF (LLVFP) THEN
    IF (CDCONF(5:5) /= 'Z'.AND.CDCONF(5:5) /= 'E') THEN
      IF (ALLOCATED(GPP)) DEALLOCATE(GPP)
      ALLOCATE(GPP(NPROMA,NFPGT1,NGPBLKS))
      DO JGFL=1,NGHG
        LLGHG_ACTIVE(JGFL)=YGHG(JGFL)%LACTIVE
      ENDDO
      DO JGFL=1,NTRAC
        LLTRAC_ACTIVE(JGFL)=YTRAC(JGFL)%LACTIVE
      ENDDO
      DO JGFL=1,NAERO
        LLAERO_ACTIVE(JGFL)=YAERO(JGFL)%LACTIVE
      ENDDO
      DO JGFL=1,NGRG
          LLGRG_ACTIVE(JGFL)=YGRG(JGFL)%LACTIVE
      ENDDO
      DO JGFL=1,NGFL_EXT
          LLEXT_ACTIVE(JGFL)=YEXT(JGFL)%LACTIVE
          LLEXT_CDERS(JGFL)=YEXT(JGFL)%LCDERS
          IEXT_MP(JGFL)=YEXT(JGFL)%MP
          IEXT_MPL(JGFL)=YEXT(JGFL)%MPL
          IEXT_MPM(JGFL)=YEXT(JGFL)%MPM
      ENDDO
      CALL GSTATS(1435,0)
      IF (.NOT. LFPART2)   THEN
        CALL SUALLOCUF
        CALL FPINVTRCUF
      ENDIF

      LLUD=YUOM%LACTIVE.AND.YUAL%LACTIVE
      LLDD=YDOM%LACTIVE.AND.YDAL%LACTIVE

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1)&
!$OMP&PRIVATE(JKGLO,IBL,IOFF,IST1,IEND1,JGFL,JLEV,JI,ZGFL,ZGFLL,ZGFLM,ZAERO,ZGHG,ZTRAC,ZGRG,ZERA40,ZLRCH4,ZCH4S)
      DO JKGLO=1,NGPTOT,NPROMA
        IBL=(JKGLO-1)/NPROMA+1
        IOFF=JKGLO
        IST1=1
        IEND1=MIN(NPROMA,NGPTOT-JKGLO+1)

        IF(NGHG > 0 .AND. LL_LRCH4) THEN
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZLRCH4(JI,JLEV)=GFL(JI,JLEV,YLRCH4%MP,IBL)
            ENDDO
          ENDDO
        ENDIF
        IF(NGHG > 0 .AND. LL_CH4S) THEN
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZCH4S(JI,JLEV)=GFL(JI,JLEV,YCH4S%MP,IBL)
            ENDDO
          ENDDO
        ENDIF

        DO JGFL=1,NERA40
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZERA40(JI,JLEV,JGFL)=GFL(JI,JLEV,YERA40(JGFL)%MP,IBL)
            ENDDO
          ENDDO
        ENDDO

        DO JGFL=1,NGHG
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZGHG(JI,JLEV,JGFL)=GFL(JI,JLEV,YGHG(JGFL)%MP,IBL)
            ENDDO 
          ENDDO
        ENDDO

        DO JGFL=1,NTRAC
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZTRAC(JI,JLEV,JGFL)=GFL(JI,JLEV,YTRAC(JGFL)%MP,IBL)
            ENDDO
          ENDDO
        ENDDO

        DO JGFL=1,NAERO
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZAERO(JI,JLEV,JGFL)=GFL(JI,JLEV,YAERO(JGFL)%MP,IBL)
            ENDDO 
          ENDDO
        ENDDO

        DO JGFL=1,NGRG
          DO JLEV=1,NFLEVG
            DO JI=IST1,IEND1
              ZGRG(JI,JLEV,JGFL)=GFL(JI,JLEV,YGRG(JGFL)%MP,IBL)
            ENDDO 
          ENDDO
        ENDDO
        
        IF (NGFL_EXT > 0) THEN
          DO JGFL=1,NGFL_EXT
            DO JLEV=1,NFLEVG
              DO JI=IST1,IEND1
                ZGFL(JI,JLEV,JGFL)=GFL(JI,JLEV,YEXT(JGFL)%MP,IBL)
              ENDDO 
            ENDDO
            IF (LLEXT_CDERS(JGFL)) THEN
              DO JLEV=1,NFLEVG
                DO JI=IST1,IEND1
                  ZGFLL(JI,JLEV,JGFL)=GFL(JI,JLEV,YEXT(JGFL)%MPL,IBL)
                  ZGFLM(JI,JLEV,JGFL)=GFL(JI,JLEV,YEXT(JGFL)%MPM,IBL)
                ENDDO 
              ENDDO
            ENDIF
          ENDDO
        ENDIF

        CALL VPOS(CDCONF(5:5),IST1,IEND1,IOFF,                                &
         & GMV(1,1,YT0%MU  ,IBL),GMV(1,1,YT0%MUL ,IBL),GMV(1,1,YT0%MDIV,IBL), &
         & GMV(1,1,YT0%MV  ,IBL),GMV(1,1,YT0%MVL ,IBL),GMV(1,1,YT0%MVOR,IBL), &
         & GMV(1,1,YT0%MT  ,IBL),GMV(1,1,YT0%MTL ,IBL),GMV(1,1,YT0%MTM ,IBL), &
         & GMV(1,1,YT0%MSPD,IBL),GMV(1,1,YT0%MSVD,IBL),GMV(1,1,YT0%MEDOT,IBL),&
         & GMVS(1,YT0%MSP,IBL),GMVS(1,YT0%MSPL,IBL),GMVS(1,YT0%MSPM,IBL),     &
         & YQ%LACTIVE,  GFL(1,1,YQ%MP   ,IBL),                                &
         & YQ%LCDERS,   GFL(1,1,YQ%MPL  ,IBL),GFL(1,1,YQ%MPM  ,IBL),          &
         & YL%LACTIVE,  GFL(1,1,YL%MP   ,IBL),                                &
         & YL%LCDERS,   GFL(1,1,YL%MPL  ,IBL),GFL(1,1,YL%MPM  ,IBL),          &
         & YI%LACTIVE,  GFL(1,1,YI%MP   ,IBL),                                &
         & YI%LCDERS,   GFL(1,1,YI%MPL  ,IBL),GFL(1,1,YI%MPM  ,IBL),          &
         & YA%LACTIVE,  GFL(1,1,YA%MP   ,IBL),                                &
         & YA%LCDERS,   GFL(1,1,YA%MPL  ,IBL),GFL(1,1,YA%MPM  ,IBL),          &
         & YS%LACTIVE,  GFL(1,1,YS%MP   ,IBL),                                &
         & YS%LCDERS,   GFL(1,1,YS%MPL  ,IBL),GFL(1,1,YS%MPM  ,IBL),          &
         & YR%LACTIVE,  GFL(1,1,YR%MP   ,IBL),                                &
         & YR%LCDERS,   GFL(1,1,YR%MPL  ,IBL),GFL(1,1,YR%MPM  ,IBL),          &
         & YG%LACTIVE,  GFL(1,1,YG%MP   ,IBL),                                &
         & YG%LCDERS,   GFL(1,1,YG%MPL  ,IBL),GFL(1,1,YG%MPM  ,IBL),          &
         & YH%LACTIVE,  GFL(1,1,YH%MP   ,IBL),                                &
         & YH%LCDERS,   GFL(1,1,YH%MPL  ,IBL),GFL(1,1,YH%MPM  ,IBL),          &
         & YTKE%LACTIVE,GFL(1,1,YTKE%MP ,IBL),                                &
         & YTKE%LCDERS, GFL(1,1,YTKE%MPL,IBL),GFL(1,1,YTKE%MPM,IBL),          &
         & YO3%LACTIVE, GFL(1,1,YO3%MP  ,IBL),                                &
         & YO3%LCDERS,  GFL(1,1,YO3%MPL ,IBL),GFL(1,1,YO3%MPM,IBL),           &
         & LLUD      ,  GFL(1,1,YUAL%MP,IBL),GFL(1,1,YUOM%MP,IBL),            &
         & LLDD      ,  GFL(1,1,YDAL%MP,IBL),GFL(1,1,YDOM%MP,IBL),            &
         & YUEN%LACTIVE, GFL(1,1,YUEN%MP,IBL),                                &
         & YUNEBH%LACTIVE, GFL(1,1,YUNEBH%MP,IBL),                            &
         & LLAERO_ACTIVE,ZAERO,&
         & LLGRG_ACTIVE,ZGRG,&
         & LLGHG_ACTIVE,ZGHG,LLTRAC_ACTIVE,ZTRAC,&
         & LLEXT_ACTIVE,ZGFL,ZERA40, ZLRCH4, ZCH4S,    &
         & OROG(IOFF),RCORI(IOFF),GM(IOFF),GEMU(IOFF),GELAM(IOFF),OROGL(IOFF),&
         & OROGM(IOFF))  
      ENDDO
!$OMP END PARALLEL DO
      IF (LFPART2) CALL DEALLOCUF
      CALL GSTATS(1435,1)
      IF (LELAM) THEN
        CALL EBIPOS
      ENDIF
    ELSE
      CALL ENDVPOS(CDCONF(5:5))
    ENDIF

  ELSEIF(LLHFP) THEN

!*          6.2  HORIZONTAL POST-PROCESSING COMPUTATIONS.


!*          6.2.1 NON-LAGGED PART OF COMPUTAIONS

    IF (.NOT.ALLOCATED(ZFPBUF1)) ALLOCATE(ZFPBUF1(NAFPB1,NFPBFIN))
    IF (.NOT.ALLOCATED(ZPARFP1)) ALLOCATE(ZPARFP1(NFPBFIN))
    ZPARFP1(:)=1.0_JPRB
    CALL GSTATS(1434,0)
!$OMP PARALLEL PRIVATE(JKGLO,ICEND,IBL,IOFF,JROF,IOFFBL,ZFPBUF1AUX)
    IF (.NOT.ALLOCATED(ZFPBUF1AUX)) ALLOCATE(ZFPBUF1AUX(NPROMA,NFPBFIN))
!$OMP DO SCHEDULE(DYNAMIC,1)
    DO JKGLO=1,IFPCOMP,NPROMA
      ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1
      IOFF=JKGLO
      CALL HPOS(CDCONF(5:5),ICEND,JKGLO,RCORI(IOFF),GM(IOFF),OROG(IOFF), &
       & ZPARFP1,ZFPBUF1AUX)  
      IF (NFPBFIN > 0) THEN
        IF (NFPWIDE > 0) THEN
          DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IFPCOMP)
            ZFPBUF1(NFPCORE(JROF),:)=ZFPBUF1AUX(JROF-JKGLO+1,:)
          ENDDO
        ELSE
          IOFFBL=(IBL-1)*NPROMA
          DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IFPCOMP)
            ZFPBUF1(IOFFBL+JROF-JKGLO+1,:)=ZFPBUF1AUX(JROF-JKGLO+1,:)
          ENDDO
        ENDIF
      ENDIF
    ENDDO
!$OMP END DO
    IF (ALLOCATED(ZFPBUF1AUX)) DEALLOCATE(ZFPBUF1AUX)
!$OMP END PARALLEL
    CALL GSTATS(1434,1)

!*          6.2.2 DISTRIBUTED MEMORY: COMMUNICATION BETWEEN PROCESSORS

    IF (NFPWIDE > 0) THEN

!               Buffer "FPBUF1":

      IF (NFPBFIN > 0) THEN
        IF (NPROC > 1) THEN
          CALL SLCOMM(NAFPB1,NFPPROCS,&
           & NFPRPT,NFPSPT,NFPSENDPOS,NFPRECVPOS,NFPSENDPTR,&
           & NFPRECVPTR,NFPCOMM,&
           & IDUMARR,NFPBFIN,LLINC,0,ZFPBUF1)  
        ENDIF

        CALL SLEXTPOL(NFPWIDE,NFPWIDE,&
         & NAFPB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFPBFIN,NFPSTA,NFPONL,NFPOFF,&
         & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,IDUMARR,1,&
         & ZPARFP1,ZFPBUF1)  
      ENDIF

    ENDIF

!*          6.2.4 LAGGED PART OF HORIZONTAL POST-PROCESSING

    CALL FPOSHOR(ZFPBUF1,CDCONF(5:5))

    IF (ALLOCATED(ZFPBUF1)) DEALLOCATE(ZFPBUF1)
    IF (ALLOCATED(ZPARFP1)) DEALLOCATE(ZPARFP1)
    
  ELSE
    CALL ABOR1('SCAN2M : UNKNOWN CONFIGURATION CDCONF(5:5)')
  ENDIF
ENDIF

!     ------------------------------------------------------------------
!*       7.    GRID POINT CALCULATIONS FOR ANALYSIS.
!     ------------------------------------------------------------------

!*       7.1  GRID PONT CALCULATIONS FOR JG

IF (LLJGNR) THEN

  DO JKGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1

    IOFF=JKGLO
    IF (CDCONF(6:6) == 'H') THEN
      CALL SUJBVCOORD (ICEND,JKGLO,&
       & GMV(1,1,YT0%MT,IBL),GFL(1,1,YQ%MP,IBL),GMVS(1,YT0%MSP,IBL))  
    ELSE
      CALL ABOR1 ('SCAN2M: INVALID VALUE FOR CDCONF(6:6)')
    ENDIF
  ENDDO
ENDIF

!*       7.2  GRID POINT CALCULATIONS FOR CANARI

IF (CDCONF(6:6) == '1') THEN
  ALLOCATE(ZUT(NPROMA,NFLEVG))
  ALLOCATE(ZVT(NPROMA,NFLEVG))
  ALLOCATE(ZTT(NPROMA,NFLEVG))
  ALLOCATE(ZQT(NPROMA,NFLEVG))
  ALLOCATE(ZLT(NPROMA,NFLEVG))
  ALLOCATE(ZIT(NPROMA,NFLEVG))
  ALLOCATE(ZRT(NPROMA,NFLEVG))
  ALLOCATE(ZST(NPROMA,NFLEVG))
  ALLOCATE(ZPS(NPROMA))
  ALLOCATE(ZSP_SB(NPROMA,YSP_SBD%NLEVS,YSP_SBD%NDIM))
  ALLOCATE(ZSP_SG(NPROMA,YSP_SGD%NDIM))
  ALLOCATE(ZSP_RR(NPROMA,YSP_RRD%NDIM))
  ALLOCATE(ZSP_CI(NPROMA,YSP_CID%NDIM))
  ALLOCATE(ZSP_X2(NPROMA,YSP_X2D%NDIM))
  ALLOCATE(ZSD_VF(NPROMA,YSD_VFD%NDIM))
  ALLOCATE(ZSD_VV(NPROMA,YSD_VVD%NDIM))
  ALLOCATE(ZSD_VX(NPROMA,YSD_VXD%NDIM))
  ALLOCATE(ZRCORI(NPROMA))
  ALLOCATE(ZGM(NPROMA))
  ALLOCATE(ZGELAT(NPROMA))
  ALLOCATE(ZGELAM(NPROMA))
  ALLOCATE(ZGNORDL(NPROMA))
  ALLOCATE(ZGNORDM(NPROMA))
  ALLOCATE(ZGEMU(NPROMA))
  ALLOCATE(ZCAGUE(2,NPROMA))
  ALLOCATE(ZESIG(2*NFLEVG+5,NPROMA))
  ALLOCATE(ZSGPHI(NFLEVG,NPROMA))
  ALLOCATE(ZSGHUM(NFLEVG,NPROMA))
  ITASK=1
  DO JKGLO=1,NGPTOT,NPROMA
    IBL=(JKGLO-1)/NPROMA+1
    IST=1
    IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
! packing analysis points according to the required domain mask
    INDEX=0
    IOFF=JKGLO-1
    DO JROF = 1 , IEND
      IOFF=IOFF+1
      IF (SP_CI(JROF,YSP_CI%YCI(3)%MP0,IBL) > 0.5_JPRB) THEN
        INDEX=INDEX+1
        ZUT(INDEX,1:NFLEVG)=GMV(JROF,:,YT0%MU,IBL)
        ZVT(INDEX,1:NFLEVG)=GMV(JROF,:,YT0%MV,IBL)
        ZTT(INDEX,1:NFLEVG)=GMV(JROF,:,YT0%MT,IBL)
        ZQT(INDEX,1:NFLEVG)=GFL(JROF,1:NFLEVG,YQ%MP,IBL)
        IF (YL%LACTIVE) THEN
            ZLT(INDEX,1:NFLEVG)=GFL(JROF,1:NFLEVG,YL%MP,IBL)
        ELSE
            ZLT(INDEX,1:NFLEVG)=0.0_JPRB
        ENDIF        
        IF (YI%LACTIVE) THEN
           ZIT(INDEX,1:NFLEVG)=GFL(JROF,1:NFLEVG,YI%MP,IBL)
        ELSE
           ZIT(INDEX,1:NFLEVG)=0.0_JPRB
        ENDIF      
        IF (YR%LACTIVE) THEN
           ZRT(INDEX,1:NFLEVG)=GFL(JROF,1:NFLEVG,YR%MP,IBL)
        ELSE
           ZRT(INDEX,1:NFLEVG)=0.0_JPRB
        ENDIF
        IF (YS%LACTIVE) THEN
           ZST(INDEX,1:NFLEVG)=GFL(JROF,1:NFLEVG,YS%MP,IBL)
        ELSE
           ZST(INDEX,1:NFLEVG)=0.0_JPRB
        ENDIF
        ZPS(INDEX)=GMVS(JROF,YT0%MSP,IBL)
        ZSP_SB(INDEX,:,:) = SP_SB(JROF,:,:,IBL)
        ZSP_SG(INDEX,:) = SP_SG(JROF,:,IBL)
        ZSP_RR(INDEX,:) = SP_RR(JROF,:,IBL)
        ZSP_CI(INDEX,:) = SP_CI(JROF,:,IBL)
        ZSP_X2(INDEX,:) = SP_X2(JROF,:,IBL)
        ZSD_VF(INDEX,:) = SD_VF(JROF,:,IBL)
        ZSD_VV(INDEX,:) = SD_VV(JROF,:,IBL)
        ZSD_VX(INDEX,:) = SD_VX(JROF,:,IBL)
        ZGM(INDEX)=GM(IOFF)
        ZGELAT(INDEX)=GELAT(IOFF)
        ZGELAM(INDEX)=GELAM(IOFF)
        ZGNORDL(INDEX)=GNORDL(IOFF)
        ZGNORDM(INDEX)=GNORDM(IOFF)
        ZGEMU(INDEX)=GEMU(IOFF)
        ZRCORI(INDEX)=RCORI(IOFF)
        ZCAGUE(:,INDEX)=QCAGUE(:,IOFF)
        ZESIG(:,INDEX)=ESIG(:,IOFF)
        ZSGPHI(:,INDEX)=RSGPHI(:,IOFF)
        ZSGHUM(:,INDEX)=RSGHUM(:,IOFF)
      ENDIF
    ENDDO
    INBPT=INDEX
    IF (INBPT > 0) THEN
! analysis
      CALL CAPOTX(ITASK,INBPT,ZUT,ZVT,ZTT,ZQT,ZLT,ZIT,ZRT,ZST,ZPS,&
       & ZSP_SB,ZSP_SG,ZSP_RR,ZSP_CI,ZSP_X2,ZSD_VF,ZSD_VV,ZSD_VX,&
       & ZGM,ZGELAT,ZGELAM,ZGNORDL,ZGNORDM,ZRCORI,&
       & ZGEMU,ZCAGUE,ZESIG,ZSGPHI,ZSGHUM)  
! results saving
      IOFF=JKGLO-1
      INDEX=0
      DO JROF = 1 , IEND
        IOFF=IOFF+1
        IF (SP_CI(JROF,YSP_CI%YCI(3)%MP0,IBL) > 0.5_JPRB) THEN
          INDEX=INDEX+1
          GMV(JROF,:,YT0%MU,IBL)=ZUT(INDEX,1:NFLEVG)
          GMV(JROF,:,YT0%MV,IBL)=ZVT(INDEX,1:NFLEVG)
          GMV(JROF,:,YT0%MT,IBL)=ZTT(INDEX,1:NFLEVG)
          GFL(JROF,1:NFLEVG,YQ%MP,IBL)=ZQT(INDEX,1:NFLEVG)
          IF (YL%LACTIVE) GFL(JROF,1:NFLEVG,YL%MP,IBL)=ZLT(INDEX,1:NFLEVG)
          IF (YI%LACTIVE) GFL(JROF,1:NFLEVG,YI%MP,IBL)=ZIT(INDEX,1:NFLEVG)
          GMVS(JROF,YT0%MSP,IBL)=ZPS(INDEX)
          SP_SB(JROF,:,:,IBL)=ZSP_SB(INDEX,:,:)
          SP_SG(JROF,:,IBL)=ZSP_SG(INDEX,:)
          SP_RR(JROF,:,IBL)=ZSP_RR(INDEX,:)
          SP_CI(JROF,:,IBL)=ZSP_CI(INDEX,:)
          SP_X2(JROF,:,IBL)=ZSP_X2(INDEX,:)
          SD_VF(JROF,:,IBL)=ZSD_VF(INDEX,:)
          SD_VV(JROF,:,IBL)=ZSD_VV(INDEX,:)
          SD_VX(JROF,:,IBL)=ZSD_VX(INDEX,:)
          RSGPHI(:,IOFF)=ZSGPHI(:,INDEX)
          RSGHUM(:,IOFF)=ZSGHUM(:,INDEX)
        ENDIF
      ENDDO
    ENDIF
  ENDDO

  GFLT1(:,:,:,:)=GFL(:,:,:,:)
  GMVT1(:,:,:,:)=GMV(:,:,:,:)
  GMVT1S(:,:,:)=GMVS(:,:,:)

  IF (ALLOCATED(ZUT)) DEALLOCATE(ZUT)
  IF (ALLOCATED(ZVT)) DEALLOCATE(ZVT)
  IF (ALLOCATED(ZTT)) DEALLOCATE(ZTT)
  IF (ALLOCATED(ZQT)) DEALLOCATE(ZQT)
  IF (ALLOCATED(ZLT)) DEALLOCATE(ZLT)
  IF (ALLOCATED(ZIT)) DEALLOCATE(ZIT)
  IF (ALLOCATED(ZRT)) DEALLOCATE(ZRT)
  IF (ALLOCATED(ZST)) DEALLOCATE(ZST)
  IF (ALLOCATED(ZPS)) DEALLOCATE(ZPS)
  IF (ALLOCATED(ZSP_SB)) DEALLOCATE(ZSP_SB)
  IF (ALLOCATED(ZSP_SG)) DEALLOCATE(ZSP_SG)
  IF (ALLOCATED(ZSP_RR)) DEALLOCATE(ZSP_RR)
  IF (ALLOCATED(ZSP_CI)) DEALLOCATE(ZSP_CI)
  IF (ALLOCATED(ZSP_X2)) DEALLOCATE(ZSP_X2)
  IF (ALLOCATED(ZSD_VF)) DEALLOCATE(ZSD_VF)
  IF (ALLOCATED(ZSD_VV)) DEALLOCATE(ZSD_VV)
  IF (ALLOCATED(ZSD_VX)) DEALLOCATE(ZSD_VX)
  IF (ALLOCATED(ZRCORI)) DEALLOCATE(ZRCORI)
  IF (ALLOCATED(ZGM)) DEALLOCATE(ZGM)
  IF (ALLOCATED(ZGELAT)) DEALLOCATE(ZGELAT)
  IF (ALLOCATED(ZGELAM)) DEALLOCATE(ZGELAM)
  IF (ALLOCATED(ZGNORDL)) DEALLOCATE(ZGNORDL)
  IF (ALLOCATED(ZGNORDM)) DEALLOCATE(ZGNORDM)
  IF (ALLOCATED(ZGEMU)) DEALLOCATE(ZGEMU)
  IF (ALLOCATED(ZCAGUE)) DEALLOCATE(ZCAGUE)
  IF (ALLOCATED(ZESIG)) DEALLOCATE(ZESIG)
  IF (ALLOCATED(ZSGPHI)) DEALLOCATE(ZSGPHI)
  IF (ALLOCATED(ZSGHUM)) DEALLOCATE(ZSGHUM)

ENDIF

CALL GSTATS(8,1)

!     ------------------------------------------------------------------
!*       8.    "TIME-STEPPING" FOR GRID-POINT GFL FIELDS
!     ------------------------------------------------------------------

IF (CDCONF(7:7) /= '0') THEN
  LLADNMI= CDCONF(4:4) == 'E' .OR. CDCONF(4:4) == 'F'
  IF(.NOT.LLADNMI.AND..NOT.LLHFP.AND..NOT.LLVFP.AND.&
   &((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL))) THEN
    ! ??? Must it be done for the corrector of LPC_FULL ???
    CALL GSTATS(1032,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,IBL,JGFL,JLEV,JL)
    DO JKGLO=1,IGPCOMP,NPROMA
      IBL=(JKGLO-1)/NPROMA+1
      ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
      DO JGFL=1,YGFL%NUMFLDS
        IF(YGFLC(JGFL)%LGP.AND.YGFLC(JGFL)%LT1) THEN
          IF (YGFLC(JGFL)%NCOUPLING==0) THEN
            GFL(1:ICEND,1:NFLEVG,YGFLC(JGFL)%MP,IBL) = &
             & GFLT1(1:ICEND,1:NFLEVG,YGFLC(JGFL)%MP1,IBL)
          ENDIF
        ENDIF
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1032,1)
  ENDIF
ENDIF
IF (.NOT. LFPART2) CALL DEALLOCUF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SCAN2M',1,ZHOOK_HANDLE)
END SUBROUTINE SCAN2M

