SUBROUTINE SCAN2MAD(CDCONF)

!**** *SCAN2MAD* - Grid-point space computations (adjoint)

!     Purpose. Computations in grid-point space
!     --------

!     Interface.
!     ----------
!        *CALL* *SCAN2MAD(....)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)
!        --------------------

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   see includes below.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud   *ECMWF*

!     Modifications.
!     --------------
!        Original : 91-04-01
!        Bill Heckley and Mats Hamrud - 31:03:92 io schem
!        Drasko Vasiljevic and Mats Hamrud - 31:03:92 io schem
!        Laurent Labbe - 15:04:93 orography as control var. (SC2CGAP,CPG2AD)
!        Mats hamrud   - 93-11-19 (use only one Fourier file)
!        J.N.Thepaut - 94-05-18 Humidity control in var. job
!        Mats Hamrud - 94-11-02 New transform for Jb computaions
!        Modified 93-11-10 by R. EL Khatib (Non hydrostatic variables)
!        F.Bouttier 95-12-07 change COBSLAGAD interface
!        Modified 96-02-20 L. Isaksen  : Message passing option added
!        Modified 97-04-04 M. Janiskova : spectral transforms for rad.coef.
!        Modified 97-12-18 E. Holm: Ozone included in Eulerian model
!                                   for data assimilation
!        Modified 98-03-04 C. Temperton : bugfix for adjoint of SWE
!        Modified 98-01-21 K. YESSAD : add dummy arguments to SLEXTPOLAD.
!        Modified 98-10-22 C. Temperton : adjoint of semi-Lagrangian SWE
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!        - removal of LRPOLE in YOMCT0 and code under LRPOLE.
!        Modified 98-10-27 G. Mozdzynski : load grid-point trajectory
!        Modified 99-01-26 C. Temperton : NTRSLTYPE=2
!        Modified 99-02-01 M. Charron : non-hydrostatic
!        Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!        Modified 99-03-18 C. Temperton: Add code for the case
!                                        (LMESSP.AND.LSLAG)
!        Modified 99-08    D.Salmond   : Remove NSLBCT for LMESSP=.TRUE.
!        Modified 99-10-19 C. Temperton : adjoint of 3D semi-Lagrangian !!
!        Modified 00-03-07 C. Temperton : bugfix for SWE
!        Modified 00-08-11 Y. Tremolet  : use module TRAJECTORY
!        Modified 00-07-19 G. Mozdzynski: bit reproducibility support
!        Modified 00-10-26 M. Fisher: wavelet Jb and Jb vertical coordinate
!        Modified 00-10-25 M.Hamrud     : Transform package
!        Modified 01-01-29 C. Temperton : case LSPRT=.TRUE.
!        C. Fischer   01-03-06: Merge LAM version into global code
!        Modified 01-06-20 D.Salmond   : Introduce GP_MODEL_AD
!        Modified 01-08-24 E. Holm     : Modified calls to  JGNRAD/IAD
!                                        because of new control variable for humidity
!        Modified 02-09-30 P. Smolikova : Interface to SC2CGAP for d4 in NH
!        Modified 03-06-02 C. Soci : nullify GPP when reallocated
!        C. Fischer : 03-06-03 use igpcomp/ngptot_cap in call to cobs for LAM/Jo
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib 04-05-24 Allocate MASK_SL1 anytime because it is argument of subroutine
!        C Soci & C Fischer 04-06-18 GMVs set to zero up to NGPTOT for aladin/ifs compliency
!        Y.Tremolet    01-Aug-2005 Number sections consistently with SCAN2M
!        D Salmond 21-09-05 Fix for LSPRT=.true.
!        E. Holm   06-07-11 Move NSTEP=0 to CVARGPAD and remove unused option 
!                           CDCONF(6)=W.
!        K. Yessad Dec 2008: merge the different (E)SLEXTPOL.. -> (E)SLEXTPOL.
!        H. Hersbach Dec 2009: reset surface perts (moved from ec_phys_ad), 
!                              ZPARITY determined in COBSLAGAD.
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,&
 & NPROMA   ,&
 & NGPBLKS  ,&
 & NDGENL   ,NDGSAL   ,NDGLG    ,&
 & NDGSAG   ,NDGENG
USE YOMCT0   , ONLY :     LOBS     ,LELAM    ,LSPRT, NSTOP
USE YOMCT3   , ONLY : NSTEP
USE YOMSC2   , ONLY : NSLWIDE  ,NFLDOBB1
USE YOMGEM   , ONLY :    NGPTOT   ,&
 & NGPTOT_CAP,NLOENG
USE YOMGC    , ONLY : GELAM    ,GELAT    ,GM
USE YOMMP    , ONLY : NASLB1   ,NSLSTA   ,NSLONL   ,NSLOFF   ,&
 & MYFRSTACTLAT,MYLSTACTLAT     ,NFRSTLOFF
USE YOMTNEWT , ONLY : LIDMODEL
USE YOMMASK  , ONLY : MASK_SL1
USE TRAJECTORY_MOD, ONLY: LTRAJGP, GET_TRAJ_GRID
USE YOMGMV , ONLY : GMV,GMVS,GMV5,GMV5S,GMVT1,GMVT1S,GMV_DEPART,GMVS_DEPART,&
 & YT0,YT1,YT5,NDIMGMV
USE YOMGFL     , ONLY : GFL,GFL5,GFLT1,GFL_DEPART
USE YOM_YGFL , ONLY : YGFL,YQ,YGFLC
USE TESTVAR_MIX, ONLY: LTESTADJM
USE YOMVAR, ONLY: LMODERR
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, GPMODERR

!    -------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9), INTENT(IN) :: CDCONF

!    -------------------------------------------------------------------

REAL(KIND=JPRB), ALLOCATABLE :: ZOBBUF1(:,:)
REAL(KIND=JPRB) :: ZPARITY(NFLDOBB1)
INTEGER(KIND=JPIM) :: IBL, ICEND, IEND,ICSTA,IOFF,JKGLO,IGPCOMP,ICENDE,IST
INTEGER(KIND=JPIM) :: JGFL, J
LOGICAL :: LLMODEL

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

#include "abor1.intfb.h"
#include "cobsad.intfb.h"
#include "cobslagad.intfb.h"
#include "ctvtot5.intfb.h"
#include "ctvtotad.intfb.h"
#include "gp_model_ad.intfb.h"
#include "reset_spert.intfb.h"
#include "lcnorggad.intfb.h"
#include "slextpolad.intfb.h"
#include "add_modbias_ad.intfb.h"

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SCAN2MAD',0,ZHOOK_HANDLE)
!    -------------------------------------------------------------------

!    -------------------------------------------------------------------
!*       1.    INITIAL SETUP, MAIN LOOP CONTROL.
!    -------------------------------------------------------------------

! Try to guess when we are in an actual model time step...
LLMODEL =       (CDCONF(2:2)=='A') &
        & .AND. (CDCONF(3:3)=='A') &
        & .AND. (CDCONF(4:4)=='0' .OR. CDCONF(4:4)=='A') &
        & .AND. (CDCONF(5:5)=='0') &
        & .AND. (CDCONF(6:6)=='0' .OR. CDCONF(6:6)=='V')

! Total number of grid points - ndglg*ndlon in global / ndguxg*ndlon in LAM
! used for model computations
IF(.NOT.LELAM) THEN
  IGPCOMP=NGPTOT
ELSE
  IGPCOMP=MIN(NGPTOT,NGPTOT_CAP)
ENDIF

!    -------------------------------------------------------------------
!*       2.   TRAJECTORY HANDLING
!    -------------------------------------------------------------------

!IF(.NOT.(LSLAG.AND.NTRSLTYPE <= 1)) THEN
!       (If LSLAG.AND.NTRSLTYPE.LE.1 then this has
!       already been done)
  IF(CDCONF(4:4) /= '0'.OR. CDCONF(6:6) == 'G' &
     & .OR. CDCONF(6:6) == 'X' ) THEN

    IF( LTRAJGP )THEN
!             retrieve grid-point trajectory
      IF (.NOT.ALLOCATED(GMV5)) ALLOCATE(GMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
      IF (.NOT.ALLOCATED(GMV5S)) ALLOCATE(GMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
      IF (.NOT.ALLOCATED(GFL5)) ALLOCATE(GFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
      CALL GSTATS(16,0)
      CALL GET_TRAJ_GRID(GMV5,GMV5S,GFL5,KSTEP=NSTEP)
      CALL GSTATS(16,1)
    ELSE
      IF(.NOT.ALLOCATED(GMV5)) CALL ABOR1('SCAN2MTL:GMV5 NOT ALLOCATED')
    ENDIF
  ENDIF
!ENDIF

! If LSPRT=.T. trajectory temperature contains Tv, so it is converted to T here

IF (LSPRT) THEN
  CALL GSTATS(1044,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JKGLO,IBL,IST,IEND)
  DO JKGLO=1,NGPTOT,NPROMA
    IBL=(JKGLO-1)/NPROMA+1
    IST =1
    IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    CALL CTVTOT5(1,IEND,GMV5(1,1,YT5%MT,IBL),GFL5(1,1,YQ%MP5,IBL))
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1044,1)
ENDIF

!     ------------------------------------------------------------------
!*       8.    "TIME-STEPPING" FOR GRID-POINT GFL FIELDS
!     ------------------------------------------------------------------

IF (CDCONF(7:7) /= '0') THEN
  CALL GSTATS(1044,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JKGLO,IBL,JGFL)
  DO JKGLO=1,NGPTOT,NPROMA
    IBL=(JKGLO-1)/NPROMA+1
    DO JGFL=1,YGFL%NUMFLDS
      IF(YGFLC(JGFL)%LGP.AND.YGFLC(JGFL)%LT1) THEN
        IF (YGFLC(JGFL)%NCOUPLING==0) THEN
          GFLT1(:,:,YGFLC(JGFL)%MP1,IBL) = GFL(:,:,YGFLC(JGFL)%MP,IBL)
          GFL(:,:,YGFLC(JGFL)%MP,IBL)= 0._JPRB
        ENDIF
      ENDIF
    ENDDO
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1044,1)
ENDIF

! Following loop moved from gp_model_ad
CALL GSTATS(1044,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,IBL,JGFL)
DO JKGLO=1,NGPTOT,NPROMA
  IBL=(JKGLO-1)/NPROMA+1
  GMV(:,:,1:YT0%NDIM,IBL)=0.0_JPRB
  GMVS(:,1:YT0%NDIMS,IBL)=0.0_JPRB
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LT1) THEN
      IF (LTESTADJM .AND. YGFLC(JGFL)%LGP .AND. CDCONF(7:7) == '0') THEN
!        GR: in test of adjoint we start from a non-zero state so
!        we must avoid zeroing if there was no swapping to GFLT1 before...      
!        to be cleaned...
      ELSE
         GFL(:,:,YGFLC(JGFL)%MP,IBL) = 0.0_JPRB
      ENDIF
      IF(YGFLC(JGFL)%LCDERS) THEN
        GFL(:,:,YGFLC(JGFL)%MPL,IBL) = 0.0_JPRB
        GFL(:,:,YGFLC(JGFL)%MPM,IBL) = 0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1044,1)

!    -------------------------------------------------------------------
!*    7. NORM LOCALIZATION
!    -------------------------------------------------------------------

IF(CDCONF(5:5) == 'L')THEN
  ICSTA=1
  CALL GSTATS(1045,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,IBL,IOFF)
  DO JKGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    IOFF=JKGLO
    CALL LCNORGGAD (ICSTA,ICEND,&
     & GELAM(IOFF),GELAT(IOFF),GM(IOFF),&
     & GMV(1,1,YT0%MU,IBL) ,GMV(1,1,YT0%MV,IBL) ,GMV(1,1,YT0%MT,IBL),&
     & GFL(1,1,YQ%MP,IBL) ,GMVS(1,YT0%MSP,IBL),&
     & GMVT1(1,1,YT1%MU,IBL) ,GMVT1(1,1,YT1%MV,IBL) ,GMVT1(1,1,YT1%MT,IBL),&
     & GFLT1(1,1,YQ%MP1,IBL) ,GMVT1S(1,YT1%MSP,IBL))
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1045,1)
ENDIF

!     ------------------------------------------------------------------
!*       6.    INITIALIZE ADJOINT SURFACE FIELDS
!     ------------------------------------------------------------------

IF (NSTEP == NSTOP) CALL RESET_SPERT(NSTEP)

!     ------------------------------------------------------------------
!*       5.    GRIDPOINT MODEL (ADJOINT).
!     ------------------------------------------------------------------

CALL GSTATS(35,0)

IF (CDCONF(4:4) /= '0') THEN
  CALL GP_MODEL_AD(CDCONF)
ENDIF

CALL GSTATS(35,1)
!     ------------------------------------------------------------------
!*       4.    COMPARISON WITH OBSERVATIONS
!     ------------------------------------------------------------------

IF (CDCONF(6:6) == 'V' .OR. CDCONF(6:6) == 'F') THEN


!*       4.3 LAGGED PART OF COMPUTATIONS.

  IF (LOBS) THEN
    ALLOCATE(ZOBBUF1(NASLB1,NFLDOBB1))
    CALL GSTATS(1042,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
    DO J=1,NFLDOBB1
      ZOBBUF1(:,J) = 0.0_JPRB
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1042,1)
    CALL COBSLAGAD(NASLB1,ZPARITY,ZOBBUF1)

!*       4.2 DISTRIBUTED MEMORY : COMMUNICATION BETWEEN PROCESSORS

    IF(.NOT.ALLOCATED(MASK_SL1)) ALLOCATE(MASK_SL1(NASLB1+4))
    CALL SLEXTPOLAD(NSLWIDE,&
     & NASLB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFLDOBB1,NSLSTA,NSLONL,NSLOFF,&
     & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
     & ZPARITY,ZOBBUF1,MASK_SL1)

! COMMUNICATION NOT NEEDED

    CALL GSTATS(35,0)

    IF (LTESTADJM) THEN
      ZOBBUF1(:,:) = 0.0_JPRB
    ENDIF

!*       4.1 NON-LAGGED PART OF COMPUTATIONS

    CALL GSTATS(1043,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,ICENDE,IBL,IOFF)
    DO JKGLO=1,IGPCOMP,NPROMA
      ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
      ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1
      IOFF=JKGLO

      IF (LMODERR.AND.NTYPE_MODERR==3) THEN
        CALL ADD_MODBIAS_AD(ICEND,IBL,JKGLO,ZOBBUF1,GPMODERR(1))
      ENDIF

      CALL COBSAD(1,ICEND,JKGLO,ICENDE,&
       & GM(IOFF),&
       & GMV(1,1,1,IBL),GMVS(1,1,IBL),GFL(1,1,1,IBL),ZOBBUF1)
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1043,1)
    DEALLOCATE(ZOBBUF1)

    IF (LIDMODEL) THEN
      GMV_DEPART(:,:,1:YT0%NDIM,:) =GMV_DEPART(:,:,1:YT0%NDIM,:)+ &
       & GMV(:,:,1:YT0%NDIM,:)
      GMVS_DEPART(:,1:YT0%NDIMS,:) = GMVS_DEPART(:,1:YT0%NDIMS,:)+ &
       & GMVS(:,1:YT0%NDIMS,:)
      GFL_DEPART(:,:,1:YGFL%NDIM0,:) = GFL_DEPART(:,:,1:YGFL%NDIM0,:)+ &
       & GFL(:,:,1:YGFL%NDIM0,:)
    ENDIF

    CALL GSTATS(35,1)

  ENDIF
ENDIF

!*       4.0 RESET ADJOINT SURFACE FIELDS

IF (NSTEP == 0) CALL RESET_SPERT(NSTEP)

!    -------------------------------------------------------------------
!*       3.    INITIAL GRIDPOINT COMPUTATIONS
!    -------------------------------------------------------------------

!*       3.2 CONVERSION OF TTx ARRAY FROM 'TV' TO T

CALL GSTATS(35,0)

IF (LSPRT) THEN
  CALL GSTATS(1047,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JKGLO,IBL,IST,IEND)
  DO JKGLO=1,NGPTOT,NPROMA
    IBL=(JKGLO-1)/NPROMA+1
    IST =1
    IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    CALL CTVTOTAD(1,IEND,&
     & GMV(1,1,YT0%MT,IBL),GFL(1,1,YQ%MP,IBL),&
     & GMV5(1,1,YT5%MT,IBL),GFL5(1,1,YQ%MP5,IBL))
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1047,1)
ENDIF

CALL GSTATS(35,1)

!*       3.x IDENTITY MODEL
IF(CDCONF(3:3) /= '0')THEN
  IF (LIDMODEL .AND. CDCONF(6:6) == 'V') THEN
    GMV(:,:,1:YT0%NDIM,:) = GMV_DEPART(:,:,1:YT0%NDIM,:)
    GMVS(:,1:YT0%NDIMS,:) = GMVS_DEPART(:,1:YT0%NDIMS,:)
    GFL(:,:,1:YGFL%NDIM0,:) = GFL_DEPART(:,:,1:YGFL%NDIM0,:)
  ENDIF
ENDIF

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SCAN2MAD',1,ZHOOK_HANDLE)
END SUBROUTINE SCAN2MAD
