SUBROUTINE SIM4D(KINDIC,KDPROB,YDVAZX,PZF,YDVAZG,KITER)

!**** *SIM4D*  - Controls 4-D variational job at level 2.

!     Purpose.
!     --------
!           Simulator (in minimization language). Compute the cost-function
!        and its gradient by calling the model and its adjoint.

!     Interface.
!     ----------
!        *CALL* *SIM4D(KINDIC,KDPROB,YDVAZX,PZF,YDVAZG,KITER)

!        Explicit arguments :
!        --------------------
!        KINDIC : not used (for interface with M1QN3)
!        KDPROB : not used (for interface with M1QN3)
!        YDVAZX  : STATE VARIABLE
!        PZF    : COST FUNCTION
!        YDVAZG  : GRADIENT
!        KITER  : index of iteration given by the caller

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        SU2YOM, SUSPE0, CAINAD, SUSPEC, CAIN, CNT3, CNT3AD,
!       EVALJG, EVCOST, SUALLDFI, DEALLDFI, EVJCDFI

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        D. Vasiljevic 90-11-19
!        Bill Heckley  90-11-20 : CVAR ETC
!        F.Rabier and J.N Thepaut  91-01-15 : 4-d diagnostics
!        Bill Heckley  91-02-20 : CHOICE OF CONTROL VARIABLE
!                                             INITIALIZATION OF COST FNS
!        Bill Heckley  91-06-06 : Jg optional
!        P.C and J.N.T 91-09-24 : complete change of the design.
!        Bill Heckley  91-10-10 : efficiency of NCNTVAR=2 see also COSJG & CNT3
!        L. Labbe      93-04-15 : SIM4D used by sensitivity job
!        F. Rabier     93-12-10 : Option to read trajectory from external file
!                                 Option to write gradient w.r.t SCALP in.prod.
!        F. Rabier     94-06-24 : Option to use full traj. from external file
!                                 Option to use TL model for minimisation
!        J.N. Thepaut           : move of the filter up to sim4d.
!        F. Rabier     95-01-25 : Write-up of increments in sensitivity config.
!        Mike Fisher            : alternative Jb calculation if LBEQLTL
!        L. Isaksen             : Reordering of spectral arrays
!        M. Hamrud     95-10-01 : Interface block for SUSPEC
!        M. Hamrud     95-10-30 : New form of CLCONF
!        F. Bouttier   96-01-11 : Secure consistency between KDPROB & NVADIM
!        L. Isaksen    96-04-04 : Adapted for Message Passing
!        J. Derber     96-14-17 : alternative Jb calculation if LSTABAL
!        F. Bouttier : 96-01-11    : Secure consistency of KDPROB with NVADIM
!        F. Bouttier : 96-05-30    : Print iteration index if given
!        F. Rabier     96-09-25 : no alloc/dealloc of traj when TL option
!        E. Andersson  97-05-21 : Make iter. counter available in /common/
!        F. Bouttier   97-10-20 & M. Hamrud : Speed up Jb in cont.var space
!        F. Bouttier   97-08-05 : add printouts and scrap RINCLJB
!        E. Holm       97-12-01 : Ozone included
!        F. Rabier     98-01-07 : add PPFLUSH  and bugfix in LGRASCAL
!        M.Hamrud    : 98-01-10 : Minimisation parallelized
!        T. Bergot     98-01    : pb alloc traj if L801TL=F (suallt / deallt)
!        L. Dragulanescu97-05-23: 3DVAR ALADIN
!        C. Fischer& C. Soci 98-03-25: modify call to ecain/cain in section 7
!        R. El Khatib  98-04-02 : Cleanup 3DVAR ALADIN
!        E. Andersson  98-08-18 : TOVS control variable
!        C. Fischer& M. Siroka 98-12-16: aladin weights for metric term
!        C. Fischer :  99-06-21 : nvarmax for global cost function (aladin)
!        A. Horanyi    99-06-23 : Special treatment of mean wind for ALADIN
!        R. El Khatib 99-02-05 ESPNORMSM => SPNORM
!        F. Bouttier   99-12-13 : cater for cost-functions without a Jb
!        J. Barkmeijer 00-11-06 : option LFORCE to use perturbed tendencies
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        Y.Tremolet    02-08-07 : Use control_vectors.
!        P. Smolikova 02-09-30 : interface to SUSPEC for d4 in NH
!        C. Fischer   03-09-30 : add zjblam to Jb and remove call to suellt
!        D. Dee        03-11-25 : introduced AFJPCOST for parameter estimation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        P.Burton :  29-09-04 LVARBC cleanup
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        F. Vana, R. Brozkova 13-Jan-2009 : switching off M-F physics before
!                                           variational computation
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : NCONF    ,NSTOP    ,NCNTVAR  ,&
 & LGUESS   ,NPROC   ,LBACKG  
USE YOMCT1   , ONLY : N1SDI
USE YOMSLREP , ONLY : LSLADREP
USE YOMCT3   , ONLY : NSTEP
USE YOMCOSJB , ONLY : FJBCOST, LJBZERO, LJPZERO, LJTZERO, LJLZERO
USE YOMCOSJO , ONLY : LADJOBS
USE YOMCVA   , ONLY : YSCALPSQRT, YRSCALPSQRT, YVAZ0
USE YOMVAR   , ONLY :     NSIMU    ,NSIM4D   ,NDIAG, &
 & NFRREF   ,NFRANA   ,NFRGRA   ,NSIM4DL  , LAVARC ,&
 & NITER4D  ,LTWGRA   ,LTWINC   ,LSKIPMIN ,LMODERR ,&
 & LGRASCAL, LJCDFI   ,LMPCGL   ,LAVCGL   ,LCONGRAD
USE YOMSIMPHL, ONLY : LSIMPH   ,LTRAJPS
USE YOPHLC   , ONLY : LSPHLC
USE YOMPHY   , ONLY : LMPHYS   ,LNEIGE   ,LCVRA    ,LGWD     ,&
  & LHMTO    ,LNEBCO   ,LRAY     ,LSFHYD  ,L3MT, LCVPRO, LCDDPRO,&
  & LSTRA    ,LTHERMO  ,LVDIF    ,LNEBN    ,LCVPP, LPTKE, LCONDWT
USE YOMPHY0  , ONLY : ECMNP    ,GWDCD    ,GWDSE
USE YOMPHY2  , ONLY : XMULAF
USE YOMSP    , ONLY : SPA2, SPA3, SPA1
USE YOMVRTL  , ONLY : L131TL   ,L801TL   ,LTLINT
USE YOMLCZ   ,ONLY  : LFORCE   ,LFORCEWR
USE YOMJG    , ONLY : LSUBBG, LCTLFG
USE YOMJQ    , ONLY : FJQCOST, LJQZERO
USE VARBC_SETUP, ONLY : AFJPCOST
USE YOMMP    , ONLY : NPSP
USE YOMDIM   , ONLY : NFD2D
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL     ,YGFLC    ,YQ       ,YO3
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, SPMODERR, GPMODERR, NDIM_MODERR,NPRTMODERR
USE CONTROL_VECTORS
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM)               :: KINDIC ! Argument NOT used
INTEGER(KIND=JPIM)               :: KDPROB ! Argument NOT used
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZX 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZF 
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KITER 
!     ------------------------------------------------------------------

!   IANATS/IGRATS : ARRAY CONTAINING ITERATIONS FOR WHICH THE
!   ANALYSIS/GRADIENT WILL BE WRITTEN TO FILE
INTEGER(KIND=JPIM) :: IANATS(0:NSIM4DL/NFRANA),IGRATS(0:NSIM4DL/NFRGRA)
INTEGER(KIND=JPIM) :: IREFTS(0:NSTOP/NFRREF)
INTEGER(KIND=JPIM) :: IHECPS(0:NSTOP)

! * LOCAL TYPE "control_vector" VARIABLES:
TYPE(control_vector) :: YL_ZWORK

LOGICAL :: LLFIRST, LLINOR, llprt

INTEGER(KIND=JPIM) :: ICONF, J, IDUM, jj, JGFL

REAL(KIND=JPRB) :: ZPSC, zjbtovs, zjblam, zjbpara, zjbcost, zjqcost, zjbts
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "cain.intfb.h"
#include "cainin.intfb.h"
#include "chavar.intfb.h"
#include "chavarin.intfb.h"
#include "chavarinad.intfb.h"
#include "cnt3.intfb.h"
#include "cnt3ad.intfb.h"
#include "cnt3tl.intfb.h"
#include "evcost.intfb.h"
#include "evjcdfi.intfb.h"
#include "fjvarbc.intfb.h"
#include "lcnortl.intfb.h"
#include "monvar.intfb.h"
#include "ppclose.intfb.h"
#include "ppflush.intfb.h"
#include "precond.intfb.h"
#include "scaas.intfb.h"
#include "stepo.intfb.h"
#include "su2yom.intfb.h"
#include "suallav.intfb.h"
#include "sualldfi.intfb.h"
#include "suallt.intfb.h"
#include "sbsfgs.intfb.h"
#include "sucos.intfb.h"
#include "deallav.intfb.h"
#include "dealldfi.intfb.h"
#include "suspec.intfb.h"
#include "suslad3.intfb.h"
#include "deallt.intfb.h"
#include "evjq.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIM4D',0,ZHOOK_HANDLE)
CALL GSTATS(21,0)

IDUM=-999
NITER4D=KITER

IF (NSIM4D<=1.OR.NSIM4D==NSIM4DL) write(nulout,*)'SIM4D: LSUBBG=',LSUBBG

IF (LSKIPMIN.AND.NSIM4D==NSIM4DL) YDVAZX=0.0_JPRB

!        1.    Initialize
!              -----------

!        1.1   INITIALISE ANALYSIS/GRADIENT WRITE-OUT EVENTS

WRITE(NULOUT,*) '-------Simulator : initializations ---------',&
 & '-------------------------------'  
CALL MONVAR(IREFTS,IANATS,IGRATS,IHECPS)
IF(NSIM4D == 0) THEN
  WRITE(UNIT=NULOUT,FMT='('' AN WRITE-OUT EVENTS, IANATS '')')
  WRITE(UNIT=NULOUT,FMT='(40I2)') (IANATS(J),J=0,NSIMU/NFRANA)
  WRITE(UNIT=NULOUT,FMT='('' GR WRITE OUT EVENTS, IGRATS '')')
  WRITE(UNIT=NULOUT,FMT='(40I2)') (IGRATS(J),J=0,NSIMU/NFRGRA)
ENDIF

CALL SU2YOM
NSTEP=0

!        1.2   RE-INITIALISE COST FUNCTIONS

ICONF=NCONF/100

IF(ICONF == 1.OR.ICONF == 7.OR.ICONF == 8)THEN
  CALL SUCOS
ENDIF

!        1.3   RESET GRADIENT (YDVAZG) TO ZERO.
!*             OR TO GRADIENT OF Jb

CALL GSTATS(49,0)
IF(LGUESS)THEN
  WRITE(NULOUT,*) '-------Simulator : cost-function Jb --------',&
   & '-------------------------------'  

  YDVAZG%data(:) = YDVAZX%data(:)
  IF (.NOT.LADJOBS) THEN
    IF (LCTLFG) THEN
      YDVAZG%data(:) = YDVAZG%data(:) + YVAZ0%data(:)
    ELSEIF (LMODERR) THEN
      YDVAZG%moderr(:) = YDVAZG%moderr(:) + YVAZ0%moderr(:)
    ENDIF
  ENDIF

  IF (LJBZERO.AND.YDVAZG%linitcv) YDVAZG%initcv(:) = 0.0_JPRB
  IF (LJTZERO.AND.YDVAZG%ltovscv) YDVAZG%tovscv(:) = 0.0_JPRB
  IF (LJLZERO.AND.YDVAZG%lam1d  ) YDVAZG%lamcv (:) = 0.0_JPRB
  IF (LJQZERO.OR.(LMODERR.AND.NTYPE_MODERR==1)) THEN
    IF (YDVAZG%nmoderr>0) YDVAZG%moderr(:) = 0.0_JPRB
  ENDIF

  IF (YDVAZX%npara>0) THEN
    IF (LJPZERO) THEN
      AFJPCOST=0.0_JPRB
      YDVAZG%params(:) = 0.0_JPRB
    ELSE
      CALL FJVARBC(AFJPCOST, YDVAZX%params, YDVAZG%params, YDVAZX%npara)  
    ENDIF
  ENDIF

  CALL CTLVEC_SQNORM(YDVAZG, FJBCOST, FJQCOST, zjbtovs, zjblam, zjbpara, zjbts)

  FJBCOST = 0.5_JPRB * (FJBCOST + zjbtovs + zjblam + zjbts)
  FJQCOST = 0.5_JPRB * FJQCOST

ELSE     ! if not(LGUESS) - it happens e.g. for sensitivity computations
  WRITE(NULOUT,*) '-------Simulator : no Jb, gradient set to zero ------',&
   & '-------------------------------'  
  YDVAZG=0.0_JPRB
ENDIF
CALL GSTATS(49,1)

!        1.4   ALLOCATE ADDITIONAL MEMORY FOR TRAJECTORY.

IF(NCONF == 801.AND..NOT.L131TL.AND..NOT.L801TL) THEN
  CALL SUALLT
ENDIF

!     .   1.5  Initialize the digital filter when the Jc is formulated
!     .        in terms of the DFI (memory allocation done in SU2DFI)

IF(L131TL.AND.LJCDFI) THEN
  CALL SUALLDFI(NULOUT)
ENDIF
IF(L131TL.AND.LAVARC) THEN
  CALL SUALLAV(NULOUT)
ENDIF
!     ------------------------------------------------------------------

!        2.  Fill SPA3 by the control variable and come back to physical
!            space.
!        ---------------------------------------------------------------
CALL GSTATS(50,0)
LLINOR=.FALSE.
LLFIRST=.TRUE.
IF(ICONF /= 1) THEN
  CALL SUSPEC(SPA3,SPA2,SPA1,KFILE=1)
  CALL ALLOCATE_CTLVEC(YL_ZWORK)
  CALL CHAVAR(YL_ZWORK)
  CALL CAIN  (YL_ZWORK)
  CALL DEALLOCATE_CTLVEC(YL_ZWORK)
ENDIF

!        2.5   CHANGE FROM CONTROL VARIABLES TO MODEL VARIABLES

WRITE(NULOUT,*) '-------Simulator : convert state from control ',&
 & 'to physical space -----------'  

CALL CHAVARIN (YDVAZX)

!        2.7   CHANGE FROM MODEL VARIABLES to TL MODEL VARIABLES

IF (L131TL.OR.L801TL) THEN
  WRITE(NULOUT,*) '-------Simulator : change from model to TL ',&
   & ' variables -------------------'  
  IF (LSUBBG) CALL SBSFGS
ENDIF

IF (L801TL) THEN
!back to model variables
  IF(LBACKG) THEN
    CALL ALLOCATE_CTLVEC(YL_ZWORK)
    CALL CAININ(YL_ZWORK)
    LSUBBG=.FALSE.
    NCNTVAR=2
    CALL CHAVARIN(YL_ZWORK)
    NCNTVAR=1
    LSUBBG=.TRUE.
    CALL DEALLOCATE_CTLVEC(YL_ZWORK)
  ENDIF

  LTWINC = .TRUE.
  NSTEP = 0
  CALL STEPO('A00000000')
  CALL PPFLUSH
  CALL PPCLOSE
  LTWINC = .FALSE.
ENDIF
IF (NSIM4D==0) CALL FLUSH(NULOUT)

IF (LSKIPMIN.AND.NSIM4D==NSIM4DL) THEN
  IF (NPSP==1) SPA2(:,1:NFD2D)=0.0_JPRB
  SPA3(:,:,:)=0.0_JPRB
  IF (YQ%LGP)  GFL(:,:,YQ%MP ,:)=0.0_JPRB
  IF (YO3%LGP) GFL(:,:,YO3%MP,:)=0.0_JPRB
  IF (LMODERR) THEN
    DO jj=1,NDIM_MODERR
      SPMODERR(jj)=0.0_JPRB
      GPMODERR(jj)=0.0_JPRB
    ENDDO
  ENDIF
ENDIF

CALL GSTATS(50,2)

!     ------------------------------------------------------------------

!        3.    DIRECT INTEGRATION.
!              -------------------

WRITE(NULOUT,*) '-------Simulator : call direct and adjoint ',&
 & ' model -----------------------'  
IF (L131TL.OR.L801TL) THEN
  LTLINT=.TRUE.
  IF(L131TL.OR.(L801TL.AND.NSIM4D /= 0)) THEN
    IF (LFORCE) THEN
      CALL ALLOCATE_CTLVEC(YL_ZWORK)
      LFORCEWR=.TRUE.
      CALL LCNORTL
      LFORCEWR=.FALSE.
      YL_ZWORK=0.0_JPRB
      CALL CAIN(YL_ZWORK)
      CALL DEALLOCATE_CTLVEC(YL_ZWORK)
    ENDIF
    CALL CNT3TL
  ENDIF
  LTLINT=.FALSE.
ELSE
  !! k.y: no ABOR1 should be put there! (used at METEO-FRANCE)
  !! CALL ABOR1('SIM4D: CALL CNT3: Is that ever used?')
  CALL CNT3
ENDIF
IF (NSIM4D==0) CALL FLUSH(NULOUT)

!     Computation of Jc when the DFI is used

IF(L131TL.AND.LJCDFI) THEN
  llprt=(NSIM4D==NSIM4DL)
  CALL EVJCDFI(NULOUT,llprt)
ENDIF

! Grad Jq can only be computed after TL has run if NTYPE_MODERR==1
IF ((LMODERR.AND.NTYPE_MODERR==1).AND..NOT.LJQZERO) THEN
  CALL EVJQ
ENDIF

!     ------------------------------------------------------------------

!        4.    ADJOINT INTEGRATION.
!              --------------------

! First switching off the MF non-linear physics
!  (to be able to run simplified physics, if asked)
IF(ICONF == 8.AND.(.NOT.LECMWF))THEN
  LMPHYS=.FALSE.
  LCVPP=.FALSE.
  LTHERMO=.FALSE.
  LVDIF=.FALSE.
  LRAY=.FALSE.
  LGWD=.FALSE.
  LCVRA=.FALSE.
  LSTRA=.FALSE.
  L3MT=.FALSE.
  LCVPRO=.FALSE.
  LCDDPRO=.FALSE.
  LPTKE=.FALSE.
  LCONDWT=.FALSE.
  LSFHYD=.FALSE.
  LNEBCO=.FALSE.
  LNEBN=.FALSE.
  LNEIGE=.FALSE.
  LHMTO=.FALSE.
  DO JGFL=1,YGFL%NUMFLDS
    YGFLC(JGFL)%LADV=.FALSE.
  ENDDO
  IF (LTRAJPS) THEN
    LSIMPH=.TRUE.
    YQ%LADV=.TRUE.
    LNEIGE=.TRUE.
    LSPHLC=.FALSE.
    ECMNP=7000._JPRB
    GWDCD=6._JPRB
    GWDSE=3.5E-03_JPRB
    XMULAF=0.0_JPRB
  ENDIF
ENDIF

CALL CNT3AD
IF (LSLADREP.AND.NSIM4D==0.AND.NPROC>1)THEN
  WRITE(NULOUT,'("SIM4D: CALLING SUSLAD3")')
  CALL SUSLAD3
ENDIF
IF (NSIM4D==0) CALL FLUSH(NULOUT)

!     ------------------------------------------------------------------

!        5.    COME BACK TO CONTROL VARIABLE SPACE AND FILL THE GRADIENT.
!              -----------------------------------------------------------

!        5.2   COME BACK TO CONTROL VARIABLE SPACE

CALL GSTATS(50,3)
IF (L801TL.AND.LBACKG) THEN
  WRITE(NULOUT,*) 'APPLY SQRT B'
  NCNTVAR=2
  LSUBBG=.FALSE.
  CALL CHAVARINAD (YDVAZG)
  LSUBBG=.TRUE.
  NCNTVAR=1
ELSE
  CALL CHAVARINAD (YDVAZG)
ENDIF
IF (LJQZERO.AND. NTYPE_MODERR==5) THEN
! make sure that model error remains zero, as if model error contrib to Jo was also zero...
! in this case LJQZERO means "strong_constr" through LMODERR=true
  FJQCOST=0.0_JPRB
  IF (YDVAZG%nmoderr>0) YDVAZG%moderr(:) = 0.0_JPRB
ENDIF
CALL GSTATS(50,1)

!     ------------------------------------------------------------------

!*       6.    SUM UP COST FUNCTION, RELEASE SPACE FOR TRAJECTORY.
!              ---------------------------------------------------

IF(NCONF == 801.AND..NOT.L131TL.AND..NOT.L801TL) THEN
  CALL DEALLT
ENDIF

CALL GSTATS(48,0)
CALL EVCOST(PZF)
CALL SCAAS(IDUM,YDVAZG,YDVAZG,ZPSC)
ZPSC=SQRT(ZPSC)/2.0
CALL GSTATS(48,1)
IF (.NOT.(LCONGRAD.OR.LAVCGL) .OR. NITER4D <= 5 .OR. NSIM4D==NSIM4DL) THEN
  WRITE(NULOUT,'('' GREPGRAD - LSIMPLE,ITER,SIM,GRAD,J   '',&
   & 2(1X,I3),2(1X,E22.16))')&
   & NITER4D,NSIM4D,ZPSC,PZF  
ENDIF

IF(NITER4D <= 5 .OR. NSIM4D==NSIM4DL) THEN

  WRITE(NULOUT,'('' GREPGRAD_hex - ITER,SIM,GRAD,J   '',&
   & 2(1X,I3),2(1X,Z17))')&
   & NITER4D,NSIM4D,ZPSC,PZF  
ENDIF

!     .   6.1  Deallocate memory used by the DFI

IF(L131TL.AND.LJCDFI) THEN
  CALL DEALLDFI(NULOUT)
ENDIF
IF(L131TL.AND.LAVARC) THEN
  CALL DEALLAV(NULOUT)
ENDIF
!     ------------------------------------------------------------------

!        7.    DIAGNOSTIC WRITE OUT OF GRADIENT
!              --------------------------------

IF(ICONF == 1) THEN
  IF(IGRATS(NSIM4D/NFRGRA) == 1.AND.MOD(NSIM4D,NFRGRA) == 0 &
     & .OR. NSIM4D == NSIM4DL) THEN  

    IF (LGRASCAL) THEN
!           WRITE GRADIENT WITH RESPECT TO SCALP INNER-PRODUCT
      CALL CAIN(YDVAZG)
      LTWGRA=.TRUE.

      CALL STEPO('A00000000')
      LTWGRA=.FALSE.
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!        8.    WRITE ANALYSIS
!              --------------

!        8.2   INCREASE SIMULATION COUNTER

IF((ICONF == 1.OR.ICONF == 8).AND.NDIAG /= 0.AND.NSIM4D /= NSIM4DL) THEN
  NSIM4D=NSIM4D+1
ENDIF
IF (NSIM4D<=2) THEN
  CALL FLUSH(NULOUT)
ELSE
  N1SDI=0
ENDIF

!        8.5   CONVERT GRADIENT FROM SCAAS TO PROSCA INNER PRODUCT
!              ---------------------------------------------------
!            (Note: CONGRAD preconditions using an explicit change
!             of variable, so don't do anything here if LAVCGL==.true.)

IF (LMPCGL .AND. .NOT.(LAVCGL .OR. LCONGRAD)) THEN
  YDVAZG%data = YDVAZG%data * YSCALPSQRT%data
  CALL PRECOND(YDVAZG,-1)
  YDVAZG%data = YDVAZG%data * YRSCALPSQRT%data
ENDIF

IF (NPRTMODERR>=2.AND.(NSIM4D<=2.OR.NSIM4D==NSIM4DL)) THEN
  CALL CTLVEC_SQNORM(YDVAZG, zjbcost, zjqcost, zjbtovs, zjblam, zjbpara, zjbts)
  WRITE(NULOUT,*)'SIM4D NORM GRAD IC, MODERR=',SQRT(zjbcost),SQRT(zjqcost)
ENDIF
!     ------------------------------------------------------------------

CALL GSTATS(21,1)
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIM4D',1,ZHOOK_HANDLE)
END SUBROUTINE SIM4D
