SUBROUTINE SPC2M(CDCONF)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NUMP
USE YOMLAP   , ONLY : MYMS

!**** *SPC2M* - Multi tasking interface to SPC2

!     Purpose.
!     --------
!        Multi tasking interface (also calls SPC2 single-tasked if
!        multi-tasking not requested).

!**   Interface.
!     ----------
!        *CALL* *SPC2M(CDCONF)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)

!        Implicit arguments :
!        --------------------
!                            NONE.
!     Method.
!     -------

!     Externals.   SPC2 - Spectral space computations - 2-D case
!     ----------   Called by SPCH.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-11-24
!        Modified by K. YESSAD (JAN 1994): multitasking on zonal wave number.
!        MPP Group: 95-10-01 Support for Distributed Memory version
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad 15-May-2006: memory optimisations for stretched geometry
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JM, IM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "spc2.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPC2M',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    LOOP OVER ZONAL WAVENUMBER M.
!              -----------------------------

DO JM=1,NUMP
  IM=MYMS(JM)
  CALL SPC2(CDCONF,IM,JM)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPC2M',1,ZHOOK_HANDLE)
END SUBROUTINE SPC2M
