SUBROUTINE SPCH(CDCONF)

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0    ,ONLY : NCONF


!**** *SPCH* - Caller for spectral space computations

!     Purpose.
!     --------
!        Prepare for calling spectral space computations by setting
!        up IO-scheme etc. Call SPCM - multi-tasking interface
!        to spectral space computations.

!**   Interface.
!     ----------
!        *CALL* *SPCH(CDCONF)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)
!        --------------------

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   SPCM           - spectral space computations
!     ----------   
!                  SPC2M          - spectral space computations - 2D

!                  Called by STEPO and STEPOTL.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original   : 93-03-01
!        L. Labbe   : 93-04-15 - NCONF=821 barotropic model
!        K. Yessad  : 93-03-18 - Add extra horizontal diffusion.
!        K. Yessad  : 93-10-15 - semi-implicit computations
!                                with not reduced divergence.
!        K. Yessad  : 94-01-15 - semi-implicit computations
!           with not reduced divergence + new formulation of variable
!           mesh horizontal diffusion + multitasking on m instead of n.
!        MPP Group  : 95-10-01 - Message Passing option added
!        L. Isaksen : 96-05-05 - Data structure of ZSPxG arrays changed
!        L. Isaksen : 96-12-12 - Merge Message Passing with SH-memory
!        D. Dent    : 97-10-02 - replace SECOND by USER_CLOCK
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

LOGICAL :: LL2D
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "spc2m.intfb.h"
#include "spcm.intfb.h"

!     ------------------------------------------------------------------

!*       1.    CALL SPECTRAL COMPUTATIONS.
!              ---------------------------

!        1.1  Spectral computations multitasked on M (SPCM or SPC2M).

IF (LHOOK) CALL DR_HOOK('SPCH',0,ZHOOK_HANDLE)
LL2D= NCONF/100 == 2 .OR. NCONF/10 == 42 .OR. NCONF/10 == 52 

IF (LL2D) THEN
  CALL SPC2M(CDCONF)
ELSE
  CALL SPCM(CDCONF)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCH',1,ZHOOK_HANDLE)
END SUBROUTINE SPCH
