SUBROUTINE SPCHAD(CDCONF)

!**** *SPCHAD* - Caller for spectral space computations - adjoint

!     Purpose.
!     --------
!        Prepare for calling spectral space computations by setting
!        up IO-scheme etc. Call SPCMAD - multi-tasking interface
!        to spectral space computations.

!**   Interface.
!     ----------
!        *CALL* *SPCHAD(CDCONF)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)
!        --------------------

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   SPCMAD          - spectral space computations
!     ----------  
!                  SPC2MAD         - spectral space computations - 2D

!                  Called by STEPOAD.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original   : 93-03-01
!        K. Yessad  : 93-03-18 Add extra horizontal diffusion.
!        L. Labbe   : 93-04-28 NCONF=821 = 2D model
!        K. Yessad  : 94-03-15 All spectral computations now multitasked
!                              on zonal wave nomber m.
!        L. Isaksen : 96-02-20 Adapted for Message Passing
!        L. Isaksen : 96-05-05 Data structure of ZSPxG arrays changed
!        D. Paradis : 96-10-17 Splitting spcdm in spcdmsi and spcdmhd
!        L. Isaksen : 96-12-12 Merge Message Passing with SH-memory
!        D. Dent    : 97-10-02 - replace SECOND by USER_CLOCK
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NCONF
USE YOMDYN   , ONLY : TDT
USE YOMCT3   , ONLY : NSTEP
USE YOMCT2   , ONLY : NSTOP2
USE YOMLCZ   , ONLY : LFORCE   ,YSPFORCE
USE CONTROL_VECTORS

IMPLICIT NONE

! * DUMMY:
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
!     ------------------------------------------------------------------

! * LOCAL VARIABLES:
TYPE(control_vector) :: YL_ZVAGR

LOGICAL :: LL2D
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cainin.intfb.h"
#include "spc2mad.intfb.h"
#include "spcmad.intfb.h"

!     ------------------------------------------------------------------

!*       1.    CALL SPECTRAL COMPUTATIONS.
!              ---------------------------

!        1.1  Spectral computations multitasked on M (SPCMAD or SPC2MAD).

IF (LHOOK) CALL DR_HOOK('SPCHAD',0,ZHOOK_HANDLE)
LL2D= NCONF/100 == 2 .OR. NCONF/10 == 42 .OR. NCONF/10 == 52

IF (LL2D) THEN
  CALL SPC2MAD(CDCONF)
ELSE
  CALL SPCMAD(CDCONF)
ENDIF
IF(LFORCE) THEN
  IF(NSTEP==NSTOP2-1) YSPFORCE=0.0_JPRB
  CALL ALLOCATE_CTLVEC(YL_ZVAGR)
  CALL CAININ(YL_ZVAGR)
  YSPFORCE%data = TDT * YL_ZVAGR%data + YSPFORCE%data
  CALL DEALLOCATE_CTLVEC(YL_ZVAGR)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCHAD',1,ZHOOK_HANDLE)
END SUBROUTINE SPCHAD
