SUBROUTINE TESTLIEVOL

!**** *TESTLIEVOL*  - Computation of the direct model/tangent 
!                     linear model
 
!     PURPOSE.
!     --------
!           Computation of evolution of the perturbation using
!           two nonlinear integrations (direct model) from different
!           initial states and the tangent linear integration with
!           initial state equal to analysis increment

!**   INTERFACE.
!     ----------
!        *CALL* *TESTLIEVOL*

!        EXPLICIT ARGUMENTS :
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS :
!        --------------------
!        NONE

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.   SU2YOM - INITIALIZE LEVEL 2 COMMONS
!     ----------   SUALLT - ALLOCATE SPACE FOR TRAJECTORY
!                  CNT3   - CONTROLS DIRECT INTEGRATION
!                  SUPERT - INITIALIZE PERTURBATION
!                  CAIN   - CANONICAL INJECTION
!                  CNT3TL - CONTROLS TANGENT LINEAR INTEGRATION
!                  CAININ - INVERSE OF THE CANONICAL INJECTION
!                  SCAAS  - SCALAR PRODUCT
!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2000-06-01 - based on testli.F90
!        Y.Tremolet : 02-10-01 Use control_vectors.
!        M.Janiskova: 03-05    lphylin=false for NL computation
!        C. Fischer : 03-09-30 remove call to suellt
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        M.Janiskova:  14-Dec-2005 Set cloud related perturbations to zero
!        M.Janiskova:  12-Mar-2009 Initialize switch for grid-point cloud diag.
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NFLEVG
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT1   , ONLY : N1HIS,   N1GDI 
USE YOMCVA   , ONLY : YVAZX
USE YOMVAR   , ONLY : NSIM4D
USE YOMSP    , ONLY : SPA3, SPA2, SPA1,SPOR
USE YOEPHY   , ONLY : LEPCLD
USE YOPHNC   , ONLY : LERADSW2, LENCLD2
USE YOERAD   , ONLY : NTSW, NSW
USE YOEPHLI  , ONLY : LPHYLIN
USE CONTROL_VECTORS
USE GFL_SUBS_MOD ,ONLY  : DEACT_CLOUD_GFL,REACT_CLOUD_GFL
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YA, YL, YI, YR, YS

!     ------------------------------------------------------------------

IMPLICIT NONE

! * LOCAL TYPE "control_vector" VARIABLES:
TYPE(control_vector) :: YL_ZVATRA(8)

INTEGER(KIND=JPIM) ::  JJ, JLAMDA
INTEGER(KIND=JPIM) ::  ISW, I1GDI

LOGICAL :: LLSPOR, LLPCLD, LLSWR2, LLDIAG

REAL(KIND=JPRB) :: ZVALU
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "cain.intfb.h"
#include "cainin.intfb.h"
#include "cnt3.intfb.h"
#include "cnt3tl.intfb.h"
#include "spnorm.intfb.h"
#include "su2yom.intfb.h"
#include "suaersn.intfb.h"
#include "suallt.intfb.h"
#include "suclopn.intfb.h"
#include "sugridf.intfb.h"
#include "sugridu.intfb.h"
#include "supert.intfb.h"
#include "suspec.intfb.h"
#include "suswn.intfb.h"

!     ------------------------------------------------------------------

!*       1.    INITIALIZATIONS.
!              ----------------
 
IF (LHOOK) CALL DR_HOOK('TESTLIEVOL',0,ZHOOK_HANDLE)

LLPCLD=.FALSE.
LLSWR2=.FALSE.
LLDIAG=.FALSE.

DO JJ=1,8
  CALL ALLOCATE_CTLVEC(YL_ZVATRA(JJ))
ENDDO

!*       1.1   INITIALIZE YOMCT2

CALL SU2YOM

!*       1.2   ALLOCATE EXTRA SPACE FOR TRAJECTORY.

CALL SUALLT

!*       1.3   INITIALIZATION OF THE BASIC PERTURBATION

ZVALU=1.0_JPRB
CALL SUPERT(YVAZX,ZVALU)

!*       1.4   STORAGE OF THE BASIC STATE

CALL CAININ(YL_ZVATRA(5))

!*       1.5   COMPUTATION OF THE TRAJECTORY

CALL CAIN(YL_ZVATRA(5))

N1HIS=1
NSIM4D=10
WRITE (NULOUT,*) 'NORM BEFORE CNT3 - FIRST GUESS'
CALL SPNORM(0)
 
IF (LEPCLD) THEN
  CALL SUGRIDU(0)
ENDIF

IF (LECMWF .AND. (LEPCLD .OR. LENCLD2)) THEN
  LLDIAG=.TRUE.
  I1GDI = N1GDI
  N1GDI = 1
ENDIF

CALL CNT3
CALL CAININ(YL_ZVATRA(1))

IF (LLDIAG) THEN
  N1GDI = I1GDI
  LLDIAG=.FALSE.
ENDIF

!*       1.6   COMPUTATION OF THE PERTURBATION

LLSPOR=.FALSE.
CALL SUSPEC(SPA3,SPA2,SPA1,KFILE=2)

WRITE (NULOUT,*) 'NORM AFTER SUSPEC - ANALYSIS'
WRITE(0,*) 'NORM AFTER SUSPEC - ANALYSIS -mj'
CALL SPNORM(0)
CALL CAININ(YVAZX)
YL_ZVATRA(2)%data = YVAZX%data - YL_ZVATRA(5)%data

!     ------------------------------------------------------------------

!*       2.  TANGENT-LINEAR EVOLUTION
!            ------------------------

!*       2.1  CHANGE OF CLOUD SCHEME

! Switch-off prognostic cloud scheme

IF (LEPCLD) THEN
  LEPCLD=.FALSE.
  LLPCLD=.TRUE.
  CALL DEACT_CLOUD_GFL
ENDIF

! Put perturbation of cloud variables to zero

IF (LENCLD2) THEN
  GFL(:,:,YA%MP,:) = 0.0_JPRB
  GFL(:,:,YL%MP,:) = 0.0_JPRB
  GFL(:,:,YI%MP,:) = 0.0_JPRB
  GFL(:,:,YR%MP,:) = 0.0_JPRB
  GFL(:,:,YS%MP,:) = 0.0_JPRB

  N1GDI = 1
ENDIF

! Change to just 2 spectral intervals for shortwave radiation

IF (LERADSW2 .AND. NSW > 2) THEN
  LLSWR2=.TRUE.
  
  ISW=NSW
  NSW=2
  CALL SUSWN   (NTSW, NSW)
  CALL SUCLOPN (NTSW, NSW,NFLEVG)
  CALL SUAERSN (NTSW, NSW)
ENDIF

!*       2.2  COMPUTATION OF CNT3TL * PERT

CALL CAIN(YL_ZVATRA(2))

N1HIS=0
NSIM4D=40
WRITE (NULOUT,*) 'NORM BEFORE CNT3TL- PERTURB'
WRITE(0,*) 'NORM BEFORE CNT3TL- PERTURB -mj'
CALL SPNORM(0)
CALL CNT3TL
N1HIS=1
CALL CAININ(YL_ZVATRA(3))

!     ------------------------------------------------------------------

!*       3.    LOOP OVER LAMBDA ( PERT. = LAMBDA * BASIC PERT. )
!              -------------------------------------------------

!*       3.1  CHANGE OF CLOUD SCHEME

! Switch-on prognostic cloud scheme

IF (LLPCLD) THEN
  LEPCLD=.TRUE.
  CALL REACT_CLOUD_GFL
ENDIF

! Change back to original number of spectral intervals for sw

IF (LLSWR2) THEN
  NSW=ISW
  CALL SUSWN   (NTSW, NSW)
  CALL SUCLOPN (NTSW, NSW,NFLEVG)
  CALL SUAERSN (NTSW, NSW)
ENDIF

LPHYLIN=.FALSE.

!     DO JLAMDA=10,0,-1
JLAMDA=0

!*       3.2   COMPUTATION OF ( STATE + PERT )

IF (LEPCLD .OR. LENCLD2) THEN
  N1GDI = 1
  CALL SUGRIDU(0)
ENDIF

YL_ZVATRA(4)%data = YL_ZVATRA(5)%data + YL_ZVATRA(2)%data * 10._JPRB**(-JLAMDA)

!*       3.3   COMPUTATION OF CNT3 ( STATE + PERT )

CALL CAIN(YL_ZVATRA(4))

N1HIS=0
NSIM4D=20+JLAMDA
WRITE (NULOUT,*) 'NORM BEFORE CNT3 FOR JLAMDA=',JLAMDA
CALL SPNORM(0)
CALL SUGRIDF(1,SPOR,LLSPOR)
CALL CNT3
N1HIS=1
CALL CAININ(YL_ZVATRA(6))

DO JJ=1,8
  CALL DEALLOCATE_CTLVEC(YL_ZVATRA(JJ))
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TESTLIEVOL',1,ZHOOK_HANDLE)
END SUBROUTINE TESTLIEVOL
