MODULE CPLNG_PREPARE_CHEM_MOD

  IMPLICIT NONE

  PRIVATE

  PUBLIC CPLNG_PREPARE_CHE

CONTAINS

SUBROUTINE CPLNG_PREPARE_CHE
  !
  !**** *CPLNG_PREPARE_CHE*  - Prepare rest of the exchanged meteo fields for chemistry
  !
  !     Purpose.
  !     --------
  !       Prepare the rest of the exchanged meteo fields for chemistry
  !
  !**   Interface.
  !     ----------
  !       *CALL*  *CPLNG_PREPARE_CHE*
  !
  !     Author:
  !     -------
  !       T. van Noije, KNMI, Jan-2014
  !       File created for EC-Earth v3 from storeoasis3tm.F90 (used in v2)
  !
  !     Modifications.
  !     --------------
  !
  !     -----------------------------------------------------------
  USE PARKIND1 , ONLY : JPIM, JPRB
  USE YOMHOOK  , ONLY : LHOOK, DR_HOOK
  USE YOMGEM   , ONLY : VAH, VBH, NGPTOT  ! As, Bs, total number of grid columns on a PE

  ! gridded fields
  USE YOMDIM ,  ONLY : NPROMA   ! working dimension for grid-point computations
  USE YOMDIM ,  ONLY : NFLEVG   ! number of levels in grid point space
  USE YOMGMV ,  ONLY : GMV, GMVS, YT0
  USE YOMGFL ,  ONLY : GFL
  USE YOM_YGFL, ONLY : YQ
  USE YOMGC  ,  ONLY : OROG   ! orography 
  
  ! spectral fields
  USE YOMDIM   , ONLY : NSMAX, NUMP ! truncation, nr of local wave numbers, nr of real coeff
  USE YOMLAP   , ONLY : MYMS   ! local wave numbers
  USE YOMDIM   , ONLY : NFLEVL   ! number of levels in Fourier and Legendre space
  USE YOMSP    , ONLY : SPSP, SPVOR, SPDIV  ! log.surf.pres, vorticity, divergence

  USE CPLNG_DATA_MOD

  IMPLICIT NONE

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  ! items to extract local data:
  INTEGER(KIND=JPIM) :: JSTGLO, ISTC, ICEND, INDEX
  INTEGER(KIND=JPIM) :: IBL
  INTEGER(KIND=JPIM) :: IWAVE, ISPM, ISPN, ISPK
  INTEGER(KIND=JPIM) :: J, JLEV, NLEV

  !-------------- For vertical remapping of 3D fields exchanged with TM5
  REAL(KIND=JPRB), ALLOCATABLE :: PRESTH_IFS(:,:), PRESTH_TM5(:,:) ! pressure thickness
  REAL(KIND=JPRB)    :: ZPRESH(NPROMA,0:NFLEVG) ! half level pressure
  INTEGER(KIND=JPIM) :: JTOP, JBOT, JIND

#include "sc2rdg.intfb.h"
#include "gppreh.intfb.h"
  
  !     -----------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('CPLNG_PREPARE_CHE',0,ZHOOK_HANDLE)

  ! ** Layer pressure thickness **
  ALLOCATE(PRESTH_IFS(NGPTOT,NFLEVG  ))
  ALLOCATE(PRESTH_TM5(NGPTOT,NLEV_TM5))
  
  DO JSTGLO = 1, NGPTOT, NPROMA
     ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
     ISTC=1
     IBL=(JSTGLO-1)/NPROMA+1
     DO J=ISTC,ICEND
        ZPRESH(J,NFLEVG) = EXP(GMVS(J,YT0%MSP,IBL))
     ENDDO     
     CALL GPPREH(NPROMA,ISTC,ICEND,NFLEVG,VAH,VBH,ZPRESH)
     DO JLEV=1,NFLEVG
        DO J=ISTC,ICEND
           PRESTH_IFS(JSTGLO+J-1,JLEV) = ZPRESH(J,JLEV) - ZPRESH(J,JLEV-1)
        ENDDO
     ENDDO
     DO JLEV = 1, NLEV_TM5
        JTOP = LEV_IDX_MIN(JLEV)
        JBOT = LEV_IDX_MAX(JLEV)
        DO J=ISTC,ICEND
           PRESTH_TM5(JSTGLO+J-1,JLEV) = ZPRESH(J,JBOT) - ZPRESH(J,JTOP-1)
        ENDDO
     ENDDO
  ENDDO
  
  ! ** Gridded fields **
    
  NLEV = CPLNG_FLD(CPLNG_IDX('A_TMP'))%NUM_LVL   ! can be either NFLEVG or NLEV_TM5

  ! Surface pressure
  DO JSTGLO = 1, NGPTOT, NPROMA
     ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
     ISTC=1
     IBL=(JSTGLO-1)/NPROMA+1
     DO J=ISTC,ICEND
        INDEX=(IBL-1)*NPROMA+J
        CPLNG_FLD(CPLNG_IDX('A_SPRES'))%D(INDEX,1,1) = EXP(GMVS(J,YT0%MSP,IBL))
     ENDDO
  ENDDO

  ! Orography
  CPLNG_FLD(CPLNG_IDX('A_OROG'))%D(:,1,1) = OROG
  
  ! Air temperature (mass-weighted-averaged over the layers)
  !  Gridded U/V-winds are not used but could be added here, replace YT0%MT
  !  with YTO%MU for U-wind, and YT0%MV for V-wind. 
  DO JLEV = 1, NLEV
     JTOP = LEV_IDX_MIN(JLEV)
     JBOT = LEV_IDX_MAX(JLEV)
     
     DO JSTGLO = 1, NGPTOT, NPROMA
        ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        ISTC=1
        IBL=(JSTGLO-1)/NPROMA+1
        DO J=ISTC,ICEND
           INDEX=(IBL-1)*NPROMA+J

           CPLNG_FLD( CPLNG_IDX('A_TMP'))%D(INDEX,JLEV,1) = 0.0_JPRB

           DO JIND = JTOP, JBOT
              CPLNG_FLD( CPLNG_IDX('A_TMP'))%D(INDEX,JLEV,1) = &
            & CPLNG_FLD( CPLNG_IDX('A_TMP'))%D(INDEX,JLEV,1) + GMV(J,JIND,YT0%MT,IBL) * PRESTH_IFS(INDEX,JIND)
           ENDDO

           CPLNG_FLD( CPLNG_IDX('A_TMP'))%D(INDEX,JLEV,1) = &
         & CPLNG_FLD( CPLNG_IDX('A_TMP'))%D(INDEX,JLEV,1) / PRESTH_TM5(INDEX,JLEV)
        ENDDO
     ENDDO
  ENDDO
     
  ! Humidity (mass-weighted-averaged over the layers)
  DO JLEV = 1, NLEV
     JTOP = LEV_IDX_MIN(JLEV)
     JBOT = LEV_IDX_MAX(JLEV)

     DO JSTGLO = 1, NGPTOT, NPROMA
        ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        ISTC=1
        IBL=(JSTGLO-1)/NPROMA+1
        DO J=ISTC,ICEND
           INDEX=(IBL-1)*NPROMA+J
           
           CPLNG_FLD( CPLNG_IDX('A_HUM'))%D(INDEX,JLEV,1) = 0.0_JPRB

           DO JIND = JTOP, JBOT
              CPLNG_FLD( CPLNG_IDX('A_HUM'))%D(INDEX,JLEV,1) = &
            & CPLNG_FLD( CPLNG_IDX('A_HUM'))%D(INDEX,JLEV,1) + GFL(J,JIND,YQ%MP,IBL) * PRESTH_IFS(INDEX,JIND)
           ENDDO

           CPLNG_FLD( CPLNG_IDX('A_HUM'))%D(INDEX,JLEV,1) = &
         & CPLNG_FLD( CPLNG_IDX('A_HUM'))%D(INDEX,JLEV,1) / PRESTH_TM5(INDEX,JLEV)
        ENDDO
     ENDDO
  ENDDO


  ! ** Spectral fields **

  NLEV = CPLNG_FLD(CPLNG_IDX('A_VOR'))%NUM_LVL   ! can be either NFLEVL(=NFLEVG with LTM5) or NLEV_TM5
  
  ! Spectral info
  ISPK = 1                  ! index in coeff array
  DO IWAVE = 1, NUMP        ! loop over waves on this pe
     ISPM = MYMS(IWAVE)     ! global wave number
     DO ISPN = ISPM, NSMAX  ! loop over complex coeff
    
        CPLNG_FLD(CPLNG_IDX('A_SPINF'))%D(ISPK  ,1,1) = ISPM*1000.0 + ISPN + 0*0.01
        CPLNG_FLD(CPLNG_IDX('A_SPINF'))%D(ISPK+1,1,1) = -1.0 * CPLNG_FLD(CPLNG_IDX('A_SPINF'))%D(ISPK,1,1)
    
        ISPK = ISPK + 2     ! next pair
     ENDDO
  ENDDO
    
  ! Log surface pressure
  CPLNG_FLD(CPLNG_IDX('A_LNSP'))%D(:,1,1) = SPSP(:)
    
  ! Potential vorticity (non-weighted average over layers)
  DO JLEV = 1, NLEV
     JTOP = LEV_IDX_MIN(JLEV)
     JBOT = LEV_IDX_MAX(JLEV)
     
     CPLNG_FLD( CPLNG_IDX('A_VOR'))%D(:,JLEV,1) = SUM(SPVOR(JTOP:JBOT,:), DIM=1) / ((JBOT-JTOP+1)*1.0_JPRB)
  ENDDO
    
  ! Divergence (non-weighted average over layers)
  DO JLEV = 1, NLEV
     JTOP = LEV_IDX_MIN(JLEV)
     JBOT = LEV_IDX_MAX(JLEV)
     
     CPLNG_FLD(CPLNG_IDX('A_DIV'))%D(:,JLEV,1) = SUM(SPDIV(JTOP:JBOT,:), DIM=1) / ((JBOT-JTOP+1)*1.0_JPRB)
  ENDDO
  
  ! ** Cleanup **
  DEALLOCATE(PRESTH_TM5, PRESTH_IFS)
  
  IF (LHOOK) CALL DR_HOOK('CPLNG_PREPARE_CHE',1,ZHOOK_HANDLE)

END SUBROUTINE CPLNG_PREPARE_CHE

END MODULE CPLNG_PREPARE_CHEM_MOD
