SUBROUTINE DFI1(KDFITYP,KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)

!**** *DFI1*  - Controls integration for digital filter initialization

!     Purpose.
!     --------
!     CONTROLS  DIGITAL FILTER INITIALIZATION ON LEVEL 1

!**   Interface.
!     ----------
!        *CALL* *DFI1(...)*

!        Explicit arguments :
!        --------------------
!        KDFITYP : 1..8: option for type of DFI
!        KSTDFI  : number of diab. DFI integration steps in one direction
!        PSTEP   : diabatic time step for dfi
!        KSTDFIA : number of adiab. DFI integration steps in one direction
!        PSTEPA  : adiabatic time step for dfi
!        LDIFH  : controls horizontal diffusion for adiabatic integrations

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------

!     Externals.
!     ----------
!        DFI2   : next level DFI routine
!        DFI2MOD:        "              (modified version FOR NEDFI=5)

!     Reference.
!     ----------
!     ARPEGE/ALADIN DOCUMENTATION

!     Author.
!     -------
!       Gabor Radnoti GMAP/MICECO
!       Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays

!     Modifications.
!     --------------
!        Original : 92-12-24
!        D. Giard : 93-07-28 time step as argument ; EINIBACK,EDIACORR,
!                            EINIFIE replaced by COPSP and DIFSP
!        D. Giard : 94-10-18 adiabatic time-step
!        Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!        D.Giard, J.Latour : 96-09-20 spectral arrays are no longer arguments 
!        D. Giard : 98-01-29 LDIFH as argument
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDFITYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDFI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDFIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTEPA 
LOGICAL           ,INTENT(IN)    :: LDIFH 
LOGICAL :: LLDCOR, LLDIAB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "dfi2.intfb.h"
#include "dfi2mod.intfb.h"

!  First 3 DFI options: (KDFITYP=1...3)

IF (LHOOK) CALL DR_HOOK('DFI1',0,ZHOOK_HANDLE)
IF (KDFITYP <= 3) THEN
  LLDIAB=.TRUE.
  IF (KDFITYP == 1) LLDIAB=.FALSE.
  LLDCOR=.TRUE.
  IF (KDFITYP == 2) LLDCOR=.FALSE.
  CALL DFI2(KDFITYP,LLDIAB,LLDCOR,KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)
ENDIF

!  ADFI + DDFI (KDFITYP=4)

IF (KDFITYP == 4) THEN
  LLDCOR=.TRUE.
  LLDIAB=.FALSE.
  CALL DFI2(KDFITYP,LLDIAB,LLDCOR,KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)
  LLDIAB=.TRUE.
  CALL DFI2(KDFITYP,LLDIAB,LLDCOR,KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)
ENDIF

!  Modified DFI (KDFITYP=5)

IF (KDFITYP == 5) CALL DFI2MOD(KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)

!  Forward or Backward+Forward DDFI (KDFITYP=6)

IF (KDFITYP >= 6) THEN
  LLDCOR=.FALSE.
  LLDIAB=.TRUE.
  CALL DFI2(KDFITYP,LLDIAB,LLDCOR,KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)
ENDIF

IF (LHOOK) CALL DR_HOOK('DFI1',1,ZHOOK_HANDLE)
END SUBROUTINE DFI1

