SUBROUTINE DFI2(KDFITYP,LDIAB,LDCOR,KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVL   ,NFD2D    ,NFLSUR   ,NSPEC2   ,NS3D
USE YOMCT0   , ONLY : NSTOP    ,LELAM    ,LOBSC1   ,LSPRT
USE YOMCT2   , ONLY : NSTAR2   ,NSTOP2
USE YOMDYN   , ONLY : TSTEP
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMSP    , ONLY : SPA3     ,SPA2  ,SPOR ,SPD2D 
USE YOMPHY   , ONLY : LMPHYS   ,LREASUR
USE YOEPHY   , ONLY : LEPHYS
USE YOMINI   , ONLY : LBIAS    ,LINCR    ,LSCRINI
USE YOMLUN   , ONLY : NULOUT


!**** *DFI2*  - Controls integration for digital filter initialization

!     Purpose.
!     --------
!     CONTROLS  DIGITAL FILTER INITIALIZATION ON LEVEL 2

!**   Interface.
!     ----------
!        *CALL* *DFI2(...)*

!        Explicit arguments :
!        --------------------
!        KDFITYP: 1..6 option for DFI type
!        LDIAB  : controls if filter is diabatic
!        LDCOR  : controls if correction is necessary in case of diabatic filter
!        KSTDFI : number of diab. integration steps in one direction
!        PSTEP  : diabatic time step for dfi
!        KSTDFIA: number of adiab. integration steps in one direction
!        PSTEPA : adiabatic time step for dfi
!        LDIFH  : controls horizontal diffusion for adiabatic integrations

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------

!     Externals.
!     ----------
!       DFI3   : deepest level DFI control routine
!       ZEROACU: sets accumulation arrays zero
!       COPSP  : duplicates spectral state vectors
!       DIFSP  : calculates difference of two sets of fields
!       CORSP  : calculates difference of two sets of fields
!       REAST  : resets astronomical constants after each integration session
!       SUGRIDF : initialize gridpoint fields
!       EINTFAC : interface for SPA1 (specific ALADIN-glob. array)

!     Reference.
!     ----------
!     ARPEGE/ALADIN DOCUMENTATION

!     Author.
!     -------
!       Gabor Radnoti GMAP/MICECO

!     Modifications.
!     --------------
!        Original : 92-12-24
!        D. Giard : 93-07-28 EINIBACK, EDIACORR, EINIFIE replaced by
!                            COPSP and DIFSP ; PSTEP as argument ;
!                            horizontal diffusion handled with CDCONF(9)
!        D. Giard : 93-07-28 options KDFITYP=7 and 8
!        D. Giard : 94-10-18 adiabatic time-step
!        V. Ivanovici, D. Giard : 95-01-19 backward+forward filtering
!        M. Hamrud: 95-10-01 interface block for SUSPEC
!        L.Isaksen: 95-06-06 reordering of spectral arrays
!        D.Giard  : 97-02-10 call to CORSP
!        M.Siroka : 97-05-26 incremental DFI for LELAM
!        E. Holm  : 97-11-28 call to SUSPEC changed (incl. ozone)
!        D. Giard : 98-01-29 LDIFH as argument
!        R. El Khatib : 98-04-02 : Cleanup arguments to SUSPEC
!        D. Giard : 98-08-27 modifications for finalisation
!        D. Giard : 00-07-11 screening and corrections for finalisation
!        D. Giard : 00-11-06 fix bug in incremental for ALADIN
!        P. Smolikova 02-09-30 : interface to SUSPEC for d4 in NH
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDFITYP 
LOGICAL           ,INTENT(IN)    :: LDIAB
LOGICAL           ,INTENT(IN)    :: LDCOR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDFI
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDFIA
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTEPA
LOGICAL           ,INTENT(IN)    :: LDIFH
REAL(KIND=JPRB) :: ZDSPA3(NFLSUR,NSPEC2,NS3D),   ZCSPA3(NFLSUR,NSPEC2,NS3D)
REAL(KIND=JPRB) :: ZDSPD2D(NSPEC2,NFD2D),   ZCSPD2D(NSPEC2,NFD2D)
REAL(KIND=JPRB) :: ZDSPA1(NFLEVL,2),   ZCSPA1(NFLEVL,2),   ZUVAVG(NFLEVL,2)

INTEGER(KIND=JPIM) :: IDEC, IDFI, ININDAT, INSSSSS, ISTAR2, ISTOP, ISTOP2, JL

LOGICAL :: LLEPHYS, LLMPHYS, LLSPOR

REAL(KIND=JPRB) :: ZEPS, ZTSTEP, ZEPL
REAL(KIND=JPRB),POINTER :: ZSPUB(:), ZSPVB(:)
REAL(KIND=JPRB),TARGET :: ZSPA1(NFLSUR,2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "copsp.intfb.h"
#include "corsp.intfb.h"
#include "dfi3.intfb.h"
#include "difsp.intfb.h"
#include "eintfac.intfb.h"
#include "reast.intfb.h"
#include "redate.intfb.h"
#include "especrt.intfb.h"
#include "zeroacu.intfb.h"
#include "specrt.intfb.h"
#include "sugridf.intfb.h"
#include "suspec.intfb.h"


!*       0.    Initialize local variables and arrays
!              -------------------------------------
IF (LHOOK) CALL DR_HOOK('DFI2',0,ZHOOK_HANDLE)
ZEPS= 1.E-14_JPRB
ZEPL= 1._JPRB
LLSPOR =.FALSE.
ZSPUB=> ZSPA1(:,1)
ZSPVB=> ZSPA1(:,2)

WRITE (NULOUT,*) 'CALL TO DFI',KDFITYP


CALL ZEROACU(ZDSPA3,ZDSPD2D,ZDSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,1)
CALL COPSP(ZCSPA3,ZCSPD2D,ZCSPA1,SPA3,SPD2D,ZUVAVG)

!*       1. save constants to local ones before overwriting
!           -----------------------------------------------

LLMPHYS  = LMPHYS
LLEPHYS  = LEPHYS
ZTSTEP   = TSTEP
ISTAR2   = NSTAR2
ISTOP2   = NSTOP2
ISTOP    = NSTOP
NSTAR2   = 999
NSTOP2   = 999
NSTOP    = 999
ININDAT  = NINDAT
INSSSSS  = NSSSSS

!*       2.1 Launching different DFI sessions


IF (KDFITYP < 6) THEN
!                              *********************************
!        2.1.1 DFI session:    * BACKWARD:      IDFI=====1     *
!                              *********************************

  IDFI=1
  CALL DFI3(LDIAB,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
   & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  
  IF (.NOT.LDIAB) THEN
    CALL COPSP(SPA3,SPD2D,ZUVAVG,ZCSPA3,ZCSPD2D,ZCSPA1)
    IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
  ENDIF

!                              *********************************
!*        2.1.2 DFI session:   * FORWARD/1:     IDFI=====2     *
!                              *********************************
  IDFI=2
  IF (LDIAB) CALL DFI3(LDIAB,KSTDFI,IDFI,LDIFH,PSTEP,LLMPHYS,LLEPHYS,&
   & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  
  IF (LDIAB.AND.LDCOR) THEN
    IF (LELAM)  CALL EINTFAC(ZUVAVG,1)
    CALL CORSP(ZCSPA3,ZCSPD2D,ZCSPA1,SPA3,SPD2D,ZUVAVG)
  ENDIF

!                              *********************************
!         2.1.3 DFI session:   * FORWARD/2:     IDFI=====3     *
!                              *********************************

  IDFI=3
  CALL DFI3(LDIAB,KSTDFI,IDFI,LDIFH,PSTEP,LLMPHYS,LLEPHYS,&
   & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  
  IF (.NOT.LDIAB .AND. ABS(PSTEP-PSTEPA) > ZEPS)&
   & CALL ABOR1('DFI2 :  DIAB. AND ADIAB. TIMESTEPS DIFFER')  

ELSEIF (KDFITYP == 6) THEN

!                              *********************************
!           2.1.4 DFI session: * FORWARD  :     IDFI=====4     *
!                              *********************************

  LSCRINI= LOBSC1
  IF (LSCRINI .AND. ABS(ISTOP*ZTSTEP - 2*KSTDFI*PSTEP) >= ZEPL)&
   & CALL ABOR1('DFI2 : SPANS FOR FINALISATION AND SCREENING DIFFER')  

  IDFI=4
  CALL DFI3(LDIAB,KSTDFI,IDFI,LDIFH,PSTEP,LLMPHYS,LLEPHYS,&
   & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

ELSEIF (KDFITYP == 7) THEN

!                              *********************************
!        2.1.5 DFI session:    * BACKWARD+FORWARD : IDFI=====6 *
!                              *********************************

  IDFI=6
  CALL DFI3(.FALSE.,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
   & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  
  CALL COPSP(SPA3,SPD2D,ZUVAVG,ZDSPA3,ZDSPD2D,ZDSPA1)
  IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
  WRITE (NULOUT,*) 'SET ACCUMULATED VARIABLES TO ZERO',KDFITYP
  CALL ZEROACU(ZDSPA3,ZDSPD2D,ZDSPA1)
  CALL DFI3( LDIAB ,KSTDFI ,IDFI,LDIFH,PSTEP ,LLMPHYS,LLEPHYS,&
   & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  
ENDIF

!*       3. Reset original constants and surface fields
!           -------------------------------------------

IF (KDFITYP == 6) THEN
  IDFI=998
ELSE
  IDFI=999
ENDIF
CALL REAST(IDFI,KSTDFI,ININDAT,INSSSSS,IDEC)
IF (LREASUR) THEN
  IF (IDFI == 999) THEN
    CALL SUGRIDF(0,SPOR,LLSPOR)
  ELSE
    CALL REDATE
    CALL SUGRIDF(8,SPOR,LLSPOR)
  ENDIF
ENDIF
LMPHYS = LLMPHYS
LEPHYS = LLEPHYS
TSTEP  = ZTSTEP
NSTAR2 = ISTAR2
NSTOP2 = ISTOP2
NSTOP  = ISTOP

!*       4. Compute final spectral fields
!           -----------------------------

IF ((LDCOR.AND.LDIAB) .OR. (LBIAS.AND.IDFI == 999)) THEN
  CALL DIFSP(SPA3,SPD2D,ZUVAVG,ZDSPA3,ZDSPD2D,ZDSPA1,&
   & ZCSPA3,ZCSPD2D,ZCSPA1)  
  IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
ELSEIF (LBIAS) THEN
  CALL SUSPEC(SPA3,SPA2,ZSPA1,KFILE=8)
  IF (LELAM) THEN
    DO JL=1,NFLEVL
      ZUVAVG(JL,1)=ZSPUB(JL)
      ZUVAVG(JL,2)=ZSPVB(JL)
    ENDDO
  ENDIF
  CALL CORSP(SPA3,SPD2D,ZUVAVG,ZDSPA3,ZDSPD2D,ZDSPA1)
  IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
ELSEIF (LINCR) THEN
  CALL SUSPEC(SPA3,SPA2,ZSPA1,KFILE=3)
  IF (LELAM) THEN
    DO JL=1,NFLEVL
      ZUVAVG(JL,1)=ZSPUB(JL)
      ZUVAVG(JL,2)=ZSPVB(JL)
    ENDDO
  ENDIF
  CALL CORSP(SPA3,SPD2D,ZUVAVG,ZDSPA3,ZDSPD2D,ZDSPA1)
  IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
ELSE
  CALL COPSP(SPA3,SPD2D,ZUVAVG,ZDSPA3,ZDSPD2D,ZDSPA1)
  IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
ENDIF

IF (LSPRT) THEN
  IF (LELAM) THEN
    CALL ESPECRT(0)
  ELSE
    CALL SPECRT(0)
  ENDIF
ENDIF

IF (LBIAS .OR. IDFI /= 999) THEN
  NSTOP =0
  NSTOP2=0
  NSTAR2=0
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DFI2',1,ZHOOK_HANDLE)
END SUBROUTINE DFI2
