SUBROUTINE DFI2MOD(KSTDFI,PSTEP,KSTDFIA,PSTEPA,LDIFH)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVL   ,NFD2D    ,NFLSUR   ,NSPEC2   ,NS3D
USE YOMCT0   , ONLY : NSTOP    ,LELAM
USE YOMCT2   , ONLY : NSTAR2   ,NSTOP2
USE YOMDYN   , ONLY : TSTEP
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMSP    , ONLY : SPA3     ,SPD2D    ,SPOR
USE YOMPHY   , ONLY : LMPHYS   ,LREASUR
USE YOEPHY   , ONLY : LEPHYS

!**** *DFI2MOD*  - Controls integration for digital filter initialization

!     Purpose.
!     --------
!     CONTROLS  MODIFIED DIGITAL FILTER INITIALIZATION ON LEVEL 2

!**   Interface.
!     ----------
!        *CALL* *DFI2MOD(KSTDFI,PSTEP,KSTDFIA,PSTEPA)*

!        Explicit arguments :
!        --------------------
!        KSTDFI : number of integration steps in one direction
!        PSTEP  : time step for dfi
!        KSTDFIA: number of adiab. integration steps in one direction
!        PSTEPA : adiabatic time step for dfi
!        LDIFH  : controls horizontal diffusion for adiabatic integrations

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------

!     Externals.
!     ----------
!       DFI3   : deepest level DFI control routine
!       ZEROACU: sets accumulation arrays zero
!       COPSP  : duplicates spectral state vectors
!       DIFSP  : calculates difference of two sets of fields
!       CORSP  : calculates difference of two sets of fields
!       REAST  : resets astronomical constants after each integration session
!       SUGRIDF : initialize gridpoint fields
!       EINTFAC : interface for SPA1 (specific ALADIN-glob. array)

!     Reference.
!     ----------
!     ARPEGE/ALADIN DOCUMENTATION

!     Author.
!     -------
!       Gabor Radnoti GMAP/MICECO

!     Modifications.
!     --------------
!        Original : 92-12-24
!        D. Giard : 93-07-28 EINIBACK, EDIACORR, EINIFIE replaced by
!                            COPSP and DIFSP ; PSTEP as argument ;
!                            horizontal diffusion handled with CDCONF(9)
!        D. Giard : 95-02-23 adiabatic timestep
!        L.Isaksen: 95-06-06 reordering of spectral arrays
!        D. Giard : 97-02-10 call to CORSP
!        D. Giard : 98-01-29 LDIFH as argument
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDFI
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDFIA
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTEPA
LOGICAL           ,INTENT(IN)    :: LDIFH
INTEGER(KIND=JPIM) :: IDEC, IDFI, ININDAT, INSSSSS, ISTAR2, ISTOP, ISTOP2, IFILE

LOGICAL :: LLDIAB, LLEPHYS, LLMPHYS, LLSPOR

REAL(KIND=JPRB) :: ZTSTEP


#include "copsp.intfb.h"
#include "corsp.intfb.h"
#include "dfi3.intfb.h"
#include "difsp.intfb.h"
#include "eintfac.intfb.h"
#include "reast.intfb.h"
#include "zeroacu.intfb.h"
#include "sugridf.intfb.h"

REAL(KIND=JPRB) :: ZDSPA3(NFLSUR,NSPEC2,NS3D),ZDSPD2D(NSPEC2,NFD2D)
REAL(KIND=JPRB) :: ZDSPA1(NFLEVL,2)
REAL(KIND=JPRB) :: ZBSPA3(NFLSUR,NSPEC2,NS3D),ZBSPD2D(NSPEC2,NFD2D)
REAL(KIND=JPRB) :: ZBSPA1(NFLEVL,2)
REAL(KIND=JPRB) :: ZASPA3(NFLSUR,NSPEC2,NS3D),ZASPD2D(NSPEC2,NFD2D)
REAL(KIND=JPRB) :: ZASPA1(NFLEVL,2)
REAL(KIND=JPRB) :: ZCSPA3(NFLSUR,NSPEC2,NS3D),ZCSPD2D(NSPEC2,NFD2D)
REAL(KIND=JPRB) :: ZCSPA1(NFLEVL,2),ZUVAVG(NFLEVL,2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*       0.    Initialize local variables and arrays
!              -------------------------------------
IF (LHOOK) CALL DR_HOOK('DFI2MOD',0,ZHOOK_HANDLE)
CALL ZEROACU(ZDSPA3,ZDSPD2D,ZDSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,1)
CALL COPSP(ZCSPA3,ZCSPD2D,ZCSPA1,SPA3,SPD2D,ZUVAVG)

!*       1. save constants to local ones before overwriting
!           -----------------------------------------------

LLMPHYS = LMPHYS
LLEPHYS = LEPHYS
ZTSTEP  = TSTEP
ISTAR2  = NSTAR2
ISTOP2  = NSTOP2
ISTOP   = NSTOP
NSTOP2  = 999
NSTOP   = 999
ININDAT = NINDAT
INSSSSS = NSSSSS

!* Launching different DFI sessions

!                           *********************************
!        2. DFI session:    * BACKWARD ADIABATIC            *
!                           *********************************

IDFI=1
LLDIAB=.FALSE.
CALL DFI3(LLDIAB,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

IF (LELAM)  CALL EINTFAC(ZUVAVG,1)
CALL COPSP(ZASPA3,ZASPD2D,ZASPA1,SPA3,SPD2D,ZUVAVG)
CALL COPSP(SPA3,SPD2D,ZUVAVG,ZCSPA3,ZCSPD2D,ZCSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)

!                           *********************************
!        3. DFI session:    * FORWARD  ADIABATIC            *
!                           *********************************
IDFI=3
LLDIAB=.FALSE.
CALL DFI3(LLDIAB,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

CALL COPSP(ZCSPA3,ZCSPD2D,ZCSPA1,ZDSPA3,ZDSPD2D,ZDSPA1)
CALL COPSP(SPA3,SPD2D,ZUVAVG,ZASPA3,ZASPD2D,ZASPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
CALL ZEROACU(ZDSPA3,ZDSPD2D,ZDSPA1)

!                           *********************************
!*       4. DFI session:    * FORWARD/1:     DIABATIC       *
!                           *********************************
IDFI=5
LLDIAB=.FALSE.
CALL DFI3(LLDIAB,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  
IDFI=2
LLDIAB=.TRUE.
CALL DFI3(LLDIAB,KSTDFI ,IDFI,LDIFH,PSTEP ,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

IF (LELAM)  CALL EINTFAC(ZUVAVG,1)
CALL COPSP(ZBSPA3,ZBSPD2D,ZBSPA1,SPA3,SPD2D,ZUVAVG)

!                           *********************************
!        5. DFI session:    *    FORWARD/2:  DIABATIC       *
!                           *********************************
IDFI=3
LLDIAB=.TRUE.
CALL DFI3(LLDIAB,KSTDFI ,IDFI,LDIFH,PSTEP ,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

CALL COPSP(ZASPA3,ZASPD2D,ZASPA1,ZDSPA3,ZDSPD2D,ZDSPA1)
CALL COPSP(SPA3,SPD2D,ZUVAVG,ZBSPA3,ZBSPD2D,ZBSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)
CALL ZEROACU(ZDSPA3,ZDSPD2D,ZDSPA1)

!                           *********************************
!        6. DFI session:    * BACKWARD ADIABATIC            *
!                           *********************************

IDFI=1
LLDIAB=.FALSE.
CALL DFI3(LLDIAB,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

CALL COPSP(SPA3,SPD2D,ZUVAVG,ZBSPA3,ZBSPD2D,ZBSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)

!                           *********************************
!        7. DFI session:    * FORWARD  ADIABATIC            *
!                           *********************************
IDFI=3
LLDIAB=.FALSE.
CALL DFI3(LLDIAB,KSTDFIA,IDFI,LDIFH,PSTEPA,LLMPHYS,LLEPHYS,&
 & ININDAT,INSSSSS,ZDSPA3,ZDSPD2D,ZDSPA1)  

CALL COPSP(SPA3,SPD2D,ZUVAVG,ZDSPA3,ZDSPD2D,ZDSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)

!*       8. Correction
!           ----------

IF (LELAM)  CALL EINTFAC(ZUVAVG,1)
CALL CORSP(ZCSPA3,ZCSPD2D,ZCSPA1,SPA3,SPD2D,ZUVAVG)
CALL DIFSP(SPA3,SPD2D,ZUVAVG,ZASPA3,ZASPD2D,ZASPA1,ZCSPA3,ZCSPD2D,ZCSPA1)
IF (LELAM)  CALL EINTFAC(ZUVAVG,-1)

!*       9. Reset original constants
!           ------------------------

LMPHYS = LLMPHYS
LEPHYS = LLEPHYS
TSTEP  = ZTSTEP
NSTAR2 = ISTAR2
NSTOP2 = ISTOP2
NSTOP  = ISTOP
IDFI=999
CALL REAST(IDFI,KSTDFI,ININDAT,INSSSSS,IDEC)
IF (LREASUR) THEN
  LLSPOR =.FALSE.
  IFILE=0
  CALL SUGRIDF(IFILE,SPOR,LLSPOR)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DFI2MOD',1,ZHOOK_HANDLE)
END SUBROUTINE DFI2MOD
