SUBROUTINE DFI3(LDIAB,KDIGLST,KDFI,LDIFH,PTSTEP,LDMPHYS,LDEPHYS,&
 & KNINDAT,KNSSSSS,PSPA3,PSPA2,PSPA1)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOEPHY   , ONLY : LEPHYS
USE YOMCT0   , ONLY : LNHDYN   ,LTWOTL   ,LELAM    ,LOBSC1   ,NSTOP
USE YOMCT2   , ONLY : NSTAR2   ,NSTOP2
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG, NFD2D    ,NFLSUR   ,NSPEC2   , &
 &                    NS3D  
USE YOMDYN   , ONLY : TSTEP    ,TDT      ,NSITER   ,LSTRHD   ,LSIDG    , &
 &                    NCURRENT_ITER
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMGEM   , ONLY : NGPTOT
USE YOMINI   , ONLY : LSCRINI
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMPHY   , ONLY : LMPHYS
USE YOM_YGFL , ONLY : YGFL
USE YOMSP    , ONLY : SPA3, SPD2D, SPA1
USE YOMJCDFI , ONLY : RACCSPA3,RACCSPA2,RACCSPA1

!**** *DFI3*  - Controls integration for digital filter initialization
!                   on lowest level

!     Purpose.
!     --------
!     CONTROLS  DIGITAL FILTER INITIALIZATION ON LEVEL 3 (LOWEST LEVEL)

!**   Interface.
!     ----------
!        *CALL* *DFI3(...)*

!        Explicit arguments :
!        --------------------
!        LDIAB: controls if filter is diabatic
!        KDIGLST: number of integration steps in one direction
!        KDFI   : indicates type of integration session (backw.,
!                   forw./1 , forw./2 , forw./1+2 )
!        LDIFH: controls horizontal diffusion backwards
!        PSTEP  : time-step
!        LDMPHYS,LDEPHYS: original values of physics switches
!        KNINDAT,KNSSSSS: original starting model time (date,second)
!        PSPA3...: accumulation arrays of spectral state variables

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!     ARPEGE/ALADIN DOCUMENTATION

!     Author.
!     -------
!       Gabor Radnoti GMAP/MICECO

! Modifications
! -------------
!   Original : 92-12-24
!   D. Giard : 93-07-28 horizontal diffusion handled with CLCONF(9)
!   D. Giard : 93-09-17 correction of diabatic integrations
!   D. Giard : 94-01-24 : zonal diagnostics
!   K. YESSAD (APRIL 1994): unified horizontal diffusion scheme.
!   V. Ivanovici, D. Giard : 95-01-19 backward+forward filtering
!   Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!   K. YESSAD (NOV 1995): removal of HDC horizontal diffusion scheme.
!   M.Hamrud : 95-10-20 Changed CLCONF
!   M. TOLSTYKH (OCT 1996) Fourier space horizontal diffusion
!   D.Dent   : 97-10-03 replace SECOND by USER_CLOCK
!   D. Giard : 98-01-29 : non-hydrostatic
!   D. Giard : 98-01-29 : horizontal diffusion for adiabatic time-steps
!   D. Giard : 98-08-27 : modifications for finalisation
!   C.Temperton : 99-08-16 : removed redundant RKROMA
!   M.Hamrud : 99-07-25 : Remove Matsuno
!   D. Giard : 00-07-07 : predictor-corrector scheme; update
!   D. Giard : 00-07-11 : screening for finalisation
!   A. Bogatchev : 02-03-29 : Introducing of LPC_XXXX keys for PC NH
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!      Dec-2003 K. Yessad  cleaning in horizontal diffusion.
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR)
!   K. Yessad     08-Feb-2005 SI scheme for ARPEGE NH model
!   Y. Bouteloup 1-April-2006 : Store and restore middle time value of GFLGP (not filtered)
!   Y.Tremolet    27-Nov-2008 Jc-DFI for long windows
!   F. Vana  13-Jan-2009 : Dummy update for SUHDU
!   A.Bogatchev&C.Fischer : 09-06-2009 restore accumulation in DF
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDIAB
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIGLST
INTEGER(KIND=JPIM),INTENT(INOUT) :: KDFI
LOGICAL           ,INTENT(IN)    :: LDIFH 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTEP 
LOGICAL           ,INTENT(IN)    :: LDMPHYS 
LOGICAL           ,INTENT(IN)    :: LDEPHYS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNINDAT
INTEGER(KIND=JPIM),INTENT(IN)    :: KNSSSSS
REAL(KIND=JPRB), INTENT(OUT)     :: PSPA3(NFLSUR,NSPEC2,NS3D)
REAL(KIND=JPRB), INTENT(OUT)     :: PSPA2(NSPEC2,NFD2D)
REAL(KIND=JPRB), INTENT(OUT)     :: PSPA1(NFLEVL,2)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IACTIM, IDEC, ISIGN, ISTART, ISTEP, ISTOP
INTEGER(KIND=JPIM) :: JSITER, JST, JSTEP, IGPP, ISUB

LOGICAL :: LLWRITE, LLOBSC1, LLSLOT

REAL(KIND=JPRB) :: ZT1, ZT2
REAL(KIND=JPRB), ALLOCATABLE :: ZGFLGP(:,:)    !  Buffer to store midle value of grib point GFL
CHARACTER :: CLCONF*9, CLDAYF*39, CLTIMEOD*10, CL1*1, CL4*1, CL6*1, CL9*1
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"
#include "chkobtim.intfb.h"
#include "digfil.intfb.h"
#include "digp.intfb.h"
#include "elsrw.intfb.h"
#include "reast.intfb.h"
#include "obsv.intfb.h"
#include "stepo.intfb.h"
#include "suhdu.intfb.h"
#include "suheg.intfb.h"
#include "sunhsi.intfb.h"
#include "updobs.intfb.h"
#include "updtim.intfb.h"
#include "sualldfi.intfb.h"
#include "dealldfi.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DFI3',0,ZHOOK_HANDLE)

!* 0.  Allocate ZGFLGP array
!      ---------------------------------

IGPP = YGFL%NUMGPFLDS*NFLEVG
ALLOCATE(ZGFLGP(NGPTOT,IGPP))

!* 1.  Set up for different DFI sessions
!      ---------------------------------

CLCONF(1:1)='0'
CLCONF(2:2)='A'
CLCONF(3:3)='A'
CLCONF(4:4)='A'
CLCONF(5:5)='0'
CLCONF(6:6)='0'
CLCONF(7:7)='A'
CLCONF(8:8)='A'
! CLCONF(9:9)='A' if horizontal diffusion required, 'I' else

LLWRITE=.FALSE.
LLOBSC1=.FALSE.
IACTIM=1

!                              *********************************
!        1.1   DFI session:    *        BACKWARD               *
!                              *********************************

IF (KDFI == 1) THEN
  ISTART= 0
  TSTEP=-PTSTEP
  LMPHYS=.FALSE.
  LEPHYS=.FALSE.
  CLCONF(9:9)='I'
  IF (LDIAB) THEN
    ISTOP= KDIGLST-1
  ELSE
    ISTOP= KDIGLST
  ENDIF
!                              *********************************
!        1.2   DFI session:    *          FORWARD/1:           *
!                              *********************************
ELSEIF (KDFI == 2) THEN
  ISTART= 0
  TSTEP=PTSTEP
  LMPHYS=LDMPHYS
  LEPHYS=LDEPHYS
  CLCONF(9:9)='A'
  IF (LDIAB) THEN
    ISTOP= KDIGLST-1
  ELSE
    ISTOP= KDIGLST
  ENDIF
!                              *********************************
!        1.3   DFI session:    *        FORWARD/2:             *
!                              *********************************
ELSEIF (KDFI == 3) THEN
  ISTART= 0
  ISTOP= KDIGLST
  TSTEP=PTSTEP
  IF (LDIAB) THEN
    LMPHYS=LDMPHYS
    LEPHYS=LDEPHYS
    CLCONF(9:9)='A'
  ELSE
    LMPHYS=.FALSE.
    LEPHYS=.FALSE.
    CLCONF(9:9)='I'
  ENDIF
!                              *********************************
!        1.4   DFI session:    *        FORWARD/1 & 2:         *
!                              *********************************
ELSEIF (KDFI == 4) THEN
  ISTART= 0
  ISTOP= 2*KDIGLST
  TSTEP=PTSTEP
  IF (LDIAB) THEN
    LMPHYS=LDMPHYS
    LEPHYS=LDEPHYS
    CLCONF(9:9)='A'
  ELSE
    LMPHYS=.FALSE.
    LEPHYS=.FALSE.
    CLCONF(9:9)='I'
  ENDIF
  LLWRITE=.TRUE.
  LLOBSC1=LOBSC1 .AND. LSCRINI
!                              *********************************
!        1.5   DFI session:    *   ONE STEP FORWARD FOR MDDFI  *
!                              *********************************
ELSEIF (KDFI == 5) THEN
  ISTART= 0
  ISTOP= 0
  TSTEP=PTSTEP
  LMPHYS=.FALSE.
  LEPHYS=.FALSE.
  CLCONF(9:9)='I'
!                              *********************************
!        1.6   DFI session:    *   BACKWARD AND FORWARD        *
!                              *********************************
ELSEIF (KDFI == 6) THEN
  ISTART=0
  ISTOP=2*KDIGLST
  IF (LDIAB) THEN
    TSTEP=PTSTEP
    LMPHYS=LDMPHYS
    LEPHYS=LDEPHYS
    ISIGN=1
    CLCONF(9:9)='A'
  ELSE
    TSTEP=-PTSTEP
    LMPHYS=.FALSE.
    LEPHYS=.FALSE.
    ISIGN=-1
    CLCONF(9:9)='I'
  ENDIF

ELSE
  CALL ABOR1(' UNKNOWN OPTION FOR DFI3')
ENDIF
IF (LDIFH) CLCONF(9:9)='A'

CL1=CLCONF(1:1)
CL4=CLCONF(4:4)
CL6=CLCONF(6:6)
CL9=CLCONF(9:9)

CALL REAST(KDFI,KDIGLST,KNINDAT,KNSSSSS,IDEC)

!-----------------------------------------------------------------

!* 2.  Temporal loop
!      -------------

NSTAR2=ISTART
NSTOP2=ISTOP
IF (LLOBSC1) NSTOP =ISTOP

DO JSTEP=ISTART,ISTOP

  CALL USER_CLOCK(PTOTAL_CP=ZT1)

!        2.1  Filter accumulation if necessary

  JST = JSTEP
  ISUB=1
  IF (KDFI == 1)                                 JST=-JSTEP
  IF ((KDFI == 2.AND.LDIAB).OR.(KDFI == 4).OR.(KDFI == 6))JST=JSTEP-KDIGLST
  IF ((KDFI == 2.AND..NOT.LDIAB).OR.(KDFI == 5)) JST=JSTEP-KDIGLST-1
  IF ( (KDFI == 1.AND..NOT.LDIAB) .OR.&
   & (KDFI == 2.AND.(LDIAB.OR.JSTEP > 0)) .OR.&
   & (KDFI == 3.AND.(LDIAB.OR.JSTEP > 0)) .OR.&
   & (KDFI == 4) .OR. (KDFI == 6) ) THEN
   CALL SUALLDFI(NULOUT)
   CALL DIGFIL(JST,KDIGLST,ISUB,SPA3,SPD2D,SPA1,LDIAB)  
   PSPA3=PSPA3 + RACCSPA3(:,:,:,1)
   PSPA2=PSPA2 + RACCSPA2(:,:,1)
   PSPA1=PSPA1 + RACCSPA1
   CALL DEALLDFI(NULOUT)
  ENDIF

!    Store GFLGP values at middle time 
   IF (JSTEP == ISTOP/2) then
      CALL DIGP(1,IGPP,ZGFLGP)
   ENDIF   

!        2.2  Coupling

  IF (LELAM) THEN
    IF (KDFI == 4) THEN
      NSTEP=JSTEP
    ELSEIF (KDFI == 6) THEN
      NSTEP=ISIGN*JSTEP+MAX(0,ISIGN)*IDEC
    ELSE
      NSTEP=JST
    ENDIF
    NSTAR2=0
    CALL ELSRW(.TRUE.)
    NSTAR2=ISTART
  ENDIF

!        2.3  Current value of the time step length

  IF (JSTEP == 0.OR. LTWOTL) THEN
    TDT=TSTEP
  ELSE
    TDT=2.0_JPRB*TSTEP
  ENDIF
  CALL UPDTIM(JSTEP,TDT,TSTEP,.FALSE.)

!        2.4  Reset semi-implicit solver in the multilevel model
!             Reset horizontal diffusion

  IF (LELAM) THEN
    ! * SI scheme
    IF (LNHDYN) THEN
      CALL SUNHSI(NULOUT,.FALSE.)
    ENDIF
  ELSE
    ! * SI scheme
    IF (LNHDYN) THEN
      CALL SUNHSI(NULOUT,.FALSE.)
    ELSE
      IF (LSIDG) CALL SUHEG
    ENDIF
    ! * Horizontal diffusion scheme
    IF (CL9 == 'A') THEN
      IF (LSTRHD) CALL SUHDU
    ENDIF
  ENDIF

!        2.5  Time step

!       Reset YOMCT3
  NSTEP=JSTEP

! Write fields at half the time-span for finalisation
  IF (LLWRITE.AND.NSTEP == IDEC) THEN
    CLCONF(1:1)='A'
  ELSE
    CLCONF(1:1)=CL1
  ENDIF

! Prepare screening
  IF (LLOBSC1) THEN
    LLSLOT=.FALSE.
    CALL CHKOBTIM(LLSLOT,IACTIM)
    IF(LLSLOT) THEN
      CLCONF(6:6)='V'
    ELSE
      CLCONF(6:6)=CL6
    ENDIF
  ENDIF

! Time-step
  IF (NSITER > 0) THEN

    ! Non-hydrostatic or RUBC or predictor-corrector time-step

    NCURRENT_ITER=0
    CLCONF(9:9)='T'
    CALL STEPO(CLCONF)
    CLCONF(1:1)=CL1
    CLCONF(6:6)=CL6

    DO JSITER=1,NSITER-1
      NCURRENT_ITER=JSITER
      CLCONF(4:4)='S'
      CLCONF(9:9)='S'
      CALL STEPO(CLCONF)
    ENDDO

    NCURRENT_ITER=NSITER
    CLCONF(4:4)='S'
    CLCONF(9:9)=CL9
    CALL STEPO(CLCONF)
    CLCONF(4:4)=CL4

  ELSE
    ! Ordinary time step
    NCURRENT_ITER=0
    CALL STEPO(CLCONF)
  ENDIF

!        2.6  Printings

  IF (KDFI == 1) THEN
    ISTEP=-NSTEP
  ELSEIF (KDFI == 2) THEN
    ISTEP=NSTEP
  ELSEIF (KDFI == 3) THEN
    ISTEP=NSTEP+KDIGLST
  ELSEIF (KDFI == 4) THEN
    ISTEP=NSTEP
  ELSEIF (KDFI == 5) THEN
    ISTEP=-1
  ELSEIF (KDFI == 6) THEN
    ISTEP=ISIGN*NSTEP
  ENDIF

  CALL USER_CLOCK(PTOTAL_CP=ZT2)
  CALL DATE_AND_TIME(TIME=CLTIMEOD)
  IF ((KDFI == 1).AND.(ISTEP == 0)) THEN
    WRITE (UNIT=CLDAYF,FMT=&
     & '(1X,A,'':'',A,'':'',A,1X,''DFI STEP   -'',I1,&
     & ''/'',I3,'' +CPU='',F6.3)')&
     & CLTIMEOD(1:2),CLTIMEOD(3:4),CLTIMEOD(5:6),&
     & ISTEP,2*KDIGLST,ZT2-ZT1  
  ELSE
    WRITE (UNIT=CLDAYF,FMT=&
     & '(1X,A,'':'',A,'':'',A,1X,''DFI STEP '',SP,I4,SS,&
     & ''/'',I3,'' +CPU='',F6.3)')&
     & CLTIMEOD(1:2),CLTIMEOD(3:4),CLTIMEOD(5:6),&
     & ISTEP,2*KDIGLST,ZT2-ZT1  
  ENDIF
  WRITE(NULERR,*) CLDAYF
  WRITE (UNIT=NULOUT, FMT='(A39)') CLDAYF

ENDDO

!-----------------------------------------------------------------

!* 3.  Reset YOMCT3
!      ------------

! Last call for screening
IF (LLOBSC1) THEN
  CLCONF='0AA00V000'
  WRITE(NULOUT,'('' NSTEP ='',I6,'' OBSV    '',A9)') NSTEP,CLCONF
  CALL OBSV('DI')
  CALL UPDOBS(1)
ENDIF

!-----------------------------------------------------------------

!* 4.  Reset GFLGP to middle window values
!      -----------------------------------

CALL DIGP(-1,IGPP,ZGFLGP)
DEALLOCATE(ZGFLGP)
NSTEP=0

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DFI3',1,ZHOOK_HANDLE)
END SUBROUTINE DFI3
