SUBROUTINE DIGFILAD(KSTEP, KDIGLST, KSUB, PSPA3, PSPA2, PSPA1, LDIAB)

!     PURPOSE
!     -------
!      Adjoint of DIGFIL used to compute the gradient of Jc-DFI

!     INTERFACE
!     ---------
!        *CALL* DIGFILAD(...)*

!     EXPLICIT ARGUMENTS
!     ------------------
!        KSTEP: actual timestep
!        KDIGLST: number of integration steps in one direction
!        KSUB:
!        PSPA3...: QNORM *B_n
!        LDIAB: .TRUE. (kept)

!     IMPLICIT ARGUMENTS : none
!     ------------------

!     EXTERNALS:
!     ----------

!     REFERENCE
!     ---------
!            ARPEGE/ALADIN documentation

!     AUTHOR
!     ------
!       P. Gauthier GMAP/CNRM

!     MODIFICATIONS
!     -------------
!         Original: 1999-01-12
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Termonia : 15-Dec-2008 modifs for SSDFI + cleanings
!        Y.Tremolet    27-Nov-2008 Jc-DFI for long windows
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LELAM
USE YOMDIM   , ONLY : NFLEVL   ,NFD2D    ,NFLSUR   ,NSPEC2   ,NS3D
USE YOMDFI   , ONLY : NSTDFI   ,HWOPF    ,HWOPFA   ,HWF0     ,HWFA0
USE YOMVAR   , ONLY : LJCDFI
USE YOMJCDFI , ONLY : RACCSPA3, RACCSPA2, RACCSPA1
USE YOMMP    , ONLY : MYSETV   ,NBSETSP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIGLST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSUB
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPA3(NFLSUR,NSPEC2,NS3D) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPA2(NSPEC2,NFD2D) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPA1(NFLEVL,2)
LOGICAL           ,INTENT(IN)    :: LDIAB 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZH(NSPEC2), ZHAV
INTEGER(KIND=JPIM) :: JK, JF, J
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('DIGFILAD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

CALL GSTATS(1898,0)

IF (KSUB>SIZE(RACCSPA3,4).OR.KSTEP>KDIGLST) CALL ABOR1('DIGFILAD: Index error')

!     Actual value of filter weights

IF (LDIAB) THEN
  ZH(1:NSPEC2)=HWOPF(1:NSPEC2,KDIGLST+1-KSTEP)
  ZHAV=HWF0(KDIGLST+1-KSTEP)
ELSE
  ZH(1:NSPEC2)=HWOPFA(1:NSPEC2,KDIGLST+1-KSTEP)
  ZHAV=HWFA0(KDIGLST+1-KSTEP)
ENDIF
IF (LJCDFI.AND.KSTEP==NSTDFI) THEN
  ! ky: valid at least for non-scale dependent DFI (constant ZH).
  !     I don't know what to put there for scale dependent DFI.
  DO J=1,NSPEC2
    ZH(J)=ZH(J)-1.0_JPRB
  ENDDO
  ZHAV=ZHAV-1.0_JPRB
ENDIF

!     Add the contribution to SPA3 and SPA2

IF (LELAM) THEN
  DO JK=1,NFLEVL
    PSPA1(JK,1)=PSPA1(JK,1)+ZHAV*RACCSPA1(JK,1)
    PSPA1(JK,2)=PSPA1(JK,2)+ZHAV*RACCSPA1(JK,2)
  ENDDO
ENDIF

IF (MYSETV  ==  NBSETSP) THEN
  DO JF=1,NFD2D
    DO J=1,NSPEC2
      PSPA2(J,JF)=PSPA2(J,JF)+ZH(J)*RACCSPA2(J,JF,KSUB)
    ENDDO
  ENDDO
ENDIF

!cjfe pb omp...
DO JF=1,NS3D
!cjfe!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J,JK)
  DO J=1,NSPEC2
    DO JK=1,NFLEVL
      PSPA3(JK,J,JF)=PSPA3(JK,J,JF)+ZH(J)*RACCSPA3(JK,J,JF,KSUB)
    ENDDO
  ENDDO
!cjfe!$OMP END PARALLEL DO
ENDDO
CALL GSTATS(1898,1)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('DIGFILAD',1,ZHOOK_HANDLE)
END SUBROUTINE DIGFILAD
