SUBROUTINE SUDFI

!     PURPOSE
!     -------
!     INITIALIZE CONSTANTS FOR DFI

!     INTERFACE
!     ---------
!     *CALL* SUDFI*

!       EXPLICIT ARGUMENTS : None
!       ------------------

!       IMPLICIT ARGUMENTS : Common YOMDFI
!       ------------------

!     REFERENCE
!     ---------
!     ARPEGE/ALADIN documentation

!     AUTHOR
!     ------
!     Gabor Radnoti GMAP/MICECO

!     MODIFICATIONS
!     -------------
!     Original: 1992.12.24
!     R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!     D. Giard     : 93-07-28 SUFW called by SUDFI now; option NEDFI=0
!     D. Giard     : 93-08-24 options NEDFI=7 and 8 ; recursive filters
!     D. Giard     : 93-10-26 higher order recursive filters
!     D. Giard     : 94-01-03 date in case of forward filtering
!     D. Giard     : 94-10-18 adiabatic time-step
!                             effective delay for recursive filters
!     V. Ivanovici : 95-01-19 options for incremental init. moved to SUINI
!     D. Giard                backward-forward recursive filtering
!     D. Giard     : 96-01-18 Dolph-Chebyshev filters
!                             filter type defined by NTPDFI
!                             initialization of boundary fields suppressed
!     D. Giard     : 98-01-29 key LADIFH
!     D. Giard     : 98-08-27 Set-up through command line
!     D. Giard     : 99-07-20 Set-up through command line for 4d-var
!     D. Giard     : 99-10-22 Dolph window (from P. Gauthier)
!     D. Giard     : 00-07-11 NSTOP reset to 0 in DFI2 for finalisation
!     Y. Tremolet  : 00-03-20 Removed test on NEINI (pb if LJCDFI)
!     C. Moussy(jma) : 02-09-06 Abort with LTENC
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     B. Chapnik   : 02-May-2007 in case NCONF==131, LJCDFI==true added,
!                            otherwise possible pb in trajectory computation for FGAT.
!     K. Yessad    : 21 Feb 2007 Make variables appear in the printings.
!        Y.Tremolet    27-Nov-2008 Jc-DFI for long windows
!     P. Termonia  " 15 Dec 2008 modifs for SSDFI
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMCT0   , ONLY : NSTOP    ,NCONF    ,LELAM, LTENC
USE YOMCT0B  , ONLY : LECMWF
USE YOMINI   , ONLY : NEINI    ,LBIAS    ,LINCR
USE YOMDFI   , ONLY : HWREC    ,NTPDFI   ,&
 & NORDFI   ,NEDFI    ,NSTDFI   ,NSTDFIA  ,TAUS     ,&
 & TAUP     ,TAUC     ,RTDFI    ,RTDFIA   ,LRECURS  ,&
 & LADIFH  , NOFFSETDFI, RDFIS
USE YOMJCDFI , ONLY : LDIVONLY, NSUBDFI
USE YOMARG   , ONLY : NARGS    ,UTSTEP
USE YOMVAR   , ONLY : LJCDFI
USE YOMMODEL_ERROR, ONLY: N_COUPLED_WINDOWS

!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB), EXTERNAL :: FSPAND
INTEGER(KIND=JPIM) :: ID, J
LOGICAL :: LLD
REAL(KIND=JPRB) :: ZH, ZRI, ZSP, ZSPAN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"

!     ------------------------------------------------------------------

#include "namdfi.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDFI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!       1. SET UP DEFAULT VALUES FOR NAMDFI
!          --------------------------------

!        1.0 Check initialization type

!        1.1 Set implicit default values

ZH= 3600._JPRB
LRECURS = .FALSE.
LADIFH  = .FALSE.
NTPDFI  = 1
NORDFI  = 2
TAUS    =  3._JPRB*ZH
TAUP    = 12._JPRB*ZH
TAUC    =  6._JPRB*ZH
NEDFI   = 0
RTDFI   = 900._JPRB
RTDFIA  = RTDFI
NSTDFI  = 12
NSTDFIA = NSTDFI
NSUBDFI=1
NOFFSETDFI=0
RDFIS   = 0._JPRB

! Modifications if command line is used

IF (NARGS /= 0) THEN
  LADIFH = .TRUE.
  NTPDFI = 4
  RTDFI  = UTSTEP
  RTDFIA = UTSTEP
  IF (NCONF/100 == 0 .OR. NCONF/100 == 2 .OR. NCONF == 302) THEN
    NEDFI = 7
    IF (LELAM) THEN
      TAUS =  3._JPRB*ZH
    ELSE
      TAUS =  5._JPRB*ZH
    ENDIF
    ZRI = .05_JPRB
    LLD = .TRUE.
    ZSP = FSPAND(TAUS,ZRI,LLD,RTDFI)
    NSTDFI = NINT(ZSP/2.0_JPRB)
    IF (2.0_JPRB*NSTDFI < ZSP) NSTDFI=NSTDFI+1
  ELSEIF (NCONF/100 == 1 .AND. LJCDFI) THEN
    NEDFI = 6
    IF (MOD(NSTOP,2) == 0) THEN
      NSTDFI = NSTOP/2
    ELSE
      IF (.NOT.LECMWF) CALL ABOR1(' SUDFI: JC-DFI : ODD NUMBER OF TIMESTEPS')
    ENDIF
    ZSP = 2.0_JPRB*NSTDFI*RTDFI/ZH
    IF (NINT(ZSP)== 6) TAUS = 3.0_JPRB*ZH
    IF (NINT(ZSP)==12) TAUS = 1.5_JPRB*ZH
  ENDIF
ENDIF

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
  NEDFI =6
  NTPDFI=5
  IF (NCONF==131 .AND. N_COUPLED_WINDOWS>1) THEN
    NSTDFI = NSTOP/(2*N_COUPLED_WINDOWS)
    IF (2*N_COUPLED_WINDOWS*NSTDFI /= NSTOP) THEN
      CALL ABOR1('SUDFI: Error coupled windows')
    ENDIF
    NSUBDFI=N_COUPLED_WINDOWS
  ELSEIF (NCONF/100==0 .AND. LJCDFI) THEN  ! 12h span by default
    NSTDFI = NINT( 6.0_JPRB * 3600.0_JPRB / RTDFI )
    IF (NSTDFI>NSTOP/2) NSTDFI=NSTOP/2     ! for outer loop in 6h 4D-Var
    NSUBDFI=NSTOP/(2*NSTDFI)
  ELSE
    NSTDFI = NSTOP/2
  ENDIF
  ZSP = 2.0_JPRB*NSTDFI*RTDFI/ZH
  IF (NINT(ZSP)==12) TAUS = 1.5_JPRB*ZH
  LDIVONLY = .TRUE.
ELSE
  LDIVONLY = .FALSE.
ENDIF

!       2. READ NAMDFI
!          -----------

CALL POSNAM(NULNAM,'NAMDFI')
READ(NULNAM,NAMDFI)

!       3. CHECK NAMDFI
!          ------------

IF (RTDFI <= 0.0_JPRB) CALL ABOR1(' SUDFI: TIME-STEP FOR DFI <= 0 !')
IF (NSTDFI > 90) CALL ABOR1(' SUDFI: TOO MANY DFI STEPS')
IF (NEDFI < 0.OR.NEDFI > 7) NEDFI=0
IF (NEDFI > 0.AND.NEDFI < 6.AND.LRECURS)&
 & CALL ABOR1(' SUDFI: PB WITH OPTION AND FILTER')  
IF ((NEDFI > 2.AND.NEDFI < 6) .AND. (LBIAS.OR.LINCR))&
 & CALL ABOR1(' SUDFI: INCREMENTAL DFI NOT CODED FOR NEDFI=345')  

IF (LRECURS) THEN
  IF (TAUC <= 0.0_JPRB) CALL ABOR1(' SUDFI: CHECK CUT-OFF PERIOD !')
  IF (NTPDFI < 1.OR.NTPDFI > 3) NTPDFI=1
  IF (NORDFI < 1.OR.NORDFI > 20)&
   & CALL ABOR1(' SUDFI: INVALID ORDER FOR RECURSIVE FILTER')  
  IF (NORDFI /= 2.AND.NTPDFI == 2)&
   & CALL ABOR1(' SUDFI: ORDER 2 ONLY FOR FLAT-DELAY RECURSIVE FILTER')  
  IF (2*NSTDFI < NORDFI)&
   & CALL ABOR1(' SUDFI: NUMBER OF TIME-STEPS LOWER THAN ORDER !')  

ELSE
  IF (NTPDFI < 1.OR.NTPDFI > 5) NTPDFI=1
  IF (NTPDFI == 5) THEN
    IF (TAUC <= 0.0_JPRB) CALL ABOR1(' SUDFI: CHECK CUT-OFF PERIOD !')
    TAUS= NSTDFI*RTDFI
  ELSEIF (NTPDFI == 4) THEN
    IF (TAUS <= 0.0_JPRB) CALL ABOR1(' SUDFI: CHECK STOP-BAND PERIOD !')
    TAUC= 2.0_JPRB*TAUS
  ELSEIF (NTPDFI == 3) THEN
    IF (NSTDFI >= 64)CALL ABOR1(' SUDFI: TOO MANY DFI STEPS FOR OPTIMAL FILTER')
    IF (TAUS <= 0.0_JPRB.OR.TAUP <= 0.0_JPRB.OR.TAUP < TAUS)&
     & CALL ABOR1(' SUDFI: CHECK STOP-BAND AND PASS-BAND PERIODS !')  
    TAUC= 2.0_JPRB/((1.0_JPRB/TAUS)+(1.0_JPRB/TAUP))
  ELSE
    TAUC= 2.0_JPRB*NSTDFI*RTDFI
  ENDIF
ENDIF

ZSPAN= 2.0_JPRB*NSTDFI*RTDFI
IF (NEDFI == 1.OR.NEDFI == 6) THEN
  RTDFIA= RTDFI
  NSTDFIA= NSTDFI
ELSE
  IF (RTDFIA <= 0.0_JPRB) CALL ABOR1(' SUDFI: ADIAB. TIME-STEP FOR DFI <= 0 !')
  RTDFIA= MIN(RTDFI,RTDFIA)
  NSTDFIA= NINT(ZSPAN/RTDFIA/2.0_JPRB)
  RTDFIA= ZSPAN/REAL(2*NSTDFIA,JPRB)
  IF (NEDFI == 4.OR.NEDFI == 5.OR.NEDFI == 7) THEN
    IF (NSTDFIA > 90) CALL ABOR1(' SUDFI: TOO MANY ADIAB. DFI STEPS')
    IF (NSTDFIA >= 64.AND. (NTPDFI == 3.AND..NOT.LRECURS))&
     & CALL ABOR1(' SUDFI: TOO MANY ADIAB. DFI STEPS FOR OPTIMAL FILTER')  
  ENDIF
ENDIF

! bc. add LJCDFI so that the condition only refers to the variational part of 
! conf 131 (in FGAT a forecast is run before)
IF (NARGS /= 0 .AND. LJCDFI) THEN 
  IF (RTDFI /= UTSTEP) CALL ABOR1(' SUDFI: JC-DFI : TIMESTEPS DIFFER')
  IF (N_COUPLED_WINDOWS>1) THEN
    IF (2*N_COUPLED_WINDOWS*NSTDFI/=NSTOP) CALL ABOR1(' SUDFI: JC-DFI : SPANS DIFFER')
  ELSE
    IF (NOFFSETDFI+2*NSTDFI>NSTOP) CALL ABOR1('SUDFI: JC-DFI SPAN TOO LONG')
  ENDIF
ENDIF
!          tendancy coupling not coded for this case
IF (NEINI == 2.AND.NEDFI == 6.AND.LTENC) THEN
  CALL ABOR1(' SUDFI: bad DFI options with LTENC')
ENDIF

!       4. PRINT
!          -----

WRITE(NULOUT,'(/,'' ****COMMON /YOMDFI/****'',/)')
IF (NEDFI == 0)WRITE(NULOUT,'('' NEDFI=0: ONLY COMPUTING DFI WEIGHTS'')')
IF (NEDFI == 1)WRITE(NULOUT,'('' NEDFI=1: ADFI FOR INITIAL FIELDS'')')
IF (NEDFI == 2) &
 & WRITE(NULOUT,'('' NEDFI=2: NON-CORRECTED DDFI FOR INITIAL FIELDS'')')
IF (NEDFI == 3)WRITE(NULOUT,'('' NEDFI=3: DDFI FOR INITIAL FIELDS'')')
IF (NEDFI == 4)WRITE(NULOUT,'('' NEDFI=4: ADFI+DDFI FOR INITIAL FIELDS'')')
IF (NEDFI == 5)WRITE(NULOUT,'('' NEDFI=5: MODIFIED DDFI FOR INITIAL FIELDS'')')
IF (NEDFI == 6) THEN
  WRITE(NULOUT,'('' NEDFI=6: FORWARD DF FOR INITIAL FIELDS OR 4D-VAR'')')
  IF (LRECURS) THEN
    ID= INT(MAX(0.0_JPRB,2.0_JPRB*NSTDFI*RTDFI-HWREC(2*NORDFI+3)))
  ELSE
    ID= INT(NSTDFI*RTDFI)
  ENDIF
  WRITE(NULOUT,'('' FILTERED FIELDS VALID AT INITIAL DATE +'',I6,''s'')') ID
  IF (NCONF/100 /= 1) THEN
!   What does that mean?????
    WRITE(UNIT=NULOUT,FMT='('' NSTOP RESET TO 0, UNLESS HARDLY ANY DELAY'')')
!   What does that mean?????
  ENDIF
ENDIF
IF (NEDFI == 7) &
 & WRITE(NULOUT,'('' NEDFI=7: BACKWARD+FORWARD DFI FOR INITIAL FIELDS'')')
WRITE(NULOUT,'('' '')')

IF (NEDFI == 0.OR.NEDFI == 6) THEN
  WRITE(NULOUT,&
   & '('' NUMBER OF DFI TIMESTEPS (2*NSTDFI):'',&
   & I3,'' TIME-STEP (RTDFI):'',F7.2)') 2*NSTDFI,RTDFI  
ELSEIF (NEDFI == 1) THEN
  WRITE(NULOUT,&
   & '('' NUMBER OF DFI TIMESTEPS IN ONE DIRECTION (NSTDFI):'',&
   & I3,'' TIME-STEP (RTDFI):'',F7.2)') NSTDFI,RTDFI  
ELSEIF (NEDFI == 7) THEN
  WRITE(NULOUT,&
   & '('' NUMBER OF ADIABATIC DFI TIMESTEPS IN ONE DIRECTION (2*NSTDFIA) :'',&
   & I3,'' TIME-STEP (RTDFIA):'',F7.2)') 2*NSTDFIA,RTDFIA  
  WRITE(NULOUT,&
   & '('' NUMBER OF DIABATIC DFI TIMESTEPS IN ONE DIRECTION (2*NSTDFI) :'',&
   & I3,'' TIME-STEP (RTDFI):'',F7.2)') 2*NSTDFI,RTDFI  
ELSE
  WRITE(NULOUT,&
   & '('' NUMBER OF DFI TIMESTEPS IN ONE DIRECTION (ADIABATIC) :'',&
   & I3,'' TIME-STEP:'',F7.2)') NSTDFIA,RTDFIA  
  WRITE(NULOUT,&
   & '('' NUMBER OF DFI TIMESTEPS IN ONE DIRECTION ( DIABATIC) :'',&
   & I3,'' TIME-STEP:'',F7.2)') NSTDFI,RTDFI  
ENDIF
IF (NEDFI /= 0 .AND. NEDFI /= 6 .AND. LADIFH)&
 & WRITE(NULOUT,'('' HORIZONTAL DIFFUSION FOR ADIAB. TIMESTEPS'')')  
WRITE(NULOUT,'('' '')')

IF (LRECURS) THEN
  WRITE(NULOUT,'('' Case LRECURS=T:'')')
  IF (NTPDFI == 1) WRITE(NULOUT,&
   & '('' CASE NTPDFI=1: A QUICK-START RECURSIVE FILTER OF ORDER NORDFI='',I2,&
   & '' IS USED'')') NORDFI  
  IF (NTPDFI == 2) WRITE(NULOUT,&
   & '('' CASE NTPDFI=2: A FLAT-DELAY RECURSIVE FILTER OF ORDER NORDFI='',I2,&
   & '' IS USED'')') NORDFI  
  IF (NTPDFI == 3) WRITE(NULOUT,&
   & '('' CASE NTPDFI=3: A BUTTERWORTH RECURSIVE FILTER OF ORDER NORDFI='',I2,&
   & '' IS USED'')') NORDFI  
  WRITE(NULOUT,&
   & '(''   CUT-OFF PERIOD (TAUC/3600)='',F7.3,''h'',&
   & ''   DELAY (th.)='',F7.3,''h'',&
   & ''   DELAY (eff.)='',F7.3,''h'',&
   & ''   TIME-SPAN (2*NSTDFI*RTDFI/3600)='',F7.3,''h'')')&
   & TAUC/ZH,HWREC(2*NORDFI+2)/ZH,HWREC(2*NORDFI+3)/ZH,ZSPAN/ZH  
  WRITE(NULOUT,'('' BASIC DFI WEIGHTS (HWREC):'')')
  WRITE(NULOUT,'(10E11.3E2)') (HWREC(J+1),J=0,NORDFI)
  WRITE(NULOUT,'(10E11.3E2)') (HWREC(J+NORDFI+1),J=1,NORDFI)

ELSEIF (NTPDFI == 5) THEN
  WRITE(NULOUT,'('' Case LRECURS=F, NTPDFI=5:'')')
  WRITE(NULOUT,'('' A DOLPH-CHEBYSHEV WINDOW IS USED'')')
  WRITE(NULOUT,&
   & '(''   CUT-OFF PERIOD (TAUC/3600)='',F7.3,''h'',&
   & ''   TIME-SPAN (2*NSTDFI*RTDFI/3600)='',F7.3,''h'')')&
   & TAUC/ZH,ZSPAN/ZH  

ELSEIF (NTPDFI == 4) THEN
  WRITE(NULOUT,'('' Case LRECURS=F, NTPDFI=4:'')')
  WRITE(NULOUT,'('' A DOLPH-CHEBYSHEV FILTER IS USED'')')
  WRITE(NULOUT,&
   & '(''   STOP-BAND EDGE (TAUS/3600)='',F7.3,''h'',&
   & ''   TIME-SPAN (2*NSTDFI*RTDFI/3600)='',F7.3,''h'')')&
   & TAUS/ZH,ZSPAN/ZH  
  ZRI = .05_JPRB
  LLD = NEDFI == 7
  ZSP = FSPAND(TAUS,ZRI,LLD,RTDFI)
  WRITE(NULOUT,&
   & '(''   MINIMUM TIME-SPAN (r=.05) ='',F7.3,''h'')') ZSP*RTDFI/ZH  

ELSEIF (NTPDFI == 3) THEN
  WRITE(NULOUT,'('' Case LRECURS=F, NTPDFI=3:'')')
  WRITE(NULOUT,'('' AN OPTIMAL FILTER IS USED'')')
  WRITE(NULOUT,&
   & '(''   STOP-BAND EDGE (TAUS/3600)='',F7.3,''h'',&
   & ''   PASS-BAND EDGE (TAUP/3600)='',F7.3,''h'',&
   & ''   CUT-OFF PERIOD (TAUC/3600)='',F7.3,''h'',&
   & ''   TIME-SPAN (2*NSTDFI*RTDFI/3600)='',F7.3,''h'')')&
   & TAUS/ZH,TAUP/ZH,TAUC/ZH,ZSPAN/ZH  

ELSE
  WRITE(NULOUT,'('' Case LRECURS=F, NTPDFI=1 or 2:'')')
  WRITE(NULOUT,'('' AN IDEAL LOW-PASS FILTER IS USED'')')
  WRITE(NULOUT,'(''   CUT-OFF PERIOD (TAUC/3600)='',F7.3,''h'')') TAUC/ZH
  IF (NTPDFI == 2) WRITE(NULOUT,'('' NTPDFI=2: LANCZOS WINDOW IS USED'')')
ENDIF
WRITE(NULOUT,'('' '')')
WRITE(NULOUT,'('' RDFIS = '',/,(10E11.3E2))') RDFIS
WRITE(NULOUT,*)'SUDFI: NSTOP,NSTDFI,NSUBDFI=',NSTOP,NSTDFI,NSUBDFI

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDFI',1,ZHOOK_HANDLE)
END SUBROUTINE SUDFI
