SUBROUTINE SUINI

!**** *SUINI*  - INITIALIZE COMMON YEMINI

!     PURPOSE
!     -------
!      INITIALIZE CONTROL VARIABLES FOR INITIALIZATION

!**   INTERFACE
!     ---------
!        *CALL* *SUINI*

!     EXTERNALS:
!     ----------
!                SUDFI - INITIALIZE DIGITAL FILTERING VARIABLES
!                SUENMI - INITIALIZE NMI VARIABLES FOR ALADIN (to be coded !)

!     EXPLICIT ARGUMENTS : None
!     ------------------

!     IMPLICIT ARGUMENTS : Common YEMINI, YOMCT0, YOMNMIA, YOMARG, YOMLUN
!     ------------------

!     REFERENCE
!     ---------
!     ARPEGE/ALADIN documentation

!     AUTHOR
!     ------
!     Gabor Radnoti GMAP/MICECO

!     MODIFICATIONS
!     -------------
!      Original: 1992.12.24
!      R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!      R. El Khatib : 93-05-06 Set-up through command line
!      F.Rabier     : 94-01-13 ECMWF var. defaults, incremental 3d/4d var.
!      and J-N Thepaut          ( moved from SUNMI by D. Giard )
!      D. Giard     : 94-11-02 control of NEINICL / NEINI
!      V. Ivanovici : 95-01-19 switches for incremental initialization :
!      and D. Giard            LBIAS , LINCR
!      D. Giard     : 96-01-18 initialization for boundary fields suppressed
!      D. Giard     : 97-06-27 call to SUENMI suppressed
!      D. Giard     : 98-09-16 change set-up through command line (no nmi)
!      D. Giard     : 99-07-19 LINITER moved from YOMNMIA to YOMINI
!      D. Giard     : 00-07-11 NSTOP reset to 0 in DFI2 for bias
!                              screening during digital filter finalisation
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      P.Termonia   : 08-12-22 move the call to SUDFI to SU0YOMB (for SSDFI)
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LFPART2  ,NCONF    ,LOBSC1   ,LELAM
USE YOMCT0B  , ONLY : LECMWF
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMINI   , ONLY : NEINI    ,LBIAS    ,LINCR    ,LINITER  ,LSCRINI

IMPLICIT NONE

#include "namini.h"

!*      1. SET UP DEFAULT VALUES FOR YEMINI
!          --------------------------------

!        1.1 Set implicit default values

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
IF (LHOOK) CALL DR_HOOK('SUINI',0,ZHOOK_HANDLE)
NEINI=0
LBIAS=.FALSE.
LINCR=.FALSE.
LINITER=.FALSE.
LSCRINI=.FALSE.

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
  IF ((NCONF == 1.OR.NCONF == 302).AND.LOBSC1) THEN
    NEINI=0
  ENDIF
ELSE
ENDIF

!*      2. READ NAMINI
!          -----------

CALL POSNAM(NULNAM,'NAMINI')
READ(NULNAM,NAMINI)

IF (LFPART2) NEINI=0
IF (NEINI < 0.OR.NEINI > 4)CALL ABOR1(' SUINI : ILLEGAL VALUE FOR NEINI !')
IF (NEINI /= 2.AND.NEINI /= 4) THEN
  LBIAS= .FALSE.
  LINCR= .FALSE.
ENDIF
IF (LBIAS) THEN
  LINCR= .FALSE.
ENDIF

!*      3. PRINT FINAL VALUES AND WARNING
!          -------------------------------

WRITE(UNIT=NULOUT,FMT='(/,'' ****COMMON /YEMINI/****'')')
WRITE(UNIT=NULOUT,FMT='('' NEINI = '',I1)') NEINI

IF (NEINI == 0)&
 & WRITE(UNIT=NULOUT,FMT='('' INITIAL FIELDS ARE NOT INITIALIZED'',&
 & '' IN ARPEGE/ALADIN'')')  
IF (NEINI == 1)&
 & WRITE(UNIT=NULOUT,FMT='('' INITIAL FIELDS ARE INITIALIZED'',&
 & '' BY NMI IN ARPEGE/ALADIN'')')  
IF (NEINI == 2)&
 & WRITE(UNIT=NULOUT,FMT='('' INITIAL FIELDS ARE INITIALIZED'',&
 & '' BY DFI IN ARPEGE/ALADIN'')')  
IF (NEINI == 3)&
 & WRITE(UNIT=NULOUT,FMT='('' INITIAL FIELDS ARE NOT INITIALIZED'',&
 & '' BUT NORMAL MODES ARE COMPUTED'')')  
IF (NEINI == 4)&
 & WRITE(UNIT=NULOUT,FMT='('' INITIAL FIELDS ARE INITIALIZED'',&
 & '' BY DFI AND NORMAL MODES ARE COMPUTED'')')  
IF (LBIAS)&
 & WRITE(UNIT=NULOUT,FMT='('' COMPUTING INITIALIZATION INCREMENT'',&
 & '' ; NSTOP RESET TO 0 !'')')  
IF (LINCR)WRITE(UNIT=NULOUT,FMT='('' INCREMENTAL INITIALIZATION'')')

IF (LHOOK) CALL DR_HOOK('SUINI',1,ZHOOK_HANDLE)
END SUBROUTINE SUINI
