SUBROUTINE CPANGM(KPROMA,KFLEV,KSTART,KEND,&
 & PU,PV,PGEMU,PGSQM2,PSINLO,PCOSLO,&
 & PRES,PDELP,PGAW)  

!**** *CPANGM* - COMPUTE ANGULAR MOMENTUMS  

!     Purpose. COMPUTE THE EFFECTIVE ANGULAR MOMENTUM OF THE
!     -------- EARTH FOR EACH LATITUDE.

!**   Interface.
!     ----------
!        *CALL* *CPANGM(...)

!        Explicit arguments :
!        --------------------

!       KPROMA                 - HORIZONTAL DIMENSION                 (INPUT)
!       KFLEV                  - NUMBER OF MODEL LEVELS               (INPUT)
!       KGL1,KGL2              - KGL1 TO KGL2 LATITUDES OF WORK.      (INPUT)
!       PU                     - U-WIND                               (INPUT)
!       PV                     - V-WIND                               (INPUT)
!       PDELP(KPROMA,0:KFLEV+1)- PRESSURE DIFFERENCE ACROSS LAYERS.   (INPUT)

!        Implicit arguments :   NONE.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None.
!        Called by POSTPHY.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-08-15
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPROMA
USE YOMCST   , ONLY : RPI
USE YOMANGM  , ONLY : RXPS     ,RX3PS    ,RXUV     ,RX3UV    ,&
 & X1APNH   ,X2APNH   ,X3APNH   ,X1BPNH   ,X2BPNH   ,&
 & X3BPNH   ,X1APSH   ,X2APSH   ,X3APSH   ,X1BPSH   ,&
 & X2BPSH   ,X3BPSH   ,XSP  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSLO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KPROMA,0:KFLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGAW(KPROMA) 
!     INPUT:
REAL(KIND=JPRB) :: ZX1APNHA(NPROMA)
REAL(KIND=JPRB) :: ZX2APNHA(NPROMA)
REAL(KIND=JPRB) :: ZX3APNHA(NPROMA)
REAL(KIND=JPRB) :: ZX1APSHA(NPROMA)
REAL(KIND=JPRB) :: ZX2APSHA(NPROMA)
REAL(KIND=JPRB) :: ZX3APSHA(NPROMA)
REAL(KIND=JPRB) :: ZXSPA   (NPROMA)
REAL(KIND=JPRB) :: ZX1BPNHA(NPROMA)
REAL(KIND=JPRB) :: ZX2BPNHA(NPROMA)
REAL(KIND=JPRB) :: ZX3BPNHA(NPROMA)
REAL(KIND=JPRB) :: ZX1BPSHA(NPROMA)
REAL(KIND=JPRB) :: ZX2BPSHA(NPROMA)
REAL(KIND=JPRB) :: ZX3BPSHA(NPROMA)

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZX1APNH, ZX1APSH, ZX1BPNH, ZX1BPSH, ZX2APNH,&
 & ZX2APSH, ZX2BPNH, ZX2BPSH, ZX3APNH, ZX3APSH, &
 & ZX3BPNH, ZX3BPSH, ZXSP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*      1.  COMPUTES ANGULAR MOMENTUMS.

!*       1.1   2-D PARTS OF ANGULAR MOMENTUM

IF (LHOOK) CALL DR_HOOK('CPANGM',0,ZHOOK_HANDLE)
ZX1APNHA(:) = 0.0_JPRB
ZX2APNHA(:) = 0.0_JPRB
ZX3APNHA(:) = 0.0_JPRB
ZX1APSHA(:) = 0.0_JPRB
ZX2APSHA(:) = 0.0_JPRB
ZX3APSHA(:) = 0.0_JPRB

DO JROF=KSTART,KEND
  IF(PGEMU(JROF) >= 0) THEN
    ZX1APNHA(JROF) = PCOSLO(JROF)*PRES(JROF,KFLEV)*&
     & PGEMU(JROF)*PGSQM2(JROF)*RXPS*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
    ZX2APNHA(JROF) = PSINLO(JROF)*PRES(JROF,KFLEV)*&
     & PGEMU(JROF)*PGSQM2(JROF)*RXPS*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
    ZX3APNHA(JROF) = PRES(JROF,KFLEV)*&
     & PGSQM2(JROF)**2*RX3PS*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
  ELSE
    ZX1APSHA(JROF) = PCOSLO(JROF)*PRES(JROF,KFLEV)*&
     & PGEMU(JROF)*PGSQM2(JROF)*RXPS*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
    ZX2APSHA(JROF) = PSINLO(JROF)*PRES(JROF,KFLEV)*&
     & PGEMU(JROF)*PGSQM2(JROF)*RXPS*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
    ZX3APSHA(JROF) = PRES(JROF,KFLEV)*&
     & PGSQM2(JROF)**2*RX3PS*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
  ENDIF
  ZXSPA(JROF) = PRES(JROF,KFLEV)*PGAW(JROF)
ENDDO

ZX1APNH = SUM(ZX1APNHA(KSTART:KEND))
ZX2APNH = SUM(ZX2APNHA(KSTART:KEND))
ZX3APNH = SUM(ZX3APNHA(KSTART:KEND))
ZX1APSH = SUM(ZX1APSHA(KSTART:KEND))
ZX2APSH = SUM(ZX2APSHA(KSTART:KEND))
ZX3APSH = SUM(ZX3APSHA(KSTART:KEND))
ZXSP    = SUM(ZXSPA   (KSTART:KEND))

!*       1.2   3-D PARTS OF ANGULAR MOMENTUM

ZX1BPNHA(:) = 0.0_JPRB
ZX2BPNHA(:) = 0.0_JPRB
ZX3BPNHA(:) = 0.0_JPRB
ZX1BPSHA(:) = 0.0_JPRB
ZX2BPSHA(:) = 0.0_JPRB
ZX3BPSHA(:) = 0.0_JPRB

DO JLEV=1,KFLEV
  DO JROF=KSTART,KEND
    IF(PGEMU(JROF) >= 0) THEN
      ZX1BPNHA(JROF) = ZX1BPNHA(JROF)+PDELP(JROF,JLEV)*&
       & (PU(JROF,JLEV)*PGEMU(JROF)*PCOSLO(JROF)-&
       & PV(JROF,JLEV)*PSINLO(JROF))*RXUV*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
      ZX2BPNHA(JROF) = ZX2BPNHA(JROF)+PDELP(JROF,JLEV)*&
       & (PU(JROF,JLEV)*PGEMU(JROF)*PSINLO(JROF)+&
       & PV(JROF,JLEV)*PCOSLO(JROF))*RXUV*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
      ZX3BPNHA(JROF) = ZX3BPNHA(JROF)+&
       & PDELP(JROF,JLEV)*PU(JROF,JLEV)*PGSQM2(JROF)*&
       & RX3UV*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
    ELSE
      ZX1BPSHA(JROF) = ZX1BPSHA(JROF)+PDELP(JROF,JLEV)*&
       & (PU(JROF,JLEV)*PGEMU(JROF)*PCOSLO(JROF)-&
       & PV(JROF,JLEV)*PSINLO(JROF))*RXUV*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
      ZX2BPSHA(JROF) = ZX2BPSHA(JROF)+PDELP(JROF,JLEV)*&
       & (PU(JROF,JLEV)*PGEMU(JROF)*PSINLO(JROF)+&
       & PV(JROF,JLEV)*PCOSLO(JROF))*RXUV*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
      ZX3BPSHA(JROF) = ZX3BPSHA(JROF)+&
       & PDELP(JROF,JLEV)*PU(JROF,JLEV)*PGSQM2(JROF )*&
       & RX3UV*2.0_JPRB*RPI*2.0_JPRB*PGAW(JROF)  
    ENDIF
  ENDDO
ENDDO

ZX1BPNH = SUM(ZX1BPNHA(KSTART:KEND))
ZX2BPNH = SUM(ZX2BPNHA(KSTART:KEND))
ZX3BPNH = SUM(ZX3BPNHA(KSTART:KEND))
ZX1BPSH = SUM(ZX1BPSHA(KSTART:KEND))
ZX2BPSH = SUM(ZX2BPSHA(KSTART:KEND))
ZX3BPSH = SUM(ZX3BPSHA(KSTART:KEND))

!*      1.3  UPDATE GLOBAL VALUES

!$OMP CRITICAL
X1APNH = X1APNH+ZX1APNH
X2APNH = X2APNH+ZX2APNH
X3APNH = X3APNH+ZX3APNH
X1APSH = X1APSH+ZX1APSH
X2APSH = X2APSH+ZX2APSH
X3APSH = X3APSH+ZX3APSH
XSP    = XSP+ZXSP
X1BPNH = X1BPNH+ZX1BPNH
X2BPNH = X2BPNH+ZX2BPNH
X3BPNH = X3BPNH+ZX3BPNH
X1BPSH = X1BPSH+ZX1BPSH
X2BPSH = X2BPSH+ZX2BPSH
X3BPSH = X3BPSH+ZX3BPSH
!$OMP END CRITICAL
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPANGM',1,ZHOOK_HANDLE)
END SUBROUTINE CPANGM
