SUBROUTINE CPCFU(KST,KND,KSTC,KENDC,KSTGLO,LDFSTEP,&
 & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFCCQL, PFCCQN, PFCSQL, PFCSQN, PFPLCL,&
 & PFPLCN, PFPLCG, PFPLCH, PFPLSL, PFPLSN, PFPLSG, PFPLSH, PFRMH,  PFRSO ,&
 & PFRSOC, PFRTH , PFRTHC, PFDIS , PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU,&
 & PSTRTV, PDELP , PNEB  , PQICE , PQLI  , PRES  , PSNS  , PO3   , PQ    ,&
 & PWS   , PFEVL , PFEVN , PCLCT , PCLCH , PCLCM , PCLCL , PCLCC , PFCLL ,&
 & PFCLN , PFCS  , PFONTE, PFCHSP, PFLWSP, PRUISS, PRUISP, PFEVV , PFTR  ,&
 & PRUISL, PFGEL , PFGELS, PFRSODS,PFRSOPS,PFRSOPT,PFRSOLU,PFRTHDS)

!**** *CPCFU* - CUMUL OF FLUX DIAGNOSTICS.

!     Purpose.
!     --------
!           DIAGNOSTICS OF PHYSICAL FLUXES.

!**   Interface.
!     ----------
!        *CALL* *CPCFU*

!        Explicit arguments :
!        --------------------

!       KPROMA                 - HORIZONTAL DIMENSION                 (INPUT)
!       KGL1,KGL2              - KGL1 TO KGL2 LATITUDES OF WORK.      (INPUT)
!       KSTABUF                - KSTABUF(JGL) FOR LATITUDE JGL        (INPUT)
!       FLUXES COMING FROM THE PHYSICAL PARAMETERIZATIONS             (INPUT)

!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        O. CLARY

!     Modifications.
!     --------------
!        Original : 06/92 D'APRES CUMCO
!        MODIFIED BY C. DREVETON 17/06/92
!        MODIFIED BY C. DREVETON 29/03/93 (Liquid and solid humidity)
!        MODIFIED BY E. BAZILE   24/03/94 (Flux de condensation)
!        MODIFIED BY E. BAZILE   11/02/94 (Surface Fluxes)
!        MODIFIED BY R. EL KHATIB 18/08/93 (Partial cloudiness)
!        MODIFIED BY R. EL KHATIB 07/03/94 (LREACFU)
!        MODIFIED BY M. DEQUE 27/06/96 (New test for clear sky)
!        MODIFIED BY JPh.PIEDELIEVRE 07/03/97 (Distributed Memory)
!        MODIFIED BY E. BAZILE 12/02/99 (Flux de gel)
!        MODIFIED BY J.M. PIRIOU 19/04/99 (Moon radiation)
!      R. El Khatib : 00-03-22 Indexes of convection
!        Modified : 00-03 by R. El Khatib - derived types + filtered duration of total precip.
!        Modified : 01-08-07 R. El Khatib - pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        Y.Seity       15-Jan-2007 add graupel and hail 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMDYN   , ONLY : TSTEP
USE YOMCFU   , ONLY : NFDCFU   ,NTYPCFU  ,TYPE_CFU ,LREACFU  ,NFRRC
USE YOMCST   , ONLY : RG
USE YOMCT3   , ONLY : NSTEP
USE YOMGFUB  , ONLY : GFUBUF
USE PTRGFU   , ONLY : MSTRDU   ,MSTRDV   ,MSTRCU   ,MSTRCV   ,MFDICQ   ,&
 & MFDICS   ,MSTRTU   ,MSTRTV   ,MFDITQ   ,MFDITS   ,MFPLCL   ,&
 & MFPLSL   ,MFPLCN   ,MFPLSN   ,MFPLCG   ,MFPLSG   ,MFPLCH   ,&
 & MFPLSH   ,MFCCQL   ,MFCCQN   ,MFCSQL   ,&
 & MFCSQN   ,MFRSO    ,MFRTH    ,MFNEB    ,MWS      ,MSNS     ,&
 & MQTOT    ,MFNEBT   ,MOZONT   ,MSPRES   ,MFCL     ,MFCS     ,&
 & MFDISH   ,MFTOPH   ,MQICE    ,MQLI     ,MFRSOC0  ,MFRTHC0  ,&
 & MFRSOPS  ,MFRSOPT  ,MFRSOLU  ,MFRSODS  ,MFRTHDS  ,MFRTHC1  ,&
 & MFRSOC1  ,MFCLL    ,MFCLN    ,MFEVL    ,MFEVN    ,MFONTE   ,&
 & MFCHSP   ,MFEVV    ,MFLWSP   ,MFTR     ,MRUISL   ,MRUISP   ,&
 & MRUISS   ,MNEBCON  ,MNEBHAU  ,MNEBMOY  , &
 & MNEBBAS  ,MFGEL    ,MFGELS   ,MFDUTP  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KENDC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDFSTEP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCG(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCH(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSG(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSH(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRMH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOC(NPROMA,0:1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTHC(NPROMA,0:1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFDIS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEB(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQICE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVN(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCC(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLN(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFONTE(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCHSP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLWSP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVV(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTR(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGELS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSODS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOPS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOPT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOLU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTHDS(NPROMA) 
!     local array

REAL(KIND=JPRB) :: ZGFU(NPROMA,NFDCFU)
REAL(KIND=JPRB) :: ZFPREC9(NPROMA) ! past cumulated flux of total precipitations
REAL(KIND=JPRB) :: ZFPREC0(NPROMA) ! instantaneous flux of total precipitations
REAL(KIND=JPRB) :: ZSQPRE(NPROMA)  ! instantaneous flux of total squared precipitations

REAL(KIND=JPRB) :: ZEPS

INTEGER(KIND=JPIM) :: IJLEV, JLEV, JNIV, JF, IFLD, IOFF, ILEVB, JI, IADD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"

!     ------------------------------------------------------------------

!*    1.    PREPARATIONS

!*    1.1   READ IN FIELDS

IF (LHOOK) CALL DR_HOOK('CPCFU',0,ZHOOK_HANDLE)
IF (.NOT.LDFSTEP .OR. LREACFU) THEN
  CALL SC2RDG(GFUBUF,KSTGLO,KSTC,KENDC,ZGFU)
ELSE
  ZGFU=0.0_JPRB
ENDIF

!*    1.2  SAVE TOTAL PRECIPITATIONS CUMULATED FLUX

IF (MFDUTP > 0) THEN
  ZEPS=EPSILON(1.0_JPRB)*100._JPRB
  ZFPREC9(KST:KND)=0.0_JPRB
  ZFPREC0(KST:KND)=0.0_JPRB
  ZSQPRE(KST:KND)=0.0_JPRB
  IOFF=0
  DO JF=1,NTYPCFU

    IADD=1+IOFF
!   Last level only, in order to get the total precipitations :
    ILEVB=TYPE_CFU(JF)%ILEV
    IFLD=ILEVB+IOFF
    IJLEV=TYPE_CFU(JF)%INUMLEV(ILEVB)

    IF (IADD == MFPLCL) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLCL(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLCN) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLCN(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLCG) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLCG(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLCH) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLCH(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLSL) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLSL(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLSN) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLSN(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLSG) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLSG(KST:KND,IJLEV)
    ELSEIF (IADD == MFPLSH) THEN
      ZFPREC9(KST:KND)=ZFPREC9(KST:KND)+ZGFU(KST:KND,IFLD)
      ZFPREC0(KST:KND)=ZFPREC0(KST:KND)+PFPLSH(KST:KND,IJLEV)
    ENDIF

    IOFF=IOFF+TYPE_CFU(JF)%ILEV
  ENDDO
  ZSQPRE(KST:KND)=(ZFPREC0(KST:KND)**2)*TSTEP
  ZFPREC0(KST:KND)=ZFPREC0(KST:KND)*TSTEP
ENDIF

!     ------------------------------------------------------------------

!*    2.    ACCUMULATE FLUXES.
!           ------------------

IOFF=0
DO JF=1,NTYPCFU
  IADD=1+IOFF
  DO JLEV=1,TYPE_CFU(JF)%ILEV

    IFLD=JLEV+IOFF
    IJLEV=TYPE_CFU(JF)%INUMLEV(JLEV)

    IF (IADD == MSTRDU) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSTRDU(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MSTRDV) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSTRDV(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MSTRCU) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSTRCU(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MSTRCV) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSTRCV(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFDICQ) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PDIFCQ(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFDICS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PDIFCS(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MSTRTU) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSTRTU(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MSTRTV) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSTRTV(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFDITQ) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PDIFTQ(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFDITS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PDIFTS(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLCL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLCL(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFCCQL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCCQL(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLCN) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLCN(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLCG) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLCG(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLCH) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLCH(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFCCQN) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCCQN(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLSL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLSL(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFCSQL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCSQL(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLSN) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLSN(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLSG) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLSG(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFPLSH) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFPLSH(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFCSQN) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCSQN(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFRSO) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSO(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFRTH) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRTH(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFNEB) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PNEB(KST:KND,IJLEV)*TSTEP
    ELSEIF (IADD == MFNEBT) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PCLCT(KST:KND)*TSTEP
    ELSEIF (IADD == MFCLL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCLL(KST:KND)*TSTEP
    ELSEIF (IADD == MFCLN) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCLN(KST:KND)*TSTEP
    ELSEIF (IADD == MFEVL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFEVL(KST:KND)*TSTEP
    ELSEIF (IADD == MFEVN) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFEVN(KST:KND)*TSTEP
    ELSEIF (IADD == MFONTE) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFONTE(KST:KND)*TSTEP
    ELSEIF (IADD == MFCHSP) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCHSP(KST:KND)*TSTEP
    ELSEIF (IADD == MFLWSP) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFLWSP(KST:KND)*TSTEP
    ELSEIF (IADD == MRUISS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PRUISS(KST:KND)*TSTEP
    ELSEIF (IADD == MRUISP) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PRUISP(KST:KND)*TSTEP
    ELSEIF (IADD == MFEVV) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFEVV(KST:KND)*TSTEP
    ELSEIF (IADD == MFTR) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFTR(KST:KND)*TSTEP
    ELSEIF (IADD == MRUISL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PRUISL(KST:KND)*TSTEP
    ELSEIF (IADD == MFGEL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFGEL(KST:KND)*TSTEP
    ELSEIF (IADD == MFGELS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFGELS(KST:KND)*TSTEP
    ELSEIF (IADD == MFCS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFCS(KST:KND)*TSTEP
    ELSEIF (IADD == MWS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PWS(KST:KND)*TSTEP
    ELSEIF (IADD == MSNS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PSNS(KST:KND)*TSTEP
    ELSEIF (IADD == MSPRES) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PRES(KST:KND)*TSTEP
    ELSEIF (IADD == MFCL) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+&
       & (PFCLL(KST:KND)+PFCLN(KST:KND))*TSTEP  
    ELSEIF (IADD == MFDISH) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFDIS(KST:KND)*TSTEP
    ELSEIF (IADD == MFTOPH) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRMH(KST:KND)*TSTEP
    ELSEIF (IADD == MQTOT) THEN
      DO JNIV=1,NFLEVG
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PQ(KST:KND,JNIV)&
         & *PDELP(KST:KND,JNIV)/RG*TSTEP  
      ENDDO
    ELSEIF (IADD == MQICE) THEN
      DO JNIV=1,NFLEVG
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PQICE(KST:KND,JNIV)&
         & *PDELP(KST:KND,JNIV)/RG*TSTEP  
      ENDDO
    ELSEIF (IADD == MQLI) THEN
      DO JNIV=1,NFLEVG
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PQLI(KST:KND,JNIV)&
         & *PDELP(KST:KND,JNIV)/RG*TSTEP  
      ENDDO
    ELSEIF (IADD == MOZONT) THEN
      DO JNIV=1,NFLEVG
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PO3(KST:KND,JNIV)&
         & *PDELP(KST:KND,JNIV)/RG*TSTEP  
      ENDDO
    ELSEIF (IADD == MFRSODS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSODS(KST:KND)*TSTEP
    ELSEIF (IADD == MFRSOPS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSOPS(KST:KND)*TSTEP
    ELSEIF (IADD == MFRSOPT) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSOPT(KST:KND)*TSTEP
    ELSEIF (IADD == MFRTHDS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRTHDS(KST:KND)*TSTEP
    ELSEIF (IADD == MFRSOLU) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSOLU(KST:KND)*TSTEP
    ELSEIF (IADD == MNEBCON) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PCLCC(KST:KND)*TSTEP
    ELSEIF (IADD == MNEBHAU) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PCLCH(KST:KND)*TSTEP
    ELSEIF (IADD == MNEBMOY) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PCLCM(KST:KND)*TSTEP
    ELSEIF (IADD == MNEBBAS) THEN
      ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PCLCL(KST:KND)*TSTEP
    ELSEIF (IADD == MFRSOC0) THEN
      IF (MOD(NSTEP,NFRRC) == 0) THEN
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSOC(KST:KND,0)*TSTEP*NFRRC
      ENDIF
    ELSEIF (IADD == MFRSOC1) THEN
      IF (MOD(NSTEP,NFRRC) == 0) THEN
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRSOC(KST:KND,1)*TSTEP*NFRRC
      ENDIF
    ELSEIF (IADD == MFRTHC0) THEN
      IF (MOD(NSTEP,NFRRC) == 0) THEN
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRTHC(KST:KND,0)*TSTEP*NFRRC
      ENDIF
    ELSEIF (IADD == MFRTHC1) THEN
      IF (MOD(NSTEP,NFRRC) == 0) THEN
        ZGFU(KST:KND,IFLD)=ZGFU(KST:KND,IFLD)+PFRTHC(KST:KND,1)*TSTEP*NFRRC
      ENDIF
    ELSEIF (IADD == MFDUTP) THEN
      DO JI=KST,KND
        IF (ZFPREC9(JI) > ZEPS) THEN
          ZGFU(JI,IFLD)=ZGFU(JI,IFLD)*((ZFPREC9(JI)+ZFPREC0(JI))**2)&
           & /((ZFPREC9(JI)**2)+(ZGFU(JI,IFLD)*ZSQPRE(JI)))  
        ELSEIF (ZFPREC0(JI) > ZEPS) THEN
          ZGFU(JI,IFLD)=TSTEP
        ELSE
          ZGFU(JI,IFLD)=0.0_JPRB
        ENDIF
      ENDDO
    ELSE
      CALL ABOR1('CPCFU : ERROR IN POINTERS') 
    ENDIF

  ENDDO
  IOFF=IOFF+TYPE_CFU(JF)%ILEV
ENDDO

!     ------------------------------------------------------------------

!*       3.    WRITE OUT ACC. FIELDS .
!              -----------------------

!*    3.1   WRITE OUT FIELDS

CALL SC2WRG(GFUBUF,KSTGLO,KSTC,KENDC,ZGFU)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPCFU',1,ZHOOK_HANDLE)
END SUBROUTINE CPCFU
