SUBROUTINE CPPHDDH( &
 !     INPUT.
 & KPROMA,KPROMVC,KPROMDLR,KSTART,KPROF,KFLEV,LDDYN,&
 & LDVERCOR,PVCRSSAH,PRDLR,&
 & PNORDL, PNORDM, PUZGEO, PVMGEO,&
 & PSLAT, PCLAT, PLON,&
 & PSNS,PTP,PTS,&
 & PWP,PWS,PT0,PLSM,&
 & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFCQNG,&
 & PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH,&
 & PFRTHDS,PFHPCL, PFHPCN, PFHPSL, PFHPSN,&
 & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV,&
 & PCT,&
 & PFCHSP, PFCLL, PFCLN, PFCS, PFEVL, PFEVN, PFLWSP,&
 & PFONTE, PRUISP, PRUISS,&
 & PENTRA, PENTRV,&
 & PDIFTQL,PDIFTQN,PDIFCQL,PDIFCQN,PFCQLNG,PFCQNNG,&
 & PFHSCL,PFHSCN,PFHSSL,PFHSSN, PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
 & PFCQLF,PFCQIF,PFSQLF,PFSQIF,&
 & PFPFPL,PFPFPN,PFPFPCL,PFPFPCN,&
 & PFPEVPL,PFPEVPN,PFPEVPCL,PFPEVPCN,&
 & PFCQRNG, PFCQSNG,&
 & PHI0,PUCLS,PVCLS,PTCLS,PQCLS,PCLPH,PALB,PGZ0,PGZ0H,&
 !     OUTPUT.
 & PDHCV, PDHCS)
!---------------------------------------------------------------------------
!**** *CPPHDDH*  - CALCUL DES FLUX/TENDANCES DYNAMIQUES
!                  DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!---------------------------------------------------------------------------
!     BUT
!     ---
!      CALCULS PHYSIQUES POUR LES DIAGNOSTICS DDH
!      AINSI QUE DES VARIABLES AU SOL A L'INSTANT COURANT

!   ** SURVEILLER LE POSITIONNEMENT DES CHAMPS AU MOYEN DE L'INDICATEUR
!                           IDHCV

!     ARGUMENTS D ENTREE (INPUT)
!     --------------------------

! - DIMENSIONS
!     KPROMA : DIMENSION HORIZONTALE.
!     KPROMVC: horizontal dimension for arrays used only if "lvercor=T".
!     KPROMDLR: horizontal dimension for arrays used only if "lrwsdlr=T".
!     KSTART : DEBUT DE LA BOUCLE HORIZONTALE.
!     KPROF  : FIN DE LA BOUCLE HORIZONTALE.
!     KFLEV  : DIMENSION VERTICALE ET NB DE NIVEAUX

! - LOGIQUES DE CONTROLE
!     LDDYN  : activation des DDH pour la dynamique.
!     LDVERCOR: .T./.F.: hypothese de couche mince abandonnee/appliquee

! - TABLEAUX DE DONNEES
!     PVCRSSAH       : half level values of "rs/a".
!     PRDLR          : array containing "r/a" and "a/r".
!     PNORDL, PNORDM : DIRECTION DU NORD GEOGRAPHIQUE
!     PUZGEO, PVMGEO : VENT ZONAL ET MERIDIEN VRAIS GEOGRAPHIQUES
!                      (UNIQUEMENT SI LHDHKS OU LHDMCI OU LHDENT VRAIS)
!     PSLAT,  PCLAT  : SINUS RESP. COSINUS OF LATITUDE
!     PLON           : LONGITUDE
!     PSNS       : MASSE PAR UNITE DE SURFACE DE LA COUCHE NEIGEUSE.
!     PTP        : TEMPERATURE EN PROFONDEUR.
!     PTS        : TEMPERATURE DE SURFACE.
!     PWP        : CONTENU EN EAU DU RESERVOIR EN PROFONDEUR.
!     PWS        : CONTENU EN EAU DU RESERVOIR DE SURFACE.
!     PT0        : TEMPERATURE A INSTANT 0 / TEMPERATURE AT TIME LEVEL 0
!     PHI0       : OROGRAPHIE (HAUTEUR GEOPOTENTIELLE "gz" A L'INSTANT t SUR LES INTERCOUCHES).
!     PLSM       : INDICE TERRE/MER.
!     PUCLS      : COMPOSANTE U DU VENT A 10 METRES (DIAGNOSTIQUE).
!     PVCLS      : COMPOSANTE V DU VENT A 10 METRES (DIAGNOSTIQUE).
!     PTCLS      : TEMPERATURE A 2 METRES (DIAGNOSTIQUE).
!     PQCLS      : HUMIDITE SPECIFIQUE A 2 METRES (DIAGNOSTIQUE).
!     PCLPH      : HAUTEUR (EN METRES) DE LA COUCHE LIMITE PLANETAIRE.
!     PALB       : ALBEDO DE SURFACE COURANT.
!     PGZ0       : G*LONGUEUR DE RUGOSITE DYNAMIQUE.
!     PGZ0H      : G*LONGUEUR DE RUGOSITE THERMIQUE.
!     PDIFCQ     : FLUX CONVECTIF D'HUMIDITE SPECIFIQUE (HORS PLUIE/NEIGE).
!     PDIFCS     : FLUX CONVECTIF D'ENTHALPIE (HORS PLUIE/NEIGE).
!     PDIFTQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
!     PDIFTS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
!     PFCQNG     : FLUX FICTIF D'EAU POUR CORRIGER LES Q<0.
!     PFPLCL     : PRECIPITATIONS CONVECTIVES SOUS FORME LIQUIDE.
!     PFPLCN     : PRECIPITATIONS CONVECTIVES SOUS FORME NEIGE.
!     PFPLSL     : PRECIPITATIONS STRATIFORMES SOUS FORME LIQUIDE.
!     PFPLSN     : PRECIPITATIONS STRATIFORMES SOUS FORME NEIGE.
!     PFRSO      : FLUX DE RAYONNEMENT SOLAIRE.
!     PFRTH      : FLUX DE RAYONNEMENT THERMIQUE.
!     PFRTHDS    : FLUX DE RAYONNEMENT THERMIQUE DESCENDANT A LA SURFACE.
!     PFHPCL     : FLUX ENTHALPIE CONVECTIVES SOUS FORME LIQUIDE.
!     PFHPCN     : FLUX ENTHALPIE CONVECTIVES SOUS FORME NEIGE.
!     PFHPSL     : FLUX ENTHALPIE STRATIFORMES SOUS FORME LIQUIDE.
!     PFHPSN     : FLUX ENTHALPIE STRATIFORMES SOUS FORME NEIGE.
!     PSTRCU     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "U".
!     PSTRCV     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "V".
!     PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
!     PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".
!     PSTRTU     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "U".
!     PSTRTV     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "V".
!     PCT        : ???
!     PFCHSP     : FLUX DE CHALEUR DE LA SURFACE VERS LE SOL PROFOND.
!     PFCLL      : FLUX DE CHALEUR LATENTE SUR EAU LIQUIDE (OU SOL HUMIDE).
!     PFCLN      : FLUX DE CHALEUR LATENTE SUR NEIGE (OU GLACE).
!     PFCS       : FLUX DE CHALEUR SENSIBLE EN SURFACE.
!     PFEVL      : FLUX DE VAPEUR D'EAU SUR EAU LIQUIDE (OU SOL HUMIDE).
!     PFEVN      : FLUX DE VAPEUR D'EAU SUR NEIGE (OU GLACE).
!     PFLWSP     : FLUX D'EAU LIQUIDE DE LA SURFACE VERS LE SOL PROFOND.
!     PFONTE     : FLUX D'EAU CORRESPONDANT A LA FONTE DE NEIGE EN SURFACE.
!     PRUISP     : FLUX DE RUISSELLEMENT EN PROFONDEUR.
!     PRUISS     : FLUX DE RUISSELLEMENT EN SURFACE.
!     PENTRA     : ENTROPIE AIR SEC / SPECIFIC ENTROPY OF DRY AIR
!     PENTRV     : ENTROPIE VAPEUR D'EAU / SPECIFIC ENTROPY OF WATER VAPOUR
!     PDIFTQL    : FLUX D'EAU LIQUIDE TURBULENT.
!     PDIFTQN    : FLUX D'EAU NEIGEUSE TURBULENT.
!     PDIFCQL    : FLUX D'EAU LIQUIDE TURBULENT CONVECTIF.
!     PDIFCQN    : FLUX D'EAU NEIGEUSE TURBULENT CONVECTIF.
!     PFCQLNG    : FLUX D'EAU LIQUIDE DE CORR. DES VALEURS < 0.
!     PFCQNNG    : FLUX D'EAU NEIGEUSE DE CORR. DES VALEURS < 0.
!     PFHSCL     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS CONV. LIQ.
!     PFHSCN     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS CONV. NEI.
!     PFHSSL     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS STRA. LIQ.
!     PFHSSN     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS STRA. NEI.
!     PFEPFP     : PSEUDO FLUX D'ENTHALPIE LIE A L'ENERGIE PRECIPITATIONS.
!     PFCQLF     : FLUX DE CONDENSATION CONV. LIQ.
!     PFCQIF     : FLUX DE CONDENSATION CONV. NEI.
!     PFSQLF     : FLUX DE CONDENSATION STRA. LIQ.
!     PFSQIF     : FLUX DE CONDENSATION STRA. NEI.

!     ENTREES IMPLICITES
!     ------------------
!       UNITE NULOUT (/YOMLUN/)
!       CONSTANTES UNIVERSELLES, COMMON /YOMCST/
!       FONCTIONS THERMODYNAMIQUES, COMDECK *FCTTRM*

!       CONTROLE PHYSIQUE DE BASE : YOMPHY : NDPSFI
!       CONSTANTES PHYSIQUES (NORMALEMENT INUTILES)
!       YOMPHY0, YOMPHY1, YOMPHY2, YOMPHY3

!       ELEMENTS DE CONTROLE DDH
!       INDICATEURS LOGIQUES, COMMON /YOMLDDH/
!        DANS LE COMDECK * YOMMDDH * POUR LES MASQUES
!       DIMENSIONS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /DDHDIM/
!       ET DIMENSIONS POUR LES TABLEAUX DE CUMULS
!       INDICATEURS DE POSITIONS DANS LES TABLEAUX DE CUMUL

!     ARGUMENTS DE SORTIES (OUTPUT)
!     -----------------------------
!      DANS PDHCV, FLUX/TENDANCES PHYSIQUES INSTANTANEES
!      DANS PDHCS  VARIABLES ET FLUX AU SOL

!     REMARQUES SUPPLEMENTAIRES
!     -------------------------

!     L'abandon d'hypothese de couche mince conduit a adopter les conventions
!     suivantes pour les flux: ce qu'on stocke dans les tableaux contenant les
!     flux ce sont des quantites extensives (par exemple une masse de vapeur
!     d'eau) par unite de "pseudo-surface Delta x_a * Delta y_a", la pseudo-
!     surface etant egale a la vraie surface sur le niveau de pression ou
!     le pseudo-rayon "rs" vaut exactement le rayon moyen "a". Ce ne sont donc
!     pas les vrais flux (qui eux devraient etre calcules par rapport a la
!     vraie surface "S = Delta x Delta y = (rs/a)**2 Delta x_a * Delta y_a"),
!     mais ces pseudo-flux ont l'avantage de rester des quantites additives
!     (contrairement aux vrais flux), donc en particulier on peut prendre
!     la difference de deux pseudo-flux a deux niveaux differents pour calculer
!     la divergence verticale de flux (et donc la tendance),
!     ou additionner les pseudo-flux pour
!     calculer un flux moyen sur une grande surface. C'est cette convention
!     qui minimise les modifications a apporter dans les DDH (et aussi la
!     physique) vis a vis de l'abandon d'hypothese de couche mince.
!     Par abus de langage on appellera "flux" ces pseudo-flux.

!     Une des consequences de cette convention est que dans cette routine
!     le facteur "rs/a" ne va intervenir qu'a deux endroits:
!     - dans le calcul des flux lies au moment "MT" (car la definition de "MT"
!       fait intervenir "rs").
!     - dans le calcul de moyennes horizontales de champs-sol ou la il faut
!       ajouter la ponderation par "(rs/a)**2".

!     ORIGINAL
!     --------
!     91-03-04, Alain Joly.

!     MODIFICATIONS
!     -------------
!     92-03-01, M. Janousek: diagnostic de l'entropie.
!     92-06-01, M. Janousek: diagnostic du moment cinetique.
!     92-06-29, J.M. Piriou: changement de signe des chaleurs latentes.
!     97-02-25, J.M. Piriou: soil inertia.
!     00-08-08, K. YESSAD: relaxation of thin layer hypothesis.
!     00-08-08, K. YESSAD: cleanings (dummy arguments, correct false comments)
!     R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     2004-02-03, F. Vinit : ajout de variables et flux free-style pour Meteo-France.
!     05-06-20, F. Bouyssel: Lopez's prognostic scheme
!     06-01-25, F.Bouyssel: Add PFPEVPL,N Remove PFHMLTS,PFHEVPP,PFPEVPP,PFADVP
!     07-05-31, T.Kovacic: Added PFPEVPCL,PFPEVPCN,PFPFPCL,PFPFPCN,PFCQRNG,
!                                PFCQSNG, PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     08-12-18 O.Riviere Add new dataflow for DDH under key LFLEXDIA
!   K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!-------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMRIP   , ONLY : NSSSSS   ,RSTATI
USE YOMPHY   , ONLY : NDPSFI   ,LNEIGE   ,LCONDWT  ,LPROCLD, L3MT, LSTRAPRO
USE YOMCST   , ONLY : ROMEGA   ,RA  ,RG,  RCW, RCS ,RCPD  ,RCPV  ,RLMLT
USE YOMPHY1  , ONLY : RTINER
USE YOEPHY   , ONLY : LEPHYS
USE YOMARPHY  ,ONLY : LMSE
USE YOMLDDH  , ONLY : LHDHKS   ,LHDMCI   ,LHDENT   ,LHDLIST, LFLEXDIA
USE YOMMDDH  , ONLY : NDHCVSU  ,NDHCSSU  ,NDHVV    ,NDHFVD   ,&
 & NDHVS    ,NDHFSD   ,NDHFSP   ,NDHVFS   ,NDHAVP   ,NDHAHKP  ,&
 & NDHBHKP  ,NDHBMCP  
USE YOMSDDH  , ONLY : LHDQLN   ,LHDADD
USE YOMDYNA  , ONLY : LRWSDLR
USE PAR_RDLR , ONLY : JP_RSAH  ,JP_DIMR
USE DDH_MIX,ONLY :ADD_FIELD_3D
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMVC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMDLR
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LDDYN 
LOGICAL           ,INTENT(IN)    :: LDVERCOR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSAH(KPROMVC,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDLR(KPROMDLR,0:KFLEV,JP_DIMR)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUZGEO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVMGEO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLAT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLAT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTHDS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCHSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLWSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFONTE(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENTRA(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENTRV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEPFP(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPCQ(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQIF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFSQLF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFSQIF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPCL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPCN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPCL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPCN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQRNG(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQSNG(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLPH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0H(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDHCV(KPROMA,0:KFLEV,NDHCVSU) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDHCS(KPROMA,NDHCSSU) 

!     ------------------------------------------------------------------
REAL(KIND=JPRB) :: ZTMPAF(KPROMA,0:KFLEV) ! temp array for add_field_3D
REAL(KIND=JPRB) :: ZCLON(KPROMA), ZSLON(KPROMA)
REAL(KIND=JPRB) :: ZFPLL(KPROMA,0:KFLEV) ! ZFPLL : FLUX DE PRECIPITATIONS LIQUIDES
REAL(KIND=JPRB) :: ZFPLN(KPROMA,0:KFLEV) ! ZFPLN : FLUX DE PRECIPITATIONS SOLIDES
REAL(KIND=JPRB) :: ZFPSLM(KPROMA) ! ZFPSLM : FLUX DE PRECIP STRAT TOTAL INTERCOUCHE HAUT
REAL(KIND=JPRB) :: ZFPCLM(KPROMA) ! ZFPCLM : FLUX DE PRECIP CONV  TOTAL INTERCOUCHE HAUT
!   FLUX DE QTE DE MVT TURBULENTS DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTTUG(KPROMA,0:KFLEV), ZSTTVG(KPROMA,0:KFLEV)
!   FLUX DE QTE DE MVT CONVECTIFS DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTCUG(KPROMA,0:KFLEV), ZSTCVG(KPROMA,0:KFLEV)
!   FLUX DE QTE DE MVT ONDES OROG.DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTDUG(KPROMA,0:KFLEV), ZSTDVG(KPROMA,0:KFLEV)
!   PSEUDO-RAYON "rs" SUR LES INTERCOUCHES:
REAL(KIND=JPRB) :: ZRS(KPROMA,0:KFLEV)
!   INDICE TERRE-MER * (rs/a)**2
REAL(KIND=JPRB) :: ZLSM(KPROMA)

INTEGER(KIND=JPIM) :: IDHCS, IDHCV, IDH, JLEV, JROF

REAL(KIND=JPRB) :: ZDISKC, ZDISKD, ZDISKT, ZDLH, ZDLQ, ZDNH,&
 & ZDNQ, ZDPSFI, ZDTH, ZDTQ, ZEVALH, ZEVALQ, &
 & ZEVANH, ZEVANQ, ZFONTH, ZFONTQ, ZFORLH, ZFORLQ, &
 & ZFORNH, ZFORNQ, ZFPCL, ZFPSL, ZIBFONT, ZIBL, &
 & ZIBN, ZIBNEG, ZIBNEI, ZIBPL, ZIBPLM, ZIBT, &
 & ZNEG, ZPROPL, ZPROPN, ZRL, ZRLP, ZTOT, ZUSRG, &
 & ZEPS, ZFHSSL, ZFHSSN, ZFHSCL, ZFHSCN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPHDDH',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!**   1.- INITIALISATIONS DIVERSES

ZUSRG = 1.0_JPRB/RG
ZDPSFI = REAL(NDPSFI,JPRB)
ZEPS = 1.E-4_JPRB
IDHCV=0
IDH=0

DO JLEV = 0, KFLEV
  DO JROF=KSTART,KPROF
    ZFPLL(JROF,JLEV) = PFPLCL(JROF,JLEV) + PFPLSL(JROF,JLEV)
    ZFPLN(JROF,JLEV) = PFPLCN(JROF,JLEV) + PFPLSN(JROF,JLEV)
    ZSTTUG(JROF,JLEV) = PNORDM(JROF)*PSTRTU(JROF,JLEV)-&
     & PNORDL(JROF)*PSTRTV(JROF,JLEV)  
    ZSTTVG(JROF,JLEV) = PNORDL(JROF)*PSTRTU(JROF,JLEV)+&
     & PNORDM(JROF)*PSTRTV(JROF,JLEV)  
    ZSTCUG(JROF,JLEV) = PNORDM(JROF)*PSTRCU(JROF,JLEV)-&
     & PNORDL(JROF)*PSTRCV(JROF,JLEV)  
    ZSTCVG(JROF,JLEV) = PNORDL(JROF)*PSTRCU(JROF,JLEV)+&
     & PNORDM(JROF)*PSTRCV(JROF,JLEV)  
    ZSTDUG(JROF,JLEV) = PNORDM(JROF)*PSTRDU(JROF,JLEV)-&
     & PNORDL(JROF)*PSTRDV(JROF,JLEV)  
    ZSTDVG(JROF,JLEV) = PNORDL(JROF)*PSTRDU(JROF,JLEV)+&
     & PNORDM(JROF)*PSTRDV(JROF,JLEV)  
  ENDDO
ENDDO

IF ( LHDMCI ) THEN
  DO JROF=KSTART,KPROF
    ZCLON(JROF) = COS(PLON(JROF)+ROMEGA*(RSTATI-NSSSSS))
    ZSLON(JROF) = SIN(PLON(JROF)+ROMEGA*(RSTATI-NSSSSS))
  ENDDO
ENDIF

!     ------------------------------------------------------------------
!**   2.- CALCUL DES FLUX PHYSIQUES

IF ( LHDHKS ) THEN
  IF(LDDYN) THEN
    IDHCV=NDHVV+NDHFVD
  ELSE
    IDHCV=0
  ENDIF
  IDH=IDHCV
  
  IF(LFLEXDIA) THEN
 
    ZTMPAF(KSTART:KPROF,0:KFLEV)= &
     & ZDPSFI*(ZFPLL(KSTART:KPROF,0:KFLEV)+ZFPLN(KSTART:KPROF,0:KFLEV))
    CALL ADD_FIELD_3D(ZTMPAF,'FPPSUMFPL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDIFTQ,'FQVTUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDIFCQ,'FQVTURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFPLSL,'FQVPRECISTL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFPLSN,'FQVPRECISTN','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFPLCL,'FQVPRECICOL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFPLCN,'FQVPRECICON','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFCQNG,'FQVTURQNEGA','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(ZSTTUG,'FUUTUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(ZSTTVG,'FVVTUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(ZSTCUG,'FUUTURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(ZSTCVG,'FVVTURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(ZSTDUG,'FUUONDEGREL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(ZSTDVG,'FVVONDEGREL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDIFTS,'FCTTUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDIFCS,'FCTTURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFRSO,'FCTRAYSOL1','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PFRTH,'FCTRAYTER1','F','ARP',.TRUE.,.TRUE.)

    IF  ( L3MT.OR.LSTRAPRO )  THEN
!       This is ALARO!      
      IF (NDPSFI==1) THEN        
        ZTMPAF=PFHSSL*(1-RCPD/RCW)
        CALL ADD_FIELD_3D(ZTMPAF,'FCTPRECISTL','F','ALO',.TRUE.,.TRUE.)
        ZTMPAF=PFHSSN*(1-RCPD/RCS)
        CALL ADD_FIELD_3D(ZTMPAF,'FCTPRECISTN','F','ALO',.TRUE.,.TRUE.)
        ZTMPAF=PFHSCL*(1-RCPD/RCW)
        CALL ADD_FIELD_3D(ZTMPAF,'FCTPRECICOL','F','ALO',.TRUE.,.TRUE.)
        ZTMPAF=PFHSCN*(1-RCPD/RCS)
        CALL ADD_FIELD_3D(ZTMPAF,'FCTPRECICON','F','ALO',.TRUE.,.TRUE.)
        ZTMPAF=PFHSSL*RCPD/RCW+PFHSSN*RCPD/RCS+PFHSCL*RCPD/RCW+PFHSCN*RCPD/RCS+PFEPFP
        CALL ADD_FIELD_3D(ZTMPAF,'FCTCMP','F','ALO',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFCMPCQ,'FQVCMPC','F','ALO',.TRUE.,.TRUE.)
      ELSE
        CALL ADD_FIELD_3D(PFHSSL,'FCTPRECISTL','F','ALO',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFHSSN,'FCTPRECISTN','F','ALO',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFHSCL,'FCTPRECICOL','F','ALO',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFHSCN,'FCTPRECICON','F','ALO',.TRUE.,.TRUE.)
      ENDIF
        
      CALL ADD_FIELD_3D(PFHPCL,'FCTPRECSCOL','F','ALO',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHPCN,'FCTPRECSCON','F','ALO',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHPSL,'FCTPRECSSTL','F','ALO',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHPSN,'FCTPRECSSTN','F','ALO',.TRUE.,.TRUE.)
      
    ELSE !alaro
      CALL ADD_FIELD_3D(PFHPSL,'FCTPRECISTL','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHPSN,'FCTPRECISTN','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHPCL,'FCTPRECICOL','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHPCN,'FCTPRECICON','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHSCL,'FCTPRECSCOL','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHSCN,'FCTPRECSCON','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHSSL,'FCTPRECSSTL','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFHSSN,'FCTPRECSSTN','F','ARP',.TRUE.,.TRUE.)  
    ENDIF !alaro
      
    IF(LHDQLN) THEN
      CALL ADD_FIELD_3D(PDIFTQL,'FQLTUR','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PDIFTQN,'FQNTUR','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PDIFCQL,'FQLTURCONV','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PDIFCQN,'FQNTURCONV','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFCQLNG,'FQLTURQNEGA','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFCQNNG,'FQNTURQNEGA','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFCQLF,'FQTCONDECOL','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFCQIF,'FQTCONDECON','F','ARP',.TRUE.,.TRUE.)  
      CALL ADD_FIELD_3D(PFSQLF,'FQTCONDESTL','F','ARP',.TRUE.,.TRUE.)
      CALL ADD_FIELD_3D(PFSQIF,'FQTCONDESTN','F','ARP',.TRUE.,.TRUE.)  
      
      IF ( (LCONDWT.AND.LPROCLD).OR. & 
           & ( L3MT.OR.LSTRAPRO ) ) THEN

        CALL ADD_FIELD_3D(PFPEVPL,'FQVEVPL','F','ARP',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFPEVPN,'FQVEVPN','F','ARP',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFPFPL,'FQLFPL','F','ARP',.TRUE.,.TRUE.)
        CALL ADD_FIELD_3D(PFPFPN,'FQNFPN','F','ARP',.TRUE.,.TRUE.)

        IF ( L3MT.OR.LSTRAPRO ) THEN

          CALL ADD_FIELD_3D(PFPEVPCL,'FQRPEVC','F','ALO',.TRUE.,.TRUE.)
          CALL ADD_FIELD_3D(PFPEVPCN,'FQSPEVC','F','ALO',.TRUE.,.TRUE.)
          CALL ADD_FIELD_3D(PFPFPCL,'FQLPFPC','F','ALO',.TRUE.,.TRUE.)
          CALL ADD_FIELD_3D(PFPFPCN,'FQNPFPC','F','ALO',.TRUE.,.TRUE.)
          CALL ADD_FIELD_3D(PFCQRNG,'FQRNG','F','ALO',.TRUE.,.TRUE.)
          CALL ADD_FIELD_3D(PFCQSNG,'FQSNG','F','ALO',.TRUE.,.TRUE.)

          IF (NDPSFI==1) THEN
            CALL ADD_FIELD_3D(PFCMPSL,'FQLCMP','F','ARP',.TRUE.,.TRUE.)
            CALL ADD_FIELD_3D(PFCMPSN,'FQNCMP','F','ARP',.TRUE.,.TRUE.)
          ENDIF
        ENDIF
      ENDIF
    ENDIF


  ELSE! LFLEXDIA


    DO JLEV = 0, KFLEV
      DO JROF=KSTART,KPROF
!       FPPSUMFPL
        PDHCV(JROF,JLEV,IDH+1) = ZDPSFI*(ZFPLL(JROF,JLEV)+ZFPLN(JROF,JLEV))
!       FQVTUR
        PDHCV(JROF,JLEV,IDH+2) = PDIFTQ(JROF,JLEV)
!       FQVTURCONV
        PDHCV(JROF,JLEV,IDH+3) = PDIFCQ(JROF,JLEV)
!       FQVPRECISTL
        PDHCV(JROF,JLEV,IDH+4) = PFPLSL(JROF,JLEV)
!       FQVPRECISTN
        PDHCV(JROF,JLEV,IDH+5) = PFPLSN(JROF,JLEV)
!       FQVPRECICOL
        PDHCV(JROF,JLEV,IDH+6) = PFPLCL(JROF,JLEV)
!       FQVPRECICON
        PDHCV(JROF,JLEV,IDH+7) = PFPLCN(JROF,JLEV)
!       FQVTURQNEGAT
        PDHCV(JROF,JLEV,IDH+8) = PFCQNG(JROF,JLEV)
!       FUUTUR
!       FVVTUR
        PDHCV(JROF,JLEV,IDH+9) = ZSTTUG(JROF,JLEV)
        PDHCV(JROF,JLEV,IDH+10) = ZSTTVG(JROF,JLEV)
!       FUUTURCONV
!       FVVTURCONV
        PDHCV(JROF,JLEV,IDH+11) = ZSTCUG(JROF,JLEV)
        PDHCV(JROF,JLEV,IDH+12) = ZSTCVG(JROF,JLEV)
!       FUUONDEGREL
!       FVVONDEGREL
        PDHCV(JROF,JLEV,IDH+13) = ZSTDUG(JROF,JLEV)
        PDHCV(JROF,JLEV,IDH+14) = ZSTDVG(JROF,JLEV)
!       FCTTUR
        PDHCV(JROF,JLEV,IDH+15) = PDIFTS(JROF,JLEV)
!       FCTTURCONV
        PDHCV(JROF,JLEV,IDH+16) = PDIFCS(JROF,JLEV)
!       FCTRAYSOL1
        PDHCV(JROF,JLEV,IDH+17) = PFRSO(JROF,JLEV)
!       FCTRAYTER1
        PDHCV(JROF,JLEV,IDH+18) = PFRTH(JROF,JLEV)
      
        IF  ( L3MT.OR.LSTRAPRO )  THEN
!         This is ALARO!      
          IF (NDPSFI==1) THEN
            ZFHSSL = (RCPD/RCW)*PFHSSL(JROF,JLEV)
            ZFHSSN = (RCPD/RCS)*PFHSSN(JROF,JLEV)
            ZFHSCL = (RCPD/RCW)*PFHSCL(JROF,JLEV)
            ZFHSCN = (RCPD/RCS)*PFHSCN(JROF,JLEV)
!           FCTPRECISTL
            PDHCV(JROF,JLEV,IDH+19) = PFHSSL(JROF,JLEV)-ZFHSSL
!           FCTPRECISTN
            PDHCV(JROF,JLEV,IDH+20) = PFHSSN(JROF,JLEV)-ZFHSSN
!           FCTPRECICOL
            PDHCV(JROF,JLEV,IDH+21) = PFHSCL(JROF,JLEV)-ZFHSCL
!           FCTPRECICON
            PDHCV(JROF,JLEV,IDH+22) = PFHSCN(JROF,JLEV)-ZFHSCN
!           FCTCMP 
            IDH=IDH+1
            PDHCV(JROF,JLEV,IDH+22)=ZFHSSL+ZFHSSN+ZFHSCL+ZFHSCN+PFEPFP(JROF,JLEV)
!           FQVCMPC
            IDH=IDH+1
            PDHCV(JROF,JLEV,IDH+22) = PFCMPCQ(JROF,JLEV)
          ELSE
!           FCTPRECISTL
            PDHCV(JROF,JLEV,IDH+19) = PFHSSL(JROF,JLEV)
!           FCTPRECISTN
            PDHCV(JROF,JLEV,IDH+20) = PFHSSN(JROF,JLEV)
!           FCTPRECICOL
            PDHCV(JROF,JLEV,IDH+21) = PFHSCL(JROF,JLEV)
!           FCTPRECICON
            PDHCV(JROF,JLEV,IDH+22) = PFHSCN(JROF,JLEV)
          ENDIF
!         FCTPRECCSCOL
          PDHCV(JROF,JLEV,IDH+23) = PFHPCL(JROF,JLEV)
!         FCTPRECCSCON
          PDHCV(JROF,JLEV,IDH+24) = PFHPCN(JROF,JLEV)
!         FCTPRECCSSTL
          PDHCV(JROF,JLEV,IDH+25) = PFHPSL(JROF,JLEV)
!         FCTPRECCSSTN
          PDHCV(JROF,JLEV,IDH+26) = PFHPSN(JROF,JLEV)
        ELSE
          !     FCTPRECISTL
          PDHCV(JROF,JLEV,IDH+19) = PFHPSL(JROF,JLEV)
!         FCTPRECISTN
          PDHCV(JROF,JLEV,IDH+20) = PFHPSN(JROF,JLEV)
!         FCTPRECICOL
          PDHCV(JROF,JLEV,IDH+21) = PFHPCL(JROF,JLEV)
!         FCTPRECICON
          PDHCV(JROF,JLEV,IDH+22) = PFHPCN(JROF,JLEV)
!         FCTPRECCSCOL
          PDHCV(JROF,JLEV,IDH+23) = PFHSCL(JROF,JLEV)
!         FCTPRECCSCON
          PDHCV(JROF,JLEV,IDH+24) = PFHSCN(JROF,JLEV)
!         FCTPRECCSSTL
          PDHCV(JROF,JLEV,IDH+25) = PFHSSL(JROF,JLEV)
!         FCTPRECCSSTN
          PDHCV(JROF,JLEV,IDH+26) = PFHSSN(JROF,JLEV)
        ENDIF
      ENDDO
    ENDDO
    IF(LHDQLN) THEN
      DO JLEV = 0, KFLEV
        DO JROF=KSTART,KPROF
!             FQLTUR
          PDHCV(JROF,JLEV,IDH+27) = PDIFTQL(JROF,JLEV)
!             FQNTUR
          PDHCV(JROF,JLEV,IDH+28) = PDIFTQN(JROF,JLEV)
!             FQLTURCONV
          PDHCV(JROF,JLEV,IDH+29) = PDIFCQL(JROF,JLEV)
!             FQNTURCONV
          PDHCV(JROF,JLEV,IDH+30) = PDIFCQN(JROF,JLEV)
!             FQLTURQNEGAT
          PDHCV(JROF,JLEV,IDH+31) = PFCQLNG(JROF,JLEV)
!            FQNTURQNEGAT
          PDHCV(JROF,JLEV,IDH+32) = PFCQNNG(JROF,JLEV)
!           FQTCONDECOL
          PDHCV(JROF,JLEV,IDH+33) = PFCQLF(JROF,JLEV)
!           FQTCONDECON
          PDHCV(JROF,JLEV,IDH+34) = PFCQIF(JROF,JLEV)
!           FQTCONDESTL
          PDHCV(JROF,JLEV,IDH+35) = PFSQLF(JROF,JLEV)
!           FQTCONDESTN
          PDHCV(JROF,JLEV,IDH+36) = PFSQIF(JROF,JLEV)

          IF ( (LCONDWT.AND.LPROCLD).OR. & 
           & ( L3MT.OR.LSTRAPRO ) ) THEN
!             FPEVPL
            PDHCV(JROF,JLEV,IDH+37) = PFPEVPL(JROF,JLEV)
!             FPEVPN
            PDHCV(JROF,JLEV,IDH+38) = PFPEVPN(JROF,JLEV)
!             FPFPL
            PDHCV(JROF,JLEV,IDH+39) = PFPFPL(JROF,JLEV)
!             FPFPN
            PDHCV(JROF,JLEV,IDH+40) = PFPFPN(JROF,JLEV)
            IF ( L3MT.OR.LSTRAPRO ) THEN
!             ... , FQRPEVC
              PDHCV(JROF,JLEV,IDH+41) = PFPEVPCL (JROF,JLEV)
!             ... , FQSPEVC
              PDHCV(JROF,JLEV,IDH+42) = PFPEVPCN (JROF,JLEV)
!             ... , FQLPFPC
              PDHCV(JROF,JLEV,IDH+43) = PFPFPCL (JROF,JLEV)
!             ... , FQNPFPC
              PDHCV(JROF,JLEV,IDH+44) = PFPFPCN (JROF,JLEV)
!             ... , FQRNG
              PDHCV(JROF,JLEV,IDH+45) = PFCQRNG (JROF,JLEV)
!             ... , FQSNG
              PDHCV(JROF,JLEV,IDH+46) = PFCQSNG (JROF,JLEV)
    
              IF (NDPSFI==1) THEN
!               FQLCMP
                IDH=IDH+1
                PDHCV(JROF,JLEV,IDH+46) = PFCMPSL(JROF,JLEV)
!               FQNCMP
                IDH=IDH+1
                PDHCV(JROF,JLEV,IDH+46) = PFCMPSN(JROF,JLEV)
              ENDIF
            ENDIF
          ENDIF

        ENDDO
      ENDDO
    ENDIF
  ENDIF! LFLEXDIA
ENDIF




IF ( LHDMCI ) THEN
  IF (LDVERCOR) THEN
    ZRS(KSTART:KPROF,0:KFLEV)=RA*PVCRSSAH(KSTART:KPROF,0:KFLEV)
  ELSEIF (LRWSDLR) THEN
    ZRS(KSTART:KPROF,0:KFLEV)=RA*PRDLR(KSTART:KPROF,0:KFLEV,JP_RSAH)
  ELSE
    ZRS(KSTART:KPROF,0:KFLEV)=RA
  ENDIF
  IDHCV=NDHVV+NDHFVD+NDHAHKP
  DO JLEV = 0, KFLEV
    DO JROF=KSTART,KPROF
!     FA1TUR
!     FA2TUR
!     FA3TUR
      PDHCV(JROF,JLEV,IDHCV+1) = ZRS(JROF,JLEV)*( ZSTTVG(JROF,JLEV)*&
       & ZSLON(JROF)-&
       & ZSTTUG(JROF,JLEV)*ZCLON(JROF)*PSLAT(JROF))  
      PDHCV(JROF,JLEV,IDHCV+2) = ZRS(JROF,JLEV)*(-ZSTTVG(JROF,JLEV)*&
       & ZCLON(JROF)-&
       & ZSTTUG(JROF,JLEV)*ZSLON(JROF)*PSLAT(JROF))  
      PDHCV(JROF,JLEV,IDHCV+3) = ZRS(JROF,JLEV)*ZSTTUG(JROF,JLEV)*PCLAT(JROF)
!     FA1TURCONV
!     FA2TURCONV
!     FA3TURCONV
      PDHCV(JROF,JLEV,IDHCV+4) = ZRS(JROF,JLEV)*( ZSTCVG(JROF,JLEV)*&
       & ZSLON(JROF)-&
       & ZSTCUG(JROF,JLEV)*ZCLON(JROF)*PSLAT(JROF))  
      PDHCV(JROF,JLEV,IDHCV+5) = ZRS(JROF,JLEV)*(-ZSTCVG(JROF,JLEV)*&
       & ZCLON(JROF)-&
       & ZSTCUG(JROF,JLEV)*ZSLON(JROF)*PSLAT(JROF))  
      PDHCV(JROF,JLEV,IDHCV+6) = ZRS(JROF,JLEV)*ZSTCUG(JROF,JLEV)*PCLAT(JROF)
!     FA1ONDEGREL
!     FA2ONDEGREL
!     FA3ONDEGREL
      PDHCV(JROF,JLEV,IDHCV+7) = ZRS(JROF,JLEV)*( ZSTDVG(JROF,JLEV)*&
       & ZSLON(JROF)-&
       & ZSTDUG(JROF,JLEV)*ZCLON(JROF)*PSLAT(JROF))  
      PDHCV(JROF,JLEV,IDHCV+8) = ZRS(JROF,JLEV)*(-ZSTDVG(JROF,JLEV)*&
       & ZCLON(JROF)-&
       & ZSTDUG(JROF,JLEV)*ZSLON(JROF)*PSLAT(JROF))  
      PDHCV(JROF,JLEV,IDHCV+9) = ZRS(JROF,JLEV)*ZSTDUG(JROF,JLEV)*PCLAT(JROF)
    ENDDO
  ENDDO
  
  IF(LFLEXDIA) THEN
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+1),'FA1TUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+2),'FA2TUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+3),'FA3TUR','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+4),'FA1TURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+5),'FA2TURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+6),'FA3TURCONV','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+7),'FA1ONDEGREL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+8),'FA2ONDEGREL','F','ARP',.TRUE.,.TRUE.)
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+9),'FA3ONDEGREL','F','ARP',.TRUE.,.TRUE.)
  ENDIF

ENDIF
!     ------------------------------------------------------------------
!**   3.- CALCUL DES TENDANCES PHYSIQUES

!*         - DISSIPATION D'ENERGIE CINETIQUE.

IF ( LHDHKS ) THEN
  IF(LDDYN) THEN
    IDHCV=NDHVV+NDHFVD+NDHAVP
  ELSE
    IDHCV=NDHAVP
  ENDIF
  DO JLEV = 1, KFLEV
    DO JROF=KSTART,KPROF
!     TKKDISSIPTUR
      ZDISKT = -( PUZGEO(JROF,JLEV)*&
       & (ZSTTUG(JROF,JLEV)-ZSTTUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTTVG(JROF,JLEV)-ZSTTVG(JROF,JLEV-1)) )  
      PDHCV(JROF,JLEV,IDHCV+1) = ZDISKT
!     TKKDISSIPCONV
      ZDISKC = -( PUZGEO(JROF,JLEV)*&
       & (ZSTCUG(JROF,JLEV)-ZSTCUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTCVG(JROF,JLEV)-ZSTCVG(JROF,JLEV-1)) )  
      PDHCV(JROF,JLEV,IDHCV+2) = ZDISKC
!     TKKDISSIPGREL
      ZDISKD = -( PUZGEO(JROF,JLEV)*&
       & (ZSTDUG(JROF,JLEV)-ZSTDUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTDVG(JROF,JLEV)-ZSTDVG(JROF,JLEV-1)) )  
      PDHCV(JROF,JLEV,IDHCV+3) = ZDISKD
    ENDDO
  ENDDO
  IF(LFLEXDIA) THEN
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+1),'TKKDISSTUR','F','ARP',.TRUE.,.TRUE.)  
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+2),'TKKDISSCONV','F','ARP',.TRUE.,.TRUE.)  
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+3),'TKKDISSGREL','F','ARP',.TRUE.,.TRUE.) 
  ENDIF 

!IF LHDADD: IF
  IF(LHDADD) THEN

!*         - PRECIPITATIONS STRATIFORMES.

    ZNEG=1.E-12_JPRB
!     ZIBNEI=1. SI PRECIPITATIONS EN DEUX PHASES.
    IF(LNEIGE) THEN
      ZIBNEI=1.0_JPRB
    ELSE
      ZIBNEI=0.0_JPRB
    ENDIF
!     INITIALISATION
    DO JROF=KSTART,KPROF
      ZFPSL = PFPLSL(JROF,0) + PFPLSN(JROF,0)
!     ZIBPLM = 1. SI FLUX SIGNIFICATIF EN JLEV-1
      ZIBPLM = MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFPSL-ZNEG))
!     PROPORTION AU SOMMET
      ZFPSLM(JROF) = ZIBPLM*PFPLSL(JROF,0)/MAX(ZNEG,ZFPSL)
    ENDDO
    DO JLEV=1, KFLEV
      DO JROF=KSTART,KPROF
        ZFPSL=PFPLSL(JROF,JLEV)+PFPLSN(JROF,JLEV)
!     ZIBPL=1. SI FLUX SIGNIFICATIF EN JLEV
        ZIBPL=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFPSL-ZNEG))
!     PROPORTION A LA BASE DE LA COUCHE
        ZRLP=ZIBPL*PFPLSL(JROF,JLEV)/MAX(ZNEG,ZFPSL)
!     PROPORTIONS DU MELANGE: MOYENNE PONDEREE DE LA COUCHE (JLEV-1)
!     ET (JLEV) AVEC LES POIDS RESPECTIFS 1. ET ZIBPL
        ZRL=(ZFPSLM(JROF)+ZIBPL*ZRLP)/(1.0_JPRB+ZIBPL)
        ZRL=MAX(ZRL,1.0_JPRB-ZIBNEI)
!     ZDLQ: VARIATION DU FLUX DE PRECIPITATION LIQUIDE DANS LA COUCHE.
        ZDLQ=PFPLSL(JROF,JLEV)-PFPLSL(JROF,JLEV-1)
        ZIBL=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDLQ))
!     ZDNQ: VARIATION DU FLUX DE PRECIPITATION NEIGEUX DANS LA COUCHE.
        ZDNQ=PFPLSN(JROF,JLEV)-PFPLSN(JROF,JLEV-1)
        ZIBN=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDNQ))
!     ZDTQ: VARIATION DU FLUX DE PRECIPITATION DANS LA COUCHE: TOTAL.
        ZDTQ=ZDLQ+ZDNQ
        ZIBT=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDTQ))
!     ZDLH: VARIATION DU FLUX D'ENTHALPIE LIQUIDE DANS LA COUCHE.
        ZDLH=PFHPSL(JROF,JLEV)-PFHPSL(JROF,JLEV-1)
!     ZDNH: VARIATION DU FLUX D'ENTHALPIE NEIGEUSE DANS LA COUCHE.
        ZDNH=PFHPSN(JROF,JLEV)-PFHPSN(JROF,JLEV-1)
!     ZDTH: VARIATION DU FLUX D'ENTHALPIE DANS LA COUCHE: TOTAL.
        ZDTH=ZDLH+ZDNH
!     1. FLUX DE PRECIPITATIONS. ---------------------------------
!     FONTE
        ZFONTQ=ZIBT*(-ZIBL*(1.0_JPRB-ZIBN)*ZDNQ &
         & +(1.0_JPRB-ZIBL)*ZIBN*ZDLQ*ZIBNEI)&
         & +(1.0_JPRB-ZIBT)*((1.0_JPRB-ZRL)*ZDLQ-ZRL*ZDNQ)  
!     FORMATION LIQUIDE
        ZFORLQ=ZIBT*(ZIBL*ZIBN*ZDLQ+ZIBL*(1.0_JPRB-ZIBN)*ZDTQ)
!     EVAPORATION LIQUIDE
        ZEVALQ=(1.0_JPRB-ZIBT)*ZRL*ZDTQ
!     FORMATION NEIGEUSE
        ZFORNQ=ZIBT*ZIBNEI*(ZIBL*ZIBN*ZDNQ+(1.0_JPRB-ZIBL)*ZIBN*ZDTQ)
!     EVAPORATION NEIGEUSE
        ZEVANQ=(1.0_JPRB-ZIBT)*(1.0_JPRB-ZRL)*ZDTQ
!     2. FLUX D'ENTHALPIE. ---------------------------------------
!     ENTHALPIE DE FONTE
        ZFONTH=-RLMLT*ZFONTQ
!     AFIN D'ESTIMER LA PART DES TERMES FORMATION, EVAPORATION ET FONTE
!     DANS LE BILAN TOTAL D'ENTHALPIE DE LA COUCHE, ON DISTINGUE
!     PREMIEREMENT LA FONTE DU GEL
        ZIBFONT=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFONTQ))
!     DANS UN SECOND TEMPS, ON CALCULE LA PART DE LA FORMATION DANS LE
!     TOTAL (FORMATION+EVAPORATION)
        ZTOT=ZFORLQ+ZEVALQ
        ZIBNEG=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZTOT-ZNEG))
        ZPROPL=ZIBNEG*ZFORLQ/MAX(ZNEG,ZTOT)
        ZTOT=ZFORNQ+ZEVANQ
        ZIBNEG=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZTOT-ZNEG))
        ZPROPN=ZIBNEG*ZFORNQ/MAX(ZNEG,ZTOT)
!     ON CALCULE ENFIN LA PARTITION D'ENTHALPIE
        ZFORLH=ZPROPL     *(ZDLH+(1.0_JPRB-ZIBFONT)*ZFONTH)
        ZEVALH=(1.0_JPRB-ZPROPL)*(ZDLH+(1.0_JPRB-ZIBFONT)*ZFONTH)
        ZFORNH=ZPROPN     *(ZDNH+ZIBFONT     *ZFONTH)
        ZEVANH=(1.0_JPRB-ZPROPN)*(ZDNH+ZIBFONT     *ZFONTH)
!     3. ECRITURE DES RESULTATS. ---------------------------------
!     FORMATION DE PLUIE
!     TQVFORMPSTL
!     TCTFORMPSTL
        PDHCV(JROF,JLEV,IDHCV+ 4)=-ZFORLQ
        PDHCV(JROF,JLEV,IDHCV+ 5)=-ZFORLH
!     EVAPORATION DE PLUIE
!     TQVEVAPPSTL
!     TCTEVAPPSTL
        PDHCV(JROF,JLEV,IDHCV+ 6)=-ZEVALQ
        PDHCV(JROF,JLEV,IDHCV+ 7)=-ZEVALH
!     FORMATION DE NEIGE
!     TQVFORMPSTN
!     TCTFORMPSTN
        PDHCV(JROF,JLEV,IDHCV+ 8)=-ZFORNQ
        PDHCV(JROF,JLEV,IDHCV+ 9)=-ZFORNH
!     EVAPORATION DE NEIGE
!     TQVEVAPPSTN
!     TCTEVAPPSTN
        PDHCV(JROF,JLEV,IDHCV+10)=-ZEVANQ
        PDHCV(JROF,JLEV,IDHCV+11)=-ZEVANH
!     FONTE
!     TQVFONTPST
!     TCTFONTPST
        PDHCV(JROF,JLEV,IDHCV+12)=ZFONTQ
        PDHCV(JROF,JLEV,IDHCV+13)=ZFONTH
!     PRET POUR LE NIVEAU SUIVANT
        ZFPSLM(JROF) = ZRLP
      ENDDO
    ENDDO

  IF(LFLEXDIA) THEN
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+4),'TQVFORMPSTL','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+5),'TCTFORMPSTL','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+6),'TQVEVAPPSTL','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+7),'TCTEVAPPSTL','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+8),'TQVFORMPSTN','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+9),'TCTFORMPSTN','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+10),'TQVEVAPPSTN','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+11),'TCTEVAPPSTN','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+12),'TQVFONTPST','F','ARP',.TRUE.,.TRUE.) 
     CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+13),'TCTFONTPST','F','ARP',.TRUE.,.TRUE.) 
  ENDIF

!*         - PRECIPITATIONS CONVECTIVES.

!     INITIALISATION
    DO JROF=KSTART,KPROF
      ZFPCL = PFPLCL(JROF,0) + PFPLCN(JROF,0)
!     ZIBPLM = 1. SI FLUX SIGNIFICATIF EN JLEV-1
      ZIBPLM = MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFPCL-ZNEG))
!     PROPORTION AU SOMMET
      ZFPCLM(JROF) = ZIBPLM*PFPLCL(JROF,0)/MAX(ZNEG,ZFPCL)
    ENDDO
    DO JLEV=1, KFLEV
      DO JROF=KSTART,KPROF
        ZFPCL=PFPLCL(JROF,JLEV)+PFPLCN(JROF,JLEV)
!     ZIBPL=1. SI FLUX SIGNIFICATIF EN JLEV
        ZIBPL=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFPCL-ZNEG))
!     PROPORTION A LA BASE DE LA COUCHE
        ZRLP=ZIBPL*PFPLCL(JROF,JLEV)/MAX(ZNEG,ZFPCL)
!     PROPORTIONS DU MELANGE: MOYENNE PONDEREE DE LA COUCHE (JLEV-1)
!     ET (JLEV) AVEC LES POIDS RESPECTIFS 1. ET ZIBPL
        ZRL=(ZFPCLM(JROF)+ZIBPL*ZRLP)/(1.0_JPRB+ZIBPL)
        ZRL=MAX(ZRL,1.0_JPRB-ZIBNEI)
!     ZDLQ: VARIATION DU FLUX DE PRECIPITATION LIQUIDE DANS LA COUCHE.
        ZDLQ=PFPLCL(JROF,JLEV)-PFPLCL(JROF,JLEV-1)
        ZIBL=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDLQ))
!     ZDNQ: VARIATION DU FLUX DE PRECIPITATION NEIGEUX DANS LA COUCHE.
        ZDNQ=PFPLCN(JROF,JLEV)-PFPLCN(JROF,JLEV-1)
        ZIBN=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDNQ))
!     ZDTQ: VARIATION DU FLUX DE PRECIPITATION DANS LA COUCHE: TOTAL.
        ZDTQ=ZDLQ+ZDNQ
        ZIBT=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDTQ))
!     ZDLH: VARIATION DU FLUX D'ENTHALPIE LIQUIDE DANS LA COUCHE.
        ZDLH=PFHPCL(JROF,JLEV)-PFHPCL(JROF,JLEV-1)
!     ZDNH: VARIATION DU FLUX D'ENTHALPIE NEIGEUSE DANS LA COUCHE.
        ZDNH=PFHPCN(JROF,JLEV)-PFHPCN(JROF,JLEV-1)
!     ZDTH: VARIATION DU FLUX D'ENTHALPIE DANS LA COUCHE: TOTAL.
        ZDTH=ZDLH+ZDNH
!     1. FLUX DE PRECIPITATIONS. ---------------------------------
!     FONTE
        ZFONTQ=ZIBT*(-ZIBL*(1.0_JPRB-ZIBN)*ZDNQ &
         & +(1.0_JPRB-ZIBL)*ZIBN*ZDLQ*ZIBNEI)&
         & +(1.0_JPRB-ZIBT)*((1.0_JPRB-ZRL)*ZDLQ-ZRL*ZDNQ)  
!     FORMATION LIQUIDE
        ZFORLQ=ZIBT*(ZIBL*ZIBN*ZDLQ+ZIBL*(1.0_JPRB-ZIBN)*ZDTQ)
!     EVAPORATION LIQUIDE
        ZEVALQ=(1.0_JPRB-ZIBT)*ZRL*ZDTQ
!     FORMATION NEIGEUSE
        ZFORNQ=ZIBT*ZIBNEI*(ZIBL*ZIBN*ZDNQ+(1.0_JPRB-ZIBL)*ZIBN*ZDTQ)
!     EVAPORATION NEIGEUSE
        ZEVANQ=(1.0_JPRB-ZIBT)*(1.0_JPRB-ZRL)*ZDTQ
!     2. FLUX D'ENTHALPIE. ---------------------------------------
!     ENTHALPIE DE FONTE
        ZFONTH=-RLMLT*ZFONTQ
!     AFIN D'ESTIMER LA PART DES TERMES FORMATION, EVAPORATION ET FONTE
!     DANS LE BILAN TOTAL D'ENTHALPIE DE LA COUCHE, ON DISTINGUE
!     PREMIEREMENT LA FONTE DU GEL
        ZIBFONT=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFONTQ))
!     DANS UN SECOND TEMPS, ON CALCULE LA PART DE LA FORMATION DANS LE
!     TOTAL (FORMATION+EVAPORATION)
        ZTOT=ZFORLQ+ZEVALQ
        ZIBNEG=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZTOT-ZNEG))
        ZPROPL=ZIBNEG*ZFORLQ/MAX(ZNEG,ZTOT)
        ZTOT=ZFORNQ+ZEVANQ
        ZIBNEG=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZTOT-ZNEG))
        ZPROPN=ZIBNEG*ZFORNQ/MAX(ZNEG,ZTOT)
!     ON CALCULE ENFIN LA PARTITION D'ENTHALPIE
        ZFORLH=ZPROPL     *(ZDLH+(1.0_JPRB-ZIBFONT)*ZFONTH)
        ZEVALH=(1.0_JPRB-ZPROPL)*(ZDLH+(1.0_JPRB-ZIBFONT)*ZFONTH)
        ZFORNH=ZPROPN     *(ZDNH+ZIBFONT     *ZFONTH)
        ZEVANH=(1.0_JPRB-ZPROPN)*(ZDNH+ZIBFONT     *ZFONTH)
!     3. ECRITURE DES RESULTATS. ---------------------------------
!     FORMATION DE PLUIE
!     TQVFORMPCOL
!     TCTFORMPCOL
        PDHCV(JROF,JLEV,IDHCV+14)=-ZFORLQ
        PDHCV(JROF,JLEV,IDHCV+15)=-ZFORLH
!     EVAPORATION DE PLUIE
!     TQVEVAPPCOL
!     TCTEVAPPCOL
        PDHCV(JROF,JLEV,IDHCV+16)=-ZEVALQ
        PDHCV(JROF,JLEV,IDHCV+17)=-ZEVALH
!     FORMATION DE NEIGE
!     TQVFORMPCON
!     TCTFORMPCON
        PDHCV(JROF,JLEV,IDHCV+18)=-ZFORNQ
        PDHCV(JROF,JLEV,IDHCV+19)=-ZFORNH
!     EVAPORATION DE NEIGE
!     TQVEVAPPCON
!     TCTEVAPPCON
        PDHCV(JROF,JLEV,IDHCV+20)=-ZEVANQ
        PDHCV(JROF,JLEV,IDHCV+21)=-ZEVANH
!     FONTE
!     TQVFONTPCO
!     TCTFONTPCO
        PDHCV(JROF,JLEV,IDHCV+22)=ZFONTQ
        PDHCV(JROF,JLEV,IDHCV+23)=ZFONTH
!     PRET POUR LE NIVEAU SUIVANT
        ZFPCLM(JROF) = ZRLP
      ENDDO
    ENDDO
    
    IF(LFLEXDIA) THEN 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+14),'TQVFORMPCOL','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+15),'TCTFORMPCOL','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+16),'TQVEVAPPCOL','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+17),'TCTEVAPPCOL','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+18),'TQVFORMPCON','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+19),'TCTFORMPCON','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+20),'TQVEVAPPCON','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+21),'TCTEVAPPCON','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+22),'TQVFONTPCO','F','ARP',.TRUE.,.TRUE.) 
      CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+23),'TCTFONTPCO','F','ARP',.TRUE.,.TRUE.) 
    ENDIF



!IF LHDADD: ENDIF
  ENDIF
ENDIF

IF ( LHDENT ) THEN
  IDHCV=NDHVV+NDHFVD+NDHAVP+NDHBHKP+NDHBMCP
  DO JLEV = 1, KFLEV
    DO JROF=KSTART,KPROF
!     TSSDISSIPTUR
      PDHCV(JROF,JLEV,IDHCV+1) = ( PUZGEO(JROF,JLEV)*&
       & (ZSTTUG(JROF,JLEV)-ZSTTUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTTVG(JROF,JLEV)-ZSTTVG(JROF,JLEV-1)) ) / PT0(JROF,JLEV)  
!     TSSDISSIPCONV
      PDHCV(JROF,JLEV,IDHCV+2) = ( PUZGEO(JROF,JLEV)*&
       & (ZSTCUG(JROF,JLEV)-ZSTCUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTCVG(JROF,JLEV)-ZSTCVG(JROF,JLEV-1)) ) / PT0(JROF,JLEV)  
!     TSSDISSIPGREL
      PDHCV(JROF,JLEV,IDHCV+3) = ( PUZGEO(JROF,JLEV)*&
       & (ZSTDUG(JROF,JLEV)-ZSTDUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTDVG(JROF,JLEV)-ZSTDVG(JROF,JLEV-1)) ) / PT0(JROF,JLEV)  
!     TSSTURQVTOT
      PDHCV(JROF,JLEV,IDHCV+4) =&
       & (PENTRA(JROF,JLEV)-PENTRV(JROF,JLEV) + RCPV-RCPD)*&
       & ( PDIFTQ(JROF,JLEV)-PDIFTQ(JROF,JLEV-1) +&
       & PDIFCQ(JROF,JLEV)-PDIFCQ(JROF,JLEV-1) )  
!     TSSRAY1
      PDHCV(JROF,JLEV,IDHCV+5) =&
       & - ( PFRSO(JROF,JLEV)-PFRSO(JROF,JLEV-1) +&
       & PFRTH(JROF,JLEV)-PFRTH(JROF,JLEV-1) ) / PT0(JROF,JLEV)  
!     TSSTURCTTOT
      PDHCV(JROF,JLEV,IDHCV+6) =&
       & - ( PDIFTS(JROF,JLEV)-PDIFTS(JROF,JLEV-1) +&
       & PDIFCS(JROF,JLEV)-PDIFCS(JROF,JLEV-1) ) / PT0(JROF,JLEV)  
    ENDDO
  ENDDO

  IF(LFLEXDIA) THEN
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+1),'TSSDISSTUR','F','ARP',.TRUE.,.TRUE.) 
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+2),'TSSDISSCONV','F','ARP',.TRUE.,.TRUE.) 
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+3),'TSSDISSGREL','F','ARP',.TRUE.,.TRUE.) 
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+4),'TSSTURQVTOT','F','ARP',.TRUE.,.TRUE.) 
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+5),'TSSRAY1','F','ARP',.TRUE.,.TRUE.) 
    CALL ADD_FIELD_3D(PDHCV(:,:,IDHCV+6),'TSSTURCTTOT','F','ARP',.TRUE.,.TRUE.) 
  ENDIF
ENDIF





!     ------------------------------------------------------------------
!**   4.- CHAMPS AU SOL

IDHCS=0

IF (LDVERCOR) THEN
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF) &
   & *PVCRSSAH(KSTART:KPROF,KFLEV)*PVCRSSAH(KSTART:KPROF,KFLEV)  
ELSEIF (LRWSDLR) THEN
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF) &
   & *PRDLR(KSTART:KPROF,KFLEV,JP_RSAH)*PRDLR(KSTART:KPROF,KFLEV,JP_RSAH)  
ELSE
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF)
ENDIF

!     VARIABLES
!     ---------
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = ZLSM(JROF)
    IF(LEPHYS.OR.LMSE) THEN
      PDHCS(JROF,IDHCS+2) = ZLSM(JROF)*PTS(JROF)
      PDHCS(JROF,IDHCS+3) = ZLSM(JROF)*PTP(JROF)
    ELSE
      PDHCS(JROF,IDHCS+2) = ZLSM(JROF)/PCT(JROF)*PTS(JROF)
      PDHCS(JROF,IDHCS+3) = ZLSM(JROF)/PCT(JROF)*RTINER*PTP(JROF)
    ENDIF
    PDHCS(JROF,IDHCS+4) = ZLSM(JROF)*PWS(JROF)
    PDHCS(JROF,IDHCS+5) = ZLSM(JROF)*PWP(JROF)
    PDHCS(JROF,IDHCS+6) = ZLSM(JROF)*PSNS(JROF)
  ENDDO

  IDHCS = IDHCS + 6

ENDIF

!    FREE_STYLE VARIABLES
!    --------------------

IDHCS = NDHVS
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = PTCLS(JROF)
    PDHCS(JROF,IDHCS+2) = PQCLS(JROF)
    PDHCS(JROF,IDHCS+3) = PUCLS(JROF)
    PDHCS(JROF,IDHCS+4) = PVCLS(JROF)
    PDHCS(JROF,IDHCS+5) = PHI0(JROF,KFLEV)
    PDHCS(JROF,IDHCS+6) = PGZ0(JROF)
    PDHCS(JROF,IDHCS+7) = PGZ0H(JROF)
    PDHCS(JROF,IDHCS+8) = PALB(JROF)
    PDHCS(JROF,IDHCS+9) = PCLPH(JROF)
  ENDDO
ENDIF

!     FLUX EN SURFACE
!     ---------------

IDHCS = NDHVS + NDHVFS + NDHFSD
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = ZLSM(JROF)*PFRSO(JROF,KFLEV)
    PDHCS(JROF,IDHCS+2) = ZLSM(JROF)*PFRTH(JROF,KFLEV)
    PDHCS(JROF,IDHCS+3) = ZLSM(JROF)*PFCLL (JROF)
    PDHCS(JROF,IDHCS+4) = ZLSM(JROF)*PFCLN (JROF)
    PDHCS(JROF,IDHCS+5) = ZLSM(JROF)*PFCS  (JROF)
    PDHCS(JROF,IDHCS+6) = ZLSM(JROF)*PFCHSP(JROF)
    PDHCS(JROF,IDHCS+7) = -ZLSM(JROF)*PFONTE(JROF)*RLMLT
    PDHCS(JROF,IDHCS+8) = ZLSM(JROF)*PFPLSL(JROF,KFLEV)
    PDHCS(JROF,IDHCS+9) = ZLSM(JROF)*PFPLSN(JROF,KFLEV)
    PDHCS(JROF,IDHCS+10) = ZLSM(JROF)*PFPLCL(JROF,KFLEV)
    PDHCS(JROF,IDHCS+11) = ZLSM(JROF)*PFPLCN(JROF,KFLEV)
    PDHCS(JROF,IDHCS+12) = ZLSM(JROF)*PFEVL (JROF)
    PDHCS(JROF,IDHCS+13) = ZLSM(JROF)*PFEVN (JROF)
    PDHCS(JROF,IDHCS+14) = ZLSM(JROF)*PFLWSP(JROF)
    PDHCS(JROF,IDHCS+15) = ZLSM(JROF)*PFONTE(JROF)
    PDHCS(JROF,IDHCS+16) = ZLSM(JROF)*PRUISS(JROF)
    PDHCS(JROF,IDHCS+17) = ZLSM(JROF)*PRUISP(JROF)
  ENDDO

  IDHCS = IDHCS + 17

ENDIF

!       FREE_STYLE FLUXES
!       -----------------

IDHCS = NDHVS + NDHVFS + NDHFSD + NDHFSP
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = PFRSO(JROF,KFLEV)/MAX(1.0_JPRB-PALB(JROF),ZEPS)
    PDHCS(JROF,IDHCS+2) = PFRTHDS(JROF)
  ENDDO

  IDHCS = IDHCS + 2

ENDIF

!     ------------------------------------------------------------------

IF ( LHDLIST ) THEN
  WRITE (NULOUT,*) ' DDH * CPPH IDHCV=',IDHCV,' IDHCS = ',IDHCS
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPHDDH',1,ZHOOK_HANDLE)
END SUBROUTINE CPPHDDH
