SUBROUTINE CPPHDDHE(KPROMA, KSTART, KPROF, KFLEV, &
 !     INPUT.
 & KDDHI,PDHSF,PNORDL, PNORDM, PUZGEO, PVMGEO,&
 & PSLAT, PCLAT, PLON, PLSM, PVFCI, PRSAH,&
 & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFCQNG,&
 & PFPLCL, PFPLCN, PFPLSL, PFPLSN,&
 & PFRSO , PFRTH , &
 & PFHPCL, PFHPCN, PFHPSL, PFHPSN,&
 & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV,&
 & PFCQLNG,PFCQNNG,&
 & PFCQLF,PFCQIF,PFSQLF,PFSQIF,&
 & PDHTLS ,PDHTSS ,PDHTTS ,PDHTIS ,&
 & PDHSSS ,PDHIIS ,PDHWLS ,PDHFSS ,&
 !     OUTPUT.
 & PDHCV, PDHCS)  
!---------------------------------------------------------------------------
!**** *CPPHDDHE*  - CALCUL DES FLUX/TENDANCES DYNAMIQUES
!                  DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!---------------------------------------------------------------------------
!     BUT
!     ---
!      CALCULS PHYSIQUES POUR LES DIAGNOSTICS DDH
!      AINSI QUE DES VARIABLES AU SOL A L INSTANT COURANT

!   ** SURVEILLER LE POSITIONNEMENT DES CHAMPS AU MOYEN DE L INDICATEUR
!                           IDHCV

!     ARGUMENTS D ENTREE
!     ------------------

! - DIMENSIONS

!     KPROMA : DIMENSION HORIZONTALE.
!     KSTART : DEBUT DE LA BOUCLE HORIZONTALE.
!     KPROF  : FIN DE LA BOUCLE HORIZONTALE.
!     KFLEV  : DIMENSION VERTICALE ET NB DE NIVEAUX

! - TABLEAUX DE SORTIES ET AUTRES INFORMATIONS DDH
!     PDHCV :  TABLEAUX DE RESULTAT INITIALISE A 0.
!     PDHCS :  TABLEAUX DE RESULTAT AU SOL INITIALISE A 0.

! - TABLEAUX DE DONNEES
!     PSLAT,  PCLAT  : SINUS RESP. COSINUS OF LATITUDE
!     PLON           : LONGITUDE
!     PNORDL, PNORDM : DIRECTION DU NORD GEOGRAPHIQUE
!     PUZGEO, PVMGEO : VENT ZONAL ET MERIDIEN VRAIS GEOGRAPHIQUES
!            (UNIQUEMENT SI LHDHKS OU LHDMCI OU LHDENT VRAIS)

! - 1D (GEOGRAPHIQUE) .

! PLSM       : land-sea mask (values are 0 or 1)
! PVFCI      : sea-ice fraction

! - Deep layer eqns arrays.
! PRSAH      : "r/a" at half levels.

! - TABLEAUX ISSUS DES PARAMETRISATIONS
! - 2D (0:KFLEV) .

! PDIFCQ     : FLUX CONVECTIF D'HUMIDITE SPECIFIQUE (HORS PLUIE/NEIGE).
! PDIFCS     : FLUX CONVECTIF D'ENTHALPIE (HORS PLUIE/NEIGE).
! PDIFTQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
! PDIFTS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
! PFCQNG     : FLUX FICTIF D'EAU POUR CORRIGER LES Q<0.
! PFPLCL     : PRECIPITATIONS CONVECTIVES SOUS FORME LIQUIDE.
! PFPLCN     : PRECIPITATIONS CONVECTIVES SOUS FORME NEIGE.
! PFPLSL     : PRECIPITATIONS STRATIFORMES SOUS FORME LIQUIDE.
! PFPLSN     : PRECIPITATIONS STRATIFORMES SOUS FORME NEIGE.
! PFRSO      : FLUX DE RAYONNEMENT SOLAIRE.
! PFRTH      : FLUX DE RAYONNEMENT THERMIQUE.
! PSTRCU     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "U".
! PSTRCV     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "V".
! PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".
! PSTRTU     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "U".
! PSTRTV     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "V".

! - TABLEAUX ISSUS DES CALCULS DE TENDANCES
! - 2D (0:KFLEV) .

! PFHPCL     : FLUX ENTHALPIE CONVECTIVES SOUS FORME LIQUIDE.
! PFHPCN     : FLUX ENTHALPIE CONVECTIVES SOUS FORME NEIGE.
! PFHPSL     : FLUX ENTHALPIE STRATIFORMES SOUS FORME LIQUIDE.
! PFHPSN     : FLUX ENTHALPIE STRATIFORMES SOUS FORME NEIGE.

! - 1D (DIAGNOSTIQUE) .

! PFCQLNG    : FLUX D'EAU LIQUIDE DE CORR. DES VALEURS < 0.
! PFCQNNG    : FLUX D'EAU NEIGEUSE DE CORR. DES VALEURS < 0.
! PFCQLF     : FLUX DE CONDENSATION CONV. LIQ.
! PFCQIF     : FLUX DE CONDENSATION CONV. NEI.
! PFSQLF     : FLUX DE CONDENSATION STRA. LIQ.
! PFSQIF     : FLUX DE CONDENSATION STRA. NEI.

!     --- NEW SURFACE DIAGNOSTICS

! PDHTLS     : Variables and fluxes for individual tiles
! PDHTSS     : Variables and fluxes for snow energy budget
! PDHTTS     : Variables and fluxes for soil energy budget
! PDHTIS     : Variables and fluxes for sea-ice energy budget
! PDHSSS     : Variables and fluxes for snow water budget
! PDHIIS     : Variables and fluxes for interception layer water budget
! PDHWLS     : Variables and fluxes for soil water budget
! PDHFSS     : Free-style variables and fluxes 

!     ARGUMENTS IMPLICITES
!     --------------------
!       UNITE NULOUT (/YOMLUN/)
!       CONSTANTES UNIVERSELLES, COMMON /YOMCST/
!       FONCTIONS THERMODYNAMIQUES, COMDECK *FCTTRM*

!       CONTROLE PHYSIQUE DE BASE : YOMPHY : NDPSFI
!       CONSTANTES PHYSIQUES (NORMALEMENT INUTILES)
!       YOMPHY0, YOMPHY1, YOMPHY2, YOMPHY3

!       ELEMENTS DE CONTROLE DDH
!       INDICATEURS LOGIQUES, COMMON /YOMLDDH/
!        DANS LE COMDECK * YOMMDDH * POUR LES MASQUES
!       DIMENSIONS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /DDHDIM/
!       ET DIMENSIONS POUR LES TABLEAUX DE CUMULS
!       INDICATEURS DE POSITIONS DANS LES TABLEAUX DE CUMULS

!     AUTRES ENTREES
!     --------------

!     SORTIES
!     -------
!      DANS PDHCV, FLUX/TENDANCES PHYSIQUES INSTANTANEES
!      DANS PDHCS  VARIABLES ET FLUX AU SOL

!     ORIGINAL
!     --------
!     91-03-04, Alain Joly.

!     MODIFICATIONS
!     -------------
!     92-03-01, M. Janousek: diagnostic de l'entropie.
!     92-06-01, M. Janousek: diagnostic du moment cinetique.
!     92-06-29, J.M. Piriou: changement de signe des chaleurs latentes.
!     94-01-17, C. Jakob: rewriting of moist processes for ECMWF
!                         physics
!     99-06-23, J. Teixeira: Surface tiling
!     00-05-15, C. Jakob : new diagnostic surface fields 
!     R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      20-Apr-2004 Minor optimization.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LVERCOR 
USE YOMDIM   , ONLY : NPROMDLR 
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMRIP   , ONLY : NSSSSS   ,RSTATI
USE YOMPHY   , ONLY : NDPSFI   ,LNEIGE
USE YOMCST   , ONLY : ROMEGA   ,RG       ,RLVTT    ,RLSTT    ,RLMLT
USE YOMLDDH  , ONLY : LHDHKS   ,LHDMCI
USE YOMMDDH  , ONLY : NDHCVSU  ,NDHCSSU  ,NDHVS    ,NDHFSD   ,&
 & NDHFSP   ,NDHVFS  ,NDHAVP   ,NDHVTLS  ,NDHFTLS  ,&
 & NDHVTSS  ,NDHFTSS ,NDHVTTS  ,NDHFTTS  ,NDHVTIS  ,&
 & NDHFTIS  ,NDHVSSS ,NDHFSSS  ,NDHVIIS  ,NDHFIIS  ,&
 & NDHVWLS  ,NDHFWLS ,NDHFFS  
USE YOMSDDH  , ONLY : LHDQLN   ,LHDADD
USE YOMDPHY  , ONLY : NCSI     ,NCSNEC   ,NTILES
USE SURFACE_FIELDS_MIX  , ONLY : YSP_SBD

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM)               :: KDDHI(KPROMA) ! Argument NOT used
REAL(KIND=JPRB)                  :: PDHSF(KPROMA) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUZGEO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVMGEO(KPROMA,KFLEV) 
REAL(KIND=JPRB)                  :: PSLAT(KPROMA) ! Argument NOT used
REAL(KIND=JPRB)                  :: PCLAT(KPROMA) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFCI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSAH(NPROMDLR,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQIF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFSQLF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFSQIF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHTLS(KPROMA,NTILES,NDHVTLS+NDHFTLS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHTSS(KPROMA,NCSNEC,NDHVTSS+NDHFTSS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHTTS(KPROMA,YSP_SBD%NLEVS,NDHVTTS+NDHFTTS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHTIS(KPROMA,NCSI,NDHVTIS+NDHFTIS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHSSS(KPROMA,NCSNEC,NDHVSSS+NDHFSSS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHIIS(KPROMA,NDHVIIS+NDHFIIS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHWLS(KPROMA,YSP_SBD%NLEVS,NDHVWLS+NDHFWLS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHFSS(KPROMA,NDHVFS+NDHFFS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDHCV(KPROMA,0:KFLEV,NDHCVSU) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDHCS(KPROMA,NDHCSSU) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZCLON(KPROMA), ZSLON(KPROMA)

REAL(KIND=JPRB) :: ZFPSLM(KPROMA), ZFPCLM(KPROMA)
REAL(KIND=JPRB) :: ZFPSL (KPROMA), ZFPCL (KPROMA)
!     ZFPSLM, ZFPCLM : FLUX DE PRECIPITATIONS TOTAL INTERCOUCHE HAUT

!     FLUX DE QTE DE MVT TURBULENTS DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTTUG(KPROMA,2), ZSTTVG(KPROMA,2)
!     FLUX DE QTE DE MVT CONVECTIFS DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTCUG(KPROMA,2), ZSTCVG(KPROMA,2)
!     FLUX DE QTE DE MVT ONDES OROG.DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTDUG(KPROMA,2), ZSTDVG(KPROMA,2)

REAL(KIND=JPRB) :: ZLSM(KPROMA) 

INTEGER(KIND=JPIM) :: IDHCS, IDHCV, ILEN, JLEV, JROF, JVAR, JTILE, JFLX
INTEGER(KIND=JPIM) :: ICURLEV, ILEVM1
REAL(KIND=JPRB) :: ZDISKC, ZDISKD, ZDISKT, ZDLH, ZDLQ, ZDNH,&
 & ZDNQ, ZDPSFI, ZDTH, ZDTQ, ZEVALH, ZEVALQ, &
 & ZEVANH, ZEVANQ, ZFONTH, ZFONTQ, ZFORLH, ZFORLQ, &
 & ZFORNH, ZFORNQ, ZFPLL, ZFPLN, ZFPL,ZIBL, ZIBN, &
 & ZIBNEI, ZIBPL, ZIBT, ZNEG, ZRL, ZRLP, ZUSRG  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPHDDHE',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!**   1.- INITIALISATIONS DIVERSES

ZUSRG = 1.0_JPRB/RG
ZDPSFI = REAL(NDPSFI,JPRB)
ILEN=KPROF-KSTART+1

IF ( LHDMCI ) THEN
  DO JROF=KSTART,KPROF
    ZCLON(JROF) = COS(PLON(JROF)+ROMEGA*(RSTATI-NSSSSS))
    ZSLON(JROF) = SIN(PLON(JROF)+ROMEGA*(RSTATI-NSSSSS))
  ENDDO
ENDIF

ICURLEV = 0
DO JLEV = 0, KFLEV
  ICURLEV = MOD(ICURLEV,2)+1
  ILEVM1  = MOD(ICURLEV,2)+1
  DO JROF=KSTART,KPROF
    ZSTTUG(JROF,ICURLEV) = PNORDM(JROF)*PSTRTU(JROF,JLEV)-&
     & PNORDL(JROF)*PSTRTV(JROF,JLEV)  
    ZSTTVG(JROF,ICURLEV) = PNORDL(JROF)*PSTRTU(JROF,JLEV)+&
     & PNORDM(JROF)*PSTRTV(JROF,JLEV)  
    ZSTCUG(JROF,ICURLEV) = PNORDM(JROF)*PSTRCU(JROF,JLEV)-&
     & PNORDL(JROF)*PSTRCV(JROF,JLEV)  
    ZSTCVG(JROF,ICURLEV) = PNORDL(JROF)*PSTRCU(JROF,JLEV)+&
     & PNORDM(JROF)*PSTRCV(JROF,JLEV)  
    ZSTDUG(JROF,ICURLEV) = PNORDM(JROF)*PSTRDU(JROF,JLEV)-&
     & PNORDL(JROF)*PSTRDV(JROF,JLEV)  
    ZSTDVG(JROF,ICURLEV) = PNORDL(JROF)*PSTRDU(JROF,JLEV)+&
     & PNORDM(JROF)*PSTRDV(JROF,JLEV)  
  ENDDO



!     ------------------------------------------------------------------
!**   2.- CALCUL DES FLUX PHYSIQUES

  IDHCV = 0

  IF ( LHDHKS ) THEN
    DO JROF=KSTART,KPROF
!     FPPSUMFPL
      ZFPLL = PFPLCL(JROF,JLEV) + PFPLSL(JROF,JLEV)
      ZFPLN = PFPLCN(JROF,JLEV) + PFPLSN(JROF,JLEV)
      ZFPL  = ZFPLL  + ZFPLN
      PDHCV(JROF,JLEV,IDHCV+1)  = ZDPSFI*ZFPL
!     FQVTUR
      PDHCV(JROF,JLEV,IDHCV+2)  = PDIFTQ(JROF,JLEV)
!     FQVTURCONV
      PDHCV(JROF,JLEV,IDHCV+3)  = PDIFCQ(JROF,JLEV)
!     FQVPRECISTL
      PDHCV(JROF,JLEV,IDHCV+4)  = PFPLSL(JROF,JLEV)
!     FQVPRECISTN
      PDHCV(JROF,JLEV,IDHCV+5)  = PFPLSN(JROF,JLEV)
!     FQVPRECICOL
      PDHCV(JROF,JLEV,IDHCV+6)  = PFPLCL(JROF,JLEV)
!     FQVPRECICON
      PDHCV(JROF,JLEV,IDHCV+7)  = PFPLCN(JROF,JLEV)
!     FQVTURQNEGAT
      PDHCV(JROF,JLEV,IDHCV+8)  = PFCQNG(JROF,JLEV)
!     FUUTUR
!     FVVTUR
      PDHCV(JROF,JLEV,IDHCV+9)  = ZSTTUG(JROF,ICURLEV)
      PDHCV(JROF,JLEV,IDHCV+10) = ZSTTVG(JROF,ICURLEV)
!     FUUTURCONV
!     FVVTURCONV
      PDHCV(JROF,JLEV,IDHCV+11) = ZSTCUG(JROF,ICURLEV)
      PDHCV(JROF,JLEV,IDHCV+12) = ZSTCVG(JROF,ICURLEV)
!     FUUONDEGREL
!     FVVONDEGREL
      PDHCV(JROF,JLEV,IDHCV+13) = ZSTDUG(JROF,ICURLEV)
      PDHCV(JROF,JLEV,IDHCV+14) = ZSTDVG(JROF,ICURLEV)
!     FCTTUR
      PDHCV(JROF,JLEV,IDHCV+15) = PDIFTS(JROF,JLEV)
!     FCTTURCONV
      PDHCV(JROF,JLEV,IDHCV+16) = PDIFCS(JROF,JLEV)
!     FCTRAYSOL1
      PDHCV(JROF,JLEV,IDHCV+17) = PFRSO(JROF,JLEV)
!     FCTRAYTER1
      PDHCV(JROF,JLEV,IDHCV+18) = PFRTH(JROF,JLEV)
!     FCTPRECISTL
      PDHCV(JROF,JLEV,IDHCV+19) = PFHPSL(JROF,JLEV)
!     FCTPRECISTN
      PDHCV(JROF,JLEV,IDHCV+20) = PFHPSN(JROF,JLEV)
!     FCTPRECICOL
      PDHCV(JROF,JLEV,IDHCV+21) = PFHPCL(JROF,JLEV)
!     FCTPRECICON
      PDHCV(JROF,JLEV,IDHCV+22) = PFHPCN(JROF,JLEV)
!     FCTPRECCSCOL
!CC         PDHCV(JROF,JLEV,IDHCV+23) = PFHSCL(JROF,JLEV)
      PDHCV(JROF,JLEV,IDHCV+23) = 0.0_JPRB
!     FCTPRECCSCON
!CC         PDHCV(JROF,JLEV,IDHCV+24) = PFHSCN(JROF,JLEV)
      PDHCV(JROF,JLEV,IDHCV+24) = 0.0_JPRB
!     FCTPRECCSSTL
!CC         PDHCV(JROF,JLEV,IDHCV+25) = PFHSSL(JROF,JLEV)
      PDHCV(JROF,JLEV,IDHCV+25) = 0.0_JPRB
!     FCTPRECCSSTN
!CC         PDHCV(JROF,JLEV,IDHCV+26) = PFHSSN(JROF,JLEV)
      PDHCV(JROF,JLEV,IDHCV+26) = 0.0_JPRB
    ENDDO


    IF(LHDQLN) THEN
      DO JROF=KSTART,KPROF
!           FQLTUR
!CC            PDHCV(JROF,JLEV,IDHCV+27) = PDIFTQL(JROF,JLEV)
        PDHCV(JROF,JLEV,IDHCV+27) = 0.0_JPRB
!           FQNTUR
!CC           PDHCV(JROF,JLEV,IDHCV+28) = PDIFTQN(JROF,JLEV)
        PDHCV(JROF,JLEV,IDHCV+28) = 0.0_JPRB
!           FQLTURCONV
!CC           PDHCV(JROF,JLEV,IDHCV+29) = PDIFCQL(JROF,JLEV)
        PDHCV(JROF,JLEV,IDHCV+29) = 0.0_JPRB
!           FQNTURCONV
!CC           PDHCV(JROF,JLEV,IDHCV+30) = PDIFCQN(JROF,JLEV)
        PDHCV(JROF,JLEV,IDHCV+30) = 0.0_JPRB
!           FQLTURQNEGAT
        PDHCV(JROF,JLEV,IDHCV+31) = PFCQLNG(JROF,JLEV)
!           FQNTURQNEGAT
        PDHCV(JROF,JLEV,IDHCV+32) = PFCQNNG(JROF,JLEV)
!           FQTCONDECOL
        PDHCV(JROF,JLEV,IDHCV+33) = PFCQLF(JROF,JLEV)
!           FQTCONDECON
        PDHCV(JROF,JLEV,IDHCV+34) = PFCQIF(JROF,JLEV)
!           FQTCONDESTL
        PDHCV(JROF,JLEV,IDHCV+35) = PFSQLF(JROF,JLEV)
!           FQTCONDESTN
        PDHCV(JROF,JLEV,IDHCV+36) = PFSQIF(JROF,JLEV)
      ENDDO
    ENDIF
    IDHCV = IDHCV + 22

  ENDIF

!     ------------------------------------------------------------------
!**   3.- CALCUL DES TENDANCES PHYSIQUES

  IDHCV = NDHAVP

!*         - DISSIPATION D'ENERGIE CINETIQUE.

  IF ( LHDHKS .AND. JLEV >0) THEN
    DO JROF=KSTART,KPROF
!     TKKDISSIPTUR
      ZDISKT = -( PUZGEO(JROF,JLEV)*&
       & (ZSTTUG(JROF,ICURLEV)-ZSTTUG(JROF,ILEVM1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTTVG(JROF,ICURLEV)-ZSTTVG(JROF,ILEVM1)) )  
      PDHCV(JROF,JLEV,IDHCV+1) = ZDISKT
!     TKKDISSIPCONV
      ZDISKC = -( PUZGEO(JROF,JLEV)*&
       & (ZSTCUG(JROF,ICURLEV)-ZSTCUG(JROF,ILEVM1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTCVG(JROF,ICURLEV)-ZSTCVG(JROF,ILEVM1)) )  
      PDHCV(JROF,JLEV,IDHCV+2) = ZDISKC
!     TKKDISSIPGREL
      ZDISKD = -( PUZGEO(JROF,JLEV)*&
       & (ZSTDUG(JROF,ICURLEV)-ZSTDUG(JROF,ILEVM1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTDVG(JROF,ICURLEV)-ZSTDVG(JROF,ILEVM1)) )  
      PDHCV(JROF,JLEV,IDHCV+3) = ZDISKD
    ENDDO

!IF LHDADD: IF
    IF(LHDADD) THEN

!*         - PRECIPITATIONS STRATIFORMES.

      ZNEG=1.E-12_JPRB
!     ZIBNEI=1. SI PRECIPITATIONS EN DEUX PHASES.
      IF(LNEIGE) THEN
        ZIBNEI=1.0_JPRB
      ELSE
        ZIBNEI=0.0_JPRB
      ENDIF
 !     INITIALISATION
      IF(JLEV == 1) THEN
        DO JROF=KSTART,KPROF
          ZFPSLM(JROF)=0.0_JPRB
        ENDDO
      ENDIF

      DO  JROF=KSTART,KPROF
        ZFPSL(JROF)=PFPLSL(JROF,JLEV)+PFPLSN(JROF,JLEV)
!     ZIBPL=1. SI FLUX SIGNIFICATIF EN JLEV
!        ZIBPL=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFPSL(JROF)-ZNEG))
!     PROPORTION A LA BASE DE LA COUCHE
        IF(ZFPSL(JROF) > ZNEG) THEN
          ZRLP=PFPLSL(JROF,JLEV)/ZFPSL(JROF)
        ELSE
          ZRLP=0.0_JPRB
        ENDIF
!     PROPORTIONS DU MELANGE: MOYENNE PONDEREE DE LA COUCHE (JLEV-1)
!     ET (JLEV) AVEC LES POIDS RESPECTIFS 1. ET ZIBPL
        ZRL=ZFPSLM(JROF)
!           ZRL=(ZFPSLM(JROF)+ZIBPL*ZRLP)/(1.+ZIBPL)
!           ZRL=MAX(ZRL,1.-ZIBNEI)
!     ZDLQ: VARIATION DU FLUX DE PRECIPITATION LIQUIDE DANS LA COUCHE.
        ZDLQ=PFPLSL(JROF,JLEV)-PFPLSL(JROF,JLEV-1)
        IF(ZDLQ > 0.0_JPRB) THEN
          ZIBL=1.0_JPRB
        ELSE
          ZIBL=0.0_JPRB
        ENDIF
!     ZDNQ: VARIATION DU FLUX DE PRECIPITATION NEIGEUX DANS LA COUCHE.
        ZDNQ=PFPLSN(JROF,JLEV)-PFPLSN(JROF,JLEV-1)
        IF(ZDNQ > 0.0_JPRB) THEN
          ZIBN=1.0_JPRB
        ELSE
          ZIBN=0.0_JPRB
        ENDIF
!     ZDTQ: VARIATION DU FLUX DE PRECIPITATION DANS LA COUCHE: TOTAL.
        ZDTQ=ZDLQ+ZDNQ
        IF(ZDTQ > 0.0_JPRB) THEN
          ZIBT=1.0_JPRB
        ELSE
          ZIBT=0.0_JPRB
        ENDIF
!     ZDLH: VARIATION DU FLUX D'ENTHALPIE LIQUIDE DANS LA COUCHE.
        ZDLH=PFHPSL(JROF,JLEV)-PFHPSL(JROF,JLEV-1)
!     ZDNH: VARIATION DU FLUX D'ENTHALPIE NEIGEUSE DANS LA COUCHE.
        ZDNH=PFHPSN(JROF,JLEV)-PFHPSN(JROF,JLEV-1)
!     ZDTH: VARIATION DU FLUX D'ENTHALPIE DANS LA COUCHE: TOTAL.
        ZDTH=ZDLH+ZDNH
!     1. PRECIPITATION FLUXES AND ENTHALPIES   --------------------

!*******************************************************************

!  CAUTION: THE CALCULATION OF ENTHALPIES IS ONLY VALID FOR 
!           CONSTANT VALUES OF LATENT HEATS !

!           FOR EVAPORATION CALCULATIONS THE PARTITIONING OF
!           RAIN AND SNOW IS TAKEN FROM LEVEL JLEV-1 (CONSISTENT
!           WITH ECMWF PHYSICS CY50) !

!*******************************************************************  

!     FONTE
        ZFONTQ=ZIBT*(-ZIBL*(1.0_JPRB-ZIBN)*ZDNQ &
         & +(1.0_JPRB-ZIBL)*ZIBN*ZDLQ*ZIBNEI)&
         & +(1.0_JPRB-ZIBT)*((1.0_JPRB-ZRL)*ZDLQ-ZRL*ZDNQ)  
        ZFONTH=-RLMLT*ZFONTQ
!     FORMATION LIQUIDE
        ZFORLQ=ZIBT*(ZIBL*ZIBN*ZDLQ+ZIBL*(1.0_JPRB-ZIBN)*ZDTQ)
        ZFORLH=-RLVTT*ZFORLQ
!     EVAPORATION LIQUIDE
        ZEVALQ=(1.0_JPRB-ZIBT)*ZRL*ZDTQ
        ZEVALH=-RLVTT*ZEVALQ
!     FORMATION NEIGEUSE
        ZFORNQ=ZIBT*ZIBNEI*(ZIBL*ZIBN*ZDNQ+(1.0_JPRB-ZIBL)*ZIBN*ZDTQ)
        ZFORNH=-RLSTT*ZFORNQ
!     EVAPORATION NEIGEUSE
        ZEVANQ=(1.0_JPRB-ZIBT)*(1.0_JPRB-ZRL)*ZDTQ
        ZEVANH=-RLSTT*ZEVANQ
!     3. ECRITURE DES RESULTATS. ---------------------------------
!     FORMATION DE PLUIE
!     TQVFORMPSTL
!     TCTFORMPSTL
        PDHCV(JROF,JLEV,IDHCV+ 4)=-ZFORLQ
        PDHCV(JROF,JLEV,IDHCV+ 5)=-ZFORLH
!     EVAPORATION DE PLUIE
!     TQVEVAPPSTL
!     TCTEVAPPSTL
        PDHCV(JROF,JLEV,IDHCV+ 6)=-ZEVALQ
        PDHCV(JROF,JLEV,IDHCV+ 7)=-ZEVALH
!     FORMATION DE NEIGE
!     TQVFORMPSTN
!     TCTFORMPSTN
        PDHCV(JROF,JLEV,IDHCV+ 8)=-ZFORNQ
        PDHCV(JROF,JLEV,IDHCV+ 9)=-ZFORNH
!     EVAPORATION DE NEIGE
!     TQVEVAPPSTN
!     TCTEVAPPSTN
        PDHCV(JROF,JLEV,IDHCV+10)=-ZEVANQ
        PDHCV(JROF,JLEV,IDHCV+11)=-ZEVANH
!     FONTE
!     TQVFONTPST
!     TCTFONTPST
        PDHCV(JROF,JLEV,IDHCV+12)=ZFONTQ
        PDHCV(JROF,JLEV,IDHCV+13)=ZFONTH
!     PRET POUR LE NIVEAU SUIVANT
        ZFPSLM(JROF) = ZRLP
      ENDDO

!*         - PRECIPITATIONS CONVECTIVES.

!     INITIALISATION
      
      IF(JLEV == 1) THEN
        DO JROF=KSTART,KPROF
          ZFPCLM(JROF)=0.0_JPRB
        ENDDO
      ENDIF

      DO JROF=KSTART,KPROF
        ZFPCL(JROF) = PFPLCL(JROF,JLEV)+PFPLCN(JROF,JLEV)
!     ZIBPL=1. SI FLUX SIGNIFICATIF EN JLEV
!        ZIBPL=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZFPCL(JROF)-ZNEG))
!     PROPORTION A LA BASE DE LA COUCHE
        IF(ZFPCL(JROF) > ZNEG) THEN
          ZRLP=ZIBPL*PFPLCL(JROF,JLEV)/ZFPCL(JROF)
        ELSE
          ZRLP=0.0_JPRB
        ENDIF
!     PROPORTIONS DU MELANGE: MOYENNE PONDEREE DE LA COUCHE (JLEV-1)
!     ET (JLEV) AVEC LES POIDS RESPECTIFS 1. ET ZIBPL
        ZRL=ZFPCLM(JROF)
!           ZRL=(ZFPCLM(JROF)+ZIBPL*ZRLP)/(1.+ZIBPL)
!           ZRL=MAX(ZRL,1.-ZIBNEI)
!     ZDLQ: VARIATION DU FLUX DE PRECIPITATION LIQUIDE DANS LA COUCHE.
        ZDLQ=PFPLCL(JROF,JLEV)-PFPLCL(JROF,JLEV-1)
        IF(ZDLQ > 0.0_JPRB) THEN
          ZIBL=1.0_JPRB
        ELSE
          ZIBL=0.0_JPRB
        ENDIF
!     ZDNQ: VARIATION DU FLUX DE PRECIPITATION NEIGEUX DANS LA COUCHE.
        ZDNQ=PFPLCN(JROF,JLEV)-PFPLCN(JROF,JLEV-1)
        IF(ZDNQ > 0.0_JPRB) THEN
          ZIBN=1.0_JPRB
        ELSE
          ZIBN=0.0_JPRB
        ENDIF
!     ZDTQ: VARIATION DU FLUX DE PRECIPITATION DANS LA COUCHE: TOTAL.
        ZDTQ=ZDLQ+ZDNQ
        IF(ZDTQ > 0.0_JPRB) THEN
          ZIBT=1.0_JPRB
        ELSE
          ZIBT=0.0_JPRB
        ENDIF
!     ZDLH: VARIATION DU FLUX D'ENTHALPIE LIQUIDE DANS LA COUCHE.
        ZDLH=PFHPCL(JROF,JLEV)-PFHPCL(JROF,JLEV-1)
!     ZDNH: VARIATION DU FLUX D'ENTHALPIE NEIGEUSE DANS LA COUCHE.
        ZDNH=PFHPCN(JROF,JLEV)-PFHPCN(JROF,JLEV-1)
!     ZDTH: VARIATION DU FLUX D'ENTHALPIE DANS LA COUCHE: TOTAL.
        ZDTH=ZDLH+ZDNH
!     1. PRECIPITATION FLUXES AND ENTHALPIES ------------------------

!*******************************************************************

!  CAUTION: THE CALCULATION OF ENTHALPIES IS ONLY VALID FOR 
!           CONSTANT VALUES OF LATENT HEATS !

!           FOR EVAPORATION CALCULATIONS THE PARTITIONING OF
!           RAIN AND SNOW IS TAKEN FROM LEVEL JLEV-1 (CONSISTENT
!           WITH ECMWF PHYSICS CY50) !

!*******************************************************************  

!     FONTE
        ZFONTQ=ZIBT*(-ZIBL*(1.0_JPRB-ZIBN)*ZDNQ &
         & +(1.0_JPRB-ZIBL)*ZIBN*ZDLQ*ZIBNEI)&
         & +(1.0_JPRB-ZIBT)*((1.0_JPRB-ZRL)*ZDLQ-ZRL*ZDNQ)  
        ZFONTH=-RLMLT*ZFONTQ
!     FORMATION LIQUIDE
        ZFORLQ=ZIBT*(ZIBL*ZIBN*ZDLQ+ZIBL*(1.0_JPRB-ZIBN)*ZDTQ)
        ZFORLH=-RLVTT*ZFORLQ
!     EVAPORATION LIQUIDE
        ZEVALQ=(1.0_JPRB-ZIBT)*ZRL*ZDTQ
        ZEVALH=-RLVTT*ZEVALQ
!     FORMATION NEIGEUSE
        ZFORNQ=ZIBT*ZIBNEI*(ZIBL*ZIBN*ZDNQ+(1.0_JPRB-ZIBL)*ZIBN*ZDTQ)
        ZFORNH=-RLSTT*ZFORNQ
!     EVAPORATION NEIGEUSE
        ZEVANQ=(1.0_JPRB-ZIBT)*(1.0_JPRB-ZRL)*ZDTQ
        ZEVANH=-RLSTT*ZEVANQ
!     3. ECRITURE DES RESULTATS. ---------------------------------
!     FORMATION DE PLUIE
!     TQVFORMPCOL
!     TCTFORMPCOL
        PDHCV(JROF,JLEV,IDHCV+14)=-ZFORLQ
        PDHCV(JROF,JLEV,IDHCV+15)=-ZFORLH
!     EVAPORATION DE PLUIE
!     TQVEVAPPCOL
!     TCTEVAPPCOL
        PDHCV(JROF,JLEV,IDHCV+16)=-ZEVALQ
        PDHCV(JROF,JLEV,IDHCV+17)=-ZEVALH
!     FORMATION DE NEIGE
!     TQVFORMPCON
!     TCTFORMPCON
        PDHCV(JROF,JLEV,IDHCV+18)=-ZFORNQ
        PDHCV(JROF,JLEV,IDHCV+19)=-ZFORNH
!     EVAPORATION DE NEIGE
!     TQVEVAPPCON
!     TCTEVAPPCON
        PDHCV(JROF,JLEV,IDHCV+20)=-ZEVANQ
        PDHCV(JROF,JLEV,IDHCV+21)=-ZEVANH
!     FONTE
!     TQVFONTPCO
!     TCTFONTPCO
        PDHCV(JROF,JLEV,IDHCV+22)=ZFONTQ
        PDHCV(JROF,JLEV,IDHCV+23)=ZFONTH
!C     PRET POUR LE NIVEAU SUIVANT
        ZFPCLM(JROF) = ZRLP
      ENDDO
    ENDIF
  ENDIF
ENDDO

!     ------------------------------------------------------------------
!**   4.- CHAMPS AU SOL

IDHCS = 0

IF (LVERCOR) THEN
  ! ky: correct code not yet implemented (because PVCRSSAH not yet available in
  !     the ECMWF lagged physics), run provisionally with the thin layer code.
  !! ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF) &
  !!  & *PVCRSSAH(KSTART:KPROF,KFLEV)*PVCRSSAH(KSTART:KPROF,KFLEV)
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF)
ELSEIF (LRWSDLR) THEN
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF) &
   & *PRSAH(KSTART:KPROF,KFLEV)*PRSAH(KSTART:KPROF,KFLEV)
ELSE
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF)
ENDIF

!     NORMAL VARIABLES
!     ----------------

! Tiles

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVTLS
    DO JTILE=1,NTILES
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JVAR-1)*NTILES+JTILE) = PDHTLS(JROF,JTILE,JVAR)&
!         &+REAL(IDHCS+(JVAR-1)*NTILES+JTILE)
!cj
        PDHCS(JROF,IDHCS+(JVAR-1)*NTILES+JTILE) = PDHTLS(JROF,JTILE,JVAR)
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHVTLS*NTILES
ENDIF

! Snow energy budget

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVTSS
    DO JLEV=1,NCSNEC
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JVAR-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
!         &PDHTSS(JROF,JLEV,JVAR)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JVAR-1)*NCSNEC+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JVAR-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
         & PDHTSS(JROF,JLEV,JVAR)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHVTSS*NCSNEC
ENDIF

! Soil energy budget

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVTTS
    DO JLEV=1,YSP_SBD%NLEVS
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JVAR-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
!         &PDHTTS(JROF,JLEV,JVAR)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JVAR-1)*YSP_SBD%NLEVS+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JVAR-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
         & PDHTTS(JROF,JLEV,JVAR)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHVTTS*YSP_SBD%NLEVS
ENDIF

! Sea-ice energy budget

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVTIS
    DO JLEV=1,NCSI
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JVAR-1)*NCSI+JLEV) = PVFCI(JROF)*&
!         &PDHTIS(JROF,JLEV,JVAR)&
!         &+PVFCI(JROF)*REAL(IDHCS+(JVAR-1)*NCSI+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JVAR-1)*NCSI+JLEV) = PVFCI(JROF)*&
         & PDHTIS(JROF,JLEV,JVAR)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHVTIS*NCSI
ENDIF

! Snow water budget

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVSSS
    DO JLEV=1,NCSNEC
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JVAR-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
!         &PDHSSS(JROF,JLEV,JVAR)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JVAR-1)*NCSNEC+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JVAR-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
         & PDHSSS(JROF,JLEV,JVAR)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHVSSS*NCSNEC
ENDIF

! Interception layer water budget

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVIIS
    DO JROF=KSTART,KPROF
!cj
! temporarily add index
!      PDHCS(JROF,IDHCS+JVAR) = ZLSM(JROF)*PDHIIS(JROF,JVAR)&
!       &+ZLSM(JROF)*REAL(IDHCS+JVAR)
!cj
      PDHCS(JROF,IDHCS+JVAR) = ZLSM(JROF)*PDHIIS(JROF,JVAR)
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHVIIS
ENDIF

! Soil water budget

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVWLS
    DO JLEV=1,YSP_SBD%NLEVS
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JVAR-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
!         &PDHWLS(JROF,JLEV,JVAR)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JVAR-1)*YSP_SBD%NLEVS+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JVAR-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
         & PDHWLS(JROF,JLEV,JVAR)  
      ENDDO
    ENDDO
  ENDDO
ENDIF

!     FREE_STYLE VARIABLES
!     --------------------

IDHCS = NDHVS

IF ( LHDHKS ) THEN
  DO JVAR=1,NDHVFS
    DO JROF=KSTART,KPROF
      PDHCS(JROF,IDHCS+JVAR) = PDHFSS(JROF,JVAR)
    ENDDO
  ENDDO
ENDIF

!     FLUX EN SURFACE
!     ---------------

IDHCS = NDHVS + NDHVFS + NDHFSD

!     Standard fluxes

! Tiles

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFTLS
    DO JTILE=1,NTILES
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JFLX-1)*NTILES+JTILE) = &
!         &PDHTLS(JROF,JTILE,NDHVTLS+JFLX)&
!         &+REAL(IDHCS+(JFLX-1)*NTILES+JTILE)
!cj
        PDHCS(JROF,IDHCS+(JFLX-1)*NTILES+JTILE) = &
         & PDHTLS(JROF,JTILE,NDHVTLS+JFLX)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHFTLS*NTILES
ENDIF

! Snow energy budget

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFTSS
    DO JLEV=1,NCSNEC
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JFLX-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
!         &PDHTSS(JROF,JLEV,NDHVTSS+JFLX)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JFLX-1)*NCSNEC+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JFLX-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
         & PDHTSS(JROF,JLEV,NDHVTSS+JFLX)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHFTSS*NCSNEC
ENDIF

! Soil energy budget

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFTTS
    DO JLEV=1,YSP_SBD%NLEVS
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JFLX-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
!         &PDHTTS(JROF,JLEV,NDHVTTS+JFLX)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JFLX-1)*YSP_SBD%NLEVS+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JFLX-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
         & PDHTTS(JROF,JLEV,NDHVTTS+JFLX)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHFTTS*YSP_SBD%NLEVS
ENDIF

! Sea-ice energy budget

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFTIS
    DO JLEV=1,NCSI
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JFLX-1)*NCSI+JLEV) = PVFCI(JROF)*&
!         &PDHTIS(JROF,JLEV,NDHVTIS+JFLX)&
!         &+PVFCI(JROF)*REAL(IDHCS+(JFLX-1)*NCSI+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JFLX-1)*NCSI+JLEV) = PVFCI(JROF)*&
         & PDHTIS(JROF,JLEV,NDHVTIS+JFLX)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHFTIS*NCSI
ENDIF

! Snow water budget

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFSSS
    DO JLEV=1,NCSNEC
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JFLX-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
!         &PDHSSS(JROF,JLEV,NDHVSSS+JFLX)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JFLX-1)*NCSNEC+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JFLX-1)*NCSNEC+JLEV) = ZLSM(JROF)*&
         & PDHSSS(JROF,JLEV,NDHVSSS+JFLX)  
      ENDDO
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHFSSS*NCSNEC
ENDIF

! Interception layer water budget

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFIIS
    DO JROF=KSTART,KPROF
!cj
! temporarily add index
!      PDHCS(JROF,IDHCS+JFLX) = ZLSM(JROF)*PDHIIS(JROF,NDHVIIS+JFLX)&
!       &+ZLSM(JROF)*REAL(IDHCS+JFLX)
!cj
      PDHCS(JROF,IDHCS+JFLX) = ZLSM(JROF)*PDHIIS(JROF,NDHVIIS+JFLX)
    ENDDO
  ENDDO
  IDHCS = IDHCS + NDHFIIS
ENDIF

! Soil water budget

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFWLS
    DO JLEV=1,YSP_SBD%NLEVS
      DO JROF=KSTART,KPROF
!cj
! temporarily add index
!        PDHCS(JROF,IDHCS+(JFLX-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
!         &PDHWLS(JROF,JLEV,NDHVWLS+JFLX)&
!         &+ZLSM(JROF)*REAL(IDHCS+(JFLX-1)*YSP_SBD%NLEVS+JLEV)
!cj
        PDHCS(JROF,IDHCS+(JFLX-1)*YSP_SBD%NLEVS+JLEV) = ZLSM(JROF)*&
         & PDHWLS(JROF,JLEV,NDHVWLS+JFLX)  
      ENDDO
    ENDDO
  ENDDO
ENDIF

! FREE_STYLE FLUXES

IDHCS = NDHVS + NDHVFS + NDHFSD + NDHFSP

IF ( LHDHKS ) THEN
  DO JFLX=1,NDHFFS
    DO JROF=KSTART,KPROF
      PDHCS(JROF,IDHCS+JFLX) = PDHFSS(JROF,NDHVFS+JFLX)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPHDDHE',1,ZHOOK_HANDLE)
END SUBROUTINE CPPHDDHE
