SUBROUTINE CPXFU(KST,KND,KSTC,KENDC,KSTGLO,LDRESET,&
 & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFPLCL, PFPLCN,  PFPLCG, PFPLCH,&
 & PFPLSL, PFPLSN, PFPLSG, PFPLSH, PFRSO,&
 & PFRTH  ,PCLCT,  PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV, PNEB,&
 & PQICE  ,PQLI,&
 & PXUCLS, PXVCLS, PXTCLS, PXQCLS, PXRHCLS,PCLCH,  PCLCM,  PCLCL,  PCLCC,PFTHW,&
 & PFONTE, PFCHSP, PFLWSP, PFEVL,  PRUISP, PRUISS, PCCAPE, PCMOCON,PXCLPH,&
 & PXUGST ,PXVGST, PFEVV,  PFTR,   PRUISL )

!**** *CPXFU* - INTERFACE FOR INSTANTANEOUS FLUXES.

!     Purpose.
!     --------
!           DIAGNOSTICS OF PHYSICAL FLUXES.

!**   Interface.
!     ----------
!        *CALL* *CPXFU*

!        Explicit arguments :
!        --------------------

!       NPROMA                 - HORIZONTAL DIMENSION                 (INPUT)
!       KGL1,KGL2              - KGL1 TO KGL2 LATITUDES OF WORK.      (INPUT)
!       KSTABUF                - KSTABUF(JGL) FOR LATITUDE JGL        (INPUT)
!       FLUXES COMING FROM THE PHYSICAL PARAMETERIZATIONS             (INPUT)

!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        O. CLARY

!     Modifications.
!     --------------
!        Original : 06/92 D'APRES CUMCO
!        Modified : 18/08/93 R. El khatib (PBL temperature extrema)
!        Modified : 09/09/93 R. El khatib (Partial cloudiness)
!        Modified : 07/10/94 EB and DG (surface fluxes)
!        Modified : 13/03/97 JPh. Piedelievre (Distributed Memory)
!      R. El Khatib : 00-03-22 indexes of convection and PBL calculations
!        Modified : 00-03 by R. El Khatib - derived types
!        Modified : 01-08-07 by R. El Khatib - Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        Y.Bouteloup   26-Mars-2004 introduction of MTS radiance
!        F.Bouyssel    10-Nov-2005 introduction of PQLI,PQICE
!        Y.Seity       15-Jan-2007 introduction of Graupel and Hail
!        Y.Seity       30-Jan-2008 introduction of Theta'w' flux
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMXFU   , ONLY : NFDXFU   ,NTYPXFU  ,TYPE_XFU
USE YOMXFUB  , ONLY : XFUBUF
USE PTRXFU   , ONLY : MXTRDU   ,MXTRDV   ,MXTRCU   ,MXTRCV   ,MXDICQ   ,&
 & MXDICS   ,MXTRTU   ,MXTRTV   ,MXDITQ   ,MXDITS   ,MXPLCL   ,&
 & MXPLSL   ,MXPLCN   ,MXPLSN   ,MXPLCG   ,MXPLSG   ,MXPLCH   ,&
 & MXPLSH   ,MXRSO    ,MXRTH    ,MXNEB    ,&
 & MXNEBT   ,MXFEVL   ,MXFONTE  ,MXFCHSP  ,MXFEVV   ,MXFLWSP  ,&
 & MXFTR    ,MXRUISL  ,MXRUISP  ,MXRUISS  ,MXNBCON  ,MXNBHAU  ,&
 & MXNBMOY  ,MXCAPE   ,MXMOCON  ,MXNBBAS  ,MXCLPH   ,&
 & MXUGST   ,MXVGST   ,MXUCLS   ,MXVCLS   ,MXTCLS   ,MXQCLS   ,&
 & MXRHCLS  ,MXTN     ,MXTX     ,MXHUN    ,MXHUX    ,MXTHW    ,&
 & MXQICE   ,MXQLI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KENDC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDRESET 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCG(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCH(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSG(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSH(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEB(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQICE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXUCLS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXVCLS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTCLS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXQCLS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXRHCLS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCC(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFONTE(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTHW(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCHSP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLWSP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCAPE(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCMOCON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXCLPH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXUGST(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXVGST(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVV(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTR(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISL(NPROMA) 
!     local array

REAL(KIND=JPRB) :: ZGFU(NPROMA,NFDXFU)

INTEGER(KIND=JPIM) :: IJLEV, JLEV, JF, IFLD, IOFF, IADD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"

!     ------------------------------------------------------------------

!*    1.    READ IN INST. FIELDS

!*    1.1   READ IN FIELDS

IF (LHOOK) CALL DR_HOOK('CPXFU',0,ZHOOK_HANDLE)
IF (.NOT.LDRESET) THEN
  CALL SC2RDG(XFUBUF,KSTGLO,KSTC,KENDC,ZGFU)
ELSE
  ZGFU=0.0_JPRB
ENDIF

!     ------------------------------------------------------------------

!*    2.    COPY FLUXES
!           -----------

IOFF=0
DO JF=1,NTYPXFU
  IADD=1+IOFF
  DO JLEV=1,TYPE_XFU(JF)%ILEV

    IFLD=JLEV+IOFF
    IJLEV=TYPE_XFU(JF)%INUMLEV(JLEV)

    IF (IADD == MXTRDU) THEN
      ZGFU(KST:KND,IFLD)=PSTRDU(KST:KND,IJLEV) 
    ELSEIF (IADD == MXTRDV) THEN
      ZGFU(KST:KND,IFLD)=PSTRDV(KST:KND,IJLEV) 
    ELSEIF (IADD == MXTRCU) THEN
      ZGFU(KST:KND,IFLD)=PSTRCU(KST:KND,IJLEV) 
    ELSEIF (IADD == MXTRCV) THEN
      ZGFU(KST:KND,IFLD)=PSTRCV(KST:KND,IJLEV) 
    ELSEIF (IADD == MXDICQ) THEN
      ZGFU(KST:KND,IFLD)=PDIFCQ(KST:KND,IJLEV) 
    ELSEIF (IADD == MXDICS) THEN
      ZGFU(KST:KND,IFLD)=PDIFCS(KST:KND,IJLEV) 
    ELSEIF (IADD == MXTRTU) THEN
      ZGFU(KST:KND,IFLD)=PSTRTU(KST:KND,IJLEV) 
    ELSEIF (IADD == MXTRTV) THEN
      ZGFU(KST:KND,IFLD)=PSTRTV(KST:KND,IJLEV) 
    ELSEIF (IADD == MXDITQ) THEN
      ZGFU(KST:KND,IFLD)=PDIFTQ(KST:KND,IJLEV) 
    ELSEIF (IADD == MXDITS) THEN
      ZGFU(KST:KND,IFLD)=PDIFTS(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLCL) THEN
      ZGFU(KST:KND,IFLD)=PFPLCL(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLCN) THEN
      ZGFU(KST:KND,IFLD)=PFPLCN(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLCG) THEN
      ZGFU(KST:KND,IFLD)=PFPLCG(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLCH) THEN
      ZGFU(KST:KND,IFLD)=PFPLCH(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLSL) THEN
      ZGFU(KST:KND,IFLD)=PFPLSL(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLSN) THEN
      ZGFU(KST:KND,IFLD)=PFPLSN(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLSG) THEN
      ZGFU(KST:KND,IFLD)=PFPLSG(KST:KND,IJLEV) 
    ELSEIF (IADD == MXPLSH) THEN
      ZGFU(KST:KND,IFLD)=PFPLSH(KST:KND,IJLEV) 
    ELSEIF (IADD == MXRSO) THEN
      ZGFU(KST:KND,IFLD)=PFRSO(KST:KND,IJLEV) 
    ELSEIF (IADD == MXRTH) THEN
      ZGFU(KST:KND,IFLD)=PFRTH(KST:KND,IJLEV) 
    ELSEIF (IADD == MXNEB) THEN
      ZGFU(KST:KND,IFLD)=PNEB(KST:KND,IJLEV) 
    ELSEIF (IADD == MXQICE) THEN
      ZGFU(KST:KND,IFLD)=PQICE(KST:KND,IJLEV) 
    ELSEIF (IADD == MXQLI) THEN
      ZGFU(KST:KND,IFLD)=PQLI(KST:KND,IJLEV) 
    ELSEIF (IADD == MXNEBT) THEN
      ZGFU(KST:KND,IFLD)=PCLCT(KST:KND) 
    ELSEIF (IADD == MXFEVL) THEN
      ZGFU(KST:KND,IFLD)=PFEVL(KST:KND) 
    ELSEIF (IADD == MXFONTE) THEN
      ZGFU(KST:KND,IFLD)=PFONTE(KST:KND) 
    ELSEIF (IADD == MXFCHSP) THEN
      ZGFU(KST:KND,IFLD)=PFCHSP(KST:KND) 
    ELSEIF (IADD == MXFLWSP) THEN
      ZGFU(KST:KND,IFLD)=PFLWSP(KST:KND) 
    ELSEIF (IADD == MXRUISS) THEN
      ZGFU(KST:KND,IFLD)=PRUISS(KST:KND) 
    ELSEIF (IADD == MXRUISP) THEN
      ZGFU(KST:KND,IFLD)=PRUISP(KST:KND) 
    ELSEIF (IADD == MXFEVV) THEN
      ZGFU(KST:KND,IFLD)=PFEVV(KST:KND) 
    ELSEIF (IADD == MXFTR) THEN
      ZGFU(KST:KND,IFLD)=PFTR(KST:KND) 
    ELSEIF (IADD == MXRUISL) THEN
      ZGFU(KST:KND,IFLD)=PRUISL(KST:KND) 
    ELSEIF (IADD == MXNBCON) THEN
      ZGFU(KST:KND,IFLD)=PCLCC(KST:KND) 
    ELSEIF (IADD == MXNBHAU) THEN
      ZGFU(KST:KND,IFLD)=PCLCH(KST:KND) 
    ELSEIF (IADD == MXNBMOY) THEN
      ZGFU(KST:KND,IFLD)=PCLCM(KST:KND) 
    ELSEIF (IADD == MXNBBAS) THEN
      ZGFU(KST:KND,IFLD)=PCLCL(KST:KND) 
    ELSEIF (IADD == MXCAPE) THEN
      ZGFU(KST:KND,IFLD)=PCCAPE(KST:KND) 
    ELSEIF (IADD == MXMOCON) THEN
      ZGFU(KST:KND,IFLD)=PCMOCON(KST:KND) 
    ELSEIF (IADD == MXCLPH) THEN
      ZGFU(KST:KND,IFLD)=PXCLPH(KST:KND) 
    ELSEIF (IADD == MXUGST) THEN
      ZGFU(KST:KND,IFLD)=PXUGST(KST:KND) 
    ELSEIF (IADD == MXVGST) THEN
      ZGFU(KST:KND,IFLD)=PXVGST(KST:KND) 
    ELSEIF (IADD == MXUCLS) THEN
      ZGFU(KST:KND,IFLD)=PXUCLS(KST:KND) 
    ELSEIF (IADD == MXVCLS) THEN
      ZGFU(KST:KND,IFLD)=PXVCLS(KST:KND) 
    ELSEIF (IADD == MXTCLS) THEN
      ZGFU(KST:KND,IFLD)=PXTCLS(KST:KND) 
    ELSEIF (IADD == MXQCLS) THEN
      ZGFU(KST:KND,IFLD)=PXQCLS(KST:KND) 
    ELSEIF (IADD == MXRHCLS) THEN
      ZGFU(KST:KND,IFLD)=MAX(0.0_JPRB,MIN(1.0_JPRB,PXRHCLS(KST:KND)))
    ELSEIF (IADD == MXTN) THEN
      IF (LDRESET) THEN
        ZGFU(KST:KND,IFLD)=100000._JPRB
      ENDIF
      ZGFU(KST:KND,IFLD)=MIN(ZGFU(KST:KND,IFLD),PXTCLS(KST:KND))
    ELSEIF (IADD == MXTX) THEN
      IF (LDRESET) THEN
        ZGFU(KST:KND,IFLD)=0.0_JPRB
      ENDIF
      ZGFU(KST:KND,IFLD)=MAX(ZGFU(KST:KND,IFLD),PXTCLS(KST:KND)) 
    ELSEIF (IADD == MXHUN) THEN
      IF (LDRESET) THEN
        ZGFU(KST:KND,IFLD)=1.0_JPRB
      ENDIF
      ZGFU(KST:KND,IFLD)=MIN(ZGFU(KST:KND,IFLD),PXRHCLS(KST:KND))
      ZGFU(KST:KND,IFLD)=MAX(0.0_JPRB,MIN(1.0_JPRB,ZGFU(KST:KND,IFLD)))
    ELSEIF (IADD == MXHUX) THEN
      IF (LDRESET) THEN
        ZGFU(KST:KND,IFLD)=0.0_JPRB
      ENDIF
      ZGFU(KST:KND,IFLD)=MAX(ZGFU(KST:KND,IFLD),PXRHCLS(KST:KND))
      ZGFU(KST:KND,IFLD)=MAX(0.0_JPRB,MIN(1.0_JPRB,ZGFU(KST:KND,IFLD)))
    ELSEIF (IADD == MXTHW) THEN  
      ZGFU(KST:KND,IFLD)=PFTHW(KST:KND) 
    ELSE
      CALL ABOR1('CPXFU : ERROR IN POINTERS') 
    ENDIF

  ENDDO
  IOFF=IOFF+TYPE_XFU(JF)%ILEV
ENDDO

!     ------------------------------------------------------------------

!*       3.    WRITE OUT FIELDS .
!              -----------------

!*    3.1   WRITE OUT FIELDS

CALL SC2WRG(XFUBUF,KSTGLO,KSTC,KENDC,ZGFU)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPXFU',1,ZHOOK_HANDLE)
END SUBROUTINE CPXFU
