SUBROUTINE CUMCOE(KPROMA,KSTA,KEND,KSTGLO,&
 & PSTRDU,PSTRTU,PSTRDV,PSTRTV,&
 & PFRTH ,PFRSO ,PFCLL ,PFCLN ,PFCS,&
 & PDIFTQ,&
 & PFPLCL,PFPLCN,PFPLSL,PFPLSN,&
 & PRUISS,PRUISP)  

!**** *CUMCOE* - CUMUL OF EC COUPLED FIELDS.

!     Purpose.
!     --------
!           DIAGNOSTICS OF PHYSICAL SURFACE FLUXES.

!**   Interface.
!     ----------
!        *CALL* *CUMCOE*

!        Explicit arguments :
!        --------------------

!       KPROMA         - HORIZONTAL DIMENSION                 (INPUT)
!       KSTA,KEND      - START AND END INDEX OF NPROMA CHUNK  (INPUT)
!       KSTGLO         - START ADDRESS OF CHUNK               (INPUT)
!       PSTRDU(KPROMA) - FLUX "GRAVITY WAVE DRAG" "U".        (INPUT)
!       PSTRTU(KPROMA) - FLUX TURBULENT DE QTE DE MVT "U".    (INPUT)
!       PSTRDV(KPROMA) - FLUX "GRAVITY WAVE DRAG" "V".        (INPUT)
!       PSTRTV(KPROMA) - FLUX TURBULENT DE QTE DE MVT "V".    (INPUT)
!       PFRTH (KPROMA,1) - FLUX DE RAYONNEMENT THERMIQUE.     (INPUT)
!       PFRSO (KPROMA,1) - FLUX DE RAYONNEMENT SOLAIRE.       (INPUT)
!       PFCLL (KPROMA,1) - FLUX DE CHALEUR LATENTE EAU.       (INPUT)
!       PFCLN (KPROMA,1) - FLUX DE CHALEUR LATENTE NEIGE      (INPUT)
!       PFCS  (KPROMA,1) - FLUX DE CHALEUR SENSIBLE           (INPUT)
!       PDIFTQ(KPROMA) - MOISTURE FLUX AT SURFACE             (INPUT)
!       PFPLCL(KPROMA) - PRECIP. CONVECTIVES LIQUIDES.        (INPUT)
!       PFPLCN(KPROMA) - PRECIP. CONVECTIVES NEIGEUSES.       (INPUT)
!       PFPLSL(KPROMA) - PRECIP. STRATIFORMES LIQUIDES.       (INPUT)
!       PFPLSN(KPROMA) - PRECIP. STRATIFORMES NEIGEUSES.      (INPUT)
!       PRUISS(KPROMA) - RUISSELLEMENT SURFACE                (INPUT)
!       PRUISP(KPROMA) - RUISSELLEMENT PROFONDEUR             (INPUT)

!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        T. Stockdale.    Based on CUMCO.

!     Modifications.
!     --------------
!   22.02.96  T Stockdale:  Introduce NSTABUF to allow S-Lagrangian
!   18.09.96  T.Stockdale:  Corrected version of Cy15r2 changes
!   08.09.98  T.Stockdale:  Working version for LMESSP=true
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDYN   , ONLY : TSTEP
USE YOMGCO   , ONLY : STRSU    ,STRSV    ,FCHAS    ,FRSOS    ,&
 & FHUMS    ,RUIST    ,FCHLL    ,FCHLN    ,FCHSS    ,&
 & FHUML    ,FHUMN  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)                  :: PSTRDU(KPROMA) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(KPROMA) 
REAL(KIND=JPRB)                  :: PSTRDV(KPROMA) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(KPROMA,1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(KPROMA,1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLL(KPROMA,1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLN(KPROMA,1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCS(KPROMA,1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISP(KPROMA) 
INTEGER(KIND=JPIM) :: IBEG, IL, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CUMCOE',0,ZHOOK_HANDLE)
IBEG=KSTGLO-1
DO JROF=KSTA,KEND
  IL=IBEG+JROF
  STRSU(IL)  =STRSU(IL)  + PSTRTU(JROF)*TSTEP
  STRSV(IL)  =STRSV(IL)  + PSTRTV(JROF)*TSTEP
  RUIST(IL)  =RUIST(IL)  +(PRUISS(JROF)+PRUISP(JROF))*TSTEP
  FHUML(IL)  =FHUML(IL)  +(PFPLCL(JROF)+PFPLSL(JROF))*TSTEP
  FHUMN(IL)  =FHUMN(IL)  +(PFPLCN(JROF)+PFPLSN(JROF))*TSTEP
  FRSOS(IL,1)=FRSOS(IL,1)+ PFRSO(JROF,1)*TSTEP
  FCHLL(IL,1)=FCHLL(IL,1)+ PFCLL(JROF,1)*TSTEP
  FCHLN(IL,1)=FCHLN(IL,1)+ PFCLN(JROF,1)*TSTEP
  FCHSS(IL,1)=FCHSS(IL,1)+ PFCS (JROF,1)*TSTEP
  FCHAS(IL,1)=FCHAS(IL,1)&
   & +(PFRTH(JROF,1)+PFCLL(JROF,1)+PFCLN(JROF,1)&
   & +PFCS(JROF,1))*TSTEP  
  FHUMS(IL,1)=FHUMS(IL,1)&
   & +(PFPLCL(JROF)+PFPLCN(JROF)+PFPLSL(JROF)+PFPLSN(JROF)&
   & + PDIFTQ(JROF))*TSTEP  
ENDDO

IF (LHOOK) CALL DR_HOOK('CUMCOE',1,ZHOOK_HANDLE)
END SUBROUTINE CUMCOE
