SUBROUTINE GPTCNORM(LDGPTCNORM,KU,KF,KDGLN,KDGLX,KPROM,KDLSUR,&
 & KROF0,KLONG,CDF,PF)  

!**** *GPTCNORM*   - PRINTS NORMS ON GRID-POINT VALUES OF TRANSMISSION
!                    COEFFICIENTS

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *GPTCNORM(.....)*

!        Explicit arguments :
!        --------------------
!        * INPUT:
!          LDGPTCNORM      - controls which norms have to be printed.
!          KU              - logical output unit.
!          KF              - pseudo-vertical dimension of array PF.
!          KDGLN           - lower DM-global latitude for the current
!                            grid-point packet.
!          KDGLX           - upper DM-global latitude for the current
!                            grid-point packet.
!          KPROM           - grid-point dimension.
!          KDLSUR          - length > max (KLONG(jgl)).
!          KROF0           - start address for each latitude.
!          KLONG           - length for each latitude.
!          CDF             - field identifier.
!          PF              - field, statistics on which have to be done.

!        * OUTPUT:

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------
!       none

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP). 

!     Modifications.
!     --------------
!        Original : FEB 1999
!        Modified 99-07-20 by K. YESSAD: protect SQRT to avoid argument<0.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROM 
LOGICAL           ,INTENT(IN)    :: LDGPTCNORM(3) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KU 
INTEGER(KIND=JPIM)               :: KDLSUR ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KROF0(KDGLN:KDGLX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLONG(KDGLN:KDGLX) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PF(KPROM,1:KF) 
INTEGER(KIND=JPIM) :: ILONG
INTEGER(KIND=JPIM) :: ILONT
INTEGER(KIND=JPIM) :: IROF0
INTEGER(KIND=JPIM) :: JF
INTEGER(KIND=JPIM) :: JGL
INTEGER(KIND=JPIM) :: JLON

LOGICAL :: LLGPTCNORM

REAL(KIND=JPRB) :: ZF(KDLSUR,KDGLN:KDGLX,1:KF)
REAL(KIND=JPRB) :: ZMOY
REAL(KIND=JPRB) :: ZMOY2(KDGLN:KDGLX)
REAL(KIND=JPRB) :: ZMOY3(1:KF)
REAL(KIND=JPRB) :: ZSTD
REAL(KIND=JPRB) :: ZSTD2(KDGLN:KDGLX)
REAL(KIND=JPRB) :: ZSTD3(1:KF)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

! 0/ Preliminary prints:

IF (LHOOK) CALL DR_HOOK('GPTCNORM',0,ZHOOK_HANDLE)
LLGPTCNORM=LDGPTCNORM(2).OR.LDGPTCNORM(3)

IF (LLGPTCNORM) THEN
  CALL FLUSH(KU)
  WRITE (KU,*)
  WRITE (KU,'(1X,A)') ' ===  STATISTICS IN GPTCNORM === '
  WRITE (KU,*)
ENDIF

! ------------------------------------------------------------------

! 1/ Memory transfer:

LLGPTCNORM=LDGPTCNORM(2).OR.LDGPTCNORM(3)

IF (LDGPTCNORM(1).OR.LLGPTCNORM) THEN
  ILONT=0
  DO JGL=KDGLN,KDGLX
    IROF0=KROF0(JGL)
    ILONG=KLONG(JGL)
    ILONT=ILONT+ILONG
    ZF(1:ILONG,JGL,1:KF)=PF(IROF0+1:IROF0+ILONG,1:KF)
  ENDDO
ENDIF

! ------------------------------------------------------------------

! 2/ Global statistics:

IF (LDGPTCNORM(1)) THEN
! * average ZMOY and standard deviation ZSTD:
  ZMOY3(1:KF)=0.0_JPRB
  ZMOY=0.0_JPRB
  ZSTD3(1:KF)=0.0_JPRB
  ZSTD=0.0_JPRB
  DO JGL=KDGLN,KDGLX
    DO JLON=1,KLONG(JGL)
      ZMOY3(1:KF)=ZMOY3(1:KF)+ZF(JLON,JGL,1:KF)
      ZSTD3(1:KF)=ZSTD3(1:KF)+ZF(JLON,JGL,1:KF)*ZF(JLON,JGL,1:KF)
    ENDDO
  ENDDO
  DO JF=1,KF
    ZMOY=ZMOY+ZMOY3(JF)
    ZSTD=ZSTD+ZSTD3(JF)
  ENDDO
  ZMOY=ZMOY/(REAL(KF,JPRB)*REAL(ILONT,JPRB))
  ZSTD=ZSTD/(REAL(KF,JPRB)*REAL(ILONT,JPRB)) - ZMOY*ZMOY
  ZSTD=MAX(0.0_JPRB,ZSTD)
  ZSTD=SQRT(ZSTD)
  WRITE(KU,'(1X,A,A,A,I4,A,I4,A,E20.14,A,E20.14)') &
   & ' - GLOBAL AVERAGE AND ST. DEV. OF FIELD ',CDF, &
   & ' BETWEEN LAT ',KDGLN,' AND ',KDGLX,': ', &
   & ZMOY,', ',ZSTD  
ENDIF

! ------------------------------------------------------------------

! 3/ Statistics per latitude:

IF (LDGPTCNORM(2)) THEN
! * average ZMOY2 and standard deviation ZSTD2:
  ZMOY2(KDGLN:KDGLX)=0.0_JPRB
  ZSTD2(KDGLN:KDGLX)=0.0_JPRB
  DO JGL=KDGLN,KDGLX
    ZMOY3(1:KF)=0.0_JPRB
    ZSTD3(1:KF)=0.0_JPRB
    DO JLON=1,KLONG(JGL)
      ZMOY3(1:KF)=ZMOY3(1:KF)+ZF(JLON,JGL,1:KF)
      ZSTD3(1:KF)=ZSTD3(1:KF)+ZF(JLON,JGL,1:KF)*ZF(JLON,JGL,1:KF)
    ENDDO
    DO JF=1,KF
      ZMOY2(JGL)=ZMOY2(JGL)+ZMOY3(JF)
      ZSTD2(JGL)=ZSTD2(JGL)+ZSTD3(JF)
    ENDDO
  ENDDO
  DO JGL=KDGLN,KDGLX
    ZMOY2(JGL)=ZMOY2(JGL)/(REAL(KF,JPRB)*REAL(KLONG(JGL),JPRB))
    ZSTD2(JGL)=ZSTD2(JGL)/(REAL(KF,JPRB)*REAL(KLONG(JGL),JPRB)) &
     & -ZMOY2(JGL)*ZMOY2(JGL)  
    ZSTD2(JGL)=MAX(0.0_JPRB,ZSTD2(JGL))
  ENDDO
  ZSTD2(KDGLN:KDGLX)=SQRT(ZSTD2(KDGLN:KDGLX))
  WRITE(KU,*)
  DO JGL=KDGLN,KDGLX
    WRITE(KU,'(1X,A,A,A,I4,A,E20.14,A,E20.14)') &
     & ' - AVERAGE AND ST. DEV. OF FIELD ',CDF, &
     & ' FOR LATITUDE ',JGL,': ', &
     & ZMOY2(JGL),', ',ZSTD2(JGL)  
  ENDDO
ENDIF

! ------------------------------------------------------------------

! 4/ Statistics per layer:

IF (LDGPTCNORM(3)) THEN
! * average ZMOY3 and standard deviation ZSTD3:
  ZMOY3(1:KF)=0.0_JPRB
  ZSTD3(1:KF)=0.0_JPRB
  DO JGL=KDGLN,KDGLX
    DO JLON=1,KLONG(JGL)
      ZMOY3(1:KF)=ZMOY3(1:KF)+ZF(JLON,JGL,1:KF)
      ZSTD3(1:KF)=ZSTD3(1:KF)+ZF(JLON,JGL,1:KF)*ZF(JLON,JGL,1:KF)
    ENDDO
  ENDDO
  ZMOY3(1:KF)=ZMOY3(1:KF)/REAL(ILONT,JPRB)
  ZSTD3(1:KF)=ZSTD3(1:KF)/REAL(ILONT,JPRB) - ZMOY3(1:KF)*ZMOY3(1:KF)
  DO JF=1,KF
    ZSTD3(JF)=MAX(0.0_JPRB,ZSTD3(JF))
  ENDDO
  ZSTD3(1:KF)=SQRT(ZSTD3(1:KF))
  WRITE(KU,*)
  DO JF=1,KF
    WRITE(KU,'(1X,A,A,A,I4,A,I4,A,I4,A,E20.14,A,E20.14)') &
     & ' - AVERAGE AND ST. DEV. OF FIELD ',CDF, &
     & ' BETWEEN LAT ',KDGLN,' AND ',KDGLX, &
     & ' FOR LAYER ',JF,': ', &
     & ZMOY3(JF),', ',ZSTD3(JF)  
  ENDDO
ENDIF

! ------------------------------------------------------------------

! 5/ End prints:

IF (LLGPTCNORM) THEN
  WRITE (KU,*)
  WRITE (KU,'(1X,A)') ' ===  END STATISTICS IN GPTCNORM === '
  WRITE (KU,*)
  CALL FLUSH(KU)
ENDIF

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTCNORM',1,ZHOOK_HANDLE)
END SUBROUTINE GPTCNORM
