#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
SUBROUTINE GRIB_CODE_MESSAGE(KGRIB_HANDLE,KGRIBCD,KLEV,CDREPR,CDLTYPE,&
 & KSMAX,LDGRAD,KTOP,KBOT)
USE PARKIND1 , ONLY : JPIM     ,JPRB    ,JPIB
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : CTYPE,NSTEPINI,LOBSC1,LFPOS,NCONF
USE SURFACE_FIELDS_MIX  , ONLY : YSP_SB, YSP_SBD
USE YOMDIM   , ONLY : NSMAX, NSPEC2G
USE YOMGRB   , ONLY : NSTEPLPP ,&
 & NLOCGRB  ,&
 & NGRBMX2T ,NGRBMN2T ,NGRB10FG ,&
 & NGRBMX2T6,NGRBMN2T6,NGRB10FG6,&
 & NLEG     ,NGRBBTMP ,NREFERENCE
USE YOMFPG   , ONLY : NFPMAX
USE YOMDYN   , ONLY : TSTEP
USE YOMCT3   , ONLY : NSTEP
USE YOMVAR   , ONLY : NUPTRA, MUPTRA, LTWGRA, LTWINC, LTWBGV, LTWCGL,&
 & NFGFCLEN, LBGM
USE YOMSKF   , ONLY : LTWSKFY, LTWSKFZ
USE YOMANEB  , ONLY : NANERADS
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE GRIB_API_INTERFACE, ONLY : IGRIB_SET_VALUE
USE YOMLCZ   , ONLY : NRITZREAD, NRITZNUMB, NRITZGBH1, NRITZGBH2
USE YOMVAREPS, ONLY : NFCHO_TRUNC_INI, NFCLENGTH_INI, LVAREPS
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, NCOMP_MODERR
USE YOMDYNCORE, ONLY: LPPSTEPS
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS, NSTADD, NSTASS
USE YOMLUN   , ONLY : NULOUT
USE YOMECEARTH, ONLY : LECEARTH,LECEARTH_UNCOUP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGRIB_HANDLE
INTEGER(KIND=JPIM),INTENT(IN)    :: KGRIBCD
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
CHARACTER         ,INTENT(IN)    :: CDREPR*(*)
CHARACTER         ,INTENT(IN)    :: CDLTYPE*(*)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: KSMAX 
LOGICAL           ,INTENT(OUT)   :: LDGRAD
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTOP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KBOT

INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

INTEGER(KIND=JPIM) :: ISPEC2, ILSI, ILSQ, ILST, ISTEPLPP, ISMAX
INTEGER(KIND=JPIM) :: ISTEP
INTEGER(KIND=JPIB) :: ISEC, ISECLPP, ISECSTART, ISEC0, ISECOVERLAP
INTEGER(KIND=JPIM) :: IJPIM

REAL(KIND=JPRB) :: ZBOT, ZTOP
REAL(KIND=JPRB) :: ZRDAT(YSP_SBD%NLEVS),ZRDAW(YSP_SBD%NLEVS)
INTEGER(KIND=JPIM) :: IFREQS(NANERADS)

LOGICAL :: LLFPOSRES,LLBAFERROR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! for EC-EARTH
INTEGER(KIND=JPIM) :: IH0,IJ0,IM0,IA0,IDD,ISS,IHR,IMIN,ISC,IJOUR,IMOIS,IAN,ILMOIS(12)
INTEGER(KIND=JPIM) :: IINDAT,ITIME
INTEGER(KIND=JPIB) :: ISECLPPE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "fcttim.h"
#include "updcalsec.intfb.h"

IF (LHOOK) CALL DR_HOOK('GRIB_CODE_MESSAGE',0,ZHOOK_HANDLE)
LLFPOSRES = LFPOS
IF(LLFPOSRES) THEN
  ISMAX = NFPMAX(1)
  ISPEC2 = (ISMAX+1)*(ISMAX+2)
ELSE
  ISPEC2 = NSPEC2G
  ISMAX = NSMAX
ENDIF

IF (PRESENT(KSMAX)) THEN
  IF(CDREPR == 'SH') THEN
    ISMAX=KSMAX
    ISPEC2=(KSMAX+1)*(KSMAX+2)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'pentagonalResolutionParameterJ',ISMAX)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'pentagonalResolutionParameterK',ISMAX)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'pentagonalResolutionParameterM',ISMAX)
  ENDIF
ENDIF

LLBAFERROR = CTYPE == 'ef' .OR. CTYPE == 'ea'
IF(CTYPE == 'eb') CALL ABOR1('GRIB_CODE_MESSAGE : Unknown type -  eb ')

CALL SURF_INQ(PRDAT=ZRDAT,PRDAW=ZRDAW)
!*       1.1   TIME INDICATORS

IF( LPPSTEPS ) THEN
  ISEC  = NSTEP*3600._JPRB  
ELSE
  ISEC  = NSTEP*TSTEP
ENDIF

IF(CTYPE == 'fc'.AND.LOBSC1) THEN
  ISEC=ISEC+NSTEPINI*3600
ENDIF

! Check if ISEC is still in the JPIM range. If not (for long runs), some of the
! later IGRIB_SET_VALUE calls will have corrupted arguments. Hence, issue a
! warning. It is hoped that testing only ISEC here is sufficient.
IF(ISEC>HUGE(IJPIM)) THEN
    WRITE(NULOUT,*) 'GRIB_CODE_MESSAGEW: WARNING: ISEC out of JPIM range!'
ENDIF

!* CHANGE GRIB HEADERS TO WRITE FC IF VAREPS

!   Consider a VAREPS system with two legs: ModelA and modelB

!           t=0              ISEC0  ISECOVERLAP
!   ModelA:  |-----------------|----x----|

!                             t=0  ISEC
!   ModelB:                    |----x-------------------|------------|
!
!   In ModelB, to define correctly the time-step in the grib-header,
!   ISEC is shifted by ISEC0. Then, if ISEC is less than ISECOVERLAP
!   data are written in the overlap stream efov (1034-1032), otherwise
!   in strem enfo (1035-1033).

IF(LVAREPS.AND.(NLEG >= 2)) THEN
  ISEC0=NFCHO_TRUNC_INI*3600
  ISEC =ISEC0+ISEC
  ISECOVERLAP=NFCLENGTH_INI*3600
  IF(ISEC <= ISECOVERLAP .AND. NLEG == 2 ) THEN
    IF(NREFERENCE > 0 ) THEN
      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stream',1032)
    ELSE
      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stream',1034)
    ENDIF
  ENDIF
ELSE
  ISEC0=0
ENDIF

ECEARTH: IF (.NOT.(LECEARTH.OR.LECEARTH_UNCOUP)) THEN
CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'timeRangeIndicator',0)
IF(LLBAFERROR) THEN
  ISTEP=0
  IF(CTYPE == 'ef') THEN
    ISTEP=NFGFCLEN
  ELSEIF(CTYPE /= 'eb') THEN
    IF(LBGM) ISTEP=NFGFCLEN
  ENDIF
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepType','instant')
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepUnits','h')
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'endStep',ISTEP)
ELSEIF(ISEC<= 0) THEN
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepType','instant')
  IF(CTYPE /= 'an') THEN
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'timeRangeIndicator',1)
  ENDIF
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepUnits','s')
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'endStep',INT(ISEC,JPIM))
ELSE
  IF(KGRIBCD == NGRBMX2T.OR.KGRIBCD == NGRBMN2T.OR.KGRIBCD == NGRB10FG) THEN
!       these parameters apply to time since the last post-processing step
    ISTEPLPP = NSTEPLPP(KGRIBCD)
    ISECLPP  = ISTEPLPP*TSTEP+ISEC0
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepUnits','s')
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepType','max')
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'startStep',INT(ISECLPP,JPIM))
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'endStep',INT(ISEC,JPIM))
    NSTEPLPP(KGRIBCD) = NSTEP
  ELSEIF(KGRIBCD == NGRBMX2T6.OR.KGRIBCD == NGRBMN2T6.OR.KGRIBCD == NGRB10FG6) THEN
!       these parameters apply to the last 6 hours
    IF (MOD(ISEC,3600) /= 0) CALL ABOR1('GRIB_CODE_MESSAGE : 6 hour min/min only on whole hours ')
    ISECSTART  = MAX(ISEC-6*3600,ISEC0)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepUnits','s')
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepType','max')
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'startStep',INT(ISECSTART,JPIM))
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'endStep',INT(ISEC,JPIM))
  ELSE
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepUnits','s')
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepType','instant')
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'endStep',INT(ISEC,JPIM))
  ENDIF
ENDIF

ELSE ECEARTH

! test setting for a new ref dat for EC-EARTH
  IJ0=NDD(NINDAT)
  IM0=NMM(NINDAT)
  IA0=NCCAA(NINDAT)
  IH0=NSSSSS/3600_JPIM

  ISTEPLPP = NSTEP
  ISECLPPE  = INT(ISTEPLPP,KIND(ISECLPPE))*INT(TSTEP,KIND(ISECLPPE))+ISEC0
  IDD=INT(ISECLPPE/86400_JPIM,JPIM)
  ISS=INT(MOD(ISECLPPE,86400_JPIM),JPIM)
  CALL UPDCALSEC(IH0,IJ0,IM0,IA0,IDD,ISS,IHR,IMIN,ISC,IJOUR,IMOIS,IAN,ILMOIS,NULOUT)

  IINDAT=10000*IAN+100*IMOIS+IJOUR
  ITIME=IHR*100+IMIN

  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'timeRangeIndicator',0)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'date',IINDAT)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'time',ITIME)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepUnits','s')
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'stepType','instant')
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'startStep',0)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'endStep',0)
! may neeed more adjustments for avg/max/min
ENDIF ECEARTH


!*        1.2   LEVEL(S)
KTOP=-1
KBOT=-1
IF(CDLTYPE(1:2) == 'SF') THEN
  IF(YSP_SBD%NUMFLDS > 0) THEN
    ILST = ISRCHEQ(YSP_SBD%NLEVS,YSP_SB%YT%IGRBCODE(:),1,KGRIBCD)
    IF(ILST <= YSP_SBD%NLEVS) THEN
      ZBOT     = SUM(ZRDAT(1:ILST))
      ZTOP     = ZBOT-ZRDAT(ILST)
      KTOP     = MIN( ZTOP*100.0_JPRB+0.1_JPRB , 255._JPRB)
      KBOT     = MIN( ZBOT*100.0_JPRB+0.1_JPRB , 255._JPRB)
    ENDIF
  ENDIF
  IF(YSP_SBD%NUMFLDS > 1) THEN
    ILSQ = ISRCHEQ(YSP_SBD%NLEVS,YSP_SB%YQ%IGRBCODE(:),1,KGRIBCD)
    IF(ILSQ <= YSP_SBD%NLEVS) THEN
      ZBOT     = SUM(ZRDAW(1:ILSQ))
      ZTOP     = ZBOT-ZRDAW(ILSQ)
      KTOP     = MIN( ZTOP*100.0_JPRB+0.1_JPRB , 255._JPRB)
      KBOT     = MIN( ZBOT*100.0_JPRB+0.1_JPRB , 255._JPRB)
    ENDIF
  ENDIF
  IF(YSP_SBD%NUMFLDS > 2) THEN
    ILSI = ISRCHEQ(YSP_SBD%NLEVS,YSP_SB%YTL%IGRBCODE(:),1,KGRIBCD)
    IF(ILSI <= YSP_SBD%NLEVS) THEN
      ZBOT     = SUM(ZRDAW(1:ILSI))
      ZTOP     = ZBOT-ZRDAW(ILSI)
      KTOP     = MIN( ZTOP*100.0_JPRB+0.1_JPRB , 255._JPRB)
      KBOT     = MIN( ZBOT*100.0_JPRB+0.1_JPRB , 255._JPRB)
    ENDIF
  ENDIF
ENDIF
!        1.3   PARAMETER


!*        1.7    ECMWF LOCAL GRIB
IF(NLOCGRB == 1) THEN
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',CTYPE)
!!$ IF(LLBAFERROR) THEN
!!$    IF(CTYPE == 'ef') THEN
!!$      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',CTYPE)
!!$    ELSE
!!$      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type','ea')
!!$    ENDIF
!!$  ELSE
!!$    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',CTYPE)
!!$    IF(CTYPE == 'an') THEN
!!$      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',CTYPE)
!!$    ELSEIF(CTYPE == '3v') THEN
!!$      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',CTYPE)
!!$    ELSEIF(CTYPE == '4v') THEN
!!$      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',CTYPE)
!!$    ELSE
!!$      CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',9)
!!$    ENDIF
!!$  ENDIF
ENDIF

IF(NLOCGRB == 9) THEN       ! Singular vectors
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'forecastOrSingularVectorNumber',NRITZNUMB)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'numberOfSingularVectorsComputed',NRITZREAD)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'NINT_LOG10_RITZ',NRITZGBH1)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'NINT_RITZ_EXP',NRITZGBH2)
ENDIF

IF(LLBAFERROR) THEN
  IF(KGRIBCD == NGRBBTMP) THEN
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'localDefinitionNumber',14)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'channelNumber',KLEV)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'scalingFactorForFrequencies',1)
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'numberOfFrequencies',NANERADS)
    IFREQS(:) = 0 !Dummy values
    CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'listOfScaledFrequencies',IFREQS)
  ENDIF
ENDIF

IF(CTYPE == 'me') THEN
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'localDefinitionNumber',25)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',35)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'number',1)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'numberOfComponents',NCOMP_MODERR)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'modelErrorType',NTYPE_MODERR)
ENDIF
  

IF (NCONF==131 .AND. LTWINC) THEN
!!$  KSEC1(37) = 20 DOES NOT EXIST
!!$  KSEC1(39) = 33 ?????
!!$  KSEC1(42) = NUPTRA
!!$  KSEC1(43) = MUPTRA
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'localDefinitionNumber',20)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'type',33)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'iteration',NUPTRA)
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'totalNumberOfIterations',MUPTRA)
ENDIF
IF (.NOT.LECMWF) THEN
  ! remark KY+GD: this seems necessary for use at METEO-FRANCE.
  CALL IGRIB_SET_VALUE(KGRIB_HANDLE,'generatingProcessIdentifier',1)
ENDIF
LDGRAD=LTWGRA .OR. LTWINC .OR. LTWBGV .OR. LTWCGL .OR. LTWSKFY .OR. LTWSKFZ

IF (LHOOK) CALL DR_HOOK('GRIB_CODE_MESSAGE',1,ZHOOK_HANDLE)
END SUBROUTINE GRIB_CODE_MESSAGE
