SUBROUTINE GRIDPOINT_NORM(YDGRID,CDGREP,LDLEVS)

!     Purpose
!     -------
!       Computes norm of distributed grid point fields.

!     Author
!     ------
!       Y. Tremolet

!     Modifications
!     -------------
!       16-Jan-04: Original
!       G.Mozdzynski  19-Sept-2008 gpnorm optimisations
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULOUT
USE YOMMP   , ONLY: MYPROC
USE GRIDPOINT_FIELDS_MIX, ONLY: GRIDPOINT_FIELD

IMPLICIT NONE

TYPE(GRIDPOINT_FIELD), INTENT(IN) :: YDGRID
CHARACTER(LEN=*), INTENT(IN) :: cdgrep
LOGICAL, OPTIONAL, INTENT(IN) :: LDLEVS

REAL(KIND=JPRB), ALLOCATABLE :: ZGP(:,:,:), znorms(:,:)
REAL(KIND=JPRB) :: zave, zmin, zmax
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,jn,joff,jlen,iflds
LOGICAL :: llevs
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------
#include "gpnorm1.intfb.h"
! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_NORM',0,ZHOOK_HANDLE)

iflds=YDGRID%NG3D*YDGRID%NFLEVG+YDGRID%NG2D
llevs=.true.
IF (PRESENT(LDLEVS)) llevs=LDLEVS

IF (iflds>0) THEN
  ALLOCATE(ZGP(YDGRID%NPROMA,iflds,YDGRID%NGPBLKS))
  ALLOCATE(znorms(3,iflds))

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,joff,jlen,jn,jf,jl,jj)
  DO jb=1,YDGRID%NGPBLKS
    joff=(jb-1)*YDGRID%NPROMA
    jlen=MIN(YDGRID%NPROMA,YDGRID%NGPTOT-joff)
    jn=0
    DO jf=1,YDGRID%NG3D
      DO jl=1,YDGRID%NFLEVG
        jn=jn+1
        DO jj=1,jlen
          ZGP(jj,jn,jb)=YDGRID%GP3D(jj,jl,jf,jb)
        ENDDO
      ENDDO
    ENDDO
    DO jf=1,YDGRID%NG2D
      jn=jn+1
      DO jj=1,jlen
        ZGP(jj,jn,jb)=YDGRID%GP2D(jj,jf,jb)
      ENDDO
    ENDDO
  ENDDO
!$OMP END PARALLEL DO

  CALL GPNORM1(ZGP,iflds,llevs,PNORMS=znorms)

  IF (MYPROC==1) THEN
    WRITE(NULOUT,*)cdgrep,' Grid Point Norms'
    IF (llevs) THEN
      WRITE(NULOUT,'(5(1X,A))') 'FLD','LEV', &
        & '       AVERAGE       ','       MINIMUM       ','       MAXIMUM       '
      jn=0
      DO jf=1,YDGRID%NG3D
        DO jl=1,YDGRID%NFLEVG
          jn=jn+1
          WRITE(NULOUT,999)YDGRID%NGRIB3(jf),jl,znorms(:,jn)
        ENDDO
      ENDDO
      jl=YDGRID%NFLEVG
      DO jf=1,YDGRID%NG2D
        jn=jn+1
        WRITE(NULOUT,999)YDGRID%NGRIB2(jf),jl,znorms(:,jn)
      ENDDO
999   FORMAT(2(1X,I3),3(1X,ES21.14))
    ELSE
      WRITE(NULOUT,'(4(1X,A))') 'FLD', &
        & '       AVERAGE       ','       MINIMUM       ','       MAXIMUM       '
      jn=0
      DO jf=1,YDGRID%NG3D
        zave=SUM(znorms(1,jn+1:jn+YDGRID%NFLEVG))/YDGRID%NFLEVG
        zmin=MINVAL(znorms(2,jn+1:jn+YDGRID%NFLEVG))
        zmax=MAXVAL(znorms(3,jn+1:jn+YDGRID%NFLEVG))
        jn=jn+YDGRID%NFLEVG
        WRITE(NULOUT,888)YDGRID%NGRIB3(jf),zave,zmin,zmax
      ENDDO
      DO jf=1,YDGRID%NG2D
        jn=jn+1
        WRITE(NULOUT,888)YDGRID%NGRIB2(jf),znorms(:,jn)
      ENDDO
888   FORMAT(1X,I3,3(1X,ES21.14))
    ENDIF
  ENDIF

  DEALLOCATE(ZGP,znorms)
ENDIF

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_NORM',1,ZHOOK_HANDLE)
END SUBROUTINE GRIDPOINT_NORM
