SUBROUTINE INIFAOUT(CDCONF,CDINC4,KUNIT)

!**** *INIFAOUT*  - Program to write ARPEGE files

!     Purpose.
!     --------
!           WRITE an ARPEGE file

!        Explicit arguments :
!        --------------------
!     Externals.
!     ----------

!         FAITOU
!         FANDAR    (see manipulation de Fichier ARPEGE)

!     Author.
!     -------
!        MPP Group *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 96-01-01
!        Original : 98-03-13 Remove LSPLITOUT case
!        D. Giard : 98-09-07 Save grid-point fields for finalisation
!       Modified  99-10-20  R. El Khatib : use suppdate
!        C. Fischer:99-05-19 debug ltwlcz key for nconf=601
!        D. Giard : 99-07-19 LINITER moved from YOMNMIA to YOMINI
!        D. Giard : 00-07-11 new temporary file for finalisation
!    R. El Khatib : 01-03-16 IDATEF(10)=time of previous event
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   R. El Khatib : 05-02-21 specific frequency array to reset instantaneous
!                  fluxes
!   R. El Khatib : 23-Oct-2008 use suofname
!   R. El Khatib : 15-Sep-2008 I/O savings
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NTRJSH   ,NSCRTCH   
USE YOMMP    , ONLY : MYPROC   ,NSTROUT
USE YOMOP    , ONLY : CNMCA    ,CFNRF    ,CFNAN    ,CFNGR    ,&
 & CFANS    ,LINC
USE YOMDIM   , ONLY : NFLEVG   ,NS3D     ,NS2D
USE YOMVAR   , ONLY : NSIM4D   ,LTRREF   ,LTWANA   ,LTWGRA   ,LTWLCZ
USE YOMCT0   , ONLY : NCONF    ,LOBSC1   ,LELAM    ,LFBDAP
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE SURFACE_FIELDS_MIX  , ONLY : NPROGSURF
USE YOMINI   , ONLY : LINITER
USE YOMXFU   , ONLY : NFRRAZ   ,NRAZTS   ,JPFUXT   ,N1RAZ    ,LXFU

IMPLICIT NONE

CHARACTER(LEN=1), INTENT(IN)  :: CDCONF 
CHARACTER(LEN=4), INTENT(OUT) :: CDINC4

INTEGER(KIND=JPIM),INTENT(OUT)   :: KUNIT

CHARACTER(LEN=214) :: CLFCFIC
CHARACTER(LEN=217) :: CLFIC
CHARACTER(LEN=5) :: CLR1, CLNSTEP
CHARACTER(LEN=1) :: CL
CHARACTER(LEN=3) :: CLPROC, CLSIM4D
CHARACTER(LEN=4) :: CLR2
CHARACTER(LEN=16) :: CLSCRTCH

INTEGER(KIND=JPIM) :: IDATEF(11)

INTEGER(KIND=JPIM) :: IMTS, INBARI, INBARP, INIMES, IREP, IDTIME
INTEGER(KIND=JPIM) :: ISTEP, IINC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "suppdate.intfb.h"
#include "suofname.intfb.h"

#include "fcttim.h"

IF (LHOOK) CALL DR_HOOK('INIFAOUT',0,ZHOOK_HANDLE)
WRITE(CLNSTEP,'(I5.4)') NSTEP
WRITE(CLSIM4D,'(I3.3)') NSIM4D

!*           OPEN FILES : REFERENCE TRAJECTORY OR CURRENT TRAJECTORY
!            -------------------------------------------------------

!*           OPEN FILES.
!            -----------

IF (LTRREF) THEN

  CLR1 = CFNRF(1:5)
  CLR2 = CFNRF(6:9)
  IF (NSTEP < 0) THEN
    CLFCFIC=CLR1//CLR2//CLNSTEP
  ELSE
    IF (LINC) THEN
      IINC=NINT(REAL(NSTEP,JPRB)*TSTEP/3600._JPRB)
    ELSE
      IINC=NSTEP
    ENDIF
    WRITE(CDINC4,'(I4.4)') IINC
    CLFCFIC=CLR1//CLR2//CDINC4
  ENDIF

  WRITE(NULOUT,'('' WRITE TRAJECTORY ON FILE '',A16)')TRIM(CLFCFIC)
  INBARP = NFLEVG * NS3D + NS2D
  INIMES = 0
  INBARI = 0

ELSEIF(LTWANA) THEN

  IF (NSTEP < 0) THEN
    CL='-'
    IINC=ABS(NSTEP)
  ELSEIF (NCONF/100 == 1.OR.LOBSC1) THEN
    CL=CFNAN(10:10)
    IMTS=NINT(REAL(NSTEP,JPRB)*TSTEP/60._JPRB)
    IINC=(IMTS/60)*100+MOD(IMTS,60)
  ELSE
    CL=CFNAN(10:10)
    IINC=NSTEP
  ENDIF
  WRITE(CDINC4,'(I4.4)') IINC
  CLFCFIC=CFNAN(1:6)//CLSIM4D//CL//CDINC4

  WRITE(NULOUT,'('' WRITE ANALYSIS ON FILE '',A16)')TRIM(CLFCFIC)
  INBARP = NFLEVG * NS3D + NS2D
  INIMES = 0
  INBARI = 0

ELSEIF(LTWGRA) THEN

  IF (NSTEP < 0) THEN
    CL='-'
    IINC=ABS(NSTEP)
  ELSEIF (NCONF/100 == 1) THEN
    CL=CFNGR(10:10)
    IMTS=NINT(REAL(NSTEP,JPRB)*TSTEP/60._JPRB)
    IINC=(IMTS/60)*100+MOD(IMTS,60)
  ELSE
    CL=CFNGR(10:10)
    IINC=NSTEP
  ENDIF
  WRITE(CDINC4,'(I4.4)') IINC
  CLFCFIC=CFNGR(1:6)//CLSIM4D//CL//CDINC4

  WRITE(NULOUT,'('' WRITE GRADIENT ON FILE '',A16)')TRIM(CLFCFIC)
  INBARP = NFLEVG * NS3D + NS2D
  INIMES = 0
  INBARI = 0

ELSEIF (LTWLCZ) THEN

  CLFCFIC="SVARPE000+0000"
  IF (NSTEP < 0) THEN
    CLFCFIC(10:10)='-'
    IINC=ABS(NSTEP)
  ELSEIF (NCONF/100 == 6) THEN
    IMTS=NINT(REAL(NSTEP,JPRB)*TSTEP/60._JPRB)
    IINC=(IMTS/60)*100+MOD(IMTS,60)
  ELSE
    IINC=NSTEP
  ENDIF
  WRITE(CLFCFIC(7:9),'(I3.3)') NSIM4D
  WRITE(CLFCFIC(11:14),'(I4.4)') IINC
  WRITE(NULOUT,'('' WRITE SV '',I2, '' ON FILE '',A16)')NSIM4D,CLFCFIC
  INBARP = NFLEVG * NS3D + NS2D
  INIMES = 0
  INBARI = 0

ELSEIF (LINITER) THEN

  CLFCFIC= CFANS
  INBARP = NPROGSURF + NFLEVG * NS3D + NS2D
  INIMES = 0
  INBARI = 0

ELSE

  IF (NSTEP < 0) THEN
    CALL SUOFNAME(KSTEP=NSTEP,CDLABEL='ICMSH',CDOFNAME=CLFCFIC)
  ELSE
    IF (CDCONF == 'P'.OR.CDCONF == 'X') THEN
      ISTEP=NSTEP-1
    ELSE
      ISTEP=NSTEP
    ENDIF
    CALL SUOFNAME(KSTEP=ISTEP,CDLABEL='ICMSH',CDOFNAME=CLFCFIC,CDINC=CDINC4)
  ENDIF

  INBARP = NFLEVG * NS3D + NS2D
  INIMES = 0
  INBARI = 0

ENDIF

WRITE(CLPROC,'(I3.3)') MYPROC
IF (MYPROC /= 1) THEN
  CLFIC=TRIM(CLFCFIC)//'_'//CLPROC
ELSE
  CLFIC=CLFCFIC
ENDIF

IF ( MYPROC == 1 ) THEN

  KUNIT=NTRJSH
  CALL FAITOU(IREP,NTRJSH,.TRUE.,CLFIC,'UNKNOWN',&
   & .TRUE.,.TRUE., INIMES,INBARP,INBARI,CNMCA)  
  CALL LFIMST(IREP,NTRJSH,.FALSE.)

!*           DECIDE ON SUITABLE TIME UNIT.
!            -----------------------------

  IDTIME=0
  IF (LFBDAP.AND.LXFU) THEN
    CALL SUPPDATE(IDTIME,NRAZTS,JPFUXT,NFRRAZ,N1RAZ,IDATEF)
  ELSE
    CALL SUPPDATE(KDTIME=IDTIME,KDIME=1,KPPDATE=IDATEF)
  ENDIF
  CALL FANDAR(IREP,NTRJSH,IDATEF)

!  IDENTIFICATEUR DES FICHIERS D EBAUCHES

  IF (LELAM) THEN
    CALL FAUTIF(IREP,NTRJSH,'ALD')
  ELSE
    CALL FAUTIF(IREP,NTRJSH,'ARP')
  ENDIF

ELSEIF ( MYPROC <= NSTROUT ) THEN

  KUNIT=NSCRTCH
  CALL FANOUV(IREP,KUNIT,.FALSE.,CLSCRTCH,'NEW',.TRUE.,.TRUE.,1,1,INBARI,&
   & CNMCA)

ELSE

  KUNIT=-9

ENDIF

IF (LHOOK) CALL DR_HOOK('INIFAOUT',1,ZHOOK_HANDLE)
END SUBROUTINE INIFAOUT
