SUBROUTINE INIPGO(KBLOCKS,KNBARP)

!**** *INIPGO*  - DDH output : inititialize pseudo-grib coding

!     Purpose.
!     --------
!     Compute space needed for pseudo grib coding of DDH.
!     Allocate space for real and integer blocks for subsequent
!     pseudo grib coding and output of DDH data.
!     Reset pointers and counters used for coding.

!**   Interface.
!     ----------
!        *CALL* *INIPGO(..)*

!        Explicit arguments :     KBLOCKS - number of blocks : global=1
!        --------------------                                  zonal =NDHKD
!                                                              masks =NDHNOM
!                                 KNBARP  - number of records

!        Implicit arguments :      None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.   SUALPGO - memory manager allocation
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-03-01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMMDDH  , ONLY : NDHCVSU  ,NDHCSSU  ,NDHVV    ,NDHVS
USE YOMPGO   , ONLY : NPTCH    ,NPTIN    ,NPTRE    ,NPTIND   ,&
 & NLENNAM  ,NLENRBL2 ,NLENREA  ,NLENIND  ,NLENINT  ,&
 & NLENCHA  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOCKS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNBARP 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sualpgo.intfb.h"

!     ------------------------------------------------------------------

!*       1.  COMPUTE DIMENSIONS NEEDED - ALLOCATE SPACE.
!            -------------------------------------------

IF (LHOOK) CALL DR_HOOK('INIPGO',0,ZHOOK_HANDLE)
NLENREA=KBLOCKS*((NFLEVG+1)*(NDHCVSU+NDHVV)+(NDHCSSU+NDHVS))+1000
NLENIND=KNBARP
NLENINT=NLENIND+1000
NLENCHA=6000
NLENNAM=12

CALL SUALPGO

!     ------------------------------------------------------------------

!*       2.  RESET POINTERS.
!            ---------------

NPTCH=1
NPTIN=1
NPTRE=1
NPTIND=0
NLENRBL2=0

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('INIPGO',1,ZHOOK_HANDLE)
END SUBROUTINE INIPGO
