SUBROUTINE PPCLOSE

!**** *PPCLOSE*  - Close post processing data files

!     Purpose.
!     --------
!     Close post processing data files at end of a post processing
!     timestep or at end of model integration

!        Explicit arguments :     
!        --------------------    

!     Method.
!     -------
!        See documentation

!     Externals.         PB_CLOSE -
!     ----------         MPL_WRITE -
!                        MPL_CLOSE -

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        George Mozdzynski *ECMWF*

!     Modifications.
!     --------------
!       Original : 97-10-15 
!       Modified : 00-12-08 G.Mozdzynski: MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MYPROC
USE YOMIO    , ONLY : NPPFILES ,NPPBUFL  ,NPPBUFH  ,NPPBUFS  ,&
 & NPPBUF2  ,NPPBUFA  ,RPPBUFWAT,CPPLFN   ,NPPOUTTYPE  
USE YOMLUN   , ONLY : NULOUT
USE MPL_MODULE

IMPLICIT NONE

CHARACTER (LEN = 24) ::  CLFN
INTEGER(KIND=JPIM) :: IDUM(1)

INTEGER(KIND=JPIM) :: IBUF, IERR, IOP, IREQ, IU, J, JA

REAL(KIND=JPRB) :: Z0, Z1
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!     LOOP OVER OPEN FILES

IF (LHOOK) CALL DR_HOOK('PPCLOSE',0,ZHOOK_HANDLE)
CALL GSTATS(1738,0)
DO J=1,NPPFILES
  IF( NPPBUFS(J) == 1 )THEN
    IBUF=J
    CLFN=CPPLFN(IBUF)
    IU=NPPBUFH(IBUF)
    IF( NPPOUTTYPE(IBUF) == 1 )THEN
      IF( MYPROC == 1 )THEN
        CALL PBCLOSE(IU,IERR)
        IF( IERR < 0 )THEN
          WRITE(NULOUT,'("PPCLOSE: ERROR ON PBCLOSE, IERR=",I6,&
           & " FILE=",A)')IERR,CLFN  
          CALL ABOR1('PPCLOSE: ERROR ON PBCLOSE')
        ENDIF
      ENDIF
    ELSEIF( NPPOUTTYPE(IBUF) == 3.OR. NPPOUTTYPE(IBUF) == 4 )THEN
      CALL MPL_CLOSE(IU,IERR)
      IF( IERR /= 0 )THEN
        WRITE(NULOUT,'("PPCLOSE: PROBLEM ON MPL_CLOSE OF FILE - ",&
         & A," IERR=",I6)')CLFN,IERR  
        CALL ABOR1('PPCLOSE: PROBLEM ON MPL_CLOSE')
      ENDIF
    ELSEIF( NPPOUTTYPE(IBUF) == 5.OR. NPPOUTTYPE(IBUF) == 6 )THEN

!           CHECK FOR ANY OUTSTANDING OUTPUT BY PERFORMING
!           MPI_WAIT (IOP=5) OR MPI_FILE_WRITE_ORDERED_END (IOP=6)

      CALL USER_CLOCK(PELAPSED_TIME=Z0)
      IOP=NPPOUTTYPE(IBUF)
      DO JA=1,2
        IF( NPPBUFA(2,JA,IBUF) == 1 )THEN
          IREQ=NPPBUFA(1,JA,IBUF)
          CALL MPL_WRITE(IU,IOP,IDUM,0,IREQ,IERR)
          IF( IERR /= 0 )THEN
            WRITE(NULOUT,'("PPCLOSE: PROBLEM ON MPL_WRITE ",&
             & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
            CALL ABOR1('PPCLOSE: PROBLEM ON MPL_WRITE')
          ENDIF
          NPPBUFA(1,JA,IBUF)=0
          NPPBUFA(2,JA,IBUF)=0
        ENDIF
      ENDDO
      CALL USER_CLOCK(PELAPSED_TIME=Z1)

!           UPDATE STATISTICS

      RPPBUFWAT(IBUF)=RPPBUFWAT(IBUF)+Z1-Z0

!           NOW CLOSE FILE

      CALL MPL_CLOSE(IU,IERR)
      IF( IERR /= 0 )THEN
        WRITE(NULOUT,'("PPCLOSE: PROBLEM ON MPL_CLOSE OF FILE - ",&
         & A," IERR=",I6)')CLFN,IERR  
        CALL ABOR1('PPCLOSE: PROBLEM ON MPL_CLOSE')
      ENDIF
    ELSE
      CALL ABOR1('PPCLOSE: NPPOUTTYPE INVALID')
    ENDIF

!         CLEAR ENTRY FOR CLOSED FILE

    CPPLFN(IBUF)=""
    NPPOUTTYPE(IBUF)=0
    NPPBUFL(IBUF)=0
    NPPBUFH(IBUF)=0
    NPPBUFS(IBUF)=0
    NPPBUF2(IBUF)=0
    NPPBUFA(:,:,IBUF)=0
  ENDIF
ENDDO
CALL GSTATS(1738,1)

IF (LHOOK) CALL DR_HOOK('PPCLOSE',1,ZHOOK_HANDLE)
END SUBROUTINE PPCLOSE
