SUBROUTINE PPEDDH (PDHNOM,PDDHCV,PDDHCS,CDNAMX,KDATEF)
!-----------------------------------------------------------------------
!**** PPEDDH * IMPRESSION DE BILANS EN MOYENNE VERTICALE POUR UN DOMAINE
!              DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!-----------------------------------------------------------------------
!     BUT
!     ---
!     EDITION SUR LISTING DES BILANS EN MOYENNE VERTICALE
!     DU DOMAINE IDENTIFIE PAR PDHNOM

!     ARGUMENTS D'ENTREE
!     ------------------
!      PDHNOM(11) : IDENTIFICATION DU DOMAINE
!                  PDHNOM(11) EST LE TYPE DE DOMAINE
!      PDDHCV(0:NFLEVG,NDHCV+NDHVV) : PROFILS VERTICAUX
!                  LA STRUCTURE DU TABLEAU EST RESUMEE DANS
!                  PPSYDH, SINON VOIR YOMTDDH ET CPDYDDH/CPPHDDH
!      PDDHCS(NDHCV+NDHVV) : SOL
!      CDNAMX*4 : NOM DE L EXPERIENCE
!      KDATEF(11) : TABLEAU DE DATE DECOMPOSEE

!     ARGUMENTS DE SORTIE
!     -------------------
!     LISTING, UNITE NULOUT

!     ORIGINAL
!     --------
!     91-02-01, Alain Joly.

!     MODIFICATIONS
!     -------------
!     92-03-01, M. Janousek: diagnostic de l'entropie.
!     92-06-01, M. Janousek: diagnostic du moment cinetique.
!     94-01-18, J.M. Piriou: impression du listing en flux
!                            et non plus energies.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT
USE YOMRIP   , ONLY : NSSSSS   ,RSTATI
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE YOMCST   , ONLY : RPI      ,ROMEGA   ,RLVTT    ,RLSTT    ,RLMLT
USE YOMLDDH  , ONLY : LHDHKS   ,LHDMCI   ,LHDENT
USE YOMMDDH  , ONLY : NDHCS    ,NDHCV    ,NDHCVSU  ,NDHCSSU  ,&
 & NDHVV    ,NDHFVD   ,NDHVS    ,NDHVFS   ,NDHVHK   ,&
 & NDHVMC   ,NDHAVD   ,NDHAVP   ,NDHAHKD  ,NDHBHKD  ,&
 & NDHAHKP  ,NDHBHKP  ,NDHAMCD  ,NDHBMCD  ,NDHBMCP  
USE YOMSDDH  , ONLY : NHDQLNVA ,NHDPASVA ,LHDQLN   ,LHDADD   ,LHDPAS

IMPLICIT NONE

!     ------------------------------------------------------------------
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHNOM(11) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDDHCV(0:NFLEVG,NDHCVSU+NDHVV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDDHCS(NDHCSSU+NDHVS) 
CHARACTER         ,INTENT(IN)    :: CDNAMX*4 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDATEF(11) 
CHARACTER :: CLDOM*20, CL24C*24, CL10C*10, CLOUT*10
REAL(KIND=JPRB) :: ZDHIV (NDHCV+NDHVV)

INTEGER(KIND=JPIM) :: IDHCS, IDHCV, ISEN, ISHK, ISMC, ITYPE, JCV, JJPASS, JLEV

REAL(KIND=JPRB) :: ZA10, ZA11, ZA1AD, ZA1C0, ZA1CS, ZA1D0, ZA1DH,&
 & ZA1DS, ZA1FC, ZA1FD, ZA1FG, ZA1FO, ZA1FP, &
 & ZA1FT, ZA1G0, ZA1GS, ZA1NA, ZA1O0, ZA1OS, &
 & ZA1P0, ZA1PS, ZA1T0, ZA1TS, ZA20, ZA21, ZA2AD, &
 & ZA2C0, ZA2CS, ZA2D0, ZA2DH, ZA2DS, ZA2FC, &
 & ZA2FD, ZA2FG, ZA2FO, ZA2FP, ZA2FT, ZA2G0, &
 & ZA2GS, ZA2NA, ZA2O0, ZA2OS, ZA2P0, ZA2PS, &
 & ZA2T0, ZA2TS, ZA30, ZA31, ZA3AD, ZA3C0, ZA3CS, &
 & ZA3D0, ZA3DH, ZA3DS, ZA3FC, ZA3FD, ZA3FG, &
 & ZA3FO, ZA3FP, ZA3FT, ZA3G0, ZA3GS, ZA3O0, &
 & ZA3OS, ZA3P0, ZA3PS, ZA3T0, ZA3TS, ZANM, &
 & ZBA1, ZBA2, ZBA3, ZBCOV, ZBCT, ZBEN, ZBKK, &
 & ZBMAR, ZBQL, ZBQN, ZBQV, ZCOVCT, ZCT0, ZCT1, &
 & ZCTCCL, ZCTCCL0, ZCTCCLS, ZCTCCN, ZCTCCN0, &
 & ZCTCCNS, ZCTCFON, ZCTCOP, ZCTCOV, ZCTCSL, &
 & ZCTCSL0, ZCTCSLS, ZCTCSN, ZCTCSN0, ZCTCSNS, &
 & ZCTDH, ZCTDISS, ZCTFD, ZCTFD0, ZCTFDS, ZCTFT, &
 & ZCTFT0, ZCTFTC, ZCTFTC0, ZCTFTCS, ZCTFTS, &
 & ZCTPCL, ZCTPCL0, ZCTPCLE, ZCTPCLF, ZCTPCLS, &
 & ZCTPCN, ZCTPCN0, ZCTPCNE, ZCTPCNF, ZCTPCNS, &
 & ZCTPSL, ZCTPSL0, ZCTPSLE, ZCTPSLF, ZCTPSLS, &
 & ZCTPSN, ZCTPSN0, ZCTPSNE, ZCTPSNF, ZCTPSNS, &
 & ZCTRAS, ZCTRAS0, ZCTRASS, ZCTRAT, ZCTRAT0, &
 & ZCTRATS, ZCTSFON, ZEN0, ZEN1, ZENCT, ZEND0, &
 & ZENDH, ZENDIC, ZENDID, ZENDISS, ZENDIT, ZENDS, &
 & ZENFD, ZENHUM, ZENP, ZENP0, ZENPCT, ZENPS, &
 & ZENRAD, ZEP0, ZEP1, ZEPDH, ZEPFD, ZEPFD0, &
 & ZEPFDS, ZERO, ZGMTSEC, ZHEURE, ZHR0, ZHR1, &
 & ZKK0, ZKK1, ZKKCOV, ZKKDH, ZKKDIC, ZKKDID, &
 & ZKKDIT, ZKKFD, ZKKFD0, ZKKFDM, ZKKFDM0, ZKKFDMS, &
 & ZKKFDS, ZLAT0, ZLFTT, ZLON0, ZLSTT, ZLVTT, &
 & ZMAR0, ZMAR1, ZMARDH, ZMARFD, ZMARFD0, ZMARFDS, &
 & ZMARFP, ZMARFP0, ZMARFPS, ZMNORM, ZNORM, &
 & ZNORMEQ, ZNT0, ZNT1, ZOM0, ZOM1, ZOUT, ZPASS0, &
 & ZPASS1, ZPTER, ZQL0, ZQL1, ZQLDH, ZQLFD, &
 & ZQLFNG, ZQLFT, ZQLFTC, ZQN0, ZQN1, ZQNDH, &
 & ZQNFD, ZQNFNG, ZQNFT, ZQNFTC, ZQNORM, ZQTCCL, &
 & ZQTCCN, ZQTCSL, ZQTCSN, ZQTPCL, ZQTPCL0, &
 & ZQTPCLE, ZQTPCLF, ZQTPCLS, ZQTPCN, ZQTPCN0, &
 & ZQTPCNE, ZQTPCNF, ZQTPCNS, ZQTPSL, ZQTPSL0, &
 & ZQTPSLE, ZQTPSLF, ZQTPSLS, ZQTPSN, ZQTPSN0, &
 & ZQTPSNE, ZQTPSNF, ZQTPSNS, ZQV0, ZQV1, ZQVCFON, &
 & ZQVDH, ZQVFD, ZQVFD0, ZQVFDS, ZQVFNG, ZQVFNG0, &
 & ZQVFNGS, ZQVFT, ZQVFT0, ZQVFTC, ZQVFTC0, &
 & ZQVFTCS, ZQVFTS, ZQVSFON, ZSBNS, ZSBTP, ZSBTS, &
 & ZSBWP, ZSBWS, ZSFCHSP, ZSFCLL, ZSFCLN, ZSFCS, &
 & ZSFEVL, ZSFEVN, ZSFLWSP, ZSFONTE, ZSFPLCL, &
 & ZSFPLCN, ZSFPLSL, ZSFPLSN, ZSFRSO, ZSFRTH, &
 & ZSLFONT, ZSNS0, ZSNS1, ZSRUISP, ZSRUISS, &
 & ZSSS, ZSTP0, ZSTP1, ZSTS0, ZSTS1, ZSWP0, &
 & ZSWP1, ZSWS0, ZSWS1, ZUPTER, ZUSSSS  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('PPEDDH',0,ZHOOK_HANDLE)
WRITE (NULOUT,*) ' '

!     ------------------------------------------------------------------
!**   0.- INITIALISATIONS DE SECURITE.
!         AFIN DE CONTOURNER LE MESSAGE DE WARNING PRODUIT PAR
!         LE LOGICIEL DE CONTROLE DES INITIALISATIONS DE VARIABLES
!         EN CAS DE VARIABLES INITIALISEES A L'INTERIEUR DE
!         TESTS IF-ENDIF, ON PORTE UNE VALEUR QUELCONQUE
!         DANS LESDITES VARIABLES.

ZLAT0=0.0_JPRB
ZLON0=0.0_JPRB

!     ------------------------------------------------------------------
!**   1.- IDENTIFICATION DE LA SORTIE

ITYPE=NINT(PDHNOM(11))
WRITE (NULOUT,*) ' DDH * DDH * DDH * DDH * DDH * DDH * DDH '
WRITE (NULOUT,*) ' '
WRITE (NULOUT,*) '      ---------------------------- DDH'
ZSSS=REAL(NSTEP,JPRB)*TSTEP
ZHEURE=ZSSS/3600._JPRB
ZUSSSS=1.0_JPRB/ZSSS
IF(ZHEURE > 72._JPRB) THEN
  ZOUT=.001_JPRB*NINT(ZHEURE/24._JPRB*1000._JPRB)
  CLOUT=' JOURS.'
ELSE
  ZOUT=.001_JPRB*NINT(ZHEURE*1000._JPRB)
  IF(ZOUT >= 2.0_JPRB) THEN
    CLOUT=' HEURES.'
  ELSE
    CLOUT=' HEURE.'
  ENDIF
ENDIF
WRITE (NULOUT,*) '       BILAN EN MOYENNE VERTICALE     ',&
 & ' EXPERIENCE ',CDNAMX,&
 & ' DATE INITIALE ',KDATEF(3),'/',KDATEF(2),'/',KDATEF(1),&
 & ' ECHEANCE : ',ZOUT,CLOUT  
WRITE (NULOUT,*) '      ---------------------------- DDH'
WRITE (NULOUT,*) ' '
IF ( ITYPE == 5 ) THEN
  WRITE (NULOUT,*) ' DDH * DOMAINE GLOBAL '
  WRITE (NULOUT,*) '               ------ '
  WRITE (CLDOM,'(A20)') 'GLOBAL '
ENDIF
IF ( ITYPE == 6 ) THEN
  WRITE (NULOUT,*) ' DDH * BANDE DE LATITUDE '
  WRITE (NULOUT,*) ' BANDE ',PDHNOM(2),' SUR ',PDHNOM(3),&
   & ' SINUS(LATITUDE GEO)=',PDHNOM(4)  
  WRITE (NULOUT,*) ' '
  WRITE (CLDOM,'(A8,F4.0,8X)') 'LATITUDE ',PDHNOM(2)
ENDIF
IF ( ITYPE == 1.OR. ITYPE == 4 ) THEN
  WRITE (NULOUT,*) ' DDH * POINT '
  WRITE (NULOUT,*) '      -------'
  WRITE (NULOUT,*) ' PLAN ',PDHNOM(1),' DOMAINE ',PDHNOM(2),&
   & ' INDICES JLON=',PDHNOM(5),' JGL=',PDHNOM(6)  
  WRITE (NULOUT,*) ' LONGITUDE GEO=',PDHNOM(3),&
   & ' SINUS(LATITUDE GEO)=',PDHNOM(4)  
  WRITE (NULOUT,*) ' '
  WRITE (CLDOM,'(A7,F5.0,A1,F5.0,A1,1X)')&
   & 'POINT (',PDHNOM(5),',',PDHNOM(6),')'  
ENDIF
IF ( ITYPE == 2.OR. ITYPE == 3 ) THEN
  WRITE (NULOUT,*) ' DDH * DOMAINE LIMITE '
  WRITE (NULOUT,*) '      ----------------'
  WRITE (NULOUT,*) ' PLAN ',PDHNOM(1),' DOMAINE ',PDHNOM(2)
  WRITE (NULOUT,5012) PDHNOM(3), PDHNOM(4), PDHNOM(5),&
   & PDHNOM(6), PDHNOM(7), PDHNOM(8), PDHNOM(9), PDHNOM(10)  
  WRITE (NULOUT,*) ' '
  WRITE (CLDOM,'(A8,F3.0,A1,F5.0,A1,2X)')&
   & 'LIMITE (',PDHNOM(1),',',PDHNOM(2),')'  
ENDIF
WRITE (NULOUT,*) ' DDH * BILAN A NSTEP=',NSTEP,&
 & ' SOIT ',ZSSS,' S ',' SOIT ',&
 & ZOUT,CLOUT  
WRITE (NULOUT,*) ' '
ISHK=0
ISMC=0
ISEN=0

!     ------------------------------------------------------------------
!**   2.- CALCUL DES MOYENNES VERTICALES INDIFFERENCIEES
!        OPERATION QUI A UN SENS SUR LES VARIABLES ET TENDANCES
!      MAIS NON SUR LES FLUX !

DO JCV=1, NDHCV+NDHVV
  ZDHIV(JCV)=0.0_JPRB
ENDDO
DO JLEV=0, NFLEVG
  DO JCV=1, NDHCV+NDHVV
    ZDHIV(JCV)=ZDHIV(JCV) + PDDHCV(JLEV,JCV)
  ENDDO
ENDDO

!     ------------------------------------------------------------------
!**   3.- IMPRESSION DU BILAN MASSES, ENERGIES, SOL

IDHCV=0

IF ( LHDHKS ) THEN
  ISHK=1
!    ----------------------
!*    BILAN DE MASSE D'AIR
!    ----------------------
  IDHCV=0
  ZMAR0=ZDHIV(IDHCV+1)

  IDHCV=NDHVV
  ZMARDH=ZDHIV(IDHCV+1)

  IDHCV=NDHVV + NDHAVD
  ZMARFD0=PDDHCV(0,IDHCV+1)
  ZMARFDS=PDDHCV(NFLEVG,IDHCV+1)
  ZMARFD=ZMARFD0 - ZMARFDS

  IDHCV=NDHVV + NDHFVD
  ZMARFP0=PDDHCV(0,IDHCV+1)
  ZMARFPS=PDDHCV(NFLEVG,IDHCV+1)
  ZMARFP=ZMARFP0 - ZMARFPS

  IDHCV=NDHCV

  ZMAR1=ZDHIV(IDHCV+1)
!     BILAN
  ZBMAR=ZMAR1 - ZMAR0 - ZMARDH - ZMARFD - ZMARFP

  ZMNORM=1.E+06_JPRB

!    -------------------------------
!*    BILAN DE MASSE DE VAPEUR D'EAU
!    -------------------------------
  IDHCV=0
  ZQV0=ZDHIV(IDHCV+2)

  IDHCV=NDHVV
  ZQVDH=ZDHIV(IDHCV+2)

  IDHCV=NDHVV + NDHAVD
  ZQVFD0=PDDHCV(0,IDHCV+2)
  ZQVFDS=PDDHCV(NFLEVG,IDHCV+2)
  ZQVFD=ZQVFD0 - ZQVFDS

  IDHCV=NDHVV + NDHFVD
  ZQVFT0=PDDHCV(0,IDHCV+2)
  ZQVFTS=PDDHCV(NFLEVG,IDHCV+2)
  ZQVFT=ZQVFT0 - ZQVFTS
  ZQVFTC0=PDDHCV(0,IDHCV+3)
  ZQVFTCS=PDDHCV(NFLEVG,IDHCV+3)
  ZQVFTC=ZQVFTC0 - ZQVFTCS
  ZQTPSL0=PDDHCV(0,IDHCV+4)
  ZQTPSLS=PDDHCV(NFLEVG,IDHCV+4)
  ZQTPSL=ZQTPSL0 - ZQTPSLS
  ZQTPSN0=PDDHCV(0,IDHCV+5)
  ZQTPSNS=PDDHCV(NFLEVG,IDHCV+5)
  ZQTPSN=ZQTPSN0 - ZQTPSNS
  ZQTPCL0=PDDHCV(0,IDHCV+6)
  ZQTPCLS=PDDHCV(NFLEVG,IDHCV+6)
  ZQTPCL=ZQTPCL0 - ZQTPCLS
  ZQTPCN0=PDDHCV(0,IDHCV+7)
  ZQTPCNS=PDDHCV(NFLEVG,IDHCV+7)
  ZQTPCN=ZQTPCN0 - ZQTPCNS
  ZQVFNG0=PDDHCV(0,IDHCV+8)
  ZQVFNGS=PDDHCV(NFLEVG,IDHCV+8)
  ZQVFNG=ZQVFNG0 - ZQVFNGS

  IDHCV=NDHCV
  ZQV1=ZDHIV(IDHCV+2)
!     BILAN
  ZBQV=ZQV1 - ZQV0 - ZQVDH - ZQVFD - ZQVFT -&
   & ZQVFTC - ZQTPSL - ZQTPSN - ZQTPCL - ZQTPCN - ZQVFNG  

!    --------------------------------
!*    BILAN D'EAU LIQUIDE ET NEIGEUSE
!    --------------------------------

  IF(LHDQLN) THEN

    IDHCV=0
    ZQL0=ZDHIV(IDHCV+9)
    ZQN0=ZDHIV(IDHCV+10)

    IDHCV=NDHCV
    ZQL1=ZDHIV(IDHCV+9)
    ZQN1=ZDHIV(IDHCV+10)

    IDHCV=NDHVV
    ZQLDH=ZDHIV(IDHCV+14)
    ZQNDH=ZDHIV(IDHCV+15)

    IDHCV=NDHVV + NDHAVD
    ZQLFD=PDDHCV(0,IDHCV+11)-PDDHCV(NFLEVG,IDHCV+11)
    ZQNFD=PDDHCV(0,IDHCV+12)-PDDHCV(NFLEVG,IDHCV+12)

    IDHCV=NDHVV + NDHFVD
    ZQLFT=PDDHCV(0,IDHCV+27)-PDDHCV(NFLEVG,IDHCV+27)
    ZQNFT=PDDHCV(0,IDHCV+28)-PDDHCV(NFLEVG,IDHCV+28)
    ZQLFTC=PDDHCV(0,IDHCV+29)-PDDHCV(NFLEVG,IDHCV+29)
    ZQNFTC=PDDHCV(0,IDHCV+30)-PDDHCV(NFLEVG,IDHCV+30)
    ZQLFNG=PDDHCV(0,IDHCV+31)-PDDHCV(NFLEVG,IDHCV+31)
    ZQNFNG=PDDHCV(0,IDHCV+32)-PDDHCV(NFLEVG,IDHCV+32)
    ZQTCCL=PDDHCV(0,IDHCV+33)-PDDHCV(NFLEVG,IDHCV+33)
    ZQTCCN=PDDHCV(0,IDHCV+34)-PDDHCV(NFLEVG,IDHCV+34)
    ZQTCSL=PDDHCV(0,IDHCV+35)-PDDHCV(NFLEVG,IDHCV+35)
    ZQTCSN=PDDHCV(0,IDHCV+36)-PDDHCV(NFLEVG,IDHCV+36)
  ELSE
    ZQL0=0.0_JPRB
    ZQN0=0.0_JPRB
    ZQL1=0.0_JPRB
    ZQN1=0.0_JPRB
    ZQLDH=0.0_JPRB
    ZQNDH=0.0_JPRB
    ZQLFD=0.0_JPRB
    ZQNFD=0.0_JPRB
    ZQLFT=0.0_JPRB
    ZQNFT=0.0_JPRB
    ZQLFTC=0.0_JPRB
    ZQLFTC=0.0_JPRB
    ZQNFTC=0.0_JPRB
    ZQLFNG=0.0_JPRB
    ZQNFNG=0.0_JPRB
    ZQTCCL=ZQTPCL
    ZQTCCN=ZQTPCN
    ZQTCSL=ZQTPSL
    ZQTCSN=ZQTPSN
  ENDIF

!     BILAN
  ZBQL=ZQL1-ZQL0-ZQLDH-ZQLFD-ZQLFT-ZQLFTC-ZQTPCL-ZQTPSL+ZQTCCL+ZQTCSL
  ZBQN=ZQN1-ZQN0-ZQNDH-ZQNFD-ZQNFT-ZQNFTC-ZQTPCN-ZQTPSN+ZQTCCN+ZQTCSN

!    -------------------------------
!*    BILAN D'ENERGIE CINETIQUE
!    -------------------------------
  IDHCV=0
  ZKK0=ZDHIV(IDHCV+5)

  IDHCV=NDHVV
  ZKKDH=ZDHIV(IDHCV+7)
  ZKKCOV=ZDHIV(IDHCV+8)

  IDHCV=NDHVV + NDHAVD
  ZKKFD0=PDDHCV(0,IDHCV+7)
  ZKKFDS=PDDHCV(NFLEVG,IDHCV+7)
  ZKKFD=ZKKFD0 - ZKKFDS
  ZKKFDM0=PDDHCV(0,IDHCV+8)
  ZKKFDMS=PDDHCV(NFLEVG,IDHCV+8)
  ZKKFDM=ZKKFDM0 - ZKKFDMS

  IDHCV=NDHVV + NDHFVD + NDHAVP
  ZKKDIT=ZDHIV(IDHCV+1)
  ZKKDIC=ZDHIV(IDHCV+2)
  ZKKDID=ZDHIV(IDHCV+3)

  IDHCV=NDHCV
  ZKK1=ZDHIV(IDHCV+5)
!     BILAN
  ZBKK=ZKK1 - ZKK0  - ZKKDH - ZKKCOV - ZKKFD - ZKKFDM -&
   & ZKKDIT - ZKKDIC - ZKKDID  

!    ------------------------------------
!*    BILAN D'ENERGIE INTERNE/POTENTIELLE
!    ------------------------------------
  IDHCV=0
  ZCT0=ZDHIV(IDHCV+6)
  ZEP0=ZDHIV(IDHCV+7)

  IDHCV=NDHVV
  ZCTDH=ZDHIV(IDHCV+9)
  ZCTCOV=ZDHIV(IDHCV+10)
  ZCTCOP=ZDHIV(IDHCV+11)
  ZEPDH=ZDHIV(IDHCV+12)

  IDHCV=NDHVV + NDHAVD
  ZCTFD0=PDDHCV(0,IDHCV+9)
  ZCTFDS=PDDHCV(NFLEVG,IDHCV+9)
  ZCTFD=ZCTFD0 - ZCTFDS
  ZEPFD0=PDDHCV(0,IDHCV+10)
  ZEPFDS=PDDHCV(NFLEVG,IDHCV+10)
  ZEPFD=ZEPFD0 - ZEPFDS

  IDHCV=NDHVV + NDHFVD
  ZCTFT0=PDDHCV(0,IDHCV+15)
  ZCTFTS=PDDHCV(NFLEVG,IDHCV+15)
  ZCTFT=ZCTFT0 - ZCTFTS
  ZCTFTC0=PDDHCV(0,IDHCV+16)
  ZCTFTCS=PDDHCV(NFLEVG,IDHCV+16)
  ZCTFTC=ZCTFTC0 - ZCTFTCS
  ZCTRAS0=PDDHCV(0,IDHCV+17)
  ZCTRASS=PDDHCV(NFLEVG,IDHCV+17)
  ZCTRAS=ZCTRAS0 - ZCTRASS
  ZCTRAT0=PDDHCV(0,IDHCV+18)
  ZCTRATS=PDDHCV(NFLEVG,IDHCV+18)
  ZCTRAT=ZCTRAT0 - ZCTRATS
  ZCTPSL0=PDDHCV(0,IDHCV+19)
  ZCTPSLS=PDDHCV(NFLEVG,IDHCV+19)
  ZCTPSL=ZCTPSL0 - ZCTPSLS
  ZCTPSN0=PDDHCV(0,IDHCV+20)
  ZCTPSNS=PDDHCV(NFLEVG,IDHCV+20)
  ZCTPSN=ZCTPSN0 - ZCTPSNS
  ZCTPCL0=PDDHCV(0,IDHCV+21)
  ZCTPCLS=PDDHCV(NFLEVG,IDHCV+21)
  ZCTPCL=ZCTPCL0 - ZCTPCLS
  ZCTPCN0=PDDHCV(0,IDHCV+22)
  ZCTPCNS=PDDHCV(NFLEVG,IDHCV+22)
  ZCTPCN=ZCTPCN0 - ZCTPCNS
  ZCTCCL0=PDDHCV(0,IDHCV+23)
  ZCTCCLS=PDDHCV(NFLEVG,IDHCV+23)
  ZCTCCL=ZCTCCL0 - ZCTCCLS
  ZCTCCN0=PDDHCV(0,IDHCV+24)
  ZCTCCNS=PDDHCV(NFLEVG,IDHCV+24)
  ZCTCCN=ZCTCCN0 - ZCTCCNS
  ZCTCSL0=PDDHCV(0,IDHCV+25)
  ZCTCSLS=PDDHCV(NFLEVG,IDHCV+25)
  ZCTCSL=ZCTCSL0 - ZCTCSLS
  ZCTCSN0=PDDHCV(0,IDHCV+26)
  ZCTCSNS=PDDHCV(NFLEVG,IDHCV+26)
  ZCTCSN=ZCTCSN0 - ZCTCSNS

  IDHCV=NDHCV
  ZCT1=ZDHIV(IDHCV+6)
  ZEP1=ZDHIV(IDHCV+7)
!     BILAN
  ZCTDISS=- ZKKDIT - ZKKDIC - ZKKDID
  ZBCT=ZCT1 - ZCT0 - ZCTDH - ZCTFD - ZCTCOV &
   & - ZCTRAS - ZCTRAT - ZCTFT - ZCTFTC &
   & - ZCTPCL - ZCTPCN - ZCTPSL - ZCTPSN &
   & - ZCTCCL - ZCTCCN - ZCTCSL - ZCTCSN &
   & - ZCTCOP - ZCTDISS  

!     BILAN DE LA CONVERSION INTERNE
  ZCOVCT=-ZCTCOV -ZCTCOP +ZEPDH +ZEPFD
  ZBCOV=ZKKCOV - ZCOVCT

!    ---------------------------------------
!*    BILANS ADDITIONNELS DE PRECIPITATIONS
!    ---------------------------------------

  IF(LHDADD) THEN
    IDHCV=NDHVV + NDHFVD + NDHAVP
!     STRATIFORME  LIQUIDE  FORMATION
    ZQTPSLF=ZDHIV(IDHCV+4 )
    ZCTPSLF=ZDHIV(IDHCV+5 )
!     STRATIFORME  LIQUIDE  EVAPORATION
    ZQTPSLE=ZDHIV(IDHCV+6 )
    ZCTPSLE=ZDHIV(IDHCV+7 )
!     STRATIFORME  SOLIDE   FORMATION
    ZQTPSNF=ZDHIV(IDHCV+8 )
    ZCTPSNF=ZDHIV(IDHCV+9 )
!     STRATIFORME  SOLIDE   EVAPORATION
    ZQTPSNE=ZDHIV(IDHCV+10)
    ZCTPSNE=ZDHIV(IDHCV+11)
!     STRATIFORME  FONTE
    ZQVSFON=ZDHIV(IDHCV+12)
    ZCTSFON=ZDHIV(IDHCV+13)
!     CONVECTIF    LIQUIDE  FORMATION
    ZQTPCLF=ZDHIV(IDHCV+14)
    ZCTPCLF=ZDHIV(IDHCV+15)
!     CONVECTIF    LIQUIDE  EVAPORATION
    ZQTPCLE=ZDHIV(IDHCV+16)
    ZCTPCLE=ZDHIV(IDHCV+17)
!     CONVECTIF    SOLIDE   FORMATION
    ZQTPCNF=ZDHIV(IDHCV+18)
    ZCTPCNF=ZDHIV(IDHCV+19)
!     CONVECTIF    SOLIDE   EVAPORATION
    ZQTPCNE=ZDHIV(IDHCV+20)
    ZCTPCNE=ZDHIV(IDHCV+21)
!     CONVECTIF    FONTE
    ZQVCFON=ZDHIV(IDHCV+22)
    ZCTCFON=ZDHIV(IDHCV+23)
  ELSE
    ZQTPSLF=0.0_JPRB
    ZCTPSLF=0.0_JPRB
    ZQTPSLE=0.0_JPRB
    ZCTPSLE=0.0_JPRB
    ZQTPSNF=0.0_JPRB
    ZCTPSNF=0.0_JPRB
    ZQTPSNE=0.0_JPRB
    ZCTPSNE=0.0_JPRB
    ZQVSFON=0.0_JPRB
    ZCTSFON=0.0_JPRB
    ZQTPCLF=0.0_JPRB
    ZCTPCLF=0.0_JPRB
    ZQTPCLE=0.0_JPRB
    ZCTPCLE=0.0_JPRB
    ZQTPCNF=0.0_JPRB
    ZCTPCNF=0.0_JPRB
    ZQTPCNE=0.0_JPRB
    ZCTPCNE=0.0_JPRB
    ZQVCFON=0.0_JPRB
    ZCTCFON=0.0_JPRB
  ENDIF

!     AVERTISSEMENT SUR ABSENCE DE GRADIENT DE VENT

  WRITE (NULOUT,*) ' DDH *** AVERTISSEMENT , L''APPORT ',&
   & ' HORIZONTAL D''ENERGIE CINETIQUE NE TIENT PAS COMPTE '  
  WRITE (NULOUT,*) '         DE L''ADVECTION ** ',&
   & ' LE TERME ENTIER EST COMPTE NUL DANS LE BILAN '  

!    --------------
!*    BILAN AU SOL
!    --------------

  IDHCS=0

  ZPTER=PDDHCS(IDHCS+1)
  ZUPTER=0.0_JPRB
  IF ( ZPTER < 1.E-07_JPRB ) THEN
    WRITE (NULOUT,*) ' DDH --- LE DOMAINE EST SUR MER '
  ENDIF
  IF ( ZPTER >= 1.E-07_JPRB ) THEN
    ZUPTER=1.0_JPRB/ZPTER
    IF ( ITYPE == 5 ) ZUPTER=1.0_JPRB
    WRITE (NULOUT,*) ' DDH --- PROPORTION DE TERRE : ',&
     & ZPTER,' CORRECTION DE SURFACE DES VARIABLES ET FLUX : ',&
     & ZUPTER  
    WRITE (NULOUT,*) ' '
  ENDIF
!     VARIABLES AU SOL INITIALES
  ZSTS0=PDDHCS(IDHCS+2)*ZUPTER
  ZSTP0=PDDHCS(IDHCS+3)*ZUPTER
  ZSWS0=PDDHCS(IDHCS+4)*ZUPTER
  ZSWP0=PDDHCS(IDHCS+5)*ZUPTER
  ZSNS0=PDDHCS(IDHCS+6)*ZUPTER

  IDHCS=NDHVS+NDHVFS

!     FLUX EN SURFACE
  ZSFRSO=PDDHCS(IDHCS+1)*ZUPTER
  ZSFRTH=PDDHCS(IDHCS+2)*ZUPTER
  ZSFCLL=PDDHCS(IDHCS+3)*ZUPTER
  ZSFCLN=PDDHCS(IDHCS+4)*ZUPTER
  ZSFCS=PDDHCS(IDHCS+5)*ZUPTER
  ZSFCHSP=PDDHCS(IDHCS+6)*ZUPTER
  ZSLFONT=PDDHCS(IDHCS+7)*ZUPTER
  ZSFPLSL=PDDHCS(IDHCS+8)*ZUPTER
  ZSFPLSN=PDDHCS(IDHCS+9)*ZUPTER
  ZSFPLCL=PDDHCS(IDHCS+10)*ZUPTER
  ZSFPLCN=PDDHCS(IDHCS+11)*ZUPTER
  ZSFEVL=PDDHCS(IDHCS+12)*ZUPTER
  ZSFEVN=PDDHCS(IDHCS+13)*ZUPTER
  ZSFLWSP=PDDHCS(IDHCS+14)*ZUPTER
  ZSFONTE=PDDHCS(IDHCS+15)*ZUPTER
  ZSRUISS=PDDHCS(IDHCS+16)*ZUPTER
  ZSRUISP=PDDHCS(IDHCS+17)*ZUPTER

!     VARIABLES AU SOL FINALES

  IDHCS=NDHCS

  ZSTS1=PDDHCS(IDHCS+2)*ZUPTER
  ZSTP1=PDDHCS(IDHCS+3)*ZUPTER
  ZSWS1=PDDHCS(IDHCS+4)*ZUPTER
  ZSWP1=PDDHCS(IDHCS+5)*ZUPTER
  ZSNS1=PDDHCS(IDHCS+6)*ZUPTER

!     BILAN THERMIQUE COUCHE SUPERFICIELLE
  ZSBTS=ZSTS1 - ZSTS0 - ZSFRSO - ZSFRTH - ZSFCLL &
   & - ZSFCLN - ZSFCS + ZSFCHSP - ZSLFONT  
!     BILAN THERMIQUE COUCHE PROFONDE
  ZSBTP=ZSTP1 - ZSTP0 - ZSFCHSP
!     BILAN HYDRIQUE COUCHE SUPERFICIELLE
  ZSBWS=ZSWS1 - ZSWS0 - ZSFPLSL - ZSFEVL + ZSRUISS &
   & - ZSFONTE + ZSFLWSP - ZSFPLCL  
!     BILAN HYDRIQUE COUCHE PROFONDE
  ZSBWP=ZSWP1 - ZSWP0 - ZSFLWSP + ZSRUISP
!     BILAN HYDRIQUE COUCHE DE NEIGE
  ZSBNS=ZSNS1 - ZSNS0 - ZSFPLSN - ZSFEVN + ZSFONTE- ZSFPLCN
!     NORMALISATION DES DONNEES EN KG/M**2
  ZQNORM=1.E+06_JPRB
  ZERO=0.0_JPRB
!     CHALEURS LATENTES NORMALISEES
  ZLSTT=RLSTT/ZQNORM
  ZLVTT=RLVTT/ZQNORM
  ZLFTT=RLMLT/ZQNORM

  WRITE (NULOUT,*) ' DDH *** AVERTISSEMENT ',&
   & ' LES LIGNES OUVERTES PAR -- DONT -- N ENTRENT PAS   '  
  WRITE (NULOUT,*) ' DANS LES BILANS : ELLES DONNENT ',&
   & ' UNE DECOMPOSITION DE LA VALEUR NETTE QUI PRECEDE '  
  WRITE (NULOUT,*) ' '

!*    TABLEAU-BILAN DE REFERENCE.

  WRITE (NULOUT,*) ' '
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5032)
  WRITE (NULOUT,5033) CLDOM
  WRITE (NULOUT,5032)
  WRITE (NULOUT,5034) ZHEURE, ZPTER*100._JPRB
  WRITE (NULOUT,5035)
  WRITE (NULOUT,5037) '            ','            ',&
   & '           ','            ','            ','           ',&
   & '           ','           '  
  WRITE (CL10C,'(''SUB='',F6.4)') ZLSTT
  WRITE (NULOUT,5036) '             ',CL10C,&
   & '  ENTHALPIE ','  ENERGIE   ','  MASSE D  ','  ENERGIE   ',&
   & '  ENERGIE   ',' RESERV.NEIGE',' RESERV.EAU',' RESERV.EAU'  
  WRITE (CL10C,'(''VAP='',F6.4)') ZLVTT
  WRITE (NULOUT,5036) 'CHALEUR LAT  ',CL10C,&
   & '            ',' CINETIQUE  ','    EAU    ',' COUCHE SUP ',&
   & ' COUCHE PROF','  SURFACE   ','  SURFACE  ','  PROFONDE '  
  WRITE (CL10C,'(''FUS='',F6.4)') ZLFTT
  WRITE (NULOUT,5036) '             ',CL10C,&
   & '            ','            ',' (10**(-6) ','            ',&
   & '            ',' (10**(-6)  ',' (10**(-6) ',' (10**(-6) '  
  WRITE (NULOUT,5037) '  (J/M**2)  ','  (J/M**2)  ',&
   & '  KG/M**2)  ','  (J/M**2)  ','  (J/M**2)  ','  KG/M**2)  ',&
   & '  KG/M**2) ','  KG/M**2) '  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5062) '   VALEURS INITIALES    ',&
   & ZCT0   ,ZKK0   ,ZQNORM*ZQV0   ,&
   & ZSTS0  ,ZSTP0  ,&
   & ZQNORM*ZSNS0  ,ZQNORM*ZSWS0  ,ZQNORM*ZSWP0  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5062) '   VALEURS COURANTES    ',&
   & ZCT1   ,ZKK1   ,ZQNORM*ZQV1   ,&
   & ZSTS1  ,ZSTP1  ,&
   & ZQNORM*ZSNS1  ,ZQNORM*ZSWS1  ,ZQNORM*ZSWP1  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5037) '            ','            ',&
   & ' (10**(-6)  ','            ','            ',' (10**(-6)  ',&
   & ' (10**(-6) ',' (10**(-6) '  
  WRITE (NULOUT,5037) '  (W/M**2)  ','  (W/M**2)  ',&
   & ' KG/M**2/S) ','  (W/M**2)  ','  (W/M**2)  ',' KG/M**2/S) ',&
   & ' KG/M**2/S)',' KG/M**2/S)'  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5038) '   FLUX MOYEN           '&
   & ,(ZCT1-ZCT0)*ZUSSSS &
   & ,(ZKK1-ZKK0)*ZUSSSS &
   & ,ZQNORM*(ZQV1-ZQV0)*ZUSSSS &
   & ,(ZSTS1-ZSTS0)*ZUSSSS &
   & ,(ZSTP1-ZSTP0)*ZUSSSS &
   & ,ZQNORM*(ZSNS1-ZSNS0)*ZUSSSS &
   & ,ZQNORM*(ZSWS1-ZSWS0)*ZUSSSS &
   & ,ZQNORM*(ZSWP1-ZSWP0)*ZUSSSS  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5038) '   TERMES RESIDUELS     ',&
   & ZBCT*ZUSSSS   ,ZBKK*ZUSSSS   ,ZQNORM*ZBQV*ZUSSSS   ,&
   & ZSBTS*ZUSSSS  ,ZSBTP*ZUSSSS  ,&
   & ZQNORM*ZSBNS*ZUSSSS  ,ZQNORM*ZSBWS*ZUSSSS  ,ZQNORM*ZSBWP*ZUSSSS  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5042) '    I SOLAIRE ALTITUDE  ',ZCTRAS0*ZUSSSS
  WRITE (NULOUT,5042) ' RAY- THERMIQUE ALTI    ',ZCTRAT0*ZUSSSS
  WRITE (NULOUT,5050) '    I SOLAIRE SURFACE   ',&
   & -ZCTRASS*ZUSSSS,ZSFRSO*ZUSSSS  
  WRITE (NULOUT,5050) '    I THERMIQUE SURFACE ',&
   & -ZCTRATS*ZUSSSS,ZSFRTH*ZUSSSS  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5040) '    I DISSIPATION       ',&
   & -ZKKDIT*ZUSSSS ,ZKKDIT*ZUSSSS  
  WRITE (NULOUT,5050) ' TUR- CHALEUR SENSIBLE  ',ZCTFT*ZUSSSS  ,ZSFCS*ZUSSSS
  WRITE (NULOUT,5052) '    I CHALEUR LATENTE   ',&
   & ZQNORM*ZQVFT*ZUSSSS,&
   & (ZSFCLL + ZSFCLN)*ZUSSSS ,&
   & ZQNORM*ZSFEVN*ZUSSSS ,ZQNORM*ZSFEVL*ZUSSSS  
  WRITE (NULOUT,5041) '    I DONT Q NEGATIF    ',ZQNORM*ZQVFNG*ZUSSSS
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5043) '     I MELANGE TURB     ',&
   & ZCTFTC*ZUSSSS ,ZQNORM*ZQVFTC*ZUSSSS  
  WRITE (NULOUT,5040) '     I DISSIPATION      ',&
   & -ZKKDIC*ZUSSSS ,ZKKDIC*ZUSSSS  
  WRITE (NULOUT,5053) ' CONV- BILAN PRECIP LIQ.',&
   & ZCTPCL*ZUSSSS ,ZQNORM*ZQTPCL*ZUSSSS ,&
   & ZQNORM*ZSFPLCL*ZUSSSS  
  WRITE (NULOUT,5054) '     I BILAN PRECIP NEI.',&
   & ZCTPCN*ZUSSSS ,ZQNORM*ZQTPCN*ZUSSSS ,&
   & ZQNORM*ZSFPLCN*ZUSSSS  
  IF(LHDADD) THEN
    WRITE (NULOUT,5060) '     I   DONT           '
    WRITE (NULOUT,5043) '     I  FORMATION   LIQ.',&
     & ZCTPCLF*ZUSSSS,ZQNORM*ZQTPCLF*ZUSSSS  
    WRITE (NULOUT,5043) '     I  EVAPORATION LIQ.',&
     & ZCTPCLE*ZUSSSS,ZQNORM*ZQTPCLE*ZUSSSS  
    WRITE (NULOUT,5043) '     I  FORMATION   NEI.',&
     & ZCTPCNF*ZUSSSS,ZQNORM*ZQTPCNF*ZUSSSS  
    WRITE (NULOUT,5043) '     I  EVAPORATION NEI.',&
     & ZCTPCNE*ZUSSSS,ZQNORM*ZQTPCNE*ZUSSSS  
    WRITE (NULOUT,5043) '     I  FONTE           ',&
     & ZCTCFON*ZUSSSS,ZQNORM*ZQVCFON*ZUSSSS  
  ENDIF
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5053) ' PRE.I BILAN PRECIP LIQ.',&
   & ZCTPSL*ZUSSSS ,ZQNORM*ZQTPSL*ZUSSSS ,&
   & ZQNORM*ZSFPLSL*ZUSSSS  
  WRITE (NULOUT,5054) ' STR.I BILAN PRECIP NEI.',&
   & ZCTPSN*ZUSSSS ,ZQNORM*ZQTPSN*ZUSSSS ,&
   & ZQNORM*ZSFPLSN*ZUSSSS  
  IF(LHDADD) THEN
    WRITE (NULOUT,5060) '     I   DONT           '
    WRITE (NULOUT,5043) '     I  FORMATION   LIQ.',&
     & ZCTPSLF*ZUSSSS,ZQNORM*ZQTPSLF*ZUSSSS  
    WRITE (NULOUT,5043) '     I  EVAPORATION LIQ.',&
     & ZCTPSLE*ZUSSSS,ZQNORM*ZQTPSLE*ZUSSSS  
    WRITE (NULOUT,5043) '     I  FORMATION   NEI.',&
     & ZCTPSNF*ZUSSSS,ZQNORM*ZQTPSNF*ZUSSSS  
    WRITE (NULOUT,5043) '     I  EVAPORATION NEI.',&
     & ZCTPSNE*ZUSSSS,ZQNORM*ZQTPSNE*ZUSSSS  
    WRITE (NULOUT,5043) '     I  FONTE           ',&
     & ZCTSFON*ZUSSSS,ZQNORM*ZQVSFON*ZUSSSS  
  ENDIF
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5040) ' ONDES DE RELIEF        ',&
   & -ZKKDID*ZUSSSS ,ZKKDID*ZUSSSS  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5045) '    I ECHANGES SOL-SOL  ',&
   & -ZSFCHSP*ZUSSSS,ZSFCHSP*ZUSSSS,&
   & ZQNORM*ZERO*ZUSSSS   ,-ZQNORM*ZSFLWSP*ZUSSSS,&
   & ZQNORM*ZSFLWSP*ZUSSSS  
  WRITE (NULOUT,5045) ' SOL- FONTE DE NEIGE    ',&
   & ZSLFONT*ZUSSSS,ZERO*ZUSSSS ,&
   & -ZQNORM*ZSFONTE*ZUSSSS,ZQNORM*ZSFONTE*ZUSSSS,&
   & ZQNORM*ZERO*ZUSSSS  
  WRITE (NULOUT,5047) '    I RUISSELLEMENT     ',&
   & ZQNORM*ZERO*ZUSSSS   ,-ZQNORM*ZSRUISS*ZUSSSS,&
   & -ZQNORM*ZSRUISP*ZUSSSS  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5044) ' CONVERSION I TRAV.F.PR.',ZKKCOV*ZUSSSS
  WRITE (NULOUT,5042) ' ADIABATIQUE- INTERNE   ',(ZCTCOV+ZCTCOP)*ZUSSSS
  WRITE (NULOUT,5060) '       ---- ET ----     '
  WRITE (CL24C,'(1X,A10,1X,F11.5,1X)') 'FLUX H EP=',-ZEPDH*ZUSSSS
  WRITE (NULOUT,5060) CL24C
  WRITE (CL24C,'(1X,A10,1X,F11.5,1X)') 'FLUX V EP=',-ZEPFD*ZUSSSS
  WRITE (NULOUT,5060) CL24C
  WRITE (CL24C,'(4X,E15.8,5X)') ZBCOV*ZUSSSS
  WRITE (NULOUT,5060) '  (BILAN CONVERSION:)   '
  WRITE (NULOUT,5060) CL24C
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5039) ' APPORT  DYN HORIZONTAL ',&
   & ZCTDH*ZUSSSS  ,ZKKDH*ZUSSSS  ,ZQNORM*ZQVDH*ZUSSSS  
  WRITE (NULOUT,5030)
  WRITE (NULOUT,5039) '  FLUX DYN VERTICAUX    ',&
   & ZCTFD*ZUSSSS  ,ZKKFD*ZUSSSS  ,ZQNORM*ZQVFD*ZUSSSS  
  WRITE (NULOUT,5039) '  FLUX DU AUX PRECIP.   ',&
   & ZERO*ZUSSSS   ,ZKKFDM*ZUSSSS ,ZQNORM*ZERO*ZUSSSS  
  WRITE (NULOUT,5030)

!*    TABLEAU-BILAN ATMOSPHERE.

  WRITE(NULOUT,6000)
  WRITE(NULOUT,6010)
  WRITE(NULOUT,'(A,F9.3,3A,27X,A)')&
   & 'I HEURE=',ZHEURE,'H '&
   & ,'DDH * BILAN ATMOSPHERIQUE, DOMAINE ',CLDOM,'I'  
  WRITE(NULOUT,6010)
  WRITE(NULOUT,6030)
  WRITE(NULOUT,'(20A)')&
   & 'I                       '&
   & ,'II','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              ',' I'  
  WRITE(NULOUT,'(A,F6.4,11A)')&
   & 'I             SUB=',ZLSTT &
   & ,'II','   ENTHALPIE  '&
   & ,'I','   ENERGIE    '&
   & ,'I','     EAU      '&
   & ,'I','     EAU      '&
   & ,'I','     EAU      ',' I'  
  WRITE(NULOUT,'(A,F6.4,11A)')&
   & 'I CHALEUR LAT VAP=',ZLVTT &
   & ,'II','              '&
   & ,'I','  CINETIQUE   '&
   & ,'I','    VAPEUR    '&
   & ,'I','   LIQUIDE    '&
   & ,'I','   SOLIDE     ',' I'  
  WRITE(NULOUT,'(A,F6.4,11A)')&
   & 'I             FUS=',ZLFTT &
   & ,'II','              '&
   & ,'I','              '&
   & ,'I','  (10**(-6)   '&
   & ,'I','  (10**(-6)   '&
   & ,'I','  (10**(-6)   ',' I'  
  WRITE(NULOUT,'(20A)')&
   & 'I                       '&
   & ,'II','  (J/M**2)    '&
   & ,'I','  (J/M**2)    '&
   & ,'I','   KG/M**2)   '&
   & ,'I','   KG/M**2)   '&
   & ,'I','   KG/M**2)   ',' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,5(2A,F11.0,A),A)')&
   & 'I   VALEURS INITIALES   '&
   & ,'II','  ',ZCT0,' '&
   & ,'I','  ',ZKK0,' '&
   & ,'I','  ',ZQV0*ZQNORM,' '&
   & ,'I','  ',ZQL0*ZQNORM,' '&
   & ,'I','  ',ZQN0*ZQNORM,' ',' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,5(2A,F11.0,A),A)')&
   & 'I   VALEURS FINALES     '&
   & ,'II','  ',ZCT1,' '&
   & ,'I','  ',ZKK1,' '&
   & ,'I','  ',ZQV1*ZQNORM,' '&
   & ,'I','  ',ZQL1*ZQNORM,' '&
   & ,'I','  ',ZQN1*ZQNORM,' ',' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(20A)')&
   & 'I                       '&
   & ,'II','              '&
   & ,'I','              '&
   & ,'I','  (10**(-6)   '&
   & ,'I','  (10**(-6)   '&
   & ,'I','  (10**(-6)   '&
   & ,' I'  
  WRITE(NULOUT,'(20A)')&
   & 'I                       '&
   & ,'II','   (W/M**2)   '&
   & ,'I','   (W/M**2)   '&
   & ,'I','  KG/M**2/S)  '&
   & ,'I','  KG/M**2/S)  '&
   & ,'I','  KG/M**2/S)  '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,5(2A,F11.5,A),A)')&
   & 'I   FLUX MOYEN          '&
   & ,'II','  ',(ZCT1-ZCT0)*ZUSSSS,' '&
   & ,'I','  ',(ZKK1-ZKK0)*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*(ZQV1-ZQV0)*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*(ZQL1-ZQL0)*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*(ZQN1-ZQN0)*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,5(2A,F11.5,A),A)')&
   & 'I   TERMES RESIDUELS    '&
   & ,'II','  ',ZBCT*ZUSSSS,' '&
   & ,'I','  ',ZBKK*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*ZBQV*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*ZBQL*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*ZBQN*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,(2A,F11.5,A),4(2A),A)')&
   & 'I    I SOLAIRE ALTITUDE '&
   & ,'II','  ',ZCTRAS0*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(2A,F11.5,A),4(2A),A)')&
   & 'I RAY- THERMIQ ALTITUDE '&
   & ,'II','  ',ZCTRAT0*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(2A,F11.5,A),4(2A),A)')&
   & 'I    I SOLAIRE SURFACE  '&
   & ,'II','  ',-ZCTRASS*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(2A,F11.5,A),4(2A),A)')&
   & 'I    I THERMIQ SURFACE  '&
   & ,'II','  ',-ZCTRATS*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,2(2A,F11.5,A),3(2A),A)')&
   & 'I     I DISSIPATION     '&
   & ,'II','  ',-ZKKDIT*ZUSSSS,' '&
   & ,'I','  ',ZKKDIT*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(2A,F11.5,A),4(2A),A)')&
   & 'I  TUR- CHAL. SENSIBLE  '&
   & ,'II','  ',ZCTFT*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(5A,(2A,F11.5,A),2(2A),A)')&
   & 'I     I CHAL. LATENTE   '&
   & ,'II','              '&
   & ,'I','              '&
   & ,'I','  ',ZQNORM*ZQVFT*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(5A,3(2A,F11.5,A),A)')&
   & 'I     I DONT Q NEGATIF  '&
   & ,'II','              '&
   & ,'I','              '&
   & ,'I','  ',ZQNORM*ZQVFNG*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*ZQLFNG*ZUSSSS,' '&
   & ,'I','  ',ZQNORM*ZQNFNG*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,(A,F11.5,A),A,3(A,F11.5,A),A)')&
   & 'I     I MELANGE TURB.   '&
   & ,'II  ',ZCTFTC*ZUSSSS,' '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQVFTC*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQLFTC*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQNFTC*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,'(A,2(A,F11.5,A),4A)')&
   & 'I     I DISSIPATION     '&
   & ,'II  ',-ZKKDIC*ZUSSSS,' '&
   & ,'I  ',ZKKDIC*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(2A,F11.5,A),2A,2(2A,F11.5,A),2A,A)')&
   & 'I CONV- CONDENS. LIQ.   '&
   & ,'II','  ',ZCTPCL*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','  ',ZQNORM*ZQTCCL*ZUSSSS,' '&
   & ,'I','  ',-ZQNORM*ZQTCCL*ZUSSSS,' '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(4A,(A,F11.5,A),2A)')&
   & 'I     I PRECIPI. LIQ.   '&
   & ,'II              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQTPCL*ZUSSSS,' '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,3((A,F11.5,A),A))')&
   & 'I     I CONDENS. NEI.   '&
   & ,'II  ',ZCTPCN*ZUSSSS,' '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQTCCN*ZUSSSS,' '&
   & ,'I              '&
   & ,'I  ',-ZQNORM*ZQTCCN*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,'(5A,(A,F11.5,A),A)')&
   & 'I     I PRECIPI. NEI.   '&
   & ,'II              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQTPCN*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I     I CHA. SENS. LIQ. '&
   & ,'II  ',ZCTCCL*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I     I CHA. SENS. NEI. '&
   & ,'II  ',ZCTCCN*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  IF(LHDADD) THEN
    WRITE(NULOUT,'(20A)')&
     & 'I     I  DONT           '&
     & ,'II','              '&
     & ,'I','              '&
     & ,'I','              '&
     & ,'I','              '&
     & ,'I','              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I FORMATION   LIQ.'&
     & ,'II  ',ZCTPCLF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPCLF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I EVAPORATION LIQ.'&
     & ,'II  ',ZCTPCLE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPCLE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I FORMATION   NEI.'&
     & ,'II  ',ZCTPCNF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPCNF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I EVAPORATION NEI.'&
     & ,'II  ',ZCTPCNE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPCNE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I FONTE           '&
     & ,'II  ',ZCTCFON*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQVCFON*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
  ENDIF
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,(2A,F11.5,A),2A,2(2A,F11.5,A),2A,A)')&
   & 'I     I CONDENS. LIQ.   '&
   & ,'II','  ',ZCTPSL*ZUSSSS,' '&
   & ,'I','              '&
   & ,'I','  ',ZQNORM*ZQTCSL*ZUSSSS,' '&
   & ,'I','  ',-ZQNORM*ZQTCSL*ZUSSSS,' '&
   & ,'I','              '&
   & ,' I'  
  WRITE(NULOUT,'(4A,(A,F11.5,A),2A)')&
   & 'I STRA- PRECIPI. LIQ.   '&
   & ,'II              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQTPSL*ZUSSSS,' '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,3((A,F11.5,A),A))')&
   & 'I     I CONDENS. NEI.   '&
   & ,'II  ',ZCTPSN*ZUSSSS,' '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQTCSN*ZUSSSS,' '&
   & ,'I              '&
   & ,'I  ',-ZQNORM*ZQTCSN*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,'(5A,(A,F11.5,A),A)')&
   & 'I     I PRECIPI. NEI.   '&
   & ,'II              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I  ',ZQNORM*ZQTPSN*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I     I CHA. SENS. LIQ. '&
   & ,'II  ',ZCTCSL*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I     I CHA. SENS. NEI. '&
   & ,'II  ',ZCTCSN*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  IF(LHDADD) THEN
    WRITE(NULOUT,'(20A)')&
     & 'I     I  DONT           '&
     & ,'II','              '&
     & ,'I','              '&
     & ,'I','              '&
     & ,'I','              '&
     & ,'I','              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I FORMATION   LIQ.'&
     & ,'II  ',ZCTPSLF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPSLF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I EVAPORATION LIQ.'&
     & ,'II  ',ZCTPSLE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPSLE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I FORMATION   NEI.'&
     & ,'II  ',ZCTPSNF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPSNF*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I EVAPORATION NEI.'&
     & ,'II  ',ZCTPSNE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQTPSNE*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
    WRITE(NULOUT,'(A,(A,F11.5,A),A,(A,F11.5,A),3A)')&
     & 'I     I FONTE           '&
     & ,'II  ',ZCTSFON*ZUSSSS,' '&
     & ,'I              '&
     & ,'I  ',ZQNORM*ZQVSFON*ZUSSSS,' '&
     & ,'I              '&
     & ,'I              '&
     & ,' I'  
  ENDIF
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,2(A,F11.5,A),4A)')&
   & 'I PREC- EN. POT. ET CIN.'&
   & ,'II  ',ZCTCOP*ZUSSSS,' '&
   & ,'I  ',ZKKFDM*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,2(A,F11.5,A),5A)')&
   & 'I   ONDES DE RELIEF     '&
   & ,'II  ',-ZKKDID*ZUSSSS,' '&
   & ,'I  ',ZKKDID*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I      I BILAN          '&
   & ,'II  ',ZBCOV*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(7A)')&
   & 'I ADIAB- DONT           '&
   & ,'II              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I      I FLUX HOR. EP.  '&
   & ,'II  ',-ZEPDH*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I      I FLUX VER. EP.  '&
   & ,'II  ',-ZEPFD*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(2A,1(A,F11.5,A),4A)')&
   & 'I      I TRAV. F. PRES. '&
   & ,'II              '&
   & ,'I  ',ZKKCOV*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,'(A,(A,F11.5,A),5A)')&
   & 'I      I CONV. AD.-INT. '&
   & ,'II  ',ZCTCOV*ZUSSSS,' '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,'I              '&
   & ,' I'  
  WRITE(NULOUT,6000)
  WRITE(NULOUT,'(A,5(A,F11.5,A),A)')&
   & 'I DYN- FLUX HORIZONT.   '&
   & ,'II  ',ZCTDH*ZUSSSS,' '&
   & ,'I  ',ZKKDH*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQVDH*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQLDH*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQNDH*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,'(A,5(A,F11.5,A),A)')&
   & 'I    I FLUX VERTICAL    '&
   & ,'II  ',ZCTFD*ZUSSSS,' '&
   & ,'I  ',ZKKFD*ZUSSSS*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQVFD*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQLFD*ZUSSSS,' '&
   & ,'I  ',ZQNORM*ZQNFD*ZUSSSS,' '&
   & ,' I'  
  WRITE(NULOUT,6000)

!     - LIGNE DE SEPARATION
  6000 FORMAT ('DDH',98('-'),'I')
!     - LIGNE BLANCHE
  6010 FORMAT ('I',100X,'I')
!     - LIGNE DOMAINE
!     - LIGNE PARTIELLE
  6030 FORMAT ('I',23X,'II',75('-'),'I')

!    -------------------------
!*    TABLEAU-BILAN DE MASSE.
!    -------------------------

  WRITE (NULOUT,*) ' '
  WRITE (NULOUT,5030)
  WRITE (NULOUT,*) ' I   - BILAN DE MASSE ',' (10**(-6)  KG/M**2) '
  WRITE (NULOUT,*) ' '
  WRITE (NULOUT,5020) ZMNORM*ZMAR0, ZMNORM*ZMARDH,&
   & ZMNORM*ZMARFD0, ZMNORM*ZMARFDS, ZMNORM*ZMARFP0,&
   & ZMNORM*ZMARFPS, ZMNORM*ZMAR1, ZMNORM*ZBMAR  

!    -------------------------------------------------------
!*    TABLEAU-VARIABLES DIAGNOSTIQUES INITIALES ET FINALES.
!    -------------------------------------------------------

  WRITE(NULOUT,'(A)') ' '
  WRITE(NULOUT,'(A,72(''-''))') 'DDH'
  WRITE(NULOUT,'(A,73X,A)') 'I','I'
  WRITE(NULOUT,'(A,F9.3,3A,1X,A)')&
   & 'I HEURE=',ZHEURE,'H '&
   & ,'DDH * VAR. DIAGNOSTIQUES, DOMAINE ',CLDOM,'I'  
  WRITE(NULOUT,'(A,73X,A)') 'I','I'
  WRITE(NULOUT,'(A,73(''-''),A)') 'I','I'
  WRITE(NULOUT,'(3A)')&
   & 'I   VARIABLE   I      VALEUR INITIALE       I'&
   & ,'        VALEUR FINALE        I'  
  WRITE(NULOUT,'(A,73(''-''),A)') 'I','I'

!     VITESSE VERTICALE EN COORDONNEE PRESSION.

  IDHCV=0
  ZOM0=ZDHIV(IDHCV+10+NHDQLNVA+NHDPASVA)/ZDHIV(IDHCV+1)
  IDHCV=NDHCV
  ZOM1=ZDHIV(IDHCV+10+NHDQLNVA+NHDPASVA)/ZDHIV(IDHCV+1)
  WRITE(NULOUT,'(A,2(E17.10,A))')&
   & 'I  OMEGA (P/S) I     ',ZOM0,'      I      '&
   & ,ZOM1,'      I'  

!     NEBULOSITE TOTALE LOCALE.

  IDHCV=0
  ZNT0=ZDHIV(IDHCV+9+NHDQLNVA+NHDPASVA)/ZDHIV(IDHCV+1)
  IDHCV=NDHCV
  ZNT1=ZDHIV(IDHCV+9+NHDQLNVA+NHDPASVA)/ZDHIV(IDHCV+1)
  WRITE(NULOUT,'(A,2(E17.10,A))')&
   & 'I  NEBULOSITE  I     ',ZNT0,'      I      '&
   & ,ZNT1,'      I'  

!     HUMIDITE RELATIVE.

  IDHCV=0
  ZHR0=ZDHIV(IDHCV+8)/ZDHIV(IDHCV+1)
  IDHCV=NDHCV
  ZHR1=ZDHIV(IDHCV+8)/ZDHIV(IDHCV+1)
  WRITE(NULOUT,'(A,2(E17.10,A))')&
   & 'I  HUM. RELAT. I     ',ZHR0,'      I      '&
   & ,ZHR1,'      I'  

!     VARIABLES PASSIVES.

  IF(LHDPAS) THEN
    DO JJPASS=1,NHDPASVA
      IDHCV=0
      ZPASS0=ZDHIV(IDHCV+9+NHDQLNVA+JJPASS-1)
      IDHCV=NDHCV
      ZPASS1=ZDHIV(IDHCV+9+NHDQLNVA+JJPASS-1)
      WRITE(NULOUT,'(A,I2.2,A,2(E17.10,A))')&
       & 'I  PASS ',JJPASS,'     I     ',ZPASS0,'      I      '&
       & ,ZPASS1,'      I'  
    ENDDO
  ENDIF
  WRITE(NULOUT,'(A,72(''-''))') 'DDH'
  WRITE(NULOUT,*) ' '

!     FIN BILAN LHDHKS
!     ----------------

ENDIF

!     ------------------------------------------------------------------
!**   4.- IMPRESSION DU BILAN MOMENT CINETIQUE.

IF ( LHDMCI ) THEN
  ISMC=1
!     -----------------------------
!*     BALANCE OF ANGULAR MOMENTUM
!     -----------------------------
  IDHCV=NDHVHK*ISHK

  ZA10=ZDHIV(IDHCV+1)
  ZA20=ZDHIV(IDHCV+2)
  ZA30=ZDHIV(IDHCV+3)

  IDHCV=NDHVV+NDHAHKD*ISHK

  ZA1DH=ZDHIV(IDHCV+1)
  ZA2DH=ZDHIV(IDHCV+2)
  ZA3DH=ZDHIV(IDHCV+3)

  ZA1AD=ZDHIV(IDHCV+4)
  ZA2AD=ZDHIV(IDHCV+5)
  ZA3AD=ZDHIV(IDHCV+6)

  ZA1NA=ZDHIV(IDHCV+7)
  ZA2NA=ZDHIV(IDHCV+8)

  IDHCV=NDHVV+NDHAVD+NDHBHKD*ISHK

  ZA1D0=PDDHCV(0,IDHCV+1)
  ZA1DS=PDDHCV(NFLEVG,IDHCV+1)
  ZA1FD=ZA1D0-ZA1DS
  ZA2D0=PDDHCV(0,IDHCV+2)
  ZA2DS=PDDHCV(NFLEVG,IDHCV+2)
  ZA2FD=ZA2D0-ZA2DS
  ZA3D0=PDDHCV(0,IDHCV+3)
  ZA3DS=PDDHCV(NFLEVG,IDHCV+3)
  ZA3FD=ZA3D0-ZA3DS

  ZA1O0=PDDHCV(0,IDHCV+4)
  ZA1OS=PDDHCV(NFLEVG,IDHCV+4)
  ZA1FO=ZA1O0-ZA1OS
  ZA2O0=PDDHCV(0,IDHCV+5)
  ZA2OS=PDDHCV(NFLEVG,IDHCV+5)
  ZA2FO=ZA2O0-ZA2OS
  ZA3O0=PDDHCV(0,IDHCV+6)
  ZA3OS=PDDHCV(NFLEVG,IDHCV+6)
  ZA3FO=ZA3O0-ZA3OS

  ZA1P0=PDDHCV(0,IDHCV+7)
  ZA1PS=PDDHCV(NFLEVG,IDHCV+7)
  ZA1FP=ZA1P0-ZA1PS
  ZA2P0=PDDHCV(0,IDHCV+8)
  ZA2PS=PDDHCV(NFLEVG,IDHCV+8)
  ZA2FP=ZA2P0-ZA2PS
  ZA3P0=PDDHCV(0,IDHCV+9)
  ZA3PS=PDDHCV(NFLEVG,IDHCV+9)
  ZA3FP=ZA3P0-ZA3PS

  IDHCV=NDHVV+NDHFVD+NDHAHKP*ISHK

  ZA1T0=PDDHCV(0,IDHCV+1)
  ZA1TS=PDDHCV(NFLEVG,IDHCV+1)
  ZA1FT=ZA1T0-ZA1TS
  ZA2T0=PDDHCV(0,IDHCV+2)
  ZA2TS=PDDHCV(NFLEVG,IDHCV+2)
  ZA2FT=ZA2T0-ZA2TS
  ZA3T0=PDDHCV(0,IDHCV+3)
  ZA3TS=PDDHCV(NFLEVG,IDHCV+3)
  ZA3FT=ZA3T0-ZA3TS

  ZA1C0=PDDHCV(0,IDHCV+4)
  ZA1CS=PDDHCV(NFLEVG,IDHCV+4)
  ZA1FC=ZA1C0-ZA1CS
  ZA2C0=PDDHCV(0,IDHCV+5)
  ZA2CS=PDDHCV(NFLEVG,IDHCV+5)
  ZA2FC=ZA2C0-ZA2CS
  ZA3C0=PDDHCV(0,IDHCV+6)
  ZA3CS=PDDHCV(NFLEVG,IDHCV+6)
  ZA3FC=ZA3C0-ZA3CS

  ZA1G0=PDDHCV(0,IDHCV+7)
  ZA1GS=PDDHCV(NFLEVG,IDHCV+7)
  ZA1FG=ZA1G0-ZA1GS
  ZA2G0=PDDHCV(0,IDHCV+8)
  ZA2GS=PDDHCV(NFLEVG,IDHCV+8)
  ZA2FG=ZA2G0-ZA2GS
  ZA3G0=PDDHCV(0,IDHCV+9)
  ZA3GS=PDDHCV(NFLEVG,IDHCV+9)
  ZA3FG=ZA3G0-ZA3GS

  IDHCV=NDHCV+NDHVHK*ISHK

  ZA11=ZDHIV(IDHCV+1)
  ZA21=ZDHIV(IDHCV+2)
  ZA31=ZDHIV(IDHCV+3)

!     BALANCE

  ZBA1=ZA11 - ZA10 - ZA1AD - ZA1NA - ZA1FD - ZA1FO &
   & - ZA1FP - ZA1FT - ZA1FC - ZA1FG  
  ZBA2=ZA21 - ZA20 - ZA2AD - ZA2NA - ZA2FD - ZA2FO &
   & - ZA2FP - ZA2FT - ZA2FC - ZA2FG  
  ZBA3=ZA31 - ZA30 - ZA3AD         - ZA3FD - ZA3FO &
   & - ZA3FP - ZA3FT - ZA3FC - ZA3FG  

!     COMPUTE THE ANGLE BETWEEN OF THE ANGULAR MOMENTUM VECTOR AND
!     THE EARTH ROTATION AXIS

  ZNORM=SQRT(ABS(ZA11**2 +ZA21**2 +ZA31**2))
  ZNORMEQ=SQRT(ABS(ZA11**2 +ZA21**2))
  IF (ZNORMEQ*ZNORM /= 0.0_JPRB) THEN
    ZLAT0=ACOS(ZA31/ZNORM)*180._JPRB/RPI
    ZGMTSEC=REAL(MOD(NINT(RSTATI,JPIB)+NSSSSS,86400),JPRB)
    ZLON0=(ACOS(ZA11/ZNORMEQ)-ROMEGA*ZGMTSEC)*180._JPRB/RPI
  ENDIF

!*    PRINT RESULTS

  ZANM=1.E-6_JPRB

  WRITE (NULOUT,5200)
  WRITE (NULOUT,5201) NINT(LOG10(ZANM))
  WRITE (NULOUT,5200)
  WRITE (NULOUT,5202)
  WRITE (NULOUT,5203)
  WRITE (NULOUT,5200)
  WRITE (NULOUT,5204) ZA10*ZANM,ZA1DH*ZANM,ZA1FD*ZANM,ZA1FP*ZANM,&
   & ZA1FT*ZANM,ZA1FC*ZANM,ZA1FG*ZANM,&
   & ZA1FO*ZANM,ZA1AD*ZANM,ZA1NA*ZANM,ZA11*ZANM,&
   & ZBA1*ZANM  
  WRITE (NULOUT,5200)
  WRITE (NULOUT,5205) ZA20*ZANM,ZA2DH*ZANM,ZA2FD*ZANM,ZA2FP*ZANM,&
   & ZA2FT*ZANM,ZA2FC*ZANM,ZA2FG*ZANM,&
   & ZA2FO*ZANM,ZA2AD*ZANM,ZA2NA*ZANM,ZA21*ZANM,&
   & ZBA2*ZANM  
  WRITE (NULOUT,5200)
  WRITE (NULOUT,5206) ZA30*ZANM,ZA3DH*ZANM,ZA3FD*ZANM,ZA3FP*ZANM,&
   & ZA3FT*ZANM,ZA3FC*ZANM,ZA3FG*ZANM,&
   & ZA3FO*ZANM,ZA3AD*ZANM,ZA31*ZANM,ZBA3*ZANM  
  WRITE (NULOUT,5200)
  WRITE (NULOUT,5207)
  IF (ZNORM*ZNORMEQ /= 0.0_JPRB) THEN
    WRITE(NULOUT,5208) ZLAT0,ZLON0
  ELSEIF (ZNORM == 0.0_JPRB) THEN
    WRITE(NULOUT,5209)
  ELSE
    WRITE(NULOUT,5210)
  ENDIF

  WRITE(NULOUT,*) 'DIVERG. : ',ZA1DH,ZA2DH,ZA3DH
!     END OF BALANCE LHDMCI
!     ---------------------

ENDIF

!     ------------------------------------------------------------------
!**   5.- IMPRESSION DU BILAN ENTROPIE.

IF ( LHDENT ) THEN
  ISEN=1
!     --------------------
!*     BALANCE OF ENTROPY
!     --------------------
  IDHCV=NDHVHK*ISHK+NDHVMC*ISMC

  ZEN0=ZDHIV(IDHCV+1)

  IDHCV=NDHVV+NDHAHKD*ISHK+NDHAMCD*ISMC

  ZENDH=ZDHIV(IDHCV+1)
  ZENPCT=ZDHIV(IDHCV+2)

  IDHCV=NDHVV+NDHAVD+NDHBHKD*ISHK+NDHBMCD*ISMC

  ZEND0=PDDHCV(0,IDHCV+1)
  ZENDS=PDDHCV(NFLEVG,IDHCV+1)
  ZENFD=ZEND0 - ZENDS
  ZENP0=PDDHCV(0,IDHCV+2)
  ZENPS=PDDHCV(NFLEVG,IDHCV+2)
  ZENP=ZENP0 - ZENPS

  IDHCV=NDHVV+NDHFVD+NDHAVP+NDHBHKP*ISHK+NDHBMCP*ISMC

  ZENDIT=ZDHIV(IDHCV+1)
  ZENDIC=ZDHIV(IDHCV+2)
  ZENDID=ZDHIV(IDHCV+3)
  ZENDISS=ZENDIT+ZENDIC+ZENDID

  ZENHUM=ZDHIV(IDHCV+4)

  ZENRAD=ZDHIV(IDHCV+5)

  ZENCT=ZDHIV(IDHCV+6)

  IDHCV=NDHCV+NDHVHK*ISHK+NDHVMC*ISMC

  ZEN1=ZDHIV(IDHCV+1)

!     BALANCE

  ZBEN=ZEN1 - ZEN0 - ZENDH - ZENFD - ZENP - ZENDISS -&
   & ZENHUM - ZENRAD - ZENCT - ZENPCT  

!*    PRINT RESULTS

  WRITE (NULOUT,5100)
  WRITE (NULOUT,5101)
  WRITE (NULOUT,5100)
  WRITE (NULOUT,5102)
  WRITE (NULOUT,5103)
  WRITE (NULOUT,5100)
  WRITE (NULOUT,5104) ZEN0,ZENDH,ZENFD,ZENP,ZENDIT,ZENDIC,ZENDID,&
   & ZENHUM,ZENCT,ZENRAD,ZENPCT,ZEN1,ZBEN  
  WRITE (NULOUT,5100)

!     END OF BALANCE LHDENT
!     ---------------------

ENDIF

!     FORMATS.
!     --------

5020 FORMAT (T3,'I MASSE        I FLUX HORIZ.  I FLUX DYN.    I',&
 & ' FLUX DYN.    I FLUX PHYS.   I FLUX PHYS.   I',&
 & ' MASSE        I I I RESIDU       I',/,T3,&
 & 'I INITIALE     I              I SOMMET       I',&
 & ' SURFACE      I SOMMET       I SURFACE      I',&
 & ' FINALE       I I I              I',/,T3,&
 & 106('-'),' I ',16('-'),/,&
 & T3,'-',7(14X,'-'),' I ','-',14X,'-',/,&
 & T3,'-',1X,F12.0,1X,'I',4(1X,F12.0,1X,'.'),&
 & 1X,F12.0,1X,'I',1X,F12.0,1X,'-',' I ','-',1X,F12.0,1X,'-',/,&
 & T3,'-',7(14X,'-'),' I ','-',14X,'-',/,&
 & T3,106('-'),' I ',16('-'))  

!     - LIGNE DE SEPARATION
5030 FORMAT (T1,'DDH',130('-'))
!     - LOGO
5031 FORMAT (T1,'-',33('-DDH'))
!     - LIGNE BLANCHE
5032 FORMAT (T1,'I',131X,'I')
!     - LIGNE DOMAINE
5033 FORMAT(T1,'I',28X,&
 & 'DDH * BILAN ENERGETIQUE ET HYDROLOGIQUE CUMULE _ DOMAINE ',&
 & A20,26X,'I' )  
!     - LIGNE ESPACE CONCERNE (ZHEURE ET ZPTER*100.)
5034 FORMAT (T1,'I',5X,'HEURE= ',F7.3,'H',4X,&
 & 'II',14X,'ATMOSPHERE',14X,'I I',14X,&
 & 'TERRE ( ',F6.2,' % DU DOMAINE )',21X,'I' )  
!     - LIGNE SEPARATION PARTIELLE
5035 FORMAT (T1,'I',24X,107('-'),'I' )
!     - LIGNE IDENTITE DES COLONNES (MOT 13C, MOT 10C, 8 MOTS 12C
!       OU 11C SI EAU
5036 FORMAT (T1,'I',A13,1X,A10,'II',&
 & A12,'I',A12,'II',A11,'I I',&
 & A12,'I',A12,'II',A13,'I',A11,'I',A11,'I')  
!     - LIGNE IDENTITE DES COLONNES SANS PRE-TITRE
5037 FORMAT (T1,'I',24X,'II',&
 & A12,'I',A12,'II',A11,'I I',&
 & A12,'I',A12,'II',A13,'I',A11,'I',A11,'I')  
!     - LIGNE DE CHIFFRES PLEINE (TITRE (24 C), 8 CHIFFRES)
5038 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',F11.5,1X,'II',F10.5,1X,'I I',&
 & F11.5,1X,'I',F11.5,1X,'II',&
 & F12.5,1X,'I',F10.5,1X,'I',F10.5,1X,'I' )  
!     - LIGNE ATMOSPHERE SEULE
5039 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',F11.5,1X,'II',F10.5,1X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE ENERGETIQUE ATM SEULE
5040 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',F11.5,1X,'II',11X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE EAU ATM SEULE
5041 FORMAT (T1,'I',A24,'II',&
 & 12X,'I',12X,'II',F10.5,1X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE ENTHALPIE SEULE
5042 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',12X,'II',11X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE ENTHALPIE ET EAU ATM SEULES
5043 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',12X,'II',F10.5,1X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE ENERGIE CINETIQUE SEULE
5044 FORMAT (T1,'I',A24,'II',&
 & 12X,'I',F11.5,1X,'II',11X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE SOL SEUL
5045 FORMAT (T1,'I',A24,'II',&
 & 12X,'I',12X,'II',11X,'I I',&
 & F11.5,1X,'I',F11.5,1X,'II',&
 & F12.5,1X,'I',F10.5,1X,'I',F10.5,1X,'I' )  
!     - LIGNE BILAN EN EAU DU SOL SEUL
5047 FORMAT (T1,'I',A24,'II',&
 & 12X,'I',12X,'II',11X,'I I',&
 & 12X,'I',12X,'II',&
 & F12.5,1X,'I',F10.5,1X,'I',F10.5,1X,'I' )  
!     - LIGNE BILAN ENTHALPIE ET COUCHE SOL SUP SEUL
5050 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',12X,'II',11X,'I I',&
 & F11.5,1X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
!     - LIGNE EAU ATM, ENERGIE SOL SUP, HAUT NEIGE, EAU SOL SUP
5052 FORMAT (T1,'I',A24,'II',&
 & 12X,'I',12X,'II',F10.5,1X,'I I',&
 & F11.5,1X,'I',12X,'II',&
 & F12.5,1X,'I',F10.5,1X,'I',11X,'I' )  
!     - LIGNE ENTHALPIE, EAU ATM, EAU SOL SUP
5053 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',12X,'II',F10.5,1X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',F10.5,1X,'I',11X,'I' )  
!     - LIGNE ENTHALPIE, EAU ATM, NEIGE SOL
5054 FORMAT (T1,'I',A24,'II',&
 & F11.5,1X,'I',12X,'II',F10.5,1X,'I I',&
 & 12X,'I',12X,'II',&
 & F12.5,1X,'I',11X,'I',11X,'I' )  
!     - LIGNE SANS DONNEES CHIFFREES DIRECTES (TITRE DE LIGNE 24C SEUL)
5060 FORMAT (T1,'I',A24,'II',&
 & 12X,'I',12X,'II',11X,'I I',&
 & 12X,'I',12X,'II',&
 & 13X,'I',11X,'I',11X,'I' )  
5061 FORMAT(A,F5.2,A)
!     - LIGNE DE CHIFFRES PLEINE (TITRE (24 C), 8 CHIFFRES)
5062 FORMAT (T1,'I',A24,'II',&
 & F11.0,1X,'I',F11.0,1X,'II',F10.0,1X,'I I',&
 & F11.0,1X,'I',F11.0,1X,'II',&
 & F12.0,1X,'I',F10.0,1X,'I',F10.0,1X,'I' )  
5100 FORMAT (T1,124('-'))
5101 FORMAT (T1,'I',T124,'I'/ T1,'I',&
 & T30,7('*'),' BALANCE OF ENTROPY [ J / (KG K M**2) ] ',&
 & 7('*'),T124,'I'/ T1,'I',T124,'I')  
5102 FORMAT(T1,'I',11X,'II',10X,2('I',8X),'I',20X,2('I',8X),'I',&
 & 9X,'I',8X,'II',11X,'I',9X,'I'/&
 & T1,'I',' INITIAL   ','II',' HORIZ.   ','I',' DYNAM. ',&
 & 'I',' PRECIP.','I','    DISSIPATION      ','I',&
 & ' HUMID. ','I',' ENTHAL.','I',' RADIAT. ','I','PREC.TR.',&
 & 'II',' FINAL     ','I',' RESIDUAL','I')  
5103 FORMAT(T1,'I',' ENTROPY   ','II',' DIVERG.  ','I',' FLUX   ',&
 & 'I',' FLUX   ','I',' TURB ','I',' CONV ','I',' GWD  ','I',&
 & ' DIFFUS.','I',' DIFFUS.','I',' TENDEN. ','I',' ENTHAL.',&
 & 'II',' ENTROPY   ','I',9X,'I'/&
 & T1,'I',11X,'II',10X,2('I',8X),3('I',6X),2('I',8X),'I',&
 & 9X,'I',8X,'II',11X,'I',9X,'I')  
5012 FORMAT ( T3,'COORDONNEES (LONGITUDE GEO,',&
 & 'SINUS(LATITUDE GEO) )=',&
 & 4(1X,'(',F8.4,',',F8.4,')') )  
5104 FORMAT(T1,'I',11X,'II',10X,2('I',8X),3('I',6X),2('I',8X),'I',&
 & 9X,'I',8X,'II',11X,'I',9X,'I'/&
 & T1,'I',F11.0,'II',F10.0,2('I',F8.0),3('I',F6.0),&
 & 2('I',F8.0),'I',F9.0,'I',F8.0,'II',F11.0,'I',F9.0,'I'/&
 & T1,'I',11X,'II',10X,2('I',8X),3('I',6X),2('I',8X),'I',&
 & 9X,'I',8X,'II',11X,'I',9X,'I')  
5200 FORMAT (T1,128('-'))
5201 FORMAT (T1,'I',T128,'I'/ T1,'I',T20,7('*'),&
 & ' BALANCE OF ANGULAR MOMENTUM * 10**',I3,&
 & '  [ KG * M**2 /(S * M**2)] ',&
 & 7('*'),T128,'I'/ T1,'I',T128,'I')  
5202 FORMAT(T1,'I',11X,'II',11X,'II',8X,2('I',8X),'I',23X,'I',8X,&
 & 2('I',8X),'II',11X,'I',9X,'I'/&
 & T1,'I',' COMPONENT ','II',&
 & ' INITIAL   ','II',' HORIZ. ','I',' DYNAM. ',&
 & 'I',' PRECIP.','I','     DISSIPATION        ','I',&
 & ' OROGR. ','I',' ADJUST.','I','NONAXIAL',&
 & 'II',' FINAL     ','I',' RESIDUAL','I')  
5203 FORMAT(T1,'I',11X,'II','ANG.MOMENT.','II',' DIVERG.','I',&
 & ' FLUX   ','I',' FLUX   ','I','  TURB ','I','  CONV ','I',&
 & '  GWD  ','I',' FLUX   ','I',' TENDEN.','I',' MOMENTS',&
 & 'II','ANG.MOMENT.','I',9X,'I'/&
 & T1,'I',11X,'II',11X,'II',8X,2('I',8X),3('I',7X),&
 & 3('I',8X),'II',11X,'I',9X,'I')  
5204 FORMAT(T1,'I',11X,'II',11X,'II',8X,2('I',8X),3('I',7X),&
 & 2('I',8X),'I',8X,'II',11X,'I',9X,'I'/&
 & T1,'I',' EQUATOR.1 ','II',F11.0,'II',F8.0,2('I',F8.0),&
 & 3('I',F7.0),3('I',F8.0),'II',F11.0,&
 & 'I',F9.0,'I'/T1,'I',11X,'II',11X,'II',8X,2('I',8X),&
 & 3('I',7X),3('I',8X),'II',11X,'I',9X,'I')  
5205 FORMAT(T1,'I',11X,'II',11X,'II',8X,2('I',8X),3('I',7X),&
 & 3('I',8X),'II',11X,'I',9X,'I'/&
 & T1,'I',' EQUATOR.2 ','II',F11.0,'II',F8.0,2('I',F8.0),&
 & 3('I',F7.0),3('I',F8.0),'II',F11.0,&
 & 'I',F9.0,'I'/T1,'I',11X,'II',11X,'II',8X,2('I',8X),&
 & 3('I',7X),3('I',8X),'II',11X,'I',9X,'I')  
5206 FORMAT(T1,'I',11X,'II',11X,'II',8X,2('I',8X),3('I',7X),&
 & 3('I',8X),'II',11X,'I',9X,'I'/&
 & T1,'I',' POLAR     ','II',F11.0,'II',F8.0,2('I',F8.0),&
 & 3('I',F7.0),2('I',F8.0),'I',8X,'II',F11.0,&
 & 'I',F9.0,'I'/T1,'I',11X,'II',11X,'II',8X,2('I',8X),&
 & 3('I',7X),3('I',8X),'II',11X,'I',9X,'I')  
5207 FORMAT(/T10,'POSITION OF INTERSECTION OF THE ANGULAR MOMENTUM ',&
 & 'VECTOR WITH THE ROTATING EARTH :')  
5208 FORMAT(T20,'LATITUDE : ',F9.4,' DEGREES'/T20,'LONGITUDE: ',F9.4/)
5209 FORMAT(T10,'! ERROR !!!   THE ANGULAR MOMENTUM IS ZERO'/)
5210 FORMAT(T20,'*** POSITION AT THE NORTH POLE ***'/)
IF (LHOOK) CALL DR_HOOK('PPEDDH',1,ZHOOK_HANDLE)
END SUBROUTINE PPEDDH
