SUBROUTINE PPFLUSH

!**** *PPFLUSH*  - Flush post processing data

!     Purpose.
!     --------
!     Flush contents of post processing file buffers at the end
!     of a post processing timestep

!        Explicit arguments :     
!        --------------------    

!     Method.
!     -------
!        See documentation

!     Externals.  PBWRITE - 
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        George Mozdzynski *ECMWF*

!     Modifications.
!     --------------
!       Original : 97-10-15 
!       Modified : 00-12-08 G.Mozdzynski: MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MYPROC
USE YOMIO    , ONLY : NPPFILES ,NPPBUF   ,NPPBUFL  ,NPPBUFH  ,&
 & NPPBUFS  ,NPPBUF2  ,NPPBUFA  ,NPPBUFWRI,RPPBUFWRT,&
 & RPPBUFWAT,CPPLFN   ,NPPOUTTYPE  
USE YOMLUN   , ONLY : NULOUT
USE YOMMSC   , ONLY : NINTLEN
USE MPL_MODULE

IMPLICIT NONE

CHARACTER (LEN = 24) ::  CLFN
INTEGER(KIND=JPIM) :: IDUM(1)

INTEGER(KIND=JPIM) :: I2, IBUF, IERR, ILEN, IOP, IREQ, IU, J

REAL(KIND=JPRB) :: Z0, Z1, Z2, Z3
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!     LOOP OVER OPEN FILES

IF (LHOOK) CALL DR_HOOK('PPFLUSH',0,ZHOOK_HANDLE)

DO J=1,NPPFILES
  IF( NPPBUFS(J) == 1 )THEN
    IBUF=J
    CLFN=CPPLFN(IBUF)
    IU=NPPBUFH(IBUF)
    I2=NPPBUF2(IBUF)

!     THERE IS DATA TO WRITE OUT, SO DO IT AND UPDATE STATS

    Z0=0.0_JPRB
    Z1=0.0_JPRB
    Z2=0.0_JPRB
    Z3=0.0_JPRB
    IF( NPPOUTTYPE(IBUF) == 1 )THEN
      IF( NPPBUFL(IBUF) > 0 )THEN
        CALL USER_CLOCK(PELAPSED_TIME=Z0)
        IF( MYPROC == 1 )THEN
          ILEN=NPPBUFL(IBUF)*NINTLEN
          CALL PBWRITE(IU,NPPBUF(1,I2,IBUF),ILEN,IERR)
          IF( IERR < 0 )THEN
            WRITE(NULOUT,'("PPFLUSH: ERROR ON PBWRITE, IERR=",I6,&
             & " FILE=",A)')IERR,CPPLFN(IBUF)  
            CALL ABOR1('PPFLUSH: ERROR ON PBWRITE')
          ENDIF
        ENDIF
        CALL USER_CLOCK(PELAPSED_TIME=Z1)
        NPPBUFWRI(IBUF)=NPPBUFWRI(IBUF)+1
      ENDIF
    ELSEIF( NPPOUTTYPE(IBUF) == 3 )THEN
      IF( NPPBUFL(IBUF) > 0 )THEN

!             PERFORM MPI_FILE_WRITE_SHARED

        CALL USER_CLOCK(PELAPSED_TIME=Z0)
        IOP=1
        ILEN=NPPBUFL(IBUF)
        CALL MPL_WRITE(IU,IOP,NPPBUF(1:ILEN,I2,IBUF),ILEN,0,IERR)
        IF( IERR /= 0 )THEN
          WRITE(NULOUT,'("PPFLUSH: PROBLEM ON MPL_WRITE ",&
           & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
          CALL ABOR1('FLUSH: PROBLEM ON MPL_WRITE')
        ENDIF
        CALL USER_CLOCK(PELAPSED_TIME=Z1)
        NPPBUFWRI(IBUF)=NPPBUFWRI(IBUF)+1
      ENDIF
    ELSEIF( NPPOUTTYPE(IBUF) == 4 )THEN

!             PERFORM MPI_FILE_WRITE_ORDERED

      CALL USER_CLOCK(PELAPSED_TIME=Z0)
      IOP=2
      ILEN=NPPBUFL(IBUF)
      CALL MPL_WRITE(IU,IOP,NPPBUF(1:ILEN,I2,IBUF),ILEN,0,IERR)
      IF( IERR /= 0 )THEN
        WRITE(NULOUT,'("PPFLUSH: PROBLEM ON MPL_WRITE ",&
         & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
        CALL ABOR1('FLUSH: PROBLEM ON MPL_WRITE')
      ENDIF
      CALL USER_CLOCK(PELAPSED_TIME=Z1)
      NPPBUFWRI(IBUF)=NPPBUFWRI(IBUF)+1
    ELSEIF( NPPOUTTYPE(IBUF) == 5 )THEN
      IF( NPPBUFL(IBUF) > 0 )THEN

!             PERFORM MPI_FILE_IWRITE_SHARED

        CALL USER_CLOCK(PELAPSED_TIME=Z0)
        IOP=3
        ILEN=NPPBUFL(IBUF)
        CALL MPL_WRITE(IU,IOP,NPPBUF(1:ILEN,I2,IBUF),ILEN,IREQ,IERR)
        IF( IERR /= 0 )THEN
          WRITE(NULOUT,'("PPFLUSH: PROBLEM ON MPL_WRITE ",&
           & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
          CALL ABOR1('PPFLUSH: PROBLEM ON MPL_WRITE')
        ENDIF
        CALL USER_CLOCK(PELAPSED_TIME=Z1)
        NPPBUFA(1,I2,IBUF)=IREQ
        NPPBUFA(2,I2,IBUF)=1
        NPPBUFWRI(IBUF)=NPPBUFWRI(IBUF)+1

!             SWITCH BUFFER AND WAIT FOR ANY OUTSTANDING I/O TO COMPLETE
!             BY PERFORMING MPI_WAIT

        NPPBUF2(IBUF)=2-NPPBUF2(IBUF)+1
        I2=NPPBUF2(IBUF)
        IF( NPPBUFA(2,I2,IBUF) == 1 )THEN
          CALL USER_CLOCK(PELAPSED_TIME=Z2)
          IOP=5
          IREQ=NPPBUFA(1,I2,IBUF)
          CALL MPL_WRITE(IU,IOP,IDUM,0,IREQ,IERR)
          IF( IERR /= 0 )THEN
            WRITE(NULOUT,'("PPFLUSH: PROBLEM ON MPL_WRITE ",&
             & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
            CALL ABOR1('PPFLUSH: PROBLEM ON MPL_WRITE')
          ENDIF
          NPPBUFA(1,I2,IBUF)=0
          NPPBUFA(2,I2,IBUF)=0
          CALL USER_CLOCK(PELAPSED_TIME=Z3)
        ENDIF
      ENDIF
    ELSEIF( NPPOUTTYPE(IBUF) == 6 )THEN

!             PERFORM MPI_FILE_WRITE_ORDERED_BEGIN

      CALL USER_CLOCK(PELAPSED_TIME=Z0)
      IOP=4
      ILEN=NPPBUFL(IBUF)
      CALL MPL_WRITE(IU,IOP,NPPBUF(1:ILEN,I2,IBUF),ILEN,IREQ,IERR)
      IF( IERR /= 0 )THEN
        WRITE(NULOUT,'("PPFLUSH: PROBLEM ON MPL_WRITE ",&
         & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
        CALL ABOR1('PPFLUSH: PROBLEM ON MPL_WRITE')
      ENDIF
      CALL USER_CLOCK(PELAPSED_TIME=Z1)
      NPPBUFA(1,I2,IBUF)=IREQ
      NPPBUFA(2,I2,IBUF)=1
      NPPBUFWRI(IBUF)=NPPBUFWRI(IBUF)+1

!             SWITCH BUFFER AND WAIT FOR ANY OUTSTANDING I/O TO COMPLETE
!             BY PERFORMING MPI_FILE_WRITE_ORDERED_END

      NPPBUF2(IBUF)=2-NPPBUF2(IBUF)+1
      I2=NPPBUF2(IBUF)
      IF( NPPBUFA(2,I2,IBUF) == 1 )THEN
        CALL USER_CLOCK(PELAPSED_TIME=Z2)
        IOP=6
        IREQ=NPPBUFA(1,I2,IBUF)
        CALL MPL_WRITE(IU,IOP,IDUM,0,IREQ,IERR)
        IF( IERR /= 0 )THEN
          WRITE(NULOUT,'("PPFLUSH: PROBLEM ON MPL_WRITE ",&
           & "FILE=",A," IOP=",I1," IERR=",I6)')CLFN,IOP,IERR  
          CALL ABOR1('PPFLUSH: PROBLEM ON MPL_WRITE')
        ENDIF
        NPPBUFA(1,I2,IBUF)=0
        NPPBUFA(2,I2,IBUF)=0
        CALL USER_CLOCK(PELAPSED_TIME=Z3)
      ENDIF
    ELSE
      CALL ABOR1('PPFLUSH: NPPOUTTYPE INVALID')
    ENDIF

    RPPBUFWRT(IBUF)=RPPBUFWRT(IBUF)+Z1-Z0
    RPPBUFWAT(IBUF)=RPPBUFWAT(IBUF)+Z3-Z2

!         BUFFER NOW EMPTY

    NPPBUFL(IBUF)=0
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('PPFLUSH',1,ZHOOK_HANDLE)
END SUBROUTINE PPFLUSH
