SUBROUTINE PREGRBENC(KGRIBCD,KLEV,CDREPR,CDLTYPE,&
 & KSEC0,KSEC1,KSEC2,KSEC3,KSEC4,&
 & PDATA,LDFPOSRES,KSMAX,KPTS,KDGLG,KLOENG,PFIRSTLAT,PLASTLAT,&
 & LDGEOPRESET)

!**** *PREGRBENC*  - Modify GRIB headers

!     Purpose.  To modify the GRIB headers before calling GRIBEX.
!     --------

!**   Interface.
!     ----------
!        *CALL* *PREGRBENC(...)

!        Explicit arguments :   KGRIBCD - GRIB code             (input)
!        --------------------   KLEV    - level                 (input)
!                               CDREPR  - representation type   (input)
!                               CDLTYPE - level type            (input)
!                               KSEC0 to KSEC4 - ???            (in-out)
!                               PDATA   - data (spectral only)  (input)
!                               LDFPOSRES - Full-Pos output res.(input)
!                               KSMAX   - spectral truncation   (input)
!                               KPTS    - number of grid points (input)
!                               KDGLG   - numner of latitudes   (input,optional)
!                               KLOENG  - number of points on each latitude (input,optional)
!                               PFIRSTLAT - latitude of first grid row (input, optional)
!                               PLASTLAT  - latitude of last grid row (input, optional)

!        Implicit arguments :  Common YOMGRB
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.   FITSPECTRUM - estimation of P for spectral data
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!       Original : 93-10-18
!       Modified : 95-01-25 M.Hamrud (packing factors for clouds etc.)
!       Modified : 95-10-03 N. Wedi ( new sections MSEC etc. )
!       Modified : 98-10-31 D.Dent  ( MSTEPLPP changed to NSTEPLPP)
! temporary version valid only for one subdomain 
!       Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!        - removal of a commented part adding some polar values.
!       Modified : 02-04-03 Y. Tremolet: Added arguments for resolution
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Tremolet    10-Nov-2004 Option to force ECMWF local GRIB definition
!        P.Viterbo     07-Jan-2005 Higher accuracy for FSR and SD
!        R.Buizza : 10-03-2005 Change GRIB header for VAREPS
!        Y.Tremolet    29-Mar-2005 Remove KSECLOC
!        M.Fisher      03-Mar-2005 Allow different grid resolutions
!        G. Desroziers and K. Yessad (sept 2005):
!         - split option LTRAJHR into LTRAJHR_ALTI and LTRAJHR_SURF.
!         - adapt option LTRAJHR to METEO-FRANCE configurations.
!        S. Serrar     07-Sep-2006 scaling for a few tracers (table 210)
!        S. Serrar : 22-03-2007 GRIB parameters for table 162 
!        P. Lopez : 11-11-2008 Bitmap representation for ducting diagnostics
!                              (table 228, codes 015 to 019)
!        Y. Takaya : 29-01-2009 GRIB parameters for table 151
!        P. Bechtold + Nils Wedi : 13-03-2009 Add extended 128 table 228
!        T. Wilhelmsson : 25-Mar-10 Add 6 hourly min/max fields
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI
USE YOMLUN   , ONLY : NULOUT, NULERR
USE YOMCT0   , ONLY : CTYPE    ,NCONF    ,LOBSC1   ,NSTEPINI,LFPOS
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NSPEC2G,NFLEVG,NSMAX,NDGLG
USE SURFACE_FIELDS_MIX  , ONLY : YSP_SB, YSP_SBD
USE YOMDYN   , ONLY : TSTEP
USE YOMFPC   , ONLY : CFPFMT
USE YOMGRB   , ONLY : MSEC0,  MSEC1,    MSEC3,  MSEC4    ,&
 & NSTEPLPP ,MSEC2GG  ,MSEC2SPP, MSEC2SPM,&
 & NSEC0,  NSEC1,    NSEC3,  NSEC4, &
 & NSEC2GG  ,NSEC2SPP, NSEC2SPM,  &
 & NLOCGRB  ,MSMAXNP  ,NBITSSH  ,NGRBZ    ,&
 & NGRBLNSP ,NGRBMX2T ,NGRBMN2T ,NGRBMX2T6,NGRBMN2T6,NGRBCLWC ,NGRBCIWC ,&
 & NGRBCC   ,NGRBSD   ,NGRBFSR  ,NGRB10FG ,NGRB10FG6,NSMAXNP  ,&
 & NLEG     ,NREFERENCE   ,NGRBBTMP
USE YOMVAR   , ONLY : NUPTRA, MUPTRA, NSIM4D, LTWGRA, LTWINC, LTWBGV, LTWCGL,&
 & NFGFCLEN, LBGM
USE YOMANEB  , ONLY : NANERADS
USE YOMSKF   , ONLY : LTWSKFY, LTWSKFZ
USE YOMFPD   , ONLY : NLAT
USE YOMFPG   , ONLY : NFPMAX   ,NFPRGRI  ,NFPLEV
USE YOMGEM   , ONLY : NLOENG
USE YOMLCZ   , ONLY : NRITZREAD, NRITZNUMB, NRITZGBH1, NRITZGBH2
USE YOMVAREPS, ONLY : NFCHO_TRUNC_INI, NFCLENGTH_INI, LVAREPS,&
                     &NAFVAREPS,NAFVAREPSGC
USE YOMDYNCORE, ONLY: LPPSTEPS
USE FITSPECTRUM_MOD, ONLY: FITSPECTRUM

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGRIBCD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
CHARACTER         ,INTENT(IN)    :: CDREPR*(*) 
CHARACTER         ,INTENT(IN)    :: CDLTYPE*(*) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSEC0(:) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSEC1(:) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSEC2(:) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSEC3(:) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSEC4(:) 
REAL(KIND=JPRB)   ,OPTIONAL,INTENT(IN) :: PDATA(:), PFIRSTLAT, PLASTLAT
LOGICAL           ,OPTIONAL,INTENT(IN) :: LDFPOSRES 
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: KSMAX 
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: KPTS 
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: KDGLG
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN),TARGET :: KLOENG(:)
LOGICAL           , OPTIONAL, INTENT(IN) :: LDGEOPRESET

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

INTEGER(KIND=JPIM) :: IDAY, IDAYLPP, ISPEC2, IHOUR,&
 & IHOURLPP, ILSI, ILSQ, ILST, IPTS, ISEC, ISECLPP, &
 & ISTEPLPP, ISTRUNC, JGL,ILEVS,ISMAX,ILEV,ILATS,IDGLG, &
 & IDLON,ISEC0, IHOUR0, ISECOVERLAP, ISTEP
INTEGER(KIND=JPIM) :: ISHOUR, IHDAY, JAF

INTEGER(KIND=JPIM), POINTER :: ILOENG(:)
INTEGER(KIND=JPIM) :: ISIZE0,ISIZE1,ISIZE2,ISIZE3,ISIZE4
REAL(KIND=JPRB) :: ZBETA0, ZBETA1, ZBOT, ZP, ZTOP
REAL(KIND=JPRB) :: ZRDAT(YSP_SBD%NLEVS),ZRDAW(YSP_SBD%NLEVS)

LOGICAL :: LLFPOSRES,LLJAF,LLBAFERROR
LOGICAL :: LLGEOPRESET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "fcttim.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PREGRBENC',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CALL GSTATS(1718,0)
LLGEOPRESET = .FALSE.
IF(PRESENT(LDGEOPRESET)) LLGEOPRESET=LDGEOPRESET

ISHOUR=3600
IHDAY=24

CALL SURF_INQ(PRDAT=ZRDAT,PRDAW=ZRDAW)
LLBAFERROR = CTYPE == 'ef' .OR. CTYPE == 'eb' .OR. CTYPE == 'ea'

IF(PRESENT(LDFPOSRES)) THEN
  LLFPOSRES=LDFPOSRES
ELSE
  LLFPOSRES=LFPOS
ENDIF

ISIZE0 = SIZE(KSEC0)
ISIZE1 = SIZE(KSEC1)
ISIZE2 = SIZE(KSEC2)
ISIZE3 = SIZE(KSEC3)
ISIZE4 = SIZE(KSEC4)
IF(LLFPOSRES) THEN
  IF(ISIZE0 < SIZE(MSEC0)) CALL ABOR1('PREGRBENC:KSEC0 TOO SMALL!')
  KSEC0(1:SIZE(MSEC0)) = MSEC0(:)
  IF(ISIZE1 < SIZE(MSEC1)) CALL ABOR1('PREGRBENC:KSEC1 TOO SMALL!')
  KSEC1(1:SIZE(MSEC1)) = MSEC1(:)
  IF(.NOT. LLGEOPRESET) THEN
    IF(CDREPR == 'SH') THEN
      IF(CDLTYPE(1:2) == 'ML') THEN
        IF(ISIZE2 < SIZE(MSEC2SPM)) THEN
          WRITE(NULERR,*) ' PREGRBENC, ISIZE2=',ISIZE2,&
           &' SIZE(MSEC2SPM)=',SIZE(MSEC2SPM),' CDLTYPE=',CDLTYPE(1:2)
          CALL ABOR1('PREGRBENC:KSEC2 TOO SMALL!')
        ENDIF
        KSEC2(1:SIZE(MSEC2SPM)) = MSEC2SPM(:)
      ELSE
        IF(ISIZE2 < SIZE(MSEC2SPP)) THEN
          WRITE(NULERR,*) ' PREGRBENC, ISIZE2=',ISIZE2,&
           &' SIZE(MSEC2SPP)=',SIZE(MSEC2SPP),' CDLTYPE=',CDLTYPE(1:2)
          CALL ABOR1('PREGRBENC:KSEC2 TOO SMALL!')
        ENDIF
        KSEC2(1:SIZE(MSEC2SPP)) = MSEC2SPP(:)
      ENDIF
    ELSE
      IF(ISIZE2 < SIZE(MSEC2GG)) THEN
        WRITE(NULERR,*) ' PREGRBENC, CDREPR=',CDREPR,' ISIZE2=',ISIZE2,&
         &' SIZE(MSEC2GG)=',SIZE(MSEC2GG)
        CALL ABOR1('PREGRBENC:KSEC2 TOO SMALL!')
      ENDIF
      KSEC2(1:SIZE(MSEC2GG)) = MSEC2GG(:)
    ENDIF
  ENDIF
  IF(ISIZE3 < SIZE(MSEC3)) CALL ABOR1('PREGRBENC:KSEC3 TOO SMALL!')
  KSEC3(1:SIZE(MSEC3)) = MSEC3(:)
  IF(ISIZE4 < SIZE(MSEC4)) CALL ABOR1('PREGRBENC:KSEC4 TOO SMALL!')
  KSEC4(1:SIZE(MSEC4)) = MSEC4(:)
ELSE
  IF(ISIZE0 < SIZE(NSEC0)) CALL ABOR1('PREGRBENC:KSEC0 TOO SMALL!')
  KSEC0(1:SIZE(NSEC0)) = NSEC0(:)
  IF(ISIZE1 < SIZE(NSEC1)) CALL ABOR1('PREGRBENC:KSEC1 TOO SMALL!')
  KSEC1(1:SIZE(NSEC1)) = NSEC1(:)
  IF(.NOT. LLGEOPRESET) THEN
    IF(CDREPR == 'SH') THEN
      IF(CDLTYPE(1:2) == 'ML') THEN
        IF(ISIZE2 < SIZE(NSEC2SPM)) THEN
          WRITE(NULERR,*) ' PREGRBENC, ISIZE2=',ISIZE2,&
           &' SIZE(NSEC2SPM)=',SIZE(NSEC2SPM),' CDLTYPE=',CDLTYPE(1:2)
          CALL ABOR1('PREGRBENC:KSEC2 TOO SMALL!')
        ENDIF
        KSEC2(1:SIZE(NSEC2SPM)) = NSEC2SPM(:)
      ELSE
        IF(ISIZE2 < SIZE(NSEC2SPP)) THEN
          WRITE(NULERR,*) ' PREGRBENC, ISIZE2=',ISIZE2,&
           &' SIZE(NSEC2SPP)=',SIZE(NSEC2SPP),' CDLTYPE=',CDLTYPE(1:2)
          CALL ABOR1('PREGRBENC:KSEC2 TOO SMALL!')
        ENDIF
        KSEC2(1:SIZE(NSEC2SPP)) = NSEC2SPP(:)
      ENDIF
    ELSE
      IF(ISIZE2 < SIZE(NSEC2GG)) THEN
        WRITE(NULERR,*) ' PREGRBENC, ISIZE2=',ISIZE2,&
         &' SIZE(NSEC2GG)=',SIZE(NSEC2GG),' CDREPR=',CDREPR
        CALL ABOR1('PREGRBENC:KSEC2 TOO SMALL!')
      ENDIF
      KSEC2(1:SIZE(NSEC2GG)) = NSEC2GG(:)
    ENDIF
  ENDIF
  IF(ISIZE3 < SIZE(NSEC3)) CALL ABOR1('PREGRBENC:KSEC3 TOO SMALL!')
  KSEC3(1:SIZE(NSEC3)) = NSEC3(:)
  IF(ISIZE4 < SIZE(NSEC4)) CALL ABOR1('PREGRBENC:KSEC4 TOO SMALL!')
  KSEC4(1:SIZE(NSEC4)) = NSEC4(:)
ENDIF

!     ------------------------------------------------------------------

!*       1.    PREPARE GRIBHEADERS
!        -------------------------

!*       1.0   PRELIMINARY CALCULATIONS

!WRITE(NULOUT,*)'PREGRBENC1 - KSEC1(10)',KSEC1(10)
!WRITE(NULOUT,*)'PREGRBENC1 - KSEC1(11)',KSEC1(11)
!WRITE(NULOUT,*)'PREGRBENC1 - KSEC1(12)',KSEC1(12)
!WRITE(NULOUT,*)'PREGRBENC1 - KSEC1(13)',KSEC1(13)

IF(LLFPOSRES) THEN
  ISPEC2 = (NFPMAX(1)+1)*(NFPMAX(1)+2)
  ILEVS = NFPLEV
  ISMAX = NFPMAX(1)
  ISTRUNC  = MSMAXNP
  ILEV = KLEV
ELSE
  ISPEC2 = NSPEC2G
  ILEVS = NFLEVG
  ISMAX = NSMAX
  ISTRUNC  = NSMAXNP
  ILEV = KLEV
ENDIF

IF (PRESENT(KSMAX)) THEN
  ISMAX=KSMAX
  ISPEC2=(KSMAX+1)*(KSMAX+2)
  KSEC2(2)=KSMAX
  KSEC2(3)=KSMAX
  KSEC2(4)=KSMAX
ENDIF

IF (PRESENT(KLOENG)) THEN
  ILOENG => KLOENG
ELSE
  ILOENG => NLOENG
ENDIF

IF (PRESENT(KDGLG)) THEN
  IDGLG=KDGLG
ELSE
  IDGLG=NDGLG
ENDIF

IF (SIZE(ILOENG)<IDGLG) THEN
  WRITE(NULOUT,*)'PREGRBENC - SIZE(ILOENG)/=IDGLG: ',SIZE(ILOENG),IDGLG
  CALL ABOR1 ('PREGRBENC - SIZE(ILOENG)/=IDGLG')
ENDIF

IF (PRESENT(KLOENG) .OR. PRESENT(KDGLG)) THEN
  KSEC2(2)=0
  KSEC2(3)=IDGLG
  KSEC2(10)=IDGLG/2
  KSEC2(23:22+IDGLG) = ILOENG(1:IDGLG)
  IDLON=MAXVAL(ILOENG(1:IDGLG))
  KSEC2(8)=(360*1000*(IDLON-1))/IDLON
  KSEC2(9)=360*1000/IDLON
ENDIF

IF (PRESENT(PFIRSTLAT)) KSEC2(4)=180._JPRB/RPI*1000._JPRB*PFIRSTLAT
IF (PRESENT(PLASTLAT))  KSEC2(7)=180._JPRB/RPI*1000._JPRB*PLASTLAT

!*       1.1   TIME INDICATORS

IF( LPPSTEPS ) THEN
  ISEC  = NSTEP*3600._JPRB  
ELSE
  ISEC  = NSTEP*TSTEP
ENDIF

IF(CTYPE == 'fc'.AND.LOBSC1) THEN
  ISEC=ISEC+NSTEPINI*ISHOUR
ENDIF

!* CHANGE GRIB HEADERS TO WRITE FC IF VAREPS

!   Consider a VAREPS system with two legs: ModelA and modelB

!           t=0              ISEC0  ISECOVERLAP 
!   ModelA:  |-----------------|----x----|

!                             t=0  ISEC
!   ModelB:                    |----x-------------------|------------|

!   In ModelB, to define correctly the time-step in the grib-header, 
!   ISEC is shifted by ISEC0. Then, if ISEC is less than ISECOVERLAP
!   data are written in the overlap stream efov (1034-1032), otherwise 
!   in strem enfo (1035-1033).

ISEC0=0
ISECOVERLAP=0
IF(LVAREPS.AND.(NLEG >= 2)) THEN
  ISEC0=NFCHO_TRUNC_INI*ISHOUR
  ISEC =ISEC0+ISEC
  ISECOVERLAP=NFCLENGTH_INI*ISHOUR
ENDIF

IHOUR0=ISEC0/ISHOUR
IHOUR = ISEC/ISHOUR
IDAY  = IHOUR/IHDAY
IF(LLBAFERROR) THEN
  ISTEP=0
  IF(CTYPE == 'ef') THEN
    ISTEP=NFGFCLEN
  ELSEIF(CTYPE /= 'eb') THEN
    IF(LBGM) ISTEP=NFGFCLEN
  ENDIF
  KSEC1(15) = 1
  KSEC1(16) = ISTEP
  KSEC1(18) = 0
ELSEIF (ISEC <= 0) THEN
  KSEC1(15) = 1
  KSEC1(16) = 0
  KSEC1(17) = 0
  IF(CTYPE == 'an') THEN
    KSEC1(18) = 0
  ELSE
    KSEC1(18) = 1
  ENDIF
ELSE
  IF(LVAREPS) THEN
! VAREPS: if the forecast is in the overlap period set stream equal to the overlap stream
!WRITE(NULOUT,*)'PREGRBENC2 - KSEC1(10)',KSEC1(10)
!WRITE(NULOUT,*)'PREGRBENC2 - KSEC1(11)',KSEC1(11)
!WRITE(NULOUT,*)'PREGRBENC2 - KSEC1(12)',KSEC1(12)
!WRITE(NULOUT,*)'PREGRBENC2 - KSEC1(13)',KSEC1(13)

    IF(ISEC <= ISECOVERLAP .AND. NLEG == 2 ) THEN
      IF(NREFERENCE > 0 ) THEN
          KSEC1(40) = 1032
      ELSE
          KSEC1(40) = 1034
      ENDIF
    ENDIF

! VAREPS: for accumulated variables and variables with fields defined 
! over a time interval (ie variables with LLJAF=T and max and min 2mT 
! and wind gusts), add in the GRIB header both the information of the 
! initial and final accumulation period (the list of accumulated fields 
! is defined in SUVAREPS)

    LLJAF=.FALSE.
    DO JAF=1,NAFVAREPS
      IF ( KGRIBCD == NAFVAREPSGC(JAF) ) LLJAF=.TRUE.
    ENDDO
    IF(LLJAF) THEN
      IHOURLPP = IHOUR0
      IDAYLPP  = IHOURLPP/IHDAY
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOURLPP
        KSEC1(17) = IHOUR
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOURLPP/3
        KSEC1(17) = IHOUR/3
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOURLPP/6
        KSEC1(17) = IHOUR/6
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOURLPP/12
        KSEC1(17) = IHOUR/12
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*24) THEN
        KSEC1(15) = 2
        KSEC1(16) = IDAYLPP
        KSEC1(17) = IDAY
        KSEC1(18) = 2
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(17) = 0
        KSEC1(18) = 10
      ENDIF
      NSTEPLPP(KGRIBCD) = NSTEP
    ENDIF

    IF(KGRIBCD == NGRBMX2T.OR.KGRIBCD == NGRBMN2T.OR.KGRIBCD == NGRB10FG) THEN

! In VAREPS if the LEG is great/equal than 2 then the initial step ISECLPP
! should be shifted by ISEC0 (please note that the final step IHOUR0 has been 
! already shifted few lines above)
 
      IHOURLPP = IHOUR0
      IDAYLPP  = IHOURLPP/IHDAY

      ISTEPLPP = NSTEPLPP(KGRIBCD)
      ISECLPP  = ISTEPLPP*TSTEP+ISEC0
      IHOURLPP = ISECLPP/ISHOUR

      IDAYLPP  = IHOURLPP/IHDAY
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOURLPP
        KSEC1(17) = IHOUR
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOURLPP/3
        KSEC1(17) = IHOUR/3
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOURLPP/6
        KSEC1(17) = IHOUR/6
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOURLPP/12
        KSEC1(17) = IHOUR/12
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*24) THEN
        KSEC1(15) = 2
        KSEC1(16) = IDAYLPP
        KSEC1(17) = IDAY
        KSEC1(18) = 2
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(17) = 0
        KSEC1(18) = 10
      ENDIF
      NSTEPLPP(KGRIBCD) = NSTEP
    ELSEIF(KGRIBCD == NGRBMX2T6.OR.KGRIBCD == NGRBMN2T6.OR.KGRIBCD == NGRB10FG6) THEN

! In VAREPS if the LEG is great/equal than 2 then the initial step ISECLPP
! should be shifted by ISEC0 (please note that the final step IHOUR0 has been 
! already shifted few lines above)

      IF (MOD(ISEC,3600) /= 0) CALL ABOR1('PREGRBENC: 6 hour min/min only on whole hours ')

      IHOURLPP = IHOUR0
      IDAYLPP  = IHOURLPP/IHDAY

      ISECLPP  = MAX(ISEC-6*3600,ISEC0)
      IHOURLPP = ISECLPP/ISHOUR

      IDAYLPP  = IHOURLPP/IHDAY
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOURLPP
        KSEC1(17) = IHOUR
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOURLPP/3
        KSEC1(17) = IHOUR/3
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOURLPP/6
        KSEC1(17) = IHOUR/6
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOURLPP/12
        KSEC1(17) = IHOUR/12
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*24) THEN
        KSEC1(15) = 2
        KSEC1(16) = IDAYLPP
        KSEC1(17) = IDAY
        KSEC1(18) = 2
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(17) = 0
        KSEC1(18) = 10
      ENDIF
    ELSE
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOUR
        KSEC1(18) = 0
      ELSEIF (IHOUR <= 65535) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOUR
        KSEC1(18) = 10
      ELSEIF (IHOUR <= 65535*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOUR/3
        KSEC1(18) = 10
      ELSEIF (IHOUR <= 65535*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOUR/6
        KSEC1(18) = 10
      ELSEIF (IHOUR <= 65535*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOUR/12
        KSEC1(18) = 10
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(18) = 10
      ENDIF
      KSEC1(17) = 0
    ENDIF

  ELSE

    IF(KGRIBCD == NGRBMX2T.OR.KGRIBCD == NGRBMN2T.OR.KGRIBCD == NGRB10FG) THEN
!       these parameters apply to time since the last post-processing step
      ISTEPLPP = NSTEPLPP(KGRIBCD)
      ISECLPP  = ISTEPLPP*TSTEP
      IHOURLPP = ISECLPP/ISHOUR
      IDAYLPP  = IHOURLPP/IHDAY
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOURLPP
        KSEC1(17) = IHOUR
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOURLPP/3
        KSEC1(17) = IHOUR/3
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOURLPP/6
        KSEC1(17) = IHOUR/6
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOURLPP/12
        KSEC1(17) = IHOUR/12
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*24) THEN
        KSEC1(15) = 2
        KSEC1(16) = IDAYLPP
        KSEC1(17) = IDAY
        KSEC1(18) = 2
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(17) = 0
        KSEC1(18) = 10
      ENDIF
      NSTEPLPP(KGRIBCD) = NSTEP
    ELSEIF(KGRIBCD == NGRBMX2T6.OR.KGRIBCD == NGRBMN2T6.OR.KGRIBCD == NGRB10FG6) THEN
!       these parameters apply to time since the last 6 hours

      IF (MOD(ISEC,3600) /= 0) CALL ABOR1('PREGRBENC: 6 hour min/min only on whole hours ')

      ISECLPP  = MAX(ISEC-6*3600,0)
      IHOURLPP = ISECLPP/ISHOUR
      IDAYLPP  = IHOURLPP/IHDAY
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOURLPP
        KSEC1(17) = IHOUR
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOURLPP/3
        KSEC1(17) = IHOUR/3
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOURLPP/6
        KSEC1(17) = IHOUR/6
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOURLPP/12
        KSEC1(17) = IHOUR/12
        KSEC1(18) = 2
      ELSEIF (IHOUR <= 255*24) THEN
        KSEC1(15) = 2
        KSEC1(16) = IDAYLPP
        KSEC1(17) = IDAY
        KSEC1(18) = 2
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(17) = 0
        KSEC1(18) = 10
      ENDIF
    ELSE
      IF (IHOUR <= 255) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOUR
        KSEC1(18) = 0
      ELSEIF (IHOUR <= 65535) THEN
        KSEC1(15) = 1
        KSEC1(16) = IHOUR
        KSEC1(18) = 10
      ELSEIF (IHOUR <= 65535*3) THEN
        KSEC1(15) = 10
        KSEC1(16) = IHOUR/3
        KSEC1(18) = 10
      ELSEIF (IHOUR <= 65535*6) THEN
        KSEC1(15) = 11
        KSEC1(16) = IHOUR/6
        KSEC1(18) = 10
      ELSEIF (IHOUR <= 65535*12) THEN
        KSEC1(15) = 12
        KSEC1(16) = IHOUR/12
        KSEC1(18) = 10
      ELSE
        KSEC1(15) = 2
        KSEC1(16) = IDAY
        KSEC1(18) = 10
      ENDIF
      KSEC1(17) = 0
    ENDIF
  ENDIF
ENDIF
KSEC1(19) = 0
KSEC1(20) = 0

!*        1.2   LEVEL(S)

IF(CDLTYPE(1:2) == 'SF') THEN
  KSEC1(7) = 1
  KSEC1(8) = 0
  KSEC1(9) = 0
  IF(YSP_SBD%NUMFLDS > 0) THEN
    ILST = ISRCHEQ(YSP_SBD%NLEVS,YSP_SB%YT%IGRBCODE(:),1,KGRIBCD)
    IF(ILST <= YSP_SBD%NLEVS) THEN
      KSEC1(7) = 112
      ZBOT     = SUM(ZRDAT(1:ILST))
      ZTOP     = ZBOT-ZRDAT(ILST)
      KSEC1(8) = MIN( ZTOP*100.0_JPRB+0.1_JPRB , 255._JPRB)
      KSEC1(9) = MIN( ZBOT*100.0_JPRB+0.1_JPRB , 255._JPRB)
    ENDIF
  ENDIF
  IF(YSP_SBD%NUMFLDS > 1) THEN
    ILSQ = ISRCHEQ(YSP_SBD%NLEVS,YSP_SB%YQ%IGRBCODE(:),1,KGRIBCD)
    IF(ILSQ <= YSP_SBD%NLEVS) THEN
      KSEC1(7) = 112
      ZBOT     = SUM(ZRDAW(1:ILSQ))
      ZTOP     = ZBOT-ZRDAW(ILSQ)
      KSEC1(8) = MIN( ZTOP*100.0_JPRB+0.1_JPRB , 255._JPRB)
      KSEC1(9) = MIN( ZBOT*100.0_JPRB+0.1_JPRB , 255._JPRB)
    ENDIF
  ENDIF
  IF(YSP_SBD%NUMFLDS > 2) THEN
    ILSI = ISRCHEQ(YSP_SBD%NLEVS,YSP_SB%YTL%IGRBCODE(:),1,KGRIBCD)
    IF(ILSI <= YSP_SBD%NLEVS) THEN
      KSEC1(7) = 112
      ZBOT     = SUM(ZRDAW(1:ILSI))
      ZTOP     = ZBOT-ZRDAW(ILSI)
      KSEC1(8) = MIN( ZTOP*100.0_JPRB+0.1_JPRB , 255._JPRB)
      KSEC1(9) = MIN( ZBOT*100.0_JPRB+0.1_JPRB , 255._JPRB)
    ENDIF
  ENDIF
ELSEIF(CDLTYPE(1:2) == 'ML') THEN
  KSEC1(7) = 109
  KSEC1(8) = ILEV
  KSEC1(9) = ILEV
ELSEIF(CDLTYPE(1:2) == 'PL') THEN
  IF(ILEV >= 100) THEN
    KSEC1(7) = 100
    KSEC1(8) = ILEV/100
    KSEC1(9) = ILEV/100
  ELSE
    KSEC1(7) = 210
    KSEC1(8) = ILEV
    KSEC1(9) = ILEV
  ENDIF
ELSEIF(CDLTYPE(1:2) == 'PV') THEN
  KSEC1(7) = 117
  KSEC1(8) = ILEV
  KSEC1(9) = ILEV
ELSEIF(CDLTYPE(1:2) == 'TH') THEN
  KSEC1(7) = 113
  KSEC1(8) = ILEV
  KSEC1(9) = ILEV
ELSE
  WRITE(NULOUT,'('' PREGRBENC - LEVEL TYPE NOT SUPPORTED '')')
  CALL ABOR1('PREGRBENC: ABOR1 CALLED')
ENDIF

!*        1.3   PARAMETER

IF(KGRIBCD < 256) THEN
  KSEC1(6) = KGRIBCD
ELSEIF(KGRIBCD >= 162100 .AND. KGRIBCD <= 162113) THEN
  KSEC1(1)=162              ! ERA40 GFL fields 
  KSEC1(6) = KGRIBCD-162000
ELSEIF(KGRIBCD >= 151000 .AND. KGRIBCD <= 151255) THEN
  KSEC1(1)=151              ! OCEAN FIELDS
  KSEC1(6) = KGRIBCD-151000
ELSEIF(KGRIBCD > 210000 .AND. KGRIBCD < 228000) THEN
  KSEC1(1)=210              ! Code table for GFL extra-fields
  IF((KGRIBCD >= 210067 .AND. KGRIBCD <= 210070) .OR.  KGRIBCD == 210185) KSEC1(5)=192
  IF(KGRIBCD == 210185 .OR. KGRIBCD == 210182 .OR. KGRIBCD == 210181) KSEC1(23)=18
  KSEC1(6) = KGRIBCD-210000

! Beware when adding a new table - the next IF needs to be changed - DJS

ELSEIF(KGRIBCD >= 228000) THEN
  KSEC1(1)=228              ! extended table for 128 parameters
  KSEC1(6) = KGRIBCD-228000
ELSE
  WRITE(NULERR,*)'pregrbenc: INVALID GRIB CODE SUPPLIED!',KGRIBCD
  WRITE(NULERR,*)'PLEASE USE <256  OR FROM 162100 TO 162113'
  WRITE(NULERR,*)'                 OR FROM 151000 TO 151255'
  WRITE(NULERR,*)'                 OR > 210000'
  CALL ABOR1('')
ENDIF

!*        1.4   PACKING

IF(CDREPR == 'SH') THEN
  KSEC4(2) = NBITSSH
  KSEC4(4) = 64
  IF(KGRIBCD == NGRBZ   .AND.CDLTYPE(1:2) == 'ML') THEN
    KSEC4(17) = 500
  ELSEIF (KGRIBCD == NGRBLNSP.AND.CDLTYPE(1:2) == 'ML'&
     & .AND..NOT.(     LTWGRA .OR. LTWINC .OR. LTWBGV &
     & .OR. LTWCGL .OR. LTWSKFY .OR. LTWSKFZ)) THEN  
    KSEC4(17) = 500
  ELSEIF(ISTRUNC < ISMAX .AND. (.NOT.LLFPOSRES .OR. CFPFMT=="MODEL")) THEN
    CALL FITSPECTRUM(ISMAX,ISPEC2,PDATA,ISTRUNC,ZBETA0,ZBETA1)
    ZP        = -ZBETA1
    ZP        = MAX(-9.9_JPRB,MIN(9.9_JPRB,ZP))
    KSEC4(17) = INT(ZP*1000._JPRB)
  ELSE
    KSEC4(17) = 0
  ENDIF
  KSEC4(18) = ISTRUNC
  KSEC4(19) = ISTRUNC
  KSEC4(20) = ISTRUNC
ELSEIF(CDREPR == 'GG') THEN
  IF(KGRIBCD == NGRBCC) THEN
    KSEC4(2)  = 8
  ELSEIF(KGRIBCD == NGRBSD.OR.KGRIBCD == NGRBFSR) THEN
    KSEC4(2)  = 24
  ELSEIF(KGRIBCD == NGRBCLWC.AND.CDLTYPE(1:2) == 'PL') THEN
    KSEC4(2)  = 12
  ELSEIF(KGRIBCD == NGRBCIWC.AND.CDLTYPE(1:2) == 'PL') THEN
    KSEC4(2)  = 12
  ELSE
    KSEC4(2)  = 16
  ENDIF
  KSEC4(4)  = 0
  KSEC4(17) = 0
  KSEC4(18) = 0
  KSEC4(19) = 0
  KSEC4(20) = 0
ELSE
  WRITE(NULOUT,'('' PREGRBENC - REPR .TYPE NOT SUPPORTED '')')
  CALL ABOR1('PREGRBENC: ABOR1 CALLED')
ENDIF

!*        1.5   TYPE OF DATA/NUMBER OF DATA POINTS

IF(CDREPR == 'SH') THEN
  KSEC4(1) =  ISPEC2
  KSEC4(3) = 128
ELSEIF(CDREPR == 'GG') THEN
  IPTS = 0
  IF(LLFPOSRES) THEN
    ILATS = NLAT(1)
    DO JGL=1,ILATS
      IPTS = IPTS+NFPRGRI(JGL)
    ENDDO
  ELSE
    ILATS = IDGLG
    DO JGL=1,ILATS
      IPTS = IPTS+ILOENG(JGL)
    ENDDO
  ENDIF
  IF (PRESENT(KPTS)) THEN
    IPTS=KPTS
  ENDIF
  KSEC4(1) = IPTS
  KSEC4(3) = 0
ELSE
  WRITE(NULOUT,'('' PREGRBENC - REPR .TYPE NOT SUPPORTED '')')
  CALL ABOR1('PREGRBENC: ABOR1 CALLED')
ENDIF

!*        1.6    VERTICAL CO-ORDINATE TABLE INCLUDED OR NOT

IF(.NOT.LLGEOPRESET) THEN
  IF(CDREPR == 'GG') THEN
    IF(CDLTYPE(1:2) == 'ML') THEN
      KSEC2(12) = 2*(ILEVS+1)
    ELSE
      KSEC2(12) = 0
    ENDIF
  ENDIF
ENDIF

!*        1.7    ECMWF LOCAL GRIB

IF(NLOCGRB == 1) THEN
  IF(LLBAFERROR) THEN
    IF(CTYPE == 'ef') THEN
      KSEC1(39) = 12  
    ELSE
      KSEC1(39) = 13
    ENDIF
  ELSE
    IF(CTYPE == 'an') THEN
      KSEC1(39) = 2
    ELSEIF(CTYPE == '3v') THEN
      KSEC1(39) = 5
    ELSEIF(CTYPE == '4v') THEN
      KSEC1(39) = 6
    ELSEIF(CTYPE == 'cf') THEN
      KSEC1(39) = 10
    ELSEIF(CTYPE == 'pf') THEN
      KSEC1(39) = 11
    ELSEIF(CTYPE == 'cv') THEN
      KSEC1(39) = 65
    ELSE
      KSEC1(39) = 9
    ENDIF
  ENDIF
ENDIF

IF(NLOCGRB == 9) THEN       ! Singular vectors
  KSEC1(42) = NRITZNUMB
  KSEC1(44) = NRITZREAD
  KSEC1(54) = NRITZGBH1
  KSEC1(55) = NRITZGBH2
ENDIF



!     Set local sectors when SVs computation (CONF 601)
!     - KSEC1(37) = 9    (ECMWF local GRID use definition identifier)
!     - KSEC1(39) = 62   (Type: 62 for SVs)
!     - KSEC1(40) = 1035 (Stream: 1035 for ensemble)

KSEC1(37) = NLOCGRB
IF(LLBAFERROR) THEN
  IF(KGRIBCD == NGRBBTMP) THEN
    KSEC1(37) = 14                   ! Local section number
    KSEC1(44) = KLEV
    KSEC1(45) = 1                    ! Scaling factor
    KSEC1(46) = NANERADS             ! No of channels
    KSEC1(48:48+KSEC1(47)-1) = 123   ! Frequencies (dummy for now).
  ENDIF
ENDIF
IF(NLOCGRB == 9.AND.IABS(NCONF)/100 == 6) THEN
  KSEC1(37) = 9
  KSEC1(39) = 62
  KSEC1(40) = 1035
ENDIF

IF(NLOCGRB == 7.AND.IABS(NCONF)/100 == 8) THEN
  KSEC1(42) = NSIM4D
ENDIF

IF (NCONF==131 .AND. LTWINC) THEN
  IF(KGRIBCD < 256) THEN
    KSEC1(1)  = 200
  ELSEIF(KGRIBCD > 210000) THEN
    KSEC1(1) = 211              ! Code table for GFL extra-fields
  ELSE
    WRITE(NULERR,*)'pregrbenc: INVALID GRIB CODE SUPPLIED!',KGRIBCD
    WRITE(NULERR,*)'PLEASE USE <256 OR FROM 210000 TO 210255.'
    CALL ABOR1('')
  ENDIF
  KSEC1(37) = 20
  KSEC1(39) = 33
  KSEC1(42) = NUPTRA
  KSEC1(43) = MUPTRA
ENDIF

IF (.NOT.LECMWF) THEN
  ! remark KY+GD: this seems necessary for use at METEO-FRANCE.
  KSEC1(3)=1
ENDIF

IF (KSEC1(39)==33.OR.KSEC1(39)==35) THEN
  IF(KGRIBCD < 256) THEN
    KSEC1(1)  = 200
  ELSEIF(KGRIBCD > 210000) THEN
    KSEC1(1) = 211              ! Code table for GFL extra-fields
  ELSE
    WRITE(NULERR,*)'pregrbenc: INVALID GRIB CODE SUPPLIED!',KGRIBCD
    WRITE(NULERR,*)'PLEASE USE <256 OR FROM 210000 TO 210255.'
    CALL ABOR1('')
  ENDIF
ENDIF

CALL GSTATS(1718,1)

!     ------------------------------------------------------------------

!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(10)',KSEC1(10)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(11)',KSEC1(11)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(12)',KSEC1(12)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(13)',KSEC1(13)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(14)',KSEC1(14)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(15)',KSEC1(15)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(16)',KSEC1(16)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(17)',KSEC1(17)
!WRITE(NULOUT,*)'END OF PREGRBENC - KSEC1(18)',KSEC1(18)


IF (LHOOK) CALL DR_HOOK('PREGRBENC',1,ZHOOK_HANDLE)
END SUBROUTINE PREGRBENC
