SUBROUTINE PREPFDB

!**** *PREPFDB* - prepare FDB for output

!     Purpose.
!     --------
!     Set FDB parameters

!**   Interface.
!     ----------
!        *CALL* *PREPFDB

!        Explicit arguments :     NONE
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------

!     Externals.  ISETVALFDB ,UPDCAL
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!       Original : 00-02-02 From WR..
!       Modified : 01-05-03  D. Richardson - multi-analysis stream
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R.Buizza : 10-03-2005 VAREPS
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : CTYPE   ,LOBSC1  ,NSTEPINI
USE YOMCT3   , ONLY : NSTEP
USE YOMGRB   , ONLY : NJDIAG ,NJITER ,&
 & NSTREAM, NREFERENCE,  NLEG
USE YOMOP    , ONLY : NFDBREF
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMLUN   , ONLY : NULOUT
USE YOMDYN   , ONLY : TSTEP
USE YOMVAREPS, ONLY : NFCHO_TRUNC_INI, NFCLENGTH_INI, LVAREPS
USE YOMDYNCORE, ONLY: LPPSTEPS

USE FDBSUBS_MOD

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS

CHARACTER :: CLSENSJ*1,CLSENSI*2
CHARACTER :: CLDATE*8,CLTIME*2,CLSTEP*6

INTEGER(KIND=JPIM)  ::  ISEC
INTEGER(KIND=JPIM)  ::  ISTAT,IDD,IMM,IYYYY,IHOUR1,ID1,IM1,IYYYY1,IHOUR,IYYMMD,IHOURS
INTEGER(KIND=JPIM)  ::  IFCHO_RES,IFCDA_INI,ISHOUR,IHDAY
INTEGER(KIND=JPIM) :: ILMO(12)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "updcal.intfb.h"
#include "fcttim.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PREPFDB',0,ZHOOK_HANDLE)

ISHOUR=3600
IHDAY=24

ISEC  = NSTEP*TSTEP
IHOUR = ISEC/ISHOUR

! Stream dependent settings
SELECT CASE (NSTREAM)
CASE(1036) ! Sensitivity forecast
  WRITE(CLSENSJ,'(I1.1)') NJDIAG
  WRITE(CLSENSI,'(I2.2)') NJITER
  ISTAT = ISETVALFDBSUBS(NFDBREF,'diagnostic',CLSENSJ)
  ISTAT = ISETVALFDBSUBS(NFDBREF,'iteration',CLSENSI)
END SELECT


! Date ,time and step settings

IF(CTYPE == 'fg') THEN
  IDD    = NDD(NINDAT)
  IMM    = NMM(NINDAT)
  IYYYY  = NCCAA(NINDAT)
  IHOUR1 = MOD(IHOUR+NCTH(NSSSSS),IHDAY)
  IF(IHOUR+NCTH(NSSSSS) >= IHDAY) THEN
    CALL UPDCAL(IDD,IMM,IYYYY,1,ID1,IM1,IYYYY1,ILMO,NULOUT)
  ELSE
    ID1    = IDD
    IM1    = IMM
    IYYYY1 = IYYYY
  ENDIF
  IYYMMD = ID1+100*IM1+10000*IYYYY1
  WRITE(CLDATE,'(I8.8)') IYYMMD
  ISTAT = ISETVALFDBSUBS(NFDBREF,'date',CLDATE)
  WRITE(CLTIME,'(I2.2)') IHOUR1
  ISTAT = ISETVALFDBSUBS(NFDBREF,'time',CLTIME)
ENDIF

IF(LVAREPS.AND.(NLEG >= 2)) THEN

! VAREPS: define initial date/time for FDB

  IDD    = NDD(NINDAT)
  IMM    = NMM(NINDAT)
  IYYYY  = NCCAA(NINDAT)
  IHOURS = NSSSSS/ISHOUR

  IFCDA_INI=NFCHO_TRUNC_INI/IHDAY
  IFCHO_RES=NFCHO_TRUNC_INI-IFCDA_INI*IHDAY
  IF(IFCHO_RES == 0) THEN

!   VAREPS: if the shift is a full number of days, then:
!           - subtract IFCDA_INI days to date

    CALL UPDCAL(IDD,IMM,IYYYY,-IFCDA_INI,ID1,IM1,IYYYY1,ILMO,NULOUT)
    IHOUR1=IHOURS
  ELSE

!   VAREPS: if the shift is NOT a full number of days, then:
!           - add IFCHO_RES to the initial time IHOURS
!           - if IHOURS+IFCHO_RES is more than 24, then:
!             - add 1 to IFCDA_INI
!             - redefine initial time as IHOURS-24
!           - subtract IFCDA_INI days to date, and 

    IHOUR1=IHOURS+IFCHO_RES
    IF(IHOUR1 >= IHDAY) THEN
      IFCDA_INI=IFCDA_INI+1
      IHOUR1=IHOUR1-IHDAY
      CALL UPDCAL(IDD,IMM,IYYYY,-IFCDA_INI,ID1,IM1,IYYYY1,ILMO,NULOUT)
    ELSE
      CALL UPDCAL(IDD,IMM,IYYYY,-IFCDA_INI,ID1,IM1,IYYYY1,ILMO,NULOUT)
    ENDIF
  ENDIF

!   VAREPS: 
!   - change forecast step: add NFCHO_TRUNC_INI to IHOUR 
!   - check if overlap period, then write in stream STREAMOVERLAP

  IHOUR=IHOUR+NFCHO_TRUNC_INI

  IF(IHOUR <= NFCLENGTH_INI) THEN
    IF(NREFERENCE > 0 ) THEN   
        ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','efho')
    ELSE
        ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','efov')
    ENDIF
  ELSE
    IF(NREFERENCE > 0 ) THEN   
        ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','enfh')
    ELSE
        ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','enfo')
    ENDIF
 ENDIF 

!   VAREPS: define data and time

  IYYMMD = ID1+100*IM1+10000*IYYYY1
  WRITE(CLDATE,'(I8.8)') IYYMMD
  IF(NREFERENCE > 0 ) THEN   
     ISTAT = ISETVALFDBSUBS(NFDBREF,'hdate',CLDATE)
  ELSE
     ISTAT = ISETVALFDBSUBS(NFDBREF,'date',CLDATE)
  ENDIF
  WRITE(CLTIME,'(I2.2)') IHOUR1
  ISTAT = ISETVALFDBSUBS(NFDBREF,'time',CLTIME)
ELSE
  IF(CTYPE == 'fc'.AND.LOBSC1) THEN
    IDD    = NDD(NINDAT)
    IMM    = NMM(NINDAT)
    IYYYY  = NCCAA(NINDAT)
    IHOURS=NSSSSS/ISHOUR
    IHOUR1=IHOURS-NSTEPINI
    IHOUR=IHOUR+NSTEPINI
    IF(IHOUR1 < 0) THEN
      IHOUR1=IHOUR1+IHDAY
      CALL UPDCAL(IDD,IMM,IYYYY,-1,ID1,IM1,IYYYY1,ILMO,NULOUT)
    ELSE
      IYYYY1=IYYYY
      IM1=IMM
      ID1=IDD
    ENDIF
    IYYMMD = ID1+100*IM1+10000*IYYYY1
    WRITE(CLDATE,'(I8.8)') IYYMMD
    ISTAT = ISETVALFDBSUBS(NFDBREF,'date',CLDATE)
    WRITE(CLTIME,'(I2.2)') IHOUR1
    ISTAT = ISETVALFDBSUBS(NFDBREF,'time',CLTIME)
  ENDIF
ENDIF

IF( LPPSTEPS ) THEN
  WRITE(CLSTEP,'(I6.6)') NSTEP
ELSE
  WRITE(CLSTEP,'(I6.6)') IHOUR
ENDIF

ISTAT = ISETVALFDBSUBS(NFDBREF,'step',CLSTEP)
IF (LHOOK) CALL DR_HOOK('PREPFDB',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE PREPFDB
