SUBROUTINE SPMCUF

! Purpose:
! --------
!   *SPMCUF*  - Filters ln P_s with a recursive high-pass filter
!               providing a diagnostic instantaneous field for 
!               monitoring the update frequency of the coupling files 
!               for ALADIN

! Interface:
! ----------
!   *CALL* *SPMCUF*

! Externals:
! ----------
!   None.

! Method:
! -------
!   See documentation.

! Reference:
! ----------
!   Mon. Wea. Rev., 132, 2130-2141

! Author:
! -------
!   01-Sep-2004 Piet Termonia                    *RMI Belgium*

! Modicifations:
! --------------
! End Modifications
!------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMCT3    ,ONLY : NSTEP
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK
USE YOMDIM    ,ONLY : NSPEC2
USE YOMSP     ,ONLY : SPSP
USE YOMMP    , ONLY : MYSETV   ,NBSETSP
USE YOMMCUF   ,ONLY : NCUFNR   ,NCUFOR   ,RMCUFA   ,RMCUFB   ,RMCUFSP  ,&
                     &RMCUFFP
IMPLICIT NONE

INTEGER(KIND=JPIM) :: JOR      ,JNR      ,JSP
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

!------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPMCUF',0,ZHOOK_HANDLE)

IF (MYSETV /= NBSETSP) THEN
  IF (LHOOK) CALL DR_HOOK('SPMCUF',1,ZHOOK_HANDLE)
  RETURN
ENDIF

! 1. Filter
! ---------

DO JSP=1,NSPEC2
  RMCUFSP(JSP,0)=SPSP(JSP)
ENDDO
IF (NSTEP > NCUFOR) THEN
  DO JNR=1,NCUFNR
    DO JSP=1,NSPEC2
      RMCUFFP(JSP,0,JNR)=RMCUFA(0,JNR)*RMCUFSP(JSP,0)
    ENDDO
    DO JOR=1,NCUFOR
      DO JSP=1,NSPEC2
        RMCUFFP(JSP,0,JNR)=RMCUFFP(JSP,0,JNR)         &
             & + RMCUFA(JOR,JNR)*RMCUFSP(JSP,JOR)     &
             & - RMCUFB(JOR,JNR)*RMCUFFP(JSP,JOR,JNR)
      ENDDO
    ENDDO
  ENDDO
ENDIF

! 2. Time shifting
! ----------------

DO JOR=NCUFOR,1,-1
  DO JSP=1,NSPEC2
    RMCUFSP(JSP,JOR)=RMCUFSP(JSP,JOR-1)
  ENDDO
ENDDO
DO JNR=1,NCUFNR
  DO JOR=NCUFOR,1,-1
    DO JSP=1,NSPEC2
      RMCUFFP(JSP,JOR,JNR)=RMCUFFP(JSP,JOR-1,JNR)
    ENDDO
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('SPMCUF',1,ZHOOK_HANDLE)

END SUBROUTINE SPMCUF
