SUBROUTINE SPNORMBE(PX,PY,PSN,PSM,KLEV,PMET,KFLEV,KFLSUR,LDNWAVE)

!**** *SPNORMBE* - Compute norms in spectral space for KE, multitasking interface

!     Purpose.
!     --------
!        Compute the norm in spectral space, used for KE

!**   Interface.
!     ----------
!        *CALL* *SPNORMBE(PX,PY,PSN,PSM,KLEV,PMET,KFLEV,KFLSUR,LDNWAVE)

!        Explicit arguments :
!        --------------------

!        PX      : Input array nr 1
!        PY      : Input array nr 2
!        PSN     : spectrum at constant n
!        PSM     : spectrum at constant m
!        KLEV    : number of levels of computation
!        PMET    : Metric
!        KFLEV   : first dimensioning of output arrays.
!        KFLSUR  : first dimensioning of input array.
!        LDNWAVE : .TRUE. to compute spectrum at constant n

!        Implicit arguments :  none.
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------
!      Calls SPNORMBM.
!      Called by SPNORM.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-12-20
!        Modified : 95-06-06 L.Isaksen   Reordering of spectral arrays
!        L Isaksen: 97-02-02 Merge with message passing version
!        D Dent   : 97-10-03 replace SECOND by USER_CLOCK
!      R. El Khatib 99-02-09 LDNWAVE ; overdimension NSPECG for PMET
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK
USE YOMDIM   , ONLY : NSPEC2, NUMP, NSPECG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLSUR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX(:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSM(:,:) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMET(:) 
LOGICAL           ,INTENT(IN)    :: LDNWAVE 
REAL(KIND=JPRB) :: ZSN(KFLEV,NUMP), ZSM(KFLEV,NUMP)

INTEGER(KIND=JPIM) :: JLEV, JML
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "spnormbm.intfb.h"

!     ------------------------------------------------------------------

!*       1.    Call SPNORMBM.
!              --------------

IF (LHOOK) CALL DR_HOOK('SPNORMBE',0,ZHOOK_HANDLE)
CALL SPNORMBM(PX,PSN,PSM,KLEV,PMET,KFLEV,KFLSUR,LDNWAVE)

CALL SPNORMBM(PY,ZSN,ZSM,KLEV,PMET,KFLEV,KFLSUR,LDNWAVE)

IF (LDNWAVE) THEN
  DO JLEV=1,KLEV
    DO JML=1,NUMP
      PSN(JLEV,JML)=(PSN(JLEV,JML)+ZSN(JLEV,JML))*0.5_JPRB
    ENDDO
  ENDDO
ENDIF
DO JLEV=1,KLEV
  DO JML=1,NUMP
    PSM(JLEV,JML)=(PSM(JLEV,JML)+ZSM(JLEV,JML))*0.5_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPNORMBE',1,ZHOOK_HANDLE)
END SUBROUTINE SPNORMBE
