SUBROUTINE SUALMDH
!****
!     ------------------------------------------------------------------

!     SUALMDH * DECLARATION DES MASQUES ET DE LEURS AUXILLIAIRES
!     DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!     ------------------------------------

!       -------------------------------------------------------
!     BUT
!     ---
!       COMPTE TENU DES INDICATEURS LOGIQUES, DECLARE ET ALLOUE DE
!     DE LA MEMOIRE AUX MASQUES DES DIFFERENTS DOMAINES HORIZONAUX

!     ARGUMENTS D ENTREE
!     ------------------

!     ARGUMENTS IMPLICITES
!     --------------------
!       UNITE NULOUT
!       INDICATEURS LOGIQUES, COMMON /YOMLDDH/
!         DANS LE COMDECK * YOMMDDH * POUR LES MASQUES
!       DIMENSIONS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /DDHDIM/
!       POINTEURS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /POMMDDH/
!       DECLARATION DES MASQUES ET LEURS AUXILLIAIRES

!     AUTRES ENTREES
!     --------------

!     SORTIES
!     -------
!       INITIALISE LES POINTEURS /POMMDDH/

!     ECRIT PAR
!     --------- ALAIN JOLY

!     13/4/90 - 10/90
!        R. El Khatib : 01-08-07 Pruning options
!        D.Salmond & M.Hortal : 22-Nov-2005 Mods for coarser/finer physics
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPROC    ,NPRINTLEV,LALLOPR
USE YOMLUN   , ONLY : NULOUT
USE YOMLDDH  , ONLY : LSDDH    ,LHDZON   ,LHDDOP
USE YOMMDDH  , ONLY : NDHKD    ,NDHNPU   ,NDHBPX   ,NDHNOM   ,NDHDDX   ,&
 & NDDHLA   ,NDDHPU   ,NDDHI    ,NLRDDH   ,NURDDH   ,NLXDDH   ,&
 & NUXDDH   ,HDSFLA   ,HDSFDU   ,HDSF  
USE YOMPADDH , ONLY : NGLALIST ,NGPUMASK
USE YOMGEM   , ONLY : NGPTOT
USE YOM_PHYS_GRID , ONLY : PHYS_GRID
USE YOMCOAPHY  , ONLY  : NPHYINT

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU, IGPTOT

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUALMDH',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
IF (LLP) THEN
  WRITE (NULOUT,*) ' DDH* DECLARATION DES PLANS MEMOIRES '
  WRITE (NULOUT,*) '      ------------------------------ '

  WRITE (NULOUT,*) '         PLANS BANDES ZONALES'
ENDIF
IF(NPHYINT > 0) THEN
  IGPTOT=PHYS_GRID%NGPTOT
ELSE
  IGPTOT=NGPTOT
ENDIF
!     MASQUES BANDES DE LATITUDES
!     ---------------------------
IF ( LHDZON ) THEN
  ALLOCATE(NDDHLA(IGPTOT))
  IF(LLP)WRITE(IU,9) 'NDDHLA   ',SIZE(NDDHLA),SHAPE(NDDHLA)

!     AUXILLIAIRES DE LECTURE DES DOMAINES INTERNES EN DOMAINES UTILISATEUR
!     ------------------------------------------------------------------
  ALLOCATE(NLRDDH(NDHDDX,NDHKD))
  IF(LLP)WRITE(IU,9) 'NLRDDH   ',SIZE(NLRDDH),SHAPE(NLRDDH)
  ALLOCATE(NLXDDH(NDHKD))
  IF(LLP)WRITE(IU,9) 'NLXDDH   ',SIZE(NLXDDH),SHAPE(NLXDDH)

!     TABLEAUX DE POIDS POUR LES MOYENNES ZONALES
!     ------------------------------------------------------------------
  ALLOCATE(HDSFLA(NDHKD))
  IF(LLP)WRITE(IU,9) 'HDSFLA   ',SIZE(HDSFLA),SHAPE(HDSFLA)
  ALLOCATE(NGLALIST(NDHKD+1,NPROC))
  IF(LLP)WRITE(IU,9) 'NGLALIST ',SIZE(NGLALIST),SHAPE(NGLALIST)

ENDIF

IF ( LHDDOP ) THEN
  IF (LLP) WRITE (NULOUT,*) '         PLANS UTILISATEUR '

!     MASQUES UTILISATEUR
!     -------------------
  ALLOCATE(NDDHPU(IGPTOT,NDHNPU))
  IF(LLP)WRITE(IU,9) 'NDDHPU   ',SIZE(NDDHPU),SHAPE(NDDHPU)

!     AUXILLIAIRES DE LECTURE DES DOMAINES INTERNES EN DOMAINES UTILISATEUR
!     ------------------------------------------------------------------
  ALLOCATE(NURDDH(NDHDDX,0:NDHBPX,NDHNPU))
  IF(LLP)WRITE(IU,9) 'NURDDH   ',SIZE(NURDDH),SHAPE(NURDDH)
  ALLOCATE(NUXDDH(0:NDHBPX,NDHNPU))
  IF(LLP)WRITE(IU,9) 'NUXDDH   ',SIZE(NUXDDH),SHAPE(NUXDDH)

!     TABLEAUX DE POIDS POUR LES MOYENNES HORIZONTALES
!     ------------------------------------------------------------------
  ALLOCATE(HDSFDU(0:NDHBPX,NDHNPU))
  IF(LLP)WRITE(IU,9) 'HDSFDU   ',SIZE(HDSFDU),SHAPE(HDSFDU)
  ALLOCATE(NGPUMASK(NDHNOM,NPROC))
  IF(LLP)WRITE(IU,9) 'NGPUMASK ',SIZE(NGPUMASK),SHAPE(NGPUMASK)
ENDIF

IF ( LSDDH ) THEN
  IF (LLP)WRITE (NULOUT,*) '  DECLARE MASQUE INTERNE ET POIDS LOCAUX '
  ALLOCATE(NDDHI(IGPTOT))
  IF(LLP)WRITE(IU,9) 'NDDHI    ',SIZE(NDDHI ),SHAPE(NDDHI )
  ALLOCATE(HDSF(IGPTOT))
  IF(LLP)WRITE(IU,9) 'HDSF     ',SIZE(HDSF  ),SHAPE(HDSF  )
ENDIF

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUALMDH',1,ZHOOK_HANDLE)
END SUBROUTINE SUALMDH
