SUBROUTINE SUALTDH
!****
!     ------------------------------------------------------------------

!     SUALTDH * DECLARATION DES TABLEAUX DE CUMULS
!     DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!     ------------------------------------

!       -------------------------------------------------------
!     BUT
!     ---
!       COMPTE TENU DES INDICATEURS LOGIQUES, UTILISANT LE RESULTAT
!     DE *SUMDDH* ET LE NOMBRE DE DOMAINES INTERNES (NDHIDH),
!     CE PROGRAMME ALLOUE LA MEMOIRE DES TABLEAUX DE CUMULS

!     ARGUMENTS D ENTREE
!     ------------------

!     ARGUMENTS IMPLICITES
!     --------------------
!       UNITE NULOUT
!       INDICATEURS LOGIQUES, COMMON /YOMLDDH/
!         DANS LE COMDECK * YOMMDDH * POUR LES MASQUES
!       DIMENSIONS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /DDHDIM/
!       ET DIMENSIONS POUR LES TABLEAUX DE CUMULS
!         EN PARTICULIER NDHIDH
!       POINTEURS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /POMMDDH/
!       DECLARATION DES MASQUES ET LEURS AUXILLIAIRES
!         DANS LE COMDECK * YOMTDDH * POUR LES TABLEAUX DE CUMULS
!       POINTEURS PROPRES AUX TABLEAUX DE CUMULS DDH, COMMON /POMTDDH/
!       DECLARATION DES TABLEAUX DE CUMULS

!     AUTRES ENTREES
!     --------------

!     SORTIES
!     -------
!        INITIALISE LES POINTEURS /POMTDDH/
!        INITIALISE LES TABLEAUX DE CUMULS A ZERO

!     ECRIT PAR
!     --------- ALAIN JOLY

!     25/1/91
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud     : 20-Apr-2004 OpenMP improvements of DDH
!        T.Kovacic    : 16-Mar-2007 Allocation of APFT  
!        T.Kovacic    : 01-May-2007 INIAPFT_BP002
!        O.Riviere: 01-oct-2008 removal of iniapft_bp002
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR
USE YOMLUN   , ONLY : NULOUT
USE YOMLDDH  , ONLY : LSDDH
USE YOMMDDH  , ONLY : NDHIDH   ,NDHCS    ,NDHCV
USE YOMTDDH  , ONLY : HDCVB0   ,HDCVB1   ,HDCS0    ,HDCS1
USE OML_MOD, ONLY : OML_MAX_THREADS
USE YOMDIM,    ONLY : NPROMA, NFLEVG
USE YOMPHY   , ONLY : L3MT, LSTRAPRO
USE YOMARPHY , ONLY : LMPA

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU,ITHRMAX

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE


IF (LHOOK) CALL DR_HOOK('SUALTDH',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
ITHRMAX=OML_MAX_THREADS()
IF (LLP) THEN
  WRITE (NULOUT,*) ' DDH * DECLARATION DES TABLEAUX DE CUMULS'
  WRITE (NULOUT,*) ' ----------------------------------------'
  WRITE (NULOUT,*) ' '
  WRITE (NULOUT,*) ' DDH * NDHCV = ',NDHCV,' NDHCS = ',NDHCS,&
   & ' NDHIDH = ',NDHIDH  
ENDIF


!     DECLARATIONS PROPREMENT DITES
!     -----------------------------

IF(NDHCV*(NFLEVG+1)*NDHIDH >= 1) THEN
  ALLOCATE(HDCVB0(NDHCV*(NFLEVG+1),NDHIDH,ITHRMAX))
  IF(LLP)WRITE(IU,9) 'HDCVB0   ',SIZE(HDCVB0),SHAPE(HDCVB0)
  ALLOCATE(HDCVB1(NDHCV*(NFLEVG+1),NDHIDH,ITHRMAX))
  IF(LLP)WRITE(IU,9) 'HDCVB1   ',SIZE(HDCVB1),SHAPE(HDCVB1)
ENDIF
IF(NDHCS*NDHIDH >= 1) THEN
  ALLOCATE(HDCS0(NDHCS,NDHIDH,ITHRMAX))
  IF(LLP)WRITE(IU,9) 'HDCS0    ',SIZE(HDCS0),SHAPE(HDCS0)
  ALLOCATE(HDCS1(NDHCS,NDHIDH,ITHRMAX))
  IF(LLP)WRITE(IU,9) 'HDCS1   ',SIZE(HDCS1),SHAPE(HDCS1)
ENDIF

!*    INITIALISATION DES TABLEAUX DE CUMULS
!     -------------------------------------

IF ( LSDDH ) THEN
  HDCVB0(:,:,:)=0.0_JPRB
  HDCVB1(:,:,:)=0.0_JPRB
  HDCS0(:,:,:)=0.0_JPRB
  HDCS1(:,:,:)=0.0_JPRB
ENDIF


9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUALTDH',1,ZHOOK_HANDLE)
END SUBROUTINE SUALTDH
