SUBROUTINE SUOFNAME(KSTEP,KDIGITS,CDLABEL,CDOFNAME,CDINC)

!**** *SUOFNAME*  - Setup an output filename

!     Purpose.
!     --------
!           To compute an output filename

!     Explicit arguments :
!     --------------------

!           KSTEP    : (input) time step
!           KDIGITS  : (input optional) number of digits for timestamp if timestep >= 0
!           CDLABEL  : (input) file label
!           CDOFNAME : (output) filename (including timestamp)
!           CDINC    : (output optional) timestamp

!     Externals.
!     ----------

!     Author.
!     -------
!        Ryad EL KHATIB *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 23-Oct-2008 from inifaout and others
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMOP    , ONLY : CFPATH   ,LINC     ,NLPATH     
USE YOMCT0   , ONLY : CNMEXP
USE YOMDYN   , ONLY : TSTEP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)            :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN), OPTIONAL  :: KDIGITS
CHARACTER(LEN=*)  ,INTENT(IN)            :: CDLABEL
CHARACTER(LEN=*)  ,INTENT(OUT)           :: CDOFNAME
CHARACTER(LEN=*)  ,INTENT(OUT), OPTIONAL :: CDINC

INTEGER(KIND=JPIM) :: IINC, ILENINC, ILENOFNAME, IDIGITS, IST
CHARACTER(LEN=5)   :: CLSTEP
CHARACTER(LEN=6)   :: CLINC6

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*           COMPUTE TIME STAMP & OUTPUT FILE NAME
!            -------------------------------------

IF (LHOOK) CALL DR_HOOK('SUOFNAME',0,ZHOOK_HANDLE)

IDIGITS=4
IF (PRESENT(KDIGITS)) THEN
  IF (KDIGITS > 6) THEN
    CALL ABOR1('SOUFNAME:KDIGITS COULD NOT EXCEED 6')
  ELSE
    IDIGITS=MAX(1,KDIGITS)
  ENDIF
ENDIF

CDOFNAME=' '
CLINC6  =' '
IST=6-IDIGITS+1

IF (CDLABEL=='ICMCM') THEN

! coupled fields for COM method
! Sure we don't use LINC ?
! Should we support CFPATH ?
  IF (KSTEP < 0) THEN
    ILENOFNAME=LEN(CDLABEL)+4+LEN(CLSTEP)
    IF (ILENOFNAME > LEN(CDOFNAME)) CALL ABOR1('SUOFNAME:CDOFNAME TOO SHORT')
    WRITE(CLSTEP,'(I5.4)') KSTEP
    CDOFNAME=CDLABEL//CNMEXP(1:4)//CLSTEP
    IF (PRESENT(CDINC)) THEN
      ILENINC=LEN(CLSTEP)
      IF (ILENINC > LEN(CDINC)) CALL ABOR1('SUOFNAME:CDINC TOO SHORT')
      CDINC=CLSTEP
    ENDIF
  ELSE
    ILENOFNAME=LEN(CDLABEL)+5+IDIGITS
    IF (ILENOFNAME > LEN(CDOFNAME)) CALL ABOR1('SUOFNAME:CDOFNAME TOO SHORT')
    WRITE(CLINC6,'(I6.6)') IINC
    CDOFNAME=CDLABEL//CNMEXP(1:4)//'+'//CLINC6(IST:6)
    IF (PRESENT(CDINC)) THEN
      IF (IDIGITS > LEN(CDINC)) CALL ABOR1('SUOFNAME:CDINC TOO SHORT')
      CDINC=CLINC6(IST:6)
    ENDIF
  ENDIF

ELSE

! 'standard' ICMSH case
  IF (KSTEP < 0) THEN
    WRITE(CLSTEP,'(I5.4)') KSTEP
    IF (NLPATH >=1) THEN
      ILENOFNAME=NLPATH+LEN(CDLABEL)+4+LEN(CLSTEP)
      IF (ILENOFNAME > LEN(CDOFNAME)) CALL ABOR1('SUOFNAME:CDOFNAME TOO SHORT')
      CDOFNAME=CFPATH(1:NLPATH)//CDLABEL//CNMEXP(1:4)//CLSTEP
    ELSE
      ILENOFNAME=LEN(CDLABEL)+4+LEN(CLSTEP)
      IF (ILENOFNAME > LEN(CDOFNAME)) CALL ABOR1('SUOFNAME:CDOFNAME TOO SHORT')
      CDOFNAME=CDLABEL//CNMEXP(1:4)//CLSTEP
    ENDIF
    IF (PRESENT(CDINC)) THEN
      ILENINC=LEN(CLSTEP)
      IF (ILENINC > LEN(CDINC)) CALL ABOR1('SUOFNAME:CDINC TOO SHORT')
      CDINC=CLSTEP
    ENDIF
  ELSE
    IF (LINC) THEN
      IINC=NINT(REAL(KSTEP,JPRB)*TSTEP/3600._JPRB)
    ELSE
      IINC=KSTEP
    ENDIF
    WRITE(CLINC6,'(I6.6)') IINC
    IF (NLPATH >=1) THEN
      ILENOFNAME=NLPATH+LEN(CDLABEL)+5+IDIGITS
      IF (ILENOFNAME > LEN(CDOFNAME)) CALL ABOR1('SUOFNAME:CDOFNAME TOO SHORT')
      CDOFNAME=CFPATH(1:NLPATH)//CDLABEL//CNMEXP(1:4)//'+'//CLINC6(IST:6)
    ELSE
      ILENOFNAME=LEN(CDLABEL)+5+IDIGITS
      IF (ILENOFNAME > LEN(CDOFNAME)) CALL ABOR1('SUOFNAME:CDOFNAME TOO SHORT')
      CDOFNAME=CDLABEL//CNMEXP(1:4)//'+'//CLINC6(IST:6)
    ENDIF
    IF (PRESENT(CDINC)) THEN
      IF (IDIGITS > LEN(CDINC)) CALL ABOR1('SUOFNAME:CDINC TOO SHORT')
      CDINC=CLINC6(IST:6)
    ENDIF
  ENDIF

ENDIF

IF (LHOOK) CALL DR_HOOK('SUOFNAME',1,ZHOOK_HANDLE)

END SUBROUTINE SUOFNAME
