SUBROUTINE SUPPDATE(KDTIME,KEVENTTS,KDIME,KFREVENT,K1EVENT,KPPDATE)

!**** *SUPPDATE*  - Setup post-processind date

!     Purpose.
!     --------
!           To initialize the date to write out on an output file
!           (historical file or post-processing file)

!**   Interface.
!     ----------
!        *CALL* *SUPPDATE(...)*

!        Explicit arguments :
!        ------------------
!         KDTIME   - kind of date for the file : 
!                    0 <=> date of the model
!                    1 <=> date of the starting file + model forecast range
!                   (resulting date will differ if the starting file is already
!                    a forecast)
!         KEVENTTS - array of time step events
!         KDIME    - dimension of KEVENTTS array
!         KFREVENT - frequency events
!         K1EVENT   - overriding switch for events
!         KPPDATE  - date in output file

!        Implicit arguments :
!        --------------------
!        See 'USE MODULE' above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        ABOR1

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     Modifications.
!     --------------
!        ORIGINAL : 99-07-20 from NEWFA
!    R. El Khatib : 99-10-20 model forecast range rounded to the nearest hour 
!                   as the file stamp
!    R. El Khatib : 99-12-21 Fix : If the initial file is initialized,
!                            then the output files are necessarily initialized.
!    R. El Khatib : 00-06-05 : LTIMEP0
!    R. El Khatib : 01-03-16 : KPPDATE(10)=time of the previous event if any ;
!                    =0 else ; LFPART2 more robust
!    R. El Khatib : 03-09-12 : fix for negative NSTEP
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!    R. El Khatib : 05-02-21 : bugfixes on this mess ...
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NINISH   ,NULOUT   ,NPPPSH
USE YOMCT0   , ONLY : NCONF    ,NSTART   ,LFPART2  ,NSTOP
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE YOMINI   , ONLY : NEINI
USE YOMOP    , ONLY : CFNISH   ,LTIMEP0  ,CFNPL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)           :: KDIME 
INTEGER(KIND=JPIM),INTENT(IN)           :: KDTIME 
INTEGER(KIND=JPIM),INTENT(IN), OPTIONAL :: KEVENTTS(0:KDIME) 
INTEGER(KIND=JPIM),INTENT(IN), OPTIONAL :: KFREVENT
INTEGER(KIND=JPIM),INTENT(IN), OPTIONAL :: K1EVENT
INTEGER(KIND=JPIM),INTENT(OUT)          :: KPPDATE(11) 
! IHH0  : forecast time range in the initial file (in hours)
! IHH   : forecast time range since the initial file date (in hours)
! IDATEF: date in starting file (if KDTIME=1 only) 

INTEGER(KIND=JPIM) :: IDATEF(11)
INTEGER(KIND=JPIM), ALLOCATABLE :: IEVENTS(:)

INTEGER(KIND=JPIM) :: IHH, IHH0, INBARI, IREP, JS, IPRESTEP, ISS, ILS

REAL(KIND=JPRB) :: ZISS, ZISSX, ZLS, ZTSTEP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

CHARACTER (LEN = 16) ::  CLLEC

#include "abor1.intfb.h"

#include "fcttim.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPPDATE',0,ZHOOK_HANDLE)

!*    1.    DECIDE SUITABLE DATE
!           --------------------


IF (KDTIME == 0) THEN

! Year : 
  KPPDATE(1) = NCCAA(NINDAT)
! Month : 
  KPPDATE(2) = NMM(NINDAT)
! Day : 
  KPPDATE(3) = NDD(NINDAT)
! Hour in the day (0 or 12) : 
  KPPDATE(4) = NSSSSS / 3600
! Minute in the hour (0) : 
  KPPDATE(5) = 0
  IHH0=0

ELSEIF(KDTIME == 1) THEN

  CLLEC='CADRE LECTURE   '
  INBARI=0
  IF (.NOT.LFPART2) THEN
    CALL FAITOU(IREP,NINISH,.TRUE.,CFNISH,'OLD',.TRUE.,.TRUE.,1,1,INBARI,CLLEC)
    CALL LFIMST(IREP,NINISH,.FALSE.)
    CALL FADIES(IREP,NINISH,IDATEF)
    CALL FAIRME(IREP,NINISH,'UNKNOWN')
  ELSE
    CALL FAITOU(IREP,NPPPSH,.TRUE.,CFNPL,'OLD',.TRUE.,.TRUE.,1,1,INBARI,CLLEC)
    CALL LFIMST(IREP,NPPPSH,.FALSE.)
    CALL FADIES(IREP,NPPPSH,IDATEF)
    CALL FAIRME(IREP,NPPPSH,'UNKNOWN')
  ENDIF
! Year : 
  KPPDATE(1) = IDATEF(1)
! Month : 
  KPPDATE(2) = IDATEF(2)
! Day : 
  KPPDATE(3) = IDATEF(3)
! Hour in the day (0 or 12) : 
  KPPDATE(4) = IDATEF(4)
! Minute in the hour (0) : 
  KPPDATE(5) = IDATEF(5)
  IF (IDATEF(7) == 0) THEN
!   Analysis : 
    IHH0=0
  ELSE
!   forecast : 
    IF (IDATEF(6) == 1) THEN
      IHH0=IDATEF(7)*3600
    ELSEIF(IDATEF(6) == 2) THEN
      IHH0=IDATEF(7)*3600*24
    ELSE
      CALL ABOR1 (' SUPPDATE : UNKNOWN VALUE IDATEF(6)')
    ENDIF
  ENDIF

ELSE

  CALL ABOR1('SUPPDATE : UNKNOWN VALUE KDTIME')

ENDIF

IHH=NINT((REAL(NSTEP,JPRB)*TSTEP+IHH0)/3600._JPRB)
IF (IHH <= 0) THEN
  IF (KDTIME == 0) THEN
    KPPDATE(6) = 1
    KPPDATE(7) = 0
    KPPDATE(8) = 0
  ELSE
    KPPDATE(6) = IDATEF(6)
    KPPDATE(7) = IDATEF(7)
    KPPDATE(8) = IDATEF(8)
  ENDIF
  IF (LTIMEP0) THEN
!   Time for 4D-Var 
    KPPDATE(9) = 1
  ELSE
!   Old times of analysis ...
    IF (NCONF == 701 .OR. NCONF == 131) THEN
!     Unitialised analysis : 
      KPPDATE(9) = 0
    ELSEIF (NEINI == 0) THEN
!     If the initial file is initialized, then the output file is 
!     necessarily initialized (we cannot remove the initialization of 
!     initialized data !
      IF (KDTIME == 0) THEN
        CLLEC='CADRE LECTURE   '
        INBARI=0
        CALL FAITOU(IREP,NINISH,.TRUE.,CFNISH,'OLD',.TRUE.,.TRUE.,1,1,INBARI,&
         & CLLEC)  
        CALL LFIMST(IREP,NINISH,.FALSE.)
        CALL FADIES(IREP,NINISH,IDATEF)
        CALL FAIRME(IREP,NINISH,'UNKNOWN')
      ENDIF
      KPPDATE(9) = IDATEF(9)
    ELSE
!     Initialised analysis in any case : 
      KPPDATE(9) = 1
    ENDIF
  ENDIF
  IF (KDTIME == 0) THEN
    KPPDATE(10) = 0
    KPPDATE(11) = 0
  ELSE
    KPPDATE(10) = IDATEF(10)
    KPPDATE(11) = IDATEF(10)
  ENDIF
ELSE
! Forecast : 
  IF (IHH < 65000) THEN
!   In hours : 
    KPPDATE(6) = 1
    KPPDATE(7) = IHH
  ELSE
!   In days :
    KPPDATE(6) = 2
    KPPDATE(7) = IHH/24
  ENDIF
  KPPDATE(8) = 0
  KPPDATE(9) = 10
  IF (NSTEP <= NSTART) THEN
    IF (KDTIME == 1) THEN
       KPPDATE(10) = IDATEF(10)
       KPPDATE(11) = IDATEF(11)
    ELSE
      KPPDATE(10)=0
      KPPDATE(11)=0
    ENDIF
  ELSEIF (PRESENT(KEVENTTS).AND.PRESENT(K1EVENT).AND.PRESENT(KFREVENT)) THEN
!   Compute previous time of output since model start
    ALLOCATE(IEVENTS(0:NSTOP/KFREVENT))
    IEVENTS(:)=0
    IPRESTEP=NSTART
    IF (KEVENTTS(0) >= 1) THEN
      DO JS=1,KEVENTTS(0)
        ISS=KEVENTTS(JS)*KFREVENT
        IF(ISS >= NSTART.AND.ISS <= NSTOP)THEN
          IEVENTS(KEVENTTS(JS))=K1EVENT
        ENDIF
      ENDDO
      DO JS=KEVENTTS(0),1,-1
        ISS=KEVENTTS(JS)*KFREVENT
        IF(ISS >= NSTART.AND.ISS < NSTEP)THEN
          IF (IEVENTS(KEVENTTS(JS)) /= 0) THEN
            IPRESTEP=ISS
            EXIT
          ENDIF
        ENDIF
      ENDDO
    ELSEIF(KEVENTTS(0) <= -1) THEN
      IF(ABS(TSTEP) < TINY(ZLS))THEN
        ZTSTEP=TINY(ZLS)
      ELSE
        ZTSTEP=TSTEP
      ENDIF
      ZISSX=REAL(HUGE(ILS),JPRB)
      DO JS=1,-KEVENTTS(0)
        ZISS=3600._JPRB*REAL(-KEVENTTS(JS),JPRB)*REAL(KFREVENT,JPRB)/ZTSTEP
        ISS=NINT(MIN(ZISSX,ZISS))
        IF(ISS >= NSTART.AND.ISS <= NSTOP)THEN
          IEVENTS(ISS/KFREVENT)=K1EVENT
        ENDIF
      ENDDO
      DO JS=-KEVENTTS(0),1,-1
        ZISS=3600._JPRB*REAL(-KEVENTTS(JS),JPRB)*REAL(KFREVENT,JPRB)/ZTSTEP
        ISS=NINT(MIN(ZISSX,ZISS))
        IF(ISS >= NSTART.AND.ISS < NSTEP)THEN
          IF (IEVENTS(ISS/KFREVENT) /= 0) THEN
            IPRESTEP=ISS
            EXIT
          ENDIF
        ENDIF
      ENDDO
    ELSE
      DO JS=NSTART,NSTOP
        IF(MOD(JS,KFREVENT) == 0)THEN
          IEVENTS(JS/KFREVENT)=K1EVENT
        ENDIF
      ENDDO 
      DO JS=NSTEP-1, NSTART, -1
        IF(MOD(JS,KFREVENT) == 0)THEN
          IF (IEVENTS(JS/KFREVENT) /= 0) THEN
            IPRESTEP=JS
            EXIT
          ENDIF
        ENDIF
      ENDDO
    ENDIF
    IHH=NINT((REAL(IPRESTEP,JPRB)*TSTEP+IHH0)/3600._JPRB)
    IF (KPPDATE(6) == 1) THEN
!     In hours :
      KPPDATE(10) = IHH
    ELSEIF(KPPDATE(6) == 2) THEN
!     In days :
      KPPDATE(10) = IHH/24
    ENDIF
    KPPDATE(11) = 0
    DEALLOCATE(IEVENTS)
  ELSE
!   No past event, put 0.
    KPPDATE(10) = 0
    KPPDATE(11) = 0
  ENDIF
ENDIF

!*    2.    PRINT OUT THE DATE
!           ------------------

WRITE(NULOUT,*) 'SUPPDATE WRITES OUT IDATEF = ',KPPDATE

IF (LHOOK) CALL DR_HOOK('SUPPDATE',1,ZHOOK_HANDLE)

END SUBROUTINE SUPPDATE
