SUBROUTINE WMOVPH

!**** *WMOVPH* -

!     Purpose.
!     --------
!     Movies of physical fields

!**   Interface.
!     ----------
!        *CALL* *WMOVPH*

!        Explicit arguments :     none
!        --------------------

!     Method.
!     -------
!        See documentation

!     Author.
!     -------
!        Yves Bouteloup  *METEO-FRANCE*   (like WMOVIE)

!     Modifications.
!     --------------
!        Original : 93-04-01
!        Modified : 97-08-06 P. Lamboley format 4.2
!        Modified : 98-03-13 VALPARAISO
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of the last argument (.TRUE.) in 'CALL DIWRGRID'.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB     ,JPIS
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NPPPSH
USE YOMCT0   , ONLY : NPROC
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE YOMWM    , ONLY : FTHSOR
USE YOMGEM   , ONLY : NGPTOTG  ,NGPTOT
USE YOMMP    , ONLY : MYPROC

IMPLICIT NONE

CHARACTER :: CLCOMM *30
INTEGER(KIND=JPIM) :: ISEC
INTEGER(KIND=JPIS) :: ISHORT(NGPTOTG)
INTEGER(KIND=JPIM) :: ILEV,IBITS,ICOMPRESS
REAL(KIND=JPRB) ::  ZFTHSORG(NGPTOTG)
REAL(KIND=JPRB) :: ZTIME

INTEGER(KIND=JPIM) :: IOPROC, IROF

LOGICAL :: LLOPEN

REAL(KIND=JPRB) :: ZEPS, ZMAX, ZMIN, ZRA, ZRB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "diwrgrid.intfb.h"
#include "wmovieh.intfb.h"
#include "wrimoph.intfb.h"

IF (LHOOK) CALL DR_HOOK('WMOVPH',0,ZHOOK_HANDLE)
IOPROC=1

!     RECV GLOBAL FIELDS ZFTHSORG

IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,1,0,NGPTOT,FTHSOR)
ELSE
  ZFTHSORG(1:NGPTOT)=FTHSOR(1:NGPTOT)
  IF (NPROC > 1)CALL DIWRGRID(IOPROC,1,1,NGPTOTG,ZFTHSORG)
ENDIF

IF (MYPROC == IOPROC) THEN
  INQUIRE (NPPPSH,OPENED=LLOPEN)
  IF (.NOT.LLOPEN) CALL WMOVIEH

!       Write out comments
  CLCOMM='RAYONNEMENT SORTANT'
  ISEC=NSTEP*TSTEP
  ZTIME=REAL(NSTEP,JPRB)*TSTEP/3600._JPRB
  WRITE (NULOUT,100) CLCOMM,ZTIME,ISEC
  100 FORMAT ('WMOVIE : WRITING ',A,'. TIME=',F6.2,' (',I8,'s).' )

  ILEV=0
  ZEPS=EPSILON(1.0_JPRB)*100._JPRB
  IBITS=16
  ICOMPRESS=0
  ZMIN=ZFTHSORG(1)
  ZMAX=ZMIN
  ZMIN=MIN(ZMIN,MINVAL(ZFTHSORG(1:NGPTOTG)))
  ZMAX=MAX(ZMAX,MAXVAL(ZFTHSORG(1:NGPTOTG)))
  IF (ZMAX-ZMIN < ZEPS) THEN
    ZRA=1.0_JPRB
  ELSE
    ZRA=65534._JPRB/(ZMAX-ZMIN)
  ENDIF
  ZRB=-ZRA*(ZMAX+ZMIN)*0.5_JPRB
  IROF=1
  DO IROF=1,NGPTOTG
    ISHORT(IROF)=NINT(ZRA*ZFTHSORG(IROF)+ZRB)
  ENDDO
  CALL WRIMOPH(NSTEP,TSTEP,ILEV,CLCOMM,IBITS,ICOMPRESS,ZRA,&
   & ZRB,ISHORT,NGPTOTG,NPPPSH)  
ENDIF

IF (LHOOK) CALL DR_HOOK('WMOVPH',1,ZHOOK_HANDLE)
END SUBROUTINE WMOVPH
