SUBROUTINE WRCFUPP

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULUSR1
USE YOMCT0   , ONLY : CNMEXP
USE YOMCT1   , ONLY : LRFILAF
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE YOMOP    , ONLY : CNMCA    ,LINC
USE YOMMP    , ONLY : MYPROC   ,NSTROUT
USE YOMCFU   , ONLY : NFDCFU

!**** * WRCFUPP * - ROUTINE TO WRITE SPECIFIC FILE FOR INSTANTANEOUS FLUXES

!     PURPOSE.
!     --------
!        WRITES YOMCFU ON DISK

!**   INTERFACE.
!     ----------
!        *CALL* *WRCFUPP*

!     EXPLICIT ARGUMENTS :  NONE
!     --------------------

!     IMPLICIT ARGUMENTS :
!     --------------------
!        COMMON  YOMCFU

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!         SUPPDATE
!         NEWFA
!         FANOUV
!         FAIRNO
!         FAIRME
!         LFILAF
!         WRCFUDM

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        O.CLARY

!        ORIGINAL : 06/92
!        97-03-13 : JPh. Piedelievre (Distributed Memory)
!        01-08-06 : R. El khatib modularization + debug
!        05-02-22 : R. El khatib change interface to SUPPDATE
!----------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=3) :: CLIDEN
CHARACTER(LEN=16) :: CLFNSH
INTEGER(KIND=JPIM) :: IDATEF(11)
INTEGER(KIND=JPIM) :: IINC, INBARI, INBARP, INIMES, IREP, IDTIME
LOGICAL :: LLIMST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "newfa.intfb.h"
#include "suppdate.intfb.h"
#include "wrfu.intfb.h"

!     -------------------------------------------------------------

!*       1.    PREPARATIONS
!              ------------

IF (LHOOK) CALL DR_HOOK('WRCFUPP',0,ZHOOK_HANDLE)
WRITE(NULOUT,*) ' START  WRCFUPP  '

INBARP=NFDCFU
LLIMST=.TRUE.
INIMES=1
IDTIME=0
CLIDEN='ARP'
IF (NSTEP >= 0 .AND. LINC) THEN
  IINC=NINT(REAL(NSTEP,JPRB)*TSTEP/3600._JPRB)
ELSE
  IINC=NSTEP
ENDIF
WRITE(CLFNSH,'(''ICMFL'',A4,''+'',I4.4)') CNMEXP(1:4),IINC

!*       2.    OPEN FILE
!              ---------

IF (MYPROC == 1) THEN
  CALL SUPPDATE(KDTIME=IDTIME,KDIME=1,KPPDATE=IDATEF)
  CALL NEWFA(INBARP,CNMCA,NULUSR1,CLFNSH,LLIMST,INIMES,CLIDEN,IDATEF)
ELSEIF (MYPROC <= NSTROUT) THEN
  CALL FANOUV(IREP,NULUSR1,.FALSE.,CLFNSH,'NEW',.TRUE.,LLIMST,INIMES,INBARP,&
   & INBARI,CNMCA)  
ENDIF 

!*       3.  WRITE GRID POINT FIELDS ON *FA* FILE
!            -------------------------------------

CALL WRFU(NULUSR1)

!*       4.    CLOSE FILE
!              ----------

IF (MYPROC == 1) THEN
  IF (LRFILAF) THEN
    CALL LFILAF(IREP,NULUSR1,.FALSE.)
  ENDIF
  CALL FAIRME(IREP,NULUSR1,'UNKNOWN')
ELSEIF (MYPROC <= NSTROUT) THEN
  CALL FAIRNO(IREP,NULUSR1,'DELETE')
ENDIF

IF (LHOOK) CALL DR_HOOK('WRCFUPP',1,ZHOOK_HANDLE)
END SUBROUTINE WRCFUPP
