SUBROUTINE WRFU(KNUMER)

!**** * WRFU  * - ROUTINE TO EXTRACT AND WRITE CUMULATED FLUXES

!     PURPOSE.
!     --------
!        WRITES YOMGFU ON DISK

!**   INTERFACE.
!     ----------
!        *CALL* *WRFU*

!     EXPLICIT ARGUMENTS :
!     --------------------
!         KNUMER : logical unit number of the file

!     IMPLICIT ARGUMENTS :
!     --------------------
!        COMMON  YOMGFU

!     METHOD.
!     -------
!     EXTERNALS.
!     ----------

!         FAIENC    (SEE MANIPULATION DE FICHIER ARPEGE)
!         EXTGPFT
!         GRNORMA

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        PHILIPPE COURTIER

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-05-23
!        FICHIERS ARPEGE (O. CLARY D'APRES WRTRA ET CPREP2) : 09/91
!        DIMENSION DE ZREAL: O.CLARY 06/92
!        Modified by C. Dreveton, 29-03-93 : solid and liquid water
!        Modififed: 93-08-18 by R. El Khatib (partial cloudiness)
!        Modified : 94-06-06 by J.L. Ricard (Cleaning LFSOIL)
!        Modififed: 95-06-02 by R. El Khatib (GRNORMA)
!        Modified : 95-10-22 by M.Hamrud (inteface block for EXTGPFT)
!        Modified : 97-02-27 by J.Ph. Piedelievre (Distributed Memory)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of the last argument (.F. or .T.) in 'CALL DIWRGRID'.
!        Modified 99-02-12 by JF Estrade: correct send of aladin map factor
!        Modified 99-02-12 by E.Bazile: (Frozen soil fluxes)
!        Modified 99-04-19 by J.M. Piriou: Moon radiation.
!    R. El Khatib : 99-12-28 remove poles rows
!        Modified : 00-03-29 R. El Khatib Indexes of convection
!        Modified : 00-03 by R. El Khatib - derived types
!        Modified : 01-11 by S. Ivatek-Sahdan - bf - problem in ALADIN extension
!                        zone, all fluxes from NGPTOT_CAP+1 to NGPTOT => 0
!    R. El Khatib : 02-01-17 ALADIN E-zone treatment moved to cnt4.
!    R. El Khatib : 02-21-20 Cleaning
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        -----------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCFU   , ONLY : NFDCFU   ,NTYPCFU  ,TYPE_CFU
USE YOMDIM   , ONLY : NPROMA
USE YOMGEM   , ONLY : NGPTOT
USE YOMCT0   , ONLY : LRPLANE
USE YOMGFUB  , ONLY : GFUBUF
USE YOMGC    , ONLY : GM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMER 
REAL(KIND=JPRB) :: ZREAL(NGPTOT,NFDCFU)
REAL(KIND=JPRB) :: ZGP(NPROMA,NFDCFU)

CHARACTER(LEN=4) :: CLPREF(NFDCFU)
CHARACTER(LEN=16) :: CLSUFF(NFDCFU)
INTEGER(KIND=JPIM) :: ILEV(NFDCFU), IPAR(NFDCFU)

INTEGER(KIND=JPIM) :: IEND, JF, JROF, IST, JKGLO, IOFF, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sc2rdg.intfb.h"
#include "wrgp2fa.intfb.h"

!     ---------------------------------------------------------------

!*       1.  PREPARATIONS

IF (LHOOK) CALL DR_HOOK('WRFU',0,ZHOOK_HANDLE)
IOFF=0
DO JF=1,NTYPCFU
  DO JL=1,TYPE_CFU(JF)%ILEV
    CLPREF(IOFF+JL)=TYPE_CFU(JF)%CLPREF(JL)
    ILEV(IOFF+JL)=TYPE_CFU(JF)%INUMLEV(JL)
    CLSUFF(IOFF+JL)=TYPE_CFU(JF)%CLSUFF
    IPAR(IOFF+JL)=TYPE_CFU(JF)%IPARITY
  ENDDO
  IOFF=IOFF+TYPE_CFU(JF)%ILEV
ENDDO

!*       2.  READ THE BUFFER

IST=1
DO JKGLO=1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  CALL SC2RDG(GFUBUF,JKGLO,IST,IEND,ZGP)
  DO JF=1,NFDCFU
    DO JROF=IST,IEND
      ZREAL(JROF+JKGLO-1,JF)=ZGP(JROF,JF)
    ENDDO
  ENDDO
ENDDO

!       3.  DE-REDUCE VECTOR FIELDS

IF (LRPLANE) THEN
  DO JF=1,NFDCFU
    IF (IPAR(JF) == 1) THEN
      ZREAL(1:NGPTOT,JF)=ZREAL(1:NGPTOT,JF)*GM(1:NGPTOT)
    ENDIF
  ENDDO
ENDIF

!       4.   PACK & WRITE OUT TO FILE

CALL WRGP2FA(ZREAL,CLPREF,ILEV,CLSUFF,NFDCFU,KNUMER)

!     ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRFU',1,ZHOOK_HANDLE)
END SUBROUTINE WRFU
