SUBROUTINE WRMLPPA(CDCONF)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCFU   , ONLY : LCUMFU
USE YOMCT0   , ONLY : LCANARI  ,LELAM    ,LFBDAP   ,&
 & LOBSC1   ,LOUTPUT ,LRPLANE  ,LSPRT    ,LNHDYN   ,&
 & NCONF
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMCT1   , ONLY : LRFILAF
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLSUR   ,NPROMA   ,NSPEC2
USE YOMGC    , ONLY : GM
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG  
USE SURFACE_FIELDS_MIX, ONLY : NSURFL,&
 & SP_SB, YSP_SBD, YSP_SB, SP_SG, YSP_SGD, YSP_SG, &
 & SP_RR, YSP_RRD, YSP_RR, SD_WS, YSD_WSD, YSD_WS, &
 & SD_WW, YSD_WWD, YSD_WW, &
 & SP_EP, YSP_EPD, YSP_EP, SP_X2, YSP_X2D, YSP_X2, &
 & SP_CI, YSP_CID, YSP_CI, &
 & SD_VF, YSD_VFD, YSD_VF, SD_VP, YSD_VPD, YSD_VP, &
 & SD_VV, YSD_VVD, YSD_VV, SD_VD, YSD_VDD, YSD_VD,&
 & SD_VA, YSD_VAD, YSD_VA, SD_VC, YSD_VCD, YSD_VC
USE YOMLUN   , ONLY : NULHWF   ,NULOUT
USE YOMMCC   , ONLY : LMCC01
USE YOMMP    , ONLY : MYPROC   ,NSTROUT
USE YOMOP    , ONLY : CFNHWF   ,MAXFIELD ,NAGDIO   ,NBGOUT
USE YOMPHY   , ONLY : LMPHYS   ,LSOLV
USE YOMPPC   , ONLY : LRSUP
USE YOMSP    , ONLY : SPSPD    ,SPSVD    ,SPNHX    ,SPT      ,SPSP 
USE YOMXFU   , ONLY : LXFU
USE YOM_YGFL , ONLY : YGFL


!     Purpose.
!     --------
!     Write out the model level fields to ARPEGE file

!**   Interface.
!     ----------
!        *CALL* *WRMLPPA(CDCONF)

!        Explicit arguments :     CDCONF - configuration of call
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Note de travail ARPEGE NR 17

!     Author.
!     -------
!        MPP Group *METEO-FRANCE*

!     Modifications.
!     --------------
!   Original 96-02-05 (From WRMLPP)
!   Modified 98-04-06  R. El Khatib No packing for surface geopotential
!   Modified 98-02-13 R. El Khatib DIWRSPE -> DIWRSPE+SPREORD
!   Modified 98-08-10 by K. YESSAD Removal of LRPOLE option.
!    - removal of argument .FALSE. in 'CALL SPEREE'.
!    - removal of the last argument (.FALSE.) in 'CALL DIWRGRID'.
!   Modified 98-09-07 D. Giard Save grid-point fields - finalisation
!   Modified 98-09-10 C. Fischer Change name of passive tracers
!   Modified 99-07-20 M. Hamrud  Remove options
!   Modified 99-07-06 A. Peuch Add ozone
!   Modified 99-07-19 D. Giard LINITER moved from YOMNMIA to YOMINI
!   Modified 99-07-29 R. El Khatib Fix CFNHWF vs NSTROUT
!   Modified 99-12-28 R. El Khatib Remove poles rows
!   Modified 00-07-11 D. Giard Spectral fields written during finalisation
!   Modified 00-08-01 A. Peuch Name of scalar variables
!   Modified 00-11-04 S. Ivatek-Sahdan Merge with EWRMLPPDM + cleaning
!   Modified 01-03-19 S. Ivatek-Sahdan call DIWRSPE -> call DIWRSPE0 
!   Modified 01-03-21 S. Ivatek-Sahdan Add call WRGRIDUA
!   Modified 01-03-22 S. Ivatek-Sahdan bf of mergeing
!   Modified 01-06-25 J. Masek - rescaling of new NH variables
!   R. El Khatib : 01-08-07 Pruning options + wrspeca
!   Modified 01-10-17 P. Marquet: INUMG instead of IFIELD in ZREALG
!   Modified 02-03-08 C. Fischer: rescaling of NH var. moved to wrspeca
!   Modified 02-09-30 P. Smolikova : interface to WRSPECA for d4 in NH
!   O.Spaniel    : 03-04-15 cleaning-added interface wrspeca.h
!   R. El Khatib : 03-04-17 Cleanups
!   K. YESSAD    : 03-07-04 Bugfix for lsprt=true.
!   R. El Khatib : 03-08-08 gfl+get rid of wrspeca dummies since it is called
!                  one here
!   R. El Khatib : 03-08-18 Roughness lengths not packed
!   G. Hello : 03-09-19 rescaling of NH var
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   Modified 02-07-17 P. Marquet add VCLIA for aerosol files
!   M.Hamrud      01-Jul-2006 Revised surface fields
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   Y.Seity        11-Jan-2008 Add CDCONF(1:1)='S' for surfex ouput files
!   R. El Khatib : 15-Sep-2008 I/O savings
!     -------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1), INTENT(IN)   :: CDCONF

!     ------------------------------------------------------------------

CHARACTER(LEN=4):: CLINC
CHARACTER(LEN=8):: CLTYP(MAXFIELD)
CHARACTER(LEN=16):: CLLNOM
CHARACTER(LEN=16):: CLPARA(MAXFIELD)

CHARACTER(LEN=16), ALLOCATABLE :: CLNOMA(:)

INTEGER(KIND=JPIM):: ILEV(MAXFIELD)

INTEGER(KIND=JPIM), ALLOCATABLE :: ILNOMA(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: ILONGD(:)

REAL(KIND=JPRB), ALLOCATABLE :: ZSPT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZREALBUF(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZREALG(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVALCO(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZZVALCO(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPSPD(:,:),ZSPSVD(:,:),ZSPSP(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPNHX(:,:)

INTEGER(KIND=JPIM) :: IUNTIN
INTEGER(KIND=JPIM):: IDIMG, IEND, IFCAN,&
 & IFIELD, IFIELDST, IFISBA, &
 & IFL, JFLD, IREK, &
 & IJUMP, IMAXFL, &
 & INUMG, &
 & IOPROC, IREP,  &
 & IVALCO, &
 & J, JC, JDIM,  &
 & JV  ,JSTGLO,IBL,JL

INTEGER(KIND=JPIM) :: INGRIG,INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "dicomout.intfb.h"
#include "diwrgrid.intfb.h"
#include "especrt.intfb.h"
#include "inifaout.intfb.h"
#include "specrt.intfb.h"
#include "spnh_conv_nhvar.intfb.h"
#include "spnh_conv_prhs.intfb.h"
#include "suarpio.intfb.h"
#include "wrfu.intfb.h"
#include "wrgridua.intfb.h"
#include "wrspeca.intfb.h"
#include "wrxfu.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRMLPPA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.   PREPARATIONS.
!             -------------

INUMG=0
IMAXFL=NSURFL
IVALCO = NGPTOTG+2
JDIM=2+NGPTOTG

ALLOCATE (CLNOMA(IMAXFL))
ALLOCATE (ILNOMA(IMAXFL))
ALLOCATE (ILONGD(IMAXFL))
ALLOCATE (ZREALBUF(NGPTOT,IMAXFL))

IF(LOUTPUT) WRITE(NULOUT,*) 'WRMLPPA NSTEP=',NSTEP

CALL SUARPIO(2)

CALL INIFAOUT(CDCONF,CLINC,IUNTIN)

ALLOCATE (ZREALG(NGPTOTG,IMAXFL))
ALLOCATE (ZVALCO(IVALCO,IMAXFL))
ALLOCATE (ZZVALCO(IVALCO))

CLNOMA=""
ZREALG(:,:)=0.0_JPRB
ZREALBUF(:,:)=0.0_JPRB
ZVALCO=0.0_JPRB

!     ------------------------------------------------------------

!        1.2    COMPUTE T FROM RT AND Q
!               -----------------------

! * "Tv" is saved in ZSPT;
!   one then computes "T" from "Tv" and "q" and fills SPT with "T".
SELECT CASE (LSPRT)
CASE (.TRUE.)
  ALLOCATE(ZSPT(NFLSUR,NSPEC2))
  ZSPT(1:NFLSUR,1:NSPEC2)=SPT(1:NFLSUR,1:NSPEC2)
  SELECT CASE (LELAM)
  CASE (.TRUE.)
    CALL ESPECRT(1)
  CASE DEFAULT
    CALL SPECRT(1)
  END SELECT
END SELECT

!       1.3     COMPUTE PD,VD
!               -------------
IF (LNHDYN) THEN
  IF (LRWSDLR) ALLOCATE(ZSPSP(NSPEC2))
  ALLOCATE(ZSPSPD(NFLSUR,NSPEC2))
  ALLOCATE(ZSPSVD(NFLSUR,NSPEC2))
  ALLOCATE(ZSPNHX(NFLSUR,NSPEC2))
    
!copy to avoid overwriting
  IF (LRWSDLR) ZSPSP(:)=SPSP(:)
  ZSPSPD(:,:)=SPSPD(:,:)
  ZSPSVD(:,:)=SPSVD(:,:)
  ZSPNHX(:,:)=SPNHX(:,:)
    
!scaling
  CALL SPNH_CONV_NHVAR(.TRUE.)
  IF (LRWSDLR) CALL SPNH_CONV_PRHS(.TRUE.)
ENDIF

!     ------------------------------------------------------------
!*       2.     SEND SPECTRAL DATA ON MODEL LEVELS.
!               -----------------------------------

IF ((CDCONF == 'A').OR.(CDCONF == 'F').OR.(CDCONF == 'D').OR.&
   & (CDCONF == 'J').OR.(CDCONF == 'S')) THEN  
  CALL WRSPECA(IUNTIN)
ENDIF

! * One now restores "Tv" in SPT.
SELECT CASE (LSPRT)
CASE (.TRUE.)
  SPT(1:NFLSUR,1:NSPEC2)=ZSPT(1:NFLSUR,1:NSPEC2)
  DEALLOCATE(ZSPT)
END SELECT
!restore NH-model variables
IF (LNHDYN) THEN
  IF (LRWSDLR) SPSP(:)=ZSPSP(:)
  SPSPD(:,:)=ZSPSPD(:,:)
  SPSVD(:,:)=ZSPSVD(:,:)
  SPNHX(:,:)=ZSPNHX(:,:)

  IF (LRWSDLR) DEALLOCATE(ZSPSP) 
  DEALLOCATE(ZSPSPD)
  DEALLOCATE(ZSPSVD)
  DEALLOCATE(ZSPNHX)
ENDIF

!     ------------------------------------------------------------
!*       3.    SEND OUT GRID POINT FIELDS.
!              ---------------------------

IF( (LRSUP.AND.LMPHYS.AND.(CDCONF == 'A'.OR.&
   & CDCONF == 'F'.OR.CDCONF == 'P'.OR.CDCONF == 'X'.OR.CDCONF == 'S').OR.&
   & LOBSC1.OR.NCONF == 131).OR.&
   & (LELAM.AND.NCONF /= 601) ) THEN  

!*       3.1    COPY FROM NPROMA FORMAT TO FIELD FORMAT

  IREK=IMAXFL
  IJUMP=MIN(NGPTOT,NPROMA)
  IFIELDST=IMAXFL
  IDIMG=NGPTOTG

!*       3.2   SEND PROGNOSTIC FIELDS

  IFIELD=0


  DO JC=1,YSP_SBD%NLEVS
    DO JV=1,YSP_SBD%NUMFLDS
      IF(YSP_SB%YSB(JV)%CNAME(JC) /= '                ') THEN
        INUMG=INUMG+1
        CLTYP(INUMG)=YSP_SB%YSB(JV)%CNAME(JC)(1:4)
        CLPARA(INUMG)=YSP_SB%YSB(JV)%CNAME(JC)(5:)
        ILEV(INUMG)=0
        DO JSTGLO=1,NGPTOT,NPROMA
          IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
          IBL=(JSTGLO-1)/NPROMA+1
          DO JL=1,IEND
            ZREALBUF(JSTGLO+JL-1,INUMG) = SP_SB(JL,JC,JV,IBL)
          ENDDO
        ENDDO
!  Send grid point field
        IF (NAGDIO(1,INUMG) /= MYPROC) THEN
          IOPROC=NAGDIO(1,INUMG)
          CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
        ELSE
          DO J=1,NGPTOT
            ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
          ENDDO
        ENDIF
      ENDIF
    ENDDO
  ENDDO


  DO JV=1,YSP_SGD%NUMFLDS
    IF(YSP_SG%YSG(JV)%CNAME /= '                ') THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSP_SG%YSG(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSP_SG%YSG(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SP_SG(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSP_RRD%NUMFLDS
    IF(YSP_RR%YRR(JV)%CNAME /= '                ') THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSP_RR%YRR(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSP_RR%YRR(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SP_RR(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO


  DO JC=1,YSP_EPD%NLEVS
    DO JV=1,YSP_EPD%NUMFLDS
      IF(YSP_EP%YEP(JV)%CNAME(JC) /= '                ') THEN
        INUMG=INUMG+1
        CLTYP(INUMG)=YSP_EP%YEP(JV)%CNAME(JC)(1:4)
        CLPARA(INUMG)=YSP_EP%YEP(JV)%CNAME(JC)(5:)
        ILEV(INUMG)=0
        DO JSTGLO=1,NGPTOT,NPROMA
          IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
          IBL=(JSTGLO-1)/NPROMA+1
          DO JL=1,IEND
            ZREALBUF(JSTGLO+JL-1,INUMG) = SP_EP(JL,JC,JV,IBL)
          ENDDO
        ENDDO
!  Send grid point field
        IF (NAGDIO(1,INUMG) /= MYPROC) THEN
          IOPROC=NAGDIO(1,INUMG)
          CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
        ELSE
          DO J=1,NGPTOT
            ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
          ENDDO
        ENDIF
      ENDIF
    ENDDO
  ENDDO

  DO JV=1,YSP_X2D%NUMFLDS
    IF(YSP_X2%YX2(JV)%CNAME /= '                ') THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSP_X2%YX2(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSP_X2%YX2(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SP_X2(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  IF (LCANARI.AND.LSOLV) THEN
    IFISBA=5
  ELSE
    IFISBA=0
  ENDIF

  IFCAN=YSP_CID%NUMFLDS-IFISBA
  DO JV=1,IFCAN
    IF(YSP_CI%YCI(JV)%CNAME /= '                ') THEN
      IF(YSP_CI%YCI(JV)%CNAME /= 'SURFANALYSISMASK'.AND. &
       & YSP_CI%YCI(JV)%CNAME /= 'SURFSST.CLIM.   '     ) THEN  
        INUMG=INUMG+1
        CLTYP(INUMG)=YSP_CI%YCI(JV)%CNAME(1:4)
        CLPARA(INUMG)=YSP_CI%YCI(JV)%CNAME(5:)
        ILEV(INUMG)=0
        DO JSTGLO=1,NGPTOT,NPROMA
          IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
          IBL=(JSTGLO-1)/NPROMA+1
          DO JL=1,IEND
            ZREALBUF(JSTGLO+JL-1,INUMG) = SP_CI(JL,JV,IBL)
          ENDDO
        ENDDO
        IF (LRPLANE) THEN
          IF (YSP_CI%YCI(JV)%CNAME == 'CLSVENT.ZONAL   '.OR. &
           & YSP_CI%YCI(JV)%CNAME == 'CLSVENT.MERIDIEN'    ) &
           & ZREALBUF(:,INUMG)=ZREALBUF(:,INUMG)*GM(:)  
        ENDIF
!  Send grid point field
        IF (NAGDIO(1,INUMG) /= MYPROC) THEN
          IOPROC=NAGDIO(1,INUMG)
          CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
        ELSE
          DO J=1,NGPTOT
            ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
          ENDDO
        ENDIF
      ENDIF
    ENDIF
  ENDDO

!*       3.3   SEND CLIMATOLOGICAL FIELDS

  DO JV=1,YSD_VFD%NUMFLDS
    IF(YSD_VF%YVF(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_VF%YVF(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_VF%YVF(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_VF(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_VPD%NUMFLDS
    IF(YSD_VP%YVP(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_VP%YVP(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_VP%YVP(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_VP(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_VVD%NUMFLDS
    IF(YSD_VV%YVV(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_VV%YVV(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_VV%YVV(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_VV(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_VAD%NUMFLDS
    IF(YSD_VA%YVA(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_VA%YVA(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_VA%YVA(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_VA(JL,JV,IBL)
        ENDDO
      ENDDO
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_VCD%NUMFLDS
    IF(YSD_VC%YVC(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_VC%YVC(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_VC%YVC(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_VC(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_VDD%NUMFLDS
    IF(YSD_VD%YVD(JV)%CNAME /= '                ') THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_VD%YVD(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_VD%YVD(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_VD(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_WSD%NUMFLDS
    IF(YSD_WS%YWS(JV)%CNAME /= '                ') THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_WS%YWS(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_WS%YWS(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_WS(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  DO JV=1,YSD_WWD%NUMFLDS
    IF(YSD_WW%YWW(JV)%CNAME /= '                ') THEN
      INUMG=INUMG+1
      CLTYP(INUMG)=YSD_WW%YWW(JV)%CNAME(1:4)
      CLPARA(INUMG)=YSD_WW%YWW(JV)%CNAME(5:)
      ILEV(INUMG)=0
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          ZREALBUF(JSTGLO+JL-1,INUMG) = SD_WW(JL,JV,IBL)
        ENDDO
      ENDDO
!  Send grid point field
      IF (NAGDIO(1,INUMG) /= MYPROC) THEN
        IOPROC=NAGDIO(1,INUMG)
        CALL DIWRGRID(IOPROC,INUMG,0,NGPTOT,ZREALBUF(1,INUMG))
      ELSE
        DO J=1,NGPTOT
          ZREALG(J,INUMG)=ZREALBUF(J,INUMG)
        ENDDO
      ENDIF
    ENDIF
  ENDDO



ELSE

  WRITE(UNIT=NULOUT,FMT='(A)')'! SURFACE FIELDS NOT WRITTEN'

ENDIF   ! IF(LRSUP.AND.LMPHYS ...
NBGOUT=INUMG   ! used in diwrgrid and wrdisfa

DO JFLD=1,NBGOUT
  IF (NAGDIO(1,JFLD) == MYPROC) THEN
!  Recv grid point field and write it
    IOPROC=NAGDIO(1,JFLD)
    CALL DIWRGRID(IOPROC,JFLD,1,IDIMG,ZREALG(1,JFLD))
    IF (TRIM(CLTYP(JFLD))//CLPARA(JFLD) == 'SURFZ0.FOIS.G' .OR. &
       & TRIM(CLTYP(JFLD))//CLPARA(JFLD) == 'SURFZ0REL.FOIS.G' .OR. &
       & TRIM(CLTYP(JFLD))//CLPARA(JFLD) == 'SURFGZ0.THERM') THEN  
!     Do not pack roughness lengths
      CALL FAVEUR(IREP,IUNTIN,INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
      INGRIG=0
      CALL FAGOTE(IREP,IUNTIN,INGRIG,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
    ENDIF
    CALL FACOND (IREP,IUNTIN,CLTYP(JFLD),1,CLPARA(JFLD),&
     & ZREALG(1,JFLD),.FALSE.,CLNOMA(JFLD),ILNOMA(JFLD),&
     & ZVALCO(1,JFLD),ILONGD(JFLD))  
    IF (TRIM(CLTYP(JFLD))//CLPARA(JFLD) == 'SURFZ0.FOIS.G' .OR. &
       & TRIM(CLTYP(JFLD))//CLPARA(JFLD) == 'SURFZ0REL.FOIS.G' .OR. &
       & TRIM(CLTYP(JFLD))//CLPARA(JFLD) == 'SURFGZ0.THERM') THEN  
!     Reset packing after the treatment of roughness lengths
      CALL FAGOTE(IREP,IUNTIN,INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
    ENDIF
    IF (MYPROC /= 1)&
     & CALL DICOMOUT(CLNOMA(JFLD),ILNOMA(JFLD),ZVALCO(1,JFLD),&
     & ILONGD(JFLD),JDIM,JFLD+3000,NBGOUT)  
    IF(LOUTPUT)&
     & WRITE(NULOUT,'(2X,A,A,I3.3,'' WRITTEN TO ARPEGE FILE '')')&
     & CLTYP(JFLD),CLPARA(JFLD),ILEV(JFLD)  
  ENDIF
ENDDO

IF (MYPROC == 1) THEN
  DO IFL=1,NBGOUT
    IF (NAGDIO(1,IFL) /= 1)&
     & CALL DICOMOUT(CLNOMA(IFL),ILNOMA(IFL),ZVALCO(1,IFL),&
     & ILONGD(IFL),JDIM,IFL+3000,NBGOUT)  
    CLLNOM=CLNOMA(IFL)
    ZZVALCO=0.0_JPRB
 
    ! ZVALCO may carry non representable reals wich causes
    ! problems for some compilers, therefore make a clean
    ! memcopy
    CALL ECMWF_TRANSFER(ZZVALCO(1:ILONGD(IFL))    ,JPRB*ILONGD(IFL),   &
                       & ZVALCO(1:ILONGD(IFL),IFL),JPRB*ILONGD(IFL))

    CALL FAISAN (IREP,IUNTIN,CLLNOM(1:ILNOMA(IFL)),ZZVALCO,ILONGD(IFL))
  ENDDO
ENDIF

DEALLOCATE (ZVALCO)
DEALLOCATE (ZZVALCO)
DEALLOCATE (ILNOMA)
DEALLOCATE (ILONGD)
DEALLOCATE (CLNOMA)

IF (ALLOCATED(ZREALBUF)) DEALLOCATE(ZREALBUF)
IF (ALLOCATED(ZREALG)) DEALLOCATE(ZREALG)

!      -----------------------------------------------------------
!*       4.    WRITE OUT UPPER AIR GRID POINT FIELDS.
!              --------------------------------------

IF (YGFL%NUMGPFLDS>0) THEN
  CALL WRGRIDUA(IUNTIN)
ENDIF

!     ------------------------------------------------------------------
!*       5.    WRITE INSTANTANEOUS DIAGNOSTICS.
!              --------------------------------

IF (LXFU.AND.LFBDAP.AND.&
   & (CDCONF == 'A'.OR.CDCONF == 'F'.OR.CDCONF == 'X'.OR.CDCONF == 'S')) THEN  
  CALL WRXFU(IUNTIN)
ENDIF

!     ------------------------------------------------------------------
!*       6.    WRITE CUMULATED DIAGNOSTICS.
!              ----------------------------

IF (LCUMFU.AND.LFBDAP.AND.&
   & (((CDCONF == 'A'.OR.CDCONF == 'F'.OR.CDCONF == 'S').AND.(NSTEP > 0)).OR.&
   & (CDCONF == 'G'))) THEN  
  CALL WRFU(IUNTIN)
ENDIF

!     ------------------------------------------------------------------
!*       7.    CLOSE FILES.
!              ------------

IF (MYPROC <= NSTROUT) THEN
  IF (MYPROC == 1) THEN
    IF (LRFILAF) THEN
      CALL LFILAF(IREP,IUNTIN,.FALSE.)
    ENDIF
    CALL FAIRME(IREP,IUNTIN,'KEEP')
    IF ((NSTEP >= 0).AND.(CDCONF == 'A'.OR.CDCONF == 'F'.OR.&
       & CDCONF == 'P'.OR.CDCONF == 'X'.OR.CDCONF == 'S')) THEN  
      OPEN (UNIT=NULHWF,FILE=CFNHWF,FORM='FORMATTED')
      REWIND(NULHWF)
      WRITE(NULHWF,'(A4)') CLINC
      CLOSE(UNIT=NULHWF,STATUS='KEEP')
    ENDIF
  ELSE
    CALL FAIRNO(IREP,IUNTIN,'DELETE')
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRMLPPA',1,ZHOOK_HANDLE)
END SUBROUTINE WRMLPPA
