SUBROUTINE WRMLPPG(CDCONF)

!**** *WRMLPPG*  - writes out the model level fields in GRIB

!     Purpose.
!     --------
!     Write out the model level fields in GRIB

!**   Interface.
!     ----------
!        *CALL* *WRMLPPG(CDCONF)

!        Explicit arguments :     CDCONF - configuration of call
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.   MODNSEC - modify GRIB headers
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!       Original : 93-10-20 (From WRMLPP, exclude ARPEGE files)
!       Modified : 94-04-26 M.Hamrud (introduction of MODNSEC)
!       Modified : 94-06-13 C.Temperton (spectral RT option)
!       Modified : 94-06-20 M.Hamrud ( upper air grid-point file)
!       Modified : 95-02-08 M.Fisher (write random background vectors
!                                     and eigenvectors of the Hessian)
!       Modified : 95-06-06 L.Isaksen (Reordering of spectral arrays)
!       Modified : 95-10-20 M.Hamrud (Interface block for EXTGPF)
!       Modified : 95-10-20 M.Hamrud (Merge with DM version)
!       Modified : 96-03    A. Untch (grid-point prognostic ozone (EC))
!       Modified : 96-10-09 F. Rabier (add timestep in PP of increments)
!       Modified : 97-01-31 M. Fisher write vectors for the SKF
!       Modified : 97-10-14 G. Mozdzynski support mpi-io
!       Modified : 98-06-03 E. Holm (spectral prognostic ozone (EC))
!       Modified : 99-05-06 D.Dent  (constrain cloud field range)
!    R. El Khatib : 99-12-28 remove poles rows
!       Modified : 00-02-16 T. Stockdale (seasonal stream)
!       Modified : 99-03-26 P. Viterbo (surface tiling)
!       Modified : 00-09-01 D.Dent: convert from MPE to MPL
!       Modified : 00-12-21 G.Mozdzynski: perform grib encoding in
!                                         parallel for NOUTTYPE=1
!    R. El Khatib : 00-11-03 Optional communications in extgpf
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!       P. Viterbo  : 24-05-2004   Change surface units
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMP    , ONLY :         MYSETV   ,NOUTTYPE ,&
 & NPTRLL   ,NBSETLEV ,NBSETSP    
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMCT0   , ONLY : LSPRT    ,LOBSC1
USE YOMDIM   , ONLY : NFLEVG   ,NFLEVL  ,&
 & NPROMA   ,NSPEC2   ,&
 & NS3D     ,NS2D  
USE YOMGC    , ONLY : OROG
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPT      ,SPQ      ,&
 & SPO3     ,SPSP     ,SPOR  
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : LEPHYS
USE YOMGEM   , ONLY :    NGPTOT   ,NGPTOTG  ,NGPTOTMX
USE YOMVAR   , ONLY :         &
 & LTWBGV   ,LTWCGL, LTWINC 
USE SURFACE_FIELDS_MIX , ONLY : GPOPER, TYPE_SFL_COMM
USE YOMGRB   , ONLY :             &
 & NGRBZ    ,NGRBT    ,NGRBQ    ,NGRBVO   ,&
 & NGRBSD   ,NGRBLSP  ,NGRBCP   ,NGRBSF   ,NGRBLNSP ,&
 & NGRBD    ,NGRBSDOR ,NGRBSR   ,NGRBE    ,&
 & NGRBSRC  ,NGRBO3   ,NGRBRO   ,NGRBSSRO   ,NGRBSRO   ,&
 & NGRBCSF  ,NGRBLSF  ,NGRBFSR  ,NGRBCLWC ,NGRBCIWC ,&
 & NGRBCC     
USE YOMCST   , ONLY : RG
USE YOMPPC   , ONLY : M2DGGP   ,M3DSPM   ,M3DGGM   ,NO2DGG   ,&
 & NO3DSPM  ,NO3DGGM  ,LRSUP  
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL     ,YGFLC    
USE YOMSKF   , ONLY : LTWSKFY  ,LTWSKFZ
USE YOECLDP  , ONLY : RLMIN    ,RAMIN
USE YOETHF   , ONLY : RHOH2O

IMPLICIT NONE

#include "abor1.intfb.h"
#include "prepfdb.intfb.h"
#include "specrt.intfb.h"
#include "wroutgpgb.intfb.h"
#include "wroutspgb.intfb.h"

!     EXTERNAL INTEGER FUNCTIONS
CHARACTER(LEN=1)                 :: CDCONF ! Argument NOT used
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

CHARACTER (LEN = 1) ::  CLMODE

REAL(KIND=JPRB),ALLOCATABLE :: ZFPDAT(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZREAL(:,:)
REAL(KIND=JPRB) :: ZFIELD(NPROMA,1)
REAL(KIND=JPRB) :: ZSPT(NFLEVL,NSPEC2)
INTEGER(KIND=JPIM) :: IGRIBIOGG(2,MAX(NO2DGG+1,NO3DGGM*NFLEVG))
INTEGER(KIND=JPIM) :: IGRIBIOSH(4,NS3D*NFLEVG+NS2D)

INTEGER(KIND=JPIM) :: IBSET, IEND, IF, IFIELDST, IFOUND, IGGMAX, &
 & IGRIBCD, IJUMP, ILEVEL, ILOCLV, IPT, ISHMAX, &
 & IST, IZZ, J, JCODE, JF, JGFL, JLEV, JROF, JSTGLO, IBL, IFLD, IFF, IBLK

LOGICAL :: LLWRSURF

TYPE(TYPE_SFL_COMM) :: YLCOM
REAL(KIND=JPRB) :: ZSCALE, ZZSCALE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

!     ------------------------------------------------------------------

!*       1.    PREPARATIONS.
!              --------------

IF (LHOOK) CALL DR_HOOK('WRMLPPG',0,ZHOOK_HANDLE)

!*       1.1   PREPARE FDB

WRITE(NULOUT,*) 'WRITING HISTORY FILE - WRMLPPG'
IF(NOUTTYPE == 2) THEN
  CALL PREPFDB
ENDIF
CLMODE='w'

!      -----------------------------------------------------------

!*       2.    WRITE SPECTRAL DATA ON MODEL LEVELS.
!              ------------------------------------

!*       2.1 Build list of spectral fields to be output

! Initialise igribiosh data structure
!   1 - grib code
!   2 - level (used in file name)
!   3 - bset to send this field
!   4 - local level for sending bset

ISHMAX = 0
IF(NO3DSPM > 0) THEN

!     VORTICITY.

  IGRIBCD = NGRBVO
  IF(ISRCHEQ(NO3DSPM,M3DSPM,1,IGRIBCD) <= NO3DSPM) THEN
    DO JLEV=1,NFLEVG
      ISHMAX = ISHMAX+1
      IGRIBIOSH(1,ISHMAX) = IGRIBCD
      IGRIBIOSH(2,ISHMAX) = JLEV
      IGRIBIOSH(3,ISHMAX) = NBSETLEV(JLEV)
      IGRIBIOSH(4,ISHMAX) = JLEV-NPTRLL(NBSETLEV(JLEV))+1
    ENDDO
  ENDIF

!     DIVERGENCE.

  IGRIBCD = NGRBD
  IF(ISRCHEQ(NO3DSPM,M3DSPM,1,IGRIBCD) <= NO3DSPM) THEN
    DO JLEV=1,NFLEVG
      ISHMAX = ISHMAX+1
      IGRIBIOSH(1,ISHMAX) = IGRIBCD
      IGRIBIOSH(2,ISHMAX) = JLEV
      IGRIBIOSH(3,ISHMAX) = NBSETLEV(JLEV)
      IGRIBIOSH(4,ISHMAX) = JLEV-NPTRLL(NBSETLEV(JLEV))+1
    ENDDO
  ENDIF

!     THERMODYNAMIC VARIABLES.

  IF (LSPRT) THEN
    ZSPT(:,:) = SPT(:,:)
    CALL SPECRT(1,LTWINC)
  ENDIF
  IGRIBCD = NGRBT
  IF(ISRCHEQ(NO3DSPM,M3DSPM,1,IGRIBCD) <= NO3DSPM) THEN
    DO JLEV=1,NFLEVG
      ISHMAX = ISHMAX+1
      IGRIBIOSH(1,ISHMAX) = IGRIBCD
      IGRIBIOSH(2,ISHMAX) = JLEV
      IGRIBIOSH(3,ISHMAX) = NBSETLEV(JLEV)
      IGRIBIOSH(4,ISHMAX) = JLEV-NPTRLL(NBSETLEV(JLEV))+1
    ENDDO
  ENDIF

!     GFL

  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LSP) THEN
      IGRIBCD=YGFLC(JGFL)%IGRBCODE
      IF(ISRCHEQ(NO3DSPM,M3DSPM,1,IGRIBCD) <= NO3DSPM) THEN
        DO JLEV=1,NFLEVG
          ISHMAX = ISHMAX+1
          IGRIBIOSH(1,ISHMAX) = IGRIBCD
          IGRIBIOSH(2,ISHMAX) = JLEV
          IGRIBIOSH(3,ISHMAX) = NBSETLEV(JLEV)
          IGRIBIOSH(4,ISHMAX) = JLEV-NPTRLL(NBSETLEV(JLEV))+1
        ENDDO
      ENDIF
    ENDIF
  ENDDO

!     SURFACE PRESSURE.

  IGRIBCD = NGRBLNSP
  IF(ISRCHEQ(NO3DSPM,M3DSPM,1,IGRIBCD) <= NO3DSPM) THEN
    ISHMAX = ISHMAX+1
    IGRIBIOSH(1,ISHMAX) = IGRIBCD
    IGRIBIOSH(2,ISHMAX) = 1
    IGRIBIOSH(3,ISHMAX) = NBSETSP
    IGRIBIOSH(4,ISHMAX) = 1
  ENDIF

!     SPECTRAL OROGRAPHY.

  IGRIBCD = NGRBZ
  IF(ISRCHEQ(NO3DSPM,M3DSPM,1,IGRIBCD) <= NO3DSPM) THEN
    ISHMAX = ISHMAX+1
    IGRIBIOSH(1,ISHMAX) = IGRIBCD
    IGRIBIOSH(2,ISHMAX) = 1
    IGRIBIOSH(3,ISHMAX) = NBSETSP
    IGRIBIOSH(4,ISHMAX) = 1
  ENDIF

  IFF=0
  DO JF=1,ISHMAX
    IBSET  = IGRIBIOSH(3,JF)
    IF( IBSET == MYSETV )THEN
      IFF=IFF+1
    ENDIF
  ENDDO

  ALLOCATE(ZFPDAT(NSPEC2,IFF))

  IFF=0
  DO JF=1,ISHMAX
    IGRIBCD = IGRIBIOSH(1,JF)
    ILEVEL = IGRIBIOSH(2,JF)
    IBSET  = IGRIBIOSH(3,JF)
    ILOCLV = IGRIBIOSH(4,JF)
    IF( IBSET == MYSETV )THEN
      IFF=IFF+1
      IF    ( IGRIBCD == NGRBZ )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPOR(J)
        ENDDO
      ELSEIF( IGRIBCD == NGRBT )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPT(ILOCLV,J)
        ENDDO
      ELSEIF( IGRIBCD == NGRBQ )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPQ(ILOCLV,J)
        ENDDO
      ELSEIF( IGRIBCD == NGRBO3 )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPO3(ILOCLV,J)
        ENDDO
      ELSEIF( IGRIBCD == NGRBVO )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPVOR(ILOCLV,J)
        ENDDO
      ELSEIF( IGRIBCD == NGRBLNSP )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPSP(J)
        ENDDO
      ELSEIF( IGRIBCD == NGRBD )THEN
        DO J=1,NSPEC2
          ZFPDAT(J,IFF) = SPDIV(ILOCLV,J)
        ENDDO
      ELSE
        WRITE(NULERR,'(A,I6)')' WRMLPPG: SH INVALID IGRIBCD ',IGRIBCD
        CALL ABOR1(' WRMLPPG: SH INVALID IGRIBCD ')
      ENDIF
    ENDIF
  ENDDO
  CALL WROUTSPGB(ZFPDAT,ISHMAX,IGRIBIOSH,'m')
  DEALLOCATE(ZFPDAT)
  IF (LSPRT) THEN
    SPT(:,:) = ZSPT(:,:)
  ENDIF
ENDIF

!      -----------------------------------------------------------

LLWRSURF = .NOT.(LTWBGV.OR.LTWCGL.OR.LTWSKFY.OR.LTWSKFZ) &
         & .AND.(LRSUP.AND.(LMPHYS.OR.LEPHYS).OR.LOBSC1)

!*       4.0   WRITE OUT SURFACE GRID POINT FIELDS
!              -----------------------------------

IF ( LLWRSURF ) THEN

!*       4.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT


  ALLOCATE(ZREAL(NGPTOT,NO2DGG))
  IGGMAX=0
  DO JCODE=1,NO2DGG
    IGRIBCD = M2DGGP(JCODE)
    YLCOM%IGRBCODE = IGRIBCD
    CALL GPOPER('GETGRIBPOS',KBL=1,YDCOM=YLCOM)
    IF(YLCOM%L_OK) THEN
      IGGMAX = IGGMAX+1
      IGRIBIOGG(1,IGGMAX) = IGRIBCD
      DO JSTGLO=1,NGPTOT,NPROMA
        IST=1
        IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBLK=(JSTGLO-1)/NPROMA+1
        CALL GPOPER('GETFIELD',KBL=IBLK,YDCOM=YLCOM,PFIELD=ZFIELD)
        DO J=JSTGLO,JSTGLO+IEND-IST
          ZREAL(J,IGGMAX)=ZFIELD(J-JSTGLO+1,1)
        ENDDO
      ENDDO
    ELSE  
      IF(IGRIBCD /= NGRBZ) THEN
        WRITE(NULOUT,'(A,I4,A)') ' WRMLPP - GRIB CODE ',IGRIBCD,' NOT FOUND '
      ENDIF
    ENDIF
  ENDDO

  IGRIBCD = NGRBZ
  IFOUND  = ISRCHEQ(NO2DGG,M2DGGP,1,IGRIBCD)
  IF(IFOUND <= NO2DGG) THEN
    IGGMAX = IGGMAX+1
    IGRIBIOGG(1,IGGMAX) = IGRIBCD
    DO JROF=1,NGPTOT
      ZREAL(JROF,IGGMAX) = OROG(JROF)
    ENDDO
  ENDIF
  IGRIBIOGG(2,:) = 0

  DO JF=1,IGGMAX
    IGRIBCD = IGRIBIOGG(1,JF)

!*       4.2   SCALINGS FOR ECMWF MARS ARCHIVING

    IF = JF
    IF    (IGRIBCD == NGRBSD  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBLSP ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBCP  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSF  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSR  ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IGRIBCD == NGRBE   ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSRC ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSSRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBCSF ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBLSF ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBFSR ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IGRIBCD == NGRBSDOR) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSE
      ZSCALE = 1.0_JPRB
    ENDIF

    IF (ZSCALE /= 1.0_JPRB) THEN
      DO J=1,NGPTOT
        ZREAL(J,JF) = ZREAL(J,JF)*ZSCALE
      ENDDO
    ENDIF
  ENDDO

  CALL WROUTGPGB(ZREAL,NGPTOT,NGPTOTG,NGPTOTMX,IGGMAX,IGRIBIOGG,'s',CLMODE)
  DEALLOCATE(ZREAL)

ENDIF

!      -----------------------------------------------------------

!*       5.0   WRITE OUT UPPER AIR GRID POINT FIELDS
!              --------------------------------------

!*       5.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT

IGGMAX = 0
DO JCODE=1,NO3DGGM
  IGRIBCD = M3DGGM(JCODE)
  IPT=0
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LGP .AND. YGFLC(JGFL)%IGRBCODE ==IGRIBCD) THEN
      IPT=JGFL
    ENDIF
  ENDDO
  IF(IPT == 0) THEN
    WRITE(NULERR,'('' WRMLPPG - GRIB CODE UNKNOWN '')')
    WRITE(NULERR,'('' GRIB CODE = '',I3.3)') IGRIBCD
    CALL ABOR1('WRMLPPG: ABOR1 CALLED')
  ENDIF
  DO JLEV=1,NFLEVG
    IGGMAX = IGGMAX+1
    IPT    = IPT+1
    IGRIBIOGG(1,IGGMAX) = IGRIBCD
    IGRIBIOGG(2,IGGMAX) = JLEV
  ENDDO
ENDDO

IFIELDST = IGGMAX
IJUMP    = MIN(NGPTOT,NPROMA)
ALLOCATE(ZREAL(NGPTOT,IGGMAX))
IFLD=0
DO JF=1,IFIELDST/NFLEVG
  DO JLEV=1,NFLEVG
    IFLD=IFLD+1
    IGRIBCD = IGRIBIOGG(1,IFLD)
    ILEVEL  = IGRIBIOGG(2,IFLD)
    DO JGFL=1,YGFL%NUMFLDS
      IF(YGFLC(JGFL)%LGP .AND. YGFLC(JGFL)%IGRBCODE == IGRIBCD) THEN
        IPT=JGFL
      ENDIF
    ENDDO
    DO JSTGLO=1,NGPTOT,NPROMA
      IST=1
      IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
      IBL=(JSTGLO-1)/NPROMA+1
      DO J=IST,IEND
        ZREAL(JSTGLO-1+J,IFLD)=GFL(J,JLEV,IPT,IBL)
      ENDDO
    ENDDO
    
    IF(IGRIBCD == NGRBCC.OR.&
       & IGRIBCD == NGRBCLWC.OR.&
       & IGRIBCD == NGRBCIWC) THEN  
      IF(IGRIBCD == NGRBCLWC) THEN
        ZSCALE=RLMIN
      ELSEIF(IGRIBCD == NGRBCIWC) THEN
        ZSCALE=RLMIN
      ELSEIF(IGRIBCD == NGRBCC  ) THEN
        ZSCALE=RAMIN
      ENDIF
      !                 prevent small values falling outside physical range
      ZZSCALE=1.0_JPRB/ZSCALE
      DO J=1,NGPTOT
        IZZ=ZREAL(J,IFLD)*ZZSCALE
        ZREAL(J,IFLD)=IZZ*ZSCALE
      ENDDO
    ENDIF
  ENDDO
ENDDO

!*       5.3   WRITE FIELDS

IF(IGGMAX > 0) THEN
  CALL WROUTGPGB(ZREAL,NGPTOT,NGPTOTG,NGPTOTMX,IGGMAX,IGRIBIOGG,'m',CLMODE)
ENDIF
DEALLOCATE(ZREAL)

!     ---------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRMLPPG',1,ZHOOK_HANDLE)
END SUBROUTINE WRMLPPG
