SUBROUTINE WRMLPPLG(CDCONF)

!**** *WRMLPPLG*  - writes out some model level fields in GRIB

!     Purpose.
!     --------
!     Write out model level fields in lagged mode, after going
!     into grid-point space and calling the physics.

!**   Interface.
!     ----------
!        *CALL* *WRMLPPLG(CDCONF)

!        Explicit arguments :     CDCONF - configuration of call
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  
!     ----------

!         GRIBEX   - GRIB coding
!         WRBUDG   - write out "budget record"
!         MODNSEC  - modify GRIB headers

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        MARS users guide

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!       Original : 93-10-25 (From WRMLPPL)
!       Modified : 94-04-26 M.Hamrud (introduction of MODNSEC)
!       Modified : 94-06-13 C.Temperton (spectral RT option)
!       Modified : 95-06-06 L. Isaksen - Reordering of spectral arrays
!       Modified : 95-10-20 M.Hamrud (Interface block for EXTGPF)
!       Modified : 95-10-20 M.Hamrud (Merge with DM version)
!       Modified : 96-10-09 F. Rabier (add timestep in PP of increments)
!       Modified : 97-10-14 G. Mozdzynski (support mpi-io)
!    R. El Khatib : 99-12-28 remove poles rows
!       Modified : 00-09-01 D.Dent: convert from MPE to MPL
!       Modified : 00-12-21 G.Mozdzynski: perform grib encoding in
!                                         parallel for NOUTTYPE=1
!    R. El Khatib : 00-11-03 Optional communications in extgpf
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMP    , ONLY : NOUTTYPE 
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NCONF    ,LOBSC1   
USE YOMDIM   , ONLY : NPROMA   
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : LEPHYS
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG  ,NGPTOTMX
USE YOMVAR   , ONLY : LTWBGV   ,LTWCGL  
USE SURFACE_FIELDS_MIX , ONLY : GPOPER, TYPE_SFL_COMM
USE YOMGRB   , ONLY :             &
 & NGRBBV   ,NGRBSWVL1 ,NGRBSD   ,NGRBLSP  ,&
 & NGRBCP   ,NGRBSF   ,NGRBSDOR ,NGRBSWVL2 ,NGRBSR   ,&
 & NGRBE    ,NGRBSWVL3 ,NGRBSRC  ,NGRBRO   ,NGRBSWVL4 ,&
 & NGRBCSF  ,NGRBLSF  ,NGRBFSR  
USE YOMCST   , ONLY : RDAY     ,RG
USE YOMPPC   , ONLY :    M2DGGPL  ,NO2DGG   ,NO2DGGL  , LRSUP  

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
CHARACTER(LEN=1)                 :: CDCONF ! Argument NOT used
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

CHARACTER (LEN = 1) ::  CLMODE
REAL(KIND=JPRB), ALLOCATABLE :: ZREAL(:,:)
REAL(KIND=JPRB) :: ZFIELD(NPROMA,1)
INTEGER(KIND=JPIM) :: IGRIBIOGG(2,NO2DGG+1)

INTEGER(KIND=JPIM) ::  IEND,  IGGMAX, IGRIBCD, &
 & IST, J, JCODE, JF, JSTGLO ,IBLK

TYPE(TYPE_SFL_COMM) :: YLCOM
REAL(KIND=JPRB) :: ZD1, ZD2, ZD3, ZD4, ZSCALE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

#include "prepfdb.intfb.h"
#include "wrbudg.intfb.h"
#include "wroutgpgb.intfb.h"

!     ------------------------------------------------------------------

!*       1.    PREPARATIONS.
!              --------------

!*       1.1   PREPARE FDB

IF (LHOOK) CALL DR_HOOK('WRMLPPLG',0,ZHOOK_HANDLE)
IF(NOUTTYPE == 2) THEN
  CALL PREPFDB
ENDIF
CLMODE='a'

!      -----------------------------------------------------------

!*       4.0   WRITE OUT SURFACE GRID POINT FIELDS
!              -----------------------------------

IF ( .NOT.(LTWBGV.OR.LTWCGL) .AND.&
   & (LRSUP.AND.(LMPHYS.OR.LEPHYS).OR.LOBSC1.OR.&
   & NCONF == 131) ) THEN  

  ALLOCATE(ZREAL(NGPTOT,NO2DGGL))

!*       4.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT


  IGGMAX=0
  DO JCODE=1,NO2DGGL
    IGRIBCD = M2DGGPL(JCODE)
    YLCOM%IGRBCODE = IGRIBCD
    CALL GPOPER('GETGRIBPOS',KBL=1,YDCOM=YLCOM)
    IF(YLCOM%L_OK) THEN
      IGGMAX = IGGMAX+1
      IGRIBIOGG(1,IGGMAX) = IGRIBCD
      DO JSTGLO=1,NGPTOT,NPROMA
        IST=1
        IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBLK=(JSTGLO-1)/NPROMA+1
        CALL GPOPER('GETFIELD',KBL=IBLK,YDCOM=YLCOM,PFIELD=ZFIELD)
        DO J=JSTGLO,JSTGLO+IEND-IST
          ZREAL(J,IGGMAX)=ZFIELD(J-JSTGLO+1,1)
        ENDDO
      ENDDO
    ELSE
      IF(IGRIBCD /= NGRBBV) THEN
        WRITE(NULOUT,'(A,I4,A)') ' WRMLPPLG - GRIB CODE ',IGRIBCD,' NOT FOUND '
      ENDIF
    ENDIF
  ENDDO

  IGRIBIOGG(2,:) = 0


  DO JF=1,IGGMAX
    IGRIBCD = IGRIBIOGG(1,JF)
!*       4.2   SCALINGS FOR ECMWF MARS ARCHIVING

    ZD1 = 0.07_JPRB
    ZD2 = 0.21_JPRB
    ZD3 = 0.72_JPRB
    ZD4 = 1.89_JPRB
    IF    (IGRIBCD == NGRBSWVL1 ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBSD  ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBLSP ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBCP  ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBSF  ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBSWVL2) THEN
      ZSCALE = 1.0E-3_JPRB*ZD1/ZD2
    ELSEIF(IGRIBCD == NGRBSR  ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IGRIBCD == NGRBE   ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBSWVL3) THEN
      ZSCALE = 1.0E-3_JPRB*ZD1/ZD3
    ELSEIF(IGRIBCD == NGRBSRC ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBRO  ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBSWVL4) THEN
      ZSCALE = 1.0E-3_JPRB*ZD1/ZD4
    ELSEIF(IGRIBCD == NGRBCSF ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBLSF ) THEN
      ZSCALE = 1.0E-3_JPRB
    ELSEIF(IGRIBCD == NGRBFSR ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IGRIBCD == NGRBSDOR) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSE
      ZSCALE = 1.0_JPRB
    ENDIF
    IF(ZSCALE /= 1.0_JPRB) THEN
      DO J=1,NGPTOT
        ZREAL(J,JF) = ZREAL(J,JF)*ZSCALE
      ENDDO
    ENDIF
    
  ENDDO

  CALL WROUTGPGB(ZREAL,NGPTOT,NGPTOTG,NGPTOTMX,IGGMAX,IGRIBIOGG,'s',CLMODE)
  DEALLOCATE(ZREAL)
!*       4.4   WRITE OUT BUDGET RECORD
  IF(ISRCHEQ(NO2DGGL,M2DGGPL,1,128) <= NO2DGGL) THEN
    CALL WRBUDG
  ENDIF

ENDIF

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRMLPPLG',1,ZHOOK_HANDLE)
END SUBROUTINE WRMLPPLG
