SUBROUTINE WRMODERR(CDFILE,YDSPERR,YDGPERR,kcomp)

!   Purpose.
!   --------
!     Save model error in GRIB format.

!   Author.
!   -------
!     Y. Tremolet

!   Modifications.
!   --------------
!     Original   03-01-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        Y.Tremolet    18-Mar-2004 Re-write
!        Y.Tremolet    15-Jul-2004 Add prints for diagnostics
!        Y.Tremolet    22-Aug-2005 One component at a time
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPRB, JPIM
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCT0  , ONLY: CTYPE
USE YOMLUN  , ONLY: NULOUT
USE YOMDYN  , ONLY: TSTEP
USE YOMMODEL_ERROR, ONLY: NCOMP_MODERR, LSCALERR, TSTEP_ERR, &
                     & SPCTLMODERR, SPGPMODERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

CHARACTER(LEN=*), INTENT(IN) :: CDFILE
TYPE(spectral_field) , INTENT(IN) :: YDSPERR
TYPE(gridpoint_field), INTENT(IN) :: YDGPERR
INTEGER(KIND=JPIM), INTENT(IN), OPTIONAL :: kcomp

INTEGER(KIND=JPIM) :: icomp
REAL(KIND=JPRB) :: zh
REAL(KIND=JPRB) :: ZHOOK_HANDLE
CHARACTER(LEN=2) :: CLTYPE

#include "write_spec_grib.intfb.h"
#include "grid2spec.intfb.h"
#include "spec_concat.intfb.h"
#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRMODERR',0,ZHOOK_HANDLE)

IF (NCOMP_MODERR>1) THEN
  IF (PRESENT(kcomp)) THEN
    icomp=kcomp
  ELSE
    CALL ABOR1('WRMODERR: kcomp required when NCOMP_MODERR>1')
  ENDIF
ELSE
  icomp=1
  IF (PRESENT(kcomp)) icomp=kcomp
ENDIF
IF (icomp>NCOMP_MODERR) CALL ABOR1('WRMODERR: icomp>NCOMP_MODERR')

! If forcing is time-step dependant, it needs to be normalised.
IF (LSCALERR) THEN
  IF (TSTEP_ERR==0.0_JPRB) THEN
    zh=3600.0_JPRB/TSTEP
  ELSE
    zh=3600.0_JPRB/TSTEP_ERR
  ENDIF
ENDIF

CALL GRID2SPEC(YDGPERR,SPGPMODERR)
CALL SPEC_CONCAT(YDSPERR,SPGPMODERR,SPCTLMODERR)

write(nulout,*)'WRMODERR: Writing model error to file ',CDFILE
CALL SPECTRAL_NORM_LEVS(SPCTLMODERR,'WRMODERR')
IF (LSCALERR) CALL SELF_MUL(SPCTLMODERR,zh)

CLTYPE=CTYPE
CTYPE='me'
CALL WRITE_SPEC_GRIB(CDFILE,SPCTLMODERR)
CTYPE=CLTYPE

IF (LHOOK) CALL DR_HOOK('WRMODERR',1,ZHOOK_HANDLE)
END SUBROUTINE WRMODERR
