SUBROUTINE WROUTSPGB(PSPFLD,KSHMAX,KGRIBIO,CDLEVTYPE)

!**** *WROUTSPGB* _ GRIB codes and writes out spectral fields

!     Purpose.
!     --------
!     Write out spectral fields in GRIB

!**   Interface.
!     ----------
!        *CALL* *WROUTSPGB*(...)

!        Explicit arguments :    
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation
!        - spectral part would only work if input-resol = output resol
!          for vertical and horizontal resolution

!     Externals.   PREGRBENC - modify GRIB headers
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!       Original : 01-12-14 Adapted from WR****
!       P.Towers : 02-11-13 Fixes for fewer writers than gatherers
!       P.Towers : 03-04-23 Added ISETFIELDCOUNTFDB logic
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        G. Radnoti:   25-Aug-2004 treatment of tstep<0 for LTWINC
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!       R. El Khatib : 23-Oct-2008 use suofname
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM,    JPIB,     JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NSTOP  ,NCONF
USE YOMCT3   , ONLY : NSTEP
USE YOMLUN   , ONLY : NULOUT
USE YOMVAR   , ONLY : NSIM4D   ,MCGLVEC  ,MBGVEC   ,LTRREF   ,&
 & LTWANA   ,LTWGRA   ,LTWINC   ,LTWBGV   ,&
 & LTWCGL  
USE YOMSKF   , ONLY : MSKFVEC  ,LTWSKFY  ,LTWSKFZ
USE YOMDIM   , ONLY : NSPEC2
USE YOMOP    , ONLY : CFNRF    ,CFNAN    ,CFNGR    ,CFNINSH  ,CFNBGV   ,&
 & CFNCGL   ,CFNSKFY  ,CFNSKFZ  ,LINC
USE YOMMP    , ONLY : NOUTTYPE
USE YOMDYN   , ONLY : TSTEP
USE YOMRIP   , ONLY : NINDAT ,NSSSSS ,NYMLPPSP
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_PUT, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, Y_IOSTREAM_FDB,&
 & CLOSE_IOREQUEST

IMPLICIT NONE

INTEGER(KIND=JPIM),PARAMETER :: ISHOUR=3600
INTEGER(KIND=JPIM),PARAMETER :: IHDAY=24

!DUMMY VARIABLES

INTEGER(KIND=JPIM),INTENT(IN)    :: KSHMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPFLD(NSPEC2,KSHMAX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGRIBIO(4,KSHMAX) 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDLEVTYPE 
INTEGER(KIND=JPIM)               :: ILEN
!     EXTERNAL INTEGER FUNCTIONS

INTEGER(KIND=JPIM) :: IINC,IMTS
INTEGER(KIND=JPIM) :: IHOUR
INTEGER(KIND=JPIB) :: ISEC
INTEGER(KIND=JPIM) :: IH0,IJ0,IM0,IA0,IDD,ISS,IHR,IMIN,ISC,IJOUR,IMOIS,IAN,ILMOIS(12)
INTEGER(KIND=JPIM) :: IYM
CHARACTER :: CLFNSH*210,CLFNRF*30,CLFANA*30,CLR1*5,CLR2*4,CLMODE*1
CHARACTER :: CLFINC*30
CHARACTER :: CLFGRA*30
CHARACTER :: CLFSKF*30
CHARACTER :: CLFCGL*30
CHARACTER :: CLFBGV*30
CHARACTER :: CLEVT*2

REAL(KIND=JPRB) :: ZHOOK_HANDLE

TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

#include "abor1.intfb.h"
#include "suofname.intfb.h"
#include "fcttim.h"
#include "updcalsec.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WROUTSPGB',0,ZHOOK_HANDLE)

IA0=NCCAA(NINDAT)
IM0=NMM(NINDAT)
IJ0=NDD(NINDAT)
IH0=NSSSSS/ISHOUR
IF (NSTEP <= 0) NYMLPPSP=0
ISEC=NINT(NSTEP*TSTEP,KIND(ISEC))
IHOUR=INT(ISEC/ISHOUR,KIND(IHOUR))
IDD=IHOUR/IHDAY
ISS=MODULO(IHOUR,IHDAY)*ISHOUR
CALL UPDCALSEC(IH0,IJ0,IM0,IA0,IDD,ISS,IHR,IMIN,ISC,IJOUR,IMOIS,IAN,ILMOIS,NULOUT)
IYM=IAN*100+IMOIS

IF(CDLEVTYPE == 'm') THEN
  CLEVT='ML'
ELSEIF(CDLEVTYPE == 'p') THEN
  CLEVT='PL'
ELSEIF(CDLEVTYPE == 'v') THEN
  CLEVT='PV'
ELSEIF(CDLEVTYPE == 't') THEN
  CLEVT='TH'
ELSE
  CALL ABOR1('WROUTSPGB:UNKNOWN LEVEL TYPE')
ENDIF

CLMODE = 'w'

IF(NOUTTYPE /= 2.AND. KSHMAX > 0 )THEN

!      Open output file

  IF (LTRREF) THEN
    
    CLR1 = CFNRF(1:5)
    CLR2 = CFNRF(6:9)
    IF (NSTEP < 0) THEN
      WRITE(CLFNRF,'(A5,A4,I7.6)') CLR1,CLR2,NSTEP
    ELSE
      IF (LINC) THEN
        IINC = NINT(NSTEP*TSTEP/3600._JPRB,KIND(IINC))
      ELSE
        IINC = NSTEP
      ENDIF
      WRITE(CLFNRF,'(A5,A4,''+'',I6.6)') CLR1,CLR2,IINC
    ENDIF
    CLFNSH=CLFNRF

  ELSEIF(LTWANA) THEN
      
    CLFANA = CFNAN
    IF (NSTEP < 0) THEN
      CLFANA(10:10) = '-'
      IINC = ABS(NSTEP)
    ELSEIF (NCONF/100 == 1) THEN
      IMTS = NINT(NSTEP*TSTEP/60._JPRB,KIND(IMTS))
      IINC = (IMTS/60)*100+MOD(IMTS,60)
    ELSE
      IINC = NSTEP
    ENDIF
    WRITE(CLFANA(7:9),'(I3.3)') NSIM4D
    WRITE(CLFANA(11:),'(I6.6)') IINC
    CLFNSH=CLFANA
      
  ELSEIF(LTWINC) THEN
    
    CLFINC = CFNINSH
! Output files labelled with time step in configuration 501 (TL evolution)
    IF (NCONF == 501) THEN
      IINC = NSTEP
    ELSE
      IMTS = NINT(REAL(NSTEP,JPRB)*TSTEP/60._JPRB)
      IINC = (IMTS/60)*100+MOD(IMTS,60)
    ENDIF
    WRITE(CLFINC(9:11),'(I3.3)') NSIM4D
    IF (IINC >= 0) THEN
      WRITE(CLFINC(13:),'(I6.6)') IINC
    ELSE
      WRITE(CLFINC(12:12),'(A1)') '-'
      WRITE(CLFINC(13:),'(I6.6)') -IINC
    ENDIF
    CLFNSH=CLFINC
    
  ELSEIF(LTWGRA) THEN
      
    CLFGRA = CFNGR
    IF (NSTEP < 0) THEN
      CLFGRA(10:10) = '-'
      IINC = ABS(NSTEP)
    ELSEIF (NCONF/100 == 1) THEN
      IMTS = NINT(REAL(NSTEP,JPRB)*TSTEP/60._JPRB)
      IINC = (IMTS/60)*100+MOD(IMTS,60)
    ELSEIF (IABS(NCONF)/100 == 8) THEN
      CLFGRA(10:10) = '-'
      IINC = NSTOP-NSTEP
    ELSE
      IINC = NSTEP
    ENDIF
    WRITE(CLFGRA(7:9),'(I3.3)') NSIM4D
    WRITE(CLFGRA(11:),'(I6.6)') IINC
    CLFNSH=CLFGRA
    
  ELSEIF(LTWBGV) THEN
    
    CLFBGV = CFNBGV
    WRITE(CLFBGV(7:),'(I3.3)') MBGVEC
    CLFNSH=CLFBGV
      
  ELSEIF(LTWCGL) THEN
    
    CLFCGL = CFNCGL
    WRITE(CLFCGL(7:),'(I3.3)') MCGLVEC
    CLFNSH=CLFCGL
    
  ELSEIF(LTWSKFY) THEN
    
    CLFSKF = CFNSKFY
    WRITE(CLFSKF(7:),'(I3.3)') MSKFVEC
    CLFNSH=CLFSKF
    
  ELSEIF(LTWSKFZ) THEN

    CLFSKF = CFNSKFZ
    WRITE(CLFSKF(7:),'(I3.3)') MSKFVEC
    CLFNSH=CLFSKF
      
  ELSE

    CALL SUOFNAME(KSTEP=NSTEP,KDIGITS=6,CDLABEL='ICMSH',CDOFNAME=CLFNSH)
    IF(NSTEP>0) THEN
      ILEN=LEN(TRIM(CLFNSH))-6
      IF ((NYMLPPSP == 0).OR.(IYM == NYMLPPSP).OR. &
    ((IYM /= NYMLPPSP).AND.((IJOUR /= 1).OR.(IHR /= 0)))) THEN
        WRITE(CLFNSH(ILEN:),'("+",I6)') IYM
      ELSE
        WRITE(CLFNSH(ILEN:),'("+",I6)') NYMLPPSP
      ENDIF
    ENDIF

  ENDIF

ENDIF

! IF ((NSTEP <= 0).OR.(NYMLPPSP == 0).OR. &
!  & ((IYM /= NYMLPPSP).AND.((IJOUR /= 1).OR.(IHR /= 0)))) THEN
!   CLMODE='w'
! ELSE
!   CLMODE='a'
! ENDIF

CLMODE='a'

IF ((NSTEP <= 0).OR.((NYMLPPSP /= 0).AND. &
 & (IYM /= NYMLPPSP).AND.(IJOUR == 1).AND.(IHR == 0))) THEN
  NYMLPPSP = NYMLPPSP
ELSE
  NYMLPPSP = IYM
ENDIF

CALL SETUP_IOREQUEST(YL_IOREQUEST,'SPECTRAL_FIELDS',LDGRIB=.TRUE.,&
 & KGRIB2D=KGRIBIO(1,:),KLEVS2D=KGRIBIO(2,:),KBSET2D=KGRIBIO(3,:),&
 & CDLEVTYPE=CLEVT)
IF(NOUTTYPE == 2) THEN
  CALL IO_PUT(Y_IOSTREAM_FDB,YL_IOREQUEST,PR2=PSPFLD)
ELSE
  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFNSH),CDMODE=CLMODE,KIOMASTER=1)
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR2=PSPFLD)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)
ENDIF
 
CALL CLOSE_IOREQUEST(YL_IOREQUEST)

IF (LHOOK) CALL DR_HOOK('WROUTSPGB',1,ZHOOK_HANDLE)

END SUBROUTINE WROUTSPGB
