SUBROUTINE WRPHTRAJT (KSTEP,KSTA,KEND,KJUMP,KFIELDS,&
 & PTENTVD5,PTENQVD5,PTENUVD5,PTENVVD5,&
 & PTENTGW5,PTENUGW5,PTENVGW5,&
 & PTENTCO5,PTENQCO5,PTENUCO5,PTENVCO5,&
 & PTENTCL5,PTENUWM5,PTENVWM5,&
 & PDIFTQ5 ,PDIFTS5 ,PEMTED5 ,PTRSOL5,&
 & PA5 ,PL5 ,PI5,&
 & PLUDE5  ,PLU5    ,&
 & PFPLCL5 ,PFPLCN5 ,PUCFL5  ,PVCFL5 ,PTCFL5 ,PQCFL5)  

!**** *WRPHTRAJT* - Write out the trajectory of phys.tendencies 
!                   to work file

!     Purpose.  Write out the trajectory of phys.tendencies 
!               to work file

!**   Interface.
!     ----------
!        *CALL* *WRPHTRAJT(...)*

!        Explicit arguments :  KSTEP   - time step            (input)
!        --------------------  KSTA    - start adress         (input)
!                              KEND    - end adress           (input)
!                              KJUMP   - offset into gp array (input)
!                              KFIELDS - number of fields     (input)
!                              PTENTVD5 to PTRSOL5: trajectory of
!                                        phys.tendencies to be written (input)

!        Implicit arguments :  None
!        --------------------

!     Method.
!     -------

!     Externals. FILLB    - interface to packing routines
!     ---------- 

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Marta Janiskova  *ECMWF*

!     Modifications.
!     --------------
!        Original : 2000-01-20
!        Modified : 
!           P. Lopez   ECMWF  2003-12:   Added arguments PLUDE5 and PLU5.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!           M. Janiskova  18-Sep-2009: added tendencies needed for non-orog.gwd
!           M. Janiskova  18-May-2010: Cleaning & saving fields for LEMWAVE
!        ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMIOP   , ONLY : NPCKFPH5 ,NEXPBPH5
USE YOMTPHY  , ONLY : TRAJPHTE
USE YOPHNC   , ONLY : LETRAJPT ,LERADSW2 ,LERADN2, LERADLW2, LENCLD2, LEGWWMS2
USE YOEPHY   , ONLY : LEMWAVE

!        ------------------------------------------------------------------

IMPLICIT NONE

! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KJUMP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENTVD5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENQVD5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENUVD5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENVVD5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENTGW5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENUGW5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENVGW5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENTCO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENQCO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENUCO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENVCO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENTCL5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENUWM5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENVWM5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ5(NPROMA,NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS5(NPROMA,NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMTED5(NPROMA,NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTRSOL5(NPROMA,NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLUDE5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLU5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL5(NPROMA,NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN5(NPROMA,NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUCFL5(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCFL5(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCFL5(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCFL5(NPROMA)
INTEGER(KIND=JPIM) :: IBLOCKS, IJUMP,  IPCKUNIT, ISTAGPB, IWORDS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "fillb3.intfb.h"

!     ------------------------------------------------------------------

!*       1.    WRITE OUT GRIDPOINT ARRAY.
!              --------------------------

!*       1.1       CALCULATE LENGTHS

IF (LHOOK) CALL DR_HOOK('WRPHTRAJT',0,ZHOOK_HANDLE)
IWORDS = KEND-KSTA+1

!*       1.2       WRITE OUT BUFFER

IJUMP=KJUMP-1
IF (NPCKFPH5 == 1) THEN
  ISTAGPB = 1+IJUMP*KFIELDS
ELSE

!  It is assumed that TRAJPHYS data always is read/written in
!  NPROMA chunks (except for the last potentially smaller chunk).
!  Restrictions are necessary because reading/writing of packed
!  data has to start on a word boundary.

  IBLOCKS=IJUMP/NPROMA
  IF (IBLOCKS*NPROMA /= IJUMP)&
   & CALL ABOR1('WRPHTRAJT: INCONSISTENT WRITING OF PACKED DATA')  
  IPCKUNIT = (NPROMA+NPCKFPH5-1)/NPCKFPH5
  ISTAGPB = 1+IPCKUNIT*IBLOCKS*KFIELDS
ENDIF

!        1.3      UPPER AIR 3D FIELDS

IF (.NOT. LETRAJPT) THEN
  IF (LERADN2.OR.LERADSW2.OR.LERADLW2) THEN
    CALL FILLB3(NFLEVG+1,IWORDS,PEMTED5(KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(NFLEVG+1,IWORDS,PTRSOL5(KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF
  IF (LENCLD2) THEN
    CALL FILLB3(NFLEVG,IWORDS,PA5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PL5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PI5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PLUDE5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PLU5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF
  IF (LEGWWMS2) THEN
    CALL FILLB3(NFLEVG,IWORDS,PTENUWM5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(NFLEVG,IWORDS,PTENVWM5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF
ELSE
  CALL FILLB3(NFLEVG,IWORDS,PTENTVD5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENQVD5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENUVD5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENVVD5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENTGW5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENUGW5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENVGW5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENTCO5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENQCO5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENUCO5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  CALL FILLB3(NFLEVG,IWORDS,PTENVCO5 (KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)  

  CALL FILLB3(NFLEVG+1,IWORDS,PDIFTQ5(KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)
  CALL FILLB3(NFLEVG+1,IWORDS,PDIFTS5(KSTA,1),TRAJPHTE(:,KSTEP),&
   & ISTAGPB,NPCKFPH5,NEXPBPH5)

  IF (LERADN2.OR.LERADSW2.OR.LERADLW2) THEN
    CALL FILLB3(NFLEVG+1,IWORDS,PEMTED5(KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(NFLEVG+1,IWORDS,PTRSOL5(KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF

  IF (LENCLD2) THEN
    CALL FILLB3(NFLEVG,IWORDS,PA5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PL5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PI5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)    
    CALL FILLB3(NFLEVG,IWORDS,PLUDE5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
    CALL FILLB3(NFLEVG,IWORDS,PLU5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)  
  ENDIF

  IF (.NOT. LEMWAVE) THEN
    CALL FILLB3(NFLEVG,IWORDS,PTENTCL5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF

  IF (LENCLD2 .AND. LEMWAVE) THEN
    CALL FILLB3(NFLEVG+1,IWORDS,PFPLCL5(KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(NFLEVG+1,IWORDS,PFPLCN5(KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(1,IWORDS,PUCFL5(KSTA),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(1,IWORDS,PVCFL5(KSTA),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(1,IWORDS,PTCFL5(KSTA),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(1,IWORDS,PQCFL5(KSTA),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF

  IF (LEGWWMS2) THEN
    CALL FILLB3(NFLEVG,IWORDS,PTENUWM5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
    CALL FILLB3(NFLEVG,IWORDS,PTENVWM5 (KSTA,1),TRAJPHTE(:,KSTEP),&
     & ISTAGPB,NPCKFPH5,NEXPBPH5)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRPHTRAJT',1,ZHOOK_HANDLE)
END SUBROUTINE WRPHTRAJT

