SUBROUTINE WRSPECA(KUNIT)

!**** *WRSPECA*  - Write the spectral fields to FA (Arpege or Aladin)

!     Purpose.
!     --------
!         Write the spectral fields of the model to ARPEGE ALADIN file.

!**   Interface.
!     ----------
!        *CALL* *WRSPECA(.....)*

!        Explicit arguments :
!        --------------------
!        NTRJSH : file logical unit number
!        SPVOR etc. - spectral fields

!        Implicit arguments :
!        --------------------
!        See modules above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        ABOR1
!        FANOUV
!        FAIRNO
!        FAISAN
!        FACOND
!        SPREORD
!        DIWRSPE0
!        SUMPIOH

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        Stjepan IVATEK-SAHDAN and Ryad El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!   Original : 01-04-17 from suspeca
!   Modified 02-03-08 C. Fischer - rescaling of new NH variables
!   Modified 02-09-30 P. Smolikova - rescaling of d4 in NH
!   R. El Khatib : 03-08-05 gfl+remove dummies since it it called only here.
!   O.Spaniel    : 03-04-15 cleaning-see interface wrspeca.h
!   R. El Khatib : 03-04-17 Cleanups
!   Modified 03-05-01 A. Bogatchev - check sizes and gnhpdvdconv
!   G. Hello : supress the call to gnhpdvdconv
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   P.Termonia: 03-09-01 writing RMCUFFP
!   D. Paradis & R. El Khatib : 04-07-22 GRIBEX
!   Y. Bouteloup : 05-10-18 YCVGQ not write even it is spectral
!   Y. Seity     : 23-06 2006: remove YCVGQ specificities and replace by more
!                  general tests on LREQOUT
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   R. El Khatib : 15-Sep-2008 I/O savings
!     ----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NTRJSH   ,NULOUT
USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG   ,NFLSUR   ,NSPEC2G  ,NSPEC2   ,&
 & NSEFRE   ,NF3D  
USE YOMLAP   , ONLY : NVALUE   ,RLAPIN   ,RLAPDI
USE YOMCT0   , ONLY : NPROC    ,LNHDYN   ,LELAM    ,NPRTRV   ,NCONF    ,&
 & NPRTRW  
USE YOMDYN   , ONLY : SIDELP
USE YOMMP    , ONLY : MYPROC   ,MYSETV   ,NSTROUT  ,NPTRLL   ,NBSETLEV ,&
 & NBSETSP  ,MYSETW   ,MYLEVS  
USE YOMMPG   , ONLY : NPOSSP
USE YOMFA    , ONLY :   YFAPSI   ,YFAKHI   ,YFAT     ,YFAPD    ,YFAVD    , &
 & YFASP    ,YFAUGEO  ,YFAVGEO  ,YFAFSP1  ,YFAFSP2  ,YFAFSP3  ,YFAFSP4  , &
 & YFAFSP5
USE YOMVAR   , ONLY : LTWGRA
USE YOMDYNA  , ONLY : LNHX
USE YOMSENS  , ONLY : LGRVOL
USE YOMGEM   , ONLY : NGPTOT   ,VP00
USE YOM_YGFL , ONLY : YGFL     ,YGFLC   
USE YOMSP    , ONLY : SP5SP    ,SPDIV    ,SPGFL    ,SPOR     ,SPSP     ,&
 & SPSPD    ,SPSVD    ,SPVOR    ,SPT      ,SPUB     ,&
 & SPVB  
USE YOMAFN   , ONLY : TFP_FIS
USE YOMMCUF  , ONLY : NCUFNR   ,RMCUFFP

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KUNIT ! logical unit for file

!     IFLDSPG: total number of fields to read
!     IFLDSCH: number of fields in each chunk
!     ICHUNKS: number of chunks of fields
!     ICHOFF : chunk offset for fields
!     INFD   : number of fields in the current chunk for each processor
!     IFLDOFF: fields offset after distribution among NSTROUT procs.
!     CLPREF : prefix of fields to read 
!     CLSUFF : suffix of fields to read
!     ILEVG  : global level of fields to read
!     ILEVL  : local level of fields to send/receive
!     IVSET  : vertical set each field belongs (for 2nd level of distribution)
!     IVDIS  : vertical set corresponding to each chunk
!     IFLDG  : field index in global arrays
!     ZDATA  : a spectral field written to file
!     ZSPBUF : a chunk of of spectral buffer fields to distribute ; 
!              NB : while receiving, ZSPBUF contains global spectra ; 
!                   while sending, it contain local spectra
!     ISPOFF : offset of spectral coefficient
!     JACTION: action : 0 to read and send ; 1 to receive.
!     CLNOMA : name of fields to write out
!     ILNOMA : length of the names of fields to write out
!     ZVALCO : packed fields to write out
!     ILONGD : length of the packed fields to write out
!     IMASK  : Binary mask to communicate packed fields+name
!     ISIZPK : dimension of a field for packing

INTEGER(KIND=JPIM) :: IFLDSPG, ICHUNKS, JCH, ICHOFF, IPT, JV, JGFL, JSP, ISPOFF
INTEGER(KIND=JPIM) :: JFLD, IFLDG, JACTION, IRECV, ISEND, JROC, ISIZPK
INTEGER(KIND=JPIM) :: INGRIB, INBPDG, INBCSP, ISTRON, IPUILA, IDMOPL, INGRIG
INTEGER(KIND=JPIM) :: INFD(NPROC), IFLDOFF(NPROC), IREP,I3DINI, ITAG
INTEGER(KIND=JPIM) :: IUNTIN
INTEGER(KIND=JPIM) :: IPTR(1)
LOGICAL   :: LLCOSP = .TRUE.
LOGICAL   :: LLFILE_TO_MODEL = .FALSE.
LOGICAL   :: LLINOR
REAL(KIND=JPRB)    ::    ZWGHT(NFLEVL)

INTEGER(KIND=JPIM),    ALLOCATABLE :: ILEVG (:), IVSET (:), ILEVL (:), IVDIS(:) 
INTEGER(KIND=JPIM),    ALLOCATABLE :: ILEVG2(:), IVSET2(:), ILEVL2(:), IFLDSCH(:)
INTEGER(KIND=JPIM),    ALLOCATABLE :: ILNOMA(:,:), ILONGD(:,:), IMASK(:,:)
CHARACTER (LEN = 4),  ALLOCATABLE :: CLPREF(:), CLPREF2(:)
CHARACTER (LEN = 12), ALLOCATABLE :: CLSUFF(:), CLSUFF2(:)
CHARACTER (LEN = 16), ALLOCATABLE :: CLNOMA(:,:)
REAL(KIND=JPRB) ,      ALLOCATABLE :: ZSPBUF(:,:), ZSPU(:,:), ZSPV(:,:), ZDATA(:)
REAL(KIND=JPRB) ,      ALLOCATABLE :: ZVALCO(:,:)
REAL(KIND=JPRB) ,      ALLOCATABLE :: ZSPBUFTSP(:),ZGDSP(:),ZGD5SP(:)
REAL(KIND=JPRB) :: ZAVE(1,1), ZMAX(1,1), ZMIN(1,1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "diwrspe0.intfb.h"
#include "gathflnm.intfb.h"
#include "ereespe.intfb.h"
#include "esperee.intfb.h"
#include "reespe.intfb.h"
#include "speree.intfb.h"
#include "spreord.intfb.h"
#include "sumpioh.intfb.h"
#include "evduvgeo.intfb.h"

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRSPECA',0,ZHOOK_HANDLE)
I3DINI=0
LLINOR = .TRUE.
IPTR(1)=1

IF (MYPROC <= NSTROUT) THEN
  ALLOCATE(ZDATA(NSEFRE))
ENDIF

!*       0. PRE-CONDITIONMENT OF WIND FOR ALADIN
!           ------------------------------------

!        0.1 Compute geographical wind from Vorticity, Divergence & mean wind

IF (LELAM) THEN
  ALLOCATE(ZSPU(NFLSUR,NSPEC2))
  ALLOCATE(ZSPV(NFLSUR,NSPEC2))
  WRITE (NULOUT,*)' WRSPECA : VOR,DIV + MEAN U,V --> U.GEO,V.GEO '
  CALL EVDUVGEO(SPVOR,SPDIV,SPUB,SPVB,ZSPU,ZSPV)
ENDIF

!*       1. FIND NUMBER OF FIELDS TO WRITE TO FILE
!           --------------------------------------

!        1.2 Count fields

IFLDSPG=0
! Orography :
IF (LLINOR) THEN
  IFLDSPG=IFLDSPG+1
ENDIF
! Ps and 3D fields :
IF (I3DINI == 0) THEN
  IFLDSPG = IFLDSPG + 1 + NF3D*NFLEVG + NCUFNR
ENDIF
! Take out the "X" part of divergence when set
IF (LNHX) THEN
  IFLDSPG = IFLDSPG - NFLEVG
ENDIF
! Remore Spectral GFL not Required in output
DO JGFL=1, YGFL%NUMFLDS
  IF (YGFLC(JGFL)%LSP) THEN
    IF (.NOT. YGFLC(JGFL)%LREQOUT) THEN
      IFLDSPG=IFLDSPG-NFLEVG
    ENDIF   
  ENDIF
ENDDO

!*       2. WRITE TO FILE
!           -------------

IF (IFLDSPG > 0) THEN

!        2.1 Setup various local variables related to file

  ALLOCATE(CLPREF(IFLDSPG),CLSUFF(IFLDSPG),ILEVG (IFLDSPG))
  ALLOCATE(ILEVL(IFLDSPG),IVSET (IFLDSPG))
  IPT=0

! Orography :
  IF (LLINOR) THEN
    IPT=IPT+1
    CLPREF(IPT)=TFP_FIS%CLNAME(1:4)
    ILEVG (IPT)=0
    ILEVL (IPT)=0
    CLSUFF(IPT)=TFP_FIS%CLNAME(5:)
    IVSET (IPT)=NBSETSP
  ENDIF

! Ps and 3D fields :
  IF (I3DINI == 0) THEN
    IPT=IPT+1
    CLPREF(IPT)=YFASP%CLNAME(1:4)
    ILEVG (IPT)=0
    ILEVL (IPT)=0
    CLSUFF(IPT)=YFASP%CLNAME(5:)
    IVSET (IPT)=NBSETSP
    IF (NCUFNR>=1) THEN
      IPT=IPT+1
      CLPREF(IPT)=YFAFSP1%CLNAME(1:4)
      ILEVG (IPT)=0
      ILEVL (IPT)=0
      CLSUFF(IPT)=YFAFSP1%CLNAME(5:)
      IVSET (IPT)=NBSETSP
    ENDIF
    IF (NCUFNR>=2) THEN
      IPT=IPT+1
      CLPREF(IPT)=YFAFSP2%CLNAME(1:4)
      ILEVG (IPT)=0
      ILEVL (IPT)=0
      CLSUFF(IPT)=YFAFSP2%CLNAME(5:)
      IVSET (IPT)=NBSETSP
    ENDIF
    IF (NCUFNR>=3) THEN
      IPT=IPT+1
      CLPREF(IPT)=YFAFSP3%CLNAME(1:4)
      ILEVG (IPT)=0
      ILEVL (IPT)=0
      CLSUFF(IPT)=YFAFSP3%CLNAME(5:)
      IVSET (IPT)=NBSETSP
    ENDIF
    IF (NCUFNR>=4) THEN
      IPT=IPT+1
      CLPREF(IPT)=YFAFSP4%CLNAME(1:4)
      ILEVG (IPT)=0
      ILEVL (IPT)=0
      CLSUFF(IPT)=YFAFSP4%CLNAME(5:)
      IVSET (IPT)=NBSETSP
    ENDIF
    IF (NCUFNR>=5) THEN
      IPT=IPT+1
      CLPREF(IPT)=YFAFSP5%CLNAME(1:4)
      ILEVG (IPT)=0
      ILEVL (IPT)=0
      CLSUFF(IPT)=YFAFSP5%CLNAME(5:)
      IVSET (IPT)=NBSETSP
    ENDIF
    IF (NF3D > 0) THEN
      CLPREF(IPT+1:)='S   '
      DO JFLD=IPT+1, IFLDSPG
        ILEVG (JFLD) = MOD(JFLD-(IPT+1),NFLEVG) + 1
        IVSET (JFLD) = NBSETLEV(ILEVG(JFLD))
        ILEVL (JFLD) = ILEVG(JFLD)-NPTRLL(IVSET(JFLD))+1 
      ENDDO 
!     Psi & Khi or U & V :
      IF (LELAM) THEN
        CLSUFF(IPT+1:IPT+NFLEVG)=YFAUGEO%CLNAME(1:12)
        CLSUFF(IPT+NFLEVG+1:IPT+2*NFLEVG)=YFAVGEO%CLNAME(1:12)
      ELSE
        CLSUFF(IPT+1:IPT+NFLEVG)=YFAPSI%CLNAME(1:12)
        CLSUFF(IPT+NFLEVG+1:IPT+2*NFLEVG)=YFAKHI%CLNAME(1:12)
      ENDIF
      IPT=IPT+2*NFLEVG
!     Temperature :
      CLSUFF(IPT+1:IPT+NFLEVG)=YFAT%CLNAME(1:12)
      IPT=IPT+NFLEVG
!     Pressure departure & Vertical divergence:
      IF (LNHDYN) THEN
        CLSUFF(IPT+1:IPT+NFLEVG)=YFAPD%CLNAME(1:12)
        CLSUFF(IPT+NFLEVG+1:IPT+2*NFLEVG)=YFAVD%CLNAME(1:12)
        IPT=IPT+2*NFLEVG
      ENDIF
!     Other prognostic variables :
      DO JGFL=1, YGFL%NUMFLDS
        IF (YGFLC(JGFL)%LSP) THEN
          IF (YGFLC(JGFL)%LREQOUT) THEN
             CLSUFF(IPT+1:IPT+NFLEVG)=YGFLC(JGFL)%CNAME(1:12)
             IPT=IPT+NFLEVG
          ENDIF   
        ENDIF
      ENDDO
    ENDIF
  ENDIF

  IF (IPT /= IFLDSPG) THEN
    CALL ABOR1('WRSPECA : FIELDS DESCRIPTORS ARE UNCOMPLETE !')
  ENDIF 

!        2.2 Setup chunks of fields

! Within a chunk, all fields must belong to the same V-set
  IF (NPROC > 1) THEN
!   Spare communications
    ICHUNKS=NPRTRV
    ALLOCATE(IFLDSCH(NPRTRV),IVDIS(NPRTRV))
    IF (NPRTRV > 1) THEN
!     Reorder descriptors array and count the number of fields per chunk
      ALLOCATE(CLPREF2(IFLDSPG)) ; CLPREF2(:)=CLPREF(:)
      ALLOCATE(CLSUFF2(IFLDSPG)) ; CLSUFF2(:)=CLSUFF(:)
      ALLOCATE(ILEVG2 (IFLDSPG)) ; ILEVG2 (:)=ILEVG (:)
      ALLOCATE(ILEVL2 (IFLDSPG)) ; ILEVL2 (:)=ILEVL (:)
      ALLOCATE(IVSET2 (IFLDSPG)) ; IVSET2 (:)=IVSET (:)
      IFLDG=0
      DO JV=1, NPRTRV
        IVDIS(JV)=JV
        IFLDSCH(JV)=0
        DO JFLD=1, IFLDSPG
          IF (IVSET2(JFLD) == JV) THEN
            IFLDG=IFLDG+1
            CLPREF(IFLDG)=CLPREF2(JFLD)
            CLSUFF(IFLDG)=CLSUFF2(JFLD)
            ILEVG (IFLDG)=ILEVG2 (JFLD)
            ILEVL (IFLDG)=ILEVL2 (JFLD)
            IVSET (IFLDG)=IVSET2 (JFLD)
            IFLDSCH(JV)=IFLDSCH(JV)+1
          ENDIF
        ENDDO
      ENDDO
      DEALLOCATE(CLPREF2)
      DEALLOCATE(CLSUFF2)
      DEALLOCATE(ILEVG2 )
      DEALLOCATE(ILEVL2 )
      DEALLOCATE(IVSET2 )
    ELSE
      IFLDSCH(:)=IFLDSPG
      IVDIS(:)=1
    ENDIF
    ISEND=0
    IRECV=1
  ELSE
!   Spare memory
    ICHUNKS=IFLDSPG
    ALLOCATE(IFLDSCH(ICHUNKS),IVDIS(ICHUNKS))
    IFLDSCH(:)=1
    IVDIS(:)=1
    ISEND=1
    IRECV=1
  ENDIF

  ISIZPK=NSEFRE+2 ! Overdimension enables to write non-packed data

!        2.3 Distribute and write data

  DO JACTION=ISEND,IRECV
  
    ICHOFF=0
  
    DO JCH=1, ICHUNKS
  
!*         2.3.1 Setup distribution among I/O processors

      CALL SUMPIOH(IFLDSCH(JCH),NPROC,NSTROUT,INFD,IFLDOFF)

      IF (JACTION == IRECV .AND. INFD(MYPROC) > 0) THEN
        ALLOCATE(ZSPBUF(NSPEC2G,INFD(MYPROC)))
      ENDIF

!*         2.3.3 Processors resume the spectral buffer (and possibly send)

      IF (IVDIS(JCH) == MYSETV) THEN

!       Action because we are the proper V-set (actual sending)
        DO JROC=1, NSTROUT
          IF (JACTION == ISEND .AND. JROC /= MYPROC .AND. INFD(JROC) > 0) THEN     
!           I prepare to send to the others :
            ISPOFF=0
            ALLOCATE(ZSPBUF(NSPEC2,INFD(JROC)))
          ELSEIF (JACTION == IRECV .AND. JROC == MYPROC) THEN
!           I prepare to fill my own buffer :
            ISPOFF=NPOSSP(MYSETW)-1
          ELSE 
            CYCLE
          ENDIF
          DO JFLD=1,INFD(JROC)
            IFLDG=ICHOFF+IFLDOFF(JROC)+JFLD
            IF     (CLSUFF(IFLDG) == TFP_FIS%CLNAME(5:)) THEN
!             Orography :
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=SPOR(JSP)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFASP%CLNAME(5:)) THEN
!             Surface pressure :
              IF (LTWGRA) THEN
!               Write gradient w.r.t. surface pressure dJ/dps=dJ/(ps.dLogps)
                WRITE(NULOUT,*) ' CONVERTING GRAD/LOGSP INTO GRAD/SP'
                ALLOCATE(ZSPBUFTSP(NSPEC2),ZGDSP(NGPTOT),ZGD5SP(NGPTOT))
                ZSPBUFTSP(1:NSPEC2)=SPSP(1:NSPEC2)
                IF (LELAM) THEN
                  CALL ESPEREE(1,1,ZSPBUFTSP,ZGDSP)
                ELSE
                  CALL SPEREE(1,1,ZSPBUFTSP,ZGDSP)
                ENDIF
                IF (NCONF/100 /= 1) THEN
                  ZSPBUFTSP(1:NSPEC2)=SP5SP(1:NSPEC2)
                  IF (LELAM) THEN
                    CALL ESPEREE(1,1,ZSPBUFTSP,ZGD5SP)
                  ELSE
                    CALL SPEREE(1,1,ZSPBUFTSP,ZGD5SP)
                  ENDIF
                ELSE
!                 In variational, SP5SP is corrupted by COSJC and COSJG
!                 Use approximate Ps=1015hPa
                  ZGD5SP(1:NGPTOT)=LOG(VP00)
                ENDIF
                ZGDSP(1:NGPTOT)=ZGDSP(1:NGPTOT)/EXP(ZGD5SP(1:NGPTOT))
                IF (LELAM) THEN
                  CALL EREESPE(1,1,ZSPBUFTSP,ZGDSP)
                ELSE
                  CALL REESPE(1,1,ZSPBUFTSP,ZGDSP)
                ENDIF
                DO JSP=1,NSPEC2
                  ZSPBUF(JSP+ISPOFF,JFLD)=ZSPBUFTSP(JSP)
                ENDDO
                DEALLOCATE(ZSPBUFTSP,ZGDSP,ZGD5SP)
              ELSE
                DO JSP=1,NSPEC2
                  ZSPBUF(JSP+ISPOFF,JFLD)=SPSP(JSP)
                ENDDO
              ENDIF
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=SPSP(JSP)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP1%CLNAME(5:) .AND. NCUFNR >=1) THEN
!             Filtered ln Ps nr 1 for monitoring the coupling updates 
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=RMCUFFP(JSP,0,1)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP2%CLNAME(5:) .AND. NCUFNR >=2) THEN
!             Filtered ln Ps nr 2 for monitoring the coupling updates 
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=RMCUFFP(JSP,0,2)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP3%CLNAME(5:) .AND. NCUFNR >=3) THEN
!             Filtered ln Ps nr 3 for monitoring the coupling updates 
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=RMCUFFP(JSP,0,3)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP4%CLNAME(5:) .AND. NCUFNR >=4) THEN
!             Filtered ln Ps nr 4 for monitoring the coupling updates 
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=RMCUFFP(JSP,0,4)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP5%CLNAME(5:) .AND. NCUFNR >=5) THEN
!             Filtered ln Ps nr 5 for monitoring the coupling updates 
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=RMCUFFP(JSP,0,5)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAUGEO%CLNAME(1:12)) THEN
!             Wind/x momentum
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=ZSPU(ILEVL(IFLDG),JSP)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAVGEO%CLNAME(1:12)) THEN
!             Wind/y momentum :
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=ZSPV(ILEVL(IFLDG),JSP)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAPSI%CLNAME(1:12)) THEN
!             Psi : 
              IF (LTWGRA) THEN
                IF (LGRVOL) THEN
                  ZWGHT(1:NFLEVL)=1.0_JPRB/SIDELP(MYLEVS(1:NFLEVL))
                ELSE
                  ZWGHT(1:NFLEVL)=1.0_JPRB
                ENDIF
                DO JSP=1,NSPEC2
                  ZSPBUF(JSP+ISPOFF,JFLD)=&
                   & SPDIV(ILEVL(IFLDG),JSP)*RLAPDI(NVALUE(JSP))*&
                   & ZWGHT(ILEVL(IFLDG))  
                ENDDO
              ELSE
                DO JSP=1,NSPEC2
                  ZSPBUF(JSP+ISPOFF,JFLD)=&
                   & SPDIV(ILEVL(IFLDG),JSP)*RLAPIN(NVALUE(JSP))  
                ENDDO
              ENDIF
            ELSEIF (CLSUFF(IFLDG) == YFAKHI%CLNAME(1:12)) THEN
!             Khi :
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=&
                 & SPVOR(ILEVL(IFLDG),JSP)*RLAPIN(NVALUE(JSP))  
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAT%CLNAME(1:12)) THEN
!             Temperature :
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=SPT(ILEVL(IFLDG),JSP)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAPD%CLNAME(1:12)) THEN
!             NH Pressure departure :
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=SPSPD(ILEVL(IFLDG),JSP)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAVD%CLNAME(1:12)) THEN
!             NH vertical divergence :
              DO JSP=1,NSPEC2
                ZSPBUF(JSP+ISPOFF,JFLD)=SPSVD(ILEVL(IFLDG),JSP)
              ENDDO
            ELSE
!             Other prognostic variables :
              DO JGFL=1, YGFL%NUMFLDS
                IF (YGFLC(JGFL)%LSP) THEN
                  IF (CLSUFF(IFLDG) == YGFLC(JGFL)%CNAME(1:12)) THEN
                    DO JSP=1,NSPEC2
                      ZSPBUF(JSP+ISPOFF,JFLD) = &
                       & SPGFL(ILEVL(IFLDG),JSP,YGFLC(JGFL)%MPSP)  
                    ENDDO
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO
          IF (JACTION == ISEND .AND. JROC /= MYPROC .AND. INFD(JROC) > 0) THEN     
!           I send : 
            CALL DIWRSPE0(JROC,IVDIS(JCH),JCH,ISEND,NSPEC2,INFD(JROC),NPOSSP,&
             & NPRTRW,ZSPBUF)  
            DEALLOCATE(ZSPBUF)
          ELSEIF (JACTION == IRECV .AND. JROC == MYPROC) THEN
!           My part of buffer is filled :
            EXIT
          ENDIF
        ENDDO

      ENDIF

      IF (JACTION == IRECV .AND. INFD(MYPROC) > 0) THEN     
     
!*      2.3.2 I/O processors receive and pack fields, then resend to processor #1
     
        IF (NPROC > 1) THEN
          CALL DIWRSPE0(MYPROC,IVDIS(JCH),JCH,IRECV,NSPEC2G,INFD(MYPROC),&
           & NPOSSP,NPRTRW,ZSPBUF)  
        ENDIF

        IUNTIN=KUNIT

        ALLOCATE(CLNOMA(INFD(MYPROC),1),ILNOMA(INFD(MYPROC),1),&
         & ILONGD(INFD(MYPROC),1))  
        ALLOCATE(ZVALCO(ISIZPK,INFD(MYPROC)))
        CALL FAVEUR(IREP,IUNTIN,INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
!       data should be reordered only if packing type is not -1 and not 3
!       Notice : If INGRIB=-1 or 3 the field will be a bit larger than if 
!       INGRIB=0, 1 or 2 because then there is no reordering : so the 
!       column 0 is doubled.
        IF (INGRIB==-1 .OR. INGRIB==3) THEN
          INGRIG=-1
        ELSE
          INGRIG=0
        ENDIF
        DO JFLD=1,INFD(MYPROC)
          IFLDG=ICHOFF+IFLDOFF(MYPROC)+JFLD
          IF (TRIM(CLPREF(IFLDG))//CLSUFF(IFLDG) == TFP_FIS%CLNAME) THEN
!           Do not pack surface geopotential
            CALL FAGOTE(IREP,IUNTIN,INGRIG,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
          ENDIF
          IF (INGRIG==0) THEN
            CALL SPREORD(1,ZDATA,ZSPBUF(1,JFLD),LLFILE_TO_MODEL)
            CALL FACOND(IREP,IUNTIN,CLPREF(IFLDG),ILEVG(IFLDG),CLSUFF(IFLDG),&
             & ZDATA,LLCOSP,CLNOMA(JFLD,1),ILNOMA(JFLD,1),&
             & ZVALCO(1,JFLD),ILONGD(JFLD,1))  
          ELSE
            CALL FACOND(IREP,IUNTIN,CLPREF(IFLDG),ILEVG(IFLDG),CLSUFF(IFLDG),&
             & ZSPBUF(1,JFLD),LLCOSP,CLNOMA(JFLD,1),ILNOMA(JFLD,1),&
             & ZVALCO(1,JFLD),ILONGD(JFLD,1))
          ENDIF
          IF (TRIM(CLPREF(IFLDG))//CLSUFF(IFLDG) == TFP_FIS%CLNAME) THEN
!           Reset packing after the treatment of surface geopotential           
            CALL FAGOTE(IREP,IUNTIN,INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
          ENDIF
        ENDDO
        DEALLOCATE(ZSPBUF)

        IF (MYPROC /= 1) THEN
          ITAG=MYPROC+(JCH-1)*NPROC
          ALLOCATE(IMASK(INFD(MYPROC),1)) ; IMASK(:,:)=1
          CALL GATHFLNM(0,ITAG,INFD(MYPROC),1,CLNOMA,ILNOMA,ZVALCO,ILONGD,&
           &IMASK,ISIZPK,ZAVE,ZMIN,ZMAX,.FALSE.,IPTR)
          DEALLOCATE(IMASK)
        ELSE 
          DO JFLD=1,INFD(MYPROC)
            CALL FAISAN(IREP,NTRJSH,CLNOMA(JFLD,1)(1:ILNOMA(JFLD,1)),&
             & ZVALCO(1,JFLD),ILONGD(JFLD,1))  
          ENDDO
        ENDIF

        DEALLOCATE(CLNOMA,ILNOMA,ILONGD)
        DEALLOCATE(ZVALCO)
         
      ENDIF
  
      ICHOFF=ICHOFF+IFLDSCH(JCH)

    ENDDO ! JCH

!   2.4 Processor #1 finally collects all fields from the other I/O processors

    IF (MYPROC == 1 .AND. NSTROUT > 1 .AND. JACTION == IRECV) THEN
      DO JCH=1, ICHUNKS
        CALL SUMPIOH(IFLDSCH(JCH),NPROC,NSTROUT,INFD,IFLDOFF)
        DO JROC=2, NSTROUT
          IF( INFD(JROC) > 0 ) THEN
            ALLOCATE(CLNOMA(INFD(JROC),1),ILNOMA(INFD(JROC),1),ILONGD(INFD(JROC),1))
            ALLOCATE(ZVALCO(ISIZPK,INFD(JROC)))
            ITAG=JROC+(JCH-1)*NPROC
            ALLOCATE(IMASK(INFD(JROC),1))
            CALL GATHFLNM(1,ITAG,INFD(JROC),1,CLNOMA,ILNOMA,ZVALCO,ILONGD,IMASK,&
             &ISIZPK,ZAVE,ZMIN,ZMAX,.FALSE.,IPTR)
            DO JFLD=1,INFD(JROC)
              CALL FAISAN(IREP,NTRJSH,CLNOMA(JFLD,1)(1:ILNOMA(JFLD,1)),&
               & ZVALCO(1,JFLD),ILONGD(JFLD,1))  
            ENDDO
            DEALLOCATE(CLNOMA,ILNOMA,ILONGD)
            DEALLOCATE(ZVALCO)
            DEALLOCATE(IMASK)
          ENDIF
        ENDDO
      ENDDO
    ENDIF

  ENDDO ! JACTION

!   2.4 Release descriptors

  DEALLOCATE(CLPREF,CLSUFF,ILEVG)
  DEALLOCATE(ILEVL,IVSET)
  DEALLOCATE(IFLDSCH,IVDIS)

ENDIF ! (IFLDSPG > 0)

!        2.1 Release conditionnally allocated arrays

IF (LELAM) THEN
  DEALLOCATE(ZSPU)
  DEALLOCATE(ZSPV)
ENDIF

IF (MYPROC <= NSTROUT) THEN
  DEALLOCATE(ZDATA)
ENDIF

IF (LHOOK) CALL DR_HOOK('WRSPECA',1,ZHOOK_HANDLE)

END SUBROUTINE WRSPECA
