SUBROUTINE ZERODDH
!****
!     ------------------------------------------------------------------

!     INITIALIZING THE WORK ARRAYS
!     DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!     ------------------------------------

!       -------------------------------------------------------
!     GOAL
!     ----
!      INITIALISING THE WORK ARRAYS IN *YOMTDDH* AT THE BEGINING
!     OF EACH TIMESTEP
!      2 OPERATIONS ARE PERFORMED
!     1- ZEROING THE INSTANTANEOUS VALUES IN HDCV1
!     2- COPY OF THE ACCUMULATED VALUES OF HDCV1 INTO HDCV0

!      LA VECTORISATION SE FAIT SELON LE CHAMP. ON NE DISTINGUE PAS
!      SELON LES NIVEAUX
!       LA COHERENCE DANS LES DIMENSIONS EST DONC CRITIQUE !
!       --------------------------------          --------

!     INPUT PARAMETERS
!     ----------------
!      NONE

!     ARGUMENTS IMPLICITES
!     --------------------
!       UNITE NULOUT
!       INDICATEURS LOGIQUES, COMMON /YOMLDDH/
!         DANS LE COMDECK * YOMMDDH * POUR LES MASQUES
!       DIMENSIONS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /DDHDIM/
!       ET DIMENSIONS POUR LES TABLEAUX DE CUMULS
!         DANS LE COMDECK * YOMTDDH * POUR LES TABLEAUX DE CUMULS
!       POINTEURS PROPRES AUX TABLEAUX DE CUMULS DDH, COMMON /POMTDDH/
!       DECLARATION DES TABLEAUX DE CUMULS

!     AUTRES ENTREES
!     --------------

!     OUTPUT
!     ------
!      INITIAL VALUES FOR EACH TIMESTEP OF ALL KPROC PLANS OF THE WORK ARRAYS
!      OF /YOMTDDH/

!     WRITTEN BY
!     ----------

!     MATS HAMRUD / PEDRO VITERBO         JAN 93
!        R. El Khatib : 01-08-07 Pruning options
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE YOMMDDH  , ONLY : NDHCS    ,NDHCV    ,&
 & NDHVV    ,NDHVS    ,NDHVFS  ,NDHIDH
USE YOMTDDH  , ONLY : HDCVB0   ,HDCVB1   ,HDCS0    ,HDCS1
USE YOMLDDH, ONLY : LFLEXDIA 
USE DDH_MIX, ONLY:NTOTFIELD,NTOTVAR,LALLOC,LMASKDDH,MK_MASK
USE OML_MOD, ONLY : OML_MAX_THREADS
IMPLICIT NONE

INTEGER(KIND=JPIM) :: IPLFIL, IPLVAR,ITHRMAX,JTHREAD,JDHI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!     0.- POINTEURS DE SEPARATION VARIABLES/TENDANCES-FLUX
!       DERNIER MOT CONTENANT UNE VARIABLE VERTICALE
IF (LHOOK) CALL DR_HOOK('ZERODDH',0,ZHOOK_HANDLE)

ITHRMAX=OML_MAX_THREADS()
IPLVAR = NDHVV*(NFLEVG+1)
!       DERNIER MOT CONTENANT UN CHAMP
IPLFIL = NDHCV*(NFLEVG+1)

!*
!     ------------------------------------------------------------------
!     2.- ZEROING OUT THE INSTANTANEOUS VALUES


IF (LFLEXDIA) THEN
  IPLVAR=(1+NFLEVG)*(NTOTFIELD-NDHVV)

IF (NTOTFIELD /=0) THEN

  IF (.NOT.ALLOCATED(LMASKDDH)) THEN
   ! creates mask for DDH
    CALL MK_MASK
   ! if NTOTFIELD greater than size oh hdcvb0/1 aborts
   ! in this case value of NDHCV in sunddh.F90 should
   ! be manually increased
   IF(NDHCV<NTOTFIELD) CALL ABOR1('ZERODDH')
  ENDIF

  IPLFIL=NTOTFIELD*(NFLEVG+1) 
  DO JTHREAD=1,ITHRMAX
     DO JDHI=1,NDHIDH
       WHERE (LMASKDDH(1:IPLFIL)) ! mask true if element is a variable
         !zeroing inst. values
         HDCVB1(1:IPLFIL,JDHI,JTHREAD)=0.0_JPRB
       ELSEWHERE
         !copying accumulated fluxes/tendencies
         HDCVB0(1:IPLFIL,JDHI,JTHREAD)=HDCVB1(1:IPLFIL,JDHI,JTHREAD)
       ENDWHERE
     ENDDO
  ENDDO

  ELSE
    HDCVB0(:,:,:)=0.0_JPRB
    HDCVB1(:,:,:)=0.0_JPRB
  ENDIF

ELSE

  !     2.- ZEROING OUT THE INSTANTANEOUS VALUES

  HDCVB1(1:IPLVAR,:,:) = 0.0_JPRB

  !     ------------------------------------------------------------------
  !     3.- COPYING THE ACCUMULATED FLUXES/TENDENCIES
  HDCVB0(IPLVAR+1:IPLFIL,:,:) = HDCVB1(IPLVAR+1:IPLFIL,:,:)

ENDIF

HDCS1(1:NDHVS+NDHVFS,:,:) = 0.0_JPRB

HDCS0(NDHVS+NDHVFS+1:NDHCS,:,:) = HDCS1(NDHVS+NDHVFS+1:NDHCS,:,:)

IF (LHOOK) CALL DR_HOOK('ZERODDH',1,ZHOOK_HANDLE)
END SUBROUTINE ZERODDH
