SUBROUTINE CPCLIMI(KFLD,KOD,PGPIN,PFPOUT)

!**** *CPCLIMI*  - STORES INPUT CLIMATOLOGY IN AN INTERPOLATION BUFFER.

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!       *CALL* *CPCLIMI*

!        EXPLICIT ARGUMENTS
!        --------------------


!        IMPLICIT ARGUMENTS
!        --------------------
!        See lower #include.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!         Called by SUFPWFPBUF.

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        K. YESSAD (MARCH 1997) after routine CPCLIMO.

!     MODIFICATIONS.
!     --------------
!        K.YESSAD : (AUGUST 1997):
!          - cleaning for cy18: corrections of bugs and false comments.
!        K. YESSAD (JAN 1998): merge of some FULL-POS and SL routines
!          relative to the halo.
!          - call to FPEXTPOL replaced by call to SLEXTPOL.
!          - call to FPREAD+FPWRIT replaced by call to SLCOMM.
!          - call to FPADD3P replaced by a simple memory transfer.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0, LLPOLE and code under LRPOLE, LLPOLE.
!          - removal of NDGLG, LRPOLE, LLPOLE in 'CALL FPSCAW'.
!          - removal of LRPOLE in 'CALL INIPEX'. 
!        G.Mozdzynski 02-10-01: support for radiation on-demand comms
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!   R. El Khatib  20-May-2005 NFPWIDE moved to YOMWFPDS
!   R. El Khatib  17-Oct-2008 bf for ALADIN fullpos with equal distribution
!                             of C+I+E (RDISTR_E=1.)
!   K. Yessad Dec 2008: merge SLCOMM+SLCOMM1 -> SLCOMM.
!   K. Yessad Dec 2008: merge the different (E)SLEXTPOL.. -> (E)SLEXTPOL.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LALLOPR  ,NPRINTLEV, NPROC   ,LELAM
USE YOMFPC   , ONLY : NFPDOM
USE YOMPFPB  , ONLY : NLENGFPB ,GFPBUF   ,NSTAGFPB
USE YOMFPG   , ONLY : LFPOSBUF ,NFPRGPL
USE PTRFP4   , ONLY : YRQGFP
USE TYPE_FPRQPHYS, ONLY : ALLOCATE_FPRQPHY,INQUIRE_FPRQPHY
USE YOMFPSC2 , ONLY : NFPROMA  ,NFPBLOCS ,NFPSTART ,NFPEND
USE YOMFPDIM , ONLY : NFPBFIN  ,NFPBFOUT
USE YOMMP    , ONLY : NAFPB1   ,NFPSTA   ,NFPONL   ,NFPOFF   ,NFRSTLOFF,&
 & NFPPROCS ,NFPRPT  ,NFPSPT   ,NFPSENDPOS,NFPRECVPOS,NFPSENDPTR,&
 & NFPRECVPTR,NFPCOMM ,MYFRSTACTLAT, MYLSTACTLAT,NFPCORE
USE YOMDIM   , ONLY : NPROMA   ,NDGLG    ,NDGSAG   ,NDGENG   ,NDGSAL   ,&
 & NDGENL
USE YOMGEM   , ONLY : NGPTOT   ,NLOENG   ,NGPTOT_CAP
USE YOMWFPDS , ONLY : NFPWIDE
USE YOMGC    , ONLY : RCORI    ,OROG     ,GM

!-----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)  :: KFLD
INTEGER(KIND=JPIM),INTENT(IN)  :: KOD (KFLD)
REAL(KIND=JPRB)   ,INTENT(OUT) :: PGPIN(NAFPB1,KFLD) 
REAL(KIND=JPRB),  INTENT(OUT)  :: PFPOUT(NFPRGPL,KFLD) 

!-----------------------------------------------------------------------------

CHARACTER (LEN = 9) ::  CLCONF

LOGICAL :: LLINC=.FALSE.
INTEGER(KIND=JPIM) :: JF, JD, JKGLO, ICEND, IBL, JROF, IOFFBL, IST, IEND
INTEGER(KIND=JPIM) :: IDUMARR(2)
INTEGER(KIND=JPIM) :: IGPCOMP

REAL(KIND=JPRB), ALLOCATABLE :: ZFPBUF1AUX(:,:), ZPARFP1(:)
REAL(KIND=JPRB) :: ZGFP(NFPROMA,KFLD)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------------

#include "fposhor.intfb.h"
#include "hpos.intfb.h"
#include "sc2rdgfp.intfb.h"
#include "slcomm.intfb.h"
#include "slextpol.intfb.h"
#include "eslextpol.intfb.h"
#include "sufpsc2b.intfb.h"

!-----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPCLIMI',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

!*       4.1 Setup request

WRITE(NULOUT,'('' CPCLIMI CALLED '')')


NFPBFIN =KFLD
NFPBFOUT=KFLD

! Changed NGPTOT into IGPCOMP in order to pass properly
! the scan of NFPCORE values for the LAM models
IF (LELAM.AND.LFPOSBUF) THEN
  IGPCOMP=MIN(NGPTOT,NGPTOT_CAP)
ELSE
  IGPCOMP=NGPTOT
ENDIF

CALL ALLOCATE_FPRQPHY(YRQGFP,NFPDOM,1,NFPBFOUT)
IF (NPRINTLEV >= 1.OR. LALLOPR) THEN
  CALL INQUIRE_FPRQPHY(YRQGFP,'YRQGFP',NULOUT)
ENDIF
DO JF=1,NFPBFOUT
  YRQGFP%ICOD(JF)=KOD(JF)
  YRQGFP%ILEV(JF)=1
  YRQGFP%IDOM(1,JF)=NFPDOM
  YRQGFP%IVEC(JF)=0
ENDDO
DO JF=1,NFPBFOUT
  DO JD=1,NFPDOM
    YRQGFP%IDMP(JD,1,JF)=JD
  ENDDO
ENDDO

CLCONF(1:9)='0000G0000'

!*       4.2 Get halo and interpolated/output field

PGPIN(:,:)=0._JPRB
! Halo
ALLOCATE(ZPARFP1(NFPBFIN))
ALLOCATE(ZFPBUF1AUX(NPROMA,NFPBFIN))
DO JKGLO=1,IGPCOMP,NPROMA
  ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBL=(JKGLO-1)/NPROMA+1
  CALL HPOS(CLCONF(5:5),ICEND,JKGLO,RCORI(JKGLO),GM(JKGLO),OROG(JKGLO),&
   & ZPARFP1,ZFPBUF1AUX)
  IF (NFPWIDE > 0) THEN
    DO JF=1,NFPBFIN
      DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IGPCOMP)
        PGPIN(NFPCORE(JROF),JF)=ZFPBUF1AUX(JROF-JKGLO+1,JF)
      ENDDO
    ENDDO
  ELSE
    IOFFBL=(IBL-1)*NPROMA
    DO JF=1,NFPBFIN
      DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IGPCOMP)
        PGPIN(IOFFBL+JROF-JKGLO+1,JF)=ZFPBUF1AUX(JROF-JKGLO+1,JF)
      ENDDO
    ENDDO
  ENDIF
ENDDO
DEALLOCATE(ZFPBUF1AUX)
IF (NFPWIDE > 0) THEN
  IF (NPROC > 1) THEN
    CALL SLCOMM(NAFPB1,NFPPROCS,&
     & NFPRPT,NFPSPT,NFPSENDPOS,NFPRECVPOS,NFPSENDPTR,&
     & NFPRECVPTR,NFPCOMM,&
     & IDUMARR,NFPBFIN,LLINC,0,PGPIN)
  ENDIF
  IF (LELAM) THEN
    CALL ESLEXTPOL(NFPWIDE,NFPWIDE,&
     & NAFPB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFPBFIN,NFPSTA,NFPONL,NFPOFF,&
     & NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,IDUMARR,1,&
     & PGPIN)
  ELSE
    CALL SLEXTPOL(NFPWIDE,NFPWIDE,&
     & NAFPB1,NDGSAG,NDGENG,NDGSAL,NDGENL,NFPBFIN,NFPSTA,NFPONL,NFPOFF,&
     & NDGLG,NLOENG,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,IDUMARR,1,&
     & ZPARFP1,PGPIN)
  ENDIF
ENDIF
DEALLOCATE(ZPARFP1)

! Interpolated or output field
ALLOCATE(NSTAGFPB(NFPBLOCS))
CALL SUFPSC2B(NFPBFOUT,NSTAGFPB,NLENGFPB,NFPROMA,NFPBLOCS)
ALLOCATE(GFPBUF(NLENGFPB))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990) 'NSTAGFPB ',SIZE(NSTAGFPB),SHAPE(NSTAGFPB)
  WRITE(NULOUT,9990) 'GFPBUF   ',SIZE(GFPBUF),SHAPE(GFPBUF)
ENDIF
CALL FPOSHOR(PGPIN,CLCONF(5:5))
DO JD=1,NFPBLOCS
  IST =NFPSTART(JD)
  IEND=NFPEND(JD)
  CALL SC2RDGFP(NFPROMA,NFPBFOUT,GFPBUF,NSTAGFPB(JD),IST,IEND,ZGFP)
  DO JF=1,NFPBFOUT
    DO JROF=IST,IEND
      PFPOUT((JD-1)*NFPROMA+JROF,JF)=ZGFP(JROF,JF)
    ENDDO
  ENDDO
ENDDO
DEALLOCATE(NSTAGFPB)
DEALLOCATE(GFPBUF)

! ---------------------------------------------------------------------
9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
9991 FORMAT(1X,'ARRAY ',A10,' DEALLOCATED ')

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPCLIMI',1,ZHOOK_HANDLE)
END SUBROUTINE CPCLIMI

