SUBROUTINE CPGRIDF(YD_PBIN,YD_PBOUT)

!**** *CPGRIDF*  - copy grid point fields

!     Purpose.
!     --------
!        To copy a grid point buffer/workfile on another buffer/workfile. 

!**   Interface.
!     ----------
!        *CALL* *CPGRIDF

!        Explicit arguments :      
!        --------------------

!             PBIN    : Buffer to copy from
!             PBOUT   : Buffer to copy to 

!        Implicit arguments :      
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original   : 95-07-20
!        L. Isaksen : 96-05-05  Adapted for Message Passing
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of code under LRPOLE.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT,NULERR
USE GRIDPOINT_BUFFERS_MIX , ONLY : gridpoint_buffer

IMPLICIT NONE

TYPE(GRIDPOINT_BUFFER),INTENT(IN)    :: YD_PBIN 
TYPE(GRIDPOINT_BUFFER),INTENT(INOUT) :: YD_PBOUT 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPGRIDF',0,ZHOOK_HANDLE)
IF( .NOT. ASSOCIATED (YD_PBIN%GPBUF)) THEN
  WRITE(NULERR,*)'CPGRIDF:GRID-POINT BUFFER NOT ALLOCATED ',YD_PBIN%CLNAME
  CALL ABOR1('CPGRIDF:GRID-POINT BUFFER NOT ALLOCATED')
ENDIF
IF( .NOT.YD_PBIN%LFILLED) THEN
  WRITE(NULERR,*)'CPGRIDF:GRID-POINT BUFFER NOT FILLED ',YD_PBIN%CLNAME
  CALL ABOR1('CPGRIDF:GRID-POINT BUFFER NOT FILLED')
ENDIF
IF (YD_PBIN%IFIELDS /= YD_PBOUT%IFIELDS) THEN
  WRITE (NULOUT,*) 'CPGRIDF: YD_PBIN%IFIELDS /= YD_PBOUT%IFIELDS : ',&
   & YD_PBIN%IFIELDS,YD_PBOUT%IFIELDS  
  CALL ABOR1 ('CPGRIDF HAS YD_PBIN%IFIELDS /= YD_PBOUT%IFIELDS')
ENDIF

YD_PBOUT%GPBUF(:) = YD_PBIN%GPBUF(:)
YD_PBOUT%LFILLED  = .TRUE.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPGRIDF',1,ZHOOK_HANDLE)
END SUBROUTINE CPGRIDF
