SUBROUTINE FPFILTER

!**** *FPFILTER*  - COMPUTE MATRIXES FOR POST-PROCESSING SPECTRAL FILTER

!     PURPOSE.
!     --------
!        To initialize the filtering matrixes for the post-processing of
!        derivatives

!**   INTERFACE.
!     ----------
!       *CALL* *FPFILTER*

!        EXPLICIT ARGUMENTS
!        --------------------
!           None.

!       IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        D. Giard : 94-12-22  gestion des fichiers de matrices
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El khatib 03-03-15 read/write filtering matrixes
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULDILA  ,NULCONT
USE YOMDIM   , ONLY : NSMAX    ,NCMAX    ,NUMP
USE YOMFPC   , ONLY : NFPDOM
USE PARFPOS  , ONLY : JPOSLEN
USE YOMFPF   , ONLY : NFMAX    ,RFPFIL   ,RFPMAT   ,LFPRDFIL ,LFPWRFIL ,&
           & NFPCMAX ,NFPGLFI  ,LFPBED   ,RFPBED   ,RFPLTF
USE YOMLFI   , ONLY : NIMES    ,NFACTD
USE YOMLAP   , ONLY : MYMS
USE YOMCT0   , ONLY : NPROC

IMPLICIT NONE

!     CLDICO  : name of a record on matrix LFI file
!     ZDICOLU : dilat./contract. matrix elements related to a zonal wave number
!     ZDICO   : same as ZDICOLU, overdimensioned up to the nearest odd number
!     ZSPECT  : vector in the spectral work sphere
!     ZSPECC  : vector in the spectral geographical sphere
!     ZFILTC  : filtered vector in the spectral geographical sphere

CHARACTER (LEN = 16) :: CLDICO
CHARACTER (LEN = 16) :: CLLEC(NFPDOM)
CHARACTER (LEN = 15+JPOSLEN) :: CLFPFN(NFPDOM)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPECT(:,:), ZSPECC(:,:), ZFILTC(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDICO(:,:), ZDICOLU(:),ZZ(:)

INTEGER(KIND=JPIM) :: ICMXSUR, ICRANGE, INAR, INARI, IND, IOS, IREP, &
 & ISRANGE, ITMXSUR, JC, JDOM, JF, IM, JMLOC, JN, JT, IBARP, ICMAX, &
 & IFPCT0, II, IDIM, ISMAX, IPMAX, IERR, ISHAPE, ISRANGE2
INTEGER(KIND=JPIM) :: IULFP(NFPDOM), IOFF(NUMP+1), IDIMAT

LOGICAL :: LLEX, LLOP, LLOPEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZHEADER(5), ZSCAL

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "ini3wrfp.intfb.h"
#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*       1.    COMPUTE FILTERING MATRIXES
!              --------------------------

!*    1.1   Allocate matrixes arrays

IF (LHOOK) CALL DR_HOOK('FPFILTER',0,ZHOOK_HANDLE)

IF (LFPWRFIL .AND. NPROC > 1) THEN
  CALL ABOR1('FPFILTER: LFPWRFIL NOT READY FOR NPROC>1')
ENDIF

IREP=0
INARI=0
INAR=NSMAX+1

ITMXSUR=2*((NSMAX+1)/2)+1

IOFF(1)=0
DO JMLOC=1,NUMP
  IM=MYMS(JMLOC)
  IOFF(JMLOC+1)=IOFF(JMLOC)+(NSMAX+1-IM)**2
ENDDO

IDIMAT=2*(IOFF(NUMP+1)/2)+1
ALLOCATE(RFPMAT(IDIMAT,NFPDOM))
WRITE(NULOUT,9990) 'RFPMAT   ',SIZE(RFPMAT),SHAPE(RFPMAT)
9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!*    1.2.1   Open matrixes files and compute dimensions if filters not read

IF (.NOT.LFPRDFIL) THEN
  INQUIRE(FILE='MATDILA',IOSTAT=IOS,EXIST=LLEX,OPENED=LLOP)
  LLOPEN= IOS == 0.AND. LLEX .AND. LLOP
  IF (.NOT.LLOPEN) THEN
    CALL LFIAFM(IREP,NULDILA,NFACTD)
    CALL LFIOUV(IREP,NULDILA,.TRUE.,'MATDILA','OLD',.TRUE.,.FALSE.,NIMES,INAR,&
     & INARI)
    CALL LFIAFM(IREP,NULCONT,NFACTD)
    CALL LFIOUV(IREP,NULCONT,.TRUE.,'MATCONT','OLD',.TRUE.,.FALSE.,NIMES,INAR,&
     & INARI)
  ELSE
    CALL LFIPOS(IREP,NULDILA)
    CALL LFIPOS(IREP,NULCONT)
  ENDIF
  ICMXSUR=2*((NCMAX+1)/2)+1
  ALLOCATE(ZSPECC(ICMXSUR,NSMAX+1))
  ALLOCATE(ZFILTC(ICMXSUR,NSMAX+1))
  ALLOCATE(ZSPECT(ITMXSUR,NSMAX+1))  
  ALLOCATE(ZDICO(ICMXSUR,ITMXSUR))
  ALLOCATE(ZDICOLU((NSMAX+1)*(NCMAX+1)))
ENDIF

!     1.2.2 Open filtering matrixes if read or write

IF (LFPRDFIL.OR.LFPWRFIL) THEN
  IFPCT0=-9
  CALL INI3WRFP(CLFPFN,CLLEC,IULFP,IBARP,IFPCT0)
  IDIM=SIZE(ZHEADER)
  WRITE (CLDICO,'(A)') 'HEADER'
  IERR=0
  DO JDOM=1,NFPDOM
    IF (NFMAX(JDOM) < NFPCMAX) THEN
      CLFPFN(JDOM)(1:11)='matrix.fil.'
      CALL LFIAFM(IREP,IULFP(JDOM),NFPGLFI)
      IF (LFPRDFIL) THEN
        CALL LFIOUV(IREP,IULFP(JDOM),.TRUE.,CLFPFN(JDOM),'OLD',.TRUE.,.FALSE.,&
         & NIMES,INAR,INARI)
      ELSE
        CALL LFIOUV(IREP,IULFP(JDOM),.TRUE.,CLFPFN(JDOM),'NEW',.TRUE.,.FALSE.,&
         & NIMES,INAR,INARI)
      ENDIF
      IF (LFPRDFIL) THEN
        CALL LFILEC(IREP,IULFP(JDOM),CLDICO,ZHEADER,IDIM)
        ISMAX=INT(ZHEADER(1),KIND=JPIM)
        IPMAX=INT(ZHEADER(2),KIND=JPIM)
        ICMAX=INT(ZHEADER(3),KIND=JPIM)
        ISHAPE=INT(ZHEADER(4),KIND=JPIM)
        IF (ISHAPE == 0) THEN
          WRITE(NULOUT,'(''READING FILTERING MATRIX '',A,'' ISMAX = '',I4,  &
           & '' IPMAX = '',I4,'' ICMAX = '',I4,'' SHAPE = THX  ZFPBED = '', &
           & E14.7)') CLFPFN(JDOM),ISMAX,IPMAX,ICMAX,ZHEADER(5)
        ELSE
          WRITE(NULOUT,'(''READING FILTERING MATRIX '',A,'' ISMAX = '',I4,  &
           & '' IPMAX = '',I4,'' ICMAX = '',I4,'' SHAPE = GAUSSIAN  ZFPLTF = '', &
           & E14.7)') CLFPFN(JDOM),ISMAX,IPMAX,ICMAX,ZHEADER(5)
        ENDIF
        IF (ICMAX < NFPCMAX) THEN
          IERR=1
          WRITE(NULOUT,*) 'FILTERING MATRIX ERROR : ICMAX < NFPCMAX !'
        ENDIF
        IF (ISMAX /= NSMAX) THEN
          IERR=1
          WRITE(NULOUT,*) 'FILTERING MATRIX ERROR : ISMAX /= NSMAX !'
        ENDIF
        IF (IPMAX /= NFMAX(JDOM)) THEN
          IERR=1
          WRITE(NULOUT,*) 'FILTERING MATRIX ERROR : IPMAX /= NFMAX !'
        ENDIF
        IF (ISHAPE /= 0 .AND. LFPBED) THEN
          IERR=1
          WRITE(NULOUT,*) 'FILTERING MATRIX ERROR : FILTER SHAPE DOES NOT CONFORM LFPBED !'
        ELSE
          ZSCAL=1000._JPRB
          IF (LFPBED) THEN
            IF (ABS(RFPBED-ZHEADER(5)) > ZSCAL*SPACING(RFPBED)) THEN
              IERR=1
              WRITE(NULOUT,*) 'FILTERING MATRIX ERROR : ZFPBED /= RFPBED !'
            ENDIF
          ELSE
            IF (ABS(RFPLTF-ZHEADER(5)) > ZSCAL*SPACING(RFPLTF)) THEN
              IERR=1
              WRITE(NULOUT,*) 'FILTERING MATRIX ERROR : ZFPLTF /= RFPLTF !'
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      IF (LFPWRFIL) THEN
        ZHEADER(1)=REAL(NSMAX,KIND=JPRB) 
        ZHEADER(2)=REAL(NFMAX(JDOM),KIND=JPRB) 
        ZHEADER(3)=REAL(NCMAX,KIND=JPRB)
        IF (LFPBED) THEN
          ZHEADER(4)=0._JPRB
          ZHEADER(5)=RFPBED
          WRITE(NULOUT,'(''WRITING FILTERING MATRIX '',A,'' NSMAX = '',I4,  &
           & '' NFMAX = '',I4,'' NCMAX = '',I4,'' SHAPE = THX  RFPBED = '', &
           & E14.7)') CLFPFN(JDOM),NSMAX,NFMAX(JDOM),NCMAX,RFPBED
        ELSE
          ZHEADER(4)=1._JPRB
          ZHEADER(5)=RFPLTF
          WRITE(NULOUT,'(''WRITING FILTERING MATRIX '',A,'' NSMAX = '',I4,  &
           & '' NFMAX = '',I4,'' NCMAX = '',I4,'' SHAPE = GAUSSIAN  RFPLTF = '', &
           & E14.7)') CLFPFN(JDOM),NSMAX,NFMAX(JDOM),NCMAX,RFPLTF
        ENDIF
        CALL LFIECR(IREP,IULFP(JDOM),CLDICO,ZHEADER,IDIM)
      ENDIF
    ENDIF
  ENDDO
  IF (IERR == 1) CALL ABOR1('FPFILTER:FILTERING MATRIX ERROR')
  ALLOCATE(ZZ(ITMXSUR*(NSMAX+1)))
ENDIF


!*    1.3   Compute filtering matrixes for each zonal wavenumber

DO JMLOC=1,NUMP

  IM=MYMS(JMLOC)

  ISRANGE=NSMAX+1-IM
  ISRANGE2=ISRANGE**2
  IF (IM == 0) WRITE (CLDICO,'(A,I5.4)') 'WNUM 0000'
  IF (IM > 0) WRITE (CLDICO,'(A,I5.4)') 'WNUM',IM

  IF (.NOT.LFPRDFIL) THEN

    ICRANGE=NCMAX+1-IM

!*    1.3.1   Canonical injection into the geographical sphere

    DO JF=1,ISRANGE
      DO JN=IM,NSMAX
        ZSPECT(JN+1-IM,JF)=0.0_JPRB
      ENDDO
      ZSPECT(JF,JF)=1.0_JPRB
    ENDDO

    CALL LFILEC(IREP,NULDILA,CLDICO,ZDICOLU,ISRANGE*ICRANGE)

    IND=-ICRANGE
    DO JT=1,ISRANGE
      IND=IND+ICRANGE
      DO JC=1,ICRANGE
        ZDICO(JC,JT)=ZDICOLU(IND+JC)
      ENDDO
    ENDDO

    CALL MXMAOP(ZDICO,1,ICMXSUR,ZSPECT,1,ITMXSUR,ZSPECC,1,ICMXSUR,ICRANGE,&
     & ISRANGE,ISRANGE)  

!*    1.3.2   Computation of filtering matrix for each subdomain

    CALL LFILEC(IREP,NULCONT,CLDICO,ZDICOLU,ISRANGE*ICRANGE)

    IND=-ICRANGE
    DO JT=1,ISRANGE
      IND=IND+ICRANGE
      DO JC=1,ICRANGE
        ZDICO(JC,JT)=ZDICOLU(IND+JC)
      ENDDO
    ENDDO

  ENDIF

  DO JDOM=1,NFPDOM
    IF (NFMAX(JDOM) < NFPCMAX) THEN

      IF (LFPRDFIL) THEN
        CALL LFILEC(IREP,IULFP(JDOM),CLDICO,ZZ,ISRANGE2)
      ELSE
        DO JF=1,ISRANGE
          DO JN=IM,NCMAX
            ZFILTC(JN+1-IM,JF)=ZSPECC(JN+1-IM,JF)*RFPFIL(JN,JDOM)
          ENDDO
        ENDDO
        CALL MXMAOP(ZDICO,ICMXSUR,1,ZFILTC,1,ICMXSUR,ZSPECT,1,ITMXSUR,ISRANGE,&
         & ICRANGE,ISRANGE)
      ENDIF

      II=0
      DO JF=1,ISRANGE
        DO JN=1,ISRANGE
          II=II+1
          IF (LFPRDFIL) THEN
            RFPMAT(IOFF(JMLOC)+II,JDOM)=ZZ(II)
          ELSE
            RFPMAT(IOFF(JMLOC)+II,JDOM)=MIN(1._JPRB,MAX(-1._JPRB,ZSPECT(JN,JF)))
            IF (LFPWRFIL) THEN
              ZZ(II)=RFPMAT(IOFF(JMLOC)+II,JDOM)
            ENDIF
          ENDIF
        ENDDO
      ENDDO

      IF (LFPWRFIL.AND..NOT.LFPRDFIL) THEN
        CALL LFIECR(IREP,IULFP(JDOM),CLDICO,ZZ,ISRANGE2)
      ENDIF

    ENDIF
  ENDDO

ENDDO

IF (.NOT.LFPRDFIL) THEN
  DEALLOCATE(ZSPECC)
  DEALLOCATE(ZFILTC)
  DEALLOCATE(ZSPECT)
  DEALLOCATE(ZDICO)
  DEALLOCATE(ZDICOLU)
  IF (.NOT.LLOPEN) THEN
    CALL LFIFER(IREP,NULDILA,'UNKNOWN')
    CALL LFIFER(IREP,NULCONT,'UNKNOWN')
  ENDIF
ENDIF

!     Close filtering matrixes

IF (LFPRDFIL.OR.LFPWRFIL) THEN
  DEALLOCATE(ZZ)
  DO JDOM=1,NFPDOM
    IF (NFMAX(JDOM) < NFPCMAX) THEN
      CALL LFIFER(IREP,IULFP(JDOM),'KEEP')
    ENDIF 
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('FPFILTER',1,ZHOOK_HANDLE)
END SUBROUTINE FPFILTER
