SUBROUTINE FPLAKE(KBLOC,PCLI,PROW)

!**** *FPLAKE*  - FULL-POS corrector for lakes & islands : overwrite with
!                 climatology data

!     PURPOSE.
!     --------
!        To correct physical fields after the horizontal interpolations so
!        that lakes & islands points are overwritten with climatology data

!**   INTERFACE.
!     ----------
!       *CALL* *FPLAKE*

!        EXPLICIT ARGUMENTS
!        --------------------

!         KBLOC  : bloc number
!         PCLI   : climatology field to use
!         PROW  : output row for physics

!        IMPLICIT ARGUMENTS
!        ------------------
!          see modules.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!       none.

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        JAOUAD BOUTAHAR *MAROC-METEO* & RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2000-03-01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMWFPB  , ONLY : MWIC
USE YOMFPSC2 , ONLY : NFPROMA  ,NFPSTART ,NFPEND   ,NFPBLOFF

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOC
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLI(NFPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PROW(NFPROMA) 

INTEGER(KIND=JPIM) :: JI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1. CORRECTION FOR LAKE & ISLANDS
!           -----------------------------

IF (LHOOK) CALL DR_HOOK('FPLAKE',0,ZHOOK_HANDLE)

DO JI=NFPSTART(KBLOC),NFPEND(KBLOC)
  IF (MWIC(NFPBLOFF(KBLOC)+JI) /= 0) PROW(JI)=PCLI(JI)
ENDDO

IF (LHOOK) CALL DR_HOOK('FPLAKE',1,ZHOOK_HANDLE)

END SUBROUTINE FPLAKE
