SUBROUTINE FPMODXFU

!**** *FPMODXFU*  - Preliminary modification of instantaneous fluxes 
!                   for Post-Processing
!     PURPOSE.
!     --------
!        To prepare input fields before horizontal interpolations

!**   INTERFACE.
!     ----------
!       *CALL* *FPMODXFU*

!        EXPLICIT ARGUMENTS
!        --------------------
!           None

!        IMPLICIT ARGUMENTS
!        --------------------
!        NONE.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-08-06
!        R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Seity      15-Jan-2007 add Graupel and Hail
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMAFN   , ONLY : GFP_XLSP ,GFP_XCP  ,GFP_XLSS ,GFP_XCSF,&
                    &  GFP_XLSG ,GFP_XCSG ,GFP_XLSH ,GFP_XCSH
USE YOMFPC   , ONLY : C1FPXFU
USE YOMXFU   , ONLY : NFDXFU
USE YOMXFUB  , ONLY : XFUBUF
USE PTRXFU   , ONLY : MXPLCL   ,MXPLSL   ,MXPLCN   ,MXPLSN,&
                     &  MXPLCG   ,MXPLSG   ,MXPLCH   ,MXPLSH    

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IFLDPTR(NFDXFU)

INTEGER(KIND=JPIM) :: IFLD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fpmodprec.intfb.h"

!*    1. FIND FIELDS TO MODIFY 
!        ---------------------

IF (LHOOK) CALL DR_HOOK('FPMODXFU',0,ZHOOK_HANDLE)
IFLD=0
IF (INDEX(C1FPXFU,GFP_XLSP%CLNAME) /= 0) THEN   ! Large scale rain
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLSL
ENDIF
IF (INDEX(C1FPXFU,GFP_XCP%CLNAME) /= 0) THEN    ! Convective rain
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLCL
ENDIF
IF (INDEX(C1FPXFU,GFP_XLSS%CLNAME) /= 0) THEN   ! Large Scale Snow fall
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLSN
ENDIF
IF (INDEX(C1FPXFU,GFP_XCSF%CLNAME) /= 0) THEN   ! Convective  Snow fall
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLCN
ENDIF
IF (INDEX(C1FPXFU,GFP_XLSG%CLNAME) /= 0) THEN   ! Large Scale Graupel fall
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLSG
ENDIF
IF (INDEX(C1FPXFU,GFP_XCSG%CLNAME) /= 0) THEN   ! Convective  Graupel fall
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLCG
ENDIF
IF (INDEX(C1FPXFU,GFP_XLSH%CLNAME) /= 0) THEN   ! Large Scale Hail fall
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLSH
ENDIF
IF (INDEX(C1FPXFU,GFP_XCSH%CLNAME) /= 0) THEN   ! Convective  Hail fall
  IFLD=IFLD+1
  IFLDPTR(IFLD)=MXPLCH
ENDIF

!*    2. MODIFY FIELDS 
!        -------------

IF (IFLD > 0) THEN
  CALL FPMODPREC(IFLD,XFUBUF,IFLDPTR)
ENDIF

IF (LHOOK) CALL DR_HOOK('FPMODXFU',1,ZHOOK_HANDLE)
END SUBROUTINE FPMODXFU
