SUBROUTINE FPPS(KPROMA,KST,KND,KOPLEV,POROG,PGEOP,PT,PQ,PI,PL,&
 & PR,PS,PG,PSP,PST,PSPPP,PRESH,PLNPR,PALPH)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMGEM   , ONLY : VBH      ,VAH      ,VC       ,VDELB
USE YOMSTA   , ONLY : RDTDZ1
USE YOMCST   , ONLY : RG

!**** *FPPS*  - FULL-POS pressure post-processing

!     PURPOSE.
!     --------
!        To compute the pressures of a set of surfaces, given their 
!        geopotentials

!**   INTERFACE.
!     ----------
!       *CALL* *FPPS*

!        EXPLICIT ARGUMENTS
!        --------------------

!        * INPUT:
!        KPROMA  : horizontal dimension
!        KST     : start of work
!        KND     : depth of work
!        KOPLEV  : number of output surfaces
!        POROG   : input surface geopotential
!        PGEOP   : output geopotential
!        PT      : input temperature
!        PQ      : input specific moisture
!        PI      : input ice
!        PL      : input liquid water
!        PR      : input rain
!        PS      : input snow
!        PG      : input graupel
!        PSP     : input surface pressure
!        PST     : input surface temperature

!        * OUTPUT:
!        PSPPP   : output pressures

!        * OPTIONAL INPUT:
!        PRESH   : half level hydrostatic pressure
!        PLNPR   : "delta" (see in GPXYB)
!        PALPH   : "alpha" (see in GPXYB)

!        IMPLICIT ARGUMENTS
!        ------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!   ORIGINAL   : 00-08-10 after VEINE
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOP(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PS(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PG(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PST(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPPP(KPROMA,KOPLEV) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRESH(KPROMA,0:NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PLNPR(KPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PALPH(KPROMA,NFLEVG)

!---------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZPRESH(KPROMA,0:NFLEVG) ! Half levels input presures
REAL(KIND=JPRB) :: ZGEOPH(KPROMA,0:NFLEVG) ! Half levels input geopotentials
REAL(KIND=JPRB) :: ZR(KPROMA,NFLEVG)       ! Full levels R
REAL(KIND=JPRB) :: ZMDPHIS(KPROMA)         ! Difference (input PHIs - output PHI)
REAL(KIND=JPRB) :: ZPBLO(KPROMA)           ! output Ps for (output orog < input orog)
REAL(KIND=JPRB) :: ZPABV(KPROMA)           ! output Ps for (output orog > input orog)
REAL(KIND=JPRB) :: ZA(KPROMA)              ! =1 if (output orog < input orog) ; else =0
REAL(KIND=JPRB) :: ZTSC(KPROMA)            ! standart temperature for the output surface
REAL(KIND=JPRB) :: ZDTDPHI                 ! standart gradient dT/(g*dZ)

REAL(KIND=JPRB) :: ZDELP(KPROMA,NFLEVG), ZRDELP(KPROMA,NFLEVG), ZLNPR(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRTGR(KPROMA,NFLEVG), ZRPRE (KPROMA,NFLEVG), ZALPH(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRPP (KPROMA,NFLEVG), ZGEOPF(KPROMA,NFLEVG), ZKAP (KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZCP  (KPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: IL(KPROMA) ! index of first input half level below output surface
INTEGER(KIND=JPIM) :: JLP, JLEV, JI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!---------------------------------------------------------------------------

#include "gpgeo.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "pppmer.intfb.h"

!---------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPPS',0,ZHOOK_HANDLE)

!---------------------------------------------------------------------------

!*       1.    COMPUTATION OF MODEL Cp, R AND R/Cp FROM Q
!              ------------------------------------------

CALL GPRCP(KPROMA,KST,KND,NFLEVG,PQ=PQ,PQI=PI,PQL=PL,PQR=PR,PQS=PS,PQG=PG, &
 & PCP=ZCP,PR=ZR,PKAP=ZKAP)

!*       2.    COMPUTATION OF MODEL LEVELS PRESSURES & GEOPOTENTIALS
!              -----------------------------------------------------

IF (PRESENT(PRESH).AND.PRESENT(PLNPR).AND.PRESENT(PALPH)) THEN
  ZPRESH(KST:KND,0:NFLEVG)=PRESH(KST:KND,0:NFLEVG)
  ZLNPR(KST:KND,1:NFLEVG)=PLNPR(KST:KND,1:NFLEVG)
  ZALPH(KST:KND,1:NFLEVG)=PALPH(KST:KND,1:NFLEVG)
ELSE
  ZPRESH(KST:KND,NFLEVG)=PSP(KST:KND)
  CALL GPPREH(KPROMA,KST,KND,NFLEVG,VAH,VBH,ZPRESH)
  CALL GPXYB(KPROMA,KST,KND,NFLEVG,VDELB,VC,ZPRESH,ZDELP,ZRDELP,ZLNPR,ZALPH,&
   & ZRTGR,ZRPRE,ZRPP)  
ENDIF
ZGEOPH(KST:KND,NFLEVG)=POROG(KST:KND)
CALL GPGEO(KPROMA,KST,KND,NFLEVG,ZGEOPH,ZGEOPF,PT,ZR,ZLNPR,ZALPH)

!*       3.    COMPUTATION OF SURFACE PRESSURE
!              -------------------------------

ZDTDPHI=RDTDZ1/RG

DO JLP=1,KOPLEV

!*       3.1   Compare surfaces

  ZMDPHIS(KST:KND)=POROG(KST:KND)-PGEOP(KST:KND,JLP)
  ZA(KST:KND)=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZMDPHIS(KST:KND)))

!*       3.2   Case (output orog < input orog) : use msl pressure calculation

  IF (MAXVAL(ZA(KST:KND)) > 0.0_JPRB) THEN

    ZTSC(KST:KND)=PST(KST:KND)-ZDTDPHI*ZMDPHIS(KST:KND)
    CALL PPPMER(KPROMA,KST,KND,PSP,ZMDPHIS,PST,ZTSC,ZPBLO)

  ELSE

    ZPBLO(KST:KND)=0.0_JPRB

  ENDIF

!*       3.3   Case (output orog > input orog) : P=Po.exp((Phi-Phio)/RT) 

  IF (MINVAL(ZA(KST:KND)) < 1.0_JPRB) THEN

!*       3.3.1  Find nearest model half level below output surface 

    IL(KST:KND)=NFLEVG
    DO JLEV = NFLEVG, 1, -1
      DO JI = KST,KND
        IL(JI) = IL(JI) + &
         & MAX(0.0_JPRB,SIGN(1.0_JPRB,PGEOP(JI,JLP)-ZGEOPH(JI,JLEV)))*&
         & (JLEV-IL(JI))  
      ENDDO
    ENDDO

!*       3.3.2 Compute pressure

    DO JI = KST,KND
      ZPABV(JI) = ZPRESH(JI,IL(JI))*&
       & EXP((ZGEOPH(JI,IL(JI))-PGEOP(JI,JLP))/(ZR(JI,IL(JI))*PT(JI,IL(JI))))  
    ENDDO

  ELSE

    ZPABV(KST:KND)=0.0_JPRB

  ENDIF

!*       3.4   Combination 

  PSPPP(KST:KND,JLP)=ZA(KST:KND)*ZPBLO(KST:KND) &
   & +(1.0_JPRB-ZA(KST:KND))*ZPABV(KST:KND)  

ENDDO

!-------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPPS',1,ZHOOK_HANDLE)
END SUBROUTINE FPPS
