!ocl list_copy(32,NFPOFF)
!ocl list_copy(32,NFPEXT)
SUBROUTINE FPSCAW(KLEN,KSTART,KEND,KL0,KBLOCK)

!**** *FPSCAW  -  FULL-POS:
!                 Numbering of the points (I is the interpolation point):
!                     13       5       6      14

!                      7       1       2       8
!                                 (I)
!                      9       3       4      10

!                     15      11      12      16

!                 Once known the coordinates 
!                 (DM-local ilon, DM-local ilat) of the points numbered 
!                 13, 7, 9, 15 (respectively (MLONN,MLAT-1), (MLON,MLAT),
!                 (MLOS,MLAT+1), (MLOSS,MLAT+2)), one computes the position
!                 irof=KL0(ilen,inumlat) in the interpolation buffer
!                 dimensioned with NPROMA. KL0(.,inumlat =1,2,3,4) correspond 
!                 respectively with (MLONN,MLAT-1), (MLON,MLAT),
!                 (MLOS,MLAT+1), (MLOSS,MLAT+2), 
!                 i.e. resp. points numbered 13, 7, 9
!                 and 15 (for consistency with routine LASCAW).

!                 This routine has been written after LASCAW.
!                 SUHOW1+SUHOW2+FPSCAW for FULL-POS plays the same part as
!                 LASCAW for semi-Lagrangian or observation interpolations.

!                 Computations are DM-local if distributed memory.

!**   Interface.
!     ----------
!        *CALL* *FPSCAW(...)

!        Explicit arguments :
!        --------------------
!        All the other following quantities involved in DM computations 
!         are DM-local.

!        INPUT:
!          KLEN       - horizontal dimension
!          KSTART     - first element of arrays where computations are 
!                       performed.
!          KEND       - depth of work.
!          KBLOCK      - pp packet pointer

!        OUTPUT:
!          KL0        - index of the four western points
!                       of the 16 points interpolation grid.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        No external.

!     Reference.
!     ----------
!        Documentation about FULL-POS.

!     Author.
!     -------
!        K. YESSAD (MARCH 1997).

!     Modifications.
!     --------------
!        K.YESSAD : (AUGUST 1997):
!          - cleaning for cy18: corrections of false comments.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         The consequence is that the two dummy arguments LDPOLMODEL,LDPOLFPOS
!         disappear (always false), dummy argument KDGLG becomes useless and
!         disappears.
!        Modified : 2000-03-01 by  Jaouad Boutahar (bilinear interpolation)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib 28-Jul-2006 Porting to NEC
!        K. Yessad: 28-02-2007 DM-features optimisations in FULL-POS
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP     ,ONLY : NFRSTLOFF,NFPEXT   ,NFPOFF
USE YOMDIM    ,ONLY : NDGUNG   ,NDGUXG   ,NDLUNG   ,NDLUXG
USE YOMWFPB   ,ONLY : MLAT     ,MLON     ,MLOS     ,MLONN    ,MLOSS
USE YOMWFPDS  ,ONLY : NFPWIDE
USE YOMFPSC2  ,ONLY : NFPBLOFF_DEP

!     ----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KL0(KLEN,NFPWIDE+1) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOCK

!     ----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ILA0, ILA1, ILA1G, ILA2, ILA2G, ILA3, ILA3G, ILAG, ILO
INTEGER(KIND=JPIM) :: ILO1, ILO2, ILO3, JLEN,ILOC

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ----------------------------------------------------------------------

!CDIR DUPLICATE(NFPOFF,32)
!CDIR DUPLICATE(NFPEXT,32)

!     ----------------------------------------------------------------------

#include "abor1.intfb.h"

!     ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPSCAW',0,ZHOOK_HANDLE)

!     ----------------------------------------------------------------------

!*       2.    COMPUTE ADRESSES
!              ----------------

IF (NFPWIDE  == 0) THEN

  !      2.1  Pointer for  buffer transfer

  DO JLEN=KSTART,KEND

    ! * Block offset :
    KL0(JLEN,1)=NFPBLOFF_DEP(KBLOCK) + JLEN

  ENDDO

ELSEIF (NFPWIDE  == 3) THEN

  !      2.2  Pointer for 12 points interpolations and bilinear interpolation.

  DO JLEN=KSTART,KEND

    ! * Block offset :
    ILOC=NFPBLOFF_DEP(KBLOCK) + JLEN

    ! * Latitude of point 1:
    ILA1G=MLAT(ILOC)+NFRSTLOFF
    ILA1=MIN(MAX(ILA1G,NDGUNG),NDGUXG)-NFRSTLOFF

    ! * Latitude of point 3:
    ILA2G =ILA1G+1
    ILA2=MIN(MAX(ILA2G,NDGUNG),NDGUXG)-NFRSTLOFF

    ! * Latitude of point 5:
    ILAG  =ILA1G-1
    ILA0=MIN(MAX(ILAG,NDGUNG),NDGUXG)-NFRSTLOFF

    ! * Latitude of point 11:
    ILA3G =ILA2G+1
    ILA3=MIN(MAX(ILA3G,NDGUNG),NDGUXG)-NFRSTLOFF

    ILO =MIN(MAX(MLONN(ILOC),NDLUNG-1),NDLUXG-1)
    ILO1=MIN(MAX(MLON (ILOC),NDLUNG-1),NDLUXG-1)
    ILO2=MIN(MAX(MLOS (ILOC),NDLUNG-1),NDLUXG-1)
    ILO3=MIN(MAX(MLOSS(ILOC),NDLUNG-1),NDLUXG-1)

    KL0(JLEN,1)=NFPOFF(ILA0)+NFPEXT(ILO ,ILA0)
    KL0(JLEN,2)=NFPOFF(ILA1)+NFPEXT(ILO1,ILA1)
    KL0(JLEN,3)=NFPOFF(ILA2)+NFPEXT(ILO2,ILA2)
    KL0(JLEN,4)=NFPOFF(ILA3)+NFPEXT(ILO3,ILA3)

  ENDDO

ELSE

  CALL ABOR1('FPSCAW : INTERNAL ERROR ON NFPWIDE')

ENDIF

!     ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPSCAW',1,ZHOOK_HANDLE)

END SUBROUTINE FPSCAW
