SUBROUTINE HPOS(CDCONF,KNUMB,KSTGLO,PRCORI,PGM,POROG,PARFP1,PFPBUF1)

!**** *HPOS*  - HORIZONTAL POST-PROCESSING

!     PURPOSE.
!     --------
!        FILL THE "SEMI-LAGRANGIAN" BUFFERS WITH THE FIELDS TO TRANSFER
!        OR INTERPOLATE

!**   INTERFACE.
!     ----------
!       *CALL* *HPOS*

!        EXPLICIT ARGUMENTS
!        --------------------
!         INPUT:
!          CDCONF : configuration of work
!          KNUMB  : number of elements of arrays for which computations
!                   are performed (used in message passing version)
!          KSTGLO : global offset (used in message passing version)
!          PRCORI : Coriolis parameter
!          PGM    : mapping factor
!          POROG  : orography.

!         OUTPUT:
!          PARFP1 : parity for interpolation buffer PFPBUF1.
!                   1=scalar; -1=vector.
!          PFPBUF1: interpolation buffer containing the fields to interpolate

!        IMPLICIT ARGUMENTS
!        --------------------
!        se #include below.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!     SC2RDG,CVLANISO,ACSOLW

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 95-11-22 R. El Khatib (no poles rows)
!        Modified : 96-02-07 R. El Khatib (use NLOSP )
!        Modified : 96-02-07 R. El Khatib (interpolate Hollman wind times
!                   map factor).
!        Modified FEB 1997 by K. YESSAD:
!         - PFPBUF1 and PFPBUF2 dimensioned with NPROMA, remove KPROMH.
!         - parallelisation of FULL-POS.
!        Modified OCT 1997 by K. YESSAD: correction of false comments
!           and bad phasing features.
!        R. El Khatib : 96-02-07 interpolate Hollman wind times map factor
!        Modified : 97-07-29 Cristina Madeira (ISBA Fields in FULL-POS)
!        Modified : 97-10-01 by Adam Dziedzic (ISBA schema Douville Fields)
!        Modified : 97-12-16 by Dominique Giard (frozen deep soil water)
!        R. El Khatib 98-03-12 interpolated orography in buffer 2
!        Modified : 98-02-04 by K. YESSAD: IB1 and IB2 always = 1 for DM,
!           due to removal of FPADD3P.
!        Modified : 98-03-12 by R. El Khatib (cleanup adiabatic pp.)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - removal of LRPOLE in YOMCT0 and in 'CALL INIPEX'.
!        Modified : 98-09-10 by R. El Khatib CVLANISO ; full case LFPOSHOR=.F.
!        Modified : 99-03-15 by D. Giard (Wdf,Hu,Wsf,ACSOLW)
!        Modified : 99-03-26 by J. Teixeira/P. Viterbo: Surface tiling
!        Modified : 99-04-19 by J.M. Piriou (Moon radiation).
!    R. El Khatib : 99-10-25 More flexibility for Ps/Log(Ps)
!    R. El Khatib : 00-03-22 Support for new phys. fields
!        Modified : 00-10-25 by D. Giard (als,alv)
!    R. El Khatib : 00-07-24 debug Tx - Tn
!    R. El Khatib : 00-07-28 Buffer 2 only if fields exists inside
!    R. El Khatib : 00-08-07 conf 'G' for auxilary surface fields
!    R. El Khatib : 01-04-03 Derived type  - cleanings
!    R. El Khatib : 01-05-04 Duration of total precipitations,HUn,HUx
!    R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    D.Dent       : 02-05-15 Improve handling of 2D extra fields
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!    JJMorcrette  : 02-11-04 PAR, UV-B, CAPE
!    M.Hamrud     : 01-Oct-2003 CY28 Cleaning
!    F.Taillefer  : 23-Jun-2004 Add O3 and aerosols fields
!    Y.Bouteloup  : 2004-03-26 introduction of MTS radiance
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!    A. Tompkins  : 11-02-04 TCLW,TCIW
!        M.Hamrud      01-Jul-2006 Revised surface fields
!    S.Serrar     : 26-09-2005  total column for CO2 and SF6
!    A. Dethof    : 15-Dec_05 Add total column GRG fields
!    S. Serrar    : 07-Sep-06 include tracers for diagnostics (GEMS)
!    R. Engelen   : 02-Jun_06 CO2 and SF6 replaced by generic GHG
!    JJMorcrette 20060721 PP of clear-sky PAR and TOA solar incident radiation
!    JJMorcrette 20060807 PP of VIMD
!    JJMorcrette  : 20060625 MODIS albedo
!    R. El Khatib : 02-21-20 Bugfix on the ordering of dynamic fields
!    JJMorcrette  : 20060925 DU, BC, OM, SO2, SOA climatological fields
!    G. Balsamo   : 20070115 Soil type
!    S. Serrar    : 20070717 methane surface fields
!    Y. Seity     : 07-01-15 Add graupel and hail precipitations 
!    H. Hersbach  : 20080606 Add ocean current
!    P. Lopez     : 26-Jun-2009 Added 100 m zonal and meridional wind fields
!    P. Lopez     : 22-Oct-2008 Added ducting diagnostic fields
!    JJMorcrette  : 20091201 Total and clear-sky direct SW radiation flux at surface 
!    H. Hersbach  : 04-Dec-2009 Introduce 10m-neutral wind and friction velocity
!    S. Boussetta/G.Balsamo  05-June-2009   Added variable LAI fields
!    R. Forbes    : 01-Mar-2010: Add TCRW, TCSW diagnostics
!    JJMorcrette  : 20100212  PP of CBASE, 0DEGL and VISIH
!    T. Wilhelmsson : 25-Mar-10 Add 6 hourly min/max fields
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARFPOS  , ONLY : JPOSFSU  ,JPOSVX2
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMFPDIM , ONLY : NFPBFIN  ,NFPBFOUT ,NFPGT0B  ,NFPAUXB
USE YOMAUXB  , ONLY : GAUXBUF
USE SURFACE_FIELDS_MIX , ONLY : NSURF,&
 & SP_SB, YSP_SB, SP_SG, YSP_SG,&
 & SP_RR, YSP_RR, SP_CI, YSP_CI, SD_WS, YSD_WS,&
 & SD_VD, YSD_VD, SD_VX, YSD_VX,&
 & SD_VF, YSD_VF, SD_VV, YSD_VV, SD_VP, YSD_VP, &
 & SD_VA, YSD_VA, SD_VC, YSD_VC, SD_X2, YSD_X2
USE YOMPHY   , ONLY : LMPHYS   ,LNEIGE   ,LHMTO    ,LOZONE   ,LSOLV, LFGEL
USE YOEPHY   , ONLY : LEPHYS
USE YOMCFU   , ONLY : NFDCFU   ,LTSTRD   ,LTSTRC   ,LTSTRT   ,&
 & LTFPLC   ,LTFPLS   ,LTFR  
USE YOMGFUB  , ONLY : GFUBUF
USE PTRGFU   , ONLY : MSTRDU   ,MSTRDV   ,MSTRCU   ,MSTRCV   ,&
 & MFDICQ   ,MFDICS   ,MSTRTU   ,MSTRTV   ,MFDITQ   ,&
 & MFDITS   ,MFPLCL   ,MFPLSL   ,MFPLCN   ,MFPLSN   ,&
 & MFPLCG   ,MFPLSG   ,MFPLCH   ,MFPLSH   ,&
 & MFRSO    ,MFRSOLU  ,MFRTH    ,MWS      ,MSNS     ,&
 & MQTOT    ,MFNEBT   ,MOZONT   ,MSPRES   ,MFCL     ,&
 & MFCS     ,MFDISH   ,MFTOPH   ,MQICE    ,MQLI     ,&
 & MFRSOC0  ,MFRTHC0  ,MFRSOPS  ,MFRSOPT  ,MFRSODS  ,&
 & MFRTHDS  ,MFCLL    ,MFCLN    ,MFEVL    ,MFEVN    ,&
 & MFONTE   ,MFCHSP   ,MFEVV    ,MFLWSP   ,MFTR     ,&
 & MRUISL   ,MRUISP   ,MRUISS   ,MNEBCON  ,MNEBHAU  ,&
 & MFRSOC1  ,MFRTHC1  ,MNEBMOY  ,MNEBBAS  ,MFDUTP  
USE YOMXFU   , ONLY : NFDXFU   ,LTXTRD   ,LTXTRC   ,LTXTRT   ,LTXR
USE YOMXFUB  , ONLY : XFUBUF
USE PTRXFU   , ONLY : MXTRDU   ,MXTRDV   ,MXTRCU   ,MXTRCV   ,&
 & MXDICQ   ,MXDICS   ,MXTRTU   ,MXTRTV   ,MXDITQ   ,&
 & MXDITS   ,MXPLCL   ,MXPLSL   ,MXPLCN   ,MXPLSN   ,&
 & MXPLCG   ,MXPLSG   ,MXPLCH   ,MXPLSH   ,&
 & MXRSO    ,MXRTH    ,MXNEBT   ,MXUCLS   ,&
 & MXVCLS   ,MXTCLS   ,MXQCLS   ,MXRHCLS  ,MXTN     ,&
 & MXCAPE   ,MXMOCON  ,MXCLPH   ,MXUGST   ,MXVGST   ,&
 & MXHUX    ,MXHUN    ,&
 & MXTX     ,MXNBCON  ,MXNBHAU  ,MXNBMOY  ,MXNBBAS  
USE YOMAFN   , ONLY : TFP_DYNDS,TFP_SP   ,TFP_ABS  ,GFP_PHYDS , &
 & GFP_LSM  ,GFP_CVL  ,GFP_CVH  ,GFP_TVH  ,GFP_TVL  , &
 & GFP_LAIH ,GFP_LAIL , GFP_SOTY , &
 & GFP_GFIS ,GFP_ST   ,GFP_DST  ,GFP_RDST ,GFP_SSW  ,GFP_ISTL3, &
 & GFP_DSW  ,GFP_CSSW ,GFP_CDSW ,GFP_SD   ,GFP_SR   ,GFP_ISTL2, &
 & GFP_AL   ,GFP_EMIS ,GFP_SDOG ,GFP_VEG  ,GFP_ACOT ,GFP_ISTL1, &
 & GFP_DPAT ,GFP_STL1 ,GFP_STL2 ,GFP_STL3 ,GFP_STL4 ,GFP_SST  , &
 & GFP_SWL1 ,GFP_SWL2 ,GFP_SWL3 ,GFP_SWL4 ,GFP_ISOR ,GFP_CI   , &
 & GFP_ANOR ,GFP_SLOR ,GFP_LSRH ,GFP_SRC  ,GFP_TTRC ,GFP_ISTL4, &
 & GFP_SKT  ,GFP_LSP  ,GFP_CP   ,GFP_SF   ,GFP_BLD  ,GFP_RSN  , &
 & GFP_SSHF ,GFP_SLHF ,GFP_MSLD ,GFP_TCC  ,GFP_10U  ,GFP_ASN  , &
 & GFP_10V  ,GFP_2T   ,GFP_2D   ,GFP_SSR  ,GFP_STR  ,GFP_TSN  , &
 & GFP_TSR  ,GFP_TTR  ,GFP_EWSS ,GFP_NSSS ,GFP_E    ,GFP_TCWV , &
 & GFP_CCC  ,GFP_LCC  ,GFP_MCC  ,GFP_HCC  ,GFP_LGWS , &
 & GFP_MGWS ,GFP_GWD  ,GFP_MX2T ,GFP_MN2T ,GFP_MX2T6,GFP_MN2T6, &
 & GFP_RO   ,GFP_SRO  ,GFP_SSRO ,GFP_SSRC , &
 & GFP_ALB  ,GFP_IEWSS,GFP_INSSS,GFP_ISSHF,GFP_IE   ,GFP_CSMR , &
 & GFP_CSF  ,GFP_LSF  ,GFP_Z0F  ,GFP_LZ0H ,GFP_TCW  ,GFP_FDSW , &
 & GFP_PSRHU,GFP_PADOU,GFP_PADOV,GFP_PCAAG,GFP_PCAPG,GFP_FSSW , &
 & GFP_FSU  ,GFP_SSRD ,GFP_STRD ,GFP_TCO3 ,GFP_TCGHG,GFP_TCGRG, &
 & GFP_TCTRAC,GFP_CHAR,GFP_IVEG ,GFP_TP   ,GFP_SP   , &
 & GFP_CLSP ,GFP_CCP  ,GFP_CDUTP,GFP_SMLT ,GFP_TSRC ,GFP_RSMIN, &
 & GFP_CLSS ,GFP_CCSF ,GFP_CLSG ,GFP_CCSG ,GFP_CLSH ,GFP_CCSH , &
 & GFP_CUSS ,GFP_CVSS ,GFP_CSSH ,GFP_ARG  , &
 & GFP_CSLH ,GFP_CTSP ,GFP_CTCC ,GFP_CBLD ,GFP_CSSR ,GFP_SAB  , &
 & GFP_CSTR ,GFP_CTSR ,GFP_CTTR ,GFP_CCCC ,GFP_CHCC ,GFP_D2   , &
 & GFP_CMCC ,GFP_CLCC ,GFP_CUGW ,GFP_CVGW ,GFP_CE   ,GFP_LAI  , &
 & GFP_CS   ,GFP_CLHE ,GFP_CLHS ,GFP_CWS  ,GFP_SUND ,GFP_HV   , &
 & GFP_CSNS ,GFP_CQTO ,GFP_CTO3 ,GFP_CTME ,GFP_CICE ,GFP_Z0H  , &
 & GFP_CLI  ,GFP_CCVU ,GFP_CCVV ,GFP_CCVQ ,GFP_CCVS ,GFP_IC   , &
 & GFP_CTUQ ,GFP_CTUS ,GFP_CSOC ,GFP_CTHC ,GFP_CSOP ,GFP_ALSN , &
 & GFP_CTOP ,GFP_CSOD ,GFP_CTHD ,GFP_CFON ,GFP_CCHS ,GFP_ALV  , &
 & GFP_CEAS ,GFP_CSRU ,GFP_CDRU ,GFP_CIRU ,GFP_CETP ,GFP_ALS  , &
 & GFP_CTSOC,GFP_CTTHC,GFP_LSPF ,GFP_ES   ,GFP_BLH  ,GFP_SNDE , &
 & GFP_CTP  ,GFP_XTCC ,GFP_X10U ,GFP_X10V ,GFP_X2T  ,GFP_BSR  , &
 & GFP_X2SH ,GFP_X2RH ,GFP_XCCC ,GFP_XHCC ,GFP_XMCC ,GFP_BAAL , &
 & GFP_XLCC ,GFP_XX2T ,GFP_XN2T ,GFP_XCVU , &
 & GFP_XCVV ,GFP_XCVQ ,GFP_XCVS ,GFP_XTUU ,GFP_XTUV ,GFP_STRC , &
 & GFP_XTUQ ,GFP_XTUS ,GFP_XGDU ,GFP_XGDV ,GFP_XLSP ,GFP_10FG , &
 & GFP_10FG6,GFP_XCP  ,GFP_XLSS ,GFP_XLSG ,GFP_XLSH ,GFP_XCSF , &
 & GFP_XCSG ,GFP_XCSH ,GFP_XSSR ,GFP_XSTR ,GFP_XTTR , &
 & GFP_X10FF,GFP_XCAPE,GFP_XMOCO,GFP_XCLPH,GFP_XTSR ,GFP_LAN  , &
 & GFP_XUGST,GFP_XVGST,GFP_XX2HU,GFP_XN2HU,GFP_IDZ0 , &
 & GFP_ITZ0 ,GFP_INCTX,GFP_INCTN,GFP_INCHX,GFP_INCHN,GFP_VX2  , &
 & GFP_SFIS ,GFP_SPAR ,GFP_SUVB ,GFP_CAPE ,GFP_TCLW ,GFP_TCIW , &
 & GFP_TCRW ,GFP_TCSW , & 
 & GFP_ASEA ,GFP_ALAN ,GFP_ASOO ,GFP_ADES ,GFP_O3A  ,GFP_O3B  , &
 & GFP_O3C  ,GFP_SDFOR,GFP_SFDIR,GFP_SCDIR,GFP_CBASE,GFP_0DEGL,GFP_VISIH,&
 & GFP_CO2NBF,GFP_CO2OF,GFP_CO2APF ,GFP_CO2FIRE ,GFP_SF6,GFP_SPARC,GFP_STINC,GFP_VIMD , &
 & GFP_CH4AG,GFP_CH4F,GFP_ALUVP,GFP_ALUVD,GFP_ALNIP,GFP_ALNID                    , & 
 & GFP_BCBF ,GFP_BCFF ,GFP_BCGF ,GFP_OMBF ,GFP_OMFF ,GFP_OMGF , &
 & GFP_SO2L ,GFP_SO2H ,GFP_SOGF ,GFP_VOLC ,GFP_VOLE ,GFP_SOA  , &
 & GFP_AERDEP,GFP_AERLTS,GFP_AERSCC,GFP_GRGFLX,GFP_UCUR,GFP_VCUR,GFP_TRACFLX, &
 & GFP_DNDZN,GFP_DNDZA,GFP_DCTB,GFP_TPLB,GFP_TPLT, &
 & GFP_100U,GFP_100V,GFP_10NU,GFP_10NV,GFP_ZUST
USE YOMFPC   , ONLY : NFPCLI   ,LFPLOSP  ,WSXI     ,WDXI     ,RFPCORR  , &
 & RFPCSAB  ,RFPCD2
USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN  ,NFPTRDYN 
USE YOMCST   , ONLY : RPI      ,RG
USE YOMSTA   , ONLY : RDTDZ1
USE YOMFPG   , ONLY : LFPOSHOR
USE YOMPHY1  , ONLY : GCONV    ,RD1
USE PTRFP4   , ONLY : YRQGFP
USE YOMOMPDIST , ONLY : GPP
USE YOMARPHY , ONLY : LMSE
USE YOM_YGFL , ONLY : JPGHG,NGHG,YGHG,JPGRG,NGRG,YGRG,JPTRAC,NTRAC,YTRAC
USE YOMGRB   , ONLY : NGRBGHG,NGRBGRG,NGRBTRAC
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

!     DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(KNUMB) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(KNUMB) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KNUMB) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PARFP1(NFPBFIN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPBUF1(NPROMA,NFPBFIN) 
CHARACTER (LEN = 7) ::  CLLOCK
REAL(KIND=JPRB), ALLOCATABLE :: ZAUX(:,:)
REAL(KIND=JPRB) :: ZGFU(NPROMA,NFDCFU)
REAL(KIND=JPRB) :: ZXFU(NPROMA,NFDXFU)
REAL(KIND=JPRB) :: ZDIF(NPROMA)
REAL(KIND=JPRB) :: ZWFC(NPROMA),ZWPMX(NPROMA),ZWSAT(NPROMA),ZWSMX(NPROMA)
REAL(KIND=JPRB) :: ZWWILT(NPROMA)
REAL(KIND=JPRB) :: ZDUMM(NPROMA)
REAL(KIND=JPRB) :: ZGM2(NPROMA)
REAL(KIND=JPRB) :: ZRCORI(NPROMA), ZGM(NPROMA), ZOROG(NPROMA)


INTEGER(KIND=JPIM) :: INC, I0, IORDER, IG0, IFLD, IST, IEND, IJ
INTEGER(KIND=JPIM) :: INFLEV, IVARSU, IEXTR2, II
INTEGER(KIND=JPIM) :: IBL, J, JVAR, JFLD, JROF, JL, IC, JI, IANO
INTEGER(KIND=JPIM) :: JGHG,IGHG,JTRAC,ITRAC,JGRG,IGRG,INDEX

LOGICAL :: LLCONV, LLHMT, LLU, LLV

REAL(KIND=JPRB) ::  Z1SWSX, ZDIFMAX, ZDTSGDZ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "acsolw.intfb.h"
#include "cvlaniso.intfb.h"
#include "sc2rdg.intfb.h"

!     IST     : first adress in post-processing buffers to read
!     IEND    : last  adress in post-processing buffers to read
!     ZAUX    : unfitted fields array
!     ------------------------------------------------------------------

!*       1. PRELIMINARY INITIALISATIONS.
!           ----------------------------

IF (LHOOK) CALL DR_HOOK('HPOS',0,ZHOOK_HANDLE)

IBL=(KSTGLO-1)/NPROMA+1

IST=1
IEND=KNUMB

ZRCORI(1:KNUMB)=PRCORI(1:KNUMB)
ZGM   (1:KNUMB)=PGM   (1:KNUMB)
ZOROG (1:KNUMB)=POROG (1:KNUMB)

!     ------------------------------------------------------------------

!*       3. DYNAMICS
!           --------

IF (CDCONF == 'A' .OR. CDCONF == 'I') THEN

  IF (.NOT.ALLOCATED(GPP)) CALL ABOR1('HPOS : GPP NOT ALLOCATED')
  IF (NFPGT0B /= SIZE(GPP,2)) CALL ABOR1('HPOS : GPP WRONG SECOND DIMENSION')

  IF (NFPAUXB > 0) THEN
    ALLOCATE(ZAUX(NPROMA,NFPAUXB))
    CALL SC2RDG(GAUXBUF,KSTGLO,IST,IEND,ZAUX)
  ENDIF

  DO JI=IST,IEND
    ZGM2(JI)=ZGM(JI)**2
  ENDDO

  IG0=1
  DO J=1, NFPOSDYN
    IJ=NFPTRDYN(J)
    INC=0
    DO  JL=1, QFPTYPE%ILEV(IJ)
      INC=INC+QFPTYPE%ISPD(JL,IJ)
    ENDDO
    IFLD=IG0
    IF (QFPTYPE%ISF(IJ) >= 1) THEN
!     Spectrally fitted data
      I0=QFPTYPE%IGT1(IJ,0)
      DO JFLD=IFLD,IFLD+INC-1
        II=I0+JFLD-IFLD
        DO JI=IST,IEND
          PFPBUF1(JI,JFLD)=GPP(JI,II,IBL)
        ENDDO
      ENDDO
    ELSE
!     Non-spectrally fitted data
      I0=QFPTYPE%IGPX(IJ)
      DO JFLD=IFLD,IFLD+INC-1
        II=I0+JFLD-IFLD
        DO JI=IST,IEND
          PFPBUF1(JI,JFLD)=ZAUX(JI,II)
        ENDDO
      ENDDO
    ENDIF
    IG0=IG0+INC
!   Tensors aspects : 
    IORDER=ABS(TFP_DYNDS(IJ)%IORDR)
    IF (IORDER == 1) THEN
      PARFP1(IFLD:IFLD+INC-1)=-1.0_JPRB
    ELSE
      PARFP1(IFLD:IFLD+INC-1)=+1.0_JPRB
    ENDIF
    IF (LFPOSHOR .AND. IORDER > 0) THEN
      SELECT CASE (IORDER)
      CASE (1) 
!         Vectors => multiply by map factor if actual horizontal interpolations
        DO JFLD=IFLD,IFLD+INC-1
          DO JI=IST,IEND
            PFPBUF1(JI,JFLD)=PFPBUF1(JI,JFLD)*ZGM(JI)
          ENDDO
        ENDDO
      CASE (2)
!         2nd order derivatives => multiply by square map factor 
!         if actual horizontal interpolations
        DO JFLD=IFLD,IFLD+INC-1
          DO JI=IST,IEND
            PFPBUF1(JI,JFLD)=PFPBUF1(JI,JFLD)*ZGM2(JI)
          ENDDO
        ENDDO
      CASE (3:)
        DO JFLD=IFLD,IFLD+INC-1
          DO JI=IST,IEND
            PFPBUF1(JI,JFLD)=PFPBUF1(JI,JFLD)*ZGM(JI)**IORDER
          ENDDO
        ENDDO
      END SELECT
    ENDIF 
!   Particular cases :
    IF (IJ==TFP_ABS%ICOD .AND. LFPOSHOR) THEN
!     Absolute vorticity => add Coriolis parameter
      DO JFLD=IFLD,IFLD+INC-1
        DO JI=IST,IEND
          PFPBUF1(JI,JFLD)=PFPBUF1(JI,JFLD)+ZRCORI(JI)
        ENDDO
      ENDDO
    ELSEIF (IJ==TFP_ABS%ICOD .AND. .NOT.LFPOSHOR) THEN
!     (Reduced) Absolute vorticity => add (Reduced) Coriolis parameter
      DO JFLD=IFLD,IFLD+INC-1
        DO JI=IST,IEND
          PFPBUF1(JI,JFLD)=PFPBUF1(JI,JFLD)+ZRCORI(JI)/ZGM2(JI)
        ENDDO
      ENDDO
    ELSEIF (IJ==TFP_SP%ICOD .AND. LFPOSHOR .AND.(.NOT.LFPLOSP) ) THEN
!     Ps => Ln(Ps)
      DO JFLD=IFLD,IFLD+INC-1
        DO JI=IST,IEND
          PFPBUF1(JI,JFLD)=LOG(PFPBUF1(JI,JFLD))
        ENDDO
      ENDDO
    ENDIF
  ENDDO

  IF (NFPAUXB > 0) DEALLOCATE(ZAUX)

ENDIF

!     ------------------------------------------------------------------

!*       5. COPY PHYSICAL FIELDS FROM MODEL ARRAYS/WORKFILES TO BUFFER PFPBUF1.
!           ------------------------------------------------------------------

ZDTSGDZ=RDTDZ1/RG

IF (NSURF > 0) THEN

!*       5.1 Read model fields.


  IF (NFPCLI >= 2) THEN
    DO JI=IST,IEND
      ZDIF(JI)=ABS(ZOROG(JI)-SD_VX(JI,YSD_VX%YORO%MP,IBL))
    ENDDO
    ZDIFMAX=MAXVAL(ZDIF(IST:IEND))
    IF (ZDIFMAX > RFPCORR) THEN
      WRITE (NULOUT,'(A,F9.2,A)') & 
       & ' MAX. DIFFERENCE BETWEEN SOURCE AND MODEL OROGRAPHIES :  ',&
       & ZDIFMAX,' J/kg '  
      WRITE (NULOUT,'(A,F9.2,A)')' ALLOWED DIFFERENCE : ', RFPCORR,' J/kg '
      WRITE (NULOUT,'(A)')' TOO MUCH DIFFERENCE >> RERUN CONFIGURATION 923', &
       & ' OR INCREASE RFPCORR IN NAMELIST NAMFPC '
      CALL ABOR1(' HPOS: SOURCE AND MODEL OROGRAPHIES MISMATCH ')
    ENDIF
  ENDIF
  IF (NFPCLI >= 3 .AND. LMPHYS) THEN
    DO JI=IST,IEND
      ZDIF(JI)=ABS(SD_VV(JI,YSD_VV%YSAB%MP,IBL)-SD_VX(JI,YSD_VX%YSAB%MP,IBL))
    ENDDO
    ZDIFMAX=MAXVAL(ZDIF(IST:IEND))
    IF (ZDIFMAX > RFPCSAB) THEN
      WRITE (NULOUT,'(A,F9.6,A)') &
       & ' MAX. DIFFERENCE BETWEEN SOURCE AND MODEL PERCENTAGE OF SAND :  ', &
       & ZDIFMAX,' % '  
      WRITE (NULOUT,'(A,F9.6,A)')' ALLOWED DIFFERENCE : ',RFPCSAB,' % '
      WRITE (NULOUT,'(A)')' TOO MUCH DIFFERENCE >> RERUN CONFIGURATION 923', &
       & ' OR INCREASE RFPCSAB IN NAMELIST NAMFPC ' 
      CALL ABOR1(' HPOS: SOURCE AND MODEL SAND MISMATCH ')
    ENDIF
    DO JI=IST,IEND
      ZDIF(JI)=ABS(SD_VV(JI,YSD_VV%YD2%MP,IBL)-SD_VX(JI,YSD_VX%YXD2%MP,IBL))
    ENDDO
    ZDIFMAX=MAXVAL(ZDIF(IST:IEND))
    IF (ZDIFMAX > RFPCD2) THEN
      WRITE (NULOUT,'(A,F9.4,A)') &
       & ' MAX. DIFFERENCE BETWEEN SOURCE AND MODEL SOIL DEPTH :  ', &
       & ZDIFMAX,' m ' 
      WRITE (NULOUT,'(A,F9.4,A)')' ALLOWED DIFFERENCE : ',RFPCD2,' m '
      WRITE (NULOUT,'(A)')' TOO MUCH DIFFERENCE >> RERUN CONFIGURATION 923', &
       & ' OR INCREASE RFPCD2 IN NAMELIST NAMFPC ' 
      CALL ABOR1(' HPOS: SOURCE AND MODEL SOIL DEPTH MISMATCH ')
    ENDIF
  ENDIF

ENDIF

IF (CDCONF == 'P'.AND. NFDCFU > 0) THEN
  CALL SC2RDG(GFUBUF,KSTGLO,IST,IEND,ZGFU)
ENDIF

IF (CDCONF == 'P' .OR. CDCONF == 'G') THEN

  IF (NFDXFU > 0) THEN
    CALL SC2RDG(XFUBUF,KSTGLO,IST,IEND,ZXFU)
  ENDIF

!*       5.2 Initialize parity array and fill buffer.

  ZDUMM(:)=1.0_JPRB
  IEXTR2=0

  DO JFLD=1,NFPBFOUT

    IFLD=JFLD
    IC=YRQGFP%ICOD(IFLD)

    IF (LFPOSHOR) THEN
      IANO=GFP_PHYDS(IC)%IANO
    ELSE
!     No interpolation => only buffers transfers
      IANO=0
    ENDIF

    IVARSU=0
    DO JVAR=1,JPOSFSU
      IF (IC == GFP_FSU(JVAR)%ICOD) THEN
        IVARSU=JVAR
        EXIT
      ENDIF
    ENDDO

    DO JVAR=1,JPOSVX2
      IF (IC == GFP_VX2(JVAR)%ICOD) THEN
        IEXTR2=IEXTR2+1
      ENDIF
    ENDDO 

    IF     (IC == GFP_LSM%ICOD .OR. &
     &     (IC == GFP_LAN%ICOD .AND. NFPCLI == 0)) THEN
!     Land-sea mask
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YLSM%MP,IBL)
    ELSEIF (IC == GFP_RDST%ICOD) THEN
!     Interpolated surface temperature
      PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YT%MP0,IBL)
    ELSEIF (IC == GFP_ST%ICOD .OR. (LMPHYS .AND. IC == GFP_SST%ICOD)) THEN
!     Surface temperature
      IF (IANO == 0 .OR. NFPCLI <= 1) THEN
!       Interpolation of Ts
        PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YT%MP0,IBL)
      ELSE
!       Interpolation of the anomaly Ts-Ts*
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=SP_RR(JI,YSP_RR%YT%MP0,IBL)-SD_VX(JI,YSD_VX%YTSC%MP,IBL) &
           & -ZDTSGDZ*(ZOROG(JI)-SD_VX(JI,YSD_VX%YORO%MP,IBL))  
        ENDDO
      ENDIF
    ELSEIF (LEPHYS .AND. (IC == GFP_SST%ICOD)) THEN
!     Sea surface temperature.
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSST%MP,IBL)

!        ECMWF extra fields (CNSKT to CNDPAT).

    ELSEIF (IC == GFP_SKT%ICOD) THEN
!           Interpolation of skin temp.
      PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YT%MP0,IBL)
    ELSEIF (IC == GFP_STL1%ICOD) THEN
!           Interpolation of TsLev1
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YT%MP0,IBL)
    ELSEIF (IC == GFP_STL2%ICOD) THEN
!           Interpolation of TsLev2
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,2,YSP_SB%YT%MP0,IBL)
    ELSEIF (IC == GFP_STL3%ICOD ) THEN
!           Interpolation of TsLev3
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,3,YSP_SB%YT%MP0,IBL)
    ELSEIF (IC == GFP_STL4%ICOD ) THEN
!           Interpolation of TsLev4
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,4,YSP_SB%YT%MP0,IBL)
    ELSEIF (IC == GFP_SRC%ICOD ) THEN
!           Interpolation of SRC
      PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YW%MP0,IBL)
    ELSEIF (IC == GFP_SWL1%ICOD ) THEN
!           Interpolation of SWLev1
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YQ%MP0,IBL)
    ELSEIF (IC == GFP_SWL2%ICOD ) THEN
!           Interpolation of SWLev2
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,2,YSP_SB%YQ%MP0,IBL)
    ELSEIF (IC == GFP_SWL3%ICOD ) THEN
!           Interpolation of SWLev3
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,3,YSP_SB%YQ%MP0,IBL)
    ELSEIF (IC == GFP_SWL4%ICOD ) THEN
!           Interpolation of SWLev4
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,4,YSP_SB%YQ%MP0,IBL)
    ELSEIF (IC == GFP_ISTL1%ICOD ) THEN
!              Interpolation of ISTLev1
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YTL%MP0,IBL)
    ELSEIF (IC == GFP_ISTL2%ICOD ) THEN
!              Interpolation of ISTLev2
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,2,YSP_SB%YTL%MP0,IBL)
    ELSEIF (IC == GFP_ISTL3%ICOD ) THEN
!              Interpolation of ISTLev3
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,3,YSP_SB%YTL%MP0,IBL)
    ELSEIF (IC == GFP_ISTL4%ICOD ) THEN
!              Interpolation of ISTLev4
      PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,4,YSP_SB%YTL%MP0,IBL)
    ELSEIF (IC == GFP_CI%ICOD ) THEN
!              Interpolation of ice cover.                                
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCI%MP,IBL)
    ELSEIF (IC == GFP_CVL%ICOD ) THEN
!              Interpolation of CVL.                                
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCVL%MP,IBL)
    ELSEIF (IC == GFP_CVH%ICOD ) THEN
!              Interpolation of CVH.                                
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCVH%MP,IBL)
    ELSEIF (IC == GFP_TVL%ICOD ) THEN
!              Interpolation of TVL.                                
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YTVL%MP,IBL)
    ELSEIF (IC == GFP_TVH%ICOD ) THEN
!              Interpolation of TVH.                                
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YTVH%MP,IBL)
    ELSEIF (IC == GFP_TSN%ICOD ) THEN
!              Interpolation of snow temperature
      PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YT%MP0,IBL)
    ELSEIF (IC == GFP_ASN%ICOD ) THEN
!              Interpolation of snow albedo.                                
      PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YA%MP0,IBL)
    ELSEIF (IC == GFP_RSN%ICOD ) THEN
!              Interpolation of snow density.                                
      PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YR%MP0,IBL)
    ELSEIF (IC == GFP_DST%ICOD) THEN
      IF (IANO == 0) THEN
!       Interpolation of Tp
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=SP_SB(JI,1,YSP_SB%YT%MP0,IBL)
        ENDDO
      ELSEIF (NFPCLI <= 1) THEN
!       Interpolation of Tp-Ts
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=SP_SB(JI,1,YSP_SB%YT%MP0,IBL)-SP_RR(JI,YSP_RR%YT%MP0,IBL)
        ENDDO
      ELSE
!       Interpolation of Tp-Tp*
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=SP_SB(JI,1,YSP_SB%YT%MP0,IBL)-SD_VX(JI,YSD_VX%YTPC%MP,IBL) &
           & -ZDTSGDZ*(ZOROG(JI)-SD_VX(JI,YSD_VX%YORO%MP,IBL))  
        ENDDO 
      ENDIF
    ELSEIF (IC == GFP_SSW%ICOD) THEN
      IF (IANO == 0) THEN
!       Interpolation of Ws
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=SP_RR(JI,YSP_RR%YW%MP0,IBL)
        ENDDO
      ELSE
!       Compute Wsmax 
        IF (LSOLV) THEN
!         ISBA option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VF(1,YSD_VF%YLSM%MP,IBL),SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSEIF (LMSE) THEN
!         SURFEX option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VX(1,YSD_VX%YLSM%MP,IBL),SD_VX(1,YSD_VX%YIVEG%MP,IBL),&
           & SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSE
          ZWSMX(IST:IEND)=WSXI
        ENDIF
        IF (NFPCLI <= 1) THEN
!         Interpolation of relative value Ws/Wsmax
          PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YW%MP0,IBL)/ZWSMX(IST:IEND)
        ELSE
!         Interpolation of the anomaly of Ws/Wsmax 
          PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YW%MP0,IBL)/ZWSMX(IST:IEND) &
           & -SD_VX(IST:IEND,YSD_VX%YPWS%MP,IBL)  
        ENDIF
      ENDIF
    ELSEIF (IC == GFP_DSW%ICOD) THEN
      IF (IANO == 0) THEN
!       Interpolation of Wd
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=SP_SB(JI,1,YSP_SB%YQ%MP0,IBL)
        ENDDO
      ELSE
!       Compute Wdmax
        IF (LSOLV) THEN
!         ISBA option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VF(1,YSD_VF%YLSM%MP,IBL),SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSEIF (LMSE) THEN
!         SURFEX option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VX(1,YSD_VX%YLSM%MP,IBL),SD_VX(1,YSD_VX%YIVEG%MP,IBL),&
           & SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSE
          ZWPMX(IST:IEND)=WDXI
        ENDIF
        IF (NFPCLI <= 1) THEN
!         Interpolation of relative value Wd/Wdmax
          PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YQ%MP0,IBL)/ZWPMX(IST:IEND)
        ELSE
!         Interpolation of the anomaly of Wd/Wdmax
          PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YQ%MP0,IBL)/ZWPMX(IST:IEND) &
           & -SD_VX(IST:IEND,YSD_VX%YPWP%MP,IBL)
        ENDIF
      ENDIF
    ELSEIF (IC == GFP_FSSW%ICOD) THEN
      IF (.NOT.LFGEL.AND..NOT.LMSE) THEN
        PFPBUF1(IST:IEND,IFLD)=0.0_JPRB  
      ELSEIF (IANO == 0) THEN
!       Interpolation of Wsf
        IF(.NOT.LMSE) THEN
          PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YRR(4)%MP0,IBL)
        ELSE
          PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YRR(3)%MP0,IBL)
        ENDIF
      ELSE
!       Interpolation of Wsf/Wsmax
        IF (LSOLV) THEN
!         ISBA option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VF(1,YSD_VF%YLSM%MP,IBL),SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSEIF (LMSE) THEN
!         SURFEX option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VX(1,YSD_VX%YLSM%MP,IBL),SD_VX(1,YSD_VX%YIVEG%MP,IBL),&
           & SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSE
          ZWSMX(IST:IEND)=WSXI
        ENDIF
        IF (.NOT.LMSE) THEN
          PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YRR(4)%MP0,IBL)/ZWSMX(IST:IEND)
        ELSE
          PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YRR(3)%MP0,IBL)/ZWSMX(IST:IEND)
        ENDIF
      ENDIF
    ELSEIF (IC == GFP_FDSW%ICOD) THEN
      IF (.NOT.LFGEL.AND..NOT.LMSE) THEN
        PFPBUF1(IST:IEND,IFLD)=0.0_JPRB
      ELSEIF (IANO == 0) THEN
!       Interpolation of Wdf
        PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YSB(3)%MP0,IBL)
      ELSE
!       Interpolation of Wdf/Wdmax
        IF (LSOLV) THEN
!         ISBA option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VF(1,YSD_VF%YLSM%MP,IBL),SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSEIF (LMSE) THEN
!         SURFEX option
          LLHMT=.FALSE.
          CLLOCK='ACSOLW '
          CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
           & SD_VX(1,YSD_VX%YLSM%MP,IBL),SD_VX(1,YSD_VX%YIVEG%MP,IBL),&
           & SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
           & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        ELSE
          ZWPMX(IST:IEND)=WDXI
        ENDIF
        PFPBUF1(IST:IEND,IFLD)=SP_SB(IST:IEND,1,YSP_SB%YSB(3)%MP0,IBL)/ZWPMX(IST:IEND)
      ENDIF
    ELSEIF (IC == GFP_CSSW%ICOD) THEN
      CALL ABOR1('THIS RIGHT HAND SIDE FIELD IS NEVER SET, I HAVE REMOVED IT/MH')
!      PFPBUF1(IST:IEND,IFLD)=SD_VX(IST:IEND,YSD_VX%YWSC%MP,IBL)
    ELSEIF (IC == GFP_CDSW%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VP(IST:IEND,YSD_VP%YTPC%MP,IBL)
    ELSEIF (IC == GFP_SD%ICOD) THEN
      IF (IANO == 0 .OR. NFPCLI <= 1) THEN
!       Interpolation of absolute snow depth
        PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YF%MP0,IBL)
      ELSE
!       Interpolation of snow depth anomaly
        PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YF%MP0,IBL)-&
         & SD_VX(IST:IEND,YSD_VX%YSNO%MP,IBL)
      ENDIF
    ELSEIF (IC == GFP_SR%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YZ0F%MP,IBL)
    ELSEIF (IC == GFP_IDZ0%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YZ0F%MP,IBL)
    ELSEIF (IC == GFP_ITZ0%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YZ0H%MP,IBL)
    ELSEIF (IC == GFP_Z0H%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YZ0H%MP,IBL)
    ELSEIF (IC == GFP_AL%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YALBF%MP,IBL)

    ELSEIF (IC == GFP_ALUVP%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YALUVP%MP,IBL)
    ELSEIF (IC == GFP_ALUVD%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YALUVD%MP,IBL)
    ELSEIF (IC == GFP_ALNIP%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YALNIP%MP,IBL)
    ELSEIF (IC == GFP_ALNID%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YALNID%MP,IBL)

! aerosol climatological fields
    ELSEIF (IC == GFP_AERDEP%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YAERDEP%MP,IBL)
    ELSEIF (IC == GFP_AERLTS%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YAERLTS%MP,IBL)
    ELSEIF (IC == GFP_AERSCC%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YAERSCC%MP,IBL)

    ELSEIF (IC == GFP_BCBF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YBCBF%MP,IBL)
    ELSEIF (IC == GFP_BCFF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YBCFF%MP,IBL)
    ELSEIF (IC == GFP_BCGF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YBCGF%MP,IBL)

    ELSEIF (IC == GFP_OMBF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YOMBF%MP,IBL)
    ELSEIF (IC == GFP_OMFF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YOMFF%MP,IBL)
    ELSEIF (IC == GFP_OMGF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YOMGF%MP,IBL)

    ELSEIF (IC == GFP_SO2L%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSO2L%MP,IBL)
    ELSEIF (IC == GFP_SO2H%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSO2H%MP,IBL)
    ELSEIF (IC == GFP_SOGF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSOGF%MP,IBL)

    ELSEIF (IC == GFP_VOLC%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVOLC%MP,IBL)
    ELSEIF (IC == GFP_VOLE%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVOLE%MP,IBL)
    ELSEIF (IC == GFP_SOA%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSOA%MP,IBL)
    ELSEIF (ANY(IC == GFP_GRGFLX(:)%ICOD)) THEN
     DO JGRG=1,NGRG
        IF (IC == GFP_GRGFLX(JGRG)%ICOD) THEN
         PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VF%YGRGFLX(JGRG)%MP,IBL)
       ENDIF
      ENDDO
    ELSEIF (ANY(IC == GFP_TRACFLX(:)%ICOD)) THEN
     DO JTRAC=1,NTRAC
        IF (IC == GFP_TRACFLX(JTRAC)%ICOD) THEN
         PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VF%YTRACFLX(JTRAC)%MP,IBL)
       ENDIF
      ENDDO
    ELSEIF (IC == GFP_SDFOR%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSDFOR%MP,IBL)
    ELSEIF (IC == GFP_CO2NBF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCO2B%MP,IBL)
    ELSEIF (IC == GFP_CO2OF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCO2O%MP,IBL)
    ELSEIF (IC == GFP_CO2APF%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCO2A%MP,IBL)
    ELSEIF (IC == GFP_CO2FIRE%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCO2F%MP,IBL)
    ELSEIF (IC == GFP_CH4AG%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCH4AG%MP,IBL)
    ELSEIF (IC == GFP_CH4F%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YCH4F%MP,IBL)
    ELSEIF (IC == GFP_SF6%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSF6%MP,IBL)
    ELSEIF (IC == GFP_EMIS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YEMISF%MP,IBL)

! LAI variable fields 
    ELSEIF (IC == GFP_LAIH%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YLAIH%MP,IBL)
    ELSEIF (IC == GFP_LAIL%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YLAIL%MP,IBL)

    ELSEIF (IC == GFP_SOTY%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSOTY%MP,IBL)
    ELSEIF (IC == GFP_SDOG%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YGETRL%MP,IBL)
    ELSEIF (IC == GFP_VEG%ICOD ) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVEG%MP,IBL)
    ELSEIF (IC == GFP_ISOR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVRLAN%MP,IBL)
    ELSEIF (IC == GFP_ANOR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVRLDI%MP,IBL)
    ELSEIF (IC == GFP_SLOR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YSIG%MP,IBL)
    ELSEIF (IC == GFP_LSRH%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YLZ0H%MP,IBL)
    ELSEIF (IC == GFP_GFIS%ICOD .OR. IC == GFP_SFIS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZOROG(IST:IEND)
    ELSEIF (IC == GFP_ACOT%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVRLAN%MP,IBL)
    ELSEIF (IC == GFP_DPAT%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVRLDI%MP,IBL)
    ELSEIF (IC == GFP_IVEG%ICOD)  THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YIVEG%MP,IBL)
    ELSEIF (IC == GFP_RSMIN%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YRSMIN%MP,IBL)
    ELSEIF (IC == GFP_ARG%ICOD)   THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YARG%MP,IBL)
    ELSEIF (IC == GFP_SAB%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YSAB%MP,IBL)
    ELSEIF (IC == GFP_D2%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YD2%MP,IBL)
    ELSEIF (IC == GFP_LAI%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YLAI%MP,IBL)
    ELSEIF (IC == GFP_HV%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YHV%MP,IBL)
    ELSEIF (IC == GFP_ALS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YALS%MP,IBL)
    ELSEIF (IC == GFP_ALV%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VV(IST:IEND,YSD_VV%YALV%MP,IBL)
    ELSEIF (IC == GFP_IC%ICOD)  THEN
      PFPBUF1(IST:IEND,IFLD)=SP_RR(IST:IEND,YSP_RR%YRR(3)%MP0,IBL)
    ELSEIF (IC == GFP_ALSN%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YSG(2)%MP0,IBL)
    ELSEIF (IC == GFP_SNDE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SP_SG(IST:IEND,YSP_SG%YSG(3)%MP0,IBL)
    ELSEIF (IC == GFP_BSR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YZ0RLF%MP,IBL)
    ELSEIF (IC == GFP_BAAL%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YALBSF%MP,IBL)
    ELSEIF (IC == GFP_ASEA%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VA(IST:IEND,YSD_VA%YSEA%MP,IBL)
    ELSEIF (IC == GFP_ALAN%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VA(IST:IEND,YSD_VA%YLAN%MP,IBL)
    ELSEIF (IC == GFP_ASOO%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VA(IST:IEND,YSD_VA%YSOO%MP,IBL)
    ELSEIF (IC == GFP_ADES%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VA(IST:IEND,YSD_VA%YDES%MP,IBL)
    ELSEIF (IC == GFP_O3A%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VC(IST:IEND,YSD_VC%YA%MP,IBL)
    ELSEIF (IC == GFP_O3B%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VC(IST:IEND,YSD_VC%YB%MP,IBL)
    ELSEIF (IC == GFP_O3C%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VC(IST:IEND,YSD_VC%YC%MP,IBL)
    ELSEIF (IC == GFP_PADOU%ICOD) THEN
!     first component of vector
!        (sigma**2)*(1-gamma)/(1+gamma) * (cos(2*alpha),sin(2*alpha))
!     where sigma = standard deviation of orography
!           gamma = Anisotropy coefficient of topography
!           alpha = Direction of the principal axis of the topography
      LLCONV=.TRUE.
      LLU=.TRUE.
      LLV=.FALSE.
      CALL CVLANISO(IST,IEND,NPROMA,LLCONV,LLU,LLV,SD_VF(1,YSD_VF%YVRLAN%MP,IBL), &
       & SD_VF(1,YSD_VF%YVRLDI%MP,IBL),PFPBUF1(1,JFLD),ZDUMM,SD_VF(1,YSD_VF%YGETRL%MP,IBL))  
      IF (LFPOSHOR) THEN
!       Simulate a "reduced" field :
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=PFPBUF1(JI,IFLD)/ZGM(JI)
        ENDDO
      ENDIF
    ELSEIF (IC == GFP_PADOV%ICOD) THEN
!     Second component of vector
!        (sigma**2)*(1-gamma)/(1+gamma) * (cos(2*alpha),sin(2*alpha))
!     where sigma = standard deviation of orography
!           gamma = Anisotropy coefficient of topography
!           alpha = Direction of the principal axis of the topography
      LLCONV=.TRUE.
      LLU=.FALSE.
      LLV=.TRUE.
      CALL CVLANISO(IST,IEND,NPROMA,LLCONV,LLU,LLV,SD_VF(1,YSD_VF%YVRLAN%MP,IBL), &
       & SD_VF(1,YSD_VF%YVRLDI%MP,IBL),ZDUMM,PFPBUF1(1,JFLD),SD_VF(1,YSD_VF%YGETRL%MP,IBL))  
      IF (LFPOSHOR) THEN
!       Simulate a "reduced" field :
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=PFPBUF1(JI,IFLD)/ZGM(JI)
        ENDDO
      ENDIF
    ELSEIF (IC == GFP_PSRHU%ICOD) THEN
!     Relative surface moisture
      IF (NFPCLI <= 2) THEN
        Z1SWSX=1.0_JPRB/WSXI
        PFPBUF1(IST:IEND,JFLD)=0.5_JPRB* &
         & (1.0_JPRB-COS(RPI*SP_RR(IST:IEND,YSP_RR%YW%MP0,IBL)*Z1SWSX))  
      ELSE
!       ISBA option
        LLHMT=.TRUE.
        CLLOCK='ACSOLW '
        CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL), &
         & SD_VF(1,YSD_VF%YLSM%MP,IBL),SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
         & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        DO JROF=IST,IEND
          Z1SWSX=1.0_JPRB/(ZWFC(JROF)*GCONV*RD1)
          PFPBUF1(JROF,JFLD)=0.5_JPRB* &
           & (1.0_JPRB-COS(RPI*MIN(SP_RR(JROF,YSP_RR%YW%MP0,IBL)*Z1SWSX,1.0_JPRB)))
        ENDDO
      ENDIF
    ELSEIF (IC == GFP_PCAPG%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SP_CI(IST:IEND,YSP_CI%YCI(1)%MP0,IBL)
    ELSEIF (IC == GFP_PCAAG%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SP_CI(IST:IEND,YSP_CI%YCI(2)%MP0,IBL)

!        add the ECMWF diagnostics/physics fields (CNLSP to CNTCO3).

    ELSEIF (IC == GFP_LSP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLSP%MP,IBL)
    ELSEIF (IC == GFP_CP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YCP%MP,IBL)
!           Total precipitation
    ELSEIF (IC == GFP_TP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLSP%MP,IBL)+SD_VD(IST:IEND,YSD_VD%YCP%MP,IBL)
    ELSEIF (IC == GFP_SF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSF%MP,IBL)
    ELSEIF (IC == GFP_BLD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YBLD%MP,IBL)
    ELSEIF (IC == GFP_SSHF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSSHF%MP,IBL)
    ELSEIF (IC == GFP_SLHF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSLHF%MP,IBL)
    ELSEIF (IC == GFP_MSLD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YMSL%MP,IBL)
    ELSEIF (IC == GFP_SP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSP%MP,IBL)
    ELSEIF (IC == GFP_TCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCC%MP,IBL)
    ELSEIF (IC == GFP_10U%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y10U%MP,IBL)
    ELSEIF (IC == GFP_10V%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y10V%MP,IBL)
    ELSEIF (IC == GFP_2T%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y2T%MP,IBL)
    ELSEIF (IC == GFP_2D%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y2D%MP,IBL)
    ELSEIF (IC == GFP_SSR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSSR%MP,IBL)
    ELSEIF (IC == GFP_STR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSTR%MP,IBL)
    ELSEIF (IC == GFP_TSR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTSR%MP,IBL)
    ELSEIF (IC == GFP_TTR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTTR%MP,IBL)
    ELSEIF (IC == GFP_EWSS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YEWSS%MP,IBL)
    ELSEIF (IC == GFP_NSSS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YNSSS%MP,IBL)
    ELSEIF (IC == GFP_E%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YE%MP,IBL)
    ELSEIF (IC == GFP_CCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YCCC%MP,IBL)
    ELSEIF (IC == GFP_LCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLCC%MP,IBL)
    ELSEIF (IC == GFP_MCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YMCC%MP,IBL)
    ELSEIF (IC == GFP_HCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YHCC%MP,IBL)
    ELSEIF (IC == GFP_LGWS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLGWS%MP,IBL)
    ELSEIF (IC == GFP_MGWS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YMGWS%MP,IBL)
    ELSEIF (IC == GFP_GWD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YGWD%MP,IBL)
    ELSEIF (IC == GFP_MX2T%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YMX2T%MP,IBL)
    ELSEIF (IC == GFP_MN2T%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YMN2T%MP,IBL)
    ELSEIF (IC == GFP_MX2T6%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=MAXVAL(SD_VD(IST:IEND,YSD_VD%YMX2T6(:)%MP,IBL),DIM=2)
    ELSEIF (IC == GFP_MN2T6%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=MINVAL(SD_VD(IST:IEND,YSD_VD%YMN2T6(:)%MP,IBL),DIM=2)
    ELSEIF (IC == GFP_RO%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YRO%MP,IBL)
    ELSEIF (IC == GFP_SRO%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSRO%MP,IBL)    
    ELSEIF (IC == GFP_SSRO%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSSRO%MP,IBL)  
    ELSEIF (IC == GFP_ALB%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YALB%MP,IBL)
    ELSEIF (IC == GFP_IEWSS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YIEWSS%MP,IBL)
    ELSEIF (IC == GFP_INSSS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YINSSS%MP,IBL)
    ELSEIF (IC == GFP_ISSHF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YISSHF%MP,IBL)
    ELSEIF (IC == GFP_IE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YIE%MP,IBL)
    ELSEIF (IC == GFP_CSF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YCSF%MP,IBL)
    ELSEIF (IC == GFP_LSF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLSSF%MP,IBL)
    ELSEIF (IC == GFP_Z0F%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YZ0F%MP,IBL)
    ELSEIF (IC == GFP_LZ0H%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLZ0H%MP,IBL)
    ELSEIF (IC == GFP_TCW%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCW%MP,IBL)
    ELSEIF (IC == GFP_TCWV%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCWV%MP,IBL)
    ELSEIF (IC == GFP_TCLW%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCLW%MP,IBL)
    ELSEIF (IC == GFP_TCIW%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCIW%MP,IBL)
    ELSEIF (IC == GFP_TCRW%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCRW%MP,IBL)
    ELSEIF (IC == GFP_TCSW%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCSW%MP,IBL)
    ELSEIF (IC == GFP_SSRD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSSRD%MP,IBL)
    ELSEIF (IC == GFP_STRD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSTRD%MP,IBL)
    ELSEIF (IC == GFP_BLH%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YBLH%MP,IBL)
    ELSEIF (IC == GFP_SUND%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSUND%MP,IBL)
    ELSEIF (IC == GFP_TSRC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTSRC%MP,IBL)
    ELSEIF (IC == GFP_TTRC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTTRC%MP,IBL)
    ELSEIF (IC == GFP_SSRC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSSRC%MP,IBL)
    ELSEIF (IC == GFP_STRC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSTRC%MP,IBL)
    ELSEIF (IC == GFP_ES%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YES%MP,IBL)
    ELSEIF (IC == GFP_SMLT%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSMLT%MP,IBL)
    ELSEIF (IC == GFP_10FG%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y10FG%MP,IBL)
    ELSEIF (IC == GFP_10FG6%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=MAXVAL(SD_VD(IST:IEND,YSD_VD%Y10FG6(:)%MP,IBL),DIM=2)
    ELSEIF (IC == GFP_LSPF%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YLSPF%MP,IBL)
    ELSEIF (IC == GFP_VIMD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YVIMD%MP,IBL)
    ELSEIF (IC == GFP_TCO3%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCO3%MP,IBL)
    ELSEIF (ANY(IC == GFP_TCGHG(:)%ICOD)) THEN
       DO JGHG=1,JPGHG
         IF (IC == GFP_TCGHG(JGHG)%ICOD) THEN
           DO IGHG=1,NGHG
             IF(YGHG(IGHG)%IGRBCODE == NGRBGHG(JGHG)) INDEX=IGHG
           ENDDO
           PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCGHG(INDEX)%MP,IBL)
         ENDIF
       ENDDO
    ELSEIF (ANY(IC == GFP_TCTRAC(:)%ICOD)) THEN
       DO JTRAC=1,JPTRAC
         IF (IC == GFP_TCTRAC(JTRAC)%ICOD) THEN
           DO ITRAC=1,NTRAC
             IF(YTRAC(ITRAC)%IGRBCODE == NGRBTRAC(JTRAC)) INDEX=ITRAC
           ENDDO
           PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCTRAC(INDEX)%MP,IBL)
         ENDIF
       ENDDO
    ELSEIF (ANY(IC == GFP_TCGRG(:)%ICOD)) THEN
      DO JGRG=1,JPGRG
        IF (IC == GFP_TCGRG(JGRG)%ICOD) THEN
          DO IGRG=1,NGRG
            IF(YGRG(IGRG)%IGRBCODE == NGRBGRG(JGRG)) INDEX=IGRG
          ENDDO
          PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTCGRG(INDEX)%MP,IBL)
        ENDIF
      ENDDO
    ELSEIF (IC == GFP_CHAR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_WS(IST:IEND,YSD_WS%YCHAR%MP,IBL)
    ELSEIF (IC == GFP_SPAR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSPAR%MP,IBL)
    ELSEIF (IC == GFP_SUVB%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSUVB%MP,IBL)
    ELSEIF (IC == GFP_CAPE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YCAPE%MP,IBL)
    ELSEIF (IC == GFP_SPARC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSPARC%MP,IBL)
    ELSEIF (IC == GFP_STINC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSTINC%MP,IBL)
    ELSEIF (IC == GFP_SFDIR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSFDIR%MP,IBL)
    ELSEIF (IC == GFP_SCDIR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YSCDIR%MP,IBL)
    ELSEIF (IC == GFP_CBASE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YCBASE%MP,IBL)
    ELSEIF (IC == GFP_0DEGL%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y0DEGL%MP,IBL)
    ELSEIF (IC == GFP_VISIH%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YVISIH%MP,IBL)
    ELSEIF (IC == GFP_100U%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y100U%MP,IBL)
    ELSEIF (IC == GFP_100V%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y100V%MP,IBL)
    ELSEIF (IC == GFP_ZUST%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YZUST%MP,IBL)
    ELSEIF (IC == GFP_10NU%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y10NU%MP,IBL)
    ELSEIF (IC == GFP_10NV%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%Y10NV%MP,IBL)
    ELSEIF (IC == GFP_DNDZN%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YDNDZN%MP,IBL)
    ELSEIF (IC == GFP_DNDZA%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YDNDZA%MP,IBL)
    ELSEIF (IC == GFP_DCTB%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YDCTB%MP,IBL)
    ELSEIF (IC == GFP_TPLB%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTPLB%MP,IBL)
    ELSEIF (IC == GFP_TPLT%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=SD_VD(IST:IEND,YSD_VD%YTPLT%MP,IBL)
    ELSEIF (IVARSU > 0) THEN
!     Compute here !
      PFPBUF1(IST:IEND,IFLD)=0.0_JPRB
    ELSEIF (IEXTR2 > 0) THEN
!     Additional extra physics fields
      PFPBUF1(IST:IEND,IFLD)=SD_X2(IST:IEND,YSD_X2%YX2(IEXTR2)%MP,IBL)
    ELSEIF (IC == GFP_CLSP%ICOD) THEN
!           Stratiform precipitations
      IF(LTFPLS)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLSL)
    ELSEIF (IC == GFP_CCP%ICOD ) THEN
!           Convective precipitations
      IF(LTFPLC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLCL)
    ELSEIF (IC == GFP_CLSS%ICOD.AND.LNEIGE) THEN
!           Stratiform snow
      IF(LTFPLS)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLSN)
    ELSEIF (IC == GFP_CCSF%ICOD.AND.LNEIGE) THEN
!           Convective snow
      IF(LTFPLC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLCN)
    ELSEIF (IC == GFP_CLSG%ICOD.AND.LNEIGE) THEN
!           Stratiform graupel
      IF(LTFPLS)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLSG)
    ELSEIF (IC == GFP_CCSG%ICOD.AND.LNEIGE) THEN
!           Convective graupel
      IF(LTFPLC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLCG)
    ELSEIF (IC == GFP_CLSH%ICOD.AND.LNEIGE) THEN
!           Stratiform hail
      IF(LTFPLS)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLSH)
    ELSEIF (IC == GFP_CCSH%ICOD.AND.LNEIGE) THEN
!           Convective hail
      IF(LTFPLC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFPLCH)
    ELSEIF (IC == GFP_CUSS%ICOD) THEN
!           Turbulence on U
      IF(LTSTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MSTRTU)
    ELSEIF (IC == GFP_CVSS%ICOD) THEN
!           Turbulence on V
      IF(LTSTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MSTRTV)
    ELSEIF (IC == GFP_CSSH%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFCS)
    ELSEIF (IC == GFP_CSMR%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSOLU)
    ELSEIF (IC == GFP_CSLH%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFCL)
    ELSEIF (IC == GFP_CTSP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MSPRES)
    ELSEIF (IC == GFP_CTCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFNEBT)
    ELSEIF (IC == GFP_CBLD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFDISH)
    ELSEIF (IC == GFP_CSSR%ICOD) THEN
!           Surface solar radiative flux
      IF(LTFR)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=1
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFRSO)
    ELSEIF (IC == GFP_CSTR%ICOD) THEN
!           Surface thermic radiative flux
      IF(LTFR)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=1
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFRTH)
    ELSEIF (IC == GFP_CTSR%ICOD) THEN
!           Top solar radiative flux
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSO)
    ELSEIF (IC == GFP_CTTR%ICOD) THEN
!           Top thermic radiative flux
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRTH)
    ELSEIF (IC == GFP_CCCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MNEBCON)
    ELSEIF (IC == GFP_CHCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MNEBHAU)
    ELSEIF (IC == GFP_CMCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MNEBMOY)
    ELSEIF (IC == GFP_CLCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MNEBBAS)
    ELSEIF (IC == GFP_CUGW%ICOD) THEN
!           Gravity wave drag on U
      IF(LTSTRD)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MSTRDU)
    ELSEIF (IC == GFP_CVGW%ICOD) THEN
!           Gravity wave drag on V
      IF(LTSTRD)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MSTRDV)
    ELSEIF (IC == GFP_CE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFEVL)
    ELSEIF (IC == GFP_CS%ICOD  ) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFEVN)
    ELSEIF (IC == GFP_CLHE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFCLL)
    ELSEIF (IC == GFP_CLHS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFCLN)
    ELSEIF (IC == GFP_CWS%ICOD ) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MWS)
    ELSEIF (IC == GFP_CSNS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MSNS)
    ELSEIF (IC == GFP_CQTO%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MQTOT)
    ELSEIF (IC == GFP_CTO3%ICOD.AND.LOZONE) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MOZONT)
    ELSEIF (IC == GFP_CTME%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFTOPH)
    ELSEIF (IC == GFP_CICE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MQICE)
    ELSEIF (IC == GFP_CLI%ICOD ) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MQLI)
    ELSEIF (IC == GFP_CCVU%ICOD) THEN
!           Convection on U
      IF(LTSTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MSTRCU)
    ELSEIF (IC == GFP_CCVV%ICOD) THEN
!           Convection on V
      IF(LTSTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MSTRCV)
    ELSEIF (IC == GFP_CCVQ%ICOD) THEN
!           Convection on Q
      IF(LTSTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFDICQ)
    ELSEIF (IC == GFP_CCVS%ICOD) THEN
!           Convection on Cp.T
      IF(LTSTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFDICS)
    ELSEIF (IC == GFP_CTUQ%ICOD) THEN
!           Turbulence on Q
      IF(LTSTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFDITQ)
    ELSEIF (IC == GFP_CTUS%ICOD) THEN
!           Turbulence on Cp.T
      IF(LTSTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,INFLEV+MFDITS)
    ELSEIF (IC == GFP_CTSOC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSOC0)
    ELSEIF (IC == GFP_CSOC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSOC1)
    ELSEIF (IC == GFP_CTTHC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRTHC0)
    ELSEIF (IC == GFP_CTHC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRTHC1)
    ELSEIF (IC == GFP_CSOP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSOPS)
    ELSEIF (IC == GFP_CTOP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSOPT)
    ELSEIF (IC == GFP_CSOD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRSODS)
    ELSEIF (IC == GFP_CTHD%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFRTHDS)
    ELSEIF (IC == GFP_CFON%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFONTE)
    ELSEIF (IC == GFP_CCHS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFCHSP)
    ELSEIF (IC == GFP_CEAS%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFLWSP)
    ELSEIF (IC == GFP_CSRU%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MRUISS)
    ELSEIF (IC == GFP_CDRU%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MRUISP)
    ELSEIF (IC == GFP_CIRU%ICOD.AND.LSOLV) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MRUISL)
    ELSEIF (IC == GFP_CETP%ICOD.AND.LSOLV) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFEVV)
    ELSEIF (IC == GFP_CTP%ICOD.AND.LSOLV ) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFTR)
    ELSEIF (IC == GFP_CDUTP%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZGFU(IST:IEND,MFDUTP)
    ELSEIF (IC == GFP_XTCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXNEBT)
    ELSEIF (IC == GFP_X10U%ICOD.AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXUCLS)
    ELSEIF (IC == GFP_X10V%ICOD.AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXVCLS)
    ELSEIF (IC == GFP_X10FF%ICOD.AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=SQRT(&
       & (ZXFU(IST:IEND,MXUCLS)*ZGM(IST:IEND))**2+&
       & (ZXFU(IST:IEND,MXVCLS)*ZGM(IST:IEND))**2)  
    ELSEIF (IC == GFP_X2T%ICOD.AND.LHMTO ) THEN
      IF (IANO == 0) THEN
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTCLS)
      ELSE
!       Interpolation of T2m-Ts
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTCLS)-SP_RR(IST:IEND,YSP_RR%YT%MP0,IBL)
      ENDIF
    ELSEIF (IC == GFP_X2SH%ICOD.AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXQCLS)
    ELSEIF (IC == GFP_X2RH%ICOD.AND.LHMTO) THEN
      IF (IANO == 0) THEN
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXRHCLS)
      ELSEIF(NFPCLI <= 2) THEN
!       Interpolation of HU2m-HUs
        Z1SWSX=1.0_JPRB/WSXI
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXRHCLS) &
         & - 0.5_JPRB*(1.0_JPRB-COS(RPI*SP_RR(IST:IEND,YSP_RR%YW%MP0,IBL)*Z1SWSX))  
      ELSE
!       ISBA option
        LLHMT=.TRUE.
        CLLOCK='ACSOLW '
        CALL ACSOLW(IST,IEND,NPROMA,SD_VV(1,YSD_VV%YARG%MP,IBL),SD_VV(1,YSD_VV%YD2%MP,IBL),          &
         & SD_VF(1,YSD_VF%YLSM%MP,IBL),SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),LLHMT,ZWFC,ZWPMX, &
         & ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
        DO JROF=IST,IEND
          Z1SWSX=1.0_JPRB/(ZWFC(JROF)*GCONV*RD1)
          PFPBUF1(JROF,IFLD)=ZXFU(JROF,MXRHCLS)- 0.5_JPRB &
           & *(1.0_JPRB-COS(RPI*MIN(SP_RR(JROF,YSP_RR%YW%MP0,IBL)*Z1SWSX,1.0_JPRB)))
        ENDDO
      ENDIF
    ELSEIF (IC == GFP_XCCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXNBCON)
    ELSEIF (IC == GFP_XHCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXNBHAU)
    ELSEIF (IC == GFP_XMCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXNBMOY)
    ELSEIF (IC == GFP_XLCC%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXNBBAS)
    ELSEIF (IC == GFP_XX2T%ICOD.AND.LHMTO) THEN
      IF (IANO == 0) THEN
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTX)
      ELSE
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTX)-ZXFU(IST:IEND,MXTCLS)
      ENDIF 
    ELSEIF (IC == GFP_XN2T%ICOD.AND.LHMTO) THEN
      IF (IANO == 0) THEN
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTN)
      ELSE
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTN)-ZXFU(IST:IEND,MXTCLS)
      ENDIF
    ELSEIF (IC == GFP_XX2HU%ICOD.AND.LHMTO) THEN
      IF (IANO == 0) THEN
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXHUX)
      ELSE
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXHUX)-ZXFU(IST:IEND,MXRHCLS)
      ENDIF
    ELSEIF (IC == GFP_XN2HU%ICOD.AND.LHMTO) THEN
      IF (IANO == 0) THEN
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXHUN)
      ELSE
        PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXHUN)-ZXFU(IST:IEND,MXRHCLS)
      ENDIF
    ELSEIF (IC == GFP_INCTX%ICOD .AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTX)-ZXFU(IST:IEND,MXTCLS)
    ELSEIF (IC == GFP_INCTN%ICOD .AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXTN)-ZXFU(IST:IEND,MXTCLS)
    ELSEIF (IC == GFP_INCHN%ICOD .AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXHUN)-ZXFU(IST:IEND,MXRHCLS)
    ELSEIF (IC == GFP_INCHX%ICOD .AND.LHMTO) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXHUX)-ZXFU(IST:IEND,MXRHCLS)
    ELSEIF (IC == GFP_XCVU%ICOD) THEN
!           Convection on U
      IF(LTXTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXTRCU)
    ELSEIF (IC == GFP_XCVV%ICOD) THEN
!           Convection on V
      IF(LTXTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXTRCV)
    ELSEIF (IC == GFP_XCVQ%ICOD) THEN
!           Convection on Q
      IF(LTXTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXDICQ)
    ELSEIF (IC == GFP_XCVS%ICOD) THEN
!           Convection on Cp.T
      IF(LTXTRC)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXDICS)
    ELSEIF (IC == GFP_XTUU%ICOD) THEN
!           Turbulence on U
      IF(LTXTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXTRTU)
    ELSEIF (IC == GFP_XTUV%ICOD) THEN
!           Turbulence on V
      IF(LTXTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXTRTV)
    ELSEIF (IC == GFP_XTUQ%ICOD) THEN
!           Turbulence on Q
      IF(LTXTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXDITQ)
    ELSEIF (IC == GFP_XTUS%ICOD) THEN
!           Turbulence on Cp.T
      IF(LTXTRT)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXDITS)
    ELSEIF (IC == GFP_XGDU%ICOD) THEN
!           Gravity wave drag on U
      IF(LTXTRD)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXTRDU)
    ELSEIF (IC == GFP_XGDV%ICOD) THEN
!           Gravity wave drag on V
      IF(LTXTRD)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=0
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV+MXTRDV)
    ELSEIF (IC == GFP_XLSP%ICOD) THEN
!     Stratiform precipitations
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLSL)
    ELSEIF (IC == GFP_XCP%ICOD ) THEN
!     Convective precipitations
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLCL)
    ELSEIF (IC == GFP_XLSS%ICOD.AND.LNEIGE) THEN
!     Stratiform snow
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLSN)
    ELSEIF (IC == GFP_XCSF%ICOD.AND.LNEIGE) THEN
!     Convective snow
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLCN)
    ELSEIF (IC == GFP_XLSG%ICOD.AND.LNEIGE) THEN
!     Stratiform graupel
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLSG)
    ELSEIF (IC == GFP_XCSG%ICOD.AND.LNEIGE) THEN
!     Convective graupel
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLCG)
    ELSEIF (IC == GFP_XLSH%ICOD.AND.LNEIGE) THEN
!     Stratiform hail
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLSH)
    ELSEIF (IC == GFP_XCSH%ICOD.AND.LNEIGE) THEN
!     Convective hail
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXPLCH)
    ELSEIF (IC == GFP_XSSR%ICOD) THEN
!           Surface solar radiative flux
      IF(LTXR)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=1
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV-1+MXRSO)
    ELSEIF (IC == GFP_XSTR%ICOD) THEN
!           Surface thermic radiative flux
      IF(LTXR)THEN
        INFLEV=NFLEVG
      ELSE
        INFLEV=1
      ENDIF
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,INFLEV-1+MXRTH)
    ELSEIF (IC == GFP_XTSR%ICOD) THEN
!           Top solar radiative flux
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXRSO)
    ELSEIF (IC == GFP_XTTR%ICOD) THEN
!           Top thermic radiative flux
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXRTH)
    ELSEIF (IC == GFP_XCAPE%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXCAPE)
    ELSEIF (IC == GFP_XMOCO%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXMOCON)
    ELSEIF (IC == GFP_XCLPH%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXCLPH)
    ELSEIF (IC == GFP_XUGST%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXUGST)
    ELSEIF (IC == GFP_XVGST%ICOD) THEN
      PFPBUF1(IST:IEND,IFLD)=ZXFU(IST:IEND,MXVGST)
    ELSEIF (IC == GFP_UCUR%ICOD) THEN
!     u-component of ocean current.
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YUCUR%MP,IBL)
    ELSEIF (IC == GFP_VCUR%ICOD) THEN
!     v-component of ocean current.
      PFPBUF1(IST:IEND,IFLD)=SD_VF(IST:IEND,YSD_VF%YVCUR%MP,IBL)

    ELSE
      PFPBUF1(IST:IEND,IFLD)=0.0_JPRB
    ENDIF

!   Tensors aspects :
    IORDER=ABS(GFP_PHYDS(IC)%IORDR)
    IF (IORDER == 1) THEN
      PARFP1(IFLD)=-1.0_JPRB
    ELSE
      PARFP1(IFLD)=+1.0_JPRB
    ENDIF
    IF (LFPOSHOR .AND. IORDER > 0) THEN
      SELECT CASE (IORDER)
      CASE (1)
!         Vectors => multiply by map factor if actual horizontal interpolations
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=PFPBUF1(JI,IFLD)*ZGM(JI)
        ENDDO
      CASE (2)
!         2nd order derivatives => multiply by square map factor
!         if actual horizontal interpolations
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=PFPBUF1(JI,IFLD)*ZGM2(JI)
        ENDDO
      CASE (3:)
        DO JI=IST,IEND
          PFPBUF1(JI,IFLD)=PFPBUF1(JI,IFLD)*ZGM(JI)**IORDER
        ENDDO
      END SELECT
    ENDIF

  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('HPOS',1,ZHOOK_HANDLE)
END SUBROUTINE HPOS
