SUBROUTINE INI3WRFP(CDFPFN,CDLEC,KUNIT,KBARP,KFPCT0)

!**** *INI3WRFP*  - PREPARE TO OPEN A SET OF FILES ARPEGE/ALADIN FOR
!                   POST-PROCESSING PURPOSE

!     PURPOSE.
!     --------
!           To compute the variables needed before opening files for 
!           post-processing purpose

!**   INTERFACE.
!     ----------
!       *CALL* *INI3WRFP(CDFPFN,CDLEC,KUNIT,KBARP,KFPCT0)*

!        EXPLICIT ARGUMENTS
!        --------------------
!            CDFPFN : complete filename
!            CDLEC  : file framework
!            KUNIT  : logical unit number of output files
!            KBARP  : expected number of fields to be written out
!            KFPCT0 : Kind of call to CNT0 => kind of file to set up
!                     0 or 1  => Output post-processing file
!                     -1      => Intermediate post-processing file "PX"
!                     2 or -2 => Background file for bogussing
!                     -9  => output climatology

!        IMPLICIT ARGUMENTS
!        --------------------
!         See modules above

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 98-12-31  from wrhfpsm/dm
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!    R. El Khatib : 04-08-10 Use an explicit granularity factor
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NPPPSH   ,NULFP01  ,NULFP02  ,&
 & NULFP03 ,NULFP04  ,NULFP05  ,NULFP06  ,NULFP07  ,&
 & NULFP08 ,NULFP09  ,NULFP10  ,NULFP11  ,NULFP12  ,&
 & NULFP13 ,NULFP14  ,NULFP15  
USE PARFPOS  , ONLY : JPOSDOM
USE YOMFPC   , ONLY : NFPDOM   ,CFPFMT  ,CFPDOM
USE YOMDYN   , ONLY : TSTEP
USE YOMCT3   , ONLY : NSTEP
USE YOMOP    , ONLY : CFNPL    ,LINC
USE YOMFPOP  , ONLY : CFPFN    ,CFPCA    ,CFPFNBG  ,CFPCABG
USE YOM4FPOS , ONLY : NFPOSPHY
USE YOMFP4L  , ONLY : NDYNFPL  ,NFPSPBG
USE YOMFPF   , ONLY : NFPGLFI

IMPLICIT NONE

CHARACTER(LEN=*)  ,INTENT(OUT)   :: CDFPFN(NFPDOM) 
CHARACTER(LEN=*)  ,INTENT(OUT)   :: CDLEC(NFPDOM) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KUNIT(NFPDOM) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KBARP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPCT0 
CHARACTER :: CLINC*4

!     CLINC  : output filename increment

INTEGER(KIND=JPIM) :: IULFP(JPOSDOM)

INTEGER(KIND=JPIM) :: IINC, ILEN, JD, IREP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*    0. Preparations

IF (LHOOK) CALL DR_HOOK('INI3WRFP',0,ZHOOK_HANDLE)
IF (ABS(KFPCT0) /= 2) THEN
  IULFP(01)=NULFP01
  IULFP(02)=NULFP02
  IULFP(03)=NULFP03
  IULFP(04)=NULFP04
  IULFP(05)=NULFP05
  IULFP(06)=NULFP06
  IULFP(07)=NULFP07
  IULFP(08)=NULFP08
  IULFP(09)=NULFP09
  IULFP(10)=NULFP10
  IULFP(11)=NULFP11
  IULFP(12)=NULFP12
  IULFP(13)=NULFP13
  IULFP(14)=NULFP14
  IULFP(15)=NULFP15
ELSE
  IULFP(01)=NULFP15
  IULFP(02)=NULFP14
  IULFP(03)=NULFP13
  IULFP(04)=NULFP12
  IULFP(05)=NULFP11
  IULFP(06)=NULFP10
  IULFP(07)=NULFP09
  IF (NFPDOM > 7) THEN
!   Needs more free unit number ... but it should never happen !
    CALL ABOR1('INI3WRFP : INTERNAL ERROR ON IULFP')
  ENDIF
ENDIF
IF (LINC) THEN
  IINC=NINT(REAL(NSTEP,JPRB)*TSTEP/3600._JPRB)
ELSE
  IINC=NSTEP
ENDIF
WRITE(CLINC,'(I4.4)') IINC

!*    1. Set file unit numbers & names

IF (KFPCT0 == -1) THEN
  KUNIT (1)=NPPPSH
  CDFPFN(1)=CFNPL
  CDLEC (1)=CFPCA(1)
ELSEIF (KFPCT0 == 0.OR.KFPCT0 == 1) THEN
  DO JD=1,NFPDOM
    ILEN=INDEX(CFPFN(JD),' ')-1
    CDFPFN(JD)=CFPFN(JD)(1:ILEN)//'+'//CLINC
    KUNIT (JD)=IULFP(JD)
    CDLEC (JD)=CFPCA(JD)
  ENDDO
ELSEIF (ABS(KFPCT0) == 2) THEN
  DO JD=1,NFPDOM
    CDFPFN(JD)=CFPFNBG(JD)
    KUNIT (JD)=IULFP(JD)
    CDLEC (JD)=CFPCABG(JD)
  ENDDO
ELSEIF (KFPCT0 == -9) THEN
  DO JD=1,NFPDOM
    ILEN=INDEX(CFPDOM(JD)//' ',' ')-1
    CDFPFN(JD)='const.clim.'//CFPDOM(JD)(1:ILEN)
    KUNIT (JD)=IULFP(JD)
    CDLEC (JD)='CLIM.'//CFPDOM(JD)
  ENDDO
ELSE
  CALL ABOR1('INI3WRFP : INTERNAL ERROR ON KFPCT0')
ENDIF

!*    2. Granularity factor

DO JD=1,NFPDOM
  CALL LFIAFM(IREP,KUNIT(JD),NFPGLFI)
ENDDO

!*    3. Miscellaneous

IF (CFPFMT == 'MODEL') THEN
  KBARP = NFPSPBG+NDYNFPL+NFPOSPHY
ELSE
  KBARP = NDYNFPL+NFPOSPHY
ENDIF

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('INI3WRFP',1,ZHOOK_HANDLE)
END SUBROUTINE INI3WRFP
