SUBROUTINE SPECFITA

!**** *SPECFITA*  - spectral fit of dynamic fields from FA.
!                     and altitude correction on surface fields

!     Purpose.
!     --------
!        To spectrally fit the grid-point pre-processed dynamic data
!        and store all model fields in the appropriate model arrays.
!        Altitude correction is done on surface fields when relevent.

!**   Interface.
!     ----------
!        *CALL* *SPECFITA*

!        Explicit arguments :
!        ------------------
!               None

!        Implicit arguments :
!        ------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        ABOR1
!        Spectral transforms routines
!           REESPE,EREESPE,SPEREE,ESPEREE,UVSPE

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        K. YESSAD *METEO-FRANCE* after routines
!         SPECFITASM (prev. named SPECFITA) of RYAD EL KHATIB *METEO-FRANCE*
!         SUSPECADM and WRMLPPA of MPP VALPARAISO team (*METEO-FRANCE*)

!     Modifications.
!     --------------
!        ORIGINAL : 97-06-12
!        Modified by K. YESSAD (OCT 1997): residual bugs corrections in DM code.
!        R. El Khatib : 97-12-24 : fully rewritten for efficiency of DM
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - removal of .FALSE. in 'CALL SPEREE'.
!        R. El Khatib : 98-08-31 : adiabatic pp.
!        H. HADDOUCH : 99-11-26 : Ozone pp.
!        R. El Khatib : 00-08-18 derived types
!        R. El Khatib : 01-08-07 Pruning options
!        R. El Khatib : 02-02-19 Substantial re-write to escape a MP barrier
!        R. El Khatib : 03-08-06 gfl
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        R. El Ouaraini & R. El Khatib : 05-07-28 Add CUF fields.
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        P. Marquet    22 fev 2008 : set ZMAXPHI=4000._JPRB*RG (previous value 3000._JPRB*RG)
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RG
USE YOMSTA   , ONLY : RDTDZ1
USE YOMLUN   , ONLY : NULOUT
USE YOMXFU   , ONLY : LXFU     ,LXCLS    ,LXTTCLS
USE YOMDIM   , ONLY : NSPEC2   ,NFLEVG   ,NPROMA   ,NGPBLKS
USE SURFACE_FIELDS_MIX, ONLY : SP_SB,  YSP_SB, SP_RR, YSP_RR, SD_VF, YSD_VF
USE YOMCT0   , ONLY : LELAM    ,LSPRT, LAROME, NCONF, LSCREEN 
USE YOMGEM   , ONLY : NGPTOT
USE YOMAFN   , ONLY : TFP_LNSP   ,TFP_FIS  ,TFP_U    ,TFP_V    ,TFP_T    ,TFP_WWS  ,&
 & TFP_PD  ,TFP_VW  ,TFP_DYNDS  ,TFP_CUF1  ,TFP_CUF2  ,TFP_CUF3  ,TFP_CUF4  ,TFP_CUF5
USE YOMPHY   , ONLY : LMPHYS
USE YOMGC    , ONLY :        OROG
USE YOMXFUB  , ONLY : XFUBUF
USE PTRXFU   , ONLY : MXTN    ,MXTX     ,MXTCLS
USE YOMCFU   , ONLY : LCUMFU
USE YOM_YGFL , ONLY : YGFL    ,YGFLC
USE YOMGFL   , ONLY : GFL     ,GFLT1
USE YOMGMV   , ONLY : GMVT1   ,GMVT1S   ,YT1
USE PARFPOS  , ONLY : JPOSDYN
USE YOMMCUF  , ONLY : RMCUFGP1  ,RMCUFGP2  ,RMCUFGP3  ,RMCUFGP4  ,RMCUFGP5 

IMPLICIT NONE

CHARACTER(LEN=9) :: CLCONF
CHARACTER(LEN=4),  ALLOCATABLE :: CLPREF(:)
CHARACTER(LEN=12), ALLOCATABLE :: CLSUFF(:)

LOGICAL :: LLSPOR

INTEGER(KIND=JPIM) :: IPT, IFILE, JL, JI, JGFL, IEND, IGFL, JF, JBLK, IOFF, IBLK
INTEGER(KIND=JPIM) :: JKGLO, IST ,JROF

INTEGER(KIND=JPIM) :: IGPSP   ! pointers for gridpoint Ln(Ps)
INTEGER(KIND=JPIM) :: IGPU    ! pointers for gridpoint zonal wind
INTEGER(KIND=JPIM) :: IGPV    ! pointers for gridpoint meridional wind
INTEGER(KIND=JPIM) :: IGPT    ! pointers for gridpoint temperature
INTEGER(KIND=JPIM) :: IGPPD   ! pointers for gridpoint NH pressure departure
INTEGER(KIND=JPIM) :: IGPVD   ! pointers for gridpoint NH vertical divergence
INTEGER(KIND=JPIM) :: IGPWW   ! pointers for gridpoint NH true surface velocity
INTEGER(KIND=JPIM) :: IGPCUF1 ! pointers for gridpoint filtred ln(Ps), NCUFNR=1
INTEGER(KIND=JPIM) :: IGPCUF2 ! pointers for gridpoint filtred ln(Ps), NCUFNR=2
INTEGER(KIND=JPIM) :: IGPCUF3 ! pointers for gridpoint filtred ln(Ps), NCUFNR=3
INTEGER(KIND=JPIM) :: IGPCUF4 ! pointers for gridpoint filtred ln(Ps), NCUFNR=4
INTEGER(KIND=JPIM) :: IGPCUF5 ! pointers for gridpoint filtred ln(Ps), NCUFNR=5
INTEGER(KIND=JPIM) :: IGPOR   ! pointers for gridpoint interpolated orography
INTEGER(KIND=JPIM) :: IFLDGP  ! number of gridpoint pseudo-spectral fields to read

INTEGER(KIND=JPIM) :: IPPMP(JPOSDYN) ! pointer of pp fields in gridpoint array
INTEGER(KIND=JPIM), ALLOCATABLE :: ILEV(:)

! ZGPOR, ZFTOR : not fitted, fitted interpolated surface geopotential
! ZGPFI : not fitted output surface geopotential (fitted one is OROG)

REAL(KIND=JPRB) :: ZDIFMAX, ZDTSGDZ, ZMAXPHI
REAL(KIND=JPRB) :: ZGPOR(NGPTOT), ZFTOR(NGPTOT), ZGPFI(NGPTOT)
REAL(KIND=JPRB) :: ZDIFINT(NGPTOT), ZDIFOUT(NGPTOT), ZDIF(NGPTOT)
REAL(KIND=JPRB) :: ZREEL(NGPTOT), ZSPEC(NSPEC2)
REAL(KIND=JPRB) :: ZXFU(NPROMA,XFUBUF%IFIELDS)

REAL(KIND=JPRB), ALLOCATABLE :: ZGPFIELDS(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ereespe.intfb.h"
#include "especrt.intfb.h"
#include "esperee.intfb.h"
#include "etransdirh.intfb.h"
#include "rdpxfa.intfb.h"
#include "reespe.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"
#include "specrt.intfb.h"
#include "speree.intfb.h"
#include "sugrcfu.intfb.h"
#include "sugridf.intfb.h"
#include "sugrxfu.intfb.h"
#include "suorog.intfb.h"
#include "transdirh.intfb.h"

!      -----------------------------------------------------------

!*       1.    SETUP DIMENSIONS AND ALLOCATE DESCRIPTORS ARRAYS
!              ------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPECFITA',0,ZHOOK_HANDLE)

IFLDGP=0
DO JF=1,JPOSDYN
  IFLDGP=IFLDGP+TFP_DYNDS(JF)%ILMOD
ENDDO

ALLOCATE(CLPREF(IFLDGP),CLSUFF(IFLDGP),ILEV(IFLDGP))
IF (.NOT. ALLOCATED(RMCUFGP1))  ALLOCATE(RMCUFGP1(1:NGPTOT))
IF (.NOT. ALLOCATED(RMCUFGP2))  ALLOCATE(RMCUFGP2(1:NGPTOT))
IF (.NOT. ALLOCATED(RMCUFGP3))  ALLOCATE(RMCUFGP3(1:NGPTOT))
IF (.NOT. ALLOCATED(RMCUFGP4))  ALLOCATE(RMCUFGP4(1:NGPTOT))
IF (.NOT. ALLOCATED(RMCUFGP5))  ALLOCATE(RMCUFGP5(1:NGPTOT))

!*       1.    SETUP DESCRIPTORS AND POINTERS
!              ------------------------------

IPT=0
DO JF=1,JPOSDYN
  SELECT CASE (TFP_DYNDS(JF)%ILMOD)
  CASE (1) ! 2D primitive field for the post-processing
    IPPMP(JF)=IPT+1      
    IPT=IPT+1
    IF (IPT > IFLDGP) CALL ABOR1('SPECFITA : ERROR IPT')
    CLPREF(IPT)=TFP_DYNDS(JF)%CLNAME(1:4)
    CLSUFF(IPT)=TFP_DYNDS(JF)%CLNAME(5:)
    ILEV  (IPT)=1
  CASE (2:) ! 3D primitive field for the post-processing
    IPPMP(JF)=IPT+1      
    DO JL=1,NFLEVG
      IPT=IPT+1
      IF (IPT > IFLDGP) CALL ABOR1('SPECFITA : ERROR IPT')
      CLPREF(IPT)='S   '
      CLSUFF(IPT)=TFP_DYNDS(JF)%CLNAME(1:12)
      ILEV  (IPT)=JL
    ENDDO
  CASE DEFAULT
    IPPMP(JF)=0 
  END SELECT
ENDDO

! Specific pointers needed because of specfic treatments :
IGPOR  =IPPMP(TFP_FIS%ICOD)
IGPSP  =IPPMP(TFP_LNSP%ICOD)
IGPU   =IPPMP(TFP_U%ICOD)
IGPV   =IPPMP(TFP_V%ICOD)
IGPT   =IPPMP(TFP_T%ICOD)
IGPPD  =IPPMP(TFP_PD%ICOD)
IGPVD  =IPPMP(TFP_VW%ICOD)
IGPWW  =IPPMP(TFP_WWS%ICOD)
IGPCUF1=IPPMP(TFP_CUF1%ICOD)
IGPCUF2=IPPMP(TFP_CUF2%ICOD)
IGPCUF3=IPPMP(TFP_CUF3%ICOD)
IGPCUF4=IPPMP(TFP_CUF4%ICOD)
IGPCUF5=IPPMP(TFP_CUF5%ICOD)

!*       2.    READ AND DISTRIBUTE GRID-POINT PRE-PROCESSED FIELDS
!              ---------------------------------------------------

IFILE=5
IST=1

!        2.1   Read pseudo-spectral fields

ALLOCATE(ZGPFIELDS(NGPTOT,IFLDGP))
CALL RDPXFA(ZGPFIELDS,CLPREF,ILEV,CLSUFF,IFLDGP,IFILE)
DEALLOCATE(CLPREF,CLSUFF,ILEV)
ZGPOR(1:NGPTOT)=ZGPFIELDS(1:NGPTOT,IGPOR)

!        2.2   Surface gridpoint fields

LLSPOR=.FALSE.
CALL SUGRIDF(IFILE,ZSPEC,LLSPOR)
DO JKGLO=1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBLK=(JKGLO-1)/NPROMA+1
  DO JI=IST,IEND
    ZGPFI(JI+JKGLO-1)=SD_VF(JI,YSD_VF%YFP1%MP,IBLK)
  ENDDO
ENDDO

IF (LCUMFU) THEN
  CALL SUGRCFU(IFILE)
ENDIF

IF (LXFU) THEN
  CALL SUGRXFU(IFILE)
ENDIF

!*       3.    PRE-COMPUTATIONS
!              ----------------

!*       3.2     Convert NH half level vertical velocity into pseudo-vertical divergence

IF (IGPWW > 0 .AND. IGPVD > 0) THEN
  DO JL=IGPVD+1,IGPVD+NFLEVG-1
    DO JROF=1,NGPTOT
      ZREEL(JROF)=-RG*(ZGPFIELDS(JROF,JL)-ZGPFIELDS(JROF,JL-1))
      ZGPFIELDS(JROF,JL-1)=ZREEL(JROF)
    ENDDO
  ENDDO
  DO JROF=1,NGPTOT
    ZREEL(JROF)=-RG*(ZGPFIELDS(JROF,IGPWW)-ZGPFIELDS(JROF,IGPVD+NFLEVG-1))
    ZGPFIELDS(JROF,IGPVD+NFLEVG-1)=ZREEL(JROF)
  ENDDO
ENDIF

!*       4.    SPECTRAL FITS
!              -------------

!        4.1 gmv

IF(.NOT.ALLOCATED(GMVT1))  ALLOCATE(GMVT1 (NPROMA,NFLEVG,YT1%NDIM,NGPBLKS))
IF(.NOT.ALLOCATED(GMVT1S)) ALLOCATE(GMVT1S(NPROMA,YT1%NDIMS,NGPBLKS))
GMVT1 (:,:,:,:)=0.0_JPRB
GMVT1S(:,:,:)  =0.0_JPRB

DO JBLK=1,NGPBLKS
  IOFF=(JBLK-1)*NPROMA
  DO JL=1,NFLEVG
    DO JI=1,MIN(NPROMA,NGPTOT-IOFF)
      IF (IGPU > 0) GMVT1(JI,JL,YT1%MU,JBLK)=ZGPFIELDS(JI+IOFF,IGPU+JL-1)
      IF (IGPV > 0) GMVT1(JI,JL,YT1%MV,JBLK)=ZGPFIELDS(JI+IOFF,IGPV+JL-1)
      IF (IGPT > 0) GMVT1(JI,JL,YT1%MT,JBLK)=ZGPFIELDS(JI+IOFF,IGPT+JL-1)
      IF (IGPPD > 0) GMVT1(JI,JL,YT1%MSPD,JBLK)=ZGPFIELDS(JI+IOFF,IGPPD+JL-1)
      IF (IGPVD > 0) GMVT1(JI,JL,YT1%MSVD,JBLK)=ZGPFIELDS(JI+IOFF,IGPVD+JL-1)
    ENDDO
  ENDDO
  DO JI=1,MIN(NPROMA,NGPTOT-IOFF)
    IF (IGPSP > 0)   GMVT1S(JI,YT1%MSP,JBLK)=ZGPFIELDS(JI+IOFF,IGPSP)
  ENDDO
ENDDO
IF (IGPCUF1 > 0) RMCUFGP1(1:NGPTOT)=ZGPFIELDS(1:NGPTOT,IGPCUF1)
IF (IGPCUF2 > 0) RMCUFGP2(1:NGPTOT)=ZGPFIELDS(1:NGPTOT,IGPCUF2)
IF (IGPCUF3 > 0) RMCUFGP3(1:NGPTOT)=ZGPFIELDS(1:NGPTOT,IGPCUF3)
IF (IGPCUF4 > 0) RMCUFGP4(1:NGPTOT)=ZGPFIELDS(1:NGPTOT,IGPCUF4)
IF (IGPCUF5 > 0) RMCUFGP5(1:NGPTOT)=ZGPFIELDS(1:NGPTOT,IGPCUF5)

!        4.2 gfl 

IF(.NOT.ALLOCATED(GFLT1))  ALLOCATE(GFLT1(NPROMA,NFLEVG,YGFL%NDIM1,NGPBLKS))
IF(.NOT.ALLOCATED(GFL  ))  ALLOCATE(GFL  (NPROMA,NFLEVG,YGFL%NDIM ,NGPBLKS))
GFLT1(:,:,:,:)=0.0_JPRB
GFL  (:,:,:,:)=0.0_JPRB

DO JF=1,JPOSDYN
  SELECT CASE (TFP_DYNDS(JF)%ILMOD)
  CASE (1:) 
    DO JGFL=1,YGFL%NUMFLDS
      IF (TFP_DYNDS(JF)%CLNAME == YGFLC(JGFL)%CNAME) THEN
        DO JBLK=1,NGPBLKS
          IOFF=(JBLK-1)*NPROMA
          DO JL=1,NFLEVG
            IGFL=IPPMP(JF)+JL-1
            IF (YGFLC(JGFL)%LSP) THEN
              DO JI=1,MIN(NPROMA,NGPTOT-IOFF)
                GFLT1(JI,JL,YGFLC(JGFL)%MP1,JBLK)=ZGPFIELDS(JI+IOFF,IGFL)
              ENDDO
            ELSEIF(YGFLC(JGFL)%LGP) THEN
              DO JI=1,MIN(NPROMA,NGPTOT-IOFF)
                GFL(JI,JL,YGFLC(JGFL)%MP,JBLK)=ZGPFIELDS(JI+IOFF,IGFL)
              ENDDO
            ELSE
              CALL ABOR1('SPECFITA : GFL NEITHER SP NOR GP')
            ENDIF
          ENDDO
        ENDDO
      ENDIF
    ENDDO
  END SELECT
ENDDO

DEALLOCATE(ZGPFIELDS)

IF (LELAM) THEN
  CLCONF(8:8)='U'
  CALL ETRANSDIRH(CLCONF)
ELSE
  CLCONF(8:8)='A'
  CALL TRANSDIRH(CLCONF)
ENDIF
! Deallocations of GMVT1,GMVT1S,GFLT1 are performed inside (E)TRANSDIRH
! under condition with OPENMP

!        4.2   Interpolated & output orographies

ZREEL(:)=ZGPOR(:)
IF (LELAM) THEN
  CALL EREESPE(1,1,ZSPEC,ZREEL)
  CALL ESPEREE(1,1,ZSPEC,ZFTOR)
  ZREEL(:)=ZGPFI(:)  
  CALL EREESPE(1,1,ZSPEC,ZREEL)
ELSE
  CALL REESPE(1,1,ZSPEC,ZREEL)
  CALL SPEREE(1,1,ZSPEC,ZFTOR)
  ZREEL(:)=ZGPFI(:)
  CALL REESPE(1,1,ZSPEC,ZREEL)
ENDIF
CALL SUOROG(ZSPEC)

ZDTSGDZ=RDTDZ1/RG
ZMAXPHI=4000._JPRB*RG

ZDIFINT(:)=ZFTOR(:)-ZGPOR(:)
ZDIFMAX=0.0_JPRB
DO JI=1,NGPTOT
  ZDIFMAX=MAX(ZDIFMAX,ABS(ZDIFINT(JI)))
ENDDO
WRITE (NULOUT,FMT='(''MAXIMUM LOCAL DIFFERENCE BETWEEN FITTED AND '',&
 & ''UNFITTED INTERPOLATED OROGRAPHIES : '',F9.2, '' J/kg '')') ZDIFMAX  
WRITE (NULOUT,FMT='('' ALLOWED DIFFERENCE : '',F9.2,'' J/kg '')') ZMAXPHI
IF (ZDIFMAX > ZMAXPHI) THEN
  WRITE (NULOUT,FMT='('' TOO MUCH DIFFERENCE '')')
  CALL ABOR1('SPECFITA : ABOR1 CALLED')
ENDIF
DO JROF=1,NGPTOT
  ZDIFINT(JROF)=ZDTSGDZ*ZDIFINT(JROF)
ENDDO

DO JROF=1,NGPTOT
  ZDIFOUT(JROF)=OROG(JROF)-ZGPFI(JROF)
ENDDO
ZDIFMAX=0.0_JPRB
DO JI=1,NGPTOT
  ZDIFMAX=MAX(ZDIFMAX,ABS(ZDIFOUT(JI)))
ENDDO
WRITE (NULOUT,FMT='(''MAXIMUM LOCAL DIFFERENCE BETWEEN FITTED AND '',&
 & ''UNFITTED OUTPUT OROGRAPHIES : '',F9.2, '' J/kg '')') ZDIFMAX  
WRITE (NULOUT,FMT='('' ALLOWED DIFFERENCE : '',F9.2,'' J/kg '')') ZMAXPHI
IF (ZDIFMAX > ZMAXPHI) THEN
  WRITE (NULOUT,FMT='('' TOO MUCH DIFFERENCE '')')
  CALL ABOR1('SPECFITA : ABOR1 CALLED')
ENDIF
DO JROF=1,NGPTOT
  ZDIFOUT(JROF)=ZDTSGDZ*ZDIFOUT(JROF)
ENDDO

DO JROF=1,NGPTOT
  ZDIF(JROF)=ZFTOR(JROF)-OROG(JROF)
ENDDO
ZDIFMAX=0.0_JPRB
DO JI=1,NGPTOT
  ZDIFMAX=MAX(ZDIFMAX,ABS(ZDIF(JI)))
ENDDO
WRITE (NULOUT,FMT='(''MAXIMUM LOCAL DIFFERENCE BETWEEN FITTED OUTPUT'',&
 & '' AND FITTED INTERPOLATED OROGRAPHIES :  '',F9.2, '' J/kg '')') ZDIFMAX  

!        4.3   Vertical corrections for temperatures and save corrected interpolated orography

DO JKGLO=1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBLK=(JKGLO-1)/NPROMA+1
  DO JI=IST,IEND
    SD_VF(JI,YSD_VF%YFP1%MP,IBLK)=ZFTOR(JI+JKGLO-1)
  ENDDO
  IF (LMPHYS) THEN
    DO JI=IST,IEND
      SP_RR(JI,YSP_RR%YFP1%MP,IBLK) = SP_RR(JI,YSP_RR%YFP1%MP,IBLK)+ZDIFINT(JI+JKGLO-1)
      SP_RR(JI,YSP_RR%YT%MP,IBLK)   = SP_RR(JI,YSP_RR%YT%MP,IBLK)  +ZDIFOUT(JI+JKGLO-1)
    ENDDO
  ENDIF
  IF (LMPHYS.AND.((.NOT.LAROME).OR.(LAROME.AND.(LSCREEN.OR.(NCONF == 131))))) THEN
    DO JI=IST,IEND
      SP_SB(JI,1,YSP_SB%YT%MP,IBLK) = SP_SB(JI,1,YSP_SB%YT%MP,IBLK)+ZDIFOUT(JI+JKGLO-1)
    ENDDO
  ENDIF
  IF (LXCLS.OR.LXTTCLS) THEN
    CALL SC2RDG(XFUBUF,JKGLO,IST,IEND,ZXFU)
    IF (LXCLS) THEN
      DO JI=IST,IEND
        ZXFU(JI,MXTCLS)=ZXFU(JI,MXTCLS)+ZDIFOUT(JI+JKGLO-1)
      ENDDO
    ENDIF
    IF (LXTTCLS) THEN
      DO JI=IST,IEND
        ZXFU(JI,MXTX)=ZXFU(JI,MXTX)+ZDIFOUT(JI+JKGLO-1)
        ZXFU(JI,MXTN)=ZXFU(JI,MXTN)+ZDIFOUT(JI+JKGLO-1)
      ENDDO
    ENDIF
    CALL SC2WRG(XFUBUF,JKGLO,IST,IEND,ZXFU)
  ENDIF
ENDDO

!*       5.    COMPUTE RT FROM T AND Q
!              -----------------------

IF (LSPRT) THEN
  IF (LELAM) THEN
    CALL ESPECRT(0)
  ELSE
    CALL SPECRT(0)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPECFITA',1,ZHOOK_HANDLE)
END SUBROUTINE SPECFITA
