SUBROUTINE SU4FPOS(KTIME)

!**** *SU4FPOS*  - INITIALIZE FULL-POS level 4

!     PURPOSE.
!     --------
!        INITIALIZE COMMON BLOCKS YOMFP4 AND PTRFP4, WHICH CONTAIN THE REQUESTS
!           FOR POST-PROCESSING AT A GIVEN TIME STEP.

!**   INTERFACE.
!     ----------
!       *CALL* *SU4FPOS*

!        EXPLICIT ARGUMENTS
!        --------------------
!        KTIME  : time in model

!        IMPLICIT ARGUMENTS
!        --------------------
!        NONE.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!    R. El Khatib : 98-04-10 : simplifications
!    D.Dent        :99-05-06 More flexible FULL-POS post-processing options
!    D.Dent        :00-01-10 put prints under LTRACEFP switch
!    R. El Khatib : 00-08-18 derived types
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMAFN   , ONLY : TFP_DYNDS
USE YOMFPC   , ONLY : CFPDOM   ,RFP3P    ,RFP3H    ,RFP3TH   ,RFP3PV   ,&
 & NRFP3S   ,LTRACEFP ,LFPCNT  
USE YOM4FPOS , ONLY :     NOFPP    ,NOFPH    , &
 & NOFPS    ,NOFPPV   ,NOFPTH   ,NCPFP    ,NCHFP    ,NCTFP    , &
 & NCVFP    ,NCSFP    ,NAFPP    ,NAFPH    ,NAFPT    ,NAFPV    , &
 & NAFPS    ,NIPFP    ,NIHFP    ,NITFP    ,NIVFP    ,NISFP    , &
 & NPFP     ,NHFP     ,NTFP     ,NVFP     ,NSFP     ,NOFP2A   , &
 & NC2FP    ,NI2FP    ,NOFP2F   ,NTFPP    ,NTFPH    ,NTFPTH   , &
 & NTFPPV   ,NTFPS  
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : NPRINTLEV

IMPLICIT NONE

INTEGER(KIND=JPIB),INTENT(IN)    :: KTIME 
CHARACTER(LEN=16)  :: CL
CHARACTER(LEN=120) :: CLYFILE

INTEGER(KIND=JPIM) :: ILEV, JD, JFLD, JLEV

LOGICAL :: LLNOPPFIL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ppreq.intfb.h"
#include "ppreset.intfb.h"
#include "sufpdyn.intfb.h"
#include "sufpphy.intfb.h"

!*       1. INITIALIZE REQUESTS ON PHYSICS AND FLUXES
!           -----------------------------------------

IF (LHOOK) CALL DR_HOOK('SU4FPOS',0,ZHOOK_HANDLE)
LLNOPPFIL=.TRUE.
IF (LECMWF) THEN
  CALL PPRESET(KTIME,LLNOPPFIL,CLYFILE)
ELSE
  IF (LFPCNT) THEN
    CALL PPREQ(KTIME,LLNOPPFIL,CLYFILE)
  ENDIF
ENDIF

!*       1.2 PHYSICAL FIELDS

CALL SUFPPHY(LLNOPPFIL,CLYFILE)

!*       2. INITIALIZE REQUESTS ON DYNAMICS
!           -------------------------------

CALL SUFPDYN(NTFPP,NTFPH,NTFPTH,NTFPPV,NTFPS,NOFP2F,&
 & NOFPP,NOFPH,NOFPTH,NOFPPV,NOFPS,&
 & NAFPP,NAFPH,NAFPT,NAFPV,NAFPS,NOFP2A,&
 & NCPFP,NCHFP,NCTFP,NCVFP,NCSFP,NC2FP,&
 & NPFP,NHFP,NTFP,NVFP,NSFP,&
 & NIPFP,NIHFP,NITFP,NIVFP,NISFP,NI2FP,&
 & LLNOPPFIL,CLYFILE)  

!*       3. PRINT OUT REQUESTS
!           ------------------

IF (LTRACEFP .AND. NPRINTLEV >= 2 .AND. &
   & (NTFPP+NTFPH+NTFPTH+NTFPPV+NTFPS+NOFP2F) > 0) THEN  

  WRITE(NULOUT,*) ' THE FOLLOWING DYNAMIC FIELDS WILL BE POST_PROCESSED ', &
   & 'AT THE CURRENT TIME STEP :'  

!        Post-processing on P levels :
  DO JFLD=1,NTFPP
    DO JLEV=1,NOFPP(JFLD)
      IF (RFP3P(NPFP(JLEV,JFLD)) < 100000._JPRB) THEN
        ILEV=NINT(RFP3P(NPFP(JLEV,JFLD))/100._JPRB)
      ELSE
        ILEV=NINT(RFP3P(NPFP(JLEV,JFLD))-100000._JPRB)
      ENDIF
      WRITE(NULOUT,FMT='(''P'',I3.3,A12,''/'',10(1X,A7))')&
       & ILEV,TFP_DYNDS(NCPFP(JFLD))%CLNAME(1:12),&
       & (CFPDOM(NIPFP(JD,JLEV,JFLD)),JD=1,NAFPP(JLEV,JFLD))  
    ENDDO
  ENDDO

!        Post-processing on H levels :
  CL(1:1)='H'
  DO JFLD=1,NTFPH
    CL(5:)=TFP_DYNDS(NCHFP(JFLD))%CLNAME(1:12)
    DO JLEV=1,NOFPH(JFLD)
      WRITE(UNIT=CL(2:4),FMT='(I3.3)') NINT(RFP3H(NHFP(JLEV,JFLD)))
      WRITE(NULOUT,FMT='(A16,''/'',10(1X,A7))')&
       & CL,(CFPDOM(NIHFP(JD,JLEV,JFLD)),JD=1,NAFPH(JLEV,JFLD))  
    ENDDO
  ENDDO

  CL(1:1)='T'
  DO JFLD=1,NTFPTH
    CL(5:)=TFP_DYNDS(NCTFP(JFLD))%CLNAME(1:12)
    DO JLEV=1,NOFPTH(JFLD)
      WRITE(UNIT=CL(2:4),FMT='(I3.3)') NINT(RFP3TH(NTFP(JLEV,JFLD)))
      WRITE(NULOUT,FMT='(A16,''/'',10(1X,A7))')&
       & CL,(CFPDOM(NITFP(JD,JLEV,JFLD)),JD=1,NAFPT(JLEV,JFLD))  
    ENDDO
  ENDDO

!        Post-processing on PV levels :
  CL(1:1)='V'
  DO JFLD=1,NTFPPV
    CL(6:)=TFP_DYNDS(NCVFP(JFLD))%CLNAME(1:12)
    DO JLEV=1,NOFPPV(JFLD)
      ILEV=NINT(RFP3PV(NVFP(JLEV,JFLD))*1.E+09_JPRB)
      WRITE(UNIT=CL(2:5),FMT='(I4.4)') ILEV
      WRITE(NULOUT,FMT='(A16,''/'',10(1X,A7))')&
       & CL,(CFPDOM(NIVFP(JD,JLEV,JFLD)),JD=1,NAFPV(JLEV,JFLD))  
    ENDDO
  ENDDO

!        Post-processing on S levels :
  DO JFLD=1,NTFPS
    DO JLEV=1,NOFPS(JFLD)
      WRITE(NULOUT,FMT='(''S'',I3.3,A12,''/'',10(1X,A7))')&
       & NRFP3S(NSFP(JLEV,JFLD)),TFP_DYNDS(NCSFP(JFLD))%CLNAME(1:12),&
       & (CFPDOM(NISFP(JD,JLEV,JFLD)),JD=1,NAFPS(JLEV,JFLD))  
    ENDDO
  ENDDO

!        Post-processing on 2D fields :
  DO JFLD=1,NOFP2F
    WRITE(NULOUT,FMT='(A16,''/'',10(1X,A7))')&
     & TFP_DYNDS(NC2FP(JFLD))%CLNAME,(CFPDOM(NI2FP(JD,JFLD)),&
     & JD=1,NOFP2A(JFLD))  
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('SU4FPOS',1,ZHOOK_HANDLE)
END SUBROUTINE SU4FPOS
