SUBROUTINE SUALFPOS

!**** *SUALFPOS* - Allocate the working arrays for Full-POS computation.

!     Purpose.
!     -------
!           Allocation of the working arrays used exclusively by Full-POS

!**   Interface.
!     ---------
!     *CALL* * SUALFPOS*

!     Explicit arguments :  None
!     --------------------
!        Called by SUALLO

!     Implicit arguments :
!     --------------------
!        Pointers of comdecks

!     Method.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        ALLOCATE

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 96-12-13 R. El Khatib (output geometry + points
!                   locations allocated in sufpg)
!        Modified by K. YESSAD (MARCH 1997): parallelisation of FULL-POS.
!    R. El Khatib : 97-06-02  3 NH fields
!    R. El Khatib : 98-04-06  Tunable spectral packing + simplifications
!    R. El Khatib : 99-10-06  remove useless variable NOFPLEV
!    R. El Khatib : 00-08-18 derived type
!       D. Dent   : 00-04-12 Support for computation of Montgomery geopotential
!    R. El Khatib : 01-12-19 "B-level" parallelization
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSDYN
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR
USE YOMFPC   , ONLY : CFPFMT   ,NFPDOM   ,NFP2DF   ,NFP3DFS  , &
 & NFP3DFP  ,NFP3DFH  ,NFP3DFT  ,NFP3DFV  , &
 & NFP3P    ,NFP3H    ,NFP3TH   ,NFP3PV   ,NFP3S    ,NFPXLEV  , &
 & NFPDPHY  
USE YOMFPD   , ONLY : NLAT
USE YOMFPG   , ONLY : RFPMU    ,NFPMEN
USE YOM4FPOS , ONLY : NCPHYFP  ,NAPHYFP  ,NIPHYFP  ,NOFPP    ,NOFPH    , &
 & NOFPS    ,NOFPPV   ,NOFPTH   ,NCPFP    ,NCHFP    ,NCTFP    , &
 & NCVFP    ,NCSFP    ,NAFPP    ,NAFPH    ,NAFPT    ,NAFPV    , &
 & NAFPS    ,NIPFP    ,NIHFP    ,NITFP    ,NIVFP    ,NISFP    , &
 & NPFP     ,NHFP     ,NTFP     ,NVFP     ,NSFP     ,NOFP2A   , &
 & NC2FP    ,NI2FP  
USE YOMVPOS  , ONLY : XFPLEV   ,QFPTYPE  ,NFPTRDYN
USE YOMGRB   , ONLY : MSEC2GG
USE TYPE_FPRQDYNS, ONLY : ALLOCATE_FPRQDYN, INQUIRE_FPRQDYN

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    ALLOCATE SPACE FOR ARRAYS.
!              --------------------------

IF (LHOOK) CALL DR_HOOK('SUALFPOS',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
IF (LLP) WRITE(NULOUT,'('' SUALFPOS PRINTOUTS '')')

!*       1.1   OUTPUT GEOMETRY

IF(CFPFMT == 'GAUSS'.OR.CFPFMT == 'MODEL') THEN
  ALLOCATE(RFPMU(NLAT(1)))
  IF(LLP)WRITE(IU,9) 'RFPMU    ',SIZE(RFPMU   ),SHAPE(RFPMU   )
  ALLOCATE(NFPMEN(NLAT(1)))
  IF(LLP)WRITE(IU,9) 'NFPMEN   ',SIZE(NFPMEN  ),SHAPE(NFPMEN  )
ENDIF

!*       1.3   TIME DEPENDANT ARRAYS

ALLOCATE(NOFPP(NFP3DFP))
IF(LLP)WRITE(IU,9) 'NOFPP    ',SIZE(NOFPP   ),SHAPE(NOFPP   )
ALLOCATE(NOFPH(NFP3DFH))
IF(LLP)WRITE(IU,9) 'NOFPH    ',SIZE(NOFPH   ),SHAPE(NOFPH   )
ALLOCATE(NOFPS(NFP3DFS))
IF(LLP)WRITE(IU,9) 'NOFPS    ',SIZE(NOFPS   ),SHAPE(NOFPS   )
ALLOCATE(NOFPPV(NFP3DFV))
IF(LLP)WRITE(IU,9) 'NOFPPV   ',SIZE(NOFPPV   ),SHAPE(NOFPPV   )
ALLOCATE(NOFPTH(NFP3DFT))
IF(LLP)WRITE(IU,9) 'NOFPTH   ',SIZE(NOFPTH   ),SHAPE(NOFPTH   )

ALLOCATE(NCPFP(NFP3DFP))
IF(LLP)WRITE(IU,9) 'NCPFP    ',SIZE(NCPFP   ),SHAPE(NCPFP   )
ALLOCATE(NCHFP(NFP3DFH))
IF(LLP)WRITE(IU,9) 'NCHFP    ',SIZE(NCHFP   ),SHAPE(NCHFP   )
ALLOCATE(NCTFP(NFP3DFT))
IF(LLP)WRITE(IU,9) 'NCTFP    ',SIZE(NCTFP   ),SHAPE(NCTFP   )
ALLOCATE(NCVFP(NFP3DFV))
IF(LLP)WRITE(IU,9) 'NCVFP    ',SIZE(NCVFP   ),SHAPE(NCVFP   )
ALLOCATE(NCSFP(NFP3DFS))
IF(LLP)WRITE(IU,9) 'NCSFP    ',SIZE(NCSFP   ),SHAPE(NCSFP   )

ALLOCATE(NPFP (NFP3P,NFP3DFP))
IF(LLP)WRITE(IU,9) 'NPFP     ',SIZE(NPFP   ),SHAPE(NPFP   )
ALLOCATE(NAFPP(NFP3P,NFP3DFP))
IF(LLP)WRITE(IU,9) 'NAFPP    ',SIZE(NAFPP   ),SHAPE(NAFPP   )
ALLOCATE(NHFP (NFP3H,NFP3DFH))
IF(LLP)WRITE(IU,9) 'NHFP     ',SIZE(NHFP   ),SHAPE(NHFP   )
ALLOCATE(NAFPH(NFP3H,NFP3DFH))
IF(LLP)WRITE(IU,9) 'NAFPH    ',SIZE(NAFPH   ),SHAPE(NAFPH   )
ALLOCATE(NTFP (NFP3TH,NFP3DFT))
IF(LLP)WRITE(IU,9) 'NTFP     ',SIZE(NTFP   ),SHAPE(NTFP   )
ALLOCATE(NAFPT(NFP3TH,NFP3DFT))
IF(LLP)WRITE(IU,9) 'NAFPT    ',SIZE(NAFPT   ),SHAPE(NAFPT   )
ALLOCATE(NVFP (NFP3PV,NFP3DFV))
IF(LLP)WRITE(IU,9) 'NVFP     ',SIZE(NVFP   ),SHAPE(NVFP   )
ALLOCATE(NAFPV(NFP3PV,NFP3DFV))
IF(LLP)WRITE(IU,9) 'NAFPV    ',SIZE(NAFPV   ),SHAPE(NAFPV   )
ALLOCATE(NSFP (NFP3S,NFP3DFS))
IF(LLP)WRITE(IU,9) 'NSFP     ',SIZE(NSFP   ),SHAPE(NSFP   )
ALLOCATE(NAFPS(NFP3S,NFP3DFS))
IF(LLP)WRITE(IU,9) 'NAFPS    ',SIZE(NAFPS   ),SHAPE(NAFPS   )

ALLOCATE(NIPFP(NFPDOM,NFP3P,NFP3DFP))
IF(LLP)WRITE(IU,9) 'NIPFP    ',SIZE(NIPFP   ),SHAPE(NIPFP   )
ALLOCATE(NIHFP(NFPDOM,NFP3H,NFP3DFH))
IF(LLP)WRITE(IU,9) 'NIHFP    ',SIZE(NIHFP   ),SHAPE(NIHFP   )
ALLOCATE(NITFP(NFPDOM,NFP3TH,NFP3DFT))
IF(LLP)WRITE(IU,9) 'NITFP    ',SIZE(NITFP   ),SHAPE(NITFP   )
ALLOCATE(NIVFP(NFPDOM,NFP3PV,NFP3DFV))
IF(LLP)WRITE(IU,9) 'NIVFP    ',SIZE(NIVFP   ),SHAPE(NIVFP   )
ALLOCATE(NISFP(NFPDOM,NFP3S,NFP3DFS))
IF(LLP)WRITE(IU,9) 'NISFP    ',SIZE(NISFP   ),SHAPE(NISFP   )

ALLOCATE(NC2FP(NFP2DF))
IF(LLP)WRITE(IU,9) 'NC2FP    ',SIZE(NC2FP   ),SHAPE(NC2FP   )
ALLOCATE(NOFP2A(NFP2DF))
IF(LLP)WRITE(IU,9) 'NOFP2A   ',SIZE(NOFP2A   ),SHAPE(NOFP2A   )
ALLOCATE(NI2FP(NFPDOM,NFP2DF))
IF(LLP)WRITE(IU,9) 'NI2FP    ',SIZE(NI2FP   ),SHAPE(NI2FP   )

ALLOCATE(XFPLEV  (NFPXLEV))
IF(LLP)WRITE(IU,9) 'XFPLEV   ',SIZE(XFPLEV   ),SHAPE(XFPLEV   )

ALLOCATE(NCPHYFP(NFPDPHY))
IF(LLP)WRITE(IU,9) 'NCPHYFP  ',SIZE(NCPHYFP),SHAPE(NCPHYFP)
ALLOCATE(NAPHYFP(NFPDPHY))
IF(LLP)WRITE(IU,9) 'NAPHYFP  ',SIZE(NAPHYFP),SHAPE(NAPHYFP)
ALLOCATE(NIPHYFP(NFPDOM,NFPDPHY))
IF(LLP)WRITE(IU,9) 'NIPHYFP  ',SIZE(NIPHYFP),SHAPE(NIPHYFP)

CALL ALLOCATE_FPRQDYN(QFPTYPE,NFPDOM,NFPXLEV,JPOSDYN)
IF(LLP) CALL INQUIRE_FPRQDYN(QFPTYPE,'QFPTYPE',IU)

ALLOCATE(NFPTRDYN(JPOSDYN))
IF(LLP)WRITE(IU,9) 'NFPTRDYN ',SIZE(NFPTRDYN),SHAPE(NFPTRDYN)

!*       1.6 GRIB CODE POSTPROCESSING PARAMETERS

ALLOCATE(MSEC2GG(22+NLAT(1)))
IF(LLP)WRITE(IU,9) 'MSEC2GG  ',SIZE(MSEC2GG),SHAPE(MSEC2GG)

! ---------------------------------------------------------------------
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUALFPOS',1,ZHOOK_HANDLE)
END SUBROUTINE SUALFPOS
