SUBROUTINE SUBFPOS

!**** *SUBFPOS*  - INITIALIZE FULL-POS (2nd part)

!     PURPOSE.
!     --------
!        INITIALIZE MODULE YOMFPG WHICH DESCRIBES THE OUTPUT GEOMETRY
!        INITIALIZE MODULE YOMFPF WHICH DESCRIBES THE SPECTRAL FILTER TO
!            BE APPLIED
!        INITIALIZE MODULE YOMFPOP WHICH CONTAINS POST-PROCESSING
!            FILE-HANDLING PARAMETERS
!        INITIALIZE MODULES YOMSC2/YOMFPSC2 WHICH CONTAIN THE CONTROL ARRAYS
!            FOR HORIZONTAL POST-PROCESSING ROWS
!        INITIALIZE MODULES YOMRFPDS, YOMWFPDS, PTRFPB2 WHICH CONTAIN
!            RESPECTIVELY THE DESCRIPTORS OF BUFFERS/WORKFILE OF 
!            - CLIMATOLOGY & GEOMETRY
!            - WEIGHTS AND INDEXES
!            - AUXILARY FIELDS FOR HORIZONTAL POST-PROCESSING
!        INITIALIZE MODULE YOMFPIOS WHICH CONTAINS THE PARAMETERS FOR
!            IO SCHEME ON POST-PROCESSING BUFFERS

!**   INTERFACE.
!     ----------
!       *CALL* *SUBFPOS*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE.

!        IMPLICIT ARGUMENTS
!        --------------------
!        NONE.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        R. El Khatib 98-10-08 Reorganisation SUFPSC2/SUFPSC2B/SUFPIOS/SUFPTR2
!                              move suafpos into subfpos
!        R. El Khatib  : 99-04-22 NFPVT0 moved inside from SU0YOMB
!        R. El Khatib  : 99-08-06 SUFPG moved from SU0YOMB ; initialization 
!                        of fullpos-related variables of YOMSC2 in SUFPWIDE
!        R. El Khatib  : 00-07-31 Remove SUAFPDS
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. El Khatib     20-May-2005 NFPWIDE moved to YOMFPDS
!        K. Yessad: 27-02-2007 Optimisation of distributed memory in FULL-POS
!        K. Yessad Aug-2007: allow to relax the C1 constraint (NDLNPR=0) in NH
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LFPOS
USE YOMFPC   , ONLY : LTRACEFP
USE YOMMP    , ONLY : NAFPB1
USE YOMWFPDS , ONLY : NFPWIDE
USE YOMRFPDS , ONLY : NRFPOS
USE YOMFPDIM , ONLY : NFPAUXB
USE YOMFPSC2 , ONLY : NFPBLOCS ,NFPROMA  ,NFPSTART ,NFPEND   ,NFPSORT  , &
 & NFPBLOFF ,NFPBLOCS_DEP,NFPROMA_DEP,NFPSTART_DEP,NFPEND_DEP,NFPSORT_DEP, &
 & NFPBLOFF_DEP
USE YOMFPG   , ONLY : LFPDISTRIB

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=39) :: CLFMT='(1X,''ARRAY '',A10,'' ALLOCATED '',8I8)'
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sualfpos.intfb.h"
#include "sufpf.intfb.h"
#include "sufpg.intfb.h"
#include "sufpios.intfb.h"
#include "sufpoph.intfb.h"
#include "sufpsc2.intfb.h"
#include "sufpsc2_dep.intfb.h"
#include "sufptr2.intfb.h"
#include "sufpwide.intfb.h"
#include "sufprfpds.intfb.h"
#include "sufpwfpds.intfb.h"
#include "sufprfpbuf.intfb.h"
#include "sufpwfpbuf.intfb.h"
#include "fpiniphy.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUBFPOS',0,ZHOOK_HANDLE)
CALL GSTATS(2001,0)

!     ------------------------------------------------------------------

IF (LFPOS) THEN

!*       1. CONTROL REQUEST VS SETUP OF THE PHYSICS IN THE MODEL
!           ----------------------------------------------------

  CALL FPINIPHY

!*       2. MISCELLANEOUS ALLOCATIONS
!           -------------------------

  CALL SUALFPOS

!*       3. INITIALIZE MODULE YOMFPG
!           ------------------------

  CALL SUFPG

!*       4. INITIALIZE MODULE YOMWFDS
!           -------------------------

  CALL SUFPWFPDS

!*       5. INITIALIZE MODULE YOMSC2
!           ------------------------

  CALL SUFPWIDE

!*       6. INITIALIZE MODULE YOMFPF
!           ------------------------

  CALL SUFPF

!*       7. INITIALIZE MODULE YOMFPOP
!           -------------------------

  CALL SUFPOPH

!*       8. INITIALIZE MODULE YOMFPSC2
!           --------------------------

  CALL SUFPSC2_DEP

  IF (LFPDISTRIB) THEN

    CALL SUFPSC2

  ELSE

    NFPROMA=NFPROMA_DEP
    NFPBLOCS=NFPBLOCS_DEP

    IF (ALLOCATED(NFPSTART)) DEALLOCATE(NFPSTART)
    ALLOCATE(NFPSTART(NFPBLOCS))
    IF (ALLOCATED(NFPEND)) DEALLOCATE(NFPEND)
    ALLOCATE(NFPEND(NFPBLOCS))
    IF (ALLOCATED(NFPBLOFF)) DEALLOCATE(NFPBLOFF)
    ALLOCATE(NFPBLOFF(NFPBLOCS))
    IF (LTRACEFP) THEN
      WRITE(NULOUT,CLFMT) 'NFPSTART ',SIZE(NFPSTART),SHAPE(NFPSTART)
      WRITE(NULOUT,CLFMT) 'NFPEND   ',SIZE(NFPEND),SHAPE(NFPEND)
      WRITE(NULOUT,CLFMT) 'NFPBLOFF ',SIZE(NFPBLOFF),SHAPE(NFPBLOFF)
    ENDIF
    IF (ALLOCATED(NFPSORT)) DEALLOCATE(NFPSORT)
    ALLOCATE(NFPSORT(NFPROMA,NFPBLOCS))
    IF (LTRACEFP) WRITE(NULOUT,CLFMT) 'NFPSORT   ',SIZE(NFPSORT),SHAPE(NFPSORT)

    NFPSTART(:)=NFPSTART_DEP(:)
    NFPEND(:)=NFPEND_DEP(:)
    NFPSORT(:,:)=NFPSORT_DEP(:,:)
    NFPBLOFF(:)=NFPBLOFF_DEP(:)

    WRITE(NULOUT,'(A)') ' SUBFPOS: case LFPDISTRIB=F '
    WRITE(NULOUT,'(A)') ' NFPROMA=NFPROMA_DEP; NFPBLOCS=NFPBLOCS_DEP '
    WRITE(NULOUT,'(A)') ' NFPSTART=NFPSTART_DEP; NFPEND=NFPEND_DEP '
    WRITE(NULOUT,'(A)') ' NFPSORT=NFPSORT_DEP; NFPBLOFF=NFPBLOFF_DEP '

  ENDIF

!*       9. INITIALIZE MODULES YOMRFPDS, YOMWFDS, YOMFPTR2
!           ----------------------------------------------

  CALL SUFPRFPDS
  CALL SUFPTR2

!*      10. INITIALIZE MODULE YOMFPIOS & BUFFER CONTROL ARRAYS
!           --------------------------------------------------

  CALL SUFPIOS

  IF (NRFPOS > 0) CALL SUFPRFPBUF

  CALL SUFPWFPBUF

ELSE

  NFPWIDE=0
  NAFPB1=0
  NFPAUXB=0

ENDIF

!     ------------------------------------------------------------------

CALL GSTATS(2001,1)
IF (LHOOK) CALL DR_HOOK('SUBFPOS',1,ZHOOK_HANDLE)
END SUBROUTINE SUBFPOS
