SUBROUTINE SUFPCFU

!**** *SUFPCFU*  - INITIALIZE CUMULATED FLUXES SWITCHES FOR POST-PROCESSING

!     PURPOSE.
!     --------
!        Modify default values to allow post-processing

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPCFU*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS
!        --------------------
!        YOMFPC
!        YOMCFU

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified 99-04-19 by J.M. Piriou: Moon radiation.
!    R. El Khatib : 00-03-22 Support for convection indexes out of the model
!        R. El Khatib : 01-05-04 Duration of total precipitations
!        R. El Khatib : 01-08-07 pruning options
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR
USE YOMCT0   , ONLY : NSTOP
USE YOMAFN   , ONLY : GFP_CLSP  ,GFP_CCP  ,GFP_CLSG , GFP_CCSG  ,&
 & GFP_CLSH ,GFP_CCSH ,GFP_CLSS ,GFP_CCSF ,GFP_CUSS , &
 & GFP_CVSS ,GFP_CSSH ,GFP_CSLH ,GFP_CTSP ,GFP_CSMR ,GFP_CTP  , &
 & GFP_CTCC ,GFP_CBLD ,GFP_CSSR ,GFP_CSTR ,GFP_CTSR ,GFP_CETP , &
 & GFP_CTTR ,GFP_CCCC ,GFP_CHCC ,GFP_CMCC ,GFP_CLCC ,GFP_CIRU , &
 & GFP_CUGW ,GFP_CVGW ,GFP_CE   ,GFP_CS   ,GFP_CLHE ,GFP_CDRU , &
 & GFP_CLHS ,GFP_CWS  ,GFP_CSNS ,GFP_CQTO ,GFP_CDUTP,GFP_CTTHC, &
 & GFP_CTO3 ,GFP_CTME ,GFP_CICE ,GFP_CLI  ,GFP_CCVU ,GFP_CTSOC, &
 & GFP_CCVV ,GFP_CCVQ ,GFP_CCVS ,GFP_CTUQ ,GFP_CTUS ,GFP_CSRU , &
 & GFP_CSOC ,GFP_CTHC ,GFP_CSOP ,GFP_CTOP ,GFP_CSOD ,GFP_CEAS , &
 & GFP_CTHD ,GFP_CFON ,GFP_CCHS  
USE YOMFPC   , ONLY : CFPCFU   ,NFPCFU
USE YOMCFU   , ONLY : LFRRC    ,LCUMFU   ,LREACFU  ,LSTRD    ,LSTRC  ,&
 & LSTRT    ,LFPLC    ,LFPLCG  ,LFPLCH   ,LFPLS    ,LFPLSG   ,LFPLSH  ,&
 & LFR      ,LAMIP    ,LRAYS   ,LNEBTT   ,LFSF     ,LMOON    ,LFSOIL   ,&
 & LNEBPAR  ,LFDUTP   ,LRAYD

IMPLICIT NONE

CHARACTER :: CLMARK*39

INTEGER(KIND=JPIM) :: IUCV, IUGW, IUSS, IVCV, IVGW, IVSS, JFLD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*       1. MODIFY DEFAULT VALUES
!           ---------------------

IF (LHOOK) CALL DR_HOOK('SUFPCFU',0,ZHOOK_HANDLE)
IF (NFPCFU > 0) THEN

  LCUMFU=.TRUE.
  IF (NSTOP == 0) THEN
    LREACFU=.TRUE.
  ENDIF

  IUSS=0
  IVSS=0
  IUGW=0
  IVGW=0
  IUCV=0
  IVCV=0

  DO JFLD=1,NFPCFU
    IF     (CFPCFU(JFLD) == GFP_CLSP%CLNAME) THEN
      LFPLS=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCP%CLNAME ) THEN
      LFPLC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLSS%CLNAME) THEN
      LFPLS=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCSF%CLNAME) THEN
      LFPLC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLSG%CLNAME ) THEN
      LFPLSG=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCSG%CLNAME ) THEN
      LFPLCG=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLSH%CLNAME ) THEN
      LFPLSH=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCSH%CLNAME ) THEN
      LFPLCH=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CUSS%CLNAME) THEN
      LFSF=.TRUE.
      IUSS=1
    ELSEIF (CFPCFU(JFLD) == GFP_CVSS%CLNAME) THEN
      LFSF=.TRUE.
      IVSS=1
    ELSEIF (CFPCFU(JFLD) == GFP_CSSH%CLNAME) THEN
      LFSF=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSLH%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTSP%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTCC%CLNAME) THEN
      LNEBTT=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CBLD%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSSR%CLNAME) THEN
      LFR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSTR%CLNAME) THEN
      LFR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTSR%CLNAME) THEN
      LFR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTTR%CLNAME) THEN
      LFR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCCC%CLNAME) THEN
      LNEBPAR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CHCC%CLNAME) THEN
      LNEBPAR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CMCC%CLNAME) THEN
      LNEBPAR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLCC%CLNAME) THEN
      LNEBPAR=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CUGW%CLNAME) THEN
      LSTRD=.TRUE.
      IUGW=1
    ELSEIF (CFPCFU(JFLD) == GFP_CVGW%CLNAME) THEN
      LSTRD=.TRUE.
      IVGW=1
    ELSEIF (CFPCFU(JFLD) == GFP_CE%CLNAME) THEN
      LFSF=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CS%CLNAME  ) THEN
      LFSF=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLHE%CLNAME) THEN
      LFSF=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLHS%CLNAME) THEN
      LFSF=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CWS%CLNAME ) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSNS%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CQTO%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTO3%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTME%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CICE%CLNAME) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CLI%CLNAME ) THEN
      LAMIP=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCVU%CLNAME) THEN
      LSTRC=.TRUE.
      IUCV=1
    ELSEIF (CFPCFU(JFLD) == GFP_CCVV%CLNAME) THEN
      LSTRC=.TRUE.
      IVCV=1
    ELSEIF (CFPCFU(JFLD) == GFP_CCVQ%CLNAME) THEN
      LSTRC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCVS%CLNAME) THEN
      LSTRC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTUQ%CLNAME) THEN
      LSTRT=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTUS%CLNAME) THEN
      LSTRT=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSOC%CLNAME) THEN
      LFRRC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTHC%CLNAME) THEN
      LFRRC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTSOC%CLNAME) THEN
      LFRRC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTTHC%CLNAME) THEN
      LFRRC=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSOP%CLNAME) THEN
      LRAYS=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTOP%CLNAME) THEN
      LRAYS=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSOD%CLNAME) THEN
      LRAYD=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTHD%CLNAME) THEN
      LRAYD=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSMR%CLNAME) THEN
      LMOON=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CFON%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CCHS%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CEAS%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CSRU%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CDRU%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CIRU%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CETP%CLNAME) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CTP%CLNAME ) THEN
      LFSOIL=.TRUE.
    ELSEIF (CFPCFU(JFLD) == GFP_CDUTP%CLNAME) THEN
      LFDUTP=.TRUE.
    ELSE
      CLMARK=CFPCFU(JFLD)//' NOT FOUND '
      WRITE(NULERR,*) CLMARK
    ENDIF
  ENDDO
  IF ((IUSS /= IVSS).OR.(IUGW /= IVGW).OR.(IUCV /= IVCV)) THEN
    CALL ABOR1('U AND V COMPONENTS ARE BOTH NECESSARY !')
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('SUFPCFU',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPCFU
