SUBROUTINE SUFPCONF(KSCAN,CDVFP,CDHFP,CDLFP,CDWFP,CDFMT)

!**** *SUFPCONF*  - INITIALIZE FULL-POS CONFIGURATION LETTERS FOR STEPO

!     PURPOSE.
!     --------
!        Initialise for each scan the configuration letters for vertical
!        post-processing, horizontal post-processing and for IOPACK.

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPCONF*

!        EXPLICIT ARGUMENTS
!        --------------------
!        KSCAN : Scan index                                (input)
!        CDVFP : Configuration for vertical post-processing
!        CDHFP : Configuration for horizontal post-processing
!        CDLFP : Configuration for lagged post-processing
!        CDWFP : Configuration for IOPACK
!        CDFMT: Output format : no horizontal interpolations if CDFMT='MODEL'

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!           NONE

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        R. El Khatib 98-08-14 Simplify scan controls
!        R. El Khatib 00-07-31 Simplify scan controls
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSCAN 
CHARACTER(LEN=1)  ,INTENT(OUT)   :: CDVFP 
CHARACTER(LEN=1)  ,INTENT(OUT)   :: CDHFP 
CHARACTER(LEN=1)  ,INTENT(OUT)   :: CDLFP 
CHARACTER(LEN=1)  ,INTENT(OUT)   :: CDWFP 
CHARACTER(LEN=5)  ,INTENT(IN)    :: CDFMT 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
!     ------------------------------------------------------------------

!*       1.    SCAN FOR EACH LEVEL TYPE
!              ------------------------

IF (LHOOK) CALL DR_HOOK('SUFPCONF',0,ZHOOK_HANDLE)
IF     (KSCAN == 0) THEN
!        Internal scan on model levels   :
  CDVFP='M'
  CDHFP='A'
  CDLFP='0'
  CDWFP='Z'
ELSEIF (KSCAN == 1) THEN
!        Post-processing on P levels :
  CDVFP='B'
  IF (CDFMT == 'MODEL') THEN
    CDHFP='0'
  ELSE
    CDHFP='A'
  ENDIF
  CDLFP='0'
  CDWFP='E'
ELSEIF (KSCAN == 2) THEN
!        Post-processing on H levels :
  IF (CDFMT == 'MODEL') THEN
    CDVFP='H'
    CDHFP='0'
    CDLFP='0'
  ELSE
    CDVFP='M'
    CDHFP='I'
    CDLFP='Z'
  ENDIF
  CDWFP='U'
ELSEIF (KSCAN == 3) THEN
!        Post-processing on PV levels :
  CDVFP='V'
  IF (CDFMT == 'MODEL') THEN
    CDHFP='0'
  ELSE
    CDHFP='A'
  ENDIF
  CDLFP='0'
  CDWFP='Y'
ELSEIF (KSCAN == 4) THEN
!        Post-processing on TH levels :
  CDVFP='T'
  IF (CDFMT == 'MODEL') THEN
    CDHFP='0'
  ELSE
    CDHFP='A'
  ENDIF
  CDLFP='0'
  CDWFP='M'
ELSEIF (KSCAN == 5) THEN
!        Post-processing on eta levels :
  IF (CDFMT == 'MODEL') THEN
    CDVFP='S'
    CDHFP='0'
    CDLFP='0'
  ELSE
    CDVFP='M'
    CDHFP='I'
    CDLFP='E'
  ENDIF
  CDWFP='Z'
ELSE
  CALL ABOR1 ('UNKNOWN VERTICAL LEVEL TYPE OF POST-PROCESSING !')
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUFPCONF',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPCONF
