SUBROUTINE SUFPF

!**** *SUFPF*  - SET UP POST-PROCESSING SPECTRAL FILTER

!     PURPOSE.
!     --------
!        To initialize the profile of the spectral post-processing filter

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPF*

!        EXPLICIT ARGUMENTS
!        --------------------
!           None.

!       IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!     g. radnoti   : 95-01-30: no filter in ALADIN
!     R. EL KHATIB : 97-07-25 : safer computation of NFMAX in ALADIN
!     R. El Khatib : 01-04-09 : enable filtering for ECMWF and for spectra + cleanings
!     R. El Khatib : 01-08-07 Pruning options
!     R. El Khatib : 03-04-17 Fullpos improvments
!     R. El Khatib : 03-08-26 LFPMAT (bugfix)
!     M.Hamrud     : 01-Oct-2003 CY28 Cleaning
!     R. El Khatib : 04-03-25 LFPWRFIL ,LFPRDFIL
!     F. Taillefer : 06-04-01 prepare ALADIN filters
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSDOM
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMDIM   , ONLY : NCMAX    ,NSMAX    ,NSPEC2   ,NMSMAX
USE YOMGEM   , ONLY : RSTRET
USE YOMCT0   , ONLY : LELAM
USE YOMFPC   , ONLY : CFPFMT   ,NFPDOM   ,LTRACEFP
USE YOMFPD   , ONLY : NLAT     ,NLON     ,RLATN    ,RLATS    ,RLONW    ,&
 & RLONE  
USE YOMFPG   , ONLY : NFPMAX   ,FPSTRET  ,NMFPMAX
USE YOMFPF   , ONLY : NFMAX    ,RFPFIL   ,RFPBED   ,RFPLTF   ,LFPFIL   ,&
 & LFPBED  ,LFPMAT   ,LFPWRFIL ,LFPRDFIL ,NFPCMAX  ,NFPGLFI  ,RFPEXPC
USE YOMLAP   , ONLY : NVALUE
USE YEMLAP   , ONLY : MVALUE

IMPLICIT NONE

INTEGER(KIND=JPIM) :: J, JDOM, JN, IN, IM

REAL(KIND=JPRB) :: Z1, Z2, ZEPS, ZK, ZNFMAX, Z1SNX2, Z1SMX2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"

#include "namfpf.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPF',0,ZHOOK_HANDLE)
ZEPS=1.E-10_JPRB

!*       1.    SET DEFAULT VALUES
!              ------------------

LFPBED=.TRUE.
NFPGLFI=12
Z1SNX2=1.0_JPRB/REAL(NSMAX**2,JPRB)

ALLOCATE(RFPEXPC(NSPEC2))

SELECT CASE (LELAM)
CASE(.TRUE.)
  RFPBED=0.0_JPRB
  RFPLTF=6._JPRB
  IF (CFPFMT == 'GAUSS'.OR.CFPFMT == 'MODEL') THEN
    DO J=1,NFPDOM
      NFMAX(J)=MIN(NFPMAX(J),NMFPMAX)
    ENDDO
  ELSE
    DO J=1,NFPDOM
      Z1=(360._JPRB*REAL(NLON(J),JPRB)/MAX(RLONE(J)-RLONW(J),ZEPS)-1.0_JPRB) &
       & /3._JPRB
      Z2=(360._JPRB*REAL(NLAT(J),JPRB)/MAX(RLATN(J)-RLATS(J),ZEPS)-1.0_JPRB) &
       & /3._JPRB
      ZNFMAX=MIN(Z1,Z2)
      NFMAX(J)=INT(ZNFMAX)
    ENDDO
  ENDIF
  LFPWRFIL=.FALSE.
  LFPRDFIL=.FALSE.
  SELECT CASE (NMSMAX)
  CASE (0)
    Z1SMX2=0.0_JPRB
  CASE DEFAULT
    Z1SMX2=1.0_JPRB/REAL(NMSMAX,JPRB)**2
  END SELECT
  DO J=1,NSPEC2
    IN=NVALUE(J)
    Z1=REAL(IN,JPRB)**2
    IM=MVALUE(J)
    Z2=REAL(IM,JPRB)**2
    RFPEXPC(J)=(Z1*Z1SNX2)+(Z2*Z1SMX2)
  ENDDO
CASE DEFAULT
! Default value of RFPBED for continuity with the previous formulation ;
! gives roughly RFPBED = 3.08
  RFPBED=-LOG(LOG(9._JPRB)/48._JPRB)
  RFPLTF=4._JPRB
  IF (CFPFMT == 'GAUSS'.OR.CFPFMT == 'MODEL') THEN
    DO J=1,NFPDOM
      NFMAX(J)=NINT(REAL(NFPMAX(J),JPRB)*FPSTRET)
    ENDDO
  ELSE
    DO J=1,NFPDOM
      Z1=(360._JPRB*REAL(NLON(J),JPRB)/MAX(RLONE(J)-RLONW(J),ZEPS)-1.0_JPRB) &
       & /3._JPRB  
      Z2=(360._JPRB*REAL(NLAT(J),JPRB)/MAX(RLATN(J)-RLATS(J),ZEPS)-1.0_JPRB) &
       & /3._JPRB  
      ZNFMAX=MIN(Z1,Z2)
      NFMAX(J)=INT(ZNFMAX)
    ENDDO
  ENDIF
  LFPWRFIL=.FALSE.
  LFPRDFIL=.FALSE.
  DO J=1,NSPEC2
    IN=NVALUE(J)
    Z2=REAL(IN,JPRB)**2
    RFPEXPC(J)=Z2*Z1SNX2
  ENDDO
END SELECT

DO J=NFPDOM+1,JPOSDOM
  NFMAX(J)=0
ENDDO

!*       2.    READ NAMELIST
!              -------------

CALL POSNAM(NULNAM,'NAMFPF')
READ(NULNAM,NAMFPF)

!*       3.    CHECK NAMELIST VALUES AND COMPUTE FILTER
!              ----------------------------------------

! For compatibility : consider NCMAX if matrixes are used.
NFPCMAX=MAX(NCMAX,NINT(REAL(NSMAX,KIND=JPRB)*RSTRET))

ALLOCATE(LFPFIL(NFPDOM))

LFPFIL(:)=.FALSE.

SELECT CASE (LELAM)
CASE (.TRUE.)
  ALLOCATE(RFPFIL(NSPEC2,NFPDOM))
  LFPMAT=.FALSE.
  DO JDOM=1,NFPDOM
    IF (NFMAX(JDOM) > 0) THEN
      LFPFIL(JDOM)=.TRUE.
      ZK=-0.5_JPRB*RFPLTF/REAL(MIN(NSMAX,NMSMAX,NFMAX(JDOM))**2,JPRB)
      DO JN=1,NSPEC2
        IN=NVALUE(JN)
        IM=MVALUE(JN)
        RFPFIL(JN,JDOM)=EXP(ZK*((REAL(IN,JPRB)**2)+(REAL(IM,JPRB)**2)))
      ENDDO
    ENDIF
  ENDDO
  IF (.NOT.ANY(LFPFIL)) THEN
    WRITE(NULOUT,'(2A)') ' GAUSSIAN FILTER ON DERIVATIVES HAS BEEN DISABLED', &
     & ' BY NEGATIVE OR NULL TRUNCATIONS IN NAMELIST'
  ENDIF
CASE (.FALSE.)
  ALLOCATE(RFPFIL(0:NFPCMAX,NFPDOM))
  SELECT CASE (LFPBED)
  CASE (.TRUE.)
!   "THX" filters only if the output resolution is coarser than the model :
    ZK=EXP(-RFPBED)
    DO JDOM=1,NFPDOM
      IF (NFMAX(JDOM) > 0 .AND. NFMAX(JDOM) < NFPCMAX) THEN
        LFPFIL(JDOM)=.TRUE.
        DO JN=0,NFPCMAX
          RFPFIL(JN,JDOM)=0.5_JPRB*(1.0_JPRB-TANH(ZK*REAL(JN-NFMAX(JDOM),JPRB)))
        ENDDO
      ENDIF
    ENDDO
    IF (.NOT.ANY(LFPFIL)) THEN
      WRITE(NULOUT,'(2A)') ' THE POST-PROCESSING RESOLUTION IS NEVER COARSER', &
       & ' THAN THE MODEL RESOLUTION'  
    ENDIF
  CASE DEFAULT
!       New gaussian filter applied on all (sub)domains :
    DO JDOM=1,NFPDOM
      IF (NFMAX(JDOM) > 0) THEN
        LFPFIL(JDOM)=.TRUE.
        ZK=-0.5_JPRB*RFPLTF/REAL(MIN(NFPCMAX,NFMAX(JDOM))**2,JPRB)
        DO JN=0,NFPCMAX
          RFPFIL(JN,JDOM)=EXP(ZK*REAL(JN,JPRB)**2)
        ENDDO
      ENDIF
    ENDDO
    IF (.NOT.ANY(LFPFIL)) THEN
      WRITE(NULOUT,'(2A)') ' GAUSSIAN FILTER ON DERIVATIVES HAS BEEN DISABLED', &
       & ' BY NEGATIVE OR NULL TRUNCATIONS IN NAMELIST'  
    ENDIF
  END SELECT 
  LFPMAT=(NFPCMAX > NSMAX)
END SELECT

!*       4.    PRINT OUT FINAL VALUES
!              ----------------------

IF(LTRACEFP) THEN
  WRITE(UNIT=NULOUT,FMT='('' MODULE YOMFPF'')')
  WRITE(UNIT=NULOUT,FMT='('' LFPMAT = '',L2,'' LFPBED = '',L2,'' RFPBED = '', &
   & E14.7,'' RFPLTF = '',E14.7,'' NFPCMAX = '',I5)') LFPMAT, LFPBED, RFPBED, &
   & RFPLTF, NFPCMAX
  WRITE(UNIT=NULOUT,FMT='('' LFPWRFIL = '',L2,'' LFPRDFIL = '',L2, &
   & '' NFPGLFI = '',I3)') LFPWRFIL, LFPRDFIL, NFPGLFI
  IF (NFPDOM > 0) THEN
    WRITE(UNIT=NULOUT,FMT='('' (J, NFMAX, LFPFIL) '')')
    WRITE(UNIT=NULOUT,FMT='(10('' ('',I2,I5,L2,'')''))') &
     & (J,NFMAX(J),LFPFIL(J), J=1,NFPDOM)  
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('SUFPF',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPF
