SUBROUTINE SUFPG1(KFPRGP,LDSAME,LDEZO,LDESAME)

!**** *SUFPG1*  - INITIALIZE OUTPUT GEOMETRY PARAMETRES (Full-POS)

!     PURPOSE.
!     --------
!        SETS DEFAULT VALUES, THEN READS NAMELIST NAMFPG AND CHECKS IT.

!**   INTERFACE. *CALL* *SUFPG1*
!     ----------

!        EXPLICIT ARGUMENTS :
!        ------------------
!        OUTPUT:
!        KFPRGP : Number of points on the global output grid.
!        LDSAME : .TRUE. if the output grid is the same as the input grid 
!                  (horizontal aspects only, and not extension zone !)
!        LDEZO  : .TRUE. if biperiodicisation will take place
!        LDESAME : .TRUE. if the output E zone is the same as the input one

!        IMPLICIT ARGUMENTS
!        ------------------
!        PARDIM, YOMDIM
!        PARFPOS
!        YOMFPC, YOMFPD
!        YOMFPG, NAMFPG
!        YOMVV1, YOMGEM, YOMLEG
!        YOMCST

!     EXTERNALS.
!     ----------
!      Calls SUGAWA, REMARK2

!     AUTHOR.    RYAD EL KHATIB *METEO-FRANCE*
!     -------
!     MODIFICATIONS.
!     --------------
!        970304 ORIGINAL from previous SUFPG+SUEFPG
!        9703   K.YESSAD: parallelisation of FULL-POS.
!        970717 R. El Khatib (compute LDSAME)
!        97-31-10 R. El Khatib (Modify default values)
!        98-03-17 P. Le Moigne (Non linear grid truncation)
!        98-06    R. El Khatib LFPOLE=.FALSE. always!
!        98-08-10 K. YESSAD: removal of LFPOLE option.
!         - removal of LFPOLE in YOMFPG and in 'CALL SUNMEN'.
!         - removal of LFPOLE calculation. 
!        98-08-26 R. El Khatib case LMAP=.FALSE.
!        98-10-01 R. El Khatib Remove NFPGP ; move computation of NFPSIZEG
!                  from SUFPD to SUFPG1 ; compute LDEZO ;  LFPOSHOR=> LDSAME
!        R. El Khatib : 99-03-09 Simplify fullpos control ; NFPSIZEL
!        99-04-15 R. El Khatib translation in Aladin
!        R. El Khatib : 00-01-31 Fix default value for FPNLGINC/NFPMEN
!        M. Janousek : 01-04-13 New domain and projection definition
!        R. El Khatib : 01-08-07 Pruning options + LDESAME
!        R. El Khatib : 03-08-25 gfl
!        R. El Khatib : 03-04-17 Fullpos improvments
!        JD. Gril     : 05-02-08 Initialization of LFPMRT
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        K.Yessad (Nov 2008): rename arp/SUGAW into arp/SUGAWA.
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT   ,NULNAM   ,NULERR
USE PARDIM   , ONLY : JPMXLE
USE YOMCT0   , ONLY : NPRINTLEV,LELAM    ,LOUTPUT
USE YOMCT0B  , ONLY : LECMWF
USE YOMFPCT0 , ONLY : NFPCT0
USE YOMDIM   , ONLY : NDGLG    ,NFLEVG   ,NSMAX    ,&
 & NDLON    ,NDLUXG   ,NDGUXG   ,NMSMAX  
USE YOMFPC   , ONLY : CFPFMT   ,CFPDOM   ,NRFP3S   ,NFPDOM   ,NFP3S
USE YOMFPD   , ONLY : NLAT     ,NLON     ,RLONC    ,RLATC    ,NFPLUX   ,&
 & NFPGUX   ,RLONW    ,RLONE    ,RLATS    ,RLATN   ,RDELX    ,RDELY    ,&
 & NFPSIZEG ,NFPSIZEL  
USE YOMFPG   , ONLY :  NFPMAX   ,NFPRGRI  ,FPVALH   ,FPVBH    ,RFPMU    ,&
 & NFPMEN   ,NFPLEV   ,NFPHTYP  ,NFPTTYP  ,NMFPMAX  ,FPMUCEN  ,FPLOCEN  ,&
 & FPSTRET  ,FPLX     ,FPLY     ,FPLON0   ,FPLAT0   ,FPRPK    ,FPNLGINC ,&
 & LFPMAP   ,LFPMRT   ,LFPOSVER ,LFPDISTRIB
USE YOMGEM   , ONLY : NLOENG   ,VBH      ,VAH      ,NSTTYP   ,NMENG    ,&
 & NHTYP    ,RMUCEN   ,RLOCEN   ,RSTRET   ,VP00  
USE YOMLEG   , ONLY : RMU
USE YOMCST   , ONLY : RPI
USE YEMGEO   , ONLY : ELON0    ,ELAT0    ,ELX      ,ELY      ,LMAP     ,&
 & EDELX    ,EDELY    ,ELONC    ,ELATC    ,LMRT
USE YOMFPEZO , ONLY : NFPEG
USE IOSTREAM_MIX , ONLY : INI_IOSTREAM

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(OUT)   :: KFPRGP 
LOGICAL           ,INTENT(OUT)   :: LDSAME 
LOGICAL           ,INTENT(OUT)   :: LDEZO 
LOGICAL           ,INTENT(OUT)   :: LDESAME 

!-----------------------------------------------------------------------

CHARACTER (LEN = 39) :: CL

REAL(KIND=JPRB) :: ZW(NLAT(1)), ZSQM2(NLAT(1))

REAL(KIND=JPRB) :: ZFPMU(NLAT(1))

INTEGER(KIND=JPIM) :: IERR, J, JGL, JLEV

REAL(KIND=JPRB) :: Z, Z1, Z2, ZEPS, ZNFMAX, ZNFPMAX, ZRDTODG

! -- ky: for call to SUGAWA --------------
INTEGER(KIND=JPIM) :: IDGLG,ISTART,IK,IODD,JN
REAL(KIND=JPRB),ALLOCATABLE :: ZFN(:,:), ZFNLAT(:)
REAL(KIND=JPRB) :: ZFNN
! ----------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "abor1.intfb.h"
#include "sudefo_vv1.intfb.h"
#include "posnam.intfb.h"
#include "sugawa.intfb.h"
#include "sunmen.intfb.h"

!-----------------------------------------------------------------------

#include "namfpg.h"

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPG1',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------
!*       1. SET DEFAULT VALUES
!           -------------------

ZEPS = EPSILON(1.0_JPRB)*10000._JPRB
ZRDTODG=180._JPRB/RPI

NFPLEV = NFLEVG
DO JLEV = 0,NFLEVG
  FPVALH(JLEV) = VAH(JLEV)
  FPVBH (JLEV) = VBH(JLEV)
ENDDO

IF (.NOT.LELAM) THEN
  IF (CFPFMT == 'GAUSS'.OR.CFPFMT == 'MODEL') THEN

    FPSTRET = RSTRET
    NFPTTYP = NSTTYP
    FPMUCEN = RMUCEN
    FPLOCEN = RLOCEN
    IF (NLAT(1) == NDGLG) THEN
      NFPHTYP = NHTYP
      DO JGL = 1,NDGLG
        NFPRGRI(JGL) = NLOENG(JGL)
      ENDDO
    ELSE
      NFPHTYP = 0
      DO JGL = 1,NLAT(1)
        NFPRGRI(JGL) = NLON(1)
      ENDDO
    ENDIF
    FPNLGINC = 1._JPRB

    NFPMAX(1) = NSMAX
    NMFPMAX   = 1

    FPLON0 = 0._JPRB
    FPLAT0 = 0._JPRB
    LFPMRT=.FALSE.
    FPRPK = -9._JPRB
    FPLX = 0._JPRB
    FPLY = 0._JPRB

  ELSEIF (CFPFMT == 'LALON') THEN

    FPSTRET = 1._JPRB
    NFPTTYP = 1
    FPMUCEN = 1._JPRB
    FPLOCEN = 0._JPRB
    NFPHTYP = 0
    FPNLGINC = 1._JPRB

    DO J = 1, NFPDOM
      Z1=(360._JPRB*REAL(NLON(J),JPRB)/MAX(RLONE(J)-RLONW(J),ZEPS)-1.0_JPRB)/ &
       & 3._JPRB  
      Z2=(360._JPRB*REAL(NLAT(J),JPRB)/MAX(RLATN(J)-RLATS(J),ZEPS)-1.0_JPRB)/ &
       & 3._JPRB  
      ZNFMAX=MIN(Z1,Z2)
      NFPMAX(J)=INT(ZNFMAX)
    ENDDO
    NMFPMAX = 1

    FPLON0 = 0._JPRB
    FPLAT0 = 0._JPRB
    LFPMRT=.FALSE.
    FPRPK = -9._JPRB
    FPLX = 0._JPRB
    FPLY = 0._JPRB

  ELSEIF (CFPFMT == 'LELAM') THEN

    FPSTRET = 1._JPRB
    NFPTTYP = 1
    FPMUCEN = 1._JPRB
    FPLOCEN = 0._JPRB
    NFPHTYP = 0
    DO JGL = 1,NLAT(1)
      NFPRGRI(JGL) = NLON(1)
    ENDDO
    FPNLGINC = 1._JPRB

    Z1=(REAL(NLAT(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
    Z2=(REAL(NLON(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
    NFPMAX(1) = INT(Z1)
    NMFPMAX   = INT(Z2)

    FPLX = RDELX(1)*REAL(NLON(1),JPRB)
    FPLY = RDELY(1)*REAL(NLAT(1),JPRB)
    IF     (CFPDOM(1) == 'BELG') THEN
      LFPMRT=.FALSE.
      FPLON0= 2.57831001_JPRB
      FPLAT0= 46.46884918_JPRB
    ELSEIF (CFPDOM(1) == 'SLOV') THEN
      LFPMRT=.FALSE.
      FPLON0= 17.0_JPRB
      FPLAT0= 46.24470064_JPRB
    ELSEIF (CFPDOM(1) == 'MARO') THEN
      LFPMRT=.FALSE.
      FPLON0= -7.00000000000002_JPRB
      FPLAT0= 31.56059436_JPRB
    ELSEIF (CFPDOM(1) == 'OPMA') THEN
      LFPMRT=.FALSE.
      FPLON0= -7.00000000000002_JPRB
      FPLAT0= 31.56059436_JPRB
    ELSEIF (CFPDOM(1) == 'LACE12') THEN
      LFPMRT=.FALSE.
      FPLON0= 17.0_JPRB
      FPLAT0= 46.24470064_JPRB
    ELSEIF (CFPDOM(1) == 'LACE14') THEN
      LFPMRT=.FALSE.
      FPLON0= 17.0_JPRB
      FPLAT0= 46.24470064_JPRB
    ELSEIF (CFPDOM(1) == 'LACE20') THEN
      LFPMRT=.FALSE.
      FPLON0= 17.0_JPRB
      FPLAT0= 46.24470064_JPRB
    ELSEIF (CFPDOM(1) == 'ROUM') THEN
      LFPMRT=.FALSE.
      FPLON0= 25.0_JPRB
      FPLAT0= 44.77301983_JPRB
    ELSEIF (CFPDOM(1) == 'FRAN') THEN
      LFPMRT=.FALSE.
      FPLON0= 2.57831001_JPRB
      FPLAT0= 46.46884918_JPRB
    ELSE
!           Old domain ALADIN-FRANCE
      LFPMRT=.FALSE.
      FPLON0= 2.57831001_JPRB
      FPLAT0= 46.46884918_JPRB
    ENDIF

  ENDIF
ELSE
  IF (CFPFMT == 'GAUSS') THEN

    FPSTRET = 1._JPRB
    NFPTTYP = 1
    FPMUCEN = 1
    FPLOCEN = 0._JPRB
    NFPHTYP = 0
    DO JGL = 1,NLAT(1)
      NFPRGRI(JGL) = NLON(1)
    ENDDO
    FPNLGINC = 1._JPRB

    Z1=(REAL(NLON(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
    NFPMAX(1) = INT(Z1)
    NMFPMAX   = 1

    FPLON0 = 0._JPRB
    FPLAT0 = 0._JPRB
    LFPMRT=.FALSE.
    FPRPK = -9._JPRB
    FPLX = 0._JPRB
    FPLY = 0._JPRB

  ELSEIF (CFPFMT == 'LALON') THEN

    FPSTRET = 1._JPRB
    NFPTTYP = 1
    FPMUCEN = SIN(ELAT0)
    FPLOCEN = ELON0
    NFPHTYP = 0
    FPNLGINC = 1._JPRB

    DO J = 1, NFPDOM
      Z1=(REAL(NLAT(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
      Z2=(REAL(NLON(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
      ZNFMAX=MIN(Z1,Z2)
      NFPMAX(J) = INT(ZNFMAX)
    ENDDO
    NMFPMAX = 1

    FPLON0=ELON0*ZRDTODG
    FPLAT0=ELAT0*ZRDTODG
    LFPMRT=.FALSE.
    FPLX=ELX
    FPLY=ELY

  ELSEIF (CFPFMT == 'LELAM'.OR.CFPFMT == 'MODEL') THEN

    FPSTRET = 1._JPRB
    NFPTTYP = 1
    FPMUCEN = SIN(ELAT0)
    FPLOCEN = ELON0
    NFPHTYP = 0
    DO JGL = 1,NLAT(1)
      NFPRGRI(JGL) = NLON(1)
    ENDDO
    FPNLGINC = 1._JPRB

    Z1=(REAL(NLAT(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
    Z2=(REAL(NLON(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
    NFPMAX(1) = INT(Z1)
    NMFPMAX   = INT(Z2)

    FPLON0=ELON0*ZRDTODG
    FPLAT0=ELAT0*ZRDTODG
    LFPMRT=LMRT
    FPLX=ELX
    FPLY=ELY

  ENDIF
ENDIF

!*       2. READ NAMELIST
!           -------------

!     Read everything
CALL POSNAM(NULNAM,'NAMFPG')
READ(NULNAM,NAMFPG)

IF (NFPLEV /= NFLEVG) THEN
!       Set a default value for FPVALH,FPVBH if possible
  CALL SUDEFO_VV1(LECMWF,NFPLEV,FPVALH,FPVBH,VP00)
!       Posible modification of FPVALH,FPVBH from namelist
  CALL POSNAM(NULNAM,'NAMFPG')
  READ(NULNAM,NAMFPG)
ENDIF

!*       3. CHECK VALUES
!           ------------

IF (NFPCT0 < 0) THEN     ! horizontal interpolations only
  NFPLEV = NFLEVG
  DO JLEV = 0,NFLEVG
    FPVALH(JLEV) = VAH(JLEV)
    FPVBH (JLEV) = VBH(JLEV)
  ENDDO
ENDIF

LFPOSVER=(NFPLEV == NFLEVG) &
 & .AND.(MAXVAL(ABS(FPVALH(0:NFLEVG)-VAH(0:NFLEVG)))<ZEPS) &
 & .AND.(MAXVAL(ABS(FPVBH(0:NFLEVG)-VBH(0:NFLEVG)))<ZEPS)

IF (CFPFMT == 'MODEL'.AND..NOT.LELAM) THEN

  FPSTRET = RSTRET
  NFPTTYP = NSTTYP
  FPMUCEN = RMUCEN
  FPLOCEN = RLOCEN
  NFPHTYP = NHTYP
  DO JGL = 1,NDGLG
    NFPRGRI(JGL) = NLOENG(JGL)
  ENDDO
  FPNLGINC = 999._JPRB
  NFPMAX(1) = NSMAX
  NMFPMAX   = 1

  FPLON0 = 0._JPRB
  FPLAT0 = 0._JPRB
  LFPMRT = .FALSE.
  FPLX = 0._JPRB
  FPLY = 0._JPRB
  LFPMAP = .TRUE.

ELSEIF (CFPFMT == 'LALON') THEN

  FPSTRET = 1._JPRB
  NFPHTYP = 0
  FPNLGINC = 1._JPRB

  IF (.NOT.LELAM) THEN
    DO J = 1, NFPDOM
      Z1=(360._JPRB*REAL(NLON(J),JPRB)/MAX(RLONE(J)-RLONW(J),ZEPS)-1.0_JPRB)/ &
       & 3._JPRB  
      Z2=(360._JPRB*REAL(NLAT(J),JPRB)/MAX(RLATN(J)-RLATS(J),ZEPS)-1.0_JPRB)/ &
       & 3._JPRB  
      ZNFMAX=MIN(Z1,Z2)
      NFPMAX(J) = INT(ZNFMAX)
    ENDDO
  ELSE
    DO J = 1, NFPDOM
      Z1=(REAL(NLAT(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
      Z2=(REAL(NLON(1),JPRB)-1.0_JPRB)/3._JPRB+ZEPS
      ZNFPMAX=MIN(Z1,Z2)
      NFPMAX(J) = INT(ZNFPMAX)
    ENDDO
  ENDIF
  NMFPMAX = 1

  FPLON0 = 0._JPRB
  FPLAT0 = 0._JPRB
  LFPMRT = .FALSE.
  FPRPK = -9._JPRB
  FPLX = 0._JPRB
  FPLY = 0._JPRB
  LFPMAP = .TRUE.

ELSEIF (CFPFMT == 'MODEL'.AND.LELAM) THEN

  FPSTRET = 1._JPRB
  NFPTTYP = 1
  FPMUCEN = 1._JPRB
  FPLOCEN = 0._JPRB
  NFPHTYP = 0
  DO JGL = 1,NLAT(1)
    NFPRGRI(JGL) = NLON(1)
  ENDDO
  FPNLGINC = 999._JPRB

  NFPMAX(1) = NSMAX
  NMFPMAX   = NMSMAX

  FPLON0=ELON0*ZRDTODG
  FPLAT0=ELAT0*ZRDTODG
  LFPMRT = LMRT
  FPLX=ELX
  FPLY=ELY
  LFPMAP=LMAP

ELSEIF (CFPFMT == 'GAUSS') THEN

  IF (NFPHTYP == 0) THEN
    DO JGL = 1,NLAT(1)
      NFPRGRI(JGL) = NLON(1)
    ENDDO
  ELSE
!   Ensure symetry of the grid
    DO JGL = 1, NLAT(1)/2
      NFPRGRI(NLAT(1)+1-JGL) = NFPRGRI(JGL)
    ENDDO
  ENDIF

  NMFPMAX = 1

  FPLON0 = 0._JPRB
  FPLAT0 = 0._JPRB
  LFPMRT = .FALSE.
  FPRPK = -9._JPRB
  FPLX = 0._JPRB
  FPLY = 0._JPRB
  LFPMAP = .TRUE.

ELSEIF (CFPFMT == 'LELAM') THEN

  LFPMAP  = FPSTRET >= 0.0_JPRB
  FPSTRET = 1._JPRB
  NFPHTYP = 0
  DO JGL = 1,NLAT(1)
    NFPRGRI(JGL) = NLON(1)
  ENDDO

ENDIF

IF (NFPTTYP == 1) THEN
  FPMUCEN = 1._JPRB
  FPLOCEN = 0._JPRB
ENDIF

IF (.NOT.LFPMAP) THEN
  FPRPK = -9._JPRB
ENDIF

IERR = 0

IF (NFPTTYP /=2 .AND. NFPTTYP /=1) THEN
  IERR = IERR+1
  WRITE(NULERR,*) 'ERROR NFPTTYP'
ENDIF

IF (CFPFMT == 'GAUSS'.OR.(CFPFMT == 'MODEL'.AND..NOT.LELAM)) THEN

  IF ((FPSTRET-1.0_JPRB) < -ZEPS) THEN
    IERR = IERR+1
    WRITE(NULERR,*)  'FPSTRET<1. FORBIDDEN, CHANGE THE POLE !'
  ENDIF
  IF (NFPHTYP /= 2.AND.NFPHTYP /= 0) THEN
    IERR = IERR+1
    WRITE(NULERR,*) 'ERROR NFPHTYP'
  ELSEIF (NFPHTYP /= 0) THEN
    DO JGL = 1, (NLAT(1)+1)/2
      IF (NFPRGRI(JGL) < 1.OR. NFPRGRI(JGL) > NLON(1)) THEN
        IERR = IERR+1
        WRITE (UNIT=CL,FMT='(''ERROR NFPRGRI('',I3,'')'')') JGL
        WRITE(NULERR,*) CL
        CL = ' '
      ENDIF
    ENDDO
  ENDIF

ELSEIF (CFPFMT == 'LELAM'.OR.(CFPFMT == 'MODEL'.AND.LELAM)) THEN
  IF (LFPMRT .AND. (ABS(FPLAT0) >= ZEPS)) THEN
    WRITE(NULOUT,*) 'SUFPG1: FPLAT0=',FPLAT0,&
    & ' MUST BE EQUAL ZERO IF LFPMRT IS TRUE!'
    CALL ABOR1('SUFPG1: LFPMRT & FPLAT0 INCONSISTENT')
  ENDIF
ENDIF

IF (NFPLEV > JPMXLE) THEN
  IERR = IERR+1
  WRITE(NULERR,*)  ' NFPLEV BIGGER THAN PARAMETER JPMXLE'
ENDIF

IF ((ABS(FPVALH(NFPLEV)) > ZEPS).OR.&
   & (ABS(FPVBH(0)) > ZEPS).OR.&
   & (ABS(FPVBH(NFPLEV)-1.0_JPRB) > ZEPS)) THEN  
  IERR=IERR+1
  WRITE(NULERR,*)  ' WRONG LIMITS *FPVALH* OR *FPVBH*'
ENDIF
IF (NFP3S > NFPLEV) THEN
  IERR = IERR+1
  WRITE (UNIT=CL,FMT='(''NFP3S CANNOT BE GREATER THAN NFPLEV'')')
  WRITE(NULERR,*) CL
  CL = ' '
ENDIF
DO JLEV = 1, NFP3S
  IF (NRFP3S(JLEV) < 1.OR. NRFP3S(JLEV) > NFPLEV) THEN
    IERR = IERR+1
    WRITE (UNIT=CL,FMT='(''ERROR NRFP3S('',I3,'')'')') JLEV
    WRITE(NULERR,*) CL
  ENDIF
ENDDO

DO J = 1, NFPDOM
  IF (NFPMAX(J) <= 0) THEN
    IERR = IERR+1
    WRITE (UNIT=CL,FMT='(''ERROR NFPMAX('',I2,'')'')') J
    WRITE(NULERR,*) CL
    CL = ' '
  ENDIF
ENDDO
IF (NMFPMAX < 0) THEN
  IERR = IERR+1
  WRITE (UNIT=CL,FMT='(''ERROR NMFPMAX'')')
  WRITE(NULERR,*) CL
  CL = ' '
ENDIF

IF (IERR > 0) CALL ABOR1('SUFPG1: ABOR1 CALLED')

LDSAME=.TRUE.
IF ((CFPFMT == 'GAUSS'.OR.CFPFMT == 'MODEL').AND..NOT.LELAM) THEN
  IF (ABS(FPSTRET-RSTRET) > ZEPS) LDSAME=.FALSE.
  IF (NFPTTYP /= NSTTYP)           LDSAME=.FALSE.
  IF (ABS(FPMUCEN-RMUCEN) > ZEPS) LDSAME=.FALSE.
  IF (ABS(FPLOCEN-RLOCEN) > ZEPS) LDSAME=.FALSE.
  IF (NLON(1) /= NDLON)            LDSAME=.FALSE.
  IF (NLAT(1) /= NDGLG) THEN
    LDSAME=.FALSE.
  ELSE
    DO JGL=1,NDGLG
      IF (NFPRGRI(JGL) /= NLOENG(JGL)) THEN
        LDSAME=.FALSE.
        EXIT
      ENDIF
    ENDDO
  ENDIF
ELSEIF((CFPFMT == 'LELAM'.OR.CFPFMT == 'MODEL').AND.LELAM) THEN
  IF (LMAP) THEN
    Z=ABS(MOD(FPLON0  -ELON0*ZRDTODG,360._JPRB))
    IF (Z > ZEPS.AND.(360._JPRB-Z) > ZEPS) LDSAME=.FALSE.
    Z=ABS(MOD(FPLAT0  -ELAT0*ZRDTODG,360._JPRB))
    IF (Z > ZEPS.AND.(360._JPRB-Z) > ZEPS) LDSAME=.FALSE.
    Z=ABS(MOD(RLATC(1)-ELATC*ZRDTODG,360._JPRB))
    IF (Z > ZEPS.AND.(360._JPRB-Z) > ZEPS) LDSAME=.FALSE.
    Z=ABS(MOD(RLONC(1)-ELONC*ZRDTODG,360._JPRB))
    IF (Z > ZEPS.AND.(360._JPRB-Z) > ZEPS) LDSAME=.FALSE.
    IF ((LMRT.AND..NOT.LFPMRT).OR.(.NOT.LMRT.AND.LFPMRT)) LDSAME=.FALSE.
    IF (ABS(RDELX(1)-EDELX) > ZEPS) LDSAME=.FALSE.
    IF (ABS(RDELY(1)-EDELY) > ZEPS) LDSAME=.FALSE.
  ELSE
    IF (ABS(FPLX-ELX) > ZEPS)        LDSAME=.FALSE.
    IF (ABS(FPLY-ELY) > ZEPS)        LDSAME=.FALSE.
  ENDIF
  IF (NFPLUX /= NDLUXG)               LDSAME=.FALSE.
  IF (NFPGUX /= NDGUXG)               LDSAME=.FALSE.
  IF (NFPTTYP /=1)                    LDSAME=.FALSE.
ELSE
  LDSAME=.FALSE.
ENDIF

LDESAME=LELAM.AND.LDSAME.AND.(NLON(1) == NDLON).AND.(NLAT(1) == NDGLG)

LDEZO=(CFPFMT == 'LELAM').AND.&
 &    (NLAT(1)*NLON(1)-NFPLUX*NFPGUX > 0).AND.&
 &    .NOT.LDESAME

!*       4. COMPUTE LINEAR TO QUADRATIC GLOBAL GRID & SINES OF LATITUDES
!           ------------------------------------------------------------

IF (CFPFMT == 'GAUSS') THEN

  ! ky: Calls a version of SUGAW which should be the mirror of the
  !     current version of tfl/module/sugaw_mod.F90.
  !     We can probably call tfl/module/sugaw_mod.F90 there
  !     but this must be checked (modification planned after CY36).

  ! * preliminary calculations to compute input quantity ZFNLAT
  !   (k.y.: coded after what I found in tfl/module/suleg_mod.F90).
  IDGLG=NLAT(1)
  ISTART=1
  ALLOCATE(ZFN(0:IDGLG,0:IDGLG))
  ALLOCATE(ZFNLAT(0:IDGLG/2))
  ! Belousov, Swarztrauber use ZFN(0,0)=SQRT(2._JPRB)
  ! IFS normalisation chosen to be 0.5*Integral(Pnm**2) = 1
  ZFN(0,0)=2._JPRB
  DO JN=ISTART,IDGLG
    ZFNN=ZFN(0,0)
    DO JGL=1,JN 
      ZFNN=ZFNN*SQRT(1._JPRB-0.25_JPRB/REAL(JGL**2,JPRB))
    ENDDO
    IODD=MOD(JN,2)
    ZFN(JN,JN)=ZFNN
    DO JGL=2,JN-IODD,2
      ZFN(JN,JN-JGL)=ZFN(JN,JN-JGL+2) &
       & *REAL((JGL-1)*(2*JN-JGL+2),JPRB)/REAL(JGL*(2*JN-JGL+1),JPRB)
    ENDDO
  ENDDO
  IODD=MOD(IDGLG,2)
  IK=IODD
  DO JGL=IODD,IDGLG,2
    ZFNLAT(IK)=ZFN(IDGLG,JGL)
    IK=IK+1
  ENDDO

  ! * call to SUGAWA:
  CALL SUGAWA(NLAT(1),ZFNLAT,RFPMU,ZFPMU,ZW)

  ! * deallocations:
  DEALLOCATE(ZFN)
  DEALLOCATE(ZFNLAT)

  DO JGL = 1, NLAT(1)
    ZSQM2(JGL) = SQRT(1.0_JPRB-RFPMU(JGL)*RFPMU(JGL))
  ENDDO
  CALL SUNMEN(NFPMAX(1),NFPHTYP,NLAT(1),NFPRGRI,ZSQM2,1,NLAT(1),&
   & NFPMEN,FPNLGINC)  
ELSEIF (CFPFMT == 'MODEL'.AND..NOT.LELAM) THEN
  DO JGL = 1,NDGLG
    RFPMU (JGL) = RMU  (JGL)
    NFPMEN(JGL) = NMENG(JGL)
  ENDDO
ENDIF

!*       5. GLOBAL NUMBER OF OUTPUT POINTS
!           ------------------------------

KFPRGP = 0
NFPEG  = 0
NFPSIZEG(:)=0
DO J=1, NFPDOM
  IF (CFPFMT == 'GAUSS'.OR.(CFPFMT == 'MODEL'.AND..NOT.LELAM)) THEN
!         Gaussian grid
    DO JGL = 1, NLAT(J)
      NFPSIZEG(J) = NFPSIZEG(J) + NFPRGRI(JGL)
    ENDDO
    NFPSIZEL(J) = 0
  ELSEIF (CFPFMT == 'LELAM'.OR.(CFPFMT == 'MODEL'.AND.LELAM)) THEN
!         LAM domain => 
    IF (.NOT.LDSAME.OR.LDEZO) THEN
!     select C+I area if true horizontal processing
!     (interpolations or biperiodicisation)
      NFPSIZEG(J)=NFPLUX*NFPGUX
      NFPSIZEL(J)=NLAT(J)*NLON(J)-NFPLUX*NFPGUX
    ELSE
!           else : full buffer transfers
      NFPSIZEG(J)=NLAT(J)*NLON(J)
      NFPSIZEL(J)=0
    ENDIF
  ELSE
!         LAtxLON subdomains
    NFPSIZEG(J)=NLAT(J)*NLON(J)
    NFPSIZEL(J)=0
  ENDIF
  KFPRGP=KFPRGP+NFPSIZEG(J)
  NFPEG=NFPEG+NFPSIZEL(J)
ENDDO

!*       6. PRINT OUT FINAL VALUES
!           ----------------------

IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
  WRITE(UNIT=NULOUT,FMT='('' COMMON YOMFPG'')')
  WRITE(UNIT=NULOUT,FMT='('' NFPTTYP = '',&
   & I6,'' NFPHTYP = '',I6)') NFPTTYP, NFPHTYP  
  WRITE(UNIT=NULOUT,FMT='('' FPMUCEN = '',E15.7,'' FPLOCEN = '',&
   & E15.7,'' FPSTRET = '',E15.7,'' FPNLGINC = '',E15.7)') &
   & FPMUCEN,FPLOCEN,FPSTRET,FPNLGINC  
  IF (CFPFMT == 'GAUSS'.OR.(CFPFMT == 'MODEL'.AND..NOT.LELAM)) THEN
    WRITE(UNIT=NULOUT,FMT='('' (J,NFPRGRI,NFPMEN) '')')
    WRITE(UNIT=NULOUT,FMT='(8(1X,''('',I4,I4,I4,'')''))')&
     & (J,NFPRGRI(J),NFPMEN(J),J = 1,NLAT(1))  
  ENDIF
  WRITE(UNIT=NULOUT,FMT='('' LFPMAP = '',L2)') LFPMAP
  IF (.NOT.LFPMAP) THEN
    WRITE(UNIT=NULOUT,FMT='('' FPLX = '',E15.7,'' FPLY = '',E15.7)') FPLX, FPLY
  ENDIF
  WRITE(UNIT=NULOUT,FMT='('' FPLON0 = '',E15.7,'' FPLAT0 = '',E15.7)')&
   & FPLON0,FPLAT0  
  WRITE(UNIT=NULOUT,FMT='('' LFPMRT = '',L2)') LFPMRT
  WRITE(UNIT=NULOUT,FMT='('' (J  NFPMAX  NFPSIZEG NFPSIZEL) '')')
  WRITE(UNIT=NULOUT,FMT='(10('' ('',I2,2X,I5,2X,I5,2X,I5,'')''))')&
   & (J,NFPMAX(J),NFPSIZEG(J),NFPSIZEL(J), J = 1,NFPDOM)  
  WRITE(UNIT=NULOUT,FMT='('' NMFPMAX = '',I6, '' NFPEG = '',I6)')NMFPMAX, NFPEG

  WRITE(UNIT=NULOUT,FMT='('' NFPLEV  = '',I6)') NFPLEV
  DO JLEV = 0,NFPLEV
    WRITE(UNIT=NULOUT,FMT = '('' FPVALH('',I2,'') = '',&
     & F12.6,''     FPVBH('',I2,'') = '',F12.10)')&
     & JLEV,FPVALH(JLEV),JLEV,FPVBH(JLEV)  
  ENDDO
ENDIF
CALL INI_IOSTREAM(KFPMAX=NFPMAX)

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPG1',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPG1
