SUBROUTINE SUFPGRIB

!**** *SUFPGRIB* - Routine to intitialize parameters for GRIB coding
!                - used for full-pos output file

!     Purpose.
!     --------
!         Set up parameters for GRIB coding including GRIB codes
!     for all parameters.

!**   Interface.
!     ----------
!        *CALL* *SUFPGRIB*

!        Explicit arguments :  None.
!        --------------------

!        Implicit arguments :  COMMON YOMGRB.
!        --------------------

!     Method.  
!     -------  

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-03-01
!        Modified : 93-04-15 L.Labbe - NCONF=821 : 2D model
!        Modified : 93-12-02 M.Hamrud - ECMWF extended GRIB
!        Modified : 94-04-26 M.Hamrud - NSTEPLPP
!        Modified : 95-10-01 N. Wedi - full-pos use 
!        Modified : MARCH 1997 by K. YESSAD: parallelisation of FULL-POS.
!          - RFPLA(1) becomes RFPLA1.
!          - RFPLA(NLAT(1)) becomes RFPLANLAT1.
!        Modified : 98-10-31 D.Dent - MSTEPLPP changed to NSTEPLPP l-p
!        Modified : 98-10-06 R. El Khatib - replace RFPLA1, RFPLANLAT1 by
!                   ArcSin (RFPMU(1)), ArcSin(RFPMU(NLAT(1)))
!        Modified : 00-02-16  T. Stockdale - seasonal stream
!        Modified : 01-05-31  D. Richardson - multi-analysis stream
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        S. Serrar    : 05-06-23 missing values to -9999.
!                       (in case section 3 defined for GRIB)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NCONF
USE YOMGRB   , ONLY : JPN3SP   ,JPN2SP   ,MSEC1    ,MSEC2SPP ,&
 & MSEC2SPM ,MSEC3    ,MSEC4    ,MGRBS3   ,MGRBS2   ,&
 & NSTEPLPP ,SSEC3    ,MSEC2GG  ,SSEC2    ,&
 & MSMAXNP  ,&
 & NGRBZ    ,NGRBT    ,NGRBQ    ,NGRBVO   ,NGRBLNSP ,&
 & NGRBD    ,&
 & NSEC1

USE YOMCST   , ONLY : RPI
USE YOMFPD   , ONLY : NLAT     ,NLON
USE YOMFPG   , ONLY : NFPMAX   ,NFPRGRI  ,FPVALH   ,FPVBH    ,&
 & NFPLEV   ,NFPHTYP  ,NFPTTYP  ,FPMUCEN  ,FPLOCEN  ,&
 & FPSTRET  ,RFPMU  

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IDGNH, ILAT, J
LOGICAL      :: LLFIRST=.TRUE.
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!     ------------------------------------------------------------------

!*       0.    READ NAMELIST
!              -------------

IF (LHOOK) CALL DR_HOOK('SUFPGRIB',0,ZHOOK_HANDLE)
IF(NFPMAX(1) >= 213) THEN
  MSMAXNP=20
ELSE
  MSMAXNP=MIN(10,NFPMAX(1))
ENDIF

MSMAXNP=MIN(MSMAXNP,NFPMAX(1))

!     ------------------------------------------------------------------

!*       1.    SET DEFAULTS FOR INPUT TO GRIBEX.
!              ---------------------------------

! Section 1 from NSEC1, see routine sugrib

MSEC1(:) = NSEC1(:)


!*       1.2   INTEGER BLOCK 2, SPECTRAL REPRESENTATION
!*             PRESSURE/POT. TEMPERATURE/POT.VORTICITY LEVELS
IF(NFPTTYP >= 2) THEN
  MSEC2SPP(1)=80
ELSEIF(ABS(FPSTRET-1.0_JPRB) >= 1.E-14_JPRB) THEN
  MSEC2SPP(1)=70
ELSE
  MSEC2SPP(1)=50
ENDIF
MSEC2SPP(2)=NFPMAX(1)
MSEC2SPP(3)=NFPMAX(1)
MSEC2SPP(4)=NFPMAX(1)
MSEC2SPP(5)=1
MSEC2SPP(6)=2
DO J=7,11
  MSEC2SPP(J)=0
ENDDO
MSEC2SPP(12)=0
MSEC2SPP(13)=0
MSEC2SPP(14)=0
IF(NFPTTYP >= 2) THEN
  MSEC2SPP(15)=180._JPRB/RPI*1000._JPRB*ASIN(FPMUCEN)
  MSEC2SPP(16)=180._JPRB/RPI*1000._JPRB*FPLOCEN
ELSE
  MSEC2SPP(15)=0
  MSEC2SPP(16)=0
ENDIF
DO J=17,22
  MSEC2SPP(J)=0
ENDDO

!*       1.3   INTEGER BLOCK 2, SPECTRAL REPRESENTATION,MODEL LEVELS.
IF(NFPTTYP >= 2) THEN
  MSEC2SPM(1)=80
ELSEIF(ABS(FPSTRET-1.0_JPRB) >= 1.E-14_JPRB) THEN
  MSEC2SPM(1)=70
ELSE
  MSEC2SPM(1)=50
ENDIF
MSEC2SPM(2)=NFPMAX(1)
MSEC2SPM(3)=NFPMAX(1)
MSEC2SPM(4)=NFPMAX(1)
MSEC2SPM(5)=1
MSEC2SPM(6)=2
DO J=7,11
  MSEC2SPM(J)=0
ENDDO
MSEC2SPM(12)=2*(NFPLEV+1)
MSEC2SPM(13)=0
MSEC2SPM(14)=0
IF(NFPTTYP >= 2) THEN
  MSEC2SPM(15)=180._JPRB/RPI*1000._JPRB*ASIN(FPMUCEN)
  MSEC2SPM(16)=180._JPRB/RPI*1000._JPRB*FPLOCEN
ELSE
  MSEC2SPM(15)=0
  MSEC2SPM(16)=0
ENDIF
DO J=17,22
  MSEC2SPM(J)=0
ENDDO

!*       1.4   INTEGER BLOCK 2, GAUSSIAN GRID REPRESENTATION.
IF(NFPTTYP >= 2) THEN
  MSEC2GG(1)=34
ELSEIF(ABS(FPSTRET-1.0_JPRB) >= 1.E-14_JPRB) THEN
  MSEC2GG(1)=24
ELSE
  MSEC2GG(1)=4
ENDIF
MSEC2GG(1)=4
MSEC2GG(2)=NLON(1)
ILAT=NLAT(1)
MSEC2GG(3)=ILAT
MSEC2GG(4)=180._JPRB/RPI*1000._JPRB*ASIN(RFPMU(1))
MSEC2GG(5)=0
MSEC2GG(6)=0
MSEC2GG(7)=180._JPRB/RPI*1000._JPRB*ASIN(RFPMU(NLAT(1)))
MSEC2GG(8)=(360*1000*(NLON(1)-1))/NLON(1)
MSEC2GG(9)=360*1000/NLON(1)
IDGNH =(ILAT+1)/2
MSEC2GG(10)=IDGNH
MSEC2GG(11)=0
MSEC2GG(12)=0
MSEC2GG(13)=0
MSEC2GG(14)=0
IF(NFPTTYP >= 2) THEN
  MSEC2GG(15)=180._JPRB/RPI*1000._JPRB*ASIN(FPMUCEN)
  MSEC2GG(16)=180._JPRB/RPI*1000._JPRB*FPLOCEN
ELSE
  MSEC2GG(15)=0
  MSEC2GG(16)=0
ENDIF
MSEC2GG(17)=MIN(NFPHTYP,1)
MSEC2GG(18)=0
!     MSEC2GG(19)=????
MSEC2GG(19)=0
MSEC2GG(20)=0
MSEC2GG(21)=0
MSEC2GG(22)=0

! Poles not added in reduced grid table 
! use NFPRGRI(1) for NPole and SPole  

DO J=1,NLAT(1)
  MSEC2GG(22+J)=NFPRGRI(J)
ENDDO

!*       1.5   INTEGER BLOCK 3
MSEC3(1)=0
MSEC3(2)=-9999

!*       1.6   INTEGER BLOCK 4
DO J=1,42
  MSEC4(J)=0
ENDDO

!*       1.7   REAL BLOCK 2

IF(.NOT.ALLOCATED(SSEC2)) THEN
  ALLOCATE(SSEC2  (10+2*(NFPLEV+1)))
  WRITE(NULOUT,9990) 'SSEC2    ',SIZE(SSEC2),SHAPE(SSEC2)
  9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
ENDIF

SSEC2(1)=0._JPRB
SSEC2(2)=FPSTRET
DO J=3,10
  SSEC2(J)=0.0_JPRB
ENDDO

DO J=0,NFPLEV
  SSEC2(J+11)=FPVALH(J)
  SSEC2(J+11+NFPLEV+1)=FPVBH(J)
ENDDO

!*       1.8   REAL BLOCK 3
SSEC3(1)=0._JPRB
SSEC3(2)=-9999._JPRB

!     ------------------------------------------------------------------

!*       2.    SET UP GRIB FIELD CODES.
!              ------------------------

!*       2.1   SPECTRAL FIELDS.
DO J=0,JPN3SP
  MGRBS3(J)=0
ENDDO
MGRBS3(1)=NGRBVO
MGRBS3(2)=NGRBD
MGRBS3(3)=NGRBT
MGRBS3(4)=NGRBQ
DO J=0,JPN2SP
  MGRBS2(J)=0
ENDDO
IF(NCONF == 201.OR.NCONF == 421.OR.NCONF == 521) THEN
  MGRBS2(1)=NGRBZ
ELSE
  MGRBS2(1)=NGRBLNSP
ENDIF

!     ------------------------------------------------------------------

!*       3.    SET TIME OF PREVIOUS P.P. TO 0 FOR ALL GRIB CODES.
!              --------------------------------------------------

IF(LLFIRST) THEN
  DO J=128,255
    NSTEPLPP(J)=0
  ENDDO
  LLFIRST=.FALSE.
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPGRIB',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPGRIB
