SUBROUTINE SUFPOPH

!**** *SUFPOPH*  - INITIALIZE POST-PROCESSING FILE-HANDLING PARAMETERS

!     PURPOSE.
!     --------
!        INITIALIZE COMMON YOMFPOP WHICH CONTAINS POST-PROCESSING
!            FILE-HANDLING PARAMETERS

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPOPH*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE.

!        IMPLICIT ARGUMENTS
!        --------------------
!        YOMFPOP
!        YOMFPC
!        PARFPOS, PARDIM

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        g radnoti: 95-02-28: aladin case
!        R. El Khatib : 97-07-17 : 
!         - case ALADIN files : no recomputation of corners by EGGX
!        Modified 98-08-10 by K. YESSAD: removal of LFPOLE option.
!         - removal of LFPOLE in YOMFPG.
!         - removal of LFPOLE in 'CALL SUFRAME'.
!        R. El Khatib : 99-03-10 : Variables for bogussing ; frame name more robust.
!        M. Janousek : 01-04-14 : New domain and projection parameters
!        R. El Khatib : 01-08-07 Pruning options
!        R. El Khatib : 02-09-03 Fix for aladin 2D model
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        JD. GRIL 10-02-2005 Add flag for Mercator Rotated/Tilted
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : CNMEXP   ,LARPEGEF ,LELAM
USE YOMFPC   , ONLY : CFPDIR   ,CFPFMT   ,CFPDOM   ,NFPDOM   ,&
 & NFPINCR ,LTRACEFP  
USE YOMFPD   , ONLY : NLAT     ,NLON     ,RLATN    ,RLATS    ,&
 & RLONW    ,RLONE    ,NFPLUX   ,&
 & NFPGUX   ,&
 & RLATC    ,RLONC    ,RDELX    ,RDELY  
USE YOMFPG   , ONLY : NFPMAX   ,NFPRGRI  ,FPVALH   ,FPVBH    ,&
 & RFPMU    ,NFPMEN   ,NFPLEV   ,&
 & NFPTTYP  ,NMFPMAX  ,&
 & FPMUCEN  ,FPLOCEN  ,FPSTRET  ,&
 & FPLX     ,FPLY     ,FPLON0   ,&
 & FPLAT0   ,FPRPK    ,LFPMAP   ,LFPMRT  
USE YOMFPOP  , ONLY : CFPFN    ,CFPCA    ,CFPFNBG  ,CFPCABG
USE YOMCST   , ONLY : RPI
USE YOMGEM   , ONLY : VP00

IMPLICIT NONE

REAL(KIND=JPRB) :: ZVALH(0:NFPLEV)

INTEGER(KIND=JPIM) :: ILENPR, ISTRON, ITRONC, J, JDOM, JLEV, IDOM
INTEGER(KIND=JPIM) :: IBZONL, IBZONG

LOGICAL :: LLGARD, LLFPMRT

REAL(KIND=JPRB) :: ZLAT0, ZLATN, ZLATC, ZLATS, ZLON0,&
 & ZLONE, ZLONC, ZLONW, ZXWN, ZYWN, ZDELX, ZDELY  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sueframe.intfb.h"
#include "sufpgrib.intfb.h"
#include "suframe.intfb.h"

!*       1. SET FILE NAME FORMATS & FRAMES
!           ------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPOPH',0,ZHOOK_HANDLE)
ILENPR=INDEX(CFPDIR,' ')-1
DO J=1, NFPDOM
  CFPFN(J)=CFPDIR(1:ILENPR)//CNMEXP(1:4)//CFPDOM(J)
  WRITE(UNIT=CFPCA(J),FMT='(''CADRE.SORTIE.#'',I2.2)') J
  IF (NFPINCR /= 0) THEN
    CFPFNBG(J)='BGPX'//CNMEXP(1:4)//CFPDOM(J)
    WRITE(UNIT=CFPCABG(J),FMT='(''CADRE.BOGUSS.#'',I2.2)') J
  ENDIF
ENDDO

IF (LARPEGEF) THEN

  DO JLEV=0,NFPLEV
    ZVALH(JLEV)=FPVALH(JLEV)/VP00
  ENDDO
  LLGARD = .TRUE.

!*       2. INITIALISE OUTPUT FRAME OF FILE *ARPEGE*
!           ----------------------------------------

  IF (CFPFMT == 'GAUSS'.OR.(CFPFMT == 'MODEL'.AND..NOT.LELAM)) THEN
    CALL SUFRAME(CFPCA(1),FPMUCEN,FPLOCEN,FPSTRET,NFPTTYP,&
     & NFPMAX(1),NLAT(1),NLON(1),NFPRGRI,NFPMEN,RFPMU,&
     & LLGARD,NFPLEV,VP00,ZVALH,FPVBH)  

!*       3. INITIALISE OUTPUT FRAME OF FILE *ALADIN*
!           ----------------------------------------

  ELSEIF (CFPFMT == 'LELAM'.OR.(CFPFMT == 'MODEL'.AND.LELAM)) THEN
    ZLONC=RLONC(1)*RPI/180._JPRB
    ZLATC=RLATC(1)*RPI/180._JPRB
    ZLATN=RLATN(1)*RPI/180._JPRB
    ZLATS=RLATS(1)*RPI/180._JPRB
    ZLONE=RLONE(1)*RPI/180._JPRB
    ZLONW=RLONW(1)*RPI/180._JPRB
    ZLON0=FPLON0*RPI/180._JPRB
    ZLAT0=FPLAT0*RPI/180._JPRB
    LLFPMRT=LFPMRT
    ZXWN=2.0_JPRB*RPI/FPLX
    ZYWN=2.0_JPRB*RPI/FPLY
    ISTRON=MAX(0,MIN(11,NMFPMAX,NFPMAX(1))-1)
    IF (CFPFMT == 'MODEL') THEN
      IDOM = 1
    ELSEIF (CFPFMT == 'LELAM' .AND. &
       & .NOT.(NLON(1) == NFPLUX .AND. NLAT(1) == NFPGUX)) THEN  
      IDOM = -1
    ELSE
      IDOM = 0
    ENDIF
    IBZONL=MIN(8,(NFPLUX-1)/2)
    IBZONG=MIN(8,(NFPGUX-1)/2)
    CALL SUEFRAME(CFPCA(1),LFPMAP,NMFPMAX,NFPMAX(1),NLAT(1),NLON(1),&
     & LLGARD,LLFPMRT,&
     & ZLON0,ZLAT0,ZLONC,ZLATC,RDELX(1),RDELY(1),&
     & ZLONW,ZLATS,ZLONE,ZLATN,FPRPK,&
     & FPLX,FPLY,ZXWN,ZYWN,1,NFPLUX,1,NFPGUX,IBZONL,IBZONG,NFPLEV,VP00,&
     & ZVALH,FPVBH,IDOM,ISTRON)  

!*       4. INITIALISE OUTPUT FRAMES OF FILE *LATLON*
!           -----------------------------------------

  ELSE
    DO J=1, NFPDOM
      ZLONC=RLONC(J)*RPI/180._JPRB
      ZLATC=RLATC(J)*RPI/180._JPRB
      ZDELX=RDELX(J)*RPI/180._JPRB
      ZDELY=RDELY(J)*RPI/180._JPRB
      ZLATN=RLATN(J)*RPI/180._JPRB
      ZLATS=RLATS(J)*RPI/180._JPRB
      ZLONE=RLONE(J)*RPI/180._JPRB
      ZLONW=RLONW(J)*RPI/180._JPRB
      ZLON0=FPLON0*RPI/180._JPRB
      ZLAT0=FPLAT0*RPI/180._JPRB
      LLFPMRT=.FALSE.
      ZXWN=0._JPRB
      ZYWN=0._JPRB
      ITRONC=11 ! arbitrary values since there are no spectral fields in
      ISTRON=10 ! such files
      IDOM = 0
      IBZONL=MIN(8,(NLON(J)-1)/2)
      IBZONG=MIN(8,(NLAT(J)-1)/2)
      CALL SUEFRAME(CFPCA(J),LFPMAP,ITRONC,ITRONC,&
       & NLAT(J),NLON(J),LLGARD,LLFPMRT,&
       & ZLON0,ZLAT0,ZLONC,ZLATC,ZDELX,ZDELY,&
       & ZLONW,ZLATS,ZLONE,ZLATN,&
       & FPRPK,FPLX,FPLY,&
       & ZXWN,ZYWN,1,NLON(J),1,NLAT(J),&
       & IBZONL,IBZONG,NFPLEV,VP00,ZVALH,FPVBH,IDOM,&
       & ISTRON)  
    ENDDO
  ENDIF

!*      5. INITIALIZE PARAMETERS FOR GRIB CODING 
!          -------------------------------------

ELSE
  CALL SUFPGRIB
ENDIF

!*       6. PRINT OUT FILE NAMES AND FRAME NAMES
!           ------------------------------------

IF (NFPDOM >= 1) THEN
  IF(LTRACEFP) THEN
    WRITE(NULOUT,FMT='('' COMMON YOMFPOPH'')')
    DO JDOM=1, NFPDOM
      WRITE(NULOUT,FMT='('' FRAME = '',A16,'' FILE NAME = '',A180)')&
       & CFPCA(JDOM),CFPFN(JDOM)  
      IF (NFPINCR /= 0) THEN
        WRITE(NULOUT,FMT='('' BACKGROUND FRAME = '',A16,&
         & '' BACKGROUND FILE NAME = '',A180)') CFPCABG(JDOM),&
         & CFPFNBG(JDOM)  
      ENDIF
    ENDDO
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('SUFPOPH',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPOPH
