SUBROUTINE SUFPRFPDS

!**** *SUFPRFPDS*  - INITIALIZE FULL-POS AUXILARY FIELDS DESCRIPTORS

!     PURPOSE.
!     --------
!        INITIALIZE THE DESCRIPTORS OF THE AUXILARY FIELDS NEEDED FOR
!        HORIZONTAL POST-PROCESSING

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPRFPDS*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE.

!        IMPLICIT ARGUMENTS
!        --------------------
!        YOMRFPDS

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 96-01-15 R. El Khatib (to write out "real" wind in output 
!                   coordinates (x,y) in case of limited area output)
!        Modified : 96-12-13 R. El Khatib (output geometry needed in any case)
!        Modified : 97-07-23 by Cristina Madeira (ISBA Fields in FULL-POS)
!        Modified : 97-10-01 by Adam Dziedzic(ISBA schema Douville Fields)
!        R. El Khatib : 98-03-18 Interpolated orography not pre-computed
!        R. El Khatib : 98-08-28 Adiabatic pp.
!        R. El Khatib : 98-09-11 MFPFISO and MFPLSMO for output clim only
!        R. El Khatib : 98-09-11 LFPOSHOR
!        D. Giard     : 00-10-25 albedos for bare ground and vegetation
!        R. Randriamampianina 00-11-17 use of LVGSN
!        R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib : 18-Aug-2008 Limit interactions with physics
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE PARFPOS  , ONLY : JPOSPHY
USE YOMAFN   , ONLY : GFP_LSM  ,GFP_GFIS ,GFP_ST   ,GFP_DST  ,GFP_CSSW , &
 & GFP_CDSW ,GFP_SD   ,GFP_SR   ,GFP_BSR  ,GFP_AL   ,GFP_EMIS , &
 & GFP_SDOG ,GFP_VEG  ,GFP_ACOT ,GFP_DPAT ,GFP_IVEG ,GFP_RSMIN, &
 & GFP_ARG  ,GFP_SAB  ,GFP_D2   ,GFP_LAI  ,GFP_Z0H  ,GFP_BAAL , &
 & GFP_ALS  ,GFP_ALV  ,GFP_PHYDS,GFP_ASEA ,GFP_ALAN ,GFP_ASOO , &
 & GFP_ADES ,GFP_O3A  ,GFP_O3B  ,GFP_O3C  ,GFP_LAN
USE YOMFPC   , ONLY : NFPCLI   ,LTRACEFP ,NFPMASK  ,NFPPHY   ,MFPPHY   , &
 & CFPPHY 
USE YOMRFPDS , ONLY : MOFPTR   ,MFPVSO   ,NFPVSO   ,NRFPOS   ,MFPGMO   , &
 & MFPGNXO  ,MFPGNYO  
USE YOMPHY   , ONLY : LMPHYS   ,LSOLV
USE YOEPHY   , ONLY : LEPHYS
USE YOMFPG   , ONLY : LFPOSHOR
USE YOMCT0   , ONLY : LALLOPR  ,LARPEGEF
USE YOMARPHY , ONLY : LMSE

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IFPVSO(JPOSPHY) ! temporary list of climatology fields
INTEGER(KIND=JPIM) :: IPTR, ICOD, JFLD

CHARACTER(LEN=35) :: CLFMT='(1X,''ARRAY '',A10,'' ALLOCATED '',8I8)'
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

!*       1. DEFAULT VALUES
!           --------------

IF (LHOOK) CALL DR_HOOK('SUFPRFPDS',0,ZHOOK_HANDLE)
MFPGMO   =0
MFPGNXO  =0
MFPGNYO  =0
MOFPTR(:)=0

!*       2. CLIMATOLOGICAL OUTPUT FIELDS
!           ----------------------------

IPTR=0

IF (NFPCLI > 0) THEN

  CALL SET_NEXT(GFP_GFIS%ICOD)

  IF (LMPHYS.OR.LEPHYS) THEN

    IF (NFPMASK >= 1) THEN
      CALL SET_NEXT(GFP_LSM%ICOD)
      IF (NFPMASK >= 2) THEN
        CALL SET_NEXT(GFP_LAN%ICOD)
      ENDIF
    ENDIF

    IF (NFPCLI > 1) THEN

      CALL SET_NEXT(GFP_ST%ICOD)
      CALL SET_NEXT(GFP_DST%ICOD)
      CALL SET_NEXT(GFP_CSSW%ICOD)
      CALL SET_NEXT(GFP_SD%ICOD)
      CALL SET_NEXT(GFP_CDSW%ICOD)
      CALL SET_NEXT(GFP_AL%ICOD)
      CALL SET_NEXT(GFP_EMIS%ICOD)
      CALL SET_NEXT(GFP_SDOG%ICOD)
      CALL SET_NEXT(GFP_VEG%ICOD)
      CALL SET_NEXT(GFP_SR%ICOD)
      CALL SET_NEXT(GFP_ACOT%ICOD)
      CALL SET_NEXT(GFP_DPAT%ICOD)

    ENDIF

    IF (LSOLV.OR.LMSE) THEN
      CALL SET_NEXT(GFP_ARG%ICOD)
      CALL SET_NEXT(GFP_SAB%ICOD)
      CALL SET_NEXT(GFP_D2%ICOD)
      CALL SET_NEXT(GFP_IVEG%ICOD)
      CALL SET_NEXT(GFP_RSMIN%ICOD)
      CALL SET_NEXT(GFP_LAI%ICOD)
      CALL SET_NEXT(GFP_Z0H%ICOD)
      CALL SET_NEXT(GFP_BSR%ICOD)
      CALL SET_NEXT(GFP_BAAL%ICOD)
    ENDIF

    DO JFLD=1,NFPPHY
      IF (CFPPHY(JFLD)==GFP_ALS%CLNAME.OR.MFPPHY(JFLD)==GFP_ALS%IGRIB) THEN
        CALL SET_NEXT(GFP_ALS%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_ALV%CLNAME.OR.MFPPHY(JFLD)==GFP_ALV%IGRIB) THEN
        CALL SET_NEXT(GFP_ALV%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_ASEA%CLNAME.OR.MFPPHY(JFLD)==GFP_ASEA%IGRIB) THEN
         CALL SET_NEXT(GFP_ASEA%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_ALAN%CLNAME.OR.MFPPHY(JFLD)==GFP_ALAN%IGRIB) THEN
        CALL SET_NEXT(GFP_ALAN%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_ASOO%CLNAME.OR.MFPPHY(JFLD)==GFP_ASOO%IGRIB) THEN
        CALL SET_NEXT(GFP_ASOO%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_ADES%CLNAME.OR.MFPPHY(JFLD)==GFP_ADES%IGRIB) THEN
        CALL SET_NEXT(GFP_ADES%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_O3A%CLNAME.OR.MFPPHY(JFLD)==GFP_O3A%IGRIB) THEN
        CALL SET_NEXT(GFP_O3A%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_O3B%CLNAME.OR.MFPPHY(JFLD)==GFP_O3B%IGRIB) THEN
        CALL SET_NEXT(GFP_O3B%ICOD)
      ENDIF
      IF (CFPPHY(JFLD)==GFP_O3C%CLNAME.OR.MFPPHY(JFLD)==GFP_O3C%IGRIB) THEN
        CALL SET_NEXT(GFP_O3C%ICOD)
      ENDIF
    ENDDO

  ENDIF

ENDIF

NFPVSO=IPTR
ALLOCATE(MFPVSO(NFPVSO))
IF (LALLOPR) THEN
  WRITE(NULOUT,CLFMT) 'MFPVSO   ',SIZE(MFPVSO),SHAPE(MFPVSO)
ENDIF
MFPVSO(1:NFPVSO)=IFPVSO(1:NFPVSO)

!*       4. OUTPUT GEOMETRY
!           ---------------

IPTR=IPTR+1
MFPGMO=IPTR
IF (LFPOSHOR) THEN
  IPTR=IPTR+1
  MFPGNXO=IPTR
  IPTR=IPTR+1
  MFPGNYO=IPTR
ENDIF

!*       5. TOTAL NUMBER OF FIELDS
!           ----------------------

NRFPOS=IPTR

!*       6. PRINT FINAL VALUES
!           ------------------

IF(LTRACEFP) THEN
  WRITE(UNIT=NULOUT,FMT='('' MODULE YOMRFPDS'')')
  WRITE(UNIT=NULOUT,FMT='('' NFPVSO = '',I2,'' NRFPOS = '',I2)') NFPVSO, NRFPOS
  IF (NFPVSO > 0) THEN
    IF (LARPEGEF) THEN
      WRITE(NULOUT,FMT='(''(MOFPTR GFP_PHYDS%CLNAME)'')')
      WRITE(UNIT=NULOUT,FMT='(6('' ('',I2,1X,A16,'')''))') &
       & (MOFPTR(MFPVSO(JFLD)),GFP_PHYDS(MFPVSO(JFLD))%CLNAME, JFLD=1,NFPVSO)  
    ELSE
      WRITE(NULOUT,FMT='(''(MOFPTR GFP_PHYDS%IGRIB)'')')
      WRITE(UNIT=NULOUT,FMT='(6('' ('',I2,1X,I5,'')''))') &
       & (MOFPTR(MFPVSO(JFLD)),GFP_PHYDS(MFPVSO(JFLD))%IGRIB, JFLD=1,NFPVSO)  
    ENDIF
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('SUFPRFPDS',1,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

CONTAINS

SUBROUTINE SET_NEXT(KCOD)

INTEGER(KIND=JPIM), INTENT(IN)    :: KCOD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUFPRFPDS:SET_NEXT',0,ZHOOK_HANDLE)
IPTR=IPTR+1
IFPVSO(IPTR)=KCOD
MOFPTR(KCOD)=IPTR
IF (LHOOK) CALL DR_HOOK('SUFPRFPDS:SET_NEXT',1,ZHOOK_HANDLE)

END SUBROUTINE SET_NEXT

END SUBROUTINE SUFPRFPDS
