SUBROUTINE SUFPSC2B(KFIELDS,KSTABF,KLENBF,KPROMA,KPCT)

!**** *SUFPSC2B*  - POST-PROCESSING BUFFER/WORKFILE CONTROL

!     PURPOSE.
!     --------
!        To setup control arrays a dimensions of a buffer or workfile
!        for horizontal post-processing purpose

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPSC2B*

!        EXPLICIT ARGUMENTS
!        ------------------
!        INPUT : 
!          KFIELDS : number of fields in buffer/workfile
!          KPROMA : horizontal segmentation
!          KPCT   : number of packets in buffer/workfile
!        OUTPUT : 
!          KSTABF : start adresses of packets in buffer
!          KLENBF : buffer lenght

!        IMPLICIT ARGUMENTS
!        ------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION ABOUT FULL-POS.

!     EXTERNALS.
!     ----------
!        NONE.

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        Original 98-10-08 from SUFPSC2/SUFPIOS
!    R. El Khatib : 01-12-19 remove KLEN
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPCT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KSTABF(KPCT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLENBF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM) :: ILEN, JPCT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1. BUFFER LENGTH
!           -------------

IF (LHOOK) CALL DR_HOOK('SUFPSC2B',0,ZHOOK_HANDLE)
ILEN=KPROMA*KFIELDS
KLENBF=ILEN*KPCT

!*       2. START ADRESS OF EACH PACKET IN BUFFER
!           --------------------------------------
DO JPCT=1,KPCT
  KSTABF(JPCT)=ILEN*(JPCT-1)
ENDDO

IF (LHOOK) CALL DR_HOOK('SUFPSC2B',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPSC2B
