SUBROUTINE SUFPSUW

! Purpose :
! -------
!   *SUFPSUW* Compute weights for horizontal interpolations of surface fields

! Interface :
! ---------

! Externals :
! ---------
!   see below

! Method :
! ------

! Reference :
! ---------

! Author :
! ------
!   03-Aug-2005 R. El Khatib  *METEO-FRANCE*

! Modifications :
! -------------
!   K. Yessad: 28-Feb-2007: optimisation of distributed memory in FULL-POS.
! End Modifications
!-----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LELAM
USE YOMAFN   , ONLY : GFP_LSM  ,GFP_LAN
USE YOMDIM   , ONLY : NDGLG    ,NDGSAFPH ,NDGENFPH ,NDGUXG   ,NDLUXG
USE YOMLEG   , ONLY : RLATIG
USE YOMFPC   , ONLY : NFPMASK  ,NFPSURFEX
USE YOMFPG   , ONLY : NFPRGPL_DEP,NFPRGPL  ,RFPLA_DEP,RFPLO_DEP,LFPOSBUF ,&
 & LFPDISTRIB
USE YOMPHY1  , ONLY : TMERGL
USE YOMWFPDS , ONLY : LWLAN12  ,LWSEA12  ,LWLAN04  ,LWSEA04  ,LQUAD    ,&
 & NFPWIDE  ,LWIC
USE YOMWFPB  , ONLY : MLAT     ,MLON     ,MLOS     ,MLONN    ,MLOSS    ,&
 & WLAN04   ,WLAN12   ,WSEA04   ,WSEA12
USE YOMMP    , ONLY : NAFPB1   ,NFRSTLOFF
USE YEMGEO   , ONLY : EDELX    ,EDELY
USE YOMCST   , ONLY : RPI
USE YOMGEM   , ONLY : NLOENG   ,R4JP

!-----------------------------------------------------------------------------

IMPLICIT NONE

! ZDLAT : Meridian linear weight.
! ZDLO  : Zonal linear weight for the latitude of the point numbered 5.
! ZDLO1 : Zonal linear weight for the latitude of the point numbered 1.
! ZDLO2 : Zonal linear weight for the latitude of the point numbered 3.
! ZDLO3 : Zonal linear weight for the latitude of the point numbered 11.

REAL(KIND=JPRB) :: ZDLAT(NFPRGPL_DEP)
REAL(KIND=JPRB) :: ZDLO(NFPRGPL_DEP), ZDLO1(NFPRGPL_DEP)
REAL(KIND=JPRB) :: ZDLO2(NFPRGPL_DEP), ZDLO3(NFPRGPL_DEP)
REAL(KIND=JPRB) :: ZTSIN(NAFPB1), ZTSOUT(NFPRGPL_DEP)
REAL(KIND=JPRB) :: ZMASKIN(NAFPB1,1), ZMASKOUT(NFPRGPL,1)
REAL(KIND=JPRB) :: ZMASKOUT_DEP(NFPRGPL_DEP,1)
REAL(KIND=JPRB) :: ZLANDIN(NAFPB1,1), ZLANDOUT(NFPRGPL,1)
REAL(KIND=JPRB) :: ZLANDOUT_DEP(NFPRGPL_DEP,1)

INTEGER(KIND=JPIM) :: IEND,ILSIM,IST,J,IBINL,IBLOCK
INTEGER(KIND=JPIM) :: IC, IFLDS
INTEGER(KIND=JPIM) :: ICOD(1), IL0(NFPRGPL_DEP,0:NFPWIDE)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------------

#include "cpclimi.intfb.h"
#include "fpscaw.intfb.h"
#include "fptratod.intfb.h"
#include "suehow1.intfb.h"
#include "suehowlsm.intfb.h"
#include "sufpcip.intfb.h"
#include "suhow1.intfb.h"
#include "suhowlsm.intfb.h"

!-----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPSUW',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

! 1. Compute adresses & linear weights
!    ---------------------------------

IF (LFPOSBUF) THEN
  IF (LQUAD) THEN
    IBINL=0
  ELSE
    IBINL=1
  ENDIF
  IF (LELAM) THEN
    CALL SUEHOW1(NDGLG,NDGSAFPH,NDGENFPH,NFRSTLOFF,RPI,&
     & NLOENG(NDGSAFPH+NFRSTLOFF),&
     & IBINL,RFPLA_DEP,RFPLO_DEP,NFPRGPL_DEP,EDELX,EDELY,1,&
     & NDLUXG,1,NDGUXG,MLAT,MLON,MLONN,MLOS,MLOSS,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3)
  ELSE
    CALL SUHOW1(NDGSAFPH,NDGENFPH,NFRSTLOFF,R4JP,RPI,&
     & NLOENG(NDGSAFPH+NFRSTLOFF),RLATIG(NDGSAFPH+NFRSTLOFF),IBINL,RFPLA_DEP,&
     & RFPLO_DEP,NFPRGPL_DEP,MLAT,MLON,MLONN,MLOS,MLOSS,ZDLAT,&
     & ZDLO,ZDLO1,ZDLO2,ZDLO3)
  ENDIF
ENDIF


! 2. Compute weights with land-sea mask
!    ----------------------------------

! Adressing:
IST=1
IEND=NFPRGPL_DEP
IBLOCK=1
CALL FPSCAW(NFPRGPL_DEP,IST,IEND,IL0,IBLOCK)

! No use of sea/ice aspect => provisional initialization of Ts/SST:
ILSIM=1
DO J=1,NAFPB1
  ZTSIN(J)=TMERGL
ENDDO
DO J=1,NFPRGPL_DEP
  ZTSOUT(J)=TMERGL
ENDDO

! Land-Sea mask in input/output :
ICOD(1)=GFP_LSM%ICOD
CALL CPCLIMI(1,ICOD,ZMASKIN,ZMASKOUT)
IF (NFPMASK >= 1) THEN
  ! Land proportion in output :
  ICOD(1)=GFP_LAN%ICOD
  CALL CPCLIMI(1,ICOD,ZLANDIN,ZLANDOUT)
ENDIF

! Transposition between arrival geometry and departure geometry DM-partitions:
IF (LFPDISTRIB.AND.LFPOSBUF) THEN
  IC=1
  IFLDS=1
  CALL FPTRATOD(IC,IFLDS,ZMASKOUT,ZMASKOUT_DEP)
  CALL FPTRATOD(IC,IFLDS,ZLANDOUT,ZLANDOUT_DEP)
ELSE
  ! Simply copy ZMASKOUT in ZMASKOUT_DEP and ZLANDOUT in ZLANDOUT_DEP:
  ZMASKOUT_DEP(:,:)=ZMASKOUT(:,:)
  ZLANDOUT_DEP(:,:)=ZLANDOUT(:,:)
ENDIF

! Compute weights with Land-Sea mask
IF (NFPMASK >= 1 .AND. (LWLAN12 .OR. LWLAN04)) THEN
  IF (NFPMASK == 2) THEN
    IF (NFPSURFEX == 1) THEN
      ! Force land everywhere because we don't have land prop. from surfex
      DO J=1,NFPRGPL_DEP
        ZMASKOUT_DEP(J,1)=1._JPRB
      ENDDO
    ELSE
      DO J=1,NFPRGPL_DEP
        ZMASKOUT_DEP(J,1)=REAL(CEILING(ZLANDOUT_DEP(J,1)))
      ENDDO
    ENDIF
  ENDIF
  IF (LWLAN12) THEN
    IBINL=12
    IF (LELAM) THEN
      CALL SUEHOWLSM(NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),&
       & ILSIM,IBINL,NFPRGPL_DEP,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,EDELY,WLAN12)
    ELSE
      CALL SUHOWLSM(NDGSAFPH,NDGENFPH,NFRSTLOFF,RLATIG(NDGSAFPH+NFRSTLOFF),&
       & NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),ILSIM,&
       & IBINL,NFPRGPL_DEP,MLAT,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,WLAN12)
    ENDIF
  ENDIF
  IF (LWLAN04) THEN
    IBINL=4
    IF (LELAM) THEN
      CALL SUEHOWLSM(NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),&
       & ILSIM,IBINL,NFPRGPL_DEP,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,EDELY,WLAN04)
    ELSE
      CALL SUHOWLSM(NDGSAFPH,NDGENFPH,NFRSTLOFF,RLATIG(NDGSAFPH+NFRSTLOFF),&
       & NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),ILSIM,&
       & IBINL,NFPRGPL_DEP,MLAT,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,WLAN04)
    ENDIF
  ENDIF
  IF (LWIC) CALL SUFPCIP(ZMASKIN(1,1),ZMASKOUT_DEP(1,1),IL0)
ENDIF

! Compute weights with Sea mask
IF (NFPMASK >= 2 .AND. (LWSEA12 .OR. LWSEA04)) THEN
  IF (NFPSURFEX == 1) THEN
    ! Force sea everywhere because we don't have land prop. from surfex
    DO J=1,NFPRGPL_DEP
      ZMASKOUT_DEP(J,1)=0._JPRB
    ENDDO
  ELSE
    DO J=1,NFPRGPL_DEP
      ZMASKOUT_DEP(J,1)=REAL(CEILING(1._JPRB-ZLANDOUT_DEP(J,1)))
    ENDDO
  ENDIF
  IF (LWSEA12) THEN
    IBINL=12
    IF (LELAM) THEN
      CALL SUEHOWLSM(NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),&
       & ILSIM,IBINL,NFPRGPL_DEP,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,EDELY,WSEA12)
    ELSE
      CALL SUHOWLSM(NDGSAFPH,NDGENFPH,NFRSTLOFF,RLATIG(NDGSAFPH+NFRSTLOFF),&
       & NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),ILSIM,&
       & IBINL,NFPRGPL_DEP,MLAT,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,WSEA12)
    ENDIF
  ENDIF
  IF (LWSEA04) THEN
    IBINL=4
    IF (LELAM) THEN
      CALL SUEHOWLSM(NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),&
       & ILSIM,IBINL,NFPRGPL_DEP,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,EDELY,WSEA04)
    ELSE
      CALL SUHOWLSM(NDGSAFPH,NDGENFPH,NFRSTLOFF,RLATIG(NDGSAFPH+NFRSTLOFF),&
       & NAFPB1,TMERGL,ZTSIN,ZMASKIN(1,1),ZTSOUT,ZMASKOUT_DEP(1,1),ILSIM,&
       & IBINL,NFPRGPL_DEP,MLAT,IL0,ZDLAT,ZDLO,ZDLO1,ZDLO2,ZDLO3,WSEA04)
    ENDIF
  ENDIF
ENDIF

!-----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPSUW',1,ZHOOK_HANDLE)

END SUBROUTINE SUFPSUW
