SUBROUTINE SUFPVSET_DIR(KVSETSC,KVSETVD)

!**** *SUFPVSET_DIR * - Setup V-set for Full-Pos direct transforms.

!     Purpose.
!     --------
!        To setup V-set for Full-Pos direct transforms.

!**   Interface.  CALL SUFPVSET_DIR(...)
!     ---------- 

!     Explicit arguments : 
!     --------------------
!        KVSETSC : V-set for each scalar field
!        KVSETVD : V-set for each pair of field (Vorticity,Divergence)
!                  (they MUST be on the same V-set)

!     Externals.
!     ----------
!        None.

!     Reference.
!     ----------
!        Fullpos technical & users guide.

!     Author.
!     -------
!        Ryad El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 03-02-11 From TRANSDIR_FP
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN ,NFPTRDYN
USE YOMCT0   , ONLY : NPRTRV
USE YOMFPDIM , ONLY : NFPSCAG  ,NFPVECG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(OUT)   :: KVSETSC(NFPSCAG) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KVSETVD(NFPVECG) 
INTEGER(KIND=JPIM) :: J, JF, JLEV, IJ, IPSCA, IPVEC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPVSET_DIR',0,ZHOOK_HANDLE)
SELECT CASE (NPRTRV)
CASE (1)
  KVSETSC(:)=1
  KVSETVD(:)=1
CASE (2:)
  IPSCA=0
  IPVEC=0
  DO J=1,NFPOSDYN
    IJ=NFPTRDYN(J)
    SELECT CASE (QFPTYPE%ISF(IJ))
    CASE (1:)
      DO JLEV=1,QFPTYPE%ILEV(IJ)
        SELECT CASE (QFPTYPE%ILED(IJ))
        CASE (:-1)
!               V (U is on the same V-set)
          DO JF=1,QFPTYPE%ISKP(IJ)
            IPVEC=IPVEC+1
            KVSETVD(IPVEC)=QFPTYPE%ISET(JLEV,IJ)
          ENDDO
        CASE (0)
!               Scalar
          DO JF=1,QFPTYPE%ISKP(IJ)
            IPSCA=IPSCA+1
            KVSETSC(IPSCA)=QFPTYPE%ISET(JLEV,IJ)
          ENDDO
        CASE (2,3)
!               Pair (U,V) for Vor or Div => treat 1 over 2
          DO JF=1,QFPTYPE%ISKP(IJ),2
            IPVEC=IPVEC+1
            KVSETVD(IPVEC)=QFPTYPE%ISET(JLEV,IJ)
          ENDDO
        END SELECT
      ENDDO
    END SELECT
  ENDDO 
  IF (IPSCA /= NFPSCAG) CALL ABOR1('SUFPVSET_DIR : INTERNAL ERROR IPSCA')
  IF (IPVEC /= NFPVECG) CALL ABOR1('SUFPVSET_DIR : INTERNAL ERROR IPVEC')
CASE DEFAULT
  CALL ABOR1('SUFPVSET_DIR : INTERNAL ERROR NPRTRV')
END SELECT

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPVSET_DIR',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPVSET_DIR
