SUBROUTINE SUFPVSET_INV(KVSETSC,KVSETVD)

!**** *SUFPVSET_INV * - Set up V-set for Full-Pos inverse transforms.

!     Purpose
!     -------
!       To Set up V-set for Full-Pos inverse transforms

!**   Interface.  CALL SUFPVSET_INV(...)
!     ---------- 

!     Explicit arguments : 
!     --------------------
!        KVSETSC : V-set for each scalar field
!        KVSETVD : V-set for each pair of field (Vorticity,Divergence)
!                  (they MUST be on the same V-set)

!     Externals.
!     ----------
!        None.

!     Reference.
!     ----------
!        Fullpos technical & users guide.

!     Author.
!     -------
!        Ryad El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 03-02-11 From TRANSINV_FP
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN ,NFPTRDYN
USE YOMCT0   , ONLY : NPRTRV
USE YOMFPDIM , ONLY : NFPISCAG ,NFPIVECG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(OUT)   :: KVSETSC(NFPISCAG) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KVSETVD(NFPIVECG) 
INTEGER(KIND=JPIM) :: J, INC, IJ, IPSCA, IPVEC, JLEV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPVSET_INV',0,ZHOOK_HANDLE)
SELECT CASE (NPRTRV)
CASE (1)
  KVSETSC(:)=1
  KVSETVD(:)=1
CASE (2:)
  IPSCA=0
  IPVEC=0
  DO J=1,NFPOSDYN
    IJ=NFPTRDYN(J)
    SELECT CASE (QFPTYPE%ISF(IJ))
    CASE (1:)
      DO JLEV=1,QFPTYPE%ILEV(IJ)
        INC=QFPTYPE%ISPD(JLEV,IJ)
        SELECT CASE (QFPTYPE%ILED(IJ))
        CASE (:-1)
!               V (same V-set as U) :
          KVSETVD(IPVEC+1:IPVEC+INC)=QFPTYPE%ISET(JLEV,IJ)
          IPVEC=IPVEC+INC
        CASE (0,2,3)
!               "Scalar"
          KVSETSC(IPSCA+1:IPSCA+INC)=QFPTYPE%ISET(JLEV,IJ)
          IPSCA=IPSCA+INC
        END SELECT
      ENDDO
    END SELECT
  ENDDO
  IF (IPSCA /= NFPISCAG) CALL ABOR1('SUFPVSET_INV : INTERNAL ERROR IPSCA')
  IF (IPVEC /= NFPIVECG) CALL ABOR1('SUFPVSET_INV : INTERNAL ERROR IPVEC')
END SELECT

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPVSET_INV',1,ZHOOK_HANDLE)

END SUBROUTINE SUFPVSET_INV
