SUBROUTINE SUFPWIDE

!**** *SUFPWIDE*  Initialize control of scan2 
!                 related to horizontal post-processing

!     Purpose.
!     -------
!           Initialize. part of YOMSC2, the common that controls the
!           multi - tasking and buffer usage in the grid - point
!           calculations under SCAN2MSM/DM.

!     Interface.
!     ---------
!        *CALL*  *SUFPWIDE*

!        Explicit arguments : None
!        ------------------
!        Implicit arguments :
!        ------------------
!        see above "USE MODULE"

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Several multitasking routines (LOCK... and EV... routines).
!        SUSIZBUF,SLCSET

!     Author.
!     ------
!       Ryad EL KHATIB      * METEO-FRANCE *

!     Modifications.
!     -------------
!        Original  : 99-08-06 from SUSC2
!      R. El Khatib: 00-03-14 Susizbuf not called if dm.
!      G.Mozdzynski: 01-12-18 support radiation grid
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      R. El Khatib    20-May-2005 NFPWIDE moved to YOMWFPDS
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NFPCORE  ,NAFPB1   ,NFPSTA   ,NFPONL   ,NFPOFF   ,&
 & NFPPROCS,NFPRPT   ,NFPSPT   ,NFRSTLOFF,NFPEXT   ,&
 & NFPSENDPOS,NFPRECVPOS,NFPSENDPTR,NFPRECVPTR,NFPCOMM        ,&
 & NPTRFLOFF ,MYFRSTACTLAT,MYLSTACTLAT,NSTA,NONL    ,&
 & NPTRFRSTLAT,NFRSTLAT,NLSTLAT,MY_REGION_NS,MY_REGION_EW  
USE YOMDIM   , ONLY : NDGLG    ,NPROMA   ,NGPBLKS  ,&
 & NDGSAG   ,NDGSAL   ,NDGENG   ,NDGENL   ,NDLON    ,NDGUXG   ,&
 & NDGUXL   ,NDGSAFPH ,NDGENFPH ,NDSUR1   ,&
 & NDLSUR   ,NDGSUR  
USE YOMCT0   , ONLY : NPROC    ,NPRINTLEV,&
 & LOUTPUT  ,LALLOPR  
USE YOMWFPDS , ONLY : NFPWIDE
USE YOMGEM   , ONLY : NGPTOTG  ,NGPTOT   ,NGPTOT_CAP ,NLOENG
USE YOMLEG   , ONLY : RMU      ,RSQM2

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IFPRPTSUR, IFPSPTSUR, IU, IFPMAPLEN

LOGICAL :: LLP

INTEGER(KIND=JPIM), ALLOCATABLE :: IFPSENDPOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IFPRECVPOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IFPMAP(:,:)
INTEGER(KIND=JPIM) :: IFPSENDPTR(NPROC+1), IFPRECVPTR(NPROC+1), IFPCOMM(NPROC)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "slcset.intfb.h"

!    -------------------------------------------------------------------

!*     2. INITIALIZE BUFFER WIDTH & NUMBER OF BUFFERS
!         -------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPWIDE',0,ZHOOK_HANDLE)

!*     3. INITIALIZE CONTROL ARRAYS FOR HORIZONTAL INTERPOLATIONS
!         -------------------------------------------------------

!*        3.0   PREPARATIONS  

LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT

!*        3.1   ALLOCATIONS AND COMPUTATIONS

!     -- Allocate arrays based on NFPWIDE and initialised in SLCSET,
!        call SLCSET.  Arrays NFPSENDPOS, NFPRECVPOS, NFPSENDPTR, NFPRECVPTR,
!        and NFPCOMM are useless and not initialised if NPROC=1.
ALLOCATE(NFPSTA(NDGSAL-NFPWIDE:NDGENL+NFPWIDE))
IF (LLP) WRITE(IU,9) 'NFPSTA   ',SIZE(NFPSTA),SHAPE(NFPSTA)
ALLOCATE(NFPONL(NDGSAL-NFPWIDE:NDGENL+NFPWIDE))
IF (LLP) WRITE(IU,9) 'NFPONL   ',SIZE(NFPONL),SHAPE(NFPONL)
ALLOCATE(NFPOFF(NDGSAL-NFPWIDE:NDGENL+NFPWIDE))
IF (LLP) WRITE(IU,9) 'NFPOFF   ',SIZE(NFPOFF),SHAPE(NFPOFF)
ALLOCATE(NFPEXT(1-NDLON:NDLON+NDLON,1-NFPWIDE:NDGENL+NFPWIDE))
ALLOCATE(NFPCORE(NGPTOT))
IF(LLP)WRITE(IU,9) 'NFPCORE  ',SIZE(NFPCORE  ),SHAPE(NFPCORE)
IF (NPROC > 1) THEN
  IFPRPTSUR=NGPTOTG
  IFPSPTSUR=2*NGPTOTG
ELSE
  IFPRPTSUR=0
  IFPSPTSUR=0
ENDIF
ALLOCATE(IFPSENDPOS(IFPSPTSUR))
ALLOCATE(IFPRECVPOS(IFPRPTSUR))
ALLOCATE(IFPMAP(4,NDGLG))
CALL SLCSET('FP',NFPWIDE,IFPRPTSUR,IFPSPTSUR,MY_REGION_NS,MY_REGION_EW,&
 & NDGLG,NDLON,NDGSAG,NDGENG,NDGUXL,NDGUXG,NDGSAL,NDGENL,&
 & NDSUR1,NDLSUR,NDGSUR,NGPTOT,NGPTOT_CAP,&
 & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
 & NSTA,NONL,NLOENG,NPTRFRSTLAT,NFRSTLAT,NLSTLAT,&
 & RMU,RSQM2,&
 & NFPSTA,NFPONL,NFPOFF,NFPEXT,&
 & NFPCORE,NAFPB1,NFPPROCS,NFPRPT,NFPSPT,&
 & IFPSENDPOS,IFPRECVPOS,IFPSENDPTR,IFPRECVPTR,IFPCOMM,IFPMAP,IFPMAPLEN)  
IF (NFPWIDE == 0 .AND. NAFPB1 /= NPROMA*NGPBLKS) THEN
  NAFPB1=NPROMA*NGPBLKS
  WRITE(NULOUT,*) '*** NAFPB1 RESET TO NPROMA*NGPBLKS=',NAFPB1
ENDIF
IF (NPROC > 1 .AND. NFPWIDE > 0) THEN
  ALLOCATE(NFPSENDPOS(NFPSPT))
  IF (LLP) WRITE(IU,9) 'NFPSENDPOS',SIZE(NFPSENDPOS),SHAPE(NFPSENDPOS)
  ALLOCATE(NFPRECVPOS(NFPRPT))
  IF (LLP) WRITE(IU,9) 'NFPRECVPOS',SIZE(NFPRECVPOS),SHAPE(NFPRECVPOS)
  ALLOCATE(NFPSENDPTR(NFPPROCS+1))
  IF (LLP) WRITE(IU,9) 'NFPSENDPTR',SIZE(NFPSENDPTR),SHAPE(NFPSENDPTR)
  ALLOCATE(NFPRECVPTR(NFPPROCS+1))
  IF (LLP) WRITE(IU,9) 'NFPRECVPTR',SIZE(NFPRECVPTR),SHAPE(NFPRECVPTR)
  ALLOCATE(NFPCOMM(NFPPROCS))
  IF (LLP) WRITE(IU,9) 'NFPCOMM   ',SIZE(NFPCOMM   ),SHAPE(NFPCOMM   )
  NFPSENDPOS(1:NFPSPT)=IFPSENDPOS(1:NFPSPT)
  NFPRECVPOS(1:NFPRPT)=IFPRECVPOS(1:NFPRPT)
  NFPSENDPTR(1:NFPPROCS+1)=IFPSENDPTR(1:NFPPROCS+1)
  NFPRECVPTR(1:NFPPROCS+1)=IFPRECVPTR(1:NFPPROCS+1)
  NFPCOMM(1:NFPPROCS)=IFPCOMM(1:NFPPROCS)
ENDIF
DEALLOCATE(IFPSENDPOS)
DEALLOCATE(IFPRECVPOS)
DEALLOCATE(IFPMAP)

NDGSAFPH = MAX(NDGSAG,NDGSAL+NFRSTLOFF-NFPWIDE)-NFRSTLOFF
NDGENFPH = MIN(NDGENG,NDGENL+NFRSTLOFF+NFPWIDE)-NFRSTLOFF

IF (LLP) WRITE(IU,9) 'NFPEXT   ',SIZE(NFPEXT),SHAPE(NFPEXT)

!*        3.1   PRINTS ON OUTPUT

IF (LOUTPUT) THEN
  IF (NPRINTLEV >= 1) THEN
    WRITE(NULOUT,'('' SUFPWIDE PRINTS OUT'')')
    WRITE(NULOUT,'('' NFPWIDE ='',I4)') &
     & NFPWIDE  
    WRITE(NULOUT,'(1X,A,I4,A,I4)') &
     & ' NDGSAFPH = ',NDGSAFPH,' NDGENFPH = ',NDGENFPH
  ENDIF
ENDIF

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPWIDE',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPWIDE
