SUBROUTINE SUFPXFU

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR
USE YOMCT0   , ONLY : NSTOP
USE YOMAFN   , ONLY : GFP_XTCC ,GFP_X10U ,GFP_X10V ,GFP_X2T  ,GFP_X2SH , &
 & GFP_X2RH ,GFP_XCCC ,GFP_XHCC ,GFP_XMCC ,GFP_XUGST,GFP_XGUST, &
 & GFP_XLCC ,GFP_XX2T ,GFP_XN2T ,GFP_XCVU ,GFP_XVGST,GFP_XCLPH, &
 & GFP_XCVV ,GFP_XCVQ ,GFP_XCVS ,GFP_XTUU ,GFP_XTUV ,GFP_XMOCO, &
 & GFP_XTUQ ,GFP_XTUS ,GFP_XGDU ,GFP_XGDV ,GFP_XLSP ,GFP_XCAPE, &
 & GFP_XCP  ,GFP_XLSS ,GFP_XCSF ,GFP_XSSR ,GFP_XSTR ,GFP_X10FF, &
 & GFP_XTSR ,GFP_XTTR ,GFP_XX2HU,GFP_XN2HU,TFP_ASMC ,TFP_TX   , &
 & GFP_XLSG ,GFP_XCSG , &
 & GFP_XLSH ,GFP_XCSH ,TFP_TN   ,TFP_CAPE ,TFP_HUN  ,TFP_HUX 
USE YOMFPC   , ONLY : CFPXFU   ,NFPXFU   ,CFP2DF   ,NFP2DF   ,NFPCAPE ,&
 & LMOCONVAR
USE YOMXFU   , ONLY : LXFU     ,LREAXFU  ,LXTRD    ,LXTRC    ,LXTTCLS  ,&
 & LXTRT    ,LXPLC    ,LXPLCG  ,LXPLCH   ,LXPLS    ,LXPLSG   ,LXPLSH   ,&
 & LXR      ,LXQCLS   ,LXICV    ,&
 & LXCLP    ,LXTGST   ,LXNEBTT  ,LXNEBPA  ,LXCLS    ,LXHHCLS   


!**** *SUFPXFU*  - INITIALIZE INSTANTANEOUS FLUXES SWITCHES FOR POST-PROCESSING

!     PURPOSE.
!     --------
!        Modify default values to allow post-processing

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPXFU*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS
!        --------------------
!        YOMFPC
!        YOMXFU

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!    R. El Khatib : 00-03-22 Support for convection indexes + PBL calculation out of the model
!    R. El Khatib : 00-03-22 Remove redundant fields
!    R. El Khatib : 01-04-23 LXQCLS, LXTTCLS,LXHHCLS
!    R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!    Y. Seity     : 07-01-23 Add graupel and hail 
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER :: CLMARK*39

INTEGER(KIND=JPIM) :: IU10, IUBL, IUCV, IUGW, IUTU, IV10, IVBL, IVCV, IVGW, IVTU, JFLD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*       1. MODIFY DEFAULT VALUES
!           ---------------------

IF (LHOOK) CALL DR_HOOK('SUFPXFU',0,ZHOOK_HANDLE)
IF (NFPXFU > 0) THEN

  LXFU=.TRUE.

  IU10=0
  IV10=0
  IUCV=0
  IVCV=0
  IUTU=0
  IVTU=0
  IUGW=0
  IVGW=0
  IUBL=0
  IVBL=0

  DO JFLD=1,NFPXFU
    IF     (CFPXFU(JFLD) == GFP_XTCC%CLNAME) THEN
      LXNEBTT=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_X10U%CLNAME) THEN
      LXCLS=.TRUE.
      IU10=1
    ELSEIF (CFPXFU(JFLD) == GFP_X10V%CLNAME) THEN
      LXCLS=.TRUE.
      IV10=1
    ELSEIF (CFPXFU(JFLD) == GFP_X2T%CLNAME ) THEN
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_X2SH%CLNAME) THEN
      LXQCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_X2RH%CLNAME) THEN
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCCC%CLNAME) THEN
      LXNEBPA=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XHCC%CLNAME) THEN
      LXNEBPA=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XMCC%CLNAME) THEN
      LXNEBPA=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XLCC%CLNAME) THEN
      LXNEBPA=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XX2T%CLNAME) THEN
      LXTTCLS=.TRUE.
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XN2T%CLNAME) THEN
      LXTTCLS=.TRUE.
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XX2HU%CLNAME) THEN
      LXHHCLS=.TRUE.
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XN2HU%CLNAME) THEN
      LXHHCLS=.TRUE.
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCVU%CLNAME) THEN
      LXTRC=.TRUE.
      IUCV=1
    ELSEIF (CFPXFU(JFLD) == GFP_XCVV%CLNAME) THEN
      LXTRC=.TRUE.
      IVCV=1
    ELSEIF (CFPXFU(JFLD) == GFP_XCVQ%CLNAME) THEN
      LXTRC=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCVS%CLNAME) THEN
      LXTRC=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XTUU%CLNAME) THEN
      LXTRT=.TRUE.
      IUTU=1
    ELSEIF (CFPXFU(JFLD) == GFP_XTUV%CLNAME) THEN
      LXTRT=.TRUE.
      IVTU=1
    ELSEIF (CFPXFU(JFLD) == GFP_XTUQ%CLNAME) THEN
      LXTRT=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XTUS%CLNAME) THEN
      LXTRT=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XGDU%CLNAME) THEN
      LXTRD=.TRUE.
      IUGW=1
    ELSEIF (CFPXFU(JFLD) == GFP_XGDV%CLNAME) THEN
      LXTRD=.TRUE.
      IVGW=1
    ELSEIF (CFPXFU(JFLD) == GFP_XLSP%CLNAME) THEN
      LXPLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCP%CLNAME ) THEN
      LXPLC=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XLSS%CLNAME) THEN
      LXPLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCSF%CLNAME) THEN
      LXPLC=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XLSG%CLNAME) THEN
      LXPLSG=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCSG%CLNAME ) THEN
      LXPLCG=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XLSH%CLNAME) THEN
      LXPLSH=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCSH%CLNAME ) THEN
      LXPLCH=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XSSR%CLNAME) THEN
      LXR=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XSTR%CLNAME) THEN
      LXR=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XTSR%CLNAME) THEN
      LXR=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XTTR%CLNAME) THEN
      LXR=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_X10FF%CLNAME) THEN
      LXCLS=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCAPE%CLNAME) THEN
      LXICV=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XMOCO%CLNAME) THEN
      LXICV=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XCLPH%CLNAME) THEN
      LXCLP=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XGUST%CLNAME) THEN
      LXTGST=.TRUE.
    ELSEIF (CFPXFU(JFLD) == GFP_XUGST%CLNAME) THEN
      LXTGST=.TRUE.
      IUBL=1
    ELSEIF (CFPXFU(JFLD) == GFP_XVGST%CLNAME) THEN
      LXTGST=.TRUE.
      IVBL=1
    ELSE
      CLMARK=CFPXFU(JFLD)//' NOT FOUND '
      WRITE(NULERR,*) CLMARK
    ENDIF
  ENDDO
  IF ((IU10 /= IV10).OR.&
     & (IUTU /= IVTU).OR.&
     & (IUBL /= IVBL).OR.&
     & (IUGW /= IVGW).OR.&
     & (IUCV /= IVCV)) THEN  
    CALL ABOR1('U AND V COMPONENTS ARE BOTH NECESSARY !')
  ENDIF
ENDIF

! Tcls,RHcls must be read if Cape is computed from PBL fields : 
DO JFLD=1,NFP2DF
  IF (CFP2DF(JFLD)==TFP_CAPE%CLNAME .AND. NFPCAPE==4) THEN
    LXFU=.TRUE.
    LXCLS=.TRUE.
    EXIT
  ENDIF
ENDDO
! Tn, Tx, Tcls must be read (to compute increments) if Tn,Tx are recomputed : 
DO JFLD=1,NFP2DF
  IF (CFP2DF(JFLD)==TFP_TX%CLNAME .OR. &
     & CFP2DF(JFLD)==TFP_TN%CLNAME) THEN  
    LXFU=.TRUE.
    LXTTCLS=.TRUE.
    LXCLS=.TRUE.
    EXIT
  ENDIF
ENDDO
! HUn,HUx, HUcls must be read (to compute increments) if HUn,HUx are recomputed :
DO JFLD=1,NFP2DF
  IF (CFP2DF(JFLD)==TFP_HUX%CLNAME .OR. &
     & CFP2DF(JFLD)==TFP_HUN%CLNAME) THEN  
    LXFU=.TRUE.
    LXHHCLS=.TRUE.
    LXCLS=.TRUE.
    EXIT
  ENDIF
ENDDO
! analysed surface Moisture convergence needs CLS wind & moisture, not for varpack :
DO JFLD=1,NFP2DF
  IF ((CFP2DF(JFLD)==TFP_ASMC%CLNAME).AND.(.NOT.LMOCONVAR)) THEN
    LXFU=.TRUE.
    LXCLS=.TRUE.
    EXIT
  ENDIF
ENDDO

IF (LXFU.AND.NSTOP == 0) THEN
  LREAXFU=.TRUE.
ENDIF

IF (LHOOK) CALL DR_HOOK('SUFPXFU',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPXFU
