SUBROUTINE SUMPFPOS(LDFPBUF,PLA,PLO,PGM,PNORX,PNORY)

!**** *SUMPFPOS*  - SET UP MESSAGE PASSINF FOR FULL-POS

!     PURPOSE.
!     --------
!        To initialize control array for the distribution and 
!        to distribute some global fields (DM distribution linked
!        to arrival geometry).

!        Computes the following variables:
!        RFPLA,RFPLO,RFPGM,RFPNORX,RFPNORY
!        NFPRGPL,NFPRGPLX
!        NFPRGPNUM,NFPRGPIND
!        NUMPROCFP

!**   INTERFACE.
!     ----------
!       *CALL* *SUMPFPOS*

!        EXPLICIT ARGUMENTS
!        ------------------
!         INPUT : 
!         LDFPBUF      : .FALSE. if model and fullpos buffer have the same 
!                        characteristics
!         PLA, PLO, PGM: global latitudes, longitudes,map factor
!         PNORX, PNORY : global compass

!        IMPLICIT ARGUMENTS
!        ------------------
!        See modules above

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!       SUPROCFP

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        Original : 98-10-05 from sufpg
!        R. El Khatib 99-02-05 Get rid of LMESSP ; KNUMD
!        D.Dent       00-01-10 test on NPRTRV /= 1
!        R. El Khatib 00-10-20 map factor distributed unconditionnaly for vertical calculations
!        R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad: 27-Feb-2007 Move old code in SUMPFPOS_DEP,
!         adapt to DM-arrival geometry.
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NPROC    ,LALLOPR
USE YOMFPG   , ONLY : RFPLA    ,RFPLO    ,RFPGM    ,RFPNORX  ,&
 & RFPNORY  ,NFPRGPL  ,NFPRGPLX ,NFPRGPG  ,LFPDISTRIB
USE YOMFPGIND, ONLY : NFPRGPNUM,NFPRGPIND,NFPRGPIND_DEP, NFPRGPNUM_DEP
USE YOMMP    , ONLY : MYPROC   ,NUMPROCFP ,NUMPROCFP_DEP
USE YOMFPC   , ONLY : LTRACEFP
USE IOSTREAM_MIX , ONLY : INI_IOSTREAM

!-----------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDFPBUF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLA(NFPRGPG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLO(NFPRGPG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NFPRGPG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORX(NFPRGPG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORY(NFPRGPG) 

!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IPL, JJPG, IPLP(NPROC), II, IROC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "suprocfp.intfb.h"

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMPFPOS',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------

!*     0. WARNING
!         -------

IF(LTRACEFP) WRITE(NULOUT,'('' SUMPFPOS PRINTS OUT : '')')

!*     1. COMPUTE CONTROL ARRAY FOR DISTRIBUTION
!         --------------------------------------

!     Processor number each output point belongs :
ALLOCATE(NUMPROCFP(NFPRGPG))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990) 'NUMPROCFP',SIZE(NUMPROCFP),SHAPE(NUMPROCFP)
ENDIF

!     Number of output point each processor treats : 
ALLOCATE(NFPRGPNUM(NPROC))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990) 'NFPRGPNUM',SIZE(NFPRGPNUM),SHAPE(NFPRGPNUM)
ENDIF

IF (NPROC == 1) THEN

  NUMPROCFP(:)=1

  NFPRGPNUM(:)=NFPRGPG
  NFPRGPL=NFPRGPG
  NFPRGPLX=NFPRGPG

ELSE

  IF (LFPDISTRIB) THEN
    CALL SUPROCFP(NUMPROCFP)
  ELSE
    NUMPROCFP(:)=NUMPROCFP_DEP(:)
  ENDIF

  NFPRGPNUM=0
  DO II=1,NFPRGPG
    NFPRGPNUM(NUMPROCFP(II))=NFPRGPNUM(NUMPROCFP(II))+1
  ENDDO


  NFPRGPL=NFPRGPNUM(MYPROC)
  NFPRGPLX=MAXVAL(NFPRGPNUM)

ENDIF

!     Global index of each output point in each processor
ALLOCATE(NFPRGPIND(NFPRGPLX,NPROC))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990)'NFPRGPIND',SIZE(NFPRGPIND),SHAPE(NFPRGPIND)
ENDIF
IF (NFPRGPLX > 0) THEN
  NFPRGPIND(:,:)=-9999999
  IF (NPROC == 1) THEN
    DO JJPG=1,NFPRGPL
      NFPRGPIND(JJPG,:)=JJPG
    ENDDO
  ELSE
    IF (LFPDISTRIB) THEN

!!    DO JROC=1,NPROC
!!      IPL=0
!!      DO JJPG=1,NFPRGPG
!!        IF (NUMPROCFP(JJPG) == JROC) THEN
!!          IPL=IPL+1
!!          NFPRGPIND(IPL,JROC)=JJPG
!!        ENDIF
!!      ENDDO
!!    ENDDO

      IPLP(:)=0
      DO JJPG=1,NFPRGPG 
        IROC=NUMPROCFP(JJPG)
        IPLP(IROC)=IPLP(IROC)+1
        NFPRGPIND(IPLP(IROC),IROC)=JJPG
      ENDDO

    ELSE
      NFPRGPIND(:,:)=NFPRGPIND_DEP(:,:)
    ENDIF
  ENDIF
ENDIF

!*     2. DISTRIBUTE FIELDS
!         -----------------

! ky: if lfpdistrib=f, this calculation will automatically ensure
!     that rfpla=rfpla_dep, rfplo=rfplo_dep.

IF (LDFPBUF) THEN
  ALLOCATE(RFPLA(NFPRGPL))
  ALLOCATE(RFPLO(NFPRGPL))
  ALLOCATE(RFPNORX(NFPRGPL))
  ALLOCATE(RFPNORY(NFPRGPL))
ENDIF
ALLOCATE(RFPGM(NFPRGPL))
IF (LALLOPR) THEN
  IF (LDFPBUF) THEN
    WRITE(NULOUT,9990) 'RFPLA   ',SIZE(RFPLA),SHAPE(RFPLA)
    WRITE(NULOUT,9990) 'RFPLO   ',SIZE(RFPLO),SHAPE(RFPLO)
    WRITE(NULOUT,9990) 'RFPNORX ',SIZE(RFPNORX),SHAPE(RFPNORX)
    WRITE(NULOUT,9990) 'RFPNORY ',SIZE(RFPNORY),SHAPE(RFPNORY)
  ENDIF
  WRITE(NULOUT,9990) 'RFPGM   ',SIZE(RFPGM),SHAPE(RFPGM)
ENDIF
IF (NFPRGPL > 0) THEN
  IF (NFPRGPL == NFPRGPG) THEN
    ! All points are treated by the same processor           
    IF (LDFPBUF) THEN
      RFPLA  (:)=PLA  (:)
      RFPLO  (:)=PLO  (:)
      RFPNORX(:)=PNORX(:)
      RFPNORY(:)=PNORY(:)
    ENDIF
    RFPGM  (:)=PGM  (:)
  ELSE
    ! Actual distribution
    IPL=0
    DO JJPG=1,NFPRGPG
      IF (NUMPROCFP(JJPG) == MYPROC) THEN
        IPL=IPL+1
        IF (LDFPBUF) THEN
          RFPLA(IPL)=PLA(JJPG)
          RFPLO(IPL)=PLO(JJPG)
          RFPNORX(IPL)=PNORX(JJPG)
          RFPNORY(IPL)=PNORY(JJPG)
        ENDIF
        RFPGM(IPL)=PGM(JJPG)
      ENDIF
    ENDDO
  ENDIF
ENDIF

!*     3. FINAL PRINTS OUT
!         ----------------

IF(LTRACEFP) WRITE(UNIT=NULOUT,FMT='('' NFPRGPL = '',I6,'' NFPRGPLX = '',I6)')&
 & NFPRGPL, NFPRGPLX  
CALL INI_IOSTREAM(KNUMPROCFP=NUMPROCFP,KFPRGPL=NFPRGPL,KFPRGPG=NFPRGPG,KFPRGPLX=NFPRGPLX)
! ---------------------------------------------------------------------
9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUMPFPOS',1,ZHOOK_HANDLE)
END SUBROUTINE SUMPFPOS
