SUBROUTINE SUMPFPOS_DEP(LDFPBUF,PLA,PLO,KNUMD)

!**** *SUMPFPOS_DEP*  - SET UP MESSAGE PASSING FOR FULL-POS

!     PURPOSE.
!     --------
!        To initialize control array for the distribution and 
!        to distribute some global fields (DM-distribution linked to
!        departure geometry).

!        Computes the following variables:
!        RFPLA_DEP,RFPLO_DEP
!        NFPRGPL_DEP,NFPRGPLX_DEP,NFPNUMD_DEP
!        NUMPROCFP_DEP

!**   INTERFACE.
!     ----------
!       *CALL* *SUMPFPOS_DEP*

!        EXPLICIT ARGUMENTS
!        ------------------
!         INPUT : 
!         LDFPBUF      : .FALSE. if model and fullpos buffer have the same 
!                        characteristics
!         PLA, PLO     : global latitudes, longitudes
!         KNUMD        : global domain index

!        IMPLICIT ARGUMENTS
!        ------------------
!        See modules above

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!       SUPROCFP_DEP

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE* (SUMPFPOS)
!        K. Yessad (rename into SUMPFPOS_DEP)

!     MODIFICATIONS.
!     --------------
!        Original : 98-10-05 from sufpg
!        R. El Khatib 99-02-05 Get rid of LMESSP ; KNUMD
!        D.Dent       00-01-10 test on NPRTRV /= 1
!        R. El Khatib 00-10-20 map factor distributed unconditionnaly for vertical calculations
!        R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad : 27-Feb-2007 Rename into SUMPFPOS_DEP, adapt, clean.
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NPROC    ,LALLOPR
USE YOMFPG   , ONLY : RFPLA_DEP,RFPLO_DEP,&
 & NFPRGPL_DEP,NFPRGPLX_DEP,NFPRGPG  ,NFPNUMD_DEP  
USE YOMFPGIND, ONLY : NFPRGPNUM_DEP,NFPRGPIND_DEP
USE YOMMP    , ONLY : MYPROC   ,NUMPROCFP_DEP
USE YOMFPC   , ONLY : LTRACEFP

!-----------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDFPBUF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLA(NFPRGPG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLO(NFPRGPG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMD(NFPRGPG) 

!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IPL, JJPG, JROC, II, IPLP(NPROC), IROC

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "suprocfp_dep.intfb.h"

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMPFPOS_DEP',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------

!*     0. WARNING
!         -------

IF(LTRACEFP) WRITE(NULOUT,'('' SUMPFPOS_DEP PRINTS OUT : '')')

!*     1. COMPUTE CONTROL ARRAY FOR DISTRIBUTION
!         --------------------------------------

!     Processor number each output point belongs :
ALLOCATE(NUMPROCFP_DEP(NFPRGPG))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990) 'NUMPROCFP_DEP',SIZE(NUMPROCFP_DEP),SHAPE(NUMPROCFP_DEP)
ENDIF

!     Number of output point each processor treats :
ALLOCATE(NFPRGPNUM_DEP(NPROC))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990) 'NFPRGPNUM_DEP',SIZE(NFPRGPNUM_DEP),SHAPE(NFPRGPNUM_DEP)
ENDIF

IF (NPROC == 1) THEN

  NUMPROCFP_DEP(:)=1

  NFPRGPNUM_DEP(:)=NFPRGPG
  NFPRGPL_DEP=NFPRGPG
  NFPRGPLX_DEP=NFPRGPG

ELSE

  CALL SUPROCFP_DEP(LDFPBUF,PLA,PLO,NUMPROCFP_DEP)

!!DO JROC=1,NPROC
!!  NFPRGPNUM_DEP(JROC)=COUNT(NUMPROCFP_DEP  ==  JROC)
!!ENDDO

  NFPRGPNUM_DEP=0
  DO II=1,NFPRGPG
    NFPRGPNUM_DEP(NUMPROCFP_DEP(II))=NFPRGPNUM_DEP(NUMPROCFP_DEP(II))+1
  ENDDO

  NFPRGPL_DEP=NFPRGPNUM_DEP(MYPROC)
  NFPRGPLX_DEP=MAXVAL(NFPRGPNUM_DEP)

ENDIF

!     Global index of each output point in each processor
ALLOCATE(NFPRGPIND_DEP(NFPRGPLX_DEP,NPROC))
IF (LALLOPR) THEN
  WRITE(NULOUT,9990)'NFPRGPIND_DEP',SIZE(NFPRGPIND_DEP),SHAPE(NFPRGPIND_DEP)
ENDIF
IF (NFPRGPLX_DEP > 0) THEN
  NFPRGPIND_DEP(:,:)=-9999999
  IF (NPROC == 1) THEN
    DO JJPG=1,NFPRGPL_DEP
      NFPRGPIND_DEP(JJPG,:)=JJPG
    ENDDO
  ELSE
!!  DO JROC=1,NPROC
!!    IPL=0
!!    DO JJPG=1,NFPRGPG
!!      IF (NUMPROCFP_DEP(JJPG) == JROC) THEN
!!        IPL=IPL+1
!!        NFPRGPIND_DEP(IPL,JROC)=JJPG
!!      ENDIF
!!    ENDDO
!!  ENDDO

    IPLP(:)=0
    DO JJPG=1,NFPRGPG
      IROC=NUMPROCFP_DEP(JJPG)
      IPLP(IROC)=IPLP(IROC)+1
      NFPRGPIND_DEP(IPLP(IROC),IROC)=JJPG
    ENDDO

  ENDIF
ENDIF

!*     2. DISTRIBUTE FIELDS
!         -----------------

IF (LDFPBUF) THEN
  ALLOCATE(RFPLA_DEP(NFPRGPL_DEP))
  ALLOCATE(RFPLO_DEP(NFPRGPL_DEP))
ENDIF
ALLOCATE(NFPNUMD_DEP(NFPRGPL_DEP))
IF (LALLOPR) THEN
  IF (LDFPBUF) THEN
    WRITE(NULOUT,9990) 'RFPLA_DEP   ',SIZE(RFPLA_DEP),SHAPE(RFPLA_DEP)
    WRITE(NULOUT,9990) 'RFPLO_DEP   ',SIZE(RFPLO_DEP),SHAPE(RFPLO_DEP)
  ENDIF
  WRITE(NULOUT,9990) 'NFPNUMD_DEP ',SIZE(NFPNUMD_DEP),SHAPE(NFPNUMD_DEP)
ENDIF

IF (NFPRGPL_DEP > 0) THEN
  IF (NFPRGPL_DEP == NFPRGPG) THEN
    ! All points are treated by the same processor           
    IF (LDFPBUF) THEN
      RFPLA_DEP(:)=PLA(:)
      RFPLO_DEP(:)=PLO(:)
    ENDIF
    NFPNUMD_DEP(:)=KNUMD(:)
  ELSE
    ! Actual distribution
    IPL=0
    DO JJPG=1,NFPRGPG
      IF (NUMPROCFP_DEP(JJPG) == MYPROC) THEN
        IPL=IPL+1
        IF (LDFPBUF) THEN
          RFPLA_DEP(IPL)=PLA(JJPG)
          RFPLO_DEP(IPL)=PLO(JJPG)
        ENDIF
        NFPNUMD_DEP(IPL)=KNUMD(JJPG)
      ENDIF
    ENDDO
  ENDIF
ENDIF

!*     3. FINAL PRINTS OUT
!         ----------------

IF(LTRACEFP) WRITE(UNIT=NULOUT, &
 & FMT='('' NFPRGPL_DEP = '',I6,'' NFPRGPLX_DEP = '',I6)')&
 & NFPRGPL_DEP, NFPRGPLX_DEP  

! ---------------------------------------------------------------------
9990 FORMAT(1X,'ARRAY ',A13,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUMPFPOS_DEP',1,ZHOOK_HANDLE)
END SUBROUTINE SUMPFPOS_DEP
