SUBROUTINE SUVPOS(CDCONF,PPLEV,KPPLEV,KI2FLD,KN2FLD,KI3FLD,KN3FLD,KINDLEV,&
 & KLEVS,PLEV,KFPLEV,KSDO3,KSDO2,KSDP3,KSDP2,LDSURF)  

!**** *SUVPOS*  - INITIALIZE COMMON YOMVPOS

!     PURPOSE.
!     --------
!        Initialize common YOMVPOS which contains control variables
!        for vertical/horizontal post-processing
!        "B-level" parallelization is manage as follows :
!        the attributed V-set to a given level of a given field is "the one that is the less used" ;
!        Exceptions for vector fields : U/V corresponding to the same level are attributed the same
!        V-set

!**   INTERFACE.
!     ----------
!       *CALL* *SUVPOS*

!        EXPLICIT ARGUMENTS
!        --------------------
!        CDCONF : configuration of work
!        PPLEV  : values of post-processing levels
!        KPPLEV : number of post-processing levels
!        KI2FLD : 2D-field pointers
!        KN2FLD : number of 2D-fields
!        KI3FLD : 3D-field pointers
!        KN3FLD : number of 3D-fields
!        KINDLEV: post-processing level pointers for each field
!        KLEVS  : number of post-processing levels for each field
!        KSDO2  : number of subdomains for each 2D field
!        KSDO3  : number of subdomains for each level of each 3D field
!        KSDP2  : subdomain indexes for each 2D field
!        KSDP3  : subdomain indexes for each level of each 3D field
!        LDSURF : .TRUE. to compute surface fields in this scan

!          OUTPUT
!     ==== 3D VARIABLES ===

!     PLEV    : values of post-processing levels
!     KFPLEV  : number of post-processing levels

!        IMPLICIT ARGUMENTS
!        --------------------
!          YOMAFN (input)

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!    R. El Khatib : 97-06-02  3 NH fields
!    D.Dent       : 99-05-06  support for log surface pressure
!    R. El Khatib : 00-08-18 derived type
!    D. Dent      : 00-04-12  support for computation of Montgomery geopotential
!    R. El Khatib : 01-05-07 control HUn,HUx
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvments
!    R. El Khatib : 03-11-13 Fix on QFPTYPE%ISPD
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSDYN
USE YOMAFN   , ONLY : TFP_DYNDS,TFP_U    ,TFP_V    ,TFP_PSI  ,TFP_KHI  ,&
 & TFP_ABS  ,TFP_VOR  ,TFP_DIV  ,TFP_TN   ,TFP_TX   ,TFP_HUN  ,&
 & TFP_HUX  ,TFP_P    ,TFP_TH   ,TFP_PV   ,&
 & TFP_SMC  ,TFP_ASMC   
USE YOMFPC   , ONLY : NFPDOM   ,CFPFMT   ,NFPXLEV  ,LFITP    ,LFITT    ,&
 & LFITV  
USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN ,NFPTRDYN ,MYSFSET
USE YOMCT0   , ONLY : NPRTRV
USE YOMDIM   , ONLY : NFLEVG
USE YOMMP    , ONLY : MYSETV
USE YOMFPF   , ONLY : LFPBED

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KN2FLD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KN3FLD 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPLEV(KPPLEV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KI2FLD(KN2FLD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KI3FLD(KN3FLD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KINDLEV(KPPLEV,KN3FLD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS(KN3FLD) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLEV(NFPXLEV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSDO3(KPPLEV,KN3FLD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSDO2(KN2FLD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSDP3(NFPDOM,KPPLEV,KN3FLD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSDP2(NFPDOM,KN2FLD) 
LOGICAL           ,INTENT(IN)    :: LDSURF 
INTEGER(KIND=JPIM) :: JDOM, J, JLEV, ICOD, I, IB, JJ, IJ, IVSET, ISPREAD, IU, IV

INTEGER(KIND=JPIM) :: IVCOUNT(NPRTRV) ! Number of fields for each V-set in spectral transform
INTEGER(KIND=JPIM) :: IVPTR(NPRTRV)   ! Pointer of fields for each V-set in output array
INTEGER(KIND=JPIM) :: ISFPPLEV        ! Filter indicator of the current pp. level
INTEGER(KIND=JPIM) :: ILOC(1)

LOGICAL :: LLSUPL  ! a supplementary condition
LOGICAL :: LLMODEL ! .T. if post-processing on model grid
LOGICAL :: LLFIT   ! Overwriting of the atribute fit/no-fit
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    PRESET REQUEST
!              --------------

IF (LHOOK) CALL DR_HOOK('SUVPOS',0,ZHOOK_HANDLE)
LLMODEL = (CFPFMT == 'MODEL')

QFPTYPE%LL(:)=.FALSE.

!*       2.    SETUP REQUEST
!              -------------

IF (CDCONF /= 'M') THEN

!        2.1 post-processing levels

  KFPLEV=KPPLEV
  DO JLEV=1,KPPLEV
    PLEV(JLEV)=PPLEV(JLEV)
  ENDDO

!*       2.2  Request on 2D fields

  IF (LDSURF) THEN
    DO J=1,KN2FLD
      ICOD=KI2FLD(J)
!     The next line enable to remove automatically the uncomputable fields
      QFPTYPE%LL(ICOD)=(INDEX(TFP_DYNDS(ICOD)%CLNIL,CDCONF) == 0).OR.LLMODEL
      SELECT CASE (TFP_DYNDS(ICOD)%LLGP)
      CASE (.TRUE.)
        IF (ICOD == TFP_ASMC%ICOD .OR. ICOD == TFP_SMC%ICOD) THEN
!           Force spectral fit
          QFPTYPE%ISF(ICOD)=MAX(TFP_DYNDS(ICOD)%ISF,1)
        ELSE
!           No spectral fit
          QFPTYPE%ISF(ICOD)=0
        ENDIF
      CASE DEFAULT
        SELECT CASE (CDCONF)
        CASE ('Z','E')
          QFPTYPE%ISF(ICOD)=0
        CASE DEFAULT
          QFPTYPE%ISF(ICOD)=TFP_DYNDS(ICOD)%ISF
        END SELECT 
      END SELECT
      QFPTYPE%ILEV(ICOD)=1
      QFPTYPE%ILVP(1,ICOD)=1
      QFPTYPE%IDOM(1,ICOD)=KSDO2(J)
    ENDDO
    DO J=1,KN2FLD
      ICOD=KI2FLD(J)
      DO JDOM=1,KSDO2(J)
        QFPTYPE%IDMP(JDOM,1,ICOD)=KSDP2(JDOM,J)
      ENDDO
    ENDDO
  ENDIF

!*       2.3  Request on 3D fields 

  SELECT CASE (CDCONF)
  CASE ('B')
    ISFPPLEV=TFP_P%ISF
    LLFIT=LFITP
  CASE ('T')
    ISFPPLEV=TFP_TH%ISF
    LLFIT=LFITT
  CASE ('V')
    ISFPPLEV=TFP_PV%ISF
    LLFIT=LFITV
  CASE ('S')
    ISFPPLEV=1
    LLFIT=.TRUE.
  CASE DEFAULT
    ISFPPLEV=0
    LLFIT=.FALSE.
  END SELECT

  DO J=1,KN3FLD
    ICOD=KI3FLD(J)
!   The next line enable to remove automatically the uncomputable fields
    QFPTYPE%LL(ICOD)=(INDEX(TFP_DYNDS(ICOD)%CLNIL,CDCONF) == 0)
    SELECT CASE (TFP_DYNDS(ICOD)%LLGP)
    CASE (.TRUE.)
!       No spectral fit
      QFPTYPE%ISF(ICOD)=0
    CASE DEFAULT
      SELECT CASE (LLFIT)
      CASE (.TRUE.)
        QFPTYPE%ISF(ICOD)=MAX(TFP_DYNDS(ICOD)%ISF,ISFPPLEV)
      CASE DEFAULT
        QFPTYPE%ISF(ICOD)=0
      END SELECT 
    END SELECT
    QFPTYPE%ILEV(ICOD)=KLEVS(J)
  ENDDO
  DO J=1,KN3FLD
    ICOD=KI3FLD(J)
    DO JLEV=1,QFPTYPE%ILEV(ICOD)
      QFPTYPE%ILVP(JLEV,ICOD)=KINDLEV(JLEV,J)
      QFPTYPE%IDOM(JLEV,ICOD)=KSDO3(JLEV,J)
    ENDDO
  ENDDO
  DO J=1,KN3FLD
    ICOD=KI3FLD(J)
    DO JLEV=1,KLEVS(J)
      DO JDOM=1,QFPTYPE%IDOM(JLEV,ICOD)
        QFPTYPE%IDMP(JDOM,JLEV,ICOD)=KSDP3(JDOM,JLEV,J)
      ENDDO
    ENDDO
  ENDDO

ELSE

!*       2.4 Request of model primitive variables for the post-processing

  KFPLEV=NFLEVG
  DO JLEV=1,NFLEVG
    PLEV(JLEV)=REAL(JLEV,JPRB)
  ENDDO
  DO J=1, JPOSDYN
    QFPTYPE%LL(J)=(TFP_DYNDS(J)%ILMOD > 0)
    QFPTYPE%ISF(J)=0
    QFPTYPE%ILEV(J)=TFP_DYNDS(J)%ILMOD
  ENDDO
  DO J=1, JPOSDYN
    DO JLEV=1,QFPTYPE%ILEV(J)
      QFPTYPE%ILVP(JLEV,J)=JLEV
      QFPTYPE%IDOM(JLEV,J)=1
      QFPTYPE%IDMP(1,JLEV,J)=1
    ENDDO
  ENDDO

ENDIF

!*       3.    CONTROL REQUESTS
!              ----------------

!        3.0   For continuity with the previous libraries

! U,V,Psi,Khi are not filtered if "THX" filter is selected
IF (LFPBED) THEN
  QFPTYPE%ISF(TFP_U%ICOD)=MIN(1,QFPTYPE%ISF(TFP_U%ICOD))
  QFPTYPE%ISF(TFP_V%ICOD)=MIN(1,QFPTYPE%ISF(TFP_V%ICOD))
  QFPTYPE%ISF(TFP_PSI%ICOD)=MIN(1,QFPTYPE%ISF(TFP_PSI%ICOD))
  QFPTYPE%ISF(TFP_KHI%ICOD)=MIN(1,QFPTYPE%ISF(TFP_KHI%ICOD))
ENDIF

!        3.1 Tensor aspects

QFPTYPE%ILED(:)=0
! Vorticities, divergence :
QFPTYPE%ILED(TFP_ABS%ICOD)=2
QFPTYPE%ILED(TFP_VOR%ICOD)=2
QFPTYPE%ILED(TFP_DIV%ICOD)=3
! Psi and Khi are computed from U and V in spectral space :
QFPTYPE%ILED(TFP_KHI%ICOD)=1
QFPTYPE%ILED(TFP_PSI%ICOD)=-TFP_KHI%ICOD
! Surface Moisture Convergence is computed as DIV(q.Ucls,q.Vcls) ;
QFPTYPE%ILED(TFP_SMC%ICOD)=3
QFPTYPE%ILED(TFP_ASMC%ICOD)=3
! Vectors :
DO J=1,JPOSDYN
  IF (TFP_DYNDS(J)%IORDR < 0) THEN
!   V => find U :
    DO JJ=1,JPOSDYN
      IF (TFP_DYNDS(JJ)%CLPAIR == TFP_DYNDS(J)%CLPAIR .AND. JJ /= J) THEN
        QFPTYPE%ILED(JJ)=1
        QFPTYPE%ILED(J)=-JJ
        EXIT
      ENDIF
    ENDDO
  ENDIF
ENDDO

!        3.2 The case of auxilary fields

QFPTYPE%LLXR(:)=.FALSE.

IF (LLMODEL) THEN
! Vectors will be considered as individual "scalars" if fitted
  DO J=1,JPOSDYN
    IF (ABS(TFP_DYNDS(J)%IORDR) == 1) THEN
      QFPTYPE%ILED(J)=QFPTYPE%ILED(J)*(1-MIN(QFPTYPE%ISF(J),1))
    ENDIF
  ENDDO 
ELSE
! Add missing components (for spectral fit or compass)
  DO J=1,JPOSDYN
    IF (QFPTYPE%ILED(J) < 0) THEN
      IU=J
      IV=-QFPTYPE%ILED(J)
      LLSUPL=.FALSE.
      SELECT CASE (QFPTYPE%LL(IU))
      CASE (.TRUE.)
        IF (.NOT.QFPTYPE%LL(IV)) THEN
          I=IU
          IB=IV
          LLSUPL=.TRUE.
        ENDIF
      CASE DEFAULT
        IF (QFPTYPE%LL(IV)) THEN
          IB=IU
          I=IV
          LLSUPL=.TRUE.
        ENDIF
      END SELECT 
      IF (LLSUPL) THEN
        QFPTYPE%LL(IB)=.TRUE.
        QFPTYPE%LLXR(IB)=.TRUE. 
        QFPTYPE%ISF(IB)=QFPTYPE%ISF(I)
        QFPTYPE%ILEV(IB)=QFPTYPE%ILEV(I)
        DO JLEV=1,QFPTYPE%ILEV(I)
          QFPTYPE%ILVP(JLEV,IB)=QFPTYPE%ILVP(JLEV,I)
          QFPTYPE%IDOM(JLEV,IB)=QFPTYPE%IDOM(JLEV,I)
        ENDDO
        DO JLEV=1,QFPTYPE%ILEV(I)
          DO JDOM=1,QFPTYPE%IDOM(JLEV,I)
            QFPTYPE%IDMP(JDOM,JLEV,IB)=QFPTYPE%IDMP(JDOM,JLEV,I)
          ENDDO
        ENDDO
      ENDIF
    ENDIF
  ENDDO
ENDIF

!        3.3   Number of fields needed in gridpoint space :

QFPTYPE%ISKP(:)=1

! Vor and Div are computed from U and V if spectral fit is active
QFPTYPE%ISKP(TFP_ABS%ICOD)=1+MIN(QFPTYPE%ISF(TFP_ABS%ICOD),1)
QFPTYPE%ISKP(TFP_VOR%ICOD)=1+MIN(QFPTYPE%ISF(TFP_VOR%ICOD),1)
QFPTYPE%ISKP(TFP_DIV%ICOD)=1+MIN(QFPTYPE%ISF(TFP_DIV%ICOD),1)

! Surface Moisture Convergence is computed as DIV(q.Ucls,q.Vcls) ;
QFPTYPE%ISKP(TFP_SMC%ICOD)=2
QFPTYPE%ISKP(TFP_ASMC%ICOD)=2

!        3.4   Special difficulties related to spectral space :

! Khi, Psi can be computed only in spectral space :
I=TFP_KHI%ICOD  ; QFPTYPE%LL(I)=QFPTYPE%LL(I).AND.(QFPTYPE%ISF(I) >= 1)
I=TFP_PSI%ICOD  ; QFPTYPE%LL(I)=QFPTYPE%LL(I).AND.(QFPTYPE%ISF(I) >= 1)

IF (LLMODEL) THEN

! Absolute Vorticity as spectral field would need to add the Coriolis parameter
! just before writing out to file and thus extra spectral transforms.
! Better remove it for the time being :
  I=TFP_ABS%ICOD ; QFPTYPE%LL(I)=QFPTYPE%LL(I).AND.(QFPTYPE%ISF(I) == 0)

! Computation of Extrema are out of object since cls fields are at disposal :
  QFPTYPE%LL(TFP_TX%ICOD)=.FALSE.
  QFPTYPE%LL(TFP_TN%ICOD)=.FALSE.
  QFPTYPE%LL(TFP_HUX%ICOD)=.FALSE.
  QFPTYPE%LL(TFP_HUN%ICOD)=.FALSE.

! Moisture convergence as a gridpoint field would need
! an extra inverse spectral transform. Better remove it for the time being :
  I=TFP_SMC%ICOD  ; QFPTYPE%LL(I)=QFPTYPE%LL(I).AND.(QFPTYPE%ISF(I) >= 1)
  I=TFP_ASMC%ICOD ; QFPTYPE%LL(I)=QFPTYPE%LL(I).AND.(QFPTYPE%ISF(I) >= 1)

ENDIF

!        3.5   Nullify dimensions for non-requested fields and count requested ones

NFPOSDYN=0
DO J=1,JPOSDYN
  IF (.NOT.QFPTYPE%LL(J)) THEN
    QFPTYPE%ISF(J)=0
    QFPTYPE%ISKP(J)=0
    QFPTYPE%ILED(J)=0
    QFPTYPE%ILEV(J)=0
  ELSE
    NFPOSDYN=NFPOSDYN+1
    NFPTRDYN(NFPOSDYN)=J
  ENDIF
ENDDO

!        3.6 Count how many arrays fields are spread over

QFPTYPE%ISPD(:,:)=0
DO J=1,NFPOSDYN
  IJ=NFPTRDYN(J)
  ISPREAD=MAX(0,QFPTYPE%ISF(IJ)-2)
  DO JLEV=1,QFPTYPE%ILEV(IJ)
    QFPTYPE%ISPD(JLEV,IJ)=ISPREAD*(QFPTYPE%IDOM(JLEV,IJ))+(1-ISPREAD)
  ENDDO
ENDDO

!        3.7 "B-level" distribution (balanced here for direct transforms)

QFPTYPE%ISET(:,:)=1
IF (NPRTRV > 1) THEN
  IVCOUNT(:)=0
! Make 2 scans to improve balance : first those which have more that 1 field : 
  DO J=1,NFPOSDYN
    IJ=NFPTRDYN(J)
    IF (QFPTYPE%ISF(IJ) >= 1 .AND. QFPTYPE%ISKP(IJ) > 1) THEN
      DO JLEV=1,QFPTYPE%ILEV(IJ)
        IF (QFPTYPE%ILED(IJ) /= 1) THEN
!         Any field but U-vector momenta in direct transforms :
          ILOC=MINLOC(IVCOUNT)
          IVSET=ILOC(1)
          QFPTYPE%ISET(JLEV,IJ)=IVSET
          IVCOUNT(IVSET)=IVCOUNT(IVSET)+QFPTYPE%ISKP(IJ)
          IF (QFPTYPE%ILED(IJ) < 0) THEN
!           V-vector momenta => put counterpart U on the same set
            I=-QFPTYPE%ILED(IJ)
            QFPTYPE%ISET(JLEV,I)=IVSET
            IVCOUNT(IVSET)=IVCOUNT(IVSET)+QFPTYPE%ISKP(I)
          ENDIF
        ENDIF
      ENDDO
    ENDIF
  ENDDO
  DO J=1,NFPOSDYN
    IJ=NFPTRDYN(J)
    IF (QFPTYPE%ISF(IJ) >= 1 .AND. QFPTYPE%ISKP(IJ) == 1) THEN
      DO JLEV=1,QFPTYPE%ILEV(IJ)
        IF (QFPTYPE%ILED(IJ) /= 1) THEN
!         Any field but U-vector momenta in direct transforms :
          ILOC=MINLOC(IVCOUNT)
          IVSET=ILOC(1)
          QFPTYPE%ISET(JLEV,IJ)=IVSET
          IVCOUNT(IVSET)=IVCOUNT(IVSET)+QFPTYPE%ISKP(IJ)
          IF (QFPTYPE%ILED(IJ) < 0) THEN
!           V-vector momenta => put counterpart U on the same set
            I=-QFPTYPE%ILED(IJ)
            QFPTYPE%ISET(JLEV,I)=IVSET
            IVCOUNT(IVSET)=IVCOUNT(IVSET)+QFPTYPE%ISKP(I)
          ENDIF
        ENDIF
      ENDDO
    ENDIF
  ENDDO
ENDIF

QFPTYPE%ILOC(:,:)=0
IVPTR(:)=0
DO J=1,NFPOSDYN
  IJ=NFPTRDYN(J)
  IF (QFPTYPE%ISF(IJ) >= 1) THEN
    DO JLEV=1,QFPTYPE%ILEV(IJ)
      IVSET=QFPTYPE%ISET(JLEV,IJ)
      IVPTR(IVSET)=IVPTR(IVSET)+1
      QFPTYPE%ILOC(JLEV,IJ)=IVPTR(IVSET)
    ENDDO
  ENDIF
ENDDO 

MYSFSET=MYSETV

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVPOS',1,ZHOOK_HANDLE)
END SUBROUTINE SUVPOS
