#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE VPOS(CDCONF,KST,KEND,KSTGLO,                                   &
 & PU,PUL,PDIV,PV,PVL,PVOR,PT,PTL,PTM,PSPD,PSVD,PEV,PSP,PSPL,PSPM,        &
 & LDQ,   PQ,   LDQDER,   PQL,  PQM,  LDL,   PL,   LDLDER,   PLL,  PLM,   &
 & LDI,   PI,   LDIDER,   PIL,  PIM,  LDA,   PA,   LDADER,   PAL,  PAM,   &
 & LDS,   PS,   LDSDER,   PSL,  PSM,  LDR,   PR,   LDRDER,   PRL,  PRM,   &
 & LDGR,  PGR,  LDGRDER,  PGRL, PGRM, LDHL,  PHL,  LDHLDER,  PHLL, PHLM,  & 
 & LDTKE, PTKE, LDTKEDER, PTKEL,PTKEM,LDO3,  PO3,  LDO3DER,  PO3L, PO3M,  &
 & LDUD, PUAL,    PUOM,   LDDD, PDAL,  PDOM, LDUEN, PUEN, LDUNEBH,PUNEBH, & 
 & LDAERO,PAERO,LDGRG,PGRG,LDGHG,PGHG,LDTRAC,PTRAC,LDEXT, PEXT, &
 & PERA40,PLRCH4,PCH4S,POROG,PRCORI,PGM,PGEMU,PGELAM,POROGL,POROGM)  

!**** *VPOS*  - VERTICAL POST-PROCESSING - FULL POS

! PURPOSE.
! --------
!  Full-POS interface to POS.

! INTERFACE.
! ----------
!   *CALL* *VPOS*

! EXPLICIT ARGUMENTS
! --------------------
! CDCONF       : Configuration of work
! KST,KEND     : First and last adress of computation in NPROMA-sized arrays
! KSTGLO       : Start adress of computation in NGPTOT-sized arrays
! PU, PUL      : Zonal/x wind, its longitudinal/x derivative
! PDIV         : Divergence
! PVOR         : Vorticity
! PT           : Temperature (or virtual temperature if LSPRT)
! PTL,PTM      : Temperature (or virtual temperature if LSPRT) longitudinal/x and meridional/y derivatives
! PSPD,PSVD    : NH scaled pressure departure and vertical divergence
! PEV          : <eta dot dp/deta> on top of the atmosphere
! PSP,PSPL,PSPM: Surface pressure, its longitudinal/x and meridional/y derivatives
! LDQ          : .TRUE. if humidity present
! PQ           : Humidity
! LDQDER       : .TRUE. if humidity derivatives present
! PQL,PQM      : Humidity longitudinal/x and meridional/y derivatives
! LDL          : .TRUE. if liquid water present
! PL           : Liquid water
! LDLDER       : .TRUE. if liquid water derivatives present
! PLL,PLM      : Liquid water longitudinal/x and meridional/y derivatives
! LDI          : .TRUE. if ice present
! PI           : Ice
! LDIDER       : .TRUE. if ice derivatives present
! PIL,PIM      : Ice longitudinal/x and meridional/y derivatives
! LDA          : .TRUE. if cloud fraction present
! PA           : Cloud fraction
! LDADER       : .TRUE. if cloud fraction derivative present
! PAL,PAM      : Cloud fraction longitudinal/x and meridional/y derivatives
! LDS          : .TRUE. if snow present
! PS           : Snow
! LDSDER       : .TRUE. if snow derivative present
! PSL,PSM      : Snow longitudinal/x and meridional/y derivatives
! LDR          : .TRUE. if rain present
! PR           : Rain
! LDRDER       : .TRUE. if rain derivative present
! PRL,PRM      : Rain longitudinal/x and meridional/y derivatives
! LDGR         : .TRUE. if graupel present
! PGR          : Graupel
! LDGRDER      : .TRUE. if graupel derivative present
! PGRL,PGRM    : Graupel longitudinal/x and meridional/y derivatives
! LDHL         : .TRUE. if hail present
! PHL          : Hail
! LDHLDER      : .TRUE. if hail derivative present
! PHLL,PHLM    : Hail longitudinal/x and meridional/y derivatives
! LDTKE        : .TRUE. if TKE present
! PTKE         : TKE
! LDTKEDER     : .TRUE. if TKE derivative present
! PTKEL,PTKEM  : TKE longitudinal/x and meridional/y derivatives
! LDO3         : .TRUE. if ozone present
! PO3          : Ozone
! LDO3DER      : .TRUE. if ozone derivatives present
! PO3L,PO3M    : Ozone longitudinal/x and meridional/y derivatives
! LDUD         : .TRUE. if prognostic updraught 
! PUOM, PUAL   : ud velocity and mesh fraction
! LDDD         : .TRUE. if prognostic downdraught 
! PUOM, PUAL   : dd velocity and mesh fraction
! LDUEN        : .T. if updraught prognostic entrainment
! PUEN         : updraught prognostic entrainment
! LDUNEBH      : .T. if updraught pseudo historic cloud fraction
! PUNEBH       :  updraught pseudo historic cloud fraction
! LDAERO       : .TRUE. if aerosols present
! PAERO        : Aerosols
! LDGRG        : .TRUE. if GRG  present
! PGRG         : GEMS GRG 
! LDGHG        : .TRUE. if greenhouse gases present
! PGHG         : Greenhouse Gases
! LDTRAC       : .TRUE. if diagnostic tracers present
! PTRAC        : Diagnostic tracers
! LDEXT        : .TRUE. if extra fields present
! PEXT         : Extra fields
! POROG        : Surface geopotential
! PRCORI       : Coriolis force
! PGM          : Map factor
! PGEMU        : sine of geographic latitude
! PGELAM       : geographic longitude
! POROGL,POROGM: Surface geopotential longitudinal/x and meridional/y derivatives

! IMPLICIT ARGUMENTS
! --------------------

! METHOD.
! -------
!   SEE DOCUMENTATION

! EXTERNALS.
! ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 95-01-20 Thierry Bergot (spectral filtering for all
!                   fields in PV levels except wind component and Psi/Khi)
!        Modified : 95-03-01 by M.Hamrud (F90 re-write)
!        Modified : MARCH 1997 by K. YESSAD (parallelisation).
!    R. El Khatib : 97-06-02  3 NH fields
!    R. El Khatib : 98-08-27  Adiabatic pp.
!    R. El Khatib : 99-04-01 PEV
!    D.Dent       : 99-05-06  Support for log surface pressure
!    R. El Khatib : 99-08-25 Bugfix on surface temperatures 
!    R. El Khatib : 00-07-28 Cleaning
!       D. Dent   : 00-04-12 Support for computation of Montgomery geopotential
!       N. Wedi   : 01-11-08 Correct fullpos extrapolation
!    R. El Khatib : 01-04-23 send RHcls instead of Qcls to PHYMFPOS
!    R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-08-12 gfl
!    R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!      R. Engelen      15-Mar-2005 Greenhouse Gases
!        A. Untch      15-Mar-2005 Aerosols
!        J. Flemming   11-Apr-2005 Aerosol replaced with GRG
!    G. Hello     : 05-01-26 arome gfl
!    R. El Ouaraini & R. El Khatib : 05-07-28 Full-pos of CUF fields
!    M. Bellus    : 29-Sep-2006 Prognostic convection and MAC scheme (ALARO-0)
!    S. Serrar    : 07-09-2006 include tracers used for diagnostics only (GEMS)
!        M.Hamrud      01-Jul-2006 Revised surface fields
!    S. Serrar    : 22-03-2007 Post-Process ERA40 diagnostic fields
!    S. Serrar    : 17-07-2007 Two GFL fields methane related
!    N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LFPART2
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMFPDIM , ONLY : NFPGT1   ,NFPAUXB
USE YOMAUXB  , ONLY : GAUXBUF
USE YOMPHY   , ONLY : LMPHYS
USE YOMCST   , ONLY : RG
USE YOMSTA   , ONLY : RDTDZ1   ,NLEXTRAP
USE YOMGEM   , ONLY : VBH      ,VAH
USE YOMXFUB  , ONLY : XFUBUF
USE YOMXFU   , ONLY : NFDXFU
USE PTRXFU   , ONLY : MXRHCLS   ,MXTCLS  ,MXUCLS   ,MXVCLS, MXQCLS
USE YOMFPG   , ONLY : FPVALH   ,FPVBH    ,NFPLEV
USE YOMVPOS  , ONLY : XFPLEV   ,NXFPLEV
USE YOMOMPDIST , ONLY : GPP
USE YOM_YGFL , ONLY : NGFL_EXT, NAERO, NGHG, NGRG, NTRAC, NERA40
USE SURFACE_FIELDS_MIX , ONLY : SP_RR, YSP_RR,&
 & SD_VF, YSD_VF, SD_VV, YSD_VV, SD_XA, SP_SG, YSP_SG
USE YOMMCUF  , ONLY : RMCUFGP1  ,RMCUFGP2  ,RMCUFGP3  ,RMCUFGP4  ,RMCUFGP5

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOR(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPD(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSVD(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEV(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPM(NPROMA) 
LOGICAL           ,INTENT(IN)    :: LDQ 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDQDER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL(NPROMA,NFLEVG) 
LOGICAL                          :: LDLDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PLL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PLM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI(NPROMA,NFLEVG) 
LOGICAL                          :: LDIDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PIL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PIM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDA 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PA(NPROMA,NFLEVG) 
LOGICAL                          :: LDADER ! Argument NOT used
REAL(KIND=JPRB)                  :: PAL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PAM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDS 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PS(NPROMA,NFLEVG) 
LOGICAL                          :: LDSDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PSL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PSM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDR 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PR(NPROMA,NFLEVG) 
LOGICAL                          :: LDRDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PRL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PRM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDGR 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGR(NPROMA,NFLEVG) 
LOGICAL                          :: LDGRDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PGRL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PGRM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDHL 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHL(NPROMA,NFLEVG) 
LOGICAL                          :: LDHLDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PHLL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PHLM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDTKE 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE(NPROMA,NFLEVG) 
LOGICAL                          :: LDTKEDER ! Argument NOT used
REAL(KIND=JPRB)                  :: PTKEL(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PTKEM(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDO3 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3(NPROMA,NFLEVG) 
LOGICAL                          :: LDO3DER ! Argument NOT used
REAL(KIND=JPRB)                  :: PO3L(NPROMA,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)                  :: PO3M(NPROMA,NFLEVG) ! Argument NOT used
LOGICAL           ,INTENT(IN)    :: LDUD
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUAL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUOM(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDDD
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDAL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDOM(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDUEN
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUEN(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDUNEBH
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUNEBH(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDAERO(NAERO)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAERO(NPROMA,NFLEVG,NAERO)
LOGICAL           ,INTENT(IN)    :: LDGRG(NGRG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGRG(NPROMA,NFLEVG,NGRG)
LOGICAL           ,INTENT(IN)    :: LDGHG(NGHG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGHG(NPROMA,NFLEVG,NGHG)
LOGICAL           ,INTENT(IN)    :: LDTRAC(NTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTRAC(NPROMA,NFLEVG,NTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PERA40(NPROMA,NFLEVG,NERA40)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLRCH4(NPROMA,NFLEVG),PCH4S(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDEXT(NGFL_EXT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXT(NPROMA,NFLEVG,NGFL_EXT)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(KEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(KEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(KEND) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(KEND) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF, IBL

REAL(KIND=JPRB) :: ZDTDZSG    ! dT/g.dZ

REAL(KIND=JPRB) :: ZTSO(NPROMA)           ! Output surface temperature
REAL(KIND=JPRB) :: ZIOROG(NPROMA)         ! Interpolated surface geopotential
REAL(KIND=JPRB) :: ZTSI(NPROMA)           ! Interpolated surface temperature
REAL(KIND=JPRB) :: ZXFU(NPROMA,0:NFDXFU)  ! Instantaneous fields
REAL(KIND=JPRB) :: ZAUX(NPROMA,NFPAUXB)   !
REAL(KIND=JPRB) :: ZOROG(NPROMA),ZOROGL(NPROMA),ZOROGM(NPROMA)
REAL(KIND=JPRB) :: ZGEMU(NPROMA),ZGELAM(NPROMA)
REAL(KIND=JPRB) :: ZRCORI(NPROMA),ZGM(NPROMA),ZGRHL(NPROMA,NFLEVG)

REAL(KIND=JPRB),ALLOCATABLE :: ZIPH(:,:), ZIPF(:,:), ZTSEA(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "ctstar.intfb.h"
#include "gppre.intfb.h"
#include "phymfpos.intfb.h"
#include "pos.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VPOS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PREPARATIONS
!              ------------

IBL=(KSTGLO-1)/NPROMA+1

ZDTDZSG=RDTDZ1/RG


DO JROF=KST,KEND
  ZOROG (JROF)=POROG (JROF)
  ZRCORI(JROF)=PRCORI(JROF)
  ZGM   (JROF)=PGM   (JROF)
  ZGEMU (JROF)=PGEMU (JROF)
  ZGELAM (JROF)=PGELAM (JROF)
  ZOROGL(JROF)=POROGL(JROF)
  ZOROGM(JROF)=POROGM(JROF)
  IF (LFPART2) THEN
    ZIOROG(JROF)=SD_VF(JROF,YSD_VF%YFP1%MP,IBL)
  ELSE
    ZIOROG(JROF)=ZOROG(JROF)
  ENDIF
ENDDO

IF (LMPHYS) THEN
  DO JROF=KST,KEND
    IF (LFPART2) THEN
      ZTSI(JROF)=SP_RR(JROF,YSP_RR%YFP1%MP0,IBL)
    ELSE
      ZTSI(JROF)=SP_RR(JROF,YSP_RR%YT%MP0,IBL)
    ENDIF
    ZTSO(JROF)=SP_RR(JROF,YSP_RR%YT%MP0,IBL)
  ENDDO
ELSE
! ECMWF: only one T profile
!  ZTSI extrapolated from T(NLEXTRAP)
!  ZTSO not used
! Compute surface temperatures :
  ALLOCATE(ZIPH(NPROMA,0:NFLEVG))
  ALLOCATE(ZIPF(NPROMA,NFLEVG))
  ALLOCATE(ZTSEA(NPROMA))
  DO JROF=KST,KEND
    ZIPH(JROF,NFLEVG)=EXP(PSP(JROF))
  ENDDO
  CALL GPPRE(NPROMA,KST,KEND,NFLEVG,VAH,VBH,ZIPH,ZIPF)
  CALL CTSTAR(NPROMA,KST,KEND,PT(1,NLEXTRAP),ZIPH(1,NFLEVG), &
   & ZIPF(1,NLEXTRAP),ZIOROG,ZTSI,ZTSEA)  
  DO JROF=KST,KEND
    ZTSO(JROF)=ZTSEA(JROF)+ZDTDZSG*ZOROG(JROF)
  ENDDO
  DEALLOCATE(ZIPH)
  DEALLOCATE(ZIPF)
  DEALLOCATE(ZTSEA)
ENDIF

IF (NFDXFU > 0) THEN
  CALL SC2RDG(XFUBUF,KSTGLO,KST,KEND,ZXFU(1:NPROMA,1:NFDXFU))
ENDIF

!*       4.    PERFORM VERTICAL POST-PROCESSING
!              --------------------------------

CALL POS(CDCONF,KST,KEND,XFPLEV,NXFPLEV,KSTGLO,FPVALH,FPVBH,NFPLEV,ZIOROG,ZTSI,ZTSO, &
 & PU,PUL,PDIV,PV,PVL,PVOR,PT,PTL,PTM,LDQ,PQ,LDQDER,PQL,PQM,LDL,PL,LDI,PI,    &
 & LDS,PS,LDR,PR,LDGR,PGR,LDHL,PHL,LDTKE,PTKE,&
 & PSP,PSPL,PSPM,LDEXT,PEXT,LDA,PA,LDO3,PO3,&
 & LDUD, PUAL, PUOM, LDDD, PDAL, PDOM, LDUEN, PUEN, LDUNEBH, PUNEBH,&
 & LDAERO,PAERO,LDGHG,PGHG,LDTRAC,PTRAC,LDGRG,PGRG,&
 & PSPD,PSVD,PEV,PLRCH4,PCH4S,PERA40,ZOROG,ZRCORI,ZGM,ZOROGL,ZOROGM,GPP(1,1,IBL),NFPGT1,ZAUX,     &
 & NFPAUXB,SD_XA(1,1,1,IBL),RMCUFGP1(KSTGLO),RMCUFGP2(KSTGLO),RMCUFGP3(KSTGLO),&
 & RMCUFGP4(KSTGLO),RMCUFGP5(KSTGLO))  

!*       5.    PERFORM PHYSICO-DYNAMIC POST-PROCESSING OVER THE MODEL SURFACE
!              --------------------------------------------------------------
ZGRHL(:,:)=0._JPRB
IF(LDGR) THEN
  IF (LDHL) THEN
    ZGRHL=PGR+PHL
  ELSE
    ZGRHL=PGR
  ENDIF
ENDIF

CALL PHYMFPOS(NPROMA,KST,KEND,ZOROG,ZGM,ZGEMU,ZGELAM,PU,PV,PT,&
 & LDQ,PQ,LDL,PL,LDI,PI,LDR, &
 & PR,LDS,PS,LDGR,ZGRHL,LDA,PA,PSP,SP_RR(1,YSP_RR%YT%MP0,IBL),&
 & SP_SG(1,YSP_SG%YF%MP0,IBL),SP_RR(1,YSP_RR%YW%MP0,IBL), &
 & SP_RR(1,YSP_RR%YRR(4)%MP0,IBL),SD_VV(1,YSD_VV%YHV%MP,IBL),  &
 & SD_VF(1,YSD_VF%YZ0F%MP,IBL), SD_VF(1,YSD_VF%YLSM%MP,IBL),       &
 & SD_VV(1,YSD_VV%YZ0H%MP,IBL),SD_VF(1,YSD_VF%YVEG%MP,IBL), &
 & SD_VV(1,YSD_VV%YIVEG%MP,IBL),SD_VV(1,YSD_VV%YARG%MP,IBL),          &
 & SD_VV(1,YSD_VV%YD2%MP,IBL),SD_VV(1,YSD_VV%YSAB%MP,IBL),&
 & ZXFU(1,MXTCLS),ZXFU(1,MXRHCLS),ZXFU(1,MXQCLS),            &
 & ZXFU(1,MXUCLS), ZXFU(1,MXVCLS),ZAUX,NFPAUXB,GPP(1,1,IBL),NFPGT1)  

!*       6.    WRITE OUT GRIDPOINT FIELDS INTO BUFFERS/WORKFILES
!              -------------------------------------------------

IF (NFPAUXB > 0) THEN
  CALL SC2WRG(GAUXBUF,KSTGLO,KST,KEND,ZAUX)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('VPOS',1,ZHOOK_HANDLE)
END SUBROUTINE VPOS
