SUBROUTINE WRSFP(CDCONF)

!**** *WRSFP*  - WRITE OUT POST-PROCESSED DYNAMIC FIELDS TO ARPEGE/ALADIN FILE
!                AS SPECTRAL COEFFICIENTS OR GRID-POINT FIELDS FOR THOSE WHICH 
!                ARE NOT SPECTRALLY FITTED.

!     PURPOSE.
!     --------
!        To copy the vertically post-processed fields from post-processing
!        spectral arrays or unfitted upper air array to output files

!**   INTERFACE.
!     ----------
!       *CALL* *WRSFP*

!        EXPLICIT ARGUMENTS
!        ------------------
!        CDCONF : configuration of work

!        IMPLICIT ARGUMENTS
!        ------------------
!        See modules above.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 98-08-03 R. El Khatib
!        Modified : 99-01-06 Substantial rewrite from wrsfpsm/dm
!        Modified : 00-01-10 D.Dent : pass first dimension to FPSPNORM
!    R. El Khatib : 99-12-23 Change arg. to PREPACKA and GATHFLNM
!    R. El Khatib : 01-03-16 IDATEF(10)=time of previous event ;
!                   N3DINI for tests
!    R. El Khatib : 01-04-10 Enable post-processing of filtered spectra
!    R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!    R. El Khatib : 04-07-22 SPREORD moved inside PREPACKA
!    R. El Khatib : 05-02-22 Change interface to SUPPDATE
!    R. El Khatib : 05-03-03 Prints out limited with epsilon(1.)
!     -----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NPROC    ,N3DINI
USE PARFPOS  , ONLY : JPOSLEN  ,JPOSDIR
USE YOMFPC   , ONLY : CFPIDEN  ,NFPGRIB
USE YOMFP4L  , ONLY : NDYNFPL  ,NFPSPBL, NFPSSETG ,NFPSLOCG
USE YOMCT1   , ONLY : LRFILAF
USE YOMFPOP  , ONLY : CFPCA
USE YOMFPSP  , ONLY : SPBFP
USE YOMMP    , ONLY : MYPROC   ,NSTROUT
USE YOMFPCT0 , ONLY : NFPCT0
USE YOMXFU   , ONLY : NRAZTS   ,JPFUXT   ,NFRRAZ   ,N1RAZ    ,LXFU

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
!     CLP     : prefix of field names
!     CLSUFF  : suffix of field names
!     ILEV    : field level to be written out on file
!     INBIT   : Number of bits for coding
!     IPTR    : pointer of each subdomain in the global overdimensionned
!               output array
!     ISIZEG  : global size of a field for each subdomain (including Ezone)
!     ISIZPK  : overdimensionned value of ISIZEG for safety while packing
!     IREC    : number of fields to be written out on file
!     LLDYN   : .TRUE. if dynamic fields conf. ; else physical fields

!     IFILE  : kind of file to open
!     IDTIME : time indicator ;
!              1 = date of the starting file + model forecast range
!     IDATEF : date in initial file
!     CLFPFN : complete filename
!     IULFP  : logical unit number of output files

!     INFG   : total number of spectral fields to write out

!     INFL   : local number of spectral fields in packing distribution
!              (NSTROUT)
!     INFD   : number of spectral fields for each processor in packing
!              distribution (NSTROUT)
!     IFLDOFF: fields offset after distribution among NSTROUT procs.
!     IFLDA  : absolute field pointer to be extracted

!     ZSPECFL: field-distributed spectral array

!     CLNMAL : local fields names 
!     ILOGAL : actual length of a field for local fields
!     INOMAL : actual length of a field name for local fields
!     IMASKL : binary mask ; always =1 here
!     ZSPECFD: reordered specteal fields for a processor-set of fields
!     CLNMAD : fields names for a processor-set of field
!     ILOGAD : actual length of fields for a processor-set of fields
!     INOMAD : actual length of a field name for a processor-set of fields
!     IMASKD : binary mask ; always =1 here

!     ZAVEG  : global norms of the fields

INTEGER(KIND=JPIM) :: IPTR(1), IREC(1), ISIZEG(1), ISIZPK(1)

CHARACTER (LEN=JPOSDIR+4+JPOSLEN+5) :: CLFPFN(1)
CHARACTER (LEN = 16) ::  CLPRINT, CLFPCA(1)
INTEGER(KIND=JPIM) :: IULFP(1), IDATEF(11)

INTEGER(KIND=JPIM) :: INFD(NPROC), IFLDOFF(NPROC)

REAL(KIND=JPRB),     ALLOCATABLE :: ZSPECFL(:,:), ZSPECFD(:,:)
REAL(KIND=JPRB),     ALLOCATABLE :: ZAVEG(:)
INTEGER(KIND=JPIM),  ALLOCATABLE :: INOMAL(:,:), ILOGAL(:,:)
INTEGER(KIND=JPIM),  ALLOCATABLE :: INOMAD(:,:), ILOGAD(:,:)
INTEGER(KIND=JPIM),  ALLOCATABLE :: IMASKL(:,:), IMASKD(:,:)
INTEGER(KIND=JPIM),  ALLOCATABLE :: ILEV(:), INBIT(:)
CHARACTER(LEN=16),ALLOCATABLE :: CLNMAL(:,:), CLNMAD(:,:)
CHARACTER(LEN=8),ALLOCATABLE :: CLP(:)
CHARACTER(LEN=12),ALLOCATABLE :: CLSUFF(:)

INTEGER(KIND=JPIM) :: IDOM, INBARP, INFG, INFL,&
 & INIMES, IOFFL, IOUTPROC, IREP, ITAG, JFLD, &
 & JROC, IDTIME, IFILE, ILEN  

LOGICAL :: LLCOSP, LLDYN, LLIMST, LLNORM, LLEXIST, LLFICP

REAL(KIND=JPRB) :: ZAVED(1,1), ZAVEL(1,1), ZMAXD(1,1), ZMAXL(1,1)
REAL(KIND=JPRB) :: ZMIND(1,1), ZMINL(1,1)
REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gathflnm.intfb.h"
#include "fpspnorm.intfb.h"
#include "ini1wrfp.intfb.h"
#include "ini2wrfp.intfb.h"
#include "ini3wrfp.intfb.h"
#include "newfa.intfb.h"
#include "openfpfa.intfb.h"
#include "prepacka.intfb.h"
#include "sumpioh.intfb.h"
#include "suppdate.intfb.h"
#include "trwvtof.intfb.h"
#include "wrhfp.intfb.h"

!     ------------------------------------------------------------------

!*    1. PREPARATIONS
!        ------------

IF (LHOOK) CALL DR_HOOK('WRSFP',0,ZHOOK_HANDLE)
LLCOSP=.TRUE.
CALL INI1WRFP(CDCONF,LLCOSP,INFG,IPTR,IREC,LLDYN,ISIZEG,ISIZPK)

IDOM=1

IOUTPROC=1

!*    2. EXTRACT AND COPY SPECTRAL FIELDS
!        --------------------------------

IF (INFG > 0) THEN

!*      2.1 SETUP CONTIGUS DISTRIBUTION AMONG "I/O" PROCS (<=> INFL>0)

  CALL SUMPIOH(INFG,NPROC,NSTROUT,INFD,IFLDOFF)
  INFL=INFD(MYPROC)

!*      2.3 COMPUTE SPECTRAL NORMS AND SEND TO OUTPUT PROCESSOR

  ALLOCATE(ZAVEG(INFG))
  CALL FPSPNORM(SPBFP,NFPSPBL,INFG,IOUTPROC,NFPSSETG,NFPSLOCG,ZAVEG)
  ZEPS=EPSILON(1._JPRB)
  DO JFLD=1,INFG
    IF (ABS(ZAVEG(JFLD)) < ZEPS) ZAVEG(JFLD)=0._JPRB
  ENDDO

!*      2.4 TRANSPOSE THE WAVE-DISTRIBUTED SET OF FIELDS TO THE 
!           FIELD-DISTRIBUTED SET OF SPECTRA

  ALLOCATE(ZSPECFL(ISIZPK(1),INFL))
  CALL TRWVTOF(NFPSSETG,NFPSLOCG,INFG,SPBFP,NFPSPBL,ZSPECFL,INFL,INFD, &
   & IFLDOFF,ISIZPK(1))
  DEALLOCATE(SPBFP)

!       2.5 IO PROCESSORS PRECONDITION DATA

  IF (INFL > 0) THEN

!*        2.5.3 Setup parameters for packing and field name construction

    ALLOCATE(CLP(INFL))
    ALLOCATE(ILEV(INFL))
    ALLOCATE(CLSUFF(INFL))
    ALLOCATE(INBIT(INFL))
    ALLOCATE(IMASKL(INFL,IDOM))

    IOFFL=IFLDOFF(MYPROC)
    CALL INI2WRFP(CDCONF,LLDYN,LLCOSP,INFL,IDOM,IOFFL,CLP,CLSUFF,&
     & ILEV,INBIT,IMASKL)  

!         2.5.4 Pack data & buid field names

    ALLOCATE(ILOGAL(INFL,IDOM))
    ALLOCATE(CLNMAL(INFL,1))
    ALLOCATE(INOMAL(INFL,1))

    CALL PREPACKA(1,ZSPECFL,CLP,ILEV,CLSUFF,INFL,ISIZPK(1),IMASKL, &
     & LLCOSP,NFPGRIB,INBIT,CFPCA,INOMAL,ILOGAL,CLNMAL,IPTR,ISIZEG,ISIZPK)  

!*        2.5.5 Send packed data & names in a single message

    IF (MYPROC /= IOUTPROC) THEN
      LLNORM=.FALSE.
      ITAG=MYPROC
      CALL GATHFLNM(0,ITAG,INFL,IDOM,CLNMAL,INOMAL,ZSPECFL,&
       &ILOGAL,IMASKL,ISIZPK(1),ZAVEL,ZMINL,ZMAXL,LLNORM,IPTR)
      DEALLOCATE(IMASKL)
      DEALLOCATE(ILOGAL)
      DEALLOCATE(CLNMAL)
      DEALLOCATE(INOMAL)      
    ENDIF

!*        2.5.6 Release useless arrays

    DEALLOCATE(INBIT)
    DEALLOCATE(CLSUFF)
    DEALLOCATE(ILEV)
    DEALLOCATE(CLP)

  ENDIF

  IF (MYPROC /= IOUTPROC) DEALLOCATE(ZSPECFL)

!*      2.6 OUTPUT PROCESSOR COLLECTS DATA AND WRITE OUT TO FILES

  IF (MYPROC == IOUTPROC) THEN

!*        2.6.2 Open file

    INIMES = 0
    LLIMST=.TRUE.
    CALL INI3WRFP(CLFPFN,CLFPCA,IULFP,INBARP,NFPCT0)
    IDTIME=1-N3DINI
    IF (LXFU) THEN
      CALL SUPPDATE(IDTIME,NRAZTS,JPFUXT,NFRRAZ,N1RAZ,IDATEF)
    ELSE
      CALL SUPPDATE(KDTIME=IDTIME,KDIME=1,KPPDATE=IDATEF)
    ENDIF

    INQUIRE(FILE=CLFPFN(1),EXIST=LLEXIST)
    IF (LLEXIST) THEN
      IFILE=0
      CALL OPENFPFA(1,IULFP(1),CLFPFN(1),IFILE,IDATEF,&
       & ILEN,ILEN,LLFICP,.FALSE.)  
    ELSE
      CALL NEWFA(INBARP,CLFPCA(1),IULFP(1),CLFPFN(1),LLIMST,INIMES,&
       & CFPIDEN,IDATEF)  
    ENDIF

!   Print title the first time : 
    WRITE(NULOUT,FMT='(''FULL-POS SPNORMS'')')

!*        2.6.3 Write out data and norms

    DO JROC=1,NPROC
      IF (INFD(JROC) > 0) THEN

        IF (JROC == MYPROC) THEN
!               use my own data
          DO JFLD=1,INFL
            CALL FAISAN(IREP,IULFP(1),&
             & CLNMAL(JFLD,1)(1:INOMAL(JFLD,1)),ZSPECFL(1,JFLD),&
             & ILOGAL(JFLD,1))  
            CLPRINT=CLNMAL(JFLD,1)(1:INOMAL(JFLD,1))//' '
            WRITE(NULOUT,FMT='(1X, A16,'' : '',E21.15)')&
             & CLPRINT,ZAVEG(IFLDOFF(JROC)+JFLD)  
          ENDDO
          DEALLOCATE(ZSPECFL)
          DEALLOCATE(CLNMAL)
          DEALLOCATE(ILOGAL)
          DEALLOCATE(INOMAL)
          DEALLOCATE(IMASKL)
        ELSE
!               collect data from the other I/O processors :
          ALLOCATE(ZSPECFD(ISIZPK(1),INFD(JROC)))
          ALLOCATE(ILOGAD(INFD(JROC),IDOM))
          ALLOCATE(INOMAD(INFD(JROC),1))
          ALLOCATE(CLNMAD(INFD(JROC),1))
          ALLOCATE(IMASKD(INFD(JROC),IDOM))
          LLNORM=.FALSE.
          ITAG=JROC
          CALL GATHFLNM(1,ITAG,INFD(JROC),IDOM,CLNMAD,INOMAD,&
           &ZSPECFD,ILOGAD,IMASKD,ISIZPK(1),ZAVED,ZMIND,ZMAXD,&
           &LLNORM,IPTR)
          DO JFLD=1,INFD(JROC)
            CALL FAISAN(IREP,IULFP(1),&
             & CLNMAD(JFLD,1)(1:INOMAD(JFLD,1)),ZSPECFD(1,JFLD),&
             & ILOGAD(JFLD,1))  
            CLPRINT=CLNMAD(JFLD,1)(1:INOMAD(JFLD,1))//' '
            WRITE(NULOUT,FMT='(1X, A16,'' : '',E21.15)')&
             & CLPRINT,ZAVEG(IFLDOFF(JROC)+JFLD)  
          ENDDO
          DEALLOCATE(ZSPECFD)
          DEALLOCATE(CLNMAD)
          DEALLOCATE(ILOGAD)
          DEALLOCATE(INOMAD)
          DEALLOCATE(IMASKD)
        ENDIF

      ENDIF
    ENDDO

!*        2.6.4 Close file

    IF (LRFILAF) THEN
      CALL LFILAF(IREP,IULFP,.FALSE.)
    ENDIF
    CALL LFIMST(IREP,IULFP,.FALSE.)
    CALL FAIRME(IREP,IULFP,'KEEP')

  ENDIF

!*      2.7 RELEASE ARRAYS

  DEALLOCATE(ZAVEG)

ENDIF ! INFG

!*    3. EXTRACT AND COPY GRID POINT UNFITTED FIELDS
!        -------------------------------------------

IF (NDYNFPL > 0) THEN
  CALL WRHFP(CDCONF)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRSFP',1,ZHOOK_HANDLE)
END SUBROUTINE WRSFP
