#ifdef DOC
!     ------------------------------------------------------------------
!     Functions needed for the background dependent change 
!     of humidity control variable
!
!     Elias Holm    02-08-30:  Original
!     Elias Holm    03-12-05:  Functions made vertical resolution
!                              independent and coefficient setup
!                              moved to sujbchvar.F90
!     Elias Holm    04-01-23:  Changed name from fqchvar to fjbchvar
!                              and added functions for ozone
!     Elias Holm    05-10-20:  Change FQT (and add FQS) to represent
!                              tropopause transition of coefficients
!     Elias Holm    07-12-13:  Add functions for Q-T balance and
!                              more detailed supersaturated statisitics
!     ------------------------------------------------------------------
#endif

INTEGER(KIND=JPIM) :: KILEV, KSLEV
REAL(KIND=JPRB) :: PRH, PRHB, POZONE
REAL(KIND=JPRB) :: FQS, FQT, FSUB
REAL(KIND=JPRB) :: FQL, FQH, FO3, FQMIN, FQMAX
REAL(KIND=JPRB) :: FQSIGMA, FQTBAL, FO3SIGMA

#ifdef DOC
!     1. Tropopause transition of errors and q-T correlation
#endif
FQS(KSLEV)=RQS(MAX(MIN(KSLEV,6),0))
FQT(KSLEV)=RQT(MAX(MIN(KSLEV,6),0))
#ifdef DOC
!     2. Normalization for q/q_s(T^b)-based cv
!        Combines: L = Subsaturated (rh^b < 1)
!                  H = Supersaturated (rh^b > 1)
!        Inputs:   kilev = model level
!                  prhb  = relative humidity background
!                  prh   = relative humidity
#endif
FSUB(PRHB) = 0.5_JPRB+SIGN(0.5_JPRB,1.0_JPRB-PRHB)
FQL(KILEV,PRH) = MIN(MAX(PRH,RQLMIN(KILEV)),RQLMAX(KILEV))
FQH(KILEV,PRH) = MIN(MAX(PRH,RQHMIN(KILEV)),RQHMAX(KILEV))
FQSIGMA(KILEV,KSLEV,PRHB,PRH) = FQS(KILEV-KSLEV)& 
 &  *(           FSUB(PRHB) *(                  RQLSTD(KILEV,0)& 
 &                            +FQL(KILEV,PRH)*( RQLSTD(KILEV,1)&
 &                            +FQL(KILEV,PRH)*( RQLSTD(KILEV,2)&
 &                            +FQL(KILEV,PRH)*( RQLSTD(KILEV,3)&
 &                            +FQL(KILEV,PRH)*( RQLSTD(KILEV,4)&
 &                            +FQL(KILEV,PRH)*( RQLSTD(KILEV,5)))))))&
 &    +(1.0_JPRB-FSUB(PRHB))*(                  RQHSTD(KILEV,0)&
 &                            +FQH(KILEV,PRH)*( RQHSTD(KILEV,1)&
 &                            +FQH(KILEV,PRH)*( RQHSTD(KILEV,2)&
 &                            +FQH(KILEV,PRH)*( RQHSTD(KILEV,3)&
 &                            +FQH(KILEV,PRH)*( RQHSTD(KILEV,4)&
 &                            +FQH(KILEV,PRH)*( RQHSTD(KILEV,5))))))))
! EH: This limiting is noisy, keep for development but use the simpler one below
!FQMIN(KILEV,PRHB) =            FSUB(PRHB) *RQLMIN(KILEV)&
!                  & +(1.0_JPRB-FSUB(PRHB))*RQHMIN(KILEV)
!FQMAX(KILEV,PRHB) =            FSUB(PRHB) *RQLMAX(KILEV)&
!                  & +(1.0_JPRB-FSUB(PRHB))*RQHMAX(KILEV)
FQMIN(KILEV,PRHB) = RQLMIN(KILEV)
FQMAX(KILEV,PRHB) = RQHMAX(KILEV)
#ifdef DOC
!        3. Humidity-temperature coupling
#endif
FQTBAL(KILEV,KSLEV,PRHB)= MAX(0.0_JPRB,&
              &          FQT(KILEV-KSLEV)*FSUB(PRHB) &
              &  *(                   RQTBAL(KILEV,0)&
              &    +FQL(KILEV,PRHB)*( RQTBAL(KILEV,1)&
              &    +FQL(KILEV,PRHB)*( RQTBAL(KILEV,2)&
              &    +FQL(KILEV,PRHB)*( RQTBAL(KILEV,3)&
              &    +FQL(KILEV,PRHB)*( RQTBAL(KILEV,4)&
              &    +FQL(KILEV,PRHB)*( RQTBAL(KILEV,5))))))))
#ifdef DOC
!     3. Normalization coefficients for background ozone
!        Inputs: kilev    = model level
!                pozone   = ozone mass mixing ratio
!        Note: A special treatment is needed for low ozone due
!              to patches of constant low values  
#endif
FO3(KILEV,POZONE)= MAX(POZONE,RO3MIN(KILEV))
FO3SIGMA(KILEV,POZONE) =                      RO3STD(KILEV,0)&
                 &       +FO3(KILEV,POZONE)*( RO3STD(KILEV,1)& 
                 &       +FO3(KILEV,POZONE)*( RO3STD(KILEV,2)&
                 &       +FO3(KILEV,POZONE)*( RO3STD(KILEV,3)&
                 &       +FO3(KILEV,POZONE)*( RO3STD(KILEV,4)&
                 &       +FO3(KILEV,POZONE)*( RO3STD(KILEV,5))))))
