!***************************************************************************
!                                                          DEBUT DE QASTOB *
!***************************************************************************
!  CD QASTOB:
!  ---------  MODELISATION STATISTIQUE DES CORRELATIONS ENTRE ERREURS
!             D'OBSERVATION.

!  AUTEUR:    MCP + PB   LE 27/05/91
!  -------

!  HYPOTHESES:
!  ----------
!            POUR LES RADIOSONDAGES, LES COVARIANCES ENTRE ERREURS DE
!            TEMPERATURE SONT OBTENUES PAR DERIVATION DES COVARIANCES
!            ENTRE ERREURS DE GEOPOTENTIEL (LES ECARTS-TYPES DOIVENT
!            DONC ETRE COHERENTS).
!            POUR LES SATEM, LES EPAISSEURS SONT CONSIDEREES COMME DES
!            DERIVEES VERTICALES (TEMPERATURES) ET NON COMME DES DIFFE-
!            RENCES, CONTRAIREMENT AUX ERREURS GUESS. LE MODELE ME
!            SEMBLE FAUX ET EST A REVOIR (MCP LE 25/04/91)

!  MODIFICATIONS:  effet de pepite                             V.Casse 03/92
!  -------------
!***************************************************************************

! A. DEFINITION DES VARIABLES ET FONCTIONS UTILISEES.
!    -----------------------------------------------

! A.1 ARGUMENTS DES FONCTIONS IN-LINE:
!     - - - - - - - - - - - - - - - -
!     PD   : DISTANCE (METRES) ENTRE LES 2 POINTS CONCERNES;
!     P.   : PRESSION (PASCALS) DES POINTS CONCERNES;
!     PET..: ECART-TYPE D'ERREUR D'OBS POUR PARAMETRE ET POINT CONCERNES
!            (VALEURS EXPERIMENTALES).

!   ATTENTION! LA VARIABLE PD DEPEND DE LA SPHERE SUR LAQUELLE ON SE PLACE
!              ATTENTION DONC DANS LES ROUTINES D'APPEL...

!   ARGUMENTS A INITIALISER PAR LE CD QAERRO:

!     PKZO5  : "PORTEE" VERTICALE Z RS
!     PKHO5  : "PORTEE" VERTICALE H RS
!     PKUO5  : "PORTEE" VERTICALE U RS
!     PKUO6  : "PORTEE" VERTICALE U PILOT
!     PKDZO7 : "PORTEE" VERTICALE DZ SATEM

!     PODZO7 : "PORTEE" HORIZONTALE DZ SATEM

! A.2 FONCTIONS IN-LINE UTILISEES (DU CD QASTAT):
!     - - - - - - - - - - - - - - - - - - - - - -
!     FV    : FONCTION DE STRUCTURE VERTICALE
!     FRVTT : FONCTION DE CORR. VERTICALE   T_T
!     FRVTZ :    "         "        "       T_Z
!     FRVZT :    "         "        "       Z_T
!     FRHTT :    "         "    HORIZONTALE T_T

!     ------------------------------------------------------------------

! B. MODELES DE COVARIANCE. (TERME GENERIQUE:  FC X Y O N ,   AVEC
!    ---------------------   X,Y : VARIABLES METEO  ET  N: TYPE D'OBS)

! B.3 SATOB:
!     - - -
!  VENTS:
REAL(KIND=JPRB) :: FCUUO3
REAL(KIND=JPRB) :: PET1,PET2,PD,P_1,P_2
FCUUO3(PET1,PET2,PD,P_1,P_2) = 0.0_JPRB
!  HUMIDITES:
REAL(KIND=JPRB) :: FCHHO3
FCHHO3(PET1,PET2,PD,P_1,P_2) = 0.0_JPRB

! B.5 RADIOSONDAGES:
!     - - - - - - - -
!  GEOPOTENTIEL ET TEMPERATURE:
REAL(KIND=JPRB) :: FCZZO5,FCTZO5,FCZTO5,FCTTO5
REAL(KIND=JPRB) :: PKZO5
FCZZO5(PET1,PET2,P_1,P_2,PKZO5) =&
 &0.8_JPRB*PET1*PET2*FV(P_1,P_2,PKZO5)
FCTZO5(PET1,PET2,P_1,P_2,PKZO5) =&
 &0.8_JPRB*PET1*PET2*FRVTZ(P_1,P_2,PKZO5)
FCZTO5(PET1,PET2,P_1,P_2,PKZO5) = &
&0.8_JPRB*PET1*PET2*FRVZT(P_1,P_2,PKZO5)
FCTTO5(PET1,PET2,P_1,P_2,PKZO5) = &
&0.8_JPRB*PET1*PET2*FRVTT(P_1,P_2,PKZO5)
!  HUMIDITES:
REAL(KIND=JPRB) :: FCHHO5 
REAL(KIND=JPRB) :: PKHO5
FCHHO5(PET1,PET2,P_1,P_2,PKHO5) = 0.8_JPRB*PET1*PET2*FV(P_1,P_2,PKHO5)
!  VENTS:
REAL(KIND=JPRB) :: FCUUO5
REAL(KIND=JPRB) :: PKUO5
FCUUO5(PET1,PET2,P_1,P_2,PKUO5) = 0.8_JPRB*PET1*PET2*FV(P_1,P_2,PKUO5)

! B.6 PILOT:
!     - - - -
!  VENTS:
REAL(KIND=JPRB) :: FCUUO6
REAL(KIND=JPRB) :: PKUO6
FCUUO6(PET1,PET2,P_1,P_2,PKUO6) = 0.8_JPRB*PET1*PET2*FV(P_1,P_2,PKUO6)

! B.7 SATEM:
!     - - - -
!  EPAISSEURS (ON LES CONSIDERE COMME DES TEMPERATURES AU NIVEAU MEDIAN):
REAL(KIND=JPRB) :: FCZZO7
REAL(KIND=JPRB) :: P_1S,P_1I,P_2S,P_2I,PODZO7,PKDZO7
FCZZO7(PET1,PET2,PD,P_1S,P_1I,P_2S,P_2I,PODZO7,PKDZO7) = 0.8_JPRB *&
     &PET1*PET2*FRHTT(PD,PODZO7)&
              &*FRVTT(SQRT(P_1S*P_1I),SQRT(P_2S*P_2I),PKDZO7)
!  HUMIDITES:
REAL(KIND=JPRB) :: FCHHO7
FCHHO7(PET1,PET2,PD,P_1,P_2) = 0.0_JPRB
!                                                           ****************
!                                                            FIN DE QASTOB *
!***************************************************************************

