MODULE AEOLUS_GETAMD_MOD
CONTAINS
SUBROUTINE AEOLUS_GETAMD(PTF5,&
&                        PPRESH5, PGEOPH5, &
&                        PPRESF5, PGEOPF5, &
&                        PUF5,    PVF5,    &
&                        PQF5,    PCLWF5,  PCIWF5, PCCF5, &
&                        PRHF5,   PUV10M5,                &
&                        KLEN,    KDLEN                   &
&                       )
!  #[ 

!     Purpose.
!     --------
!       Contains IFS-Aeolus interface routines

!     Author.
!     -------
!        David Tan *ECMWF*

!     Modifications.
!     --------------
!     2007-Mar-23   David Tan         Original
!     2007-Jun-01   David Tan         Coding norm compliance
!     2007-Jul-04   David Tan         Prototype aeolus_getamd
!     2007-Jul-19   David Tan         Aeolus_getamd: tidy variable names
!                                     without skipping L2B obs
!     2007-Sep-25   David Tan         Make zhook_handle local to each routine
!     2007-Nov-04   T.Wilhelmsson
!                   O.Spaniel       removing of severe breaks of coding norms
!     2007-Nov-27   David Tan         Use trigger l1b_bin01
!     2008-Jan-25   David Tan         Use aeolus_auxmet.offset@aeolus_hdr
!     2008-Jan-31   David Tan         Use aeolus_hdr.offset@sat
!     2008-Feb-14   David Tan         Suspend use of aeolus offsets.
!     2008-Mar-01   David Tan         Use JMASTER in aeolus offsets.
!     2008-Sep-22   David Tan         Use ROBSU(JMASTER,*) in debug print of auxmet offset.
!     2008-Oct-01   David Tan         Recognize predicted orbit locations via aeolus_hdrflag.

!     ------------------------------------------------------------------
USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT
USE YOMANCS  , ONLY : RMDI
USE YOMDB

!     -------------------------------------------------------------------------
! Type and variable definitions
IMPLICIT NONE

!!   Model U/V/T/Q/O3/CLW/CIW/CC/CSW/CRW/CGW at observation points, model levels
!REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG) ::&
! & ZUF5,ZVF5,PTF5,ZQF5,ZO3F5,ZCLWF5,ZCIWF5,ZCCF5,ZQN5, &
! & ZCSWF5,ZCRWF5,ZCGWF5

REAL(KIND=JPRB), INTENT(IN), DIMENSION(KDLEN,0:NFLEVG) :: PPRESH5,PGEOPH5
REAL(KIND=JPRB), INTENT(IN), DIMENSION(KDLEN,NFLEVG)   :: PPRESF5,PGEOPF5

REAL(KIND=JPRB), INTENT(IN), DIMENSION(KDLEN,0:NFLEVG) ::&
 & PUF5,PVF5,PTF5,&
 & PQF5,&
 & PCLWF5,PCIWF5,PCCF5

REAL(KIND=JPRB), INTENT(IN), DIMENSION(KDLEN,NFLEVG) :: PRHF5

INTEGER(KIND=JPIM), PARAMETER :: IDBDY2=2
REAL(KIND=JPRB), INTENT(IN), DIMENSION(KDLEN,IDBDY2) :: PUV10M5

INTEGER(KIND=JPIM),INTENT(IN) :: KLEN
INTEGER(KIND=JPIM),INTENT(IN) :: KDLEN 

! Local variables
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! Other local variables
INTEGER(KIND=JPIM), PARAMETER :: IHDRFLAG_ACTUAL=0
INTEGER(KIND=JPIM), PARAMETER :: ISONDE_TYPE_BIN01_L1B=1
INTEGER(KIND=JPIM) :: i_aeolus_hdr ! indices for ODB arrays
INTEGER(KIND=JPIM) :: i_sat        ! indices for ODB arrays

! Loop counters
INTEGER(KIND=JPIM) :: ILEV
INTEGER(KIND=JPIM) :: i_num_auxmet
INTEGER(KIND=JPIM) :: jauxmet

INTEGER(KIND=JPIM) :: JOBS
INTEGER(KIND=JPIM) :: JMASTER

#include "openmp_obs.h"

  !  #[
       
  ! 1.0 Preliminaries

  IF (LHOOK) CALL DR_HOOK('AEOLUS_GETAMD_MOD:AEOLUS_GETAMD',0,ZHOOK_HANDLE)

  !  #]

  !  #[
       
  ! 2.0 Write Aux Met Data to ODB tables

    WRITE (NULOUT,*) ' aeolus_getamd: not yet implemented'
    WRITE (NULOUT,*) ' aeolus_getamd: size(ptf5(:,1)) = ', size(ptf5(:,1))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ptf5(1,:)) = ', size(ptf5(1,:))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ptf5     ) = ', size(ptf5)
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBHDR(:,1)) = ', size(ROBHDR(:,1))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBHDR(1,:)) = ', size(ROBHDR(1,:))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBHDR     ) = ', size(ROBHDR)
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBDDR(:,1)) = ', size(ROBDDR(:,1))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBDDR(1,:)) = ', size(ROBDDR(1,:))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBDDR     ) = ', size(ROBDDR)
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBSU(:,1))  = ', size(ROBSU(:,1))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBSU(1,:))  = ', size(ROBSU(1,:))
    WRITE (NULOUT,*) ' aeolus_getamd: size(ROBSU     )  = ', size(ROBSU)
    WRITE (NULOUT,*) ' aeolus_getamd: size(SATHDR(:,1)) = ', size(SATHDR(:,1))
    WRITE (NULOUT,*) ' aeolus_getamd: size(SATHDR(1,:)) = ', size(SATHDR(1,:))
    WRITE (NULOUT,*) ' aeolus_getamd: size(SATHDR     ) = ', size(SATHDR)
    CALL FLUSH(NULOUT)

  i_num_auxmet=NFLEVG+1
  JMASTER = 0
  OBS_LOOP: DO JOBS = 1, KLEN

  ! 2.1 Cycle non-master/non-predicted observations, otherwise set array indices

       IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) /= ISONDE_TYPE_BIN01_L1B ) THEN
             WRITE (NULOUT,'(A,2I21)') &
& ' Aeolus_getamd: non-master obs in layer_loop for jobs/sonde_type =           :', &
& jobs, ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR)
          CALL FLUSH(NULOUT)
          CYCLE OBS_LOOP
       ENDIF

       JMASTER = JMASTER + 1

       IF( ROBSU(JMASTER,MDB_AEO_HDRFLAG_AT_AEOLUS_HDR) > IHDRFLAG_ACTUAL ) THEN
          WRITE (NULOUT,'(A,3I21)') &
& ' Aeolus_getamd: debug11a: predicted and/or nadir location in layer_loop for jobs/jmaster/hdrflag =           :', &
& jobs, JMASTER, ROBSU(JMASTER,MDB_AEO_HDRFLAG_AT_AEOLUS_HDR)
          CALL FLUSH(NULOUT)
       ENDIF

       ! TODO: consider the following use of aeolus_hdr.offset@sat
       !i_aeolus_hdr = ( JOBS - 1 ) / 288 ! Integer division ! Offset
       !i_aeolus_hdr = ROBDDR(JOBS, MLNK_SAT2AEOLUS_HDR(1)) ! Offset
       !
       i_sat        = ROBHDR(JOBS,  MLNK_HDR2SAT(1))        ! Offset
       i_sat        = i_sat + 1

       IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == ISONDE_TYPE_BIN01_L1B )THEN
          WRITE (NULOUT,'(A,2I11)') &
& ' Aeolus_getamd: layer1 debug jobs, i_sat(unused) =           :', &
& jobs, i_sat
          CALL FLUSH(NULOUT)
       ENDIF

       ! TODO: re-visit the following calculations
       !i_sat        = JOBS
       !i_aeolus_hdr = ROBDDR(i_sat, MLNK_SAT2AEOLUS_HDR(1)) ! Offset
       !i_aeolus_hdr = i_aeolus_hdr + 1
       !JAUXMET      = ROBSU(i_aeolus_hdr,MLNK_AEOLUS_HDR2AEOLUS_AUXMET(1)) ! Offset
       !JAUXMET      = ROBSU(JMASTER,MLNK_AEOLUS_HDR2AEOLUS_AUXMET(1)) ! Offset
       !Debug-20080331 JAUXMET      = ( JOBS - 1 ) * i_num_auxmet ! Offset test

  ! 2.2 Write Aux Met Data to ODB tables for model levels
    LEVEL_LOOP: DO ILEV = 1, NFLEVG

      IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == ISONDE_TYPE_BIN01_L1B )THEN
       WRITE (NULOUT,'(A,I11,I11)') &
& ' Aeolus_getamd: filling   layer data for jobs, ilev =     :', &
& jobs, ilev
       CALL FLUSH(NULOUT)
      ENDIF

         JAUXMET = ( JMASTER - 1 ) * i_num_auxmet ! Offset
         JAUXMET =   JAUXMET + ILEV

         SATHDR(JAUXMET, MDB_LEV_AT_AEOLUS_AUXMET )     = ILEV

         SATHDR(JAUXMET, MDB_PTOP_AT_AEOLUS_AUXMET)     = PPRESH5(JOBS,ILEV-1)
         SATHDR(JAUXMET, MDB_PNOM_AT_AEOLUS_AUXMET)     = PPRESF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_ZTOP_AT_AEOLUS_AUXMET)     = PGEOPH5(JOBS,ILEV-1)
         SATHDR(JAUXMET, MDB_ZNOM_AT_AEOLUS_AUXMET)     = PGEOPF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_U_AT_AEOLUS_AUXMET)        = RMDI ! PUF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_V_AT_AEOLUS_AUXMET)        = RMDI ! PVF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_T_AT_AEOLUS_AUXMET)        = PTF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_RH_AT_AEOLUS_AUXMET)       = PRHF5( JOBS, ILEV )
         SATHDR(JAUXMET, MDB_Q_AT_AEOLUS_AUXMET)        = PQF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_CC_AT_AEOLUS_AUXMET)       = PCCF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_CLWC_AT_AEOLUS_AUXMET)     = PCLWF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_CIWC_AT_AEOLUS_AUXMET)     = PCIWF5(JOBS,ILEV)
         SATHDR(JAUXMET, MDB_ERROR_T_AT_AEOLUS_AUXMET)  = 0.8_JPRB
         SATHDR(JAUXMET, MDB_ERROR_RH_AT_AEOLUS_AUXMET) = 0.4_JPRB
         SATHDR(JAUXMET, MDB_ERROR_P_AT_AEOLUS_AUXMET)  = RMDI ! not filled in away from surface
    ENDDO LEVEL_LOOP


  ! 2.2 Write Aux Met Data to ODB tables for model surface
    ILEV = NFLEVG+1

      IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) /= ISONDE_TYPE_BIN01_L1B )THEN
          WRITE (NULOUT,'(A,2I21)') &
& ' Aeolus_getamd: non-master obs in sfc_loop for jobs/sonde_type =           :', &
& jobs, ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR)
          CALL FLUSH(NULOUT)
          CYCLE OBS_LOOP
      ENDIF

      IF( ROBSU(JMASTER,MDB_AEO_HDRFLAG_AT_AEOLUS_HDR) > IHDRFLAG_ACTUAL ) THEN
             WRITE (NULOUT,'(A,3I21)') &
& ' Aeolus_getamd: debug11b: predicted location in sfc_loop for jobs/jmaster/hdrflag =           :', &
& jobs, jmaster, ROBSU(JMASTER,MDB_AEO_HDRFLAG_AT_AEOLUS_HDR)
             CALL FLUSH(NULOUT)
      ELSE
       WRITE (NULOUT,'(A,I11)') &
& ' Aeolus_getamd: filling surface data for jobs =           :', &
& jobs
       CALL FLUSH(NULOUT)
      ENDIF

      IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == ISONDE_TYPE_BIN01_L1B )THEN
       WRITE (NULOUT,'(A,2I11)') &
& ' Aeolus_getamd: sfc debug jobs, i_sat(unused) =           :', &
& jobs, i_sat
       CALL FLUSH(NULOUT)
      ENDIF

       i_sat        = JOBS
       !
       i_aeolus_hdr = ROBDDR(i_sat, MLNK_SAT2AEOLUS_HDR(1)) ! Offset
       i_aeolus_hdr = i_aeolus_hdr + 1
       !When sql did not have WHERE (sonde_type@hdr==1):
       !JAUXMET      = ROBSU(i_aeolus_hdr,MLNK_AEOLUS_HDR2AEOLUS_AUXMET(1)) ! Offset
       JAUXMET      = ROBSU(JMASTER,MLNK_AEOLUS_HDR2AEOLUS_AUXMET(1)) ! Offset
       IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == ISONDE_TYPE_BIN01_L1B )THEN
         WRITE (NULOUT,'(A,4I11)') &
& ' Aeolus_getamd: sfc indices jobs, i_sat, i_aeolus_hdr, robsu/auxmet_offset =           :', &
& jobs, i_sat, i_aeolus_hdr, JAUXMET
         CALL FLUSH(NULOUT)
       ENDIF

       !JAUXMET      = 0 ! Offset test
       !Debug-20080331 JAUXMET      = ( JOBS - 1 ) * i_num_auxmet ! Offset test
       JAUXMET      = ( JMASTER - 1 ) * i_num_auxmet ! Offset test
       JAUXMET      = JAUXMET + ILEV
       IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == ISONDE_TYPE_BIN01_L1B )THEN
         WRITE (NULOUT,'(A,4I11)') &
& ' Aeolus_getamd: sfc indices jobs, i_sat, i_aeolus_hdr, JAUXMET =           :', &
& jobs, i_sat, i_aeolus_hdr, JAUXMET
         CALL FLUSH(NULOUT)
       ENDIF

        SATHDR(JAUXMET, MDB_LEV_AT_AEOLUS_AUXMET )     = 0
        SATHDR(JAUXMET, MDB_PTOP_AT_AEOLUS_AUXMET )    = PPRESH5(JOBS,NFLEVG)
        SATHDR(JAUXMET, MDB_PNOM_AT_AEOLUS_AUXMET )    = PPRESH5(JOBS,NFLEVG)
        SATHDR(JAUXMET, MDB_ZTOP_AT_AEOLUS_AUXMET )    = PGEOPH5(JOBS,NFLEVG)
        SATHDR(JAUXMET, MDB_ZNOM_AT_AEOLUS_AUXMET )    = PGEOPH5(JOBS,NFLEVG)
        SATHDR(JAUXMET, MDB_U_AT_AEOLUS_AUXMET )       = PUV10M5(JOBS,1)
        SATHDR(JAUXMET, MDB_V_AT_AEOLUS_AUXMET )       = PUV10M5(JOBS,2)
        SATHDR(JAUXMET, MDB_T_AT_AEOLUS_AUXMET)        = RMDI
        SATHDR(JAUXMET, MDB_RH_AT_AEOLUS_AUXMET)       = RMDI
        SATHDR(JAUXMET, MDB_Q_AT_AEOLUS_AUXMET)        = RMDI
        SATHDR(JAUXMET, MDB_CC_AT_AEOLUS_AUXMET)       = RMDI
        SATHDR(JAUXMET, MDB_CLWC_AT_AEOLUS_AUXMET)     = RMDI
        SATHDR(JAUXMET, MDB_CIWC_AT_AEOLUS_AUXMET)     = RMDI
        SATHDR(JAUXMET, MDB_ERROR_T_AT_AEOLUS_AUXMET)  = RMDI
        SATHDR(JAUXMET, MDB_ERROR_RH_AT_AEOLUS_AUXMET) = RMDI
        SATHDR(JAUXMET, MDB_ERROR_P_AT_AEOLUS_AUXMET)  = 0.5_JPRB ! Only do for surface pressure

  ENDDO OBS_LOOP

  !  #]

  !  #[
       
  ! 3.0 Closing

  IF (LHOOK) CALL DR_HOOK('AEOLUS_GETAMD_MOD:AEOLUS_GETAMD',1,ZHOOK_HANDLE)

  !  #]
END SUBROUTINE AEOLUS_GETAMD
END MODULE AEOLUS_GETAMD_MOD
