MODULE AEOLUS_L2BP_WRAPPER_MOD
CONTAINS
SUBROUTINE AEOLUS_L2BP_WRAPPER(KLEN) ! Placeholder for future development
!  #[ 

!     Purpose.
!     --------
!       Contains IFS-Aeolus interface routines

!     Author.
!     -------
!        David Tan *ECMWF*

!     Modifications.
!     --------------
!     2007-Mar-23   David Tan         Original
!     2007-May-30   David Tan         Lower rejection flag
!     2007-Jun-01   David Tan         Coding norm compliance
!     2007-Jun-12   David Tan         Set vertco_type@hdr
!     2007-Sep-25   David Tan         Make zhook_handle local to each routine
!     2007-Nov-04   T.Wilhelmsson
!                   O.Spaniel       removing of severe breaks of coding norms
!     2007-Nov-27   David Tan         Fill/use aeolus_2b table
!     2007-Dec-05   David Tan         Aeolus_l2bp_wrapper: new looping/indexing
!     2008-Jan-24   David Tan         New dummy L2B retrievals
!     2008-Jan-25   David Tan         Disable ilen_step_l1b.
!     2008-Jan-31   David Tan         Use aeolus_hdr.offset@sat
!     2008-Feb-14   David Tan         Suspend use of aeolus offsets.
!     2008-Mar-06   David Tan         Transfer from aeolus_2b table to body table
!                                       moved to pp_obs/aeolus_l2b_to_body.F90
!     2008-Apr-01   David Tan         Use JMASTER in aeolus offsets.
!     2008-Aug-11   Tan/Poli          Interface to aeolus project.
!     2008-Sep-09   Tan               Pass SATPRED for aeolus_1b data
!     2008-Sep-10   D. Tan            Pass MDB_*_AT_AEOLUS_1B
!     2008-Sep-12   D. Tan            Pass MDBxxx (xxx@hdr)
!     2008-Sep-17   D. Tan            Upgrade Mie_ref_pulse/meas_data to 20 cols
!     2008-Oct-01   D. Tan            Cycle predicted locations
!     2008-Nov-04   D. Tan            Pass MDB_*_AT_AEOLUS_HDR
!     2008-Nov-06   D. Tan            Pass MDB_ for dem_int_alt and 3 SNRs
!     2009-Feb-16   D. Tan            Reduce continuation lines

!     ------------------------------------------------------------------
USE PARKIND1 , ONLY : JPIS, JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE YOMNMCOD , ONLY : NHEIGVC ! For debug prints only
USE YOMLUN   , ONLY : NULOUT, NULERR
USE YOMDB
USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG
USE YOMANCS  , ONLY : RMDI

!     -------------------------------------------------------------------------
! Type and variable definitions
IMPLICIT NONE

INTERFACE
subroutine aeolus_l2bp_primary_ec(JOBS,JMASTER, &
&                                 NFLEVG, RG, RMDI, &
&                                 ROBHDR, &
&                                 aeolus_hdr_in_odb, &
&                                 l1b_in_odb, amd_in_odb, l2b_in_odb, &
!                                 MDB_LIST,&
& MDB_TSLOT_AT_INDEX , MDBDAT             , MDBETM     , MDBLAT     , MDBLON     , &

& mdb_geoid_seprtn_at_aeolus_hdr , mdb_mie_gwdhls_at_aeolus_hdr   , mdb_mie_gwdhlswt_at_aeolus_hdr , &
& mdb_ray_gwdhls_at_aeolus_hdr   , mdb_ray_gwdhlswt_at_aeolus_hdr , &
& mdb_laser_wavlen_at_aeolus_hdr , mdb_mie_errquant_at_aeolus_hdr , mdb_ray_errquant_at_aeolus_hdr , &
& mdb_tripod_corrn_at_aeolus_hdr , mdb_mie_msmnsens_at_aeolus_hdr , mdb_mie_mszrfreq_at_aeolus_hdr , &
& mdb_mie_msrspesd_at_aeolus_hdr , mdb_mie_rpmnsens_at_aeolus_hdr , &
& mdb_ray_msmnsens_at_aeolus_hdr , mdb_ray_mszrfreq_at_aeolus_hdr , &
& mdb_ray_msrspesd_at_aeolus_hdr , mdb_ray_rpmnsens_at_aeolus_hdr , &
& mdb_dem_int_alt_at_aeolus_hdr  , &

& mdb_azimuth_at_aeolus_1b        , mdb_channel_at_aeolus_1b        , &
& mdb_aeolus_bin1b_at_aeolus_1b   , mdb_aeolus_meas_at_aeolus_1b    , &
& mdb_lat_at_aeolus_1b            , mdb_lon_at_aeolus_1b            , &
& mdb_altitude_at_aeolus_1b       , mdb_aocs_los_veloc_at_aeolus_1b , &
& mdb_scat_ratio_at_aeolus_1b     , mdb_mie_ref_pulse_at_aeolus_1b  , & ! Array of 20 elements
& mdb_raya_refpulse_at_aeolus_1b  , mdb_rayb_refpulse_at_aeolus_1b  , &
& mdb_mie_meas_data_at_aeolus_1b  , & ! Array of 20 elements
& mdb_mie_usefulsig_at_aeolus_1b  , mdb_raya_usefulsig_at_aeolus_1b , mdb_rayb_usefulsig_at_aeolus_1b , &
& mdb_mie_snr_at_aeolus_1b        , &
& mdb_raya_snr_at_aeolus_1b       , mdb_rayb_snr_at_aeolus_1b       , &

& mdb_lat_at_aeolus_auxmet     , mdb_lon_at_aeolus_auxmet     , &
& mdb_lev_at_aeolus_auxmet     , mdb_ptop_at_aeolus_auxmet    , mdb_pnom_at_aeolus_auxmet    , &
& mdb_ztop_at_aeolus_auxmet    , mdb_znom_at_aeolus_auxmet    , &
& mdb_u_at_aeolus_auxmet       , mdb_v_at_aeolus_auxmet       , &
& mdb_t_at_aeolus_auxmet       , mdb_rh_at_aeolus_auxmet      , &
& mdb_q_at_aeolus_auxmet       , mdb_cc_at_aeolus_auxmet      , &
& mdb_clwc_at_aeolus_auxmet    , mdb_ciwc_at_aeolus_auxmet    , &
& mdb_error_t_at_aeolus_auxmet , mdb_error_rh_at_aeolus_auxmet, &
& mdb_error_p_at_aeolus_auxmet , &
& error_flag)
  !  #[

! Description:
! to interface ECMWF IFS variables to aeolus_l2bp_primary

! Method
! see: Aeolus L2BP software documentation (subroutine version?)

! Current Code Owner: ECMWF/Aeolus L2BP Project

!  Written by:  David Tan, ECMWF
!  Created:     06-Aug-2007

!  Modifications:
!     06-Aug-2007 D. Tan    Original stub.
!     30-Jan-2008 D. Tan    Remove integer kind i_
!     11-Aug-2008 Tan/Poli  Interface to IFS
!     10-Sep-2008 D. Tan    Pass MDB_*_AT_AEOLUS_1B
!     17-Sep-2008 D. Tan    Upgrade Mie_ref_pulse/meas_data to 20 cols


! Code Description:
!   Language:          Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
! Declarations:
! Modules used:

!    Declarations from Aeolus project
  USE YOMLUN_IFSAUX,    ONLY : NULOUT, NULERR

!   Declarations from ECMWF IFS/IFSAUX
  Use parkind1, Only : JPIM, JPRB
  INTEGER(KIND=JPIM),           intent(in) :: JOBS, JMASTER
  INTEGER(KIND=JPIM),           intent(in)  :: NFLEVG
  REAL(KIND=JPRB),              intent(in)  :: RG
  REAL(KIND=JPRB),              intent(in)  :: RMDI

! ODB Arrays as defined in ifs/common/yomdb_vars.h
!REAL(KIND=JPRB), intent( in), pointer :: robhdr(:,:)
!REAL(KIND=JPRB), intent( in), pointer :: l1b_in_odb(:,:), amd_in_odb(:,:)
!REAL(KIND=JPRB), intent(out), pointer :: l2b_in_odb(:,:)
REAL(KIND=JPRB), pointer :: robhdr(:,:)
REAL(KIND=JPRB), pointer :: aeolus_hdr_in_odb(:,:)
REAL(KIND=JPRB), pointer :: l1b_in_odb(:,:), amd_in_odb(:,:)
REAL(KIND=JPRB), pointer :: l2b_in_odb(:,:)

! Pointers to columns in ROBHDR
INTEGER(KIND=JPIM), intent(in) :: MDB_TSLOT_AT_INDEX !  'tslot@index'
INTEGER(KIND=JPIM), intent(in) :: MDBDAT             !  ' date@hdr'
INTEGER(KIND=JPIM), intent(in) :: MDBETM             !  ' time@hdr'
INTEGER(KIND=JPIM), intent(in) :: MDBLAT             !  '  lat@hdr'
INTEGER(KIND=JPIM), intent(in) :: MDBLON             !  '  lon@hdr'

! Pointers to columns in aeolus_hdr_in_odb
INTEGER(KIND=JPIM), intent(in) :: mdb_geoid_seprtn_at_aeolus_hdr     !  'geoid_seprtn@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_gwdhls_at_aeolus_hdr       !  'mie_gwdhls@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_gwdhlswt_at_aeolus_hdr     !  'mie_gwdhlswt@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_gwdhls_at_aeolus_hdr       !  'ray_gwdhls@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_gwdhlswt_at_aeolus_hdr     !  'ray_gwdhlswt@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_laser_wavlen_at_aeolus_hdr     !  'laser_wavlen@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_errquant_at_aeolus_hdr(3)  !  'mie_errquant@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_errquant_at_aeolus_hdr(4)  !  'ray_errquant@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_tripod_corrn_at_aeolus_hdr(16) !  'tripod_corrn@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_msmnsens_at_aeolus_hdr     !  'mie_msmnsens@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_mszrfreq_at_aeolus_hdr     !  'mie_mszrfreq@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_msrspesd_at_aeolus_hdr     !  'mie_msrspesd@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_rpmnsens_at_aeolus_hdr     !  'mie_rpmnsens@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_msmnsens_at_aeolus_hdr     !  'ray_msmnsens@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_mszrfreq_at_aeolus_hdr     !  'ray_mszrfreq@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_msrspesd_at_aeolus_hdr     !  'ray_msrspesd@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_ray_rpmnsens_at_aeolus_hdr     !  'ray_rpmnsens@aeolus_hdr'
INTEGER(KIND=JPIM), intent(in) :: mdb_dem_int_alt_at_aeolus_hdr      !  'dem_int_alt@aeolus_hdr'

! Pointers to columns in l1b_in_odb
INTEGER(KIND=JPIM), intent(in) :: mdb_azimuth_at_aeolus_1b       
INTEGER(KIND=JPIM), intent(in) :: mdb_channel_at_aeolus_1b       
INTEGER(KIND=JPIM), intent(in) :: mdb_aeolus_bin1b_at_aeolus_1b  
INTEGER(KIND=JPIM), intent(in) :: mdb_aeolus_meas_at_aeolus_1b   
INTEGER(KIND=JPIM), intent(in) :: mdb_lat_at_aeolus_1b           
INTEGER(KIND=JPIM), intent(in) :: mdb_lon_at_aeolus_1b           
INTEGER(KIND=JPIM), intent(in) :: mdb_altitude_at_aeolus_1b      
INTEGER(KIND=JPIM), intent(in) :: mdb_aocs_los_veloc_at_aeolus_1b
INTEGER(KIND=JPIM), intent(in) :: mdb_scat_ratio_at_aeolus_1b    
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_ref_pulse_at_aeolus_1b(20)
INTEGER(KIND=JPIM), intent(in) :: mdb_raya_refpulse_at_aeolus_1b 
INTEGER(KIND=JPIM), intent(in) :: mdb_rayb_refpulse_at_aeolus_1b 
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_meas_data_at_aeolus_1b(20)
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_usefulsig_at_aeolus_1b 
INTEGER(KIND=JPIM), intent(in) :: mdb_raya_usefulsig_at_aeolus_1b
INTEGER(KIND=JPIM), intent(in) :: mdb_rayb_usefulsig_at_aeolus_1b
INTEGER(KIND=JPIM), intent(in) :: mdb_mie_snr_at_aeolus_1b 
INTEGER(KIND=JPIM), intent(in) :: mdb_raya_snr_at_aeolus_1b
INTEGER(KIND=JPIM), intent(in) :: mdb_rayb_snr_at_aeolus_1b

! Pointers to columns in amd_in_odb
INTEGER(KIND=JPIM), intent(in) :: mdb_lat_at_aeolus_auxmet     !  'lat@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_lon_at_aeolus_auxmet     !  'lon@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_lev_at_aeolus_auxmet     !  'lev@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_ptop_at_aeolus_auxmet    !  'ptop@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_pnom_at_aeolus_auxmet    !  'pnom@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_ztop_at_aeolus_auxmet    !  'ztop@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_znom_at_aeolus_auxmet    !  'znom@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_u_at_aeolus_auxmet       !  'u@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_v_at_aeolus_auxmet       !  'v@aeolus_auxmet'

INTEGER(KIND=JPIM), intent(in) :: mdb_t_at_aeolus_auxmet       !  't@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_rh_at_aeolus_auxmet      !  'rh@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_q_at_aeolus_auxmet       !  'q@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_cc_at_aeolus_auxmet      !  'cc@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_clwc_at_aeolus_auxmet    !  'clwc@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_ciwc_at_aeolus_auxmet    !  'ciwc@aeolus_auxmet'

INTEGER(KIND=JPIM), intent(in) :: mdb_error_t_at_aeolus_auxmet       !  'error_t@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_error_rh_at_aeolus_auxmet      !  'error_rh@aeolus_auxmet'
INTEGER(KIND=JPIM), intent(in) :: mdb_error_p_at_aeolus_auxmet       !  'error_p@aeolus_auxmet'

!  character(len=maxlen_IntAuc), intent(in)  :: str_value
!  integer(IntAuc),              intent(out) :: value
  integer,                      intent(out) :: error_flag

end subroutine aeolus_l2bp_primary_ec

END INTERFACE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 

! local variables           

REAL(KIND=JPRB)    :: ZHOOK_HANDLE
INTEGER(KIND=JPIM) :: imax_num_l2b_obs_classes ! TODO: make this a parameter?
INTEGER(KIND=JPIM) :: istep_l2b_obs_classes    ! TODO: make this a parameter?
INTEGER(KIND=JPIM) :: inum_l2b_obs_classes ! Might vary for each L1B obs
INTEGER(KIND=JPIM) :: JOBS, i_l2b     ! loops
INTEGER(KIND=JPIM) :: i_l2b_channel    ! loops
INTEGER(KIND=JPIM) :: i_l2b_rangebin   ! loops
INTEGER(KIND=JPIM) :: i_aeolus_2b ! indices for ODB arrays
INTEGER(KIND=JPIM) :: i_aeolus_2b_offset ! indices for ODB arrays
INTEGER(KIND=JPIM) :: i_aeolus_hdr ! indices for ODB arrays
INTEGER(KIND=JPIM) :: i_sat        ! indices for ODB arrays
INTEGER(KIND=JPIM) :: JMASTER

! Other local variables
INTEGER(KIND=JPIM), PARAMETER :: JP_NUM_AEOLUS_RANGEBINS=24
INTEGER(KIND=JPIM), PARAMETER :: JP_SONDE_TYPE_BIN01_L1B=1
INTEGER(KIND=JPIM), PARAMETER :: JP_HDRFLAG_PREDICTED=2

! Aeolus variables
!    character(len=maxlen_IntAuc) :: str_value
!    integer(IntAuc)              :: value
!    integer(i_)                  :: error_flag
    character(len=4)   :: cl_str_value
    integer(kind=jpim) :: i_error_flag

! Dummy outputs from Aeolus L2B processor
REAL(KIND=JPRB)    :: zdummy_cog_lat, zdummy_cog_lon, zdummy_zbase, zdummy_ztop, zdummy_hlos
REAL(KIND=JPRB)    :: zdummy_zmid

!#include "fcobs.h"
#include "openmp_obs.h"

  !  #[
       
  ! 1.0 Preliminaries

  IF (LHOOK) CALL DR_HOOK('AEOLUS_L2BP_WRAPPER_MOD:AEOLUS_L2BP_WRAPPER',0,ZHOOK_HANDLE)

  imax_num_l2b_obs_classes = 5
  istep_l2b_obs_classes    = imax_num_l2b_obs_classes + 1 ! allows for L1B rows (hlos_method=0)
  JMASTER = 0

  !  #]

  !  #[
       
  ! 2.0 Interface to Aeolus Level-2B processor

    WRITE (NULOUT,*) ' aeolus_l2bp_wrapper: not yet implemented'

    ! 2.1 Aeolus setup

    ! Small test

    i_error_flag = 0 ! no_error

    cl_str_value='+255'
    !call GetIntAuc(str_value,value,error_flag)

    !call aeolus_l2bp_setup()

    ! loop through each observation but only generate Level-2B retrievals when Obs=L1b-Bin01

    OBS_LOOP: DO JOBS = 1, klen

      IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) /= JP_SONDE_TYPE_BIN01_L1B ) CYCLE OBS_LOOP
      JMASTER = JMASTER + 1

       IF( ROBSU(JMASTER,MDB_AEO_HDRFLAG_AT_AEOLUS_HDR) >= JP_HDRFLAG_PREDICTED ) THEN
          WRITE (NULOUT,'(A,3I21)') &
& ' Aeolus_l2bp_wrapper_mod: debug12: predicted location in obs_loop for jobs/jmaster/hdrflag =           :', &
& jobs, JMASTER, ROBSU(JMASTER,MDB_AEO_HDRFLAG_AT_AEOLUS_HDR)
          CALL FLUSH(NULOUT)
          CYCLE OBS_LOOP
       ENDIF

      ! For each master L1B observation in ODB, perform L2B retrievals and store in ODB
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBHDR)      = ', size(ROBHDR)
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBHDR(:,1)) = ', size(ROBHDR(:,1))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBHDR(1,:)) = ', size(ROBHDR(1,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBHDR(:,:)) = ', size(ROBHDR(:,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBDDR)      = ', size(ROBDDR)
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBDDR(:,1)) = ', size(ROBDDR(:,1))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBDDR(1,:)) = ', size(ROBDDR(1,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBDDR(:,:)) = ', size(ROBDDR(:,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBSU)      = ', size(ROBSU)
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBSU(:,1)) = ', size(ROBSU(:,1))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBSU(1,:)) = ', size(ROBSU(1,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(ROBSU(:,:)) = ', size(ROBSU(:,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATPRED)      = ', size(SATPRED)
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATPRED(:,1)) = ', size(SATPRED(:,1))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATPRED(1,:)) = ', size(SATPRED(1,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATPRED(:,:)) = ', size(SATPRED(:,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATBODY)      = ', size(SATBODY)
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATBODY(:,1)) = ', size(SATBODY(:,1))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATBODY(1,:)) = ', size(SATBODY(1,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATBODY(:,:)) = ', size(SATBODY(:,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATHDR)      = ', size(SATHDR)
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATHDR(:,1)) = ', size(SATHDR(:,1))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATHDR(1,:)) = ', size(SATHDR(1,:))
     WRITE(NULERR,*) 'dt-debug: aeolus_l2bp_wrapper has size(SATHDR(:,:)) = ', size(SATHDR(:,:))
      call aeolus_l2bp_primary_ec(JOBS,JMASTER, &
&                                 NFLEVG, RG, RMDI, &
&                                 ROBHDR, &
!                                 ROBDDR, & ! sathdr_screen_aeolus_sat
&                                 ROBSU,  & ! sathdr_screen_aeolus_hdr
&                                 SATPRED, SATHDR, SATBODY, &
!                                 MDB_LIST,&
& MDB_TSLOT_AT_INDEX , MDBDAT             , MDBETM     , MDBLAT     , MDBLON     , &

& MDB_GEOID_SEPRTN_AT_AEOLUS_HDR , MDB_MIE_GWDHLS_AT_AEOLUS_HDR   , MDB_MIE_GWDHLSWT_AT_AEOLUS_HDR , &
& MDB_RAY_GWDHLS_AT_AEOLUS_HDR   , MDB_RAY_GWDHLSWT_AT_AEOLUS_HDR , &
& MDB_LASER_WAVLEN_AT_AEOLUS_HDR , MDB_MIE_ERRQUANT_AT_AEOLUS_HDR , MDB_RAY_ERRQUANT_AT_AEOLUS_HDR , &
& MDB_TRIPOD_CORRN_AT_AEOLUS_HDR , MDB_MIE_MSMNSENS_AT_AEOLUS_HDR , MDB_MIE_MSZRFREQ_AT_AEOLUS_HDR , &
& MDB_MIE_MSRSPESD_AT_AEOLUS_HDR , MDB_MIE_RPMNSENS_AT_AEOLUS_HDR , &
& MDB_RAY_MSMNSENS_AT_AEOLUS_HDR , MDB_RAY_MSZRFREQ_AT_AEOLUS_HDR , &
& MDB_RAY_MSRSPESD_AT_AEOLUS_HDR , MDB_RAY_RPMNSENS_AT_AEOLUS_HDR , &
& MDB_DEM_INT_ALT_AT_AEOLUS_HDR  , &

& MDB_AZIMUTH_AT_AEOLUS_1B        , MDB_CHANNEL_AT_AEOLUS_1B        , &
& MDB_AEOLUS_BIN1B_AT_AEOLUS_1B   , MDB_AEOLUS_MEAS_AT_AEOLUS_1B    , &
& MDB_LAT_AT_AEOLUS_1B            , MDB_LON_AT_AEOLUS_1B            , &
& MDB_ALTITUDE_AT_AEOLUS_1B       , MDB_AOCS_LOS_VELOC_AT_AEOLUS_1B , &
& MDB_SCAT_RATIO_AT_AEOLUS_1B     , MDB_MIE_REF_PULSE_AT_AEOLUS_1B  , & ! Array of 20 elements
& MDB_RAYA_REFPULSE_AT_AEOLUS_1B  , MDB_RAYB_REFPULSE_AT_AEOLUS_1B  , &
& MDB_MIE_MEAS_DATA_AT_AEOLUS_1B  , & ! Array of 20 elements
& MDB_MIE_USEFULSIG_AT_AEOLUS_1B  , MDB_RAYA_USEFULSIG_AT_AEOLUS_1B , MDB_RAYB_USEFULSIG_AT_AEOLUS_1B , &
& MDB_MIE_SNR_AT_AEOLUS_1B        , MDB_RAYA_SNR_AT_AEOLUS_1B       , MDB_RAYB_SNR_AT_AEOLUS_1B       , &

& MDB_LAT_AT_AEOLUS_AUXMET     , MDB_LON_AT_AEOLUS_AUXMET     , &
& MDB_LEV_AT_AEOLUS_AUXMET     , MDB_PTOP_AT_AEOLUS_AUXMET    , MDB_PNOM_AT_AEOLUS_AUXMET    , &
& MDB_ZTOP_AT_AEOLUS_AUXMET    , MDB_ZNOM_AT_AEOLUS_AUXMET    , &
& MDB_U_AT_AEOLUS_AUXMET       , MDB_V_AT_AEOLUS_AUXMET       , &
& MDB_T_AT_AEOLUS_AUXMET       , MDB_RH_AT_AEOLUS_AUXMET      , &
& MDB_Q_AT_AEOLUS_AUXMET       , MDB_CC_AT_AEOLUS_AUXMET      , &
& MDB_CLWC_AT_AEOLUS_AUXMET    , MDB_CIWC_AT_AEOLUS_AUXMET    , &
& MDB_ERROR_T_AT_AEOLUS_AUXMET , MDB_ERROR_RH_AT_AEOLUS_AUXMET, &
& MDB_ERROR_P_AT_AEOLUS_AUXMET , &
& i_error_flag)

      IF (i_error_flag /= 0) THEN
         write (NULOUT,*) 'aeolus_l2bp_wrapper: aeolus_l2bp_primary_ec i_error_flag = ', i_error_flag
         IF (LHOOK) CALL DR_HOOK('AEOLUS_L2BP_WRAPPER_MOD:AEOLUS_L2BP_WRAPPER',1,ZHOOK_HANDLE)
         RETURN
      ELSE
         write (NULOUT,*) 'aeolus_l2bp_wrapper: aeolus_l2bp_primary_ec returns ok'
      ENDIF


      WRITE (NULOUT,*) 'dt-debug: Vertical coord ROBODY(JOBS,MDBVCO), NHEIGVC = ',&
                      & ROBODY(JOBS,MDBVCO), NHEIGVC

      ! Fill ODB aeolus_2b table.
      ! This table is used in lag tasks to create L2B.EE files.
      ! The placeholders used represent output from a dummy L2B processor.
      inum_l2b_obs_classes = imax_num_l2b_obs_classes ! TODO: replace with
                                                    ! actual number for this L1B obs.
      aeolus_2b_channel_loop: DO i_l2b_channel = 0, 1 ! Rayleigh and Mie channels (or vice versa?)
       aeolus_2b_rangebin_loop: DO i_l2b_rangebin = 1, jp_num_aeolus_rangebins
        aeolus_2b_class_loop: DO i_l2b = 1, inum_l2b_obs_classes

         ! TODO: consider the following use of aeolus_hdr.offset@sat

         !i_sat               = ROBHDR(JOBS,  MLNK_HDR2SAT(1))        ! Offset
         !i_sat               = i_sat + 1
         i_sat = JOBS

         i_aeolus_hdr        = ROBDDR(i_sat, MLNK_SAT2AEOLUS_HDR(1)) ! Offset
         i_aeolus_hdr        = i_aeolus_hdr + 1

         !i_aeolus_2b_offset  = ROBSU(i_aeolus_hdr,MLNK_AEOLUS_HDR2AEOLUS_2B(1))
         !i_aeolus_2b_offset  = ( JOBS - 1 ) * ( 2 * jp_num_aeolus_rangebins * istep_l2b_obs_classes )! Offset test
         i_aeolus_2b_offset  = ( JMASTER - 1 ) * ( 2 * jp_num_aeolus_rangebins * istep_l2b_obs_classes )! Offset test

         IF( (i_l2b_channel == 0) .AND. (i_l2b_rangebin == 1) .AND. (i_l2b == 1) )THEN
          IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == JP_SONDE_TYPE_BIN01_L1B )THEN
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: i_aeolus_hdr, i_aeolus_2b_offset = ',&
                            &           i_aeolus_hdr, i_aeolus_2b_offset
            CALL FLUSH(NULOUT)
          ENDIF
         ENDIF

         i_aeolus_2b = i_aeolus_2b_offset &
                     & + ( i_l2b_channel * jp_num_aeolus_rangebins + i_l2b_rangebin -1 ) * &
                     &   istep_l2b_obs_classes &
                     & + 1 &   ! allows for L1B rows (i_l2b/hlos_method=0)
                     & + i_l2b ! Index in aeolus_2b table ! TODO: change when kset issue solved?
         IF ( i_aeolus_2b > size(SATBODY(:,1)) ) THEN
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: i_aeolus_2b exceeds size(satbody(:,1)) = ',&
                            & i_aeolus_2b, size(SATBODY(:,1))
            CALL FLUSH(NULOUT)
         ENDIF

         ! Sanity check of current values vs pre-filled values for channel, aeolus_bin2b and hlos_method
          IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == JP_SONDE_TYPE_BIN01_L1B )THEN
            WRITE (NULOUT,*) 'dt-debug: i_aeolus_2b, i_l2b = ',&
                            & i_aeolus_2b, i_l2b
            CALL FLUSH(NULOUT)
          ENDIF
         if (      ( i_l2b_channel  /= SATBODY(i_aeolus_2b, MDB_CHANNEL_AT_AEOLUS_2B)      ) &
            & .or. ( i_l2b_rangebin /= SATBODY(i_aeolus_2b, MDB_BIN2B_AT_AEOLUS_2B)        ) &
            & .or. ( i_l2b          /= SATBODY(i_aeolus_2b, MDB_HLOS_METHOD_AT_AEOLUS_2B)  ) ) then
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: WARNING: INDEX CONFLICT, OVERWRITING MAY OCCUR!!'
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: i_aeolus_2b, i_l2b = ',&
                            & i_aeolus_2b, i_l2b
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: channel     current/pre-filled = ',&
                            & i_l2b_channel, SATBODY(i_aeolus_2b, MDB_CHANNEL_AT_AEOLUS_2B )
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: bin2b       current/pre-filled = ',&
                            & i_l2b_rangebin, SATBODY(i_aeolus_2b, MDB_BIN2B_AT_AEOLUS_2B )
            WRITE (NULOUT,*) 'aeolus_l2bp_wrapper: hlos_method current/pre-filled = ',&
                            & i_l2b, SATBODY(i_aeolus_2b, MDB_HLOS_METHOD_AT_AEOLUS_2B )
            CALL FLUSH(NULOUT)
         else
          IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == JP_SONDE_TYPE_BIN01_L1B )THEN
            WRITE(NULOUT,*) 'dt-debug: aeolus_2b table entries match correctly'
            CALL FLUSH(NULOUT)
          ENDIF
         endif

       IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) == JP_SONDE_TYPE_BIN01_L1B )THEN
         ! Set some dummy values for outputs of L2B processor
         zdummy_cog_lat = ROBHDR(JOBS,MDBLAT) + i_l2b * 0.2_JPRB ! Latitude ! TODO: generalize placeholder
         zdummy_cog_lon = ROBHDR(JOBS,MDBLON) - i_l2b * 0.2_JPRB ! Longitude ! TODO: generalize placeholder

         zdummy_zmid    = ROBODY(JOBS,MDBPPP)
         zdummy_zmid    = zdummy_zmid + ( 1 - i_l2b_rangebin ) * 10000._JPRB ! Steps of 10000 m2/s2
         IF( i_l2b_rangebin > 16 )THEN
            zdummy_zmid = zdummy_zmid + ( i_l2b_rangebin - 16 ) * ( 10000._JPRB - 4000._JPRB ) ! Reduce steps to 4000 m2/s2
         ENDIF
         zdummy_zmid    = zdummy_zmid + i_l2b_channel * 100._JPRB - i_l2b * 10._JPRB

         zdummy_zbase   = zdummy_zmid - 5._JPRB ! zbase geopotential height (not pressure) ! TODO: generalize placeholder
         zdummy_ztop    = zdummy_zmid + 5._JPRB ! ztop  geopotential height (not pressure) ! TODO: generalize placeholder
         zdummy_hlos    = ROBODY(JOBS,MDBVAR) - i_l2b_rangebin * 1.0_JPRB &
                                            & + i_l2b          * 0.1_JPRB &
                                            & + i_l2b_channel * 0.01_JPRB ! HLOS ! TODO: generalize placeholder

         SATBODY(i_aeolus_2b, MDB_AZIMUTH_AT_AEOLUS_2B )      = 97. ! TODO: generalize placeholder
         !! SATBODY(i_aeolus_2b, MDB_CHANNEL_AT_AEOLUS_2B )      = i_l2b_channel  ! TODO: overwrite pre-filled value?
         !! SATBODY(i_aeolus_2b, MDB_BIN2B_AT_AEOLUS_2B )        = i_l2b_rangebin ! TODO: overwrite pre-filled value?
         !! SATBODY(i_aeolus_2b, MDB_HLOS_METHOD_AT_AEOLUS_2B )  = i_l2b          ! TODO: overwrite pre-filled value?
         SATBODY(i_aeolus_2b, MDB_COG_LAT_AT_AEOLUS_2B )      = zdummy_cog_lat
         SATBODY(i_aeolus_2b, MDB_COG_LON_AT_AEOLUS_2B )      = zdummy_cog_lon
         SATBODY(i_aeolus_2b, MDB_COG_ZBASE_AT_AEOLUS_2B )    = zdummy_zbase       ! TODO: factor of g
         SATBODY(i_aeolus_2b, MDB_COG_ZTOP_AT_AEOLUS_2B )     = zdummy_ztop        ! TODO: factor of g
         SATBODY(i_aeolus_2b, MDB_HLOS_AT_AEOLUS_2B )         = zdummy_hlos
         SATBODY(i_aeolus_2b, MDB_HLOS_ERR_AT_AEOLUS_2B )     = 15._JPRB ! HLOS_error ! TODO: generalize placeholder
       ENDIF

        ENDDO aeolus_2b_class_loop
       ENDDO aeolus_2b_rangebin_loop
      ENDDO aeolus_2b_channel_loop
    ENDDO OBS_LOOP ! JOBS

  !  #]

  !  #[
       
  ! 3.0 Closing

  IF (LHOOK) CALL DR_HOOK('AEOLUS_L2BP_WRAPPER_MOD:AEOLUS_L2BP_WRAPPER',1,ZHOOK_HANDLE)

  !  #]
END SUBROUTINE AEOLUS_L2BP_WRAPPER
END MODULE AEOLUS_L2BP_WRAPPER_MOD
