MODULE CONTROL_VECTORS_COMM_MOD

!   Author.
!   -------
!     Y. Tremolet

!   Modifications.
!   --------------
!     Original   02-10-28
!     C.Fischer  03-03-06  Aladin spectral treatment added.
!     M.Hamrud   01-Oct-2003 CY28 Cleaning
!     M.Fisher   17-Mar-2004 Wavelet Jb
!     Y.Tremolet 25-Jul-2004 Model Error and LINITCV
! ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0,  ONLY: NPROC, NPRTRV, NPRTRW, LELAM
USE YOMDIM,  ONLY: NFLEVL, NFLEVG, NPROMA
USE YOMMP ,  ONLY: MYPROC, NPTRLL, NUMLL
USE YOMDIMO, ONLY: NOBNTVG, NOBNTVBP
USE YOMTAG , ONLY: MT_DISTRIBUTED_VECTOR
USE YOMLUN,  ONLY: NULOUT, NULERR
USE SPECTRAL_FIELDS_MOD, ONLY : SPECTRAL_FIELD, DEALLOCATE_SPEC, INQ_SPEC
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR, ASSIGNMENT(=)
USE CONTROL_VECTORS_DATA_MIX, ONLY: NGPTOTL, NBLOCKSL, NSCALES,&
 & NVATOVV,NVATOVOFF,NLENTOVSCVP,NLENTOVSCVG
USE YOMWAVELET, ONLY: GRID_DEFINITION
USE MPL_MODULE, ONLY : MPL_SCATTERV, MPL_GATHERV, MPL_ALLGATHERV, MPL_ALLTOALLV, MPL_BROADCAST
USE YOMGEM, ONLY : RMUCEN   ! Only here to avoid dependency problem, nothing actually used
USE YOMTS, ONLY : NVATSOFF
USE YOMTVRAD, ONLY : NVATOVINDX

IMPLICIT NONE
PRIVATE
PUBLIC SCATTER_CTLVEC, MULTISCATTER_CTLVEC,&
     & GATHER_CTLVEC, ALLGATHER_CTLVEC, MULTIGATHER_CTLVEC,&
     & ALLGATHER_CVSECTION, SET_CVSECTION, GET_CVINDEX  

INTERFACE
#include "gath_grid.h"
#include "dist_grid.h"
END INTERFACE

#include "abor1.intfb.h"
#include "pe2set.intfb.h"
#include "set2pe.intfb.h"

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

SUBROUTINE SCATTER_CTLVEC(ydcv, kproc, pcv)

!     Purpose.
!     --------
!       Distribute control variable from one PE.

!     Author.
!     -------
!       Yannick Tremolet *ECMWF*

!     Modifications.
!     --------------
!       Original  : 02-08-07
! ----------------------------------------------

TYPE(control_vector), INTENT(INOUT) :: ydcv
INTEGER(KIND=JPIM), INTENT(IN) :: kproc
REAL(KIND=JPRB), INTENT(IN) :: pcv(:)

REAL(KIND=JPRB), ALLOCATABLE :: zbuf(:)
INTEGER(KIND=JPIM) :: ilens(NPROC),idisps(NPROC),ilenr,ileng,jp
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:SCATTER_CTLVEC',0,ZHOOK_HANDLE)

CALL GSTATS(254,0)
IF (myproc==kproc .AND. SIZE(pcv)/=ydcv%nsizeg) THEN
  WRITE (NULERR,*) 'SIZE(pcv)=',SIZE(pcv),' ydcv%nsizeg=',ydcv%nsizeg
  CALL ABOR1('SCATTER_CTLVEC: pcv wrong size')
ENDIF

! Set indices and allocate buffer
! -------------------------------
CALL get_cvlengths(ydcv, ilens)
idisps(1)=0
DO jp=2,NPROC
  idisps(jp)=idisps(jp-1)+ilens(jp-1)
ENDDO
ilenr=ydcv%nsizel

ileng=SUM(ilens(:))  ! ileng is NOT ydcv%nsizeg because LAM and PARAM var are duplicated.
!?cf I don't understand this comment for the LAM ?
IF (myproc==kproc) THEN
  ALLOCATE(zbuf(ileng))
ELSE
  ALLOCATE(zbuf(1))
ENDIF

! Pack data on root processor
! ---------------------------
CALL GSTATS(1846,0)
IF (myproc==kproc) CALL pack_cv(ydcv,ydcv%nsizeg,pcv,ileng,ilens,idisps,zbuf)
CALL GSTATS(1846,1)

! Exchange data
! -------------
CALL GSTATS_BARRIER(714)
CALL GSTATS(524,0)
CALL MPL_SCATTERV(precvbuf=ydcv%data,kroot=kproc,psendbuf=zbuf,&
 & ksendcounts=ilens,cdstring='CONTROL_VECTORS_COMM_MOD:SCATTER_CTLVEC')  
CALL GSTATS(524,1)
CALL GSTATS_BARRIER2(714)

DEALLOCATE(zbuf)
CALL GSTATS(254,1)

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:SCATTER_CTLVEC',1,ZHOOK_HANDLE)
END SUBROUTINE SCATTER_CTLVEC

! ------------------------------------------------------------------

SUBROUTINE MULTISCATTER_CTLVEC(ydcv, kproc, pcv, kvec_start, kvec_end)

!     Purpose.
!     --------
!       Distribute control variables from multiple PE.

!     Author.
!     -------
!       Tomas Wilhelmsson *ECMWF*

!     Modifications.
!     --------------
!       Original  : 09-08-28
! ----------------------------------------------

TYPE(control_vector), INTENT(INOUT) :: ydcv(kvec_start:kvec_end)
INTEGER(KIND=JPIM), INTENT(IN) :: kproc(:)
REAL(KIND=JPRB), INTENT(IN) :: pcv(:)
INTEGER(KIND=JPIM), INTENT(IN)   :: kvec_start, kvec_end 

REAL(KIND=JPRB), ALLOCATABLE :: zsendbuf(:),zrecvbuf(:)
INTEGER(KIND=JPIM) :: isendcounts(NPROC), irecvcounts(NPROC)
INTEGER(KIND=JPIM) :: ilens(NPROC,kvec_start:kvec_end),idisps(NPROC)
INTEGER(KIND=JPIM) :: ileng(kvec_start:kvec_end),jp,jj,j,icnt
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:MULTISCATTER_CTLVEC',0,ZHOOK_HANDLE)

CALL GSTATS(257,0)

DO jj = kvec_start, kvec_end
  IF (myproc==kproc(jj) .AND. SIZE(pcv)/=ydcv(jj)%nsizeg) THEN
    WRITE (NULERR,*) 'jj=',jj,'SIZE(pcv)=',SIZE(pcv),' ydcv(jj)%nsizeg=',ydcv(jj)%nsizeg
    CALL ABOR1('MULTISCATTER_CTLVEC: pcv wrong size')
  ENDIF

  ! Set indices and allocate sendbuffer
  ! -----------------------------------
  CALL get_cvlengths(ydcv(jj), ilens(:,jj))

  ileng(jj)=SUM(ilens(:,jj)) 

  IF (myproc==kproc(jj)) THEN
    ALLOCATE(zsendbuf(ileng(jj)))
    DO jp = 1, NPROC
       isendcounts(jp) = ilens(jp,jj)
    ENDDO

    idisps(1)=0
    DO jp=2,NPROC
       idisps(jp)=idisps(jp-1)+ilens(jp-1,jj)
    ENDDO
  
    ! Pack data on root processor
    ! ---------------------------
    CALL GSTATS(1847,0)
    CALL pack_cv(ydcv(jj),ydcv(jj)%nsizeg,pcv,ileng(jj),ilens(:,jj),idisps,zsendbuf)
    CALL GSTATS(1847,1)
  ENDIF
ENDDO

IF (.NOT. ALLOCATED(zsendbuf)) THEN
  ALLOCATE(zsendbuf(1))
  isendcounts(1:NPROC) = 0
ENDIF

! Allocate receive buffer
! -----------------------
ALLOCATE(zrecvbuf(SUM(ilens(myproc,:))))

irecvcounts(1:NPROC) = 0
DO jj = kvec_start, kvec_end
  irecvcounts(kproc(jj)) = SIZE(ydcv(jj)%data)
ENDDO

! Exchange data
! -------------
CALL GSTATS_BARRIER(705)
CALL GSTATS(525,0)
CALL MPL_ALLTOALLV(zsendbuf, isendcounts, zrecvbuf, irecvcounts,&
                 & CDSTRING='CONTROL_VECTORS_COMM_MOD:MULTISCATTER_CTLVEC')  
CALL GSTATS(525,1)
CALL GSTATS_BARRIER2(705)

icnt = 0
DO jj = kvec_start, kvec_end
  DO j=1, SIZE(ydcv(jj)%data)
    icnt = icnt + 1
    ydcv(jj)%data(j) = zrecvbuf(icnt)
  ENDDO
ENDDO

IF (SUM(ilens(myproc,:))/=icnt) CALL ABOR1('MULTISCATTER_CTLVEC: recvbuf wrong size')

DEALLOCATE(zsendbuf)
DEALLOCATE(zrecvbuf)
CALL GSTATS(257,1)

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:MULTISCATTER_CTLVEC',1,ZHOOK_HANDLE)
END SUBROUTINE MULTISCATTER_CTLVEC

! ------------------------------------------------------------------

SUBROUTINE GATHER_CTLVEC(ydcv, kproc, pcv)

!     Purpose.
!     --------
!       Gather control variable on one PE.

!     Author.
!     -------
!       Yannick Tremolet *ECMWF*

!     Modifications.
!     --------------
!       Original  : 02-08-07
! ----------------------------------------------

TYPE(control_vector), INTENT(IN) :: ydcv
INTEGER(KIND=JPIM), INTENT(IN) :: kproc
REAL(KIND=JPRB), INTENT(OUT) :: pcv(:)

REAL(KIND=JPRB), ALLOCATABLE :: zbuf(:)
INTEGER(KIND=JPIM) :: ilenr(NPROC),idispr(NPROC),ileng,jp
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:GATHER_CTLVEC',0,ZHOOK_HANDLE)

CALL GSTATS(253,0)
IF (myproc==kproc) THEN
  IF (SIZE(pcv)/=ydcv%nsizeg) CALL ABOR1('GATHER_CTLVEC: pcv wrong size')
ENDIF

! Allocate receive buffer
! -----------------------
CALL get_cvlengths(ydcv, ilenr)

idispr(1)=0
DO jp=2,NPROC
  idispr(jp)=idispr(jp-1)+ilenr(jp-1)
ENDDO

ileng=SUM(ilenr(:))
IF (myproc==kproc) THEN
  ALLOCATE(zbuf(ileng))
ELSE
  ALLOCATE(zbuf(1))
ENDIF

! Exchange data
! -------------
CALL GSTATS_BARRIER(716)
CALL GSTATS(528,0)
CALL MPL_GATHERV(PSENDBUF=ydcv%data, KROOT=kproc, PRECVBUF=zbuf, &
               & KRECVCOUNTS=ilenr, &
               & CDSTRING='CONTROL_VECTORS_COMM_MOD:GATHER_CTLVEC')  
CALL GSTATS(528,1)
CALL GSTATS_BARRIER2(716)

! Unpack data on root processor
! -----------------------------
IF (myproc==kproc) THEN
  CALL GSTATS(1857,0)
  CALL unpack_cv(ydcv,ileng,ilenr,idispr,zbuf,ydcv%nsizeg,pcv)
  CALL GSTATS(1857,1)
ENDIF
DEALLOCATE(zbuf)
CALL GSTATS(253,1)

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:GATHER_CTLVEC',1,ZHOOK_HANDLE)
END SUBROUTINE GATHER_CTLVEC

! ------------------------------------------------------------------

SUBROUTINE ALLGATHER_CTLVEC(ydcv, pcv)

TYPE(control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB), INTENT(OUT) :: pcv(:)

REAL(KIND=JPRB), ALLOCATABLE :: zbuf(:)
INTEGER(KIND=JPIM) :: ilenr(NPROC),idispr(NPROC),ileng,ilens,jp
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

! This routine is not used in 4DVAR.

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:ALLGATHER_CTLVEC',0,ZHOOK_HANDLE)

CALL GSTATS(251,0)
IF (SIZE(pcv)/=ydcv%nsizeg) CALL ABOR1('ALLGATHER_CTLVEC: pcv wrong size')

! Setup indices and buffers
! -------------------------
CALL get_cvlengths(ydcv,ilenr)

idispr(1)=0
DO jp=2,NPROC
  idispr(jp)=idispr(jp-1)+ilenr(jp-1)
ENDDO

ilens=ydcv%nsizel
ileng=SUM(ilenr(:))   ! ileng is NOT ydcv%nsizeg because LAM and PARAM var are duplicated.
!?cf I don't understand this comment for the LAM ?
ALLOCATE(zbuf(ileng))

! Exchange data
! -------------
CALL GSTATS_BARRIER(720)
CALL GSTATS(529,0)
CALL MPL_ALLGATHERV(PSENDBUF=ydcv%data,PRECVBUF=zbuf,KRECVCOUNTS=ilenr,&
 & CDSTRING='CONTROL_VECTORS_COMM_MOD:ALLGATHER_CTLVEC')  
CALL GSTATS(529,1)
CALL GSTATS_BARRIER2(720)

! Unpack data
! -----------
CALL unpack_cv(ydcv,ileng,ilenr,idispr,zbuf,ydcv%nsizeg,pcv)
DEALLOCATE(zbuf)
CALL GSTATS(251,1)

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:ALLGATHER_CTLVEC',1,ZHOOK_HANDLE)
END SUBROUTINE ALLGATHER_CTLVEC

! ------------------------------------------------------------------

SUBROUTINE MULTIGATHER_CTLVEC(ydcv, kproc, pcv, kvec_start, kvec_end)

!     Purpose.
!     --------
!       Gather multiple control variables on multiple PEs.

!     Author.
!     -------
!       Tomas Wilhelmsson *ECMWF*

!     Modifications.
!     --------------
!       Original  : 09-08-26
! ----------------------------------------------

TYPE(control_vector), INTENT(IN) :: ydcv(:)
INTEGER(KIND=JPIM), INTENT(IN)   :: kproc(:)
REAL(KIND=JPRB), INTENT(OUT)     :: pcv(:)
INTEGER(KIND=JPIM), INTENT(IN)   :: kvec_start, kvec_end 

REAL(KIND=JPRB), ALLOCATABLE :: zsendbuf(:),zrecvbuf(:)
INTEGER(KIND=JPIM) :: isendcounts(NPROC), irecvcounts(NPROC)
INTEGER(KIND=JPIM) :: ilenr(NPROC,kvec_start:kvec_end),idispr(NPROC)
INTEGER(KIND=JPIM) :: ileng(kvec_start:kvec_end),jp,jj,j,icnt
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:MULTIGATHER_CTLVEC',0,ZHOOK_HANDLE)

CALL GSTATS(258,0)
DO jj = kvec_start, kvec_end
  IF (myproc==kproc(jj)) THEN
    IF (SIZE(pcv)/=ydcv(jj)%nsizeg) CALL ABOR1('MULTIGATHER_CTLVEC: pcv wrong size')
  ENDIF

  ! Allocate receive buffer
  ! -----------------------
  CALL get_cvlengths(ydcv(jj), ilenr(:,jj))

  ileng(jj)=SUM(ilenr(:,jj))

  IF (myproc==kproc(jj)) THEN
    ALLOCATE(zrecvbuf(ileng(jj)))
    DO jp = 1, NPROC
       irecvcounts(jp) = ilenr(jp,jj)
    ENDDO
  ENDIF
ENDDO

IF (.NOT. ALLOCATED(zrecvbuf)) THEN
  ALLOCATE(zrecvbuf(1))
  irecvcounts(1:NPROC) = 0
ENDIF

! Allocate and fill send buffer
! -----------------------------

ALLOCATE(zsendbuf(SUM(ilenr(myproc,:))))

isendcounts(1:NPROC) = 0
icnt = 0
DO jj = kvec_start, kvec_end
  DO j=1, SIZE(ydcv(jj)%data)
    icnt = icnt + 1
    zsendbuf(icnt) = ydcv(jj)%data(j)
  ENDDO
  isendcounts(kproc(jj)) = SIZE(ydcv(jj)%data)
ENDDO

IF (SUM(ilenr(myproc,:))/=icnt) CALL ABOR1('MULTIGATHER_CTLVEC: sendbuf wrong size')

! Exchange data
! -------------
CALL GSTATS_BARRIER(724)
CALL GSTATS(536,0)
CALL MPL_ALLTOALLV(zsendbuf, isendcounts, zrecvbuf, irecvcounts,&
                 & CDSTRING='CONTROL_VECTORS_COMM_MOD:MULTIGATHER_CTLVEC')  
CALL GSTATS(536,1)
CALL GSTATS_BARRIER2(724)

! Unpack data on root processor
! -----------------------------

DO jj = kvec_start, kvec_end
  IF (myproc==kproc(jj)) THEN

    idispr(1)=0
    DO jp=2,NPROC
      idispr(jp)=idispr(jp-1)+ilenr(jp-1,jj)
    ENDDO

    CALL GSTATS(1859,0)
    CALL unpack_cv(ydcv(jj),ileng(jj),ilenr(:,jj),idispr,zrecvbuf,ydcv(jj)%nsizeg,pcv)
    CALL GSTATS(1859,1)
  ENDIF
ENDDO

DEALLOCATE(zsendbuf)
DEALLOCATE(zrecvbuf)
CALL GSTATS(258,1)

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:MULTIGATHER_CTLVEC',1,ZHOOK_HANDLE)
END SUBROUTINE MULTIGATHER_CTLVEC

! ------------------------------------------------------------------

SUBROUTINE ALLGATHER_CVSECTION(ydcv, psec, kstart, kend)

!     Purpose.
!     --------
!       Gather part of the control variable on all PEs.
!       Replaces DVSECTION with a name that makes it explicit
!       that this is an expensive function.

!     Author.
!     -------
!       Yannick Tremolet

!     Modifications.
!     --------------
!       Original  : 02-08-12
! ----------------------------------------------

TYPE(control_vector), INTENT(IN) :: ydcv
INTEGER(KIND=JPIM), INTENT(IN) :: kstart, kend
REAL(KIND=JPRB), INTENT(OUT) :: psec(kstart:kend)
REAL(KIND=JPRB), ALLOCATABLE :: zwork(:)
INTEGER(KIND=JPIM) :: iv,iw,jv,jw,jj,it,jm,im,jp,isplen,jpt,ioff1,ioff3,ipt,iomaster,itag,ito(1)
TYPE(SPECTRAL_FIELD) :: ylsp
REAL(KIND=JPRB), ALLOCATABLE :: ZLOCGP(:,:,:), ZGLOBGP(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:ALLGATHER_CVSECTION',0,ZHOOK_HANDLE)

IF (kstart<1)       CALL ABOR1('ALLGATHER_CVSECTION: kstart<1')
IF (kend>ydcv%nsizeg) CALL ABOR1('ALLGATHER_CVSECTION: kend>sizeg')
IF (kstart>kend   ) CALL ABOR1('ALLGATHER_CVSECTION: kstart>kend')

IF (ydcv%lwaveletjb)  THEN

! We arbitrarily define the 'first' elements of the control vector to
! be the first elements of the first global field.

  IF (kstart>GRID_DEFINITION(1)%NGPTOTG) &
    & CALL ABOR1('ALLGATHER_CVSECTION: kstart>GRID_DEFINITION(1)%NGPTOTG .AND. ydcv%lwaveletjb')

  IF (kend>GRID_DEFINITION(1)%NGPTOTG) &
    & CALL ABOR1('ALLGATHER_CVSECTION: kend>GRID_DEFINITION(1)%NGPTOTG .AND. ydcv%lwaveletjb')

  If (ydcv%nscales/=NSCALES) CALL ABOR1 ('ALLGATHER_CVSECTION: ydcv%nscales/=NSCALES')

  ALLOCATE (ZLOCGP(GRID_DEFINITION(1)%NGPTOT,1,1))
  ALLOCATE (ZGLOBGP(GRID_DEFINITION(1)%NGPTOTG,1))

  IOFF1=0
  IOFF3=0

  DO JPT=1,GRID_DEFINITION(1)%NGPTOT
    IPT=ydcv%ibgn_scale(1)+IOFF1+IOFF3
    ZLOCGP(JPT,1,1) = ydcv%initcv(IPT)
    IOFF1=IOFF1+1
    IF (MOD(IOFF1,NPROMA)==0) THEN
      IOFF1=0
      IOFF3=IOFF3 + NPROMA*(nflevg*ydcv%ns3d+ydcv%ns2d)
    ENDIF
  ENDDO

  IOMASTER=1
  ITO(1)=IOMASTER
  IF (NPROC==1) THEN
    psec(kstart:kend) = ZLOCGP(kstart:kend,1,1) 
  ELSE
    CALL GSTATS_BARRIER(722)
!!!    CALL GSTATS(537,0)
    CALL GATH_GRID(PGPG=ZGLOBGP,PGP=ZLOCGP,KTO=ITO,KFGATHG=1,&
     & KRESOL=GRID_DEFINITION(1)%NRESOL_ID)
     psec(kstart:kend) = ZGLOBGP(kstart:kend,1) 
    ITAG=99
    CALL MPL_BROADCAST(psec(kstart:kend),KROOT=IOMASTER,KTAG=ITAG, &
     & CDSTRING='CONTROL_VECTORS_COMM_MOD:ALLGATHER_CVSECTION')
!!!    CALL GSTATS(537,1)
    CALL GSTATS_BARRIER2(722)
  ENDIF

  DEALLOCATE (ZLOCGP)
  DEALLOCATE (ZGLOBGP)

ELSE

! Using the fact the the first elements of the control vector
! are the M=0 components of the first 2D field. They are 2 times
! NSMAX+1 elements on the same processor. In current 4D-Var this
! is more than we need.
! In Aladin, this is 4 times NSMAX+1 for wavenumber M=0.

  IF (LELAM) THEN
    isplen=4*(ydcv%nsmax+1)
  ELSE
    isplen=2*(ydcv%nsmax+1)
  ENDIF

  IF (kend<=isplen) THEN
    CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
    jj=0
    DO iv=1,NPRTRV
      IF (ylsp%ns2l(iv)>0) THEN
        jv=iv
        jj=jj+1
      ENDIF
    ENDDO
    IF (jj/=1) CALL ABOR1('Error finding jv')
    jj=0
    DO iw=1,NPRTRW
      DO jm=1,ylsp%NUMPP(iw)
        im=ylsp%ms_proc(jm,iw)
        IF (im==0) THEN
          jw=iw
          jj=jj+1
        ENDIF
      ENDDO
    ENDDO
    IF (jj/=1) CALL ABOR1('Error finding jw')
    CALL deallocate_spec(ylsp)
  
    CALL SET2PE(jp,0,0,jw,jv)
  
    IF (myproc==jp) THEN
      psec(kstart:kend)=ydcv%data(kstart:kend)
    ENDIF
    it=MT_DISTRIBUTED_VECTOR
    CALL GSTATS_BARRIER(722)
    CALL GSTATS(615,0)
    CALL MPL_BROADCAST (psec(kstart:kend),KROOT=jp,KTAG=it, &
                      & CDSTRING='CONTROL_VECTORS_COMM_MOD:ALLGATHER_CVSECTION')
    CALL GSTATS(615,1)
    CALL GSTATS_BARRIER2(722)
  ELSE
    write(nulout,*)'WARNING: CVSECTION using inefficient version, '// &
     & 'kstart, kend=',kstart,kend  
    ALLOCATE(zwork(ydcv%nsizeg))
    CALL ALLGATHER_CTLVEC(ydcv, zwork)
    psec(kstart:kend)=zwork(kstart:kend)
    DEALLOCATE(zwork)
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:ALLGATHER_CVSECTION',1,ZHOOK_HANDLE)
END SUBROUTINE ALLGATHER_CVSECTION

! ------------------------------------------------------------------

SUBROUTINE SET_CVSECTION(psec, ydcv, kstart, kend)

!     Purpose.
!     --------
!       Set part of the control variable on all PEs.

!     Author.
!     -------
!       Yannick Tremolet

!     Modifications.
!     --------------
!       Original  : 02-08-13
! ----------------------------------------------

INTEGER(KIND=JPIM), INTENT(IN) :: kstart, kend
REAL(KIND=JPRB), INTENT(IN) :: psec(kstart:kend)
TYPE(control_vector), INTENT(INOUT) :: ydcv
REAL(KIND=JPRB), ALLOCATABLE :: zwork(:)
TYPE(SPECTRAL_FIELD) :: ylsp
INTEGER(KIND=JPIM) :: iv,iw,jv,jw,jj,jm,im,jp,isplen,jpt,ioff1,ioff3,ipt, &
                    & iomaster,ito(1)
REAL(KIND=JPRB), ALLOCATABLE :: ZLOCGP(:,:,:), ZGLOBGP(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:SET_CVSECTION',0,ZHOOK_HANDLE)
CALL GSTATS(252,0)
IF (kstart<1      ) CALL ABOR1('SET_CVSECTION: kstart<1')
IF (kend>ydcv%nsizeg) CALL ABOR1('SET_CVSECTION: kend>sizeg')
IF (kstart>kend   ) CALL ABOR1('SET_CVSECTION: kstart>kend')

IF (ydcv%lwaveletjb) THEN

! We arbitrarily define the 'first' elements of the control vector to
! be the first elements of the first global field. The field is gathered onto
! one processor. Elements kstart:kend are replaced. Then it is scattered back.

  IF (kstart>GRID_DEFINITION(1)%NGPTOTG) &
    & CALL ABOR1('SET_CVSECTION: kstart>GRID_DEFINITION(1)%NGPTOTG .AND. ydcv%lwaveletjb')

  IF (kend>GRID_DEFINITION(1)%NGPTOTG) &
    & CALL ABOR1('SET_CVSECTION: kend>GRID_DEFINITION(1)%NGPTOTG .AND. ydcv%lwaveletjb')

  If (ydcv%nscales/=NSCALES) CALL ABOR1 ('SET_CVSECTION: ydcv%nscales/=NSCALES')

  ALLOCATE (ZLOCGP(GRID_DEFINITION(1)%NGPTOT,1,1))
  ALLOCATE (ZGLOBGP(GRID_DEFINITION(1)%NGPTOTG,1))

  IOFF1=0
  IOFF3=0

  DO JPT=1,GRID_DEFINITION(1)%NGPTOT
    IPT=ydcv%ibgn_scale(1)+IOFF1+IOFF3
    ZLOCGP(JPT,1,1) = ydcv%initcv(IPT)
    IOFF1=IOFF1+1
    IF (MOD(IOFF1,NPROMA)==0) THEN
      IOFF1=0
      IOFF3=IOFF3 + NPROMA*(nflevg*ydcv%ns3d+ydcv%ns2d)
    ENDIF
  ENDDO

  IOMASTER=1
  ITO(1)=IOMASTER
  IF (NPROC==1) THEN
    ZLOCGP(kstart:kend,1,1) = psec(kstart:kend)
  ELSE
    CALL GATH_GRID(PGPG=ZGLOBGP,PGP=ZLOCGP,KTO=ITO,KFGATHG=1,&
     & KRESOL=GRID_DEFINITION(1)%NRESOL_ID)
    IF (MYPROC==IOMASTER) THEN
      ZGLOBGP(kstart:kend,1) = psec(kstart:kend)
    ENDIF
    CALL DIST_GRID(PGPG=ZGLOBGP,PGP=ZLOCGP,KFROM=ITO,KFDISTG=1,&
     & KRESOL=GRID_DEFINITION(1)%NRESOL_ID)
  ENDIF

  IOFF1=0
  IOFF3=0

  DO JPT=1,GRID_DEFINITION(1)%NGPTOT
    IPT=ydcv%ibgn_scale(1)+IOFF1+IOFF3
    ydcv%initcv(IPT) = ZLOCGP(JPT,1,1)
    IOFF1=IOFF1+1
    IF (MOD(IOFF1,NPROMA)==0) THEN
      IOFF1=0
      IOFF3=IOFF3 + NPROMA*(nflevg*ydcv%ns3d+ydcv%ns2d)
    ENDIF
  ENDDO

  DEALLOCATE (ZLOCGP)
  DEALLOCATE (ZGLOBGP)

ELSE

! Using the fact the the first elements of the control vector
! are the M=0 components of the first 2D field. They are 2 times
! NSMAX+1 elements on the same processor. In current 4D-Var this
! is more than we need.
! In Aladin, this is 4 times NSMAX+1 for wavenumber M=0.

  IF (LELAM) THEN
    isplen=4*(ydcv%nsmax+1)
  ELSE
    isplen=2*(ydcv%nsmax+1)
  ENDIF

  IF (kend<=isplen) THEN
    CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
    jj=0
    DO iv=1,NPRTRV
      IF (ylsp%ns2l(iv)>0) THEN
        jv=iv
        jj=jj+1
      ENDIF
    ENDDO
    IF (jj/=1) CALL ABOR1('Error finding jv')
    jj=0
    DO iw=1,NPRTRW
      DO jm=1,ylsp%NUMPP(iw)
        im=ylsp%ms_proc(jm,iw)
        IF (im==0) THEN
          jw=iw
          jj=jj+1
        ENDIF
      ENDDO
    ENDDO
    IF (jj/=1) CALL ABOR1('Error finding jw')
    CALL deallocate_spec(ylsp)

    CALL SET2PE(jp,0,0,jw,jv)

    IF (myproc==jp) THEN
      ydcv%data(kstart:kend)=psec(kstart:kend)
    ENDIF
  ELSE
! Even that could be done without communication... YT
    write(nulout,*)'WARNING: CVSECTION using inefficient version, '// &
     & 'kstart, kend=',kstart,kend  
    ALLOCATE(zwork(ydcv%nsizeg))
    CALL ALLGATHER_CTLVEC(ydcv, zwork)
    zwork(kstart:kend)=psec(kstart:kend)
    ydcv = zwork
    DEALLOCATE(zwork)
  ENDIF
ENDIF
CALL GSTATS(252,1)

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:SET_CVSECTION',1,ZHOOK_HANDLE)
END SUBROUTINE SET_CVSECTION

! ------------------------------------------------------------------

SUBROUTINE GET_CVINDEX(ydcv, kindx)
TYPE(control_vector), INTENT(IN) :: ydcv
INTEGER(KIND=JPIM), INTENT(OUT) :: kindx(:)

TYPE(SPECTRAL_FIELD) :: ylsp
INTEGER(KIND=JPIM) :: ja,jb,ia,ib,ii,jf,jfoff,jm,im,ilen,jsoff,jn,il,jl,jj, &
                    & ifoff,jcomp
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:GET_CVINDEX',0,ZHOOK_HANDLE)

IF (SIZE(kindx)/=ydcv%nsizel) CALL ABOR1('GET_CVINDEX: kindx wrong size')
IF (ydcv%lwaveletjb)  CALL ABOR1('GET_CVINDEX: not yet coded for wavelet Jb')

ii=0
jfoff=0
CALL PE2SET(MYPROC,ia,ib,ja,jb)

! initial condition
! -----------------
IF (ydcv%linitcv) THEN
  CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
  IF (.NOT.LELAM) THEN
    DO jf=1,ylsp%ns2d
      DO jm=1,ylsp%NUMP
        im=ylsp%ms_proc(jm,ja)
        ilen=2*(ylsp%nsmax-im+1)
        jsoff=jfoff+ylsp%NASM0G(im)-1
        DO jn=1,ilen
          ii=ii+1
          kindx(ii)=jsoff+jn
        ENDDO
      ENDDO
      jfoff=jfoff+ylsp%nspec2g
    ENDDO
  ELSE
    DO jf=1,ylsp%ns2d
      DO jm=1,ylsp%NUMP
        im=ylsp%ms_proc(jm,ja)
        ilen=ylsp%ncpl4m(im)
        jsoff=jfoff+ylsp%NESM0G(im)-1
        DO jn=1,ilen
          ii=ii+1
          kindx(ii)=jsoff+jn
        ENDDO
      ENDDO
      jfoff=jfoff+ylsp%nspec2g
    ENDDO
  ENDIF
  jfoff=ylsp%nspec2g*ylsp%ns2g
  IF (.NOT.LELAM) THEN
    DO jf=1,ylsp%ns3d
      DO jm=1,ylsp%NUMP
        im=ylsp%ms_proc(jm,ja)
        ilen=2*(ylsp%nsmax-im+1)
        jsoff=jfoff+(ylsp%NASM0G(im)-1)*ylsp%nflevg
        DO jn=1,ilen
          DO jl=1,ylsp%nflevl
            il=ylsp%nptrll(jb)+jl-1
            ii=ii+1
            kindx(ii)=jsoff+il
          ENDDO
          jsoff=jsoff+ylsp%nflevg
        ENDDO
      ENDDO
      jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
    ENDDO
  ELSE
    DO jf=1,ylsp%ns3d
      DO jm=1,ylsp%NUMP
        im=ylsp%ms_proc(jm,ja)
        ilen=ylsp%ncpl4m(im)
        jsoff=jfoff+(ylsp%NESM0G(im)-1)*ylsp%nflevg
        DO jn=1,ilen
          DO jl=1,ylsp%nflevl
            il=ylsp%nptrll(jb)+jl-1
            ii=ii+1
            kindx(ii)=jsoff+il
          ENDDO
          jsoff=jsoff+ylsp%nflevg
        ENDDO
      ENDDO
      jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
    ENDDO
  ENDIF
  CALL deallocate_spec(ylsp)
ENDIF

! model error if needed
! ---------------------
IF (ydcv%nmoderr>0) THEN
  CALL inq_spec(ylsp,ydcv%nserr,ydcv%nserr,ydcv%nve3d,ydcv%nve2d,0)
  DO jcomp=1,ydcv%nmoderr
    ifoff=jfoff
    IF (.NOT.LELAM) THEN
      DO jf=1,ylsp%ns2d
        DO jm=1,ylsp%NUMP
          im=ylsp%ms_proc(jm,ja)
          ilen=2*(ylsp%nsmax-im+1)
          jsoff=ifoff+ylsp%NASM0G(im)-1
          DO jn=1,ilen
            ii=ii+1
            kindx(ii)=jsoff+jn
          ENDDO
        ENDDO
        ifoff=ifoff+ylsp%nspec2g
      ENDDO
      jfoff=jfoff+ylsp%nspec2g*ylsp%ns2g
      DO jf=1,ylsp%ns3d
        DO jm=1,ylsp%NUMP
          im=ylsp%ms_proc(jm,ja)
          ilen=2*(ylsp%nsmax-im+1)
          jsoff=jfoff+(ylsp%NASM0G(im)-1)*ylsp%nflevg
          DO jn=1,ilen
            DO jl=1,ylsp%nflevl
              il=ylsp%nptrll(jb)+jl-1
              ii=ii+1
              kindx(ii)=jsoff+il
            ENDDO
            jsoff=jsoff+ylsp%nflevg
          ENDDO
        ENDDO
        jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
      ENDDO
    ELSE
      DO jf=1,ylsp%ns2d
        DO jm=1,ylsp%NUMP
          im=ylsp%ms_proc(jm,ja)
          ilen=ylsp%ncpl4m(im)
          jsoff=ifoff+ylsp%NESM0G(im)-1
          DO jn=1,ilen
            ii=ii+1
            kindx(ii)=jsoff+jn
          ENDDO
        ENDDO
        ifoff=ifoff+ylsp%nspec2g
      ENDDO
      jfoff=jfoff+ylsp%nspec2g*ylsp%ns2g
      DO jf=1,ylsp%ns3d
        DO jm=1,ylsp%NUMP
          im=ylsp%ms_proc(jm,ja)
          ilen=ylsp%ncpl4m(im)
          jsoff=jfoff+(ylsp%NESM0G(im)-1)*ylsp%nflevg
          DO jn=1,ilen
            DO jl=1,ylsp%nflevl
              il=ylsp%nptrll(jb)+jl-1
              ii=ii+1
              kindx(ii)=jsoff+il
            ENDDO
            jsoff=jsoff+ylsp%nflevg
          ENDDO
        ENDDO
        jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
      ENDDO
    ENDIF
  ENDDO
  CALL deallocate_spec(ylsp)
ENDIF

! TOVS variable if needed
! -----------------------
IF (ydcv%ltovscv) THEN
  DO jf=1,NVATOVV
    DO jj=1,NLENTOVSCVP(myproc)
      ii=ii+1
      kindx(ii)=jfoff+NVATOVOFF(myproc)+jj
    ENDDO
    jfoff=jfoff+NLENTOVSCVG
  ENDDO
ENDIF
 
! TS variable if needed
! -----------------------
IF (ydcv%ltscv) THEN
  DO jj=1,NOBNTVBP(myproc)
    ii=ii+1
    kindx(ii)=jfoff+NVATSOFF(myproc)+jj
  ENDDO
  jfoff=jfoff+NOBNTVG
ENDIF


! LAM variable if needed
! -----------------------
IF (ydcv%lam1d) THEN
  DO jf=1,ydcv%ns1d
    DO jl=1,NFLEVL
      il=nptrll(jb)-1+jl
      ii=ii+1
      kindx(ii)=jfoff+il
    ENDDO
    jfoff=jfoff+NFLEVG
  ENDDO
ENDIF

! PARAM variable if needed
! ------------------------
IF (ydcv%npara>0) THEN
  DO jf=1,ydcv%npara
    ii=ii+1
    kindx(ii)=jfoff+jf
  ENDDO
  jfoff=jfoff+ydcv%npara
ENDIF

IF (ii/=ydcv%nsizel) CALL ABOR1('GET_CVINDEX: ii /= nsizel')

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:GET_CVINDEX',1,ZHOOK_HANDLE)
END SUBROUTINE GET_CVINDEX

! ------------------------------------------------------------------

SUBROUTINE GET_CVLENGTHS(ydcv, klen)

! Get length of local component of ydcv on each proc.

TYPE(control_vector), INTENT(IN) :: ydcv
INTEGER(KIND=JPIM), INTENT(OUT) :: klen(NPROC)

TYPE(SPECTRAL_FIELD) :: ylsp
INTEGER(KIND=JPIM) :: jp,ja,jb,ia,ib,ispec2,jm,im,ilen,imylen
INTEGER(KIND=JPIM) :: ITEMP

REAL(KIND=JPRB) :: ZHOOK_HANDLE


IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:GET_CVLENGTHS',0,ZHOOK_HANDLE)

DO jp=1,NPROC
  CALL PE2SET(jp,ia,ib,ja,jb)

! initial condition
! -----------------
  IF (ydcv%linitcv) THEN
    IF (ydcv%lwaveletjb) THEN
      if (ydcv%nscales==NSCALES) then
        klen(jp) = NPROMA*(NFLEVG*ydcv%ns3d+ydcv%ns2d)*SUM((NGPTOTL(:,ia,ib)+NPROMA-1)/NPROMA)
      else
        ITEMP=3+NSCALES-ydcv%nscales
        klen(jp) = NPROMA*(NFLEVG*ydcv%ns3d+ydcv%ns2d) &
                & *( 2*((NGPTOTL(2+NSCALES-ydcv%nscales,ia,ib)+NPROMA-1)/NPROMA)      &
                &   +SUM((NGPTOTL(ITEMP:,ia,ib)+NPROMA-1)/NPROMA))
      endif

      IF (jp==MYPROC) THEN
        if (ydcv%nscales==NSCALES) then
          imylen = nproma*(nflevg*ydcv%ns3d+ydcv%ns2d)*sum(NBLOCKSL(:))
        else
          ITEMP=3+NSCALES-ydcv%nscales
          imylen = nproma*(nflevg*ydcv%ns3d+ydcv%ns2d) &
                & *(2*nblocksl(2+NSCALES-ydcv%nscales)+sum(nblocksl(ITEMP:)))
        endif
        IF (klen(jp)/=imylen) THEN
          WRITE (NULOUT,*) 'GET_CVLENGTHS: Error calculating initicv lengths'
          WRITE (NULOUT,*) 'GET_CVLENGTHS: jp=',jp,' klen(jp)=',klen(jp),' imylen=',imylen
          CALL ABOR1 ('GET_CVLENGTHS: Error calculating initicv lengths')
        ENDIF
      ENDIF
    ELSE
      CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
      ispec2=0
      IF (.NOT.LELAM) THEN
        DO jm=1,ylsp%NUMPP(ja)
          im=ylsp%ms_proc(jm,ja)
          ilen=ylsp%nsmax-im+1
          ispec2=ispec2+2*ilen
        ENDDO
      ELSE
        DO jm=1,ylsp%NUMPP(ja)
          im=ylsp%ms_proc(jm,ja)
          ilen=ylsp%ncpl4m(im)
          ispec2=ispec2+ilen
        ENDDO
      ENDIF
      klen(jp)=ispec2*ylsp%ns3d*ylsp%numll(jb)   ! 3D fields
      klen(jp)=klen(jp)+ispec2*ylsp%ns2l(jb)     ! 2D fields
      CALL deallocate_spec(ylsp)
    ENDIF
  ELSE
    klen(jp)=0
  ENDIF

! model error
! -----------
  IF (ydcv%nmoderr>0) THEN
    CALL inq_spec(ylsp,ydcv%nserr,ydcv%nserr,ydcv%nve3d,ydcv%nve2d,0)
    ispec2=0
    IF (.NOT.LELAM) THEN
      DO jm=1,ylsp%NUMPP(ja)
        im=ylsp%ms_proc(jm,ja)
        ilen=ylsp%nsmax-im+1
        ispec2=ispec2+2*ilen
      ENDDO
    ELSE
      DO jm=1,ylsp%NUMPP(ja)
        im=ylsp%ms_proc(jm,ja)
        ilen=ylsp%ncpl4m(im)
        ispec2=ispec2+ilen
      ENDDO
    ENDIF
    ispec2 = ispec2 * ydcv%nmoderr
    klen(jp)=klen(jp)+ispec2*ylsp%ns3d*ylsp%numll(jb)  ! 3D fields
    klen(jp)=klen(jp)+ispec2*ylsp%ns2l(jb)             ! 2D fields
    CALL deallocate_spec(ylsp)
  ENDIF

! TOVS variable
! -------------
  IF (ydcv%ltovscv) THEN
    klen(jp)=klen(jp)+NVATOVV*NLENTOVSCVP(jp)
  ENDIF
 
! TS variable
! -------------
  IF (ydcv%ltscv) THEN
    klen(jp)=klen(jp)+NOBNTVBP(jp)
  ENDIF


! LAM variable
! ------------
  IF (ydcv%lam1d) THEN
    klen(jp)=klen(jp)+ydcv%ns1d*numll(jb)
  ENDIF

! PARAM variable
! --------------
  klen(jp)=klen(jp)+ydcv%npara

ENDDO

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:GET_CVLENGTHS',1,ZHOOK_HANDLE)
END SUBROUTINE GET_CVLENGTHS

! ------------------------------------------------------------------

SUBROUTINE PACK_CV(ydcv,kcv,pcv,kbuf,klens,kdisps,pbuf)

! Arrange global cv in send buffer.

TYPE(CONTROL_VECTOR), INTENT(IN) :: ydcv
INTEGER(KIND=JPIM), INTENT(IN) :: kcv,kbuf
INTEGER(KIND=JPIM), INTENT(IN) :: klens(nproc),kdisps(nproc)
REAL(KIND=JPRB), INTENT(IN)  :: pcv(kcv)
REAL(KIND=JPRB), INTENT(OUT) :: pbuf(kbuf)

TYPE(SPECTRAL_FIELD) :: ylsp
INTEGER(KIND=JPIM) :: jp,ja,jb,ia,ib,ii,jf,jfoff,jm,im,ilen,jsoff,jn,il,jl, &
                    & jj,ifoff,jscale, jrof, icend, ibl, inflds, igp, &
                    & igptot, iaaaarrrrgh, ioff, igl2, igl1, ilast, jlon, &
                    & jgl, iloff, igloff, jkglo, iscale, jcomp
INTEGER(KIND=JPIM), ALLOCATABLE :: ivatovindxinv(:)
REAL(KIND=JPRB), ALLOCATABLE :: zgploc(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:PACK_CV',0,ZHOOK_HANDLE)

IF (ydcv%ltovscv) THEN
  ALLOCATE(ivatovindxinv(NLENTOVSCVG))
  DO jj = 1, NLENTOVSCVG
    ivatovindxinv(nvatovindx(jj)) = jj
  END DO
ENDIF

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1)&
!$OMP& PRIVATE(jp,ii,jfoff,ia,ib,ja,jb,inflds,jscale,iscale,igptot,zgploc,jf,&
!$OMP& IGLOFF,IGL1,IGL2,IOFF,ILAST,JJ,ILEN,ILOFF,JGL,JLON,iaaaarrrrgh,igp,jkglo,&
!$OMP& ICEND,IBL,jrof,jm,im,jsoff,jn,jl,il,ylsp,jcomp,ifoff)
DO jp=1,NPROC
  ii=kdisps(jp)
  jfoff=0
  CALL PE2SET(jp,ia,ib,ja,jb)

! Pack initial condition (but not if wavelet Jb)
! ----------------------------------------------
  IF (ydcv%linitcv) THEN
    IF (ydcv%lwaveletjb) THEN

      inflds = nflevg*ydcv%ns3d+ydcv%ns2d

      DO jscale=1,ydcv%nscales
        iscale=max(jscale,2)+NSCALES-ydcv%nscales

        igptot=GRID_DEFINITION(iscale)%NGPTOTL(ia,ib)
        ALLOCATE (zgploc(igptot))

        DO jf=1,inflds
          IGLOFF = GRID_DEFINITION(iscale)%NPTRFRSTLAT(ia)
          IGL1 =   GRID_DEFINITION(iscale)%NFRSTLAT(ia)
          IGL2 =   GRID_DEFINITION(iscale)%NLSTLAT(ia)
          IOFF = 0

          IF (ia>1) THEN
            IF( GRID_DEFINITION(iscale)%NLSTLAT(ia-1) == GRID_DEFINITION(iscale)%NFRSTLAT(ia) )THEN
              ILAST = GRID_DEFINITION(iscale)%NLSTLAT(ia-1)-1
            ELSE
              ILAST = GRID_DEFINITION(iscale)%NLSTLAT(ia-1)
            ENDIF
            DO JJ=GRID_DEFINITION(iscale)%NFRSTLAT(1),ILAST
              IOFF = IOFF+GRID_DEFINITION(iscale)%NLOENG(JJ)
            ENDDO
          ENDIF
  
          ILEN = 0
          ILOFF = 0
          DO JGL=IGL1,IGL2
            DO JLON=1,GRID_DEFINITION(iscale)%NONL(IGLOFF+JGL-IGL1,ib)
              iaaaarrrrgh = IOFF+ILOFF+GRID_DEFINITION(iscale)%NSTA(IGLOFF+JGL-IGL1,ib)+JLON-1 &
                           & + jfoff
              zgploc(ILEN+JLON) =pcv(iaaaarrrrgh)
            ENDDO
            ILEN = ILEN + GRID_DEFINITION(iscale)%NONL(IGLOFF+JGL-IGL1,ib)
            ILOFF = ILOFF + GRID_DEFINITION(iscale)%NLOENG(JGL)
          ENDDO

          igp = 0
          DO jkglo=1,igptot,NPROMA
            ICEND=MIN(NPROMA,igptot-JKGLO+1)
            IBL=(JKGLO-1)/NPROMA+1
            DO jrof=1,ICEND
              igp = igp + 1
              pbuf(ii+jrof+NPROMA*(jf-1+inflds*(ibl-1))) = zgploc(igp)
            ENDDO
            DO jrof=ICEND+1,NPROMA
              pbuf(ii+jrof+NPROMA*(jf-1+inflds*(ibl-1))) = 0.0_JPRB
            ENDDO
          ENDDO

!         increment the offset into the gathered control vector
          jfoff=jfoff+GRID_DEFINITION(iscale)%NGPTOTG

        ENDDO

        DEALLOCATE (zgploc)

!       increment the offset into the send buffer
        ii=ii+NPROMA*inflds*((igptot+NPROMA-1)/NPROMA)
      ENDDO

    ELSE
      CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
      IF (.NOT.LELAM) THEN
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=jfoff+ylsp%NASM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pbuf(ii)=pcv(jsoff+jn)
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g
        ENDDO
      ELSE
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=jfoff+ylsp%NESM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pbuf(ii)=pcv(jsoff+jn)
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g
        ENDDO
      ENDIF
      jfoff=ylsp%nspec2g*ylsp%ns2g
      IF (.NOT.LELAM) THEN
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=jfoff+(ylsp%NASM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pbuf(ii)=pcv(jsoff+il)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ELSE
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=jfoff+(ylsp%NESM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pbuf(ii)=pcv(jsoff+il)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ENDIF
      CALL deallocate_spec(ylsp)
    ENDIF
  ENDIF

! Pack model error if needed
! --------------------------
  IF (ydcv%nmoderr>0) THEN
    CALL inq_spec(ylsp,ydcv%nserr,ydcv%nserr,ydcv%nve3d,ydcv%nve2d,0)
    DO jcomp=1,ydcv%nmoderr
      ifoff=jfoff
      IF (.NOT.LELAM) THEN
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=ifoff+ylsp%NASM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pbuf(ii)=pcv(jsoff+jn)
            ENDDO
          ENDDO
          ifoff=ifoff+ylsp%nspec2g
        ENDDO
        jfoff=jfoff+ylsp%nspec2g*ylsp%ns2g
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=jfoff+(ylsp%NASM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pbuf(ii)=pcv(jsoff+il)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ELSE
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=ifoff+ylsp%NESM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pbuf(ii)=pcv(jsoff+jn)
            ENDDO
          ENDDO
          ifoff=ifoff+ylsp%nspec2g
        ENDDO
        jfoff=jfoff+ylsp%nspec2g*ylsp%ns2g
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=jfoff+(ylsp%NESM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pbuf(ii)=pcv(jsoff+il)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ENDIF
    ENDDO
    CALL deallocate_spec(ylsp)
  ENDIF

! Pack TOVS variable if needed
! ----------------------------
  IF (ydcv%ltovscv) THEN
    DO jf=1,NVATOVV
      DO jj=1,NLENTOVSCVP(jp)
        ii=ii+1
        pbuf(ii)=pcv(jfoff+ivatovindxinv(NVATOVOFF(jp)+jj))
      ENDDO
      jfoff=jfoff+NLENTOVSCVG
    ENDDO
  ENDIF
 
! Pack TS variable if needed
! ----------------------------
  IF (ydcv%ltscv) THEN
    DO jj=1,NOBNTVBP(jp)
      ii=ii+1
      pbuf(ii)=pcv(jfoff+NVATSOFF(jp)+jj)
    ENDDO
    jfoff=jfoff+NOBNTVG
  ENDIF


! Pack LAM variable if needed
! ----------------------------
  IF (ydcv%lam1d) THEN
    DO jf=1,ydcv%ns1d
      DO jl=1,numll(jb)
        il=nptrll(jb)-1+jl
        ii=ii+1
        pbuf(ii)=pcv(jfoff+il)
      ENDDO
      jfoff=jfoff+NFLEVG
    ENDDO
  ENDIF

! Pack PARAM variable if needed
! -----------------------------
  IF (ydcv%npara>0) THEN
    DO jf=1,ydcv%npara
      ii=ii+1
      pbuf(ii)=pcv(jfoff+jf)
    ENDDO
    jfoff=jfoff+ydcv%npara
  ENDIF

  IF( ii-kdisps(jp) > klens(jp) )THEN
    WRITE(NULOUT,'("CONTROL_VECTORS_COMM_MOD:PACK_CV ii-kdisps>klens: JP,II-KDISPS,KLENS=",&
     &3(2X,I10))')jp,ii-kdisps(jp),klens(jp)
    CALL ABOR1('CONTROL_VECTORS_COMM_MOD:PACK_CV  ii-kdisps>klens')
  ENDIF

ENDDO
!$OMP END PARALLEL DO

IF (ydcv%ltovscv) THEN
  DEALLOCATE(ivatovindxinv)
ENDIF

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:PACK_CV',1,ZHOOK_HANDLE)
END SUBROUTINE PACK_CV

! ------------------------------------------------------------------

SUBROUTINE UNPACK_CV(ydcv,kbuf,klenr,kdispr,pbuf,kcv,pcv)

! Arrange receive buffer in global cv.

TYPE(CONTROL_VECTOR), INTENT(IN) :: ydcv
INTEGER(KIND=JPIM), INTENT(IN) :: kcv,kbuf
INTEGER(KIND=JPIM), INTENT(IN) :: klenr(nproc),kdispr(nproc)
REAL(KIND=JPRB), INTENT(IN)  :: pbuf(kbuf)
REAL(KIND=JPRB), INTENT(OUT) :: pcv(kcv)

TYPE(SPECTRAL_FIELD) :: ylsp
INTEGER(KIND=JPIM) :: jp,ja,jb,ia,ib,ii,jf,jfoff,jm,im,ilen,jsoff,jn,il,jl, &
                    & jj,ifoff,jscale, jrof, icend, ibl, inflds, igp, &
                    & igptot, iaaaarrrrgh, ioff, igl2, igl1, ilast, jlon, &
                    & jgl, iloff, igloff, jkglo, iscale, jcomp
INTEGER(KIND=JPIM), ALLOCATABLE :: ivatovindxinv(:)
REAL(KIND=JPRB), ALLOCATABLE :: zgploc(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:UNPACK_CV',0,ZHOOK_HANDLE)

IF (ydcv%ltovscv) THEN
  ALLOCATE(ivatovindxinv(NLENTOVSCVG))
  DO jj = 1, NLENTOVSCVG
    ivatovindxinv(nvatovindx(jj)) = jj
  END DO
ENDIF

! Unpack initial condition (but not if wavelet Jb)
! ------------------------------------------------
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1)&
!$OMP& PRIVATE(jp,ii,jfoff,ia,ib,ja,jb,inflds,jscale,iscale,igptot,zgploc,jf,&
!$OMP& igp,jkglo,ICEND,IBL,jrof,IGLOFF,IGL1,IGL2,IOFF,ILAST,JJ,ILEN,ILOFF,&
!$OMP& JGL,JLON,iaaaarrrrgh,jm,im,jsoff,jn,jl,il,ylsp,jcomp,ifoff)
DO jp=1,NPROC
  ii=kdispr(jp)
  jfoff=0
  CALL PE2SET(jp,ia,ib,ja,jb)

  IF (ydcv%linitcv) THEN
    IF (ydcv%lwaveletjb) THEN

      inflds = nflevg*ydcv%ns3d+ydcv%ns2d

      DO jscale=1,ydcv%nscales
        iscale=max(jscale,2)+NSCALES-ydcv%nscales

        igptot=GRID_DEFINITION(iscale)%NGPTOTL(ia,ib)
        ALLOCATE (zgploc(igptot))

        DO jf=1,inflds
          igp = 0
          DO jkglo=1,igptot,NPROMA
            ICEND=MIN(NPROMA,igptot-JKGLO+1)
            IBL=(JKGLO-1)/NPROMA+1
            DO jrof=1,ICEND
              igp = igp + 1
              zgploc(igp) = pbuf(ii+jrof+NPROMA*(jf-1+inflds*(ibl-1)))
            ENDDO
          ENDDO

          IGLOFF = GRID_DEFINITION(iscale)%NPTRFRSTLAT(ia)
          IGL1 =   GRID_DEFINITION(iscale)%NFRSTLAT(ia)
          IGL2 =   GRID_DEFINITION(iscale)%NLSTLAT(ia)
          IOFF = 0

          IF (ia>1) THEN
            IF( GRID_DEFINITION(iscale)%NLSTLAT(ia-1) == GRID_DEFINITION(iscale)%NFRSTLAT(ia) )THEN
              ILAST = GRID_DEFINITION(iscale)%NLSTLAT(ia-1)-1
            ELSE
              ILAST = GRID_DEFINITION(iscale)%NLSTLAT(ia-1)
            ENDIF
            DO JJ=GRID_DEFINITION(iscale)%NFRSTLAT(1),ILAST
              IOFF = IOFF+GRID_DEFINITION(iscale)%NLOENG(JJ)
            ENDDO
          ENDIF
  
          ILEN = 0
          ILOFF = 0
          DO JGL=IGL1,IGL2
            DO JLON=1,GRID_DEFINITION(iscale)%NONL(IGLOFF+JGL-IGL1,ib)
              iaaaarrrrgh = IOFF+ILOFF+GRID_DEFINITION(iscale)%NSTA(IGLOFF+JGL-IGL1,ib)+JLON-1 &
                           & + jfoff
              pcv(iaaaarrrrgh) = zgploc(ILEN+JLON)
            ENDDO
            ILEN = ILEN + GRID_DEFINITION(iscale)%NONL(IGLOFF+JGL-IGL1,ib)
            ILOFF = ILOFF + GRID_DEFINITION(iscale)%NLOENG(JGL)
          ENDDO

!         increment the offset into the gathered control vector
          jfoff=jfoff+GRID_DEFINITION(iscale)%NGPTOTG

        ENDDO

        DEALLOCATE (zgploc)

!       increment the offset into the send buffer
        ii=ii+NPROMA*inflds*((igptot+NPROMA-1)/NPROMA)
      ENDDO

    ELSE
      CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
      IF (.NOT.LELAM) THEN
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=jfoff+ylsp%NASM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pcv(jsoff+jn)=pbuf(ii)
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g
        ENDDO
      ELSE
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=jfoff+ylsp%NESM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pcv(jsoff+jn)=pbuf(ii)
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g
        ENDDO
      ENDIF
      jfoff=ylsp%nspec2g*ylsp%ns2g
      IF (.NOT.LELAM) THEN
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=jfoff+(ylsp%NASM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pcv(jsoff+il)=pbuf(ii)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ELSE
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=jfoff+(ylsp%NESM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pcv(jsoff+il)=pbuf(ii)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ENDIF
      CALL deallocate_spec(ylsp)
    ENDIF
  ENDIF

! Unpack model error if needed
! ----------------------------
  IF (ydcv%nmoderr>0) THEN
    CALL inq_spec(ylsp,ydcv%nserr,ydcv%nserr,ydcv%nve3d,ydcv%nve2d,0)
    DO jcomp=1,ydcv%nmoderr
      ifoff=jfoff
      IF (.NOT.LELAM) THEN
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=ifoff+ylsp%NASM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pcv(jsoff+jn)=pbuf(ii)
            ENDDO
          ENDDO
          ifoff=ifoff+ylsp%nspec2g
        ENDDO
        jfoff=jfoff+ylsp%nspec2g*ylsp%ns2g
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=2*(ylsp%nsmax-im+1)
            jsoff=jfoff+(ylsp%NASM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pcv(jsoff+il)=pbuf(ii)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ELSE
        DO jf=1,ylsp%ns2l(jb)
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=ifoff+ylsp%NESM0G(im)-1
            DO jn=1,ilen
              ii=ii+1
              pcv(jsoff+jn)=pbuf(ii)
            ENDDO
          ENDDO
          ifoff=ifoff+ylsp%nspec2g
        ENDDO
        jfoff=jfoff+ylsp%nspec2g*ylsp%ns2g
        DO jf=1,ylsp%ns3d
          DO jm=1,ylsp%NUMPP(ja)
            im=ylsp%ms_proc(jm,ja)
            ilen=ylsp%ncpl4m(im)
            jsoff=jfoff+(ylsp%NESM0G(im)-1)*ylsp%nflevg
            DO jn=1,ilen
              DO jl=1,ylsp%numll(jb)
                il=ylsp%nptrll(jb)+jl-1
                ii=ii+1
                pcv(jsoff+il)=pbuf(ii)
              ENDDO
              jsoff=jsoff+ylsp%nflevg
            ENDDO
          ENDDO
          jfoff=jfoff+ylsp%nspec2g*ylsp%nflevg
        ENDDO
      ENDIF
    ENDDO
    CALL deallocate_spec(ylsp)
  ENDIF

! Unpack TOVS variable if needed
! ------------------------------
  IF (ydcv%ltovscv) THEN
    DO jf=1,NVATOVV
      DO jj=1,NLENTOVSCVP(jp)
        ii=ii+1
        pcv(jfoff+ivatovindxinv(NVATOVOFF(jp)+jj))=pbuf(ii)
      ENDDO
      jfoff=jfoff+NLENTOVSCVG
    ENDDO
  ENDIF

! Unpack TS variable if needed
! ------------------------------
  IF (YDCV%ltscv) THEN
    DO jj=1,NOBNTVBP(jp)
      ii=ii+1
      pcv(jfoff+NVATSOFF(jp)+jj)=pbuf(ii)
    ENDDO
    jfoff=jfoff+NOBNTVG
  ENDIF


! Unpack LAM variable if needed
! -----------------------------
  IF (ydcv%lam1d) THEN
    DO jf=1,ydcv%ns1d
      DO jl=1,numll(jb)
        il=nptrll(jb)-1+jl
        ii=ii+1
        pcv(jfoff+il)=pbuf(ii)
      ENDDO
      jfoff=jfoff+NFLEVG
    ENDDO
  ENDIF

! Unpack PARAM variable if needed
! -------------------------------
  IF (ydcv%npara>0) THEN
    DO jf=1,ydcv%npara
      ii=ii+1
      pcv(jfoff+jf)=pbuf(ii)
    ENDDO
    jfoff=jfoff+ydcv%npara
  ENDIF

  IF( ii-kdispr(jp) > klenr(jp) )THEN
    WRITE(NULOUT,'("CONTROL_VECTORS_COMM_MOD:UNPACK_CV ii-kdispr>klenr: JP,II-KDISPR,KLENR=",&
     &3(2X,I10))')jp,ii-kdispr(jp),klenr(jp)
    CALL ABOR1('CONTROL_VECTORS_COMM_MOD:UNPACK_CV  ii-kdispr>klenr')
  ENDIF

ENDDO
!$OMP END PARALLEL DO

IF (ydcv%ltovscv) THEN
  DEALLOCATE(ivatovindxinv)
ENDIF

IF (LHOOK) CALL DR_HOOK('CONTROL_VECTORS_COMM_MOD:UNPACK_CV',1,ZHOOK_HANDLE)
END SUBROUTINE UNPACK_CV

! ------------------------------------------------------------------

END MODULE CONTROL_VECTORS_COMM_MOD
