!****  COUPL04_MODULE *  - VARIABLE DEFINITION FOR OASIS4-COUPLED RUNS 

!     PURPOSE.
!     --------
!       MODULE CONTAINING VARIABLES FOR PRISM EXCHANGE - OASIS 4

!     PARAMETER        DESCRIPTION                                  
!     ---------         -----------                                  !      
!      JP_FIELDS       Number of transient field , i.e. field to be exchanged 
!      JP_NDIM=3       Dimensions of field of fields 

!      IVALID_SHAPE    DIMENSION OF LOCAL DATA PER GRID

!      TYPE GRID        Derived type containing grid information
!       %ID             ID 
!       %ITYPE          PRISM GRID TYPE 
!       %CNAME          NAME macthing with SMIOC
!       %ISHAPE         DIMENSION OF LOCAL DATA PER GRID
!       %ID_POINT       ID for PRISM point type
!       %CNAME_POINT    Name for PRISM point type (i.e. centre, vaergae etc)
!       %LNEW_POINT     GRID POINTs CHANGING in time

!      TYPE EXCH_FIELD  Deived type conatining transient field information
!       %IVAR_ID         VARIABLE ID
!       %CVAR_NAME       VARIABLE NAME    
!       %L_IN            INPUT  FIELD
!       %L_OUT           OUTPUT FIELD
!       %TIME_CALL       Frequency of call in code (MINIMAL PERIOD IN SMIOC.XML)
!       %IGRID           ID of grid  
!       %IPOINTER        Pointer for start GFL or GMV  array
!       %MASK            ID for Mask (to be defined by PRISM_SET_MASK)!*
!       %CMEM            memory array (GMV, GFL, GPP, COR, SPA 
!       %L_SPEC          Spectral fields 
!       %L_IC            exchange before model physics 
!       %L_ACC           accumulated value , needs "de-accumulation"   
!       %IORDER          Order of call in get_exchnage   

!     MODEL_TIME          model time 
!     MODEL_TIME_BOUNDS(2)   model time bound of valid period
!     DINCR1, DINCR2     time interval  =/- half a model time step 
!     DINCRMODEL         model time step in seconds
!     LGLOBALEXGP        exchange of global fields via Master
!     LGLOBALEXSP        exchange of global spectral fields via Master 
!                                       one MPI task only   
!     MEX_PROCGP            Process number for exchange  GP 
!     MEX_PROCSP            Process number for exchange  SP 

!     COMP_NAME       NAME of Aplication  (component local name IN SMIOC.XML)
!     ICOMP_ID        ID of Component

!     Method:
!     ------
!         replace PRISM_DUMMY with PRISM and link & include to OASIS4 libs 
!         for coupled runs

!         PRISM_DUMMY will call MPL_ABORT if LCOUPLO4_ENV

!     REFERENCE.
!     ----------
!                   OASIS4 User guide
!     AUTHOR.
!     -------
!     2005-22-10  J. Flemming

!     MODIFICATIONS.
!     -------

MODULE COUPLO4_MIX

  USE PARKIND1  ,ONLY : JPIM, JPRB

  USE PRISM_DUMMY_MOD

  IMPLICIT NONE

  SAVE

  INTEGER(KIND=JPIM), PARAMETER  :: JP_FIELDS=70  ! max number of fields

  ! for PRISM_reduced_gaussiangrid (1 horizotal 1 vertical)
  INTEGER(KIND=JPIM), PARAMETER  :: JP_NDIM=2    

  INTEGER (KIND=JPIM)                      :: NBR_FIELDS ! number of fields 

  ! Define transient field  definition variables

  TYPE TRANSIENT
     INTEGER(KIND=JPIM)   :: ID
     CHARACTER (LEN=132)  :: CNAME
     LOGICAL              :: L_IN
     LOGICAL              :: L_OUT
     REAL(KIND=JPRB)      :: T_CALL 
     INTEGER(KIND=JPIM)   :: IGRID 
     INTEGER(KIND=JPIM)   :: IPOINTER
     INTEGER(KIND=JPIM)   :: MASK
     CHARACTER(LEN=3)     :: CMEM
     LOGICAL              :: L_SPEC
     LOGICAL              :: L_IC
     LOGICAL              :: L_ACC
     INTEGER(KIND=JPIM)   :: IORDER 

  END TYPE TRANSIENT

  TYPE(TRANSIENT), DIMENSION(JP_FIELDS) :: YFIELD_CPL

  ! Define grid definition variables

  TYPE GRID_TYP
     INTEGER(KIND=JPIM)   :: ID
     INTEGER(KIND=JPIM)   :: ITYPE 
     CHARACTER(LEN=128)   :: CNAME 
     INTEGER(KIND=JPIM)   :: ISHAPE(2,JP_NDIM)
     INTEGER(KIND=JPIM)   :: ID_POINTS
     CHARACTER(LEN=128)   :: CNAME_POINTS 
     LOGICAL              :: LNEW_POINTS
  END TYPE GRID_TYP

  INTEGER(KIND=JPIM), PARAMETER :: NBR_GRIDS   = 4 
  !  IGRID_ID = 1  multi level field
  !  IGRID_ID = 2  single level field

  TYPE(GRID_TYP), DIMENSION(NBR_GRIDS) :: YIFS_GRID


  ! PRISM time
  TYPE(PRISM_TIME_STRUCT) :: YMODEL_TIME
  TYPE(PRISM_TIME_STRUCT) :: YMODEL_TIME_BOUNDS(2)

  REAL(KIND=JPRB)  :: EINCR1     ! delta minus TIME interval 
  REAL(KIND=JPRB)  :: EINCR2     ! delta plus TIME interval 
  REAL(KIND=JPRB)  :: EINCRMODEL ! MODEL TIME STEP

   ! Exchange modus
  LOGICAL                   :: LGLOBALEXCHANGEGP ! Exchange of global fields 
  INTEGER(KIND=JPIM)        :: MEX_PROCGP ! Process number for exchange
  LOGICAL                   :: LGLOBALEXCHANGESP ! Exchange of global fields 
  INTEGER(KIND=JPIM)        :: MEX_PROCSP ! Process number for exchange

  CHARACTER(LEN=128) :: COMP_NAME = 'ifs'
  INTEGER(KIND=JPIM) :: ICOMP_ID

! Storage of previous accumulated field 
  REAL(KIND=JPRB),ALLOCATABLE  :: RPREVACC(:,:)
! Storage of previous accumulated field index 
  INTEGER(KIND=JPIM),ALLOCATABLE  :: INDEX_RPREVACC(:)

END MODULE COUPLO4_MIX
