MODULE GRIDPOINT_FIELDS_MIX

!   Purpose
!   -------
!     Flexible data structure to handle grid point fields.

!   Author
!   ------
!     Yannick Tremolet

!   Modifications
!   -------------
!     Original    16-Jan-04  
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMDIM  , ONLY: NPROMA, NGPBLKS, NFLEVG
USE YOMGEM  , ONLY: NGPTOT, NGPTOTG
USE YOMGRB  , ONLY: NGRBD, NGRBVO, NGRBU, NGRBV, NGRBT, NGRBQ, NGRBO3, NGRBLNSP
USE INDEXFIND_MOD, ONLY: INDXFIND

IMPLICIT NONE
SAVE
PRIVATE
PUBLIC GRIDPOINT_FIELD, ALLOCATE_GRID, DEALLOCATE_GRID, ASSIGNMENT(=), &
     & SELF_ADD, SELF_SUB, SELF_MUL, HUGIFY_ENDGRID, CLIP_GRID

! ------------------------------------------------------------------

! There could be a type GRIDPOINT_STRUCT similar to what Mike has
! done in yomwavlet and a pointer to a GRIDPOINT_STRUCT in the
! GRIDPOINT_FIELD type.

TYPE GRIDPOINT_FIELD
  REAL(KIND=JPRB), POINTER :: GP3D(:,:,:,:)
  REAL(KIND=JPRB), POINTER :: GP2D(:,:,:)
  INTEGER(KIND=JPIM) :: NG3D
  INTEGER(KIND=JPIM) :: NG2D
  INTEGER(KIND=JPIM) :: NPROMA
  INTEGER(KIND=JPIM) :: NGPBLKS
  INTEGER(KIND=JPIM) :: NFLEVG
  INTEGER(KIND=JPIM) :: NGPTOT
  INTEGER(KIND=JPIM) :: NGPTOTG
  INTEGER(KIND=JPIM), POINTER :: NGRIB(:), NGRIB2(:), NGRIB3(:)
  INTEGER(KIND=JPIM), POINTER :: NPARAMID(:), NPARAMID2(:), NPARAMID3(:)
  REAL(KIND=JPRB), POINTER :: DIV(:,:,:)
  REAL(KIND=JPRB), POINTER :: VOR(:,:,:)
  REAL(KIND=JPRB), POINTER :: W_U(:,:,:)
  REAL(KIND=JPRB), POINTER :: W_V(:,:,:)
  REAL(KIND=JPRB), POINTER :: TEM(:,:,:)
  REAL(KIND=JPRB), POINTER :: HUM(:,:,:)
  REAL(KIND=JPRB), POINTER :: OZN(:,:,:)
  REAL(KIND=JPRB), POINTER :: SPR(:,:)
  REAL(KIND=JPRB), POINTER :: WIND(:,:,:,:)
  REAL(KIND=JPRB), POINTER :: SCAL(:,:,:,:)
END TYPE GRIDPOINT_FIELD

! ------------------------------------------------------------------

INTERFACE ASSIGNMENT (=)
MODULE PROCEDURE assign_gp_gp, assign_scalar_gp
END INTERFACE

INTERFACE OPERATOR (.EQV.)
MODULE PROCEDURE equiv_grid
END INTERFACE

INTERFACE OPERATOR (.NEQV.)
MODULE PROCEDURE nequiv_grid
END INTERFACE

INTERFACE SELF_ADD  ! What we really want here is ASSIGNMENT (+=)
MODULE PROCEDURE SELF_ADD_GP, SELF_ADD_SCAL
END INTERFACE

INTERFACE SELF_SUB  ! What we really want here is ASSIGNMENT (-=)
MODULE PROCEDURE SELF_SUB_GP
END INTERFACE

INTERFACE SELF_MUL  ! What we really want here is ASSIGNMENT (*=)
MODULE PROCEDURE SELF_MUL_GP, SELF_MUL_SCAL
END INTERFACE

! ------------------------------------------------------------------
#include "abor1.intfb.h"
! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

SUBROUTINE ALLOCATE_GRID(YDGRID,kg3d,kg2d,kgrib,kparamid)

TYPE (GRIDPOINT_FIELD), INTENT(OUT) :: YDGRID
INTEGER(KIND=JPIM), INTENT(IN) :: kg3d, kg2d, kgrib(kg3d+kg2d)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: kparamid(kg3d+kg2d)
INTEGER(KIND=JPIM) :: indd, indv, indt, indu, indw, indq, indo, inds, iwind
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:ALLOCATE_GRID',0,ZHOOK_HANDLE)

YDGRID%NG3D=kg3d
YDGRID%NG2D=kg2d
YDGRID%NPROMA =NPROMA
YDGRID%NGPBLKS=NGPBLKS
YDGRID%NFLEVG =NFLEVG
YDGRID%NGPTOT =NGPTOT
YDGRID%NGPTOTG=NGPTOTG
ALLOCATE(YDGRID%ngrib(kg3d+kg2d))
ALLOCATE(YDGRID%nparamid(kg3d+kg2d))
YDGRID%ngrib(:)=kgrib(:)
IF(PRESENT(kparamid)) THEN
  YDGRID%nparamid(:)=kparamid(:)
ELSE
  YDGRID%nparamid(:)=YDGRID%ngrib(:)
ENDIF
YDGRID%ngrib3 => YDGRID%ngrib(1:kg3d)
YDGRID%ngrib2 => YDGRID%ngrib(kg3d+1:kg3d+kg2d)
YDGRID%nparamid3 => YDGRID%nparamid(1:kg3d)
YDGRID%nparamid2 => YDGRID%nparamid(kg3d+1:kg3d+kg2d)

ALLOCATE(YDGRID%GP3D(YDGRID%NPROMA,YDGRID%NFLEVG,YDGRID%NG3D,YDGRID%NGPBLKS))
ALLOCATE(YDGRID%GP2D(YDGRID%NPROMA,YDGRID%NG2D,YDGRID%NGPBLKS))

NULLIFY(YDGRID%DIV)
NULLIFY(YDGRID%VOR)
NULLIFY(YDGRID%W_U)
NULLIFY(YDGRID%W_V)
NULLIFY(YDGRID%TEM)
NULLIFY(YDGRID%HUM)
NULLIFY(YDGRID%OZN)
indd=INDXFIND(YDGRID%ngrib3,NGRBD)
indv=INDXFIND(YDGRID%ngrib3,NGRBVO)
indu=INDXFIND(YDGRID%ngrib3,NGRBU)
indw=INDXFIND(YDGRID%ngrib3,NGRBV)
indt=INDXFIND(YDGRID%ngrib3,NGRBT)
indq=INDXFIND(YDGRID%ngrib3,NGRBQ)
indo=INDXFIND(YDGRID%ngrib3,NGRBO3)
IF (indd>0) YDGRID%DIV => YDGRID%gp3d(:,:,indd,:)
IF (indv>0) YDGRID%VOR => YDGRID%gp3d(:,:,indv,:)
IF (indu>0) YDGRID%W_U => YDGRID%gp3d(:,:,indu,:)
IF (indw>0) YDGRID%W_V => YDGRID%gp3d(:,:,indw,:)
IF (indt>0) YDGRID%TEM => YDGRID%gp3d(:,:,indt,:)
IF (indq>0) YDGRID%HUM => YDGRID%gp3d(:,:,indq,:)
IF (indo>0) YDGRID%OZN => YDGRID%gp3d(:,:,indo,:)

NULLIFY(YDGRID%SPR)
inds=INDXFIND(YDGRID%ngrib2,NGRBLNSP)
IF (inds>0) YDGRID%SPR => YDGRID%gp2d(:,inds,:)

NULLIFY(YDGRID%WIND)
NULLIFY(YDGRID%SCAL)
iwind=0
IF (indd>0) iwind=iwind+1
IF (indv>0) iwind=iwind+1
IF (indu>0) iwind=iwind+1
IF (indw>0) iwind=iwind+1
IF (indd>iwind.OR.indv>iwind.OR.indu>iwind.OR.indw>iwind) &
  & CALL ABOR1('GRIDPOINT_FIELDS_MIX:ALLOCATE_GRID Error wind')
IF (iwind>0) YDGRID%WIND => YDGRID%gp3d(:,:,1:iwind,:)
YDGRID%SCAL => YDGRID%gp3d(:,:,iwind+1:YDGRID%ng3d,:)

CALL HUGIFY_ENDGRID(YDGRID)

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:ALLOCATE_GRID',1,ZHOOK_HANDLE)
END SUBROUTINE ALLOCATE_GRID

! ------------------------------------------------------------------

SUBROUTINE DEALLOCATE_GRID(YDGRID)

TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDGRID
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:DEALLOCATE_GRID',0,ZHOOK_HANDLE)

NULLIFY(YDGRID%DIV)
NULLIFY(YDGRID%VOR)
NULLIFY(YDGRID%W_U)
NULLIFY(YDGRID%W_V)
NULLIFY(YDGRID%TEM)
NULLIFY(YDGRID%ngrib2)
NULLIFY(YDGRID%ngrib3)
NULLIFY(YDGRID%nparamid2)
NULLIFY(YDGRID%nparamid3)
IF (ASSOCIATED(YDGRID%ngrib)) DEALLOCATE(YDGRID%ngrib)
IF (ASSOCIATED(YDGRID%nparamid)) DEALLOCATE(YDGRID%nparamid)
IF (ASSOCIATED(YDGRID%GP3D )) DEALLOCATE(YDGRID%GP3D)
IF (ASSOCIATED(YDGRID%GP2D )) DEALLOCATE(YDGRID%GP2D)

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:DEALLOCATE_GRID',1,ZHOOK_HANDLE)
END SUBROUTINE DEALLOCATE_GRID

! ------------------------------------------------------------------

SUBROUTINE assign_scalar_gp(YDGRID,pscalar)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDGRID
REAL(KIND=JPRB), INTENT(IN) :: pscalar
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:ASSIGN_SCALAR_GP',0,ZHOOK_HANDLE)

YDGRID%gp3d(:,:,:,:)=pscalar
YDGRID%gp2d(:,:,:)  =pscalar
CALL HUGIFY_ENDGRID(YDGRID)

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:ASSIGN_SCALAR_GP',1,ZHOOK_HANDLE)
END SUBROUTINE assign_scalar_gp

! ------------------------------------------------------------------

SUBROUTINE assign_gp_gp(YDGRID1,YDGRID2)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDGRID1
TYPE (GRIDPOINT_FIELD), INTENT(IN)    :: YDGRID2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:ASSIGN_GP_GP',0,ZHOOK_HANDLE)
IF (YDGRID1.NEQV.YDGRID2) CALL ABOR1('Incompatible grid point fields')

YDGRID1%gp3d(:,:,:,:)=YDGRID2%gp3d(:,:,:,:)
YDGRID1%gp2d(:,:,:)  =YDGRID2%gp2d(:,:,:)
CALL HUGIFY_ENDGRID(YDGRID1)

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:ASSIGN_GP_GP',1,ZHOOK_HANDLE)
END SUBROUTINE assign_gp_gp

! ------------------------------------------------------------------

LOGICAL FUNCTION equiv_grid(YDGRID1,YDGRID2)
TYPE(GRIDPOINT_FIELD), INTENT(IN) :: YDGRID1
TYPE(GRIDPOINT_FIELD), INTENT(IN) :: YDGRID2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

LOGICAL :: ll
INTEGER(KIND=JPIM) :: jf

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:EQUIV_GRID',0,ZHOOK_HANDLE)

ll = .true.
ll = ll .AND. (YDGRID1%ng2d==YDGRID2%ng2d)
ll = ll .AND. (YDGRID1%ng3d==YDGRID2%ng3d)
IF (ll) THEN
  DO jf=1,YDGRID1%ng2d+YDGRID1%ng3d
    ll = ll .AND. (YDGRID1%ngrib(jf)==YDGRID2%ngrib(jf))
  ENDDO
ENDIF

equiv_grid = ll

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:EQUIV_GRID',1,ZHOOK_HANDLE)
END FUNCTION equiv_grid

! ------------------------------------------------------------------

LOGICAL FUNCTION nequiv_grid(YDGRID1,YDGRID2)
TYPE(GRIDPOINT_FIELD), INTENT(IN) :: YDGRID1
TYPE(GRIDPOINT_FIELD), INTENT(IN) :: YDGRID2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:NEQUIV_GRID',0,ZHOOK_HANDLE)

nequiv_grid = .NOT. equiv_grid(YDGRID1,YDGRID2)

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:NEQUIV_GRID',1,ZHOOK_HANDLE)
END FUNCTION nequiv_grid

! ------------------------------------------------------------------

SUBROUTINE HUGIFY_ENDGRID(YDGRID)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDGRID
REAL(KIND=JPRB) :: ZZZ
INTEGER(KIND=JPIM) :: ilast, jf, jb, jl, jj
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:HUGIFY_ENDGRID',0,ZHOOK_HANDLE)

ilast=YDGRID%NGPTOT-(YDGRID%NGPBLKS-1)*YDGRID%NPROMA
jb=YDGRID%NGPBLKS
zzz=HUGE(zzz)

DO jf=1,YDGRID%NG3D
  DO jl=1,YDGRID%NFLEVG
    DO jj=ilast+1,YDGRID%NPROMA
      YDGRID%GP3D(jj,jl,jf,jb)=zzz
    ENDDO
  ENDDO
ENDDO
DO jf=1,YDGRID%NG2D
  DO jj=ilast+1,YDGRID%NPROMA
    YDGRID%GP2D(jj,jf,jb)=zzz
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:HUGIFY_ENDGRID',1,ZHOOK_HANDLE)
END SUBROUTINE HUGIFY_ENDGRID

! ------------------------------------------------------------------

SUBROUTINE SELF_ADD_GP(YDA,YDB)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDA
TYPE (GRIDPOINT_FIELD), INTENT(IN)    :: YDB
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_ADD_GP',0,ZHOOK_HANDLE)

IF (YDA.NEQV.YDB) CALL ABOR1('SELF_ADD_GP: Incompatible gridpoint fields')
IF (YDA%NGPTOTG/=YDB%NGPTOTG .OR. YDA%NGPTOT/=YDB%NGPTOT) &
  & CALL ABOR1('SELF_ADD_GP: Incompatible gridpoint resolutions')

!yt!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,ilen,jf,jl,jj)
DO jb=1,YDA%NGPBLKS
  ilen=MIN(YDA%NPROMA, YDA%NGPTOT-(jb-1)*YDA%NPROMA)
  DO jf=1,YDA%NG3D
    DO jl=1,YDA%NFLEVG
      DO jj=1,ilen
        YDA%GP3D(jj,jl,jf,jb) = YDA%GP3D(jj,jl,jf,jb) + YDB%GP3D(jj,jl,jf,jb)
      ENDDO
    ENDDO
  ENDDO
  DO jf=1,YDA%NG2D
    DO jj=1,ilen
      YDA%GP2D(jj,jf,jb) = YDA%GP2D(jj,jf,jb) + YDB%GP2D(jj,jf,jb)
    ENDDO
  ENDDO
ENDDO
!yt!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_ADD_GP',1,ZHOOK_HANDLE)
END SUBROUTINE SELF_ADD_GP
! ------------------------------------------------------------------

SUBROUTINE SELF_ADD_SCAL(YDA,PB)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDA
REAL(KIND=JPRB), INTENT(IN) :: PB
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_ADD_SCAL',0,ZHOOK_HANDLE)

!yt!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,ilen,jf,jl,jj)
DO jb=1,YDA%NGPBLKS
  ilen=MIN(YDA%NPROMA, YDA%NGPTOT-(jb-1)*YDA%NPROMA)
  DO jf=1,YDA%NG3D
    DO jl=1,YDA%NFLEVG
      DO jj=1,ilen
        YDA%GP3D(jj,jl,jf,jb) = PB + YDA%GP3D(jj,jl,jf,jb)
      ENDDO
    ENDDO
  ENDDO
  DO jf=1,YDA%NG2D
    DO jj=1,ilen
      YDA%GP2D(jj,jf,jb) = PB + YDA%GP2D(jj,jf,jb)
    ENDDO
  ENDDO
ENDDO
!yt!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_ADD_SCAL',1,ZHOOK_HANDLE)
END SUBROUTINE SELF_ADD_SCAL
! ------------------------------------------------------------------

SUBROUTINE SELF_SUB_GP(YDA,YDB)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDA
TYPE (GRIDPOINT_FIELD), INTENT(IN)    :: YDB
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_SUB_GP',0,ZHOOK_HANDLE)

IF (YDA.NEQV.YDB) CALL ABOR1('SELF_SUB_GP: Incompatible gridpoint fields')
IF (YDA%NGPTOTG/=YDB%NGPTOTG .OR. YDA%NGPTOT/=YDB%NGPTOT) &
  & CALL ABOR1('SELF_SUB_GP: Incompatible gridpoint resolutions')

!yt!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,ilen,jf,jl,jj)
DO jb=1,YDA%NGPBLKS
  ilen=MIN(YDA%NPROMA, YDA%NGPTOT-(jb-1)*YDA%NPROMA)
  DO jf=1,YDA%NG3D
    DO jl=1,YDA%NFLEVG
      DO jj=1,ilen
        YDA%GP3D(jj,jl,jf,jb) = YDA%GP3D(jj,jl,jf,jb) - YDB%GP3D(jj,jl,jf,jb)
      ENDDO
    ENDDO
  ENDDO
  DO jf=1,YDA%NG2D
    DO jj=1,ilen
      YDA%GP2D(jj,jf,jb) = YDA%GP2D(jj,jf,jb) - YDB%GP2D(jj,jf,jb)
    ENDDO
  ENDDO
ENDDO
!yt!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_SUB_GP',1,ZHOOK_HANDLE)
END SUBROUTINE SELF_SUB_GP
! ------------------------------------------------------------------

SUBROUTINE SELF_MUL_GP(YDA,YDB)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDA
TYPE (GRIDPOINT_FIELD), INTENT(IN)    :: YDB
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_MUL_GP',0,ZHOOK_HANDLE)

IF (YDA.NEQV.YDB) CALL ABOR1('SELF_MUL_GP: Incompatible gridpoint fields')
IF (YDA%NGPTOTG/=YDB%NGPTOTG .OR. YDA%NGPTOT/=YDB%NGPTOT) &
  & CALL ABOR1('SELF_MUL_GP: Incompatible gridpoint resolutions')

!yt!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,ilen,jf,jl,jj)
DO jb=1,YDA%NGPBLKS
  ilen=MIN(YDA%NPROMA, YDA%NGPTOT-(jb-1)*YDA%NPROMA)
  DO jf=1,YDA%NG3D
    DO jl=1,YDA%NFLEVG
      DO jj=1,ilen
        YDA%GP3D(jj,jl,jf,jb) = YDA%GP3D(jj,jl,jf,jb) * YDB%GP3D(jj,jl,jf,jb)
      ENDDO
    ENDDO
  ENDDO
  DO jf=1,YDA%NG2D
    DO jj=1,ilen
      YDA%GP2D(jj,jf,jb) = YDA%GP2D(jj,jf,jb) * YDB%GP2D(jj,jf,jb)
    ENDDO
  ENDDO
ENDDO
!yt!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_MUL_GP',1,ZHOOK_HANDLE)
END SUBROUTINE SELF_MUL_GP

! ------------------------------------------------------------------

SUBROUTINE SELF_MUL_SCAL(YDA,PB)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDA
REAL(KIND=JPRB), INTENT(IN) :: PB
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_MUL_SCAL',0,ZHOOK_HANDLE)

!yt!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,ilen,jf,jl,jj)
DO jb=1,YDA%NGPBLKS
  ilen=MIN(YDA%NPROMA, YDA%NGPTOT-(jb-1)*YDA%NPROMA)
  DO jf=1,YDA%NG3D
    DO jl=1,YDA%NFLEVG
      DO jj=1,ilen
        YDA%GP3D(jj,jl,jf,jb) = PB * YDA%GP3D(jj,jl,jf,jb)
      ENDDO
    ENDDO
  ENDDO
  DO jf=1,YDA%NG2D
    DO jj=1,ilen
      YDA%GP2D(jj,jf,jb) = PB * YDA%GP2D(jj,jf,jb)
    ENDDO
  ENDDO
ENDDO
!yt!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:SELF_MUL_SCAL',1,ZHOOK_HANDLE)
END SUBROUTINE SELF_MUL_SCAL
! ------------------------------------------------------------------

SUBROUTINE CLIP_GRID(YDA, PMIN2D, PMAX2D, PMIN3D, PMAX3D)
TYPE (GRIDPOINT_FIELD), INTENT(INOUT) :: YDA
REAL(KIND=JPRB), INTENT(IN), DIMENSION(YDA%NG2D)             ,OPTIONAL :: PMIN2D
REAL(KIND=JPRB), INTENT(IN), DIMENSION(YDA%NG2D)             ,OPTIONAL :: PMAX2D
REAL(KIND=JPRB), INTENT(IN), DIMENSION(YDA%NFLEVG, YDA%NG3D) ,OPTIONAL :: PMIN3D
REAL(KIND=JPRB), INTENT(IN), DIMENSION(YDA%NFLEVG, YDA%NG3D) ,OPTIONAL :: PMAX3D
INTEGER(KIND=JPIM) :: jb,jf,jl,jj,ilen

LOGICAL :: LLMIN2D, LLMAX2D, LLMIN3D, LLMAX3D

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:CLIP_GRID',0,ZHOOK_HANDLE)

LLMIN2D=PRESENT(PMIN2D)
LLMAX2D=PRESENT(PMAX2D)
LLMIN3D=PRESENT(PMIN3D)
LLMAX3D=PRESENT(PMAX3D)


!yt!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jb,ilen,jf,jl,jj)
DO jb=1,YDA%NGPBLKS
  ilen=MIN(YDA%NPROMA, YDA%NGPTOT-(jb-1)*YDA%NPROMA)
  IF (LLMIN3D.OR.LLMAX3D) THEN
    DO jf=1,YDA%NG3D
      DO jl=1,YDA%NFLEVG
        DO jj=1,ilen
          IF (LLMAX3D) THEN
            YDA%GP3D(jj,jl,jf,jb) = MIN(PMAX3D(jl,jf),YDA%GP3D(jj,jl,jf,jb))
          ENDIF
          IF (LLMIN3D) THEN
            YDA%GP3D(jj,jl,jf,jb) = MAX(PMIN3D(jl,jf),YDA%GP3D(jj,jl,jf,jb))
          ENDIF
        ENDDO
      ENDDO
    ENDDO
  ENDIF
  IF (LLMIN2D.OR.LLMAX2D) THEN
    DO jf=1,YDA%NG2D
      DO jj=1,ilen
        IF (LLMAX2D) THEN
          YDA%GP2D(jj,jf,jb) = MIN( PMAX2D(jf), YDA%GP2D(jj,jf,jb) )
        ENDIF
        IF (LLMIN2D) THEN
          YDA%GP2D(jj,jf,jb) = MAX( PMIN2D(jf), YDA%GP2D(jj,jf,jb) )
        ENDIF
      ENDDO
    ENDDO
  ENDIF
ENDDO
!yt!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GRIDPOINT_FIELDS_MIX:CLIP_GRID',1,ZHOOK_HANDLE)
END SUBROUTINE CLIP_GRID
! ------------------------------------------------------------------

END MODULE GRIDPOINT_FIELDS_MIX
