MODULE INDEXFIND_MOD

!   Purpose
!   -------
!     Find position of value in array. Like Fortran functions
!     MINLOC and MAXLOC, INTLOC and REALOC assume lower bound
!     index is 1.

!   Author
!   ------
!     Yannick Tremolet

!   Modifications
!   -------------
!     Original    23-Aug-04
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK

IMPLICIT NONE
PRIVATE
PUBLIC INDXFIND

INTERFACE INDXFIND
MODULE PROCEDURE INDINLOC, INDRELOC
END INTERFACE

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

FUNCTION INDINLOC(karray,kval)

INTEGER(KIND=JPIM) :: INDINLOC
INTEGER(KIND=JPIM), INTENT(IN) :: karray(:),kval
INTEGER(KIND=JPIM) :: ii,jj
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('INDEXFIND_MOD:INDINLOC',0,ZHOOK_HANDLE)
jj=0
ii=1
DO WHILE (ii<=SIZE(karray) .AND. jj==0)
  IF (karray(ii)==kval) jj=ii
  ii=ii+1
ENDDO
INDINLOC=jj
IF (LHOOK) CALL DR_HOOK('INDEXFIND_MOD:INDINLOC',1,ZHOOK_HANDLE)
END FUNCTION INDINLOC

! ------------------------------------------------------------------

FUNCTION INDRELOC(parray,pval)

INTEGER(KIND=JPIM) :: INDRELOC
REAL(KIND=JPRB), INTENT(IN) :: parray(:),pval
INTEGER(KIND=JPIM) :: ii,jj
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('INDEXFIND_MOD:INDRELOC',0,ZHOOK_HANDLE)
jj=0
ii=1
DO WHILE (ii<=SIZE(parray) .AND. jj==0)
  IF (parray(ii)==pval) jj=ii
  ii=ii+1
ENDDO
INDRELOC=jj
IF (LHOOK) CALL DR_HOOK('INDEXFIND_MOD:INDRELOC',1,ZHOOK_HANDLE)
END FUNCTION INDRELOC

! ------------------------------------------------------------------

END MODULE INDEXFIND_MOD
