MODULE OBSHOR_CACHE_MIX

! TO REDUCE GETDB-CALLS IN OBSHOR/OBSHORAD.F90
! OBSTYPE, ABNOB & MAPOMM ARE STORED PER TIMESLOT
! AS RESULT GETDB-CALLS ARE CALLED ONLY ONCE PER 
! PARTICULAR TIMESLOT

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NACTIM 
USE YOMLUN   , ONLY :    NULERR

IMPLICIT NONE

SAVE
PUBLIC

TYPE OBSHOR_CACHE_T
LOGICAL   :: PROCESSED
INTEGER(KIND=JPIM) :: NSIZE
INTEGER(KIND=JPIM), POINTER :: OBSTYPE(:)
INTEGER(KIND=JPIM), POINTER :: OBSGRP(:)
INTEGER(KIND=JPIM), POINTER :: MAPOMM(:)
INTEGER(KIND=JPIM), POINTER :: ABNOB(:)
END TYPE OBSHOR_CACHE_T

TYPE(OBSHOR_CACHE_T), ALLOCATABLE :: OBSHOR_CACHE(:)

CONTAINS

SUBROUTINE OBSHOR_CACHE_INIT()
IMPLICIT NONE
INTEGER(KIND=JPIM) :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('OBSHOR_CACHE_MIX:OBSHOR_CACHE_INIT',0,ZHOOK_HANDLE)
IF (ALLOCATED(OBSHOR_CACHE) .AND. LHOOK) CALL DR_HOOK('OBSHOR_CACHE_MIX:OBSHOR_CACHE_INIT',1,ZHOOK_HANDLE)
IF (ALLOCATED(OBSHOR_CACHE)) RETURN
ALLOCATE(OBSHOR_CACHE(NACTIM))
DO J=1,NACTIM
  OBSHOR_CACHE(J)%PROCESSED = .FALSE.
  OBSHOR_CACHE(J)%NSIZE = 0
  NULLIFY(OBSHOR_CACHE(J)%OBSTYPE)
  NULLIFY(OBSHOR_CACHE(J)%OBSGRP)
  NULLIFY(OBSHOR_CACHE(J)%MAPOMM)
  NULLIFY(OBSHOR_CACHE(J)%ABNOB)
ENDDO
IF (LHOOK) CALL DR_HOOK('OBSHOR_CACHE_MIX:OBSHOR_CACHE_INIT',1,ZHOOK_HANDLE)
END SUBROUTINE OBSHOR_CACHE_INIT

SUBROUTINE OBSHOR_CACHE_ALLOC(CDROUTINE, KTSLOTNO, KLEN)
IMPLICIT NONE
CHARACTER(LEN=*), INTENT(IN) :: CDROUTINE
INTEGER(KIND=JPIM), INTENT(IN)        :: KTSLOTNO, KLEN
INTEGER(KIND=JPIM) :: ILEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTERFACE
#include "abortdb.h"
END INTERFACE
IF (LHOOK) CALL DR_HOOK('OBSHOR_CACHE_MIX:OBSHOR_CACHE_ALLOC',0,ZHOOK_HANDLE)
CALL OBSHOR_CACHE_INIT()
IF (KTSLOTNO >= 1 .AND. KTSLOTNO <= NACTIM) THEN
  IF (.NOT.OBSHOR_CACHE(KTSLOTNO)%PROCESSED) THEN
! We want always to be able to access at least the first element, albeit dummy, when KLEN=0
    ILEN = MAX(1,KLEN) 
    ALLOCATE(OBSHOR_CACHE(KTSLOTNO)%OBSTYPE(ILEN)) ; OBSHOR_CACHE(KTSLOTNO)%OBSTYPE(1) = 0
    ALLOCATE(OBSHOR_CACHE(KTSLOTNO)%OBSGRP(ILEN)) ; OBSHOR_CACHE(KTSLOTNO)%OBSGRP(1) = 0
    ALLOCATE(OBSHOR_CACHE(KTSLOTNO)%MAPOMM(ILEN))  ; OBSHOR_CACHE(KTSLOTNO)%MAPOMM(1)  = 0
    ALLOCATE(OBSHOR_CACHE(KTSLOTNO)%ABNOB(ILEN))   ; OBSHOR_CACHE(KTSLOTNO)%ABNOB(1)   = 0
    OBSHOR_CACHE(KTSLOTNO)%NSIZE = KLEN
    OBSHOR_CACHE(KTSLOTNO)%PROCESSED = .FALSE. ! Still not processed
  ELSEIF (OBSHOR_CACHE(KTSLOTNO)%PROCESSED .AND. &
     & OBSHOR_CACHE(KTSLOTNO)%NSIZE /= KLEN) THEN  
    WRITE(NULERR,*)'***ERROR IN '//CDROUTINE//&
     & ': UNEXPECTED NO. OF OBS. =',KLEN,&
     & '; PREVIOUS CALL OF THIS TIMESLOT LETS US EXPECT =',&
     & OBSHOR_CACHE(KTSLOTNO)%NSIZE,&
     & '; TIMESLOT=',KTSLOTNO  
    CALL ABORTDB(CDROUTINE, 'UNEXPECTED NO. OF OBS.', KLEN)
  ENDIF
ELSE
  WRITE(NULERR,*)'***ERROR IN '//CDROUTINE//&
   & ': INVALID TIMESLOT=',KTSLOTNO,&
   & '; MUST BE BETWEEN 1 AND NACTIM=',NACTIM  
  CALL ABORTDB(CDROUTINE, 'INVALID TIMESLOT ; MUST BE BETWEEN 1 AND NACTIM', &
   & KTSLOTNO)  
ENDIF
IF (LHOOK) CALL DR_HOOK('OBSHOR_CACHE_MIX:OBSHOR_CACHE_ALLOC',1,ZHOOK_HANDLE)
END SUBROUTINE OBSHOR_CACHE_ALLOC

END MODULE OBSHOR_CACHE_MIX
