MODULE PAR_RDLR

! Parameter pointers and dimensions for array (Z,P)RDLR, used in the
! Wood and Staniforth deep-layer formulation of NH equations.
! (Z,P)RDLR contains (r/a), (a/r), grad(r/a), at full and half levels.

USE PARKIND1  ,ONLY : JPIM

IMPLICIT NONE

SAVE

! JP_RSAF : parameter pointer for (r/a) at full levels.
! JP_RSAH : parameter pointer for (r/a) at half levels.
! JP_ASRF : parameter pointer for (a/r) at full levels.
! JP_ASRH : parameter pointer for (a/r) at half levels.
! JP_RSAFL: parameter pointer for zonal comp of grad(r/a) at full levels.
! JP_RSAFM: parameter pointer for merid comp of grad(r/a) at full levels.
! JP_RSAHL: parameter pointer for zonal comp of grad(r/a) at half levels.
! JP_RSAHM: parameter pointer for merid comp of grad(r/a) at half levels.
! JP_DIMR : total number of fields in (Z,P)RDLR.
! JP_DIMR0: total number of fields in (Z,P)RDLR at time t.
! JP_DIMR9: total number of fields in (Z,P)RDLR at time t-dt (useless grad.).
! JP_DIMR1: total number of fields in (Z,P)RDLR at time t+dt (useless grad.).
! Remark for gradient: the gradient computed is the semi-reduced one
!  (i.e. (r/a)*nabla).

INTEGER(KIND=JPIM),PARAMETER:: JP_RSAF=1
INTEGER(KIND=JPIM),PARAMETER:: JP_RSAH=2
INTEGER(KIND=JPIM),PARAMETER:: JP_ASRF=3
INTEGER(KIND=JPIM),PARAMETER:: JP_ASRH=4
INTEGER(KIND=JPIM),PARAMETER:: JP_RSAFL=5
INTEGER(KIND=JPIM),PARAMETER:: JP_RSAFM=6
INTEGER(KIND=JPIM),PARAMETER:: JP_RSAHL=7
INTEGER(KIND=JPIM),PARAMETER:: JP_RSAHM=8
INTEGER(KIND=JPIM),PARAMETER:: JP_DIMR0=8
INTEGER(KIND=JPIM),PARAMETER:: JP_DIMR9=4
INTEGER(KIND=JPIM),PARAMETER:: JP_DIMR1=4
INTEGER(KIND=JPIM),PARAMETER:: JP_DIMR=8

END MODULE PAR_RDLR
