MODULE PARFPOS

USE PARKIND1  ,ONLY : JPIM

USE PARDIM

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------

! === basic dimensions for Full POST-PROCESSING ===

!     JPOSDOM : Maximum number of horizontal (sub)domains
!     JPOSLEN : Maximum length of a (sub)domain name
!     JPOSLIS : Maximum number of groups of subdomains
!     JPOSDIR : Maximum length of the path (or prefix) for the output files
!     JPOSLE  : Maximum number of eta levels on the output subdomain
!     JPOSGL  : Maximum number of latitude rows of the output gaussian grid

!     JPOSSCVA: Maximum number of post-processable passive scalars
!     JPOSPHY : Maximum number of physical fields
!     JPOSSGP : Maximum number of surface gridpoint fields
!     JPOSCFU : Maximum number of cumulated fluxes
!     JPOSXFU : Maximum number of instantaneous fluxes
!     JPOS3P  : Maximum number of pp. pressure levels
!     JPOS3H  : Maximum number of pp. height (above orography) levels
!     JPOS3TH : Maximum number of pp. potential temperature levels
!     JPOS3PV : Maximum number of pp. potential vorticity levels
!     JPOS3S  : Maximum number of pp. eta levels
!     JPOSSCVA: Maximum number of passive scalars
!     JPOSVX2 : Maximum number of free gp/sp upper air fields or extra surface fields
!     JPOSFSU : Maximum number of free gp/sp surface fields 
!     JPOS3DF : Maximum number of 3D dynamic fields
!     JPOS2DF : Maximum number of 2D dynamic fields
!     JPOSDYN : Maximum number of dynamic fields 
!     JPOSGHGFLX : Maximum number of post-processable greenhouse gases
!     JPOSTRACFLX : Maximum number of post-processable tracers (used for diagnostics only)
!     JPOSGRGFLX : Maximum number of post-processable reactive gases
!     JPOSGHG : Maximum number of post-processable greenhouse gases
!     JPOSTRAC : Maximum number of post-processable tracers (used for diagnostics only)
!     JPOSGRG : Maximum number of post-processable reactive gases
!     JPOSAERO: Maximum number of post-processable aerosols
!     JPOSERA40 : Maximum number of post-processable ERA40 GFL fields
!     JPOSNOGW  : Maximum number of post-processable noro gwd scheme fields
!     JPOSUVP : Maximum number of post-processable fields from UV processor
!     JPOSAEROUT: Maximum number of post-processable output aerosol fields

INTEGER(KIND=JPIM), PARAMETER :: JPOSERA40=14
INTEGER(KIND=JPIM), PARAMETER :: JPOSNOGW=2
INTEGER(KIND=JPIM), PARAMETER :: JPOSDOM=15
INTEGER(KIND=JPIM), PARAMETER :: JPOSLEN=10
INTEGER(KIND=JPIM), PARAMETER :: JPOSLIS=10
INTEGER(KIND=JPIM), PARAMETER :: JPOSDIR=180
INTEGER(KIND=JPIM), PARAMETER :: JPOSLE=JPMXLE
INTEGER(KIND=JPIM), PARAMETER :: JPOSGL=JPMXGL

INTEGER(KIND=JPIM), PARAMETER :: JPOSCFU=63
INTEGER(KIND=JPIM), PARAMETER :: JPOSXFU=63
INTEGER(KIND=JPIM), PARAMETER :: JPOS3P=75
INTEGER(KIND=JPIM), PARAMETER :: JPOS3H=127
INTEGER(KIND=JPIM), PARAMETER :: JPOS3TH=75
INTEGER(KIND=JPIM), PARAMETER :: JPOS3PV=75
INTEGER(KIND=JPIM), PARAMETER :: JPOS3S=JPOSLE
INTEGER(KIND=JPIM), PARAMETER :: JPOSSCVA=5
INTEGER(KIND=JPIM), PARAMETER :: JPOSGHG=3
INTEGER(KIND=JPIM), PARAMETER :: JPOSGHGFLX=6
INTEGER(KIND=JPIM), PARAMETER :: JPOSTRAC=10
INTEGER(KIND=JPIM), PARAMETER :: JPOSTRACFLX=10
INTEGER(KIND=JPIM), PARAMETER :: JPOSGRG=5
INTEGER(KIND=JPIM), PARAMETER :: JPOSGRGFLX=5
INTEGER(KIND=JPIM), PARAMETER :: JPOSAERO=15
INTEGER(KIND=JPIM), PARAMETER :: JPOSAERO2=2*JPOSAERO
INTEGER(KIND=JPIM), PARAMETER :: JPOSAEROUT=8
INTEGER(KIND=JPIM), PARAMETER :: JPOSUVP=2
INTEGER(KIND=JPIM), PARAMETER :: JPOSVX2=255
! 15 + 1 (Soiltype) +2 (lai) +2(runoff)
! + 2 (Non-Hydro) + 4 (Modis-Albedo) + 3 (ocean T)
! 14 = 5 + 2 Anton + 5 Philippe + 2 (direct SW)
! + 5 for Tomas
INTEGER(KIND=JPIM), PARAMETER :: JPOSFSU=28
INTEGER(KIND=JPIM), PARAMETER :: JPOSSGP=172+JPOSVX2+JPOSFSU+JPOSGHGFLX+JPOSTRACFLX+JPOSGRGFLX+10+5+2+2+3+2+5
INTEGER(KIND=JPIM), PARAMETER :: JPOSPHY=JPOSSGP+JPOSCFU+JPOSXFU
INTEGER(KIND=JPIM), PARAMETER :: &
 & JPOS3DF=63+JPOSSCVA+JPOSVX2+JPOSAERO+JPOSGHG+JPOSGRG+JPOSTRAC+JPOSERA40+JPOSNOGW &
 &+JPOSUVP+JPOSAEROUT
INTEGER(KIND=JPIM), PARAMETER :: JPOS2DF=66+JPOSFSU+JPOSAERO2
INTEGER(KIND=JPIM), PARAMETER :: JPOSDYN=JPOS3DF+JPOS2DF

!     ------------------------------------------------------------------
END MODULE PARFPOS
