MODULE QACVEG

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!****--------------------------------------------------------------------------
!****   QACVEG : Contient les variables controlant l'analyse de
!****   ------   surface associee a ISBA
!****
!****   Auteur : D. Giard    03/94
!****      Mod : E. Bazile   01/97 Retrait du biais moyen de temperature
!****                               et/ou d'humidite
!****            D. Giard    06/97 Differents rappels clim.
!****            F.Taillefer 02/98 Transfert RINCW et RBIAx dans GPPBUF
!****       S. Ivatek-Sahdan 09/02 Smoothing of SWI and changing Wp
!****--------------------------------------------------------------------------

! coefficients definissant la dependance des coefficients d'interpolation
! optimale de l'analyse d'humidite par rapport aux caracteristiques de surface,
! pour une heure donnee de la journee de reference (1<=i<=24)
! vg= indice de vegetation   h=indice foliaire/resistance stomatique minimale
! ws/T2m  : (1-vg)*(aT1+aT2*vg+aT3*vg**2) *(wfc-wwilt)/dwr
! ws/Hu2m : (1-vg)*(aH1+aH2*vg+aH3*vg**2) *(wfc-wwilt)/dwr
! wp/T2m  : [(1-vg)*(bT1+bT2*vg+bT3*vg**2)+h*vg*(cT1+cT2*vg)] *(wfc-wwilt)/dwr
! wp/Hu2m : [(1-vg)*(bT1+bH2*vg+bH3*vg**2)+h*vg*(cH1+cH2*vg)] *(wfc-wwilt)/dwr
! VGAT1 , VGAT2 , VGAT3
! VGAH1 , VGAH2 , VGAH3
! VGBT1 , VGBT2 , VGBT3 , VGCT1 , VGCT2
! VGBH1 , VGBH2 , VGBH3 , VGCH1 , VGCH2

! ecarts-type d'erreurs de "prevision",
! pour une heure donnee de la journee de reference (1<=i<=24)
! T2m  -> SIGT2MP
! Hu2m -> sigH2mp= SIGHP1+SIGHP2*vg  depend des caracteristiques de surface

! ecarts-type d'erreurs d'observation et texture de reference
! T2m  -> SIGT2MR
! Hu2m -> SIGH2MR
! ADWR -> (wfc-wwilt) (ref)

! ecarts-type d'erreurs d'observation
! T2m  -> SIGT2MO
! Hu2m -> SIGH2MO
! LSGOBS : activee si l'une des valeurs ci-dessus differe de la reference

! seuils sur la force du vent a 10m , la duree du jour , les precipitations,
! ws/wfc, evaporation en (mm/jour), presence de glace
!   V10MX , MINDJ , SPRECIP , SWFC  , SEVAP , SICE , SMU0

! cles et coefficients de reglage
!     LIMVEG : activation de la limitation a wp > veg*wwilt
!     LHUMID : activation de la clef LIMVEG  ou du seuil SWFC limitee
!     LISSEW : activation du lissage des increments de wp (si NLISSEW=3)
!     NLISSEW: nombre de reseaux anterieurs utilisables pour le lissage
!     ANEBUL , NNEBUL : dependance par rapport a la nebulosite
!     SNEIGT , NNEIGT : dependance par rapport a la neige (T)
!     SNEIGW , NNEIGW : dependance par rapport a la neige (w)
! coefficients pour le retrait du biais de T2m et Hu2m
!     SCOEFT , SCOEFH : =0. pas de retrait / =1. increment mis a zero

! champs d'increments pour le lissage (transfert dans GPPBUF de RINCW)
!     RINCW(jrof,j)  j=1/2/3 pour reseau courant - 06h/12h/18h
!     CINCW(j) : nom des champs d'increments dans les fichiers ARPEGE

! biais moyen
!     RBIAT(jrof) : biais moyen de temperature }  tableaux supprimes et mis dans
!     RBIAH(jrof) : biais moyen d'humidite     }   GPPBUF (adresse: MCANRI0)

! reglage du rappel clim
!     RCLIMTS : coef. multiplicateur de RCLIMCA pour Ts
!     RCLIMTP : coef. multiplicateur de RCLIMCA pour Tp
!     RCLIMWS : coef. multiplicateur de RCLIMCA pour Ws
!     RCLIMWP : coef. multiplicateur de RCLIMCA pour Wp
!     RCLIMN  : impact de la fraction de neige
!     RCLIMV  : coef. multiplicateur sur desert pour Ws, Wp

! lissage spatial du SWI et modification de Wp a partir du SWI lisse
!     L_SM_WP : lissage spatial du SWI (Soil Wetness Index) et modif Wp
!     NR_SM_WP: nombre d'appel de la routine de lissage spatial du SWI
!     RA_SM_WP: distance caracteristique du lissage spatial (en metres)

!****--------------------------------------------------------------------------
REAL(KIND=JPRB) :: VGAT1(24)
REAL(KIND=JPRB) :: VGAT2(24)
REAL(KIND=JPRB) :: VGAT3(24)
REAL(KIND=JPRB) :: VGAH1(24)
REAL(KIND=JPRB) :: VGAH2(24)
REAL(KIND=JPRB) :: VGAH3(24)
REAL(KIND=JPRB) :: VGBT1(24)
REAL(KIND=JPRB) :: VGBT2(24)
REAL(KIND=JPRB) :: VGBT3(24)
REAL(KIND=JPRB) :: VGBH1(24)
REAL(KIND=JPRB) :: VGBH2(24)
REAL(KIND=JPRB) :: VGBH3(24)
REAL(KIND=JPRB) :: VGCT1(24)
REAL(KIND=JPRB) :: VGCT2(24)
REAL(KIND=JPRB) :: VGCH1(24)
REAL(KIND=JPRB) :: VGCH2(24)
REAL(KIND=JPRB) :: SIGT2MP(24)
REAL(KIND=JPRB) :: SIGHP1
REAL(KIND=JPRB) :: SIGHP2(24)
REAL(KIND=JPRB) :: SIGT2MR
REAL(KIND=JPRB) :: SIGH2MR
REAL(KIND=JPRB) :: ADWR
INTEGER(KIND=JPIM) :: NLISSEW
LOGICAL :: LSGOBS
LOGICAL :: LIMVEG
INTEGER(KIND=JPIM) :: MINDJ
REAL(KIND=JPRB) :: V10MX
REAL(KIND=JPRB) :: SPRECIP
REAL(KIND=JPRB) :: SWFC
LOGICAL :: LHUMID
LOGICAL :: LISSEW
REAL(KIND=JPRB) :: SIGT2MO
REAL(KIND=JPRB) :: SIGH2MO
REAL(KIND=JPRB) :: ANEBUL
INTEGER(KIND=JPIM) :: NNEBUL
REAL(KIND=JPRB) :: SNEIGT
INTEGER(KIND=JPIM) :: NNEIGT
REAL(KIND=JPRB) :: SNEIGW
INTEGER(KIND=JPIM) :: NNEIGW
REAL(KIND=JPRB) :: SCOEFT
REAL(KIND=JPRB) :: SCOEFH
REAL(KIND=JPRB) :: RCLIMTS
REAL(KIND=JPRB) :: RCLIMTP
REAL(KIND=JPRB) :: RCLIMWS
REAL(KIND=JPRB) :: RCLIMWP
REAL(KIND=JPRB) :: RCLIMN
REAL(KIND=JPRB) :: RCLIMV
REAL(KIND=JPRB) :: SEVAP
REAL(KIND=JPRB) :: SICE
REAL(KIND=JPRB) :: SMU0
LOGICAL :: L_SM_WP
INTEGER(KIND=JPIM) :: NR_SM_WP
REAL(KIND=JPRB) :: RA_SM_WP
!****--------------------------------------------------------------------------
END MODULE QACVEG
