MODULE QAREF

USE PARKIND1  ,ONLY : JPRB

USE QAPASS

IMPLICIT NONE

SAVE

!***  ------------------------------------------------------------------
!**    QAREF : PROFILS DE REFERENCE DES PARAMETRES DU MODELE STATISTIQUE
!***  ------------------------------------------------------------------
!     REF_STAT(x,1) : pression de reference
!     REF_STAT(x,2) : ecart-type d'erreur de reference pour le geopotentiel
!     REF_STAT(x,3) : ecart-type d'erreur de reference pour la temperature
!     REF_STAT(x,4) : ecart-type d'erreur de reference pour le vent
!     REF_STAT(x,5) : ecart-type d'erreur de reference pour l'humidite relative
!     REF_STAT(x,6) : portee verticale de reference
!     REF_STAT(x,7) : portee horizontale de reference

!     REF_S_SST     : ecart-type pour la SST
!     REF_S_SN      : ecart-type pour la neige
!     REF_S_T2      : ecart-type pour T2m
!     REF_S_H2      : ecart-type pour Hu2m
!     REF_S_V1      : ecart-type pour V10m

!     REF_A_SST     : portee horizontale de reference pour la SST
!     REF_A_SN      : portee horizontale de reference pour la neige
!     REF_A_T2      : portee horizontale de reference pour T2m
!     REF_A_H2      : portee horizontale de reference pour H2m
!     REF_A_VOR1    : portee horizontale de reference pour la vorticite a 10m
!     REF_A_DIV1    : portee horizontale de reference pour la divergence a 10m

!     REF_AP_SN     : portee "verticale" de reference pour la neige

!     REF_NU_BL     : coefficient d'ageostrophisme dans la couche limite
!     REF_KP_BL     : coefficient d'extension verticale pour la couche limite

!     REF_MU[1,2,3] : parametres dans l'expression du rapport E.T. vent divergent / E.T. vent rotationnel
!                     MU = (REF_MU1-MAX(0.,REF_MU2*(REF_MU3-90000.)/10000.))*SIN(LAT)
!     REF_NU[1,2,3] : parametres dans l'expression du coefficient d'ageostrophisme
!                     NU = (REF_NU1+MAX(0.,REF_NU2*(REF_NU3-90000.)/10000.))*MAX(COS(LAT),SQRT(3.)*0.5)
!     REF_PHUD      : rapport des portees horizontales de reference pour la divergence et le geopotentiel 
!     REF_PHHU      : rapport des portees horizontales de reference pour l'humidite relative et le geopotentiel 
!     REF_PVH       : rapport des portees verticales de reference pour l'humidite relative et le geopotentiel 
!     REF_COEFN     : coefficient multiplicatif des E.T. du geopotentiel pour les latitudes [+25,+90]
!     REF_COEFT     : coefficient multiplicatif des E.T. du geopotentiel pour les latitudes [-15,+15]
!     REF_COEFS     : coefficient multiplicatif des E.T. du geopotentiel pour les latitudes [-90,-25]
 
REAL(KIND=JPRB) :: REF_STAT(JPNVS,7)
REAL(KIND=JPRB) :: REF_S_SST, REF_S_SN, REF_S_T2, REF_S_H2, REF_S_V1,&
 & REF_A_SST, REF_A_SN, REF_A_T2, REF_A_H2,&
 & REF_A_VOR1,REF_A_DIV1,REF_NU_BL,REF_KP_BL,&
 & REF_MU1,REF_MU2,REF_MU3,REF_NU1,REF_NU2,REF_NU3,&
 & REF_PHUD,REF_PHHU,REF_PVH,REF_COEFN,REF_COEFT,REF_COEFS,&
 & REF_AP_SN  
!***  ------------------------------------------------------------------
END MODULE QAREF
