MODULE SPECTRAL_COLUMNS_MIX

!   Purpose.
!   --------
!     Provide data type to handle columns of spectral coefs.

!   Author.
!   -------
!     Yannick Tremolet

!   Modifications.
!   -------------- 
!     Original    02-Mar-04  
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMGRB  , ONLY: NGRBD, NGRBVO, NGRBT, NGRBQ, NGRBO3, NGRBLNSP
USE YOMDIM  , ONLY: NFLEVL, NFLEVG
USE YOMCT0  , ONLY: NPRTRV
USE YOMMP   , ONLY: MYSETV, NUMLL
USE INDEXFIND_MOD, ONLY: INDXFIND

IMPLICIT NONE
SAVE
PRIVATE
PUBLIC SPECTRAL_COLS, ALLOCATE_SPCOLS, DEALLOCATE_SPCOLS

! ------------------------------------------------------------------

TYPE SPECTRAL_COLS
  REAL(KIND=JPRB), POINTER :: SP3V(:,:,:)
  REAL(KIND=JPRB), POINTER :: SP2V(:,:)
  INTEGER(KIND=JPIM) :: NS2D
  INTEGER(KIND=JPIM) :: NS3D
  INTEGER(KIND=JPIM) :: NSIZEL
  INTEGER(KIND=JPIM) :: NFLEVL
  INTEGER(KIND=JPIM) :: NFLEVG
  INTEGER(KIND=JPIM) :: NSPEC2V
  INTEGER(KIND=JPIM), POINTER :: NUMLL(:)
  INTEGER(KIND=JPIM), POINTER :: NPTRLL(:)
  INTEGER(KIND=JPIM), POINTER :: NPTRSV(:)
  INTEGER(KIND=JPIM), POINTER :: NGRIB(:), NGRIB2(:), NGRIB3(:)
  REAL(KIND=JPRB), POINTER :: DIV(:,:)
  REAL(KIND=JPRB), POINTER :: VOR(:,:)
  REAL(KIND=JPRB), POINTER :: TEM(:,:)
  REAL(KIND=JPRB), POINTER :: HUM(:,:)
  REAL(KIND=JPRB), POINTER :: OZN(:,:)
  REAL(KIND=JPRB), POINTER :: SPR(:)
END TYPE SPECTRAL_COLS

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

SUBROUTINE allocate_spcols(ydcols,kspec2,ks3d,ks2d,kgrib)

TYPE(SPECTRAL_COLS), INTENT(OUT) :: ydcols
INTEGER(KIND=JPIM),  INTENT(IN)  :: kspec2, ks3d, ks2d, kgrib(ks3d+ks2d)

INTEGER(KIND=JPIM) :: indd, indv, indt, indq, indo, inds, ii, ir, jb
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SPECTRAL_COLUMNS_MIX:ALLOCATE_SPCOLS',0,ZHOOK_HANDLE)

ALLOCATE(ydcols%numll(NPRTRV+1))
ALLOCATE(ydcols%nptrll(NPRTRV+1))
ALLOCATE(ydcols%nptrsv(NPRTRV+1))
ALLOCATE(ydcols%ngrib(ks3d+ks2d))

ydcols%ns2d=ks2d
ydcols%ns3d=ks3d

ydcols%nflevl=nflevl
ydcols%nflevg=nflevg
ydcols%numll(:)=NUMLL(:)
ydcols%NPTRLL(1)=1
DO jb=1,NPRTRV
  ydcols%NPTRLL(jb+1)=ydcols%NPTRLL(jb)+ydcols%NUMLL(jb)
ENDDO

ii=kspec2/NPRTRV
ir=kspec2-ii*NPRTRV
ydcols%NPTRSV(1)=1
DO jb=1,ir
  ydcols%NPTRSV(jb+1)=ydcols%NPTRSV(jb)+(ii+1)
ENDDO
DO jb=ir+1,NPRTRV
  ydcols%NPTRSV(jb+1)=ydcols%NPTRSV(jb)+ii
ENDDO
ydcols%nspec2v=ydcols%NPTRSV(MYSETV+1)-ydcols%NPTRSV(MYSETV)

ydcols%ngrib(:)=kgrib(:)
ydcols%ngrib3 => ydcols%ngrib(1:ks3d)
ydcols%ngrib2 => ydcols%ngrib(ks3d+1:ks3d+ks2d)

ALLOCATE(ydcols%sp2v(ydcols%nspec2v,ydcols%ns2d))
ALLOCATE(ydcols%sp3v(ydcols%nflevg,ydcols%nspec2v,ydcols%ns3d))

ydcols%nsizel=SIZE(ydcols%sp3v)+SIZE(ydcols%sp2v)

NULLIFY(ydcols%DIV)
NULLIFY(ydcols%VOR)
NULLIFY(ydcols%TEM)
NULLIFY(ydcols%HUM)
NULLIFY(ydcols%OZN)
NULLIFY(ydcols%SPR)
indd=INDXFIND(ydcols%ngrib3,NGRBD)
indv=INDXFIND(ydcols%ngrib3,NGRBVO)
indt=INDXFIND(ydcols%ngrib3,NGRBT)
indq=INDXFIND(ydcols%ngrib3,NGRBQ)
indo=INDXFIND(ydcols%ngrib3,NGRBO3)
IF (indd>0) ydcols%DIV => ydcols%sp3v(:,:,indd)
IF (indv>0) ydcols%VOR => ydcols%sp3v(:,:,indv)
IF (indt>0) ydcols%TEM => ydcols%sp3v(:,:,indt)
IF (indq>0) ydcols%HUM => ydcols%sp3v(:,:,indq)
IF (indo>0) ydcols%OZN => ydcols%sp3v(:,:,indo)

inds=INDXFIND(ydcols%ngrib2,NGRBLNSP)
IF (inds>0) ydcols%SPR => ydcols%sp2v(:,inds)

IF (LHOOK) CALL DR_HOOK('SPECTRAL_COLUMNS_MIX:ALLOCATE_SPCOLS',1,ZHOOK_HANDLE)
END SUBROUTINE allocate_spcols

! ------------------------------------------------------------------

SUBROUTINE deallocate_spcols(ydcols)

TYPE(SPECTRAL_COLS), INTENT(INOUT) :: ydcols
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SPECTRAL_COLUMNS_MIX:DEALLOCATE_SPCOLS',0,ZHOOK_HANDLE)
NULLIFY(ydcols%div)
NULLIFY(ydcols%vor)
NULLIFY(ydcols%tem)
NULLIFY(ydcols%hum)
NULLIFY(ydcols%ozn)
NULLIFY(ydcols%spr)
NULLIFY(ydcols%ngrib2)
NULLIFY(ydcols%ngrib3)

IF (ASSOCIATED(ydcols%sp2v  )) DEALLOCATE(ydcols%sp2v)
IF (ASSOCIATED(ydcols%sp3v  )) DEALLOCATE(ydcols%sp3v)
IF (ASSOCIATED(ydcols%numll )) DEALLOCATE(ydcols%numll)
IF (ASSOCIATED(ydcols%nptrll)) DEALLOCATE(ydcols%nptrll)
IF (ASSOCIATED(ydcols%ngrib )) DEALLOCATE(ydcols%ngrib)
IF (ASSOCIATED(ydcols%nptrsv)) DEALLOCATE(ydcols%nptrsv)

IF (LHOOK) CALL DR_HOOK('SPECTRAL_COLUMNS_MIX:DEALLOCATE_SPCOLS',1,ZHOOK_HANDLE)
END SUBROUTINE deallocate_spcols

! ------------------------------------------------------------------

END MODULE SPECTRAL_COLUMNS_MIX
