MODULE TRAJ_PHYSICS_MOD

!     Purpose.
!     --------
!       Manage physics trajectory

!     Author.
!     -------
!        Y. Tremolet *ECMWF*

!     Modifications.
!     --------------
!        Original : 01-03-02
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y. Tremolet: 10-10-03 fix problem with first time step.
!        M.Hamrud      10-Mar-2004 Fix bug with traj_zero
!        Y.Tremolet    18-Mar-2004 Add lprttraj option for diagnostics
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN , ONLY: NULOUT
USE YOMDIM , ONLY: NPROMA, NGPBLKS, NFLEVG
USE YOMDPHY, ONLY: NTILES
USE SURFACE_FIELDS_MIX  , ONLY : YSP_SBD
USE YOMCT0 , ONLY: LIFSTRAJ, NSTART
USE YOMTRAJ, ONLY: NSTEPTRAJ, NTRAJP, MSTEPTRAJW, MSTEPTRAJR, &
 & TRAJ_PHYS, TRAJ_ZERO, LPRTTRAJ
USE YOM_YGFL , ONLY : YGFL, NGHG, YGHG, NGRG, YGRG, NAERO, YAERO, &
 & YLRCH4, YLRCH4_NL

IMPLICIT NONE

PRIVATE
PUBLIC ALLOCATE_TRAJ_PHYS, DEALLOCATE_TRAJ_PHYS, &
 & STORE_TRAJ_PHYS, GET_TRAJ_PHYS 
 
#include "abor1.intfb.h"

!-----------------------------------------------------------------------
CONTAINS
!-----------------------------------------------------------------------

SUBROUTINE ALLOCATE_TRAJ_PHYS

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:ALLOCATE_TRAJ_PHYS',0,ZHOOK_HANDLE)
NTRAJP=9*NFLEVG+2*(NFLEVG+1)+3*YSP_SBD%NLEVS+5*NTILES+10+2*NGHG*NFLEVG+2*NGRG*NFLEVG+ &
     & 2*NAERO*NFLEVG
IF(NGHG > 0 .AND. YLRCH4_NL%LGP) NTRAJP=NTRAJP+NFLEVG !Methane loss rate


IF (LIFSTRAJ) THEN
  CALL ABOR1('Cannot interpolate physics trajectory')
ELSE
  ALLOCATE(TRAJ_PHYS(NPROMA,NTRAJP,NGPBLKS,NSTEPTRAJ))
  WRITE(NULOUT,9)'TRAJ_PHYS ',SIZE(TRAJ_PHYS),SHAPE(TRAJ_PHYS)
  IF (MSTEPTRAJW(nstart+1)<0) THEN
    ALLOCATE(TRAJ_ZERO(NPROMA,NTRAJP,NGPBLKS))
    WRITE(NULOUT,9)'TRAJ_ZERO ',SIZE(TRAJ_ZERO),SHAPE(TRAJ_ZERO)
  ENDIF
ENDIF

9 FORMAT(1X,'GREPTRAJ ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:ALLOCATE_TRAJ_PHYS',1,ZHOOK_HANDLE)
END SUBROUTINE ALLOCATE_TRAJ_PHYS

!-----------------------------------------------------------------------

SUBROUTINE DEALLOCATE_TRAJ_PHYS

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:DEALLOCATE_TRAJ_PHYS',0,ZHOOK_HANDLE)
IF (ALLOCATED(TRAJ_PHYS)) DEALLOCATE(TRAJ_PHYS)
IF (ALLOCATED(TRAJ_ZERO)) DEALLOCATE(TRAJ_ZERO)

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:DEALLOCATE_TRAJ_PHYS',1,ZHOOK_HANDLE)
END SUBROUTINE DEALLOCATE_TRAJ_PHYS

!-----------------------------------------------------------------------

SUBROUTINE STORE_TRAJ_PHYS(PTT95,PQT95,PGFLT95,PUT95,PVT95,PVVEL5,PSPT95,PSPT15, &
 & PTET5,PTEQ5,PTEGFL5,PTEU5,PTEV5,PEMTED5,PTRSOL5,PWSA95, &
 & PTSA95,PSNS95,PWL95,PTL95,PTIA95,PNTOP5,PNBAS5, &
 & PACPR5,PGZ0M5,PLZ0H5,PUSTRTI5,PVSTRTI5,PAHFSTI5, &
 & PEVAPTI5,PTSKTI5,KSTEP,KBLK,KSTA,KEND)  

INTEGER(KIND=JPIM),INTENT(IN) :: KBLK,KSTEP,KSTA,KEND
REAL(KIND=JPRB),INTENT(IN) :: PTT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PQT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PGFLT95 (NPROMA,NFLEVG,YGFL%NDIM)
REAL(KIND=JPRB),INTENT(IN) :: PUT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PVT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PSPT95  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PSPT15  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PTEQ5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PTEGFL5 (NPROMA,NFLEVG,YGFL%NDIM1)
REAL(KIND=JPRB),INTENT(IN) :: PTET5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PTEU5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PTEV5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PVVEL5  (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PEMTED5 (NPROMA,NFLEVG+1)
REAL(KIND=JPRB),INTENT(IN) :: PTRSOL5 (NPROMA,NFLEVG+1)
REAL(KIND=JPRB),INTENT(IN) :: PWSA95  (NPROMA,YSP_SBD%NLEVS)
REAL(KIND=JPRB),INTENT(IN) :: PTSA95  (NPROMA,YSP_SBD%NLEVS)
REAL(KIND=JPRB),INTENT(IN) :: PTIA95  (NPROMA,YSP_SBD%NLEVS)
REAL(KIND=JPRB),INTENT(IN) :: PSNS95  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PWL95   (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PTL95   (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PNTOP5  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PNBAS5  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PACPR5  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PGZ0M5  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PLZ0H5  (NPROMA)
REAL(KIND=JPRB),INTENT(IN) :: PUSTRTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(IN) :: PVSTRTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(IN) :: PAHFSTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(IN) :: PEVAPTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(IN) :: PTSKTI5 (NPROMA,NTILES)

INTEGER(KIND=JPIM) :: jj,jf,istep,jgfl
INTEGER(KIND=JPIM), SAVE :: IISTEP1, IISTEP2
REAL(KIND=JPRB) :: ZHOOK_HANDLE
DATA IISTEP1/-99/
DATA IISTEP2/-99/

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:STORE_TRAJ_PHYS',0,ZHOOK_HANDLE)
IF (LIFSTRAJ) CALL ABOR1('Cannot interpolate physics trajectory')

istep=MSTEPTRAJW(kstep)
IF ((kstep==nstart+1).AND.(istep<0)) istep=MSTEPTRAJW(nstart)

IF (istep>0) THEN
  jj=0

!       1.      UPPER AIR 3D FIELDS
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTT95 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PQT95 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PUT95 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PVT95 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTET5 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTEQ5 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTEU5 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTEV5 (ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PVVEL5(ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG+1
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PEMTED5(ksta:kend,jf)
  ENDDO
  DO jf=1,NFLEVG+1
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTRSOL5(ksta:kend,jf)
  ENDDO
  DO JGFL=1,NGHG
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PGFLT95(ksta:kend,jf,YGHG(JGFL)%MP9_PH)
    ENDDO
  ENDDO
  DO JGFL=1,NGHG
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTEGFL5(ksta:kend,jf,YGHG(JGFL)%MP1)
    ENDDO
  ENDDO
  IF(NGHG > 0 .AND. YLRCH4_NL%LGP) THEN
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PGFLT95(ksta:kend,jf,YLRCH4%MP9_PH)
    ENDDO
  ENDIF
  DO JGFL=1,NGRG
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PGFLT95(ksta:kend,jf,YGRG(JGFL)%MP9_PH)
    ENDDO
  ENDDO
  DO JGFL=1,NGRG
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTEGFL5(ksta:kend,jf,YGRG(JGFL)%MP1)
    ENDDO
  ENDDO
  DO JGFL=1,NAERO
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PGFLT95(ksta:kend,jf,YAERO(JGFL)%MP9_PH)
    ENDDO
  ENDDO
  DO JGFL=1,NAERO
    DO jf=1,NFLEVG
      jj=jj+1
      TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTEGFL5(ksta:kend,jf,YAERO(JGFL)%MP1)
    ENDDO
  ENDDO

!       2.      SOIL 3D FIELDS
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTSA95(ksta:kend,jf)
  ENDDO
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PWSA95(ksta:kend,jf)
  ENDDO
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTIA95(ksta:kend,jf)
  ENDDO

!       3.     SURFACE 2D FIELDS
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PSPT95(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PSPT15(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PSNS95(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PWL95(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTL95(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PNTOP5(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PNBAS5(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PACPR5(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PGZ0M5(ksta:kend)
  jj=jj+1
  TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PLZ0H5(ksta:kend)

!       4.      SOIL 2D FIELDS (TILES)
  DO jf=1,NTILES
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PUSTRTI5(ksta:kend,jf)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PVSTRTI5(ksta:kend,jf)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PAHFSTI5(ksta:kend,jf)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PEVAPTI5(ksta:kend,jf)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    TRAJ_PHYS(ksta:kend,jj,kblk,istep)=PTSKTI5 (ksta:kend,jf)
  ENDDO

!$OMP MASTER
  IF (LPRTTRAJ.AND.IISTEP1/=ISTEP) THEN
    WRITE(NULOUT,*)'GREPTRAJ STORE_TRAJ_PHYS istep=',istep
    IISTEP1=ISTEP
  ENDIF
!$OMP END MASTER

  IF(JJ > NTRAJP) THEN
    CALL ABOR1('TRAJ_PHYSICS_MOD:STORE_TRAJ_PHYS - NTRAJP TOO SMALL')
  ENDIF
ENDIF
IF ((kstep==nstart).AND.(MSTEPTRAJW(nstart+1)<0)) THEN
  IF(.NOT. ALLOCATED(TRAJ_ZERO)) THEN
    CALL ABOR1('TRAJ_PHYSICS_MOD:STORE_TRAJ_PHYS - TRAJ_ZERO NOT ALLOCATED')
  ENDIF
  TRAJ_ZERO(ksta:kend,:,kblk)=TRAJ_PHYS(ksta:kend,:,kblk,istep)
  IF (LPRTTRAJ.AND.IISTEP2/=ISTEP) THEN
    WRITE(NULOUT,*)'GREPTRAJ STORE_TRAJ_PHYS set TRAJ_ZERO'
    IISTEP2=ISTEP
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:STORE_TRAJ_PHYS',1,ZHOOK_HANDLE)
END SUBROUTINE STORE_TRAJ_PHYS

!-----------------------------------------------------------------------

SUBROUTINE GET_TRAJ_PHYS(PTT95,PQT95,PGFLT95,PUT95,PVT95,PVVEL5,PSPT95,PSPT15, &
 & PTET5,PTEQ5,PTEGFL5,PTEU5,PTEV5,PEMTED5,PTRSOL5,PWSA95, &
 & PTSA95,PSNS95,PWL95,PTL95,PTIA95,PNTOP5,PNBAS5, &
 & PACPR5,PGZ0M5,PLZ0H5,PUSTRTI5,PVSTRTI5,PAHFSTI5, &
 & PEVAPTI5,PTSKTI5,KSTEP,KBLK,KSTA,KEND)  

INTEGER(KIND=JPIM),INTENT(IN) :: KBLK,KSTEP,KSTA,KEND
REAL(KIND=JPRB),INTENT(OUT) :: PTT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PQT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PUT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PGFLT95 (NPROMA,NFLEVG,YGFL%NDIM)
REAL(KIND=JPRB),INTENT(OUT) :: PVT95   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PSPT95  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PSPT15  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PTEQ5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PTEGFL5 (NPROMA,NFLEVG,YGFL%NDIM1)
REAL(KIND=JPRB),INTENT(OUT) :: PTET5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PTEU5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PTEV5   (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PVVEL5  (NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(OUT) :: PEMTED5 (NPROMA,NFLEVG+1)
REAL(KIND=JPRB),INTENT(OUT) :: PTRSOL5 (NPROMA,NFLEVG+1)
REAL(KIND=JPRB),INTENT(OUT) :: PWSA95  (NPROMA,YSP_SBD%NLEVS)
REAL(KIND=JPRB),INTENT(OUT) :: PTSA95  (NPROMA,YSP_SBD%NLEVS)
REAL(KIND=JPRB),INTENT(OUT) :: PTIA95  (NPROMA,YSP_SBD%NLEVS)
REAL(KIND=JPRB),INTENT(OUT) :: PSNS95  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PWL95   (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PTL95   (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PNTOP5  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PNBAS5  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PACPR5  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PGZ0M5  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PLZ0H5  (NPROMA)
REAL(KIND=JPRB),INTENT(OUT) :: PUSTRTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(OUT) :: PVSTRTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(OUT) :: PAHFSTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(OUT) :: PEVAPTI5(NPROMA,NTILES)
REAL(KIND=JPRB),INTENT(OUT) :: PTSKTI5 (NPROMA,NTILES)

INTEGER(KIND=JPIM) :: jj,jf,istep,jgfl
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:GET_TRAJ_PHYS',0,ZHOOK_HANDLE)
istep=MSTEPTRAJR(kstep)

jj=0

IF ((kstep==nstart).AND.(MSTEPTRAJW(nstart+1)<0)) THEN

!       1.      UPPER AIR 3D FIELDS
  DO jf=1,NFLEVG
    jj=jj+1
    PTT95 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PQT95 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PUT95 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PVT95 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTET5 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTEQ5 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTEU5 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTEV5 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PVVEL5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG+1
    jj=jj+1
    PEMTED5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NFLEVG+1
    jj=jj+1
    PTRSOL5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO JGFL=1,NGHG
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YGHG(JGFL)%MP9_PH)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDDO
  DO JGFL=1,NGHG
    DO jf=1,NFLEVG
      jj=jj+1
      PTEGFL5 (ksta:kend,jf,YGHG(JGFL)%MP1)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDDO
  IF(NGHG > 0 .AND. YLRCH4_NL%LGP) THEN
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YLRCH4%MP9_PH)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDIF
  DO JGFL=1,NGRG
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YGRG(JGFL)%MP9_PH)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDDO
  DO JGFL=1,NGRG
    DO jf=1,NFLEVG
      jj=jj+1
      PTEGFL5 (ksta:kend,jf,YGRG(JGFL)%MP1)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDDO
  DO JGFL=1,NAERO
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YAERO(JGFL)%MP9_PH)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDDO
  DO JGFL=1,NAERO
    DO jf=1,NFLEVG
      jj=jj+1
      PTEGFL5 (ksta:kend,jf,YAERO(JGFL)%MP1)=TRAJ_ZERO(ksta:kend,jj,kblk)
    ENDDO
  ENDDO

!       2.      SOIL 3D FIELDS
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    PTSA95(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    PWSA95(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    PTIA95(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO

!       3.     SURFACE 2D FIELDS
  jj=jj+1
  PSPT95(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PSPT15(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PSNS95(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PWL95(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PTL95(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PNTOP5(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PNBAS5(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PACPR5(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PGZ0M5(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)
  jj=jj+1
  PLZ0H5(ksta:kend)=TRAJ_ZERO(ksta:kend,jj,kblk)

!       4.      SOIL 2D FIELDS (TILES)
  DO jf=1,NTILES
    jj=jj+1
    PUSTRTI5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PVSTRTI5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PAHFSTI5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PEVAPTI5(ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PTSKTI5 (ksta:kend,jf)=TRAJ_ZERO(ksta:kend,jj,kblk)
  ENDDO

ELSE

!       1.      UPPER AIR 3D FIELDS
  DO jf=1,NFLEVG
    jj=jj+1
    PTT95 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PQT95 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PUT95 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PVT95 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTET5 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTEQ5 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTEU5 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PTEV5 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG
    jj=jj+1
    PVVEL5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG+1
    jj=jj+1
    PEMTED5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NFLEVG+1
    jj=jj+1
    PTRSOL5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO JGFL=1,NGHG
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YGHG(JGFL)%MP9_PH)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDDO
  DO JGFL=1,NGHG
    DO jf=1,NFLEVG
      jj=jj+1
      PTEGFL5 (ksta:kend,jf,YGHG(JGFL)%MP1)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDDO
  IF(NGHG > 0 .AND. YLRCH4_NL%LGP) THEN
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YLRCH4%MP9_PH)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDIF
  DO JGFL=1,NGRG
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YGRG(JGFL)%MP9_PH)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDDO
  DO JGFL=1,NGRG
    DO jf=1,NFLEVG
      jj=jj+1
      PTEGFL5 (ksta:kend,jf,YGRG(JGFL)%MP1)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDDO
  DO JGFL=1,NAERO
    DO jf=1,NFLEVG
      jj=jj+1
      PGFLT95 (ksta:kend,jf,YAERO(JGFL)%MP9_PH)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDDO
  DO JGFL=1,NAERO
    DO jf=1,NFLEVG
      jj=jj+1
      PTEGFL5 (ksta:kend,jf,YAERO(JGFL)%MP1)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
    ENDDO
  ENDDO

!       2.      SOIL 3D FIELDS
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    PTSA95(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    PWSA95(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,YSP_SBD%NLEVS
    jj=jj+1
    PTIA95(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO

!       3.     SURFACE 2D FIELDS
  jj=jj+1
  PSPT95(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PSPT15(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PSNS95(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PWL95(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PTL95(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PNTOP5(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PNBAS5(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PACPR5(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PGZ0M5(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  jj=jj+1
  PLZ0H5(ksta:kend)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)

!       4.      SOIL 2D FIELDS (TILES)
  DO jf=1,NTILES
    jj=jj+1
    PUSTRTI5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PVSTRTI5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PAHFSTI5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PEVAPTI5(ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO
  DO jf=1,NTILES
    jj=jj+1
    PTSKTI5 (ksta:kend,jf)=TRAJ_PHYS(ksta:kend,jj,kblk,istep)
  ENDDO

ENDIF
IF(JJ > NTRAJP) THEN
  CALL ABOR1('TRAJ_PHYSICS_MOD:GET_TRAJ_PHYS - NTRAJP TOO SMALL')
ENDIF
IF (LPRTTRAJ) WRITE(NULOUT,*)'GREPTRAJ GET_TRAJ_PHYS istep=',istep

IF (LHOOK) CALL DR_HOOK('TRAJ_PHYSICS_MOD:GET_TRAJ_PHYS',1,ZHOOK_HANDLE)
END SUBROUTINE GET_TRAJ_PHYS

!-----------------------------------------------------------------------

END MODULE TRAJ_PHYSICS_MOD
