MODULE TRAJ_SURFACE_MOD

!     Purpose.
!     --------
!       Manage surface trajectory

!     Author.
!     -------
!        Y. Tremolet *ECMWF*

!     Modifications.
!     --------------
!        Original : 2000-11-23
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Tremolet    18-Mar-2004 Add lprttraj option for diagnostics
!        G. Desroziers and K. Yessad (sept 2005):
!         - split option LTRAJHR into LTRAJHR_ALTI and LTRAJHR_SURF.
!         - adapt option LTRAJHR to METEO-FRANCE configurations.
!        M.Hamrud      01-Jul-2006  Revised surface fields
!        Y.Tremolet    27-Nov-2008 Long window 4D-Var
!     ------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT,NULERR
USE YOMDIM   , ONLY : NPROMA, NGPBLKS, NSPEC2
USE YOMVAR   , ONLY : NUPTRA
USE YOMMODEL_ERROR, ONLY: N_COUPLED_WINDOWS
USE SURFACE_FIELDS_MIX , ONLY : GPOPER
USE YOMCT0   , ONLY : LIFSTRAJ, NINTERPTRAJ, LARPEGEF_TRAJHR,NSTOP
USE YOMTRAJ, ONLY: NGP5, NTRAJ_CST, TRAJ_SRFC, TRAJ_CST, &
 & MSTEPTRAJW, MSTEPTRAJR, LTRAJCST, TRAJ_BUF, LPRTTRAJ, &
 & NSTEPTRAJ, MTRAJ_GRIB,MTYPE_SURF_TRAJ,MIOTRAJSURF
USE YOMOP    , ONLY : CFNTRAJHRSURF

IMPLICIT NONE

PRIVATE
PUBLIC ALLOCATE_TRAJ_SFC, DEALLOCATE_TRAJ_SFC,FORCE_SFC_TRAJ, &
 & STORE_TRAJ_SFC, STORE_TRAJ_CST, GET_TRAJ_SFC, READ_TRAJ_SFC  

#include "read_grid_traj.intfb.h"
#include "read_surfgrid_traj_fromfa.intfb.h"
#include "write_grid_traj.intfb.h"
#include "abor1.intfb.h"

!-----------------------------------------------------------------------
CONTAINS
!-----------------------------------------------------------------------

SUBROUTINE ALLOCATE_TRAJ_SFC

INTEGER(KIND=JPIM) :: J1,J2,J3
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:ALLOCATE_TRAJ_SFC',0,ZHOOK_HANDLE)
IF (lifstraj) THEN
  ALLOCATE(TRAJ_SRFC(NPROMA,NGP5,NGPBLKS,1))
! TRAJ_SRFC(:,:,:,:)=0.0   ! For copy into TRAJ_BUF
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J3,J2,J1)
  DO J3=1,NGPBLKS
    DO J2=1,NGP5
      DO J1=1,NPROMA
        TRAJ_SRFC(J1,J2,J3,1)=0.0   ! For copy into TRAJ_BUF
      ENDDO
    ENDDO
  ENDDO
!$OMP END PARALLEL DO
ELSE
  ALLOCATE(TRAJ_SRFC(NPROMA,NGP5,NGPBLKS,nsteptraj))
ENDIF
WRITE(NULOUT,9)'TRAJ_SRFC ',SIZE(TRAJ_SRFC),SHAPE(TRAJ_SRFC)
IF (LTRAJCST) THEN
  ALLOCATE(TRAJ_CST(NPROMA,NTRAJ_CST,NGPBLKS))
  WRITE(NULOUT,9)'TRAJ_CST  ',SIZE(TRAJ_CST ),SHAPE(TRAJ_CST)
ENDIF
ALLOCATE(MIOTRAJSURF(NSTEPTRAJ))
MIOTRAJSURF(:) = -1
WRITE(NULOUT,9)'MIOTRAJSURF',SIZE(MIOTRAJSURF),SHAPE(MIOTRAJSURF)

9 FORMAT(1X,'GREPTRAJ ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:ALLOCATE_TRAJ_SFC',1,ZHOOK_HANDLE)
END SUBROUTINE ALLOCATE_TRAJ_SFC

!-----------------------------------------------------------------------

SUBROUTINE DEALLOCATE_TRAJ_SFC

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:DEALLOCATE_TRAJ_SFC',0,ZHOOK_HANDLE)
IF (ALLOCATED(TRAJ_SRFC)) DEALLOCATE(TRAJ_SRFC)
IF (ALLOCATED(TRAJ_CST )) DEALLOCATE(TRAJ_CST)
IF (ALLOCATED(MTYPE_SURF_TRAJ)) DEALLOCATE(MTYPE_SURF_TRAJ)
IF (ALLOCATED(MIOTRAJSURF)) DEALLOCATE(MIOTRAJSURF)

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:DEALLOCATE_TRAJ_SFC',1,ZHOOK_HANDLE)
END SUBROUTINE DEALLOCATE_TRAJ_SFC

!-----------------------------------------------------------------------

SUBROUTINE STORE_TRAJ_SFC(KSTEP,KBLK,KSTA,KEND)

INTEGER(KIND=JPIM),INTENT(IN) :: KBLK,KSTEP,KSTA,KEND
INTEGER(KIND=JPIM) :: istep,idummy(1),jstep,istept
CHARACTER(LEN=20) :: clfile
LOGICAL :: llfirst,llast
REAL(KIND=JPRB) :: ZTEMP(NPROMA,NGP5)
REAL(KIND=JPRB) :: ZDUMMY(NSPEC2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:STORE_TRAJ_SFC',0,ZHOOK_HANDLE)

istep=MSTEPTRAJW(kstep)
istept=istep
IF (lifstraj) istept=min(istep,1)

IF (istep>0) THEN
  IF (istep == 1) THEN
    llfirst = .TRUE.
  ELSE
    llfirst = .FALSE.
  ENDIF
  llast=.true.
  DO JSTEP=kstep+1,nstop
    IF(MSTEPTRAJW(JSTEP) > 0) llast=.false.
  ENDDO

  CALL GPOPER('TRAJSTORE',KBL=KBLK,PFIELD=ZTEMP)
  TRAJ_SRFC(KSTA:KEND,:,kblk,istept) = ZTEMP(KSTA:KEND,:)
  IF (lifstraj .AND. (kblk==NGPBLKS)) THEN
    IF(MIOTRAJSURF(istep)== -1) THEN
      DO JSTEP=1,istep-1
        IF(MIOTRAJSURF(jstep) /= jstep)THEN
          WRITE(NULERR,*)'HOLE IN SURF TRAJ',jstep,ISTEP,MIOTRAJSURF(jstep)
          CALL ABOR1('STORE_TRAJ_SURF:HOLE')
        ENDIF
      ENDDO
    CLFILE(1:20)=CFNTRAJHRSURF
    write(clfile( 7: 8),'(I2.2)')nuptra
      write(clfile(18:20),'(I3.3)') 0
    write(nulout,*)'GREPTRAJ Saving surface trajectory in ',clfile

    ALLOCATE(TRAJ_BUF(NPROMA,NGP5,NGPBLKS))
      TRAJ_BUF(:,:,:)=TRAJ_SRFC(:,:,:,istept)
    CALL WRITE_GRID_TRAJ(clfile,mtraj_grib,idummy,mtype_surf_traj, &
       & 0,ngp5,TRAJ_BUF,ZDUMMY,NINTERPTRAJ,llfirst,llast)
      MIOTRAJSURF(ISTEP)=ISTEP
    DEALLOCATE(TRAJ_BUF)
    ELSE
      WRITE(NULOUT,*)'GREPTRAJ Warning: Surf traj at step,',KSTEP,ISTEP,&
       & 'already written'  
  ENDIF
 
  ENDIF
  IF (LPRTTRAJ) WRITE(NULOUT,*)'GREPTRAJ STORE_TRAJ_SFC istep=',istep
ENDIF

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:STORE_TRAJ_SFC',1,ZHOOK_HANDLE)
END SUBROUTINE STORE_TRAJ_SFC

!-----------------------------------------------------------------------

SUBROUTINE STORE_TRAJ_CST(KSTEP,KBLK,KSTA,KEND)

INTEGER(KIND=JPIM),INTENT(IN) :: KBLK,KSTEP,KSTA,KEND
INTEGER(KIND=JPIM) :: istep
REAL(KIND=JPRB) :: ZTEMP(NPROMA,NTRAJ_CST)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL :: LLFIRST
DATA LLFIRST /.TRUE./

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:STORE_TRAJ_CST',0,ZHOOK_HANDLE)
istep=MSTEPTRAJW(kstep)
IF (istep==1) THEN
  CALL GPOPER('TRAJSTORECST',KBL=KBLK,PFIELD=ZTEMP)
  TRAJ_CST(KSTA:KEND,:,kblk) = ZTEMP(KSTA:KEND,:)
  IF (LPRTTRAJ.AND.LLFIRST) THEN
    WRITE(NULOUT,*)'GREPTRAJ STORE_TRAJ_CST: constants saved'
    LLFIRST=.FALSE.
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:STORE_TRAJ_CST',1,ZHOOK_HANDLE)
END SUBROUTINE STORE_TRAJ_CST

!-----------------------------------------------------------------------

SUBROUTINE FORCE_SFC_TRAJ(KSTEP,KBLK,KSTA,KEND)

INTEGER(KIND=JPIM),INTENT(IN) :: KBLK,KSTEP,KSTA,KEND
INTEGER(KIND=JPIM) :: istep
INTEGER(KIND=JPIM), SAVE :: IISTEP 
REAL(KIND=JPRB) :: ZTEMP(NPROMA,NGP5)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:FORCE_SFC_TRAJ',0,ZHOOK_HANDLE)
istep=MSTEPTRAJW(kstep)
IF (lifstraj) istep=min(istep,1)

IF (istep>0) THEN
  ZTEMP(KSTA:KEND,:) = TRAJ_SRFC(KSTA:KEND,:,kblk,istep)
  CALL GPOPER('SET0TOTRAJ',KBL=KBLK,PFIELD=ZTEMP)
!$OMP MASTER
  IF (LPRTTRAJ.AND.IISTEP/=ISTEP) THEN
    WRITE(NULOUT,*)'GREPTRAJ FORCE_SFC_TRAJ istep=',istep
    IISTEP=ISTEP
  ENDIF 
!$OMP END MASTER
ENDIF

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:FORCE_SFC_TRAJ',1,ZHOOK_HANDLE)
END SUBROUTINE FORCE_SFC_TRAJ

!-----------------------------------------------------------------------

SUBROUTINE GET_TRAJ_SFC(KSTEP,KBLK,KSTA,KEND,PSP_SB,PSP_SG,PSP_RR,&
 & PSD_VF,PSD_VD,PSD_VV,PSD_WS)

INTEGER(KIND=JPIM),INTENT(IN) :: KBLK,KSTEP,KSTA,KEND
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSP_SB(:,:,:)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSP_SG(:,:)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSP_RR(:,:)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSD_VF(:,:)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSD_VD(:,:)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSD_VV(:,:)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PSD_WS(:,:)
INTEGER(KIND=JPIM) :: jf,istep
INTEGER(KIND=JPIM), SAVE :: IISTEP
REAL(KIND=JPRB) :: ZTEMP(NPROMA,NGP5)
REAL(KIND=JPRB) :: ZTEMPC(NPROMA,NTRAJ_CST)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
DATA IISTEP /-99/

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:GET_TRAJ_SFC',0,ZHOOK_HANDLE)
istep=MSTEPTRAJR(kstep)

! Have to set these to zero as GPOPER does not know about ksta:kend
ZTEMP=0.0_JPRB
ZTEMPC=0.0_JPRB

DO jf=1,NGP5
  ZTEMP(ksta:kend,jf)=TRAJ_SRFC(ksta:kend,jf,kblk,istep)
ENDDO

DO jf=1,NTRAJ_CST
  ZTEMPC(ksta:kend,jf)=TRAJ_CST(ksta:kend,jf,kblk)
ENDDO
CALL GPOPER('GETTRAJ',PSP_SB=PSP_SB,PSP_SG=PSP_SG,PSP_RR=PSP_RR,&
 & PSD_VF=PSD_VF,PSD_VD=PSD_VD,PSD_VV=PSD_VV,PSD_WS=PSD_WS,PFIELD=ZTEMP,PFIELD2=ZTEMPC)

!$OMP MASTER
IF (LPRTTRAJ.AND.IISTEP/=ISTEP) THEN
  WRITE(NULOUT,*)'GREPTRAJ GET_TRAJ_SFC istep=',istep
  IISTEP=ISTEP
ENDIF
!$OMP END MASTER

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:GET_TRAJ_SFC',1,ZHOOK_HANDLE)
END SUBROUTINE GET_TRAJ_SFC

!-----------------------------------------------------------------------

SUBROUTINE READ_TRAJ_SFC(kstep,LDLASTRAJ)

INTEGER(KIND=JPIM), INTENT(IN) :: kstep
LOGICAL, INTENT(IN) :: LDLASTRAJ
INTEGER(KIND=JPIM) :: J1,J2,J3
CHARACTER(LEN=23) :: CLFILE
REAL(KIND=JPRB) :: ZHOOK_HANDLE,zdummy(0,0,0,0)
INTEGER(KIND=JPIM) :: IDUM(1),isub,ipersub,istep
LOGICAL :: llnew,llclose

IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:READ_TRAJ_SFC',0,ZHOOK_HANDLE)

LLCLOSE = LDLASTRAJ
ALLOCATE(TRAJ_BUF(NPROMA,NGP5,NGPBLKS))
!TRAJ_BUF(:,:,:)=0.0
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J3,J2,J1)
  DO J3=1,NGPBLKS
    DO J2=1,NGP5
      DO J1=1,NPROMA
        TRAJ_BUF(J1,J2,J3)=0.0
      ENDDO
    ENDDO
  ENDDO
!$OMP END PARALLEL DO

IF (LARPEGEF_TRAJHR) THEN
  write(nulout,*)'GREPTRAJ Reading surface trajectory from ARPEGE file'
  CALL READ_SURFGRID_TRAJ_FROMFA(KSTEP,TRAJ_BUF)
ELSE
  IF (N_COUPLED_WINDOWS>0) THEN
    ipersub=(NSTEPTRAJ-1)/N_COUPLED_WINDOWS
    isub=N_COUPLED_WINDOWS-(kstep-1)/ipersub
    istep=MOD(kstep-1,ipersub)+1
    IF (kstep==NSTEPTRAJ) THEN
      isub=1
      istep=ipersub+1
    ENDIF
    llnew=(istep==1)
    IF (isub>1) llclose=llclose.OR.(istep==ipersub)
    write(nulout,*)'READ_TRAJ_SFC: N_COUPLED_WINDOWS, NSTEPTRAJ=',N_COUPLED_WINDOWS, NSTEPTRAJ
    write(nulout,*)'READ_TRAJ_SFC: kstep,ipersub,isub,istep=',kstep,ipersub,isub,istep
    write(nulout,*)'READ_TRAJ_SFC: llnew,llclose=',llnew,llclose
    IF (ipersub*N_COUPLED_WINDOWS/=NSTEPTRAJ-1) CALL ABOR1('READ_TRAJ_SFC: Error ipersub')
    IF (isub<1 .OR. isub>N_COUPLED_WINDOWS) CALL ABOR1('READ_TRAJ_SFC: Error isub')
    IF (istep<1 .OR. istep>ipersub+1) CALL ABOR1('READ_TRAJ_SFC: Error istep')
    clfile='TRAJHRXX_YY/trajsurf000'
    write(clfile( 7: 8),'(I2.2)')nuptra
    write(clfile(10:11),'(I2.2)')isub
  ELSE
    llnew=(kstep==1)               !!  Mats branch
    clfile='                       '
  CLFILE(1:20)=CFNTRAJHRSURF
  write(CLFILE( 7: 8),'(I2.2)')nuptra
    write(CLFILE(18:20),'(I3.3)')0   !!  Mats branch
!   write(CLFILE(18:20),'(I3.3)')kstep
  ENDIF
  write(nulout,*)'GREPTRAJ Reading surface trajectory from ',CLFILE
  CALL READ_GRID_TRAJ(TRIM(CLFILE),IDUM,mtype_surf_traj,0,ngp5,zdummy,TRAJ_BUF,llnew,LDCLOSE=LLCLOSE)
ENDIF

!TRAJ_SRFC(:,:,:,kstep)=TRAJ_BUF(:,:,:)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J3,J2,J1)
  DO J3=1,NGPBLKS
    DO J2=1,NGP5
      DO J1=1,NPROMA
        TRAJ_SRFC(J1,J2,J3,KSTEP)=TRAJ_BUF(J1,J2,J3)
      ENDDO
    ENDDO
  ENDDO
!$OMP END PARALLEL DO
DEALLOCATE(TRAJ_BUF)
IF (LHOOK) CALL DR_HOOK('TRAJ_SURFACE_MOD:READ_TRAJ_SFC',1,ZHOOK_HANDLE)
END SUBROUTINE READ_TRAJ_SFC

!-----------------------------------------------------------------------

END MODULE TRAJ_SURFACE_MOD
