MODULE WATCH_ARRAYS_MOD

!   Purpose
!   -------
!     Look for 'not a number' or infinite values in arrays
!     of real numbers.

!   Author
!   ------
!     Yannick Tremolet

!   Modifications
!   -------------
!     Original    02-Mar-04
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULERR

IMPLICIT NONE
SAVE
PRIVATE
PUBLIC WATCH_ARRAY

INTEGER(KIND=JPIM) :: ninf,ntin,nnan

INTERFACE WATCH_ARRAY
MODULE PROCEDURE WATCH_ARRAY0, WATCH_ARRAY1, WATCH_ARRAY2, WATCH_ARRAY3, &
               & WATCH_ARRAY4
END INTERFACE

! ------------------------------------------------------------------
#include "abor1.intfb.h"
! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

SUBROUTINE WATCH_ARRAY0(PPP,CDSTR)

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN) :: PPP
CHARACTER(LEN=*), INTENT(IN) :: CDSTR
INTEGER(KIND=JPIM) :: ii,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY0',0,ZHOOK_HANDLE)

ilen=1
CALL cma_flpcheck(PPP,ilen,ii,0,ninf,ntin,nnan)
CALL PRT_WATCH(CDSTR)

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY0',1,ZHOOK_HANDLE)
END SUBROUTINE WATCH_ARRAY0

! ------------------------------------------------------------------

SUBROUTINE WATCH_ARRAY1(PPP,CDSTR)

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN) :: PPP(:)
CHARACTER(LEN=*), INTENT(IN) :: CDSTR
INTEGER(KIND=JPIM) :: ii,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY1',0,ZHOOK_HANDLE)

ilen=SIZE(PPP)
CALL cma_flpcheck(PPP,ilen,ii,0,ninf,ntin,nnan)
CALL PRT_WATCH(CDSTR)

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY1',1,ZHOOK_HANDLE)
END SUBROUTINE WATCH_ARRAY1

! ------------------------------------------------------------------

SUBROUTINE WATCH_ARRAY2(PPP,CDSTR)

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN) :: PPP(:,:)
CHARACTER(LEN=*), INTENT(IN) :: CDSTR
INTEGER(KIND=JPIM) :: ii,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY2',0,ZHOOK_HANDLE)

ilen=SIZE(PPP)
CALL cma_flpcheck(PPP,ilen,ii,0,ninf,ntin,nnan)
CALL PRT_WATCH(CDSTR)

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY2',1,ZHOOK_HANDLE)
END SUBROUTINE WATCH_ARRAY2

! ------------------------------------------------------------------

SUBROUTINE WATCH_ARRAY3(PPP,CDSTR)

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN) :: PPP(:,:,:)
CHARACTER(LEN=*), INTENT(IN) :: CDSTR
INTEGER(KIND=JPIM) :: ii,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY3',0,ZHOOK_HANDLE)

ilen=SIZE(PPP)
CALL cma_flpcheck(PPP,ilen,ii,0,ninf,ntin,nnan)
CALL PRT_WATCH(CDSTR)

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY3',1,ZHOOK_HANDLE)
END SUBROUTINE WATCH_ARRAY3

! ------------------------------------------------------------------

SUBROUTINE WATCH_ARRAY4(PPP,CDSTR)

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN) :: PPP(:,:,:,:)
CHARACTER(LEN=*), INTENT(IN) :: CDSTR
INTEGER(KIND=JPIM) :: ii,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY4',0,ZHOOK_HANDLE)

ilen=SIZE(PPP)
CALL cma_flpcheck(PPP,ilen,ii,0,ninf,ntin,nnan)
CALL PRT_WATCH(CDSTR)

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:WATCH_ARRAY4',1,ZHOOK_HANDLE)
END SUBROUTINE WATCH_ARRAY4

! ------------------------------------------------------------------

SUBROUTINE PRT_WATCH(CDSTR)

IMPLICIT NONE
CHARACTER(LEN=*), INTENT(IN) :: CDSTR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:PRT_WATCH',0,ZHOOK_HANDLE)

IF (nnan>0) THEN
  WRITE(nulerr,*)CDSTR,' WATCH_ARRAY: not a number',nnan
  CALL ABOR1('WATCH_ARRAY: not a number')
ENDIF
IF (ninf>0) THEN
  WRITE(nulerr,*)CDSTR,' WATCH_ARRAY: infinite number',ninf
  CALL ABOR1('WATCH_ARRAY: infinite number')
ENDIF
IF (ntin>0) THEN
  WRITE(nulerr,*)CDSTR,' WATCH_ARRAY: tiny number',ntin
  CALL ABOR1('WATCH_ARRAY: tiny number')
ENDIF

! IF (ninf>0.OR.ntin>0.OR.nnan>0) CALL C_DRHOOK_RAISE(6)

IF (LHOOK) CALL DR_HOOK('WATCH_ARRAYS_MOD:PRT_WATCH',1,ZHOOK_HANDLE)
END SUBROUTINE PRT_WATCH

! ------------------------------------------------------------------

END MODULE WATCH_ARRAYS_MOD
