MODULE YEMDIM

USE PARKIND1, ONLY : JPIM

USE PERDIM,   ONLY : JPN1SP

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------

!*    Dimensions of model working arrays (YEMDIM)

!   NS3DFD :   maximum number of 3D fields to be direct transformed
!                                 meridionally
!   NS2DFD :   number of 2D fields to be direct  transformed meridionally
!   NS3DFI :   number of 3D fields to be inverse transformed meridionally
!   NS2DFI :   number of 2D fields to be inverse transformed meridionally
!   NS3DYX :   number of 3D fields to transfered to the grid-point fields
!   NS2DYX :   number of 2D fields to transfered to the grid-point fields
!   NFTM   :   the third dimension of the temporary working arrays for
!               the meridional FFT-s
!   NDGLSUR:   number of rows of latitudes with overdimensioning
!   NSECPLG:   dimension for the Laplace operator
!   NSECPL :   dimension for the Laplace operator (for one PE)

!   NEDOM   : key: options for file ALADIN :
!                        -1 --- Gridpoint file
!                         1 --- Spectral file (Aladin standard)

!*    Width of coupling/relaxation belts

!   NBZONG :   the half-difference between the size of inner and outer
!              zone in y direction
!   NBZONL :   the half-difference between the size of inner and outer
!              zone in y direction

!*    Number of the lateral boundary variables

!   the following two parameters are obsolete since gmv/gfl (CF)
!   NG2D3   : number of 2D lateral boundary variables
!   NG3D3   : number of 3D lateral boundary variables

!   NGALFU  : maximum number of fundamental coupled variables
!   NGALEF  : number of different coupling function alpha

!*   Logical for coupling and biperiodicisation
!    LECOBI : logical for coupling and biperiodicisation
!             It is true if NBZONL ne 0 or NBZONG ne 0
!             or NDLUXG ne NDLON or NDGUXG ne NDGL

!    NISNAX : zonal limit wavenumbers within the ellipse
!    NISMAX : meridional limit wavenumbers within the ellipse

INTEGER(KIND=JPIM) :: NS3DFD
INTEGER(KIND=JPIM) :: NS2DFD
INTEGER(KIND=JPIM) :: NS3DFI
INTEGER(KIND=JPIM) :: NS2DFI
INTEGER(KIND=JPIM) :: NS3DYX
INTEGER(KIND=JPIM) :: NS2DYX
INTEGER(KIND=JPIM) :: NFTM
INTEGER(KIND=JPIM) :: NDGLSUR
INTEGER(KIND=JPIM) :: NSECPL
INTEGER(KIND=JPIM) :: NSECPLG
INTEGER(KIND=JPIM) :: NBZONG
INTEGER(KIND=JPIM) :: NBZONL
INTEGER(KIND=JPIM) :: NGALFU
INTEGER(KIND=JPIM) :: NGALEF
INTEGER(KIND=JPIM) :: NEDOM
LOGICAL :: LECOBI

INTEGER(KIND=JPIM) :: NAS1B(0:JPN1SP)
INTEGER(KIND=JPIM) :: NAS1E(0:JPN1SP)

INTEGER(KIND=JPIM), ALLOCATABLE :: NISMAX(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: NISNAX(:)

!     ------------------------------------------------------------------
END MODULE YEMDIM
