MODULE YEMDYN

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------

!===========   MAIN HORIZONTAL DIFFUSION SCHEME  ==============================

! * LEVEL AND WAVENUMBER DEPENDENT INVERSE CHARACTERISTIC TIMES:
! RDIVORE  : for diffusion of vorticity.
! RDIDIVE  : for diffusion of divergence.
! RDITE    : for diffusion of temperature.
! RDIGFLE  : for diffusion of GFL vars.
! RDIPDE   : for diffusion of pressure departure (NH).
! RDIVDE   : for diffusion of vertical divergence (NH).
! RDISPE   : for diffusion of surface pressure.

REAL(KIND=JPRB),ALLOCATABLE:: RDIVORE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDIDIVE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDITE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDIGFLE(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDIPDE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDIVDE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDISPE(:)

!===========   ADDITIONAL HORIZONTAL DIFFUSION SCHEME FOR SLHD  ===============

! * LEVEL AND WAVENUMBER DEPENDENT INVERSE CHARACTERISTIC TIMES:
! RDSVORE  : for diffusion of vorticity.
! RDSDIVE  : for diffusion of divergence.
! RDSVDE   : for diffusion of vertical divergence (NH).

REAL(KIND=JPRB),ALLOCATABLE:: RDSVORE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDSDIVE(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: RDSVDE(:,:)

!===========   RADIATIVE UPPER BOUNDARY CONDITION  ============================

! XKCOEF   : RUBC coefficient
! HDRUBC   : Diffusion of coupling geopotential on top of atmosphere.

REAL(KIND=JPRB),ALLOCATABLE:: XKCOEF(:)
REAL(KIND=JPRB) :: HDRUBC

!===========   UPDATE FREQUENCY OF LBC ========================================

! NEFRCL   : frequency of updating the lateral boundary coupling fields.
!            The LBC fields will be updated every NEFRCL time steps.
! NDIMCPL  : number of fields with true LCOUPLING attribute.
! TEFRCL   : Time interval between two updatings of the lateral boundary fields

INTEGER(KIND=JPIM) :: NEFRCL
INTEGER(KIND=JPIM) :: NDIMCPL
REAL(KIND=JPRB) :: TEFRCL

!===========   MISCELLANEOUS  =================================================

! TCDIS    : ??? (no comment provided, used in the transform package).

REAL(KIND=JPRB) :: TCDIS

!     ------------------------------------------------------------------
END MODULE YEMDYN
