! Library:phys $Id: yhlrad.F90 4917 2007-02-07 13:05:42Z towil $

MODULE YHLRAD

USE PARKIND1  ,ONLY : JPIM, JPRB 

IMPLICIT NONE
SAVE

!     Purpose:
!     --------

!     VARIABLES FOR THE HIRLAM RADIATION SCHEME.
!     THIS MODULE REPLACES HIRLAM'S COMRAD.inc.

!     IT CONTAINS THE FORMER COMMON BLOCKS
!        - COMRAD
!        - COMRAD2
!        - COMREFF


!     Modifications:
!     --------------
!     02-Jun-2005 S.Niemela       Original version for ARPEGE/IFS/ALADIN.      
!                              a) mtabm moved to SUHLRAD
!                              b) csur2  not used --> removed.
!                                 cetyp  not used --> removed.
!                                 chybcr not used --> removed.
!                                 remx   not used --> removed. 
!                                 cemc1  not used --> removed.
!                                 cemc2  not used --> removed.

!     03-Oct-2005 S.Niemela    a) ceqt not needed in YHLRAD
!                                 ---> removed.
!                              b) Some parameters not used by HLRADIA.
!                                 List of removed:
!                                 c10e,cem1,cem2,cem3,cem4,ctrmin

!     19-Oct-2005 S.Niemela    a) sa,cosd,sind removed from YHLRAD
!     End modifications
!     ------------------------------------------------------

! ntabm         :
! cqpmin        :  Minimum water path length.
! cpat          :  Scaling factor involved in calculating water
!                  path in cm of precipitable water.
! ckilgr        :  Conversion factor from kilo to gram.
! csur1,csur3   :  Emission constants for surface flux calculations. 
! cosmin        :  constant used in shortwave radiation.
! cdegr         :  constant used in shortwave radiation.
! cpatsh        :  constant used in shortwave radiation.
! caadry        :  constant used in shortwave radiation.
! caahum        :  constant used in shortwave radiation.
! cbbdry        :  constant used in shortwave radiation.
! cbbhum        :  constant used in shortwave radiation.
! csac          :  constant used in shortwave radiation.
! cfrac1        :  emissivity constant.
! cfrac2        :  emissivity constant.
! caak          :
! cask          :
! cpf           :
! cpc           :  Coefficient associated with cooling rate below clouds.
! cpsref        :  Reference pressure (Pa).
! cdpsur        :
! crefrad       :
! shtab(1:NLEV) :  Table applied in humid clear air absorbtion.

! cqco2         :  Carbon dioxide mass mixing ratio.
! ceml[1-4]     :  Water vapor line (&p-type continuum). 
!                  emissivity parameters.
! cpate and,
! ceme[1-5][a-b]:  Water vapor e-type continuum emissivity parameters.
! cpatc and,
! cemc[1-4][a-b]:  Carbon dioxide emissivity parameters.
! ca[1-5]       :  Coefficients for water vapour line and 
!                  continuum absorption overlap factor.
! cb[1-5]       :  Coefficients for water vapour and carbon dioxide 
!                  overlap factor.
! cemadd        :  Contribution of other factors than H2O line absorption
!                  to single-layer clear-sky emissivity.


! Following parameters are needed in definition of 
! the effective radius parameterization.

! cmmy, remn, creland, cresea, b10a, b10b, b11, b12, b13a, b13b,
! b14, a1, a2, a3

! ---------------
! HIRLAM'S COMRAD
! ---------------
INTEGER(KIND=JPIM) :: ntabm
REAL(KIND=JPRB) :: cqpmin
REAL(KIND=JPRB) :: cpat
REAL(KIND=JPRB) :: ckilgr
REAL(KIND=JPRB) :: csur1
REAL(KIND=JPRB) :: csur3
REAL(KIND=JPRB) :: cosmin
REAL(KIND=JPRB) :: cdegr
REAL(KIND=JPRB) :: cpatsh
REAL(KIND=JPRB) :: caadry
REAL(KIND=JPRB) :: caahum
REAL(KIND=JPRB) :: cbbdry
REAL(KIND=JPRB) :: cbbhum
REAL(KIND=JPRB) :: csac
REAL(KIND=JPRB) :: cfrac1
REAL(KIND=JPRB) :: cfrac2
REAL(KIND=JPRB) :: caak
REAL(KIND=JPRB) :: cask
REAL(KIND=JPRB) :: cpf
REAL(KIND=JPRB) :: cpc
REAL(KIND=JPRB) :: cpsref
REAL(KIND=JPRB) :: cdpsur
REAL(KIND=JPRB) :: crefrad
REAL(KIND=JPRB),ALLOCATABLE :: shtab(:)

! ----------------
! HIRLAM'S COMRAD2
! ----------------
REAL(KIND=JPRB) :: cqco2
REAL(KIND=JPRB) :: ceml1
REAL(KIND=JPRB) :: ceml2
REAL(KIND=JPRB) :: ceml3
REAL(KIND=JPRB) :: ceml4
REAL(KIND=JPRB) :: cpate
REAL(KIND=JPRB) :: ceme1a
REAL(KIND=JPRB) :: ceme2a
REAL(KIND=JPRB) :: ceme1b
REAL(KIND=JPRB) :: ceme2b
REAL(KIND=JPRB) :: ceme3b
REAL(KIND=JPRB) :: ceme4b
REAL(KIND=JPRB) :: ceme5b
REAL(KIND=JPRB) :: cpatc
REAL(KIND=JPRB) :: cemc1a
REAL(KIND=JPRB) :: cemc2a
REAL(KIND=JPRB) :: cemc3a
REAL(KIND=JPRB) :: cemc1b
REAL(KIND=JPRB) :: cemc2b
REAL(KIND=JPRB) :: cemc3b
REAL(KIND=JPRB) :: cemc4b
REAL(KIND=JPRB) :: ca1
REAL(KIND=JPRB) :: ca2
REAL(KIND=JPRB) :: ca3
REAL(KIND=JPRB) :: ca4
REAL(KIND=JPRB) :: ca5
REAL(KIND=JPRB) :: cb1
REAL(KIND=JPRB) :: cb2
REAL(KIND=JPRB) :: cb3
REAL(KIND=JPRB) :: cb4
REAL(KIND=JPRB) :: cb5
REAL(KIND=JPRB) :: cemadd

! ----------------
! HIRLAM'S COMREFF
! ----------------
REAL(KIND=JPRB) :: cmmy
REAL(KIND=JPRB) :: remn
REAL(KIND=JPRB) :: creland
REAL(KIND=JPRB) :: cresea
REAL(KIND=JPRB) :: b10a
REAL(KIND=JPRB) :: b10b
REAL(KIND=JPRB) :: b11
REAL(KIND=JPRB) :: b12
REAL(KIND=JPRB) :: b13a
REAL(KIND=JPRB) :: b13b
REAL(KIND=JPRB) :: b14
REAL(KIND=JPRB) :: a1
REAL(KIND=JPRB) :: a2
REAL(KIND=JPRB) :: a3

END MODULE YHLRAD
