MODULE YO_AERO_M7
  
  ! *yo_aero_m7*  Contains paramters, switches and initialization
  !               routines for the m7 aerosol scheme obtained from 
  !               a merger between from mo_aero_m7 and mo_aero
  !
  ! Authors:
  ! --------
  ! P. Stier (MPI)                                 2001/2002 
  ! E. Vignati and J. Wilson (JRC / EI)            2000

  ! JJMorcrette 20100120  Rewritten and adapted to ECMWF IFS


USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!-------------------------------------------------------------------------------------
LOGICAL :: LEPAEM7  ! .true. if M7 aerosol model is activated
LOGICAL :: LM7UPHYS ! .true. if M7 aerosol microphysics is activated
INTEGER(KIND=JPIM) :: NAERM7

!===== parameters originally imported from mo_aero ===================================

CHARACTER(LEN=2) :: cmode(7)
CHARACTER(LEN=3) :: ctype(6)

INTEGER(KIND=JPIM) :: ndust, nseasalt, npist, nemiss, ntype

REAL(KIND=JPRB) :: rsulfate2sulfur
REAL(KIND=JPRB) :: rbb_wsoc_perc,  rbge_wsoc_perc,  rom2oc 
REAL(KIND=JPRB) :: rcmr_ff,  rcmr_bb,  rcmr_bg,  rcmr_sk,  rcmr_sa,  rcmr_sc
REAL(KIND=JPRB) :: rfacso2,  rso2ts

REAL(KIND=JPRB) :: rm2n_bcki_ff, rm2n_bcki_bb, rm2n_bcks_bb, rm2n_ocki_ff &
  &              , rm2n_ocki_bb, rm2n_ocki_bg, rm2n_ocks_bb, rm2n_ocks_bg &
  &              , rm2n_s4ks_sk, rm2n_s4as_sa, rm2n_s4cs_sc

!===== parameters originally imported from mo_aero_m7 ================================

LOGICAL :: lsnucl,  lscoag,  lscond 
  
INTEGER(KIND=JPIM) :: nnucl 

LOGICAL :: lmass_diag

INTEGER(KIND=JPIM) :: naermod, nmod,  nss,  nsol,  ngas,  nsulf , nm7aer, ntrac
                        
INTEGER(KIND=JPIM) :: iso4ns,  iso4ks,  iso4as,  iso4cs & 
    &              ,  ibcks ,  ibcas ,  ibccs ,  ibcki  &
    &              ,  iocks ,  iocas ,  ioccs ,  iocki  &
    &              ,  issas ,  isscs                    &
    &              ,  iduas ,  iducs ,  iduai ,  iduci   

INTEGER(KIND=JPIM) :: inucs ,  iaits ,  iaccs ,  icoas  ,  iaiti ,  iacci ,  icoai    

REAL(KIND=JPRB) :: crdiv(4)

REAL(KIND=JPRB) :: sigma(7) 
REAL(KIND=JPRB) :: sigmaln(7)

REAL(KIND=JPRB) :: cmr2ras(7)
REAL(KIND=JPRB) :: cmr2mmr(7)
REAL(KIND=JPRB) :: cmedr2mmedr(7)
REAL(KIND=JPRB) :: cmr2ram(7)
REAL(KIND=JPRB) :: ram2cmr(7)

REAL(KIND=JPRB) :: cmin_aerml, cmin_aernl

REAL(KIND=JPRB) :: caccso4(7)

REAL(KIND=JPRB) :: crh

REAL(KIND=JPRB) :: bk, avo, rerg, r_kcal
  
REAL(KIND=JPRB) :: dh2so4 , ddust , dbc   , doc    , dnacl  , dna2so4 &
    &           ,  dnahso4, dh2o  , wh2so4, wh2o   , wso4   , wso2    &
    &           ,  wna    , wcl   , wnacl , wna2so4, wnahso4

REAL(KIND=JPRB) :: critn , fmax  , cLayerThickness

REAL(KIND=JPRB) :: wvb(17), gmb(9)

LOGICAL :: locoagmask(7,7)

REAL(KIND=JPRB) :: rmonss1, rmonss2  !  coefficient for sea-salt Monahan et al. 1986

REAL(KIND=JPRB) :: rmass1(0:40), rmass2(0:40) & ! mass fluxes per wind class [kg m-2 s-1]
    &            , rnumb1(0:40), rnumb2(0:40) & ! number fluxes per wind class [m-2 s-1]
    &            , rdmas1(0:40), rdmas2(0:40) & ! mass flux gradient for interpol [kg m-3]
    &            , rdnum1(0:40), rdnum2(0:40)   ! number flux gradient for interpol. [m-3]

REAL(KIND=JPRB) :: pm6rp_lon_lev(7), pras(7), pas(7),  pcrtcst(7)


!Other parameters taken from various routines

REAL(KIND=JPRB) :: rcarbon_nh(12), rcarbon_sh

!-- mo_aero_trac
REAL(KIND=JPRB)    :: rdens(32), rosm(32), rion(32), rmowg(32), rhenr(32), rdryr(32)
INTEGER(KIND=JPIM) :: ndryd(32), nsolu(32), nmode(32)
LOGICAL            :: lwetd(32), lsedi(32)
CHARACTER(LEN=3)   :: cphas(32), ctrac(32)

!-- bgc_dust_calc_emis ... see su_aerm7 for computation
REAL(KIND=JPRB) :: rfcon_ai, rfcon_ci, rmmr_ai, rmmr_ci

!--- Mean mass scavenging coefficients normalized by rain-rate [kg m-2]:
!    Rain: Seinfeld & Pandis, Fig 20.15:
REAL(KIND=JPRB) :: rbcr(7)
!    Snow: Little available, graphs in Prupbacher show similar order of magnitude in 
!          collection efficiency as rain,therefore assume typical mean value as above:
REAL(KIND=JPRB) :: rbcs(7)
!--- Assumed updraft velocity in convective clouds [m s-1]:
REAL(KIND=JPRB) :: rwu

!--- Prescribed scavenging ratios:
REAL(KIND=JPRB) :: csr_strat_wat(7)
REAL(KIND=JPRB) :: csr_strat_mix(7)
REAL(KIND=JPRB) :: csr_strat_ice(7)
REAL(KIND=JPRB) :: csr_conv(7)

!-------------------------------------------------------------------------------------
END MODULE YO_AERO_M7
