MODULE YOECUMF2

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     ----------------------------------------------------------------------
!*    ** *YOECUMF2* - PARAMETERS FOR NEW LINEARIZED CUMULUS MASS-FLUX SCHEME
!     ----------------------------------------------------------------------

REAL(KIND=JPRB) :: ENTRPEN2
REAL(KIND=JPRB) :: ENTRSCV2
REAL(KIND=JPRB) :: ENTRORG2
REAL(KIND=JPRB) :: ENTRDD2
REAL(KIND=JPRB) :: RMFCMIN2
REAL(KIND=JPRB) :: RMFDEPS2
REAL(KIND=JPRB) :: RDEPTHS2
REAL(KIND=JPRB) :: RPRCON2
REAL(KIND=JPRB) :: RTAU2
REAL(KIND=JPRB) :: RCPECONS2
REAL(KIND=JPRB) :: RCUCOV2
REAL(KIND=JPRB) :: RTAUMEL2
REAL(KIND=JPRB) :: RUVPER2
REAL(KIND=JPRB) :: RMFSOLCT2
REAL(KIND=JPRB) :: RMFSOLTQ2
REAL(KIND=JPRB) :: RMFSOLUV2
REAL(KIND=JPRB) :: RMFCFL2
REAL(KIND=JPRB) :: RMFLIC2
REAL(KIND=JPRB) :: RMFLIA2

LOGICAL :: LMFMID2
LOGICAL :: LMFDD2
LOGICAL :: LMFDUDV2
LOGICAL :: LMFUVDIS2

INTEGER(KIND=JPIM) :: NJKT12, NJKT22, NJKT32

!*    ** *YOECUMF2* - PARAMETERS FOR NEW LINEARIZED CUMULUS MASS-FLUX SCHEME

!     P.LOPEZ                       E. C. M. W. F.      21/8/2006
!     Inspired from M.TIEDTKE       E. C. M. W. F.      18/1/89

!     NAME      TYPE      PURPOSE
!     ----      ----      -------

!     LMFMID2    LOGICAL  TRUE IF MIDLEVEL    CONVECTION IS SWITCHED ON
!     LMFDD2     LOGICAL  TRUE IF CUMULUS DOWNDRAFT      IS SWITCHED ON
!     LMFDUDV2   LOGICAL  TRUE IF CUMULUS FRICTION       IS SWITCHED ON
!     LMFUVDIS2  LOGICAL  TRUE IF KIN.ENERGY DISSIPATION BY CUMULUS FRICTION IS SWITCHED ON
!     ENTRPEN2   REAL     ENTRAINMENT RATE FOR PENETRATIVE CONVECTION
!     ENTRSCV2   REAL     ENTRAINMENT RATE FOR SHALLOW CONVECTION
!     ENTRORG2   REAL     COEFFICIENT FOR ORGANIZED ENTRAINMENT DEEP CONVECTION
!     ENTRDD2    REAL     ENTRAINMENT RATE FOR CUMULUS DOWNDRAFTS
!     RMFCMIN2   REAL     MINIMUM MASSFLUX VALUE (FOR SAFETY)
!     RMFDEPS2   REAL     FRACTIONAL MASSFLUX FOR DOWNDRAFTS AT LFS
!     RDEPTHS2   REAL     MAXIMUM ALLOWED CLOUD THICKNESS FOR SHALLOW
!     RPRCON2    REAL     COEFFICIENTS FOR DETERMINING CONVERSION
!                         FROM CLOUD WATER TO RAIN
!     RCPECONS2  REAL     COEFFICIENT FOR RAIN EVAPORATION BELOW CLOUD
!     RCUCOV2    REAL     CONVECTIVE CLOUD COVER FOR RAIN EVPORATION
!     RTAUMEL2   REAL     TIME CONSTANT FOR MELTING
!     RTAU2      REAL     ADJUSTMENT TIME SCALE IN CAPE CLOSURE
!     RMFCFL2    REAL     MULTIPLE OF CFL STABILITY CRITERIUM 
!     RMFLIC2    REAL     USE CFL (1) OR ABSOLUT MASS FLUX LIMIT (0)
!     RMFLIA2    REAL     VALUE OF ABSOLUT MASS FLUX LIMIT
!     RMFSOLCT2  REAL     SOLVER FOR MASSFLUX ADVECTION EQUATION FOR TRACERS
!                         0 : EXPLICIT,  0-1 : SEMI-IMPLICIT, >=1 : FULLY IMPLICIT.
!     RMFSOLUV2  REAL     SOLVER FOR MASSFLUX ADVECTION EQUATION FOR MOMENTUM
!     RMFSOLTQ2  REAL     SOLVER FOR MASSFLUX ADVECTION EQUATION FOR T AND Q
!     RUVPER2    REAL     UPDRAIGHT VELOCITY PERTURBATION AT KLEV FOR IMPLICIT

!     NJKT12, NJKT22, NJKT32  INTEGER LEVEL LIMITS FOR CUBASEN2, CUDDRAFN2
!     ---------------------------------------------------------------------
END MODULE YOECUMF2
