MODULE YOEPHY

USE PARKIND1  ,ONLY : JPRB, JPIM

IMPLICIT NONE

SAVE

!     -----------------------------------------------------------------
!*    ** *YOEPHY* - SWITCHES RELATED TO DIABATIC PROCESSES
!     -----------------------------------------------------------------

!        * E.C.M.W.F. PHYSICS PACKAGE *

LOGICAL :: LEPHYS
LOGICAL :: LECOND
LOGICAL :: LECUMF
LOGICAL :: LEDCLD
LOGICAL :: LEEVAP
LOGICAL :: LEGWDG
LOGICAL :: LEGWWMS
LOGICAL :: LEOZOC
LOGICAL :: LEQNGT
LOGICAL :: LERADI
LOGICAL :: LERADS
LOGICAL :: LESICE
LOGICAL :: LESURF
LOGICAL :: LEVDIF
LOGICAL :: LAGPHY
LOGICAL :: LEPCLD
LOGICAL :: LEO3CH
LOGICAL :: LO3CH_OLDVER
LOGICAL :: LECO2DIU
LOGICAL :: LBUD23
LOGICAL :: LBUD23RST
LOGICAL :: LBUDCYCLE
LOGICAL :: LEMETHOX
LOGICAL :: LERA40
LOGICAL :: LECURR
LOGICAL :: LVDFTRAC
LOGICAL :: LMFTRAC
LOGICAL :: LMFSCAV
LOGICAL :: LERAIN
LOGICAL :: LE4ALB
LOGICAL :: LEMWAVE
LOGICAL :: LESMOS
LOGICAL :: LEOCWA
LOGICAL :: LEOCCO
LOGICAL :: LEOCSA
LOGICAL :: LEOCLA
LOGICAL :: LEVGEN
LOGICAL :: LESSRO
LOGICAL :: LEFLAKE
LOGICAL :: LEOCML
LOGICAL :: LESMOSS
LOGICAL :: LECLIPQT0
LOGICAL :: LECLIPCLDT0
LOGICAL :: LDUCTDIA
LOGICAL :: LESN09
LOGICAL :: LELAIV
LOGICAL :: LDYNVAR
REAL(KIND=JPRB) :: RTHRFRTI
INTEGER (KIND=JPIM) :: NEPHYS_PCFULL

! SOME VARIABLES RELATED TO THE DIFFERENT NBF CO2 CLIMATOLOGIES

! NCO2CLIMYY1    ===> First year in the NBF CO2 climatology
! NCO2CLIMYY2    ===> Last year in the NBF CO2 climatology
! NCO2CLIMN1     ===> starting time for the diurnally variable fluxes (in minutes)
! NCO2CLIMN2     ===> end time for the diurnally variable fluxes (in minutes)
! NCO2CLIMFRQ    ===> time frequency at which the diurnal cycle is provided (in minutes)

INTEGER(KIND=JPIM) :: NCO2CLIMYY1
INTEGER(KIND=JPIM) :: NCO2CLIMYY2
INTEGER(KIND=JPIM) :: NCO2CLIMN1
INTEGER(KIND=JPIM) :: NCO2CLIMN2
INTEGER(KIND=JPIM) :: NCO2CLIMFRQ


!     REFERENCE.
!     ----------

!     J.-J. MORCRETTE       E.C.M.W.F.      91/07/14

!     MODIFICATIONS
!     -------------

!     P. Viterbo   ECMWF   03-12-2004  Include user-defined RTHRFRTI
!     G. Balsamo   ECMWF   14-03-2007  Include LEVGEN, LESSRO switches
!     G. Balsamo   ECMWF   07-04-2008  Include LEFLAKE switch
!     G. Balsamo   ECMWF   13-10-2008  Include LEOCML switch
!     G. Balsamo   ECMWF   13-10-2008  Include LESNWD switch
!     P. Lopez     ECMWF   24-01-2009  Include LDUCTDIA switch
!     G. Balsamo   ECMWF   24-02-2009  Include full set of SNOW switches
!     Y. Takaya    ECMWF   21-08-2009  Include LEOCLA switch
!     S. Boussetta/G.Balsamo May 2009  Include variable LAI switch LELAIV 
!     E. Dutra             18-11-2009  snow 2009 cleaning of logicals - LESN09 to activate all 
!     Y. Takaya/P. de Rosnay May 2020  SSS for SMOS
!     ------------------------------------------------------------------

!  NAME     TYPE     PURPOSE
!  ----  :  ----   : ---------------------------------------------------
! LEPHYS : LOGICAL : SWITCH THE FULL E.C.M.W.F. PHYSICS PACKAGE ON
! LAGPHY : LOGICAL : IF TRUE, PHYSICS PACKAGE CALLED IN LAGGED MODE
! LECOND : LOGICAL : TURN THE LARGE-SCALE CONDENSATION ON
! LECUMF : LOGICAL : TURN THE MASS-FLUX CUMULUS CONVECTION SCHEME ON
! LEDCLD : LOGICAL : TURN THE DIAGNOSTIC CLOUD SCHEME ON
! LEPCLD : LOGICAL : TURN THE PROGNOSTIC CLOUD SCHEME ON
! LEEVAP : LOGICAL : TURN THE EVAPORATION OF PRECIPITATION ON
! LEGWDG : LOGICAL : TURN THE GRAVITY WAVE DRAG ON
! LEGWWMS: LOGICAL : TURN THE WARNER-MCINTYRE-SCINOCCA NON-OROGRAPHIC GRAVITY WAVE SCHEME ON
! LEOZOC : LOGICAL : TURN THE CLIMATOLOGICAL OZONE ON
! LEQNGT : LOGICAL : TURN THE NEGATIVE HUMIDITY FIXER ON
! LERADI : LOGICAL : TURN THE RADIATION SCHEME ON
! LERADS : LOGICAL : TURN THE INTERACTIVE SURFACE RADIATIVE PROPERTIESON
! LESICE : LOGICAL : TURN THE INTERACTIVE SEA ICE PROCESSES ON
! LESURF : LOGICAL : TURN THE INTERACTIVE SURFACE PROCESSES ON
! LEVDIF : LOGICAL : TURN THE VERTICAL DIFFUSION ON
! LEO3CH : LOGICAL : TURN THE O3 CHEMISTRY ON (for EC prog. ozone)
! LO3CH_OLDVER : LOGICAL : USE OLD VERSION (v2.3) OF OZONE CHEMISTRY
! LECO2DIU: LOGICAL: UPDATE THE NBF CO2 FLUXES CLIMATOLOGY DURING THE RUN OF THE MODEL (introduce a diurnal cycle)
! LBUD23 : LOGICAL : SWITCH FOR 3 AND 2 DIMENSIONAL BUDGETS 
! LBUD23RST : LOGICAL : TRUE WHEN BUDGETS ARE TO BE RESET
! LBUDCYCLE: LOGICL: SWITCH FOR DIURNAL CYCLE DIAGNOSTICS
! LEMETHOX: LOGICAL: TURN THE METHANE OXIDATION ON
! LERA40 : LOGICAL : EXTRA PHYSICS DIAGNOSTICS FOR ERA40
! LECURR : LOGICAL : IF TRUE, OCEAN CURRENT BOUNDARY CONDITION IS USED
! LVDFTRAC: LOGICAL: TURN TRACER TRANSPORT BY VERTICAL DIFFUSION ON 
! LMFTRAC: LOGICAL : TURN TRACER TRANSPORT BY MASS FLUX CONVECTION ON
! LMFSCAV:LOGICAL  : TURN TRACER WET SCAAVENGING IN CUMULUS CONVECTION SCHEME ON
! LERAIN : LOGICAL : RAIN ASSIMILATION
! LE4ALB : LOGICAL : MODIS ALBEDO ON (DIRECT+DIFFUSE)*(UVIS+NIR) 
! LEOCWA : LOGICAL : WARM OCEAN LAYER PARAMETRIZATION
! LEOCCO : LOGICAL : COOL OCEAN SKIN PARAMETRIZATION
! LEOCSA : LOGICAL : SALINTY EFFECT ON SATURATION AT OCEAN SURFACE
! LEOCLA : LOGICAL : LANGMUIR CIRCULATION EFFECT IN SKIN LAYER SCHEME
! RTHRFRTI : INTEGER : MINIMUM FRACTION FOR ALL SURFACE TILES
! LE4ALB : LOGICAL  : MODIS ALBEDO (UV-Vis+NIR)x(direct+diffuse) 
! LEVGEN : LOGICAL  : VAN GENUCHTEN HYDROLOGY (with SOIL TYPE field)
! LESSRO : LOGICAL  : OROGRAPHIC (VIC-TYPE) RUNOFF
! LEFLAKE : LOGICAL  : LAKE MODEL FLAKE
! LEOCML : LOGICAL  : OCEAN MIX LAYER MODEL
! LECLIPQT0: LOGICAL: REMOVE NEGATIVE Q AND SUPERSATURATION AT INITIAL TIME
! LECLIPCLDT0: LOGICAL: REMOVE NEGATIVE CLOUD WATER/ICE AND BOUND CLOUD COVER AT INITIAL TIME
! LDUCTDIA : LOGICAL  : COMPUTATIONS AND ARCHIVING OF DUCTING DIAGNOSTICS
! LESN09 : LOGICAL  : IF true use the snow 2009 scheme (liquid water, density, snow cover fraction, exposed and forest albedo)
! LELAIV : LOGICAL  : IF FALSE OVERWRITE CLIMATE LAI FIELDS WITH LUT VALUES
! LESMOSS : LOGICAL : SMOS SEA SURFACE SALINITY MONITORING
! LDYNVAR : LOGICAL : Save DYNVAR output
!     -----------------------------------------------------------------
END MODULE YOEPHY
