MODULE YOMANCS

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!*
!      YOMANCS - ANALYSIS NUMERICAL CONSTANTS
!                     DEFINED IN SUBROUTINE SUANCT

!        D. VASILJEVIC     ECMWF    06/03/1990

!     ANALYSIS NUMERICAL CONSTANTS
!     VARIABLE / TYPE /                   MEANING                      /
!     --------   ----                     -------
!      NMDI       I      MISSING INDICATOR (INTEGER)
!      RMDI       I      MISSING INDICATOR (REAL)
!      RMDB       I      BUFR MISSING INDICATOR
!      RUPMLIM
!      RDWMLIM
!      EPSBUFR
!      NBITS   () I      MASKING CONSTANTS
!      NOBTIS     I      NO. OF BITS IN A WORD
!      NOBYTES    I       NO. OF BYTES PER WORD
!      NCMBYTES   I      NO. OF BYTES USED BY CMA
!      RADIANS    R      RADIANS
!      RDEGREES   R      DEGREES
!      EPSMAC     R      MACHINE PRECISION
!      RSQRT2PI   R      SQRT(2*PI), USED IN VARIOATIONAL QC
!      RNLATLIM   R      LAT. LIMIT NORTH
!      RSLATLIM   R      LAT. LIMIT SOUTH
!      RWLONLIM   R      LON. LIMIT WEST
!      RELONLIM   R      LON. LIMIT EAST
!      RSYNANMH   R      SYNOP (NON-SHIP) ANEMOMETER HEIGHT
!      RSYNBARH   R      SYNOP (NON-SHIP) BAROMETER HEIGHT
!      RSHIPDH    R      SHIP DECK HEIGHT
!      RSANMOMH   R      SHIP ANEMOMETER HEIGHT
!      RSBAROMH   R      SHIP BAROMETER HEIGHT
!      RDRIBUDH   R      DRIBU DECK HEIGHT
!      RDANMOMH   R      DRIBU ANEMOMETER HEIGHT
!      RDBAROMH   R      DRIBU BAROMETER HEIGHT

!      RDAYPYEA   I    NO. OF DAYS PER YEAR
!      RHOUPDAY   I    NO. HOURS PER DAY
!      NHOUPDAY   I    NO. HOURS PER DAY
!      RMINPHOU   I    NO. OF MINUTES PER HOUR
!      NMINPHOU   I    NO. OF MINUTES PER HOUR
!      RSECPMIN   I    NO. OF SECONDES PER HOUR
!      NSECPMIN   I    NO. OF SECONDES PER HOUR
!      RDAYFINH   I    FRACTION OF A DAY IN HOURS
!      NDAYFINH   I    FRACTION OF A DAY IN HOURS
!      RREDPERR   I    PERSISTENCE ERROR REDUCTION
!      RREDPEW1   I    PERSISTENCE ERROR WEIGHT REDUCTION 1
!      RREDPEW2   I    PERSISTENCE ERROR WEIGHT REDUCTION 2
!      RGWOSW     R    RATIO OF GEOSTROPHIC TO SURFACE WIND OVER SEA
!      RAIRDEN    R    DENSITY OF AIR
!      RSHPCO     R    CONSTANT FOR SHIP PRESSURE TENDENCY CORRECTION
!      RSINAG     R    SIN OF ANGLE BETWEEN ACTUAL AND GEOSTROPHIC WIND
!      RCOSAG     R    COS OF ANGLE BETWEEN ACTUAL AND GEOSTROPHIC WIND
!      RPAPM      R    PRESSURE CONVERSION

!      RT0        R
!      RKELVN     R
!      RTT        R    =RT0=273.16= TRIPLE POINT TEMPERATURE
!      R2ES       R    CONSTANTS USED FOR COMPUTATION OF SATURATION
!      R3LES      R    MIXING RATIO OVER LIQUID WATER(*R_LES*) OR
!      R3IES      R    ICE(*R_IES*).
!      R4LES      R
!      R4IES      R
!      RTWAT      R    RTWAT=RTT
!      RTICE      R    RTICE=RTT-23
!      RRD        R    DRY AIR GAS CONSTANT
!      RRV        R    WATER VAPOUR GAS CONSTANT

INTEGER(KIND=JPIM) :: NMDI
INTEGER(KIND=JPIM) :: NOBITS
INTEGER(KIND=JPIM) :: NOBYTES
INTEGER(KIND=JPIM) :: NCMBYTES
!INTEGER_M :: NBITS(JPMXBIT)
INTEGER(KIND=JPIM) :: NBITS(32)

INTEGER(KIND=JPIM) :: NHOUPDAY
INTEGER(KIND=JPIM) :: NMINPHOU
INTEGER(KIND=JPIM) :: NSECPMIN
INTEGER(KIND=JPIM) :: NDAYFINH

REAL(KIND=JPRB) :: RMDI
REAL(KIND=JPRB) :: RMDB
REAL(KIND=JPRB) :: RUPMLIM
REAL(KIND=JPRB) :: RDWMLIM
REAL(KIND=JPRB) :: EPSBUFR
REAL(KIND=JPRB) :: RADIANS
REAL(KIND=JPRB) :: RDEGREES
REAL(KIND=JPRB) :: EPSMAC
REAL(KIND=JPRB) :: RSQRT2PI
REAL(KIND=JPRB) :: RNLATLIM
REAL(KIND=JPRB) :: RSLATLIM
REAL(KIND=JPRB) :: RWLONLIM
REAL(KIND=JPRB) :: RELONLIM
REAL(KIND=JPRB) :: RSYNANMH
REAL(KIND=JPRB) :: RSYNBARH
REAL(KIND=JPRB) :: RSHIPDH
REAL(KIND=JPRB) :: RSANMOMH
REAL(KIND=JPRB) :: RSBAROMH
REAL(KIND=JPRB) :: RDRIBUDH
REAL(KIND=JPRB) :: RDANMOMH
REAL(KIND=JPRB) :: RDBAROMH

REAL(KIND=JPRB) :: RDAYPYEA
REAL(KIND=JPRB) :: RHOUPDAY
REAL(KIND=JPRB) :: RMINPHOU
REAL(KIND=JPRB) :: RSECPMIN
REAL(KIND=JPRB) :: RDAYFINH
REAL(KIND=JPRB) :: RREDPERR
REAL(KIND=JPRB) :: RREDPEW1
REAL(KIND=JPRB) :: RREDPEW2
REAL(KIND=JPRB) :: RGWOSW
REAL(KIND=JPRB) :: RAIRDEN
REAL(KIND=JPRB) :: RSHPCO
REAL(KIND=JPRB) :: RSINAG
REAL(KIND=JPRB) :: RCOSAG
REAL(KIND=JPRB) :: RPAPM

REAL(KIND=JPRB) :: RKELVN
REAL(KIND=JPRB) :: RT0
REAL(KIND=JPRB) :: RTT
REAL(KIND=JPRB) :: R2ES
REAL(KIND=JPRB) :: R3LES
REAL(KIND=JPRB) :: R3IES
REAL(KIND=JPRB) :: R4LES
REAL(KIND=JPRB) :: R4IES
REAL(KIND=JPRB) :: RTWAT
REAL(KIND=JPRB) :: RTICE
REAL(KIND=JPRB) :: RRD
REAL(KIND=JPRB) :: RRV

!     ------------------------------------------------------------------

END MODULE YOMANCS
