MODULE YOMCLA

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

! ===========================================================================
! Variables used in part 1 of conf 923 (orography and envelope).
! ===========================================================================

! LNORO    : orography read from an external file.
! LNLSM    : land-sea mask read from an external file, possible only if LNORO=T.
! LNEWORO  : use Bouteloup orography optimisation cost function.
! LNEWORO2 : use Jerczynski orography optimisation cost function.
! LSPSMORO : use a diffusion operator for spectral smoothing of orography (linear grid).
! LKEYF    : spectral orography calculation.

LOGICAL :: LNORO
LOGICAL :: LNLSM
LOGICAL :: LNEWORO
LOGICAL :: LNEWORO2
LOGICAL :: LSPSMORO
LOGICAL :: LKEYF

! NLISSZ: smoothing number of iterations (contribution from the orography to
!         the roughness length).
! NLISSR: number of iterations to smooth orography.
! NLISSP: type of smoothing (0 = no smoothing).

INTEGER(KIND=JPIM) :: NLISSZ
INTEGER(KIND=JPIM) :: NLISSR
INTEGER(KIND=JPIM) :: NLISSP

! FACZ0:  factor to tune the orographic component of the roughness length.
! FENVN:  orography envelope: factor of the added height on pole of interest
!         (high resolution pole in ARPEGE).
! FENVS:  cf. FENVN, but in ARPEGE only, at the pole of low resolution.
! QMAX:   maximum weight in the orography optimisation cost function.
! QMIN:   minimum weight in the orography optimisation cost function.
! HDIM:   scale factor for the difference to the initial orography.
! HMIN:   reference height in the weight calculation.
! FACE:   1. for the initial formula, 0. for a polynomial function
!         (if Jerczynski cost tunction).
! QPOWER: exponent of the polynomial term (if Jerczynski cost tunction).
! QCONST: fraction of the polynomial term (if Jerczynski cost tunction).
! XINCOC: part of the sea fraction in the weight if FACE=1
!         (if Jerczynski cost tunction).
! SCEXT:  modulation factor in the extension zone (0. if nothing done).
! FLISA:  tuning parameter if NLISSP=2.
! FLISB:  smoothing threshold (wave number) if NLISSP=2.

REAL(KIND=JPRB) :: FACZ0
REAL(KIND=JPRB) :: FENVN
REAL(KIND=JPRB) :: FENVS
REAL(KIND=JPRB) :: QMAX
REAL(KIND=JPRB) :: QMIN
REAL(KIND=JPRB) :: HDIM
REAL(KIND=JPRB) :: HMIN
REAL(KIND=JPRB) :: FACE
REAL(KIND=JPRB) :: QPOWER
REAL(KIND=JPRB) :: QCONST
REAL(KIND=JPRB) :: XINCOC
REAL(KIND=JPRB) :: SCEXT
REAL(KIND=JPRB) :: FLISA
REAL(KIND=JPRB) :: FLISB

END MODULE YOMCLA
