MODULE YOMCMBDY

USE PARKIND1,ONLY : JPIM
USE PARCMA,  ONLY : JPMXAD, JPMXUP, JPMXOCT, JPMXOTP

IMPLICIT NONE

SAVE

!*    COMDECK YOMCMBDY - CMA BODY ENTRY FORMAT

!        D. VASILJEVIC     ECMWF   08/09/1994

!*     YOMCMBF - CMA FIXED PART BODY ENTRY FORMAT

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     SELF-DEF. NCMFBL       I     FIXED PART OF BODY ENTRY LENGTH
!         "     NCMFIBL      I     FIXED PART BODY ENTRY INT. SEC. LEN.
!         "     NCMFRBL      I     FIXED PART BODY ENTRY REAL SEC. LEN.

!         1     NCMVNM       I     VARIABLE NUMBER
!         2     NCMVCO       I     VERTICAL COORDINATE TYPE
!         3     NCMRDFL      IP    OBSERVATION FLAGS (RDB)
!         4     NCMFLG       IP    OBSERVATION FLAGS
!         5     NCMDSTA      IP    OBSERVATION STATUS
!         6     NCMDEV1      IP    OBSERVATION EVENTS (PART 1)
!         7     NCMDBLE      IP    OBSERVATION BLACKLIST EVENTS
!         8     NCMESQN      I     ENTRY SQ. NO. 
!         9     NCMPPP       R     VERTICAL COORDINATE REFERENCE 1
!        10     NCMPRL       R     VERTICAL COORDINATE REFERENCE 2
!        11     NCMVAR       R     OBSERVED VARIABLE
!        12     NCMOMN       R     OBSERVED MINUS ANALYSED VALUE
!        13     NCMOMF       R     OBSERVED MINUS FIRST GUESS VALUE
!        14     NCMFOE       R     FINAL OBSERVATION ERROR
!        15     NCMOER       R     OBSERVATION ERROR
!        16     NCMRER       R     REPRESENTATIVENESS ERROR
!        17     NCMPER       R     PERSISTENCE ERROR
!        18     NCMFGE       R     FIRST GUESS ERROR
!        19     NCMFGC1      R     FIRST GUESS CHECK CONSTANT 1
!        20     NCMFGC2      I     FIRST GUESS CHECK CONSTANT 2

INTEGER(KIND=JPIM) :: NCMFBL            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFIBL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFRBL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMVNM            ! ROBODY pk5int:varno@body
INTEGER(KIND=JPIM) :: NCMVCO            ! ROBODY pk5int:vertco_type@body
INTEGER(KIND=JPIM) :: NCMRDFL           ! ROBODY DATUM_RDBFLAG_t:rdbflag@body
INTEGER(KIND=JPIM) :: NCMFLG            ! ROBODY DATUM_FLAG_t:anflag@body
INTEGER(KIND=JPIM) :: NCMDSTA           ! ROBODY STATUS_t:status@body
INTEGER(KIND=JPIM) :: NCMDEV1           ! ROBODY DATUM_EVENT1_t:event1@body
INTEGER(KIND=JPIM) :: NCMDBLE           ! ROBODY DATUM_BLACKLIST_t:blacklist@body
INTEGER(KIND=JPIM) :: NCMESQN           ! ROBODY pk5int:entryno@body
INTEGER(KIND=JPIM) :: NCMPPP            ! ROBODY pk9real:press@body
INTEGER(KIND=JPIM) :: NCMPRL            ! ROBODY pk9real:press_rl@body
INTEGER(KIND=JPIM) :: NCMVAR            ! ROBODY real8:obsvalue@body
!ODB pk9real:aux1@body AS ROBODY
INTEGER(KIND=JPIM) :: NCMOMN            ! ROBODY pk9real:an_depar@update
INTEGER(KIND=JPIM) :: NCMOMF            ! ROBODY pk9real:fg_depar@update
INTEGER(KIND=JPIM) :: NCMFOE            ! ROBODY pk9real:final_obs_error@errstat
INTEGER(KIND=JPIM) :: NCMOER            ! ROBODY pk9real:obs_error@errstat
INTEGER(KIND=JPIM) :: NCMRER            ! ROBODY pk9real:repres_error@errstat
INTEGER(KIND=JPIM) :: NCMPER            ! ROBODY pk9real:pers_error@errstat
INTEGER(KIND=JPIM) :: NCMFGE            ! ROBODY pk9real:fg_error@errstat
INTEGER(KIND=JPIM) :: NCMFGC1           ! ROBODY pk9real:fg_check_1@update
INTEGER(KIND=JPIM) :: NCMFGC2           ! ROBODY pk9real:fg_check_2@update

!-----------------------------------------------------------------------

!*     YOMCMBFP - CMA FIXED BODY ENTRY PACKED INFO. FORMAT

!     PRESSURE AND VARIABLE (RDB) FLAGS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMRDFL   NCMBPFBP     I     PRESSURE FLAG BIT POS.
!               NCMBPFOC     I     PRESSURE FLAG BITS OCC.
!               NCMBVFBP     I     VARIABLE FLAG BIT POS.
!               NCMBVFOC     I     VARIABLE FLAG BITS OCC.

!        BOTH PRESSURE AND VARIABLE FLAGS HAVE THE SAME STRUCTURE:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------

!               NCMBHSBP     I     HUMAN MONITOR SUBS. BIT POS.
!               NCMBHSOC     I     HUMAN MONITOR SUBS. BITS OCC.
!               NCMBQSBP     I     Q/C SUBS. BIT POS.
!               NCMBQSOC     I     Q/C SUBS. BITS OCC.
!               NCMBORBP     I     OVER-RIDE FLAG BIT POS.
!               NCMBOROC     I     OVER-RIDE FLAG BITS OCC.
!               NCMBFLBP     I     PARAM. FLAG BIT POS. 
!               NCMBFLOC     I     PARAM. FLAG BITS OCC.
!               NCMBSHBP     I     PARAM. FLAG SET BY H/QC BIT POS.
!               NCMBSHOC     I     PARAM. FLAG SET BY H/QC BITS OCC.
!               NCMBPABP     I     PAR. JUDGED BY PREV. ANAL. BIT POS.
!               NCMBPAOC     I     PAR. JUDGED BY PREV. ANAL. BITS OCC.
!               NCMBUABP     I     PARAM. USED BY PREV. ANAL. BIT POS.
!               NCMBUAOC     I     PARAM. USED BY PREV. ANAL. BITS OCC.

!     ANALYSIS VARIABLE FLAGS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMFLG    NCMAFFBP     I     FINAL FLAG BIT POS.
!               NCMAFFOC     I     FINAL FLAG BITS OCC.
!               NCMAGFBP     I     FIRST GUESS FLAG BIT POS.
!               NCMAGFOC     I     FIRST GUESS FLAG BITS OCC.
!               NCMADFBP     I     DEPARTURE FLAG BIT POS.
!               NCMADFOC     I     DEPARTURE FLAG BITS OCC.
!               NCMAVFBP     I     VARIATIONAL QUALITY FLAG BIT POS.
!               NCMAVFOC     I     VARIATIONAL QUALITY FLAG BITS OCC.
!               NCMABFBP     I     BLACKLIST FLAG BIT POS.
!               NCMABFOC     I     BLACKLIST FLAG BITS OCC.

!        BOTH PRESSURE AND VARIABLE FLAGS HAVE THE SAME STRUCTURE

!     DATUM STATUS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMDSTA   NCMDACBP     I     ACTIVE DATUM BIT POS.
!               NCMDACOC     I     ACTIVE DATUM BITS OCC.
!               NCMDPABP     I     PASSIVE DATUM BIT POS.
!               NCMDPAOC     I     PASSIVE DATUM BITS OCC.
!               NCMDREBP     I     REJECTED DATUM BIT POS.
!               NCMDREOC     I     REJECTED DATUM BITS OCC.
!               NCMDBLBP     I     BLACKLISTED DATUM BIT POS.
!               NCMDBLOC     I     BLACKLISTED DATUM BITS OCC.

!     DATUM EVENTS (PART 1):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMDEV1

!     DATUM BLACKLIST EVENTS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMDBLE

INTEGER(KIND=JPIM) :: NCMBPFBP
INTEGER(KIND=JPIM) :: NCMBPFOC ! BITMAP bit15:press@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBVFBP
INTEGER(KIND=JPIM) :: NCMBVFOC ! BITMAP bit15:varno@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBHSBP
INTEGER(KIND=JPIM) :: NCMBHSOC ! BITMAP merge_bit1:humon@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBQSBP
INTEGER(KIND=JPIM) :: NCMBQSOC ! BITMAP merge_bit1:QCsub@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBORBP
INTEGER(KIND=JPIM) :: NCMBOROC ! BITMAP merge_bit1:override@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBFLBP
INTEGER(KIND=JPIM) :: NCMBFLOC ! BITMAP merge_bit2:flag@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBSHBP
INTEGER(KIND=JPIM) :: NCMBSHOC ! BITMAP merge_bit1:HQC_flag@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBPABP
INTEGER(KIND=JPIM) :: NCMBPAOC ! BITMAP merge_bit2:judged_prev_an@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMBUABP
INTEGER(KIND=JPIM) :: NCMBUAOC ! BITMAP merge_bit1:used_prev_an@DATUM_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMAFFBP
INTEGER(KIND=JPIM) :: NCMAFFOC ! BITMAP bit4:final@DATUM_FLAG_t
INTEGER(KIND=JPIM) :: NCMAGFBP
INTEGER(KIND=JPIM) :: NCMAGFOC ! BITMAP bit4:fg@DATUM_FLAG_t
INTEGER(KIND=JPIM) :: NCMADFBP
INTEGER(KIND=JPIM) :: NCMADFOC ! BITMAP bit4:depar@DATUM_FLAG_t
INTEGER(KIND=JPIM) :: NCMAVFBP
INTEGER(KIND=JPIM) :: NCMAVFOC ! BITMAP bit4:varQC@DATUM_FLAG_t
INTEGER(KIND=JPIM) :: NCMABFBP
INTEGER(KIND=JPIM) :: NCMABFOC ! BITMAP bit4:blacklist@DATUM_FLAG_t
INTEGER(KIND=JPIM) :: NCMDACBP
INTEGER(KIND=JPIM) :: NCMDACOC ! BITMAP bit1:active@STATUS_t
INTEGER(KIND=JPIM) :: NCMDPABP
INTEGER(KIND=JPIM) :: NCMDPAOC ! BITMAP bit1:passive@STATUS_t
INTEGER(KIND=JPIM) :: NCMDREBP
INTEGER(KIND=JPIM) :: NCMDREOC ! BITMAP bit1:rejected@STATUS_t
INTEGER(KIND=JPIM) :: NCMDBLBP
INTEGER(KIND=JPIM) :: NCMDBLOC ! BITMAP bit1:blacklisted@STATUS_t

!-----------------------------------------------------------------------

!*     YOMCMBR - CMA RUN DEPENDENT PART BODY ENTRY FORMAT

!     INCREMENTAL RUN (REPEATED FOR NO. OF UPDATES):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     SELF-DEF. NCMRBLN      I     RUN DEPENDENT PART BODY ENTRY LEN.
!         "     NCMRBIN      I     R. D. BODY ENTRY PART INT. SEC. LEN.
!         "     NCMRBRN      I     R. D. BODY ENTRY PART REAL SEC. LEN.

!               NCMIOM0      R     OBS. MINUS UPD. U INITIAL VALUE
!               NCMIFC1      R     OBS. MINUS UPD. U HIGH RES. VALUE
!               NCMIFC2      R     OBS. MINUS UPD. U LOW RES. VALUE
!               NCMIOMN      R     OBS. MINUS UPD. U FINAL VALUE
!               NCMIOMSN     R     OBS. MINUS UPD. U SIMULATION N VALUE

!     NON-INCREMENTAL RUN:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMIOMSN     R     OBS. MINUS UPD. U SIMULATION N VALUE

!     CANARI RUN (french OI):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMRBVC      I     vertical coordonate type
!               NCMRBPIO     R     pressure used in CANARI (Log P)
!               NCMRBOE      R     obs. std dev at bottom layer

INTEGER(KIND=JPIM) :: NCMRBLN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMRBIN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMRBRN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMIOM0(JPMXUP)   ! ROBODY pk9real:initial[$NMXUPD]@update
INTEGER(KIND=JPIM) :: NCMIFC1(JPMXUP)   ! ROBODY pk9real:hires[$NMXUPD]@update
INTEGER(KIND=JPIM) :: NCMIFC2(JPMXUP)   ! ROBODY pk9real:lores[$NMXUPD]@update
INTEGER(KIND=JPIM) :: NCMIOMN(JPMXUP)   ! ROBODY pk9real:final[$NMXUPD]@update
INTEGER(KIND=JPIM) :: NCMIOMSN(JPMXAD,JPMXUP) ! OMIT_ROBSIM pk9real:value[$JPMXAD*$NMXUPD]@simul
INTEGER(KIND=JPIM) :: NCMRBVC           ! ROBODY pk9int:mf_vertco_type@update
INTEGER(KIND=JPIM) :: NCMRBPIO          ! ROBODY pk9real:mf_log_p@update
INTEGER(KIND=JPIM) :: NCMRBOE           ! ROBODY pk9real:mf_stddev@update

!-----------------------------------------------------------------------

!*     YOMCMBRP- CMA RUN DEPENDENT PART PACKED INFO BODY ENTRY
!                      FORMAT

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------

!                            /YOMCMBRP/

!-----------------------------------------------------------------------

!*     YOMCMBT - CMA OPTIONAL (OBS./CODE TYPE DEPENDENT) DEPENDENT
!                      PART BODY ENTRY FORMAT

!     SYNOP/AIREP/DRIBU/SATOB/TEMP/PILOT/SATEM/PAOB/SCATT./R.RAD.

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     SELF-DEF. NCMTBLN      I     OPTIONAL PART BODY ENTRY LENGTHS
!         "     NCMTBLNI     I     OPT. PART BODY ENTRY INT. SEC. LEN.

!               NCMDEV2      I     DATUM EVENTS (PART 2) WORD POS.

!     SYNOP:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSYPC      I     PRESSURE CODE

!     TEMP:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMTELID     I     TEMP LEVEL ID.
!               NCMTETII     R     TEMP TIME INCREMENT
!               NCMTELTI     R     TEMP LATITUDE INCREMENT
!               NCMTELNI     R     TEMP LONGITUDE INCREMENT

!     PILOT:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMPILID     I     PILOT LEVEL ID.
!               NCMPITII     R     PILOT TIME INCREMENT
!               NCMPILTI     R     PILOT LATITUDE INCREMENT
!               NCMPILNI     R     PILOT LONGITUDE INCREMENT

!     SATEM (TOVS RADIANCE):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMTORB      R     RADIANCE BIAS CORRECTION
!               NCM1DVC      R     1D VAR RADIANCE COST 
!               NCMTORDE     R     RADIANCE DEPARTURE

!     SATEM (ATOVS; LEVEL 1C/GEOS RADIANCE):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMTORBA     R     RADIANCE BIAS CORRECTION
!               NCM1DVCA     R     1D VAR RADIANCE COST 
!               NCMTORDEA    R     RADIANCE DEPARTURE

!     SATEM (SSMI/TRMM):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSSRB      R     RADIANCE BIAS CORRECTION
!               NCMS1DVC     R     1D VAR RADIANCE COST 
!               NCMSSRDE     R     RADIANCE DEPARTURE
!               NCMSSCCF     R     CHANNEL CENTRE FREQUENCY
!               NCMSSCBW     R     CHANNEL BAND WIDTH
!               NCMSSANP     R     ANTENNA POLARISATION

!     SATEM (RETRIEVED OZONE LAYERS):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMRO3HEI    R     (TANGENT) HEIGHT

!     SCATT.:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSCBAA     R     BEAM AZIMUTH ANGLE
!               NCMSCBIA     R     BEAM INCIDENCE ANGLE
!               NCMSCKP      R     Kp (INSTRUMENT NOISE)
!               NCMSCRES     R     INVERSION RESIDUAL
!               NCMSCDIS     R     DIRECTIONAL SKILL  

INTEGER(KIND=JPIM) :: NCMTBLN(JPMXOCT,JPMXOTP)  ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMTBLNI(JPMXOCT,JPMXOTP) ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMDEV2            &! ROBODY DATUM_EVENT2_t:event2@body     
 & (JPMXOCT,JPMXOTP)  
INTEGER(KIND=JPIM) :: NCMSYPC           ! ROBODY pk9int:ppcode@body
INTEGER(KIND=JPIM) :: NCMTELID          ! ROBODY LEVEL_t:level@body
INTEGER(KIND=JPIM) :: NCMTETII          ! OMIT_SATBODY pk9real:time_inc@tempilot_body
INTEGER(KIND=JPIM) :: NCMTELTI          ! OMIT_SATBODY pk9real:lat_inc@tempilot_body
INTEGER(KIND=JPIM) :: NCMTELNI          ! OMIT_SATBODY pk9real:lon_inc@tempilot_body
INTEGER(KIND=JPIM) :: NCMPILID          ! ROBODY LEVEL_t:level@body
INTEGER(KIND=JPIM) :: NCMPITII          ! OMIT_SATBODY pk9real:time_inc@tempilot_body
INTEGER(KIND=JPIM) :: NCMPILTI          ! OMIT_SATBODY pk9real:lat_inc@tempilot_body
INTEGER(KIND=JPIM) :: NCMPILNI          ! OMIT_SATBODY pk9real:lon_inc@tempilot_body
INTEGER(KIND=JPIM) :: NCMTORB           ! ROBODY pk9real:biascorr@body
INTEGER(KIND=JPIM) :: NCM1DVC           ! SATBODY pk9real:radcost@rtovs_body
INTEGER(KIND=JPIM) :: NCMTORDE          ! SATBODY pk9real:depar@rtovs_body
INTEGER(KIND=JPIM) :: NCMTORBA          ! ROBODY pk9real:biascorr@body
INTEGER(KIND=JPIM) :: NCM1DVCA          ! OMIT_SATBODY pk9real:radcost@atovs_body
INTEGER(KIND=JPIM) :: NCMTORDEA         ! OMIT_SATBODY pk9real:depar@atovs_body
INTEGER(KIND=JPIM) :: NCMSSRB           ! ROBODY pk9real:biascorr@body
INTEGER(KIND=JPIM) :: NCMS1DVC          ! SATBODY pk9real:radcost@ssmi_body
INTEGER(KIND=JPIM) :: NCMSSRDE          ! SATBODY pk9real:depar@ssmi_body
INTEGER(KIND=JPIM) :: NCMSSCCF          ! SATBODY pk9real:frequency@ssmi_body
INTEGER(KIND=JPIM) :: NCMSSCBW          ! SATBODY pk9real:bandwidth@ssmi_body
INTEGER(KIND=JPIM) :: NCMSSANP          ! SATBODY pk9real:polarisation@ssmi_body
INTEGER(KIND=JPIM) :: NCMSCBAA          ! SATBODY pk9real:azimuth@scatt_body
INTEGER(KIND=JPIM) :: NCMSCBIA          ! SATBODY pk9real:incidence@scatt_body
INTEGER(KIND=JPIM) :: NCMSCKP           ! SATBODY pk9real:Kp@scatt_body
INTEGER(KIND=JPIM) :: NCMSCRES          ! SATBODY pk9real:invresid@scatt_body
INTEGER(KIND=JPIM) :: NCMSCDIS          ! SATBODY pk9real:dirskill@scatt_body
INTEGER(KIND=JPIM) :: NCMRO3HEI         ! ROBODY  pk9real:aux1@body

!-----------------------------------------------------------------------

!*     YOMCMBTP- CMA OPTIONAL (OBS./CODE TYPE DEPENDENT) DEPENDENT
!                      PART BODY ENTRY PACKED INFO. FORMAT

!     SYNOP/AIREP/SATOB/DRIBU/TEMP/PILOT/SATEM/PAOB/SCATT./R.RADIANCE

!     DATUM EVENTS (PART 2):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMDEV2

!     TEMP:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMTELID  NTLIDBP      I     TEMP LEV. ID. BIT POS.
!               NTLIDOC      I     TEMP LEV. ID. BITS OCC.

!       STRUCTURE OF THE LEVEL ID IS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NTMWLBP      I     MAX WIND LEVEL BIT POS.
!               NTMWLOC      I     MAX WIND LEVEL BITS OCC.
!               NTTROBP      I     TROPOPAUSE BIT POS.
!               NTTROOC      I     TROPOPAUSE BITS OCC. 
!               NTDPABP      I     D PART BIT POS.
!               NTDPAOC      I     D PART BITS OCC.
!               NTCPABP      I     C PART BIT POS.
!               NTCPAOC      I     C PART BITS OCC.
!               NTBPABP      I     B PART BIT POS. 
!               NTBPAOC      I     B PART BITS OCC.
!               NTAPABP      I     A PART BIT POS.
!               NTAPAOC      I     A PART BITS OCC.
!               NTSURBP      I     SURFACE LEVEL BIT POS.
!               NTSUROC      I     SURFACE LEVEL BITS OCC.
!               NTSWLBP      I     SIGNIFICANT WIND LEVEL BIT POS.
!               NTSWLOC      I     SIGNIFICANT WIND LEVEL BITS OCC.
!               NTSTLBP      I     SIGNIFICANT TEMPR. LEVEL BIT POS.
!               NTSTLOC      I     SIGNIFICANT TEMPR. LEVEL BITS OCC.

!     PILOT:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMPILID  NPLIDBP      I     PILOT LEV. ID. BIT POS.
!               NPLIDOC      I     PILOT LEV. ID. BITS OCC.

!       STRUCTURE OF THE LEVEL ID IS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NPMWLBP      I     MAX WIND LEVEL BIT POS.
!               NPMWLOC      I     MAX WIND LEVEL BITS OCC.
!               NPTROBP      I     TROPOPAUSE BIT POS.
!               NPTROOC      I     TROPOPAUSE BITS OCC. 
!               NPDPABP      I     D PART BIT POS.
!               NPDPAOC      I     D PART BITS OCC.
!               NPCPABP      I     C PART BIT POS.
!               NPCPAOC      I     C PART BITS OCC.
!               NPBPABP      I     B PART BIT POS. 
!               NPBPAOC      I     B PART BITS OCC.
!               NPAPABP      I     A PART BIT POS.
!               NPAPAOC      I     A PART BITS OCC.
!               NPSURBP      I     SURFACE LEVEL BIT POS.
!               NPSUROC      I     SURFACE LEVEL BITS OCC.
!               NPSWLBP      I     SIGNIFICANT WIND LEVEL BIT POS.
!               NPSWLOC      I     SIGNIFICANT WIND LEVEL BITS OCC.
!               NPSTLBP      I     SIGNIFICANT TEMPR. LEVEL BIT POS.
!               NPSTLOC      I     SIGNIFICANT TEMPR. LEVEL BITS OCC.

INTEGER(KIND=JPIM) :: NTLIDBP
INTEGER(KIND=JPIM) :: NTLIDOC ! BITMAP bit9:id@LEVEL_t
INTEGER(KIND=JPIM) :: NTMWLBP
INTEGER(KIND=JPIM) :: NTMWLOC ! BITMAP bit1:maxwind@LEVEL_t
INTEGER(KIND=JPIM) :: NTTROBP
INTEGER(KIND=JPIM) :: NTTROOC ! BITMAP bit1:tropopause@LEVEL_t
INTEGER(KIND=JPIM) :: NTDPABP
INTEGER(KIND=JPIM) :: NTDPAOC ! BITMAP bit1:D_part@LEVEL_t
INTEGER(KIND=JPIM) :: NTCPABP
INTEGER(KIND=JPIM) :: NTCPAOC ! BITMAP bit1:C_part@LEVEL_t
INTEGER(KIND=JPIM) :: NTBPABP
INTEGER(KIND=JPIM) :: NTBPAOC ! BITMAP bit1:B_part@LEVEL_t
INTEGER(KIND=JPIM) :: NTAPABP
INTEGER(KIND=JPIM) :: NTAPAOC ! BITMAP bit1:A_part@LEVEL_t
INTEGER(KIND=JPIM) :: NTSURBP
INTEGER(KIND=JPIM) :: NTSUROC ! BITMAP bit1:surface@LEVEL_t
INTEGER(KIND=JPIM) :: NTSWLBP
INTEGER(KIND=JPIM) :: NTSWLOC ! BITMAP bit1:signwind@LEVEL_t
INTEGER(KIND=JPIM) :: NTSTLBP
INTEGER(KIND=JPIM) :: NTSTLOC ! BITMAP bit1:signtemp@LEVEL_t
INTEGER(KIND=JPIM) :: NPLIDBP
INTEGER(KIND=JPIM) :: NPLIDOC ! BITMAP bit9:id@LEVEL_t
INTEGER(KIND=JPIM) :: NPMWLBP
INTEGER(KIND=JPIM) :: NPMWLOC ! BITMAP bit1:maxwind@LEVEL_t
INTEGER(KIND=JPIM) :: NPTROBP
INTEGER(KIND=JPIM) :: NPTROOC ! BITMAP bit1:tropopause@LEVEL_t
INTEGER(KIND=JPIM) :: NPDPABP
INTEGER(KIND=JPIM) :: NPDPAOC ! BITMAP bit1:D_part@LEVEL_t
INTEGER(KIND=JPIM) :: NPCPABP
INTEGER(KIND=JPIM) :: NPCPAOC ! BITMAP bit1:C_part@LEVEL_t
INTEGER(KIND=JPIM) :: NPBPABP
INTEGER(KIND=JPIM) :: NPBPAOC ! BITMAP bit1:B_part@LEVEL_t
INTEGER(KIND=JPIM) :: NPAPABP
INTEGER(KIND=JPIM) :: NPAPAOC ! BITMAP bit1:A_part@LEVEL_t
INTEGER(KIND=JPIM) :: NPSURBP
INTEGER(KIND=JPIM) :: NPSUROC ! BITMAP bit1:surface@LEVEL_t
INTEGER(KIND=JPIM) :: NPSWLBP
INTEGER(KIND=JPIM) :: NPSWLOC ! BITMAP bit1:signwind@LEVEL_t
INTEGER(KIND=JPIM) :: NPSTLBP
INTEGER(KIND=JPIM) :: NPSTLOC ! BITMAP bit1:signtemp@LEVEL_t

!-----------------------------------------------------------------------

END MODULE YOMCMBDY
