MODULE YOMCMHDR

USE PARKIND1,ONLY : JPIM
USE PARCMA,  ONLY : JPMXOCT, JPMXOTP, JPMXUP

IMPLICIT NONE

SAVE

!*    COMDECK YOMCMHDR - CMA HEADER FORMAT

!        D. VASILJEVIC     ECMWF   08/09/1994

!**   MODIFICATIONS
!     -------------
!     Niels Bormann        22/06/2001 : Added coldest cluster temp for satob
!     Lueder v. Bremen     01/05/2003 : Added channel frequencies and zenith
!                                       angle for satob

!*     YOMCMHF - FIXED PART CMA HEADER FORMAT

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     SELF-DEF. NCMFHL       I     FIXED HEADER PART LENGTH (TOTAL)
!         "     NCMFIHL      I     FIXED HEADER PART LENGTH (INT.)
!         "     NCMFCHL      I     FIXED HEADER PART LENGTH (CHA.)
!         "     NCMFRHL      I     FIXED HEADER PART LENGTH (REAL) 

!         1     NCMRLN       I     TOTAL REPORT LENGTH (INTEGERS+REALS)
!         2     NCMIRLN      I     INTEGER REPORT LENGTH (FIX.+OPT.)

!         3     NCMHLN       I     TOTAL HEADER LENGTH (INTEGERS+REALS)
!         4     NCMIHLN      I     INTEGER HEADER LENGTH (FIX.+OPT.)
!         5     NCMHFLN      I     FIXED HEADER PART LEN. (INT.+REAL)
!         6     NCMFHILN     I     FIXED HEADER PART LEN. (INTEGER)
!         7     NCMFHCLN     I     FIXED HEADER PART LEN. (CHARACT)
!         8     NCMOHILN     I     OPT. HEADER PART LEN. (INTEGER)

!         9     NCMBLN       I     TOTAL BODY ENTRY LEN. (INT.+REAL)
!        10     NCMBILN      I     BODY ENTRY LENGTH (INTEGER)
!        11     NCMBFL       I     TOTAL FIXED BODY ENTRY PART LENGTH
!        12     NCMBFIL      I     INTEGER F. BODY ENTRY PART LENGTH
!        13     NCMBRL       I     TOTAL RUN DEP. BODY ENTRY PART LEN.
!        14     NCMBRIL      I     INTEGER  R. D. BODY ENTRY PART LEN.
!        15     NCMBTL       I     TOTAL T. DEP. BODY ENTRY PART LEN.
!        16     NCMBTIL      I     INTEGER T. D. BODY ENTRY PART LEN.

!        17     NCMONM       I     OBSERVATION SEQUENCE
!        18     NCMOTP       I     OBSERVATION TYPE
!        19     NCMOCH       IP    OBSERVATION CHARACTERISTICS
!        20     NCMDAT       IP    OBS. DATE
!        21     NCMETM       IP    OBS. EXACT TIME
!        22     NCMNLV       I     NO. OF BODY ENTRIES
!        23     NCMRFL       IP    REPORT'S FLAGS
!        24     NCMRST       IP    REPORT'S STATUS
!        25     NCMREV1      IP    REPORT'S EVENTS (PART 1)
!        26     NCMRBLE      IP    REPORT'S BLACKLIST EVENTS
!        27     NCMBOX       I     SORTING BOX
!        28     NCMSTD       I     site dependent

!        29     NCMSUBTYPE   I     BUFR SUBTYPE FOR REFERENCE

!        30     NCMSID       I     STATION ID (CHARACTER NO. 1)
!        31     NCMSID2      I     STATION ID (CHARACTER NO. 2)
!        32     NCMSID3      I     STATION ID (CHARACTER NO. 3)
!        33     NCMSID4      I     STATION ID (CHARACTER NO. 4)
!        34     NCMSID5      I     STATION ID (CHARACTER NO. 5)
!        35     NCMSID6      I     STATION ID (CHARACTER NO. 6)
!        36     NCMSID7      I     STATION ID (CHARACTER NO. 7)
!        37     NCMSID8      I     STATION ID (CHARACTER NO. 8)

!        38     NCMLAT       R     LATITUDE
!        39     NCMLON       R     LONGITUDE
!        40     NCMALT       R     ALTITUDE
!        41     NCMMOR       R     MODEL'S OROGRAPHY
!        42     NCMTLA       R     TRANSFORMED LAT.
!        43     NCMTLO       R     TRANSFORMED LON.

INTEGER(KIND=JPIM) :: NCMFHL            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFIHL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFCHL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFRHL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMRLN            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMIRLN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMHLN            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMIHLN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMHFLN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFHILN          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMFHCLN          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMOHILN          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBLN            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBILN           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBFL            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBFIL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBRL            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBRIL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBTL            ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMBTIL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMONM            ! ROBHDR int:seqno@hdr
INTEGER(KIND=JPIM) :: NCMOTP            ! ROBHDR pk5int:obstype@hdr
INTEGER(KIND=JPIM) :: NCMOCH            ! ROBHDR OBSCHAR_t:obschar@hdr
INTEGER(KIND=JPIM) :: NCMDAT            ! ROBHDR YYYYMMDD:date@hdr
INTEGER(KIND=JPIM) :: NCMETM            ! ROBHDR HHMMSS:time@hdr
INTEGER(KIND=JPIM) :: NCMNLV            ! OMIT_MOBHDR pk5int:nlev@hdr
INTEGER(KIND=JPIM) :: NCMRFL            ! ROBHDR REPORT_RDBFLAG_t:rdbflag@hdr
INTEGER(KIND=JPIM) :: NCMRST            ! ROBHDR STATUS_t:status@hdr
INTEGER(KIND=JPIM) :: NCMREV1           ! ROBHDR REPORT_EVENT1_t:event1@hdr
INTEGER(KIND=JPIM) :: NCMRBLE           ! ROBHDR REPORT_BLACKLIST_t:blacklist@hdr
INTEGER(KIND=JPIM) :: NCMBOX            ! ROBHDR pk9int:sortbox@hdr
INTEGER(KIND=JPIM) :: NCMSTD            ! ROBHDR pk9int:sitedep@hdr
INTEGER(KIND=JPIM) :: NCMSUBTYPE        ! ROBHDR pk5int:subtype@hdr
INTEGER(KIND=JPIM) :: NCMSID            ! ROBHDR string:statid@hdr
INTEGER(KIND=JPIM) :: NCMSID2           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSID3           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSID4           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSID5           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSID6           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSID7           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSID8           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMLAT            ! ROBHDR real8:lat@hdr
INTEGER(KIND=JPIM) :: NCMLON            ! ROBHDR real8:lon@hdr
INTEGER(KIND=JPIM) :: NCMALT            ! ROBHDR pk9real:stalt@hdr
INTEGER(KIND=JPIM) :: NCMMOR            ! ROBHDR pk9real:modoro@hdr
INTEGER(KIND=JPIM) :: NCMTLA            ! ROBHDR pk9real:trlat@hdr
INTEGER(KIND=JPIM) :: NCMTLO            ! ROBHDR pk9real:trlon@hdr

!-----------------------------------------------------------------------

!*     YOMCMHFP - CMA FIXED PART HEADER PACKED INFO. FORMAT

!        OBSERVATION CHARACTERISTICS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMOCH    NCDTBP       I     OBS. CODE TYPE BIT POS.
!               NCDTOC       I     OBS. CODE TYPE NO. OF BITS OCC.
!               NINSTBP      I     INS. TYPE BIT POS.
!               NINSTOC      I     INS. TYPE NO. OF BITS OCC.
!               NRETTBP      I     RETRIEVAL TYPE BIT POS.
!               NRETTOC      I     RETRIEVAL TYPE NO. OF BITS OCC.
!               NARETBP      I     GEOGRAPHICAL AREA BIT POS.
!               NARETOC      I     GEOGRAPHICAL AREA NO. OF BITS OCC.

!        DATE:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMDAT    NYYOFF       I     YEAR OFFSET
!               NMMOFF       I     MONTH OFFSET
!               NDDOFF       I     DAY OFFSET

!        EXACT TIME:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMETM    NHHOFF       I     HOUR OFFSET  
!               NMIOFF       I     MINUTE OFFSET
!               NSSOFF       I     SECONDE OFFSET

!        REPORT'S (RDB) FLAGS: 

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMRFL    NLTFLBP      I     LATITUDE FLAGS BIT POS.
!               NLTFLOC      I     LATITUDE FLAGS NO. OF BITS OCC.
!               NLNFLBP      I     LONGITUDE FLAGS BIT POS.
!               NLNFLOC      I     LONGITUDE FLAGS NO. OF BITS OCC.
!               NDTFLBP      I     DATE FLAGS BIT POS.
!               NDTFLOC      I     DATE FLAGS NO. OF BITS OCC.
!               NTMFLBP      I     TIME FLAGS BIT POS.
!               NTMFLOC      I     TIME FLAGS NO. OF BITS OCC.
!               NALFLBP      I     ALTITUDE FLAGS BIT POS.
!               NALFLOC      I     ALTITUDE FLAGS NO. OF BITS OCC.  

!         FOR EACH PARAMETER (LAT, LON, DATE, TIME AND ALT.) THE 
!         FLAG PATTERN HAS (THE SAME) FOLLOWING STRUCTURE: 

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------

!               NHMSBP       I     HUMAN MONITOR SUB. BIT POS.
!               NHMSOC       I     HUMAN MONITOR SUB. BITS OCC.
!               NQCSBP       I     Q/C SUB. BIT POS.
!               NQCSOC       I     Q/C SUB. BITS OCC.
!               NORFBP       I     OVER-RIDE FLAG BIT POS.
!               NORFOC       I     OVER-RIDE FLAG BITS OCC.
!               NFLGBP       I     PARAM FLAG BIT POS.
!               NFLGOC       I     PARAM FLAG BITS OCC.
!               NHFSBP       I     PARAM FLAG SET BY H/QC BIT POS.
!               NHFSOC       I     PARAM FLAG SET BY H/QC BITS OCC.

!        REPORT'S STATUS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMRST    NCMRACBP     I     ACTIVE REPORT BIT POS.
!               NCMRACOC     I     ACTIVE REPORT BITS OCC.
!               NCMRPABP     I     PASSIVE REPORT BIT POS.
!               NCMRPAOC     I     PASSIVE REPORT BITS OCC.
!               NCMRREBP     I     REJECTED REPORT BIT POS.
!               NCMRREOC     I     REJECTED REPORT BITS OCC.
!               NCMRBLBP     I     BLACKLISTED REPORT BIT POS.
!               NCMRBLOC     I     BLACKLISTED REPORT BITS OCC.

!        REPORT'S EVENTS (PART 1):

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMREV1

!        REPORT'S BLACKLIST EVENTS:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMREV1

INTEGER(KIND=JPIM) :: NCDTBP
INTEGER(KIND=JPIM) :: NCDTOC ! BITMAP bit9:codetype@OBSCHAR_t
INTEGER(KIND=JPIM) :: NINSTBP
INTEGER(KIND=JPIM) :: NINSTOC ! BITMAP bit10:instype@OBSCHAR_t
INTEGER(KIND=JPIM) :: NRETTBP
INTEGER(KIND=JPIM) :: NRETTOC ! BITMAP bit6:retrtype@OBSCHAR_t
INTEGER(KIND=JPIM) :: NARETBP
INTEGER(KIND=JPIM) :: NARETOC ! BITMAP bit6:geoarea@OBSCHAR_t
INTEGER(KIND=JPIM) :: NYYOFF
INTEGER(KIND=JPIM) :: NMMOFF
INTEGER(KIND=JPIM) :: NDDOFF
INTEGER(KIND=JPIM) :: NHHOFF
INTEGER(KIND=JPIM) :: NMIOFF
INTEGER(KIND=JPIM) :: NSSOFF
INTEGER(KIND=JPIM) :: NLTFLBP
INTEGER(KIND=JPIM) :: NLTFLOC ! BITMAP bit6:lat@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NLNFLBP
INTEGER(KIND=JPIM) :: NLNFLOC ! BITMAP bit6:lon@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NDTFLBP
INTEGER(KIND=JPIM) :: NDTFLOC ! BITMAP bit6:date@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NTMFLBP
INTEGER(KIND=JPIM) :: NTMFLOC ! BITMAP bit6:time@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NALFLBP
INTEGER(KIND=JPIM) :: NALFLOC ! BITMAP bit6:stalt@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NHMSBP
INTEGER(KIND=JPIM) :: NHMSOC ! BITMAP merge_bit1:humon@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NQCSBP
INTEGER(KIND=JPIM) :: NQCSOC ! BITMAP merge_bit1:QCsub@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NORFBP
INTEGER(KIND=JPIM) :: NORFOC ! BITMAP merge_bit1:override@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NFLGBP
INTEGER(KIND=JPIM) :: NFLGOC ! BITMAP merge_bit2:flag@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NHFSBP
INTEGER(KIND=JPIM) :: NHFSOC ! BITMAP merge_bit1:HQC_flag@REPORT_RDBFLAG_t
INTEGER(KIND=JPIM) :: NCMRACBP
INTEGER(KIND=JPIM) :: NCMRACOC ! BITMAP bit1:active@STATUS_t
INTEGER(KIND=JPIM) :: NCMRPABP
INTEGER(KIND=JPIM) :: NCMRPAOC ! BITMAP bit1:passive@STATUS_t
INTEGER(KIND=JPIM) :: NCMRREBP
INTEGER(KIND=JPIM) :: NCMRREOC ! BITMAP bit1:rejected@STATUS_t
INTEGER(KIND=JPIM) :: NCMRBLBP
INTEGER(KIND=JPIM) :: NCMRBLOC ! BITMAP bit1:blacklisted@STATUS_t

!-----------------------------------------------------------------------

!     OPTIONAL (OBS./CODE TYPE DEPENDENT) PART:

!     SYNOP/AIREP/DRIBU/SATOB/TEMP/PILOT/SATEM/PAOB/SCATT./R.RAD.

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     SELF-DEF. NCMOHL       I     OPTIONAL PART HEADER LENGTHS (TOTAL)
!         "     NCMOHIL      I     OPTIONAL PART HEADER LENGTHS (INT.)
!         "     NCMINS       I     INST. SPEC. WORD POSITIONS
!         "     NCMREV2      I     REPORT EVENTS (PART 2) WORD POS.
!         "     NCMMOPR      I     BEGINNIG OF MODEL PROFILE SECT.

!     SYNOP:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMHOANS     I     HEIGHT OF ANEMOMETER
!               NCMHOBAS     I     HEIGHT OF BAROMETRE

!     SATOB:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSBCMM     I     CLOUD MOTION COMP. METHOD
!               NCMSBIUP     I     INS. DATA USED IN PROC.
!               NCMSBDPT     I     DATA PROC. TECHNIQUE USED
!               NCM_TB_AT_SATOB I  COLDEST CLUSTER TEMPERATURE
!               NCMSBQI1     I     QUALITY INDICATOR 1
!               NCMSBQI2     I     QUALITY INDICATOR 2
!               NCMSBQI3     I     QUALITY INDICATOR 3
!               NCMSBCHAN_FREQ I   Satellite Channel Centre Frequency
!               NCMSBZENITH  I     Satellite Zenith Angle

!    DRIBU

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMHOAND     I     HEIGHT OF ANEMOMETER
!               NCMHOBAD     I     HEIGHT OF BAROMETRE

!    TEMP

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMHOANT     I     HEIGHT OF ANEMOMETER
!               NCMHOBAT     I     HEIGHT OF BAROMETRE

!    PILOT

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMHOANP     I     HEIGHT OF ANEMOMETER
!               NCMHOBAP     I     HEIGHT OF BAROMETRE

!     SATEM (TOVS RADIANCES)

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSOE       I     SOLAR ELEVATION
!               NCMCHU       I     SAT. CHANNEL(S) USED IN COMPUTATION
!               NCMDPT       I     SAT. DATA PROCESSING TECHNIQUE USED
!               NCMSLC       I     SATELLITE LOCATION COUNTER
!               NCMVS1       I     VERTICAL SIGNIFICANCE 1
!               NCMOZO       I     OZON
!               NCMVS2       I     VERTICAL SIGNIFICANCE 2
!               NCMPP1       I     PRESSURE 1
!               NCMCLC       I     CLOUD COVER
!               NCMVS3       I     VERTICAL SIGNIFICANCE 3
!               NCMLSQ       I     LAND/SEA QUALIFIER
!               NCMHLS       I     HEIGHT OF LAND SURFACE
!               NCMSKT       I     SKIN TEMPERATURE
!               NCMPP2       I     PRESSURE 2
!               NCMVS4       I     VERTICAL SIGNIFICANCE 4
!               NCM1DIT      I     1D VAR ITERATION NUMBER
!               NCM1DER      I     1D VAR ERROR(S)
!               NCM1DCU      I     1D VAR SATELLITE CHANNEL(S) USED  
!               NCM1DCU1     I     1D VAR SATELLITE CHANNEL(S) USED  
!               NCM1DPF      I     PRESAT SUMMARY FLAGS
!               NCM11DIN     I     1D VAR NO. OF ITER. FOR CONVERGENCE
!               NCM11DFI     I     1D VAR FAILURE INDICATOR

!             1D VAR SINGLE LEVEL VARIABLES
!               NCM11DS      I     BEGINNING OF 1D VAR SIN. LEVEL SEC.
!               NCM1DBP      I     BACKGROUND SURFACE PRESSURE
!               NCM1DAP      I     1D VAR ADJUSTED SURFACE PRESSURE
!               NCM1DBST     I     BACKGROUND SKIN TEMPERATURE
!               NCM1DAST     I     1D VAR ADJUSTED SKIN TEMPERATURE
!               NCM1DB2T     I     BACKGROUND 2M TEMPERATURE
!               NCM1DA2T     I     1D VAR ADJUSTED 2M TEMPERATURE
!               NCM1DB2Q     I     BACKGROUND 2M SPECIFIC HUMIDITY
!               NCM1DA2Q     I     1D VAR ADJUSTED 2M SPECIFIC HUMIDITY
!               NCM1DBME     I     BACKGROUND MICROWAVE SUR. EMISS.
!               NCM1DAME     I     1D VAR ADJUSTED MICROWAVE SUR. EMISS.
!               NCM1DBTO     I     BACKGROUND TOTAL COLUMN OZONE
!               NCM1DATO     I     1D VAR ADJUSTED TOTAL COLUMN OZONE
!               NCM1DBCP     I     BACKGROUND CLOUD TOP PRESSURE
!               NCM1DACP     I     1D VAR ADJUSTED CLOUD TOP PRESSURE
!               NCM1DBCA     I     BACKGROUND CLOUD AMOUNT
!               NCM1DACA     I     1D VAR ADJUSTED CLOUD AMOUNT

!             REPEATED FOR 1D VAR LEVELS
!               NCM11DL      I     BEGINNING OF 1D VAR RET VAR. SECT.
!               NCM1DVP      I     1D VAR PRESSURE
!               NCM1DVT1     I     1D VAR TEMPERATURE
!               NCM1DVT2     I     1D VAR TEMPERATURE
!               NCM1DVQ1     I     1D VAR SPECIFIC HUMIDITY
!               NCM1DVQ2     I     1D VAR SPECIFIC HUMIDITY

!             BIAS CORRECTION SECTION
!               NCM1BCP1     I     PREDICTOR 1
!               NCM1BCP2     I     PREDICTOR 2
!               NCM1BCP3     I     PREDICTOR 3
!               NCM1BCP4     I     PREDICTOR 4
!               NCM1BCP5     I     PREDICTOR 5
!               NCM1BCP6     I     PREDICTOR 6
!               NCM1BCP7     I     PREDICTOR 7
!               NCM1BCP8     I     PREDICTOR 8
!               NCM1BCP9     I     PREDICTOR 9
!               NCM1BCP10    I     PREDICTOR 10
!               NCM1BCP11    I     PREDICTOR 11
!               NCM1BCP12    I     PREDICTOR 12
!               NCM1BCP13    I     PREDICTOR 13
!               NCM1BCP14    I     PREDICTOR 14
!               NCM1BCP15    I     PREDICTOR 15
!               NCM1BCP16    I     PREDICTOR 16

!     SATEM (LEVEL-1C/GEOS RADIANCES)

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSSIA      I     SATELLITE SENSOR INDICATOR
!               NCMSCLA      I     SCAN LINE NUMBER                   
!               NCMFOVA      I     FIELD OF VIEW NUMBER               
!               NCMSTHA      I     HEIGHT OF STATION         
!               NCMSAZA      I     SATELLITE ZENITH ANGLE 
!               NCMSABA      I     BEARING OR AZIMUTH
!               NCMSOZA      I     SOLAR ZENITH ANGLE     
!               NCMSOBA      I     SOLAR AZIMUTH
!               NCMCLCA      I     CLOUD COVER
!               NCMVS3A      I     VERTICAL SIGNIFICANCE 3
!               NCMLSQA      I     LAND/SEA QUALIFIER
!               NCMHLSA      I     HEIGHT OF LAND SURFACE
!               NCMSKTA      I     SKIN TEMPERATURE
!               NCMPP2A      I     PRESSURE 2
!               NCMVS4A      I     VERTICAL SIGNIFICANCE 4
!               NCM1DITA     I     1D VAR ITERATION NUMBER
!               NCM1DERA     I     1D VAR ERROR(S)
!               NCM1DCUA     I     1D VAR SATELLITE CHANNEL(S) USED  
!               NCM1DCU1A    I     1D VAR SATELLITE CHANNEL(S) USED  
!               NCM1DPFA     I     PRESAT SUMMARY FLAGS
!               NCM11DINA    I     1D VAR NO. OF ITER. FOR CONVERGENCE
!               NCM11DFIA    I     1D VAR FAILURE INDICATOR

!             1D VAR SINGLE LEVEL VARIABLES
!               NCM11DSA     I     BEGINNING OF 1D VAR SIN. LEVEL SEC.
!               NCM1DBPA     I     BACKGROUND SURFACE PRESSURE
!               NCM1DAPA     I     1D VAR ADJUSTED SURFACE PRESSURE
!               NCM1DBSTA    I     BACKGROUND SKIN TEMPERATURE
!               NCM1DASTA    I     1D VAR ADJUSTED SKIN TEMPERATURE
!               NCM1DB2TA    I     BACKGROUND 2M TEMPERATURE
!               NCM1DA2TA    I     1D VAR ADJUSTED 2M TEMPERATURE
!               NCM1DB2QA    I     BACKGROUND 2M SPECIFIC HUMIDITY
!               NCM1DA2QA    I     1D VAR ADJUSTED 2M SPECIFIC HUMIDITY
!               NCM1DBMEA    I     BACKGROUND MICROWAVE SUR. EMISS.
!               NCM1DAMEA    I     1D VAR ADJUSTED MICROWAVE SUR. EMISS.
!               NCM1DBTOA    I     BACKGROUND TOTAL COLUMN OZONE
!               NCM1DATOA    I     1D VAR ADJUSTED TOTAL COLUMN OZONE
!               NCM1DBCPA    I     BACKGROUND CLOUD TOP PRESSURE
!               NCM1DACPA    I     1D VAR ADJUSTED CLOUD TOP PRESSURE
!               NCM1DBCAA    I     BACKGROUND CLOUD AMOUNT
!               NCM1DACAA    I     1D VAR ADJUSTED CLOUD AMOUNT

!             REPEATED FOR 1D VAR LEVELS
!               NCM11DLA     I     BEGINNING OF 1D VAR RET VAR. SECT.
!               NCM1DVPA     I     1D VAR PRESSURE
!               NCM1DVT1A    I     1D VAR TEMPERATURE
!               NCM1DVT2A    I     1D VAR TEMPERATURE
!               NCM1DVQ1A    I     1D VAR SPECIFIC HUMIDITY
!               NCM1DVQ2A    I     1D VAR SPECIFIC HUMIDITY
!               NCM1DVO1A    I     1D VAR OZONE
!               NCM1DVO2A    I     1D VAR OZONE

!             BIAS CORRECTION SECTION
!               NCM1BCP1A    I     PREDICTOR 1
!               NCM1BCP2A    I     PREDICTOR 2
!               NCM1BCP3A    I     PREDICTOR 3
!               NCM1BCP4A    I     PREDICTOR 4
!               NCM1BCP5A    I     PREDICTOR 5
!               NCM1BCP6A    I     PREDICTOR 6
!               NCM1BCP7A    I     PREDICTOR 7
!               NCM1BCP8A    I     PREDICTOR 8
!               NCM1BCP9A    I     PREDICTOR 9
!               NCM1BCP10A   I     PREDICTOR 10
!               NCM1BCP11A   I     PREDICTOR 11
!               NCM1BCP12A   I     PREDICTOR 12
!               NCM1BCP13A   I     PREDICTOR 13
!               NCM1BCP14A   I     PREDICTOR 14
!               NCM1BCP15A   I     PREDICTOR 15
!               NCM1BCP16A   I     PREDICTOR 16
!               NCM1BCPTS1   I     SKIN TEMPERATURE, per NRESUPD

!     SATEM (SSMI/TRMM)

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMORNO      I     ORBIT NO.
!               NCMSLNO      I     SCAN LINE NO.
!               NCMPNAS      I     POSITION NUMBER ALONG SCAN
!               NCMTOS       I     TYPE OF SURFACE
!               NCMVSG       I     VERTICAL SIGNIFICANCE
!               NCMZAN       I     ZENITH ANGLE
!               NCME1LA      I     FIRST EXTRA POINT LAT
!               NCME1LO      I     FIRST EXTRA POINT LON
!               NCME1TS      I     FIRST EXTRA POINT TYPE OF SURFACE
!               NCME1VS      I     FIRST EXTRA POINT VERTICAL SIG.
!               NCME1BT1     I     FIRST EXTRA POINT TEMPERATURE 1
!               NCME1BT2     I     FIRST EXTRA POINT TEMPERATURE 2
!               NCME2LA      I     SECOND EXTRA POINT LAT
!               NCME2LO      I     SECOND EXTRA POINT LON
!               NCME2TS      I     SECOND EXTRA POINT TYPE OF SURFACE
!               NCME2VS      I     SECOND EXTRA POINT VERTICAL SIG.
!               NCME2BT1     I     SECOND EXTRA POINT TEMPERATURE 1
!               NCME2BT2     I     SECOND EXTRA POINT TEMPERATURE 2
!               NCME3LA      I     THIRD EXTRA POINT LAT
!               NCME3LO      I     THIRD EXTRA POINT LON
!               NCME3TS      I     THIRD EXTRA POINT TYPE OF SURFACE
!               NCME3VS      I     THIRD EXTRA POINT VERTICAL SIG.
!               NCME3BT1     I     THIRD EXTRA POINT TEMPERATURE 1
!               NCME3BT2     I     THIRD EXTRA POINT TEMPERATURE 2
!               NCME1TQ      I     FIRST EXTRA TIME QUOLIFIER
!               NCME1DA      I     FIRST EXTRA DATE
!               NCME1TI      I     FIRST EXTRA TIME
!               NCME2TQ      I     SECOND EXTRA TIME QUOLIFIER
!               NCME2DA      I     SECOND EXTRA DATE
!               NCME2TI      I     SECOND EXTRA TIME
!               NCME3TQ      I     THIRD EXTRA TIME QUOLIFIER
!               NCME3DA      I     THIRD EXTRA DATE
!               NCME3TI      I     THIRD EXTRA TIME

!             1D VAR SSMI/TRMM RELATED VARIABLES
!               NCMSSMST     I     SSMI/TRMM SURFACE TYPE   
!               NCM1DSTQ     I     SURFACE TYPE QC
!               NCM1DNIT     I     NO. OF ITERATIONS FOR CONVERGENCE
!               NCM1DFIN     I     FAILURE INDICATOR
!               NCMIES       I     1D VAR ESTIMATE OF SCATTERING
!               NCMSSISI     I     SSMI/TRMM INDEPENDENT SCATTERING INDEX
!               NCM1DERR     I     1D VAR ESTIMATE OF RAIN RATE (MM/H)
!               NCMSSERR     I     SSMI/TRMM INDEPENDENT ESTIMATE OF RAIN
!                                  RATE
!               NCM1DREP     I     1D VAR RETRIEVED ERROR FOR TPW
!               NCM1DREW     I     1D VAR RETRIEVED ERROR FOR WIND SPEED
!               NCM1DREC     I     1D VAR RETRIEVED ERROR FOR CLOUD
!                                  LIQUIED WATER PATH
!               NCMIEREP     I     INDEPENDENT ESTIMATE OF ERROR FOR TPW
!               NCMIEREW     I     INDEPENDENT ESTIMATE OF ERROR FOR
!                                  WIND SPEED
!               NCMIEREC     I     INDEPENDENT ESTIMATE OF ERROR FOR
!                                  CLOUD LIQUIED WATER PATH

!             1D VAR SINGLE LEVEL VARIABLES
!               NCM11DSS     I     BEGINNING OF 1D VAR SIN. LEVEL SEC.
!               NCM1BPS      I     BACKGROUND SURFACE PRESSURE
!               NCM1BSTS     I     BACKGROUND SKIN TEMPERATURE
!               NCM1B2TS     I     BACKGROUND 2M TEMPERATURE
!               NCM1B2QS     I     BACKGROUND 2M SPECIFIC HUMIDITY  
!               NCM1BPWS     I     BACKGROUND PWC
!               NCM1BCLS     I     BACKGROUND CLOUD LIQUIED WATER
!               NCM1BSUS     I     BACKGROUND 10M U COMPONENT
!               NCM1BSVS     I     BACKGROUND 10M V COMPONENT
!               NCM1A2QS     I     1D VAR ADJUSTED 2M SPECIFIC HUMIDITY  
!               NCM1APWS     I     1D VAR ADJUSTED PWC
!               NCM1ACLS     I     1D VAR ADJUSTED CLOUD LIQUIED WATER
!               NCM1ASWS     I     1D VAR ADJUSTED 10M WIND SPEED
!               NCMSIPWS     I     IND. EST. OF PWC
!               NCMSICLS     I     IND. EST. OF CLOUD LIQUIED WATER
!               NCMSISWS     I     IND. EST. OF 10M WIND SPEED

!             REPEATED FOR 1D VAR LEVELS
!               NCM11DLS     I     BEGINNING OF 1D VAR RET VAR. SECT.
!               NCM1DVPS     I     PRESSURE
!               NCM1DVBT     I     BACKGROUND TEMPERATURE
!               NCM1DVAT     I     1D VAR ADJUSTED TEMPERATURE
!               NCM1DVBQ     I     BACKGROUND SPECIFIC HUMIDITIE
!               NCM1DVAQ     I     1D VAR ADJUSTED SPECIFIC HUMIDITIE
!               NCM1DVBC     I     BACKGROUND CLOUD LIQUIED WATER
!               NCM1DVAC     I     1D VAR ADJUSTED CLOUD LIQUIED WATER

!     SATEM (RETRIEVED OZONE LAYERS)

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMRO3IT      I     INSTRUMENT TYPE FOR RETRIEVED ATMOSPHERIC GASES
!               NCMRO3PT      I     PRODUCT TYPE FOR RETRIEVED ATMOSPHERIC GASES
!               NCMRO3LA1     I     LATITUDE  FIELD OF VIEW CORNER 1
!               NCMRO3LO1     I     LONGITUDE FIELD OF VIEW CORNER 1
!               NCMRO3LA2     I     LATITUDE  FIELD OF VIEW CORNER 2
!               NCMRO3LO2     I     LONGITUDE FIELD OF VIEW CORNER 2
!               NCMRO3LA3     I     LATITUDE  FIELD OF VIEW CORNER 3
!               NCMRO3LO3     I     LONGITUDE FIELD OF VIEW CORNER 3
!               NCMRO3LA4     I     LATITUDE  FIELD OF VIEW CORNER 4
!               NCMRO3LO4     I     LONGITUDE FIELD OF VIEW CORNER 4
!               NCMRO3SOE     I     SOLAR ELEVATION
!               NCMRO3FOV     I     FIELD OF VIEW NUMBER
!               NCMRO3CLC     I     CLOUD COVER
!               NCMRO3CP      I     PRESSURE AT TOP OF CLOUD
!               NCMRO3QR      I     QUALITY OF RETRIEVAL
!               NCMRO3NL      I     NUMBER OF RETRIEVED LAYERS

!     SCATT.:

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!               NCMSCCNO     I     CELL NO.
!               NCMSCPFL     I     PRODUCT FLAG

INTEGER(KIND=JPIM) :: NCMOHL(JPMXOCT,JPMXOTP)  ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMOHIL(JPMXOCT,JPMXOTP) ! LENGTH ignore:this@please
!ODB TYPEDEF INSTRM_t AS pk9int
INTEGER(KIND=JPIM) :: NCMINS             &! ROBHDR INSTRM_t:instspec@hdr     
 & (JPMXOCT,JPMXOTP)  
INTEGER(KIND=JPIM) :: NCMREV2            &! ROBHDR REPORT_EVENT2_t:event2@hdr       
 & (JPMXOCT,JPMXOTP)  
INTEGER(KIND=JPIM) :: NCMMOPR(JPMXOCT,JPMXOTP) ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMHOANS          ! ROBHDR pk9real:anemoht@hdr
INTEGER(KIND=JPIM) :: NCMHOAND          ! ROBHDR pk9real:anemoht@hdr
INTEGER(KIND=JPIM) :: NCMHOANT          ! ROBHDR pk9real:anemoht@hdr
INTEGER(KIND=JPIM) :: NCMHOANP          ! ROBHDR pk9real:anemoht@hdr
INTEGER(KIND=JPIM) :: NCMHOBAS          ! ROBHDR pk9real:baroht@hdr
INTEGER(KIND=JPIM) :: NCMHOBAD          ! ROBHDR pk9real:baroht@hdr
INTEGER(KIND=JPIM) :: NCMHOBAT          ! ROBHDR pk9real:baroht@hdr
INTEGER(KIND=JPIM) :: NCMHOBAP          ! ROBHDR pk9real:baroht@hdr
INTEGER(KIND=JPIM) :: NCMSBCMM          ! SATHDR pk9int:comp_method@satob
INTEGER(KIND=JPIM) :: NCMSBIUP          ! SATHDR pk9int:instdata@satob
INTEGER(KIND=JPIM) :: NCMSBDPT          ! SATHDR pk9int:dataproc@satob
INTEGER(KIND=JPIM) :: NCM_TB_AT_SATOB   ! SATHDR pk9int:tb@satob
INTEGER(KIND=JPIM) :: NCMSBQI1          ! SATHDR pk9int:QI_1@satob
INTEGER(KIND=JPIM) :: NCMSBQI2          ! SATHDR pk9int:QI_2@satob
INTEGER(KIND=JPIM) :: NCMSBQI3          ! SATHDR pk9int:QI_3@satob
INTEGER(KIND=JPIM) :: NCMSBCHAN_FREQ    ! SATHDR pk9int:chan_freq@satob
INTEGER(KIND=JPIM) :: NCMSBZENITH       ! SATHDR pk9int:zenith@satob
INTEGER(KIND=JPIM) :: NCMSOE            ! SATHDR pk9real:solar_elevation@rtovs
INTEGER(KIND=JPIM) :: NCMCHU            ! SATHDR pk9int:numchan@rtovs
INTEGER(KIND=JPIM) :: NCMDPT            ! SATHDR pk9int:dataproc@rtovs
INTEGER(KIND=JPIM) :: NCMSLC            ! SATHDR pk9int:loc_counter@rtovs
INTEGER(KIND=JPIM) :: NCMVS1            ! SATHDR pk9int:vertsign_1@rtovs
INTEGER(KIND=JPIM) :: NCMOZO            ! SATHDR pk9real:ozone@rtovs
INTEGER(KIND=JPIM) :: NCMVS2            ! SATHDR pk9int:vertsign_2@rtovs
INTEGER(KIND=JPIM) :: NCMPP1            ! SATHDR pk9real:press_1@rtovs
INTEGER(KIND=JPIM) :: NCMCLC            ! SATHDR pk9int:cloud_cover@rtovs
INTEGER(KIND=JPIM) :: NCMVS3            ! SATHDR pk9int:vertsign_3@rtovs
INTEGER(KIND=JPIM) :: NCMLSQ            ! SATHDR pk9int:landsea@rtovs
INTEGER(KIND=JPIM) :: NCMHLS            ! SATHDR pk9real:landsurf_height@rtovs
INTEGER(KIND=JPIM) :: NCMSKT            ! SATHDR pk9real:skintemp@rtovs
INTEGER(KIND=JPIM) :: NCMPP2            ! SATHDR pk9real:press_2@rtovs
INTEGER(KIND=JPIM) :: NCMVS4            ! SATHDR pk9int:vertsign_4@rtovs
INTEGER(KIND=JPIM) :: NCM1DIT           ! SATHDR pk9int:iterno_1dvar@rtovs
INTEGER(KIND=JPIM) :: NCM1DER           ! SATHDR pk9int:error_1dvar@rtovs
INTEGER(KIND=JPIM) :: NCM1DCU           ! SATHDR pk9int:channel_1dvar_0@rtovs
INTEGER(KIND=JPIM) :: NCM1DCU1          ! SATHDR pk9int:channel_1dvar_1@rtovs
INTEGER(KIND=JPIM) :: NCM1DPF           ! SATHDR pk9int:presat_flags@rtovs
INTEGER(KIND=JPIM) :: NCM11DIN          ! SATHDR pk9int:iterno_conv_1dvar@rtovs
INTEGER(KIND=JPIM) :: NCM11DFI          ! SATHDR pk9int:failure_1dvar@rtovs
INTEGER(KIND=JPIM) :: NCM11DS           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCM1DBP           ! SATSLEV pk9real:surfpress_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DAP           ! SATSLEV pk9real:surfpress_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBST          ! SATSLEV pk9real:skintemp_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DAST          ! SATSLEV pk9real:skintemp_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DB2T          ! SATSLEV pk9real:t2m_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DA2T          ! SATSLEV pk9real:t2m_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DB2Q          ! SATSLEV pk9real:q2m_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DA2Q          ! SATSLEV pk9real:q2m_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBME          ! SATSLEV pk9real:mwave_surfemiss_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DAME          ! SATSLEV pk9real:mwave_surfemiss_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBTO          ! SATSLEV pk9real:totcol_ozone_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DATO          ! SATSLEV pk9real:totcol_ozone_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBCP          ! SATSLEV pk9real:cloud_top_press_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DACP          ! SATSLEV pk9real:cloud_top_press_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBCA          ! SATSLEV pk9real:cloud_amount_1@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DACA          ! SATSLEV pk9real:cloud_amount_2@rtovs_slev
INTEGER(KIND=JPIM) :: NCM11DL           ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCM1DVP           ! SATMLEV pk9real:press@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVT1          ! SATMLEV pk9real:temp_1@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVT2          ! SATMLEV pk9real:temp_2@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVQ1          ! SATMLEV pk9real:q_1@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVQ2          ! SATMLEV pk9real:q_2@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVO1          ! SATMLEV pk9real:ozone_1@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVO2          ! SATMLEV pk9real:ozone_2@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1BCP1          ! SATPRED pk9real:predictor_1@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP2          ! SATPRED pk9real:predictor_2@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP3          ! SATPRED pk9real:predictor_3@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP4          ! SATPRED pk9real:predictor_4@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP5          ! SATPRED pk9real:predictor_5@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP6          ! SATPRED pk9real:predictor_6@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP7          ! SATPRED pk9real:predictor_7@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP8          ! SATPRED pk9real:predictor_8@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP9          ! SATPRED pk9real:predictor_9@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP10         ! SATPRED pk9real:predictor_10@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP11         ! SATPRED pk9real:predictor_11@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP12         ! SATPRED pk9real:predictor_12@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP13         ! SATPRED pk9real:predictor_13@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP14         ! SATPRED pk9real:predictor_14@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP15         ! SATPRED pk9real:predictor_15@rtovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP16         ! SATPRED pk9real:predictor_16@rtovs_pred
INTEGER(KIND=JPIM) :: NCMSSIA           ! ROBHDR pk9int:sensor@hdr
!----SATHDR pk9int:sensor@atovs
!----ODB pk9int:sensor@hdr AS SATHDR IN CCMA
INTEGER(KIND=JPIM) :: NCMSCLA           ! SATHDR pk9int:scanline@atovs
INTEGER(KIND=JPIM) :: NCMFOVA           ! SATHDR pk9int:fov@atovs
INTEGER(KIND=JPIM) :: NCMSTHA           ! SATHDR pk9real:station_height@atovs
INTEGER(KIND=JPIM) :: NCMSAZA           ! SATHDR pk9real:zenith@atovs
INTEGER(KIND=JPIM) :: NCMSABA           ! SATHDR pk9real:bearing_azimuth@atovs
INTEGER(KIND=JPIM) :: NCMSOZA           ! SATHDR pk9real:solar_zenith@atovs
INTEGER(KIND=JPIM) :: NCMSOBA           ! SATHDR pk9real:solar_azimuth@atovs
INTEGER(KIND=JPIM) :: NCMCLCA           ! SATHDR pk9real:cloud_coveratovs
INTEGER(KIND=JPIM) :: NCMVS3A           ! SATHDR pk9int:vertsign_3@atovs
INTEGER(KIND=JPIM) :: NCMLSQA           ! SATHDR pk9int:landsea@atovs
INTEGER(KIND=JPIM) :: NCMHLSA           ! SATHDR pk9real:landsurf_height@atovs
INTEGER(KIND=JPIM) :: NCMSKTA           ! SATHDR pk9real:skintemp@atovs
INTEGER(KIND=JPIM) :: NCMPP2A           ! SATHDR pk9real:press_2@atovs
INTEGER(KIND=JPIM) :: NCMVS4A           ! SATHDR pk9int:vertsign_4@atovs
INTEGER(KIND=JPIM) :: NCM1DITA          ! SATHDR pk9int:iterno_1dvar@atovs
INTEGER(KIND=JPIM) :: NCM1DERA          ! SATHDR pk9int:error_1dvar@atovs
INTEGER(KIND=JPIM) :: NCM1DCUA          ! SATHDR pk9int:channel_1dvar_0@atovs
INTEGER(KIND=JPIM) :: NCM1DCU1A         ! SATHDR pk9int:channel_1dvar_1@atovs
INTEGER(KIND=JPIM) :: NCM1DPFA          ! SATHDR pk9int:presat_flags@atovs
INTEGER(KIND=JPIM) :: NCM11DINA         ! SATHDR pk9int:iterno_conv_1dvar@atovs
INTEGER(KIND=JPIM) :: NCM11DFIA         ! SATHDR pk9int:failure_1dvar@atovs
INTEGER(KIND=JPIM) :: NCM11DSA          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCM1DBPA          ! OMIT_SATSLEV pk9real:bg_surfpress@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DAPA          ! OMIT_SATSLEV pk9real:adj_surfpress@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBSTA         ! OMIT_SATSLEV pk9real:bg_skintemp@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DASTA         ! OMIT_SATSLEV pk9real:adj_skintemp@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DB2TA         ! OMIT_SATSLEV pk9real:bg_t2m@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DA2TA         ! OMIT_SATSLEV pk9real:adj_t2m@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DB2QA         ! OMIT_SATSLEV pk9real:bg_q2m@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DA2QA         ! OMIT_SATSLEV pk9real:adj_q2m@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBMEA         ! OMIT_SATSLEV pk9real:bg_mwave_surfemiss@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DAMEA         ! OMIT_SATSLEV pk9real:adj_mwave_surfemiss@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBTOA         ! OMIT_SATSLEV pk9real:bg_totcol_ozone@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DATOA         ! OMIT_SATSLEV pk9real:adj_totcol_ozone@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBCPA         ! OMIT_SATSLEV pk9real:bg_cloud_top_press@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DACPA         ! OMIT_SATSLEV pk9real:adj_cloud_top_press@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DBCAA         ! OMIT_SATSLEV pk9real:bg_cloud_amount@rtovs_slev
INTEGER(KIND=JPIM) :: NCM1DACAA         ! OMIT_SATSLEV pk9real:adj_cloud_amount@rtovs_slev
INTEGER(KIND=JPIM) :: NCM11DLA          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCM1DVPA          ! OMIT_SATMLEV pk9real:press@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVT1A         ! OMIT_SATMLEV pk9real:temp_1@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVT2A         ! OMIT_SATMLEV pk9real:temp_2@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVQ1A         ! OMIT_SATMLEV pk9real:q_1@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVQ2A         ! OMIT_SATMLEV pk9real:q_2@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVO1A         ! OMIT_SATMLEV pk9real:ozone_1@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1DVO2A         ! OMIT_SATMLEV pk9real:ozone_2@rtovs_mlev
INTEGER(KIND=JPIM) :: NCM1BCP1A         ! SATPRED pk9real:predictor_1@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP2A         ! SATPRED pk9real:predictor_2@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP3A         ! SATPRED pk9real:predictor_3@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP4A         ! SATPRED pk9real:predictor_4@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP5A         ! SATPRED pk9real:predictor_5@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP6A         ! SATPRED pk9real:predictor_6@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP7A         ! SATPRED pk9real:predictor_7@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP8A         ! SATPRED pk9real:predictor_8@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP9A         ! SATPRED pk9real:predictor_9@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP10A        ! SATPRED pk9real:predictor_10@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP11A        ! SATPRED pk9real:predictor_11@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP12A        ! SATPRED pk9real:predictor_12@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP13A        ! SATPRED pk9real:predictor_13@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP14A        ! SATPRED pk9real:predictor_14@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP15A        ! SATPRED pk9real:predictor_15@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCP16A        ! SATPRED pk9real:predictor_16@atovs_pred
INTEGER(KIND=JPIM) :: NCM1BCPTS1(JPMXUP)! SATPRED pk9real:skintemp[$NMXUPD]@atovs_pred
INTEGER(KIND=JPIM) :: NCMSCCNO          ! SATHDR pk9int:cellno@scatt
INTEGER(KIND=JPIM) :: NCMSCPFL          ! SATHDR pk9int:prodflag@scatt

INTEGER(KIND=JPIM) :: NCMORNO           ! SATHDR pk9int:orbit@ssmi
INTEGER(KIND=JPIM) :: NCMSLNO           ! SATHDR pk9int:scanline@ssmi
INTEGER(KIND=JPIM) :: NCMPNAS           ! SATHDR pk9int:scanpos@ssmi
INTEGER(KIND=JPIM) :: NCMTOS            ! SATHDR pk9int:typesurf@ssmi
INTEGER(KIND=JPIM) :: NCMVSG            ! SATHDR pk9int:vertsign@ssmi
INTEGER(KIND=JPIM) :: NCMZAN            ! SATHDR pk9real:zenith@ssmi
INTEGER(KIND=JPIM) :: NCME1LA           ! SATHDR pk9real:lat_1@ssmi
INTEGER(KIND=JPIM) :: NCME1LO           ! SATHDR pk9real:lon_1@ssmi
INTEGER(KIND=JPIM) :: NCME1TQ           ! SATHDR pk9real:timequal_1@ssmi
INTEGER(KIND=JPIM) :: NCME1DA           ! SATHDR YYYYMMDD:date_1@ssmi
INTEGER(KIND=JPIM) :: NCME1TI           ! SATHDR HHMMSS:time_1@ssmi
INTEGER(KIND=JPIM) :: NCME1TS           ! SATHDR pk9int:surftype_1@ssmi
INTEGER(KIND=JPIM) :: NCME1VS           ! SATHDR pk9int:vertsign_1@ssmi
INTEGER(KIND=JPIM) :: NCME1BT1          ! SATHDR pk9real:temp1_1@ssmi
INTEGER(KIND=JPIM) :: NCME1BT2          ! SATHDR pk9real:temp2_1@ssmi
INTEGER(KIND=JPIM) :: NCME2LA           ! SATHDR pk9real:lat_2@ssmi
INTEGER(KIND=JPIM) :: NCME2LO           ! SATHDR pk9real:lon_2@ssmi
INTEGER(KIND=JPIM) :: NCME2TQ           ! SATHDR pk9real:timequal_2@ssmi
INTEGER(KIND=JPIM) :: NCME2DA           ! SATHDR YYYYMMDD:date_2@ssmi
INTEGER(KIND=JPIM) :: NCME2TI           ! SATHDR HHMMSS:time_2@ssmi
INTEGER(KIND=JPIM) :: NCME2TS           ! SATHDR pk9int:surftype_2@ssmi
INTEGER(KIND=JPIM) :: NCME2VS           ! SATHDR pk9int:vertsign_2@ssmi
INTEGER(KIND=JPIM) :: NCME2BT1          ! SATHDR pk9real:temp1_2@ssmi
INTEGER(KIND=JPIM) :: NCME2BT2          ! SATHDR pk9real:temp2_2@ssmi
INTEGER(KIND=JPIM) :: NCME3LA           ! SATHDR pk9real:lat_3@ssmi
INTEGER(KIND=JPIM) :: NCME3LO           ! SATHDR pk9real:lon_3@ssmi
INTEGER(KIND=JPIM) :: NCME3TQ           ! SATHDR pk9real:timequal_3@ssmi
INTEGER(KIND=JPIM) :: NCME3DA           ! SATHDR YYYYMMDD:date_3@ssmi
INTEGER(KIND=JPIM) :: NCME3TI           ! SATHDR HHMMSS:time_3@ssmi
INTEGER(KIND=JPIM) :: NCME3TS           ! SATHDR pk9int:surftype_3@ssmi
INTEGER(KIND=JPIM) :: NCME3VS           ! SATHDR pk9int:vertsign_3@ssmi
INTEGER(KIND=JPIM) :: NCME3BT1          ! SATHDR pk9real:temp1_3@ssmi
INTEGER(KIND=JPIM) :: NCME3BT2          ! SATHDR pk9real:temp2_3@ssmi
INTEGER(KIND=JPIM) :: NCM11DSS          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCM11DLS          ! LENGTH ignore:this@please
INTEGER(KIND=JPIM) :: NCMSSMST          ! SATHDR pk9int:surftype@ssmi
INTEGER(KIND=JPIM) :: NCM1DSTQ          ! SATHDR pk9int:QC_surftype@ssmi
INTEGER(KIND=JPIM) :: NCM1DNIT          ! SATHDR pk9int:iterno_conv_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCM1DFIN          ! SATHDR pk9int:failure_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCMIES            ! SATHDR pk9real:scattering_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCMSSISI          ! SATHDR pk9real:scattering_indep@ssmi
INTEGER(KIND=JPIM) :: NCM1DERR          ! SATHDR pk9real:rainrate_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCMSSERR          ! SATHDR pk9real:rainrate_indep@ssmi
INTEGER(KIND=JPIM) :: NCM1DREP          ! SATHDR pk9real:tpw_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCM1DREW          ! SATHDR pk9real:wspeed_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCM1DREC          ! SATHDR pk9real:clwpath_1dvar@ssmi
INTEGER(KIND=JPIM) :: NCMIEREP          ! SATHDR pk9real:tpw_indep@ssmi
INTEGER(KIND=JPIM) :: NCMIEREW          ! SATHDR pk9real:wspeed_indep@ssmi
INTEGER(KIND=JPIM) :: NCMIEREC          ! SATHDR pk9real:clwpath_indep@ssmi
INTEGER(KIND=JPIM) :: NCM1BPS           ! SATSLEV pk9real:surfpress_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1BSTS          ! SATSLEV pk9real:skintemp_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1B2TS          ! SATSLEV pk9real:t2m_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1B2QS          ! SATSLEV pk9real:q2m_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1BPWS          ! SATSLEV pk9real:pwc_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1BCLS          ! SATSLEV pk9real:clw_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1BSUS          ! SATSLEV pk9real:u10m_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1BSVS          ! SATSLEV pk9real:v10m_1@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1A2QS          ! SATSLEV pk9real:q2m_2@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1APWS          ! SATSLEV pk9real:pwc_2@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1ACLS          ! SATSLEV pk9real:clw_2@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1ASWS          ! SATSLEV pk9real:wspeed10m_2@ssmi_slev
INTEGER(KIND=JPIM) :: NCMSIPWS          ! SATSLEV pk9real:indep_pwc@ssmi_slev
INTEGER(KIND=JPIM) :: NCMSICLS          ! SATSLEV pk9real:indep_clw@ssmi_slev
INTEGER(KIND=JPIM) :: NCMSISWS          ! SATSLEV pk9real:indep_wspeed10m@ssmi_slev
INTEGER(KIND=JPIM) :: NCM1DVPS          ! SATMLEV pk9real:press@ssmi_mlev
INTEGER(KIND=JPIM) :: NCM1DVBT          ! SATMLEV pk9real:temp_1@ssmi_mlev
INTEGER(KIND=JPIM) :: NCM1DVAT          ! SATMLEV pk9real:temp_2@ssmi_mlev
INTEGER(KIND=JPIM) :: NCM1DVBQ          ! SATMLEV pk9real:q_1@ssmi_mlev
INTEGER(KIND=JPIM) :: NCM1DVAQ          ! SATMLEV pk9real:q_2@ssmi_mlev
INTEGER(KIND=JPIM) :: NCM1DVBC          ! SATMLEV pk9real:clw_1@ssmi_mlev
INTEGER(KIND=JPIM) :: NCM1DVAC          ! SATMLEV pk9real:clw_2@ssmi_mlev
INTEGER(KIND=JPIM) :: NCMRO3IT          ! SATHDR pk9int:instrument_type@reo3
INTEGER(KIND=JPIM) :: NCMRO3PT          ! SATHDR pk5int:product_type@reo3
INTEGER(KIND=JPIM) :: NCMRO3LA1         ! SATHDR pk9real:lat_fovcorner_1@reo3
INTEGER(KIND=JPIM) :: NCMRO3LO1         ! SATHDR pk9real:lon_fovcorner_1@reo3
INTEGER(KIND=JPIM) :: NCMRO3LA2         ! SATHDR pk9real:lat_fovcorner_2@reo3
INTEGER(KIND=JPIM) :: NCMRO3LO2         ! SATHDR pk9real:lon_fovcorner_2@reo3
INTEGER(KIND=JPIM) :: NCMRO3LA3         ! SATHDR pk9real:lat_fovcorner_3@reo3
INTEGER(KIND=JPIM) :: NCMRO3LO3         ! SATHDR pk9real:lon_fovcorner_3@reo3
INTEGER(KIND=JPIM) :: NCMRO3LA4         ! SATHDR pk9real:lat_fovcorner_4@reo3
INTEGER(KIND=JPIM) :: NCMRO3LO4         ! SATHDR pk9real:lon_fovcorner_4@reo3
INTEGER(KIND=JPIM) :: NCMRO3SOE         ! SATHDR pk9real:solar_elevation@reo3
INTEGER(KIND=JPIM) :: NCMRO3FOV         ! SATHDR pk9int:fov@reo3
INTEGER(KIND=JPIM) :: NCMRO3CLC         ! SATHDR pk9real:cloud_cover@reo3
INTEGER(KIND=JPIM) :: NCMRO3CP          ! SATHDR pk9real:cloud_top_press@reo3
INTEGER(KIND=JPIM) :: NCMRO3QR          ! SATHDR pk5int:quality_retrieval@reo3
INTEGER(KIND=JPIM) :: NCMRO3NL          ! SATHDR pk9int:number_layers@reo3

!-----------------------------------------------------------------------

!     OPTIONAL (OBS./CODE TYPE DEPENDENT) PART PACKED INFO FORMAT:

!     SYNOP/AIREP/SATOB/DRIBU/TEMP/PILOT/SATEM/PAOB/SCATT./R.RADIANCE

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMINS    NCMITBP      I     INST. TYPE BIT POS.
!               NCMITOC      I     INST. TYPE BITS OCC.

!     AIREP

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMINS    NPHFLBP      I     FLIGHT PHASE BIT POS.
!               NPHFLOC      I     FLIGHT PHASE BITS OCC.

!     SATOB

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMINS    NSBI1BP      I     I1 PARM. BIT POS.
!               NSBI1OC      I     I1 PARM. BITS OCC.
!               NSBI2BP      I     I2 PARM. BIT POS.
!               NSBI2OC      I     I2 PARM. BITS OCC.

!     DRIBU

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMINS    NDK2BP       I     K2 PARM. BIT POS.
!               NDK2OC       I     K2 PARM. BITS OCC.
!               NDK3BP       I     K3 PARM. BIT POS.
!               NDK3OC       I     K3 PARM. BITS OCC.
!               NDK4BP       I     K4 PARM. BIT POS.
!               NDK4OC       I     K4 PARM. BITS OCC.

!     PILOT

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMINS    NPIA4BP      I     A4 PARM. BIT POS.
!               NPIA4OC      I     A4 PARM. BITS OCC.

!     SATEM

!     WORD NO./ VARIABLE / TYPE /                MEANING               /
!     --------  --------   ----                  -------
!     NCMINS    NSMI3BP      I     I3 PARM. BIT POS.
!               NSMI3OC      I     I3 PARM. BITS OCC.
!               NSMI2BP      I     I2I2 PARM. BIT POS.
!               NSMI2OC      I     I2I2 PARM. BITS OCC.
!               NSMI4BP      I     I4 PARM. BIT POS.
!               NSMI4OC      I     I4 PARM. BITS OCC.
!               NSMI1BP      I     I1 PARM. BIT POS.
!               NSMI1OC      I     I1 PARM. BITS OCC.
!               NTOAABP      I     A PARM. BIT POS.
!               NTOAAOC      I     A PARM. BITS OCC.
!               NTOBBBP      I     B PARM. BIT POS.
!               NTOBBOC      I     B PARM. BITS OCC.
!               NTOCCBP      I     C PARM. BIT POS.
!               NTOCCOC      I     C PARM. BITS OCC.
!               NTOVVBP      I     V PARM. BIT POS.
!               NTOVVOC      I     V PARM. BITS OCC.
!               NTOWWBP      I     W PARM. BIT POS.
!               NTOWWOC      I     W PARM. BITS OCC.
!               NTOXXBP      I     X PARM. BIT POS.
!               NTOXXOC      I     X PARM. BITS OCC.
!               NTOYYBP      I     Y PARM. BIT POS.
!               NTOYYOC      I     Y PARM. BITS OCC.
!               NTOZZBP      I     Z PARM. BIT POS.
!               NTOZZOC      I     Z PARM. BITS OCC.

INTEGER(KIND=JPIM) :: NCMITBP(JPMXOCT,JPMXOTP)
INTEGER(KIND=JPIM) :: NCMITOC(JPMXOCT,JPMXOTP) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NPHFLBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NPHFLOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NSBI1BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NSBI1OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NSBI2BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NSBI2OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NDK2BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NDK2OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NDK3BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NDK3OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NDK4BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NDK4OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NPIA4BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NPIA4OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NSMI3BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NSMI3OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NSMI2BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NSMI2OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NSMI4BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NSMI4OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NSMI1BP(JPMXOCT)
INTEGER(KIND=JPIM) :: NSMI1OC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOAABP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOAAOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOBBBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOBBOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOCCBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOCCOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOVVBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOVVOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOWWBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOWWOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOXXBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOXXOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOYYBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOYYOC(JPMXOCT) ! BITMAP bit:@
INTEGER(KIND=JPIM) :: NTOZZBP(JPMXOCT)
INTEGER(KIND=JPIM) :: NTOZZOC(JPMXOCT) ! BITMAP bit:@

!ODB pk9int:procid@index AS ROBHDR
!ODB pk9int:target@index AS ROBHDR
!ODB pk9int:kset@index AS ROBHDR
!ODB pk9int:tslot@index AS ROBHDR
!ODB pk9int:abnob@index AS ROBHDR

!ODB LINK index TO hdr
!ODB LINK hdr TO body
!ODB LINK hdr TO update
!ODB LINK hdr TO errstat

!ODB LINK hdr TO sat
!----ODB string:satid@sat AS SATHDR

!ODB LINK sat TO atovs
!ODB LINK sat TO rtovs
!ODB LINK sat TO ssmi
!ODB LINK sat TO scatt
!ODB LINK sat TO satob

!----ODB LINK sat TO atovs_pred IN CCMA

!---ODB LINK atovs TO atovs_body
!ODB LINK atovs TO atovs_pred

!ODB LINK rtovs TO rtovs_slev
!ODB LINK rtovs TO rtovs_mlev
!ODB LINK rtovs TO rtovs_pred
!ODB LINK rtovs TO rtovs_body

!ODB LINK ssmi TO ssmi_slev
!ODB LINK ssmi TO ssmi_mlev
!ODB LINK ssmi TO ssmi_body

!ODB LINK scatt TO scatt_body

!-----------------------------------------------------------------------

END MODULE YOMCMHDR
