MODULE YOMCOCTP
!ODB SET $SYNOP = 1
!ODB SET $AIREP = 2
!ODB SET $SATOB = 3
!ODB SET $DRIBU = 4
!ODB SET $TEMP = 5
!ODB SET $PILOT = 6
!ODB SET $SATEM = 7
!ODB SET $ATOVS = 210
!ODB SET $RTOVS = 211
!ODB SET $TOVS = 212
!ODB SET $SSMI = 215
!ODB SET $SMOS = 400
!ODB SET $PAOB = 8
!ODB SET $SCATT = 9
!ODB SET $LIMB = 10
!ODB SET $RADAR = 13
!ODB SET $BG = 1
!ODB SET $ADJ = 2
USE PARKIND1  ,ONLY : JPIM
USE PARDIMO

IMPLICIT NONE

SAVE

!*     YOMCOCTP - CMA OBS./CODE TYPES

!        D. VASILJEVIC   ECMWF     15/09/94

!     MODIFICATONS
!     ------------
!       23/12/2004   Niels Bormann  RARAD becomes LIMB; new code types for LIMB
!       24/04/2009   Joaquin Munoz Sabater  New code types for SMOS L-band radiances

!     CMA OBS./CODE TYPES

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NOOBST        I      NO. OF OBS. TYPES
!     NOCODT        I      NO. OF OBS. CODE TYPES
!     NSYNOP        I      SYNOP
!     NSYNOPSQ      I      SYNOP
!     NSYNCT        I      NO. OF SYNOP CODE TYPES
!     NSRSCD        I      SYNOP LAND REPORT
!     NSRSCDSQ      I      SYNOP LAND REPORT SQ. NO.
!     NATSCD        I      SYNOP LAND AUTOMATIC REPORT
!     NATSCDSQ      I      SYNOP LAND AUTOMATIC REPORT SQ. NO.
!     NSHSCD        I      SYNOP-SHIP REPORT
!     NSHSCDSQ      I      SYNOP-SHIP REPORT SQ. NO.
!     NABSCD        I      SYNOP-SHIP ABBREVIATED REPORT
!     NABSCDSQ      I      SYNOP-SHIP ABBREVIATED REPORT SQ. NO.
!     NSHRED        I      SYNOP SHRED REPORT
!     NSHREDSQ      I      SYNOP SHRED REPORT SQ. NO.
!     NATSHS        I      SYNOP AUTOMATIC SHIP REPORT
!     NATSHSSQ      I      SYNOP AUTOMATIC SHIP REPORT SQ. NO.
!     NSYRCD        I      SYNOR REPORT
!     NSYRCDSQ      I      SYNOR REPORT SQ. NO.
!     NRADOH        I      RADOME
!     NRADOHSQ      I      RADOME SQ. NO.
!     NMETAR        I      METAR
!     NMETARSQ      I      METAR SQ. NO.
!     NPGPS         I      Ground-based GPS
!     NPGPSSQ       I      Ground-based GPS SQ. NO.
!     NAIREP        I      AIREP
!     NAIREPSQ      I      AIREP
!     NAIRCT        I      NO. OF AIREP CODE TYPES
!     NAIRCD        I      AIRCRAFT REPORT
!     NAIRCDSQ      I      AIRCRAFT REPORT SQ. NO.
!     NCODAR        I      CODAR REPORT
!     NCODARSQ      I      CODAR REPORT SQ. NO.
!     NCOLBA        I      COLBA REPORT
!     NCOLBASQ      I      COLBA REPORT SQ. NO.
!     NAMDAR        I      AMDAR AIRCRAFT REPORT
!     NAMDARSQ      I      AMDAR AIRCRAFT REPORT SQ. NO.
!     NACARS        I      ACARS AIRCRAFT REPORT
!     NACARSSQR     I      ACARS AIRCRAFT REPORT SQ. NO.
!     NSIMAI        I      SIMULATED AIREP REPORT
!     NSIMAISQ      I      SIMULATED AIREP REPORT SQ. NO.
!     NSATOB        I      SATOB
!     NSATOBSQ      I      SATOB
!     NSABCT        I      NO. OF SATOB CODE TYPES
!     NSTBCD        I      SATOB REPORT
!     NSTBCDSQ      I      SATOB REPORT SQ. NO.
!     NSTB86        I      HIGH RES. VIS WIND SATOB REPORT
!     NSTB86SQ      I      HIGH RES. VIS WIND SATOB REPORT SQ. NO.
!     NSTB87        I      AMV REPORT
!     NSTB87SQ      I      AMV REPORT SQ. NO.
!     NSST          I      SST REPORT
!     NSSTSQ        I      SST REPORT SQ. NO.
!     NDRIBU        I      DRIBU
!     NDRIBUSQ      I      DRIBU
!     NDRICT        I      NO. OF DRIBU CODE TYPES
!     NDRBCD        I      DRIBU REPORT
!     NDRBCDSQ      I      DRIBU REPORT SQ. NO.
!     NBATHY        I      BATHY REPORT
!     NBATHYSQ      I      BATHY REPORT SQ. NO.
!     NTESAC        I      TESAC REPORT
!     NTESACSQ      I      TESAC REPORT SQ. NO.
!     NDERS1        I      ERS1 AS DRIBU
!     NDERS1SQ      I      ERS1 AS DRIBU SQ. NO.
!     NTEMP         I      TEMP
!     NTEMPSQ       I      TEMP
!     NTEMCT        I      NO. OF TEMP CODE TYPES
!     NLDTCD        I      LAND TEMP REPORT
!     NLDTCDSQ      I      LAND TEMP REPORT SQ. NO.
!     NSHTCD        I      TEMP SHIP REPORT
!     NSHTCDSQ      I      TEMP SHIP REPORT SQ. NO.
!     NTDROP        I      TEMP-DROP REPORT
!     NTDROPSQ      I      TEMP-DROP REPORT SQ. NO.
!     NROCOB        I      ROCOB REPORT
!     NROCOBSQ      I      ROCOB REPORT SQ. NO.
!     NROCSH        I      ROCOB SHIP REPORT
!     NROCSHSQ      I      ROCOB SHIP REPORT SQ. NO.
!     NMBTMP        I      MOBILE TEMP REPORT
!     NMBTMPSQ      I      MOBILE TEMP REPORT SQ. NO.
!     NSIMTE        I      SIMULATED TEMP
!     NSIMTESQ      I      SIMULATED TEMP SQ. NO.
!     NPILOT        I      PILOT
!     NPILOTSQ      I      PILOT
!     NPILCT        I      NO. OF PILOT CODE TYPES
!     NLDPCD        I      LAND PILOT REPORT
!     NLDPCDSQ      I      LAND PILOT REPORT SQ. NO.
!     NSHPCD        I      SHIP PILOT REPORT
!     NSHPCDSQ      I      SHIP PILOT REPORT SQ. NO.
!     NWPPCD        I      AMERICAN WIND PROFILERS
!     NWPPCDSQ      I      AMERICAN WIND PROFILERS SQ. NO.
!     NEWPPCD       I      EUROPEAN WIND PROFILERS
!     NEWPPCDSQ     I      EUROPEAN WIND PROFILERS SQ. NO.
!     NJWPPCD       I      JAPANESE WIND PROFILERS
!     NJWPPCDSQ     I      JAPANESE WIND PROFILERS SQ. NO.
!     NMBPLT        I      MOBILE WIND PILOT
!     NMBPLTSQ      I      MOBILE WIND PILOT SQ. NO.
!     NDWLPCD       I      DOPPLER WIND LIDAR
!     NDWLPCDSQ     I      DOPPLER WIND LIDAR SQ. NO.
!     NSATEM        I      SATEM
!     NSATEMSQ      I      SATEM
!     NSAMCT        I      NO. OF SATEM CODE TYPES
!     NSTMCD        I      SATEM REPORT
!     NSTMCDSQ      I      SATEM REPORT SQ. NO.
!     NSTOVS        I      HIGH RESOLUTION SATELLITE REPORT
!     NSTOVSSQ      I      HIGH RESOLUTION SATELLITE REPORT SQ. NO.
!     NSTRO3        I      RETRIEVED OZONE LAYERS REPORT
!     NSTRO3SQ      I      RETRIEVED OZONE LAYERS REPORT SQ. NO.
!     NSTDWL        I      HIGH RES. SIMUL. DWL SAT. REPORT
!     NSTDWLSQ      I      HIGH RES. SIMUL. DWL SAT. REPORT SQ. NO.
!     NSTTOV        I      HIGH RES. SIMUL. TOVS SAT. REPORT
!     NSTTOVSQ      I      HIGH RES. SIMUL. TOVS SAT. REPORT SQ. NO.
!     NGTSTB        I      GTS BUFR SATEM REPORT (250 KM TEMP. AND
!                          HUM. PROFILES)
!     NGTSTBSQ      I      GTS BUFR SATEM REPORT (250 KM TEMP. AND
!                          HUM. PROFILES) SQ. NO.
!     NGTST1        I      GTS BUFR SATEM CLEAR RAD. REPORT
!     NGTST1SQ      I      GTS BUFR SATEM CLEAR RAD. REPORT SQ. NO.
!     NGTST2        I      GTS BUFR SATEM RETR PROFILES AND
!                          CLEAR RADIANCES REPORT
!     NGTST2SQ      I      GTS BUFR SATEM RETR PROFILES AND
!                          CLEAR RADIANCES REPORT SQ. NO.
!     NGTHRB        I      HIGH RES. 80 KM BUFR NON GTS SATEM RETR.
!                          PROFILES
!     NGTHRBSQ      I      HIGH RES. 80 KM BUFR NON GTS SATEM RETR.
!                          PROFILES SQ. NO.
!     NGTHR1        I      HIGH RES 80 KM BUFR NON GTS SATEM CLEAR
!                          RADIANCE REPORT
!     NGTHR1SQ      I      HIGH RES 80 KM BUFR NON GTS SATEM CLEAR
!                          RADIANCE REPORT SQ. NO.
!     NGTHR2        I      HIGH RES 80 KM BUFR NON GTS SATEM PROFILES
!                          AND CLEAR RADIANCE REPORT
!     NGTHR2SQ      I      HIGH RES 80 KM BUFR NON GTS SATEM PROFILES
!                          AND CLEAR RADIANCE REPORT SQ. NO.
!     NSSMI         I      SSMI REPORT
!     NSSMISQ       I      SSMI REPORT SQ. NO.
!     NPAOB         I      PAOB
!     NPAOBSQ       I      PAOB
!     NPAOCT        I      NO. OF PAOB CODE TYPES
!     NPABCD        I      PAOB REPORT
!     NPABCDSQ      I      PAOB REPORT SQ. NO.
!     NTYPHO        I      CYCLON REPORT
!     NTYPHOSQ      I      CYCLON REPORT SQ. NO.
!     NSCATT        I      SCATTEROMETER
!     NSCATTSQ      I      SCATTEROMETER
!     NSCACT        I      NO. OF SCATTEROMETER CODE TYPES
!     NSCAT1        I      SCATTEROMETER CODE TYPE 1
!     NSCAT1SQ      I      SCATTEROMETER CODE TYPE 1 SQ. NO.
!     NSCAT2        I      SCATTEROMETER CODE TYPE 2
!     NSCAT2SQ      I      SCATTEROMETER CODE TYPE 2 SQ. NO.
!     NSCAT3        I      SCATTEROMETER CODE TYPE 3
!     NSCAT3SQ      I      SCATTEROMETER CODE TYPE 3 SQ. NO.
!     NSCAT4        I      SCATTEROMETER CODE TYPE 4
!     NSCAT4SQ      I      SCATTEROMETER CODE TYPE 4 SQ. NO.
!     NSCAT5        I      SCATTEROMETER CODE TYPE 5
!     NSCAT5SQ      I      SCATTEROMETER CODE TYPE 5 SQ. NO.
!     NSCAT6        I      SCATTEROMETER CODE TYPE 6
!     NSCAT6SQ      I      SCATTEROMETER CODE TYPE 6 SQ. NO.
!     NLIMB         I      LIMB OBSERVATIONS
!     NLIMBSQ       I      LIMB OBSERVATIONS
!     NLIMBCT       I      NO. LIMB OBSERVATION CODE TYPES
!     NGPSRO        I      GPS RADIO OCCULTATION CODE TYPE
!     NGPSROSQ      I      GPS RADIO OCCULTATION SQ. NO.
!     NLRAD         I      LIMB RADIANCES CODE TYPE
!     NLRADSQ       I      LIMB RADIANCES SQ. NO.
!     NRADAR        I      RADAR
!     NRADARSQ      I      RADAR SQ. NO
!     NRADA1        I      RADAR CODE TYPE 1
!     NRADA1SQ      I      RADAR CODE TYPE 1 SQ. NO.
!     NSMOS         I      SMOS CODE TYPE
!     NSMOSSQ       I      SMOS CODE TYPE SQ. NO.
!     NTCWC         I      TCWC (MERIS) CODE TYPE
!     NTCWCSQ       I      TCWC(MERIS) CODE TYPe SQ. NO.


INTEGER(KIND=JPIM) :: NOOBST
INTEGER(KIND=JPIM) :: NOCODT
INTEGER(KIND=JPIM) :: NSYNOP
INTEGER(KIND=JPIM) :: NSYNOPSQ
INTEGER(KIND=JPIM) :: NSYNCT
INTEGER(KIND=JPIM) :: NSRSCD
INTEGER(KIND=JPIM) :: NSRSCDSQ
INTEGER(KIND=JPIM) :: NATSCD
INTEGER(KIND=JPIM) :: NATSCDSQ
INTEGER(KIND=JPIM) :: NSHSCD
INTEGER(KIND=JPIM) :: NSHSCDSQ
INTEGER(KIND=JPIM) :: NABSCD
INTEGER(KIND=JPIM) :: NABSCDSQ
INTEGER(KIND=JPIM) :: NSHRED
INTEGER(KIND=JPIM) :: NSHREDSQ
INTEGER(KIND=JPIM) :: NATSHS
INTEGER(KIND=JPIM) :: NATSHSSQ
INTEGER(KIND=JPIM) :: NSYRCD
INTEGER(KIND=JPIM) :: NSYRCDSQ
INTEGER(KIND=JPIM) :: NRADOH
INTEGER(KIND=JPIM) :: NRADOHSQ
INTEGER(KIND=JPIM) :: NMETAR
INTEGER(KIND=JPIM) :: NMETARSQ
INTEGER(KIND=JPIM) :: NPGPS
INTEGER(KIND=JPIM) :: NPGPSSQ
INTEGER(KIND=JPIM) :: NAIREP
INTEGER(KIND=JPIM) :: NAIREPSQ
INTEGER(KIND=JPIM) :: NAIRCT
INTEGER(KIND=JPIM) :: NAIRCD
INTEGER(KIND=JPIM) :: NAIRCDSQ
INTEGER(KIND=JPIM) :: NCODAR
INTEGER(KIND=JPIM) :: NCODARSQ
INTEGER(KIND=JPIM) :: NCOLBA
INTEGER(KIND=JPIM) :: NCOLBASQ
INTEGER(KIND=JPIM) :: NAMDAR
INTEGER(KIND=JPIM) :: NAMDARSQ
INTEGER(KIND=JPIM) :: NACARS
INTEGER(KIND=JPIM) :: NACARSSQ
INTEGER(KIND=JPIM) :: NSIMAI
INTEGER(KIND=JPIM) :: NSIMAISQ
INTEGER(KIND=JPIM) :: NSATOB
INTEGER(KIND=JPIM) :: NSATOBSQ
INTEGER(KIND=JPIM) :: NSABCT
INTEGER(KIND=JPIM) :: NSTBCD
INTEGER(KIND=JPIM) :: NSTBCDSQ
INTEGER(KIND=JPIM) :: NSTB86
INTEGER(KIND=JPIM) :: NSTB86SQ
INTEGER(KIND=JPIM) :: NSTB87
INTEGER(KIND=JPIM) :: NSTB87SQ
INTEGER(KIND=JPIM) :: NSST
INTEGER(KIND=JPIM) :: NSSTSQ
INTEGER(KIND=JPIM) :: NDRIBU
INTEGER(KIND=JPIM) :: NDRIBUSQ
INTEGER(KIND=JPIM) :: NDRICT
INTEGER(KIND=JPIM) :: NDRBCD
INTEGER(KIND=JPIM) :: NDRBCDSQ
INTEGER(KIND=JPIM) :: NBATHY
INTEGER(KIND=JPIM) :: NBATHYSQ
INTEGER(KIND=JPIM) :: NTESAC
INTEGER(KIND=JPIM) :: NTESACSQ
INTEGER(KIND=JPIM) :: NDERS1
INTEGER(KIND=JPIM) :: NDERS1SQ
INTEGER(KIND=JPIM) :: NTEMP
INTEGER(KIND=JPIM) :: NTEMPSQ
INTEGER(KIND=JPIM) :: NTEMCT
INTEGER(KIND=JPIM) :: NLDTCD
INTEGER(KIND=JPIM) :: NLDTCDSQ
INTEGER(KIND=JPIM) :: NSHTCD
INTEGER(KIND=JPIM) :: NSHTCDSQ
INTEGER(KIND=JPIM) :: NTDROP
INTEGER(KIND=JPIM) :: NTDROPSQ
INTEGER(KIND=JPIM) :: NROCOB
INTEGER(KIND=JPIM) :: NROCOBSQ
INTEGER(KIND=JPIM) :: NROCSH
INTEGER(KIND=JPIM) :: NROCSHSQ
INTEGER(KIND=JPIM) :: NMBTMP
INTEGER(KIND=JPIM) :: NMBTMPSQ
INTEGER(KIND=JPIM) :: NSIMTE
INTEGER(KIND=JPIM) :: NSIMTESQ
INTEGER(KIND=JPIM) :: NPILOT
INTEGER(KIND=JPIM) :: NPILOTSQ
INTEGER(KIND=JPIM) :: NPILCT
INTEGER(KIND=JPIM) :: NLDPCD
INTEGER(KIND=JPIM) :: NLDPCDSQ
INTEGER(KIND=JPIM) :: NSHPCD
INTEGER(KIND=JPIM) :: NSHPCDSQ
INTEGER(KIND=JPIM) :: NWPPCD
INTEGER(KIND=JPIM) :: NWPPCDSQ
INTEGER(KIND=JPIM) :: NEWPPCD
INTEGER(KIND=JPIM) :: NEWPPCDSQ
INTEGER(KIND=JPIM) :: NJWPPCD
INTEGER(KIND=JPIM) :: NJWPPCDSQ
INTEGER(KIND=JPIM) :: NMBPLT
INTEGER(KIND=JPIM) :: NMBPLTSQ
INTEGER(KIND=JPIM) :: NDWLPCD
INTEGER(KIND=JPIM) :: NDWLPCDSQ
INTEGER(KIND=JPIM) :: NSATEM
INTEGER(KIND=JPIM) :: NSATEMSQ
INTEGER(KIND=JPIM) :: NSAMCT
INTEGER(KIND=JPIM) :: NSTMCD
INTEGER(KIND=JPIM) :: NSTMCDSQ
INTEGER(KIND=JPIM) :: NSTOVS
INTEGER(KIND=JPIM) :: NSTOVSSQ
INTEGER(KIND=JPIM) :: NSTRO3
INTEGER(KIND=JPIM) :: NSTRO3SQ
INTEGER(KIND=JPIM) :: NSTDWL
INTEGER(KIND=JPIM) :: NSTDWLSQ
INTEGER(KIND=JPIM) :: NSTTOV
INTEGER(KIND=JPIM) :: NSTTOVSQ
INTEGER(KIND=JPIM) :: NGTSTB
INTEGER(KIND=JPIM) :: NGTSTBSQ
INTEGER(KIND=JPIM) :: NGTST1
INTEGER(KIND=JPIM) :: NGTST1SQ
INTEGER(KIND=JPIM) :: NGTST2
INTEGER(KIND=JPIM) :: NGTST2SQ
INTEGER(KIND=JPIM) :: NGTHRB
INTEGER(KIND=JPIM) :: NGTHRBSQ
INTEGER(KIND=JPIM) :: NGTHR1
INTEGER(KIND=JPIM) :: NGTHR1SQ
INTEGER(KIND=JPIM) :: NGTHR2
INTEGER(KIND=JPIM) :: NGTHR2SQ
INTEGER(KIND=JPIM) :: NSSMI
INTEGER(KIND=JPIM) :: NSSMISQ
INTEGER(KIND=JPIM) :: NPAOB
INTEGER(KIND=JPIM) :: NPAOBSQ
INTEGER(KIND=JPIM) :: NPAOCT
INTEGER(KIND=JPIM) :: NPABCD
INTEGER(KIND=JPIM) :: NPABCDSQ
INTEGER(KIND=JPIM) :: NTYPHO
INTEGER(KIND=JPIM) :: NTYPHOSQ
INTEGER(KIND=JPIM) :: NSCATT
INTEGER(KIND=JPIM) :: NSCATTSQ
INTEGER(KIND=JPIM) :: NSCACT
INTEGER(KIND=JPIM) :: NSCAT1
INTEGER(KIND=JPIM) :: NSCAT1SQ
INTEGER(KIND=JPIM) :: NSCAT2
INTEGER(KIND=JPIM) :: NSCAT2SQ
INTEGER(KIND=JPIM) :: NSCAT3
INTEGER(KIND=JPIM) :: NSCAT3SQ
INTEGER(KIND=JPIM) :: NSCAT4
INTEGER(KIND=JPIM) :: NSCAT4SQ
INTEGER(KIND=JPIM) :: NSCAT5
INTEGER(KIND=JPIM) :: NSCAT5SQ
INTEGER(KIND=JPIM) :: NSCAT6
INTEGER(KIND=JPIM) :: NSCAT6SQ
INTEGER(KIND=JPIM) :: NLIMB
INTEGER(KIND=JPIM) :: NLIMBSQ
INTEGER(KIND=JPIM) :: NLIMBCT
INTEGER(KIND=JPIM) :: NGPSRO
INTEGER(KIND=JPIM) :: NGPSROSQ
INTEGER(KIND=JPIM) :: NLRAD
INTEGER(KIND=JPIM) :: NLRADSQ
INTEGER(KIND=JPIM) :: NRADAR
INTEGER(KIND=JPIM) :: NRADARSQ
INTEGER(KIND=JPIM) :: NRDRCT
INTEGER(KIND=JPIM) :: NRADA1
INTEGER(KIND=JPIM) :: NRADA1SQ
INTEGER(KIND=JPIM) :: NSMOS
INTEGER(KIND=JPIM) :: NSMOSSQ
INTEGER(KIND=JPIM) :: NTCWC
INTEGER(KIND=JPIM) :: NTCWCSQ


CHARACTER(LEN=32) :: CCMOOTP(JPNOTP) ! Names of observation types
INTEGER(KIND=JPIM) :: MNOCTP(JPNOTP)          ! Number of codetypes, per obs type
CHARACTER(LEN=32) :: CCMOCTP(511)    ! Names of code types
INTEGER(KIND=JPIM) :: MSQBYCTP(511)           ! Converts from codetype to sequence No.

!-----------------------------------------------------------------------
END MODULE YOMCOCTP
