MODULE YOMCOM

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!-----------------------------------------------------------------------
!**** *SUMCC* * - MODULE INCLUDING VARIABLES FOR THE COM COUPLING

!     AUTHOR.
!     -------
!        Jean-Philippe Piedelievre CNRM/GMGEC/EAC

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 99-02-24
!       03-04-08  Philippe Piedelievre CNRM/GMGEC/EAC
!-----------------------------------------------------------------------

! NVCOM     : number of parameters for the COM scheme.

!*    MASK FOR MIXING DYNAMICAL-STATISTICAL SST SCHEME

! ALATN1    : first latitude in northern hemisphere for the SST mask
!             (dynamical-statistical)
! ALATN2, ALATN3, ALATN4 (idem)
! ALATS1, ALATS2, ALATS3, ALATS4 (idem for the south)

INTEGER(KIND=JPIM) :: NVCOM

REAL(KIND=JPRB) ::    ALATN1, ALATN2, ALATN3, ALATN4
REAL(KIND=JPRB) ::    ALATS1, ALATS2, ALATS3, ALATS4
REAL(KIND=JPRB) ::    GAMAT,  GAMAD

!*    DIAGNOSTICS POUR COUPLAGE COM

! FCSTSU    STRESS SURFACE DE U
! FCSTSV    STRESS SURFACE DE V
! FCCHAS    FLUX CHALEUR SURFACE
! FCRSOS    FLUX RAYT SOLAIRE SURFACE
! FCCHLL    FLUX CHALEUR LATENTE EAU LIQUIDE
! FCCHLN    FLUX CHALEUR LATENTE NEIGE
! FCCHSS    FLUX CHALEUR SENSIBLE

REAL(KIND=JPRB),ALLOCATABLE:: FCSTSU(:)
REAL(KIND=JPRB),ALLOCATABLE:: FCSTSV(:)
REAL(KIND=JPRB),ALLOCATABLE:: FCCHAS(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: FCRSOS(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: FCCHLL(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: FCCHLN(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: FCCHSS(:,:)

! OMLDTH    PROFONDEUR DE LA COUCHE DE MELANGE OCEANIQUE
! GTTLIN    GRADIENT THERMIQUE DE LA THERMOCLINE
! SSTPRE    SST DYNAMIQUE
! SSTMSK    MASQUE DE MIXAGE DYNAMIQUE-STATISTIQUE

REAL(KIND=JPRB),ALLOCATABLE:: OMLDTH(:)
REAL(KIND=JPRB),ALLOCATABLE:: GTTLIN(:)
REAL(KIND=JPRB),ALLOCATABLE:: SSTPRE(:)
REAL(KIND=JPRB),ALLOCATABLE:: SSTMSK(:)

REAL(KIND=JPRB) ::  HMEL_MIN
REAL(KIND=JPRB),ALLOCATABLE:: TRAFLX(:)
LOGICAL :: LOMLDTH

!     -----------------------------------------------------------------
END MODULE YOMCOM
