MODULE YOMCOSP_VARIABLES

  USE PARKIND1, ONLY : JPIM, JPRB
  USE YOMDIM,   ONLY : NFLEVG, NPROMA, NGPBLKS
  USE YOMCT0,   ONLY : NSTART, NSTOP, NPROC
  USE YOMGEM   , ONLY : NGPTOT
  USE PHY_DIAG_MOD, ONLY: T_DIAG
  ! COSP
  USE MOD_COSP_CONSTANTS
  USE MOD_COSP_TYPES
  USE MOD_COSP_MODIS_SIMULATOR
  USE MOD_COSP

  IMPLICIT NONE

  SAVE

  LOGICAL :: COSP_USE_VGRID,COSP_RUN,COSP_CSAT_VGRID,COSP_DEBUG
  LOGICAL :: COSP_USE_PRECIPITATION_FLUXES,COSP_USE_REFF
  REAL(KIND=JPRB), PARAMETER :: ZMISS   = -1.0E-4_JPRB
  REAL(KIND=JPRB), PARAMETER :: ZMINVAL = 1.0E-6_JPRB
  INTEGER :: COSP_IBL,COSP_KIDIA,COSP_KFDIA,COSP_IOFF,COSP_KLONSAVE
  INTEGER :: COSP_ICEND
  INTEGER :: COSP_TSTEP
  INTEGER :: COSP_NPOINTS
  INTEGER :: COSP_NLEVELS
  INTEGER :: COSP_NCOLUMNS
  INTEGER :: COSP_NPOINTS_IT
  INTEGER :: COSP_NLVGRID
  INTEGER :: COSP_USE_MIE_TABLES,COSP_MELT_LAY
  REAL(KIND=JPRB) :: COSP_RADAR_FREQ,COSP_EMSFC_LW,COSP_K2
  INTEGER :: COSP_SURFACE_RADAR,COSP_USE_GAS_ABS,COSP_DO_RAY
  INTEGER :: COSP_NPRMTS_MAX_HYDRO
  INTEGER :: COSP_NUMAERO
  INTEGER :: COSP_NPRMTS_MAX_AERO
  INTEGER :: COSP_LIDAR_ICE_TYPE
  INTEGER :: COSP_OVERLAP
  INTEGER :: COSP_ISCCP_TOPHEIGHT
  INTEGER :: COSP_ISCCP_TOPHEIGHT_DIRECTION
  INTEGER(KIND=JPIM) :: COSP_OUTPUT_FREQUENCY

  TYPE(CLASS_PARAM) :: HP
  
  CHARACTER(LEN=100) :: COSP_CLOUDSAT_MICRO_SCHEME
  ! LOGICAL FLAGS FOR OUTPUT DATA
  LOGICAL :: COSP_LPCTISCCP,  & ! ISCCP MEAN CLOUD TOP PRESSURE
       COSP_LCLISCCP,         & ! ISCCP CLOUD AREA FRACTION
       COSP_LBOXPTOPISCCP,    & ! ISCCP CTP IN EACH COLUMN
       COSP_LBOXTAUISCCP,     & ! ISCCP OPTICAL EPTH IN EACH COLUMN
       COSP_LTAUISCCP,        & ! ISCCP MEAN OPTICAL DEPTH
       COSP_LCLTISCCP,        & ! ISCCP TOTAL CLOUD FRACTION
       COSP_LMEANTBISCCP,     & ! ISCCP MEAN ALL-SKY 10.5MICRON BRIGHTNESS TEMP
       COSP_LMEANTBCLRISCCP,  & ! ISCCP MEAN CLEAR-SKY 10.5MICRON BRIGHTNESS TEMP
       COSP_LALBISCCP,        & ! ISCCP MEAN CLOUD ALBEDO
       COSP_LCLMISR,          & ! MISR CLOUD FRACTION
       COSP_LCLTMODIS,        & ! MODIS TOTAL CLOUD FRACTION
       COSP_LCLWMODIS,        & ! MODIS LIQUID CLOUD FRACTION
       COSP_LCLIMODIS,        & ! MODIS ICE CLOUD FRACTION
       COSP_LCLHMODIS,        & ! MODIS HIGH-LEVEL CLOUD FRACTION
       COSP_LCLMMODIS,        & ! MODIS MID-LEVEL CLOUD FRACTION
       COSP_LCLLMODIS,        & ! MODIS LOW-LEVEL CLOUD FRACTION
       COSP_LTAUTMODIS,       & ! MODIS TOTAL CLOUD OPTICAL THICKNES
       COSP_LTAUWMODIS,       & ! MODIS LIQUID OPTICAL THICKNESS
       COSP_LTAUIMODIS,       & ! MODIS ICE OPTICAL THICKNESS
       COSP_LTAUTLOGMODIS,    & ! MODIS TOTAL CLOUD OPTICAL THICKNESS (LOG10 MEAN)
       COSP_LTAUWLOGMODIS,    & ! MODIS LIQUID OPTICAL THICKNESS (LOG10 MEAN)
       COSP_LTAUILOGMODIS,    & ! MODIS ICE OPTICAL THICKNESS (LOG10 MEAN)
       COSP_LREFFCLWMODIS,    & ! MODIS LIQUID CLOUD PARTICLE SIZE
       COSP_LREFFCLIMODIS,    & ! MODIS ICE PARTICLE SIZE
       COSP_LPCTMODIS,        & ! MODIS CLOUD TOP PRESSURE
       COSP_LLWPMODIS,        & ! MODIS CLOUD LIQUID WATER PATH
       COSP_LIWPMODIS,        & ! MODIS CLOUD ICE WATER PATH
       COSP_LCLMODIS,         & ! MODIS CLOUD AREA FRACTION
       COSP_LATB532,          & ! CALIPSO ATTENUATED TOTAL BACKSCATTER (532NM)
       COSP_LLIDARBETAMOL532, & ! CALIPSO MOLECULAR BACKSCATTER (532NM)
       COSP_LCFADLIDARSR532,  & ! CALIPSO SCATTERING RATIO CFAD
       COSP_LCLCALIPSO2,      & ! CALIPSO CLOUD FRACTION UNDETECTED BY CLOUDSAT
       COSP_LCLCALIPSO,       & ! CALIPSO CLOUD AREA FRACTION
       COSP_LCLHCALIPSO,      & ! CALIPSO HIGH-LEVEL CLOUD FRACTION
       COSP_LCLLCALIPSO,      & ! CALIPSO LOW-LEVEL CLOUD FRACTION
       COSP_LCLMCALIPSO,      & ! CALIPSO MID-LEVEL CLOUD FRACTION
       COSP_LCLTCALIPSO,      & ! CALIPSO TOTAL CLOUD FRACTION
       COSP_LCLTLIDARRADAR,   & ! CALIPSO-CLOUDSAT TOTAL CLOUD FRACTION
       COSP_LCLCALIPSOLIQ,    & ! CALIPSO LIQUID CLOUD AREA FRACTION
       COSP_LCLCALIPSOICE,    & ! CALIPSO ICE CLOUD AREA FRACTION
       COSP_LCLCALIPSOUN,     & ! CALIPSO UNDETECTED CLOUD AREA FRACTION
       COSP_LCLCALIPSOTMP,    & ! CALIPSO UNDETECTED CLOUD AREA FRACTION
       COSP_LCLCALIPSOTMPLIQ, & ! CALIPSO LIQUID CLOUD AREA FRACTION
       COSP_LCLCALIPSOTMPICE, & ! CALIPSO ICE CLOUD AREA FRACTION
       COSP_LCLCALIPSOTMPUN,  & ! CALIPSO UNDETECTED CLOUD AREA FRACTION
       COSP_LCLTCALIPSOLIQ,   & ! CALIPSO LIQUID TOTAL CLOUD FRACTION
       COSP_LCLTCALIPSOICE,   & ! CALIPSO ICE TOTAL CLOUD FRACTION
       COSP_LCLTCALIPSOUN,    & ! CALIPSO UNDETECTED TOTAL CLOUD FRACTION
       COSP_LCLHCALIPSOLIQ,   & ! CALIPSO HIGH-LEVEL LIQUID CLOUD FRACTION
       COSP_LCLHCALIPSOICE,   & ! CALIPSO HIGH-LEVEL ICE CLOUD FRACTION
       COSP_LCLHCALIPSOUN,    & ! CALIPSO HIGH-LEVEL UNDETECTED CLOUD FRACTION
       COSP_LCLMCALIPSOLIQ,   & ! CALIPSO MID-LEVEL LIQUID CLOUD FRACTION
       COSP_LCLMCALIPSOICE,   & ! CALIPSO MID-LEVEL ICE CLOUD FRACTION
       COSP_LCLMCALIPSOUN,    & ! CALIPSO MID-LEVEL UNDETECTED CLOUD FRACTION
       COSP_LCLLCALIPSOLIQ,   & ! CALIPSO LOW-LEVEL LIQUID CLOUD FRACTION
       COSP_LCLLCALIPSOICE,   & ! CALIPSO LOW-LEVEL ICE CLOUD FRACTION
       COSP_LCLLCALIPSOUN,    & ! CALIPSO LOW-LEVEL UNDETECTED CLOUD FRACTION
       COSP_LCFADDBZE94,      & ! CLOUDSAT RADAR REFLECTIVITY CFAD
       COSP_LDBZE94,          & ! CLOUDSAT RADAR REFLECTIVITY
       COSP_LPARASOLREFL,     & ! PARASOL REFLECTANCE
       COSP_LTBRTTOV,         & ! RTTOV MEAN BRIGHTNESS TEMPERATURE
       COSP_LRADAR_SIM,       &
       COSP_LLIDAR_SIM,       &
       COSP_LISCCP_SIM,       &
       COSP_LMISR_SIM,        &
       COSP_LMODIS_SIM,       &
       COSP_LRTTOV_SIM,       &
       COSP_LTOFFSET,         &
       COSP_LFRACOUT,         &
       COSP_LCRIMODIS,        &
       COSP_LCRLMODIS

  TYPE(COSP_CONFIG)     :: CFG      ! CONFIGURATION OPTIONS
  TYPE(COSP_GRIDBOX)    :: GBX      ! GRIDBOX INPUTS
  TYPE(COSP_SUBGRID)    :: SGX      ! SUBGRID INPUTS
  TYPE(COSP_SGHYDRO)    :: SGHYDRO  ! SUBGRID INFO FOR HYDROMETEORS
  TYPE(COSP_VGRID)      :: VGRID    ! INFORMATION ON VERTICAL GRID OF STATS
  TYPE(COSP_SGRADAR)    :: SGRADAR  ! OUTPUT FROM RADAR SIMULATOR
  TYPE(COSP_SGLIDAR)    :: SGLIDAR  ! OUTPUT FROM LIDAR SIMULATOR
  TYPE(COSP_ISCCP)      :: ISCCP    ! OUTPUT FROM ISCCP SIMULATOR
  TYPE(COSP_MISR)       :: MISR     ! OUTPUT FROM MISR SIMULATOR
  TYPE(COSP_MODIS)      :: MODIS    ! OUTPUT FROM MODIS SIMULATOR
  TYPE(COSP_RTTOV)      :: RTTOV    ! OUTPUT FROM RTTOV
  TYPE(COSP_RADARSTATS) :: STRADAR  ! SUMMARY STATISTICS FROM RADAR SIMULATOR
  TYPE(COSP_LIDARSTATS) :: STLIDAR  ! SUMMARY STATISTICS FROM LIDAR SIMULATOR

  ! Diagnostics
  TYPE(T_DIAG),POINTER :: D_MODIS_TOTAL, D_MODIS_LIQ, D_MODIS_ICE
  TYPE(T_DIAG),POINTER :: D_CALIPSO_LCC, D_CALIPSO_MCC, D_CALIPSO_HCC, D_CALIPSO_TCC
  TYPE(T_DIAG),POINTER :: D_ISCCP_TOTAL_CF, D_ISCCP_CLDALB, D_ISCCP_CTP
  TYPE(T_DIAG),POINTER :: D_COSP3D

END MODULE YOMCOSP_VARIABLES
