MODULE YOMCVER

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

! -----------------------------------------------------------------------------

! * Variables related to vertical discretisation in finite elements:
! LVERTFE : .T./.F. Finite element/conventional vertical discretisation.
! LRNHC1  : .T.: in the NH model, ensure constraint C1 by using (FD,NDLNPR=1)
!           in the linear terms (SI scheme) even if other vertical
!           discretisations (ex VFE) are activated in the non-linear model.

! LVFE_LAPL     : VFE for vertical Laplacian term (NH model)
! LVFE_LAPL_BC  : VFE for boundary cond. in vertical Laplacian term (NH model)
!               : if inner domain is purely in VFE manner
! LVFE_X_TERM   : VFE X-term (NH model)
! LVFE_Z_TERM   : VFE Z-term (w on full levels in NH model)
! LVFE_GW       : VFE for vertical velocity (NH model); in this case
!                 vertical velocity is at full levels.
! LVFE_DELNHPRE : VFE to compute [Delta pre] at full levels.
! LVFE_GWMPA    : VFE for AROME physics vertical velocity
!                 (NH model with AROME physics)
! LVFE_DERIB    : If VFE for derivatives in non-advective terms, use RDERB.
! LVFE_DBCS     : If VFE for derivatives in non-advective terms, use RDERI
!                 and correct surface boundary condition.
! LVFE_DBCT     : If VFE for derivatives in non-advective terms, use RDERI
!                 and correct top boundary condition.

! NVSCH   : type of basis if the finite element vertical discretisation is used.
!           1: linear functions.
!           3: Hermite cubic functions.
! RINTE   : matricial operator for vertical integrations in the
!           finite element vertical discretisation.
! RDERI   : matricial operator for vertical first-order derivatives in the
!           finite element vertical discretisation.
!           Does not take account of top and bottom boundary conditions.
! RDERB   : version of RDERI taking account of top and bottom boundary conds.
! RDDERI  : matricial operator for vertical second-order derivatives in the
!           finite element vertical discretisation (NH model only).

LOGICAL :: LVERTFE
LOGICAL :: LRNHC1
LOGICAL :: LVFE_LAPL
LOGICAL :: LVFE_LAPL_BC
LOGICAL :: LVFE_X_TERM
LOGICAL :: LVFE_Z_TERM
LOGICAL :: LVFE_GW
LOGICAL :: LVFE_DELNHPRE
LOGICAL :: LVFE_GWMPA
LOGICAL :: LVFE_DERIB
LOGICAL :: LVFE_DBCS
LOGICAL :: LVFE_DBCT

INTEGER(KIND=JPIM) :: NVSCH
REAL(KIND=JPRB),ALLOCATABLE :: RINTE(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RDERI(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RDERB(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RDDERI(:,:)

! -----------------------------------------------------------------------------

! * Variables related to use of spline cubic vertical interpolations
!   in the semi-Lagrangian scheme:
! LVSPLIP            : .T. if vertical spline cubic SL interpolations for O3.
! RVSPTRI,RVSPC      : are used to re-profile the field to be interpolated
!                      in routine VSPLTRANS.
! RFAA,RFBB,RFCC,RFDD: are used in the computation of the vertical weights.
! VRDETAR            : ratio (eta(lbar)-eta(lbar-1))/(eta(lbar-1)-eta(lbar-2)),
!                      is used in the interpolation routine LAITVSPCQM to
!                      ensure monotonicity and conservation properties.
LOGICAL :: LVSPLIP
LOGICAL :: LSVTSM                ! Stratospheric vertical trajectory smoothed
REAL(KIND=JPRB),ALLOCATABLE :: RVSPTRI(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RVSPC(:)
REAL(KIND=JPRB),ALLOCATABLE :: RFAA(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RFBB(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RFCC(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: RFDD(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: VRDETAR(:)

! -----------------------------------------------------------------------------

END MODULE YOMCVER
