MODULE YOMERSCA

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE PARERSCA

IMPLICIT NONE

SAVE

!*** BLOCK YOMERSCA
!     CONSTANTS NEEDED FOR SCATTEROMETER INVERSION
!**********************************************************************
!     A180,A270,A360,ARAD computational constant

!     SIGMAX            MAX VALUE FOR SIGMA0
!     XKPMAX            MAX           NOISE
!     XMISMAX           MAX           MISSING PACKET
!     XLFRMAX           MAX           ACCEPTED LAND FRACION
!     MBIT              NUMBER OF BITS FOR NEW QUALITY FLAG
!     EXPP              EXPONENT FOR DISTANCE COMPUTATION
!     EXPPINV           EXP TO THE POWER -1

!     ACMOD4(JPCMOD4) BASIC COEFFICIENTS OF BACKSCATTER MODEL CMOD4
!     VALTT(JPANT,JPCELL+1)THEORETICAL ERS INTERVAL VALUES FOR INCIDENCE ANGLE
!     BIAS(JPBIAS1:JPBIAS2) COEFFICIENTS OF INCIDENCE ONLY DEPENDENT SIGMA0 BIAS CORRECTION
!     BIASBM(JPANT,JPCELL,JPSAT) ERS COEFFICIENTS OF BEAM DEPENDENT SIGMA0 BIAS CORRECTION
!     BIASABM(JPANT,JPACELL,JPSCAT) ASCAT COEFFICIENTS OF BEAM DEPENDENT SIGMA0 BIAS CORRECTION
!     VALSPL(JPSPL) SPLINE REFERENCE VALUES FOR WIND SPEED BIAS CORRECTION
!     XSPL(JPSPL+5,JPCELL,JPSAT) SPLINE COEFFICIENTS OF WIND SPEED BIAS CORRECTION
!     TAB(JPVIT*JPPHI*JPTETA) SIGMA0 TABLE FOR CMOD4 WITH "BIAS" TERM ONLY (Cmod_Zad)
!     V0          1ST WIND SPEED
!     DV          WIND SPEED INTERVAL
!     XPHI0       1ST WIND DIRECTION
!     DPHI        WIND DIRECTION INTERVAL       
!     TETA0       1ST INCIDENCE ANGLE
!     DTETA       INCIDENCE ANGLE INTERVAL
!     NKNOTSP7    NUMBER OF KNOTS FOR SPECIFYING SPEED BIAS SPLINE CORRECTIONS + 7
!                 USED BY NAG ROUTINE E02BBF
!     RKNOTS, RSPLCOEFF, RSPLEND ARE ALL USED BY NEW ERS BIAS CORRECTION CODE
!     ERS_SPEED_BIASCOR(0:JPSPEEDMAX,JPCELL,JPSAT)        ERS BIAS CORRECTIONS
!     ASCAT_SPEED_BIASCOR(0:JPSPEEDMAX,JPACELL,JPASCAT) ASCAT BIAS CORRECTIONS
!     RSCAMDI     MISSING VALUES (REAL)
!     MSCAMDI     MISSING VALUE (INTEGER)
!     ASCAT_RESOL ASCAT PIXEL RESOLUTION /LR=25000m/HR=12500m/

REAL(KIND=JPRB) :: A180
REAL(KIND=JPRB) :: A270
REAL(KIND=JPRB) :: A360
REAL(KIND=JPRB) :: ARAD
REAL(KIND=JPRB) :: SIGMAX
REAL(KIND=JPRB) :: XKPMAX
REAL(KIND=JPRB) :: XLFRMAX
REAL(KIND=JPRB) :: XMISMAX
INTEGER(KIND=JPIM) :: MBIT
REAL(KIND=JPRB) :: mbitpad
REAL(KIND=JPRB) :: EXPP
REAL(KIND=JPRB) :: EXPPINV
REAL(KIND=JPRB) :: ACMOD4(JPCMOD4)
REAL(KIND=JPRB) :: VALTT(JPANT,JPCELL+1)
REAL(KIND=JPRB) :: BIAS(JPBIAS1:JPBIAS2)
REAL(KIND=JPRB) :: BIASBM(JPANT,JPCELL,JPSAT)
REAL(KIND=JPRB) :: BIASABM(JPANT,JPACELL,JPASCAT)
REAL(KIND=JPRB) :: VALSPL(JPSPL)
REAL(KIND=JPRB) :: XSPL(JPSPL+5,JPCELL,JPSAT)
REAL(KIND=JPRB) :: TAB(JPVIT*JPPHI*JPTETA)
REAL(KIND=JPRB) :: V0
REAL(KIND=JPRB) :: DV
REAL(KIND=JPRB) :: XPHI0
REAL(KIND=JPRB) :: DPHI
REAL(KIND=JPRB) :: TETA0
REAL(KIND=JPRB) :: DTETA
INTEGER(KIND=JPIM) :: NKNOTSP7
REAL(KIND=JPRB), ALLOCATABLE :: RKNOTS(:,:,:), RSPLCOEFF(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: RSPLEND(:,:)
REAL(KIND=JPRB) :: ERS_SPEED_BIASCOR(0:JPSPEEDMAX,JPCELL,JPSAT)
REAL(KIND=JPRB) :: ASCAT_SPEED_BIASCOR(0:JPSPEEDMAX,JPACELL,JPASCAT)
REAL(KIND=JPRB) :: RSCAMDI
INTEGER(KIND=JPIM) :: MSCAMDI
REAL(KIND=JPRB) :: ASCAT_RESOL
END MODULE YOMERSCA
