MODULE YOMFPG

USE PARKIND1, ONLY : JPIM, JPRB
USE PARFPOS,  ONLY : JPOSDOM, JPOSGL, JPOSLE

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------

!*    Defining the output possibly transformed geometry
!     When not precised variables are DM-global for distributed memory.
!     Abbreviation DM-ARR stands for "DM distribution for arrival geometry"
!     Abbreviation DM-DEP stands for "DM distribution for departure geometry"

!========== FOR ALL KINDS OF OUTPUT (SUB)DOMAINS =========

! NFPMAX      : (global) truncation order
! NFPLEV      : (global) number of levels
! FPVALH      : (global) "A" coefficients of vertical system
! FPVBH       : (global) "B" coefficients of vertical system
! RFPLA       : (local)  latitudes of the output points (DM-ARR)
! RFPLO       : (local)  longitudes of the output points (DM-ARR)
! RFPLA_DEP   : (local)  latitudes of the output points (DM-DEP)
! RFPLO_DEP   : (local)  longitudes of the output points (DM-DEP)
! RFPGM       : (local)  map factor (DM-ARR)
! RFPNORX     : (local)  meridional/along X component of compass matrix (DM-ARR)
! RFPNORY     : (local)  longitudinal/along Y component of compass matrix (DM-ARR)
! NFPNUMD_DEP : (local)  subdomain index of the output points (DM-DEP)
! NFPRGPG     : (global) total number of output points where to interpolate
! NFPRGPL     : (local)  total number of output points where to interpolate (DM-ARR)
! NFPRGPLX    : (global) maximum value of NFPRGPL among all processors (DM-ARR)
! NFPRGPL_DEP : (local)  total number of output points where to interpolate (DM-DEP)
! NFPRGPLX_DEP: (global) maximum value of NFPRGPL among all processors (DM-DEP)
! LFPOSHOR    : (global) .TRUE. for actual horizontal processing
! LFPOSVER    : (global) .TRUE. if pp on a set of model levels identical to
!               the departure one (i.e. NFPLEV=NFLEVG, FPVALH=VAH, FPVBH=VBH). 
! LFPEZO      : (global) .TRUE. for actual biperiodicisation
! LFPOSBUF    : (global) .TRUE. to set up fullpos buffer (else : model-shaped buffer)
! LFPDISTRIB  : (global) .TRUE.: additional transposition between departure
!               geometry and arrival geometry in FULL-POS.

!========== FOR GAUSSIAN GRID AS OUTPUT ONLY =============

! FPMUCEN : MU OF THE POLE OF INTEREST
! FPLOCEN : LONGITUDE OF THE POLE OF INTEREST
! NFPHTYP : 0 = regular grid
!         : 1 = number of points proportional to sqrt(1-mu**2)
!         : 2 = number of points read on namelist namfpg
! FPNLGINC: increment to get non-linear grid
! NFPRGRI : nunber of active points on a parallel
! NFPMEN  : max. zonal wave number for a given parallel
! FPSTRET : STRETCHING FACTOR
! NFPTTYP : 1 = POLE OF STRETCHING, POLE OF THE COLLOCATION GRID
!             AT THE NORTHERN POLE OF THE REAL EARTH.
!           2 = POLE OF STRETCHING, POLE OF THE COLLOCATION GRID
!             ANYWHERE ON THE REAL EARTH.
! RFPMU   : sine of the outpu gaussian latitudes

!========== FOR ALADIN SUBDOMAIN AS OUTPUT ONLY =============

! NMFPMAX: Truncation in y
! LFPMAP : .T./.F. if the domain is defined through its coordinates/wavelengths
! FPLX   : wavelength in x
! FPLY   : wavelength in y
! FPLON0 : LA0 geographic longitude of ref. for the projection !
! FPLAT0 : FI0 geographic latitude of ref. for the projection  !  IN DEGREES !!!
! LFPMRT : If .T. in Mercator case => use of Rotated/Tilted option
! FPRPK  : K projection parameter and definition

INTEGER(KIND=JPIM) :: NFPMAX(JPOSDOM)
INTEGER(KIND=JPIM) :: NFPRGRI(JPOSGL)
REAL(KIND=JPRB) :: FPVALH(0:JPOSLE)
REAL(KIND=JPRB) :: FPVBH(0:JPOSLE)

REAL(KIND=JPRB),ALLOCATABLE:: RFPLA(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPLO(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPLA_DEP(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPLO_DEP(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPMU(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPGM(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPNORX(:)
REAL(KIND=JPRB),ALLOCATABLE:: RFPNORY(:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NFPMEN(:)
INTEGER(KIND=JPIM) :: NFPLEV
INTEGER(KIND=JPIM) :: NFPRGPG
INTEGER(KIND=JPIM) :: NFPRGPL
INTEGER(KIND=JPIM) :: NFPRGPLX
INTEGER(KIND=JPIM) :: NFPRGPL_DEP
INTEGER(KIND=JPIM) :: NFPRGPLX_DEP
INTEGER(KIND=JPIM) :: NFPHTYP
INTEGER(KIND=JPIM) :: NFPTTYP
INTEGER(KIND=JPIM) :: NMFPMAX
REAL(KIND=JPRB) :: FPMUCEN
REAL(KIND=JPRB) :: FPLOCEN
REAL(KIND=JPRB) :: FPSTRET
REAL(KIND=JPRB) :: FPLX
REAL(KIND=JPRB) :: FPLY
REAL(KIND=JPRB) :: FPLON0
REAL(KIND=JPRB) :: FPLAT0
REAL(KIND=JPRB) :: FPRPK
REAL(KIND=JPRB) :: FPNLGINC
LOGICAL :: LFPMAP
LOGICAL :: LFPMRT
LOGICAL :: LFPOSHOR
LOGICAL :: LFPOSVER
LOGICAL :: LFPEZO
LOGICAL :: LFPOSBUF
LOGICAL :: LFPDISTRIB
INTEGER(KIND=JPIM),ALLOCATABLE:: NFPNUMD_DEP(:)

!     ------------------------------------------------------------------
END MODULE YOMFPG
