MODULE YOMGRB

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOM_YGFL  , ONLY : JPGHG, JPGRG , JPTRAC
USE YOM_GRIB_CODES

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------
!*    GRIB CODING DESCRIPTORS

! NLOCGRB - ECMWF LOCAL USAGE IDENTIFIER
! NTOTENS - TOTAL NUMBER OF FORCASTS IN ENSEMBLE
! NENSFNB - ENSAMBLE FORECAST NUMBER
! NCALVAL - cal/val number
! NSTREAM - EXPLICIT STREAM NUMBER (OR ZERO FOR DEFAULT STREAMS TO BE USED)
! NSYSTEM - FOR USE IN SEASONAL STREAM (DIFFERENT OPERATIONAL SYSTEMS)
! NMETHOD - FOR USE IN SEASONAL STREAM (DIFFERENT ENSEMBLES)
! NREFERENCE - FOR USE IN HINDCAST STREAM 
! NCONSENSUS - FOR MULTI_ANALYSIS STREAM (1 if consensus)
! NDWD    - FOR MULTI_ANALYSIS STREAM (1 if DWD analysis used)
! NMFR    - FOR MULTI_ANALYSIS STREAM
! NNCEP   - FOR MULTI_ANALYSIS STREAM
! NUKM   - FOR MULTI_ANALYSIS STREAM
! NSTEPLPP(II) - STEP FOR PREVIOUS P.P. OF VARIABLE WITH GRIB CODE II
! NSMAXNP - SUBTRUNCTATION FOR COMPLEX PACKING OF SPECTRAL COEFF
! NBITSSH - NUMBER OF BITS FOR PACKING SPECTRAL COEFF.
! NBITSGG - NUMBER OF BITS FOR PACKING GRID-POINT DATA
! NJDIAG  - SENSITIVITY DIAGNOSTIC NUMBER (1=>J1, 2=>J2, 3=>J3, 4=>J4)
! NJDOMAI - SENSITIVITY DIAGNOSTIC MASK REGION (0=>global, 1=>europe,
!                                               2=>N.H., 3=>S.H.)
! NLEG    - current VAREPS leg number (eg 1(2) for the T399(T255) part of a T399-T255 VAREPS)

! NGRBSP2, NGRBSP3 - Grib codes for fields in SPA2 and SPA3
! NGRBGP2, NGRBGP3 - Grib codes for grid point fields
!---------------------------------------------------
! NGRBTCTRAC(JPTRAC)  - 210184 Total column TRAC2: Sulfure hexaFluoride
!                     - 210183 Total column TRAC1: Radon
! NGRBGHG(JPGHG)   - 210061 GHG1: Carbon dioxide
!                  - 210062 GHG2: Methane
!                  - 210063 GHG3: Nitrous oxide
! NGRBKCH4         - 210071 CH4 loss rate because of reaction with OH
! NGRBTCGHG(JPGHG) - 210064 Total column GHG1: Carbon Dioxide
!                  - 210065 Total column GHG2: Methane
!                  - 210066 Total column GHG3: Nitrous Oxide

!---------------------------------------------------
! NGRBCO2OF        - 210067 CO2 - ocean flux
! NGRBCO2NBF       - 210068 CO2 - biosphere flux
! NGRBCO2APF       - 210069 CO2 - anthropogenic emissions
! NGRBCO2FIRE      - 210080 CO2 - biomass burning
! NGRBCH4F         - 210070 CH4 surface fluxes - aggregated field
! NGRBCH4FIRE      - 210082 CH4 - fire emissions
!
!---------------------------------------------------
! NGRBGRG(JPGRG)   - 210129 GRG1: NOx Nitrogen oxides
!                  - (210121 GRG1: NO2 Nitrogen dioxide)
!                  - 210122 GRG2: SO2 Sulphur dioxide
!                  - 210123 GRG3: CO Carbon monoxide
!                  - 210124 GRG4: HCHO Formaldehyde
!                  - 210203 GRG5: GO3 GEMS Ozone

! NGRBTCGRG(JPGRG) - 210130 Total column GRG1: Nitrogen oxides
!                  - (210125 Total column GRG1: Nitrogen dioxide)
!                  - 210126 Total column GRG2: Sulphur dioxide
!                  - 210127 Total column GRG3: Carbon monoxide
!                  - 210128 Total column GRG4: Formaldehyde
!                  - 210206 Total column GRG5: GEMS Ozone
! NGRBGRGSFC(JPGRG)   - 210101 GRG1: NOx Nitrogen oxides SFC flux
!                     - 210102 GRG2: SO2 Sulphur dioxide SFC flux
!                  - 210103 GRG3: CO Carbon monoxide SFC flux 
!                  - 210104 GRG4: HCHO Formaldehyde SFC flux
!                  - 210105 GRG5: GO3 GEMS Ozone SFC flux



! NGRBTRAC(JPTRAC)  - 210182 TRAC2: SF6
!                   - 210181 TRAC1: Radon

INTEGER(KIND=JPIM), PARAMETER :: JPN3SP=10
INTEGER(KIND=JPIM), PARAMETER :: JPN2SP=4

INTEGER(KIND=JPIM) :: NSEC0(2)
INTEGER(KIND=JPIM) :: NSEC1(2048)
INTEGER(KIND=JPIM) :: NSEC2SPP(22)
INTEGER(KIND=JPIM) :: NSEC2SPM(22)
INTEGER(KIND=JPIM) :: NSEC3(2)
INTEGER(KIND=JPIM) :: NSEC4(42)
INTEGER(KIND=JPIM) :: NGRBS3(0:JPN3SP)
INTEGER(KIND=JPIM) :: NGRBS2(0:JPN2SP)
INTEGER(KIND=JPIM) :: NSTEPLPP(255)
REAL(KIND=JPRB) :: RSEC3(2)

INTEGER(KIND=JPIM),ALLOCATABLE:: NSEC2GG(:)
REAL(KIND=JPRB),ALLOCATABLE:: RSEC2(:)

INTEGER(KIND=JPIM) :: MSEC0(2)
INTEGER(KIND=JPIM) :: MSEC1(2048)
INTEGER(KIND=JPIM) :: MSEC2SPP(22)
INTEGER(KIND=JPIM) :: MSEC2SPM(22)
INTEGER(KIND=JPIM) :: MSEC3(2)
INTEGER(KIND=JPIM) :: MSEC4(42)
INTEGER(KIND=JPIM) :: MGRBS3(0:JPN3SP)
INTEGER(KIND=JPIM) :: MGRBS2(0:JPN2SP)
REAL(KIND=JPRB) :: SSEC3(2)

INTEGER(KIND=JPIM),ALLOCATABLE:: MSEC2GG(:)
REAL(KIND=JPRB),ALLOCATABLE:: SSEC2(:)

INTEGER(KIND=JPIM), ALLOCATABLE :: NGRBSP2(:), NGRBSP3(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: NGRBGP2(:), NGRBGP3(:)

INTEGER(KIND=JPIM) :: NLOCGRB
INTEGER(KIND=JPIM) :: NTOTENS
INTEGER(KIND=JPIM) :: NENSFNB
INTEGER(KIND=JPIM) :: NCALVAL
INTEGER(KIND=JPIM) :: NSTREAM
INTEGER(KIND=JPIM) :: NSYSTEM
INTEGER(KIND=JPIM) :: NMETHOD
INTEGER(KIND=JPIM) :: NREFERENCE
INTEGER(KIND=JPIM) :: NCONSENSUS
INTEGER(KIND=JPIM) :: NDWD
INTEGER(KIND=JPIM) :: NMFR
INTEGER(KIND=JPIM) :: NNCEP
INTEGER(KIND=JPIM) :: NUKM
INTEGER(KIND=JPIM) :: NSMAXNP
INTEGER(KIND=JPIM) :: NBITSSH
INTEGER(KIND=JPIM) :: NBITSGG
INTEGER(KIND=JPIM) :: NJDIAG
INTEGER(KIND=JPIM) :: NJDOMAI
INTEGER(KIND=JPIM) :: NJITER
INTEGER(KIND=JPIM) :: MLOCGRB
INTEGER(KIND=JPIM) :: MTOTENS
INTEGER(KIND=JPIM) :: MENSFNB
INTEGER(KIND=JPIM) :: MSMAXNP
INTEGER(KIND=JPIM) :: MBITSSH
INTEGER(KIND=JPIM) :: MBITSGG
INTEGER(KIND=JPIM) :: MJDIAG
INTEGER(KIND=JPIM) :: MJDOMAI
INTEGER(KIND=JPIM) :: MJITER

INTEGER(KIND=JPIM) :: NLEG

INTEGER(KIND=JPIM), DIMENSION(JPGHG) :: NGRBGHG = (/&
 & 210061, 210062, 210063/)

INTEGER(KIND=JPIM), DIMENSION(JPGHG) :: NGRBTCGHG = (/&
 & 210064, 210065, 210066/)

INTEGER(KIND=JPIM), DIMENSION(JPTRAC) :: NGRBTRAC

INTEGER(KIND=JPIM), DIMENSION(JPTRAC) :: NGRBTCTRAC
! = ( / & 210183, 210184/) 

INTEGER(KIND=JPIM), DIMENSION(JPTRAC) :: NGRBTRACSFC

INTEGER(KIND=JPIM), DIMENSION(JPGRG) :: NGRBGRG 

INTEGER(KIND=JPIM), DIMENSION(JPGRG) :: NGRBTCGRG 

INTEGER(KIND=JPIM), DIMENSION(JPGRG) :: NGRBGRGSFC
 
!     ------------------------------------------------------------------
END MODULE YOMGRB
