MODULE YOMIASI

USE PARKIND1, ONLY  : JPIM, JPRB

USE YOMTVRAD, ONLY  : MSENSOR_AIRS, MSENSOR_IASI

IMPLICIT NONE

SAVE

! Sets up structures to be used in processing of AIRS and IASI.

! 1st Feb 2006   Original Version    A. Collard.

INTEGER(KIND=JPIM), Parameter :: JP__Digital_Filter = 1

INTEGER(KIND=JPIM), Parameter :: JP__Min_Sensor_Index = MSENSOR_AIRS
INTEGER(KIND=JPIM), Parameter :: JP__Max_Sensor_Index = MSENSOR_IASI

TYPE Cloud_Detect_Type
   INTEGER(KIND=JPIM)          :: M__Sensor        ! Unique ID for sensor
   INTEGER(KIND=JPIM)          :: N__Filter_Method ! Averaging filter for 
                                                   ! cloud detection
   INTEGER(KIND=JPIM)          :: N__Num_Bands     ! Number of cloud detection
                                                   ! bands
   INTEGER(KIND=JPIM),POINTER  :: N__GradChkInterval(:) ! Window used in 
                                                   ! gradient calculation
   INTEGER(KIND=JPIM),POINTER  :: N__Band_Size(:)  ! No. of chans in each band
   INTEGER(KIND=JPIM),POINTER  :: N__Bands(:,:)    ! List of chans in bands
   INTEGER(KIND=JPIM),POINTER  :: N__Window_Width(:) ! List of filter window 
                                                     ! widths per band
   INTEGER(KIND=JPIM),POINTER  :: N__Window_Bounds(:,:) ! Channels to use for 
                                          ! quick exit spectral gradient check
   INTEGER(KIND=JPIM),POINTER  :: N__BandToUse(:)    ! Which band to use for cloud 
                                                     ! height (used in cross-band)

   LOGICAL  :: L__Do_Quick_Exit       ! Allow quick exit
   LOGICAL  :: L__Do_CrossBand        ! Cross band cloud detection

   ! N.B. The next two could be extended to band arrays 
   REAL(KIND=JPRB),POINTER  :: R__BT_Threshold(:)   ! BT threshold for 
                                                    ! cloud  contamination
   REAL(KIND=JPRB),POINTER  :: R__Grad_Threshold(:) ! Gradient threshold for 
                                                    ! cloud contamination
   REAL(KIND=JPRB),POINTER  :: R__Window_Grad_Threshold(:) ! Window gradient 
                                             ! threshold for quick exit test 

   ! Aerosol detection parameters

   LOGICAL  :: L__Do_AerosolDetection   

   INTEGER(KIND=JPIM) :: N__Num_Aerosol_Tests

   INTEGER(KIND=JPIM), POINTER :: N__Num_Aerosol_Chans(:)

   INTEGER(KIND=JPIM), POINTER :: N__Aerosol_Chans(:,:)
   REAL(KIND=JPRB),POINTER     :: R__Aerosol_Abscissae(:,:)
   REAL(KIND=JPRB),POINTER     :: R__Aerosol_Threshold(:)
   REAL(KIND=JPRB),POINTER     :: R__AerosolMinNorm(:)

END TYPE Cloud_Detect_Type
    
TYPE(Cloud_Detect_Type) :: &
&             S__Cloud_Detect_Setup(JP__Min_Sensor_Index:JP__Max_Sensor_Index)


END MODULE YOMIASI
