MODULE YOMJG

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!*
!      /YOMJG/ - Jb RELEVANT PARAMETERS

!     BILL HECKLEY     *ECMWF*

!     --------- definition of Jb through namelist NAMJG --------------------

!     LCFCE  : .T. ===> Use horizontally constant background errors   T= YES
!     LSPFCE : .T. ===> background error normalization in spectral space
!     L3DBGERR:.T. ===> Use 3d (nonseperable) variances of background error
!     LJBTEST  To activate the technical test of Jb components
!     REDNMC   Rate of standard deviation reduction from the NMC covariances
!     REDQGLOLAM: Factor between sigmab(q) from GLObal model Arpege and the LAM Aladin  
!     LFACHRO: Increase climate variance in tropics using ad hoc factor
!     LCLMSFCE: Use climatalogical variances in error growth model
!     LRDQERR: T=Read background error for Q from file. F=Calculate it.
!     LSBLATLONG: T=errgrib on lat/lon grid, F=errgrib on Gaussian grid
!     NGEOBAL:   1 ===> (default) spectral balance on work sphere
!                2 ===> balance on geographical sphere using TRAGEO
!                3 ===> approx. balance on geographical sphere without TRAGEO
!     NGEOCOR:   1 ===> (default) homogeneous correlations on work sphere
!                2 ===> homogeneous correls on geographical sphere using SPDICO
!     NJBVERB  level of diagnostics printouts in Jb setups : (default is 1)
!              0=terse  1=normal  2=verbose
!     LJBENER  make Jb equal to a total energy scalar product
!     RAMENER  amount of energy vs. standard Jb (LJBENER option)
!     RSCENER  scaling of the total energy Jb (LJBENER option)
!     LCORCOSU enforce compactly supported correlations
!     LCORCOSU_VCOR compact support on vertical correlations (instead of covariances)
!     LJGNRSGP normalize forecast gridpoint fields (q and o3) in spectral space
!     L_JBVCOORD:  .T. ===> correlations in "Jb vertical coordinates"
!     LJB_UNIVARIATE:    .T. => Jb is univariate. No balance operators.
!     LJB_OMEGA_BALANCE: .T. => balance operator includes an omega equation
!     LJB_OMEGA_BALANCE_RELAX : .T. => Apply optional scale-dependent relaxation of omega balance
!     LJB_NONLINEAR_BALANCE: .T. => use the linearized nonlinear balance eqn.
!     LJB_NONLINEAR_BALANCE_RELAX : .T. => Apply optional scale-dependent relaxation of non linear balance
!     LJB_NONLINEAR_CVHUM: .T. => use the nonlinearly normalized relative
!                                 humidity control variable
!                                 (linearized in the inner loops)
!     LJB_NONLINEAR_CVO3: .T. => use the nonlinearly normalized ozone
!                                 control variable
!                                 (linearized in the inner loops)
!     LJB_SUPSAT: .T. => humidity cv with supersaturation and q-T balance
!     LJBBALBETA: .T. => use the beta-plane balance for Aladin Jb
!     NPSMAX: truncation of the bi-Fourier f for the beta-plane in Aladin Jb
!     TMEANUVER: Vertical profiles of std dev for Aladin mean wind components (U,V)
!     ELC is a length-scale at which the omega and non-linear balances
!     are (optionally) relaxed to a zero value
!     LJB_TLBALSTAT: .T. => Use TL instead of full balance operators when 
!                           generating BG statistics
!     RJB_CSLEN: Compact support length scale when generating BG statistics
!     LBALSTRATO: .T. => use the balance operators in stratosphere
!     NLEVBAL0, NLEVBAL1: levels for switching off balance in the stratoshere
!     --------------------  Jb Internal arrays ------------------------

!     FGSCNM3D: INVERSE HORIZONTAL PREDICTION ERROR CORRELATION MATRIX
!               IN SPECTRAL SPACE - FOR VARIABLES IN SPA3JB
!     FGSCNM2D: INVERSE HORIZONTAL PREDICTION ERROR CORRELATION MATRIX
!               IN SPECTRAL SPACE - FOR VARIABLES IN SPA2JB

!     SIBI    : INVERSE OF VERTICAL STRUCTURE MATRIX (B) - NEEDED FOR CVAR2...
!     FP2TPS  : P TO T,PS OPERATOR - NEEDED FOR CVAR2IN/AD

!     FCEMN3D : global mean profiles of bg errors for fields in SPA3JB
!     FCEMN2D : global mean profiles of bg errors for fields in SPA2JB
!     FCEMN_DERIVED : global mean bg errors for fields not in JB
!                     (Used in the forecast-error calculation, only.)
!         %PS : Global rms values of background errors (lnsp)
!         %U  : Global rms values of background errors (zonal wind component)
!         %Z  : Global rms values of background errors (height)
!         %T  : Global rms values of background errors (temperature)

!     FCEIMN  : Global rms values of input background errors

!     VCORS   : Structure containing vertical correlation matrices
!     VCORS%EVALS : MATRIX OF EIGENVALUES OF VERTICAL CORRELATIONS
!     VCORS%EVECS : MATRIX OF EIGENVECTORS OF VERTICAL CORRELATIONS

!     FGMWNE : MASS-WEIGHTED NORMALIZED FC ERRORS FROM OI

!     MAPMOCV(NGPTOT) : global mapping of the modification operators on grid.
!        On each point JJ the matrices to use are FMOCVx(:,:,:,MAPMOCV(JJ))

!     STPS(:,:,:),SDIV(:,:,:),SO3(:,:,:),BFACT1(:),BFACT2(:) : coefficients
!        of the generalized statistical balance operator (J. Derber)
!     SHUM(:,:,:),BFACT(:) - extension of new JB for ALADIN
!     SCORI_R(:),SCORI_I(:) : bi-Fourier f for JB in ALADIN

!     N_SPJB_VARS    : number of model variables that have a
!                      spectral representation in SPA3JB/SPA2JB.
!     M_GRIBCODES     : Used for NAMELIST input of the GRIB codes that
!                       identify each variable in Jb. These codes are
!                       used to match Jb variable with model variables.
!     M_GRIBCODES_FCE : Used for NAMELIST input of the GRIB codes that
!                       identify the forecast error fields.
!     C_COR_STRINGS   : Used for NAMELIST input of they strings that
!                       identify each variable in Jb. These strings are
!                       used to match Jb variables with data in the ".cv"
!                       statistics file.
!     SPJB_VARS_INFO : information about the fields in SPA3JB/SPA2JB.

!     --------------- Jb internal variables ----------------------------

!     LCTLFG   Keep FG in control space (i.e. do not use LSUBBG)
!     LSUBBG   .T. ===> CHAVAR subtracts background
!              .F. ===> CHAVAR assumes background already subtracted
!              This is an internal switch used to control the
!              operation of routines CVAR2, CVAR3, CVARBL, FLTRG and
!              their inverses and adjoints.

!     NOFCEF : Number of fields on forecast error file

!     EPSBALW,
!     EPSBALM: Relative weight given to Gravity mode terms compared to
!              Rossby in Jg cost function. For Mass and for Wind.
!     NSMAX_JB max meridian wave number for B stats.
!     NMSMAX_JB max zonal wave number for B stats.
!-------------- GEMS specific variables ------------------------
!     NAEROCV: Number of aerosol-related control variables
!     JPAEROCV: Max number of aerosol-related control variables

LOGICAL :: L3DBGERR
LOGICAL :: LCFCE
LOGICAL :: LSUBBG
LOGICAL :: LCTLFG
LOGICAL :: LRDQERR
LOGICAL :: LSBLATLONG
LOGICAL :: LJBTEST
LOGICAL :: LSPFCE
LOGICAL :: LJBENER
LOGICAL :: LFACHRO
LOGICAL :: LCLMSFCE
LOGICAL :: LCORCOSU
LOGICAL :: LCORCOSU_VCOR
LOGICAL :: LJGNRSGP
LOGICAL :: L_JBVCOORD
LOGICAL :: LJB_UNIVARIATE
LOGICAL :: LJB_OMEGA_BALANCE
LOGICAL :: LJB_OMEGA_BALANCE_RELAX
LOGICAL :: LJB_NONLINEAR_BALANCE
LOGICAL :: LJB_NONLINEAR_BALANCE_RELAX
LOGICAL :: LJB_NONLINEAR_CVHUM
LOGICAL :: LJB_NONLINEAR_CVO3
LOGICAL :: LJB_SUPSAT
LOGICAL :: LJBBALBETA
LOGICAL :: LJB_TLBALSTAT

INTEGER(KIND=JPIM) :: NGEOBAL
INTEGER(KIND=JPIM) :: NGEOCOR
INTEGER(KIND=JPIM) :: NJBVERB
INTEGER(KIND=JPIM) :: NPSMAX

LOGICAL :: LBALSTRATO
INTEGER(KIND=JPIM) :: NLEVBAL0, NLEVBAL1

REAL(KIND=JPRB) :: REDNMC
REAL(KIND=JPRB) :: REDQGLOLAM
REAL(KIND=JPRB) :: RAMENER
REAL(KIND=JPRB) :: RSCENER
REAL(KIND=JPRB) :: RJB_CSLEN
REAL(KIND=JPRB) :: ELC

LOGICAL :: LEVSREDNMC
INTEGER(KIND=JPIM) :: NLEVREDNMC

INTEGER(KIND=JPIM) :: NLEV_REDINC
REAL(KIND=JPRB),ALLOCATABLE:: REDINC(:)

REAL(KIND=JPRB),ALLOCATABLE:: SIBI(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: FP2TPS(:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: FGSCNM3D(:,:,:), FGSCNM2D(:,:)

REAL(KIND=JPRB),ALLOCATABLE:: EPSBALM(:)
REAL(KIND=JPRB),ALLOCATABLE:: EPSBALW(:)
REAL(KIND=JPRB),ALLOCATABLE:: TMEANUVER(:,:)

TYPE FCEMN_DERIVED_STRUCT
  REAL(KIND=JPRB) :: PS
  REAL(KIND=JPRB) :: PSU
  REAL(KIND=JPRB), POINTER :: U (:)
  REAL(KIND=JPRB), POINTER :: Z (:)
  REAL(KIND=JPRB), POINTER :: T (:)
  REAL(KIND=JPRB), POINTER :: O3(:)
END TYPE FCEMN_DERIVED_STRUCT

TYPE (FCEMN_DERIVED_STRUCT) :: FCEMN_DERIVED

TYPE VCORS_STRUCT
  REAL(KIND=JPRB), POINTER :: EVALS(:,:)
  REAL(KIND=JPRB), POINTER :: EVECS(:,:,:)
END TYPE VCORS_STRUCT

TYPE (VCORS_STRUCT), ALLOCATABLE :: VCORS(:)

REAL(KIND=JPRB),ALLOCATABLE:: FCEMN3D(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: FCEMN2D(:)
REAL(KIND=JPRB),ALLOCATABLE:: FCEIMN(:)
REAL(KIND=JPRB),ALLOCATABLE:: FGMWNE(:)
REAL(KIND=JPRB),ALLOCATABLE:: STPS(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE:: SDIV(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE:: SO3(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE:: BFACT1(:)
REAL(KIND=JPRB),ALLOCATABLE:: BFACT2(:)
REAL(KIND=JPRB),ALLOCATABLE:: SHUM(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE:: BFACT(:)
REAL(KIND=JPRB),ALLOCATABLE:: SCORI_R(:)
REAL(KIND=JPRB),ALLOCATABLE:: SCORI_I(:)
INTEGER(KIND=JPIM),ALLOCATABLE:: MAPMOCV(:)

INTEGER(KIND=JPIM) :: NOFCEF

TYPE SPJB_VARS_INFO_STRUCT
  INTEGER(KIND=JPIM) :: IGRIBCODE     ! GRIB code for model field
  INTEGER(KIND=JPIM) :: IGRIBCODE_FCE ! GRIB code for background error
  INTEGER(KIND=JPIM) :: IPT         ! index into model (SPA3/SPA2/SPGFL/GLF)
  INTEGER(KIND=JPIM) :: IPTJB       ! index into SPA3JB/SPA2JB
  INTEGER(KIND=JPIM) :: IPTFCE      ! index into FCEBUF
  INTEGER(KIND=JPIM) :: IPTBG       ! index into SP7A3, GP7A3 or SP7A2
  LOGICAL            :: L_IN_SPA3   ! .T. => maps to model SPA3 array
  LOGICAL            :: L_IN_SPA2   ! .T. => maps to model SAP2 array
  LOGICAL            :: L_IN_SPGFL  ! .T. => maps to model SPGFL array
  LOGICAL            :: L_IN_GPGFL  ! .T. => maps to model GPGFL array
  CHARACTER(LEN=40)  :: COR_STRING  ! Identifying string in ".cv" file
END TYPE SPJB_VARS_INFO_STRUCT

INTEGER(KIND=JPIM) :: N_SPJB_VARS
INTEGER(KIND=JPIM), DIMENSION(100) :: M_GRIBCODES, M_GRIBCODES_FCE
CHARACTER(LEN=40) , DIMENSION(100) :: C_COR_STRINGS
TYPE(SPJB_VARS_INFO_STRUCT), ALLOCATABLE ::SPJB_VARS_INFO(:)

INTEGER(KIND=JPIM) :: NSMAX_JB
INTEGER(KIND=JPIM) :: NMSMAX_JB

INTEGER(KIND=JPIM) :: NAEROCV
INTEGER(KIND=JPIM), PARAMETER :: JPAEROCV=1
!     ------------------------------------------------------------------

END MODULE YOMJG
