MODULE YOMJQ

!   Purpose
!   -------
!     Data structures to store the components of model error
!     covariance matrix.

!   Author
!   ------
!     Yannick Tremolet

!   Modifications
!   -------------
!     Original    16-Jan-04  
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPRB, JPIM
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
SAVE

LOGICAL :: LQEQB    ! Use B as model error covariance matrix
LOGICAL :: LJQBAL   ! Use Jb balance operators in Jq

LOGICAL :: LJQHCOR  ! Use horizontal correlations
LOGICAL :: LJQVCOR  ! Use vertical   correlations
LOGICAL :: LJQZERO  ! Model error without a Jq term (dangerous)

REAL(KIND=JPRB) :: FJQCOST  ! Model error term of cost function

! ERRMODMN: Mean profiles of model error
TYPE ERRMODMN_STRUCT
  REAL(KIND=JPRB), POINTER :: V2D(:)      => NULL()
  REAL(KIND=JPRB), POINTER :: V3D(:,:)    => NULL()
  REAL(KIND=JPRB), POINTER :: V2DINV(:)   => NULL()
  REAL(KIND=JPRB), POINTER :: V3DINV(:,:) => NULL()
  REAL(KIND=JPRB), POINTER :: VALPHA(:)   => NULL()
  REAL(KIND=JPRB) :: VSURF
END TYPE ERRMODMN_STRUCT

TYPE (ERRMODMN_STRUCT) :: ERRMODMN

! Limit model error to stratosphere
INTEGER(KIND=JPIM) :: NLEVERR0, NLEVERR1

! ERRHCOR: Horizontal model error correlation matrix (inverse)
TYPE ERRMODHCOR_STRUCT
  REAL(KIND=JPRB), POINTER :: SP2D(:,:)   => NULL()
  REAL(KIND=JPRB), POINTER :: SP3D(:,:,:) => NULL()
END TYPE ERRMODHCOR_STRUCT

TYPE (ERRMODHCOR_STRUCT) :: ERRHCOR

! ERRVCOR: Vertical model error correlation matrix
TYPE ERRMODVCOR_STRUCT
  REAL(KIND=JPRB), POINTER :: VOR(:,:,:) => NULL()
  REAL(KIND=JPRB), POINTER :: DIV(:,:,:) => NULL()
  REAL(KIND=JPRB), POINTER :: TPS(:,:,:) => NULL()
  REAL(KIND=JPRB), POINTER :: HUM(:,:,:) => NULL()
  REAL(KIND=JPRB), POINTER :: OZN(:,:,:) => NULL()
END TYPE ERRMODVCOR_STRUCT

TYPE (ERRMODVCOR_STRUCT) :: ERRVCOR, ERRVCORIN

! Info local Ns
INTEGER(KIND=JPIM), ALLOCATABLE :: NVALERR(:)

! Generate data for model error statistics
LOGICAL :: LSTATMERR

! Store M(x_i) for computation of M(x_i)-x_{i+1} in Jq.
TYPE(SPECTRAL_FIELD), ALLOCATABLE :: SPJQMODERR(:)
TYPE(SPECTRAL_FIELD), ALLOCATABLE :: SPJQMODER5(:)

END MODULE YOMJQ
