MODULE YOMLIMB

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!*

!     /YOMLIMB/  N. Bormann  05/11/2004 

!     MODIFICATIONS:


!     VARIABLES DEFINING LOGISTICS FOR LIMB COMPUTATIONS.

!  Define sensor numbering
INTEGER(KIND=JPIM), PARAMETER :: MXSENSOR_LRAD   = 2  ! Maximum number of rtlimb sensors
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MIPAS   = 32 ! RTLIMB MIPAS ID
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MLS     = 33 ! RTLIMB MLS ID
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MIPASSQ = 1  ! MIPAS sensor sequence number
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MLSSQ   = 2  ! MLS sensor sequence number
INTEGER(KIND=JPIM), PARAMETER :: MXCHAN_LRAD      = 500 ! Maximum number of channels (used for ROERR_LRAD & ROBIAS_LRAD only)

! List linking BUFR-SATID, RT-PLATFORM/SERIES-NUMBER, RT-SATELLITE-NUMBER
INTEGER(KIND=JPIM), PARAMETER :: NRTLIMB_LIST=2
INTEGER(KIND=JPIM)            :: IRTLIMB_LIST(NRTLIMB_LIST, 3) 

! Thesholds used in cloud and top screening for MIPAS:
! PP_MIPAS_ZCLOUD_THRESH - Use cloud detect. for tan hgts below this level [m]
! PP_MIPAS_CI_THRESH     - Cloud index threshold (Kramer et al. at Envis.Symp.)
! PP_MIPAS_WR_THRESH   - Threshold on clearest MIPAS channel (960.7 cm-1)[r.u.]
! PP_MIPAS_ZTAN_MAX      - Maximum tangent height allowed [m]
! PP_MIPAS_ZTAN_MIN      - Minimum tangent height allowed [m]
REAL(KIND=JPRB), PARAMETER :: PP_MIPAS_ZCLOUD_THRESH = 39000._JPRB
REAL(KIND=JPRB), PARAMETER :: PP_MIPAS_CI_THRESH = 5._JPRB!
REAL(KIND=JPRB), PARAMETER :: PP_MIPAS_WR_THRESH = 100._JPRB ! (Dudhia: 125.)
REAL(KIND=JPRB), PARAMETER :: PP_MIPAS_ZTAN_MAX  = 55000._JPRB
REAL(KIND=JPRB), PARAMETER :: PP_MIPAS_ZTAN_MIN  = 12000._JPRB

! Parameters used in RTLIMB preparations
! PP_TOLERANCE - Allow use of RTLIMB coefficient outside limits by this fract.
! PP_MESO_P    - Pressure limit [Pa] above which mesosperic extrapolation of
!                temperature above model top is applied
! PP_MESO_DT_BY_DLNP - Fixed mesospheric temperature lapse rate used for 
!                extrapolation above model top
REAL(KIND=JPRB), PARAMETER :: PP_TOLERANCE       = 0.1_JPRB
REAL(KIND=JPRB), PARAMETER :: PP_MESO_P          = 60._JPRB
REAL(KIND=JPRB), PARAMETER :: PP_MESO_DT_BY_DLNP = 15._JPRB

!     ROERR_LRAD   R    OBSERVATION ERRORS FOR LIMB RADIANCES
!     ROBIAS_LRAD  R    OBSERVATION BIASES FOR LIMB RADIANCES
REAL(KIND=JPRB) :: ROERR_LRAD(0:MXSENSOR_LRAD,MXCHAN_LRAD)
REAL(KIND=JPRB) :: ROBIAS_LRAD(0:MXSENSOR_LRAD,MXCHAN_LRAD)

! Some dimensions. These are set via sulimb or rtl_setup, defined through
! the used data or through values defined in the RTLIMB code.

INTEGER(KIND=JPIM) :: NLIMBGRP ! Size of Y_LIMBGRP_TABLE
INTEGER(KIND=JPIM) :: NJPPF_RTL   ! Maximum number of profs that can be processed in one go
INTEGER(KIND=JPIM) :: NJPNSAT_RTL ! Maximum number of coefficient files
INTEGER(KIND=JPIM) :: NPNAV_RTL   ! Number of profile variables
INTEGER(KIND=JPIM) :: NPMXTAN  ! Maximum number of tangent heights

LOGICAL            :: LQTAN_PRES ! Use tangent pressures?


! Define table for administration of sat-ID, sensor and codetype...
TYPE y_limbgrp_t
 INTEGER(KIND=JPIM)         :: bufrsatid          ! BUFR Satellite Identifier
 INTEGER(KIND=JPIM)         :: codetype           ! Obs codetype (as in ODB)
 INTEGER(KIND=JPIM)         :: sensor             ! Sensor used by RTLIMB
 INTEGER(KIND=JPIM)         :: satgroup           ! Satellite data group number
 INTEGER(KIND=JPIM)         :: rtid               ! SatID used by RTLIMB
 INTEGER(KIND=JPIM)         :: rtseries           ! Series/platform
 INTEGER(KIND=JPIM)         :: rtsubtype          ! Subtype (not used)
 INTEGER(KIND=JPIM)         :: rtcoef_pos         ! Pointer to RT-coefficients
 INTEGER(KIND=JPIM)         :: nlsat              ! Number of levels used in RT model
 REAL(KIND=JPRB), POINTER   :: radpre(:)          ! Pressure levels used in RT model
 REAL(KIND=JPRB), POINTER   :: tminrt(:)          ! Limit profiles for 
 REAL(KIND=JPRB), POINTER   :: tmaxrt(:)          ! validity of RT coefficients
 REAL(KIND=JPRB), POINTER   :: qminrt(:)
 REAL(KIND=JPRB), POINTER   :: qmaxrt(:)
 REAL(KIND=JPRB), POINTER   :: o3minrt(:)
 REAL(KIND=JPRB), POINTER   :: o3maxrt(:)
 INTEGER(KIND=JPIM)         :: nmxchan            ! Number of channels in coef structure
 INTEGER(KIND=JPIM), POINTER:: ivalchan(:)        ! List of valid channel numbers
END TYPE y_limbgrp_t

TYPE(y_limbgrp_t), ALLOCATABLE, TARGET :: Y_LIMBGRP_TABLE(:)  ! Limb data group admin table

TYPE y_limb_chan_sel_t
 ! channel selection structure for limb sounding
 LOGICAL                  :: active         ! Use channel selection?
 CHARACTER(Len=256)       :: file           ! Name of channel selection file
 INTEGER(Kind=jpim)       :: nchan          ! Number of selected channels
 INTEGER(Kind=jpim)       :: nheight        ! Number of heights used for mask
 REAL(Kind=jprb), Pointer :: wno(:)         ! Selected wavenumbers 
 REAL(Kind=jprb), Pointer :: msk_heights(:) ! Height borders for mask
 LOGICAL, Pointer         :: msk(:,:)       ! Mask
 REAL(Kind=jprb), Pointer :: height_low(:)  ! Lowest height
 REAL(Kind=jprb), Pointer :: height_up(:)   ! Heighest height     
END TYPE y_limb_chan_sel_t

TYPE(y_limb_chan_sel_t), ALLOCATABLE, TARGET  :: Y_LIMB_CHAN_SEL(:) ! Channel selection 


! for setting up 2d plane


REAL(KIND=JPRB), PARAMETER :: dtheta = 6.27844E-3_JPRB ! T511 in Radians
!REAL(KIND=JPRB), PARAMETER :: dtheta = 0.01976042_JPRB  ! 125 km in radians

! SBH 7-4-2006

! GPSRO SAT IDs 

INTEGER(KIND=JPIM), PARAMETER :: MMETOP_1      = 3 
INTEGER(KIND=JPIM), PARAMETER :: MMETOP_2      = 4
INTEGER(KIND=JPIM), PARAMETER :: MMETOP_3      = 5
INTEGER(KIND=JPIM), PARAMETER :: MCHAMP        = 41
INTEGER(KIND=JPIM), PARAMETER :: MTERRASARX    = 42
INTEGER(KIND=JPIM), PARAMETER :: MGRACE_A      = 722
INTEGER(KIND=JPIM), PARAMETER :: MGRACE_B      = 723
INTEGER(KIND=JPIM), PARAMETER :: MCOSMIC_1     = 740
INTEGER(KIND=JPIM), PARAMETER :: MCOSMIC_2     = 741
INTEGER(KIND=JPIM), PARAMETER :: MCOSMIC_3     = 742
INTEGER(KIND=JPIM), PARAMETER :: MCOSMIC_4     = 743
INTEGER(KIND=JPIM), PARAMETER :: MCOSMIC_5     = 744
INTEGER(KIND=JPIM), PARAMETER :: MCOSMIC_6     = 745


!     ------------------------------------------------------------------

END MODULE YOMLIMB
