MODULE YOMLSFORC

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     -------------------------------------------------------------------------

!*    Control variables for the FORCINGS

! LGEOST_UV_FRC : switch for geostrophic forcing 
!                 (with a constant Coriolis parameter)
! RCORIO_FORC   : f_plan value of the Coriolis parameter used in the geostrophic forcing. 
!                 Default is RCORIO_FORC=10-4 s-1
! NGEOST_UV_TIME: time step between 2 forcing times (in seconds)
! NGEOST_U_DEB  : index of the first zonal geostrophic wind in GFL%YFORC
! NGEOST_U_NUM  : total number of zonal geostrophic wind in GFL%YFORC
!                 (if NGEOST_U_NUM=1 a constant forcing is applied)
! NGEOST_V_DEB  : index of the first meridional geostrophic wind in GFL%YFORC
! NGEOST_V_NUM  : total number of meridional geostrophic wind in GFL%YFORC
!                 (if NGEOST_V_NUM=1 a constant forcing is applied)

! LT_ADV_FRC    : switch for large scale temperature advection on the temperature field
! NT_ADV_TIME   : time step between 2 forcing times (in seconds)
! NT_ADV_DEB    : index of the first temperature tendency in GFL%YFORC
! NT_ADV_NUM    : total number of temperature tendencies in GFL%YFORC

! LQV_ADV_FRC   : switch for large scale specific humidity advection on the specific humidity field
! NQV_ADV_TIME  : time step between 2 forcing times (in seconds)
! NQV_ADV_DEB   : index of the first specific humidity tendency in GFL%YFORC
! NQV_ADV_NUM   : total number of specific humidity tendencies in GFL%YFORC

! LSW_FRC       : switch for large scale vertical advection 
!                (specify by w=Dz/Dt in m/s)
! NLSW_TIME     : time step between 2 forcing times (in seconds)
! NLSW_DEB      : index of the first large scale vertical velocity w in GFL%YFORC
! NLSW_NUM      : total number of large scale vertical velocities in GFL%YFORC

LOGICAL            :: LGEOST_UV_FRC      
REAL(KIND=JPRB)    :: RCORIO_FORC         
INTEGER(KIND=JPIM) :: NGEOST_UV_TIME      
INTEGER(KIND=JPIM) :: NGEOST_U_DEB        
INTEGER(KIND=JPIM) :: NGEOST_U_NUM        
INTEGER(KIND=JPIM) :: NGEOST_V_DEB        
INTEGER(KIND=JPIM) :: NGEOST_V_NUM        

LOGICAL            :: LT_ADV_FRC         
INTEGER(KIND=JPIM) :: NT_ADV_TIME         
INTEGER(KIND=JPIM) :: NT_ADV_DEB          
INTEGER(KIND=JPIM) :: NT_ADV_NUM          

LOGICAL            :: LQV_ADV_FRC        
INTEGER(KIND=JPIM) :: NQV_ADV_TIME        
INTEGER(KIND=JPIM) :: NQV_ADV_DEB         
INTEGER(KIND=JPIM) :: NQV_ADV_NUM        

LOGICAL            :: LSW_FRC           
INTEGER(KIND=JPIM) :: NLSW_TIME           
INTEGER(KIND=JPIM) :: NLSW_DEB            
INTEGER(KIND=JPIM) :: NLSW_NUM             

!     ------------------------------------------------------------------
END MODULE YOMLSFORC
