MODULE YOMMCC

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------

!*    KEYS FOR CLIMATE VERSION

! Coupling
! LMCC01 = .T.   ===> THE BOUNDARY CONDITIONS ARE READ IN CLIMATOLOGY FILES
!        = .F.   ===> THEY ARE READ IN THE HISTORY FILE
! LMCC02 = .T.   ===> THE SEA-ICE IS CONSIDERED AS LAND
!        = .F.   ===> THE SEA-ICE IS CONSIDERED AS OCEAN
! LMCC03 = .T.   ===> THE SST IS GIVEN BY THE COUPLER (ARPEGE VERSION)
!        = .F.   ===> THE SST IS GIVEN BY THE HIST. /BY THE CLIM. (LMCC01)
! LMCC04 = .T.   ===> OASIS COUPLER ACTIVE IN ECMWF ROUTINES
!        = .F.   ===> NO ECMWF OASIS COUPLING
! LMCC05 = .T.   ===> IFS VERSION OF SST COUPLING (COM SCHEME)
!        = .F.   ===> NO IFS COUPLING
! LMCC06 = .T.   ===> IFS VERSION OF SST COUPLING (TKE SCHEME)
!        = .F.   ===> NO IFS COUPLING

! NOACOMM        ===> METHOD OF OASIS COMMUNICATION (1=FORTRAN PIPES,
!                      2=C PIPES, 3=SVIPC)
! LMCCEC = .T.   ===> BOUNDARY CONDITIONS UPDATED BY ECMWF ROUTINES
!        = .F.   ===> THEY ARE NOT UPDATED
! LMCCIEC= .T.   ===> THE BOUNDARY CONDITIONS (SST) ARE INTERPOLATED IN TIME
!                     (DEFAULT) CLIMATOLOGY FILES
!        = .F.   ===> THEY ARE NOT INTERPOLATED
! LMCCICEIC =.T. ===> ICE INITIAL CONDITIONS SENT TO COUPLER AT STARTUP
!           =.F  ===> NO OUTPUT OF ICE
! NCLIMTOT       ===> TOTAL NUMBER OF FIELDS TO BE UPDATED IN TIME
! NCLIMR         ===> NUMBER OF UPDATING FIELDS TO BE READ FROM FILE
!                      (OTHERS COME FROM OASIS COUPLER)
! NCLIGC         ===> GRIB CODES OF THE FIELDS TO BE TIME UPDATED
! NCLIAER        ===> GRIB CODES OF CLIM.AEROSOL FIELDS TO BE TIME UPDATED
! NDIFC          ===> DIFFERENCE, IN DAYS, BETWEEN CONSECUTIVE TIMES
!                     IN THE CLIMATE FILE (ONLY USED FOR LMCCIEC=.FALSE.)
! NJDCR          ===> JULIAN DAY OF THE NEXT FIELD TO BE READ
!                     IN THE CLIMATE FILE (ONLY USED FOR LMCCIEC=.FALSE.)
! NP1            ===> INDEX FOR THE FIRST FILE IN TIME INTERPOLATION
!                     (ONLY USED FOR LMCCIEC=.TRUE.)
! NP2            ===> INDEX FOR THE SECOND FILE IN TIME INTERPOLATION
!                     (ONLY USED FOR LMCCIEC=.TRUE.)
! NPAER_1            ===> INDEX FOR THE FIRST FILE IN TIME INTERPOLATION
!                     (ONLY USED FOR LEAERDIU=.TRUE.)
! NPAER_2            ===> INDEX FOR THE SECOND FILE IN TIME INTERPOLATION
!                     (ONLY USED FOR LEAERDIU=.TRUE.)
! NPCO2_1            ===> INDEX FOR THE FIRST FILE IN TIME INTERPOLATION
!                     (ONLY USED FOR LECO2DIU=.TRUE.)
! NPCO2_2            ===> INDEX FOR THE SECOND FILE IN TIME INTERPOLATION
!                     (ONLY USED FOR LECO2DIU=.TRUE.)
! NUNITC         ===> PBOPEN UNIT FOR CLIMATE FILE
! CLIMR          ===> HOLDS THE LAST CLIMATE FILES READ (ONLY USED FOR
!                     LMCCEC=.TRUE. .AND. LMCC04=.FALSE.)
! CLIMRAER       ===> HOLDS THE LAST NBF AER FILES READ (ONLY USED FOR
!                     LEAERDIU=.TRUE.
! CLIMRCO2       ===> HOLDS THE LAST NBF CO2 FILES READ (ONLY USED FOR
!                     LECO2DIU=.TRUE.

! EC-Earth variables
! NCLIMR_SEA     ===> NUMBER OF UPDATING SST AND SEA-ICE FIELDS TO BE READ
!                     FROM ICMSEA FILE
! NCLIGC_SEA     ===> GRIB CODES OF THE ICMSEA FIELDS TO BE TIME UPDATED
! CLIMR_SEA      ===> HOLDS THE LAST SST AND SEA-ICE FIELDS
! LRDISTL1       ===> SET TRUE IF THE SEA-ICE TEMPERATURE (ISTL1) IS PROVIDED
!                     IN THE ICMSEA FILE (DEFAULT: FALSE)
! END of EC-Earth variables

! Conf 923
! LMCC12 = .T.   ===> CONF 923 CALLS INCLIB
! LMCC19 = .T.   ===> CONF 923 CALLS INCLIR

! New conf 923
! LN923  = .T.   ===> New version : call to INCLIn, n=N923 (1-11)

! NFRCPL     : COUPLING FREQUENCY for IFS/NEMO
! NFRCPL_CHE : COUPLING FREQUENCY for IFS/TM5

INTEGER(KIND=JPIM) :: N923
INTEGER(KIND=JPIM) :: NCLIMTOT
INTEGER(KIND=JPIM) :: NCLIMR
INTEGER(KIND=JPIM) :: NCLIGC(15)
INTEGER(KIND=JPIM) :: NCLIAER(15)
INTEGER(KIND=JPIM) :: NDIFC
INTEGER(KIND=JPIM) :: NJDCR
INTEGER(KIND=JPIM) :: NP1
INTEGER(KIND=JPIM) :: NP2
INTEGER(KIND=JPIM) :: NPCO2_1, NPCO2_2
INTEGER(KIND=JPIM) :: NPAER_1, NPAER_2
INTEGER(KIND=JPIM) :: NUNITC
INTEGER(KIND=JPIM) :: NOACOMM
REAL(KIND=JPRB),ALLOCATABLE:: CLIMR(:,:,:),CLIMRCO2(:,:,:),CLIMRAER(:,:,:)

INTEGER(KIND=JPIM)          :: NCLIMR_SEA, NCLIGC_SEA(3)
REAL(KIND=JPRB),ALLOCATABLE :: CLIMR_SEA(:,:,:)
LOGICAL                     :: LRDISTL1

LOGICAL :: LMCC01
LOGICAL :: LMCC02
LOGICAL :: LMCC03
LOGICAL :: LMCC04
LOGICAL :: LMCC05
LOGICAL :: LMCC06
LOGICAL :: LMCCEC
LOGICAL :: LMCCIEC
LOGICAL :: LMCCICEIC
LOGICAL :: LMCC12
LOGICAL :: LMCC19
LOGICAL :: LN923

INTEGER(KIND=JPIM) :: NFRCPL
INTEGER(KIND=JPIM) :: NFRCPL_CHE

END MODULE YOMMCC
