MODULE YOMMKODB

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE PARCMA    ,ONLY : JPMXVAR, JPMXOCT, JPMXOTP

IMPLICIT NONE

SAVE

!*     YOMMKODB - MAKE ODB RUN PARAMETERS

!        D. VASILJEVIC   ECMWF     15/9/94

!     MODIFICATIONS
!     -------------
!       23/12/2004   Niels Bormann  RARAD becomes LIMB; new code types for LIMB

!     NAME                             TYPE                  MEANING
!     ----                             ----                  -------
!     LNOEDGES                          L       NO EDGES (TIME) SWITCH
!     LZTCONS                           L       IF Z-T CONSISTENCY IS REQUIRED
!     LSPEHUM                           L       IF Q REQUIRED
!     LQERRFRH                          L       IF Q ERROR FUNCTION OF RH ERROR
!     LQERRFT                           L       IF Q ERROR FUNCTION OF T  ERROR
!     LQERRFP                           L       IF Q ERROR FUNCTION OF P  ERROR
!     LTENDCOR                          L       PRESSURE TENDENCY CORRECTION SWITCH
!     LERRPERS                          L       PERSSITENCE ERROR SWITCH
!     LBIASCV                           L       USE BIAS CORRECTED VALUE SWITCH
!     LTBIASBUFR                        L       TEMPERATURE BIAS CORRECTION TO BE DONE ON
!                                               BUFR (OTHERWISE IT IS DONE ON CMA)
!     LTBIASC                           L       USE T BIAS CORRECTED VALUE SWITCH
!     LHBIASC                           L       USE H BIAS CORRECTED VALUE SWITCH
!     LBIASCVO                          L       USE BIAS CORRECTED VALUE SWITCH
!                                               (OLD STYLE)
!     LSTOBIAS                          L       TO STORE T BIAS IN REP. ERROR WORD SWITCH
!     L_NEW_T_BIAS                      L       NEW STYLE RS T BIAS
!     L_NEW_T_BIAS_SOLAR_vs_all_mean    L       NEW STYLE RS T BIAS SOLAR PART vs all_mean
!     L_NEW_T_BIAS_SOLAR_vs_day_mean    L       NEW STYLE RS T BIAS SOLAR PART vs day_mean
!     L_NEW_T_BIAS_SOLAR_vs_night_m  L       NEW STYLE RS T BIAS SOLAR PART vs night_mean
!     L_NEW_T_BIAS_ALL_MEAN             L       NEW STYLE RS T BIAS ALL MEAN
!     L_NEW_T_BIAS_DAY_MEAN             L       NEW STYLE RS T BIAS DAY MEAN PART
!     L_NEW_T_BIAS_NIGHT_MEAN           L       NEW STYLE RS T BIAS NIGHTMEAN PART
!     L_NEW_RH_BIAS                     L       NEW STYLE RS RH BIAS
!     L_NEW_RH_BIAS_SOLAR_vs_all_mean   L       NEW STYLE RS RH BIAS SOLAR PART vs all_mean
!     L_NEW_RH_BIAS_SOLAR_vs_day_mean   L       NEW STYLE RS RH BIAS SOLAR PART vs day_mean
!     L_NEW_RH_BIAS_SOLAR_vs_night_m L       NEW STYLE RS RH BIAS SOLAR PART vs night_mean
!     L_NEW_RH_BIAS_ALL_MEAN            L       NEW STYLE RS RH BIAS ALL MEAN
!     L_NEW_RH_BIAS_DAY_MEAN            L       NEW STYLE RS RH BIAS DAY MEAN PART
!     L_NEW_RH_BIAS_NIGHT_MEAN          L       NEW STYLE RS RH BIAS NIGHT MEAN PART

!     L_T_REF_SONDE_BIAS_ADJUST         L       T REF SONDE TO ADJUST BIAS TO
!     C_T_REF_SONDE_NAME                C       T REFERANCE SONDE NAME
!     N_T_REF_SONDE_CODE                C       T REFERANCE SONDE CODE
!     L_RH_REF_SONDE_BIAS_ADJUST        L       RH REF SONDE TO ADJUST BIAS TO
!     C_RH_REF_SONDE_NAME               C       RH REFERANCE SONDE NAME
!     N_RH_REF_SONDE_CODE               C       RH REFERANCE SONDE CODE

!     LCHSHIP                           L       SWITCH TO ASSIGN REAL SHIP CHARACTERISTICS
!     LCHDRIBU                          L       SWITCH TO ASSIGN REAL DRIBU CHARACTERISTICS
!     LSHDRCHO                          L       SWITCH TO SWITCH OVER FROM SHIP TO DRIBU
!     LSHDRCHOC                         L       SWITCH TO COMPLETE SWITCH OVER FROM SHIP TO DRIBU

!     LAMRDBQC                          L       SATOB RDB QC SWITCH

!     LSCATINT                          L       SCAT. INTERFACE SWITCH
!     LSINOSOL                          L       SCAT. INTERFACE "NO SOLUTION" CASE DIAG.
!     LSCATTHI                          L       SCAT. THINNING SWITCH
!     LTOTSCTH                          L       TOTAL SCAT. THINNING
!     LTOTREM                           L       TOTAL REMOVAL OF A REPORT
!     LNOPROC                           L       NO PROCESSING OF OBS.
!     LODBDDRMOD                        L       TO MODIFY ODB DDR SWITCH
!     LODBDDRMODF                       L       FULL ODB DDR MODIFICATION
!     LODBDDRMODS                       L       SHORT ODB DDR MODIFICATION
!     LMKCMARPLRUN                      L       MAKECMA REPLACMENT RUN
!     NTHINSCA                          I       SCAT. THINNING FACTOR:
!                                               (1,2,4) = (25, 50, 100) KM RESOLUTION (moved to 
!                                               yomscc)
!     NBIASCOR                          I       TYPE OF SPEED BIAS CORRECTION ( 1 used until CY21R3,
!                                               2 from CY21R4 )
!                                               if(NBIASCOR==0) no speed bias correction for ERS/ASCAT

!     NMKCMVSE()                        I       LIST OF VARIABLES TO BE SELECTED FOR CMA

!     NOSORTSL                          I       NUMBER OF TIME SLOTS IN 4DVAR OBS. SORTING

!     L_LAND_PS_BIAS                    L       LAND SYNOP PS BIAS CORRECTION SWITCH
!     L_SHIP_PS_BIAS                    L       SHIP SYNOP PS BIAS CORRECTION SWITCH
!     L_METAR_PS_BIAS                   L       METAR SYNOP PS BIAS CORRECTION SWITCH
!     L_DRIBU_PS_BIAS                   L       DRIBU PS BIAS CORRECTION SWITCH
!     L_SCAT_BIASSIG                    L    SIGMA BIAS CORRECTION FOR NSCAT2,NSCAT3 (ERS/ASCAT)


LOGICAL :: LNOEDGES
LOGICAL :: LSURPR
LOGICAL :: LZTCONS
LOGICAL :: LSPEHUM
LOGICAL :: LQERRFRH
LOGICAL :: LQERRFT
LOGICAL :: LQERRFP
LOGICAL :: LTENDCOR
LOGICAL :: LERRPERS
LOGICAL :: LBIASCV
LOGICAL :: LTBIASBUFR
LOGICAL :: LTBIASC
LOGICAL :: LHBIASC
LOGICAL :: LBIASCVO
LOGICAL :: LSTOBIAS
LOGICAL :: L_NEW_T_BIAS
LOGICAL :: L_NEW_T_BIAS_SOLAR_vs_all_mean
LOGICAL :: L_NEW_T_BIAS_SOLAR_vs_day_mean
LOGICAL :: L_NEW_T_BIAS_SOLAR_vs_night_m
LOGICAL :: L_NEW_T_BIAS_ALL_MEAN
LOGICAL :: L_NEW_T_BIAS_DAY_MEAN
LOGICAL :: L_NEW_T_BIAS_NIGHT_MEAN
LOGICAL :: L_NEW_RH_BIAS
LOGICAL :: L_NEW_RH_BIAS_SOLAR_vs_all_mean
LOGICAL :: L_NEW_RH_BIAS_SOLAR_vs_day_mean
LOGICAL :: L_NEW_RH_BIAS_SOLAR_vs_night_m
LOGICAL :: L_NEW_RH_BIAS_ALL_MEAN
LOGICAL :: L_NEW_RH_BIAS_DAY_MEAN
LOGICAL :: L_NEW_RH_BIAS_NIGHT_MEAN
LOGICAL :: L_T_REF_SONDE_BIAS_ADJUST
CHARACTER(LEN=64)  :: C_T_REF_SONDE_NAME
INTEGER(KIND=JPIM) :: N_T_REF_SONDE_CODE
LOGICAL :: L_RH_REF_SONDE_BIAS_ADJUST
CHARACTER(LEN=64)  :: C_RH_REF_SONDE_NAME
INTEGER(KIND=JPIM) :: N_RH_REF_SONDE_CODE

LOGICAL :: LCHSHIP
LOGICAL :: LCHDRIBU
LOGICAL :: LSHDRCHO
LOGICAL :: LSHDRCHOC
LOGICAL :: LAMRDBQC
LOGICAL :: LSCATINT
LOGICAL :: LSINOSOL
LOGICAL :: LSCATTHI
LOGICAL :: LTOTSCTH
LOGICAL :: LTOTREM
LOGICAL :: LNOPROC
LOGICAL :: LODBDDRMOD
LOGICAL :: LODBDDRMODF
LOGICAL :: LODBDDRMODS
LOGICAL :: LMKCMARPLRUN
LOGICAL :: L_LAND_PS_BIAS
LOGICAL :: L_SHIP_PS_BIAS
LOGICAL :: L_METAR_PS_BIAS
LOGICAL :: L_DRIBU_PS_BIAS
LOGICAL :: L_SCAT_BIASSIG
INTEGER(KIND=JPIM) :: NMKCMVSE(JPMXVAR,JPMXOCT,JPMXOTP)
!INTEGER_M :: NTHINSCA
INTEGER(KIND=JPIM) :: NBIASCOR
INTEGER(KIND=JPIM) :: NOSORTSL

!*              RUN GLOBAL PARAMETERS

!     NAME      TYPE                  MEANING
!     ----      ----                  -------
!     L3DVAR      L       MAKE CMA 3D VAR RUN
!     LINCREM     L       3D/4D VAR INCREMENTAL RUN
!     LCANARI     L       CANARI (FRENCH OI) SWITCH

!     NANDAT      I       ANALYSIS DATE, IN FORM YYYYMMDD (NOW IN YOMANA)
!     NANTIM      I       ANALYSIS TIME, IN FORM HHMMSS (NOW IN YOMANA)
!     NANCDY      I       ANALYSIS DATE, AS CENTURY DAY
!     NANHOU      I       ANALYSIS TIME, IN HOURS FROM 00Z
!     NANMIN      I       ANALYSIS TIME, IN MINUTES FROM 00Z (NOW IN YOMANA)

!     NTBMAR      I       4D BACKWARD TIME MARGIN (IN MIN.)
!     NTFMAR      I       4D FOREWARD TIME MARGIN (IN MIN.)
!     N4DMIN      I       4D TIME LENGTH (IN MIN.)

!     NCENDA      I       CENTRAL DAY IN 4-D TIME INTERVAL RELATIVE
!                         TO THE START
!     NCENHO      I       CENTRAL HOUR IN 4D-TIME INTERVAL RELATIVE
!                         TO THE START

!     OBNLAT      R       NORTHERN BOUNDARY OF OBSERVATION AREA
!     OBSLAT      R       SOUTHERN BOUNDARY OF OBSERVATION AREA
!     OBWLON      R       WESTERN BOUNDARY OF OBSERVATION AREA
!     OBELON      R       EASTERN BOUNDARY OF OBSERVATION AREA

!     EXNLAT      R       NORTHERN BOUNDARY FOR EXCLUSION AREA
!     EXSLAT      R       SOUTHERN BOUNDARY FOR EXCLUSION AREA
!     EXWLON      R       WESTERN BOUNDARY FOR EXCLUSION AREA
!     EXELON      R       EASTERN BOUNDARY FOR EXCLUSION AREA

!       SWITCHES FOR CMA OBS./CODE TYPES TO BE CONSIDERED:
!     LSYNOP      L       SYNOP
!     LCD011      L       SYNOP LAND (CODE 11)
!     LCD014      L       SYNOP LAND (CODE 14)
!     LCD021      L       SYNOP SHIP (CODE 21)
!     LCD022      L       SYNOP SHIP (CODE 22)
!     LCD023      L       SYNOP SHIP (CODE 23)
!     LCD024      L       SYNOP SHIP (CODE 24)
!     LAIREP      L       AIRCRAFT
!     LCD041      L       AIRCRAFT (CODE 41)
!     LCD141      L       AIRCRAFT (CODE 141)
!     LCD142      L       AIRCRAFT (CODE 142)
!     LCD144      L       AIRCRAFT (CODE 144)
!     LCD145      L       AIRCRAFT (CODE 145)
!     LCD241      L       AIRCRAFT (CODE 241)
!     LSATOB      L       SATOB
!     LCD088      L       SATOB (CODE 88)
!     LCD089      L       SATOB (CODE 89)
!     LCD090      L       SATOB AMV (CODE 90)
!     LCD188      L       SATOB (CODE 188)
!     LODRIBU     L       DRIFTING BUOY
!     LCD063      L       DRIBU (CODE 63)
!     LCD064      L       DRIBU (CODE 64)
!     LCD165      L       DRIBU (CODE 165)
!     LCD160      L       DRIBU (CODE 160)
!     LTEMP       L       TEMP
!     LCD035      L       TEMP (CODE 35)
!     LCD036      L       TEMP (CODE 36)
!     LCD037      L       TEMP (CODE 37)
!     LCD135      L       TEMP (CODE 135)
!     LCD137      L       TEMP (CODE 137)
!     LCD039      L       TEMP (CODE 39)
!     LCD040      L       TEMP (CODE 40)
!     LOPILOT     L       PILOT
!     LCD032      L       PILOT (CODE 32)
!     LCD033      L       PILOT (CODE 33)
!     LCD034      L       PILOT (CODE 34)
!     LCD131      L       PILOT (CODE 131)
!     LCD134      L       PILOT (CODE 134)
!     LSATEM      L       SATEM
!     LCD086      L       SATEM (CODE 86)
!     LCD186      L       SATEM (CODE 186)
!     LCD185      L       SATEM (CODE 185)
!     LCD184      L       SATEM (CODE 184)
!     LCD206      L       SATEM (CODE 206)
!     LCD200      L       SATEM (CODE 200)
!     LCD201      L       SATEM (CODE 201)
!     LCD202      L       SATEM (CODE 202)
!     LCD210      L       SATEM (CODE 210)
!     LCD211      L       SATEM (CODE 211)
!     LCD212      L       SATEM (CODE 212)
!     LCD215      L       SATEM (CODE 215)
!     LOPAOB      L       PAOB
!     LCD180      L       PAOB (CODE 180)
!     LSCATT      L       SCATTEROMETER
!     LASCATSM    L       ASCAT Soil Moisture Obs.
!     LCD008      L       SCATT. (CODE 8)
!     LCD122      L       SCATT. (CODE 122)
!     LCD210SC    L       SCATT. (CODE 210)
!     LCD300      L       SCATT. (CODE 300 - NSCAT)
!     LCD301      L       SCATT. (CODE 301 - QSCAT)
!     LCD139      L       SCATT. (CODE 139 - ASCAT)
!     LCD302      L       SCATT. (CODE 302 - OCEANSAT2)
!     LALIMB      L       LIMB OBSERVATIONS
!     LCD250      L       RADIO OCCULTATION
!     LCD251      L       LIMB RADIANCES
!     LRADAR      L       RADAR
!     LCD192      L       RADAR reflectivity BUFR CODE 192

!INTEGER_M :: NANDAT
!INTEGER_M :: NANTIM
INTEGER(KIND=JPIM) :: NANCDY
INTEGER(KIND=JPIM) :: NANHOU
!INTEGER_M :: NANMIN
INTEGER(KIND=JPIM) :: NTBMAR
INTEGER(KIND=JPIM) :: NTFMAR
INTEGER(KIND=JPIM) :: N4DMIN

REAL(KIND=JPRB) :: OBNLATE
REAL(KIND=JPRB) :: OBSLATE
REAL(KIND=JPRB) :: OBWLONE
REAL(KIND=JPRB) :: OBELONE
REAL(KIND=JPRB) :: EXNLAT
REAL(KIND=JPRB) :: EXSLAT
REAL(KIND=JPRB) :: EXWLON
REAL(KIND=JPRB) :: EXELON

LOGICAL :: L3DVAR
!LOGICAL :: L3DFGAT
!LOGICAL LINCREM
!LOGICAL LCANARI
LOGICAL :: LSYNOP
LOGICAL :: LCD011
LOGICAL :: LCD014
LOGICAL :: LCD021
LOGICAL :: LCD022
LOGICAL :: LCD023
LOGICAL :: LCD024
LOGICAL :: LAIREP
LOGICAL :: LCD041
LOGICAL :: LCD141
LOGICAL :: LCD142
LOGICAL :: LCD144
LOGICAL :: LCD145
LOGICAL :: LCD241
LOGICAL :: LSATOB
LOGICAL :: LCD088
LOGICAL :: LCD089
LOGICAL :: LCD090
LOGICAL :: LCD188
LOGICAL :: LODRIBU
LOGICAL :: LCD063
LOGICAL :: LCD064
LOGICAL :: LCD165
LOGICAL :: LCD160
LOGICAL :: LTEMP
LOGICAL :: LCD035
LOGICAL :: LCD036
LOGICAL :: LCD037
LOGICAL :: LCD135
LOGICAL :: LCD137
LOGICAL :: LCD039
LOGICAL :: LCD040
LOGICAL :: LOPILOT
LOGICAL :: LCD032
LOGICAL :: LCD033
LOGICAL :: LCD034
LOGICAL :: LCD134
LOGICAL :: LCD131
LOGICAL :: LSATEM
LOGICAL :: LCD086
LOGICAL :: LCD186
LOGICAL :: LCD185
LOGICAL :: LCD184
LOGICAL :: LCD206
LOGICAL :: LCD200
LOGICAL :: LCD201
LOGICAL :: LCD202
LOGICAL :: LCD210
LOGICAL :: LCD211
LOGICAL :: LCD212
LOGICAL :: LCD215
LOGICAL :: LOPAOB
LOGICAL :: LCD180
LOGICAL :: LSCATT
LOGICAL :: LCD008
LOGICAL :: LCD122
LOGICAL :: LCD139
LOGICAL :: LCD302
LOGICAL :: LCD210SC
LOGICAL :: LCD300
LOGICAL :: LCD301
LOGICAL :: LALIMB
LOGICAL :: LCD250 
LOGICAL :: LCD251 
LOGICAL :: LRADAR
LOGICAL :: LCD192
LOGICAL :: LASCATSM
!-----------------------------------------------------------------------

END MODULE YOMMKODB
