MODULE YOMMODEL_ERROR

!     Purpose.
!     --------
!       Data and controls for model error in 4D-Var.

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original    18-Mar-2004
!       M.Jidane 09-04-2006 Cy31 Phasing
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
SAVE

INTEGER(KIND=JPIM) :: NTYPE_MODERR  ! Type of model error
                                    ! 0 - No model error
                                    ! 1 - Full state
                                    ! 2 - Forcing (constant or periodic)
                                    ! 3 - Bias-like error
                                    ! 4 - Multiple increments

REAL(KIND=JPRB) :: WEAK4D_INTERV    ! Time interval between components (hours)
REAL(KIND=JPRB) :: ALPHAQ           ! Coef. model error term in cost function
REAL(KIND=JPRB) :: TSTEP_ERR        ! Forcing time-step
LOGICAL :: LBGMODERR                ! Compute Jq from background (not zero)
LOGICAL :: LFGMODERR                ! Use first guess for model error
LOGICAL :: LSCALERR                 ! Scale model error for I/O

INTEGER(KIND=JPIM) :: NCOMP_MODERR  ! Number of components in model error
INTEGER(KIND=JPIM) :: NSTEP_MODERR  ! Number of intervals  in model error
INTEGER(KIND=JPIM) :: NDIM_MODERR   ! Total number of components in model error
INTEGER(KIND=JPIM) :: NORDER_MODERR ! Order of model error forcing

INTEGER(KIND=JPIM) :: NSERR         ! Spectral truncation for model error
INTEGER(KIND=JPIM) :: NVE3D         ! Number of model error 3D fields
INTEGER(KIND=JPIM) :: NVE2D         ! Number of model error 2D fields
INTEGER(KIND=JPIM) :: NVE3SP        ! Number of model error 3D SP fields
INTEGER(KIND=JPIM) :: NVE2SP        ! Number of model error 2D SP fields
INTEGER(KIND=JPIM) :: NVE3GP        ! Number of model error 3D GP fields
INTEGER(KIND=JPIM) :: NVE2GP        ! Number of model error 2D GP fields

INTEGER(KIND=JPIM) :: NPERIOD_4DVAR ! 4DVAR frequency in hours (from PERIOD_4D)

INTEGER(KIND=JPIM) :: NPRTMODERR    ! Printing level for diagnostics

TYPE(GRIDPOINT_FIELD), ALLOCATABLE :: GPMODERR(:)   ! Gridpoint model error
TYPE(GRIDPOINT_FIELD), ALLOCATABLE :: GPFGMODERR(:) ! Grid point ME first guess
TYPE(GRIDPOINT_FIELD), ALLOCATABLE :: GPMODERR_HYB(:)   ! Gridpoint model error
TYPE(SPECTRAL_FIELD), ALLOCATABLE  :: SPMODERR(:)   ! Spectral model error
TYPE(SPECTRAL_FIELD), ALLOCATABLE  :: SPFGMODERR(:) ! Spectral   ME first guess
TYPE(SPECTRAL_FIELD), ALLOCATABLE  :: SPMODERR_HYB(:)   ! Spectral model error aux
TYPE(SPECTRAL_FIELD)               :: SPCTLMODERR   ! Spectral model error CV
TYPE(SPECTRAL_FIELD)               :: SPGPMODERR    ! Spectral ME for GP trans

INTEGER(KIND=JPIM), ALLOCATABLE :: NGRBMERRCTL(:)   ! Grib code model error
INTEGER(KIND=JPIM), ALLOCATABLE :: NGRBMERRSP(:)    ! Grib spectral model error
INTEGER(KIND=JPIM), ALLOCATABLE :: NGRBMERRGP(:)    ! Grib gridpoint model error
INTEGER(KIND=JPIM), ALLOCATABLE :: MERRGFL(:)       ! Index model error in GFL
INTEGER(KIND=JPIM), ALLOCATABLE :: MERRGMV(:)       ! Index model error in GMV
INTEGER(KIND=JPIM), ALLOCATABLE :: MERRGMVS(:)      ! Index model error in GMVS
INTEGER(KIND=JPIM), ALLOCATABLE :: MERROBS(:)       ! Index model error in COBS
INTEGER(KIND=JPIM), ALLOCATABLE :: MERRSP2(:)       ! Index model error in SPA2
INTEGER(KIND=JPIM), ALLOCATABLE :: MERRSP3(:)       ! Index model error in SPA3

INTEGER(KIND=JPIM), ALLOCATABLE :: MSTEPERR(:)      ! Index model error steps
REAL(KIND=JPRB), ALLOCATABLE    :: FSTEPERR(:,:)    ! Coef. periodic forcing
LOGICAL, ALLOCATABLE :: LOVERLAPPING4D(:)           ! Overlapping steps
LOGICAL              :: L_WARM_START_MODERR         ! Not first cycle 
INTEGER(KIND=JPIM) :: N_COUPLED_WINDOWS             ! Number of coupled (sub)windows

REAL(KIND=JPRB) :: ALPHAQ_T         ! Coef. temperature model error term
REAL(KIND=JPRB) :: ALPHAQ_D         ! Coef. divergence  model error term
REAL(KIND=JPRB) :: ALPHAQ_V         ! Coef. vorticity   model error term
REAL(KIND=JPRB) :: ALPHAQ_Q         ! Coef. humidity    model error term
REAL(KIND=JPRB) :: ALPHAQ_O         ! Coef. ozone       model error term

END MODULE YOMMODEL_ERROR
