module yommwave
      
use parkind1, only: jpim, jprb
use parmwave, only: nmwave_diag, nmwave_p, nmwave_gnrm, nmax_chan

implicit none

save

real    (kind=jprb) :: rlimit_sd   ! obs-grid search distance limit  [m]
real    (kind=jprb) :: rlimit_lat  ! upper sin(latitude) limit       []
real    (kind=jprb) :: rlimit_ts   ! lower snow water path limit     [kg/m^2]
real    (kind=jprb) :: rlimit_tp   ! lower total precip path limit   [kg/m^2]

real    (kind=jprb) :: rlimit_obs_err ! max obs. err. for which to compute gradients [K]

real (kind=jprb) :: rsplit_obs_err ! proportion of error model allocated to obs or fg []

logical             :: lemwave_active
logical             :: ldmwave_test
logical             :: ldmwave_sens
logical             :: ldmwave_land


integer (kind=jpim), allocatable :: ob_from    (:,:,:)
integer (kind=jpim), allocatable :: ob_jobs    (:,:,:)
integer (kind=jpim), allocatable :: ob_jrof_gp (:,:,:)
integer (kind=jpim), allocatable :: ob_index   (:,:)
integer (kind=jpim), allocatable :: gp_count   (:,:)
integer (kind=jpim), allocatable :: ob_count   (:)

integer (kind=jpim), allocatable :: mwave_step(:,:,:)
integer (kind=jpim), allocatable :: mwave_buf_step(:,:,:)
real    (kind=jprb), allocatable :: mwave_oerr_step (:,:,:,:)
real    (kind=jprb), allocatable :: mwave_zen_buf_step (:,:,:)
integer (kind=jpim) :: mwave_max_idim_ob = 0
integer (kind=jpim) :: mwave_max_chan = 0

integer (kind=jpim) :: n_satsens = 0            ! Number of satellites being processed
integer (kind=jpim) :: ichan_sens               ! Number of satellites being processed

type mwave_phys_type
  integer (kind=jpim) :: nlevels                ! number of atmospheric levels

  real    (kind=jprb), pointer :: p   (:)       ! full-level model pressure (hPa)
  real    (kind=jprb), pointer :: ph  (:)       ! half-level model pressure (hPa)
  real    (kind=jprb), pointer :: t   (:)       ! temperature (K)
  real    (kind=jprb), pointer :: q   (:)       ! specific humidity (kg/kg)
  real    (kind=jprb), pointer :: rfl (:)      ! stratiform rain flux (kg/(m2))
  real    (kind=jprb), pointer :: sfl (:)      ! stratiform snow flux (kg/(m2))
  real    (kind=jprb), pointer :: crfl (:)      ! convective rain flux (kg/(m2))
  real    (kind=jprb), pointer :: csfl (:)      ! convective snow flux (kg/(m2))
  real    (kind=jprb), pointer :: cwc (:)       ! cloud liquid water mixing ratio (kg/kg)
  real    (kind=jprb), pointer :: cic (:)       ! ice water mixing ratio (kg/kg)
  real    (kind=jprb), pointer :: cc  (:)       ! cloud cover (0-1)
  real    (kind=jprb), pointer :: pfrc (:)      ! stratiform precip fraction (0-1)

  real    (kind=jprb) :: u10                    ! 10m wind speed, u (m/s)
  real    (kind=jprb) :: v10                    ! 10m wind speed, v (m/s)
  real    (kind=jprb) :: t2m                    ! 2m temperature (K)
  real    (kind=jprb) :: q2m                    ! 2m specific humidity (kg/kg)
  real    (kind=jprb) :: tsk                    ! skin temperature (K)
  real    (kind=jprb) :: lsm                    ! land-sea mask ()
  real    (kind=jprb) :: sim                    ! sea-ice mask ()
  real    (kind=jprb) :: slat                   ! sin (latitude) ()
  real    (kind=jprb) :: slon                   ! sin (longitude) ()
  real    (kind=jprb) :: clon                   ! cos (longitude) ()
  real    (kind=jprb) :: orog                   ! surface elevation (m)
end type mwave_phys_type  

type mwave_rad_type
  integer (kind=jpim) :: screen (nmax_chan)     ! screening flag

  real    (kind=jprb) :: diag (nmwave_diag)     ! diagnostic output     
  real    (kind=jprb) :: p19  (nmwave_p)        ! normalised polarisation difference (19 GHz) 
  real    (kind=jprb) :: p37  (nmwave_p)        ! normalised polarisation difference (37 GHz)    
  real    (kind=jprb) :: grad_norm (nmwave_gnrm)! sensitivity from gradient norm

  real    (kind=jprb) :: zenith                 ! satellite zenith angle (degrees)
  real    (kind=jprb) :: azimuth                ! satellite azimuth angle (degrees,clockwise from N)

  real    (kind=jprb) :: obs  (nmax_chan)       ! observed radiances (K)
  real    (kind=jprb) :: fw   (nmax_chan)       ! modelled radiances, forward (K)
  real    (kind=jprb) :: tl   (nmax_chan)       ! modelled radiances, perturbation (K/?)
  real    (kind=jprb) :: ad   (nmax_chan)       ! modelled radiances, gradient (?)
  real    (kind=jprb) :: oerr (nmax_chan)       ! observation error (K)

! add entrees for emissivity

  real    (kind=jprb) :: eesat    (nmax_chan)              ! emissivity from satellite
  real    (kind=jprb) :: eemod    (nmax_chan)              ! emissivity from model
  real    (kind=jprb) :: trans    (nmax_chan)              ! atmospheric transmission 
  real    (kind=jprb) :: eesurf   (nmax_chan)              ! atmospheric transmission 
  real    (kind=jprb) :: pers1   (nmax_chan)              ! emissivity used in fw model 
  real    (kind=jprb) :: pers2   (nmax_chan)              ! emissivity used in fw model (for test)


end type mwave_rad_type  

type mwave_ids
  integer (kind=jpim) :: bufr_id   ! satellite buffer ID e.g., 241=F08, 249=SSMIS, etc.
  integer (kind=jpim) :: satseries ! satellite series number.  e.g., NOAA=1, DMSP=2, etc.
  integer (kind=jpim) :: satid     ! satellite platform ID.  e.g. NOAA-17=17, F-15=15.
  integer (kind=jpim) :: sensid    ! Sensor ID.  e.g., SSMI=6, TMI=9, AMSUA=3.
  integer (kind=jpim) :: nchannels ! No of channels on sensor.  E.g., SSMI=7, AMSUB=5.
  real    (kind=jprb) :: zenith    ! Fixed or default zenith angle 
  integer (kind=jpim) :: nchan_sens ! channel for sensitivity test
  integer (kind=jpim) :: i19v, i19h, i37v, i37h  ! channel indices for 19v/h and 37v/h
  real    (kind=jprb) :: rtot_err_clr(nmax_chan) ! Total error by channel (clear sky)  [K]
  real    (kind=jprb) :: rtot_err_cld(nmax_chan) ! Total error by channel (cloud/rain) [K]
  real    (kind=jprb) :: rfg_err_clr (nmax_chan) ! FG error estimate in 'clear'  skies [K]
  real    (kind=jprb) :: rp37_clr    (nmax_chan) ! Definition of `clear'  in P37 [0-1]
  real    (kind=jprb) :: rp37_cld    (nmax_chan) ! Definition of `cloudy' in P37 [0-1]
  real    (kind=jprb) :: rlimit_fgc  (nmax_chan) ! FG departure limit [sigmas]
  real    (kind=jprb) :: rcwn        (nmax_chan) ! Central wavenmuber of channel [cm-1]
end type mwave_ids

type (mwave_ids), allocatable :: mwave_id (:)  ! Sensor Information Array

! Status code bitfield.  
integer (kind=jpim), parameter :: &
 & mwave_active        = 0 , & ! Observation active if only this bit set
 & mwave_obs_tb        = 1 , & ! Observed TB out of bounds
 & mwave_negative_q    = 2 , & ! Negative Q in model profile
 & mwave_tsfc          = 3 , & ! Surface T < 274K in model
 & mwave_land          = 4 , & ! Land in model
 & mwave_sea_ice       = 5 , & ! Sea ice in model
 & mwave_latitude      = 6 , & ! Latitude outside bounds
 & mwave_no_obs        = 7 , & ! No obs for this grid point (grid point space only)
 & mwave_fg_depar      = 8 , & ! FG depar outside limit
 & mwave_contaminated  = 9 , & ! Obs rejected because neighbour on low-res grid rejected
 & mwave_passive       = 10, & ! Passive monitoring (== fail(EXPERIMENTAL))
 & mwave_missing       = 11, & ! Whole obs rejected if any 'important' channels are bad
 & mwave_no_grid       = 12, & ! No grid point for this obs
 & mwave_cold_sector   = 13, & ! Winter cold sectors: model has too little water cloud
 & mwave_excess_snow   = 14, & ! RTTOV-SCATT is way too responsive to snow at higher freq.
 & mwave_missing_tb    = 15, & ! Observation value is missing (=RMDI)
 & mwave_ssmis_hack    = 16    ! We cannot run RTTOV-SCATT for SSMIS chs. 20-24

! The following list of "non-serious errors" are ones for which we would
! still like to calculate FG and analysis departures, for monitoring purposes, plus
! "active" and "passive" which are allowed in the TL and AD too.
integer (kind=jpim), parameter :: notserious = &
 &   ibset(0,mwave_active     ) + ibset(0,mwave_passive    ) &
 & + ibset(0,mwave_cold_sector) + ibset(0,mwave_excess_snow) &
 & + ibset(0,mwave_contaminated) 

! The following list contains statuses for which no departures should be
! calculated, but which should not cause rejection of the whole
! observation
integer (kind=jpim), parameter :: norejectobs = &
 &   ibset(0,mwave_missing_tb ) + ibset(0,mwave_ssmis_hack ) &
 & + ibset(0,mwave_no_obs     )

! Cloud and rain tests bitfield. The record is partitioned as follows:

! Bits | 32  ...         16 | 15  ...          0 |
!      |     analysis       |     first guess    |

! Each individual record (analysis or first guess) looks like this:

! Bits |  >3       | 3 to 2                    | 1 to 0                        |
!      |  spare    | rain test (37v-37h < 40K) | cloud test (LWP > 0.01 kgm-2) |

! And each two-bit test field looks like this:

!  Model    Observation   Binary    Decimal    Meaning in contingency table

!  True       True          11         3       Hit
!  True       False         10         2       False alarm
!  False      True          01         1       Miss
!  False      False         00         0       Correct negative

! Note that the observational information is the same for both analysis and FG.

integer (kind=jpim), parameter :: cldbit_shift = 16 
integer (kind=jpim), parameter :: cldbit_cloud = 0 
integer (kind=jpim), parameter :: cldbit_rain  = 2
 
! ---------------------------------------------------------------------

contains
 
! ---------------------------------------------------------------------

function mwave_ok (kstatus) 

! Test status flag to see if observation is OK to process.
! Function works for scalar
     
integer (kind=jpim), intent (in) :: kstatus
logical                          :: mwave_ok

mwave_ok = kstatus == ibset(0,mwave_active)  &
   &  .or. kstatus == ibset(ibset(0,mwave_active),mwave_passive) 

end function mwave_ok 

! ---------------------------------------------------------------------

function mwave_ok_a1 (kstatus) 

! Test status flag to see if any observation in a 1D array
! is OK to process.
       
integer (kind=jpim), dimension(:), intent (in) :: kstatus
logical              :: mwave_ok_a1

mwave_ok_a1 = any( &
   &       kstatus == ibset(0,mwave_active)  &
   &  .or. kstatus == ibset(ibset(0,mwave_active),mwave_passive))

end function mwave_ok_a1 

! ---------------------------------------------------------------------

! NB - F95 allows an "elemental" function which would replace
! the pairs above with a single one able to handle scalars and
! all array shapes/sizes

! ---------------------------------------------------------------------

function mwave_serious (kstatus) 

! Test status flag to see if observation has "serious" errors. 
! No FG departure should be computed in these cases.
     
integer (kind=jpim), intent (in) :: kstatus
logical                          :: mwave_serious

integer (kind=jpim) :: iserious_mask

iserious_mask  = not( notserious )
mwave_serious = iand(kstatus, iserious_mask) /= 0 
 
end function mwave_serious 

! ---------------------------------------------------------------------

function mwave_rejectobs (kstatus) 

! Test status flag to see if observation has errors in one channel
! that should cause rejection of the whole observation.
     
integer (kind=jpim), intent (in) :: kstatus
logical                          :: mwave_rejectobs

integer (kind=jpim) :: irejectobs_mask

irejectobs_mask  = iand(not(notserious), not(norejectobs)) 
mwave_rejectobs = iand(kstatus, irejectobs_mask) /= 0 
 
end function mwave_rejectobs 

! ---------------------------------------------------------------------

function mwave_ok_a3 (klen, yd_mwave_rad) 

! Test status flag to see if any observation is OK to process.
! Function works for 3 dimensional "array" created by checking
! the screening flag for the whole yd_mwave_rad structure

integer (kind=jpim)   , intent (in) :: klen
type (mwave_rad_type) , intent (in) :: yd_mwave_rad(klen,n_satsens)       
logical              :: mwave_ok_a3

integer (kind=jpim) :: jsat, ichan

mwave_ok_a3 = .false.

do jsat = 1, n_satsens
  do ichan = 1, mwave_id(jsat) % nchannels
    mwave_ok_a3 = mwave_ok_a3 .or. mwave_ok_a1(yd_mwave_rad(:,jsat) % screen(ichan))
  enddo
enddo

end function mwave_ok_a3

! ---------------------------------------------------------------------

function mwave_error_model (pp37, ksat, kch, ldobs) 

! This function returns an estimate of the background or observation error, 
! in observation space.
!
! Our starting point is what we can estimate easily: the total error, i.e
! (obs - FG) error. To describe how this varies with changing cloud and
! rain amounts, we parametrise this as a function of 37GHz polarisation
! difference (an average of FG and obs, to prevent sampling bias). 
! The clear sky error is our baseline, and we arbritrarily assign (usually) 
! 1K of this to the background and the rest to the observations. On top of 
! this, the cloudy error is allocated proportionally either to obs or 
! background.

real(kind=jprb),    intent(in) :: pp37  ! Mean 37GHz pol. diff. (cloud/clr indicator)
integer(kind=jpim), intent(in) :: ksat  ! Satellite number in mwave_id table
integer(kind=jpim), intent(in) :: kch   ! Channel
logical,            intent(in) :: ldobs  ! .true. = return obs error ; .false. = fg error
real(kind=jprb)                :: mwave_error_model ! Error [K]

real(kind=jprb) :: zclr_tot, zcld_tot, zclr_p37, zcld_p37
real(kind=jprb) :: zremains, ztot_err
real(kind=jprb) :: zclr_obs, zclr_fg,  zcld_obs, zcld_fg

! Total error, as a function of P37, based on our model.
zclr_tot = mwave_id (ksat) % rtot_err_clr(kch)
zcld_tot = mwave_id (ksat) % rtot_err_cld(kch) 
zclr_p37 = mwave_id (ksat) % rp37_clr(kch)
zcld_p37 = mwave_id (ksat) % rp37_cld(kch)
ztot_err = ( (zclr_p37 - pp37) * zcld_tot + (pp37 - zcld_p37) * zclr_tot ) &
         & / (zclr_p37 - zcld_p37)
ztot_err = min(ztot_err, zcld_tot)
ztot_err = max(ztot_err, zclr_tot)

! Clear sky FG & obs errors
zclr_fg  = mwave_id (ksat) % rfg_err_clr(kch)
zclr_obs = sqrt( max(mwave_id (ksat) % rtot_err_clr(kch)**2 - zclr_fg**2, 0.0_jprb))

! Additional contribtion from cloudy sky FG & obs errors is split
! between FG and obs 
zremains = max( ztot_err**2 - zclr_fg**2 - zclr_obs**2, 0.0_jprb )
zcld_fg  = sqrt( (1.0_jprb - rsplit_obs_err) * zremains )
zcld_obs = sqrt(             rsplit_obs_err  * zremains )

! Current FG & obs errors
zcld_fg  = sqrt(  zclr_fg**2 +  zcld_fg**2 )
zcld_obs = sqrt( zclr_obs**2 + zcld_obs**2 )

! Return calculated error to the user
if (ldobs) then
  mwave_error_model = zcld_obs
else
  mwave_error_model = zcld_fg
endif

end function mwave_error_model 

! ---------------------------------------------------------------------

end module yommwave
