MODULE YOMNMCOD

USE PARKIND1  ,ONLY : JPIM

USE PARCMA, ONLY : JPMXPCD, JPMXSBI1, JPMXSBI2, JPMXSMI2, JPMXSMV, JPMXSMW, &
  & JPMXSMX, JPMXSMY, JPMXSMA, JPMXSMB, JPMXSMC


IMPLICIT NONE

SAVE

!*     YOMNMCOD - VARIOUS CMA CODES (NUMERICAL VERSION)

!        D. VASILJEVIC   ECMWF     15/09/94

!     SYNOP PRESSURE CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NMXPPCD       I     MAX. NO. OF PRESSURE CODES
!     NPRESCD( 1)   I     SEA LEVEL PRESSURE
!     NPRESCD( 2)   I     STATION LEVEL PRESSURE
!     NPRESCD( 3)   I     850MB GEOPOTENTIAL
!     NPRESCD( 4)   I     700MB GEOPOTENTIAL
!     NPRESCD( 5)   I     500GPM PRESSURE
!     NPRESCD( 6)   I     1000GPM PRESSURE
!     NPRESCD( 7)   I     2000GPM PRESSURE
!     NPRESCD( 8)   I     3000GPM PRESSURE
!     NPRESCD( 9)   I     4000GPM PRESSURE
!     NPRESCD(10)   I     900MB GEOPOTENTIAL
!     NPRESCD(11)   I     1000MB GEOPOTENTIAL
!     NPRESCD(12)   I     500MB GEOPOTENTIAL
!     NPRESCD(13)   I     925MB GEOPOTENTIAL

!     TEMP/PILOT LEVEL ID.:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NMXLID        I     MAX. NO. OF LEVEL ID. CODES
!     VERTICAL COORDINATE CODES

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NPRESVC       I     PRESSURE VERTICAL COORDINATE
!     NHEIGVC       I     HEIGHT VERTICAL COORDINATE
!     NTOVCVC       I     TOVS CHANNEL
!     NSCACVC       I     SCATT. CHANNEL

!     SATOB I1 AND I2I2 CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NMXSBI1       I     MAX. NO. OF I1 CODES
!     NSBI1  ( 1)   I     EUROPE
!     NSBI1  ( 2)   I     JAPAN
!     NSBI1  ( 3)   I     USA
!     NSBI1  ( 4)   I     USSR
!     NSBI1  ( 5)   I     INDIA
!     NMXSBI2       I     MAX. NO. OF I2I2 CODES
!     NSBI2I2( 1)   I     METEOSAT
!     NSBI2I2( 2)   I     PRETORIA
!     NSBI2I2( 3)   I     GOES
!     NSBI2I2( 4)   I     JAPAN
!     NSBI2I2( 5)   I     INDIA

!     PILOT A4 CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!                   I     A4

!     SATEM I1, I2I2, I4 AND I3 CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!                   I     I1
!     NMXSMI2       I     MAX. NO. OF I2I2 CODES
!     NSMI2I2( 1)   I     NOAA- 9
!     NSMI2I2( 2)   I     NOAA-10
!     NSMI2I2( 3)   I     NOAA-11
!     NSMI2I2( 4)   I     NOAA-12
!                   I     I4
!                   I     I3

!     TOVS (PROFILES) V, W, X, Y, Z, A, B AND C CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NMXSMVV       I     MAX. NO. OF V CODES
!     NSMVV  ( 1)   I     NO RETREIVAL
!     NSMVV  ( 2)   I     HIRS+MSU
!     NSMVV  ( 3)   I     HIRS
!     NMXSMWW       I     MAX. NO. OF W CODES
!     NSMWW  ( 1)   I     NO RETRIEVAL
!     NSMWW  ( 2)   I     HIRS+MSU
!     NSMWW  ( 3)   I     HIRS
!     NMXSMXX       I     MAX. NO. OF X CODES
!     NSMXX  ( 1)   I     NO RETRIEVAL
!     NSMXX  ( 2)   I     HIRS(1,2,3,8,9,16,17) + MSU(4)
!     NSMXX  ( 3)   I     HIRS(1,2,3,8,9,16,17)
!     NSMXX  ( 4)   I     HIRS(1,2,3,9,17) + MSU(4)
!     NSMXX  ( 5)   I     HIRS(1,2,3,9,17)
!     NMXSMYY       I     MAX. NO. OF Y CODES
!     NSMYY  ( 1)   I     NO RETRIEVAL
!     NSMYY  ( 2)   I     HIRS+SSU+MSU(3,4)
!     NSMYY  ( 3)   I     HIRS+MSU(3,4)
!     NSMYY  ( 4)   I     SSU+MSU(3,4)
!     NSMYY  ( 5)   I     HIRS+SSU
!     NSMYY  ( 6)   I     HIRS
!     NSMYY  ( 7)   I     MSU(3,4)
!     NMXSMAA       I     MAX. NO. OF A CODES
!     NSMAA  ( 1)   I     NO HIRS/2 DATA
!     NSMAA  ( 2)   I     CLEAR RAD. DERIVED FROM CLEAR SPOTS
!     NSMAA  ( 3)   I     CLEAR RAD. DERIVED FROM N* METHOD
!     NMXSMBB       I     MAX. NO. OF B CODES
!     NSMBB  ( 1)   I     NO HIRS/2 DATA
!     NSMBB  ( 2)   I     ALL HIRS/2 CHANNEL ARE USED
!     NSMBB  ( 3)   I     TROPOSPHERIC HIRS/2 CHANNELS WERE UNUSABLE
!                         DUE TO CLOUDS AND ONLY STRATOSPHERIC
!                         CHANNEL WERE USED
!     NMXSMCC       I     MAX NO. OF C CODES
!     NSMCC  ( 1)   I     STATISTICAL RETRIEVAL
!     NSMCC  ( 2)   I     MINIMUM INFORMATION RETRIEVAL
!     NSMCC  ( 3)   I     MINIMUM INFORMATION RETRIEVAL ATTEMPTED BUT
!                         STATISTICAL RETRIEVAL USED

!     RETRIEVAL TYPE CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------

!     GEOGRAPHICAL AREA TYPE CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------

!     INSTRUMENT TYPE CODES:

!     NAME        TYPE                   MEANING
!     ----        ----                   -------

!     CMA DDR INTEGR/REAL/CHARCTER SECTIONS CODES

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NINTESC      I       INTEGER SECTION CODE  
!     NREALSC      I       REAL SECTION CODE  
!     NCHARSC      I       CHARACTER SECTION CODE  

INTEGER(KIND=JPIM) :: NMXPPCD
INTEGER(KIND=JPIM) :: NPRESCD(JPMXPCD)
INTEGER(KIND=JPIM) :: NMXLID
INTEGER(KIND=JPIM) :: NPRESVC
INTEGER(KIND=JPIM) :: NHEIGVC
INTEGER(KIND=JPIM) :: NTOVCVC
INTEGER(KIND=JPIM) :: NSCACVC
INTEGER(KIND=JPIM) :: NMXSBI1
INTEGER(KIND=JPIM) :: NSBI1(JPMXSBI1)
INTEGER(KIND=JPIM) :: NMXSBI2
INTEGER(KIND=JPIM) :: NSBI2I2(JPMXSBI2)
INTEGER(KIND=JPIM) :: NMXSMI2
INTEGER(KIND=JPIM) :: NSMI2I2(JPMXSMI2)
INTEGER(KIND=JPIM) :: NMXSMVV
INTEGER(KIND=JPIM) :: NSMVV(JPMXSMV)
INTEGER(KIND=JPIM) :: NMXSMWW
INTEGER(KIND=JPIM) :: NSMWW(JPMXSMW)
INTEGER(KIND=JPIM) :: NMXSMXX
INTEGER(KIND=JPIM) :: NSMXX(JPMXSMX)
INTEGER(KIND=JPIM) :: NMXSMYY
INTEGER(KIND=JPIM) :: NSMYY(JPMXSMY)
INTEGER(KIND=JPIM) :: NMXSMAA
INTEGER(KIND=JPIM) :: NSMAA(JPMXSMA)
INTEGER(KIND=JPIM) :: NMXSMBB
INTEGER(KIND=JPIM) :: NSMBB(JPMXSMB)
INTEGER(KIND=JPIM) :: NMXSMCC
INTEGER(KIND=JPIM) :: NSMCC(JPMXSMC)
INTEGER(KIND=JPIM) :: NINTESC
INTEGER(KIND=JPIM) :: NREALSC
INTEGER(KIND=JPIM) :: NCHARSC

!-----------------------------------------------------------------------

END MODULE YOMNMCOD
