MODULE YOMNMEV

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE PARCMA,  ONLY : JPMXRE1, JPMXRE2, JPMXOTP, JPMXDE1, JPMXDE2, &
  & JPMXDE1, JPMXRBLE, JPMXDBLE

IMPLICIT NONE

SAVE

!*     YOMNMEV - CMA EVENTS (NUMERICAL VERSION)

!        D. VASILJEVIC   ECMWF     15/09/94

!     REPORT EVENTS PART 1:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NMXREV1           I     MAX. NO. OF EVENTS
!     NOREV1            I     NO. OF DEFINED EVENTS
!     NR1EVENT( 1)      I     NO DATA IN THE REPORT
!     NR1EVENT( 2)      I     ALL DATA REJECTED
!     NR1EVENT( 3)      I     BAD REPORTING PRACTICE
!     NR1EVENT( 4)      I     REJECTED DUE TO RDB FLAG
!     NR1EVENT( 5)      I     ACTIVATED DUE TO RDB FLAG
!     NR1EVENT( 6)      I     ACTIVATED BY WHITELIST
!     NR1EVENT( 7)      I     HORIZONTAL POSITION OUT OF RANGE
!     NR1EVENT( 8)      I     VERTICAL POSITION OUT OF RANGE
!     NR1EVENT( 9)      I     TIME OUT OF RANGE
!     NR1EVENT(10)      I     REDUNDANT REPORT
!     NR1EVENT(11)      I     REPORT OVER LAND
!     NR1EVENT(12)      I     REPORT OVER SEA
!     NR1EVENT(13)      I     MISSING STATION ALTITUDE
!     NR1EVENT(14)      I     MODEL SURF. TOO FAR FROM STAT. ALT.
!     NR1EVENT(15)      I     REPORT REJECTED THROUGH THE NAMELIST
!     NR1EVENT(16)      I     FAILED QUALITY CONTROL

!     REPORT EVENTS PART 2:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NMXREV2           I     MAX. NO. OF EVENTS

!     SYNOP EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 1)      I     NO. OF DEFINED EVENTS

!     AIREP EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 2)      I     NO. OF DEFINED EVENTS

!     SATOB EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 3)      I     NO. OF DEFINED EVENTS

!     DRIBU EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 4)      I     NO. OF DEFINED EVENTS

!     TEMP EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 5)      I     NO. OF DEFINED EVENTS
!     NR2EVENT( 1, 5)   I     OLD STYLE Z BIAS CORRECTION APPLIED
!     NR2EVENT( 2, 5)   I     NEW STYLE T BIAS CORRECTION APPLIED
!     NR2EVENT( 3, 5)   I     H BIAS CORRECTION APPLIED

!     PILOT EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 6)      I     NO. OF DEFINED EVENTS

!     SATEM EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 7)      I     NO. OF DEFINED EVENTS

!     PAOB EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 8)      I     NO. OF DEFINED EVENTS

!     SCATT. EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  ( 9)      I     NO. OF DEFINED EVENTS
!     NR2EVENT( 1, 9)   I     THINNED REPORT
!     NR2EVENT( 2, 9)   I     REPORTED WIND DIRECTIONS ARE TOO CLOSE TOGETHER
!     NR2EVENT( 3, 9)   I     REPORT NOT IN QuikSCAT/OCEANSAT2 SWEET SPOTS
!     NR2EVENT( 4, 9)   I     REPORT RAIN CONTAMINATED

!     LIMB EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NOREV2  (10)      I     NO. OF DEFINED EVENTS

!     DATA EVENTS PART 1:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NMXDEV1           I     MAX. NO. OF EVENTS
!     NODEV1            I     NO. OF DEFINED EVENTS
!     ND1EVENT( 1)      I     MISSING VERTICAL COORDINATE
!     ND1EVENT( 2)      I     MISSING OBSERVED VALUE
!     ND1EVENT( 3)      I     MISSING FIRST GUESS VALUE
!     ND1EVENT( 4)      I     REJECTED DUE TO RDB FLAG
!     ND1EVENT( 5)      I     ACTIVATED DUE TO RDB FLAG
!     ND1EVENT( 6)      I     ACTIVATED BY WHITELIST
!     ND1EVENT( 7)      I     BAD REPORTING PRACTICE
!     ND1EVENT( 8)      I     VERTICAL POSITION OUT OF RANGE
!     ND1EVENT( 9)      I     RFERENCE LEVEL POSITION OUT OF RANGE
!     ND1EVENT(10)      I     TOO BIG FIRST GUESS DEPARTURE
!     ND1EVENT(11)      I     TOO BIG DEPARTURE IN ASSIMILATION
!     ND1EVENT(12)      I     TOO BIG OBSERVATION ERROR
!     ND1EVENT(13)      I     REDUNDANT DATUM
!     ND1EVENT(14)      I     REDUNDANT LEVEL
!     ND1EVENT(15)      I     REPORT OVER LAND
!     ND1EVENT(16)      I     REPORT OVER SEA
!     ND1EVENT(17)      I     NOT AN ANALYSIS VARIABLE
!     ND1EVENT(18)      I     DUPLICATED DATUM/LEVEL
!     ND1EVENT(19)      I     TOO MANY SURFACE DATA/LEVELS
!     ND1EVENT(20)      I     MULTI LEVEL CHECK
!     ND1EVENT(21)      I     LEVEL SELECTION
!     ND1EVENT(22)      I     VERTICAL CONSISTENCY CHECK
!     ND1EVENT(23)      I     VERTICAL COORDINATE CHANGED FEOM Z TO P
!     ND1EVENT(24)      I     DATUM REJECTED THROUGH THE NAMELIST
!     ND1EVENT(25)      I     COMBINED FLAGGING
!     ND1EVENT(26)      I     DATUM REJECTED DUE TO REJECTED REPORT
!     ND1EVENT(27)      I     VARIATIONAL QC PERFORMED

!     DATA EVENTS PART 2:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NMXDEV2           I     MAX. NO. OF EVENTS

!     SYNOP EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 1)      I     NO. OF DEFINED EVENTS

!     AIREP EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 2)      I     NO. OF DEFINED EVENTS

!     SATOB EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 3)      I     NO. OF DEFINED EVENTS

!     DRIBU EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 4)      I     NO. OF DEFINED EVENTS

!     TEMP EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 5)      I     NO. OF DEFINED EVENTS
!     ND2EVENT( 1, 5)   I     BIAS CORRECTED VALUE USED

!     PILOT EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 6)      I     NO. OF DEFINED EVENTS

!     SATEM EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 7)      I     NO. OF DEFINED EVENTS
!     ND2EVENT( 1, 7)   I     NOT PREDEFINED LAYER
!     ND2EVENT( 2, 7)   I     LAYER FORMED BY THINNING UP
!     ND2EVENT( 3, 7)   I     LAYER FORMED BY SUMMING UP
!     ND2EVENT( 4, 7)   I     CHANNEL NOT USED IN ANALYSIS
!     ND2EVENT( 5, 7)   I     OVERWRITTEN BY ADVAR

!     PAOB EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 8)      I     NO. OF DEFINED EVENTS

!     SCATT EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  ( 9)      I     NO. OF DEFINED EVENTS

!     LIMB EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NODEV2  (10)      I     NO. OF DEFINED EVENTS

!     REPORT BLACKLIST EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NMXRBLE           I     MAX. NO. OF REPORT BLACKLIST EVENTS
!     NORBLE            I     NO. OF DEFINED REPORT BLACKLIST EVENTS
!     NRBLEVEN( 1)      I     MONTHLY MONITORING
!     NRBLEVEN( 2)      I     CONSTANT BLACKLISTING
!     NRBLEVEN( 3)      I     EXPERIMENTAL BLACKLISTING
!     NRBLEVEN( 4)      I     WHITELISTING
!     NRBLEVEN( 5)      I     EXPERIMENTAL WHITELISTING
!     NRBLEVEN( 6)      I     OBSERVATION TYPE BLACKLISTED
!     NRBLEVEN( 7)      I     STATION ID BLACKLISTED
!     NRBLEVEN( 8)      I     CODE TYPE BLACKLISTED
!     NRBLEVEN( 9)      I     INSTRUMENT TYPE BLACKLISTED
!     NRBLEVEN(10)      I     DATE BLACKLISTED
!     NRBLEVEN(11)      I     TIME BLACKLISTED
!     NRBLEVEN(12)      I     LATITUDE BLACKLISTED
!     NRBLEVEN(13)      I     LONGITUDE BLACKLISTED
!     NRBLEVEN(14)      I     STATION ALTITUDE BLACKLISTED
!     NRBLEVEN(15)      I     BLACKLISTED DUE TO LAND/SEA MASK
!     NRBLEVEN(16)      I     BLACKLISTED DUE TO MODEL OROGRAPHY
!     NRBLEVEN(17)      I     BLACKLISTED DUE TO DISTANCE FROM REF. POINT

!     DATA BLACKLIST EVENTS:

!     NAME            TYPE                   MEANING
!     ----            ----                   -------
!     NMXDBLE           I     MAX. NO. OF EVENTS
!     NODBLE            I     NO. OF DEFINED EVENTS
!     NDBLEVEN( 1)      I     PRESSURE BLACKLISTED
!     NDBLEVEN( 2)      I     VARIABLE NAME BLACKLISTED
!     NDBLEVEN( 3)      I     BLACKLISTED DUE TO PRESSURE CODE
!     NDBLEVEN( 4)      I     BLACKLISTED DUE TO TYPE OF VERTICAL COORDINATE
!     NDBLEVEN( 5)      I     BLACKLISTED DUE TO OBSERVED VALUE
!     NDBLEVEN( 6)      I     BLACKLISTED DUE TO FIRST GUESS DEPARTURE

INTEGER(KIND=JPIM) :: NEVBITOC
INTEGER(KIND=JPIM) :: NEVUPLIM
INTEGER(KIND=JPIM) :: NEVLVLIM
INTEGER(KIND=JPIM) :: NMXREV1
INTEGER(KIND=JPIM) :: NOREV1
INTEGER(KIND=JPIM) :: NR1EVENT(JPMXRE1)
INTEGER(KIND=JPIM) :: NMXREV2
INTEGER(KIND=JPIM) :: NOREV2(JPMXOTP)
INTEGER(KIND=JPIM) :: NR2EVENT(JPMXRE2,JPMXOTP)
INTEGER(KIND=JPIM) :: NMXDEV1
INTEGER(KIND=JPIM) :: NODEV1
INTEGER(KIND=JPIM) :: ND1EVENT(JPMXDE1)
INTEGER(KIND=JPIM) :: NMXDEV2
INTEGER(KIND=JPIM) :: NODEV2(JPMXOTP)
INTEGER(KIND=JPIM) :: ND2EVENT(JPMXDE2,JPMXOTP)
INTEGER(KIND=JPIM) :: NMXRBLE
INTEGER(KIND=JPIM) :: NORBLE
INTEGER(KIND=JPIM) :: NRBLEVEN(JPMXRBLE)
INTEGER(KIND=JPIM) :: NMXDBLE
INTEGER(KIND=JPIM) :: NODBLE
INTEGER(KIND=JPIM) :: NDBLEVEN(JPMXDBLE)

!-----------------------------------------------------------------------

!ODB BITMAP bit1:no_data@REPORT_EVENT1_t
!ODB BITMAP bit1:all_rejected@REPORT_EVENT1_t
!ODB BITMAP bit1:bad_practice@REPORT_EVENT1_t
!ODB BITMAP bit1:rdb_rejected@REPORT_EVENT1_t
!ODB BITMAP bit1:rdb_activated@REPORT_EVENT1_t

!ODB BITMAP bit1:whitelist_activated@REPORT_EVENT1_t
!ODB BITMAP bit1:horipos_outrange@REPORT_EVENT1_t
!ODB BITMAP bit1:vertpos_outrange@REPORT_EVENT1_t
!ODB BITMAP bit1:time_outrange@REPORT_EVENT1_t
!ODB BITMAP bit1:redundant@REPORT_EVENT1_t

!ODB BITMAP bit1:land@REPORT_EVENT1_t
!ODB BITMAP bit1:sea@REPORT_EVENT1_t
!ODB BITMAP bit1:stalt_missing@REPORT_EVENT1_t
!ODB BITMAP bit1:modsurf_stalt_distance@REPORT_EVENT1_t
!ODB BITMAP bit1:namelist_rejected@REPORT_EVENT1_t

!ODB BITMAP bit1:QC_failed@REPORT_EVENT1_t

!ODB TYPEDEF REPORT_EVENT2_t AS pk9int

!ODB BITMAP bit1:monthly@REPORT_BLACKLIST_t
!ODB BITMAP bit1:constant@REPORT_BLACKLIST_t
!ODB BITMAP bit1:experimental@REPORT_BLACKLIST_t
!ODB BITMAP bit1:whitelist@REPORT_BLACKLIST_t
!ODB BITMAP bit1:whitelist_experimental@REPORT_BLACKLIST_t

!ODB BITMAP bit1:obstype@REPORT_BLACKLIST_t
!ODB BITMAP bit1:codetype@REPORT_BLACKLIST_t
!ODB BITMAP bit1:instype@REPORT_BLACKLIST_t
!ODB BITMAP bit1:retrtype@REPORT_BLACKLIST_t
!ODB BITMAP bit1:statid@REPORT_BLACKLIST_t

!ODB BITMAP bit1:date@REPORT_BLACKLIST_t
!ODB BITMAP bit1:time@REPORT_BLACKLIST_t
!ODB BITMAP bit1:lat@REPORT_BLACKLIST_t
!ODB BITMAP bit1:lon@REPORT_BLACKLIST_t
!ODB BITMAP bit1:landsea@REPORT_BLACKLIST_t

!ODB BITMAP bit1:modoro@REPORT_BLACKLIST_t
!ODB BITMAP bit1:modPS@REPORT_BLACKLIST_t
!ODB BITMAP bit1:modTS@REPORT_BLACKLIST_t
!ODB BITMAP bit1:modT2M@REPORT_BLACKLIST_t
!ODB BITMAP bit1:dist@REPORT_BLACKLIST_t

!ODB BITMAP bit1:scanpos@REPORT_BLACKLIST_t
!ODB BITMAP bit1:QI_1@REPORT_BLACKLIST_t
!ODB BITMAP bit1:QI_2@REPORT_BLACKLIST_t
!ODB BITMAP bit1:QI_3@REPORT_BLACKLIST_t

!ODB BITMAP bit1:vertco_missing@DATUM_EVENT1_t
!ODB BITMAP bit1:obsvalue_missing@DATUM_EVENT1_t
!ODB BITMAP bit1:fg_missing@DATUM_EVENT1_t
!ODB BITMAP bit1:rdb_rejected@DATUM_EVENT1_t
!ODB BITMAP bit1:rdb_activated@DATUM_EVENT1_t

!ODB BITMAP bit1:whitelist_activated@DATUM_EVENT1_t
!ODB BITMAP bit1:bad_practice@DATUM_EVENT1_t
!ODB BITMAP bit1:vertpos_outrange@DATUM_EVENT1_t
!ODB BITMAP bit1:reflevel_outrange@DATUM_EVENT1_t
!ODB BITMAP bit1:fg2big@DATUM_EVENT1_t

!ODB BITMAP bit1:depar2big@DATUM_EVENT1_t
!ODB BITMAP bit1:obs_error2big@DATUM_EVENT1_t
!ODB BITMAP bit1:datum_redundant@DATUM_EVENT1_t
!ODB BITMAP bit1:level_redundant@DATUM_EVENT1_t
!ODB BITMAP bit1:land@DATUM_EVENT1_t

!ODB BITMAP bit1:sea@DATUM_EVENT1_t
!ODB BITMAP bit1:not_analysis_varno@DATUM_EVENT1_t
!ODB BITMAP bit1:duplicate@DATUM_EVENT1_t
!ODB BITMAP bit1:levels2many@DATUM_EVENT1_t
!ODB BITMAP bit1:multilevel_check@DATUM_EVENT1_t

!ODB BITMAP bit1:level_selection@DATUM_EVENT1_t
!ODB BITMAP bit1:vertco_consistency@DATUM_EVENT1_t
!ODB BITMAP bit1:vertco_type_changed@DATUM_EVENT1_t
!ODB BITMAP bit1:namelist_rejected@DATUM_EVENT1_t
!ODB BITMAP bit1:combined_flagging@DATUM_EVENT1_t

!ODB BITMAP bit1:report_rejected@DATUM_EVENT1_t
!ODB BITMAP bit1:varQC_performed@DATUM_EVENT1_t

!ODB TYPEDEF DATUM_EVENT2_t AS pk9int

!ODB BITMAP bit1:varno@DATUM_BLACKLIST_t
!ODB BITMAP bit1:vertco_type@DATUM_BLACKLIST_t
!ODB BITMAP bit1:press@DATUM_BLACKLIST_t
!ODB BITMAP bit1:press_rl@DATUM_BLACKLIST_t
!ODB BITMAP bit1:obsvalue@DATUM_BLACKLIST_t

!ODB BITMAP bit1:fg_depar@DATUM_BLACKLIST_t
!ODB BITMAP bit1:fg_error@DATUM_BLACKLIST_t
!ODB BITMAP bit1:obs_error@DATUM_BLACKLIST_t

END MODULE YOMNMEV
