MODULE YOMNNE
USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!! Contains data for neural_network bias correction (AIRS)

!! Number the network !!
INTEGER(KIND=JPIM)          :: NINPUTS               !number of inputs
INTEGER(KIND=JPIM)          :: NINPPB                !INPut Plus Bias : accounts for bias to input
INTEGER(KIND=JPIM)          :: NHIDDEN               !number of neurons on hidden layer
INTEGER(KIND=JPIM)          :: NDIM                  !Dimension of 1D 'W' and 'GW' arrays
INTEGER(KIND=JPIM)          :: NOUTPUTS              !number of outputs

!! Logicals for neural network configuration (Namelist Namnn) !!
LOGICAL            :: LNNWRI                ! .T. = DUMP NEURAL NETWORK LEARNING FILES 
LOGICAL            :: LNNMSKRS              ! .T. = DUMP ONLY NEAR RADIOSONDES         
LOGICAL            :: LNNBC                 ! .T. = RUN NEURAL NETWORK BIAS CORRECTION
LOGICAL            :: LNNHC                 ! .T. = RUN NEURAL NETWORK OBS OPERATOR CORRECTION 
LOGICAL            :: LNNLEARN              !learning/testing mode
LOGICAL            :: LNNREAD               !read/calculate weights, normalization, PCA, Jacobian
LOGICAL            :: LNNPCA                !PCA pre-processing
LOGICAL            :: LNNSELECT             !select useful input PCs (in pre-processing)
LOGICAL            :: LNNJAC                !write sensitivity files
LOGICAL            :: LNNOUT                !test performance
LOGICAL            :: LNNM1QN3              !use M1QN3 minimisation program

!! Define some constants which can be varied !!
INTEGER(KIND=JPIM)          :: NO_OF_EPOCHS          !maximum number of epochs
REAL(KIND=JPRB)             :: SCAL                  !scale for input/output normalisation
REAL(KIND=JPRB)             :: SELECTINP             !threshold for input selection
REAL(KIND=JPRB)             :: NU                    !regularization factor (for performance fn)
REAL(KIND=JPRB)             :: NU2                   !regularization factor (for performance fn)
REAL(KIND=JPRB)             :: BETA                  !coef for analysis correction before learning
INTEGER(KIND=JPIM),ALLOCATABLE:: LISS(:)             !offset for regularization

!! Arrays : NN Bias Correction Jacobian Matrix !!
REAL(KIND=JPRB),ALLOCATABLE :: NNJAC(:,:,:)

!! Arrays : Weights, Normalization, PCA, Jacobian stats !!
REAL(KIND=JPRB),ALLOCATABLE :: W(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: INP_MOY(:,:), INP_STD(:,:), OUT_MOY(:), OUT_STD(:)
REAL(KIND=JPRB),ALLOCATABLE :: EIG_VEC(:,:), EIG_VAL(:)
REAL(KIND=JPRB),ALLOCATABLE :: JAC_MOY(:,:),JAC_STD(:,:)

!! Data for neural_network forward_backward_prop !!
INTEGER(KIND=JPIM)          :: NPATS                 !number of patterns
REAL(KIND=JPRB),ALLOCATABLE :: TRAININP(:,:)         !input data set
REAL(KIND=JPRB),ALLOCATABLE :: TRAINOUT(:)           !output data set
REAL(KIND=JPRB),ALLOCATABLE :: OUTPRED(:)            !calculated bias (in normalized space)
REAL(KIND=JPRB),ALLOCATABLE :: GINP(:,:)             !jacobian w/r to inputs (or Principal Components)

END MODULE YOMNNE
