MODULE YOMOBS

USE PARKIND1, ONLY : JPIM
USE PARDIMO,  ONLY : JPNOTP
USE PARCMA,   ONLY : JPMXOCT, JPMXOTP

IMPLICIT NONE

SAVE

!*
!      YOMOBS - OBSERVATION PARAMETERS

!        D. VASILJEVIC   ECMWF     20/3/90

!     NAME    TYPE     MEANING
!     ----    ----     -------

!     LSLREJ    L   SEA/LAND REJECTION SWITCH
!     LSLRW10   L   SEA/LAND 10M WIND REJECTION SWITCH
!     LSLRT2    L   SEA/LAND 2M TEMPERATURE REJECTION SWITCH
!     LSLRRH2   L   SEA/LAND 2M REL. HUMIDITY REJECTION SWITCH
!     LHDREJ    L   HEIGHT DIFFERENCE REJECTION SWITCH
!     LHDRW10   L   HEIGHT DIFFERENCE 10M WIND REJECTION SWITCH
!     LHDRT2    L   HEIGHT DIFFERENCE 2M TEMPERATURE REJECTION SWITCH
!     LHDRRH2   L   HEIGHT DIFFERENCE 2M REL. HUMIDITY REJECTION SWITCH
!     LSCASUR   L   USE FOR SCAT THE SAME OBS OPERATOR AS SYNOP WINDS
!     LOBCLN    L   .TRUE. CLEAN THE NON WANTED WAGONS OF THE OBS ARRAY
!     LINCREM   L   INCREMENTAL SWITCH
!     LMFSCRE   L   METEO-FRANCE SCREENING (according to obs. types)
!     LMFSCREEN L   METEO-FRANCE SCREENING
!     LCAPACH   L   USE OF APACHE
!     LCACHMT   L   USE OF ACHMT
!     NOBSHOR   I   OBSERVATION INTERPOLATION NO.
!                      201 =  4 POINTS INTERPOLATION
!                      203 = 12 POINTS INTERPOLATION

!     LMKCMARPL L  MAKECMA REPLACEMENT
!     LMKCMARPLO  L  RUN MAKECMA REPLACEMENT ONLY
!     LRPLSWAPOUT L  SWAP OUT DB AFTER MKCMARPL
!     LSATOBTHIN  L  NEW SATOB THINNING
!     LTHINNER_PRINT   L LOCAL PRINTING SWITCH
!     LSATOBTHIN_NO_SQ L SATOB THINNING BUT NOT USING OBS SQ NO.
!     LSATAMTHIN_NO_SQ L SATAM(SATOB) THINNING BUT NOT USING OBS SQ NO.
!     LTOVSTHIN_NO_SQ  L TOVS THINNING BUT NOT USING OBS SQ NO.
!     LRAD1CTHIN_NO_SQ L RAD1C THINNING BUT NOT USING OBS SQ NO.
!     LSSMITHIN_NO_SQ  L SSMI THINNING BUT NOT USING OBS SQ NO.
!     LSCATTTHIN_NO_SQ L SCATT THINNING BUT NOT USING OBS SQ NO.
!     LSCATT_NEUTRAL   L ASSIMILATE SCATTEROMETER AS NEUTRAL WIND.
!     LSATEMTHIN_NO_SQ L SATEM THINNING BUT NOT USING OBS SQ NO.
!     LREO3THIN_NO_SQ  L REO3 THINNING BUT NOT USING OBS SQ NO.
!     LGPSROTHIN_NO_SQ L GPSRO THINNING BUT NOT USING OBS SQ NO.
!     LSYNOPPAOB_RED_NO_SQ    L SYNOP/PAOB REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LSYNOPPAOB_REDRP_NO_SQ  L SYNOP/PAOB (1) REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LSYNOPPAOB_REDRP1_NO_SQ L SYNOP/PAOB (2) REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LSHIP_NO_SQ             L SHIP REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LAIREPS_NO_SQ           L AIREPS REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LAIREPS_DUP_NO_SQ       L AIREPS REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LDRIBUS_NO_SQ           L DRIBUS REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LTEMPPILOT_NO_SQ        L TEMP/PILOT REDUNDANCY CHECK NO SEQUENCE NO. USED
!     LTEMPSYNOP_NO_SQ        L TEMP/SYNOP REDUNDANCY CHECK NO SEQUENCE NO. USED

!     L_F_SCR_WORK_TS         L FINAL SCREENING WORK TO BE DONE WITHIN TIMESLOT LOOP
!     LRSTBIAS_ODB            L RS T BIAS USING ODB
!     LMKCMARPL_OPENMP        L TO RUN MKCMARPL TIMESLOTS IN OPENMP-MODE (suobs.F90)
!     L_OBS_ERR_INCREASE_Hretr L TO INCREASE SYNOP Z ERR IN Hretr rather than in Prech
!     LAMSUB_FULL             L   USE OF AMSUB DATA IN FULL RESOLUTION (90 FOV)
!     LSOE                    L OT ACTIVATE RIGHT COMPUTATION OF SOE IN BLACKLIST, IF NOT SOE IS PUT AT 90.

!     LVDFTRAJ                L GET SURFACE QUANTITIES    FROM VDFMAIN IN EC_PHYS
!     LVDFMIN                 L GET SURFACE PERTURBATIONS FROM VDFMAIN IN EC_PHYS
!     LZ0FC                   L USE FORECAST, RATHER THAN CLIMATOLOGICAL SURFACE ROUGHNESS LENGTH

LOGICAL :: LINCREM
LOGICAL :: LOBCLN
LOGICAL :: LSLREJ
LOGICAL :: LSLRW10
LOGICAL :: LSOE
LOGICAL :: LSLRT2
LOGICAL :: LSLRRH2
LOGICAL :: LHDREJ
LOGICAL :: LHDRW10
LOGICAL :: LHDRT2
LOGICAL :: LHDRRH2
LOGICAL :: LSCASUR
LOGICAL :: LMFSCRE(JPNOTP)
LOGICAL :: LMFSCREEN
LOGICAL :: LCAPACH
LOGICAL :: LCACHMT
LOGICAL :: LMKCMARPL(JPMXOCT,JPMXOTP)
LOGICAL :: LMKCMARPLO
LOGICAL :: LRPLSWAPOUT
LOGICAL :: LTHINNER_PRINT
LOGICAL :: LSATOBTHIN_NO_SQ
LOGICAL :: LSATAMTHIN_NO_SQ
LOGICAL :: LTOVSTHIN_NO_SQ
LOGICAL :: LRAD1CTHIN_NO_SQ
LOGICAL :: LSSMITHIN_NO_SQ
LOGICAL :: LSCATTTHIN_NO_SQ
LOGICAL :: LSCATT_NEUTRAL
LOGICAL :: LSATEMTHIN_NO_SQ
LOGICAL :: LREO3THIN_NO_SQ
LOGICAL :: LGPSROTHIN_NO_SQ
LOGICAL :: LSYNOPPAOB_RED_NO_SQ
LOGICAL :: LSYNOPPAOB_REDRP_NO_SQ
LOGICAL :: LSYNOPPAOB_REDRP1_NO_SQ
LOGICAL :: LSHIP_NO_SQ
LOGICAL :: LAIREPS_NO_SQ
LOGICAL :: LAIREPS_DUP_NO_SQ
LOGICAL :: LDRIBUS_NO_SQ
LOGICAL :: LTEMPPILOT_NO_SQ
LOGICAL :: LTEMPSYNOP_NO_SQ
LOGICAL :: L_F_SCR_WORK_TS
LOGICAL :: LRSTBIAS_ODB
LOGICAL :: LMKCMARPL_OPENMP
LOGICAL :: L_OBS_ERR_INCREASE_Hretr
LOGICAL :: LAMSUB_FULL
LOGICAL :: LVDFTRAJ
LOGICAL :: LVDFMIN
LOGICAL :: LZ0FC

INTEGER(KIND=JPIM) :: NOBSHOR

!     ------------------------------------------------------------------

END MODULE YOMOBS
