module yomonedvar
      
use parkind1  , only: jpim, jprb
use paronedvar, only: npred_bias, nmax_satsens, nmax_channels

implicit none

save

real (kind=jprb), allocatable:: rcor_t (:,:)                               ! B for T
real (kind=jprb), allocatable:: rcor_q (:,:)                               ! B for q
real (kind=jprb), allocatable:: rcor_r (:,:)                               ! B for rh'
          
real    (kind=jprb) :: rlimit_rr                     ! rr limit for fg            [kg/m^2/s^1]
real    (kind=jprb) :: rlimit_sf                     ! snow fraction limit        []
real    (kind=jprb) :: rlimit_an_dep                 ! AN-departure growth limit  []
real    (kind=jprb) :: rlimit_inc                    ! TCWV increment check limit []
real    (kind=jprb) :: rlimit_ls                     ! land-sea fraction limit    []
real    (kind=jprb) :: rlimit_lat_1dv                ! upper sin(latitude) limit  []
real    (kind=jprb) :: rlimit_sd_1dv                 ! search distance limit      [m]
real    (kind=jprb) :: rconvergence                  ! convergence target         []
real    (kind=jprb) :: repsg_max                     ! convergence limit          []

! TB limit (in K) beyond which instrument errors are considered
! to indicate a non-assimilated channel. 
real    (kind=jprb) :: rlimit_err ! [K]
 
real    (kind=jprb) :: rsig_uv10                     ! Assumed 10m wind BG err std. dev.[m/s^1]
real    (kind=jprb) :: ran_err                       ! AN error fraction of BG error for TCWV
integer (kind=jpim) :: n_satsens                     ! Number of sensors used in 1dvar
logical :: lerain_anerr, lerain_lintest, lerain_adjtest, lerain_lb, lerain_clearsky

type onedvar_ids
  integer (kind=jpim) :: bufr_id   ! satellite buffer ID e.g., 241=F08, 249=SSMIS, etc.
  integer (kind=jpim) :: satseries ! satellite series number.  e.g., NOAA=1, DMSP=2, etc.
  integer (kind=jpim) :: satid     ! satellite platform ID.  e.g. NOAA-17=17, F-15=15.
  integer (kind=jpim) :: sensid    ! Sensor ID.  e.g., SSMI=6, TMI=9, AMSUA=3.
  integer (kind=jpim) :: nchannels ! No of channels on sensor.  E.g., SSMI=7, AMSUB=5.
  integer (kind=jpim) :: stream1d  ! Which 1DVar obs stream to use for this sensor     
  real    (kind=jprb) :: robs_err  (nmax_channels) ! 1sigma error for each channel, O+F
  real    (kind=jprb) :: rlimit_dep (nmax_channels) ! FG departure limit
  real    (kind=jprb) :: robs_bias (nmax_channels,0:npred_bias) ! bias correction coefficients
  real    (kind=jprb) :: zenangle  ! Zenith Angle (degrees) for sensor
end type onedvar_ids

type (onedvar_ids) :: onedvar_id (nmax_satsens)  ! Sensor Information Array

! 1DVar streams: Each stream may be separately turned active or passive
! and may use a combination of different sensors and satellites

! The number of streams
integer (kind=jpim) :: stream1d_num

! What to do with each stream?
integer (kind=jpim) :: stream1d_mode (nmax_satsens)

! Actively retrieved in 1D-Var and passed to 4D-Var as good obs:
integer (kind=jpim), parameter :: stream1d_active = 1
 
! Use 1D-Var for passive monitoring only:
integer (kind=jpim), parameter :: stream1d_passive = 0

! 1DVAR status code bitfield. 0 (no bits set) indicates a succesful retrieval. 
! Any of the following bits set indicate failure or monitoring-only observations
integer (kind=jpim), parameter :: &
 & ifail_no_grid_pt    = 0 , & ! Observation has no matching grid point (obs space only)
 & ifail_clear         = 1 , & ! Obs neither rainy nor cloudy
 & ifail_land          = 2 , & ! Land
 & ifail_fg_t_q        = 3 , & ! FG T or q profile outside bounds
 & ifail_fg_tb_depar   = 4 , & ! FG depar > 50K
 & ifail_minim_inc     = 5 , & ! Minimisation increments too big
 & ifail_minim_t       = 6 , & ! Minimisation T profile outside bounds
 & ifail_minim_q       = 7 , & ! Minimisation Q profile outside bounds
 & ifail_minim_wind    = 8 , & ! Minimisation wind increment too big
 & ifail_no_iterations = 9 , & ! No iterations on minimisation
 & ifail_cost_function = 10, & ! Cost function increased
 & ifail_tb_increment  = 11, & ! Analysed Tbs too far from observed
 & ifail_an_tb_depar   = 12, & ! Analysis Tb departures much worse than FG
 & ifail_tcwv_increment= 13, & ! TCWV increment too big
 & ifail_frozen_precip = 14, & ! Excess frozen precip in column
 & ifail_minim_failed  = 15, & ! M1QN3 minimisation failed (failure code logged in kim_dv)
 & ifail_no_convergence= 16, & ! Cost function gradient indicates minimisation did not converge
 & ifail_sensor        = 17, & ! Not a recognised sensor
 & ifail_passive       = 18, & ! Passive monitoring only
 & ifail_latitude      = 19, & ! latitude bounds
 & ifail_tsfc_ice      = 20, & ! Tsfc < 275K or sea-ice
 & ifail_obs_tb_bounds = 21    ! Obs Tb outside bounds

end module yomonedvar
