MODULE YOMOP

USE PARKIND1  ,ONLY : JPIM

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------

! NFDBREF:   data base reference number

! CNAMDB : data base name
! CIDDB  : data base id
! CNAMFS : file name in FDB (spectral fields)
! CNAMFG : file name in FDB (grid point fields)
! CNAMFD : field name in FDB

! CFNSH : pfn for trajectory spectral data file
! CFNTLSH : pfn for tangent linear trajectory spectral data file
! CFNPL : PFN FOR PRESSURE LEVEL SPECTRAL DATA FILE
! CFNVAREPS : pfn for accumulated fields for VAREPS
! CFNGG : pfn for trajectory grid point data file
! CFNRF : pfn for background=first guess/reference spectral data file
! CFNAN : pfn for analysis field during simulation spectral data file
! CFNGR : pfn for gradient field during simulation spectral data file
! CFNINSH : pfn for spectral part of increment field during simulation 
! CFNINGG : pfn for grid point part of increment field during simulation
! CFNFUN : pfn for total 4-D cost function (by simulations)
! CFNSNC : pfn for 4-D cost function (by wave number and simulations)
! CFNIT0 : pfn for total 4-D distance at t0 (by simulations)
! CFNPT0 : pfn for 4-D distance (at the begining, end of the minimisation
!                                and by wave number)
! CFNOPA : pfn for ECMWF operational analysis
! CFNDEP : pfn for model-ECMWF op. analysis departures.
! CFNISH : pfn for initial spectral data file
! CFNIGG : pfn for initial grid point data file
! CFNGSH : pfn for initial point of the minimization spectral data file
! CFNGGG : pfn for initial point of the minimization grid point data file
! CFNDDH : pfn for files produced by the DDH diagnostics.
! CFNHWF : pfn for history witness file
! CFNBS  : pfn for bias spectral data file (dig. filtered guess - guess)
! CFANS  : pfn for files containing only surface fields assimilation.
! CFNBGV : pfn for random background vector file
! CFNCGL : pfn for (scaled) eigenvector of the Hessian (CGL algorithm)
! CFNSKFY: pfn for 'y' vector of the simplified Kalman filter
! CFNSKFZ: pfn for 'z' vector of the simplified Kalman filter
! CFNFGI : pfn for background low resolution (incremental) SH
! CFNANI : pfn for analysis low resolution (incremental) SH
! CFNFGIG: pfn for background low resolution (incremental) GP
! CFNANIG: pfn for analysis low resolution (incremental) GP
! CFNTC  : pfn for radiation coefficients file for simplified radiation scheme
! CFNTRAJHRGRID: pfn for files containing the high resolution trajectory:
!                GRIB grid-point fields.
! CFNTRAJHRSPEC: pfn for files containing the high resolution trajectory:
!                GRIB upper air spectral fields.
! CFNTRAJHRSURF: pfn for files containing the high resolution trajectory:
!                GRIB surface fields.
! CFNTRAJHR    : pfn for files containing the high resolution trajectory:
!                ARPEGE files.
! CFNTRAJBGGRID: pfn for files containing the background:
!                GRIB grid-point fields.
! CFNTRAJBGSPEC: pfn for files containing the background:
!                GRIB upper air spectral fields.
! CFNTRAJBG    : pfn for files containing the background:
!                ARPEGE files.
! CEFLS   : pfn for lateral boundary compacted files (special internal file)
! CEFNLSH : pfn for lateral boundary coupling files (ALADIN file)
! CETSTAMP : Time stamp for lateral boundary coupling files (ALADIN file)
! CFNBGHRSH    : pfn for the high resolution background at all times
!                where the increments are added: spectral part.
! CFNBGHRGG    : pfn for the high resolution background at all times
!                where the increments are added: gridpoint part.
! CNMCA  : name of standard frame of *FA*
! LINC   : incremental switch for files (true : hour, false : time step)
! LBCINC : incremental switch for input LBC files (true : hour, false : number)
! LTIMEP0: switch for time written out on FA files : .t. => like a forecast.
! CFPATH : directory name (ending with a '/') of time-dependent output files
! NLPATH : length of CFPATH
! NAGDIO : to read arpege DM grid point fields
! NADCIO : to read arpege DM matcont matdila fields
! NAROIO : to read arpege DM matrotc fields
! NBGOUT : number of grid points file for OUTPUT
! NBSOUT : number of spectral file for OUTPUT
! NCADFORM:format of *FA* frames in new limited area files
!          0=old format, 1=new format

CHARACTER(LEN=14) :: CNAMDB
CHARACTER(LEN=4)  :: CIDDB
CHARACTER(LEN=11) :: CNAMFS
CHARACTER(LEN=11) :: CNAMFG
CHARACTER(LEN=16) :: CNAMFD
CHARACTER(LEN=210):: CFNSH
CHARACTER(LEN=16) :: CFNTLSH
CHARACTER(LEN=16) :: CFNGG
CHARACTER(LEN=16) :: CFNISH
CHARACTER(LEN=16) :: CFNIGG
CHARACTER(LEN=16) :: CFNPL
CHARACTER(LEN=16) :: CFNVAREPS
CHARACTER(LEN=16) :: CNMCA
CHARACTER(LEN=16) :: CFNRF
CHARACTER(LEN=16) :: CFANS
CHARACTER(LEN=16) :: CFNAN
CHARACTER(LEN=16) :: CFNGR
CHARACTER(LEN=16) :: CFNINSH
CHARACTER(LEN=16) :: CFNINGG
CHARACTER(LEN=16) :: CFNFUN
CHARACTER(LEN=16) :: CFNSNC
CHARACTER(LEN=16) :: CFNIT0
CHARACTER(LEN=16) :: CFNPT0
CHARACTER(LEN=256):: CFNHWF
CHARACTER(LEN=16) :: CFNOPA
CHARACTER(LEN=20) :: CFNDEP
CHARACTER(LEN=16) :: CFNGSH
CHARACTER(LEN=16) :: CFNGGG
CHARACTER(LEN=16) :: CFNDDH
CHARACTER(LEN=200):: CFPATH
CHARACTER(LEN=16) :: CFNBS
CHARACTER(LEN=16) :: CFNBGV
CHARACTER(LEN=16) :: CFNCGL
CHARACTER(LEN=16) :: CFNFGI
CHARACTER(LEN=16) :: CFNANI
CHARACTER(LEN=16) :: CFNFGIG
CHARACTER(LEN=16) :: CFNANIG
CHARACTER(LEN=16) :: CFNSKFY
CHARACTER(LEN=16) :: CFNSKFZ
CHARACTER(LEN=16) :: CFNTC
CHARACTER(LEN=17) :: CFNTRAJHRGRID
CHARACTER(LEN=17) :: CFNTRAJHRSPEC
CHARACTER(LEN=20) :: CFNTRAJHRSURF
CHARACTER(LEN=16) :: CFNTRAJHR
CHARACTER(LEN=17) :: CFNTRAJBGGRID
CHARACTER(LEN=17) :: CFNTRAJBGSPEC
CHARACTER(LEN=16) :: CFNTRAJBG
CHARACTER(LEN=16) :: CEFLS
CHARACTER(LEN=200) :: CEFNLSH
CHARACTER(LEN=5) :: CETSTAMP
CHARACTER :: CFNBGHRSH*16
CHARACTER :: CFNBGHRGG*16

INTEGER(KIND=JPIM),ALLOCATABLE:: NAGDIO(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NADCIO(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NAROIO(:,:)

LOGICAL :: LINC
LOGICAL :: LBCINC
INTEGER(KIND=JPIM) :: NFDBREF
INTEGER(KIND=JPIM) :: NLPATH
INTEGER(KIND=JPIM) :: MAXFIELD
INTEGER(KIND=JPIM) :: NBGOUT
INTEGER(KIND=JPIM) :: NBSOUT
LOGICAL :: LTIMEP0

INTEGER(KIND=JPIM) :: NCADFORM

!     ------------------------------------------------------------------
END MODULE YOMOP
