MODULE YOMRLX

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE PARDIM

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------

! LRLXG  : USE OF RELAXATION TERMS

! LRLXVO : relaxation switch for vorticity (altitude)
! LRLXDI : relaxation switch for divergence (altitude)
! LRLXTE : relaxation switch for temperature (altitude)
! LRLXQ : relaxation switch for specific humidity (altitude)
! LRLXQI : relaxation switch for ice water (altitude)
! LRLXQL : relaxation switch for liquid water (altitude)
! LRLXQC : relaxation switch for cloud fraction (altitude)
! LRLXLP : relaxation switch for log(surf pressure)

! NFRLXG : number of analysis time steps in memory
! NFRLXU : interval between two references fields (in hours)
!          (this is equal to NFRHIS in nudging)
! NPRLX3D: number of 3D parameters being relaxed
! NPRLX2D: number of 2D parameters being relaxed
! NRLXAN : number of reference steps

! XRLXVO : relaxation coefficient for vorticity (altitude)
! XRLXDI : relaxation coefficient for divergence (altitude)
! XRLXTE : relaxation coefficient for temperature (altitude)
! XRLXQ : relaxation coefficient for temperature (altitude)
! XRLXLP : relaxation coefficient for log(surf pressure)

! ALATRLX1 : northern limit
! ALATRLX2 : southern limit
! ALONRLX1 : western limit in [0,360]
! ALONRLX2 : eastern limit in [0,360] (E<W will span Greenwich)

! AXRLX : smoothing parameter in longitude
! AYRLX : smoothing parameter in latitude
! AZRLX : smoothing parameter in the vertical

! NRLXLMIN : lowermost model level
! NRLXLMAX : lowermost model level

INTEGER(KIND=JPIM) :: NFRLXG, NFRLXU
INTEGER(KIND=JPIM) :: NPRLX3D, NPRLX2D
INTEGER(KIND=JPIM) :: NRLXAN

LOGICAL :: LRLXG

LOGICAL :: LRLXVO
LOGICAL :: LRLXDI
LOGICAL :: LRLXTE
LOGICAL :: LRLXLP
LOGICAL :: LRLXQ
LOGICAL :: LRLXQI
LOGICAL :: LRLXQL
LOGICAL :: LRLXQC

REAL(KIND=JPRB) :: XRLXVO
REAL(KIND=JPRB) :: XRLXDI
REAL(KIND=JPRB) :: XRLXTE
REAL(KIND=JPRB) :: XRLXLP
REAL(KIND=JPRB) :: XRLXQ

REAL(KIND=JPRB) :: ALATRLX1
REAL(KIND=JPRB) :: ALATRLX2
REAL(KIND=JPRB) :: ALONRLX1
REAL(KIND=JPRB) :: ALONRLX2

REAL(KIND=JPRB) :: AXRLX
REAL(KIND=JPRB) :: AYRLX
REAL(KIND=JPRB) :: AZRLX

INTEGER(KIND=JPIM) :: NRLXLMIN
INTEGER(KIND=JPIM) :: NRLXLMAX

END MODULE YOMRLX

