MODULE YOMSCC

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMTVRAD

IMPLICIT NONE

SAVE

!*     YOMSCC - OBSERVATION SCREENING CONTROL PARAMETERS
!                     SETUP IN SUBROUTINE*SUOBS/NAMELIST*NAMSCC

!        HEIKKI JARVINEN   ECMWF     15/3/96
!        ELISABETH GERARD  ECMWF     15/01/1999 (Modification)
!            SSM/I wind speed and total column water vapour
!            have their own thinning parameter.
!        DAVID TAN           ECMWF     25/11/2003 (Modification)
!            Variables for Analysis Ensembles
!        CHRIS PAYAN       MF        ??/04/2005
!            addition of NICESEAMASK integer key
!        HANS HERSBACH       ECMWF     15/11/2007
!            introduce NSCAWSOLMAX: max number of scat ambiguities
!            introduce LQSCATAZI
!        CHRIS PAYAN       MF        ??/04/2007
!            addition of NSCAT5_RQC integer key
!        PAUL POLI         MF        06/03/2007
!            addition of thinning parameters for GPSRO

!     NAME      TYPE                  MEANING
!     ----      ----                  -------

!     LSCDPR      L    SCREENING DIAGNOSTIC PRINTOUT (DEFAULT = .FALSE.)
!                      - CONTROLS THE AMOUNT OF PRINTOUT IN SCREENING RUN
!     LSCRE4D     L    OBSERVATION SCREENING IN 4D-MODE (DEFAULT .F. = 3D-MODE)
!     LSRERUN     L    RE-RUNNING IFS WITH A SECOND-HAND CMA-FILE (F)
!     LTARGDROP   L    RELAX BACKGROUND CHECK FOR TARGETED DROPSONDES

!     RMIND_TOV  - Minimum distance ( degrees ) between TOVS obs
!     RFIND_TOV  - Average distance ( degrees ) between TOVS obs
!     RMIND_SSMI - Minimum distance ( degrees ) between SSMI obs
!     RMIND_SATOB - Minimum distance ( degrees ) between SATOB obs
!     RFIND_SATOB - Average distance ( degrees ) between SATOB obs
!     RMIND_RAD1C  - Minimum distance (degrees) between RAD1C obs
!     RFIND_RAD1C  - Average distance (degrees) between RAD1C obs
!     RFIND_SSMI_PWC - Average distance (degrees) between SSMI obs 
!                      for total column water vapour
!     RFIND_SSMI_WSP - Average distance (degrees) between SSMI obs 
!                      for surface wind speed
!     RMIND_SATAM - Minimum distance (degrees) between SATAM obs
!     RFIND_SATAM - Average distance (degrees) between SATAM obs
!     RFIND_AIREP - Average distance (degrees) between AIREP obs
!     RMIND_SCATT_ERS - Minimum distance (degrees) between SCAT obs
!     RFIND_SCATT_ERS - Average distance (degrees) between SCAT obs
!     RMIND_GPSRO - Minimum distance (degrees) between GPSRO obs
!     RFIND_GPSRO - Average distance (degrees) between GPSRO obs
!     RMIND_RADAR - Minimum distance (degrees) between RADAR obs
!     RFIND_RADAR - Average distance (degrees) between RADAR obs
!     NTHINSCA    I    Thinning parameter for ERS scatterometer
!     NICESEAMASK - /1/2/3 ice-sea mask based on /SST/ice cc/both
!     NSCAWSOLMAX - Maximum allowed number or ambigious scatterometer wind solutions
!     LQSCATAZI   - if T, perform QC on azimith diversity for QuikSCAT
!     LOSCATAZI   - if T, perform QC on azimith diversity for OCEANSAT2
!     NSCAT5_RQC  - /1,2/ Quikscat RainQC based on /raincontamination,dist2cone/
!     FL_MAXSPEED - Maximum speed expected for aircrafts in km/hour
!                   (used to remove unrealistic AIREP reports)
!     SELAIREPPRE - Pressure levels used for aircraft thinning
!     RAIREPTHIN  - Vertical thinning distance in pressure [Pa] used for aircraft thinning (default 1500.)
!     RAIREPPCENTTHIN  - Minimum difference of pressure ratio between levels used for aircraft thinning (default 5%==0.05)
!     RAIREPTOPPRES - Top pressure level [Pa] for aircraft thinning (default 10000.)
 

!     LPERTURB    L    TO PERTURB IN ANALYSIS ENSEMBLE (DEFAULT = .FALSE.)
!     NAENSEMBLE  I    ANALYSIS ENSEMBLE GROUP NUMBER
!     NAEMEMBER   I    ANALYSIS ENSEMBLE MEMBER NUMBER

LOGICAL :: LSCDPR
LOGICAL :: LSCRE4D
LOGICAL :: LSRERUN
LOGICAL :: LTARGDROP
LOGICAL :: LPERTURB
LOGICAL :: LQSCATAZI
LOGICAL :: LOSCATAZI

REAL(KIND=JPRB) :: RMIND_TOV
REAL(KIND=JPRB) :: RFIND_TOV
REAL(KIND=JPRB) :: RMIND_RAD1C(0:MXSENSOR)
REAL(KIND=JPRB) :: RFIND_RAD1C(0:MXSENSOR)
REAL(KIND=JPRB) :: RMIND_SSMI
REAL(KIND=JPRB) :: RFIND_SSMI_PWC
REAL(KIND=JPRB) :: RFIND_SSMI_WSP
REAL(KIND=JPRB) :: RMIND_SATOB
REAL(KIND=JPRB) :: RFIND_SATOB
REAL(KIND=JPRB) :: RMIND_SATAM
REAL(KIND=JPRB) :: RFIND_SATAM
REAL(KIND=JPRB) :: RFIND_AIREP
REAL(KIND=JPRB) :: RMIND_SCATT_ERS
REAL(KIND=JPRB) :: RFIND_SCATT_ERS
REAL(KIND=JPRB) :: RMIND_GPSRO
REAL(KIND=JPRB) :: RFIND_GPSRO
REAL(KIND=JPRB) :: RMIND_RADAR
REAL(KIND=JPRB) :: RFIND_RADAR
REAL(KIND=JPRB) :: FL_MAXSPEED
REAL(KIND=JPRB), ALLOCATABLE :: SELAIREPPRE(:)
REAL(KIND=JPRB) :: RAIREPTHIN
REAL(KIND=JPRB) :: RAIREPPCENTTHIN
REAL(KIND=JPRB) :: RAIREPTOPPRES

INTEGER(KIND=JPIM) :: NTHINSCA
INTEGER(KIND=JPIM) :: NICESEAMASK
INTEGER(KIND=JPIM) :: NSCAWSOLMAX
INTEGER(KIND=JPIM) :: NSCAT5_RQC
INTEGER(KIND=JPIM) :: NAENSEMBLE, NAEMEMBER

!-----------------------------------------------------------------------

END MODULE YOMSCC
