module yomsmos
      
use parkind1, only: jpim, jprb
use parsmos,  only: npol_max, rlimit_sd

implicit none

save

integer(kind=jpim)   :: nlenmax      ! max number of obs for allocation 

logical :: lesmos_active

integer (kind=jpim), allocatable :: ob_from    (:,:,:)
integer (kind=jpim), allocatable :: ob_jobs    (:,:,:)
integer (kind=jpim), allocatable :: ob_jrof_gp (:,:,:)
integer (kind=jpim), allocatable :: ob_index   (:,:)
integer (kind=jpim), allocatable :: gp_count   (:,:)
integer (kind=jpim), allocatable :: ob_count   (:,:)

type smos_phys_type
 integer (kind=jpim) :: nlevelsur          ! number of surface levels
! prognostic/diagnostic meteo variables 
  real    (kind=jprb), pointer :: sm (:)   ! soil moisture 
  real    (kind=jprb), pointer :: st (:)   ! soil temperature
  real    (kind=jprb)   :: skt             ! skin temperature
  real    (kind=jprb)   :: t2m             ! 2m temperature
  real    (kind=jprb)   :: sndt            ! snow density
  real    (kind=jprb)   :: sndp            ! snow depth
! vegetation
  real    (kind=jprb)   :: laih            ! LAI high vegetation
  real    (kind=jprb)   :: lail            ! LAI low vegetation
  real    (kind=jprb)   :: fch             ! fraction cover high vegetation
  real    (kind=jprb)   :: fcl             ! fraction cover low vegetation
 integer  (kind=jpim)   :: vegth           ! high vegetation type
 integer  (kind=jpim)   :: vegtl           ! low vegetation type
! static variables
  real    (kind=jprb)   :: or              ! geopotential at surface (orography)
  real    (kind=jprb)   :: lsm             ! land-sea mask
 integer  (kind=jpim)   :: stp             ! soil type
end type smos_phys_type  

type smos_tb_type
  integer (kind=jpim) :: screen (0:npol_max-1) ! screening flag 
  real    (kind=jprb) :: obs    (0:npol_max-1) ! observed brightness temperature
  real    (kind=jprb) :: fw     (0:npol_max-1) ! modelled brightness temperature
  real    (kind=jprb) :: ang    (0:npol_max-1) ! incidence angle 
  real    (kind=jprb) :: oerr              ! observation error 
  real    (kind=jprb) :: bias              ! observation bias  
end type smos_tb_type  

! Status code bitfield.  
integer (kind=jpim), parameter :: &
 & smos_use            = 0 , & ! Observation active if no other bits set
 & smos_obs_tb         = 1 , & ! Observed TB out of bounds
 & smos_anaprop        = 2 , & ! Anomalous propagation in model
 & smos_sea            = 3 , & ! Sea or lake point in model
 & smos_snow           = 4 , & ! Surface snowfall probable according to model 2m temp.
 & smos_no_obs         = 5 , & ! No obs for this grid point (grid point space only)
 & smos_fg_depar       = 6 , & ! FG depar outside limit
 & smos_contaminated   = 7     ! Obs rejected because neighbour on low-res grid rejected

! Value of status code bitfield when observation is active
integer (kind=jpim), parameter :: smos_active = ibset(0,smos_use)

end module yomsmos
