MODULE YOMTHLIM

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!*     YOMTHLIM - LIMITS FOR DATA THINNING
!                       DEFINED IN ROUTINE SUFGLIM

!        HEIKKI JARVINEN   ECMWF     10/6/95

!     NAME      TYPE                  MEANING
!     ----      ----                  -------

!     NSCIND1     I    HELP ARRAY TO DEDUCE TOVS SCANNING POSITION
!     NSCIND2     I    HELP ARRAY TO DEDUCE TOVS SCANNING POSITION
!     RSCAWLI     R    UPPER WIND SPEED LIMIT FOR USE OF SCATT. WIND (IN M/S)
!     RSCACLI     R    MAX SEA ICE CONCENTRATION LIMIT FOR USE OF SCATT. WIND 
!     RSCATLI     R    MIN SEA SURFACE TEMPERATURE LIMIT FOR USE OF SCATT. WIND 
!     RREDTSL     R    TEMP/SYNOP MASS DATA REDUNDANCY REJ. LIMIT (IN PA)
!     RSHIDIS     R    SHIP/DRIBU DISTANCE LIMIT (IN M) (ONE DEGREE AT EQ.)
!     NSCAMIN     I    MINIMUM NUMBER OF SCATT DATA PER CELL FOR Q.C.
!     NSCATWVC    I    NUMBER OF ERS   WAVE VECTOR CELLS 
!     NASCAWVC    I    NUMBER OF ASCAT WAVE VECTOR CELLS
!     NNSCAWVC    I    NUMBER OF NSCAT WAVE VECTOR CELLS
!     NQSCAWVC    I    NUMBER OF QSCAT WAVE VECTOR CELLS
!     NOSCAWVC    I    NUMBER OF OCEANSCAT-2 WAVE VECTOR CELLS
!     RSCATDIS    R    LIMIT FOR THE AVERAGE ERS   DISTANCE TO THE CONE
!     RASCADIS    R    LIMIT FOR THE AVERAGE ASCAT DISTANCE TO THE CONE
!     RNSCADIS    R    LIMIT FOR THE AVERAGE NSCAT MLE
!     RQSCADIS    R    LIMIT FOR THE AVERAGE QSCAT MLE
!     ROSCADIS    R    LIMIT FOR THE AVERAGE OCEANSAT-2 MLE
!     RSCAFAC     R    FACTOR APPLIED FOR LIMIT

REAL(KIND=JPRB) :: RSCAWLI
REAL(KIND=JPRB) :: RSCACLI
REAL(KIND=JPRB) :: RSCATLI
REAL(KIND=JPRB) :: RREDTSL
REAL(KIND=JPRB) :: RSHIDIS
REAL(KIND=JPRB) :: RSCAFAC
INTEGER(KIND=JPIM) :: NSCIND1(30)
INTEGER(KIND=JPIM) :: NSCIND2(6)
INTEGER(KIND=JPIM) :: NSCAMIN
INTEGER(KIND=JPIM) :: NSCATWVC
INTEGER(KIND=JPIM) :: NASCAWVC
INTEGER(KIND=JPIM) :: NNSCAWVC
INTEGER(KIND=JPIM) :: NQSCAWVC
INTEGER(KIND=JPIM) :: NOSCAWVC

REAL(KIND=JPRB),ALLOCATABLE:: RSCATDIS(:)
REAL(KIND=JPRB),ALLOCATABLE:: RASCADIS(:)
REAL(KIND=JPRB),ALLOCATABLE:: RNSCADIS(:)
REAL(KIND=JPRB),ALLOCATABLE:: RQSCADIS(:)
REAL(KIND=JPRB),ALLOCATABLE:: ROSCADIS(:)

!-----------------------------------------------------------------------

END MODULE YOMTHLIM
