MODULE YOMTVRAD

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!*

!     /YOMTVRAD/  J. PAILLEUX 90/11/30

! Rose Munro (Dec 1997) Add pointers for multiple satellite series, 
!                       id's and subtypes 
! Rose Munro (Jun 1998) Add QMAXRT, O3MAXRT and O3MINRT for new RT model
! Dick Dee   2004-03-19 Support for variational sat bias correction
! L.v.Bremen (Apr 2003) Add MSG and MODIS in character list CHGEOSATN for AMV 
! M Szyndel  (March 2005) Add MTSAT, FY-2 and update MSG radiances
!     TABLES DEFINING ALL THE LOGISTICS FOR TOVS RADIANCE COMPUTATION.
! A.Benedetti 2005-Mar-22 Add MODIS in SATEM sensor list 
! Richard Engelen (Mar 2004) Add CO2MAXRT and CO2MINRT for new RT model and
!    CO2BGE and CO2CLRT as default values for background CO2
! N Bormann (Feb 2006)  Add METOP
! P Bauer/G Kelly/N Bormann (Feb 2006) Add AMSRE, SSMIS, TMI
! V Guidard (March 2007) Add IASI (after A Collard)
! A Collard (Oct 2006) Add IASI
! N Bormann (Nov 2007) Adjust CHGEOSATN
! N Bormann (Dec 2007) Update for RTTOV-9
! A Inness  (Aug 2009) Add varno to SATREO3GRP_TABLE
! J Munoz Sabater (May 2009) Introduce SMOS
! V Guidard (Feb 2009) Add LCO2_DIAG_AIRS
! W Bell (Feb 2009) Add Coriolis-Windsat
! Q Lu      (Mar 2009) Add FY3A

!   COMMENTS:
!   --------

!     RADPRE  : PRESSURE LEVELS USED IN THE RAD. TRANSF. MODEL
!     VDZCOR  : SATEM THICKNESS CORRELATION MATRIX
!     OERRDZ  : SATEM THICKNESS OBSERVATION ERRORS
!     OERRPWCREL: SATEM PWC OBSERVATION ERROR, relative to SPWC
!     ORERPWC : SATEM PWC ROUND-OFF OBSERVATION ERROR
!     NJPPF   : number of tovs that the tovs-code is processing in one go
!     O3CLRT  : OZONE climatology.
!     TOVSCVBG: TOVS control variable, background
!     TOVSCVER: TOVS control variable, standard deviation of error in bg
!     TOVSCVX : TOVS control variable, value
!     TOVSCVX5: TOVS control variable, trajectory
!     NVATOVINDX: Index to TOVS data in control variable
!     NVATOVOFF:  Offset to TOVS data in control variable
!     NVATOVLEN:  Local length TOVS data in control variable
!     NVATOVLENP:  Length TOVS data in control variable all procs
!     TVTSBGE_LAND : TOVS Ts background error, land
!     TVTSBGE_SEA  : TOVS Ts background error, sea
!     TVTSBGE_ICE  : TOVS Ts background error, ice
!     CO2BGE  : CO2 background error
!     CO2CLRT : CO2 climatology
!     MEMIS_SW     : switch for new ice /land classif method for emis
!     AIRSCHAN: used AIRS channels
!     AIRSCHAN_BACK: AIRS channels
!     IASICHAN: used IASI channels
!     IASICHAN_BACK: IASI channels
!     IASICHAN: IASI channels

!  VARIABLES AND PARAMETERS USED AS DIMENSIONS IN THESE ARRAYS ARE
!  RESPECTIVELY DEFINED IN COMDECKS "YOMDIMO" AND "PARDIMO".

REAL(KIND=JPRB),ALLOCATABLE:: RADPRE(:)
REAL(KIND=JPRB),ALLOCATABLE:: O3CLRT(:)

REAL(KIND=JPRB),ALLOCATABLE:: TOVSCVBG(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: TOVSCVER(:,:)
REAL(KIND=JPRB),ALLOCATABLE, TARGET :: TOVSCVX(:,:)
REAL(KIND=JPRB),ALLOCATABLE:: TOVSCVX5(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NVATOVINDX(:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NVATOVOFF(:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NVATOVLENP(:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NVALCHAN(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE:: NVALCHAN_MTS(:,:)
INTEGER(KIND=JPIM) :: NVATOVLEN
INTEGER(KIND=JPIM) :: NVATOVLENG
REAL(KIND=JPRB) :: TVTSBGE_LAND
REAL(KIND=JPRB) :: TVTSBGE_SEA
REAL(KIND=JPRB) :: TVTSBGE_ICE
REAL(KIND=JPRB) :: CO2BGE_S, CO2BGE_T
REAL(KIND=JPRB) :: CO2CLRT_S, CO2CLRT_T

INTEGER(KIND=JPIM) :: NHULAY
INTEGER(KIND=JPIM) :: NJPPF , NJPNSAT

! Define SATOB methods numbering
INTEGER(KIND=JPIM), PARAMETER :: MXMETHOD         = 5
INTEGER(KIND=JPIM), PARAMETER :: MMETHOD_WVCL     = 0
INTEGER(KIND=JPIM), PARAMETER :: MMETHOD_IR       = 1
INTEGER(KIND=JPIM), PARAMETER :: MMETHOD_VIS      = 2
INTEGER(KIND=JPIM), PARAMETER :: MMETHOD_WVMIX    = 3
INTEGER(KIND=JPIM), PARAMETER :: MMETHOD_COM_SPEC = 4
INTEGER(KIND=JPIM), PARAMETER :: MMETHOD_WVMW     = 5
CHARACTER(LEN=8), PARAMETER :: CHMETHOD(0:MXMETHOD) = &
 & (/'WVCL    ','IR      ','VIS     ','WVMIX   ','COM_CHAN','WVMW    '/)  

! Define Geostationary Satellite names
INTEGER(KIND=JPIM), PARAMETER :: MXGEOSATN         = 7
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_METEOSAT = 0
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_GMS      = 1
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_GOES     = 2
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_MSG      = 3
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_INDSAT   = 4
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_FY       = 5
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_XXXXXX   = 6
INTEGER(KIND=JPIM), PARAMETER :: MGEOSATN_MODIS    = 7
CHARACTER(LEN=9), PARAMETER :: CHGEOSATN(0:MXGEOSATN) = &
  & (/'METEOSAT ','GMS      ','GOES/NOAA','MSG      ','INDSAT   ','FY       ',&
  &   'XXXXX    ','MODIS    '/)

! Add pointer arrays which define which satellite series, id's and 
! subtypes are in the CMA. Also variables for passing dimension 
! information into SUADVAR. An id_list array is also included
! which contains corresponding pointers for rttov numbers.

INTEGER(KIND=JPIM)  :: MSATDIM, MSUBDIM
INTEGER(KIND=JPIM)  :: MNSERIES
INTEGER(KIND=JPIM)  :: MEMIS_SW
INTEGER(KIND=JPIM), ALLOCATABLE :: MKSERIES(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MNSATID(:),   MNSUBTYPE(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MKSATID(:,:), MKSUBTYPE(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MBSATID(:,:), MBSUBTYPE(:,:)

! Define series numbering.
INTEGER(KIND=JPIM), PARAMETER :: MXSERIES         =23
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_NOAA     =1
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_DMSP     =2
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_METEOSAT =3
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_GOES     =4
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_GMS      =5
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_FY2      =6
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_TRMM     =7
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_EOS      =9
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_METOP    =10
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_MTSAT    =15
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_CORIOLIS =16
INTEGER(KIND=JPIM), PARAMETER :: MSERIES_FY3      =23
! MSERIES_NASA has been changed to allow for FY-2. Value chosen is 
! beyond series range of RTTOV at present.
CHARACTER(LEN=8), PARAMETER :: CHSERIES(1:MXSERIES) = &
 & (/'NOAA    ','DMSP    ','METEOSAT','GOES    ','GMS     ','FY-2    ',&
 &   'TRMM    ','ERS     ','EOS     ','METOP   ','ENVISAT ','MSG     ',&
 &   'FY-1    ','ADEOS   ','MTSAT   ','CORIOLIS','NPOESS  ','GIFTS   ',&
 &   'AURA    ','        ','        ','        ','FY-3    '/)
! Many of the series in string CHSERIES are not implemented; the values
! are set as a reminder that RTTOV uses these series numbers. 

! Define sensor numbering.
INTEGER(KIND=JPIM), PARAMETER :: MXSENSOR         =43
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_HIRS     = 0
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MSU      = 1
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_SSU      = 2
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_AMSUA    = 3
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_AMSUB    = 4
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_SSMI     = 6
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_VTPR1    = 7
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_VTPR2    = 8
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_TMI      = 9
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_SSMIS    =10
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_AIRS     =11
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MODIS    =13
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MHS      =15
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_IASI     =16
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_AMSRE    =17
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_SMOS     =18
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_METEOSAT =20
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MSG      =21
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MSG_HR   =29
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_GOESIMG  =22
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MTSATIMG =24
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_VISSR    =25
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_WINDSAT  =30
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MWTS     =40
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MWHS     =41
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_IRAS     =42
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_MWRI     =43
INTEGER(KIND=JPIM), PARAMETER :: NSENSOR_MERIS    =174
INTEGER(KIND=JPIM), PARAMETER :: NSENSOR_MOPITT   =090

CHARACTER(LEN=8), PARAMETER :: CHSENSOR(0:MXSENSOR) = &
 & (/'HIRS    ','MSU     ','SSU     ','AMSUA   ','AMSUB   ','        '&
 & ,'SSMI    ','VTPR1   ','VTPR2   ','TMI     ','SSMIS   '&
 & ,'AIRS    ','        ','MODIS   ','        ','MHS     '&
 & ,'IASI    ','AMSR-E  ','SMOS    ','        ','METEOSAT'&
 & ,'MSG     ','GOESIMG ','        ','MTSATIMG','VISSR   '&
 & ,'        ','        ','        ','MSG HR  ','WINDSAT '&
 & ,'        ','        ','        ','        ','        '&
 & ,'        ','        ','        ','        ','MWTS    '&
 & ,'MWHS    ','IRAS    ','MWRI    '/)  

! Define channel numbering offsets, by sensor.
INTEGER(KIND=JPIM), PARAMETER :: MSENSOR_CHANOFFL(0:MXSENSOR)= &
 & (/0,20,24,27,42,0,0,0,8,0,0,0,0,0,0,42,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, &
 &   0,0,0,0,0,0,0,0,0,0,0,0,0/)  

! Define channel counts by sensor (AIRS and IASI are channel subsets): 
INTEGER(KIND=JPIM), PARAMETER :: M__NumAIRSChans = 324
INTEGER(KIND=JPIM), PARAMETER :: M__MaxAIRSChans = 2378
INTEGER(KIND=JPIM)  :: M__NumIASIChans = -999
INTEGER(KIND=JPIM), PARAMETER :: M__MaxIASIChans = 8461
INTEGER(KIND=JPIM)  :: MSENSOR_CHANCOUNT(0:MXSENSOR)= &
 & (/20,4,3,15,5,0,7,8,8,9,24,M__NumAIRSChans,0,0,0,5,-999, &
 &   12,18,0,2,8,4,0,4,4,0,0,0,8,16, &
 &   0,0,0,0,0,0,0,0,0,4,5,20,10/)  

! Define usable AIRS channels
INTEGER(KIND=JPIM), PARAMETER :: AIRSCHAN(1:M__NumAIRSChans) = &
 & (/1,6,7,10,11,15,16,17,20,21,22,24,27,28,30,36,39,40,42,51,52,54,55,56,&
 & 59,62,63,68,69,71,72,73,74,75,76,77,78,79,80,82,83,84,86,92,93,98,99,&
 & 101,104,105,108,110,111,113,116,117,123,124,128,129,138,139,144,145,&
 & 150,151,156,157,159,162,165,168,169,170,172,173,174,175,177,179,180,&
 & 182,185,186,190,192,193,198,201,204,207,210,213,215,216,218,221,224,&
 & 226,227,232,239,248,250,251,252,253,256,257,261,262,267,272,295,299,&
 & 300,305,308,309,310,318,321,325,333,338,355,362,375,453,475,484,497,&
 & 528,587,672,787,791,843,870,914,950,1003,1012,1019,1024,1030,1038,1048,&
 & 1069,1079,1082,1083,1088,1090,1092,1095,1104,1111,1115,1116,1119,1120,&
 & 1123,1130,1138,1142,1178,1199,1206,1221,1237,1252,1260,1263,1266,1278,&
 & 1285,1290,1301,1304,1329,1371,1382,1400,1401,1402,1403,1415,1424,1449,&
 & 1455,1466,1471,1477,1479,1488,1500,1519,1520,1538,1545,1565,1574,1583,&
 & 1593,1614,1627,1636,1644,1652,1669,1674,1681,1694,1708,1717,1723,1740,&
 & 1748,1751,1756,1763,1766,1771,1777,1780,1783,1794,1800,1803,1806,1812,&
 & 1826,1843,1852,1865,1866,1867,1868,1869,1872,1873,1875,1876,1877,1881,&
 & 1882,1883,1884,1897,1901,1911,1917,1918,1921,1923,1924,1928,1937,1938,&
 & 1939,1941,1946,1947,1948,1958,1971,1973,1988,1995,2084,2085,2097,2098,&
 & 2099,2100,2101,2103,2104,2106,2107,2108,2109,2110,2111,2112,2113,2114,&
 & 2115,2116,2117,2118,2119,2120,2121,2122,2123,2128,2134,2141,2145,2149,&
 & 2153,2164,2189,2197,2209,2226,2234,2280,2318,2321,2325,2328,2333,2339,&
 & 2348,2353,2355,2357,2363,2370,2371,2377/)  

INTEGER(KIND=JPIM) :: AIRSCHAN_BACK(M__MaxAIRSChans)

INTEGER(KIND=JPIM), ALLOCATABLE :: IASICHAN(:)

INTEGER(KIND=JPIM) :: IASICHAN_BACK(M__MaxIASIChans)

! Define for which sensors radiance calculations should be performed in hretr
LOGICAL :: LRTCALC_HRETR(0:MXSENSOR)= &
 & (/.TRUE. ,.TRUE. ,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.TRUE. ,.TRUE. ,.FALSE.,.FALSE.,&
 &   .TRUE. ,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .TRUE. ,.FALSE.,.FALSE.,.FALSE.,.TRUE. ,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.TRUE.,.FALSE./)  
! Define for which sensors cloudy radiance calculations should be performed in hretr
LOGICAL :: LCLD_RTCALC_SCREEN(0:MXSENSOR)= &
 & (/.FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE./)
! Define for which sensors cloudy radiance calculations should be performed in the assimilation
! Note: This is not yet working correctly
LOGICAL :: LCLD_RTCALC_ASSIM(0:MXSENSOR)= &
 & (/.FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE.,&
 &   .FALSE.,.FALSE.,.FALSE./)

!     LCO2_DIAG_AIRS: switch for CO2-slicing on AIRS data
LOGICAL :: LCO2_DIAG_AIRS = .FALSE.

! Switch for RTTOV-internal interpolation
LOGICAL  :: LRTTOV_INTERPOL = .TRUE.

! Switch whether a fatal error in RTTOV should trigger an abort 
LOGICAL   :: LABORT_RTTOV_FAILURE = .FALSE.

! Define table for administration of sat-ID, sensor and codetype...
TYPE satgrp_t
INTEGER(KIND=JPIM)         :: bufrid                   ! BUFR Satellite Identifier
INTEGER(KIND=JPIM)         :: codetype                 ! Obs codetype (as in ODB)
INTEGER(KIND=JPIM)         :: sensor                   ! Sensor (as defined above)
INTEGER(KIND=JPIM)         :: satgroup                 ! Satellite data group number
INTEGER(KIND=JPIM)         :: retrtype                 ! Retrieval type
INTEGER(KIND=JPIM)         :: rtid                     ! SatID used by RTTOV
INTEGER(KIND=JPIM)         :: rtseries                 ! Series (as defined above)
INTEGER(KIND=JPIM)         :: rtsubtype                ! Subtype (see getsatid)
INTEGER(KIND=JPIM)         :: rtcoef_pos               ! Pointer to RT-coefficients
LOGICAL                    :: cld_rtcalc_screen        ! Perform cloudy RT computations in screening
LOGICAL                    :: cld_rtcalc_assim         ! Perform cloudy RT computations in the assimilation
INTEGER(KIND=JPIM)         :: rtid_ix                  ! SatID index (for getbias)
INTEGER(KIND=JPIM)         :: rtseries_ix              ! Series index (for getbias)
INTEGER(KIND=JPIM)         :: rtsubtype_ix             ! Subtype index (for getbias)
INTEGER(KIND=JPIM)         :: nrtchannels              ! No. of channels, this sensor
INTEGER(KIND=JPIM),POINTER :: rtchan_list(:)           ! Chans that RTTOV can compute
INTEGER(KIND=JPIM),POINTER :: varbc_ix(:)              ! Bias parameter table index
END TYPE satgrp_t
TYPE(satgrp_t), ALLOCATABLE :: SATGRP_TABLE(:)         ! Sat data group admin table
INTEGER(KIND=JPIM) :: NSATGRP                          ! Size of SATGRP_TABLE

! Define table for administration of SATOB sat-ID, codetype...
TYPE satobgrp_t
INTEGER(KIND=JPIM)         :: bufrid                   ! BUFR Satellite Identifier
INTEGER(KIND=JPIM)         :: codetype                 ! Obs codetype (as in ODB)
INTEGER(KIND=JPIM)         :: method                   ! Computational method
INTEGER(KIND=JPIM)         :: satgroup                 ! Satellite data group number
INTEGER(KIND=JPIM)         :: ipcor                    ! Index of correction model
LOGICAL                    :: asym_fgcheck             ! Asymmetric FG-check on/off
CHARACTER(LEN=32)          :: obs_oper                 ! Obs operator method 
REAL(KIND=JPRB)            :: sigma                    ! Sigma used for obs operator [Pa]
REAL(KIND=JPRB)            :: zrejmod_factor           ! Factor for fg check tuning
REAL(KIND=JPRB)            :: oerr_factor              ! Factor for obs error tuning

INTEGER(KIND=JPIM)         :: rtid                     ! SatID used by RTTOV
INTEGER(KIND=JPIM)         :: rtseries                 ! Series (as defined above)
INTEGER(KIND=JPIM)         :: rtsubtype                ! Subtype (see getsatid)
INTEGER(KIND=JPIM)         :: rtcoef_pos               ! Pointer to RT-coefficients
END TYPE satobgrp_t
TYPE(satobgrp_t), ALLOCATABLE :: SATOBGRP_TABLE(:)     ! SATOB data group table
INTEGER(KIND=JPIM) :: NSATOBGRP                        ! Size of SATGRP_TABLE

! Define table for administration of sat-ID, sensor and codetype...
TYPE satreo3grp_t
INTEGER(KIND=JPIM)         :: bufrid                   ! BUFR Satellite Identifier
INTEGER(KIND=JPIM)         :: sensor                   ! Sensor (as defined above)
INTEGER(KIND=JPIM)         :: varno                    ! Variable number
INTEGER(KIND=JPIM)         :: satgroup                 ! Satellite data group number
INTEGER(KIND=JPIM)         :: retrtype                 ! Retrieval type
INTEGER(KIND=JPIM)         :: prodtype                 ! Product type
END TYPE satreo3grp_t
TYPE(satreo3grp_t), ALLOCATABLE :: SATREO3GRP_TABLE(:)         ! Sat data group admin table
INTEGER(KIND=JPIM) :: NSATREO3GRP                          ! Size of SATREO3GRP_TABLE


!     ------------------------------------------------------------------

END MODULE YOMTVRAD
