MODULE YOMVAREPS

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE
SAVE

!     -----------------------------------------------------------------
!*    ** *YOMVAREPS* - CONTROL PARAMETERS FOR VAREPS CONFIGURATION
!     -----------------------------------------------------------------

INTEGER(KIND=JPIM), PARAMETER :: NAFVAREPSMAX=31

INTEGER(KIND=JPIM) :: NRES_MA
INTEGER(KIND=JPIM) :: NRES_MB
INTEGER(KIND=JPIM) :: NFCHO_TRUNC_INI
INTEGER(KIND=JPIM) :: NFCLENGTH_INI
INTEGER(KIND=JPIM) :: NST_FCLENGTH_INI
INTEGER(KIND=JPIM) :: NFCHO_TRUNC
INTEGER(KIND=JPIM) :: NST_TRUNC_INI
INTEGER(KIND=JPIM) :: NST_TRUNC
INTEGER(KIND=JPIM) :: NFCHO_HDIF_TRANS
INTEGER(KIND=JPIM) :: NST_HDIF_TRANS
INTEGER(KIND=JPIM) :: NST_HDIF_INI
INTEGER(KIND=JPIM) :: NAFVAREPS
INTEGER(KIND=JPIM) :: NAFVAREPSGC(NAFVAREPSMAX)
REAL(KIND=JPRB) :: HDIRVOR_MA
REAL(KIND=JPRB) :: HDIRDIV_MA
REAL(KIND=JPRB) :: HDIRT_MA
REAL(KIND=JPRB) :: HDIRQ_MA
REAL(KIND=JPRB) :: HDIRVOR_MB
REAL(KIND=JPRB) :: HDIRDIV_MB
REAL(KIND=JPRB) :: HDIRT_MB
REAL(KIND=JPRB) :: HDIRQ_MB
LOGICAL :: LVAREPS
LOGICAL :: L_INC_HDIF

!     ------------------------------------------------------------------

!*     Variables for controlling VAREPS

!      Original

!      Author: Roberto Buizza - 10 March 2005

!      LVAREPS          : .T. when running with variable resolution
!      L_INC_HDIF       : .T. to increase horizontal diffusion linearly from the default values 
!                         of NRES_MA resolution to the default values of NRES_MB resolution
!      NFCHO_TRUNC_INI  : forecast step used to define the ICs (ie NFCHO_TRUNC of previous VAREPS LEG)
!      NFCLENGTH_INI    : length of the forecast of the previous VAREPS LEG
!      NST_FCLENGTH_INI : NFCLENGTH_INI in number of time steps of current LEG
!      NFCHO_TRUNC      : forecast hour when resolution is reduced from NRES_MA to NRES_MB
!      NST_TRUNC_INI    : NFCHO_TRUNC_INI in number of time steps
!      NST_TRUNC        : NFCHO_TRUNC in number of time steps of current LEG                     
!      NFCHO_HDIF_TRANS : number of hours during which hor-diff is increased
!      NST_HDIF_TRANS   : number of steps during which hor-diff is increased
!      NST_HDIF_INI     : initial time step when hor-diff is increased
!      NAFVAREPSMAX     : maximum number of accumulated fields to be re-set if VAREPS LEG gt 1
!      NAFVAREPS        : number of accumulated fields to be re-set if VAREPS LEG gt 1
!      NAFVAREPSGC      : grid codes of accumulated fields to be re-set
!      NRES_MA          : spectral truncation of first part of integration (model_A)
!      NRES_MB          : spectral truncation of second part of integration (model_B)
!      HDIRVOR_MA       : initial hor diff model_A
!      HDIRDIV_MA       : initial hor diff model_A
!      HDIRT_MA         : initial hor diff model_A
!      HDIRQ_MA         : initial hor diff model_A
!      HDIRVOR_MB       : final hor diff model_B to which hdif-model_A tend to
!      HDIRDIV_MB       : final hor diff model_B to which hdif-model_A tend to
!      HDIRT_MB         : final hor diff model_B to which hdif-model_A tend to
!      HDIRQ_MB         : final hor diff model_B to which hdif-model_A tend to

!      Schematic of a 3-LEG system with overlap periods:

!      LEG1:   !................!..............!
!              ^                ^              ^
!        NFCHO_TRUNC_INI   NFCHO_TRUNC       FCLENGTH
!      LEG2:                    !..............!..................!........!
!                               ^              ^                  ^        ^
!         in hours .. NFCHO_TRUNC_INI     NFCLENGTH_INI    NFCHO_TRUNC  FCLENGTH      
!         in num t-steps ..  (NST=0)   (NST_NFCLENGTH_INI)  (NST_TRUNC)
!      LEG3:                                                      !........!................!
!                                                                 ^        ^                ^
!         in hours ..                                  NFCHO_TRUNC_INI NFCLENGTH_INI      FCLENGTH   
!         in num t-steps ..                               (NST=0)     (NST_NFCLENGTH_INI)  

!      Schematic of a 3-LEG system T399(0-7d)+T255(6d-15d)+T159(14d-30d) with 1-day overlap periods
!      between each legs (ie the T255 starts at 6d and not 7d, and the T159 starts at 14d)

!      LEG1:   !0............ 144 ......... 168!
!      LEG2:              !144(144+0) .. 168(144+24) .... 336(144+192) .. 360(144+216)!
!      LEG3:                                             !336(336+0)   .. 360(336+24) .. 720(336+384)!
!              ^                ^              ^                  ^        ^                ^

!      Then the T-variables should be set:
!      - LEG1 (0-7d):
!        - NFCHO_TRUNC_INI=0
!        - NFCLENGTH_INI  =0  
!        - NFCHO_TRUNC    =144 (=6d)
!        - FCLENGTH       =168 (=7d)
!      - LEG2 (6d-15d):
!        - NFCHO_TRUNC_INI=144 (=0+144=NFCHO_TRUNC_INI(LEG1)+NFCHO_TRUNC(LEG1))
!        - NFCLENGTH_INI  =168 (=0+168=NFCHO_TRUNC_INI(LEG1)+FCLENGTH(LEG1))
!        - NFCHO_TRUNC    =192 (=8d=14d-6d)
!        - FCLENGTH       =216 (=9d=15d-6d)
!      - LEG3 (14d-30d):
!        - NFCHO_TRUNC_INI=336 (=144+192=NFCHO_TRUNC_INI(LEG2)+NFCHO_TRUNC(LEG2))
!        - NFCLENGTH_INI  =360 (=144+216=NFCHO_TRUNC_INI(LEG2)+FCLENGTH(LEG2))
!        - NFCHO_TRUNC    =0   (since there is no other leg)
!        - FCLENGTH       =384 (=16d)


END MODULE YOMVAREPS
