MODULE YOMVNMB

USE PARKIND1, ONLY : JPIM
USE PARCMA,   ONLY : JPMXVAR

IMPLICIT NONE

SAVE

!*     YOMVNMB - CMA VARIABLE NUMBERING

!        D. VASILJEVIC   ECMWF     15/09/94

!     NAME        TYPE                   MEANING
!     ----        ----                   -------
!     NOVARIB       I     NO. OF VARIABLES
!     NVNUMAX       I     MAXIMUM VARIABLE NUMBER
!     NVNUMB(  1)   I     UPPER AIR U COMPONENT
!     NVNUMB(  2)   I     UPPER AIR V COMPONENT
!     NVNUMB(  3)   I     GEOPOTENTIAL
!     NVNUMB(  4)   I     THICKNESS
!     NVNUMB(  5)   I     UPPER AIR REL. HUMIDITY
!     NVNUMB(  6)   I     PWC
!     NVNUMB(  7)   I     2M REL. HUMIDITY
!     NVNUMB(  8)   I     UPPER AIR TEMPERATURE
!     NVNUMB(  9)   I     UPPER AIR DEW POINT
!     NVNUMB( 10)   I     2M TEMPERATURE
!     NVNUMB( 11)   I     2M DEW POINT
!     NVNUMB( 12)   I     SURFACE TEMPERATURE
!     NVNUMB( 13)   I     PRESSURE TENDENCY
!     NVNUMB( 14)   I     PAST WEATHER (W)
!     NVNUMB( 15)   I     PRESENT WEATHER (WW)
!     NVNUMB( 16)   I     VISIBILITY
!     NVNUMB( 17)   I     TYPE OF HIGH CLOUDS (Ch)
!     NVNUMB( 18)   I     TYPE OF MIDDLE CLOUDS (Cm)
!     NVNUMB( 19)   I     TYPE OF LOW CLOUDS (Cl)
!     NVNUMB( 20)   I     CLOUD BASE HEIGHT (Nh)
!     NVNUMB( 21)   I     LOW CLOUD AMOUNT (N)
!     NVNUMB( 22)   I     ADDITIONAL CLOUD GROUP HEIGHT (hh)
!     NVNUMB( 23)   I     ADDITIONAL CLOUD GROUP TYPE (C)
!     NVNUMB( 24)   I     ADDITIONAL CLOUD GROUP AMOUNT (Ns)
!     NVNUMB( 25)   I     SNOW DEPTH
!     NVNUMB( 26)   I     STATE OF GROUND (E)
!     NVNUMB( 27)   I     GROUND TEMPERATURE (TgTg)
!     NVNUMB( 28)   I     SPECIAL PHENOMENA (SpSp)
!     NVNUMB( 29)   I     SPECIAL PHENOMENA (SpSp)
!     NVNUMB( 30)   I     ICE CODE TYPE (Rs)
!     NVNUMB( 31)   I     ICE THICKNESS (EsEs)
!     NVNUMB( 32)   I     ICE (Is)
!     NVNUMB( 33)   I     ORIGINAL TIME PERIOD OF RAIN OBS. (TrTr)
!     NVNUMB( 34)   I     6HR RAIN (LIQUID PART)
!     NVNUMB( 35)   I     MAX. TEMPERATURE (JJ)
!     NVNUMB( 36)   I     SHIP SPEED (Vs)
!     NVNUMB( 37)   I     SHIP DIRECTION (Ds)
!     NVNUMB( 38)   I     WAVE HEIGHT
!     NVNUMB( 39)   I     WAVE PERIOD
!     NVNUMB( 40)   I     WAVE DIRECTION
!     NVNUMB( 41)   I     GENERAL CLOUD GROUP
!     NVNUMB( 42)   I     REL. HUMIDITY FROM LOW CLOUDS
!     NVNUMB( 43)   I     REL. HUMIDITY FROM MIDDLE CLOUDS
!     NVNUMB( 44)   I     REL. HUMIDITY FROM HIGH CLOUDS
!     NVNUMB( 45)   I     TOTAL AMOUNT OF CLOUDS
!     NVNUMB( 46)   I     6HR SNOWFALL (SOLID PART OF RAIN)
!     NVNUMB( 47)   I     SURFACE PRESSURE
!     NVNUMB( 48)   I     WIND DIRECTION
!     NVNUMB( 49)   I     WIND FORCE
!     NVNUMB( 50)   I     BRIGHTNESS TEMPERATURE
!     NVNUMB( 51)   I     RAW RADIANCE
!     NVNUMB( 52)   I     CLOUD AMOUNT FROM SATELLITE
!     NVNUMB( 53)   I     SIGMA 0
!     NVNUMB( 54)   I     WIND SHEAR (DU)
!     NVNUMB( 55)   I     WIND SHEAR (DV)
!     NVNUMB( 56)   I     10M U COMPONENT
!     NVNUMB( 57)   I     10M V COMPONENT
!     NVNUMB( 58)   I     LAYER REL. HUMIDITY
!     NVNUMB( 59)   I     AUX. VARIABLE
!     NVNUMB( 60)   I     CLOUD LIQUID WATER
!     NVNUMB( 61)   I     AMBIGUOUS V COMPONENT
!     NVNUMB( 62)   I     AMBIGUOUS U COMPONENT
!     NVNUMB( 63)   I     SPECIFIC HUMIDITY (Q)
!     NVNUMB( 64)   I     AMBIGUOUS WIND DIRECTION
!     NVNUMB( 65)   I     AMBIGUOUS WIND SPEED
!     NVNUMB( 66)   I     VERTICAL SPEED
!     NVNUMB( 67)   I     VIRTUAL TEMPERATURE
!     NVNUMB( 68)   I     LAYER OZONE
!     NVNUMB( 69)   I     HEIGHT
!     NVNUMB( 70)   I     CLW?? In hvnmtlt.F90?
!     NVNUMB( 74)   I     RADAR REFLECTIVITY
!     NVNUMB( 77)   I     HORIZONTAL LINE-OF-SIGHT WIND COMPONENT
!     NVNUMB( 78)   I     AEROSOL OPTICAL DEPTH
!     NVNUMB( 79)   I     Limb radiances
!     NVNUMB( 80)   I     GRG1: Nitrogen Dioxide
!     NVNUMB( 81)   I     GRG2: Sulphur Dioxide
!     NVNUMB( 82)   I     GRG3: Carbon Monoxide
!     NVNUMB( 83)   I     GRG4: Formaldehyde
!     NVNUMB( 84)   I     GRG5: Ozone
!     NVNUMB( 85)   I     Cloud optical depth  (COD) 
!     NVNUMB( 86)   I     Ratio of fine mode to total aerosol optical depth at 0.55 microns (RAO) 
!     NVNUMB( 87)   I     Aerosol optical depth multi-channel (ODA) 
!     NVNUMB( 88)   I     Aerosol reflectance multi-channel (RFA) 
!     NVNUMB( 89)   I     NORMALIZED SOIL MOISTURE (0-100%) 
!     NVNUMB( 90)   I     SOIL MOISTURE 
!     NVNUMB( 91)   I     Phase of aircraft flight (from BUFR-code 008004/BoM)
!     NVNUMB( 92)   I     Height assignment method (from BUFR-code 002231/BoM)
!     NVNUMB( 93)   I     RADAR RADIAL WIND
!     NVNUMB( 94)   I     GHG1: CO2
!     NVNUMB( 95)   I     GHG2: CH4
!     NVNUMB( 96)   I     GHG3: N2O
!     NVNUMB( 97)   I     Real part brightness temperatures in L-band 
!     NVNUMB( 98)   I     Imaginary part brightness temperatures in L-band  


INTEGER(KIND=JPIM) :: NOVARIB  ,NVNUMAX
INTEGER(KIND=JPIM) :: NVNUMB(JPMXVAR)

!-----------------------------------------------------------------------

END MODULE YOMVNMB
