MODULE YOMWFPB

! Indexes and weights for horizontal interpolations:

! --- DM distribution for departure geometry.

! MLAT  : index of the nearest northern latitude
! MLON  : index of the pt. 7 (prev. nearest north-western longitude = 1)
! MLOS  : index of the pt. 9 (prev. nearest south-western longitude = 3)
! MLONN : index of the pt. 13 (prev. nearest northern longitude from MLON = 5)
! MLOSS : index of the pt. 15 (prev. nearest southern longitude from MLOS = 11)

!       Numbering of neighbouring points on the input grid :

!                     13       5       6

!          MLAT ->     7       1       2       8
!                                  x
!                      9       3       4      10

!                     15      11      12

! MWIC_DEP : indicator for isolated lake or island created
!         =0 : at least ine surrounding point of the same nature
!         =1 : isolated lake point
!         =2 : isolated island point

! WSTD04 : weights for bilinear interpolations, without mask
! WLAN04 : weights for bilinear interpolations, with land-sea mask
! WSEA04 : weights for bilinear interpolations, with sea mask

! WSTD12 : weights for quadratic interpolations, without mask
! WLAN12 : weights for quadratic interpolations, with land-sea mask
! WSEA12 : weights for quadratic interpolations, with sea mask

! --- DM distribution for arrival geometry.
! MWIC   : indicator for isolated lake or island created
!         =0 : at least ine surrounding point of the same nature
!         =1 : isolated lake point
!         =2 : isolated island point

!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE


INTEGER(KIND=JPIM), ALLOCATABLE :: MLAT(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MLON(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MLOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MLONN(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MLOSS(:)

INTEGER(KIND=JPIM), ALLOCATABLE :: MWIC_DEP(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MWIC(:)

REAL(KIND=JPRB), ALLOCATABLE :: WSTD04(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: WLAN04(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: WSEA04(:,:)

REAL(KIND=JPRB), ALLOCATABLE :: WSTD12(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: WLAN12(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: WSEA12(:,:)

!------------------------------------------------------------------------------

END MODULE YOMWFPB
