SUBROUTINE MWAVE_GET(PMWAVE_OBS_BUF,KMWAVE_BUF,PMWAVE_ZEN_BUF,PMWAVE_AZI_BUF)

!**** *MWAVE_GET* - 4D-VAR RAIN ASSIMILATION

!     P. Bauer, P. Lopez, D. Salmond   ECMWF    Aug 2006

!     PURPOSE.
!     -------
!     GET SSM/I OBS FROM ODB TO GRID-POINT SPACE

!**   INTERFACE.
!**   ---------
!**   *CALL* * MWAVE_GET()*  (FROM GP_MODEL) 

!        Explicit arguments :


!     AUTHOR: Deborah Salmond
!     DATE  : 01-08-2006

!     Modifications :
!     11-Apr-2008   Alan Geer    Multiple sensors
!     05-Mar-2009   W. Bell      Add azimuthal angle for Windsat
!     01-Jun-2009   Alan Geer    Passive monitoring enabled; per channel status
!     01-Nov-2009   Anne Fouilloux  Obs to grid point mapping now in ODB
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMGEM    ,ONLY : NGPTOT
USE YOMECTAB  ,ONLY : NTSLOTNOS, NTSLOTNOE
USE YOMANCS  , ONLY : RMDI
USE PARMWAVE , ONLY : NMAX_CHAN, RMISSING_ANGLE
USE YOMMWAVE , ONLY : N_SATSENS, MWAVE_ID, MWAVE_ACTIVE, MWAVE_PASSIVE, MWAVE_NO_OBS
 
IMPLICIT NONE

REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_OBS_BUF (NGPTOT,NMAX_CHAN,N_SATSENS)
INTEGER(KIND=JPIM) ,INTENT(INOUT) :: KMWAVE_BUF     (NGPTOT,NMAX_CHAN,N_SATSENS)
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_ZEN_BUF (NGPTOT,N_SATSENS)
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_AZI_BUF (NGPTOT,N_SATSENS)

INTEGER(KIND=JPIM) :: ILEN,ICH,JOBS,ICHAN
INTEGER(KIND=JPIM) :: JBODY, INFO(2), IRET, IBODY, JROF, ISAT
INTEGER(KIND=JPIM) :: INSTA, IDSTA(4)
INTEGER(KIND=JPIM) :: ICMBDY
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZINFO(0)

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "ngedsta.intfb.h"

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('MWAVE_GET',0,ZHOOK_HANDLE)

INFO(1)=NTSLOTNOS
INFO(2)=NTSLOTNOE

CALL GETDB('MWAVE_GET_SSMI',0, ILEN, INFO, 2, ZINFO, 0, &
   & -1, -1, -1, -1, -1, -1)

PMWAVE_OBS_BUF(:,:,:) = RMDI
PMWAVE_ZEN_BUF(:,:) = RMDI
PMWAVE_AZI_BUF(:,:) = RMDI
KMWAVE_BUF(:,:,:) = IBSET(0, MWAVE_NO_OBS)

DO JOBS=1,ILEN

  ! Find satellite index in our local table
  DO ISAT=1,N_SATSENS
    IF(ROBHDR(JOBS,MDB_SATID_AT_SAT) == MWAVE_ID(ISAT) % BUFR_ID ) EXIT
  ENDDO

  ! Grid point index
  JROF = ROBHDR(JOBS,MDB_GP_NUMBER)

  ! Sateliite zenith and azimuth angles
  PMWAVE_ZEN_BUF(JROF,ISAT) = ROBHDR(JOBS,MDB_ZENITH_AT_SAT)
  IF (PMWAVE_ZEN_BUF(JROF,ISAT) < 0.0_JPRB) PMWAVE_ZEN_BUF(JROF,ISAT) = RMISSING_ANGLE
  PMWAVE_AZI_BUF(JROF,ISAT) = ROBHDR(JOBS,MDB_AZIMUTH_AT_SAT)
  IF (PMWAVE_AZI_BUF(JROF,ISAT) < 0.0_JPRB) PMWAVE_AZI_BUF(JROF,ISAT) = RMISSING_ANGLE

  ICMBDY=MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  DO JBODY=1,ICMBDY

    IBODY=MLNKH2B(JOBS)+(JBODY-1)
    ICHAN=ROBODY(IBODY,MDBPPP)

    DO ICH=1,NMAX_CHAN
      IF(ICHAN == ICH)THEN

        ! Observed brightness temperatures
        PMWAVE_OBS_BUF(JROF,ICH,ISAT) = ROBODY(IBODY,MDBVAR)

        ! Initialise status flag 
        KMWAVE_BUF(JROF,ICH,ISAT) = IBSET(0,MWAVE_ACTIVE)

        ! Check for passive monitoring data and flag (in actual fact,
        ! because blacklisting comes at end of FG traj, this is pointless here).
        CALL NGEDSTA(INT(ROBODY(IBODY,MDBDSTA)),INSTA,IDSTA)         
        IF (IDSTA(2) == 1) KMWAVE_BUF(JROF,ICH,ISAT) = &
          &     IBSET( KMWAVE_BUF(JROF,ICH,ISAT), MWAVE_PASSIVE )     

        EXIT
      ENDIF
    ENDDO
  ENDDO

ENDDO

CALL PUTDB('MWAVE_GET_SSMI', 0, IRET, INFO, 0, ZINFO, 0)

IF (LHOOK) CALL DR_HOOK('MWAVE_GET',1,ZHOOK_HANDLE)

END SUBROUTINE MWAVE_GET
