SUBROUTINE MWAVE_GET_AD(PMWAVE_OBS_BUF,PMWAVE_ZEN_BUF, PMWAVE_AZI_BUF, PMWAVE_OERR_BUF,PMWAVE_PERS1_BUF,KMWAVE_BUF)

!**** *MWAVE_GET_AD* - 4D-VAR RAIN ASSIMILATION

!     P. Bauer, P. Lopez, D. Salmond   ECMWF    Aug 2006

!     PURPOSE.
!     -------
!     GET MDB_TBVALUEAD FROM ODB TO GRID-POINT SPACE

!**   INTERFACE.
!**   ---------
!**   *CALL* * MWAVE_GET_AD()*  (FROM GP_MODEL_AD)

!        Explicit arguments :


!     AUTHOR: Deborah Salmond
!     DATE  : 01-08-2006

!     Modifications :
!     25-Apr-2008   Alan Geer    Multiple sensor extensions
!     09-Mar-2009   W. Bell      Add azimuth angle for windsat-like sensors
!     10-Jun-2009   Alan Geer    Per-channel status flag
!     01-Nov-2009   Anne Fouilloux  Obs to grid point mapping now in ODB
!     16-Dec-2009   F. Karbou	 Allow surface emissivity handling
!     15-Feb-2010   Alan Geer    Make adjoint sum bit-reproducible
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMGEM    ,ONLY : NGPTOT
USE YOMECTAB  ,ONLY : NTSLOTNOS, NTSLOTNOE
USE YOMANCS  , ONLY : RMDI
USE PARMWAVE , ONLY : NMAX_CHAN, RMISSING_ANGLE
USE YOMMWAVE , ONLY : N_SATSENS, MWAVE_ID, MWAVE_NO_OBS, &
  & MWAVE_CONTAMINATED, MWAVE_OK

IMPLICIT NONE

REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_OBS_BUF(NGPTOT,NMAX_CHAN,N_SATSENS)
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_ZEN_BUF(NGPTOT,N_SATSENS)
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_AZI_BUF(NGPTOT,N_SATSENS)
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_OERR_BUF(NGPTOT,NMAX_CHAN,N_SATSENS)
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PMWAVE_PERS1_BUF(NGPTOT,NMAX_CHAN,N_SATSENS)
INTEGER(KIND=JPIM) ,INTENT(INOUT) :: KMWAVE_BUF(NGPTOT,NMAX_CHAN,N_SATSENS)

INTEGER(KIND=JPIM) :: IMWAVE_CNT(NGPTOT,NMAX_CHAN,N_SATSENS)
INTEGER(KIND=JPIM), ALLOCATABLE :: IMWAVE_SEQ(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZMWAVE_OBS(:,:,:,:)
INTEGER(KIND=JPIM) :: ILEN,ICH,JOBS,ICHAN,IMAXCNT,IMAXSEQ(1)
INTEGER(KIND=JPIM) :: JBODY, INFO(2), IRET, IBODY, JROF, ISAT
INTEGER(KIND=JPIM) :: ICMBDY, ICNT
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZINFO(0)

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('MWAVE_GET_AD',0,ZHOOK_HANDLE)

IF(NTSLOTNOS <= NTSLOTNOE)THEN
  INFO(1)=NTSLOTNOS
  INFO(2)=NTSLOTNOE
ELSE
  INFO(1)=NTSLOTNOE
  INFO(2)=NTSLOTNOS
ENDIF

CALL GETDB('MWAVE_GET_SSMI',0, ILEN, INFO, 2, ZINFO, 0, &
 & -1, -1, -1, -1, -1, -1)

PMWAVE_OBS_BUF(:,:,:) = 0.0_JPRB
PMWAVE_OERR_BUF(:,:,:) = RMDI
PMWAVE_PERS1_BUF(:,:,:) = RMDI
PMWAVE_ZEN_BUF(:,:) = RMDI
PMWAVE_AZI_BUF(:,:) = RMDI
KMWAVE_BUF(:,:,:) = IBSET(0, MWAVE_NO_OBS)
IMWAVE_CNT(:,:,:) = 0

! First pass through data
DO JOBS=1,ILEN

  ! Find satellite index in our local table
  DO ISAT=1,N_SATSENS
    IF(ROBHDR(JOBS,MDB_SATID_AT_SAT) == MWAVE_ID(ISAT) % BUFR_ID ) EXIT
  ENDDO

  ! Grid point index
  JROF = ROBHDR(JOBS,MDB_GP_NUMBER)

  ! Sateliite zenith and azimuth angles. Take maximum in case of more than one
  ! obs per grid point (max is bit-reproducible for any order of data)
  PMWAVE_ZEN_BUF(JROF,ISAT) = MAX( PMWAVE_ZEN_BUF(JROF,ISAT), ROBHDR(JOBS,MDB_ZENITH_AT_SAT))
  IF (PMWAVE_ZEN_BUF(JROF,ISAT) < 0.0_JPRB) PMWAVE_ZEN_BUF(JROF,ISAT) = RMISSING_ANGLE
  PMWAVE_AZI_BUF(JROF,ISAT) = MAX( PMWAVE_AZI_BUF(JROF,ISAT), ROBHDR(JOBS,MDB_AZIMUTH_AT_SAT))
  IF (PMWAVE_AZI_BUF(JROF,ISAT) < 0.0_JPRB) PMWAVE_AZI_BUF(JROF,ISAT) = RMISSING_ANGLE

  ICMBDY=MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  DO JBODY=1,ICMBDY

    IBODY=MLNKH2B(JOBS)+(JBODY-1)
    ICHAN=ROBODY(IBODY,MDBPPP)

    DO ICH=1,NMAX_CHAN
      IF(ICHAN == ICH)THEN

        ! Observation error - take maximum in the case of more than one obs per grid point
        PMWAVE_OERR_BUF(JROF,ICH,ISAT) = &
          & MAX( PMWAVE_OERR_BUF(JROF,ICH,ISAT), ROBODY(IBODY,MDBFOE))

        ! Surface emissivity 
        PMWAVE_PERS1_BUF(JROF,ICH,ISAT) = &
          & MAX( PMWAVE_PERS1_BUF(JROF,ICH,ISAT), ROBODY(IBODY,MDB_MWEMIS_SAT2))

        ! Status flag
        IF ( BTEST(KMWAVE_BUF(JROF,ICH,ISAT) , MWAVE_NO_OBS) ) THEN

          ! This is the first observation associated with this grid point
          IMWAVE_CNT(JROF,ICH,ISAT) = 1
          KMWAVE_BUF(JROF,ICH,ISAT) = ROBODY(IBODY,MDB_DATUM_TBFLAG)

        ELSE

          ! There is more than one obs for this grid point
          IMWAVE_CNT(JROF,ICH,ISAT) = IMWAVE_CNT(JROF,ICH,ISAT) + 1
         
          ! Rejected observations will be "contaminating" unrejected ones, so
          ! flag this if it happens.
          IF (   (.NOT. MWAVE_OK( KMWAVE_BUF(JROF,ICH,ISAT))) &
           & .OR.(.NOT. MWAVE_OK( INT(ROBODY(IBODY,MDB_DATUM_TBFLAG)))) )  THEN   
            KMWAVE_BUF(JROF,ICH,ISAT) = IBSET( KMWAVE_BUF(JROF,ICH,ISAT), MWAVE_CONTAMINATED )
          ENDIF

          ! Merge together the status flags.
          KMWAVE_BUF(JROF,ICH,ISAT) = IOR( KMWAVE_BUF(JROF,ICH,ISAT), INT(ROBODY(IBODY,MDB_DATUM_TBFLAG)) )

        ENDIF

        EXIT
      ENDIF
    ENDDO
  ENDDO

ENDDO

! Allocate arrays for computing bit-reproducible adjoint sums
IMAXCNT = MAXVAL(IMWAVE_CNT)
IMWAVE_CNT(:,:,:) = 0
ALLOCATE( IMWAVE_SEQ(NGPTOT,NMAX_CHAN,N_SATSENS,IMAXCNT) ) 
ALLOCATE( ZMWAVE_OBS(NGPTOT,NMAX_CHAN,N_SATSENS,IMAXCNT) ) 

! Second pass through data - store info
DO JOBS=1,ILEN

  ! Find satellite index in our local table
  DO ISAT=1,N_SATSENS
    IF(ROBHDR(JOBS,MDB_SATID_AT_SAT) == MWAVE_ID(ISAT) % BUFR_ID ) EXIT
  ENDDO

  ! Grid point index
  JROF = ROBHDR(JOBS,MDB_GP_NUMBER)

  ICMBDY=MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  DO JBODY=1,ICMBDY

    IBODY=MLNKH2B(JOBS)+(JBODY-1)
    ICHAN=ROBODY(IBODY,MDBPPP)

    DO ICH=1,NMAX_CHAN
      IF(ICHAN == ICH)THEN

        IMWAVE_CNT(JROF,ICH,ISAT) = IMWAVE_CNT(JROF,ICH,ISAT) + 1
        ZMWAVE_OBS(JROF,ICH,ISAT,IMWAVE_CNT(JROF,ICH,ISAT)) = ROBODY(IBODY,MDB_TBVALUEAD)
        IMWAVE_SEQ(JROF,ICH,ISAT,IMWAVE_CNT(JROF,ICH,ISAT)) = INT(ROBODY(IBODY,MDBONM))

        EXIT
      ENDIF
    ENDDO
  ENDDO

ENDDO

! Compute bit-reproducible adjoint sums, ordering by ODB sequence number. 
! Works for sequence numbers in range 0 - HUGE(IMWAVE_SEQ).
DO JROF = 1,NGPTOT
  DO ICH = 1,NMAX_CHAN
    DO ISAT = 1,N_SATSENS
      DO ICNT = 1,IMWAVE_CNT(JROF,ICH,ISAT)

        IMAXSEQ = MAXLOC(IMWAVE_SEQ(JROF,ICH,ISAT,1:IMWAVE_CNT(JROF,ICH,ISAT)))
        IMWAVE_SEQ(JROF,ICH,ISAT,IMAXSEQ(1)) = -1

        PMWAVE_OBS_BUF(JROF,ICH,ISAT) = PMWAVE_OBS_BUF(JROF,ICH,ISAT) + &
          & ZMWAVE_OBS(JROF,ICH,ISAT,IMAXSEQ(1))

      ENDDO
    ENDDO
  ENDDO
ENDDO

DEALLOCATE( IMWAVE_SEQ, ZMWAVE_OBS ) 

CALL PUTDB('MWAVE_GET_SSMI', 0, IRET, INFO, 0, ZINFO, 0)

IF (LHOOK) CALL DR_HOOK('MWAVE_GET_AD',1,ZHOOK_HANDLE)

END SUBROUTINE MWAVE_GET_AD
