subroutine mwave_postproc ( &
 & kdlen,      & ! obs  dimension
 & kmxcount,   & ! data dimension
 & klen,       & ! actual number of obs (profiles)
 & kcount,     & ! actual number of data for each obs
 & kgrp,       & ! satgrp_table index
 & pp,         & ! channel index
 & pomf,       & ! Obs minus first guess (before bias correction) [K]
 & pfg,        & ! First guess [K]
 & pbias,      & ! VarBC bias correction [K]
 & kcloudflag  & ! Cloud and rain identification. See yommwave.F90
 & )

!**** 4D-Var in clouds/precipitation for microwave radiances 

! Postprocessing for diagnostics. Runs in HOP after bias corrections 
! have been calculated.  

! Changes
! -------

! 02-Sept-2008  Alan Geer      Initial version


!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook, dr_hook

!* IFS
use yomtvrad, only: satgrp_table
use yommwave, only: cldbit_cloud, cldbit_rain
use yomancs,  only: rmdi

implicit none
          
!* Interface variables
integer(kind=jpim),intent(in)  :: kdlen, klen, kmxcount, kgrp
integer(kind=jpim),intent(in)  :: kcount    (kdlen) 
real(kind=jprb)   ,intent(in)  :: pp        (kdlen,kmxcount) 
real(kind=jprb)   ,intent(in)  :: pomf      (kdlen,kmxcount) 
real(kind=jprb)   ,intent(in)  :: pfg       (kdlen,kmxcount)
real(kind=jprb)   ,intent(in)  :: pbias     (kdlen,kmxcount) 
integer(kind=jpim),intent(out) :: kcloudflag(kdlen) 

!* Local variables
integer(kind=jpim)           :: jobs, jch, jbody, inchan, isensor
real(kind=jprb), allocatable :: zobs(:), zfgb(:) 
real(kind=jprb)              :: zlwp
logical                      :: llcloud, ll37_obs, llwp_obs, ll37_fg, llwp_fg 
real (kind=jprb)             :: zhook_handle

!* External functions
logical, external :: mwimager_cloud
    
if (lhook) call dr_hook ('MWAVE_POSTPROC',0,zhook_handle)

kcloudflag(:) = 0

inchan  = satgrp_table(kgrp)%nrtchannels
isensor = satgrp_table(kgrp)%sensor
allocate(zobs(inchan), zfgb(inchan))

do jobs=1,klen

  zobs(:) = 0.0_JPRB
  zfgb(:) = 0.0_JPRB
 
  ! Order channels by number 
  do jbody=1,kcount(jobs)
    do jch=1,inchan
      if (nint(pp(jobs,jbody))==jch) then     
        
        if( pfg(jobs,jbody)   /= rmdi .and. pomf(jobs,jbody) /= rmdi .and. &
          & pbias(jobs,jbody) /= rmdi ) then

          ! Observation  
          zobs(jch) = pfg(jobs,jbody) + pomf(jobs,jbody) 

          ! Bias corrected first guess (in contrast to usual where we 
          ! consider that it is the observation that gets corrected).
          zfgb(jch) = pfg(jobs,jbody) + pbias(jobs,jbody)    

        else
          zobs(jch) = rmdi
          zfgb(jch) = rmdi
        endif

        exit
      endif
    enddo
  enddo

  ! Perform simple cloud and rain tests on the brightness temperatures
  llcloud = mwimager_cloud(inchan, zobs, isensor, zlwp, ll37_obs, llwp_obs, .true.) 
  llcloud = mwimager_cloud(inchan, zfgb, isensor, zlwp, ll37_fg,  llwp_fg,  .true.) 
 
  ! Pack a bitfield with the test results (see yommwave.F90)
  if(llwp_obs) kcloudflag(jobs) = ibset(kcloudflag(jobs),cldbit_cloud) 
  if(llwp_fg ) kcloudflag(jobs) = ibset(kcloudflag(jobs),cldbit_cloud + 1) 
  if(ll37_obs) kcloudflag(jobs) = ibset(kcloudflag(jobs),cldbit_rain) 
  if(ll37_fg ) kcloudflag(jobs) = ibset(kcloudflag(jobs),cldbit_rain + 1) 
 
enddo

deallocate(zobs, zfgb)

if (lhook) call dr_hook ('MWAVE_POSTPROC',1,zhook_handle) 

end subroutine mwave_postproc
