subroutine mwave_read_sat_error(yd_mwaveid, kfile_id)

! This IFS subroutine will read in the observation errors 
! for the all-sky assimilation of mwave imagers, for a 
! specific sensor.

! Original Code: 13/10/2006  Chris O'Dell

! Modifications:

! 24-Aug-2007   Alan Geer   Read a channel/sat dependent departure threshold
! 15-Apr-2008   Alan Geer   Modified to use in the 4D-Var code 
! 23-Sep-2008   Alan Geer   Read channel for adjoint sensitivity test
! 10-Jun-2009   Alan Geer   Read parameters for FG error estimation for BGQC

!**** Modules
use parkind1  , only: jpim, jprb
use yomhook   , only: lhook,   dr_hook
use yommwave  , only: mwave_ids
use yomlun    , only: nulerr

implicit none

! In/Out Variables
type (mwave_ids), intent(inout)           :: yd_mwaveid ! struct containing sat id's for a single sensor
Integer(Kind=jpim), Optional, Intent (in) :: kfile_id   ! file logical unit number

! local variables
Character (len=256):: clerrname  ! file name 
integer(kind=jpim) :: ioerr, ifileid, ichannel_number, instrument(3), ichan
real(kind=jprb)    :: zhook_handle

if (lhook) call dr_hook ('MWAVE_READ_SAT_ERROR',0,zhook_handle)

ifileid = 10
if (present(kfile_id)) ifileid = kfile_id

!* Construct file name.
instrument(1) = yd_mwaveid % satseries
instrument(2) = yd_mwaveid % satid 
instrument(3) = yd_mwaveid % sensid
call mwave_get_file_name( ioerr, instrument, clerrname )
If ( ioerr /= 0 ) Then
  write (nulerr,*) ' Error in constructing Satellite Error File Name!'
  call abor1 ('MWAVE_READ_SAT_ERROR: Error making filename')
  IF (LHOOK) CALL DR_HOOK('MWAVE_READ_SAT_ERROR',1,ZHOOK_HANDLE)
  Return
Endif

!* Open file
ioerr = 0
open (ifileid, file = trim(clerrname), status = 'old', iostat = ioerr)
if (ioerr /= 0) then
  write (nulerr,*) ' No observation error file ' // trim(clerrname) // ' found!'
  call abor1 ('MWAVE_READ_SAT_ERROR: mwave observation error file missing')
  if (lhook) call dr_hook ('MWAVE_READ_SAT_ERROR',1,zhook_handle)
  return
endif

!* Read error and FG departure thresholds
read (ifileid,*) ! read header line
read (ifileid,*) yd_mwaveid % nchan_sens ! Channel to use for adjoint sens test
read (ifileid,*) ! read header line
read (ifileid,*) ! read header line
do ichan = 1, yd_mwaveid % nchannels
  read (ifileid,*) ichannel_number, &
     & yd_mwaveid % rtot_err_clr (ichan), &
     & yd_mwaveid % rtot_err_cld (ichan), &
     & yd_mwaveid % rfg_err_clr (ichan), &   ! FG error used in BGQC
     & yd_mwaveid % rp37_clr (ichan), &      ! FG error used in BGQC
     & yd_mwaveid % rp37_cld (ichan), &      ! FG error used in BGQC
     & yd_mwaveid % rlimit_fgc   (ichan)
enddo
close (ifileid)

if (lhook) call dr_hook ('MWAVE_READ_SAT_ERROR',1,zhook_handle)

end subroutine mwave_read_sat_error
