!*    NAMELIST NAMMKODB - MAKE RUN PARAMETERS

!        D. VASILJEVIC   ECMWF     5/1/01

#ifdef DOC

!     NAME                            TYPE                  MEANING
!     ----                            ----                  -------
!     LNOEDGES                          L       NO EDGES (TIME) SWITCH
!     LZTCONS                           L       IF Z-T CONSITENCY IS REQUIRED
!     LSPEHUM                           L       IF Q REQUIRED
!     LQERRFRH                          L       IF Q ERROR FUNCTION OF RH ERROR
!     LQERRFT                           L       IF Q ERROR FUNCTION OF T  ERROR
!     LQERRFP                           L       IF Q ERROR FUNCTION OF P  ERROR
!     LTENDCOR                          L       PRESSURE TENDENCY CORRECTION SWITCH
!     LERRPERS                          L       RPERSSITENCE ERROR SWITCH
!     LBIASCV                           L       BIAS CORRECTED VALUE TO BE TAKEN SWITCH
!     LTBIASBUFR                        L       TEMPERATURE BIAS CORRECTION TO BE DONE ON
!                                               BUFR (OTHERWISE IT IS DONE ON ODB)
!     LTBIASC                           L       T BIAS CORRECTED VALUE TO BE TAKEN SWITCH
!     LHBIASC                           L       H BIAS CORRECTED VALUE TO BE TAKEN SWITCH
!     LBIASCVO                          L       BIAS CORRECTED VALUE TO BE TAKEN SWITCH
!                                               (OLD STYLE)
!     LSTOBIAS                          L       STORE T BIAS IN REP. ERROR WORD SWITCH
!     L_NEW_T_BIAS                      L       NEW STYLE RS T BIAS
!     L_NEW_T_BIAS_SOLAR_vs_all_mean    L       NEW STYLE RS T BIAS solar part vs all mean
!     L_NEW_T_BIAS_SOLAR_vs_day_mean    L       NEW STYLE RS T BIAS solar part vs day mean
!     L_NEW_T_BIAS_SOLAR_vs_night_m  L       NEW STYLE RS T BIAS solar part vs night mean
!     L_NEW_T_BIAS_ALL_MEAN             L       NEW STYLE RS T BIAS all mean
!     L_NEW_T_BIAS_DAY_MEAN             L       NEW STYLE RS T BIAS day mean part
!     L_NEW_T_BIAS_NIGHT_MEAN           L       NEW STYLE RS T BIAS night mean part
!     L_NEW_RH_BIAS                     L       NEW STYLE RS RH BIAS
!     L_NEW_RH_BIAS_SOLAR_vs_all_mean   L       NEW STYLE RS RH BIAS solar part vs all mean
!     L_NEW_RH_BIAS_SOLAR_vs_day_mean   L       NEW STYLE RS RH BIAS solar part vs day mean
!     L_NEW_RH_BIAS_SOLAR_vs_night_m L       NEW STYLE RS RH BIAS solar part vs night mean
!     L_NEW_RH_BIAS_all_MEAN            L       NEW STYLE RS RH BIAS all mean
!     L_NEW_RH_BIAS_day_MEAN            L       NEW STYLE RS RH BIAS day part
!     L_NEW_RH_BIAS_night_MEAN          L       NEW STYLE RS RH BIAS night part
!
!     L_T_REF_SONDE_BIAS_ADJUST         L       T REF SONDE TO ADJUST BIAS TO
!     C_T_REF_SONDE_NAME                C       T REFERANCE SONDE NAME
!     N_T_REF_SONDE_CODE                C       T REFERANCE SONDE CODE
!     L_RH_REF_SONDE_BIAS_ADJUST        L       RH REF SONDE TO ADJUST BIAS TO
!     C_RH_REF_SONDE_NAME               C       RH REFERANCE SONDE NAME
!     N_RH_REF_SONDE_CODE               C       RH REFERANCE SONDE CODE

!     LCHSHIP                           L       SWITCH TO ASSIGN REAL SHIP CHARCTERISTICS
!     LCHDRIBU                          L       SWITCH TO ASSIGN REAL DRIBU CHARCTERISTICS
!     LSHDRCHO                          L       SWITCH TO CHANGE OVER FROM SHI TO DRIBU
!     LSHDRCHOC                         L       SWITCH TO COMPLETE CHANGE OVER FROM SHI TO DRIBU

!     LAMRDBQC                          L       SATOB RDB QC SWITCH

!     LSCATINT                          L       SCAT. INTERFACE SWITCH
!     LSINOSOL                          L       SCAT. INTERFACE "NO SOLUTION" CASE DIAG.
!     LSCATTHI                          L       SCAT. THINNING SWITCH
!     LTOTSCTH                          L       TOTAL SCAT. THINNING
!     LTOTREM                           L       TOTAL REMOVAL OF A REPORT
!     LNOPROC                           L       NO PROCESSING OF OBS.
!     LODBDDRMOD                        L       TO MODIFY ODB DDR SWITCH
!     LODBDDRMODF                       L       FULL ODB DDR MODIFICATION
!     LODBDDRMODS                       L       SHORT ODB DDR MODIFICATION
!     LMKCMARPLRUN                      L       MAKECMA REPLACMENT RUN
!     NTHINSCA                          I       SCAT. THINNING FACTOR:
!                                               (1,2,4) = (25, 50, 100) KM RESOLUTION (moved to
!                                               namscc)
!     NBIASCOR                          I       TYPE OF SPEED BIAS CORRECTION ( 1 used until CY21R3,
!                                               2 from CY21R4 )
!                                               if(NBIASCOR==0) no speed bias correction for ERS and ASCAT


!     NMKCMVSE()                        I       LIST OF VARIABLES TO BE SELECTED FOR CMA/ODB

!     NOSORTSL                          I       NUMBER OF TIME SLOTS IN 4DVAR OBS. SORTING

!     L3DVAR                            L       MAKE ODB 3D VAR RUN
!     L3DFGAT                           L       MAKE ODB 3D VAR FGAT RUN
!     LINCREM                           L       3D/4D VAR INCREMENTAL RUN
!     LCANARI                           L       CANARI (FRENCH OI) SWITCH

!     NANDAT                            I       ANALYSIS DATE, IN FORM YYYYMMDD
!     NANTIM                            I       ANALYSIS TIME, IN FORM HHMMSS
!     NANCDY                            I       ANALYSIS DATE, AS CENTURY DAY
!     NANHOU                            I       ANALYSIS TIME, IN HOURS FROM 00Z
!     NANMIN                            I       ANALYSIS TIME, IN MINUTES FROM 00Z

!     NTBMAR                            I       4D BACKWARD TIME MARGIN (IN MIN.)
!     NTFMAR                            I       4D FOREWARD TIME MARGIN (IN MIN.)
!     N4DMIN                            I       4D TIME LENGTH (IN MIN.)

!     NCENDA                            I       CENTRAL DAY IN 4-D TIME INTERVAL RELATIVE
!                                               TO THE START
!     NCENHO                            I       CENTRAL HOUR IN 4D-TIME INTERVAL RELATIVE
!                                               TO THE START

!     OBNLAT                            R       NORTHERN BOUNDARY OF OBSERVATION AREA
!     OBSLAT                            R       SOUTHERN BOUNDARY OF OBSERVATION AREA
!     OBWLON                            R       WESTERN BOUNDARY OF OBSERVATION AREA
!     OBELON                            R       EASTERN BOUNDARY OF OBSERVATION AREA
!     EXNLAT                            R       NORTHERN BOUNDARY FOR EXCLUSION AREA
!     EXSLAT                            R       SOUTHERN BOUNDARY FOR EXCLUSION AREA
!     EXWLON                            R       WESTERN BOUNDARY FOR EXCLUSION AREA
!     EXELON                            R       EASTERN BOUNDARY FOR EXCLUSION AREA

!       SWITCHES FOR CMA OBS./CODE TYPES TO BE CONSIDERED:
!     LSYNOP                            L       SYNOP
!     LCD011                            L       SYNOP LAND (CODE 11)
!     LCD014                            L       SYNOP LAND (CODE 14)
!     LCD021                            L       SYNOP SHIP (CODE 21)
!     LCD022                            L       SYNOP SHIP (CODE 22)
!     LCD023                            L       SYNOP SHIP (CODE 23)
!     LCD024                            L       SYNOP SHIP (CODE 24)
!     LAIREP                            L       AIRCRAFT
!     LCD041                            L       AIRCRAFT (CODE 41)
!     LCD141                            L       AIRCRAFT (CODE 141)
!     LCD142                            L       AIRCRAFT (CODE 142)
!     LCD144                            L       AIRCRAFT (CODE 144)
!     LCD145                            L       AIRCRAFT (CODE 145)
!     LCD241                            L       AIRCRAFT (CODE 241)
!     LSATOB                            L       SATOB
!     LCD088                            L       SATOB (CODE 88)
!     LCD089                            L       SATOB (CODE 89)
!     LCD090                            L       SATOB AMV (CODE 90)
!     LCD188                            L       SATOB (CODE 188)
!     LODRIBU                           L       DRIFTING BUOY
!     LCD063                            L       DRIBU (CODE 63)
!     LCD064                            L       DRIBU (CODE 64)
!     LCD165                            L       DRIBU (CODE 165)
!     LCD160                            L       DRIBU (CODE 160)
!     LTEMP                             L       TEMP
!     LCD035                            L       TEMP (CODE 35)
!     LCD036                            L       TEMP (CODE 36)
!     LCD037                            L       TEMP (CODE 37)
!     LCD135                            L       TEMP (CODE 135)
!     LCD137                            L       TEMP (CODE 137)
!     LCD039                            L       TEMP (CODE 39)
!     LCD040                            L       TEMP (CODE 40)
!     LOPILOT                           L       PILOT
!     LCD032                            L       PILOT (CODE 32)
!     LCD033                            L       PILOT (CODE 33)
!     LCD034                            L       PILOT (CODE 34)
!     LCD131                            L       PILOT (CODE 131)
!     LCD134                            L       PILOT (CODE 134)
!     LSATEM                            L       SATEM
!     LCD086                            L       SATEM (CODE 86)
!     LCD186                            L       SATEM (CODE 186)
!     LCD185                            L       SATEM (CODE 185)
!     LCD184                            L       SATEM (CODE 184)
!     LCD206                            L       SATEM (CODE 206)
!     LCD200                            L       SATEM (CODE 200)
!     LCD201                            L       SATEM (CODE 201)
!     LCD202                            L       SATEM (CODE 202)
!     LCD210                            L       SATEM (CODE 210)
!     LCD211                            L       SATEM (CODE 211)
!     LCD212                            L       SATEM (CODE 212)
!     LCD215                            L       SATEM (CODE 215)
!     LOPAOB                            L       PAOB
!     LCD180                            L       PAOB (CODE 180)
!     LSCATT                            L       SCATTEROMETER
!     LASCATSM                          L       ASCAT Soil Moisture
!     LCD008                            L       SCATT. (CODE 8)
!     LCD122                            L       SCATT. (CODE 122)
!     LCD210SC                          L       SCATT. (CODE 210)
!     LCD300                            L       SCATT. (CODE 300 - NSCAT)
!     LCD301                            L       SCATT. (CODE 301 - QSCAT)
!     LCD302                            L       SCATT. (CODE 302 - OCEANSAT-2)
!     LALIMB                            L       LIMB OBS
!     LCD250                            L       RADIO OCCULTATION (CODE 250)
!     LCD250                            L       LIMB RADIANCES (CODE 251)

!NMKCMVSE(JPMXVAR,JPMXOCT,JPMXOTP), NTHINSCA, NBIASCOR, NOSORTSL,*
!     L_LAND_PS_BIAS                    L       LAND SYNOP PS BIAS CORRECTION SWITCH
!     L_SHIP_PS_BIAS                    L       SHIP SYNOP PS BIAS CORRECTION SWITCH
!     L_METAR_PS_BIAS                   L       METAR SYNOP PS BIAS CORRECTION SWITCH
!     L_DRIBU_PS_BIAS                   L       DRIBU PS BIAS CORRECTION SWITCH
!     L_SCAT_BIASSIG                    L       SIGMA BIAS CORRECTION FOR NSCAT2,NSCAT3 (ERS/ASCAT)

#endif

NAMELIST/NAMMKODB/&
&LNOEDGES ,LSURPR, LZTCONS, LSPEHUM, LQERRFRH, LQERRFT, LQERRFP,&
&LTENDCOR, LERRPERS, LBIASCV, LTBIASBUFR, LTBIASC, LHBIASC, LBIASCVO,LSTOBIAS,&
&L_NEW_T_BIAS, L_NEW_T_BIAS_SOLAR_vs_all_mean, L_NEW_T_BIAS_SOLAR_vs_day_mean, L_NEW_T_BIAS_SOLAR_vs_night_m,&
&L_NEW_T_BIAS_ALL_MEAN, L_NEW_T_BIAS_DAY_MEAN, L_NEW_T_BIAS_NIGHT_MEAN,&
&L_NEW_RH_BIAS, L_NEW_RH_BIAS_SOLAR_vs_all_mean, L_NEW_RH_BIAS_SOLAR_vs_day_mean, L_NEW_RH_BIAS_SOLAR_vs_night_m,&
&L_NEW_RH_BIAS_ALL_MEAN, L_NEW_RH_BIAS_DAY_MEAN, L_NEW_RH_BIAS_NIGHT_MEAN,&
&L_T_REF_SONDE_BIAS_ADJUST, C_T_REF_SONDE_NAME,  N_T_REF_SONDE_CODE,&
&L_RH_REF_SONDE_BIAS_ADJUST, C_RH_REF_SONDE_NAME, N_RH_REF_SONDE_CODE,&
&LCHSHIP, LCHDRIBU, LSHDRCHO, LSHDRCHOC, LAMRDBQC, LSCATINT, LSINOSOL,&
&LSCATTHI, LTOTSCTH, LTOTREM, LNOPROC, LODBDDRMOD, LODBDDRMODF, LODBDDRMODS,&
&LMKCMARPLRUN,&
!&NMKCMVSE, NTHINSCA, NBIASCOR, NOSORTSL,&
&NMKCMVSE, NBIASCOR, NOSORTSL,&
&NANDAT, NANTIM, NANCDY, NANHOU, NANMIN, NTBMAR, NTFMAR, N4DMIN,&
&OBNLATE,OBSLATE, OBWLONE, OBELONE, EXNLAT, EXSLAT, EXWLON, EXELON,&
!&L3DVAR, L3DFGAT, LINCREM, LCANARI,&
&L3DVAR,&
&LSYNOP, LCD011, LCD014, LCD021,LCD022, LCD023, LCD024,&
&LAIREP, LCD041, LCD141, LCD142, LCD144, LCD145, LCD241,&
&LSATOB, LCD088, LCD089, LCD090, LCD188,&
&LODRIBU, LCD063, LCD064, LCD165, LCD160,&
&LTEMP, LCD035, LCD036, LCD037, LCD135, LCD137, LCD039, LCD040,&
&LOPILOT, LCD032, LCD033, LCD034, LCD131, LCD134,&
&LSATEM, LCD086, LCD186, LCD185, LCD184, LCD206, LCD200, LCD201,&
&        LCD202, LCD210, LCD211, LCD212, LCD215,&
&LOPAOB, LCD180,&
&LSCATT, LASCATSM, LCD008, LCD122, LCD210SC, LCD300, LCD301,LCD302,&
&LALIMB, LCD250, LCD251,&
&L_LAND_PS_BIAS, L_SHIP_PS_BIAS, L_METAR_PS_BIAS, L_DRIBU_PS_BIAS,&
&L_SCAT_BIASSIG
