SUBROUTINE HOUSPE3(PFI,PVORO,PDIVO,PHGTO,KVMO,KM)

!**** *HOUSPE3* - ROUTINE TO FORM SPECTRAL FIELDS FROM DEFINED
!                HOUGH COEFFICIENTS, FOR A GIVEN VERTICAL MODE

!     PURPOSE.   TRANSFORM FROM HOUGH SPACE TO "STANDARD SPACE"
!     --------

!**   INTERFACE.
!     ----------
!        *CALL* *HOUSPE3(PFI,PVORO,PDIVO,PHGTO,KVMO,KM)*

!        EXPLICIT ARGUMENTS :  PFI - HOUGH COMPONENTS        - INPUT
!                              PVORO- VORTICITY            \
!                              PDIVO- DIVERGENCE           \ - OUTPUT
!                              PHGTO- GEOPOTENTIAL         \
!                              KVMO - VERTICAL MODE INDEX    - INPUT
!                              KM   - ZONAL WAVENUMBER INDEX - INPUT
!        IMPLICIT ARGUMENTS :
!        --------------------

!   METHOD.     THE SPECTRUM OF HOUGH MODES IS INPUT THROUGH PFI.
!   -------     THIS SPECTRUM IS THEN PROJECTED BACK INTO "STANDARD SPACE"
!               BY MULTIPLYING BY THE MATRIX CONTAINING THE NORMAL MODES.
!               THIS MATRIX HAS BEEN PREVIOUSLY COMPUTED BY SUMODE3. THE
!               FIELDS ARE PUT INTO PVOR-PHGT.

!     EXTERNALS.  REORDO3 - REORDER AND NORMALIZE FIELDS (TRUNCATION NXMAX)
!     ----------  MIO...  - IO ROUTINES
!                 MXMAOP  - MATRIX MULTIPLY ( SGEMMX )

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        BILL HECKLEY                       *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 88-12-07
!        CLIVE TEMPERTON : 90-01-14 : HOUSPE => HOUSPE3
!        PASCAL MARQUET  : 91-02-15 : TRUNCATION "NXMAX"
!        MATS  HAMRUD    : 93-02-08 : MULTITASKING, IO-SCHEME
!        LARS  ISAKSEN   : 94-08-17 : MULTITASKING MOVED TO HIGHER LEVEL
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NCMAX    ,NXMAX
USE YOMHOUB  , ONLY : HOUBUF   ,NSTAHOUB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFI(6*(NXMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORO(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVO(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHGTO(2*(NCMAX+1-KM)) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMO 
REAL(KIND=JPRB) ::ZFI(6*(NXMAX+1-KM))
INTEGER(KIND=JPIM) ::  IORDR, IREC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "reordo3.intfb.h"

!     ------------------------------------------------------------------

!*       1.    GO BACK TO SPECTRAL SPACE.
!              --------------------------

IF (LHOOK) CALL DR_HOOK('HOUSPE3',0,ZHOOK_HANDLE)
IREC=(KVMO-1)*(NXMAX+1)+KM+1
IORDR=3*(NXMAX-KM+1)
CALL MXMAOP(HOUBUF(NSTAHOUB(IREC)+1),1,IORDR,PFI,2,1,ZFI,2,1,IORDR,IORDR,2)

!     ------------------------------------------------------------------

!*       2.    REPLACE STATE VARIABLES.
!              ------------------------

CALL REORDO3(ZFI,PVORO,PDIVO,PHGTO,KVMO,KM,-1)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('HOUSPE3',1,ZHOOK_HANDLE)
END SUBROUTINE HOUSPE3

