SUBROUTINE MO3DPRJ(KMODE,KSPACE,KVMOD,KVMODPP,KUMVMO,PVORDI,&
 & PDIVDI,PTDI,PSPDI,PC,PB,PUP,PDIAG,PLOW,PFQ,PFP)  

!**** *MO3DPRJ*  - PROJECTION ONTO NORMAL MODES

!     Purpose.
!     --------
!         Projection of fields, according to options (KMODE,KSPACE)

!**   Interface.
!     ----------
!        *CALL* *MO3DPRJ(...)

!        Explicit arguments :
!        --------------------
!          KMODE : describes the mode of projection
!                  0 = NO PROJECTION REQUIRED
!                  1 = PROJECTION ON ROSSBY MODES
!                  2 = PROJECTION ON GRAVITY MODES
!                  3 = PROJECTION ON GRAV. MODES AND DIVIDE BY FREQ.
!                      (COMPUTATION OF MACHENHAUER NNMI INCREMENTS)

!          KSPACE: describes the space where the proj. will be used :
!                1 = USED IN SPECTRAL SPACE (VOR, DIV and HGT spectral arrays)
!                2 = USED IN HOUGH SPACE (result in FILT in YOMNMI, see FLTMODE)

!          KVMOD   : Total number of vertical modes
!          KVMODPP : Number of vertical modes on this PE
!          KUMVMO  : Number of vertical modes on each B-set

!          PVORDI,PDIVDI,PTDI,PSPDI    : spectral arrays
!          PC,PB,PUP,PDIAG,PLOW,PFQ,PFP: help arrays for implicit NMI

!        Implicit arguments :  Logical switches in YOMNMI and YOMLFI
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------  VTRANM  - MACRO TASKING INTERFACE FOR VERTICAL TRANSFORMS
!                 TRAGEO  - TRANSFORMS TO GEOGRAPHICAL SPHERE
!                 MOPRJM  - MACRO TASKING INTERFACE FOR VERTICAL PROJECTIONS

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        F. Rabier      *Meteo France*

!     Modifications.
!     --------------
!        Original : 91-08-28 (mixing between ex-FLTMODE and ex-NNMI3)
!                   (authors: C. Temperton,B. Heckley,P.Marquet)
!        Modified : Bill Heckley 27:02:92 Vertical projection made optional
!        Modified : Bill Heckley 10:09:92 bug-fix in call to MOPRJ
!        Modified : Bill Heckley 02:02:93 Vertical projection now always done.
!                                Extra filtering control for variational
!                                analysis under control of LFLTBAL.
!        Modified : D. Giard 93-01-22  new solution for stretching
!        Modified : D. Giard 93-03-19  NTMAX instead of NSMAX; call to VTRAN;
!                        LTRANS checked in SUGEM1; diagnostics for increments
!        Modified : D. Giard 93-04-08  ISPORTS=NSTTYP-1
!        Modified : D. Giard 94-07-26  previous opening of matrixes or not
!        Modified : L. Isaksen 94-11-11 Macro tasking over wavenumbers and modes
!        Modified : L. Isaksen 95-01-20 Reproducible calculation of energy diagnostics
!        Modified : L. Isaksen 95-06-06 Reordering of spectral arrays
!        Modified : D. Giard   95-08-30 corrections 
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG   ,NSMAX    ,NCMAX    ,&
 & NCPEC    ,NCPEC2   ,NTMAX    ,NTPEC2   ,&
 & NUMP     ,NUMCP    ,NUMTP  
USE YOMGEM   , ONLY : NSTTYP   ,RLOCEN
USE YOMLUN   , ONLY : NULOUT   ,NULDILA  ,NULCONT  ,NULROTC
USE YOMLAP   , ONLY : NACM0    ,NATM0    ,NCVALUE  ,NTVALUE  ,&
 & MYMS     ,RLAPIN   ,RCAPDI   ,RCAPIN   ,RTAPDI   ,&
 & RTAPIN  
USE YOMDYN   , ONLY : SIVP
USE YOMNMIA  , ONLY : NVMODM   ,NVMODJB  ,LNMIRQ   ,&
 & LFLTBAL  ,LTRANS   ,LRGENE  
USE YOMCT0   , ONLY : NPROC    ,NPRTRV
USE YOMMUL   , ONLY : N
USE YOMMP    , ONLY : MYPROC   ,MYSETV   ,NPTRCV   ,NPTRTV   ,&
 & NVMODIST ,NCPEC2V  ,NTPEC2V  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPACE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMOD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMODPP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUMVMO(NPRTRV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORDI(NFLEVL,NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVDI(NFLEVL,NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTDI(NFLEVL,NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDI(NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC(NCPEC2,NVMODM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUP(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIAG(NCPEC2,NVMODM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOW(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFQ(NCPEC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFP(NCPEC) 
!*******************************************************
!**  ARRAYS ON THE T"NTMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVDTT(NTPEC2,KVMODPP,3)
REAL(KIND=JPRB) :: ZVDTT1(NTPEC2V,KVMOD,3)
REAL(KIND=JPRB) :: ZVORDIG(NFLEVG,NCPEC2V),ZDIVDIG(NFLEVG,NCPEC2V)
REAL(KIND=JPRB) :: ZTDIG(NFLEVG,NCPEC2V),ZSPDIG(NCPEC2V)
!***********************************************************
!**  ARRAYS ON THE T"NCMAX" OF THE REAL GEOGRAPHIC SPHERE **
!***********************************************************
REAL(KIND=JPRB) :: ZVDTR(NCPEC2,KVMODPP,3)
REAL(KIND=JPRB) :: ZVDTR1(NCPEC2V,KVMOD,3)
REAL(KIND=JPRB) :: ZENERT(KVMOD), ZENERTL(NUMCP,KVMODPP)
REAL(KIND=JPRB) :: ZENERI(KVMOD), ZENERIL(NUMCP,KVMODPP)

INTEGER(KIND=JPIM) :: I, IBUFLEN, IDIRTR, IFIELD, IFLAG, IFLDSUR,&
 & IK, IM, IMODE, IOMASTER, &
 & ISPORTS, ITYPFLD, IVMOD2, J, JK, JMLOC, JN  

LOGICAL :: LLPRINT, LLTRAGEO

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "moprjm.intfb.h"
#include "trageo.intfb.h"
#include "trmtov.intfb.h"
#include "trvtoh.intfb.h"
#include "vmodeenergy.intfb.h"
#include "vtranm.intfb.h"

IF (LHOOK) CALL DR_HOOK('MO3DPRJ',0,ZHOOK_HANDLE)
IOMASTER=MIN(NPROC,1)
LLPRINT=MYPROC == IOMASTER
!      -----------------------------------------------------------------
!           The subroutine TRAGEO is likely to be used if LLTRAGEO=TRUE :
!                    if RSTRET.NE.1 or if NSTTYP.NE.1
!              -----------------------------------------------

!   LTRANS : NNMI is required and possible on the transformed sphere
!   LLTRAGEO : FALSE if already on the geographic sphere
LLTRAGEO = (NSMAX /= NCMAX) .OR. (NSTTYP /= 1)

!*    1.   PROJECT FIELDS ONTO VERTICAL MODES
!          ----------------------------------

IF (LFLTBAL) THEN
  IMODE = 1
ELSE
  IMODE = 2
ENDIF

IF (LTRANS.OR.(.NOT.LLTRAGEO)) THEN
  IF (NPRTRV > 1) THEN

!         transposition to vertical spectral columns   :PxxxDI-->ZxxxDIG

    IBUFLEN=MAX((3*NFLEVL+1)*NCPEC2,(3*NFLEVG+1)*NCPEC2V)
    CALL TRMTOV(NCPEC2,NCPEC2V,NPTRCV,PVORDI,PDIVDI,PTDI,PSPDI,&
     & ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,IBUFLEN,-1,IMODE)  

!         vertical transform (on the current sphere)   :ZxxDIG-->ZVDTR1

    CALL VTRANM(NCPEC2V,ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,&
     & ZVDTR1(1,1,1),ZVDTR1(1,1,2),ZVDTR1(1,1,3),&
     & KVMOD,-1,IMODE)  

!         transposition to subset of normal modes      :ZVDTR1-->ZVDTR

    IBUFLEN=MAX(3*NCPEC2*KVMODPP,3*NCPEC2V*KVMOD)
    CALL TRVTOH(NCPEC2,NCPEC2V,NPTRCV,KVMOD,KVMODPP,KUMVMO,&
     & ZVDTR(1,1,1),ZVDTR(1,1,2),ZVDTR(1,1,3),&
     & ZVDTR1(1,1,1),ZVDTR1(1,1,2),ZVDTR1(1,1,3),&
     & IBUFLEN,-1)  
  ELSE

!         vertical transform (on the current sphere)   :ZVDTR-->PxxDI

    CALL VTRANM(NCPEC2,PVORDI,PDIVDI,PTDI,PSPDI,&
     & ZVDTR(1,1,1),ZVDTR(1,1,2),ZVDTR(1,1,3),&
     & KVMOD,-1,IMODE)  
  ENDIF
ELSE
  IF (NPRTRV > 1) THEN

!         transposition to vertical spectral columns   :PxxDI-->ZxxDIG

    IBUFLEN=MAX((3*NFLEVL+1)*NTPEC2,(3*NFLEVG+1)*NTPEC2V)
    CALL TRMTOV(NTPEC2,NTPEC2V,NPTRTV,PVORDI,PDIVDI,PTDI,PSPDI,&
     & ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,IBUFLEN,-1,IMODE)  

!         vertical transform (on the current sphere)   :ZxxDIG-->ZVDTT1

    CALL VTRANM(NTPEC2V,ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,&
     & ZVDTT1(1,1,1),ZVDTT1(1,1,2),ZVDTT1(1,1,3),&
     & KVMOD,-1,IMODE)  

!         transposition to subset of normal modes      :ZVDTT1-->ZVDTT

    IBUFLEN=MAX(3*NTPEC2*KVMODPP,3*NTPEC2V*KVMOD)
    CALL TRVTOH(NTPEC2,NTPEC2V,NPTRTV,KVMOD,KVMODPP,KUMVMO,&
     & ZVDTT(1,1,1),ZVDTT(1,1,2),ZVDTT(1,1,3),&
     & ZVDTT1(1,1,1),ZVDTT1(1,1,2),ZVDTT1(1,1,3),&
     & IBUFLEN,-1)  
  ELSE

!         vertical transform (on the current sphere)   :PxxDI-->ZVDTT

    CALL VTRANM(NTPEC2,PVORDI,PDIVDI,PTDI,PSPDI,&
     & ZVDTT(1,1,1),ZVDTT(1,1,2),ZVDTT(1,1,3),&
     & KVMOD,-1,IMODE)  
  ENDIF

!      -----------------------------------------------------------------

!*    2.   GO FROM TRANSFORMED TO GEOGRAPHIC SPHERE (ZVDTT--> ZVDTR)
!*         (include the case of a real geographic sphere with  higher
!*         truncation -NCMAX- used to realize the LNMI or the NLNMI)
!          ----------------------------------------------------------

  WRITE(NULOUT,'('' GOTO THE REAL GEOGRAPHIC SPHERE :'')')

!*    2.1   Vort. and Div. (transformed sphere) are transformed into
!*          Stream Fct and Velocity Potential before the call to TRAGEO.

  WRITE(NULOUT,'(''   COMPUTE STREAM FONCTION AND VELOCITY'',&
   & '' POTENTIAL ON THE TRANSFORMED SPHERE'')')  
  DO JK=1,KVMODPP
    DO J=1,NTPEC2
      ZVDTT(J,JK,1)=RTAPIN(NTVALUE(J))*ZVDTT(J,JK,1)
      ZVDTT(J,JK,2)=RTAPIN(NTVALUE(J))*ZVDTT(J,JK,2)
    ENDDO
  ENDDO

!*    2.2   goto the real geographic sphere : ZVDTT --> ZVDTR

  IFIELD  = KVMODPP*3
  IFLDSUR = 2*INT(IFIELD/2) + 1
  ISPORTS = NSTTYP-1
  ITYPFLD = 2
  IDIRTR  = 1
  WRITE(NULOUT,'(''   CALL TO TRAGEO, IDIR = +1 '')')

  CALL TRAGEO(NULROTC,NULDILA,NULOUT,IFIELD,&
   & IFLDSUR,NTMAX,NATM0,NTPEC2,NUMTP,NCMAX,NACM0,&
   & NCPEC2,NUMCP,MYMS,ISPORTS,ITYPFLD,IDIRTR,RLOCEN,&
   & 0.0_JPRB,ZVDTT,ZVDTR)  

!*    2.3   Vort. and Div. on the real sphere are obtained using the
!*          Stream Fct and the Velocity Potential after  TRAGEO.

  WRITE(NULOUT,'(''   COMPUTE VORTICITY AND DIVERGENCE'',&
   & '' ON THE REAL SPHERE'')')  
  DO JK=1,KVMODPP
    DO J=1,NCPEC2
      ZVDTR(J,JK,1)=RCAPDI(NCVALUE(J))*ZVDTR(J,JK,1)
      ZVDTR(J,JK,2)=RCAPDI(NCVALUE(J))*ZVDTR(J,JK,2)
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*    3.   PROJECT ONTO SELECTED MODES DEFINED BY LTYPDB AND KMODE
!          -------------------------------------------------------

IF (LFLTBAL) THEN
  IVMOD2 = NVMODJB
ELSE
  IVMOD2 = KVMOD
ENDIF

!*    3.1   diagnostics if LRGENE : energy in gravity mode tendencies.

IF ((KSPACE == 1) .AND. (KMODE >= 2).AND. LRGENE .AND. .NOT.LFLTBAL) THEN

  IF (LLPRINT) WRITE(NULOUT,&
   & '('' ***************************************************'',/,&
   & '' ****** COMPUTE THE ENERGY IN GRAVITY MODES : ******'',/,&
   & '' ***************************************************'')')  

  ZENERTL(:,:)=0.0_JPRB

  N=-1
  IFLAG=1

  CALL MOPRJM(ZVDTR,PC,PB,PUP,PDIAG,PLOW,PFQ,PFP,ZENERTL,&
   & KVMOD,KVMODPP,2,KSPACE,IFLAG)  

!   Gather vertical mode energy in a reproducible manner

  CALL VMODEENERGY(IOMASTER,KVMOD,KVMODPP,NCMAX,NUMCP,KUMVMO,ZENERTL,ZENERT)

ENDIF

!*    3.2   projection

IF (LNMIRQ .AND. LLPRINT) THEN
  WRITE(NULOUT,&
   & '('' ***************************************************'',/,&
   & '' ******  COMPUTE NMI INCREMENTS (MACHENHAUER ) *****'',/,&
   & '' ***************************************************'')')  
ENDIF

ZENERIL(:,:)=0.0_JPRB

N=-1
IFLAG=2

CALL MOPRJM(ZVDTR,PC,PB,PUP,PDIAG,PLOW,PFQ,PFP,ZENERIL,&
 & KVMOD,KVMODPP,KMODE,KSPACE,IFLAG)  

!     ------------------------------------------------------------------

!*    4.   FROM GEOGRAPHIC TO TRANSFORMED SPHERE (ZVDTR --> ZVDTT)
!*         (include the case of a real geographic sphere with higher
!*         truncation - NCMAX - used to realize a LNMI or a NLNMI)
!          ---------------------------------------------------------

IF (KSPACE /= 2) THEN
  IF (LLTRAGEO.AND.(.NOT.LTRANS)) THEN
!         goto the transformed sphere : ZVDTR --> ZVDTT

    WRITE(NULOUT,'('' COME BACK TO THE TRANSFORMED SPHERE :'')')

!*    4.1   Vort. and Div. on the real sphere are transformed into
!*          Stream Fct and Velocity Potential before the call to TRAGEO.

    WRITE(NULOUT,'(''  COMPUTE STREAM FONCTION AND VELOCITY'',&
     & '' POTENTIAL ON THE REAL SPHERE'')')  
    DO JK=1,KVMODPP
      DO J=1,NCPEC2
        ZVDTR(J,JK,1)=RCAPIN(NCVALUE(J))*ZVDTR(J,JK,1)
        ZVDTR(J,JK,2)=RCAPIN(NCVALUE(J))*ZVDTR(J,JK,2)
      ENDDO
    ENDDO

!*    4.2   goto the transformed sphere : ZVDTR --> ZVDTT

    WRITE(NULOUT,'(''  CALL TO TRAGEO, DIR = -1 '')')
    IDIRTR = -1
    CALL TRAGEO(NULROTC,NULCONT,NULOUT,&
     & IFIELD,IFLDSUR,NTMAX,NATM0,NTPEC2,NUMTP,NCMAX,&
     & NACM0,NCPEC2,NUMCP,MYMS,ISPORTS,ITYPFLD,IDIRTR,&
     & RLOCEN,0.0_JPRB,ZVDTT,ZVDTR)  

!*    4.3 Vort. and Div. (transformed sphere) are obtained using the
!*        Stream Fct and the Velocity Potential after TRAGEO.

    WRITE(NULOUT,'(''  COMPUTE VORTICITY AND DIVERGENCE'',&
     & '' ON THE TRANSFORMED SPHERE'')')  
    DO JK=1,KVMODPP
      DO J=1,NTPEC2
        ZVDTT(J,JK,1)=RTAPDI(NTVALUE(J))*ZVDTT(J,JK,1)
        ZVDTT(J,JK,2)=RTAPDI(NTVALUE(J))*ZVDTT(J,JK,2)
      ENDDO
    ENDDO

!     ------------------------------------------------------------------

!*    5.     PROJECTION ONTO VERTICAL MODES
!            ------------------------------

!*    5.1 Case LLTRAGEO.AND..NOT.LTRANS (TRAGEO is used).

!  diagnostics
    IF ((KSPACE == 1).AND.(KMODE >= 3).AND. LRGENE .AND. .NOT.LFLTBAL) THEN
      DO JK=1,KVMODPP
        IK=NVMODIST(JK,MYSETV)
        ZENERIL(:,JK)=0.0_JPRB
        DO JMLOC=1,NUMP
          IM=MYMS(JMLOC)
          IF (IM == 0) THEN
            DO JN=0,NSMAX
              I=NATM0(0)+JN*2
              ZENERIL(JMLOC,JK)=ZENERIL(JMLOC,JK) +&
               & 0.5_JPRB*ZVDTT(I,JK,3)**2 - 0.5_JPRB*SIVP(IK)*RTAPIN(JN)*&
               & (ZVDTT(I,JK,1)**2+ZVDTT(I,JK,2)**2)  
            ENDDO
          ELSE
            DO JN=IM,NSMAX
              I=NATM0(IM)+(JN-IM)*2
              ZENERIL(JMLOC,JK)=ZENERIL(JMLOC,JK) +&
               & ZVDTT(I,JK,3)**2+ZVDTT(I+1,JK,3)**2 &
               & - SIVP(IK)*RTAPIN(JN)*&
               & (ZVDTT(I,JK,1)**2+ZVDTT(I+1,JK,1)**2+&
               & ZVDTT(I,JK,2)**2+ZVDTT(I+1,JK,2)**2)  
            ENDDO
          ENDIF
        ENDDO
      ENDDO

!   Gather vertical mode energy in a reproducible manner

      CALL VMODEENERGY(IOMASTER,KVMOD,KVMODPP,NCMAX,NUMCP,&
       & KUMVMO,ZENERIL,ZENERI)  
    ENDIF

    IF (NPRTRV > 1) THEN

!           transposition to subset of normal modes         :ZVDTT-->ZVDTT1

      IBUFLEN=MAX(3*NTPEC2*KVMODPP,3*NTPEC2V*KVMOD)
      CALL TRVTOH(NTPEC2,NTPEC2V,NPTRTV,KVMOD,KVMODPP,KUMVMO,&
       & ZVDTT(1,1,1),ZVDTT(1,1,2),ZVDTT(1,1,3),&
       & ZVDTT1(1,1,1),ZVDTT1(1,1,2),ZVDTT1(1,1,3),&
       & IBUFLEN,-1)  

!           vertical transform (on the transformed sphere)  :ZVDTT1-->ZxxDIG

      CALL VTRANM(NTPEC2V,ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,&
       & ZVDTT1(1,1,1),ZVDTT1(1,1,2),ZVDTT1(1,1,3),&
       & KVMOD,+1,IMODE)  

!           vertical transform (on the current sphere)      :ZxxDIG-->PxxDI

      IBUFLEN=MAX((3*NFLEVL+1)*NTPEC2,(3*NFLEVG+1)*NTPEC2V)
      CALL TRMTOV(NTPEC2,NTPEC2V,NPTRTV,PVORDI,PDIVDI,PTDI,PSPDI,&
       & ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,IBUFLEN,-1,IMODE)  
    ELSE

!           vertical transform (on the transformed sphere)  :ZVDTT-->PxxDI

      CALL VTRANM(NTPEC2,PVORDI,PDIVDI,PTDI,PSPDI,&
       & ZVDTT(1,1,1),ZVDTT(1,1,2),ZVDTT(1,1,3),&
       & KVMOD,+1,IMODE)  

    ENDIF
  ELSE

!*    5.2  Case LTRANS.OR.NOT.LLTRAGEO (TRAGEO is not used).

!  diagnostics
    IF ((KSPACE == 1).AND.(KMODE >= 3).AND. LRGENE .AND. .NOT.LFLTBAL) THEN
      DO JK=1,KVMODPP
        ZENERIL(:,JK)=0.0_JPRB
        DO JMLOC=1,NUMP
          IM=MYMS(JMLOC)
          IK=NVMODIST(JK,MYSETV)
          IF (IM == 0) THEN
            DO JN=IM,NSMAX
              I = NACM0(IM)+(JN-IM)*2
              ZENERIL(JMLOC,JK)=ZENERIL(JMLOC,JK) +&
               & 0.5_JPRB*ZVDTR(I,JK,3)**2 - 0.5_JPRB*SIVP(IK)*RLAPIN(JN)*&
               & (ZVDTR(I,JK,1)**2+ZVDTR(I,JK,2)**2)  
            ENDDO
          ELSE
            DO JN=IM,NSMAX
              I = NACM0(IM)+(JN-IM)*2
              ZENERIL(JMLOC,JK)=ZENERIL(JMLOC,JK) +&
               & ZVDTR(I,JK,3)**2+ZVDTR(I+1,JK,3)**2 &
               & - SIVP(IK)*RLAPIN(JN)*&
               & (ZVDTR(I,JK,1)**2+ZVDTR(I+1,JK,1)**2+&
               & ZVDTR(I,JK,2)**2+ZVDTR(I+1,JK,2)**2)  
            ENDDO
          ENDIF
        ENDDO
      ENDDO

!   Gather vertical mode energy in a reproducible manner

      CALL VMODEENERGY(IOMASTER,KVMOD,KVMODPP,NCMAX,NUMCP,&
       & KUMVMO,ZENERIL,ZENERI)  
    ENDIF

    IF (NPRTRV > 1) THEN

!           transposition from subset of normal modes    :ZVDTR-->ZVDTR1

      IBUFLEN=MAX(3*NCPEC2*KVMODPP,3*NCPEC2V*KVMOD)
      CALL TRVTOH(NCPEC2,NCPEC2V,NPTRCV,KVMOD,KVMODPP,KUMVMO,&
       & ZVDTR(1,1,1),ZVDTR(1,1,2),ZVDTR(1,1,3),&
       & ZVDTR1(1,1,1),ZVDTR1(1,1,2),ZVDTR1(1,1,3),&
       & IBUFLEN,+1)  

!           vertical transform (on the current sphere)   :ZVDTR1-->ZVORDIG

      CALL VTRANM(NCPEC2V,ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,&
       & ZVDTR1(1,1,1),ZVDTR1(1,1,2),ZVDTR1(1,1,3),&
       & KVMOD,+1,IMODE)  

!           transposition from vertical spectral columns  :ZVORDIG-->PxxxDI

      IBUFLEN=MAX((3*NFLEVL+1)*NCPEC2,(3*NFLEVG+1)*NCPEC2V)
      CALL TRMTOV(NCPEC2,NCPEC2V,NPTRCV,PVORDI,PDIVDI,PTDI,&
       & PSPDI,ZVORDIG,ZDIVDIG,ZTDIG,ZSPDIG,IBUFLEN,&
       & +1,IMODE)  
    ELSE

!           vertical transform (on the current sphere)    :ZVDTR-->PxxDI

      CALL VTRANM(NCPEC2,PVORDI,PDIVDI,PTDI,PSPDI,&
       & ZVDTR(1,1,1),ZVDTR(1,1,2),ZVDTR(1,1,3),&
       & KVMOD,+1,IMODE)  
    ENDIF
  ENDIF

  IF (MYPROC == IOMASTER .AND. (KSPACE == 1) .AND. (KMODE == 2)&
     & .AND. LRGENE .AND. .NOT.LFLTBAL) THEN  
    WRITE(NULOUT,&
     & '(/,'' ENERGY IN PROJECTED TENDENCIES :''&
     & ,/,('' VERTICAL MODE'',I3,2X,E25.18))')&
     & (JK,ZENERT(JK),JK=1,KVMOD)  
    WRITE(NULOUT,'( ''  '')')
  ENDIF
  IF (MYPROC == IOMASTER .AND.&
     & (KSPACE == 1) .AND. (KMODE == 3.OR.KMODE == 4)&
     & .AND. LRGENE .AND. .NOT.LFLTBAL) THEN  
    WRITE(NULOUT,&
     & '(/,'' ENERGY IN PROJECTED TENDENCIES AND INCREMENTS :''&
     & ,/,('' VERTICAL MODE'',I3,2X,E25.18,3X,E25.18))')&
     & (JK,ZENERT(JK),ZENERI(JK),JK=1,KVMOD)  
    WRITE(NULOUT,'( ''  '')')
  ENDIF

ENDIF

IF (LHOOK) CALL DR_HOOK('MO3DPRJ',1,ZHOOK_HANDLE)
END SUBROUTINE MO3DPRJ
