SUBROUTINE MOPRJM(PVDTR,PC,PB,PUP,PDIAG,PLOW,PFQ,PFP,PENERT,&
 & KVMOD,KVMODPP,KMODE,KSPACE,KFLAG)  

!**** *MOPRJM*

!     PURPOSE.   MULTI-TASKING MOPRJ
!     --------

!**   INTERFACE.
!     ----------
!        *CALL* *MOPRJM(...)*

!        EXPLICIT ARGUMENTS :
!        --------------------
!           PVDTR   - INPUT/OUTPUT
!           PC      - INPUT
!           PB      - INPUT
!           PUP     - INPUT
!           PDIAG   - INPUT
!           PLOW    - INPUT
!           PFQ     - INPUT
!           PFP     - INPUT
!           PENERT  - INPUT/OUTPUT
!           KVMOD   - INPUT
!           KVMODPP - INPUT
!           KMODE   - INPUT
!           KSPACE  - INPUT
!           KFLAG   - 1 : output=PENERT  2 : output=PVDTR

!        IMPLICIT ARGUMENTS :
!        --------------------

!     METHOD.   NONE
!     -------

!     EXTERNALS.  MOPRJ   - PROJECTION
!     ----------  REORD   - Reorders spectral coefficients
!                 ABOR1   - Self explanatory

!     AUTHOR.
!     -------
!        MATS HAMRUD AND ERIK ANDERSSON         *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL  : 94-05-17
!        L. Isaksen: 94-11-11 Macro tasking over wavenumbers and modes
!        D. Giard and
!        L. Isaksen: 95-01-20 Reproducible diagnostics and cleaning
!        L. Isaksen: 95-06-06 Reordering of spectral arrays, and
!                             change interface to PFP and PFQ
!        F. Veerse : 97-11-18 Dimensions of ZVORI,ZDIVI,ZHGTI
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NCMAX    ,NCPEC    ,NCPEC2   ,NUMCP
USE YOMNMIA  , ONLY : NVMODF   ,NVMODF1  ,NVMODF2  ,NVMODM   ,&
 & NVMODJB  ,LFLTBAL  
USE YOMMUL   , ONLY : N
USE YOMLAP   , ONLY : NACM0    ,MYMS     ,RCAPIN
USE YOMDYN   , ONLY : SIVP
USE YOMMP    , ONLY : MYSETV   ,NVMODIST

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KVMODPP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVDTR(NCPEC2,KVMODPP,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC(NCPEC2,NVMODM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUP(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIAG(NCPEC2,NVMODM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOW(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFQ(NCPEC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFP(NCPEC) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PENERT(NUMCP,KVMODPP) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMOD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPACE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLAG 
REAL(KIND=JPRB) :: ZVORI(2*(NCMAX+1)),ZDIVI(2*(NCMAX+1)),ZHGTI(2*(NCMAX+1))

INTEGER(KIND=JPIM) :: ICMVAL, ICMVAL1, IIK, IK, IKLOC, IM, IMLOC,&
 & IMNV, IMP, IRE, IVMOD, IVMOD2, IVMOD2P, JN, &
 & JSP  

REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "moprj.intfb.h"
#include "reord.intfb.h"

! Set-up
IF (LHOOK) CALL DR_HOOK('MOPRJM',0,ZHOOK_HANDLE)
IF (LFLTBAL) THEN
  IVMOD2 = NVMODJB
  IVMOD  = NVMODF
ELSE
  IVMOD2 = KVMOD
  IVMOD  = KVMOD
ENDIF
IVMOD2P=KVMODPP

200 CONTINUE
N=N+1
IMNV=N
IF (IMNV  <  IVMOD2P*NUMCP) THEN
  IKLOC=IMNV/NUMCP+1
  IK=NVMODIST(IKLOC,MYSETV)
  IMLOC=MOD(IMNV,NUMCP)+1
  IM=MYMS(IMLOC)

  ICMVAL=NACM0(IM)
  ICMVAL1=(NACM0(IM)-1)/2+1

! Extract fields for (k,m)
  CALL REORD(PVDTR(1,IKLOC,1),PVDTR(1,IKLOC,2),PVDTR(1,IKLOC,3),&
   & ZVORI,ZDIVI,ZHGTI,IM,+1)  

! Project
  IIK = MIN(IK,IVMOD)
  CALL MOPRJ(ZVORI,ZDIVI,ZHGTI,ZVORI,ZDIVI,ZHGTI,&
   & PC(ICMVAL,IIK),PB(ICMVAL),PUP(ICMVAL),&
   & PDIAG(ICMVAL,IIK),PLOW(ICMVAL),PFQ(ICMVAL1),&
   & PFP(ICMVAL1),KMODE,IIK,IM,KSPACE)  

  IF (KFLAG == 1) THEN

! Compute energy
    IF (IM == 0) THEN
      DO JN=1,NCMAX
        PENERT(IMLOC,IKLOC)=PENERT(IMLOC,IKLOC) +0.5_JPRB*ZHGTI(JN+1)**2 &
         & - 0.5_JPRB*SIVP(IK)*RCAPIN(JN)*(ZVORI(JN+1)**2 + ZDIVI(JN+1)**2)  
      ENDDO
    ELSE
      DO JN=IM,NCMAX
        IRE=JN+1-IM
        IMP=NCMAX+1-IM+IRE
        PENERT(IMLOC,IKLOC)=PENERT(IMLOC,IKLOC) +&
         & ZHGTI(IRE)**2+ZHGTI(IMP)**2 - SIVP(IK)*RCAPIN(JN)*&
         & (ZVORI(IRE)**2+ZVORI(IMP)**2+ZDIVI(IRE)**2+ZDIVI(IMP)**2)  
      ENDDO
    ENDIF

  ELSEIF(KFLAG == 2) THEN

! Progressively reduce amplitude of projections
    IF (LFLTBAL .AND. IK < IVMOD2.AND. IK >= NVMODF1+1) THEN
      ZEPS = REAL(NVMODF2-IK,JPRB)/REAL(NVMODF2-NVMODF1,JPRB)
      DO JSP=1,(NCMAX+1-IM)*(MIN(1,IM)+1)
        ZVORI(JSP) = ZVORI(JSP)*ZEPS
        ZDIVI(JSP) = ZDIVI(JSP)*ZEPS
        ZHGTI(JSP) = ZHGTI(JSP)*ZEPS
      ENDDO
    ENDIF

! Reorganize fields
    CALL REORD(PVDTR(1,IKLOC,1),PVDTR(1,IKLOC,2),PVDTR(1,IKLOC,3),&
     & ZVORI,ZDIVI,ZHGTI,IM,-1)  

  ELSE
    CALL ABOR1(' KFLAG NOT EQ. 1 OR 2, IN MOPRJM')
  ENDIF
  GOTO 200
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('MOPRJM',1,ZHOOK_HANDLE)
END SUBROUTINE MOPRJM
