SUBROUTINE MVTEND(PVOR,PDIV,PT,PSP)

!**** *MVTEND* - COPYING SPECTRAL TENDENCIES WHEN NTMAX > NSMAX .

!     PURPOSE.   TO COPY SPECTRAL TENDENCIES IN CASE OF STRETCHING
!     --------    (USED IN NONLINEAR NORMAL MODE INITIALIZATION)

!**   INTERFACE.
!     ----------
!        *CALL* *MVTEND(PVOR,PDIV,PT,PSP)*

!        EXPLICIT ARGUMENTS :  PVOR : ARRAY RECEIVING SPTVOR
!        --------------------  PDIV : ..... ......... SPTDIV
!                              PT   : ..... ......... SPTT
!                              PSP  : ..... ......... SPTSP

!        IMPLICIT ARGUMENTS :  YOMSPT (SPTVOR,SPTDIV,SPTT,SPTSP)
!        --------------------  (spectral tendencies)

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. Giard  * METEO-FRANCE *

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 93-03-19
!        D. GIARD : 93-04-05  case NFTHER=0
!        D. GIARD : 93-05-13  tendencies up to NSMAX+1 for vor,div
!      L. ISAKSEN : 93-05-13? Reordering of spectral arrays
!        D. Giard : 95-08-30  corrections
!      L. ISAKSEN : 96-02-20  NFLEV --> NFLEVL, cleaning
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!***************************************************************************

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NFTHER   ,NSMAX    ,NTMAX    ,NTPEC2
USE YOMSPTT  , ONLY : SPTVOR   ,SPTDIV   ,SPTT     ,SPTSP
USE YOMLAP   , ONLY : NATM0
USE YOMMP    , ONLY : MYSETW   ,NPROCM

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVOR(NFLEVL,NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIV(NFLEVL,NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT(NFLEVL,NTPEC2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSP(NTPEC2) 
INTEGER(KIND=JPIM) :: I, J, JIR, JK, JM, JN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*  1. vor=div=0 for m=n=0

IF (LHOOK) CALL DR_HOOK('MVTEND',0,ZHOOK_HANDLE)
IF (MYSETW == NPROCM(0)) THEN
  DO JK=1,NFLEVL
    SPTVOR(JK,NATM0(0))= 0.0_JPRB
    SPTDIV(JK,NATM0(0))= 0.0_JPRB
  ENDDO
ENDIF

!*  2. From m=0 to NTMAX and n=m to NTMAX

DO JK=1,NFLEVL
  DO J=1,NTPEC2
    PVOR(JK,J)= SPTVOR(JK,J)
    PDIV(JK,J)= SPTDIV(JK,J)
    IF (NFTHER >= 1) PT(JK,J)= SPTT(JK,J)
  ENDDO
ENDDO
DO J=1,NTPEC2
  PSP(J)= SPTSP(J)
ENDDO

IF (NSMAX < NTMAX) THEN

!*  3. Vorticity and divergence reset to 0. for n > NSMAX , m <= NSMAX

  DO JM=0,NSMAX
    DO JIR=0,1
      DO JN=NSMAX+1,NTMAX
        I=NATM0(JM)+(JN-JM)*2+JIR
        DO JK=1,NFLEVL
          PVOR(JK,I)= 0.0_JPRB
          PDIV(JK,I)= 0.0_JPRB
        ENDDO
      ENDDO
    ENDDO
  ENDDO

!*  4. Fields reset to 0. for m > NSMAX 

  DO JK=1,NFLEVL
    DO J=NATM0(NSMAX+1),NTPEC2
      PVOR(JK,J)= 0.0_JPRB
      PDIV(JK,J)= 0.0_JPRB
      IF (NFTHER >= 1) PT(JK,J)= 0.0_JPRB
    ENDDO
  ENDDO
  DO J=NATM0(NSMAX+1),NTPEC2
    PSP(J)= 0.0_JPRB
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('MVTEND',1,ZHOOK_HANDLE)
END SUBROUTINE MVTEND
