SUBROUTINE NNMI3TL

!**** *NNMI3TL* - NON-LINEAR NORMAL MODE INITIALIZATION FOR 3-D P.E.M.
!                 TANGENT-LINEAR.

!     PURPOSE.   TO PERFORM THE NON-LINEAR NORMAL MODE INITIALIZATION
!     --------   FOR A 3-D P. E. MODEL.( TANGENT LINEAR ).

!**   INTERFACE.
!     ----------
!        *CALL* *NNMI3TL*

!        EXPLICIT ARGUMENTS :  NONE
!        --------------------

!        IMPLICIT ARGUMENTS :  THE SPECTRAL ARRAYS OF THE MODEL
!        --------------------

!     METHOD.   THE TENDENCIES ARE COMPUTED BY AN APPROPIATE CALL TO STEPO.
!     -------   THE TENDENCIES ARE THEN CHANGED INTO NORMAL MODE SPACE
!               AND PROJECTED ONTO THE GRAVITY MODES DIVIDED BY THE
!               FREQUENCIES. WE GO BACK INTO SPECTRAL SPACE AND DEDUCT THE
!               PROJECTED FIELDS FROM THE STATE VARIABLES. THE PROCESS IS
!               REPEATED NITNMI TIMES.

!     EXTERNALS.  STEPOTL  - COMPUTE THE TENDENCIES
!     ----------  MO3DPRJ- PROJECTION ON MODES
!                 SPNORM - COMPUTE NORMS IN SPECTRAL SPACE FOR ALL LEVELS
!                 SPNORM - COMPUTE NORMS IN SPECTRAL SPACE FOR ALL LEVELS
!                 MVTEND - COPYING TENDENCIES TO ARGUMENTS OF MO3DPRJ
!                 SUALLTT- ALLOCATE SPACE FOR SPECTRAL TENDENCIES
!                 DEALLTT- DESALLOCATE SPACE FOR SPECTRAL TENDENCIES
!                 RDPINMI- READ HELP ARRAYS FOR IMPLICIT NMI
!                 USER_CLOCK- timing

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        Jean-Noel THEPAUT *METEO FRANCE EERM/CRMD*
!        - BASED ON THE ROUTINE NNMI2TL WRITTEN BY
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 90-03-15
!        D. GIARD : 92-11-06 : additionnal NNMI step to test convergence
!                              cleaning
!        D. GIARD : 92-11-26 : LRSPSL to restore Ps, final form
!        D. GIARD : 93-04-05 : NNMI with stretching (see NNMI3)
!        D. GIARD : 93-05-24 : cleaning
!        J.N. Thepaut   : 94-02-21 : tidying up of initialization of ps
!        L Isaksen : 95-06-06 : Reordering of spectral arrays
!        D. Giard  : 95-09-06 : corrections
!        M.Hamrud  : 95-10-30 : Changed CLCONF def.
!        L Isaksen : 96-02-02 : Message Passing option added
!        D.Dent     : 97-10-03 : Changed SECOND to USER_CLOCK
!        C.Temperton: 01-01-24 : Add LEULNMI to TL code
!        G. Desroziers : 02-09-16 also treat Aladin mean wind trajectory
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!***************************************************************************

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NSMAX    ,NSPEC2   ,NCPEC    ,&
 & NCPEC2   ,NTMAX    ,NTPEC2  
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPT      ,SPQ      ,SPSP,&
 & SP5A3, SP5A2  
USE YOMDYN   , ONLY : TSTEP    ,TDT
USE YOMNMIA  , ONLY : NLEX     ,NITNMI   ,NVMODM   ,NVMOD    ,&
 & NVMODPP  ,LRPIMP   ,LSPNDG   ,LRSPSL  
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NSTART   ,LSLAG    ,LTWOTL
USE YOMCT1   , ONLY : N1SDI
USE YOMCT3   , ONLY : NSTEP
USE YOMLAP   , ONLY : NASM0    ,NSTREL   ,NSPZERO
USE YOMMP    , ONLY : MYSETW   ,NPROCM   ,NUMVMO   ,NPSP
USE YOMVAR   , ONLY : LTRREF
USE TRAJECTORY_MOD, ONLY: LTRAJGP, GET_TRAJ_SPEC

IMPLICIT NONE

INTEGER(KIND=JPIM) :: I, IMODE, IMODES, ISPACE, J, JITER, JK, JN

LOGICAL :: LLRSPSL, LLSLAGX, LLTWOTLX

REAL(KIND=JPRB) :: ZT1, ZT2

!    DUMMY REAL
REAL(KIND=JPRB) :: ZDUM(1,1)

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "mo3dprj.intfb.h"
#include "mvtend.intfb.h"
#include "rdpinmi.intfb.h"
#include "spnorm.intfb.h"
#include "stepo.intfb.h"
#include "stepotl.intfb.h"
#include "sualltt.intfb.h"
#include "dealltt.intfb.h"
#include "updtim.intfb.h"

!*******************************************************
!**  ARRAYS ON THE T"NSMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVOR(NFLEVL,NSPEC2),ZDIV(NFLEVL,NSPEC2),ZT(NFLEVL,NSPEC2)
REAL(KIND=JPRB) :: ZSP(NSPEC2),ZQ(NFLEVL,NSPEC2)
!*******************************************************
!**  ARRAYS ON THE T"NTMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVORDI(NFLEVL,NTPEC2),ZDIVDI(NFLEVL,NTPEC2)
REAL(KIND=JPRB) :: ZTDI(NFLEVL,NTPEC2),ZSPDI(NTPEC2)
!***********************************************************
!**  ARRAYS ON THE T"NCMAX" OF THE REAL GEOGRAPHIC SPHERE **
!***********************************************************
REAL(KIND=JPRB) :: ZC(NCPEC2,NVMODM),ZDIAG(NCPEC2,NVMODM)
REAL(KIND=JPRB) :: ZFP(NCPEC),ZFQ(NCPEC),ZB(NCPEC2),ZLOW(NCPEC2),ZUP(NCPEC2)

CHARACTER (LEN = 9) ::  CLCONF,CLCONA
CHARACTER (LEN = 39) ::  CLMARK,CLTIME
CHARACTER (LEN = 10) ::  CLTIMEOD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       0.    PREAMBLE.
!              ---------

!*       0.1   SET UP CONFIGURATIONS FOR STEPO.

!     Compute the tendencies with CLCONF(4)='E' (see CPG) :
!          - a time step of 1 second for dynamics (ZDT=1)
!          - a time step of 0 second for physics  (ZDTPHY=0)

IF (LHOOK) CALL DR_HOOK('NNMI3TL',0,ZHOOK_HANDLE)
CLCONA(1:9)='0AAE00A?0'
IF (NTMAX == NSMAX) THEN
  CLCONA(8:8)='A'
ELSE
  CLCONA(8:8)='T'
ENDIF

!*    0.2    DIAGNOSTICS.

WRITE(NULOUT,'('' NON LINEAR 3-D NORMAL MODE INITIALIZATION'')')
IF (LRPIMP) THEN
  IF (NLEX == 0) THEN
    WRITE(NULOUT,'('' PURE IMPLICIT SCHEME  '')')
  ELSE
    WRITE(NULOUT,'('' MIXED (EXPL.-IMPL.) SCHEME  '')')
  ENDIF
ELSE
  WRITE(NULOUT,'('' PURE EXPLICIT SCHEME  '')')
ENDIF
IF (N1SDI == 1.AND.LSPNDG) THEN
  WRITE(NULOUT,'('' SPECTRUM DIAGNOSTICS BEFORE NMI ITERATIONS'')')
  CALL SPNORM(0)
ENDIF

!*    0.3    READ OF HELP ARRAYS FOR IMPLICIT NMI SCHEME

IF (LRPIMP) THEN
  IMODES = NVMOD
  CALL RDPINMI(IMODES,ZFP,ZFQ,ZB,ZLOW,ZUP,ZC,ZDIAG)
ENDIF

!*    0.5    RESET SEMI-LAGRANGIAN SWITCH.

! Convert to Eulerian model for computation of tendencies,
! having first saved the value of the semi-Lagrangian switch.

LLSLAGX  = LSLAG
LSLAG  = .FALSE.
LLTWOTLX = LTWOTL
LTWOTL = .FALSE.

!     ------------------------------------------------------------------
!     *******************************
!     ** MAIN LOOP OVER ITERATIONS **
!     *******************************

ITERATION_LOOP: DO JITER=1,NITNMI

  CALL USER_CLOCK(PTOTAL_CP=ZT1)

!       If LLRSPSL=.TRUE., Ps is not updated
!       LLRSPSL must be .FALSE. at the last iteration

  LLRSPSL=LRSPSL
  IF (JITER == NITNMI) LLRSPSL=.FALSE.

!     ------------------------------------------------------------------

!*       1.    SAVE STATE VARIABLES.
!              ---------------------

  IF (MYSETW == NPROCM(0)) THEN
    SPVOR(1:NFLEVL,NASM0(0))= 0.0_JPRB
    SPDIV(1:NFLEVL,NASM0(0))= 0.0_JPRB
  ENDIF
  ZVOR(1:NFLEVL,:)= SPVOR(1:NFLEVL,:)
  ZDIV(1:NFLEVL,:)= SPDIV(1:NFLEVL,:)
  ZT  (1:NFLEVL,:)= SPT  (1:NFLEVL,:)
  ZQ  (1:NFLEVL,:)= SPQ  (1:NFLEVL,:)

  IF (NPSP == 1) THEN
    IF (MYSETW == NPROCM(0)) THEN
      DO JN=0,NSMAX
        SPSP(NSPZERO(JN))=0.0_JPRB
      ENDDO
    ENDIF
    ZSP(:)= SPSP(:)
  ENDIF

!     ------------------------------------------------------------------

!*       2.    COMPUTE TENDENCIES.
!              -------------------

  NSTEP= NSTART-NITNMI-1+JITER
  CALL UPDTIM(0,1.0_JPRB,1.0_JPRB,.FALSE.)

  IF (.NOT.LTRAJGP) THEN
    IF (LTRREF) THEN
      CLCONF(1:9) = 'BE0000000'
    ELSE
      CALL GSTATS(15,0)
      CALL get_traj_spec(SP5A3,SP5A2,ZDUM,KSTEP=nstep)
      CALL GSTATS(15,1)
      CLCONF(1:9)='0E0000000'
    ENDIF
    CALL STEPO(CLCONF)
  ENDIF

  IF (NTMAX /= NSMAX) CALL SUALLTT
  CALL STEPOTL(CLCONA)

  IF (NTMAX == NSMAX) THEN
    IF (MYSETW == NPROCM(0)) THEN
      SPVOR(1:NFLEVL,NASM0(0))= 0.0_JPRB
      SPDIV(1:NFLEVL,NASM0(0))= 0.0_JPRB
    ENDIF
    ZVORDI(1:NFLEVL,:)= SPVOR(1:NFLEVL,:)
    ZDIVDI(1:NFLEVL,:)= SPDIV(1:NFLEVL,:)
    ZTDI  (1:NFLEVL,:)= SPT  (1:NFLEVL,:)

    IF (NPSP == 1) THEN
      ZSPDI(:)= SPSP(:)
    ENDIF

  ELSE
    CALL MVTEND(ZVORDI,ZDIVDI,ZTDI,ZSPDI)
    CALL DEALLTT
  ENDIF

!     ------------------------------------------------------------------

!*       3.    PROJECT TENDENCIES ON GRAVITY MODES
!              -----------------------------------

! IMODE=3: projection on grav. waves and division by freq.
! ISPACE=1: working space = spectral space

  IMODE=3
  ISPACE=1
  CALL MO3DPRJ(IMODE,ISPACE,NVMOD,NVMODPP,NUMVMO,ZVORDI,ZDIVDI,&
   & ZTDI,ZSPDI,ZC,ZB,ZUP,ZDIAG,ZLOW,ZFQ,ZFP)  

!     ------------------------------------------------------------------

!*       4.    UPDATE STATE VARIABLES.
!              -----------------------

  IF (NSMAX < NTMAX) THEN
    DO JK=1,NFLEVL
      DO J=1,NSPEC2
        I= NSTREL(J)
        SPVOR(JK,J)= ZVOR(JK,J) - ZVORDI(JK,I)
        SPDIV(JK,J)= ZDIV(JK,J) - ZDIVDI(JK,I)
        SPT  (JK,J)= ZT  (JK,J) - ZTDI  (JK,I)
        SPQ  (JK,J)= ZQ  (JK,J)
      ENDDO
    ENDDO
    IF (NPSP == 1) THEN
      IF (LLRSPSL) THEN
        SPSP(:)= ZSP(:)
      ELSE
        DO J=1,NSPEC2
          I= NSTREL(J)
          SPSP(J)= ZSP(J) - ZSPDI(I)
        ENDDO
      ENDIF
    ENDIF
  ELSE
    SPVOR(1:NFLEVL,:)= ZVOR(1:NFLEVL,:) - ZVORDI(1:NFLEVL,:)
    SPDIV(1:NFLEVL,:)= ZDIV(1:NFLEVL,:) - ZDIVDI(1:NFLEVL,:)
    SPT  (1:NFLEVL,:)= ZT  (1:NFLEVL,:) - ZTDI  (1:NFLEVL,:)
    SPQ  (1:NFLEVL,:)= ZQ  (1:NFLEVL,:)
    IF (NPSP == 1) THEN
      IF (LLRSPSL) THEN
        SPSP(:)= ZSP(:)
      ELSE
        SPSP(:)= ZSP(:) - ZSPDI(:)
      ENDIF
    ENDIF
  ENDIF
  IF (MYSETW == NPROCM(0)) THEN
    SPVOR(1:NFLEVL,NASM0(0))= 0.0_JPRB
    SPDIV(1:NFLEVL,NASM0(0))= 0.0_JPRB
  ENDIF

  WRITE(NULOUT,'('' NON-LINEAR NMI, ITERATION='',I2)') JITER
  IF (N1SDI == 1.AND.LSPNDG) THEN
    WRITE(NULOUT,FMT=&
     & '('' SPECTRUM DIAGNOSTICS AFTER'',I2,'' NMI ITERATION(S)'')')&
     & JITER  
    CALL SPNORM(0)
  ENDIF
  CALL USER_CLOCK(PTOTAL_CP=ZT2)
  CALL DATE_AND_TIME(TIME=CLTIMEOD)
  WRITE (UNIT=CLTIME,FMT=997)&
   & CLTIMEOD(1:2),CLTIMEOD(3:4),CLTIMEOD(5:6),ZT2-ZT1  
  WRITE (UNIT=CLMARK,FMT=998) JITER,NITNMI
  WRITE(NULOUT,*) CLTIME
  WRITE(NULOUT,*) CLMARK

!       ** END OF MAIN LOOP OVER ITERATIONS **

ENDDO ITERATION_LOOP

CALL UPDTIM(0,TDT,TSTEP,.FALSE.)

!*    5.1   RESET SEMI-LAGRANGIAN SWITCH.

LSLAG  = LLSLAGX
LTWOTL = LLTWOTLX

997 FORMAT (1X,A,'':'',A,'':'',A,1X,F11.5,' CPU S IN ITERATION')
998 FORMAT (' INITIAL. , 3D NNMI : ITERATION ',I2,' / ',I2)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NNMI3TL',1,ZHOOK_HANDLE)
END SUBROUTINE NNMI3TL
