SUBROUTINE SPEHOU3(PVORI,PDIVI,PHGTI,PFO,KVMO,KM)

!**** *SPEHOU3* - ROUTINE TO TRANSFORM FROM "STANDARD SPACE" TO
!                 HOUGH SPACE.

!     PURPOSE.   A SET OF "STANDARD SPACE" COEFFICIENTS IS USED
!     --------   TO CREATE THE CORRESPONDING SET OF FIELDS IN HOUGH
!                SPACE.

!**   INTERFACE.
!     ----------
!        *CALL* *SPEHOU3(PVORI,PDIVI,PHGTI,PFO,KVMO,KM)

!        EXPLICIT ARGUMENTS :  PVORI- VORTICITY            \
!        --------------------  PDIVI- DIVERGENCE           \ - INPUT
!                              PHGTI- GEOPOTENTIAL         \
!                              PFO  - HOUGH COMPONENTS       - OUTPUT
!                              KVMO - VERTICAL MODE INDEX    - INPUT
!                              KM   - ZONAL WAVENUMBER INDEX - INPUT
!        IMPLICIT ARGUMENTS :
!        --------------------

!     METHOD.   THE FIELDS ARE REORDERED AND NORMALIZED. WE GO INTO NORMAL
!     -------   MODE SPACE BY MULTIPLYING BY THE MATRIX CONSISTING OF THE
!               "STANDARD SPACE" EXPRESSED IN NORMAL MODES.THIS MATRIX HAS
!               BEEN PREVIOUSLY COMPUTED BY SUMODE3.

!     EXTERNALS.  REORDO3 - REORDER AND NORMALIZE FIELDS (TRUNCATION NXMAX)
!     ----------  MXMAOP  - MATRIX MULTIPLY ( using SGEMMX )
!                 MIO..   - IO ROUTINES

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        BILL HECKLEY                       *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 88-12-07
!        CLIVE TEMPERTON : 90-01-14 : SPEHOU => SPEHOU3
!        PASCAL MARQUET  : 91-02-15 : T"NCMAX" to T"NXMAX" (MIXED SCHEME)
!        MATS  HAMRUD    : 93-02-08 : MULTITASKING, IO-SCHEME
!        LARS  ISAKSEN   : 94-11-11 : MULTITASKING MOVED TO HIGHER LEVEL
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NCMAX    ,NXMAX
USE YOMSPHB  , ONLY : SPHBUF   ,NSTASPHB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORI(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVI(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHGTI(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFO(6*(NXMAX+1-KM)) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMO 
REAL(KIND=JPRB) :: ZFO(6*(NXMAX+1-KM))
INTEGER(KIND=JPIM) ::  IORDR, IREC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "reordo3.intfb.h"

!     ------------------------------------------------------------------

!*       1.    RE-ORDER AND NORMALIZE FIELDS.
!              ------------------------------

IF (LHOOK) CALL DR_HOOK('SPEHOU3',0,ZHOOK_HANDLE)
CALL REORDO3(ZFO,PVORI,PDIVI,PHGTI,KVMO,KM,1)

!     ------------------------------------------------------------------

!        2.    GO INTO NORMAL MODE SPACE.
!              --------------------------

IREC=(KVMO-1)*(NXMAX+1)+KM+1
IORDR=3*(NXMAX-KM+1)
CALL MXMAOP(SPHBUF(NSTASPHB(IREC)+1),1,IORDR,ZFO,2,1,PFO,2,1,IORDR,IORDR,2)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPEHOU3',1,ZHOOK_HANDLE)
END SUBROUTINE SPEHOU3

