SUBROUTINE SUALLTT

!**** *SUALLTT * - Allocate additional space for spectral tendencies,
!                           in case of stretching.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SUALLTT*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        D. Giard * METEO-FRANCE * , from SUALLPT

!     Modifications.
!     --------------
!        Original : 93-03-19
!        D. Giard : 93-05-24  fields set to 0.
!        L. Isaksen : 95-06-06  Reordering of spectral arrays
!        L. Isaksen : 96-02-20  Adapted for Message Passing
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVL   ,NFTHER   ,NTPEC2
USE YOMSPTT  , ONLY : SPTVOR   ,SPTDIV   ,SPTT     ,SPTSP
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

!*       1.    PREAMBLE.
!              ---------

IF (LHOOK) CALL DR_HOOK('SUALLTT',0,ZHOOK_HANDLE)
WRITE(NULOUT,FMT='(''  ALLOCATE SPACE FOR SPECTRAL TENDENCIES'')')

!*       2.    ALLOCATION.
!              -----------

!** VORTICITY
ALLOCATE(SPTVOR(NFLEVL,NTPEC2))
WRITE(NULOUT,999) 'SPTVOR   ', SIZE(SPTVOR), SHAPE(SPTVOR)
!** DIVERGENCE
ALLOCATE(SPTDIV(NFLEVL,NTPEC2))
WRITE(NULOUT,999) 'SPTDIV   ', SIZE(SPTDIV), SHAPE(SPTDIV)
!** TEMPERATURE
ALLOCATE(SPTT(NFLEVL,NTPEC2))
WRITE(NULOUT,999) 'SPTT     ', SIZE(SPTT  ), SHAPE(SPTT  )
!** SURFACE PRESSURE
ALLOCATE(SPTSP(NTPEC2))
WRITE(NULOUT,999) 'SPTSP    ', SIZE(SPTSP ), SHAPE(SPTSP )

999 FORMAT(' ARRAY ',A10,' ALLOCATED ',8I8)

!*       3.    EVERYTHING SET TO 0.
!              --------------------

SPTVOR(:,:)= 0.0_JPRB
SPTDIV(:,:)= 0.0_JPRB
IF (NFTHER >= 1) SPTT(:,:)= 0.0_JPRB

SPTSP(:)= 0.0_JPRB

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUALLTT',1,ZHOOK_HANDLE)
END SUBROUTINE SUALLTT
