SUBROUTINE SUNMI

!**** *SUNMI * - Routine to initialize the control common for NMI

!     Purpose.
!     --------
!           Initialize COMMON YOMNMIA.

!**   Interface.
!     ----------
!        *CALL* *SUNMI*

!        Explicit arguments : None.
!        --------------------

!        Implicit arguments :
!        --------------------
!              COMMON YOMNMIA
!              COMMON YOMARG

!     Method.
!     -------
!        See documentation

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original    : 87-10-15
!        Bill Heckley: 88-12-07 option to compute normal modes LCONMO
!        CLIVE TEMPERTON : 90-02-02 : ADD NVMOD FOR 3-D NMI
!        Bill Heckley: 90-06-14 options for diabatic initialization,
!                               filtering, and handling of tidal signal.
!        P. Marquet  : 91-04-01 NTIDL, LTIDEDG, FREQ2, FILTR,
!                               and options for computing LTYPxx arrays
!        Bill Heckley: 91-06-05 SET LCONMO IF FILTERING OR Jc IS REQUIRED
!        Bill Heckley: 91-10-10 SET SWITCHES IF 3/4-D VAR IF LCVROSS IS .T.
!        P. Marquet  : 91-10-15 NTIDMIN, LTIDEME, LTIDCEP
!        P. Marquet  : 91-05-11 YOMNMI --> (YOMNMIA and YOMNMIB)
!        D. Giard    : 92-01-16 some tests on LCONMO suppressed
!                               test on (LNMIDB,LEXTIDE,LSTDNMI) added
!                               tests on LEXTIDE, LNMIDB simplified
!        D. Giard    : 92-03-09 logical switch LNOINI added
!        Bill Heckley: 92-03    logical switch LVPROJ added
!        D. Giard    : 92-07-10 FILTER,REFHEI suppressed
!                               LICONV,NOPTMAR,NOPTEME,LNMIPR added
!        Bill Heckley: 93-02-02 logical switch LVPROJ removed
!                               logical switch LFLTBAL added
!        Bill Heckley: 93-03-04 Move NITNMI from SUCT0 to SUNMI
!        Bill Heckley: 93-??-?? add NVMODF,NVMODF1,NVMODF2
!        D. Giard    : 93-03-19 some additionnal tests
!        D. Giard    : 93-04-02 cleaning
!        R. El khatib: 93-02-09 control variable NEINI
!        D. Giard    : 93-04-27 LCOTYxx=.FALSE. if NEINI=0 or 2; output
!                               on NULOUT (argument KULOUT suppressed)
!        Modified by R. El Khatib  :93-05-06 Set-up through command line
!        D. Giard    : 93-05-13 test on LCOIMP/LRPIMP ; tests on NCONF ;
!                               case of 3d lnmi
!        C. Temperton: 93-11-01 changes and additions to ECMWF defaults
!        F.Rabier and J-N Thepaut 94-01-13 :
!                 a) ECMWF variational defaults
!                 b) switch to initialise increments (incremental 3d/4d)
!        D. Giard    : 94-01-09 corrections, NOPTDIA for diabatic terms
!        D. Giard    : 95-06-06 time-step for tendencies = RTACUMT
!        C. Temperton: 96-01-09 LASSI (adiabatic NMI of small scales only)
!        C. Temperton: 97-04-10 LASSI always TRUE at ECMWF for NCONF=1    
!                               (even when LOBSC1 is also TRUE)
!        D. Giard    : 99-07-19 LINITER moved from YOMNMIA to YOMINI
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M. Drusch   : 07-02-01 introduce nconf 302
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMARG   , ONLY : NARGS    ,NUFLEV   ,NUSMAX
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMNMIA  , ONLY : FILTR    ,NLEX     ,NITNMI   ,&
 & NVMODF   ,NVMODF1  ,NVMODF2  ,NVMODM   ,NVMOD    ,&
 & LNMIRQ   ,LNMI     ,LNLNMI   ,LSTDNMI  ,&
 & LRPIMP   ,LNPROJ   ,LRPROJ   ,LGPROJ   ,&
 & LFLTBAL  ,LTRANS   ,LSPNDG   ,LRGENE   ,&
 & LRSPSL   ,LCONMO   ,LCOIMP   ,LCOTYGR  ,&
 & LNOINI   ,LICONV   ,LNMIPR   ,&
 & LNMIFF   ,LASSI    ,NFILTM  
USE YOMCT0   , ONLY : NCONF    ,LSLAG    
USE YOMCT0B  , ONLY : LECMWF
USE YOMVAR   , ONLY : LJCNMI   ,LJCNMTL
USE YOMINI   , ONLY : NEINI

IMPLICIT NONE

INTEGER(KIND=JPIM) ::  J

LOGICAL :: LL2D, LL3D, LL4, LL5, LL6

REAL(KIND=JPRB) :: ZFILT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"

#include "namnmi.h"

!      -----------------------------------------------------------------

!        1.    Initialize YOMNMIA - Default values
!              ------------------------------------

! First, Meteo-France defaults
IF (LHOOK) CALL DR_HOOK('SUNMI',0,ZHOOK_HANDLE)
IF (.NOT.LECMWF) THEN

!      1.1 General variables

  NITNMI = 3
  NVMOD  = 4
  NLEX   = 26
  DO J=1,50
    FILTR(J) = 0.50_JPRB
  ENDDO
  LRPIMP = .FALSE.
  LRGENE = .FALSE.
  LTRANS = .FALSE.
  LNMIFF = .FALSE.
! initialization
  LNMIRQ = .FALSE.
  LNMI   = .FALSE.
  LNLNMI = .FALSE.
  LSTDNMI= .FALSE.
  LSPNDG = .FALSE.
  LRSPSL = .FALSE.
  LICONV = .FALSE.
  LASSI  = .FALSE.
! projection
  LNPROJ = .FALSE.
  LRPROJ = .FALSE.
  LGPROJ = .FALSE.
! variational analysis filtering and balance
! (initialized in SUJG0 if NCONF/100=1)
  IF (NCONF/100 == 1) THEN
    NVMODF = -999
    NVMODF1= -999
    NVMODF2= -999
  ELSE
    NVMODF = NVMOD
    NVMODF1= NVMOD
    NVMODF2= NVMOD
  ENDIF
! computing help arrays
  LCONMO = .FALSE.
  LCOIMP = .FALSE.
  LCOTYGR= .FALSE.
  LNMIPR = .FALSE.

!     Modifications if command line is used

  IF (NARGS /= 0) THEN
    LNMIRQ =.TRUE.
    LICONV =.TRUE.
    LNLNMI =.TRUE.
    LSTDNMI=.TRUE.
    LNMIFF =.TRUE.
    LRPIMP =.TRUE.
    LRGENE =.TRUE.
    LCONMO =.TRUE.
    LCOIMP=.TRUE.
    NVMOD=4
    FILTR(1)=.50_JPRB
    FILTR(2)=.40_JPRB
    IF (NUFLEV == 15) THEN
      FILTR(3)=.30_JPRB
      FILTR(4)=.23_JPRB
    ELSEIF (NUFLEV == 21.OR.NUFLEV == 24) THEN
      FILTR(3)=.30_JPRB
      FILTR(4)=.25_JPRB
    ELSEIF (NUFLEV == 27.OR.NUFLEV == 31) THEN
      FILTR(3)=.31_JPRB
      FILTR(4)=.26_JPRB
    ENDIF
    IF (NUSMAX == 21) THEN
      NLEX=13
    ELSE
      NLEX=26
    ENDIF
  ENDIF

! Then, ECMWF defaults

ELSE

!      1.3 General variables

  NITNMI = 2
  NVMOD  = 5
  NLEX   = 21
  DO J=1,50
    FILTR(J) = 0.50_JPRB
  ENDDO
  LRPIMP = .TRUE.
  LRGENE = .TRUE.
  LTRANS = .FALSE.
  LNMIFF = .FALSE.
! initialization
  LNMIRQ = .TRUE.
  LNMI   = .FALSE.
  LNLNMI = .TRUE.
  LSTDNMI= .FALSE.
  LSPNDG = .FALSE.
  LRSPSL = .TRUE.
  LICONV = .FALSE.
  IF (NCONF == 1.OR.NCONF == 302) THEN
    LASSI = .TRUE.
  ELSE
    LASSI = .FALSE.
  ENDIF
  NFILTM = 19 ! Only used for LASSI=TRUE
! projection
  LNPROJ = .FALSE.
  LRPROJ = .FALSE.
  LGPROJ = .FALSE.
! variational analysis filtering and balance (initialised in SUJG0)
  NVMODF = -999
  NVMODF1= -999
  NVMODF2= -999
! computing help arrays
  LCONMO = .TRUE.
  LCOIMP = .TRUE.
  LCOTYGR= .FALSE.
  LNMIPR = .FALSE.

ENDIF

!      1.5 Internal variables

LNOINI  = .FALSE.
LFLTBAL = .FALSE.

IF (.NOT.LECMWF) NVMODM = NVMOD

LL4     = .FALSE.
LL5     = .FALSE.
LL6     = .FALSE.
LL2D    = .FALSE.
LL3D    = .FALSE.

!     ------------------------------------------------------------------

!*       2.    Modify default values - Read NAMNMI
!              -----------------------------------

!        2.1   Read namelist

CALL POSNAM(NULNAM,'NAMNMI')
READ(NULNAM,NAMNMI)

!     ------------------------------------------------------------------

!*       3.    Modify values as appropriate.
!              -----------------------------

!      3.0 Will normal modes be used ?

! NEINI : 0   NO INITIALIZATION
!         1   NORMAL MODE INITIALIZATION
!         2   DIGITAL FILTER INITIALIZATION ; NORMAL MODES ARE NOT USED
!         3   NO INITIALIZATION BUT NORMAL MODES ARE USED
!         4   DIGITAL FILTER INITIALIZATION ; NORMAL MODES ARE USED

IF (NEINI == 0.OR.NEINI == 2) THEN
  LNOINI = .TRUE.
  LCONMO = .FALSE.
  LCOIMP = .FALSE.
  LCOTYGR= .FALSE.
  LNMIRQ = .FALSE.
  LJCNMI = .FALSE.
  LJCNMTL= .FALSE.
  LNPROJ = .FALSE.
  NITNMI = 0
  NVMOD = 0

  WRITE(UNIT=NULOUT,FMT='(/,'' YOMNMIA VARIABLES MODIFIED BY SUNMI'')')

  WRITE(UNIT=NULOUT,FMT=&
   & '('' LNOINI='',L2,'' LCONMO='',L2,'' LCOIMP='',L2,&
   & '' LCOTYGR='',L2,&
   & '' LNMIRQ='',L2,'' LJCNMI='',L2,'' LJCNMTL='',L2,&
   & '' LNPROJ='',L2,'' NITNMI='',I4,'' NVMOD'',I6)')&
   & LNOINI,LCONMO,LCOIMP,LCOTYGR,LNMIRQ,LJCNMI,&
   & LJCNMTL,LNPROJ,NITNMI,NVMOD  

  IF (LHOOK) CALL DR_HOOK('SUNMI',1,ZHOOK_HANDLE)
  RETURN
ELSE
  IF (NEINI == 3.OR.NEINI == 4) THEN
    LNMIRQ = .FALSE.
    LNPROJ = .FALSE.
  ENDIF
ENDIF

!      3.1 Stupid values --> abor1

IF (NVMOD < 1.OR.NVMOD > 50.OR.NLEX < 0)&
 & CALL ABOR1(' NAMNMI : STUPID VALUE FOR NVMOD OR NLEX')  
ZFILT = 0._JPRB
DO J=1,NVMOD
  ZFILT = MAX (ZFILT,FILTR(J))
ENDDO
IF (ZFILT == 0.0_JPRB)CALL ABOR1(' NAMNMI : STUPID VALUES IN FILTR')

!      3.2 Initialization requested

! configuration
LL2D=NCONF == 201.OR.NCONF == 421.OR.NCONF == 521
LL3D=NCONF == 1.OR.NCONF == 131.OR.NCONF == 302 &
 & .OR.NCONF == 401.OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801  

IF (LNMIRQ.OR.NCONF == 131) THEN

! check configuration
  IF (.NOT.(LL2D.OR.LL3D))&
   & CALL ABOR1(' NMI NOT CODED FOR THIS CONFIGURATION !')  

  LNPROJ = .FALSE.
! LASSI --> adiabatic, implicit
  IF (LASSI) THEN
    LRPIMP = .TRUE.
    LRSPSL = .FALSE.
    NLEX   = 0
  ENDIF
! nonlinear NMI
  IF (LNLNMI) THEN

    LNMI = .FALSE.
    IF (.NOT.LRGENE .OR. (NCONF /= 1.AND.NCONF /= 302)) LICONV = .FALSE.
    IF (NITNMI < 1.AND. .NOT.LICONV)&
     & CALL ABOR1(' NNMI : AT LEAST 1 ITERATION REQUIRED !')  
! 2d nnmi --> adiabatic
    IF (LL2D) THEN
      LRSPSL = .FALSE.
    ENDIF
! linear NMI --> adiabatic
  ELSEIF (LNMI) THEN
! 3d lnmi performed with LNPROJ=LRPROJ=.TRUE.
    IF (LL3D) THEN
      LL5 = .TRUE.
      LNMIRQ = .FALSE.
      LNPROJ = .TRUE.
      LRPROJ = .TRUE.
    ENDIF
! neither nonlinear nor linear NMI --> abor1
  ELSE
    CALL ABOR1(' NAMNMI : TYPE OF INITIALIZATION NOT CHOSEN')
  ENDIF
ENDIF

!      3.3 Projection requested

IF (LNPROJ) THEN
  IF (LL2D)CALL ABOR1(' PROJECTION PERFORMED WITH LNMI IF 2D !')
  IF (.NOT.LRPROJ.AND..NOT.LGPROJ)&
   & CALL ABOR1(' NAMNMI : TYPE OF PROJECTION NOT CHOSEN')  
ENDIF

!      3.4 General

IF (NEINI == 1.AND. .NOT.(LNMIRQ.OR.LNPROJ)) LL6=.TRUE.
IF (.NOT.LRPIMP) LCOIMP=.FALSE.
! Checking NVMODF,NVMODF1,NVMODF2
IF (NCONF/100 /= 1) THEN
  NVMODF = NVMOD
  NVMODF1= NVMOD
  NVMODF2= NVMOD
ENDIF
! NVMODM can be changed in SUJG0 if NCONF/100=1
NVMODM = NVMOD

!      3.5 Variational configurations

IF (LJCNMI.OR.LJCNMTL) LNOINI = .FALSE.

!     COMPUTATION OF THE MODES IS NEEDED FOR Jc IN 3&4-D VAR
IF ((NCONF==131).AND.(LJCNMI.OR.LJCNMTL)) THEN
  IF (.NOT.LRPIMP .OR. (LRPIMP.AND.NLEX > 0)) LCONMO=.TRUE.
  IF (LRPIMP) LCOIMP=.TRUE.
ENDIF

!     ------------------------------------------------------------------

!*       4.    Print final values and warnings
!              -------------------------------

WRITE(UNIT=NULOUT,FMT='(/,'' COMMON YOMNMIA'')')

!      4.1 Final namelist

WRITE(UNIT=NULOUT,FMT=&
 & '('' LRPIMP='',L2,'' LRGENE='',L2,&
 & '' LTRANS='',L2,'' LNMIFF='',L2)')&
 & LRPIMP,LRGENE,LTRANS,LNMIFF  

WRITE(UNIT=NULOUT,FMT=&
 & '('' LNMIRQ='',L2,'' LNMI='',L2,'' LNLNMI='',L2,&
 & '' LASSI='',L2,'' LSTDNMI='',L2,&
 & '' LSPNDG='',L2,'' LRSPSL='',L2,'' LICONV='',L2)')&
 & LNMIRQ,LNMI,LNLNMI,LASSI,LSTDNMI,LSPNDG,LRSPSL,LICONV  

WRITE(UNIT=NULOUT,FMT=&
 & '('' LNPROJ='',L2,'' LRPROJ='',L2,'' LGPROJ='',L2)')&
 & LNPROJ,LRPROJ,LGPROJ  

WRITE(UNIT=NULOUT,FMT=&
 & '('' LCONMO='',L2,'' LCOIMP='',L2,'' LCOTYGR='',L2,&
 & '' LNMIPR='',L2)')&
 & LCONMO,LCOIMP,LCOTYGR,LNMIPR  

WRITE(UNIT=NULOUT,FMT='('' FILTR(1:NVMOD) :  '',&
 & 5(10(1X,F7.3),/) )') (FILTR(J),J=1,NVMOD)  

WRITE(UNIT=NULOUT,FMT=&
 & '('' NVMOD='',I3,'' NLEX='',I3,'' NITNMI='',I3,&
 & '' NVMODF='',I4,'' NVMODF1='',I4,'' NVMODF2='',I4)')&
 & NVMOD,NLEX,NITNMI,NVMODF,NVMODF1,NVMODF2  
WRITE(UNIT=NULOUT,FMT='('' NFILTM='',I3)') NFILTM

!      4.2 Description of choice

IF (LCONMO) THEN
  WRITE(NULOUT,'(A)')&
   & ' ',' *** EXPLICIT NORMAL MODES WILL FIRST BE COMPUTED ***'  
  IF (LNMIPR) THEN
    WRITE(NULOUT,'(A)') '  THEIR FREQUENCIES WILL BE PRINTED'
  ELSE
    WRITE(NULOUT,'(A)') '  THEIR FREQUENCIES WON''T BE PRINTED'
  ENDIF
ELSE
  WRITE(NULOUT,'(A)')&
   & ' ',' *** EXPLICIT NORMAL MODES WILL BE READ ON FILES  ***'  
  IF (LCOTYGR) THEN
    WRITE(NULOUT,'(A)')' *** EXPLICIT TYPE ARRAY WILL BE COMPUTED  ***'
  ELSE
    WRITE(NULOUT,'(A)')' *** EXPLICIT TYPE ARRAY WILL BE READ ON FILES  ***'
  ENDIF
ENDIF
IF (LCOIMP) THEN
  WRITE(NULOUT,'(A)')' *** IMPLICIT OPERATORS WILL BE COMPUTED ***'
ELSEIF (LRPIMP) THEN
  WRITE(NULOUT,'(A)')' *** IMPLICIT OPERATORS WILL BE READ ON FILES  ***'
ENDIF

WRITE(NULOUT,'(A)') ' ','     INITIALIZATION PARAMETERS'
WRITE(NULOUT,'(A,I3)') ' NUMBER OF VERTICAL MODES ',NVMOD
WRITE(NULOUT,'(A,L2)')' SWITCH FOR PARTIALLY IMPLICIT INITIALIZATION ',LRPIMP
IF (LRPIMP) THEN
  WRITE(NULOUT,'(A,I3)')' WAVENUMBER LIMIT FOR EXPLICIT INITIALIZATION ',NLEX
ELSEIF (LTRANS) THEN
  WRITE(NULOUT,&
   & '('' EXPLICIT NORMAL MODES WILL BE COMPUTED ON THE '',&
   & ''TRANSFORMED SPHERE, IF POSSIBLE'')')  
ENDIF
IF (LNMIFF) THEN
  WRITE(NULOUT,'(A)')' EXPLICIT MODES ARE DIVIDED IN FAST / SLOW MODES '
ELSE
  WRITE(NULOUT,'(A)')' EXPLICIT MODES ARE DIVIDED IN GRAVITY / ROSSBY MODES '
ENDIF
IF (LRGENE) WRITE(NULOUT,'(A)')&
 & ' THE ENERGY IN THE GRAVITY PART WILL BE COMPUTED'  

IF (LNMIRQ) THEN
  WRITE(NULOUT,'(A)')&
   & ' ','     NORMAL MODE INITIALIZATION WILL BE CARRIED OUT.'  
  IF (LNLNMI) THEN
    WRITE(NULOUT,'(A)') ' INITIALIZATION IS NON-LINEAR'
    WRITE(NULOUT,'(A,I3)') ' NUMBER OF ITERATIONS ',NITNMI
    IF (LASSI) WRITE(NULOUT,'(A)')' ADIABATIC NMI OF SMALL SCALES ONLY'
    IF (LSPNDG) WRITE(NULOUT,'(A)')&
     & ' A SPECTRAL DIAGNOSTIC WILL BE MADE AT EACH ITERATION'  
    IF (LRSPSL) THEN
      WRITE(NULOUT,'('' SURFACE PRESSURE WILL BE RESTORED''&
       & ,'' ON ALL BUT THE LAST ITERATION'')')  
    ELSE
      WRITE(NULOUT,'(A)')' SURFACE PRESSURE WILL NOT BE RESTORED'
    ENDIF
    IF (LSLAG) THEN
      WRITE(NULOUT,'(A)')' EULERIAN DYNAMICAL TENDENCIES WILL BE USED'
    ENDIF
    IF (LICONV) WRITE(NULOUT,&
     & '('' AN ADDITIONNAL, DUMMY ITERATION WILL BE PERFORMED,''&
     & ,'' TO TEST CONVERGENCE'')')  
  ELSE
    WRITE(NULOUT,'(A)')' INITIALIZATION IS LINEAR AND ADIABATIC (2D)'
  ENDIF
ENDIF

IF (LNPROJ) THEN
  WRITE(NULOUT,'(A)')&
   & ' ','     INITIALIZATION WILL NOT BE CARRIED OUT,',&
   & ' BUT FIELDS WILL BE FILTERED BY PROJECTION ON TO NORMAL MODES'  
  IF (LRPROJ.AND.LGPROJ) THEN
    WRITE(NULOUT,'(A)')&
     & ' FIELDS WILL BE PROJECTED ONTO BOTH ROSSBY AND GRAVITY &
     & MODES'  
  ELSEIF (LRPROJ) THEN
    WRITE(NULOUT,'(A)')' FIELDS WILL BE PROJECTED ONTO ROSSBY MODES ALONE'
  ELSEIF (LGPROJ) THEN
    WRITE(NULOUT,'(A)')' FIELDS WILL BE PROJECTED ONTO GRAVITY MODES ALONE'
  ENDIF
ENDIF

!      4.3 Warnings

IF (LL4) WRITE(NULOUT,'(A)')&
 & ' ! SOME OPTIONS ARE NOT AVAILABLE FOR 2D NNMI !!'  
IF (LL5) WRITE(NULOUT,'(A)')&
 & ' ! 3D LNMI REQUIRES LNPROJ=LRPROJ=.TRUE., NOT LNMIRQ !!'  
IF (LL6) WRITE(NULOUT,'(A)')' ! NEINI=1 BUT NO NMI REQUESTED !!'

WRITE(NULOUT,'(A)') ' '

!     ------------------------------------------------------------------
!     ------------------------------------------------------------------

!*       5.    EXAMPLES OF NAMELISTS :
!              -----------------------

!    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!       Mixed explicit/implicit 3D diabatic NNMI, for a streched     :
!     model, with an exlusion of the tides using the CEPMMT's method :
!    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!----------------
! &NAMCT0
!    LECMWF = .F. : check defaults else !
!    NCONF  = 1   : a direct 3D model is required.
!    NSTOP  = 3   : there will be 3 time steps after the NMI.
!     .......
! &END
!----------------
! &NAMDIM
!    NDGLG  = 192 : number of points (latitude).
!    NDGLON = 384 : number of points (longitude).
!    NFLEVG =  21 : number of levels for the 3D model.
!    NSMAX  = 127 : the truncation of the 3D model (transformed sphere).
!    NTMAX  = 129 : the truncation for tendencies (transformed sphere).
!    NCMAX  = 511 : large implicit and mixed trunc. on the real geog. sphere.
!    NXMAX  =  30 : small explicit truncation on the real geographic sphere.
!     .......
! &END
!----------------
! &NAMGEM
!    RSTRET  = 3.5  : the stretching factor for the stretched model.
!    RMUCEN  = xx.x : sinus of the latitude of the pole of interest.
!    RLOCEN  = xx.x : longitude of the pole of interest (in radiant).
!    NSTTYP  =  2   : the pole is tilted.
!     .......
! &END
!----------------

! &NAMNMI

!     LNMIRQ=.TRUE.     : a NNMI is required.
!     LSTDNMI=.TRUE.    : the standard NNMI is required (call to CNMI in CNT3,
!                         otherwise CMAC is called in CNT3)

!     LCONMO =.????.    | if it is TRUE  :  the operators are computed and
!     LCOTYGR=.????.    |                   written on units 61 to 65 (YOMLUN).
!     LCOIMP =.????.    |                   been previously computed and stored.

!     NITNMI = 2        : 2 iterations of NMI scheme before the run of the model
!     NVMOD  =  4       : only the 4 first vertical modes will be initialized.
!                         (highest equivalent depth)
!     LNMIFF = .TRUE.   : the gravity/Rossby modes are distinguished
!                         purely on the basis of their frequencies
!     FILTR(1)=0.50     |
!     FILTR(2)=0.40     | to define the cut off values for the definition of
!     FILTR(3)=0.30     | the gravity/Rossby components, these values can
!     FILTR(4)=0.25     | therefore be different for the various vertical modes.

!     LNLNMI = .TRUE.   : the NMI will be non-linear.
!     LTRANS = .FALSE.  : the NNMI will not be realized on the transformed sph.
!                        (the method of the appendix of Courtier-Geleyn 1988
!                         is not used).
!     LRPIMP = .TRUE.   : the NNMI will be partially implicit (mixed scheme).
!                       | the implicit increments are retained for n>20,
!     NLEX   =   25     | the explicit increments are retained for n<=25
!                       | the implicit increments are retained for n>25,
!     LSPNDG = .FALSE.  : the spectral diagnostic is not req. after each iter.
!     LRGENE = .TRUE.   : the energy in the gravity part is computed and written
!                         for each mode and at each iteration of the NNMI.

! &END

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNMI',1,ZHOOK_HANDLE)
END SUBROUTINE SUNMI
