SUBROUTINE VTRANMAD(KSPEC2,PVORDI,PDIVDI,PTDI,PSPDI,PVOR,PDIV,PT,&
 & KVMOD,KSIGN,KMODE)  

!**** *VTRANMAD* - MACRO TASKING INTERFACE TO VERTICAL TRANSFORMS FOR NMI.

!     PURPOSE.   MACRO TASKING INTERFACE TO VTRANAD
!     -------- 

!**   INTERFACE.
!     ----------
!      *CALL* *VTRANMAD(KSPEC2,PVORDI,PDIVDI,PTDI,PSPDI,PVOR,PDIV,PT,KSIGN,KMODE)

!        EXPLICIT ARGUMENTS :  KSPEC2 - "HORIZONTAL" DIMENSION OF ARRAYS
!        ------------------
!                              PVORDI - VORTICITY   ) SPECTRAL COEFFS IN
!                              PDIVDI - DIVERGENCE  ) THE HORIZONTAL,
!                KMODE=1 |.....PTDI   - "P"         ) "PHYSICAL" SPACE IN
!                        |.....PSPDI  - NOT USED    ) THE VERTICAL

!                KMODE=2 |.....PTDI   - TEMPERATURE ) "PHYSICAL" SPACE IN
!                        |.....PSPDI  - LN(PS)      ) THE VERTICAL

!                              PVOR   - VORTICITY   ) SPECTRAL COEFFS IN THE
!                              PDIV   - DIVERGENCE  ) HORIZONTAL, "NORMAL MODE"
!                              PT     - "P"         ) SPACE IN THE VERTICAL

!                              KVMOD  - NUMBER OF VERTICAL MODES
!                              KSIGN  - = +1 FOR TRANSFORMS FROM NORMAL MODE
!                                            SPACE TO PHYSICAL SPACE,
!                                       = -1 FOR TRANSFORMS FROM PHYSICAL SPACE
!                                            TO NORMAL MODE SPACE.

!        IMPLICIT ARGUMENTS : none
!        ------------------

!     METHOD.   MULTIPLICATION OF COLUMN VECTORS BY THE MATRIX OF
!     -------   VERTICAL NORMAL MODES, OR ITS INVERSE. EXTRA
!               MANIPULATIONS BETWEEN T, LN(PS) AND "P", DEPENDING
!               ON KMODE.

!         see ARPEGE documentation vol1 chapter 5

!     EXTERNALS.  VTRANAD - Vertical transform during NMI
!     ----------  

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        Lars Isaksen *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 95-01-11 
!        Modified : 95-06-06 L. Isaksen - Reordering of spectral arrays
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEC2 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORDI(NFLEVG,KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVDI(NFLEVG,KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTDI(NFLEVG,KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDI(KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVOR(KSPEC2,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIV(KSPEC2,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KSPEC2,NFLEVG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMOD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSIGN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "vtranad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VTRANMAD',0,ZHOOK_HANDLE)
CALL VTRANAD(KSPEC2,KSPEC2,PVORDI,PDIVDI,PTDI,PSPDI,&
 & PVOR,PDIV,PT,KVMOD,KSIGN,KMODE)  
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VTRANMAD',1,ZHOOK_HANDLE)
END SUBROUTINE VTRANMAD
