SUBROUTINE DEPERERR(PHOUPDAY,PPI,KANCDY,PDAYPYEA,PG)

!**** SUBROUTINE DEPERERR - DEFINE OBS. PERSISTENCE ERRORS

!        D. VASILJEVIC    ECMWF    17/10/1994

!     PURPOSE
!     -------

!        DEFINE OBSERVATION PERSISTENCE ERROR

!**   INTERFACE
!     ---------

!         CALL DEPERERR(PHOUPDAY,PPI,KANCDY,PDAYPYEA,PG)
!     WHERE:
!           PHOUPDAY = NO. OF HOURS PER DAY
!           PPI      = PI
!           KANCDY   = CENTURY DAY OF ANALYSIS DAY
!           PDAYPYEA = NO. OF DAYS PER YEAR
!           PG       = g

!         DEPERERR IS CALLED BY SUBROUTINE SUPERERR

!     METHOD
!     ------

!        SET ALL OBS. PERSISTENCE ERROR RELATED CONSTANTS/PARAMETERS.

!**   HISTORY
!     -------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPERR  , ONLY : UPM10070 ,UPM69925 ,UPM24900 ,VPM10070 ,&
 & VPM69925 ,VPM24900 ,GPM10070 ,GPM69925 ,GPM24900 ,&
 & TPM10070 ,TPM69925 ,TPM24900 ,UP110070 ,UP169925 ,&
 & UP124900 ,VP110070 ,VP169925 ,VP124900 ,GP110070 ,&
 & GP169925 ,GP124900 ,TP110070 ,TP169925 ,TP124900 ,&
 & RPERER1  

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PHOUPDAY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KANCDY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDAYPYEA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PG 
REAL(KIND=JPRB) :: ZTERM1, ZTERM2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

!*       1.       MAX. PERSISTENCE ERROR (OVER 24HR)
!                 ----------------------------------
!                       1000-700, 699-250, 249-  0   (MB)
!                  U      6.4       12.7     19.1    (M/S)
!                  V      6.4       12.7     19.1    (M/S)
!                  Z     48.        60.      72.      (M)
!                  T      6.         7.       8.     (DEGREES) 

!*          1.1   U COMPONENT

IF (LHOOK) CALL DR_HOOK('DEPERERR',0,ZHOOK_HANDLE)
UPM10070=  6.4_JPRB
UPM69925= 12.7_JPRB
UPM24900= 19.1_JPRB

!*          1.2   V COMPONENT

VPM10070=  6.4_JPRB
VPM69925= 12.7_JPRB
VPM24900= 19.1_JPRB

!*          1.3   Z

GPM10070= 48._JPRB*PG
GPM69925= 60._JPRB*PG
GPM24900= 72._JPRB*PG

!*          1.4   T

TPM10070=  6._JPRB
TPM69925=  7._JPRB
TPM24900=  8._JPRB
!*
!-----------------------------------------------------------------------

!        2.       MAX. PERSIST. ERROR (OVER 1HR)
!                 ------------------------------

!*          2.1   U COMPONENT

UP110070=  UPM10070 / PHOUPDAY
UP169925=  UPM69925 / PHOUPDAY
UP124900=  UPM24900 / PHOUPDAY

!*          2.2   V COMPONENT

VP110070=  VPM10070 / PHOUPDAY
VP169925=  VPM69925 / PHOUPDAY
VP124900=  VPM24900 / PHOUPDAY

!*          2.3   Z

GP110070=  GPM10070 / PHOUPDAY
GP169925=  GPM69925 / PHOUPDAY
GP124900=  GPM24900 / PHOUPDAY

!*          2.4   T

TP110070=  TPM10070 / PHOUPDAY
TP169925=  TPM69925 / PHOUPDAY
TP124900=  TPM24900 / PHOUPDAY
!*
!-----------------------------------------------------------------------

!        3.       PERSISTENCE ERROR (TERM 1)
!                 --------------------------

ZTERM1  = 2.0_JPRB*PPI*MOD(REAL(KANCDY,JPRB),PDAYPYEA) / PDAYPYEA
ZTERM2  = ZTERM1 + PPI/2.0_JPRB
RPERER1 = SIN(ZTERM2)
!*
!-----------------------------------------------------------------------

!        4.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('DEPERERR',1,ZHOOK_HANDLE)
END SUBROUTINE DEPERERR
