SUBROUTINE PEREREV(KPAR,PLEV1,PLAT,POBTIM,&
 & PWEIGHT,&
 & KOUT,KVNUMB,KVNUMBL,PLAT20,PP700,PP250,&
 & PREDPERR,PMINPHOU,PADIANS,PMDI,&
 & PERERR)  

!**** SUBROUTINE PEREREV - PERSISTENCE ERROR EVALUATION (TERM 2,
!                          AND FINAL)

!        D. VASILJEVIC    ECMWF    17/10/1994

!     PURPOSE
!     -------

!        FINAL PERSISTENCE ERROR EVALUATION

!**   INTERFACE
!     ---------

!        CALL PEREREV(KPAR,PLEV1,PLAT,POBTIM,
!    1                PWEIGHT,
!    2                KOUT,KVNUMB,KVNUMBL,PLAT20,PP700,PP250,
!    3                PREDPERR,PMINPHOU,PADIANS,PMDI,
!    4                PERERR)

!     WHERE:
!            KPAR     = VARIABLE FOR WHICH PERSIST. ERROR IS LOOKED FOR
!            PLEV1    = PRESSURE LEVEL 1 (VERTICAL REF. 1)
!            PLAT     = LAT.
!            POBTIM   = TIME DIFFERENCE
!            PWEIGHT  = WEIGHT
!            KOUT     = OUTPUT UNIT
!            KVNUMB   = VARIABLE'S NO.s ARRAY
!            KVNUMBL  = LEN. OF KVNUMB ARRAY
!            PLAT20   = LAT 20D (CONSTANT)
!            PP700    = 700hPa (CONSTANT)
!            PP250    = 250hPa (CONSTANT)
!            PREDPERR = HOUTLY PERSISTENCE ERROR REDUCTION CONSTANT
!            PMINPHOU = NO. OF MINUTES PER HOUR
!            PADIANS  = RADIANCE
!            PMDI     = MISSING INDICATOR
!            PERERR   = FINAL PER. ERR.

!        PEREREV IS CALLED BY SUBROUTINES:

!     METHOD
!     ------

!        THE PERSISTENCE ERROR ITSELF IS FORMULATED AS:
!           ERRPER  =  (ERPRM1/6)*(1+2*SIN(ABS(2*LAT)))* B * DT
!     WHERE,
!           ERRPER = PERSISTENCE ERROR
!           ERPRM1 = MAX. PERSISTENCE ERROR FOR 1 HOUR
!           LAT    = LATITUDE OF OBSERVATION IN QUESTION
!           B      = 1.5 + (0.5*MIN(MAX(LAT,-20),20)/20) * A (TERM 2)
!           A      = SIN(2*PI*(DAY/365.25)+PI/2)             (TERM 1)
!           DAY    = DAY OF YEAR
!           DT     = FRACTION OF DAY
!     PARAMETER B (TERM 2) AND ERRPER (FINAL PERSIST. ERROR) ARE CALC-
!     ULATED IN THIS ROUTINE.  IN ADDITION TO THAT, IF OBSERVATION IS
!     SYNOP LAND OR SYNOP SHIP AND PRESSURE TENDENCY CORRECTION HAS
!     BEEN APPLIED FINAL PERSISTENCE ERROR IS REDUCED. REDUCTION FACTOR
!     4 IS USED IF CORRECTION IS FOR LAND STATION OR FOR A SHIP WHICH
!     TENDENCY HAS ALSO BEEN CORRECTED FOR A SHIP MOVEMENT. REDUCTION
!     FACTOR IS 2 IF NO ACCOUNT OF SHIP MOVEMENT HAS BEEN POSSIBLE.

!**   HISTORY
!     -------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPERR  , ONLY : UP110070 ,UP169925 ,UP124900 ,VP110070 ,&
 & VP169925 ,VP124900 ,GP110070 ,GP169925 ,GP124900 ,&
 & TP110070 ,TP169925 ,TP124900 ,RPERER1  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KVNUMBL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPAR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBTIM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWEIGHT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOUT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVNUMB(KVNUMBL) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT20 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PP700 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PP250 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREDPERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMINPHOU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PADIANS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMDI 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PERERR 
REAL(KIND=JPRB) :: Z1OMIPHO, ZMX01ERR, ZPERER2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!-----------------------------------------------------------------------

!        1.       PERSISTENCE ERROR (TERM 2)
!                 --------------------------

IF (LHOOK) CALL DR_HOOK('PEREREV',0,ZHOOK_HANDLE)
ZPERER2 = 1.5_JPRB + (0.5_JPRB*MIN(MAX(PLAT,-PLAT20),PLAT20)/PLAT20)*RPERER1

!-----------------------------------------------------------------------

!*       2.       DEFINE WEIGHT. FACTOR DUE TO TENDENCY CORR. (IT IS AN
!                 -----------------------------------------------------
!                 INPUT ARGUMENT NOW)
!                 -------------------

!*
!-----------------------------------------------------------------------

!        3.       FIND OUT PARAMETER/LAYER
!                 ------------------------

ZMX01ERR = PMDI

!*          3.1   U COMPONENT

IF(KPAR == KVNUMB( 1).OR.KPAR == KVNUMB(56)    )             THEN

!*          3.1.1 BELOW 700hPa

  IF(PLEV1 >=  PP700)                    THEN
    ZMX01ERR = UP110070

!*          3.1.2 BELOW 250hPa

  ELSEIF(PLEV1 >=  PP250)               THEN
    ZMX01ERR = UP169925

!*          3.1.3 ABOVE 250hPa

  ELSE
    ZMX01ERR = UP124900
  ENDIF

!*          3.2   V COMPONENT

ELSEIF(KPAR == KVNUMB( 2).OR.KPAR == KVNUMB(57)    )        THEN

!*          3.2.1 BELOW 700hPa

  IF(PLEV1 >=  PP700)                    THEN
    ZMX01ERR = VP110070

!*          3.2.2 BELOW 250hPa

  ELSEIF(PLEV1 >=  PP250)               THEN
    ZMX01ERR = VP169925

!*          3.2.3 ABOVE 250hPa

  ELSE
    ZMX01ERR = VP124900
  ENDIF

!*          3.3   TEMPERATURE

ELSEIF(KPAR == KVNUMB( 8).OR.KPAR == KVNUMB(10)    )        THEN

!*          3.3.1 BELOW 700hPa

  IF(PLEV1 >=  PP700)                    THEN
    ZMX01ERR = TP110070

!*          3.3.2 BELOW 250hPa

  ELSEIF(PLEV1 >=  PP250)               THEN
    ZMX01ERR = TP169925

!*          3.3.3 ABOVE 250hPa

  ELSE
    ZMX01ERR = TP124900
  ENDIF

!*          3.4   GEOPOTENTIAL

ELSEIF(KPAR == KVNUMB( 3))        THEN

!*          3.4.1 BELOW 700hPa

  IF(PLEV1 >=  PP700)                THEN
    ZMX01ERR = GP110070

!*          3.4.2 BELOW 250hPa

  ELSEIF(PLEV1 >=  PP250)           THEN
    ZMX01ERR = GP169925

!*          3.4.3 ABOVE 250hPa

  ELSE
    ZMX01ERR = GP124900
  ENDIF

!*          3.5   NOT KNOWN PARAMETER

ELSE
  WRITE(KOUT,'(''0ERROR IN SUBROUTINE PEREREV'')')
  WRITE(KOUT,'('' NOT KNOWN OBS. VARIABLE'',I10)') KPAR
  CALL ABOR1('SUBROUTINE PEREREV')
ENDIF

!*          3.6   REDUCE MAX. HOURLY PER. ERROR BY 6
!                 (IT IS LIKE THAT IN THE OI)

ZMX01ERR = ZMX01ERR / PREDPERR
!*
!-----------------------------------------------------------------------

!        4.       FINAL PERSISTENCE ERROR
!                 -----------------------

Z1OMIPHO=  1.0_JPRB / PMINPHOU
PERERR  =  ZMX01ERR *&
 & (1.0_JPRB+2.0_JPRB*SIN(2.0_JPRB*ABS(PLAT)*PADIANS)) *&
 & ZPERER2 *&
 & ABS(POBTIM) *&
 & PWEIGHT *&
 & Z1OMIPHO  
!*
!-----------------------------------------------------------------------

!        5.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('PEREREV',1,ZHOOK_HANDLE)
END SUBROUTINE PEREREV
