SUBROUTINE SUOBSERR

!**** SUBROUTINE SUOBSERR - OBSERVATION ERRORS

!        D. VASILJEVIC    ECMWF    17/10/1994

!     PURPOSE
!     -------

!        SET OBSERVATION ERROR CONSTANTS

!**   INTERFACE
!     ---------

!        SUOBSERR IS CALLED BY SUBROUTINE MAKECMA

!     METHOD
!     ------

!        SET ALL OBS. ERROR CONSTANTS/PARAMETERS.

!**   EXTERNALS
!     ---------

!        CMOCTMAP- MAP OBS./CODE TYPE

!**   HISTORY
!     -------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-DEc-2003 CY28R1 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCMA   , ONLY : NMXOTP   ,NMXOCT
USE YOMCST   , ONLY : RG
USE YOMANCS  , ONLY : RMDI
USE YOMOERR  , ONLY : NMXESTLV ,NMXESTLY ,NMXEAREA ,OEWIND   ,&
 & OETEMP   ,OEHEIG   ,OECLETH  ,OECLOTH  ,OEMICTH  ,&
 & RHERRY   ,RHERR1   ,RHERR2   ,RHERR3   ,RROUND  
USE YOMCOCTP , ONLY : NSYNOP   ,NSYNCT   ,NSRSCD   ,NATSCD   ,&
 & NSHSCD   ,NABSCD   ,NSHRED   ,NATSHS   ,NAIREP   ,&
 & NAIRCT   ,NAIRCD   ,NSATOB   ,NSABCT   ,NSTBCD   ,&
 & NDRIBU   ,NDRICT   ,NDRBCD   ,NTEMP    ,NTEMCT   ,&
 & NLDTCD   ,NPILOT   ,NPILCT   ,NLDPCD   ,NSATEM   ,&
 & NSAMCT   ,NSTMCD   ,NPAOB    ,NPAOCT   ,NPABCD   ,&
 & NMETAR  
USE YOMLVLY  , ONLY : NMXSTLV  ,NMXLAYER
USE YOMLUN   , ONLY : NULOUT
USE YOMCOSJO , ONLY : NVAR_U,  NVAR_U10,  NVAR_T,  NVAR_PS, &
 & RHUBEROETUNING, &
 & LHUBERBGQCG


IMPLICIT NONE

INTEGER(KIND=JPIM) :: IAREA, ICDT, IOBT, JAREA, JCODE, JS1, JS2, JS3, JS4
REAL(KIND=JPRB) :: ZRETUNE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cmoctmap.intfb.h"

!        1.       PRESET
!                 ------

!*          1.1   MAX. NO. OF OBS. ERROR AREAS AND LEVELS/LAYERS

IF (LHOOK) CALL DR_HOOK('SUOBSERR',0,ZHOOK_HANDLE)
NMXEAREA =       3
NMXESTLV =  NMXSTLV
NMXESTLY =  NMXLAYER

!*          1.1   PRESET ARRAYS

DO JS2 = 1 , NMXEAREA
  DO JS3 = 1 , NMXOCT
    DO JS4 = 1 , NMXOTP
      DO JS1 = 1 , NMXESTLV
        OEWIND (JS1,JS2,JS3,JS4)  =  RMDI
        OETEMP (JS1,JS2,JS3,JS4)  =  RMDI
        OEHEIG (JS1,JS2,JS3,JS4)  =  RMDI
      ENDDO
      DO JS1 = 1 , NMXESTLY
        OECLETH(JS1,JS2,JS3,JS4)  =  RMDI
        OECLOTH(JS1,JS2,JS3,JS4)  =  RMDI
        OEMICTH(JS1,JS2,JS3,JS4)  =  RMDI
      ENDDO
    ENDDO
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        2.       REL. HUM. ERRORS
!                 ----------------

!*          2.1   FIXED REL. HUM. ERRORS

RHERRY  = 13._JPRB/100._JPRB
RHERR1  = 17._JPRB/100._JPRB
RHERR2  = 23._JPRB/100._JPRB
RHERR3  = 28._JPRB/100._JPRB

!*          2.2   PWC ROUNDING ERROR

RROUND  =  0.0005_JPRB
!*
!-----------------------------------------------------------------------

!        3.       OBSERVATION RADIOSONDE WIND ERRORS
!                 ----------------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NTEMP ,NLDTCD,IOBT,ICDT)

!*          3.1   ALL AREAS

DO JAREA = 1,NMXEAREA

!*          3.1.1 ALL CODE TYPES

  DO JCODE = 1,NTEMCT
    OEWIND ( 1,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 2,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 3,JAREA,JCODE,IOBT)  =  1.90_JPRB
    OEWIND ( 4,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND ( 5,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OEWIND ( 6,JAREA,JCODE,IOBT)  =  2.60_JPRB
    OEWIND ( 7,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OEWIND ( 8,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OEWIND ( 9,JAREA,JCODE,IOBT)  =  2.40_JPRB
    OEWIND (10,JAREA,JCODE,IOBT)  =  2.20_JPRB
    OEWIND (11,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND (12,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND (13,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND (14,JAREA,JCODE,IOBT)  =  2.30_JPRB
    OEWIND (15,JAREA,JCODE,IOBT)  =  3.00_JPRB
    
    OEWIND ( :,JAREA,JCODE,IOBT)  = &
     & OEWIND(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_U,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        4.       OBS. RADIOSONDE HEIGHT ERRORS
!                 -----------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NTEMP ,NLDTCD,IOBT,ICDT)

!*          4.1   AREA 1 (NORTH AMERICA)

IAREA = 1

!*          4.1.1 ALL CODE TYPES

DO JCODE = 1,NTEMCT
  OEHEIG ( 1,IAREA,JCODE,IOBT)  =  4.30_JPRB*RG
  OEHEIG ( 2,IAREA,JCODE,IOBT)  =  4.40_JPRB*RG
  OEHEIG ( 3,IAREA,JCODE,IOBT)  =  5.20_JPRB*RG
  OEHEIG ( 4,IAREA,JCODE,IOBT)  =  8.40_JPRB*RG
  OEHEIG ( 5,IAREA,JCODE,IOBT)  =  9.80_JPRB*RG
  OEHEIG ( 6,IAREA,JCODE,IOBT)  = 10.70_JPRB*RG
  OEHEIG ( 7,IAREA,JCODE,IOBT)  = 11.80_JPRB*RG
  OEHEIG ( 8,IAREA,JCODE,IOBT)  = 13.20_JPRB*RG
  OEHEIG ( 9,IAREA,JCODE,IOBT)  = 15.20_JPRB*RG
  OEHEIG (10,IAREA,JCODE,IOBT)  = 18.10_JPRB*RG
  OEHEIG (11,IAREA,JCODE,IOBT)  = 19.50_JPRB*RG
  OEHEIG (12,IAREA,JCODE,IOBT)  = 22.50_JPRB*RG
  OEHEIG (13,IAREA,JCODE,IOBT)  = 25.00_JPRB*RG
  OEHEIG (14,IAREA,JCODE,IOBT)  = 32.00_JPRB*RG
  OEHEIG (15,IAREA,JCODE,IOBT)  = 40.00_JPRB*RG
  
  OEHEIG ( :,IAREA,JCODE,IOBT)  = &
   & OEHEIG(:,IAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO

!*          4.2   AREA 2 (AFRICA, ARABIA AND SOUTH AMERICA)

IAREA = 2

!*          4.2.1 ALL CODE TYPES

DO JCODE = 1,NTEMCT
  OEHEIG ( 1,IAREA,JCODE,IOBT)  =  6.45_JPRB*RG
  OEHEIG ( 2,IAREA,JCODE,IOBT)  =  6.60_JPRB*RG
  OEHEIG ( 3,IAREA,JCODE,IOBT)  =  7.80_JPRB*RG
  OEHEIG ( 4,IAREA,JCODE,IOBT)  = 12.60_JPRB*RG
  OEHEIG ( 5,IAREA,JCODE,IOBT)  = 14.70_JPRB*RG
  OEHEIG ( 6,IAREA,JCODE,IOBT)  = 16.05_JPRB*RG
  OEHEIG ( 7,IAREA,JCODE,IOBT)  = 17.70_JPRB*RG
  OEHEIG ( 8,IAREA,JCODE,IOBT)  = 19.80_JPRB*RG
  OEHEIG ( 9,IAREA,JCODE,IOBT)  = 22.80_JPRB*RG
  OEHEIG (10,IAREA,JCODE,IOBT)  = 27.15_JPRB*RG
  OEHEIG (11,IAREA,JCODE,IOBT)  = 29.25_JPRB*RG
  OEHEIG (12,IAREA,JCODE,IOBT)  = 33.75_JPRB*RG
  OEHEIG (13,IAREA,JCODE,IOBT)  = 37.50_JPRB*RG
  OEHEIG (14,IAREA,JCODE,IOBT)  = 48.00_JPRB*RG
  OEHEIG (15,IAREA,JCODE,IOBT)  = 60.00_JPRB*RG
  
  OEHEIG ( :,IAREA,JCODE,IOBT)  = &
   & OEHEIG(:,IAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO

!*          4.3   AREA 3 (THE REST)

IAREA = 3

!*          4.3.1 ALL CODE TYPES

DO JCODE = 1,NTEMCT
  OEHEIG ( 1,IAREA,JCODE,IOBT)  =  8.60_JPRB*RG
  OEHEIG ( 2,IAREA,JCODE,IOBT)  =  8.80_JPRB*RG
  OEHEIG ( 3,IAREA,JCODE,IOBT)  = 10.40_JPRB*RG
  OEHEIG ( 4,IAREA,JCODE,IOBT)  = 16.80_JPRB*RG
  OEHEIG ( 5,IAREA,JCODE,IOBT)  = 19.60_JPRB*RG
  OEHEIG ( 6,IAREA,JCODE,IOBT)  = 21.40_JPRB*RG
  OEHEIG ( 7,IAREA,JCODE,IOBT)  = 23.60_JPRB*RG
  OEHEIG ( 8,IAREA,JCODE,IOBT)  = 26.40_JPRB*RG
  OEHEIG ( 9,IAREA,JCODE,IOBT)  = 30.40_JPRB*RG
  OEHEIG (10,IAREA,JCODE,IOBT)  = 36.20_JPRB*RG
  OEHEIG (11,IAREA,JCODE,IOBT)  = 39.00_JPRB*RG
  OEHEIG (12,IAREA,JCODE,IOBT)  = 45.00_JPRB*RG
  OEHEIG (13,IAREA,JCODE,IOBT)  = 50.00_JPRB*RG
  OEHEIG (14,IAREA,JCODE,IOBT)  = 64.00_JPRB*RG
  OEHEIG (15,IAREA,JCODE,IOBT)  = 80.00_JPRB*RG

  OEHEIG ( :,IAREA,JCODE,IOBT)  = &
   & OEHEIG(:,IAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO
!*
!-----------------------------------------------------------------------

!        5.       OBS. RADIOSONDE TEMPERATURE ERRORS
!                 ----------------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NTEMP ,NLDTCD,IOBT,ICDT)

!*          5.1   ALL AREAS

DO JAREA = 1,NMXEAREA

!*          5.1.1 ALL CODE TYPES

  ZRETUNE=0.7_JPRB
  DO JCODE = 1,NTEMCT
    OETEMP ( 1,JAREA,JCODE,IOBT)  =  1.40_JPRB*ZRETUNE
    OETEMP ( 2,JAREA,JCODE,IOBT)  =  1.25_JPRB*ZRETUNE
    OETEMP ( 3,JAREA,JCODE,IOBT)  =  1.10_JPRB*ZRETUNE
    OETEMP ( 4,JAREA,JCODE,IOBT)  =  0.95_JPRB*ZRETUNE
    OETEMP ( 5,JAREA,JCODE,IOBT)  =  0.90_JPRB*ZRETUNE
    OETEMP ( 6,JAREA,JCODE,IOBT)  =  1.00_JPRB*ZRETUNE
    OETEMP ( 7,JAREA,JCODE,IOBT)  =  1.15_JPRB*ZRETUNE
    OETEMP ( 8,JAREA,JCODE,IOBT)  =  1.20_JPRB*ZRETUNE
    OETEMP ( 9,JAREA,JCODE,IOBT)  =  1.25_JPRB*ZRETUNE
    IF (LHUBERBGQCG) THEN
      OETEMP (10,JAREA,JCODE,IOBT)  =  1.70_JPRB*ZRETUNE
      OETEMP (11,JAREA,JCODE,IOBT)  =  2.00_JPRB*ZRETUNE
      OETEMP (12,JAREA,JCODE,IOBT)  =  2.00_JPRB*ZRETUNE
      OETEMP (13,JAREA,JCODE,IOBT)  =  2.00_JPRB*ZRETUNE
      OETEMP (14,JAREA,JCODE,IOBT)  =  2.10_JPRB*ZRETUNE
      OETEMP (15,JAREA,JCODE,IOBT)  =  2.60_JPRB*ZRETUNE
    ELSE    
      OETEMP (10,JAREA,JCODE,IOBT)  =  1.30_JPRB*ZRETUNE
      OETEMP (11,JAREA,JCODE,IOBT)  =  1.40_JPRB*ZRETUNE
      OETEMP (12,JAREA,JCODE,IOBT)  =  1.40_JPRB*ZRETUNE
      OETEMP (13,JAREA,JCODE,IOBT)  =  1.40_JPRB*ZRETUNE
      OETEMP (14,JAREA,JCODE,IOBT)  =  1.50_JPRB*ZRETUNE
      OETEMP (15,JAREA,JCODE,IOBT)  =  2.10_JPRB*ZRETUNE
    ENDIF

    OETEMP ( :,JAREA,JCODE,IOBT)  = & 
     & OETEMP(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        6.       OBSERVATION SATOB WIND ERRORS
!                 -----------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NSATOB,NSTBCD,IOBT,ICDT)

!*          6.1   ALL AREAS

DO JAREA = 1,NMXEAREA


!*          6.1.1 ALL CODE TYPES

  DO JCODE = 1,NSABCT
    OEWIND ( 1,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND ( 2,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND ( 3,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND ( 4,JAREA,JCODE,IOBT)  =  3.50_JPRB
    OEWIND ( 5,JAREA,JCODE,IOBT)  =  4.30_JPRB
    OEWIND ( 6,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND ( 7,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND ( 8,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND ( 9,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND (10,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND (11,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND (12,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND (13,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND (14,JAREA,JCODE,IOBT)  =  5.00_JPRB
    OEWIND (15,JAREA,JCODE,IOBT)  =  5.70_JPRB
    
    OEWIND ( :,JAREA,JCODE,IOBT)  = & 
     & OEWIND(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_U,ICDT,IOBT)
  ENDDO

ENDDO
!*
!-----------------------------------------------------------------------

!        7.       OBSERVATION AIREP WIND ERRORS
!                 -----------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NAIREP,NAIRCD,IOBT,ICDT)

!*          7.1   ALL AREAS

DO JAREA = 1,NMXEAREA

!*          7.1.1 ALL CODE TYPES

  DO JCODE = 1,NAIRCT
    OEWIND ( 1,JAREA,JCODE,IOBT)  =  2.46_JPRB
    OEWIND ( 2,JAREA,JCODE,IOBT)  =  2.51_JPRB
    OEWIND ( 3,JAREA,JCODE,IOBT)  =  2.56_JPRB
    OEWIND ( 4,JAREA,JCODE,IOBT)  =  2.71_JPRB
    OEWIND ( 5,JAREA,JCODE,IOBT)  =  2.81_JPRB
    OEWIND ( 6,JAREA,JCODE,IOBT)  =  2.86_JPRB
    OEWIND ( 7,JAREA,JCODE,IOBT)  =  2.91_JPRB
    OEWIND ( 8,JAREA,JCODE,IOBT)  =  2.96_JPRB
    OEWIND ( 9,JAREA,JCODE,IOBT)  =  2.91_JPRB
    OEWIND (10,JAREA,JCODE,IOBT)  =  2.76_JPRB
    OEWIND (11,JAREA,JCODE,IOBT)  =  2.66_JPRB
    OEWIND (12,JAREA,JCODE,IOBT)  =  2.66_JPRB
    OEWIND (13,JAREA,JCODE,IOBT)  =  2.86_JPRB
    OEWIND (14,JAREA,JCODE,IOBT)  =  3.06_JPRB
    OEWIND (15,JAREA,JCODE,IOBT)  =  3.36_JPRB
 
    OEWIND ( :,JAREA,JCODE,IOBT)  = & 
     & OEWIND(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_U,ICDT,IOBT)
  ENDDO

!*          7.1.2 AIREP-MANUAL (NAIRCD=141)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NAIREP,NAIRCD,IOBT,ICDT)
  OEWIND ( 1,JAREA,ICDT,IOBT)  =  2.86_JPRB
  OEWIND ( 2,JAREA,ICDT,IOBT)  =  2.91_JPRB
  OEWIND ( 3,JAREA,ICDT,IOBT)  =  2.96_JPRB
  OEWIND ( 4,JAREA,ICDT,IOBT)  =  3.11_JPRB
  OEWIND ( 5,JAREA,ICDT,IOBT)  =  3.21_JPRB
  OEWIND ( 6,JAREA,ICDT,IOBT)  =  3.26_JPRB
  OEWIND ( 7,JAREA,ICDT,IOBT)  =  3.31_JPRB
  OEWIND ( 8,JAREA,ICDT,IOBT)  =  3.36_JPRB
  OEWIND ( 9,JAREA,ICDT,IOBT)  =  3.31_JPRB
  OEWIND (10,JAREA,ICDT,IOBT)  =  3.16_JPRB
  OEWIND (11,JAREA,ICDT,IOBT)  =  3.06_JPRB
  OEWIND (12,JAREA,ICDT,IOBT)  =  3.06_JPRB
  OEWIND (13,JAREA,ICDT,IOBT)  =  3.26_JPRB
  OEWIND (14,JAREA,ICDT,IOBT)  =  3.46_JPRB
  OEWIND (15,JAREA,ICDT,IOBT)  =  3.76_JPRB

  OEWIND ( :,JAREA,ICDT,IOBT)  = & 
   & OEWIND(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_U,ICDT,IOBT)
ENDDO
!*
!-----------------------------------------------------------------------

!        8.       OBSERVATION AIREP TEMPERATURE ERRORS
!                 ------------------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NAIREP,NAIRCD,IOBT,ICDT)

!*          8.1   ALL AREAS

DO JAREA = 1,NMXEAREA

!*          8.1.1 ALL CODE TYPES

  DO JCODE = 1,NAIRCT
    OETEMP ( 1,JAREA,JCODE,IOBT)  =  1.40_JPRB
    OETEMP ( 2,JAREA,JCODE,IOBT)  =  1.18_JPRB
    OETEMP ( 3,JAREA,JCODE,IOBT)  =  1.00_JPRB
    OETEMP ( 4,JAREA,JCODE,IOBT)  =  0.98_JPRB
    OETEMP ( 5,JAREA,JCODE,IOBT)  =  0.96_JPRB
    OETEMP ( 6,JAREA,JCODE,IOBT)  =  0.95_JPRB
    OETEMP ( 7,JAREA,JCODE,IOBT)  =  0.95_JPRB
    OETEMP ( 8,JAREA,JCODE,IOBT)  =  1.06_JPRB
    OETEMP ( 9,JAREA,JCODE,IOBT)  =  1.18_JPRB
    OETEMP (10,JAREA,JCODE,IOBT)  =  1.30_JPRB
    OETEMP (11,JAREA,JCODE,IOBT)  =  1.40_JPRB
    OETEMP (12,JAREA,JCODE,IOBT)  =  1.50_JPRB
    OETEMP (13,JAREA,JCODE,IOBT)  =  1.60_JPRB
    OETEMP (14,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OETEMP (15,JAREA,JCODE,IOBT)  =  2.10_JPRB
 
    OETEMP ( :,JAREA,JCODE,IOBT)  = & 
     & OETEMP(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)
  ENDDO

!*          8.1.2 AIREP-MANUAL (NAIRCD=141)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NAIREP,NAIRCD,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  1.65_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.43_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.25_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.23_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.21_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.20_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.20_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.31_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.43_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  1.55_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  1.65_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  1.75_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  1.85_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  2.05_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  2.35_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)
ENDDO
!*
!-----------------------------------------------------------------------

!        9.       OBSERVATION SYNOP WIND ERRORS
!                 -----------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSRSCD,IOBT,ICDT)

!*          9.1   ALL AREAS

DO JAREA = 1,NMXEAREA

!*          9.1.1 ALL CODE TYPES

  DO JCODE = 1,NSYNCT
    OEWIND ( 1,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND ( 2,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND ( 3,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND ( 4,JAREA,JCODE,IOBT)  =  3.10_JPRB
    OEWIND ( 5,JAREA,JCODE,IOBT)  =  3.30_JPRB
    OEWIND ( 6,JAREA,JCODE,IOBT)  =  3.50_JPRB
    OEWIND ( 7,JAREA,JCODE,IOBT)  =  2.90_JPRB
    OEWIND ( 8,JAREA,JCODE,IOBT)  =  2.90_JPRB
    OEWIND ( 9,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND (10,JAREA,JCODE,IOBT)  =  1.90_JPRB
    OEWIND (11,JAREA,JCODE,IOBT)  =  1.70_JPRB
    OEWIND (12,JAREA,JCODE,IOBT)  =  1.70_JPRB
    OEWIND (13,JAREA,JCODE,IOBT)  =  1.70_JPRB
    OEWIND (14,JAREA,JCODE,IOBT)  =  2.20_JPRB
    OEWIND (15,JAREA,JCODE,IOBT)  =  2.70_JPRB

    OEWIND ( :,JAREA,JCODE,IOBT)  = & 
     & OEWIND(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_U10,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        10.      OBS. SYNOP HEIGHT ERRORS
!                 ------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSRSCD,IOBT,ICDT)

!*          10.1  ALL AREAS

DO JAREA = 1,NMXEAREA

!*          10.1.1 LAND STATION (CODE TYPE 11; manual)

  ZRETUNE   = 1.0_JPRB
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSRSCD,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  =  5.60_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  =  7.20_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  =  8.60_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 12.10_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 14.90_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 18.80_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 25.40_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 27.70_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 32.40_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 39.40_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 50.30_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 59.30_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 69.80_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 96.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  =114.20_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = & 
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)

!*          10.1.2 LAND STATION (CODE TYPE 14; automatic)

  ZRETUNE   = 0.75_JPRB
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NATSCD,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  =  5.60_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  =  7.20_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  =  8.60_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 12.10_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 14.90_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 18.80_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 25.40_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 27.70_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 32.40_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 39.40_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 50.30_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 59.30_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 69.80_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 96.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  =114.20_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = & 
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)

!*          10.1.3 SHIP STATION (CODE TYPE 21; manual)

  IF (LHUBERBGQCG) THEN
    ZRETUNE   = 0.84_JPRB
  ELSE
    ZRETUNE   = 0.71_JPRB
  ENDIF

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSHSCD,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = & 
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)

!*          10.1.4 SHIP STATION (CODE TYPE 22; manual)

  IF (LHUBERBGQCG) THEN
    ZRETUNE   = 0.84_JPRB
  ELSE
    ZRETUNE   = 0.71_JPRB
  ENDIF

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NABSCD,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = & 
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)

!*          10.1.5 SHIP STATION (CODE TYPE 23; manual)

  IF (LHUBERBGQCG) THEN
    ZRETUNE   = 0.84_JPRB
  ELSE
    ZRETUNE   = 0.71_JPRB
  ENDIF

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSHRED,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = & 
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)

!*          10.1.6 SHIP STATION (CODE TYPE 24; automatic)

  ZRETUNE   = 0.42_JPRB
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NATSHS,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  = 10.00_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = & 
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)

!*          10.1.7 METAR STATION (CODE TYPE 140)
  ZRETUNE   = 1.00_JPRB
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NMETAR,IOBT,ICDT)
  OEHEIG ( 1,JAREA,ICDT,IOBT)  =  5.60_JPRB*RG*ZRETUNE
  OEHEIG ( 2,JAREA,ICDT,IOBT)  =  7.20_JPRB*RG*ZRETUNE
  OEHEIG ( 3,JAREA,ICDT,IOBT)  =  8.60_JPRB*RG*ZRETUNE
  OEHEIG ( 4,JAREA,ICDT,IOBT)  = 12.10_JPRB*RG*ZRETUNE
  OEHEIG ( 5,JAREA,ICDT,IOBT)  = 14.90_JPRB*RG*ZRETUNE
  OEHEIG ( 6,JAREA,ICDT,IOBT)  = 18.80_JPRB*RG*ZRETUNE
  OEHEIG ( 7,JAREA,ICDT,IOBT)  = 25.40_JPRB*RG*ZRETUNE
  OEHEIG ( 8,JAREA,ICDT,IOBT)  = 27.70_JPRB*RG*ZRETUNE
  OEHEIG ( 9,JAREA,ICDT,IOBT)  = 32.40_JPRB*RG*ZRETUNE
  OEHEIG (10,JAREA,ICDT,IOBT)  = 39.40_JPRB*RG*ZRETUNE
  OEHEIG (11,JAREA,ICDT,IOBT)  = 50.30_JPRB*RG*ZRETUNE
  OEHEIG (12,JAREA,ICDT,IOBT)  = 59.30_JPRB*RG*ZRETUNE
  OEHEIG (13,JAREA,ICDT,IOBT)  = 69.80_JPRB*RG*ZRETUNE
  OEHEIG (14,JAREA,ICDT,IOBT)  = 96.00_JPRB*RG*ZRETUNE
  OEHEIG (15,JAREA,ICDT,IOBT)  =114.20_JPRB*RG*ZRETUNE

  OEHEIG ( :,JAREA,ICDT,IOBT)  = &
   & OEHEIG(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO

!*
!-----------------------------------------------------------------------

!        11.      OBS. SYNOP TEMPERATURE ERRORS
!                 -----------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSRSCD,IOBT,ICDT)

!*          11.1  ALL AREAS

DO JAREA = 1,NMXEAREA

!*          11.1.1 LAND STATION (CODE TYPE 11)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSRSCD,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  2.00_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.50_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.30_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.20_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.30_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.50_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.90_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  2.00_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  2.20_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  2.40_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  2.50_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  2.50_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  2.50_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)

!*          11.1.2 LAND STATION (CODE TYPE 14)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NATSCD,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  2.00_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.50_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.30_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.20_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.30_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.50_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.90_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  2.00_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  2.20_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  2.40_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  2.50_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  2.50_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  2.50_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)

!*          11.1.3 SHIP STATION (CODE TYPE 21)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSHSCD,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  1.80_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)

!*          11.1.4 SHIP STATION (CODE TYPE 22)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NABSCD,IOBT,ICDT)
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  1.80_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)

!*          11.1.5 SHIP STATION (CODE TYPE 23)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NSHRED,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  1.80_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)

!*          11.1.6 SHIP STATION (CODE TYPE 24)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NATSHS,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  1.80_JPRB
  
  OETEMP ( :,JAREA,ICDT,IOBT)  = & 
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)

!*          11.1.7 METAR STATION (CODE TYPE 140)

  CALL CMOCTMAP('CODETOSQ',NULOUT,NSYNOP,NMETAR,IOBT,ICDT)
  OETEMP ( 1,JAREA,ICDT,IOBT)  =  2.00_JPRB
  OETEMP ( 2,JAREA,ICDT,IOBT)  =  1.50_JPRB
  OETEMP ( 3,JAREA,ICDT,IOBT)  =  1.30_JPRB
  OETEMP ( 4,JAREA,ICDT,IOBT)  =  1.20_JPRB
  OETEMP ( 5,JAREA,ICDT,IOBT)  =  1.30_JPRB
  OETEMP ( 6,JAREA,ICDT,IOBT)  =  1.50_JPRB
  OETEMP ( 7,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 8,JAREA,ICDT,IOBT)  =  1.80_JPRB
  OETEMP ( 9,JAREA,ICDT,IOBT)  =  1.90_JPRB
  OETEMP (10,JAREA,ICDT,IOBT)  =  2.00_JPRB
  OETEMP (11,JAREA,ICDT,IOBT)  =  2.20_JPRB
  OETEMP (12,JAREA,ICDT,IOBT)  =  2.40_JPRB
  OETEMP (13,JAREA,ICDT,IOBT)  =  2.50_JPRB
  OETEMP (14,JAREA,ICDT,IOBT)  =  2.50_JPRB
  OETEMP (15,JAREA,ICDT,IOBT)  =  2.50_JPRB

  OETEMP ( :,JAREA,ICDT,IOBT)  = &
   & OETEMP(:,JAREA,ICDT,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)
ENDDO
!*
!-----------------------------------------------------------------------

!        12.      OBS. DRIBU WIND ERRORS
!                 ----------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NDRIBU,NDRBCD,IOBT,ICDT)

!*          12.1  ALL AREAS

DO JAREA = 1,NMXEAREA

!*          12.1.1 ALL CODE TYPES

  DO JCODE = 1,NDRICT
    OEWIND ( 1,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 2,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 3,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 4,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 5,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 6,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 7,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 8,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 9,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND (10,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND (11,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND (12,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND (13,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND (14,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND (15,JAREA,JCODE,IOBT)  =  1.80_JPRB
   
    OEWIND ( :,JAREA,JCODE,IOBT)  = & 
     & OEWIND(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_U10,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        13.      OBS. DRIBU HEIGHT ERRORS
!                 ------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NDRIBU,NDRBCD,IOBT,ICDT)

!*          13.1  ALL AREAS
DO JAREA = 1,NMXEAREA

!*          13.1.1 ALL CODE TYPES

  ZRETUNE   = 0.71_JPRB
  DO JCODE = 1,NDRICT
    OEHEIG ( 1,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 2,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 3,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 4,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 5,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 6,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 7,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 8,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG ( 9,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG (10,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG (11,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG (12,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG (13,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG (14,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
    OEHEIG (15,JAREA,JCODE,IOBT)  =  7.00_JPRB*RG*ZRETUNE
   
    OEHEIG ( :,JAREA,JCODE,IOBT)  = & 
     & OEHEIG(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        14.      OBS. DRIBU TEMPERATURE ERRORS

CALL CMOCTMAP('CODETOSQ',NULOUT,NDRIBU,NDRBCD,IOBT,ICDT)

!*          14.1  ALL AREAS

DO JAREA = 1,NMXEAREA

!*          14.1.1 ALL CODE TYPES

  DO JCODE = 1,NDRICT
    OETEMP ( 1,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OETEMP ( 2,JAREA,JCODE,IOBT)  =  1.50_JPRB
    OETEMP ( 3,JAREA,JCODE,IOBT)  =  1.30_JPRB
    OETEMP ( 4,JAREA,JCODE,IOBT)  =  1.20_JPRB
    OETEMP ( 5,JAREA,JCODE,IOBT)  =  1.30_JPRB
    OETEMP ( 6,JAREA,JCODE,IOBT)  =  1.50_JPRB
    OETEMP ( 7,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OETEMP ( 8,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OETEMP ( 9,JAREA,JCODE,IOBT)  =  1.90_JPRB
    OETEMP (10,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OETEMP (11,JAREA,JCODE,IOBT)  =  2.20_JPRB
    OETEMP (12,JAREA,JCODE,IOBT)  =  2.40_JPRB
    OETEMP (13,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OETEMP (14,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OETEMP (15,JAREA,JCODE,IOBT)  =  2.50_JPRB
   
    OETEMP ( :,JAREA,JCODE,IOBT)  = & 
     & OETEMP(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_T,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        15.      OBS. PAOB HEIGHT ERRORS
!                 -----------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NPAOB ,NPABCD,IOBT,ICDT)

!*          15.1  ALL AREAS

DO JAREA = 1,NMXEAREA

!*          15.1.1 ALL CODE TYPES

  DO JCODE = 1,NPAOCT
    OEHEIG ( 1,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 2,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 3,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 4,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 5,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 6,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 7,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 8,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG ( 9,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG (10,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG (11,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG (12,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG (13,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG (14,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
    OEHEIG (15,JAREA,JCODE,IOBT)  = 24.00_JPRB*RG
   
    OEHEIG ( :,JAREA,JCODE,IOBT)  = & 
     & OEHEIG(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        16.      OBSERVATION PILOT WIND ERRORS
!                 -----------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NPILOT,NLDPCD,IOBT,ICDT)

!*          16.1  ALL AREAS

DO JAREA = 1,NMXEAREA

!*          16.1.1 ALL CODE TYPES

  DO JCODE = 1,NPILCT
    OEWIND ( 1,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 2,JAREA,JCODE,IOBT)  =  1.80_JPRB
    OEWIND ( 3,JAREA,JCODE,IOBT)  =  1.90_JPRB
    OEWIND ( 4,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND ( 5,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OEWIND ( 6,JAREA,JCODE,IOBT)  =  2.60_JPRB
    OEWIND ( 7,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OEWIND ( 8,JAREA,JCODE,IOBT)  =  2.50_JPRB
    OEWIND ( 9,JAREA,JCODE,IOBT)  =  2.40_JPRB
    OEWIND (10,JAREA,JCODE,IOBT)  =  2.20_JPRB
    OEWIND (11,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND (12,JAREA,JCODE,IOBT)  =  2.00_JPRB
    OEWIND (13,JAREA,JCODE,IOBT)  =  2.10_JPRB
    OEWIND (14,JAREA,JCODE,IOBT)  =  2.30_JPRB
    OEWIND (15,JAREA,JCODE,IOBT)  =  3.00_JPRB
   
    OEWIND ( :,JAREA,JCODE,IOBT)  = & 
     & OEWIND(:,JAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_U,ICDT,IOBT)
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        17.      PILOT HEIGHT ERRORS
!                 -------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NPILOT,NLDPCD,IOBT,ICDT)

!*          17.1  AREA 1 (NORTH AMERICA)

IAREA = 1

!*          17.1.1 ALL CODE TYPES

DO JCODE = 1,NPILCT
  OEHEIG ( 1,IAREA,JCODE,IOBT)  =  4.30_JPRB*RG
  OEHEIG ( 2,IAREA,JCODE,IOBT)  =  4.40_JPRB*RG
  OEHEIG ( 3,IAREA,JCODE,IOBT)  =  5.20_JPRB*RG
  OEHEIG ( 4,IAREA,JCODE,IOBT)  =  8.40_JPRB*RG
  OEHEIG ( 5,IAREA,JCODE,IOBT)  =  9.80_JPRB*RG
  OEHEIG ( 6,IAREA,JCODE,IOBT)  = 10.70_JPRB*RG
  OEHEIG ( 7,IAREA,JCODE,IOBT)  = 11.80_JPRB*RG
  OEHEIG ( 8,IAREA,JCODE,IOBT)  = 13.20_JPRB*RG
  OEHEIG ( 9,IAREA,JCODE,IOBT)  = 15.20_JPRB*RG
  OEHEIG (10,IAREA,JCODE,IOBT)  = 18.10_JPRB*RG
  OEHEIG (11,IAREA,JCODE,IOBT)  = 19.50_JPRB*RG
  OEHEIG (12,IAREA,JCODE,IOBT)  = 22.50_JPRB*RG
  OEHEIG (13,IAREA,JCODE,IOBT)  = 25.00_JPRB*RG
  OEHEIG (14,IAREA,JCODE,IOBT)  = 32.00_JPRB*RG
  OEHEIG (15,IAREA,JCODE,IOBT)  = 40.00_JPRB*RG
   
  OEHEIG ( :,IAREA,JCODE,IOBT)  = & 
   & OEHEIG(:,IAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO

!*          17.2  AREA 2 (AFRICA, ARABIA AND SOUTH AMERICA)

IAREA = 2

!*          17.2.1 ALL CODE TYPES

DO JCODE = 1,NPILCT
  OEHEIG ( 1,IAREA,JCODE,IOBT)  =  8.60_JPRB*RG
  OEHEIG ( 2,IAREA,JCODE,IOBT)  =  8.80_JPRB*RG
  OEHEIG ( 3,IAREA,JCODE,IOBT)  = 10.40_JPRB*RG
  OEHEIG ( 4,IAREA,JCODE,IOBT)  = 16.80_JPRB*RG
  OEHEIG ( 5,IAREA,JCODE,IOBT)  = 19.60_JPRB*RG
  OEHEIG ( 6,IAREA,JCODE,IOBT)  = 21.40_JPRB*RG
  OEHEIG ( 7,IAREA,JCODE,IOBT)  = 23.60_JPRB*RG
  OEHEIG ( 8,IAREA,JCODE,IOBT)  = 26.40_JPRB*RG
  OEHEIG ( 9,IAREA,JCODE,IOBT)  = 30.40_JPRB*RG
  OEHEIG (10,IAREA,JCODE,IOBT)  = 36.20_JPRB*RG
  OEHEIG (11,IAREA,JCODE,IOBT)  = 39.00_JPRB*RG
  OEHEIG (12,IAREA,JCODE,IOBT)  = 45.00_JPRB*RG
  OEHEIG (13,IAREA,JCODE,IOBT)  = 50.00_JPRB*RG
  OEHEIG (14,IAREA,JCODE,IOBT)  = 64.00_JPRB*RG
  OEHEIG (15,IAREA,JCODE,IOBT)  = 80.00_JPRB*RG
   
  OEHEIG ( :,IAREA,JCODE,IOBT)  = & 
   & OEHEIG(:,IAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO

!*          17.3  AREA 3 (THE REST)

IAREA = 3

!*          17.3.1 ALL CODE TYPES

DO JCODE = 1,NPILCT
  OEHEIG ( 1,IAREA,JCODE,IOBT)  =  6.45_JPRB*RG
  OEHEIG ( 2,IAREA,JCODE,IOBT)  =  6.60_JPRB*RG
  OEHEIG ( 3,IAREA,JCODE,IOBT)  =  7.80_JPRB*RG
  OEHEIG ( 4,IAREA,JCODE,IOBT)  = 12.60_JPRB*RG
  OEHEIG ( 5,IAREA,JCODE,IOBT)  = 14.70_JPRB*RG
  OEHEIG ( 6,IAREA,JCODE,IOBT)  = 16.05_JPRB*RG
  OEHEIG ( 7,IAREA,JCODE,IOBT)  = 17.70_JPRB*RG
  OEHEIG ( 8,IAREA,JCODE,IOBT)  = 19.80_JPRB*RG
  OEHEIG ( 9,IAREA,JCODE,IOBT)  = 22.80_JPRB*RG
  OEHEIG (10,IAREA,JCODE,IOBT)  = 27.15_JPRB*RG
  OEHEIG (11,IAREA,JCODE,IOBT)  = 29.25_JPRB*RG
  OEHEIG (12,IAREA,JCODE,IOBT)  = 33.75_JPRB*RG
  OEHEIG (13,IAREA,JCODE,IOBT)  = 37.50_JPRB*RG
  OEHEIG (14,IAREA,JCODE,IOBT)  = 48.00_JPRB*RG
  OEHEIG (15,IAREA,JCODE,IOBT)  = 60.00_JPRB*RG
   
  OEHEIG ( :,IAREA,JCODE,IOBT)  = & 
   & OEHEIG(:,IAREA,JCODE,IOBT)*RHUBEROETUNING(:,NVAR_PS,ICDT,IOBT)
ENDDO
!*
!-----------------------------------------------------------------------

!        18.      SATEM/TOVS THICKNESS ERRORS
!                 ---------------------------

CALL CMOCTMAP('CODETOSQ',NULOUT,NSATEM,NSTMCD,IOBT,ICDT)

!*          18.1   ALL AREAS

DO JAREA = 1,NMXEAREA

!*          18.2   ALL CODE TYPES

  DO JCODE = 1,NSAMCT

!*          18.3   CLEAR ERRORS

    OECLETH( 1,JAREA,JCODE,IOBT)  = 27.0_JPRB*RG
    OECLETH( 2,JAREA,JCODE,IOBT)  = 17.0_JPRB*RG
    OECLETH( 3,JAREA,JCODE,IOBT)  = 25.0_JPRB*RG
    OECLETH( 4,JAREA,JCODE,IOBT)  = 53.0_JPRB*RG
    OECLETH( 5,JAREA,JCODE,IOBT)  = 33.0_JPRB*RG
    OECLETH( 6,JAREA,JCODE,IOBT)  = 36.0_JPRB*RG
    OECLETH( 7,JAREA,JCODE,IOBT)  = 74.0_JPRB*RG

!*          18.4   CLOUDY ERRORS

    OECLOTH( 1,JAREA,JCODE,IOBT)  = 33.0_JPRB*RG
    OECLOTH( 2,JAREA,JCODE,IOBT)  = 20.0_JPRB*RG
    OECLOTH( 3,JAREA,JCODE,IOBT)  = 31.0_JPRB*RG
    OECLOTH( 4,JAREA,JCODE,IOBT)  = 56.0_JPRB*RG
    OECLOTH( 5,JAREA,JCODE,IOBT)  = 33.0_JPRB*RG
    OECLOTH( 6,JAREA,JCODE,IOBT)  = 36.0_JPRB*RG
    OECLOTH( 7,JAREA,JCODE,IOBT)  = 74.0_JPRB*RG

!*          18.5   MICROWAVE ERRORS

    OEMICTH( 1,JAREA,JCODE,IOBT)  = 33.0_JPRB*RG
    OEMICTH( 2,JAREA,JCODE,IOBT)  = 20.0_JPRB*RG
    OEMICTH( 3,JAREA,JCODE,IOBT)  = 31.0_JPRB*RG
    OEMICTH( 4,JAREA,JCODE,IOBT)  = 56.0_JPRB*RG
    OEMICTH( 5,JAREA,JCODE,IOBT)  = 33.0_JPRB*RG
    OEMICTH( 6,JAREA,JCODE,IOBT)  = 36.0_JPRB*RG
    OEMICTH( 7,JAREA,JCODE,IOBT)  = 74.0_JPRB*RG
  ENDDO
ENDDO
!*
!-----------------------------------------------------------------------

!        19.      RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SUOBSERR',1,ZHOOK_HANDLE)
END SUBROUTINE SUOBSERR
