SUBROUTINE SUPERERR

!**** SUBROUTINE SUPERERR - SET UP OBS. PERSISTENCE ERRORS

!        D. VASILJEVIC    ECMWF    17/10/1994

!     PURPOSE
!     -------

!        SET OBSERVATION PERSISTENCE ERROR

!**   INTERFACE
!     ---------

!         SUPERERR IS CALLED BY SUBROUTINE MAKECMA

!     METHOD
!     ------

!        SET ALL OBS. PERSISTENCE ERROR RELATED CONSTANTS/PARAMETERS.

!**   EXTERNALS
!     ---------

!        DEPERERR - DEFINE PERSSITENCE ERROR TERM 1 (COMMON ROUTINE
!                   WITH IFS)

!**   HISTORY
!     -------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMKODB , ONLY : NANCDY
USE YOMCST   , ONLY : RPI      ,RG
USE YOMANCS  , ONLY : RDAYPYEA ,RHOUPDAY
USE YOMERR   , ONLY : LPERERCO
USE YOMPERR  , ONLY : UPM10070 ,UPM69925 ,UPM24900 ,VPM10070 ,&
 & VPM69925 ,VPM24900 ,GPM10070 ,GPM69925 ,GPM24900 ,&
 & TPM10070 ,TPM69925 ,TPM24900 ,UP110070 ,UP169925 ,&
 & UP124900 ,VP110070 ,VP169925 ,VP124900 ,GP110070 ,&
 & GP169925 ,GP124900 ,TP110070 ,TP169925 ,TP124900 ,&
 & RPERER1  

IMPLICIT NONE

REAL(KIND=JPRB) :: ZTERM1, ZTERM2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "depererr.intfb.h"

!*

!        0.       DEFINE IT HRERE OR CALL IFS COMMON ROUTINE
!                 ------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPERERR',0,ZHOOK_HANDLE)
IF(LPERERCO)       THEN
  CALL DEPERERR(RHOUPDAY,RPI,NANCDY,RDAYPYEA,RG)
  GO TO 4
ENDIF
!*
!-----------------------------------------------------------------------

!        1.       MAX. PERSISTENCE ERROR (OVER 24HR)
!                 ----------------------------------
!                       1000-700, 699-250, 249-  0   (MB)
!                  U      6.4       12.7     19.1    (M/S)
!                  V      6.4       12.7     19.1    (M/S)
!                  Z     48.        60.      72.      (M)
!                  T      6.         7.       8.     (DEGREES) 

!*          1.1   U COMPONENT

UPM10070=  6.4_JPRB
UPM69925= 12.7_JPRB
UPM24900= 19.1_JPRB

!*          1.2   V COMPONENT

VPM10070=  6.4_JPRB
VPM69925= 12.7_JPRB
VPM24900= 19.1_JPRB

!*          1.3   Z

GPM10070= 48._JPRB*RG
GPM69925= 60._JPRB*RG
GPM24900= 72._JPRB*RG

!*          1.4   T

TPM10070=  6._JPRB
TPM69925=  7._JPRB
TPM24900=  8._JPRB
!*
!-----------------------------------------------------------------------

!        2.       MAX. PERSIST. ERROR (OVER 1HR)
!                 ------------------------------

!*          2.1   U COMPONENT

UP110070=  UPM10070 / RHOUPDAY
UP169925=  UPM69925 / RHOUPDAY
UP124900=  UPM24900 / RHOUPDAY

!*          2.2   V COMPONENT

VP110070=  VPM10070 / RHOUPDAY
VP169925=  VPM69925 / RHOUPDAY
VP124900=  VPM24900 / RHOUPDAY

!*          2.3   Z

GP110070=  GPM10070 / RHOUPDAY
GP169925=  GPM69925 / RHOUPDAY
GP124900=  GPM24900 / RHOUPDAY

!*          2.4   T

TP110070=  TPM10070 / RHOUPDAY
TP169925=  TPM69925 / RHOUPDAY
TP124900=  TPM24900 / RHOUPDAY
!*
!-----------------------------------------------------------------------

!        3.       PERSISTENCE ERROR (TERM 1)
!                 --------------------------

ZTERM1  = 2.0_JPRB*RPI*MOD(REAL(NANCDY,JPRB),RDAYPYEA) / RDAYPYEA
ZTERM2  = ZTERM1 + RPI/2.0_JPRB
RPERER1 = SIN(ZTERM2)
!*
!-----------------------------------------------------------------------

!        4.       RETURN
!                 ------

4 CONTINUE
IF (LHOOK) CALL DR_HOOK('SUPERERR',1,ZHOOK_HANDLE)
END SUBROUTINE SUPERERR
