SUBROUTINE AWPRFIN(KPILP,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE AWPRFIN - PROCESS INPUT AMERICAN WIND PROFILERs

!        D. VASILJEVIC     ECMWF     13/12/00 (original 04/07/94)

!     PURPOSE
!     -------

!        PROCESS INPUT AMERICAN WIND PROFILERs

!**   INTERFACE
!     ---------

!        CALL AWPRFIN(KPILP,
!                     KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                     LDMESAGE,LDTAKEN)
!     WHERE:
!            KPILP   = REPORT POINTER
!            KOBTYP  = OBS. TYPE
!            KCDTYP  = CODE TYPE
!            KOBSQNO = OBS. TYPE SQ. NO.
!            KCDSQNO = CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT UPDATED) SWITCH  

!        AWPRFIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        ERRSTAT  - OBS. ERROR
!        PILOTBE  - UPDATE AMERICAN WIND PROFILERs BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE
!        OBSCHAR  - OBS. CHARACTERISTICS

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMCMBDY , ONLY :    NPSWLBP  
USE YOMANCS  , ONLY : NMDI     ,RMDI     ,RDEGREES ,RADIANS  ,EPSMAC   ,RUPMLIM  ,RDWMLIM  
USE YOMVNMB  , ONLY : NVNUMB
USE YOMMKODB , ONLY :  NMKCMVSE ,LTOTREM
USE YOMNMCOD , ONLY :   NHEIGVC
USE YOMCST   , ONLY : RG       ,RTT
USE YOETHF   , ONLY : RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES  
USE YOMLUN   , ONLY : NULOUT

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPILP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT)   :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE, &
 & IOBTIME, &
 & IVNM, JWIND, &
 & ISTART, IEND, ISIGW, IPAUFLG, ISTID, INSTYP, IREPST, &
 & IRDBMXR, IFRLAT, IFRLON, IFRDAT, IFRTIM, IFRALT  

INTEGER(KIND=JPIM) :: IU, &
 & IUUSTAT, IUUEVEN1, IUUEVEN2, IUURDBFL, IUUANFLG, &
 & IUUPFLG, IUUFLG, IUUVERCO, IUULEVID  
INTEGER(KIND=JPIM) :: IV, &
 & IVVSTAT, IVVEVEN1, IVVEVEN2, IVVRDBFL, IVVANFLG, &
 & IVVPFLG, IVVFLG, IVVVERCO, IVVLEVID  
INTEGER(KIND=JPIM) :: IAUX, JALL, &
 & IAUSTAT, IAUEVEN1, IAUEVEN2, IAURDBFL, IAUANFLG, &
 & IAUPFLG, IAUFLG  , IAUVERCO, IAULEVID  

LOGICAL :: LLFAIL, LLREMOVE

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, ZALT, &
 & ZTIMDIF, &
 & ZWEIGHT, &
 & ZPRESAU, ZHEIGOG  

REAL(KIND=JPRB) :: ZUUPRES, ZUUPRL, ZU, &
 & ZUUOERR, ZUUPERR, ZUUFERR, ZUURERR, &
 & ZUUFGER, ZUUFGCC1, ZUUFGCC2, &
 & ZUUFGDEP, ZUUANDEP  
REAL(KIND=JPRB) :: ZVVPRES, ZVVPRL, ZV, &
 & ZVVOERR, ZVVPERR, ZVVFERR, ZVVRERR, &
 & ZVVFGER, ZVVFGCC1, ZVVFGCC2, &
 & ZVVFGDEP, ZVVANDEP  
REAL(KIND=JPRB) :: ZAU, ZAUPRES, ZAUPRL, &
 & ZAUPERR, ZAUOERR, ZAUFERR, ZAURERR, &
 & ZAUFGER, ZAUFGCC1, ZAUFGCC2, &
 & ZAUFGDEP, ZAUANDEP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "errstat.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "pilotbe.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"
#include "rdbflr.intfb.h"
#include "z2picao.intfb.h"

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET

!                       ------
IF (LHOOK) CALL DR_HOOK('AWPRFIN',0,ZHOOK_HANDLE)
ZLAT   = ROBHDR(KPILP,MDBLAT) * RDEGREES
ZLON   = ROBHDR(KPILP,MDBLON) * RDEGREES
ZALT   = ROBHDR(KPILP,MDBALT)
IOBDATE= ROBHDR(KPILP,MDBDAT)
IOBTIME= ROBHDR(KPILP,MDBETM)
CLSTID = TRANSFER(ROBHDR(KPILP,MDBSID), CLSTID)
ISTID  = 0
READ(CLSTID(1:8),'(A8)') ISTID
IREPST = ROBHDR(KPILP,MDBRST)
CALL RDBFLR(KPILP,IRDBMXR,IFRLAT,IFRLON,IFRDAT,IFRTIM,IFRALT)

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KPILP,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KPILP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.1       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.2         BODY PARAMETERS

ALL_DATA_LOOP: DO JALL = MLNKH2B(KPILP),MLNKH2B(KPILP+1)-1

!*          3.2.1       WE ACCEPT: U and V

  IF(ROBODY(JALL  ,MDBVNM) == NVNUMB(  1) .OR. &
   & ROBODY(JALL  ,MDBVNM) == NVNUMB(  2)       ) GO TO 322  

!*          3.2.1.0     UNEXPECTED VARIABLE

  IAUX = JALL
  IVNM = ROBODY(IAUX  ,MDBVNM)
  WRITE(NULOUT,'('' REO3SIN; UNEXPECTED VARIABLE: '',I10)') IVNM
  LDMESAGE = .TRUE.

!*          3.2.1.1     SET/GET

  CALL GETSETE(IAUX ,&
   & ZAUPRES,ZAUPRL,ZAU ,&
   & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
   & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
   & ZAUFGDEP,ZAUANDEP,&
   & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
   & IAUFLG,IAUPFLG)  

  ZAUPRES = ROBODY(IAUX,MDBPPP)
  IAUVERCO= ROBODY(IAUX,MDBVCO)
  IAULEVID= ROBODY(IAUX,MDBPILID)

!*          3.2.1.2     VARIBLE

  ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.2.1.3     ERROR STATISTICS

!*          3.2.1.4     STATUS, EVENTS, FLAGS

  IAUSTAT = ICHSTAT_REJECT( IAUSTAT )

!*          3.2.1.5     DEPARTURES

!*          3.2.1.6     UPDATE BODY ENTRY

  CALL PILOTBE(IAUX    ,&
   & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
   & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
   & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
   & ZAUFGDEP,ZAUANDEP,&
   & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
   & IUULEVID                                      )  

!*          3.2.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

  322 CONTINUE

!*          3.2.1.1     VERTICAL COORDINATE

  3211 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 3212  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF

!*          3.2.1.2     VARIABLE

  3212 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3213  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.2.1.3     ERROR STATISTICS

  3213 CONTINUE
  ROBODY(JALL  ,MDBOER) = RMDI
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.2.3       CLOSE ALL DATA LOOP

ENDDO ALL_DATA_LOOP

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (WIND)
!                       -----------------------

ZWEIGHT = 1.0_JPRB

!*          4.1         LOOK FOR WIND DATA

ISTART  = MLNKH2B(KPILP)
IEND    = MLNKH2B(KPILP+1)-1
WIND_DATA_LOOP: DO JWIND   = ISTART , IEND

!*          4.2         U COMPONENT

  IF(ROBODY(JWIND,MDBVNM) == NVNUMB(  1)) THEN
    IU   = JWIND
    IVNM = NVNUMB(  1)

!*          4.2.0       SET/GET

    CALL GETSETE(IU ,&
     & ZUUPRES ,ZUUPRL  ,ZU      ,&
     & ZUUFGER ,ZUUFGCC1,ZUUFGCC2,&
     & ZUUPERR ,ZUUOERR ,ZUUFERR ,ZUURERR ,&
     & ZUUFGDEP,ZUUANDEP,&
     & IUUSTAT ,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
     & IUUFLG  ,IUUPFLG)  

    ZUUPRES = ROBODY(IU,MDBPPP  )
    IUULEVID= ROBODY(IU,MDBPILID)
    IF(IUULEVID == 0)  THEN
      ISIGW = 1
      IUULEVID = INSERT(IUULEVID,ISIGW,NPSWLBP)
    ENDIF
    IUUVERCO= ROBODY(IU,MDBVCO  )
   
!*          4.2.1       VERTICAL COORDINATE

    ZPRESAU = RMDI
    IPAUFLG = NMDI
    IF(IUUVERCO == NHEIGVC)     THEN
      IF(ZALT    == RMDI .OR. &
         & ZUUPRES == RMDI       )  THEN  
        IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
      ELSE
        ZHEIGOG = ZALT + ZUUPRES
        CALL Z2PICAO(ZHEIGOG,ZPRESAU)
        ZPRESAU = ZPRESAU * 100._JPRB
        IPAUFLG = MAX(IUUPFLG,IFRALT)
        ZUUPRES = ZHEIGOG * RG
        IUUPFLG = IPAUFLG
      ENDIF
    ELSE
      ZPRESAU = ZUUPRES
      IPAUFLG = IUUPFLG
    ENDIF

    IF(ZUUPRES  == RMDI .OR. &
       & IUULEVID == NMDI .OR. &
       & IUUVERCO == NMDI       ) THEN  
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF

!*          4.2.2       VARIABLE

    IF(ROBODY(IU,MDBVAR) == RMDI    )        THEN
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF
    ZU      = ROBODY(IU,MDBVAR)

!*          4.2.3       ERROR STATISTICS

    IF(ZPRESAU /= RMDI )  THEN
      CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
       & IVNM,IUUSTAT,ZPRESAU,ZUUPRL,&
       & ZUUPERR,ZUUOERR,ZUUFERR,ZWEIGHT)  
    ENDIF

!*          4.2.4       STATUS

    IF(NMKCMVSE(  1,KCDSQNO,KOBSQNO) == 0)          THEN
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF
    CALL PPVAFL(IUUPFLG,IUUFLG,IUURDBFL)

!*          4.2.5       DEPARTURES

!*          4.2.6       UPDATE/CREATE BODY ENTRY

    CALL PILOTBE(IU      ,&
     & IVNM    ,IUUVERCO,ZUUPRES ,ZUUPRL  ,ZU      ,&
     & ZUUOERR ,ZUUPERR ,ZUUFERR ,ZUURERR ,&
     & ZUUFGER ,ZUUFGCC1,ZUUFGCC2,&
     & ZUUFGDEP,ZUUANDEP,&
     & IUUSTAT ,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
     & IUULEVID                                      )  

!*          4.3         V COMPONENT (ASSUMED TO BE THE NEXT)

  ELSEIF(ROBODY(JWIND,MDBVNM) == NVNUMB(  2)) THEN
    IV   = JWIND
    IVNM = NVNUMB(  2)

!*          4.3.0       SET/GET

    CALL GETSETE(IV ,&
     & ZVVPRES,ZVVPRL,ZV  ,&
     & ZVVFGER,ZVVFGCC1,ZVVFGCC2,&
     & ZVVPERR,ZVVOERR,ZVVFERR,ZVVRERR,&
     & ZVVFGDEP,ZVVANDEP,&
     & IVVSTAT,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
     & IVVFLG,IVVPFLG)  

    ZVVPRES = ROBODY(IV,MDBPPP  )
    IVVLEVID= ROBODY(IV,MDBPILID)
    IF(IVVLEVID == 0)  THEN
      ISIGW = 1
      IVVLEVID = INSERT(IVVLEVID,ISIGW,NPSWLBP)
    ENDIF
    IVVVERCO= ROBODY(IV,MDBVCO  )

!*          4.3.1       VERTICAL COORDINATE

    ZPRESAU = RMDI
    IPAUFLG = NMDI
    IF(IVVVERCO == NHEIGVC)     THEN
      IF(ZALT    == RMDI .OR. &
         & ZVVPRES == RMDI       )  THEN  
        IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
      ELSE
        ZHEIGOG = ZALT + ZVVPRES
        CALL Z2PICAO(ZHEIGOG,ZPRESAU)
        ZPRESAU = ZPRESAU * 100._JPRB
        IPAUFLG = MAX(IVVPFLG,IFRALT)
        ZVVPRES = ZHEIGOG * RG
        IVVPFLG = IPAUFLG
      ENDIF
    ELSE
      ZPRESAU = ZVVPRES
      IPAUFLG = IVVPFLG
    ENDIF

    IF(ZVVPRES  == RMDI .OR. &
       & IVVLEVID == NMDI .OR. &
       & IVVVERCO == NMDI       ) THEN  
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF

!*          4.3.2       VARIABLE

    IF(ROBODY(IV,MDBVAR) == RMDI    )        THEN
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF
    ZV      = ROBODY(IV,MDBVAR)

!*          4.3.3       ERROR STATISTICS

    IF(ZPRESAU /= RMDI )  THEN
      CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
       & IVNM,IVVSTAT,ZPRESAU,ZVVPRL,&
       & ZVVPERR,ZVVOERR,ZVVFERR,ZWEIGHT)  
    ENDIF

!*          4.3.4       STATUS

    IF(NMKCMVSE(  1,KCDSQNO,KOBSQNO) == 0)          THEN
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF
    CALL PPVAFL(IVVPFLG,IVVFLG,IVVRDBFL)

!*          4.3.5       DEPARTURES

!*          4.3.6       UPDATE/CREATE BODY ENTRY

    CALL PILOTBE(IV      ,&
     & IVNM    ,IVVVERCO,ZVVPRES ,ZVVPRL  ,ZV      ,&
     & ZVVOERR ,ZVVPERR ,ZVVFERR ,ZVVRERR ,&
     & ZVVFGER ,ZVVFGCC1,ZVVFGCC2,&
     & ZVVFGDEP,ZVVANDEP,&
     & IVVSTAT ,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
     & IVVLEVID                                      )  
  ENDIF
ENDDO WIND_DATA_LOOP

!*
!-----------------------------------------------------------------------

!        5.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

5 CONTINUE

!*          5.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          5.2         ODB REPORT UPDATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        6.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

6 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        7.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('AWPRFIN',1,ZHOOK_HANDLE)
END SUBROUTINE AWPRFIN
