SUBROUTINE BLACKSAT(KSET,KLEN)

!-------------------------------------------------------------------
!     Interface CNRM pour effectuer le "blacklisting" 
!          des tovs et ers

!          PUECH Dominique   GMAP     15/OCT/1998

!     ***CALL BLACKSAT(KSET)
!        KSET = numero du jeu d'observation
!        appel dans decis.F

!     appel externe :

! Modifications
!     26/03/1999   preparation sat 205   D.Puech
!     06/07/1999   modif nom blakctovs -> blacksat
!                  traitement ers
!     03/04/2000   -hirs6 sur glace + adaptation F.R.
!     07/08/2001   traitement RAD1C E.Gerard
!     26/06/2003   traitement HIRS et optimisation AMSUA D.Lacroix 
!     09/10/2003   traitement AIRS T.Auligne
!     08/09/2003   traitement AMSUB Z.Sahlaoui
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     26/06/2004   traitement scatterometres  C. Payan
!     22/12/2004   traitement SSMI E.Gerard
!     08/02/2005   routine SATAM only, SCATT dans blackcln C.Payan
!     22/09/2005   reajustement du controle de SSMI Z.Sahlaoui
!-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMVNMB  , ONLY : NVNUMB
USE YOMLUN   , ONLY : NULOUT
USE YOMCOCTP , ONLY : NGTHRB   ,NSATEM
USE YOMANCS  , ONLY : RDEGREES
USE YOMTVRAD , ONLY : MXSENSOR, MSENSOR_HIRS, MSENSOR_AMSUA, &
 & MSENSOR_AIRS, MSENSOR_METEOSAT, MSENSOR_MSG_HR,&
 & MSENSOR_AMSUB,MSENSOR_SSMI,MSENSOR_MHS
USE YOMDIMO  , ONLY : NMXTCHU

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET, KLEN
INTEGER(KIND=JPIM),PARAMETER :: JPTCHAN=2442 !47(ATOVS) + 2(MVIRI) +8(SEVIRI) + 2378(AIRS) + 7(SSMI)
#include "fcobs.h"
#include "qabitu.h"

INTEGER(KIND=JPIM) :: JOBS, IOBTYP
INTEGER(KIND=JPIM) :: ICMLSQ, JNLV, ICMVNM
INTEGER(KIND=JPIM) :: ICMPPP, ISCPOS
INTEGER(KIND=JPIM) :: ICDT, IWINDOW_CH(0:MXSENSOR), ISSIOB, ICHOFF
INTEGER(KIND=JPIM) :: ICLWP_CH1(0:MXSENSOR), ICLWP_CH2(0:MXSENSOR)
INTEGER(KIND=JPIM) :: ICLD_CH11, ICLD_CH12
REAL(KIND=JPRB) :: ZWCD(0:MXSENSOR)
REAL(KIND=JPRB) :: ZMODOR, ZCMLAT, ZTS, ZCMLSQ, ZAZIM
REAL(KIND=JPRB) :: ZCLWP, ZCLWP_CH1, ZCLWP_CH2, Z_D0, Z_D1, Z_D2
REAL(KIND=JPRB) :: ZCLD_CH11, ZCLD_CH12
REAL(KIND=JPRB) :: Z_PRED_SST
REAL(KIND=JPRB) :: ZVR_CH791,ZVR_CH914,ZVR_CH1285,ZVR_CH1301,ZVR_CH2333
REAL(KIND=JPRB) :: ZTB(JPTCHAN), ZDP(JPTCHAN), ZVR(JPTCHAN)

! initialisations pour tovs
INTEGER(KIND=JPIM) :: IBL_PARTOUT(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_TERRE(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_MODORO(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_MER(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_NUAGEUX(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_LAT(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_SEAICE(JPTCHAN)
INTEGER(KIND=JPIM) :: IBL_SET(JPTCHAN)
INTEGER(KIND=JPIM) :: INDEX(NMXTCHU)
LOGICAL :: LL_LFLAG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('BLACKSAT',0,ZHOOK_HANDLE)
IF(KSET == 1)WRITE(NULOUT,'('' *** BLACKLISTING DES TOVS '')')

! initialisation par canal 0=ok  1=bl
IBL_TERRE(:)=1
IBL_MODORO(:)=1
IBL_MER(:)=1
IBL_NUAGEUX(:)=1
IBL_LAT(:)=1
IBL_SEAICE(:)=1
LL_LFLAG=.TRUE.

! num canaux        1  2  3  4  5  6  7  8  9 10 11 12 13 14
! HIRS             15 16 17 18 19 20 21 22 23 24 25 26 27
IBL_PARTOUT(1:27)=(/1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0,&
                  & 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 /)  
IBL_TERRE(1:27)=(/  0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1,&
                  & 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 /)  
IBL_MODORO(1:27)=(/ 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1,&
                  & 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 /)  
 
! num canaux             28 29 30 31 32 33 34 35 36 37 38 39 40 41 42
! AMSUA                   1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 
IBL_TERRE(28:42)     =(/  1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1 /)
IBL_MODORO(28:42)    =(/  1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1 /)
IBL_MER(28:42)       =(/  1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1 /)
IBL_SEAICE(28:42)    =(/  1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1 /)
IBL_NUAGEUX(28:42)   =(/  1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1 /)
IBL_LAT(28:42)       =(/  1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1 /)

! num canaux             43 44 45 46 47
! AMSUB                   1  2  3  4  5 
IBL_PARTOUT(43:47)   =(/  1, 1, 0, 0, 0 /)
IBL_MER(43:47)       =(/  0, 0, 0, 0, 0 /)
IBL_TERRE(43:47)     =(/  0, 0, 0, 0, 1 /)
IBL_MODORO(43:47)    =(/  0, 0, 1, 1, 0 /)
IBL_SEAICE(43:47)    =(/  0, 0, 1, 1, 1 /)
IBL_NUAGEUX(43:47)   =(/  0, 0, 1, 1, 1 /)

!2 canaux            1    2
!* Meteosat/MVIRI
IBL_TERRE(48:49)  = (/ 0, 0 /)
IBL_MODORO(48:49) = (/ 0, 0 /)
IBL_MER(48:49)    = (/ 0, 0 /)
IBL_SEAICE(48:49) = (/ 0, 0 /)
IBL_NUAGEUX(48:49)= (/ 1, 1 /)
IBL_LAT(48:49)    = (/ 0, 0 /)

!8 canaux  1    2    3    4    5    6    7    8
!* MSG/SEVIRI
IBL_TERRE(50:57)  = (/ 1, 0, 0, 1, 1, 1, 1, 1 /)
IBL_MODORO(50:57) = (/ 1, 0, 0, 1, 1, 1, 1, 1 /)
IBL_MER(50:57)    = (/ 1, 0, 0, 0, 1, 0, 0, 1 /)
IBL_SEAICE(50:57) = (/ 1, 0, 0, 1, 1, 1, 1, 1 /)
IBL_NUAGEUX(50:57)= (/ 1, 0, 0, 1, 1, 1, 1, 1 /)
IBL_LAT(50:57)    = (/ 1, 0, 0, 0, 1, 0, 0, 1 /)

!324 channels AIRS
!Index     1    2    3    4    5    6    7    8    9   10   11   12   13   14   15   16   17   18   19   20

INDEX=&
 &(/ 1,   6,   7,  10,  11,  15,  16,  17,  20,  21,  22,  24,  27,  28,  30,  36,  39,  40,  42,  51,&
 &  52,  54,  55,  56,  59,  62,  63,  68,  69,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  82,&
 &  83,  84,  86,  92,  93,  98,  99, 101, 104, 105, 108, 110, 111, 113, 116, 117, 123, 124, 128, 129,&
 & 138, 139, 144, 145, 150, 151, 156, 157, 159, 162, 165, 168, 169, 170, 172, 173, 174, 175, 177, 179,&
 & 180, 182, 185, 186, 190, 192, 193, 198, 201, 204, 207, 210, 213, 215, 216, 218, 221, 224, 226, 227,&
 & 232, 239, 248, 250, 251, 252, 253, 256, 257, 261, 262, 267, 272, 295, 299, 300, 305, 308, 309, 310,&
 & 318, 321, 325, 333, 338, 355, 362, 375, 453, 475, 484, 497, 528, 587, 672, 787, 791, 843, 870, 914,&
 & 950,1003,1012,1019,1024,1030,1038,1048,1069,1079,1082,1083,1088,1090,1092,1095,1104,1111,1115,1116,&
 &1119,1120,1123,1130,1138,1142,1178,1199,1206,1221,1237,1252,1260,1263,1266,1278,1285,1290,1301,1304,&
 &1329,1371,1382,1400,1401,1402,1403,1415,1424,1449,1455,1466,1471,1477,1479,1488,1500,1519,1520,1538,&
 &1545,1565,1574,1583,1593,1614,1627,1636,1644,1652,1669,1674,1681,1694,1708,1717,1723,1740,1748,1751,&
 &1756,1763,1766,1771,1777,1780,1783,1794,1800,1803,1806,1812,1826,1843,1852,1865,1866,1867,1868,1869,&
 &1872,1873,1875,1876,1877,1881,1882,1883,1884,1897,1901,1911,1917,1918,1921,1923,1924,1928,1937,1938,&
 &1939,1941,1946,1947,1948,1958,1971,1973,1988,1995,2084,2085,2097,2098,2099,2100,2101,2103,2104,2106,&
 &2107,2108,2109,2110,2111,2112,2113,2114,2115,2116,2117,2118,2119,2120,2121,2122,2123,2128,2134,2141,&
 &2145,2149,2153,2164,2189,2197,2209,2226,2234,2280,2318,2321,2325,2328,2333,2339,2348,2353,2355,2357,&
 &2363,2370,2371,2377 /)
INDEX(:)=INDEX(:)+57
IBL_MER(INDEX((/3,6,9,10,11,13,14,18,21,29,44,45,46,47,49,50,52,53,55,56/)))=0

IBL_LAT(58:2435)=IBL_MER(58:2435)
IBL_TERRE(58:2435)=IBL_MER(58:2435) ! CEP sur
!IBL_TERRE(261:2435)=1                  ! terre
IBL_MODORO(58:2435)=1
IBL_SEAICE(58:2435)=1         !IBL_TERRE(58:2435)
IBL_NUAGEUX(58:2435)=1        !IBL_TERRE(58:2435)

! num canaux              2436 2437 2438 2439 2440 2441 2442
! SSMI                       1    2    3    4    5    6    7
IBL_TERRE(2436:2442)  =  (/  1,   1,   1,   1,   1,   1,   1 /)
IBL_NUAGEUX(2436:2442)=  (/  1,   1,   1,   1,   1,   1,   1 /)

!*
!     ------------------------------------------------------------------

IWINDOW_CH(:) = 0
IWINDOW_CH(MSENSOR_HIRS) = 8
IWINDOW_CH(MSENSOR_AMSUA)= 4
IWINDOW_CH(MSENSOR_AMSUB)= 2
IWINDOW_CH(MSENSOR_MHS)= 2
IWINDOW_CH(MSENSOR_AIRS) = 787
IWINDOW_CH(MSENSOR_MSG_HR) = 6
IWINDOW_CH(MSENSOR_METEOSAT) = 2
ICLWP_CH1(:) = 0
ICLWP_CH2(:) = 0
ICLWP_CH1(MSENSOR_AMSUA) = 1
ICLWP_CH2(MSENSOR_AMSUA) = 2
ICLD_CH11                = 11
ICLD_CH12                = 12

!     ------------------------------------------------------------------

! boucle sur le nombre d'enregistrements
! ============================================

REPORT_LOOP: DO JOBS = 1,KLEN

  IOBTYP = ROBHDR(JOBS,MDBOTP)

  IF(IOBTYP /= NSATEM) CYCLE REPORT_LOOP

  ICDT   = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)


! traitement des tovs
!------------------------
   IF ( ICDT == NGTHRB ) THEN

    IBL_SET = 0
    ZMODOR = ROBHDR(JOBS,MDB_OROGRAPHY) ! orography modele
    ZCMLAT = ROBHDR(JOBS,MDBLAT)*RDEGREES ! latitude obs
    ZCMLSQ = ROBHDR(JOBS,MDB_LSM) ! model land-sea mask : 0=sea
    IF (ZCMLSQ < 0.1) THEN
      ICMLSQ = 0                 ! mer
    ELSE
      ICMLSQ = 1                 ! terre
    ENDIF
    ZTS    = ROBHDR(JOBS,MDB1BCP3A)
    ZAZIM  = SATHDR(JOBS,MDB_AZIMUTH_AT_SAT) ! azimuth
    ISCPOS = SATHDR(JOBS,MDB_SCANPOS_AT_RADIANCE) ! position de scan
    ISSIOB = ROBHDR(JOBS,MDBSSIA)

    ZWCD = 1000000.0_JPRB
    IF(ISSIOB /= MSENSOR_HIRS .AND. ISSIOB /= MSENSOR_AMSUA &
    &  .AND. ISSIOB /= MSENSOR_AMSUB .AND. ISSIOB /= MSENSOR_SSMI &
    &  .AND. ISSIOB /= MSENSOR_MHS &
    &  .AND. ISSIOB /= 11 .AND. ISSIOB /= 20 &
    &  .AND. ISSIOB /= 21 .AND. ISSIOB /= 29) WRITE(0,*) 'PB in ISSIOB',ISSIOB
    IF (ISSIOB == MSENSOR_HIRS) THEN
      IF(ISCPOS < 1.OR.ISCPOS > 56) WRITE(0,*) 'PB in MDB_SCANPOS_AT_RADIANCE ',ISCPOS
    ELSEIF (ISSIOB == MSENSOR_AMSUA) THEN
      IF(ISCPOS < 1.OR.ISCPOS > 30) WRITE(0,*) 'PB in MDB_SCANPOS_AT_RADIANCE ',ISCPOS
    ELSEIF (ISSIOB == MSENSOR_AMSUB .OR. ISSIOB == MSENSOR_MHS) THEN
      IF(ISCPOS < 1.OR.ISCPOS > 90) WRITE(0,*) 'PB in MDB_SCANPOS_AT_RADIANCE ',ISCPOS
    ELSEIF (ISSIOB == MSENSOR_SSMI) THEN
      IF(ISCPOS < 1.OR.ISCPOS > 64) WRITE(0,*) 'PB in MDB_SCANPOS_AT_RADIANCE ',ISCPOS
    ELSEIF (ISSIOB == 11) THEN
      IF(ISCPOS < 1.OR.ISCPOS > 90) WRITE(0,*) 'PB in MDB_SCANPOS_AT_RADIANCE ',ISCPOS
    ENDIF

! boucle sur le nombre d'obs (canaux) pour initialisation 

    DATUM_INIT_CHECK: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
      ICMPPP = ROBODY(JNLV,MDBPPP)

      IF(ICMPPP == IWINDOW_CH(ISSIOB)) ZWCD(ISSIOB) = ROBODY(JNLV,MDBOMF)
      IF(ICMPPP == ICLWP_CH1(ISSIOB))  ZCLWP_CH1 = ROBODY(JNLV,MDBVAR)
      IF(ICMPPP == ICLWP_CH2(ISSIOB))  ZCLWP_CH2 = ROBODY(JNLV,MDBVAR)
      IF(ICMPPP == ICLD_CH11)          ZCLD_CH11 = ROBODY(JNLV,MDBOMF)
      IF(ICMPPP == ICLD_CH12)          ZCLD_CH12 = ROBODY(JNLV,MDBOMF)

!  Evaluate Obs-guess departure for HIRS channel 8
      IF (ISSIOB == MSENSOR_HIRS ) THEN
        IF (ICMPPP == IWINDOW_CH(ISSIOB)) ZWCD(ISSIOB)= ROBODY(JNLV,MDBOMF)
      ENDIF

!  Evaluate predicted SST for AIRS from channels 791, 914, 1285, 1301 & 2333
      IF (ISSIOB == MSENSOR_AIRS ) THEN
        IF (ICMPPP == 791) THEN
          ZVR_CH791= ROBODY(JNLV,MDBVAR)
        ELSEIF (ICMPPP == 914) THEN
          ZVR_CH914= ROBODY(JNLV,MDBVAR)
        ELSEIF (ICMPPP == 1285) THEN
          ZVR_CH1285= ROBODY(JNLV,MDBVAR)
        ELSEIF (ICMPPP == 1301) THEN
          ZVR_CH1301= ROBODY(JNLV,MDBVAR)
        ELSEIF (ICMPPP == 2333) THEN
          ZVR_CH2333= ROBODY(JNLV,MDBVAR)
        ENDIF
      ENDIF
      ZTB(ICMPPP)=ROBODY(JNLV,MDBVAR)-ROBODY(JNLV,MDBOMF)
      ZVR(ICMPPP)=ROBODY(JNLV,MDBVAR)
      ZDP(ICMPPP)=ROBODY(JNLV,MDBOMF)
    ENDDO DATUM_INIT_CHECK

    IF (LL_LFLAG) THEN

!   Tests on HIRS channels
!   3 Pixels on each scan edge are eliminated
      IF (ISSIOB == MSENSOR_HIRS ) THEN
        IBL_SET(1:27)=IBL_PARTOUT(1:27)
        IF ((ISCPOS <= 3) .OR. (ISCPOS >= 54)) THEN
          IBL_SET(1:27)=1
        ELSE
          IF (ICMLSQ == 1) THEN
            IBL_SET(1:19) = IBL_SET(1:19) + IBL_TERRE(1:19)
            IF(ZMODOR > 1500) IBL_SET(1:19) = &
             & IBL_SET(1:19) + IBL_MODORO(1:19)  
          ENDIF

        ! Rejection of all channels over seaice
          IF ((ZTS < 271.45_JPRB) .AND. (ICMLSQ == 0)) THEN
            IBL_SET( 1:19)=IBL_SET( 1:19)+1
          ENDIF 

!       ! Blacklist at high latitudes
!       IF ( (ZCMLAT >= 70.) .OR. (ZCMLAT <= -70.) ) THEN
!         IBL_SET( 1:19)=IBL_SET( 1:19)+1
!       ENDIF

        ! Tests de contamination nuageuse

          IF (ZCLD_CH11 < -3.0) IBL_SET(11) = 1
          IF (ZCLD_CH12 < -3.0) IBL_SET(12) = 1

          IF ((ZWCD(ISSIOB) < -1.0) .OR. (ZWCD(ISSIOB) > 2.0)) IBL_SET(7) = 1
          IF (ABS(ZWCD(ISSIOB)) > 4.0) IBL_SET(11) = 1
          IF (ABS(ZWCD(ISSIOB)) > 10.0) IBL_SET(12) = 1

        ! Area : 90S-60S
          IF ( (ZCMLAT >= -90.) .AND. (ZCMLAT <= -60.) ) THEN
            IF (ABS(ZWCD(ISSIOB)) > 12.0) IBL_SET(4) = 1
            IF (ABS(ZWCD(ISSIOB)) > 6.0) IBL_SET(5) = 1
            IF (ABS(ZWCD(ISSIOB)) > 4.0) IBL_SET(6) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(14) = 1
            IF (ABS(ZWCD(ISSIOB)) > 4.0) IBL_SET(15) = 1
          ENDIF
        ! Area : 60S-30S
          IF ( (ZCMLAT > -60.) .AND. (ZCMLAT <= -30.) ) THEN
            IF (ABS(ZWCD(ISSIOB)) > 10.0) IBL_SET(4) = 1
            IF (ABS(ZWCD(ISSIOB)) > 4.0) IBL_SET(5) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(6) = 1
            IF ((ZWCD(ISSIOB) < -2.5) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(14) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(15) = 1
          ENDIF
        ! Area : 30S-30N
          IF ( (ZCMLAT > -30.) .AND. (ZCMLAT <= 30.) ) THEN
            IF (ABS(ZWCD(ISSIOB)) > 5.0) IBL_SET(4) = 1
            IF ((ZWCD(ISSIOB) < -2.0) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(5) = 1
            IF ((ZWCD(ISSIOB) < -1.5) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(6) = 1
            IF ((ZWCD(ISSIOB) < -1.6) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(14) = 1
            IF ((ZWCD(ISSIOB) < -2.0) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(15) = 1
          ENDIF
        ! Area : 30N-60N
          IF ( (ZCMLAT > 30.) .AND. (ZCMLAT <= 60.) ) THEN
            IF (ABS(ZWCD(ISSIOB)) > 8.0) IBL_SET(4) = 1
            IF (ABS(ZWCD(ISSIOB)) > 4.0) IBL_SET(5) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(6) = 1
            IF ((ZWCD(ISSIOB) < -2.0) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(14) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(15) = 1
          ENDIF
        ! Area : 60N-90N
          IF ( (ZCMLAT >= 60.) .AND. (ZCMLAT <= 90.) ) THEN
            IF (ABS(ZWCD(ISSIOB)) > 8.0) IBL_SET(4) = 1
            IF (ABS(ZWCD(ISSIOB)) > 4.0) IBL_SET(5) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(6) = 1
            IF ((ZWCD(ISSIOB) < -2.5) .OR. (ZWCD(ISSIOB) > 3.0)) IBL_SET(14) = 1
            IF (ABS(ZWCD(ISSIOB)) > 3.0) IBL_SET(15) = 1
          ENDIF
        ENDIF

!   Tests on AMSUA channels
!   3 Pixels on each scan edge are eliminated

      ELSEIF (ISSIOB == MSENSOR_AMSUA ) THEN
        IF ((ISCPOS <= 3) .OR. (ISCPOS >= 28)) THEN
          IBL_SET(28:42)=1
        ELSE
          IF (ICMLSQ == 1) THEN
            IBL_SET(28:42)=IBL_SET(28:42)+IBL_TERRE(28:42)
            IF (ZMODOR >  500) IBL_SET(32)=IBL_SET(32)+IBL_MODORO(32)
            IF (ZMODOR > 1500) IBL_SET(33)=IBL_SET(33)+IBL_MODORO(33)
          ENDIF

          IF (ICMLSQ == 0) THEN
            IBL_SET(28:42)=IBL_SET(28:42)+IBL_MER(28:42)
            IF (ZTS < 271.45_JPRB) IBL_SET(28:42)=&
             & IBL_SET(28:42)+IBL_SEAICE(28:42)  
          ENDIF

          ZCLWP = 0
          IF (             (ICMLSQ == 0) .AND. (ZTS >= 271.45_JPRB) .AND. &
             & (ZCLWP_CH1 < 285._JPRB) .AND. (ZCLWP_CH2 < 285._JPRB) ) THEN  
          ! Calcul du CLWP
            Z_D0 = 8.240 - (2.622 - 1.846*COS(ZAZIM))*COS(ZAZIM)
            Z_D1 = 0.754
            Z_D2 = -2.265
            ZCLWP = COS(ZAZIM)*(Z_D0 + Z_D1*LOG(285.0-ZCLWP_CH1) + Z_D2*LOG(285.0-ZCLWP_CH2))
          ENDIF

          IF ( (ICMLSQ == 0) .AND. ((ABS(ZWCD(ISSIOB)) > 0.7) .OR. (ZCLWP > 0.1_JPRB)) ) THEN
            IBL_SET(28:42)=IBL_SET(28:42)+IBL_NUAGEUX(28:42)
            IF (ABS(ZCMLAT) <= 30.) THEN
              IBL_SET(28:42)=IBL_SET(28:42)+IBL_LAT(28:42)
            ENDIF
          ENDIF

          IF ( (ICMLSQ == 1) .AND. (ABS(ZWCD(ISSIOB)) > 0.7))  THEN
            IBL_SET(28:42)=IBL_SET(28:42)+IBL_NUAGEUX(28:42)
            IF (ABS(ZCMLAT) <= 30.) THEN
              IBL_SET(28:42)=IBL_SET(28:42)+IBL_LAT(28:42)
            ENDIF
          ENDIF

        ENDIF

!   Tests on Meteosat/MVIRI channels
     ELSEIF (ISSIOB == MSENSOR_METEOSAT ) THEN
        IF (ICMLSQ == 1) THEN
          IBL_SET(48:49) = IBL_SET(48:49) + IBL_TERRE(48:49)
          IF(ZMODOR > 1500) IBL_SET(48:49) = &
                         &  IBL_SET(48:49) + IBL_MODORO(48:49)
        ENDIF
        IF (ICMLSQ == 0 ) THEN
          IBL_SET(48:49) = IBL_SET(48:49) + IBL_MER(48:49)
          IF(ZTS < 271.45_JPRB) IBL_SET(48:49) = &
                         &  IBL_SET(48:49) + IBL_SEAICE(48:49)
        ENDIF
        IF (ABS(ZWCD(ISSIOB)) > 2.5_JPRB) &
            & IBL_SET(48:49) = IBL_SET(48:49) + IBL_NUAGEUX(48:49)

!   Tests on MSG/SEVIRI channels
     ELSEIF (ISSIOB == MSENSOR_MSG_HR ) THEN
        IF (ICMLSQ == 1) THEN
          IBL_SET(50:57) = IBL_SET(50:57) + IBL_TERRE(50:57)
          IF(ZMODOR > 1500) IBL_SET(50:57) = &
                         &  IBL_SET(50:57) + IBL_MODORO(50:57)
        ENDIF
        IF (ICMLSQ == 0 ) THEN
          IBL_SET(50:57) = IBL_SET(50:57) + IBL_MER(50:57)
          IF(ZTS < 271.45_JPRB) IBL_SET(50:57) = &
                         &  IBL_SET(50:57) + IBL_SEAICE(50:57)
        ENDIF
        IF (ABS(ZWCD(ISSIOB)) > 4.5_JPRB) &
            & IBL_SET(50:57) = IBL_SET(50:57) + IBL_NUAGEUX(50:57)

      ELSEIF (ISSIOB == MSENSOR_AMSUB .OR. ISSIOB == MSENSOR_MHS) THEN
        IF ((ISCPOS <= 9) .OR. (ISCPOS >= 82)) THEN
          IBL_SET(43:47)=1
        ELSE
          IBL_SET(43:47)=IBL_PARTOUT(43:47)
          IF (ICMLSQ == 1) THEN
            IBL_SET(43:47)=IBL_SET(43:47)+IBL_TERRE(43:47)
            IF (ZMODOR > 1000) IBL_SET(46)=IBL_SET(46)+IBL_MODORO(46)
            IF (ZMODOR > 1500) IBL_SET(45)=IBL_SET(45)+IBL_MODORO(45)
          ENDIF

          IF (ZTS < 278._JPRB) THEN
            IBL_SET(43:47)=IBL_SET(43:47)+IBL_SEAICE(43:47)
          ELSE
            IF (ABS(ZWCD(ISSIOB)) > 5._JPRB ) THEN
              IBL_SET(43:47)=IBL_SET(43:47)+IBL_NUAGEUX(43:47)
            ENDIF
          ENDIF
        ENDIF

! -----------------------
!  Tests on AIRS channels
! -----------------------
      ELSEIF (ISSIOB == MSENSOR_AIRS ) THEN
    ! 4  Pixels on each scan edge are eliminated
        IF ((ISCPOS < 10) .OR. (ISCPOS > 80)) IBL_SET(58:2435)=1

    ! Bad-quality-flag channels are eliminated
        IF(ROBODY(JNLV,MDBRDFL) /= 0) IBL_SET(ICMPPP)=1

    ! Test on AIRS VIS/NIR clear pourcentage (day-time only!)
        IF((ROBHDR(JOBS,MDB_CLDCOVER_AT_ATOVS) >= 5).AND. &
         & (ROBHDR(JOBS,MDB_CLDCOVER_AT_ATOVS) < 100)) &
         & IBL_SET(58:2435) = IBL_SET(58:2435) + IBL_NUAGEUX(58:2435)  

          Z_PRED_SST=8.28206-0.97957*ZVR_CH791+0.60529*ZVR_CH914 &
           & +1.7444*ZVR_CH1285-0.40379*ZVR_CH1301  
    !--------------------------------------------
    ! Tests for cloud detection (Mitch Goldberg)
    !--------------------------------------------
    ! Test on longwave window channel
          IF (ZVR_CH914 < 270._JPRB) IBL_SET(58:2435) = IBL_SET(58:2435) + IBL_NUAGEUX(58:2435)

    ! Test on SST versus transparent shortwave window channel (night-time only!)
!        IF(ROBODY(JNLV,MDB_SOLAR_ZENITH_AT_SAT)>85 .AND. ((ZVR_CH2333-PRED_SST)>10_JPRB)) &
!          IBL_SET(58:2435) = IBL_SET(58:2435) + IBL_NUAGEUX(58:2435)

    ! Test on SST versus AIRS predicted SST from shortwave and longwave
          IF (((ZTS-Z_PRED_SST)<-0.6_JPRB).OR.((ZTS-Z_PRED_SST)>3.5_JPRB)) &
           & IBL_SET(58:2435) = IBL_SET(58:2435) + IBL_NUAGEUX(58:2435)  
!        ENDIF

    !------------------
    ! Pixels over land
        IF (ICMLSQ == 1) THEN
          IBL_SET(58:2435) = IBL_SET(58:2435) + IBL_TERRE(58:2435)
          IF(ZMODOR > 1500) IBL_SET(58:2435) = &
           & IBL_SET(58:2435) + IBL_MODORO(58:2435)  
        ENDIF

    ! Pixels over ocean
        IF (ICMLSQ == 0 ) THEN
          IBL_SET(58:2435) = IBL_SET(58:2435) + IBL_MER(58:2435)
          IF(ZTS < 271.45_JPRB) IBL_SET(58:2435) = &
           & IBL_SET(58:2435) + IBL_SEAICE(58:2435)  
          IF( ABS(ZCMLAT) < 30._JPRB ) IBL_SET(58:2435) = &
           & IBL_SET(58:2435) + IBL_LAT(58:2435)  
        ENDIF
      
!   Tests on SSMI channels
      ELSEIF (ISSIOB == MSENSOR_SSMI ) THEN

      ! Pixels over land (according to model) 
      ! (when extracting the data, done over land according to NESDIS) 
        IF (ICMLSQ == 1) THEN
          IBL_SET(2436:2442)=IBL_SET(2436:2442)+IBL_TERRE(2436:2442)
        ENDIF

      ! Cloudy pixels done in rad1cemis.F90

      ENDIF
    ENDIF
!--------------------------------------------------------------

! boucle sur le nombre d'obs

    DATUM_LOOP2: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
      ICMVNM = ROBODY(JNLV,MDBVNM)

      IF(ICMVNM == NVNUMB(6)) THEN  ! traitement pwc
        ROBODY(JNLV,MDBDSTA)=ZCHSTAT_BLACKLST(ROBODY(JNLV,MDBDSTA))
        CYCLE DATUM_LOOP2
      ENDIF

      IF (ISSIOB == MSENSOR_HIRS ) THEN
        ICHOFF=0
      ELSEIF (ISSIOB == 1 ) THEN
        ICHOFF=20
      ELSEIF (ISSIOB == 2 ) THEN
        ICHOFF=24
      ELSEIF (ISSIOB == MSENSOR_AMSUA ) THEN
        ICHOFF=27
      ELSEIF (ISSIOB == MSENSOR_AMSUB .OR. ISSIOB == MSENSOR_MHS) THEN
        ICHOFF=42
      ELSEIF (ISSIOB == MSENSOR_METEOSAT ) THEN
        ICHOFF=47
      ELSEIF (ISSIOB == MSENSOR_MSG_HR ) THEN
        ICHOFF=49
      ELSEIF (ISSIOB == MSENSOR_AIRS ) THEN
        ICHOFF=57
      ELSEIF (ISSIOB == MSENSOR_SSMI ) THEN
        ICHOFF=2435
      ENDIF

      IF(ICMVNM /= NVNUMB(50)) CYCLE DATUM_LOOP2  ! TOVS uniquement
      ICMPPP = ROBODY(JNLV,MDBPPP)

!   CHECK BACKGROUND AND MEASURED TB

!   FOR SSMI
      IF (ISSIOB == MSENSOR_SSMI ) THEN
        IF  ((ZTB(ICMPPP) < 50_JPRB) .OR. &
          & (ZTB(ICMPPP) > 350_JPRB)) THEN
          WRITE(NULOUT, *) 'BACKGROUND TB PROBLEM :',ZTB(ICMPPP),'Sensor',ISSIOB,'Channel',ICMPPP
          IBL_SET(ICMPPP+ICHOFF)=1 ! channel  eliminated
        ELSEIF ((ZVR(ICMPPP) < 50_JPRB) .OR. &
          & (ZVR(ICMPPP) > 350_JPRB)) THEN
          WRITE(NULOUT, *) 'MEASUREMENT  PROBLEM :',ZVR(ICMPPP),'Sensor',ISSIOB,'Channel',ICMPPP
          IBL_SET(ICMPPP+ICHOFF)=1 ! channel  eliminated
        ELSEIF (ABS(ZDP(ICMPPP)) > 20_JPRB) THEN
!          WRITE(NULOUT, *) 'DEPARTURE PROBLEM :',ZDP(ICMPPP),'channel',ICMPPP
          IBL_SET(ICMPPP+ICHOFF)=1 ! channel  eliminated
        ENDIF
!   FOR OTHER SENSORS
      ELSE
        IF  ((ZTB(ICMPPP) < 150_JPRB) .OR. &
           & (ZTB(ICMPPP) > 350_JPRB)) THEN
          WRITE(NULOUT, *) 'BACKGROUND TB PROBLEM :',ZTB(ICMPPP),'Sensor',ISSIOB,'Channel',ICMPPP
          IBL_SET(ICMPPP+ICHOFF)=1 ! channel  eliminated
        ELSEIF ((ZVR(ICMPPP) < 150_JPRB) .OR. &
           & (ZVR(ICMPPP) > 350_JPRB)) THEN
          WRITE(NULOUT, *) 'MEASUREMENT  PROBLEM :',ZVR(ICMPPP),'Sensor',ISSIOB,'Channel',ICMPPP
          IBL_SET(ICMPPP+ICHOFF)=1 ! channel  eliminated
        ELSEIF (ABS(ZDP(ICMPPP)) > 20_JPRB) THEN
!          WRITE(NULOUT, *) 'DEPARTURE PROBLEM :',ZDP(ICMPPP)
          IBL_SET(ICMPPP+ICHOFF)=1 ! channel  eliminated
        ENDIF
      ENDIF

! changement eventuel du status de la donnee
      IF( IBL_SET(ICMPPP+ICHOFF) > 0 )&
       & ROBODY(JNLV,MDBDSTA)=ZCHSTAT_BLACKLST(ROBODY(JNLV,MDBDSTA))  
    ENDDO DATUM_LOOP2
   ENDIF

ENDDO REPORT_LOOP

IF (LHOOK) CALL DR_HOOK('BLACKSAT',1,ZHOOK_HANDLE)
END SUBROUTINE BLACKSAT
