SUBROUTINE CHDSTA(PREPORT,KRLN,KSQN,CDDSTA)

!**** SUBROUTINE CHDSTA - CHANGE DATUM STATUS

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995

!*    PURPOSE
!     -------

!        CHANGE DATUM STATUS TO ACTIVE/PASSIVE/REJECTED/BLACKLISTED

!**   INTERFACE
!     ---------

!     ***CALL CHDSTA(PREPORT,KRLN,KSQN,CDDSTA)
!     WHERE PREPORT = THE WHOLE REPORT IN AN REAL ARRAY
!           KRLN    = THE REPORT LENGTH
!           KSQN    = SEQUNCE NUMBER OF THE OF THE DATUM
!           CDDSTA  = NEW DATUM STATUS

!        CHDSTA IS CALLED BY VARIOUS SUBROUTINES

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!        H. Jarvinen 12 Dec 1996 : A modified subroutine call
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCMHDR , ONLY : NCMHLN   ,NCMBLN   ,NCMNLV
USE YOMCMBDY , ONLY : NCMDSTA  ,NCMESQN  ,NCMDACBP ,NCMDPABP ,&
 & NCMDREBP ,NCMDBLBP  
USE YOMLUN   , ONLY : NULERR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KRLN 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PREPORT(KRLN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSQN 
CHARACTER(LEN=8)  ,INTENT(IN)    :: CDDSTA 
INTEGER(KIND=JPIM) :: IBLN, ICMDSTA, IHLN, INLV, IPOS, ISQN, J
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcobs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.1   INTIALIZE

IF (LHOOK) CALL DR_HOOK('CHDSTA',0,ZHOOK_HANDLE)
INVAL = 1

!*          1.2   POINTERS

IHLN = INTREA(PREPORT(NCMHLN))
IBLN = INTREA(PREPORT(NCMBLN))
INLV = INTREA(PREPORT(NCMNLV))

!*          1.3   FOR EXTENDED CMA

IF(KSQN <= INLV) THEN
  IPOS = IHLN + (KSQN-1)*IBLN
  ISQN = INTREA(PREPORT(IPOS+NCMESQN))
  IF(ISQN == KSQN) GOTO 2
ENDIF

!*          1.4   FOR COMPRESSED CMA

DO J = 1, INLV
  IPOS = IHLN + (J-1)*IBLN
  ISQN = INTREA(PREPORT(IPOS+NCMESQN))
  IF(ISQN == KSQN) GOTO 2
ENDDO

!*          1.5   SOMETHING IS WRONG

CALL ABOR1(' CHDSTA : SEQUENCE NUMBER NOT FOUND ')
!*
!     ------------------------------------------------------------------

!        2.       CHANGE DATUM STATUS
!                 -------------------

!*          2.0   INTIALIZE

2 CONTINUE
ICMDSTA = INTREA(PREPORT(IPOS+NCMDSTA))

!           2.1   ACTIVE

IF(CDDSTA == '  ACTIVE') THEN
  ICMDSTA = INSERT(ICMDSTA, INVAL, NCMDACBP)

!           2.2   PASSIVE

ELSEIF(CDDSTA == ' PASSIVE') THEN
  ICMDSTA = INSERT(ICMDSTA, INVAL, NCMDPABP)

!           2.3   REJECT

ELSEIF(CDDSTA == '  REJECT') THEN
  ICMDSTA = INSERT(ICMDSTA, INVAL, NCMDREBP)

!           2.4   BLACKLISTED

ELSEIF(CDDSTA == 'BLACKLST') THEN
  ICMDSTA = INSERT(ICMDSTA, INVAL, NCMDBLBP)

!           2.5   SOMETHING ELSE ?

ELSE
  WRITE(NULERR,'('' *** UNKNOWN KEYWORD : '',A)') CDDSTA
  WRITE(NULERR,'(''     - TRY "  ACTIVE" '')')
  WRITE(NULERR,'(''           " PASSIVE" '')')
  WRITE(NULERR,'(''           "  REJECT" '')')
  WRITE(NULERR,'(''           "BLACKLST" '')')
  CALL ABOR1('CHDSTA:UNKNOWN KEYWORD')
ENDIF

!           2.6   UPDATE STATUS WORD

PREPORT(IPOS+NCMDSTA)= REAINT(ICMDSTA)
!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                 ------
IF (LHOOK) CALL DR_HOOK('CHDSTA',1,ZHOOK_HANDLE)
END SUBROUTINE CHDSTA

