SUBROUTINE CHECKAIRPOS(KPATH,KNOAIR,KIDAIR,KMAX,KIDPAI)

!**** SUBROUTINE CHECKAIRPOS - REMOVE AIREPS WITH UNREALISTIC POSITION

!          LARS ISAKSEN ECMWF          31/01/2001

!*    PURPOSE
!     -------

!        REMOVE REPORTS DUE TO UNREALISTIC AIRCRAFT SPEED.
!     PRESELECTION HAS ALREADY BEEN MADE IN MOVPL BY DEFINING PATHS
!     AND POINTER TABLES.

!**   INTERFACE
!     ---------

!     ***CALL*CHECKAIRPOS(KPATH,KNOAIR)
!     WHERE KPATH  = FLIGHT PATH IN QUESTION
!           KNOAIR = NUMBER OF AIREP REPORTS IN THIS PATH
!           KIDAIR  = TOTAL NUMBER OF FLIGHT PATHS (FOR DIMENSIONING)
!           KMAX  = MAXIMUM NUMBER OF REPORTS IN A PATH (FOR DIMENSIONING)
!           KIDPAI  = IDENTIFIERS TO EACH REPORT IN EACH PATH

!        CHECKAIRPOS IS CALLED BY SUBROUTINE MOVPL

!*    METHOD
!     ------

!        UNREALISTIC AIRCRAFT POSITIONS ARE REJECTED FROM PATH
!        BEFORE THINNING.

!*    EXTERNALS
!     ---------

!        REFTIM - REFERENCE DATE
!        REJMV  - REJECT A MOVING PLATFORM

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCOCTP , ONLY : NAIREP
USE YOMANA   , ONLY : NANDAT   ,NANTIM
USE YOMANCS  , ONLY : RDEGREES
USE YOMSCC   , ONLY : LSCDPR   ,FL_MAXSPEED
USE YOMSCREE , ONLY : CSUPERSONIC ,NSUPERSONIC
USE YOMDB
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KIDAIR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPATH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNOAIR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDPAI(KIDAIR,KMAX) 
CHARACTER(LEN=8) ::  CLSTID

INTEGER(KIND=JPIM) :: IANRTM, ICMONM, ICMOTP,&
 & IOBDAT, IOBETM, IREALISTIC,&
 & IPOS, J1, J2, JOBS, IOBRTM(KNOAIR), INUMREJ,&
 & IMINDIFF, IMAX, INUM, IMAXINDEX, JSUPER  
INTEGER(KIND=JPIM) :: IIDATE, IITIME
REAL(KIND=JPRB) :: ZOBLAT(KNOAIR), ZOBLON(KNOAIR), ZFLMAXSPEED
REAL(KIND=JPRB) :: ZKEY  (KNOAIR)


LOGICAL :: LLSPEEDOK(KNOAIR,KNOAIR),LLREJECTED(KNOAIR),LLFLSPEEDBAD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "flspeedbad.intfb.h"
#include "reftim.intfb.h"
#include "rejmv.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------
!*

!           1.1   OBSERVATION TYPE

IF (LHOOK) CALL DR_HOOK('CHECKAIRPOS',0,ZHOOK_HANDLE)
ICMOTP = NAIREP
!*
!           1.2   ANALYSIS TIME (IN MINUTES FROM REFERENCE DATE)

CALL REFTIM(NANDAT,NANTIM,IANRTM)
!*
!           1.3   INITIALIZE SORTING AND KEY ARRAYS

INUMREJ=0
LLREJECTED(:)=.FALSE.

ZKEY  (:) = 0.0_JPRB
!*
!     ------------------------------------------------------------------

!        2.       SORT THE OBSERVATIONS AND PREPAPE FOR
!                 REJECTION OF  UNREALISTIC OBSERVATIONS 
!                 (FLIGTH SPEED > FL_MAXSPEED (1200 km/h) (Concorde???)
!                 ---------------------
!*
!           2.1   CHECK IF AIRCRAFT IS SUPERSONIC

IPOS = KIDPAI(KPATH,1)

!*          2.1.1 CONVERT STATION ID TO CHARACTER

CLSTID = TRANSFER(ROBHDR(IPOS,MDBSID), CLSTID)

!*          2.1.2 CHECK IF AIRCRAFT IS SUPERSONIC
!                 - INCREASE ACCEPTABLE SPEED TO 2*1200 KM/HOUR

ZFLMAXSPEED=FL_MAXSPEED
DO JSUPER = 1, NSUPERSONIC
  IF(CLSTID == CSUPERSONIC(JSUPER)) THEN
    ZFLMAXSPEED=2.*FL_MAXSPEED
    EXIT
  ENDIF
ENDDO

PATH_LOOP1: DO JOBS = 1, KNOAIR
!*
!           2.2   POINTER TO GLOBAL AIREP TIME-LOCATION ARRAY

  IPOS = KIDPAI(KPATH,JOBS)
!*
!              2.2.1   OBSERVATION LATITUDE AND LONGITUDE

  ZOBLAT(JOBS) = ROBHDR(IPOS,MDBLAT)
  ZOBLON(JOBS) = ROBHDR(IPOS,MDBLON)
!*
!              2.2.2   OBSERVATION TIME (IN MINUTES FROM REFERENCE DATE)

  IOBDAT = ROBHDR(IPOS,MDBDAT)
  IOBETM = ROBHDR(IPOS,MDBETM)
  CALL REFTIM(IOBDAT,IOBETM,IOBRTM(JOBS))

!*          2.3   CLOSE THE LOOP

ENDDO PATH_LOOP1

!*          2.4   REJECT AIREPS WITH UNREALISTIC POSITIONS

!*            2.4.1   FIND ALL SUSPECT HIGH SPEED PAIRS

LLSPEEDOK(:,:)=.TRUE.

DO J1=1,KNOAIR
  DO J2=J1+1,KNOAIR

    IMINDIFF=ABS(IOBRTM(J1)-IOBRTM(J2))

    CALL FLSPEEDBAD(ZOBLAT(J1),ZOBLON(J1),ZOBLAT(J2),ZOBLON(J2),&
     & IMINDIFF,ZFLMAXSPEED,LLFLSPEEDBAD)  
    IF (LLFLSPEEDBAD) THEN
      LLSPEEDOK(J1,J2)=.FALSE.
      LLSPEEDOK(J2,J1)=.FALSE.
    ENDIF
  ENDDO
ENDDO

!*            2.4.2   REJECT MOST SUSPECT MEASUREMENT. CONTINUE TO
!                     REMOVE SUSPECT MEASUREMENTS UNTIL A 
!                     CONSISTENT TRACK IS OBTAINED

IMAX=99
DO WHILE (IMAX > 0)
  IMAX=-99
  DO J1=1,KNOAIR
    INUM=0
    DO J2=1,KNOAIR
      IF (.NOT.LLSPEEDOK(J1,J2)) INUM=INUM+1
    ENDDO
    IF (INUM > IMAX) THEN
      IMAX=INUM
      IMAXINDEX=J1
    ENDIF
  ENDDO
  IF (IMAX > 0) THEN
    INUMREJ=INUMREJ+1
    LLREJECTED(IMAXINDEX)=.TRUE.
    IPOS = KIDPAI(KPATH,IMAXINDEX)
    ICMONM = ROBHDR(IPOS,MDBONM)
    CALL REJMV(IPOS,ICMOTP,ICMONM)

    IIDATE = ROBHDR(IPOS,MDBDAT)
    IITIME = ROBHDR(IPOS,MDBETM) 
    IF(LSCDPR) THEN
      WRITE(NULOUT,'(A,A8,A,2F8.2,2I10)') &
       & 'CHECKAIRPOS; REJECT AIREP ',CLSTID,' WITH UNREALISTIC LAT/LON: ',&
       & ZOBLAT(IMAXINDEX)*RDEGREES,ZOBLON(IMAXINDEX)*RDEGREES,&
       & IIDATE, IITIME 
    ENDIF
    LLSPEEDOK(IMAXINDEX,:)=.TRUE.
    LLSPEEDOK(:,IMAXINDEX)=.TRUE.
  ENDIF
ENDDO

!*            2.4.3   IF ONLY ONE LEFT IT IS REJECTED. 
!                     IF AT LEAST 80% IS REJECTED THE WHOLE PATH IS
!                     CONSIDERED SUSPECT AND ALL REPORTS ARE REJECTED

IF (KNOAIR-INUMREJ == 1 .OR. INUMREJ*10/MAX(KNOAIR,1) >= 8) THEN
  DO J1=1,KNOAIR
    IF (.NOT.LLREJECTED(J1)) THEN
      IPOS = KIDPAI(KPATH,J1)
      ICMONM = ROBHDR(IPOS,MDBONM)
      CALL REJMV(IPOS,ICMOTP,ICMONM)
      IF(LSCDPR) THEN
        IIDATE = ROBHDR(IPOS,MDBDAT)
        IITIME = ROBHDR(IPOS,MDBETM)
        WRITE(NULOUT,'(A,2F8.2,2I10)') &
         & 'CHECKAIRPOS; REJECT AIREP WITH UNREALISTIC LAT/LON: ',&
         & ZOBLAT(J1)*RDEGREES,ZOBLON(J1)*RDEGREES,&
         & IIDATE, IITIME
      ENDIF
    ENDIF
  ENDDO
  INUMREJ=KNOAIR
ENDIF

IREALISTIC=KNOAIR-INUMREJ

!     ------------------------------------------------------------------

!        5.       RETURN
!                 ------


IF (LHOOK) CALL DR_HOOK('CHECKAIRPOS',1,ZHOOK_HANDLE)
END SUBROUTINE CHECKAIRPOS
