SUBROUTINE CHRSTA(PREPORT,KRLN,CDRSTA)

!**** SUBROUTINE CHRSTA - CHANGE REPORT STATUS

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995

!*    PURPOSE
!     -------

!        CHANGE REPORT STATUS TO ACTIVE/PASSIVE/REJECTED/BLACKLISTED

!**   INTERFACE
!     ---------

!     ***CALL CHRSTA(PREPORT,KRLN,CDRSTA)
!     WHERE PREPORT = THE WHOLE REPORT IN AN REAL ARRAY
!           KRLN    = THE REPORT LENGTH
!           CDRSTA  = NEW REPORT STATUS

!        CHRSTA IS CALLED BY VARIOUS SUBROUTINES

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!        H. Jarvinen 12 Dec 1996 : A modified subroutine call
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCMHDR , ONLY : NCMRST   ,NCMRACBP ,NCMRPABP ,NCMRREBP ,NCMRBLBP
USE YOMLUN   , ONLY : NULERR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KRLN 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PREPORT(KRLN) 
CHARACTER(LEN=8)  ,INTENT(IN)    :: CDRSTA 
INTEGER(KIND=JPIM) :: ICMRST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcobs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

IF (LHOOK) CALL DR_HOOK('CHRSTA',0,ZHOOK_HANDLE)
ICMRST = INTREA(PREPORT(NCMRST))
INVAL = 1
!*
!     ------------------------------------------------------------------

!        2.       CHANGE REPORT STATUS
!                 --------------------

!           2.1   ACTIVE

IF(CDRSTA == '  ACTIVE') THEN
  ICMRST = INSERT(ICMRST, INVAL, NCMRACBP)

!           2.2   PASSIVE

ELSEIF(CDRSTA == ' PASSIVE') THEN
  ICMRST = INSERT(ICMRST, INVAL, NCMRPABP)

!           2.3   REJECT

ELSEIF(CDRSTA == '  REJECT') THEN
  ICMRST = INSERT(ICMRST, INVAL, NCMRREBP)

!           2.4   BLACKLISTED

ELSEIF(CDRSTA == 'BLACKLST') THEN
  ICMRST = INSERT(ICMRST, INVAL, NCMRBLBP)

!           2.5   SOMETHING ELSE ?

ELSE
  WRITE(NULERR,'('' *** UNKNOWN KEYWORD : '',A)') CDRSTA
  WRITE(NULERR,'(''     - TRY "  ACTIVE" '')')
  WRITE(NULERR,'(''           " PASSIVE" '')')
  WRITE(NULERR,'(''           "  REJECT" '')')
  WRITE(NULERR,'(''           "BLACKLST" '')')
  CALL ABOR1(' CHRSTA:UNKNOWN KEYWORD ')
ENDIF

!           2.6   UPDATE STATUS WORD

PREPORT(NCMRST)= REAINT(ICMRST)
!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                 ------
IF (LHOOK) CALL DR_HOOK('CHRSTA',1,ZHOOK_HANDLE)
END SUBROUTINE CHRSTA
