SUBROUTINE COMTC

!**** SUBROUTINE COMTC - COMPUTE JO IN THE CASE OF TIME CORR. OBS. ERRORS

!     HEIKKI JARVINEN and ERIK ANDERSSON,  ECMWF        15/JUN/1997

!*    PURPOSE
!     -------

!       CALCULE THE COST FUNCTION FOR THE SYNOP AND DRIBU OBSERVATIONS
!     FOR WHICH THE OBSERVATION ERRORS ARE CORRELATED IN TIME.

!**   INTERFACE
!     ---------

!     ***CALL COMTC

!        COMTC IS CALLED BY TASKOB

!**   EXTERNALS
!     ---------

!*    METHOD
!     ------

!**   MODIFICATIONS
!     -------------
!        Y.Tremolet    29-May-2008 Print Jo table by bins
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIMO  , ONLY : NACTIM
USE YOMCT0   , ONLY : NPROC    ,LOBSREF  ,LOBSC1
USE YOMLUN   , ONLY : NULOUT
USE YOMCOCTP , ONLY : NSYNOP   ,NDRIBU   ,MSQBYCTP
USE YOMCOSJO , ONLY : NVAR_Z   ,NVAR_PS  ,NITERQC  ,NTCOBS   ,NTCLOC   ,&
 & RTCEFT   ,RTCPART  ,LVARQC   ,LQVARQC  ,NTCOFF   ,&
 & NTCABN_L ,NTCOPT_L ,NTCDPT_L ,NTCOWN   ,&
 & NTCTIM   ,NTCVOI   ,RTCNDP   ,RTCNDPHR ,RTCAQC   ,&
 & RTCLQC   ,NTCOBS_L ,NTCIXG_L ,LVARQCG  ,&
 & JOT      ,FJOS, FJOADJ, LADJOBS, NPRTBINS
USE YOMVNMB  , ONLY : NVNUMB
USE YOMVAR   , ONLY : NITER    ,NITER4D  ,LGCVJO   ,NUPTRA
USE YOMMP    , ONLY : MYPROC
USE YOMANCS  , ONLY : NMDI     ,RMDI     ,RSQRT2PI
USE YOMMSC   , ONLY : N_DEFAULT_REAL_KIND, N_DOUBLE_KIND
USE YOMMKODB , ONLY : NTBMAR, NTFMAR
USE YOMDB

IMPLICIT NONE

#include "fcobs.h"

REAL(KIND=JPRB) :: ZCORM(NACTIM,NACTIM)
REAL(KIND=JPRB) :: ZDIAGSQ(NACTIM)
REAL(KIND=JPRB) :: ZTCNDP_W(NTCOBS),ZTCNDPHR_W(NTCOBS),ZOM1DEPHR_W(NTCOBS)
REAL(KIND=JPRB) :: ZFJOL(NTCLOC),ZQCWL(NTCLOC),ZDETO(NTCLOC),ZFJOLHR(NTCLOC)
REAL(KIND=JPRB) :: ZRIGHTM,ZLEFTTM

INTEGER(KIND=JPIM), PARAMETER :: JPVO=3
REAL(KIND=JPRB), TARGET  :: ZVO(NTCOBS,JPVO)
REAL(KIND=JPRB), POINTER :: ZOM1DEP(:)  ,ZQCW(:)  ,ZFJO(:)
REAL(KIND=JPRB), TARGET  :: ZVO_W(NTCOBS,JPVO)
REAL(KIND=JPRB), POINTER :: ZOM1DEP_W(:),ZQCW_W(:),ZFJO_W(:)

INTEGER(KIND=JPIM) :: IOWN_W(NTCOBS)
INTEGER(KIND=JPIM) :: INOBS(NTCLOC)

REAL(KIND=JPRB) :: ZCORTC,T1,T2,ZZ &
 & ,ZAQC,ZLQC,ZPNGE,ZPGE,ZNORM,ZAOB,ZEMJON &
 & ,ZFJOS,ZOM1DEPQC(1),ZFJON,ZFJOQC,ZJO_OFFSET,ZTIME
INTEGER(KIND=JPIM) :: JSTLOC,JC_L,J,J1,J2,JI,JJ &
 & ,IPROC,IOFF,IN,IVOI,IOBTYP,INFO,INFOARR(1) &
 & ,ICOUNT(1),IMXLEN,IMXNVO,ISOBTY,IDIM,IXG,IABNOB &
 & ,IOBS,IBODY(1,1),IVNM,IVAR,IRET,ICDTYP,ICTPSQ &
 & ,IOBDATE, IOBTIME, ITMO100, indxtime
LOGICAL :: LLVARQC,LLITERQC,LLQVARQC_MIN
REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!*
!     ------------------------------------------------------------------

!*       0.    START-UP
!              --------

!*          0.1   CORRELATION FUNCTIONS

ZCORTC(T1,T2)=EXP(-(T1-T2)*(T1-T2)/(RTCEFT*RTCEFT))

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "gathertc.intfb.h"
#include "setqccma.intfb.h"
#include "timdif.intfb.h"

#include "openmp_obs.h"

!*          0.2   IF NO TIME CORRELATED DATA, DO NOTHING
IF (LHOOK) CALL DR_HOOK('COMTC',0,ZHOOK_HANDLE)
IF(NTCOBS == 0 .AND. LHOOK) CALL DR_HOOK('COMTC',1,ZHOOK_HANDLE)
IF(NTCOBS == 0) RETURN

CALL GETDB('COMTC', NUPTRA, IRET, INFOARR, 0, ZINFO, 0,&
 & -1, -1, -1, -1, -1, -1)  

!IF (IRET == 0) RETURN

WRITE(NULOUT,'('' *** COMTC ARRIVAL '',3I12)') NTCOBS,NTCLOC,IRET

!*          0.3   INITIALIZE LOCAL ARRAYS

LLQVARQC_MIN=LQVARQC .AND. .NOT.LOBSC1

ZVO(:,:)    = RMDI
ZVO_W(:,:)  = RMDI
ZTCNDP_W(:) = RMDI
IF(LLQVARQC_MIN) ZTCNDPHR_W(:) = RMDI
IOWN_W(:)   = NMDI

!*          0.4   ESTABLISH POINTERS

ZOM1DEP   => ZVO(:,1)
ZQCW      => ZVO(:,2)
ZFJO      => ZVO(:,3)
ZOM1DEP_W => ZVO_W(:,1)
ZQCW_W    => ZVO_W(:,2)
ZFJO_W    => ZVO_W(:,3)

!     ------------------------------------------------------------------

!        1.       PREPARATIONS
!                 ------------

!*          1.1   WHICH PROCESSOR TO DO THE WORK?
!*                THIS LOOP VECTORIZES ON FUJITSU.

DO JSTLOC=1,NTCLOC
  IPROC=MOD(JSTLOC-1,NPROC)+1

!*          1.2   FIND OUT OFFSET AND NUMBER OF CORRELATED DATA (IN)

  IOFF=NTCOFF(JSTLOC)
  IF(JSTLOC /= NTCLOC) THEN
    IN=NTCOFF(JSTLOC+1)-IOFF
  ELSE
    IN=NTCOBS-IOFF+1
  ENDIF

!*          1.3   REMEMBER WHO IS GOING TO DO THE WORK

  INOBS(JSTLOC)=IN
  IOWN_W(IOFF:IOFF+IN-1)=IPROC
ENDDO
IF(MAXVAL(INOBS(:)) > NACTIM)CALL ABOR1('COMTC: ERROR, MAX(INOBS) > NACTIM')

!*          1.4   GET THE SECTION OF RTCNDP NEEDED BY THIS PE

CALL GATHERTC(NTCOBS,1,RTCNDP,NTCOWN,ZTCNDP_W,IOWN_W)
IF(LLQVARQC_MIN)&
 & CALL GATHERTC(NTCOBS,1,RTCNDPHR,NTCOWN,ZTCNDPHR_W,IOWN_W)  

!*
!     ------------------------------------------------------------------

!*       2.    COMPUTE TIME CORRELATED JO AND VARQC
!              ------------------------------------

LLITERQC=NITER4D >= NITERQC .AND. NITERQC <= NITER &
 & .AND. .NOT.LOBSREF .AND. LVARQCG  

!*          2.1   LOOP THE NUMBER OF STATION LOCATIONS

JO_LOOP: DO JSTLOC=1,NTCLOC
  IOFF=NTCOFF(JSTLOC)
  IPROC=IOWN_W(IOFF)
  IF(IPROC /= MYPROC) CYCLE JO_LOOP
  IN=INOBS(JSTLOC)

!*          2.3   OBSERVATION TYPE AND VARQC SWITCH

  IVOI   = NTCVOI(IOFF)
  IOBTYP = MOD(IVOI/1000,10)
  IF(IOBTYP /= NSYNOP .AND. IOBTYP /= NDRIBU) THEN
    CALL ABOR1('COMTC: ERROR, IOBTYP ne NSYNOP,NDRIBU')
  ENDIF

!*          2.4   FILL CORRELATION MATRIX

  IF(RTCPART /= 0.0_JPRB) THEN

    DO J1=2,IN
      DO J2=1,J1-1
        ZCORM(J1,J2)=RTCPART &
         & *ZCORTC(NTCTIM(IOFF+J1-1)*60._JPRB,NTCTIM(IOFF+J2-1)*60._JPRB)  
        ZCORM(J2,J1)=ZCORM(J1,J2)
      ENDDO
    ENDDO
    DO J=1,IN
      ZCORM(J,J)=1.0_JPRB
    ENDDO
  ENDIF

!*          2.5   PROJECT NORM DEP ONTO O-1. RESULT IN ZOM1DEP_W.
!*                FIND DETERMINANT

  ZOM1DEP_W(IOFF:IOFF+IN-1)=ZTCNDP_W(IOFF:IOFF+IN-1)
  IF(LLQVARQC_MIN) &
   & ZOM1DEPHR_W(IOFF:IOFF+IN-1)=ZTCNDPHR_W(IOFF:IOFF+IN-1)  
  IF(RTCPART == 0.0_JPRB) THEN
    ZDETO(JSTLOC)=1.0_JPRB
  ELSE
    IF (JPRB == N_DEFAULT_REAL_KIND) THEN
      CALL SPOTRF ('L',IN,ZCORM,NACTIM,INFO)
    ELSEIF (JPRB == N_DOUBLE_KIND) THEN
      CALL DPOTRF ('L',IN,ZCORM,NACTIM,INFO)
    ELSE
      CALL ABOR1 ('REAL(KIND=JPRB) is neither default real nor double precision')
    ENDIF

    IF(INFO /= 0) CALL ABOR1('COMTC: CHOLESKI ERROR')

!--- mulitply by O^(1/2) so we end up with O^(-1/2) after SPOTRF/DPOTRF

    IF (LGCVJO) THEN
      DO JI=IN,1,-1
        ZZ=0.0_JPRB
        DO JJ=1,JI
          ZZ = ZZ + ZCORM(JI,JJ)*ZOM1DEP_W(IOFF-1+JJ)
        ENDDO
        ZOM1DEP_W(IOFF-1+JI) = ZZ
      ENDDO
    ENDIF

    IF (JPRB == N_DEFAULT_REAL_KIND) THEN
      CALL SPOTRS('L',IN,1,ZCORM,NACTIM,ZOM1DEP_W(IOFF),NACTIM,INFO)
      IF(LLQVARQC_MIN) CALL SPOTRS('L',IN,1,ZCORM,NACTIM,&
       & ZOM1DEPHR_W(IOFF),NACTIM,INFO)  
    ELSEIF (JPRB == N_DOUBLE_KIND) THEN
      CALL DPOTRS('L',IN,1,ZCORM,NACTIM,ZOM1DEP_W(IOFF),NACTIM,INFO)
      IF(LLQVARQC_MIN) CALL DPOTRS('L',IN,1,ZCORM,NACTIM,&
       & ZOM1DEPHR_W(IOFF),NACTIM,INFO)  
    ELSE
      CALL ABOR1 ('REAL(KIND=JPRB) is neither default real nor double precision')
    ENDIF

    IF(INFO /= 0) CALL ABOR1('COMTC: INVERSION ERROR')
    DO J=1,IN
      ZDIAGSQ(J)=ZCORM(J,J)*ZCORM(J,J)
    ENDDO
    ZDETO(JSTLOC)=PRODUCT(ZDIAGSQ(1:IN))
  ENDIF

!*          2.6   JO WITHOUT VARQC

  ZFJOL(JSTLOC)=0.5_JPRB*SUM(ZTCNDP_W(IOFF:IOFF+IN-1)*ZOM1DEP_W(IOFF:IOFF+IN-1))
  IF(LLQVARQC_MIN) ZFJOLHR(JSTLOC)=&
   & 0.5_JPRB*SUM(ZTCNDPHR_W(IOFF:IOFF+IN-1)*ZOM1DEPHR_W(IOFF:IOFF+IN-1))  
ENDDO JO_LOOP

!*          2.7   MODIFY JO WITH VARQC. COMPUTE QC-WEIGHT
!*                THIS IS ASSUMING THAT ALL 'IN' OBS ARE AFFECTED
!*                BY THE SAME GROSS ERROR. ONLY ONE QC-WEIGHT PER
!*                GROUP OF CORRELATED DATA IS COMPUTED.
!*                THIS LOOP VECTORIZES ON FUJITSU.

QC_LOOP: DO JSTLOC=1,NTCLOC
  IOFF=NTCOFF(JSTLOC)
  IPROC=IOWN_W(IOFF)
  IF(IPROC /= MYPROC) CYCLE QC_LOOP

  IVOI   = NTCVOI(IOFF)
  IOBTYP = MOD(IVOI/1000,10)
  LLVARQC=LLITERQC .AND. LVARQC(NVAR_PS,IOBTYP)
  IF(LLVARQC) THEN

    IN=INOBS(JSTLOC)
    ZAQC=RTCAQC(JSTLOC)
    ZLQC=RTCLQC(JSTLOC)
    ZPNGE=(1.0_JPRB-ZAQC)**IN
    ZPGE=1.0_JPRB-ZPNGE
    ZNORM=SQRT(ZDETO(JSTLOC))*(RSQRT2PI/(2.0_JPRB*ZLQC))**IN
    ZAOB=ZPGE*ZNORM/ZPNGE

    IF(LLQVARQC_MIN) THEN
      ZFJON=ZFJOLHR(JSTLOC)     !Constant during inner-loop iterations
    ELSE
      ZFJON=ZFJOL(JSTLOC)
    ENDIF

    ZEMJON=EXP(-0.5_JPRB*ZFJON)
    ZFJOQC=-2.0_JPRB*LOG( (ZAOB + ZEMJON)/(ZAOB + 1.0_JPRB) )
    ZQCWL(JSTLOC)=1.0_JPRB-ZAOB/(ZAOB+ZEMJON)

    IF(LLQVARQC_MIN) THEN
      ZJO_OFFSET=ZFJOQC-ZFJON*ZQCWL(JSTLOC)
      ZFJOQC=ZFJOL(JSTLOC)*ZQCWL(JSTLOC)+ZJO_OFFSET
    ENDIF
    ZFJOL(JSTLOC)=ZFJOQC

  ELSE
    ZQCWL(JSTLOC)=1.0_JPRB
  ENDIF
ENDDO QC_LOOP

CP_LOOP: DO JSTLOC=1,NTCLOC
  IOFF=NTCOFF(JSTLOC)
  IPROC=IOWN_W(IOFF)
  IF(IPROC /= MYPROC) CYCLE CP_LOOP
  IN=INOBS(JSTLOC)
  ZFJO_W(IOFF:IOFF+IN-1)=ZFJOL(JSTLOC)/REAL(IN,JPRB)
  ZQCW_W(IOFF:IOFF+IN-1)=ZQCWL(JSTLOC)
ENDDO CP_LOOP

!     ------------------------------------------------------------------

!        3.       BUILD OUTPUT-VIEW OF THE RESULT ARRAYS
!                 --------------------------------------

CALL GATHERTC(NTCOBS,JPVO,ZVO_W,IOWN_W,ZVO,NTCOWN)

!*       4.       STORE RESULTS INTO CMA TO BE RE-USED IN THE ADJOINT
!*                ONLY DO THE ONES THAT BELONG ON THIS PROCESSOR

ZLEFTTM = REAL(NTBMAR,JPRB)
ZRIGHTM = REAL(NTFMAR,JPRB)

ICOUNT=1
IMXLEN=1
IMXNVO=1
ISOBTY=1
IDIM=1
STORE_LOOP: DO JC_L=1,NTCOBS_L
  IXG=NTCIXG_L(JC_L)

!*          4.2   JO, PER STATION

  ZFJOS=ZFJO(IXG)

!*          4.3   STORE JO IN JO-TABLES, ONLY ON OWN PE

  IOBS   = NTCOPT_L(JC_L)
  IOBTYP = ROBHDR(IOBS,MDBOTP)
  ICDTYP = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  ICTPSQ = MSQBYCTP(ICDTYP)
  IBODY(1,1)  = NTCDPT_L(JC_L)
  IVNM   = ROBODY(IBODY(1,1),MDBVNM)

  IF(IVNM == NVNUMB(3)) THEN
    IVAR=NVAR_Z
  ELSE
    IVAR=NVAR_PS
  ENDIF

  ZTIME = 0.0_JPRB
  IOBDATE= ROBHDR(IOBS,MDBDAT)
  IOBTIME= ROBHDR(IOBS,MDBETM)
  ITMO100 = IOBTIME / 100
  CALL TIMDIF(IOBDATE,ITMO100,ZTIME)
  ZTIME=((ZTIME-ZLEFTTM)/(ZRIGHTM-ZLEFTTM)) * REAL(NPRTBINS,JPRB)
  indxtime=MIN(NPRTBINS,INT(ZTIME)+1)

  JOT(IOBTYP)%jost(ICTPSQ)%cost(IVAR,indxtime)= &
   & JOT(IOBTYP)%jost(ICTPSQ)%cost(IVAR,indxtime) + ZFJOS  
  JOT(IOBTYP)%jost(ICTPSQ)%count(IVAR,indxtime)= &
   & JOT(IOBTYP)%jost(ICTPSQ)%count(IVAR,indxtime) + 1
  JOT(IOBTYP)%jost(ICTPSQ)%obserr(IVAR)= &
   & JOT(IOBTYP)%jost(ICTPSQ)%obserr(IVAR) + ROBODY(IBODY(1,1),MDBFOE)**2  

  IF(JOT(IOBTYP)%jost(ICTPSQ)%bgerr(IVAR) == RMDI) THEN
!   Do Nothing
  ELSEIF(ROBODY(IBODY(1,1),MDBFGE) == RMDI) THEN
    JOT(IOBTYP)%jost(ICTPSQ)%bgerr(IVAR) = RMDI
  ELSE
    JOT(IOBTYP)%jost(ICTPSQ)%bgerr(IVAR)= &
     & JOT(IOBTYP)%jost(ICTPSQ)%bgerr(IVAR) + ROBODY(IBODY(1,1),MDBFGE)**2  
  ENDIF

  IF (LADJOBS) THEN
    CALL ABOR1('COMTC: No impact')
    JOT(IOBTYP)%jost(ICTPSQ)%obimpact(IVAR,indxtime)= &
     & JOT(IOBTYP)%jost(ICTPSQ)%obimpact(IVAR,indxtime) ! + ?????  
  ENDIF

  IABNOB = NTCABN_L(JC_L)
  FJOS(IABNOB)=FJOS(IABNOB)+ZFJOS
  IF (LADJOBS) THEN
    CALL ABOR1('COMTC: No adjoint test')
    FJOADJ(IABNOB)=FJOADJ(IABNOB) ! + ?????
  ENDIF

!*          4.4   STORE IN CMA, ONLY ON OWN PE

  ZOM1DEPQC(1)=ZOM1DEP(IXG)*ZQCW(IXG)
  LLVARQC=LLITERQC .AND. LVARQC(IVAR,IOBTYP)
  CALL SETQCCMA(IMXLEN,IMXNVO,IDIM,LLVARQC,&
   & 1,ICOUNT,IBODY,&
   & ZOM1DEPQC,ZQCW(IXG:IXG))  

!*          4.6   END LOOP OVER STATIONS

ENDDO STORE_LOOP

!*          5.2   RETURN

CALL PUTDB('COMTC', NUPTRA, IRET, INFOARR, 0, ZINFO, 0)

IF (LHOOK) CALL DR_HOOK('COMTC',1,ZHOOK_HANDLE)
END SUBROUTINE COMTC
