SUBROUTINE DECIS(KTSLOT)

!**** SUBROUTINE DECIS - CONTROLS IFS OBS. SCREENING DECISIONS

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        CONTROLS IFS OBS. SCREENING DECISIONS

!**   INTERFACE
!     ---------

!     ***CALL* *DECIS(KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!        DECIS IS CALLED BY SUBROUTINE SCREEN

!*    METHOD
!     ------

!        CONTROLS IFS OBSERVATION SCREENING DECISION MAKING.
!     OBSERVATIONS ARE HANDLED IN SETS AS DEFINED IN ECSET.

!*    EXTERNALS
!     ---------

!        GETDB- GET DB DATA
!        PUTDB- PUT DB DATA
!        PRECH - PRELIMINARY CHECKS: RDB FLAGS, REPORTING PRACTICE ETC.
!        BLACK - BLACKLISTING
!        FIRST - FIRST GUESS CHECK
!        VERCO - VERTICAL CONSISTENCY OF MULTILEVEL REPORTS
!        MISCE - A MISCELLANEOUS COLLECTION OF INDEPENDENT CHECKS
!        GLOBA - MAKE GLOBAL TIME-LOCATION ARRAYS FOR SELECTED PLATFORMS
!        REDUN - ALL REDUNDANCY CHECKS
!        THINN - THINNING OF DATA

!**   MODIFICATIONS
!     -------------
!    08/07/1999 : D. Puech : pas d'appel scaqc pour LMFSCREEN
!                            balcktovs -> blacksat
!    08/09/1999 : P. Moll  : appel conditionnel de blackcln (cleaning
!                            a haute resolution) 
!    26/06/2004 : C. Payan : appel de SCAQC pour QSCAT pour LMFSCREEN
!    06/11/2007 : S. Saarinen : V"ahensin Dr.Hook-kutsuja muutamalla
!    27/04/2007 : C. Payan : LMFSCREEN dependance for SCAT removed
!    14/01/2009 : C. Payan : old thinning pruning (>=CY35T2)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMOBSET_THSAFE
USE YOMOBS
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMCOCTP , ONLY : NSCATT   ,NAIREP
USE OML_MOD, ONLY : OML_MY_THREAD

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KTSLOT

INTEGER(KIND=JPIM) :: INFO(1), IRET, ITID
INTEGER(KIND=JPIM) :: JSET, ILEN, ISET

LOGICAL :: LLDUPLI, LLSCAQC, LLREDUN, LLTHINN

REAL(KIND=JPRB) :: ZINFO(1)

CHARACTER(LEN=3) :: CLH

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZHOOK_ECSET_LOOP123
!REAL(KIND=JPRB) :: ZHOOK_BARR
!REAL(KIND=JPRB) :: ZHOOK_BARRTS

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "blackhat.intfb.h"
#include "dupli.intfb.h"
#include "dupli_no_sq.intfb.h"
#include "first.intfb.h"
#include "flgtst.intfb.h"
#include "misce.intfb.h"
#include "new_thinn.intfb.h"
#include "prech.intfb.h"
#include "redun.intfb.h"
#include "scaqc.intfb.h"
#include "verco.intfb.h"

!*
!     ------------------------------------------------------------------

!*       1.       START
!                 -----

IF (LHOOK) CALL DR_HOOK('DECIS',0,ZHOOK_HANDLE)
ITID = OML_MY_THREAD()
WRITE(CLH,'(i3.3)') KTSLOT

WRITE(NULOUT,'('' *** DECIS '',I10,'' SETS'')') OBSET(ITID)%NSETOT
LLDUPLI = .TRUE.
LLDUPLI = .TRUE.
LLSCAQC = .TRUE.
LLREDUN = .TRUE.
LLTHINN = .TRUE.
!*
!     ------------------------------------------------------------------

!*       2.       MAKE THE INDEPENDENT DECISIONS FIRST
!                 ------------------------------------

IF (OBSET(ITID)%NSETOT == 0) THEN
  WRITE(NULOUT,'(''     NO ECSETS AVAILABLE '')')
  WRITE(NULOUT,'(''     FOR TIME SLOT: '',I10)') KTSLOT
ENDIF

!           2.1   PRELIMINARY CHECKS

WRITE(NULOUT,'('' *** DECIS_1'')')
IF (LHOOK) CALL DR_HOOK('DECIS>ECSET_LOOP123',0,ZHOOK_ECSET_LOOP123)
ECSET_LOOP123: DO JSET = 1, OBSET(ITID)%NSETOT
  ISET = JSET

!           2.1.0 INITIALIZE DB (LOCAL; ALL REPs)

  IRET = 0
  CALL GETDB('DECIS',1,IRET,INFO,0,ZINFO,0,&
   & ISET, KTSLOT, -1, -1, -1, -1)  
  ILEN = IRET
  IF(ILEN /= OBSET(ITID)%MLNSET(JSET) ) THEN
    WRITE(NULERR,'('' *** ERROR IN DECIS(_1) '')')
    WRITE(NULERR,'(''     SLOT: '',I10)') KTSLOT
    WRITE(NULERR,'(''     SET : '',I10)') ISET
    WRITE(NULERR,'(''     ILEN: '',I10)') ILEN
    WRITE(NULERR,'(''     MLNSET'',I10)') OBSET(ITID)%MLNSET(JSET)
    CALL ABOR1(' DECIS(_1) : ILEN AND MLNSET NOT EQUAL ')
  ENDIF

!           2.1.1 RDB FLAGS, REPORTING PRACTICE ETC.

  CALL PRECH(ISET)

!           2.1.2 PUT THE DB AWAY

  CALL PUTDB('DECIS',1,IRET,INFO,0,ZINFO,0)

!           2.3   BLACKLIST

!           2.3.1 BLACKLISTING

  CALL BLACKHAT(KTSLOT,ISET)

!           2.5   ALL OTHER CHECKS

!           2.5.0 INITIALIZE DB (LOCAL; ALL REPs)

  IRET = 0
  CALL GETDB('DECIS',2,IRET,INFO,0,ZINFO,0,&
   & ISET, KTSLOT, -1, -1, -1, -1)  
  ILEN = IRET
  IF(ILEN /= OBSET(ITID)%MLNSET(JSET) ) THEN
    WRITE(NULERR,'('' *** ERROR IN DECIS(_2) '')')
    WRITE(NULERR,'(''     SLOT: '',I10)') KTSLOT
    WRITE(NULERR,'(''     SET : '',I10)') ISET
    WRITE(NULERR,'(''     ILEN: '',I10)') ILEN
    WRITE(NULERR,'(''     MLNSET'',I10)') OBSET(ITID)%MLNSET(JSET)
    CALL ABOR1(' DECIS(_2) : ILEN AND MLNSET NOT EQUAL ')
  ENDIF
!           2.5.1 FIRST GUESS CHECK

  CALL FIRST(ISET)

!           2.5.2 VERTICAL CONSISTENCY OF MULTILEVEL REPORTS

  CALL VERCO(ISET)

!           2.5.3 REMAINING INDEPENDENT DECISIONS

  CALL MISCE(ISET)

!*          2.5.4 TRANSFORM THE FLAG INFORMATION INTO REPORT/DATUM STATUS

  CALL FLGTST(ISET)

!           2.5.5 PUT THE DB AWAY

  CALL PUTDB('DECIS',2,IRET,INFO,0,ZINFO,0)

!*          2.6   NEXT AVAILABLE SET

ENDDO ECSET_LOOP123
IF (LHOOK) CALL DR_HOOK('DECIS>ECSET_LOOP123',1,ZHOOK_ECSET_LOOP123)

!*
!     ------------------------------------------------------------------

!*       3.       REMOVE DUPLICATED AIREP REPORTS
!                 -------------------------------

WRITE(NULOUT,'('' *** DECIS_3'')')
CALL FLUSH(NULOUT)

!          3.0    "ABSORB" ALL LOAD IMBALANCE HERE !

!IF (LHOOK) CALL DR_HOOK('DECIS_BARRIER#'//clh,0,ZHOOK_BARRTS)
!IF (LHOOK) CALL DR_HOOK('DECIS_BARRIER',0,ZHOOK_BARR)
!CALL MPL_BARRIER(CDSTRING='DECIS_BARRIER')
!IF (LHOOK) CALL DR_HOOK('DECIS_BARRIER',1,ZHOOK_BARR)
!IF (LHOOK) CALL DR_HOOK('DECIS_BARRIER#'//clh,1,ZHOOK_BARRTS)

!           3.1   INITIALIZE DB (GLOBAL; ALL ACTIVE AIREPS)

IRET = 0
CALL GETDB('DECIS',3,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1,NAIREP, -1, -1)  
IF(IRET == 0) THEN
  WRITE(NULOUT,'(''     NO REQUIRED (AIREP) REPORTS AVAILABLE '')')
ELSE
  WRITE(NULOUT,'(''     REPORTS (AIREP) FOUND: '',I10)') IRET
ENDIF
ILEN = IRET

!*          3.2   REMOVE DUPLICATED AIREP REPORTS

IF(LLDUPLI) THEN
  IF(LAIREPS_DUP_NO_SQ) THEN
    CALL DUPLI_NO_SQ(ILEN)
  ELSE
    CALL DUPLI      (ILEN)
  ENDIF
ENDIF

!           3.3   PUT THE DB AWAY

CALL PUTDB('DECIS',3,IRET,INFO,0,ZINFO,0)

!*
!     ------------------------------------------------------------------

!*       4.       MAKE THE DECISIONS THAT DEPEND ALSO ON OTHER SETS
!                 -------------------------------------------------

!           4.1   ALL REDUNDANCY CHECKS, INCLUDING LEVEL SELECTION

IF(LLREDUN) THEN
  CALL REDUN(KTSLOT)
ENDIF

!           4.2   QUALITY CHECK OF THE SCATT DATA

WRITE(NULOUT,'('' *** DECIS_4'')')
CALL FLUSH(NULOUT)

!           4.2.1 INITIALIZE DB (LOCAL, ACTIVE SCAT, ALL SETS, THIS TIMESLOT)

IRET = 0
CALL GETDB('DECIS',4,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1,NSCATT, -1, -1)  

!           4.2.3 DATA FOUND (OR NOT) PERFORM QC (OTHERWISE AN MP-DEADLOCK)

ILEN = IRET
WRITE(NULOUT,'(''     REPORTS (SCAT) FOUND: '',I10)') IRET
CALL FLUSH(NULOUT)
IF(LLSCAQC) THEN
  CALL SCAQC(ILEN,'ERS')
  CALL SCAQC(ILEN,'NSCAT')
  CALL SCAQC(ILEN,'QSCAT')
  CALL SCAQC(ILEN,'ASCAT')
  CALL SCAQC(ILEN,'OSCAT')
ENDIF

!           4.2.4 PUT THE DB AWAY

CALL PUTDB('DECIS',4,IRET,INFO,0,ZINFO,0)

!           4.3   THINNING OF DATA

IF(LLTHINN) THEN
  CALL NEW_THINN(KTSLOT)
ENDIF
!*
!     ------------------------------------------------------------------

!        5.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('DECIS',1,ZHOOK_HANDLE)
END SUBROUTINE DECIS
