SUBROUTINE EWPRFIN(KPILP,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE EWPRFIN - PROCESS INPUT EUROPEAN PROFILERS

!        D. VASILJEVIC     ECMWF     13/12/00 (original 04/07/94)

!     PURPOSE
!     -------

!        PROCESS EUROPEAN PROFILERS

!**   INTERFACE
!     ---------

!        CALL EWPRFIN(KPILP,
!                     KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                     LDMESAGE,LDTAKEN)
!     WHERE:
!            KPILP   = REPORT POINTER
!            KOBTYP  = OBS. TYPE
!            KCDTYP  = CODE TYPE
!            KOBSQNO = OBS. TYPE SQ. NO.
!            KCDSQNO = CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT UPDATED) SWITCH  

!        EWPRFIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        ERRSTAT  - OBS. ERROR
!        PILOTBE  - UPDATE PILOT BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE
!        OBSCHAR  - OBS. CHARACTERISTICS

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB 
USE YOMCMBDY , ONLY :    &
 & NPSURBP  ,NPSUROC  ,NPSWLBP    
USE YOMANCS  , ONLY : NMDI     ,RMDI     ,RDEGREES ,RADIANS  ,EPSMAC   ,RUPMLIM  ,RDWMLIM  
USE YOMVNMB  , ONLY : NVNUMB
USE YOMMKODB , ONLY :  LERRPERS, NMKCMVSE ,LTOTREM
USE YOMNMCOD , ONLY :   NHEIGVC
USE YOMCST   , ONLY : RG       ,RTT
USE YOETHF   , ONLY : RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES  
USE YOMCOCTP , ONLY :   NJWPPCD  ,NWPPCD
USE YOMLUN   , ONLY : NULOUT
USE LOCAL_TRAFOS

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPILP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT) :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
LOGICAL :: LLDD, LLFF, LLU, LLV

!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE, &
 & IOBTIME, IVNM, JWIND, JLE, ISTART, IEND, ISIGW, &
 & ISTID, INSTYP, ICDTYPE, IWMO_BLOCK, INSTID, &
 & IREPST, INOEPLE, INOEPL  

INTEGER(KIND=JPIM) :: IDD, &
 & IDDSTAT, IDDEVEN1, IDDEVEN2, IDDRDBFL, IDDANFLG, &
 & IDDPFLG, IDDFLG, IDDVERCO, IDDLEVID, IDDSURF, IQCFLAGDD  
INTEGER(KIND=JPIM) :: IFF, &
 & IFFSTAT, IFFEVEN1, IFFEVEN2, IFFRDBFL, IFFANFLG, &
 & IFFPFLG, IFFFLG, IFFVERCO, IFFLEVID, IFFSURF, IQCFLAGFF  
INTEGER(KIND=JPIM) :: IU, &
 & IUUSTAT, IUUEVEN1, IUUEVEN2, IUURDBFL, IUUANFLG, &
 & IUUPFLG, IUUFLG, IUUVERCO, IUULEVID  
INTEGER(KIND=JPIM) :: IV, &
 & IVVSTAT, IVVEVEN1, IVVEVEN2, IVVRDBFL, IVVANFLG, &
 & IVVPFLG, IVVFLG, IVVVERCO, IVVLEVID  
INTEGER(KIND=JPIM) :: IAUX, JALL, &
 & IAUSTAT, IAUEVEN1, IAUEVEN2, IAURDBFL, IAUANFLG, &
 & IAUPFLG, IAUFLG  , IAUVERCO, IAULEVID  

LOGICAL :: LLFAIL, LLREMOVE

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, &
 & ZTIMDIF, &
 & ZWEIGHT, &
 & ZPRESAU, ZHEIGOG, ZALT  

REAL(KIND=JPRB) :: ZDDPRES, ZDDPRL, ZDIR, &
 & ZDDPERR, ZDDOERR, ZDDFERR, ZDDRERR, &
 & ZDDFGER, ZDDFGCC1, ZDDFGCC2, &
 & ZDDFGDEP, ZDDANDEP  
REAL(KIND=JPRB) :: ZFFPRES, ZFFPRL, ZSPD, &
 & ZFFPERR, ZFFOERR, ZFFFERR, ZFFRERR, &
 & ZFFFGER, ZFFFGCC1, ZFFFGCC2, &
 & ZFFFGDEP, ZFFANDEP  
REAL(KIND=JPRB) :: ZUUPRES, ZUUPRL, ZU, &
 & ZUUOERR, ZUUPERR, ZUUFERR, ZUURERR, &
 & ZUUFGER, ZUUFGCC1, ZUUFGCC2, &
 & ZUUFGDEP, ZUUANDEP  
REAL(KIND=JPRB) :: ZVVPRES, ZVVPRL, ZV, &
 & ZVVOERR, ZVVPERR, ZVVFERR, ZVVRERR, &
 & ZVVFGER, ZVVFGCC1, ZVVFGCC2, &
 & ZVVFGDEP, ZVVANDEP  
REAL(KIND=JPRB) :: ZAU, ZAUPRES, ZAUPRL, &
 & ZAUPERR, ZAUOERR, ZAUFERR, ZAURERR, &
 & ZAUFGER, ZAUFGCC1, ZAUFGCC2, &
 & ZAUFGDEP, ZAUANDEP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "errstat.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "pilotbe.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"
#include "z2picao.intfb.h"
#include "finoerr.intfb.h"

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET
!                       ------
IF (LHOOK) CALL DR_HOOK('EWPRFIN',0,ZHOOK_HANDLE)
ZLAT   = ROBHDR(KPILP,MDBLAT) * RDEGREES
ZLON   = ROBHDR(KPILP,MDBLON) * RDEGREES
ZALT   = ROBHDR(KPILP,MDBALT)
IOBDATE= ROBHDR(KPILP,MDBDAT)
IOBTIME= ROBHDR(KPILP,MDBETM)
ICDTYPE= ROBHDR(KPILP,MDB_CODETYPE_AT_HDR)
CLSTID = TRANSFER(ROBHDR(KPILP,MDBSID), CLSTID)
ISTID  = 0
READ(CLSTID(1:8),'(A8)') ISTID
IREPST = ROBHDR(KPILP,MDBRST)
INOEPLE= 4 

!*          1.1         CHECK CODE TYPE AND IF:
!                       1. wmo block 47 set it to japanese (131)
!                       2. wmo block 72 set it to american (34)

110 CONTINUE
READ(CLSTID,*,ERR=111,END=111) INSTID

IF(INSTID > 0.AND.INSTID <= 99999)   THEN
  IWMO_BLOCK = INSTID/1000
  IF(    IWMO_BLOCK == 47)  THEN
    ICDTYPE = NJWPPCD
  ELSEIF(IWMO_BLOCK == 72)  THEN
    ICDTYPE = NWPPCD
  ELSEIF(IWMO_BLOCK == 74)  THEN
    ICDTYPE = NWPPCD
  ELSEIF(IWMO_BLOCK == 70)  THEN
    ICDTYPE = NWPPCD
  ENDIF

  IF(ICDTYPE /= KCDTYP)       THEN
    ROBHDR(KPILP,MDB_CODETYPE_AT_HDR) = ICDTYPE
  ENDIF
ENDIF

111 CONTINUE

!LLFAIL   = .FALSE.
!LLREMOVE = .FALSE.
!CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
!           &LDMESAGE,LLFAIL,LLREMOVE)

!*          3.1.0       UPDATE REPORT STATUS

!ROBHDR(KPILP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

!IF(LLFAIL  ) THEN
!  IF(LTOTREM) GO TO 7
!ENDIF

!*          3.1.2       REMOVED REPORT

!IF(LLREMOVE) THEN
!  IF(LTOTREM) GO TO 7
!ENDIF

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KPILP,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KPILP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 7
ENDIF

!*          3.1.2       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 7
ENDIF

!*          3.1.3       CHECK ALTITUDE

IF(ZALT /= RMDI ) THEN
  IF(.NOT.(ZALT >= RDWMLIM.AND.&
   & ZALT <= RUPMLIM     )) GO TO 3131  
  ROBHDR(KPILP,MDBALT)  = RMDI
  ZALT                  = RMDI
ENDIF
3131 CONTINUE
IF(ZALT   == RMDI   )                  THEN
  WRITE(NULOUT,'(''0EWPRFIN; ALTITUDE MISSING;'',&
   & '' HENCE REPORT NOT CREATED (REJECTED)'')')  
  CALL FLUSH(NULOUT)
  ROBHDR(KPILP,MDBRST) = ZCHSTAT_REJECT(ROBHDR(KPILP,MDBRST))
  LDMESAGE = .TRUE.
  GO TO 6
ENDIF

!*          3.2         BODY PARAMETERS

!*          3.2.1       WE ACCEPT: DD, FF, U(10m), V(10m)

ALL_DATA_LOOP1: DO JALL = MLNKH2B(KPILP),MLNKH2B(KPILP+1)-1,INOEPLE
  INOEPL = 0
  LLDD   = .FALSE.
  LLFF   = .FALSE.
  LLU    = .FALSE.
  LLV    = .FALSE.
  321 CONTINUE
  IF(INOEPL == INOEPLE ) THEN
    IF(LLDD .AND. &
     & LLFF .AND. &
     & LLU  .AND. &
     & LLV         ) CYCLE ALL_DATA_LOOP1  
    WRITE(NULOUT,'('' EWPRFIN; NO. OF ENTRIES PER LEVEL EXCEEDED: '')')
    WRITE(NULOUT,'(''          LLDD, LLFF, LLU, LLV: '',4(L1,1X))') &
     & LLDD, LLFF, LLU, LLV  
    WRITE(NULOUT,'('' LAT, LON: '',2(F10.2,1X))') ZLAT,ZLON
    CALL ABOR1(' EWPRFIN : TOO MANY ENTRIES ')
  ELSE
    IF(     ROBODY(JALL+INOEPL,MDBVNM) == NVNUMB( 48)) THEN
      LLDD    = .TRUE.
      GO TO 3217
    ELSEIF(ROBODY(JALL+INOEPL,MDBVNM) == NVNUMB( 49)) THEN
      LLFF    = .TRUE.
      GO TO 3217
    ELSEIF(ROBODY(JALL+INOEPL,MDBVNM) == NVNUMB( 56) .OR. &
       & ROBODY(JALL+INOEPL,MDBVNM) == NVNUMB(  1)       ) THEN  
      LLU     = .TRUE.
      GO TO 3217
    ELSEIF(ROBODY(JALL+INOEPL,MDBVNM) == NVNUMB( 57) .OR. &
       & ROBODY(JALL+INOEPL,MDBVNM) == NVNUMB(  2)       ) THEN  
      LLV     = .TRUE.
      GO TO 3217
    ELSE
      IAUX    = JALL + INOEPL
      IVNM    = ROBODY(IAUX  ,MDBVNM)
      WRITE(NULOUT,'('' EWPRFIN; UNEXPECTED VARIABLE: '',I10)') IVNM
      LDMESAGE = .TRUE.

!*          3.2.1.0     SET/GET

      CALL GETSETE(IAUX ,&
       & ZAUPRES,ZAUPRL,ZAU ,&
       & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
       & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
       & ZAUFGDEP,ZAUANDEP,&
       & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
       & IAUFLG,IAUPFLG)  

      ZAUPRES = ROBODY(IAUX,MDBPPP)
      IAUVERCO= ROBODY(IAUX,MDBVCO)
      IAULEVID= ROBODY(IAUX,MDBPILID)

!*          3.2.1.2     VARIABLE

      ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.2.1.3     ERROR STATISTICS

!*          3.2.1.4     STATUS, EVENTS, FLAGS

      IAUSTAT = ICHSTAT_REJECT( IAUSTAT )

!*          3.2.1.5     DEPARTURES

!*          3.2.1.6     UPDATE BODY ENTRY

      CALL PILOTBE(IAUX    ,&
       & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
       & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
       & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
       & ZAUFGDEP,ZAUANDEP,&
       & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
       & IDDLEVID                                      )  
    ENDIF
  ENDIF

!*          3.2.1.7     GO CHECK IF LEVEL FINISHED:

  3217 CONTINUE
  INOEPL = INOEPL + 1
  GO TO 321
ENDDO ALL_DATA_LOOP1

!*          3.2.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

ALL_DATA_LOOP2: DO JALL = MLNKH2B(KPILP),MLNKH2B(KPILP+1)-1

!*          3.2.1.1     VERTICAL COORDINATE

  3211 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 3212  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF

!*          3.2.1.2     VARIABLE

  3212 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3213  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.2.1.4     ERROR STATISTICS

  3213 CONTINUE
  ROBODY(JALL  ,MDBOER) = RMDI
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.2.1.5     ASSOCIATED FIELD

  IF(ROBODY(JALL  ,MDB_AUX_AT_BODY(1)) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL  ,MDB_AUX_AT_BODY(1)) >= RDWMLIM.AND.&
     & ROBODY(JALL  ,MDB_AUX_AT_BODY(1)) <= RUPMLIM     )) GO TO 323  
    ROBODY(JALL  ,MDB_AUX_AT_BODY(1)) = RMDI
  ENDIF

!*          3.2.3       CLOSE ALL DATA LOOP2

  323 CONTINUE
ENDDO ALL_DATA_LOOP2

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (GEOPOTENTIAL)
!                       -------------------------------

4 CONTINUE

!*
!-----------------------------------------------------------------------
!        5.             OBSERVATION BODY (WIND)
!                       -----------------------

5 CONTINUE
ZWEIGHT = 1.0_JPRB

!*          5.1         LOOK FOR WIND DATA

WIND_DATA_LOOP: DO JWIND   = MLNKH2B(KPILP) , MLNKH2B(KPILP+1)-1 , INOEPLE

!*          5.1.1       PRESET

  ZDIR      = RMDI
  ZDDPRES   = RMDI
  IDDFLG    = 0
  IDDPFLG   = 0
  IDDVERCO  = NMDI
  IDDLEVID  = NMDI
  IQCFLAGDD = NMDI

  ZSPD      = RMDI
  ZFFPRES   = RMDI
  IFFFLG    = 0
  IFFPFLG   = 0
  IFFVERCO  = NMDI
  IFFLEVID  = NMDI
  IQCFLAGFF = NMDI

!*          5.1.2       GET POINTERS

  IDD     = NMDI
  IFF     = NMDI
  IU      = NMDI
  IV      = NMDI
  ISTART  = JWIND
  IEND    = ISTART + INOEPLE - 1
  DO JLE = ISTART , IEND
    IF(ROBODY(JLE,MDBVNM) == NVNUMB( 48))        IDD = JLE
    IF(ROBODY(JLE,MDBVNM) == NVNUMB( 49))        IFF = JLE
    IF(ROBODY(JLE,MDBVNM) == NVNUMB(  1) .OR. &
     & ROBODY(JLE,MDBVNM) == NVNUMB( 56)       ) IU  = JLE  
    IF(ROBODY(JLE,MDBVNM) == NVNUMB(  2) .OR. &
     & ROBODY(JLE,MDBVNM) == NVNUMB( 57)       ) IV  = JLE  
  ENDDO

!*          5.2         WIND DIRECTION

  IF(IDD == NMDI )    THEN
    WRITE(NULOUT,'('' EWPRFIN; DD ENTRY MISSING '')')
    WRITE(NULOUT,'('' LAT, LON: '',2(F10.2,1X))') ZLAT,ZLON
    CALL FLUSH(NULOUT)
    CALL ABOR1(' EWPRFIN : DD MISSING ')
  ENDIF
  IVNM    = NVNUMB( 48)

!*          5.2.0       SET/GET

  CALL GETSETE(IDD,&
   & ZDDPRES,ZDDPRL,ZDIR,&
   & ZDDFGER,ZDDFGCC1,ZDDFGCC2,&
   & ZDDPERR,ZDDOERR,ZDDFERR,ZDDRERR,&
   & ZDDFGDEP,ZDDANDEP,&
   & IDDSTAT,IDDEVEN1,IDDEVEN2,IDDRDBFL,IDDANFLG,&
   & IDDFLG,IDDPFLG)  

  ZDDPRES = ROBODY(IDD,MDBPPP  )
  IDDVERCO= ROBODY(IDD,MDBVCO  )
  IDDLEVID= ROBODY(IDD,MDBPILID)
  IDDSURF = IBITS(IDDLEVID,NPSURBP,NPSUROC)
  IQCFLAGDD = 1
  IF(ROBODY(IDD   ,MDB_AUX_AT_BODY(1)) /= RMDI ) THEN
    IQCFLAGDD = ROBODY(IDD   ,MDB_AUX_AT_BODY(1))
  ENDIF
  
  IF(ZDDPRES == RMDI .OR. IDDVERCO /= NHEIGVC) THEN
    IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
  ELSE
    ZDDPRES = (ZALT+ZDDPRES) * RG
  ENDIF
  
!*          5.2.2       VARIABLE

  IF(ROBODY(IDD  ,MDBVAR) /= RMDI ) THEN
    ZDIR    = ROBODY(IDD  ,MDBVAR  )
  ELSE
    IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
  ENDIF

!*          5.2.3       ERROR STATISTICS

!*          5.2.4       STATUS, EVENTS, FLAGS

  IF(NMKCMVSE( 48,KCDSQNO,KOBSQNO) == 0)  THEN
    IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
  ENDIF
  CALL PPVAFL(IDDPFLG,IDDFLG,IDDRDBFL)

!*          5.2.5       DEPARTURES

!*          5.2.6       UPDATE BODY ENTRY

  CALL PILOTBE(IDD     ,&
   & IVNM    ,IDDVERCO,ZDDPRES ,ZDDPRL  ,ZDIR    ,&
   & ZDDOERR ,ZDDPERR ,ZDDFERR ,ZDDRERR ,&
   & ZDDFGER ,ZDDFGCC1,ZDDFGCC2,&
   & ZDDFGDEP,ZDDANDEP,&
   & IDDSTAT ,IDDEVEN1,IDDEVEN2,IDDRDBFL,IDDANFLG,&
   & IDDLEVID                                      )  

!*          5.3         WIND SPEED (ASSUMPTION THAT IS NEXT)

  IF(IFF == NMDI )    THEN
    WRITE(NULOUT,'('' EWPRFIN; FF ENTRY MISSING '')')
    WRITE(NULOUT,'('' LAT, LON: '',2(F10.2,1X))') ZLAT,ZLON
    CALL ABOR1(' EWPRFIN : FF MISSING ')
  ENDIF
  IVNM    = NVNUMB( 49)

!*          5.3.0       SET/GET

  CALL GETSETE(IFF,&
   & ZFFPRES,ZFFPRL,ZSPD,&
   & ZFFFGER,ZFFFGCC1,ZFFFGCC2,&
   & ZFFPERR,ZFFOERR,ZFFFERR,ZFFRERR,&
   & ZFFFGDEP,ZFFANDEP,&
   & IFFSTAT,IFFEVEN1,IFFEVEN2,IFFRDBFL,IFFANFLG,&
   & IFFFLG,IFFPFLG)  

  ZFFPRES = ROBODY(IFF,MDBPPP)
  IFFVERCO= ROBODY(IFF  ,MDBVCO  )
  IFFLEVID= ROBODY(IFF  ,MDBPILID)
  IFFSURF = IBITS(IFFLEVID,NPSURBP,NPSUROC)
  IQCFLAGFF = 1
  IF(ROBODY(IFF   ,MDB_AUX_AT_BODY(1)) /= RMDI ) THEN
    IQCFLAGFF = ROBODY(IFF   ,MDB_AUX_AT_BODY(1))
  ENDIF

  IF(ZFFPRES == RMDI .OR. IFFVERCO /= NHEIGVC) THEN
    IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
  ELSE
    ZFFPRES = (ZALT+ZFFPRES) * RG
  ENDIF

!*          5.3.2       VARIABLE

  IF(ROBODY(IFF  ,MDBVAR) /= RMDI ) THEN
    ZSPD    = ROBODY(IFF  ,MDBVAR  )
  ELSE
    IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
  ENDIF

!*          5.3.3       ERROR STATISTICS

    IF(LERRPERS)           THEN
      ZFFPERR  = 1.0_JPRB ! Rough first estimate of persistence error
    ENDIF
    ZFFOERR  = 2.0_JPRB ! Rough first estimate of obs. error =2 m/s
    CALL FINOERR(ZFFPERR,ZFFOERR,ZFFFERR)

!*          5.3.4       STATUS, EVENTS, FLAGS

  IF(NMKCMVSE( 49,KCDSQNO,KOBSQNO) == 0)  THEN
    IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
  ENDIF
  CALL PPVAFL(IFFPFLG,IFFFLG,IFFRDBFL)

!*          5.3.5       DEPARTURES

!*          5.3.6       UPDATE BODY ENTRY

  CALL PILOTBE(IFF     ,&
   & IVNM    ,IFFVERCO,ZFFPRES ,ZFFPRL  ,ZSPD    ,&
   & ZFFOERR ,ZFFPERR ,ZFFFERR ,ZFFRERR ,&
   & ZFFFGER ,ZFFFGCC1,ZFFFGCC2,&
   & ZFFFGDEP,ZFFANDEP,&
   & IFFSTAT ,IFFEVEN1,IFFEVEN2,IFFRDBFL,IFFANFLG,&
   & IFFLEVID                                      )  

!*          5.4         CROSSCHECK DD AND FF

  IF(IFFVERCO /= IDDVERCO .OR. &
     & ZFFPRES  /= ZDDPRES  .OR. &
     & IQCFLAGDD/= IQCFLAGFF.OR. &
     & IFFSURF  /= IDDSURF        ) THEN  
    WRITE(NULOUT,'('' EWPRFIN; DD AND FF MIXUP '')')
    WRITE(NULOUT,'('' LAT, LON: '',2(F10.2,1X))') ZLAT,ZLON
    WRITE(NULOUT,'('' IFFVERCO , IDDVERCO :'',2(I10,1X))') IFFVERCO,IDDVERCO
    WRITE(NULOUT,'('' IFFSURF  , IDDSURF  :'',2(I10,1X))') IFFSURF,IDDSURF
    WRITE(NULOUT,'('' IQCFLAGDD, IQCFLAGFF:'',2(I10,1X))') IQCFLAGDD,IQCFLAGFF
    WRITE(NULOUT,'('' ZFFPRES  , ZDDPRES  :'',2(F10.2,1X))') ZFFPRES,ZDDPRES
    CALL FLUSH(NULOUT)
    CALL ABOR1(' EWPRFIN : DD/FF MIXUP ')
  ENDIF

!*          5.5         U COMPONENT

  IF(IU  == NMDI )    THEN
    WRITE(NULOUT,'('' EWPRFIN; U ENTRY MISSING '')')
    WRITE(NULOUT,'('' LAT, LON: '',2(F10.2,1X))') ZLAT,ZLON
    CALL ABOR1(' EWPRFIN : U MISSING ')
  ENDIF

  IVNM    = NVNUMB(   1)
  IF(IDDSURF == 1 ) IVNM    = NVNUMB(  56)

!*          5.5.0       SET/GET

  CALL GETSETE(IU ,&
   & ZUUPRES,ZUUPRL,ZU  ,&
   & ZUUFGER,ZUUFGCC1,ZUUFGCC2,&
   & ZUUPERR,ZUUOERR,ZUUFERR,ZUURERR,&
   & ZUUFGDEP,ZUUANDEP,&
   & IUUSTAT,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
   & IUUFLG,IUUPFLG)  

  ZUUPRES  = ZDDPRES
  IUUPFLG  = IDDPFLG
  IUULEVID = IDDLEVID
  IF(IUULEVID == 0) THEN
    ISIGW    = 1
    IUULEVID = INSERT(IUULEVID,ISIGW,NPSWLBP)
  ENDIF
  IUUVERCO = IDDVERCO
   
!*          5.5.1       VERTICAL COORDINATE

  IF(ZUUPRES  == RMDI    .OR. &
     & IUULEVID == NMDI    .OR. &
     & IUUVERCO /= NHEIGVC .OR. &
     & IUUVERCO == NMDI          ) THEN  
    IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
  ENDIF

!*          5.5.2       VARIABLE

  IF(ZDIR /= RMDI.AND.ZSPD /= RMDI .AND. IQCFLAGDD == 0)        THEN
! IF(ZDIR /= RMDI.AND.ZSPD /= RMDI                     )        THEN
    ZU      = UCOM(ZDIR,ZSPD)
    IUUFLG  = MAX(IDDFLG,IFFFLG)
  ELSE
    IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
  ENDIF

!*          5.5.3       ERROR STATISTICS

  IF(ZUUPRES /= RMDI .AND. IUUVERCO == NHEIGVC) THEN
    ZHEIGOG = ZUUPRES / RG
    CALL Z2PICAO(ZHEIGOG,ZPRESAU)
    ZPRESAU = ZPRESAU * 100._JPRB
    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IUUSTAT,ZPRESAU,ZUUPRL,&
     & ZUUPERR,ZUUOERR,ZUUFERR,ZWEIGHT)  
  ENDIF

!*          5.5.4       STATUS

  IF(NMKCMVSE(  1,KCDSQNO,KOBSQNO) == 0)          THEN
    IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
  ENDIF
  CALL PPVAFL(IUUPFLG,IUUFLG,IUURDBFL)

!*          5.5.5       DEPARTURES

!*          5.5.6       UPDATE/CREATE BODY ENTRY

  CALL PILOTBE(IU      ,&
   & IVNM    ,IUUVERCO,ZUUPRES ,ZUUPRL  ,ZU      ,&
   & ZUUOERR ,ZUUPERR ,ZUUFERR ,ZUURERR ,&
   & ZUUFGER ,ZUUFGCC1,ZUUFGCC2,&
   & ZUUFGDEP,ZUUANDEP,&
   & IUUSTAT ,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
   & IUULEVID                                      )  

!*          5.6         V COMPONENT

  IF(IV  == NMDI )    THEN
    WRITE(NULOUT,'('' EWPRFIN; V ENTRY MISSING '')')
    WRITE(NULOUT,'('' LAT, LON: '',2(F10.2,1X))') ZLAT,ZLON
    CALL ABOR1(' EWPRFIN : V MISSING ')
  ENDIF

  IVNM    = NVNUMB(   2)
  IF(IDDSURF == 1 ) IVNM    = NVNUMB(  57)

!*          5.5.0       SET/GET

  CALL GETSETE(IV ,&
   & ZVVPRES,ZVVPRL,ZV  ,&
   & ZVVFGER,ZVVFGCC1,ZVVFGCC2,&
   & ZVVPERR,ZVVOERR,ZVVFERR,ZVVRERR,&
   & ZVVFGDEP,ZVVANDEP,&
   & IVVSTAT,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
   & IVVFLG,IVVPFLG)  

  ZVVPRES  = ZDDPRES
  IVVPFLG  = IDDPFLG
  IVVLEVID = IDDLEVID
  IF(IVVLEVID == 0) THEN
    ISIGW    = 1
    IVVLEVID = INSERT(IVVLEVID,ISIGW,NPSWLBP)
  ENDIF
  IVVVERCO = IDDVERCO
   
!*          5.5.1       VERTICAL COORDINATE

  IF(ZVVPRES  == RMDI    .OR. &
     & IVVLEVID == NMDI    .OR. &
     & IVVVERCO /= NHEIGVC .OR. &
     & IVVVERCO == NMDI          ) THEN  
    IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
  ENDIF

!*          5.5.2       VARIABLE

  IF(ZDIR /= RMDI.AND.ZSPD /= RMDI .AND. IQCFLAGFF == 0)        THEN
! IF(ZDIR /= RMDI.AND.ZSPD /= RMDI                     )        THEN
    ZV      = VCOM(ZDIR,ZSPD)
    IVVFLG  = MAX(IDDFLG,IFFFLG)
  ELSE
    IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
  ENDIF

!*          5.5.3       ERROR STATISTICS

  IF(ZVVPRES /= RMDI .AND. IVVVERCO == NHEIGVC) THEN
    ZHEIGOG = ZVVPRES / RG
    CALL Z2PICAO(ZHEIGOG,ZPRESAU)
    ZPRESAU = ZPRESAU * 100._JPRB
    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IVVSTAT,ZPRESAU,ZVVPRL,&
     & ZVVPERR,ZVVOERR,ZVVFERR,ZWEIGHT)  
  ENDIF

!*          5.5.4       STATUS

  IF(NMKCMVSE(  2,KCDSQNO,KOBSQNO) == 0)          THEN
    IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
  ENDIF
  CALL PPVAFL(IVVPFLG,IVVFLG,IVVRDBFL)

!*          5.5.5       DEPARTURES

!*          5.5.6       UPDATE/CREATE BODY ENTRY

  CALL PILOTBE(IV      ,&
   & IVNM    ,IVVVERCO,ZVVPRES ,ZVVPRL  ,ZV      ,&
   & ZVVOERR ,ZVVPERR ,ZVVFERR ,ZVVRERR ,&
   & ZVVFGER ,ZVVFGCC1,ZVVFGCC2,&
   & ZVVFGDEP,ZVVANDEP,&
   & IVVSTAT ,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
   & IVVLEVID                                      )  
ENDDO WIND_DATA_LOOP

!*
!-----------------------------------------------------------------------

!        6.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

6 CONTINUE

!*          6.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          6.2         ODB REPORT UPDATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        7.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

7 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        8.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('EWPRFIN',1,ZHOOK_HANDLE)
END SUBROUTINE EWPRFIN
