SUBROUTINE FINDL(PCMPPP,KSTDLE,PDIFF)

!**** SUBROUTINE FINDL - FIND CLOSEST STANDARD (MANDATORY) PRESSURE LEVEL

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995

!*    PURPOSE
!     -------

!        FIND CLOSEST STANDARD (MANDATORY) PRESSURE LEVEL AND THE DISTANCE
!     TO IT.

!**   INTERFACE
!     ---------

!     ***CALL FINDL(PCMPPP,KSTDLE,PDIFF)
!     WHERE PCMPPP = GIVEN PRESSURE OF OBSERVATION (IN PASCALS)
!           KSTDLE = INDEX OF CLOSEST STANDARD PRESSURE LEVEL
!           PDIFF  = DISTANCE TO THIS STD- LEVEL (IN PASCALS)

!        FINDL IS CALLED BY SUBROUTINE LEVEL

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLEVSE , ONLY : NSTDPL   ,RSTDPL

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PCMPPP 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KSTDLE 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFF 
INTEGER(KIND=JPIM) :: J

REAL(KIND=JPRB) :: ZDIFF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------
!*
!     ------------------------------------------------------------------

!        2.       FIND CLOSEST STD-PRESSURE LEVEL AND DISTANCE TO IT
!                 --------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FINDL',0,ZHOOK_HANDLE)
KSTDLE = 1
PDIFF  = ABS(PCMPPP - RSTDPL(1))
DO J = 2, NSTDPL
  ZDIFF = ABS(PCMPPP - RSTDPL(J))
  IF(ZDIFF <= PDIFF) THEN
    PDIFF  = ZDIFF
    KSTDLE = J
  ENDIF
ENDDO
!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                 ------
IF (LHOOK) CALL DR_HOOK('FINDL',1,ZHOOK_HANDLE)
END SUBROUTINE FINDL

