SUBROUTINE FLAGO2N( KO2N )

!-----------------------------------------------------------------------
!  FLAGO2N : Conversion des flags format MF vers le format CEPMMT et vice-versa
!  --------  Flags conversion, from MF format to ECMWF format and vice-versa
!  auteur                : Ph.Caille 09/98
!  modifications         :
!  Sous-programme amont  : SCREEN
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NOBTOT
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KO2N 
INTEGER(KIND=JPIM) :: IRET, INFO(1)

INTEGER(KIND=JPIM) ::   I84210(0:8)

INTEGER(KIND=JPIM) :: ICLN, ICLNCO, ICQE, ICQECO,&
 & ICQF, ICQFCO, ICQK, ICQKCO, ICQV, ICQVCO, &
 & IDBLE, IDEV1, IDSTA, IDSTA1, IDSTA2, IDSTA3, &
 & IDSTA4, IFLG, INCLN, INCQE, INCQF, &
 & INCQK, INCQV, IRBLE, IREV1, JO, JW  

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"
#include "qabitu.h"

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('FLAGO2N',0,ZHOOK_HANDLE)
ICQFCO = 1
ICQECO = 5
ICQKCO = 9
ICQVCO = 13
ICLNCO = 17
I84210 = 0

IF ( KO2N == 1 ) THEN
  I84210(1) = 0
  I84210(2) = 1
  I84210(4) = 2
  I84210(8) = 3
ELSEIF ( KO2N == -1 ) THEN
  I84210(0) = 1
  I84210(1) = 2
  I84210(2) = 4
  I84210(3) = 8
ELSE
  CALL ABOR1('wrong value for FLAGO2N argument')
ENDIF

!-----------------------------------------------------------------------

IRET=0
INFO=0
CALL GETDB('FLAGO',0,IRET,INFO,0,ZINFO,0,-1,-1,-1,-1,-1,-1)

!  - 1 -  For each observation
!         --------------------

DO JO=1,NOBTOT

  IREV1 = ROBHDR(JO,MDBREV1)
  IRBLE = ROBHDR(JO,MDBRBLE)

!-----------------------------------------------------------------------

!  - 2 -  For each variable
!         -----------------

  DO JW= MLNKH2B(JO),MLNKH2B(JO+1)-1

!  - 2.1 -  analysis flag (MDBFLG)
!           ----------------------

    IF ( KO2N == 1 ) THEN
      IFLG  = ROBODY(JW,MDBFLG)
      ICQF  = KBITLU(IFLG,ICQFCO,4)
      ICQE  = KBITLU(IFLG,ICQECO,4)
      ICQK  = KBITLU(IFLG,ICQKCO,4)
      ICQV  = KBITLU(IFLG,ICQVCO,4)
      ICLN  = KBITLU(IFLG,ICLNCO,4)
      INCQF = I84210( ICQF )
      INCQE = I84210( ICQE )
      INCQK = I84210( ICQK )
      INCQV = I84210( ICQV )
      INCLN = I84210( ICLN )
      IFLG  = 0
      IFLG  = KBITEC(IFLG,INCQF,ICQFCO)
      IFLG  = KBITEC(IFLG,INCQE,ICQECO)
      IFLG  = KBITEC(IFLG,INCQK,ICQKCO)
      IFLG  = KBITEC(IFLG,INCQV,ICQVCO)
      IFLG  = KBITEC(IFLG,INCLN,ICLNCO)
      ROBODY(JW,MDBFLG) = IFLG
    ELSE
      IDSTA  = ROBODY(JW,MDBDSTA)
      IDSTA1 = KBITLU(IDSTA,1,1)
      IDSTA2 = KBITLU(IDSTA,2,1)
      IDSTA3 = KBITLU(IDSTA,3,1)
      IDSTA4 = KBITLU(IDSTA,4,1)
      IFLG   = 0
      IF ( IDSTA1 == 1 ) THEN
        IFLG = KBIT1OU0(IFLG,1,1)
      ELSE
        IF ( IDSTA2 == 1 ) IFLG = KBIT1OU0(IFLG,1,2)
        IF ( IDSTA4 == 1 ) IFLG = KBIT1OU0(IFLG,1,4)
        IF ( IDSTA3 == 1 ) THEN
          IFLG = KBIT1OU0(IFLG,1,3)
        ELSE
          IFLG = KBIT1OU0(IFLG,1,1)
        ENDIF
      ENDIF
      ROBODY(JW,MDBFLG) = IFLG
    ENDIF

!  - 2.2 -  data event first part flag (MDBDEV1)
!           ------------------------------------

    IDEV1 = ROBODY(JW,MDBDEV1)

!  - 2.3 -  data black-list event (MDBDBLE)
!           -------------------------------

    IDBLE = ROBODY(JW,MDBDBLE)
  ENDDO
ENDDO

CALL PUTDB('FLAGO',0,IRET,INFO,0,ZINFO,0)

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FLAGO2N',1,ZHOOK_HANDLE)
END SUBROUTINE FLAGO2N
