SUBROUTINE FLGDCO(KOBS,KBODY,KRDBMX,KFLAGS)

!**** SUBROUTINE FLGDCO - COMBINED DATUM FLAGGING

!          HEIKKI JARVINEN   ECMWF          15/NOV/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        RELATIVE AND SPECIFIC HUMIDITIES ARE DERIVED FROM TEMPERATURE.
!     IF TEMPERATURE AT THE SAME PRESSURE HAS BEEN FLAGGED (BY QUALITY
!     CONTROL), THE SAME FLAG SHOULD BE APPLIED ALSO FOR HUMUDITY
!     VARIABLES AT THE SAME PRESSURE. IF THE TEMPERATURE IS REJECTED,
!     REJECT ALSO THE HUMIDITY.

!**   INTERFACE
!     ---------

!     ***CALL FLGDCO(KOBS,KBODY,KRDBMX,KFLAGS)
!     WHERE KOBS   = POINTER TO THE REPORT
!           KBODY  = POINTER TO THE VARIABLE
!           KRDBMX = MAXIMUM REPORT DATA BASE FLAG
!           KFLAGS = DATUM FLAG VALUES

!        FLGDCO IS CALLED BY SUBROUTINE FLGTST

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!        FLGDMX  - GET THE MAXIMUM DATUM FLAG
!        NFLGDSE - SET DATUM FLAG

!**   MODIFICATIONS
!     -------------
!        H. Jarvinen 12 Dec 1996 : A modified subroutine call
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMANCS  , ONLY : RDEGREES
USE YOMALIM  , ONLY : NFLLIM
USE YOMVNMB  , ONLY : NVNUMB
USE YOMSCC   , ONLY : LSCDPR
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBODY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KRDBMX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLAGS(5) 
INTEGER(KIND=JPIM) :: IDSTA(4), IFLAGS(5) 

INTEGER(KIND=JPIM) :: IDFLMX, IHUM, INSTA, IOTP,&
 & IVCO, IVNM, IVRF, &
 & JBODY  

LOGICAL :: LLPRINT

REAL(KIND=JPRB) :: ZLAT, ZLON, ZPPP, ZPRF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "flgdmx.intfb.h"
#include "nflgdse.intfb.h"
#include "ngedsta.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.1   COMBINED FLAGGING ONLY FOR VARIABLES T/T2M

IF (LHOOK) CALL DR_HOOK('FLGDCO',0,ZHOOK_HANDLE)
IVNM = ROBODY(KBODY,MDBVNM)
IF(IVNM /= NVNUMB( 8) .AND. IVNM /= NVNUMB(10)) GOTO 3

!*          1.2   FLAGGED VARIABLES ONLY
!                 - CONSIDER RDB-, FIRST GUESS AND DEPARTURE FLAGS

IF(KRDBMX == NFLLIM(0) .AND. KFLAGS(2) == NFLLIM(0) .AND.&
 & KFLAGS(3) == NFLLIM(0)) GOTO 3  

!*          1.3   CHECK FOR THE POINTERS

!*
!     ------------------------------------------------------------------

!        2.       COMBINE TEMPERATURE/HUMIDITY FLAGS
!                 ----------------------------------

!*          2.1   GET HEADER INFORMATION AND START LOOP

IVCO = ROBODY(KBODY,MDBVCO)
ZPPP = ROBODY(KBODY,MDBPPP)
DAT_LOOP: DO JBODY = KBODY+1,MLNKH2B(KOBS+1)-1

!*          2.4   CHECK VERTICAL COORDINATE

  IVRF = ROBODY(JBODY,MDBVCO)
  IF(IVRF /= IVCO) CYCLE DAT_LOOP
  ZPRF = ROBODY(JBODY,MDBPPP)
  IF(ZPRF /= ZPPP) EXIT DAT_LOOP
!*
!           2.5   TEMPERATURE (T/T2M) FLAG TO HUMIDITY (RH/RH2M/Q)

  IHUM = ROBODY(JBODY,MDBVNM)
  IF(IHUM == NVNUMB( 5) .OR. IHUM == NVNUMB( 7) .OR.IHUM == NVNUMB(63)) THEN
!*
!           2.6   REJECT HUMIDITY IF TEMPERATURE REJECTED

    CALL NGEDSTA(INT(ROBODY(KBODY,MDBDSTA)),INSTA,IDSTA)
    IF(IDSTA(3) == 1) THEN
      ROBODY(JBODY,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(JBODY,MDBDSTA))
      ROBODY(JBODY,MDBDEV1) = ZCHEV1( ROBODY(JBODY,MDBDEV1), ND1EVENT(25))
    ENDIF
!*
!           2.7   GET THE MAXIMUM DATUM FLAG

    CALL FLGDMX(JBODY,IDFLMX,IFLAGS)
!*
!           2.8   REPLACE FIRST GUESS DATUM FLAG

    IF(IFLAGS(2) < KFLAGS(2)) THEN
      CALL NFLGDSE(JBODY,'FIRST',KFLAGS(2))
      ROBODY(JBODY,MDBDEV1) = ZCHEV1( ROBODY(JBODY,MDBDEV1), ND1EVENT(25))
      LLPRINT = .TRUE.
    ENDIF
!*
!           2.9   REPLACE DEPARTURE DATUM FLAG

    IF(IFLAGS(3) < KFLAGS(3)) THEN
      CALL NFLGDSE(JBODY,'DEPAR',KFLAGS(3))
      ROBODY(JBODY,MDBDEV1) = ZCHEV1( ROBODY(JBODY,MDBDEV1), ND1EVENT(25))
      LLPRINT = .TRUE.
    ENDIF
!*
!           2.10  INFORM

    IF(LSCDPR .AND. LLPRINT) THEN
      IOTP = ROBHDR(KOBS,MDBOTP)
      ZLAT = ROBHDR(KOBS,MDBLAT)*RDEGREES
      ZLON = ROBHDR(KOBS,MDBLON)*RDEGREES
      WRITE(NULOUT,&
       & '('' T-FLAGS ('',I2,1X,I2,'' ) OVERRIDE RH/Q-FLAGS ('',&
       & I2,1X,I2,'' ) FOR OBSTYP '',I2,'' AT LAT/LON '',&
       & 2(F7.2,1X))') KFLAGS(2),KFLAGS(3),IFLAGS(2),IFLAGS(3),&
       & IOTP,ZLAT,ZLON  
    ENDIF
  ENDIF
!*
!           2.11  CLOSE THE LOOP

ENDDO DAT_LOOP

!     ------------------------------------------------------------------

3 CONTINUE
IF (LHOOK) CALL DR_HOOK('FLGDCO',1,ZHOOK_HANDLE)
END SUBROUTINE FLGDCO
