SUBROUTINE FLGDSE(PREPORT,KRLN,KSQN,CDFLAG,KFLAG)

!**** SUBROUTINE FLGDSE - SET DESIRED DATUM FLAG

!          HEIKKI JARVINEN   ECMWF          15/NOV/1995

!*    PURPOSE
!     -------

!        SET DESIRED DATUM FLAG (FINAL/F.G./DEPAR/VAR Q.C./BLACKLIST)

!**   INTERFACE
!     ---------

!     ***CALL FLGDSE(PREPORT,KRLN,KSQN,CDFLAG,KFLAG)
!     WHERE PREPORT = THE WHOLE REPORT IN AN REAL ARRAY
!           KRLN    = THE REPORT LENGTH
!           KSQN    = SEQUENCE NUMBER OF THE DATUM
!           CDFLAG  = NAME OF THE DESIRED FLAG
!           KFLAG   = DESIRED VALUE OF THAT FLAG

!        FLGDSE IS CALLED BY VARIOUS SUBROUTINES

!*    METHOD
!     ------

!        CRACK AND REPACK THE PACKED DATUM FLAG WORD

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!        H. Jarvinen 12 Dec 1996 : A modified subroutine call
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCMHDR , ONLY : NCMHLN   ,NCMBLN   ,NCMNLV
USE YOMCMBDY , ONLY : NCMFLG   ,NCMESQN  ,NCMAFFBP ,NCMAFFOC ,&
 & NCMAGFBP ,NCMAGFOC ,NCMADFBP ,NCMADFOC ,NCMAVFBP ,&
 & NCMAVFOC ,NCMABFBP ,NCMABFOC  
USE YOMLUN   , ONLY : NULERR
USE YOMSCREE , ONLY : NDFLMX   ,NDFLMN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KRLN 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PREPORT(KRLN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSQN 
CHARACTER(LEN=5)  ,INTENT(IN)    :: CDFLAG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLAG
INTEGER(KIND=JPIM) :: IBLN, ICMFLG, IHLN, INLV, IPOS, ISQN, J
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcobs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.1   POINTERS

IF (LHOOK) CALL DR_HOOK('FLGDSE',0,ZHOOK_HANDLE)
IHLN = INTREA(PREPORT(NCMHLN))
IBLN = INTREA(PREPORT(NCMBLN))
INLV = INTREA(PREPORT(NCMNLV))

!*          1.2   FOR EXTENDED CMA

IF(KSQN <= INLV) THEN
  IPOS = IHLN + (KSQN-1)*IBLN
  ISQN = INTREA(PREPORT(IPOS+NCMESQN))
  IF(ISQN == KSQN) GOTO 2
ENDIF

!*          1.3   FOR COMPRESSED CMA

DO J = 1, INLV
  IPOS = IHLN + (J-1)*IBLN
  ISQN = INTREA(PREPORT(IPOS+NCMESQN))
  IF(ISQN == KSQN) GOTO 2
ENDDO

!*          1.4   SOMETHING IS WRONG

CALL ABOR1(' FLGDSE : SEQUENCE NUMBER NOT FOUND ')
!*
!     ------------------------------------------------------------------

!        2.       SET THE DESIRED DATUM FLAG
!                 --------------------------

!*          2.0   INTIALIZE AND TEST THE FLAG VALUE BEFORE INSERTING

2 CONTINUE
ICMFLG = INTREA(PREPORT(IPOS+NCMFLG))
IF(KFLAG < NDFLMN .OR. KFLAG > NDFLMX) THEN
  WRITE(NULERR,'('' *** DATUM FLAG OUT OF RANGE '')')
  WRITE(NULERR,'(''     - flag value '',I4)') KFLAG
  WRITE(NULERR,'(''     - min.limit  '',I4)') NDFLMN
  WRITE(NULERR,'(''     - max.limit  '',I4)') NDFLMX
  CALL ABOR1('FLGDSE: DATUM FLAG OUT OF RANGE')
ENDIF

!*          2.1   FINAL FLAG

IF(CDFLAG == 'FINAL') THEN
  CALL MVBITS(KFLAG,0,NCMAFFOC,ICMFLG,NCMAFFBP)

!*          2.2   FIRST GUESS FLAG

ELSEIF(CDFLAG == 'FIRST') THEN
  CALL MVBITS(KFLAG,0,NCMAGFOC,ICMFLG,NCMAGFBP)

!*          2.3   DEPARTURE FLAG

ELSEIF(CDFLAG == 'DEPAR') THEN
  CALL MVBITS(KFLAG,0,NCMADFOC,ICMFLG,NCMADFBP)

!*          2.4   VARIATIONAL QUALITY CONTROL FLAG

ELSEIF(CDFLAG == 'VARQC') THEN
  CALL MVBITS(KFLAG,0,NCMAVFOC,ICMFLG,NCMAVFBP)

!*          2.5   BLACKLIST FLAG

ELSEIF(CDFLAG == 'BLACK') THEN
  CALL MVBITS(KFLAG,0,NCMABFOC,ICMFLG,NCMABFBP)

!*          2.6   SOMETHING IS NOT QUITE RIGHT

ELSE
  WRITE(NULERR,'('' *** UNKNOWN KEYWORD '', A)') CDFLAG
  WRITE(NULERR,'(''     - TRY :   '')')
  WRITE(NULERR,'(''       FINAL/FIRST/DEPAR/VARQC/BLACK '')')
  CALL ABOR1('FLGDSE: UNKNOWN KEYWORD')
ENDIF

!           2.7   UPDATE THE DATUM FLAG WORD

PREPORT(IPOS+NCMFLG) = REAINT(ICMFLG)
!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                  ------
IF (LHOOK) CALL DR_HOOK('FLGDSE',1,ZHOOK_HANDLE)
END SUBROUTINE FLGDSE
