SUBROUTINE FLGTST(KSET)

!**** SUBROUTINE FLGTST - TRANSFORM DATUM/REPORT FLAG INFORMATION TO STATUS

!          HEIKKI JARVINEN   ECMWF          15/NOV/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        IN THE IFS OBSERVATION SCREENING, THE DATUM/REPORT FLAGS ARE
!     SET IN VARIOUS CHECKS, LIKE PRE-CHECK, BLACKLISTING AND FIRST GUESS
!     CHECKS. DATUM/REPORT STATUS WORD IS LEFT UNTOUCHED IN THOSE ROUTINES.
!     AFTER THESE CHECKS AND BEFORE ANY REDUNDANCY CHECKS ARE PERFORMED,
!     ALL THE FLAG INFORMATION, TOGETHER WITH THE RDB-FLAGS (REPORT
!     DATA BASE), IS USED TO MODIFY THE DATUM/REPORT STATUS WORD. FLGTST
!     WILL, IN OTHER WORDS, TRANSFORM DATUM/REPORT FLAG INFORMATION TO
!     A STATUS.

!**   INTERFACE
!     ---------

!     ***CALL FLGTST(KSET)
!     WHERE KSET  = DB LEN

!        FLGTST IS CALLED BY SUBROUTINE DECIS

!*    METHOD
!     ------

!        CRACK THE FLAG WORDS AND MAKE DECISIONS ABOUT THE DATUM/REPORT
!     STATUS.

!*    EXTERNALS
!     ---------

!        RDBFLR  - GET THE (MAXIMUM) RDB-FLAG FOR REPORT
!        RDBFLD  - GET THE (MAXIMUM) RDB-FLAG FOR DATUM
!        FLGDMX  - GET THE MAXIMUM DATUM FLAG
!        FLGDCO  - PERFORM COMBINED FLAGGING
!        NGENADA - GET THE NUMBER OF ACTIVE DATA IN THE REPORT

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCMBDY , ONLY : NTLIDBP  ,NTCPABP  ,NTCPAOC  ,NTAPABP  ,&
 & NTAPAOC  ,NPLIDBP  ,NPCPABP  ,NPCPAOC  ,&
 & NPAPABP  ,NPAPAOC  
USE YOMOBSET_THSAFE
USE YOMCOCTP , ONLY : NTEMP    ,NPILOT
USE YOMALIM  , ONLY : NFLLIM
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMEV  , ONLY : NR1EVENT ,ND1EVENT
USE YOMDB
USE YOMLUN   , ONLY : NULOUT
USE YOMSCC   , ONLY : LSCDPR
USE OML_MOD, ONLY : OML_MY_THREAD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM) :: IFLAGS(5)

INTEGER(KIND=JPIM) :: IAPART, ICDTYP,&
 & ICPART, IDFLMX, IFDPRE, IFDVAR, &
 & IFRALT, IFRDAT, IFRLAT, IFRLON, IFRTIM, &
 & ILEVID, INACD, INLV, IOBTYP, IPOS, IRDBMXD, &
 & IRDBMXR, IVNM, JNLV, JOBS, ITID
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "flgdco.intfb.h"
#include "flgdmx.intfb.h"
#include "ngenada.intfb.h"
#include "rdbfld.intfb.h"
#include "rdbflr.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

IF (LHOOK) CALL DR_HOOK('FLGTST',0,ZHOOK_HANDLE)
ITID = OML_MY_THREAD()
IF(LSCDPR) WRITE(NULOUT,'(''     TRANSFORM DATUM/REPORT FLAG TO STATUS'')')

!     ------------------------------------------------------------------

!        2.       START THE MAIN LOOPS
!                 --------------------

!*          2.1   LOOP OVER ALL REPS

OBS_LOOP: DO JOBS = 1, OBSET(ITID)%MLNSET(KSET)

!*          2.2   FIND OUT OBSERVATION POINTER, ETC.

  IOBTYP = ROBHDR(JOBS,MDBOTP)
  ICDTYP = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)

!     ------------------------------------------------------------------

!        3.       REPORT FLAGS
!                 ------------

!*          3.1   GET THE RDB-FLAGS FOR REPORT

  CALL RDBFLR(JOBS,IRDBMXR,IFRLAT,IFRLON,IFRDAT,IFRTIM,IFRALT)

!*             3.1.1   RDB-FLAG LIMIT FOR LATITUDE IS TWO

  IF(IFRLAT >= NFLLIM(2)) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(4))
  ENDIF

!*             3.1.2   RDB-FLAG LIMIT FOR LONGITUDE IS TWO

  IF(IFRLON >= NFLLIM(2)) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(4))
  ENDIF

!*             3.1.3   RDB-FLAG LIMIT FOR DATE IS TWO

  IF(IFRDAT >= NFLLIM(2)) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(4))
  ENDIF

!*             3.1.4   RDB-FLAG LIMIT FOR TIME IS TWO

  IF(IFRTIM >= NFLLIM(2)) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(4))
  ENDIF

!*             3.1.5   RDB-FLAG FOR ALTITUDE IS NOT CONSIDERED

!*          3.2   NUMBER OF BODY ENTRIES (IF EMPTY REPORT)

  INLV = MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  IF(INLV == 0) THEN
    CYCLE OBS_LOOP
  ENDIF
!*
!     ------------------------------------------------------------------

!        4.       DATUM FLAGS
!                 -----------

!*          4.1   START THE LOOP OVER ALL THE DATA

  DAT_LOOP: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
    IPOS = JNLV
    IVNM = ROBODY(IPOS,MDBVNM)
    IAPART = 0
    ICPART = 0
!*
!           4.2   GET THE (MAXIMUM) RDB-FLAG FOR DATUM

    CALL RDBFLD(IPOS,IRDBMXD,IFDPRE,IFDVAR)
!*
!           4.3   GET THE MAXIMUM DATUM FLAG

    CALL FLGDMX(IPOS,IDFLMX,IFLAGS)
!*
!           4.4   PERFORM COMBINED FLAGGING (T TO RH/Q)

    CALL FLGDCO(JOBS,IPOS,IRDBMXD,IFLAGS)
!*
!           4.5   GET THE TEMP/PILOT LEVEL IDENTIFIER (A/C PART)

    IF(IOBTYP == NTEMP) THEN
      ILEVID = ROBODY(IPOS,MDBTELID)
      IAPART = IBITS(ILEVID,NTLIDBP+NTAPABP,NTAPAOC)
      ICPART = IBITS(ILEVID,NTLIDBP+NTCPABP,NTCPAOC)
    ELSEIF(IOBTYP == NPILOT) THEN
      ILEVID = ROBODY(IPOS,MDBPILID)
      IAPART = IBITS(ILEVID,NPLIDBP+NPAPABP,NPAPAOC)
      ICPART = IBITS(ILEVID,NPLIDBP+NPCPABP,NPCPAOC)
    ENDIF

!           4.6   GET DATUM STATUS FROM RDB-FLAG INFORMATION

!*             4.6.1   MAX RDB-FLAG 3 :: ALL DATA REJECTED

    IF(IRDBMXD == NFLLIM(3)) THEN
      ROBODY(IPOS,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IPOS,MDBDSTA))
      ROBODY(IPOS,MDBDEV1) = ZCHEV1( ROBODY(IPOS,MDBDEV1),ND1EVENT(4))

!*             4.6.2   MAX RDB-FLAG 2 :: ALL NON-STD-LEVEL DATA REJECTED

    ELSEIF(IRDBMXD == NFLLIM(2)) THEN
      IF(IAPART == 0.AND. ICPART == 0) THEN
        ROBODY(IPOS,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IPOS,MDBDSTA))
        ROBODY(IPOS,MDBDEV1) = ZCHEV1( ROBODY(IPOS,MDBDEV1),ND1EVENT(4))
      ENDIF
    ENDIF

!*             4.6.3   RDB PRESSURE FLAG 1,2 OR 3 :: SELECTED DATA REJECTED

    IF(IFDPRE >= NFLLIM(1)) THEN
      IF(IVNM /= NVNUMB(12) .AND.&
         & IVNM /= NVNUMB(25) .AND.&
         & IVNM /= NVNUMB(46)      ) THEN  
        ROBODY(IPOS,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IPOS,MDBDSTA))
        ROBODY(IPOS,MDBDEV1) = ZCHEV1( ROBODY(IPOS,MDBDEV1),ND1EVENT(4))
      ENDIF
    ENDIF
!*
!           4.7   GET DATUM STATUS FROM DATUM FLAG INFORMATION
!                 - CONSIDER FIRST GUESS AND DEPARTURE FLAGS ONLY

!              4.7.1   DATUM FLAG 3 :: ALL DATA REJECTED

    IF(IFLAGS(2) == NFLLIM(3) .OR. IFLAGS(3) == NFLLIM(3)) THEN
      ROBODY(IPOS,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IPOS,MDBDSTA))

!              4.7.2   DATUM FLAG 2 :: ALL NON-STANDARD-LEVEL DATA REJECTED

    ELSEIF(IFLAGS(2) == NFLLIM(2) .OR.IFLAGS(3) == NFLLIM(2)    ) THEN
      IF(IAPART == 0.AND. ICPART == 0) THEN
        ROBODY(IPOS,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IPOS,MDBDSTA))
      ENDIF
    ENDIF
!*
!     ------------------------------------------------------------------

!        5.       CLOSE THE LOOP OVER ...
!                 -----------------------
!*
!           5.1   ... THE REPORT

  ENDDO DAT_LOOP

!*          5.2   CHECK WHETHER ANY ACTIVE DATA IS LEFT

  CALL NGENADA(JOBS,INACD)
  IF(INACD == 0) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT( 2))
  ENDIF
!*
!           5.3   ... THE OBSERVATIONS IN JSET

ENDDO OBS_LOOP
!*
!     ------------------------------------------------------------------

!        6.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('FLGTST',1,ZHOOK_HANDLE)
END SUBROUTINE FLGTST
