SUBROUTINE FLSPEEDBAD(PLAT1,PLON1,PLAT2,PLON2,KMINDIFF,PFLMAXSPEED, &
 & LDFLSPEEDBAD)  

!**** SUBROUTINE FLSPEEDBAD - CHECK IF FLIGHT SPEED IS UNREALISIC

!          LARS ISAKSEN   ECMWF          29/01/2001

!*    PURPOSE
!     -------

!     Based on arch distance on the sphere from (plat1,plon1) to
!     (plat2,plon2) it is checked if an aircraft realistically can
!     fly this distance in the specified time.
!     To reduce computational costs ZFLSPEED and FL_MAXSPEED are
!     transformed quantities, i.e. not km/hour.

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     DUMMY VARIABLES
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMINDIFF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLMAXSPEED 
LOGICAL           ,INTENT(OUT)   :: LDFLSPEEDBAD 
!     LOCAL VARIABLES
REAL(KIND=JPRB)   :: ZA, ZB, ZAA, ZC, ZD, ZBB, ZE, ZF, ZCC, ZFLSPEED
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('FLSPEEDBAD',0,ZHOOK_HANDLE)
ZA=COS(PLAT2)*COS(PLON2)
ZB=COS(PLAT1)*COS(PLON1)

ZAA=(ZA-ZB)**2

ZC=COS(PLAT2)*SIN(PLON2)
ZD=COS(PLAT1)*SIN(PLON1)

ZBB=(ZC-ZD)**2

ZE=SIN(PLAT2)
ZF=SIN(PLAT1)

ZCC=(ZE-ZF)**2

ZFLSPEED=(ZAA+ZBB+ZCC)/(REAL(ABS(KMINDIFF))+0.5)**2

LDFLSPEEDBAD = (ZFLSPEED > PFLMAXSPEED)
IF (LHOOK) CALL DR_HOOK('FLSPEEDBAD',1,ZHOOK_HANDLE)

END SUBROUTINE FLSPEEDBAD
