SUBROUTINE GEFGER(KTSLOT)

!***  *GEFGER* - INTERPOLATE FORECAST ERROR TO OBSERVATIONS

!     PURPOSE.
!     --------

!        INTERPOLATE FORECAST ERROR TO OBSERVATION LOCATIONS. 

!**   INTERFACE .
!     ----------
!        *CALL* *GEFGER(KLEN)
!     WHERE KLEN  = DB LEN

!        EXPLICIT ARGUMENTS:
!        -------------------

!        IMPLICIT ARGUMENTS:
!        -------------------

!     METHOD.
!     -------
!        FORECAST ERRORS FOR THE CURRENT ANALYSIS ARE ARCHIVED IN MARS.
!        THESE FIELDS  ARE READ IN AND ARE INTERPOLATED IN HORIZONTAL
!        AND VERTICAL TO THE OBSERVATION LOCATIONS.
!        IF THERE ARE NO ERRORS FOR CERTAIN VARIABLES (CURRENTLY IN OI
!        THERE IS NO ERROR FOR PS NOR Q), THEY ARE SPECIFIED.

!     EXTERNALS.
!     ----------

!        SUHIFCE  - INTERPOLATE FORECAST ERRORS IN THE HORIZONTAL
!        SUVIFCE  - INTERPOLATE FORECAST ERRORS IN THE VERTICAL
!        GPPRE    - CALCULATE PRESSURE ON MODELS LEVELS

!     REFERENCE.
!     ----------

!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS.

!     AUTHOR.
!     -------

!     HEIKKI JARVINEN   ECMWF   1/05/96
!     DRASKO VASILJEVIC ECMWF   1/10/99

!     MODIFICATIONS.
!     --------------
!     Jean-Noel Thepaut Meteo-France 4/11/98 : use geographical coordinates
!                of the observation points to do the interpolations.
!     M Hamrud 22/01/2001 : move the reading of errors to new routine 
!                           INIFGER to avoid repeating for all time slots
!     C Koepken 30/10/2001: use HIRS channels to estimate errors for 
!                           GOES imager channels
!        R. El Khatib : 01-08-07 Pruning options
!     M Szyndel 15/8/03:  Use HIRS channels to estimate errors in MSG radiances
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     M Szyndel        23-Mar-2004 Add MTSAT and FY-2
!     P Poli           28-Aug-2006 Set GPSRO f.g. error for MF
!     Antje Dethof     12-May-2005 Introduce GRG fields
!     David Tan        02-Mar-2006 Introduce Aeolus HLoS (Doppler wind lidar)
!     Alan Geer        10-Jun-2009 FG error estimates for all-sky sensors
!     Anne Fouilloux   31-Mar-2010 Use new codetype for meris
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT, NULERR
USE YOMCT0B  , ONLY : LECMWF
USE YOMCST   , ONLY : RG       ,RV       ,RCPV     ,RETV     ,&
 & RCW      ,RCS      ,RLVTT    ,RLSTT    ,RTT      ,&
 & RALPW    ,RBETW    ,RGAMW    ,RALPS    ,RBETS    ,&
 & RGAMS    ,RALPD    ,RBETD    ,RGAMD  
USE YOMOBSET_THSAFE
USE YOMOBSET , ONLY : MTYPOB
USE YOMTVRAD , ONLY : MSENSOR_METEOSAT   ,MSENSOR_CHANOFFL, &
 & MSENSOR_VTPR1,MSENSOR_VTPR2,MSENSOR_HIRS,MSENSOR_MSU   , &
 & MSENSOR_SSU  ,MSENSOR_AMSUA,MSENSOR_AMSUB,MSENSOR_GOESIMG, &
 & MSENSOR_SSMI, MSENSOR_MSG, MSENSOR_MTSATIMG, MSENSOR_VISSR, &
 & MSENSOR_SSMIS, MSENSOR_TMI, MSENSOR_AMSRE, MSENSOR_MHS, &
 & SATGRP_TABLE, SATOBGRP_TABLE
USE YOMCOCTP , ONLY : NSATEM ,NSSMI  ,NGTHRB ,NTCWC ,NSATOB ,NLIMB
USE YOMANCS  , ONLY : RMDI     ,RDEGREES
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMGEM   , ONLY : VBH      ,VAH
USE YOMGRB   , ONLY : NGRBT    ,NGRBU    ,NGRBV    ,NGRBQ    ,&
 & NGRBLNSP ,NGRBGH   ,NGRBBTMP  ,NGRBTCWV ,NGRBO3   ,&
 & NGRBTCO3, NGRBR ,NGRBGRG                                
USE YOMDB
USE YOMSCC   , ONLY : LSCDPR
USE YOMFGER  , ONLY : JPMAXVAR ,MNROWS  ,MNVARS  ,MNFLDS  ,&
 & MNPTE   ,MNLEVMX  ,MLONE   ,MFIELDS ,MNLEVS  ,&
 & RZLATE  ,RZLON0E  ,RZDLONE ,RZA     ,RZB     ,RZEGRID  
USE GOMS_MIX  , ONLY : YGOMUA,YGOMS, YGOMS_2D
USE YOMECTAB , ONLY : NOBSPROFS
USE YOM_YGFL , ONLY : JPGRG
USE YOMMWAVE, ONLY : MWAVE_ID, N_SATSENS, MWAVE_ERROR_MODEL 
USE MPL_MODULE
USE OML_MOD, ONLY : OML_MY_THREAD
USE YOMLIMB,   ONLY : Y_LIMBGRP_TABLE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTSLOT 
INTEGER(KIND=JPIM) :: ICDTYPE, ICH, ICMCH, ICMVCO,&
 & ICMVNM, IOBS, ILEV, ILEVA, &
 & ILEVA2, ILEVB, ILEVB2, &
 & INLEVM,  INLV,  IOBTYP, &
 & ISENSOR, ITLEV, IVAR, IVGH, &
 & IVLNSP, IVO3, IVQ, IVR, IVT, IVTBT, IVTCO3, IVTCWV, &
 & IVU, IVV, JLEV, JNLV, JOBS, JSET, JVAR, &
 & IDEVN1, IDEV17, IDSTAT, IDREJ, INSTA, IDSTA(4), &
 & IMAPOMM, ITID, JGRG, JSAT, ISATID

INTEGER(KIND=JPIM), DIMENSION(JPGRG) :: IVGRG

LOGICAL :: LLGH, LLLNSP, LLO3, LLQ, LLRAD1C, LLMWAVE, LLT, LLTBT,&
 & LLTCO3, LLTCWV, LLU, LLV, LLVINT, LLR, &
 & LLAERSM, LLAERLG, LLSATFOUND  

LOGICAL, DIMENSION(JPGRG) :: LLGRG

REAL(KIND=JPRB) :: Z, ZCMPPP(1,1), ZCMPRL, ZESOP, ZEXP, ZFGEMIN,&
 & ZFGER, ZMODPS, ZOBLAT(1), ZOBLON(1), ZQS, ZTTT(1,1), &
 & ZW   , ZOBER

REAL(KIND=JPRB) :: ZP37

#include "fcobs.h"
#include "fcttrm.h"

!-----------------------------------------------------------------------


INTEGER(KIND=JPIM), DIMENSION(JPMAXVAR) :: IFLDOFF

REAL(KIND=JPRB) ::  ZVALM(2),ZMODPH(0:NFLEVG),ZMODPF(NFLEVG),ZMODT(0:NFLEVG)

REAL(KIND=JPRB),DIMENSION(:,:), ALLOCATABLE :: ZIGRID,ZPRSFI,ZVALH
INTEGER(KIND=JPIM) :: IPREVSL, INFO(1), IRET, IGRP
REAL(KIND=JPRB) :: ZINFO(1)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "gppre.intfb.h"
#include "ngedsta.intfb.h"
#include "suhifce.intfb.h"
#include "suvifce.intfb.h"

#include "openmp_obs.h"
!-----------------------------------------------------------------------

!        0.    INITIALIZE

!           0.1   INFORM

IF (LHOOK) CALL DR_HOOK('GEFGER',0,ZHOOK_HANDLE)
ITID = OML_MY_THREAD()
WRITE(NULOUT,'(''     GEFGER '')')

!           0.2   CHECK CONSISTENCY

!IF (KLEN /= SUM(OBSET(ITID)%MLNSET(1:OBSET(ITID)%NSETOT))) THEN
!  WRITE(NULOUT,'('' *** UNEXPECTED NO. OF HEADER ENTRIES :'',2I12)') &
!   & KLEN, SUM(OBSET(ITID)%MLNSET(1:OBSET(ITID)%NSETOT))  
!  CALL FLUSH(NULOUT)
!  CALL ABOR1(' GEFGER ')
!ENDIF

!           0.3   INITIALIZE SOME PARAMETERS

LLT  = .FALSE.
LLU = .FALSE.
LLV = .FALSE.
LLQ = .FALSE.
LLR = .FALSE.
LLO3 = .FALSE.
LLLNSP  = .FALSE.
LLGH = .FALSE.
LLTBT   = .FALSE.
LLTCWV  = .FALSE.
LLTCO3  = .FALSE.
LLAERSM = .FALSE.
LLAERLG = .FALSE.
DO JGRG=1,JPGRG
  LLGRG(JGRG) = .FALSE.
ENDDO


!           1.5   SETUP LOGICAL PARAMETERS AND INFORM

DO JVAR = 1, MNVARS
  IFLDOFF(JVAR) = SUM(MNLEVS(1:JVAR-1))
  IF(MFIELDS(JVAR) == NGRBT ) THEN
    LLT = .TRUE.
    IVT = JVAR
    WRITE(NULOUT,'(1X,I4,'' TEMPERATURE AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBU) THEN
    LLU = .TRUE.
    IVU = JVAR
    WRITE(NULOUT,'(1X,I4,'' U-COMPONENT AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBV) THEN
    LLV = .TRUE.
    IVV = JVAR
    WRITE(NULOUT,'(1X,I4,'' V-COMPONENT AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBQ) THEN
    LLQ = .TRUE.
    IVQ = JVAR
    WRITE(NULOUT,'(1X,I4,'' SPEC. HUM.  AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBR) THEN
    LLR = .TRUE.
    IVR = JVAR
    WRITE(NULOUT,'(1X,I4,'' REL. HUM.  AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBO3) THEN
    LLO3 = .TRUE.
    IVO3 = JVAR
    WRITE(NULOUT,'(1X,I4,'' OZONE  AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBLNSP ) THEN
    LLLNSP  = .TRUE.
    IVLNSP  = JVAR
    WRITE(NULOUT,'(1X,I4,'' LOG(PS)     AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBGH) THEN
    LLGH = .TRUE.
    IVGH = JVAR
    WRITE(NULOUT,'(1X,I4,'' GEOP.HEIGHT AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBBTMP) THEN
    LLTBT = .TRUE.
    IVTBT = JVAR
    WRITE(NULOUT,'(1X,I4,'' TOVS RAD  FOR '',I2,'' CHANNELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
    WRITE(NULOUT,*) ' Channels: ',NINT(RZA(1:MNLEVS(JVAR),JVAR))
  ELSEIF(MFIELDS(JVAR) == NGRBTCWV ) THEN
    LLTCWV  = .TRUE.
    IVTCWV  = JVAR
    WRITE(NULOUT,'(1X,I4,'' T.C. WATER V. AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBTCO3 ) THEN
    LLTCO3  = .TRUE.
    IVTCO3  = JVAR
    WRITE(NULOUT,'(1X,I4,'' T.C. OZONE AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)  
  ELSEIF(MFIELDS(JVAR) == NGRBGRG(1) ) THEN
    LLGRG(1)  = .TRUE.
    IVGRG(1)  = JVAR
    WRITE(NULOUT,'(1X,I4,'' GRG1   AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)
  ELSEIF(MFIELDS(JVAR) == NGRBGRG(2) ) THEN
    LLGRG(2)  = .TRUE.
    IVGRG(2)  = JVAR
    WRITE(NULOUT,'(1X,I4,'' GRG2   AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)
  ELSEIF(MFIELDS(JVAR) == NGRBGRG(3) ) THEN
    LLGRG(3)  = .TRUE.
    IVGRG(3)  = JVAR
    WRITE(NULOUT,'(1X,I4,'' GRG3   AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)
  ELSEIF(MFIELDS(JVAR) == NGRBGRG(4) ) THEN
    LLGRG(4)  = .TRUE.
    IVGRG(4)  = JVAR
    WRITE(NULOUT,'(1X,I4,'' GRG4   AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)
  ELSEIF(MFIELDS(JVAR) == NGRBGRG(5) ) THEN
    LLGRG(5)  = .TRUE.
    IVGRG(5)  = JVAR
    WRITE(NULOUT,'(1X,I4,'' GRG5   AT '',I2,'' LEVELS '')')&
     & MFIELDS(JVAR),MNLEVS(JVAR)
  ENDIF
ENDDO

!-----------------------------------------------------------------------

!        2.    PERFORM INTERPOLATION, CURRENTLY DATUM BY DATUM

!           2.1   PREPARE FOR INTERPOLATION IN THE HORIZONTAL
!                 -------------------------------------------

!           2.2   NUMBER OF POINTS IN THE HORIZONTAL

ALLOCATE(ZIGRID(1,MNFLDS))
ALLOCATE(ZVALH(1,0:MNLEVMX))
ALLOCATE(ZPRSFI(1,0:MNLEVMX))
IF(LSCDPR) THEN
  WRITE(NULOUT,9990) 'ZIGRID   ',SIZE(ZIGRID   ),SHAPE(ZIGRID   )
  WRITE(NULOUT,9990) 'ZVALH    ',SIZE(ZVALH    ),SHAPE(ZVALH    )
  WRITE(NULOUT,9990) 'ZPRSFI   ',SIZE(ZPRSFI   ),SHAPE(ZPRSFI   )
ENDIF

!           2.3   LOOP OVER ALL THE OBSERVATION SETS

!*          2.3.1 DEFINE SET LEN COUNTER

IPREVSL = 0

SET_LOOP: DO JSET = 1, OBSET(ITID)%NSETOT

IOBTYP = MOD(OBSET(ITID)%MTYPOB(JSET),100)
IGRP =MOD(OBSET(ITID)%MTYPOB(JSET)/100,100)

IF(IOBTYP == NSATEM) THEN
  ICDTYPE=SATGRP_TABLE(IGRP)%codetype
ELSEIF(IOBTYP == NSATOB)THEN
  ICDTYPE= SATOBGRP_TABLE(IGRP)%codetype
ELSEIF(IOBTYP == NLIMB)THEN
  ICDTYPE= Y_LIMBGRP_TABLE(IGRP)%codetype  ! For ODB
ELSE
  ICDTYPE=-1
ENDIF

  CALL GETDB('SUFGER', 1, IRET, INFO, 0, ZINFO, 0, &
   & JSET, KTSLOT, -1, IOBTYP, ICDTYPE, -1)
!*
!           2.4   LOOP OVER ALL THE OBSERVATIONS

  OBS_LOOP: DO JOBS = 1, OBSET(ITID)%MLNSET(JSET)

!*          2.5   OBSERVATION LATITUDE/LONGITUDE (IN RADIANS)

    IOBS = JOBS+IPREVSL
    IMAPOMM = ROBHDR(IOBS,MDB_MAPOMM_AT_INDEX)
!   IOBTYP = ROBHDR(IOBS,MDBOTP)
!   IF(IOBTYP /= ROBHDR(IOBS,MDBOTP)) write(0,*)'IOBTYP',IOBTYP,ROBHDR(IOBS,MDBOTP)
    ZOBLAT(1) = ROBHDR(IOBS,MDBLAT)
    ZOBLON(1) = ROBHDR(IOBS,MDBLON)
!   ICDTYPE= ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
!   IF(ICDTYPE /= ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)) write(0,*)'ICDTYPE',ICDTYPE,ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)

    IF(IOBTYP == NSATEM) THEN
      LLRAD1C= (ICDTYPE == NGTHRB)
      LLMWAVE= (ICDTYPE == NSSMI)
!*AF add LLTCWV as meris has a new codetype...
      IF(LLRAD1C .OR. LLMWAVE .OR. LLTCWV) THEN
        ISENSOR = SATHDR(IOBS,MDBSSIA)
        ISATID  = SATHDR(IOBS,MDB_SATID_AT_SAT)
      ENDIF
    ENDIF

!*          2.6   INTERPOLATE FORECAST ERRORS IN THE HORIZONTAL

    CALL SUHIFCE(RZLON0E,RZDLONE,RZLATE,MNROWS,MLONE,ZOBLON,ZOBLAT,1,&
     & MNPTE,MNFLDS,RZEGRID,ZIGRID)  

!           2.7   PREPARE FOR INTERPOLATION IN THE VERTICAL
!                 -----------------------------------------

!*          2.8   MODEL SURFACE PRESSURE AT OBSERVATION LOCATION

!    WRITE(NULOUT,*)"IMAPOMM, IOBSTYPE sont dans gefger",IMAPOMM,ROBHDR(IOBS,MDBOTP)
    IF (NOBSPROFS(IOBTYP)==1) THEN
       ZMODPS = YGOMS(IMAPOMM)%SP
       ZMODPS=EXP(ZMODPS)
    ELSE
       ZMODPS = YGOMS_2D(IMAPOMM)%SP(1)
       ZMODPS=EXP(ZMODPS)
    ENDIF
!*
!           2.9   INITIALIZE VARIABLE LOOP PARAMETERS

    INLV = MLNKH2B(IOBS+1)-MLNKH2B(IOBS)
!*

!           2.10  LOOP OVER ALL THE VARIABLES

    DAT_LOOP: DO JNLV = MLNKH2B(IOBS),MLNKH2B(IOBS+1)-1
      ILEV = JNLV
      LLVINT = .FALSE.
      INLEVM = 1
      ZVALM(1) = RMDI
      ZVALM(2) = RMDI
      ZFGEMIN  = TINY(Z)

!           2.11  VARIABLE NAME

      ICMVNM = ROBODY(ILEV,MDBVNM)

!           2.12  TYPE OF VERTICAL COORDINATE AND ITS VALUE

      ICMVCO = ROBODY(ILEV,MDBVCO )
      IDEVN1 = ROBODY(ILEV,MDBDEV1)
      IDSTAT = ROBODY(ILEV,MDBDSTA)
      ZCMPPP(1,1) = ROBODY(ILEV,MDBPPP )
!     write(0,*)'GEFGER',MDBPRL,MDBVNM,JSET,KTSLOT,IOBS,IRET,ILEV,IOBTYP,ICDTYPE
      ZCMPRL = ROBODY(ILEV,MDBPRL )

!           2.13  TEST FOR VERTICAL COORDINATE

!              2.13.1  PRESSURE  AS A VERTICAL COORDINATE IS ALL RIGHT

      IF(ICMVCO == 1) THEN
        IF(ZCMPPP(1,1) == RMDI) CYCLE DAT_LOOP

!              2.13.2  HEIGHT AS A VERTICAL COORDINATE

      ELSEIF(ICMVCO == 2) THEN

!                 2.13.2.1  SFC PRESSURE OR SSMI WIND SPEED.
!                           USE MODEL SFC PRESSURE.

        IF(ICMVNM == NVNUMB(47) .OR. &
           & (IOBTYP == NSATEM .AND. ICMVNM == NVNUMB(49))) THEN  
          ZCMPPP(1,1) = ZMODPS
          ICMVCO = 1
        ENDIF

!                 2.13.2.2  GPSRO observations

        IF((ICMVNM == NVNUMB(76)).AND.(.NOT.LECMWF)) THEN
          ICMVCO = 1
        ENDIF

!              2.13.3  TOVS CHANNEL AS A VERTICAL COORDINATE

      ELSEIF(ICMVCO == 3) THEN

!                 2.13.3.1  NO VERTICAL INTERPOLATION WILL BE REQUIRED

        IF(ICMVNM == NVNUMB(50)) THEN
          ICMCH  = NINT(ZCMPPP(1,1))
          IF(LLRAD1C) THEN
            IF(ISENSOR == MSENSOR_METEOSAT) THEN
              IF(ICMCH == 1) ICMCH=12                ! Meteosat WV like HIRS-12
              IF(ICMCH == 2) ICMCH=8                 ! Meteosat IR like HIRS-8
            ELSEIF(ISENSOR == MSENSOR_MSG) THEN
              SELECT CASE (ICMCH)
              CASE(1)
                ICMCH=19                             ! MSG 3.7 like HIRS 19
              CASE(2,3)                              ! MSG WV Channels like
                ICMCH=12                             ! HIRS 12
              CASE(4,6,7)                            ! MSG split window + 8.7
                ICMCH=8                              ! like HIRS 8
              CASE(5)                                ! MSG Ozone channel
                ICMCH=9                              ! like HIRS 9
              CASE(8)                                ! MSG 13.4 channel
                ICMCH=7                              ! like HIRS 7
              END SELECT
            ELSEIF(ISENSOR == MSENSOR_MTSATIMG) THEN
              SELECT CASE (ICMCH)
              CASE(1,2)                              ! MTSAT split window
                ICMCH=8                              ! like HIRS 8
              CASE(3)                                ! MTSAT WV channel
                ICMCH=12                             ! HIRS 12
              CASE(4)
                ICMCH=19                             ! MTSAT 3.9 like HIRS 19
              END SELECT
            ELSEIF(ISENSOR == MSENSOR_VISSR) THEN
              SELECT CASE (ICMCH)
              CASE(1,2)                              ! VISSR split window
                ICMCH=8                              ! like HIRS 8
              CASE(3)                                ! VISSR WV channel
                ICMCH=12                             ! HIRS 12
              END SELECT
            ElSEIF(ISENSOR == MSENSOR_GOESIMG) THEN
              IF(ICMCH == 1) ICMCH=19                ! Goes 3.9 like HIRS-19
              IF(ICMCH == 2) ICMCH=12                ! Goes WV like HIRS-12
              IF(ICMCH == 3) ICMCH=8                 ! Goes 10.7 like HIRS-8
              IF(ICMCH == 4) ICMCH=8                 ! Goes 12.0 like HIRS-8
            ELSEIF(ISENSOR == MSENSOR_VTPR1 .OR. ISENSOR == MSENSOR_VTPR2) THEN
              IF(ICMCH == 7) ICMCH=12                ! VTPR WV like HIRS-12
            ELSEIF(ISENSOR == MSENSOR_HIRS .OR. ISENSOR == MSENSOR_MSU &
               & .OR.  ISENSOR == MSENSOR_SSU   .OR. ISENSOR == MSENSOR_AMSUA &
               & .OR.  ISENSOR == MSENSOR_AMSUB .OR. ISENSOR == MSENSOR_MHS) THEN  
              ICMCH=ICMCH+MSENSOR_CHANOFFL(ISENSOR)  ! Add channel offset
            ELSEIF(ISENSOR == MSENSOR_SSMI) THEN
              ICMCH=ICMCH+47                         ! Appended after AMSUB/MHS
            ELSEIF(ISENSOR == MSENSOR_SSMIS) THEN
              SELECT CASE (ICMCH)
               CASE(1,2,3,4)
                 ICMCH=29+ICMCH                      ! AMSU-A 3-6
               CASE(5,6,7)
                 ICMCH=30+ICMCH                      ! AMSU-A 8-10
               CASE(8)
                 ICMCH=44                            ! AMSU-B 2
               CASE(9)
                 ICMCH=47                            ! AMSU-B 5
               CASE(10)
                 ICMCH=46                            ! AMSU-B 4
               CASE(11)
                 ICMCH=45                            ! AMSU-B 3
               CASE(12)
                 ICMCH=49                            ! SSMI 2
               CASE(13)
                 ICMCH=48                            ! SSMI 1
               CASE(14)
                 ICMCH=50                            ! SSMI 3
               CASE(15)
                 ICMCH=52                            ! SSMI 5
               CASE(16)
                 ICMCH=51                            ! SSMI 4
               CASE(17)
                 ICMCH=53                            ! SSMI 6
               CASE(18)
                 ICMCH=54                            ! SSMI 7
               CASE(19,20,21)
                 ICMCH=0                             ! WIll default to 3.0 K
               CASE(22)
                 ICMCH=41                            ! AMSU-A 14
               CASE(23)
                 ICMCH=40                            ! AMSU-A 13
               CASE(24)
                 ICMCH=39                            ! AMSU-A 12
              END SELECT
            ELSEIF(ISENSOR == MSENSOR_AMSRE) THEN
              SELECT CASE (ICMCH)
               CASE(1,2,3,4)
                 ICMCH=0                             ! WIll default to 3.0 K
               CASE(5,6,7)
                 ICMCH=43+ICMCH                      ! SSMI 1-3
               CASE(8)
                 ICMCH=50                            ! SSMI 3
               CASE(9,10,11,12)
                 ICMCH=42+ICMCH                      ! SSMI 4-7
              END SELECT
            ELSEIF(ISENSOR == MSENSOR_TMI) THEN
              SELECT CASE (ICMCH)
               CASE(1,2)
                 ICMCH=0                             ! WIll default to 3.0 K
               CASE(3,4,5,6,7,8,9)
                 ICMCH=45+ICMCH                      ! SSMI 1-7
              END SELECT
            ELSE
              ICMCH=0                                ! WIll default to 10.0 K
            ENDIF
          ENDIF
          ICMVCO = 1
        ENDIF

!              2.13.4  SCATT CHANNEL AS A VERTICAL COORDINATE

      ELSEIF(ICMVCO == 4) THEN

!                 2.13.4.1  SCATT WINDS. USE MODEL SFC PRESSURE.

        IF(ICMVNM == NVNUMB(61) .OR. ICMVNM == NVNUMB(62)) THEN
          ZCMPPP(1,1) = ZMODPS
          ICMVCO = 1
        ENDIF
      ENDIF

!              2.13.5  THE REST SHOULD BE IN PRESSURE COORDINATE

      IF(ICMVCO /= 1) THEN
        CALL NGEDSTA(IDSTAT,INSTA,IDSTA)
        IDREJ  = IDSTA(3)  
        IDEV17 = IBITS(IDEVN1,ND1EVENT(17)-1,1)
        IF(IDREJ == 1 .AND. IDEV17 == 1) THEN
          CYCLE DAT_LOOP
        ELSE
!         DO NOT PRINT FOR DD AND FF (TOO BIG), AND THE REST
!         IF(ICMVNM == NVNUMB( 48) .OR.&
!           &ICMVNM == NVNUMB( 49)      ) CYCLE DAT_LOOP
!         WRITE(NULOUT,'('' *** UNEXPECTED VERTICAL COORDINATE'',&
!          &'' IN  GEFGER: '')')
!         WRITE(NULOUT,'('' '',2(F9.2,1X),3(I3,1X),F9.2,1x,2(i2,1x))')&
!          &ZOBLAT*RDEGREES,ZOBLON*RDEGREES,&
!          &IOBTYP,ICMVNM,ICMVCO,ZCMPPP(1,1),IDEV17,IDREJ
!         CALL FLUSH(NULOUT)
!         IF(IOBTYP /= 2)        THEN
!           CALL ABOR1(' GEFGER ')
!         ELSE
          CYCLE DAT_LOOP
!         ENDIF
        ENDIF
      ENDIF

!           2.14  SELECT VARIABLE TO BE INTERPOLATED

!              2.14.1 TEMPERATURE
      IF((ICMVNM == NVNUMB( 8) .OR. ICMVNM == NVNUMB(10) .OR.&
         & ICMVNM == NVNUMB(12)) .AND. LLT) THEN  
        IVAR = IVT
        LLVINT = .TRUE.
        ZFGEMIN=0.01_JPRB

!              2.14.2 U-COMPONENT
      ELSEIF((ICMVNM == NVNUMB( 1) .OR. ICMVNM == NVNUMB(56) .OR.&
         & ICMVNM == NVNUMB(62)) .AND. LLU) THEN  
        IVAR = IVU
        LLVINT = .TRUE.
        ZFGEMIN=0.01_JPRB

!              2.14.3 V-COMPONENT
      ELSEIF((ICMVNM == NVNUMB( 2) .OR. ICMVNM == NVNUMB(57) .OR.&
         & ICMVNM == NVNUMB(61)) .AND. LLV) THEN  
        IVAR = IVV
        LLVINT = .TRUE.
        ZFGEMIN=0.01_JPRB

!              2.14.4 SPECIFIC HUMIDITY
      ELSEIF(ICMVNM == NVNUMB(63) .AND. LLQ) THEN
        IVAR = IVQ
        LLVINT = .TRUE.

!              2.14.4b RELATIVE HUMIDITY
      ELSEIF(ICMVNM == NVNUMB(5) .AND. LLR) THEN
        IVAR = IVR
        LLVINT = .TRUE.

!              2.14.5 SURFACE PRESSURE, OR CONVERT FROM HEIGHT
      ELSEIF(ICMVNM == NVNUMB(47)) THEN
        IF(LLLNSP) THEN
          IVAR = IVLNSP
          LLVINT = .FALSE.
        ELSE
          IVAR = IVGH
          LLVINT = .TRUE.
        ENDIF
        ZFGEMIN=0.1_JPRB

!              2.14.6 TOTAL COLUMN OZONE
      ELSEIF(ICMVNM == NVNUMB(68) .AND. LLTCO3) THEN
        IVAR = IVTCO3
        LLVINT = .FALSE.

!              2.14.7 TOTAL COLUMN WATER VAPOUR
      ELSEIF(ICMVNM == NVNUMB(6) .AND. LLTCWV) THEN
        IVAR = IVTCWV
        LLVINT = .FALSE.

!              2.14.8 GEOPOTENTIAL HEIGHT

      ELSEIF(ICMVNM == NVNUMB( 3) .AND. LLGH) THEN
        IVAR = IVGH
        LLVINT = .TRUE.

!              2.14.9 THICKNESS

      ELSEIF(ICMVNM == NVNUMB( 4) .AND. LLGH) THEN
        IVAR = IVGH
        LLVINT = .TRUE.
        INLEVM = 2

!              2.14.10 TOVS RADIANCES

      ELSEIF(ICMVNM == NVNUMB(50) .AND. LLTBT) THEN
        IVAR = IVTBT
        LLVINT = .FALSE.
        ZFGEMIN=0.01_JPRB

!              2.14.11 AEROSOL OPTICAL DEPTH 
      ELSEIF(ICMVNM == NVNUMB(78)) THEN
        IVAR = 0 
        LLVINT = .FALSE.
        ZFGEMIN=0.01_JPRB

!              2.14.12 CLOUD OPTICAL DEPTH
      ELSEIF(ICMVNM == NVNUMB(85)) THEN
        IVAR = 0
        LLVINT = .FALSE.
        ZFGEMIN=0.01_JPRB

!              2.14.13 RATIO OF FINE MODE TO TOTAL OPTICAL DEPTH
      ELSEIF(ICMVNM == NVNUMB(86)) THEN
        IVAR = 0
        LLVINT = .FALSE.
        ZFGEMIN=0.01_JPRB

!              2.14.14 AEROSOL OPTICAL DEPTH (MULTI-CHANNEL)
      ELSEIF(ICMVNM == NVNUMB(87)) THEN
        IVAR = 0
        LLVINT = .FALSE.
        ZFGEMIN=0.01_JPRB

!              2.14.15 AEROSOL REFLECTANCE 
      ELSEIF(ICMVNM == NVNUMB(88)) THEN
        IVAR = 0
        LLVINT = .FALSE.
        ZFGEMIN=0.01_JPRB

!              2.14.12a HLoS WIND COMPONENT - same sigma_b as U-WIND

      ELSEIF(ICMVNM == NVNUMB(77) .AND. LLU) THEN
        IVAR = IVU
        LLVINT = .TRUE.
        ZFGEMIN=0.01_JPRB

!              2.14.12b HLoS WIND COMPONENT - try V-WIND if U-WIND absent

      ELSEIF(ICMVNM == NVNUMB(77) .AND. LLV) THEN
        IVAR = IVV
        LLVINT = .TRUE.
        ZFGEMIN=0.01_JPRB

      ENDIF

!           2.15  ALLOCATE AND FILL ARRAYS

      ILEVB=-1
      ILEVB2=-1
      IF(LLVINT) THEN
        DO JLEV=1,MNLEVS(IVAR)
          ZPRSFI(1,JLEV) = RZA(JLEV,IVAR)+ ZMODPS*RZB(JLEV,IVAR)
          ZVALH (1,JLEV) = ZIGRID(1,JLEV+IFLDOFF(IVAR))
          IF(ZPRSFI(1,JLEV) > ZCMPPP(1,1) .AND.&
           & ILEVB == -1                     ) ILEVB=JLEV  
          IF(INLEVM == 2) THEN
            IF(ZPRSFI(1,JLEV) > ZCMPRL .AND.&
             & ILEVB2 == -1                  ) ILEVB2=JLEV  
          ENDIF
        ENDDO
        ZVALH(1,0) = ZVALH(1,1)
        ZPRSFI(1,0) = 0.0_JPRB

!*          2.16  INTERPOLATE FORECAST ERRORS IN THE VERTICAL

        IF(ILEVB == -1) THEN
          ZVALM(1)=ZVALH (1,MNLEVS(IVAR))
        ELSE
          ILEVA=ILEVB-1
          ZW=(ZPRSFI(1,ILEVB)-ZCMPPP(1,1))/(ZPRSFI(1,ILEVB)-ZPRSFI(1,ILEVA))
          ZVALM(1)=ZVALH(1,ILEVB)+ZW*(ZVALH(1,ILEVA)-ZVALH(1,ILEVB))
        ENDIF

        IF(INLEVM == 2) THEN
          IF(ILEVB2 == -1) THEN
            ZVALM(2)=ZVALH (1,MNLEVS(IVAR))
          ELSE
            ILEVA2=ILEVB2-1
            ZW=(ZPRSFI(1,ILEVB2)-ZCMPRL)/(ZPRSFI(1,ILEVB2)-ZPRSFI(1,ILEVA2))
            ZVALM(2)=ZVALH(1,ILEVB2)+ZW*(ZVALH(1,ILEVA2)-ZVALH(1,ILEVB2))
          ENDIF
        ENDIF

!EA           ZPRSF0(1) = ZCMPPP(1,1)
!EA           IF(INLEVM == 2) ZPRSF0(2) = ZCMPRL
!EA           CALL SUVIFCE(ZVALH,ZPRSFI(1,1),1,1,MNLEVS(IVAR),
!EA  &                     ZVALM,ZPRSF0,INLEVM)

!*          2.18  ALTERNATIVE ESTIMATE OF BACKGROUND ERRORS

      ELSE

!              2.18.1 SPECIFIC HUMIDITY WITH AID OF MODEL TEMPERATURE

        IF(ICMVNM == NVNUMB(63) .AND. .NOT.LLQ) THEN
          IF(ZCMPPP(1,1) /= RMDI) THEN

!                 2.18.1.1   CALCULATE PRESSURE ON MODELS LEVELS

            ZMODPH(NFLEVG) = ZMODPS
            CALL GPPRE(1,1,1,NFLEVG,VAH,VBH,ZMODPH,ZMODPF)

!*                2.18.1.2 INTERPOLATE MODEL TEMPERATURE IN THE VERTICAL

            DO JLEV=1, NFLEVG
              ZMODT(JLEV) = YGOMUA(IMAPOMM)%T(JLEV)
            ENDDO
            ZMODT(0) = ZMODT(1)
            CALL SUVIFCE(ZMODT,ZMODPF,1,1,NFLEVG,ZTTT,ZCMPPP,1)

!*                2.18.1.3 CALCULATE SATURATION Q

            ZESOP = ES(ZTTT(1,1))/ZCMPPP(1,1)
            ZQS   = FOQS(ZESOP)
            IF(ABS(ZOBLAT(1)*RDEGREES)  >=  30.0_JPRB) THEN
              ZVALM(1) = ZQS*0.10_JPRB
            ELSE
              ZVALM(1) = ZQS*0.15_JPRB
            ENDIF
          ELSE
            ZVALM(1) = RMDI
          ENDIF

!              2.18.2 RELATIVE HUMIDITY ERROR IS JUST A SIMPLE CONSTANT
!                     If not RH found on file
!                     RH_2m = NVNUMB(7) is always set here.

        ELSEIF(ICMVNM == NVNUMB(5) .OR. ICMVNM == NVNUMB(7)) THEN
          IF(ABS(ZOBLAT(1)*RDEGREES)  >=  30.0_JPRB) THEN
            ZVALM(1) = 0.10_JPRB
          ELSE
            ZVALM(1) = 0.15_JPRB
          ENDIF

!              2.18.3 TOVS BRIGHTNESS TEMPERATURE
!                     CHECK THAT THE CHANNEL NUMBER IS THERE

        ELSEIF(ICMVNM == NVNUMB(50)) THEN
          IF(ICDTYPE == NSSMI) THEN

            ! All-sky radiances - mean normalised 37GHz polarisation difference
            ZP37 = 0.5 * (SATHDR (IOBS,MDB_OB_P37) &
               & + SATHDR (IOBS,MDB_FG_P37)) 

            ! Find satellite index in mwave_id array
            LLSATFOUND=.FALSE.
            DO JSAT=1,N_SATSENS
              IF ( ISATID == MWAVE_ID(JSAT) % BUFR_ID ) THEN
                LLSATFOUND=.TRUE.
                EXIT
              ENDIF
            ENDDO
            IF (.NOT. LLSATFOUND) THEN
              WRITE(NULERR,*) ' GEFGER all sky satellite ',ISATID,' unknown in : ', &
                & MWAVE_ID(:) % BUFR_ID
              CALL ABOR1(' GEFGER all-sky (mwave) FG error estimate - sensor unknown ')
            ENDIF

            ICH = NINT(ZCMPPP(1,1))
            IF ( ICH <= MWAVE_ID (JSAT) % NCHANNELS) THEN
              ! Estimate FG error 
              ZVALM(1) = MWAVE_ERROR_MODEL (ZP37, JSAT, ICH, LDOBS = .FALSE.)
            ELSE
              ZVALM(1) = 3._JPRB
            ENDIF
          ELSE

            ! Normal satellite radiances
          ZVALM(1) = 3._JPRB        ! Default value
          IF(LLTBT) THEN
            ITLEV=0
            DO JLEV=1,MNLEVS(IVAR)
              ICH=NINT(RZA(JLEV,IVAR))
              IF(ICH == ICMCH) ITLEV=JLEV
            ENDDO
            IF(ITLEV /= 0) ZVALM(1)=ZIGRID(1,ITLEV+IFLDOFF(IVAR))
          ENDIF

          ENDIF
!              2.18.4 SSMI TCWV. There IS now FGE for total PWC.
        ELSEIF(ICMVNM == NVNUMB(6) .AND. LLTCWV) THEN
          IF(ICDTYPE == NTCWC) THEN
            ZVALM(1) = ZIGRID(1,1+IFLDOFF(IVAR))
          ENDIF

!              2.18.5 TOTAL COLUMN OZONE (No FGE for ozone layers, yet)
        ELSEIF(ICMVNM == NVNUMB(68) .AND. LLTCO3) THEN
          ZVALM(1) = ZIGRID(1,1+IFLDOFF(IVAR))

!              2.18.6 AEROSOL OPTICAL DEPTH
        ELSEIF(ICMVNM == NVNUMB(78)) THEN
          ZVALM(1)=3._JPRB 

!              2.18.7 GROUND-BASED GPS
        ELSEIF(ICMVNM == NVNUMB(75)) THEN
          ZVALM(1) = 0.0124_JPRB

!              2.18.7 GPS RO
        ELSEIF ((ICMVNM == NVNUMB(76)).AND.(.NOT.LECMWF)) THEN
          ZOBER = ROBODY(ILEV,MDBFOE)
          IF (ZOBER == RMDI) THEN
            ZVALM(1) = RMDI
          ELSE
            ZVALM(1) = ZOBER
          ENDIF

!              2.18.7  RADAR REFLECTIVITY
        ELSEIF(ICMVNM == NVNUMB(74)) THEN
        WRITE(NULOUT,*) 'ICMVNM',ICMVNM,'NVNUMB(74)',NVNUMB(74)
           ZVALM(1) = 3.0_JPRB
           WRITE(NULOUT,*) 'radar reflectivity: SETTING FG ERROR TO 3'
         ENDIF
      ENDIF

!*          2.19  CONVERT THE UNITS OF FORECAST ERROR

!*             2.19.1 HEIGHT TO GEOPOTENTIAL
      IF(ICMVNM == NVNUMB( 3) .OR. ICMVNM == NVNUMB( 4)) THEN
        IF(ZVALM(1) /= RMDI)ZVALM(1) = ZVALM(1)*RG
        IF(INLEVM == 2.AND. ZVALM(2) /= RMDI)ZVALM(2) = ZVALM(2)*RG

!              2.19.2 SURFACE PRESSURE
      ELSEIF(ICMVNM == NVNUMB(47)) THEN
        IF(LLLNSP) THEN
          ZVALM(1) = ZMODPS*ZIGRID(1,1+IFLDOFF(IVAR))
        ELSE
          IF(ZVALM(1) /= RMDI) ZVALM(1) = ZVALM(1)*RG*1.225_JPRB
        ENDIF
      ENDIF

!           2.20  THICKNESS Approximate formula
      IF(ICMVNM == NVNUMB( 4) .AND. LLGH) THEN
        ZEXP = ((LOG(ZCMPPP(1,1))-LOG(ZCMPRL))/LOG(2.0_JPRB))**2
        ZFGER = ZVALM(1)*ZVALM(1) + ZVALM(2)*ZVALM(2)&
         & - 2*EXP(-ZEXP/2.0_JPRB)*ZVALM(1)*ZVALM(2)  
        ZVALM(1)=SQRT(ZFGER)
      ENDIF

!*          2.21  INSERT THE INTERPOLATED BACKGROUND ERROR INTO ODB

      IF(ZVALM(1) == RMDI) THEN
        ROBODY(ILEV,MDBFGE) = RMDI
      ELSE
        ROBODY(ILEV,MDBFGE) = MAX(ZVALM(1),ZFGEMIN)
      ENDIF

!*          2.22  CLOSE THE LOOPS

!*          2.221 DATA LOOP

    ENDDO DAT_LOOP

!*          2.222 OBS LOOP

  ENDDO OBS_LOOP

! IPREVSL = IPREVSL + OBSET(ITID)%MLNSET(JSET)

  CALL PUTDB('SUFGER', 1, IRET, INFO, 0, ZINFO, 0)

ENDDO SET_LOOP

!*          2.23  RELEASE THE SPACE

DEALLOCATE (ZIGRID)
DEALLOCATE (ZVALH )
DEALLOCATE (ZPRSFI)
IF(LSCDPR) THEN
  WRITE(NULOUT,9991) 'ZIGRID   '
  WRITE(NULOUT,9991) 'ZVALH    '
  WRITE(NULOUT,9991) 'ZPRSFI   '
ENDIF

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
9991 FORMAT(1X,'ARRAY ',A10,' DEALLOCATED ')

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GEFGER',1,ZHOOK_HANDLE)
END SUBROUTINE GEFGER
