SUBROUTINE INISSMIP

!**** SUBROUTINE INISSMIP - INITIALISE SSMI PROCESSING

!        D. VASILJEVIC    ECMWF   06/09/1994

!**   PURPOSE
!     -------

!        SSMI PROCESSING INITIALISATION

!**   INTERFACE
!     ---------

!        CALL INISSMIP

!        INISSNIP IS CALLED BY SUOBS

!**   METHOD
!     ------

!        NECESSARY SSMI PARAMETERS/ARRAYS ARE DEFINED.

!**   EXTERNALS
!     ---------

!        SSMIMAS - CREATE THINNING MASK FOR SSM/I
!        ENDRUN  - ABORT

!**   MODIFICTIONS
!     ------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARSSMI
USE YOMSSMI
USE YOMLUN   , ONLY : NULOUT   ,NULNAM

IMPLICIT NONE

#include "namssmi.h"

INTEGER(KIND=JPIM) :: IERR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "ssmimas.intfb.h"

!*
!        1.       GET NAMELIST PARAMETERS
!                 -----------------------

IF (LHOOK) CALL DR_HOOK('INISSMIP',0,ZHOOK_HANDLE)
LSSMITHI = .FALSE.

NMXSMLI  = JPMXSMLI
NMXSMEL  = JPMXSMEL

NSSMISKI = 4

WRITE(NULOUT,'('' READ NAMSSMI'')')
CALL FLUSH(NULOUT)
CALL POSNAM(NULNAM,'NAMSSMI')
READ(NULNAM,NAMSSMI)
!*
!-----------------------------------------------------------------------

!        2.       CROSS-CHECK VALUES
!                 ------------------

IF(NMXSMLI > JPMXSMLI)      THEN
  WRITE(NULOUT,'(''0ERROR IN INISSMIP'')')
  WRITE(NULOUT,'('' EXPECTED NMXSMLI: '',I10)') JPMXSMLI
  WRITE(NULOUT,'('' FOUND NMXSMLI   : '',I10)') NMXSMLI
  CALL ABOR1('SUBROUTINE INISSMIP')
ENDIF
IF(NMXSMEL > JPMXSMEL)      THEN
  WRITE(NULOUT,'(''0ERROR IN INISSMIP'')')
  WRITE(NULOUT,'('' EXPECTED NMXSMEL: '',I10)') JPMXSMEL
  WRITE(NULOUT,'('' FOUND NMXSMEL   : '',I10)') NMXSMEL
  CALL ABOR1('SUBROUTINE INISSMIP')
ENDIF
!*
!-----------------------------------------------------------------------

!        3.       INITIALISE ARRAYS
!                 -----------------

IERR = 0
CALL SSMIMAS(NSSMISKI,NMXSMLI ,LSSMILMA,&
 & NMXSMEL ,LSSMIEMA,&
 & IERR                       )  
IF(IERR /= 0)        THEN
  WRITE(NULOUT,'(''0ERROR IN INISSMIP'')')
  WRITE(NULOUT,'('' IERR ='',I10)') IERR
  CALL ABOR1('SUBROUTINE INISSMIP')
ENDIF
!*
!-----------------------------------------------------------------------

!        4.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('INISSMIP',1,ZHOOK_HANDLE)
END SUBROUTINE INISSMIP
