SUBROUTINE LEVEL1CGEOS_OB (KTSLOT, KPOOLNO)

!**** SUBROUTINE LEVEL1CGEOS_OB - PROCESS LEVEL 1C AND GEOSTATIONARY RADIANCES

!          DRASKO VASILJEVIC ECMWF          01/MAR/2001

!*    METHOD
!     ------

!        THE ODB IS SCANNED FOR LEVEL 1C AND GEOSTATIONARY RADIQNCES AND
!     MAKECMA FUNCTIONS ARE CARRIED OUT.

!*    INTERFACE
!     ---------

!     ***CALL*LEVEL1CGEOS_OB (KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!*    EXTERNALS
!     ---------

!        GETDB    - GET DB
!        PUTDB    - STORE DB
!        CMOCTMAP - GET OBS./CODE TYPE SEQUENCE NO.s
!        RAD1CIN  - PROCESS INPUT LEVEL 1Cs
!        GEOSRIN  - PROCESS INPUT GEOS. radiaces
!        ABOR1    - ABNORMAL TERMINATION

!        LEVEL1CGEOS_OB IS CALLED BY SUBROUTINE MKCMARPL

!**   MODIFICATIONS
!     -------------

!     23/03/05 M. Szyndel Added FY-2 and MTSAT.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMCOCTP , ONLY :       NSATEM   ,NGTHRB   
USE YOMTVRAD , ONLY : MSENSOR_METEOSAT   ,MSENSOR_GOESIMG, MSENSOR_MSG, &
     &                MSENSOR_MTSATIMG, MSENSOR_VISSR
USE YOMLUN   , ONLY : NULOUT   

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)   :: KTSLOT
INTEGER(KIND=JPIM), INTENT(IN)   :: KPOOLNO
INTEGER(KIND=JPIM) :: INFO(1), IRET

INTEGER(KIND=JPIM) :: ILEN, IOBTYP, ICDTYPE, IOBS
INTEGER(KIND=JPIM) :: IOBSQNO,ICDSQNO
INTEGER(KIND=JPIM) ::  JOBS, ISENSOR

LOGICAL :: LLMESAGE, LLTAKEN

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cmoctmap.intfb.h"
#include "geosrin.intfb.h"
#include "rad1cin.intfb.h"

#include "openmp_obs.h"

!*       1.       LEVEL1C/GEOS OBS
!                 ----------------

!*          1.1   INITIALIZE DB (LOCALL; ALL LEVEL1C/GEOS REPs)

IF (LHOOK) CALL DR_HOOK('LEVEL1CGEOS_OB',0,ZHOOK_HANDLE)
IRET = 0
CALL GETDB('LEVEL1CGEOS',1,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, KPOOLNO, -1, -1, -1)  
IF(IRET == 0) THEN
  WRITE(NULOUT,'(''     NO REPORTS (LEVEL1C/GEOS) AVAILABLE '')')
ELSE
  WRITE(NULOUT,'(''     REPORTS (LEVEL1C/GEOS) FOUND: '',I10)') IRET
ENDIF
CALL FLUSH(NULOUT)
ILEN = IRET

!*          1.2   LOOP OVER ALL THE LEVEL1C/GEOS OBSERVATIONS

OBS_LOOP: DO JOBS = 1, ILEN
  IOBS = JOBS
  IOBTYP  = ROBHDR(IOBS,MDBOTP)
  ICDTYPE = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  ISENSOR = ROBHDR(IOBS,MDBSSIA)
  LLMESAGE= .FALSE.
  LLTAKEN = .FALSE.
  CALL CMOCTMAP('CODETOSQ',NULOUT,IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO)

!*          1.3   SATEM

  IF(IOBTYP == NSATEM )              THEN

!*          1.3.1 LEVEL1C/GEOS

    IF(ICDTYPE == NGTHRB )             THEN

!*          1.3.2 GEOS

      IF(ISENSOR == MSENSOR_METEOSAT .OR. ISENSOR == MSENSOR_MSG .OR. &
         & ISENSOR == MSENSOR_GOESIMG .OR. ISENSOR == MSENSOR_MTSATIMG .OR. &
         & ISENSOR == MSENSOR_VISSR      )   THEN  
        CALL GEOSRIN(IOBS,&
         & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
         & LLMESAGE,LLTAKEN)  
      ELSE
        CALL RAD1CIN(IOBS,&
         & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
         & LLMESAGE,LLTAKEN)  
      ENDIF

!*          1.3.3 UNKNOWN LEVEL1C/GEOS

    ELSE
      WRITE(NULOUT,'(''0ERROR IN SUBROUTINE LEVEL1CGEOS_OB'')')
      WRITE(NULOUT,'('' UNKNOWN LEVEL1C/GEOS CODE TYPE'',I10)') ICDTYPE
      CALL ABOR1(' LEVEL1CGEOS_OB; WRONG LEVEL1C/GEOS CODE TYPE ')
    ENDIF

!*          1.2.11UNKNOWN SATOB OBS TYPE

  ELSE
    WRITE(NULOUT,'(''0ERROR IN SUBROUTINE LEVEL1CGEOS_OB'')')
    WRITE(NULOUT,'('' UNKNOWN SATEM OBS TYPE'',I10)') IOBTYP
    CALL ABOR1('SUBROUTINE LEVEL1CGEOS_OB; WRONG SATEM OBS TYPE ')
  ENDIF
ENDDO OBS_LOOP

!           1.3   PUT THE DB AWAY

CALL PUTDB('LEVEL1CGEOS',1,IRET,INFO,0,ZINFO,0)

!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('LEVEL1CGEOS_OB',1,ZHOOK_HANDLE)
END SUBROUTINE LEVEL1CGEOS_OB
