SUBROUTINE LNDSYIN(KSYNP,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE LNDSYIN - PROCESS INPUT LAND SYNOP

!        D. VASILJEVIC     ECMWF     13/12/00 (original 04/07/94)

!     PURPOSE
!     -------

!        PROCESS INPUT LAND SYNOP

!**   INTERFACE
!     ---------

!        CALL LNDSYIN(KSYNP,
!                     KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                     LDMESAGE,LDTAKEN)
!     WHERE:
!            KSYNP   = REPORT POINTER
!            KOBTYP  = OBS. TYPE
!            KCDTYP  = CODE TYPE
!            KOBSQNO = OBS. TYPE SQ. NO.
!            KCDSQNO = CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT UPDATED) SWITCH  

!        LNDSYIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        ERRSTAT  - OBS. ERROR
!        SYNOPBE  - UPDATE SYNOP BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE
!        OBSCHAR  - OBS. CHARACTERISTICS

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMCMHDR , ONLY : NALFLBP  ,NFLGBP   ,NFLGOC
USE YOMCMBDY , ONLY :  NCMBVFBP ,NCMBFLBP ,NCMBFLOC
USE YOMANCS  , ONLY : NMDI     ,RMDI     ,RDEGREES ,RADIANS  ,EPSMAC   ,RPAPM    ,&
 & RKELVN   ,RREDPEW1 ,&
 & RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES    ,RTT      ,RUPMLIM  ,RDWMLIM    
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMEV  , ONLY : ND1EVENT ,ND2EVENT
USE YOMNMCOD , ONLY : NPRESVC  ,NHEIGVC  ,NPRESCD
USE YOMMKODB , ONLY : LERRPERS ,NMKCMVSE ,&
 & LTENDCOR ,LTOTREM ,&
 & L_LAND_PS_BIAS
USE YOMLIM   , ONLY :       NFLLIMOP  ,&
 & RRHLIM   ,RRHTLIM  
USE YOMOERR  , ONLY : RHERR1   ,RHERR2   ,RHERR3
USE YOMERR   , ONLY : LRHERRMO
USE YOMCST   , ONLY : RG
USE YOMLUN   , ONLY : NULOUT   
USE LOCAL_TRAFOS

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSYNP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT) :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE,&
 & IOBTIME, IPFLG,&
 & IVNM, IPCODE, IREPPRECODE, IREPGEOCODE,&
 & JDDFF, JPs, IPs, JZ, IZ, IVECFL, IVERCO,&
 & IGFLG, IALFLG, IUWINDF, IVWINDF, IUSHIPF, IVSHIPF,&
 & IGMSLFLG, IGSTLFLG, IREPPREFLG, IREPGEOFLG,&
 & IPRESCD, &
 & JR,IRRFLG, JDD, JFF, &
 & IR, IRA6FLG, IRSDFLG, IRLQFLG, &
 & ISTID, INSTYP, IREPST  
INTEGER(KIND=JPIM) :: IPMSLFLG, IPSTAFLG

INTEGER(KIND=JPIM) :: IDD, &
 & IDDSTAT, IDDEVEN1, IDDEVEN2, IDDRDBFL, IDDANFLG, &
 & IDDPFLG, IDDFLG  
INTEGER(KIND=JPIM) :: IFF, &
 & IFFSTAT, IFFEVEN1, IFFEVEN2, IFFRDBFL, IFFANFLG, &
 & IFFPFLG, IFFFLG  
INTEGER(KIND=JPIM) :: IU10M, JU10M, &
 & IUUSTAT, IUUEVEN1, IUUEVEN2, IUURDBFL, IUUANFLG, &
 & IUUPFLG, IUUFLG  
INTEGER(KIND=JPIM) :: IV10M, JV10M, &
 & IVVSTAT, IVVEVEN1, IVVEVEN2, IVVRDBFL, IVVANFLG, &
 & IVVPFLG, IVVFLG  
INTEGER(KIND=JPIM) :: JT2M, IT2M, &
 & ITTSTAT, ITTEVEN1, ITTEVEN2, ITTRDBFL, ITTANFLG, &
 & ITTPFLG, ITTFLG  
INTEGER(KIND=JPIM) :: IZZSTAT, IZZEVEN1, IZZEVEN2, IZZRDBFL, IZZANFLG, &
 & IZZPFLG, IZZFLG  
INTEGER(KIND=JPIM) :: IPSSTAT, IPSEVEN1, IPSEVEN2, IPSRDBFL, IPSANFLG, &
 & IPSPFLG, IPSFLG  
INTEGER(KIND=JPIM) :: JRH2M, IRH2M, &
 & IRHSTAT, IRHEVEN1, IRHEVEN2, IRHRDBFL, IRHANFLG, &
 & IRHPFLG, IRHFLG  
INTEGER(KIND=JPIM) :: JTD2M, ITD2M, &
 & ITDSTAT, ITDEVEN1, ITDEVEN2, ITDRDBFL, ITDANFLG, &
 & ITDPFLG, ITDFLG  
INTEGER(KIND=JPIM) :: JPT, IPT, &
 & IPTSTAT, IPTEVEN1, IPTEVEN2, IPTRDBFL, IPTANFLG, &
 & IPTPFLG, IPTFLG  
INTEGER(KIND=JPIM) :: JSR, ISR, &
 & ISRSTAT, ISREVEN1, ISREVEN2, ISRRDBFL, ISRANFLG, &
 & ISRPFLG, ISRFLG  
INTEGER(KIND=JPIM) :: JSNOW, ISNOW, &
 & ISNSTAT, ISNEVEN1, ISNEVEN2, ISNRDBFL, ISNANFLG, &
 & ISNPFLG, ISNFLG  
INTEGER(KIND=JPIM) :: JRD, IRD, &
 & IRDSTAT, IRDEVEN1, IRDEVEN2, IRDRDBFL, IRDANFLG, &
 & IRDPFLG, IRDFLG  
INTEGER(KIND=JPIM) :: JLR, ILR, &
 & ILRSTAT, ILREVEN1, ILREVEN2, ILRRDBFL, ILRANFLG, &
 & ILRPFLG, ILRFLG  
INTEGER(KIND=JPIM) :: IAUX, JALL, &
 & IAUSTAT, IAUEVEN1, IAUEVEN2, IAURDBFL, IAUANFLG, &
 & IAUPFLG, IAUFLG  , IAUVERCO  
INTEGER(KIND=JPIM) :: IDBRDFL

LOGICAL :: LLFAIL, LLREMOVE, LLSHIP, LLTEND, LLSHIPC, LLDONEBCH
LOGICAL :: LLMSLP, LLSTAP, LLREPPRE, LLREPGEO
LOGICAL ::  LLHIGHSY
LOGICAL :: LLPs, LLT2M, LLTd2M, LLDD, LLFF, LLU10m, LLV10m, &
 & LLRH2m, LLPt, LLSD, LLRR, LLRt, LLSf, LLZ  

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, &
 & ZPRES, ZTIMDIF, &
 & ZWEIGHT, &
 & ZVERCO, ZPRESAU, ZHEAUX, ZGEOP, ZGEOPMSL, ZGEOPSTL, &
 & ZALT  ,&
 & ZUWIND, ZVWIND, ZSHIPU, ZSHIPV, ZAUXPT, &
 & ZR, ZRAIN6HR, ZRAINSD, &
 & ZRAINLQ, ZI, ZREPPRE, ZREPGEO  
REAL(KIND=JPRB) :: ZPMSL, ZPSTA 
REAL(KIND=JPRB) :: ZDIR, ZDDPRES, ZDDPRL, &
 & ZDDPERR, ZDDOERR, ZDDFERR, ZDDRERR, &
 & ZDDFGER, ZDDFGCC1, ZDDFGCC2, &
 & ZDDFGDEP, ZDDANDEP  
REAL(KIND=JPRB) :: ZSPD, ZFFPRES, ZFFPRL, &
 & ZFFPERR, ZFFOERR, ZFFFERR, ZFFRERR, &
 & ZFFFGER, ZFFFGCC1, ZFFFGCC2, &
 & ZFFFGDEP, ZFFANDEP  
REAL(KIND=JPRB) :: ZU, ZUUPRES, ZUUPRL, &
 & ZUUPERR, ZUUOERR, ZUUFERR, ZUURERR, &
 & ZUUFGER, ZUUFGCC1, ZUUFGCC2, &
 & ZUUFGDEP, ZUUANDEP  
REAL(KIND=JPRB) :: ZV, ZVVPRES, ZVVPRL, &
 & ZVVPERR, ZVVOERR, ZVVFERR, ZVVRERR, &
 & ZVVFGER, ZVVFGCC1, ZVVFGCC2, &
 & ZVVFGDEP, ZVVANDEP  
REAL(KIND=JPRB) :: ZT2M, ZTTPRES, ZTTPRL, &
 & ZTTPERR, ZTTOERR, ZTTFERR, ZTTRERR, &
 & ZTTFGER, ZTTFGCC1, ZTTFGCC2, &
 & ZTTFGDEP, ZTTANDEP  
REAL(KIND=JPRB) :: ZTD2M, ZTDPRES, ZTDPRL, &
 & ZTDPERR, ZTDOERR, ZTDFERR, ZTDRERR, &
 & ZTDFGER, ZTDFGCC1, ZTDFGCC2, &
 & ZTDFGDEP, ZTDANDEP  
REAL(KIND=JPRB) :: ZRH2M, ZRHPRES, ZRHPRL, &
 & ZRHPERR, ZRHOERR, ZRHFERR, ZRHRERR, &
 & ZRHFGER, ZRHFGCC1, ZRHFGCC2, &
 & ZRHFGDEP, ZRHANDEP  
REAL(KIND=JPRB) :: ZPT, ZPTPRES, ZPTPRL, &
 & ZPTPERR, ZPTOERR, ZPTFERR, ZPTRERR, &
 & ZPTFGER, ZPTFGCC1, ZPTFGCC2, &
 & ZPTFGDEP, ZPTANDEP  
REAL(KIND=JPRB) :: ZSN, ZSNPRES, ZSNPRL, &
 & ZSNPERR, ZSNOERR, ZSNFERR, ZSNRERR, &
 & ZSNFGER, ZSNFGCC1, ZSNFGCC2, &
 & ZSNFGDEP, ZSNANDEP  
REAL(KIND=JPRB) :: ZRD, ZRDPRES, ZRDPRL, &
 & ZRDPERR, ZRDOERR, ZRDFERR, ZRDRERR, &
 & ZRDFGER, ZRDFGCC1, ZRDFGCC2, &
 & ZRDFGDEP, ZRDANDEP  
REAL(KIND=JPRB) :: ZSR, ZSRPRES, ZSRPRL, &
 & ZSRPERR, ZSROERR, ZSRFERR, ZSRRERR, &
 & ZSRFGER, ZSRFGCC1, ZSRFGCC2, &
 & ZSRFGDEP, ZSRANDEP  
REAL(KIND=JPRB) :: ZZZ, ZZZPRES, ZZZPRL, &
 & ZZZPERR, ZZZOERR, ZZZFERR, ZZZRERR, &
 & ZZZFGER, ZZZFGCC1, ZZZFGCC2, &
 & ZZZFGDEP, ZZZANDEP  
REAL(KIND=JPRB) :: ZPS, ZPSPRES, ZPSPRL, &
 & ZPSPERR, ZPSOERR, ZPSFERR, ZPSRERR, &
 & ZPSFGER, ZPSFGCC1, ZPSFGCC2, &
 & ZPSFGDEP, ZPSANDEP, ZPSBIAS  
REAL(KIND=JPRB) :: ZLR, ZLRPRES, ZLRPRL, &
 & ZLRPERR, ZLROERR, ZLRFERR, ZLRRERR, &
 & ZLRFGER, ZLRFGCC1, ZLRFGCC2, &
 & ZLRFGDEP, ZLRANDEP  
REAL(KIND=JPRB) :: ZAU, ZAUPRES, ZAUPRL, &
 & ZAUPERR, ZAUOERR, ZAUFERR, ZAURERR, &
 & ZAUFGER, ZAUFGCC1, ZAUFGCC2, &
 & ZAUFGDEP, ZAUANDEP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "errstat.intfb.h"
#include "finoerr.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"
#include "ptendcor.intfb.h"
#include "synopbe.intfb.h"
#include "z2picao.intfb.h"

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET
!                       ------

IF (LHOOK) CALL DR_HOOK('LNDSYIN',0,ZHOOK_HANDLE)
ZLAT     = ROBHDR(KSYNP,MDBLAT) * RDEGREES
ZLON     = ROBHDR(KSYNP,MDBLON) * RDEGREES
ZALT     = ROBHDR(KSYNP,MDBALT)
IALFLG   = IBITS(INT(ROBHDR(KSYNP,MDBRFL)),NALFLBP+NFLGBP,NFLGOC)
IOBDATE  = ROBHDR(KSYNP,MDBDAT)
IOBTIME  = ROBHDR(KSYNP,MDBETM)
CLSTID   = TRANSFER(ROBHDR(KSYNP,MDBSID), CLSTID)
ISTID    = 0
IREPST   = ROBHDR(KSYNP,MDBRST)

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KSYNP,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KSYNP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.2       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.2         BODY PARAMETERS

LLPs   = .FALSE.
LLT2M  = .FALSE.
LLTd2M = .FALSE.
LLDD   = .FALSE.
LLFF   = .FALSE.
LLU10m = .FALSE.
LLV10m = .FALSE.
LLRH2m = .FALSE.
LLPt   = .FALSE.
LLSD   = .FALSE.
LLRR   = .FALSE.
LLRt   = .FALSE.
LLSf   = .FALSE.
LLZ    = .FALSE.
ALL_DATA_LOOP: DO JALL = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1

!*          3.2.1       WE ACCEPT: Ps, T2m, Td2m, DD, FF, U10m, V10m,
!                                  Rh2m, Pt, Sd, RR, Rt, Sf AND Z

  IF(     ROBODY(JALL  ,MDBVNM) == NVNUMB( 47)) THEN
    LLPs    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 10)) THEN
    LLT2M   = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 11)) THEN
    LLTd2M  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 48)) THEN
    LLDD    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 49)) THEN
    LLFF    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 56)) THEN
    LLU10m  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 57)) THEN
    LLV10m  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB(  7)) THEN
    LLRH2m  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 13)) THEN
    LLPt    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 25)) THEN
    LLSD    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 33)) THEN
    LLRR    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 34)) THEN
    LLRt    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 46)) THEN
    LLSf    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB(  3)) THEN
    LLZ     = .TRUE.
    GO TO 322
  ELSE
    IAUX = JALL
    IVNM = ROBODY(IAUX  ,MDBVNM)
!    WRITE(NULOUT,'('' LNDSYIN; UNEXPECTED VARIABLE: '',I10)') IVNM

!*          3.2.1.0     SET/GET

    CALL GETSETE(IAUX ,&
     & ZAUPRES,ZAUPRL,ZAU ,&
     & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
     & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
     & ZAUFGDEP,ZAUANDEP,&
     & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
     & IAUFLG,IAUPFLG)  

    ZAUPRES = ROBODY(IAUX,MDBPPP)
    IAUVERCO= ROBODY(IAUX,MDBVCO)

!*          3.2.1.2     VARIABLE

    ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.2.1.3     ERROR STATISTICS

!*          3.2.1.4     STATUS, EVENTS, FLAGS

    IAUSTAT = ICHSTAT_REJECT( IAUSTAT )

!*          3.2.1.5     DEPARTURES

!*          3.2.1.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IAUX    ,&
     & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
     & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
     & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
     & ZAUFGDEP,ZAUANDEP,&
     & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG, &
     & IPRESCD                                       )  
  ENDIF

!*          3.2.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

  322 CONTINUE

!*          3.2.2.1     PRESSURE

  3221 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 3222  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF

!*          3.2.2.2     VARIABLE

  3222 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3223  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.2.2.3     ERROR STATISTICS

  3223 CONTINUE
  ROBODY(JALL  ,MDBOER) = RMDI
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.2.3       CLOSE ALL DATA LOOP

ENDDO ALL_DATA_LOOP

!*          3.2.3       CHECK IF ALL PRESENT

IF(LLPs   .AND. &
 & LLT2M  .AND. &
 & LLTd2M .AND. &
 & LLDD   .AND. &
 & LLFF   .AND. &
 & LLU10m .AND. &
 & LLV10m .AND. &
 & LLRH2m .AND. &
 & LLPt   .AND. &
 & LLSD   .AND. &
 & LLRR   .AND. &
 & LLRt   .AND. &
 & LLSf   .AND. &
 & LLZ           ) GO TO 324  
WRITE(NULOUT,'('' LNDSYIN; '')')
WRITE(NULOUT,'(''          LLPs   :'',L1)') LLPs
WRITE(NULOUT,'(''          LLT2M  :'',L1)') LLT2M
WRITE(NULOUT,'(''          LLTd2M :'',L1)') LLTd2M
WRITE(NULOUT,'(''          LLDD   :'',L1)') LLDD
WRITE(NULOUT,'(''          LLFF   :'',L1)') LLFF
WRITE(NULOUT,'(''          LLU10m :'',L1)') LLU10m
WRITE(NULOUT,'(''          LLV10m :'',L1)') LLV10m
WRITE(NULOUT,'(''          LLRH2m :'',L1)') LLRH2m
WRITE(NULOUT,'(''          LLPt   :'',L1)') LLPt
WRITE(NULOUT,'(''          LLSD   :'',L1)') LLSD
WRITE(NULOUT,'(''          LLRR   :'',L1)') LLRR
WRITE(NULOUT,'(''          LLRt   :'',L1)') LLRt
WRITE(NULOUT,'(''          LLSf   :'',L1)') LLSf
WRITE(NULOUT,'(''          LLZ    :'',L1)') LLZ

!*          3.2.4       CHECK IF LOW/HIGH LEVEL SYNOP
!                       - IF MSL PRESSURE PRESENT ==> LOW LEVEL SYNOP

324 CONTINUE
LLHIGHSY = .FALSE.
Ps00_LOOP: DO JPs = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPs  ,MDBVNM) == NVNUMB( 47)) THEN
    IF(ROBODY(JPs  ,MDBSYPC) == NPRESCD( 1)) GO TO 325
  ENDIF
ENDDO Ps00_LOOP

LLHIGHSY = .TRUE.

!*          3.2.5       ?

325 CONTINUE

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (OUTPUT TO ODB)
!                       --------------------------------

ZWEIGHT = 1.0_JPRB

!*          4.0         DECIDE ON VERTICAL COORDINATE; 
!                       CHECK IF PRESSURE PRESENT; WIND AND
!                       TEMPERATURE ARE DONE ANYWAY BUT ASSIGN
!                       VERTICAL COOORDINATE AS HEIGHT SET TO 0.

ZPRES      = RMDI
IPFLG      = 0
IPRESCD    = NMDI

ZGEOP      = RMDI
IGFLG      = 0

ZREPPRE    = RMDI
LLREPPRE   = .FALSE.
IREPPREFLG = 0
IREPPRECODE= NMDI

ZREPGEO    = RMDI
LLREPGEO   = .FALSE.
IREPGEOFLG = 0
IREPGEOCODE= NMDI

ZPMSL      = RMDI
ZPSTA      = RMDI
LLMSLP     = .FALSE.
LLSTAP     = .FALSE.
IPMSLFLG   = 0
IPSTAFLG   = 0

!*          4.0.0       GET PRESSURE

Ps0_LOOP: DO JPs = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPs  ,MDBVNM) == NVNUMB( 47)) THEN
    IPS     = JPs
    400 CONTINUE
    IPCODE = ROBODY(IPS  ,MDBSYPC)

!*          4.0.1       LOW LEVEL SYNOP

    IF(.NOT.LLHIGHSY)   THEN

!*          4.0.1.1     MSL PRESSURE

      IF(IPCODE == NPRESCD( 1)) THEN
        LLMSLP   = .TRUE.
        ZPMSL    = ROBODY(IPS,MDBVAR)
        IPMSLFLG = IBITS(INT(ROBODY(IPS,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
        ZGEOPMSL = 0.0_JPRB
        IGMSLFLG = 0
        IGMSLFLG = MAX(IGMSLFLG,IPMSLFLG)
        ROBODY(IPS,MDBPPP ) = ZGEOPMSL
        ROBODY(IPS,MDBRDFL) = 0
        IDBRDFL = ROBODY(IPS,MDBRDFL)
        CALL PPVAFL(IGMSLFLG,IPMSLFLG,IDBRDFL)
        ROBODY(IPS,MDBRDFL) = IDBRDFL

!*          4.0.1.1     STATION LEVEL PRESSURE

      ELSEIF(IPCODE == NPRESCD( 2)) THEN
        LLSTAP   = .TRUE.
        ZPSTA    = ROBODY(IPS,MDBVAR)
        IPSTAFLG = IBITS(INT(ROBODY(IPS,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
        ZGEOPSTL = ZALT
        IF(ZGEOPSTL /= RMDI) ZGEOPSTL = ZGEOPSTL * RG
        IGSTLFLG = IALFLG
        IGSTLFLG = MAX(IGSTLFLG,IPSTAFLG)
        ROBODY(IPS,MDBPPP ) = ZGEOPSTL
        ROBODY(IPS,MDBRDFL) = 0
        IDBRDFL = ROBODY(IPS,MDBRDFL)
        CALL PPVAFL(IGSTLFLG,IPSTAFLG,IDBRDFL)
        ROBODY(IPS,MDBRDFL) = IDBRDFL
      ELSE
        WRITE(NULOUT,'('' LNDSYIN; PROBLEM WITH LOW SYNOP P CODE: '')') IPCODE
        CALL ABOR1(' LNDSYIN : ')
      ENDIF

!*          4.0.2       HIGH LEVEL SYNOP

    ELSE

!*          4.0.2.1     STATION LEVEL PRESSURE

      IF(IPCODE == NPRESCD( 2)) THEN
        LLSTAP   = .TRUE.
        ZPSTA    = ROBODY(IPS,MDBVAR)
        IPSTAFLG = IBITS(INT(ROBODY(IPS,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
        ZGEOPSTL = ZALT
        IF(ZGEOPSTL /= RMDI) ZGEOPSTL = ZGEOPSTL * RG
        IGSTLFLG = IALFLG
        IGSTLFLG = MAX(IGSTLFLG,IPSTAFLG)
        ROBODY(IPS,MDBPPP ) = ZGEOPSTL
        ROBODY(IPS,MDBRDFL) = 0
        IDBRDFL = ROBODY(IPS,MDBRDFL)
        CALL PPVAFL(IGSTLFLG,IPSTAFLG,IDBRDFL)
        ROBODY(IPS,MDBRDFL) = IDBRDFL
        
!*          4.0.2.2     REPORTED PRESSURE/GEOPOTENTIAL

      ELSEIF(IPCODE ==        NMDI .OR.&
         & IPCODE == NPRESCD( 5) .OR.&
         & IPCODE == NPRESCD( 6) .OR.&
         & IPCODE == NPRESCD( 7) .OR.&
         & IPCODE == NPRESCD( 8) .OR.&
         & IPCODE == NPRESCD( 9)      ) THEN  

!*          4.0.3.1     PRESSURE LEVEL AT WHICH GEOPOTENTIAL IS REPORTED

        LLREPPRE   = .TRUE.
        IREPPRECODE= IPCODE
        ZREPPRE    = ROBODY(IPS  ,MDBVAR)
        IREPPREFLG = IBITS(INT(ROBODY(IPs  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)

!*          4.0.3.1.1   ASSOCIATE GEOPOTENTIAL WITH PRESSURE

        Z0_LOOP: DO JZ = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
          IF(ROBODY(JZ   ,MDBVNM) == NVNUMB(  3)) THEN
            LLREPGEO   = .TRUE.
            IREPGEOCODE= ROBODY(JZ   ,MDBSYPC)
            ZREPGEO    = ROBODY(JZ   ,MDBVAR)
            IREPGEOFLG = IBITS(INT(ROBODY(JZ ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
          ENDIF
        ENDDO Z0_LOOP
      ELSE
        WRITE(NULOUT,'('' LNDSYIN; PROBLEM WITH HIGH SYNOP P CODE: '')') IPCODE
        CALL ABOR1(' LNDSYIN : ')
      ENDIF
    ENDIF
  ENDIF
ENDDO Ps0_LOOP

!*          4.0.4       NORMAL (LOW LEVEL) SYNOP

IPCODE = NMDI
IF(.NOT.LLHIGHSY) THEN

!*          4.0.4.1     STATION LEVEL PRESSURE

  IF( ZPSTA /= RMDI) THEN
    ZPRES = ZPSTA
    IPFLG = IPSTAFLG
    IPCODE= NPRESCD( 2)
    ZGEOP = ZGEOPSTL
    IGFLG = IGSTLFLG

!*          4.0.4.2     MSL PRESSURE

  ELSEIF(ZPMSL /= RMDI ) THEN
    ZPRES = ZPMSL
    IPFLG = IPMSLFLG
    IPCODE= NPRESCD( 1)
    ZGEOP = ZGEOPMSL
    IGFLG = IGMSLFLG
  ELSE
    ZPRES = ZPSTA
    IPFLG = IPSTAFLG
    IPCODE= NPRESCD( 2)
    ZGEOP = ZGEOPSTL
    IGFLG = IGSTLFLG
  ENDIF

!*          4.0.5       HIGH LEVEL SYNOP

ELSE

!*          4.0.5.1     STATION LEVEL PRESSURE

  IF(ZPSTA /= RMDI ) THEN
    ZPRES = ZPSTA
    IPFLG = IPSTAFLG
    IPCODE= NPRESCD( 2)
    ZGEOP = ZGEOPSTL
    IGFLG = IGSTLFLG

!*          4.0.5.2     PRESSURE LEVEL AT WHICH GEOPOTENTIAL IS REPORTED

  ELSE
    IF(ZREPPRE  /= RMDI) THEN
      ZPRES = ZREPPRE
      IPFLG = IREPPREFLG
      IPCODE= IREPGEOCODE
      IF(ZREPGEO /= RMDI) THEN
        ZGEOP = ZREPGEO
        IGFLG = IREPGEOFLG
      ENDIF

!*          4.0.5.3     FIND GEOPOTENTIAL

    ELSE
      IF(ZREPGEO /= RMDI) THEN
        ZGEOP = ZREPGEO
        IGFLG = IREPGEOFLG
        IPCODE= IREPPRECODE
      ENDIF
    ENDIF
  ENDIF
ENDIF

!*          4.0.6       CHECK IF ZPRES, ZGEOP AND ZALT PRESENT

IF(ZPRES == RMDI.AND.ZGEOP == RMDI.AND.ZALT  == RMDI     )      THEN
  WRITE(NULOUT,'(''0LANSYIN; NO PRESSURE, GEOPOTENTIAL AND'',&
   & '' STATION HEIGHT ARE NOT AVAILABLE,'',&
   & '' HENCE REPORT AND BODY ENTRIES REJECTED'')')  
  WRITE(NULOUT,'('' LAT: : '',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON: : '',F10.2)') ZLON
  ROBHDR(KSYNP,MDBRST) = ZCHSTAT_REJECT( ROBHDR(KSYNP,MDBRST) )

!*          4.0.6.0     REJECT ALL BODIES

  ALL_LOOP: DO JALL    = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
    ROBODY(JALL,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(JALL,MDBDSTA) )

!*          4.0.6.1     SET ANY OBS. VALUE TO MISSING
!                       (CURRENT FEEDBACK NEEDS THIS)

    ROBODY(JALL,MDBVAR)  = RMDI
  ENDDO ALL_LOOP 
  GO TO 5
ENDIF

!*          4.0.7       VERTICAL COORDINATE

ZVERCO = ZPRES
IVECFL = IPFLG
IVERCO = NPRESVC
IF(ZPRES  == RMDI)            THEN
  IF(ZGEOP /= RMDI)             THEN
    ZVERCO = ZGEOP
    IVECFL = IGFLG
    IVERCO = NHEIGVC
  ELSE
    IF(ZALT /= RMDI)             THEN
      ZVERCO = ZALT * RG
      IVECFL = IALFLG
      IVERCO = NHEIGVC
    ENDIF
  ENDIF
ENDIF

!*          4.0.8       IF HEIGHT VERTICAL COORDINATE GET A PRESSURE
!                       FOR OBS. ERROR CALCULATIONS

ZPRESAU = ZPRES
IF(IVERCO == NHEIGVC)         THEN
  ZHEAUX  = ZVERCO / RG
  CALL Z2PICAO(ZHEAUX,ZPRESAU)
  ZPRESAU = ZPRESAU * 100._JPRB
ENDIF

!*          4.0.9       GET Pt

ZPT   = RMDI
IPTFLG= 0
PT0_LOOP: DO JPT = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPT  ,MDBVNM) == NVNUMB( 13)) THEN
    ZPT     = ROBODY(JPT  ,MDBVAR)
    IPTFLG  = IBITS(INT(ROBODY(JPT  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
  ENDIF
ENDDO PT0_LOOP

!*          4.0.10      GET DD

ZDIR  = RMDI
IDDFLG= 0
DD0_LOOP: DO JDD = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JDD  ,MDBVNM) == NVNUMB( 48)) THEN
    ZDIR    = ROBODY(JDD  ,MDBVAR)
    IDDFLG  = IBITS(INT(ROBODY(JDD  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
  ENDIF
ENDDO DD0_LOOP

!*          4.0.11      GET FF

ZSPD  = RMDI
IFFFLG= 0
FF0_LOOP: DO JFF = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JFF  ,MDBVNM) == NVNUMB( 49)) THEN
    ZSPD    = ROBODY(JFF  ,MDBVAR)
    IFFFLG  = IBITS(INT(ROBODY(JFF  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
  ENDIF
ENDDO FF0_LOOP

!*          4.1         PRESSURE TENDENCY CORRECTION

IF(LTENDCOR)      THEN
  LLSHIP  = .FALSE.
  LLTEND  = .FALSE.
  LLSHIPC = .FALSE.

!*          4.1.1       SEE IF PRESSURE/HEIGHT PRESENT

  IF(ZPRES == RMDI .AND. ZGEOP == RMDI)         GO TO 420

!*          4.1.2       WIND COMPONENTS AND FLAGS

  ZUWIND  = RMDI
  IUWINDF = 0
  ZVWIND  = RMDI
  IVWINDF = 0

!*          4.1.3       SHIP COMPONENTS AND FLAGS

  ZSHIPU  = RMDI
  IUSHIPF = 0
  ZSHIPV  = RMDI
  IVSHIPF = 0

!*          4.1.4       DO CORRECTION
  
  IF((IPCODE ==       NMDI .AND.ZREPPRE /= RMDI).OR.&
     & IPCODE  == NPRESCD( 1).OR.&
     & IPCODE  == NPRESCD( 2).OR.&
     & IPCODE  == NPRESCD( 5).OR.&
     & IPCODE  == NPRESCD( 6).OR.&
     & IPCODE  == NPRESCD( 7).OR.&
     & IPCODE  == NPRESCD( 8).OR.&
     & IPCODE  == NPRESCD( 9)                          ) THEN  
    IF(ZPRES /= RMDI)                                    THEN
      CALL PTENDCOR(LLSHIP,LLTEND,LLSHIPC,ZLAT,ZLON,ZTIMDIF,&
       & ZSHIPU,IUSHIPF,ZSHIPV,IVSHIPF,&
       & ZPRES,ZPT,IPTFLG,&
       & ZUWIND,IUWINDF,ZVWIND,IVWINDF           )  
      ZPRESAU = ZPRES
      ZVERCO  = ZPRES
    ENDIF
  ELSEIF(IPCODE ==        NMDI.OR.&
     & IPCODE == NPRESCD(11).OR.&
     & IPCODE == NPRESCD(10).OR.&
     & IPCODE == NPRESCD( 3).OR.&
     & IPCODE == NPRESCD( 4).OR.&
     & IPCODE == NPRESCD(12)    )     THEN  
    IF(ZGEOP /= RMDI)                       THEN
      IF(ZPT /= RMDI)                      THEN
        ZAUXPT = ZPT/RPAPM
      ELSE
        ZAUXPT = ZPT
      ENDIF
      CALL PTENDCOR(LLSHIP,LLTEND,LLSHIPC,ZLAT,ZLON,ZTIMDIF,&
       & ZSHIPU,IUSHIPF,ZSHIPV,IVSHIPF,&
       & ZGEOP,ZAUXPT,IPTFLG,&
       & ZUWIND,IUWINDF,ZVWIND,IVWINDF           )  
      IF(IVERCO == NHEIGVC)         THEN
        ZHEAUX  = ZGEOP / RG
        CALL Z2PICAO(ZHEAUX,ZPRESAU)
        ZPRESAU = ZPRESAU * 100._JPRB
        ZVERCO = ZGEOP
      ENDIF
    ENDIF
  ENDIF

!*          4.1.4.1     REDIFINE THE PERSISTENCE ERROR WEIGHT

  IF(LLTEND)      THEN
    ZWEIGHT = ZWEIGHT / RREDPEW1
  ENDIF
ENDIF

!*          4.2         LOOK FOR WIND DIRECTION/SPEED DATA

420 CONTINUE
ZDIR   = RMDI
ZSPD   = RMDI
IDDFLG = 0
IFFFLG = 0
ZDDPRES= RMDI
ZFFPRES= RMDI
IDDPFLG= 0
IFFPFLG= 0
DDFF_LOOP: DO JDDFF = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1

!*          4.2.1       WIND DIRECTION

  IF(ROBODY(JDDFF,MDBVNM) == NVNUMB( 48)) THEN
    IDD     = JDDFF
    IVNM    = NVNUMB( 48)

!*          4.2.1.0     SET/GET

    CALL GETSETE(IDD,&
     & ZDDPRES,ZDDPRL,ZDIR,&
     & ZDDFGER,ZDDFGCC1,ZDDFGCC2,&
     & ZDDPERR,ZDDOERR,ZDDFERR,ZDDRERR,&
     & ZDDFGDEP,ZDDANDEP,&
     & IDDSTAT,IDDEVEN1,IDDEVEN2,IDDRDBFL,IDDANFLG,&
     & IDDFLG,IDDPFLG)  

    ZDDPRES  = ZVERCO

    IDDPFLG  = IVECFL
 
    IPRESCD  = IPCODE

!*          4.2.1.2     VARIABLE

    IF(ROBODY(IDD  ,MDBVAR) /= RMDI )       THEN
      ZDIR   = ROBODY(IDD  ,MDBVAR)
      IDDFLG = IBITS(IDDRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
    ENDIF

!*          4.2.1.3     ERROR STATISTICS

!*          4.2.1.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 48,KCDSQNO,KOBSQNO) == 0)  THEN
      IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
    ENDIF
    IDDRDBFL  = 0
    CALL PPVAFL(IDDPFLG,IDDFLG,IDDRDBFL)

!*          4.2.1.5     DEPARTURES

!*          4.2.1.6     UPDATE BODY ENTRY
 
    CALL SYNOPBE(IDD     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZDDPRL  ,ZDIR    ,&
     & ZDDOERR ,ZDDPERR ,ZDDFERR ,ZDDRERR ,&
     & ZDDFGER ,ZDDFGCC1,ZDDFGCC2,&
     & ZDDFGDEP,ZDDANDEP,&
     & IDDSTAT ,IDDEVEN1,IDDEVEN2,IDDRDBFL,IDDANFLG,&
     & IPRESCD                                       )  

!*          4.2.2       WIND FORCE

  ELSEIF(ROBODY(JDDFF,MDBVNM) == NVNUMB( 49)) THEN
    IFF     = JDDFF
    IVNM    = NVNUMB( 49)

!*          4.2.2.0     SET/GET

    CALL GETSETE(IFF,&
     & ZFFPRES,ZFFPRL,ZSPD,&
     & ZFFFGER,ZFFFGCC1,ZFFFGCC2,&
     & ZFFPERR,ZFFOERR,ZFFFERR,ZFFRERR,&
     & ZFFFGDEP,ZFFANDEP,&
     & IFFSTAT,IFFEVEN1,IFFEVEN2,IFFRDBFL,IFFANFLG,&
     & IFFFLG,IFFPFLG)  

    ZFFPRES  = ZVERCO

    IFFPFLG  = IVECFL
 
    IPRESCD  = IPCODE

!*          4.2.2.2     VARIABLE

    IF(ROBODY(IFF  ,MDBVAR) /= RMDI )        THEN
      ZSPD   = ROBODY(IFF  ,MDBVAR)
      IFFFLG = IBITS(IFFRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
    ENDIF

!*          4.2.2.3     ERROR STATISTICS

    IF(LERRPERS)           THEN
      ZFFPERR  = 1.0_JPRB ! Rough first estimate of persistence error
    ENDIF
    ZFFOERR  = 2.0_JPRB ! Rough first estimate of obs. error =2 m/s
    CALL FINOERR(ZFFPERR,ZFFOERR,ZFFFERR)

!*          4.2.2.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 49,KCDSQNO,KOBSQNO) == 0)          THEN
      IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
    ENDIF
    IFFRDBFL  = 0
    CALL PPVAFL(IFFPFLG,IFFFLG,IFFRDBFL)

!*          4.2.2.5     DEPARTURES

!*          4.2.2.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IFF     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZFFPRL  ,ZSPD    ,&
     & ZFFOERR ,ZFFPERR ,ZFFFERR ,ZFFRERR ,&
     & ZFFFGER ,ZFFFGCC1,ZFFFGCC2,&
     & ZFFFGDEP,ZFFANDEP,&
     & IFFSTAT ,IFFEVEN1,IFFEVEN2,IFFRDBFL,IFFANFLG,&
     & IPRESCD                                       )  

  ENDIF
ENDDO DDFF_LOOP

!*          4.2.3       CHECK DD, FF AND PRESSURE

!*          4.2.4       10m(?) U COMPONENT

U10M_LOOP: DO JU10M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JU10M,MDBVNM) == NVNUMB( 56)) THEN
    IU10M = JU10M
    IVNM  = NVNUMB( 56)

!*          4.2.4.0     SET/GET

    CALL GETSETE(IU10M,&
     & ZUUPRES,ZUUPRL,ZU  ,&
     & ZUUFGER,ZUUFGCC1,ZUUFGCC2,&
     & ZUUPERR,ZUUOERR,ZUUFERR,ZUURERR,&
     & ZUUFGDEP,ZUUANDEP,&
     & IUUSTAT,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
     & IUUFLG,IUUPFLG)  

    ZUUPRES  = ZVERCO

    IUUPFLG  = IVECFL
 
    IPRESCD  = IPCODE

!*          4.2.4.2     VARIABLE

    IF(ZDIR /= RMDI.AND.ZSPD /= RMDI    )        THEN
      ZU     = UCOM(ZDIR,ZSPD)
      IUUFLG = MAX(IDDFLG,IFFFLG)
    ELSE
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF

!*          4.2.4.3     ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IUUSTAT,ZPRESAU,ZUUPRL,&
     & ZUUPERR,ZUUOERR,ZUUFERR,ZWEIGHT)  

!*          4.2.4.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 56,KCDSQNO,KOBSQNO) == 0)          THEN
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF
    IUURDBFL  = 0
    CALL PPVAFL(IUUPFLG,IUUFLG,IUURDBFL)

!*          4.2.4.5     DEPARTURES

!*          4.2.4.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IU10M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZUUPRL  ,ZU      ,&
     & ZUUOERR ,ZUUPERR ,ZUUFERR ,ZUURERR ,&
     & ZUUFGER ,ZUUFGCC1,ZUUFGCC2,&
     & ZUUFGDEP,ZUUANDEP,&
     & IUUSTAT ,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO U10M_LOOP

!*          4.2.5       10m(?) V COMPONENT

V10M_LOOP: DO JV10M    = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JV10M,MDBVNM) == NVNUMB( 57)) THEN
    IV10M = JV10M
    IVNM  = NVNUMB( 57)

!*          4.2.5.0     SET/GET

    CALL GETSETE(IV10M,&
     & ZVVPRES,ZVVPRL,ZV  ,&
     & ZVVFGER,ZVVFGCC1,ZVVFGCC2,&
     & ZVVPERR,ZVVOERR,ZVVFERR,ZVVRERR,&
     & ZVVFGDEP,ZVVANDEP,&
     & IVVSTAT,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
     & IVVFLG,IVVPFLG)  

    ZVVPRES  = ZVERCO

    IVVPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.2.5.2     VARIABLE

    IF(ZDIR /= RMDI.AND.ZSPD /= RMDI    )        THEN
      ZV      = VCOM(ZDIR,ZSPD)
      IVVFLG  = MAX(IDDFLG,IFFFLG)
    ELSE
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF

!*          4.2.5.3     ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IVVSTAT,ZPRESAU,ZVVPRL,&
     & ZVVPERR,ZVVOERR,ZVVFERR,ZWEIGHT)  

!*          4.2.5.4     STATUS, EVENTS, FLAGS
  
    IF(NMKCMVSE( 57,KCDSQNO,KOBSQNO) == 0)          THEN
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF
    IVVRDBFL  = 0
    CALL PPVAFL(IVVPFLG,IVVFLG,IVVRDBFL)

!*          4.2.5.5     DEPARTURES

!*          4.2.5.5     UPDATE BODY ENTRY

    CALL SYNOPBE(IV10M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZVVPRL  ,ZV      ,&
     & ZVVOERR ,ZVVPERR ,ZVVFERR ,ZVVRERR ,&
     & ZVVFGER ,ZVVFGCC1,ZVVFGCC2,&
     & ZVVFGDEP,ZVVANDEP,&
     & IVVSTAT ,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO V10M_LOOP

!*          4.3         2m(?) TEMPERATURE

ZT2M     = RMDI
ITTFLG   = 0
ITTPFLG  = IVECFL
T2M_LOOP: DO JT2M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JT2M ,MDBVNM) == NVNUMB( 10)) THEN
    IT2M = JT2M
    IVNM = NVNUMB( 10)

!*          4.3.0       SET/GET

    CALL GETSETE(IT2M,&
     & ZTTPRES,ZTTPRL,ZT2M,&
     & ZTTFGER,ZTTFGCC1,ZTTFGCC2,&
     & ZTTPERR,ZTTOERR,ZTTFERR,ZTTRERR,&
     & ZTTFGDEP,ZTTANDEP,&
     & ITTSTAT,ITTEVEN1,ITTEVEN2,ITTRDBFL,ITTANFLG,&
     & ITTFLG,ITTPFLG)  

    ZTTPRES  = ZVERCO

    ITTPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.3.2       VARIABLE

    IF(ROBODY(IT2M ,MDBVAR) /= RMDI ) THEN
      ZT2M  = ROBODY(IT2M ,MDBVAR)
      ITTFLG= IBITS(ITTRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      ITTSTAT = ICHSTAT_REJECT( ITTSTAT )
    ENDIF

!*          4.3.3       ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,ITTSTAT,ZPRESAU,ZTTPRL,&
     & ZTTPERR,ZTTOERR,ZTTFERR,ZWEIGHT)  

!*          4.3.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 10,KCDSQNO,KOBSQNO) == 0)          THEN
      ITTSTAT = ICHSTAT_REJECT( ITTSTAT )
    ENDIF
    ITTRDBFL  = 0
    CALL PPVAFL(ITTPFLG,ITTFLG,ITTRDBFL)

!*          4.3.5       DEPARTURES

!*          4.3.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IT2M    ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZTTPRL  ,ZT2M    ,&
     & ZTTOERR ,ZTTPERR ,ZTTFERR ,ZTTRERR ,&
     & ZTTFGER ,ZTTFGCC1,ZTTFGCC2,&
     & ZTTFGDEP,ZTTANDEP,&
     & ITTSTAT ,ITTEVEN1,ITTEVEN2,ITTRDBFL,ITTANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO T2M_LOOP

!*          4.4         DEW POINT

ZTD2M  = RMDI
ITDFLG = 0
ITDPFLG= IVECFL
DEWPOINT_LOOP: DO JTD2M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JTD2M,MDBVNM) == NVNUMB( 11)) THEN
    ITD2M = JTD2M
    IVNM = NVNUMB( 11)

!*          4.4.0       SET/GET

    CALL GETSETE(ITD2M,&
     & ZTDPRES,ZTDPRL,ZTD2M,&
     & ZTDFGER,ZTDFGCC1,ZTDFGCC2,&
     & ZTDPERR,ZTDOERR,ZTDFERR,ZTDRERR,&
     & ZTDFGDEP,ZTDANDEP,&
     & ITDSTAT,ITDEVEN1,ITDEVEN2,ITDRDBFL,ITDANFLG,&
     & ITDFLG,ITDPFLG)  

    ZTDPRES  = ZVERCO

    ITDPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.4.2       VARIABLE

    IF(ROBODY(ITD2M,MDBVAR) /= RMDI ) THEN
      ZTD2M = ROBODY(ITD2M,MDBVAR)
      ITDFLG= IBITS(ITDRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      ITDSTAT = ICHSTAT_REJECT( ITDSTAT )
    ENDIF

!*          4.4.3       ERROR STATISTICS

!*          4.4.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 11,KCDSQNO,KOBSQNO) == 0)          THEN
      ITDSTAT = ICHSTAT_REJECT( ITDSTAT )
    ENDIF
    ITDRDBFL  = 0
    CALL PPVAFL(ITDPFLG,ITDFLG,ITDRDBFL)

!*          4.4.5       DEPARTURES

!*          4.4.6       UPDATE BODY ENTRY

    CALL SYNOPBE(ITD2M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZTDPRL  ,ZTD2M   ,&
     & ZTDOERR ,ZTDPERR ,ZTDFERR ,ZTDRERR ,&
     & ZTDFGER ,ZTDFGCC1,ZTDFGCC2,&
     & ZTDFGDEP,ZTDANDEP,&
     & ITDSTAT ,ITDEVEN1,ITDEVEN2,ITDRDBFL,ITDANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO DEWPOINT_LOOP

!*          4.5         RELATIVE HUMIDITY

REL_HUMIDITY_DATA_LOOP: DO JRH2M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JRH2M,MDBVNM) == NVNUMB(  7)) THEN
    IRH2M = JRH2M
    IVNM  = NVNUMB(  7)

!*          4.5.0       SET/GET

    CALL GETSETE(IRH2M,&
     & ZRHPRES,ZRHPRL,ZRH2M,&
     & ZRHFGER,ZRHFGCC1,ZRHFGCC2,&
     & ZRHPERR,ZRHOERR,ZRHFERR,ZRHRERR,&
     & ZRHFGDEP,ZRHANDEP,&
     & IRHSTAT,IRHEVEN1,IRHEVEN2,IRHRDBFL,IRHANFLG,&
     & IRHFLG,IRHPFLG)  

    ZRHPRES  = ZVERCO

    IRHPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.5.2       VARIABLE

    IF( ZT2M /= RMDI .AND. ZTD2M /= RMDI ) THEN
      IRHFLG   = MAX(ITTFLG,ITDFLG)
      IF(IRHFLG < NFLLIMOP(2))  THEN
        LLDONEBCH= .FALSE.
        ZRH2M    = FOEEWMO( ZTD2M ) / FOEEWMO( ZT2M )

!*          4.5.3       ERROR STATISTICS

!*          4.5.3.2     OBSERVATION ERROR

        IF(LRHERRMO)       THEN
          ZRHOERR = -0.0015_JPRB * ZT2M + 0.54_JPRB
          ZRHOERR = MIN(0.18_JPRB,MAX(0.06_JPRB,ZRHOERR))
        ELSE
          ZRHOERR = RHERR1
          IF(ZRH2M < RRHLIM )  ZRHOERR = RHERR2
          IF(ZT2M  < RRHTLIM)  ZRHOERR = RHERR3
        ENDIF

!*          4.5.3.3     FINAL OBSERVATION ERROR

        ZRHFERR = ZRHOERR

!*          4.5.3.4     COMBINED FLAG TO HI

      ELSE
        ZRH2M   = RMDI
        IRHSTAT = ICHSTAT_REJECT( IRHSTAT )
      ENDIF

!*          4.5.3.5     NO VARIABLE

    ELSE
      IRHSTAT = ICHSTAT_REJECT( IRHSTAT )
    ENDIF

!*          4.5.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  7,KCDSQNO,KOBSQNO) == 0)          THEN
      IRHSTAT = ICHSTAT_REJECT( IRHSTAT )
    ENDIF
    IRHRDBFL  = 0
    CALL PPVAFL(IRHPFLG,IRHFLG,IRHRDBFL)

!*          4.4.5       DEPARTURES

!*          4.5.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IRH2M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZRHPRL  ,ZRH2M   ,&
     & ZRHOERR ,ZRHPERR ,ZRHFERR ,ZRHRERR ,&
     & ZRHFGER ,ZRHFGCC1,ZRHFGCC2,&
     & ZRHFGDEP,ZRHANDEP,&
     & IRHSTAT ,IRHEVEN1,IRHEVEN2,IRHRDBFL,IRHANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO REL_HUMIDITY_DATA_LOOP

!*          4.6         PRESSURE TENDENCY

PRESSURE_TENDENCY_LOOP: DO JPT = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPT,MDBVNM) == NVNUMB( 13)) THEN
    IPT   = JPT
    IVNM  = NVNUMB( 13)

!*          4.6.0       SET/GET

    CALL GETSETE(IPT,&
     & ZPTPRES,ZPTPRL,ZPT ,&
     & ZPTFGER,ZPTFGCC1,ZPTFGCC2,&
     & ZPTPERR,ZPTOERR,ZPTFERR,ZPTRERR,&
     & ZPTFGDEP,ZPTANDEP,&
     & IPTSTAT,IPTEVEN1,IPTEVEN2,IPTRDBFL,IPTANFLG,&
     & IPTFLG,IPTPFLG)  

    ZPTPRES  = ZVERCO

    IPTPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.6.2       VARIABLE

    IF(ROBODY(IPT,MDBVAR) /= RMDI ) THEN
      ZPT   = ROBODY(IPT,MDBVAR)
      IPTFLG= IBITS(IPTRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IPTSTAT = ICHSTAT_REJECT( IPTSTAT )
    ENDIF

!*          4.6.3       ERROR STATISTICS

!*          4.6.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 13,KCDSQNO,KOBSQNO) == 0)          THEN
      IPTSTAT = ICHSTAT_REJECT( IPTSTAT )
    ENDIF
    IPTRDBFL  = 0
    CALL PPVAFL(IPTPFLG,IPTFLG,IPTRDBFL)

!*          4.6.5       DEPARTURES

!*          4.6.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IPT     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZPTPRL  ,ZPT     ,&
     & ZPTOERR ,ZPTPERR ,ZPTFERR ,ZPTRERR ,&
     & ZPTFGER ,ZPTFGCC1,ZPTFGCC2,&
     & ZPTFGDEP,ZPTANDEP,&
     & IPTSTAT ,IPTEVEN1,IPTEVEN2,IPTRDBFL,IPTANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO PRESSURE_TENDENCY_LOOP

!*          4.7         SNOW

SNOW_LOOP: DO JSNOW = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JSNOW,MDBVNM) == NVNUMB( 25)) THEN
    ISNOW = JSNOW
    IVNM  = NVNUMB( 25)

!*          4.7.0       SET/GET

    CALL GETSETE(ISNOW,&
     & ZSNPRES,ZSNPRL,ZSN ,&
     & ZSNFGER,ZSNFGCC1,ZSNFGCC2,&
     & ZSNPERR,ZSNOERR,ZSNFERR,ZSNRERR,&
     & ZSNFGDEP,ZSNANDEP,&
     & ISNSTAT,ISNEVEN1,ISNEVEN2,ISNRDBFL,ISNANFLG,&
     & ISNFLG,ISNPFLG)  

    ZSNPRES  = ZVERCO

    ISNPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.7.2       VARIABLE

    IF(ROBODY(ISNOW,MDBVAR) /= RMDI ) THEN
      ZSN   = ROBODY(ISNOW,MDBVAR)
      ISNFLG= IBITS(ISNRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      ISNSTAT = ICHSTAT_REJECT( ISNSTAT )
    ENDIF

!*          4.7.3       ERROR STATISTICS

!*          4.7.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 25,KCDSQNO,KOBSQNO) == 0)          THEN
      ISNSTAT = ICHSTAT_REJECT( ISNSTAT )
    ENDIF
    ISNRDBFL  = 0
    CALL PPVAFL(ISNPFLG,ISNFLG,ISNRDBFL)

!*          4.7.5       DEPARTURES

!*          4.7.6       CREATE BODY ENTRY

    CALL SYNOPBE(ISNOW   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZSNPRL  ,ZSN     ,&
     & ZSNOERR ,ZSNPERR ,ZSNFERR ,ZSNRERR ,&
     & ZSNFGER ,ZSNFGCC1,ZSNFGCC2,&
     & ZSNFGDEP,ZSNANDEP,&
     & ISNSTAT ,ISNEVEN1,ISNEVEN2,ISNRDBFL,ISNANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO SNOW_LOOP

!*          4.8         RAIN DURATION

RAIN_DURATION_LOOP: DO JRD = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JRD,MDBVNM) == NVNUMB( 33)) THEN
    IRD   = JRD
    IVNM  = NVNUMB( 33)

!*          4.8.0       SET/GET

    CALL GETSETE(IRD,&
     & ZRDPRES,ZRDPRL,ZRD ,&
     & ZRDFGER,ZRDFGCC1,ZRDFGCC2,&
     & ZRDPERR,ZRDOERR,ZRDFERR,ZRDRERR,&
     & ZRDFGDEP,ZRDANDEP,&
     & IRDSTAT,IRDEVEN1,IRDEVEN2,IRDRDBFL,IRDANFLG,&
     & IRDFLG,IRDPFLG)  

    ZRDPRES  = ZVERCO

    IRDPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.8.2       VARIABLE

    IF(ROBODY(IRD  ,MDBVAR) /= RMDI ) THEN
      ZRD   = ROBODY(IRD  ,MDBVAR)
      IRDFLG= IBITS(IRDRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IRDSTAT = ICHSTAT_REJECT( IRDSTAT )
    ENDIF

!*          4.8.3       ERROR STATISTICS

!*          4.8.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 33,KCDSQNO,KOBSQNO) == 0)          THEN
      IRDSTAT = ICHSTAT_REJECT( IRDSTAT )
    ENDIF
    IRDRDBFL  = 0
    CALL PPVAFL(IRDPFLG,IRDFLG,IRDRDBFL)

!*          4.8.5       DEPARTURES

!*          4.8.6       CREATE BODY ENTRY

    CALL SYNOPBE(IRD     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZRDPRL  ,ZRD     ,&
     & ZRDOERR ,ZRDPERR ,ZRDFERR ,ZRDRERR ,&
     & ZRDFGER ,ZRDFGCC1,ZRDFGCC2,&
     & ZRDFGDEP,ZRDANDEP,&
     & IRDSTAT ,IRDEVEN1,IRDEVEN2,IRDRDBFL,IRDANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO RAIN_DURATION_LOOP

!*          4.9         RAIN

490 CONTINUE
ZR     = RMDI
IRRFLG = 0
RAIN_LOOP_1: DO JR  = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JR   ,MDBVNM) == NVNUMB( 34)) THEN
    IR    = JR
    ZR    = ROBODY(IR   ,MDBVAR)
    IRRFLG= IBITS(INT(ROBODY(IR   ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
    GO TO 491
  ENDIF
ENDDO RAIN_LOOP_1

!*          4.9.1        6HR RAIN FALL

491 CONTINUE
ZRAIN6HR = RMDI
IRA6FLG  = 0
IF(ZR /= RMDI.AND.ZRD /= RMDI     )           THEN
  ZRAIN6HR = ZR*6._JPRB/ZRD
  IRA6FLG  = MAX(IRRFLG,IRDFLG)
ENDIF

!*          4.9.2        SPLIT RAIN INFO. INTO LIQUED/SOLID PART

ZRAINSD = RMDI
IRSDFLG = 0
ZRAINLQ = RMDI
IRLQFLG = 0
IF(ZRAIN6HR /= RMDI.AND.ZT2M     /= RMDI     )           THEN

!*          4.9.2.1      ALL THE RAIN INFO. IS A SOLID PART

  IF(ZT2M  <= RKELVN-2.0_JPRB)              THEN
    ZRAINSD = ZRAIN6HR
    IRSDFLG = MAX(IRA6FLG,ITTFLG)

!*          4.9.2.2      RAIN INFO. IS A MIXTURE OF LIQUED AND SOLID
!                        PARTS

  ELSEIF(ZT2M  <= RKELVN+2.0_JPRB)         THEN
    ZI      = 1.0_JPRB - ((ZT2M -(RKELVN-2.0_JPRB))/4._JPRB)
    ZRAINSD = ZRAIN6HR * ZI
    IRSDFLG = MAX(IRA6FLG,ITTFLG )
    ZRAINLQ = MAX(ZRAIN6HR-ZRAINSD,0.0_JPRB)
    IRLQFLG = MAX(IRA6FLG,ITTFLG )

!*          4.9.2.3      ALL THE RAIN INFO. IS A LIQUED PART

  ELSE
    ZRAINLQ = ZRAIN6HR
    IRLQFLG = MAX(IRA6FLG,ITTFLG )
  ENDIF
ENDIF

!*          4.9.3        SOLID PART (SNOWFALL)

SOLID_RAIN_LOOP: DO JSR  = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JSR  ,MDBVNM) == NVNUMB( 46))          THEN
    ISR   = JSR
    IVNM   = NVNUMB( 46)

!*          4.9.3.0     SET/GET

    CALL GETSETE(ISR,&
     & ZSRPRES,ZSRPRL,ZSR ,&
     & ZSRFGER,ZSRFGCC1,ZSRFGCC2,&
     & ZSRPERR,ZSROERR,ZSRFERR,ZSRRERR,&
     & ZSRFGDEP,ZSRANDEP,&
     & ISRSTAT,ISREVEN1,ISREVEN2,ISRRDBFL,ISRANFLG,&
     & ISRFLG,ISRPFLG)  

    ZSRPRES  = ZVERCO

    ZSR      = ZRAINSD
    ISRFLG   = IRSDFLG

    ISRPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.9.3.2     VARIABLE

    IF(ZSR == RMDI ) THEN
      ISRSTAT = ICHSTAT_REJECT( ISRSTAT )
    ENDIF

!*          4.9.3.3     ERROR STATISTICS

!*          4.9.3.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 46,KCDSQNO,KOBSQNO) == 0)          THEN
      ISRSTAT = ICHSTAT_REJECT( ISRSTAT )
    ENDIF
    ISRRDBFL  = 0
    CALL PPVAFL(ISRPFLG,ISRFLG,ISRRDBFL)

!*          4.9.3.5     DEPARTURES

!*          4.9.3.6     UPDATE BODY ENTRY

    CALL SYNOPBE(ISR     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZSRPRL  ,ZSR     ,&
     & ZSROERR ,ZSRPERR ,ZSRFERR ,ZSRRERR ,&
     & ZSRFGER ,ZSRFGCC1,ZSRFGCC2,&
     & ZSRFGDEP,ZSRANDEP,&
     & ISRSTAT ,ISREVEN1,ISREVEN2,ISRRDBFL,ISRANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO SOLID_RAIN_LOOP

!*          4.9.4        LIQUED PART (RAIN)

494 CONTINUE
LIQUED_RAIN_LOOP: DO JLR  = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JLR  ,MDBVNM) == NVNUMB( 34)) THEN
    ILR  = JLR
    IVNM = NVNUMB( 34)

!*          4.9.4.0     SET/GET

    CALL GETSETE(ILR,&
     & ZLRPRES,ZLRPRL,ZLR ,&
     & ZLRFGER,ZLRFGCC1,ZLRFGCC2,&
     & ZLRPERR,ZLROERR,ZLRFERR,ZLRRERR,&
     & ZLRFGDEP,ZLRANDEP,&
     & ILRSTAT,ILREVEN1,ILREVEN2,ILRRDBFL,ILRANFLG,&
     & ILRFLG,ILRPFLG)  

    ZLRPRES  = ZVERCO

    ZLR      = ZRAINLQ
    ILRFLG   = IRLQFLG

    ILRPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.9.4.2     VARIABLE

    IF(ZLR == RMDI ) THEN
      ILRSTAT = ICHSTAT_REJECT( ILRSTAT )
    ENDIF

!*          4.9.4.3     ERROR STATISTICS

!*          4.9.4.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 34,KCDSQNO,KOBSQNO) == 0)          THEN
      ILRSTAT = ICHSTAT_REJECT( ILRSTAT )
    ENDIF
    ILRRDBFL  = 0
    CALL PPVAFL(ILRPFLG,ILRFLG,ILRRDBFL)

!*          4.9.4.5     DEPARTURES

!*          4.9.4.6     UPDATE BODY ENTRY

    CALL SYNOPBE(ILR     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZSRPRL  ,ZLR     ,&
     & ZLROERR ,ZLRPERR ,ZLRFERR ,ZLRRERR ,&
     & ZLRFGER ,ZLRFGCC1,ZLRFGCC2,&
     & ZLRFGDEP,ZLRANDEP,&
     & ILRSTAT ,ILREVEN1,ILREVEN2,ILRRDBFL,ILRANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO LIQUED_RAIN_LOOP

!*          4.10        GEOPOTENTIAL

Z_LOOP: DO JZ = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JZ   ,MDBVNM) == NVNUMB(  3)) THEN
    IZ   = JZ
    IVNM = NVNUMB(  3)

!*          4.10.0      SET/GET

    CALL GETSETE(IZ ,&
     & ZZZPRES,ZZZPRL,ZZZ ,&
     & ZZZFGER,ZZZFGCC1,ZZZFGCC2,&
     & ZZZPERR,ZZZOERR,ZZZFERR,ZZZRERR,&
     & ZZZFGDEP,ZZZANDEP,&
     & IZZSTAT,IZZEVEN1,IZZEVEN2,IZZRDBFL,IZZANFLG,&
     & IZZFLG,IZZPFLG)  

    IF(LLHIGHSY) THEN
!*    EMULATE CURRENT SYSTEM
      IF(ZPSTA /=RMDI) THEN
        ZZZ      = ZGEOP
        ZZZPRES  = ZPRES
        IZZFLG   = IGFLG
        IZZPFLG  = IVECFL
        IPRESCD  = IPCODE
      ELSE
        ZZZPRES  = ROBODY(IZ,MDBPPP )
        ZZZ      = ROBODY(IZ,MDBVAR )
        IPRESCD  = ROBODY(IZ,MDBSYPC)
      ENDIF
    ELSE
      ZZZPRES  = ZPRES
      ZZZ      = ZGEOP
      IZZFLG   = IGFLG
      IZZPFLG  = IVECFL
      IPRESCD  = IPCODE
    ENDIF

!*          4.10.2      VARIABLE

    IF(ZZZ     == RMDI ) THEN
      IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
!   ELSE
!     IF(LLSTAP .AND. ZPSTA /= RMDI) THEN
!       IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
!     ENDIF
    ENDIF

!*          4.10.3      ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IZZSTAT,ZPRESAU,ZZZPRL,&
     & ZZZPERR,ZZZOERR,ZZZFERR,ZWEIGHT)  

!*          4.10.3.4    STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  3,KCDSQNO,KOBSQNO) == 0)          THEN
      IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
    ENDIF
    IZZRDBFL  = 0
    CALL PPVAFL(IZZPFLG,IZZFLG,IZZRDBFL)

!*          4.10.3.5    DEPARTURES

!*          4.10.3.6    UPDATE BODY ENTRY

    CALL SYNOPBE(IZ      ,&
     & IVNM    ,NPRESVC ,ZZZPRES ,ZZZPRL  ,ZZZ     ,&
     & ZZZOERR ,ZZZPERR ,ZZZFERR ,ZZZRERR ,&
     & ZZZFGER ,ZZZFGCC1,ZZZFGCC2,&
     & ZZZFGDEP,ZZZANDEP,&
     & IZZSTAT ,IZZEVEN1,IZZEVEN2,IZZRDBFL,IZZANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO Z_LOOP

!*          4.11        PRESSURE

4110 CONTINUE
Ps_LOOP: DO JPs = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPs  ,MDBVNM) == NVNUMB( 47)) THEN
    IPS      = JPs
    IVNM     = NVNUMB( 47)
    IPRESCD  = ROBODY(IPS  ,MDBSYPC)

!*          4.11.0      SET/GET

    CALL GETSETE(IPS,&
     & ZPSPRES,ZPSPRL,ZPS ,&
     & ZPSFGER,ZPSFGCC1,ZPSFGCC2,&
     & ZPSPERR,ZPSOERR,ZPSFERR,ZPSRERR,&
     & ZPSFGDEP,ZPSANDEP,&
     & IPSSTAT,IPSEVEN1,IPSEVEN2,IPSRDBFL,IPSANFLG,&
     & IPSFLG,IPSPFLG)  

!*          4.11.0.1    HIGH LEVEL STATION

    IF(LLHIGHSY) THEN
      IF(IPRESCD  == NPRESCD( 2)) THEN
        ZPSPRES  = ZGEOPSTL
        IPSPFLG  = IGSTLFLG
      ELSE
        ZPSPRES  = ROBODY(IPS,MDBPPP )
        ZPS      = ROBODY(IPS,MDBVAR )
        IPRESCD  = ROBODY(IPS,MDBSYPC)
      ENDIF

!*          4.11.0.2    LOW LEVEL STATION

    ELSE
      IF(IPRESCD  == NPRESCD( 1)) THEN
        ZPSPRES  = 0.0_JPRB
        IPSPFLG  = 0
      ELSEIF(IPRESCD  == NPRESCD( 2)) THEN
        ZPSPRES  = ZGEOPSTL
        IPSPFLG  = IGSTLFLG
      ENDIF
    ENDIF

!*          4.11.2      VARIABLE

    ZPS   = ROBODY(IPS  ,MDBVAR)
    IF(ZPS /= RMDI ) THEN

!*          4.11.2.1    LOW LEVEL STATION (PS on)

      IF(.NOT.LLHIGHSY) THEN
!       IF((LLMSLP .AND. ZPMSL /= RMDI) .AND.&
!         &(LLSTAP .AND. ZPSTA /= RMDI) .AND.&
!         IPRESCD  == NPRESCD( 1)             )   THEN
        IF(LLMSLP .AND. LLSTAP)     THEN
          IF(IPRESCD  == NPRESCD( 1)) THEN
            IF(ZPSTA /= RMDI)           THEN
              IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
              IPSEVEN1= ICHEV1( IPSEVEN1,ND1EVENT(17))
            ENDIF
          ENDIF
        ENDIF

!*          4.11.2.2    HIGH LEVEL STATION (PS on)

      ELSE
        IF(IPRESCD  /= NPRESCD( 2))      THEN
          IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
          IPSEVEN1= ICHEV1( IPSEVEN1,ND1EVENT(17))
        ENDIF
      ENDIF

!*          4.11.2.3    BIAS CORRECTION

      IF(L_LAND_PS_BIAS) THEN
        ZPSBIAS = ROBODY(IPS,MDBTORB)
        IF(ZPSBIAS /= RMDI) THEN
          ZPS = ZPS - ZPSBIAS
          IPSEVEN2 = ICHEV2( IPSEVEN2,ND2EVENT( 1,KOBTYP))
        ENDIF
      ENDIF

    ELSE

!*          4.11.2.3    MISSING PS

      IF(LLMSLP .AND. LLSTAP)     THEN
        IF(IPRESCD  == NPRESCD( 1)) THEN
          IPSEVEN1= ICHEV1( IPSEVEN1,ND1EVENT(17))
        ENDIF
      ENDIF
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF

!*          4.11.3      ERROR STATISTICS

!*          4.11.3.1    PERSISTENCE ERROR

    IF(LERRPERS)           THEN
      IF(ZZZPERR /= RMDI)    THEN
        ZPSPERR  = ZZZPERR*RPAPM
      ELSE
        IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
      ENDIF
    ENDIF

!*          4.11.3.2    OBSERVATION ERROR

    IF(ZZZOERR /= RMDI)    THEN
      ZPSOERR  = ZZZOERR*RPAPM
    ELSE
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF

!*          4.11.3.3    PRESSURE FINAL OBSERVATION ERROR

    CALL FINOERR(ZPSPERR,ZPSOERR,ZPSFERR)

!*          4.11.4      STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 47,KCDSQNO,KOBSQNO) == 0)          THEN
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF
    IPSRDBFL  = 0
    CALL PPVAFL(IPSPFLG,IPSFLG,IPSRDBFL)

!*          4.11.5      DEPARTURES

!*          4.11.6      UPDATE BODY ENTRY

    CALL SYNOPBE(IPS     ,&
     & IVNM    ,NHEIGVC ,ZPSPRES ,ZPSPRL  ,ZPS     ,&
     & ZPSOERR ,ZPSPERR ,ZPSFERR ,ZPSRERR ,&
     & ZPSFGER ,ZPSFGCC1,ZPSFGCC2,&
     & ZPSFGDEP,ZPSANDEP,&
     & IPSSTAT ,IPSEVEN1,IPSEVEN2,IPSRDBFL,IPSANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO Ps_LOOP
!*
!-----------------------------------------------------------------------

!        5.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

5 CONTINUE

!*          5.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          5.2         ODB REPORT UPDATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        6.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

6 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        7.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('LNDSYIN',1,ZHOOK_HANDLE)
END SUBROUTINE LNDSYIN
