SUBROUTINE MKCMARPL(KTSLOT, KPOOLNO)

!**** SUBROUTINE MKCMARPL - MKCMA FUNCTIONS PERFORMED IN IFS

!          DRASKO VASILJEVIC ECMWF          01/MAR/2001

!*    METHOD
!     ------

!        THE ODB IS SCANNED FOR ALL OBSERVATIONS AND MAKECMA FUNCTIONS
!     ARE CARRIED OUT.

!*    INTERFACE
!     ---------

!     ***CALL*MKCMARPL(KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!*    EXTERNALS
!     ---------

!        CONVENTIONAL_OB - PROCESS CONV OBS 
!        SATOB_OB        - PROCESS SATOB OBS
!        TOVSRTOVS_OB    - PROCESS TOVS/RTOVS OBS (obsolescent)
!        LEVEL1CGEOS_OB  - PROCESS LEVEL 1C/GEOS OBS
!        OZONE_OB        - PROCESS OZONE OBS
!        SCAT_OB         - PROCESS SCAT OBS
!        SSMITRMM_OB     - PROCESS SSMI/TRMM OBS
!        ABOR1           - ABNORMAL TERMINATION

!        MKCMARPL IS CALLED BY SUBROUTINE SUOBA

!**   MODIFICATIONS
!     -------------
!     2007/04/12 C. Payan Meteo-France : LMKCMARPL is now a 2D array, allowing
!     a selective access to subroutines of mkcmarpl by codetype/obstype

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMLUN    ,ONLY : NULOUT   
USE YOMOBS    ,ONLY : LMKCMARPL
USE YOMCOCTP  ,ONLY : NSYNOPSQ ,NAIREPSQ ,NSATOBSQ &
                    &,NDRIBUSQ ,NTEMPSQ  ,NPILOTSQ &
                    &,NSATEMSQ ,NSTMCDSQ ,NSTOVSSQ ,NGTHRBSQ ,NSTRO3SQ &
                    &,NPAOBSQ  ,NSCATTSQ

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTSLOT 
INTEGER(KIND=JPIM),INTENT(iN)    :: KPOOLNO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL         :: LLCONVOB

#include "conventional_ob.intfb.h"
#include "level1cgeos_ob.intfb.h"
#include "ozone_ob.intfb.h"
#include "satob_ob.intfb.h"
#include "scat_ob.intfb.h"
!include "ssmitrmm_ob.intfb.h"

!     ------------------------------------------------------------------

!*       0.       INFORMU
!                 -------

IF (LHOOK) CALL DR_HOOK('MKCMARPL',0,ZHOOK_HANDLE)
WRITE(NULOUT,'('' * MKCMARPL; MKCMA FUNCTIONS'')')
WRITE(NULOUT,'(''   TIME SLOT: '',I10)') KTSLOT
WRITE(NULOUT,'(''   ALL SETS'')')
CALL FLUSH(NULOUT)

!     ------------------------------------------------------------------

!*       1.       CONVENTIONAL OBS (BUT NOT SATOB)
!                 --------------------------------

LLCONVOB=ANY((/LMKCMARPL(:,NSYNOPSQ),LMKCMARPL(:,NAIREPSQ),LMKCMARPL(:,NDRIBUSQ)&
             &,LMKCMARPL(:,NTEMPSQ) ,LMKCMARPL(:,NPILOTSQ)                      &
             &,LMKCMARPL(NSTMCDSQ,NSATEMSQ),LMKCMARPL(NSTOVSSQ,NSATEMSQ)        &
             &,LMKCMARPL(:,NPAOBSQ)/))
IF (LLCONVOB) THEN
        WRITE(NULOUT,'(''     GET CONVENTIONAL OBS'')')
        CALL FLUSH(NULOUT)
        CALL CONVENTIONAL_OB(KTSLOT, KPOOLNO)
ENDIF
!     ------------------------------------------------------------------

!*       2.       SATOB OBS
!                 ---------

IF (ANY(LMKCMARPL(:,NSATOBSQ))) THEN
        WRITE(NULOUT,'(''     GET SATOB OBS'')')
        CALL FLUSH(NULOUT)
        CALL SATOB_OB       (KTSLOT, KPOOLNO)
ENDIF

!     ------------------------------------------------------------------

!*       3.       TOVS/RTOVS OBS
!                 --------------

!if part active again, control by LMKCMARPL(?,satemsq) (like the other obstypes)
        !!WRITE(NULOUT,'(''     GET TOVS/RTOVS OBS'')')
        !!CALL FLUSH(NULOUT)
        !!CALL TOVSRTOVS_OB   (KTSLOT, KPOOLNO)

!     ------------------------------------------------------------------

!*       4.       LEVEL 1C/GEOS OBS
!                 -----------------

IF (LMKCMARPL(NGTHRBSQ,NSATEMSQ)) THEN
        WRITE(NULOUT,'(''     GET LEVEL 1C/GEOS OBS'')')
        CALL FLUSH(NULOUT)
        CALL LEVEL1CGEOS_OB (KTSLOT, KPOOLNO)
ENDIF

!     ------------------------------------------------------------------

!*       5.       OZONE OBS
!                 ---------

IF (LMKCMARPL(NSTRO3SQ,NSATEMSQ)) THEN
        WRITE(NULOUT,'(''     GET OZONE OBS'')')
        CALL FLUSH(NULOUT)
        CALL OZONE_OB       (KTSLOT, KPOOLNO)
ENDIF

!     ------------------------------------------------------------------

!*       6.       SCAT OBS
!                 --------

IF (ANY(LMKCMARPL(:,NSCATTSQ))) THEN
        WRITE(NULOUT,'(''     GET SCAT OBS'')')
        CALL FLUSH(NULOUT)
        CALL SCAT_OB        (KTSLOT, KPOOLNO)
ENDIF

!     ------------------------------------------------------------------

!*       7.       SSMI/TRMM OBS
!                 -------------

!ssmi -- awaits for new SSM/I implementation from Peter Bauer/Deborah Salmond
#ifdef OLD_SSMI_IMPLEMENTATION
WRITE(NULOUT,'(''     GET SSMI/TRMM OBS'')')
CALL FLUSH(NULOUT)
!if part active again, control by LMKCMARPL(?,?) (like the other obstypes)
!--CALL SSMITRMM_OB    (KTSLOT, KPOOLNO)
#endif

!*
!     ------------------------------------------------------------------

!        8.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('MKCMARPL',1,ZHOOK_HANDLE)
END SUBROUTINE MKCMARPL
