SUBROUTINE NEW_THINNER(KLEN,KBOXNO,PBOXSZ,CD_MODE,&
 & KOBSTYPE_RQ,KCDTYPE_RQ,KSENSOR_RQ,KVNM_RQ)  

!**** SUBROUTINE NEW_THINNER - HORIZONTAL THINNING OF SATELLITE DATA

!          HEIKKI JARVINEN   ECMWF          15/08/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999
!          ERIK ANDERSSON    ECMWF          01/02/2000

!*    PURPOSE
!     -------

!        REDUCE THE NUMBER OF SATELLITE REPORTS TO A SUITABLE DENSITY.

!**   INTERFACE
!     ---------

!     ***CALL*NEW_THINNER(KLEN,KBOXNO,PBOXSZ,CD_MODE,&
!           &         KOBSTYPE_RQ,KCDTYPE_RQ,KSENSOR_RQ,KVNM_RQ)
!     WHERE KLEN       = Length of observation array
!           KBOXNO     = THINNING BOX NO.
!           PBOXSZ     = BoxSize for thinning (in degrees)
!           CD_MODE    = Modus operandii
!           KOBSTYPE_RQ= Observation type to process (-1 means ANY)
!           KCDTYPE_RQ = Code type to process        (-1 means ANY)
!           KSENSOR_RQ = Process like which sensor   (-1 means ANY)
!           KVNM_RQ    = Variable number to process  (-1 means whole report)

!        NEW_THINNER IS CALLED BY SUBROUTINE THINN

!*    METHOD
!     ------

!        THINNING IS PERFORMED WITH AID OF A GLOBAL
!     ARRAY IN EACH PROCESSOR OF AN MPP-COMPUTER.
!     EACH PROCESSOR PERFORMS THE THINNING OF THE GLOBAL SET OF DATA.
!     THE ARRAY OF DATA IS SORTED ACCORDING TO A KEY, THE CONTENT OF WHICH
!     DEPENDS ON WHICH DATA TYPE IS BEING THINNED.

!        ONLY ONE REPORT PER BOX IS ALLOWED THE ONE THAT HAS THE HIGHEST
!     SELECTION PRIORITY WITHIN THE BOX IS COSEN.
!     AT THE FIRST CALL OF THINNER, A STRINGENT CRITERION FOR THE BOX
!     SIZE IS APPLIED (~50KM) TO ENSURE A MINIMUM DISTANCE BETWEEN RE-
!     PORTS.  AT THE SECOND CALL THE THINNING GOES DOWN TO ~250KM RE-
!     SOLUTION (PBOXSZ) DEPENDING ON DATA TYPE.

!*    EXTERNALS
!     ---------

!        REFTIM - REFERENCE DATE

!**   MODIFICATIONS
!     -------------
!     27/08/2001  CHRISTINA KOPKEN  ECMWF Introduce percent of clear
!                    pixels of CSR into thinning for geos-radiances/METEOSAT.
!     30/10/2001  CHRISTINA KOPKEN  ECMWF introduce goes img, similar to meteosat
!     17/04/2002  CHRISTINA KOPKEN  ECMWF introduce thinning criteria for goesimg
!     05/08/2004  ANTJE DETHOF  ECMWF introduce thinning criteria for REO3 data
!     06/03/2007  PAUL POLI         MF    introduce thinning for GPSRO
!     30/03/2007  N. Bormann ECMWF Allow combined thinning of different sensors
!     14/10/2008  ANTJE DETHOF  Add varno as critrion for REO3 thinning
!     A. Fouilloux   31/03/10: use new codetype for MERIS data (replaced NSSMI by NTCWC)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMANCS  , ONLY : RMDI
USE YOMNMEV  , ONLY :  NR1EVENT
USE YOMCT0   , ONLY : LALLOPR
USE YOMTVRAD , ONLY :  &
 & MXSENSOR,CHSENSOR  
USE YOMCOCTP , ONLY : NGTHRB, NSTRO3, NTCWC
USE YOMDB
USE YOMOBS
USE ECSORT_MIX   , ONLY : KEYSORT

IMPLICIT NONE

!     DUMMY VARIABLES
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBOXNO 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBOXSZ 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CD_MODE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSTYPE_RQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYPE_RQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSENSOR_RQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVNM_RQ 
!     LOCAL VARIABLES
REAL(KIND=JPRB)   , ALLOCATABLE :: ZKEY(:,:)
INTEGER(KIND=JPIM), PARAMETER   :: icolumns=4,ibx=2,itm=3,isq=4,ijo=1
INTEGER(KIND=JPIM), PARAMETER   :: I_MKEY(icolumns-1)=(/ibx,itm,isq/)

INTEGER(KIND=JPIM) :: ISENSOR,IVARNO, &
 & IOBS, IKEY, IOBSTYPE, &
 & IOLDKEY, ILEN, IACTIVE, ICOUNT, &
 & IRET, ICDTYPE, ITHINNED  
INTEGER(KIND=JPIM) :: IRSTA(KLEN,4),ICV(KLEN)
INTEGER(KIND=JPIM) :: JOBS,JACTIVE,J,JBODY
CHARACTER(LEN=8) :: CL_CDSENSOR
CHARACTER(LEN=3) :: CLREO3

LOGICAL, DIMENSION(KLEN) :: LL_ACTIVE,LL_TYPE
LOGICAL :: LL_SATOB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "gersta_v.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        0.       INITIALIZE
!                 ----------
!*
!           1.1   LOCAL VARIABLES

IF (LHOOK) CALL DR_HOOK('NEW_THINNER',0,ZHOOK_HANDLE)
ILEN= KLEN
LL_SATOB = (CD_MODE == 'SATOB' .OR. CD_MODE == 'SATAM')

IF(CD_MODE == 'STMCD') THEN
! Do nothing
ELSEIF(CD_MODE == 'RAD1C') THEN
! Do nothing
ELSEIF(CD_MODE == 'GEOS') THEN
! Do nothing
ELSEIF(CD_MODE == 'TCWC') THEN
! Do nothing
ELSEIF(CD_MODE == 'SSMIS') THEN
! Do nothing
ELSEIF(CD_MODE == 'SATOB') THEN
! Do nothing
ELSEIF(CD_MODE == 'SATAM') THEN
! Do nothing
ELSEIF(CD_MODE == 'SCATT') THEN
! Do nothing
ELSEIF(CD_MODE == 'REO3') THEN
! Do nothing
ELSEIF(CD_MODE == 'GPSRO') THEN
! Do nothing
ELSE
  WRITE(NULERR,'(''BAD THINNING MODE IN NEW_THINNER '',A5)') CD_MODE
  CALL ABOR1('NEW_THINNER: BAD THINNING MODE')
ENDIF

IF(CD_MODE /= 'REO3' .AND. KSENSOR_RQ >= 0 .AND. KSENSOR_RQ <= MXSENSOR) THEN
  CL_CDSENSOR=CHSENSOR(KSENSOR_RQ)
ELSEIF(CD_MODE == 'REO3') THEN
  WRITE(CLREO3,'(I3.3)') KSENSOR_RQ
  CL_CDSENSOR='     '//CLREO3
ELSE
  CL_CDSENSOR='        '
ENDIF


!     ------------------------------------------------------------------

!        1.       LOOP OBSERVATIONS
!                 -----------------

!           1.1   INDICATE WHICH JOBS ARE THE REQUESTED TYPE OF DATA

DO JOBS = 1,ILEN
  IOBSTYPE  = ROBHDR(JOBS,MDBOTP)
  ICDTYPE   = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)
  IVARNO  = 0
  IF(ICDTYPE == NGTHRB ) THEN
    ISENSOR = ROBHDR(JOBS,MDBSSIA)
  ELSEIF(ICDTYPE == NSTRO3) THEN
    ISENSOR = ROBHDR(JOBS,MDBSSIA)
    IVARNO  = 0
    DO JBODY=MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
      IF(ROBODY(JBODY,MDBVNM) == KVNM_RQ) IVARNO=ROBODY(JBODY,MDBVNM)
    ENDDO
  ELSEIF( ICDTYPE == NTCWC) THEN
    ISENSOR = ROBHDR(JOBS,MDBSSIA)
  ELSE
    ISENSOR = 0
  ENDIF

  IF(ICDTYPE == NSTRO3) THEN
    LL_TYPE(JOBS)=(IOBSTYPE == KOBSTYPE_RQ.OR. KOBSTYPE_RQ== -1) .AND. &
     & (ICDTYPE  == KCDTYPE_RQ .OR. KCDTYPE_RQ == -1) .AND. &
     & (IVARNO   == KVNM_RQ    .OR. KVNM_RQ == -1)
  ELSEIF( ICDTYPE == NTCWC) THEN
    LL_TYPE(JOBS)=(IOBSTYPE == KOBSTYPE_RQ.OR. KOBSTYPE_RQ== -1) .AND. &
     & (ICDTYPE  == KCDTYPE_RQ .OR. KCDTYPE_RQ == -1) .AND. &
     & (ISENSOR  == KSENSOR_RQ .OR. KSENSOR_RQ == -1)
  ELSE
    LL_TYPE(JOBS)=(IOBSTYPE == KOBSTYPE_RQ.OR. KOBSTYPE_RQ== -1) .AND. &
     & (ICDTYPE  == KCDTYPE_RQ .OR. KCDTYPE_RQ == -1)
  ENDIF

ENDDO

ICOUNT=COUNT(LL_TYPE(1:ILEN))
WRITE(NULOUT,'('' NEW_THINNER: MATCHING REQUEST COUNT - '',I8,2I4,2(1X,A8),F8.2)') &
 & ICOUNT,KOBSTYPE_RQ,KCDTYPE_RQ,CL_CDSENSOR,CD_MODE,PBOXSZ  
IF(ICOUNT == 0)  GO TO 8

!*          1.3   VARIOUS UN_VECTORIZABLE STUFF

LL_ACTIVE(:)=.FALSE.
CALL GERSTA_V(ROBHDR(1:ILEN,MDBRST),ICV,IRSTA)

OBS_LOOP1: DO JOBS = 1,ILEN
  IF(.NOT.LL_TYPE(JOBS)) CYCLE OBS_LOOP1

!*          1.3.1 REPORT STATUS

  IF(IRSTA(JOBS,2) == 1 .OR. IRSTA(JOBS,3) == 1 .OR. IRSTA(JOBS,4) == 1) &
   & CYCLE OBS_LOOP1  

!*          1.3.2 ACTIVE DATA

! CALL NGENADA(JOBS,INADA)
! IF(INADA == 0) CYCLE OBS_LOOP1

!*          1.3.3 STORED SORTING KEYS

  IF(ALL(ROBHDR(JOBS,MDB_THINNINGKEY_AT_HDR(1:JP_NUMTHBOX)) == RMDI)) &
   & CYCLE OBS_LOOP1

!*          1.3.4 DEFINE ACTIVE REPORT

  LL_ACTIVE(JOBS)=.TRUE.

ENDDO OBS_LOOP1

ICOUNT=COUNT(LL_ACTIVE(1:ILEN))
WRITE(NULOUT,'('' NEW_THINNER: ACTIVE DATA COUNT      - '',I8,2I4,2(1X,A8),F8.2)') &
 & ICOUNT,KOBSTYPE_RQ,KCDTYPE_RQ,CL_CDSENSOR,CD_MODE,PBOXSZ  
IF(ICOUNT == 0)  GO TO 8

!     ------------------------------------------------------------------

!        2.       ALLOCATE AND PRESET SORTING ARRAYS
!                 ----------------------------------

!           2.1   ALLOCATE KEY, SORTING INDEX AND POINTER/SORT ARRAY

ALLOCATE(ZKEY(ICOUNT+MOD(ICOUNT+1,2),icolumns))
IF(LALLOPR) WRITE(NULOUT,9990) 'ZKEY     ',SIZE(ZKEY     ),SHAPE(ZKEY     )
ZKEY(:,:) = 0.0_JPRB

!     ------------------------------------------------------------------

!        3.       FILL SORT ARRAY
!                 ---------------

!           3.1   LOOP OVER OBSERVATIONS FILL SORTING KEY 

IACTIVE = 0
DO JOBS = 1, ILEN
  IF(LL_ACTIVE(JOBS)) THEN
    IACTIVE = IACTIVE + 1
    ZKEY(IACTIVE,ijo) = JOBS
    ZKEY(IACTIVE,isq) = ROBHDR(JOBS,MDBONM)
  ENDIF
ENDDO

!           3.2   CROSS CHECK COUNTS

IF(IACTIVE /= ICOUNT) THEN
  WRITE(NULERR,'('' *** ERROR IN NEW_THINNER (1): '')')
  WRITE(NULERR,'(''     IACTIVE,ICOUNT = '',2I10)') IACTIVE,ICOUNT
  CALL ABOR1('NEW_THINNER: IACTIVE /= ICOUNT')
ENDIF

!*
!     ------------------------------------------------------------------

!        5.       SORT THE OBSERVATIONS
!                 ---------------------
!*

!*          5.1   GET THE REST OF SORTING KEY

DO JACTIVE = 1,ICOUNT
  IOBS = ZKEY(JACTIVE,ijo)

!*          5.1.1 BOX KEY

  IF(KBOXNO >= 1 .AND. KBOXNO <= JP_NUMTHBOX) THEN
    ZKEY(JACTIVE,ibx) = ROBHDR(IOBS,MDB_THINNINGKEY_AT_HDR(KBOXNO))
  ENDIF

!*          5.1.2 TIME KEY

  ZKEY(JACTIVE,itm) = ROBHDR(IOBS,MDB_THINNINGTIMEKEY_AT_HDR)

!*          5.1.3 CLOSE THE LOOP

ENDDO

!*          5.9    SORT AND CHECK FOR SORTING ERRORS

CALL KEYSORT(IRET,ZKEY,ICOUNT,multikey=I_MKEY)

!     ------------------------------------------------------------------

!*       6.       PERFORM THE THINNING
!                 --------------------

IOLDKEY=NINT(ZKEY(1,ibx))

!*          6.3   LOOP OVER ALL ACTIVE OBSERVATIONS
!                 ALLOW ONE REPORT PER THINNING BOX ONLY

DO JACTIVE = 2,ICOUNT
  IOBS = ZKEY(JACTIVE,ijo)
  IKEY=NINT(ZKEY(JACTIVE,ibx))

  IF(IKEY == IOLDKEY) THEN
    LL_ACTIVE(IOBS)=.FALSE.
  ELSE
    IOLDKEY = IKEY
  ENDIF
ENDDO

!*          6.4   FLAG THINNED DATA

IF(KVNM_RQ == -1 .OR. CD_MODE == 'REO3') THEN

!*          6.4.1 REJECT THE REPORT, if thinned

  DO J = 1,ICOUNT
    IOBS = ZKEY(J,ijo)
    IF(.NOT.LL_ACTIVE(IOBS)) THEN
      ROBHDR(IOBS,MDBRST ) = ZCHSTAT_REJECT(ROBHDR(IOBS,MDBRST))
      ROBHDR(IOBS,MDBREV1) = ZCHEV1(ROBHDR(IOBS,MDBREV1),NR1EVENT(10))
    ENDIF
  ENDDO
ENDIF

ITHINNED=ICOUNT-COUNT(LL_ACTIVE(1:ILEN))
WRITE(NULOUT,'('' NEW_THINNER: REDUNDANT REPORT COUNT - '',I8,2I4,2(X,A8),F8.2/)') &
 & ITHINNED,KOBSTYPE_RQ,KCDTYPE_RQ,CL_CDSENSOR,CD_MODE,PBOXSZ  

!     ------------------------------------------------------------------

!        7.       RELEASE WORK SPACE
!                 ------------------

DEALLOCATE(ZKEY)
IF(LALLOPR) WRITE(NULOUT,9991) 'ZKEY     '

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
9991 FORMAT(1X,'ARRAY ',A10,' DEALLOCATED ')

!     ------------------------------------------------------------------

!        8.       RETURN
!                 ------
8 CONTINUE
IF (LHOOK) CALL DR_HOOK('NEW_THINNER',1,ZHOOK_HANDLE)
END SUBROUTINE NEW_THINNER
