SUBROUTINE OBADAT

!**** SUBROUTINE OBADAT - FIND DATE AND TIME FROM OBSERVATION ARRAY

!        D. VASILJEVIC     ECMWF     24/3/90

!     PURPOSE
!     -------

!        FIND THE DATE, TIME AND OTEHR INFORMATIONS FROM A OBS. (CMA)
!     ARRAY FILE

!**   INTERFACE
!     ---------

!        CALL OBADAT

!        OBADAT IS CALLED BY SUBROUTINE SUDIMO

!**   METHOD
!     ------

!        THE ANALYSIS DATE, TIME AND OTHER INFORMATIONS ARE EXTRACTED FROM
!     A OBS. (CMA) ARRAY FILE DDRS. FOR EXAMPLE A NUMBER OF PARAMETERS :
!     NO. OF OBS., LENGHT OF ALL OBS. REPORTS ETC., WILL BE DONE AT LATTER
!     STAGE (AFTER PREOB IS MODIFIED).

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------

!        D. VASILJEVIC 23/05/91 : INTRODUCTION OF PROPER TIME SLOTS
!        H. JARVINEN   20/09/95 : NEW CMA- FILE FORMAT
!        M. HAMRUD     20/09/95 : NEW CMA- FILE FORMAT
!        Ph. Caille    10/03/96 : CANARI adaptation to CMA file new format
!        H. JARVINEN   15/07/96 : INTRODUCE SSM/I DATA
!        S. Saarinen   14/11/97 : Support for the new CMA I/O-library
!        H. Jarvinen   14/01/98 : Support re-running IFS with used CMA
!        E. Gerard     05/05/99 : LSSPASS can be .FALSE.
!        S. Saarinen   19/06/02 : Call a new routine SUOBSADDR
!        N. Bormann    23/12/04 : RARAD becomes LIMB
!        C. Payan      13/04/07 : LMKCMARPL 2D array
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NOBTOT   ,NOBSLN   ,NACTIM   ,NDZLAY
USE YOMCT0   , ONLY : NCONF    ,NSTOP    ,LCANARI  ,LOBSC1   ,&
 & LNOBGON, LSCREEN  
USE YOMCT0B  , ONLY : LECMWF
USE YOMECTAB , ONLY : NTSLPT   ,NTSLTOB  ,NTSLTOBOFF,NTSLTOV  ,&
 & NTSLTOVOFF,NTSLNTV  ,NTSLNTVOFF,NTSLSCA  ,&
 & NTSLSCAOFF  
USE YOMCMDDR , ONLY : NLEND1I  ,NCD1OT   ,NCD1OD   ,NCD1NPU  ,&
 & NCD1NUP  ,NCD1AD   ,NCD1RM   ,&
 & NCD1TO   ,NCD1NSY  ,NCD1NAI  ,NCD1NSA  ,NCD1NDB  ,&
 & NCD1NTE  ,NCD1NPI  ,NCD1NSM  ,NCD1NT   ,NCD1NPA  ,&
 & NCD1SC   ,NCD1NSSM ,NCD1AL   ,NCD11DL  ,NCD11DV  ,&
 & NCD11DS  ,NCD1MPF  ,NCD1MSV  ,NCD1MLV  ,NCD1MUV  ,&
 & NCD11DLS ,NCD11DVS ,NCD11DSS ,NLEND2I  ,NCD2TL   ,&
 & NCD1SOH  ,NCD1TSL_NUMSLOTS  
USE YOMANA   , ONLY : NANDAT   ,NANTIM   ,NANMIN
USE YOMOBS
USE YOMMKODB
USE YOMVAR   , ONLY : NUPTRA
USE YOMLUN   , ONLY : NULOUT   ,NULERR   ,NULNAM
USE YOMCMA
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMSCC   , ONLY : LSRERUN
USE YOMOBA   , ONLY : MOBDD1   ,MOBDD2
USE YOMCMDDR

IMPLICIT NONE

#include "namobs.h"
#include "nammkodb.h"

INTEGER(KIND=JPIM) :: IDDR1(NLEND1I),IDDR2(NLEND2I)
INTEGER(KIND=JPIM) :: IHHMNSS, IHHMOD, IMNMOD, IMODPFI, ISSMOD, IUPTRA
CHARACTER (LEN = 40) ::  CLREM1,CLREM2,CLREM3,CLREM4,CLREM5,CLREM6
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "init_odb_tables.h"
END INTERFACE

!     ------------------------------------------------------------------
!*       1.       INITIAL DEFINITIONS
!                 -------------------
#include "abor1.intfb.h"
#include "ifsodbddr1f.intfb.h"
#include "ifsodbddr1s.intfb.h"
#include "ifsodbddr2f.intfb.h"
#include "ifsodbddr2s.intfb.h"
#include "posnam.intfb.h"
#include "rd_obs_boxes.intfb.h"
#include "readoba.intfb.h"
#include "suanct.intfb.h"
#include "suobsaddr.intfb.h"

IF (LHOOK) CALL DR_HOOK('OBADAT',0,ZHOOK_HANDLE)
LECFIO = .TRUE.

CLREM1 = '****************************************'
CLREM2 = '*                                      *'
CLREM3 = '*  ATTENTION    EXP. SANS OBSERVATION  *'
CLREM4 = '*  ---------                           *'
CLREM5 = '*  WARNING    RUN WITHOUT OBSERVATION  *'
CLREM6 = '*  -------                             *'

!     ------------------------------------------------------------------
!*       1.       PROCESS OBS. ARRAY DDR'S
!                 ------------------------

WRITE(NULOUT,FMT='('' ***  OBADAT  ***'')')
CALL SUANCT

!*          1.1   OPEN ODB, READ DDR'S

CALL READOBA

IF (LECFIO) THEN
  IDDR1(:)=MOBDD1(:)
  IDDR2(:)=MOBDD2(:)
ELSE
  IF ( LNOBGON ) THEN
    GOTO 2000
  ELSE
    CALL ABOR1('OBADAT: PROBLEM OPENING CMAFILE ')
  ENDIF
ENDIF

!*          1.1.1 GET NAMELISTS AND FIX DDRS HERE FOR NOW

CALL POSNAM(NULNAM,'NAMOBS')
READ(NULNAM,NAMOBS)
CALL POSNAM(NULNAM,'NAMMKODB')
READ(NULNAM,NAMMKODB)
IF(LODBDDRMOD) THEN
  WRITE(NULOUT,*) 'OBADAT; DDRS TO BE MODIFIED '
  CALL FLUSH(NULOUT)
  IF(LODBDDRMODF)     THEN
    WRITE(NULOUT,*) 'OBADAT; FULL DDRS MODIFICATION '
    CALL FLUSH(NULOUT)
    CALL IFSODBDDR1F(IDDR1(NCD1SQ),&
     & IDDR1,&
     & IDDR1(NCD1DR ),IDDR1(NCD1MR ),IDDR1(NCD1RM ),IDDR1(NCD1TO  ),&
     & IDDR1(NCD1NSY),IDDR1(NCD1NAI),IDDR1(NCD1NSA),IDDR1(NCD1NDB ),&
     & IDDR1(NCD1NTE),IDDR1(NCD1NPI),IDDR1(NCD1NSM),IDDR1(NCD1NT  ),&
     & IDDR1(NCD1NAT ),&
     & IDDR1(NCD1NPA),IDDR1(NCD1SC ),IDDR1(NCD1NLM),IDDR1(NCD1NSSM),&
     & IDDR1(NCD1NRO3),&
     & IDDR1(NCD1AL  )                                               )  
    CALL IFSODBDDR2F(IDDR2,15,15,15)
  ELSEIF(LODBDDRMODS) THEN
    WRITE(NULOUT,*) 'OBADAT; SHORT DDRS MODIFICATION '
    CALL FLUSH(NULOUT)
    CALL IFSODBDDR1S(IDDR1)
    CALL IFSODBDDR2S(IDDR2)
  ELSE
    WRITE(NULOUT,*) 'POBLEM WITH DDRS '
    CALL ABOR1('SUBROUTINE OBADAT')
  ENDIF
ENDIF

!*          1.1.3 GET INFORMATION ABOUT NUMBER OF UPDATES.

LSRERUN = .FALSE.
IF(LOBSC1.OR.NCONF == 131) THEN
  IUPTRA = IDDR1(NCD1NUP)
  WRITE(NULOUT,*) 'NUPTRA FROM NAMELIST = ',NUPTRA
  WRITE(NULOUT,*) 'NUPTRA FROM CMA      = ',IUPTRA
  IF(NUPTRA /= IUPTRA) THEN
    WRITE(NULOUT,*) '** WARNING NUPTRA INCONSISTENCY    **'
    WRITE(NULERR,*) '** WARNING NUPTRA INCONSISTENCY    **'
    WRITE(NULOUT,*) '** i.e. bad CMA file or job rerun  **'
    LSRERUN = .TRUE.
    WRITE(NULOUT,*) 'LSRERUN -> TRUE '
  ENDIF
  IF(LOBSC1) THEN
    NUPTRA=NUPTRA+1
    WRITE(NULOUT,*) 'NUPTRA UPDATED -> NUPTRA = ',NUPTRA
  ENDIF
ENDIF

!*          1.1.4 FOR THE MOMENT  RE-RUN DOES NOT WORK IN SCREENING WITH MKCMARPL=true
!                 ==> Need to fix this ASAP; for now re-run *FULL* MAKEODB

IF (LSRERUN .AND. LSCREEN .AND. ANY(LMKCMARPL)) THEN
  WRITE(NULERR,*) '*** ERROR: FOR THE MOMENT RE-RUN DOES NOT WORK IN SCREENING WITH MKCMARPL=true'
!!  WRITE(NULERR,*) '           PLEASE RE-RERUN THE MAKEODB-family and RE-SUBMIT analysis job'
  WRITE(NULERR,*) '           PLEASE FORCE TO EXECUTE THE RESTARTODB-task and RE-SUBMIT the analysis job'
  CALL ABOR1('OBADAT: RE-RUN DOES NOT WORK IN SCREENING WITH MKCMARPL=true')
ENDIF

!*          1.2   EXTRACT DATE AND TIME

NANDAT  = IDDR1(NCD1OD)
NANTIM  = IDDR1(NCD1OT)
IHHMOD  = NSSSSS / 3600
IMNMOD  = ( NSSSSS - IHHMOD*3600 ) / 60
ISSMOD  = NSSSSS - IHHMOD*3600 - IMNMOD*60
IHHMNSS = IHHMOD*10000 + IMNMOD*100 + ISSMOD

!*          1.3   DATE AND TIME CONTROL

IF ( LCANARI .AND. (NANDAT /= NINDAT.OR.NANTIM /= IHHMNSS) ) THEN
  WRITE (NULOUT,131) NANDAT,NANTIM,NINDAT,IHHMNSS
  131 FORMAT(//,' Incoherences date ou heure ',//,' date fichier ',&
   & 'd''obs. : ',I8,'     heure fichier d''obs. : ',I6,/,&
   & ' date modele         : ',I8,'     heure modele     ',&
   & '    : ',I6,//)  
  LECFIO = .FALSE.
  IF ( LNOBGON ) THEN
    GOTO 2000
  ELSE
    CALL ABOR1('SUBROUTINE OBADAT')
  ENDIF
ELSEIF ( .NOT.LECMWF .AND. (NCONF == 1.OR.NCONF == 302) .AND. (NANDAT /= NINDAT.OR.NANTIM /= IHHMNSS) ) THEN
  WRITE (NULOUT,132) NANDAT,NANTIM,NINDAT,IHHMNSS
  132 FORMAT(//,' Incoherences date ou heure ',//,' date fichier ',&
   & 'd''obs. : ',I8,'     heure fichier d''obs. : ',I6,/,&
   & ' date modele         : ',I8,'     heure modele     ',&
   & '    : ',I6,//)  
  IF(NSTOP == 0)THEN
    CALL ABOR1(' OBADAT : MAUVAISE DATE OU FICHIER SANS OBSERVATION ')
  ELSE
    WRITE (NULOUT,*) ' WARNING OBADAT : MAUVAISE DATE OU FICHIER SANS OBSERVATION '
  ENDIF
ENDIF

!*          1.7   PRESET OBSERVATION RELATED PARAMETERS

!*          1.7.1 TOTAL NO. CMA FILES

NTSLPT (:) = 0
NTSLTOB(:) = 0
NTSLTOV(:) = 0
NTSLNTV(:) = 0
NTSLSCA(:) = 0
NTSLTOBOFF(:) = 0
NTSLTOVOFF(:) = 0
NTSLNTVOFF(:) = 0
NTSLSCAOFF(:) = 0

!*          1.9.7 TOTAL NO. OF TIME SLOTS

NACTIM = IDDR1(NCD1TSL_NUMSLOTS)

WRITE(NULOUT,*) 'NACTIM: ',NACTIM

!*          1.9.8 TOTAL NO. OF OBS. DEP.

NUSDUPD = IDDR1(NCD1NUP)
NRESUPD = IDDR1(NCD1NPU)
!*AF 14/03/2010 NODEPT  = IDDR1(NCD1AD )
IF(NRESUPD == 0) THEN
  LINCREM = .FALSE.
ELSE
  LINCREM = .TRUE.
ENDIF

IF((NCONF == 131.AND. NUPTRA+1 > NRESUPD) .OR.&
   & (LOBSC1.AND. NUPTRA   > NRESUPD)       ) THEN  
  WRITE(NULERR,'('' *** ERROR IN OBADAT'')')
  WRITE(NULERR,'('' NUPTRA TOO BIG. CMA RE-USED? '')')
  WRITE(NULERR,*) ' NUPTRA: ',NUPTRA,' NRESUPD: ',NRESUPD
  LECFIO = .FALSE.
  IF ( LNOBGON ) THEN
    GOTO 2000
  ELSE
    CALL ABOR1('OBADAT: NUPTRA TOO BIG. CMA RE-USED? ')
  ENDIF
ENDIF

!*          1.10.3  NUMBER OF SATEM LAYERS

NDZLAY= IDDR2(NCD2TL)
IF(.NOT.LECMWF)THEN
  IF ( NDZLAY /= 14.AND. NDZLAY /= 7 ) NDZLAY = 7
ENDIF

!*          1.10.4  NO. OF 1D VAR RETRIEVED LEVELS (TOVS & SSMI)

!*AF 14/04/2010 NO1DVLV  = IDDR1(NCD11DL)
!*AF 14/04/2010 NO1DVLVS = IDDR1(NCD11DLS)

!*          1.10.5  NO. OF 1D VAR  UPPER AIR RETRIEVED VARIABLES

!*AF 14/04/2010 NO1DVVA  = IDDR1(NCD11DV)
!*AF 14/04/2010 NO1DVVAS = IDDR1(NCD11DVS)

!*          1.10.6  NO. OF 1D VAR SINGLE LEVEL RETRIEVED VARIABLES

!*AF 14/04/2010 NO1DVSLV = IDDR1(NCD11DS)
!*AF 14/04/2010 NO1DSLVS = IDDR1(NCD11DSS)

!*          1.10.7  MODEL PROFILE IND.

!*AF 14/04/2010 IMODPFI = IDDR1(NCD1MPF)

!*AF 14/04/2010 LMODPRF = BTEST(IMODPFI,0)
!*AF 14/04/2010 LMPSYNOP= BTEST(IMODPFI,1)
!*AF 14/04/2010 LMPAIREP= BTEST(IMODPFI,2)
!*AF 14/04/2010 LMPSATOB= BTEST(IMODPFI,3)
!*AF 14/04/2010 LMPDRIBU= BTEST(IMODPFI,4)
!*AF 14/04/2010 LMPTEMP = BTEST(IMODPFI,5)
!*AF 14/04/2010 LMPPILOT= BTEST(IMODPFI,6)
!*AF 14/04/2010 LMPSATEM= BTEST(IMODPFI,7)
!*AF 14/04/2010 LMPPAOB = BTEST(IMODPFI,8)
!*AF 14/04/2010 LMPSCAT = BTEST(IMODPFI,9)
!*AF 14/04/2010 LMPLIMB = BTEST(IMODPFI,10)

!*          1.10.8  NO. OF MODEL SURFACE VARIABLES

!*AF 14/04/2010 NOMODSV = IDDR1(NCD1MSV)

!*          1.10.9  NO. OF MODEL LEVELS

!*AF 14/04/2010 NOMODLV = IDDR1(NCD1MLV)

!*          1.10.10 NO. OF MODEL UPPERAIR VARIABLES

!*AF 14/04/2010 NOMODUV = IDDR1(NCD1MUV)

!*          1.10.11 TOVS/SSMI/RAD1C OPTIONAL HEADERS
!                   (TEMPORARY SOLUTION) 

!*AF 14/04/2010 WRITE(NULOUT,FMT='('' OBADAT; NCD1SOH,IDDR1(NCD1SOH):'',&
!*AF 14/04/2010  & 2(I10,1x))') NCD1SOH,IDDR1(NCD1SOH)  

!*AF 14/04/2010 LTOVSOH = BTEST(IDDR1 (NCD1SOH), 0)
!*AF 14/04/2010 LTOPASS = BTEST(IDDR1 (NCD1SOH), 1)
!*AF 14/04/2010 LTO1DVA = BTEST(IDDR1 (NCD1SOH), 2)
!*AF 14/04/2010 LTO1DSL = BTEST(IDDR1 (NCD1SOH), 3)
!*AF 14/04/2010 LTO1DUL = BTEST(IDDR1 (NCD1SOH), 4)
!*AF 14/04/2010 LTOBICO = BTEST(IDDR1 (NCD1SOH), 5)
!*AF 14/04/2010 WRITE(NULOUT,FMT='('' OBADAT; LTOVSOH, LTOPASS, LTO1DVA,'',&
!*AF 14/04/2010  & '' LTO1DSL, LTO1DUL, LTOBICO: '',6(L1,1x))')&
!*AF 14/04/2010  & LTOVSOH,LTOPASS,LTO1DVA,LTO1DSL,LTO1DUL,LTOBICO  
!*AF 14/04/2010 IF (.NOT.LTOVSOH .OR. .NOT.LTOPASS .OR. &
!*AF 14/04/2010    & .NOT.LTO1DVA .OR. .NOT.LTO1DSL .OR. .NOT.LTO1DUL) THEN  
!*AF 14/04/2010   CALL ABOR1('OBADAT: PROBLEM WITH TOVS OPTIONAL HEADER ')
!*AF 14/04/2010 ENDIF

!*AF 14/04/2010 LATOVOH = BTEST(IDDR1 (NCD1SOH),20)
!*AF 14/04/2010 LATPASS = BTEST(IDDR1 (NCD1SOH),21)
!*AF 14/04/2010 LAT1DVA = BTEST(IDDR1 (NCD1SOH),22)
!*AF 14/04/2010 LAT1DSL = BTEST(IDDR1 (NCD1SOH),23)
!*AF 14/04/2010 LAT1DUL = BTEST(IDDR1 (NCD1SOH),24)
!*AF 14/04/2010 LATBICO = BTEST(IDDR1 (NCD1SOH),25)
!*AF 14/04/2010 WRITE(NULOUT,FMT='('' OBADAT; LATOVOH, LATPASS, LAT1DVA,'',&
!*AF 14/04/2010  & '' LAT1DSL, LAT1DUL, LATBICO: '',6(L1,1x))')&
!*AF 14/04/2010  & LATOVOH,LATPASS,LAT1DVA,LAT1DSL,LAT1DUL,LATBICO  
!*AF 14/04/2010 IF (.NOT.LATOVOH .OR. .NOT.LATPASS .OR. .NOT.LATBICO) THEN
!*AF 14/04/2010   CALL ABOR1('OBADAT: PROBLEM WITH RAD1C OPTIONAL HEADER ')
!*AF 14/04/2010 ENDIF

!  switch put by FT cause problem with MF obs files (10/06/98)
!*AF 14/04/2010 IF (LECMWF) THEN
!*AF 14/04/2010   LSSMIOH = BTEST(IDDR1 (NCD1SOH),10)
!*AF 14/04/2010   LSSPASS = BTEST(IDDR1 (NCD1SOH),11)
!*AF 14/04/2010   LSS1DVA = BTEST(IDDR1 (NCD1SOH),12)
!*AF 14/04/2010   LSS1DSL = BTEST(IDDR1 (NCD1SOH),13)
!*AF 14/04/2010   LSS1DUL = BTEST(IDDR1 (NCD1SOH),14)
!*AF 14/04/2010   WRITE(NULOUT,FMT='('' OBADAT; LSSMIOH,'',&
!*AF 14/04/2010    & '' LSSPASS, LSS1DVA, LSS1DSL, LSS1DUL: '',&
!*AF 14/04/2010    & 5(L1,1x))') LSSMIOH,LSSPASS,LSS1DVA,LSS1DSL,LSS1DUL  
!*AF 14/04/2010   IF (.NOT.LSSMIOH .OR. .NOT.LSS1DVA .OR. &
!*AF 14/04/2010      & .NOT.LSS1DSL .OR. .NOT.LSS1DUL     ) THEN  
!*AF 14/04/2010     CALL ABOR1('OBADAT: PROBLEM WITH SSMI OPTIONAL HEADER ')
!*AF 14/04/2010   ENDIF
!*AF 14/04/2010 ELSE
!*AF 14/04/2010   LSSMIOH = .FALSE.
!*AF 14/04/2010   LSSPASS = .FALSE.
!*AF 14/04/2010   LSS1DVA = .FALSE.
!*AF 14/04/2010   LSS1DSL = .FALSE.
!*AF 14/04/2010   LSS1DUL = .FALSE.
!*AF 14/04/2010 ENDIF

!- Update DDRs

IF (LECFIO) THEN
  MOBDD1(:)=IDDR1(:)
  MOBDD2(:)=IDDR2(:)
ENDIF

!*          1.8   LOOP OVER ALL FILES

IF ( .NOT. LECFIO ) GOTO 2000

CALL RD_OBS_BOXES

!     ------------------------------------------------------------------
!*          2.  INITIALISATIONS AND PRINTS IF UNSUCCESSFUL READING
!*              OF CMA FILES IN OPERATIONAL MODE ( LNOBGON = .T. )
!               --------------------------------------------------

2000 CONTINUE

CALL SUOBSADDR(-1)

IF (LNOBGON) THEN
  IF ( .NOT.LECFIO .OR. NOBTOT == 0 ) THEN
    WRITE(NULERR,*) CLREM1
    WRITE(NULERR,*) CLREM2
    WRITE(NULERR,*) CLREM3
    WRITE(NULERR,*) CLREM4
    WRITE(NULERR,*) CLREM2
    WRITE(NULERR,*) CLREM1

    WRITE(NULERR,*) CLREM1
    WRITE(NULERR,*) CLREM2
    WRITE(NULERR,*) CLREM5
    WRITE(NULERR,*) CLREM6
    WRITE(NULERR,*) CLREM2
    WRITE(NULERR,*) CLREM1

    NOBTOT = 1
    NACTIM = 1
    NOBSLN = 1
  ENDIF
ENDIF

!     ------------------------------------------------------------------
!*          3.  OTHERS PRINTS
!               -------------

WRITE(NULOUT,'('' LINCREM= '',L5,'' NUSDUPD= '',I4,&
 & '' NRESUPD= '',I4,'' NODEPT= '', I4,'' NUPTRA= '',I4)')&
 & LINCREM,NUSDUPD,NRESUPD,NODEPT,NUPTRA  

!*AF 14/04/2010 WRITE(NULOUT,'('' NDZLAY= '',I4,'' NO1DVLV= '',I4,&
!*AF 14/04/2010  & '' NO1DVVA= '',I4,'' NO1DVSLV= '', I4)')&
!*AF 14/04/2010  & NDZLAY,NO1DVLV,NO1DVVA,NO1DVSLV  

!*AF 14/04/2010 WRITE(NULOUT,'('' NO1DVLVS= '',I4,'' NO1DVVAS= '',I4,&
!*AF 14/04/2010  & '' NO1DSLVS= '',I4)')&
!*AF 14/04/2010  & NO1DVLVS,NO1DVVAS,NO1DSLVS  

!*AF 14/04/2010 WRITE(NULOUT,'('' NOMODSV= '',I4,'' NOMODLV= '',I4,&
!*AF 14/04/2010  & '' NOMODUV= '',I4)')&
!*AF 14/04/2010  & NOMODSV,NOMODLV,NOMODUV  

!     ------------------------------------------------------------------

IF (LSCREEN) THEN
  CALL INIT_ODB_TABLES  ! will create some empty tables if needed (depends on what is read in the namelists)
ENDIF

IF (LHOOK) CALL DR_HOOK('OBADAT',1,ZHOOK_HANDLE)
END SUBROUTINE OBADAT
