SUBROUTINE OBINSTP(PLAT   ,PLON   ,KSTID  ,&
 & KOBTYP ,KCDTYP ,&
 & KINSTP                           )  

!**** SUBROUTINE  OBINSTP - OBS. INSTRUMENT TYPE SPECIFICATIONS

!        D. VASILJEVIC     ECMWF     04/07/94

!     PURPOSE
!     -------

!        DEFINE OBS. INSTRUMENT TYPE SPECIFICATIONS

!**   INTERFACE
!     ---------

!        CALL OBINSTP(PLAT   ,PLON   ,KSTID  ,
!                     KOBTYP ,KCDTYP ,KOTMAP ,KOCMAP ,
!                     KCHAN  ,KINS   ,KOTMAP ,
!                     KINSTP                          )
!     WHERE:
!           PLAT    = LATITUDE
!           PLON    = LONGITUDE
!           KSTID   = STATION ID.
!           KOBTYP  = OBS. TYPE
!           KCDTYP  = OBS. CODE TYPE
!           KINSTP  = INSTRUMENT TYPE

!        OBINSTP IS CALLED BY SUBROUTINES:

!     METHOD
!     ------

!        THE OBS. INSTRUMENT SPECIFICATIONS ARE STORED AT AN
!     APPROPIRIATE PLACE.

!     EXTERNALS
!     ---------

!        ENDRUN   - ABORT
!        SATEMIS  - SATEM INSTRUMENT SPEC.
!        TOVSLRIS - TOVS LOW RESOLUTION INSTRUMENT SPEC.
!        TOVSHRIS - TOVS HIGH RESOLUTION INSTRUMENT SPEC.

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        N.Bormann     23-Dec-2004 RARAD becomes LIMB

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : NMDI
USE YOMINSTP , ONLY : NSYNINTP ,NSHPINTP ,NACFINTP ,NSTBITGO ,&
 & NSTBITME ,NSTBITIN ,NSTBITJA  
USE YOMLIM   , ONLY : NSTBSID1 ,NSTBSID2 ,NSTBSID3 ,NSTBSID4 ,NSTBSID5, &
 & NSTBSID_TERRA,NSTBSID_AQUA
USE YOMCOCTP , ONLY : NSYNOP   ,NSRSCD   ,NATSCD   ,NMETAR   ,NPGPS    ,&
 & NAIREP   ,&
 & NSATOB   ,NDRIBU   ,NTEMP    ,NPILOT   ,NSATEM   ,&
 & NSTMCD   ,NSTOVS   ,NGTHRB   ,NGTHR1   ,&
 & NGTHR2   ,NPAOB    ,NSCATT   ,NLIMB  
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTID 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KINSTP 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!*
!-----------------------------------------------------------------------

!        0.             PRESET
!                       ------

!*          0.1         MAP OBS. CODE TYPE INTO SQ. NO. (ALREADY DONE)

!*          0.2         INSTRUMENT TYPE

IF (LHOOK) CALL DR_HOOK('OBINSTP',0,ZHOOK_HANDLE)
KINSTP = NMDI

!*          0.3         LOGICAL SWITCHES

!*
!-----------------------------------------------------------------------

!        1.             SYNOP
!                       -----

IF(KOBTYP == NSYNOP)          THEN

!*          1.1         DEFINE INS. TYPE

!*          1.1.1       LAND STATIONS

  IF(KCDTYP == NSRSCD.OR.KCDTYP == NATSCD.OR.KCDTYP == NMETAR)      THEN
    KINSTP  = NSYNINTP

!*          1.1.2       GROUND-BASED GPS

  ELSEIF(KCDTYP == NPGPS) THEN
    KINSTP  = NMDI

!*          1.1.3       SEA STATIONS (SHIPS)

  ELSE
    KINSTP  = NSHPINTP
  ENDIF
!*
!-----------------------------------------------------------------------

!        2.             AIREP
!                       -----

ELSEIF(KOBTYP == NAIREP)     THEN

!*          2.1         INST. TYPE

  KINSTP  = NACFINTP
!*
!-----------------------------------------------------------------------

!        3.             SATOB
!                       -----

ELSEIF(KOBTYP == NSATOB)     THEN
  IF(KSTID /= NMDI)                  THEN
    IF(KSTID < NSTBSID1)              THEN
      KINSTP = NSTBITME
    ELSEIF(KSTID < NSTBSID2)         THEN
      KINSTP = NSTBITJA
    ELSEIF(KSTID < NSTBSID3 .OR. KSTID == NSTBSID_TERRA .OR. &
   & KSTID == NSTBSID_AQUA)  THEN
      KINSTP = NSTBITGO
    ELSEIF(KSTID >= NSTBSID4.AND.KSTID < NSTBSID5     )    THEN
      KINSTP = NSTBITIN
    ELSE
      WRITE(NULOUT,'(''0WARNING IN SUBROUTINE OBINSTP'')')
      WRITE(NULOUT,'('' UNKNOWN STATION ID:'',I10)')   KSTID
      WRITE(NULOUT,'('' OBS. TYPE         :'',I10)')   KOBTYP
      WRITE(NULOUT,'('' CODE TYPE         :'',I10)')   KCDTYP
      WRITE(NULOUT,'('' LATITUDE          :'',F10.2)') PLAT
      WRITE(NULOUT,'('' LONGITUDE         :'',F10.2)') PLON
      WRITE(NULOUT,'('' INS. TYPE/I1/I2I2 SET TO MISSING'')')
    ENDIF
  ENDIF
!*
!-----------------------------------------------------------------------

!        4.             DRIBU
!                       -----

ELSEIF(KOBTYP == NDRIBU)     THEN
!*
!-----------------------------------------------------------------------

!        5.             TEMP
!                       ----

ELSEIF(KOBTYP == NTEMP )     THEN
!*
!-----------------------------------------------------------------------

!        6.             PILOT
!                       -----

ELSEIF(KOBTYP == NPILOT)     THEN
!*
!-----------------------------------------------------------------------

!        7.             SATEM INSTRUMENT SPECIFICATIONS
!                       -------------------------------

ELSEIF(KOBTYP == NSATEM)     THEN

!*          7.1         SATEM/LOW RES. TOVS

  IF(KCDTYP == NSTMCD)          THEN

!*          7.1.1       INST. TYPE

    KINSTP  = 89

!*          7.2         LOW RESOLUTION TOVS

  ELSEIF(KCDTYP == NSTOVS)     THEN

!*          7.2.1       INST. TYPE

!*          7.3         HIGH RESOLUTION TOVS/RTOVS

  ELSEIF(KCDTYP == NGTHR2.OR.KCDTYP == NGTHR1    ) THEN

!*          7.3.1       INST. TYPE

!*          7.4         HIGH RESOLUTION ATOVS (LEVEL 1C/GEOS)

  ELSEIF(KCDTYP == NGTHRB)     THEN
 
!*          7.4.1       INST. TYPE

  ENDIF
!*
!-----------------------------------------------------------------------

!        8.             PAOB INSTRUMENT SPECIFICATIONS
!                       ------------------------------

ELSEIF(KOBTYP == NPAOB )     THEN

!*          8.1         INST. TYPE

!*
!-----------------------------------------------------------------------

!        9.             SCAT. INSTRUMENT SPECIFICATIONS
!                       -------------------------------

ELSEIF(KOBTYP == NSCATT)     THEN

!*          9.1         INST. TYPE

!*
!-----------------------------------------------------------------------

!        10.            LIMB OBSERVATIONS INSTRUMENT SPECIFICATIONS
!                       -------------------------------------------

ELSEIF(KOBTYP == NLIMB)     THEN
!*
!-----------------------------------------------------------------------

!        11.            UNKNOWN OBS. TYPE
!                       -----------------

ELSE
  WRITE(NULOUT,'(''0ERROR IN SUBROUTINE OBINSTP'')')
  WRITE(NULOUT,'('' OBS. TYPE NOT KNOWN:'',I10)') KOBTYP
  CALL ABOR1('SUBROUTINE OBINSTP')
ENDIF
!*
!-----------------------------------------------------------------------

!        12.            MESSAGES
!                       --------

12 CONTINUE
!*
!-----------------------------------------------------------------------

!        13.            RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('OBINSTP',1,ZHOOK_HANDLE)
END SUBROUTINE OBINSTP
