SUBROUTINE OBSGEN

!**** *OBSGEN*  - match 'observations' (analysed screen level parameters) and 
!                 the corresponding model fields

!     Purpose.
!     --------
!     (1) collocate observation time and model time step
!     (2) collocate analysis time and model time step
!     (3) read observations from grib
!     (4) extract fg fields from the global (internal) arrays

!**   Interface.
!     ----------
!        *CALL* *OBSGEN

!        Explicit arguments :      None
!        --------------------

!        Implicit arguments :      None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M. Drusch  *ECMWF*

!     Modifications.
!     --------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM    ,ONLY : NPROMA, NGPBLKS
USE YOMCT0    ,ONLY : NSTOP
USE YOMGEM    ,ONLY : NGPTOT
USE YOMRIP    ,ONLY : NSSSSS, NINDAT
USE YOMDYN    ,ONLY : TSTEP
USE YOMLUN    ,ONLY : NULOUT
USE IOSTREAM_MIX  ,ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, CLOSE_IOSTREAM, &
    &                 TYPE_IOSTREAM, TYPE_IOREQUEST, CLOSE_IOREQUEST
USE YOMSEKF   ,ONLY : NOBS_SCREEN, &
    &                 T2M_OBS, TD2M_OBS, &
    &                 NSCREENTIME, NSCREENDATE, NOBS_TSTEP, &
    &                 NANATIME, NANADATE, NANA_TSTEP, NANAL

IMPLICIT NONE

REAL(KIND=JPRB)    ::  ZHOOK_HANDLE
REAL(KIND=JPRB), ALLOCATABLE :: ZANA(:,:,:)
REAL(KIND=JPRB)    :: ZMODTIME, ZOBSTIME, ZOBSTIME_H, ZOBSTIME_M, ZMIN, ZDIFF
REAL(KIND=JPRB)    :: ZDD_OFF

INTEGER(KIND=JPIM) :: JLENG, JBLOCK, JL, JKGLO, JK
INTEGER(KIND=JPIM) :: JOBSTIM, JANATIM
INTEGER(KIND=JPIM) :: JTST, IMODTIME, IHHMOD, IMMMOD
INTEGER(KIND=JPIM) :: IOBS_TSTEP 
INTEGER(KIND=JPIM) :: IFIELDS
INTEGER(KIND=JPIM), ALLOCATABLE :: IGRIB2D(:)

TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST):: YL_IOREQUEST

CHARACTER(LEN = 22), ALLOCATABLE :: CL_OBSFILE(:)
CHARACTER(LEN = 22) :: CLF
CHARACTER(LEN = 2)  :: CLOBS

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('OBSGEN',0,ZHOOK_HANDLE)

!     1.0 Initialize fields      ---------------------------------------

NOBS_TSTEP(:) = 0
NANA_TSTEP(:) = 0

!     2.0 Timestep vs 'real' observation time    ----------------------------

WRITE(NULOUT,*) 'OBSGEN:', NOBS_SCREEN

DO JOBSTIM = 1, NOBS_SCREEN
  ZOBSTIME = NSCREENTIME(JOBSTIM)*1.0
  ZMIN = 1000.
  IMODTIME = 0
  DO JTST = 0, NSTOP
    IMODTIME = NSSSSS + (JTST * TSTEP)
    IHHMOD = IMODTIME / 3600
    IMMMOD = ((NSSSSS+ (JTST * TSTEP)) - IHHMOD * 3600) / 60
    IF (IHHMOD >= 24) IHHMOD = IHHMOD - 24
    ZMODTIME = IHHMOD + IMMMOD/60.
    ZDIFF = ABS(ZMODTIME - ZOBSTIME)
    WRITE(NULOUT,'(i3,4(x,f9.4))') JTST, ZMODTIME, ZOBSTIME, ZDIFF, ZMIN
    IF (ZDIFF < ZMIN) THEN
      ZMIN = ZDIFF
      IOBS_TSTEP = JTST 
    ENDIF
  ENDDO
  NOBS_TSTEP(JOBSTIM) = IOBS_TSTEP
ENDDO

WRITE(NULOUT,*) 'OBSGEN: Observation time - model time step matching (loop):'
DO JK = 1, NOBS_SCREEN
  WRITE(NULOUT,*) 'NSCREENTIME(JK):', NSCREENTIME(JK)
  WRITE(NULOUT,*) 'NOBS_TSTEP(JK):', NOBS_TSTEP(JK)
ENDDO

! Testing Klaus
DO JOBSTIM = 1, NOBS_SCREEN
  ZDD_OFF=(NSCREENDATE(JOBSTIM)-100*(NSCREENDATE(JOBSTIM)/100))-(NINDAT-100*(NINDAT/100))
  ZOBSTIME_H = INT(NSCREENTIME(JOBSTIM)/ 1)
  ZOBSTIME_M = 0
!  ZOBSTIME_H = INT(NSCREENTIME(JOBSTIM)/ 100)
!  ZOBSTIME_M = INT(NSCREENTIME(JOBSTIM)-ZOBSTIME_H*100)
  ZOBSTIME = ZOBSTIME_H*60*60 + ZOBSTIME_M*60 + ZDD_OFF*24*60*60
  IMODTIME = NSSSSS
  ZDIFF = ZOBSTIME - NSSSSS
  JTST=ZDIFF/TSTEP
!  JTST=ZDIFF/TSTEP+1
  WRITE(NULOUT,*) 'Test Klaus, JTST: ', ZDD_OFF, NSCREENDATE(JOBSTIM), NINDAT, NSCREENTIME(JOBSTIM), ZDIFF, JTST
ENDDO

!     3.0 Timestep vs analyses time    ----------------------------

DO JANATIM = 1, NANAL
  ZOBSTIME = NANATIME(JANATIM)*1.0
  ZMIN = 1000.
  IMODTIME = 0
  DO JTST = 0, NSTOP
    IMODTIME = NSSSSS + (JTST * TSTEP)
    IHHMOD = IMODTIME / 3600
    IMMMOD = ((NSSSSS+ (JTST * TSTEP)) - IHHMOD * 3600) / 60
    IF (IHHMOD >= 24) IHHMOD = IHHMOD - 24
    ZMODTIME = IHHMOD + IMMMOD/60.
    ZDIFF = ABS(ZMODTIME - ZOBSTIME)
    WRITE(NULOUT,'(i3,4(x,f9.4))') JTST, ZMODTIME, ZOBSTIME, ZDIFF, ZMIN
    IF (ZDIFF < ZMIN) THEN
      ZMIN = ZDIFF
      IOBS_TSTEP = JTST 
    ENDIF
  ENDDO
  NANA_TSTEP(JANATIM) = IOBS_TSTEP
ENDDO

DO JK = 1, NANAL
  WRITE(NULOUT,*) 'OBSGEN: Analysis time - model time step matching (loop):'
  WRITE(NULOUT,*) 'NANATIME(JK):', NANATIME(JK)
  WRITE(NULOUT,*) 'NANA_TSTEP(JK):', NANA_TSTEP(JK)
ENDDO

! Testing Klaus
DO JOBSTIM = 1, NANAL
  ZDD_OFF=(NANADATE(JOBSTIM)-100*(NANADATE(JOBSTIM)/100))-(NINDAT-100*(NINDAT/100))
  ZOBSTIME_H = INT(NANATIME(JOBSTIM)/ 100)
  ZOBSTIME_M = INT(NANATIME(JOBSTIM)-ZOBSTIME_H*100)
  ZOBSTIME = ZOBSTIME_H*60*60 + ZOBSTIME_M*60 + ZDD_OFF*24*60*60
  IMODTIME = NSSSSS
  ZDIFF = ZOBSTIME - NSSSSS
  JTST=ZDIFF/TSTEP+1
  WRITE(NULOUT,*) 'Test Klaus, JTST: ', NANATIME(JOBSTIM), ZDIFF, JTST
ENDDO

!     4.0 Read the 'observations' from grib and write to arrays ------------

IFIELDS = 2

ALLOCATE(T2M_OBS(NPROMA,NGPBLKS,NOBS_SCREEN))
ALLOCATE(TD2M_OBS(NPROMA,NGPBLKS,NOBS_SCREEN))
ALLOCATE(CL_OBSFILE(NOBS_SCREEN))
ALLOCATE(IGRIB2D(IFIELDS))
ALLOCATE(ZANA(NGPTOT,IFIELDS,1))

T2M_OBS(:,:,:) = 0.0_JPRB
TD2M_OBS(:,:,:) = 0.0_JPRB
ZANA(:,:,:) = 0.0_JPRB

DO JOBSTIM = 1, NOBS_SCREEN
  WRITE(CLOBS,'(i2.2)') JOBSTIM
  CL_OBSFILE(JOBSTIM) = 'sekf_an2m'//CLOBS
ENDDO

IGRIB2D(1) = 167
IGRIB2D(2) = 168

DO JOBSTIM = 1, NOBS_SCREEN

  CLF = CL_OBSFILE(JOBSTIM)

  WRITE(NULOUT,*) 'CLF:', CLF

  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLF),CDMODE='r',KIOMASTER=1)
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
   &   KGRIB2D=IGRIB2D(1:IFIELDS),KPROMA=NGPTOT)
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=ZANA(:,1:IFIELDS,:))
  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)

  DO JKGLO=1,NGPTOT,NPROMA
    JBLOCK=(JKGLO-1)/NPROMA + 1
    JLENG=MIN(NPROMA,NGPTOT-JKGLO+1)
    DO JL=1,JLENG
      T2M_OBS(JL,JBLOCK,JOBSTIM) = ZANA(JKGLO+JL-1,1,1)
      TD2M_OBS(JL,JBLOCK,JOBSTIM)= ZANA(JKGLO+JL-1,2,1)
    ENDDO
  ENDDO
ENDDO

DEALLOCATE(CL_OBSFILE)
DEALLOCATE(ZANA)
DEALLOCATE(IGRIB2D)

!     5.0 Assign the bg / perturbed fg data ---------------------------------

!ALLOCATE(VSM_FG_OBS_S(NPROMA,NSLAY,NGPBLKS,NOBS_SCREEN,N_SEKF_CV+1))
!ALLOCATE(VSM_FG_ANA(NPROMA,NSLAY,NGPBLKS,NANAL))
!ALLOCATE(T2M_FG_OBS_S(NPROMA,NGPBLKS,NOBS_SCREEN,N_SEKF_CV+1))
!ALLOCATE(TD2M_FG_OBS_S(NPROMA,NGPBLKS,NOBS_SCREEN,N_SEKF_CV+1))

!VSM_FG_OBS_S(:,:,:,:,:) = 0.
!VSM_FG_ANA(:,:,:,:) = 0.
!T2M_FG_OBS_S(:,:,:,:) = 0.
!TD2M_FG_OBS_S(:,:,:,:) = 0.

!! store soil moisture at observation times
!DO JOBSTIM = 1, NOBS_SCREEN
!  IOBS_TSTEP = NOBS_TSTEP(JOBSTIM)
!  DO JKGLO=1,NGPTOT,NPROMA
!    JBLOCK=(JKGLO-1)/NPROMA + 1
!    JLENG=MIN(NPROMA,NGPTOT-JKGLO+1)
!    DO JL=1,JLENG
!      VSM_FG_OBS_S(JL,:,JBLOCK,JOBSTIM,:) = VOL_SM_FG(JL,:,JBLOCK,IOBS_TSTEP,:)
!    ENDDO
!  ENDDO
!ENDDO

!! store soil moisture at analyses times
!DO JANATIM = 1, NANAL
!  IOBS_TSTEP = NANA_TSTEP(JANATIM)
!  DO JKGLO=1,NGPTOT,NPROMA
!    JBLOCK=(JKGLO-1)/NPROMA + 1
!    JLENG=MIN(NPROMA,NGPTOT-JKGLO+1)
!    DO JL=1,JLENG
!      VSM_FG_ANA(JL,:,JBLOCK,JANATIM) = VOL_SM_FG(JL,:,JBLOCK,IOBS_TSTEP,1)
!    ENDDO
!  ENDDO
!ENDDO

!! store screen level parameters at observation times
!DO JOBSTIM = 1, NOBS_SCREEN
!  IOBS_TSTEP = NOBS_TSTEP(JOBSTIM)
!  DO JKGLO=1,NGPTOT,NPROMA
!    JBLOCK=(JKGLO-1)/NPROMA + 1
!    JLENG=MIN(NPROMA,NGPTOT-JKGLO+1)
!    DO JL=1,JLENG
!      T2M_FG_OBS_S(JL,JBLOCK,JOBSTIM,:) = T2M_FG(JL,JBLOCK,IOBS_TSTEP,:)
!      TD2M_FG_OBS_S(JL,JBLOCK,JOBSTIM,:)= TD2M_FG(JL,JBLOCK,IOBS_TSTEP,:)
!    ENDDO
!  ENDDO
!ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('OBSGEN',1,ZHOOK_HANDLE)
END SUBROUTINE OBSGEN
