SUBROUTINE OPK_OBSCOR(KNUMOBS,PX,PY)

!**** *OPK* - Computes resultant control vector for linear operator

!     Purpose.
!     --------
!           For input vector gives the result of tangent linear,adjoint,
!        or combination of these operators acting on the input vector.

!**   Interface.
!     ----------
!        *CALL* *OPK_OBSCOR(KNUMOBS,PX,PY)

!        Explicit arguments :
!        --------------------
!        KNUMOBS: dimension of vector
!        PX : input variable  (DO NOT MODIFY in the sub)
!        PY: output variable
!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------


!     Reference.
!     ----------
!        ECMWF research department documentation of the IFS

!     Author.
!     -------
!        Gabor Radnoti    *ECMWF*

!     Modifications.
!     --------------
!        Original : 06-08-31 

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMLUN   , ONLY : NULERR
USE YOMCOSJO , ONLY : OBSCOR_HOR_LSCALE, NOBSCOR_INLEVS, LUV_OBSCOR, L3D_OBSCOR, &
                      &  OBSCOR_GRID_DEF, OBS_ARR_GL2, NMASK, NACTIM_START, NACTIM_END 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
IMPLICIT NONE

INTEGER(KIND=JPIM),  INTENT(IN)  :: KNUMOBS
REAL(KIND=JPRB),  INTENT(IN)     :: PX(KNUMOBS)
REAL(KIND=JPRB),  INTENT(OUT)    :: PY(KNUMOBS)

INTEGER(KIND=JPIM) :: IND_LAT,IND_LON,IND_TSLOT,IND_PRES,INVARS,IACTIM
INTEGER(KIND=JPIM) :: ILEVL
REAL(KIND=JPRB), ALLOCATABLE    :: ZGRID(:,:,:),ZSPEC(:,:),ZSPDIV(:,:) 
REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTERFACE
#include "inv_transad.h"
END INTERFACE

#include "interp_obsad.intfb.h"
#include "interp_obs.intfb.h"
#include "gen_corr_pert.intfb.h"
#include "abor1.intfb.h"
IF (LHOOK) CALL DR_HOOK('OPK_OBSCOR',0,ZHOOK_HANDLE)
call gstats(303,0)
IACTIM=NACTIM_END(NMASK)-NACTIM_START(NMASK)+1
IF (LUV_OBSCOR(NMASK)) THEN
  INVARS=2
ELSE
  INVARS=1
ENDIF
! order of descriptors in OBS_ARR_GL2 must be same as set in suobscor!
IND_LAT=1
IND_LON=2
IND_TSLOT=3
IF (L3D_OBSCOR(NMASK)) THEN
   IF (LUV_OBSCOR(NMASK)) THEN
     IND_PRES=7
   ELSE
     IND_PRES=6
   ENDIF
ELSE
   IND_PRES=-9999
ENDIF
IF (IND_PRES /= 7) THEN
!  WRITE(NULERR,*) 'OPK_OBSCOR:ABOR1, IND_PRES= ',IND_PRES
!  CALL ABOR1('OBSCOR LANCZOS NOT TESTED FOR OTHER THAN WIND')
ENDIF
ALLOCATE(ZGRID(OBSCOR_GRID_DEF%NGPTOT,IACTIM*NOBSCOR_INLEVS(NMASK)*INVARS,1))
IF (L3D_OBSCOR(NMASK)) THEN
  CALL INTERP_OBSAD(NMASK,ZGRID,OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LAT),OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LON), &
    & OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_TSLOT),PX,KNUMOBS/INVARS,NOBSCOR_INLEVS(NMASK),INVARS, &
    & PPRES=OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_PRES))
ELSE
  CALL INTERP_OBSAD(NMASK,ZGRID,OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LAT),OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LON), &
    & OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_TSLOT),PX,KNUMOBS/INVARS,NOBSCOR_INLEVS(NMASK),INVARS)
ENDIF
ILEVL=NOBSCOR_INLEVS(NMASK)*IACTIM
ALLOCATE(ZSPEC(ILEVL,OBSCOR_GRID_DEF%NSPEC2))
ZSPEC(:,:)=0._JPRB
IF (INVARS == 1) THEN
  CALL INV_TRANSAD(PSPSCALAR=ZSPEC,PGP=ZGRID,KRESOL=OBSCOR_GRID_DEF%NRESOL_ID)
  CALL GEN_CORR_PERT(ZGRID,OBSCOR_HOR_LSCALE(NMASK),'SCALAR',PSPEC=ZSPEC,LD_USE_REDUCED_RESOL=.TRUE.)
ELSEIF (INVARS == 2) THEN
  ALLOCATE(ZSPDIV(ILEVL,OBSCOR_GRID_DEF%NSPEC2))
  ZSPDIV(:,:)=0._JPRB
  CALL INV_TRANSAD(PSPVOR=ZSPEC,PSPDIV=ZSPDIV,PGP=ZGRID,KRESOL=OBSCOR_GRID_DEF%NRESOL_ID)
  CALL GEN_CORR_PERT(ZGRID,OBSCOR_HOR_LSCALE(NMASK),'ROTUV',PSPEC=ZSPEC,LD_USE_REDUCED_RESOL=.TRUE.)
  DEALLOCATE(ZSPDIV)
ELSE
  CALL ABOR1('ONLY SCALAR OR WIND IS ALLOWED, i.e. INVARS CAN BE 1 OR 2')
ENDIF
DEALLOCATE(ZSPEC)
IF (L3D_OBSCOR(NMASK)) THEN
  CALL INTERP_OBS(NMASK,ZGRID,OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LAT),OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LON), &
  & OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_TSLOT),PY,KNUMOBS/INVARS,NOBSCOR_INLEVS(NMASK),INVARS, &
  & PPRES=OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_PRES))
ELSE
  CALL INTERP_OBS(NMASK,ZGRID,OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LAT),OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_LON), &
  & OBS_ARR_GL2(1:KNUMOBS/INVARS,IND_TSLOT),PY,KNUMOBS/INVARS,NOBSCOR_INLEVS(NMASK),INVARS)
ENDIF
DEALLOCATE(ZGRID)
call gstats(303,1)
IF (LHOOK) CALL DR_HOOK('OPK_OBSCOR',1,ZHOOK_HANDLE)
END SUBROUTINE OPK_OBSCOR
