SUBROUTINE OZONE_OB       (KTSLOT, KPOOLNO)

!**** SUBROUTINE OZONE_OB - PROCESS LEVEL 1C AND GEOSTATIONARY RADIANCES

!          DRASKO VASILJEVIC ECMWF          01/MAR/2001

!*    METHOD
!     ------

!        THE ODB IS SCANNED FOR OZONE DATA AND
!     MAKECMA FUNCTIONS ARE CARRIED OUT.

!*    INTERFACE
!     ---------

!     ***CALL*OZONE_OB       (KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!*    EXTERNALS
!     ---------

!        GETDB    - GET DB
!        PUTDB    - STORE DB
!        CMOCTMAP - GET OBS./CODE TYPE SEQUENCE NO.s
!        REO3SIN  - PROCESS INPUT O3S
!        ABOR1    - ABNORMAL TERMINATION

!        OZONE_OB IS CALLED BY SUBROUTINE MKCMARPL

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMCOCTP , ONLY : NSATEM   ,NSTRO3
USE YOMLUN   , ONLY : NULOUT   

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)   :: KTSLOT
INTEGER(KIND=JPIM), INTENT(IN)   :: KPOOLNO
INTEGER(KIND=JPIM) :: INFO(1), IRET

INTEGER(KIND=JPIM) :: ILEN, IOBTYP, ICDTYPE, IOBS
INTEGER(KIND=JPIM) :: IOBSQNO,ICDSQNO
INTEGER(KIND=JPIM) ::  JOBS

LOGICAL :: LLMESAGE, LLTAKEN

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cmoctmap.intfb.h"
#include "reo3sin.intfb.h"

#include "openmp_obs.h"

!*       1.       OZONE OBS
!                 ---------

!*          1.1   INITIALIZE DB (LOCALL; ALL OZONE REPs)

IF (LHOOK) CALL DR_HOOK('OZONE_OB',0,ZHOOK_HANDLE)
IRET = 0
CALL GETDB('OZONE',1,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, KPOOLNO, -1, -1, -1)  
IF(IRET == 0) THEN
  WRITE(NULOUT,'(''     NO REPORTS (OZONE) AVAILABLE '')')
ELSE
  WRITE(NULOUT,'(''     REPORTS (OZONE) FOUND: '',I10)') IRET
ENDIF
CALL FLUSH(NULOUT)
ILEN = IRET

!*          1.2   LOOP OVER ALL THE OZONE OBSERVATIONS

OBS_LOOP: DO JOBS = 1, ILEN
  IOBS = JOBS
  IOBTYP  = ROBHDR(IOBS,MDBOTP)
  ICDTYPE = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  LLMESAGE= .FALSE.
  LLTAKEN = .FALSE.
  CALL CMOCTMAP('CODETOSQ',NULOUT,IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO)

!*          1.3   SATEM

  IF(IOBTYP == NSATEM )              THEN

!*          1.3.1 OZONE

    IF(ICDTYPE == NSTRO3 )             THEN
      CALL REO3SIN(IOBS,&
       & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       & LLMESAGE,LLTAKEN)  

!*          1.3.3 UNKNOWN OZONE

    ELSE
      WRITE(NULOUT,'(''0ERROR IN SUBROUTINE OZONE_OB'')')
      WRITE(NULOUT,'('' UNKNOWN OZONE CODE TYPE'',I10)') ICDTYPE
      CALL ABOR1(' OZONE_OB; WRONG OZONE CODE TYPE ')
    ENDIF

!*          1.2.11UNKNOWN SATEM OBS TYPE

  ELSE
    WRITE(NULOUT,'(''0ERROR IN SUBROUTINE OZONE_OB'')')
    WRITE(NULOUT,'('' UNKNOWN SATEM OBS TYPE'',I10)') IOBTYP
    CALL ABOR1('SUBROUTINE OZONE_OB; WRONG SATEM OBS TYPE ')
  ENDIF
ENDDO OBS_LOOP

!           1.3   PUT THE DB AWAY

CALL PUTDB('OZONE',1,IRET,INFO,0,ZINFO,0)

!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('OZONE_OB',1,ZHOOK_HANDLE)
END SUBROUTINE OZONE_OB
