SUBROUTINE PAOBIN(KPAOBP,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE PAOBIN - PROCESS INPUT PAOB

!        D. VASILJEVIC     ECMWF     13/12/00 (original 04/07/94)

!     PURPOSE
!     -------

!        PROCESS INPUT PAOB

!**   INTERFACE
!     ---------

!        CALL PAOBIN(KPAOBP,
!                    KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                    LDMESAGE,LDTAKEN)
!     WHERE:
!            KPAOBP  = REPORT POINTER
!            KOBTYP  = OBS. TYPE
!            KCDTYP  = CODE TYPE
!            KOBSQNO = OBS. TYPE SQ. NO.
!            KCDSQNO = CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT UPDATED) SWITCH  

!        PAOBIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        ERRSTAT  - OBS. ERROR
!        PAOBBE   - UPDATE PAOB BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE
!        OBSCHAR  - OBS. CHARACTERISTICS

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMCMBDY , ONLY : NCMBPFBP ,NCMBVFBP ,NCMBFLBP ,NCMBFLOC
USE YOMANCS  , ONLY : RMDI     ,RDEGREES ,RADIANS  ,EPSMAC   ,RPAPM    ,RUPMLIM  ,RDWMLIM  
USE YOMCST   , ONLY : RTT
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMCOD , ONLY : NPRESVC  ,NHEIGVC
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMMKODB , ONLY : LERRPERS ,NMKCMVSE ,LTOTREM
USE YOETHF   , ONLY : RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES  
USE YOMLUN   , ONLY : NULOUT

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPAOBP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT) :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE, &
 & IOBTIME, IPFLG, IPFLG1, IPFLG2, &
 & IVNM, IREPST, JPPP , ISTID, INSTYP

INTEGER(KIND=JPIM) :: JZZZ, IZ, &
 & IZZSTAT, IZZEVEN1, IZZEVEN2, IZZRDBFL, IZZANFLG, &
 & IZZPFLG, IZZFLG  
INTEGER(KIND=JPIM) :: JPs, IPs, &
 & IPSSTAT, IPSEVEN1, IPSEVEN2, IPSRDBFL, IPSANFLG, &
 & IPSPFLG, IPSFLG  
INTEGER(KIND=JPIM) :: IAUX, JALL, &
 & IAUSTAT, IAUEVEN1, IAUEVEN2, IAURDBFL, IAUANFLG, &
 & IAUPFLG, IAUFLG  , IAUVERCO  

LOGICAL :: LLFAIL, LLREMOVE

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, &
 & ZPRES, ZTIMDIF, &
 & ZWEIGHT  

REAL(KIND=JPRB) :: ZZZ, ZZZPRES, ZZZPRL, &
 & ZZZPERR, ZZZOERR, ZZZFERR, ZZZRERR, &
 & ZZZFGER, ZZZFGCC1, ZZZFGCC2, &
 & ZZZFGDEP, ZZZANDEP  
REAL(KIND=JPRB) :: ZPS, ZPSPRES, ZPSPRL, &
 & ZPSPERR, ZPSOERR, ZPSFERR, ZPSRERR, &
 & ZPSFGER, ZPSFGCC1, ZPSFGCC2, &
 & ZPSFGDEP, ZPSANDEP  
REAL(KIND=JPRB) :: ZAU, ZAUPRES, ZAUPRL, &
 & ZAUPERR, ZAUOERR, ZAUFERR, ZAURERR, &
 & ZAUFGER, ZAUFGCC1, ZAUFGCC2, &
 & ZAUFGDEP, ZAUANDEP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "errstat.intfb.h"
#include "finoerr.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "paobbe.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET
!                       ------

!*          1.1         GET OBS. PARAMETERS FROM "HEADER"

IF (LHOOK) CALL DR_HOOK('PAOBIN',0,ZHOOK_HANDLE)
ZLAT   = ROBHDR(KPAOBP,MDBLAT) * RDEGREES
ZLON   = ROBHDR(KPAOBP,MDBLON) * RDEGREES
IOBDATE= ROBHDR(KPAOBP,MDBDAT)
IOBTIME= ROBHDR(KPAOBP,MDBETM)
CLSTID = TRANSFER(ROBHDR(KPAOBP,MDBSID), CLSTID)
ISTID  = 0
!READ(CLSTID(1:8),'(A8)') ISTID
IREPST = ROBHDR(KPAOBP,MDBRST)

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KPAOBP,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KPAOBP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.2       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.2         BODY PARAMETERS

ALL_DATA_LOOP: DO JALL = MLNKH2B(KPAOBP),MLNKH2B(KPAOBP+1)-1

!*          3.2.1       WE ACCEPT: Ps AND Z (AND CONDITIONALLY Tv)

  IF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 47) .OR. &
   & ROBODY(JALL  ,MDBVNM) == NVNUMB(  3)       ) GO TO 322  
  IF(ROBODY(JALL  ,MDBVNM) /= NVNUMB( 67)) THEN
    WRITE(NULOUT,'('' PAOBIN; UNEXPECTED VARIABLE: '',I10)') &
     & ROBODY(JALL ,MDBVNM)  
    CALL FLUSH(NULOUT)
  ENDIF

!*          3.2.1.0     SET/GET

  IAUX = JALL
  IVNM = ROBODY(IAUX  ,MDBVNM)
  CALL GETSETE(IAUX ,&
   & ZAUPRES,ZAUPRL,ZAU ,&
   & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
   & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
   & ZAUFGDEP,ZAUANDEP,&
   & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
   & IAUFLG,IAUPFLG)  

  ZAUPRES = ROBODY(IAUX,MDBPPP)
  IAUVERCO= ROBODY(IAUX,MDBVCO)

!*          3.2.1.2     VARIBLE

  ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.2.1.3     ERROR STATISTICS

!*          3.2.1.4     STATUS, EVENTS, FLAGS

  IAUSTAT = ICHSTAT_REJECT( IAUSTAT )
  IAUEVEN1= ICHEV1(IAUEVEN1,ND1EVENT(17))

!*          3.2.1.5     DEPARTURES

!*          3.2.1.6     UPDATE BODY ENTRY

  CALL PAOBBE (IAUX    ,&
   & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
   & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
   & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
   & ZAUFGDEP,ZAUANDEP,&
   & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG  )  

!*          3.2.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

  322 CONTINUE

!*          3.2.1.1     VERTICAL COORDINATE

  3211 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 3212  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF

!*          3.2.1.2     VARIABLE

  3212 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3213  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.2.1.3     ERROR STATISTICS

  3213 CONTINUE
  ROBODY(JALL  ,MDBOER) = RMDI
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.2.3       CLOSE ALL DATA LOOP

ENDDO ALL_DATA_LOOP

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (OUTPUT TO ODB)
!                       --------------------------------

ZWEIGHT = 1.0_JPRB

!*          4.1         GET PRESSURE FIRST

ZPRES  = RMDI
IPFLG  = 0
IPFLG1 = 0
IPFLG2 = 0
P0_LOOP: DO JPPP  = MLNKH2B(KPAOBP),MLNKH2B(KPAOBP+1)-1
  IF(ROBODY(JPPP ,MDBVNM) == NVNUMB( 47)) THEN
    IF(ROBODY(JPPP ,MDBVAR) /= RMDI )       THEN
      ZPRES = ROBODY(JPPP ,MDBVAR)
      IPFLG1= IBITS(INT(ROBODY(JPPP ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
      IPFLG2= IBITS(INT(ROBODY(JPPP ,MDBRDFL)),NCMBPFBP+NCMBFLBP,NCMBFLOC)
      IPFLG = MAX(IPFLG1,IPFLG2)
      GO TO 420
    ENDIF
  ENDIF
ENDDO P0_LOOP

!*          4.1.1       NO PRESSURE

WRITE(NULOUT,'(''0PAOBIN ; THERE IS NO PRESSURE '')')
CALL FLUSH(NULOUT)

!*          4.2         SORT OUT HEIGHT

420 CONTINUE
ZZZ   = 0.0_JPRB
IZZFLG= 0
Z_LOOP: DO JZZZ  = MLNKH2B(KPAOBP),MLNKH2B(KPAOBP+1)-1
  IF(ROBODY(JZZZ ,MDBVNM) == NVNUMB(  3)) THEN
    IZ = JZZZ
    IVNM  = NVNUMB(  3)

!*          4.2.0       SET/GET
    CALL GETSETE(IZ ,&
     & ZZZPRES,ZZZPRL,ZZZ ,&
     & ZZZFGER,ZZZFGCC1,ZZZFGCC2,&
     & ZZZPERR,ZZZOERR,ZZZFERR,ZZZRERR,&
     & ZZZFGDEP,ZZZANDEP,&
     & IZZSTAT,IZZEVEN1,IZZEVEN2,IZZRDBFL,IZZANFLG,&
     & IZZFLG,IZZPFLG)  

    ZZZPRL   = RMDI
    ZZZPRES  = ZPRES
    ZZZ      = 0.0_JPRB

    IZZFLG   = 0
    IZZPFLG  = IPFLG

!*          4.2.2       VARIABLE

    IF(ZZZ     == RMDI .OR. ZZZPRES == RMDI ) THEN
      IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
    ENDIF

!*          4.2.3       ERROR STATISTICS

    IF(ZZZPRES /= RMDI ) THEN
      CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
       & IVNM,IZZSTAT,ZZZPRES,ZZZPRL,&
       & ZZZPERR,ZZZOERR,ZZZFERR,ZWEIGHT)  
    ENDIF

!*          4.2.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  3,KCDSQNO,KOBSQNO) == 0)          THEN
      IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
    ENDIF
    CALL PPVAFL(IZZPFLG,IZZFLG,IZZRDBFL)

!*          4.2.5       DEPARTURES

!*          4.2.6       UPDATE BODY ENTRY

    CALL PAOBBE (IZ      ,&
     & IVNM    ,NPRESVC ,ZZZPRES ,ZZZPRL  ,ZZZ     ,&
     & ZZZOERR ,ZZZPERR ,ZZZFERR ,ZZZRERR ,&
     & ZZZFGER ,ZZZFGCC1,ZZZFGCC2,&
     & ZZZFGDEP,ZZZANDEP,&
     & IZZSTAT ,IZZEVEN1,IZZEVEN2,IZZRDBFL,IZZANFLG  )  
  ENDIF
ENDDO Z_LOOP

!*          4.3         SORT OUT PREESSURE

430 CONTINUE
Ps_LOOP: DO JPs = MLNKH2B(KPAOBP),MLNKH2B(KPAOBP+1)-1
  IF(ROBODY(JPs  ,MDBVNM) == NVNUMB( 47)) THEN
    IPS  = JPs
    IVNM = NVNUMB( 47)

!*          4.3.0       SET/GET

    CALL GETSETE(IPS,&
     & ZPSPRES,ZPSPRL,ZPS ,&
     & ZPSFGER,ZPSFGCC1,ZPSFGCC2,&
     & ZPSPERR,ZPSOERR,ZPSFERR,ZPSRERR,&
     & ZPSFGDEP,ZPSANDEP,&
     & IPSSTAT,IPSEVEN1,IPSEVEN2,IPSRDBFL,IPSANFLG,&
     & IPSFLG,IPSPFLG)  

    ZPSPRES  = ZZZ

    IPSFLG   = IBITS(IPSRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    IPSPFLG  = IBITS(IPSRDBFL,NCMBPFBP+NCMBFLBP,NCMBFLOC)

!*          4.3.2       VARIBLE

    IF(ROBODY(IPS  ,MDBVAR) /= RMDI ) THEN
      ZPS   = ROBODY(IPS  ,MDBVAR)
    ELSE
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF

!*          4.3.3       ERROR STATISTICS

!*          4.3.3.1     PERSISTENCE ERROR

    IF(LERRPERS)           THEN
      IF(ZZZPERR /= RMDI )   THEN
        ZPSPERR  = ZZZPERR*RPAPM
      ENDIF
    ENDIF

!*          4.3.3.2     OBSERVATION ERROR

    IF(ZZZOERR /= RMDI )   THEN
      ZPSOERR  = ZZZOERR*RPAPM
    ENDIF

!*          4.3.3.3     PRESSURE FINAL OBSERVATION ERROR

    CALL FINOERR(ZPSPERR,ZPSOERR,ZPSFERR)

!*          4.3.4       STATUS, EVENTS, FLAGSTATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 47,KCDSQNO,KOBSQNO) == 0)          THEN
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF
    CALL PPVAFL(IPSPFLG,IPSFLG,IPSRDBFL)

!*          4.3.5       DEPARTURES

!*          4.3.6       UPDATE BODY ENTRY

    CALL PAOBBE(IPS     ,&
     & IVNM    ,NHEIGVC ,ZPSPRES ,ZPSPRL  ,ZPS     ,&
     & ZPSOERR ,ZPSPERR ,ZPSFERR ,ZPSRERR ,&
     & ZPSFGER ,ZPSFGCC1,ZPSFGCC2,&
     & ZPSFGDEP,ZPSANDEP,&
     & IPSSTAT ,IPSEVEN1,IPSEVEN2,IPSRDBFL,IPSANFLG  )  
  ENDIF
ENDDO Ps_LOOP
!*
!-----------------------------------------------------------------------

!        5.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

5 CONTINUE

!*          5.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          5.2         ODB REPORT UPDATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        6.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

6 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        7.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('PAOBIN',1,ZHOOK_HANDLE)
END SUBROUTINE PAOBIN
