SUBROUTINE PERTOBS

!**** SUBROUTINE PERTOBS - ADD RANDOM PERTURBATIONS TO OBSERVATIONS

!          MIKE FISHER ECMWF 11-Oct-2004

!*    PURPOSE
!     -------

!          ADD RANDOM PERTURBATIONS TO OBSERVATIONS

!**   INTERFACE
!     ---------

!     ***CALL PERTOBS

!        PERTOBS IS CALLED FROM DECIS

!**   EXTERNALS
!     ---------

!*    METHOD
!     ------

!       LOOPS OVER OBS DEALING WITH ONE SPATIALLY-CORELATED TYPE/CHANNEL AT AT
!       TIME. IN EACH CASE, IT CONSTRUCTS A FIELD OF CORERLATED ERROR, AND
!       SAMPLES IT TO DETERMINE A PERTURBATION, WHICH IS ADDED TO THE
!       OBSERVATION.

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,  DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMOBSET , ONLY : MLNSET, NSETOT, MMXBDY
USE YOMTVRAD , ONLY : SATOBGRP_TABLE, NSATOBGRP
USE YOMDIM   , ONLY : NDGLG
USE YOMDIMO  , ONLY : NACTIM, NOBTOT
USE YOMCOCTP , ONLY : NSATOB
USE YOMCT0   , ONLY : NSTART, NSTOP
USE YOMDYN   , ONLY : TSTEP
USE YOMGEM   , ONLY : NGPTOT, NGPTOTG, NLOENG
USE YOMSCC   , ONLY : NAENSEMBLE, NAEMEMBER, LSCRE4D
USE YOMANCS  , ONLY : RMDI
USE YOMVNMB  , ONLY : NVNUMB
USE YOMCST   , ONLY : RPI
USE YOMECTAB , ONLY : NTSLTOB, NTSLTOBOFF
USE YOMDB
USE RANDOM_NUMBERS_MIX

IMPLICIT NONE

TYPE(randomNumberStream), ALLOCATABLE :: YL_RanNumStream(:)

INTEGER(KIND=JPIM) :: JSET, ILEN, IDLEN, IMXBDY, JSLOT, ICODE, &
                    & IRET, JGRP, ISEED, INLEVS, ICOUNT_DSTA, IACTIM, &
                    & ICOUNT_RSTA, JOBS, JBODY, JLAT, ISTSLOT, IU, IV, JLEV, &
                    & IOFFTOT, ITSLOT, IOBTOT, JT
INTEGER(KIND=JPIM) :: INFO(1), IDSTA(4), IRSTA(4)
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZINFO(1), ZALPHA, Z_AUTO_CORR, Z_LENGTH_SCALE, &
                 & Z_TSLOT_LEN, ZNORM, ZPERT, ZP, ZSIGO_UPPER, ZSIGO_LOWER, &
                 & ZP_UPPER, ZP_LOWER
REAL(KIND=JPRB) :: ZLON0E(NDGLG), ZDLONE(NDGLG), ZLATE(NDGLG), ZOBLON(1), ZOBLAT(1)
REAL(KIND=JPRB), ALLOCATABLE :: ZGRIDG(:,:,:,:), ZGRIDL(:,:,:), ZIGRID(:,:)

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "ecset.intfb.h"
#include "gatherbdy.intfb.h"
#include "gen_corr_pert.intfb.h"
#include "ngedsta.intfb.h"
#include "ngersta.intfb.h"
#include "pertobs_uncorr.intfb.h"
#include "setran.intfb.h"
#include "suhifce.intfb.h"

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('PERTOBS',0,ZHOOK_HANDLE)
WRITE(NULOUT,'('' *** START PERTURBING  OBSERVATIONS '')')

!    Uncorrelated observations

ALLOCATE(YL_RanNumStream(NSETOT))

IF (LSCRE4D) THEN
  IACTIM=NACTIM
ELSE
  IACTIM=1
  CALL ECSET(-1,NOBTOT,0)
ENDIF

TIMESLOT_LOOP: DO JSLOT = 1, IACTIM
  IF(LSCRE4D) THEN
    IOBTOT  = NTSLTOB   (JSLOT)
    IOFFTOT = NTSLTOBOFF(JSLOT)
    CALL ECSET(JSLOT,IOBTOT,IOFFTOT)
    WRITE(NULOUT,'(A,I3)')' ADDING PERTURBATIONS FOR TIMESLOT ',JSLOT
    ITSLOT=JSLOT
  ELSE
    ITSLOT=1
  ENDIF

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JSET,ILEN,IDLEN,IMXBDY)
  DO JSET=NSETOT,1,-1
    ILEN   = MLNSET(JSET)
    IF(MOD(ILEN,2) == 0) THEN
      IDLEN  =ILEN+1
    ELSE
      IDLEN  =ILEN
    ENDIF
    IMXBDY = MAX(MMXBDY(JSET),1)

    CALL PERTOBS_UNCORR (IDLEN,IMXBDY,JSET,ITSLOT,YL_RanNumStream(JSET))
  ENDDO
!$OMP END PARALLEL DO

ENDDO TIMESLOT_LOOP

DEALLOCATE(YL_RanNumStream)

!    Correlated observations: SATOBS

Z_LENGTH_SCALE = 200.0E3_JPRB  ! length scale of horizontal correlation
Z_AUTO_CORR    = 5400.0_JPRB   ! decorrelation-time coefficient
ZP_UPPER       = 20000.0_JPRB    ! pressure (Pa) of upper level
ZP_LOWER       = 70000.0_JPRB    ! pressure (Pa) of lower level
ZSIGO_UPPER    = 4.0_JPRB      ! obs error stdev (component) at upper level
ZSIGO_LOWER    = 2.5_JPRB      ! obs error stdev (component) at lower level
INLEVS=2

ALLOCATE(ZIGRID(1,2*INLEVS))
ALLOCATE(ZGRIDG(NGPTOTG,INLEVS,2,2))  ! indices are: (point,level,var,slot)
ALLOCATE(YL_RanNumStream(1))

ZLON0E(:) = 0.0_JPRB
ZDLONE = 2.0_JPRB*RPI/REAL(NLOENG(1:NDGLG),JPRB)

!  This is not correct, but good enough. (To do it properly, message-pass GELAT)
DO JLAT=1,NDGLG
  ZLATE(JLAT)  = 0.5*RPI - (REAL(JLAT,JPRB)-0.5_JPRB)*RPI/REAL(NDGLG,JPRB)
ENDDO

ISEED=1111*(NAENSEMBLE * 100 + NAEMEMBER)
CALL SETRAN (ISEED,YL_RanNumStream(1))

DO JGRP=1,NSATOBGRP

  ISTSLOT=1
  DO JSLOT = 1, NACTIM

!        Generate spatially-correlated perturbations in grid space on INLEVS
!        levels and for two consecutive time slots.

    ALLOCATE(ZGRIDL(NGPTOT,(3-ISTSLOT)*INLEVS*2,1))

    CALL GEN_CORR_PERT (ZGRIDL,Z_LENGTH_SCALE,'ROTUV',YDSTREAM=YL_RanNumStream(1))

    DO JT=ISTSLOT,2
      DO JLEV=1,INLEVS
        IU = JLEV+(JT-ISTSLOT)*INLEVS
        IV = IU + (3-ISTSLOT)*INLEVS
        CALL GATHERBDY (ZGRIDL(:,IU,1),ZGRIDG(:,JLEV,1,JT))
        CALL GATHERBDY (ZGRIDL(:,IV,1),ZGRIDG(:,JLEV,2,JT))
      ENDDO
    ENDDO

    DEALLOCATE(ZGRIDL)

!         Loop over the observation sets

    DO JSET=1,NSETOT
      ICODE= SATOBGRP_TABLE(JGRP)%codetype

!     Fetch the SATOBS (NB: passing JGRP as the "sensor". This will be matched
!     with "areatype" in the SQL.)

      CALL GETDB('PERTOBS_CORR',1,ILEN,INFO,0,ZINFO,0,&
     & JSET, JSLOT, -1, NSATOB, ICODE, JGRP)

!     Interpolate the perturbation from the INLEVS levels, maintaining variance.

      OBS_LOOP: DO JOBS = 1,ILEN
!      Skip passive/rejected/blacklisted reports
        CALL NGERSTA(INT(ROBHDR(JOBS,MDBRST)),ICOUNT_RSTA,IRSTA)
        IF (( IRSTA(2) + IRSTA(3) + IRSTA(4) ) > 0 ) CYCLE OBS_LOOP

        BODY_LOOP: DO JBODY = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
!      Skip passive/rejected/blacklisted reports
          CALL NGEDSTA(INT(ROBODY(JBODY,MDBDSTA)),ICOUNT_DSTA,IDSTA)
          IF( ( IDSTA(2) + IDSTA(3) + IDSTA(4) ) > 0 ) CYCLE BODY_LOOP

!      Skip missing latitude, longitude, pressure or final obs error
          IF(ROBODY(JBODY,MDBLAT)     == RMDI) CYCLE BODY_LOOP
          IF(ROBODY(JBODY,MDBLON)     == RMDI) CYCLE BODY_LOOP
          IF(ROBODY(JBODY,MDBPPP)     == RMDI) CYCLE BODY_LOOP
          IF(ROBODY(JBODY,MDBFOE)     == RMDI) CYCLE BODY_LOOP

!      Interpolate the perturbation to the observation location
          IF (ROBODY(JBODY,MDBVNM) == NVNUMB(1))THEN
            ZP=ROBODY(JBODY,MDBPPP)       ! pressure of observation (Pa)
            ZOBLAT(1)=ROBHDR(JOBS,MDBLAT) ! latitude of observation (radians)
            ZOBLON(1)=ROBHDR(JOBS,MDBLON) ! longitiude of observation (radians)
            CALL SUHIFCE (ZLON0E,ZDLONE,ZLATE,NDGLG,NLOENG(1:NDGLG),ZOBLON,ZOBLAT,1,&
                        & NGPTOTG,INLEVS,ZGRIDG(:,:,1,1),ZIGRID(1,1:INLEVS)) 
            CALL SUHIFCE (ZLON0E,ZDLONE,ZLATE,NDGLG,NLOENG(1:NDGLG),ZOBLON,ZOBLAT,1,&
                        & NGPTOTG,INLEVS,ZGRIDG(:,:,2,1),ZIGRID(1,INLEVS+1:2*INLEVS)) 
            ZALPHA=(ZP-ZP_UPPER)/(ZP_LOWER-ZP_UPPER)
            ZALPHA=MAX(0.0_JPRB,MIN(1.0_JPRB,ZALPHA))
            ZNORM = 1.0_JPRB/SQRT(ZALPHA**2 + (1.0_JPRB-ZALPHA)**2)
            ZPERT=            ZALPHA*ZNORM*ZSIGO_LOWER*ZIGRID(1,1) &
                & +(1.0_JPRB-ZALPHA)*ZNORM*ZSIGO_UPPER*ZIGRID(1,2)
          ELSEIF (ROBODY(JBODY,MDBVNM) == NVNUMB(2)) THEN
            ZPERT=            ZALPHA*ZNORM*ZSIGO_LOWER*ZIGRID(1,INLEVS+1) &
                & +(1.0_JPRB-ZALPHA)*ZNORM*ZSIGO_UPPER*ZIGRID(1,INLEVS+2)
          ENDIF

!      Add the perturbation to the observation, and update the relevant departures

          IF (ROBODY(JBODY,MDBVAR)     /= RMDI) &
            & ROBODY(JBODY,MDBVAR)     = ROBODY(JBODY,MDBVAR)     + ZPERT
          IF (ROBODY(JBODY,MDBOMF)     /= RMDI) &
            & ROBODY(JBODY,MDBOMF)     = ROBODY(JBODY,MDBOMF)     + ZPERT
          IF (ROBODY(JBODY,MDBIFC1(1)) /= RMDI) &
            & ROBODY(JBODY,MDBIFC1(1)) = ROBODY(JBODY,MDBIFC1(1)) + ZPERT
        ENDDO BODY_LOOP
      ENDDO OBS_LOOP

      CALL PUTDB('PERTOBS_CORR',1,IRET,INFO,0,ZINFO,0)
    ENDDO

!       Account for time correlation using a lag-one Markov process.

    Z_TSLOT_LEN = REAL(NSTOP-NSTART,JPRB)*TSTEP/REAL(NACTIM-1,JPRB)
    ZALPHA = EXP(-Z_TSLOT_LEN/Z_AUTO_CORR)
    ZNORM = 1.0_JPRB/SQRT(ZALPHA**2 + (1.0_JPRB-ZALPHA)**2)
    ZGRIDG(:,:,:,1) = ZALPHA*ZNORM*ZGRIDG(:,:,:,1) + (1.0_JPRB-ZALPHA)*ZNORM*ZGRIDG(:,:,:,2)
    ISTSLOT=2
  ENDDO
ENDDO

DEALLOCATE(YL_RanNumStream)
DEALLOCATE(ZGRIDG)
DEALLOCATE(ZIGRID)

WRITE(NULOUT,'('' ***  END  PERTURBING  OBSERVATIONS '')')
IF (LHOOK) CALL DR_HOOK('PERTOBS',1,ZHOOK_HANDLE)
END SUBROUTINE PERTOBS
