SUBROUTINE PGPSIN(KSYNP,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE PGPSIN - PROCESS INPUT GPS SYNOP

!        D. VASILJEVIC     ECMWF     13/12/00 (original 07/01/04)

!     PURPOSE
!     -------

!        PROCESS INPUT GPS SYNOP

!**   INTERFACE
!     ---------

!        CALL PGPSIN(KSYNP,
!                     KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                     LDMESAGE,LDTAKEN)
!     WHERE:
!            KSYNP   = REPORT POINTER
!            KOBTYP  = OBS. TYPE
!            KCDTYP  = CODE TYPE
!            KOBSQNO = OBS. TYPE SQ. NO.
!            KCDSQNO = CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT UPDATED) SWITCH  

!        PGPSIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        SYNOPBE  - UPDATE SYNOP BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMCMHDR , ONLY : NALFLBP  ,NFLGBP   ,NFLGOC
USE YOMCMBDY , ONLY : NCMBVFBP ,NCMBFLBP ,NCMBFLOC
USE YOMANCS  , ONLY : RMDI     ,RDEGREES ,RADIANS ,&
 & RUPMLIM  ,RDWMLIM  ,EPSMAC   ,&
 & RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES    ,RTT
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMCOD , ONLY : NPRESVC
USE YOMMKODB , ONLY : LERRPERS ,NMKCMVSE ,LTOTREM    
USE YOMLUN   , ONLY : NULOUT   

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSYNP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT) :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE,&
 & IOBTIME,&
 & IVNM, IPCODE, &
 & IVECFL,IVERCO,&
 & IALFLG,IPRESCD, &
 & ISTID, INSTYP, IREPST  

INTEGER(KIND=JPIM) :: japdss,iapdss,&
 & iauflg,iaupflg,iauanflg,iaueven2,iaurdbfl,iapdsspflg,iauverco, &
 & iapdssflg,iaueven1,jall,iaux,iaustat, &
 & IAPDSTAT, IAPDEVEN1, IAPDEVEN2, IAPDRDBFL, IAPDANFLG, &
 & IAPDPFLG, IAPDFLG  

LOGICAL :: LLFAIL, LLREMOVE
LOGICAL :: LLAPDSS

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, ZTIMDIF, &
 & ZVERCO, ZALT  ,&
 & zau,zaufger,zaufgcc1,zauprl,zaupres,zaufgcc2, &
 & zaufgdep,zauandep,zaurerr,zauperr,zauoerr,zauferr
REAL(KIND=JPRB) :: ZAPDSS, ZAPDPRES, ZAPDPRL, &
 & ZAPDPERR, ZAPDOERR, ZAPDFERR, ZAPDRERR, &
 & ZAPDFGER, ZAPDFGCC1, ZAPDFGCC2, &
 & ZAPDFGDEP, ZAPDANDEP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "finoerr.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"
#include "synopbe.intfb.h"

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET
!                       ------

IF (LHOOK) CALL DR_HOOK('PGPSIN',0,ZHOOK_HANDLE)
ZLAT     = ROBHDR(KSYNP,MDBLAT) * RDEGREES
ZLON     = ROBHDR(KSYNP,MDBLON) * RDEGREES
ZALT     = ROBHDR(KSYNP,MDBALT)
IALFLG   = IBITS(INT(ROBHDR(KSYNP,MDBRFL)),NALFLBP+NFLGBP,NFLGOC)
IOBDATE  = ROBHDR(KSYNP,MDBDAT)
IOBTIME  = ROBHDR(KSYNP,MDBETM)
CLSTID   = TRANSFER(ROBHDR(KSYNP,MDBSID), CLSTID)
ISTID    = 0
IREPST   = ROBHDR(KSYNP,MDBRST)

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KSYNP,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KSYNP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.2       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.2         BODY PARAMETERS

LLAPDSS= .FALSE.
ALL_DATA_LOOP: DO JALL = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1

!*          3.2.1       WE ACCEPT: adpss only

  IF(     ROBODY(JALL  ,MDBVNM) == NVNUMB( 75)) THEN
    LLAPDSS = .TRUE.
    GO TO 322
  ELSE
    IAUX = JALL
    IVNM = ROBODY(IAUX  ,MDBVNM)
    WRITE(NULOUT,'('' PGPSIN; UNEXPECTED VARIABLE: '',I10)') IVNM

!*          3.2.1.0     SET/GET

    CALL GETSETE(IAUX ,&
     & ZAUPRES,ZAUPRL,ZAU ,&
     & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
     & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
     & ZAUFGDEP,ZAUANDEP,&
     & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
     & IAUFLG,IAUPFLG)  

    ZAUPRES = ROBODY(IAUX,MDBPPP)
    IAUVERCO= ROBODY(IAUX,MDBVCO)

!*          3.2.1.2     VARIABLE

    ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.2.1.3     ERROR STATISTICS

!*          3.2.1.4     STATUS, EVENTS, FLAGS

    IAUSTAT = ICHSTAT_REJECT( IAUSTAT )

!*          3.2.1.5     DEPARTURES

!*          3.2.1.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IAUX    ,&
     & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
     & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
     & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
     & ZAUFGDEP,ZAUANDEP,&
     & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG, &
     & IPRESCD                                       )  
  ENDIF

!*          3.2.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

  322 CONTINUE

!*          3.2.2.1     PRESSURE

  3221 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 3222  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF

!*          3.2.2.2     VARIABLE

  3222 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3223  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.2.2.3     ERROR STATISTICS

  3223 CONTINUE
  ROBODY(JALL  ,MDBOER) = RMDI
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.2.3       CLOSE ALL DATA LOOP

ENDDO ALL_DATA_LOOP

!*          3.2.3       CHECK IF ALL PRESENT

IF(LLAPDSS) GO TO 324  

WRITE(NULOUT,'('' PGPSIN; '')')
WRITE(NULOUT,'(''       LLAPDSS   :'',L1)') LLAPDSS
324 CONTINUE

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (OUTPUT TO ODB)
!                       --------------------------------

!*          4.0         DECIDE ON VERTICAL COORDINATE; 

!*          4.0.7       VERTICAL COORDINATE

ZVERCO = RMDI
IVECFL = 0
IVERCO = NPRESVC

!*          4.3         GET ATM. PATH DELAY In SAT. SIGNAL

ZAPDSS  = RMDI
IAPDSSFLG= 0
IAPDSSPFLG= IVECFL
APDSS_LOOP: DO JAPDSS = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JAPDSS ,MDBVNM) == NVNUMB( 75)) THEN
    IAPDSS = JAPDSS
    IVNM = NVNUMB( 75)

!*          4.3.0       SET/GET

    CALL GETSETE(IAPDSS,&
     & ZAPDPRES,ZAPDPRL,ZAPDSS,&
     & ZAPDFGER,ZAPDFGCC1,ZAPDFGCC2,&
     & ZAPDPERR,ZAPDOERR,ZAPDFERR,ZAPDRERR,&
     & ZAPDFGDEP,ZAPDANDEP,&
     & IAPDSTAT,IAPDEVEN1,IAPDEVEN2,IAPDRDBFL,IAPDANFLG,&
     & IAPDFLG,IAPDPFLG)  

    ZAPDPRES  = ZVERCO

    IAPDPFLG  = IVECFL

    IPRESCD  = IPCODE

!*          4.3.2       VARIABLE

    IF(ROBODY(IAPDSS ,MDBVAR) /= RMDI ) THEN
      ZAPDSS  = ROBODY(IAPDSS ,MDBVAR)
      IAPDFLG= IBITS(IAPDRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IAPDSTAT = ICHSTAT_REJECT( IAPDSTAT )
    ENDIF

!*          4.3.3       ERROR STATISTICS

!*          4.3.3.1    PERSISTENCE ERROR

    IF(LERRPERS)           THEN
      ZAPDPERR  = 0.010_JPRB ! Rough first estimate of persistence error
    ENDIF

!*          4.3.3.2    OBSERVATION ERROR

    ZAPDOERR  = 0.010_JPRB ! Rough first estimate of obs. error =10mm

!*          4.3.3.3    PRESSURE FINAL OBSERVATION ERROR

    CALL FINOERR(ZAPDPERR,ZAPDOERR,ZAPDFERR)

!*          4.3.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(75,KCDSQNO,KOBSQNO) == 0)          THEN
      IAPDSTAT = ICHSTAT_REJECT( IAPDSTAT )
    ENDIF
    IAPDRDBFL  = 0
    CALL PPVAFL(IAPDPFLG,IAPDFLG,IAPDRDBFL)

!*          4.3.5       DEPARTURES

!*          4.3.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IAPDSS    ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZAPDPRL  ,ZAPDSS    ,&
     & ZAPDOERR ,ZAPDPERR ,ZAPDFERR ,ZAPDRERR ,&
     & ZAPDFGER ,ZAPDFGCC1,ZAPDFGCC2,&
     & ZAPDFGDEP,ZAPDANDEP,&
     & IAPDSTAT ,IAPDEVEN1,IAPDEVEN2,IAPDRDBFL,IAPDANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO APDSS_LOOP
!*
!-----------------------------------------------------------------------

!        5.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

5 CONTINUE

!*          5.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          5.2         ODB REPORT UPDATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        6.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

6 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        7.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('PGPSIN',1,ZHOOK_HANDLE)
END SUBROUTINE PGPSIN
