SUBROUTINE PREAMB

!**** SUBROUTINE PREAMB - PRINT THE PREAMBLE TO THE VAR. ANALYSIS

!        D. VASILJEVIC   ECMWF     13/3/90

!     PURPOSE
!     -------
!        PRINT THE PREAMBLE TO THE VAR. JOB

!**   INTERFACE
!     ---------
!        PREAMB IS CALLED FROM SUBROUTINE SUOBS

!     METHOD
!     ------
!     THE ANALYSIS DATE,
!     TIME ARE PRINTED, FOLLOWED BY THE OTHER RUN
!     CONTROL PARAMETERS.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NOBTOT   ,NOBTOV   ,NOBSCA   ,NOBNTV   ,NACTIM
USE YOMCMA   , ONLY : NRESUPD
USE YOMLUN   , ONLY : NULOUT
USE YOMANA   , ONLY : NANDAT   ,NANTIM
USE YOMOBS   , ONLY : LINCREM
USE YOMECTAB , ONLY : NTSLVSTST

IMPLICIT NONE

CHARACTER ::    CLDATE*8,CLTIME*6

INTEGER(KIND=JPIM) ::  JTSL
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!     ------------------------------------------------------------------
!*       1.       PRINT RUN DESCRIPTION
!                 ---------------------

!*          1.1   EXECUTION DATE AND TIME

IF (LHOOK) CALL DR_HOOK('PREAMB',0,ZHOOK_HANDLE)
WRITE(NULOUT,'(''1'',40X,''ECMWF DATA ASSIMILATION'',/,&
 & 41X,''-----------------------'',/,&
 & 41X,''VARIATIONAL JOB (OBS. PART)'',/,&
 & 41X,''---------------------------'',/)')  

!*          1.3.2 INCREMENTAL/NON-INCREMENTAL

IF(LINCREM)     THEN
  WRITE(NULOUT,'('' FOR INCREMENTAL APPROACH VARIATIONAL'', '' ANALYSIS'')')
  WRITE(NULOUT,'('' WITH PLANNED NO. OF UPDATES:'',I10)') NRESUPD
ENDIF

!*          1.4   ANALYSIS TIME AND DATE

WRITE(CLDATE,'(I8)') NANDAT
WRITE(CLTIME,'(I6)') NANTIM
WRITE(NULOUT,'(''0ANALYSIS DATE: '',A)')&
 & CLDATE(7:8)//'/'//CLDATE(5:6)//'/'//CLDATE(1:4)  
WRITE(NULOUT,'('' ANALYSIS TIME: '',A)')&
 & CLTIME(1:2)//':'//CLTIME(3:4)//':'//CLTIME(5:6)  

!*          1.5   TIMESLOT VS TIMESTEP

DO JTSL = 1 , NACTIM
  WRITE(NULOUT,'('' TIMESLOT:'',I3,&
   & '' TO BE USED WITH TIMESTEP:'',I3)') &
   & JTSL,NTSLVSTST(JTSL)  
ENDDO
!     ------------------------------------------------------------------
!*       2.       DIAGNOSTICS
!                 -----------

WRITE(NULOUT,'(1X,'' NO. OF OBS. PRESENT IN MEMORY: '',I10)')NOBTOT
WRITE(NULOUT,'(1X,'' NO. OF TOVS PRESENT IN MEMORY: '',I10)')NOBTOV
WRITE(NULOUT,'(1X,'' NO. OF NON-TOVS PRESENT IN MEMORY : '',I10)')NOBNTV
WRITE(NULOUT,'(1X,'' NO. OF SCA OBS. PRESENT IN MEMORY : '',I10)')NOBSCA

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PREAMB',1,ZHOOK_HANDLE)
END SUBROUTINE PREAMB
