SUBROUTINE REDTS(KOBS_TE,KOBS_SY)

!**** SUBROUTINE REDTS - REDUNDANT MASS OBSERVATION IN TEMP/SYNOP PAIR

!          HEIKKI JARVINEN   ECMWF          15/OCT/1995
!          DRASKO VASILJEVIC ECMWF          25/OCT/1999

!*    PURPOSE
!     -------

!        FOR COLLOCATED TEMP AND SYNOP OBSERVATIONS, A SYNOP MASS
!     OBSERVATION MAY BE REDUNDANT FOR THE TEMP MASS OBSERVATION.

!**   INTERFACE
!     ---------

!        CALL REDTS(KOBS_TE,KOBS_SY)
!     WHERE KOBS_TE = POINTER TO TEMP REPORT
!           KOBS_SY = POINTER TO SYNOP REPORT

!        REDTS IS CALLED BY SUBROUTINE REDSM

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!        GEDSTA - GET DATUM STATUS
!        CHDSTA - CHANGE DATUM STATUS
!        CHDEVE - CHANGE DATUM EVENT

!**   MODIFICATIONS
!     -------------
!        H. Jarvinen 12 Dec 1996 : A modified subroutine call

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR
USE YOMCOCTP , ONLY : NSYNOP   ,NTEMP
USE YOMANCS  , ONLY : RMDI
USE YOMTHLIM , ONLY : RREDTSL
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS_TE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS_SY 
INTEGER(KIND=JPIM) :: IDSTA(4)

INTEGER(KIND=JPIM) :: ICOUNT, ILEV, IOTP, JNLV

REAL(KIND=JPRB) :: ZCMPPP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ngedsta.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE WITH THE TEMP REPORT
!                 -------------------------------
!*
!           1.1   COLLECT HEADER INFORMATION OF REFERENCE REPORT

IF (LHOOK) CALL DR_HOOK('REDTS',0,ZHOOK_HANDLE)
IOTP = ROBHDR(KOBS_TE,MDBOTP)

!*
!           1.2   CHECK THAT IT REALLY IS A TEMP

IF(IOTP /= NTEMP) THEN
  WRITE(UNIT=NULERR,FMT='('' LOGICAL ERROR (1) IN REDTS '')')
  CALL ABOR1('REDTS: LOGICAL ERROR (1)')
ENDIF
ZCMPPP = RMDI
!*
!           1.3   START LOOPING OVER THE TEMP REPORT

DAT_LOOP1: DO JNLV = MLNKH2B(KOBS_TE),MLNKH2B(KOBS_TE+1)-1
  ILEV = JNLV
!*
!           1.4   SEARCH FOR THE LOWEST ACTIVE TEMP GEOPOTENTIAL DATUM

  IF(ROBODY(ILEV,MDBVNM) == NVNUMB( 3)) THEN
    CALL NGEDSTA(INT(ROBODY(ILEV,MDBDSTA)),ICOUNT,IDSTA)
    IF(IDSTA(2) == 1.OR. IDSTA(3) == 1.OR. IDSTA(4) == 1) THEN
      CYCLE DAT_LOOP1
    ELSE
      ZCMPPP = ROBODY(ILEV,MDBPPP)
      EXIT DAT_LOOP1
    ENDIF
  ENDIF
ENDDO DAT_LOOP1
!*
!           1.6   DID YOU FIND ANY ACTIVE ONES?

IF(ZCMPPP == RMDI) GOTO 3
!*
!     ------------------------------------------------------------------

!        2.       NOW LOOK INSIDE THE SYNOP REPORT
!                 ------------------------------
!*
!           2.1   COLLECT HEADER INFORMATION

IOTP = ROBHDR(KOBS_SY,MDBOTP)

!*
!           2.3   CHECK THAT IT REALLY IS A SYNOP

IF(IOTP /= NSYNOP) THEN
  WRITE(UNIT=NULERR,FMT='('' LOGICAL ERROR (2) IN REDTS '')')
  CALL ABOR1('REDTS:  LOGICAL ERROR (2)')
ENDIF
!*
!           2.4   START LOOPING OVER THE SYNOP REPORT

DAT_LOOP2: DO JNLV = MLNKH2B(KOBS_SY),MLNKH2B(KOBS_SY+1)-1
  ILEV = JNLV
!*
!           2.5   SYNOP GEOPOTENTIAL OBSERVATION ACTIVE?

  IF(ROBODY(ILEV,MDBVNM) == NVNUMB( 3)) THEN
    CALL NGEDSTA(INT(ROBODY(ILEV,MDBDSTA)),ICOUNT,IDSTA)
    IF(IDSTA(2) == 1.OR. IDSTA(3) == 1.OR. IDSTA(4) == 1) THEN
      CYCLE DAT_LOOP2
    ENDIF
    IF((ROBODY(ILEV,MDBPPP)-RREDTSL) < ZCMPPP) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT(13))
    ENDIF
!*
!           2.6   SYNOP PRESSURE OBSERVATION. ACTIVE?

  ELSEIF(ROBODY(ILEV,MDBVNM) == NVNUMB(47)) THEN
    CALL NGEDSTA(INT(ROBODY(ILEV,MDBDSTA)),ICOUNT,IDSTA)
    IF(IDSTA(2) == 1.OR. IDSTA(3) == 1.OR. IDSTA(4) == 1) THEN
      CYCLE DAT_LOOP2
    ENDIF
    IF((ROBODY(ILEV,MDBVAR)-RREDTSL) < ZCMPPP) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT(13))
    ENDIF
  ENDIF
ENDDO DAT_LOOP2
!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                 ------
3 CONTINUE
IF (LHOOK) CALL DR_HOOK('REDTS',1,ZHOOK_HANDLE)
END SUBROUTINE REDTS
