SUBROUTINE REINI

!**** SUBROUTINE REINI - RE-INITIALIZE FLAGS AND EVENTS

!          HEIKKI JARVINEN   ECMWF          15/JAN/1998
!          SAMI SAARINEN     ECMWF          04/JAN/2000  : Adapted for ODB

!*    PURPOSE
!     -------

!        RE-INITIALIZE FLAGS AND EVENTS IN CASE OF RERUNNING IFS

!**   INTERFACE
!     ---------

!     ***CALL* *REINI

!        REINI IS CALLED BY SUBROUTINE OBADAT

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NOBTOT
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMSCC   , ONLY : LSRERUN
USE YOMCT0B  , ONLY : LECMWF
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM) :: ILEV, INFO(1), IRET, JOBS

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.0   INFORM

IF (LHOOK) CALL DR_HOOK('REINI',0,ZHOOK_HANDLE)
IF(LSRERUN) THEN
  WRITE(NULOUT,'('' *** RE-INITIALIZE FLAGS AND EVENTS '')')
ELSE
  GOTO 5
ENDIF
!*
!           1.1   FETCH DATA FROM DATABASE

IRET = 0
CALL GETDB('REINI', 0, IRET, INFO, 0,ZINFO,0, &
 & -1, -1, -1, -1, -1, -1)  

IF (IRET /= NOBTOT) THEN
  WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
  WRITE(NULERR,*)'Expecting NOBTOT=',NOBTOT,', but got =',IRET
  CALL ABORTDB('REINI','INCONSISTENCIES FROM GETDB',IRET)
ENDIF

!*
!     ------------------------------------------------------------------

!        2.       LOOP OVER ALL THE REPORTS IN THE OBSERVATIONS
!                 ---------------------------------------------
!*
!           2.0   START LOOP

REPORT_LOOP: DO JOBS = 1, NOBTOT

!           2.1   RE-INITIALIZE REPORT FLAGS AND EVENTS

  ROBHDR(JOBS,MDBRST ) = 0.0_JPRB
  ROBHDR(JOBS,MDBREV1) = 0.0_JPRB
  ROBHDR(JOBS,MDBRBLE) = 0.0_JPRB

ENDDO REPORT_LOOP

!*
!     ------------------------------------------------------------------

!        3.       LOOP OVER ALL VARIABLES IN THE REPORTS
!                 --------------------------------------

!*          3.0   START LOOP

IF (NOBTOT>0) THEN
  DATUM_LOOP: DO ILEV = MLNKH2B(1),MLNKH2B(NOBTOT+1)-1

!*          3.1   RE-INITIALIZE DATUM FLAGS AND EVENTS

    ROBODY(ILEV,MDBFLG ) = 0.0_JPRB
    ROBODY(ILEV,MDBDEV1) = 0.0_JPRB
    ROBODY(ILEV,MDBDBLE) = 0.0_JPRB
    if (LECMWF) then
      ROBODY(ILEV,MDBDSTA) = 0.0_JPRB
    else
      if (ROBODY(ILEV,MDBDSTA) >= 24.) ROBODY(ILEV,MDBDSTA) = 0.0_JPRB
    endif

  ENDDO DATUM_LOOP
ENDIF

!*
!           4.   UPDATE DATABASE

CALL PUTDB('REINI', 0, IRET, INFO, 0,ZINFO,0)

!*
!     ------------------------------------------------------------------

!        5.       RETURN
!                 ------

5 CONTINUE
IF (LHOOK) CALL DR_HOOK('REINI',1,ZHOOK_HANDLE)
END SUBROUTINE REINI
