SUBROUTINE REO3SIN(KTSP ,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE REO3SIN - PROCESS INPUT RETRIEVED AtMOSPERIC PARAMETERS:
!**** OZONE LAYERS, TEMPERATURE, WATER VAPOUR LAYERS, GRG variables, GHG variables
!                        - AND AEROSOL OPTICAL DEPTH

!        E. HOLM/D. VASILJEVIC     ECMWF     2000-01-11

!     PURPOSE
!     -------

!        PROCESS INPUT RETRIEVED OZONE LAYERS

!**   INTERFACE
!     ---------

!        CALL REO3SIN(KTSP ,
!                     KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                     LDMESAGE,LDTAKEN)
!     WHERE:
!            KTSP    = REPORT POINTER
!            KOBTYP  = ODB OBS. TYPE
!            KCDTYP  = ODB CODE TYPE
!            KOBSQNO = ODB OBS. TYPE SQ. NO.
!            KCDSQNO = ODB CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT CREATED) SWITCH  

!        REO3SIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        ERRSTAT  - OBS. ERROR
!        RAD1CBE  - UPDATE TOVS/SATEM BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE
!        OBSCHAR  - OBS. CHARACTERISTICS

!     MODIFICATIONS
!     -------------
!        Antje Dethof: 2003-04-28 Put instrument type into odb
!        Antje Dethof: 2003-08-14 Check min value of obs error
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 
!        A.Benedetti   18-Mar-2005 Included aerosol optical depth 
!        A.Dethof      09-May-2005 Include GRG variables
!        R.Engelen     20-Jun-2006 Include GHG variables

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMANCS  , ONLY : RMDI     ,RDEGREES ,RADIANS  ,EPSMAC   ,RUPMLIM  ,RDWMLIM  
USE YOMCST   , ONLY : RTT      
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMCOD , ONLY : NPRESVC
USE YOMMKODB , ONLY :  NMKCMVSE ,LTOTREM
USE YOETHF   , ONLY : RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES  
USE YOM_YGFL , ONLY : JPGHG, JPGRG
USE YOMLUN   , ONLY : NULOUT

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTSP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT) :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE, &
 & IOBTIME, IVNM, ISTID, INSTYP, IREPST  
INTEGER(KIND=JPIM) ::  ISATID, IPROTYP, IFLDVIEW, IQRETRIE, &
 & INORETLY  

INTEGER(KIND=JPIM) :: IO3, JO3, &
 & IO3STAT, IO3EVEN1, IO3EVEN2, IO3RDBFL, IO3ANFLG, &
 & IO3PFLG, IO3FLG  
INTEGER(KIND=JPIM) :: IT, JT, &
 & ITSTAT, ITEVEN1, ITEVEN2, ITRDBFL, ITANFLG, &
 & ITPFLG, ITFLG  
INTEGER(KIND=JPIM) :: IPWC, JPWC, &
 & IPWCSTAT, IPWCEVEN1, IPWCEVEN2, IPWCRDBFL, IPWCANFLG, &
 & IPWCPFLG, IPWCFLG  
INTEGER(KIND=JPIM) :: IAOD, JAOD, &
 & IAODSTAT, IAODEVEN1, IAODEVEN2, IAODRDBFL, IAODANFLG, &
 & IAODPFLG, IAODFLG 
INTEGER(KIND=JPIM) :: IGHG, JGHG, JGHGVAR, &
 & IGHGSTAT, IGHGEVEN1, IGHGEVEN2, IGHGRDBFL, IGHGANFLG, &
 & IGHGPFLG, IGHGFLG  
INTEGER(KIND=JPIM) :: ICOD, JCOD, &
 & ICODSTAT, ICODEVEN1, ICODEVEN2, ICODRDBFL, ICODANFLG, &
 & ICODPFLG, ICODFLG
INTEGER(KIND=JPIM) :: IRAO, JRAO, &
 & IRAOSTAT, IRAOEVEN1, IRAOEVEN2, IRAORDBFL, IRAOANFLG, &
 & IRAOPFLG, IRAOFLG
INTEGER(KIND=JPIM) :: IRFA, JRFA, &
 & IRFASTAT, IRFAEVEN1, IRFAEVEN2, IRFARDBFL, IRFAANFLG, &
 & IRFAPFLG, IRFAFLG
INTEGER(KIND=JPIM) :: IODA, JODA, &
 & IODASTAT, IODAEVEN1, IODAEVEN2, IODARDBFL, IODAANFLG, &
 & IODAPFLG, IODAFLG
INTEGER(KIND=JPIM) :: IGRG, JGRG, JGRGVAR, &
 & IGRGSTAT, IGRGEVEN1, IGRGEVEN2, IGRGRDBFL, IGRGANFLG, &
 & IGRGPFLG, IGRGFLG
INTEGER(KIND=JPIM) :: IAUX, JALL, &
 & IAUSTAT, IAUEVEN1, IAUEVEN2, IAURDBFL, IAUANFLG, &
 & IAUPFLG, IAUFLG  , IAUVERCO  
INTEGER(KIND=JPIM) :: IRETR,INAK

LOGICAL :: LLFAIL, LLREMOVE

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, &
 & ZTIMDIF, &
 & ZWEIGHT, ZLATFV1, ZLONFV1, ZLATFV2, ZLONFV2, ZLATFV3, ZLONFV3, &
 & ZLATFV4, ZLONFV4, ZSOLELE, ZCLOCOV, ZCLOTPRE, &
 & ZSOZEN, ZSOAZI, ZSTZEN, ZSTAZI
INTEGER(KIND=JPIM) :: ISIIND, ISFIND    
! BEN - define the parameters for error on AOD as a function of scattering angle  (Crouzille et al., 2007)
REAL(KIND=JPRB), PARAMETER :: CAERPA=0.007, CAERPB=0.0012, CAERPC=0.001
REAL(KIND=JPRB) :: ZDEGTORAD, ZRADTODEG, ZPI
LOGICAL :: LLAODERRSC 

REAL(KIND=JPRB) :: ZO3PRES, ZO3PRL, ZO3, &
 & ZO3OERR, ZO3PERR, ZO3FERR, ZO3RERR, &
 & ZO3FGER, ZO3FGCC1, ZO3FGCC2, &
 & ZO3FGDEP, ZO3ANDEP  
REAL(KIND=JPRB) :: ZTPRES, ZTPRL, ZT, &
 & ZTOERR, ZTPERR, ZTFERR, ZTRERR, &
 & ZTFGER, ZTFGCC1, ZTFGCC2, &
 & ZTFGDEP, ZTANDEP  
REAL(KIND=JPRB) :: ZPWCPRES, ZPWCPRL, ZPWC, &
 & ZPWCOERR, ZPWCPERR, ZPWCFERR, ZPWCRERR, &
 & ZPWCFGER, ZPWCFGCC1, ZPWCFGCC2, &
 & ZPWCFGDEP, ZPWCANDEP  
REAL(KIND=JPRB) :: ZGHGPRES, ZGHGPRL, ZGHG, &
 & ZGHGOERR, ZGHGPERR, ZGHGFERR, ZGHGRERR, &
 & ZGHGFGER, ZGHGFGCC1, ZGHGFGCC2, &
 & ZGHGFGDEP, ZGHGANDEP  
REAL(KIND=JPRB) :: ZAODPRES, ZAODPRL, ZAOD, &
 & ZAODOERR, ZAODPERR, ZAODFERR, ZAODRERR, &
 & ZAODFGER, ZAODFGCC1, ZAODFGCC2, &
 & ZAODFGDEP, ZAODANDEP
REAL(KIND=JPRB) :: ZCODPRES, ZCODPRL, ZCOD, &
 & ZCODOERR, ZCODPERR, ZCODFERR, ZCODRERR, &
 & ZCODFGER, ZCODFGCC1, ZCODFGCC2, &
 & ZCODFGDEP, ZCODANDEP
REAL(KIND=JPRB) :: ZRAOPRES, ZRAOPRL, ZRAO, &
 & ZRAOOERR, ZRAOPERR, ZRAOFERR, ZRAORERR, &
 & ZRAOFGER, ZRAOFGCC1, ZRAOFGCC2, &
 & ZRAOFGDEP, ZRAOANDEP
REAL(KIND=JPRB) :: ZRFAPRES, ZRFAPRL, ZRFA, &
 & ZRFAOERR, ZRFAPERR, ZRFAFERR, ZRFARERR, &
 & ZRFAFGER, ZRFAFGCC1, ZRFAFGCC2, &
 & ZRFAFGDEP, ZRFAANDEP
REAL(KIND=JPRB) :: ZODAPRES, ZODAPRL, ZODA, &
 & ZODAOERR, ZODAPERR, ZODAFERR, ZODARERR, &
 & ZODAFGER, ZODAFGCC1, ZODAFGCC2, &
 & ZODAFGDEP, ZODAANDEP
REAL(KIND=JPRB) :: ZGRGPRES, ZGRGPRL, ZGRG, &
 & ZGRGOERR, ZGRGPERR, ZGRGFERR, ZGRGRERR, &
 & ZGRGFGER, ZGRGFGCC1, ZGRGFGCC2, &
 & ZGRGFGDEP, ZGRGANDEP
REAL(KIND=JPRB) :: ZAU, ZAUPRES, ZAUPRL, &
 & ZAUPERR, ZAUOERR, ZAUFERR, ZAURERR, &
 & ZAUFGER, ZAUFGCC1, ZAUFGCC2, &
 & ZAUFGDEP, ZAUANDEP  

REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "finoerr.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"
#include "reo3be.intfb.h"
#include "abor1.intfb.h"
!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET
!                       ------
IF (LHOOK) CALL DR_HOOK('REO3SIN',0,ZHOOK_HANDLE)
ZEPS=1.E-12_JPRB
IRETR = -9

!*          1.1         GET OBS. PARAMETERS FROM "HEADER"

!*          1.1.0       BASIC ONES

ZLAT   = ROBHDR(KTSP ,MDBLAT) * RDEGREES
ZLON   = ROBHDR(KTSP ,MDBLON) * RDEGREES
IOBDATE= ROBHDR(KTSP ,MDBDAT)
IOBTIME= ROBHDR(KTSP ,MDBETM)
CLSTID = TRANSFER(ROBHDR(KTSP ,MDBSID), CLSTID)
ISTID  = 0
!READ(CLSTID(1:8),'(A8)') ISTID
IREPST = ROBHDR(KTSP,MDBRST)

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  
INSTYP  = SATHDR(KTSP ,MDBRO3IT )

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KTSP ,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KTSP ,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.2       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.2         REPORT EXTRA INFO

!*          3.2.1       SATHDR

!*          3.2.1.1     SAT ID

ISATID   = SATHDR(KTSP ,MDB_SATID_AT_SAT)

!*          3.2.1.2     INSTRUMENT TYPE (REDO IT)

INSTYP  = SATHDR(KTSP ,MDBRO3IT )
ROBHDR(KTSP ,MDBSSIA) = INSTYP

!*          3.2.1.3     PRODUCT TYPE

IPROTYP  = SATHDR(KTSP ,MDBRO3PT )

!*          3.2.1.4     FIELD OF VIEW

IFLDVIEW = SATHDR(KTSP ,MDBRO3FOV)

!*          3.2.1.5     QUALITY RETRIEVAL

IQRETRIE = SATHDR(KTSP ,MDBRO3QR )

!*          3.2.1.6     NO. OF RETRIEVED LAYERS

INORETLY = SATHDR(KTSP ,MDBRO3NL )

!*          3.2.1.7     LAT/LON FIELD OF VIEW

ZLATFV1  = SATHDR(KTSP ,MDBRO3LA1)
ZLONFV1  = SATHDR(KTSP ,MDBRO3LO1)
ZLATFV2  = SATHDR(KTSP ,MDBRO3LA2)
ZLONFV2  = SATHDR(KTSP ,MDBRO3LO2)
ZLATFV3  = SATHDR(KTSP ,MDBRO3LA3)
ZLONFV3  = SATHDR(KTSP ,MDBRO3LO3)
ZLATFV4  = SATHDR(KTSP ,MDBRO3LA4)
ZLONFV4  = SATHDR(KTSP ,MDBRO3LO4)

!*          3.2.1.8     SOLAR ELEVATION

ZSOLELE  = SATHDR(KTSP ,MDBRO3SOE)

!*          3.2.1.9     CLOUD COVER

ZCLOCOV  = SATHDR(KTSP ,MDBRO3CLC)

!*          3.2.1.10    CLOUD TOP PRESSURE

ZCLOTPRE = SATHDR(KTSP ,MDBRO3CP )

!BEN - the following are MODIS AEROSOL parameters

!*         3.2.1.11     SOLAR ZENITH ANGLE (deg)

ZSOZEN=    SATHDR(KTSP,MDB_SOLAR_ZENITH_AT_SAT)

!*         3.2.1.12     SOLAR AZIMUTH ANGLE (deg)

ZSOAZI=    SATHDR(KTSP,MDB_SOLAR_AZIMUTH_AT_SAT)

!*         3.2.1.13     SATELLITE ZENITH ANGLE (deg)

ZSTZEN=    SATHDR(KTSP,MDB_ZENITH_AT_SAT)

!*         3.2.1.14     SATELLITE AZIMUTH ANGLE (deg)

ZSTAZI=    SATHDR(KTSP,MDB_AZIMUTH_AT_SAT)

!*         3.2.1.15     SNOW ICE INDICATOR

ISIIND=    SATHDR(KTSP,MDBAERSII)

!*         3.2.1.16     SURFACE TYPE INDICATOR
! 0 = land
! 5 = ocean
! 6 = coast
! 7 = desert 

ISFIND=    SATHDR(KTSP,MDBAERSTF)

!*          3.3         BODY PARAMETERS

!*          3.3.1       WE ACCEPT: 1. RETRIEVED OZONE LAYERS
!                                  2. T
!                                  3. PWC
!                                  4. AEROSOL OPTICAL DEPTH AT 0.55 MICRONS (AOD)
!                                  5. GRG(JPGRG)
!                                  6. CLOUD OPTICAL DEPTH (COD) 
!                                  7. RATIO OF FINE MODE TO TOTAL AEROSOL OPTICAL DEPTH AT 0.55 MICRONS (RAO)
!                                  8. AEROSOL REFLECTANCE (RFA)
!                                  9. AEROSOL OPTICAL DEPTH (MULTI-CHANNEL) (ODA) 

ALL_DATA_LOOP: DO JALL = MLNKH2B(KTSP),MLNKH2B(KTSP+1)-1
  IF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 68).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB(  8).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB(  6).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 78).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 80).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 81).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 82).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 83).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 84).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 85).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 86).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 87).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 88).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 89).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 94).OR.&
   & ROBODY(JALL  ,MDBVNM) == NVNUMB( 95)     ) THEN
     
    GO TO 332  

  ELSE 
!*          3.3.1.0     UNEXPECTED VARIABLE
   IAUX = JALL
   IVNM = ROBODY(IAUX  ,MDBVNM)
   WRITE(NULOUT,'('' REO3SIN; UNEXPECTED VARIABLE: '',I10)') IVNM
   LDMESAGE = .TRUE.
   CALL ABOR1('SUBROUTINE REO3SIN; WRONG REO3 VARIABLE ') 

  ENDIF 
  
!*          3.3.1.1     SET/GET

  CALL GETSETE(IAUX ,&
   & ZAUPRES,ZAUPRL,ZAU ,&
   & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
   & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
   & ZAUFGDEP,ZAUANDEP,&
   & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
   & IAUFLG,IAUPFLG)  

  ZAUPRES = ROBODY(IAUX,MDBPPP)
  IAUVERCO= ROBODY(IAUX,MDBVCO)

!*          3.3.1.3     VARIBLE

  ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.3.1.4     ERROR STATISTICS

!*          3.3.1.5     STATUS, EVENTS, FLAGS

  IAUSTAT = ICHSTAT_REJECT( IAUSTAT )

!*          3.3.1.6     DEPARTURES

!*          3.3.1.7     UPDATE BODY ENTRY

  CALL REO3BE (IAUX    ,&
   & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
   & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
   & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
   & ZAUFGDEP,ZAUANDEP,&
   & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG  )  

!*          3.3.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

  332 CONTINUE

!*          3.3.1.0     Write averaging kernel info into retrype
!                       IRETR=0 if no ak info in data
!                       IRETR=1 if ak info in data


  IF (IRETR == -9) THEN
    IF (NROWS_ROBODY /= NROWS_SATBODY ) THEN
      WRITE(NULOUT,'('' REO3SIN : NROWS_ROBODY /= NROWS_SATBODY '',I4,x,I4)') &
       & NROWS_ROBODY, NROWS_SATBODY
      LDMESAGE = .TRUE.
      CALL ABOR1('SUBROUTINE REO3SIN: NROWS_ROBODY /= NROWS_SATBODY ')
    ENDIF
    INAK = SATBODY(JALL,MDB_NAK_AT_REO3_BODY)
    IF(INAK > 0) THEN
       IRETR = 1
    ELSE
       IRETR = 0
    ENDIF
    ROBHDR(KTSP ,MDB_RETRTYPE_AT_HDR) = IRETR
  ENDIF


!*          3.3.1.1     VERTICAL COORDINATE

  3311 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 33111  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF
  33111 CONTINUE
  IF(ROBODY(JALL  ,MDBPRL) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPRL) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPRL) <= RUPMLIM     )) GO TO 3312  
    ROBODY(JALL ,MDBPRL ) = RMDI
  ENDIF

!*          3.3.1.2     VARIABLE

  3312 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3313  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.3.1.3     OBS. ERROR

  3313 CONTINUE
  IF(ROBODY(JALL  ,MDBOER) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBOER) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBOER) <= RUPMLIM) .AND. &
     & ( ABS(ROBODY(JALL ,MDBOER)) > ZEPS)    ) GO TO 3314  
    ROBODY(JALL ,MDBOER ) = RMDI
  ENDIF

!*          3.3.1.4     OBS. ERROR for averaging kernels

  3314 CONTINUE
  IF(ROBHDR(KTSP ,MDB_RETRTYPE_AT_HDR) == 1 ) THEN
    IF(ROBODY(JALL  ,MDBOERAK) /= RMDI ) THEN
      IF(.NOT.(ROBODY(JALL ,MDBOERAK) >= RDWMLIM.AND.&
       & ROBODY(JALL ,MDBOERAK) <= RUPMLIM) .AND. &
       & ( ABS(ROBODY(JALL ,MDBOERAK)) > ZEPS)    ) GO TO 3315  
      ROBODY(JALL ,MDBOERAK ) = RMDI
    ENDIF
  ELSE
    ROBODY(JALL ,MDBOERAK ) = RMDI
  ENDIF
!*          3.3.1.5     THE REST OF ERROR STATISTICS

  3315 CONTINUE
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.3.3       CLOSE ALL DATA LOOP

ENDDO ALL_DATA_LOOP

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (OUTPUT TO ODB)
!                       --------------------------------

ZWEIGHT = 1.0_JPRB

!*          4.1         LOOK FOR DATA

!*          4.2         RETRIEVED OZONE LAYERS

O3_LOOP: DO JO3   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JO3  ,MDBVNM) == NVNUMB( 68)) THEN
    IO3   = JO3
    IVNM  = NVNUMB( 68)

!*          4.2.0       SET/GET

    CALL GETSETE(IO3,&
     & ZO3PRES,ZO3PRL,ZO3 ,&
     & ZO3FGER,ZO3FGCC1,ZO3FGCC2,&
     & ZO3PERR,ZO3OERR,ZO3FERR,ZO3RERR,&
     & ZO3FGDEP,ZO3ANDEP,&
     & IO3STAT,IO3EVEN1,IO3EVEN2,IO3RDBFL,IO3ANFLG,&
     & IO3FLG,IO3PFLG)  

    ZO3PRES= ROBODY(IO3,MDBPPP)
    ZO3PRL = ROBODY(IO3,MDBPRL)

!*          4.2.2       VARIABLE

!    IF(ROBODY(IO3  ,MDBVAR) /= RMDI .AND. ROBODY(IO3  ,MDBVAR) /=  0._JPRB) THEN
    IF(ROBODY(IO3  ,MDBVAR) /= RMDI) THEN
      ZO3     = ROBODY(IO3  ,MDBVAR  )
    ELSE
      IO3STAT = ICHSTAT_REJECT( IO3STAT )
    ENDIF

!*          4.2.3       ERROR STATISTICS

    ZO3PERR = RMDI
    ZO3OERR = ROBODY(IO3,MDBOER)
! The following change was suggested by Antje, but should not go into operations yet.
!    IF(ZO3 /= RMDI ) THEN
!      IF(ZO3OERR == RMDI) ZO3OERR = 0.10*ZO3 ! missing obs errors set to 10% of obs value
!      ZO3OERR = MAX(ZO3OERR,0.05*ZO3)        ! minimun value of 5% obsvalue
!    ELSE
!      ZO3OERR = RMDI
!    ENDIF
    CALL FINOERR(ZO3PERR,ZO3OERR,ZO3FERR)

!*          4.2.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 68,KCDSQNO,KOBSQNO) == 0)  THEN
      IO3STAT = ICHSTAT_REJECT( IO3STAT )
    ENDIF
    CALL PPVAFL(IO3PFLG,IO3FLG,IO3RDBFL)

!*          4.2.5       DEPARTURES

!*          4.2.6       UPDATE BODY ENTRY

    CALL REO3BE (IO3     ,&
     & IVNM    ,NPRESVC ,ZO3PRES ,ZO3PRL  ,ZO3     ,&
     & ZO3OERR ,ZO3PERR ,ZO3FERR ,ZO3RERR ,&
     & ZO3FGER ,ZO3FGCC1,ZO3FGCC2,&
     & ZO3FGDEP,ZO3ANDEP,&
     & IO3STAT ,IO3EVEN1,IO3EVEN2,IO3RDBFL,IO3ANFLG  )  
  ENDIF
ENDDO O3_LOOP

!*          4.3         TEMPERATURE

T_LOOP: DO JT    = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JT   ,MDBVNM) == NVNUMB(  8)) THEN
    IT    = JT
    IVNM  = NVNUMB(  8)

!*          4.3.0       SET/GET

    CALL GETSETE(IT ,&
     & ZTPRES,ZTPRL,ZT ,&
     & ZTFGER,ZTFGCC1,ZTFGCC2,&
     & ZTPERR,ZTOERR,ZTFERR,ZTRERR,&
     & ZTFGDEP,ZTANDEP,&
     & ITSTAT,ITEVEN1,ITEVEN2,ITRDBFL,ITANFLG,&
     & ITFLG,ITPFLG)  

    ZTPRES= ROBODY(IT,MDBPPP)
    ZTPRL = ROBODY(IT,MDBPRL)

!*          4.3.2       VARIABLE

    IF(ROBODY(IT   ,MDBVAR) /= RMDI ) THEN
      ZT      = ROBODY(IT   ,MDBVAR  )
    ELSE
      ITSTAT  = ICHSTAT_REJECT( ITSTAT )
    ENDIF

!*          4.3.3       ERROR STATISTICS

    ZTPERR = RMDI
    ZTOERR = ROBODY(IT ,MDBOER)
    CALL FINOERR(ZTPERR,ZTOERR,ZTFERR)

!*          4.3.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  8,KCDSQNO,KOBSQNO) == 0)  THEN
      ITSTAT  = ICHSTAT_REJECT( ITSTAT )
    ENDIF
    CALL PPVAFL(ITPFLG,ITFLG,ITRDBFL)

!*          4.3.5       DEPARTURES

!*          4.3.6       UPDATE BODY ENTRY

    CALL REO3BE (IT     ,&
     & IVNM   ,NPRESVC,ZTPRES ,ZTPRL  ,ZT      ,&
     & ZTOERR ,ZTPERR ,ZTFERR ,ZTRERR ,&
     & ZTFGER ,ZTFGCC1,ZTFGCC2,&
     & ZTFGDEP,ZTANDEP,&
     & ITSTAT ,ITEVEN1,ITEVEN2,ITRDBFL,ITANFLG  )  
  ENDIF
ENDDO T_LOOP

!*          4.4         RETRIEVED PWC

PWC_LOOP: DO JPWC   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JPWC  ,MDBVNM) == NVNUMB(  6)) THEN
    IPWC  = JPWC
    IVNM  = NVNUMB(  6)

!*          4.4.0       SET/GET

    CALL GETSETE(IPWC,&
     & ZPWCPRES,ZPWCPRL,ZPWC ,&
     & ZPWCFGER,ZPWCFGCC1,ZPWCFGCC2,&
     & ZPWCPERR,ZPWCOERR,ZPWCFERR,ZPWCRERR,&
     & ZPWCFGDEP,ZPWCANDEP,&
     & IPWCSTAT,IPWCEVEN1,IPWCEVEN2,IPWCRDBFL,IPWCANFLG,&
     & IPWCFLG,IPWCPFLG)  

    ZPWCPRES= ROBODY(IPWC,MDBPPP)
    ZPWCPRL = ROBODY(IPWC,MDBPRL)

!*          4.4.2       VARIABLE

    IF(ROBODY(IPWC  ,MDBVAR) /= RMDI ) THEN
      ZPWC     = ROBODY(IPWC  ,MDBVAR  )
    ELSE
      IPWCSTAT = ICHSTAT_REJECT( IPWCSTAT )
    ENDIF

!*          4.4.3       ERROR STATISTICS

    ZPWCPERR = RMDI
    ZPWCOERR = ROBODY(IPWC,MDBOER)
    CALL FINOERR(ZPWCPERR,ZPWCOERR,ZPWCFERR)

!*          4.4.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  6,KCDSQNO,KOBSQNO) == 0)  THEN
      IPWCSTAT = ICHSTAT_REJECT( IPWCSTAT )
    ENDIF
    CALL PPVAFL(IPWCPFLG,IPWCFLG,IPWCRDBFL)

!*          4.4.5       DEPARTURES

!*          4.4.6       UPDATE BODY ENTRY

    CALL REO3BE (IPWC     ,&
     & IVNM    ,NPRESVC ,ZPWCPRES ,ZPWCPRL  ,ZPWC     ,&
     & ZPWCOERR ,ZPWCPERR ,ZPWCFERR ,ZPWCRERR ,&
     & ZPWCFGER ,ZPWCFGCC1,ZPWCFGCC2,&
     & ZPWCFGDEP,ZPWCANDEP,&
     & IPWCSTAT ,IPWCEVEN1,IPWCEVEN2,IPWCRDBFL,IPWCANFLG  )  
  ENDIF
ENDDO PWC_LOOP
!-----------------------------------------------------------------------
!*          4.5         GHG
GHG_LOOP: DO JGHG=1,JPGHG
  GHG_VAR_LOOP: DO JGHGVAR   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
    IF(ROBODY(JGHGVAR  ,MDBVNM) == NVNUMB(  93+JGHG )) THEN
      IGHG  = JGHGVAR
      IVNM  = NVNUMB(  93+JGHG )

!*          4.4.0       SET/GET

      CALL GETSETE(IGHG,&
       & ZGHGPRES,ZGHGPRL,ZGHG ,&
       & ZGHGFGER,ZGHGFGCC1,ZGHGFGCC2,&
       & ZGHGPERR,ZGHGOERR,ZGHGFERR,ZGHGRERR,&
       & ZGHGFGDEP,ZGHGANDEP,&
       & IGHGSTAT,IGHGEVEN1,IGHGEVEN2,IGHGRDBFL,IGHGANFLG,&
       & IGHGFLG,IGHGPFLG)  

      ZGHGPRES= ROBODY(IGHG,MDBPPP)
      ZGHGPRL = ROBODY(IGHG,MDBPRL)

!*          4.4.2       VARIABLE

      IF(ROBODY(IGHG  ,MDBVAR) /= RMDI ) THEN
        ZGHG     = ROBODY(IGHG  ,MDBVAR  )
      ELSE
        IGHGSTAT = ICHSTAT_REJECT( IGHGSTAT )
      ENDIF

!*          4.4.3       ERROR STATISTICS

      ZGHGPERR = RMDI
      ZGHGOERR = ROBODY(IGHG,MDBOER)

      IF(ZGHG /= RMDI) THEN
        ZGHGOERR = MAX(ZGHGOERR,0.03*ZGHG)        ! minimun value of 3% obsvalue
      ELSE
        ZGHGOERR = RMDI
      ENDIF
      
      CALL FINOERR(ZGHGPERR,ZGHGOERR,ZGHGFERR)

!*          4.4.4       STATUS, EVENTS, FLAGS

      IF(NMKCMVSE(  6,KCDSQNO,KOBSQNO) == 0)  THEN
        IGHGSTAT = ICHSTAT_REJECT( IGHGSTAT )
      ENDIF
      CALL PPVAFL(IGHGPFLG,IGHGFLG,IGHGRDBFL)

!*          4.4.5       DEPARTURES

!*          4.4.6       UPDATE BODY ENTRY

      CALL REO3BE (IGHG     ,&
       & IVNM    ,NPRESVC ,ZGHGPRES ,ZGHGPRL  ,ZGHG     ,&
       & ZGHGOERR ,ZGHGPERR ,ZGHGFERR ,ZGHGRERR ,&
       & ZGHGFGER ,ZGHGFGCC1,ZGHGFGCC2,&
       & ZGHGFGDEP,ZGHGANDEP,&
       & IGHGSTAT ,IGHGEVEN1,IGHGEVEN2,IGHGRDBFL,IGHGANFLG  )  
    ENDIF
  ENDDO GHG_VAR_LOOP
ENDDO GHG_LOOP

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!*          4.5         GRG

GRG_LOOP: DO JGRG=1,JPGRG
  GRG_VAR_LOOP: DO JGRGVAR   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
    IF(ROBODY(JGRGVAR  ,MDBVNM) == NVNUMB( 79+JGRG)) THEN
      IGRG  = JGRGVAR
      IVNM  = NVNUMB( 79+JGRG)

!*          4.5.1       SET/GET

      CALL GETSETE(IGRG,&
       & ZGRGPRES,ZGRGPRL,ZGRG ,&
       & ZGRGFGER,ZGRGFGCC1,ZGRGFGCC2,&
       & ZGRGPERR,ZGRGOERR,ZGRGFERR,ZGRGRERR,&
       & ZGRGFGDEP,ZGRGANDEP,&
       & IGRGSTAT,IGRGEVEN1,IGRGEVEN2,IGRGRDBFL,IGRGANFLG,&
       & IGRGFLG,IGRGPFLG)

      ZGRGPRES= ROBODY(IGRG,MDBPPP)
      ZGRGPRL = ROBODY(IGRG,MDBPRL)

!*          4.5.2       VARIABLE

      IF(ROBODY(IGRG  ,MDBVAR) /= RMDI .AND. ROBODY(IGRG  ,MDBVAR) /= 0._JPRB ) THEN
        ZGRG  = ROBODY(IGRG  ,MDBVAR  )
      ELSE
        IGRGSTAT = ICHSTAT_REJECT( IGRGSTAT )
      ENDIF

!*          4.5.3       ERROR STATISTICS
!                       Different observation errors for observations
!                       with and without averaging kernels

      ZGRGPERR = RMDI
    
      IF(ROBHDR(KTSP ,MDB_RETRTYPE_AT_HDR) == 1 ) THEN
!        ZGRGOERR = ROBODY(IGRG,MDBOERAK)       ! with averaging kernels
        ZGRGOERR = ROBODY(IGRG,MDBOER)          ! without averaging kernels
        IF(ZGRGOERR == RMDI) ZGRGOERR = 0.10*ZGRG ! missing obs errors set to 10% of obs value
      ELSE
        ZGRGOERR = ROBODY(IGRG,MDBOER)            ! without averaging kernels
        IF(ZGRGOERR == RMDI) ZGRGOERR = 0.10*ZGRG ! missing obs errors set to 10% of obs value
      ENDIF

      IF(ZGRG /= RMDI) THEN
        ZGRGOERR = MAX(ZGRGOERR,0.05*ZGRG)        ! minimun value of 5% obsvalue
      ELSE
        ZGRGOERR = RMDI
      ENDIF

      
      CALL FINOERR(ZGRGPERR,ZGRGOERR,ZGRGFERR)

!*          4.5.4       STATUS, EVENTS, FLAGS

      IF(NMKCMVSE( 79+JGRG,KCDSQNO,KOBSQNO) == 0)  THEN
        IGRGSTAT = ICHSTAT_REJECT( IGRGSTAT )
      ENDIF
      CALL PPVAFL(IGRGPFLG,IGRGFLG,IGRGRDBFL)

!*          4.5.5       DEPARTURES

!*          4.5.6       UPDATE BODY ENTRY

      CALL REO3BE (IGRG     ,&
       & IVNM    ,NPRESVC ,ZGRGPRES ,ZGRGPRL  ,ZGRG     ,&
       & ZGRGOERR ,ZGRGPERR ,ZGRGFERR ,ZGRGRERR ,&
       & ZGRGFGER ,ZGRGFGCC1,ZGRGFGCC2,&
       & ZGRGFGDEP,ZGRGANDEP,&
       & IGRGSTAT ,IGRGEVEN1,IGRGEVEN2,&
       & IGRGRDBFL,IGRGANFLG  )
    ENDIF
  ENDDO GRG_VAR_LOOP
ENDDO GRG_LOOP
!-----------------------------------------------------------------------
!*          4.6         RETRIEVED AEROSOL OPTICAL DEPTH AT 0.55 MICRONS (AOD) 

! BEN - set this flag to true for parameterization of error on aerosol optical depth depending on scattering angle 
LLAODERRSC=.TRUE.
! BEN - constants
ZDEGTORAD = (ACOS(-1.0_JPRB)/180._JPRB)
ZRADTODEG =(1.0_JPRB/ZDEGTORAD)
ZPI=ACOS(-1.0_JPRB)
 
AOD_LOOP: DO JAOD   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JAOD  ,MDBVNM) == NVNUMB( 78)) THEN
    IAOD  = JAOD
    IVNM  = NVNUMB( 78)

!*          4.6.0       SET/GET

    CALL GETSETE(IAOD,&
     & ZAODPRES,ZAODPRL,ZAOD ,&
     & ZAODFGER,ZAODFGCC1,ZAODFGCC2,&
     & ZAODPERR,ZAODOERR,ZAODFERR,ZAODRERR,&
     & ZAODFGDEP,ZAODANDEP,&
     & IAODSTAT,IAODEVEN1,IAODEVEN2,IAODRDBFL,IAODANFLG,&
     & IAODFLG,IAODPFLG)  

    ZAODPRES= ROBODY(IAOD,MDBPPP)
    ZAODPRL = ROBODY(IAOD,MDBPRL)

!*          4.6.2       VARIABLE

    IF(ROBODY(IAOD  ,MDBVAR) /= RMDI ) THEN
      ZAOD     = ROBODY(IAOD  ,MDBVAR  )
    ELSE
      IAODSTAT = ICHSTAT_REJECT( IAODSTAT )
    ENDIF

!*          4.6.3       ERROR STATISTICS

    ZAODPERR = RMDI
    ZAODOERR  = 0.1_JPRB

    IF(ISFIND == 5) THEN   ! check surface flag indicator to select AOD over ocean only 
      ZAODOERR  = 0.05_JPRB 
    ENDIF

    CALL FINOERR(ZAODPERR,ZAODOERR,ZAODFERR)

!*          4.6.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 78,KCDSQNO,KOBSQNO) == 0)  THEN
      IAODSTAT = ICHSTAT_REJECT( IAODSTAT )
    ENDIF
    CALL PPVAFL(IAODPFLG,IAODFLG,IAODRDBFL)

!*          4.6.5       DEPARTURES

!*          4.6.6       UPDATE BODY ENTRY

    CALL REO3BE (IAOD     ,&
     & IVNM    ,NPRESVC ,ZAODPRES ,ZAODPRL  ,ZAOD     ,&
     & ZAODOERR ,ZAODPERR ,ZAODFERR ,ZAODRERR ,&
     & ZAODFGER ,ZAODFGCC1,ZAODFGCC2,&
     & ZAODFGDEP,ZAODANDEP,&
     & IAODSTAT ,IAODEVEN1,IAODEVEN2,IAODRDBFL,IAODANFLG  )  
  ENDIF
ENDDO AOD_LOOP
!*
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!*          4.7         RETRIEVED CLOUD OPTICAL DEPTH (COD) 

COD_LOOP: DO JCOD   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JCOD  ,MDBVNM) == NVNUMB( 85)) THEN
    ICOD  = JCOD
    IVNM  = NVNUMB( 85)

!*          4.7.0       SET/GET

    CALL GETSETE(ICOD,&
     & ZCODPRES,ZCODPRL,ZCOD ,&
     & ZCODFGER,ZCODFGCC1,ZCODFGCC2,&
     & ZCODPERR,ZCODOERR,ZCODFERR,ZCODRERR,&
     & ZCODFGDEP,ZCODANDEP,&
     & ICODSTAT,ICODEVEN1,ICODEVEN2,ICODRDBFL,ICODANFLG,&
     & ICODFLG,ICODPFLG)  

    ZCODPRES= ROBODY(ICOD,MDBPPP)
    ZCODPRL = ROBODY(ICOD,MDBPRL)

!*          4.7.2       VARIABLE

    IF(ROBODY(ICOD  ,MDBVAR) /= RMDI ) THEN
      ZCOD     = ROBODY(ICOD  ,MDBVAR  )
    ELSE
      ICODSTAT = ICHSTAT_REJECT( ICODSTAT )
    ENDIF

!*          4.7.3       ERROR STATISTICS

    ZCODPERR = RMDI
! BEN 20% error on cloud optical depth
    ZCODOERR  = MAX(0.2_JPRB*ZCOD,0.01_JPRB) 
    CALL FINOERR(ZCODPERR,ZCODOERR,ZCODFERR)

!*          4.7.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 85,KCDSQNO,KOBSQNO) == 0)  THEN
      ICODSTAT = ICHSTAT_REJECT( ICODSTAT )
    ENDIF
    CALL PPVAFL(ICODPFLG,ICODFLG,ICODRDBFL)

!*          4.7.5       DEPARTURES

!*          4.7.6       UPDATE BODY ENTRY

    CALL REO3BE (ICOD     ,&
     & IVNM    ,NPRESVC ,ZCODPRES ,ZCODPRL  ,ZCOD     ,&
     & ZCODOERR ,ZCODPERR ,ZCODFERR ,ZCODRERR ,&
     & ZCODFGER ,ZCODFGCC1,ZCODFGCC2,&
     & ZCODFGDEP,ZCODANDEP,&
     & ICODSTAT ,ICODEVEN1,ICODEVEN2,ICODRDBFL,ICODANFLG  )  
  ENDIF
ENDDO COD_LOOP
!*
!--------------------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------------------
!*          4.8         RETRIEVED RATIO OF FINE MODE TO TOTAL AEROSOL OPTICAL DEPTH AT 0.55 MICRONS (RAO) 

RAO_LOOP: DO JRAO   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JRAO  ,MDBVNM) == NVNUMB( 86)) THEN
    IRAO  = JRAO
    IVNM  = NVNUMB( 86)

!*          4.8.0       SET/GET

    CALL GETSETE(IRAO,&
     & ZRAOPRES,ZRAOPRL,ZRAO ,&
     & ZRAOFGER,ZRAOFGCC1,ZRAOFGCC2,&
     & ZRAOPERR,ZRAOOERR,ZRAOFERR,ZRAORERR,&
     & ZRAOFGDEP,ZRAOANDEP,&
     & IRAOSTAT,IRAOEVEN1,IRAOEVEN2,IRAORDBFL,IRAOANFLG,&
     & IRAOFLG,IRAOPFLG)  

    ZRAOPRES= ROBODY(IRAO,MDBPPP)
    ZRAOPRL = ROBODY(IRAO,MDBPRL)

!*          4.8.2       VARIABLE

    IF(ROBODY(IRAO  ,MDBVAR) /= RMDI ) THEN
      ZRAO     = ROBODY(IRAO  ,MDBVAR  )
    ELSE
      IRAOSTAT = ICHSTAT_REJECT( IRAOSTAT )
    ENDIF

!*          4.8.3       ERROR STATISTICS

    ZRAOPERR = RMDI
!BEN Set observation error to a very large value to fail these observations (temporary measure)
! ZRAOOERR  = MAX(0.2_JPRB*ZRAO,0.01_JPRB)
    ZRAOOERR  = MAX(5.0_JPRB*ZRAO,10.0_JPRB)
    CALL FINOERR(ZRAOPERR,ZRAOOERR,ZRAOFERR)

!*          4.8.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 86,KCDSQNO,KOBSQNO) == 0)  THEN
      IRAOSTAT = ICHSTAT_REJECT( IRAOSTAT )
    ENDIF
    CALL PPVAFL(IRAOPFLG,IRAOFLG,IRAORDBFL)

!*          4.8.5       DEPARTURES

!*          4.8.6       UPDATE BODY ENTRY

    CALL REO3BE (IRAO     ,&
     & IVNM    ,NPRESVC ,ZRAOPRES ,ZRAOPRL  ,ZRAO     ,&
     & ZRAOOERR ,ZRAOPERR ,ZRAOFERR ,ZRAORERR ,&
     & ZRAOFGER ,ZRAOFGCC1,ZRAOFGCC2,&
     & ZRAOFGDEP,ZRAOANDEP,&
     & IRAOSTAT ,IRAOEVEN1,IRAOEVEN2,IRAORDBFL,IRAOANFLG  )  
  ENDIF
ENDDO RAO_LOOP
!*
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!*          4.9         AEROSOL REFLECTANCE (RFA) 

RFA_LOOP: DO JRFA   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JRFA  ,MDBVNM) == NVNUMB( 88)) THEN
    IRFA  = JRFA
    IVNM  = NVNUMB( 88)

!*          4.9.0       SET/GET

    CALL GETSETE(IRFA,&
     & ZRFAPRES,ZRFAPRL,ZRFA ,&
     & ZRFAFGER,ZRFAFGCC1,ZRFAFGCC2,&
     & ZRFAPERR,ZRFAOERR,ZRFAFERR,ZRFARERR,&
     & ZRFAFGDEP,ZRFAANDEP,&
     & IRFASTAT,IRFAEVEN1,IRFAEVEN2,IRFARDBFL,IRFAANFLG,&
     & IRFAFLG,IRFAPFLG)  

    ZRFAPRES= ROBODY(IRFA,MDBPPP)
    ZRFAPRL = ROBODY(IRFA,MDBPRL)

!*          4.9.2       VARIABLE

    IF(ROBODY(IRFA  ,MDBVAR) /= RMDI ) THEN
      ZRFA     = ROBODY(IRFA  ,MDBVAR  )
    ELSE
      IRFASTAT = ICHSTAT_REJECT( IRFASTAT )
    ENDIF

!*          4.9.3       ERROR STATISTICS

    ZRFAPERR = RMDI
!BEN Set observation error to a very large value to fail these observations (temporary measure)
! ZRFAOERR  = MAX(0.2_JPRB*ZRAO,0.01_JPRB)
    ZRFAOERR  = MAX(5.0_JPRB*ZRFA,10.0_JPRB)
!! BEN Set observation error to a more reasonable value
!    ZRFAOERR  = MAX(0.5_JPRB*ZRFA,10.0_JPRB)

    CALL FINOERR(ZRFAPERR,ZRFAOERR,ZRFAFERR)

!*          4.9.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 88,KCDSQNO,KOBSQNO) == 0)  THEN
      IRFASTAT = ICHSTAT_REJECT( IRFASTAT )
    ENDIF
    CALL PPVAFL(IRFAPFLG,IRFAFLG,IRFARDBFL)

!*          4.9.5       DEPARTURES

!*          4.9.6       UPDATE BODY ENTRY

    CALL REO3BE (IRFA     ,&
     & IVNM    ,NPRESVC ,ZRFAPRES ,ZRFAPRL  ,ZRFA     ,&
     & ZRFAOERR ,ZRFAPERR ,ZRFAFERR ,ZRFARERR ,&
     & ZRFAFGER ,ZRFAFGCC1,ZRFAFGCC2,&
     & ZRFAFGDEP,ZRFAANDEP,&
     & IRFASTAT ,IRFAEVEN1,IRFAEVEN2,IRFARDBFL,IRFAANFLG  )  
  ENDIF
ENDDO RFA_LOOP
!*
!---------------------------------------------------------------------------------
!*          4.10       MULTI-CHANNEL  AEROSOL OPTICAL DEPTH (ODA) 

ODA_LOOP: DO JODA   = MLNKH2B(KTSP ),MLNKH2B(KTSP +1)-1
  IF(ROBODY(JODA  ,MDBVNM) == NVNUMB( 87)) THEN
    IODA  = JODA
    IVNM  = NVNUMB( 87)

!*          4.10.0       SET/GET

    CALL GETSETE(IODA,&
     & ZODAPRES,ZODAPRL,ZODA ,&
     & ZODAFGER,ZODAFGCC1,ZODAFGCC2,&
     & ZODAPERR,ZODAOERR,ZODAFERR,ZODARERR,&
     & ZODAFGDEP,ZODAANDEP,&
     & IODASTAT,IODAEVEN1,IODAEVEN2,IODARDBFL,IODAANFLG,&
     & IODAFLG,IODAPFLG)  

    ZODAPRES= ROBODY(IODA,MDBPPP)
    ZODAPRL = ROBODY(IODA,MDBPRL)

!*          4.10.2       VARIABLE

    IF(ROBODY(IODA  ,MDBVAR) /= RMDI ) THEN
      ZODA     = ROBODY(IODA  ,MDBVAR  )
    ELSE
      IODASTAT = ICHSTAT_REJECT( IODASTAT )
    ENDIF

!*          4.10.3       ERROR STATISTICS

    ZODAPERR = RMDI
!BEN Set observation error to a very large value to fail these observations (temporary measure)
!    ZODAOERR  = MAX(0.2_JPRB*ZODA,0.01_JPRB)
    ZODAOERR  = MAX(5.0_JPRB*ZODA,10.0_JPRB)
    CALL FINOERR(ZODAPERR,ZODAOERR,ZODAFERR)

!*          4.10.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 87,KCDSQNO,KOBSQNO) == 0)  THEN
      IODASTAT = ICHSTAT_REJECT( IODASTAT )
    ENDIF
    CALL PPVAFL(IODAPFLG,IODAFLG,IODARDBFL)

!*          4.10.5       DEPARTURES

!*          4.10.6       UPDATE BODY ENTRY

    CALL REO3BE (IODA     ,&
     & IVNM    ,NPRESVC ,ZODAPRES ,ZODAPRL  ,ZODA     ,&
     & ZODAOERR ,ZODAPERR ,ZODAFERR ,ZODARERR ,&
     & ZODAFGER ,ZODAFGCC1,ZODAFGCC2,&
     & ZODAFGDEP,ZODAANDEP,&
     & IODASTAT ,IODAEVEN1,IODAEVEN2,IODARDBFL,IODAANFLG  )  
  ENDIF
ENDDO ODA_LOOP
!*
!-----------------------------------------------------------------------

!        5.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

5 CONTINUE

!*          5.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          5.2         ODB REPORT CREATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        6.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

6 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        7.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('REO3SIN',1,ZHOOK_HANDLE)
END SUBROUTINE REO3SIN
