SUBROUTINE REPRA(KOBS,KPOS,KPOSM)

!**** SUBROUTINE REPRA - CHECK REPORTING PRACTICE FOR SYNOP/TEMP/PILOT

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        CHECK REPORTING PRACTICE OF GEOPOTENTIAL OBSERVATIONS IN
!     SYNOP/TEMP/PILOT REPORTS, AND IN SURFACE PRESSURE OBSERVATIONS
!     IN SYNOP REPORTS.

!**   INTERFACE
!     ---------

!     ***CALL*REPRA(KOBS,KPOS,KPOSM)
!     WHERE KOBS   = POINTER TO CURRENT REPORT
!           KPOS   = POINTER TO CURRENT OBSERVATION
!           KPOSM  = POSITION OF THE REPORT IN MODEL VECTOR

!        REPRA IS CALLED BY SUBROUTINE PRECH

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!        NGEREVE - GET REPORT EVENTS
!        NGEDEVE - GET DATUM EVENTS

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : RG
USE YOMCOCTP , ONLY : NSYNOP   ,NTEMP    ,NPILOT
USE YOMANCS  , ONLY : RDEGREES
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMCOD , ONLY : NPRESCD
USE YOMSCC   , ONLY : LSCDPR
USE YOMNMEV  , ONLY : NR1EVENT ,ND1EVENT
USE YOMCHEV  , ONLY : CR1EVENT
USE YOMRPLIM , ONLY : RH300    ,RH800    ,RH1700   ,RH2300   ,&
 & RH3700   ,RP700    ,RP850    ,RP1000  
USE GOMS_MIX , ONLY : YGOMS
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPOS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPOSM 
INTEGER(KIND=JPIM) :: IRLIST(32), IDLIST(32)

INTEGER(KIND=JPIM) :: ICMPILID, ICMTELID, ICMVCO, ICMVNM,&
 & IDAT, IDEVENT, ILEVID, IOBTYP, IREF, IREVENT, &
 & ISYPCD  

LOGICAL :: LLPRACT

REAL(KIND=JPRB) :: ZCMPPP, ZCMVAR, ZDIF, ZMPS, ZNOR, ZOBLAT,&
 & ZOBLON, ZPPCRR, ZREF, ZSTALT  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ngedeve.intfb.h"
#include "ngereve.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

IF (LHOOK) CALL DR_HOOK('REPRA',0,ZHOOK_HANDLE)
LLPRACT = .FALSE.
ILEVID  = -9999
!*
!     ------------------------------------------------------------------

!        2.       PREPARE FOR CHECKING REPORTING PRACTICE
!                 ---------------------------------------

!*          2.1   FIND OUT REPORT HEADER INFORMATION

IOBTYP = ROBHDR(KOBS,MDBOTP)

ZSTALT = ROBHDR(KOBS,MDBALT)
ZOBLAT = ROBHDR(KOBS,MDBLAT)*RDEGREES
ZOBLON = ROBHDR(KOBS,MDBLON)*RDEGREES
!*
!           2.2   GET REPORT EVENTS

CALL NGEREVE(INT(ROBHDR(KOBS,MDBREV1)),IREVENT,IRLIST)
!*
!           2.3   FIND OUT DATUM INFORMATION

ICMVNM = ROBODY(KPOS,MDBVNM)
ICMVCO = ROBODY(KPOS,MDBVCO)
ZCMPPP = ROBODY(KPOS,MDBPPP)
ZCMVAR = ROBODY(KPOS,MDBVAR)
IF(ICMVCO == 1) ZCMPPP = ZCMPPP/100._JPRB
IF(ICMVCO == 2) ZCMPPP = ZCMPPP/RG
!*
!           2.4   GET DATUM EVENTS

CALL NGEDEVE(INT(ROBODY(KPOS,MDBDEV1)),IDEVENT,IDLIST)
!*
!     ------------------------------------------------------------------

!        3.       SYNOP REPORTS
!                 -------------

IF(IOBTYP == NSYNOP) THEN
  ISYPCD = ROBODY(KPOS,MDBSYPC)
!*
!           3.1   GEOPOTENTIAL OBSERVATION

  IF(ICMVNM == NVNUMB( 3)) THEN
!*
!           3.2   CHECK FOR MISSING STATION ALTITUDE

    IF(IRLIST(NR1EVENT(13)) == 1) THEN
      IF(LSCDPR) THEN
        WRITE(NULOUT,'('' * REPRA: '',A)') CR1EVENT(13)
      ENDIF
      GOTO 7
    ENDIF
!*
!           3.3   BAD REPORTING PRACTICE IF
!                 - STATION ALTITUDE ABOVE 800M  AND REPORTS PMSL
!                 - STATION ALTITUDE ABOVE 800M  AND REPORTS 1000MB LEVEL
!                 - STATION ALTITUDE ABOVE 1700M AND REPORTS 900MB  LEVEL
!                 - STATION ALTITUDE BELOW 300M  AND REPORTS 900MB  LEVEL
!                 - STATION ALTITUDE ABOVE 2300M AND REPORTS 850MB  LEVEL
!                 - STATION ALTITUDE BELOW 800M  AND REPORTS 850MB  LEVEL
!                 - STATION ALTITUDE ABOVE 3700M AND REPORTS 700MB  LEVEL
!                 - STATION ALTITUDE BELOW 2300M AND REPORTS 700MB  LEVEL
!                 - STATION ALTITUDE BELOW 3700M AND REPORTS 500MB  LEVEL

    IF(&
       & (ISYPCD == NPRESCD( 1) .AND. ZSTALT > RH800 ) .OR.&
       & (ISYPCD == NPRESCD(11) .AND. ZSTALT > RH800 ) .OR.&
       & (ISYPCD == NPRESCD(10) .AND. ZSTALT > RH1700) .OR.&
       & (ISYPCD == NPRESCD(10) .AND. ZSTALT < RH300 ) .OR.&
       & (ISYPCD == NPRESCD( 3) .AND. ZSTALT > RH2300) .OR.&
       & (ISYPCD == NPRESCD( 3) .AND. ZSTALT < RH800 ) .OR.&
       & (ISYPCD == NPRESCD( 4) .AND. ZSTALT > RH3700) .OR.&
       & (ISYPCD == NPRESCD( 4) .AND. ZSTALT < RH2300) .OR.&
       & (ISYPCD == NPRESCD(12) .AND. ZSTALT < RH3700) .OR.&
       & (ISYPCD == NPRESCD( 5) .AND. ABS(ZSTALT- 500._JPRB) > RH800) .OR.&
       & (ISYPCD == NPRESCD( 6) .AND. ABS(ZSTALT-1000._JPRB) > RH800) .OR.&
       & (ISYPCD == NPRESCD( 7) .AND. ABS(ZSTALT-2000._JPRB) > RH800) .OR.&
       & (ISYPCD == NPRESCD( 8) .AND. ABS(ZSTALT-3000._JPRB) > RH800) .OR.&
       & (ISYPCD == NPRESCD( 9) .AND. ABS(ZSTALT-4000._JPRB) > RH800))&
       & THEN  
      LLPRACT = .TRUE.
    ENDIF
!*
!           3.4   SEA LEVEL PRESSURE OBSERVATION

  ELSEIF(ICMVNM == NVNUMB(47) .AND. ISYPCD == NPRESCD( 1)) THEN
    IDAT = ROBHDR(KOBS,MDBDAT)
    IREF = 19820430
    ZMPS = EXP(YGOMS(KPOSM)%SP)/100._JPRB
    ZREF = 1060._JPRB
    ZNOR = 1000._JPRB
    ZDIF = 30._JPRB
!*
!             3.4.1   AFTER APRIL 1982 AND LARGE OBSERVED VALUE

    IF(IDAT > IREF .AND. ZCMPPP > ZREF) THEN
!*
!             3.4.2   COMPARE WITH MODEL SURFACE PRESSURE

      IF(ABS(ZCMPPP-ZMPS-100._JPRB) < ZDIF) THEN
        ROBODY(KPOS,MDBPPP) = ROBODY(KPOS,MDBPPP) - 10000._JPRB
        ROBODY(KPOS,MDBDEV1) = ZCHEV1( ROBODY(KPOS,MDBDEV1),ND1EVENT( 7))
!*
!             3.4.3   INFORM

        ZPPCRR = ROBODY(KPOS,MDBPPP)
        IF(LSCDPR) WRITE(NULOUT,'(&
         & '' - PROBABLY INCORRECT SYNOP SURFACE PRESSURE OBS.'',&
         & '' CORRECTED WITH 100hPa TO '',F7.2)') ZPPCRR  
      ENDIF
!*
!             3.4.4   BEFORE APRIL 1982

    ELSEIF(IDAT <= IREF) THEN
      IF(ABS(ZCMPPP-ZMPS) > ZDIF) THEN
!*
!             3.4.5   PRESSURE MORE THAN 1000hPa

        IF(ZCMPPP >= ZNOR) THEN
          IF(ABS(ZCMPPP-ZMPS-100._JPRB) <= ABS(ZCMPPP-ZMPS) .OR.&
             & ABS(ZCMPPP-ZMPS-100._JPRB) < ZDIF) THEN  
            ROBODY(KPOS,MDBPPP) = ROBODY(KPOS,MDBPPP) - 10000._JPRB
            ROBODY(KPOS,MDBDEV1) = ZCHEV1( ROBODY(KPOS,MDBDEV1),ND1EVENT( 7))
          ENDIF
!*
!             3.4.6   PRESSURE LESS THAN 1000hPa

        ELSE
          IF(ABS(ZCMPPP-ZMPS+100._JPRB) <= ABS(ZCMPPP-ZMPS) .OR.&
             & ABS(ZCMPPP-ZMPS+100._JPRB) < ZDIF) THEN  
            ROBODY(KPOS,MDBPPP) = ROBODY(KPOS,MDBPPP) + 10000._JPRB
            ROBODY(KPOS,MDBDEV1) = ZCHEV1( ROBODY(KPOS,MDBDEV1),ND1EVENT( 7))
          ENDIF
        ENDIF
      ENDIF
    ENDIF
  ENDIF

!*
!     ------------------------------------------------------------------

!        4.       TEMP REPORTS
!                 ------------

ELSEIF(IOBTYP == NTEMP) THEN
!*
!           4.1   GEOPOTENTIAL OBSERVATION

  IF(ICMVNM /= NVNUMB( 3)) GOTO 7
  ICMTELID = ROBODY(KPOS,MDBTELID)
  ILEVID   = IBITS(ICMTELID,6,1)
  IF(ILEVID /= 1) GOTO 7
!*
!           4.2   CHECK FOR MISSING VERTICAL COORDINATE

  IF(IDLIST(ND1EVENT( 1)) == 1) THEN
    IF(LSCDPR) THEN
      WRITE(NULOUT,'('' * REPRA: MISSING VERT.CO. '',I8)') IOBTYP
    ENDIF
    GOTO 7
  ENDIF
!*
!           4.3   BAD REPORTING PRACTICE IF
!                 - STATION ALTITUDE ABOVE 800M  AND REPORTS 1000MB LEVEL
!                 - STATION ALTITUDE ABOVE 2300M AND REPORTS 850MB  LEVEL
!                 - STATION ALTITUDE ABOVE 3700M AND REPORTS 700MB  LEVEL

  IF(ZCMPPP >= RP1000.AND. ZSTALT > RH800 ) LLPRACT=.TRUE.
  IF(ZCMPPP >= RP850.AND. ZSTALT > RH2300) LLPRACT=.TRUE.
  IF(ZCMPPP >= RP700.AND. ZSTALT > RH3700) LLPRACT=.TRUE.
!*
!     ------------------------------------------------------------------

!        5.       PILOT REPORTS
!                 -------------

ELSEIF(IOBTYP /= NPILOT) THEN
!*
!           5.1   GEOPOTENTIAL OBSERVATIONS

  IF(ICMVNM /= NVNUMB( 3)) GOTO 7
  ICMPILID = ROBODY(KPOS,MDBPILID)
  ILEVID   = IBITS(ICMPILID,6,1)
  IF(ILEVID /= 1) GOTO 7
!*
!           5.2   CHECK FOR MISSING VERTICAL COORDINATE

  IF(IDLIST(ND1EVENT( 1)) == 1) THEN
    IF(LSCDPR) THEN
      WRITE(NULOUT,'('' * REPRA: MISSING VERT.CO. '',I8)') IOBTYP
    ENDIF
    GOTO 7
  ENDIF
!*
!           5.3   BAD REPORTING PRACTICE IF
!                 - STATION ALTITUDE ABOVE 800M  AND REPORTS 1000MB LEVEL
!                 - STATION ALTITUDE ABOVE 2300M AND REPORTS 850MB  LEVEL
!                 - STATION ALTITUDE ABOVE 3700M AND REPORTS 700MB  LEVEL

  IF(ZCMPPP >= RP1000.AND. ZSTALT > RH800 ) LLPRACT=.TRUE.
  IF(ZCMPPP >= RP850.AND. ZSTALT > RH2300) LLPRACT=.TRUE.
  IF(ZCMPPP >= RP700.AND. ZSTALT > RH3700) LLPRACT=.TRUE.
ENDIF
!     ------------------------------------------------------------------

!        6.       TAKE ACTION
!                 -----------

IF(LLPRACT) THEN
  ROBODY(KPOS,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(KPOS,MDBDSTA))
  ROBODY(KPOS,MDBDEV1) = ZCHEV1( ROBODY(KPOS,MDBDEV1),ND1EVENT( 7))
ENDIF
!*
!     ------------------------------------------------------------------

!        7.       RETURN
!                 ------

7 CONTINUE
IF (LHOOK) CALL DR_HOOK('REPRA',1,ZHOOK_HANDLE)
END SUBROUTINE REPRA
