SUBROUTINE S0TOWIND(PSIGM,PINC,PBA,&
 & KSOLMAX,KSOL,PVS,PPHI,PRES,PSKILL,PKPM)  
!*******************************************************************************

!     PURPOSE : INVERSION SIGMA0 TRIPLET -> 2-AMBIGUOUS WIND VECTOR
!               (RANK 1 AND FIRST ONE IN THE OPPOSITE DIRECTION)

!     INPUT:
!             PSIGM(JPANT) SIGMA0 VALUES (LINEAR) 
!             PINC(JPANT)  INCIDENCE ANGLE (DEGREE)
!             PBA(JPANT)   AZIMUTH ANGLE (DEGREE)

!     OUTPUT:
!            PVS(KSOLMAX)   WIND SPEED
!            PPHI(KSOLMAX)  WIND DIRECTION
!            PRES(KSOLMAX)  DISTANCES FROM SIGMA0 TRIPLET
!                           TO CONE SURFACE DESCRIBED BY CMOD4
!            KSOL           NUMBER OF POSSIBLE SOLUTIONS
!            PSKILL         SKILL
!            PKPM(JPANT)    MODELISED KP
!     COMMONS
!            DEFTABLE : DESCRIPTION OF THE TABLE: 1ST SPEED,STEP AND NB OF SPEED
!                                               ; SAME FOR WIND DIRECTION
!                                               ; SAME FOR INCIDENCE ANGLE

!     SUBROUTINES
!            MINIMA : FINDS THE MINIMA OF A TABLE
!            SPEEEST: ESTIMATES THE WIND SPEED FROM SIGMA0 FORE AND AFT

!*******************************************************************

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARERSCA , ONLY : JPANT    ,JPVIT    ,JPPHI    ,JPDIRMAX
USE YOMERSCA , ONLY : A180     ,A360     ,EXPP     ,EXPPINV  ,&
 & TAB      ,V0       ,DV       ,DPHI     ,TETA0    ,&
 & RSCAMDI  ,MSCAMDI  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSOLMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIGM(JPANT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINC(JPANT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBA(JPANT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KSOL 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVS(KSOLMAX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPHI(KSOLMAX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRES(KSOLMAX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSKILL 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKPM(JPANT) 
!     LOCAL VARIABLES 

INTEGER(KIND=JPIM) :: IDIR(JPANT),IADT(JPANT),IADPHI(JPANT)
REAL(KIND=JPRB)    :: ZSLIN(JPANT)
INTEGER(KIND=JPIM) :: IV(JPDIRMAX)
REAL(KIND=JPRB)    :: ZD(JPDIRMAX)
INTEGER(KIND=JPIM) :: IMIN(JPDIRMAX/2)
REAL(KIND=JPRB)    :: ZVREM(JPDIRMAX/2),ZDREM(JPDIRMAX/2),ZPHIREM(JPDIRMAX/2)

INTEGER(KIND=JPIM) :: IAD, IDINC, IKVS, IMAX, INBMIN, IND, IPHI1R, ISX, IV1, &
 & IV2, IVGUES, J, JA, JB, JC, JJ, JS  

REAL(KIND=JPRB) :: ZD1, ZD2, ZDA, ZDIF, ZDIFX, ZDISMN, ZNORM, ZNS2, ZPHI, &
 & ZPHIA, ZRES, ZRES1, ZRESLIM, ZSIG, ZSIG1, ZSIG2, ZSIGMOY, ZSPEED, ZV, ZVA, &
 & ZVS, ZX, ZXINCMOY  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "minima.intfb.h"
#include "speeest.intfb.h"

IF (LHOOK) CALL DR_HOOK('S0TOWIND',0,ZHOOK_HANDLE)
DO J=1,JPANT
  IDIR(J)=MSCAMDI
  IADT(J)=MSCAMDI
  IADPHI(J)=MSCAMDI
  ZSLIN(J)=RSCAMDI
ENDDO
Do J=1,JPDIRMAX
  IV(J)=MSCAMDI
  ZD(J)=RSCAMDI
ENDDO
DO J=1,JPDIRMAX/2
  ZVREM(j)=RSCAMDI
  ZDREM(j)=RSCAMDI
  ZPHIREM(j)=RSCAMDI
ENDDO

!**EVALUATE TETA COMPONENT OF TABLE ADDRESS

IDINC=JPVIT*JPPHI
DO J=1,JPANT
  IADT(J)=(INT(PINC(J))-TETA0)*IDINC
  ZSLIN(J)=PINC(J)-INT(PINC(J))
ENDDO

!**EVALUATE WIND SPEED USING ZSIGMA(1) AND ZSIGMA(3)

ZSIGMOY= 0.5_JPRB*(PSIGM(1)**EXPP+PSIGM(3)**EXPP)**(EXPPINV)
ZXINCMOY=0.5_JPRB*(PINC(1)+PINC(3))
CALL SPEEEST(ZSIGMOY,ZXINCMOY,ZSPEED)

!**CONVERT SPEED TO TABLE INDEX

IVGUES=MIN(INT((ZSPEED-V0)/DV)+1,JPVIT)
IKVS = 1
IV(1) = MAX(1,IVGUES)

IPHI1R = 0.0_JPRB

!**COMPUTE PHI-COMPONENT OF TABLE ADDRESS (TO START WITH PHI IS 0/NORTH)

DO J=1,JPANT
  IDIR(J)=IPHI1R + INT(PBA(J))
  IF(IDIR(J) < 0) IDIR(J)=IDIR(J)+INT(A360)
  IDIR(J)=MOD(IDIR(J),INT(A360))
  IF(IDIR(J) > INT(A180)) IDIR(J)=INT(A360)-IDIR(J)
  IADPHI(J)=IADT(J)+NINT(IDIR(J)/DPHI)*JPVIT
ENDDO

!**FIND WIND SPEED

IV1=IV(1)
IV2=IV(1)+IKVS
ZD1 =0.0_JPRB
DO J=1,JPANT
  IAD=IADPHI(J)+IV1
  ZSIG1=TAB(IAD)
  ZSIG2=TAB(IAD+IDINC)
  ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
  ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP)  ! POWER TRANSFORMATION
  ZD1 =ZD1 +ZSIG*ZSIG
ENDDO
ZD2 =0.0_JPRB
DO J=1,JPANT
  IAD=IADPHI(J)+IV2
  ZSIG1=TAB(IAD)
  ZSIG2=TAB(IAD+IDINC)
  ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
  ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP)  ! POWER TRANSFORMATION
  ZD2 =ZD2 +ZSIG*ZSIG
ENDDO
IF(ZD1 < ZD2)THEN
  1099 CONTINUE
  IF(IV1 < 2.OR.IV1 >= JPVIT)GOTO 1000
  IV2=IV1
  ZD2=ZD1
  IV1=IV1-IKVS
  ZD1 =0.0_JPRB
  DO J=1,JPANT
    IAD=IADPHI(J)+IV1
    ZSIG1=TAB(IAD)
    ZSIG2=TAB(IAD+IDINC)
    ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
    ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP) !POWER TRANSFORMATION
    ZD1 =ZD1 +ZSIG*ZSIG
  ENDDO
  IF(ZD1 < ZD2) THEN
    GO TO 1099
  ELSE
    IV(1)=IV2
    ZD(1)=ZD2
    GOTO 3099
  ENDIF
ELSE
  2099 CONTINUE
  IF(IV2 < 2.OR.IV2 >= JPVIT)GOTO 1000
  IV1=IV2
  ZD1=ZD2
  IV2=IV2+IKVS
  ZD2 =0.0_JPRB
  DO J=1,JPANT
    IAD=IADPHI(J)+IV2
    ZSIG1=TAB(IAD)
    ZSIG2=TAB(IAD+IDINC)
    ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
    ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP) !POWER TRANSFORMATION
    ZD2 =ZD2 +ZSIG*ZSIG
  ENDDO
  IF(ZD1 >= ZD2) THEN
    GO TO 2099
  ELSE
    IV(1)=IV1
    ZD(1)=ZD1
    GOTO 3099
  ENDIF
ENDIF
1000 CONTINUE

! TEST FOR TOO LIGHT WINDS

IF(IV1 < 2.OR.IV2 < 2)THEN
  ZD(1)=0.0_JPRB
  IV(1)=0
ELSE

! TEST FOR TOO STRONG WINDS

  ZD(1)=RSCAMDI
ENDIF
3099 CONTINUE
!*******************************************************************
IF( IV(1) == 0 )THEN
  KSOL = 0
  IF (LHOOK) CALL DR_HOOK('S0TOWIND',1,ZHOOK_HANDLE)
  RETURN
ENDIF
ZDISMN = ZD(1)
ZRES1 = ZD(1)

!   SEARCH ON PHI AT PPHI (5) DEGREE STEPS

!      PRINT*,'NINT(DPHI)',NINT(DPHI)
IMAX=INT(A360)/NINT(DPHI)
DO JJ=2,IMAX
  IPHI1R=IPHI1R+NINT(DPHI)

! CMOD5: PREVENT OVERSHOOT GMF; IF FG > 20, START AT 20 M/s.
  IV(JJ)=MIN(INT((20.-V0)/DV)+1,IV(JJ-1))

!     COMPUTE PHI COMPONENT OF TABLE ADRESS

  DO J=1,JPANT
    IDIR(J)=IPHI1R +INT(PBA(J))
    IF(IDIR(J) < 0) IDIR(J)=IDIR(J)+INT(A360)
    IDIR(J)=MOD(IDIR(J),INT(A360))
    IF(IDIR(J) > INT(A180))IDIR(J)=INT(A360)-IDIR(J)
    IADPHI(J)=IADT(J)+NINT(IDIR(J)/DPHI)*JPVIT
  ENDDO
  IV1=IV(JJ)
  IV2=IV(JJ)+IKVS
  ZD1 =0.0_JPRB
  DO J=1,3
    IAD=IADPHI(J)+IV1
    ZSIG1=TAB(IAD)
    ZSIG2=TAB(IAD+IDINC)
    ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
    ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP)  ! POWER TRANSFORMATION
    ZD1 =ZD1 +ZSIG*ZSIG
  ENDDO
  ZD2 =0.0_JPRB
  DO J=1,JPANT
    IAD=IADPHI(J)+IV2
    ZSIG1=TAB(IAD)
    ZSIG2=TAB(IAD+IDINC)
    ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
    ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP)  ! POWER TRANSFORMATION
    ZD2 =ZD2 +ZSIG*ZSIG
  ENDDO
  IF(ZD1 < ZD2)THEN
    1089 CONTINUE
    IF(IV1 < 2.OR.IV1 >= JPVIT)GOTO 2000
    IV2=IV1
    ZD2=ZD1
    IV1=IV1-IKVS
    ZD1 =0.0_JPRB
    DO J=1,JPANT
      IAD=IADPHI(J)+IV1
      ZSIG1=TAB(IAD)
      ZSIG2=TAB(IAD+IDINC)
      ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
      ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP) !POWER TRANSFORMATION
      ZD1 =ZD1 +ZSIG*ZSIG
    ENDDO
    IF(ZD1 < ZD2) THEN
      GO TO 1089
    ELSE
      IV(JJ)=IV2
      ZD(JJ)=ZD2
      GOTO 3089
    ENDIF
  ELSE
    2089 CONTINUE
    IF(IV2 < 2.OR.IV2 >= JPVIT) GOTO 2000
    IV1=IV2
    ZD1=ZD2
    IV2=IV2+IKVS
    ZD2 =0.0_JPRB
    DO J=1,JPANT
      IAD=IADPHI(J)+IV2
      ZSIG1=TAB(IAD)
      ZSIG2=TAB(IAD+IDINC)
      ZSIG=ZSIG1+(ZSIG2-ZSIG1)*ZSLIN(J)
      ZSIG=(ZSIG**EXPP - PSIGM(J)**EXPP) !POWER TRANSFORMATION
      ZD2 =ZD2 +ZSIG*ZSIG
    ENDDO
    IF(ZD1 >= ZD2) THEN
      GO TO 2089
    ELSE
      IV(JJ)=IV1
      ZD(JJ)=ZD1
      GOTO 3089
    ENDIF
  ENDIF
  2000 CONTINUE

! TEST FOR TOO LIGHT WINDS

  IF(IV1 < 2.OR.IV2 < 2)THEN
    ZD(JJ)=0.0_JPRB
    IV(JJ)=0
  ELSE

! TEST FOR TOO STRONG WINDS

    ZD(JJ)=RSCAMDI
  ENDIF
  3089 CONTINUE
!*******************************************************************
  IF(IV(JJ) == 0)THEN
    KSOL=0
    ZDISMN = 0.0_JPRB
    IF (LHOOK) CALL DR_HOOK('S0TOWIND',1,ZHOOK_HANDLE)
    RETURN
  ENDIF
  ZDISMN = ZDISMN + ZD(JJ)
  ZRES1 = MIN( ZRES1, ZD(JJ) )
  IKVS=1
  IF(IV(JJ) < IV(JJ-1))IKVS=-1
ENDDO
ZDISMN = ZDISMN/IMAX

!  SEARCH FOR MINIMA

INBMIN=IMAX
CALL MINIMA (ZD,IMAX,IMAX/2,INBMIN,IMIN,ZRES1)
IF(INBMIN == 0)THEN
  ZDISMN = 0.0_JPRB
  KSOL = 0
  IF (LHOOK) CALL DR_HOOK('S0TOWIND',1,ZHOOK_HANDLE)
  RETURN
ENDIF

DO JJ=1,INBMIN
  IND=IMIN(JJ)
  ZVREM(JJ)=(IV(IND)+V0)*DV
  ZDREM(JJ)=ZD(IND)
  ZPHIREM(JJ)=(REAL(IND,JPRB)-1.0_JPRB)*DPHI
ENDDO

!     REORDER WITH INCREASING DISTANCE

DO JA=1,INBMIN-1
  ZPHIA=ZPHIREM(JA)
  ZVA=ZVREM(JA)
  ZDA=ZDREM(JA)
  JC=JA
  DO JB=JA+1,INBMIN
    IF(ZDREM(JB) < ZDREM(JA))THEN
      ZDREM(JA)=ZDREM(JB)
      ZVREM(JA)=ZVREM(JB)
      ZPHIREM(JA)=ZPHIREM(JB)
      JC=JB
    ENDIF
  ENDDO
  ZPHIREM(JC)=ZPHIA
  ZVREM(JC)=ZVA
  ZDREM(JC)=ZDA
ENDDO

! RESET LEVEL OF RESIDUAL ACCORDING TO NOISE ESTIMATED AT |S0|, |V|, AND THETA
!                             CONSIDER POWER TRANSFORMATION S0
ZV = ZVREM(1)                               ! AS IV(I) SHOULD BE > 0
ZX = ( 45._JPRB - PINC(2) )/27._JPRB
ZNS2 = ( PSIGM(1)**(2.0_JPRB*EXPP) + PSIGM(2)**(2.0_JPRB*EXPP)&
 & +   PSIGM(3)**(2.0_JPRB*EXPP) )&
 & * ( EXPP*2.0_JPRB/100._JPRB*(1.0_JPRB + 5._JPRB/ZV)*(1.0_JPRB + ZX) )**2        

! ALLOW MORE NOISE FOR NON-LINEARITIES AT LOW V, AND EXTRAPOLATION AT HIGH V
ZNS2 = ZNS2 * (1.0_JPRB + 1.0_JPRB/ZV**2)
IF( ZV > 15._JPRB )ZNS2 = ZNS2 * ( 1.0_JPRB + (ZV - 15._JPRB)**2/100._JPRB )

ZDISMN = MAX( ZDISMN - ZDREM(1), 0.0_JPRB )
ZNORM = MAX( ZDREM(1), ZNS2 )
PSKILL = SQRT(ZDISMN/ZNORM)
DO J=1,JPANT
  PKPM(J)=SQRT(ZNS2)
ENDDO

KSOL = MIN(INBMIN,KSOLMAX)
DO JC=1,KSOL
  PRES(JC)=ZDREM(JC)/ZNS2
  PPHI(JC)=ZPHIREM(JC)
  PVS(JC)=ZVREM(JC)
ENDDO
DO J=KSOL+1,KSOLMAX
  PRES(J)=RSCAMDI
  PPHI(J)=RSCAMDI
  PVS(J)=RSCAMDI
ENDDO

! SELECT NUMBER OF SOLUTIONS REQUIRED (2)

ISX = 2       ! JUST INITIALISE FROM RANKS 1 AND 2
ZRESLIM = MAX(3._JPRB , 3._JPRB*PRES(1) )    ! SQRT(3) IN DISTANCE
DO JS = 3,4
  IF( PRES(JS-1)  /=  RSCAMDI .AND.PRES(JS)  /=  RSCAMDI.AND.&
     & PRES(JS) - PRES(JS-1)  <  ZRESLIM )THEN  
    ISX = ISX + 1
  ELSE
    GOTO 125
  ENDIF
ENDDO
125 CONTINUE

ZDIFX = MOD( PPHI(1) - PPHI(2) + 3600._JPRB, 360._JPRB )
ZDIFX = MIN( ZDIFX, 360._JPRB - ZDIFX )
DO JS = 3,ISX
  ZDIF = MOD( PPHI(1) - PPHI(JS) + 3600._JPRB, 360._JPRB )
  ZDIF = MIN( ZDIF, 360._JPRB - ZDIF )
  IF( ZDIF > ZDIFX )THEN
    ZPHI=PPHI(2)
    ZVS=PVS(2)
    ZRES=PRES(2)
    PPHI(2) = PPHI(JS)
    PVS(2)  = PVS(JS)
    PRES(2) = PRES(JS)
    PPHI(JS) = ZPHI
    PVS(JS) = ZVS
    PRES(JS)= ZRES
    ZDIFX   = ZDIF
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('S0TOWIND',1,ZHOOK_HANDLE)
END SUBROUTINE S0TOWIND
