SUBROUTINE SATOB_OB       (KTSLOT, KPOOLNO)

!**** SUBROUTINE SATOB_OB - PROCESS SATOB OBS

!          DRASKO VASILJEVIC ECMWF          01/MAR/2001

!*    METHOD
!     ------

!        THE ODB IS SCANNED FOR SATOB OBSERVATIONS AND MAKECMA FUNCTIONS
!     ARE CARRIED OUT.

!*    INTERFACE
!     ---------

!     ***CALL*SATOB_OB       (KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!*    EXTERNALS
!     ---------

!        GETDB    - GET DB
!        PUTDB    - STORE DB
!        CMOCTMAP - GET OBS./CODE TYPE SEQUENCE NO.s
!        SYNOPIN  - PROCESS INPUT SYNOPs
!        AIREPIN  - PROCESS INPUT AIREPs
!        SATOBIN  - PROCESS INPUT SATOBs
!        SATAMIN  - PROCESS INPUT SATOBs
!        DRIBUIN  - PROCESS INPUT DRIBUs
!        TEMPIN   - PROCESS INPUT TEMPs
!        PILOTIN  - PROCESS INPUT PILOTs
!        AWPRFIN  - PROCESS INPUT AMERICAN WIND PROFILERs
!        SSMIIN   - PROCESS INPUT SSMIs
!        TRMMIN   - PROCESS INPUT TRMMs
!        MERTSIN  - PROCESS INPUT SATEMs/TOVSs
!        RAD1CIN  - PROCESS INPUT LEVEL 1Cs
!        GEOSRIN  - PROCESS INPUT GEOS. radiaces
!        HIRTOIN  - PROCESS INPUT TOVS
!        PAOBIN   - PROCESS INPUT PAOBs
!        ERSIN    - PROCESS INPUT ERSs
!        NSCATIN  - PROCESS INPUT NSCATs
!        QSCATIN  - PROCESS INPUT QSCATs
!        ABOR1    - ABNORMAL TERMINATION

!        SATOB_OB        IS CALLED BY SUBROUTINE MKCMARPL

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMCOCTP , ONLY :       NSATOB   ,&
 & NSTBCD   ,NSTB86   ,NSTB87     
USE YOMMKODB , ONLY : LSATOB
USE YOMLUN   , ONLY : NULOUT   

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)   :: KTSLOT
INTEGER(KIND=JPIM), INTENT(IN)   :: KPOOLNO
INTEGER(KIND=JPIM) :: INFO(1), IRET

INTEGER(KIND=JPIM) :: ILEN, IOBTYP, ICDTYPE, IOBS
INTEGER(KIND=JPIM) :: IOBSQNO,ICDSQNO
INTEGER(KIND=JPIM) ::  JOBS

LOGICAL :: LLMESAGE, LLTAKEN

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cmoctmap.intfb.h"
#include "satamin.intfb.h"
#include "satobin.intfb.h"

#include "openmp_obs.h"

!*       1.       CONVENTIONAL OBS
!                 ----------------

!*          1.1   INITIALIZE DB (LOCALL; ALL CONV REPs)

IF (LHOOK) CALL DR_HOOK('SATOB_OB',0,ZHOOK_HANDLE)
IRET = 0
CALL GETDB('SATOB',1,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, KPOOLNO, -1, -1, -1)  
IF(IRET == 0) THEN
  WRITE(NULOUT,'(''     NO REPORTS (SATOB) AVAILABLE '')')
ELSE
  WRITE(NULOUT,'(''     REPORTS (SATOB) FOUND: '',I10)') IRET
ENDIF
CALL FLUSH(NULOUT)
ILEN = IRET

!*          1.2   LOOP OVER ALL THE SATOB OBSERVATIONS

OBS_LOOP: DO JOBS = 1, ILEN
  IOBS = JOBS
  IOBTYP  = ROBHDR(IOBS,MDBOTP)
  ICDTYPE = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  LLMESAGE= .FALSE.
  LLTAKEN = .FALSE.
  CALL CMOCTMAP('CODETOSQ',NULOUT,IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO)

!*          1.3   SATOB

  IF(IOBTYP == NSATOB )              THEN
    IF(LSATOB)                         THEN

!*          1.3.1 STANDARD SATOB

      IF(     ICDTYPE == NSTBCD .OR.&
         & ICDTYPE == NSTB86      )        THEN  
        CALL SATOBIN(IOBS,&
         & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
         & LLMESAGE,LLTAKEN)  

!*          1.3.2

      ELSEIF(ICDTYPE == NSTB87      )        THEN
        CALL SATAMIN(IOBS,&
         & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
         & LLMESAGE,LLTAKEN)  

!*          1.3.3 UNKNOWN SATOB

      ELSE
        WRITE(NULOUT,'(''0MKCMARPL; UNKNOWN SATOB CODE TYPE'',I10)') ICDTYPE
        CALL ABOR1(' MKCMARPL; WRONG SATOB CODE TYPE ')
      ENDIF
    ENDIF

!*          1.2.11UNKNOWN SATOB OBS TYPE

  ELSE
    WRITE(NULOUT,'(''0ERROR IN SUBROUTINE SATOB_OB'')')
    WRITE(NULOUT,'('' UNKNOWN SATOB OBS TYPE'',I10)') IOBTYP
    CALL ABOR1('SUBROUTINE SATOB_OB')
  ENDIF
ENDDO OBS_LOOP

!           1.3   PUT THE DB AWAY

CALL PUTDB('SATOB',1,IRET,INFO,0,ZINFO,0)

!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SATOB_OB',1,ZHOOK_HANDLE)
END SUBROUTINE SATOB_OB
