SUBROUTINE SCAT_OB        (KTSLOT, KPOOLNO)

!**** SUBROUTINE SCAT_OB - PROCESS SCATT. OBS

!          DRASKO VASILJEVIC ECMWF          01/MAR/2001

!*    METHOD
!     ------

!        THE ODB IS SCANNED FOR SCATT OBS AND
!     MAKECMA FUNCTIONS ARE CARRIED OUT.

!*    INTERFACE
!     ---------

!     ***CALL*SCAT_OB        (KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!*    EXTERNALS
!     ---------

!        GETDB    - GET DB
!        PUTDB    - STORE DB
!        CMOCTMAP - GET OBS./CODE TYPE SEQUENCE NO.s
!        SYNOPIN  - PROCESS INPUT SYNOPs
!        AIREPIN  - PROCESS INPUT AIREPs
!        SATOBIN  - PROCESS INPUT SATOBs
!        SATAMIN  - PROCESS INPUT SATOBs
!        DRIBUIN  - PROCESS INPUT DRIBUs
!        TEMPIN   - PROCESS INPUT TEMPs
!        PILOTIN  - PROCESS INPUT PILOTs
!        AWPRFIN  - PROCESS INPUT AMERICAN WIND PROFILERs
!        SSMIIN   - PROCESS INPUT SSMIs
!        TRMMIN   - PROCESS INPUT TRMMs
!        MERTSIN  - PROCESS INPUT SATEMs/TOVSs
!        RAD1CIN  - PROCESS INPUT LEVEL 1Cs
!        GEOSRIN  - PROCESS INPUT GEOS. radiaces
!        HIRTOIN  - PROCESS INPUT TOVS
!        PAOBIN   - PROCESS INPUT PAOBs
!        ERSIN    - PROCESS INPUT ERSs AND ASCATs
!        NSCATIN  - PROCESS INPUT NSCATs
!        ASCATIN  - PROCESS INPUT ASCATs
!        OSCATIN  - PROCESS INPUT OCEANSAT-2s
!        ABOR1    - ABNORMAL TERMINATION
!        ABOR1    - ABNORMAL TERMINATION

!        SCAT_OB IS CALLED BY SUBROUTINE MKCMARPL

!**   MODIFICATIONS
!     -------------
!        07-04-12 Chris Payan: call to subroutines under LMKCMARPL(2D) control
!        07-04-13 Chris Payan: merge with dal_CY32_ASCAT branch
!        07-05-10 Chris Payan: L2 data not reversed 
!                                (producteur code KNMI=99 (in insttype@hdr (sorry!)))
!        08-04-10 K. Scipal - introduction of ASCAT soil moisture data
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMCOCTP , ONLY :       NSCATT        &
 &,NSCAT1   ,NSCAT2   ,NSCAT3   ,NSCAT4   ,NSCAT5   ,NSCAT6  &
 &,NSCATTSQ ,NSCAT1SQ ,NSCAT2SQ ,NSCAT3SQ ,NSCAT4SQ ,NSCAT5SQ ,NSCAT6SQ
USE YOMOBS   , ONLY : LMKCMARPL
USE YOMLUN   , ONLY : NULOUT   
USE YOMMKODB , ONLY : LASCATSM

IMPLICIT NONE

INTEGER(KIND=JPIM) ,INTENT(IN)   :: KTSLOT
INTEGER(KIND=JPIM) ,INTENT(IN)   :: KPOOLNO
INTEGER(KIND=JPIM) :: INFO(1), IRET

INTEGER(KIND=JPIM) :: ILEN, IOBTYP, ICDTYPE, IPRODCD, IOBS
INTEGER(KIND=JPIM) :: IOBSQNO,ICDSQNO
INTEGER(KIND=JPIM) ::  JOBS
INTEGER(KIND=JPIM) :: IERS, IASCAT,  INSCAT, IQSCAT, IISCAT
INTEGER(KIND=JPIM) :: IPROD_KNMI

LOGICAL :: LLMESAGE, LLTAKEN

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cmoctmap.intfb.h"
#include "ersin.intfb.h"
#include "nscatin.intfb.h"
#include "qscatin.intfb.h"
#include "oscatin.intfb.h"
#include "ascatin.intfb.h"
#include "ascatsm_cdfmatch.intfb.h"

#include "openmp_obs.h"

!*       1.       SCAT OBS
!                 --------

!*          1.1   INITIALIZE DB (LOCALL; ALL SCAT REPs)

IF (LHOOK) CALL DR_HOOK('SCAT_OB',0,ZHOOK_HANDLE)
IPROD_KNMI=99
IRET = 0
CALL GETDB('SCAT',1,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, KPOOLNO, -1, -1, -1)  
IF(IRET == 0) THEN
  WRITE(NULOUT,'(''     NO REPORTS (SCAT) AVAILABLE '')')
ELSE
  WRITE(NULOUT,'(''     REPORTS (SCAT) FOUND: '',I10)') IRET
  IERS  = 0
  IASCAT = 0
  INSCAT = 0
  IQSCAT = 0
  IISCAT = 0
  CHECK_CODE_TYPE: DO JOBS = 1, IRET
    IOBTYP  = ROBHDR(JOBS,MDBOTP)
    ICDTYPE = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)
    IPRODCD = ROBHDR(JOBS,MDB_INSTTYPE_AT_HDR)
    IF(    ICDTYPE == NSCAT1 .OR. (ICDTYPE == NSCAT2 .AND. IPRODCD /= IPROD_KNMI) ) THEN
      IERS  = IERS + 1
    ELSEIF(ICDTYPE == NSCAT3 .AND. IPRODCD /= IPROD_KNMI)                   THEN
      IASCAT = IASCAT + 1
    ELSEIF(ICDTYPE == NSCAT4)                   THEN
      INSCAT = INSCAT + 1
    ELSEIF(ICDTYPE == NSCAT5)                   THEN
      IQSCAT = IQSCAT + 1
    ELSEIF(ICDTYPE == NSCAT6)                   THEN
      IISCAT = IISCAT + 1
    ENDIF
  ENDDO CHECK_CODE_TYPE
  WRITE(NULOUT,'(''     ERS /=KNMI   FOUND: '',I10)') IERS
  WRITE(NULOUT,'(''     ASCAT /=KNMI FOUND: '',I10)') IASCAT
  WRITE(NULOUT,'(''     NSCAT FOUND: '',I10)') INSCAT
  WRITE(NULOUT,'(''     QSCAT FOUND: '',I10)') IQSCAT
  WRITE(NULOUT,'(''     OSCAT FOUND: '',I10)') IISCAT
ENDIF
CALL FLUSH(NULOUT)
ILEN = IRET

!*          1.2   LOOP OVER ALL THE SCAT OBSERVATIONS

OBS_LOOP: DO JOBS = 1, ILEN
  IOBS = JOBS
  IOBTYP  = ROBHDR(IOBS,MDBOTP)
  ICDTYPE = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  IPRODCD = ROBHDR(IOBS,MDB_INSTTYPE_AT_HDR)
  LLMESAGE= .FALSE.
  LLTAKEN = .FALSE.
  CALL CMOCTMAP('CODETOSQ',NULOUT,IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO)

!*          1.3   SCAT
  
  IF( IOBTYP == NSCATT )            THEN

!*          1.3.1 ERS1 AND ERS2

    IF    ( ICDTYPE == NSCAT1 )                 THEN
      IF(LMKCMARPL(NSCAT1SQ,NSCATTSQ)) CALL ERSIN  (IOBS,&
       & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       & LLMESAGE,LLTAKEN)  

    ELSEIF( ICDTYPE == NSCAT2 )                 THEN
      IF(LMKCMARPL(NSCAT2SQ,NSCATTSQ) .AND. IPRODCD /= IPROD_KNMI) &
       & CALL ERSIN  (IOBS,&
       &    IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       &    LLMESAGE,LLTAKEN)  

!*          1.3.2 NSCAT

    ELSEIF( ICDTYPE == NSCAT4 )                 THEN
      IF(LMKCMARPL(NSCAT4SQ,NSCATTSQ)) CALL NSCATIN(IOBS,&
       & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       & LLMESAGE,LLTAKEN)  

!*          1.3.3 QUIKSCAT

    ELSEIF( ICDTYPE == NSCAT5 )                 THEN
      IF(LMKCMARPL(NSCAT5SQ,NSCATTSQ)) CALL QSCATIN(IOBS,&
       & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       & LLMESAGE,LLTAKEN)  

!*          1.3.4 ASCAT

    ELSEIF( ICDTYPE == NSCAT3 )                 THEN
      IF(LMKCMARPL(NSCAT3SQ,NSCATTSQ) .AND. IPRODCD /= IPROD_KNMI) &
       & CALL ASCATIN(IOBS,&
       &    IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       &    LLMESAGE,LLTAKEN)

      !*    Do the transformation from relative to absolute soil moisture
      IF (LASCATSM) CALL ASCATSM_CDFMATCH(IOBS)

!*          1.3.5 OCEANSAT2

    ELSEIF( ICDTYPE == NSCAT6 )                 THEN
      IF(LMKCMARPL(NSCAT6SQ,NSCATTSQ)) CALL OSCATIN(IOBS,&
       & IOBTYP,ICDTYPE,IOBSQNO,ICDSQNO,&
       & LLMESAGE,LLTAKEN)  

!*          1.3.5 UNKNOWN SCAT CODE TYPE
    
    ELSE 
      WRITE(NULOUT,'(''0ERROR IN SUBROUTINE SCAT_OB    '')')
      WRITE(NULOUT,'('' UNKNOWN SCAT CODE TYPE'',I10)') ICDTYPE
      CALL ABOR1(' SCAT_OB    ; WRONG SCAT CODE TYPE ')
    ENDIF

!*          1.4   UNKNOWN SCAT OBS TYPE

  ELSE
    WRITE(NULOUT,'(''0ERROR IN SUBROUTINE SCAT_OB    '')')
    WRITE(NULOUT,'('' UNKNOWN SCAT OBS TYPE'',I10)') IOBTYP
    CALL ABOR1('SUBROUTINE SCAT_OB    ; WRONG SCAT OBS TYPE ')
  ENDIF
ENDDO OBS_LOOP

!           1.3   PUT THE DB AWAY

CALL PUTDB('SCAT',1,IRET,INFO,0,ZINFO,0)

!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SCAT_OB',1,ZHOOK_HANDLE)
END SUBROUTINE SCAT_OB
