SUBROUTINE SCATSIN(KSET,KNOSOL)

!**** SUBROUTINE SCATSIN - PROCESS INPUT SCATTEROMETERS

!        D. VASILJEVIC     ECMWF     12/12/00 (original 04/07/94)

!     PURPOSE
!     -------

!        PROCESS INPUT SCATTEROMETERS

!**   INTERFACE
!     ---------

!        CALL SCATSIN(KSET,KNOSOL)
!                                                       KBODYETL )
!     WHERE:
!            KSET    = SET NO.
!            KNOSOL  = SCAT. INTERFACE NO SOLUTION COUNTER

!        SCATSIN IS CALLED BY SUBROUTINE

!     METHOD
!     ------

!        BRANCH OFF TO A REQUIRED OBS. TYPE PROCESSING.

!     EXTERNALS
!     ---------

!        ESAIN   - PROCESS ERS SCATTEROMETER OBS
!        NSCATIN - PROCESS NSCAT SCATTEROMETER OBS
!        ASCATIN - PROCESS ASCAT SCATTEROMETER OBS
!        OSCATIN - PROCESS OCEANSAT-2 SCATTEROMETER OBS


!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMOBSET , ONLY : MTYPOB   ,MLNSET
USE YOMCOCTP , ONLY :    NSCAT1   ,NSCAT2   ,NSCAT3   ,NSCAT4   ,NSCAT5   ,NSCAT6
USE YOMDB
USE YOMSCC   , ONLY : LSCDPR
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNOSOL 

INTEGER(KIND=JPIM) :: IOBTYP, ICDTYP, IOBSQNO, ICDSQNO,&
 & JSCAT, ISCAT  

!     LOGICAL SCALARS
LOGICAL ::      LLMESAGE, LLTAKEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ersin.intfb.h"
#include "ascatin.intfb.h"

!-----------------------------------------------------------------------
#include "openmp_obs.h"

!*

!        0.             PRESET
!                       ------
IF (LHOOK) CALL DR_HOOK('SCATSIN',0,ZHOOK_HANDLE)
IOBTYP = MOD(MTYPOB(KSET),100)
IF(LSCDPR) WRITE(NULOUT,'(''   PRELIMINARY SCAT WORK'')')

!*
!-----------------------------------------------------------------------

!        1.             BRANCH OFF FOR ERS/NSCAT/QSCAT
!                       ------------------------------

SCAT_LOOP: DO JSCAT = 1, MLNSET(KSET)
  ISCAT = JSCAT

  ICDTYP = ROBHDR(JSCAT,MDB_CODETYPE_AT_HDR)
  CALL DBOCTMAP('CODETOSQ',NULOUT,IOBTYP ,ICDTYP ,IOBSQNO,ICDSQNO )

!*         1.1          ERS-1

  IF(ICDTYP == NSCAT1 .OR. ICDTYP == NSCAT2      )         THEN

    CALL ERSIN(ISCAT,&
     & IOBTYP,ICDTYP,IOBSQNO,ICDSQNO,&
     & LLMESAGE,LLTAKEN)  

!*         1.2          ASCAT

  ELSEIF(ICDTYP  == NSCAT3  )         THEN

    CALL ASCATIN(ISCAT,&
     & IOBTYP,ICDTYP,IOBSQNO,ICDSQNO,&
     & LLMESAGE,LLTAKEN)  

!*         1.3          NSCAT

  ELSEIF(ICDTYP  == NSCAT4  )         THEN

!   CALL NSCATIN(ISCAT,KNOSOL)
!               &IOBTYP,ICDTYP,IOBSQNO,ICDSQNO,&
!               &LLMESAGE,LLTAKEN)

!*         1.4          QSCAT

  ELSEIF(ICDTYP  == NSCAT5  )         THEN

!   CALL QSCATIN(ISCAT,&
!               &IOBTYP,ICDTYP,IOBSQNO,ICDSQNO,&
!               &LLMESAGE,LLTAKEN)

!*         1.5          OCEANSAT2

  ELSEIF(ICDTYP  == NSCAT6  )         THEN

!   CALL OSCATIN(ISCAT,&
!               &IOBTYP,ICDTYP,IOBSQNO,ICDSQNO,&
!               &LLMESAGE,LLTAKEN)


!*          1.6         UNKNOWN SCAT CODETYPE

  ELSE
    WRITE(NULOUT,'(''0ERROR IN SUBROUTINE SCATSIN'')')
    WRITE(NULOUT,'('' UNKNOWN SCAT CODE TYPE'',I10)') ICDTYP
    CALL ABOR1('SUBROUTINE SCATSIN')
  ENDIF
!*
!           1.7         THE SET OF OBSERVATIONS

ENDDO SCAT_LOOP

!*
!-----------------------------------------------------------------------

!        2.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('SCATSIN',1,ZHOOK_HANDLE)
END SUBROUTINE SCATSIN
