SUBROUTINE SETTC

!**** SUBROUTINE SETTC - SETUP FOR TIME CORRELATED OBSERVATION ERRORS

!          HEIKKI JARVINEN AND ERIK ANDERSSON, ECMWF 25/MAY/1997

!*    PURPOSE
!     -------

!        SET UP TABLES TO BE USED IN THE CALCULATION OF THE COST FUNCTION
!     FOR THE SYNOP AND DRIBU OBSERVATIONS FOR WHICH THE OBSERVATION ERRORS
!     ARE CORRELATED IN TIME.

!**   INTERFACE
!     ---------

!     ***CALL SETTC

!        SETTC IS CALLED BY SUOBS

!**   EXTERNALS
!     ---------

!*    METHOD
!     ------

!**   MODIFICATIONS
!     -------------
!     Sami Saarinen 19980224 - Calls to the new (multi-)keysort added
!     Sami Saarinen 20010206 - ZLOARR & ZGLARR transposed for faster access

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY :    NACTIM
USE YOMOBA   , ONLY : MOBDD1
USE YOMCMDDR , ONLY : NCD1NSY  ,NCD1NDB
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMCOCTP , ONLY : NSYNOP   ,NSHSCD   ,NABSCD   ,NSHRED   ,&
 & NATSHS   ,NDRIBU  
USE YOMANA   , ONLY : NANDAT   ,NANTIM
USE YOMVNMB  , ONLY : NVNUMB
USE YOMSCREE , ONLY : NAMBPL   ,CAMBPL
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMCT0   , ONLY : NPROC    ,LSCREEN
USE YOMANCS  , ONLY : NMDI     ,RMDI
USE YOMCOSJO , ONLY :      NTCOBS   ,NTCLOC   ,LTC      ,&
 & NTCOFF   ,NTCABN_L ,NTCOPT_L ,NTCDPT_L ,&
 & NTCOWN   ,NTCTIM   ,NTCVOI   ,RTCNDP   ,&
 & RTCAQC   ,RTCLQC   ,RTCNDPHR ,&
 & NTCONM_L ,NTCIXG_L ,NTCOBS_L   
USE YOMCMBDY , ONLY : NCMDACBP ,NCMDACOC
USE ecsort_mix   , ONLY : keysort
USE YOMDB
USE MPL_MODULE

IMPLICIT NONE

#include "fcobs.h"

INTEGER(KIND=JPIM) :: IGLSYN(1)
INTEGER(KIND=JPIM) :: I_MKEY(2)
CHARACTER(LEN=8) ::  CLSTID

REAL(KIND=JPRB),    ALLOCATABLE :: ZLOARR (:,:)
REAL(KIND=JPRB),    ALLOCATABLE :: ZGLARR (:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IXCORO (:)
REAL(KIND=JPRB) :: ZAQC,ZLQC
INTEGER(KIND=JPIM) :: IREFAN,ILOSYN,IABNOB,IOBTYP,ICDTYP &
 & ,ICMVNM,IDSTA_ACTIVE &
 & ,IOBDAT,IOBTIM,IVOI,ITMN,IDIMLO,IREFOBS,IDIMGL,IPOSD &
 & ,IPOWN,IPONM,IPAQC,IPLQC,IPRFT &
 & ,IPABN,IPOPT,IPDPT,IPSTID,IPVOI &
 & ,IKEY,ISHIFT,IC,IXDIM,IY,IX,INCORO,IXGL,IXGLL,IXL &
 & ,JOBS,J,JNLV,J1,J2,JC,INFO(1),  IOBTOT, INRA  
INTEGER(KIND=JPIM) :: iret, isize, ista, iend, JROC
LOGICAL :: LLPSORZ

INTEGER(KIND=JPIM) :: JJ, IRECVCOUNTS(NPROC)

REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB), ALLOCATABLE :: ZSENDBUF(:), ZRECVBUF(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "reftim.intfb.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!*       0.    INITIALIZATION
!              --------------

!*          0.1   NUMBER OF TIME CORRELATED DATA IS SET TO ZERO
! No time-correlation before screening has been done.

IF (LHOOK) CALL DR_HOOK('SETTC',0,ZHOOK_HANDLE)
IF(LSCREEN .OR. .NOT.LTC .AND. LHOOK) CALL DR_HOOK('SETTC',1,ZHOOK_HANDLE)
IF(LSCREEN .OR. .NOT.LTC) RETURN

CALL GETDB('SETTC', 0, IRET, INFO, 0,ZINFO,0, &
 & -1, -1, -1, -1, -1, -1)  

!IF (IRET == 0) RETURN

IOBTOT = IRET

WRITE(NULOUT,'('' *** SETUP TIME CORRELATION OF OBS.ERRORS '',I10)') IOBTOT
CALL FLUSH(NULOUT)

!*          0.2   DEFINE POINTERS FOR FIRST DIMENSION OF ZLOARR

IPOWN=1
IPONM=2
IPAQC=3
IPLQC=4
IPRFT=5
IPABN=6
IPOPT=7
IPDPT=8
IPSTID=9
IPVOI=10

!*          0.3   ALLOCATE/INITIALIZE THE LOCAL ARRAY

ITMN = MAX(IPOWN,IPONM,IPAQC,IPLQC,IPRFT,IPABN,IPOPT,IPDPT,IPSTID,IPVOI)
IDIMLO = MOBDD1(NCD1NSY) + MOBDD1(NCD1NDB)
IDIMLO = MAX(IDIMLO,1)
IDIMLO = IDIMLO + MOD(IDIMLO+1,2) ! Make 1st dimension odd
ALLOCATE(ZLOARR(IDIMLO,ITMN))
WRITE(NULOUT,9990) 'ZLOARR   ',SIZE(ZLOARR   ),SHAPE(ZLOARR   )

!*          0.4   ANALYSIS REFERENCE TIME

CALL REFTIM(NANDAT,NANTIM,IREFAN)

!*
!     ------------------------------------------------------------------

!*       1.       COLLECT ALL SYNOP/DRIBU REPORTS FROM LOCAL CMA
!                 ----------------------------------------------
!*
!           1.1   LOOP OVER THE OBSERVATIONS, THIS INCLUDES ALL TIMESLOTS

ILOSYN = 0
OBS_LOOP1: DO JOBS = 1, IOBTOT
!*
!           1.2   SELECT OBSERVATION TYPES (SYNOP AND DRIBU)

  IABNOB = ROBHDR(JOBS,MDB_ABNOB_AT_INDEX)
  IOBTYP = ROBHDR(JOBS,MDBOTP)
  IF(IOBTYP /= NSYNOP .AND. IOBTYP /= NDRIBU) CYCLE OBS_LOOP1

!*          1.3   CONVERT STATION ID TO CHARACTER

  CLSTID = TRANSFER(ROBHDR(JOBS,MDBSID), CLSTID)

!*          1.4   SHIP SYNOP REPORTS
!                 - IGNORE FOR AMBIGUOUS PLATFORMS

  ICDTYP = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)
  IF(IOBTYP == NSYNOP) THEN
    IF(ICDTYP == NSHSCD .OR. ICDTYP == NABSCD  .OR.&
       & ICDTYP == NSHRED .OR. ICDTYP == NATSHS) THEN  
      DO J = 1, NAMBPL
        IF(CLSTID == CAMBPL(J)) CYCLE OBS_LOOP1
      ENDDO
    ENDIF
  ENDIF

!*          1.5   FIND Active SURFACE PRESSURE OR GEOPOTENTIAL HEIGHT Data
!*                DISREGARD THOSE DATA NOT TO BE USED

  LLPSORZ=.FALSE.
  DAT_LOOP1: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
    IDSTA_ACTIVE=IBITS(INT(ROBODY(JNLV,MDBDSTA)), NCMDACBP, NCMDACOC)
    ICMVNM = ROBODY(JNLV,MDBVNM)
    IF(IDSTA_ACTIVE == 1 .AND. &
       & (ICMVNM == NVNUMB(47) .OR. ICMVNM == NVNUMB( 3)) ) THEN  
      IF(LLPSORZ) THEN
        CALL ABOR1('SETTC: >1 PS/Z DATUM')
      ELSE
        ZAQC=ROBODY(JNLV,MDB_QC_PGE)
        ZLQC=ROBODY(JNLV,MDB_QC_L)
        LLPSORZ=.TRUE.
        IPOSD=JNLV
      ENDIF
    ENDIF
  ENDDO DAT_LOOP1
  IF(.NOT.LLPSORZ) CYCLE OBS_LOOP1

!*          1.6   INCLUDE THIS REPORT

  ILOSYN = ILOSYN + 1
  ZLOARR(ILOSYN,IPOWN) = NPRCIDS(MYPROC)
  ZLOARR(ILOSYN,IPONM) = ROBHDR(JOBS,MDBONM)
  ZLOARR(ILOSYN,IPAQC) = ZAQC
  ZLOARR(ILOSYN,IPLQC) = ZLQC
  IOBDAT = ROBHDR(JOBS,MDBDAT)
  IOBTIM = ROBHDR(JOBS,MDBETM)
  CALL REFTIM(IOBDAT,IOBTIM,IREFOBS)
  ZLOARR(ILOSYN,IPRFT) = IREFOBS
  ZLOARR(ILOSYN,IPABN) = IABNOB
  ZLOARR(ILOSYN,IPOPT) = JOBS
  ZLOARR(ILOSYN,IPDPT) = IPOSD
  READ(CLSTID(1:8),'(A8)') ZLOARR(ILOSYN,IPSTID)
  IVOI=IOBTYP*1000+ICDTYP
  ZLOARR(ILOSYN,IPVOI) = IVOI
  IF(IOBTYP >= 10.OR. ICDTYP >= 1000)CALL ABOR1('SETTC: IVOI UN-PACKABLE')

!*          1.7    CLOSE THE LOOP OVER CURRENT SET

ENDDO OBS_LOOP1
!*
!     ------------------------------------------------------------------

!        2.       COLLECT AND DISTRIBUTE THE LOCAL ARRAYS
!                 ---------------------------------------

!*          2.1   SUM GLOBALLY THE ARRAY LENGTHS

IGLSYN(1) = ILOSYN
CALL MPL_ALLREDUCE(IGLSYN, 'SUM',CDSTRING='SETTC:')
IDIMGL = IGLSYN(1)
WRITE(NULOUT,'('' IDIMGL '',i5)') IDIMGL

IF(IDIMGL == 0) GOTO 900

!*          2.2   ALLOCATE/INITIALIZE THE GLOBAL ARRAY

INRA = IDIMGL + MOD(IDIMGL+1,2) ! Make 1st dimension odd
ALLOCATE(ZGLARR(INRA,ITMN))
WRITE(NULOUT,9990) 'ZGLARR   ',SIZE(ZGLARR   ),SHAPE(ZGLARR   )

!*          2.3   SUBSTITUTE WITH THE LOCAL CONTRIBUTION

ALLOCATE(ZSENDBUF(ILOSYN * ITMN))
WRITE(NULOUT,9990) 'ZSENDBUF ',SIZE(ZSENDBUF ),SHAPE(ZSENDBUF )

JJ=0
DO J2 = 1, ITMN
  DO J1 = 1, ILOSYN
    JJ = JJ + 1
    ZSENDBUF(JJ) = ZLOARR(J1,J2)
  ENDDO
ENDDO

DEALLOCATE(ZLOARR)
WRITE(NULOUT,9991) 'ZLOARR   ',myproc

ALLOCATE(ZRECVBUF(IDIMGL * ITMN))
WRITE(NULOUT,9990) 'ZRECVBUF ',SIZE(ZRECVBUF ),SHAPE(ZRECVBUF )

IRECVCOUNTS(:) = 0
CALL MPL_ALLGATHERV(SIZE(ZSENDBUF), IRECVCOUNTS,CDSTRING='SETTC:')
WRITE(NULOUT,*)'SETTC: IRECVCOUNTS(1:NPROC)=',IRECVCOUNTS(:)

!*          2.4   COLLECT THE GLOBAL ARRAY

CALL MPL_ALLGATHERV(ZSENDBUF, ZRECVBUF, IRECVCOUNTS,CDSTRING='SETTC:')

DEALLOCATE(ZSENDBUF)
WRITE(NULOUT,9991) 'ZSENDBUF ',myproc

JJ = 0
IEND = 0
DO JROC = 1,NPROC
  ISTA = IEND + 1
  IEND = IEND + IRECVCOUNTS(JROC)/ITMN
  DO J2 = 1, ITMN
    DO J1 = ISTA, IEND
      JJ = JJ + 1
      ZGLARR(J1,J2) = ZRECVBUF(JJ)
    ENDDO
  ENDDO
ENDDO

DEALLOCATE(ZRECVBUF)
WRITE(NULOUT,9991) 'ZRECVBUF ',myproc

!*          2.5   SORT USING UNIQUE OBSERVATION SEQUENCE NUMBER
!*                SORT ONCE AGAIN USING STATION ID THIS TIME

!      IKEY = IPONM
!      CALL KEYSORT(NULOUT,IKEY,ZGLARR,ITMN,IDIMGL)
!      IKEY = IPSTID
!      CALL KEYSORT(NULOUT,IKEY,ZGLARR,ITMN,IDIMGL)

I_MKEY(1) = IPSTID
I_MKEY(2) = IPONM
isize = IDIMGL
CALL KEYSORT(iret, ZGLARR, isize, multikey=I_MKEY)

!*          2.6   PREPARE NEW SORTING KEY, WHICH IS A COMBINATION OF
!*                STATION ID, OBS-TYPE AND CODE TYPE.

ISHIFT=10000
IC=1
ZGLARR(1,IPVOI)=IC*ISHIFT+INTREA(ZGLARR(1,IPVOI))
DO J = 2, IDIMGL
  IF(ZGLARR(J,IPSTID) /= ZGLARR(J-1,IPSTID)) THEN
    IC=IC+1
  ENDIF
  ZGLARR(J,IPVOI)=IC*ISHIFT+INTREA(ZGLARR(J,IPVOI))
ENDDO
WRITE(NULOUT,*) ' SETTC: THERE ARE ',IC,' DIFFERENT ST-ID'

IKEY = IPVOI
isize = IDIMGL
CALL KEYSORT(iret, ZGLARR, isize, key=IKEY)

!*
!     ------------------------------------------------------------------

!        3.       FIND OUT THE NUMBER OF CORRELATED OBSERVATIONS
!                 ----------------------------------------------

!*          3.0   INITIALIZE

ALLOCATE(IXCORO(IDIMGL))
WRITE(NULOUT,9990) 'IXCORO   ',SIZE(IXCORO   ),SHAPE(IXCORO   )

IXDIM = 0
IY = 1
IX = 1
INCORO=1
IXCORO(1)=1

!*          3.1   SELECT THE GROUPS OF STATIONS BASED ON THEIR
!*                IDENTIFIER, OBS TYPE AND CODE TYPE
!*                REQUIRE THAT THERE ARE AT LEAST 2 OF THE SAME

DO J = 2, IDIMGL
  INCORO=INCORO+1

  IF(ZGLARR(J,IPVOI) == ZGLARR(J-1,IPVOI)) THEN
    IY = IY + 1
    IF(IY > NACTIM) THEN
      WRITE(CLSTID(1:8),'(A8)') ZGLARR(J,IPSTID)
      WRITE(NULOUT,*) ' SETTC WARNING: (ERIK will be interested) '&
       & ,IY,CLSTID(1:8),J &
       & ,INTREA(ZGLARR(J,IPVOI))&
       & ,INTREA(ZGLARR(J,IPRFT))-IREFAN  
!          Reset to NACTIM rather than ABORT!
!          There should never be more than NACTIM data of the same 
!          type and station id. There have, however, been two cases that
!          did abort. It was in both cases due to a buoy reporting from
!          two different locations. METAPS checks are supposed to spot
!          this kind of problem if there are semi-continuous time-series
!          of the bouy in the past. When that is not the case, the check
!          may pass both locations to the screening. And the sreening 
!          redundancy check won't spot the problem.
!          By setting IY=NACTIM we make those extra data be used in analysis
!          without serial correlation. No big deal, if not too frequent.
!EA         CALL ABOR1('SETTC: IY.GT.NACTIM')
      IY=NACTIM
!          Take the previous one off the list.
      INCORO=INCORO-1
    ENDIF
  ELSE
!          A minimum of two is required, to get an entry on the list.
    IF(IY >= 2) THEN
      IX = IX + 1
    ELSE
!          Take the previous (single) one off the list.
      INCORO=INCORO-1
    ENDIF
    IY = 1
  ENDIF
  IXCORO(INCORO)=J
ENDDO

IF(IY >= 2) THEN
  IXDIM=IX
ELSE
  INCORO=INCORO-1
  IXDIM=IX-1
ENDIF

NTCOBS=INCORO
NTCLOC=IXDIM
WRITE(NULOUT,'('' *** SETTC: TIME CORR COUNTERS '')')
WRITE(NULOUT,'(''     - NTCLOC '',I6)') NTCLOC
WRITE(NULOUT,'(''     - NTCOBS '',I6)') NTCOBS
IF(NTCLOC == 0) GOTO 500

!*          3.2    ALLOCATE TIME CORR ARRAYS

!*            3.2.1    A COUNTER ARRAY
IF(ALLOCATED(NTCOFF))DEALLOCATE(NTCOFF)
ALLOCATE(NTCOFF(NTCLOC))
WRITE(NULOUT,9990) 'NTCOFF   ',SIZE(NTCOFF   ),SHAPE(NTCOFF   )

!*            3.2.6    OWNER PROCESSOR
IF(ALLOCATED(NTCOWN))DEALLOCATE(NTCOWN)
ALLOCATE(NTCOWN(NTCOBS))
WRITE(NULOUT,9990) 'NTCOWN   ',SIZE(NTCOWN   ),SHAPE(NTCOWN   )

!*            3.2.7    REFERENCE TIME (In minutes)
IF(ALLOCATED(NTCTIM))DEALLOCATE(NTCTIM)
ALLOCATE(NTCTIM(NTCOBS))
WRITE(NULOUT,9990) 'NTCTIM   ',SIZE(NTCTIM   ),SHAPE(NTCTIM   )

!*            3.2.8    NORMALIZED DEPARTURES (PRESSURE OR GEOPOTENTIAL)
IF(ALLOCATED(RTCNDP))DEALLOCATE(RTCNDP)
ALLOCATE(RTCNDP(NTCOBS))
WRITE(NULOUT,9990) 'RTCNDP   ',SIZE(RTCNDP   ),SHAPE(RTCNDP   )
IF(ALLOCATED(RTCNDPHR))DEALLOCATE(RTCNDPHR)
ALLOCATE(RTCNDPHR(NTCOBS))
WRITE(NULOUT,9990) 'RTCNDPHR ',SIZE(RTCNDPHR ),SHAPE(RTCNDPHR )

!*            3.2.9    PROBABILITY OF GROSS ERROR, VARQC
IF(ALLOCATED(RTCAQC))DEALLOCATE(RTCAQC)
ALLOCATE(RTCAQC(NTCLOC))
WRITE(NULOUT,9990) 'RTCAQC   ',SIZE(RTCAQC   ),SHAPE(RTCAQC   )

!*            3.2.10   RANGE OF POSSIBLE VALUES, VARQC
IF(ALLOCATED(RTCLQC))DEALLOCATE(RTCLQC)
ALLOCATE(RTCLQC(NTCLOC))
WRITE(NULOUT,9990) 'RTCLQC   ',SIZE(RTCLQC   ),SHAPE(RTCLQC   )

!*            3.2.11   VARIOUS OBSERVATION INFORMATIONS
IF(ALLOCATED(NTCVOI))DEALLOCATE(NTCVOI)
ALLOCATE(NTCVOI(NTCOBS))
WRITE(NULOUT,9990) 'NTCVOI   ',SIZE(NTCVOI   ),SHAPE(NTCVOI   )

!*            3.2.12   INITIALIZE THE ARRAYS
NTCOFF(:) = NMDI
NTCOWN(:) = NMDI
NTCTIM(:) = NMDI
RTCNDP(:) = RMDI
RTCNDPHR(:) = RMDI
RTCAQC(:) = RMDI
RTCLQC(:) = RMDI
NTCVOI(:) = NMDI
!*
!     ------------------------------------------------------------------

!        4.       FILL THE ARRAYS AS FAR AS POSSIBLE AT THIS STAGE
!                 ------------------------------------------------

!*          4.1   SELECT THE GROUPS OF STATIONS BASED ON THEIR IDENTIFIER

IX=1
IXGL=IXCORO(1)
NTCOFF(IX) = 1
RTCAQC(IX) = ZGLARR(IXGL,IPAQC)
RTCLQC(IX) = ZGLARR(IXGL,IPLQC)
DO JC = 2, NTCOBS
  IXGL=IXCORO(JC)
  IXGLL=IXCORO(JC-1)

  IF(ZGLARR(IXGL,IPVOI) /= ZGLARR(IXGLL,IPVOI)) THEN
    IX=IX+1
    NTCOFF(IX) = JC
    RTCAQC(IX) = ZGLARR(IXGL,IPAQC)
    RTCLQC(IX) = ZGLARR(IXGL,IPLQC)
  ENDIF
ENDDO

IF(IX /= NTCLOC) THEN
  WRITE(NULERR,*) ' WRONG IX,NTCLOC: ',IX,NTCLOC
  CALL ABOR1('SETTC: WRONG IX COUNTER')
ENDIF

DO JC = 1, NTCOBS
  IXGL=IXCORO(JC)
  NTCOWN(JC) = ZGLARR(IXGL,IPOWN)
  NTCVOI(JC) = ZGLARR(IXGL,IPVOI)
  NTCTIM(JC) = ZGLARR(IXGL,IPRFT)
ENDDO

!*          4.2   CHECK NOTHING IS MISSING

IF(ANY(NTCOWN(:) == NMDI)) THEN
  WRITE(NULERR,*) ' SETTC: Some NTCOWN MISSING: ',COUNT(NTCOWN(:) == NMDI)
  CALL ABOR1('SETTC: Some NTCOWN MISSING')
ENDIF

!*          4.3   ARRAYS LOCAL TO EACH PE
!                 HOW MANY TIME CORR. DATA BELONG TO THIS PE?
!                 FOR ARRAYS RELATED TO ODB ACCESS

NTCOBS_L=COUNT(NTCOWN(:) == NPRCIDS(MYPROC))

!*            4.3.1    INDICES TO GLOBAL NTCxxx-ARRAYS
IF(ALLOCATED(NTCIXG_L))DEALLOCATE(NTCIXG_L)
ALLOCATE(NTCIXG_L(NTCOBS_L))
WRITE(NULOUT,9990) 'NTCIXG_L ',SIZE(NTCIXG_L),SHAPE(NTCIXG_L)
NTCIXG_L(:) = NMDI

!*            4.3.2    UNIQUE OBSERVATION SEQUENCE NUMBERS

IF(ALLOCATED(NTCONM_L))DEALLOCATE(NTCONM_L) 
ALLOCATE(NTCONM_L(NTCOBS_L))
WRITE(NULOUT,9990) 'NTCONM_L ',SIZE(NTCONM_L),SHAPE(NTCONM_L)
NTCONM_L(:) = NMDI

!*            4.3.3      ABSOLUTE OBSERVATION NUMBER
IF(ALLOCATED(NTCABN_L))DEALLOCATE(NTCABN_L)
ALLOCATE(NTCABN_L(NTCOBS_L))
WRITE(NULOUT,9990) 'NTCABN_L ',SIZE(NTCABN_L),SHAPE(NTCABN_L)
NTCABN_L(:) = NMDI

!*            4.3.4    OBSERVATION POINTER
IF(ALLOCATED(NTCOPT_L))DEALLOCATE(NTCOPT_L)
ALLOCATE(NTCOPT_L(NTCOBS_L))
WRITE(NULOUT,9990) 'NTCOPT_L ',SIZE(NTCOPT_L),SHAPE(NTCOPT_L)
NTCOPT_L(:) = NMDI

!*            4.3.5    DATUM POSITION POINTER
IF(ALLOCATED(NTCDPT_L))DEALLOCATE(NTCDPT_L)
ALLOCATE(NTCDPT_L(NTCOBS_L))
WRITE(NULOUT,9990) 'NTCDPT_L ',SIZE(NTCDPT_L),SHAPE(NTCDPT_L)
NTCDPT_L(:) = NMDI

!*          4.4   FILL LOCAL ARRAYS
IXL=0
DO JC = 1, NTCOBS
  IF(NTCOWN(JC) == NPRCIDS(MYPROC)) THEN
    IXGL=IXCORO(JC)
    IXL=IXL+1
    NTCIXG_L(IXL)=JC
    NTCONM_L(IXL)=ZGLARR(IXGL,IPONM)
    NTCABN_L(IXL)=ZGLARR(IXGL,IPABN)
    NTCOPT_L(IXL)=ZGLARR(IXGL,IPOPT)
    NTCDPT_L(IXL)=ZGLARR(IXGL,IPDPT)
  ENDIF
ENDDO

!*          4.5   CHECK NOTHING IS MISSING
IF(IXL /= NTCOBS_L) THEN
  WRITE(NULERR,*) ' WRONG IXL,NTCOBS_L: ',IXL,NTCOBS_L
  CALL ABOR1('SETTC: WRONG IXL COUNTER')
ENDIF
IF(ANY(NTCABN_L(:) == NMDI)) THEN
  WRITE(NULERR,*) ' WRONG MISSING: ',COUNT(NTCABN_L(:) == NMDI)
  CALL ABOR1('SETTC: NTCABN_L MISSING')
ENDIF

!*
!     ------------------------------------------------------------------

!*       5.    TERMINATE
!              ---------
500 CONTINUE

!*          5.1   DEALLOCATE THE LOCAL ARRAYS

DEALLOCATE(ZGLARR)
WRITE(NULOUT,9991) 'ZGLARR   ',myproc
DEALLOCATE(IXCORO)
WRITE(NULOUT,9991) 'IXCORO   ',myproc
900 CONTINUE
IF (ALLOCATED(ZLOARR)) THEN
  DEALLOCATE(ZLOARR)
  WRITE(NULOUT,9991) 'ZLOARR   ',myproc
ENDIF

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
9991 FORMAT(1X,'ARRAY ',A10,' DEALLOCATED on Proc',I8)

!*          5.2   RETURN

CALL PUTDB('SETTC', 0, IRET, INFO, 0,ZINFO,0)

IF (LHOOK) CALL DR_HOOK('SETTC',1,ZHOOK_HANDLE)
END SUBROUTINE SETTC
