SUBROUTINE SHIPIN(KSYNP,&
 & KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,&
 & LDMESAGE,LDTAKEN)  

!**** SUBROUTINE SHIPIN - PROCESS INPUT SHIP

!        D. VASILJEVIC     ECMWF     13/12/00 (original 04/07/94)

!     PURPOSE
!     -------

!        PROCESS INPUT SHIP

!**   INTERFACE
!     ---------

!        CALL SHIPIN(KSYNP,
!                    KOBTYP,KCDTYP,KOBSQNO,KCDSQNO,
!                    LDMESAGE,LDTAKEN)
!     WHERE:
!            KSYNP   = REPORT POINTER
!            KOBTYP  = OBS. TYPE
!            KCDTYP  = CODE TYPE
!            KOBSQNO = OBS. TYPE SQ. NO.
!            KCDSQNO = CODE TYPE SQ. NO.
!            LDMESAGE= OUTPUT MESSAGE SWITCH  
!            LDTAKEN = REPORT TAKEN ("ODB" REPORT UPDATED) SWITCH  

!        SHIPIN IS CALLED BY SUBROUTINE ????????

!     METHOD
!     ------

!     EXTERNALS
!     ---------

!        PRLMCHK  - PRELIMINARY CHECKS
!        GETSETE  - GET/SET BODY ENTRY
!        ERRSTAT  - OBS. ERROR
!        SYNOPBE  - UPDATE SYNOP BODY ENTRY
!        PPVAFL   - PRESSURE/VARIABLE FLAG RESET
!        OBINSTP  - INTSTRUMENT TYPE
!        OBSCHAR  - OBS. CHARACTERISTICS

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDB
USE YOMCMHDR , ONLY : NALFLBP  ,NFLGBP   ,NFLGOC
USE YOMCMBDY , ONLY :  NCMBVFBP ,NCMBFLBP ,NCMBFLOC
USE YOMANCS  , ONLY : NMDI     ,RMDI     ,RDEGREES ,RADIANS  ,EPSMAC   ,RPAPM    ,RREDPEW1 ,RREDPEW2 ,&
 & RTICE    ,RTWAT    ,R2ES     ,R3IES    ,&
 & R3LES    ,R4IES    ,R4LES    ,RTT      ,&
 & RSYNANMH ,RSYNBARH ,RSANMOMH ,RSBAROMH ,RDANMOMH ,RDBAROMH ,RUPMLIM  ,RDWMLIM    
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMEV  , ONLY : ND1EVENT ,ND2EVENT
USE YOMNMCOD , ONLY : NPRESVC  ,NHEIGVC  ,NPRESCD
USE YOMMKODB , ONLY : LERRPERS ,NMKCMVSE ,&
 & LTENDCOR ,LTOTREM  ,&
 & LCHSHIP  ,LCHDRIBU,LSHDRCHO ,LSHDRCHOC ,&
 & L_SHIP_PS_BIAS
USE YOMLIM   , ONLY :       NFLLIMOP  ,&
 & RRHLIM   ,RRHTLIM  
USE YOMOERR  , ONLY : RHERR1   ,RHERR2   ,RHERR3
USE YOMERR   , ONLY : LRHERRMO
USE YOMLUN   , ONLY : NULOUT   
USE LOCAL_TRAFOS

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSYNP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSQNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDSQNO 
LOGICAL           ,INTENT(INOUT) :: LDMESAGE 
LOGICAL           ,INTENT(OUT)   :: LDTAKEN 
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IOBDATE,&
 & IOBTIME, IPFLG,&
 & IVNM, IPCODE, IPSTAFLG, IPMSLFLG, &
 & JDDFF, JPs, IPs, JZ, IZ, IVECFL, IVERCO, IALFLG, IUWINDF, IVWINDF, IUSHIPF, IVSHIPF,&
 & IPRESCD, ITYPST, INSTID, JDD, JFF, &
 & ISTID, INSTYP, IREPST  

INTEGER(KIND=JPIM) :: IDD, &
 & IDDSTAT, IDDEVEN1, IDDEVEN2, IDDRDBFL, IDDANFLG, &
 & IDDPFLG, IDDFLG  
INTEGER(KIND=JPIM) :: IFF, &
 & IFFSTAT, IFFEVEN1, IFFEVEN2, IFFRDBFL, IFFANFLG, &
 & IFFPFLG, IFFFLG  
INTEGER(KIND=JPIM) :: IU10M, JU10M, &
 & IUUSTAT, IUUEVEN1, IUUEVEN2, IUURDBFL, IUUANFLG, &
 & IUUPFLG, IUUFLG  
INTEGER(KIND=JPIM) :: IV10M, JV10M, &
 & IVVSTAT, IVVEVEN1, IVVEVEN2, IVVRDBFL, IVVANFLG, &
 & IVVPFLG, IVVFLG  
INTEGER(KIND=JPIM) :: JT2M, IT2M, &
 & ITTSTAT, ITTEVEN1, ITTEVEN2, ITTRDBFL, ITTANFLG, &
 & ITTPFLG, ITTFLG  
INTEGER(KIND=JPIM) :: IZZSTAT, IZZEVEN1, IZZEVEN2, IZZRDBFL, IZZANFLG, &
 & IZZPFLG, IZZFLG  
INTEGER(KIND=JPIM) :: IPSSTAT, IPSEVEN1, IPSEVEN2, IPSRDBFL, IPSANFLG, &
 & IPSPFLG, IPSFLG  
INTEGER(KIND=JPIM) :: JRH2M, IRH2M, &
 & IRHSTAT, IRHEVEN1, IRHEVEN2, IRHRDBFL, IRHANFLG, &
 & IRHPFLG, IRHFLG  
INTEGER(KIND=JPIM) :: JTD2M, ITD2M, &
 & ITDSTAT, ITDEVEN1, ITDEVEN2, ITDRDBFL, ITDANFLG, &
 & ITDPFLG, ITDFLG  
INTEGER(KIND=JPIM) :: JPT, IPT, &
 & IPTSTAT, IPTEVEN1, IPTEVEN2, IPTRDBFL, IPTANFLG, &
 & IPTPFLG, IPTFLG  
INTEGER(KIND=JPIM) :: JSDD, ISDD, &
 & ISDDSTAT, ISDDEVEN1, ISDDEVEN2, ISDDRDBFL, ISDDANFLG, &
 & ISDDPFLG, ISDDFLG  
INTEGER(KIND=JPIM) :: JSFF, ISFF, &
 & ISFFSTAT, ISFFEVEN1, ISFFEVEN2, ISFFRDBFL, ISFFANFLG, &
 & ISFFPFLG, ISFFFLG  
INTEGER(KIND=JPIM) :: IAUX, JALL, &
 & IAUSTAT, IAUEVEN1, IAUEVEN2, IAURDBFL, IAUANFLG, &
 & IAUPFLG, IAUFLG  , IAUVERCO  
INTEGER(KIND=JPIM) :: IDBRDFL

LOGICAL :: LLFAIL, LLREMOVE, LLSHIP, LLTEND, LLSHIPC, LLDONEBCH, LLMSLP, LLSTAP
LOGICAL :: LLCHSHIP
LOGICAL :: LLPs, LLT2M, LLTd2M, LLDD, LLFF, LLU10m, LLV10m, &
 & LLRH2m, LLPt, LLSD, LLSDD, LLSFF, LLZ  

CHARACTER (LEN = 8) :: CLSTID

REAL(KIND=JPRB) :: ZLAT, ZLON, &
 & ZPRES, ZTIMDIF, &
 & ZWEIGHT, ZPSTA, ZPMSL, &
 & ZVERCO, ZPRESAU, ZHEAUX, &
 & ZALT  ,&
 & ZUWIND, ZVWIND, ZSHIPU, ZSHIPV, ZANMOMH, ZBAROMH  
REAL(KIND=JPRB) :: ZDIR, ZDDPRES, ZDDPRL, &
 & ZDDPERR, ZDDOERR, ZDDFERR, ZDDRERR, &
 & ZDDFGER, ZDDFGCC1, ZDDFGCC2, &
 & ZDDFGDEP, ZDDANDEP  
REAL(KIND=JPRB) :: ZSPD, ZFFPRES, ZFFPRL, &
 & ZFFPERR, ZFFOERR, ZFFFERR, ZFFRERR, &
 & ZFFFGER, ZFFFGCC1, ZFFFGCC2, &
 & ZFFFGDEP, ZFFANDEP  
REAL(KIND=JPRB) :: ZU, ZUUPRES, ZUUPRL, &
 & ZUUPERR, ZUUOERR, ZUUFERR, ZUURERR, &
 & ZUUFGER, ZUUFGCC1, ZUUFGCC2, &
 & ZUUFGDEP, ZUUANDEP  
REAL(KIND=JPRB) :: ZV, ZVVPRES, ZVVPRL, &
 & ZVVPERR, ZVVOERR, ZVVFERR, ZVVRERR, &
 & ZVVFGER, ZVVFGCC1, ZVVFGCC2, &
 & ZVVFGDEP, ZVVANDEP  
REAL(KIND=JPRB) :: ZT2M, ZTTPRES, ZTTPRL, &
 & ZTTPERR, ZTTOERR, ZTTFERR, ZTTRERR, &
 & ZTTFGER, ZTTFGCC1, ZTTFGCC2, &
 & ZTTFGDEP, ZTTANDEP  
REAL(KIND=JPRB) :: ZTD2M, ZTDPRES, ZTDPRL, &
 & ZTDPERR, ZTDOERR, ZTDFERR, ZTDRERR, &
 & ZTDFGER, ZTDFGCC1, ZTDFGCC2, &
 & ZTDFGDEP, ZTDANDEP  
REAL(KIND=JPRB) :: ZRH2M, ZRHPRES, ZRHPRL, &
 & ZRHPERR, ZRHOERR, ZRHFERR, ZRHRERR, &
 & ZRHFGER, ZRHFGCC1, ZRHFGCC2, &
 & ZRHFGDEP, ZRHANDEP  
REAL(KIND=JPRB) :: ZPT, ZPTPRES, ZPTPRL, &
 & ZPTPERR, ZPTOERR, ZPTFERR, ZPTRERR, &
 & ZPTFGER, ZPTFGCC1, ZPTFGCC2, &
 & ZPTFGDEP, ZPTANDEP  
REAL(KIND=JPRB) :: ZZZ, ZZZPRES, ZZZPRL, &
 & ZZZPERR, ZZZOERR, ZZZFERR, ZZZRERR, &
 & ZZZFGER, ZZZFGCC1, ZZZFGCC2, &
 & ZZZFGDEP, ZZZANDEP  
REAL(KIND=JPRB) :: ZPS, ZPSPRES, ZPSPRL, &
 & ZPSPERR, ZPSOERR, ZPSFERR, ZPSRERR, &
 & ZPSFGER, ZPSFGCC1, ZPSFGCC2, &
 & ZPSFGDEP, ZPSANDEP, ZPSBIAS  
REAL(KIND=JPRB) :: ZSDD, ZSHPDIR, ZSDDPRES, ZSDDPRL, &
 & ZSDDPERR, ZSDDOERR, ZSDDFERR, ZSDDRERR, &
 & ZSDDFGER, ZSDDFGCC1, ZSDDFGCC2, &
 & ZSDDFGDEP, ZSDDANDEP  
REAL(KIND=JPRB) :: ZSFF, ZSHPSPD, ZSFFPRES, ZSFFPRL, &
 & ZSFFPERR, ZSFFOERR, ZSFFFERR, ZSFFRERR, &
 & ZSFFFGER, ZSFFFGCC1, ZSFFFGCC2, &
 & ZSFFFGDEP, ZSFFANDEP  
REAL(KIND=JPRB) :: ZAU, ZAUPRES, ZAUPRL, &
 & ZAUPERR, ZAUOERR, ZAUFERR, ZAURERR, &
 & ZAUFGER, ZAUFGCC1, ZAUFGCC2, &
 & ZAUFGDEP, ZAUANDEP  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "errstat.intfb.h"
#include "finoerr.intfb.h"
#include "getsete.intfb.h"
#include "obinstp.intfb.h"
#include "ppvafl.intfb.h"
#include "prlmchk.intfb.h"
#include "ptendcor.intfb.h"
#include "synopbe.intfb.h"
#include "z2picao.intfb.h"

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
#include "openmp_obs.h"
!*

!        1.             PRESET
!                       ------

IF (LHOOK) CALL DR_HOOK('SHIPIN',0,ZHOOK_HANDLE)
ZLAT   = ROBHDR(KSYNP,MDBLAT) * RDEGREES
ZLON   = ROBHDR(KSYNP,MDBLON) * RDEGREES
ZALT   = ROBHDR(KSYNP,MDBALT)
IALFLG = IBITS(INT(ROBHDR(KSYNP,MDBRFL)),NALFLBP+NFLGBP,NFLGOC)
IOBDATE= ROBHDR(KSYNP,MDBDAT)
IOBTIME= ROBHDR(KSYNP,MDBETM)
CLSTID = TRANSFER(ROBHDR(KSYNP,MDBSID), CLSTID)
ISTID  = 0
READ(CLSTID(1:8),'(A8)') ISTID
IREPST = ROBHDR(KSYNP,MDBRST)

!*
!-----------------------------------------------------------------------

!        2.             GENERAL OBSERVATION INFO.
!                       -------------------------

!*          2.1         DEFINE INSTRUMENT TYPE

INSTYP  = 0
CALL OBINSTP(ZLAT,ZLON,ISTID,&
 & KOBTYP,KCDTYP,&
 & INSTYP)  

!*          2.2         SET INSTRUMENT TYPE INTO OBSERVATION
!                       CHARACTERISTICS

ROBHDR(KSYNP,MDB_INSTTYPE_AT_HDR) = INSTYP

!*
!-----------------------------------------------------------------------

!        3.             PRELIMINARY CHECKS
!                       ------------------

!*          3.1         REPORT PARAMETERS

LLFAIL   = .FALSE.
LLREMOVE = .FALSE.
CALL PRLMCHK(KOBTYP,KCDTYP,ZLAT,ZLON,IOBDATE,IOBTIME,IREPST,ZTIMDIF,&
 & LDMESAGE,LLFAIL,LLREMOVE)  

!*          3.1.0       UPDATE REPORT STATUS

ROBHDR(KSYNP,MDBRST) = IREPST

!*          3.1.1       FAILED REPORT

IF(LLFAIL  ) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.2       REMOVED REPORT

IF(LLREMOVE) THEN
  IF(LTOTREM) GO TO 6
ENDIF

!*          3.1.2       ANEMOMETER/BAROMETER HEIGHT

ZANMOMH  = RSYNANMH
ZBAROMH  = RSYNBARH
LLCHSHIP = .FALSE.

!*          3.1.2.1     SOME DRIBUS COME AS SHIP REPORTS AND
!                       THEY SHOULD BE SEPARATED HERE.
!                       IF THE ST. ID. IS 5 DIGET NUMBER AND
!                       TYPE OF STATION EQ TO 0 (AUTOMATIC) THEN
!                       A SAFE BET IS THAT IT IS A DRIBU

ZANMOMH  = RSANMOMH
ZBAROMH  = RSBAROMH
LLCHSHIP = LCHSHIP
IF(LSHDRCHO)    THEN
  ITYPST = NMDI
  IF(ROBHDR(KSYNP,MDB_STATION_TYPE_AT_HDR) /= NMDI ) THEN
    ITYPST = ROBHDR(KSYNP,MDB_STATION_TYPE_AT_HDR)
  ENDIF

  READ(CLSTID,*,ERR=142,END=142) INSTID

  IF(INSTID > 0.AND.INSTID <= 99999)   THEN
    IF(ITYPST /= NMDI)                    THEN
      IF(ITYPST == 0)                       THEN
        ZANMOMH = RDANMOMH
        ZBAROMH = RDBAROMH
        IF(LSHDRCHOC) THEN
          LLCHSHIP= LCHDRIBU
        ENDIF
      ENDIF
    ENDIF
  ENDIF
ENDIF

!*          3.1.1.2     ASSIGN THE VALUE

142   CONTINUE
IF(LLCHSHIP)    THEN
  IF(ROBHDR(KSYNP,MDBHOANS) /=RMDI ) THEN
    ZANMOMH = ROBHDR(KSYNP,MDBHOANS)
  ENDIF
ENDIF

!*          3.1.1.3     REPLACE THE VALUE

ROBHDR(KSYNP,MDBHOANS) = ZANMOMH
ROBHDR(KSYNP,MDBHOBAS) = ZBAROMH

!*          3.2         BODY PARAMETERS

LLPs   = .FALSE.
LLT2M  = .FALSE.
LLTd2M = .FALSE.
LLDD   = .FALSE.
LLFF   = .FALSE.
LLU10m = .FALSE.
LLV10m = .FALSE.
LLRH2m = .FALSE.
LLPt   = .FALSE.
LLSD   = .FALSE.
LLSDD  = .FALSE.
LLSFF  = .FALSE.
LLZ    = .FALSE.
ALL_DATA_LOOP: DO JALL = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1

!*          3.2.1       WE ACCEPT: Ps, T2m, Td2m, DD, FF, U10m, V10m,
!                                  Rh2m, Pt, SDD, SFF AND Z

  IF(     ROBODY(JALL  ,MDBVNM) == NVNUMB( 47)) THEN
    LLPs    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 10)) THEN
    LLT2M   = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 11)) THEN
    LLTd2M  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 48)) THEN
    LLDD    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 49)) THEN
    LLFF    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 56)) THEN
    LLU10m  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 57)) THEN
    LLV10m  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB(  7)) THEN
    LLRH2m  = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 13)) THEN
    LLPt    = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 36)) THEN
    LLSDD   = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB( 37)) THEN
    LLSFF   = .TRUE.
    GO TO 322
  ELSEIF(ROBODY(JALL  ,MDBVNM) == NVNUMB(  3)) THEN
    LLZ     = .TRUE.
    GO TO 322
  ELSE
    IAUX = JALL
    IVNM = ROBODY(IAUX  ,MDBVNM)
!    WRITE(NULOUT,'('' SHIPIN; UNEXPECTED VARIABLE: '',I10)') IVNM

!*          3.2.1.0     SET/GET

    CALL GETSETE(IAUX ,&
     & ZAUPRES,ZAUPRL,ZAU ,&
     & ZAUFGER,ZAUFGCC1,ZAUFGCC2,&
     & ZAUPERR,ZAUOERR,ZAUFERR,ZAURERR,&
     & ZAUFGDEP,ZAUANDEP,&
     & IAUSTAT,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG,&
     & IAUFLG,IAUPFLG)  

    ZAUPRES = ROBODY(IAUX,MDBPPP)
    IAUVERCO= ROBODY(IAUX,MDBVCO)

!*          3.2.1.2     VARIABLE

    ZAU     = ROBODY(IAUX,MDBVAR)

!*          3.2.1.3     ERROR STATISTICS

!*          3.2.1.4     STATUS, EVENTS, FLAGS

    IAUSTAT = ICHSTAT_REJECT( IAUSTAT )

!*          3.2.1.5     DEPARTURES

!*          3.2.1.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IAUX    ,&
     & IVNM    ,IAUVERCO,ZAUPRES ,ZAUPRL  ,ZAU     ,&
     & ZAUOERR ,ZAUPERR ,ZAUFERR ,ZAURERR ,&
     & ZAUFGER ,ZAUFGCC1,ZAUFGCC2,&
     & ZAUFGDEP,ZAUANDEP,&
     & IAUSTAT ,IAUEVEN1,IAUEVEN2,IAURDBFL,IAUANFLG, &
     & IPRESCD                                       )  
  ENDIF

!*          3.2.2       CHECK IF BUFR MISSING IND. FOR:
!                       1. VERTICAL COORDINATE
!                       2. VARIABLE
!                       3. ?

  322 CONTINUE

!*          3.2.2.1     PRESSURE

  3221 CONTINUE
  IF(ROBODY(JALL  ,MDBPPP) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBPPP) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBPPP) <= RUPMLIM     )) GO TO 3222  
    ROBODY(JALL ,MDBPPP ) = RMDI
  ENDIF

!*          3.2.2.2     VARIABLE

  3222 CONTINUE
  IF(ROBODY(JALL  ,MDBVAR) /= RMDI ) THEN
    IF(.NOT.(ROBODY(JALL ,MDBVAR) >= RDWMLIM.AND.&
     & ROBODY(JALL ,MDBVAR) <= RUPMLIM     )) GO TO 3223  
    ROBODY(JALL ,MDBVAR ) = RMDI
  ENDIF

!*          3.2.2.3     ERROR STATISTICS

  3223 CONTINUE
  ROBODY(JALL  ,MDBOER) = RMDI
  ROBODY(JALL  ,MDBRER) = RMDI
  ROBODY(JALL  ,MDBPER) = RMDI
  ROBODY(JALL  ,MDBFGE) = RMDI
  ROBODY(JALL  ,MDBFOE) = RMDI

!*          3.2.3       CLOSE ALL DATA LOOP

ENDDO ALL_DATA_LOOP

!*          3.2.3       CHECK IF ALL PRESENT

IF(LLPs   .AND. &
 & LLT2M  .AND. &
 & LLTd2M .AND. &
 & LLDD   .AND. &
 & LLFF   .AND. &
 & LLU10m .AND. &
 & LLV10m .AND. &
 & LLRH2m .AND. &
 & LLPt   .AND. &
 & LLSDD  .AND. &
 & LLSFF  .AND. &
 & LLZ           ) GO TO 324  
WRITE(NULOUT,'('' SHIPIN; '')')
WRITE(NULOUT,'(''          LLPs   :'',L1)') LLPs
WRITE(NULOUT,'(''          LLT2M  :'',L1)') LLT2M
WRITE(NULOUT,'(''          LLTd2M :'',L1)') LLTd2M
WRITE(NULOUT,'(''          LLDD   :'',L1)') LLDD
WRITE(NULOUT,'(''          LLFF   :'',L1)') LLFF
WRITE(NULOUT,'(''          LLU10m :'',L1)') LLU10m
WRITE(NULOUT,'(''          LLV10m :'',L1)') LLV10m
WRITE(NULOUT,'(''          LLRH2m :'',L1)') LLRH2m
WRITE(NULOUT,'(''          LLPt   :'',L1)') LLPt
WRITE(NULOUT,'(''          LLSD   :'',L1)') LLSD
WRITE(NULOUT,'(''          LLSDD  :'',L1)') LLSDD
WRITE(NULOUT,'(''          LLSFF  :'',L1)') LLSFF
WRITE(NULOUT,'(''          LLZ    :'',L1)') LLZ

!*          3.2.4       ?

324 CONTINUE

!*
!-----------------------------------------------------------------------

!        4.             OBSERVATION BODY (OUTPUT TO ODB)
!                       --------------------------------

ZWEIGHT = 1.0_JPRB

!*          4.0         DECIDE ON VERTICAL COORDINATE; 
!                       CHECK IF PRESSURE PRESENT; WIND AND
!                       TEMPERATURE ARE DONE ANYWAY BUT ASSIGN
!                       VERTICAL COOORDINATE AS HEIGHT SET TO 0.

!*          4.0.0       GET PS/Pt/DD/FF/SDD/SFF

ZPRES    = RMDI
IPFLG    = 0
IPRESCD  = NMDI
ZPSTA    = RMDI
ZPMSL    = RMDI
LLMSLP   = .FALSE.
LLSTAP   = .FALSE.
IPSTAFLG = 0
IPMSLFLG = 0
Ps0_LOOP: DO JPs = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPs  ,MDBVNM) == NVNUMB( 47)) THEN
    IPS     = JPs
    IPCODE = ROBODY(IPS  ,MDBSYPC)
    IF(IPCODE == NPRESCD( 1)) THEN
      IF(LLMSLP)                 THEN
        WRITE(NULOUT,'('' SHIPIN; MSL PRESSURE ALREADY FOUND '')')
        CALL ABOR1(' SHIPIN : ')
      ELSE
        LLMSLP   = .TRUE.
        ZPMSL    = ROBODY(IPs  ,MDBVAR)
        IPMSLFLG = IBITS(INT(ROBODY(IPs  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
        IDBRDFL  = ROBODY(IPS,MDBRDFL)
        CALL PPVAFL(IPMSLFLG,IPMSLFLG,IDBRDFL)
        ROBODY(IPS,MDBRDFL) = IDBRDFL
      ENDIF
    ELSEIF(IPCODE == NPRESCD( 2)) THEN
      IF(LLSTAP)                 THEN
        WRITE(NULOUT,'('' SHIPIN; STATION LEVEL PRESSURE ALREADY FOUND '')')
        CALL ABOR1(' SHIPIN : ')
      ELSE
        LLSTAP   = .TRUE.
        ZPSTA    = ROBODY(IPS  ,MDBVAR)
        IPSTAFLG = IBITS(INT(ROBODY(IPs  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
        IDBRDFL  = ROBODY(IPS,MDBRDFL)
        CALL PPVAFL(IPSTAFLG,IPSTAFLG,IDBRDFL)
        ROBODY(IPS,MDBRDFL) = IDBRDFL
      ENDIF
    ELSE
      WRITE(NULOUT,'('' SHIPIN; UNKNOWN PRESSURE CODE: '',I10)') IPCODE
      CALL ABOR1(' SHIPIN : ')
    ENDIF
  ENDIF
ENDDO Ps0_LOOP

IF(ZPMSL /= RMDI ) THEN
  ZPRES = ZPMSL
  IPFLG = IPMSLFLG
ENDIF

ZPT   = RMDI
IPTFLG= 0
PT0_LOOP: DO JPT = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPT  ,MDBVNM) == NVNUMB( 13)) THEN
    ZPT     = ROBODY(JPT  ,MDBVAR)
    IPTFLG  = IBITS(INT(ROBODY(JPT  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
    IPRESCD = ROBODY(JPT  ,MDBSYPC)
  ENDIF
ENDDO PT0_LOOP

ZDIR  = RMDI
IDDFLG= 0
DD0_LOOP: DO JDD = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JDD  ,MDBVNM) == NVNUMB( 48)) THEN
    ZDIR    = ROBODY(JDD  ,MDBVAR)
    IDDFLG  = IBITS(INT(ROBODY(JDD  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
    IPRESCD = ROBODY(JDD  ,MDBSYPC)
  ENDIF
ENDDO DD0_LOOP

ZSPD  = RMDI
IFFFLG= 0
FF0_LOOP: DO JFF = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JFF  ,MDBVNM) == NVNUMB( 49)) THEN
    ZSPD    = ROBODY(JFF  ,MDBVAR)
    IFFFLG  = IBITS(INT(ROBODY(JFF  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
    IPRESCD = ROBODY(JFF  ,MDBSYPC)
  ENDIF
ENDDO FF0_LOOP

ZSHPDIR=RMDI
ISDDFLG= 0
SDD0_LOOP: DO JSDD = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JSDD  ,MDBVNM) == NVNUMB( 36)) THEN
    ZSHPDIR = ROBODY(JSDD  ,MDBVAR)
    ISDDFLG = IBITS(INT(ROBODY(JSDD  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
    IPRESCD = ROBODY(JSDD  ,MDBSYPC)
  ENDIF
ENDDO SDD0_LOOP

ZSHPSPD=RMDI
ISFFFLG= 0
SFF0_LOOP: DO JSFF = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JSFF  ,MDBVNM) == NVNUMB( 37)) THEN
    ZSHPSPD = ROBODY(JSFF  ,MDBVAR)
    ISFFFLG = IBITS(INT(ROBODY(JSFF  ,MDBRDFL)),NCMBVFBP+NCMBFLBP,NCMBFLOC)
    IPRESCD = ROBODY(JSFF  ,MDBSYPC)
  ENDIF
ENDDO SFF0_LOOP

!*          4.0.1       VERTICAL COORDINATE

ZVERCO = ZPRES
IVECFL = IPFLG
IVERCO = NPRESVC
IF(ZVERCO == RMDI)            THEN
  ZVERCO = 0.0_JPRB
  IVECFL = 0
  IVERCO = NHEIGVC
ENDIF

!*          4.0.2       IF HEIGHT VERTICAL COORDINATE GET A PRESSURE
!                       FOR OBS. ERROR CALCULATIONS

ZPRESAU = ZPRES
IF(IVERCO == NHEIGVC)         THEN
  ZHEAUX  = 0.0_JPRB
  CALL Z2PICAO(ZHEAUX,ZPRESAU)
  ZPRESAU = ZPRESAU * 100._JPRB
ENDIF

!*          4.1         PRESSURE TENDENCY CORRECTION

IF(LTENDCOR)      THEN
  LLSHIP  = .TRUE.
  LLTEND  = .FALSE.
  LLSHIPC = .FALSE.
  IF(ZPRES == RMDI)         GO TO 420

!*          4.1.1       WIND COMPONENTS AND FLAGS

  ZUWIND  = RMDI
  IUWINDF = 0
  ZVWIND  = RMDI
  IVWINDF = 0
  IF(ZDIR /= RMDI.AND.ZSPD /= RMDI     )        THEN
    ZUWIND  = UCOM(ZDIR,ZSPD)
    IUWINDF = MAX(IDDFLG,IFFFLG)
    ZVWIND  = VCOM(ZDIR,ZSPD)
    IVWINDF = MAX(IDDFLG,IFFFLG)
  ENDIF

!*          4.1.2       SHIP COMPONENTS AND FLAGS

  ZSHIPU  = RMDI
  IUSHIPF = 0
  ZSHIPV  = RMDI
  IVSHIPF = 0
  IF(ZSHPDIR /= RMDI.AND.ZSHPSPD /= RMDI     )        THEN
    ZSHIPU  = UCOM(ZSHPDIR,ZSHPSPD)
    IUSHIPF = MAX(ISDDFLG,ISFFFLG)
    ZSHIPV  = VCOM(ZSHPDIR,ZSHPSPD)
    IVSHIPF = MAX(ISDDFLG,ISFFFLG)
  ENDIF

!*          4.1.3       DO CORRECTION
  
  CALL PTENDCOR(LLSHIP,LLTEND,LLSHIPC,ZLAT,ZLON,ZTIMDIF,&
   & ZSHIPU,IUSHIPF,ZSHIPV,IVSHIPF,&
   & ZPRES,ZPT,IPTFLG,&
   & ZUWIND,IUWINDF,ZVWIND,IVWINDF           )  

!*          4.1.4       REDIFINE THE PERSISTENCE ERROR WEIGHT

  IF(LLTEND)      THEN
    IF(.NOT.LLSHIPC)     THEN
      ZWEIGHT = ZWEIGHT / RREDPEW2
    ELSE
      ZWEIGHT = ZWEIGHT / RREDPEW1
    ENDIF
  ENDIF

!*          4.1.5       RESTORE CORRECTED PRESSURE

  ZPRESAU = ZPRES
  ZVERCO  = ZPRES
ENDIF

!*          4.2         LOOK FOR WIND DIRECTION/SPEED DATA

420 CONTINUE
ZDIR   = RMDI
ZSPD   = RMDI
IDDFLG = 0
IFFFLG = 0
ZDDPRES= RMDI
ZFFPRES= RMDI
IDDPFLG= 0
IFFPFLG= 0
DDFF_LOOP: DO JDDFF = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1

!*          4.2.1       WIND DIRECTION

  IF(ROBODY(JDDFF,MDBVNM) == NVNUMB( 48)) THEN
    IDD     = JDDFF
    IVNM    = NVNUMB( 48)

!*          4.2.1.0     SET/GET

    CALL GETSETE(IDD,&
     & ZDDPRES,ZDDPRL,ZDIR,&
     & ZDDFGER,ZDDFGCC1,ZDDFGCC2,&
     & ZDDPERR,ZDDOERR,ZDDFERR,ZDDRERR,&
     & ZDDFGDEP,ZDDANDEP,&
     & IDDSTAT,IDDEVEN1,IDDEVEN2,IDDRDBFL,IDDANFLG,&
     & IDDFLG,IDDPFLG)  

    ZDDPRES  = ZVERCO

    IDDPFLG  = IVECFL

!*          4.2.1.2     VARIABLE

    IF(ROBODY(IDD  ,MDBVAR) /= RMDI )       THEN
      ZDIR   = ROBODY(IDD  ,MDBVAR)
      IDDFLG = IBITS(IDDRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
    ENDIF

!*          4.2.1.3     ERROR STATISTICS

!*          4.2.1.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 48,KCDSQNO,KOBSQNO) == 0)  THEN
      IDDSTAT = ICHSTAT_REJECT( IDDSTAT )
    ENDIF
    CALL PPVAFL(IDDPFLG,IDDFLG,IDDRDBFL)

!*          4.2.1.5     DEPARTURES

!*          4.2.1.6     UPDATE BODY ENTRY
 
    CALL SYNOPBE(IDD     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZDDPRL  ,ZDIR    ,&
     & ZDDOERR ,ZDDPERR ,ZDDFERR ,ZDDRERR ,&
     & ZDDFGER ,ZDDFGCC1,ZDDFGCC2,&
     & ZDDFGDEP,ZDDANDEP,&
     & IDDSTAT ,IDDEVEN1,IDDEVEN2,IDDRDBFL,IDDANFLG,&
     & IPRESCD                                       )  

!*          4.2.2       WIND FORCE

  ELSEIF(ROBODY(JDDFF,MDBVNM) == NVNUMB( 49)) THEN
    IFF     = JDDFF
    IVNM    = NVNUMB( 49)

!*          4.2.2.0     SET/GET

    CALL GETSETE(IFF,&
     & ZFFPRES,ZFFPRL,ZSPD,&
     & ZFFFGER,ZFFFGCC1,ZFFFGCC2,&
     & ZFFPERR,ZFFOERR,ZFFFERR,ZFFRERR,&
     & ZFFFGDEP,ZFFANDEP,&
     & IFFSTAT,IFFEVEN1,IFFEVEN2,IFFRDBFL,IFFANFLG,&
     & IFFFLG,IFFPFLG)  

    ZFFPRES  = ZVERCO

    IFFPFLG  = IVECFL

!*          4.2.2.2     VARIABLE

    IF(ROBODY(IFF  ,MDBVAR) /= RMDI )        THEN
      ZSPD   = ROBODY(IFF  ,MDBVAR)
      IFFFLG = IBITS(IFFRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
    ENDIF

!*          4.2.2.3     ERROR STATISTICS

    IF(LERRPERS)           THEN
      ZFFPERR  = 1.0_JPRB ! Rough first estimate of persistence error
    ENDIF
    ZFFOERR  = 2.0_JPRB ! Rough first estimate of obs. error =2 m/s
    CALL FINOERR(ZFFPERR,ZFFOERR,ZFFFERR)

!*          4.2.2.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 49,KCDSQNO,KOBSQNO) == 0)          THEN
      IFFSTAT = ICHSTAT_REJECT( IFFSTAT )
    ENDIF
    CALL PPVAFL(IFFPFLG,IFFFLG,IFFRDBFL)

!*          4.2.2.5     DEPARTURES

!*          4.2.2.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IFF     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZFFPRL  ,ZSPD    ,&
     & ZFFOERR ,ZFFPERR ,ZFFFERR ,ZFFRERR ,&
     & ZFFFGER ,ZFFFGCC1,ZFFFGCC2,&
     & ZFFFGDEP,ZFFANDEP,&
     & IFFSTAT ,IFFEVEN1,IFFEVEN2,IFFRDBFL,IFFANFLG,&
     & IPRESCD                                       )  

  ENDIF
ENDDO DDFF_LOOP

!*          4.2.3       CHECK DD, FF AND PRESSURE

!*          4.2.4       10m(?) U COMPONENT

U10M_LOOP: DO JU10M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JU10M,MDBVNM) == NVNUMB( 56)) THEN
    IU10M = JU10M
    IVNM  = NVNUMB( 56)

!*          4.2.4.0     SET/GET

    CALL GETSETE(IU10M,&
     & ZUUPRES,ZUUPRL,ZU  ,&
     & ZUUFGER,ZUUFGCC1,ZUUFGCC2,&
     & ZUUPERR,ZUUOERR,ZUUFERR,ZUURERR,&
     & ZUUFGDEP,ZUUANDEP,&
     & IUUSTAT,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
     & IUUFLG,IUUPFLG)  

    ZUUPRES  = ZVERCO

    IUUPFLG  = IVECFL

!*          4.2.4.2     VARIABLE

    IF(ZDIR /= RMDI.AND.ZSPD /= RMDI    )        THEN
      ZU     = UCOM(ZDIR,ZSPD)
      IUUFLG = MAX(IDDFLG,IFFFLG)
    ELSE
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF

!*          4.2.4.3     ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IUUSTAT,ZPRESAU,ZUUPRL,&
     & ZUUPERR,ZUUOERR,ZUUFERR,ZWEIGHT)  

!*          4.2.4.4     STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 56,KCDSQNO,KOBSQNO) == 0)          THEN
      IUUSTAT = ICHSTAT_REJECT( IUUSTAT )
    ENDIF
    CALL PPVAFL(IUUPFLG,IUUFLG,IUURDBFL)

!*          4.2.4.5     DEPARTURES

!*          4.2.4.6     UPDATE BODY ENTRY

    CALL SYNOPBE(IU10M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZUUPRL  ,ZU      ,&
     & ZUUOERR ,ZUUPERR ,ZUUFERR ,ZUURERR ,&
     & ZUUFGER ,ZUUFGCC1,ZUUFGCC2,&
     & ZUUFGDEP,ZUUANDEP,&
     & IUUSTAT ,IUUEVEN1,IUUEVEN2,IUURDBFL,IUUANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO U10M_LOOP

!*          4.2.5       10m(?) V COMPONENT

V10M_LOOP: DO JV10M    = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JV10M,MDBVNM) == NVNUMB( 57)) THEN
    IV10M = JV10M
    IVNM  = NVNUMB( 57)

!*          4.2.5.0     SET/GET

    CALL GETSETE(IV10M,&
     & ZVVPRES,ZVVPRL,ZV  ,&
     & ZVVFGER,ZVVFGCC1,ZVVFGCC2,&
     & ZVVPERR,ZVVOERR,ZVVFERR,ZVVRERR,&
     & ZVVFGDEP,ZVVANDEP,&
     & IVVSTAT,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
     & IVVFLG,IVVPFLG)  

    ZVVPRES  = ZVERCO

    IVVPFLG  = IVECFL

!*          4.2.5.2     VARIABLE

    IF(ZDIR /= RMDI.AND.ZSPD /= RMDI    )        THEN
      ZV      = VCOM(ZDIR,ZSPD)
      IVVFLG  = MAX(IDDFLG,IFFFLG)
    ELSE
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF

!*          4.2.5.3     ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IVVSTAT,ZPRESAU,ZVVPRL,&
     & ZVVPERR,ZVVOERR,ZVVFERR,ZWEIGHT)  

!*          4.2.5.4     STATUS, EVENTS, FLAGS
  
    IF(NMKCMVSE( 57,KCDSQNO,KOBSQNO) == 0)          THEN
      IVVSTAT = ICHSTAT_REJECT( IVVSTAT )
    ENDIF
    CALL PPVAFL(IVVPFLG,IVVFLG,IVVRDBFL)

!*          4.2.5.5     DEPARTURES

!*          4.2.5.5     UPDATE BODY ENTRY

    CALL SYNOPBE(IV10M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZVVPRL  ,ZV      ,&
     & ZVVOERR ,ZVVPERR ,ZVVFERR ,ZVVRERR ,&
     & ZVVFGER ,ZVVFGCC1,ZVVFGCC2,&
     & ZVVFGDEP,ZVVANDEP,&
     & IVVSTAT ,IVVEVEN1,IVVEVEN2,IVVRDBFL,IVVANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO V10M_LOOP

!*          4.3         2m(?) TEMPERATURE

ZT2M     = RMDI
ITTFLG   = 0
ITTPFLG  = IVECFL
T2M_LOOP: DO JT2M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JT2M ,MDBVNM) == NVNUMB( 10)) THEN
    IT2M = JT2M
    IVNM = NVNUMB( 10)

!*          4.3.0       SET/GET

    CALL GETSETE(IT2M,&
     & ZTTPRES,ZTTPRL,ZT2M,&
     & ZTTFGER,ZTTFGCC1,ZTTFGCC2,&
     & ZTTPERR,ZTTOERR,ZTTFERR,ZTTRERR,&
     & ZTTFGDEP,ZTTANDEP,&
     & ITTSTAT,ITTEVEN1,ITTEVEN2,ITTRDBFL,ITTANFLG,&
     & ITTFLG,ITTPFLG)  

    ZTTPRES  = ZVERCO

    ITTPFLG  = IVECFL

!*          4.3.2       VARIABLE

    IF(ROBODY(IT2M ,MDBVAR) /= RMDI ) THEN
      ZT2M  = ROBODY(IT2M ,MDBVAR)
      ITTFLG= IBITS(ITTRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      ITTSTAT = ICHSTAT_REJECT( ITTSTAT )
    ENDIF

!*          4.3.3       ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,ITTSTAT,ZPRESAU,ZTTPRL,&
     & ZTTPERR,ZTTOERR,ZTTFERR,ZWEIGHT)  

!*          4.3.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 10,KCDSQNO,KOBSQNO) == 0)          THEN
      ITTSTAT = ICHSTAT_REJECT( ITTSTAT )
    ENDIF
    CALL PPVAFL(ITTPFLG,ITTFLG,ITTRDBFL)

!*          4.3.5       DEPARTURES

!*          4.3.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IT2M    ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZTTPRL  ,ZT2M    ,&
     & ZTTOERR ,ZTTPERR ,ZTTFERR ,ZTTRERR ,&
     & ZTTFGER ,ZTTFGCC1,ZTTFGCC2,&
     & ZTTFGDEP,ZTTANDEP,&
     & ITTSTAT ,ITTEVEN1,ITTEVEN2,ITTRDBFL,ITTANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO T2M_LOOP

!*          4.4         DEW POINT

ZTD2M  = RMDI
ITDFLG = 0
ITDPFLG= IVECFL
DEWPOINT_LOOP: DO JTD2M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JTD2M,MDBVNM) == NVNUMB( 11)) THEN
    ITD2M = JTD2M
    IVNM = NVNUMB( 11)

!*          4.4.0       SET/GET

    CALL GETSETE(ITD2M,&
     & ZTDPRES,ZTDPRL,ZTD2M,&
     & ZTDFGER,ZTDFGCC1,ZTDFGCC2,&
     & ZTDPERR,ZTDOERR,ZTDFERR,ZTDRERR,&
     & ZTDFGDEP,ZTDANDEP,&
     & ITDSTAT,ITDEVEN1,ITDEVEN2,ITDRDBFL,ITDANFLG,&
     & ITDFLG,ITDPFLG)  

    ZTDPRES  = ZVERCO

    ITDPFLG  = IVECFL

!*          4.4.2       VARIABLE

    IF(ROBODY(ITD2M,MDBVAR) /= RMDI ) THEN
      ZTD2M = ROBODY(ITD2M,MDBVAR)
      ITDFLG= IBITS(ITDRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      ITDSTAT = ICHSTAT_REJECT( ITDSTAT )
    ENDIF

!*          4.4.3       ERROR STATISTICS

!*          4.4.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 11,KCDSQNO,KOBSQNO) == 0)          THEN
      ITDSTAT = ICHSTAT_REJECT( ITDSTAT )
    ENDIF
    CALL PPVAFL(ITDPFLG,ITDFLG,ITDRDBFL)

!*          4.4.5       DEPARTURES

!*          4.4.6       UPDATE BODY ENTRY

    CALL SYNOPBE(ITD2M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZTDPRL  ,ZTD2M   ,&
     & ZTDOERR ,ZTDPERR ,ZTDFERR ,ZTDRERR ,&
     & ZTDFGER ,ZTDFGCC1,ZTDFGCC2,&
     & ZTDFGDEP,ZTDANDEP,&
     & ITDSTAT ,ITDEVEN1,ITDEVEN2,ITDRDBFL,ITDANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO DEWPOINT_LOOP

!*          4.5         RELATIVE HUMIDITY

REL_HUMIDITY_DATA_LOOP: DO JRH2M = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JRH2M,MDBVNM) == NVNUMB(  7)) THEN
    IRH2M = JRH2M
    IVNM  = NVNUMB(  7)

!*          4.5.0       SET/GET

    CALL GETSETE(IRH2M,&
     & ZRHPRES,ZRHPRL,ZRH2M,&
     & ZRHFGER,ZRHFGCC1,ZRHFGCC2,&
     & ZRHPERR,ZRHOERR,ZRHFERR,ZRHRERR,&
     & ZRHFGDEP,ZRHANDEP,&
     & IRHSTAT,IRHEVEN1,IRHEVEN2,IRHRDBFL,IRHANFLG,&
     & IRHFLG,IRHPFLG)  

    ZRHPRES  = ZVERCO

    IRHPFLG  = IVECFL

!*          4.5.2       VARIABLE

    IF( ZT2M /= RMDI .AND. ZTD2M /= RMDI ) THEN
      IRHFLG   = MAX(ITTFLG,ITDFLG)
      IF(IRHFLG < NFLLIMOP(2))  THEN
        LLDONEBCH= .FALSE.
        ZRH2M    = FOEEWMO( ZTD2M ) / FOEEWMO( ZT2M )

!*          4.5.3       ERROR STATISTICS

!*          4.5.3.2     OBSERVATION ERROR

        IF(LRHERRMO)       THEN
          ZRHOERR = -0.0015_JPRB * ZT2M + 0.54_JPRB
          ZRHOERR = MIN(0.18_JPRB,MAX(0.06_JPRB,ZRHOERR))
        ELSE
          ZRHOERR = RHERR1
          IF(ZRH2M < RRHLIM )  ZRHOERR = RHERR2
          IF(ZT2M  < RRHTLIM)  ZRHOERR = RHERR3
        ENDIF

!*          4.5.3.3     FINAL OBSERVATION ERROR

        ZRHFERR = ZRHOERR

!*          4.5.3.4     COMBINED FLAG TO HI

      ELSE
        ZRH2M   = RMDI
        IRHSTAT = ICHSTAT_REJECT( IRHSTAT )
      ENDIF

!*          4.5.3.5     NO VARIABLE

    ELSE
      IRHSTAT = ICHSTAT_REJECT( IRHSTAT )
    ENDIF

!*          4.5.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  7,KCDSQNO,KOBSQNO) == 0)          THEN
      IRHSTAT = ICHSTAT_REJECT( IRHSTAT )
    ENDIF
    CALL PPVAFL(IRHPFLG,IRHFLG,IRHRDBFL)

!*          4.4.5       DEPARTURES

!*          4.5.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IRH2M   ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZRHPRL  ,ZRH2M   ,&
     & ZRHOERR ,ZRHPERR ,ZRHFERR ,ZRHRERR ,&
     & ZRHFGER ,ZRHFGCC1,ZRHFGCC2,&
     & ZRHFGDEP,ZRHANDEP,&
     & IRHSTAT ,IRHEVEN1,IRHEVEN2,IRHRDBFL,IRHANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO REL_HUMIDITY_DATA_LOOP

!*          4.6         PRESSURE TENDENCY

PRESSURE_TENDENCY_LOOP: DO JPT = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPT,MDBVNM) == NVNUMB( 13)) THEN
    IPT   = JPT
    IVNM  = NVNUMB( 13)

!*          4.6.0       SET/GET

    CALL GETSETE(IPT,&
     & ZPTPRES,ZPTPRL,ZPT ,&
     & ZPTFGER,ZPTFGCC1,ZPTFGCC2,&
     & ZPTPERR,ZPTOERR,ZPTFERR,ZPTRERR,&
     & ZPTFGDEP,ZPTANDEP,&
     & IPTSTAT,IPTEVEN1,IPTEVEN2,IPTRDBFL,IPTANFLG,&
     & IPTFLG,IPTPFLG)  

    ZPTPRES  = ZVERCO

    IPTPFLG  = IVECFL

!*          4.6.2       VARIABLE

    IF(ROBODY(IPT,MDBVAR) /= RMDI ) THEN
      ZPT   = ROBODY(IPT,MDBVAR)
      IPTFLG= IBITS(IPTRDBFL,NCMBVFBP+NCMBFLBP,NCMBFLOC)
    ELSE
      IPTSTAT = ICHSTAT_REJECT( IPTSTAT )
    ENDIF

!*          4.6.3       ERROR STATISTICS

!*          4.6.4       STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 13,KCDSQNO,KOBSQNO) == 0)          THEN
      IPTSTAT = ICHSTAT_REJECT( IPTSTAT )
    ENDIF
    CALL PPVAFL(IPTPFLG,IPTFLG,IPTRDBFL)

!*          4.6.5       DEPARTURES

!*          4.6.6       UPDATE BODY ENTRY

    CALL SYNOPBE(IPT     ,&
     & IVNM    ,IVERCO  ,ZVERCO  ,ZPTPRL  ,ZPT     ,&
     & ZPTOERR ,ZPTPERR ,ZPTFERR ,ZPTRERR ,&
     & ZPTFGER ,ZPTFGCC1,ZPTFGCC2,&
     & ZPTFGDEP,ZPTANDEP,&
     & IPTSTAT ,IPTEVEN1,IPTEVEN2,IPTRDBFL,IPTANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO PRESSURE_TENDENCY_LOOP

!*          4.10        GEOPOTENTIAL

Z_LOOP: DO JZ = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JZ   ,MDBVNM) == NVNUMB(  3)) THEN
    IZ   = JZ
    IVNM = NVNUMB(  3)

!*          4.10.0      SET/GET

    CALL GETSETE(IZ ,&
     & ZZZPRES,ZZZPRL,ZZZ ,&
     & ZZZFGER,ZZZFGCC1,ZZZFGCC2,&
     & ZZZPERR,ZZZOERR,ZZZFERR,ZZZRERR,&
     & ZZZFGDEP,ZZZANDEP,&
     & IZZSTAT,IZZEVEN1,IZZEVEN2,IZZRDBFL,IZZANFLG,&
     & IZZFLG,IZZPFLG)  

    ZZZPRES  = ZPRES
    ZZZ      = 0.0_JPRB
    IZZFLG   = 0
    IZZPFLG  = IPFLG

!*          4.10.2      VARIABLE

    IF(ZZZPRES == RMDI ) THEN
      IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
    ENDIF

!*          4.10.3      ERROR STATISTICS

    CALL ERRSTAT(KOBTYP,KOBSQNO,KCDTYP,KCDSQNO,ZLAT,ZLON,ZTIMDIF,&
     & IVNM,IZZSTAT,ZPRESAU,ZZZPRL,&
     & ZZZPERR,ZZZOERR,ZZZFERR,ZWEIGHT)  

!*          4.10.3.4    STATUS, EVENTS, FLAGS

    IF(NMKCMVSE(  3,KCDSQNO,KOBSQNO) == 0)          THEN
      IZZSTAT = ICHSTAT_REJECT( IZZSTAT )
    ENDIF
    CALL PPVAFL(IZZPFLG,IZZFLG,IZZRDBFL)

!*          4.10.3.5    DEPARTURES

!*          4.10.3.6    UPDATE BODY ENTRY

    CALL SYNOPBE(IZ      ,&
     & IVNM    ,NPRESVC ,ZZZPRES ,ZZZPRL  ,ZZZ     ,&
     & ZZZOERR ,ZZZPERR ,ZZZFERR ,ZZZRERR ,&
     & ZZZFGER ,ZZZFGCC1,ZZZFGCC2,&
     & ZZZFGDEP,ZZZANDEP,&
     & IZZSTAT ,IZZEVEN1,IZZEVEN2,IZZRDBFL,IZZANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO Z_LOOP

!*          4.11        PRESSURE

4110 CONTINUE
Ps_LOOP: DO JPs = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JPs  ,MDBVNM) == NVNUMB( 47)) THEN
    IPS    = JPs
    IVNM   = NVNUMB( 47)
    IPCODE = ROBODY(IPS  ,MDBSYPC)

!*          4.11.0      SET/GET

    CALL GETSETE(IPS,&
     & ZPSPRES,ZPSPRL,ZPS ,&
     & ZPSFGER,ZPSFGCC1,ZPSFGCC2,&
     & ZPSPERR,ZPSOERR,ZPSFERR,ZPSRERR,&
     & ZPSFGDEP,ZPSANDEP,&
     & IPSSTAT,IPSEVEN1,IPSEVEN2,IPSRDBFL,IPSANFLG,&
     & IPSFLG,IPSPFLG)  

    IF(IPCODE == NPRESCD( 1)) THEN
      ZPS      = ROBODY(IPS,MDBVAR)
      ZPSPRES  = 0.0_JPRB
    ENDIF

!*          4.11.2      VARIABLE

    IF(ZPS /= RMDI ) THEN
      IF(IPCODE /= NPRESCD( 1))   THEN
        IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
        IPSEVEN1= ICHEV1( IPSEVEN1,ND1EVENT(17))
      ENDIF

!*          4.11.2.0    BIAS CORRECTION

      IF(L_SHIP_PS_BIAS) THEN
        ZPSBIAS = ROBODY(IPS,MDBTORB)
        IF(ZPSBIAS /= RMDI) THEN
          ZPS = ZPS - ZPSBIAS
          IPSEVEN2 = ICHEV2( IPSEVEN2,ND2EVENT( 1,KOBTYP))
        ENDIF
      ENDIF
    ELSE
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF

!*          4.11.3      ERROR STATISTICS

    IF(IPCODE == NPRESCD( 1))   THEN

!*          4.11.3.1    PERSISTENCE ERROR

      IF(LERRPERS)           THEN
        IF(ZZZPERR /= RMDI)    THEN
          ZPSPERR  = ZZZPERR*RPAPM
        ELSE
          IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
        ENDIF
      ENDIF

!*          4.11.3.2    OBSERVATION ERROR

      IF(ZZZOERR /= RMDI)    THEN
        ZPSOERR  = ZZZOERR*RPAPM
      ELSE
        IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
      ENDIF

!*          4.11.3.3    PRESSURE FINAL OBSERVATION ERROR

      CALL FINOERR(ZPSPERR,ZPSOERR,ZPSFERR)
    ENDIF

!*          4.11.4      STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 47,KCDSQNO,KOBSQNO) == 0)          THEN
      IPSSTAT = ICHSTAT_REJECT( IPSSTAT )
    ENDIF
    CALL PPVAFL(IPSPFLG,IPSFLG,IPSRDBFL)

!*          4.11.5      DEPARTURES

!*          4.11.6      UPDATE BODY ENTRY

    CALL SYNOPBE(IPS     ,&
     & IVNM    ,NHEIGVC ,ZPSPRES ,ZPSPRL  ,ZPS     ,&
     & ZPSOERR ,ZPSPERR ,ZPSFERR ,ZPSRERR ,&
     & ZPSFGER ,ZPSFGCC1,ZPSFGCC2,&
     & ZPSFGDEP,ZPSANDEP,&
     & IPSSTAT ,IPSEVEN1,IPSEVEN2,IPSRDBFL,IPSANFLG,&
     & IPRESCD                                       )  
  ENDIF
ENDDO Ps_LOOP

!*          4.12        SHIP DIRECTION

SDD_LOOP: DO JSDD  = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JSDD ,MDBVNM) == NVNUMB( 36)) THEN
    ISDD  = JSDD
    IVNM  = NVNUMB( 36)

!*          4.12.0      SET/GET

    CALL GETSETE(ISDD,&
     & ZSDDPRES,ZSDDPRL,ZSDD   ,&
     & ZSDDFGER,ZSDDFGCC1,ZSDDFGCC2,&
     & ZSDDPERR,ZSDDOERR,ZSDDFERR,ZSDDRERR,&
     & ZSDDFGDEP,ZSDDANDEP,&
     & ISDDSTAT,ISDDEVEN1,ISDDEVEN2,ISDDRDBFL,ISDDANFLG,&
     & ISDDFLG,ISDDPFLG)  

    ZSDDPRES  = ZVERCO

    ZSDD      = ZSHPDIR

    ISDDPFLG  = IVECFL

!*          4.12.2      VARIABLE

    IF(ZSDD == RMDI )        THEN
      ISDDSTAT= ICHSTAT_REJECT( ISDDSTAT )
    ENDIF

!*          4.12.3      ERROR STATISTICS

!*          4.12.4      STATUS, EVENTS, FLAGS

    IF(NMKCMVSE( 36,KCDSQNO,KOBSQNO) == 0)          THEN
      ISDDSTAT= ICHSTAT_REJECT( ISDDSTAT )
    ENDIF
    CALL PPVAFL(ISDDPFLG,ISDDFLG,ISDDRDBFL)

!*          4.12.5      DEPARTURES

!*          4.12.6      UPDATE BODY ENTRY

    CALL SYNOPBE(ISDD     ,&
     & IVNM     ,IVERCO   ,ZVERCO   ,ZSDDPRL  ,ZSDD    ,&
     & ZSDDOERR ,ZSDDPERR ,ZSDDFERR ,ZSDDRERR ,&
     & ZSDDFGER ,ZSDDFGCC1,ZSDDFGCC2,&
     & ZSDDFGDEP,ZSDDANDEP,&
     & ISDDSTAT ,ISDDEVEN1,ISDDEVEN2 ,IUURDBFL,IUUANFLG,&
     & IPRESCD                                           )  

  ENDIF
ENDDO SDD_LOOP

!*          4.13        SHIP SPEED

SFF_LOOP: DO JSFF    = MLNKH2B(KSYNP),MLNKH2B(KSYNP+1)-1
  IF(ROBODY(JSFF ,MDBVNM) == NVNUMB( 37)) THEN
    ISFF  = JSFF
    IVNM  = NVNUMB( 37)
 
!*          4.13.0      SET/GET

    CALL GETSETE(ISFF,&
     & ZSFFPRES,ZSFFPRL,ZSFF     ,&
     & ZSFFFGER,ZSFFFGCC1,ZSFFFGCC2,&
     & ZSFFPERR,ZSFFOERR,ZSFFFERR,ZSFFRERR,&
     & ZSFFFGDEP,ZSFFANDEP,&
     & ISFFSTAT,ISFFEVEN1,ISFFEVEN2,ISFFRDBFL,ISFFANFLG,&
     & ISFFFLG,ISFFPFLG)  

    ZSFFPRES  = ZVERCO

    ZSFF      = ZSHPSPD

    ISFFPFLG  = IVECFL

!*          4.13.2      VARIABLE

    IF(ZSFF == RMDI )        THEN
      ISFFSTAT= ICHSTAT_REJECT( ISFFSTAT )
    ENDIF

!*          4.13.3      ERROR STATISTICS

!*          4.13.4      STATUS, EVENTS, FLAGS
  
    IF(NMKCMVSE( 37,KCDSQNO,KOBSQNO) == 0)          THEN
      ISFFSTAT= ICHSTAT_REJECT( ISFFSTAT )
    ENDIF
    CALL PPVAFL(ISFFPFLG,ISFFFLG,ISFFRDBFL)

!*          4.13.5      DEPARTURES

!*          4.13.6      UPDATE BODY ENTRY

    CALL SYNOPBE(ISFF     ,&
     & IVNM     ,IVERCO   ,ZVERCO   ,ZSFFPRL  ,ZSFF     ,&
     & ZSFFOERR ,ZSFFPERR ,ZSFFFERR ,ZSFFRERR ,&
     & ZSFFFGER ,ZSFFFGCC1,ZSFFFGCC2,&
     & ZSFFFGDEP,ZSFFANDEP,&
     & ISFFSTAT ,ISFFEVEN1,ISFFEVEN2,ISFFRDBFL,ISFFANFLG,&
     & IPRESCD                                            )  
  ENDIF
ENDDO SFF_LOOP
!*
!-----------------------------------------------------------------------

!        5.             FINISH OFF REPORT (re)CREATION
!                       ------------------------------

5 CONTINUE

!*          5.1         UPDATE HEADER PARAMETERS (IF ANY)

!*          5.2         ODB REPORT UPDATED SWITCH

LDTAKEN = .TRUE.
!*
!-----------------------------------------------------------------------

!        6.             IF OUTPUT MESSAGE; PRINT LAT/LON
!                       --------------------------------

6 CONTINUE
IF(LDMESAGE)           THEN
  WRITE(NULOUT,'('' LAT.     :'',F10.2)') ZLAT
  WRITE(NULOUT,'('' LON.     :'',F10.2)') ZLON
  CALL FLUSH(NULOUT)
ENDIF
!*
!-----------------------------------------------------------------------

!        7.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('SHIPIN',1,ZHOOK_HANDLE)
END SUBROUTINE SHIPIN
