SUBROUTINE SPEEDCOR_NAG(KKNOTSP7, PKNOTS, PSPLCOEFF, PSPLEND,&
 & PVOLD,PVNEW)  

!   Apply wind speed bias correction using NAG spline routine

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN,   ONLY : NULOUT

IMPLICIT NONE

!   Interface

INTEGER(KIND=JPIM),INTENT(IN)    :: KKNOTSP7 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKNOTS(KKNOTSP7) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPLCOEFF(KKNOTSP7) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPLEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOLD 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVNEW 
!   Local variables

INTEGER(KIND=JPIM) :: IFAIL, ILEFT
REAL(KIND=JPRB)    :: ZNEW(4), ZSLOPE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!   Initialize local variables

IF (LHOOK) CALL DR_HOOK('SPEEDCOR_NAG',0,ZHOOK_HANDLE)
IFAIL = 1
ILEFT = 0
ZSLOPE = 0.

ZNEW(:) = 0.

IF (PVOLD > PSPLEND) THEN
  CALL E02BCF (KKNOTSP7, PKNOTS, PSPLCOEFF, PSPLEND, ILEFT, ZNEW, IFAIL)
  ZSLOPE = ZNEW(2)
  PVNEW = ZNEW(1) + (PVOLD-PSPLEND)*ZSLOPE
ELSE
  CALL E02BCF (KKNOTSP7, PKNOTS, PSPLCOEFF, PVOLD, ILEFT, ZNEW, IFAIL)
  PVNEW = ZNEW(1)
ENDIF

IF (IFAIL /= 0) THEN
  WRITE (NULOUT,*) 'Error in SPEEDCOR_NAG: E02BCF returned ',ifail
  CALL ABOR1 ('SPEEDCOR_NAG: E02BCF returned non-zero ifail')
ENDIF
IF (LHOOK) CALL DR_HOOK('SPEEDCOR_NAG',1,ZHOOK_HANDLE)

END SUBROUTINE SPEEDCOR_NAG
