SUBROUTINE SPEEEST(PSIGMOY,PINCMOY,PSPEED)
!***************************************************************************
!     PURPOSE: ESTIMATE OF WIND SPEED FROM FORE AND AFT SIGMA0
!     ACCORDING TO ACMOD4 MODEL
!     INPUT : (0.5(SIG(FORE)**EXPP+SIG(AFT)**EXPP))**(1./EXPP)
!            INCIDENCE ANGLE IN DEGREES
!     OUTPUT : ESTIMATED WIND SPEED IN M/S
!***************************************************************************

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMERSCA , ONLY : ACMOD4   ,BIAS

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIGMOY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINCMOY 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPEED 
INTEGER(KIND=JPIM) :: IA

REAL(KIND=JPRB) :: ZALPHA, ZBETA, ZBIAS, ZFACT, ZFACT1, ZFACT2,&
 & ZGAMMA, ZR1, ZR2, ZSPEED1, ZSPEED2, ZTETA2, &
 & ZTETAI  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SPEEEST',0,ZHOOK_HANDLE)
ZTETAI = (PINCMOY - 40._JPRB)/25._JPRB
ZTETA2 = (3._JPRB*ZTETAI * ZTETAI - 1.0_JPRB)/2.0_JPRB

ZALPHA = ACMOD4(1)+ACMOD4(2)*ZTETAI+ACMOD4(3)*ZTETA2
ZGAMMA = ACMOD4(4)+ACMOD4(5)*ZTETAI+ACMOD4(6)*ZTETA2
ZBETA  = ACMOD4(7)+ACMOD4(8)*ZTETAI+ACMOD4(9)*ZTETA2

IA = MIN(MAX(INT(PINCMOY),17),57)
ZR2 = PINCMOY - REAL(IA,JPRB)
ZR1 = 1.0_JPRB - ZR2
ZBIAS = ZR1*BIAS(IA) + ZR2*BIAS(IA+1)

ZFACT=LOG(PSIGMOY/ZBIAS)/LOG(10._JPRB)
ZFACT1=(ZFACT-ZALPHA)*3.2_JPRB/ZGAMMA
ZSPEED1=ZFACT1*ZFACT1
ZFACT2=(ZFACT-ZALPHA)/ZGAMMA
ZSPEED2=10._JPRB**ZFACT2
IF(ZSPEED2 > 5._JPRB) THEN
  PSPEED=ZSPEED1-ZBETA
ELSEIF(ZSPEED2 > 0.0_JPRB.AND.ZSPEED2 <= 5._JPRB) THEN
  PSPEED=ZSPEED2-ZBETA
ELSE
  PSPEED=2.0_JPRB
ENDIF
IF (LHOOK) CALL DR_HOOK('SPEEEST',1,ZHOOK_HANDLE)
END SUBROUTINE SPEEEST
