SUBROUTINE STORD(KCURPO,LDLOCAL)

!**** SUBROUTINE STORD - STORE DATUM POINTERS INTO LEVEL TABLES

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        STORE DATUM POINTERS INTO LEVEL TABLES

!*    INTERFACE
!     ---------

!     ***CALL STORD(KCURPO,LDLOCAL)
!     WHERE KCURPO  = REPORT'S CURRENT POINTER
!           LDLOCAL = .TRUE.  IF THE REPORT IF FROM THE LOCAL CMA
!                     .FALSE. IF THE REPORT IS FROM ANOTHER PE

!        STORD IS CALLED BY SUBROUTINE REDUN

!*    EXTERNALS
!     ---------

!        FINDL - FIND THE CLOSEST STD-STORD
!        UPDTA - UPDATE TABLES AND POINTERS

!**   MODIFICATIONS
!     -------------
!        H. Jarvinen 12 Dec 1996 : A modified subroutine call
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KCURPO 
LOGICAL           ,INTENT(IN)    :: LDLOCAL 
INTEGER(KIND=JPIM) :: IDSTA(4)

INTEGER(KIND=JPIM) :: ICMVCO, ICMVNM, ICOUNT, IDAT, ILEV,&
 & ISTDLE, JNLV  

REAL(KIND=JPRB) :: ZCMPPP, ZDIFF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "findl.intfb.h"
#include "ngedsta.intfb.h"
#include "updta.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------
!*
!     ------------------------------------------------------------------

!        2.       LOOP OVER THE REPORT, DATUM BY DATUM
!                 ------------------------------------

IF (LHOOK) CALL DR_HOOK('STORD',0,ZHOOK_HANDLE)
DO JNLV = MLNKH2B(KCURPO),MLNKH2B(KCURPO+1)-1
  ILEV = JNLV
!*
!           2.1  CHECK THE DATUM STATUS

  CALL NGEDSTA(INT(ROBODY(JNLV,MDBDSTA)),ICOUNT,IDSTA)
  IF(IDSTA(2) == 1.OR. IDSTA(3) == 1.OR. IDSTA(4) == 1) THEN
    CYCLE
  ENDIF
!*
!           2.2  FIND REPORT VERTICAL CO-ORDINATE

  ICMVNM = ROBODY(ILEV,MDBVNM)
  ICMVCO = ROBODY(ILEV,MDBVCO)
  ZCMPPP = ROBODY(ILEV,MDBPPP)
  IF(ICMVCO == 2) THEN
    WRITE(NULERR,'('' ERROR IN STORD-ICMVCO == 2 '')')
    CALL ABOR1('STORD: ICMVCO == 2 ')
  ENDIF
!*
!           2.3  FIND THE CLOSEST STANDARD LEVEL AND DISTANCE TO IT

  CALL FINDL(ZCMPPP,ISTDLE,ZDIFF)
!*
!           2.4  UPDATE VARIABLE COUNTERS AND POINTER TABLES

  IDAT = ILEV
  CALL UPDTA(ICMVNM,KCURPO,IDAT,ISTDLE,ZDIFF,LDLOCAL)
!*
!           2.5  CLOSE THE LOOP OVER THE REPORT

ENDDO
!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                 ------
IF (LHOOK) CALL DR_HOOK('STORD',1,ZHOOK_HANDLE)
END SUBROUTINE STORD
