SUBROUTINE SUALSCRE

!**** *SUALSCRE* - Allocate the OBS. RELATED arrays

!     Purpose.
!     -------
!           Allocation of the tables for observation screening

!**   Interface.
!**   ---------
!**         SUALSCRE is called from routine SCREEN

!     Author: HEIKKI JARVINEN
!     Date  : 95-10-18

!     Modifications
!     -------------
!     M.Hamrud 98-06-04 Suppress prints
!     E.Holm 2000-10-09 Use maximum variable number NVNUMAX for
!                       dimensioning NDOBVA, NDEPEX, and NDEPNE
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR
USE YOMLUN   , ONLY : NULOUT
USE YOMSCREE , ONLY : NRDBRR   ,NRDBRD   ,NREVTOT  ,NDEVTOT  ,&
 & NDOBVA   ,NDEPEX   ,NDEPNE  
USE YOMLEVSE , ONLY : NSTDPL   ,NLLNVA   ,NLLNDA   ,NVCOUN   ,&
 & NVPOIN   ,RSTDPL   ,GVDIST  
USE YOMVNMB  , ONLY : NVNUMAX
USE YOMCMA   , ONLY : NMXOTP

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*          1.   LEVEL SELECTION TABLES AND COUNTERS
!                -----------------------------------

IF (LHOOK) CALL DR_HOOK('SUALSCRE',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT

IF(LLP)WRITE(IU,'('' SUALSCRE PRINTOUTS '')')

ALLOCATE(NVCOUN(NSTDPL,NLLNVA))
IF(LLP)WRITE(IU,9990) 'NVCOUN   ',SIZE(NVCOUN   ),SHAPE(NVCOUN   )

ALLOCATE(NVPOIN(NSTDPL,NLLNVA,NLLNDA,4))
IF(LLP)WRITE(IU,9990) 'NVPOIN   ',SIZE(NVPOIN   ),SHAPE(NVPOIN   )

ALLOCATE(GVDIST(NSTDPL,NLLNVA,NLLNDA))
IF(LLP)WRITE(IU,9990) 'GVDIST   ',SIZE(GVDIST   ),SHAPE(GVDIST   )

ALLOCATE(RSTDPL(NSTDPL))
IF(LLP)WRITE(IU,9990) 'RSTDPL   ',SIZE(RSTDPL   ),SHAPE(RSTDPL   )

!     ------------------------------------------------------------------

!*          2.   TABLES FOR SCREENING STATISTICS
!                -------------------------------

ALLOCATE(NRDBRR(NMXOTP,5))
IF(LLP)WRITE(IU,9990) 'NRDBRR   ',SIZE(NRDBRR   ),SHAPE(NRDBRR   )

ALLOCATE(NRDBRD(NMXOTP,5))
IF(LLP)WRITE(IU,9990) 'NRDBRD   ',SIZE(NRDBRD   ),SHAPE(NRDBRD   )

ALLOCATE(NREVTOT(NMXOTP,32))
IF(LLP)WRITE(IU,9990) 'NREVTOT  ',SIZE(NREVTOT  ),SHAPE(NREVTOT  )

ALLOCATE(NDEVTOT(NMXOTP,32))
IF(LLP)WRITE(IU,9990) 'NDEVTOT  ',SIZE(NDEVTOT  ),SHAPE(NDEVTOT  )

ALLOCATE(NDOBVA(NMXOTP,NVNUMAX))
IF(LLP)WRITE(IU,9990) 'NDOBVA   ',SIZE(NDOBVA   ),SHAPE(NDOBVA   )

ALLOCATE(NDEPEX(NMXOTP,NVNUMAX))
IF(LLP)WRITE(IU,9990) 'NDEPEX   ',SIZE(NDEPEX   ),SHAPE(NDEPEX   )

ALLOCATE(NDEPNE(NMXOTP,NVNUMAX))
IF(LLP)WRITE(IU,9990) 'NDEPNE   ',SIZE(NDEPNE   ),SHAPE(NDEPNE   )

!     ------------------------------------------------------------------

!*          3.   RETURN
!                ------

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('SUALSCRE',1,ZHOOK_HANDLE)
END SUBROUTINE SUALSCRE
