SUBROUTINE SUANCT

!**** SUBROUTINE SUANCT - ANALYSIS NUMERICAL CONSTANTS

!        D. VASILJEVIC    ECMWF   06/03/1990

!     PURPOSE
!     -------

!        SET NUMERICAL CONSTANTS

!**   INTERFACE
!     ---------

!        SUANCT IS CALLED BY SUBROUTINE SUOBS

!     METHOD
!     ------

!        NECESSARY NUMERICAL CONSTANTS ARE KEPT IN COMMON/YOMANCS/.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : NMDI     ,RMDI,&
 & NBITS    ,NOBITS   ,NOBYTES  ,NCMBYTES ,&
 & RADIANS  ,RDEGREES ,EPSMAC   ,RSQRT2PI ,&
 & RNLATLIM ,RSLATLIM ,RELONLIM ,RWLONLIM ,&
 & RDAYPYEA ,RHOUPDAY ,RMINPHOU ,RSECPMIN ,&
 & NHOUPDAY ,NMINPHOU ,NSECPMIN ,&
 & RDAYFINH ,NDAYFINH ,&
 & RREDPERR ,RREDPEW1 ,RREDPEW2 ,&
 & RGWOSW   ,RAIRDEN  ,RSHPCO   ,&
 & RSINAG   ,RCOSAG   ,&
 & RSYNANMH ,RSYNBARH ,&
 & RSHIPDH  ,RSANMOMH ,RSBAROMH ,RDRIBUDH,RDANMOMH ,RDBAROMH ,&
 & RPAPM    ,&
 & RKELVN   ,RT0      ,RRD      ,RRV     ,R2ES     ,&
 & R3LES    ,R3IES    ,R4LES    , R4IES  ,RTWAT    ,RTT      ,&
 & RTICE    ,RMDB     ,RUPMLIM  ,RDWMLIM ,EPSBUFR  
USE YOMCST   , ONLY : RPI

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JS0, JS1, JS2, IOFF

REAL(KIND=JPRB) :: Z, ZOMEGA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*          1.1   SOME COMMON DATA IS INITIALIZED IN BLOCK DATA SUANCTBL

!*          1.3   RAD./DEG. CONVERTIONS

IF (LHOOK) CALL DR_HOOK('SUANCT',0,ZHOOK_HANDLE)
RADIANS =RPI/180._JPRB
RDEGREES=180._JPRB/RPI

!*          1.4   MACHINE PRECISION

EPSMAC  = EPSILON(Z)

!*          1.6   NO. OF BITS IN A INTEGER USED AND NO OF BYTES USED

NOBITS      = 32
NOBYTES     =  8
NCMBYTES    =  8

!*          1.7   SQRT(2*PI) USED IN VARIATIONAL QC

RSQRT2PI=SQRT(2.0_JPRB*RPI)
!DP
!DP
!DP
NMDI   =2147483647
RMDI   =-NMDI
RMDB   =1.7E38_JPRB
EPSBUFR=1.E-10_JPRB
RUPMLIM = RMDB+EPSBUFR
RDWMLIM = RMDB-EPSBUFR

!DP
!DP

!*          1.8   LAT/LON LIMITS

RNLATLIM =  90.00_JPRB
RSLATLIM = -90.00_JPRB
RELONLIM = 180.00_JPRB
RWLONLIM =-180.00_JPRB

!*          1.9   TIME CONSTANTS

RDAYPYEA=  365.2425_JPRB
RHOUPDAY=   24._JPRB
NHOUPDAY=   24
RMINPHOU=   60._JPRB
NMINPHOU=   60
RSECPMIN=   60._JPRB
NSECPMIN=   60
RDAYFINH=    6._JPRB
NDAYFINH=    6

!*          1.10  PERSISTENCE ERROR REDUCTION FACTOR AND WEIGHTS

RREDPERR=    6._JPRB
RREDPEW1=    4._JPRB
RREDPEW2=    2.0_JPRB

!*          1.11  RATIO OF GEOSTROPHIC TO SURFACE WIND OVER SEA,
!                 DENSITY OF AIR AND CONSTANT FOR SHIP TENDENCY
!                 CORECTION

ZOMEGA  =     .7292E-4_JPRB
RGWOSW  =    1.25_JPRB
RAIRDEN =    0.11_JPRB
RSHPCO  =    2.0_JPRB*RGWOSW*RAIRDEN*ZOMEGA

!*          1.12  SIN AND COS OF ANGLE BETWEEN ACTUAL AND GEOSTROPHIC
!                 WINDS

RSINAG  =    0.34_JPRB
RCOSAG  =    0.94_JPRB

!*          1.17  SYNOP/SHIP/DRIBU DECK/ANEMOMETER/BAROMETER HEIGHT

RSYNANMH = 10.0_JPRB
RSYNBARH =  2.0_JPRB

RSHIPDH  = 0.0_JPRB
RSANMOMH =25.0_JPRB
RSBAROMH = 0.0_JPRB
RDRIBUDH = 0.0_JPRB
RDANMOMH =10.0_JPRB
RDBAROMH = 0.0_JPRB

!*          1.18  PRESSURE/HEIGHT CONVERSION FACTOR

RPAPM  =  1.225_JPRB

!           1.18  THERMO CONSTANTS (JUST FOR NOW)

RKELVN =273.16_JPRB
RT0    =273.16_JPRB
RRD    =287.0597_JPRB
RRV    =461.5250_JPRB
RTT=RT0
R2ES=611.21_JPRB*RRD/RRV
R3LES=17.502_JPRB
R3IES=22.587_JPRB
R4LES=32.19_JPRB
R4IES=-0.7_JPRB
RTWAT=RTT
RTICE=RTT-23._JPRB

!           1.19  MASKING CONSTANTS

DO JS0 = 1    , NOBITS
  NBITS  (JS0   ) = 0
ENDDO

DO JS1 = 0    , NOBITS-1
  IOFF = JS1
  DO JS2 = IOFF , NOBITS-1
    NBITS  (JS2+1) = IBSET(NBITS  (JS2+1),JS1)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*       2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SUANCT',1,ZHOOK_HANDLE)
END SUBROUTINE SUANCT
