SUBROUTINE SUDIMO(KULOUT)
!****-------------------------------------------------------------------
!**** *SUDIMO* - Setting up the dimensioning of the arrays related
!****            to observations.
!****-------------------------------------------------------------------
!***  Purpose. Initialisation of common YOMDIMO containing the dimensions
!***  -------  of arrays related to observations.
!**   Interface.
!**   ---------
!**   *CALL* * SUDIMO(KULOUT)*
!**         KULOUT:  logical unit for the output
!**         The common YOMDIMO is an implicit output of SUDIMO
!**   Externals.
!     ---------
!        OBADAT - GET RELEVANT INFORMATIONS FROM OBS. FILE
!     Modifications
!     -------------
!        Jean Pailleux and Drasko Vasiljevic 89 - 92
!        H. Jarvinen   : 95-09-20  New CMA- file format
!        R. Saunders   : 97-09-19  Add 1 more TOVS channel
!        Per Unden     : 97-10-23  Use TEMP significant levels
!        Erik Andersson: 98-08-18  Moved TOVS levels/chans to SURAD
!        Erik Andersson: 98-08-18  TOVS control variable
!        D.Dent        : 99-05-06  allow re-ALLOCATE
!        V. Casse      : 99-08     Cleaning
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        S.Saarinen    04-Sep-2007 NMXLEN from namelist NAMDIMO
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARCMA   , ONLY :  JPMXOTP  ,JPMXOCT
USE PARDIMO  , ONLY :       JPXSOBT  ,JPXVAR   ,JPXAREA
USE YOMDIMO  , ONLY : NOBTOT   ,NOBSLN   ,NACTIM   ,NMXSOBT  ,NMXAREA  ,&
 & NMXSET   ,NMXLEN   ,L_SPAN_KSETS_ACROSS_TSLOTS, NMXTCH, &
 & NOBTOV   ,NOBSCA   ,NOBNTV   ,&
 & NOBTOVG  ,NOBTOVBP ,NOBTOVMX ,&
 & NOBNTVBP, NOBNTVG  ,&
 & L_TASKOB_USE_SORTED_TIMES  , &
 & L_TASKOBAD_USE_SORTED_TIMES, &
 & L_TASKOBTL_USE_SORTED_TIMES
USE YOMCT0   , ONLY :     LCANARI  ,LOBS     ,LSIMOB   ,&
 & NPROC  
USE YOMCT0B  , ONLY : LECMWF
USE YOMCMA   , ONLY : NMXNVO   ,NMXVAR   ,NMXOTP   ,&
 & NMXOCT   ,NMXDCL   ,NODEPT   ,NRESUPD  
USE YOMLUN   , ONLY : NULNAM
USE YOMPLDSW , ONLY : LOPT_SCALAR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 

INTEGER(KIND=JPIM) :: imxlen
character(len=20) ::  CL_ENV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "namdimo.h"

#include "obadat.intfb.h"
#include "sucmad1.intfb.h"
#include "sucmad2.intfb.h"
#include "sucmoctp.intfb.h"
#include "posnam.intfb.h"

!*    ------------------------------------------------------------------
!        1. Set up the dimensions of the obs cost-function tables, i.e.
!           number of obs types,sub-ob types,variable types, areas.
!           -------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDIMO',0,ZHOOK_HANDLE)
IF(.NOT.ALLOCATED(NOBTOVBP)) THEN
  ALLOCATE(NOBTOVBP(NPROC))
  WRITE(KULOUT,9) 'NOBTOVBP',SIZE(NOBTOVBP),SHAPE(NOBTOVBP)
ENDIF

IF(.NOT.ALLOCATED(NOBNTVBP)) THEN
   ALLOCATE(NOBNTVBP(NPROC))
   WRITE(KULOUT,9) 'NOBNTVBP',SIZE(NOBNTVBP),SHAPE(NOBNTVBP)
ENDIF


NMXOTP  = JPMXOTP   ! MAX. NO. OF OBS. TYPES (10)
NMXOCT  = JPMXOCT   ! MAX. NO. OF OBS. CODE TYPES (12)
NMXSOBT = JPXSOBT   ! MAX. NO. OF SUB-OBS. TYPES (3)
NMXVAR  = JPXVAR    ! MAX. NO. OF VARIABLES (25)
NMXAREA = JPXAREA   ! MAX. NO. OF AREAS (20)

!*    ------------------------------------------------------------------
!        2. Prescribe some max value
!           ------------------------

IF ( .NOT.LSIMOB .AND. LOBS ) THEN
  IF (LECMWF) THEN
    NMXNVO = 300
  ELSE
    NOBSCA = 1
    NMXNVO = 150
  ENDIF
ELSE
  NMXNVO = 1
ENDIF

!*          2.9   MAX. LEN. OF CMA DDR

IF (.NOT.LCANARI) THEN
  IF (.NOT.LSIMOB .AND. LOBS) THEN
    NMXDCL = 1024
  ELSE
    NMXDCL = 1
  ENDIF
ENDIF

IF ( LSIMOB .OR. .NOT.LOBS ) THEN
  NOBTOT   = 1
  NOBTOV   = 1
  NOBNTV   = 1
  NOBTOVG  = NOBTOV
  NOBTOVBP(:)=NOBTOV
  NOBTOVMX = NOBTOV
  NOBNTVG = NOBNTV
  NOBNTVBP(1) = NOBNTV
  NOBSCA   = 1
  NACTIM   = 1
  NOBSLN   = 1
  NODEPT   = 0
  NRESUPD  = 0
  NMXTCH   = 0
ELSE
  CALL SUCMAD1
  CALL SUCMAD2
  CALL SUCMOCTP(KULOUT)
  NOBTOT   = 0
  NOBTOV   = 0
  NOBNTV   = 0
  NOBSCA   = 0
  NACTIM   = 0
  NOBSLN   = 0
  NODEPT   = 0
  NRESUPD  = 0
  CALL OBADAT
ENDIF

!*    ------------------------------------------------------------------
!        3.        Set up the arrays organising the obs sets
!                  -----------------------------------------

IF (LOPT_SCALAR) THEN
  NMXLEN = 64 ! 20
  L_SPAN_KSETS_ACROSS_TSLOTS = .TRUE.
ELSE
  NMXLEN = 511
  L_SPAN_KSETS_ACROSS_TSLOTS = .TRUE.
ENDIF

IF (LSIMOB .OR. .NOT.LOBS) THEN
  NMXLEN = 1
  NMXSET = 1
ELSE
!-- Override from namelist 'NAMDIMO'
  CALL POSNAM(NULNAM,'NAMDIMO')
  READ(NULNAM,NAMDIMO)
!-- Override from export NMXLEN=<value>
  imxlen=0
  call getenv('NMXLEN',CL_ENV)
  if (CL_ENV /= ' ') then
    read(CL_ENV,*,err=9999) imxlen
    if (imxlen > 0) NMXLEN = imxlen
  endif
  9999 continue
!-- Keep within a sane interval [1..8191]
  NMXLEN = MIN(MAX(NMXLEN, 1), 8191)
  WRITE(KULOUT,'(1X,A,I6,A,L3)') &
       & 'SUDIMO: NMXLEN = ',NMXLEN,&
       & ', L_SPAN_KSETS_ACROSS_TSLOTS = ',L_SPAN_KSETS_ACROSS_TSLOTS
  NMXSET = NOBTOT/NMXLEN + 80*NACTIM
ENDIF

!*    ------------------------------------------------------------------
!        6.        PRINT OUT SET-UP
!                  ----------------

WRITE(UNIT=KULOUT,FMT='('' SUDIMO SETUP:'')')
WRITE(UNIT=KULOUT,FMT='('' NMXOTP  ='',I7,'' NMXSOBT ='',I7,&
 & '' NMXVAR  ='',I7,'' NMXAREA ='',I7,'' JPMXOTP ='',I7)')&
 & NMXOTP,NMXSOBT,NMXVAR,NMXAREA,JPMXOTP  
WRITE(UNIT=KULOUT,FMT='('' NMXNVO  ='',I7)')NMXNVO
WRITE(UNIT=KULOUT,FMT='('' NOBTOT  ='',I7,'' NOBTOV  ='',I7,&
 & '' NOBNTV  ='',I7,'' NOBSCA  ='',I7,/,&
 & '' NACTIM  ='',I7,&
 & '' NOBSLN  ='',I8,&
 & '' NOBTOVG ='',I7,'' NOBTOVMX='',I7,/,&
 & '' NOBTOVBP='',50(2X,10I8,/))')&
 & NOBTOT,NOBTOV,NOBNTV,NOBSCA,NACTIM,NOBSLN &
 & ,NOBTOVG,NOBTOVMX,NOBTOVBP  
WRITE(UNIT=KULOUT,FMT='('' NMXSET  ='',I7,'' NMXLEN  ='',I7)')NMXSET,NMXLEN

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
!----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUDIMO',1,ZHOOK_HANDLE)
END SUBROUTINE SUDIMO
