SUBROUTINE SUFGLIM

!**** SUBROUTINE SUFGLIM - SETUP THE FIRST GUESS ERRORS FLAG LIMITS

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995

!*    METHOD
!     ------

!        FIRST GUESS ERRORS FLAG LIMITS ARE DEFINED

!*    INTERFACE
!     ---------

!     ***CALL*SUFGLIM

!*    EXTERNALS
!     ---------

!        SUFGLIM IS CALLED BY SUBROUTINE SCREEN

!**   MODIFICATIONS
!     -------------
!     98-05-15 Erik Andersson - Move BgQC limits in defrun + namelist.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 
!     04-06-26 C. Payan       - fix RSCATLI with SST for MF
!     05-05    C. Payan       - use of NICESEAMASK for fixing RSCATLI
!     06-11-25 Hans Hersbach  - Add ASCAT
!     07-02-21 Hans Hersbach  - Reintroduce additional check on sea-ice for scat

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMFGLIM , ONLY : RAVRWS   ,RDF060   ,RDF090   ,RDF120
USE YOMRPLIM , ONLY : RH300    ,RH800    ,RH1700   ,RH2300   ,&
 & RH3700   ,RP500    ,RP700    ,RP850    ,RP1000  
USE YOMCT0B  , ONLY : LECMWF
USE YOMTHLIM , ONLY : RSCAWLI  ,RSCACLI  ,RSCATLI  ,RREDTSL  ,RSHIDIS  ,&
 & RSCAFAC  ,NSCIND1  ,NSCIND2  ,NSCATWVC ,NASCAWVC  ,NSCAMIN  ,&
 & RSCATDIS ,RASCADIS, NNSCAWVC ,RNSCADIS  ,NQSCAWVC ,NOSCAWVC ,RQSCADIS, ROSCADIS  

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.       FLAG LIMITS FOR FIRST GUESS CHECKS
!                 ----------------------------------

!*          1.1   WIND DIRECTION CHECK LIMITS

IF (LHOOK) CALL DR_HOOK('SUFGLIM',0,ZHOOK_HANDLE)
RAVRWS =  15._JPRB
RDF060 =  60._JPRB
RDF090 =  90._JPRB
RDF120 = 120._JPRB
!*
!     ------------------------------------------------------------------

!        2.       PRESSURE LIMITS FOR REPORTING PRACTICE CHECK
!                 --------------------------------------------
!*
!           2.1   PRESSURE LIMITS FOR SYNOP/TEMP/PILOT (IN PA)

RP500  =  50000._JPRB
RP700  =  70000._JPRB
RP850  =  85000._JPRB
RP1000 = 100000._JPRB
!*
!           2.2   HEIGHT LIMITS FOR SYNOP/TEMP/PILOT (IN M)

RH300  =    300._JPRB
RH800  =    800._JPRB
RH1700 =   1700._JPRB
RH2300 =   2300._JPRB
RH3700 =   3700._JPRB

!     ------------------------------------------------------------------

!*       3.       LIMITS FOR DATA THINNING
!                 ------------------------

!*          3.1   SPEED LIMIT FOR USE OF SCATT. WIND (IN M/S)

RSCAWLI = 35._JPRB

!*          3.2   SEA ICE CONCENTRATION LIMIT FOR USE OF SCATT. WIND

IF (LECMWF) THEN
   RSCATLI = 273.15_JPRB
ELSE
! MF
   RSCATLI = 278.0_JPRB
ENDIF

!fraction of ice
RSCACLI = 0.01_JPRB

!*          3.3   LIMIT FOR SCATT AVERAGE DISTANCE TO THE CONE

RSCAFAC     = 1.5_JPRB

NSCATWVC    = 19
ALLOCATE(RSCATDIS(NSCATWVC))
WRITE(NULOUT,9990) 'RSCATDIS  ',SIZE(RSCATDIS  ),SHAPE(RSCATDIS  )

NSCAMIN     = 100
RSCATDIS( 1) = 0.53_JPRB
RSCATDIS( 2) = 0.55_JPRB
RSCATDIS( 3) = 0.53_JPRB
RSCATDIS( 4) = 0.54_JPRB
RSCATDIS( 5) = 0.56_JPRB
RSCATDIS( 6) = 0.59_JPRB
RSCATDIS( 7) = 0.63_JPRB
RSCATDIS( 8) = 0.68_JPRB
RSCATDIS( 9) = 0.75_JPRB
RSCATDIS(10) = 0.83_JPRB
RSCATDIS(11) = 0.92_JPRB
RSCATDIS(12) = 0.99_JPRB
RSCATDIS(13) = 1.07_JPRB
RSCATDIS(14) = 1.17_JPRB
RSCATDIS(15) = 1.25_JPRB
RSCATDIS(16) = 1.33_JPRB
RSCATDIS(17) = 1.42_JPRB
RSCATDIS(18) = 1.52_JPRB
RSCATDIS(19) = 1.64_JPRB

!*          3.4   LIMIT FOR ASCAT AVERAGE MLE

NASCAWVC    = 42
ALLOCATE(RASCADIS(NASCAWVC))
WRITE(NULOUT,9990) 'RASCADIS ',SIZE(RASCADIS ),SHAPE(RASCADIS )

RASCADIS( :)  = 50.0_JPRB

RASCADIS( 1)  = 3.00_JPRB
RASCADIS( 5)  = 2.40_JPRB
RASCADIS( 9)  = 1.77_JPRB
RASCADIS(13)  = 1.64_JPRB
RASCADIS(17)  = 1.27_JPRB
RASCADIS(21)  = 0.95_JPRB

RASCADIS(22)  = 0.95_JPRB
RASCADIS(26)  = 1.22_JPRB
RASCADIS(30)  = 1.44_JPRB
RASCADIS(34)  = 1.85_JPRB
RASCADIS(38)  = 2.90_JPRB
RASCADIS(42)  = 3.00_JPRB


!*          3.5   LIMIT FOR NSCAT AVERAGE MLE

NNSCAWVC    = 24
ALLOCATE(RNSCADIS(NNSCAWVC))
WRITE(NULOUT,9990) 'RNSCADIS ',SIZE(RNSCADIS ),SHAPE(RNSCADIS )

RNSCADIS(:)  = 50.0_JPRB

!*          3.6   LIMIT FOR QSCAT AVERAGE MLE

NQSCAWVC    = 38
ALLOCATE(RQSCADIS(NQSCAWVC))
WRITE(NULOUT,9990) 'RQSCMLE  ',SIZE(RQSCADIS ),SHAPE(RQSCADIS )

RQSCADIS(:) = 50.0_JPRB

!*          3.7   LIMIT FOR OCEANSAT-2 AVERAGE MLE

NOSCAWVC    = 36
ALLOCATE(ROSCADIS(NOSCAWVC))
WRITE(NULOUT,9990) 'RISCMLE  ',SIZE(ROSCADIS ),SHAPE(ROSCADIS )

ROSCADIS(:) = 50.0_JPRB

!*          3.8   TEMP/SYNOP MASS DATA REDUNDANCY REJ. LIMIT (IN PA)

RREDTSL = 5000._JPRB

!*          3.9   SHIP/DRIBU DISTANCE LIMIT (IN M) (= ONE DEGREE AT EQ.)

RSHIDIS = 111000._JPRB

!     ------------------------------------------------------------------

!*       4.       SETUP HELP ARRAYS TO DEDUCE TOVS SCANNING POSITION
!                 --------------------------------------------------

!*          4.1   FIRST ARRAY

NSCIND1( 1) = 1
NSCIND1( 2) = 2
NSCIND1( 3) = 3
NSCIND1( 4) = 4
NSCIND1( 5) = 5
NSCIND1( 6) = 6
NSCIND1( 7) = 1
NSCIND1( 8) = 2
NSCIND1( 9) = 3
NSCIND1(10) = 4
NSCIND1(11) = 5
NSCIND1(12) = 6
NSCIND1(13) = 1
NSCIND1(14) = 2
NSCIND1(15) = 3
NSCIND1(16) = 4
NSCIND1(17) = 5
NSCIND1(18) = 6
NSCIND1(19) = 1
NSCIND1(20) = 2
NSCIND1(21) = 3
NSCIND1(22) = 4
NSCIND1(23) = 5
NSCIND1(24) = 6
NSCIND1(25) = 1
NSCIND1(26) = 2
NSCIND1(27) = 3
NSCIND1(28) = 4
NSCIND1(29) = 5
NSCIND1(30) = 6

!*          4.2   SECOND ARRAY

NSCIND2( 1) = 1
NSCIND2( 2) = 2
NSCIND2( 3) = 3
NSCIND2( 4) = 1
NSCIND2( 5) = 2
NSCIND2( 6) = 3

!     ------------------------------------------------------------------

!        5.       RETURN
!                 ------
WRITE(NULOUT,'('' *** FIRST GUESS ERROR LIMITS ARE SET '')')
WRITE(NULOUT,'('' '')')

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUFGLIM',1,ZHOOK_HANDLE)
END SUBROUTINE SUFGLIM
