SUBROUTINE SUGOMS

!     Purpose.
!     --------
!       Setup GOM array structures

!     Author.
!     -------
!        M.Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 15-02-2005
!        R.Engelen : 07-Jul-2005 Add GHG fields
!        A.Dethof  : 15-Aug-2005 Add GRG fields!       
!        A.Benedetti:30-Aug-2005 Add AERO fields
!        N. Bormann      08-Dec-2005     Settings for limb data
!        E. Wattrelot 10-Aug-2006 Add hydrometeors fields
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR,LCANARI,NCONF
USE YOMDIM   , ONLY : NFLEVG
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMVRTL  , ONLY : LOBSTL
USE YOMCOCTP , ONLY : NOOBST,NSATEM,NSCATT,NLIMB,NRADAR
USE YOMECTAB , ONLY : NOBSPROFS 
USE YOMVAR   , ONLY : LAEOLUSAMD
USE YOMDB
USE YOMLUN   , ONLY : NULOUT   ,NULERR, NULNAM  
USE YOM_YGFL , ONLY : YGFL     ,YQ,YO3,YL,YI,YA,YGHG,JPGHG,YGRG, &
 & JPGRG,YAERO,JPAERO,NGHG,NGRG,NAERO,YR,YS,YG
USE GOMS_MIX , ONLY : LGOMUA,YGOMV,YGOMEC,YGOMEC_2D,YGOMCAN, &
 & YGOMGFL,YGOMUA,YGOMS,YGOMUA5,YGOMS5,YGOMUA_2D,YGOMS_2D, &
 & NOFF,NOFF5,NGOMS,NGOMS1D,NGOMS2D,GOMDATUA,GOMDATUA5,&
 & YGOMUA5_2D,YGOMS5_2D,GOMCONSTR,GOMCONSTR_2D,SET_GOM, &
 & LGOMGFL,MGOMGFL,NGOMGFL,YGOMCANA,GOMCOCA

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IOTP,JOBS,IMAPOMM,IPREV,IPROF,JTYP,JGFL,IRET,&
 & INFO(0),JGHG,JGRG,JAERO
LOGICAL :: LLP
REAL(KIND=JPRB) :: ZINFO(0),ZHUGE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "posnam.intfb.h"

#include "namgom.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGOMS',0,ZHOOK_HANDLE)

IRET = 0
CALL GETDB('SUOBSADDR', 0, IRET, INFO, 0, ZINFO, 0,   &! Get ROBSU
 & -1, -1, -1, -1, -1, -1)  

LLP = NPRINTLEV >= 1 .OR. LALLOPR
llp =.true.
WRITE(NULOUT,*) 'SUGOMS-PREPARE GOM STRUCTURES ',NGOMS,NGOMS1D,NGOMS2D

LGOMUA(:,YGOMV%MU) = .TRUE.
LGOMUA(:,YGOMV%MV) = .TRUE.
LGOMUA(:,YGOMV%MT) = .TRUE.
LGOMUA(:,YGOMV%MQ) = .TRUE.
LGOMUA(:,YGOMV%MO3) = .FALSE.
LGOMUA(:,YGOMV%MCLW) = .FALSE.
LGOMUA(:,YGOMV%MCIW) = .FALSE.
LGOMUA(:,YGOMV%MCC) = .FALSE.
LGOMUA(:,YGOMV%MS) = .FALSE.
LGOMUA(:,YGOMV%MR) = .FALSE.
LGOMUA(:,YGOMV%MG) = .FALSE.
!   Cloud variables required if Aeolus aux met data are requested
IF(LAEOLUSAMD ) THEN
  LGOMUA(:,YGOMV%MCLW) = .TRUE.
  LGOMUA(:,YGOMV%MCIW) = .TRUE.
  LGOMUA(:,YGOMV%MCC) = .TRUE.
ENDIF
! Hydrometeor variables required for radar simulator in ALADIN/AROME
IF(YS%LACTIVE) LGOMUA(NRADAR,YGOMV%MS) =.TRUE.
IF(YR%LACTIVE) LGOMUA(NRADAR,YGOMV%MR) =.TRUE.
IF(YG%LACTIVE) LGOMUA(NRADAR,YGOMV%MG) =.TRUE.
DO JGHG=1,JPGHG
  LGOMUA(:,YGOMV%MGHG(JGHG)) = .FALSE.
ENDDO
DO JGRG=1,JPGRG
  LGOMUA(:,YGOMV%MGRG(JGRG)) = .FALSE.
ENDDO
DO JAERO=1,JPAERO
  LGOMUA(:,YGOMV%MAERO(JAERO)) = .FALSE.
ENDDO

LGOMUA(NSATEM,YGOMV%MU) = .FALSE.
LGOMUA(NSATEM,YGOMV%MV) = .FALSE.
LGOMUA(NSATEM,YGOMV%MT) = .TRUE.
LGOMUA(NSATEM,YGOMV%MQ) = .TRUE.
LGOMUA(NSATEM,YGOMV%MO3) = .FALSE.
LGOMUA(NSATEM,YGOMV%MCLW) = .FALSE.
LGOMUA(NSATEM,YGOMV%MCIW) = .FALSE.
LGOMUA(NSATEM,YGOMV%MCC) = .FALSE.
IF(YO3%LACTIVE) LGOMUA(NSATEM,YGOMV%MO3) = .TRUE.
DO JGHG=1,NGHG
 IF(YGHG(JGHG)%LACTIVE) LGOMUA(NSATEM,YGOMV%MGHG(JGHG)) = .TRUE.
ENDDO
DO JGRG=1,NGRG
 IF(YGRG(JGRG)%LACTIVE) LGOMUA(NSATEM,YGOMV%MGRG(JGRG)) = .TRUE.
ENDDO
DO JAERO=1,NAERO
 IF(YAERO(JAERO)%LACTIVE) LGOMUA(NSATEM,YGOMV%MAERO(JAERO)) = .TRUE.
ENDDO

LGOMUA(NSCATT,:) = .FALSE.
LGOMUA(NLIMB,YGOMV%MU) = .FALSE.
LGOMUA(NLIMB,YGOMV%MV) = .FALSE.
LGOMUA(NLIMB,YGOMV%MT) = .TRUE.
LGOMUA(NLIMB,YGOMV%MQ) = .TRUE.
IF(YO3%LACTIVE) LGOMUA(NLIMB,YGOMV%MO3) = .TRUE.
LGOMUA(NLIMB,YGOMV%MCLW) = .FALSE.
LGOMUA(NLIMB,YGOMV%MCIW) = .FALSE.
LGOMUA(NLIMB,YGOMV%MCC) = .FALSE.

CALL POSNAM(NULNAM,'NAMGOM')
READ(NULNAM,NAMGOM)

IF(.NOT.YQ%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MQ))) &
 & CALL ABOR1('SUGOMS:Q NOT ON and GOM ARRAY REQUESTED')
IF(.NOT.YO3%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MO3))) &
 & CALL ABOR1('SUGOMS:O3 NOT ON and GOM ARRAY REQUESTED')
IF(.NOT.YL%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MCLW))) &
 & CALL ABOR1('SUGOMS:L NOT ON and GOM ARRAY REQUESTED')
IF(.NOT.YI%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MCIW))) &
 & CALL ABOR1('SUGOMS:I NOT ON and GOM ARRAY REQUESTED')
IF(.NOT.YA%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MCC))) &
 & CALL ABOR1('SUGOMS:A NOT ON and GOM ARRAY REQUESTED')
DO JGHG=1,NGHG
  IF(.NOT.YGHG(JGHG)%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MGHG(JGHG)))) &
   & CALL ABOR1('SUGOMS:GHG NOT ON and GOM ARRAY REQUESTED')
ENDDO
DO JGRG=1,NGRG
  IF(.NOT.YGRG(JGRG)%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MGRG(JGRG)))) &
   & CALL ABOR1('SUGOMS:GRG NOT ON and GOM ARRAY REQUESTED')
ENDDO
DO JAERO=1,NAERO
  IF(.NOT.YAERO(JAERO)%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MAERO(JAERO)))) &
   & CALL ABOR1('SUGOMS:AERO NOT ON and GOM ARRAY REQUESTED')
ENDDO
! add three conditions for hydrometeors rain,snow and graupel for AROME
IF(.NOT.YS%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MS))) &
 & CALL ABOR1('SUGOMS:S NOT ON and GOM ARRAY REQUESTED')
IF(.NOT.YR%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MR))) &
 & CALL ABOR1('SUGOMS:R NOT ON and GOM ARRAY REQUESTED')
IF(.NOT.YG%LACTIVE .AND. ANY(LGOMUA(:,YGOMV%MG))) &
 & CALL ABOR1('SUGOMS:G NOT ON and GOM ARRAY REQUESTED')

IF( (ANY(LGOMUA(:,YGOMV%MCLW)) .OR.  ANY(LGOMUA(:,YGOMV%MCIW)) .OR. &
 & ANY(LGOMUA(:,YGOMV%MCC))) .AND. NCONF == 131 ) THEN
  CALL ABOR1('SUGOM - CLOUD VARIABLES IN MINIMIZATION NOT SUPPORTED')
ENDIF

WRITE(NULOUT,*)' OBS TYPE NUM U V T Q O3 CLW CIW CC GHG GRG AERO S R G'
DO JTYP =1,NOOBST
  WRITE(NULOUT,'(A,I3,1X,40L2)')' OBS TYPE ',JTYP,LGOMUA(JTYP,:)
ENDDO
ALLOCATE(YGOMGFL(YGFL%NUMFLDS))
ALLOCATE(LGOMGFL(NOOBST,YGFL%NUMFLDS))
DO JGFL=1,YGFL%NUMFLDS
  YGOMGFL(JGFL)%LINTERP=.FALSE.
  YGOMGFL(JGFL)%CSLINT='STANDARD  '
ENDDO
NGOMGFL=0

IF(ANY(LGOMUA(:,YGOMV%MQ))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YQ%MP
  MGOMGFL(YGOMV%MQ) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MQ)
ENDIF
IF(ANY(LGOMUA(:,YGOMV%MO3))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YO3%MP
  MGOMGFL(YGOMV%MO3) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MO3)
ENDIF
IF(ANY(LGOMUA(:,YGOMV%MCLW))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YL%MP
  MGOMGFL(YGOMV%MCLW) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MCLW)
  YGOMGFL(NGOMGFL)%CSLINT='LAIDLIC   '
ENDIF
IF(ANY(LGOMUA(:,YGOMV%MCIW))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YI%MP
  MGOMGFL(YGOMV%MCIW) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MCIW)
  YGOMGFL(NGOMGFL)%CSLINT='LAIDLIC   '
ENDIF
IF(ANY(LGOMUA(:,YGOMV%MCC))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YA%MP
  MGOMGFL(YGOMV%MCC) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MCC)
  YGOMGFL(NGOMGFL)%CSLINT='LAIDLIC   '
ENDIF
DO JGHG=1,NGHG
  IF(ANY(LGOMUA(:,YGOMV%MGHG(JGHG)))) THEN
    NGOMGFL = NGOMGFL+1
    YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
    YGOMGFL(NGOMGFL)%MPMDL   = YGHG(JGHG)%MP
    MGOMGFL(YGOMV%MGHG(JGHG)) = NGOMGFL
    LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MGHG(JGHG))
  ENDIF
ENDDO
DO JGRG=1,NGRG
  IF(ANY(LGOMUA(:,YGOMV%MGRG(JGRG)))) THEN
    NGOMGFL = NGOMGFL+1
    YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
    YGOMGFL(NGOMGFL)%MPMDL   = YGRG(JGRG)%MP
    MGOMGFL(YGOMV%MGRG(JGRG)) = NGOMGFL
    LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MGRG(JGRG))
  ENDIF
ENDDO
DO JAERO=1,NAERO
  IF(ANY(LGOMUA(:,YGOMV%MAERO(JAERO)))) THEN
    NGOMGFL = NGOMGFL+1
    YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
    YGOMGFL(NGOMGFL)%MPMDL   = YAERO(JAERO)%MP
    MGOMGFL(YGOMV%MAERO(JAERO)) = NGOMGFL
    LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MAERO(JAERO))
  ENDIF
ENDDO
!  below add three GOMGFL for rain, snow and graupel: e.wattrelot
IF(ANY(LGOMUA(:,YGOMV%MS))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YS%MP
  MGOMGFL(YGOMV%MS) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MS)
  YGOMGFL(NGOMGFL)%CSLINT='LAIDLIC   '
ENDIF
IF(ANY(LGOMUA(:,YGOMV%MR))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YR%MP
  MGOMGFL(YGOMV%MR) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MR)
  YGOMGFL(NGOMGFL)%CSLINT='LAIDLIC   '
ENDIF
IF(ANY(LGOMUA(:,YGOMV%MG))) THEN
  NGOMGFL = NGOMGFL+1
  YGOMGFL(NGOMGFL)%LINTERP = .TRUE.
  YGOMGFL(NGOMGFL)%MPMDL   = YG%MP
  MGOMGFL(YGOMV%MG) = NGOMGFL
  LGOMGFL(:,NGOMGFL) = LGOMUA(:,YGOMV%MG)
  YGOMGFL(NGOMGFL)%CSLINT='LAIDLIC   '
ENDIF
! end add three GOMGFL for snow, rain and graupel: e.wattrelot

WRITE(NULOUT,*)' NGOMGFL= ',NGOMGFL
WRITE(NULOUT,*)' MGOMGFL= ',MGOMGFL
WRITE(NULOUT,*) 'YGOMGFL - JGFL YGOMGFL(JGFL)%MPMDL %CSLINT'
DO JGFL=1,NGOMGFL
  WRITE(NULOUT,*) JGFL,YGOMGFL(JGFL)%MPMDL,YGOMGFL(JGFL)%CSLINT
  WRITE(NULOUT,*) JGFL,LGOMGFL(:,JGFL)
ENDDO
ALLOCATE(YGOMUA(NGOMS1D))
ALLOCATE(YGOMS(NGOMS1D))
ALLOCATE(YGOMEC(NGOMS1D))
IF(LOBSTL) THEN
  ALLOCATE(YGOMUA5(NGOMS1D))
  ALLOCATE(YGOMS5(NGOMS1D))
ENDIF

IF(LCANARI) THEN
  YGOMEC(:)%ECEMIS=5.0_JPRB
  ALLOCATE(YGOMCAN(NGOMS1D))
  ALLOCATE(YGOMCANA(NGOMS1D))
  DO JOBS=1,NGOMS
    CALL GOMCOCA(YGOMCANA(JOBS),NFLEVG)
  ENDDO
ENDIF

NOFF=0
IPREV =  -1
!* allocate gomdatua before using this array in gomconstr
ALLOCATE(GOMDATUA(0))
DO JOBS=1,NGOMS
  IOTP = ROBSU(JOBS, MDBOTP)
  IF (NOBSPROFS(IOTP) == 1) THEN
    IMAPOMM = ROBSU(JOBS,MDB_MAPOMM_AT_INDEX)
    IF(IMAPOMM < 1 .OR. IMAPOMM > NGOMS1D) THEN
      WRITE(NULERR,*) 'IMAPOMM < 1 .OR. IMAPOMM > NGOMS1D ',IMAPOMM,NGOMS1D
      CALL ABOR1('SUGOMS')
    ENDIF
    CALL GOMCONSTR(YGOMUA(IMAPOMM),NFLEVG,IOTP,IMAPOMM,.FALSE.,.FALSE.)
! ecriture de YGOMUA(IMAPOMM)%LG si JOBS=1
!    IF (JOBS==1) THEN
!    WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LG vaut pour JOBS=1",YGOMUA(IMAPOMM)%LG
!    WRITE(NULOUT,*)"et YGOMV%MG et MGOMGFL(YGOMV%MG)",YGOMV%MG, MGOMGFL(YGOMV%MG)
!    WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LR vaut pour JOBS=1",YGOMUA(IMAPOMM)%LR
!     WRITE(NULOUT,*)"et YGOMV%MR et MGOMGFL(YGOMV%MR)",YGOMV%MR, MGOMGFL(YGOMV%MR)
!     WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LS vaut pour JOBS=1",YGOMUA(IMAPOMM)%LS
!     WRITE(NULOUT,*)"et YGOMV%MS et MGOMGFL(YGOMV%MS)",YGOMV%MS, MGOMGFL(YGOMV%MS)
!    ENDIF
  ENDIF
ENDDO
DEALLOCATE(GOMDATUA)
ALLOCATE(GOMDATUA(NOFF))
IF(LLP)WRITE(NULOUT,9) 'GOMDATUA',SIZE(GOMDATUA ),SHAPE(GOMDATUA)
GOMDATUA(:)=-HUGE(GOMDATUA)
IF(LOBSTL) THEN
  ALLOCATE(GOMDATUA5(NOFF))
  IF(LLP)WRITE(NULOUT,9) 'GOMDATUA5',SIZE(GOMDATUA5 ),SHAPE(GOMDATUA5)
  GOMDATUA5(:)=-HUGE(GOMDATUA5)
ENDIF
NOFF=0
NOFF5=0
ZHUGE=-HUGE(ZHUGE)
DO JOBS=1,NGOMS
  IOTP = ROBSU(JOBS, MDBOTP)
  IF (NOBSPROFS(IOTP) == 1) THEN
    IMAPOMM = ROBSU(JOBS,MDB_MAPOMM_AT_INDEX)
    CALL GOMCONSTR(YGOMUA(IMAPOMM),NFLEVG,IOTP,IMAPOMM,.TRUE.,.FALSE.)
        YGOMUA(IMAPOMM)%LQ=LGOMUA(IOTP,YGOMV%MQ)
        YGOMUA(IMAPOMM)%LR=LGOMUA(IOTP,YGOMV%MR)
        YGOMUA(IMAPOMM)%LS=LGOMUA(IOTP,YGOMV%MS)
        YGOMUA(IMAPOMM)%LG=LGOMUA(IOTP,YGOMV%MG) 
        YGOMUA(IMAPOMM)%LO3=LGOMUA(IOTP,YGOMV%MO3)
        YGOMUA(IMAPOMM)%LCLW=LGOMUA(IOTP,YGOMV%MCLW)
        YGOMUA(IMAPOMM)%LCIW=LGOMUA(IOTP,YGOMV%MCIW)
        YGOMUA(IMAPOMM)%LCC=LGOMUA(IOTP,YGOMV%MCC)
! ecriture de YGOMUA(IMAPOMM)%LG si JOBS=1
IF (JOBS==1) THEN
     WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LG vaut pour JOBS=1",YGOMUA(IMAPOMM)%LG
     WRITE(NULOUT,*)"et YGOMV%MG et MGOMGFL(YGOMV%MG)",YGOMV%MG, MGOMGFL(YGOMV%MG)
     WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LR vaut pour JOBS=1",YGOMUA(IMAPOMM)%LR
     WRITE(NULOUT,*)"et YGOMV%MR et MGOMGFL(YGOMV%MR)",YGOMV%MR, MGOMGFL(YGOMV%MR)
     WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LS vaut pour JOBS=1",YGOMUA(IMAPOMM)%LS
     WRITE(NULOUT,*)"et YGOMV%MS et MGOMGFL(YGOMV%MS)",YGOMV%MS, MGOMGFL(YGOMV%MS)
     WRITE(NULOUT,*)"YGOMUA(IMAPOMM)%LO3  vaut pour JOBS=1",YGOMUA(IMAPOMM)%LO3
     WRITE(NULOUT,*)"et YGOMV%MO3 et MGOMGFL(YGOMV%MO3)",YGOMV%MO3, MGOMGFL(YGOMV%MO3)
ENDIF
    CALL SET_GOM(IOTP,IMAPOMM,ZHUGE)
    IF(LOBSTL) THEN
      CALL GOMCONSTR(YGOMUA5(IMAPOMM),NFLEVG,IOTP,IMAPOMM,.TRUE.,.TRUE.)
         YGOMUA5(IMAPOMM)%LQ=LGOMUA(IOTP,YGOMV%MQ)
         YGOMUA5(IMAPOMM)%LR=LGOMUA(IOTP,YGOMV%MR)
         YGOMUA5(IMAPOMM)%LS=LGOMUA(IOTP,YGOMV%MS)
         YGOMUA5(IMAPOMM)%LG=LGOMUA(IOTP,YGOMV%MG)
         YGOMUA5(IMAPOMM)%LO3=LGOMUA(IOTP,YGOMV%MO3)
         YGOMUA5(IMAPOMM)%LCLW=LGOMUA(IOTP,YGOMV%MCLW)
         YGOMUA5(IMAPOMM)%LCIW=LGOMUA(IOTP,YGOMV%MCIW)
         YGOMUA5(IMAPOMM)%LCC=LGOMUA(IOTP,YGOMV%MCC)
    ENDIF
  ENDIF
ENDDO
DO JOBS=1,NGOMS1D
  IF(YGOMUA(JOBS)%IOBS /= JOBS) CALL ABOR1('SUGOMS FAIL 1')
ENDDO


IF(NGOMS2D > 0) THEN
  ALLOCATE(YGOMUA_2D(NGOMS2D))
  ALLOCATE(YGOMS_2D(NGOMS2D))
  ALLOCATE(YGOMEC_2D(NGOMS2D))
  IF(LOBSTL) THEN
    ALLOCATE(YGOMUA5_2D(NGOMS2D))
    ALLOCATE(YGOMS5_2D(NGOMS2D))
  ENDIF

  DO JOBS=1,NGOMS
    IOTP = ROBSU(JOBS, MDBOTP)
    IF (NOBSPROFS(IOTP) > 1) THEN
      IPROF=NOBSPROFS(IOTP)
      IMAPOMM = ROBSU(JOBS,MDB_MAPOMM_AT_INDEX)
      CALL GOMCONSTR_2D(YGOMUA_2D(IMAPOMM),YGOMS_2D(IMAPOMM),YGOMEC_2D(IMAPOMM),&
       & NFLEVG,IPROF,IOTP,IMAPOMM,.FALSE.)
       YGOMUA_2D(IMAPOMM)%LQ = LGOMUA(IOTP,YGOMV%MQ)
       YGOMUA_2D(IMAPOMM)%LR = LGOMUA(IOTP,YGOMV%MR)
       YGOMUA_2D(IMAPOMM)%LS = LGOMUA(IOTP,YGOMV%MS)
       YGOMUA_2D(IMAPOMM)%LG = LGOMUA(IOTP,YGOMV%MG)
       YGOMUA_2D(IMAPOMM)%LO3= LGOMUA(IOTP,YGOMV%MO3)
       YGOMUA_2D(IMAPOMM)%LCLW= LGOMUA(IOTP,YGOMV%MCLW)
       YGOMUA_2D(IMAPOMM)%LCIW= LGOMUA(IOTP,YGOMV%MCIW)
       YGOMUA_2D(IMAPOMM)%LCC= LGOMUA(IOTP,YGOMV%MCC)
!        WRITE(NULOUT,*)"YGOMUA_2D(IMAPOMM)%LS vaut",YGOMUA_2D(IMAPOMM)%LS,IMAPOMM
!        WRITE(NULOUT,*)"YGOMUA_2D(IMAPOMM)%LG vaut",YGOMUA_2D(IMAPOMM)%LG,IMAPOMM
      IF(LOBSTL) THEN
        CALL GOMCONSTR_2D(YGOMUA5_2D(IMAPOMM),YGOMS5_2D(IMAPOMM),YGOMEC_2D(IMAPOMM),&
         & NFLEVG,IPROF,IOTP,IMAPOMM,.TRUE.)
      ENDIF  
    ENDIF
  ENDDO
ENDIF

CALL PUTDB('SUOBSADDR', 0, IRET, INFO, 0, ZINFO, 0)

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('SUGOMS',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
END SUBROUTINE SUGOMS
