SUBROUTINE SULIM

!**** SUBROUTINE SULIM - SETUP LIMITS

!        D. VASILJEVIC    ECMWF   06/09/1994

!**   PURPOSE
!     -------

!        SETUP LIMITS

!**   INTERFACE
!     ---------

!        CALL SULIM

!        SULIM IS CALLED BY

!**   METHOD
!     ------

!        NECESSARY LIMITS ARE DEFINED.

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLIM   , ONLY : NLAT90   ,NLAT20   ,NCENYEAL ,NSTBSID1 ,&
 & NSTBSID2 ,NSTBSID3 ,NSTBSID4 ,NSTBSID5 ,NFLLIMOP ,&
 & NSYCCHA  ,RLAT90   ,RLAT20   ,RLON360  ,RLON0    ,&
 & RCONLIM  ,RP700    ,RP250    ,RERRPP   ,RPPADJ   ,&
 & RPPLMT   ,RPPDIF   ,RRHLIM   ,RRHTLIM  ,NSTBSID_TERRA, NSTBSID_AQUA 
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------
!*       1.       NUMERICAL LIMITS
!                 ----------------

!*          1.1   LAT./LON. LIMITS

IF (LHOOK) CALL DR_HOOK('SULIM',0,ZHOOK_HANDLE)
NLAT90 = 9000
RLAT90 =   90._JPRB
NLAT20 = 2000
RLAT20 =   20._JPRB
RLON360=  360._JPRB
RLON0  =    0.0_JPRB

!*          1.2   CONVERGENCE LIMIT

RCONLIM=1.0_JPRB

!*          1.3   TIME LIMITS

NCENYEAL = 25

!*          1.4   PRESSURE LIMITS

RP700    = 70000._JPRB
RP250    = 25000._JPRB

!*          1.5   SATOB STATION ID. LIMITS

NSTBSID1 = 100
NSTBSID2 = 200
NSTBSID3 = 300
NSTBSID4 = 400
NSTBSID5 = 500
NSTBSID_TERRA = 783       ! TERRA
NSTBSID_AQUA  = 784       ! AQUA

!*          1.6   FLAG LIMITS (OBSPROC USE ONLY)

NFLLIMOP(1)=   1
NFLLIMOP(2)=   2
NFLLIMOP(3)=   3

!*          1.7   RELATIVE HUMIDITY LIMITS

RRHLIM  =   0.2_JPRB
RRHTLIM = 233._JPRB

!*          1.8   MSL PRESSURE CORRECTION (DUE TO SYNOP CODE CHANGE)
!                 LIMITS

NSYCCHA = 19820430
RERRPP  =   106000._JPRB
RPPADJ  =    10000._JPRB
RPPLMT  =   100000._JPRB
RPPDIF  =     3000._JPRB
!*
!-----------------------------------------------------------------------

!        2.       SETUP DONE
!                 ----------

WRITE(NULOUT,'(''0NUMERICAL LIMITS SETUP'')')
!*
!-----------------------------------------------------------------------

!        3.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SULIM',1,ZHOOK_HANDLE)
END SUBROUTINE SULIM
