SUBROUTINE SUOBS

!**** SUBROUTINE SUOBS - SET-UP OBSERVATION PROCESSING

!          D. VASILJEVIC     ECMWF          24/1/1990

!*    METHOD
!     ------

!        ALL NECESSARY FORMATRS ARE SET FIRST, AND THEN THE READ
!     OF DATA IN MEMORY IS DONE.

!*    EXTERNALS
!     ---------

!        SUOBS IS CALLED BY SUBROUTINE CVA1

!*    EXTERNALS
!     ---------

!        SUANCT        - ANALYSIS NUMERICAL CONSTANTS
!        DEFRUN        - DEFINE RUN CONTROL PARAMETERS
!        SUVNMB        - VARIABLES' NUMBERS
!        SUSCRE0       - SET UP SCREENING
!        SUOBSORT      - SET UP ROBSERVATION SORTING
!        DEPERERR      - DEFINE OBS. PERSISTENCE ERRORS (COH-Lib)
!        PREAMB        - RUN DESCRIPTION
!        IYMD2CD       - YYYYMMDD to Centurey Date (ECLIB)
!        OBATABS       - OBSERVATION TABLES
!        SUSTDO        - SET OBS SDT ERRORS
!        SURAD         - SPECIAL SET UPS FOR TOVS RADIANCES
!        SUERCOOB      - R/S VERTICAL OBSERVATION ERROR CORRELATIONS
!        SETTC         - SYNOP/DRIBU TEMPORAL CORRELATION
!        SUREO3        - Setup bias correction fields for REO3
!        GET_RS_T_BIAS - Get RS T BIAS

!*    MODIFICATIONS
!     -------------
!     Ph. Caille   960401 : CMA files new format
!     E. Andersson 971104 : Time correlation of SYNOP/DRIBU
!     E. Andersson 971106 : Space correlation of AIREP
!     F.  Veerse 04/11/97 : SURAD called by CANARI
!     F.  Veerse 17/11/97 : Correct the call to SUVNMB
!     JN Thepaut 980826   : Move call to mkglobstab to SUOBS
!     N. Bormann 21/09/2001 : Introduce call to SUAMV to set up AMV/SATOBs
!     A. Dethof  24/04/2003 : Introduce call to SUREO3 
!     N. Bormann 23/12/2004 : Introduce call to SULIMB to set up LIMB obs

!     G. Radnoti 20060320   : Introduec SUOBSCOR to set up obs error correlation
!     C. Payan   13/04/2007 : LMKCMARPL becomes a 2D array
!     R. El Khatib 31-Aug-2007 : initialize NVCGLPC
!     K. Scipal  10/04/2008 : Introduce call to ASCATSM_CDFPAR to set up ASCAT soil moisture obs.
!     Tan/Poli   20080812   : Introduce call to SUAEOLUS

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NACTIM   ,NOBTOT
USE YOMCMA   , ONLY : LECFIO
USE YOMLUN   , ONLY : NULOUT   
USE YOMCT0   , ONLY : LCANARI  , LSCREEN, NPROC, L_SPLIT_SCREEN, L_SCREEN_CALL
USE YOMANA   , ONLY : NANDAT
USE YOMCST   , ONLY : RPI      ,RG
USE YOMVAR   , ONLY :    LAVCGL   ,LMPCGL, LAEOLUSL2BP
USE YOMVCGL  , ONLY : NVCGLPC
USE YOMECTAB , ONLY :    NTSLTOB  ,NTSLTOBOFF,NTSLTOV  ,&
 & NTSLTOVOFF,NTSLNTV  ,NTSLNTVOFF,NTSLSCA  ,NTSLSCAOFF,&
 & NTSLOTNO  
USE YOMSCC   , ONLY : LSRERUN
USE YOMOBS   , ONLY : LMKCMARPL, LMKCMARPLO, LRPLSWAPOUT, LMKCMARPL_OPENMP, &
  &                                 LRSTBIAS_ODB
USE YOMMP    , ONLY : MYPROC
USE YOMMKODB , ONLY : LSCATINT ,L_SCAT_BIASSIG ,NBIASCOR &
 & ,L_NEW_T_BIAS ,L_NEW_RH_BIAS, LASCATSM
USE YOMDB    , ONLY : NPOOLS_ECMA
USE MPL_MODULE , ONLY : MPL_END
USE OML_MOD, ONLY : OML_INIT, OML_INIT_LOCK
USE YOMCOSJO   , ONLY : LOBSCOR,LOBSCORDIAG
IMPLICIT NONE

INTEGER(KIND=JPIM) :: IANCDY &
 & ,ITSLOTNO,IOBTOT,IOFFTOT,IOBTOV,IOFFTOV &
 & ,IOBNTV,IOFFNTV,IOBSCA,IOFFSCA &
 & ,JTSLOT, JSLOT, ITSLOT,JPOOLNO, IPOOLNO  
REAL(KIND=JPRB) :: ZHOUPDAY,ZDAYPYEA
INTEGER(KIND=JPIM) :: IYMD2CD,IRET
INTEGER(KIND=JPIM) :: itime(8),imkcmarpl_Time,imkcmarpl_Date
CHARACTER(LEN=1) :: CLSTAR
INTEGER(KIND=JPIM) :: JSWEEP, I_NSWEEP, JS1, JS2
INTEGER(KIND=JPIM), ALLOCATABLE :: ISWEEP(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZHOOK_TS_LOOP
LOGICAL :: LLMKCMARPL

INTERFACE
#include "swapoutdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "defrun.intfb.h"
#include "depererr.intfb.h"
#include "iniersca.intfb.h"
#include "inissmip.intfb.h"
#include "mkcmarpl.intfb.h"
#include "mkglobstab.intfb.h"
#include "obatabs.intfb.h"
#include "preamb.intfb.h"
#include "preppcm.intfb.h"
#include "readoba.intfb.h"
#include "reini.intfb.h"
#include "setcom.intfb.h"
#include "suaeolus.intfb.h"
#include "suamv.intfb.h"
#include "suanct.intfb.h"
#include "su_errors.intfb.h"
#include "sulevlay.intfb.h"
#include "sulim.intfb.h"
#include "sulimb.intfb.h"
#include "suobarea.intfb.h"
#include "suobsort.intfb.h"
#include "surad.intfb.h"
#include "sureo3.intfb.h"
#include "susatret.intfb.h"
#include "suscre0.intfb.h"
#include "suvnmb.intfb.h"
#include "windaux.intfb.h"
#include "writeoba.intfb.h"
#include "ascatsm_cdfpar.intfb.h"

#include "suobscor.intfb.h"
!     ------------------------------------------------------------------
!*       1.       PRE-SET OBSERVATION PROCESSING
!                 ------------------------------

!*       1.0  set-up analysis constants

IF (LHOOK) CALL DR_HOOK('SUOBS',0,ZHOOK_HANDLE)
CALL SUANCT

!*       1.1   define run

CALL DEFRUN

!*       1.2   define formats/constants/limits etc.

CALL SULIM
CALL SULEVLAY
CALL SUSATRET

!*       1.2.1  set-up obs. (cma) array variables' numbers

CALL SUVNMB(NULOUT)

!*       1.2.3  set-up observation flag constants

CALL SUSCRE0

!*       1.2.4  initialize obs. sorting interface

IF (.NOT.LCANARI) CALL SUOBSORT

!*       1.2.5  initialize common obs. processing parameters

CALL SETCOM

!*       1.3   define obs. persistence errors

IF (.NOT.LCANARI) THEN
  ZHOUPDAY=24._JPRB
  IANCDY=IYMD2CD(NANDAT,IRET)
  IF(IRET /= 0) CALL ABOR1('SUOBS:CALL TO IYMD2CD FAILED')
  ZDAYPYEA=365.25_JPRB
  CALL DEPERERR(ZHOUPDAY,RPI,IANCDY,ZDAYPYEA,RG)
ENDIF

!*       1.4      INITIALIZE:
!                 1. OBS ERRORS
!                 2. SCAT PROCESSING
!                 3. SSMI PROCESSING

LLMKCMARPL=ANY(LMKCMARPL)
WRITE(NULOUT,'('' SUOBS; INITIALIZE ANY(LMKCMARPL): '',L1,1X,&
              &'',LSCATINT: '',L1,'', L_SCAT_BIASSIG: '',L1,'', NBIASCOR: '',I1)') &
              & LLMKCMARPL,LSCATINT,L_SCAT_BIASSIG,NBIASCOR
IF(ANY(LMKCMARPL))  THEN
  WRITE(NULOUT,'(''        OBS ERRORS '')')
  CALL SU_ERRORS
  IF(LSCATINT)    THEN
    WRITE(NULOUT,'(''        SCAT PROCESSING '')')
    CALL INIERSCA(NBIASCOR)
    !*        1.4.1 before getting into the ODB, read parameters for the ASCAT soil moisture transformation
    IF (LASCATSM) CALL ascatsm_cdfpar
  ENDIF
  WRITE(NULOUT,'(''        SSMI PROCESSING '')')
  CALL INISSMIP
ENDIF

!*       1.5      PRINT RUN DESCRIPTION

WRITE(NULOUT,'(''        PRINT RUN DESCRIPTION '')')
CALL PREAMB
!     ------------------------------------------------------------------
!*       2.       GET OBSERVATIONS INTO MEMORY
!                 ----------------------------

WRITE(NULOUT,'(''        GET OBSERVATIONS INTO MEMORY '')')
IF (LECFIO) THEN
!  IF(LSCREEN .AND. LSRERUN) THEN
!    CALL REINI
!  ENDIF
!ENDIF
!*          2.1   IF SCREENING
!*                1. RE-INITIALIZE FLAGS AND EVENTS IN CASE OF RE-RUN
!*                2. RUN MAKECMA REPLACEMENT
!*                3. SAVE MAKECMA REPLACEMENT PRODUCTS
!*                4. CLOSE DB

  IF(LSCREEN) THEN
    WRITE(NULOUT,'('' ** SCREENING RUN '')')

!*          2.1.1    RE-INITIALIZE FLAGS AND EVENTS IN CASE OF RE-RUN

    IF(LSRERUN) THEN
      IF(.NOT.L_SCREEN_CALL) THEN
        IF(.NOT.L_SPLIT_SCREEN) THEN
          WRITE(NULOUT,'('' ** RE-INITIALIZE FLAGS AND EVENTS IN CASE OF RE-RUN'')')
          CALL REINI
        ENDIF
      ENDIF

!*          2.1.2    RUN MAKECMA REPLACEMENT

    ELSE
      IF(ANY(LMKCMARPL))  THEN
        WRITE(NULOUT,'('' ** RUN MAKECMA REPLACEMENT '')')
        CALL DATE_AND_TIME (values=ITIME)
        imkcmarpl_Time=ITIME(5)*10000+ITIME(6)*100+ITIME(7)
        imkcmarpl_Date=ITIME(1)*10000+ITIME(2)*100+ITIME(3)
        WRITE(NULOUT,'(1X,A,I8.8,2X,I6.6)') &
         & 'MAKECMA REPLACEMENT started on ',imkcmarpl_Date,imkcmarpl_Time  

!*          2.1.2.1 GET RS T BIAS CORRECTION COEFFICIENTS IF they are
!                   in ODB type db

        IF(LRSTBIAS_ODB) THEN
          CALL GSTATS(1709,0)
          CALL GET_RS_T_BIAS
          IF(L_NEW_T_BIAS.OR.L_NEW_RH_BIAS) THEN
            CALL GET_NEW_RS_TRH_BIAS
          ENDIF
          CALL GSTATS(1709,1)
        ENDIF
        
        IF (LRPLSWAPOUT) THEN
          POOLNO_LOOP: DO JPOOLNO = MYPROC, NPOOLS_ECMA, NPROC
            TIMESLOT_LOOP: DO JSLOT = 1, NACTIM
              ITSLOT = JSLOT
              IPOOLNO = JPOOLNO
              CALL MKCMARPL(ITSLOT, IPOOLNO)
            ENDDO TIMESLOT_LOOP
            CLSTAR = '*'
            CALL SWAPOUTDB(CLSTAR,IPOOLNO,.TRUE.)
          ENDDO POOLNO_LOOP
        ELSE ! The usual way -- and a faster one
          IPOOLNO = -1

          IF (LMKCMARPL_OPENMP) THEN
            I_NSWEEP = 1
!$          CALL OML_INIT_LOCK()
          ELSE
            I_NSWEEP = NACTIM
          ENDIF

          ALLOCATE(ISWEEP(I_NSWEEP+1))
          IF (LMKCMARPL_OPENMP) THEN ! Just one sweep covers all timeslots : full OpenMP-mode
            ISWEEP(1) = 1
            ISWEEP(2) = NACTIM + 1
          ELSE                  ! One-by-one
            DO JSWEEP=1,SIZE(ISWEEP)
              ISWEEP(JSWEEP) = JSWEEP
            ENDDO
          ENDIF

          SWEEP_LOOP: DO JSWEEP=1,I_NSWEEP
            JS1 = ISWEEP(JSWEEP)
            JS2 = ISWEEP(JSWEEP+1) - 1
            CALL GSTATS(1237,0)
!$OMP PARALLEL PRIVATE(JSLOT, ITSLOT, ZHOOK_TS_LOOP)
            IF (LHOOK) CALL DR_HOOK('MKCMARPL>TS_LOOP',0,ZHOOK_TS_LOOP)
!$OMP DO SCHEDULE(DYNAMIC,1)
            TIMESLOT_LOOP2: DO JSLOT = JS1, JS2
              ITSLOT = JSLOT
              CALL MKCMARPL(ITSLOT, IPOOLNO)
            ENDDO TIMESLOT_LOOP2
!$OMP END DO
            IF (LHOOK) CALL DR_HOOK('MKCMARPL>TS_LOOP',1,ZHOOK_TS_LOOP)
!$OMP END PARALLEL
            CALL GSTATS(1237,1)
          ENDDO SWEEP_LOOP
          DEALLOCATE(ISWEEP)
        ENDIF
        CALL DATE_AND_TIME (values=ITIME)
        imkcmarpl_Time=ITIME(5)*10000+ITIME(6)*100+ITIME(7)
        imkcmarpl_Date=ITIME(1)*10000+ITIME(2)*100+ITIME(3)
        WRITE(NULOUT,'(1X,A,I8.8,2X,I6.6)') &
         & 'MAKECMA REPLACEMENT finishd on ',imkcmarpl_Date,imkcmarpl_Time  

!*          1.3.3    SAVE MAKECMA REPLACEMENT PRODUCTS

        IF (LRPLSWAPOUT) THEN
          WRITE(NULOUT,'('' * SAVE MAKECMA REPLACEMENT PRODUCTS '')')
          LSCREEN=.false. ! prevents from accidental CCMA-creation
          LAVCGL=.false.  ! keeps LLSAVE definitely .true. in WRITEOBA
          CALL WRITEOBA
          IF (.NOT.LMKCMARPLO) CALL READOBA          
        ENDIF

!*          1.3.4    CLOSE DB

        IF (LMKCMARPLO)  THEN
          WRITE(NULOUT,'('' * MAKECMA REPLACEMENT RUN ONLY; CLOSE DB '')')
          IF (.NOT.LRPLSWAPOUT) THEN
            LSCREEN=.false. ! prevents from accidental CCMA-creation
            LAVCGL=.false.  ! keeps LLSAVE definitely .true. in WRITEOBA
            CALL WRITEOBA
          ENDIF
          WRITE(NULOUT,'('' * STOP '')')
          CALL MPL_END()
          STOP 'MAKECMA REPLACEMENT RUN ONLY'
        ENDIF
      ENDIF
    ENDIF
  ENDIF
ENDIF
IF (LSCREEN) CALL WINDAUX !-- Create (ff,dd)/(dd,ff) into aux_1 & aux_2 -entries for u/v

!*       2.1   OBSERVATION POINTER TABLES
!*             WITH LMESSP=.TRUE. (AND NO CMA-I/O) IT SHOULD BE
!*             SUFFICIENT TO DO THIS ONCE ONLY! THE CALLS TO OBATABS
!*             FROM OBSVAD AND EVCOST MAY BE UN-NECESSARY.

DO JTSLOT=1,NACTIM
  NTSLOTNO=JTSLOT
  ITSLOTNO=JTSLOT
! IADDR   = NTSLPT(NTSLOTNO)
  IOBTOT  = NTSLTOB(NTSLOTNO)
  IOFFTOT = NTSLTOBOFF(NTSLOTNO)
  IOBTOV  = NTSLTOV(NTSLOTNO)
  IOFFTOV = NTSLTOVOFF(NTSLOTNO)
  IOBNTV  = NTSLNTV(NTSLOTNO)
  IOFFNTV = NTSLNTVOFF(NTSLOTNO)
  IOBSCA  = NTSLSCA(NTSLOTNO)
  IOFFSCA = NTSLSCAOFF(NTSLOTNO)
  IF(IOBTOT /= 0) THEN
    CALL OBATABS(ITSLOTNO,&
     & IOBTOT ,IOBTOV ,IOBNTV ,IOBSCA ,&
     & IOFFTOT,IOFFTOV,IOFFNTV,IOFFSCA)  
  ENDIF
ENDDO

!*       2.2      Set-up for AMV/SATOBs
CALL SUAMV
 
!*       2.3      Set-up for radiances

CALL SURAD

!*       2.4      Set-up for limb observations

CALL SULIMB

!*       2.5      Set-up observation 'area' paramter

CALL SUOBAREA(-1,NOBTOT)

!*       2.6   CREATE GLOBAL OBSERVATION TABLES

IF(LECFIO) CALL MKGLOBSTAB

!     Preconditioner for minimization, etc.

NVCGLPC=0
IF (LMPCGL) THEN
  WRITE(NULOUT,*) '---- Set up Preconditioner Matrix ---'
  CALL PREPPCM
ENDIF

!*       2.7      Set-up for ozone bias correction

CALL SUREO3

!*       2.8      Set-up for observation error correlation

IF (LOBSCOR .OR. LOBSCORDIAG) CALL SUOBSCOR

!*       2.9      Set-up for Aeolus DWL HLoS

IF (LAEOLUSL2BP) CALL SUAEOLUS

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUOBS',1,ZHOOK_HANDLE)
END SUBROUTINE SUOBS
