SUBROUTINE SUOBSADDR(KTSLOTNO)

!**** SUBROUTINE SUOBSADDR - Setup MAPOMM & ABNOBs and pass them to ODB

!     PURPOSE
!     -------
!       After SUALOBS and RD_OBS_BOXES have been called, 
!       we can set MAPOMM-array and store it to ODB's @index-table
!       as a column MDB_MAPOMM_AT_INDEX.
!       We can also calculate ABNOBs here.

!       Previously MAPOMM-vector was initialized every time OBATABS was
!       called and did not exist in ODB
!       Also, ABNOB -- already an ODB entry -- was recalculated
!       while calling ECSET

!     CALLED FROM
!     -----------
!       This routine should be called from OBADAT after RD_OBS_BOXES-call

!     INPUT ARGS
!     ----------

!     KTSLOTNO - TIMESLOT number in concern which has to be either
!                -1 i.e. ALL timeslots between 1 and NACTIM or
!                >0 i.e. a timeslot number between 1 and NACTIM

!     MODIFICATIONS
!     -------------
!        S.Saarinen 2002-06-19 : ORIGINAL (ECMWF)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      15-Feb-2005 New-style GOM arrays

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE YOMECTAB , ONLY : NTSLTOBOFF,  NTSLTOB , NOBSPROFS    
USE YOMDIMO  , ONLY : NACTIM
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE GOMS_MIX     , ONLY : NGOMS,NGOMS1D,NGOMS2D

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTSLOTNO 
INTEGER(KIND=JPIM) :: IOFFTOT(NACTIM)
INTEGER(KIND=JPIM) :: JTSLOT, JSTA, JEND, INFO(0), JOBS, ICOUNT, IUNORDERED, JTSPREV
INTEGER(KIND=JPIM) :: IRET, IEXPECT, IMAPOMM, IOTP
INTEGER(KIND=JPIM) :: ICOUNT_1D,ICOUNT_2D
LOGICAL   :: LLABORT
REAL(KIND=JPRB) :: ZINFO(0)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('SUOBSADDR',0,ZHOOK_HANDLE)
write(nulout,*)'suobsaddr: KTSLOTNO, NACTIM=',KTSLOTNO, NACTIM

IF (KTSLOTNO == -1) THEN
  JSTA = 1
  JEND = NACTIM
ELSEIF (KTSLOTNO >= 1 .AND. KTSLOTNO <= NACTIM) THEN
  JSTA = KTSLOTNO
  JEND = KTSLOTNO
ELSE
  WRITE(NULERR,*)'*** Error in SUOBSADDR: KTSLOTNO is not between [1 .. NACTIM]'
  WRITE(NULERR,*)'    KTSLOTNO, NACTIM =',KTSLOTNO, NACTIM
  CALL ABORTDB('SUOBSADDR','KTSLOTNO is not between [1 .. NACTIM]',KTSLOTNO)
ENDIF

IEXPECT = SUM(NTSLTOB(JSTA:JEND))

IRET = 0
CALL GETDB('SUOBSADDR', 0, IRET, INFO, 0, ZINFO, 0,   &! Get ROBSU
 & -1, KTSLOTNO, -1, -1, -1, -1)  

write(nulout,*)'suobsaddr: IEXPECT, IRET, JSTA, JEND=',IEXPECT, IRET, JSTA, JEND

IF (IRET /= IEXPECT) THEN
  WRITE(NULERR,*) '*** Error in SUOBSADDR: INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'
  WRITE(NULERR,*) 'For timeslot=',KTSLOTNO,' expecting =',IEXPECT,', but got =',IRET
  CALL ABORTDB('SUOBSADDR','INCONSISTENCIES FROM GETDB',IRET)
ENDIF

!-- Keep the task JOBS-driven (NOT tslot i.e. JTSLOT-driven)
!   By the way: We *DO* require that the view was tslot-ORDERED_BY in SQL

!-- (0) Check that SQL has produced timeslots between valid range [JSTA..JEND] ...
!       ... and that TSLOTs are in ascending order

ICOUNT = 0
IUNORDERED = 0
JTSPREV = 0
DO JOBS=1,IRET
  JTSLOT = ROBSU(JOBS, MDB_TSLOT_AT_INDEX)
  IF (JTSLOT < JSTA .or. JTSLOT > JEND) ICOUNT = ICOUNT + 1
  IF (JTSLOT < JTSPREV) IUNORDERED = IUNORDERED + 1
  JTSPREV = JTSLOT
ENDDO

LLABORT = ((ICOUNT > 0) .OR. (IUNORDERED > 0))

IF (ICOUNT > 0) THEN
  WRITE(NULERR,*)'*** Error in SUOBSADDR: Not all retrieved timeslots in correct range'
  WRITE(NULERR,*)'    KTSLOTNO, ICOUNT=',KTSLOTNO, ICOUNT,', RANGE from ',JSTA,' to ',JEND
ENDIF

IF (IUNORDERED > 0) THEN
  WRITE(NULERR,*)'*** Error in SUOBSADDR: Timeslots not in ascending order i.e. not sorted'
  WRITE(NULERR,*)'    KTSLOTNO, IUNORDERED=',KTSLOTNO, IUNORDERED
ENDIF

IF (LLABORT) CALL ABORTDB('SUOBSADDR',&
 & 'Not all retrieved timeslots in correct range and/or timeslot-ordered',-1)  

!-- (1) Fill in ABNOB (used to be done repeatedly in "var/ecset.F90")

DO JTSLOT=JSTA,JEND
  IOFFTOT(JTSLOT) = NTSLTOBOFF(JTSLOT)
ENDDO

write(nulout,*)'suobsaddr: IOFFTOT(JSTA:JEND) in = ',IOFFTOT(JSTA:JEND)

DO JOBS=1,IRET
  JTSLOT = ROBSU(JOBS, MDB_TSLOT_AT_INDEX)
  IOFFTOT(JTSLOT) = IOFFTOT(JTSLOT) + 1
  ROBSU(JOBS, MDB_ABNOB_AT_INDEX) = IOFFTOT(JTSLOT)
ENDDO

write(nulout,*)'suobsaddr: IOFFTOT(JSTA:JEND) out = ',IOFFTOT(JSTA:JEND)

!-- (2) Fill in MAPOMM (used to be done repeatedly in "obs_preproc/obatabs.F90")

ICOUNT = 0
ICOUNT_1D = 0
ICOUNT_2D = 0
DO JOBS=1,IRET
  IOTP = ROBSU(JOBS, MDBOTP)
  IF (NOBSPROFS(IOTP) > 1) THEN
    ICOUNT_2D = ICOUNT_2D+1
    IMAPOMM = ICOUNT_2D
  ELSE
    ICOUNT_1D = ICOUNT_1D+1
    IMAPOMM = ICOUNT_1D
  ENDIF
  ROBSU(JOBS,MDB_MAPOMM_AT_INDEX) = IMAPOMM
ENDDO

NGOMS = IRET
NGOMS1D = ICOUNT_1D
NGOMS2D = ICOUNT_2D

CALL PUTDB('SUOBSADDR', 0, IRET, INFO, 0, ZINFO, 0)

IF (ICOUNT > 0) THEN
  WRITE(NULERR,*)'*** Error in SUOBSADDR: Not all SATEMs got MAPOMM assignment correctly'
  WRITE(NULERR,*)'    KTSLOTNO, NACTIM, ICOUNT =',KTSLOTNO, NACTIM, ICOUNT
  CALL ABORTDB('SUOBSADDR','Not all SATEMs got MAPOMM assignment correctly',ICOUNT)
ENDIF
IF (LHOOK) CALL DR_HOOK('SUOBSADDR',1,ZHOOK_HANDLE)
    
END SUBROUTINE SUOBSADDR
