SUBROUTINE SUOBSCOR
!**** *SUOBSCOR*  - Initialize obs error correlation model

!     Purpose.
!     --------
!          Initialize obs error correlation model

!**   Interface.
!     ----------
!        *CALL* *SUOBSCOR

!     Method.
!     -------
!    We make a global view of observations, select correlated subsets
!    and setup correlation model. If required, a selected column
!    of the correlation matrix for each correlated block is
!    put to an odb auxilliary column for diagnostic purposes 

!     Author.
!     -------
!        Gabor Radnoti  *ECMWF*      

!     Modifications.
!     --------------
!        Original   : 2006-March-27

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARCMA    ,ONLY : JP_NUMEV
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMVNMB  , ONLY : NVNUMB
USE YOMLUN   , ONLY : NULERR,NULOUT, NULTMP
USE YOMVAR   , ONLY : NUPTRA
USE YOMCOSJO , ONLY : NBCORBLK , OBSCOR_EIGVAL,  OBSCOR_TRUN, & 
 &   OBSCOR_SCALPR,LOBSCORDIAG,NOVERTRUN,EIGMIN, &
 &   OBSLATDIAG, OBSLONDIAG, NRITZ_OBSCOR, NSMAX_OBSCOR, OBSCOR_HOR_LSCALE, OBSCOR_P_UPPER, &
 &   OBSCOR_P_LOWER, OBSCOR_AUTO_COR,NOBSCOR_INLEVS, LOBSCOR_SATOB,LOBSCOR_AMSUA, LOBSCOR_SPLIT,  &
 &   LUV_OBSCOR, L3D_OBSCOR, OBSCOR_GRID_DEF, OBS_ARR_GL2, NMASK, OBSHSCALE, &
 &   NACTIM_START, NACTIM_END, obsarrtype
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMDYN   , ONLY : TSTEP
USE YOMCT0   , ONLY : NPROC, NSTART, NSTOP
USE YOMDIMO  , ONLY : NOBTOT   ,NACTIM
USE YOMCST   , ONLY : RPI
USE YOMCOCTP , ONLY : NSATOB, NSATEM, NGTHRB
USE YOMANCS  , ONLY : NMDI, RMDI
USE YOMTVRAD , ONLY : NSATOBGRP, MSENSOR_AMSUA, NSATGRP, SATGRP_TABLE
USE MPL_MODULE, ONLY : MPL_ALLREDUCE, MPL_ALLGATHERV, MPL_BROADCAST
USE YOMDB
USE ecsort_mix   , ONLY : keysort
USE RANDOM_NUMBERS_MIX, ONLY : GAUSSIAN_DISTRIBUTION, randomNumberStream 

IMPLICIT NONE
TYPE(randomNumberStream), ALLOCATABLE :: YL_RanNumStream(:)
TYPE(obsarrtype), ALLOCATABLE :: YL_ZOBS_ARR(:),YL_ZEIGVECS_LOC(:),YL_ZEIGVALS_LOC(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZX(:),ZY(:)
INTEGER(KIND=JPIM), PARAMETER :: JPMAXCORBLK=2000
INTEGER(KIND=JPIM) :: INFO(1),IRETKODE,IRET,JOBS,JBODY,IMXBDY,ICMBDY(NOBTOT),IBODY,JEV,JEV1,IPROC,ITAG
INTEGER(KIND=JPIM) :: JMASK, JJMASK, ISUM(JPMAXCORBLK),IISUM,IOFF,IGLSUM(1),IDIMGL,IRITZ, IMASK_LOC
INTEGER(KIND=JPIM) :: IRECVCOUNTS(NPROC),JROC,JJ,IEND,ISTA,J1,J2,IUV
INTEGER(KIND=JPIM) :: INDIC,IND_LAT,IND_LON,IND_TSLOT,IND_BODY,IND_PROC,JCH,ISATID, JSATID
INTEGER(KIND=JPIM) :: IND_PRES,IND_U_OR_V,INDDIAG(6),JDIAG,IDI,JGRP,I_MKEY_NUM,IDIAG
INTEGER(KIND=JPIM) :: ISEED,ITSLOT,IKEY,ILANMAX, IPROC_LOC
INTEGER(KIND=JPIM) :: JACTIM, IACTIM, ILAYERS, JLAYERS, ISTART_AMSUA, IAMSUA, ICOUNT,ICHAN_AMSUA_LOW,ICHAN_AMSUA_HIGH
REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZOBSCORTRUN,ZEPS,ZSELFCOR
REAL(KIND=JPRB) :: Z_TSLOT_LEN,ZSCALP1,ZSCALP2,ZMIN,ZACT
REAL(KIND=JPRB), ALLOCATABLE :: ZLARR(:),ZLARR_GL(:),ZCORR(:,:),ZEIGVECS(:,:),ZEIGVALS(:), ZP_UPPER(:), ZP_LOWER(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCORR_APPROX_COL(:), ZO_ARR_GL2(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGRIDL(:,:,:),ZGRIDL_TEST(:,:,:),ZSAMPLE(:),ZSAMPLE_TEST(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZZEIGVAL
INTEGER(KIND=JPIM), ALLOCATABLE :: I_MKEY(:), IDIMGL_ARR_LOC(:), INDIC_ARR_LOC(:), IBLK_GLOB(:), ISATIDLIST_AMSUA(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: ILANMAX_ARR_LOC(:),IPROC_LOC_ARR(:),IND_BLOCK(:)
LOGICAL :: LLSKIP_SATOB,LLSKIP_AMSUA,  LL_TESTADJ_INTERP_OBS,LLFIRST, LL_THIS_SPLIT, LLMYBLOCK
CHARACTER(LEN=6) :: CLTYPE
INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE
!#include "matrixin.intfb.h"
#include "abor1.intfb.h"
#include "setran.intfb.h"
#include "gen_corr_pert.intfb.h"
#include "interp_obs.intfb.h"
#include "interp_obsad.intfb.h"
#include "obscor_lanczos.intfb.h"
#include "opk_obscor.intfb.h"
#include "suobscor_resol.intfb.h"
#include "sutrans.intfb.h"
#include "openmp_obs.h"
IF (LHOOK) CALL DR_HOOK('SUOBSCOR',0,ZHOOK_HANDLE)
! 1. Get observations from odb
!    -------------------------
IF (NUPTRA == 0) THEN 
  CALL GSTATS(1741,0)
  CALL GETDB('SUOBSCOR', 0, IRETKODE, INFO, 1,ZINFO,0, &
   & -1, -1, -1, -1, -1, -1)
  CALL GSTATS(1741,1)
  IF (IRETKODE /= NOBTOT) THEN
    WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
    WRITE(NULERR,*)'Expecting NOBTOT=',NOBTOT,', but got =',IRETKODE
    CALL ABORTDB('SUOBSCOR','INCONSISTENCIES FROM GETDB',IRETKODE)
  ENDIF
  DO JOBS = 1,NOBTOT
    ICMBDY(JOBS)=MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  ENDDO
  IMXBDY=MAXVAL(ICMBDY(1:IRETKODE))
! 2. Dividing observations into correlated blocks
!    --------------------------------------------
!    THIS IS THE SECTION TO BE EXTENDED WHEN 
!    NEW CORRELATED OBS_TYPE INTRODUCED...


  ISUM(:)=0
! iactim: number of timeslots treated within correlation block
  IACTIM=NACTIM
!  IF (LOBSCOR_SPLIT) IACTIM=2
  IF (LOBSCOR_SPLIT) THEN
    ILAYERS=2
    ALLOCATE(ZP_UPPER(ILAYERS))
    ALLOCATE(ZP_LOWER(ILAYERS))
    ZP_UPPER(1:ILAYERS)=(/0.,52000./)
    ZP_LOWER(1:ILAYERS)=(/52000.,102000./)
  ELSE
    ILAYERS=1   
    ALLOCATE(ZP_UPPER(ILAYERS))
    ALLOCATE(ZP_LOWER(ILAYERS))
    ZP_UPPER(1)=20000.
    ZP_LOWER(1)=70000.
  ENDIF 
  ALLOCATE(NACTIM_START(JPMAXCORBLK))
  ALLOCATE(NACTIM_END(JPMAXCORBLK))
  ALLOCATE(OBSCOR_P_UPPER(JPMAXCORBLK))
  ALLOCATE(OBSCOR_P_LOWER(JPMAXCORBLK))
  NACTIM_START(:)=1
  NACTIM_END(:)=NACTIM
  OBSCOR_P_UPPER(:)=20000.
  OBSCOR_P_LOWER(:)=70000.
  ICHAN_AMSUA_LOW=5
  ICHAN_AMSUA_HIGH=14
  IAMSUA=COUNT(SATGRP_TABLE(1:NSATGRP)%sensor == MSENSOR_AMSUA)
  WRITE(NULOUT,*) 'NUMBER OF SATIDs WITH AMSUA:',IAMSUA 
  ALLOCATE(ISATIDLIST_AMSUA(IAMSUA))
  ISATIDLIST_AMSUA(:)=-999
  ICOUNT=0
  DO JGRP=1,NSATGRP
   IF (SATGRP_TABLE(JGRP)%sensor == MSENSOR_AMSUA) THEN   
     ICOUNT=ICOUNT+1
     ISATIDLIST_AMSUA(ICOUNT)=SATGRP_TABLE(JGRP)%bufrid
   ENDIF
  ENDDO 
  WRITE(NULOUT,*) 'SATIDs WITH AMSUA:',ISATIDLIST_AMSUA
  ISTART_AMSUA=-999
  DO JBODY=1,IMXBDY
    DO JOBS = 1,NOBTOT  
      IF(JBODY > ICMBDY(JOBS)) CYCLE
      IBODY = MLNKH2B(JOBS)+(JBODY-1) 
      ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT)=NMDI
      DO JEV=1,JP_NUMEV
        ROBODY(IBODY,MDB_OBSCOREV_AT_ERRSTAT(JEV))=RMDI
      ENDDO
!       Extra aux-tables are used for diagnostic purposes... 
      DO JDIAG=1,6
        IDI=3+JDIAG
        ROBODY(IBODY,MDB_AUX_AT_BODY(IDI))=RMDI
      ENDDO
      NBCORBLK=0
!     Satob winds:
!     If correlated SATOB winds not wanted,  LLSKIP_SATOB=true below...
      LLSKIP_SATOB= (ROBODY(IBODY,MDBVAR) == RMDI .OR. ROBHDR(JOBS,MDBLON) == RMDI .OR. &
      &           ROBHDR(JOBS,MDBLAT) == RMDI .OR. &
      &           ROBODY(IBODY,MDBFOE) == RMDI .OR. ROBODY(IBODY,MDBPPP) == RMDI)
      IF (.NOT. LOBSCOR_SATOB) LLSKIP_SATOB=.TRUE.
      DO JGRP=1,NSATOBGRP
        DO JLAYERS=1,ILAYERS
        DO JACTIM=1,NACTIM/IACTIM
          NBCORBLK=NBCORBLK+1
          IF (NBCORBLK > JPMAXCORBLK) THEN
            WRITE(NULOUT,*) 'TOO MANY OBSCOR BLOCKS'
            CALL ABOR1('TOO MANY OBSCOR BLOCKS')
          ENDIF
          IF (ISUM(NBCORBLK) == 0) THEN
            NACTIM_START(NBCORBLK)=IACTIM*(JACTIM-1)+1
            NACTIM_END(NBCORBLK)=IACTIM*JACTIM
            IF (JACTIM == NACTIM/IACTIM) NACTIM_END(NBCORBLK)=NACTIM
            OBSCOR_P_UPPER(NBCORBLK)=ZP_UPPER(JLAYERS)
            OBSCOR_P_LOWER(NBCORBLK)=ZP_LOWER(JLAYERS)
          ENDIF
          IF (ROBHDR(JOBS,MDBOTP) == NSATOB .AND. JGRP == ROBHDR(JOBS,MDB_AREATYPE_AT_HDR) .AND. &
          & (ROBODY(IBODY,MDBVNM) == NVNUMB(1) &
          & .OR.ROBODY(IBODY,MDBVNM) == NVNUMB(2))  .AND. .NOT.LLSKIP_SATOB) THEN 
             LL_THIS_SPLIT=.TRUE.
             LL_THIS_SPLIT=LL_THIS_SPLIT .AND. &   
          &  ROBHDR(JOBS,MDB_TIMESLOT_AT_TIMESLOT_INDEX) >= NACTIM_START(NBCORBLK) .AND. &
          &  ROBHDR(JOBS,MDB_TIMESLOT_AT_TIMESLOT_INDEX) <= NACTIM_END(NBCORBLK)
             IF (ILAYERS > 1) THEN
               LL_THIS_SPLIT=LL_THIS_SPLIT .AND. &
          &    ROBODY(IBODY,MDBPPP) >= OBSCOR_P_UPPER(NBCORBLK) .AND. &
          &    ROBODY(IBODY,MDBPPP) < OBSCOR_P_LOWER(NBCORBLK) 
             ENDIF
             IF (LL_THIS_SPLIT) THEN
                     ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT)=NBCORBLK
                     ISUM(NBCORBLK)=ISUM(NBCORBLK)+1
             ENDIF
          ENDIF 
        ENDDO
        ENDDO
      ENDDO
!     end SATOB
!     AMSUA radiances:
      IF (ISTART_AMSUA == -999) ISTART_AMSUA = NBCORBLK+1
!     If correlated AMSUA radiances not wanted,  LLSKIP_AMSUA=true below...
      LLSKIP_AMSUA= (ROBODY(IBODY,MDBVAR) == RMDI .OR. ROBHDR(JOBS,MDBLON) == RMDI .OR. &
      &           ROBHDR(JOBS,MDBLAT) == RMDI .OR. &
      &           ROBODY(IBODY,MDBFOE) == RMDI .OR. ROBODY(IBODY,MDBPPP) == RMDI .OR. ROBODY(IBODY,MDBFOE) > 33.) 
      IF (.NOT. LOBSCOR_AMSUA) LLSKIP_AMSUA=.TRUE.
      DO JSATID=1,IAMSUA
      DO JCH=ICHAN_AMSUA_LOW,ICHAN_AMSUA_HIGH 
      DO JACTIM=1,NACTIM/IACTIM     
        NBCORBLK=NBCORBLK+1              
        IF (NBCORBLK > JPMAXCORBLK) THEN
          WRITE(NULOUT,*) 'TOO MANY OBSCOR BLOCKS'
          CALL ABOR1('TOO MANY OBSCOR BLOCKS')
        ENDIF
        IF (ISUM(NBCORBLK) == 0) THEN
            NACTIM_START(NBCORBLK)=IACTIM*(JACTIM-1)+1
            NACTIM_END(NBCORBLK)=IACTIM*JACTIM
            IF (JACTIM == NACTIM/IACTIM) NACTIM_END(NBCORBLK)=NACTIM
        ENDIF
        LL_THIS_SPLIT= ROBHDR(JOBS,MDB_TIMESLOT_AT_TIMESLOT_INDEX) >= NACTIM_START(NBCORBLK) .AND. & 
          &  ROBHDR(JOBS,MDB_TIMESLOT_AT_TIMESLOT_INDEX) <= NACTIM_END(NBCORBLK)
        IF (ROBHDR(JOBS,MDBOTP) == NSATEM) THEN
          IF (ROBHDR(JOBS,MDB_CODETYPE_AT_HDR) == NGTHRB) THEN
            IF (ROBHDR(JOBS,MDBSSIA) == MSENSOR_AMSUA) THEN 
              ISATID=ROBHDR(JOBS,MDB_SATID_AT_SAT)
              IF (ISATID == ISATIDLIST_AMSUA(JSATID) .AND. &
                &  NINT(ROBODY(IBODY,MDBPPP)) == JCH .AND. &
                &  .NOT. LLSKIP_AMSUA .AND. LL_THIS_SPLIT) THEN

                ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT)=NBCORBLK
                ISUM(NBCORBLK)=ISUM(NBCORBLK)+1

              ENDIF 
            ENDIF
          ENDIF
        ENDIF
      ENDDO
      ENDDO
      ENDDO
!     end AMSUA
    ENDDO
  ENDDO
  DEALLOCATE(ISATIDLIST_AMSUA)
  ALLOCATE(OBSCOR_HOR_LSCALE(NBCORBLK))
  ALLOCATE(OBSCOR_AUTO_COR(NBCORBLK))
  ALLOCATE(NOBSCOR_INLEVS(NBCORBLK))
  ALLOCATE(LUV_OBSCOR(NBCORBLK))
  ALLOCATE(L3D_OBSCOR(NBCORBLK))
  ALLOCATE(IPROC_LOC_ARR(NBCORBLK))
  ALLOCATE(IND_BLOCK(NBCORBLK))
  ALLOCATE(IDIMGL_ARR_LOC(1+NBCORBLK/NPROC))
  ALLOCATE(INDIC_ARR_LOC(1+NBCORBLK/NPROC))
  ALLOCATE(ILANMAX_ARR_LOC(1+NBCORBLK/NPROC))
  ALLOCATE(IBLK_GLOB(1+NBCORBLK/NPROC))
  ALLOCATE(YL_ZOBS_ARR(1+NBCORBLK/NPROC))
  ALLOCATE(YL_ZEIGVECS_LOC(1+NBCORBLK/NPROC))
  ALLOCATE(YL_ZEIGVALS_LOC(1+NBCORBLK/NPROC))
  IDIMGL_ARR_LOC(:)=0
  INDIC_ARR_LOC(:)=0
  IBLK_GLOB(:)=0
  IPROC_LOC_ARR(:)=0
  IND_BLOCK(:)=0
  IMASK_LOC=0
  DO JMASK=1,ISTART_AMSUA-1
  ! SATOB WINDS
  ! the params below may depend on obs type/instrument
    OBSCOR_HOR_LSCALE(JMASK)= OBSHSCALE
    OBSCOR_AUTO_COR(JMASK)      = 5400.0_JPRB    !  decorrelation time coeff (sec)
    NOBSCOR_INLEVS(JMASK)       = 2              ! No. of vertical levels for interpolation
    LUV_OBSCOR(JMASK)=.TRUE.  ! Satob wind
    L3D_OBSCOR(JMASK)=.TRUE.
  ENDDO
  DO JMASK=ISTART_AMSUA,NBCORBLK
  ! AMSUA RADIANCES 
  ! the params below may depend on obs type/instrument
    OBSCOR_HOR_LSCALE(JMASK)= OBSHSCALE
    OBSCOR_AUTO_COR(JMASK)      = 2700.0_JPRB    !  decorrelation time coeff (sec)
    NOBSCOR_INLEVS(JMASK)       = 1              ! No. of vertical levels for interpolation
    LUV_OBSCOR(JMASK)=.FALSE.  ! Satob wind
    L3D_OBSCOR(JMASK)=.FALSE.
  ENDDO
  LL_TESTADJ_INTERP_OBS=.TRUE. ! only for validation purposes (adjoint test of interpolator)
!   3. Preparations
!      ------------
!   Obs correlation model on a special grid resolution
  CALL SUOBSCOR_RESOL(NSMAX_OBSCOR)
  Z_TSLOT_LEN = REAL(NSTOP-NSTART,JPRB)*TSTEP/REAL(NACTIM-1,JPRB)
  ALLOCATE(OBSCOR_EIGVAL(JP_NUMEV,NBCORBLK))
  ALLOCATE(OBSCOR_TRUN(NBCORBLK))
  ALLOCATE(OBSCOR_SCALPR(JP_NUMEV,NBCORBLK))
  ALLOCATE(NRITZ_OBSCOR(NBCORBLK))
  NRITZ_OBSCOR(1:NBCORBLK)=JP_NUMEV/NOVERTRUN
  IF (NBCORBLK > 1) THEN
    ISEED=135797531
    ALLOCATE(YL_RanNumStream(1))
    CALL SETRAN(ISEED,YL_RanNumStream(1))
  ENDIF
!   obs characteristics to carry along for setting up correlations in a reproducible manner:
  IND_LAT=1
  IND_LON=2
  IND_TSLOT=3
  IND_BODY=4
  IND_PROC=5
 
!   4. Processing correlated blocks
!    ----------------------------
 
  LLFIRST=.TRUE.
  IPROC_LOC=0
  MASK_LOOP: DO JMASK=1,NBCORBLK

!   4.1 Correlation block dependent setup 
!     ---------------------------------
       
    ILANMAX=JP_NUMEV+90 ! +90 because not every iteration gives a useful eigen-pair
    INDIC=5 ! at least 5 indicators used
    IND_U_OR_V=-9999
    IND_PRES=-9999
!   jmask=1: Ps (only for validation purposes)
!   jmask=2...NBCORBLK : SATOB winds divided into groups, order: u,v(1),...,u,v(nsatobgrp)   
    IF (LUV_OBSCOR(JMASK)) THEN
      INDIC=INDIC+1 !u or v  
      IND_U_OR_V=INDIC
    ENDIF
    IF (L3D_OBSCOR(JMASK)) THEN
      INDIC=INDIC+1 !pressure of obs
      IND_PRES=INDIC
    ENDIF
!   4.2 Taking useful obs info out, gathering and ordering for reproducibility: all into YL_ZOBS_ARR(:)%OBS_ARR_GL2(:,:)
!       -------------------------------------------------------------------------------------------------------------

    IF (ALLOCATED(ZLARR)) DEALLOCATE(ZLARR)
    ALLOCATE(ZLARR(ISUM(JMASK)*INDIC))
    IGLSUM(1) = ISUM(JMASK)
    CALL GSTATS(817,0)
    CALL MPL_ALLREDUCE(IGLSUM, 'SUM',CDSTRING='SUOBSCOR:')
    IDIMGL = IGLSUM(1)
    IF (ALLOCATED(ZLARR_GL)) DEALLOCATE(ZLARR_GL)
    ALLOCATE(ZLARR_GL(IDIMGL*INDIC))
    IRECVCOUNTS(:) = 0
    CALL MPL_ALLGATHERV(SIZE(ZLARR), IRECVCOUNTS,CDSTRING='SUOBSCOR:')
    CALL GSTATS(817,1)
    IISUM=0
    DO JBODY=1,IMXBDY
      DO JOBS = 1,NOBTOT
        IF(JBODY > ICMBDY(JOBS)) CYCLE
        IBODY = MLNKH2B(JOBS)+(JBODY-1)
!       preventing idimgl <= jp_numev
        IF (IDIMGL <= ILANMAX .AND. NINT(ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT)) == JMASK) THEN
           ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT) = NMDI
        ENDIF      

        IF (NINT(ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT)) == JMASK) THEN
          IISUM=IISUM+1
          IOFF=(IND_LAT-1)*ISUM(JMASK)
          ZLARR(IOFF+IISUM)=ROBHDR(JOBS,MDBLAT)
          IOFF=(IND_LON-1)*ISUM(JMASK)
          ZLARR(IOFF+IISUM)=ROBHDR(JOBS,MDBLON)
          ITSLOT = ROBHDR(JOBS,MDB_TIMESLOT_AT_TIMESLOT_INDEX)
          IOFF=(IND_TSLOT-1)*ISUM(JMASK)
          ZLARR(IOFF+IISUM)=ITSLOT
          IOFF=(IND_BODY-1)*ISUM(JMASK) 
          ZLARR(IOFF+IISUM)=IBODY
          IOFF=(IND_PROC-1)*ISUM(JMASK)
          ZLARR(IOFF+IISUM)=NPRCIDS(MYPROC)
          IF (LUV_OBSCOR(JMASK)) THEN
            IOFF=(IND_U_OR_V-1)*ISUM(JMASK)
            ZLARR(IOFF+IISUM)=ROBODY(IBODY,MDBVNM)      
          ENDIF
          IF (L3D_OBSCOR(JMASK)) THEN
            IOFF=(IND_PRES-1)*ISUM(JMASK)
            ZLARR(IOFF+IISUM)=ROBODY(IBODY,MDBPPP)
          ENDIF
        ENDIF
      ENDDO
    ENDDO
    IF (IDIMGL <= ILANMAX) THEN
       NRITZ_OBSCOR(JMASK)=0
       CYCLE MASK_LOOP 
    ENDIF
    CALL GSTATS(817,0)
    CALL MPL_ALLGATHERV(ZLARR, ZLARR_GL, IRECVCOUNTS,CDSTRING='SUOBSCOR:')    
    CALL GSTATS(817,1)
    IF (ALLOCATED(ZLARR)) DEALLOCATE(ZLARR)
    ALLOCATE(ZO_ARR_GL2(IDIMGL,INDIC))
    JJ = 0
    IEND = 0
    DO JROC = 1,NPROC
      ISTA = IEND + 1
      IEND = IEND + IRECVCOUNTS(JROC)/INDIC
      DO J2 = 1, INDIC
        DO J1 = ISTA, IEND
          JJ = JJ + 1
          ZO_ARR_GL2(J1,J2) = ZLARR_GL(JJ)
        ENDDO
      ENDDO
    ENDDO
    I_MKEY_NUM=3 ! at least 3 indicators for unique ordering
    IF (LUV_OBSCOR(JMASK)) I_MKEY_NUM=I_MKEY_NUM+1 ! u or v
    IF (L3D_OBSCOR(JMASK)) I_MKEY_NUM=I_MKEY_NUM+1 ! pressure
    ALLOCATE(I_MKEY(I_MKEY_NUM))
    IKEY=0
    IF (LUV_OBSCOR(JMASK)) THEN
      IKEY=IKEY+1
      I_MKEY(IKEY)=IND_U_OR_V   ! u or v (e.g. for satob winds)
    ENDIF
    IF (L3D_OBSCOR(JMASK)) THEN
      IKEY=IKEY+1
      I_MKEY(IKEY)=IND_PRES
    ENDIF
    IKEY=IKEY+1
    I_MKEY(IKEY)=IND_LAT    !latitude
    IKEY=IKEY+1
    I_MKEY(IKEY)=IND_LON    !longitude
    IKEY=IKEY+1
    I_MKEY(IKEY)=IND_TSLOT   !time-slot No.
    CALL KEYSORT(IRET, ZO_ARR_GL2, IDIMGL, MULTIKEY=I_MKEY)
!   check if U set matches V set: 
    IF (LUV_OBSCOR(JMASK)) THEN
      DO J1=1,IDIMGL/2
        IF ( ZO_ARR_GL2(J1,IND_U_OR_V) /= NVNUMB(1)) THEN
          WRITE(NULERR,*) 'index<=idimgl/2 wind U is not U'
          CALL ABOR1('index<=idimgl/2 wind U is not U')      
        ENDIF
        IF ( ZO_ARR_GL2(IDIMGL/2+J1,IND_U_OR_V) /= NVNUMB(2)) THEN
          WRITE(NULERR,*) 'index>idimgl/2 wind-V is not V'
          CALL ABOR1('index>idimgl/2 wind-V is not V')      
        ENDIF
        IF (ZO_ARR_GL2(J1,IND_LAT) /= ZO_ARR_GL2(IDIMGL/2+J1,IND_LAT)) THEN
          WRITE(NULERR,*) 'latitude mismatch in wind U-V',ZO_ARR_GL2(J1,IND_LAT),ZO_ARR_GL2(IDIMGL/2+J1,IND_LAT)
          CALL ABOR1('latitude mismatch in wind U-V')
        ENDIF
        IF (ZO_ARR_GL2(J1,IND_LON) /= ZO_ARR_GL2(IDIMGL/2+J1,IND_LON)) THEN
          WRITE(NULERR,*) 'longitude mismatch in wind U-V',ZO_ARR_GL2(J1,IND_LON),ZO_ARR_GL2(IDIMGL/2+J1,IND_LON)
          CALL ABOR1('longitude mismatch in wind U-V')
        ENDIF
        IF (L3D_OBSCOR(JMASK)) THEN 
          IF (ZO_ARR_GL2(J1,IND_PRES) /= ZO_ARR_GL2(IDIMGL/2+J1,IND_PRES)) THEN
            WRITE(NULERR,*) 'pressure mismatch in wind U-V',ZO_ARR_GL2(J1,IND_PRES),ZO_ARR_GL2(IDIMGL/2+J1,IND_PRES)
            CALL ABOR1('pressure mismatch in wind U-V')
          ENDIF
        ENDIF
        IF (ZO_ARR_GL2(J1,IND_TSLOT) /= ZO_ARR_GL2(IDIMGL/2+J1,IND_TSLOT)) THEN
          WRITE(NULERR,*) 'timeslot mismatch in wind U-V',ZO_ARR_GL2(J1,IND_TSLOT),ZO_ARR_GL2(IDIMGL/2+J1,IND_TSLOT)
          CALL ABOR1('timeslot mismatch in wind U-V')
        ENDIF
      ENDDO
    ENDIF
    IF (ALLOCATED(I_MKEY)) DEALLOCATE(I_MKEY) 
    IF (ALLOCATED(ZLARR_GL)) DEALLOCATE(ZLARR_GL)
! Given block will belong to  proc IPROC_LOC
    IPROC_LOC=IPROC_LOC+1
    IF (IPROC_LOC > NPROC) IPROC_LOC=1
    IPROC_LOC_ARR(JMASK)=IPROC_LOC
    IF (MYPROC == IPROC_LOC) THEN
      IMASK_LOC=IMASK_LOC+1
      IF (IMASK_LOC > 1+NBCORBLK/NPROC) CALL ABOR1('SUOBSCOR: too many blocks')
      IDIMGL_ARR_LOC(IMASK_LOC)=IDIMGL
      INDIC_ARR_LOC(IMASK_LOC)=INDIC
      ILANMAX_ARR_LOC(IMASK_LOC)=ILANMAX
      IBLK_GLOB(IMASK_LOC)=JMASK
      IND_BLOCK(JMASK)=IMASK_LOC
      ALLOCATE(YL_ZOBS_ARR(IMASK_LOC)%OBS_ARR_GL2(IDIMGL,INDIC))
      YL_ZOBS_ARR(IMASK_LOC)%OBS_ARR_GL2(:,:)=ZO_ARR_GL2(:,:)
    ENDIF
    DEALLOCATE(ZO_ARR_GL2)
  ENDDO MASK_LOOP
  MASK_LOOP_LOC: DO JJMASK=1,IMASK_LOC
    JMASK=IBLK_GLOB(JJMASK)
    IDIMGL=IDIMGL_ARR_LOC(JJMASK)
    INDIC=INDIC_ARR_LOC(JJMASK)
    ILANMAX=ILANMAX_ARR_LOC(JJMASK) 
    ALLOCATE(OBS_ARR_GL2(IDIMGL,INDIC))
    OBS_ARR_GL2(:,:)=YL_ZOBS_ARR(JJMASK)%OBS_ARR_GL2(:,:)
!   4.3 Computing correlations and eigen-vectors
!       ----------------------------------------
    ALLOCATE(ZEIGVECS(IDIMGL,ILANMAX))
    ALLOCATE(ZEIGVALS(ILANMAX))
!
!   4.3 Correlations as a sequence of linear transf-s (convolution), eig.-sys. by Lanczos alg.
!         ---------------------------------------------------------------------------------------

!   for SATOB winds we may use Lanczos algorithm
    IF (LUV_OBSCOR(JMASK)) THEN
      IUV=2
      CLTYPE='ROTUV'
    ELSE
      CLTYPE='SCALAR'
      IUV=1
    ENDIF
! Built-in adjoint test for  interpolator
!   IF (LL_TESTADJ_INTERP_OBS .AND. LLFIRST) THEN
    IF (LL_TESTADJ_INTERP_OBS) THEN
      ALLOCATE(ZGRIDL(OBSCOR_GRID_DEF%NGPTOT,(NACTIM_END(JMASK)-NACTIM_START(JMASK)+1)*NOBSCOR_INLEVS(JMASK)*IUV,1))
      ALLOCATE(ZGRIDL_TEST(OBSCOR_GRID_DEF%NGPTOT,(NACTIM_END(JMASK)-NACTIM_START(JMASK)+1)*NOBSCOR_INLEVS(JMASK)*IUV,1))
      ALLOCATE(ZSAMPLE(IDIMGL))
      ALLOCATE(ZSAMPLE_TEST(IDIMGL))
      ISEED=135797531+JMASK
      IF (ALLOCATED(YL_RanNumStream)) DEALLOCATE(YL_RanNumStream)
      ALLOCATE(YL_RanNumStream(1))
      CALL SETRAN(ISEED,YL_RanNumStream(1))
      CALL GAUSSIAN_DISTRIBUTION(ZSAMPLE_TEST,YL_RanNumStream(1))
      ZSAMPLE(:)=999999._JPRB
      CALL GEN_CORR_PERT (ZGRIDL,OBSCOR_HOR_LSCALE(JMASK),TRIM(CLTYPE), &
           & YL_RanNumStream(1),LD_USE_REDUCED_RESOL=.TRUE.)
      IF (L3D_OBSCOR(JMASK)) THEN
        IND_PRES=6
        IF (LUV_OBSCOR(JMASK)) IND_PRES=7
        CALL INTERP_OBS(JMASK,ZGRIDL,OBS_ARR_GL2(1:IDIMGL/IUV,IND_LAT),OBS_ARR_GL2(1:IDIMGL/IUV,IND_LON), &
           & OBS_ARR_GL2(1:IDIMGL/IUV,IND_TSLOT), &
           & ZSAMPLE,IDIMGL/IUV,NOBSCOR_INLEVS(JMASK),IUV, &
           & PPRES=OBS_ARR_GL2(1:IDIMGL/IUV,IND_PRES))
        CALL INTERP_OBSAD(JMASK,ZGRIDL_TEST,OBS_ARR_GL2(1:IDIMGL/IUV,IND_LAT),OBS_ARR_GL2(1:IDIMGL/IUV,IND_LON), &
           & OBS_ARR_GL2(1:IDIMGL/IUV,IND_TSLOT), &
           & ZSAMPLE_TEST,IDIMGL/IUV,NOBSCOR_INLEVS(JMASK),IUV, &
           & PPRES=OBS_ARR_GL2(1:IDIMGL/IUV,IND_PRES))
      ELSE
        CALL INTERP_OBS(JMASK,ZGRIDL,OBS_ARR_GL2(1:IDIMGL/IUV,IND_LAT),OBS_ARR_GL2(1:IDIMGL/IUV,IND_LON), &
           & OBS_ARR_GL2(1:IDIMGL/IUV,IND_TSLOT), &
           & ZSAMPLE,IDIMGL/IUV,NOBSCOR_INLEVS(JMASK),IUV)
        CALL INTERP_OBSAD(JMASK,ZGRIDL_TEST,OBS_ARR_GL2(1:IDIMGL/IUV,IND_LAT),OBS_ARR_GL2(1:IDIMGL/IUV,IND_LON), &
           & OBS_ARR_GL2(1:IDIMGL/IUV,IND_TSLOT), &
           & ZSAMPLE_TEST,IDIMGL/IUV,NOBSCOR_INLEVS(JMASK),IUV)
      ENDIF
      ZSCALP1=0._JPRB
      DO J1=1,IDIMGL
        ZSCALP1=ZSCALP1+ZSAMPLE(J1)*ZSAMPLE_TEST(J1)
      ENDDO
      ZSCALP2=0._JPRB
      DO J1=1,OBSCOR_GRID_DEF%NGPTOT
      DO J2=1,(NACTIM_END(JMASK)-NACTIM_START(JMASK)+1)*NOBSCOR_INLEVS(JMASK)*IUV
        ZSCALP2=ZSCALP2+ZGRIDL(J1,J2,1)*ZGRIDL_TEST(J1,J2,1)
      ENDDO
      ENDDO
      WRITE(NULERR,*) 'TEST OF ADJOINT OF INTERP_OBS'
      WRITE(NULERR,*) ' JMASK= ',JMASK
      WRITE(NULERR,*) ' <Lx,y>= ',ZSCALP1
      WRITE(NULERR,*) ' <x,L*y>= ',ZSCALP2
      DEALLOCATE(ZGRIDL)
      DEALLOCATE(ZGRIDL_TEST)
      DEALLOCATE(ZSAMPLE)
      DEALLOCATE(ZSAMPLE_TEST)
      LLFIRST=.FALSE.
    ENDIF
    NMASK=JMASK
    ISEED=135797531+JMASK
    IF (ALLOCATED(YL_RanNumStream)) DEALLOCATE(YL_RanNumStream)
    ALLOCATE(YL_RanNumStream(1))
    CALL SETRAN(ISEED,YL_RanNumStream(1))
    CALL OBSCOR_LANCZOS( &
    & ZEIGVECS,ZEIGVALS,IRITZ,ILANMAX,YL_RanNumStream(1))
    WRITE(NULOUT,*) 'NUMBER OF CONVERGED RITZ VALUES AT JMASK=',JMASK,'IS ',IRITZ
    NRITZ_OBSCOR(JMASK)=MIN(IRITZ,NRITZ_OBSCOR(JMASK)) 
    ZEIGVALS(1:NRITZ_OBSCOR(JMASK))=EIGMIN+(1.0_JPRB-EIGMIN)*ZEIGVALS(1:NRITZ_OBSCOR(JMASK))
    ALLOCATE(YL_ZEIGVECS_LOC(JJMASK)%OBS_ARR_GL2(IDIMGL,NRITZ_OBSCOR(JMASK)))
    ALLOCATE(YL_ZEIGVALS_LOC(JJMASK)%OBS_ARR_GL2(1,NRITZ_OBSCOR(JMASK)))
    YL_ZEIGVECS_LOC(JJMASK)%OBS_ARR_GL2(1:IDIMGL,1:NRITZ_OBSCOR(JMASK))=ZEIGVECS(1:IDIMGL,1:NRITZ_OBSCOR(JMASK)) 
    YL_ZEIGVALS_LOC(JJMASK)%OBS_ARR_GL2(1,1:NRITZ_OBSCOR(JMASK))=ZEIGVALS(1:NRITZ_OBSCOR(JMASK))
    IF (ALLOCATED(ZEIGVECS)) DEALLOCATE(ZEIGVECS)
    IF (ALLOCATED(ZEIGVALS)) DEALLOCATE(ZEIGVALS)
    IF (ALLOCATED(OBS_ARR_GL2)) DEALLOCATE(OBS_ARR_GL2)
    IF (ALLOCATED(ZSAMPLE)) DEALLOCATE(ZSAMPLE)
    IF (ALLOCATED(ZCORR)) DEALLOCATE(ZCORR)
  ENDDO MASK_LOOP_LOC
  DEALLOCATE(ILANMAX_ARR_LOC)
  DEALLOCATE(IBLK_GLOB)
  MASK_LOOP_GLOB: DO JMASK=1,NBCORBLK 
    WRITE(NULOUT,*) 'JMASK=',JMASK,' BELONGS TO PROC ', IPROC_LOC_ARR(JMASK) 
    IF (IPROC_LOC_ARR(JMASK) == 0) CYCLE MASK_LOOP_GLOB    
    LLMYBLOCK=(IPROC_LOC_ARR(JMASK)==MYPROC)
    IF (LLMYBLOCK) THEN
      IDIMGL=IDIMGL_ARR_LOC(IND_BLOCK(JMASK))
      IRITZ=NRITZ_OBSCOR(JMASK)
      INDIC=INDIC_ARR_LOC(IND_BLOCK(JMASK))
    ENDIF
    IF (NPROC >1 ) THEN
      ITAG=78346+JMASK*1000
      CALL MPL_BROADCAST(IDIMGL,ITAG,IPROC_LOC_ARR(JMASK),CDSTRING='SUOBSCOR:') 
      ITAG=ITAG+1
      CALL MPL_BROADCAST(IRITZ,ITAG,IPROC_LOC_ARR(JMASK),CDSTRING='SUOBSCOR:')
      ITAG=ITAG+1
      CALL MPL_BROADCAST(INDIC,ITAG,IPROC_LOC_ARR(JMASK),CDSTRING='SUOBSCOR:')
    ENDIF
    NRITZ_OBSCOR(JMASK)=IRITZ    
    ALLOCATE(ZEIGVECS(IDIMGL,NRITZ_OBSCOR(JMASK)))
    ALLOCATE(ZEIGVALS(NRITZ_OBSCOR(JMASK)))
    IF (ALLOCATED(OBS_ARR_GL2)) DEALLOCATE(OBS_ARR_GL2)
    ALLOCATE(OBS_ARR_GL2(IDIMGL,INDIC))
    IF (LLMYBLOCK) THEN
      ZEIGVECS(1:IDIMGL,1:NRITZ_OBSCOR(JMASK))=YL_ZEIGVECS_LOC(IND_BLOCK(JMASK))%OBS_ARR_GL2(1:IDIMGL,1:NRITZ_OBSCOR(JMASK))
      ZEIGVALS(1:NRITZ_OBSCOR(JMASK))=YL_ZEIGVALS_LOC(IND_BLOCK(JMASK))%OBS_ARR_GL2(1,1:NRITZ_OBSCOR(JMASK))
      OBS_ARR_GL2(1:IDIMGL,1:INDIC)=YL_ZOBS_ARR(IND_BLOCK(JMASK))%OBS_ARR_GL2(1:IDIMGL,1:INDIC)
    ENDIF
    IF (NPROC >1 ) THEN
      ITAG=ITAG+1
      CALL MPL_BROADCAST(ZEIGVECS(:,1:NRITZ_OBSCOR(JMASK)),ITAG,IPROC_LOC_ARR(JMASK),CDSTRING='SUOBSCOR:')
      ITAG=ITAG+1
      CALL MPL_BROADCAST(ZEIGVALS,ITAG,IPROC_LOC_ARR(JMASK),CDSTRING='SUOBSCOR:')
      ITAG=ITAG+1
      CALL MPL_BROADCAST(OBS_ARR_GL2,ITAG,IPROC_LOC_ARR(JMASK),CDSTRING='SUOBSCOR:')
    ENDIF
!   4.4 Save eigen-sys to odb + to module variables
!     -------------------------------------------
    DO J1=1,IDIMGL
        IPROC=NINT(OBS_ARR_GL2(J1,IND_PROC))
        IF (NPRCIDS(MYPROC) == IPROC) THEN
          IBODY=OBS_ARR_GL2(J1,IND_BODY)
          DO JEV=1,NRITZ_OBSCOR(JMASK)
            ROBODY(IBODY,MDB_OBSCOREV_AT_ERRSTAT(JEV))=ZEIGVECS(J1,JEV)
          ENDDO
        ENDIF
    ENDDO
!   check if eigen vector completely set for the given block
    DO JBODY=1,IMXBDY
      DO JOBS = 1,NOBTOT
        IF(JBODY > ICMBDY(JOBS)) CYCLE
        IBODY = MLNKH2B(JOBS)+(JBODY-1)
        IF (ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT) == JMASK .AND. &
         & ROBODY(IBODY,MDB_OBSCOREV_AT_ERRSTAT(1)) == RMDI) CALL ABOR1('SUOBSCOR: missing eigenvector component')
      ENDDO
    ENDDO
    DO JEV=1,NRITZ_OBSCOR(JMASK)
      OBSCOR_EIGVAL(JEV,JMASK)=ZEIGVALS(JEV)
    ENDDO
!rg    WRITE(NULOUT,*) ' Eigenvalues of obs corr matrix at JMASK=',JMASK,':',OBSCOR_EIGVAL(1:NRITZ_OBSCOR(JMASK),JMASK)
    OBSCOR_TRUN(JMASK)=( REAL(IDIMGL,JPRB)-SUM(ZEIGVALS(1:NRITZ_OBSCOR(JMASK))) )/REAL(IDIMGL-NRITZ_OBSCOR(JMASK),JPRB)
!rg-tmp
    OBSCOR_TRUN(JMASK)=1._JPRB
!rg-tmp
!   4.5 Diagnostics---> Print-outs + some obs error matrix columns into odb aux columns+etc in cases
!     --------------------------------------------------------------------------------------------
    WRITE(NULOUT,*) 'DIMENSION OF CORRELATION MATRIX AT OBSERVATION MASK VALUE ',JMASK, ' IS: ',IDIMGL
    WRITE(NULOUT,*) 'EIGENVALUES ARE: ',ZEIGVALS
    WRITE(NULOUT,*) 'NUMBER OF USED EIGENVECTORS IS: ',NRITZ_OBSCOR(JMASK)
    WRITE(NULOUT,*) 'SUM OF USED EIGENVALUES IS: ',SUM(ZEIGVALS(1:NRITZ_OBSCOR(JMASK)))
    WRITE(NULOUT,*) 'CONDITION NUMBER IS: ',ZEIGVALS(1)/ZEIGVALS(NRITZ_OBSCOR(JMASK))
    WRITE(NULOUT,*) 'OBSCORTRUN IS: ',OBSCOR_TRUN(JMASK)
    IF (LOBSCORDIAG) THEN
      ZEPS=0.00001_JPRB
!
!     check orthogonality of eigen-system
!
      DO JEV=1,NRITZ_OBSCOR(JMASK)
      DO JEV1=1,NRITZ_OBSCOR(JMASK)
         ZSCALP1=0._JPRB
         DO J1=1,IDIMGL
           ZSCALP1=ZSCALP1+ZEIGVECS(J1,JEV)*ZEIGVECS(J1,JEV1)
         ENDDO
         ZACT=0._JPRB
         IF (JEV == JEV1) ZACT=1._JPRB
         IF (ABS(ZSCALP1-ZACT) > ZEPS) THEN
           WRITE(NULOUT,*) 'LANCZOS ORTHOGONALITY CHECK FAILED'
           WRITE(NULOUT,*) JEV,JEV1,ZSCALP1
           CALL ABOR1('OBSCOR: LANCZOS ORTHOGONALITY CHECK')
         ENDIF
      ENDDO
      ENDDO
      WRITE(NULOUT,*) 'LANCZOS ORTHOGONALITY CHECK OKEY'
!
!     check eigen-pairs
!
      NMASK=JMASK
      ALLOCATE(ZX(IDIMGL))
      ALLOCATE(ZY(IDIMGL))
      DO JEV=1,NRITZ_OBSCOR(JMASK)
       ZX(1:IDIMGL)=ZEIGVECS(1:IDIMGL,JEV)
       CALL OPK_OBSCOR(IDIMGL,ZX,ZY)
       ZZEIGVAL=(ZEIGVALS(JEV)-EIGMIN)/(1._JPRB-EIGMIN)
       ZX(:)=ZZEIGVAL*ZX(:)
       ZSCALP1=0._JPRB
       ZSCALP2=0._JPRB
       DO J1=1,IDIMGL
         ZSCALP1=ZSCALP1+(ZY(J1)-ZX(J1))**2
         ZSCALP2=ZSCALP2+ZY(J1)**2
       ENDDO
       ZSCALP1=SQRT(ZSCALP1)
       ZSCALP2=SQRT(ZSCALP2)
       WRITE(NULOUT,*) ' EIGENVECTOR CHECK',JEV,ZSCALP1,ZSCALP2
      ENDDO
      DEALLOCATE(ZX)
      DEALLOCATE(ZY)
      INDDIAG=999999
      IDIAG=4
!     4 out of the 6 free odb aux words are used for diagnosing different matrix columns   
!   obs-corr diagnostics at the "central" observation point, U in case of winds
      OBSLATDIAG=(180._JPRB/RPI)*SUM(OBS_ARR_GL2(1:IDIMGL,IND_LAT))/REAL(IDIMGL,JPRB)
      OBSLONDIAG=(180._JPRB/RPI)*SUM(OBS_ARR_GL2(1:IDIMGL,IND_LON))/REAL(IDIMGL,JPRB)
      ZMIN=999999._JPRB
      DO J1=1,IDIMGL
          ZACT=SQRT( (OBS_ARR_GL2(J1,IND_LON)-OBSLONDIAG*RPI/180._JPRB)**2+ &
          &    (OBS_ARR_GL2(J1,IND_LAT)-OBSLATDIAG*RPI/180._JPRB)**2 )
          IF (ZACT < ZMIN-0.00001_JPRB) THEN
             ZMIN=ZACT
             INDDIAG(1)=J1
          ENDIF
      ENDDO
!     the second diagnostic point is the closest point to the North Pole and it is U in case of winds
      ZMIN=999999._JPRB
      DO J1=1,IDIMGL
        ZACT=RPI/2._JPRB-OBS_ARR_GL2(J1,IND_LAT)
        IF (ZACT < ZMIN-0.00001_JPRB) THEN
           ZMIN=ZACT
           INDDIAG(2)=J1
        ENDIF
      ENDDO
!   the other four diagnostic points are random-like (taking care that some are u some are v
!   in case of wind observations by taking 2 indices from first half, 2 from second)
      DO JDIAG=3,IDIAG
        INDDIAG(JDIAG)=(JDIAG-2)*IDIMGL/(IDIAG-1)
      ENDDO
      IF (ANY(INDDIAG(1:IDIAG) == 999999)) THEN
        WRITE(NULERR,*) ' Searching for obs corr diagnostic points failed'
        CALL ABOR1(' Searching for obs corr diagnostic points failed')
      ENDIF 
      DO JDIAG=1,IDIAG
        WRITE(NULOUT,*) 'OBS MASK, LATDIAG, LONDIAG', &
         & JMASK,OBS_ARR_GL2(INDDIAG(JDIAG),IND_LAT)*180._JPRB/RPI, &
         & OBS_ARR_GL2(INDDIAG(JDIAG),IND_LON)*180._JPRB/RPI
      ENDDO
      ZOBSCORTRUN=OBSCOR_TRUN(JMASK)
      ALLOCATE(ZCORR_APPROX_COL(IDIMGL))
      DO JDIAG=1,IDIAG
        ZCORR_APPROX_COL(:)=0._JPRB
        ZCORR_APPROX_COL(INDDIAG(JDIAG))=ZOBSCORTRUN
        DO JEV=1,NRITZ_OBSCOR(JMASK)
          DO J1=1,IDIMGL
            ZCORR_APPROX_COL(J1)=ZCORR_APPROX_COL(J1)+(ZEIGVALS(JEV)-ZOBSCORTRUN)* &
            &  ZEIGVECS(J1,JEV)*ZEIGVECS(INDDIAG(JDIAG),JEV)
          ENDDO
        ENDDO
        DO J1=1,IDIMGL
          IPROC=OBS_ARR_GL2(J1,IND_PROC)
          IBODY=OBS_ARR_GL2(J1,IND_BODY)
          IF (NPRCIDS(MYPROC) == IPROC) THEN
            ROBODY(IBODY,MDB_AUX_AT_BODY(3+JDIAG))=ZCORR_APPROX_COL(J1)
          ENDIF
        ENDDO
      ENDDO  !jdiag
!     write max of each column to odb (should be 1.0, but it is not due to truncation)
! this is quite expensive, uncomment only if really wanted:
!      DO JDIAG=1,IDIMGL
!        ZCORR_APPROX_COL(:)=0._JPRB    
!        ZCORR_APPROX_COL(JDIAG)=ZOBSCORTRUN
!        DO JEV=1,NRITZ_OBSCOR(JMASK)
!          DO J1=1,IDIMGL 
!            ZCORR_APPROX_COL(J1)=ZCORR_APPROX_COL(J1)+(ZEIGVALS(JEV)-ZOBSCORTRUN)* &
!            &  ZEIGVECS(J1,JEV)*ZEIGVECS(JDIAG,JEV)
!          ENDDO
!        ENDDO
!        zcormax=maxval(ZCORR_APPROX_COL(1:IDIMGL)) 
!   hopefully zcormax=zselfcor
!        zselfcor=ZCORR_APPROX_COL(JDIAG)
!        IPROC=OBS_ARR_GL2(JDIAG,IND_PROC)  
!        IBODY=OBS_ARR_GL2(JDIAG,IND_BODY)
!        IF (NPRCIDS(MYPROC) == IPROC) THEN
!            ROBODY(IBODY,MDB_AUX_AT_BODY(3+IDIAG+1))=zcormax
!            ROBODY(IBODY,MDB_AUX_AT_BODY(3+IDIAG+2))=zselfcor
!        ENDIF
!      ENDDO  !jdiag
!xxxx    
      DEALLOCATE(ZCORR_APPROX_COL)
    ENDIF  ! LOBSCORDIAG
    IF (ALLOCATED(ZEIGVECS)) DEALLOCATE(ZEIGVECS)
    IF (ALLOCATED(ZEIGVALS)) DEALLOCATE(ZEIGVALS)
  ENDDO MASK_LOOP_GLOB
  WRITE(NULOUT,*) 'EIGEN-SYSTEM FULLY PROCESSED...'
  DEALLOCATE(LUV_OBSCOR)
  DEALLOCATE(L3D_OBSCOR)
  DEALLOCATE(OBSCOR_HOR_LSCALE)
  DEALLOCATE(OBSCOR_P_UPPER)
  DEALLOCATE(OBSCOR_P_LOWER)
  DEALLOCATE(ZP_UPPER)
  DEALLOCATE(ZP_LOWER)
  DEALLOCATE(OBSCOR_AUTO_COR)
  DEALLOCATE(NOBSCOR_INLEVS)
  DEALLOCATE(NACTIM_END)
  DEALLOCATE(NACTIM_START)
  DEALLOCATE(IPROC_LOC_ARR)
  DEALLOCATE(IND_BLOCK)
  DEALLOCATE(IDIMGL_ARR_LOC)
  DEALLOCATE(INDIC_ARR_LOC)
  DO JMASK=1,1+NBCORBLK/NPROC
    IF (ASSOCIATED(YL_ZOBS_ARR(JMASK)%OBS_ARR_GL2)) THEN
      DEALLOCATE(YL_ZOBS_ARR(JMASK)%OBS_ARR_GL2)
    ENDIF 
    IF (ASSOCIATED(YL_ZEIGVECS_LOC(JMASK)%OBS_ARR_GL2)) THEN
      DEALLOCATE(YL_ZEIGVECS_LOC(JMASK)%OBS_ARR_GL2)
    ENDIF
    IF (ASSOCIATED(YL_ZEIGVALS_LOC(JMASK)%OBS_ARR_GL2)) THEN
      DEALLOCATE(YL_ZEIGVALS_LOC(JMASK)%OBS_ARR_GL2)
    ENDIF
  ENDDO
  DEALLOCATE(YL_ZOBS_ARR)
  DEALLOCATE(YL_ZEIGVECS_LOC)
  DEALLOCATE(YL_ZEIGVALS_LOC)
!   5. Update odb with obs corr info
!    -----------------------------

  CALL GSTATS(1741,0)
  CALL PUTDB('SUOBSCOR', 0, IRETKODE, INFO, 0,ZINFO,0)
  CALL GSTATS(1741,1)
  IF (MYPROC == 1) THEN
    OPEN(UNIT=NULTMP,FORM='UNFORMATTED',FILE='obscor.aux.dat')
    WRITE(NULTMP) NBCORBLK
    WRITE(NULTMP) OBSCOR_EIGVAL
    WRITE(NULTMP) NRITZ_OBSCOR
    WRITE(NULTMP) OBSCOR_TRUN 
    CLOSE(NULTMP)
  ENDIF
  CALL SUTRANS('RESET')
ELSE      ! NUPTRA>0: eigen-vectors in ccma, the rest from obscor.aux.dat
  OPEN(UNIT=NULTMP,FORM='UNFORMATTED',FILE='obscor.aux.dat')
  READ(NULTMP) NBCORBLK
  ALLOCATE(OBSCOR_EIGVAL(JP_NUMEV,NBCORBLK))
  ALLOCATE(OBSCOR_TRUN(NBCORBLK))
  ALLOCATE(OBSCOR_SCALPR(JP_NUMEV,NBCORBLK))
  ALLOCATE(NRITZ_OBSCOR(NBCORBLK))
  READ(NULTMP) OBSCOR_EIGVAL
  READ(NULTMP) NRITZ_OBSCOR
  READ(NULTMP) OBSCOR_TRUN
  CLOSE(NULTMP)
ENDIF
IF (LHOOK) CALL DR_HOOK('SUOBSCOR',1,ZHOOK_HANDLE)
END SUBROUTINE SUOBSCOR
