SUBROUTINE SUOBSCOR_RESOL(KSMAX)

!**** *SUOBSCOR_RESOL* - Routine to setup coarse grid for obscor model
!     Purpose.
!     --------
!     Fill up obscor_grid_struct type structure to be used  
!     in the observation horizintal correlation model. Observation
!     error correlation calculation does not need the full model resolution. 
!**   Interface.
!     ----------

!     *CALL* SUOBSCOR_RESOL 
!        Explicit arguments :
!        --------------------
!                        KSMAX (input) : spectral resolution


!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department DOcumentation of the IFS
!     Author.
!     -------
!        Gabor Radnoti  *ECMWF*

!     Modifications.
!     --------------
!        Original : 2007-11-05

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARDIM   , ONLY : JPMXGL
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NPROC, NPRTRW, N_REGIONS_NS, N_REGIONS_EW
USE YOMCOSJO , ONLY : OBSCOR_GRID_DEF
USE YOMMP    , ONLY : LSPLIT, MYPROC, MYSETW
USE YOMLAP   , ONLY : RLAPDI
USE YOMCST   , ONLY : RA
USE YOMLUN   , ONLY : NULTMP, NULOUT,NULERR
USE MPL_MODULE, ONLY : MPL_BROADCAST
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(IN) :: KSMAX

INTEGER(KIND=JPIM), PARAMETER :: JPKS=KIND(RLAPDI)
INTEGER(KIND=JPIM), PARAMETER :: JPKD=KIND(1.0_JPRB)
INTEGER(KIND=JPIM) :: NRGRI(JPMXGL)
INTEGER(KIND=JPIM) :: IDGL, IOMASTER, ITAG, INSM, JM
CHARACTER(LEN=3)   :: CLNSMAX
LOGICAL            :: LL_EXIST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "setup_trans.h"
#include "trans_inq.h"
#include "ini_spec_dist.h"
END INTERFACE
#include "sutrans.intfb.h"
#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "namrgri.h"



IF (LHOOK) CALL DR_HOOK('SUOBSCOR_RESOL',0,ZHOOK_HANDLE)
IOMASTER=1
ITAG=65456

IF( MYPROC == IOMASTER )THEN
  WRITE(CLNSMAX,'(I3.3)') KSMAX
  INQUIRE(FILE='rtablel_2'//CLNSMAX,EXIST=LL_EXIST)
  IF (.NOT. LL_EXIST) CALL ABOR1('OBSCOR RESOL NAMELIST FILE DOES NOT EXIST')
  OPEN(NULTMP,FILE='rtablel_2'//CLNSMAX,ACTION="READ")
  NRGRI(:)=0
  CALL POSNAM(NULTMP,'NAMRGRI')
  READ (NULTMP,NAMRGRI)
  CLOSE(NULTMP)
  IDGL=1
  DO WHILE( NRGRI(IDGL)>0 )
    IDGL=IDGL+1
  ENDDO
  IDGL=IDGL-1
ENDIF
IF( NPROC > 1 )THEN
  CALL MPL_BROADCAST (IDGL,ITAG,IOMASTER,CDSTRING='SUOBSCOR_RESOL:')
ENDIF
OBSCOR_GRID_DEF%NDGLG = IDGL
WRITE(NULOUT,*) ' OBSCOR_GRID_DEF%NDGLG=',OBSCOR_GRID_DEF%NDGLG
ALLOCATE (OBSCOR_GRID_DEF%NLOENG(IDGL))
IF( MYPROC == IOMASTER )THEN
  OBSCOR_GRID_DEF%NLOENG(1:IDGL)=NRGRI(1:IDGL)
ENDIF
IF( NPROC > 1 )THEN
  CALL MPL_BROADCAST (OBSCOR_GRID_DEF%NLOENG(:),ITAG,IOMASTER,CDSTRING='SUOBSCOR_RESOL:')
ENDIF
OBSCOR_GRID_DEF%NSMAX=KSMAX
CALL SUTRANS('MONO1')
CALL SETUP_TRANS (KSMAX=KSMAX, KDGL=IDGL, &
     & KLOEN=OBSCOR_GRID_DEF%NLOENG(:), &
     & LDSPLIT=LSPLIT, &
     & KRESOL=OBSCOR_GRID_DEF%NRESOL_ID)
ALLOCATE(OBSCOR_GRID_DEF%NASM0(0:KSMAX))
ALLOCATE(OBSCOR_GRID_DEF%NPTRFRSTLAT(N_REGIONS_NS))
ALLOCATE(OBSCOR_GRID_DEF%NFRSTLAT(N_REGIONS_NS))
ALLOCATE(OBSCOR_GRID_DEF%NLSTLAT(N_REGIONS_NS))
ALLOCATE(OBSCOR_GRID_DEF%NSTA(IDGL+N_REGIONS_NS-1,N_REGIONS_EW))
ALLOCATE(OBSCOR_GRID_DEF%NONL(IDGL+N_REGIONS_NS-1,N_REGIONS_EW))
ALLOCATE(OBSCOR_GRID_DEF%NGPTOTL(N_REGIONS_NS,N_REGIONS_EW))
CALL TRANS_INQ(KRESOL=OBSCOR_GRID_DEF%NRESOL_ID, &
     & KSPEC2=        OBSCOR_GRID_DEF%NSPEC2, &
     & KGPTOT=        OBSCOR_GRID_DEF%NGPTOT, &
     & KGPTOTG=       OBSCOR_GRID_DEF%NGPTOTG, &
     & KNUMP=         OBSCOR_GRID_DEF%NUMP, &
     & KASM0=         OBSCOR_GRID_DEF%NASM0, &
     & KGPTOTL=       OBSCOR_GRID_DEF%NGPTOTL, &
     & KPTRFRSTLAT=   OBSCOR_GRID_DEF%NPTRFRSTLAT, &
     & KFRSTLAT=      OBSCOR_GRID_DEF%NFRSTLAT, &
     & KLSTLAT=       OBSCOR_GRID_DEF%NLSTLAT, &
     & KSTA=          OBSCOR_GRID_DEF%NSTA, &
     & KONL=          OBSCOR_GRID_DEF%NONL)
ALLOCATE(OBSCOR_GRID_DEF%MYMS(OBSCOR_GRID_DEF%NUMP))
CALL TRANS_INQ (KRESOL=OBSCOR_GRID_DEF%NRESOL_ID, &
     & KMYMS= OBSCOR_GRID_DEF%MYMS)
ALLOCATE(OBSCOR_GRID_DEF%NPROCM(0:KSMAX))
CALL INI_SPEC_DIST(KSMAX,KSMAX,&
 & NPRTRW,MYSETW,KPROCM=OBSCOR_GRID_DEF%NPROCM)
ALLOCATE(OBSCOR_GRID_DEF%NASM0G(0:KSMAX))
ALLOCATE(OBSCOR_GRID_DEF%NSPZERO(0:KSMAX))
ALLOCATE(OBSCOR_GRID_DEF%RLAPDI(-1:KSMAX+2))
INSM=1
DO JM=0,KSMAX
  OBSCOR_GRID_DEF%NSPZERO(JM)=1+JM*2+1
  OBSCOR_GRID_DEF%NASM0G(JM)=INSM
  INSM=INSM+(KSMAX+1-JM)*2
ENDDO
DO JM=1,KSMAX+2
  OBSCOR_GRID_DEF%RLAPDI(JM)=REAL(-REAL(JM*(JM+1),JPKD)/(REAL(RA,JPKD)*REAL(RA,JPKD)),JPKS)
ENDDO
OBSCOR_GRID_DEF%RLAPDI(0)=0._JPRB
OBSCOR_GRID_DEF%RLAPDI(-1)=0._JPRB
IF (LHOOK) CALL DR_HOOK('SUOBSCOR_RESOL',1,ZHOOK_HANDLE)
END SUBROUTINE SUOBSCOR_RESOL
