SUBROUTINE SUSCRE0

!**** *SUSCRE0*  -  Setup screening commons on level 0

!     Purpose.
!     --------
!           Setup screening commons

!**   Interface.
!     ----------
!        *CALL* *SUSCRE0

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Heikki Jarvinen *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-03-15
!     Erik Andersson 971017 : Move setting of CAMBPL to this routine
!     Per Unden      971030 : Add 925 standard level
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMSCREE , ONLY : NAMBPL ,NDFLMX ,NDFLMN ,CAMBPL,&
 & NSUPERSONIC ,CSUPERSONIC   
USE YOMLEVSE , ONLY : NSTDPL
USE YOMALIM  , ONLY : NFLLIM
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LSCREEN
USE YOMCOSJO , ONLY : LTC

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    INITIALIZE ALL DIMENSIONS NEEDED FOR ALLOCATION
!              -----------------------------------------------

!           1.1   NUMBER OF STANDARD (MANDATORY) PRESSURE LEVELS

IF (LHOOK) CALL DR_HOOK('SUSCRE0',0,ZHOOK_HANDLE)
NSTDPL = 16

!     ------------------------------------------------------------------

!*       2.    INITIALIZE NUMERIC CONSTANTS
!              ----------------------------

!           2.1   MAXIMUM AND MINUMUM VALUES OF A DATUM FLAG

NDFLMX = 3
NDFLMN = 0

!*          2.2  FLAG LIMITS

NFLLIM(0:3) = (/0,1,2,3/)

!     ------------------------------------------------------------------

!*       4.    INITIALIZE CHARACTER CONSTANTS
!              ------------------------------

!           4.1   LIST OF AMBIGUOUS MOVING PLATFORMS
!                 NEEDED FOR TIME CORRELATED DATA, AS WELL AS SCREENING

IF(LSCREEN .OR. LTC) THEN
  NAMBPL = 9
  ALLOCATE(CAMBPL(NAMBPL))
  WRITE(NULOUT,9990) 'CAMBPL   ',SIZE(CAMBPL   ),SHAPE(CAMBPL   )
  CAMBPL(1) = '    SHIP'
  CAMBPL(2) = '    BOUY'
  CAMBPL(3) = '    PLAT'
  CAMBPL(4) = '     XXX'
  CAMBPL(5) = '    XXXX'
  CAMBPL(6) = '     ///'
  CAMBPL(7) = '    ////'
  CAMBPL(8) = '    RIGG'
  CAMBPL(9) = 'SHIP    '
ELSE
  NAMBPL = 0
ENDIF

!           4.2   LIST OF SUPERSONIC AIRCRAFTS
!                 NEEDED FOR SCREENING

IF(LSCREEN) THEN
  NSUPERSONIC = 5
  ALLOCATE(CSUPERSONIC(NSUPERSONIC))
  WRITE(NULOUT,9990) 'CSUPERSONIC',SIZE(CSUPERSONIC),SHAPE(CSUPERSONIC)
  CSUPERSONIC(1) = '  AFR001'
  CSUPERSONIC(2) = '  AFR002'
  CSUPERSONIC(3) = '    BAW1'
  CSUPERSONIC(4) = '    BAW2'
  CSUPERSONIC(5) = '   BAW97'
ELSE
  NSUPERSONIC = 0
ENDIF

!     ------------------------------------------------------------------

!*       5.    RETURN
!              ------

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('SUSCRE0',1,ZHOOK_HANDLE)
END SUBROUTINE SUSCRE0
