SUBROUTINE SUSCRE1

!**** *SUSCRE1*  -  Setup screening commons on level 1

!     Purpose.
!     --------
!           Setup screening commons

!**   Interface.
!     ----------
!        *CALL* *SUSCRE1

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        SUALSCRE - ALLOCATE TABLES FOR SCREENING

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Heikki Jarvinen *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-03-15
!        Modified : 
!     Per Unden      971030 : Add 925 standard level
!     Elias Holm   20001009 : Use maximum variable number NVNUMAX for
!                             dimensioning NDOBVA, NDEPEX, and NDEPNE
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMSCREE , ONLY : NGLSYN   ,NGLSHI   ,NGLAIR   ,&
 & NGLDRI   ,NGLTOV   ,NGLGEO   ,NGLSSM   ,NGLSAT   ,&
 & NGLSAM   ,NCDYTL   ,NCDSTL   ,NCDATL   ,NCDDTL   ,&
 & NCDTTL   ,NCDGTL   ,NCDITL   ,NCDWTL   ,NCDMTL   ,&
 & NRDBRR   ,NRDBRD   ,NREVTOT  ,NDEVTOT  ,NDOBVA   ,&
 & NDEPEX   ,NDEPNE  

USE YOMLEVSE , ONLY : NLLNVA   ,NLLNDA   ,N2MUPNT  ,N2MVPNT  ,&
 & N2MTPNT  ,N2MRPNT  ,NUPNT    ,NVPNT    ,NZPNT    ,&
 & NTPNT    ,NRPNT    ,RSTDPL  

USE YOMVNMB  , ONLY : NVNUMAX
USE YOMCMA   , ONLY : NMXOTP

IMPLICIT NONE

INTEGER(KIND=JPIM) :: J1, J2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sualscre.intfb.h"

!     ------------------------------------------------------------------

!*       1.    INITIALIZE ALL DIMENSIONS NEEDED FOR ALLOCATION
!              -----------------------------------------------

!           1.1   LEVEL SELECTION TABLES

IF (LHOOK) CALL DR_HOOK('SUSCRE1',0,ZHOOK_HANDLE)
NLLNVA  =   9
NLLNDA  = 200

!     ------------------------------------------------------------------

!*       2.    ALLOCATE TABLES FOR SCREENING
!              -----------------------------

CALL SUALSCRE

!     ------------------------------------------------------------------

!*       3.    INITIALIZE NUMERIC CONSTANTS
!              ----------------------------

!           3.1   STANDARD (MANDATORY) PRESSURE LEVELS (IN PASCALS)

RSTDPL ( 1) =   1000._JPRB
RSTDPL ( 2) =   2000._JPRB
RSTDPL ( 3) =   3000._JPRB
RSTDPL ( 4) =   5000._JPRB
RSTDPL ( 5) =   7000._JPRB
RSTDPL ( 6) =  10000._JPRB
RSTDPL ( 7) =  15000._JPRB
RSTDPL ( 8) =  20000._JPRB
RSTDPL ( 9) =  25000._JPRB
RSTDPL (10) =  30000._JPRB
RSTDPL (11) =  40000._JPRB
RSTDPL (12) =  50000._JPRB
RSTDPL (13) =  70000._JPRB
RSTDPL (14) =  85000._JPRB
RSTDPL (15) =  92500._JPRB
RSTDPL (16) = 100000._JPRB

!           3.2   LEVEL SELECTION

N2MUPNT =   1
N2MVPNT =   2
N2MTPNT =   3
N2MRPNT =   4
NUPNT   =   5
NVPNT   =   6
NZPNT   =   7
NTPNT   =   8
NRPNT   =   9

!     ------------------------------------------------------------------

!*       4.    INITIALIZE CHARACTER CONSTANTS
!              ------------------------------

!     ------------------------------------------------------------------

!*       5.    INITIALIZE LOGICAL VARIABLES
!              ----------------------------

!     ------------------------------------------------------------------

!*       6.    INITIALIZE TABLES AND COUNTERS FOR SCREENING
!              --------------------------------------------

!*          6.1   NUMBER OF REPORTS IN GLOBAL MOVING PLATFORM ARRAYS

NGLSYN = 0
NGLSHI = 0
NGLAIR = 0
NGLDRI = 0
NGLTOV = 0
NGLGEO = 0
NGLSSM = 0
NGLSAT = 0
NGLSAM = 0

!*          6.2   NUMBER OF ITEMS PER REPORT IN TIME-LOCATION ARRAYS

!*             6.2.1   SYNOPTIC REPORTS

NCDYTL = 15

!*             6.2.2   SHIPS

NCDSTL = 13

!*             6.2.3   AIREPS

NCDATL = 13

!*             6.2.4   DRIBUS

NCDDTL = 13

!*             6.2.5   TOVS REPORTS

NCDTTL = 13

!*             6.2.6   GEOS REPORTS

NCDGTL = 10

!*             6.2.7   SSMI REPORTS

NCDITL = 13

!*             6.2.8   SATOB REPORTS

NCDWTL = 13

!*             6.2.8   SATAM REPORTS

NCDMTL = 14

!*          6.3   RDB-FLAGGED REPORTS/DATUMS

DO J1 = 1, NMXOTP
  DO J2 = 1,  5
    NRDBRR(J1,J2) = 0
    NRDBRD(J1,J2) = 0
  ENDDO
ENDDO

!*          6.4   REPORTS/DATUM EVENTS

DO J1 = 1, NMXOTP 
  DO J2 = 1, 32
    NREVTOT(J1,J2) = 0
    NDEVTOT(J1,J2) = 0
  ENDDO
ENDDO

!*          6.5   NUMBER OF VARIABLES IN DIFFERENT OBSERVATION TYPES

DO J1 = 1, NMXOTP 
  DO J2 = 1, NVNUMAX
    NDOBVA(J1,J2) = 0
    NDEPEX(J1,J2) = 0  ! F.G. DEPARTURE EXISTS
    NDEPNE(J1,J2) = 0  ! F.G. DEPARTURE DOES NOT EXIST
  ENDDO
ENDDO
!*
!     ------------------------------------------------------------------

!*       7.    RETURN
!              ------

IF (LHOOK) CALL DR_HOOK('SUSCRE1',1,ZHOOK_HANDLE)
END SUBROUTINE SUSCRE1
