SUBROUTINE SYNOPBE(KPOINT  ,&
 & KVNM    ,KVERC   ,PREFV1  ,PREFV2  ,PVAL    ,&
 & PVOERR  ,PVPERR  ,PVFERR  ,PVRERR  ,&
 & PVFGERR ,PVFGCC1 ,PVFGCC2 ,&
 & PVFGDEP ,PVANDEP ,&
 & KVSTAT  ,KVEV1   ,KVEV2   ,KVRDBFL ,KVANFL  ,&
 & KPRESCD                                       )  

!**** SUBROUTINE SYNOPBE - CREATE SYNOP BODY ENTRY

!        D. VASILJEVIC     ECMWF     04/07/94

!     PURPOSE
!     -------

!        CREATE SYNOP BODY ENTRY

!**   INTERFACE
!     ---------

!        CALL SYNOPBE(KPOINT  ,
!                     KVNM    ,KVERC   ,PREFV1  ,PREFV2  ,PVAL    ,
!                     PVOERR  ,PVPERR  ,PVFERR  ,PVRERR  ,
!                     PVFGERR ,PVFGCC1 ,PVFGCC2 ,
!                     PVFGDEP ,PVANDEP ,
!                     KVSTAT  ,KVEV1   ,KVEV2   ,KVRDBFL ,KVANFL  ,
!                     KPRESCD                                      )

!     WHERE:
!            KPOINT  = REPORT'S BODY
!            KVNM    = VARIABLE NO.
!            KVERC   = VERTICAL COORDINATE IND.
!            PREFV1  = VERTICAL REFERENCE VALUE 1
!            PREFV2  = VERTICAL REFERENCE VALUE 2
!            PVAL    = OBSERVED VALUE
!            PVOERR  = OBSERVED VALUE OBS. ERROR
!            PVPERR  = OBSERVED VALUE PERSISTENCE ERROR
!            PVFERR  = BSERVED VALUE FINAL ERROR
!            PVRERR  = OBSERVED VALUE REPRESENTATIVNESS ERROR
!            PVFGERR = FG ERROR
!            PVFGCC1 = FG CHECK CONSTANT 1
!            PVFGCC2 = FG CHECK CONSTANT 2
!            PVFGDEP = FG DEPARTURE
!            PVANDEP = ANALYSIS DEPARTURE
!            KVSTAT  = VARIABLE STATUS
!            KVEV1   = VARIABLE EVENTS 1
!            KVEV2   = VARIABLE EVENTS 2
!            KVRDBFL = VARIABLE RDB FLAGS
!            KVANFL  = VARIABLE ANALYSIS FLAGS
!            KPRESCD = PRESSURE CODE

!        SYNOPBE IS CALLED BY SUBROUTINE SYNOPIN

!     METHOD
!     ------

!        FIRST OPEN UP BODY ENTRY AND THEN FILL UP APPROPRIATE WORDS.

!     EXTERNALS
!     ---------

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPOINT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVNM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVERC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREFV1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREFV2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVPERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFGERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFGCC1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFGCC2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFGDEP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVANDEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVSTAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVEV1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVEV2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVRDBFL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVANFL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPRESCD 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "filfbde.intfb.h"

!-----------------------------------------------------------------------
#include "openmp_obs.h"

!*

!        1.             SYNOP BODY ENTRY CREATION
!                       -------------------------

!*          1.1         GENERAL OBSERVATION BODY ENTRY UPDATE/CREATION

IF (LHOOK) CALL DR_HOOK('SYNOPBE',0,ZHOOK_HANDLE)
CALL FILFBDE(KPOINT  ,&
 & KVNM   ,KVERC  ,PREFV1 ,PREFV2 ,PVAL  ,&
 & PVFERR ,PVOERR ,PVRERR ,PVPERR ,&
 & PVFGERR,PVFGCC1,PVFGCC2,&
 & PVANDEP,PVFGDEP,&
 & KVSTAT ,KVEV1  ,KVEV2  ,KVRDBFL,KVANFL)  

!*          1.2         SYNOP SPECIFIC BODY ENTRY UPDATE/CREATION

!*          1.2.2       PRESSURE CODE

ROBODY(KPOINT,MDBSYPC) = KPRESCD
!*
!-----------------------------------------------------------------------

!        2.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('SYNOPBE',1,ZHOOK_HANDLE)
END SUBROUTINE SYNOPBE
