SUBROUTINE THIAIR(KPATH,KNOAIR,KIDAIR,KMAX,KIDPAI,PBSIZE)

!**** SUBROUTINE THIAIR - THINNING OF AIREP REPORTS

!          HEIKKI JARVINEN   ECMWF          15/AUG/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        REMOVE REDUNDANT AIREP REPORTS DUE TO EXCESSIVELY FREQUENT
!     REPORTING PRACTICE. PRESELECTION HAS ALREADY BEEN MADE IN MOVPL
!     BY DEFINING PATHS AND POINTER TABLES.

!**   INTERFACE
!     ---------

!     ***CALL*THIAIR(KPATH,KNOAIR,PBSIZE)
!     WHERE KPATH  = FLIGHT PATH IN QUESTION
!           KNOAIR = NUMBER OF AIREP REPORTS IN THIS PATH
!           KIDAIR  = TOTAL NUMBER OF FLIGHT PATHS (FOR DIMESIONING)
!           KMAX  = MAXIMUM NUMBER OF REPORTS IN A PATH (FOR DIMESIONING)
!           KIDPAI  = IDENTIFIERS TO EACH REPORT IN EACH PATH
!           PBSIZE = DESIRED BOXSIZE FOR THINNING (IN DEGREES)

!        THIAIR IS CALLED BY SUBROUTINE MOVPL

!*    METHOD
!     ------

!        THINNING IS PERFORMED WITH AID OF A GLOBAL POINTER TABLES
!     IN EACH PROCESSOR OF A MPP-COMPUTER. REPORTS ARE SORTED ACCORDING
!     TO A KEY WITH THE FOLLOWING PRIORITIES:
!       1) THINNING BOX INDEX
!       2) NUMBER OF ACTIVE DATA IN THE REPORT
!       3) TIME DIFFERENCE FROM ANALYSIS TIME
!     ONLY ONE REPORT PER BOX IS ALLOWED AND THAT HAS THE HIGHEST PRI-
!     ORITY WITHIN THE BOX.
!     THE BOXSIZE IS APPROXIMATELY 50KM.

!*    EXTERNALS
!     ---------

!        REFTIM - REFERENCE DATE
!        REJMV  - REJECT A MOVING PLATFORM

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCOCTP , ONLY : NAIREP
USE YOMANA   , ONLY : NANDAT   ,NANTIM
USE YOMANCS  , ONLY : RDEGREES ,RADIANS
USE YOMSCC   , ONLY : LSCDPR
USE YOMDB
USE ECSORT_MIX   , ONLY : KEYSORT
USE YOMCT0   , ONLY : LELAM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KIDAIR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPATH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNOAIR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDPAI(KIDAIR,KMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSIZE 
LOGICAL :: LLREJEC
CHARACTER(LEN=8) ::  CLSTID

INTEGER(KIND=JPIM) :: IANRTM, IBOX, ICMONM, ICMOTP, IDAT,&
 & IRET, INAD, IOBDAT, IOBETM, IOBRTM, IOLDKEY, &
 & IPOS, ITIM, J, JOBS, JS, IXC, IYC, ISCALE  
INTEGER(KIND=JPIM) :: INDEX (KNOAIR)
INTEGER(KIND=JPIM) :: IPOINT(KNOAIR)

REAL(KIND=JPRB) :: ZLAT, ZLON
REAL(KIND=JPRB) :: ZKEY  (KNOAIR)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ngenada.intfb.h"
#include "reftim.intfb.h"
#include "rejmv.intfb.h"

#include "fcobs.h"
#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------
!*
!WRITE(NULOUT,'('' *** THIAIR; AIREPS THINNING*'')')

!           1.1   OBSERVATION TYPE

IF (LHOOK) CALL DR_HOOK('THIAIR',0,ZHOOK_HANDLE)
ICMOTP = NAIREP
!*
!           1.2   ANALYSIS TIME (IN MINUTES FROM REFERENCE DATE)

CALL REFTIM(NANDAT,NANTIM,IANRTM)
!*
!           1.3   INITIALIZE SORTING AND KEY ARRAYS

DO J = 1, KNOAIR
  INDEX (J) = 0
  IPOINT(J) = 0
  ZKEY  (J) = 0.0_JPRB
ENDDO
!*
!     ------------------------------------------------------------------

!        2.       SORT THE OBSERVATIONS
!                 ---------------------
!*
!           2.1   LOOP OVER ALL THE AIREPS IN THIS PATH

IF(LELAM) THEN
  ISCALE=10000
ELSE
  ISCALE=1000
ENDIF
IF(PBSIZE < 360._JPRB/ISCALE) CALL ABOR1('THIAIR: PBSIZE OR ISCALE TOO SMALL')

PATH_LOOP: DO JOBS = 1, KNOAIR
!*
!           2.2   SAVE POINTER TO GLOBAL AIREP TIME-LOCATION ARRAY

  IPOS = KIDPAI(KPATH,JOBS)
  IPOINT(JOBS) = IPOS
!*
!           2.3   COLLECT THE SORTING KEY INFORMATION
!*
!              2.3.1   THINNING BOX INDEX

  ZLAT = ROBHDR(IPOS,MDBLAT)*RDEGREES
  ZLON = ROBHDR(IPOS,MDBLON)*RDEGREES
  IF(ZLON > 180._JPRB) ZLON = ZLON - 360._JPRB

  IYC  = INT(ZLAT/PBSIZE)
  IXC  = MAX(INT(360._JPRB*COS(IYC*PBSIZE*RADIANS)/PBSIZE),1)
  IBOX = (IYC+ISCALE/2)*ISCALE + INT(ZLON*IXC/360._JPRB) + ISCALE/2
!*
!              2.3.2   NUMBER OF ACTIVE DATA IN THE REPORT

  CALL NGENADA(IPOS,INAD)
!*
!              2.3.3   OBSERVATION TIME (IN MINUTES FROM REFERENCE DATE)

  IOBDAT = ROBHDR(IPOS,MDBDAT)
  IOBETM = ROBHDR(IPOS,MDBETM)
  CALL REFTIM(IOBDAT,IOBETM,IOBRTM)

!*          2.4   COMPOSE THE SORTING KEY

  ZKEY(JOBS) = REAL(IBOX,JPRB)*100._JPRB &
   & + REAL(9-INAD,JPRB)&
   & + REAL(ABS(IANRTM-IOBRTM),JPRB)/100000._JPRB  

!*          2.5   CLOSE THE LOOP

ENDDO PATH_LOOP
CALL FLUSH(NULOUT)

!*          2.6    SORT 

CALL KEYSORT(IRET,ZKEY,KNOAIR,INDEX=INDEX,INIT=.TRUE.)

!*
!     ------------------------------------------------------------------

!*       3.       PERFORM THE THINNING
!                 --------------------

!*          3.1   INITIALIZE THE OLD KEY
!                 - ALLOW ONLY ONE AIREP PER SORTING BOX. THEREFORE
!                   RECALCULATE SORTING BOX INDEX FROM THE COMPOSE KEY.

IOLDKEY = NINT(ZKEY(INDEX(1))/100._JPRB)
!*          3.2   START PRINTING

JOBS = 1
JS   = INDEX(1)
IPOS = IPOINT(JS)
ZLAT = ROBHDR(IPOS,MDBLAT)*RDEGREES
ZLON = ROBHDR(IPOS,MDBLON)*RDEGREES
IDAT = ROBHDR(IPOS,MDBDAT)
ITIM = ROBHDR(IPOS,MDBETM)
CLSTID = TRANSFER(ROBHDR(IPOS,MDBSID), CLSTID)
IF(LSCDPR) THEN
  WRITE(NULOUT,'(/,1X,A,2I4,1X,I6,1X,F20.8,2F8.2,1X,A &
   & ,3(1X,I10))') ' - OK. -   '&
   & ,JOBS,JS,IOLDKEY,ZKEY(JS),ZLAT,ZLON,CLSTID &
   & ,IDAT,ITIM,IPOS  
ENDIF

!*          3.3   LOOP OVER ALL THE OBSERVATIONS

ACTIVE_DATA_LOOP: DO JOBS = 2, KNOAIR
  IPOS = IPOINT(INDEX(JOBS))

!*          3.4   ALLOW ONE REPORT PER THINNING BOX ONLY

  IF(NINT(ZKEY(INDEX(JOBS))/100._JPRB) == IOLDKEY) THEN
    ICMONM = ROBHDR(IPOS,MDBONM)
    CALL REJMV(IPOS,ICMOTP,ICMONM)
    LLREJEC = .TRUE.
  ELSE
    IOLDKEY = NINT(ZKEY(INDEX(JOBS))/100._JPRB)
    LLREJEC = .FALSE.
  ENDIF

!*          3.5   PRINT THINNING INFORMATION

  JS = INDEX(JOBS)
  ZLAT = ROBHDR(IPOS,MDBLAT)*RDEGREES
  ZLON = ROBHDR(IPOS,MDBLON)*RDEGREES
  IDAT = ROBHDR(IPOS,MDBDAT)
  ITIM = ROBHDR(IPOS,MDBETM)
  CLSTID = TRANSFER(ROBHDR(IPOS,MDBSID), CLSTID)
  IF(LSCDPR .AND. LLREJEC) THEN
    WRITE(NULOUT,'(1X,A,2I4,1X,I6,1X,F20.8,2F8.2,1X,A &
     & ,2(1X,I10))') ' THIAIR; REJECTED: '&
     & ,JOBS,JS,IOLDKEY,ZKEY(JS),ZLAT,ZLON,CLSTID &
     & ,IDAT,ITIM  
  ELSEIF(LSCDPR) THEN
    WRITE(NULOUT,'(/,1X,A,2I4,1X,I6,1X,F20.8,2F8.2,1X,A &
     & ,3(1X,I10))') ' - OK. -   '&
     & ,JOBS,JS,IOLDKEY,ZKEY(JS),ZLAT,ZLON,CLSTID &
     & ,IDAT,ITIM,IPOS  
  ENDIF

!*          3.6   CLOSE THE LOOP

ENDDO ACTIVE_DATA_LOOP

!     ------------------------------------------------------------------

!        4.       RETURN
!                 ------
IF (LHOOK) CALL DR_HOOK('THIAIR',1,ZHOOK_HANDLE)
END SUBROUTINE THIAIR
