SUBROUTINE TIMDIF(KDATE,KTIME,PTIM)

!**** SUBROUTINE TIMDIF - OBS. ANAL. TIME DIFFERENCE

!        D. VASILJEVIC     ECMWF      20/10/1994

!     PURPOSE
!     -------

!        FIND THE TIME DIFFERENCE IN MINUTES BETWEEN THE OBSERVATION
!     AND ANALYSIS TIMES.

!**   INTERFACE
!     ---------

!        CALL TIMDIF(KDATE,KTIME,PTIM)

!     WHERE 
!           KDATE  = OBSERVATION DATE
!           KTIME  = OBSERVATION TIME
!           PTIM   = TIME DIFFERENCE IN MINUTES

!     TIMDIF IS CALLED BY

!     METHOD
!     ------

!        SIMPLE SUBTRACTION

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMKODB , ONLY : NANCDY
USE YOMANA   , ONLY : NANMIN
USE YOMANCS  , ONLY : NMDI     ,NHOUPDAY ,NMINPHOU
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
INTEGER(KIND=JPIM),INTENT(IN)    :: KDATE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTIME 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTIM 
INTEGER(KIND=JPIM), EXTERNAL :: IYMD2CD

INTEGER(KIND=JPIM) :: ICNDAY, ICNDDIF, IDATE, IRET, ITIME, ITIMO100
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!*
!        1.       FIND OUT TIME DIFFERENCE
!                 ------------------------

IF (LHOOK) CALL DR_HOOK('TIMDIF',0,ZHOOK_HANDLE)
IDATE  = KDATE
ITIME  = KTIME

!*          1.1   CHECK OBSERVATION TIME AND DATE

IF(IDATE == NMDI.OR.ITIME == NMDI    )             THEN
  WRITE(NULOUT,'(''0ERROR IN SUBROUTINE TIMDIF'')')
  WRITE(NULOUT,'('' EITHER OBS. DATE OR TIME NOT DEFINED'')')
  CALL ABOR1('SUBROUTINE TIMDIF')
ENDIF

!*          1.1.1 OBS. CENTURY DATE

IRET   = 0
ICNDAY = IYMD2CD(IDATE,IRET)
IF(IRET /= 0)     THEN
  WRITE(NULOUT,'(''0TIMDIF; ERROR: '',I10,'' IN IYMD2CD'')') IRET
  CALL ABOR1('SUBROUTINE TIMDIF')
ENDIF

!*          1.2   TIME DIFFERENCE

ICNDDIF     =     ICNDAY - NANCDY
ITIMO100    =     ITIME /100
PTIM        =     REAL(ICNDDIF*NHOUPDAY*NMINPHOU +&
 & ITIMO100*NMINPHOU +&
 & (ITIME-ITIMO100*100) -&
 & NANMIN                     )  

!*
!-----------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('TIMDIF',1,ZHOOK_HANDLE)
END SUBROUTINE TIMDIF
