SUBROUTINE TOVSHRIS(KCHAN,KDPT,&
 & KA,KB,KC,KV,KW,KX,KY,KZ       )  

!**** SUBROUTINE TOVSHRIS - CONSTRUCT HIGH RES. TOVS INSTRUMENT
!                           SPECIFICATION

!        D. VASILJEVIC     ECMWF     24/02/95

!     PURPOSE
!     -------

!        CONSTRUCT INSTRUMENT SPECIFICATION/TYPE FOR HIGH RES. TOVS

!**   INTERFACE
!     ---------

!        CALL TOVSHRIS(KCHAN,KDPT,
!                      KA,KB,KC,KV,KW,KX,KY,KZ )

!     WHERE:
!           KDPT    = INPUT DATA PROCESSING TECHNIQUE (BUFR TABLE 2022)
!           KA      = OUTPUT A PARAM.
!           KB      = OUTPUT B PARAM.
!           KC      = OUTPUT C PARAM.
!           KV      = OUTPUT V PARAM.
!           KW      = OUTPUT W PARAM.
!           KX      = OUTPUT X PARAM.
!           KY      = OUTPUT Y PARAM.
!           KZ      = OUTPUT Z PARAM.

!        TOVSHRIS IS CALLED BY SUBROUTINE OBINSSP

!     METHOD
!     ------

!        SEE RESEARCH MANUAL 5, TABLE 7.13 FOR A DESCRIPTION OF THE TOVS
!     INSTRUMENT SPECIFICATION WORD.

!**   EXTERNALS
!     ---------

!        NONE

!**   MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMANCS  , ONLY : NMDI     ,NBITS    ,RADIANS  ,RTT      ,&
 & R2ES     ,R3LES    ,R3IES    ,R4LES    ,R4IES    ,&
 & RTWAT    ,RTICE    ,EPSMAC  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KCHAN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPT 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KA 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KB 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KC 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KV 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KW 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KX 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KY 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KZ 
INTEGER(KIND=JPIM) :: IA, IB, IC, ICLD, ICLR, IG1H, IG1M, IG2H,&
 & IG2M, IG3H1, IG3H2, IG3M, IG4H1, IG4H2, IG4M, &
 & IG4S, IG5H, IG5M, IG5S, IPCLD, IS, IV, IW, &
 & IX, IY, IZ

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
!*

!        1.             CHANNEL COMBINATIONS
!                       --------------------

!*          1.1         MISSING VALUE FOR CHANNEL WORD

IF (LHOOK) CALL DR_HOOK('TOVSHRIS',0,ZHOOK_HANDLE)
IF(KCHAN == NBITS (25).OR.KCHAN ==       NMDI    ) THEN
  IV=0
  IW=0
  IX=0
  IY=0
  IZ=0
  GO TO 2
ENDIF

!*          1.2         V - CHANNEL COMBINATIONS USED TO OBTAIN LAYER
!                       PWC FOR SURFACE-700MB, 700-500MB AND 500-300MB

IG1H = IBITS(KCHAN,23,1)
IG1M = IBITS(KCHAN,22,1)
IF(IG1H == 0.AND.IG1M == 0     )          THEN
  IV = 0
ELSEIF(IG1M == 1)          THEN
  IV = 1
ELSE
  IV = 2
ENDIF

!*          1.3         W - CHANNEL COMBINATIONS USED TO OBTAIN
!                       TROPOPAUSE TEMPERATURE AND PRESSURE

IG2H = IBITS(KCHAN,19,1)
IG2M = IBITS(KCHAN,18,1)
IF(IG2H == 0.AND.IG2M == 0     )          THEN
  IW = 0
ELSEIF(IG2H == 1)          THEN
  IW = 1
ELSE
  IW = 2
ENDIF

!*          1.4         X - CHANNEL COMBINATIONS USED TO OBTAIN OZONE

IG3H1 = IBITS(KCHAN,15,1)
IG3H2 = IBITS(KCHAN,14,1)
IG3M  = IBITS(KCHAN,13,1)
IF(IG3H1 == 0.AND.IG3H2 == 0.AND.IG3M  == 0     )         THEN
  IX = 0
ELSEIF(IG3H1 == 1)         THEN
  IF(IG3M == 1)               THEN
    IX = 1
  ELSE
    IX = 2
  ENDIF
ELSEIF(IG3H2 == 1)         THEN
  IF(IG3M == 1)               THEN
    IX = 3
  ELSE
    IX = 4
  ENDIF
ELSE
  IX = 1
ENDIF

!*          1.5         Y - CHANNEL COMBINATIONS USED TO OBTAIN
!                       TROPOSPHERIC LAYER TEMPERATURES

IG4H1 = IBITS(KCHAN,10,1)
IG4H2 = IBITS(KCHAN, 9,1)
IG4M  = IBITS(KCHAN, 8,1)
IG4S  = IBITS(KCHAN, 7,1)
IF(IG4H1 == 0.AND.IG4H2 == 0.AND.IG4M  == 0.AND.IG4S  == 0     )         THEN
  IY = 0
ELSEIF(IG4H1 == 1)         THEN
  IF(IG4M == 1)               THEN
    IY = 1
  ELSE
    IY = 3
  ENDIF
ELSEIF(IG4H2 == 1)         THEN
  IY = 2
ELSEIF (IG4M == 1)         THEN
  IY = 4
ELSE
  IY = 5
ENDIF

!*          1.6         Z - CHANNEL COMBINATIONS USED TO OBTAIN
!                       STRATOSPHERIC LAYER TEMPERATURES

IG5H = IBITS(KCHAN, 4,1)
IG5S = IBITS(KCHAN, 3,1)
IG5M = IBITS(KCHAN, 2,1)
IF(IG5H == 0.AND.IG5M == 0.AND.IG5S == 0     )          THEN
  IZ = 0
ELSEIF(IG5H == 1)          THEN
  IF(IG5M == 1.AND.IG5S == 1     )          THEN
    IZ = 1
  ELSEIF(IG5S == 1)          THEN
    IZ = 4
  ELSEIF(IG5M == 1)          THEN
    IZ = 2
  ELSE
    IZ = 5
  ENDIF
ELSEIF(IG5S == 1)          THEN
  IZ = 3
ELSE
  IZ = 6
ENDIF
!*
!-----------------------------------------------------------------------

!        2.             DATA PROCESSING TECHNIQUE
!                       -------------------------

2 CONTINUE

!*          2.1         MISSING VALUE FOR DATA PROCESSING TECHNIQUE

IF(KDPT == NBITS( 7).OR.KDPT ==      NMDI    )      THEN
  IA = 0
  IB = 0
  IC = 0
  GO TO 3
ENDIF

!*          2.2         A - CLEAR RADIANCES
!                       B - CLOUDY RADIANCES

ICLR  = IBITS(KDPT, 5,1)
IPCLD = IBITS(KDPT, 4,1)
ICLD  = IBITS(KDPT, 3,1)
IF(ICLR == 1)                  THEN
  IA = 1
  IB = 1
ELSEIF(IPCLD == 1)            THEN
  IA = 2
  IB = 1
ELSE
  IA = 0
  IB = 2
ENDIF

!*          2.3         C - STATISTICAL RETRIEVAL METHOD

IS = IBITS(KDPT, 6,1)
IF(IS == 1)                    THEN
  IC = 1
ELSE
  IC = 0
ENDIF
!*
!-----------------------------------------------------------------------

!        3.             INSTRUMENT SPECIFICATION
!                       ------------------------

3 CONTINUE

!*          3.1         INSTRUMENT SPECIFICATION PARAMETERS

KA = IA
KB = IB
KC = IC
KV = IV
KW = IW
KX = IX
KY = IY
KZ = IZ
!*
!-----------------------------------------------------------------------

!        4.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('TOVSHRIS',1,ZHOOK_HANDLE)
END SUBROUTINE TOVSHRIS
