SUBROUTINE UPDTA(KCMVNM,KREP,KDAT,KSTDLE,PDIST,LDLOCAL)

!**** SUBROUTINE UPDTA - UPDATE TABLES AND COUNTERS FOR LEVEL SELECTION

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        UPDATE TABLES AND COUNTERS FOR LEVEL SELECTION. UPPER AIR AND 2M
!     VARIABLES ARE DISTINQUISHED.

!**   INTERFACE
!     ---------

!     ***CALL UPDTA(KCMVNM,KREP,KDAT,KSTDLE,PDIST)
!     WHERE KCMVNM  = VARIABLE NUMBER
!           KREP    = POINTER OF THE REPORT
!           KDAT    = POINTER OF THE VARIABLE (IE. DATUM ENTRY)
!           KSTDLE  = THE INDEX OF CLOSEST STD-LEVEL
!           PDIST   = DISTANCE TO THE STD-LEVEL
!           LDLOCAL = .TRUE.  IF THE REPORT IF FROM THE LOCAL CMA
!                     .FALSE. IF THE REPORT IS FROM ANOTHER PE

!        UPDTA IS CALLED BY SUBROUTINE LEVEL

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR, NULOUT
USE YOMVNMB  , ONLY : NVNUMB
USE YOMLEVSE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KCMVNM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KREP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTDLE 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIST 
LOGICAL           ,INTENT(IN)    :: LDLOCAL 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!        1.       UPDATE TABLES FOR 2 METRE VARIABLES
!                 -----------------------------------

!*          1.1   SET 2M U-COM. POINTER AND COUNTER

IF (LHOOK) CALL DR_HOOK('UPDTA',0,ZHOOK_HANDLE)
IF(KCMVNM == NVNUMB(56)) THEN
  NVCOUN(KSTDLE,N2MUPNT) = NVCOUN(KSTDLE,N2MUPNT) + 1
  IF(NVCOUN(KSTDLE,N2MUPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (1) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (1) ')
  ENDIF
  NVPOIN(KSTDLE,N2MUPNT,NVCOUN(KSTDLE,N2MUPNT),1) = KREP
  NVPOIN(KSTDLE,N2MUPNT,NVCOUN(KSTDLE,N2MUPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,N2MUPNT,NVCOUN(KSTDLE,N2MUPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,N2MUPNT,NVCOUN(KSTDLE,N2MUPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,N2MUPNT,NVCOUN(KSTDLE,N2MUPNT)  ) = PDIST

!*          1.2   SET 2M V-COM. POINTER AND COUNTER

ELSEIF(KCMVNM == NVNUMB(57)) THEN
  NVCOUN(KSTDLE,N2MVPNT) = NVCOUN(KSTDLE,N2MVPNT) + 1
  IF(NVCOUN(KSTDLE,N2MVPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (2) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (2) ')
  ENDIF
  NVPOIN(KSTDLE,N2MVPNT,NVCOUN(KSTDLE,N2MVPNT),1) = KREP
  NVPOIN(KSTDLE,N2MVPNT,NVCOUN(KSTDLE,N2MVPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,N2MVPNT,NVCOUN(KSTDLE,N2MVPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,N2MVPNT,NVCOUN(KSTDLE,N2MVPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,N2MVPNT,NVCOUN(KSTDLE,N2MVPNT)  ) = PDIST

!*          1.3   SET 2M T POINTER AND COUNTER

ELSEIF(KCMVNM == NVNUMB(10)) THEN
  NVCOUN(KSTDLE,N2MTPNT) = NVCOUN(KSTDLE,N2MTPNT) + 1
  IF(NVCOUN(KSTDLE,N2MTPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (3) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (3) ')
  ENDIF
  NVPOIN(KSTDLE,N2MTPNT,NVCOUN(KSTDLE,N2MTPNT),1) = KREP
  NVPOIN(KSTDLE,N2MTPNT,NVCOUN(KSTDLE,N2MTPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,N2MTPNT,NVCOUN(KSTDLE,N2MTPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,N2MTPNT,NVCOUN(KSTDLE,N2MTPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,N2MTPNT,NVCOUN(KSTDLE,N2MTPNT)  ) = PDIST

!*          1.4   SET 2M RH POINTER AND COUNTER

ELSEIF ( KCMVNM == NVNUMB(5) .OR. KCMVNM == NVNUMB(11) .OR.&
   & KCMVNM == NVNUMB(7) ) THEN  
  NVCOUN(KSTDLE,N2MRPNT) = NVCOUN(KSTDLE,N2MRPNT) + 1
  IF(NVCOUN(KSTDLE,N2MRPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (4) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (4) ')
  ENDIF
  NVPOIN(KSTDLE,N2MRPNT,NVCOUN(KSTDLE,N2MRPNT),1) = KREP
  NVPOIN(KSTDLE,N2MRPNT,NVCOUN(KSTDLE,N2MRPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,N2MRPNT,NVCOUN(KSTDLE,N2MRPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,N2MRPNT,NVCOUN(KSTDLE,N2MRPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,N2MRPNT,NVCOUN(KSTDLE,N2MRPNT)  ) = PDIST

!     ------------------------------------------------------------------

!        2.       UPDATE TABLES FOR UPPER AIR VARIABLES
!                 -------------------------------------

!*          2.1   SET U-COM. POINTER AND COUNTER

ELSEIF(KCMVNM == NVNUMB( 1)) THEN
  NVCOUN(KSTDLE,NUPNT) = NVCOUN(KSTDLE,NUPNT) + 1
  IF(NVCOUN(KSTDLE,NUPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (5) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (5) ')
  ENDIF
  NVPOIN(KSTDLE,NUPNT,NVCOUN(KSTDLE,NUPNT),1) = KREP
  NVPOIN(KSTDLE,NUPNT,NVCOUN(KSTDLE,NUPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,NUPNT,NVCOUN(KSTDLE,NUPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,NUPNT,NVCOUN(KSTDLE,NUPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,NUPNT,NVCOUN(KSTDLE,NUPNT)  ) = PDIST

!*          2.2   SET V-COM. POINTER AND COUNTER

ELSEIF(KCMVNM == NVNUMB( 2)) THEN
  NVCOUN(KSTDLE,NVPNT) = NVCOUN(KSTDLE,NVPNT) + 1
  IF(NVCOUN(KSTDLE,NVPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (6) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (6) ')
  ENDIF
  NVPOIN(KSTDLE,NVPNT,NVCOUN(KSTDLE,NVPNT),1) = KREP
  NVPOIN(KSTDLE,NVPNT,NVCOUN(KSTDLE,NVPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,NVPNT,NVCOUN(KSTDLE,NVPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,NVPNT,NVCOUN(KSTDLE,NVPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,NVPNT,NVCOUN(KSTDLE,NVPNT)  ) = PDIST

!*          2.3   SET Z POINTER AND COUNTER

ELSEIF(KCMVNM == NVNUMB( 3)) THEN
  NVCOUN(KSTDLE,NZPNT) = NVCOUN(KSTDLE,NZPNT) + 1
  IF(NVCOUN(KSTDLE,NZPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (7) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (7) ')
  ENDIF
  NVPOIN(KSTDLE,NZPNT,NVCOUN(KSTDLE,NZPNT),1) = KREP
  NVPOIN(KSTDLE,NZPNT,NVCOUN(KSTDLE,NZPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,NZPNT,NVCOUN(KSTDLE,NZPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,NZPNT,NVCOUN(KSTDLE,NZPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,NZPNT,NVCOUN(KSTDLE,NZPNT)  ) = PDIST

!*          2.4   SET T POINTER AND COUNTER

ELSEIF( KCMVNM == NVNUMB( 8)) THEN
  NVCOUN(KSTDLE,NTPNT) = NVCOUN(KSTDLE,NTPNT) + 1
  IF(NVCOUN(KSTDLE,NTPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (8) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (8) ')
  ENDIF
  NVPOIN(KSTDLE,NTPNT,NVCOUN(KSTDLE,NTPNT),1) = KREP
  NVPOIN(KSTDLE,NTPNT,NVCOUN(KSTDLE,NTPNT),2) = KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,NTPNT,NVCOUN(KSTDLE,NTPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,NTPNT,NVCOUN(KSTDLE,NTPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,NTPNT,NVCOUN(KSTDLE,NTPNT)  ) = PDIST

!*          2.5   SET Q POINTER AND COUNTER

ELSEIF ( KCMVNM == NVNUMB(63) ) THEN
  NVCOUN(KSTDLE,NRPNT) = NVCOUN(KSTDLE,NRPNT) + 1
  IF(NVCOUN(KSTDLE,NRPNT) > NLLNDA) THEN
    WRITE(NULERR,'('' NLLNDA EXCEEDED IN UPDTA (9) '')')
    CALL FLUSH(NULOUT)
    CALL ABOR1(' UPDTA : NLLNDA EXCEEDED (9) ')
  ENDIF
  NVPOIN(KSTDLE,NRPNT,NVCOUN(KSTDLE,NRPNT),1) =  KREP
  NVPOIN(KSTDLE,NRPNT,NVCOUN(KSTDLE,NRPNT),2) =  KDAT
  IF(LDLOCAL) THEN
    NVPOIN(KSTDLE,NRPNT,NVCOUN(KSTDLE,NRPNT),4) = 1
  ELSE
    NVPOIN(KSTDLE,NRPNT,NVCOUN(KSTDLE,NRPNT),4) = -1
  ENDIF
  GVDIST(KSTDLE,NRPNT,NVCOUN(KSTDLE,NRPNT)  ) =  PDIST

!*          2.6   IGNORE SEA SURFACE TEMPERATURE

ELSEIF ( KCMVNM == NVNUMB(12) ) THEN
  IF (LHOOK) CALL DR_HOOK('UPDTA',1,ZHOOK_HANDLE)
  RETURN

!*          2.7   UNKNOWN VARIABLE

ELSE
  WRITE(NULERR,'('' UNKNOWN VARIABLE IN UPDTA '',I3)') KCMVNM
  CALL ABOR1('UPDTA: UNKNOWN VARIABLE')
ENDIF

!*
!     ------------------------------------------------------------------

!        3.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('UPDTA',1,ZHOOK_HANDLE)
END SUBROUTINE UPDTA
