SUBROUTINE UPECMA(KLEN)

!**** *UPECMA* - UPDATES THE EXTENDED CMA FILE FOR COMPRESSION

!          HEIKKI JARVINEN      ECMWF       15/SEP/1995
!          DRASKO VASILJEVIC    ECMWF       04/NOV/1999

!*    PURPOSE
!     -------

!       AFTER ALL THE SCREENING DECISIONS THE EXTENDED CMA (ECMA)
!     IS UPDATED FOR THE COMPRESSION.
!     1) REPORT:
!        - ACTIVATE IF NOT REJECTED/BLACKLISTED
!        - REJECT IF ALL DATA REJECTED
!        - FOR SCATTEROMETER REPORTS TAKE INTO ACCOUNT THE THINNING
!          FLAG THAT HAS BEEN SET IN COH (WHEN CMA-FILE IS CREATED)
!     2) DATUM
!        - ACTIVATE IF NOT REJECTED/BLACKLISTED
!        - REJECT IF THE REPORT IS NOT ACTIVE
!        - REJECT TOVS THICKNESSES IF "NOT PREDEFINED LAYER"

!*    INTERFACE
!     ---------

!*      CALL UPECMA(KLEN)
!     WHERE KLEN  = DB LEN

!       UPECMA IS CALLED FROM SCREEN

!*    METHOD
!     ------

!        LOOP OVER THE CMA AND CHECK THE REPORT/DATUM STATUS. HIT PROPER
!     EVENT FLAG.

!     EXTERNALS
!     ---------

!        NGERSTA - GET REPORT STATUS
!        NGEDSTA - GET DATUM STATUS

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        T.Auligne     09-Dec-2005 Increase observation error stats 
!                                  for passive data needed in VarBC
!        C.Payan       10-Apr-2007 .not.LMFSCREEN removed for NSCATT

!*   ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0B  , ONLY : LECMWF
USE PARCMA   , ONLY : JPMXRE2  ,JPMXDE2
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMDIMO  , ONLY : NOBTOT
USE YOMCOCTP , ONLY : NSATEM   ,NSCATT  ,NGTHRB
USE YOMNMEV  , ONLY : NR1EVENT ,ND1EVENT
USE YOMVNMB  , ONLY : NVNUMB
USE YOMSCC   , ONLY : LSCDPR
USE YOMOBS   , ONLY : LMFSCREEN,L_F_SCR_WORK_TS
USE YOMCMBDY
USE YOMDB
USE YOMVAR   , ONLY : LVARBC

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM) :: IRSTA(4),IDSTA(4)
INTEGER(KIND=JPIM) :: IREPEV2(JPMXRE2),IDATEV2(JPMXDE2)

INTEGER(KIND=JPIM) :: ICMNLV, ILEV, INRJLV, INSTA, &
 & IOBTYP, ICDTYP, IVAR, &
 & JNLV, JOBS, ID2COUN, ICMRDFL, IWHITEFL, &
 & INUMACTIVEB  

LOGICAL :: LL_CYCLE_OBS_LOOP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ngedeve2.intfb.h"
#include "ngedsta.intfb.h"
#include "ngereve2.intfb.h"
#include "ngersta.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ----------------------------------------------------------------
!        0.       INITIALIZE
!                 ----------
!*
!           1.0   INITIALIZE OBSERVATION ARRAY POINTERS

IF (LHOOK) CALL DR_HOOK('UPECMA',0,ZHOOK_HANDLE)
WRITE(NULOUT,'(''     UPDATING ECMA ODB FOR COMPRESSION '')')
IF(.NOT.L_F_SCR_WORK_TS) THEN
  IF(KLEN /= NOBTOT) THEN
    WRITE(NULERR,'('' *** ERROR IN UPECMA '')')
    WRITE(NULERR,'(''     DB LEN: '',I10)') KLEN
    WRITE(NULERR,'(''     NOBTOT: '',I10)') NOBTOT
    CALL ABOR1('UPECMA ; DB LEN AND NOBTOT DIFFER')
  ENDIF
ENDIF
!*
!*
!     ------------------------------------------------------------------

!        2.       LOOP OVER THE OBSERVATION ARRAY (ECMA)
!                 --------------------------------------
!*
!           2.0   START LOOP

OBS_LOOP: DO JOBS = 1,KLEN

!*
!           2.1   COLLECT ALL RELEVANT HEADER INFORMATION

  LL_CYCLE_OBS_LOOP = .FALSE.
  ICMNLV = MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  ICDTYP = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)
  IOBTYP = ROBHDR(JOBS,MDBOTP)

!*
!           2.2   GET REPORT STATUS

  CALL NGERSTA(INT(ROBHDR(JOBS,MDBRST)),INSTA,IRSTA)
!*
!           2.3   IF REPORT REJECTED/BLACKLISTED, REJECT ACTIVE THE DATA

  IF(IRSTA(3) == 1.OR. IRSTA(4) == 1) THEN
    DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
      CALL NGEDSTA(INT(ROBODY(JNLV,MDBDSTA)),INSTA,IDSTA)
      IF(IDSTA(3) /= 1 .AND. IDSTA(4) /= 1) THEN
        ROBODY(JNLV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(JNLV,MDBDSTA))
        ROBODY(JNLV,MDBDEV1) = ZCHEV1( ROBODY(JNLV,MDBDEV1),ND1EVENT(26))
      ENDIF
    ENDDO
    LL_CYCLE_OBS_LOOP = .TRUE.
  ENDIF
!*
!           2.4   IF DATA BLACKLISTED, MAKE IT REJECTED TOO 
!                 IF DATA PASSIVE, INCREASE SigmaO to 33.33K (VarBC use)

  if (LECMWF) then
    DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
      CALL NGEDSTA(INT(ROBODY(JNLV,MDBDSTA)),INSTA,IDSTA)
      IF ( IDSTA(4) == 1 .AND. IDSTA(3) /= 1 ) THEN
        ROBODY(JNLV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(JNLV,MDBDSTA))
      ENDIF
      IF (LVARBC .AND. IOBTYP == NSATEM .AND. ICDTYP == NGTHRB .AND. &
      & ROBODY(JNLV,MDBVNM) == NVNUMB(50) .AND. IDSTA(2) == 1) THEN
        ROBODY(JNLV,MDBFOE)  = 33.33_JPRB
      ENDIF
    ENDDO
  endif
  IF (LL_CYCLE_OBS_LOOP) GOTO 212
!*
!           2.5   REJECT SCATTEROMETER REPORTS FLAGGED IN COH

  IF(IOBTYP == NSCATT) THEN
    CALL NGEREVE2(INT(ROBHDR(JOBS,MDBREV2)),IOBTYP,ICDTYP,IREPEV2)
    IF(IREPEV2(1) == 1) THEN
      ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT(ROBHDR(JOBS,MDBRST))
      IF(LSCDPR) THEN
        WRITE(NULOUT,'('' SCATT REP.EV2: '',32(I1,1X))') IREPEV2
      ENDIF
      DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
        ROBODY(JNLV,MDBDSTA) = ZCHSTAT_REJECT(ROBODY(JNLV,MDBDSTA))
        ROBODY(JNLV,MDBDEV1) = ZCHEV1(ROBODY(JNLV,MDBDEV1),ND1EVENT(26))
      ENDDO
      LL_CYCLE_OBS_LOOP = .TRUE.
    ENDIF
  ENDIF
  IF (LL_CYCLE_OBS_LOOP) GOTO 212
!*
!           2.6   LOOP OVER THE BODY ENTRIES; INITIALIZE COUNTER

  INRJLV = 0
  DAT_LOOP: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
    ILEV = JNLV
    IVAR = ROBODY(ILEV,MDBVNM)

!*
!           2.7   INCREASE COUNTER IF DATUM STATUS REJECTED/BLACKLISTED
!                 KEEP ACTIVE/PASSIVE DATA

    CALL NGEDSTA(INT(ROBODY(ILEV,MDBDSTA)),INSTA,IDSTA)
    IF ( IDSTA(3) == 1 .OR. IDSTA(4) == 1 ) THEN

!*  check if datum is white listed (only at Meteo France)

      IF ( LMFSCREEN ) THEN
        ICMRDFL  = ROBODY(ILEV,MDBRDFL)
        IWHITEFL = IBITS(ICMRDFL,NCMBVFBP+NCMBORBP,NCMBOROC)
        IF ( IWHITEFL == 0 ) THEN
          INRJLV = INRJLV + 1
          CYCLE DAT_LOOP
        ENDIF
      ELSE
        INRJLV = INRJLV + 1
        CYCLE DAT_LOOP
      ENDIF
    ENDIF
!*
!           2.8   REJECT TOVS THICKNESSES IF "NOT PREDEFINED LAYER"

    IF ( .NOT.LMFSCREEN ) THEN
      IF(IOBTYP == NSATEM .AND. IVAR == NVNUMB( 4)) THEN
        CALL NGEDEVE2(INT(ROBODY(ILEV,MDBDEV2)),ID2COUN,IDATEV2)
        IF(IDATEV2(1) == 1) THEN
          ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
          INRJLV = INRJLV + 1
          IF(LSCDPR) THEN
            WRITE(NULOUT,'('' TOVS DAT.EV2: '',32(I1,1X))') IDATEV2
          ENDIF
          CYCLE DAT_LOOP
        ENDIF
      ENDIF
    ENDIF
!*
!           2.9   DATUM APPARENTLY AN ACTIVE ONE

    ROBODY(ILEV,MDBDSTA) = ZCHSTAT_ACTIVE( ROBODY(ILEV,MDBDSTA))

  ENDDO DAT_LOOP
!*
!           2.10  IF ALL DATA REJECTED, REJECT THE REPORT AS WELL
!                 AND HIT A PROPER EVENT

  IF(INRJLV == ICMNLV) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT( 2))
!*
!           2.11  REPORT APPARENTLY AN ACTIVE ONE

  ELSE
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_ACTIVE( ROBHDR(JOBS,MDBRST))
  ENDIF
!*
!           2.12  COUNT NO. OF ACTIVE BODIES LEFT
  212  CONTINUE
  INUMACTIVEB = 0
  CALL NGERSTA(INT(ROBHDR(JOBS,MDBRST)),INSTA,IRSTA)
  IF (IRSTA(1) == 1) THEN
    DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
!!      CALL NGEDSTA(INT(ROBODY(JNLV,MDBDSTA)),INSTA,IDSTA)
      IDSTA(1) = IBITS(INT(ROBODY(JNLV,MDBDSTA)), NCMDACBP, NCMDACOC)
      IF (IDSTA(1) == 1) INUMACTIVEB = INUMACTIVEB + 1
    ENDDO
  ENDIF
  ROBHDR(JOBS,MDB_NUMACTIVEB_AT_HDR) = INUMACTIVEB
!*
!           2.13  CLOSE THE LOOP

ENDDO OBS_LOOP
!*
!     -----------------------------------------------------------------

!        3.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('UPECMA',1,ZHOOK_HANDLE)
END SUBROUTINE UPECMA
