SUBROUTINE windaux

!-- Create (ff,dd)/(dd,ff) into aux_1 & aux_2 -entries of ECMA/ODB for u/v

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIMO  , ONLY : NACTIM
USE YOMLUN   , ONLY : NULERR 
USE YOMVNMB  , ONLY : NVNUMB
USE YOMANCS  , ONLY : RMDI
USE LOCAL_TRAFOS
USE YOMDB
USE LOCAL_TRAFOS

implicit none

INTERFACE
#include "abortdb.h"
#include "getdb.h"
#include "putdb.h"
END INTERFACE

INTEGER(KIND=JPIM) :: JSLOT, JOBS, IRET, IVARNO(2), ISEQNO(2), INFO(0)
REAL(KIND=JPRB) :: ZPRESS(2), ZU, ZV, ZFORCE, ZDIR, ZINFO(0)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('WINDAUX',0,ZHOOK_HANDLE)

IF (JP_NUMAUX < 2) THEN
  WRITE(NULERR,*)'***PROGRAMMING ERROR IN WINDAUX: JP_NUMAUX=',JP_NUMAUX,' < 2'
  CALL ABORTDB('WINDAUX','JP_NUMAUX < 2',JP_NUMAUX)
ENDIF

TIMESLOT_LOOP: DO JSLOT = 1, NACTIM
  CALL GETDB('WINDAUX',0,IRET,INFO,0,ZINFO,0, &
      & -1, JSLOT, -1, -1, -1, -1)

  IF (MOD(IRET,2) /= 0) THEN
    WRITE(NULERR,*)'***ERROR IN WINDAUX: JSLOT=',JSLOT,', IRET=',IRET
    CALL ABORTDB('WINDAUX','NO. OF (U,V)-ENTRIES IN THE TIMESLOT NOT AN EVEN NUMBER',IRET)
  ENDIF

  !-- We assume that for a given (u/u10m) at its pressure level,
  !   a (v/v10m) is the next entry at the same pressure level.

  AUX_INIT: DO JOBS=1,IRET
    ROBODY(JOBS,MDB_AUX_AT_BODY(1:2)) = RMDI
  ENDDO AUX_INIT

  UV_LOOP: DO JOBS=1,IRET-1,2
    ISEQNO(1) = ROBODY(JOBS  ,MDBONM) ! seqno@hdr
    ISEQNO(2) = ROBODY(JOBS+1,MDBONM)
    IF (ISEQNO(1) /= ISEQNO(2)) CYCLE UV_LOOP ! Short-circuit
    ZPRESS(1) = ROBODY(JOBS  ,MDBPPP) ! press@body
    ZPRESS(2) = ROBODY(JOBS+1,MDBPPP)
    IF (ZPRESS(1) /= ZPRESS(2)) CYCLE UV_LOOP ! Short-circuit
    IVARNO(1) = ROBODY(JOBS  ,MDBVNM) ! varno@body
    IVARNO(2) = ROBODY(JOBS+1,MDBVNM)
    IF ((IVARNO(1) == NVNUMB( 1).AND.IVARNO(2) == NVNUMB( 2)).OR. &
     &  (IVARNO(1) == NVNUMB(56).AND.IVARNO(2) == NVNUMB(57)).OR. &
     &  (IVARNO(1) == NVNUMB(62).AND.IVARNO(2) == NVNUMB(61))) THEN
      ZU = ROBODY(JOBS  ,MDBVAR) ! obsvalue@body
      ZV = ROBODY(JOBS+1,MDBVAR)
      ZFORCE=UV2FF(ZU,ZV)
      ZDIR  =UV2DD(ZU,ZV)
      !-- U-COMPONENT: AUX_1 := WIND FORCE, AUX_2 := WIND DIRECTION IN DEGREES
      !-- V-COMPONENT: AUX_2 := WIND FORCE, AUX_1 := WIND DIRECTION IN DEGREES
      ROBODY(JOBS  ,MDB_AUX_AT_BODY(1)) = ZFORCE
      ROBODY(JOBS+1,MDB_AUX_AT_BODY(1)) = ZDIR
      ROBODY(JOBS  ,MDB_AUX_AT_BODY(2)) = ZDIR
      ROBODY(JOBS+1,MDB_AUX_AT_BODY(2)) = ZFORCE
    ENDIF
  ENDDO UV_LOOP

  CALL PUTDB('WINDAUX',0,IRET,INFO,0,ZINFO,0)
ENDDO TIMESLOT_LOOP

IF (LHOOK) CALL DR_HOOK('WINDAUX',1,ZHOOK_HANDLE)
END SUBROUTINE windaux
