SUBROUTINE SIPC_ATTACH(CDPOOLNAM,KPOOLHANDLE)

!**** *SIPC_Attach*  

!     Purpose:
!     -------
!     Attach model to pool 'cdpoolnam' having handle 'kpoolhandle' 

!**   Interface:
!     ---------
!       *CALL*  *SIPC_Attach (cdpoolnam, kpoolhandle)* 

!     Input:
!     -----
!                cdpoolnam   : Pool name

!     Output:
!     ------
!                kpoolhandle : Pool handle

!     Workspace:
!     ---------
!     None

!     Externals:
!     ---------
!     SVIPC_open

!     History:
!     -------
!       Version   Programmer     Date      Description
!       -------   ----------     ----      -----------  
!       Oasis 2.2 S. Valcke      97/08/25  created
!                 S. Valcke      98/06/02  adapted to F90 for arpege

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
!* ---------------------------- MODULE inclusion -------------------

USE YOMLUN   , ONLY : NULOUT
!* ---------------------------- Argument declarations -------------------

IMPLICIT NONE

CHARACTER (LEN = 9),INTENT(IN)   :: CDPOOLNAM
INTEGER(KIND=JPIM) ,INTENT(OUT)  :: KPOOLHANDLE
!* ---------------------------- Local declarations ----------------------

INTEGER(KIND=JPIM) :: imrc
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!* ---------------------------- Poema verses ----------------------------

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!*    1. Initializations
!        ---------------

IF (LHOOK) CALL DR_HOOK('SIPC_ATTACH',0,ZHOOK_HANDLE)
WRITE (NULOUT,*) ' '
WRITE (NULOUT,*) &
 & '           ROUTINE SIPC_Attach '  
WRITE (NULOUT,*) &
 & '           *******************     *******'  
WRITE (NULOUT,*) ' '

!* Zeroes error codes

imrc = 0

!*    2. Attach to pool 
!        --------------

CALL SVIPC_open(kpoolhandle, cdpoolnam, 0, imrc)

!* Find error if any

IF (imrc < 0) THEN
  WRITE(NULOUT,*) 'Problem in opening the pool', cdpoolnam
  CALL ABOR1('STOP in SIPC_Attach')
ELSE
  WRITE(NULOUT,*) 'Pool for field exchange is open:', cdpoolnam, kpoolhandle
ENDIF

!*    3. End of routine
!        --------------

WRITE (NULOUT,*) ' '
WRITE (NULOUT,*) &
 & '          --------- End of routine SIPC_Attach ---------'  
CALL FLUSH(NULOUT)
IF (LHOOK) CALL DR_HOOK('SIPC_ATTACH',1,ZHOOK_HANDLE)
END SUBROUTINE SIPC_ATTACH
