SUBROUTINE SIPC_WRITE_MODEL(KINDEX,KDIM,KINFOS,POUT_FIELD)
!****
!**** *SIPC_Write_Model*  

!     Purpose:
!     -------
!     Write model output field and infos to field-specific pool

!**   Interface:
!     ---------
!       *CALL*  *SIPC_Write_Model (kindex, kdim, cdjobnam, kinfos, pout_field)

!     Input:
!     -----
!        kindex : index of field to be written in total number of output fields
!        kdim   : dimension of field to be written 

!     Output:
!     ------
!        kinfos(1)    :  Initial date (integer)
!        kinfos(2)    :  Iteration number (integer)
!        kinfos(3)    :  Time since start (integer)
!        pout_field   :  Field to be written

!     Workspace:
!     ---------
!     None

!     Externals:
!     ---------
!     SVIPC_write

!     History:
!     -------
!       Version   Programmer     Date      Description
!       -------   ----------     ----      -----------  
!       2.2       S. Valcke      97/08/25  created
!                 S. Valcke      98/06/09  adapted in F90 for arpege

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!* ---------------------------- MODULE inclusion --------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_OAS   , ONLY : cpmodinf, cpjobnam
USE PAR_SIPC  , ONLY : jpbyteint, jpbyterea, jpbytecha
USE YOM_INPC  , ONLY : mpoolwrit
USE YOMLUN    , ONLY : NULOUT

!* ---------------------------- Argument declarations -------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KINDEX 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KINFOS(3)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT_FIELD(KDIM)
!* ---------------------------- Local declarations ---------------------
INTEGER(KIND=JPIM) :: isizeout, imrc
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!* ---------------------------- Poema verses ----------------------------

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!*    1. Initializations
!        ---------------

IF (LHOOK) CALL DR_HOOK('SIPC_WRITE_MODEL',0,ZHOOK_HANDLE)
WRITE (NULOUT,*) ' '
WRITE (NULOUT,*) &
 & '           ROUTINE SIPC_Write_Model '  
WRITE (NULOUT,*) &
 & '           *******************     *******'  
WRITE (NULOUT,*) ' '

!*    2. Write encapsulated infos in field-specific shared memory pool
!        (experiment name, initial date, time since start, iteration number)
!        -------------------------------------------------------------------

IF(cpmodinf == 'YES') then
  imrc = 0
  isizeout = 3*jpbytecha
  CALL SVIPC_write(mpoolwrit(kindex), cpjobnam, isizeout, imrc)
  isizeout = 3*jpbyteint
  CALL SVIPC_write(mpoolwrit(kindex), kinfos, isizeout, imrc)

!*        Find error if any

  IF (imrc < 0) THEN
    WRITE(NULOUT,*)&
     & 'Problem in writing encapsulated infos for field',&
     & kindex   
    CALL ABOR1('STOP in SIPC_Write_Model')
  ENDIF

  WRITE(NULOUT,*) 'Write encapsulated infos for field:',kindex

ENDIF

!*    3. Write field in field-specific shared memory pool
!        -----------------------------------------------
isizeout = kdim*jpbyterea
CALL SVIPC_write(mpoolwrit(kindex), pout_field, isizeout, imrc)

!*    Find error and STOP if any

IF (imrc < 0) THEN
  WRITE(NULOUT,*) &
   & 'Problem in writing in the SHM pool for field',&
   & kindex   
  CALL ABOR1('STOP in SIPC_Write_Model')
ENDIF

WRITE(NULOUT,*) 'Write in the SHM pool for field :',kindex
WRITE(NULOUT,*)  pout_field(1), pout_field(kdim)

!*    4. End of routine
!        --------------

WRITE (NULOUT,*) ' '
WRITE (NULOUT,*)' --- End of routine SIPC_Write_model ---'
CALL FLUSH(NULOUT)
IF (LHOOK) CALL DR_HOOK('SIPC_WRITE_MODEL',1,ZHOOK_HANDLE)
END SUBROUTINE SIPC_WRITE_MODEL
