SUBROUTINE SLAB(PLSM)

!**** * SLAB *

!     PURPOSE.
!     --------
!       MODELE INTEGRAL DE COUCHE DE MELANGE OCEANIQUE.  (LGAUPQ)
!       MODULE CHARGE DE L'EVOLUTION DE LA TEMPERATURE ET DE LA PROFONDEUR DE
!       LA COUCHE DE MELANGE. MODULE DESTINE A ETRE COUPLE AU MODELE
!       ATMOSPHERIQUE ARPEGE-CLIMAT.

!**   INTERFACE.
!     ----------
!       *CALL* *SLAB(PLSM)*

!     EXPLICIT ARGUMENTS :
!     --------------------
!       PLSM : LAND-SEA INDEX.

!     IMPLICIT ARGUMENTS :
!     --------------------
!       FCS: FLUX DE CHALEUR SENSIBLE CUMULE (J/M2)
!       FLATW: FLUX DE CHALEUR LATENTE CUMULE AU DESSUS DE L'EAU (J/M2)
!       FLATI: FLUX DE CHALEUR LATENTE CUMULE AU DESSUS DE LA GLACE (J/M2)
!       FSOL: FLUX SOLAIRE CUMULE (J/M2)
!       FNSOL: FLUX NON SOLAIRE CUMULE (J/M2)
!       TAUX: STRESS ZONAL CUMULE (N/M2*S)
!       TAUY: STRESS MERIDIEN CUMULE (N/M2*S)
!       GAMAT: DT/DZ A LA BASE DE LA CMO
!       GAMAD: DU/DZ A LA BASE DE LA CMO

!       VARIABLES PRONOSTIQUES EN INPUT DU MODELE A L'INSTANT T
!       *******************************************************
!       SST: TEMPERATURE DE SURFACE EN ENTREE (C)
!       HMEL: PROFONDEUR DE LA COUCHE DE MELANGE EN ENTREE (M)

!       VARIABLES PRONOSTIQUES EN OUTPUT DU MODELE A L'INSTANT T+DT
!       ***********************************************************
!       SST: TEMPERATURE DE SURFACE EN ENTREE (C)
!       HMEL: PROFONDEUR DE LA COUCHE DE MELANGE EN ENTREE (M)

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!     AUTHOR.
!     -------
!        CNRM/GMGEC/MEMO/Herve Giordani

!     Modifications.
!     --------------
!        Modified by P. Marquet 2002-02 : Many "f90-bugs"...
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMGEM   , ONLY : NGPTOT

USE YOMCOM   , ONLY : OMLDTH  ,GTTLIN  ,SSTPRE  
USE YOMCPL   , ONLY : FCSTSU  ,FCSTSV  ,FCCHAS  ,FCRSOS
USE YOMCST   , ONLY : RCW

IMPLICIT NONE

!     DUMMY ARGUMENT (REAL)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(NGPTOT) 
INTEGER(KIND=JPIM) :: IS

REAL(KIND=JPRB) :: ZRHOO, ZBILAN, ZEVOLHBIL, ZEVOLHSTR, ZEVOLH_TOT,&
 & ZEVOLTBIL, ZEVOLT_TOT, ZGAMAD, ZGAMAT, ZHMEL, &
 & ZLSM, ZSST, ZTAU  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     CONSTANTES USUELLES
IF (LHOOK) CALL DR_HOOK('SLAB',0,ZHOOK_HANDLE)
ZRHOO=1024._JPRB

!     BOUCLE SPATIALE
DO IS=1,NGPTOT

  ZLSM=PLSM(IS)
  ZSST=SSTPRE(IS)
  ZHMEL=OMLDTH(IS)
  ZGAMAT=GTTLIN(IS)
  ZGAMAD=GTTLIN(IS)

! CALCUL SUR MER SI DONNEES DISPONIBLES.
  IF (ZLSM < 0.5_JPRB) THEN

!        CALCUL DU FORCAGE MECANIQUE MOYEN EN SURFACE SUR LA PERIODE DE CUMUL
    ZTAU = SQRT(FCSTSU(IS)**2+FCSTSV(IS)**2)

!        CALCUL DU BILAN THERMIQUE MOYEN EN SURFACE SUR LA PERIODE DE CUMUL
    ZBILAN = FCRSOS(IS,1) + FCCHAS(IS,1)

!        CALCUL DES EVOLUTIONS DE LA SST ET DE H DUES AU BILAN
!        THERMIQUE DE SURFACE

!        SST
    ZEVOLTBIL = 1.0_JPRB/(ZRHOO*RCW*ZHMEL)*ZBILAN

!        HMEL
    ZEVOLHBIL = -1.0_JPRB/(ZRHOO*RCW*ZGAMAT*ZHMEL)*ZBILAN

!        CALCUL DES TENDANCE ET EVOLUTION DE H DUES AU STRESS
    ZEVOLHSTR = 1.0_JPRB/(ZRHOO*ZGAMAD*ZHMEL)*ZTAU

!        CALCUL DES TENDANCES ET EVOLUTIONS TOTALES DE LA SST ET DE H

!        SST
    ZEVOLT_TOT = ZEVOLTBIL

!        HMEL
    ZEVOLH_TOT = ZEVOLHBIL + ZEVOLHSTR

!        AVANCE TEMPORELLE
    SSTPRE(IS) = ZSST + ZEVOLT_TOT
    OMLDTH(IS) = ZHMEL + ZEVOLH_TOT

  ENDIF  !  (ZLSM < 0.5_JPRB)

ENDDO  !  IS=1,NGPTOT

IF (LHOOK) CALL DR_HOOK('SLAB',1,ZHOOK_HANDLE)
END SUBROUTINE SLAB
