SUBROUTINE WRCOM

!**** * WRCOM  * - ROUTINE TO WRITE THE COUPLED FIELDS FOR THE COM METHOD 

!     PURPOSE.
!     --------
!        WRITES YOMCOM ON DISK

!**   INTERFACE.
!     ----------
!        *CALL* *WRCOM*

!     EXPLICIT ARGUMENTS :  NONE
!     --------------------

!     IMPLICIT ARGUMENTS :
!     --------------------
!        COMMON  YOMCOM

!     METHOD.
!     -------
!     EXTERNALS.
!     ----------

!         FAITOU
!         FANDAR    (SEE MANIPULATION DE FICHIER ARPEGE)
!         FAIENC
!         FAIRME
!         ETC.

!     AUTHOR.
!     -------
!        CNRM/GMGEC/EAC/JPHP

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 99-02-10 from WRCODM
!        Modified 03-10-28 by J.P. Piedelievre : bugfix
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        R. El Khatib : 23-Oct-2008 use suofname
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NPROMA
USE YOMDYN   , ONLY : TSTEP
USE YOMOP    , ONLY : CNMCA
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMCT1   , ONLY : LRFILAF
USE YOMCT3   , ONLY : NSTEP
USE YOMGEM   , ONLY : NGPTOT   
USE YOMMP    , ONLY : MYPROC
USE YOMCOM   , ONLY : OMLDTH   ,GTTLIN   
USE SURFACE_FIELDS_MIX, ONLY : SP_RR, YSP_RR

IMPLICIT NONE

CHARACTER(LEN=16) :: CLFNSH

INTEGER(KIND=JPIM) :: JPNCHP

PARAMETER(JPNCHP=99)
CHARACTER (LEN = 16) ::  CLNCHP(JPNCHP)
REAL(KIND=JPRB) :: ZREALG(NDLON*NDGLG)
REAL(KIND=JPRB) :: ZTEMP(NGPTOT)
INTEGER(KIND=JPIM) :: IDATEF(11)
LOGICAL :: LLOPENED

INTEGER(KIND=JPIM) :: IFLD, IFLW, INBARI, INBARP, INIMES, IOPROC,&
 & IREP, ISEC, ISTEP, JNULCL  

INTEGER(KIND=JPIM) :: IDIMG, INULCL, JKGLO, IST, IEND, JROF, IROF, IBLK
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "diwrgrid.intfb.h"
#include "suofname.intfb.h"

!*       1.    PREPARATIONS.
!              --------------

IF (LHOOK) CALL DR_HOOK('WRCOM',0,ZHOOK_HANDLE)
ISTEP=MAX(0,NSTEP+1)

!*       1.1   LISTE DES CHAMPS

CLNCHP(1)='SURFTEMPERATURE'
CLNCHP(2)='PROFMIXLAY.DEPTH'
CLNCHP(3)='PROFGRT.THERMOCL'

!*       1.2   PROCESSEUR D'IO
! IOPROC MUST BE 1
IOPROC=1
IDIMG=NDGLG*NDLON
IF (MYPROC == IOPROC) THEN
!*       1.3   OPEN FILES

  CALL SUOFNAME(KSTEP=ISTEP,CDLABEL='ICMCM',CDOFNAME=CLFNSH)
!   VALEUR MIN DE INBARP COMME DANS CPREP2
  INBARP=1
  INIMES=1
  INBARI=0
  DO JNULCL=1,99
    INQUIRE(UNIT=JNULCL,OPENED=LLOPENED)
    IF (.NOT.LLOPENED) EXIT
  ENDDO
  INULCL=JNULCL
  CALL FAITOU(IREP,INULCL,.TRUE.,CLFNSH,'UNKNOWN',.TRUE.,.TRUE.,&
   & INIMES,INBARP,INBARI,CNMCA)  
  CALL LFIMST(IREP,INULCL,.FALSE.)

!*       1.4   DECIDE ON SUITABLE TIME UNIT
  ISEC=NINT(REAL(ISTEP,JPRB)*TSTEP)

!    YEAR
  IDATEF(1)=NINDAT/10000
!    MONTH
  IDATEF(2)=(NINDAT-10000*IDATEF(1) )/100
!    DAY IN THE MONTH
  IDATEF(3)=NINDAT-10000*IDATEF(1)-100*IDATEF(2)
!    HOUR IN THE DAY   (0 OR 12)
  IDATEF(4)=NSSSSS/3600
!    MINUTE IN THE HOUR  (0)
  IDATEF(5)=0

  ISEC=ISEC/3600
  IF (ISEC <= 0) THEN
!  ANALYSE INITIALISEE
    IDATEF(6)=1
    IDATEF(7)=0
    IDATEF(8)=0
    IDATEF(9)=1
  ELSE
!  PREVISION A N HEURES D ECHEANCE (OU A N JOURS)
    IF (ISEC < 65000) THEN
      IDATEF(6)=1
      IDATEF(7)=ISEC
    ELSE
      IDATEF(6)=2
      IDATEF(7)=ISEC/24
    ENDIF
    IDATEF(8)=0
    IDATEF(9)=10
  ENDIF

!   D'APRES DOC GRIB, TABLE 5 (ECMWF), IDATEF(10)=N POUR FICHIER DE
!   MOYENNE OU CUMUL DE N PREVISIONS OU ANALYSES (AVEC IDATEF(9)=113 OU 114)
!   CE QUI EST ICI ADAPTE POUR FLUX PHYSIQUES CUMULES TOUS LES PAS DE TEMPS:
!   N EST LE NOMBRE DE PAS DE TEMPS DU CUMUL

  IDATEF(10)=ISTEP
  IDATEF(11)=0
  CALL FANDAR(IREP,INULCL,IDATEF)

!  IDENTIFICATEUR DES FICHIERS D EBAUCHES
  CALL FAUTIF (IREP,INULCL,'ARP')

ENDIF ! MYPROC == 1

!     ---------------------------------------------------------------

!*    2.0   WRITE GRID POINT FIELDS ON *FA* FILE
!     -------------------------------------
IFLD=0

!* TEMPERATURE DE SURFACE

IFLW=1
IFLD=IFLD+1
DO JKGLO=1,NGPTOT,NPROMA
  IST=1
  IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBLK=(JKGLO-1)/NPROMA+1
  DO JROF=IST,IEND
    IROF=JROF+JKGLO-1
    ZTEMP(IROF)=SP_RR(JROF,YSP_RR%YT%MP0,IBLK)
  ENDDO
ENDDO
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,ZTEMP)
ELSE
  ZREALG(:NGPTOT)=ZTEMP
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  CALL FAIENC(IREP,INULCL,CLNCHP(IFLW)(:4),0,CLNCHP(IFLW)(5:),&
   & ZREALG,.FALSE.)  
ENDIF

! PROFONDEUR DE LA COUCHE DE MELANGE OCEANIQUE.

IFLW=2
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,OMLDTH)
ELSE
  ZREALG(:NGPTOT)=OMLDTH
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  CALL FAIENC(IREP,INULCL,CLNCHP(IFLW)(:4),0,CLNCHP(IFLW)(5:),&
   & ZREALG,.FALSE.)  
ENDIF

! GRADIENT THERMIQUE DE LA THERMOCLINE

IFLW=3
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,GTTLIN)
ELSE
  ZREALG(:NGPTOT)=GTTLIN
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  CALL FAIENC(IREP,INULCL,CLNCHP(IFLW)(:4),0,CLNCHP(IFLW)(5:),&
   & ZREALG,.FALSE.)  
ENDIF

!*       4.    CLOSE FDB, SAVE FILES ETC.
!              --------------------------

IF (MYPROC == IOPROC) THEN
  IF (LRFILAF) THEN
    CALL LFILAF(IREP,INULCL,.FALSE.)
  ENDIF
  CALL FAIRME(IREP,INULCL,'UNKNOWN')
ENDIF

IF (LHOOK) CALL DR_HOOK('WRCOM',1,ZHOOK_HANDLE)
END SUBROUTINE WRCOM
