SUBROUTINE WRCPL

!**** * WRCPL  * - ROUTINE TO WRITE THE COUPLED FIELDS (OASIS)

!**   INTERFACE.
!     ----------
!        *CALL* *WRCPL*

!     EXPLICIT ARGUMENTS :  NONE
!     --------------------

!     IMPLICIT ARGUMENTS :
!     --------------------
!        COMMON  YOMCPL

!     METHOD.
!     -------
!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CNRM/GMGEC/EAC/JPh Piedelievre.

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 00-11-27 (D'APRES WRCODM)
!        Modified 03-12-05 by J.P. Piedelievre : bugfix
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCPL   , ONLY : FCSTSU   ,FCSTSV    ,FCCHAS    ,&
 & FCRSOS   ,FCHUMS   ,FCRUIS    ,FCCHSS    ,&
 & FCHUML   ,FCHUMN   ,FCTSUR2   ,FCTSTS    ,&
 & FCTSFL  

USE YOMMCC   , ONLY : NFRCPL

USE YOMLUN   , ONLY : NULUSR1  ,NULOUT
USE YOMDIM   , ONLY : NDGLG    ,NDLON
USE YOMDYN   , ONLY : TSTEP
USE YOMCT0   , ONLY :     NSTOP
USE YOMCT3   , ONLY : NSTEP
USE YOMGEM   , ONLY : NGPTOT   
USE YOMMP    , ONLY : MYPROC
USE YOMCST   , ONLY : RDAY

!*  Addition for SVIPC coupling technique
USE YOM_OAS   , ONLY :   CPHAN
USE PAR_COU   , ONLY : JPFLDA2O
USE YOM_CPL

IMPLICIT NONE

!*  Addition for infos contained within the field brick if cpmodinf='YES'
!   Experiment name in the field brick, if any
!CHARACTER(len=3) CLJOBNAM_R
!   infos in the field brick, if any
INTEGER(KIND=JPIM), DIMENSION(3) :: INFOS
!CLIM
INTEGER(KIND=JPIM) :: info

!INTEGER_M :: JPNCHP

!PARAMETER(JPNCHP=99)
!CHARACTER*13 CLNCHP(JPNCHP)

REAL(KIND=JPRB) :: ZREALG(NDLON*NDGLG)
INTEGER(KIND=JPIM) :: INDEX, IDIMG

INTEGER(KIND=JPIM) :: IFLD, INDEX1, INDEX2, IOPROC, ISTEP, ISTEPCPL, JNULCL

LOGICAL :: LLOP

REAL(KIND=JPRB) :: ZCUMUL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "diwrgrid.intfb.h"
#include "sipc_write_model.intfb.h"
#include "sucpl0.intfb.h"

!     -------------------------------------------------------------

!*       1.    PREPARATIONS.
!              --------------

IF (LHOOK) CALL DR_HOOK('WRCPL',0,ZHOOK_HANDLE)
ISTEP=MAX(0,NSTEP+1)

!CLIM
ISTEPCPL=NSTEP+1
IF(NSTEP == 0) ISTEPCPL=0
!ENDCLIM
WRITE(NULOUT,*)' NSTEP,ISTEP,ISTEPCPL=',NSTEP,ISTEP,ISTEPCPL

!*       1.1   LISTE DES CHAMPS A TRANSMETTRE A OASIS.

!CLNCHP( 1)='STRESS U'
!CLNCHP( 2)='STRESS V'
!CLNCHP( 3)='FLUX CHALEUR'
!CLNCHP( 4)='RAYT.SOLAIRE'
!CLNCHP( 5)='FLUX HUMI'
!CLNCHP( 6)='RUISSEL.'
!CLNCHP( 7)='FLU.CHA.SENS'
!CLNCHP( 8)='PLUIE TOTALE'
!CLNCHP( 9)='NEIGE TOTALE'
!CLNCHP(10)='TEMPERATURE'
!CLNCHP(11)='TSTS'
!CLNCHP(12)='TSFL'

!*       1.2   PROCESSEUR D'IO

! IOPROC MUST BE 1
IOPROC=1
IDIMG= NDGLG*NDLON
IF (MYPROC == IOPROC) THEN

! Reopen flxatmos to avoid any I/O conflict (?!)

  IF (ISTEP == NSTOP) THEN
    DO JNULCL=1,99
      INQUIRE(UNIT=JNULCL,OPENED=LLOP)
      IF (.NOT.LLOP) EXIT
    ENDDO
    NULUSR1=JNULCL
    OPEN (NULUSR1,FILE='flxatmos',FORM='UNFORMATTED')
  ENDIF

ENDIF ! MYPROC == 1

!     ---------------------------------------------------------------

!*    2.0   WRITE GRID POINT FIELDS ON *FA* FILE
!     -------------------------------------

IFLD=0
ZREALG=0.0_JPRB
ZCUMUL=REAL(NFRCPL)*TSTEP

!*       2.1   WRITE 2 D FIELDS

!IFLW=1
! FCSTSU
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCSTSU)
ELSE
  ZREALG(:NGPTOT)=FCSTSU
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COZOTAUX'
      WRITE(NULUSR1) ZREALG
      WRITE(NULUSR1) 'COZOTAUV'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX1=5
      INDEX2=7
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX1,IDIMG,INFOS,ZREALG)
        CALL SIPC_WRITE_MODEL(INDEX2,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX1),ISTEP,ZREALG,INFO)
        CALL CLIM_EXPORT(CL_WRIT(INDEX2),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=2
! FCSTSV
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCSTSV)
ELSE
  ZREALG(:NGPTOT)=FCSTSV
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COMETAUY'
      WRITE(NULUSR1) ZREALG
      WRITE(NULUSR1) 'COMETAUU'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX1=6
      INDEX2=8
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX1,IDIMG,INFOS,ZREALG)
        CALL SIPC_WRITE_MODEL(INDEX2,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX1),ISTEP,ZREALG,INFO)
        CALL CLIM_EXPORT(CL_WRIT(INDEX2),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=3
! FCCHAS
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCCHAS(1,1))
ELSE
  ZREALG(:NGPTOT)=FCCHAS(:,1)
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'CONSFTOT'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=1
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=4
! FCRSOS
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCRSOS(1,1))
ELSE
  ZREALG(:NGPTOT)=FCRSOS(:,1)
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COSHFTOT'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=2
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=5
! FCHUMS
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCHUMS(1,1))
ELSE
  ZREALG(:NGPTOT)=FCHUMS(:,1)
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1) THEN

!* Total water flux 'COWATFLU'

!  WARNING !!!! ===>>> values are sent in mm/day for all water fluxes
!  ------------------------------------------------------------------

    ZREALG=ZREALG/ZCUMUL*RDAY
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COWATFLU'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=3
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=6
! FCRUIS
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCRUIS)
ELSE
  ZREALG(:NGPTOT)=FCRUIS
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1) THEN

!* Run-off 'CORUNOFF'

!  WARNING !!!! ===>>> values are sent in mm/day for all water fluxes
!  ------------------------------------------------------------------

    ZREALG=ZREALG/ZCUMUL*RDAY
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'CORUNOFF'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=4
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=7
! FCCHSS
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCCHSS(1,1))
ELSE
  ZREALG(:NGPTOT)=FCCHSS(:,1)
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1.AND.JPFLDA2O >= 9) THEN

!*         'COTHSHSU'
!  WARNING !!!! ===>>>  values are sent in mm/day for all water fluxes
!  ------------------------------------------------------------------

    ZREALG=ZREALG/ZCUMUL*RDAY
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COTHSHSU'
      WRITE(NULUSR1) ZREALG(:IDIMG)
    ELSE
      INDEX=9
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=8
! FCHUML
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCHUML)
ELSE
  ZREALG(:NGPTOT)=FCHUML
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1.AND.JPFLDA2O >= 10) THEN

!*         'COTOLIPR'
!  WARNING !!!! ===>>> values are sent in mm/day for all water fluxes
!  ------------------------------------------------------------------

    ZREALG=ZREALG/ZCUMUL*RDAY
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COTOLIPR'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=10
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=9
! FCHUMN
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCHUMN)
ELSE
  ZREALG(:NGPTOT)=FCHUMN
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1.AND.JPFLDA2O >= 11) THEN

!*         'COTOSOPR'
!  WARNING !!!! ===>>> values are sent in mm/day for all water fluxes
!  ------------------------------------------------------------------

    ZREALG=ZREALG/ZCUMUL*RDAY
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COTOSOPR'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=11
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=10
! FCTSUR2
! (Celcius temperature for coupled run)
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCTSUR2)
ELSE
  ZREALG(:NGPTOT)=FCTSUR2
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1.AND.JPFLDA2O >= 12) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COSSTSST'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=12
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=11
! FCTSTS
! ici albedo
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCTSTS)
ELSE
  ZREALG(:NGPTOT)=FCTSTS
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1.AND.JPFLDA2O >= 14) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'COALBSUR'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=14
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!IFLW=12
! FCTSFL
IFLD=IFLD+1
IF (MYPROC /= IOPROC) THEN
  CALL DIWRGRID(IOPROC,IFLD,0,NGPTOT,FCTSFL)
ELSE
  ZREALG(:NGPTOT)=FCTSFL
  CALL DIWRGRID(IOPROC,IFLD,1,IDIMG,ZREALG)
  IF (ISTEP > 1.AND.JPFLDA2O >= 13) THEN
    ZREALG=ZREALG/ZCUMUL
    IF (ISTEP == NSTOP) THEN
      WRITE(NULUSR1) 'CODFLXDT'
      WRITE(NULUSR1) ZREALG
    ELSE
      INDEX=13
      IF (CPHAN == 'SIPC') THEN
        CALL SIPC_WRITE_MODEL(INDEX,IDIMG,INFOS,ZREALG)
      ELSEIF (CPHAN == 'MPI2') THEN
        CALL CLIM_EXPORT(CL_WRIT(INDEX),ISTEP,ZREALG,INFO)
      ENDIF
    ENDIF
  ENDIF
ENDIF

!     ----------------------------------------------------------------

!*       4.    CLOSE FDB, SAVE FILES ETC.
!              --------------------------

IF (MYPROC == IOPROC) THEN
  CLOSE(NULUSR1)
ENDIF

!     ------------------------------------------------------------------

!*       5.    RESET TO ZERO
!              -------------

CALL SUCPL0

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRCPL',1,ZHOOK_HANDLE)
END SUBROUTINE WRCPL
