! AJGDB
subroutine onedvar_adjoint_test ( ksat, k1d_x, k1d_y, k1d_z, &
 & p1d_x, p1d_xt, p1d_x_tl, p1d_xt_tl)

!**** Adjoint test of observation operator for 
!     1D-Var in clouds/precipitation for microwave radiances over land or ocean

!     C. O'Dell ECMWF  December 2006

!     OENDVAR_ADJOINT_TEST is called from ONEDVAR_RAINTB (ifs/onedvar/)

!     Modifications:

!**** MODULES      

!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook, dr_hook

!* IFS
use yomlun, only : nulerr 
use yomdim, only : nflevg ! number of levels

implicit none
         
!* Common variables
integer(kind=jpim), intent(in) :: ksat, k1d_x, k1d_y, k1d_z ! in: dimensions
real(kind=jprb), intent(in), dimension(k1d_x) :: p1d_x  ! Input control vector
real(kind=jprb), intent(in), dimension(k1d_z) :: p1d_xt ! Extra stuff vector
real(kind=jprb), intent(in), dimension(k1d_x) :: p1d_x_tl ! perturbations to control vector
real(kind=jprb), intent(in), dimension(k1d_z) :: p1d_xt_tl ! perturbations to extra vector

!* Local variables
real(kind=jprb), dimension(1,k1d_x) :: z1d_x, z1d_x_tl, zz1d_x_tl, z1d_x_ad
real(kind=jprb), dimension(1,k1d_z) :: z1d_xt, z1d_xt_tl, zz1d_xt_tl, z1d_xt_ad
real(kind=jprb), dimension(1,k1d_y) :: z1d_y2, z1d_y3, z1d_y_tl, z1d_y_ad
             
real(kind=jprb) :: zip_tl, zip_ad, zz
integer(kind=jpim) :: iy, ix, iz
integer(kind=jpim), parameter :: jpks = kind (zz)
real (kind=jprb) :: zhook_handle
    
!* INTERFACES
#include "onedvar_obsop_tl.intfb.h"
#include "onedvar_obsop_gr.intfb.h"

if (lhook) call dr_hook ('ONEDVAR_ADJOINT_TEST',0,zhook_handle)

z1d_x(1,:) = p1d_x(:)
z1d_xt(1,:) = p1d_xt(:)
z1d_x_tl(1,:) = 0.0_jprb
z1d_xt_tl(1,:) = 0.0_jprb
z1d_y_tl(1,:) = 0.0_jprb

! control vector perturbations
z1d_x_tl(1,:) = p1d_x_tl(:) ! perturb T & q

! extra vector perturbations (these really should be zero)
z1d_xt_tl(1,1:2*nflevg) = p1d_xt_tl(1:2*nflevg) ! perturb Tten and qten
z1d_xt_tl(1,7*nflevg+1:7*nflevg+10) = p1d_xt_tl(7*nflevg+1:7*nflevg+10) ! perturb surface parameters

!* Save for use in 2nd scalar product
zz1d_x_tl(:,:) = z1d_x_tl(:,:) 
zz1d_xt_tl(:,:) = z1d_xt_tl(:,:)

call onedvar_obsop_tl (ksat, 1_jpim, k1d_x, k1d_y, k1d_z, &
  & z1d_x, z1d_xt, z1d_y2, z1d_x_tl, z1d_xt_tl, z1d_y_tl)

! remove any possible changes to input parameters
z1d_x(1,:) = p1d_x(:)
z1d_xt(1,:) = p1d_xt(:)

!* Initialise perturbations
z1d_x_ad(1,:) = 0.0_jprb
z1d_xt_ad(1,:) = 0.0_jprb
z1d_y_ad(1,:) = 0.0_jprb

!* Produce RTTOVSCATT_AD input: TB's, tendencies and surface precip
z1d_y_ad(1,:) = z1d_y_tl(1,:)

! Call adjoint.  Currently we only get perturbations wrt CONTROL VECTOR PARAMETERS!
! can only accept adjoint perturbations to y
call onedvar_obsop_gr (ksat, 1_jpim, k1d_x, k1d_y, k1d_z, &
    &   z1d_x, z1d_xt, z1d_y3, z1d_y_ad, z1d_x_ad) 

zip_tl = 0.0_jprb
zip_ad = 0.0_jprb

!* 1st scalar product: operator output
do iy = 1, k1d_y
   zip_tl = zip_tl + z1d_y_tl(1,iy) * z1d_y_tl(1,iy)
enddo

!* 2nd scalar product: operator input  
do ix = 1, k1d_x 
   zip_ad = zip_ad + z1d_x_ad(1,ix) * zz1d_x_tl(1,ix)
enddo

do iz = 1, k1d_z
   zip_ad = zip_ad + z1d_xt_ad(1,iz) * zz1d_xt_tl(1,iz)
enddo

!* Evaluate vs. machine precision
if (zip_tl == 0.0_jprb) then
   zz = 1.0_jprb
else
   zz = zip_tl
endif

write (nulerr,"('ONEDVAR_OBSOP_ADTEST AD: ', 2e22.12, f20.2)") zip_tl, zip_ad, &
& abs(real(real(zip_tl,jpks)-real(zip_ad,jpks),jpks)/epsilon(zz)/zz)  

if (lhook) call dr_hook ('ONEDVAR_ADJOINT_TEST',1,zhook_handle) 

end subroutine onedvar_adjoint_test

