subroutine onedvar_get_bgcor (knproc, kmyproc, kerror, kfile_id)
    
!**** Read background error correlation matrices for T, q, rh' from files for 
!     1D-Var in clouds/precipitation for SSM/I radiances 

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     1. Check files
!     2. Read files

!     ONEDVAR_GET_BGCOR is called from RTSETUP

!     Modifications:


!**** Modules

!* kind     
use parkind1  , only: jpim, jprb
use yomhook   , only: lhook,   dr_hook

!* ifs
use yomdim    , only: nflevg            
use yomonedvar, only: rcor_t, rcor_q, rcor_r
   
implicit none      

integer (kind=jpim), intent ( in) :: kmyproc
integer (kind=jpim), intent ( in) :: knproc
integer (kind=jpim), intent ( in) :: kfile_id
integer (kind=jpim), intent (out) :: kerror

integer (kind=jpim) :: ilev, i_lev, j_lev, i_dum, j_dum
Logical             :: llexistence
real (kind=jprb)    :: zhook_handle
character (len=2)   :: clflevg

#include "abor1.intfb.h"  

if (lhook) call dr_hook ('ONEDVAR_GET_BGCOR',0,zhook_handle)

!if (kmyproc == kioproc ) then
    allocate (rcor_t (nflevg,nflevg))
    allocate (rcor_q (nflevg,nflevg))
    allocate (rcor_r (nflevg,nflevg))

    write (clflevg, "(i2)") nflevg
   
    kerror = 0

    rcor_t (:,:) = 0.0_JPRB
    rcor_q (:,:) = 0.0_JPRB
    rcor_r (:,:) = 0.0_JPRB
   
!* read BG-error correlations
    inquire (FILE = 'cor_t_norm' // clflevg // '.dat', EXIST = llexistence)
    if (.not. llexistence) then 
       call abor1 ('ONEDVAR_GET_BGCOR: T - BG-error correlation file not found.')
       kerror = 1
    else
       open (kfile_id, file = 'cor_t_norm' // clflevg // '.dat', form = 'formatted')
       read (kfile_id,*) ilev
       if (ilev /= nflevg) then 
           call abor1 ('ONEDVAR_GET_BGCOR: T - BG-error No. of levels wrong.')
           kerror = 1
       else 
           do i_lev = 1, nflevg
              do j_lev = 1, nflevg
                 read (kfile_id,*) i_dum, j_dum, rcor_t (i_lev,j_lev)
              enddo
           enddo
       endif
       close (kfile_id)
    endif

    inquire (FILE = 'cor_q_norm' // clflevg // '.dat', EXIST = llexistence)
    if (.not. llexistence) then 
       call abor1 ('ONEDVAR_GET_BGCOR: Q - BG-error correlation file not found.')
       kerror = 1
    else
       open (kfile_id, file = 'cor_q_norm' // clflevg // '.dat', form = 'formatted')
       read (kfile_id,*) ilev
       if (ilev /= nflevg) then 
           call abor1 ('ONEDVAR_GET_BGCOR: Q - BG-error No. of levels wrong.')
           kerror = 1
       else 
           do i_lev = 1, nflevg
              do j_lev = 1, nflevg
                 read (kfile_id,*) i_dum, j_dum, rcor_q (i_lev,j_lev)
              enddo
           enddo
       endif
       close (kfile_id)
    endif

    inquire (FILE = 'cor_r_norm' // clflevg // '.dat', EXIST = llexistence)
    if (.not. llexistence) then 
       call abor1 ('ONEDVAR_GET_BGCOR: RH - BG-error correlation file not found.')
       kerror = 1
    else
       open (kfile_id, file = 'cor_r_norm' // clflevg // '.dat', form = 'formatted')
       read (kfile_id,*) ilev
       if (ilev /= nflevg) then 
           call abor1 ('ONEDVAR_GET_BGCOR: RH - BG-error No. of levels wrong.')
           kerror = 1
       else 
           do i_lev = 1, nflevg
              do j_lev = 1, nflevg
                 read (kfile_id,*) i_dum, j_dum, rcor_r (i_lev,j_lev)
              enddo
           enddo
       endif
       close (kfile_id)
    endif
!endif

if (lhook) call dr_hook ('ONEDVAR_GET_BGCOR',1,zhook_handle)
end subroutine onedvar_get_bgcor
