SUBROUTINE ONEDVAR_GET_BGSIG (P_SFC, POBLAT, POBLON, PSIGT, PSIGQ) 

!***  *GETBGSTD* - GET BACKGROUND STD DEVIATIONS FOR 1D-VAR

!     PURPOSE.
!     --------

!       GET BACKGROUND STD DEVIATIONS FOR 1D-VAR 

!     METHOD.      
!     -------
!        FORECAST ERRORS FOR THE CURRENT ANALYSIS ARE ARCHIVED IN MARS.
!        THESE FIELDS  ARE READ IN AND ARE INTERPOLATED IN HORIZONTAL
!        AND VERTICAL TO THE 1D-VAR OBSERVATION LOCATIONS.

!        ADAPTED FROM IFS GEFGER

!     EXTERNALS.
!     ----------

!        SUHIFCE  - INTERPOLATE FORECAST ERRORS IN THE HORIZONTAL
!        SUVIFCE  - INTERPOLATE FORECAST ERRORS IN THE VERTICAL (NOT USED)

!     AUTHOR.
!     -------
!     A. BENEDETTI 14/4/2004

!     MOFIFICATIONS
!     -------
!     P. BAUER 02/6/2005 - Inclusion of vertical interpolation

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE YOMGEM   , ONLY : VAH, VBH
USE YOMFGER  , ONLY : JPMAXVAR ,MNROWS ,MNVARS , MNFLDS ,&
                    & MNPTE    ,MLONE  ,MFIELDS, MNLEVS ,&
                    & RZLATE   ,RZLON0E,RZDLONE, RZEGRID, RZA, RZB
USE YOMGRB   , ONLY : NGRBT, NGRBQ
USE MPL_MODULE

IMPLICIT NONE

REAL(KIND=JPRB), INTENT  (IN) :: P_SFC, POBLAT, POBLON
REAL(KIND=JPRB), INTENT (OUT) :: PSIGT (NFLEVG), PSIGQ (NFLEVG)

INTEGER(KIND=JPIM) :: JLEV, JVAR, IVAR, IVT, IVQ

REAL(KIND=JPRB) :: ZFGEMIN, ZOBLAT (1), ZOBLON (1)
REAL(KIND=JPRB),ALLOCATABLE :: ZSIGT (:), ZSIGQ (:)
REAL(KIND=JPRB) :: ZP_MODEL (NFLEVG), ZPH_MODEL (NFLEVG+1), ZPH_FCE (101+1)

LOGICAL :: LLQ, LLT

!-----------------------------------------------------------------------


INTEGER(KIND=JPIM), DIMENSION(JPMAXVAR) :: IFLDOFF

REAL(KIND=JPRB),DIMENSION(:,:), ALLOCATABLE :: ZIGRID
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "suhifce.intfb.h"
#include "suvifce.intfb.h"
#include "abor1.intfb.h"  

!-----------------------------------------------------------------------

!        0.    INITIALIZE

!           0.1   INFORM

IF (LHOOK) CALL DR_HOOK('ONEDVAR_GET_BGSIG',0,ZHOOK_HANDLE)

ALLOCATE(ZIGRID(1,MNFLDS))

! INITIALIZE ARRAYS

PSIGT(:) = 0.0_JPRB 
PSIGQ(:) = 0.0_JPRB

ZOBLON (1) = POBLON
ZOBLAT (1) = POBLAT

LLT=.FALSE.
LLQ=.FALSE.

!* Set up model pressure levels

ZPH_MODEL (1:NFLEVG+1) = P_SFC * VBH (0:NFLEVG) + VAH (0:NFLEVG)
DO JLEV = 1, NFLEVG
   ZP_MODEL (JLEV) = 0.5_JPRB * (ZPH_MODEL (JLEV) + ZPH_MODEL (JLEV+1))
ENDDO

IF (MNPTE > 0) THEN

!*          2.6   INTERPOLATE FORECAST ERRORS IN THE HORIZONTAL

    CALL SUHIFCE(RZLON0E,RZDLONE,RZLATE,MNROWS,MLONE,ZOBLON,ZOBLAT,1,&
           & MNPTE,MNFLDS,RZEGRID,ZIGRID)  



    DO JVAR = 1, MNVARS
       IFLDOFF(JVAR) = SUM(MNLEVS(1:JVAR-1))
       
       IF (MFIELDS(JVAR) == NGRBT ) THEN
          LLT = .TRUE.
          IVT = JVAR
       ELSEIF(MFIELDS(JVAR) == NGRBQ) THEN
          LLQ = .TRUE.
          IVQ = JVAR
       ENDIF
    ENDDO

!           2.14.1 TEMPERATURE
    IF (LLT) THEN
       IVAR = IVT
       ALLOCATE(ZSIGT(0:MNLEVS(IVAR)))
       ZSIGT(:) = 0.0_JPRB 
       ZFGEMIN=0.01_JPRB
       DO JLEV=1,MNLEVS(IVAR)
          ZSIGT (JLEV) = MAX(ZIGRID(1,JLEV+IFLDOFF(IVAR)),ZFGEMIN)
       ENDDO

       IF (NFLEVG == MNLEVS(IVAR)) THEN
          PSIGT (1:NFLEVG) = ZSIGT (1:NFLEVG)
       ELSE
          ZPH_FCE (1:MNLEVS(IVAR)) = P_SFC * RZB (1:MNLEVS(IVAR),IVAR) + RZA (1:MNLEVS(IVAR),IVAR)

          CALL SUVIFCE (ZSIGT (0:MNLEVS(IVAR)), ZPH_FCE  (1:MNLEVS(IVAR)), 1, 1, MNLEVS (IVAR), &
                      & PSIGT (1:NFLEVG)      , ZP_MODEL (1:NFLEVG)      ,       NFLEVG)
       ENDIF
       DEALLOCATE(ZSIGT)
    ENDIF

!           2.14.2 SPECIFIC HUMIDITY
    IF( LLQ) THEN
       IVAR = IVQ
       ALLOCATE(ZSIGQ(0:MNLEVS(IVAR)))
       ZSIGQ=0.0_JPRB
       ZFGEMIN=1.0E-10
       DO JLEV=1,MNLEVS(IVAR)
          ZSIGQ (JLEV) = MAX(ZIGRID(1,JLEV+IFLDOFF(IVAR)),ZFGEMIN)
       ENDDO

       IF (NFLEVG == MNLEVS(IVAR)) THEN
          PSIGQ (1:NFLEVG) = ZSIGQ (1:NFLEVG)
       ELSE
          ZPH_FCE (1:MNLEVS(IVAR)) = P_SFC * RZB (1:MNLEVS(IVAR),IVAR) + RZA (1:MNLEVS(IVAR),IVAR)

          CALL SUVIFCE (ZSIGQ (0:MNLEVS(IVAR)), ZPH_FCE  (1:MNLEVS(IVAR)), 1, 1, MNLEVS (IVAR), &
                      & PSIGQ (1:NFLEVG)      , ZP_MODEL (1:NFLEVG)      ,       NFLEVG)
       ENDIF
       DEALLOCATE(ZSIGQ)
    ENDIF
ELSE
    CALL ABOR1 ('ONEDVAR_GET_BGSIG: No BG-Error fields')
    IF (LHOOK) CALL DR_HOOK('ONEDVAR_GET_BGSIG',1,ZHOOK_HANDLE)    
ENDIF  

!*          2.23  RELEASE THE SPACE

DEALLOCATE (ZIGRID)

IF (LHOOK) CALL DR_HOOK('ONEDVAR_GET_BGSIG',1,ZHOOK_HANDLE)
END SUBROUTINE ONEDVAR_GET_BGSIG
