subroutine onedvar_get_bias (ksat, kfov, knchannels, pred_bias, ptb_bias) 
      
! TB bias-correction for
! 1D-Var in clouds/precipitation for SSM/I radiances 

! P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

! 1. Polynominal fit with TCWV, SWS, RWP as predictors

! ONEDVAR_GET_BIAS is called from ONEDVAR_SCREEN

! Modifications: 
!   More predictors/separate satellite ID's; P. Bauer, April 2006
!   Slight changes to accomodate generalization to multiple sensors; C. O'Dell, 17 Oct 2006

use parkind1     , only: jpim, jprb     
use yomhook      , only: lhook,   dr_hook
use yomlun       , only: nulerr

use paronedvar, only: npred_bias
use yomonedvar, only: onedvar_id, n_satsens

implicit none 

integer (kind=jpim), intent(in) :: ksat  ! satellite index number (1,2, etc)
integer (kind=jpim), intent(in) :: kfov  ! sat field of view (not currently used) 
integer (kind=jpim), intent(in) :: knchannels ! number of channels for this satellite

! predictor values like IWV, wind speed, etc:
real (kind=jprb), intent(in), dimension(npred_bias) :: pred_bias  

! bias for each channel:
real (kind=jprb), intent (out), dimension (knchannels) :: ptb_bias   

integer (kind=jpim) :: i_chan, ipred_bias
real    (kind=jprb) :: zhook_handle

if (lhook) call dr_hook ('ONEDVAR_GET_BIAS',0,zhook_handle)

if (ksat < 1 .or. ksat > n_satsens) then 
  write (nulerr,*) '1D-Var Satellite ID incorrect: ', ksat
  ptb_bias (1:knchannels) = 0.0_jprb
  if (lhook) call dr_hook ('ONEDVAR_GET_BIAS',1,zhook_handle)
  return
endif

! predictor dependent term      
do i_chan = 1, knchannels

  ptb_bias (i_chan) = onedvar_id(ksat) % robs_bias (i_chan,0)

  do ipred_bias = 1, npred_bias
    ptb_bias (i_chan) = ptb_bias (i_chan) + &
     & onedvar_id(ksat) % robs_bias(i_chan,ipred_bias) * pred_bias (ipred_bias)
  enddo
enddo

if (lhook) call dr_hook ('ONEDVAR_GET_BIAS',1,zhook_handle)
end subroutine onedvar_get_bias
