subroutine onedvar_obsop_tl (ksat, kproma, k1d_x, k1d_y, k1d_z, &
                             & p1d_x5, p1d_x_xt5, p1d_y5, &      
                             &  p1d_x , p1d_x_xt , p1d_y)

!**** Observation operator for 
!     1D-Var in clouds/precipitation for SSM/I radiances 

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     1. Call simplified moist physics
!     2. Call RTTOV-interface of observation operator

!     ONEDVAR_OBSOP is called from ONEDVAR_RAINTB, ONEDVAR_SIMUL

!     Modifications:

!     29-Nov-2006  Alan Geer    Add 10m wind to control vector (should have been
!                               done at 31R1 but was not: this module only used
!                               by the optional linearity test.)

!     Dec-2006    Chris O'Dell  Slight changes necessary for multiple sensors
!     11-Nov-2007  A. Geer      RTTOV9 compatible

!**** MODULES      

!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook,   dr_hook

!* IFS
use yomdim  , only: nflevg                      
use yomgem  , only: vah, vbh
    
implicit none
         
!**** VARIABLES
!* IN/OUTPUT
integer (kind=jpim), intent    (in)                           :: ksat, kproma, k1d_x, k1d_y, k1d_z
real    (kind=jprb), intent    (in), dimension (kproma,k1d_x) :: p1d_x5   , p1d_x
real    (kind=jprb), intent (inout), dimension (kproma,k1d_z) :: p1d_x_xt5, p1d_x_xt
real    (kind=jprb), intent   (out), dimension (kproma,k1d_y) :: p1d_y5   , p1d_y
   
!* OTHER                      
integer (kind=jpim) :: i_lev, i_print, i_flevg, i_proma
integer (kind=jpim) :: ichannels

real    (kind=jprb), dimension (kproma,nflevg+1) :: zphys_ph5
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_rfl5, zphys_sfl5, zphys_tet5, zphys_teq5  
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_cwc5, zphys_cic5, zphys_cc5, zphys_t5
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_q5, zphys_p5  
real    (kind=jprb), dimension (kproma)          :: zphys_ps5 , zphys_sh5 , zphys_lh5, zphys_us5, zphys_vs5   
real    (kind=jprb), dimension (kproma)          :: zphys_crf5, zphys_srf5, z_sfc  
             
real    (kind=jprb), dimension (kproma,nflevg+1) :: zphys_ph
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_rfl, zphys_sfl, zphys_tet, zphys_teq  
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_cwc, zphys_cic, zphys_cc, zphys_t
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_q, zphys_p  
real    (kind=jprb), dimension (kproma)          :: zphys_ps , zphys_sh , zphys_lh, zphys_us, zphys_vs   
real    (kind=jprb), dimension (kproma)          :: zphys_crf, zphys_srf  
             
! adjustment variables
real (kind=jprb) :: zhook_handle
    
!* INTERFACES
#include "phys_tl.intfb.h"
!#include "onedvar_obsop_rttov.intfb.h"

if (lhook) call dr_hook ('ONEDVAR_OBSOP_TL',0,zhook_handle)

i_print   = 0  
i_flevg   = nflevg
  
!* Control variable 
zphys_t5  (1:kproma,1:nflevg)  = p1d_x5 (1:kproma,       1:  nflevg)
zphys_q5  (1:kproma,1:nflevg)  = p1d_x5 (1:kproma,nflevg+1:2*nflevg)
 
zphys_t   (1:kproma,1:nflevg)  = p1d_x  (1:kproma,       1:  nflevg)
zphys_q   (1:kproma,1:nflevg)  = p1d_x  (1:kproma,nflevg+1:2*nflevg)
 
!* Others
zphys_tet5 (1:kproma,1:nflevg) = p1d_x_xt5 (1:kproma,       1:  nflevg)
zphys_teq5 (1:kproma,1:nflevg) = p1d_x_xt5 (1:kproma,nflevg+1:2*nflevg)
        
zphys_tet  (1:kproma,1:nflevg) = p1d_x_xt  (1:kproma,       1:  nflevg)
zphys_teq  (1:kproma,1:nflevg) = p1d_x_xt  (1:kproma,nflevg+1:2*nflevg)
        
zphys_ps5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+9)
zphys_sh5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+6)
zphys_lh5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+7)
zphys_us5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+4)
zphys_vs5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+5)

zphys_ps   (1:kproma) = p1d_x_xt  (1:kproma,7*nflevg+9)
zphys_sh   (1:kproma) = p1d_x_xt  (1:kproma,7*nflevg+6)
zphys_lh   (1:kproma) = p1d_x_xt  (1:kproma,7*nflevg+7)
zphys_us   (1:kproma) = p1d_x_xt  (1:kproma,7*nflevg+4)
zphys_vs   (1:kproma) = p1d_x_xt  (1:kproma,7*nflevg+5)

!* Set up pressure at ML
do i_proma = 1, kproma
   z_sfc (i_proma) = p1d_x_xt5 (i_proma,7*nflevg+9)
   zphys_ph5 (i_proma,1:nflevg+1) = vbh (0:nflevg) * z_sfc (i_proma) + vah (0:nflevg)
   zphys_ph  (i_proma,1:nflevg+1) = 0.0_JPRB
enddo
    
do i_lev =  nflevg, 1, -1
  zphys_p5 (1:kproma,i_lev) = 0.5_JPRB * (zphys_ph5 (1:kproma,i_lev) + zphys_ph5 (1:kproma,i_lev+1))
  zphys_p  (1:kproma,i_lev) = 0.0_JPRB
enddo
     
!* Initialize for safety
zphys_rfl5 (:,:) = 0.0_JPRB
zphys_sfl5 (:,:) = 0.0_JPRB
zphys_cwc5 (:,:) = 0.0_JPRB
zphys_cic5 (:,:) = 0.0_JPRB
zphys_cc5  (:,:) = 0.0_JPRB
zphys_crf5 (:)   = 0.0_JPRB
zphys_srf5 (:)   = 0.0_JPRB

zphys_rfl  (:,:) = 0.0_JPRB
zphys_sfl  (:,:) = 0.0_JPRB
zphys_cwc  (:,:) = 0.0_JPRB
zphys_cic  (:,:) = 0.0_JPRB
zphys_cc   (:,:) = 0.0_JPRB
zphys_crf  (:)   = 0.0_JPRB
zphys_srf  (:)   = 0.0_JPRB

!* Moist physics
call phys_tl (i_print, i_flevg, kproma, &
 &   zphys_t5                  , &            ! in:  T
 &   zphys_q5                  , &            ! in:  q, rh'
 &   zphys_p5                  , &            ! in:  p
 &   zphys_ph5                 , &            ! in:  ph
 &   zphys_tet5                , &            ! in:  ten_t
 &   zphys_teq5                , &            ! in:  ten_q
 &   zphys_ps5                 , &            ! in:  z_sfc
 &   zphys_sh5                 , &            ! in:  sh_sfc
 &   zphys_lh5                 , &            ! in:  lh_sfc
 &   zphys_us5                 , &            ! in:  u_star
 &   zphys_vs5                 , &            ! in:  v_star
 &   zphys_rfl5                , &            ! out: rain flux
 &   zphys_sfl5                , &            ! out: snow flux
 &   zphys_cwc5                , &            ! out: cloud water
 &   zphys_cic5                , &            ! out: cloud ice
 &   zphys_cc5                 , &            ! out: cloud cover
 &   zphys_srf5                , &            ! out: strt. rain flux at sfc
 &   zphys_crf5                , &            ! out: conv. rain flux at sfc      
 &   zphys_t                   , &            ! in:  T
 &   zphys_q                   , &            ! in:  q, rh'
 &   zphys_p                   , &            ! in:  p
 &   zphys_ph                  , &            ! in:  ph
 &   zphys_tet                 , &            ! in:  ten_t
 &   zphys_teq                 , &            ! in:  ten_q
 &   zphys_ps                  , &            ! in:  z_sfc
 &   zphys_sh                  , &            ! in:  sh_sfc
 &   zphys_lh                  , &            ! in:  lh_sfc
 &   zphys_us                  , &            ! in:  u_star
 &   zphys_vs                  , &            ! in:  v_star
 &   zphys_rfl                 , &            ! out: rain flux
 &   zphys_sfl                 , &            ! out: snow flux
 &   zphys_cwc                 , &            ! out: cloud water
 &   zphys_cic                 , &            ! out: cloud ice
 &   zphys_cc                  , &            ! out: cloud cover
 &   zphys_srf                 , &            ! out: strt. rain flux at sfc
 &   zphys_crf )                              ! out: conv. rain flux at sfc      

p1d_x_xt5 (1:kproma,2*nflevg+1:3*nflevg) = zphys_rfl5 (1:kproma,1:nflevg)
p1d_x_xt5 (1:kproma,3*nflevg+1:4*nflevg) = zphys_sfl5 (1:kproma,1:nflevg)
p1d_x_xt5 (1:kproma,4*nflevg+1:5*nflevg) = zphys_cwc5 (1:kproma,1:nflevg)
p1d_x_xt5 (1:kproma,5*nflevg+1:6*nflevg) = zphys_cic5 (1:kproma,1:nflevg)
p1d_x_xt5 (1:kproma,6*nflevg+1:7*nflevg) = zphys_cc5  (1:kproma,1:nflevg)

p1d_x_xt  (1:kproma,2*nflevg+1:3*nflevg) = zphys_rfl  (1:kproma,1:nflevg)
p1d_x_xt  (1:kproma,3*nflevg+1:4*nflevg) = zphys_sfl  (1:kproma,1:nflevg)
p1d_x_xt  (1:kproma,4*nflevg+1:5*nflevg) = zphys_cwc  (1:kproma,1:nflevg)
p1d_x_xt  (1:kproma,5*nflevg+1:6*nflevg) = zphys_cic  (1:kproma,1:nflevg)
p1d_x_xt  (1:kproma,6*nflevg+1:7*nflevg) = zphys_cc   (1:kproma,1:nflevg)

p1d_x_xt5 (1:kproma,7*nflevg+11) = zphys_srf5 (1:kproma)                  
p1d_x_xt5 (1:kproma,7*nflevg+12) = zphys_crf5 (1:kproma)
                                            
p1d_x_xt  (1:kproma,7*nflevg+11) = zphys_srf  (1:kproma)                  
p1d_x_xt  (1:kproma,7*nflevg+12) = zphys_crf  (1:kproma)
                                            
!* RTTOVSCATT-interface
ichannels = k1d_y * kproma
    
call onedvar_obsop_tl_rttov (ksat                 , &       ! sens num
                          &  k1d_x, k1d_y, k1d_z  , &       ! dimensions       
                          &  ichannels            , &       ! for rttov 
                          &  i_flevg, kproma      , &       ! in: levels, profiles
                          &  p1d_x5               , &       ! in: control variable
                          &  p1d_x_xt5            , &       ! in: extra fields
                          &  zphys_p5             , &       ! in: pressure
                          &  zphys_ph5            , &       ! in: pressure at i/f
                          &  p1d_y5               , &       ! out: radiances
                          &  p1d_x                , &       ! in: control variable
                          &  p1d_x_xt             , &       ! in: extra fields
                          &  zphys_p              , &       ! in: pressure
                          &  zphys_ph             , &       ! in: pressure at i/f
                          &  p1d_y)                         ! out: radiances

if (lhook) call dr_hook ('ONEDVAR_OBSOP_TL',1,zhook_handle) 

end subroutine onedvar_obsop_tl
