subroutine onedvar_raintb ( kstart, kend, &
 &      kassim_mode, &   ! in: Passive or active assimilation?
 &      pt         , &   ! in:  3-d temperature
 &      pq         , &   ! in:  3-d spec. humidity
 &      pten_t     , &   ! in:  3-d temperature tendency 
 &      pten_q     , &   ! in:  3-d spec. humidity tendency         
 &      prain      , &   ! in:  3-d liquid precipitation
 &      psnow      , &   ! in:  3-d solid precipitation       
 &      pclfr      , &   ! in:  3-d cloud fraction    
 &      pcllw      , &   ! in:  3-d cloud liquid water content   
 &      pcliw      , &   ! in:  3-d cloud ice water content               
 &      pt_2m      , &   ! in:  2-d temperature @ 2m
 &      pq_2m      , &   ! in:  2-d spec. humidity @ 2m
 &      ptd_2m     , &   ! in:  2-d dew-point temperature @ 2m
 &      pu_star    , &   ! in:  2-d wind stress, u
 &      pv_star    , &   ! in:  2-d wind stress, v                             
 &      pu_10m_fg  , &   ! in:  2-d wind speed, u @ 10m, FG
 &      pv_10m_fg  , &   ! in:  2-d wind speed, v @ 10m, FG
 &      psh_sfc    , &   ! in:  2-d sensible heat flux @ sfc
 &      plh_sfc    , &   ! in:  2-d latent heat flux @ sfc
 &      psea_ice   , &   ! in:  2-d sea-ice matrix
 &      pland_sea  , &   ! in:  2-d land-sea matrix
 &      p_sfc      , &   ! in:  2-d pressure @ sfc
 &      pt_sfc     , &   ! in:  2-d skin temperature @ sfc   
! Observations:
 &      ptb_obs    , &   ! in:  3-d radiance      
 &      pslat_obs  , &   ! in:  2-d sin latitude
 &      pclon_obs  , &   ! in:  2-d cos longitude
 &      pslon_obs  , &   ! in:  2-d sin longitude
 &      ksat_obs   , &   ! in:  2-d satellite ID
 &      kfov_obs   , &   ! in:  2-d IFOV number  
 &      pzen_obs   , &   ! in:  2-d zenith angle
! Retrievals:
 &      kid_dv     , &   ! out: q/c indicator (0 = OK)                        
 &      kit_dv     , &   ! out: no. of iterations  
 &      kis_dv     , &   ! out: no. of simulations    
 &      pepsg      , &   ! out: ratio of norm of cost function gradients  
 &      kim_dv     , &   ! out: M1QN3 minimisation failure code (0 = OK)                        
 &      p_fg       , &   ! out: 3-d pressure
 &      pt_an      , &   ! out: 3-d temperature
 &      pq_an      , &   ! out: 3-d spec. humidity
 &      prr_fg     , &   ! out: 3-d rain, FG
 &      prs_fg     , &   ! out: 3-d snow, FG
 &      prl_fg     , &   ! out: 3-d cloud water, FG
 &      pri_fg     , &   ! out: 3-d cloud ice, FG
 &      prc_fg     , &   ! out: 3-d cloud cover, FG
 &      prr_an     , &   ! out: 3-d rain, AN
 &      prs_an     , &   ! out: 3-d snow, AN
 &      prl_an     , &   ! out: 3-d cloud water, AN
 &      pri_an     , &   ! out: 3-d cloud ice, AN
 &      prc_an     , &   ! out: 3-d cloud cover, AN
 &      pu_10m_an  , &   ! out: 2-d 10m u wind, AN 
 &      pv_10m_an  , &   ! out: 2-d 10m v wind, AN 
 &      prst_fg    , &   ! out: 2-d strat. precip. @ sfc, FG 
 &      prco_fg    , &   ! out: 2-d conv.  precip. @ sfc, FG 
 &      prst_an    , &   ! out: 2-d strat. precip. @ sfc, AN 
 &      prco_an    , &   ! out: 2-d conv.  precip. @ sfc, AN 
 &      ptcwv_fg   , &   ! out: 2-d total column water vapor, FG
 &      ptcwv_an   , &   ! out: 2-d total column water vapor, AN
 &      ptcwv_fg_er, &   ! out: 2-d total column water vapor error
 &      ptcwv_an_er, &   ! out: 2-d total column water vapor error
 &      pco        , &   ! out: 2-d cost function ratio, Jo_1/Jo_n 
 &      ptb_fg     , &   ! out: 3-d radiance, FG
 &      ptb_an     , &   ! out: 3-d radiance, AN
 &      ptb_er     , &   ! out: 3-d radiance, OBS-Err.
 &      ptb_bi     , &   ! out: 3-d radiance, Bias 
 &      ptb_fg_3d  , &   ! out: 3-d radiance, FG from 3D model clouds not 1D 
 &      p1ddiag     )    ! out: 2-d diagnostics of rain and cloud
     
!**** 1D-Var in clouds/precipitation for SSM/I radiances

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     Activates 1D-Var retrievals of TCWV using SSM/I TB's on gridpoint locations by:
!     1. Selecting active profiles and active channels
!     2. Applying a bias-correction to active channels
!     3. Interpolating background error standard deviations to grid point locations
!     4. Calculating BG-equivalent TB's, initializing cost-function
!     5. Minimizing the cost-function for new humidity variable
!     6. Calculating AN-equivalent TB's, analyzed TCWV and error (by fit)

!     ONEDVAR_RAINTB is called from EC_PHYS, ONEDVAR_RAINTB_HLP. It is called twice:

!     1. On the first trajectory of the 4D-var minimisation, when
!        the 1D-Var retrieval is run.

!     2. On the final trajectory of the 4D-Var minimisation, when
!        TBs are calculated from the analysis state. No retrieval is done.

!     ifail = 0 => OK; Any bits set = passive obs or failure
!     ifail = -1 reserved as an initialisation value to indicate coding errors


!     Modifications:

!     A. Geer     4-Apr-2006   Add 10m wind to control vector; check Tb analysis
!                              departures; new bitwise failure codes (ifail)
!     A. Geer    23-Oct-2006   Check M1QN3 minimisation, info to ODB   
!     C. O'Dell   17-Oct-2006  Multiple changes to allow for multiple sensors (still one
!                              sensor per profile only).  In particular, call to screening
!                              now performed INSIDE the profile loop.  Other small changes as well.
!     A. Geer    12-Dec-2006   Streamline the optional linearity test
!     C. O'Dell  15-Jan-2007   Finalize code changes for multiple sensors
!     A. Geer    06-Mar-2007   More code changes for multiple sensors. Calculate 4D-analysed 
!                              Tb and also use the 3D rain fields. Tidy code. 
!     A. Geer    12-Nov-2007   Relax FG departure limits

!**** MODULES
use yomlun   , only: nulerr
use yomvar   , only: nuptra
use yomancs  , only: rmdi 

!* KIND     
use parkind1     , only: jpim, jprb
use yomhook      , only: lhook,   dr_hook

!* IFS
use yomdim       , only: nflevg, nproma  
use yomcst       , only: rg, rpi

!* 1D-VAR
use paronedvar, only: n_max, nmax_channels, n_mode, rq_min
use yomonedvar, only: lerain_anerr, lerain_lintest, ran_err, rcor_t, &
 &  rcor_q, rlimit_inc, rsig_uv10, rlimit_an_dep, rconvergence, repsg_max, &
 &  rlimit_err, onedvar_id, stream1d_passive, lerain_adjtest, &
 &  ifail_sensor, ifail_tcwv_increment, ifail_an_tb_depar, ifail_tb_increment, &
 &  ifail_cost_function, ifail_no_convergence, ifail_minim_failed, &
 &  ifail_no_iterations, ifail_clear, ifail_passive, lerain_clearsky, &
 &  ifail_fg_t_q                 
use yom_ptr_ssmi, only: mflds_1ddiag, &
 &  m_tcwv_3d, m_rwp_3d, m_swp_3d, m_cwp_3d, m_iwp_3d, m_ra_3d, m_sn_3d, &
 &  m_tcwv_fg, m_rwp_fg, m_swp_fg, m_cwp_fg, m_iwp_fg, m_ra_fg, m_sn_fg, &
 &  m_tcwv_an, m_rwp_an, m_swp_an, m_cwp_an, m_iwp_an, m_ra_an, m_sn_an 

implicit none                         

!**** VARIABLES
!* IN/OUTPUT
integer (kind=jpim), intent (in) :: kstart, kend
integer (kind=jpim), intent (in) :: kassim_mode

real (kind=jprb), intent (in), dimension (nproma,nflevg) :: &
 & pt, pq, pten_t, pten_q, prain, psnow, pclfr,  pcllw,  pcliw   
                                                                      
real (kind=jprb), intent (in), dimension (nproma) :: &
 & pt_2m, pq_2m, ptd_2m, pu_star, pv_star, pu_10m_fg, pv_10m_fg, &
 & psh_sfc, plh_sfc, psea_ice, pland_sea, p_sfc, pt_sfc
  
real (kind=jprb), intent (in), dimension (nproma,nmax_channels) :: &
 & ptb_obs
 
real (kind=jprb), intent (in), dimension (nproma) :: &
 & pslat_obs, pslon_obs, pclon_obs, pzen_obs 
   
real (kind=jprb), intent (out), dimension (nproma,nflevg) ::  &  
  & pt_an    , &   
  & pq_an    , &
  & p_fg     , &
  & prr_fg   , &   ! out: 3-d rain, FG
  & prs_fg   , &   ! out: 3-d snow, FG
  & prl_fg   , &   ! out: 3-d cloud water, FG
  & pri_fg   , &   ! out: 3-d cloud ice, FG
  & prc_fg   , &   ! out: 3-d cloud cover, FG
  & prr_an   , &   ! out: 3-d rain, AN
  & prs_an   , &   ! out: 3-d snow, AN
  & prl_an   , &   ! out: 3-d cloud water, AN
  & pri_an   , &   ! out: 3-d cloud ice, AN
  & prc_an         ! out: 3-d cloud cover, AN

real (kind=jprb), intent (out), dimension (nproma) ::  &
  & pu_10m_an  , &    ! out: 2-d 10m wind u, AN
  & pv_10m_an  , &    ! out: 2-d 10m wind v, AN
  & ptcwv_fg   , &    ! out: 2-d TCWV, FG
  & ptcwv_an   , &    ! out: 2-d TCWV, AN
  & ptcwv_fg_er, &    ! out: 2-d TCWV, FG-Error
  & ptcwv_an_er, &    ! out: 2-d TCWV, FG-Error
  & prst_fg    , &    ! out: 2-d strat. precip. @ sfc, FG
  & prco_fg    , &    ! out: 2-d conv.  precip. @ sfc, FG
  & prst_an    , &    ! out: 2-d strat. precip. @ sfc, AN
  & prco_an    , &    ! out: 2-d conv.  precip. @ sfc, AN
  & pco               ! out: 2-d cost function ratio, Jo_1/Jo_n 
  
real (kind=jprb), intent (out), dimension (nproma,nmax_channels) :: &
  & ptb_fg     , &    ! out: 3-d TB, FG
  & ptb_an     , &    ! out: 3-d TB, AN
  & ptb_er     , &    ! out: 3-d TB, Error
  & ptb_bi     , &    ! out: 3-d TB, Bias
  & ptb_fg_3d         ! out: 3-d TB, Cost

real (kind=jprb), intent (out), dimension (nproma, mflds_1ddiag) :: p1ddiag

integer (kind=jpim), intent (out), dimension (nproma)      :: kid_dv
integer (kind=jpim), intent (out), dimension (nproma)      :: kit_dv, kim_dv, kis_dv
real (kind=jprb), intent (out), dimension (nproma)         :: pepsg
integer (kind=jpim), intent ( in), dimension (nproma)      :: ksat_obs  ,  kfov_obs 
                   
!* OTHER
real (kind=jprb), dimension (nflevg)    :: zsig_t, zsig_q, zdp
real (kind=jprb), dimension (nproma,nflevg+1)  :: zph_fg
real (kind=jprb), dimension (nproma,nflevg)    :: zt, zq

real (kind=jprb) :: zlat_obs_rad, zlon_obs_rad, zeps, zan_dep_growth
real (kind=jprb) :: zxj_s, zxj_e, zdxn_min, zeps_g, zdj_1, zdtb_norm, z1d_tcwv_a, z1d_tcwv_b
real (kind=jprb), dimension (0:n_max) :: zxj_o, zxj_b

real    (kind=jprb), allocatable :: zrz (:), zxn (:), zdj (:)
integer (kind=jpim), allocatable :: iz (:)

integer (kind=jpim) :: i_work, j_work, i_iter, i_simul, i1d_x, i1d_y, i1d_z, i_print
integer (kind=jpim) :: i_prof, j_lev, i_lev, iy, jy, ix, jx, ixx, info
integer (kind=jpim) :: ifail, inumb, iproma, istglo, iopt
integer (kind=jpim) :: isat
logical :: ll_essl, ll_depar, llxz_allocated, lly_allocated, llxy_allocated
logical :: llcontinue_clearsky
integer (kind=jpim) :: iminim_mode  
real (kind=jprb), dimension (nproma,nmax_channels) :: ztb_fg_3d_clear
 
!* 1D-VAR   
real (kind=jprb), allocatable, dimension (:)   :: zwork                                     ! for Lapack subroutine
real (kind=jprb), allocatable, dimension (:)   :: z1d_x_fg, z1d_x_an, zz1d_x_fg, z1d_x_sigb ! control variables
real (kind=jprb), allocatable, dimension (:)   :: z1d_x_tl, z1d_xt_tl
real (kind=jprb), allocatable, dimension (:)   :: z1d_x_xt, z1d_x_xt_an                     ! extra fields
real (kind=jprb), allocatable, dimension (:)   :: z1d_b_eval, zz1d_b_eval                   ! eigenvalues B

! background covariance matrices (B) and (B^-1)
real (kind=jprb), allocatable, dimension (:,:) :: zz1d_b, z1d_b, z1d_b_evec, zz1d_b_evec     

real (kind=jprb), allocatable, dimension (:,:) :: z1d_ht, z1d_hbht, z1d_a_b, z1d_a_o, z1d_a, zidentity

! Lower-triangular packed B  
real (kind=jprb), allocatable, dimension (:)   :: z1d_b_packed                                                   

real (kind=jprb), allocatable, dimension (:,:) :: zz1d_x, zz1d_y, zz1d_x_xt           
real (kind=jprb), allocatable, dimension (:,:) :: z1d_dj_o, z1d_dj_ox

! observation covariance matrices (R) and (R^-1)
real (kind=jprb), allocatable, dimension (:,:) :: z1d_r, z1d_r_inv        

! observed radiances [K]                   
real (kind=jprb), allocatable, dimension (:)   :: z1d_y, z1d_y_an, z1d_y_fg, z1d_y_lin       

real (kind=jprb) :: zhook_handle

!* Externals
external euclid_1dv, ctonb_1dv, ctcab_1dv

!* External functions
logical, external :: onedvar_passive_ok

!* INTERFACES
#include "onedvar_fstscrn.intfb.h"
#include "onedvar_screen.intfb.h"
#include "onedvar_obsop.intfb.h"
#include "onedvar_obsop_gr.intfb.h"
#include "onedvar_simul.intfb.h"
#include "onedvar_get_bgsig.intfb.h"
#include "onedvar_lintest.intfb.h"
#include "onedvar_adjoint_test.intfb.h"
#include "onedvar_find_satsens.intfb.h"
#include "matrixin.intfb.h"
#include "onedvar_diagnostics.intfb.h"

if (lhook) call dr_hook ('ONEDVAR_RAINTB',0,zhook_handle)

!* START 
ll_essl    = .true.
i_print    = 0
inumb      = 1
info       = 0
iproma     = nproma
llxz_allocated = .FALSE.
lly_allocated = .FALSE.
llxy_allocated = .FALSE.
i1d_z  = 7 * nflevg + 14  ! lots of other stuff, including skin temperature

! Initialise 4D-Var and 1D-Var observation output. Used by hdepart.F90 and in 
! other places even when the 1D-Var retrieval failed: these must be 
! flagged as RMDI.
ptcwv_fg (kstart:kend) = RMDI
ptcwv_an (kstart:kend) = RMDI
ptcwv_fg_er (kstart:kend) = RMDI
ptcwv_an_er (kstart:kend) = RMDI
ptb_fg (kstart:kend,:) = RMDI  
ptb_an (kstart:kend,:) = RMDI      
ptb_er (kstart:kend,:) = RMDI     
ptb_bi (kstart:kend,:) = RMDI     
ptb_fg_3d (kstart:kend,:) = RMDI             
ztb_fg_3d_clear (kstart:kend,:) = RMDI            

! Initialise 1D-Var specific output
kit_dv (kstart:kend) = 0 
kim_dv (kstart:kend) = 0 
kis_dv (kstart:kend) = 0 
kid_dv (kstart:kend) = 0
pepsg (kstart:kend)  = RMDI 
prst_fg (kstart:kend) = RMDI
prco_fg (kstart:kend) = RMDI
prst_an (kstart:kend) = RMDI
prco_an (kstart:kend) = RMDI
pco (kstart:kend) = RMDI
pu_10m_an (kstart:kend) = RMDI
pv_10m_an (kstart:kend) = RMDI
p_fg (kstart:kend,:) = RMDI
prr_fg (kstart:kend,:) = RMDI
prs_fg (kstart:kend,:) = RMDI
prl_fg (kstart:kend,:) = RMDI
pri_fg (kstart:kend,:) = RMDI
prc_fg (kstart:kend,:) = RMDI
pt_an (kstart:kend,:) = RMDI    
pq_an (kstart:kend,:) = RMDI   
prr_an (kstart:kend,:) = RMDI  
prs_an (kstart:kend,:) = RMDI 
prl_an (kstart:kend,:) = RMDI 
pri_an (kstart:kend,:) = RMDI
prc_an (kstart:kend,:) = RMDI        
p1ddiag (kstart:kend,:) = RMDI

zt (1:iproma,1:nflevg) = pt (1:iproma,1:nflevg)
zq (1:iproma,1:nflevg) = pq (1:iproma,1:nflevg)
   
do i_prof = kstart, kend  ! loop over active profiles
  istglo = i_prof 

  !----------------------------------------------------------------------------
  ! 1.0 Screening of observation and first guess
  !----------------------------------------------------------------------------

  ! find isat using satellite id number
  call onedvar_find_satsens( ksat_obs(i_prof), isat)
  if (isat == 0) then
    ! failed: not a recognized sensor
    kid_dv(i_prof) = ibset(kid_dv(i_prof),ifail_sensor) 
    cycle
  endif
      
  ! Set dimensions for various arrays (control vector, obs vector, extra vector, associated matrices)
  i1d_x  = 2 * nflevg + 2   ! T & Q profiles + u and v 10m wind speeds
  i1d_y = onedvar_id(isat) % nchannels

  ! Passively monitored observations are flagged 
  if (kassim_mode == stream1d_passive) then
    kid_dv(i_prof) = ibset (kid_dv(i_prof), ifail_passive)
  endif

  ! Call fast screen test
  CALL ONEDVAR_FSTSCRN(onedvar_id(isat) % sensid, i1d_y, &
    & ptb_obs(i_prof,1:i1d_y), pslat_obs(i_prof), psea_ice(i_prof), &
    & pland_sea(i_prof), pt_sfc(i_prof), kid_dv(i_prof))
    
  ! Continue processing successful observations (either active or passive) 
  if ( .not. onedvar_passive_ok(kid_dv(i_prof)) ) cycle
               
  ! get lat and longitude in radians
  zlat_obs_rad = asin (pslat_obs (i_prof))
  zlon_obs_rad = acos (pclon_obs (i_prof))
  if (pslon_obs (i_prof) <= 0.0_JPRB) zlon_obs_rad = -1.0_JPRB * zlon_obs_rad 
                
  call onedvar_screen( inumb, i1d_x     , &   ! in: scalar
   &      i1d_y, i1d_z     , &   ! in: scalar
   &      isat             , &   ! in:  satellite index number
   &      kfov_obs(i_prof) , &   ! in:  IFOV number
   &      zt(i_prof,:)     , &   ! in:  temperature profile
   &      zq(i_prof,:)     , &   ! in:  spec. humidity profile
   &      pten_t(i_prof,:) , &   ! in:  temperature tendency profile
   &      pten_q(i_prof,:) , &   ! in:  temperature tendency profile       
   &      prain(i_prof,:)  , &   ! in:  liquid precipitation profile 
   &      psnow(i_prof,:)  , &   ! in:  solid precipitation profile 
   &      pclfr(i_prof,:)  , &   ! in:  3-d cloud fraction    
   &      pcllw(i_prof,:)  , &   ! in:  3-d cloud liquid water content   
   &      pcliw(i_prof,:)  , &   ! in:  3-d cloud ice water content      
   &      pt_2m(i_prof)    , &   ! in:  temperature @ 2m
   &      pq_2m(i_prof)    , &   ! in:  spec. humidity @ 2m
   &      ptd_2m(i_prof)   , &   ! in:  dew-point temperature @ 2m
   &      pu_star(i_prof)  , &   ! in:  wind stress, u
   &      pv_star(i_prof)  , &   ! in:  wind stress, v                             
   &      pu_10m_fg(i_prof), &   ! in:  wind speed, u @ 10m
   &      pv_10m_fg(i_prof), &   ! in:  wind speed, v @ 10m
   &      psh_sfc(i_prof)  , &   ! in:  sensible heat flux @ sfc
   &      plh_sfc(i_prof)  , &   ! in:  latent heat flux @ sfc
   &      psea_ice(i_prof) , &   ! in:  sea-ice matrix
   &      pland_sea(i_prof), &   ! in:  land-sea matrix
   &      p_sfc(i_prof)    , &   ! in:  pressure @ sfc
   &      pt_sfc(i_prof)   , &   ! in:  skin temperature @ sfc
   &      pzen_obs(i_prof) , &   ! in:  zenith angle [degrees]   
   &      zlat_obs_rad     , &   ! in:  observation longitude [radians]   
   &      zlon_obs_rad     , &   ! in:  observation latitude [radians]   
   &      ptb_obs(i_prof,1:i1d_y), &   ! in: observed radiances 
   &      p_fg(i_prof,:)   , &   ! out: pressure profile FG
   &      zph_fg(i_prof,:) , &   ! out: half-pressure profile FG
   &      prr_fg(i_prof,:) , &   ! out: rain rate profile FG
   &      prs_fg(i_prof,:) , &   ! out: snow rate profile FG
   &      prl_fg(i_prof,:) , &   ! out: liq cloud profile FG
   &      pri_fg(i_prof,:) , &   ! out: ice cloud profile FG
   &      prc_fg(i_prof,:) , &   ! out: cloud fraction profile FG
   &      prst_fg(i_prof)  , &   ! out: stratiform rain FG
   &      prco_fg(i_prof)  , &   ! out: convective rain FG
   &      ptcwv_fg(i_prof) , &   ! out: tcwv fg 
   &      ptb_bi(i_prof,1:i1d_y) , &   ! out: TB bias array
   &      ptb_fg(i_prof,1:i1d_y) , &   ! out: TB first-guess array
   &      ptb_fg_3d(i_prof,1:i1d_y), &   ! out: TB first-guess array
   &      ztb_fg_3d_clear(i_prof,1:i1d_y), &   ! out: TB clear sky 3D
   &      kid_dv(i_prof))        ! out: onedvar failure code

  ! Skip profile if it failed sanity checks on T and q profile
  if ( btest(kid_dv(i_prof), ifail_fg_t_q) ) cycle
  
  ! 3D trajectory diagnostics
  call onedvar_diagnostics ( 1, 1, 1, &  
    & p_sfc(i_prof), &             ! in: pressure @ sfc
    & zt(i_prof,:), &              ! in: temperature
    & zq(i_prof,:), &              ! in: spec. humidity
    & prain(i_prof,:), &           ! in: rain
    & psnow(i_prof,:), &           ! in: snow
    & pcllw(i_prof,:), &           ! in: cloud water
    & pcliw(i_prof,:), &           ! in: cloud ice
    & p1ddiag(i_prof,m_tcwv_3d), & ! out: total column water vapour
    & p1ddiag(i_prof,m_rwp_3d), &  ! out: rain water path
    & p1ddiag(i_prof,m_swp_3d), &  ! out: snow water path
    & p1ddiag(i_prof,m_cwp_3d), &  ! out: cloud water path
    & p1ddiag(i_prof,m_iwp_3d), &  ! out: cloud ice water path
    & p1ddiag(i_prof,m_ra_3d), &   ! out: surface rain rate
    & p1ddiag(i_prof,m_sn_3d)  )   ! out: surface snow rate       

  ! 1DVAR FG diagnostics (cloud inputs are initialised to zero in 
  ! onedvar_screen whether or not moist physics is run.)
  CALL onedvar_diagnostics ( 1, 1, 1, &     
    & p_sfc(i_prof), &             ! in: pressure @ sfc
    & zt(i_prof,:), &              ! in: temperature
    & zq(i_prof,:), &              ! in: spec. humidity
    & prr_fg(i_prof,:), &          ! in: rain
    & prs_fg(i_prof,:), &          ! in: snow
    & prl_fg(i_prof,:), &          ! in: cloud water
    & pri_fg(i_prof,:), &          ! in: cloud ice
    & p1ddiag(i_prof,m_tcwv_fg), & ! out: total column water vapour
    & p1ddiag(i_prof,m_rwp_fg), &  ! out: rain water path
    & p1ddiag(i_prof,m_swp_fg), &  ! out: snow water path
    & p1ddiag(i_prof,m_cwp_fg), &  ! out: cloud water path
    & p1ddiag(i_prof,m_iwp_fg), &  ! out: cloud ice water path
    & p1ddiag(i_prof,m_ra_fg), &   ! out: surface rain rate
    & p1ddiag(i_prof,m_sn_fg)  )   ! out: surface snow rate       

  ! Only do a retrieval on successful active observations (and optionally
  ! for clear sky profiles too)     
  llcontinue_clearsky = (kid_dv(i_prof) == ibset(0, ifail_clear)) .and. lerain_clearsky
  if (kid_dv(i_prof) /= 0 .and. (.not. llcontinue_clearsky) ) cycle 

  ! Only do a retrieval on the first guess trajectory
  if (nuptra /= 0 ) cycle

  !----------------------------------------------------------------------------
  ! 2.0 1D-Var retrieval
  !----------------------------------------------------------------------------

  ! Deallocate all the relevant variables if their size has changed
  if (llxz_allocated) then
    if (size(z1d_x_fg) /= i1d_x) then
      deallocate (zxn, zdj, zrz, iz)    
      deallocate (zz1d_x_fg, z1d_x_fg, z1d_x_sigb, z1d_x_an, z1d_x_xt, z1d_x_tl)
      deallocate (zz1d_x, zz1d_x_xt, zwork, z1d_xt_tl)
      deallocate (z1d_dj_ox, z1d_a_b, z1d_a_o, z1d_a, zz1d_b, z1d_x_xt_an)
      deallocate (z1d_b, z1d_b_packed, z1d_b_eval, z1d_b_evec, zz1d_b_eval, zz1d_b_evec)
      deallocate (zidentity)
      llxz_allocated = .FALSE.
    endif
  endif            
  if (lly_allocated) then
    if (size(z1d_y) /= i1d_y) then 
      deallocate (z1d_y, z1d_y_an, z1d_y_fg, z1d_y_lin, z1d_r, z1d_r_inv, zz1d_y, z1d_hbht, z1d_dj_o)
      lly_allocated = .FALSE.
   endif
  endif            
  if (llxy_allocated) then 
    if ((size(z1d_ht,1) /= i1d_x)  .OR.  (size(z1d_ht,2) /= i1d_y)) then
      deallocate(z1d_ht)
      llxy_allocated = .FALSE.
    endif
  endif
      
  !* Allocate 1D-Var OBS arrays
  i_work = i1d_x * 64
  j_work = i1d_x * 10      
  if (.not. llxz_allocated) then
    allocate (zwork      (i_work))   
    allocate (z1d_x_fg   (i1d_x))
    allocate (z1d_x_sigb (i1d_x))
    allocate (zz1d_x_fg  (i1d_x))
    allocate (z1d_x_an   (i1d_x))
    allocate (z1d_x_tl   (i1d_x))
    allocate (z1d_xt_tl  (i1d_z))
    allocate (z1d_x_xt   (i1d_z))
    allocate (z1d_b        (i1d_x,i1d_x)) 
    allocate (z1d_b_packed (i1d_x*(i1d_x+1)/2))
    allocate (z1d_b_eval   (i1d_x))
    allocate (z1d_b_evec   (i1d_x,i1d_x))
    allocate (zz1d_b_eval  (i1d_x))
    allocate (zz1d_b_evec  (i1d_x,i1d_x))
    allocate (zxn (i1d_x))
    allocate (zdj (i1d_x))
    allocate (zrz (j_work))
    allocate (iz (5))
    allocate (zz1d_x    (inumb,i1d_x))
    allocate (z1d_x_xt_an (i1d_z))
    allocate (zz1d_b      (i1d_x,i1d_x))
    allocate (z1d_a       (i1d_x,i1d_x))
    allocate (z1d_a_b     (i1d_x,i1d_x))
    allocate (z1d_a_o     (i1d_x,i1d_x))
    allocate (z1d_dj_ox   (inumb,i1d_x))
    allocate (zz1d_x_xt (inumb,i1d_z)) 
    allocate (zidentity   (i1d_x,i1d_x))
    llxz_allocated = .TRUE.
  endif        
  if (.NOT. lly_allocated) then 
    allocate (z1d_y      (i1d_y))
    allocate (z1d_y_fg   (i1d_y))
    allocate (z1d_y_an   (i1d_y))
    allocate (z1d_y_lin  (i1d_y))
    allocate (z1d_r      (i1d_y,i1d_y))
    allocate (z1d_r_inv  (i1d_y,i1d_y))
    allocate (zz1d_y     (inumb,i1d_y))
    allocate (z1d_hbht   (i1d_y,i1d_y))   
    allocate (z1d_dj_o   (inumb,i1d_y))
    lly_allocated = .TRUE.
  endif
  if (.NOT. llxy_allocated) then 
    allocate(z1d_ht(i1d_x,i1d_y))
    llxy_allocated = .TRUE.
  endif
             
  ifail = 0
           
  !* set control variables + extra fields common to both land and ocean retrievals
  z1d_x_fg (         1:  nflevg) = zt        (i_prof,1:nflevg)
  z1d_x_fg (  nflevg+1:2*nflevg) = zq        (i_prof,1:nflevg)
  z1d_x_fg (2*nflevg+1         ) = pu_10m_fg (i_prof)
  z1d_x_fg (2*nflevg+2         ) = pv_10m_fg (i_prof)
              
  z1d_x_xt (         1:  nflevg) = pten_t    (i_prof,1:nflevg)
  z1d_x_xt (  nflevg+1:2*nflevg) = pten_q    (i_prof,1:nflevg)
  z1d_x_xt (7*nflevg+1         ) = pt_2m     (i_prof)
  z1d_x_xt (7*nflevg+2         ) = pq_2m     (i_prof)
  z1d_x_xt (7*nflevg+3         ) = ptd_2m    (i_prof)
  z1d_x_xt (7*nflevg+4         ) = pu_star   (i_prof)
  z1d_x_xt (7*nflevg+5         ) = pv_star   (i_prof)
  z1d_x_xt (7*nflevg+6         ) = psh_sfc   (i_prof)
  z1d_x_xt (7*nflevg+7         ) = plh_sfc   (i_prof)
  z1d_x_xt (7*nflevg+8         ) = pland_sea (i_prof)
  z1d_x_xt (7*nflevg+9         ) = p_sfc     (i_prof)
  z1d_x_xt (7*nflevg+10        ) = pt_sfc    (i_prof) ! only used by ocean pixels, but always put it in
 
  !* Use zenith angle if supplied, else a known fixed value for this satellite     
  if (pzen_obs(i_prof) >= 0.0_JPRB) then
    z1d_x_xt(7*nflevg+13) = pzen_obs(i_prof)
  else
    z1d_x_xt(7*nflevg+13) = onedvar_id(isat) % zenangle
  endif

  ! Latitude, converted into degrees
  z1d_x_xt (7*nflevg+14        ) = zlat_obs_rad * 180.0_JPRB / rpi
      
  !* set observations, inverse of observation error covariance matrix (diagonal)
  z1d_r     (:,:) = 0.0_JPRB
  z1d_r_inv (:,:) = 0.0_JPRB

  do iy = 1, i1d_y
    z1d_y (iy)    = ptb_obs (i_prof,iy) - ptb_bi (i_prof,iy)
    z1d_y_fg (iy) = ptb_fg (i_prof,iy)
    z1d_r (iy,iy) = onedvar_id(isat)%robs_err(iy) ** 2.0_JPRB
    z1d_r_inv (iy,iy) = 1.0_JPRB / z1d_r (iy,iy)
  enddo
             
  !* background error covariance matrix 
  call onedvar_get_bgsig (p_sfc (i_prof), zlat_obs_rad, zlon_obs_rad, zsig_t, zsig_q)

  z1d_b (:,:) = 0.0_JPRB

  do i_lev = 1, nflevg
    do j_lev = 1, nflevg   
      z1d_b (       i_lev,       j_lev) = rcor_t (i_lev,j_lev) * zsig_t (i_lev) * zsig_t (j_lev)
      z1d_b (nflevg+i_lev,nflevg+j_lev) = rcor_q (i_lev,j_lev) * zsig_q (i_lev) * zsig_q (j_lev)
    enddo
    z1d_x_sigb (i_lev)        = zsig_t (i_lev)
    z1d_x_sigb (i_lev+nflevg) = zsig_q (i_lev)
  enddo

  z1d_b (2*nflevg+1,2*nflevg+1) = rsig_uv10 * rsig_uv10 ! 10m wind BG errors
  z1d_b (2*nflevg+2,2*nflevg+2) = rsig_uv10 * rsig_uv10
  z1d_x_sigb (2*nflevg+1)       = rsig_uv10 
  z1d_x_sigb (2*nflevg+2)       = rsig_uv10 

  zz1d_b_evec (:,:) = z1d_b (:,:)
  zz1d_b      (:,:) = z1d_b (:,:)

  zidentity(:,:) = 0.0_jprb
  if (ll_essl) then 

    ! Convert B to lower-packed storage
    ixx = 0
    do jx = 1, i1d_x
      do ix = jx, i1d_x
        ixx=ixx+1
        z1d_b_packed(ixx)=z1d_b(ix,jx)
      enddo
    enddo

    ! ESSL library: eigenvectors of a symmetric matrix 
    iopt = 1
    call dspev (iopt, z1d_b_packed, z1d_b_eval, z1d_b_evec, i1d_x, i1d_x, zwork, i_work)

    do ix = 1, i1d_x
      z1d_b_eval (ix) = max (z1d_b_eval (ix), 1.0E-12_JPRB)
      zidentity(ix,ix) = 1.0_jprb
    enddo

  else      
    !* Eigenvectors/-values
!$OMP CRITICAL
    call dsyev ('V', 'U', i1d_x, zz1d_b_evec, i1d_x, zz1d_b_eval, zwork, i_work, info)
!$OMP END CRITICAL
    if (info /= 0) write (nulerr,*) 'DSYEV Problem: onedvar_raintb', info
    z1d_b_evec(:,:) = zz1d_b_evec(:,:)
    z1d_b_eval = zz1d_b_eval(:)

    do ix = 1, i1d_x
      z1d_b_eval (ix) = max (z1d_b_eval (ix), 1.0E-10_JPRB)
      zidentity(ix, ix) = 1.0_jprb
    enddo
  endif
      
  !     Eigenvalues Test
  zxn = sum(abs(matmul(z1d_b_evec, transpose(z1d_b_evec)) - zidentity),1)
  if (sum(zxn) > 1e-6_jprb) write(nulerr,*) &
   &  'onedvar_raintb: Eigenvector Test FAILED! Sum = ', sum(zxn)

  !* 1D-Var
  z1d_x_an (1:i1d_x) = z1d_x_fg (1:i1d_x)
                               
  !* 1st dJ at FG, ie x=x_fg       
  zxn   (:) = 0.0_JPRB
  zdj   (:) = 0.0_JPRB
  zxj_o (:) = 0.0_JPRB
  zxj_b (:) = 0.0_JPRB
  i_iter = 0

  call onedvar_simul (isat, i1d_x, i1d_y, i1d_z, n_max, inumb, istglo, &
   &  zxn                 , &
   &  z1d_x_fg            , &
   &  z1d_x_xt            , &
   &  z1d_b_evec          , &
   &  z1d_b_eval          , &
   &  z1d_x_sigb          , &
   &  z1d_y               , &     
   &  z1d_r_inv           , &
   &  i_iter              , &
   &  zxj_o               , &
   &  zxj_b               , &
   &  zdj                 , &
   &  ifail)

  zxj_s = zxj_b (0) + zxj_o (0)       

  !* Minimization
  zxj_e    = 0.0_JPRB
  zdxn_min = 1.0e-20_JPRB 
  zdj_1    = 0.1_JPRB * zxj_s
  zeps_g   = rconvergence
  i_iter   = n_max - 1
  i_simul  = n_max - 1

  zz1d_x_fg (:) = z1d_x_fg (:) ! changed in m1qn3_1dv

  iminim_mode = n_mode ! On output, this becomes a return status code 

  call m1qn3_1dv (isat, i1d_x, i1d_y, i1d_z, n_max, inumb, istglo, &
   &  zz1d_x_fg          , &
   &  z1d_x_xt           , &
   &  z1d_b_evec         , &
   &  z1d_b_eval         , &
   &  z1d_x_sigb          , &
   &  z1d_y              , &     
   &  z1d_r_inv          , &     
   &  zxj_o              , &
   &  zxj_b              , &
   &  euclid_1dv, ctonb_1dv, ctcab_1dv , &       
   &  i1d_x              , &
   &  zxn                , &
   &  zxj_s              , &
   &  zdj                , &
   &  zdxn_min           , &
   &  zdj_1              , &
   &  zeps_g             , &
   &  i_print            , &
   &  nulerr             , &
   &  iminim_mode         , &
   &  i_iter             , &
   &  i_simul            , &
   &  iz                 , &
   &  zrz                , &
   &  j_work             , &
   &  ifail)

  !----------------------------------------------------------------------------
  ! 3.0 Post-retrieval screening and output processing
  !----------------------------------------------------------------------------

  if (i_iter == 0) ifail = ibset (ifail, ifail_no_iterations)       

  if (ifail == 0) then     

    !* Only accept solutions where minimisation has converged (1) or the
    !* maximum number of simulations or iterations has been reached (4,5)
    if (iminim_mode /= 1 .and. iminim_mode /= 4 .and. iminim_mode /= 5) then
      ifail = ibset (ifail, ifail_minim_failed) 
    endif      
                                  
    !* Exclude profiles which have not sufficiently converged (test of cost
    !* function gradient)
    if (zeps_g > repsg_max) then
      ifail = ibset (ifail, ifail_no_convergence)   
    endif    
                                                                      
    !* zxn -> x
    z1d_x_an (:) = 0.0_JPRB
    do ix = 1, i1d_x
      do jx = 1, i1d_x
        z1d_x_an (ix) = z1d_x_an (ix) + &
          & sqrt (z1d_b_eval (jx)) * z1d_b_evec (ix,jx) * zxn (jx)
      enddo
    enddo

    z1d_x_an (:) = z1d_x_an (:) + z1d_x_fg (:)

    !* remove any negative or very small humidities
    do ix = nflevg+1, nflevg*2 
      if (z1d_x_an (ix) < rq_min) then
        z1d_x_an (ix) = rq_min
      endif
    enddo         
                      
    !* observation operator: AN 
    zz1d_x    (inumb,:) = z1d_x_an (:)
    zz1d_x_xt (inumb,:) = z1d_x_xt (:)

    call onedvar_obsop (isat, inumb, i1d_x, i1d_y, i1d_z, zz1d_x, zz1d_x_xt, zz1d_y)     

    z1d_x_xt_an (:) = zz1d_x_xt (inumb,:)
    z1d_y_an    (:) = zz1d_y    (inumb,:)

    !* check that total cost-function decreased   
    if (i_iter > 0) then     
       zxj_s = zxj_b (         0) + zxj_o (         0)         
       zxj_e = zxj_b (i_iter - 1) + zxj_o (i_iter - 1)         
    endif    

    if (zxj_e > zxj_s) ifail = ibset (ifail, ifail_cost_function)     
     
    !* check Tb departures
    zdtb_norm = 0.0
    do jy = 1, i1d_y
      if (onedvar_id(isat)%robs_err(jy) < rlimit_err) zdtb_norm = zdtb_norm + &
        & abs (z1d_y_an (jy) - z1d_y (jy)) / (3.0_JPRB * sqrt( z1d_r (jy,jy)))
    enddo

    if (zdtb_norm / i1d_y > 1.0_JPRB) ifail = ibset (ifail, ifail_tb_increment) 

    !* Check for AN departures that are significantly larger than FG departures
    ll_depar = .false.
    do jy = 1, i1d_y
      zan_dep_growth = (abs (z1d_y_an (jy) - z1d_y (jy)) - &
      & abs (z1d_y_fg (jy) - z1d_y (jy))) / sqrt( z1d_r (jy,jy))  
      ll_depar = ll_depar .or. (zan_dep_growth > rlimit_an_dep)
    enddo

    if (ll_depar) ifail = ibset (ifail, ifail_an_tb_depar)

    !* integrate TCWV in kg/m2, check increment
    ptcwv_an (i_prof) = 0.0_JPRB
    do j_lev = 1, nflevg
      zdp (j_lev) = (zph_fg (i_prof,j_lev+1) - zph_fg (i_prof,j_lev)) / rg           
      ptcwv_an (i_prof) = ptcwv_an (i_prof) + z1d_x_an (j_lev+nflevg) * zdp (j_lev)
    enddo

    if (abs (ptcwv_an (i_prof) - ptcwv_fg (i_prof)) / ptcwv_fg (i_prof) > rlimit_inc) &
    &  ifail = ibset (ifail, ifail_tcwv_increment)

    !* output (observations with succesful minimisation)
    pt_an  (i_prof,1:nflevg) = zz1d_x    (inumb,         1:  nflevg)
    pq_an  (i_prof,1:nflevg) = zz1d_x    (inumb,  nflevg+1:2*nflevg)      
    pu_10m_an (i_prof)       = zz1d_x (inumb, 2*nflevg+1) ! u10 Analysis
    pv_10m_an (i_prof)       = zz1d_x (inumb, 2*nflevg+2) ! v10 Analysis
    prr_an (i_prof,1:nflevg) = zz1d_x_xt (inumb,2*nflevg+1:3*nflevg)
    prs_an (i_prof,1:nflevg) = zz1d_x_xt (inumb,3*nflevg+1:4*nflevg) 
    prl_an (i_prof,1:nflevg) = zz1d_x_xt (inumb,4*nflevg+1:5*nflevg)
    pri_an (i_prof,1:nflevg) = zz1d_x_xt (inumb,5*nflevg+1:6*nflevg)
    prc_an (i_prof,1:nflevg) = zz1d_x_xt (inumb,6*nflevg+1:7*nflevg)

    prst_an (i_prof) = zz1d_x_xt (inumb,7*nflevg+11)  
    prco_an (i_prof) = zz1d_x_xt (inumb,7*nflevg+12)  

    do jy = 1, i1d_y  
       ptb_an (i_prof,jy) = z1d_y_an (jy)
       ptb_er (i_prof,jy) = onedvar_id(isat)%robs_err (jy)
    enddo

    if (abs (zxj_e) > 1.0E-20_JPRB)  then
        pco (i_prof) = zxj_s / zxj_e
    else
        pco (i_prof) = 1.0E-20_JPRB
    endif

  endif
  
  if (ifail == 0) then

    ! 1dvar analysis diagnostics
    call onedvar_diagnostics ( 1, 1, 1, &  
      & p_sfc(i_prof), &             ! in: pressure @ sfc
      & pt_an(i_prof,:), &           ! in: temperature
      & pq_an(i_prof,:), &           ! in: spec. humidity
      & prr_an(i_prof,:), &          ! in: rain
      & prs_an(i_prof,:), &          ! in: snow
      & prl_an(i_prof,:), &          ! in: cloud water
      & pri_an(i_prof,:), &          ! in: cloud ice
      & p1ddiag(i_prof,m_tcwv_an), & ! out: total column water vapour
      & p1ddiag(i_prof,m_rwp_an), &  ! out: rain water path
      & p1ddiag(i_prof,m_swp_an), &  ! out: snow water path
      & p1ddiag(i_prof,m_cwp_an), &  ! out: cloud water path
      & p1ddiag(i_prof,m_iwp_an), &  ! out: cloud ice water path
      & p1ddiag(i_prof,m_ra_an), &   ! out: surface rain rate
      & p1ddiag(i_prof,m_sn_an)  )   ! out: surface snow rate      

  endif
      
  !* output (all observations that we attempted a retrieval on)           
  kit_dv (i_prof) = i_iter    
  kis_dv (i_prof) = i_simul 
  pepsg (i_prof)  = zeps_g   
  kim_dv (i_prof) = iminim_mode 
  kid_dv (i_prof) = ior(kid_dv(i_prof), ifail) ! Retains any existing flags     

  !----------------------------------------------------------------------------
  ! 4.0 Calculate TCWV observation error
  !----------------------------------------------------------------------------                                 
  if (ifail == 0) then
  
    if (lerain_anerr) then

      !* 1D-Var H-linearity test and linear 1D-Var Analysis Error calculation
   
      if (lerain_lintest) then                
        call onedvar_lintest (isat, i1d_x, i1d_y, i1d_z, inumb , &      
           &  pt        (i_prof,1:nflevg), &   ! in:  temperature
           &  pq        (i_prof,1:nflevg), &   ! in:  spec. humidity
           &  pten_t    (i_prof,1:nflevg), &   ! in:  temperature tendency 
           &  pten_q    (i_prof,1:nflevg), &   ! in:  spec. humidity tendency         
           &  pt_2m     (i_prof)         , &   ! in:  temperature @ 2m
           &  pq_2m     (i_prof)         , &   ! in:  spec. humidity @ 2m
           &  ptd_2m    (i_prof)         , &   ! in:  dew-point temperature @ 2m
           &  pu_star   (i_prof)         , &   ! in:  wind stress, u
           &  pv_star   (i_prof)         , &   ! in:  wind stress, v                             
           &  pu_10m_fg (i_prof)         , &   ! in:  wind speed, u @ 10m
           &  pv_10m_fg (i_prof)         , &   ! in:  wind speed, v @ 10m
           &  psh_sfc   (i_prof)         , &   ! in:  sensible heat flux @ sfc
           &  plh_sfc   (i_prof)         , &   ! in:  latent heat flux @ sfc
           &  psea_ice  (i_prof)         , &   ! in:  sea-ice matrix
           &  pland_sea (i_prof)         , &   ! in:  land-sea matrix
           &  p_sfc     (i_prof)         , &   ! in:  pressure @ sfc
           &  pt_sfc    (i_prof)         , &   ! in:  skin temperature @ sfc   
           &  z1d_x_an(:) - z1d_x_fg(:)  , &   ! in:  perturbation in x 
           &  z1d_y_lin (1:i1d_y))             ! out: 1-d linearity test

        !* Overwrite 3D TB  with linearity test
        ptb_fg_3d (i_prof,1:i1d_y) = z1d_y_lin (1:i1d_y)         
      endif      

      !* Error calculation      
      z1d_ht   (:,:) = 0.0_JPRB
      z1d_hbht (:,:) = 0.0_JPRB

      !* H^T
      do jy = 1, i1d_y
        zz1d_x    (inumb,:) = z1d_x_an    (:)
        zz1d_x_xt (inumb,:) = z1d_x_xt_an (:)
        zz1d_y    (inumb,:) = z1d_y_an    (:) 

        z1d_dj_o (inumb,:)  = 0.0_JPRB
        z1d_dj_o (inumb,jy) = 1.0_JPRB

        call onedvar_obsop_gr (isat, inumb, i1d_x, i1d_y, i1d_z, &
                      &  zz1d_x, zz1d_x_xt, zz1d_y, z1d_dj_o, z1d_dj_ox)

        z1d_ht (1:i1d_x,jy) = z1d_dj_ox (inumb,1:i1d_x) 
      enddo 

      !* B, H^T R^-1 H
      z1d_a_b (:,:) = zz1d_b (:,:)
      z1d_a_o (:,:) = matmul (z1d_ht (:,:), matmul (z1d_r_inv (:,:), transpose (z1d_ht (:,:))))

      !* H B H^T  
      z1d_hbht (:,:) = matmul (transpose (z1d_ht (:,:)), matmul (z1d_a_b (:,:), z1d_ht (:,:)))

      !* Overwrite TB-observation error with H B H^T for ODB output (since R-entries are constant)
      do jy = 1, i1d_y
        ptb_er (i_prof,jy) = z1d_hbht (jy,jy)
      enddo     

      zeps = 100.0_JPRB * tiny (1.0_JPRB)

      !* B^-1
      call matrixin ("CHOLESKI", .false., z1d_a_b (:,:), i1d_x, i1d_x, zeps)

      !* A^-1 = B^-1 + H^T R^-1 H
      z1d_a (:,:) = z1d_a_b (:,:) + z1d_a_o (:,:)

      call matrixin ("CHOLESKI", .false., z1d_a (:,:) , i1d_x, i1d_x, zeps)

      !* TCWV-FG/AN error, (dp^T A dp)^1/2
      z1d_tcwv_b = dot_product (zdp (1:nflevg), matmul (zz1d_b (nflevg+1:2*nflevg,nflevg+1:2*nflevg), zdp (1:nflevg)))
      z1d_tcwv_a = dot_product (zdp (1:nflevg), matmul (z1d_a  (nflevg+1:2*nflevg,nflevg+1:2*nflevg), zdp (1:nflevg)))

      ptcwv_fg_er (i_prof) = sqrt (max (0.0_JPRB, z1d_tcwv_b)) 
      ptcwv_an_er (i_prof) = sqrt (max (0.0_JPRB, z1d_tcwv_a))

    else
    
      ! Parametrised analysis error   
      z1d_tcwv_b = dot_product (zdp (1:nflevg), matmul (zz1d_b (nflevg+1:2*nflevg,nflevg+1:2*nflevg), zdp (1:nflevg)))
      z1d_tcwv_a = ran_err * z1d_tcwv_b

      ptcwv_fg_er (i_prof) = sqrt (max (0.0_JPRB, z1d_tcwv_b))   
      ptcwv_an_er (i_prof) = sqrt (max (0.0_JPRB, z1d_tcwv_a)) 
      
    endif    
  endif
        
  if (lerain_adjtest) then 

    ! Optional adjoint tests
    z1d_x_xt(1:nflevg) = pten_t(i_prof,:)  ! reset tendencies to original values
    z1d_x_xt(nflevg+1:2*nflevg) = pten_q(i_prof,:) 
    z1d_x_tl = 0.0_jprb
    z1d_xt_tl = 0.0_jprb
    z1d_x_tl(2*nflevg+1:2*nflevg+2) = z1d_x_sigb(2*nflevg+1:2*nflevg+2)  ! Wind
    z1d_x_tl(1:nflevg) = z1d_x_sigb(1:nflevg) * 0.1_jprb  ! T
    z1d_x_tl(nflevg+1:2*nflevg) = -z1d_x_sigb(nflevg+1:2*nflevg) * 0.01_jprb ! Q

    call onedvar_adjoint_test(isat, i1d_x, i1d_y, i1d_z, &
      & z1d_x_fg, z1d_x_xt, z1d_x_tl, z1d_xt_tl)

  endif                        

enddo  ! loop over profiles

if (llxz_allocated) then
  deallocate (zxn, zdj, zrz, iz)    
  deallocate (zz1d_x_fg, z1d_x_fg, z1d_x_sigb, z1d_x_an, z1d_x_xt, z1d_x_tl)
  deallocate (zz1d_x, zz1d_x_xt, zwork, z1d_xt_tl)
  deallocate (z1d_dj_ox, z1d_a_b, z1d_a_o, z1d_a, zz1d_b, z1d_x_xt_an)
  deallocate (z1d_b, z1d_b_packed, z1d_b_eval, z1d_b_evec, zz1d_b_eval, zz1d_b_evec)
  deallocate (zidentity)
  llxz_allocated = .FALSE.
endif            
if (lly_allocated) then      
  deallocate (z1d_y, z1d_y_an, z1d_y_fg, z1d_y_lin, z1d_r, z1d_r_inv, zz1d_y, z1d_hbht, z1d_dj_o)
  lly_allocated = .FALSE.       
endif            
if (llxy_allocated) then         
  deallocate(z1d_ht)
  llxy_allocated = .FALSE.       
endif
  
if (lhook) call dr_hook('ONEDVAR_RAINTB',1,zhook_handle)
end subroutine onedvar_raintb
