SUBROUTINE ONEDVAR_RAINTB_SND( kstart, kend, kassim_mode, KCOPROC, KT, &
 &      pt         , &   ! in:  3-d temperature
 &      pq         , &   ! in:  3-d spec. humidity
 &      pten_t     , &   ! in:  3-d temperature tendency 
 &      pten_q     , &   ! in:  3-d spec. humidity tendency         
 &      prain      , &   ! in:  3-d liquid precipitation
 &      psnow      , &   ! in:  3-d solid precipitation       
 &      pclfr      , &   ! in:  3-d cloud fraction    
 &      pcllw      , &   ! in:  3-d cloud liquid water content   
 &      pcliw      , &   ! in:  3-d cloud ice water content              
 &      pt_2m      , &   ! in:  2-d temperature @ 2m
 &      pq_2m      , &   ! in:  2-d spec. humidity @ 2m
 &      ptd_2m     , &   ! in:  2-d dew-point temperature @ 2m
 &      pu_star    , &   ! in:  2-d wind stress, u
 &      pv_star    , &   ! in:  2-d wind stress, v                             
 &      pu_10m     , &   ! in:  2-d wind speed, u @ 10m
 &      pv_10m     , &   ! in:  2-d wind speed, v @ 10m
 &      psh_sfc    , &   ! in:  2-d sensible heat flux @ sfc
 &      plh_sfc    , &   ! in:  2-d latent heat flux @ sfc
 &      psea_ice   , &   ! in:  2-d sea-ice matrix
 &      pland_sea  , &   ! in:  2-d land-sea matrix
 &      p_sfc      , &   ! in:  2-d pressure @ sfc
 &      pt_sfc     , &   ! in:  2-d skin temperature @ sfc   
! Observations:
 &      ptb_obs    , &   ! in:  3-d radiance      
 &      pslat_obs  , &   ! in:  2-d sin latitude
 &      pclon_obs  , &   ! in:  2-d cos longitude
 &      pslon_obs  , &   ! in:  2-d sin longitude
 &      ksat_obs   , &   ! in:  2-d satellite ID 
 &      kfov_obs  , &    ! in:  2-d IFOV number  
 &      pzen_obs     )   ! in:  2-d zenith angle

USE PARKIND1  , ONLY : JPIM     ,JPRB
USE YOMDIM    , ONLY : NFLEVG   ,NPROMA   
USE PARONEDVAR, ONLY : NMAX_CHANNELS
USE MPL_MODULE, ONLY : MPL_SEND, JP_BLOCKING_STANDARD
USE YOMHOOK   , ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

integer (kind=jpim), intent (in) :: kstart, kend, kassim_mode, KCOPROC, KT
real (kind=jprb), intent (in), dimension (nproma,nflevg) ::    pt      , &
                                                            &  pq      , &
                                                            &  pten_t  , &
                                                            &  pten_q  , &
                                                            &  prain   , &
                                                            &  psnow   , &
                                                            &  pclfr   , &
                                                            &  pcllw   , &
                                                            &  pcliw
real (kind=jprb), intent (in), dimension (nproma)       ::    pt_2m    , &
                                                            & pq_2m    , &
                                                            & ptd_2m   , &
                                                            & pu_star  , &
                                                            & pv_star  , &
                                                            & pu_10m   , &
                                                            & pv_10m   , &
                                                            & psh_sfc  , &
                                                            & plh_sfc  , &
                                                            & psea_ice , &
                                                            & pland_sea, &
                                                            & p_sfc    , &
                                                            & pt_sfc
real (kind=jprb), intent (in), dimension (nproma,NMAX_CHANNELS) :: ptb_obs
real (kind=jprb), intent (in), dimension (nproma)       ::    pslat_obs , &
                                                            & pslon_obs , & 
                                                            & pclon_obs , & 
                                                            & pzen_obs 

integer (kind=jpim), intent (in), dimension (nproma) :: ksat_obs, kfov_obs

INTEGER (KIND=JPIM) :: ISTJ, IEND, ILEN, ILEV, IOFF, ICH
REAL(KIND=JPRB)     :: ZSBF(4+9*NPROMA*NFLEVG+19*NPROMA+NPROMA*NMAX_CHANNELS)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('ONEDVAR_RAINTB_SND',0,ZHOOK_HANDLE)
    ISTJ=kstart
    IEND=kend
    ILEN =IEND-ISTJ
    ZSBF(1)=0.0_JPRB
    ZSBF(2)=ISTJ+0.1_JPRB
    ZSBF(3)=IEND+0.1_JPRB
    ZSBF(4)=kassim_mode+0.1_JPRB
    IOFF=5
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pt(ISTJ:IEND,ILEV)  !TEMPERATURE
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pq(ISTJ:IEND,ILEV)  !HUMIDITY
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pten_t(ISTJ:IEND,ILEV)     !TENDENCY BEFORE CONVECTION
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pten_q(ISTJ:IEND,ILEV)     !TENDENCY BEFORE CONVECTION
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=prain(ISTJ:IEND,ILEV)  !PRECIPITATION AS RAIN ZFPLCL+ZFPLSL
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=psnow(ISTJ:IEND,ILEV)  !PRECIPITATION AS SNOW ZFPLCN+ZFPLSN
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pclfr(ISTJ:IEND,ILEV)  !CLOUD FRACTION 
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pcllw(ISTJ:IEND,ILEV)  !CLOUD WATER 
      IOFF=IOFF+NPROMA
    ENDDO
    DO ILEV=1,NFLEVG
      ZSBF(IOFF:IOFF+ILEN)=pcliw(ISTJ:IEND,ILEV)  !CLOUD ICE
      IOFF=IOFF+NPROMA
    ENDDO
    ZSBF(IOFF:IOFF+ILEN)=pt_2m(ISTJ:IEND)     !TEMPERATURE AT 2 METERS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pq_2m(ISTJ:IEND)         !SPECIFIC HUMIDITY AT 2 METERS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=ptd_2m(ISTJ:IEND)     !DEW POINT TEMPERATURE AT 2 METERS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pu_star(ISTJ:IEND)        !GRAVITY WAVE DRAG SURFACE U-STRESS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pv_star(ISTJ:IEND)        !GRAVITY WAVE DRAG SURFACE V-STRESS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pu_10m(ISTJ:IEND) !U-COMPONENT OF WIND AT 10 METERSESS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pv_10m(ISTJ:IEND)      !V-COMPONENT OF WIND AT 10 METERS
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=psh_sfc(ISTJ:IEND)     !SURFACE SENSIBLE HEAT FLUX
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=plh_sfc(ISTJ:IEND)     !SURFACE MOISTC FLUX
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=psea_ice(ISTJ:IEND)    !SEA-ICE MASK
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pland_sea(ISTJ:IEND)   !LAND/SEA MASK
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=p_sfc(ISTJ:IEND)  !PRESSURE AT SURFACE
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pt_sfc(ISTJ:IEND) !SKIN TEMPERATUREACE
    IOFF=IOFF+NPROMA
    DO ICH=1,NMAX_CHANNELS
      ZSBF(IOFF:IOFF+ILEN)=ptb_obs(ISTJ:IEND,ICH) !OBSERVATIONS
      IOFF=IOFF+NPROMA
    ENDDO
    ZSBF(IOFF:IOFF+ILEN)=pslat_obs(ISTJ:IEND)          !SINE OF LATITUDE
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pclon_obs(ISTJ:IEND)         !COSINE OF LONGITUDE
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pslon_obs(ISTJ:IEND)         !SINE OF LONGITUDE
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=ksat_obs(ISTJ:IEND) 
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=kfov_obs(ISTJ:IEND)
    IOFF=IOFF+NPROMA
    ZSBF(IOFF:IOFF+ILEN)=pzen_obs(ISTJ:IEND)          !ZENITH ANGLE? 
    
    CALL MPL_SEND(ZSBF,KTAG=KT,KDEST=KCOPROC,&
      &KMP_TYPE=JP_BLOCKING_STANDARD,CDSTRING='ONEDVAR_RAINTB_SND:')
      
IF (LHOOK) CALL DR_HOOK('ONEDVAR_RAINTB_SND',1,ZHOOK_HANDLE)
END SUBROUTINE ONEDVAR_RAINTB_SND
