subroutine onedvar_read_sat_bias( ydone_id, &  ! inout
                                & kfile_id )  ! in


! This IFS subroutine will read in the biases for the 1dvar for a specific sensor.

! ONEDVAR_READ_SAT_BIAS is called by ONEDVAR_SETUP

! Original Code: 13/10/2006  Chris O'Dell

! Modifications:


!**** Modules
use parkind1  , only: jpim, jprb
use yomhook   , only: lhook, dr_hook
use yomlun    , only: nulerr
use yomonedvar, only: onedvar_ids
use paronedvar, only: npred_bias

implicit none

! In/Out Variables
type(onedvar_ids), intent(inout)  :: ydone_id  ! struct containing sat id's for a single sensor
Integer(Kind=jpim), Optional, Intent(in) :: kfile_id  ! file logical unit number

! local variables
Character (len=256):: clbiasname  ! file name for the bias file
integer(kind=jpim) :: ioerr, ifileid, instrument(3), ichan
real(kind=jprb)    :: zhook_handle

!#include "onedvar_get_file_name.intfb.h"

if (lhook) call dr_hook ('ONEDVAR_READ_SAT_BIAS',0,zhook_handle)

! 1. Construct file name based on id information. Use RTTOV method as a template.
! 2. Open file for reading.  Error if doesn't open correctly.
! 3. Read biases & place in ydone_id struct.
! 4. Close file.

ifileid = 10
if (present(kfile_id)) ifileid = kfile_id

!* Construct bias file name.
instrument(1) = ydone_id % satseries
instrument(2) = ydone_id % satid 
instrument(3) = ydone_id % sensid
call onedvar_get_file_name( ioerr, instrument, clbiasname, .TRUE. )
If ( ioerr /= 0 ) Then
  write (nulerr,*) ' Error in constructing Satellite Bias File Name!'
  IF (LHOOK) CALL DR_HOOK('ONEDVAR_READ_SAT_BIAS',1,ZHOOK_HANDLE)
  Return
Endif

!* Open bias file
ioerr = 0
open (ifileid, file = trim(clbiasname), status = 'old', iostat = ioerr)
if (ioerr /= 0) then
  write (nulerr,*) ' No SSM/I bias correction file ' // trim(clbiasname) // ' found!'
  if (lhook) call dr_hook ('ONEDVAR_READ_SAT_BIAS',1,zhook_handle)
  return
endif

!* Read bias correction coefficients 
read (ifileid,*) ! read header line
do ichan = 1, ydone_id % nchannels
  read (ifileid,*) ydone_id % robs_bias(ichan, 0:npred_bias)
enddo
close (ifileid)

if (lhook) call dr_hook ('ONEDVAR_READ_SAT_BIAS',1,zhook_handle)

end subroutine onedvar_read_sat_bias
