SUBROUTINE AEOLUS_L2B_TO_BODY(KLEN) ! Placeholder for future development
!  #[ 

!     Purpose.
!     --------
!       Transfer Aeolus L2B retrievals from ODB-table aeolus_l2b to body table.

!     Author.
!     -------
!        David Tan *ECMWF*

!     Modifications.
!     --------------
!     2007-Mar-23   David Tan         Original developed in other source files
!     2007-May-30   David Tan         Lower rejection flag
!     2007-Jun-01   David Tan         Coding norm compliance
!     2007-Jun-12   David Tan         Set vertco_type@hdr
!     2007-Sep-25   David Tan         Make zhook_handle local to each routine
!     2007-Nov-04   T.Wilhelmsson
!                   O.Spaniel       removing of severe breaks of coding norms
!     2007-Dec-05   David Tan         Aeolus_l2bp_wrapper: new looping/indexing
!     2008-Jan-24   David Tan         New dummy L2B retrievals
!     2008-Jan-31   David Tan         Use aeolus_hdr.offset@sat
!     2008-Feb-14   David Tan         Suspend use of aeolus offsets.
!     2008-Mar-06   David Tan         Import from module/aeolus_l2bp_wrapper_mod.F90.
!     2008-Mar-26   David Tan         Allow for hlos_method=0 in aeolus_2b table.


!     ------------------------------------------------------------------
USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE YOMNMCOD , ONLY : NHEIGVC
USE YOMLUN   , ONLY : NULOUT
USE YOMDB

!     -------------------------------------------------------------------------
! Type and variable definitions
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 

! local variables           

REAL(KIND=JPRB)    :: ZHOOK_HANDLE
INTEGER(KIND=JPIM) :: imax_num_l2b_obs_classes ! TODO: make this a parameter?
INTEGER(KIND=JPIM) :: istep_l2b_obs_classes    ! TODO: make this a parameter?
INTEGER(KIND=JPIM) :: JOBS, i_l2b     ! loops
INTEGER(KIND=JPIM) :: i_l2b_channel    ! loops
INTEGER(KIND=JPIM) :: i_l2b_rangebin   ! loops
INTEGER(KIND=JPIM) :: i_aeolus_2b ! indices for ODB arrays
INTEGER(KIND=JPIM) :: i_sonde_type_at_hdr ! For sanity checks

! Other local variables
INTEGER(KIND=JPIM), PARAMETER :: JP_NUM_AEOLUS_RANGEBINS=24
INTEGER(KIND=JPIM), PARAMETER :: JP_SONDE_TYPE_LAST_L1B=100+JP_NUM_AEOLUS_RANGEBINS


! Aeolus variables
! None yet.

#include "fcobs.h"
#include "openmp_obs.h"

  !  #[
       
  ! 1.0 Preliminaries

  IF (LHOOK) CALL DR_HOOK('AEOLUS_L2B_TO_BODY',0,ZHOOK_HANDLE)

  imax_num_l2b_obs_classes = 5
  istep_l2b_obs_classes    = imax_num_l2b_obs_classes + 1 ! allows for L1B rows (hlos_method=0)

  !  #]

  !  #[
       
  ! 2.0 Transfer from ODB-table "aeolus_2b" to ODB-table "body"
       
    WRITE (NULOUT,*) ' aeolus_l2b_to_body: size(SATBODY(:,1)) = ', size(SATBODY(:,1))
    WRITE (NULOUT,*) ' aeolus_l2b_to_body: size(SATBODY(1,:)) = ', size(SATBODY(1,:))
    WRITE (NULOUT,*) ' aeolus_l2b_to_body: size(SATBODY     ) = ', size(SATBODY)
    CALL FLUSH(NULOUT)
      WRITE (NULOUT,*) 'dt-debug: MLNK_AEOLUS_HDR2AEOLUS_2B(1:2) = ',&
                      &           MLNK_AEOLUS_HDR2AEOLUS_2B(1:2)
      CALL FLUSH(NULOUT)
    OBS_LOOP2: DO JOBS = 1, klen
      ! Transfer values from aeolus_2b table to hdr and body.
      ! Used in assimilation.

      IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) <= JP_SONDE_TYPE_LAST_L1B ) CYCLE OBS_LOOP2

         i_sonde_type_at_hdr     = ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR)
         i_l2b_channel           = mod(i_sonde_type_at_hdr,10000) / 100 ! Integer division ! TODO: simplify?
         i_l2b_rangebin          = mod(i_sonde_type_at_hdr,100)         ! TODO: simplify?
         i_l2b                   = i_sonde_type_at_hdr/10000            ! Integer division! L2B-class

         i_aeolus_2b = JOBS ! Indexing is one-to-one when suitable sql employed.

         if (      ( i_l2b_channel  /= SATBODY(i_aeolus_2b, MDB_CHANNEL_AT_AEOLUS_2B)      ) &
            & .or. ( i_l2b_rangebin /= SATBODY(i_aeolus_2b, MDB_BIN2B_AT_AEOLUS_2B)        ) &
            & .or. ( i_l2b          /= SATBODY(i_aeolus_2b, MDB_HLOS_METHOD_AT_AEOLUS_2B)  ) ) then
            WRITE (NULOUT,*) 'aeolus_l2b_to_body: WARNING: INDEX CONFLICT,'//&
                            &' AEOLUS_2B RETRIEVALS INCORRECTLY TRANSFERRED TO BODY TABLE!!'
            WRITE (NULOUT,*) 'aeolus_l2b_to_body: i_aeolus_2b, JOBS = ',&
                            & i_aeolus_2b, JOBS
            WRITE (NULOUT,*) 'aeolus_l2b_to_body: channel_2b  current/pre-filled = ',&
                            & i_l2b_channel,  SATBODY(i_aeolus_2b, MDB_CHANNEL_AT_AEOLUS_2B )
            WRITE (NULOUT,*) 'aeolus_l2b_to_body: rangebin2b  current/pre-filled = ',&
                            & i_l2b_rangebin, SATBODY(i_aeolus_2b, MDB_BIN2B_AT_AEOLUS_2B )
            WRITE (NULOUT,*) 'aeolus_l2b_to_body: l2b_method current/pre-filled = ',&
                            & i_l2b,          SATBODY(i_aeolus_2b, MDB_HLOS_METHOD_AT_AEOLUS_2B )
            WRITE (NULOUT,*) 'aeolus_l2b_to_body: CYCLING OBS_LOOP2... ' ! Debug
            CALL FLUSH(NULOUT)
            CYCLE OBS_LOOP2
         else
            WRITE(NULOUT,*) 'dt-debug2: l2b-body/aeolus_2b table entries match correctly'
            CALL FLUSH(NULOUT)
         endif

         !ROBHDR(JOBS,MDBLAT) = SATBODY(i_aeolus_2b, MDB_COG_LAT_AT_AEOLUS_2B ) ! Latitude
         !ROBHDR(JOBS,MDBLON) = SATBODY(i_aeolus_2b, MDB_COG_LON_AT_AEOLUS_2B ) ! Longitude
         ! TODO: Make ROBHDR writeable if above changes are needed in ODB.

         ! ROBODY(JOBS,MDBVNM) = 187 ! = NVNUMB(77) ! Variable Number already set.      ! Was ROBODY(i_odb_l1b,MDBVNM)
         ROBODY(JOBS,MDBVCO) = NHEIGVC ! Vertical Coordinate.  TODO: Check not NPRESVC? ! Was ROBODY(i_odb_l1b,MDBVCO) 
         ROBODY(JOBS,MDBPPP) = SATBODY(i_aeolus_2b, MDB_COG_ZTOP_AT_AEOLUS_2B ) ! TODO: clarify Pressure or Altitude, and averaging
                                                                                ! TODO: factor of g
         ROBODY(JOBS,MDBVAR) = SATBODY(i_aeolus_2b, MDB_HLOS_AT_AEOLUS_2B )     ! Obs Value
         ROBODY(JOBS,MDBOER) = SATBODY(i_aeolus_2b, MDB_HLOS_ERR_AT_AEOLUS_2B ) ! Obs Error
         ROBODY(JOBS,MDBFOE) = 25._JPRB  ! Placeholder ! Final Obs Error
                                              ! PersistenceErr is 20. for now!

         ! Datum previously rejected by dwlin because of missing ObsValue (or pressure)
         ! Lower the rejected flag now that ObsValue is set
         ROBODY(JOBS,MDBDSTA) = ZCHSTAT_UNREJECT( ROBODY(JOBS,MDBDSTA) )

    ENDDO OBS_LOOP2 ! JOBS

    ! Trash varno for HLoS observations - test
    OBS_LOOP3: DO JOBS = 1, klen

      ! Reprieve for L2B observations
      IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) > JP_SONDE_TYPE_LAST_L1B ) CYCLE OBS_LOOP3

      ! Reprieve for L1B observations
      !IF( ROBHDR(JOBS,MDB_SONDE_TYPE_AT_HDR) <= JP_SONDE_TYPE_LAST_L1B ) CYCLE OBS_LOOP3

      ROBODY(JOBS,MDBVNM) = -999 ! 174 is AOD

    ENDDO OBS_LOOP3 ! JOBS

  !  #]

  !  #[
       
  ! 3.0 Closing

  IF (LHOOK) CALL DR_HOOK('AEOLUS_L2B_TO_BODY',1,ZHOOK_HANDLE)

  !  #]
END SUBROUTINE AEOLUS_L2B_TO_BODY
