SUBROUTINE ATMREF_GEMS(PTAMOY,PTRAY,PIZMOY,PHIRAD,PXMUS,PXMUV,PRM,PGB,PRP, &
                     & PBETAL,PPSL,PTODAER,PTODRAY,PROMIX)

!****ATMREF*** COMPUTES AEROSOL, RAYLEIGH AND MIXTURE REFLECTANCE


!     INTERFACE
!     ---------
! ATMREF IS CALLED BY DISCOM

!     INPUTS
!     ------

! PHIRAD          : DIFFERENCE IN AZIMUTAL ANGLE [RAD]
! PXMUS           : COSINE OF SOLAR ZENITH ANGLE
! PXMUV           : COSINE OF VIEWING ZENITH ANGLE
! PRP(KNP)        : GAUSSIAN ANGLES FOR AZIMUTH
! PRM(-KMU:KMU) : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU) : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAMOY(KNT)     : TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
! PTRAY(KNT)      : MOLECULAR OPTICAL THICKNESS PROFILE.
! PIZMOY(KNT)     : TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM
! PTODAER               : TOTAL AEROSOL OPTICAL THICKNESS
! PTODRAY               : TOTAL MOLECULAR OPTICAL THICKNESS

!     OUTPUTS
!     -------
! PROMIX  : ATMOSPHERIC REFLECTANCE OF MIXTURE

!     EXTERNALS.
!     ----------
!          OS

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007
!          A.BENEDETTI - ECMWF- 06.08.2008

!     SOURCE
!     ------
! ADAPTED FROM AMTREF.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE RT6SVALUES, ONLY : KMU, KNP, KNT, KNANG 

IMPLICIT NONE

!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
REAL(KIND=JPRB),INTENT(IN)    :: PGB(-KMU:KMU), PRP(KNP)
REAL(KIND=JPRB),INTENT(IN)    :: PHIRAD, PXMUS, PXMUV   
REAL(KIND=JPRB),INTENT(IN)    :: PPSL(0:KNANG,-1:KNANG,-KMU:KMU)          
REAL(KIND=JPRB),INTENT(IN)    :: PTAMOY(0:KNT), PTRAY(0:KNT), PIZMOY(0:KNT)
REAL(KIND=JPRB),INTENT(IN)    :: PBETAL(0:KNT,0:KNANG), PTODAER, PTODRAY
REAL(KIND=JPRB),INTENT(OUT)   :: PROMIX
REAL(KIND=JPRB),INTENT(INOUT) :: PRM(-KMU:KMU)

! LOCAL VARIABLES
! ---------------

REAL(KIND=JPRB) :: Z_PXLM2(-KMU:KMU,KNP)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "os_gems.intfb.h"


IF (LHOOK) CALL DR_HOOK('ATMREF_GEMS',0,ZHOOK_HANDLE)

! MIXTURE REFLECTANCE

CALL OS_GEMS(PTAMOY,PTRAY,PIZMOY,PHIRAD,PBETAL,PRM,PGB,PRP,PPSL,PTODAER,PTODRAY,   &
&                                                Z_PXLM2)
 
PROMIX=(Z_PXLM2(KMU,1)/PXMUS)

IF (LHOOK) CALL DR_HOOK('ATMREF_GEMS',1,ZHOOK_HANDLE)

END SUBROUTINE ATMREF_GEMS
