!  Differentiation of atmref_gems in reverse (adjoint) mode:
!   gradient, with respect to input variables: pizmoy pbetal ptamoy
!   of linear combination of output variables: pizmoy promix pbetal ptamoy

SUBROUTINE ATMREF_GEMS_AD(ptamoy, ptamoyb, ptray, pizmoy, pizmoyb, phirad, &
&  pxmus, pxmuv, prm, pgb, prp, pbetal, pbetalb, ppsl, ptodaer, &
&  ptodray, promix, promixb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues, ONLY : kmu, knp, knt, knang

IMPLICIT NONE

!****ATMREF*** COMPUTES AEROSOL, RAYLEIGH AND MIXTURE REFLECTANCE


!     INTERFACE
!     ---------
! ATMREF IS CALLED BY DISCOM

!     INPUTS
!     ------

! PHIRAD          : DIFFERENCE IN AZIMUTAL ANGLE [RAD]
! PXMUS           : COSINE OF SOLAR ZENITH ANGLE
! PXMUV           : COSINE OF VIEWING ZENITH ANGLE
! PRP(KNP)        : GAUSSIAN ANGLES FOR AZIMUTH
! PRM(-KMU:KMU) : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU) : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAMOY(KNT)     : TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
! PTRAY(KNT)      : MOLECULAR OPTICAL THICKNESS PROFILE.
! PIZMOY(KNT)     : TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM
! PTODAER               : TOTAL AEROSOL OPTICAL THICKNESS
! PTODRAY               : TOTAL MOLECULAR OPTICAL THICKNESS

!     OUTPUTS
!     -------
! PROMIX  : ATMOSPHERIC REFLECTANCE OF MIXTURE

!     EXTERNALS.
!     ----------
!          OS

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM AMTREF.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
  REAL(KIND=JPRB), INTENT(IN) :: pgb(-kmu:kmu), prp(knp)
  REAL(KIND=JPRB), INTENT(IN) :: phirad, pxmus, pxmuv
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: ptamoy(0:knt), ptray(0:knt), pizmoy(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: ptamoyb(0:knt), pizmoyb(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: pbetal(0:knt, 0:knang), ptodaer, ptodray
  REAL(KIND=JPRB), INTENT(INOUT) :: pbetalb(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(IN) :: promix
  REAL(KIND=JPRB), INTENT(INOUT) :: promixb
  REAL(KIND=JPRB), INTENT(INOUT) :: prm(-kmu:kmu)
! LOCAL VARIABLES
! ---------------
  REAL(KIND=JPRB) :: z_pxlm2(-kmu:kmu, knp)
  REAL(KIND=JPRB) :: z_pxlm2b(-kmu:kmu, knp)

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "os_gems_ad.intfb.h"

  IF (LHOOK) CALL DR_HOOK('ATMREF_GEMS_AD',0,ZHOOK_HANDLE)

! MIXTURE REFLECTANCE

  z_pxlm2b(-kmu:kmu, 1:knp) = 0._JPRB
  z_pxlm2b(kmu, 1) = promixb/pxmus
  CALL OS_GEMS_AD(ptamoy, ptamoyb, ptray, pizmoy, pizmoyb, phirad, pbetal, &
&           pbetalb, prm, pgb, prp, ppsl, ptodaer, ptodray, z_pxlm2, &
&           z_pxlm2b)

 IF (LHOOK) CALL DR_HOOK('ATMREF_GEMS_AD',1,ZHOOK_HANDLE)
END SUBROUTINE ATMREF_GEMS_AD
