SUBROUTINE CH4_TCMR_TL &
 & ( KDLEN  , KLEN  , KDCOUNT, KCOUNT, & 
 &   PPRESH, PPRESF, &
 &   YDGEMSPROF, &  
 &   PPRESH5, PPRESF5, PSOLEL, &
 &   YDGEMSPROF5, &
 &   PCH4MEAN)

!*** *CH4_TCMR_TL* Tangent Linear Operator routine for column averaged CH4 observations.

!**   INTERFACE.
!     ----------
!          *CH4_TCMR_TL* IS CALLED FROM *HOPTL*.
!     CALL CH4_TCMR_TL(KDLEN,ILEN,IMXCOUNT,ICOUNT, ZPRESH, ZPRESF, YLGEMSPROF,
!                  ZPRESH5, ZPRESF5, ZSOLEL, YLGEMSPROF5, ZXPP)
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol optical depth obs (INPUT)
!           IMXCOUNT   = Number of channels (INPUT)
!           ZPRESH,ZPRESF = Perturb in Half/Full level pressure values at obs points, model levels (INPUT)
!           YLGEMSPROF     =  Perturb in GEMS species, including aerosol mixing ratios (INPUT)
!           ZPRESH5,ZPRESF5 =   Half/Full level pressure values at obs points, model levels (INPUT)
!           YLZGEMSPROF5     = GEMS species including aerosol mixing ratios (INPUT)

!           ZXPP       = Column averaged CH4 mixing ratio (OUTPUT)



!**   AUTHOR.
!     -------
!        RICHARD ENGELEN        *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2007-01-23

!     PURPOSE.
!     --------
!     - derive tangent linear column averaged methan mixing ratio from profiles of mass mixing ratio 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_YGFL , ONLY : NGHG
USE YOMDIM, ONLY : NFLEVG
USE YOMDB
USE YOMGRB

USE YOMCST    ,ONLY : RG, RMD, RMCO2, RMCH4

USE TYPE_GEMS_PROFILES

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KDCOUNT
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH5(KDLEN,0:NFLEVG) ! PRESSURE ON HALF LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF5(KDLEN,NFLEVG)   ! PRESSURE ON FULL LEVELS FROM TRAJ

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) ! perturbation on PRESSURE ON HALF LEVELS
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF(KDLEN,NFLEVG) !  " "    PRESSURE ON FULL LEVELS
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOLEL(KDLEN)          ! SOLAR ELEVATION

TYPE (TYPE_GEMSPROF)   ,INTENT(INOUT) :: YDGEMSPROF(KDLEN,NGEMS)
TYPE (TYPE_GEMSPROF)   ,INTENT(IN)    :: YDGEMSPROF5(KDLEN,NGEMS)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PCH4MEAN(KDLEN,KDCOUNT) ! AEROSOL OPT DEPTH PERTURBATION 

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JK, JL, JJ, JF, JLEN, J, INDEX

REAL(KIND=JPRB), DIMENSION(KLEN,NFLEVG)   :: ZCO2MR5, ZCO2MR, ZCH4MR5, ZCH4MR
REAL(KIND=JPRB)    :: ZDP5, ZDPSUM5
REAL(KIND=JPRB)    :: ZMMR2VMRCO2, ZMMR2VMRCH4
REAL(KIND=JPRB), DIMENSION(60) :: ZAK_PRES, ZCH4_PRIOR
REAL(KIND=JPRB), DIMENSION(600) :: ZDUM
REAL(KIND=JPRB), DIMENSION(10,60) :: ZAK
REAL(KIND=JPRB), DIMENSION(10) :: ZAK_SELEL
REAL(KIND=JPRB), DIMENSION(NFLEVG) :: ZAK_MODEL, ZCH4_PRIOR_M
REAL(KIND=JPRB) :: ZCO2SUM5, ZCO2MEAN5, ZCH4SUM5, ZCH4SUM, ZCH4MEAN

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "intavg.h"

DATA ZAK_PRES / &
 &     0.30,    0.38,    0.50,    0.65,    0.80,    1.03,    1.33, &
 &     1.72,    2.22,    2.87,    3.81,    5.06,    6.75,    9.00, &
 &    11.97,   14.15,   16.34,   19.04,   22.27,   25.49,   29.72, &
 &    34.67,   40.47,   47.29,   55.29,   64.67,   75.65,   88.50, &
 &   103.50,  121.10,  141.70,  165.80,  194.00,  227.00,  265.00, &
 &   308.00,  356.50,  411.00,  441.60,  472.20,  506.35,  540.50, &
 &   578.55,  616.60,  637.75,  658.90,  680.05,  701.20,  724.65, &
 &   748.10,  771.55,  795.00,  820.95,  846.90,  872.85,  898.80, &
 &   927.35,  955.90,  984.45, 1013.00 /

DATA ZCH4_PRIOR / &
 & 1.6060E-07, 1.6680E-07, 1.7916E-07, 1.9768E-07, 2.1620E-07, 2.7164E-07, 3.3848E-07, &
 & 4.1428E-07, 4.9608E-07, 5.8040E-07, 6.6112E-07, 7.3326E-07, 8.0250E-07, 8.7162E-07, &
 & 9.4050E-07, 9.7070E-07, 1.0009E-06, 1.0300E-06, 1.0580E-06, 1.0860E-06, 1.1510E-06, &
 & 1.2260E-06, 1.3090E-06, 1.3950E-06, 1.4660E-06, 1.5240E-06, 1.5660E-06, 1.5990E-06, &
 & 1.6290E-06, 1.6520E-06, 1.6740E-06, 1.6930E-06, 1.7110E-06, 1.7240E-06, 1.7350E-06, &
 & 1.7430E-06, 1.7470E-06, 1.7490E-06, 1.7495E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, &
 & 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, &
 & 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06, &
 & 1.7500E-06, 1.7500E-06, 1.7500E-06, 1.7500E-06 /

DATA ZAK_SELEL / &
 &  70.0, 60.0, 50.0, 40.0, 30.0, 25.0, 20.0, 15.0, 10.0, 5.0 /
DATA ZDUM / &
 &  0.479, 0.471, 0.458, 0.438, 0.407, 0.386, 0.359, 0.324, 0.280, 0.223, &
 &  0.495, 0.487, 0.474, 0.453, 0.422, 0.400, 0.372, 0.336, 0.290, 0.229, &
 &  0.485, 0.477, 0.464, 0.445, 0.414, 0.393, 0.365, 0.330, 0.285, 0.225, &
 &  0.483, 0.475, 0.462, 0.443, 0.413, 0.392, 0.365, 0.330, 0.285, 0.225, &
 &  0.521, 0.512, 0.499, 0.479, 0.446, 0.424, 0.395, 0.357, 0.308, 0.244, &
 &  0.512, 0.504, 0.491, 0.470, 0.439, 0.417, 0.389, 0.352, 0.305, 0.243, &
 &  0.503, 0.495, 0.482, 0.463, 0.432, 0.410, 0.383, 0.347, 0.302, 0.243, &
 &  0.501, 0.493, 0.480, 0.460, 0.430, 0.408, 0.381, 0.346, 0.303, 0.248, &
 &  0.506, 0.498, 0.485, 0.465, 0.434, 0.412, 0.385, 0.351, 0.308, 0.256, &
 &  0.519, 0.510, 0.497, 0.476, 0.445, 0.423, 0.395, 0.360, 0.319, 0.268, &
 &  0.510, 0.502, 0.489, 0.469, 0.438, 0.416, 0.389, 0.357, 0.317, 0.272, &
 &  0.508, 0.500, 0.487, 0.467, 0.436, 0.415, 0.389, 0.357, 0.321, 0.280, &
 &  0.512, 0.503, 0.490, 0.471, 0.440, 0.419, 0.394, 0.363, 0.328, 0.291, &
 &  0.525, 0.517, 0.503, 0.483, 0.453, 0.432, 0.406, 0.375, 0.341, 0.307, &
 &  0.544, 0.536, 0.522, 0.502, 0.471, 0.449, 0.423, 0.392, 0.358, 0.325, &
 &  0.534, 0.526, 0.513, 0.493, 0.463, 0.442, 0.417, 0.387, 0.354, 0.324, &
 &  0.545, 0.536, 0.523, 0.503, 0.473, 0.452, 0.426, 0.396, 0.364, 0.335, &
 &  0.549, 0.541, 0.528, 0.508, 0.477, 0.457, 0.431, 0.402, 0.370, 0.342, &
 &  0.553, 0.544, 0.531, 0.512, 0.482, 0.461, 0.436, 0.407, 0.375, 0.348, &
 &  0.574, 0.566, 0.552, 0.532, 0.502, 0.481, 0.455, 0.425, 0.393, 0.367, &
 &  0.584, 0.575, 0.562, 0.542, 0.512, 0.491, 0.465, 0.435, 0.403, 0.378, &
 &  0.595, 0.587, 0.574, 0.554, 0.523, 0.503, 0.477, 0.447, 0.415, 0.390, &
 &  0.609, 0.600, 0.587, 0.568, 0.537, 0.517, 0.491, 0.461, 0.429, 0.404, &
 &  0.625, 0.617, 0.604, 0.584, 0.554, 0.533, 0.508, 0.478, 0.446, 0.420, &
 &  0.644, 0.636, 0.623, 0.603, 0.573, 0.553, 0.527, 0.497, 0.465, 0.439, &
 &  0.667, 0.659, 0.646, 0.626, 0.596, 0.576, 0.550, 0.520, 0.487, 0.460, &
 &  0.693, 0.685, 0.673, 0.653, 0.624, 0.603, 0.577, 0.547, 0.512, 0.483, &
 &  0.725, 0.717, 0.704, 0.685, 0.655, 0.635, 0.609, 0.578, 0.542, 0.511, &
 &  0.756, 0.748, 0.736, 0.717, 0.688, 0.668, 0.642, 0.611, 0.575, 0.543, &
 &  0.773, 0.765, 0.754, 0.736, 0.709, 0.690, 0.667, 0.638, 0.606, 0.580, &
 &  0.792, 0.785, 0.775, 0.759, 0.734, 0.717, 0.695, 0.670, 0.642, 0.623, &
 &  0.815, 0.809, 0.799, 0.785, 0.763, 0.747, 0.729, 0.707, 0.685, 0.675, &
 &  0.841, 0.836, 0.828, 0.815, 0.796, 0.783, 0.768, 0.750, 0.735, 0.735, &
 &  0.872, 0.868, 0.861, 0.851, 0.836, 0.826, 0.814, 0.801, 0.793, 0.807, &
 &  0.907, 0.904, 0.899, 0.892, 0.880, 0.872, 0.864, 0.856, 0.855, 0.879, &
 &  0.943, 0.941, 0.938, 0.933, 0.925, 0.920, 0.915, 0.913, 0.919, 0.956, &
 &  0.968, 0.967, 0.964, 0.961, 0.957, 0.955, 0.953, 0.955, 0.968, 1.020, &
 &  0.995, 0.995, 0.994, 0.993, 0.992, 0.993, 0.995, 1.002, 1.022, 1.090, &
 &  1.008, 1.008, 1.008, 1.008, 1.009, 1.011, 1.015, 1.025, 1.050, 1.128, &
 &  1.025, 1.026, 1.026, 1.028, 1.031, 1.035, 1.041, 1.053, 1.083, 1.171, &
 &  1.038, 1.039, 1.040, 1.043, 1.049, 1.054, 1.062, 1.078, 1.112, 1.210, &
 &  1.050, 1.051, 1.054, 1.058, 1.065, 1.071, 1.081, 1.100, 1.139, 1.247, &
 &  1.059, 1.061, 1.064, 1.069, 1.078, 1.086, 1.098, 1.119, 1.163, 1.280, &
 &  1.072, 1.075, 1.079, 1.085, 1.096, 1.105, 1.119, 1.144, 1.192, 1.317, &
 &  1.077, 1.080, 1.084, 1.091, 1.103, 1.113, 1.128, 1.154, 1.203, 1.331, &
 &  1.083, 1.086, 1.091, 1.098, 1.111, 1.122, 1.138, 1.165, 1.216, 1.345, &
 &  1.090, 1.093, 1.098, 1.107, 1.121, 1.132, 1.149, 1.177, 1.230, 1.359, &
 &  1.098, 1.101, 1.107, 1.116, 1.131, 1.143, 1.161, 1.190, 1.244, 1.373, &
 &  1.102, 1.106, 1.112, 1.121, 1.137, 1.150, 1.168, 1.198, 1.252, 1.378, &
 &  1.107, 1.111, 1.117, 1.127, 1.144, 1.157, 1.176, 1.207, 1.262, 1.382, &
 &  1.114, 1.118, 1.125, 1.136, 1.153, 1.167, 1.188, 1.219, 1.272, 1.378, &
 &  1.124, 1.128, 1.135, 1.147, 1.166, 1.181, 1.202, 1.234, 1.281, 1.347, &
 &  1.132, 1.136, 1.144, 1.157, 1.177, 1.193, 1.214, 1.241, 1.274, 1.271, &
 &  1.141, 1.146, 1.154, 1.167, 1.188, 1.203, 1.221, 1.242, 1.252, 1.170, &
 &  1.151, 1.156, 1.164, 1.177, 1.197, 1.210, 1.224, 1.234, 1.217, 1.057, &
 &  1.162, 1.167, 1.175, 1.187, 1.204, 1.214, 1.222, 1.218, 1.171, 0.939, &
 &  1.169, 1.174, 1.181, 1.191, 1.204, 1.209, 1.208, 1.187, 1.106, 0.814, &
 &  1.176, 1.180, 1.186, 1.193, 1.199, 1.197, 1.185, 1.143, 1.027, 0.690, &
 &  1.183, 1.186, 1.188, 1.190, 1.185, 1.173, 1.146, 1.079, 0.927, 0.565, &
 &  1.189, 1.191, 1.192, 1.191, 1.180, 1.162, 1.125, 1.045, 0.875, 0.505 /
 
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CH4_TCMR_TL',0,ZHOOK_HANDLE)

ZAK=reshape(ZDUM,(/10,60/))

ZMMR2VMRCH4=RMD/RMCH4
ZMMR2VMRCO2=RMD/RMCO2

IF(NGHG > 0) THEN
  DO JLEN=1,KLEN
    DO JF=1,NGEMS
      IF(YDGEMSPROF5(JLEN,JF)%IGRIB == 210061) THEN ! CO2 mixing ratio and pert
        ZCO2MR5(JLEN,:)=YDGEMSPROF5(JLEN,JF)%MODELPROF(:)
        ZCO2MR(JLEN,:)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
      ENDIF
      IF(YDGEMSPROF5(JLEN,JF)%IGRIB == 210062) THEN ! CH4 mixing ratio and pert
        ZCH4MR5(JLEN,:)=YDGEMSPROF5(JLEN,JF)%MODELPROF(:)
        ZCH4MR(JLEN,:)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
      ENDIF
    ENDDO ! loop over GEMS species
  ENDDO
ENDIF




DO JJ=1,KDCOUNT

  DO JL=1,KLEN

    ZDPSUM5=0.0_JPRB
    ZCO2SUM5 = 0.0_JPRB 
    ZCH4SUM5 = 0.0_JPRB 
    ZCH4SUM  = 0.0_JPRB 

    IF(KCOUNT(JL) == 0) THEN
      INDEX=5
    ELSE
      DO J=1,9
        IF(PSOLEL(JL) <= ZAK_SELEL(J) .AND. PSOLEL(JL) > ZAK_SELEL(J+1)) INDEX=J
      ENDDO
      IF(PSOLEL(JL) >  ZAK_SELEL(1))  INDEX=1
      IF(PSOLEL(JL) <= ZAK_SELEL(10)) INDEX=10
    ENDIF
    
    CALL INTAVG(ZAK_PRES,ZAK(INDEX,:),60,60,1,NFLEVG,PPRESF5(JL,:)/100.0,ZAK_MODEL)
    CALL INTAVG(ZAK_PRES,ZCH4_PRIOR,60,60,1,NFLEVG,PPRESF5(JL,:)/100.0,ZCH4_PRIOR_M)
    
    DO JK=1,NFLEVG

      ZDP5=PPRESH5(JL,JK)-PPRESH5(JL,JK-1)

      ZCO2SUM5=ZCO2SUM5+ ZCO2MR5(JL,JK)*ZMMR2VMRCO2*(ZDP5/RG)

      ZCH4SUM5=ZCH4SUM5+ (ZAK_MODEL(JK)*ZCH4MR5(JL,JK)*ZMMR2VMRCH4+ &
            & (1.0_JPRB-ZAK_MODEL(JK))*ZCH4_PRIOR_M(JK))*(ZDP5/RG)
      ZCH4SUM =ZCH4SUM + ZAK_MODEL(JK)*ZCH4MR(JL,JK)*(ZDP5/RG)*ZMMR2VMRCH4

      ZDPSUM5=ZDPSUM5 + (ZDP5/RG)

    ENDDO ! loop over model vertical levels

    ZCO2MEAN5=ZCO2SUM5/ZDPSUM5

    ZCH4MEAN =ZCH4SUM/ZDPSUM5

    PCH4MEAN(JL,JJ)=ZCH4MEAN*370.0E-06/ZCO2MEAN5
      
  ENDDO ! loop over horizontal observational points
ENDDO ! loop over channels

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CH4_TCMR_TL',1,ZHOOK_HANDLE)
END SUBROUTINE CH4_TCMR_TL
