SUBROUTINE COBSLAG(KASLB1,LDMVO5,PPARITY,PB1)

!**** *COBSLAG* - Observation horizontal interpolation lagged part .

!     Purpose.
!     -------
!           Observation horizontal interpolation lagged part .

!     Interface.
!     ---------
!        *CALL* *COBSLAG(...)*

!        Explicit arguments.
!        ------------------
!        PPARITY   : indicator for scalar(1) or vector(-1) field
!        PB1       : fields buffer for interpolation at observation points
!        LDMVO5    : transmitted to OBSHOR

!     Externals.
!     ---------
!        LOCATE  - MEMORY MANAGER ROUTINE (TO LOCATE AN ARRAY)
!        OBSHOR  - INTERPOLATION AT OBS. POINTS
!        MDONEBUF- I/O ROUTINE

!     Author.
!     ------
!        Drasko Vasiljevic and Mats Hamrud  *ECMWF*

!     Modifications.
!     -------------
!     92-03-21 : D.Vasiljevic and M.Hamrud  Original
!     92-06-25 : D.Dent Remove event logic
!     95-12-07 : F.Bouttier Introduce LDMVO5 in interfaces, remove CDCONF
!     97-11-17 : E.Andersson Introduce CIW
!     99-09-01 : P.Viterbo   Surface tiles
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   30-Jun-2008 J. Masek     Removed array IBUFP1 in call to OBSHOR.
!   04-Dec-2009: H.Hersbach 10m-neutral wind; introduce argument ZPARITY
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCT0   , ONLY : LCANARI  ,LCASIG
USE YOMCT0B  , ONLY : LECMWF
USE YOMSC2   , ONLY : NFLDOBB1
USE YOMPHY   , ONLY : LSOLV
USE YOMECTAB , ONLY :  NTSLTOB  ,NTSLOTNO ,NTSLOTNOS,&
 & NTSLOTNOE  
USE YOMGLOBS , ONLY : MAPSENDOBS
USE YOMLUN   , ONLY : NULOUT
USE GOMS_MIX , ONLY : NGOMGFL
USE YOMDPHY  ,ONLY  : LDIRCLSMOD

!    -------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
LOGICAL           ,INTENT(IN)    :: LDMVO5 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPARITY(NFLDOBB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(KASLB1,NFLDOBB1) 

INTEGER(KIND=JPIM) :: IOBB1AL, IOBB1ARG, &
 & IOBB1CO, IOBB1DL, &
 & IOBB1DM, IOBB1EH, IOBB1EM, IOBB1EST, IOBB1EZ, &
 & IOBB1ESN, IOBB1ET2, IOBB1EH2, IOBB1EV1, &
 & IOBB1HV, IOBB1LS, IOBB1OR, IOBB1PRS, &
 & IOBB1PRW, IOBB1GFL, IOBB1SAB, IOBB1SN, IOBB1SP, &
 & IOBB1T, IOBB1TS, IOBB1U, IOBB1V, IOBB1VG, &
 & IOBB1CI, IOBB1CVH, IOBB1CVL, IOBB1TVH, IOBB1TVL, &
 & IOBB1WL, IOBB1WS, IOBB1XOR, IOBB1XSN, IOBB1XTP, &
 & IOBB1XTS, IOBB1XWP, IOBB1XWS, IOBB1Z0, IOBB1Z0H, &
 & IPROMB, IPROMBMP,JTSLOT,IOBB1TCLS, IOBB1HUCLS,&
 & IOBB1UCU, IOBB1VCU, IOBB1CHN, IOBB1U10N, IOBB1V10N
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

#include "abor1.intfb.h"
#include "obshor.intfb.h"

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COBSLAG',0,ZHOOK_HANDLE)

!    -------------------------------------------------------------------

!*       1.    Interface to global arrays.
!              --------------------------

IOBB1U=1
IOBB1V=1
IOBB1T=1
IOBB1GFL=1
IOBB1EZ=1
IOBB1EH=1
IOBB1SP=1
IOBB1EST=1
IOBB1TS=1
IOBB1WS=1
IOBB1SN=1
IOBB1Z0=1
IOBB1WL=1
IOBB1OR=1
IOBB1CO=1
IOBB1DL=1
IOBB1DM=1
IOBB1AL=1
IOBB1EM=1
IOBB1LS=1
IOBB1VG=1
IOBB1PRW=1
IOBB1PRS=1
IOBB1ARG=1
IOBB1SAB=1
IOBB1HV=1
IOBB1Z0H=1
IOBB1XTS=1
IOBB1XWS=1
IOBB1XWP=1
IOBB1XSN=1
IOBB1XTP=1
IOBB1CI=1
IOBB1UCU=1
IOBB1VCU=1
IOBB1U10N=1
IOBB1V10N=1
IOBB1CHN=1
IOBB1CVH=1
IOBB1CVL=1
IOBB1TVH=1
IOBB1TVL=1
IOBB1ESN=1
IOBB1ET2=1
IOBB1EH2=1
IOBB1EV1=1
IOBB1TCLS=1
IOBB1HUCLS=1

IOBB1U  = 1         
IOBB1V  = IOBB1U   + NFLEVG
IOBB1T  = IOBB1V   + NFLEVG
IOBB1GFL  = IOBB1T   + NFLEVG
IOBB1SP = IOBB1GFL  + NGOMGFL*NFLEVG
IOBB1TS = IOBB1SP  + 1
IOBB1WS = IOBB1TS  + 1
IOBB1SN = IOBB1WS  + 1
IOBB1Z0 = IOBB1SN  + 1
IOBB1WL = IOBB1Z0  + 1
IOBB1U10N=IOBB1WL  + 1
IOBB1V10N=IOBB1U10N+ 1
IOBB1OR = IOBB1V10N+ 1
IOBB1CO = IOBB1OR  + 1
IOBB1DL = IOBB1CO  + 1
IOBB1DM = IOBB1DL  + 1
IOBB1AL = IOBB1DM  + 1
IOBB1EM = IOBB1AL  + 1
IOBB1LS = IOBB1EM  + 1
IF (LECMWF) THEN
  IOBB1CI  = IOBB1LS  + 1
  IOBB1UCU = IOBB1CI  + 1
  IOBB1VCU = IOBB1UCU + 1
  IOBB1CHN = IOBB1VCU + 1
  IOBB1CVH = IOBB1CHN + 1
  IOBB1CVL = IOBB1CVH + 1
  IOBB1TVH = IOBB1CVL + 1
  IOBB1TVL = IOBB1TVH + 1
  IOBB1VG  = IOBB1TVL
ELSE
  IOBB1VG  = IOBB1LS  + 1
  IOBB1TCLS = IOBB1VG + 1
  IOBB1HUCLS = IOBB1TCLS +1
ENDIF
IF (LCASIG) THEN
! model errors
  IF (LDIRCLSMOD) THEN
    IOBB1EZ  = IOBB1HUCLS + 1
  ELSE
    IOBB1EZ  = IOBB1VG   +  1
  ENDIF
  IOBB1EH  = IOBB1EZ + NFLEVG
  IOBB1EST = IOBB1EH + NFLEVG
  IOBB1ESN = IOBB1EST + 1
  IOBB1ET2 = IOBB1ESN + 1
  IOBB1EH2 = IOBB1ET2 + 1
  IOBB1EV1 = IOBB1EH2 + 1
ELSE
  IF (LDIRCLSMOD) THEN
    IOBB1EV1 = IOBB1HUCLS
  ELSE
    IOBB1EV1  = IOBB1VG
  ENDIF
ENDIF

IF (LCANARI) THEN
  IOBB1PRW = IOBB1EV1 + 1
  IOBB1PRS = IOBB1PRW + 1
! surface climatology (NVCLIX)
  IOBB1XOR = IOBB1PRS + 1
  IOBB1XTS = IOBB1XOR + 1
  IOBB1XWS = IOBB1XTS + 0
  IOBB1XWP = IOBB1XWS + 0
  IOBB1XSN = IOBB1XWP + 1
  IOBB1XTP = IOBB1XSN + 0
ELSE
  IOBB1XTP = IOBB1EV1
ENDIF

IF (LSOLV) THEN
  IOBB1ARG = IOBB1XTP + 1
  IOBB1SAB = IOBB1ARG + 1
  IOBB1HV  = IOBB1SAB + 1
  IOBB1Z0H = IOBB1HV  + 1
ELSE
  IOBB1Z0H = IOBB1XTP
ENDIF

IF( IOBB1Z0H > NFLDOBB1 )THEN
  WRITE(NULOUT,'("COBSLAG: IOBB1Z0H,NFLDOBB1=",2I6)') IOBB1Z0H,NFLDOBB1
  CALL ABOR1('COBSLAG: IOBB1Z0H > NFLDOBB1')
ENDIF

!    -------------------------------------------------------------------
!*       2.   LAGGED PART OF INTERPOLATION AT OBS. POINTS
!             -------------------------------------------

!*       2.0  LOOP OVER TIMESLOTS TO BE COMPARED

DO JTSLOT = NTSLOTNOS,NTSLOTNOE
  NTSLOTNO  = JTSLOT

!*       2.1  FIND OUT STARTING/ENDING OBS. NUMBER

  IPROMB   = MAPSENDOBS(NTSLOTNO)
  IPROMBMP = NTSLTOB(NTSLOTNO)

!*       2.2  INTERPOLATION AT OBS. POINTS

!       Note: OR,CO,AL,EM,LS,VG,PRW,PRS,ARG,SAB,HV,Z0H are constants,
!             thus neither included in TL nor AD code.
  CALL OBSHOR(&
   & LDMVO5,KASLB1,&
   & IPROMB,IPROMBMP,&
   & PPARITY,&
   & PB1(1,IOBB1U)  ,PB1(1,IOBB1V)  ,PB1(1,IOBB1T)  ,&
   & PB1(1,IOBB1GFL)  ,&
   & PB1(1,IOBB1EZ) ,PB1(1,IOBB1EH) ,&
   & PB1(1,IOBB1SP) ,PB1(1,IOBB1EST),&
   & PB1(1,IOBB1UCU),PB1(1,IOBB1VCU),PB1(1,IOBB1CHN),&
   & PB1(1,IOBB1TS) ,PB1(1,IOBB1WS) ,PB1(1,IOBB1SN) ,&
   & PB1(1,IOBB1Z0) ,PB1(1,IOBB1WL) ,&
   & PB1(1,IOBB1U10N),PB1(1,IOBB1V10N),&
   & PB1(1,IOBB1OR) ,PB1(1,IOBB1CO) ,PB1(1,IOBB1DL) ,&
   & PB1(1,IOBB1DM) ,PB1(1,IOBB1AL) ,&
   & PB1(1,IOBB1EM) ,PB1(1,IOBB1LS) ,PB1(1,IOBB1VG) ,&
   & PB1(1,IOBB1CI) ,PB1(1,IOBB1CVH),PB1(1,IOBB1CVL),&
   & PB1(1,IOBB1TVH),PB1(1,IOBB1TVL),&
   & PB1(1,IOBB1PRW),PB1(1,IOBB1PRS),PB1(1,IOBB1ARG),&
   & PB1(1,IOBB1SAB),PB1(1,IOBB1HV) ,PB1(1,IOBB1Z0H),&
   & PB1(1,IOBB1XTS),PB1(1,IOBB1XWS),&
   & PB1(1,IOBB1XWP),PB1(1,IOBB1XSN),PB1(1,IOBB1XTP),&
   & PB1(1,IOBB1ESN),PB1(1,IOBB1ET2),PB1(1,IOBB1EH2),&
   & PB1(1,IOBB1EV1),PB1(1,IOBB1TCLS),PB1(1,IOBB1HUCLS))  
ENDDO

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COBSLAG',1,ZHOOK_HANDLE)
END SUBROUTINE COBSLAG
