SUBROUTINE COBSLAGAD(KASLB1,PPARITY,PB1)

!**** *COBSLAGAD* - Observation horizontal interpolation lagged part .

!     Purpose.
!     --------
!           Observation horizontal interpolation lagged part .

!     Interface.
!     ----------
!        *CALL* *COBSLAGAD(...)*

!        Explicit arguments :
!        --------------------
!        PPARITY   : indicator for scalar (1) or vector (-1) field
!        PB1       : fields buffer for interpolation at observation points

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF documentation

!     Author.
!     -------
!        Drasko Vasiljevic and Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!      Original  : 92-03-21
!     F.Bouttier   95-12-07 : Introduce LDMVO5 in interfaces, remove CDCONF
!     L. Isaksen   96-02-20 : NFLEV --> NFLEVG
!     F.Taillefer  97-05-27 : Add DL and DM arrays
!     E. Andersson 97-11-17 : Introduce CIW and O3
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     H.Hersbach   04-12-09 : Introduce argument PPARITY
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGSAG,NDGENG
USE YOMSC2   , ONLY : NFLDOBB1
USE YOMECTAB , ONLY : NTSLTOB  ,NTSLOTNO  ,NTSLOTNOS  ,NTSLOTNOE
USE YOMGLOBS , ONLY : MAPRECVOBSAD
!    -------------------------------------------------------------------

IMPLICIT NONE

! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPARITY(NFLDOBB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(KASLB1,NFLDOBB1) 
! * INPUT/OUTPUT:
INTEGER(KIND=JPIM) :: IBUFP1(NDGSAG:NDGENG)

INTEGER(KIND=JPIM) :: IPROMB, IPROMBMP, ITS, JTSLOT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "obshorad.intfb.h"

!    -------------------------------------------------------------------
!*       2.   LAGGED PART OF INTERPOLATION AT OBS. POINTS
!             -------------------------------------------

!*       2.0  LOOP OVER TIMESLOTS TO BE COMPARED

IF (LHOOK) CALL DR_HOOK('COBSLAGAD',0,ZHOOK_HANDLE)
DO JTSLOT = NTSLOTNOS,NTSLOTNOE,-1
  NTSLOTNO  = JTSLOT

!*       2.1  INTERPOLATION AT OBS. POINTS

  ITS = NTSLOTNO
  IPROMB   = MAPRECVOBSAD(ITS)
  IPROMBMP = NTSLTOB(ITS)

!       Note: OR,CO,AL,EM,LS,VG,PRW,PRS,ARG,SAB,HV,Z0H are constants,
!             thus neither included in TL nor AD code.
  CALL OBSHORAD(IBUFP1,KASLB1,IPROMB,IPROMBMP,PPARITY,PB1)  
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COBSLAGAD',1,ZHOOK_HANDLE)
END SUBROUTINE COBSLAGAD
