SUBROUTINE CSALBR_GEMS(PTODRAY, PALB) 
!****CSALBR*** CALCULATES THE SPHERICAL ALBEDO FOR THE MOLECULAR LAYER


!     INTERFACE
!     ---------
! CSALBR IS CALLED BY SCATRA

!     INPUTS
!     ------

! PTODRAY  : TOTAL MOLECULAR OPTICAL DEPTH

!     OUTPUTS
!     -------
! PSPHALBR : RAYLEIGH SPHERICAL ALBEDO

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007
!          A.BENEDETTI - ECMWF - 06.08.2008
!          S.Martinez - METEO-FRANCE - 25.09.2009 - Portability fix

!     SOURCE
!     ------
! ADAPTED FROM CSALBR.f FROM ORIGINAL 6S CODE, SEE VERMOTTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK     , ONLY: LHOOK, DR_HOOK

USE RT6SVALUES

IMPLICIT NONE

REAL(KIND=JPRB), INTENT(IN)  :: PTODRAY
REAL(KIND=JPRB), INTENT(OUT) :: PALB

! LOCAL VARIABLE
! --------------

REAL(KIND=JPRB) :: FINTEXP3_GEMS

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CSALBR_GEMS',0,ZHOOK_HANDLE)


PALB=(3.0_JPRB*PTODRAY-FINTEXP3_GEMS(PTODRAY)*(4.0_JPRB+2.0_JPRB*PTODRAY)+2.0_JPRB*EXP(-1.0_JPRB*PTODRAY))
PALB=PALB/(4.0_JPRB+3.0_JPRB*PTODRAY)

IF (LHOOK) CALL DR_HOOK('CSALBR_GEMS',1,ZHOOK_HANDLE)
END SUBROUTINE CSALBR_GEMS

FUNCTION FINTEXP3_GEMS(PTODRAY)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK     , ONLY: LHOOK, DR_HOOK

REAL(KIND=JPRB) :: FINTEXP3_GEMS
REAL(KIND=JPRB), INTENT(IN) :: PTODRAY
REAL(KIND=JPRB) :: ZZ, FINTEXP1_GEMS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('FINTEXP3_GEMS',0,ZHOOK_HANDLE)

ZZ=(EXP(-PTODRAY)*(1.-PTODRAY)+PTODRAY*PTODRAY*FINTEXP1_GEMS(PTODRAY))/2.
FINTEXP3_GEMS=ZZ

IF (LHOOK) CALL DR_HOOK('FINTEXP3_GEMS',1,ZHOOK_HANDLE)
END FUNCTION FINTEXP3_GEMS 

FUNCTION FINTEXP1_GEMS(PTODRAY)
! accuracy 2e-07... for 0<PTODRAY<1

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK     , ONLY: LHOOK, DR_HOOK

REAL(KIND=JPRB) :: FINTEXP1_GEMS

INTEGER(KIND=JPIM):: J

REAL(KIND=JPRB), INTENT(IN) :: PTODRAY
REAL(KIND=JPRB) :: ZZ, ZA(0:5), ZFTAU

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('FINTEXP1_GEMS',0,ZHOOK_HANDLE)

DATA (ZA(I),I=0,5) /-.57721566,0.99999193,-0.24991055,     &
&                    0.05519968,-0.00976004,0.00107857/

ZZ=ZA(0)
ZFTAU=1.

DO J=1,5
  ZFTAU=ZFTAU*PTODRAY
  ZZ=ZZ+ZA(J)*ZFTAU
ENDDO
FINTEXP1_GEMS=ZZ-LOG(PTODRAY)

IF (LHOOK) CALL DR_HOOK('FINTEXP1_GEMS',1,ZHOOK_HANDLE)
END FUNCTION FINTEXP1_GEMS
