SUBROUTINE EMIS_ATLAS(K_ISENSOR,PLAT,PLON,PZANGL,KFOV,PEM,PBATS_VEG)  

!     PURPOSE.
!     -------
!     Routine to get surface emissivity 
!     from an atlas into the ODB
!     INTERFACE.
!     ---------
!     call emis_ATLAS()

!     ARGUMENTS
!     ---------
!     K_ISENSOR - input satellite sensor indicator
!     PLAT - observation latitude
!     PLON - observation longitude
!     PZANGL - observation zenith angle

!     PEM - output microwave surface emissivity        
!     PBATS_VEG - BATS vegetation type

!     EXTERNALS
!     ---------
!     * amsu_sfc  in tovscode                  

!     AUTHOR
!     ------
!     F Karbou 2006
!     N Bormann, bugfix 2007

!     MODIFICATIONS
!     -------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY  : RMDI
USE YOMDIMO   , ONLY : NMXTCH
USE YOMTVRAD  , ONLY : MSENSOR_CHANOFFL,&
 & MSENSOR_AMSUA,MSENSOR_AMSUB,MSENSOR_SSMI,MSENSOR_SSMIS,&
 & MSENSOR_MHS, CHSENSOR
USE YOMEMIS , ONLY : LATLAS_EMIS, LDYN_TSKIN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: K_ISENSOR  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZANGL
INTEGER(KIND=JPIM), INTENT(IN)   :: KFOV

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEM(NMXTCH) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBATS_VEG

REAL(KIND=JPRB) :: ZE_RET3(4), ZSSMIS_ERET(8), ZSSMI_ERET(7)
INTEGER(KIND=JPIM) ::IOFF
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZE_RET1

#include "abor1.intfb.h"


IF (LHOOK) CALL DR_HOOK('EMIS_ATLAS',0,ZHOOK_HANDLE)

PEM(:) = 2.0_JPRB

IOFF=MSENSOR_CHANOFFL(K_ISENSOR)

SELECT CASE (K_ISENSOR)
  CASE(MSENSOR_AMSUA)
    IF (PZANGL > 30) THEN
      CALL LAND_AMSUA_AN2(PLAT,PLON,ZE_RET3,PBATS_VEG)
    ELSE
      CALL LAND_AMSUA_AN1(PLAT,PLON,KFOV,ZE_RET3,PBATS_VEG)
    ENDIF
    PEM(IOFF+1)=ZE_RET3(1)
    PEM(IOFF+2)=ZE_RET3(2)
    PEM(IOFF+3:IOFF+14)=ZE_RET3(3)
    PEM(IOFF+15)=ZE_RET3(4)
  CASE(MSENSOR_AMSUB,MSENSOR_MHS)
    IF (PZANGL > 30) THEN
      CALL LAND_AMSUB_AN2 (PLAT,PLON,ZE_RET1,PBATS_VEG) 
    ELSE
      CALL LAND_AMSUB_AN1 (PLAT,PLON,KFOV,ZE_RET1,PBATS_VEG) 
    ENDIF
    PEM(IOFF+1)=ZE_RET1
    PEM(IOFF+2)=ZE_RET1
    PEM(IOFF+3:IOFF+5)=ZE_RET1
  CASE(MSENSOR_SSMI)
    CALL LAND_SSMI(PLAT,PLON,ZSSMI_ERET,PBATS_VEG)
    PEM(IOFF+1)=ZSSMI_ERET(1)
    PEM(IOFF+2)=ZSSMI_ERET(2)
    PEM(IOFF+3)=ZSSMI_ERET(3)
    PEM(IOFF+4)=ZSSMI_ERET(4)
    PEM(IOFF+5)=ZSSMI_ERET(5)
    PEM(IOFF+6)=ZSSMI_ERET(6)
    PEM(IOFF+7)=ZSSMI_ERET(7)
  CASE(MSENSOR_SSMIS)
    CALL LAND_SSMIS(PLAT,PLON,ZSSMIS_ERET,PBATS_VEG)
    PEM(IOFF+1:IOFF+7)=ZSSMIS_ERET(6)
    PEM(IOFF+8:IOFF+11)=ZSSMIS_ERET(8)      
    PEM(IOFF+12)=ZSSMIS_ERET(2)      
    PEM(IOFF+13)=ZSSMIS_ERET(1)      
    PEM(IOFF+14)=ZSSMIS_ERET(3)      
    PEM(IOFF+15)=ZSSMIS_ERET(5)      
    PEM(IOFF+16)=ZSSMIS_ERET(4)      
    PEM(IOFF+17)=ZSSMIS_ERET(7)      
    PEM(IOFF+18)=ZSSMIS_ERET(8)      
    PEM(IOFF+19:IOFF+24)=ZSSMIS_ERET(6)  
  CASE DEFAULT
    IF( LATLAS_EMIS(K_ISENSOR) .OR. LDYN_TSKIN(K_ISENSOR) ) THEN
      CALL ABOR1('No emissivity atlas implemented for '//CHSENSOR(K_ISENSOR))  
    ELSE
      PEM = RMDI
    ENDIF
END SELECT

IF (LHOOK) CALL DR_HOOK('EMIS_ATLAS',1,ZHOOK_HANDLE)
END SUBROUTINE EMIS_ATLAS
