SUBROUTINE EMIS_MW_N(K_ISENSOR,PANGL,PLS,PTS,PSN,PWS,PIC, &
 & P23R,P31R,P50R,P89R ,K_ISUR_M , PEM)  
 
!     PURPOSE.
!     -------
!     Routine called by TOVCLR to compute microwave surface emissivity 
!     for various surface conditions

!     INTERFACE.
!     ---------
!     call emis_mw()

!     ARGUMENTS
!     ---------
!     ISENSOR - input satellite sensor indicator
!     PANGL - input satellite nadir angle (deg)
!     PLS - input model land/sea flag
!     PTS - input model skin temperature
!     PSN - input model snow cover
!     PWS - input model soil moisture
!     PIC - input model ice fraction                  
!     P23R - measurement amsua ch 1
!     P31R - measurement amsua ch 2
!     P50R - measurement amsua ch 3
!     P89R - measurement amsua ch 4
!     ISUR_M- surface type              

!     PEM - output microwave surface emissivity        

!     EXTERNALS
!     ---------
!     * amsu_sfc  in tovscode                  

!     AUTHOR
!     ------
!     Tony McNally - March 1998
!     g kelly

!     MODIFICATIONS
!     -------------
!     * add ice/snow model
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P Bauer, G Kelly, N Bormann    05-Feb-2006    Add AMSRE, SSMIS, TMI
!        N Bormann     17 Feb 2006 Cleaning
!        Q Lu          10 Mar 2009 add FY3A MWTS, MWHS, MWRI

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO   , ONLY : NMXTCH
USE YOMTVRAD  , ONLY : MSENSOR_CHANOFFL,MSENSOR_CHANCOUNT,&
 & MSENSOR_MSU,MSENSOR_AMSUA,MSENSOR_AMSUB,MSENSOR_SSMI,MSENSOR_SSMIS,&
 & MSENSOR_MWTS,MSENSOR_MWHS,MSENSOR_MWRI, &
 & MSENSOR_AMSRE,MSENSOR_TMI,MSENSOR_MHS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: K_ISENSOR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PANGL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P23R 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P31R 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P50R 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P89R 
INTEGER(KIND=JPIM),INTENT(OUT)   :: K_ISUR_M 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEM(NMXTCH) 
REAL(KIND=JPRB) ::Z_TB_ERR(4),ZLPW,Z_ESEA,Z_ELND
REAL(KIND=JPRB) :: ZFREQ(4),ZE_RET(4), Z_ICE, Z_COS1
INTEGER(KIND=JPIM) ::ID_SFC,IOFF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',0,ZHOOK_HANDLE)
ZFREQ(1:4)=(/P23R,P31R,P50R,P89R/)
Z_COS1=COS(PANGL*3.14_JPRB/180.00_JPRB)

K_ISUR_M = 0
PEM(:) = 0.99_JPRB
Z_ESEA   = 0.6_JPRB
Z_ELND   = 0.95_JPRB

!     *** VARIOUS CONDITIONS NOW FOLLOW FOR EMISSIVITY ESTIMATION ***
!     ===============================================================

!     1.0 Ice free open sea or mixed ice               
!     ==================================
IOFF=MSENSOR_CHANOFFL(K_ISENSOR)

IF(PLS < 0.01_JPRB .AND. PIC < 0.01_JPRB) THEN
  PEM(:)=Z_ESEA

  IF (K_ISENSOR == MSENSOR_AMSUA) THEN
      CALL AMSU_SFC (Z_COS1, NINT(PLS), PTS, ZFREQ, ID_SFC, Z_ICE, ZLPW, ZE_RET, Z_TB_ERR)       
      K_ISUR_M=ID_SFC
  ENDIF

  IF (K_ISENSOR == MSENSOR_MSU   .OR. K_ISENSOR == MSENSOR_AMSUA .OR. &
   &  K_ISENSOR == MSENSOR_AMSUB .OR. K_ISENSOR == MSENSOR_MHS   .OR. &
   &  K_ISENSOR == MSENSOR_MWTS  .OR. K_ISENSOR == MSENSOR_MWHS  .OR. &
   &  K_ISENSOR == MSENSOR_MWRI  .OR. &
   &  K_ISENSOR == MSENSOR_SSMIS .OR. K_ISENSOR == MSENSOR_SSMI  .OR. &
   &  K_ISENSOR == MSENSOR_TMI   .OR. K_ISENSOR == MSENSOR_AMSRE ) THEN

     PEM (IOFF+1:IOFF+MSENSOR_CHANCOUNT(K_ISENSOR)) = 2.0_JPRB

  ELSE
    CALL ABOR1('NO SENSOR TYPE FOR MW EMISSIVITY')

  ENDIF

  IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
  RETURN
ENDIF

! amsua ice/snow/land

IF(K_ISENSOR == MSENSOR_AMSUA) THEN 
 
  ZFREQ(1:4)=(/P23R,P31R,P50R,P89R/)
    
  CALL AMSU_SFC (Z_COS1, NINT(PLS), PTS, ZFREQ, ID_SFC, Z_ICE, ZLPW, ZE_RET, Z_TB_ERR)       

  PEM(IOFF+1)=ZE_RET(1)
  PEM(IOFF+2)=ZE_RET(2)
  PEM(IOFF+3:IOFF+14)=ZE_RET(3)
  PEM(IOFF+15)=ZE_RET(4)
  K_ISUR_M=ID_SFC

  IF (PLS >= 0.01_JPRB .AND. PLS <= 0.5_JPRB) K_ISUR_M = 12
    
  IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
  RETURN   
ENDIF

!     2.0 Sea ice
!     ===========

!     2.1 Exposed sea ice
!     -------------------
IF(PIC >= 0.01_JPRB) THEN

!simple regression based on amsa model for msu
  IF(K_ISENSOR == MSENSOR_MSU) THEN
    IF((PTS-P23R ) < 10._JPRB ) THEN
      PEM(:)=0.95_JPRB 
    ELSEIF((PTS-P23R ) > 40._JPRB ) THEN
      PEM(:)=0.7_JPRB 
    ELSE
      PEM(:)=1.1_JPRB -.01_JPRB*(PTS-P23R)
    ENDIF
    
    IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
    RETURN
  ENDIF
ENDIF

!     3.0 Land
!     ========

!     3.1 Exposed dry land
!     --------------------
IF(PLS >= 0.01_JPRB .AND. PIC < 0.01_JPRB) THEN
  IF(PSN <= 0.0_JPRB .AND. PWS <= 10.0_JPRB) THEN
!          PRINT*,'FOUND EXPOSED DRY LAND',PLS,PTS,PSN,PWS
    PEM(:)=0.92_JPRB * PLS + (1.0 - PLS) * Z_ESEA
    IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
    RETURN
  ENDIF

!     3.2 Exposed moist land or light snow cover
!     ------------------------------------------
  IF(PSN <= 20.0_JPRB .AND. PWS > 10.0_JPRB) THEN
!          PRINT*,'FOUND EXPOSED MOIST LAND',PLS,PTS,PSN,PWS
    PEM(:)=0.95_JPRB * PLS + (1.0 - PLS) * Z_ESEA
    IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
    RETURN
  ENDIF

!     3.3 Land with deep snow cover
!     -----------------------------
  IF(PSN > 20.0_JPRB .AND. PSN <= 80.0_JPRB) THEN
    IF(PTS <= 265.0_JPRB) THEN
!            PRINT*,'FOUND LAND WITH LIGHT SNOW COVER',PLS,PTS,PSN,PWS
      PEM(:)=0.83_JPRB * PLS + (1.0 - PLS) * Z_ESEA
      IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
      RETURN
    ELSE
      PEM(:)=0.95_JPRB * PLS + (1.0 - PLS) * Z_ESEA   !revert to moist land value
      IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
  ENDIF

!     3.4 Land with very deep snow cover
!     ----------------------------------
  IF(PSN > 80.0_JPRB) THEN
    IF(PTS <= 265.0_JPRB) THEN
!           PRINT*,'FOUND LAND WITH DEEP SNOW COVER',PLS,PTS,PSN,PWS
      PEM(:)=0.80_JPRB * PLS + (1.0 - PLS) * Z_ESEA
      IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
      RETURN
    ELSE
      PEM(:)=0.95_JPRB * PLS + (1.0 - PLS) * Z_ESEA   !revert to moist land value
      IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
      RETURN
    ENDIF
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('EMIS_MW_N',1,ZHOOK_HANDLE)
END SUBROUTINE EMIS_MW_N
