FUNCTION fcseradi(P_LAT_IN_DEG)

!**** *fcseradi* - calculates effective radius used in geopotential/geometric height conversion.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates "effective" radius used in geopotential/geometric height conversion.:

!**   INTERFACE
!     ---------

!     fcseradi(P_LAT_IN_DEG) 
!     WHERE P_LAT_IN_DEG   = latitude in degrees.

!     fcseradi is called from gpscalc_nr, gpscalc_nrtl and gpscalc_nrad

!**   METHOD
!     ------
!     calculate the effective radius used to map from geopotential to 
!     geometric heights (List,1968),Smithsonian Met Tables.
    

!**   EXTERNALS
!     ---------
!     1) fcsglat 

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK


IMPLICIT NONE

! input

REAL(KIND=JPRB), INTENT(IN) :: P_LAT_IN_DEG
REAL(KIND=JPRB) :: Z_GVAL,Z_LAT_IN_RAD,Z_DG_DZ

! function
      
REAL(KIND=JPRB) :: fcsglat
REAL(KIND=JPRB) :: fcseradi
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! surface gravity at location

IF (LHOOK) CALL DR_HOOK('FCSERADI',0,ZHOOK_HANDLE)
Z_GVAL=fcsglat(P_LAT_IN_DEG)

! lat in radians

Z_LAT_IN_RAD=1.745329E-2_JPRB*P_LAT_IN_DEG

Z_DG_DZ= &
 & 3.085462E-6_JPRB+ &
 & 2.27E-9_JPRB*COS(2.0_JPRB*Z_LAT_IN_RAD)- &
 & 2.0E-12_JPRB*COS(4.0_JPRB*Z_LAT_IN_RAD)  

fcseradi=2.0_JPRB*Z_GVAL/Z_DG_DZ

IF (LHOOK) CALL DR_HOOK('FCSERADI',1,ZHOOK_HANDLE)
END FUNCTION fcseradi
