FUNCTION fcserf(P_X)

!**** *fcserf* - Calculates the Gaussian error function.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates the Gaussian error function 

!**   INTERFACE
!     ---------

!     fcserf(P_X) 
!     WHERE P_X      = Argument.

!     fcserf is called from gpscalc_alpha, gpscalc_alphatl and gpacalc_alphaad

!**   METHOD
!     ------
!     The error function from Abram+Stegun, page 299, 7.1.26.
 

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK


IMPLICIT NONE

REAL(KIND=JPRB), INTENT(IN) :: P_X
REAL(KIND=JPRB) :: Z_T
REAL(KIND=JPRB) :: fcserf
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('FCSERF',0,ZHOOK_HANDLE)

Z_T = 0.0 
      
IF (ABS(P_X) < 9.0_JPRB ) THEN
      
   IF (P_X == 0.0_JPRB ) THEN 
      
       FCSERF = 0.0_JPRB 
      
   ELSEIF (P_X > 0.0_JPRB ) THEN
      
     Z_T=1.0_JPRB/(1.0_JPRB+0.47047_JPRB*P_X)
     FCSERF=1.0_JPRB-(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*Z_T)*Z_T)*Z_T*EXP(-(P_X*P_X))
    
            
   ELSE   !(X < 0.0)!
      
      Z_T=1.0_JPRB/(1.0 -0.47047_JPRB*P_X)
     FCSERF=(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*Z_T)*Z_T)*Z_T*EXP(-(P_X*P_X))-1.0_JPRB
     
   ENDIF   
        
ELSE
  
   FCSERF = SIGN(1.0_JPRB,P_X) 
         
ENDIF 



IF (LHOOK) CALL DR_HOOK('FCSERF',1,ZHOOK_HANDLE)

END FUNCTION fcserf
