SUBROUTINE GAUSS_GEMS(PX1, PX2, KN, PX, PW)

!****GAUSS*** CALCULATION OF ANGLES AND WEIGHTS FOR GAUSS INTEGRATION


!     INTERFACE
!     ---------
! GAUSS IS CALLED BY PRE_GAUSS

!     INPUTS
!     ------
! PX1   : LOWER ANGULAR LIMIT
! PX2   : UPPER ANGULAR LIMIT
! KN : NUMBER OF ANGLES TO BE CONSIDERED 

!     OUTPUTS
!     -------
! PX(KN) : GAUSSIAN ANGLES 
! PW(KN) : GAUSSIAN WEIGHTS 

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007
!          A.BENEDETTI 6.8.2008

!     SOURCE
!     ------
! ADAPTED FROM GAUSS.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE RT6SVALUES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KN
REAL(KIND=JPRB),INTENT(IN)    :: PX1,PX2
REAL(KIND=JPRB),INTENT(OUT)   :: PX(KN),PW(KN)

! LOCAL VARIABLES
! ---------------

INTEGER(KIND=JPIM) :: IM, J1, J2
REAL(KIND=JPRB) :: ZXM, ZXL
REAL(KIND=JPRB) :: Z, Z1, ZP1, ZP2, ZP3, ZPP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GAUSS_GEMS',0,ZHOOK_HANDLE)

PX(:)=0.0_JPRB
PW(:)=0.0_JPRB
IM=INT((KN+1)/2)
ZXM=0.5_JPRB*(PX2+PX1)
ZXL=0.5_JPRB*(PX2-PX1)

! COMPUTATIONS FOR J1=1
DO J1=1,IM
  Z=cos(ZPI*(DBLE(J1)-0.25_JPRB)/(DBLE(KN)+0.5_JPRB))
  DO      ! LOOP THAT FINISHES WHEN ABS(Z-Z1).GT.PEPS
    ZP1=1.0_JPRB
    ZP2=0.0_JPRB
    DO J2=1,KN
      ZP3=ZP2
      ZP2=ZP1
      ZP1=((2.0_JPRB*DBLE(J2)-1.0_JPRB)*Z*ZP2-(DBLE(J2)-1.0_JPRB)*ZP3)/DBLE(J2)
    ENDDO
    ZPP=DBLE(KN)*(Z*ZP1-ZP2)/(Z*Z-1.0_JPRB)
    Z1=Z
    Z=Z1-ZP1/ZPP
    IF(ABS(Z-Z1) <= PEPS) THEN
      EXIT
    ENDIF
  ENDDO 
  IF(ABS(Z) < PEPS) THEN
    Z=0.0_JPRB
  ENDIF
  PX(J1)=ZXM-ZXL*Z
  PX(KN+1-J1)=ZXM+ZXL*Z
  PW(J1)=2.0_JPRB*ZXL/((1.0_JPRB-Z*Z)*ZPP*ZPP)
  PW(KN+1-J1)=PW(J1)
ENDDO

IF (LHOOK) CALL DR_HOOK('GAUSS_GEMS',1,ZHOOK_HANDLE)
END SUBROUTINE GAUSS_GEMS
