SUBROUTINE GHG_AK_AD(KDLEN,KLEN,KDCOUNT,CDPAR,KCOUNT,&
 & KAK,PWAK,PAPAK,PRESO,PXP,PXPD,YDGEMSPROF,&
 & KDPP,PXGEMS,&
 & PRESH5,PRESF5,PXP5,PXPD5,YDGEMSPROF5,PNO2,PNO25)  

!**** GHG_AK_AD - Adjoint of Observation operator for GRG

!     A. Dethof              ECMWF           22-Feb-06

!     PURPOSE
!     -------

!        FIND OUT A GRADIENT WITH RESPECT TO POST-PROCESSED VALUES.

!**   INTERFACE
!     ---------

!        CALL GHG_AK_AD(KDLEN,KLEN,KDCOUNT,CDPAR,KCOUNT,
!    I                  KAK,PWAK,PAPAK,PRESO,PXP,PXPD,YDGEMSPROF,
!    O                  KDPP,PXGRG,
!    R                  PRESH5,PRESF5,PXP5,PXPD5,YDGEMSPROF5,PNO2,PNO25)
!     WHERE KDLEN   : FIRST DIMENSION (Observations)            (INPUT)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS             (INPUT)
!           KDCOUNT : SECOND DIMENSION (Data)                   (INPUT)
!           CDPAR   : PARAMETER FOR POST-PROCESSING             (INPUT)
!           KCOUNT  : COUNTER FOR THE PARTICULAR OBS. PARAMETER (INPUT)
!           KAK     : Number of averaging kernel elements       (INPUT)
!           PWAK    : Weights for averaging kernels             (INPUT)
!           PAPAK   : Prior values for averaging kernels        (INPUT)
!           PRESO   : Output PRESSURES (ak pressures)           (INPUT)
!           PXP     : GRADIENT WITH RESPECT TO PXP5             (INOUT)
!           PXPD    : ..................                        (INOUT)
!           YDGEMSPROF: GRADIENT WITH RESPECT TO YDGEMSPROF5      (INOUT)
!           KDPP    : Number of fields, dimension of PXPP
!           PXGEMS  : GHG layer value                           (INOUT)
!           PRESH5  : HALF LEVEL PRESSURES                      (INPUT)
!           PRESF5  : FULL LEVEL PRESSURES                      (INPUT)
!           PXP5    : INTERMEDIATE ARRAY                        (INPUT)
!           PXPD5   : ..................                        (INPUT)
!           YDGEMSPROF5: GEMS array containing GRG mixing ratios (INPUT)
!           PNO2    : Perturb in NO2 (optional)                 (INOUT)
!           PNO25   : NO2 (optional)                            (INPUT)


!        GHG_AK_AD IS CALLED FROM HOPAD

!**   METHOD
!     ------

!        CALL TO PPAKAD POST-PROCESSING ROUTINE

!**   EXTERNALS
!     ---------

!        PPFLEV  - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES
!        PPAKAD  - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES

!**   AUTHOR  
!     ------
!     A.Dethof    22-Feb-2006: Adjoint of obs operator for GRG variables 
!                              using averaging kernels

!**   MODIFICATIONS
!     -------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARCMA    ,ONLY : JPMX_AK
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPPM

USE YOMCST   , ONLY : RG, RMD, RMCH4
USE YOM_YGFL , ONLY : NGHG,YGHG_NL
USE TYPE_GEMS_PROFILES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAK(KDLEN,KDCOUNT)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWAK(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPAK(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESO(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PXP(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PXPD(KDLEN,0:NFLEVG,NPPM) 
TYPE (TYPE_GEMSPROF),INTENT(INOUT) :: YDGEMSPROF(KDLEN,NGEMS)
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXGEMS(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM) 
TYPE (TYPE_GEMSPROF),INTENT(IN) :: YDGEMSPROF5(KDLEN,NGEMS)
REAL(KIND=JPRB)   ,INTENT(INOUT), OPTIONAL:: PNO2(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN), OPTIONAL:: PNO25(KDLEN,0:NFLEVG)


INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVB_AK(:,:,:),ILEVB(:,:,:,:)
LOGICAL, ALLOCATABLE :: LLBELO(:,:,:),LLBLOW(:,:),LLBELS(:,:,:),LLBLES(:,:)
LOGICAL, ALLOCATABLE :: LLBELO_AK(:,:),LLBLOW_AK(:),LLBELS_AK(:,:),LLBLES_AK(:)

REAL(KIND=JPRB), ALLOCATABLE :: ZGEMSMODEL(:,:),ZGEMSMODEL5(:,:),ZGEMS_AK(:,:)
REAL(KIND=JPRB):: ZPRESO_AK(KDLEN,JPMX_AK)

REAL(KIND=JPRB) :: ZXGEMS(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB) :: ZMMR2VMRCH4, ZRGI
REAL(KIND=JPRB) :: ZDP5(KDLEN,JPMX_AK)
REAL(KIND=JPRB) :: ZDPSUM5(KDLEN)
INTEGER(KIND=JPIM) :: JGEMS,JGHG,JLEN
INTEGER(KIND=JPIM) :: ILEN,IMXCNT,ICOUNT,IMXKAK,INPP
INTEGER(KIND=JPIM) :: IAK
LOGICAL, DIMENSION(NGHG) :: LLGHG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ppqad.intfb.h"
#include "ppflev.intfb.h"
!     ------------------------------------------------------------------
!*    1.  PREPARATION
!*    1.1 MAX. NO. OF LEVELS/obs

IF (LHOOK) CALL DR_HOOK('GHG_AK_AD',0,ZHOOK_HANDLE)

ZMMR2VMRCH4=RMD/RMCH4
ZRGI=1.0_JPRB/RG

ILEN   = KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0) GOTO 9
IMXKAK=MAXVAL(KAK(1:ILEN,1:IMXCNT))


DO JGHG=1,NGHG
  LLGHG(JGHG)=CDPAR == TRIM(YGHG_NL(JGHG)%cname)
ENDDO

INPP=0

!*    1.2 FIND OUT PRESSURE LEVEL BELOW SPECIFIED (OBSERVED) PRESSURES

ALLOCATE(ILEVB_AK(KDLEN,IMXKAK,NPPM))
ALLOCATE(LLBELO_AK(KDLEN,IMXKAK))
ALLOCATE(LLBELS_AK(KDLEN,IMXKAK))
ALLOCATE(LLBLOW_AK(IMXKAK))
ALLOCATE(LLBLES_AK(IMXKAK))

ALLOCATE(ILEVB(KDLEN,IMXKAK,NPPM,IMXCNT))
ALLOCATE(LLBELO(KDLEN,IMXKAK,IMXCNT))
ALLOCATE(LLBELS(KDLEN,IMXKAK,IMXCNT))
ALLOCATE(LLBLOW(IMXKAK,IMXCNT))
ALLOCATE(LLBLES(IMXKAK,IMXCNT))

Do ICOUNT=1,IMXCNT

  ZPRESO_AK(:,:)=PRESO(:,ICOUNT,:)

  CALL PPFLEV(KDLEN,1,ILEN,NFLEVG,IMXKAK,&
   & ZPRESO_AK,PRESH5,PRESF5,&
   & ILEVB_AK,&
   & LLBELO_AK,LLBELS_AK,LLBLOW_AK,LLBLES_AK)  

  ILEVB(1:ILEN,1:IMXKAK,:,ICOUNT)=ILEVB_AK(1:ILEN,1:IMXKAK,:)
  LLBELO(1:ILEN,1:IMXKAK,ICOUNT)=LLBELO_AK(1:ILEN,1:IMXKAK)
  LLBELS(1:ILEN,1:IMXKAK,ICOUNT)=LLBELS_AK(1:ILEN,1:IMXKAK)
  LLBLOW(1:IMXKAK,ICOUNT)=LLBLOW_AK(1:IMXKAK)
  LLBLES(1:IMXKAK,ICOUNT)=LLBLES_AK(1:IMXKAK)

ENDDO
  
!*    2. GHG variables:
!        Interpolate model values to averaging kernel  pressures 
!        and calculate model equivalent of obs as SUM(pgrg*wak)

JGHG=0
DO JGEMS=1,NGEMS
  IF(YDGEMSPROF5(1,JGEMS)%CSPECIES=='GHG') THEN
    JGHG=JGHG+1
    IF(LLGHG(JGHG)) THEN
      IF(INPP+1 > KDPP) CALL ABOR1('GHG_AK_AD: KDPP EXCEEDED, GHG')
      INPP=INPP+1

      ALLOCATE(ZGEMSMODEL(KDLEN,0:NFLEVG))
      ALLOCATE(ZGEMSMODEL5(KDLEN,0:NFLEVG))
      ALLOCATE(ZGEMS_AK(KDLEN,IMXKAK))
      DO JLEN=1,ILEN
        ZGEMSMODEL(JLEN,:)=0.0_JPRB
        ZGEMSMODEL5(JLEN,0)= &
         & YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)*ZMMR2VMRCH4
        ZGEMSMODEL5(JLEN,1:NFLEVG)= &
         & YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(:)*ZMMR2VMRCH4
      ENDDO

      DO JLEN=1,ILEN
        ZDPSUM5(JLEN)=0.0_JPRB
        DO IAK=1,IMXKAK
          IF (IAK == 1) THEN
            ZDP5(JLEN,IAK)=ZPRESO_AK(JLEN,IAK)
          ELSE
            ZDP5(JLEN,IAK)=ZPRESO_AK(JLEN,IAK)-ZPRESO_AK(JLEN,IAK-1)
          ENDIF
          ZDPSUM5(JLEN) = ZDPSUM5(JLEN) + (ZDP5(JLEN,IAK)*ZRGI)
        ENDDO
      ENDDO
      
      
      ZGEMS_AK(:,:)=0.0_JPRB
      ZXGEMS(:,:,:)=0.0_JPRB
      DO JLEN=1,ILEN
        ZXGEMS(JLEN,1,INPP) = ZXGEMS(JLEN,1,INPP) + PXGEMS(JLEN,1,INPP)/ZDPSUM5(JLEN)
        DO IAK=1,IMXKAK
          ZGEMS_AK(JLEN,IAK)=ZGEMS_AK(JLEN,IAK) + &
           &     ZXGEMS(JLEN,1,INPP)*PWAK(JLEN,1,IAK) *(ZDP5(JLEN,IAK)*ZRGI)
        ENDDO
      ENDDO
      
      CALL PPQAD(KDLEN,1,ILEN,NFLEVG,IMXKAK,1,ILEVB,&
       & PRESO,LLBELS,LLBLES,&
       & PXP,PXPD,ZGEMSMODEL,&
       & ZGEMS_AK,&
       & PXP5,PXPD5,ZGEMSMODEL5)  
      
      DO JLEN=1,ILEN
        YDGEMSPROF(JLEN,JGEMS)%MODELPROF(1:NFLEVG)=&
        & YDGEMSPROF(JLEN,JGEMS)%MODELPROF(1:NFLEVG)+ZGEMSMODEL(JLEN,1:NFLEVG)*ZMMR2VMRCH4
      ENDDO

      DEALLOCATE(ZGEMS_AK)
      DEALLOCATE(ZGEMSMODEL)
      DEALLOCATE(ZGEMSMODEL5)
    ENDIF
  ENDIF
ENDDO

DEALLOCATE(ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)
DEALLOCATE(ILEVB_AK,LLBELO_AK,LLBELS_AK,LLBLOW_AK,LLBLES_AK)

9 CONTINUE
IF (LHOOK) CALL DR_HOOK('GHG_AK_AD',1,ZHOOK_HANDLE)
END SUBROUTINE GHG_AK_AD
