SUBROUTINE GPSZEN_DELAYTL(KDLEN,KLEN,KDCOUNT,KCOUNT,&
 &                      PRESH, PGEOH, PTF, PQF, &
 &                      POROGOB,PRESOB,&
 &                      KDPP,PZEN_DELAYTL,&
 &                      PRESH5,PGEOH5,PTF5,PQF5,&
 &                      PRESOB5, &
 &                      PZHD_DELAYTL, PZWD_DELAYTL)

! Tangent linear model of the observation operator GPSZEN_DELAY
! See GPSZEN_DELAY for details.
! 14/02/2004  P. Poli        Original code
! 20/01/2006  P. Poli        Added T.L. of ZHD and ZWD as optional outputs

USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG, RMV, RMD
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KDLEN, KLEN,KDCOUNT,KCOUNT(KDLEN),KDPP
REAL(KIND=JPRB), INTENT(IN) :: PRESH(KDLEN,0:NFLEVG),PGEOH(KDLEN,0:NFLEVG)
REAL(KIND=JPRB), INTENT(IN) :: PTF(KDLEN,NFLEVG),PQF(KDLEN,NFLEVG)
REAL(KIND=JPRB), INTENT(IN) :: PRESH5(KDLEN,0:NFLEVG),PGEOH5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB), INTENT(IN) :: PTF5(KDLEN,0:NFLEVG),PQF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB), INTENT(IN) :: POROGOB(KDLEN),PRESOB(KDLEN,KDCOUNT)
REAL(KIND=JPRB), INTENT(IN) :: PRESOB5(KDLEN,KDCOUNT)
REAL(KIND=JPRB), INTENT(OUT) :: PZEN_DELAYTL(KDLEN,KDCOUNT,KDPP)
REAL(KIND=JPRB), INTENT(OUT), OPTIONAL :: PZHD_DELAYTL(KDLEN,KDCOUNT,KDPP)
REAL(KIND=JPRB), INTENT(OUT), OPTIONAL :: PZWD_DELAYTL(KDLEN,KDCOUNT,KDPP)

! local variables

INTEGER(KIND=JPIM) :: JOBS,JDATA,JLEV,ILEVP1
REAL(KIND=JPRB) :: ZREFRACT,ZREFRACTA,ZREFRACTB,ZHUPPER,ZHLOWER
REAL(KIND=JPRB) :: ZPUPPER,ZPLOWER,ZPLOWERB,ZKAPPA,ZEPSILON
REAL(KIND=JPRB) :: ZREFRACT5,ZREFRACTA5,ZREFRACTB5,ZHUPPER5,ZHLOWER5
REAL(KIND=JPRB) :: ZPUPPER5,ZPLOWER5,ZPLOWERB5,ZKAPPA5
REAL(KIND=JPRB), PARAMETER :: AVAL = 7.76E-7_JPRB
REAL(KIND=JPRB), PARAMETER :: BVAL = 3.73E-3_JPRB
LOGICAL :: LL_FLAG_ZHD, LL_FLAG_ZWD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

!-------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPSZEN_DELAYTL',0,ZHOOK_HANDLE)

! 1. initialise the variables :
! ratio of water-vapour mass/dry-air mass

ZEPSILON = RMV/RMD
PZEN_DELAYTL(:,:,:) = 0.0_JPRB 

LL_FLAG_ZHD=.FALSE.
IF (PRESENT(PZHD_DELAYTL)) THEN
   LL_FLAG_ZHD=.TRUE.
   PZHD_DELAYTL(:,:,:) = 0.0_JPRB
ENDIF
LL_FLAG_ZWD=.FALSE.
IF (PRESENT(PZWD_DELAYTL)) THEN
   LL_FLAG_ZWD=.TRUE.
   PZWD_DELAYTL(:,:,:) = 0.0_JPRB
ENDIF

!-------------------------------------------------------------------------


! assume that the refractivity varies exponentially between 
! the half-levels, as a function of geopotential height. Also assume
! the vertical integral of refrac.(N) w.r.t geopotential height ~= 
! integral w.r.t. geometric height.     

DO JDATA = 1,KDCOUNT
  DO JLEV = 0,NFLEVG-1
    ILEVP1 = JLEV+1

    DO JOBS = 1, KLEN
      IF(JDATA > KCOUNT(JOBS)) CYCLE
      
! assume temp and humidity are constant between half-levels and
! calculate their contribution to the refractivity term
      ZREFRACTA5 = AVAL/PTF5(JOBS,ILEVP1)
      ZREFRACTB5 = BVAL*PQF5(JOBS,ILEVP1)/ &
    &           (PTF5(JOBS,ILEVP1)**2 * &
    &            (ZEPSILON+(1.0_JPRB-ZEPSILON)*PQF5(JOBS,ILEVP1)))
      ZREFRACT5  = ZREFRACTA5 + ZREFRACTB5

      ZREFRACTA = -AVAL/PTF5(JOBS,ILEVP1)**2.*PTF(JOBS,ILEVP1)
      ZREFRACTB = -2.*BVAL*PQF5(JOBS,ILEVP1)*PTF(JOBS,ILEVP1)/ &
    &             (PTF5(JOBS,ILEVP1)**3 * &
    &              (ZEPSILON+(1.0_JPRB-ZEPSILON)*PQF5(JOBS,ILEVP1)))+ &
    &             BVAL*ZEPSILON*PQF(JOBS,ILEVP1)/ &
    &             (PTF5(JOBS,ILEVP1)**2 * &
    &              (ZEPSILON+(1.0_JPRB-ZEPSILON)*PQF5(JOBS,ILEVP1))**2 )
      ZREFRACT  = ZREFRACTA + ZREFRACTB

! skip the top 0.01 Pa, to avoid division by zero later
! note, PGEOH5(JOBS,0)/RG is roughly 75,000 m in the L60 model.
      ZPUPPER5 = MAX(PRESH5(JOBS,JLEV),0.01_JPRB) 
      ZPLOWER5 = PRESH5(JOBS,ILEVP1)
      ZHUPPER5 = PGEOH5(JOBS,JLEV)/RG
      ZHLOWER5 = PGEOH5(JOBS,ILEVP1)/RG
      ZKAPPA5 = LOG(ZPLOWER5/ZPUPPER5)/(ZHUPPER5-ZHLOWER5)

      IF (ZPUPPER5 == 0.01_JPRB) THEN
        ZPUPPER = 0.0_JPRB
      ELSE
        ZPUPPER = PRESH(JOBS,JLEV)
      ENDIF
      ZPLOWER = PRESH(JOBS,ILEVP1)
      ZHUPPER = PGEOH(JOBS,JLEV)/RG
      ZHLOWER = PGEOH(JOBS,ILEVP1)/RG
      ZKAPPA = (ZPLOWER/ZPLOWER5 - ZPUPPER/ZPUPPER5)/(ZHUPPER5-ZHLOWER5) - &
      &        LOG(ZPLOWER5/ZPUPPER5)/((ZHUPPER5-ZHLOWER5)**2) * &
      &        (ZHUPPER-ZHLOWER)

! this will extrapolate/interpolate pressure to the station pressure      
      IF(ZPUPPER5 >= PRESOB5(JOBS,JDATA)) CYCLE
      IF(ILEVP1 == NFLEVG .OR. &
     &   PRESH5(JOBS,ILEVP1) > PRESOB5(JOBS,JDATA)) THEN
         ZPLOWERB = PRESOB(JOBS,JDATA)
         ZPLOWERB5 = PRESOB5(JOBS,JDATA)
      ELSE
         ZPLOWERB = PRESH(JOBS,ILEVP1)
         ZPLOWERB5 = PRESH5(JOBS,ILEVP1)
      ENDIF

! add to total zenith delay integration, station by station
! assume pressure varies exponentially with geopotential height
      PZEN_DELAYTL(JOBS,JDATA,1) = PZEN_DELAYTL(JOBS,JDATA,1) + &
     &    ZREFRACT * (ZPLOWERB5-ZPUPPER5)/ZKAPPA5 + &
     &    ZREFRACT5 * ((ZPLOWERB -ZPUPPER )*ZKAPPA5 - &
     &                 (ZPLOWERB5-ZPUPPER5)*ZKAPPA  )/&
     &                (ZKAPPA5**2)

      IF (LL_FLAG_ZHD) PZHD_DELAYTL(JOBS,JDATA,1) = &
     &                PZHD_DELAYTL(JOBS,JDATA,1) + &
     &    ZREFRACTA * (ZPLOWERB5-ZPUPPER5)/ZKAPPA5 + &
     &   ZREFRACTA5 * ((ZPLOWERB -ZPUPPER )*ZKAPPA5 - &
     &                 (ZPLOWERB5-ZPUPPER5)*ZKAPPA  )/&
     &                (ZKAPPA5**2)

      IF (LL_FLAG_ZWD) PZWD_DELAYTL(JOBS,JDATA,1) = &
     &                PZWD_DELAYTL(JOBS,JDATA,1) + &
     &    ZREFRACTB * (ZPLOWERB5-ZPUPPER5)/ZKAPPA5 + &
     &   ZREFRACTB5 * ((ZPLOWERB -ZPUPPER )*ZKAPPA5 - &
     &                 (ZPLOWERB5-ZPUPPER5)*ZKAPPA  )/&
     &                (ZKAPPA5**2)

    ENDDO

  ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPSZEN_DELAYTL',1,ZHOOK_HANDLE)
END SUBROUTINE GPSZEN_DELAYTL

