SUBROUTINE HDEPART(KDOBS,KLEN,KDBDY,KMXBDY,KTYPE,&
 & KCMBDY,KVNM,PHOFX)  

!**** *HDEPART* - Compute and save departures

!     E. ANDERSSON            ECMWF          01/04/99

!     PURPOSE
!     -------

!   COMPUTE DEPARTURES

!**   INTERFACE
!     ---------
!     *CALL* *HDEPART(KDOBS,KLEN,KDBDY,KTYPE,
!                     KCMBDY,KVNM,PHOFX)*
!     WHERE KDOBS = First dimension, observations
!           KLEN  = Actual number of observations
!           KDBDY = Second dimension, data
!           KMXBDY= Actual maximum number of data.
!           KTYPE = Code type for TOVS, AIREP type
!           KCMBDY= Number of data in each report
!           KVNM  = body variable numbers
!           PHOFX = On input:  Model equivalents of obs: H(x).
!                   On output: Normalized departure. (OBS-H(x))/OBS_ERROR

!        HDEPART is called from HOP

!**   METHOD
!     ------
!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Erik Andersson  01/04/99 : Original
!     Antje Dethof    28/04/03 : Add bias correction for REO3 data
!     Y.Tremolet    18-Feb-2005 Generate "truth" for identical twin exp
!     N.Bormann       24/07/05 : Bias correction for limb radiances
!     S.Saarinen      16/08/06 : More vectorization by use of MBODYJOBS
!     S.Healy         15/08/06 : Fix for missing hi-res GPSRO departure 
!     Y.Tremolet    22-Feb-2008 Obs space adjoint test
!     A. Inness     19-Aug-2009 Add Varbc for GRG
!     A.Geer        18-Aug-2009 Rationalise use of UPDATE table (hires, lores etc.)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMVAR   , ONLY : NUPTRA, NSIM4D, NSIM4DL
USE YOMCMA   , ONLY : NRESUPD
USE YOMCT0   , ONLY : LCANARI  ,LOBSREF  ,LOBSC1   ,NCONF
USE YOMANCS  , ONLY : RMDI
USE YOMVRTL  , ONLY : LOBSTL
USE YOMVNMB  , ONLY : NVNUMB
USE varbc_airep, ONLY: LBC_AIREP
USE YOMDB
USE QALBAR   , ONLY : NRESECR
USE YOMLUN   , ONLY : NULERR, NULOUT
USE YOMDIMO  , ONLY : NOBTOTG
USE YOMCOSJO , ONLY : LADJOBS, MOBSENS
USE TESTVAR_MIX, ONLY: LTWINTRUTH, TWINDEPART, LTESTADJH, LTESTADJB
USE YOMCOCTP , ONLY : NAIREP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDOBS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDBDY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXBDY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE
INTEGER(KIND=JPIM),INTENT(IN)    :: KCMBDY(KDOBS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVNM(KDOBS,KDBDY) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHOFX(KDOBS,KDBDY) 
INTEGER(KIND=JPIM) :: JOBS,JBODY
INTEGER(KIND=JPIM) :: IIVNM,IBODY,IRESECR
REAL(KIND=JPRB) :: ZOBSV,ZOBSE,ZBIAS,ZDEP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "openmp_obs.h"

!     ------------------------------------------------------------------
!*       0.       Define which departure type to implement
!                 ----------------------------------------

IF (LHOOK) CALL DR_HOOK('HDEPART',0,ZHOOK_HANDLE)

IF (LCANARI) THEN
  IF (NRESECR == 1) THEN
    IRESECR=MDBOMF
  ELSE
    IRESECR=MDBOMN
  ENDIF
ELSE
  IRESECR=MDBOMN
ENDIF

!     ------------------------------------------------------------------
!*       1.       Departures (obs-model)
!                 ----------------------

!*          1.1   Loop the data
!*                THIS LOOP VECTORIZES (on Fujitsu)

DO IBODY=1,MLNKH2B(KLEN+1)-1

!*          1.2   The observations
  JOBS = MBODYJOBS(IBODY)
  JBODY = IBODY + 1 - MLNKH2B(JOBS)

    IIVNM=KVNM(JOBS,JBODY)
    IF(IIVNM == 0) CYCLE

!*          1.2.1 Pick up observed value, bias and obs error
!                 As well as VarQC parameters
    ZOBSV=ROBODY(IBODY,MDBVAR)
    ZOBSE=ROBODY(IBODY,MDBFOE)
    IF(LOBSTL) THEN
      ZBIAS=0.0_JPRB
    ELSEIF(IIVNM == NVNUMB(50)      ) THEN   ! brightness temp.
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(68)) THEN   ! reo3 O3
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(80)) THEN   ! reo3 NOx
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(81)) THEN   ! reo3 SO2
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(82)) THEN   ! reo3 CO
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(83)) THEN   ! reo3 For
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(84)) THEN   ! reo3 GO3
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(95)) THEN   ! reo3 ch4
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB(79)      ) THEN   ! limb radiances
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB( 6)      ) THEN   ! TCWV observations
      ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSEIF(IIVNM == NVNUMB( 8).AND.&
   &       KTYPE == NAIREP          ) THEN   ! AIREP T observations
      IF(LBC_AIREP)                     THEN
        ZBIAS=ROBODY(IBODY,MDBTORB)
      ELSE
        ZBIAS=0.0_JPRB
      ENDIF
    ELSEIF(IIVNM == NVNUMB(78)) THEN   ! aerosol optical depth (AOD)
       ZBIAS=ROBODY(IBODY,MDBTORB)
    ELSE
      ZBIAS=0.0_JPRB
    ENDIF

    IF(ZOBSV == RMDI .OR. ZBIAS == RMDI .OR. PHOFX(JOBS,JBODY) == RMDI) THEN
      ZDEP=RMDI
    ELSE

!*                Over-write observed value

      IF (LTWINTRUTH) THEN
        ZOBSV=(PHOFX(JOBS,JBODY)+ZBIAS) + TWINDEPART

        ROBODY(IBODY,MDBVAR)=ZOBSV
      ENDIF

!*          1.2.2 Compute departure. Correct for bias


!     The parenthesis in the next two lines do matter
!     for identical twin experiments to be identical.
      ZDEP=ZOBSV-(PHOFX(JOBS,JBODY)+ZBIAS)
      
!*          1.2.3 For the incremental case, set ZDEP to
!                 (y-H(xb)-Hs(xs)+Hs(xsb))

      IF(NCONF == 131 .AND. .NOT.LOBSREF.AND.ZDEP /= RMDI) THEN
        IF(LOBSTL) THEN

! set the departure to ZERO for GPSRO when hi-res dep. missing

          IF (IIVNM == NVNUMB(76) .AND. ROBODY(IBODY,MDBIFC1(NUPTRA+1)) == RMDI) THEN 
             ZDEP = 0.0_JPRB
          ELSE
             ZDEP = ZDEP-ZOBSV+ROBODY(IBODY,MDBIFC1(NUPTRA+1))
          ENDIF 
    
        ELSE
!         Various changes to the use of the UPDATE table in ODB mean that
!         this will no longer work but it should be irrelevant anyway. It 
!         can be removed from the code in cycle 37.
          CALL ABOR1('This code should never be called. Contact A. Geer for more info.')
!          ZDEP = ZDEP-ROBODY(IBODY,MDBIFC2(NUPTRA+1))&
!           & + ROBODY(IBODY,MDBIFC1(NUPTRA+1))  
        ENDIF
      ENDIF
    ENDIF

!*          1.2.4 Current departure

    ROBODY(IBODY,IRESECR)=ZDEP

!*          1.2.5 High resolution departure in MDBIFC1
!                 If NUPTRA=0 save also as FG-departure in MDBOMF

    IF(LOBSC1) THEN
      IF(NUPTRA /= NRESUPD)  ROBODY(IBODY,MDBIFC1(NUPTRA+1))=ZDEP
      IF(NUPTRA == 0)        ROBODY(IBODY,MDBOMF) = ZDEP

!*          1.2.7 Departure from low resolution analysis
    ELSEIF(NCONF == 131 .AND. NSIM4D == NSIM4DL) THEN
      ROBODY(IBODY,MDBIFC2(NUPTRA+1))=ZDEP
    ENDIF

!   Save Hb for adjoint test
    IF (LTESTADJB) THEN
      ROBODY(IBODY,MDB_OBS_DIAGS(MOBSENS,NUPTRA+1))=PHOFX(JOBS,JBODY)
    ENDIF
!*          1.2.8 Redefine phofx as normalized departure

    IF(ZDEP == RMDI .OR. ZOBSE == RMDI) THEN
      PHOFX(JOBS,JBODY)=RMDI
    ELSE
      IF (LADJOBS.OR.LTESTADJH) THEN
        PHOFX(JOBS,JBODY)=PHOFX(JOBS,JBODY)/ZOBSE
      ELSE
        PHOFX(JOBS,JBODY)=ZDEP/ZOBSE
      ENDIF
    ENDIF

!   IF(IIVNM == NVNUMB(80) ) THEN   ! reo3 NOX
!     IF(IEEE_IS_NAN(PHOFX(JOBS,JBODY))) THEN
!       WRITE(NULERR,*)'antje HDEPART: Obs value=',ZOBSV,' Departure=',ZDEP,&
!          & ' Obs err=',ZOBSE,' H(x)=',PHOFX(JOBS,JBODY),' bias=',ZBIAS
!       CALL ABOR1('HDEPART: PROBLEM WITH NOX OPERATOR. Departure=NAN ')
!     ENDIF
!   endif

!   IF(IIVNM == NVNUMB(82) ) THEN   ! reo3 CO
!     IF(IEEE_IS_NAN(PHOFX(JOBS,JBODY))) THEN
!       WRITE(NULERR,*)'antje HDEPART: Obs value=',ZOBSV,' Departure=',ZDEP,&
!          & ' Obs err=',ZOBSE,' H(x)=',PHOFX(JOBS,JBODY),' bias=',ZBIAS
!       CALL ABOR1('HDEPART: PROBLEM WITH CO OPERATOR. Departure=NAN ')
!     ENDIF
!   endif

!*          Print current departure and obs value if very few obs.
    IF (NOBTOTG<=20.AND.ZDEP/=RMDI) THEN
      WRITE(NULERR,*)'HDEPART: Obs value=',ZOBSV,' Departure=',ZDEP,&
           & ' Obs err=',ZOBSE,' H(x)=',PHOFX(JOBS,JBODY)
    ENDIF

ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('HDEPART',1,ZHOOK_HANDLE)
END SUBROUTINE HDEPART
