SUBROUTINE HINTH(KDLEN,KLEN,KDBDY,KMXBDY,KCMBDY,KOBSTYPE,&
 & PAPHIL5,PINTHUV,KVNMRQ)  

!**** *HINTH* - Find interpolation heights

!     E. ANDERSSON            ECMWF          01/04/99

!     PURPOSE
!     -------

!     Find out height to interpolate surface winds to.

!**   INTERFACE
!     ---------
!     *CALL* *HINTH(KDLEN,KLEN,KDBDY,KCMBDY,KOBSTYPE,&
!                  &PAPHIL5,PINTHUV,KVNMRQ)
!     WHERE KDLEN   = First dimension, observations
!           KLEN    = Actual number of observations
!           KDBDY   = Second dimension, data
!           KMXBDY  = Maximum number of data
!           KCMBDY  = Number of data in each report
!           KOBSTYPE= Observation type
!           PAPHIL5 = Height of lowest model level
!           PINTHUV = Interpolation height (m2/s2)
!           KVNMRQ  = Variable number from NVNUMB table

!        HINTH is called from HOP/TL/AD

!**   METHOD
!     ------
!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Erik Andersson  01/04/99 : Original
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCOCTP , ONLY : NTEMP    ,NPILOT   ,NDRIBU   ,NSYNOP
USE YOMCST   , ONLY : RG
USE YOMANCS  , ONLY : RMDI
USE YOMDB
USE YOMVNMB  , ONLY : NVNUMB
USE YOMCT0B  , ONLY : LECMWF

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDBDY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXBDY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCMBDY(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSTYPE 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIL5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PINTHUV(KDLEN) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVNMRQ(KDLEN,KDBDY) 
INTEGER(KIND=JPIM) :: JBDY,JOBS
REAL(KIND=JPRB) :: ZANMH
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"

#include "openmp_obs.h"
!     ------------------------------------------------------------------
!*    1.       Use anemometer height of CMA to determine interpolation height
!              -------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('HINTH',0,ZHOOK_HANDLE)
PINTHUV(1:KLEN) = 10._JPRB*RG

IF(ANY(KOBSTYPE == (/NSYNOP,NDRIBU,NTEMP,NPILOT/))) THEN
  DO JOBS=1,KLEN
    ZANMH = ROBHDR(JOBS,MDBHOANS)
!   ZBARH = ROBHDR(JOBS,MDBHOBAS)

    IF(ZANMH /= RMDI) THEN
      PINTHUV(JOBS) = ZANMH*RG
    ENDIF

  ENDDO
ENDIF

IF( .NOT.LECMWF .AND. KOBSTYPE /= NSYNOP ) THEN
  DO JOBS=1,KLEN
    PINTHUV(JOBS) = 10._JPRB*RG
  ENDDO
  IF (LHOOK) CALL DR_HOOK('HINTH',1,ZHOOK_HANDLE)
  RETURN
ENDIF

!*          2.1   Adjust the list of VNM-requests, according to ship height.
!                 The idea is to use U-operator within model domain and
!                 U10-operator within the model's surface layer
DO JBDY=1,KMXBDY
  DO JOBS = 1,KLEN
    IF(JBDY > KCMBDY(JOBS)) CYCLE
    IF(PINTHUV(JOBS) > PAPHIL5(JOBS)) THEN
      IF(KVNMRQ(JOBS,JBDY) == NVNUMB(56)) KVNMRQ(JOBS,JBDY)=NVNUMB(01)
      IF(KVNMRQ(JOBS,JBDY) == NVNUMB(57)) KVNMRQ(JOBS,JBDY)=NVNUMB(02)
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('HINTH',1,ZHOOK_HANDLE)
END SUBROUTINE HINTH
