!OPTIONS NODEBUG
SUBROUTINE HJO(KDOBS,KDBDY,KSET,KCMBDY,KOBSTYPE,KCDTYP,&
 & KVAR,KVNMRQ,PHOFX,KSENSOR,KTIME,KSCAN)

!**** *HJO* - Compute and store Jo

!     E. ANDERSSON            ECMWF          01/04/99

!     PURPOSE
!     -------

!    Calculate the contribution to the cost function incorporating variational
!    quality control

!**   INTERFACE
!     ---------
!     *CALL* *HJO(KDOBS,KDBDY,KSET,KCMBDY,KOBSTYPE,KCDTYP,&
!             &KVAR,PHOFX)
!     WHERE KDOBS = First dimension
!           KDBDY = Second dimension
!           KSET  = Observation set number
!           KCMBDY= Number of data in each report
!           KOBSTYPE=Observation type.
!           KCDTYP= Observation code type.
!           KVAR  = Variable number from NVAR table
!           KVNMRQ= Variable number from NVNUMB table
!           PHOFX = Normalized departure. (OBS-H(x))/OBS_ERROR
!           POROG = Surface geopotential
!           KSENSOR= Satellite Sensor
!           KSCAN= 1 or 2, second scan if LOBSCOR
!        HJO is called from HOP

!**   METHOD
!     ------
!**   EXTERNALS
!     ---------
!**   MODIFICATIONS
!     -------------
!     Erik Andersson  01/04/99 : Original
!     C.Cardinali     05/05/99 : Relax varQC for targeted dropsondes
!     Xu Li           23/05/00 : Set and associate VarQC gross error with VTPR
!     S. Healy        27/10/04 : Introduce GPSRO to JO tables
!     Niels Bormann   23/12/04 : LIMBGRP-based JO table entries for LIMB obs
!     S.Saarinen      18/08/06 : More vectorization by use of MBODYJOBS
!     Gabor Radnoti   03/03/06 : Correlated obs errors
!     H.Hersbach      22/11/07 : Flag non-closest ambiguities for scatterometer wind
!     C.Payan         27/04/07 : 0.25_jprb replaced by WSCAT5_FAC in NSCAT5 case
!     K. Scipal       10/04/08 : assured that combination of Jo's under point 1.4 is only carried out if
!                                scat information is wind and not soil moisture
!        Y.Tremolet    22-Feb-2008 Obs space adjoint test
!        Y.Tremolet    29-May-2008 Print Jo table by bins
!     C.Payan         12/06/06 : bf for scat (LLFIN definition and Jo calculation)
!     A. Fouilloux   31/03/10: use new codetype for MERIS data

!  -----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARDIMO  , ONLY : JPXVAR
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LOBSREF  ,LOBSC1
USE YOMANCS  , ONLY : RMDI     ,RSQRT2PI ,NMDI
USE YOMLUN   , ONLY : NULERR   ,NULOUT
USE YOMCOCTP , ONLY : NSCATT   ,NSYNOP   ,NDRIBU   ,NTEMP    ,NTDROP   ,NGTHRB   ,NSATEM   ,&
 & MSQBYCTP ,NSATOB  ,NSCAT5   ,NSCAT6   ,NLIMB  ,NSSMI, NTCWC
USE YOMCOSJO , ONLY : LQSCATT  ,LVARQCG  ,LVARQC   ,NITERQC  ,&
 & LTC      ,RAQC     ,RLQC     ,FJOS     ,FJOADJ, &
 & NTCOBS_L ,NTCONM_L ,RTCNDP   ,RTCNDPHR ,NTCIXG_L ,&
 & LVARQC_RAD1C ,RAQC_RAD1C ,RLQC_RAD1C   ,&
 & JOT      ,NVAR_Z,NVAR_PS,NVAR_U,NVAR_RAD,&
 & LQVARQC  ,LOBSCOR,  OBSCOR_SCALPR,  NBCORBLK,  OBSCOR_EIGVAL,  OBSCOR_TRUN, &
 & NRITZ_OBSCOR, WSCAT5_FAC ,WSCAT6_FAC, LHUBERMINQC, RHUBERLEFT, RHUBERRIGHT, NPRTBINS, OBSCOR_AUX
USE YOMVAR   , ONLY : NITER    ,NSIM4D   ,NITER4D  ,NUPTRA   ,&
 & NSIM4DL   ,MUPTRA  
USE YOMOBSET , ONLY : MLNSET   ,MMXBDY   ,MTYPOB
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMVNMB  , ONLY : NVNUMB
USE YOMSCC   , ONLY : LTARGDROP
USE YOMCMBDY , ONLY : NCMDACBP ,NCMDACOC
USE YOMDB
USE TESTVAR_MIX, ONLY: LTESTADJH
USE OML_MOD  , ONLY :OML_SET_LOCK, OML_UNSET_LOCK
USE RANDOM_NUMBERS_MIX

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDOBS
INTEGER(KIND=JPIM),INTENT(IN)    :: KDBDY
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCMBDY(KDOBS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSTYPE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCDTYP(KDOBS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVAR(KDOBS,KDBDY) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVNMRQ(KDOBS,KDBDY) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHOFX(KDOBS,KDBDY) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSENSOR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSCAN
REAL(KIND=JPRB), DIMENSION(KDOBS,KDBDY) :: ZFJOS,ZQCW,ZOM1DEP,ZFJOS_HR,ZSIGNVAL
INTEGER(KIND=JPIM),INTENT(IN)    :: KTIME(KDOBS) 
REAL(KIND=JPRB), DIMENSION(KDOBS,KDBDY) :: ZFJOAD
REAL(KIND=JPRB), ALLOCATABLE :: ZRANDOM(:)
INTEGER(KIND=JPIM) :: JOBS,JBODY, IOBSCOR_COUNT(NBCORBLK)
INTEGER(KIND=JPIM) :: ILEN,IBODY,IIVAR,IIVNM,IADD,IMXBDY,ICMONM,IGRP&
 & ,IFL,IFLAG,IABNOB,IP,ICHAN,IX(1),IXG,ICTPSQ,ICDT,IOBT,ILEVTYPE &
 & ,IOBSTYPE,ICDTYP  ,IMASK,JEV 
INTEGER(KIND=JPIM), DIMENSION(KDOBS) :: ICLOSEST
REAL(KIND=JPRB) :: ZJ1P,ZJ2P&
 & ,ZA,ZL,ZPNGE,ZPGE,ZNORM,ZAOB,ZEMJON&
 & ,ZJO_OFFSET,ZDEPHR,ZFJON,ZFJOQC,ZCLOSEST,ZPRES,ZHUBERRIGHT,ZHUBERLEFT &
 & ,ZSIGN,ZABSDEP,ZHUB,ZHUB2,ZOBSTEST, ZQ
LOGICAL :: LL_AMBIG_REM,LLITERQC,LLVARQC,LLU,LLV,LLSM,LLUPDVQC,LLQVARQC_MIN
LOGICAL :: LLERROR,LLFIN,LLACT,LLHUBERQC
TYPE(randomNumberStream) :: YL_RANDOM_STREAM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "nflgdse.intfb.h"
#include "cmoctmap.intfb.h"
#include "setran.intfb.h"
#include "fcobs.h"
#include "openmp_obs.h"
!     ------------------------------------------------------------------

!        1.       Cost function
!                 -------------

!           1.0.1 Preset
IF (LHOOK) CALL DR_HOOK('HJO',0,ZHOOK_HANDLE)
ZQCW(:,:)=1.0_JPRB
ZFJOS(:,:)=RMDI
ZFJOAD(:,:)=RMDI

ILEN=MLNSET(KSET)
IMXBDY=MMXBDY(KSET)
LL_AMBIG_REM=KOBSTYPE == NSCATT .AND. .NOT.LQSCATT

!           1.0.2 When to do Variational quality control.

LLITERQC = NITER4D >= NITERQC .AND. NITERQC <= NITER &
 & .AND. .NOT.LOBSREF .AND. LVARQCG  
LLQVARQC_MIN=LQVARQC .AND. .NOT.LOBSC1

IF(LLQVARQC_MIN) ZFJOS_HR(:,:)=RMDI
ZSIGNVAL(:,:)=1.0_JPRB

!           1.1   Time correlation. The cost-function calculation is
!                 deferred until all time slots are available.
!      This is done by storing PHOFX for later use, then setting PHOFX=RMDI

IF(LTC .AND. NTCOBS_L /= 0 .AND. &
   & (KOBSTYPE == NSYNOP .OR. KOBSTYPE == NDRIBU)) THEN  

!*!OCL NOVREC
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)

    ICMONM=ROBHDR(JOBS,MDBONM)

    IF(PHOFX(JOBS,JBODY) == RMDI) CYCLE   ! If ECMA in last trajectory
    IIVAR=KVAR(JOBS,JBODY)

!*          1.1.1 Is current obs. on the correlated list?
    IF(IIVAR == NVAR_Z .OR. IIVAR == NVAR_PS) THEN   ! Ps and Z only.
      IX=MINLOC(ABS(NTCONM_L(1:NTCOBS_L)-ICMONM))
      IF(NTCONM_L(IX(1)) == ICMONM) THEN
!*          1.1.2 Store current departure in RTCNDP (global array).
        IXG=NTCIXG_L(IX(1))
        RTCNDP(IXG)=PHOFX(JOBS,JBODY)
        PHOFX(JOBS,JBODY)=RMDI

!*          1.1.3 Quadratic VarQC requires High_Res dep. Store in RTCNDPHR.
        IF(LLQVARQC_MIN) THEN
          ZDEPHR=ROBODY(IBODY,MDBIFC1(NUPTRA+1))
          RTCNDPHR(IXG)=ZDEPHR/ROBODY(IBODY,MDBFOE)
        ENDIF
      ENDIF
    ENDIF
  ENDDO
ENDIF

!           1.2.1 Trajectory: Store VarQC parameters in OBD.
IF(LOBSC1 .AND. KSCAN==1) THEN
!*!OCL NOVREC
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)

    IF(PHOFX(JOBS,JBODY) == RMDI) CYCLE

    IIVAR=KVAR(JOBS,JBODY)

    ZA=RAQC(IIVAR,KOBSTYPE)
    ZL=RLQC(IIVAR,KOBSTYPE)

    IF(IIVAR == NVAR_RAD .AND. (KCDTYP(JOBS) == NGTHRB .OR. KCDTYP(JOBS) == NSSMI &
       &                   .OR. KCDTYP(JOBS) == NTCWC)) THEN   ! 1C-Radiances    
      ICHAN=NINT(ROBODY(IBODY,MDBPPP))
      ZA=RAQC_RAD1C(KSENSOR,ICHAN)
      ZL=RLQC_RAD1C(KSENSOR,ICHAN)
        
! Relax VarQC for dropsonde winds
    ELSEIF(KOBSTYPE == NTEMP .AND. KCDTYP(JOBS) == NTDROP .AND. &
       & IIVAR == NVAR_U) THEN  
      IF(LTARGDROP) ZA=ZA/10000000.
    ENDIF

    ROBODY(IBODY,MDB_QC_A)=ZA
    ROBODY(IBODY,MDB_QC_PGE)=ZA
    ROBODY(IBODY,MDB_QC_L)=ZL
  ENDDO
ENDIF

IF (.NOT.LOBSC1 .AND. NITER4D==0 .AND. .FALSE.) THEN
  IF (ANY(KOBSTYPE==(/1,4,5,6/))) THEN
    DO JOBS=1,ILEN
      WRITE(NULOUT,*) 'obstype,codetype,ROBHDR(JOBS,MDB_OROGRAPHY) ', &
        & KOBSTYPE,KCDTYP(JOBS),ROBHDR(JOBS,MDB_OROGRAPHY)
    ENDDO
  ENDIF
ENDIF
!*          1.2.2  Effective departure
ZOM1DEP(1:ILEN,:)=PHOFX(1:ILEN,:)

!*          1.3   Cost by datum. If U add V.

!*!OCL NOVREC
DO IBODY=1,MLNKH2B(ILEN+1)-1
  JOBS = MBODYJOBS(IBODY)
  JBODY = IBODY + 1 - MLNKH2B(JOBS)

  IF(PHOFX(JOBS,JBODY) == RMDI) CYCLE

  IIVNM=KVNMRQ(JOBS,JBODY)
  LLU=IIVNM == NVNUMB(01) .OR. IIVNM == NVNUMB(56) .OR. IIVNM == NVNUMB(62)
  LLV=IIVNM == NVNUMB(02) .OR. IIVNM == NVNUMB(57) .OR. IIVNM == NVNUMB(61)

!*          1.3.1 Cost per datum, based on current departure.
  IF(LLV) THEN
    ZFJOS(JOBS,JBODY)=RMDI
  ELSEIF(LLU) THEN
    ZFJOS(JOBS,JBODY)=0.5_JPRB*PHOFX(JOBS,JBODY)*ZOM1DEP(JOBS,JBODY)&
     & +0.5_JPRB*PHOFX(JOBS,JBODY+1)*ZOM1DEP(JOBS,JBODY+1)  
  ELSE
    ZFJOS(JOBS,JBODY)=0.5_JPRB*PHOFX(JOBS,JBODY)*ZOM1DEP(JOBS,JBODY)
  ENDIF

!*          1.3.2 Quadratic VarQC is based on High-Res departure.
  IF(LLQVARQC_MIN) THEN
    ZDEPHR=ROBODY(IBODY,MDBIFC1(NUPTRA+1))
    ZFJON=(ZDEPHR/ROBODY(IBODY,MDBFOE))**2
    IF(LLU) THEN
      ZDEPHR=ROBODY(IBODY+1,MDBIFC1(NUPTRA+1))
      ZFJON=ZFJON+(ZDEPHR/ROBODY(IBODY+1,MDBFOE))**2
    ELSEIF(LLV) THEN
      ZFJON=RMDI
    ENDIF
    ZFJOS_HR(JOBS,JBODY)=0.5_JPRB*ZFJON
    ZSIGNVAL(JOBS,JBODY)=SIGN(1.0_JPRB,ZDEPHR)
  ELSE
    ZSIGNVAL(JOBS,JBODY)=SIGN(1.0_JPRB,ZOM1DEP(JOBS,JBODY))
  ENDIF
ENDDO

!*          1.4.1 Identify the closest scatterometer ambiguity
!*                at initial and final trajectory

!MUPTRA is the max of update of the trajectory for a current analysis and
! can be tuned by namelist (MUPTRA<=NRESUPD and =2 currently at MF)
LLFIN=(NUPTRA==MUPTRA)
IF(KOBSTYPE==NSCATT .AND. LOBSC1 .AND. (NUPTRA==0 .OR. LLFIN) .AND. KSCAN==1 ) THEN
  IFLAG=10                     ! departure from first-guess
  IF(LLFIN)IFLAG=11            ! departure from analysis

  ICLOSEST(:)=NMDI
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS  = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)
    IF(JBODY==1)ZCLOSEST=HUGE(ZCLOSEST)
      
    LLACT=IBITS(INT(ROBODY(IBODY,MDBDSTA)),NCMDACBP,NCMDACOC) == 1
    IF(KVNMRQ(JOBS,JBODY)/=NVNUMB(62)  ) CYCLE    ! ambiguous winds only
    IF(ZFJOS (JOBS,JBODY)==RMDI        ) CYCLE
    IF(LLFIN.AND. .NOT. LLACT) CYCLE    ! active    winds only

    IF(ZCLOSEST>ZFJOS(JOBS,JBODY)) THEN
       ZCLOSEST=ZFJOS(JOBS,JBODY)
       ICLOSEST(JOBS)=JBODY
    ENDIF
  ENDDO

  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS  = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)
    IF(.NOT.(JBODY==ICLOSEST(JOBS) .OR. JBODY==ICLOSEST(JOBS)+1)) THEN
      ROBODY(IBODY,MDBDEV1) = ZCHEV1(ROBODY(IBODY,MDBDEV1),ND1EVENT(IFLAG))
      IF(LLFIN) &
     &ROBODY(IBODY,MDBDSTA) = ZCHSTAT_REJECT(ROBODY(IBODY,MDBDSTA))
    ENDIF
  ENDDO
ENDIF

!*          1.4.2 For wind scatterometer Jo is a combination of first
!*                and second ambiguous winds.
!For QuikScat data, more than 2 winds are allowed
! so not sure this calculation is valid in this case yet

IF(LL_AMBIG_REM) THEN
  IP=4_JPIM
  DO JOBS=1,ILEN
    IF(KCMBDY(JOBS) < 4_JPIM) CYCLE
    IF (ZFJOS(JOBS,1)==RMDI    .OR. ZFJOS(JOBS,3)==RMDI .OR. &
     & (ZFJOS(JOBS,1)==0.0_JPRB.AND.ZFJOS(JOBS,3)==0.0_JPRB)) THEN
      IMASK=1_JPIM
    ELSE
      ZJ1P=ZFJOS(JOBS,1)**IP
      ZJ2P=ZFJOS(JOBS,3)**IP
      ZFJOS(JOBS,1)=((ZJ1P*ZJ2P)/(ZJ1P+ZJ2P))**(1.0_JPRB/IP)
      IMASK=3_JPIM
    ENDIF
!V case already treated in 1.3 part
    WHERE(KVNMRQ(JOBS,IMASK:)==NVNUMB(62)) ZFJOS(JOBS,IMASK:)=RMDI
  ENDDO
ENDIF

!*          1.4.3 Currently, for scatterometer data only allow wind and soilmoisture.

IF(KOBSTYPE == NSCATT) THEN
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)

    IIVNM=KVNMRQ(JOBS,JBODY)
    LLU =IIVNM == NVNUMB(01) .OR. IIVNM == NVNUMB(56) .OR. IIVNM == NVNUMB(62)
    LLV =IIVNM == NVNUMB(02) .OR. IIVNM == NVNUMB(57) .OR. IIVNM == NVNUMB(61)
    LLSM=IIVNM == NVNUMB(89) .OR. IIVNM == NVNUMB(90)
    IF (.NOT.(LLU.OR.LLV.OR.LLSM)) ZFJOS(JOBS,JBODY)=RMDI
  ENDDO
ENDIF

!*          1.5   Modification of Jo, due to VarQC.

IF(LLITERQC) THEN
  LLERROR = .FALSE.
!*!OCL NOVREC
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)

    IF(PHOFX(JOBS,JBODY) == RMDI) CYCLE

    IIVAR=KVAR(JOBS,JBODY)

    IF(IIVAR == NVAR_RAD .AND. (KCDTYP(JOBS) == NGTHRB &
      &                    .OR. KCDTYP(JOBS) == NSSMI .OR.  KCDTYP(JOBS) == NTCWC)) THEN   ! 1C-Radiances
      LLVARQC=LVARQC_RAD1C(KSENSOR)
    ELSE
      LLVARQC=LVARQC(IIVAR,KOBSTYPE)
    ENDIF

    LLHUBERQC=LHUBERMINQC(IIVAR,KOBSTYPE)

    IF((LLVARQC .OR. LLHUBERQC) .AND. ZFJOS(JOBS,JBODY) /= RMDI) THEN
      IF(LLQVARQC_MIN) THEN
        ZFJON=ZFJOS_HR(JOBS,JBODY)     !Constant during inner-loop iterations
      ELSE
        ZFJON=ZFJOS(JOBS,JBODY)
      ENDIF
    ENDIF

    IIVNM=KVNMRQ(JOBS,JBODY)
    LLU=IIVNM == NVNUMB(01).OR.IIVNM == NVNUMB(56).OR.IIVNM == NVNUMB(62)

    IF(LLVARQC .AND. ZFJOS(JOBS,JBODY) /= RMDI) THEN

!*          1.5.1 Get A and L.
      ZA=ROBODY(IBODY,MDB_QC_A)
      ZA=ROBODY(IBODY,MDB_QC_PGE)
      ZL=ROBODY(IBODY,MDB_QC_L)

      IF (ZA>=1.0_JPRB) THEN ! Loop is now vectorizable
        LLERROR = .TRUE.
        CYCLE
      ENDIF
!*          1.5.2 U and V are quality controlled together.
      IF(LLU) THEN
        ZPNGE=(1.0_JPRB-ZA)*(1.0_JPRB-ZA)
        ZPGE=1.0_JPRB-ZPNGE
        ZNORM=RSQRT2PI*RSQRT2PI/(4._JPRB*ZL*ZL)

!*          1.5.3 Single datum (i.e. not U and V)
      ELSE
        ZPNGE=1.0_JPRB-ZA
        ZPGE=1.0_JPRB-ZPNGE
        ZNORM=RSQRT2PI/(2.0_JPRB*ZL)
      ENDIF

      ZAOB=ZPGE*ZNORM/ZPNGE
      ZEMJON=EXP(-ZFJON)
      ZFJOQC=-LOG( (ZAOB + ZEMJON)/(ZAOB + 1.0_JPRB) )
      ZQCW(JOBS,JBODY)=1.0_JPRB-ZAOB/(ZAOB+ZEMJON)
      ZHUB=1.0_JPRB
    ELSEIF(LLHUBERQC .AND. ZFJOS(JOBS,JBODY) /= RMDI) THEN
      ZABSDEP=SQRT(2.0_JPRB*ZFJON)
      ZSIGN=ZSIGNVAL(JOBS,JBODY)
      ZPRES=ROBODY(IBODY,MDBPPP)
      IF (ZPRES /= RMDI) THEN
        IF (ZPRES>85000.) THEN
          ILEVTYPE=3
        ELSEIF (ZPRES>10000.) THEN
          ILEVTYPE=2
        ELSE
          ILEVTYPE=1
        ENDIF
      ELSE
        ILEVTYPE=3
      ENDIF
      IOBSTYPE=KOBSTYPE
      ICDTYP=KCDTYP(JOBS)
      CALL CMOCTMAP('CODETOSQ',NULOUT,IOBSTYPE,ICDTYP,IOBT,ICDT)
      ZHUBERLEFT=RHUBERLEFT(IIVAR,ICDT,IOBT,ILEVTYPE)
      ZHUBERRIGHT=RHUBERRIGHT(IIVAR,ICDT,IOBT,ILEVTYPE)

      IF ((ZSIGN<0.0_JPRB .AND. ZABSDEP<ZHUBERLEFT) .OR. & 
       &   (ZSIGN>0.0_JPRB .AND. ZABSDEP<ZHUBERRIGHT)) THEN
        ZQCW(JOBS,JBODY)=1.0_JPRB
        ZFJOQC=ZFJON
      ELSE
        IF (ZSIGN<0.0_JPRB) THEN
          ZHUB=ZHUBERLEFT   
        ELSE
          ZHUB=ZHUBERRIGHT
        ENDIF
        ZHUB2=ZHUB*ZHUB/2.0_JPRB
        ZFJOQC=ZHUB*ZABSDEP-ZHUB2
        ZQCW(JOBS,JBODY)=(ZHUB/ZABSDEP)
      ENDIF
    ENDIF

    IF((LLVARQC .OR. LLHUBERQC) .AND. ZFJOS(JOBS,JBODY) /= RMDI) THEN
      IF(LLQVARQC_MIN) THEN
        ZJO_OFFSET=ZFJOQC-ZFJON*ZQCW(JOBS,JBODY)
        ZFJOQC=ZFJOS(JOBS,JBODY)*ZQCW(JOBS,JBODY)+ZJO_OFFSET
      ENDIF
      ZFJOS(JOBS,JBODY)=ZFJOQC

!*          1.5.4 Var-QC-'weight' and 'effective' departure.
      ZQ=ZQCW(JOBS,JBODY)
      IF (LOBSCOR) ZQ=SQRT(ZQ)
      ZOM1DEP(JOBS,JBODY)=ZOM1DEP(JOBS,JBODY)*ZQ
      IF(LLU .AND. LL_AMBIG_REM .AND. (JBODY+3<=KCMBDY(JOBS)) ) THEN
        ZOM1DEP(JOBS,JBODY+1:JBODY+3)=&
       &ZOM1DEP(JOBS,JBODY+1:JBODY+3)*ZQ
        ZQCW   (JOBS,JBODY+1:JBODY+3)=ZQCW(JOBS,JBODY)
      ELSEIF(LLU) THEN
        ZOM1DEP(JOBS,JBODY+1)=ZOM1DEP(JOBS,JBODY+1)*ZQ
        ZQCW   (JOBS,JBODY+1)=ZQCW(JOBS,JBODY)
      ENDIF
    ENDIF
  ENDDO
  
  IF (LLERROR) THEN
    WRITE(NULERR,*) 'HJO:VARQC A >= 1. FINAL MINIMIZATION OR TRAJECTORY ',&
      & 'HAS PROBABLY OVERWRITTEN VARQC VARIABLES. RESTART ODB AND RERUN ', &
      & 'TRAJECTORY AND MINIMIZATION STEPS'  
    CALL ABOR1('HJO:INCONSISTENCY 1')
  ENDIF
ENDIF
!
! Incorporate obs error correlation in effective departue Zeff=W^0.5 R^-1 W^0.5 Z
!
IF (LOBSCOR) THEN
  IF (NBCORBLK > 0) IOBSCOR_COUNT(1:NBCORBLK)=0
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)
    IF(PHOFX(JOBS,JBODY) == RMDI) CYCLE
    IMASK=ROBODY(IBODY,MDB_OBSCORMASK_AT_ERRSTAT)
    IF (IMASK == NMDI) CYCLE
    IF (IMASK < 1 .OR. IMASK > NBCORBLK) CALL ABORTDB('HOPTL', 'IMASK OUT OF RANGE', IMASK)
    IF (KSCAN == 1 ) THEN
      IOBSCOR_COUNT(IMASK)=IOBSCOR_COUNT(IMASK) + 1      
      DO JEV=1,NRITZ_OBSCOR(IMASK)
        OBSCOR_AUX(KSET,IMASK)%OBSCOR_SCALPR_L(JEV,IOBSCOR_COUNT(IMASK))= &
    &   ROBODY(IBODY,MDB_OBSCOREV_AT_ERRSTAT(JEV))*ZOM1DEP(JOBS,JBODY)
      ENDDO
    ELSE
      ZOM1DEP(JOBS,JBODY)=ZOM1DEP(JOBS,JBODY)/OBSCOR_TRUN(IMASK)
      DO JEV=1,NRITZ_OBSCOR(IMASK)
        ZOM1DEP(JOBS,JBODY)=ZOM1DEP(JOBS,JBODY)+ &
  &     ( 1._JPRB/OBSCOR_EIGVAL(JEV,IMASK)-1._JPRB/OBSCOR_TRUN(IMASK) ) &
  &     *OBSCOR_SCALPR(JEV,IMASK)*ROBODY(IBODY,MDB_OBSCOREV_AT_ERRSTAT(JEV))
      ENDDO
!      write(nulerr,*) 'Jo contrib, Depar, Eff.-Depar',PHOFX(JOBS,JBODY),ZOM1DEP(JOBS,JBODY)
   
    ENDIF
  ENDDO
  IF (KSCAN == 1) THEN
    IF (LHOOK) CALL DR_HOOK('HJO',1,ZHOOK_HANDLE)
    RETURN
  ENDIF
  IF(LLITERQC) THEN
!*!OCL NOVREC
    DO IBODY=1,MLNKH2B(ILEN+1)-1
      JOBS = MBODYJOBS(IBODY)
      JBODY = IBODY + 1 - MLNKH2B(JOBS)

      IF(PHOFX(JOBS,JBODY) == RMDI) CYCLE

      IIVAR=KVAR(JOBS,JBODY)

      IF(IIVAR == NVAR_RAD .AND. (KCDTYP(JOBS) == NGTHRB &
       &                     .OR. KCDTYP(JOBS) == NSSMI .OR. KCDTYP(JOBS) == NTCWC)) THEN   ! 1C-Radiances
        LLVARQC=LVARQC_RAD1C(KSENSOR)
      ELSE
        LLVARQC=LVARQC(IIVAR,KOBSTYPE)
      ENDIF

      LLHUBERQC=LHUBERMINQC(IIVAR,KOBSTYPE)

      IIVNM=KVNMRQ(JOBS,JBODY)
      LLU=IIVNM == NVNUMB(01).OR.IIVNM == NVNUMB(56).OR.IIVNM == NVNUMB(62)
      IF((LLVARQC .OR. LLHUBERQC) .AND. ZFJOS(JOBS,JBODY) /= RMDI) THEN
        ZOM1DEP(JOBS,JBODY)=ZOM1DEP(JOBS,JBODY)*SQRT(ZQCW(JOBS,JBODY))
        IF(LLU .AND. LL_AMBIG_REM .AND. (JBODY+3<=KCMBDY(JOBS)) ) THEN
          ZOM1DEP(JOBS,JBODY+1:JBODY+3)=&
       &  ZOM1DEP(JOBS,JBODY+1:JBODY+3)*SQRT(ZQCW(JOBS,JBODY))
        ELSEIF(LLU) THEN
          ZOM1DEP(JOBS,JBODY+1)=ZOM1DEP(JOBS,JBODY+1)*SQRT(ZQCW(JOBS,JBODY))
        ENDIF
      ENDIF
    ENDDO  
  ENDIF
ENDIF   ! lobscor
!*          1.5.5 Store 'effective' departure for use in adjoint.
!*                There is no need to do it in trajectory run.
IF(.NOT.LOBSC1) THEN

  IF (LTESTADJH) THEN
    ALLOCATE(ZRANDOM(MLNKH2B(ILEN+1)))
    CALL SETRAN(KSET*KOBSTYPE,YL_RANDOM_STREAM)
    CALL UNIFORM_DISTRIBUTION(ZRANDOM,YL_RANDOM_STREAM)
  ENDIF

!*!OCL NOVREC
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)

    IF (PHOFX(JOBS,JBODY) == RMDI) CYCLE

    IF(KOBSTYPE == NSCATT .AND. KCDTYP(JOBS) == NSCAT5) THEN
!     Because QuikSCAT is used at 50 km res. the weights
!     are reduced by a factor initialized in defrun
      ZOM1DEP(JOBS,JBODY)=ZOM1DEP(JOBS,JBODY)*WSCAT5_FAC
    ENDIF

    IF(KOBSTYPE == NSCATT .AND. KCDTYP(JOBS) == NSCAT6) THEN
!     Because OCEANSAT2 is used at 50 km res. the weights
!     are reduced by a factor initialized in defrun
      ZOM1DEP(JOBS,JBODY)=ZOM1DEP(JOBS,JBODY)*WSCAT6_FAC
    ENDIF

    IF (LTESTADJH) THEN
      IF (PHOFX(JOBS,JBODY)/=RMDI .AND. ROBODY(IBODY,MDBFOE)/=RMDI) THEN
        ZOBSTEST = 2.0_JPRB * ZRANDOM(IBODY) - 1.0_JPRB
        ROBODY(IBODY,MDBIOM0(NUPTRA+1)) = -ZOBSTEST
        ZFJOAD(JOBS,JBODY) = PHOFX(JOBS,JBODY) * ZOBSTEST
      ELSE
        ROBODY(IBODY,MDBIOM0(NUPTRA+1)) = RMDI
      ENDIF
    ELSE
      ROBODY(IBODY,MDBIOM0(NUPTRA+1))=ZOM1DEP(JOBS,JBODY)
    ENDIF
  ENDDO
ENDIF

!*          1.6   Store result of VarQC, if last simulation.

IF(LQVARQC) THEN
! Final trajectory - if quadratic VarQC
  LLUPDVQC=LOBSC1 .AND. LLITERQC .AND. NUPTRA == MUPTRA
ELSE
! Final simulation of the last minimisation
  LLUPDVQC=NSIM4D == NSIM4DL .AND. LLITERQC .AND. NUPTRA == MUPTRA-1
ENDIF
IF(LLUPDVQC) THEN
  DO IBODY=1,MLNKH2B(ILEN+1)-1
    JOBS = MBODYJOBS(IBODY)
    JBODY = IBODY + 1 - MLNKH2B(JOBS)

    IIVAR=KVAR(JOBS,JBODY)

    IF(IIVAR == NMDI) CYCLE ! Fix for obscure problem
    IF((LVARQC(IIVAR,KOBSTYPE) .OR. LHUBERMINQC(IIVAR,KOBSTYPE)) .AND. PHOFX(JOBS,JBODY) /= RMDI) THEN 
!        1.6.1    Aposteriori probability of gross error = ZPGE.
      ZPGE=1.0_JPRB-ZQCW(JOBS,JBODY)
      ROBODY(IBODY,MDB_QC_PGE)=ZPGE

!        1.6.2    Set VARQC flag in CMA. Reject if > 75% wrong.
      IFL = INT(ZPGE/0.25_JPRB)
      IFLAG = MAX(MIN(IFL,3),0)

      CALL NFLGDSE(IBODY,'VARQC',IFLAG)
      IF(IFLAG == 3) THEN
        ROBODY(IBODY,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IBODY,MDBDSTA))
      ENDIF

!        1.6.3    Set event to record that VarQC has been performed
      ROBODY(IBODY,MDBDEV1) = ZCHEV1( ROBODY(IBODY,MDBDEV1),ND1EVENT(27))
    ELSE
      ROBODY(IBODY,MDB_QC_PGE)=RMDI
    ENDIF
  ENDDO
ENDIF

!*          1.7   Add Jo to the actual JO-table

DO JBODY=1,IMXBDY
!OCL NOVREC
  DO JOBS=1,ILEN
    IF(JBODY > KCMBDY(JOBS)) CYCLE

    IF(ZFJOS(JOBS,JBODY) /= RMDI) THEN
      IF(KOBSTYPE == NSCATT .AND. KCDTYP(JOBS) == NSCAT5) THEN
!       Because QuikSCAT is used at 50 km res. the weights are reduced by a factor
        ZFJOS(JOBS,JBODY)=ZFJOS(JOBS,JBODY)*WSCAT5_FAC
      ENDIF
      IF(KOBSTYPE == NSCATT .AND. KCDTYP(JOBS) == NSCAT6) THEN
!       Because OCEANSAT is used at 50 km res. the weights are reduced by a factor
        ZFJOS(JOBS,JBODY)=ZFJOS(JOBS,JBODY)*WSCAT6_FAC
      ENDIF
      IABNOB = ROBHDR(JOBS,MDB_ABNOB_AT_INDEX)
      FJOS(IABNOB) = FJOS(IABNOB) + ZFJOS(JOBS,JBODY)
    ENDIF

    IF (LTESTADJH .AND. ZFJOAD(JOBS,JBODY)/=RMDI) THEN
      IABNOB = ROBHDR(JOBS,MDB_ABNOB_AT_INDEX)
      FJOADJ(IABNOB) = FJOADJ(IABNOB) + ZFJOAD(JOBS,JBODY)
    ENDIF

  ENDDO
ENDDO


!*          1.8   Add Jo to the diagnostic JO-table

!$ CALL OML_SET_LOCK()

DO JOBS=1,ILEN
  DO JBODY=1,KCMBDY(JOBS)
    IF(ZFJOS(JOBS,JBODY) == RMDI) CYCLE

    IBODY=MLNKH2B(JOBS)+(JBODY-1)
    IIVAR=KVAR(JOBS,JBODY)
    IIVNM=KVNMRQ(JOBS,JBODY)
    LLU=IIVNM == NVNUMB(01) .OR. IIVNM == NVNUMB(56) .OR. IIVNM == NVNUMB(62)
    IF(LLU) THEN
      IADD=2
    ELSE
      IADD=1
    ENDIF
    
    IF(KOBSTYPE == NSATEM .OR. KOBSTYPE == NSATOB .OR. &
       & KOBSTYPE == NLIMB ) THEN                           ! GPSRO change  
      IGRP = MOD(MTYPOB(KSET)/100,100)
      ICTPSQ  = IGRP
    ELSE
      ICTPSQ=MSQBYCTP(KCDTYP(JOBS))
    ENDIF

    IF (KTIME(JOBS)<1 .OR. KTIME(JOBS)>NPRTBINS) THEN
      write(nulerr,*)'HJO: jobs,kdobs,ilen,nprtbins=',JOBS,KDOBS,ILEN,NPRTBINS
      write(nulerr,*)'HJO: ktime=',KTIME(:)
      CALL ABOR1('HJO: Error KTIME')
    ENDIF
    IF (IIVAR<1 .OR. IIVAR>JPXVAR) THEN
      write(nulerr,*)'HJO: jobs,jbody,iivar,jpxvar=',JOBS,JBODY,IIVAR,JPXVAR
      CALL ABOR1('HJO: Error IIVAR')
    ENDIF
    IF (ASSOCIATED(JOT(KOBSTYPE)%jost(ICTPSQ)%cost)) THEN
      IF (SIZE(JOT(KOBSTYPE)%jost(ICTPSQ)%cost)==0) THEN
        write(nulerr,*)'HJO: KOBSTYPE,ICTPSQ,size(cost),JPXVAR,NPRTBINS=', &
         & KOBSTYPE,ICTPSQ,SHAPE(JOT(KOBSTYPE)%jost(ICTPSQ)%cost),JPXVAR,NPRTBINS
        CALL ABOR1('HJO: cost size 0')
      ENDIF
    ELSE
      write(nulerr,*)'HJO: KOBSTYPE,ICTPSQ,JPXVAR,NPRTBINS=', &
                         & KOBSTYPE,ICTPSQ,JPXVAR,NPRTBINS
      CALL ABOR1('HJO: cost not allocated')
    ENDIF
    IF (ASSOCIATED(JOT(KOBSTYPE)%jost(ICTPSQ)%count)) THEN
      IF (SIZE(JOT(KOBSTYPE)%jost(ICTPSQ)%count)==0) THEN
        write(nulerr,*)'HJO: KOBSTYPE,ICTPSQ,size(count),JPXVAR,NPRTBINS=', &
         & KOBSTYPE,ICTPSQ,SHAPE(JOT(KOBSTYPE)%jost(ICTPSQ)%count),JPXVAR,NPRTBINS
        CALL ABOR1('HJO: cost size 0')
      ENDIF
    ELSE
      write(nulerr,*)'HJO: KOBSTYPE,ICTPSQ,JPXVAR,NPRTBINS=', &
                         & KOBSTYPE,ICTPSQ,JPXVAR,NPRTBINS
      CALL ABOR1('HJO: count not allocated')
    ENDIF

    JOT(KOBSTYPE)%jost(ICTPSQ)%cost(IIVAR,KTIME(JOBS))= &
     & JOT(KOBSTYPE)%jost(ICTPSQ)%cost(IIVAR,KTIME(JOBS))+ZFJOS(JOBS,JBODY)
    
    JOT(KOBSTYPE)%jost(ICTPSQ)%count(IIVAR,KTIME(JOBS))= &
     & JOT(KOBSTYPE)%jost(ICTPSQ)%count(IIVAR,KTIME(JOBS)) + IADD

    JOT(KOBSTYPE)%jost(ICTPSQ)%obserr(IIVAR)= &
     & JOT(KOBSTYPE)%jost(ICTPSQ)%obserr(IIVAR) + IADD*ROBODY(IBODY,MDBFOE)**2  

    IF(JOT(KOBSTYPE)%jost(ICTPSQ)%bgerr(IIVAR) == RMDI) THEN
!     DO Nothing
    ELSEIF(ROBODY(IBODY,MDBFGE) == RMDI) THEN
      JOT(KOBSTYPE)%jost(ICTPSQ)%bgerr(IIVAR) = RMDI
    ELSE
      JOT(KOBSTYPE)%jost(ICTPSQ)%bgerr(IIVAR)= &
       & JOT(KOBSTYPE)%jost(ICTPSQ)%bgerr(IIVAR) + IADD*ROBODY(IBODY,MDBFGE)**2  
    ENDIF

  ENDDO
ENDDO

IF (ALLOCATED(ZRANDOM)) DEALLOCATE(ZRANDOM)

!     ------------------------------------------------------------------

!$ CALL OML_UNSET_LOCK()

IF (LHOOK) CALL DR_HOOK('HJO',1,ZHOOK_HANDLE)
END SUBROUTINE HJO
