SUBROUTINE HRADP_ML_TL(KDLEN,KLEN,KSET,&
 & PNVANG,PZEANG,KCOEF,&
 & PPRESH ,PTF ,PQF, &
 & PPS ,PTS ,PT2M ,PQ2M ,PCTOP,PCAMT,&
 & PPRESH5,PTF5,PQF5, &
 & PPS5,PTS5,PT2M5,PQ2M5,PCTOP5,PCAMT5)  

!****  HRADP_ML_TL  - PREPARATION FOR THE RADIANCE INTERPOLATION

!     ERIK ANDERSSON          01/04/99       ECMWF

!     PURPOSE
!     -------

!        HRADP_ML_TL PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS
!        IF THE RT INPUT IS ON MODEL LEVELS.

!**   INTERFACE
!     ---------

!        CALL HRADP_ML_TL(KDLEN,KLEN,KSET,&
!    &PNVANG,PZEANG,KCOEF,&
!    &PPRESH ,PPRESF ,PTF ,PQF ,&
!    &PS ,PTS ,PT2M ,PQ2M ,&
!    &PPRESH5,PPRESF5,PTF5,PQF5,&
!    &PPS5,PTS5,PT2M5,PQ2M5)

!        Where KDLEN   = First dimension (Number of observations)
!              KLEN    = Actual number of observations
!              KSET    = Number (numero) of the obs set to be treated
!              PNVANG  = Nadir viewing angle
!              PZEANG  = Zenith angle
!              KCOEF   = Location of rttov transmittance coeffs, for this sat
!              PPRESH5 = Half-level pressures
!              PPRESF5 = Full-level pressures
!              PTF5    = Model level temperature
!              PQF5    = Model level humidity
!              PPS5    = Surface pressure at TOVS locations
!              PTS5    = Surface skin temperature
!              PT2M5   = Two-metre temperature, for rttov
!              PQ2M5   = Two-metre spec. humidity, for rttov

!        HRADP_ML_TL IS CALLED FROM HOPTL

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     E.Andersson 990104 : Original
!     Xu Li       000523 : Add FOV defination for VTPR
!     N. Bormann  010622 : Altered call to HSATANG
!     N. Bormann  20071215 : Renamed and revamped for RTTOV-9
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE GOMS_MIX     , ONLY : LGOMUA,YGOMV
USE YOMDIM   , ONLY : NFLEVG
USE YOMTVRAD , ONLY : SATGRP_TABLE,TOVSCVX  ,&
 & TOVSCVX5  
USE YOMVAR   , ONLY : LTOVSCV, LCO2, NUPTRA, LCLDSINK
USE YOMCOCTP , ONLY : NGTHRB , NSATEM   
USE YOMOBSET , ONLY : MTYPOB
USE YOMDB
USE YOMCMA   , ONLY : NRESUPD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNVANG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZEANG(KDLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCOEF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KDLEN),PCTOP(KDLEN),PCAMT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ2M(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS5(KDLEN),PCTOP5(KDLEN),PCAMT5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ2M5(KDLEN) 

INTEGER(KIND=JPIM) :: IOBSTYPE,ICDTYP_TOVS,IGRP,ISERIES
INTEGER(KIND=JPIM) :: IPOSM
INTEGER(KIND=JPIM) :: JOBS

LOGICAL :: LLRAD1C
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "hsatang.intfb.h"

#include "openmp_obs.h"

! ---------------------------------------------------------------------
!*          1.0   Initiate, set switches

IF (LHOOK) CALL DR_HOOK('HRADP_ML_TL',0,ZHOOK_HANDLE)
IOBSTYPE = NSATEM ! Used when called from BGOBS
IF(KSET < 0) GOTO 4 ! Call from BGOBS

IOBSTYPE = MOD(MTYPOB(KSET),100)
IGRP =MOD(MTYPOB(KSET)/100,100)
ICDTYP_TOVS = SATGRP_TABLE(IGRP)%codetype
ISERIES     = SATGRP_TABLE(IGRP)%rtseries
KCOEF       = SATGRP_TABLE(IGRP)%rtcoef_pos

LLRAD1C= ICDTYP_TOVS == NGTHRB

!*          1.1   Compute satellite nadir and viewing angles.
CALL HSATANG(KDLEN,KLEN,IOBSTYPE,ISERIES,ICDTYP_TOVS,IGRP,PNVANG,PZEANG)

!*          1.2   TOVS control variable, Skin temperature.
IF(LTOVSCV .AND. LLRAD1C) THEN
!OCL NOVREC
  DO JOBS = 1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
    PTS5(JOBS) = TOVSCVX5(IPOSM,1)
    PTS(JOBS)  = TOVSCVX(IPOSM,1)
    IF (NUPTRA == NRESUPD-1) SATHDR(JOBS,MDB1BCP14A)=TOVSCVX(IPOSM,1)+TOVSCVX5(IPOSM,1)
  ENDDO
ENDIF

!*          1.2a   TOVS control variable, cloud sink
IF(LTOVSCV .AND. LLRAD1C .AND. LCLDSINK) THEN

!OCL NOVREC
  DO JOBS = 1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
      PCTOP5(JOBS)=TOVSCVX5(IPOSM,2)
      PCTOP5(JOBS)=MIN(MAX(PCTOP5(JOBS),51.0_JPRB),1012.0_JPRB)     ! trap cloud top pressure
      PCAMT5(JOBS)=TOVSCVX5(IPOSM,3)
      PCTOP(JOBS)=TOVSCVX(IPOSM,2)
      PCAMT(JOBS)=TOVSCVX(IPOSM,3)
!     IF (NUPTRA == NRESUPD-1) THEN
      ROBSU(JOBS,MDB_CTOPINC_AT_CLOUD_SINK)=TOVSCVX(IPOSM,2)+TOVSCVX5(IPOSM,2)
      ROBSU(JOBS,MDB_CAMTINC_AT_CLOUD_SINK)=TOVSCVX(IPOSM,3)+TOVSCVX5(IPOSM,3)
!     ENDIF
  ENDDO
ENDIF


!*          1.2a   TOVS control variable, CO2.
IF(LTOVSCV .AND. LLRAD1C .AND. LCO2) THEN
  CALL ABOR1('HRADP_ML_TL: CO2 as sink variable and internal RTTOV interpolation not implemented.')
ENDIF

!*          1.3   Replace ozone with climatology, if not avalable as
!                 model field.
4 CONTINUE
IF(.NOT.LGOMUA(IOBSTYPE,YGOMV%MO3)) THEN
  CALL ABOR1('HRADP_ML_TL: No ozone and internal RTTOV interpolation not implemented')
ENDIF

!*          1.6   PERFORM THE POST-PROCESSING OF 2M TEMPERATURE AND HUMIDITY.
!                 and surface pressure
!                 WHILE WAITING FOR A MORE POWERFUL PHYSICS:
!                 A)  T2M IS TAKEN EQUAL TO T(NFLEVG): LOWEST MODEL LEVEL,
!                 B) HU2M IS TAKEN EQUAL TO HU(NFLEVG): LOWEST MODEL LEVEL.

PT2M5(1:KLEN)=PTF5(1:KLEN,NFLEVG)
PT2M (1:KLEN)=PTF (1:KLEN,NFLEVG)
PQ2M5(1:KLEN)=PQF5(1:KLEN,NFLEVG)
PQ2M (1:KLEN)=PQF (1:KLEN,NFLEVG)
PPS5(1:KLEN) =PPRESH5(1:KLEN,NFLEVG)
PPS(1:KLEN)  =PPRESH(1:KLEN,NFLEVG)


IF (LHOOK) CALL DR_HOOK('HRADP_ML_TL',1,ZHOOK_HANDLE)
END SUBROUTINE HRADP_ML_TL
