SUBROUTINE HRADPTL(KDLEN,KLEN,KSET,POROG5,POROGOB,&
 & PNVANG,PZEANG,KCOEF,&
 & PPRESH ,PTF ,PTSTAR ,PT0 ,PTQOPP ,&
 & PPS ,PTS ,PT2M ,PQ2M ,PCTOP,PCAMT, &
 & PPRESH5,PPRESF5,PTF5,PTSTAR5,PT05,PTQOPP5,&
 & PPS5,PTS5,PT2M5,PQ2M5,PCTOP5,PCAMT5)  

!****  HRADPTL  - PREPARATION FOR THE RADIANCE INTERPOLATION

!     ERIK ANDERSSON          01/04/99       ECMWF

!     PURPOSE
!     -------

!        HRADPTL PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS.

!**   INTERFACE
!     ---------

!        CALL HRADPTL(KDLEN,KLEN,KSET,POROG5,POROGOB,&
!    &PNVANG,PZEANG,KCOEF,&
!    &PPRESH ,PPRESF ,PTF ,PTSTAR ,PT0 ,PTQOPP ,&
!    &PS ,PTS ,PT2M ,PQ2M ,&
!    &PPRESH5,PPRESF5,PTF5,PTSTAR5,PT05,PTQOPP5,&
!    &PPS5,PTS5,PT2M5,PQ2M5)

!        Where KDLEN   = First dimension (Number of observations)
!              KLEN    = Actual number of observations
!              KSET    = Number (numero) of the obs set to be treated
!              POROG5  = Model's orography
!              POROGOB = Observation's orography
!              PNVANG  = Nadir viewing angle
!              PZEANG  = Zenith angle
!              KCOEF   = Location of rttov transmittance coeffs, for this sat
!              PPRESH5 = Half-level pressures
!              PPRESF5 = Full-level pressures
!              PTF5    = Model level temperature
!              PTSTAR5 = Surface temperature (see CTSTAR)
!              PT05    = Surface temperature (see CTSTAR)
!              PTQOPP5 =Model T,Q and O3 on NLSAT pressure levels
!              PPS5    = Surface pressure at TOVS locations
!              PTS5    = Surface skin temperature
!              PT2M5   = Two-metre temperature, for rttov
!              PQ2M5   = Two-metre spec. humidity, for rttov

!        HRADPTL IS CALLED FROM HOPTL

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     E.Andersson 990104 : Original
!     Xu Li       000523 : Add FOV defination for VTPR
!     N. Bormann  010622 : Altered call to HSATANG
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0B  , ONLY : LECMWF
USE GOMS_MIX     , ONLY : LGOMUA,YGOMV
USE YOMDIMO  , ONLY : NLSAT, NPNAV
USE YOMDIM   , ONLY : NFLEVG
USE YOMTVRAD , ONLY : O3CLRT   ,RADPRE   ,SATGRP_TABLE,TOVSCVX  ,&
 & TOVSCVX5  
USE YOMCST   , ONLY : RPI
USE YOMANCS  , ONLY : RMDI
USE YOMVAR   , ONLY : LTOVSCV, LCO2, NUPTRA, LCLDSINK
USE YOMCOCTP , ONLY : NGTHRB , NSATEM   
USE YOMOBSET , ONLY : MTYPOB
USE YOMDB
USE YOMCMA   , ONLY : NRESUPD
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGOB(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNVANG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZEANG(KDLEN)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCOEF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT0(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTQOPP(KDLEN,NLSAT,NPNAV-1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KDLEN),PCTOP(KDLEN),PCAMT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M(KDLEN)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ2M(KDLEN)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTQOPP5(KDLEN,NLSAT,NPNAV-1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS5(KDLEN),PCTOP5(KDLEN),PCAMT5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ2M5(KDLEN)
REAL(KIND=JPRB), ALLOCATABLE :: Z_PTFINP(:,:), Z_PTFINPTL(:,:)
REAL(KIND=JPRB), ALLOCATABLE  :: Z_PTFOUT(:,:), Z_PTFOUTTL(:,:)
INTEGER(KIND=JPIM), PARAMETER :: JP_T=1, JP_Q=2, JP_O3=3, JP_CO2=4
INTEGER(KIND=JPIM), DIMENSION(KDLEN) :: ILTROP

INTEGER(KIND=JPIM) :: IOBSTYPE,ICDTYP_TOVS,IGRP,ISERIES
INTEGER(KIND=JPIM) :: IPOSM,INLAMTOP,IOBS
INTEGER(KIND=JPIM) :: JOBS,JLEV
INTEGER(KIND=JPIM) :: IPMOTOV
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZORDIF
REAL(KIND=JPRB) :: ZPSINFL,ZDP5,ZA5,ZDP,ZA
LOGICAL :: LLRAD1C
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "extraptl.intfb.h"
#include "hsatang.intfb.h"
#include "pppmer.intfb.h"
#include "pppmertl.intfb.h"
#include "tropopause.intfb.h"

#include "openmp_obs.h"

! ---------------------------------------------------------------------
!*          1.0   Initiate, set switches

IF (LHOOK) CALL DR_HOOK('HRADPTL',0,ZHOOK_HANDLE)
IOBSTYPE = NSATEM ! Used when called from BGOBS
IF(KSET < 0) GOTO 4 ! Call from BGOBS

IOBSTYPE = MOD(MTYPOB(KSET),100)
IGRP =MOD(MTYPOB(KSET)/100,100)
ICDTYP_TOVS = SATGRP_TABLE(IGRP)%codetype
ISERIES     = SATGRP_TABLE(IGRP)%rtseries
KCOEF       = SATGRP_TABLE(IGRP)%rtcoef_pos

LLRAD1C= ICDTYP_TOVS == NGTHRB

!*          1.1   Compute satellite nadir and viewing angles.
CALL HSATANG(KDLEN,KLEN,IOBSTYPE,ISERIES,ICDTYP_TOVS,IGRP,PNVANG,PZEANG)

!*          1.2   TOVS control variable, Skin temperature.
IF(LTOVSCV .AND. LLRAD1C) THEN
!OCL NOVREC
  DO JOBS = 1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
    PTS5(JOBS) = TOVSCVX5(IPOSM,1)
    PTS(JOBS)  = TOVSCVX(IPOSM,1)
    IF (NUPTRA == NRESUPD-1) SATHDR(JOBS,MDB1BCP14A)=TOVSCVX(IPOSM,1)+TOVSCVX5(IPOSM,1)
  ENDDO
ENDIF

!*          1.2a   TOVS control variable, cloud sink
IF(LTOVSCV .AND. LLRAD1C .AND. LCLDSINK) THEN

!OCL NOVREC
  DO JOBS = 1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
      PCTOP5(JOBS)=TOVSCVX5(IPOSM,2)
      PCTOP5(JOBS)=MIN(MAX(PCTOP5(JOBS),51.0_JPRB),1012.0_JPRB)     ! trap cloud top pressure
      PCAMT5(JOBS)=TOVSCVX5(IPOSM,3)
      PCTOP(JOBS)=TOVSCVX(IPOSM,2)
      PCAMT(JOBS)=TOVSCVX(IPOSM,3)
!     IF (NUPTRA == NRESUPD-1) THEN
      ROBSU(JOBS,MDB_CTOPINC_AT_CLOUD_SINK)=TOVSCVX(IPOSM,2)+TOVSCVX5(IPOSM,2)
      ROBSU(JOBS,MDB_CAMTINC_AT_CLOUD_SINK)=TOVSCVX(IPOSM,3)+TOVSCVX5(IPOSM,3)
!     ENDIF
  ENDDO
ENDIF


!*          1.2a   TOVS control variable, CO2.
IF(LTOVSCV .AND. LLRAD1C .AND. LCO2) THEN

!* Calculate tropopause level at observation location (taken from SUSHFCE.F90)

  CALL TROPOPAUSE(KDLEN,KLEN,PTQOPP5,ILTROP)


!OCL NOVREC
  DO JOBS = 1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
    DO JLEV = 1,NLSAT
      IF(JLEV < ILTROP(JOBS)) PTQOPP5(JOBS,JLEV,JP_CO2)=TOVSCVX5(IPOSM,2)
      IF(JLEV >= ILTROP(JOBS)) PTQOPP5(JOBS,JLEV,JP_CO2)=TOVSCVX5(IPOSM,3)
      IF(JLEV < ILTROP(JOBS)) PTQOPP(JOBS,JLEV,JP_CO2)=TOVSCVX(IPOSM,2)
      IF(JLEV >= ILTROP(JOBS)) PTQOPP(JOBS,JLEV,JP_CO2)=TOVSCVX(IPOSM,3)
    ENDDO
    ROBDDR(JOBS,MDB_CO2INC_S_AT_CO2_SINK)=TOVSCVX(IPOSM,2)+TOVSCVX5(IPOSM,2)
    ROBDDR(JOBS,MDB_CO2INC_T_AT_CO2_SINK)=TOVSCVX(IPOSM,3)+TOVSCVX5(IPOSM,3)

!Store Temperature lapse rate and tropopause level for use in CO2 error postprocessing
    ROBDDR(JOBS,MDB_CO2ANER_T_AT_CO2_SINK)=PTQOPP5(JOBS,ILTROP(JOBS),JP_T)-PTQOPP5(JOBS,36,JP_T)
    ROBDDR(JOBS,MDB_TROPO_AT_CO2_SINK)=ILTROP(JOBS)
  ENDDO
ENDIF

!*          1.3   Replace ozone with climatology, if not avalable as
!                 model field.
4 CONTINUE
IF(.NOT.LGOMUA(IOBSTYPE,YGOMV%MO3)) THEN
  DO JLEV = 1,NLSAT
    PTQOPP5(1:KLEN,JLEV,JP_O3)=O3CLRT(JLEV)
    PTQOPP (1:KLEN,JLEV,JP_O3)=0.0_JPRB
  ENDDO
ENDIF

!*          1.3a   Replace CO2 with global mean value when called from BGOBS
IF(KSET < 0 .AND. LCO2) THEN
  PTQOPP5(1:KLEN,1:NLSAT,JP_CO2)=378.0_JPRB
  PTQOPP (1:KLEN,1:NLSAT,JP_CO2)=0.0_JPRB
ENDIF

!*          1.4   Find fix pressure level corresponding to model's top
!                 (Assumes top model level is a pressure surface)
!                 Check is performed in terms of integer-Pascals.

IOBS=1
INLAMTOP=0
DO JLEV = 1,NLSAT
  IF(NINT(RADPRE(JLEV)) < NINT(PPRESF5(IOBS,1))) INLAMTOP=JLEV
ENDDO

!*          1.5   Replace temperature above model's top

IF(KSET > 0) THEN
  IF(LLRAD1C) THEN
    IF (LECMWF) THEN
      DO JLEV = 1,INLAMTOP
        PTQOPP5(1:KLEN,JLEV,JP_T)=273.0_JPRB !dummy strat, if needed
        PTQOPP (1:KLEN,JLEV,JP_T)=0.0_JPRB
      ENDDO
    ELSE
      IPMOTOV = NLSAT-INLAMTOP-1
      ALLOCATE(Z_PTFINP(KLEN,IPMOTOV))
      ALLOCATE(Z_PTFINPTL(KLEN,IPMOTOV))
      ALLOCATE(Z_PTFOUT(KLEN,NLSAT))
      ALLOCATE(Z_PTFOUTTL(KLEN,NLSAT))
      Z_PTFINP(1:KLEN,1:IPMOTOV) = PTQOPP5(1:KLEN,INLAMTOP+2:NLSAT,JP_T)
      Z_PTFINPTL(1:KLEN,1:IPMOTOV) = PTQOPP(1:KLEN,INLAMTOP+2:NLSAT,JP_T)
      CALL EXTRAPTL(Z_PTFINP, Z_PTFOUT,Z_PTFINPTL, Z_PTFOUTTL, KLEN)
      DO JLEV = 1,INLAMTOP
        PTQOPP5(1:KLEN,JLEV,JP_T)= Z_PTFOUT(1:KLEN,JLEV)
        PTQOPP(1:KLEN,JLEV,JP_T) = Z_PTFOUTTL(1:KLEN,JLEV)
      ENDDO
      DEALLOCATE(Z_PTFINP, Z_PTFOUT, Z_PTFINPTL, Z_PTFOUTTL)
    ENDIF
  ENDIF
ENDIF

!*          1.6   PERFORM THE POST-PROCESSING OF 2M TEMPERATURE AND HUMIDITY.
!                 WHILE WAITING FOR A MORE POWERFUL PHYSICS:
!                 A)  T2M IS TAKEN EQUAL TO T(NFLEVG): LOWEST MODEL LEVEL,
!                 B) HU2M IS TAKEN EQUAL TO HU(NLSAT): LOWEST TOVS LEVEL.

PT2M5(1:KLEN)=PTF5(1:KLEN,NFLEVG)
PT2M (1:KLEN)=PTF (1:KLEN,NFLEVG)
PQ2M5(1:KLEN)=PTQOPP5(1:KLEN,NLSAT,JP_Q)
PQ2M (1:KLEN)=PTQOPP (1:KLEN,NLSAT,JP_Q)

!*          1.7   FIND OUT PRESSURE AT THE 'STATION HEIGHT' OF EACH
!                 TOVS LOCATION. THE POST PROCESSING ROUTINE FOR MEAN
!                 SEA LEVEL PRESSURE IS USED. FIRST ADJUST T2M AND TS
!                 TO NEW HEIGHT (-6.5 K/KM).

WHERE(POROGOB(1:KLEN) == RMDI)
  ZORDIF(1:KLEN) = 0.0_JPRB
ELSEWHERE
  ZORDIF(1:KLEN) = POROG5(1:KLEN)-POROGOB(1:KLEN)
ENDWHERE
CALL PPPMER(KDLEN,1,KLEN,PPRESH5(1,NFLEVG),ZORDIF,PTSTAR5,PT05,PPS5)
CALL PPPMERTL(KDLEN,1,KLEN,PPRESH(1,NFLEVG),ZORDIF,PTSTAR,PT0 ,PPS ,&
 & PPRESH5(1,NFLEVG),PTSTAR5,PT05)  

!*          1.8   IF ANY OF THE 43 LEVELS ARE WITHIN 20 MBAR OF THE
!                 SURFACE, ADD A CORRECTION TO THE TEMPERATURE AT THAT
!                 LEVEL TO MAKE SURE THAT THE LEVEL TEMPERATURE
!                 APPROACHES (SINUOUSLY) THE NEAR-SURFACE (2M)
!                 TEMPERATURE AS SURFACE PRESSURE APPROACHES THAT LEVEL
!                 PRESSURE. DO THE SAME FOR HUMIDITY.
!                 DEFINE DOMAINE (ZPSINFL) IN WICH PS HAS AN INFLUENCE
!                 ON LEVEL TEMP. LINEAR WEIGHT (ZA5) IS A FUNCTION OF
!                 PS (AND P)

ZPSINFL=2000._JPRB
DO JLEV = 1,NLSAT
  DO JOBS = 1,KLEN
    ZDP5=PPS5(JOBS)-RADPRE(JLEV)
    ZDP =PPS (JOBS)
    IF(ABS(ZDP5) < ZPSINFL) THEN
      ZA5=0.5_JPRB*(1.0_JPRB+SIN(RPI/ZPSINFL*(ZDP5-ZPSINFL*0.5_JPRB)))
      ZA =0.5_JPRB*COS(RPI/ZPSINFL*(ZDP5-ZPSINFL*0.5_JPRB))*(RPI/ZPSINFL)*ZDP
      IF(ZA5 <= 0.0001_JPRB) ZA=0.0_JPRB
      ZA5=MAX(ZA5,0.0001_JPRB)
      PTQOPP (JOBS,JLEV,JP_T)=ZA5*PTQOPP (JOBS,JLEV,JP_T)+ZA*PTQOPP5(JOBS,JLEV,JP_T)&
       & +(1.0_JPRB-ZA5)*PT2M(JOBS)-ZA*PT2M5(JOBS)  
      PTQOPP5(JOBS,JLEV,JP_T)=ZA5*PTQOPP5(JOBS,JLEV,JP_T)+(1.0_JPRB-ZA5)*PT2M5(JOBS)
      PTQOPP (JOBS,JLEV,JP_Q)=ZA5*PTQOPP (JOBS,JLEV,JP_Q)+ZA*PTQOPP5(JOBS,JLEV,JP_Q)&
       & +(1.0_JPRB-ZA5)*PQ2M (JOBS) - ZA*PQ2M5(JOBS)  
      PTQOPP5(JOBS,JLEV,JP_Q)=ZA5*PTQOPP5(JOBS,JLEV,JP_Q)+(1.0_JPRB-ZA5)*PQ2M5(JOBS)
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('HRADPTL',1,ZHOOK_HANDLE)
END SUBROUTINE HRADPTL
