SUBROUTINE HVNMTLT(KMXVAR,CDLV,KVARL,PLAYL)

!**** *HOP* - Translate list of VNM-numbers and variable numbers

!     E. ANDERSSON            ECMWF          01/04/99

!     PURPOSE
!     -------

!     Match the variable numbers of CMA with ob operator

!**   INTERFACE
!     ---------
!     *CALL* *HVNMTLT(KMXVAR,CDLV,KVARL)*
!     WHERE KMXVAR  = Dimension
!           CDLV    = Character codes of observation operators
!           KVARL   = Corresponding NVAR variable numbers
!           PLAYL   = Layer summation factors

!        HVNMTLT is called from HOP

!**   METHOD
!     ------
!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Erik Andersson  01/04/99 : Original
!     P. Moll         99/10/15 : Put Apache and ACHMT in cy22 (for MF)
!     S. Healy        04/10/27 : Introduce GPSRO information
!     D. Tan/
!     A. Benedetti    14-Mar-05: Introduce Aeolus and aerosol information
!     N. Bormann      04/12/23 : Introduce limb radiances
!     D. Tan          14-Mar-05: Introduce Aeolus information
!     A. Benedetti    14-Mar-05: Introduce aerosol information
!     A. Dethof       12-May-05: Introduce GRG information
!     R. Engelen      20-Jun-06: Introduce GHG information
!     T. Montmerle    09-Oct-07: Introduce Doppler radar information
!     K. Scipal       10-Sep-07: Introduced ASCAT soil moisture
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCOSJO , ONLY : NVAR_T,NVAR_Z,NVAR_U,NVAR_DZ,NVAR_H,&
 & NVAR_PWC,NVAR_H2,NVAR_T2,NVAR_TS,NVAR_RR,NVAR_SN,&
 & NVAR_PS,NVAR_DD,NVAR_FF,NVAR_RAD,NVAR_CC,NVAR_U10,NVAR_LH,&
 & NVAR_TCW,NVAR_Q,NVAR_TO3,NVAR_FFS,NVAR_APD,NVAR_RO,&
 & NVAR_HLS,NVAR_AOD,&
 & NVAR_LRA,NVAR_GHG,CVAR_GHG,NVAR_GRG,CVAR_GRG,NVAR_RFL,&
 & NVAR_COD,NVAR_RAO,NVAR_RFA,NVAR_ODA,NVAR_DOW,&
 & NVAR_SM, NVAR_SMN, NVAR_SFR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KMXVAR 
CHARACTER(LEN=3)  ,INTENT(OUT)   :: CDLV(KMXVAR) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KVARL(KMXVAR) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAYL(KMXVAR,2) 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('HVNMTLT',0,ZHOOK_HANDLE)
PLAYL(1:KMXVAR,1)=1.0_JPRB
PLAYL(1:KMXVAR,2)=0.0_JPRB

CDLV(1:KMXVAR)='XXX'
KVARL(1:KMXVAR)=0

CDLV(1)='U'
KVARL(01)=NVAR_U

CDLV(2)='V'
KVARL(02)=NVAR_U

CDLV(3)='Z'
KVARL(03)=NVAR_Z

CDLV(4)='DZ'    ! Thickness
KVARL(04)=NVAR_DZ
PLAYL(4,1:2)=(/1.0_JPRB,-1.0_JPRB/)

CDLV(5)='H'
KVARL(05)=NVAR_H

CDLV(6)='PWC'   ! Total column water vapour
KVARL(06)=NVAR_PWC
PLAYL(6,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(7)='H2'
KVARL(07)=NVAR_H2

CDLV(8)='T'
KVARL(08)=NVAR_T

CDLV(10)='T2'
KVARL(10)=NVAR_T2

CDLV(12)='TS'
KVARL(12)=NVAR_TS

CDLV(34)='RR'
KVARL(34)=NVAR_RR

CDLV(46)='SN'
KVARL(46)=NVAR_SN

CDLV(47)='PS'
KVARL(47)=NVAR_PS

!CDLV(48)='DD'   ! Wind direction (Obs operator not coded at present)
KVARL(48)=NVAR_DD

CDLV(49)='FF'   ! Wind speed.
KVARL(49)=NVAR_FF

CDLV(50)='RAD'
KVARL(50)=NVAR_RAD

CDLV(52)='CC'
KVARL(52)=NVAR_CC

CDLV(56)='U10'
KVARL(56)=NVAR_U10

CDLV(57)='V10'
KVARL(57)=NVAR_U10

CDLV(58)='LH'   ! Layer RH
KVARL(58)=NVAR_LH
PLAYL(58,1:2)=(/0.5_JPRB,0.5_JPRB/)

CDLV(60)='TCW'     ! Cloud water of a layer, or the total column
KVARL(60)=NVAR_TCW
!CDLV(xx)='CLW'    ! Cloud water mixing ratio
!KVARL(xx)=NVAR_CLW

CDLV(61)='V10'  ! Yes, V comes before U
KVARL(61)=NVAR_U10

CDLV(62)='U10'
KVARL(62)=NVAR_U10

CDLV(63)='Q'
KVARL(63)=NVAR_Q

CDLV(68)='TO3'  ! O3 thickness
KVARL(68)=NVAR_TO3
PLAYL(68,1:2)=(/-1.0_JPRB,1.0_JPRB/)

!CDLV(64)='DDS'     ! Surface (10m) wind direction (not coded at present)
!KVARL(64)=NVAR_DDS ! NVAR_DDS is not yet defined
CDLV(65)='FFS'      ! Surface (10m) wind speed
KVARL(65)=NVAR_FFS

CDLV(74)='RFL'
KVARL(74)=NVAR_RFL

CDLV(75)='APD'
KVARL(75)=NVAR_APD

CDLV(76)='RO '
KVARL(76)=NVAR_RO

CDLV(77)='HLS'
KVARL(77)=NVAR_HLS

CDLV(78)='AOD'
KVARL(78)=NVAR_AOD

CDLV(79)='LRA'      
KVARL(79)=NVAR_LRA  

CDLV(80)=CVAR_GRG(1)
KVARL(80)=NVAR_GRG(1)
PLAYL(80,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(81)=CVAR_GRG(2)
KVARL(81)=NVAR_GRG(2)
PLAYL(81,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(82)=CVAR_GRG(3)
KVARL(82)=NVAR_GRG(3)
PLAYL(82,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(83)=CVAR_GRG(4)
KVARL(83)=NVAR_GRG(4)
PLAYL(83,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(84)=CVAR_GRG(5)
KVARL(84)=NVAR_GRG(5)
PLAYL(84,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(85)='COD'
KVARL(85)=NVAR_COD
CDLV(86)='RAO'
KVARL(86)=NVAR_RAO
CDLV(87)='ODA'
KVARL(87)=NVAR_ODA
CDLV(88)='RFA'
KVARL(88)=NVAR_RFA

CDLV(90)='SM '
KVARL(90)=NVAR_SM
CDLV(91)='SMN'
KVARL(91)=NVAR_SMN

CDLV(93)='DOW'
KVARL(93)=NVAR_DOW

! GHG variables
CDLV(94)=CVAR_GHG(1)
KVARL(94)=NVAR_GHG(1)
!PLAYL(94,1:2)=(/-1.0_JPRB,1.0_JPRB/)
CDLV(95)=CVAR_GHG(2)
KVARL(95)=NVAR_GHG(2)
!PLAYL(95,1:2)=(/-1.0_JPRB,1.0_JPRB/)

CDLV(97)='SFR'
KVARL(97)=NVAR_SFR

IF (LHOOK) CALL DR_HOOK('HVNMTLT',1,ZHOOK_HANDLE)
END SUBROUTINE HVNMTLT
