SUBROUTINE INV_REFL1DSTAT(KDLEN,KDBDY,KLEN,KHORIZ,KDCOUNT,KCOUNT,KPNLV,PTF5,PQF5,&
 & PREFL2D,PGEOPF5,PPRESF5,KDPP)

! SUBROUTINE INVERSION_REFL1D contains baysian inversion of reflectivities 
! by using model profiles in the neighboring of the obervation
! Eric Wattrelot 10/12/2006 

USE PARKIND1,ONLY : JPIM,  JPRB
USE YOMCST  ,ONLY : RG
USE YOMDIM  ,ONLY : NFLEVG
USE YOMHOOK ,ONLY : LHOOK, DR_HOOK
USE YOMLUN  ,ONLY : NULOUT
USE YOMANCS ,ONLY : RMDI ! 180/PI
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KDLEN,KDBDY,KLEN,KDCOUNT,KHORIZ
INTEGER(KIND=JPIM),INTENT(IN) :: KDPP
INTEGER(KIND=JPIM),INTENT(IN),DIMENSION(KLEN) :: KCOUNT
INTEGER(KIND=JPIM),INTENT(IN),DIMENSION(KDLEN,KDBDY) :: KPNLV

REAL(KIND=JPRB),INTENT(IN),   DIMENSION(KDLEN,1:NFLEVG,KHORIZ)     :: PTF5,PQF5
REAL(KIND=JPRB),INTENT(IN),   DIMENSION(KDLEN,KDCOUNT,KDPP,KHORIZ)      :: PREFL2D

REAL(KIND=JPRB),              DIMENSION(KDLEN,KDCOUNT,KHORIZ)      :: ZHU
REAL(KIND=JPRB),               DIMENSION(KDLEN,KDCOUNT) :: ZREHU, PPRESPIX
REAL(KIND=JPRB),INTENT(IN),   DIMENSION(KDLEN,NFLEVG,KHORIZ)       :: PGEOPF5, PPRESF5
REAL(KIND=JPRB)               :: ZHOOK_HANDLE
REAL(KIND=JPRB)               :: ZXSIG, ZEXP, ZDENOM, ZZCOEF, ZZCOEFINTP

INTEGER(KIND=JPIM) :: JLEN,JCOUNT,JJCOUNT,JK,JPROF,JI  
INTEGER(KIND=JPIM) :: IBODY, IIBODY

#include "openmp_obs.h"
IF (LHOOK) CALL DR_HOOK('INV_REFL1DSTAT',0,ZHOOK_HANDLE)

!*********************************************************
! 0. Initialization                                      *
!*********************************************************

ZREHU(:,:)=0._JPRB
ZXSIG = 0.2_JPRB

!********************************************************
!  Inversion of reflectivities                          *
!********************************************************

DO JLEN=1,KLEN ! loop over obs columns
   IF (KCOUNT(JLEN)==0) THEN 
     WRITE(NULOUT,*)"KCOUNT vaut zero",KCOUNT(JLEN), JLEN
     CYCLE
   ENDIF
   DO JCOUNT=1,KCOUNT(JLEN) ! loop over elevations
          
      IBODY = MLNKH2B(JLEN)+ KPNLV(JLEN,JCOUNT) -1
      ZDENOM = 0._JPRB

        DO JPROF=1,KHORIZ ! loop over model profiles

         IF (JLEN==1) THEN
              DO JI=1,NFLEVG,5
                WRITE(NULOUT,*)"PGEOPF5(1,",JI,JPROF,")",PGEOPF5(JLEN,JI,JPROF)
                WRITE(NULOUT,*)"ROBODY(MDBPRL)",ROBODY(IBODY,MDBPRL)
              ENDDO
         ENDIF
                  ! interpolation on the altitude of each elevation
                  IF (ROBODY(IBODY,MDBPRL)>=(PGEOPF5(JLEN,NFLEVG,JPROF)/RG).AND.&
                  & ROBODY(IBODY,MDBPRL)<=(PGEOPF5(JLEN,1,JPROF)/RG)) THEN
                     JK = COUNT((PGEOPF5(JLEN,:,JPROF)/RG)<=ROBODY(IBODY,MDBPRL))
                     WRITE(NULOUT,*)"la valeur de JK est",JK
                     IF (PGEOPF5(JLEN,NFLEVG-JK,JPROF)/=PGEOPF5(JLEN,NFLEVG-JK+1,JPROF))& 
                       & THEN
                       ZZCOEF = (ROBODY(IBODY,MDBPRL)-PGEOPF5(JLEN,NFLEVG-JK+1,JPROF)/RG)/&
                       & (PGEOPF5(JLEN,NFLEVG-JK,JPROF)/RG-PGEOPF5(JLEN,NFLEVG-JK+1,JPROF)/RG)
                     ELSE 
                       ZZCOEF = 1
                     ENDIF
                     ZHU(JLEN,JCOUNT,JPROF) = (1.0_JPRB - ZZCOEF)*PQF5(JLEN,NFLEVG-JK+1,JPROF)&
                      & + ZZCOEF*PQF5(JLEN,NFLEVG-JK,JPROF)
                  ELSE
!                  ZHU(JLEN,JCOUNT,JPROF) = PQF5(JLEN,NFLEVG,JPROF)
                   CYCLE
                  ENDIF
                  ZEXP = 0._JPRB

                  DO JJCOUNT=1,KCOUNT(JLEN) ! loop for zexp
                     IIBODY = MLNKH2B(JLEN)+ KPNLV(JLEN,JJCOUNT)-1
                     IF (ROBODY(IIBODY,MDBVAR)/=RMDI) THEN
                     ZEXP = ZEXP + (ROBODY(IIBODY,MDBVAR)-&
                            & PREFL2D(JLEN,JJCOUNT,1,JPROF))**2
                     ENDIF
                  ENDDO ! loop for zexp
                 IF (JPROF==25)  WRITE(NULOUT,*)"ZEXP vaut,obsvalue,refl2d",&
                       & ZEXP,ROBODY(IBODY,MDBVAR),PREFL2D(JLEN,JCOUNT,1,JPROF)

                  IF (KCOUNT(JLEN)/=0) THEN
                          ZEXP = ZEXP/ZXSIG**2/KCOUNT(JLEN)
                  ENDIF
                  ZREHU(JLEN,JCOUNT)= ZREHU(JLEN,JCOUNT) + &
                  & ZHU(JLEN,JCOUNT,JPROF)*ZEXP
                  
                  ZDENOM = ZDENOM + ZEXP

        ENDDO ! loop over model profiles
! une fois sorti des profiles voisins on norme 
     IF (ZDENOM/=0._JPRB) ZREHU(JLEN,JCOUNT) = ZREHU(JLEN,JCOUNT)/ZDENOM
!         ZREHU(JLEN,JCOUNT) =  ZREHU(JLEN,JCOUNT)
        SATBODY(IBODY,MDB_Q1_AT_RADAR_BODY) = ZREHU(JLEN,JCOUNT)
!        WRITE(NULOUT,*)"SATBODY de humidite vaut",IBODY, SATBODY(IBODY,MDB_Q1_AT_RADAR_BODY)
         ROBODY(IBODY+1,MDBVAR) = ZREHU(JLEN,JCOUNT)
         WRITE(NULOUT,*)"ROBODY(MDBVNM) de IBODY+1",ROBODY(IBODY+1,MDBVNM)

! below interpolation of the pression level on altitude of the observation
         IF (ROBODY(IBODY,MDBPRL)>=(PGEOPF5(JLEN,NFLEVG,1)/RG).AND.&
                   & ROBODY(IBODY,MDBPRL)<=(PGEOPF5(JLEN,1,1)/RG)) THEN

                     JK = COUNT((PGEOPF5(JLEN,:,1)/RG)<=ROBODY(IBODY,MDBPRL))

                     ZZCOEFINTP = (ROBODY(IBODY,MDBPRL)-PGEOPF5(JLEN,NFLEVG-JK+1,1)/RG)/ &
              &      (PGEOPF5(JLEN,NFLEVG-JK,1)/RG-PGEOPF5(JLEN,NFLEVG-JK+1,1)/RG) 
                     PPRESPIX(JLEN,JCOUNT)=LOG(PPRESF5(JLEN,NFLEVG-JK+1,1))+&
              &      (LOG(PPRESF5(JLEN,NFLEVG-JK,1))-LOG(PPRESF5(JLEN,NFLEVG-JK+1,1)))* &
              &      ZZCOEFINTP 

                     ROBODY(IBODY+1,MDBPPP)=EXP(PPRESPIX(JLEN,JCOUNT))

                     WRITE(NULOUT,*)"ROBODY(MDBVNM) de IBODY+1,pres(pix),jk,pres(jk)",ROBODY(IBODY+1,MDBVNM)&
                  & ,ROBODY(IBODY+1,MDBPPP),JK,PPRESF5(JLEN,NFLEVG-JK+1,1),ZZCOEFINTP,& 
&                    (LOG(PPRESF5(JLEN,NFLEVG-JK,1))-LOG(PPRESF5(JLEN,NFLEVG-JK+1,1)))
         ENDIF
   ENDDO ! loop over elevations
ENDDO ! loop over obs columns
IF (LHOOK) CALL DR_HOOK('INV_REFL1DSTAT',1,ZHOOK_HANDLE)

END SUBROUTINE INV_REFL1DSTAT
