!  Differentiation of iso_gems in reverse (adjoint) mode:
!   gradient, with respect to input variables: pizmoy pbetal ptodaer
!                ptamoy
!   of linear combination of output variables: pizmoy pxf pbetal
!                ptodaer ptamoy

SUBROUTINE ISO_GEMS_AD(ptamoy, ptamoyb, ptray, pizmoy, pizmoyb, pbetal, &
&  pbetalb, prm, pgb, ppsl, ptodaer, ptodaerb, ptodray, pxf, pxfb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE

!****ISO*** COMPUTES ATMOSPHERIC TRANSMISSION


!     INTERFACE
!     ---------
! ISO IS CALLED BY SCATRA

!     INPUTS
!     ------

! PRM(-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU)       : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAMOY(KNT)         : TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
! PTRAY(KNT)          : MOLECULAR OPTICAL THICKNESS PROFILE.
! PIZMOY(KNT)         : TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM
! PPSL(0:KNANG,-1:KNANG,-KMU:KMU)
! PTODAER             : TOTAL AEROSOL OPTICAL DEPTH
! PTODRAY             : TOTAL MOLECULAR OPTICAL DEPTH

!     OUTPUTS
!     -------

! PXF(-1,1) : ATMOSPHERIC TRANSMISSION 

!     EXTERNALS.
!     ----------
!          KERNEL

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM ISO.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
! GAUSSIAN ANGLES FOR THE ZENITH
  REAL(KIND=JPRB), INTENT(IN) :: prm(-kmu:kmu)
! GAUSSIAN WEIGHTS FOR THE ZENITH
  REAL(KIND=JPRB), INTENT(IN) :: pgb(-kmu:kmu)
! TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
  REAL(KIND=JPRB),INTENT(IN) :: ptamoy(0:knt)
  REAL(KIND=JPRB),INTENT(INOUT) :: ptamoyb(0:knt)
! MOLECULAR OPTICAL THICKNESS PROFILE.
  REAL(KIND=JPRB), INTENT(IN) :: ptray(0:knt)
! TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
  REAL(KIND=JPRB),INTENT(IN) :: pizmoy(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: pizmoyb(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: pbetal(0:knt, 0:knang), ptodaer, ptodray
  REAL(KIND=JPRB), INTENT(INOUT) :: pbetalb(0:knt, 0:knang), ptodaerb
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: pxf(-1:1)
  REAL(KIND=JPRB), INTENT(INOUT) :: pxfb(-1:1)
! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KNP=49, KNANG=80, KNT=20
  INTEGER(KIND=JPIM) :: jlev, jmu, jmu2, ig
  REAL(KIND=JPRB) :: zh(0:knt), zydel(0:knt), zxdel(0:knt)
  REAL(KIND=JPRB) :: zhb(0:knt), zydelb(0:knt), zxdelb(0:knt)
  REAL(KIND=JPRB) :: zaaa, zron, zbeta0, zbeta2
! AEROSOL PHASE FUNCTION
  REAL(KIND=JPRB) :: zi1(0:knt, -kmu:kmu), zi2(0:knt, -kmu:kmu)
  REAL(KIND=JPRB) :: zi1b(0:knt, -kmu:kmu), zi2b(0:knt, -kmu:kmu)
  REAL(KIND=JPRB) :: zi3(-kmu:kmu), zin(-kmu:kmu)
  REAL(KIND=JPRB) :: zi3b(-kmu:kmu), zinb(-kmu:kmu)
  REAL(KIND=JPRB) :: zinm1(-kmu:kmu), zinm2(-kmu:kmu)
  REAL(KIND=JPRB) :: zinm1b(-kmu:kmu)
  REAL(KIND=JPRB) :: zii1, zii2, zbpjk, zbpjmk, zxi1, zxi2
  REAL(KIND=JPRB) :: zii1b, zii2b, zbpjkb, zbpjmkb, zxi1b, zxi2b
  REAL(KIND=JPRB) :: z, zx, zxx, zxpk, zy, zyy
  REAL(KIND=JPRB) :: zxb, zxxb, zyb
  REAL(KIND=JPRB) :: za1, zd1, zg1, zy1, za, zb, zc, zd, zf, zzi1
  REAL(KIND=JPRB) :: zd1b, zg1b, zy1b, zab, zbb, zcb, zdb, zfb, zzi1b
  REAL(KIND=JPRB) :: zypk, zxpj, zro, zro0, zro1, zro2, zaux
  REAL(KIND=JPRB) :: zrob, zro0b, zro1b, zauxb
! OUTPUT OF KERNEL
  REAL(KIND=JPRB) :: z_pxpl(-kmu:kmu)
  REAL(KIND=JPRB) :: z_pbp(0:knt, 0:kmu, -kmu:kmu)
  REAL(KIND=JPRB) :: z_pbpb(0:knt, 0:kmu, -kmu:kmu)
  INTEGER(KIND=JPIM) :: i_branch
  INTRINSIC EXP
  REAL(KIND=JPRB) :: z_tempb6
  REAL(KIND=JPRB) :: z_tempb5
  REAL(KIND=JPRB) :: z_tempb4
  REAL(KIND=JPRB) :: z_tempb3
  REAL(KIND=JPRB) :: z_tempb2
  REAL(KIND=JPRB) :: z_tempb1
  REAL(KIND=JPRB) :: z_tempb0
  INTRINSIC MAX
  INTRINSIC ABS
  REAL(KIND=JPRB) :: z_tempb
  REAL(KIND=JPRB) :: z_abs1

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "kernel_pbp.intfb.h"
#include "kernel_pbp_ad.intfb.h"
#include "popinteger4.intfb.h"
#include "popreal8.intfb.h"
#include "pushinteger4.intfb.h"
#include "pushreal8.intfb.h"

  IF (LHOOK) CALL DR_HOOK('ISO_GEMS_AD',0,ZHOOK_HANDLE)

! VARIABLE INITIALIZATION 
  zin(:) = 0.0_JPRB
  zi1(:, :) = 0.0_JPRB
  zinm1(:) = 0.0_JPRB
  zinm2(:) = 0.0_JPRB

! CASE 1: PURE RAYLEIGH
! -------

  IF (ptodaer <= accu2 .AND. ptodray  >   ptodaer) THEN
    zaux = 0.0_JPRB
! JLEV=0 CORRESPONDS TO TOP OF THE ATMOSPHERE
    DO jlev=0,knt
      zaux = zaux + ptray(jlev)
      zh(jlev) = zaux
      zydel(jlev) = 1.0_JPRB
      zxdel(jlev) = 0.0_JPRB
    ENDDO
  ENDIF

! CASE 2: PURE AEROSOL
! -------

  IF (ptodray <= accu2 .AND. ptodaer  >   ptodray) THEN
    zaux = 0.0_JPRB
    DO jlev=0,knt
      zaux = zaux + ptamoy(jlev)
      zh(jlev) = zaux
      zydel(jlev) = 0.0_JPRB
      zxdel(jlev) = pizmoy(jlev)
    ENDDO
    CALL PUSHINTEGER4(1)
  ELSE
    CALL PUSHINTEGER4(0)
  ENDIF

! CASE 3: MIXTURE
! -------

  IF (ptodray  >   accu2 .AND. ptodaer  >   accu2) THEN
    zaux = 0.0_JPRB
    DO jlev=0,knt
      zaux = zaux + ptamoy(jlev) + ptray(jlev)
      zh(jlev) = zaux
      zxdel(jlev) = pizmoy(jlev)*ptamoy(jlev)/zh(jlev)
      zydel(jlev) = ptray(jlev)/zh(jlev)
    ENDDO
    CALL PUSHINTEGER4(1)
  ELSE
    CALL PUSHINTEGER4(0)
  ENDIF

! INCIDENT ANGLE MU_S

  zaaa = delta/(2.0_JPRB-delta)
  zron = (1.0_JPRB-zaaa)/(1.0_JPRB+2.0_JPRB*zaaa)

! RAYLEIGH PHASE FUNCTION

  zbeta0 = 1.0_JPRB
  zbeta2 = 0.5_JPRB*zron

! PRIMARY SCATTERING

  zro1 = 0.0_JPRB
  zi3(:) = 0.0_JPRB

! KERNEL COMPUTATIONS

  CALL KERNEL_PBP(0, ppsl, pbetal, z_pxpl, z_pbp)
  zi2(:, :) = 0.0_JPRB

! VERTICAL INTEGRATION, PRIMARY UPWARD RADIATION

  DO jmu=1,kmu
    zi1(knt, jmu) = 1.0_JPRB
    CALL PUSHREAL8(zyy)
    zyy = prm(jmu)
    DO jlev=knt-1,0,-1
      zi1(jlev, jmu) = EXP(-((ptodaer+ptodray-zh(jlev))/zyy))
    ENDDO
  ENDDO

! VERTICAL INTEGRATION, PRIMARY DOWNWARD RADIATION

  DO jmu=-kmu,-1
    DO jlev=0,knt
      zi1(jlev, jmu) = 0.0_JPRB
    ENDDO
  ENDDO

! ZINM2 IS INITIALIZED WITH SCATTERING COMPUTED AT N-2
! ZI3 IS INITIALIZED WITH PRIMARY SCATTERING

  DO jmu=-kmu,-1
    zinm1(jmu) = zi1(knt, jmu)
    zinm2(jmu) = zi1(knt, jmu)
    zi3(jmu) = zi1(knt, jmu)
  ENDDO
  DO jmu=1,kmu
    zinm1(jmu) = zi1(0, jmu)
    zinm2(jmu) = zi1(0, jmu)
    zi3(jmu) = zi1(0, jmu)
  ENDDO
  zro2 = zi1(0, kmu)
  zro = zi1(0, kmu)

! LOOP ON SUCCESSIVE ORDER

  DO ig=2,nord

! SUCCESSIVE ORDERS

! MULTIPLE SCATTERING SOURCE FUNCTION AT EVERY LEVEL WITHIN THE LAYER

    DO jmu=1,kmu
      CALL PUSHREAL8(zxpk)
      zxpk = z_pxpl(jmu)
      CALL PUSHREAL8(zypk)
      zypk = z_pxpl(-jmu)
      DO jlev=0,knt
        zii1 = 0.0_JPRB
        zii2 = 0.0_JPRB
        CALL PUSHREAL8(zx)
        zx = zxdel(jlev)
        zy = zydel(jlev)
        DO jmu2=1,kmu
          CALL PUSHREAL8(zxpj)
          zxpj = z_pxpl(jmu2)
          CALL PUSHREAL8(z)
          z = pgb(jmu2)
          CALL PUSHREAL8(zxi1)
          zxi1 = zi1(jlev, jmu2)
          CALL PUSHREAL8(zxi2)
          zxi2 = zi1(jlev, -jmu2)
          CALL PUSHREAL8(zbpjk)
          zbpjk = z_pbp(jlev, jmu2, jmu)*zx + zy*(zbeta0+zbeta2*zxpj*zxpk)
          CALL PUSHREAL8(zbpjmk)
          zbpjmk = z_pbp(jlev, jmu2, -jmu)*zx + zy*(zbeta0+zbeta2*zxpj*&
&            zypk)
          zii2 = zii2 + z*(zxi1*zbpjk+zxi2*zbpjmk)
          zii1 = zii1 + z*(zxi1*zbpjmk+zxi2*zbpjk)
        ENDDO
        CALL PUSHREAL8(zi2(jlev, jmu))
        zi2(jlev, jmu) = zii2
        CALL PUSHREAL8(zi2(jlev, -jmu))
        zi2(jlev, -jmu) = zii1
      ENDDO
    ENDDO

! VERTICAL INTEGRATION, UPWARD RADIATION

    DO jmu=1,kmu
      zi1(knt, jmu) = 0.0_JPRB
      zzi1 = zi1(knt, jmu)
      CALL PUSHREAL8(zyy)
      zyy = prm(jmu)
      DO jlev=knt-1,0,-1
        CALL PUSHREAL8(zf)
        zf = zh(jlev+1) - zh(jlev)
        CALL PUSHREAL8(za)
        za = (zi2(jlev+1, jmu)-zi2(jlev, jmu))/zf
        CALL PUSHREAL8(zb)
        zb = zi2(jlev, jmu) - za*zh(jlev)
        CALL PUSHREAL8(zc)
        zc = EXP(-(zf/zyy))
        CALL PUSHREAL8(zd)
        zd = 1.d+00 - zc
        CALL PUSHREAL8(zxx)
        zxx = zh(jlev) - zh(jlev+1)*zc
        CALL PUSHREAL8(zzi1)
        zzi1 = zc*zzi1 + (zd*(zb+za*zyy)+za*zxx)*0.5_JPRB
        zi1(jlev, jmu) = zzi1
      ENDDO
    ENDDO

! VERTICAL INTEGRATION, DOWNWARD RADIATION

    DO jmu=-kmu,-1
      zi1(0, jmu) = 0.0_JPRB
      zzi1 = zi1(0, jmu)
      CALL PUSHREAL8(zyy)
      zyy = prm(jmu)
      DO jlev=1,knt
        CALL PUSHREAL8(zf)
        zf = zh(jlev) - zh(jlev-1)
        CALL PUSHREAL8(zc)
        zc = EXP(zf/zyy)
        CALL PUSHREAL8(zd)
        zd = 1.d+00 - zc
        CALL PUSHREAL8(za)
        za = (zi2(jlev, jmu)-zi2(jlev-1, jmu))/zf
        CALL PUSHREAL8(zb)
        zb = zi2(jlev, jmu) - za*zh(jlev)
        CALL PUSHREAL8(zxx)
        zxx = zh(jlev) - zh(jlev-1)*zc
        CALL PUSHREAL8(zzi1)
        zzi1 = zc*zzi1 + (zd*(zb+za*zyy)+za*zxx)*0.5_JPRB
        zi1(jlev, jmu) = zzi1
      ENDDO
    ENDDO

! IN IS THE NIEME SCATTERING ORDER

    DO jmu=-kmu,-1
      zin(jmu) = zi1(knt, jmu)
    ENDDO
    DO jmu=1,kmu
      zin(jmu) = zi1(0, jmu)
    ENDDO
    zro0 = zi1(0, kmu)

! CONVERGENCE TEST (GEOMETRICAL SERIE)

    IF (ig  >   2) THEN
      CALL PUSHREAL8(z)
      z = 0.0_JPRB
      za1 = zro2
      CALL PUSHREAL8(zd1)
      zd1 = zro1
      zg1 = zro0
      IF (za1  >=  accu .AND. zd1  >=  accu .AND. zro  >=  accu) THEN
        zy = (zg1/zd1-zd1/za1)/(1.-zg1/zd1)**2*(zg1/zro)
        IF (zy  >=  0.) THEN
          zy = zy
        ELSE
          zy = -zy
        ENDIF
        IF (z  <   zy) THEN
          z = zy
        ELSE
          z = z
        ENDIF
      ENDIF
      DO jmu=-kmu,-1
        za1 = zinm2(jmu)
        zd1 = zinm1(jmu)
        zg1 = zin(jmu)
        IF (za1  /=  0.0_JPRB .AND. zd1  /=  0.0_JPRB .AND. zi3(jmu)  /=  &
&            0.0_JPRB) THEN
          zy = (zg1/zd1-zd1/za1)/(1.-zg1/zd1)**2*(zg1/zi3(jmu))
          IF (zy  >=  0.) THEN
            zy = zy
          ELSE
            zy = -zy
          ENDIF
          IF (zy  <   z) THEN
            z = z
          ELSE
            z = zy
          ENDIF
        ENDIF
      ENDDO
      DO jmu=1,kmu
        za1 = zinm2(jmu)
        zd1 = zinm1(jmu)
        zg1 = zin(jmu)
        IF (za1  /=  0.0_JPRB .AND. zd1  /=  0.0_JPRB .AND. zi3(jmu)  /=  &
&            0.0_JPRB) THEN
          zy = (zg1/zd1-zd1/za1)/(1.-zg1/zd1)**2*(zg1/zi3(jmu))
          IF (zy  >=  0.) THEN
            zy = zy
          ELSE
            zy = -zy
          ENDIF
          IF (zy  <   z) THEN
            z = z
          ELSE
            z = zy
          ENDIF
        ENDIF
      ENDDO
      IF (z  <   accu4) THEN
! SUCCESSFUL TEST (GEOMETRICAL SERIE)
        DO jmu=-kmu,-1
          CALL PUSHREAL8(zd1)
          zd1 = zinm1(jmu)
          zg1 = zin(jmu)
          IF (zd1  /=  0.0_JPRB) THEN
            CALL PUSHREAL8(zy1)
            zy1 = 1.0_JPRB - zg1/zd1
            CALL PUSHREAL8(zg1)
            zg1 = zg1/zy1
            zi3(jmu) = zi3(jmu) + zg1
            CALL PUSHINTEGER4(2)
          ELSE
            CALL PUSHINTEGER4(1)
          ENDIF
        ENDDO
        DO jmu=1,kmu
          CALL PUSHREAL8(zd1)
          zd1 = zinm1(jmu)
          zg1 = zin(jmu)
          IF (zd1  /=  0.0_JPRB) THEN
            CALL PUSHREAL8(zy1)
            zy1 = 1.0_JPRB - zg1/zd1
            CALL PUSHREAL8(zg1)
            zg1 = zg1/zy1
            zi3(jmu) = zi3(jmu) + zg1
            CALL PUSHINTEGER4(2)
          ELSE
            CALL PUSHINTEGER4(1)
          ENDIF
        ENDDO
        CALL PUSHREAL8(zd1)
        zd1 = zro1
        zg1 = zro0
        IF (zd1  >=  accu) THEN
          IF (zg1 - zd1  >=  0.) THEN
            z_abs1 = zg1 - zd1
          ELSE
            z_abs1 = -(zg1-zd1)
          ENDIF
          IF (z_abs1  >=  accu) THEN
            CALL PUSHREAL8(zy1)
            zy1 = 1.0_JPRB - zg1/zd1
            CALL PUSHREAL8(zg1)
            zg1 = zg1/zy1
            CALL PUSHINTEGER4(1)
          ELSE
            CALL PUSHINTEGER4(0)
          ENDIF
          zro = zro + zg1
          CALL PUSHINTEGER4(3)
        ELSE
          CALL PUSHINTEGER4(2)
        ENDIF
      ELSE
        CALL PUSHINTEGER4(1)
!      EXIT 
! ZINM2 IS THE (N-2)ieme SCATTERING ORDER
        DO jmu=-kmu,kmu
          zinm2(jmu) = zinm1(jmu)
        ENDDO
        zro2 = zro1
      ENDIF
    ELSE
      CALL PUSHINTEGER4(0)
    ENDIF
    IF (ig <= 2 .OR. z  >=  accu4) THEN

! ZINM1 IS THE (N-1)ieme SCATTERING ORDER

      DO jmu=-kmu,kmu
        zinm1(jmu) = zin(jmu)
      ENDDO
      zro1 = zro0
! SUM OF THE N-1 ORDERS

      DO jmu=-kmu,kmu
        zi3(jmu) = zi3(jmu) + zin(jmu)
      ENDDO
      zro = zro + zro0
!!
!! STOP IF ORDER N IS LESS THAN 1% OF THE SUM
!!
!    Z=0.0_JPRB
!    DO JMU=-KMU,KMU
!      IF (ZI3(JMU) /= 0.0_JPRB) THEN
!        ZY=ABS(ZIN(JMU)/ZI3(JMU))
!        Z=MAX(Z,ZY)
!      ENDIF
!    ENDDO
!    IF ((Z <  ACCU5).OR.(IG >  SCATORD)) THEN
!      EXIT
!    ENDIF
      CALL PUSHINTEGER4(2)
    ELSE
      CALL PUSHINTEGER4(1)
    ENDIF
  ENDDO
  zi3b(-kmu:kmu) = 0._JPRB
  DO jmu=kmu,1,-1
    zi3b(-jmu) = zi3b(-jmu) + prm(jmu)*pgb(jmu)*pxfb(0)
  ENDDO
  zrob = pxfb(-1)
  pxfb(-1) = 0._JPRB
  zi3b(kmu) = zi3b(kmu) + pxfb(1)
  zinb(-kmu:kmu) = 0._JPRB
  zydelb(0:knt) = 0._JPRB
  zxdelb(0:knt) = 0._JPRB
  zro1b = 0._JPRB
  zi1b(0:knt, -kmu:kmu) = 0._JPRB
  zi2b(0:knt, -kmu:kmu) = 0._JPRB
  zinm1b(-kmu:kmu) = 0._JPRB
  z_pbpb(0:knt, 0:kmu, -kmu:kmu) = 0._JPRB
  zhb(0:knt) = 0._JPRB
  DO ig=nord,2,-1
    CALL POPINTEGER4(i_branch)
    IF (i_branch  <   2) THEN
      zro0b = 0._JPRB
    ELSE
      zro0b = zrob
      DO jmu=kmu,-kmu,-1
        zinb(jmu) = zinb(jmu) + zi3b(jmu)
      ENDDO
      zro0b = zro0b + zro1b
      DO jmu=kmu,-kmu,-1
        zinb(jmu) = zinb(jmu) + zinm1b(jmu)
        zinm1b(jmu) = 0._JPRB
      ENDDO
      zro1b = 0._JPRB
    ENDIF
    CALL POPINTEGER4(i_branch)
    IF (i_branch  <   2) THEN
      IF (i_branch  <   1) GOTO 100
    ELSE
      IF (i_branch  <   3) THEN
        zg1b = 0._JPRB
        zd1b = 0._JPRB
      ELSE
        zg1b = zrob
        CALL POPINTEGER4(i_branch)
        IF (i_branch  <   1) THEN
          zd1b = 0._JPRB
        ELSE
          CALL POPREAL8(zg1)
          zy1b = -(zg1*zg1b/zy1**2)
          zg1b = zg1b/zy1 - zy1b/zd1
          CALL POPREAL8(zy1)
          zd1b = zg1*zy1b/zd1**2
        ENDIF
      ENDIF
      zro0b = zro0b + zg1b
      CALL POPREAL8(zd1)
      zro1b = zro1b + zd1b
      DO jmu=kmu,1,-1
        CALL POPINTEGER4(i_branch)
        IF (i_branch  <   2) THEN
          zg1b = 0._JPRB
          zd1b = 0._JPRB
        ELSE
          zg1b = zi3b(jmu)
          CALL POPREAL8(zg1)
          zy1b = -(zg1*zg1b/zy1**2)
          zg1b = zg1b/zy1 - zy1b/zd1
          CALL POPREAL8(zy1)
          zd1b = zg1*zy1b/zd1**2
        ENDIF
        zinb(jmu) = zinb(jmu) + zg1b
        CALL POPREAL8(zd1)
        zinm1b(jmu) = zinm1b(jmu) + zd1b
      ENDDO
      DO jmu=-1,-kmu,-1
        CALL POPINTEGER4(i_branch)
        IF (i_branch  <   2) THEN
          zg1b = 0._JPRB
          zd1b = 0._JPRB
        ELSE
          zg1b = zi3b(jmu)
          CALL POPREAL8(zg1)
          zy1b = -(zg1*zg1b/zy1**2)
          zg1b = zg1b/zy1 - zy1b/zd1
          CALL POPREAL8(zy1)
          zd1b = zg1*zy1b/zd1**2
        ENDIF
        zinb(jmu) = zinb(jmu) + zg1b
        CALL POPREAL8(zd1)
        zinm1b(jmu) = zinm1b(jmu) + zd1b
      ENDDO
    ENDIF
    CALL POPREAL8(zd1)
    CALL POPREAL8(z)
 100 CONTINUE
     zi1b(0, kmu) = zi1b(0, kmu) + zro0b
    DO jmu=kmu,1,-1
      zi1b(0, jmu) = zi1b(0, jmu) + zinb(jmu)
      zinb(jmu) = 0._JPRB
    ENDDO
    DO jmu=-1,-kmu,-1
      zi1b(knt, jmu) = zi1b(knt, jmu) + zinb(jmu)
      zinb(jmu) = 0._JPRB
    ENDDO
    DO jmu=-1,-kmu,-1
      zzi1b = 0._JPRB
      DO jlev=knt,1,-1
        zzi1b = zzi1b + zi1b(jlev, jmu)
        zi1b(jlev, jmu) = 0._JPRB
        CALL POPREAL8(zzi1)
        z_tempb5 = 0.5_JPRB*zzi1b
        zdb = (zb+zyy*za)*z_tempb5
        zxxb = za*z_tempb5
        zcb = zzi1*zzi1b - zdb - zh(jlev-1)*zxxb
        zbb = zd*z_tempb5
        zab = (zxx+zd*zyy)*z_tempb5 - zh(jlev)*zbb
        zzi1b = zc*zzi1b
        CALL POPREAL8(zxx)
        zhb(jlev) = zhb(jlev) + zxxb
        zhb(jlev-1) = zhb(jlev-1) - zc*zxxb
        CALL POPREAL8(zb)
        z_tempb6 = zab/zf
        zi2b(jlev, jmu) = zi2b(jlev, jmu) + z_tempb6 + zbb
        zfb = EXP(zf/zyy)*zcb/zyy - (zi2(jlev, jmu)-zi2(jlev-1, jmu))*&
&          z_tempb6/zf
        zhb(jlev) = zhb(jlev) + zfb - za*zbb
        CALL POPREAL8(za)
        zi2b(jlev-1, jmu) = zi2b(jlev-1, jmu) - z_tempb6
        CALL POPREAL8(zd)
        CALL POPREAL8(zc)
        CALL POPREAL8(zf)
        zhb(jlev-1) = zhb(jlev-1) - zfb
      ENDDO
      CALL POPREAL8(zyy)
      zi1b(0, jmu) = 0._JPRB
    ENDDO
    DO jmu=kmu,1,-1
      zzi1b = 0._JPRB
      DO jlev=0,knt-1,1
        zzi1b = zzi1b + zi1b(jlev, jmu)
        zi1b(jlev, jmu) = 0._JPRB
        CALL POPREAL8(zzi1)
        z_tempb3 = 0.5_JPRB*zzi1b
        zdb = (zb+zyy*za)*z_tempb3
        zxxb = za*z_tempb3
        zcb = zzi1*zzi1b - zdb - zh(jlev+1)*zxxb
        zbb = zd*z_tempb3
        zab = (zxx+zd*zyy)*z_tempb3 - zh(jlev)*zbb
        zzi1b = zc*zzi1b
        CALL POPREAL8(zxx)
        zhb(jlev) = zhb(jlev) + zxxb
        zhb(jlev+1) = zhb(jlev+1) - zc*zxxb
        CALL POPREAL8(zd)
        CALL POPREAL8(zc)
        z_tempb4 = zab/zf
        zfb = -((zi2(jlev+1, jmu)-zi2(jlev, jmu))*z_tempb4/zf) - EXP(-(zf/&
&          zyy))*zcb/zyy
        CALL POPREAL8(zb)
        zi2b(jlev, jmu) = zi2b(jlev, jmu) + zbb
        zhb(jlev) = zhb(jlev) - za*zbb
        CALL POPREAL8(za)
        zi2b(jlev+1, jmu) = zi2b(jlev+1, jmu) + z_tempb4
        zi2b(jlev, jmu) = zi2b(jlev, jmu) - z_tempb4
        CALL POPREAL8(zf)
        zhb(jlev+1) = zhb(jlev+1) + zfb
        zhb(jlev) = zhb(jlev) - zfb
      ENDDO
      CALL POPREAL8(zyy)
      zi1b(knt, jmu) = 0._JPRB
    ENDDO
    DO jmu=kmu,1,-1
      DO jlev=knt,0,-1
        CALL POPREAL8(zi2(jlev, -jmu))
        zii1b = zi2b(jlev, -jmu)
        zi2b(jlev, -jmu) = 0._JPRB
        CALL POPREAL8(zi2(jlev, jmu))
        zii2b = zi2b(jlev, jmu)
        zi2b(jlev, jmu) = 0._JPRB
        zxb = 0._JPRB
        zyb = 0._JPRB
        DO jmu2=kmu,1,-1
          z_tempb1 = z*zii1b
          z_tempb2 = z*zii2b
          zxi1b = zbpjk*z_tempb2 + zbpjmk*z_tempb1
          zbpjmkb = zxi2*z_tempb2 + zxi1*z_tempb1
          zxi2b = zbpjmk*z_tempb2 + zbpjk*z_tempb1
          zbpjkb = zxi1*z_tempb2 + zxi2*z_tempb1
          CALL POPREAL8(zbpjmk)
          z_pbpb(jlev, jmu2, -jmu) = z_pbpb(jlev, jmu2, -jmu) + zx*zbpjmkb
          zxb = zxb + z_pbp(jlev, jmu2, jmu)*zbpjkb + z_pbp(jlev, jmu2, -jmu&
&            )*zbpjmkb
          zyb = zyb + (zbeta0+zbeta2*zxpj*zxpk)*zbpjkb + (zbeta0+zbeta2*&
&            zxpj*zypk)*zbpjmkb
          CALL POPREAL8(zbpjk)
          z_pbpb(jlev, jmu2, jmu) = z_pbpb(jlev, jmu2, jmu) + zx*zbpjkb
          CALL POPREAL8(zxi2)
          zi1b(jlev, -jmu2) = zi1b(jlev, -jmu2) + zxi2b
          CALL POPREAL8(zxi1)
          zi1b(jlev, jmu2) = zi1b(jlev, jmu2) + zxi1b
          CALL POPREAL8(z)
          CALL POPREAL8(zxpj)
        ENDDO
        zydelb(jlev) = zydelb(jlev) + zyb
        CALL POPREAL8(zx)
        zxdelb(jlev) = zxdelb(jlev) + zxb
      ENDDO
      CALL POPREAL8(zypk)
      CALL POPREAL8(zxpk)
    ENDDO
  ENDDO
  zi1b(0, kmu) = zi1b(0, kmu) + zrob
  DO jmu=kmu,1,-1
    zi1b(0, jmu) = zi1b(0, jmu) + zinm1b(jmu) + zi3b(jmu)
    zi3b(jmu) = 0._JPRB
    zinm1b(jmu) = 0._JPRB
  ENDDO
  DO jmu=-1,-kmu,-1
    zi1b(knt, jmu) = zi1b(knt, jmu) + zinm1b(jmu) + zi3b(jmu)
    zi3b(jmu) = 0._JPRB
    zinm1b(jmu) = 0._JPRB
  ENDDO
  DO jmu=-1,-kmu,-1
    DO jlev=knt,0,-1
      zi1b(jlev, jmu) = 0._JPRB
    ENDDO
  ENDDO
  DO jmu=kmu,1,-1
    DO jlev=0,knt-1,1
      z_tempb0 = -(EXP(-((ptodray+ptodaer-zh(jlev))/zyy))*zi1b(jlev, jmu)/&
&        zyy)
      ptodaerb = ptodaerb + z_tempb0
      zhb(jlev) = zhb(jlev) - z_tempb0
      zi1b(jlev, jmu) = 0._JPRB
    ENDDO
    CALL POPREAL8(zyy)
    zi1b(knt, jmu) = 0._JPRB
  ENDDO
  CALL KERNEL_PBP_AD(0, ppsl, pbetal, pbetalb, z_pxpl, z_pbp, z_pbpb)
  CALL POPINTEGER4(i_branch)
  IF (.NOT.i_branch  <   1) THEN
    zauxb = 0._JPRB
    DO jlev=knt,0,-1
      z_tempb = zxdelb(jlev)/zh(jlev)
      zhb(jlev) = zhb(jlev) - pizmoy(jlev)*ptamoy(jlev)*z_tempb/zh(jlev) -&
&       ptray(jlev)*zydelb(jlev)/zh(jlev)**2
      zydelb(jlev) = 0._JPRB
      pizmoyb(jlev) = pizmoyb(jlev) + ptamoy(jlev)*z_tempb
      zauxb = zauxb + zhb(jlev)
      ptamoyb(jlev) = ptamoyb(jlev) + zauxb + pizmoy(jlev)*z_tempb
      zxdelb(jlev) = 0._JPRB
      zhb(jlev) = 0._JPRB
    ENDDO
  ENDIF
  CALL POPINTEGER4(i_branch)
  IF (.NOT.i_branch  <   1) THEN
    zauxb = 0._JPRB
    DO jlev=knt,0,-1
      pizmoyb(jlev) = pizmoyb(jlev) + zxdelb(jlev)
      zxdelb(jlev) = 0._JPRB
      zauxb = zauxb + zhb(jlev)
      zhb(jlev) = 0._JPRB
      ptamoyb(jlev) = ptamoyb(jlev) + zauxb
    ENDDO
  ENDIF

 IF (LHOOK) CALL DR_HOOK('ISO_GEMS_AD',1,ZHOOK_HANDLE)
END SUBROUTINE ISO_GEMS_AD
