 SUBROUTINE KERNEL_PBP(KIS,PPSL,PBETAL,PXPL,PBP)

!****KERNEL*** COMPUTES THE VALUES OF LEGENDRE POLYNOMIALS USED IN THE 
!              SUCCESSIVE ORDER OF SCATTERING METHOD.

!     INTERFACE
!     ---------
! KERNEL IS CALLED BY OS

!     INPUTS
!     ------

! KIS                  : LEGENDRE COEFFICIENT
! PRM(-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM

!     OUTPUTS
!     -------

! PXPL(-KMU:KMU)          :
! PBP(KNT,0:KMU,-KMU:KMU) :

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------

!     SOURCE
!     ------
! ADAPTED FROM KERNEL.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE RT6SVALUES

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KIS
REAL(KIND=JPRB), INTENT(IN)    :: PBETAL(0:KNT,0:KNANG)
REAL(KIND=JPRB), INTENT(IN)    :: PPSL(0:KNANG,-1:KNANG,-KMU:KMU)
REAL(KIND=JPRB), INTENT(OUT)   :: PXPL(-KMU:KMU)
REAL(KIND=JPRB), INTENT(OUT)   :: PBP(0:KNT,0:KMU,-KMU:KMU)

! LOCAL VARIABLES
! ---------------

!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
INTEGER(KIND=JPIM) :: J, JJ, JJJ, JLEV
REAL(KIND=JPRB) :: ZSBP, ZBT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('KERNEL_PBP',0,ZHOOK_HANDLE)


! INITIALIZATION OF VARIABLES
! ---------------------------

PXPL(:)=0.0_JPRB
PBP(:,:,:)=0.0_JPRB

DO J=-KMU,KMU
  PXPL(J)=PPSL(KIS,2,J)
ENDDO

DO JLEV=0,KNT
  DO J=0,KMU
    DO JJ=-KMU,KMU
      ZSBP=0.0_JPRB
      DO JJJ=KIS,KNANG
        ZBT=PBETAL(JLEV,JJJ)
        ZSBP=ZSBP+PPSL(KIS,JJJ,J)*PPSL(KIS,JJJ,JJ)*ZBT
      ENDDO
      PBP(JLEV,J,JJ)=ZSBP
    ENDDO
  ENDDO
ENDDO
WHERE (abs(PBP(:,:,:)) <  ZEPS) PBP(:,:,:)=0.0_JPRB

IF (LHOOK) CALL DR_HOOK('KERNEL_PBP',1,ZHOOK_HANDLE)

END SUBROUTINE KERNEL_PBP


