!  Differentiation of kernel_pbp in reverse (adjoint) mode:
!   gradient, with respect to input variables: pbetal
!   of linear combination of output variables: pbetal pbp

SUBROUTINE KERNEL_PBP_AD(kis, ppsl, pbetal, pbetalb, pxpl, pbp, pbpb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE

!****KERNEL*** COMPUTES THE VALUES OF LEGENDRE POLYNOMIALS USED IN THE 
!              SUCCESSIVE ORDER OF SCATTERING METHOD.

!     INTERFACE
!     ---------
! KERNEL IS CALLED BY OS

!     INPUTS
!     ------
! kIS                  : LEGENDRE COEFFICIENT
! PRM(-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM

!     OUTPUTS
!     -------
! PXPL(-KMU:KMU)          :
! PBP(KNT,0:KMU,-KMU:KMU) :

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------

!     SOURCE
!     ------
! ADAPTED FROM KERNEL.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
  INTEGER(KIND=JPIM), INTENT(IN) :: kis
  REAL(KIND=JPRB), INTENT(IN) :: pbetal(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(INOUT) :: pbetalb(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(OUT) :: pxpl(-kmu:kmu)
  REAL(KIND=JPRB), INTENT(INOUT) :: pbp(0:knt, 0:kmu, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(INOUT) :: pbpb(0:knt, 0:kmu, -kmu:kmu)
! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
  INTEGER(KIND=JPIM) :: j, jj, jjj, jlev
  REAL(KIND=JPRB) :: zsbp, zbt
  REAL(KIND=JPRB) :: zsbpb, zbtb
  LOGICAL, DIMENSION(0:knt, 0:kmu, -kmu:kmu) :: ll_mask
  LOGICAL, DIMENSION(0:knt, 0:kmu, -kmu:kmu) :: ll_mask0
  INTRINSIC ABS
  REAL(KIND=JPRB) :: z_abs1(0:knt, 0:kmu, -kmu:kmu)

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

!#include "popreal8array.intfb.h"
!#include "pushreal8array.intfb.h"

  IF (LHOOK) CALL DR_HOOK('KERNEL_PBP_AD',0,ZHOOK_HANDLE)

  CALL PUSHREAL8ARRAY(pbp, 27*26*51)

! INITIALIZATION OF VARIABLES
! ---------------------------

  pbp(:, :, :) = 0.0_JPRB
  DO jlev=0,knt
    DO j=0,kmu
      DO jj=-kmu,kmu
        zsbp = 0.0_JPRB
        DO jjj=kis,knang
          zbt = pbetal(jlev, jjj)
          zsbp = zsbp + ppsl(kis, jjj, j)*ppsl(kis, jjj, jj)*zbt
        ENDDO
        pbp(jlev, j, jj) = zsbp
      ENDDO
    ENDDO
  ENDDO
  ll_mask(:, :, :) = pbp(:, :, :)  >=  0.0
  WHERE (ll_mask(:, :, :)) 
    z_abs1 = pbp(:, :, :)
  ELSEWHERE
    z_abs1 = -pbp(:, :, :)
  ENDWHERE
  ll_mask0 = z_abs1  <   zeps
  WHERE (ll_mask0) pbpb(:, :, :) = 0._JPRB
  DO jlev=knt,0,-1
    DO j=kmu,0,-1
      DO jj=kmu,-kmu,-1
        zsbpb = pbpb(jlev, j, jj)
        pbpb(jlev, j, jj) = 0._JPRB
        DO jjj=knang,kis,-1
          zbtb = ppsl(kis, jjj, jj)*ppsl(kis, jjj, j)*zsbpb
          pbetalb(jlev, jjj) = pbetalb(jlev, jjj) + zbtb
        ENDDO
      ENDDO
    ENDDO
  ENDDO
  CALL POPREAL8ARRAY(pbp, 27*26*51)

  IF (LHOOK) CALL DR_HOOK('KERNEL_PBP_AD',1,ZHOOK_HANDLE)

END SUBROUTINE KERNEL_PBP_AD
