!  Differentiation of kernel_pbp in forward (tangent) mode:
!   variations  of output variables: pbp
!   with respect to input variables: pbetal

SUBROUTINE KERNEL_PBP_TL(kis, ppsl, pbetal, pbetald, pxpl, pbp, pbpd)

  USE PARKIND1  ,ONLY : JPIM     ,JPRB
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
  
  USE rt6svalues

  IMPLICIT NONE

!****KERNEL*** COMPUTES THE VALUES OF LEGENDRE POLYNOMIALS USED IN THE 
!              SUCCESSIVE ORDER OF SCATTERING METHOD.

!     INTERFACE
!     ---------
! KERNEL IS CALLED BY OS

!     INPUTS
!     ------
! kIS                  : LEGENDRE COEFFICIENT
! PRM(-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM

!     OUTPUTS
!     -------

! PXPL(-KMU:KMU)          :
! PBP(KNT,0:KMU,-KMU:KMU) :

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------

!     SOURCE
!     ------
! ADAPTED FROM KERNEL.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
  INTEGER(KIND=JPIM), INTENT(IN) :: kis
  REAL(KIND=JPRB), INTENT(IN) :: pbetal(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(IN) :: pbetald(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(OUT) :: pxpl(-kmu:kmu)
  REAL(KIND=JPRB), INTENT(OUT) :: pbp(0:knt, 0:kmu, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(OUT) :: pbpd(0:knt, 0:kmu, -kmu:kmu)
! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
  INTEGER(KIND=JPIM) :: j, jj, jjj, jlev
  REAL(KIND=JPRB) :: zsbp, zbt
  REAL(KIND=JPRB) :: zsbpd, zbtd
  INTRINSIC ABS
  REAL(KIND=JPRB) :: z_abs1(0:knt, 0:kmu, -kmu:kmu)

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  IF (LHOOK) CALL DR_HOOK('KERNEL_PBP_TL',0,ZHOOK_HANDLE)

! INITIALIZATION OF VARIABLES
! ---------------------------

  pxpl(:) = 0.0_JPRB
  pbpd(:, :, :) = 0._JPRB
  pbp(:, :, :) = 0.0_JPRB
  DO j=-kmu,kmu
    pxpl(j) = ppsl(kis, 2, j)
  ENDDO
  pbpd(0:knt, 0:kmu, -kmu:kmu) = 0._JPRB
  DO jlev=0,knt
    DO j=0,kmu
      DO jj=-kmu,kmu
        zsbp = 0.0_JPRB
        zsbpd = 0._JPRB
        DO jjj=kis,knang
          zbtd = pbetald(jlev, jjj)
          zbt = pbetal(jlev, jjj)
          zsbpd = zsbpd + ppsl(kis, jjj, j)*ppsl(kis, jjj, jj)*zbtd
          zsbp = zsbp + ppsl(kis, jjj, j)*ppsl(kis, jjj, jj)*zbt
        ENDDO
        pbpd(jlev, j, jj) = zsbpd
        pbp(jlev, j, jj) = zsbp
      ENDDO
    ENDDO
  ENDDO
  WHERE (pbp(:, :, :) >=  0.0) 
    z_abs1 = pbp(:, :, :)
  ELSEWHERE
    z_abs1 = -pbp(:, :, :)
  ENDWHERE
  WHERE (z_abs1   <  zeps) 
    pbpd(:, :, :) = 0._JPRB
    pbp(:, :, :) = 0.0_JPRB
  ENDWHERE

 IF (LHOOK) CALL DR_HOOK('KERNEL_PBP_TL',1,ZHOOK_HANDLE)
END SUBROUTINE KERNEL_PBP_TL
