 SUBROUTINE KERNEL_PPSL(PRM_AUX,          &
&                                      PPSL)

!****KERNEL*** COMPUTES THE VALUES OF LEGENDRE POLYNOMIALS USED IN THE 
!              SUCCESSIVE ORDER OF SCATTERING METHOD.

!     INTERFACE
!     ---------
! KERNEL IS CALLED BY OS

!     INPUTS
!     ------
! PRM_AUX(KLON,-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH

!     OUTPUTS
!     -------

! PPSL(KLON,0:KNANG,-1:KNANG,-KMU,KMU) :

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------
!      N. HUNEUUS    
!      A. BENEDETTI - ECMWF 06.08.2008

!     SOURCE
!     ------
! ADAPTED FROM KERNEL.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE RT6SVALUES

IMPLICIT NONE

REAL(KIND=JPRB), INTENT(IN)    :: PRM_AUX(KLON,-KMU:KMU)
REAL(KIND=JPRB), INTENT(OUT)   :: PPSL(KLON,0:KNANG,-1:KNANG,-KMU:KMU)

! LOCAL VARIABLES
! ---------------

INTEGER(KIND=JPIM) :: J, JJ, IK, IG, IP, IM, IS, JLON
REAL(KIND=JPRB) :: Z, ZXDB, ZA, ZB, ZC
REAL(KIND=JPRB) :: z_PRM(-KMU:KMU)

REAL(KIND=JPRB) :: ZHOOK_HANDLE


IF (LHOOK) CALL DR_HOOK('KERNEL_PPSL',0,ZHOOK_HANDLE)

! INITIALIZATION OF VARIABLES
! ---------------------------

PPSL(:,:,:,:)=0.0_JPRB

DO JLON=1,KLON
z_PRM(:)=PRM_AUX(JLON,:)
DO IS=0,KNANG
  IF (IS == 0) THEN
    PPSL(JLON,IS,0,0)=1.0_JPRB
    PPSL(JLON,IS,1,0)=z_PRM(0)
    PPSL(JLON,IS,2,0)=(3.0_JPRB*z_PRM(0)*z_PRM(0)-1.0_JPRB)*0.5_JPRB
    DO J=1,KMU
      ZC=z_PRM(J)
      PPSL(JLON,IS,0,-J)=1.0_JPRB
      PPSL(JLON,IS,0,J)=1.0_JPRB
      PPSL(JLON,IS,1,J)=ZC
      PPSL(JLON,IS,1,-J)=-ZC
      ZXDB=(3.0_JPRB*ZC*ZC-1.0_JPRB)*0.5_JPRB
      PPSL(JLON,IS,2,-J)=ZXDB
      PPSL(JLON,IS,2,J)=ZXDB
    ENDDO
    WHERE (ABS(PPSL(JLON,IS,2,:)) <  ZEPS) PPSL(JLON,IS,2,:)=0.0_JPRB
    PPSL(JLON,IS,1,0)=z_PRM(0) 
  ENDIF  
  IF (IS == 1) THEN
    Z=1.0_JPRB-z_PRM(0)*z_PRM(0)
    PPSL(JLON,IS,0,0)=0.0_JPRB
    PPSL(JLON,IS,1,0)=SQRT(Z*0.5_JPRB)
    PPSL(JLON,IS,2,0)=z_PRM(0)*PPSL(JLON,IS,1,0)*RAC3
    PPSL(JLON,IS,2,0)=-1.0_JPRB*PPSL(JLON,IS,2,0)
    DO J=1,KMU
      ZC=z_PRM(J)
      Z=1.0_JPRB-ZC*ZC
      PPSL(JLON,IS,0,J)=0.0_JPRB
      PPSL(JLON,IS,0,-J)=0.0_JPRB
      PPSL(JLON,IS,1,-J)=SQRT(Z*0.5_JPRB)
      PPSL(JLON,IS,1,J)=SQRT(Z*0.5_JPRB)
      PPSL(JLON,IS,2,J)=ZC*PPSL(JLON,IS,1,J)*RAC3
      PPSL(JLON,IS,2,-J)=-1.0_JPRB*PPSL(JLON,IS,2,J)
    ENDDO
    PPSL(JLON,IS,2,0)=-1.0_JPRB*PPSL(JLON,IS,2,0)
  ENDIF
  IF ((IS /= 0).AND.(IS /= 1)) THEN
    ZA=1.0_JPRB
    DO J=1,IS
      ZA=ZA*SQRT(DBLE(J+IS)/DBLE(J))*0.5_JPRB
    ENDDO
    ZB=ZA*SQRT(DBLE(IS)/(DBLE(IS)+1.0_JPRB))*SQRT((DBLE(IS)-1.0_JPRB)/(DBLE(IS)+2.0_JPRB))
    DO J=0,KMU
      ZC=z_PRM(J)
      Z=1.0_JPRB-ZC*ZC
      PPSL(JLON,IS,IS-1,J)=0.0_JPRB
      ZXDB=ZA*Z**(DBLE(IS)*0.5_JPRB)
      PPSL(JLON,IS,IS,-J)=ZXDB
      PPSL(JLON,IS,IS,J)=ZXDB
    ENDDO
    WHERE (PPSL(JLON,IS,IS,:) <  ZEPS) PPSL(JLON,IS,IS,:)=0.0_JPRB
  ENDIF
  IK=2
  IF (IS >  2) IK=IS

  IF (IK /= KNANG) THEN
    IG=-1
    IF(IS == 1) IG=1
    DO J=IK,KNANG-1
      IP=J+1
      IM=J-1
      ZA=(2.0_JPRB*DBLE(J)+1.0_JPRB)/SQRT((DBLE(J+IS)+1.0_JPRB)*(DBLE(J-IS)+1.0_JPRB))
      ZB=SQRT(DBLE((J+IS)*(J-IS)))/(2.0_JPRB*DBLE(J)+1.0_JPRB)
! FOR JJ=0
      ZC=z_PRM(0)
      ZXDB=ZA*(ZC*PPSL(JLON,IS,J,0)-ZB*PPSL(JLON,IS,IM,0))
      PPSL(JLON,IS,IP,0)=ZXDB
      DO JJ=1,KMU
        ZC=z_PRM(JJ)
        ZXDB=ZA*(ZC*PPSL(JLON,IS,J,JJ)-ZB*PPSL(JLON,IS,IM,JJ))
        PPSL(JLON,IS,IP,JJ)=ZXDB
        PPSL(JLON,IS,IP,-JJ)=DBLE(IG)*PPSL(JLON,IS,IP,JJ)
      ENDDO
      WHERE (abs(PPSL(JLON,IS,IP,:)) <  ZEPS) PPSL(JLON,IS,IP,:)=0.0_JPRB
      IG=-1*IG
    ENDDO
  ENDIF
ENDDO
ENDDO


IF (LHOOK) CALL DR_HOOK('KERNEL_PPSL',1,ZHOOK_HANDLE)
END SUBROUTINE KERNEL_PPSL
