SUBROUTINE MOPITT_AK_OP(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,&
 & KAK,PWAK,PAPAK,PRESO,PRESH5,PRESF5,PXP5,PXPD5,YDGEMSPROF5,&
 & KDPP,PXGEMS)  

!**** MOPITT_AK_OP - Observation operator for MOPITT V4

!     A. Dethof              ECMWF           22-Feb-06

!     PURPOSE
!     -------

!        FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBSERVED.
!        Based on MOPITT Version 4 Product User's Guide

!**   INTERFACE
!     ---------

!        CALL MOPITT_AK_OP(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,
!    I                  KAK,PWAK,PRESO,PRESH5,PRESF5,PXP5,PXPD5,YDGEMSPROF5,
!    O                  KDPP,PXGEMS)
!     WHERE KDLEN   : FIRST DIMENSION (Observations)            (INPUT)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS             (INPUT)
!           KDCOUNT : SECOND DIMENSION (Data/bodies per observation),
!                     =1 for total columns, > 1 for partial columns (INPUT)
!           CDPAR   : PARAMETER FOR POST-PROCESSING             (INPUT)
!           KOBSTYPE: OBS. TYPE                                 (INPUT)
!           KCOUNT  : COUNTER FOR THE PARTICULAR OBS. PARAMETER (INPUT)
!           KAK     : Number of averaging kernel elements       (INPUT)
!           PWAK    : Weights for averaging kernels             (INPUT)
!           PAPAK   : Apriori values for averaging kernels      (INPUT)
!           PRESO   : Output PRESSURES (ak pressures)           (INPUT)
!           PRESH5  : HALF LEVEL PRESSURES                      (INPUT)
!           PRESF5  : FULL LEVEL PRESSURES                      (INPUT)
!           PXP5    : INTERMEDIATE ARRAY                        (INPUT)
!           PXPD5   : ..................                        (INPUT)
!           YDGEMSPROF5: GRG AT MODEL FULL LEVELS                (INPUT)
!           KDPP    : Number of fields, dimension of PXPP
!           PXGEMS   : GRG layer value                           (OUTPUT)

!        MOPITT_AK_OP IS CALLED FROM HOP

!**   METHOD
!     ------

!        Calculate model total CO column (Col)  as:
!        Col_mod=Col_ap + wak*(x_mod-x_ap), where
!        wak= averaging kernel vector (transpose)
!        x_mod, x_ap= model and apriori log(VMR) profiles (vmr has to be in ppb) on ak levels
!        Col_ap= K*SUM(zdp*VMR(x_ap)): apriori total column
!        ZK=2.12E13(mol/cm2)/(hPa ppb)=9.858e-11(kg/m2)/(Pa ppb)
!        Retrieval pressure grid is 100, 200, 300, 400, 500 ,600, 700, 800, 900, psurf
!        Fewer levels if psurf<900 hPa.

!**   EXTERNALS
!     ---------

!        PPFLEV  - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES
!        PPQ     - PERFORM VERTICAL INTERPOLATION OF CO FIELD TO
!                  AVERAGING KERNEL LEVELS

!**   AUTHOR  
!     ------
!     A.Dethof    18-Nov-2009: Copy of grg_ak_op.F90 to use with  
!                              MOPITT V4 data

!**   MODIFICATIONS
!     -------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARCMA    ,ONLY : JPMX_AK
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMCST   , ONLY : RG, RMD, RMCO
USE YOMLUN   , ONLY : NULERR,NULOUT

USE YOM_YGFL , ONLY : NGRG,YGRG_NL
USE TYPE_GEMS_PROFILES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM)               :: KDCOUNT ! Argument NOT used
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM)               :: KOBSTYPE ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAK(KDLEN,KDCOUNT)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWAK(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPAK(KDLEN,KDCOUNT,JPMX_AK)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESO(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM) 
TYPE (TYPE_GEMSPROF),INTENT(IN)  :: YDGEMSPROF5(KDLEN,NGEMS)
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXGEMS(KDLEN,KDCOUNT,KDPP) 

INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVB(:,:,:)
LOGICAL, ALLOCATABLE :: LLBELO(:,:),LLBLOW(:),LLBELS(:,:),LLBLES(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGEMSMODEL5(:,:),ZGEMS_AK5(:,:)
REAL(KIND=JPRB)  :: ZPRESO(KDLEN,JPMX_AK)
REAL(KIND=JPRB)  :: ZAPAK(KDLEN,JPMX_AK)

INTEGER(KIND=JPIM) :: JGEMS,JGRG,JLEN,INAK(KDLEN)
INTEGER(KIND=JPIM) :: ILEN,IMXCNT,IMXKAK,INPP,IAK
INTEGER(KIND=JPIM) :: JLEVP
LOGICAL, DIMENSION(NGRG) :: LLGRG
REAL(KIND=JPRB) :: ZRGI, ZDP,ZMMR2VMRCO,ZK
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ppflev.intfb.h"
#include "ppq.intfb.h"
!     ------------------------------------------------------------------
!*    1.  PREPARATION
!*    1.1 MAX. NO. OF LEVELS/obs

IF (LHOOK) CALL DR_HOOK('MOPITT_AK_OP',0,ZHOOK_HANDLE)

ZRGI=1.0_JPRB/RG
ZMMR2VMRCO=RMD/RMCO*1.e9
ZK=9.858e-11

ILEN   = KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0) GOTO 9
IF(IMXCNT > 1) THEN
  WRITE(NULERR,*) 'MOPITT_AK_OP: ROUTINE DOES NOT WORK FOR IMXCNT>1. IMXCNT=',IMXCNT
  CALL ABOR1('MOPITT_AK_OP: ROUTINE DOES NOT WORK FOR IMXCNT>1.')
ENDIF

!replace bottom MOPITT AK pressure by surface pressure if that is lower
!test with full level, because half level leads to negative values in L60
ZPRESO(:,:)=PRESO(:,1,:)
INAK(:)=KAK(:,1)
DO JLEN=1,ILEN     ! loop over observations  (header entries)
   DO IAK=KAK(JLEN,1),1,-1
     IF(PRESO(JLEN,1,IAK) >= PRESF5(JLEN,NFLEVG)) THEN
       ZPRESO(JLEN,IAK)=PRESF5(JLEN,NFLEVG)
       INAK(JLEN)=IAK
     ENDIF
   ENDDO
ENDDO
IMXKAK=MAXVAL(INAK(1:ILEN))

DO JGRG=1,NGRG
  LLGRG(JGRG)=CDPAR == TRIM(YGRG_NL(JGRG)%cname)
ENDDO

INPP=0

ALLOCATE(ILEVB(KDLEN,IMXKAK,NPPM))
ALLOCATE(LLBELO(KDLEN,IMXKAK))
ALLOCATE(LLBELS(KDLEN,IMXKAK))
ALLOCATE(LLBLOW(IMXKAK))
ALLOCATE(LLBLES(IMXKAK))

!*    1.2 FIND OUT MODEL PRESSURE LEVEL BELOW SPECIFIED (averaging kernel) PRESSURES


CALL PPFLEV(KDLEN,1,ILEN,NFLEVG,IMXKAK,ZPRESO,PRESH5,PRESF5,ILEVB,&
 & LLBELO,LLBELS,LLBLOW,LLBLES)  

!*    2. Convert model CO values to VMR, interpolate model CO profile to 
!        averaging kernel pressure levels and calculate model total column 
!        CO field using MOPITT AKs

DO JLEN=1,ILEN     ! loop over observations  (header entries)
  DO IAK=1,IMXKAK
   ZAPAK(JLEN,IAK)=PAPAK(JLEN,1,IAK)*ZMMR2VMRCO
  ENDDO
ENDDO

JGRG=0
DO JGEMS=1,NGEMS
  IF(YDGEMSPROF5(1,JGEMS)%CSPECIES=='GRG') THEN
    JGRG=JGRG+1
    IF(LLGRG(JGRG)) THEN
      IF(INPP+1 > KDPP) CALL ABOR1('MOPITT_AK_OP: KDPP EXCEEDED, GRG')
      INPP=INPP+1
      ALLOCATE(ZGEMSMODEL5(KDLEN,0:NFLEVG))
      ALLOCATE(ZGEMS_AK5(KDLEN,IMXKAK))
      DO JLEN=1,ILEN
        ZGEMSMODEL5(JLEN,1:NFLEVG)= &
         & YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1:NFLEVG)*ZMMR2VMRCO           
        ZGEMSMODEL5(JLEN,0)= &
         & MAX(1.e-15_JPRB*ZMMR2VMRCO,YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)*ZMMR2VMRCO)
        DO JLEVP=1,NFLEVG
          IF(ZGEMSMODEL5(JLEN,JLEVP) <= 0._JPRB) THEN
             ZGEMSMODEL5(JLEN,JLEVP) =ZGEMSMODEL5(JLEN,JLEVP-1)
          ENDIF
        ENDDO
      ENDDO

!      WHERE(ZGEMSMODEL5(:,:) <= 0._JPRB) ZGEMSMODEL5 = 1.e-15_JPRB*ZMMR2VMRCO

!*    2.1 Model CO VMR profile interpolated to AK pressures
!         (this works only for total column CO data at the moment (ICOUNT=1).
!         If there are partial columns with averaging kernels ppq will 
!         have to be called several times or rewritten)

      CALL PPQ(KDLEN,1,ILEN,NFLEVG,IMXKAK,1,ILEVB, ZPRESO,LLBELS,LLBLES,&
       & PXP5,PXPD5,ZGEMSMODEL5, ZGEMS_AK5)

      PXGEMS(:,:,:)=0.0_JPRB

      DO JLEN=1,ILEN     ! loop over observations  (header entries)
        DO IAK=1,IMXKAK
          ZDP=0._JPRB
          IF(IAK == 1) THEN
            ZDP=0.5*ZPRESO(JLEN,IAK+1)
          ELSEIF (IAK == INAK(JLEN) ) THEN
            ZDP=0.5*(ZPRESO(JLEN,IAK)-ZPRESO(JLEN,IAK-1))
          ELSEIF (IAK < INAK(JLEN) ) THEN
            ZDP=0.5*(ZPRESO(JLEN,IAK+1)-ZPRESO(JLEN,IAK-1))
          ENDIF

          IF(IAK <= INAK(JLEN)) THEN
              IF(ZGEMS_AK5(jlen,IAK) <= 0._JPRB) &
               & CALL ABOR1('MOPITT_AK_OP: ZGEMS_AK5 <= 0. Can not calculate LOG10. ABORT!')
            PXGEMS(JLEN,1,INPP)=PXGEMS(JLEN,1,INPP) + &
             &  ZK*ZAPAK(JLEN,IAK)*ZDP + &
             &  PWAK(JLEN,1,IAK)*(LOG10(ZGEMS_AK5(jlen,IAK))-LOG10(ZAPAK(JLEN,IAK)))
          ENDIF
        ENDDO

      ENDDO
      DEALLOCATE(ZGEMSMODEL5,ZGEMS_AK5)
    ENDIF
  ENDIF
ENDDO

DEALLOCATE(ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)

9 CONTINUE
IF (LHOOK) CALL DR_HOOK('MOPITT_AK_OP',1,ZHOOK_HANDLE)

END SUBROUTINE MOPITT_AK_OP
