SUBROUTINE MOPITT_AK_TL(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,&
 & KAK,PWAK,PAPAK,PRESO,PRESH,PRESF,PXP,PXPD,YDGEMSPROF,&
 & KDPP,PXGEMS, &
 & PRESH5,PRESF5,PXP5,PXPD5,YDGEMSPROF5)  

!**** MOPITT_AK_TL - Observation operator for MOPITT V4
!                 Tangent linear version.

!     A. Dethof              ECMWF           22-Feb-2006

!     PURPOSE
!     -------

!        FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBSERVED.

!**   INTERFACE
!     ---------

!        CALL MOPITT_AK_TL(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,
!    I                  KAK,PWAK,PRESO,PRESH,PRESF,PXP,PXPD,YDGEMSPROF,
!    O                  KDPP,PXGEMS, &
!    R                  PRESH5,PRESF5,PXP5,PXPD5,YDGEMSPROF5)
! WHERE KDLEN   : FIRST DIMENSION (Observations)                      (INPUT)
!       KLEN    : ACTUAL NUMBER OF OBSERVATIONS                       (INPUT)
!       KDCOUNT : SECOND DIMENSION (Data)                             (INPUT)
!       CDPAR   : PARAMETER FOR POST-PROCESSING                       (INPUT)
!       KOBSTYPE: OBS. TYPE                                           (INPUT)
!       KCOUNT  : COUNTER FOR THE PARTICULAR OBS. PARAMETER           (INPUT)
!       KAK     : Number of averaging kernel elements                 (INPUT)
!       PWAK    : Weights for averaging kernels                       (INPUT)
!       PAPAK   : Apriori values for averaging kernels      (INPUT)
!       PRESO   : Perturb in Output PRESSURES (ak pressures)          (INPUT)
!       PRESH   : Perturb in HALF LEVEL PRESSURES                     (INPUT)
!       PRESF   : Perturb in FULL LEVEL PRESSURES                     (INPUT)
!       PXP     : Perturb in INTERMEDIATE ARRAY                       (INPUT)
!       PXPD    : Perturb in ..................                       (INPUT)
!       YDGEMSPROF: Perturb in GEMS species (GRG) AT MODEL FULL LEVELS (INPUT)
!       KDPP    : Number of fields, dimension of PXPP.
!       PXGEMS   : GRG layer value                                     (OUTPUT)
!       PRESH5  : HALF LEVEL PRESSURES                                (INPUT)
!       PRESF5  : FULL LEVEL PRESSURES                                (INPUT)
!       PXP5    : INTERMEDIATE ARRAY                                  (INPUT)
!       PXPD5   : ..................                                  (INPUT)
!       YDGEMSPROF5: GEMS species (GRG) AT MODEL FULL LEVELS           (INPUT)

!        MOPITT_AK_TL IS CALLED FROM HOPTL

!**   METHOD
!     ------

!        Tangent linear of mopitt_ak_op. Calculate model total CO column (Col)  as:
!        Col_mod=Col_ap + wak*(x_mod-x_ap), where
!        wak= averaging kernel vector (transpose)
!        x_mod, x_ap= model and apriori log(VMR) profiles (vmr has to be in ppb) on ak levels
!        Col_ap= K*SUM(zdp*VMR(x_ap)): apriori total column
!        ZK=2.12E13(mol/cm2)/(hPa ppb)=9.858e-11(kg/m2)/(Pa ppb)
!        Retrieval pressure grid is 100, 200, 300, 400, 500 ,600, 700, 800, 900, psurf
!        Fewer levels if psurf<900 hPa.


!**   EXTERNALS
!     ---------

!        PPFLEV  - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES
!        PPQ     - PERFORM VERTICAL INTERPOLATION OF CO FIELD TO
!                  AVERAGING KERNEL LEVEL
!        PPQTL   - PERFORM VERTICAL INTERPOLATION OF CO FIELD TO
!                  AVERAGING KERNEL LEVEL


!**   AUTHOR  
!     ------
!     A.Dethof    18-Nov-2009: Copy of grg_ak_tl.F90 to use with  
!                              MOPITT V4 data

!**   MODIFICATIONS
!     -------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARCMA    ,ONLY : JPMX_AK
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMCST   , ONLY : RMD, RMCO
USE YOMLUN   , ONLY : NULERR

USE YOM_YGFL , ONLY : NGRG,YGRG_NL
USE TYPE_GEMS_PROFILES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM)               :: KDCOUNT ! Argument NOT used
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM)               :: KOBSTYPE ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAK(KDLEN,KDCOUNT)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWAK(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPAK(KDLEN,KDCOUNT,JPMX_AK)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESO(KDLEN,KDCOUNT,JPMX_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH(KDLEN,0:NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF(KDLEN,NFLEVG) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD(KDLEN,0:NFLEVG,NPPM) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXGEMS(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM)
TYPE (TYPE_GEMSPROF),INTENT(IN)  :: YDGEMSPROF(KDLEN,NGEMS)
TYPE (TYPE_GEMSPROF),INTENT(IN)  :: YDGEMSPROF5(KDLEN,NGEMS)

INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVB(:,:,:)
LOGICAL, ALLOCATABLE :: LLBELO(:,:),LLBLOW(:),LLBELS(:,:),LLBLES(:)

REAL(KIND=JPRB), ALLOCATABLE :: ZGEMSMODEL(:,:),ZGEMSMODEL5(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGEMS_AK(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGEMS_AK5(:,:)

REAL(KIND=JPRB) :: ZPRESO(KDLEN,JPMX_AK)

INTEGER(KIND=JPIM) :: JGEMS,JGRG,JLEN,INAK(KDLEN)
INTEGER(KIND=JPIM) :: ILEN,IMXCNT,IMXKAK,INPP,IAK
INTEGER(KIND=JPIM) :: JLEVP
REAL(KIND=JPRB) :: ZMMR2VMRCO
LOGICAL, DIMENSION(NGRG) :: LLGRG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ppq.intfb.h"
#include "ppqtl.intfb.h"
#include "ppflev.intfb.h"
!     ------------------------------------------------------------------
!*    1.  PREPARATION
!*    1.1 MAX. NO. OF LEVELS/obs

IF (LHOOK) CALL DR_HOOK('MOPITT_AK_TL',0,ZHOOK_HANDLE)
ZMMR2VMRCO=RMD/RMCO*1.e9

ILEN   = KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0) GOTO 9
IF(IMXCNT > 1) THEN
  WRITE(NULERR,*) 'MOPITT_AK_TL: ROUTINE DOES NOT WORK FOR IMXCNT>1. IMXCNT=',IMXCNT
  CALL ABOR1('MOPITT_AK_TL: ROUTINE DOES NOT WORK FOR IMXCNT>1.')
ENDIF

!replace bottom MOPITT AK pressure by surface pressure if that is lower
!test with full level, because half level leads to negative values in L60
ZPRESO(:,:)=PRESO(:,1,:)
INAK(:)=KAK(:,1)
DO JLEN=1,ILEN     ! loop over observations  (header entries)
   DO IAK=KAK(JLEN,1),1,-1
     IF(PRESO(JLEN,1,IAK) >= PRESF5(JLEN,NFLEVG)) THEN
       ZPRESO(JLEN,IAK)=PRESF5(JLEN,NFLEVG)
       INAK(JLEN)=IAK
     ENDIF
   ENDDO
ENDDO
IMXKAK=MAXVAL(INAK(1:ILEN))

DO JGRG=1,NGRG
  LLGRG(JGRG)=CDPAR == TRIM(YGRG_NL(JGRG)%cname)
ENDDO

INPP=0

ALLOCATE(ILEVB(KDLEN,IMXKAK,NPPM))
ALLOCATE(LLBELO(KDLEN,IMXKAK))
ALLOCATE(LLBELS(KDLEN,IMXKAK))
ALLOCATE(LLBLOW(IMXKAK))
ALLOCATE(LLBLES(IMXKAK))

!*    1.2 FIND OUT PRESSURE LEVEL BELOW SPECIFIED (averaging kernel) PRESSURES

CALL PPFLEV(KDLEN,1,ILEN,NFLEVG,IMXKAK,ZPRESO,PRESH5,PRESF5, ILEVB,&
 & LLBELO,LLBELS,LLBLOW,LLBLES)  

JGRG=0
DO JGEMS=1,NGEMS
  IF(YDGEMSPROF5(1,JGEMS)%CSPECIES=='GRG') THEN
    JGRG=JGRG+1
    IF(LLGRG(JGRG)) THEN
      IF(INPP+1 > KDPP) CALL ABOR1('MOPITT_AK_TL: KDPP EXCEEDED, GRG')
      INPP=INPP+1
      ALLOCATE(ZGEMSMODEL(KDLEN,0:NFLEVG))
      ALLOCATE(ZGEMSMODEL5(KDLEN,0:NFLEVG))
      ALLOCATE(ZGEMS_AK(KDLEN,IMXKAK))
      ALLOCATE(ZGEMS_AK5(KDLEN,IMXKAK))
      DO JLEN=1,ILEN
        ZGEMSMODEL(JLEN,0)=0.0_JPRB
        ZGEMSMODEL(JLEN,1:NFLEVG)=YDGEMSPROF(JLEN,JGEMS)%MODELPROF(:)*ZMMR2VMRCO
        ZGEMSMODEL5(JLEN,0)=&
         & MAX(1.e-15_JPRB*ZMMR2VMRCO,YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)*ZMMR2VMRCO)
        ZGEMSMODEL5(JLEN,1:NFLEVG)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(:)*ZMMR2VMRCO
        DO JLEVP=1,NFLEVG
          IF(ZGEMSMODEL5(JLEN,JLEVP) <= 0._JPRB) ZGEMSMODEL5(JLEN,JLEVP) =ZGEMSMODEL5(JLEN,JLEVP-1)
        ENDDO
      ENDDO
!      WHERE(ZGEMSMODEL5(:,:) <= 0._JPRB) ZGEMSMODEL5 = 1.e-15_JPRB*ZMMR2VMRCO

! Interpolate CO profile to AK pressure levels

!*    2.1 Model CO VMR profile interpolated to AK pressures - trajectory
      CALL PPQ(KDLEN,1,ILEN,NFLEVG,IMXKAK,1,ILEVB,&
       & ZPRESO,LLBELS,LLBLES,&
       & PXP5,PXPD5,ZGEMSMODEL5,&
       & ZGEMS_AK5)

!*    2.1 Model CO VMR profile interpolated to AK pressures - perturbation
     CALL PPQTL(KDLEN,1,ILEN,NFLEVG,IMXKAK,1,ILEVB,&
      & ZPRESO,LLBELS,LLBLES,&
      & PXP,PXPD,ZGEMSMODEL,&
      & ZGEMS_AK,&
      & PXP5,PXPD5,ZGEMSMODEL5)

!*    2.3 Calculate model total column CO
!    
      PXGEMS(:,:,:)=0.0_JPRB
      DO JLEN=1,ILEN     ! loop over observations  (header entries)
        DO IAK=1,IMXKAK
          IF(IAK <= INAK(JLEN)) THEN
             PXGEMS(JLEN,1,INPP)=PXGEMS(JLEN,1,INPP) + &
              & PWAK(JLEN,1,IAK)*ZGEMS_AK(JLEN,IAK)/(ZGEMS_AK5(jlen,iak)*LOG(10._JPRB))
          ENDIF
        ENDDO
      ENDDO

      DEALLOCATE(ZGEMSMODEL,ZGEMSMODEL5,ZGEMS_AK,ZGEMS_AK5)
    ENDIF
  ENDIF
ENDDO

DEALLOCATE(ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)

9 CONTINUE
IF (LHOOK) CALL DR_HOOK('MOPITT_AK_TL',1,ZHOOK_HANDLE)

END SUBROUTINE MOPITT_AK_TL
