SUBROUTINE MPOBSEQ(LDMVO5,&
 & YDGOMUA  ,YDGOMS   ,YDGOMUA_2D         ,YDGOMS_2D)

!**** *MPOBSEQ* Message passing of obs.eq. 

!     Purpose :  Exchange GOM arrays
!     -------

!     Interface : CALL MPOBSEQ(..)
!     ---------

!       Explicit arguments : 
!       ------------------   

!       Implicit arguments : 
!       ------------------

!     Externals :   MPL_RECV - receive message
!     ---------     MPL_SEND - send message

!     Author : Mats Hamrud *ECMWF*
!     ------

!     Modifications :
!     -------------
!     95-12-08 M.Hamrud    Original
!     96-12-01 M.Hamrud    : 4-D var
!     97-02-22 B. Urban    : Added PSLXTS,PSLXWS,PSLXWP,PSLXSN,PSLXTP
!                            (surface climatology for Canari)
!     97-06-02 F.Taillefer : Add DL and DM arrays
!     97-11-17 E.Andersson : Introduce CIW and O3, Flexinle GOM-arrays.
!     97-10-10 F.Taillefer : Add model errors arrays (for Canari)
!     98-07-10 F.Taillefer : Adapt to CANARI (cycle 19 changes)
!     99-08-06 P.Moll      : Pb LOBSTL pour les champs constants 
!     99-09-01 P.Viterbo   : Surface tiles
!     00-09-01 D.Dent      : MPE to MPL conversion
!     02-05-22 F.Taillefer : Add SST array (possible land/sea obs)
!     02-08-01 D. Salmond  : LIMP_NOOLAP Option for non-overlapping
!                            message passing and buffer packing
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     04-06-23 C.Payan     : Control on ZBUFR pointer
!        M.Hamrud      15-Feb-2005 New-style GOM arrays
!     H. Hersbach   04-Dec-2009 10-m neutral wind; LUPD
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCT0   , ONLY : NPROC    ,LCANARI  ,LCASIG
USE YOMCT0B  , ONLY : LECMWF
USE YOMLUN   , ONLY : NULERR
USE YOMPHY   , ONLY : LSOLV
USE YOMCOCTP , ONLY : NOOBST
USE YOMDPHY  , ONLY : LDIRCLSMOD
USE YOMGLOBS , ONLY : NOBSSEND,NOBSRECV ,NSTARECV ,NLOCRECV,NLOCPROF ,&
 & LPREMPOBS,SENDBUF,NENDS,SENDBUF5,NENDS5
USE YOMECTAB , ONLY : NTSLOTNO
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMVRTL  , ONLY : LOBSTL
USE YOMTAG   , ONLY : MTAGOBSEQ
USE GOMS_MIX     , ONLY : TYPE_GOMUA,TYPE_GOMS,TYPE_GOMUA_2D,TYPE_GOMS_2D,&
 & LGOMUA,YGOMEC,YGOMEC_2D,YGOMCAN,YGOMCANA, &
 & NGOMGFL
USE MPL_MODULE  , ONLY : MPL_SEND, MPL_RECV, MPL_WAIT, &
 & JP_BLOCKING_STANDARD, JP_NON_BLOCKING_STANDARD,MPL_COMM_OML
USE OBSHOR_CACHE_MIX


IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDMVO5 
TYPE(TYPE_GOMUA)  ,INTENT(INOUT)   :: YDGOMUA(:) 
TYPE(TYPE_GOMS)   ,INTENT(INOUT)   :: YDGOMS(:) 
TYPE(TYPE_GOMUA_2D)  ,INTENT(INOUT)   :: YDGOMUA_2D(:) 
TYPE(TYPE_GOMS_2D)   ,INTENT(INOUT)   :: YDGOMS_2D(:) 

! Local variables
REAL(KIND=JPRB),ALLOCATABLE :: ZBUFR(:) ! Receive buffer

INTEGER(KIND=JPIM) :: IBP, IBUFLENR, &
 & IOFF, IOFFTS, &
 & IRECV,  ISEND,  ISN, &
 & ITAG,  JROC, JTYP,JTS,&
 & IOBSRECV, IBUFLENP, ISS, &
 & ILRECV(NPROC), IEND,ISTR ,&
 & ITSLOTB, ITSLOTE,ITSLOTS
             
INTEGER(KIND=JPIM), DIMENSION(NPROC) :: ISTARTR,IENDR,ISTARTS,IENDS

INTEGER(KIND=JPIM) :: ISENDREQ(NPROC-1)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!    -------------------------------------------------------------------
!*       1.  SETUPS.
!            ------

IF (LHOOK) CALL DR_HOOK('MPOBSEQ',0,ZHOOK_HANDLE)

IF(LPREMPOBS) THEN
  ITSLOTB = 1
  ITSLOTE = NACTIM
ELSE
  ITSLOTB = NTSLOTNO
  ITSLOTE = NTSLOTNO
ENDIF
ITSLOTS = ITSLOTE-ITSLOTB+1

!    -------------------------------------------------------------------
!*       2.  EXCHANGE OBS.EQ. WITH OTHER PROCESSORS.
!            --------------------------------------

IOBSRECV = SUM(NOBSRECV(:,ITSLOTB:ITSLOTE))-SUM(NOBSRECV(MYPROC,ITSLOTB:ITSLOTE))

ITAG  = MTAGOBSEQ+ITSLOTB
IF(LDMVO5) ITAG=ITAG+NACTIM

ISN  = 0
DO JTYP=1,NOOBST
  ISN = MAX(ISN,COUNT(LGOMUA(JTYP,:)))
ENDDO
ISN = ISN*NFLEVG

IBUFLENP = ISN+17
IF (LECMWF) IBUFLENP = IBUFLENP+4
IF (LDMVO5.OR..NOT.LOBSTL) IBUFLENP=IBUFLENP+8
IF (LDMVO5) IBUFLENP=IBUFLENP+1
IF (LCANARI.AND.LCASIG) IBUFLENP=IBUFLENP+2*NFLEVG+5
IF (LCANARI) IBUFLENP=IBUFLENP+4
IF (LSOLV.AND.(LDMVO5.OR..NOT.LOBSTL)) IBUFLENP=IBUFLENP+4
IF (LDIRCLSMOD) IBUFLENP=IBUFLENP+2
        
ISS = 1
DO JROC = 1,NPROC
  IF (JROC /= MYPROC) THEN
    ISTARTR(JROC) = ISS
    ISS = ISS+SUM(NOBSRECV(JROC,ITSLOTB:ITSLOTE))*IBUFLENP
    IENDR(JROC) = ISS-1
  ENDIF
ENDDO

ISS = 1
DO JROC=1,NPROC
  IF(JROC /= MYPROC) THEN
    ISTARTS(JROC) = ISS
    ISS = ISS+SUM(NOBSSEND(JROC,ITSLOTB:ITSLOTE))*IBUFLENP
  ENDIF
ENDDO
IF(LDMVO5) THEN
  IENDS(:) = NENDS5(:)
ELSE
  IENDS(:) = NENDS(:)
ENDIF
  
IBUFLENR = IBUFLENP*IOBSRECV
ALLOCATE(ZBUFR(IBUFLENR))

!    -------------------------------------------------------------------
!*           SEND AND RECEIVE MESSAGES
!            -------------------------

CALL GSTATS_BARRIER(767)
CALL GSTATS(513,0)
DO JROC = 1,NPROC-1
  ISEND = MOD(MYPROC+JROC-1,NPROC)+1
  IF(ANY(NOBSSEND(ISEND,ITSLOTB:ITSLOTE) > 0)) THEN
    IF(LDMVO5) THEN
      CALL MPL_SEND(SENDBUF5(ISTARTS(ISEND):IENDS(ISEND)),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
       & KMP_TYPE=JP_NON_BLOCKING_STANDARD,KREQUEST=ISENDREQ(JROC), &
       & CDSTRING='MPOBSEQ:') 
    ELSE
      CALL MPL_SEND(SENDBUF(ISTARTS(ISEND):IENDS(ISEND)),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
       & KMP_TYPE=JP_NON_BLOCKING_STANDARD,KREQUEST=ISENDREQ(JROC), &
       & CDSTRING='MPOBSEQ:') 
    ENDIF
  ENDIF
ENDDO

!!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1)&
!!$OMP&PRIVATE(JROC,IRECV,ISTR,IEND)
DO JROC = 1,NPROC-1
  IRECV = MOD(-JROC-1+MYPROC+NPROC,NPROC)+1
  IF(ANY(NOBSRECV(IRECV,ITSLOTB:ITSLOTE) > 0)) THEN
    ISTR=ISTARTR(IRECV)
    IEND=IENDR(IRECV)
    CALL MPL_RECV(ZBUFR(ISTR:IEND), KSOURCE=NPRCIDS(IRECV), &
     & KMP_TYPE=JP_BLOCKING_STANDARD,KTAG=ITAG,KCOMM=MPL_COMM_OML(1),&
     & KOUNT=ILRECV(IRECV),CDSTRING='MPOBSEQ:')  
  ENDIF
ENDDO
!!$OMP END PARALLEL DO

DO JROC = 1,NPROC-1
  ISEND = MOD(MYPROC+JROC-1,NPROC)+1
  IF(ANY(NOBSSEND(ISEND,ITSLOTB:ITSLOTE) > 0)) THEN
    IF(LDMVO5) THEN
      CALL MPL_WAIT(SENDBUF5, KREQUEST=ISENDREQ(JROC), &
       & CDSTRING='MPOBSEQ: WAIT FOR SEND' )  
    ELSE
      CALL MPL_WAIT(SENDBUF, KREQUEST=ISENDREQ(JROC), &
       & CDSTRING='MPOBSEQ: WAIT FOR SEND' )
    ENDIF
  ENDIF
ENDDO
CALL GSTATS(513,1)
CALL GSTATS_BARRIER2(767)

!    ------------------------------------------------------------------
!*             UNPACK RECEIVE BUFFER.
!              ---------------------
CALL GSTATS(1071,0)

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1)&
!$OMP&PRIVATE(JROC,IRECV,IBP,JTS)
DO JROC = 1,NPROC-1
  IRECV = MOD(-JROC-1+MYPROC+NPROC,NPROC)+1
  IBP = ISTARTR(IRECV)-1
  IF(ANY(NOBSRECV(IRECV,ITSLOTB:ITSLOTE) > 0)) THEN
    DO JTS=ITSLOTB,ITSLOTE
      IF(NOBSRECV(IRECV,JTS) > 0) THEN
        CALL UNPACK_RECV(IBP,IRECV,JTS)
      ENDIF
    ENDDO
    IF(LECMWF .AND. (IBP-ISTARTR(IRECV)+1 /= ILRECV(IRECV))) THEN
      WRITE(NULERR,*)'MYPROC,IRECV,IBP,ILRECV(JROC),ISTARTR(JROC) ',&
       & MYPROC,IRECV,IBP,ILRECV(IRECV),ISTARTR(IRECV)  
      CALL ABOR1(' MPOBSEQ: IBP /= ILRECV(IRECV) ')
    ENDIF
  ENDIF
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1071,1)

IF(LDMVO5) THEN
  DEALLOCATE(SENDBUF5)
ELSE
  DEALLOCATE(SENDBUF)
ENDIF
DEALLOCATE(ZBUFR)
IF (LHOOK) CALL DR_HOOK('MPOBSEQ',1,ZHOOK_HANDLE)

!    -------------------------------------------------------------------
CONTAINS
SUBROUTINE UNPACK_RECV(KBP,KRECV,KTS)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBP
INTEGER(KIND=JPIM),INTENT(IN)    :: KRECV
INTEGER(KIND=JPIM),INTENT(IN)    :: KTS
INTEGER(KIND=JPIM) :: IOFF,IOFFTS,IOBS,JOBS,JLEV,INUM,IPROF
INTEGER(KIND=JPIM) :: JGFL,IGFL,ISTGFL,IENGFL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('MPOBSEQ:UNPACK_RECV',0,ZHOOK_HANDLE)
IOFF = NSTARECV(KRECV,KTS)
DO JOBS = 1,NOBSRECV(KRECV,KTS)
  IOBS = NLOCRECV(IOFF+JOBS)
  IPROF = NLOCPROF(IOFF +JOBS)
  
  INUM = OBSHOR_CACHE(KTS)%MAPOMM(IOBS)
  IF(IPROF == 0) THEN
    IF(INUM /= YDGOMUA(INUM)%IOBS) THEN
      WRITE(NULERR,*) 'MPOBSEQ - INUM /= YDGOMUA(INUM)%IOBS ',&
       & KRECV,JOBS,IOBS,INUM,YDGOMUA(INUM)%IOBS
      CALL ABOR1('MPOBSEQ - INUM /= YDGOMUA(INUM)%IOBS ')
    ENDIF
    IF(YDGOMUA(INUM)%LU) THEN
      YDGOMUA(INUM)%U(:) = ZBUFR(KBP+1:KBP+NFLEVG)
      KBP = KBP+NFLEVG
    ENDIF
    IF(YDGOMUA(INUM)%LV) THEN
      YDGOMUA(INUM)%V(:) = ZBUFR(KBP+1:KBP+NFLEVG)
      KBP = KBP+NFLEVG
    ENDIF
    IF(YDGOMUA(INUM)%LT) THEN
      YDGOMUA(INUM)%T(:) = ZBUFR(KBP+1:KBP+NFLEVG)
      KBP = KBP+NFLEVG
    ENDIF
    IGFL = 0
    DO JGFL=1,NGOMGFL
      IF(YDGOMUA(INUM)%LGFL(JGFL)) THEN
        IGFL = IGFL+1
        ISTGFL = 1+(IGFL-1)*NFLEVG
        IENGFL = ISTGFL+NFLEVG-1
        YDGOMUA(INUM)%GFL(ISTGFL:IENGFL) = ZBUFR(KBP+1:KBP+NFLEVG)
        KBP = KBP+NFLEVG
      ENDIF
    ENDDO
  ELSE
  
    IF(INUM /= YDGOMUA_2D(INUM)%IOBS) THEN
      WRITE(NULERR,*) 'MPOBSEQ - INUM /= YDGOMUA_2D(INUM)%IOBS ',&
       & KRECV,JOBS,INUM,YDGOMUA_2D(INUM)%IOBS
      CALL ABOR1('MPOBSEQ - INUM /= YDGOMUA_2D(INUM)%IOBS ')
    ENDIF
    IF(YDGOMUA_2D(INUM)%LU) THEN
      YDGOMUA_2D(INUM)%U(:,IPROF) = ZBUFR(KBP+1:KBP+NFLEVG)
      KBP = KBP+NFLEVG
    ENDIF
    IF(YDGOMUA_2D(INUM)%LV) THEN
      YDGOMUA_2D(INUM)%V(:,IPROF) = ZBUFR(KBP+1:KBP+NFLEVG)
      KBP = KBP+NFLEVG
    ENDIF
    IF(YDGOMUA_2D(INUM)%LT) THEN
      YDGOMUA_2D(INUM)%T(:,IPROF) = ZBUFR(KBP+1:KBP+NFLEVG)
      KBP = KBP+NFLEVG
    ENDIF
    IGFL = 0
    DO JGFL=1,NGOMGFL
      IF(YDGOMUA_2D(INUM)%LGFL(JGFL)) THEN
        IGFL = IGFL+1
        ISTGFL = 1+(IGFL-1)*NFLEVG
        IENGFL = ISTGFL+NFLEVG-1
        YDGOMUA_2D(INUM)%GFL(ISTGFL:IENGFL,IPROF) = ZBUFR(KBP+1:KBP+NFLEVG)
        KBP = KBP+NFLEVG
      ENDIF
    ENDDO
  ENDIF
ENDDO



DO JOBS = 1,NOBSRECV(KRECV,KTS)
  IOBS = NLOCRECV(IOFF+JOBS)
  IPROF = NLOCPROF(IOFF+JOBS)
  INUM = OBSHOR_CACHE(KTS)%MAPOMM(IOBS)
  
  
  IF(IPROF == 0) THEN
    YDGOMS(INUM)%UL = ZBUFR(KBP+1)
    YDGOMS(INUM)%VL = ZBUFR(KBP+2)
    YDGOMS(INUM)%SP = ZBUFR(KBP+3)
    YDGOMS(INUM)%TS = ZBUFR(KBP+4)
    YDGOMS(INUM)%WS = ZBUFR(KBP+5)
    YDGOMS(INUM)%SN = ZBUFR(KBP+6)
    YDGOMS(INUM)%Z0 = ZBUFR(KBP+7)
    YDGOMS(INUM)%WL = ZBUFR(KBP+8)
    YDGOMS(INUM)%TL = ZBUFR(KBP+9)
    YDGOMS(INUM)%QL = ZBUFR(KBP+10)
    YDGOMS(INUM)%SST =ZBUFR(KBP+11)
    YDGOMS(INUM)%U0 = ZBUFR(KBP+12)
    YDGOMS(INUM)%V0 = ZBUFR(KBP+13)
    YDGOMS(INUM)%CHN= ZBUFR(KBP+14)
    YDGOMS(INUM)%U10N=ZBUFR(KBP+15)
    YDGOMS(INUM)%V10N=ZBUFR(KBP+16)
    IF (LDIRCLSMOD) THEN
       YDGOMS(INUM)%TCLS = ZBUFR(KBP+17)
       YDGOMS(INUM)%HUCLS= ZBUFR(KBP+18)
       KBP=KBP+18
    ELSE
       KBP=KBP+16
    ENDIF
    IF (LDMVO5) THEN
       YDGOMS(INUM)%LUPD=(ZBUFR(KBP+1)==1.0_JPRB)
       KBP=KBP+1
    ENDIF
    IF(LDMVO5.OR..NOT.LOBSTL) THEN
      YGOMEC(INUM)%ECOROG = ZBUFR(KBP+1)
      YGOMEC(INUM)%ECCORI = ZBUFR(KBP+2)
      YGOMEC(INUM)%ECALBE = ZBUFR(KBP+3)
      YGOMEC(INUM)%ECEMIS = ZBUFR(KBP+4)
      YGOMEC(INUM)%ECLSMA = ZBUFR(KBP+5)
      IF (LECMWF) THEN
        YGOMEC(INUM)%ECCI = ZBUFR(KBP+6)
        YGOMEC(INUM)%ECCVH = ZBUFR(KBP+7)
        YGOMEC(INUM)%ECCVL = ZBUFR(KBP+8)
        YGOMEC(INUM)%ECTVH = ZBUFR(KBP+9)
        YGOMEC(INUM)%ECTVL = ZBUFR(KBP+10)
        YGOMEC(INUM)%ECDL  = ZBUFR(KBP+11)
        YGOMEC(INUM)%ECDM  = ZBUFR(KBP+12)
        KBP = KBP+12
      ELSE
        YGOMEC(INUM)%ECVEGE = ZBUFR(KBP+6)
        YGOMEC(INUM)%ECDL   = ZBUFR(KBP+7)
        YGOMEC(INUM)%ECDM   = ZBUFR(KBP+8)
        KBP = KBP+8
      ENDIF
      IF (LSOLV) THEN
        YGOMEC(INUM)%ECARG = ZBUFR(KBP+1)
        YGOMEC(INUM)%ECSAB = ZBUFR(KBP+2)
        YGOMEC(INUM)%ECHV  = ZBUFR(KBP+3)
        YGOMEC(INUM)%ECZ0H = ZBUFR(KBP+4)
        KBP = KBP+4
      ENDIF
    ENDIF
  ELSE

    YDGOMS_2D(INUM)%UL(IPROF) = ZBUFR(KBP+1)
    YDGOMS_2D(INUM)%VL(IPROF) = ZBUFR(KBP+2)
    YDGOMS_2D(INUM)%SP(IPROF) = ZBUFR(KBP+3)
    YDGOMS_2D(INUM)%TS(IPROF) = ZBUFR(KBP+4)
    YDGOMS_2D(INUM)%WS(IPROF) = ZBUFR(KBP+5)
    YDGOMS_2D(INUM)%SN(IPROF) = ZBUFR(KBP+6)
    YDGOMS_2D(INUM)%Z0(IPROF) = ZBUFR(KBP+7)
    YDGOMS_2D(INUM)%WL(IPROF) = ZBUFR(KBP+8)
    YDGOMS_2D(INUM)%TL(IPROF) = ZBUFR(KBP+9)
    YDGOMS_2D(INUM)%QL(IPROF) = ZBUFR(KBP+10)
    YDGOMS_2D(INUM)%SST(IPROF) = ZBUFR(KBP+11)
    YDGOMS_2D(INUM)%U10N(IPROF)=ZBUFR(KBP+12)
    YDGOMS_2D(INUM)%V10N(IPROF)=ZBUFR(KBP+13)

    KBP=KBP+13
    IF(LDMVO5.OR..NOT.LOBSTL) THEN
      YGOMEC_2D(INUM)%ECOROG(IPROF) = ZBUFR(KBP+1)
      YGOMEC_2D(INUM)%ECCORI(IPROF) = ZBUFR(KBP+2)
      YGOMEC_2D(INUM)%ECALBE(IPROF) = ZBUFR(KBP+3)
      YGOMEC_2D(INUM)%ECEMIS(IPROF) = ZBUFR(KBP+4)
      YGOMEC_2D(INUM)%ECLSMA(IPROF) = ZBUFR(KBP+5)
      
      IF (LECMWF) THEN
        YGOMEC_2D(INUM)%ECCI (IPROF) = ZBUFR(KBP+6)
        YGOMEC_2D(INUM)%ECCVH(IPROF) = ZBUFR(KBP+7)
        YGOMEC_2D(INUM)%ECCVL(IPROF) = ZBUFR(KBP+8)
        YGOMEC_2D(INUM)%ECTVH(IPROF) = ZBUFR(KBP+9)
        YGOMEC_2D(INUM)%ECTVL(IPROF) = ZBUFR(KBP+10)
        YGOMEC_2D(INUM)%ECDL (IPROF) = ZBUFR(KBP+11)
        YGOMEC_2D(INUM)%ECDM (IPROF) = ZBUFR(KBP+12)
        KBP = KBP+12

      ELSE
        YGOMEC_2D(INUM)%ECVEGE(IPROF) = ZBUFR(KBP+6)
        YGOMEC_2D(INUM)%ECDL  (IPROF) = ZBUFR(KBP+7)
        YGOMEC_2D(INUM)%ECDM  (IPROF) = ZBUFR(KBP+8)
        KBP = KBP+8
      ENDIF
      IF (LSOLV) THEN
        YGOMEC_2D(INUM)%ECARG(IPROF) = ZBUFR(KBP+1)
        YGOMEC_2D(INUM)%ECSAB(IPROF) = ZBUFR(KBP+2)
        YGOMEC_2D(INUM)%ECHV (IPROF) = ZBUFR(KBP+3)
        YGOMEC_2D(INUM)%ECZ0H(IPROF) = ZBUFR(KBP+4)
        KBP = KBP+4
      ENDIF
    ENDIF
    
    
  ENDIF
ENDDO

IF(LCANARI) THEN
  DO JOBS = 1,NOBSRECV(KRECV,KTS)
    IOBS = NLOCRECV(IOFF+JOBS)
    INUM = OBSHOR_CACHE(KTS)%MAPOMM(IOBS)
    IF (LCASIG) THEN
      DO JLEV = 1,NFLEVG
        YGOMCANA(INUM)%EMMZ(JLEV) = ZBUFR(KBP+JLEV)
        YGOMCANA(INUM)%EMMH(JLEV) = ZBUFR(KBP+NFLEVG+JLEV)
      ENDDO
      KBP = KBP+2*NFLEVG
      YGOMCAN(INUM)%EMMST = ZBUFR(KBP+1)
      KBP=KBP+1
      YGOMCAN(INUM)%EMMSN = ZBUFR(KBP+1)
      YGOMCAN(INUM)%EMMT2 = ZBUFR(KBP+2)
      YGOMCAN(INUM)%EMMH2 = ZBUFR(KBP+3)
      YGOMCAN(INUM)%EMMV1 = ZBUFR(KBP+4)
      KBP=KBP+4
    ENDIF
    YGOMCAN(INUM)%ECPRWA = ZBUFR(KBP+1)
    YGOMCAN(INUM)%ECPRSN = ZBUFR(KBP+2)
    YGOMCAN(INUM)%ECCLITS = ZBUFR(KBP+3)
    YGOMCAN(INUM)%ECCLISN = ZBUFR(KBP+4)
    KBP = KBP+4
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('MPOBSEQ:UNPACK_RECV',1,ZHOOK_HANDLE)

END SUBROUTINE UNPACK_RECV

END SUBROUTINE MPOBSEQ
