SUBROUTINE OBSHOR(&
 & LDMVO5,KASLB1,KPROMB,KPROMBMP,&
 & PPARITY  , &
 & POBB1U5  , POBB1V5  , POBB1T5 , POBB1GFL5 ,&
 & POBB1EZ5 , POBB1EH5,&
 & POBB1SP5 , POBB1EST5,&
 & POBB1U05 , POBB1V05 , POBB1CHN5,&
 & POBB1TS5 , POBB1WS5 , POBB1SN5, POBB1Z05, POBB1WL5,&
 & POBB1U10N5,POBB1V10N5,&
 & POBB1OR5 , POBB1CO5 , POBB1DL5, POBB1DM5,&
 & POBB1AL5 , POBB1EM5 , POBB1LS5,&
 & POBB1VG5 ,&
 & POBB1CI5 , POBB1CVH5, POBB1CVL5,POBB1TVH5,POBB1TVL5,&
 & POBB1PW5 , POBB1PS5 ,&
 & POBB1ARG5, POBB1SAB5, POBB1HV5, POBB1Z0H5,&
 & POBB1XTS , POBB1XWS , POBB1XWP, POBB1XSN, POBB1XTP,&
 & POBB1ESN , POBB1ET2 , POBB1EH2, POBB1EV1, POBB1TCLS5, POBB1HUCLS5)  

!     *OBSHOR* - Horizontal interpolation of the model variables
!                from grid points to observation points.

!     Purpose.
!     -------
!        To prepare use of semi-lagrangian interpolation routines.

!     Interface.
!     ---------
!        CALL OBSHOR(...)

!     Explicit arguments.
!     ------------------
!            LDMVO5   = LOAD GOMU5 ETC. ARRAYS IN YOMMVO WITH TRAJECTORY VALUES
!            PPARITY  = INDICATOR FOR SCALAR(1) OR VECTOR(-1) FIELD
!            POBB1U5  = PART OF S/L BUFFER FOR U
!            POBB1V5  = PART OF S/L BUFFER FOR V
!            POBB1T5  = PART OF S/L BUFFER FOR T
!            POBB1GFL5  = PART OF S/L BUFFER FOR GFL
!            POBB1EZ5 = PART OF S/L BUFFER FOR PHI MODEL ERROR
!            POBB1EH5 = PART OF S/L BUFFER FOR Hu MODEL ERROR
!            POBB1SP5 = PART OF S/L BUFFER FOR Ps
!            POBB1EST5= PART OF S/L BUFFER FOR SST MODEL ERROR
!            POBB1U05 = PART OF S/L BUFFER FOR U-COMPONENT OF OCEAN CURRENT
!            POBB1V05 = PART OF S/L BUFFER FOR V-COMPONENT OF OCEAN CURRENT
!            POBB1CHN5= PART OF S/L BUFFER FOR WAVE CHARNOCK
!            POBB1TS5 = PART OF S/L BUFFER FOR Ts
!            POBB1WS5 = PART OF S/L BUFFER FOR Ws
!            POBB1SN5 = PART OF S/L BUFFER FOR SNOW
!            POBB1Z05 = PART OF S/L BUFFER FOR Z0
!            POBB1U10N5=PART OF S/L BUFFER FOR U-COMPONENT OF 10M NEUTRAL WIND
!            POBB1V10N5=PART OF S/L BUFFER FOR V-COMPONENT OF 10M NEUTRAL WIND
!            POBB1LS5 = PART OF S/L BUFFER FOR SEA/LAND MASK
!            POBB1WL5 = PART OF S/L BUFFER FOR Wl
!       Note: OR,CO,AL,EM,LS,VG,CI,CVH,CVL,TVH,TVL,PRW,U0,V0,
!              PRS,ARG,SAB,HV,Z0H are constants,
!             thus neither included in TL nor AD code.
!            POBB1OR5 = PART OF S/L BUFFER FOR OROGRAPHY
!            POBB1CO5 = PART OF S/L BUFFER FOR CORIOLIS PARAMETER
!            POBB1DL5 = PART OF S/L BUFFER FOR GEOGRAPHICAL NORTH DIRECTION (cos)
!            POBB1DM5 = PART OF S/L BUFFER FOR GEOGRAPHICAL NORTH DIRECTION (sin)
!            POBB1AL5 = PART OF S/L BUFFER FOR ALBEDO
!            POBB1EM5 = PART OF S/L BUFFER FOR EMISSIVITY
!            POBB1LS5 = PART OF S/L BUFFER FOR SEA/LAND MASK
!            POBB1VG5 = PART OF S/L BUFFER FOR VEGETATION
!            POBB1CI5 = PART OF S/L BUFFER FOR SEA-ICE CONCENTRATION
!            POBB1CVH5= PART OF S/L BUFFER FOR HIGH VEGETATION FRACTION
!            POBB1CVL5= PART OF S/L BUFFER FOR LOW  VEGETATION FRACTION
!            POBB1TVH5= PART OF S/L BUFFER FOR HIGH VEGETATION TYPE
!            POBB1TVL5= PART OF S/L BUFFER FOR LOW VEGETATION TYPE
!            POBB1PW5 = PART OF S/L BUFFER FOR WATER PRECIPITATION
!            POBB1PS5 = PART OF S/L BUFFER FOR SNOW PRECIPITATION
!            POBB1ARG5= PART OF S/L BUFFER FOR SILT PERCENT OF CLAY
!            POBB1SAB5= PART OF S/L BUFFER FOR SILT PERCENT OF SAND
!            POBB1HV5 = PART OF S/L BUFFER FOR RESISTANCE TO EVAPOTRANSPIRATION
!            POBB1Z0H5= PART OF S/L BUFFER FOR G*THERMAL ROUGHNESS LENGTH
!            POBB1XTS = PART OF S/L BUFFER FOR Ts clim
!            POBB1XWS = PART OF S/L BUFFER FOR Ws clim
!            POBB1XWP = PART OF S/L BUFFER FOR Wp clim
!            POBB1XSN = PART OF S/L BUFFER FOR neige clim
!            POBB1XTP = PART OF S/L BUFFER FOR Tp clim

!     Externals.
!     ---------
!        SLINT - S/L INTERPOLATOR
!        MPOBSEQ - MESSAGE PASSING OF GOM ARRAYS

!     Modifications.
!     -------------
!     90-05-19 Original by D. Vasiljevic
!     92-12-01 JN.Thepaut  : change of name conventions.
!     92-11-03 F.Bouttier  : Process just one obs for the E.K.F.
!     94-01-05 P.Moll      : Add the precipitations
!     95-09-20 H.Jarvinen  : New CMA- file format
!     95-12-07 F.Bouttier  : Add LDMVO5 option
!     96-12-01 M.Hamrud    : Message passing including 4D-var
!     97-02-20 B.Urban     : Add surface climatology for Canari
!     97-05-23 F.Taillefer : Add geog. North direction interpolation
!     97-10-10 F.Taillefer : Add model errors (for CANARI only)
!     97-11-17 E.Andersson : Introduce CIW and O3
!     98-03-05 D.Paradis   : Add CANARI with LMESSP=.T. case
!     98-07-10 F.Taillefer : Adapt previous change to cycle 19
!     98-06-20 D.Paradis   : Add CANARI/Aladin with LMESSP=.T. case
!     99-09-01 P.Viterbo   : Surface tiles
!     00-06-29 M.Zitouni   : Land/sea mask for CANARI
!     02-06-19 S.Saarinen  : CALL GETDB to get ABNOB, MAPOMM & OBSTYPE
!     08-07-04 H.Hersbach  : Add ocean current and wave Charnock
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     30-Jun-2008 J. Masek : Removed argument KIBUFP1.
!     09-12-04 H. Hersbach : 10-m neutral wind; argument PPARITY
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGSAG   ,NDGENG   ,NFLEVG
USE YOMLUN   , ONLY : NULERR
USE YOMCT0   , ONLY : LELAM    ,LCANARI
USE YOMCST   , ONLY : RPI
USE YOMGEM   , ONLY : NSTTYP
USE YOMSC2   , ONLY : NFLDOBB1
USE YOMOBS   , ONLY : LSLREJ
USE YOMECTAB , ONLY : NTSLOTNO
USE YOMGLOBS , ONLY : NSTASEND ,NCHSEND  ,RLATSEND ,&
 & RLONSEND ,NTYPSEND ,NRETSEND,LPREMPOBS
USE YOMCOCTP , ONLY : NSYNOP   ,NSRSCD   ,NATSCD   ,NSYRCD   ,NSHSCD   ,&
 & NABSCD   ,NSHRED   ,NATSHS   ,NDRIBU   ,NTEMP    ,&
 & NLDTCD   ,NROCOB   ,NSHTCD   ,NROCSH   ,NPILOT   ,&
 & NLDPCD   ,NWPPCD   ,NSHPCD   ,NRADOH  
USE GOMS_MIX
USE YOMDB
USE OBSHOR_CACHE_MIX

!    -------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
LOGICAL           ,INTENT(IN)    :: LDMVO5 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMBMP
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPARITY(NFLDOBB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POBB1U5(KASLB1,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1V5(KASLB1,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1T5(KASLB1,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1GFL5(KASLB1,NFLEVG,NGOMGFL) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1EZ5(KASLB1,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1EH5(KASLB1,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1SP5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1EST5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1U05(KASLB1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1V05(KASLB1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1CHN5(KASLB1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1TS5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1WS5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1SN5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1Z05(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1WL5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1U10N5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1V10N5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1OR5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1CO5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1DL5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1DM5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1AL5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1EM5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1LS5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1VG5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1CI5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1CVH5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1CVL5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1TVH5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1TVL5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1PW5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1PS5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1ARG5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1SAB5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1HV5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1Z0H5(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1XTS(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1XWS(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1XWP(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1XSN(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1XTP(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1ESN(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1ET2(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1EH2(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POBB1EV1(KASLB1) 
REAL(KIND=JPRB)   ,INTENT(IN),OPTIONAL    :: POBB1TCLS5(KASLB1)
REAL(KIND=JPRB)   ,INTENT(IN),OPTIONAL    :: POBB1HUCLS5(KASLB1)

!    -------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSLU    (KPROMB,NFLEVG)
REAL(KIND=JPRB) :: ZSLV    (KPROMB,NFLEVG)
REAL(KIND=JPRB) :: ZSLT    (KPROMB,NFLEVG)
REAL(KIND=JPRB) :: ZSLGFL  (KPROMB,NFLEVG,NGOMGFL)
REAL(KIND=JPRB) :: ZSLEZ   (KPROMB,NFLEVG)
REAL(KIND=JPRB) :: ZSLEH   (KPROMB,NFLEVG)
REAL(KIND=JPRB) :: ZSLSP   (KPROMB)
REAL(KIND=JPRB) :: ZSLEST  (KPROMB)
REAL(KIND=JPRB) :: ZSLTS   (KPROMB)
REAL(KIND=JPRB) :: ZSLTCLS   (KPROMB)
REAL(KIND=JPRB) :: ZSLHUCLS   (KPROMB)
REAL(KIND=JPRB) :: ZSLWS   (KPROMB)
REAL(KIND=JPRB) :: ZSLSN   (KPROMB)
REAL(KIND=JPRB) :: ZSLU0   (KPROMB)
REAL(KIND=JPRB) :: ZSLV0   (KPROMB)
REAL(KIND=JPRB) :: ZSLCHN  (KPROMB)
REAL(KIND=JPRB) :: ZSLZ0   (KPROMB)
REAL(KIND=JPRB) :: ZSLWL   (KPROMB)
REAL(KIND=JPRB) :: ZSLU10N (KPROMB)
REAL(KIND=JPRB) :: ZSLV10N (KPROMB)
REAL(KIND=JPRB) :: ZSLOROG (KPROMB)
REAL(KIND=JPRB) :: ZSLCORI (KPROMB)
REAL(KIND=JPRB) :: ZSLDL   (KPROMB)
REAL(KIND=JPRB) :: ZSLDM   (KPROMB)
REAL(KIND=JPRB) :: ZSLALBE (KPROMB)
REAL(KIND=JPRB) :: ZSLEMIS (KPROMB)
REAL(KIND=JPRB) :: ZSLLSMA (KPROMB)
REAL(KIND=JPRB) :: ZSLVEGE (KPROMB)
REAL(KIND=JPRB) :: ZSLCI   (KPROMB)
REAL(KIND=JPRB) :: ZSLCVH  (KPROMB)
REAL(KIND=JPRB) :: ZSLCVL  (KPROMB)
REAL(KIND=JPRB) :: ZSLTVH  (KPROMB)
REAL(KIND=JPRB) :: ZSLTVL  (KPROMB)
REAL(KIND=JPRB) :: ZSLPRWA (KPROMB)
REAL(KIND=JPRB) :: ZSLPRSN (KPROMB)
REAL(KIND=JPRB) :: ZSLARG  (KPROMB)
REAL(KIND=JPRB) :: ZSLSAB  (KPROMB)
REAL(KIND=JPRB) :: ZSLHV   (KPROMB)
REAL(KIND=JPRB) :: ZSLZ0H  (KPROMB)
REAL(KIND=JPRB) :: ZSLXTP  (KPROMB)
REAL(KIND=JPRB) :: ZSLXTS  (KPROMB)
REAL(KIND=JPRB) :: ZSLXWP  (KPROMB)
REAL(KIND=JPRB) :: ZSLXWS  (KPROMB)
REAL(KIND=JPRB) :: ZSLXSN  (KPROMB)
REAL(KIND=JPRB) :: ZSLESN  (KPROMB)
REAL(KIND=JPRB) :: ZSLET2  (KPROMB)
REAL(KIND=JPRB) :: ZSLEH2  (KPROMB)
REAL(KIND=JPRB) :: ZSLEV1  (KPROMB)
REAL(KIND=JPRB) :: ZSLSST  (KPROMB)
REAL(KIND=JPRB) :: ZLON(KPROMB),ZLAT(KPROMB)
INTEGER(KIND=JPIM) :: ICDTYP(KPROMB),ILSOBS(KPROMB),ICDRET(KPROMB)
INTEGER(KIND=JPIM), POINTER :: IOBTYP(:),IOBNUM(:),IOBNUMA(:)

INTEGER(KIND=JPIM) :: IOFFTS, ITS, JLEV, JOBS
INTEGER(KIND=JPIM) :: IRET, INFO(0)

REAL(KIND=JPRB) :: ZDL2, ZDM2, ZEPS, ZRAY, ZINFO(0)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

#include "fcobs.h"
#include "qabitu.h"

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "mpobseq_pack.intfb.h"
#include "mpobseq.intfb.h"
#include "slint.intfb.h"

#include "openmp_obs.h"

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('OBSHOR',0,ZHOOK_HANDLE)

!    --------------------------------------------------------------------
!*       1. PRESET.
!           ------

CALL GSTATS(43,0)
ITS = NTSLOTNO

!*       1.1   EXTRACT LAT., LON., OBS. TYPE AND OBS. NUMBERS

IOFFTS = NSTASEND(1,ITS)
DO JOBS = 1 , KPROMB
  ZLAT(JOBS) = RLATSEND(IOFFTS+JOBS)
  ZLON(JOBS) = RLONSEND(IOFFTS+JOBS)
  IF ( .NOT. LELAM ) THEN
    IF (ZLON(JOBS) < 0.0_JPRB) THEN
      ZLON(JOBS) = ZLON(JOBS) + 2.0_JPRB*RPI
    ENDIF
    IF (ZLON(JOBS) >= 2.0_JPRB*RPI) THEN
      ZLON(JOBS) = ZLON(JOBS) - 2.0_JPRB*RPI
    ENDIF
  ENDIF
  ICDTYP(JOBS) = NCHSEND(IOFFTS+JOBS)
  ICDRET(JOBS) = NRETSEND(IOFFTS+JOBS)
  ILSOBS(JOBS) = MOD(NTYPSEND(IOFFTS+JOBS),100)
ENDDO

CALL OBSHOR_CACHE_ALLOC('OBSHOR', ITS, KPROMBMP)

IF (.NOT.OBSHOR_CACHE(ITS)%PROCESSED) THEN
  IRET = 0
  CALL GSTATS(1739,0)
  CALL GETDB('OBSHOR', 0, IRET, INFO, 0, ZINFO, 0,   &! Get ROBSU
   & -1, ITS, -1, -1, -1, -1)  
  CALL GSTATS(1739,1)

  IF (IRET /= KPROMBMP) THEN
    WRITE(NULERR,*) '*** Error in OBSHOR: INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'
    WRITE(NULERR,*) 'For timeslot=',ITS,' expecting =',KPROMBMP,', but got =',IRET
    CALL ABORTDB('OBSHOR','INCONSISTENCIES FROM GETDB',IRET)
  ENDIF

! Note: Group needed for Rain Assimilation

  IF (KPROMBMP > 0) THEN
    DO JOBS=1,KPROMBMP
      OBSHOR_CACHE(ITS)%OBSTYPE(JOBS) = ROBSU(JOBS, MDBOTP)
      OBSHOR_CACHE(ITS)%OBSGRP(JOBS)  = ROBSU(JOBS, MDB_AREATYPE_AT_HDR)
      OBSHOR_CACHE(ITS)%MAPOMM(JOBS)  = ROBSU(JOBS, MDB_MAPOMM_AT_INDEX)
      OBSHOR_CACHE(ITS)%ABNOB(JOBS)   = ROBSU(JOBS, MDB_ABNOB_AT_INDEX)
    ENDDO
  ENDIF

  CALL GSTATS(1739,0)
  CALL PUTDB('OBSHOR', 0, IRET, INFO, 0, ZINFO, 0)
  CALL GSTATS(1739,1)

  OBSHOR_CACHE(ITS)%PROCESSED = .TRUE.
ENDIF

!-- Avoid copying, because this is timecritical
IOBTYP  => OBSHOR_CACHE(ITS)%OBSTYPE
IOBNUM  => OBSHOR_CACHE(ITS)%MAPOMM
IOBNUMA => OBSHOR_CACHE(ITS)%ABNOB

IF (LCANARI.AND.LSLREJ) THEN
  DO JOBS = 1,KPROMB
    IF ( ICDRET(JOBS) == 1 ) THEN
      ILSOBS(JOBS) = 2
    ELSE
      IF ( ILSOBS(JOBS) == NSYNOP ) THEN
        IF ( ICDTYP(JOBS) == NSRSCD .OR. ICDTYP(JOBS) == NATSCD .OR. &
           & ICDTYP(JOBS) == NSYRCD .OR. ICDTYP(JOBS) == NRADOH     ) THEN  
          ILSOBS(JOBS) = 1
        ELSEIF ( ICDTYP(JOBS) == NSHSCD .OR. ICDTYP(JOBS) == NABSCD .OR. &
           & ICDTYP(JOBS) == NSHRED .OR. ICDTYP(JOBS) == NATSHS     ) THEN  
          ILSOBS(JOBS) = 0
        ENDIF
      ELSEIF ( ILSOBS(JOBS) == NDRIBU ) THEN
        ILSOBS(JOBS) = 0
      ELSEIF ( ILSOBS(JOBS) == NTEMP ) THEN
        IF ( ICDTYP(JOBS) == NLDTCD .OR. ICDTYP(JOBS) == NROCOB ) THEN
          ILSOBS(JOBS) = 1
        ELSEIF ( ICDTYP(JOBS) == NSHTCD .OR. ICDTYP(JOBS) == NROCSH ) THEN
          ILSOBS(JOBS) = 0
        ENDIF
      ELSEIF ( ILSOBS(JOBS) == NPILOT ) THEN
        IF ( ICDTYP(JOBS) == NLDPCD .OR. ICDTYP(JOBS) == NWPPCD ) THEN
          ILSOBS(JOBS) = 1
        ELSEIF ( ICDTYP(JOBS) == NSHPCD ) THEN
          ILSOBS(JOBS) = 0
        ENDIF
      ENDIF
    ENDIF
  ENDDO
ENDIF

!    -------------------------------------------------------------------
!*       2. S/L INTERPOLATOR.
!           ----------------

CALL SLINT(&
 & KPROMB   ,KASLB1, &
 & ZLAT     ,ZLON, PPARITY, &
 & POBB1U5  ,POBB1V5  ,POBB1T5 ,POBB1GFL5  , &
 & POBB1EZ5 ,POBB1EH5, &
 & POBB1SP5 ,POBB1EST5, &
 & POBB1U05 ,POBB1V05 ,POBB1CHN5, &
 & POBB1TS5 ,POBB1WS5 ,POBB1SN5,POBB1Z05 ,POBB1WL5, &
 & POBB1U10N5,POBB1V10N5, &
 & POBB1OR5 ,POBB1CO5 ,POBB1DL5,POBB1DM5 ,POBB1AL5, &
 & POBB1EM5 ,POBB1LS5 ,POBB1VG5, &
 & POBB1CI5 ,POBB1CVH5,POBB1CVL5,POBB1TVH5,POBB1TVL5, &
 & POBB1PW5 ,POBB1PS5 , &
 & POBB1ARG5,POBB1SAB5,POBB1HV5,POBB1Z0H5, &
 & POBB1XTS ,POBB1XWS , &
 & POBB1XWP ,POBB1XSN ,POBB1XTP, &
 & POBB1ESN ,POBB1ET2 ,POBB1EH2,POBB1EV1 , &
 & ZSLXTS   ,ZSLXWS   , &
 & ZSLXWP   ,ZSLXSN   ,ZSLXTP  , &
 & ZSLU     ,ZSLV     ,ZSLT    ,ZSLGFL   , &
 & ZSLEZ    ,ZSLEH   , &
 & ZSLSP    ,ZSLEST   , &
 & ZSLU0    ,ZSLV0    ,ZSLCHN, &
 & ZSLTS    ,ZSLWS    ,ZSLSN   ,ZSLZ0    ,ZSLWL   , &
 & ZSLU10N  ,ZSLV10N  , &
 & ZSLDL    ,ZSLDM    , &
 & ZSLCORI  ,ZSLALBE  ,ZSLEMIS ,ZSLLSMA  , &
 & ZSLVEGE  , &
 & ZSLCI    ,ZSLCVH   ,ZSLCVL  ,ZSLTVH   ,ZSLTVL  , &
 & ZSLPRWA  ,ZSLPRSN  ,ZSLARG  ,ZSLSAB   , &
 & ZSLHV    ,ZSLZ0H   ,ZSLOROG , &
 & ZSLESN   ,ZSLET2   ,ZSLEH2  ,ZSLEV1   ,ZSLSST  , &
 & ILSOBS,POBB1TCLS5,ZSLTCLS,POBB1HUCLS5,ZSLHUCLS)


!*       2.2  Convert u and v in geographical axes

IF ( NSTTYP > 1 .OR. LELAM ) THEN
  ZEPS=1.E-8_JPRB
  DO JOBS = 1 , KPROMB
    ZRAY=SQRT(ZSLDL(JOBS)*ZSLDL(JOBS)+ZSLDM(JOBS)*ZSLDM(JOBS))
    IF (ZRAY < ZEPS*SQRT(2.0_JPRB)) THEN
      ZSLDL(JOBS)=SIGN(ZEPS,ZSLDL(JOBS))
      ZSLDM(JOBS)=SIGN(ZEPS,ZSLDM(JOBS))
    ENDIF
    ZRAY=MAX(ZRAY,ZEPS*SQRT(2.0_JPRB))
    ZSLDL(JOBS)=ZSLDL(JOBS)/ZRAY
    ZSLDM(JOBS)=ZSLDM(JOBS)/ZRAY
  ENDDO
  DO JLEV = 1 , NFLEVG
    DO JOBS = 1 , KPROMB
      ZDL2=ZSLU(JOBS,JLEV)*ZSLDM(JOBS)-ZSLV(JOBS,JLEV)*ZSLDL(JOBS)
      ZDM2=ZSLU(JOBS,JLEV)*ZSLDL(JOBS)+ZSLV(JOBS,JLEV)*ZSLDM(JOBS)
      ZSLU(JOBS,JLEV)=ZDL2
      ZSLV(JOBS,JLEV)=ZDM2
    ENDDO
  ENDDO
ENDIF


!    -------------------------------------------------------------------
!*        3. INSERT INTERPOLATED VARIABLES IN GOMU ETC.
!            ------------------------------------------

! No worries : We quarantee at least one element allocation for IOB*() in OBSHOR_CACHE_MIX !
IF (LDMVO5) THEN
  CALL MPOBSEQ_PACK(KPROMB,KPROMBMP,IOBTYP,IOBNUM,IOBNUMA,LDMVO5,&
   & ZSLU     ,ZSLV     ,ZSLT     ,ZSLGFL   ,&
   & ZSLEZ    ,ZSLEH   ,&
   & ZSLSP    ,ZSLEST   ,ZSLU0    ,ZSLV0    ,ZSLCHN  ,&
   & ZSLTS    ,ZSLWS    ,ZSLSN    ,ZSLZ0    ,ZSLWL   ,&
   & ZSLU10N  ,ZSLV10N  ,&
   & ZSLCORI  ,ZSLALBE  ,ZSLEMIS  ,ZSLLSMA  ,&
   & ZSLVEGE  ,&
   & ZSLCI    ,ZSLCVH   ,ZSLCVL   ,ZSLTVH   ,ZSLTVL  ,&
   & ZSLPRWA  ,ZSLPRSN  ,ZSLARG   ,ZSLSAB   ,&
   & ZSLHV    ,ZSLZ0H   ,ZSLOROG  ,ZSLDL    ,ZSLDM   ,&
   & ZSLXTS   ,ZSLXSN   ,&
   & ZSLESN   ,ZSLET2   ,ZSLEH2   ,ZSLEV1   ,ZSLSST  ,&
   & ZSLTCLS  ,ZSLHUCLS ,&
   & YGOMUA5,YGOMS5,YGOMUA5_2D,YGOMS5_2D)  
  CALL GSTATS(43,1)
  IF(.NOT. LPREMPOBS) THEN
    CALL GSTATS(93,0)
    CALL MPOBSEQ(LDMVO5,&
     & YGOMUA5,YGOMS5,YGOMUA5_2D,YGOMS5_2D)  
    CALL GSTATS(93,1)
  ENDIF
ELSE
  CALL MPOBSEQ_PACK(KPROMB,KPROMBMP,IOBTYP,IOBNUM,IOBNUMA,LDMVO5,&
   & ZSLU     ,ZSLV     ,ZSLT     ,ZSLGFL ,&
   & ZSLEZ    ,ZSLEH   ,&
   & ZSLSP    ,ZSLEST   ,ZSLU0    ,ZSLV0    ,ZSLCHN  ,&
   & ZSLTS    ,ZSLWS    ,ZSLSN    ,ZSLZ0    ,ZSLWL   ,&
   & ZSLU10N  ,ZSLV10N  ,&
   & ZSLCORI  ,ZSLALBE  ,ZSLEMIS  ,ZSLLSMA  ,&
   & ZSLVEGE  ,&
   & ZSLCI    ,ZSLCVH   ,ZSLCVL   ,ZSLTVH   ,ZSLTVL  ,&
   & ZSLPRWA  ,ZSLPRSN  ,ZSLARG   ,ZSLSAB   ,&
   & ZSLHV    ,ZSLZ0H   ,ZSLOROG  ,ZSLDL    ,ZSLDM   ,&
   & ZSLXTS   ,ZSLXSN   ,&
   & ZSLESN   ,ZSLET2   ,ZSLEH2   ,ZSLEV1   ,ZSLSST  ,&
   & ZSLTCLS  ,ZSLHUCLS ,&
   & YGOMUA,YGOMS,YGOMUA_2D,YGOMS_2D) 
  CALL GSTATS(43,1)
  IF(.NOT. LPREMPOBS) THEN
    CALL GSTATS(93,0)
    CALL MPOBSEQ(LDMVO5,&
     & YGOMUA,YGOMS,YGOMUA_2D,YGOMS_2D) 
    CALL GSTATS(93,1)
  ENDIF
ENDIF 

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('OBSHOR',1,ZHOOK_HANDLE)
END SUBROUTINE OBSHOR
