SUBROUTINE OBSV(CDOBJOB)
!----------------------------------------------------------------------------
!**** *OBSV* - VERTICAL PART OF THE OBSERVATION OPERATORS.
!**
!**   PURPOSE.
!**   -------
!**   PREPARE THE OBSERVATIONS SETS (CASTOR AND ECSET).
!**   MULTITASKING OF THE COMPUTATION OF THE OBS OPERATORS.
!**
!**   INTERFACE.
!**   ---------
!**   *CALL* * OBSV(CDOBJOB)*
!**         CDOBJOB = switch to select direct/TL/AD obs computations
!**         THE COMMON YOMOBA IS AN IMPLICIT INPUT OF OBSV BECAUSE
!**         OF THE DIFFERENT POINTERS NEEDED TO FETCH THE OBS NUMBERS.
!**         THE COMMON YOMOBSET (SET UP BY ECSET OR CASTOR) IS THE ONE
!**         CONTAINING THE DESCRIPTION OF THE OBS SETS, FOR OI AND FOR
!**         THE NON-CORREL OBS IN VAR ANALYSIS.

!     AUTHOR: JEAN PAILLEUX
!     DATE  : 90-02-01

!     MODIFICATIONS
!     -------------
!     JEAN PAILLEUX:   90-06-05
!     PATRICK MOLL :   90-11-23
!     JEAN PAILLEUX:   90-11-28
!     ERIK ANDERSSON:  92-07-01 MAKE TOVS SETS LOOK LIKE ANY OTHER SETS
!     ERIK ANDERSSON:  94-03-28 Make Jo accumulation reproducible
!     F. Bouttier :    95-12-28 Use configuration string
!     ERIK ANDERSSON:  96-02-01 Incorporating Scat in SURFACEO
!     ERIK ANDERSSON:  96-02-13 Diagnostic print of GOM arrays
!     ERIK ANDERSSON:  97-10-02 Time correlated SYNOP obs errors
!     D.   Dent  :     97-10-02 replace SECOND and TIMEF with USER_CLOCK
!     F. Veerse :      97-12-04 No call to SUOBAREA at low-resolution
!                               for DMN 4D-Var
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIMO  , ONLY : NACTIM   ,NOBTOT
USE YOMLUN   , ONLY : NULOUT   
USE YOMCT0   , ONLY : NCONF    ,LCANARI  ,LSCREEN  ,LOBSC1   ,LOBSREF  
USE YOMCOSJO , ONLY : LTC
USE YOMECTAB , ONLY : NTSLTOB  ,NTSLOTNO ,NTSLTOBOFF
USE YOMOBSET , ONLY : NSETOT
USE YOMVAR   , ONLY : NSIM4D   ,NDIAG    ,NSIM4DL
IMPLICIT NONE

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "castor.intfb.h"
#include "comtc.intfb.h"
#include "ecset.intfb.h"
#include "prtgom.intfb.h"
#include "taskob.intfb.h"

CHARACTER(LEN=2)  ,INTENT(IN)    :: CDOBJOB 
INTEGER(KIND=JPIM) :: IOBTOT,IOFFTOT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*    ------------------------------------------------------------------
!*    0. SET UP TO 0. THE LOCAL ARRAY CONTAINING THE COST FUNCTION
!*       FOR EACH SEPARATE TASK AND SOME ADDITONAL POINTERS
!        ---------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('OBSV',0,ZHOOK_HANDLE)
WRITE(NULOUT,*) ' START JO COMPUTATIONS   config=',CDOBJOB

CALL GSTATS(40,0)

!*    ------------------------------------------------------------------
!*    1. MOST OBS TYPES
!        --------------

!*      1.1. COMPUTATION OF THE OBSERVATION SETS

IF(LCANARI)THEN
  CALL CASTOR
ELSEIF(LOBSC1.OR.((NCONF == 131).AND.(LOBSREF))) THEN
  CALL ECSET(-1,NOBTOT,0)
ELSE
  IOBTOT = NTSLTOB(NTSLOTNO)
  IOFFTOT = NTSLTOBOFF(NTSLOTNO)
  CALL ECSET(NTSLOTNO,IOBTOT,IOFFTOT)
ENDIF

WRITE(NULOUT,*) ' Total number of observation sets: ',NSETOT

!*      1.2. MULTITASKED PROCESSING OF THE DIFFERENT SETS OF OBSERVATIONS

!       PRESET CURRENT OBS SET NUMBER (NUMERO)
!       THIS IS DONE BACKWARDS, BECAUSE IN THAT WAY TOVS WILL BE DONE FIRST
!       AND THE RELATIVELY INEXPENSIVE SCAT WILL BE DONE LAST

CALL TASKOB(CDOBJOB)

CALL GSTATS(40,1)

!*    ------------------------------------------------------------------
!*    2. CORRELATED OBSERVATION ERRORS
!        -----------------------------

!*      2.1. TIME CORRELATED SYNOP/DRIBU

IF(NTSLOTNO == NACTIM .AND. NACTIM > 1 .AND. LTC) THEN
  CALL GSTATS(90,0)
  CALL COMTC
  CALL GSTATS(90,1)
ENDIF

!*    ------------------------------------------------------------------
!*    3. Diagnostic print of the GOM arrays (Useful for debugging).
!*       In the normal incremental 3D-Var this will give three
!*       sets of output:
!*       1) RMS of FG at observation points (model levels)
!*       2) RMS of gradient, first simulation
!*       3) RMS of gradient, last  simulation
!*       NDIAG.NE.0 avoids print during GRTEST
!        ---------------------------------------------------------

!      CALL USER_CLOCK(PELAPSED_TIME=ZTIME2)
!      ZTIME=ZTIME2-ZTIME1
!!mf  IF(MOD(NSIM4D,10).EQ.0) WRITE(NULERR,*) ' OBSV TIMING: '
!!mf &         ,MYPROC,ZTIME,(NOOPOT(J),J=1,10)

IF((NCONF/100 == 1 .OR. LSCREEN) .AND. NDIAG /= 0 .AND. &
   & (NSIM4D == 0 .OR. NSIM4D == NSIM4DL) ) THEN  
  CALL PRTGOM(1,NSETOT)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('OBSV',1,ZHOOK_HANDLE)
END SUBROUTINE OBSV
