SUBROUTINE POST_OBSHOR(LDMVO5)

!     *OBSHOR* - Horizontal interpolation of the model variables
!                from grid points to observation points.

!     Purpose.
!     -------
!        To prepare use of semi-lagrangian interpolation routines.

!     Interface.
!     ---------
!        CALL OBSHOR(...)

!     Explicit arguments.
!     ------------------

!     Externals.
!     ---------
!        MPOBSEQ - MESSAGE PASSING OF GOM ARRAYS

!     Modifications.
!     -------------
!     90-05-19 Original by D. Vasiljevic
!     92-12-01 JN.Thepaut  : change of name conventions.
!     92-11-03 F.Bouttier  : Process just one obs for the E.K.F.
!     94-01-05 P.Moll      : Add the precipitations
!     95-09-20 H.Jarvinen  : New CMA- file format
!     95-12-07 F.Bouttier  : Add LDMVO5 option
!     96-12-01 M.Hamrud    : Message passing including 4D-var
!     97-02-20 B.Urban     : Add surface climatology for Canari
!     97-05-23 F.Taillefer : Add geog. North direction interpolation
!     97-10-10 F.Taillefer : Add model errors (for CANARI only)
!     97-11-17 E.Andersson : Introduce CIW and O3
!     98-03-05 D.Paradis   : Add CANARI with LMESSP=.T. case
!     98-07-10 F.Taillefer : Adapt previous change to cycle 19
!     98-06-20 D.Paradis   : Add CANARI/Aladin with LMESSP=.T. case
!     99-09-01 P.Viterbo   : Surface tiles
!     00-06-29 M.Zitouni   : Land/sea mask for CANARI
!     02-06-19 S.Saarinen  : CALL GETDB to get ABNOB, MAPOMM & OBSTYPE
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR
USE YOMECTAB , ONLY : NTSLOTNO,NTSLTOB
USE GOMS_MIX     , ONLY : YGOMUA5,YGOMS5,YGOMUA5_2D,YGOMS5_2D,&
 & YGOMUA,YGOMS,YGOMUA_2D,YGOMS_2D
USE YOMDB
USE OBSHOR_CACHE_MIX

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDMVO5 


INTEGER(KIND=JPIM) :: ITS, JOBS
INTEGER(KIND=JPIM) :: IRET, INFO(0),JTSLOT,IPROMBMP

REAL(KIND=JPRB) :: ZINFO(0)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"
#include "qabitu.h"

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "mpobseq.intfb.h"

#include "openmp_obs.h"

!    --------------------------------------------------------------------
!*       1. PRESET.
!           ------

IF (LHOOK) CALL DR_HOOK('POST_OBSHOR',0,ZHOOK_HANDLE)
CALL GSTATS(43,0)

!*       1.1   EXTRACT LAT., LON., OBS. TYPE AND OBS. NUMBERS

DO JTSLOT=1,NACTIM
  NTSLOTNO=JTSLOT
  ITS = NTSLOTNO

  IPROMBMP = NTSLTOB(ITS)

  CALL OBSHOR_CACHE_ALLOC('OBSHOR', ITS, IPROMBMP)


  IF (.NOT.OBSHOR_CACHE(ITS)%PROCESSED) THEN
    IRET = 0
    CALL GSTATS(1739,0)
    CALL GETDB('OBSHOR', 0, IRET, INFO, 0, ZINFO, 0,   &! Get ROBSU
     & -1, ITS, -1, -1, -1, -1)  
    CALL GSTATS(1739,1)

    IF (IRET /= IPROMBMP) THEN
      WRITE(NULERR,*) '*** Error in POST_OBSHOR: INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'
      WRITE(NULERR,*) 'For timeslot=',ITS,' expecting =',IPROMBMP,', but got =',IRET
      CALL ABORTDB('POST_OBSHOR','INCONSISTENCIES FROM GETDB',IRET)
    ENDIF

! Note: Group needed for Rain Assimilation

    IF (IPROMBMP > 0) THEN
      DO JOBS=1,IPROMBMP
        OBSHOR_CACHE(ITS)%OBSTYPE(JOBS) = ROBSU(JOBS, MDBOTP)
        OBSHOR_CACHE(ITS)%OBSGRP(JOBS)  = ROBSU(JOBS, MDB_AREATYPE_AT_HDR)
        OBSHOR_CACHE(ITS)%MAPOMM(JOBS)  = ROBSU(JOBS, MDB_MAPOMM_AT_INDEX)
        OBSHOR_CACHE(ITS)%ABNOB(JOBS)   = ROBSU(JOBS, MDB_ABNOB_AT_INDEX)
      ENDDO
    ENDIF

    CALL GSTATS(1739,0)
    CALL PUTDB('OBSHOR', 0, IRET, INFO, 0, ZINFO, 0)
    CALL GSTATS(1739,1)

    OBSHOR_CACHE(ITS)%PROCESSED = .TRUE.
  ENDIF
ENDDO

CALL GSTATS(43,1)

!    -------------------------------------------------------------------
!*        3. INSERT INTERPOLATED VARIABLES IN GOMU ETC.
!            ------------------------------------------

CALL GSTATS(93,0)
! No worries : We quarantee at least one element allocation for IOB*() in OBSHOR_CACHE_MIX !
IF (LDMVO5) THEN
  CALL MPOBSEQ(LDMVO5,&
   & YGOMUA5,YGOMS5,YGOMUA5_2D,YGOMS5_2D)  
ELSE
  CALL MPOBSEQ(LDMVO5,&
   & YGOMUA,YGOMS,YGOMUA_2D,YGOMS_2D) 
ENDIF 
CALL GSTATS(93,1)

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('POST_OBSHOR',1,ZHOOK_HANDLE)
END SUBROUTINE POST_OBSHOR
