SUBROUTINE PREINT(KDLEN,KLEN,KSET,POROG,&
 & PRESH5,PRESF5,PGEOPH5,PGEOPF5,&
 & PXP5,PXPD5,PXZ5,PXZD5,PCP5,PRF5,PKAP5,&
 & PUF5,PVF5,PTF5,PQF5,PO3F5,PCLWF5,PCIWF5,PCCF5,&
 & PCSWF5,PCRWF5,PCGWF5,&
 & PTSTAR5,PT05,&
 & PDELP5,PRDELP5,PLNPR5,PALPH5,PRTGR5,&
 & PRPRE5,PRPP5,YDGEMSPROF5,PAL)  

!****  PREINT  - PREPARATION OF THE VERTICAL INTERPOLATION

!     D. VASILJEVIC           02/01/91       ECMWF

!     PURPOSE
!     -------

!        PREINT PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS.

!**   INTERFACE
!     ---------

!        CALL PREINT(KDLEN,..)

!        WHERE KDLEN   = FIRST DIMENSION (OBSERVATIONS)
!              KLEN    = ACTUAL NUMBER OF OBSERVATIONS
!              KSET    = NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!              POROG   = OROGRAPHY AT OBS. PONTS
!              PRESH5  = HALF LEVEL PRESS. AT OBS. POINTS
!              PRESF5  = FULL LEVEL PRESS. AT OBS. POINTS
!              PGEOPH5 = HALF LEVEL GEOP. AT OBS. POINTS
!              PGEOPF5 = FULL LEVEL GEOP. AT OBS. POINTS
!              PXP5    = INTERMEDIATE ARRAY (FOR INTERPOLATION IN P)
!              PXPD5   = ...........................................
!              PXZ5    = INTERMEDIATE ARRAY (FOR INTERPOLATION IN Z)
!              PXZD5   = ...........................................
!              PCP5    = Cp
!              PRF5    = R
!              PKAP5   =
!              PUF5    = U VALUES (MODEL) AT OB. POINS
!              PVF5    = V VALUES (MODEL) AT OB. POINS
!              PTF5    = T VALUES (MODEL) AT OB. POINS
!              PQF5    = Q VALUES (MODEL) AT OB. POINS
!              PO3F5   = O3 VALUES (MODEL) AT OB. POINS
!              PCLWF5  = CLW VALUES (MODEL) AT OB. POINS
!              PCIWF5  = CIW VALUES (MODEL) AT OB. POINS
!              PCCF5   = CC VALUES (MODEL) AT OB. POINS
!              PCSWF5    = S (snow) VALUES (MODEL) AT OB. POINS
!              PCRWF5    = R (rain) VALUES (MODEL) AT OB. POINS
!              PCGWF5    = G (graupel) VALUES (MODEL) AT OB. POINS
!              PTSTAR5 = T*
!              PT05    = T0
!              PDELP5  = PRESSURE DIFF. ACROSS LAYERS
!              PRDELP5 = INVERSE OF THE ABOVE
!              PLNPR5  = LOG. OF THE RATIO OF PRESSURE
!              PALPH5  = COEFFICIENTS OF THE HYDROSTATICS
!              PRTGR5  = FOR PRESS. GRAD. TERM AND ENE. CONV.
!              PRPRE5  = INVERSE OF PRESSURE
!              PRPP5   = INVERSE OF P(j)*P(j+1)
!              YDGEMSPROF5   = GEMS PROFILES (MODEL) AT OB. POINS (OPTIONAL)
!              PAL     = Albedo AT OBS. PONTS

!        PREINT IS CALLED FROM UPPERAIR AND SATEM

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!        GPPREH - COMPUTES HALF LEVEL PRESSURES
!        GPPREF - COMPUTES FULL LEVEL PRESSURES
!        PPINIT - SET-UP AUX. ARRAYS FOR VERTICAL INTERPOLATION
!        GPXYB  - COMPUTES AUXILLARY ARRAYS
!        CTSTAR - COMPUTES STANDARD SURFACE TEMPERATURE AND SURFACE
!                 TEMPERATURE
!        GPRCP  -
!        GPGEO  -
!        PPINITZ- SET-UP AUX. ARRAYS FOR VERTICAL INTERPOLATION

!**   MODIFICATIONS
!     -------------
!     E.Andersson 92-06-22 : Introduce SATEM
!     E.Andersson 96-02-07 : Introduce SCATT
!     L. Isaksen  96-02-20 : NFLEV --> NFLEVG
!     M. Fisher   96-05-16 : Fill from GOMU5, etc. if in tangent linear
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Andersson 99-07-02 : Incorporate clouds
!     F.Chevallier00-03-24 : Initialisation of ozone for RTTOV
!     R. El Khatib : 01-08-07 Pruning options
!     M.Hamrud  15-Jan-2006  Revised GPRCP 
!     E. Sevault 06-May-2009 workarounds for SX9 compiler (precomputation of
!               indirect addressing + replace test on optional argument in loop
!               by local logical variable)
!     S. Martinez 24-June-2009 replace test on optional argument PAL in loop
!                               by local logical variable
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMSTA   , ONLY : NLEXTRAP
USE YOMGEM   , ONLY : VBH      ,VAH      ,VC       ,VDELB
USE YOMOBSET , ONLY : MTYPOB
USE YOMVRTL  , ONLY : LTLINT   ,LOBSTL
USE YOMDB
USE GOMS_MIX     , ONLY : YGOMEC, YGOMUA, YGOMUA5, YGOMS, YGOMS5   
USE YOM_YGFL , ONLY : NGHG, YGHG, NGRG, YGRG, NAERO, YAERO 
USE TYPE_GEMS_PROFILES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POROG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXPD5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXZ5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXZD5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKAP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO3F5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCIWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCCF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCSWF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCRWF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCGWF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTAR5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT05(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTGR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPRE5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB),INTENT(OUT),OPTIONAL   :: PAL(KDLEN) 

!     ------------------------------------------------------------------

LOGICAL :: LLSPROF5, LLPAL
INTEGER(KIND=JPIM) :: ISET,ILEN,IOBSTYPE,JOBS,JLEV,JGHG,JGRG,JAERO
INTEGER(KIND=JPIM) :: IGEMS,ISTGHG,IENGHG,ISTGRG,IENGRG,ISTAERO,IENAERO
INTEGER(KIND=JPIM) :: IGHG,IGRG,IAERO
INTEGER(KIND=JPIM) :: IPOSM(KLEN)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

TYPE (TYPE_GEMSPROF), OPTIONAL   ,INTENT(OUT) :: YDGEMSPROF5(KDLEN,NGEMS)

!     ------------------------------------------------------------------

#include "ctstar.intfb.h"
#include "gpgeo.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "ppinit.intfb.h"
#include "ppinitz.intfb.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PREINT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*
!*
!        1.       PREPARATION OF INTERPOLATION
!                 ----------------------------

LLSPROF5 = PRESENT(YDGEMSPROF5)
LLPAL = PRESENT(PAL)
ISET=ABS(KSET)
ILEN     = KLEN
IOBSTYPE = MOD(MTYPOB(ISET),100)

!   KSET < 0 INDICATES THAT MODEL DATA HAVE ALREADY BEEN FILLED
IF(KSET < 0) GOTO 120

!*          1.1   FIND OUT RELEVANT ARRAYS FOR EACH OBSERVATION

DO JOBS = 1,ILEN
  IPOSM(JOBS) = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
ENDDO

!*          Albedo
IF (LLPAL) THEN
  DO JOBS=1,ILEN
    PAL(JOBS) = YGOMEC(IPOSM(JOBS))%ECALBE
  ENDDO
ENDIF

DO JOBS = 1,ILEN

!*          1.1.1 SET OROGRAPHY

  POROG(JOBS) = YGOMEC(IPOSM(JOBS))%ECOROG

!*          1.1.2 SURFACE PRESSURE

  IF (LTLINT.AND.LOBSTL) THEN
    PRESH5(JOBS,NFLEVG) = YGOMS5(IPOSM(JOBS))%SP
  ELSE
    PRESH5(JOBS,NFLEVG) = YGOMS(IPOSM(JOBS))%SP
  ENDIF
  PRESH5(JOBS,NFLEVG) = EXP(PRESH5(JOBS,NFLEVG))
ENDDO

!*          1.1.3 FILL OUT THE AD HOC ARRAYS WITH THE MODEL VALUES FOR
!                 U, V, T AND Q

DO JOBS = 1,ILEN
  IF(IPOSM(JOBS) /= YGOMUA(IPOSM(JOBS))%IOBS) THEN
    WRITE(0,*) 'PREINT - IPOSM(JOBS) /= YGOMUA(IPOSM(JOBS))%IOBS',&
     &JOBS,IPOSM(JOBS),YGOMUA(IPOSM(JOBS))%IOBS
  ENDIF
  IF(IOBSTYPE /= YGOMUA(IPOSM(JOBS))%ITYP) THEN
    WRITE(0,*) 'PREINT - IOBSTYPE /= YGOMUA(IPOSM(JOBS))%ITYP',&
     &JOBS,IPOSM(JOBS),IOBSTYPE,YGOMUA(IPOSM(JOBS))%ITYP
  ENDIF

! Initialisation of ozone for RTTOV
  PO3F5(JOBS,1:NFLEVG) = 1.e-07

  IF(LLSPROF5) THEN
    DO IGEMS=1,NGEMS
      YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(:)=0.0_JPRB
      YDGEMSPROF5(JOBS,IGEMS)%RTTOVPROF(:)=0.0_JPRB
      YDGEMSPROF5(JOBS,IGEMS)%IGRIB=0
      YDGEMSPROF5(JOBS,IGEMS)%CSPECIES='XXX'
      YDGEMSPROF5(JOBS,IGEMS)%LRTTOV=.FALSE.
    ENDDO
  ENDIF

  IF (LTLINT.AND.LOBSTL) THEN
    IF(YGOMUA5(IPOSM(JOBS))%LU) THEN
      PUF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%U(:)
    ELSE
      PUF5(JOBS,1:NFLEVG) = YGOMS5(IPOSM(JOBS))%UL
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LV) THEN
      PVF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%V(:)
    ELSE
      PVF5(JOBS,1:NFLEVG) = YGOMS5(IPOSM(JOBS))%VL
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LT) THEN
      PTF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%T(:)
    ELSE
      PTF5(JOBS,1:NFLEVG) = YGOMS5(IPOSM(JOBS))%TL
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LQ) THEN
      PQF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%Q(:)
    ELSE
      PQF5(JOBS,1:NFLEVG) = YGOMS5(IPOSM(JOBS))%QL
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LO3) THEN
      PO3F5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%O3(:)
    ELSE
      PO3F5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LCLW) THEN
      PCLWF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%CLW(:)
    ELSE
      PCLWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LCIW) THEN
      PCIWF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%CIW(:)
    ELSE
      PCIWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LCC) THEN
      PCCF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%CC(:)
    ELSE
      PCCF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(LLSPROF5) THEN
      IGEMS=0
      IGHG=0
      DO JGHG=1,NGHG
        IF(YGOMUA5(IPOSM(JOBS))%LGHG(JGHG)) THEN
          IGEMS=IGEMS+1
          ISTGHG = 1+IGHG*NFLEVG
          IENGHG = ISTGHG+NFLEVG-1
          YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(1:NFLEVG) = &
           & YGOMUA5(IPOSM(JOBS))%GHG(ISTGHG:IENGHG)
          YDGEMSPROF5(JOBS,IGEMS)%IGRIB = YGHG(JGHG)%IGRBCODE
          YDGEMSPROF5(JOBS,IGEMS)%CSPECIES = 'GHG'
          YDGEMSPROF5(JOBS,IGEMS)%LRTTOV = .TRUE.
          IGHG=IGHG+1
        ENDIF
      ENDDO
      IGRG=0
      DO JGRG=1,NGRG
        IF(YGOMUA5(IPOSM(JOBS))%LGRG(JGRG)) THEN
          IGEMS=IGEMS+1
          ISTGRG = 1+IGRG*NFLEVG
          IENGRG = ISTGRG+NFLEVG-1
          YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(1:NFLEVG) = &
           & YGOMUA5(IPOSM(JOBS))%GRG(ISTGRG:IENGRG)
          YDGEMSPROF5(JOBS,IGEMS)%IGRIB = YGRG(JGRG)%IGRBCODE
          YDGEMSPROF5(JOBS,IGEMS)%CSPECIES = 'GRG'
          YDGEMSPROF5(JOBS,IGEMS)%LRTTOV = .FALSE.
          IGRG=IGRG+1
        ENDIF
      ENDDO
      IAERO=0
      DO JAERO=1,NAERO
        IF(YGOMUA5(IPOSM(JOBS))%LAERO(JAERO)) THEN
          IGEMS=IGEMS+1
          ISTAERO = 1+IAERO*NFLEVG
          IENAERO = ISTAERO+NFLEVG-1
          YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(1:NFLEVG) = &
           & YGOMUA5(IPOSM(JOBS))%AERO(ISTAERO:IENAERO)
          YDGEMSPROF5(JOBS,IGEMS)%IGRIB = YAERO(JAERO)%IGRBCODE
          YDGEMSPROF5(JOBS,IGEMS)%CSPECIES = 'AER'
          YDGEMSPROF5(JOBS,IGEMS)%LRTTOV = .FALSE.
          IAERO=IAERO+1
        ENDIF
      ENDDO
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LS)  THEN
      PCSWF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%S(:)
    ELSE
      PCSWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LR)  THEN
      PCRWF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%R(:)
    ELSE
      PCRWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5(IPOSM(JOBS))%LG)  THEN
      PCGWF5(JOBS,1:NFLEVG) = YGOMUA5(IPOSM(JOBS))%G(:)
    ELSE
      PCGWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
  ELSE
    IF(YGOMUA(IPOSM(JOBS))%LU) THEN
      PUF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%U(:)
    ELSE
      PUF5(JOBS,1:NFLEVG) = YGOMS(IPOSM(JOBS))%UL
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LV) THEN
      PVF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%V(:)
    ELSE
      PVF5(JOBS,1:NFLEVG) = YGOMS(IPOSM(JOBS))%VL
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LT) THEN
      PTF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%T(:)
    ELSE
      PTF5(JOBS,1:NFLEVG) = YGOMS(IPOSM(JOBS))%TL
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LQ) THEN
      PQF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%Q(:)
    ELSE
      PQF5(JOBS,1:NFLEVG) = YGOMS(IPOSM(JOBS))%QL
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LO3) THEN
      PO3F5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%O3(:)
    ELSE
      PO3F5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LCLW) THEN
      PCLWF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%CLW(:)
    ELSE
      PCLWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LCIW) THEN
      PCIWF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%CIW(:)
    ELSE
      PCIWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LCC) THEN
      PCCF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%CC(:)
    ELSE
      PCCF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF

    IF(LLSPROF5) THEN
      IGEMS=0
      IGHG=0
      DO JGHG=1,NGHG
        IF(YGOMUA(IPOSM(JOBS))%LGHG(JGHG)) THEN
          IGEMS=IGEMS+1
          ISTGHG = 1+IGHG*NFLEVG
          IENGHG = ISTGHG+NFLEVG-1
          YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(1:NFLEVG) = &
           & YGOMUA(IPOSM(JOBS))%GHG(ISTGHG:IENGHG)
          YDGEMSPROF5(JOBS,IGEMS)%IGRIB = YGHG(JGHG)%IGRBCODE
          YDGEMSPROF5(JOBS,IGEMS)%CSPECIES = 'GHG'
          YDGEMSPROF5(JOBS,IGEMS)%LRTTOV = .TRUE.
          IGHG=IGHG+1
        ENDIF
      ENDDO
      IGRG=0
      DO JGRG=1,NGRG
        IF(YGOMUA(IPOSM(JOBS))%LGRG(JGRG)) THEN
          IGEMS=IGEMS+1
          ISTGRG = 1+IGRG*NFLEVG
          IENGRG = ISTGRG+NFLEVG-1
          YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(1:NFLEVG) = &
           & YGOMUA(IPOSM(JOBS))%GRG(ISTGRG:IENGRG)
          YDGEMSPROF5(JOBS,IGEMS)%IGRIB = YGRG(JGRG)%IGRBCODE
          YDGEMSPROF5(JOBS,IGEMS)%CSPECIES = 'GRG'
          YDGEMSPROF5(JOBS,IGEMS)%LRTTOV = .FALSE.
          IGRG=IGRG+1
        ENDIF
      ENDDO
      IAERO=0
      DO JAERO=1,NAERO
        IF(YGOMUA(IPOSM(JOBS))%LAERO(JAERO)) THEN
          IGEMS=IGEMS+1
          ISTAERO = 1+IAERO*NFLEVG
          IENAERO = ISTAERO+NFLEVG-1
          YDGEMSPROF5(JOBS,IGEMS)%MODELPROF(1:NFLEVG) = &
           & YGOMUA(IPOSM(JOBS))%AERO(ISTAERO:IENAERO)
          YDGEMSPROF5(JOBS,IGEMS)%IGRIB = YAERO(JAERO)%IGRBCODE
          YDGEMSPROF5(JOBS,IGEMS)%CSPECIES = 'AER'
          YDGEMSPROF5(JOBS,IGEMS)%LRTTOV = .FALSE.
          IAERO=IAERO+1
        ENDIF
      ENDDO
    ENDIF
!rje
    IF(YGOMUA(IPOSM(JOBS))%LS) THEN
       PCSWF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%S(:)
    ELSE
      PCSWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LR) THEN
       PCRWF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%R(:)
    ELSE
       PCRWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
    IF(YGOMUA(IPOSM(JOBS))%LG) THEN
       PCGWF5(JOBS,1:NFLEVG) = YGOMUA(IPOSM(JOBS))%G(:)
    ELSE
       PCGWF5(JOBS,1:NFLEVG) = 0.0_JPRB
    ENDIF
  ENDIF
ENDDO

!*          1.2   COMPUTATION OF HALF LEVEL PRESSURES
120 CONTINUE
CALL GPPREH(KDLEN,1,ILEN,NFLEVG,VAH,VBH,PRESH5)

!*          1.3   SOME OTHER PREPARATIONS FOR INTERPOLATION

CALL GPXYB(KDLEN,1,ILEN,NFLEVG,&
 & VDELB,VC,&
 & PRESH5,&
 & PDELP5,PRDELP5,PLNPR5,PALPH5,PRTGR5,&
 & PRPRE5,PRPP5)  

!*          1.4   COMPUTATION OF HALF AND FULL LEVEL PRESSURES

CALL GPPREF(KDLEN,1,ILEN,NFLEVG,PALPH5,PRESH5,PRESF5)

!*          1.5   COMPUTION OF THE GEOPOTENTIALS ON THE MODEL LEVELS

!*          1.5.1  PRELIMINARY COMPUTATION

CALL GPRCP(KDLEN,1,ILEN,NFLEVG,PQ=PQF5(1,1),PCP=PCP5,PR=PRF5,PKAP=PKAP5)

!*          1.5.2  GEOP. CALCULATION

DO JLEV=0,NFLEVG-1
  PGEOPH5(1:ILEN,JLEV)=0.0_JPRB
ENDDO
PGEOPH5(1:ILEN,NFLEVG)=POROG(1:ILEN)
CALL GPGEO(KDLEN,1,ILEN,NFLEVG,PGEOPH5,PGEOPF5,PTF5(1,1),PRF5,PLNPR5,PALPH5)

!*          1.6   COMPUTION OF AUX. ARRAYS FOR VERTICAL INTERPOLATIONS

!*          1.6.1 VERTICAL INTERPOLATIONS IN P

CALL PPINIT (KDLEN,1,ILEN,NFLEVG,PRESH5,PRESF5,PXP5,PXPD5)

!*          1.6.2 VERTICAL INTERPOLATIONS IN Z

CALL PPINITZ(KDLEN,1,ILEN,NFLEVG,PGEOPH5,PGEOPF5,PXZ5,PXZD5)

!*          1.7   COMPUTE T* AND T0

CALL CTSTAR(KDLEN,1,ILEN,&
 & PTF5(1,NLEXTRAP),PRESH5(1,NFLEVG),PRESF5(1,NLEXTRAP),&
 & POROG,&
 & PTSTAR5,PT05)  

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PREINT',1,ZHOOK_HANDLE)
END SUBROUTINE PREINT
