SUBROUTINE PREINT2D(KDLEN,KLEN,KSET,KHORIZ,POROG,&
 & PRESH5,PRESF5,PGEOPH5,PGEOPF5,&
 & PXP5,PXPD5,PXZ5,PXZD5,PCP5,PRF5,PKAP5,&
 & PUF5,PVF5,PTF5,PQF5,PO3F5,PCLWF5,PCIWF5,PCCF5,&
 & PCSWF5,PCRWF5,PCGWF5,&
 & PTSTAR5,PT05,&
 & PDELP5,PRDELP5,PLNPR5,PALPH5,PRTGR5,&
 & PRPRE5,PRPP5)  

!****  PREINT  - PREPARATION OF THE VERTICAL INTERPOLATION

!     D. VASILJEVIC           02/01/91       ECMWF

!     PURPOSE
!     -------

!        PREINT PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS.

!**   INTERFACE
!     ---------

!        CALL PREINT(KDLEN,..)

!        WHERE KDLEN   = FIRST DIMENSION (OBSERVATIONS)
!              KLEN    = ACTUAL NUMBER OF OBSERVATIONS
!              KSET    = NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!              KHORIZ  = NUMBER of HORIZONTAL LOCATION PER OBSERVATION
!              POROG   = OROGRAPHY AT OBS. PONTS
!              PRESH5  = HALF LEVEL PRESS. AT OBS. POINTS
!              PRESF5  = FULL LEVEL PRESS. AT OBS. POINTS
!              PGEOPH5 = HALF LEVEL GEOP. AT OBS. POINTS
!              PGEOPF5 = FULL LEVEL GEOP. AT OBS. POINTS
!              PXP5    = INTERMEDIATE ARRAY (FOR INTERPOLATION IN P)
!              PXPD5   = ...........................................
!              PXZ5    = INTERMEDIATE ARRAY (FOR INTERPOLATION IN Z)
!              PXZD5   = ...........................................
!              PCP5    = Cp
!              PRF5    = R
!              PKAP5   =
!              PUF5    = U VALUES (MODEL) AT OB. POINS
!              PVF5    = V VALUES (MODEL) AT OB. POINS
!              PTF5    = T VALUES (MODEL) AT OB. POINS
!              PQF5    = Q VALUES (MODEL) AT OB. POINS
!              PO3F5   = O3 VALUES (MODEL) AT OB. POINS
!              PCLWF5  = CLW VALUES (MODEL) AT OB. POINS
!              PCIWF5  = CIW VALUES (MODEL) AT OB. POINS
!              PCCF5   = CC VALUES (MODEL) AT OB. POINS
!              PCSWF5  = CS VALUES (MODEL) AT OB. POINS
!              PTSTAR5 = T*
!              PT05    = T0
!              PDELP5  = PRESSURE DIFF. ACROSS LAYERS
!              PRDELP5 = INVERSE OF THE ABOVE
!              PLNPR5  = LOG. OF THE RATIO OF PRESSURE
!              PALPH5  = COEFFICIENTS OF THE HYDROSTATICS
!              PRTGR5  = FOR PRESS. GRAD. TERM AND ENE. CONV.
!              PRPRE5  = INVERSE OF PRESSURE
!              PRPP5   = INVERSE OF P(j)*P(j+1)

!        PREINT IS CALLED FROM UPPERAIR AND SATEM

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!        GPPREH - COMPUTES HALF LEVEL PRESSURES
!        GPPREF - COMPUTES FULL LEVEL PRESSURES
!        PPINIT - SET-UP AUX. ARRAYS FOR VERTICAL INTERPOLATION
!        GPXYB  - COMPUTES AUXILLARY ARRAYS
!        CTSTAR - COMPUTES STANDARD SURFACE TEMPERATURE AND SURFACE
!                 TEMPERATURE
!        GPRCP  -
!        GPGEO  -
!        PPINITZ- SET-UP AUX. ARRAYS FOR VERTICAL INTERPOLATION

!**   MODIFICATIONS
!     -------------
!     S. Healy: 2005-04-12 Adapted from preint.F90
!     M.Hamrud  15-Jan-2006  Revised GPRCP
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMSTA   , ONLY : NLEXTRAP
USE YOMGEM   , ONLY : VBH      ,VAH      ,VC       ,VDELB
USE YOMOBSET , ONLY : MTYPOB
USE YOMVRTL  , ONLY : LTLINT   ,LOBSTL
USE YOMDB
USE GOMS_MIX     , ONLY : YGOMEC_2D, YGOMUA_2D, YGOMUA5_2D, YGOMS_2D, YGOMS5_2D   

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POROG(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPH5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXP5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXPD5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXZ5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXZD5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKAP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO3F5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLWF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCIWF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCCF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCSWF5(KDLEN,0:NFLEVG,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCRWF5(KDLEN,0:NFLEVG,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCGWF5(KDLEN,0:NFLEVG,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTAR5(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT05(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTGR5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPRE5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPP5(KDLEN,NFLEVG,KHORIZ) 


INTEGER(KIND=JPIM) :: ISET,ILEN,IOBSTYPE,IPOSM,JOBS,JLEV,JHOR,JJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ctstar.intfb.h"
#include "gpgeo.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "ppinit.intfb.h"
#include "ppinitz.intfb.h"

#include "openmp_obs.h"
!     ------------------------------------------------------------------
!*
!*
!        1.       PREPARATION OF INTERPOLATION
!                 ----------------------------

IF (LHOOK) CALL DR_HOOK('PREINT2D',0,ZHOOK_HANDLE)

ISET=ABS(KSET)
ILEN     = KLEN
IOBSTYPE = MOD(MTYPOB(ISET),100)

!   KSET < 0 INDICATES THAT MODEL DATA HAVE ALREADY BEEN FILLED
IF(KSET < 0) GOTO 120

!*          1.1   FIND OUT RELEVANT ARRAYS FOR EACH OBSERVATION

DO JOBS = 1,ILEN
  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

!*          1.1.1 SET OROGRAPHY

  POROG(JOBS,:) = YGOMEC_2D(IPOSM)%ECOROG(:)

!*          1.1.2 SURFACE PRESSURE

  IF (LTLINT.AND.LOBSTL) THEN
    PRESH5(JOBS,NFLEVG,:) = YGOMS5_2D(IPOSM)%SP(:)
  ELSE
    PRESH5(JOBS,NFLEVG,:) = YGOMS_2D(IPOSM)%SP(:)
  ENDIF
  PRESH5(JOBS,NFLEVG,:) = EXP(PRESH5(JOBS,NFLEVG,:))
ENDDO

!*          1.1.3 FILL OUT THE AD HOC ARRAYS WITH THE MODEL VALUES FOR
!                 U, V, T AND Q


DO JOBS = 1,ILEN
  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  IF(IPOSM /= YGOMUA_2D(IPOSM)%IOBS) THEN
    WRITE(0,*) 'PREINT2D - IPOSM /= YGOMUA_2D(IPOSM)%IOBS',&
     &JOBS,IPOSM,YGOMUA_2D(IPOSM)%IOBS
  ENDIF
  IF(IOBSTYPE /= YGOMUA_2D(IPOSM)%ITYP) THEN
    WRITE(0,*) 'PREINT2D - IOBSTYPE /= YGOMUA_2D(IPOSM)%ITYP',&
     &JOBS,IPOSM,IOBSTYPE,YGOMUA_2D(IPOSM)%ITYP
  ENDIF
  

! Initialisation of ozone for RTTOV
  PO3F5(JOBS,1:NFLEVG,:) = 1.e-07

  IF (LTLINT.AND.LOBSTL) THEN
    IF(YGOMUA5_2D(IPOSM)%LU) THEN
      PUF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%U(:,:)
    ELSE
      DO JJ = 1,KHORIZ
        PUF5(JOBS,:,JJ) = YGOMS5_2D(IPOSM)%UL(JJ)
      ENDDO  
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LV) THEN
      PVF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%V(:,:)
    ELSE
      DO JJ=1,KHORIZ
        PVF5(JOBS,:,JJ) = YGOMS5_2D(IPOSM)%VL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LT) THEN
      PTF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%T(:,:)
    ELSE
      DO JJ = 1,KHORIZ 
        PTF5(JOBS,:,JJ) = YGOMS5_2D(IPOSM)%TL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LQ) THEN
      PQF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%Q(:,:)
    ELSE
      DO JJ = 1,KHORIZ
        PQF5(JOBS,:,JJ) = YGOMS5_2D(IPOSM)%QL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LO3) THEN
      PO3F5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%O3(:,:)
    ELSE
      PO3F5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LCLW) THEN
      PCLWF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%CLW(:,:)
    ELSE
      PCLWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LCIW) THEN
      PCIWF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%CIW(:,:)
    ELSE
      PCIWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LCC) THEN
      PCCF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%CC(:,:)
    ELSE
      PCCF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LS) THEN
      PCSWF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%S(:,:)
    ELSE
      PCSWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LR) THEN
      PCRWF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%R(:,:)
    ELSE
      PCRWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA5_2D(IPOSM)%LG) THEN
       PCGWF5(JOBS,1:NFLEVG,:) = YGOMUA5_2D(IPOSM)%G(:,:)
    ELSE
      PCGWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
  ELSE
    IF(YGOMUA_2D(IPOSM)%LU) THEN
      PUF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%U(:,:)
    ELSE
      DO JJ = 1,KHORIZ
        PUF5(JOBS,:,JJ) = YGOMS_2D(IPOSM)%UL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LV) THEN
      PVF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%V(:,:)
    ELSE
      DO JJ = 1,KHORIZ
        PVF5(JOBS,:,JJ) = YGOMS_2D(IPOSM)%VL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LT) THEN
      PTF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%T(:,:)
    ELSE
      DO JJ = 1,KHORIZ  
        PTF5(JOBS,:,JJ) = YGOMS_2D(IPOSM)%TL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LQ) THEN
      PQF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%Q(:,:)
    ELSE
      DO JJ = 1,KHORIZ
        PQF5(JOBS,:,JJ) = YGOMS_2D(IPOSM)%QL(JJ)
      ENDDO
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LO3) THEN
      PO3F5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%O3(:,:)
    ELSE
      PO3F5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LCLW) THEN
      PCLWF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%CLW(:,:)
    ELSE
      PCLWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LCIW) THEN
      PCIWF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%CIW(:,:)
    ELSE
      PCIWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LCC) THEN
      PCCF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%CC(:,:)
    ELSE
      PCCF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LS) THEN
      PCSWF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%S(:,:)
    ELSE
      PCSWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LR) THEN
      PCRWF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%R(:,:)
    ELSE
      PCRWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
    IF(YGOMUA_2D(IPOSM)%LG) THEN
      PCGWF5(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%G(:,:)
    ELSE
      PCGWF5(JOBS,1:NFLEVG,:) = 0.0_JPRB
    ENDIF
  ENDIF
ENDDO

!*          1.2   COMPUTATION OF HALF LEVEL PRESSURES
120 CONTINUE

DO JHOR = 1,KHORIZ

  CALL GPPREH(KDLEN,1,ILEN,NFLEVG,VAH,VBH,PRESH5(:,:,JHOR))

!*          1.3   SOME OTHER PREPARATIONS FOR INTERPOLATION

  CALL GPXYB(KDLEN,1,ILEN,NFLEVG,&
   & VDELB,VC,&
   & PRESH5(:,:,JHOR),&
   & PDELP5(:,:,JHOR),PRDELP5(:,:,JHOR),PLNPR5(:,:,JHOR),PALPH5(:,:,JHOR),PRTGR5(:,:,JHOR),&
   & PRPRE5(:,:,JHOR),PRPP5(:,:,JHOR))  

!*          1.4   COMPUTATION OF HALF AND FULL LEVEL PRESSURES

  CALL GPPREF(KDLEN,1,ILEN,NFLEVG,PALPH5(:,:,JHOR),PRESH5(:,:,JHOR),PRESF5(:,:,JHOR))

!*          1.5   COMPUTION OF THE GEOPOTENTIALS ON THE MODEL LEVELS

!*          1.5.1  PRELIMINARY COMPUTATION

  CALL GPRCP&
  &(KDLEN,1,ILEN,NFLEVG,PQ=PQF5(1,1,JHOR),PCP=PCP5(:,:,JHOR),&
  &PR=PRF5(:,:,JHOR),PKAP=PKAP5(:,:,JHOR))

!*          1.5.2  GEOP. CALCULATION

  DO JLEV=0,NFLEVG-1
    PGEOPH5(1:ILEN,JLEV,JHOR)=0.0_JPRB
  ENDDO
  PGEOPH5(1:ILEN,NFLEVG,JHOR)=POROG(1:ILEN,JHOR)
  CALL GPGEO&
  &(KDLEN,1,ILEN,NFLEVG,PGEOPH5(:,:,JHOR),PGEOPF5(:,:,JHOR),&
  &PTF5(1,1,JHOR),PRF5(:,:,JHOR),PLNPR5(:,:,JHOR),PALPH5(:,:,JHOR))

!*          1.6   COMPUTION OF AUX. ARRAYS FOR VERTICAL INTERPOLATIONS

!*          1.6.1 VERTICAL INTERPOLATIONS IN P

  CALL PPINIT(KDLEN,1,ILEN,NFLEVG,PRESH5(:,:,JHOR),&
  &PRESF5(:,:,JHOR),PXP5(:,:,:,JHOR),PXPD5(:,:,:,JHOR))

!*          1.6.2 VERTICAL INTERPOLATIONS IN Z

  CALL PPINITZ(KDLEN,1,ILEN,NFLEVG,PGEOPH5(:,:,JHOR),&
  &PGEOPF5(:,:,JHOR),PXZ5(:,:,:,JHOR),PXZD5(:,:,:,JHOR))

!*          1.7   COMPUTE T* AND T0

  CALL CTSTAR(KDLEN,1,ILEN,&
   & PTF5(1,NLEXTRAP,JHOR),PRESH5(1,NFLEVG,JHOR),PRESF5(1,NLEXTRAP,JHOR),&
   & POROG(:,JHOR),&
   & PTSTAR5(:,JHOR),PT05(:,JHOR))  

ENDDO


IF (LHOOK) CALL DR_HOOK('PREINT2D',1,ZHOOK_HANDLE)
END SUBROUTINE PREINT2D
