SUBROUTINE PREINT2DAD(KDLEN,KLEN,KSET,KHORIZ,&
 & PRESH,PRESF,PGEOPH,PGEOPF,&
 & PXP,PXPD,PXZ,PXZD,PCP,PRF,PKAP,&
 & PUF,PVF,PTF,PQF,PO3F,PCLWF,PCIWF,PCCF,&
 & PTSTAR,PT0,&
 & PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PRPRE,PRPP,&
 & PRESH5,PRESF5,&
 & PXP5,PXZ5,PCP5,PRF5,PTF5,&
 & PDELP5,PRDELP5,PLNPR5,PALPH5,&
 & PRPRE5,PRPP5)  

!****  PREINTAD  - ADJOINT OF PREPARATION OF THE VERTICAL INTERPOLATION

!     D. VASILJEVIC           02/01/91       ECMWF

!     PURPOSE
!     -------

!        ADJOINT CALCULATION OF INTERPOLATION PREPARATION.

!**   INTERFACE
!     ---------

!    1   CALL PREINT2DAD(KDLEN,KLEN,KSET,KHORIZ
!    2                 PRESH,PRESF,PGEOPH,PGEOPF,
!    3                 PXP,PXPD,PXZ,PXZD,PCP,PRF,PKAP,
!    4                 PUF,PVF,PTF,PQF,PO3F,PCLWF,PCIWF,PCCF,
!    5                 PTSTAR,PT0,
!    6                 PDELP,PRDELP,PLNPR,PALPH,PRTGR,
!    7                 PRPRE,PRPP,
!    1                 PRESH5,PRESF5,
!    2                 PXP5,PXPD5,PXZ5,PXZD5,PCP5,PRF5,PKAP5,
!    3                 PUF5,PVF5,PTF5,PQF5,PO3F5,PCLWF5,PCIWF5,PCCF5,
!    4                 PTSTAR5,PT05,
!    5                 PDELP5,PRDELP5,PLNPR5,PALPH5,PRTGR5,
!    6                 PRPRE5,PRPP5)
!      WHERE KDLEN   = FIRST DIMENSION (OBSERVATIONS)
!            KLEN    = ACTUAL NUMBER OF OBSERVATIONS
!            KSET    = NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!            KHORIZ  = NUMBER of profiles per observation
!            PRESH   =  GRAD. WITH RESPECT TO PRESH5
!            PRESF   =  ..................... PRESF5
!            PGEOPH  =  ..................... PGEOPH5
!            PGEOPF  =  ..................... PGEOPF5
!            PXP     =  ..................... PXP5
!            PXPD    =  ..................... PXPD5
!            PXZ     =  ..................... PXZ5
!            PXZD    =  ..................... PXZD5
!            PCP     =  ..................... PCP5
!            PRF     =  ..................... PRF5
!            PKAP    =  ..................... PKAP5
!            PUF     =  ..................... PUF5
!            PVF     =  ..................... PVF5
!            PTF     =  ..................... PTF5
!            PQF     =  ..................... PQF5
!            PO3F    =  ..................... PO3F5
!            PCLWF   = CLW VALUES (MODEL) AT OB. POINS
!            PCIWF   = CIW VALUES (MODEL) AT OB. POINS
!            PCCF    = CC VALUES (MODEL) AT OB. POINS
!            PTSTAR  =  ..................... PTSTAR5
!            PT0     =  ..................... PT05
!            PDELP   =  ..................... PDELP5
!            PRDELP  =  ..................... PRDELP5
!            PLNPR   =  ..................... PLNPR5
!            PALPH   =  ..................... PLAPH5
!            PRTGR   =  ..................... PRTGR5
!            PRPRE   =  ..................... PRTGR5
!            PRPP    =  ..................... PRPP5

!        PREINTAD IS CALLED FROM HOPAD

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!        CTSTARAD - ADJOINT OF CTSTAR
!        PPINITZA - .......... PPINITZ
!        PPINITAD - .......... PPINIT
!        GPGEOAD  - .......... GPGEO
!        GPRCPAD  - .......... GPRCP
!        GPPREFAD - .......... GPPREF
!        GPXYBAD  - .......... GPXYB
!        GPPREHAD - .......... GPPREH

!**   MODIFICATIONS
!     -------------
!     E.Andersson 92-06-22 : Introduce SATEM
!     E.Andersson 96-02-07 : Introduce SCATT
!     L.Isaksen   96-02-20 : NFLEV --> NFLEVG
!     E.Andersson 96-07-08 : Fujitsu vectorisation
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Andersson 99-07-02 : Incorporate clouds
!     R. El Khatib : 01-08-07 Pruning options
!     S. Healy     : 04-10-27 Switch-off surface pressure increment for GPSRO
!     M.Hamrud    15-Jan-2006  Revised GPRCP
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMSTA   , ONLY : NLEXTRAP
USE YOMGEM   , ONLY : VBH      ,VC       ,VDELB
USE GOMS_MIX     , ONLY : YGOMUA_2D,YGOMS_2D
USE YOMOBSET , ONLY : MTYPOB
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPH(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXP(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXZ(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXZD(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKAP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3F(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLWF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCIWF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSTAR(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT0(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRTGR(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPRE(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXZ5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRE5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPP5(KDLEN,NFLEVG,KHORIZ) 
INTEGER(KIND=JPIM) :: ILEN,IOBSTYPE,IPOSM,JOBS,JLEV,JHOR,JJ
REAL(KIND=JPRB) :: ZPRES(KHORIZ)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ctstarad.intfb.h"
#include "gpgeoad.intfb.h"
#include "gpprefad.intfb.h"
#include "gpprehad.intfb.h"
#include "gprcpad.intfb.h"
#include "gpxybad.intfb.h"
#include "ppinitad.intfb.h"
#include "ppinitza.intfb.h"

#include "openmp_obs.h"
!     ------------------------------------------------------------------
!*
!*
!        1.       ADJOINT OF INTERPOLATION PREPARATION
!                 ------------------------------------

IF (LHOOK) CALL DR_HOOK('PREINT2DAD',0,ZHOOK_HANDLE)
ILEN     = KLEN
IOBSTYPE = MOD(MTYPOB(KSET),100)

DO JHOR = 1,KHORIZ

  CALL CTSTARAD(KDLEN,1,ILEN,&
   & PTF(1,NLEXTRAP,JHOR),PRESH(1,NFLEVG,JHOR),PRESF(1,NLEXTRAP,JHOR),&
   & PTSTAR(:,JHOR),PT0(:,JHOR),&
   & PTF5(1,NLEXTRAP,JHOR),PRESH5(1,NFLEVG,JHOR),&
   & PRESF5(1,NLEXTRAP,JHOR))  

  CALL PPINITZA(KDLEN,1,ILEN,NFLEVG,&
   & PGEOPH(:,:,JHOR),PGEOPF(:,:,JHOR),&
   & PXZ(:,:,:,JHOR),PXZD(:,:,:,JHOR),&
   & PXZ5(:,:,:,JHOR))  

  CALL PPINITAD(KDLEN,1,ILEN,NFLEVG,PRESH(:,:,JHOR),PRESF(:,:,JHOR),&
  &PXP(:,:,:,JHOR),PXPD(:,:,:,JHOR),PRESH5(:,:,JHOR),PRESF5(:,:,JHOR),&
  &PXP5(:,:,:,JHOR))

  CALL GPGEOAD(KDLEN,1,ILEN,NFLEVG,&
   & PGEOPH(:,:,JHOR),PGEOPF(:,:,JHOR),&
   & PTF(:,:,JHOR),PRF(:,:,JHOR),PLNPR(:,:,JHOR),PALPH(:,:,JHOR),&
   & PTF5(1,1,JHOR),PRF5(:,:,JHOR),PLNPR5(:,:,JHOR),PALPH5(:,:,JHOR))  

  DO JLEV=0,NFLEVG
    PGEOPH(1:ILEN,JLEV,JHOR)=0.0_JPRB
  ENDDO

  CALL GPRCPAD (KDLEN,1,ILEN,NFLEVG,PQ=PQF(1,1,JHOR),&
  & PCP=PCP(:,:,JHOR),PR=PRF(:,:,JHOR),&
  & PKAP=PKAP(:,:,JHOR),PCP5=PCP5(:,:,JHOR),PR5=PRF5(:,:,JHOR))

  CALL GPPREFAD(KDLEN,1,ILEN,NFLEVG,&
   & PALPH(:,:,JHOR),PRESH(:,:,JHOR),PRESF(:,:,JHOR),&
   & PALPH5(:,:,JHOR),PRESH5(:,:,JHOR))

  CALL GPXYBAD(KDLEN,1,ILEN,NFLEVG,&
   & VDELB,VC,&
   & PRESH(:,:,JHOR),&
   & PDELP(:,:,JHOR),PRDELP(:,:,JHOR),&
   & PLNPR(:,:,JHOR),PALPH(:,:,JHOR),PRTGR(:,:,JHOR),&
   & PRPRE(:,:,JHOR),PRPP(:,:,JHOR),&
   & PRESH5(:,:,JHOR),PDELP5(:,:,JHOR),PRDELP5(:,:,JHOR),&
   & PLNPR5(:,:,JHOR),PRPRE5(:,:,JHOR),PRPP5(:,:,JHOR))  

  CALL GPPREHAD(KDLEN,1,ILEN,NFLEVG,VBH,PRESH(:,:,JHOR))


ENDDO

!*          6.1   STORE THE GRADIENT IN GOM. TABLES - UPPERAIR

DO JOBS = 1,ILEN
  IPOSM = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  IF(YGOMUA_2D(IPOSM)%LU) THEN
    YGOMUA_2D(IPOSM)%U(:,:) = YGOMUA_2D(IPOSM)%U(:,:)+PUF(JOBS,1:NFLEVG,:)
  ELSE
    DO JJ = 1,KHORIZ
      YGOMS_2D(IPOSM)%UL(JJ) = YGOMS_2D(IPOSM)%UL(JJ)+PUF(JOBS,NFLEVG,JJ)
    ENDDO      
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LV) THEN
    YGOMUA_2D(IPOSM)%V(:,:) = YGOMUA_2D(IPOSM)%V(:,:)+PVF(JOBS,1:NFLEVG,:)
  ELSE  
    DO JJ = 1,KHORIZ  
      YGOMS_2D(IPOSM)%VL(JJ) = YGOMS_2D(IPOSM)%VL(JJ)+PVF(JOBS,NFLEVG,JJ)
    ENDDO 
  ENDIF       
  IF(YGOMUA_2D(IPOSM)%LT) THEN
    YGOMUA_2D(IPOSM)%T(:,:) = YGOMUA_2D(IPOSM)%T(:,:)+PTF(JOBS,1:NFLEVG,:)
  ELSE 
    DO JJ = 1,KHORIZ
      YGOMS_2D(IPOSM)%TL(JJ) = YGOMS_2D(IPOSM)%TL(JJ)+PTF(JOBS,NFLEVG,JJ)
    ENDDO
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LQ) THEN
    YGOMUA_2D(IPOSM)%Q(:,:) = YGOMUA_2D(IPOSM)%Q(:,:)+PQF(JOBS,1:NFLEVG,:)
  ELSE
    DO JJ = 1,KHORIZ
      YGOMS_2D(IPOSM)%QL(JJ) = YGOMS_2D(IPOSM)%QL(JJ)+PQF(JOBS,NFLEVG,JJ)
    ENDDO
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LO3) THEN
    YGOMUA_2D(IPOSM)%O3(:,:) = YGOMUA_2D(IPOSM)%O3(:,:)+PO3F(JOBS,1:NFLEVG,:)
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LCLW) THEN
    YGOMUA_2D(IPOSM)%CLW(:,:) = YGOMUA_2D(IPOSM)%CLW(:,:)+PCLWF(JOBS,1:NFLEVG,:)
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LCIW) THEN
    YGOMUA_2D(IPOSM)%CIW(:,:) = YGOMUA_2D(IPOSM)%CIW(:,:)+PCIWF(JOBS,1:NFLEVG,:)
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LCC) THEN
    YGOMUA_2D(IPOSM)%CC(:,:) = YGOMUA_2D(IPOSM)%CC(:,:)+PCCF(JOBS,1:NFLEVG,:)
  ENDIF

ENDDO

!*          6.1.3 STORE THE GRADIENT WITH RESPECT TO SUR. VARIABLES

!OCL NOVREC
DO JOBS = 1,ILEN
  IPOSM = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

  ZPRES(:)=PRESH5(JOBS,NFLEVG,:)*PRESH(JOBS,NFLEVG,:)
  YGOMS_2D(IPOSM)%SP(:) = YGOMS_2D(IPOSM)%SP(:)+ZPRES(:)
ENDDO
!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('PREINT2DAD',1,ZHOOK_HANDLE)
END SUBROUTINE PREINT2DAD
