SUBROUTINE PREINT2DTL(KDLEN,KLEN,KSET,KHORIZ,&
 & POROG,&
 & PRESH,PRESF,PGEOPH,PGEOPF,&
 & PXP,PXPD,PXZ,PXZD,PCP,PRF,PKAP,&
 & PUF,PVF,PTF,PQF,PO3F,PCLWF,PCIWF,PCCF,&
 & PTSTAR,PT0,&
 & PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PRPRE,PRPP,&
 & PRESH5,PRESF5,&
 & PXP5,PXPD5,PXZ5,PCP5,PRF5,&
 & PTF5,PQF5,PO3F5,&
 & PTSTAR5,PT05,&
 & PDELP5,PRDELP5,PLNPR5,PALPH5,&
 & PRPRE5,PRPP5)  

!****  PREINTTL  - PREPARATION OF VERTICAL INTERPOLATION, TANGENT LINEAR

!     PREINT    D. VASILJEVIC           02/01/91       ECMWF
!     PREINTTL  M. FISHER               10/06/94       ECMWF

!     PURPOSE
!     -------

!        PREINTTL PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS.

!**   INTERFACE
!     ---------

!     SUBROUTINE PREINTTL (KDLEN,..)

!        WHERE KDLEN   = FIRST DIMENSION (OBSERVATIONS)
!              KLEN    = ACTUAL NUMBER OF OBSERVATIONS
!              KSET    = NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!              KHORIZ  = NUMBER OF PROFILES PER OB.
!              POROG   = OROGRAPHY AT OBS. PONTS
!              PRESH   = HALF LEVEL PRESS. AT OBS. POINTS
!              PRESF   = FULL LEVEL PRESS. AT OBS. POINTS
!              PGEOPH  = HALF LEVEL GEOP. AT OBS. POINTS
!              PGEOPF  = FULL LEVEL GEOP. AT OBS. POINTS
!              PXP     = INTERMEDIATE ARRAY (FOR INTERPOLATION IN P)
!              PXPD    = ...........................................
!              PXZ     = INTERMEDIATE ARRAY (FOR INTERPOLATION IN Z)
!              PXZD    = ...........................................
!              PCP     = Cp
!              PRF     = R
!              PKAP    = Kappa
!              PUF     = U VALUES (MODEL) AT OB. POINS
!              PVF     = V VALUES (MODEL) AT OB. POINS
!              PTF     = T VALUES (MODEL) AT OB. POINS
!              PQF     = Q VALUES (MODEL) AT OB. POINS
!              PO3F    = O3 VALUES (MODEL) AT OB. POINS
!              PCLWF   = CLW VALUES (MODEL) AT OB. POINS
!              PCIWF   = CIW VALUES (MODEL) AT OB. POINS
!              PCCF    = CC VALUES (MODEL) AT OB. POINS
!              PTSTAR  = T*
!              PT0     = T0
!              PDELP   = PRESSURE DIFF. ACROSS LAYERS
!              PRDELP  = INVERSE OF THE ABOVE
!              PLNPR   = LOG. OF THE RATIO OF PRESSURE
!              PALPH   = COEFFICIENTS OF THE HYDROSTATICS
!              PRTGR   = FOR PRESS. GRAD. TERM AND ENE. CONV.
!              PRPRE   = INVERSE OF PRESSURE
!              PRPP    = INVERSE OF P(j)*P(j+1)

!        PREINTTL IS CALLED FROM HOPTL

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!        GPPREHTL - COMPUTES HALF LEVEL PRESSURES
!        GPPREFTL - COMPUTES FULL LEVEL PRESSURES
!        PPINITTL - SET-UP AUX. ARRAYS FOR VERTICAL INTERPOLATION
!        GPXYBTL  - COMPUTES AUXILLARY ARRAYS
!        CTSTARTL - COMPUTES STANDARD SURFACE TEMPERATURE AND SURFACE
!                   TEMPERATURE
!        GPRCPTL  -
!        GPGEOTL  -
!        PPINITZTL - SET-UP AUX. ARRAYS FOR VERTICAL INTERPOLATION

!**   MODIFICATIONS
!     -------------
!     E.Andersson 92-06-22 : Introduce SATEM
!     L. Isaksen  96-02-20 : NFLEV --> NFLEVG
!     E.Andersson 96-02-07 : Introduce SCATT
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Andersson 99-07-02 : Incorporate clouds
!     R. El Khatib : 01-08-07 Pruning options
!     S. Healy    04-10-22 : Switch off surface pressure for NRARAD (gps)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMSTA   , ONLY : NLEXTRAP
USE YOMGEM   , ONLY : VBH      ,VC       ,VDELB
USE GOMS_MIX     , ONLY : YGOMUA_2D,YGOMS_2D
USE YOMOBSET , ONLY : MTYPOB
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POROG(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPH(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXP(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXPD(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXZ(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXZD(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKAP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO3F(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLWF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCIWF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCCF(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTAR(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT0(KDLEN,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDELP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRDELP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTGR(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPRE(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPP(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXZ5(KDLEN,0:NFLEVG,NPPM,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRF5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3F5(KDLEN,0:NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KDLEN,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KDLEN,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRE5(KDLEN,NFLEVG,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPP5(KDLEN,NFLEVG,KHORIZ) 
INTEGER(KIND=JPIM) :: IOBSTYPE,IPOSM,ISET,ILEN,JOBS,JLEV,JHOR,JJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ctstartl.intfb.h"
#include "gpgeotl.intfb.h"
#include "gppreftl.intfb.h"
#include "gpprehtl.intfb.h"
#include "gprcptl.intfb.h"
#include "gpxybtl.intfb.h"
#include "ppinittl.intfb.h"
#include "ppinitztl.intfb.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------
!*       1.       PREPARATION OF INTERPOLATION
!                 ----------------------------

IF (LHOOK) CALL DR_HOOK('PREINT2DTL',0,ZHOOK_HANDLE)
ISET=ABS(KSET)
ILEN     = KLEN
IOBSTYPE = MOD(MTYPOB(ISET),100)

!   KSET < 0 INDICATES THAT MODEL DATA HAVE ALREADY BEEN FILLED
IF(KSET < 0) GOTO 120

!*          1.1   FIND OUT RELEVANT ARRAYS FOR EACH OBSERVATION

DO JOBS = 1,ILEN
  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

!*          1.1.2 SURFACE PRESSURE

  PRESH(JOBS,NFLEVG,:) = YGOMS_2D(IPOSM)%SP(:)
  PRESH(JOBS,NFLEVG,:)=PRESH5(JOBS,NFLEVG,:)*PRESH(JOBS,NFLEVG,:)

ENDDO

DO JOBS = 1,ILEN
  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

!*          1.1.3 FILL OUT THE AD HOC ARRAYS WITH THE MODEL VALUES FOR
!                 U, V, T, Q AND OZON
  IF(YGOMUA_2D(IPOSM)%LU) THEN
    PUF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%U(:,:)
  ELSE
    DO JJ=1,KHORIZ
      PUF(JOBS,:,JJ) = YGOMS_2D(IPOSM)%UL(JJ)
    ENDDO
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LV) THEN
    PVF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%V(:,:)
  ELSE
    DO JJ = 1,KHORIZ 
      PVF(JOBS,:,JJ) = YGOMS_2D(IPOSM)%VL(JJ)
    ENDDO  
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LT) THEN
    PTF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%T(:,:)
  ELSE
    DO JJ = 1,KHORIZ
      PTF(JOBS,:,JJ) = YGOMS_2D(IPOSM)%TL(JJ)
    ENDDO  
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LQ) THEN
    PQF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%Q(:,:)
  ELSE
    DO JJ =1,KHORIZ 
      PQF(JOBS,:,JJ) = YGOMS_2D(IPOSM)%QL(JJ)
    ENDDO
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LO3) THEN
    PO3F(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%O3(:,:)
  ELSE
    PO3F(JOBS,1:NFLEVG,:) = 0.0_JPRB
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LCLW) THEN
    PCLWF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%CLW(:,:)
  ELSE
    PCLWF(JOBS,1:NFLEVG,:) = 0.0_JPRB
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LCIW) THEN
    PCIWF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%CIW(:,:)
  ELSE
    PCIWF(JOBS,1:NFLEVG,:) = 0.0_JPRB
  ENDIF
  IF(YGOMUA_2D(IPOSM)%LCC) THEN
    PCCF(JOBS,1:NFLEVG,:) = YGOMUA_2D(IPOSM)%CC(:,:)
  ELSE
    PCCF(JOBS,1:NFLEVG,:) = 0.0_JPRB
  ENDIF

ENDDO

!*          1.2   COMPUTATION OF HALF LEVEL PRESSURES

120 CONTINUE

DO JHOR = 1,KHORIZ

  CALL GPPREHTL(KDLEN,1,ILEN,NFLEVG,VBH,PRESH(:,:,JHOR))

!*          1.3   SOME OTHER PREPARATIONS FOR INTERPOLATION

  CALL GPXYBTL (KDLEN,1,ILEN,NFLEVG,&
   & VDELB,VC,&
   & PRESH(:,:,JHOR),&
   & PDELP(:,:,JHOR),PRDELP(:,:,JHOR),PLNPR(:,:,JHOR),PALPH(:,:,JHOR),PRTGR(:,:,JHOR),&
   & PRPRE(:,:,JHOR),PRPP(:,:,JHOR),&
   & PRESH5(:,:,JHOR),PDELP5(:,:,JHOR),PRDELP5(:,:,JHOR),&
   & PLNPR5(:,:,JHOR),PRPRE5(:,:,JHOR),PRPP5(:,:,JHOR))  

!*          1.4   COMPUTATION OF HALF AND FULL LEVEL PRESSURES

  CALL GPPREFTL (KDLEN,1,ILEN,NFLEVG,&
   & PALPH(:,:,JHOR),PRESH(:,:,JHOR),PRESF(:,:,JHOR),&
   & PALPH5(:,:,JHOR),PRESH5(:,:,JHOR))

!*          1.5   COMPUTION OF THE GEOPOTENTIALS ON THE MODEL LEVELS

!*          1.5.1  PRELIMINARY COMPUTATION

  CALL GPRCPTL (KDLEN,1,ILEN,NFLEVG,PQ=PQF(1,1,JHOR),&
  & PCP=PCP(:,:,JHOR),PR=PRF(:,:,JHOR),&
  & PKAP=PKAP(:,:,JHOR),PCP5=PCP5(:,:,JHOR),PR5=PRF5(:,:,JHOR))

!*          1.5.2  GEOP. CALCULATION

  DO JLEV=0,NFLEVG
    PGEOPH(1:ILEN,JLEV,JHOR)=0.0_JPRB
  ENDDO

  CALL GPGEOTL (KDLEN,1,ILEN,NFLEVG,&
   & PGEOPH(:,:,JHOR),PGEOPF(:,:,JHOR),&
   & PTF(1,1,JHOR),PRF(:,:,JHOR),PLNPR(:,:,JHOR),PALPH(:,:,JHOR),&
   & PTF5(1,1,JHOR),PRF5(:,:,JHOR),PLNPR5(:,:,JHOR),PALPH5(:,:,JHOR))  

!*          1.6   COMPUTION OF AUX. ARRAYS FOR VERTICAL INTERPOLATIONS

!*          1.6.1 VERTICAL INTERPOLATIONS IN P

  CALL PPINITTL (KDLEN,1,ILEN,NFLEVG,PRESH(:,:,JHOR),PRESF(:,:,JHOR),&
  &PXP(:,:,:,JHOR),PXPD(:,:,:,JHOR),PRESH5(:,:,JHOR),PRESF5(:,:,JHOR),PXP5(:,:,:,JHOR))

!*          1.6.2 VERTICAL INTERPOLATIONS IN Z

  CALL PPINITZTL (KDLEN,1,ILEN,NFLEVG,&
   & PGEOPH(:,:,JHOR),PGEOPF(:,:,JHOR),&
   & PXZ(:,:,:,JHOR),PXZD(:,:,:,JHOR),&
   & PXZ5(:,:,:,JHOR))  

!*          1.7   COMPUTE T* AND T0

  CALL CTSTARTL (KDLEN,1,ILEN,&
   & PTF(1,NLEXTRAP,JHOR),PRESH(1,NFLEVG,JHOR),PRESF(1,NLEXTRAP,JHOR),&
   & PTSTAR(:,JHOR),PT0(:,JHOR),&
   & PTF5(1,NLEXTRAP,JHOR),PRESH5(1,NFLEVG,JHOR),&
   & PRESF5(1,NLEXTRAP,JHOR))  

ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PREINT2DTL',1,ZHOOK_HANDLE)
END SUBROUTINE PREINT2DTL
