SUBROUTINE PREINTAD(KDLEN,KLEN,KSET,&
 & PRESH,PRESF,PGEOPH,PGEOPF,&
 & PXP,PXPD,PXZ,PXZD,PCP,PRF,PKAP,&
 & PUF,PVF,PTF,PQF,PO3F,PCLWF,PCIWF,PCCF,&
 & PTSTAR,PT0,&
 & PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PRPRE,PRPP,&
 & PRESH5,PRESF5,&
 & PXP5,PXZ5,PCP5,PRF5,PTF5,&
 & PDELP5,PRDELP5,PLNPR5,PALPH5,&
 & PRPRE5,PRPP5,YDGEMSPROF)  

!****  PREINTAD  - ADJOINT OF PREPARATION OF THE VERTICAL INTERPOLATION

!     D. VASILJEVIC           02/01/91       ECMWF

!     PURPOSE
!     -------

!        ADJOINT CALCULATION OF INTERPOLATION PREPARATION.

!**   INTERFACE
!     ---------

!    1   CALL PREINTAD(KDLEN,KLEN,KSET,
!    2                 PRESH,PRESF,PGEOPH,PGEOPF,
!    3                 PXP,PXPD,PXZ,PXZD,PCP,PRF,PKAP,
!    4                 PUF,PVF,PTF,PQF,PO3F,PCLWF,PCIWF,PCCF,
!    5                 PTSTAR,PT0,
!    6                 PDELP,PRDELP,PLNPR,PALPH,PRTGR,
!    7                 PRPRE,PRPP,
!    1                 PRESH5,PRESF5,
!    2                 PXP5,PXZ5,PXZD5,PCP5,PRF5,PTF5,
!    3                 PDELP5,PRDELP5,PLNPR5,PALPH5,PRTGR5,
!    4                 PRPRE5,PRPP5,YDGEMSPROF)
!      WHERE KDLEN   = FIRST DIMENSION (OBSERVATIONS)
!            KLEN    = ACTUAL NUMBER OF OBSERVATIONS
!            KSET    = NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!            PRESH   =  GRAD. WITH RESPECT TO PRESH5
!            PRESF   =  ..................... PRESF5
!            PGEOPH  =  ..................... PGEOPH5
!            PGEOPF  =  ..................... PGEOPF5
!            PXP     =  ..................... PXP5
!            PXPD    =  ..................... PXPD5
!            PXZ     =  ..................... PXZ5
!            PXZD    =  ..................... PXZD5
!            PCP     =  ..................... PCP5
!            PRF     =  ..................... PRF5
!            PKAP    =  ..................... PKAP5
!            PUF     =  ..................... PUF5
!            PVF     =  ..................... PVF5
!            PTF     =  ..................... PTF5
!            PQF     =  ..................... PQF5
!            PO3F    =  ..................... PO3F5
!            PCLWF   = CLW VALUES (MODEL) AT OB. POINS
!            PCIWF   = CIW VALUES (MODEL) AT OB. POINS
!            PCCF    = CC VALUES (MODEL) AT OB. POINS
!            PTSTAR  =  ..................... PTSTAR5
!            PT0     =  ..................... PT05
!            PDELP   =  ..................... PDELP5
!            PRDELP  =  ..................... PRDELP5
!            PLNPR   =  ..................... PLNPR5
!            PALPH   =  ..................... PLAPH5
!            PRTGR   =  ..................... PRTGR5
!            PRPRE   =  ..................... PRTGR5
!            PRPP    =  ..................... PRPP5
!            YDGEMSPROF    =   GEMS PROFILES (OPTIONAL)

!        PREINTAD IS CALLED FROM HOPAD

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!        CTSTARAD - ADJOINT OF CTSTAR
!        PPINITZA - .......... PPINITZ
!        PPINITAD - .......... PPINIT
!        GPGEOAD  - .......... GPGEO
!        GPRCPAD  - .......... GPRCP
!        GPPREFAD - .......... GPPREF
!        GPXYBAD  - .......... GPXYB
!        GPPREHAD - .......... GPPREH

!**   MODIFICATIONS
!     -------------
!     E.Andersson 92-06-22 : Introduce SATEM
!     E.Andersson 96-02-07 : Introduce SCATT
!     L.Isaksen   96-02-20 : NFLEV --> NFLEVG
!     E.Andersson 96-07-08 : Fujitsu vectorisation
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Andersson 99-07-02 : Incorporate clouds
!     R. El Khatib : 01-08-07 Pruning options
!     S. Healy     : 04-10-27 Switch-off surface pressure increment for RARAD
!     N. Bormann   : 04-12-23 RARAD becomes LIMB
!     M.Hamrud    15-Jan-2006  Revised GPRCP
!     NEC         11-Dec-2007 Optimizations
!     H.Hersbach  09-02-20 : Bugfix assignment surface YGOMS%*L arrays
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMSTA   , ONLY : NLEXTRAP
USE YOMGEM   , ONLY : VBH      ,VC       ,VDELB
USE GOMS_MIX     , ONLY : YGOMUA,YGOMS
USE YOMOBSET , ONLY : MTYPOB
USE YOMDB
USE YOM_YGFL , ONLY : NGHG, NGRG, NAERO 
USE TYPE_GEMS_PROFILES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOPF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXP(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXZ(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXZD(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKAP(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3F(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLWF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCIWF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSTAR(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT0(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRTGR(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPRE(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPP(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXZ5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRE5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPP5(KDLEN,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ILEN,IOBSTYPE,IPOSM,JOBS,JGHG,JGRG,JAERO
INTEGER(KIND=JPIM) :: IGEMS,ISTGHG,IENGHG,ISTGRG,IENGRG,ISTAERO,IENAERO
INTEGER(KIND=JPIM) :: IGHG,IGRG,IAERO
REAL(KIND=JPRB) :: ZPRES
REAL(KIND=JPRB) :: ZHOOK_HANDLE

TYPE (TYPE_GEMSPROF), INTENT(IN), OPTIONAL   :: YDGEMSPROF(KDLEN,NGEMS)

!     ------------------------------------------------------------------

#include "ctstarad.intfb.h"
#include "gpgeoad.intfb.h"
#include "gpprefad.intfb.h"
#include "gpprehad.intfb.h"
#include "gprcpad.intfb.h"
#include "gpxybad.intfb.h"
#include "ppinitad.intfb.h"
#include "ppinitza.intfb.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PREINTAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*
!*
!        1.       ADJOINT OF INTERPOLATION PREPARATION
!                 ------------------------------------

ILEN     = KLEN
IOBSTYPE = MOD(MTYPOB(KSET),100)
CALL CTSTARAD(KDLEN,1,ILEN,&
 & PTF(1,NLEXTRAP),PRESH(1,NFLEVG),PRESF(1,NLEXTRAP),&
 & PTSTAR,PT0,&
 & PTF5(1,NLEXTRAP),PRESH5(1,NFLEVG),&
 & PRESF5(1,NLEXTRAP))  

CALL PPINITZA(KDLEN,1,ILEN,NFLEVG,&
 & PGEOPH,PGEOPF,&
 & PXZ,PXZD,&
 & PXZ5)  

CALL PPINITAD(KDLEN,1,ILEN,NFLEVG,PRESH,PRESF,PXP,PXPD,PRESH5,PRESF5,PXP5)

CALL GPGEOAD(KDLEN,1,ILEN,NFLEVG,&
 & PGEOPH,PGEOPF,&
 & PTF,PRF,PLNPR,PALPH,&
 & PTF5(1,1),PRF5,PLNPR5,PALPH5)  

CALL GPRCPAD (KDLEN,1,ILEN,NFLEVG,PQ=PQF,&
             &PCP=PCP,PR=PRF,PKAP=PKAP,PCP5=PCP5,PR5=PRF5)

CALL GPPREFAD(KDLEN,1,ILEN,NFLEVG,&
 & PALPH,PRESH,PRESF,&
 & PALPH5,PRESH5)

CALL GPXYBAD(KDLEN,1,ILEN,NFLEVG,&
 & VDELB,VC,&
 & PRESH,&
 & PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PRPRE,PRPP,&
 & PRESH5,PDELP5,PRDELP5,PLNPR5,PRPRE5,PRPP5)  

CALL GPPREHAD(KDLEN,1,ILEN,NFLEVG,VBH,PRESH)

!*          6.1   STORE THE GRADIENT IN GOM. TABLES - UPPERAIR

!CDIR NOVECTOR
DO JOBS = 1,ILEN
  IPOSM = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  IF(YGOMUA(IPOSM)%LU) THEN
    YGOMUA(IPOSM)%U(1:NFLEVG) = YGOMUA(IPOSM)%U(1:NFLEVG)+PUF(JOBS,1:NFLEVG)
  ELSE
    YGOMS(IPOSM)%UL = YGOMS(IPOSM)%UL+SUM(PUF(JOBS,1:NFLEVG))
  ENDIF
  IF(YGOMUA(IPOSM)%LV) THEN
    YGOMUA(IPOSM)%V(1:NFLEVG) = YGOMUA(IPOSM)%V(1:NFLEVG)+PVF(JOBS,1:NFLEVG)
  ELSE
    YGOMS(IPOSM)%VL = YGOMS(IPOSM)%VL+SUM(PVF(JOBS,1:NFLEVG))
  ENDIF
  IF(YGOMUA(IPOSM)%LT) THEN
    YGOMUA(IPOSM)%T(1:NFLEVG) = YGOMUA(IPOSM)%T(1:NFLEVG)+PTF(JOBS,1:NFLEVG)
  ELSE
    YGOMS(IPOSM)%TL = YGOMS(IPOSM)%TL+SUM(PTF(JOBS,1:NFLEVG))
  ENDIF
  IF(YGOMUA(IPOSM)%LQ) THEN
    YGOMUA(IPOSM)%Q(1:NFLEVG) = YGOMUA(IPOSM)%Q(1:NFLEVG)+PQF(JOBS,1:NFLEVG)
  ELSE
    YGOMS(IPOSM)%QL = YGOMS(IPOSM)%QL+SUM(PQF(JOBS,1:NFLEVG))
  ENDIF
  IF(YGOMUA(IPOSM)%LO3) THEN
    YGOMUA(IPOSM)%O3(1:NFLEVG) = YGOMUA(IPOSM)%O3(1:NFLEVG)+PO3F(JOBS,1:NFLEVG)
  ENDIF
  IF(YGOMUA(IPOSM)%LCLW) THEN
    YGOMUA(IPOSM)%CLW(1:NFLEVG) = YGOMUA(IPOSM)%CLW(1:NFLEVG)+PCLWF(JOBS,1:NFLEVG)
  ENDIF
  IF(YGOMUA(IPOSM)%LCIW) THEN
    YGOMUA(IPOSM)%CIW(1:NFLEVG) = YGOMUA(IPOSM)%CIW(1:NFLEVG)+PCIWF(JOBS,1:NFLEVG)
  ENDIF
  IF(YGOMUA(IPOSM)%LCC) THEN
    YGOMUA(IPOSM)%CC(1:NFLEVG) = YGOMUA(IPOSM)%CC(1:NFLEVG)+PCCF(JOBS,1:NFLEVG)
  ENDIF
!rje
  IF(PRESENT(YDGEMSPROF)) THEN
    IGEMS=0
    IGHG=0
    DO JGHG=1,NGHG
      IF(YGOMUA(IPOSM)%LGHG(JGHG)) THEN
        IGEMS=IGEMS+1
        ISTGHG = 1+IGHG*NFLEVG
        IENGHG = ISTGHG+NFLEVG-1
        YGOMUA(IPOSM)%GHG(ISTGHG:IENGHG)=YGOMUA(IPOSM)%GHG(ISTGHG:IENGHG)+YDGEMSPROF(JOBS,IGEMS)%MODELPROF(1:NFLEVG)
        IGHG=IGHG+1
      ENDIF
    ENDDO
    IGRG=0
    DO JGRG=1,NGRG
      IF(YGOMUA(IPOSM)%LGRG(JGRG)) THEN
        IGEMS=IGEMS+1
        ISTGRG = 1+IGRG*NFLEVG
        IENGRG = ISTGRG+NFLEVG-1
        YGOMUA(IPOSM)%GRG(ISTGRG:IENGRG)=YGOMUA(IPOSM)%GRG(ISTGRG:IENGRG)+YDGEMSPROF(JOBS,IGEMS)%MODELPROF(1:NFLEVG)
        IGRG=IGRG+1
      ENDIF
    ENDDO
    IAERO=0
    DO JAERO=1,NAERO
      IF(YGOMUA(IPOSM)%LAERO(JAERO)) THEN
        IGEMS=IGEMS+1
        ISTAERO = 1+IAERO*NFLEVG
        IENAERO = ISTAERO+NFLEVG-1
        YGOMUA(IPOSM)%AERO(ISTAERO:IENAERO)=YGOMUA(IPOSM)%AERO(ISTAERO:IENAERO)+YDGEMSPROF(JOBS,IGEMS)%MODELPROF(1:NFLEVG)
        IAERO=IAERO+1
      ENDIF
    ENDDO
  ENDIF

ENDDO

!*          6.1.3 STORE THE GRADIENT WITH RESPECT TO SUR. VARIABLES

!OCL NOVREC
DO JOBS = 1,ILEN
  IPOSM = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

  ZPRES=PRESH5(JOBS,NFLEVG)*PRESH(JOBS,NFLEVG)
  YGOMS(IPOSM)%SP = YGOMS(IPOSM)%SP+ZPRES
ENDDO
!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PREINTAD',1,ZHOOK_HANDLE)
END SUBROUTINE PREINTAD
