SUBROUTINE PREINTSAD(KDLEN,KLEN,KSET,&
 & PRESH,&
 & PUF,PVF,PTF,PQF,PQN,PTS,&
 & PU10N,PV10N,&
 & PRF,&
 & PLNPR,PALPH,&
 & PCPTGZ,PCPTS,PBN,PBH,PBM,&
 & PAPHI,PZ0,PWS,PVEG,PWL,PSN,&
 & PRESH5,&
 & PUF5,PVF5,PTF5,PQF5,PQN5,&
 & PES5,PQS5,&
 & PZ05,PWS5,PVEG5,PWL5,PSN5,PSLM5,&
 & PTS5,PU05,PV05,PU10N5,PV10N5,PCHN5,&
 & PLNPR5,PALPH5,&
 & PAPHI5,PWET5,PRF5,PRI5)  

!****  PREINTSAD  - ADJOINT OF PREINTS

!     CARLA CARDINALI            18/03/92       ECMWF
!     P. Viterbo  04-05-28 : surf library and new surface units.

!     PURPOSE
!     -------

!        ADJOINT CALCULATION OF INTERPOLATION PREPARATION.

!**   INTERFACE
!     ---------

!        CALL PREINTSAD(KDLEN,KLEN,KSET,
!    1                  PRESH,
!    2                  PUF,PVF,PTF,PQF,PQN,PTS
!    3                  PU10N,PV10N,
!    4                  PRF,
!    5                  PLNPR,PALPH,
!    6                  PCPTGZ,PCPTS,
!    7                  PAPHI,PZ0,PWS,PVEG,PWL,PSN,
!    8                  PRESH5,
!    9                  PUF5,PVF5,PTF5,PQF5,PQN5,
!    1                  PQS5,
!    2                  PZ05,PWS5,PVEG5,PWL5,PSN5,PSLM5,
!    3                  PTS5,PU05,PV05,PCHN5,
!    4                  PLNPR5,PALPH5,
!    5                  PAPHI5,PWET5,PRF5,PRI5)
!      WHERE KDLEN   =  FIRST DIMENSION (OBSERVATIONS)
!            KLEN    =  ACTUAL NUMBER OF OBSERVATIONS
!            KSET    =  NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!            PRESH   =  GRAD. WITH RESPECT TO PRESH5
!            PUF     =  ..................... PUF5
!            PVF     =  ..................... PVF5
!            PTF     =  ..................... PTF5
!            PQF     =  ..................... PQF5
!            PQN     =  ..................... PQN5
!            PTS     =  ..................... PTS5
!            PRF     =  ..................... PRF5
!            PLNPR   =  ..................... PLNPR5
!            PALPH   =  ..................... PLAPH5
!            PCPTGZ  =  ..................... PCPTGZ5
!            PCPTS   =  ..................... PCPTS5
!            PAPHI   =  ..................... PAPHI5
!            PZ0     =  ..................... PZ05
!            PWS     =  ..................... PWS5
!            PVEG    =  ..................... PVEG5
!            PWL     =  ..................... PWL5
!            PSN     =  ..................... PSN5
!            PRESH5  = HALF LEVEL PRESS. AT OBS. POINTS
!            PUF5    = U VALUES (MODEL) AT OB. POINS
!            PVF5    = V VALUES (MODEL) AT OB. POINS
!            PTF5    = T VALUES (MODEL) AT OB. POINS
!            PQF5    = Q VALUES (MODEL) AT OB. POINS
!            PQN5    = Q WITHOUT NEGATIVE VALUES (MODEL) AT OB. POINS
!            PTS5    = SURFACE TEMPERATURE
!            PU05    = U-COMP OF OCEAN CURRENT
!            PV05    = V-COMP OF OCEAN CURRENT
!            PU10N5  = U-COMP OF 10M NEUTRAL WIND
!            PV10N5  = V-COMP OF 10M NEUTRAL WIND
!            PV05    = WAVE CHARNOCK
!            PRF5    = R
!            PWSF5   = SOIL WETNESS
!            PVEGF5  = VEGETATION
!            PWLF5   = SKIN RESERVOIR
!            PSNF5   = SNOW DEPTH
!            PSLMF5  = SEE/LAND MASK
!            PZ05    = ROUGHNESS LENGTH
!            PCPTGZ5 = STATIC ENERGY (LOWER LEVEL)
!            PWET5   = WETNESS
!            PQS5    = SATURATION HUMIDITY
!            PCPTS5  = STATIC ENERGY AT THE SURFACE
!            PRI5    = RICHARDSON NO.
!            PLNPR5  = LOG. OF THE RATIO OF PRESSURE
!            PALPH5  = COEFFICIENTS OF THE HYDROSTATICS

!        PREINTSAD IS CALLED FROM SURFACEO.

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------
!        EXCHCOAD    - ADJOINT OF EXCHCO
!        SURBOUNDAD  - ADJOINT OF SURBOUND
!        GPGEOAD     - ADJOINT OF GPGEO
!        GPRCPAD     - ADJOINT OF GPRCP

!**   MODIFICATIONS
!     -------------
!     E.Andersson 96-02-01 : Incorporating Scat in SURFACEO
!     L. Isaksen  96-02-20 : NFLEV --> NFLEVG
!     M. Fisher   96-05-16 : Fill from GSTS5, etc. if in tangent linear
!     P. MOLL     96-07-18 : ACHMT transplantation
!                            (modification of surbound and exchco)
!     E.Andersson 97-11-17 : Flexible GOM arrays. Model Z0 for SCAT.
!     E.Andersson 97-12-17 : Use model's Z0.
!     E. Gerard   99-05-05 : Interpolation of SATEM type obs.
!     P. Moll     99-08-02 : ACHMT final transplantation
!     H.Hersbach  08-07-04 : Add ocean current and wave Charnock
!     H. Hersbach 09-12-04 : EXCHCO_VDFAD; neutral wind
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMOBSET , ONLY : MTYPOB
USE YOMOBS   , ONLY : LCACHMT, LVDFTRAJ, LVDFMIN
USE YOMCOCTP , ONLY : NSCATT
USE YOMCT0B  , ONLY : LECMWF
USE YOMCST   , ONLY : RG
USE GOMS_MIX , ONLY : YGOMS,YGOMS5
USE YOEPHY   , ONLY : LECURR
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQN(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPTGZ(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPTS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBN(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBH(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBM(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVEG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWL(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQN5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PES5(KDLEN,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ05(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWL5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSN5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU05(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV05(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCHN5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWET5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KDLEN) 
LOGICAL         :: LLVDF5                       ! Get exchange coefs from IFS VDF model
LOGICAL         :: LLUN5(KDLEN)                 ! Allow perturbations in UN10
REAL(KIND=JPRB) ::ZAPHIH5(KDLEN,0:NFLEVG),ZUF5(KDLEN),ZVF5(KDLEN),ZH105(KDLEN)
REAL(KIND=JPRB) ::ZAPHIH (KDLEN,0:NFLEVG),ZUF (KDLEN),ZVF (KDLEN),ZH10 (KDLEN)
REAL(KIND=JPRB) ::ZQS(KDLEN),ZWET(KDLEN)

INTEGER(KIND=JPIM) :: IOBSTYPE,IPOSM,ILEN,JOBS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "exchcoad.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgeoad.intfb.h"
#include "qnegad.intfb.h"
#include "rouseaad.intfb.h"
#include "z0seaad.intfb.h"
#include "exchco_vdfad.intfb.h"
#include "surboundad.intfb.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------

!*        1. ARRAYS INITIALIZATION

IF (LHOOK) CALL DR_HOOK('PREINTSAD',0,ZHOOK_HANDLE)

ILEN     = KLEN
IOBSTYPE = MOD(MTYPOB(KSET),100)
! Currently, limited usage to scatt data
LLVDF5=(LVDFTRAJ.AND.(IOBSTYPE == NSCATT))


IF( .NOT.LECMWF .AND. LCACHMT ) GOTO 3

DO JOBS = 1,ILEN
  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  LLUN5(JOBS)=LVDFMIN.AND.YGOMS5(IPOSM)%LUPD
ENDDO

ZQS (1:ILEN)=0.0_JPRB
ZWET(1:ILEN)=0.0_JPRB
ZUF (1:ILEN)=0.0_JPRB
ZVF (1:ILEN)=0.0_JPRB
ZH10(1:ILEN)=0.0_JPRB
ZAPHIH5(1:ILEN,NFLEVG)=0.0_JPRB
ZAPHIH(1:ILEN,0:NFLEVG)=0.0_JPRB

!*        1.1 CORRECT FOR OCEAN CURRENT

 IF (LECURR) THEN
    ZUF5(1:ILEN)=PUF5(1:ILEN,NFLEVG)-PU05(1:ILEN)
    ZVF5(1:ILEN)=PVF5(1:ILEN,NFLEVG)-PV05(1:ILEN)
 ELSE
    ZUF5(1:ILEN)=PUF5(1:ILEN,NFLEVG)
    ZVF5(1:ILEN)=PVF5(1:ILEN,NFLEVG)
 ENDIF

CALL GPGEO(KDLEN,1,ILEN,NFLEVG,ZAPHIH5,PAPHI5,PTF5(1,1),PRF5,PLNPR5,PALPH5)

!*         2. ADJOINT OF THE TANGENT LINEAR OF
!             THE THERMODINAMIC VARIABLES AND DRAG COEFFICIENTS
IF(LLVDF5) THEN
  CALL EXCHCO_VDFAD(KDLEN,1,ILEN,LLUN5,&
  & ZUF5,ZVF5,PU10N5,PV10N5,PAPHI5(1,NFLEVG),PZ05,&
  & ZUF ,ZVF ,PU10N ,PV10N ,PAPHI (1,NFLEVG),PZ0 ,&
  & PBN ,PBM)
ELSE
 CALL EXCHCOAD(KDLEN,1,ILEN,&
 & PQN5(1,NFLEVG),PTF5(1,NFLEVG),&
 & PAPHI5(1,NFLEVG),ZAPHIH5(1,NFLEVG),PRF5(1,NFLEVG),&
 & PTS5,PQN(1,NFLEVG),PTF(1,NFLEVG),&
 & PAPHI(1,NFLEVG),ZAPHIH(1,NFLEVG),PRF(1,NFLEVG),PTS,&
 & ZUF5,ZVF5,PZ05,&
 & PUF(1,NFLEVG),PVF(1,NFLEVG),PZ0,&
 & PWET5,PQS5,&
 & ZWET,ZQS,&
 & PRI5,&
 & PBN,PBM,PBH)  
ENDIF

CALL SURBOUNDAD(KDLEN,1,ILEN,&
 & PQN5(1,NFLEVG),PTF5(1,NFLEVG),PTS5,PRESH5(1,NFLEVG),&
 & PQN(1,NFLEVG),PTF(1,NFLEVG),PTS,PRESH(1,NFLEVG),&
 & PAPHI(1,NFLEVG),&
 & PWS5,PVEG5,PWL5,PSN5,PSLM5,&
 & PWS,PVEG,PWL,PSN,&
 & PES5(1,1),PWET5,PQS5,&
 & ZWET,ZQS,&
 & PCPTGZ,PCPTS)  

CALL QNEGAD(KDLEN,1,ILEN,PQF5(1,NFLEVG),PQF(1,NFLEVG),PQN(1,NFLEVG))

!*          2.3.2  ADJOINT OF THE ROUGHNESS LENGTH OVER SEA

IF(.NOT.LECMWF) THEN
  CALL ROUSEAAD(KDLEN,1,ILEN,&
   & PAPHI5(1,NFLEVG),PUF5(1,NFLEVG),PVF5(1,NFLEVG),PSLM5,&
   & PAPHI(1,NFLEVG),PUF(1,NFLEVG),PVF(1,NFLEVG),&
   & PZ0)  
ENDIF
IF (LLVDF5) THEN
  ZH105(1:ILEN)=10.0_JPRB*RG
  CALL Z0SEAAD(KDLEN,1,ILEN,&
   & ZH105,PU10N5,PV10N5,PSLM5,PCHN5,&
   & ZH10 ,PU10N ,PV10N ,&
   & PZ0)  
  ZH10 (1:ILEN)= 0.0_JPRB
ENDIF

!*        2.3.1 CORRECT FOR OCEAN CURRENT
IF (LLVDF5) THEN
 PUF(1:ILEN,NFLEVG)=PUF(1:ILEN,NFLEVG)+ZUF(1:ILEN)
 PVF(1:ILEN,NFLEVG)=PVF(1:ILEN,NFLEVG)+ZVF(1:ILEN)
ENDIF


!*          2.3  ADJOINT OF OF GEOPOTENTIAL ON MODEL LEVELS

CALL GPGEOAD(KDLEN,1,ILEN,NFLEVG,ZAPHIH,PAPHI,PTF,PRF,&
 & PLNPR,PALPH,PTF5(1,1),PRF5,PLNPR5,PALPH5)  

!     ------------------------------------------------------------------

!*       3.       ADD THE GRADIENT TO GOM-ARRAYS
!                 ------------------------------

3 CONTINUE
!OCL NOVREC
DO  JOBS = 1,ILEN
  IPOSM = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
!  ECVEGE(IPOSM)=PVEG(JOBS)

  IF(LECMWF)THEN
    PZ0(JOBS)=PZ0(JOBS)/RG
  ENDIF

  YGOMS(IPOSM)%TS = YGOMS(IPOSM)%TS+PTS(JOBS)
  YGOMS(IPOSM)%Z0 = YGOMS(IPOSM)%Z0+PZ0(JOBS)
  YGOMS(IPOSM)%U10N=YGOMS(IPOSM)%U10N+PU10N(JOBS)
  YGOMS(IPOSM)%V10N=YGOMS(IPOSM)%V10N+PV10N(JOBS)
  IF(IOBSTYPE /= NSCATT) THEN
    YGOMS(IPOSM)%WS = YGOMS(IPOSM)%WS+PWS(JOBS)
    YGOMS(IPOSM)%WL = YGOMS(IPOSM)%WL+PWL (JOBS)
    YGOMS(IPOSM)%SN = YGOMS(IPOSM)%SN+PSN (JOBS)
  ENDIF
  PTS(JOBS)=0.0_JPRB
  PWS(JOBS)=0.0_JPRB
  PWL(JOBS)=0.0_JPRB
  PSN(JOBS)=0.0_JPRB
  PZ0(JOBS)=0.0_JPRB
  PU10N(JOBS)=0.0_JPRB
  PV10N(JOBS)=0.0_JPRB
  PVEG(JOBS)=0.0_JPRB

ENDDO
!*
!     ------------------------------------------------------------------

!        4.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('PREINTSAD',1,ZHOOK_HANDLE)
END SUBROUTINE PREINTSAD
