SUBROUTINE PREINTSTL(KDLEN,KLEN,KSET,&
 & PRESH,&
 & PUF,PVF,PTF,PQF,PQN,PTS,&
 & PU10N,PV10N,PRF,&
 & PWS,PVEG,PCI,PWL,&
 & PSN,PSLM,PZ0,&
 & PCPTGZ,PWET,PQS,PCPTS,PAPHI,&
 & PBN,PBM,PBH,&
 & PLNPR,PALPH,&
 & PRESH5,&
 & PUF5,PVF5,PTF5,PQF5,PQN5,PTS5,&
 & PU05,PV05,PU10N5,PV10N5,PCHN5,PRF5,&
 & PWS5,PVEG5,PWL5,&
 & PSN5,PSLM5,PZ05,&
 & PES5,PWET5,PQS5,PAPHI5,&
 & PRI5,PLNPR5,PALPH5)  

!****  PREINTS  - PREPARATION FOR THE SURFACE INTERPOLATION

!     CARLA CARDINALI         18/03/92       ECMWF
!     MODIFICATIONS
!     TANGENT LINEAR VERSION - MIKE FISHER *ECMWF* 13/6/94
!     P. Viterbo  04-05-28 : surf library and new surface units.

!     PURPOSE
!     -------

!        PREINT PREPARES INTERPOLATION FOR ONE SET OF
!        SYNOP OBSERVATIONS.

!**   INTERFACE
!     ---------

!     SUBROUTINE PREINTSTL(KDLEN,KLEN,KSET,
!    1                  PRESH,
!    2                  PUF,PVF,PTF,PQF,PQN,PTS,
!    3                  PU10N,PV10N,PRF,
!    4                  PWS,PVEG,PCI,PWL,
!    5                  PSN,PSLM,PZ0,
!    6                  PCPTGZ,PWET,PQS,PCPTS,PAPHI,
!    7                  PBN,PBM,PBH,
!    8                  PLNPR,PALPH,
!    R                  PRESH5,
!    R                  PUF5,PVF5,PTF5,PQF5,PQN5,PTS5,
!    R                  PU05,PV05,PU10N5,PV10N5,PCHN5,PRF5,
!    R                  PWS5,PVEG5,PWL5,
!    R                  PSN5,PSLM5,PZ05,
!    R                  PES5,PWET5,PQS5,PAPHI5,
!    R                  PRI5,PLNPR5,PALPH5)

!        WHERE KDLEN   = FIRST DIMENSION (OBSERVATIONS)
!              KLEN    = ACTUAL NUMBER OF OBSERVATIONS
!              KSET    = NUMBER (NUMERO) OF THE OBS SET TO BE TREATED
!              PRESH5  = HALF LEVEL PRESS. AT OBS. POINTS
!              PUF5    = U VALUES (MODEL) AT OB. POINS
!              PVF5    = V VALUES (MODEL) AT OB. POINS
!              PTF5    = T VALUES (MODEL) AT OB. POINS
!              PQF5    = Q VALUES (MODEL) AT OB. POINS
!              PTS5    = SURFACE TEMPERATURE
!              PU05    = U-COMP OF OCEAN CURRENT
!              PV05    = V-COMP OF OCEAN CURRENT
!              PU10N5  = U-COMP OF 10M NEUTRAL WIND
!              PV10N5  = V-COMP OF 10M NEUTRAL WIND
!              PCHN5   = WAVE CHARNOCK
!              PRF5    = R
!              PWS5    = SOIL WETNESS
!              PVEG5   = VEGETATION
!              PCI5    = SEA-ICE FRACTION
!              PWL5    = SKIN RESERVOIR
!              PSN5    = SNOW DEPTH
!              PSLM5   = SEE/LAND MASK
!              PZ05    = ROUGHNESS LENGTH
!              PCPTGZ5 = STATIC ENERGY (LOWER LEVEL)
!              PWET5   = WETNESS
!              PQS5    = SATURATION HUMIDITY
!              PCPTS5  = STATIC ENERGY AT THE SURFACE
!              PRI5    = RICHARDSON NO.
!              PBN5    = STABILITY FUNCTION (NEUTRAL)
!              PBM5    = DRAG COEFFICIENT FOR MOMENTUM
!              PBH5    = DRAG COEFFICIENT FOR HEAT
!              PLNPR5  = LOG. OF THE RATIO OF PRESSURE
!              PALPH5  = COEFFICIENTS OF THE HYDROSTATICS

!        PREINTS IS CALLED FROM SURFACEO

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!        GPRCP      - COMPUTES Cp AND R
!        GPGEO      - COMPUTES GEOPOTENTIAL AT THE MODEL LEVELS
!        ROUSEA     - CORRECTS THE ROUGHNESS LENGTS OVER SEA
!        QNEG       - PUTS NEGATIVE VALUES OF Q EQUAL TO 0
!        SURBOUND   - COMPUTES THE STATIC ENERGY
!        EXCHCO     - COMPUTES STABILITY FUNCTION

!**   MODIFICATIONS
!     -------------
!        MODIFICATION : P. MOLL 18 juillet 1996  ACHMT transplantation
!                                (modification of surbound and exchco)
!     E.Andersson 960201 : Incorporating Scat under SURFACEO
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!     E.Andersson 97-11-17 : Flexible GOM arrays. Model Z0 for SCAT.
!     E.Andersson 97-11-17 : Use model's Z0.
!     E. Gerard   99-05-05 : Interpolation of SATEM type obs.
!     P. Moll     99-08-02 : ACHMT final transplantation
!     H.Hersbach  08-07-04 : Add ocean current and wave Charnock
!     H. Hersbach 09-12-04 : EXCHCO_VDFTL; neutral wind
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE YOMOBSET , ONLY : MTYPOB
USE YOMOBS   , ONLY : LCACHMT,LVDFTRAJ,LVDFMIN
USE YOMCST   , ONLY : RG
USE YOMCOCTP , ONLY : NSCATT
USE YOMCT0B  , ONLY : LECMWF
USE GOMS_MIX , ONLY : YGOMS,YGOMS5
USE YOEPHY   , ONLY : LECURR
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQN(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVEG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCI(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWL(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSLM(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCPTGZ(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWET(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCPTS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBN(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBM(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBH(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQN5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWL5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSN5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU05(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV05(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCHN5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ05(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PES5(KDLEN,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWET5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG) 
LOGICAL         :: LLVDF5                       ! Get exchanges coefs from IFS VDF model
LOGICAL         :: LLUN5(KDLEN)                 ! Allow perturbations in UN10
REAL(KIND=JPRB) :: ZAPHIH5(KDLEN,0:NFLEVG),ZUF5(KDLEN),ZVF5(KDLEN),ZH105(KDLEN)
REAL(KIND=JPRB) :: ZAPHIH (KDLEN,0:NFLEVG),ZUF (KDLEN),ZVF (KDLEN),ZH10 (KDLEN)

INTEGER(KIND=JPIM) :: IOBSTYPE,IPOSM,ILEN,ISET,JOBS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "exchcotl.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgeotl.intfb.h"
#include "qnegtl.intfb.h"
#include "rouseatl.intfb.h"
#include "z0seatl.intfb.h"
#include "exchco_vdftl.intfb.h"
#include "surboundtl.intfb.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------
!*
!        1.       PREPARATION OF INTERPOLATION
!                 ----------------------------

IF (LHOOK) CALL DR_HOOK('PREINTSTL',0,ZHOOK_HANDLE)

LLVDF5=.FALSE.
ISET=ABS(KSET)
ILEN     = KLEN

!   KSET < 0 INDICATES THAT MODEL DATA HAVE ALREADY BEEN FILLED
IF(KSET > 0) THEN
  IOBSTYPE = MOD(MTYPOB(ISET),100)
! Currently, limited usage to scatt data
  LLVDF5=(LVDFTRAJ.AND.(IOBSTYPE == NSCATT))

!*          1.1   FIND OUT RELEVANT ARRAYS FOR EACH OBSERVATION
!*                Some of the surface parameters are not available for
!*                SCAT data, but that doesn't matter because all SCAT
!*                data are over sea (No snow, no soil wetness...).
!*                Sea-Z0 could be calculated by ROUSEA, below.

  DO JOBS = 1,ILEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

    LLUN5(JOBS)=LVDFMIN.AND.YGOMS5(IPOSM)%LUPD

    PVEG(JOBS)=0.0_JPRB
    PCI (JOBS)=0.0_JPRB
    PSLM(JOBS)=0.0_JPRB

    PTS (JOBS) = YGOMS(IPOSM)%TS
    PZ0 (JOBS) = YGOMS(IPOSM)%Z0
    PU10N(JOBS)= YGOMS(IPOSM)%U10N
    PV10N(JOBS)= YGOMS(IPOSM)%V10N
    PWS (JOBS) = YGOMS(IPOSM)%WS
    PWL (JOBS) = YGOMS(IPOSM)%WL
    PSN (JOBS) = YGOMS(IPOSM)%SN

    IF(IOBSTYPE == NSCATT) THEN
      PWS (JOBS)=0.0_JPRB
      PWL (JOBS)=0.0_JPRB
      PSN (JOBS)=0.0_JPRB
    ENDIF

    IF(LECMWF) THEN
      PZ0(JOBS)=PZ0(JOBS)/RG
    ENDIF
  ENDDO
ENDIF
!*
!     ------------------------------------------------------------------

!*       2.  PREPARATION OF SURFACE ARRAYS
!            -----------------------------

IF( .NOT.LECMWF .AND. LCACHMT .AND. LHOOK) CALL DR_HOOK('PREINTSTL',1,ZHOOK_HANDLE)
IF( .NOT.LECMWF .AND. LCACHMT) RETURN

!*          2.1   FIND OUT POST-PROCESSED GEOPOTENTIAL IN THE LOWER
!                 LEVEL MODEL

!*          2.3 COMPUTE THE GEOPOTENTIAL ON MODEL LEVELS

ZAPHIH5(1:ILEN,NFLEVG)=0.0_JPRB
ZAPHIH (1:ILEN,NFLEVG)=0.0_JPRB
CALL GPGEO(KDLEN,1,ILEN,NFLEVG,ZAPHIH5,PAPHI5,PTF5(1,1),PRF5,PLNPR5,PALPH5)
CALL GPGEOTL(KDLEN,1,ILEN,NFLEVG,&
 & ZAPHIH,PAPHI,&
 & PTF(1,1),PRF,PLNPR,PALPH,&
 & PTF5(1,1),PRF5,PLNPR5,PALPH5)  

!*        2.3.1 CORRECT FOR OCEAN CURRENT

 IF (LECURR) THEN
    ZUF5(1:ILEN)=PUF5(1:ILEN,NFLEVG)-PU05(1:ILEN)
    ZVF5(1:ILEN)=PVF5(1:ILEN,NFLEVG)-PV05(1:ILEN)
 ELSE
    ZUF5(1:ILEN)=PUF5(1:ILEN,NFLEVG)
    ZVF5(1:ILEN)=PVF5(1:ILEN,NFLEVG)
 ENDIF
 ZUF(1:ILEN)=PUF(1:ILEN,NFLEVG)
 ZVF(1:ILEN)=PVF(1:ILEN,NFLEVG)

!*          2.3.2 COMPUTE THE ROUGHNESS LENGTH OVER SEA

IF(.NOT.LECMWF) THEN
  CALL ROUSEATL(KDLEN,1,ILEN,&
   & PAPHI5(1,NFLEVG),PUF5(1,NFLEVG),PVF5(1,NFLEVG),PSLM5,&
   & PAPHI (1,NFLEVG),PUF (1,NFLEVG),PVF (1,NFLEVG),&
   & PZ0)  
ENDIF
IF (LLVDF5) THEN
  ZH105(1:ILEN)=10.0_JPRB*RG
  ZH10 (1:ILEN)= 0.0_JPRB
  CALL Z0SEATL(KDLEN,1,ILEN,&
   & ZH105,PU10N5,PV10N5,PSLM5,PCHN5,&
   & ZH10 ,PU10N ,PV10N,&
   & PZ0)
ENDIF

!*         2.4    NEW THERMODYNAMIC VARIABLE AND BOUNDARY CONDITIONS.

CALL QNEGTL(KDLEN,1,ILEN,PQF5(1,NFLEVG),PQF(1,NFLEVG),PQN(1,NFLEVG))
CALL SURBOUNDTL(KDLEN,1,ILEN,&
 & PQN5(1,NFLEVG),PTF5(1,NFLEVG),PTS5,PRESH5(1,NFLEVG),&
 & PQN(1,NFLEVG),PTF(1,NFLEVG),PTS,PRESH(1,NFLEVG),&
 & PAPHI(1,NFLEVG),&
 & PWS5,PVEG5,PWL5,PSN5,PSLM5,&
 & PWS,PVEG,PWL,PSN,&
 & PES5(1,1),PWET5,PQS5,&
 & PWET,PQS,PCPTGZ,PCPTS)  

!*         2.5    COMPUTATION OF THE MODIFIED ROUGHNESS LENGTHS.

IF(LLVDF5) THEN
  CALL EXCHCO_VDFTL(KDLEN,1,ILEN,LLUN5,&
  & ZUF5,ZVF5,PU10N5,PV10N5,PAPHI5(1,NFLEVG),PZ05,&
  & ZUF ,ZVF ,PU10N ,PV10N ,PAPHI (1,NFLEVG),PZ0 ,&
  & PBN ,PBM)
ELSE
 CALL EXCHCOTL(KDLEN,1,ILEN,&
 & PQN5(1,NFLEVG),PTF5(1,NFLEVG),&
 & PAPHI5(1,NFLEVG),ZAPHIH5(1,NFLEVG),PRF5(1,NFLEVG),&
 & PTS5,PQN(1,NFLEVG),PTF(1,NFLEVG),&
 & PAPHI(1,NFLEVG),ZAPHIH(1,NFLEVG),PRF(1,NFLEVG),PTS,&
 & ZUF5,ZVF5,PZ05,&
 & ZUF ,ZVF ,PZ0 ,&
 & PWET5,PQS5,&
 & PWET,PQS,&
 & PRI5,&
 & PBN,PBM,PBH)  
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PREINTSTL',1,ZHOOK_HANDLE)
END SUBROUTINE PREINTSTL
