SUBROUTINE RAD1CNNE(KLEN,KMXCOUNT,PVPOBS,PNNBIAS)

! Purpose :
! -------
! Interface for neural network bias correction for AIRS

! Interface :
! ---------
! KLEN     : Number of points in observation set
! KMXCOUNT : Number of observations for each point (=nb of channels)
! PNNBIAS  : Bias Corrections (for each channel) -> Output

! Externals :
! ---------
! NEURAL_SIMULATOR

! Method : 
! ------

! Reference :
! ---------

! Author :
! ------
! 17-March-2004  Thomas Auligne *METEO-FRANCE*

! Modifications :
! -------------

! ---------------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT, NULUSR3
!! USE YOMNNE   , ONLY : NINPUTS, NPATS, NNJAC,INP_MOY,INP_STD,OUT_MOY,OUT_STD,W,JAC_MOY,JAC_STD,&
!!                     & TRAININP, TRAINOUT, GINP, LNNJAC
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KMXCOUNT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVPOBS(KLEN,KMXCOUNT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNNBIAS(KLEN,KMXCOUNT) 
INTEGER(KIND=JPIM)               :: ICHINDEX
INTEGER(KIND=JPIM)               :: ICHAN(324)
INTEGER(KIND=JPIM)               :: J
INTEGER(KIND=JPIM)               :: JBODY
!! REAL(KIND=JPRB), ALLOCATABLE     :: ZNNBIAS(:)
!! LOGICAL, ALLOCATABLE             :: LL_MASK2D(:,:)
!! LOGICAL, ALLOCATABLE             :: LL_MASK1D(:)
REAL(KIND=JPRB)                  :: ZHOOK_HANDLE
REAL(KIND=JPRB)                  :: ZBIAS
INTEGER(KIND=JPIM)               :: INIU1, IKCHAN, INCHAN
CHARACTER(LEN=120)               :: clrmtberr_text
REAL(KIND=JPRB)                  :: ZFLATBIAS(324)

IF (LHOOK) CALL DR_HOOK('RAD1CNNE',0,ZHOOK_HANDLE)

PNNBIAS(:,:)=0.

!open and read file containing AIRS bias
    WRITE(NULOUT,*)'READING AIRS BIAS (file text follows):'
    INIU1=NULUSR3
    OPEN(INIU1,STATUS='OLD',FORM='FORMATTED',FILE='rmtberr_airs.dat')
    READ(INIU1,'(A120)')clrmtberr_text
    WRITE(NULOUT,'(A)')clrmtberr_text
    INCHAN=0
    DO J=1,324
      READ(INIU1,'(I10,10X,F9.5)')IKCHAN,ZBIAS
      INCHAN=INCHAN+1
      ICHAN(J)=IKCHAN
      ZFLATBIAS(J)=ZBIAS
    ENDDO
    WRITE(NULOUT,*)'AIRS BIAS READ OK:',INCHAN
    CLOSE(INIU1)

DO JBODY=1,KLEN
  DO J=1,KMXCOUNT
    DO ICHINDEX=1,324       !! Loop over AIRS channels
      IF (INT(PVPOBS(JBODY,J)) == ICHAN(ICHINDEX)) &
       & PNNBIAS(JBODY,J)=ZFLATBIAS(ICHINDEX)
    ENDDO
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('RAD1CNNE',1,ZHOOK_HANDLE)

END SUBROUTINE RAD1CNNE
