SUBROUTINE REFLSIM_2DOP(KDLEN,KDBDY,KLEN,KHORIZ,KDCOUNT,KCOUNT,KPNLV,PTF5,PQF5,PCLWF5,&
 & PCIWF5,PCSWF5,PCRWF5,PCGWF5,PGEOPF5,KDPP,PXPP)

!SUBROUTINE REFLSIM will contain observator operator for
! radar reflectivity
! M.Jurasek & E. Wattrelot 04/05/27
! needs XRHOLW, PRHODREF ?, PCIT

USE PARKIND1,ONLY : JPIM,  JPRB
USE YOMCST  ,ONLY : RPI,RA,RG,RTT
USE YOMDIM  ,ONLY : NFLEVG
USE YOMHOOK ,ONLY : LHOOK, DR_HOOK
USE YOMLUN  ,ONLY : NULOUT
USE YOMANCS ,ONLY : RDEGREES ! 180/PI
USE YOMDB
!USE YOMINTGT
USE YOMCLMICST, ONLY : MOMG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KDLEN,KDBDY,KLEN,KDCOUNT,KHORIZ
INTEGER(KIND=JPIM),INTENT(IN) :: KDPP
INTEGER(KIND=JPIM),INTENT(IN),DIMENSION(KLEN) :: KCOUNT
INTEGER(KIND=JPIM),INTENT(IN),DIMENSION(KDLEN,KDBDY) :: KPNLV

REAL(KIND=JPRB),INTENT(IN),   DIMENSION(KDLEN,0:NFLEVG,KHORIZ)     :: PTF5,PQF5
! the following line should actually be INTENT(IN), but it is so for debugging purposes
REAL(KIND=JPRB),INTENT(INOUT),DIMENSION(KDLEN,0:NFLEVG,KHORIZ)     :: PCLWF5,PCIWF5,PCSWF5,PCRWF5,PCGWF5
REAL(KIND=JPRB),INTENT(OUT),  DIMENSION(KDLEN,KDCOUNT,KDPP,KHORIZ) :: PXPP 
REAL(KIND=JPRB),INTENT(IN),   DIMENSION(KDLEN,NFLEVG,KHORIZ)       :: PGEOPF5
REAL(KIND=JPRB),DIMENSION(:), ALLOCATABLE :: ZWKGEOPF5 ! working array for PGEOPF5
REAL(KIND=JPRB)               :: ZHOOK_HANDLE

REAL(KIND=JPRB),DIMENSION(KDLEN,0:NFLEVG,KHORIZ) :: ZREFL,ZLBDA,ZREFL_MELT_CONV
REAL(KIND=JPRB),DIMENSION(:), ALLOCATABLE :: ZWKREFL ! working array for ZREFL
REAL(KIND=JPRB) :: ZDTHETA, fcintgtrap ! beamwidth
REAL(KIND=JPRB) :: ZINTEXP ! intermediate variable
REAL(KIND=JPRB),DIMENSION(0:NFLEVG) :: ZTHETA ! angle at each model level (could have more dimensions for parallelization purposes) can be suppressed 
REAL(KIND=JPRB),DIMENSION(:),ALLOCATABLE :: ZWKTHETA ! working array for ZTHETA
INTEGER(KIND=JPIM) :: JLEN,JCOUNT,JLEV,JI,JPROF ! loop dummy counters for column, obs, level and working arrays
INTEGER(KIND=JPIM) :: IMIN,IMAX,IBODY ! min and max levels for vertical interpolation (grosso modo beam vertical extension at column)
!INTEGER(KIND=JPIM) :: ICON ! error code for TRAP

!!! constants
REAL(KIND=JPRB),PARAMETER :: XRHOLW=1000._JPRB ! density of liquid water (kg/m^3)
REAL(KIND=JPRB) :: ZCXR=-1.0_JPRB
REAL(KIND=JPRB) :: ZFRAC_WATER
REAL(KIND=JPRB) :: XAR,XBR,XCCR,     XALPHAR,XNUR,XLBEXR,XLBR,XRTMINCLW
REAL(KIND=JPRB) :: XAI,XBI,          XALPHAI,XNUI,XLBEXI,XLBI,XRTMINCIW
REAL(KIND=JPRB) :: XAS,XBS,XCCS,XCXS,XALPHAS,XNUS,XLBEXS,XLBS,XRTMINCSW
REAL(KIND=JPRB) :: XAG,XBG,XCCG,XCXG,XALPHAG,XNUG,XLBEXG,XLBG,XRTMINCGW
REAL(KIND=JPRB) :: ZEQICE=0.224_JPRB,ZDMELT_FACT,ZEXP

REAL(KIND=JPRB) :: RAE ! Earth's effective radius (4a/3)
REAL(KIND=JPRB),PARAMETER :: ZCOEF=1. ! width for vertical interpolation

!!!! REMPLISSAGE DES VARIABLES (TEMPORAIRE)
!REAL(KIND=JPRB),PARAMETER :: STALT=150._JPRB
!REAL(KIND=JPRB),PARAMETER :: ANTENHT=29._JPRB
REAL(KIND=JPRB), DIMENSION(KDLEN) :: STALT,ANTENHT ! radar station altitude and antena height
!REAL(KIND=JPRB),PARAMETER :: DISTANCE=30000._JPRB
REAL(KIND=JPRB), DIMENSION(KDLEN,KDCOUNT) :: DISTANCE,ELEVATION ! elevation en degres 
! a convertir
!REAL(KIND=JPRB),PARAMETER :: ELEVATION=1.2_JPRB
!!!! FIN REMPLISSAGE DES VARIABLES (TEMPORAIRE)

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('REFLSIM_2DOP',0,ZHOOK_HANDLE)

!*********************************************************
! 0. Initialization of constants (should be done outside)*
!*********************************************************
RAE=4._JPRB/3._JPRB*RA

ZREFL(:,:,:)=0._JPRB
PXPP(:,:,:,:)=0._JPRB
! the following line should be removed (debug)
!PCLWF5(:,:)=6.E-3_JPRB
!*       0.1     CHARACTERISTICS OF THE SPECIES
!                ------------------------------


!*       0.1.1    Raindrops characteristics

XAR = (RPI/6.0_JPRB)*XRHOLW
XBR = 3.0_JPRB

XCCR = 1.E7_JPRB    ! N0_r with XCXR = -1, implicitly


!*       0.1.2    Ice crystal characteristics


XAI = 0.82_JPRB      ! Plates
XBI = 2.5_JPRB       ! Plates 

!  Note that XCCI=N_i (a locally predicted value) and XCXI=0.0, implicitly

!*       0.1.3    Snowflakes/aggregates characteristics


XAS = 0.02_JPRB
XBS = 1.9_JPRB

XCCS = 5.0_JPRB
XCXS = 1.0_JPRB

!*       0.1.4    Heavily rimed crystals characteristics


XAG = 19.6_JPRB  ! Lump graupel case
XBG = 2.8_JPRB   ! Lump graupel case

XCCG = 5.E5_JPRB
XCXG = -0.5_JPRB
! XCCG = 4.E4 ! Test of Ziegler (1988)
! XCXG = -1.0 ! Test of Ziegler (1988)

!!*       0.2     DIMENSIONAL DISTRIBUTIONS OF THE SPECIES
!                 ----------------------------------------

!*       0.2.1    Raindrops distribution

XALPHAR = 1.0_JPRB  ! Exponential law
XNUR    = 1.0_JPRB  ! Exponential law

!*       0.2.2    Ice crystal distribution

XALPHAI = 3.0_JPRB  ! Gamma law for the ice crystal volume
XNUI    = 3.0_JPRB  ! Gamma law with little dispersion

XALPHAS = 1.0_JPRB  ! Exponential law
XNUS    = 1.0_JPRB  ! Exponential law

XALPHAG = 1.0_JPRB  ! Exponential law
XNUG    = 1.0_JPRB  ! Exponential law

!*       0.2.3    Constants for shape parameter

XLBEXR = 1.0_JPRB/(-1.0_JPRB-XBR)
XLBR   = ( XAR*XCCR*MOMG(XALPHAR,XNUR,XBR) )**(-XLBEXR)

XLBEXI = 1.0_JPRB/(-XBI)
XLBI   = ( XAI*MOMG(XALPHAI,XNUI,XBI) )**(-XLBEXI)

XLBEXS = 1.0_JPRB/(XCXS-XBS)
XLBS   = ( XAS*XCCS*MOMG(XALPHAS,XNUS,XBS) )**(-XLBEXS)

XLBEXG = 1.0_JPRB/(XCXG-XBG)
XLBG   = ( XAG*XCCG*MOMG(XALPHAG,XNUG,XBG))**(-XLBEXG)

!*       0.2.4    Minimal values allowed for the mixing ratios

!ALLOCATE( XRTMIN(6) )

!XRTMIN(1) = 1.0E-20
!XRTMIN(2) = 1.0E-20
XRTMINCLW = 1.0E-10_JPRB
XRTMINCIW = 1.0E-10_JPRB
XRTMINCSW = 1.0E-10_JPRB
XRTMINCGW = 1.0E-10_JPRB

!**********************************************************************
! 1. Compute reflectivities on each model vertical level (0D operator)*
!**********************************************************************
! we can think of restricting computations to those actually needed.
! 1.1. Rain
! ajout eric wattrelot
WHERE (PQF5(:,:,:)<=0.9_JPRB) 
PCRWF5(:,:,:)=PCRWF5(:,:,:)/(1.0_JPRB-PQF5(:,:,:))*1.2_JPRB
ENDWHERE
!WHERE(PCLWF5(:,:) > XRTMINCLW)
!   ZLBDA(:,:) = XLBR*( PCLWF5(:,:) )**XLBEXR
!   WHERE (ZLBDA(:,:) > 1.0E120_JPRB) 
!     WRITE(NULOUT,*)'ZLBDA, PCLWF5 et XLBR valent respectivement',&
!     &    ZLBDA(:,:),PCLWF5(:,:),XLBR
!   ENDWHERE
!   WHERE (ZLBDA(:,:) < 1.0E120_JPRB)
!     ZREFL(:,:) = XCCR*(ZLBDA(:,:)**(ZCXR-6.0_JPRB))*MOMG(XALPHAR,XNUR,6.0_JPRB)
!   ENDWHERE
!ENDWHERE
!WHERE (ZLBDA(:,:) > 1.0E120_JPRB)
!   WRITE(NULOUT,*)'ZLBDA, PCLWF5 et XLBR valent respectivement', &
!   &    ZLBDA(:,:),PCLWF5(:,:),XLBR
!ENDWHERE
DO JPROF=1,KHORIZ
  DO JLEN=1,KLEN
    DO JLEV=1,NFLEVG
      IF (PCRWF5(JLEN,JLEV,JPROF) > XRTMINCLW) THEN
      ZLBDA(JLEN,JLEV,JPROF) = XLBR*( PCRWF5(JLEN,JLEV,JPROF) )**XLBEXR
        IF (ZLBDA(JLEN,JLEV,JPROF) < 1.0E120_JPRB) THEN
     ZREFL(JLEN,JLEV,JPROF) = XCCR*(ZLBDA(JLEN,JLEV,JPROF)**(ZCXR-6.0_JPRB))*MOMG(XALPHAR &
    &  ,XNUR,6.0_JPRB)
!        IF (JLEN==1) WRITE(NULOUT,*)"XLBR=",XLBR,"ZREFL=",ZREFL(JLEN,JLEV)&
!&     ,"MOMG=",MOMG(XALPHAR,XNUR,6.0_JPRB),"JLEV corresp=",JLEV &
!&     ,"et PCRWF5=",PCRWF5(JLEN,JLEV)
!      ELSE
!         WRITE(NULOUT,*)'ZLBDA, PCRWF5 et XLBR valent respectivement',&
!        &    ZLBDA(JLEN,JLEV),PCRWF5(JLEN,JLEV),XLBR
        ENDIF
      ENDIF
    ENDDO
  ENDDO
ENDDO 
! 1.2. Pristine ice (problematic because of PCIT)
!!$WHERE(PCIWF5(:,:) > XRTMINCIW)
!!$   ZDMELT_FACT = ( XBI/3.0 )*( (6.0*XAI)/(RPI*XRHOLW) )**(7.0/3.0)
!!$   ZEXP = 7.0*XBI/3.0-1.0
!!$   ZLBDA(:,:) = XLBI*( PCLWF5(:,:)/PCIT(:,:) )**XLBEXI
!!$   ZREFL(:,:) = ZREFL(:,:)+ZEQICE*ZDMELT_FACT                 &
!!$      &  *PCIT(:,:)*(ZLBDA(:,:)**(-ZEXP))*MOMG(XALPHAI,XNUI,ZEXP)
!!$END WHERE

! 1.3. Snow ! ajout eric wattrelot pour rapports de melange
WHERE (PQF5(:,:,:)<=0.9_JPRB) 
PCSWF5(:,:,:)=PCSWF5(:,:,:)/(1.0_JPRB-PQF5(:,:,:))*1.2_JPRB
ENDWHERE
ZDMELT_FACT = ( XBS/3.0_JPRB )*( (6.0_JPRB*XAS)/(RPI*XRHOLW) )**(7.0_JPRB/3.0_JPRB)
ZEXP = 7.0_JPRB*XBS/3.0_JPRB-1.0_JPRB
!WHERE(PCSWF5(:,:) > XRTMINCSW)
!   ZLBDA(:,:) = XLBS*( PCSWF5(:,:) )**XLBEXS
!   WHERE (ZLBDA(:,:) > 1.0E120_JPRB) 
!     WRITE(NULOUT,*)'ZLBDA, PCSWF5 et XCCS valent respectivement',&
!     &    ZLBDA(:,:),PCSWF5(:,:),XCCS
!   ENDWHERE 
!    WHERE (ZLBDA(:,:) <= 1.0E120_JPRB)
!    ZREFL(:,:) = ZREFL(:,:)+ZEQICE*ZDMELT_FACT                &
!    &    *XCCS*(ZLBDA(:,:)**(XCXS-ZEXP))*MOMG(XALPHAS,XNUS,ZEXP)
!    ENDWHERE
!ENDWHERE
!WHERE (ZLBDA(:,:) > 1.0E120_JPRB)
!  WRITE(NULOUT,*)'ZLBDA, PCSWF5 et XCCS valent respectivement',&
!   &    ZLBDA(:,:),PCSWF5(:,:),XCCS
!ENDWHERE
DO JPROF=1,KHORIZ
  DO JLEN=1,KLEN
    DO JLEV=1,NFLEVG
      IF (PCSWF5(JLEN,JLEV,JPROF) > XRTMINCSW) THEN
       ZLBDA(JLEN,JLEV,JPROF) = XLBS*( PCSWF5(JLEN,JLEV,JPROF) )**XLBEXS
         IF (ZLBDA(JLEN,JLEV,JPROF) < 1.0E120_JPRB) THEN
          ZREFL(JLEN,JLEV,JPROF) = ZREFL(JLEN,JLEV,JPROF)+ZEQICE*ZDMELT_FACT      &
        &    *XCCS*(ZLBDA(JLEN,JLEV,JPROF)**(XCXS-ZEXP))*MOMG(XALPHAS,XNUS,ZEXP)
!        IF (JLEN==1) WRITE(NULOUT,*)"pour la neige ZREFL=",ZREFL(JLEN,JLEV)&
!&      ,"MOMG=",MOMG(XALPHAS,XNUS,ZEXP),"JLEV corresp=",JLEV &
!&      ,"PCSWF5=",PCSWF5(JLEN,JLEV)
!       ELSE
!         WRITE(NULOUT,*)'ZLBDA, PCSWF5 et XLBS valent respectivement',&
!      &  ZLBDA(JLEN,JLEV),PCSWF5(JLEN,JLEV),XLBS
         ENDIF
      ENDIF
    ENDDO
  ENDDO
ENDDO
  
! 1.4. Graupel
WHERE (PQF5(:,:,:)<=0.9_JPRB) 
PCGWF5(:,:,:)=PCGWF5(:,:,:)/(1.0_JPRB-PQF5(:,:,:))*1.2_JPRB
ENDWHERE
ZFRAC_WATER = 0.14
ZDMELT_FACT = ( XBG/3.0_JPRB )*( (6.0_JPRB*XAG)/(RPI*XRHOLW) )**(7.0_JPRB/3.0_JPRB)
WHERE( PTF5(:,:,:) > RTT )
   ZREFL_MELT_CONV(:,:,:) = ((1.0-ZFRAC_WATER)*ZEQICE+ZFRAC_WATER)*ZDMELT_FACT
ELSEWHERE
   ZREFL_MELT_CONV(:,:,:) = ZEQICE*ZDMELT_FACT
ENDWHERE

ZEXP = 7.0_JPRB*XBG/3.0_JPRB-1.0_JPRB
!WHERE(PCGWF5(:,:) > XRTMINCGW)
!   ZLBDA(:,:) = XLBG*( PCGWF5(:,:) )**XLBEXG
!   ZREFL(:,:) = ZREFL(:,:)+ZREFL_MELT_CONV(:,:) &
!   &     *XCCG*(ZLBDA(:,:)**(XCXG-ZEXP))*MOMG(XALPHAG,XNUG,ZEXP)
!ENDWHERE
DO JPROF=1,KHORIZ
  DO JLEN=1,KLEN
    DO JLEV=1,NFLEVG
      IF (PCGWF5(JLEN,JLEV,JPROF) > XRTMINCGW) THEN 
     ZLBDA(JLEN,JLEV,JPROF) = XLBG*( PCGWF5(JLEN,JLEV,JPROF) )**XLBEXG
        IF (ZLBDA(JLEN,JLEV,JPROF) < 1.0E120_JPRB) THEN
          ZREFL(JLEN,JLEV,JPROF) = ZREFL(JLEN,JLEV,JPROF)+ZREFL_MELT_CONV(JLEN,JLEV,JPROF) &
        &     *XCCG*(ZLBDA(JLEN,JLEV,JPROF)**(XCXG-ZEXP))*MOMG(XALPHAG,XNUG,ZEXP)
         if (JLEN==1) WRITE(NULOUT,*)'refl calculee pour graupel, ZLBDA, PCGWF5 &
        &  et XLBG valent respectivement',&
        &    ZLBDA(JLEN,JLEV,JPROF),PCGWF5(JLEN,JLEV,JPROF),XLBG
        ELSE 
        WRITE(NULOUT,*)'ZLBDA, PCGWF5 et XLBG valent respectivement',&
        &    ZLBDA(JLEN,JLEV,JPROF),PCGWF5(JLEN,JLEV,JPROF),XLBG
        ENDIF
      ENDIF 
   ENDDO
  ENDDO
ENDDO 

! 1.5. Unit conversion
!ZREFL(:,:) = 10.0*LOG10( 1.E18_JPRB*ZREFL(:,:) ) ! Z_e in dBZ


!*******************************************************
! 2. Weight reflectivities over the vertical (Gaussian)*
!*******************************************************
!2.1 
write(NULOUT,*) shape(SATBODY),shape(SATHDR)
DO JPROF=1,KHORIZ
DO JLEN=1,KLEN ! loop over model columns of observations
   DO JCOUNT=1,KCOUNT(JLEN) ! loop over each observation data
      STALT(JLEN)=SATHDR(JLEN,MDB_STALT_AT_RADAR_STATION)
      ANTENHT(JLEN)=SATHDR(JLEN,MDB_ANTENHT_AT_RADAR_STATION)
      IBODY = MLNKH2B(JLEN)+ KPNLV(JLEN,JCOUNT) -1
      DISTANCE(JLEN, JCOUNT)=SATBODY(IBODY,MDB_DISTANCE_AT_RADAR_BODY) 
      ELEVATION(JLEN,JCOUNT)=SATBODY(IBODY,MDB_ELEVATION_AT_RADAR_BODY)
!      write(NULOUT,*) 'cette elevation vaut',JLEN,JCOUNT,ELEVATION(JLEN,JCOUNT)
!       DISTANCE(JLEN, JCOUNT)=ROBODY(IBODY,MDBVAR)
      IMIN=1
      IMAX=1
      ZDTHETA=0.8_JPRB ! en degres a convertir directly from ODB (BEAMWIDTH)
      DO JLEV=1,NFLEVG
! ajout de la conversion si refl non nulle
!         IF (ZREFL(JLEN,JLEV) > 0) ZREFL(JLEN,JLEV)=10.0*LOG10( 1.E18_JPRB*ZREFL(JLEN,JLEV) ) ! Z_e in dBZ
         ZTHETA(JLEV)=((PGEOPF5(JLEN,JLEV,1)/RG-STALT(JLEN)-ANTENHT(JLEN)+RAE)**2&
         &-DISTANCE(JLEN,JCOUNT)**2-RAE**2)/2/DISTANCE(JLEN,JCOUNT)/RAE &
             & -ELEVATION(JLEN,JCOUNT)/RDEGREES ! directly from ODB
         IF((ZTHETA(JLEV)*RDEGREES) >= .5_JPRB*ZCOEF*ZDTHETA) IMIN=JLEV
         IF((ZTHETA(JLEV)*RDEGREES) >= (-0.5_JPRB)*ZCOEF*ZDTHETA) IMAX=JLEV
!     WRITE(NULOUT,*)'IMIN et IMAX et ZTHETA et DISTANCE dans reflsim valent',IMIN,IMAX,ZTHETA(JLEV)+ &
!    & ELEVATION(JLEN,JCOUNT)/RDEGREES,DISTANCE(JLEN,JCOUNT)
      ENDDO
!     IF (JLEN==1) THEN 
!                   WRITE(NULOUT,*)'IMIN et IMAX et ZTHETA et DISTANCE dans reflsim valent',IMIN,IMAX, &
!                    & ELEVATION(1,JCOUNT)/RDEGREES,DISTANCE(1,JCOUNT)
!                   WRITE(NULOUT,*)'LAT, LON de l obs correspondante',ROBHDR(1,MDBLAT)*RDEGREES,ROBHDR(1,MDBLON)*RDEGREES
!                   WRITE(NULOUT,*)'IBODY=',IBODY,MLNKH2B(1)+(JCOUNT-1)
!     ENDIF
! next 2 lines to be removed
!IMIN=5
!IMAX=10
! we have to reshape arrays because of intrinsic TRAP - perhaps there exists a better method
      IF (IMIN < IMAX) THEN 
      ALLOCATE(ZWKGEOPF5(IMIN:IMAX),ZWKREFL(IMIN:IMAX),ZWKTHETA(IMIN:IMAX))
      DO JI=IMIN,IMAX
         ZWKGEOPF5(JI)=PGEOPF5(JLEN,IMAX+IMIN-JI,1)
!          ZWKGEOPF5(JI)=PGEOPF5(JLEN,JI)
         ZWKREFL(JI)=ZREFL(JLEN,IMAX+IMIN-JI,JPROF)
!          ZWKREFL(JI)=ZREFL(JLEN,JI)
         ZWKTHETA(JI)=ZTHETA(IMAX+IMIN-JI)
!          ZWKTHETA(JI)=ZTHETA(JI)
      ENDDO

!      CALL DTRAP(ZWKGEOPF5(:)/RG,ZWKREFL(:)* &
!         &  EXP(-8._JPRB*LOG(2._JPRB)*(ZWKTHETA(:)/ZDTHETA*RDEGREES)**2),&
!         &  IMAX-IMIN+1,PXPP(JLEN,JCOUNT,1),ICON)
!**********
! use new function from module YOMINTGT 
!*
       PXPP(JLEN,JCOUNT,1,JPROF) = fcintgtrap(ZWKGEOPF5(:)/RG,ZWKREFL(:)* &
          &  EXP(-8._JPRB*LOG(2._JPRB)*(ZWKTHETA(:)/ZDTHETA*RDEGREES)**2),&
          &  IMAX-IMIN+1)
!*
!      IF(ICON/=0) THEN
!         WRITE(NULOUT,*)"ERROR IN TRAP (MODULE REFLSIM.F90) 1"
!         print*,"N, theta, jlen, ILEN, jcount",IMAX-IMIN+1,ZDTHETA,jlen,KLEN,jcount
!          WRITE(NULOUT,*)"N, theta, jlen, ILEN, jcount",IMAX-IMIN+1,ZDTHETA,jlen,KLEN,jcount
!         print*,"GEOP",ZWKGEOPF5(IMIN:IMAX)/RG
!          WRITE(NULOUT,*)"GEOP",ZWKGEOPF5(IMIN:IMAX)/RG
!         WRITE(NULOUT,*)"REFL",ZWKREFL(IMIN:IMAX)* &
!            &  EXP(-8._JPRB*LOG(2._JPRB)*(ZWKTHETA(IMIN:IMAX)/ZDTHETA*RDEGREES)**2)
!         WRITE(NULOUT,*)"THET",EXP(-8._JPRB*LOG(2._JPRB)*(ZWKTHETA(IMIN:IMAX)/ZDTHETA*RDEGREES)**2)
!      ENDIF
!      CALL DTRAP(ZWKGEOPF5(:)/RG,                       &
!        &   EXP(-8._JPRB*LOG(2._JPRB)*(ZWKTHETA(:)/ZDTHETA*RDEGREES)**2),&
!        &   IMAX-IMIN+1,ZINTEXP            ,ICON) ! there are more efficient methods to integrate this, but we want to keep consistency
!      IF(ICON/=0) PRINT*,"ERROR IN TRAP (MODULE REFLSIM.F90) 2"
!************
! use new function from module YOMINTGT
!*
            ZINTEXP = fcintgtrap(ZWKGEOPF5(:)/RG,                       &
               &   EXP(-8._JPRB*LOG(2._JPRB)*(ZWKTHETA(:)/ZDTHETA*RDEGREES)**2),&
               &   IMAX-IMIN+1)
!*
!************
      DEALLOCATE(ZWKGEOPF5,ZWKREFL,ZWKTHETA)
      PXPP(JLEN,JCOUNT,1,JPROF)=PXPP(JLEN,JCOUNT,1,JPROF)/ZINTEXP
! conversion des unites des reflectivites si sup a 1 mm6/m2
         IF (PXPP(JLEN,JCOUNT,1,JPROF)>=1.E-18_JPRB) THEN 
              PXPP(JLEN,JCOUNT,1,JPROF)=10.0*LOG10( 1.E18_JPRB*PXPP(JLEN,JCOUNT,1,JPROF))
            IF (PXPP(JLEN,JCOUNT,1,JPROF)>=2) THEN
              WRITE(NULOUT,*)"REFL a la fin vaut",JLEN,JCOUNT,PXPP(JLEN,JCOUNT,1,JPROF)
              WRITE(NULOUT,*)"IMIN, REFL DE IMIN,IMAX ET REFL DE IMAX= puis pclw",IMIN,ZREFL(JLEN,IMIN,1),IMAX,& 
&             ZREFL(JLEN,IMAX,1),PCRWF5(JLEN,IMIN,1),PCRWF5(JLEN,IMAX,1)
            ENDIF
         ENDIF      
      ENDIF
   ENDDO
ENDDO
ENDDO
!WRITE(NULOUT,*)"altitude du radar et hauteur de l'antenne",STALT(1), ANTENHT(1) 

! other method: interpolation of Z on the column and analytic integration of Z*exp(...).

!PXPP(:,:,:)=0_JPRB

IF (LHOOK) CALL DR_HOOK('REFLSIM_2DOP',1,ZHOOK_HANDLE)


END SUBROUTINE  REFLSIM_2DOP
