!  Differentiation of rt6s_gems in reverse (adjoint) mode:
!   gradient, with respect to input variables: preflet ptaer_vertin
!                zphasel_in piz_in
!   of linear combination of output variables: preflet

SUBROUTINE RT6S_GEMS_AD(pdiffang, psolang, pvieang, pwl, pphasel_in, &
&  pphasel_inb, piz_in, piz_inb, ptray550_vertin, ptaer_vertin, &
&  ptaer_vertinb, proc, preflet, prefletb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE

!****RT6S_GEMS*** MAIN CALLING ROUTINE OF THE RADIATIVE TRANSFER CODE
!               
!  THIS CORRESPONDS TO A MODIFIED VERSION OF THE 6S RADIATIVE TRANSFER 
! MODEL DEVELLOPED AT LOA. FEATURES SUCH AS AIRPLANE MEASUREMENTS, MULTIPLE CHOICE 
! IN  THE AEROSOL MODELS, ATMOSPHERIC CONDITIONS, SURFACE MODEL FOR THE COMPUTING OF 
! SURFACE REFLECTANCE ARE ELIMINATED. THE MODEL IS ADAPTED IN ORDER TO FIT THE NEEDS
! OF GEMS PROJECT, THIS IS TO COMPUTE THE REFLECTANCE AT MULTIPLE WAVELENGTH (KWL)
! FOR A SINGLE AEROSOL MODEL WITH A GIVEN VERTICAL DISTRIBUTION.



!     INTERFACE
!     ---------
!           RT6S_GEMS IS CALLED BY ECMWF MODEL

!     INPUTS
!     ------
! PDIFFANG(KLON)    : DIFFERENCE IN AZIMUTAL ANGLE [RAD]
! PSOLANG(KLON)     : COSINE OF SOLAR ZENITH ANGLE
! PVIEANG(KLON)     : COSINE OF VIEWING ZENITH ANGLE
! PWL(KWL)                     : Wavelengths at which the calculations are done (m).
! PTRAY550_VERTIN(KLON,KLEV)   : Molecular optical thickness profile at 0.55e-6 m.
! PTAER_VERTIN(KLON,KLEV,KWL)  : Aerosol optical thickness profile.
! PHASEL_IN(KLON,KLEV,KWL,KMU) : Aerosol phase function profile.
! PIZ_IN(KLON,KLEV,KWL)        : Aerosol single scattering albedo profile.
! PROC(KLON,KWL)               : TARGET SURFACE REFLECTANCE

!     OUTPUTS
!     -------
! REFLET(KLON,KWL) = APPARENT REFLECTANCE AT KWL WAVELENGTH

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM MAIN CALLING ROUTINE IN ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
  REAL(KIND=JPRB), INTENT(IN) :: pwl(kwl), pdiffang(klon), psolang(klon&
&  ), pvieang(klon)
  REAL(KIND=JPRB), INTENT(IN) :: ptray550_vertin(klon, 0:klev), ptaer_vertin(klon, &
&  0:klev, kwl)
  REAL(KIND=JPRB), INTENT(INOUT) :: ptaer_vertinb(klon, 0:klev, kwl)
  REAL(KIND=JPRB), INTENT(IN) :: pphasel_in(klon, 0:klev, kwl, knbmu), piz_in(klon, &
&  0:klev, kwl)
  REAL(KIND=JPRB), INTENT(INOUT) :: pphasel_inb(klon, 0:klev, kwl, knbmu), piz_inb(klon&
&  , 0:klev, kwl)
  REAL(KIND=JPRB), INTENT(IN) :: proc(klon, kwl)
  REAL(KIND=JPRB), INTENT(OUT) :: preflet(klon, kwl)
  REAL(KIND=JPRB), INTENT(INOUT) :: prefletb(klon, kwl)
! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
!INTEGER(KIND=JPIM),PARAMETER  :: KWL =1 !!!!!! This value is totally invented. CHECK WITH NICOLAS.
!INTEGER(KIND=JPIM),PARAMETER  :: KLON =1, KLEV = 60 
  INTEGER(KIND=JPIM) :: jlev, jwl, jlon, jnbmu
!GAUSS WHEIGHTS AND ANGLES
  REAL(KIND=JPRB) :: zrp(knp), zgp(knp)
  REAL(KIND=JPRB) :: zrmu(knbmu), zga(knbmu)
  REAL(KIND=JPRB) :: zrm(-kmu:kmu), zgb(-kmu:kmu)
  REAL(KIND=JPRB) :: zrm_aux(klon, -kmu:kmu)
! VERTICAL DISCRETIZATION                                  
  REAL(KIND=JPRB) :: ztaer_vert(klon, 0:knt, kwl), ztray_vert(klon, 0:&
&  knt, kwl)
  REAL(KIND=JPRB) :: ztaer_vertb(klon, 0:knt, kwl)
  REAL(KIND=JPRB) :: zphasel(klon, 0:knt, kwl, knbmu), zpiz(klon, 0:knt, &
&  kwl)
  REAL(KIND=JPRB) :: zphaselb(klon, 0:knt, kwl, knbmu), zzpizb(klon, 0:knt&
&  , kwl)
! DISCOM VARIABLES
  REAL(KIND=JPRB) :: zphirad, zxmus, zxmuv
  REAL(KIND=JPRB) :: ztaer(0:knt), ztray(0:knt), zomega(0:knt), zphasef(&
&  0:knt, knbmu)
  REAL(KIND=JPRB) :: ztaerb(0:knt), zomegab(0:knt), zphasefb(0:knt, &
&  knbmu)
  REAL(KIND=JPRB) :: zroatm, zdtdir, zdtdif, zutdir, zutdif, zsphal
  REAL(KIND=JPRB) :: zroatmb, zdtdirb, zdtdifb, zutdirb, zutdifb, &
&  zsphalb
  REAL(KIND=JPRB) :: zpl(knbmu, -1:knang+1)
! KERNEL VARIABLE
  REAL(KIND=JPRB) :: zpsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB) :: zpsl_aux(klon, 0:knang, -1:knang, -kmu:kmu)
! SURFACE REFLECTANCE
  REAL(KIND=JPRB) :: zsurfref
  REAL(KIND=JPRB) :: zsurfrefb
  REAL(KIND=JPRB) :: ztemp
  INTRINSIC COS
  REAL(KIND=JPRB) :: ztempb

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "discom_gems.intfb.h"
#include "discom_gems_ad.intfb.h"
#include "pre_calc.intfb.h"
#include "vertdisc.intfb.h"
#include "vertdisc_ad.intfb.h"
#include "popreal8.intfb.h"
!#include "popreal8array.intfb.h"
#include "pushreal8.intfb.h"
!#include "pushreal8array.intfb.h"

  
  IF (LHOOK) CALL DR_HOOK('RT6S_GEMS_AD',0,ZHOOK_HANDLE)

! CALCULATION OF GAUSS WEIGHTS AND ANGLES
! ---------------------------------------

  CALL PRE_CALC(pvieang, psolang, zrp, zgp, zrm_aux, zgb, zrmu, zga, zpl, &
&          zpsl_aux)

! CONVERTING VERTICAL DISCRETIZATION INTO ONE WITH ISO-OD LAYERS
! --------------------------------------------------------------

  CALL VERTDISC(pwl, ptray550_vertin, ptaer_vertin, pphasel_in, piz_in, &
&          ztaer_vert, ztray_vert, zphasel, zpiz)

! TRUNCATION OF THE PHASE FUNCTION
! --------------------------------

  DO jlon=1,klon
    zphirad = pdiffang(jlon)
    CALL PUSHREAL8(zxmus)
    zxmus = COS(psolang(jlon)*zpi/180._JPRB)
    CALL PUSHREAL8(zxmuv)
    zxmuv = COS(pvieang(jlon)*zpi/180._JPRB)
    zrm(:) = zrm_aux(jlon, :)
    CALL PUSHREAL8ARRAY(zpsl, 81*82*51)
    zpsl(:, :, :) = zpsl_aux(jlon, :, :, :)
    DO jwl=1,kwl

! ONLY THE VERTICAL IS PASSED FROM NOW ON
! ---------------------------------------

      DO jlev=0,knt
        CALL PUSHREAL8(ztaer(jlev))
        ztaer(jlev) = ztaer_vert(jlon, jlev, jwl)
        CALL PUSHREAL8(ztray(jlev))
        ztray(jlev) = ztray_vert(jlon, jlev, jwl)
        CALL PUSHREAL8(zomega(jlev))
        zomega(jlev) = zpiz(jlon, jlev, jwl)
        DO jnbmu=1,knbmu
          zphasef(jlev, jnbmu) = zphasel(jlon, jlev, jwl, jnbmu)
        ENDDO
      ENDDO
      CALL PUSHREAL8(zsphal)
      CALL PUSHREAL8(zutdif)
      CALL PUSHREAL8(zutdir)
      CALL PUSHREAL8(zdtdif)
      CALL PUSHREAL8(zdtdir)
      CALL PUSHREAL8ARRAY(zphasef, 27*83)
      CALL PUSHREAL8ARRAY(zrm, 51)

! COMPUTATION OF ATMOSPHERIC REFLECTANCE AND TRANSMITTANCE
! --------------------------------------------------------

      CALL DISCOM_GEMS(zphirad, zxmus, zxmuv, zrm, zgb, zrp, zrmu, zga, &
&                 ztaer, ztray, zomega, zphasef, zpl, zpsl, zroatm, &
&                 zdtdir, zdtdif, zutdir, zutdif, zsphal)

! COMPUTATION OF THE SURFACE REFLECTANCE
! --------------------------------------

      zsurfref = proc(jlon, jwl)*(zdtdir+zdtdif)*(zutdir+zutdif)/(1.-&
&        proc(jlon, jwl)*zsphal)

! FINAL CALCULATION OF REFLECTANCE
! --------------------------------

      preflet(jlon, jwl) = zroatm + zsurfref
    ENDDO
  ENDDO
  zzpizb(1:klon, 0:knt, 1:kwl) = 0._JPRB
  zomegab(0:knt) = 0._JPRB
  ztaerb(0:knt) = 0._JPRB
  zphasefb(0:knt, 1:knbmu) = 0._JPRB
  zphaselb(1:klon, 0:knt, 1:kwl, 1:knbmu) = 0._JPRB
  ztaer_vertb(1:klon, 0:knt, 1:kwl) = 0._JPRB
  DO jlon=klon,1,-1
    DO jwl=kwl,1,-1
      zroatmb = prefletb(jlon, jwl)
      zsurfrefb = prefletb(jlon, jwl)
      prefletb(jlon, jwl) = 0._JPRB
      ztemp = -(proc(jlon, jwl)*zsphal) + 1.
      ztempb = proc(jlon, jwl)*zsurfrefb/ztemp
      zdtdirb = (zutdir+zutdif)*ztempb
      zdtdifb = (zutdir+zutdif)*ztempb
      zutdirb = (zdtdir+zdtdif)*ztempb
      zutdifb = (zdtdir+zdtdif)*ztempb
      zsphalb = (zdtdir+zdtdif)*(zutdir+zutdif)*proc(jlon, jwl)*ztempb/&
&        ztemp
      CALL POPREAL8ARRAY(zrm, 51)
      CALL POPREAL8ARRAY(zphasef, 27*83)
      CALL POPREAL8(zdtdir)
      CALL POPREAL8(zdtdif)
      CALL POPREAL8(zutdir)
      CALL POPREAL8(zutdif)
      CALL POPREAL8(zsphal)
      CALL DISCOM_GEMS_AD(zphirad, zxmus, zxmuv, zrm, zgb, zrp, zrmu, zga, &
&                   ztaer, ztaerb, ztray, zomega, zomegab, zphasef, &
&                   zphasefb, zpl, zpsl, zroatm, zroatmb, zdtdir, zdtdirb, &
&                   zdtdif, zdtdifb, zutdir, zutdirb, zutdif, zutdifb,&
&                   zsphal, zsphalb)
      DO jlev=knt,0,-1
        DO jnbmu=knbmu,1,-1
          zphaselb(jlon, jlev, jwl, jnbmu) = zphaselb(jlon, jlev, jwl, &
&            jnbmu) + zphasefb(jlev, jnbmu)
          zphasefb(jlev, jnbmu) = 0._JPRB
        ENDDO
        CALL POPREAL8(zomega(jlev))
        zzpizb(jlon, jlev, jwl) = zzpizb(jlon, jlev, jwl) + zomegab(jlev)
        zomegab(jlev) = 0._JPRB
        CALL POPREAL8(ztray(jlev))
        CALL POPREAL8(ztaer(jlev))
        ztaer_vertb(jlon, jlev, jwl) = ztaer_vertb(jlon, jlev, jwl) + &
&          ztaerb(jlev)
        ztaerb(jlev) = 0._JPRB
      ENDDO
    ENDDO
    CALL POPREAL8ARRAY(zpsl, 81*82*51)
    CALL POPREAL8(zxmuv)
    CALL POPREAL8(zxmus)
  ENDDO
  CALL VERTDISC_AD(pwl, ptray550_vertin, ptaer_vertin, ptaer_vertinb, &
&            pphasel_in, pphasel_inb, piz_in, piz_inb, ztaer_vert, &
&            ztaer_vertb, ztray_vert, zphasel, zphaselb, zpiz, zzpizb)
  prefletb(1:klon, 1:kwl) = 0._JPRB

 IF (LHOOK) CALL DR_HOOK('RT6S_GEMS_AD',1,ZHOOK_HANDLE)
END SUBROUTINE RT6S_GEMS_AD
