SUBROUTINE SLINTAD(&
 & KPROMB,KIBUFP1,KASLB1,KFIELDS,&
 & PLAT,PLON,POBB1,PSL)  

!**** *SLINTAD* - Horizontal interpolation of the model variables
!****             from grid points to observation points by using semi-
!****             lagrangian interpolation routunes (adjoint)

!     Purpose.
!     -------
!           Feeding the tables GOMMV,GOSMV,GSMMV,GSSMV,GSCMV with the model
!     variables at the observation points, from the model variables in
!     grid points. These tables are containing the quantity called Xmvo
!     in the scientific documentation.

!**   Interface.
!**   ---------
!**      CALL SLINTAD(..)
!**          KIBUFP1  = S/L BUFFERS TABLE
!**          POBB1    = S/L BUFFER 

!**   Externals
!     ---------
!        LASCAW - PREPARE INTERPOLATION
!        LAIDLIAD - 4 POINT INTERPOLATION (ADJOINT)
!        LAIDDIAD - 12 POINT INTERPOLATION (ADJOINT)

!     Author: Drasko Vasiljevic
!     Date  : 92-05-19

!     Modifications
!     -------------
!     92-12-01 J-N.Thepaut : change of name conventions.
!              F.Bouttier  : adjoint of Canari case
!     94-02-20 K.Yessad    : change of numbering for switch IWIS.
!     95-09-20 H.Jarvinen  : New CMA file format
!     96-12    V.Casse     : convert u and v in geographical axes
!     97-11-17 E.Andersson : Introduce CIW and O3
!     97-12-01 F.Veerse    : quick dirty fix for idlx bug
!     97-03-15 R. Randriamampianina : ALADIN 3DVAR
!     98-05-27 M. Siroka   : correction in calling ELASCAW
!     98-07-31 K. YESSAD   : removal of useless seq 2.02 and arg ZIL in LASCAW.
!     99-06-23 A. Horanyi  : correcting the call of ELASCAW (removing ZIL) 
!     03-05-15 C. Fischer  : add arg in call to elascaw (usual monkey business)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     2004-Nov F. Vana     : update of arguments of (E)LASCAW
!     09-Jan-2007 F. Vana  : arguments update for ELASCAW
!     28-Aug-2007 F. Vana  : update of arguments of (E)LASCAW
!     07-Nov-2007 J. Masek : update of arguments of (E)LASCAW
!     20-Aug-2008 F. Vana  : Weights driven interpolation.
!     03-Nov-2009 H.Hersbach : Introduce KFIELDS; number of vertical+surf fields
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGSAG   ,NDGSAL   ,NDGENG   ,NDGENL   ,&
 & NFLEVG   ,NDGSAH   ,NDGENH   ,NDGUNG   ,NDGUXG   ,&
 & NDLUNG   ,NDLUXG  
USE YOMLUN   , ONLY :    NULERR
USE YOMCST   , ONLY : RPI
USE YOMCT0   , ONLY : LRPLANE
USE YOMGEM   , ONLY : NLOENG   ,R4JP
USE YOMLEG   , ONLY : RLATI    ,RIPI0    ,RIPI1    ,RIPI2
USE YOMSC2   , ONLY : NSLWIDE  ,NFLDOBB1
USE YOMOBS   , ONLY : NOBSHOR
USE YOMMP    , ONLY : NSLONL   ,NSLOFF   ,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT

!    -------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB
INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS ! Number of vertical+surface fields
INTEGER(KIND=JPIM),INTENT(IN)    :: KIBUFP1(NDGSAH:NDGENH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(KPROMB,1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON(KPROMB,1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POBB1(KASLB1,NFLDOBB1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSL(KFIELDS,KPROMB) 

!    -------------------------------------------------------------------

REAL(KIND=JPRB) ::    ZLSDEPI(NDGSAH:NDGENH)
REAL(KIND=JPRB) ::    ZLEV(KPROMB,1)
REAL(KIND=JPRB) ::    ZDSLAT(KPROMB,    1)
REAL(KIND=JPRB) ::    ZCSLA (KPROMB,    1,2:4)
REAL(KIND=JPRB) ::    ZDSLON(KPROMB,    1,0:3)
REAL(KIND=JPRB) ::    ZCSLO (KPROMB,    1,3,2)
INTEGER(KIND=JPIM) :: IL0S  (KPROMB,    1,0:3)
INTEGER(KIND=JPIM) :: ILH0  (KPROMB,1,0:3), ILEV  (KPROMB,1)
INTEGER(KIND=JPIM) :: ISTALAT(NDGSAH:NDGENH)
!CC REAL(KIND=JPRB) ::    ZDVER (KPROMB,1)
INTEGER(KIND=JPIM) :: IL0ULI(KPROMB,2,2)   ,IL0UDI(KPROMB,0:3,0:3)
INTEGER(KIND=JPIM) :: ITRANS(0:KASLB1)
INTEGER(KIND=JPIM) :: IDEP(KPROMB,NFLEVG)

INTEGER(KIND=JPIM) :: IEND, IFLDN, IFLDX,&
 & IGLGLO, IHOR, IHVI, ILFB, ILSB, IST, IWIS, &
 & J, J1, JGL, JL, JLAT  

REAL(KIND=JPRB) :: ZDEPI, ZPIS2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! unused arguments in call to LASCAW/ELASCAW
! a) input - array dimensions contracted
INTEGER(KIND=JPIM) :: IUN_VAUTF(1)
REAL(KIND=JPRB) :: ZUN_SLD1(1)
REAL(KIND=JPRB) :: ZUN_SLD2(1)
REAL(KIND=JPRB) :: ZUN_SLD3(1)
REAL(KIND=JPRB) :: ZUN_SLDW(1)
REAL(KIND=JPRB) :: ZUN_VETAF(1)
REAL(KIND=JPRB) :: ZUN_VETAH(1)
REAL(KIND=JPRB) :: ZUN_VCUICO(1)
REAL(KIND=JPRB) :: ZUN_VSLD(1)
REAL(KIND=JPRB) :: ZUN_VSLDW(1)
REAL(KIND=JPRB) :: ZUN_KAPPA(1)
! b) output - array dimensions kept for safety
REAL(KIND=JPRB) :: ZUN_CLASLD(KPROMB,1,3)
REAL(KIND=JPRB) :: ZUN_CLOSLD(KPROMB,1,3,2)
REAL(KIND=JPRB) :: ZUN_DVER(KPROMB,1)
REAL(KIND=JPRB) :: ZUN_VINTW(KPROMB,1,3)
REAL(KIND=JPRB) :: ZUN_VINTWSLD(KPROMB,1,3)
REAL(KIND=JPRB) :: ZUN_VINTWS(KPROMB,1,1:4)
REAL(KIND=JPRB) :: ZUN_VDERW(KPROMB,1,0,0)  ! case KHVI=0
REAL(KIND=JPRB) :: ZUN_HVW(KPROMB,1,0)      ! case KHVI=0

LOGICAL         :: LLSLHD,LLSLHDQUAD

!    -------------------------------------------------------------------

#include "abor1.intfb.h"
#include "elascaw.intfb.h"
#include "laiddiobsad.intfb.h"
#include "laidliobsad.intfb.h"
#include "lascaw.intfb.h"

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SLINTAD',0,ZHOOK_HANDLE)

!    -------------------------------------------------------------------
!*       1. INTERPOLATION AT OBS. POINTS
!           ----------------------------

!*          1.2   INTERPOLATION

!*          1.2.1  PRESET:

CALL GSTATS(1869,0)
IWIS = NOBSHOR
IF ( IWIS /= 201.AND. IWIS /= 203 ) THEN
  CALL ABOR1('SLINTAD: UNKNOWN INTERPOL. NO:')
ENDIF
IFLDN = 1
IFLDX = NFLDOBB1
IHOR = 0
IHVI = 0
ZDEPI=2.0_JPRB*RPI
ZPIS2=0.5_JPRB*RPI

DO JGL = MAX(NDGSAG,NDGSAL+NFRSTLOFF-NSLWIDE)-NFRSTLOFF,&
   & MIN(NDGENG,NDGENL+NFRSTLOFF+NSLWIDE)-NFRSTLOFF  
  IGLGLO=JGL+NFRSTLOFF
  ZLSDEPI(JGL)=REAL(NLOENG(IGLGLO),JPRB)/ZDEPI
ENDDO

DO JLAT=MAX(NDGSAH,LBOUND(NSLOFF,1)),MIN(NDGENH,UBOUND(NSLOFF,1))
  ISTALAT(JLAT)=NSLOFF(JLAT)
ENDDO

ZLEV(:,:) = 0.0_JPRB

!*          1.2.2  INTERPOLATION/BUFFER PARAMETERS

! Switching off the SLHD related weights computation
LLSLHD=.FALSE.
LLSLHDQUAD=.FALSE.

IF (LRPLANE) THEN
  CALL ELASCAW(KASLB1,KPROMB,1,KPROMB,1,1,NDGSAH,NDGENH,&
   & NDLUNG,NDLUXG,NDGUNG,NDGUXG,&
   & IFLDN,IFLDX,ISTALAT,IWIS,IHOR,IHVI,&
   & LLSLHD,LLSLHDQUAD,&
   & PLON,PLAT,ZLEV,&
   & ZUN_VETAF,ZUN_VETAH,IUN_VAUTF,&
   & ZUN_VCUICO,ZUN_VSLD,ZUN_VSLDW,1,1.0_JPRB,ZUN_KAPPA,&
   & ZDSLAT,ZCSLA,ZUN_CLASLD,ZDSLON,ZCSLO,ZUN_CLOSLD,&
   & IL0S,ILH0,ILEV,&
   & ZUN_DVER,ZUN_VINTW,ZUN_VINTWSLD,ZUN_VINTWS,&
   & ZUN_VDERW,ZUN_HVW,IDEP)  
ELSE
  CALL LASCAW(KASLB1,KPROMB,1,KPROMB,1,1,NDGSAH,NDGENH,&
   & IFLDN,ISTALAT,IWIS,IHOR,IHVI,&
   & LLSLHD,LLSLHDQUAD,&
   & R4JP,ZPIS2,ZLSDEPI,RLATI(NDGSAH),&
   & RIPI0(NDGSAH),RIPI1(NDGSAH),RIPI2(NDGSAH),&
   & ZUN_SLD1,ZUN_SLD2,ZUN_SLD3,ZUN_SLDW,&
   & PLON,PLAT,ZLEV,&
   & ZUN_VETAF,ZUN_VETAH,IUN_VAUTF,&
   & ZUN_VCUICO,ZUN_VSLD,ZUN_VSLDW,1,1.0_JPRB,ZUN_KAPPA,&
   & ZDSLAT,ZCSLA,ZUN_CLASLD,ZDSLON,ZCSLO,ZUN_CLOSLD,&
   & IL0S,ILH0,ILEV,&
   & ZUN_DVER,ZUN_VINTW,ZUN_VINTWSLD,ZUN_VINTWS,&
   & ZUN_VDERW,ZUN_HVW)
ENDIF

DO J=0,KASLB1
  ITRANS(J)=J
ENDDO

ILSB=MAX(NDGSAG,MYFRSTACTLAT-NSLWIDE)-NFRSTLOFF
ILFB=MIN(NDGENG,MYLSTACTLAT +NSLWIDE)-NFRSTLOFF
DO JL=ILSB,ILFB
  IF (NLOENG(JL+NFRSTLOFF) == NSLONL(JL)) THEN
    IST =NSLOFF(JL)-1
    IEND=NSLOFF(JL)+NLOENG(JL+NFRSTLOFF)
    ITRANS(IST+1)=IEND
    ITRANS(IEND+1)=IST+2
    ITRANS(IEND+2)=IST+3
  ENDIF
ENDDO

IF( IWIS == 201) THEN
  DO J1 = 1 , KPROMB
    IL0ULI(J1,1,1) = ITRANS(IL0S(J1,1,1)+1)
    IL0ULI(J1,1,2) = ITRANS(IL0S(J1,1,1)+2)
    IL0ULI(J1,2,1) = ITRANS(IL0S(J1,1,2)+1)
    IL0ULI(J1,2,2) = ITRANS(IL0S(J1,1,2)+2)
  ENDDO
ELSEIF( IWIS == 203) THEN

  DO J1 = 1 , KPROMB
    IL0UDI(J1,0,0) = 0
    IL0UDI(J1,0,1) = ITRANS(IL0S(J1,1,0)+1)
    IL0UDI(J1,0,2) = ITRANS(IL0S(J1,1,0)+2)
    IL0UDI(J1,0,3) = 0
    IL0UDI(J1,1,0) = ITRANS(IL0S(J1,1,1)+0)
    IL0UDI(J1,1,1) = ITRANS(IL0S(J1,1,1)+1)
    IL0UDI(J1,1,2) = ITRANS(IL0S(J1,1,1)+2)
    IL0UDI(J1,1,3) = ITRANS(IL0S(J1,1,1)+3)
    IL0UDI(J1,2,0) = ITRANS(IL0S(J1,1,2)+0)
    IL0UDI(J1,2,1) = ITRANS(IL0S(J1,1,2)+1)
    IL0UDI(J1,2,2) = ITRANS(IL0S(J1,1,2)+2)
    IL0UDI(J1,2,3) = ITRANS(IL0S(J1,1,2)+3)
    IL0UDI(J1,3,0) = 0
    IL0UDI(J1,3,1) = ITRANS(IL0S(J1,1,3)+1)
    IL0UDI(J1,3,2) = ITRANS(IL0S(J1,1,3)+2)
    IL0UDI(J1,3,3) = 0
  ENDDO

ENDIF
CALL GSTATS(1869,1)

!*          1.3    UPPER-AIR FIELDS INTERPOLATION+ PS + SURFACE FIELDS

IF(IWIS == 201)         THEN
  CALL LAIDLIOBSAD(KASLB1,KPROMB,1,KPROMB,KFIELDS,&
   & IFLDN,IFLDX,&
   & ZDSLAT(:,1),ZDSLON(1,1,1),IL0ULI,&
   & POBB1,PSL)  
ELSEIF(IWIS == 203)    THEN
  CALL LAIDDIOBSAD(KASLB1,KPROMB,1,KPROMB,KFIELDS,&
   & IFLDN,IFLDX,&
   & ZCSLA,ZDSLON,ZCSLO,IL0UDI,&
   & POBB1,PSL)  
ELSE
  WRITE(UNIT=NULERR,FMT='('' *** ERROR IN SLINTAD'')')
  WRITE(UNIT=NULERR,FMT='('' UNKNOWN INERPOLATION NO.: '',I10)')IWIS
  CALL ABOR1('SLINTAD:')
ENDIF

!**---------------------------------------------------------------------
999 FORMAT(' *** ERROR IN SLINTAD *** UNKNOWN INTERPOL. NO.: ',I10)
!**---------------------------------------------------------------------

!   ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SLINTAD',1,ZHOOK_HANDLE)
END SUBROUTINE SLINTAD
