SUBROUTINE TROPOPAUSE(KDLEN,KLEN,PTQOPP5,KLTROP)

!              KDLEN   = First dimension (Number of observations)
!              KLEN    = Actual number of observations
!              PTQOPP5 =Model T,Q and O3 on NLSAT pressure levels

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMTVRAD , ONLY : RADPRE
USE YOMDIMO  , ONLY : NLSAT, NPNAV
USE YOMCST   , ONLY : RKAPPA

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
REAL(KIND=JPRB)   ,INTENT(IN) :: PTQOPP5(KDLEN,NLSAT,NPNAV-1) 
INTEGER(KIND=JPIM), DIMENSION(KDLEN), INTENT(OUT) :: KLTROP
INTEGER(KIND=JPIM), PARAMETER :: JP_T=1

REAL(KIND=JPRB),PARAMETER           :: PLIMU=45000.0_JPRB, PLIML=7500.0_JPRB, GAMMA=-0.002_JPRB
REAL(KIND=JPRB)                     :: ZTRP

REAL(KIND=JPRB),PARAMETER           :: KAP=0.286_JPRB
REAL(KIND=JPRB),PARAMETER           :: FAKTOR = -9.81_JPRB/287.0_JPRB
REAL(KIND=JPRB),PARAMETER           :: DELTAZ = 2000.0_JPRB
REAL(KIND=JPRB),PARAMETER           :: KA1=KAP-1._JPRB

REAL (KIND=JPRB)                    :: ZPMK, ZPM, ZA, ZB, ZTM, ZDTDP, ZDTDZ
REAL(KIND=JPRB)                     :: ZAG, ZBG, ZPTPH
REAL (KIND=JPRB)                    :: ZPM0, ZPMK0, ZDTDZ0
REAL(KIND=JPRB)                     :: ZP2KM, ZASUM, ZAQUER
REAL(KIND=JPRB)                     :: ZPMK2, ZPM2, ZA2, ZB2, ZTM2, ZDTDP2, ZDTDZ2
INTEGER (KIND=JPIM)                 :: JLEV, JJLEV, ICOUNT, JOBS

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('TROPOPAUSE',0,ZHOOK_HANDLE)

ZTRP=-99.0                           ! negative means not valid
KLTROP(1:KLEN)=30
DO JOBS=1,KLEN
DO JLEV=NLSAT,2,-1

   ! dt/dz
   ZPMK= .5 * (RADPRE(JLEV-1)**RKAPPA+RADPRE(JLEV)**RKAPPA)
   ZPM = ZPMK**(1/RKAPPA)              
   ZA = (PTQOPP5(JOBS,JLEV-1,JP_T)-PTQOPP5(JOBS,JLEV,JP_T))/(RADPRE(JLEV-1)**RKAPPA-RADPRE(JLEV)**RKAPPA)
   ZB = PTQOPP5(JOBS,JLEV,JP_T)-(ZA*RADPRE(JLEV)**RKAPPA)
   ZTM = ZA * ZPMK + ZB              
   ZDTDP = ZA * RKAPPA * (ZPM**KA1)
   ZDTDZ = FAKTOR*ZDTDP*ZPM/ZTM

   ! dt/dz valid?
   IF (JLEV == NLSAT)  GO TO 999     ! no, start level, initialize first
   IF (ZDTDZ <= GAMMA) GO TO 999     ! no, dt/dz < -2 K/km
   IF (ZPM > PLIMU)    GO TO 999     ! no, too low

   ! zdtdz is valid, calculate tropopause pressure
   IF (ZDTDZ0 < GAMMA) THEN
      ZAG = (ZDTDZ-ZDTDZ0) / (ZPMK-ZPMK0)    
      ZBG = ZDTDZ0 - (ZAG * ZPMK0)         
      ZPTPH = EXP(LOG((GAMMA-ZBG)/ZAG)/RKAPPA)
   ELSE
      ZPTPH = ZPM
   ENDIF

   IF (ZPTPH < PLIML) GO TO 999    
   IF (ZPTPH > PLIMU) GO TO 999          

   ! 2nd test: dtdz above 2 km must not exceed gamma
   ZP2KM = ZPTPH + DELTAZ*(ZPM/ZTM)*FAKTOR          ! p at zptph + 2km
   ZASUM = 0.0                                   ! dtdz above
   ICOUNT = 0                                   ! number of levels above

   ! test until apm < zp2km
   DO JJLEV=JLEV,2,-1

       ZPMK2 = .5 * (RADPRE(JJLEV-1)**RKAPPA+RADPRE(JJLEV)**RKAPPA)    ! p mean ^RKAPPApa
       ZPM2 = ZPMK2**(1/RKAPPA)                      ! p mean
       IF(ZPM2 > ZPTPH) GO TO 110                ! doesn't happen
       IF(ZPM2 < ZP2KM) GO TO 888                ! ptropo is valid

       ZA2 = (PTQOPP5(JOBS,JJLEV-1,JP_T)-PTQOPP5(JOBS,JJLEV,JP_T))                     ! a
       ZA2 = ZA2/(RADPRE(JJLEV-1)**RKAPPA-RADPRE(JJLEV)**RKAPPA)
       ZB2 = PTQOPP5(JOBS,JJLEV,JP_T)-(ZA2*RADPRE(JJLEV)**RKAPPA)               ! b
       ZTM2 = ZA2 * ZPMK2 + ZB2                     ! T mean
       ZDTDP2 = ZA2 * RKAPPA * (ZPM2**(RKAPPA-1))        ! dt/dp
       ZDTDZ2 = FAKTOR*ZDTDP2*ZPM2/ZTM2
       ZASUM = ZASUM+ZDTDZ2
       ICOUNT = ICOUNT+1
       ZAQUER = ZASUM/FLOAT(ICOUNT)               ! dt/dz mean
  
       ! discard ptropo ?
        IF (ZAQUER <= GAMMA) GO TO 999           ! dt/dz above < gamma

110 CONTINUE
    ENDDO                           ! test next level

888 CONTINUE                        ! zptph is valid
    ZTRP = ZPTPH
    KLTROP(JOBS)=JLEV
    EXIT

999 CONTINUE                        ! continue search at next higher level
    ZPM0 = ZPM
    ZPMK0 = ZPMK
    ZDTDZ0  = ZDTDZ

ENDDO
ENDDO


IF (LHOOK) CALL DR_HOOK('TROPOPAUSE',1,ZHOOK_HANDLE)
END SUBROUTINE TROPOPAUSE
