!  Differentiation of vertdisc in forward (tangent) mode:
!   variations  of output variables: piz phasel ptaer_vert
!   with respect to input variables: ptaer_vertin phasel_in piz_in

SUBROUTINE VERTDISC_TL(pwl, ptray550_vertin, ptaer_vertin, ptaer_vertind, &
&  phasel_in, phasel_ind, piz_in, piz_ind, ptaer_vert, ptaer_vertd, &
&  ptray_vert, phasel, phaseld, piz, pizd)

  USE PARKIND1  ,ONLY : JPIM     ,JPRB
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  USE rt6svalues

  IMPLICIT NONE

! LOOP ON KLON
! LOOP ON WAVELENGTHS
!****VERTDISC*** ROUTINE THAT DISCRETICES THE VERTICAL INTO ISO-OPTICAL
!                DEPTH LAYERS. THIS IS DONE WITH RESPECT TO THE TOTAL 
!                OPTICAL DEPTH.
! The difference of this version with respect to the original one is that this one keeps
! the same values for the first layer (TOA) as the original profiles. It uses the rest 
! of the layers for the discretisation.

!    N.HUNEEUS ,  LOA 

!     INTERFACE
!     ---------
! VERTDISC IS CALLED IN THE MAIN ROUTINE 6S_GEMS

!     INPUTS
!     ------
! PWL(KWL)                     : Wavelengths at which the calculations are done (m).
! PTRAY550_VERTIN(KLON,KLEV)   : Molecular optical thickness profile at 0.55e-6 m.
! PTAER_VERTIN(KLON,KLEV,KWL)  : Aerosol optical thickness profile.
! PHASEL_IN(KLON,KLEV,KWL,KMU) : Aerosol phase function profile.
! PIZ_IN(KLON,KLEV,KWL)        : Aerosol single scattering albedo profile.

!     OUTPUTS
!     -------
! PTAER_VERT(KLON,KNT,KWL)     : Aerosol optical thickness profile.
! PTRAY_VERT(KLON,KNT,KWL)     : Molecular optical thickness profile
! PHASEL(KLON,KLEV,KWL,KMU)    : Aerosol phase function profile.
! PIZ(KLON,KLEV,KWL)           : Aerosol single scattering albedo profile.


!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------

!     SWITCHES.
!     --------

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
  REAL(KIND=JPRB), INTENT(IN) :: pwl(kwl)
  REAL(KIND=JPRB), INTENT(IN) :: ptray550_vertin(klon, 0:klev), &
&  ptaer_vertin(klon, 0:klev, kwl)
  REAL(KIND=JPRB), INTENT(IN) :: ptaer_vertind(klon, 0:klev, kwl)
  REAL(KIND=JPRB), INTENT(IN) :: phasel_in(klon, 0:klev, kwl, knbmu), &
&  piz_in(klon, 0:klev, kwl)
  REAL(KIND=JPRB), INTENT(IN) :: phasel_ind(klon, 0:klev, kwl, knbmu), &
&  piz_ind(klon, 0:klev, kwl)
  REAL(KIND=JPRB), INTENT(OUT) :: ptaer_vert(klon, 0:knt, kwl), &
&  ptray_vert(klon, 0:knt, kwl)
  REAL(KIND=JPRB), INTENT(OUT) :: ptaer_vertd(klon, 0:knt, kwl)
  REAL(KIND=JPRB), INTENT(OUT) :: phasel(klon, 0:knt, kwl, knbmu), piz(&
&  klon, 0:knt, kwl)
  REAL(KIND=JPRB), INTENT(OUT) :: phaseld(klon, 0:knt, kwl, knbmu), &
&  pizd(klon, 0:knt, kwl)
! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20, KWL=1 
  INTEGER(KIND=JPIM) :: jlev, jdiv, jlon, jwl, jaux, jcont, jmu
!INTEGER(KIND=JPIM),PARAMETER :: ICTE=5 !TOGETHER WITH KNT THEY DETERMINE THE NUMBER OF LAYERS EACH
!ORIGINAL LAYER WILL DIVIDED
  REAL(KIND=JPRB) :: ztray(0:(klev+1)*idiv-1), ztaer(0:(klev+1)*idiv-1)&
&  , ztray_vertin(0:klev)
  REAL(KIND=JPRB) :: ztaerd(0:(klev+1)*idiv-1)
  REAL(KIND=JPRB) :: zpha(0:(klev+1)*idiv-1, knbmu), zpiz(0:(klev+1)*&
&  idiv-1)
  REAL(KIND=JPRB) :: zphad(0:(klev+1)*idiv-1, knbmu), zpizd(0:(klev+1)*&
&  idiv-1)
  REAL(KIND=JPRB) :: zod_axis, ziso_od, zodnew_aer, zodnew_ray, zodnew
  REAL(KIND=JPRB) :: zodnew_aerd
  REAL(KIND=JPRB) :: zomega, zphase(knbmu)
  REAL(KIND=JPRB) :: zomegad, zphased(knbmu)
  REAL(KIND=JPRB) :: z_pwx1
  REAL(KIND=JPRB) :: z_pwr1
  INTRINSIC FLOAT

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  IF (LHOOK) CALL DR_HOOK('VERTDISC_TL',0,ZHOOK_HANDLE)


! INITIALIZATION OF VARIABLES
! ---------------------------
  ptaer_vertd(:, :, :) = 0._JPRB
  ptaer_vert(:, :, :) = 0.0
  ptray_vert(:, :, :) = 0.0
  phaseld(:, :, :, :) = 0._JPRB
  phasel(:, :, :, :) = 0.0
  pizd(:, :, :) = 0._JPRB
  piz(:, :, :) = 0.0
  pizd(1:klon, 0:knt, 1:kwl) = 0._JPRB
  phaseld(1:klon, 0:knt, 1:kwl, 1:knbmu) = 0._JPRB
  ptaer_vertd(1:klon, 0:knt, 1:kwl) = 0._JPRB
  DO jwl=1,kwl
    DO jlon=1,klon
! JLEV=0 CORRESPONDS TO THE TOP OF THE ATMOPSHERE. THE VARIABLES WITH THE NEW VERTICAL DISCRETIZATION

!  1. MOLECULAR OPTICAL THICKNESS AT DIFFERENT WAVELENGTHS 
!     ----------------------------------------------------

!    print *,'1. MOLECULAR OPTICAL THICKNESS AT DIFFERENT WAVELENGTHS'
      DO jlev=0,klev
        z_pwx1 = pwl(jwl)/pwl550
        z_pwr1 = z_pwx1**(-4)
        ztray_vertin(jlev) = ptray550_vertin(jlon, jlev)*z_pwr1
      ENDDO

! 2. ASIGNING VALUES OF THE FIRST LAYER TO THE NEW VARIABLES
!    -------------------------------------------------------

      ptaer_vertd(jlon, 0, jwl) = ptaer_vertind(jlon, 0, jwl)
      ptaer_vert(jlon, 0, jwl) = ptaer_vertin(jlon, 0, jwl)
      ptray_vert(jlon, 0, jwl) = ztray_vertin(0)
      pizd(jlon, 0, jwl) = piz_ind(jlon, 0, jwl)
      piz(jlon, 0, jwl) = piz_in(jlon, 0, jwl)
      DO jmu=1,knbmu
        phaseld(jlon, 0, jwl, jmu) = phasel_ind(jlon, 0, jwl, jmu)
        phasel(jlon, 0, jwl, jmu) = phasel_in(jlon, 0, jwl, jmu)
      ENDDO

! JLEV=0 CORRESPONDS TO THE TOP OF THE ATMOPSHERE. THE VARIABLES WITH THE NEW VERTICAL DISCRETIZATION

!  2. CALCULATING ISO-OPTICAL THICKNESS
!     ---------------------------------

      zod_axis = 0.0
      DO jlev=1,klev
        zod_axis = zod_axis + ptaer_vertin(jlon, jlev, jwl) + &
&          ztray_vertin(jlev)
      ENDDO
!    ZISO_OD=ZOD_AXIS/FLOAT(KNT+1)
      ziso_od = zod_axis/FLOAT(knt)

!  3. SPLITTING OF ORIGINAL LAYERS INTO IDIV LAYERS
!     ---------------------------------------------
!    print *,'2. SPLITTING OF ORIGINAL LAYERS INTO IDIV LAYERS'
      ztaerd(:) = 0._JPRB
      ztaer(:) = 0.000000001_JPRB 
      ztray(:) = 0._JPRB
      zpizd(:) = 0._JPRB
      zpiz(:) = 0.000000001_JPRB
      zphad(:, :) = 0._JPRB
      zpha(:, :) = 0.000000001_JPRB
      zpizd(0:(klev+1)*idiv-1) = 0._JPRB
      zphad(0:(klev+1)*idiv-1, 1:knbmu) = 0._JPRB
      ztaerd(0:(klev+1)*idiv-1) = 0._JPRB
      DO jlev=1,klev
        DO jdiv=1,idiv
          jaux = jlev*idiv + jdiv
          ztaerd(jaux-1) = ptaer_vertind(jlon, jlev, jwl)/idiv
          ztaer(jaux-1) = ptaer_vertin(jlon, jlev, jwl)/idiv
          ztray(jaux-1) = ztray_vertin(jlev)/idiv
          zpizd(jaux-1) = piz_ind(jlon, jlev, jwl)
          zpiz(jaux-1) = piz_in(jlon, jlev, jwl)
          DO jmu=1,knbmu
            zphad(jaux-1, jmu) = phasel_ind(jlon, jlev, jwl, jmu)
            zpha(jaux-1, jmu) = phasel_in(jlon, jlev, jwl, jmu)
          ENDDO
        ENDDO
      ENDDO

!  4. NEW VERTICAL DISCRETIZATION
!     ---------------------------

!    print *,'3. NEW VERTICAL DISCRETIZATION'
      zodnew_aer = 0.0
      zodnew_ray = 0.0
      zomega = 0.0
      zphased(:) = 0._JPRB
      zphase(:) = 0.0
      jcont = 1
      zodnew_aerd = 0._JPRB
      zphased(1:knbmu) = 0._JPRB
      zomegad = 0._JPRB
      DO jlev=1,idiv*(klev+1)-1
        zodnew = zodnew_aer + zodnew_ray
        zodnew_aerd = zodnew_aerd + ztaerd(jlev)
        zodnew_aer = zodnew_aer + ztaer(jlev)
        zodnew_ray = zodnew_ray + ztray(jlev)
        zomegad = zomegad + ztaerd(jlev)*zpiz(jlev) + ztaer(jlev)*zpizd(&
&          jlev)
        zomega = zomega + ztaer(jlev)*zpiz(jlev)
        DO jmu=1,knbmu
          zphased(jmu) = zphased(jmu) + zphad(jlev, jmu)*ztaer(jlev)*&
&            zpiz(jlev) + zpha(jlev, jmu)*(ztaerd(jlev)*zpiz(jlev)+ztaer&
&            (jlev)*zpizd(jlev))
          zphase(jmu) = zphase(jmu) + zpha(jlev, jmu)*ztaer(jlev)*zpiz(&
&            jlev)
        ENDDO
        ptaer_vertd(jlon, jcont, jwl) = zodnew_aerd
        ptaer_vert(jlon, jcont, jwl) = zodnew_aer
        ptray_vert(jlon, jcont, jwl) = zodnew_ray
        pizd(jlon, jcont, jwl) = (zomegad*zodnew_aer-zomega*zodnew_aerd)&
&          /zodnew_aer**2
        piz(jlon, jcont, jwl) = zomega/zodnew_aer
        DO jmu=1,knbmu
           phaseld(jlon, jcont, jwl, jmu) = (zphased(jmu)*zomega-zphase(&
&             jmu)*zomegad)/zomega**2
           phasel(jlon, jcont, jwl, jmu) = zphase(jmu)/zomega
        ENDDO
        IF (zodnew + ztaer(jlev) + ztray(jlev)  >   ziso_od) THEN
! Starting new layer   
          zodnew_aer = 0.0
          zodnew_ray = 0.0
          zomega = 0.0
          zphased(:) = 0._JPRB
          zphase(:) = 0.0
          jcont = jcont + 1
          zodnew_aerd = 0._JPRB
          zphased(1:knbmu) = 0._JPRB
          zomegad = 0._JPRB
        ENDIF
      ENDDO
    ENDDO
  ENDDO

 IF (LHOOK) CALL DR_HOOK('VERTDISC_TL',1,ZHOOK_HANDLE)
END SUBROUTINE VERTDISC_TL
