SUBROUTINE AROWRGP_SURF(CDHMODE,KGPTOT,PBUF,KGPTOTG,PGPG,KCOUNTW,&
& KMYPROC,KIOPROC,KNPROC,KNPRCIDS,KNTAG,KGPTOTLS,KGPTOTMX,KLUOUT)

!**** *WRGP2FA*  - 

!     Purpose.
!     --------
!        To send data -  or gather all data if writting proc

!**   Interface.
!     ----------
!        *CALL* *WRGP_SURF*

!        Explicit arguments :
!        --------------------

!            PBUF    : gridpoint field on one PROC
!            PGPG    : gridpoint field, size NGPTOTG
!                    if writting proc : contain complete field
!                    if not : full of zero.... 
!            KYPROC  :my proc number (usefull for the externalised
!                     surface to know if I am a writting proc!)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!           Processor communication routines
!           DIWRGRID

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!         S.Malardel *METEO-FRANCE*

!     Modifications.
!     --------------
!        ORIGINAL : 
!     ------------------------------------------------------------------

!     ------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE MPL_MODULE ,ONLY : MPL_RECV, MPL_BARRIER, MPL_SEND
USE YOMHOOK ,ONLY : LHOOK, DR_HOOK
 
IMPLICIT NONE
 
CHARACTER(LEN=6)                     ,INTENT(IN )   :: CDHMODE
INTEGER(KIND=JPIM)                   ,INTENT(IN )   :: KGPTOT
REAL(KIND=JPRB)                      ,INTENT(INOUT) :: PBUF(KGPTOT) 
INTEGER(KIND=JPIM)                   ,INTENT(IN )   :: KGPTOTG
REAL(KIND=JPRB)                      ,INTENT(OUT)   :: PGPG(KGPTOTG)
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KCOUNTW 
INTEGER(KIND=JPIM)                   ,INTENT(IN )   :: KMYPROC 
INTEGER(KIND=JPIM)                   ,INTENT(IN )   :: KIOPROC 
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KNPROC
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KNPRCIDS(KNPROC)
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KNTAG
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KGPTOTLS(KNPROC)
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KGPTOTMX
INTEGER(KIND=JPIM)                   ,INTENT(IN)    :: KLUOUT
!     ------------------------------------------------------------------
INTEGER(KIND=JPIM)                                  :: JP,J ! loop indexes
INTEGER(KIND=JPIM)                                  :: ILEN, ITAG, ILENB, ITAGR, ISENDER, IOFFSET
REAL(KIND=JPRB)                                     :: ZFLD(KGPTOTMX)
REAL(KIND=JPRB)                                     :: ZHOOK_HANDLE   
!     ------------------------------------------------------------------
#include "diwrgrid_surf_ext.intfb.h"
#include "abor1.intfb.h"
!     ------------------------------------------------------------------
IF(LHOOK) CALL DR_HOOK('AROWRGP_SURF',0,ZHOOK_HANDLE)

 
IF (CDHMODE == 'RUN   ') THEN

!     1.    SEND FIELD!           -----------

  IF (KMYPROC /= 1) THEN
    CALL DIWRGRID_SURF_EXT(1,KCOUNTW,0,KGPTOT,PBUF(1))
  ENDIF

!     2.    RECIEVE FIELD

  IF (KMYPROC == 1) THEN
    PGPG(1:KGPTOT)=PBUF(1:KGPTOT)
    CALL DIWRGRID_SURF_EXT(1,KCOUNTW,1,KGPTOTG,PGPG(1))
  ENDIF

!  IF(KNPROC > 1) CALL MPL_BARRIER(CDSTRING='AROWRGP_SURF:')
 
ELSEIF (CDHMODE == 'PRE   ') THEN
 
  !     send fields to output node  :  KIOPROC
  IF (KMYPROC /= KIOPROC) THEN
    IF (KGPTOT > 0) THEN
      ILEN=KGPTOT
      ITAG=KNTAG+KCOUNTW
      CALL MPL_SEND(PBUF(1:ILEN),KDEST=KNPRCIDS(KIOPROC),KTAG=ITAG, &
            & CDSTRING='AROWRGP_SURF:') 
    ENDIF
  ENDIF

  !     KIOPROC receives gridpoint field
  IF (KMYPROC == KIOPROC) THEN
    DO JP=1,KNPROC
      IOFFSET=0
      DO J=1,JP-1
        IOFFSET=IOFFSET+KGPTOTLS(J)
      ENDDO
      ITAG=KNTAG+KCOUNTW
      ILENB=KGPTOTMX
      IF (JP /= KIOPROC) THEN
        CALL MPL_RECV(ZFLD(1:ILENB),KSOURCE=KNPRCIDS(JP),KTAG=ITAG, &
             & KFROM=ISENDER,KRECVTAG=ITAGR,CDSTRING='AROWRGP_SURF:') 
      ELSE
        DO J=1,KGPTOT
          ZFLD(J)=PBUF(J)
        ENDDO
      ENDIF
         
      DO J=1,KGPTOTLS(JP)
        PGPG(IOFFSET+J)=ZFLD(J)
      ENDDO
    ENDDO
  ENDIF

  IF (KNPROC > 1) CALL MPL_BARRIER(CDSTRING='AROWRGP_SURF:')

ELSE
 
  WRITE(KLUOUT,*) ' '
  WRITE(KLUOUT,*) ' IO MODE DOES NOT EXIST YET'
  CALL ABOR1('AROWRGP_SURF: IO MODE DOES NOT EXIST YET ') 
ENDIF
 
!     ------------------------------------------------------------------
IF(LHOOK) CALL DR_HOOK('AROWRGP_SURF',1,ZHOOK_HANDLE)
END SUBROUTINE AROWRGP_SURF
