SUBROUTINE BCASTCOV(klev,ksmax,pcovt,pcovv,pcovd,pcovq,pcovo,kproc)

!     Purpose:
!     --------
!       Broadcast model error covariance data to all processors.

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original 12-May-2004
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMMP   , ONLY: MYPROC
USE YOMTAG  , ONLY: MTAGFCE
USE MPL_MODULE

IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(IN) :: klev,ksmax,kproc
REAL(KIND=JPRB), INTENT(INOUT) :: pcovt(klev+1,klev+1,0:ksmax)
REAL(KIND=JPRB), INTENT(INOUT) :: pcovv(klev,klev,0:ksmax)
REAL(KIND=JPRB), INTENT(INOUT) :: pcovd(klev,klev,0:ksmax)
REAL(KIND=JPRB), INTENT(INOUT) :: pcovq(klev,klev,0:ksmax)
REAL(KIND=JPRB), INTENT(INOUT) :: pcovo(klev,klev,0:ksmax)

REAL(KIND=JPRB), ALLOCATABLE :: zbuf(:)
INTEGER(KIND=JPIM) :: ii,ioff,ilen,itag
INTEGER(KIND=JPIM) :: ish1(1),ish3(3)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('BCASTCOV',0,ZHOOK_HANDLE)

ilen=(4*klev*klev+(klev+1)*(klev+1))*(ksmax+1)
ALLOCATE(zbuf(ilen))

! Pack data
! ---------
CALL GSTATS(1876,0)
IF (MYPROC==kproc) THEN
  ioff=0
  ii=(klev+1)*(klev+1)*(ksmax+1)
  ish1(1)=ii
  zbuf(ioff+1:ioff+ii)=RESHAPE(pcovt(:,:,:),ish1)
  ioff=ioff+ii

  ii=klev*klev*(ksmax+1)
  ish1(1)=ii
  zbuf(ioff+1:ioff+ii)=RESHAPE(pcovv(:,:,:),ish1)
  ioff=ioff+ii
  zbuf(ioff+1:ioff+ii)=RESHAPE(pcovd(:,:,:),ish1)
  ioff=ioff+ii
  zbuf(ioff+1:ioff+ii)=RESHAPE(pcovq(:,:,:),ish1)
  ioff=ioff+ii
  zbuf(ioff+1:ioff+ii)=RESHAPE(pcovo(:,:,:),ish1)
  ioff=ioff+ii

  IF (ioff/=ilen) CALL ABOR1('BCASTCOV: Error packing')
ENDIF
CALL GSTATS(1876,1)

! Broadcast data on all PEs
! -------------------------
itag=MTAGFCE+1
CALL GSTATS_BARRIER(921)
CALL GSTATS(920,0)
CALL MPL_BROADCAST(ZBUF,KROOT=kproc,KTAG=itag,CDSTRING='BCASTCOV:')
CALL GSTATS(920,1)
CALL GSTATS_BARRIER2(921)

! Unpack data
! -----------
CALL GSTATS(1876,0)
IF (MYPROC/=kproc) THEN
  ioff=0
  ii=(klev+1)*(klev+1)*(ksmax+1)
  ish3(1)=klev+1
  ish3(2)=klev+1
  ish3(3)=ksmax+1
  pcovt(:,:,:)=RESHAPE(zbuf(ioff+1:ioff+ii),ish3)
  ioff=ioff+ii

  ii=klev*klev*(ksmax+1)
  ish3(1)=klev
  ish3(2)=klev
  ish3(3)=ksmax+1
  pcovv(:,:,:)=RESHAPE(zbuf(ioff+1:ioff+ii),ish3)
  ioff=ioff+ii
  pcovd(:,:,:)=RESHAPE(zbuf(ioff+1:ioff+ii),ish3)
  ioff=ioff+ii
  pcovq(:,:,:)=RESHAPE(zbuf(ioff+1:ioff+ii),ish3)
  ioff=ioff+ii
  pcovo(:,:,:)=RESHAPE(zbuf(ioff+1:ioff+ii),ish3)
  ioff=ioff+ii

  IF (ioff/=ilen) CALL ABOR1('BCASTCOV: Error unpacking')
ENDIF
CALL GSTATS(1876,1)

DEALLOCATE(zbuf)

IF (LHOOK) CALL DR_HOOK('BCASTCOV',1,ZHOOK_HANDLE)
END SUBROUTINE BCASTCOV
