SUBROUTINE CASND1(KLOC,KTOT,KDIM,KTAI)

!**** *CASND1*  Communicate one array to the first processor

!     Purpose.
!     --------
!        Communicate one array to the first processor.

!**   Interface.
!     ----------
!        *CALL* *CASND1(KLOC,KTOT,KDIM,KTAI)

!        Explicit arguments : 
!        -------------------- 
!          - KLOC: array from each processor
!          - KTOT: array sum of each KLOC
!          - KDIM: size of KLOC AND KTOT
!          - KTAI: tag for message passing

!        Implicit arguments :
!        --------------------

!     Author.
!     -------
!        MPP Group Meteo-France

!     Modifications.
!     --------------
!        Original:   98-02-12 
!        Modified: D.Dent : 09/00  MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPROC
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMTAG   , ONLY : MTAGGSUM
USE MPL_MODULE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOC(KDIM)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTOT(KDIM) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTAI 
INTEGER(KIND=JPIM) ::I_KBUF(KDIM)

INTEGER(KIND=JPIM) :: IBFLEN, IMSGLEN, ITAG1, JPROC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CASND1',0,ZHOOK_HANDLE)
IBFLEN=KDIM

IF ( NPROC > 1 ) THEN

  ITAG1=MTAGGSUM+KTAI
  IF (MYPROC == 1) THEN
    KTOT(:)=KLOC(:)
    DO JPROC = 1,NPROC-1
      CALL MPL_RECV(I_KBUF(1:IBFLEN),KTAG=ITAG1, &
       & KOUNT=IMSGLEN,CDSTRING='CASND1 :')  
      IF (IMSGLEN /= IBFLEN) CALL ABOR1('CASND1 : RECEIVED'//&
       & ' MESSAGE LENGTH OF WRONG SIZE')  
      KTOT(:)=KTOT(:)+I_KBUF(:)
    ENDDO
  ELSE
    CALL MPL_SEND(KLOC(1:KDIM),KDEST=NPRCIDS(1),KTAG=ITAG1, &
     & CDSTRING='CASND1 :')  
  ENDIF

ELSE
  KTOT(:)=KLOC(:)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASND1',1,ZHOOK_HANDLE)
END SUBROUTINE CASND1
