SUBROUTINE CASNDR1(PLOC,PTOT,KDIM,KTAI)

!**** *CASNDR1*  Communicate one real array to the first processor

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *CASNDR1(PLOC,PTOT,KDIM,KTAI)

!        Explicit arguments : 
!        -------------------- 

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group Meteo-France

!     Modifications.
!     --------------
!        Original: 98-02-19 
!        Modified: D.Dent : 00-09-01 MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPROC
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMTAG   , ONLY : MTAGGSUM
USE MPL_MODULE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOC(KDIM)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTOT(KDIM) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTAI 
REAL(KIND=JPRB) ::ZBUF(KDIM)

INTEGER(KIND=JPIM) :: IBFLEN, IMSGLEN, ITAG1, JROC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CASNDR1',0,ZHOOK_HANDLE)
IBFLEN=KDIM

IF ( NPROC > 1 ) THEN

  ITAG1=MTAGGSUM+KTAI
  IF (MYPROC == 1) THEN
    PTOT(:)=PLOC(:)
    DO JROC = 1,NPROC-1
      CALL MPL_RECV(ZBUF(1:IBFLEN),KTAG=ITAG1, &
       & KOUNT=IMSGLEN,CDSTRING='CASNDR1 :')  
      IF (IMSGLEN /= IBFLEN) CALL ABOR1('CASNDR1 : RECEIVED'//&
       & ' MESSAGE LENGTH OF WRONG SIZE')  
      PTOT(:)=PTOT(:)+ZBUF(:)
    ENDDO
  ELSE
    CALL MPL_SEND(PLOC(1:KDIM),KDEST=NPRCIDS(1),KTAG=ITAG1, &
     & CDSTRING='CASNDR1 :')  
  ENDIF
ELSE
  PTOT(:)=PLOC(:)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CASNDR1',1,ZHOOK_HANDLE)
END SUBROUTINE CASNDR1
