!OPTION! -pvctl noouterunroll -pvctl noouterstrip
SUBROUTINE COMMJBBAL(KLEV,KSMAX,KSPEC2,PFACT1,PFACT2,PSDIV,PSTPS,PSO3)

!**** *COMMJBBAL* Communicates GSA data

!**   Interface.
!     ----------
!        *CALL* *COMMJBBAL(..) 

!     Explicit arguments : CDJBTYPE                       - config string
!     -------------------- KLEV,KSMAX,KSPEC2              - dimensions
!                          PFACT1,PFACT2,PSDIV,PSTPS,PSO3 - data

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Externals:
!     ---------
!        MPL_BROADCAST -  communication interface
!     Author.
!     -------
!        F. Bouttier and M. Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original: 96-11-13 
!        E. Holm   97-09-25: Included ozone
!        D.Dent    00-09-01: MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      E. Sevault 18-May-2009 : SX9 compiler workaround : optimisations
!            "outerunroll" and "outerstrip" manually switched off to avoid
!            erroneous numerical variations.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY :   MYPROC
USE YOMTAG   , ONLY : MTAGFCE
USE YOMJG    , ONLY : SPJB_VARS_INFO
USE YOMGRB   , ONLY : NGRBO3
USE MPL_MODULE, ONLY : MPL_BROADCAST

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEC2 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFACT1(KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFACT2(KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSDIV(KLEV,KLEV,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTPS(KLEV+1,2*KLEV,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSO3(KLEV,KLEV,0:KSMAX) 
REAL(KIND=JPRB),ALLOCATABLE :: ZBUF(:)

INTEGER(KIND=JPIM) :: IBFLENR, IOFF, IOMASTER,&
 & ISZD, ISZF, ISZT, ITAG1  
LOGICAL :: LLO3

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('COMMJBBAL',0,ZHOOK_HANDLE)

LLO3 = ANY(SPJB_VARS_INFO(:)%IGRIBCODE==NGRBO3)


IF(LLO3)THEN
  IBFLENR=KSPEC2+KSPEC2+2*KLEV*KLEV*(KSMAX+1)+(KLEV+1)*(2*KLEV)*(KSMAX+1)
ELSE
  IBFLENR=KSPEC2+KSPEC2+  KLEV*KLEV*(KSMAX+1)+(KLEV+1)*(2*KLEV)*(KSMAX+1)
ENDIF

ALLOCATE(ZBUF(IBFLENR))

ISZF = SIZE(PFACT1)
ISZD = SIZE(PSDIV)
ISZT = SIZE(PSTPS)

IOMASTER = 1
ITAG1 = MTAGFCE+2

CALL GSTATS(1821,0)
IF (MYPROC == IOMASTER) THEN

  IOFF = 0
  ZBUF(IOFF+1:IOFF+ISZF) = PFACT1(:)
  IOFF = IOFF+ISZF
  ZBUF(IOFF+1:IOFF+ISZF) = PFACT2(:)
  IOFF = IOFF+ISZF
  ZBUF(IOFF+1:IOFF+ISZD) =RESHAPE(PSDIV,SHAPE(ZBUF(IOFF+1:IOFF+ISZD)))
  IOFF = IOFF+ISZD
  ZBUF(IOFF+1:IOFF+ISZT) =RESHAPE(PSTPS,SHAPE(ZBUF(IOFF+1:IOFF+ISZT)))
  IOFF = IOFF+ISZT
  IF(LLO3)THEN
    ZBUF(IOFF+1:IOFF+ISZD) =RESHAPE(PSO3,SHAPE(ZBUF(IOFF+1:IOFF+ISZD)))
    IOFF = IOFF+ISZD
  ENDIF

  IF(IOFF /= IBFLENR) THEN
    WRITE(NULOUT,*)' COMMJBBAL: IOFF /= IBFLENR ',IOFF,IBFLENR
    CALL ABOR1('COMMJBBAL: IOFF /= IBFLENR ')
  ENDIF

ENDIF
CALL GSTATS(1821,1)

CALL GSTATS_BARRIER(775)
CALL GSTATS(664,0)
CALL MPL_BROADCAST(ZBUF(1:IBFLENR),KROOT=IOMASTER,KTAG=ITAG1, &
 & CDSTRING='COMMJBBAL:')  
CALL GSTATS(664,1)
CALL GSTATS_BARRIER2(775)

CALL GSTATS(1822,0)
IF (MYPROC /= IOMASTER) THEN
  IOFF = 0
  PFACT1(:) = ZBUF(IOFF+1:IOFF+ISZF)
  IOFF = IOFF+ISZF
  PFACT2(:) = ZBUF(IOFF+1:IOFF+ISZF)
  IOFF = IOFF+ISZF
  PSDIV(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZD),SHAPE(PSDIV))
  IOFF = IOFF+ISZD
  PSTPS(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZT),SHAPE(PSTPS))
  IOFF = IOFF+ISZT
  IF(LLO3)THEN
    PSO3(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZD),SHAPE(PSO3))
    IOFF = IOFF+ISZD
  ENDIF
ENDIF
CALL GSTATS(1822,1)

IF (ALLOCATED(ZBUF)) DEALLOCATE(ZBUF)

IF (LHOOK) CALL DR_HOOK('COMMJBBAL',1,ZHOOK_HANDLE)
END SUBROUTINE COMMJBBAL
