!OPTION! -pvctl noouterunroll -pvctl noouterstrip
SUBROUTINE COMMJBDAT(CDJBTYPE,KLEV,KSMAX,PPDAT,&
 & PFCOVTPS,PFCOVQ,PFCOVD,PFCOVP,PFCOVO3)  

!**** *COMMJBDAT* Communicates GSA data

!**   Interface.
!     ----------
!        *CALL* *COMMJBDAT(..) 

!     Explicit arguments : CDJBTYPE               - config string
!     -------------------- KLEV,KSMAX             - dimensions
!                          PPDAT,PFCOVTPS,PFCOVQ,
!                          PFCOVD,PFCOVP,PFCOVO3  - data
!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Externals.
!     ----------
!        MPL_BARRIER, MPL_BROADCAST -   communication interface

!     Author.
!     -------
!        M. Hamrud  *ECMWF*

!     Modifications.
!     --------------
!       Original : 96-07-26 
!       F.Bouttier 96-11-12 : cater for STABAL96 Jb configuration
!       E.Holm     97-09-25 : included ozone
!       D.Dent     00-09-01 : MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      E. Sevault 18-May-2009 : SX9 compiler workaround : optimisations
!            "outerunroll" and "outerstrip" manually switched off to avoid
!            erroneous numerical variations.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : MYPROC
USE YOMTAG   , ONLY : MTAGFCE
USE YOMJG    , ONLY : SPJB_VARS_INFO
USE YOMGRB   , ONLY : NGRBO3
USE MPL_MODULE, ONLY : MPL_BARRIER, MPL_BROADCAST

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSMAX 
CHARACTER(LEN=10) ,INTENT(IN)    :: CDJBTYPE 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPDAT(KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCOVTPS(KLEV+1,KLEV+1,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCOVQ(KLEV,KLEV,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCOVD(KLEV,KLEV,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCOVP(KLEV,KLEV,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCOVO3(KLEV,KLEV,0:KSMAX) 
REAL(KIND=JPRB),ALLOCATABLE :: ZBUF(:)

INTEGER(KIND=JPIM) :: IBFLENR, IOFF, IOMASTER,&
 & ISZ0, ISZ1, ISZD, ITAG1  

LOGICAL :: LLO3

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('COMMJBDAT',0,ZHOOK_HANDLE)

LLO3 = ANY(SPJB_VARS_INFO(:)%IGRIBCODE==NGRBO3)


IBFLENR=(KLEV+1)+(KLEV+1)*(KLEV+1)*(KSMAX+1)+KLEV*KLEV*(KSMAX+1)
IF(CDJBTYPE=='STABAL96')THEN
  IF(LLO3)THEN
    IBFLENR=IBFLENR+3*KLEV*KLEV*(KSMAX+1)
  ELSE
    IBFLENR=IBFLENR+2*KLEV*KLEV*(KSMAX+1)
  ENDIF
ENDIF

ALLOCATE(ZBUF(IBFLENR))

ISZD = SIZE(PPDAT)
ISZ1 = SIZE(PFCOVTPS)
ISZ0 = SIZE(PFCOVQ)

IOMASTER = 1
ITAG1 = MTAGFCE+2

IF (MYPROC == IOMASTER) THEN

  CALL GSTATS(1820,0)
  IOFF = 0
  ZBUF(IOFF+1:IOFF+ISZD) = PPDAT(:)
  IOFF = IOFF+ISZD
  ZBUF(IOFF+1:IOFF+ISZ1) =RESHAPE(PFCOVTPS,SHAPE(ZBUF(IOFF+1:IOFF+ISZ1)))
  IOFF = IOFF+ISZ1
  ZBUF(IOFF+1:IOFF+ISZ0) =RESHAPE(PFCOVQ,SHAPE(ZBUF(IOFF+1:IOFF+ISZ0)))
  IOFF = IOFF+ISZ0
  IF(CDJBTYPE=='STABAL96') THEN
    ZBUF(IOFF+1:IOFF+ISZ0) =RESHAPE(PFCOVD,SHAPE(ZBUF(IOFF+1:IOFF+ISZ0)))
    IOFF = IOFF+ISZ0
    ZBUF(IOFF+1:IOFF+ISZ0) =RESHAPE(PFCOVP,SHAPE(ZBUF(IOFF+1:IOFF+ISZ0)))
    IOFF = IOFF+ISZ0
    IF(LLO3)THEN
      ZBUF(IOFF+1:IOFF+ISZ0) =RESHAPE(PFCOVO3,SHAPE(ZBUF(IOFF+1:IOFF+ISZ0)))
      IOFF = IOFF+ISZ0
    ENDIF
  ENDIF
  CALL GSTATS(1820,1)

  IF(IOFF /= IBFLENR) THEN
    WRITE(NULOUT,*)' COMMJBDAT: IOFF /= IBFLENR ',IOFF,IBFLENR
    CALL ABOR1('COMMJBDAT: IOFF /= IBFLENR ')
  ENDIF

ENDIF

CALL GSTATS_BARRIER(774)
CALL GSTATS(663,0)
CALL MPL_BROADCAST(ZBUF(1:IBFLENR),KROOT=IOMASTER,KTAG=ITAG1, &
 & CDSTRING='COMMJBDAT:')  
CALL GSTATS(663,1)
CALL GSTATS_BARRIER2(774)

IF (MYPROC /= IOMASTER) THEN
  CALL GSTATS(1819,0)
  IOFF = 0
  PPDAT(:) = ZBUF(IOFF+1:IOFF+ISZD)
  IOFF = IOFF+ISZD
  PFCOVTPS(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZ1),SHAPE(PFCOVTPS))
  IOFF = IOFF+ISZ1
  PFCOVQ(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZ0),SHAPE(PFCOVQ))
  IOFF = IOFF+ISZ0
  IF(CDJBTYPE=='STABAL96') THEN
    PFCOVD(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZ0),SHAPE(PFCOVD))
    IOFF = IOFF+ISZ0
    PFCOVP(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZ0),SHAPE(PFCOVP))
    IOFF = IOFF+ISZ0
    IF(LLO3)THEN
      PFCOVO3(:,:,:) =RESHAPE(ZBUF(IOFF+1:IOFF+ISZ0),SHAPE(PFCOVO3))
      IOFF = IOFF+ISZ0
    ENDIF
  ENDIF
  CALL GSTATS(1819,1)
ENDIF

IF (ALLOCATED(ZBUF)) DEALLOCATE(ZBUF)

CALL GSTATS(706,0)
CALL MPL_BARRIER(CDSTRING='COMMJBDAT:')
CALL GSTATS(706,1)

IF (LHOOK) CALL DR_HOOK('COMMJBDAT',1,ZHOOK_HANDLE)
END SUBROUTINE COMMJBDAT
