SUBROUTINE COMMSPNORM(LDPRTAL,LDOROG,KSMAX,&
 & KIOMASTER,KFTHER,KNUMSPFLDS,&
 & PNVOR  ,PNVORG  ,PMVOR  ,PMVORG,&
 & PNDIV  ,PNDIVG  ,PMDIV  ,PMDIVG,&
 & PNKE   ,PNKEG   ,PMKE   ,PMKEG,&
 & PNTHE  ,PNTHEG  ,PMTHE  ,PMTHEG,&
 & PNGFL  ,PNGFLG  ,PMGFL  ,PMGFLG,&
 & PNSP   ,PNSPG   ,PMSP   ,PMSPG,&
 & PNSPOR ,PNSPORG ,PMSPOR ,PMSPORG,&
 & PNTHEDE,PNTHEDEG,PMTHEDE,PMTHEDEG,&
 & PNGFLDE,PNGFLDEG,PMGFLDE,PMGFLDEG,&
 & PNSPDE ,PNSPDEG ,PMSPDE ,PMSPDEG)  

!**** *COMMSPNORM* Communicates spectral norm data

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *COMMSPNORM(...)

!        Explicit arguments : 
!        -------------------- LDPRTAL - .T. if highest level output is required (input)
!                             LDOROG  - .T. if orography has to be communicated (input)
!                             KSMAX   - truncation
!                             KFTHER  - Number of thermodynamic fields
!                             KNUMSPFLDS  - Number of spectral gfl fields
!                             KIOMASTER - processor collecting global data
!                             PNxxx   - local n wave data   (input)
!                             PNxxxG  - global n wave data  (output)
!                             PMxxx   - local m wave data   (input)
!                             PMxxxG  - global m wave data  (output)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        MPL_SEND, MPL_RECV  -  communication interface

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        L. Isaksen  *ECMWF*

!     Modifications.
!     --------------
!        Original: 97-02-02 
!        R. El Khatib 99-02-08 LDNWAVE, KSMAX, KIOMASTER
!        D.Dent       00-09-01 MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad: Apr 2005 Bug corrections.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NSPPR    ,NPROC
USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG   ,NUMP
USE YOMMP    , ONLY :   MYPROC   ,NUMPP    ,NPTRMS   ,&
 & NALLMS   ,NUMLL    ,NPTRLL   ,NPSURF   ,NPSP  
USE YOMTAG   , ONLY : MTAGSPNO
USE MPL_MODULE

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSMAX 
LOGICAL           ,INTENT(IN)    :: LDPRTAL 
LOGICAL           ,INTENT(IN)    :: LDOROG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIOMASTER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFTHER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMSPFLDS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNVOR(NFLEVL,NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNVORG(NFLEVG,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMVOR(NFLEVL,NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMVORG(NFLEVG,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNDIV(NFLEVL,NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNDIVG(NFLEVG,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMDIV(NFLEVL,NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMDIVG(NFLEVG,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNKE(NFLEVL,NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNKEG(NFLEVG,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMKE(NFLEVL,NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMKEG(NFLEVG,0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNTHE(NFLEVL,NUMP,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNTHEG(NFLEVG,0:KSMAX,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMTHE(NFLEVL,NUMP,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMTHEG(NFLEVG,0:KSMAX,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNGFL(NFLEVL,NUMP,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNGFLG(NFLEVG,0:KSMAX,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMGFL(NFLEVL,NUMP,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMGFLG(NFLEVG,0:KSMAX,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNSP(NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNSPG(0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMSP(NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMSPG(0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNSPOR(NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNSPORG(0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMSPOR(NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMSPORG(0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNTHEDE(NFLEVL,NUMP,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNTHEDEG(NFLEVG,0:KSMAX,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMTHEDE(NFLEVL,NUMP,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMTHEDEG(NFLEVG,0:KSMAX,KFTHER) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNGFLDE(NFLEVL,NUMP,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNGFLDEG(NFLEVG,0:KSMAX,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMGFLDE(NFLEVL,NUMP,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMGFLDEG(NFLEVG,0:KSMAX,KNUMSPFLDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNSPDE(NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNSPDEG(0:KSMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMSPDE(NUMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMSPDEG(0:KSMAX) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSENDBUF(((3+2*KFTHER+KNUMSPFLDS+1)*NFLEVL+3*NPSP)*NUMP)
REAL(KIND=JPRB),ALLOCATABLE :: ZRECVBUF(:)
INTEGER(KIND=JPIM) :: IADDTAG=0

INTEGER(KIND=JPIM) :: IA, IB, IBUFLENR, IBUFLENS, ILEV, IM, IOFF, IOFFDIV, IOFFKE, &
 & IOFFSP, IOFFSPDE, IOFFSPOR, IOFFTHE, &
 & IOFFTHEDE, IOFFGFLDE,  IOFFVOR, IRECVLEV, IRECVNUMP, &
 & IRECVSETA, IRECVSETB, ISTLEV, ISTOTAL, &
 & ISZPS, ISZPSOR, ISZSP, ISZSPR, ISZTHE, &
 & ITAG, ITOTAL, JF, JL, JMLOC, JROC,IOFFB,ISZGFL,IOFFGFL  
INTEGER(KIND=JPIM) :: IRECVCOUNTS(NPROC)

LOGICAL :: LLHASPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "pe2set.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COMMSPNORM',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------


IBUFLENS=SIZE(ZSENDBUF)

ITAG = MTAGSPNO+IADDTAG
IADDTAG=MOD(IADDTAG+1,100)

! ------------------------------------------------------------------

!     1.    Shift N-fields from 1..NUMP to 0..KSMAX
!           ---------------------------------------

IF (NPROC == 1.AND.NSPPR >= 2) THEN
  PNVORG(:,0:KSMAX) = PNVOR(:,1:NUMP)
  PNDIVG(:,0:KSMAX) = PNDIV(:,1:NUMP)
  PNKEG(:,0:KSMAX) = PNKE(:,1:NUMP)
  IF (KFTHER > 0) THEN
    PNTHEG(:,0:KSMAX,:) = PNTHE(:,1:NUMP,:)
  ENDIF
  IF (KNUMSPFLDS > 0) THEN
    PNGFLG(:,0:KSMAX,:) = PNGFL(:,1:NUMP,:)
  ENDIF
  PNSPG(0:KSMAX) = PNSP(1:NUMP)
  IF (LDOROG) PNSPORG(0:KSMAX) = PNSPOR(1:NUMP)
  IF (LDPRTAL) THEN
    IF (KFTHER > 0) THEN
      PNTHEDEG(:,0:KSMAX,:) = PNTHEDE(:,1:NUMP,:)
    ENDIF
    IF (KNUMSPFLDS > 0) THEN
      PNGFLDEG(:,0:KSMAX,:) = PNGFLDE(:,1:NUMP,:)
    ENDIF
    PNSPDEG(0:KSMAX) = PNSPDE(1:NUMP)
  ENDIF
ENDIF

! ------------------------------------------------------------------

!     2.    Send local data to the KIOMASTER PE
!           ----------------------------------

ISZSP = SIZE(PMVOR)

IOFF = 0
ZSENDBUF(IOFF+1:IOFF+ISZSP) =&
 & RESHAPE(PMVOR,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZSP)))  
IOFF = IOFF+ISZSP

ZSENDBUF(IOFF+1:IOFF+ISZSP) =&
 & RESHAPE(PMDIV,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZSP)))  
IOFF = IOFF+ISZSP

ZSENDBUF(IOFF+1:IOFF+ISZSP) =RESHAPE(PMKE,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZSP)))
IOFF = IOFF+ISZSP

IF (KFTHER > 0) THEN
  ISZTHE = SIZE(PMTHE)
  ZSENDBUF(IOFF+1:IOFF+ISZTHE) =&
   & RESHAPE(PMTHE,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZTHE)))  
  IOFF = IOFF+ISZTHE
ENDIF

IF (KNUMSPFLDS > 0) THEN
  ISZGFL = SIZE(PMGFL)
  ZSENDBUF(IOFF+1:IOFF+ISZGFL) =&
   & RESHAPE(PMGFL,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZGFL)))  
  IOFF = IOFF+ISZGFL
ENDIF

IF (NPSP == 1) THEN
  ISZPS = SIZE(PMSP)
  ZSENDBUF(IOFF+1:IOFF+ISZPS) =&
   & RESHAPE(PMSP,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZPS)))  
  IOFF = IOFF+ISZPS
ENDIF

IF (NPSP == 1.AND.LDOROG) THEN
  ISZPSOR = SIZE(PMSPOR)
  ZSENDBUF(IOFF+1:IOFF+ISZPSOR) =&
   & RESHAPE(PMSPOR,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZPSOR)))  
  IOFF = IOFF+ISZPSOR
ENDIF

IF (LDPRTAL) THEN
  IF (KFTHER > 0) THEN
    ISZTHE = SIZE(PMTHEDE)
    ZSENDBUF(IOFF+1:IOFF+ISZTHE) =&
     & RESHAPE(PMTHEDE,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZTHE)))  
    IOFF = IOFF+ISZTHE
  ENDIF
  IF (KNUMSPFLDS > 0) THEN
    ISZGFL = SIZE(PMGFLDE)
    ZSENDBUF(IOFF+1:IOFF+ISZGFL) =&
     & RESHAPE(PMGFLDE,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZGFL)))  
    IOFF = IOFF+ISZGFL
  ENDIF

  IF (NPSP == 1) THEN
    ISZPS = SIZE(PMSPDE)
    ZSENDBUF(IOFF+1:IOFF+ISZPS) =&
     & RESHAPE(PMSPDE,SHAPE(ZSENDBUF(IOFF+1:IOFF+ISZPS)))  
    IOFF = IOFF+ISZPS
  ENDIF
ENDIF

ISTOTAL=IOFF

IF (ISTOTAL > IBUFLENS) CALL ABOR1(' COMMSPNORM : SEND BUFFER '&
 & // 'TOO SMALL ')  

IF (NPROC > 1) THEN
  IBUFLENR=0
  DO JROC=1,NPROC
    CALL PE2SET(JROC,IA,IB,IRECVSETA,IRECVSETB)
    IRECVLEV=NUMLL(IRECVSETB)
    IRECVNUMP=NUMPP(IRECVSETA)
    LLHASPS=NPSURF(IRECVSETB) == 1

    ISTLEV=NPTRLL(IRECVSETB)
    ISZSPR=IRECVLEV*IRECVNUMP

    IOFFVOR =  0
    IOFFDIV = IOFFVOR+IRECVLEV*IRECVNUMP
    IOFFKE  = IOFFDIV+IRECVLEV*IRECVNUMP
    IOFFTHE = IOFFKE+IRECVLEV*IRECVNUMP
    IOFFGFL = IOFFTHE+IRECVLEV*IRECVNUMP*KFTHER
    IOFFSP  = IOFFGFL+IRECVLEV*IRECVNUMP*KNUMSPFLDS
    IF (LDOROG) THEN
      IOFFSPOR = IOFFSP+IRECVNUMP*NPSURF(IRECVSETB)
    ELSE
      IOFFSPOR  = IOFFSP
    ENDIF
    IF (LDPRTAL) THEN
      IOFFTHEDE = IOFFSPOR+IRECVNUMP*NPSURF(IRECVSETB)
      IOFFGFLDE = IOFFTHEDE+IRECVLEV*IRECVNUMP*KFTHER
      IOFFSPDE  = IOFFGFLDE+IRECVLEV*IRECVNUMP*KNUMSPFLDS
      ITOTAL    = IOFFSPDE+IRECVNUMP*NPSURF(IRECVSETB)
    ELSE
      ITOTAL    = IOFFSPOR+IRECVNUMP*NPSURF(IRECVSETB)
    ENDIF
    IRECVCOUNTS(JROC) = ITOTAL
    IBUFLENR=IBUFLENR+ITOTAL
  ENDDO
  ALLOCATE(ZRECVBUF(IBUFLENR))
  CALL GSTATS_BARRIER(773)
  CALL GSTATS_BARRIER(702) !Second GSTATS barrier to test "real" time of barrier
  CALL GSTATS(607,0)
  CALL MPL_GATHERV(ZSENDBUF(1:ISTOTAL),KROOT=KIOMASTER,&
   & PRECVBUF=ZRECVBUF,KRECVCOUNTS=IRECVCOUNTS, &
   & CDSTRING='COMMSPNORM:')  
  CALL GSTATS(607,1)
  CALL GSTATS_BARRIER2(773)
ELSE
  IRECVCOUNTS(1) = ISTOTAL
  IBUFLENR=ISTOTAL
  ALLOCATE(ZRECVBUF(IBUFLENR))
  ZRECVBUF(1:ISTOTAL)=ZSENDBUF(1:ISTOTAL)
ENDIF

! ------------------------------------------------------------------

!     3.    Receive and store data in global version. Local data 
!           is treated as if it was received from another PE
!           ------------------------------------------------

IF (MYPROC == KIOMASTER) THEN
  CALL GSTATS(1931,0)
  IOFFB=0
  DO JROC=1,NPROC

    CALL PE2SET(JROC,IA,IB,IRECVSETA,IRECVSETB)
    IRECVLEV=NUMLL(IRECVSETB)
    IRECVNUMP=NUMPP(IRECVSETA)
    LLHASPS=NPSURF(IRECVSETB) == 1

    ISTLEV=NPTRLL(IRECVSETB)
    ISZSPR=IRECVLEV*IRECVNUMP

    IOFFVOR =  0
    IOFFDIV = IOFFVOR+IRECVLEV*IRECVNUMP
    IOFFKE  = IOFFDIV+IRECVLEV*IRECVNUMP
    IOFFTHE = IOFFKE+IRECVLEV*IRECVNUMP
    IOFFGFL = IOFFTHE+IRECVLEV*IRECVNUMP*KFTHER
    IOFFSP  = IOFFGFL+IRECVLEV*IRECVNUMP*KNUMSPFLDS
    IF (LDOROG) THEN
      IOFFSPOR = IOFFSP+IRECVNUMP*NPSURF(IRECVSETB)
    ELSE
      IOFFSPOR  = IOFFSP
    ENDIF
    IF (LDPRTAL) THEN
      IOFFTHEDE = IOFFSPOR+IRECVNUMP*NPSURF(IRECVSETB)
      IOFFGFLDE = IOFFTHEDE+IRECVLEV*IRECVNUMP*KFTHER
      IOFFSPDE  = IOFFGFLDE+IRECVLEV*IRECVNUMP*KNUMSPFLDS
    ENDIF

    DO JMLOC=1,IRECVNUMP
      IM=NALLMS(NPTRMS(IRECVSETA)-1+JMLOC)
      DO JL=1,IRECVLEV
        ILEV=ISTLEV+JL-1
        PMVORG(ILEV,IM)=ZRECVBUF(IOFFB+IOFFVOR+(JMLOC-1)*IRECVLEV+JL)
        PMDIVG(ILEV,IM)=ZRECVBUF(IOFFB+IOFFDIV+(JMLOC-1)*IRECVLEV+JL)
        PMKEG (ILEV,IM)=ZRECVBUF(IOFFB+IOFFKE +(JMLOC-1)*IRECVLEV+JL)
        DO JF=1,KFTHER
          PMTHEG(ILEV,IM,JF)=ZRECVBUF(IOFFB+IOFFTHE+(JF-1)*&
           & IRECVLEV*IRECVNUMP+(JMLOC-1)*IRECVLEV+JL)  
        ENDDO
        DO JF=1,KNUMSPFLDS
          PMGFLG(ILEV,IM,JF)=ZRECVBUF(IOFFB+IOFFGFL+(JF-1)*&
           & IRECVLEV*IRECVNUMP+(JMLOC-1)*IRECVLEV+JL)  
        ENDDO
      ENDDO
      IF (LLHASPS) THEN
        PMSPG(IM) = ZRECVBUF(IOFFB+IOFFSP+JMLOC)
      ENDIF
      IF (LLHASPS.AND.LDOROG) THEN
        PMSPORG(IM) = ZRECVBUF(IOFFB+IOFFSPOR+JMLOC)
      ENDIF
    ENDDO
    IF (LDPRTAL) THEN
      DO JMLOC=1,IRECVNUMP
        IM=NALLMS(NPTRMS(IRECVSETA)-1+JMLOC)
        DO JL=1,IRECVLEV
          ILEV=ISTLEV+JL-1
          DO JF=1,KFTHER
            PMTHEDEG(ILEV,IM,JF)=ZRECVBUF(IOFFB+IOFFTHEDE+(JF-1)*&
             & IRECVLEV*IRECVNUMP+(JMLOC-1)*IRECVLEV+JL)  
          ENDDO
          DO JF=1,KNUMSPFLDS
            PMGFLDEG(ILEV,IM,JF)=ZRECVBUF(IOFFB+IOFFGFLDE+(JF-1)*&
             & IRECVLEV*IRECVNUMP+(JMLOC-1)*IRECVLEV+JL)  
          ENDDO
        ENDDO
        IF (LLHASPS) THEN
          PMSPDEG(IM) = ZRECVBUF(IOFFB+IOFFSPDE+JMLOC)
        ENDIF
      ENDDO
    ENDIF
    IOFFB=IOFFB+IRECVCOUNTS(JROC)
  ENDDO
  CALL GSTATS(1931,1)
ENDIF

DEALLOCATE(ZRECVBUF)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COMMSPNORM',1,ZHOOK_HANDLE)
END SUBROUTINE COMMSPNORM
