SUBROUTINE DICOMOUT(CD_CNOMA,KNOMA,PVALCO,KLONGD,KDIM,KTAG,KFIELD)

!**** *DICOMOUT*  Communicate arrays of compressed fields to the first processor

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *DICOMOUT(CNOMA,KNOMA,PVALCO,KLONGD,KDIM,KTAG)

!        Explicit arguments :
!        --------------------
!               CNOMA : character*16
!               KNOMA : effective length of CNOMA
!               PVALCO : compressed field
!               KLONGD : effective length (in REAL 64 bits) of PVALCO
!               KDIM : Declared dimension of PVALCO in calling routine
!               KTAG : loop index in calling routine
!               KFIELD : loop index in calling routine

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation
!       the dimension of local array MESS MUST be equal to local value of MAXMESS
!       (if declaration of CNOMA changes, then MAXMESS must be changed accordingly)

!     Externals.
!     ----------
!     SCATMSG, divide a received message into arrays of different types
!     GATHMSG, concatenate several arrays of different types in one message

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group Meteo-France

!     Modifications.
!     --------------
!        Original: 98-02-19
!        Optimized : 98-06  : Only one message sent or received,      
!                             Introduce PACKMSG and UNPKMSG
!        Modified: 00-09-01 by D.Dent : MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib 28-Jul-2006 Porting to NEC
!        K. Essaouini 17-jul-2009 MTAGDISTGP instead of MTAGPART (B-level) 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSDOM
USE YOMCT0   , ONLY : NPROC
USE YOMMP    , ONLY : MYPROC   ,NPRCIDS
USE YOMTAG   , ONLY : MTAGDISTGP
USE YOMOP    , ONLY : MAXFIELD
USE MPL_MODULE,ONLY : MPL_SEND ,MPL_RECV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
CHARACTER(LEN=16) ,INTENT(INOUT) :: CD_CNOMA 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNOMA 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVALCO(KDIM) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLONGD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTAG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELD 

REAL(KIND=JPRB)    :: ZMESS(KDIM+16+3)
INTEGER(KIND=JPIM) :: ITAGR, JMSGLEN, I_MAXMESS, I_MSGLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "packmsg.intfb.h"
#include "unpkmsg.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DICOMOUT',0,ZHOOK_HANDLE)
I_MAXMESS =   KDIM+16+3
ZMESS=0

IF ( NPROC > 1 ) THEN

  ITAGR=MTAGDISTGP+KFIELD*MAXFIELD*NPROC+KFIELD*JPOSDOM+KTAG

  IF (MYPROC == 1) THEN

    CALL MPL_RECV(ZMESS(1:I_MAXMESS),KTAG=ITAGR, &
     & KOUNT=JMSGLEN,CDSTRING='DICOMOUT:')  
    IF (JMSGLEN /= INT(ZMESS(3))) CALL ABOR1('DICOMOUT: RECEIVED'//&
     & ' MESSAGE LENGTH OF WRONG SIZE')  

!         write(NULOUT,*) 'Dicomout-recv ',JSENDR,ITAGR,JTAGR,
!    &                    JMSGLEN,KTAG,KFIELD
!         call flush(NULOUT)

    PVALCO = 0.E0_JPRB
    KNOMA=0
    CD_CNOMA = ' '
    CALL UNPKMSG(CD_CNOMA,KNOMA,PVALCO,KDIM,KLONGD,ZMESS,I_MAXMESS)

  ELSE

    I_MSGLON =   KLONGD + KNOMA + 3
    CALL PACKMSG(CD_CNOMA,KNOMA,PVALCO,KDIM,KLONGD,ZMESS,I_MSGLON)

!         write(NULOUT,*) 'Dicomout-send ',ITAGR,MSGLON,KTAG,KFIELD
!         call flush(NULOUT)

    CALL MPL_SEND(ZMESS(1:I_MSGLON),KDEST=NPRCIDS(1),KTAG=ITAGR, &
     & CDSTRING='DICOMOUT:')  
  ENDIF
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('DICOMOUT',1,ZHOOK_HANDLE)
END SUBROUTINE DICOMOUT
