SUBROUTINE DISGRID(KNUM,KIND,KGPTOT,PREAL,LDC,KIOPROC)

!**** *DISGRID*  - SEND OR RECEIVE GRID-POINT DATA        

!     Purpose.
!     --------
!           ARPEGE file.

!**   Interface.
!     ----------
!        *CALL* *DISGRID(.....)*

!        Explicit arguments :
!        --------------------
!           - KNUM (INPUT) : ARPEGE field number
!           - KIND (INPUT) : 0 -> SEND
!                            1 -> RECV
!           - KGPTOT (INPUT) : dimension of PREAL
!           - PREAL [INPUT (krcv=0) / OUTPUT (krcv=1)] : grid point field
!           - LDC (OPTIONAL-INPUT) : T/F: use this routine for climate/general
!              purpose (T: cf. former DISGRID_C).
!           - KIOPROC (OPTIONAL-INPUT) : IO processor number (required if LDC=T)

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        MPP Group *METEO-FRANCE*

!     Modifications.
!     --------------
!        96-01-01 : Original
!        Modified : 97-12-08 R. El Khatib NPRINTLEV
!        Modified : 00-09-01 by D.Dent : MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad : Sept 2008: merge DISGRID and DISGRID_C
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : NPRCIDS  ,MYPROC   ,NSTA     ,NONL     ,&
 & NPTRFRSTLAT,NFRSTLAT ,NLSTLAT ,NGPSET2PE
USE YOMTAG   , ONLY : MTAGDISTGP
USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY : NLOENG   ,NGPTOT   ,NGPTOTMX
USE YOMCT0   , ONLY : NPROC    ,N_REGIONS ,N_REGIONS_NS ,NPRINTLEV
USE YOMOP    , ONLY : NAGDIO
USE MPL_MODULE, ONLY : MPL_SEND ,MPL_RECV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGPTOT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIND 
REAL(KIND=JPRB),INTENT(INOUT)    :: PREAL(KGPTOT)
LOGICAL,OPTIONAL,INTENT(IN)      :: LDC
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: KIOPROC 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZFLD(NGPTOTMX)
REAL(KIND=JPRB) :: ZFLDTMP(NGPTOT)

INTEGER(KIND=JPIM) ::  IGL1, IGL2, IGLOFF, ILAST, ILEN, ILOFF,&
 & IOFF, IPARAM, ISEND, ISENDER, ISND, ITAG, &
 & ITAGR, J, JA, JB, JGL, JI, JLON, IRCV
LOGICAL :: LLC

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DISGRID',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LLC=.FALSE.
IF (PRESENT(LDC)) LLC=LDC

IF (KIND == 0) THEN

  !   distribute field to nodes

  IPARAM = KNUM
  DO JA=1,N_REGIONS_NS

    IGLOFF=NPTRFRSTLAT(JA)
    IGL1 = NFRSTLAT(JA)
    IGL2 = NLSTLAT(JA)

    IOFF=0
    IF (JA > 1) THEN
      IF ( NLSTLAT(JA-1) == NFRSTLAT(JA) ) THEN
        ILAST=NLSTLAT(JA-1)-1
      ELSE
        ILAST=NLSTLAT(JA-1)
      ENDIF
      DO J=NFRSTLAT(1),ILAST
        IOFF=IOFF+NLOENG(J)
      ENDDO
    ENDIF

    DO JB=1,N_REGIONS(JA)

      ILEN=0
      ILOFF=0
      DO JGL=IGL1,IGL2
        DO JLON=1,NONL(IGLOFF+JGL-IGL1,JB)
          ZFLD(ILEN+JLON)=PREAL(IOFF+ILOFF+NSTA(IGLOFF+JGL-IGL1,JB)+JLON-1)
        ENDDO
        ILEN=ILEN+NONL(IGLOFF+JGL-IGL1,JB)
        ILOFF=ILOFF+NLOENG(JGL)
      ENDDO

      IF (ILEN > 0) THEN
        ISND=NGPSET2PE(JA,JB)
        IF (LLC) THEN
          ISEND=NPRCIDS(ISND)
        ELSE
          ISEND=ISND
        ENDIF
        ITAG=MTAGDISTGP+IPARAM*NPROC+ISND
        IF((NPROC > 1).AND.(NPRCIDS(ISND) /= MYPROC))THEN

          CALL MPL_SEND(ZFLD(1:ILEN),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
           & CDSTRING='DISGRID:')  
          IF (NPRINTLEV == 2) THEN
            WRITE(NULOUT,*) 'DISGRID : send gridpoint field',&
             & ' to ',NPRCIDS(ISEND),' with tag ',ITAG  
          ENDIF
        ELSE
          DO JI=1,NGPTOT
            ZFLDTMP(JI)=ZFLD(JI)
          ENDDO
        ENDIF
      ENDIF
    ENDDO
  ENDDO

  !     Store data for current processor

  DO JI=1,NGPTOT
    PREAL(JI)=ZFLDTMP(JI)
  ENDDO

ELSE

  !   Receive field

  IPARAM=KNUM
  IF ( NGPTOT > 0 ) THEN
    IF (PRESENT(KIOPROC)) THEN
      IRCV=KIOPROC
    ELSE
      IRCV=NAGDIO(1,KNUM)
    ENDIF
    ITAG=MTAGDISTGP+IPARAM*NPROC+MYPROC
    IF( NPROC > 1 )THEN
      CALL MPL_RECV(PREAL(1:NGPTOT),KSOURCE=NPRCIDS(IRCV),KTAG=ITAG, &
       & KOUNT=ILEN,KFROM=ISENDER,KRECVTAG=ITAGR,CDSTRING='DISGRID:')  
      IF (NPRINTLEV == 2) THEN
        WRITE(NULOUT,*) 'DISGRID : recv gridpoint field',&
         & ' from ',ISENDER,' with tag ',ITAGR  
      ENDIF
      IF ( ILEN /= NGPTOT ) THEN
        CALL ABOR1('DISGRID: INVALID RECEIVE MESSAGE LENGTH')
      ENDIF
    ENDIF
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DISGRID',1,ZHOOK_HANDLE)
END SUBROUTINE DISGRID
