SUBROUTINE DISGRID_SURF_EXT(KGPTOT,PREAL,PFIELD)

!**** *DISGRID_SURF_EXT*  - distribute GRID-POINT DATA 
! the current processor extract ist bloc from the complete field       

!     Purpose.
!     --------
!           AROME/externalised surface file

!**   Interface.
!     ----------
!        *CALL* *DISGRID_SURF_EXT(.....)*

!        Explicit arguments :
!        --------------------
!           - KGPTOT (INPUT) : dimension of PREAL
!           - PREAL  (INPUT) : complete grid point field
!           - PFIELD (OUTPUT) : current proc bloc

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        adapted from DISGRID_C,  MPP Group *METEO-FRANCE*
!        S. Malardel (AROME)

!     Modifications.
!     --------------
!        04-03-18 : Original
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : NPRCIDS  ,MYPROC   ,NSTA     ,NONL     ,&
 & NPTRFRSTLAT,NFRSTLAT ,NLSTLAT ,NGPSET2PE
USE YOMGEM   , ONLY : NLOENG   ,NGPTOT   ,NGPTOTMX
USE YOMCT0   , ONLY : N_REGIONS ,N_REGIONS_NS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGPTOT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREAL(KGPTOT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFIELD(NGPTOT) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZFLD(NGPTOTMX)
INTEGER(KIND=JPIM) :: IGL1, IGL2, IGLOFF, ILAST, ILEN, ILOFF,&
 & IOFF, IPROC, ISND,                  &
 & J, JA, JB, JGL, JI, JLON  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DISGRID_SURF_EXT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

DO JA=1,N_REGIONS_NS
  DO JB=1,N_REGIONS(JA)
    ISND=NGPSET2PE(JA,JB)
    IPROC=NPRCIDS(ISND)

    IF(IPROC == MYPROC) THEN

!     Compute the indices of the bloc associated with current processor

      IGLOFF=NPTRFRSTLAT(JA)
      IGL1 = NFRSTLAT(JA)
      IGL2 = NLSTLAT(JA)

      IOFF=0
      IF (JA > 1) THEN
        IF ( NLSTLAT(JA-1) == NFRSTLAT(JA) ) THEN
          ILAST=NLSTLAT(JA-1)-1
        ELSE
          ILAST=NLSTLAT(JA-1)
        ENDIF
        DO J=NFRSTLAT(1),ILAST
          IOFF=IOFF+NLOENG(J)
        ENDDO
      ENDIF
         
      ILEN=0
      ILOFF=0
      DO JGL=IGL1,IGL2
        DO JLON=1,NONL(IGLOFF+JGL-IGL1,JB)
          ZFLD(ILEN+JLON)=PREAL(IOFF+ILOFF+NSTA(IGLOFF+JGL-IGL1,JB)+JLON-1)
        ENDDO
        ILEN=ILEN+NONL(IGLOFF+JGL-IGL1,JB)
        ILOFF=ILOFF+NLOENG(JGL)
      ENDDO
      
!       Store data for current processor

      DO JI=1,NGPTOT
        PFIELD(JI)=ZFLD(JI)
      ENDDO

    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('DISGRID_SURF_EXT',1,ZHOOK_HANDLE)
END SUBROUTINE DISGRID_SURF_EXT
