SUBROUTINE DISSPEC0(KIOPROC,KSETV,KCH,KIND,KSPEC2,KFIELDS,KPOSSP,KPRTRW, &
 & PSPEC,LDC,KNUM)

!**** *DISSPEC0*  - DISTRIBUTION OF A SET OF GLOBAL SPECTRAL FIELDS          

!     Purpose.
!     --------
!       To distribute a set of global spectral fields among processors. 
!       N.B. : all fields in the set are expected to belong to the same V-set   

!**   Interface.
!     ----------
!        *CALL* *DISSPEC0(.....)*

!        Explicit arguments :
!        --------------------
!         * INPUT:
!           - KIOPROC : I/O processor involved
!           - KSETV   : Set-V involved
!           - KCH     : chunk number
!           - KIND    : 0 -> SEND ; 1 -> RECV
!           - KSPEC2  : spectral dimension (send=>global ; recv=>local)
!           - KFIELDS : number of fields to send/receive
!           - KPOSSP  : location of first spectral coefficient for each set-W
!                       (+ 1 extra value locating the "last + 1" set-W)
!           - KPRTRW  : number of processors for set-W
!         * IN-OUT:
!           - PSPEC   : spectral fields to send/receive
!         * OPTIONAL INPUT:
!           - LDC     : T/F: use this routine for climate/general purpose.
!                       (T: cf. former DISSPEC)
!           - KNUM    : number of the field (required if LDC=T).

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        MPP Group *METEO-FRANCE*

!     Modifications.
!     --------------
!        95-xx-xx : Original
!        Modified : 97-12-08 R. El Khatib NPRINTLEV
!        Modified : 99-04-21 R. El Khatib completely new purpose
!        Modified : 00-09-01 by D.Dent : MPE to MPL conversion
!        Modified : 01-07-06 by R. El Khatib : Fix on MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Ouaraini & R. El Khatib: 03-Oct-2006: partionning of global
!                   spectral data among PEs 
!        Modified : 08-Nov-07 by K. Yessad: merged version of DISSPEC+DISSPEC0
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC   ,MYSETV
USE YOMCT0   , ONLY : NPRINTLEV,LELAM    ,NPROC
USE YOMTAG   , ONLY : MTAGDISTSP
USE YOMDIM   , ONLY : NMSMAX  ,NSMAX 
USE YEMDIM   , ONLY : NISNAX
USE YEMLAP   , ONLY : NCPL4M
USE YOMMPG   , ONLY : NDIM0G 
USE MPL_MODULE

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEC2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPRTRW 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIOPROC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSETV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPOSSP(KPRTRW+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPEC(KSPEC2,KFIELDS) 
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDC
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN) :: KNUM

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM)               :: IDIM0GG1(0:NMSMAX)
INTEGER(KIND=JPIM)               :: IDIM0GG2(0:NSMAX)
REAL(KIND=JPRB)                  :: ZSPEC2(KSPEC2) 

LOGICAL :: LLBSET,LLREORD,LLC

! ISND    : processor receiving data
! ZBUF    : communication buffer
! IBUFLEN : lenght ofcommunication buffer
INTEGER(KIND=JPIM) :: ILEN, ISENDER, ITAG, ITAGR, JSETW, IBUFLEN, JFLD, IP, ISND
INTEGER(KIND=JPIM) :: JM, JN, IPOS, ISP, ISPG, ISEND

REAL(KIND=JPRB), ALLOCATABLE :: ZBUF(:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "set2pe.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DISSPEC0',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LLC=.FALSE.
IF (PRESENT(LDC)) LLC=LDC

IF (LLC) THEN
  LLBSET=(KSETV==MYSETV)
  LLREORD=.FALSE.
ELSE
  LLBSET=.TRUE.
  LLREORD=.TRUE.
ENDIF

IF (KIND == 0) THEN

  ! * Reordering (like in SPREORD) when required:

  IF (LLREORD) THEN
    IF (LELAM)  THEN
      IPOS=1
      DO JM=0,NMSMAX
        IDIM0GG1(JM)=IPOS
        IPOS=IPOS+NCPL4M(JM)
      ENDDO
    ELSE
      IPOS=1
      DO JM=0,NSMAX
        IDIM0GG2(JM)=IPOS
        IPOS=IPOS+(NSMAX+1-JM)*2
      ENDDO
    ENDIF   
    IF (LELAM)  THEN
      DO JFLD=1,KFIELDS
        ZSPEC2=0.0_JPRB
        DO JM=0,NMSMAX
          DO JN=0,NISNAX(JM)
            ISP=NDIM0G(JM)+4*JN
            ISPG=IDIM0GG1(JM)+4*JN
            ZSPEC2(ISP:ISP+3)=PSPEC(ISPG:ISPG+3,JFLD)
          ENDDO
        ENDDO
        PSPEC(:,JFLD)=ZSPEC2(:)
      ENDDO
    ELSE
      DO JFLD=1,KFIELDS
        ZSPEC2=0.0_JPRB
        DO JM=0,NSMAX
          DO JN=JM,NSMAX
            ISP=NDIM0G(JM)+2*(JN-JM)
            ISPG=IDIM0GG2(JM)+2*(JN-JM)
            ZSPEC2(ISP)=PSPEC(ISPG,JFLD)
            ZSPEC2(ISP+1)=PSPEC(ISPG+1,JFLD)
          ENDDO
        ENDDO
        PSPEC(:,JFLD)=ZSPEC2(:)
      ENDDO
    ENDIF
  ENDIF

  ! * Distribute fields to nodes:

  DO JSETW=1,KPRTRW
    CALL SET2PE(ISND,0,0,JSETW,KSETV)
    ILEN = KPOSSP(JSETW+1)-KPOSSP(JSETW)
    IF ( ILEN > 0 .AND. NPRCIDS(ISND) /= MYPROC) THEN
      ! There is something to send to somebody
      IBUFLEN=ILEN*KFIELDS
      ALLOCATE(ZBUF(IBUFLEN))
      IP=1
      DO JFLD=1,KFIELDS
        ZBUF(IP:IP+ILEN-1)=PSPEC(KPOSSP(JSETW):KPOSSP(JSETW)+ILEN-1,JFLD)
        IP=IP+ILEN
      ENDDO
      IF (LLC) THEN
        ITAG=MTAGDISTSP+(KCH-1)+KNUM*NPROC+ISND
        ISEND=NPRCIDS(ISND)
      ELSE 
        ITAG=MTAGDISTSP+KCH
        ISEND=ISND
      ENDIF
      CALL MPL_SEND(ZBUF(1:IBUFLEN),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
       & CDSTRING='DISSPEC0:')  
      IF (NPRINTLEV == 2) THEN
        WRITE(NULOUT,*) 'DISSPEC0 : send spectral fields to ',&
         & NPRCIDS(ISND),' with tag ',ITAG  
      ENDIF
      DEALLOCATE(ZBUF)
    ENDIF
  ENDDO

ELSE

  ! * Receive fields:

  IF (LLBSET .AND. KSPEC2 > 0 .AND. NPRCIDS(KIOPROC) /= MYPROC) THEN
    ! There is something to receive from somebody
    IBUFLEN=KSPEC2*KFIELDS
    ALLOCATE(ZBUF(IBUFLEN))
    IF (LLC) THEN
      ITAG=MTAGDISTSP+(KCH-1)+KNUM*NPROC+MYPROC
    ELSE
      ITAG=MTAGDISTSP+KCH
    ENDIF
    CALL MPL_RECV(ZBUF(1:IBUFLEN),KSOURCE=NPRCIDS(KIOPROC),KTAG=ITAG, &
     & KOUNT=ILEN,KFROM=ISENDER,KRECVTAG=ITAGR,CDSTRING='DISSPEC0:')  
    IF (NPRINTLEV == 2) THEN
      WRITE(NULOUT,*) 'DISSPEC0 : recv spectral field from ',ISENDER,&
       & ' with tag ',ITAGR  
    ENDIF
    IF (ILEN /= IBUFLEN) THEN
      WRITE(NULOUT,*) 'ILEN = ',ILEN,' IBUFLEN = ',IBUFLEN
      CALL ABOR1('DISSPEC0: INVALID RECEIVE MESSAGE LENGHT')
    ELSE
      IP=1
      DO JFLD=1,KFIELDS
        PSPEC(1:KSPEC2,JFLD)=ZBUF(IP:IP+KSPEC2-1)
        IP=IP+KSPEC2
      ENDDO
    ENDIF
    DEALLOCATE(ZBUF)
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DISSPEC0',1,ZHOOK_HANDLE)
END SUBROUTINE DISSPEC0
