SUBROUTINE DIWRSPE0(KIOPROC,KSETV,KCH,KIND,KSPEC2,KFIELDS,KPOSSP,KPRTRW,PSPEC)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NMSMAX  ,NSMAX
USE YOMMPG   , ONLY : NDIM0G 
USE YEMDIM   , ONLY : NISNAX
USE YEMLAP   , ONLY : NCPL4M
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMCT0   , ONLY :     NPRINTLEV  , LELAM
USE YOMTAG   , ONLY : MTAGDISTSP
USE MPL_MODULE


!**** *DIWRSPE0*  - DISTRIBUTION OF A SET OF GLOBAL SPECTRAL FIELDS          

!     Purpose.
!     --------
!       To distribute a set of global spectral fields among processors. 
!       N.B. : all fields in the set are expected to belong to the same V-set   

!**   Interface.
!     ----------
!        *CALL* *DIWRSPE0(.....)*

!        Explicit arguments :
!        --------------------
!           - KIOPROC : I/O processor involved
!           - KSETV   : Set-V involved
!           - KCH     : chunk number
!           - KIND    : 0 -> SEND ; 1 -> RECV
!           - KSPEC2  : spectral dimension (send=>global ; recv=>local)
!           - KFIELDS : number of fields to send/receive
!           - KPOSSP  : location of first spectral coefficient for each set-W
!                       (+ 1 extra value locating the "last + 1" set-W)
!           - KPRTRW  : number of processors for set-W
!           - PSPEC   : spectral fields to send/receive

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------

!     Reference.
!     ----------
!      DISSPEC0 & DIWRSPE CY24T1

!     Author.
!     -------
!      MPP Group *METEO-FRANCE*

!     Modifications.
!     --------------
!        2000-11-29 : Original : S. Ivatek-Sahdan
!        Modified : 2001-03-19 by S. Ivatek-Sahdan : MPE to MPL conversion
!        Modified : 2001-07-06 by R. El Khatib : Fix on MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Ouaraini & R. El Khatib : 03-Oct-2006 : The received global spectral array is ordered according to JM=0,1,2,3,4,...
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEC2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPRTRW 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIOPROC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSETV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPOSSP(KPRTRW+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPEC(KSPEC2,KFIELDS) 
INTEGER(KIND=JPIM)               :: IDIM0GG1(0:NMSMAX)
INTEGER(KIND=JPIM)               :: IDIM0GG2(0:NSMAX)
REAL(KIND=JPRB)                  :: ZSPEC2(KSPEC2) 
! ISND    : processor receiving data
! ZBUF    : communication buffer
! IBUFLEN : lenght ofcommunication buffer

INTEGER(KIND=JPIM) :: ILEN, ISENDER, ITAG, ITAGR, JSETW, IBUFLEN, JFLD, IP, ISND, ILENB
INTEGER(KIND=JPIM) :: IPOS, JN, JM, ISP, ISPG
REAL(KIND=JPRB), ALLOCATABLE :: ZBUF(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "set2pe.intfb.h"

IF (LHOOK) CALL DR_HOOK('DIWRSPE0',0,ZHOOK_HANDLE)
IF (KIND == 0) THEN

!     distribute fields to nodes

  IF (KSPEC2 > 0 .AND. NPRCIDS(KIOPROC) /= MYPROC) THEN
!   There is something to send to somebody -KIOPROC
    IBUFLEN=KSPEC2*KFIELDS
    ALLOCATE(ZBUF(IBUFLEN))
    IP=1
    DO JFLD=1,KFIELDS
      ZBUF(IP:IP+KSPEC2-1)=PSPEC(1:KSPEC2,JFLD)
      IP=IP+KSPEC2
    ENDDO
    ITAG=MTAGDISTSP+KCH
    CALL MPL_SEND(ZBUF(1:IBUFLEN),KDEST=NPRCIDS(KIOPROC),KTAG=ITAG, &
     & CDSTRING='DIWRSPE0:')  
    IF (NPRINTLEV == 2) THEN
      WRITE(NULOUT,*) 'DIWRSPE0 :  send spectral field  ',NPRCIDS(KIOPROC),&
       & ' with tag ',ITAG  
    ENDIF
    DEALLOCATE(ZBUF)
  ENDIF

ELSE

!     Receive fields
  IF (LELAM)  THEN
    IPOS=1
    DO JM=0,NMSMAX
      IDIM0GG1(JM)=IPOS
      IPOS=IPOS+NCPL4M(JM)
    ENDDO
  ELSE
    IPOS=1
    DO JM=0,NSMAX
      IDIM0GG2(JM)=IPOS
      IPOS=IPOS+(NSMAX+1-JM)*2
    ENDDO
  ENDIF

  DO JSETW=1,KPRTRW
    CALL SET2PE(ISND,0,0,JSETW,KSETV)
    ILEN = KPOSSP(JSETW+1)-KPOSSP(JSETW)
    IF (ILEN > 0 .AND. NPRCIDS(ISND) /= MYPROC) THEN
!     There is something to receive from somebody -ISND 
      IBUFLEN=ILEN*KFIELDS
      ALLOCATE(ZBUF(IBUFLEN))
      ITAG=MTAGDISTSP+KCH
      CALL MPL_RECV(ZBUF(1:IBUFLEN),KSOURCE=NPRCIDS(ISND),KTAG=ITAG, &
       & KOUNT=ILENB,KFROM=ISENDER,KRECVTAG=ITAGR,CDSTRING='DIWRSPE0:')  
      IF (ILENB /= IBUFLEN) THEN
        WRITE(NULOUT,*) 'ILENB = ',ILENB,' IBUFLEN = ',IBUFLEN
        CALL ABOR1('DIWRSPE0: INVALID RECEIVE MESSAGE LENGHT')
      ELSE
        IP=1
        DO JFLD=1,KFIELDS
          PSPEC(KPOSSP(JSETW):KPOSSP(JSETW)+ILEN-1,JFLD)=ZBUF(IP:IP+ILEN-1)
          IP=IP+ILEN
        ENDDO
      ENDIF
      IF (NPRINTLEV == 2) THEN
        WRITE(NULOUT,*) 'DIWRSPE0 : recv spectral field from ',ISENDER,&
         & ' with tag ',ITAGR  
      ENDIF
      DEALLOCATE(ZBUF)
    ENDIF
  ENDDO

  IF (LELAM)  THEN
    DO JFLD=1,KFIELDS
      ZSPEC2=0.0_JPRB   ! tableau intermediaire pour ne pas ecraser les valeurs de PSPEC
      DO JM=0,NMSMAX
          DO JN=0,NISNAX(JM)
            ISP=NDIM0G(JM)+4*JN
            ISPG=IDIM0GG1(JM)+4*JN
            ZSPEC2(ISPG:ISPG+3)=PSPEC(ISP:ISP+3,JFLD)
          ENDDO
      ENDDO
      PSPEC(:,JFLD)=ZSPEC2(:)
    ENDDO
  ELSE
    DO JFLD=1,KFIELDS
      ZSPEC2=0.0_JPRB   ! tableau intermediaire pour ne pas ecraser les valeurs de PSPEC
      DO JM=0,NSMAX
        DO JN=JM,NSMAX
          ISP=NDIM0G(JM)+2*(JN-JM)
          ISPG=IDIM0GG2(JM)+2*(JN-JM)
          ZSPEC2(ISPG)=PSPEC(ISP,JFLD)
          ZSPEC2(ISPG+1)=PSPEC(ISP+1,JFLD)
        ENDDO
      ENDDO
      PSPEC(:,JFLD)=ZSPEC2(:)
    ENDDO
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('DIWRSPE0',1,ZHOOK_HANDLE)
END SUBROUTINE DIWRSPE0
