SUBROUTINE GATHERBDY(PRBDY,PRBDYG)

!**** *GATHERBDY*  - Gather a partitioned boundary array to global form

!     Purpose.
!     --------
!           Gather global version of a boundary field RXXX(NGPTOT) array
!           on each PE

!**   Interface.
!     ----------
!        *CALL* *GATHERBDY(PRBDY,PRBDYG)

!        Explicit arguments :
!        --------------------
!        PRBDY (NGPTOT)  : Local version of PRBDY (input)
!        PRBDYG(NGPTOTG) : Global version of PRBDY (output)

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original  : 96-05-05
!        Modified: 00-09-01 by D.Dent : MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPROC
USE YOMMP    , ONLY :         NSTA     ,NONL     ,NPTRFRSTLAT,&
 & NFRSTLAT ,NLSTLAT  
USE YOMGEM   , ONLY :     NLOENG   ,NGPTOTG
USE YOMLUN   , ONLY : NULERR
USE MPL_MODULE

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PRBDY(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRBDYG(:) 
! === END OF INTERFACE BLOCK ===
REAL(KIND=JPRB) :: ZPRBDY(NGPTOTG)

INTEGER(KIND=JPIM) :: IAPROC, IBPROC, IFPT, IGL1,&
 & IGL2, IGLOFF, ILAST, ILEN, ILOFF, &
 & IOFF, &
 & ITOTADD, ITOTREC, IV, IW, J, JGL, JLON, JROC  

INTEGER(KIND=JPIM) :: JTOT,JLEN(NPROC),JOFF(NPROC)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "pe2set.intfb.h"

!     ------------------------------------------------------------------

!     1.    Send local spectral arrays to all PEs
!           --------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GATHERBDY',0,ZHOOK_HANDLE)


JTOT=1
DO JROC=1,NPROC
  CALL PE2SET(JROC,IAPROC,IBPROC,IW,IV)
  IGL1 = NFRSTLAT(IAPROC)
  IGL2 = NLSTLAT(IAPROC)
  IGLOFF = NPTRFRSTLAT(IAPROC)
  JLEN(JROC)=0
  DO JGL=IGL1,IGL2
    JLEN(JROC) = JLEN(JROC)+NONL(IGLOFF+JGL-IGL1,IBPROC)
  ENDDO
  JOFF(JROC)=JTOT-1
  JTOT=JTOT+JLEN(JROC)
ENDDO

IF(JTOT-1 /= NGPTOTG) THEN
  WRITE(NULERR,*)'JTOT-1= ',JTOT-1,' NGPTOTG=',NGPTOTG
  CALL ABOR1(' GATHERBDY : COMPUTED WRONG DATA LENGTH')
ENDIF

CALL GSTATS_BARRIER(771)
CALL GSTATS(609,0)
CALL MPL_ALLGATHERV(PRBDY,ZPRBDY,JLEN,KRECVDISPL=JOFF,CDSTRING='GATHERBDY:')
CALL GSTATS(609,1)
CALL GSTATS_BARRIER2(771)

IFPT = 1
ITOTREC = 0
ITOTADD = 0

DO JROC=1,NPROC
  CALL PE2SET(JROC,IAPROC,IBPROC,IW,IV)
  IOFF = 0
  IF(IAPROC > 1) THEN
    IF( NLSTLAT(IAPROC-1) == NFRSTLAT(IAPROC) )THEN
      ILAST = NLSTLAT(IAPROC-1)-1
    ELSE
      ILAST = NLSTLAT(IAPROC-1)
    ENDIF
    DO J=NFRSTLAT(1),ILAST
      IOFF = IOFF+NLOENG(J)
    ENDDO
  ENDIF
  IGLOFF = NPTRFRSTLAT(IAPROC)
  IGL1 = NFRSTLAT(IAPROC)
  IGL2 = NLSTLAT(IAPROC)
  
!   Store the received data in the global array

  ILEN = 0
  ILOFF = 0
  DO JGL=IGL1,IGL2
    ITOTADD = ITOTADD+NONL(IGLOFF+JGL-IGL1,IBPROC)
    DO JLON=1,NONL(IGLOFF+JGL-IGL1,IBPROC)
      PRBDYG(IOFF+ILOFF+NSTA(IGLOFF+JGL-IGL1,IBPROC)+JLON-1)=&
       & ZPRBDY(JOFF(JROC)+ILEN+JLON+IFPT-1)  
    ENDDO
    ILEN = ILEN + NONL(IGLOFF+JGL-IGL1,IBPROC)
    ILOFF = ILOFF + NLOENG(JGL)
  ENDDO
ENDDO

IF(ITOTADD /= NGPTOTG) THEN
  WRITE(NULERR,*)'ITOTADD= ',ITOTADD,' NGPTOTG=',NGPTOTG
  CALL ABOR1(' GATHERBDY : DID NOT WORK')
ENDIF

IF (LHOOK) CALL DR_HOOK('GATHERBDY',1,ZHOOK_HANDLE)
END SUBROUTINE GATHERBDY
