SUBROUTINE GATHEREIGMD(PFGEGVNS,PFGEDVNS,PFGEGVND,PFGEDVND,&
 & PFGEGVNQ,PFGEDVNQ,PFGEGVNU,PFGEDVNU,&
 & PFGEGVNO3,PFGEDVNO3,LDOZJB)  

!**** *GATHEREIGMD*  - Gather partitioned eigenvectors to global form

!     Purpose.
!     --------
!           Gather global versions of FGExxxx on each PE. Global
!           versions are required to perform Jb calculations.

!**   Interface.
!     ----------
!        *CALL* *GATHEREIGMD(...)

!        Explicit arguments :
!        --------------------
!        PFGExxxx        : Local Eigenvector arrays  (input)

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        MPL_SEND, MPL_RECV  -  communication interface

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original  : 98-01-01
!        E. Holm   : 98-03-15 Ozone included (in call only).
!        E. Holm   : 98-03-25 Ozone included in program.
!        M.Hamrud  : 00-02-02  Removal of MYSENDV etc.
!        D.Dent    : 00-09-01  MPE to MPL conversion
!        M.Hamrud  : 01-Oct-2003 CY28 Cleaning
!        E.Sevault : 01-Sep-2006 Special O3 case pack/unpack
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX    ,NUMP
USE YOMCT0   , ONLY : NPRTRW
USE YOMLAP   , ONLY : MYMS
USE YOMMP    , ONLY : NPRCIDS  ,NUMPP    ,NPTRMS   ,NALLMS   ,MYSETV,MYSETW
USE YOMTAG   , ONLY : MTAGEIGMD
USE YOMJG    , ONLY : VCORS, SPJB_VARS_INFO, N_SPJB_VARS
USE YOMGRB   , ONLY : NGRBVO, NGRBD, NGRBT, NGRBQ, NGRBO3, NGRBLNSP
USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEGVNS(:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEDVNS(:,:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEGVND(:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEDVND(:,:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEGVNQ(:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEDVNQ(:,:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEGVNU(:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEDVNU(:,:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEGVNO3(:,:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEDVNO3(:,:,:) 
LOGICAL           ,INTENT(IN)    :: LDOZJB 
! === END OF INTERFACE BLOCK ===
REAL(KIND=JPRB),ALLOCATABLE :: ZSENDBUF(:), ZRECVBUF(:)

INTEGER(KIND=JPIM) :: IA, IB, ICOMSIZE, IDIMD, IDIMO3, IDIMQ,&
 & IDIMS, IDIMU, IEND1, IEND10, IEND2, IEND3, &
 & IEND4, IEND5, IEND6, IEND7, IEND8, IEND9, ILREC, IM, IRECV, IRECVID, IRECVNUMP, &
 & IRECVSETW, ISEND, ISTA, ISTA1, &
 & ISTA10, ISTA2, ISTA3, ISTA4, ISTA5, ISTA6, &
 & ISTA7, ISTA8, ISTA9, ITAG, IV, &
 & J2, JMLOC, JROCA,IRECVSETA,ISENDSETA, IOFF, JFIELD, IPTJB, &
 & JFIELD_VO, JFIELD_D, JFIELD_TU, JFIELD_Q, JFIELD_O3

!    INTEGER FUNCTIONS
INTEGER(KIND=JPIM) :: MYSENDSET,MYRECVSET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "pe2set.intfb.h"
#include "set2pe.intfb.h"

!     ------------------------------------------------------------------

!     1.    Store local Eigenmodes in global arrays
!           ----------------------------------------

IF (LHOOK) CALL DR_HOOK('GATHEREIGMD',0,ZHOOK_HANDLE)

!--- temporary fix
JFIELD_VO = HUGE(JFIELD_VO)
JFIELD_D  = HUGE(JFIELD_D )
JFIELD_TU = HUGE(JFIELD_TU)
JFIELD_Q  = HUGE(JFIELD_Q )
JFIELD_O3 = HUGE(JFIELD_O3)

DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB

  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBVO) THEN
    JFIELD_VO = JFIELD
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      VCORS(JFIELD)%EVALS(:,IM)   = PFGEGVNS(:,JMLOC)
      VCORS(JFIELD)%EVECS(:,:,IM) = PFGEDVNS(:,:,JMLOC)
    ENDDO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBD) THEN
    JFIELD_D = JFIELD
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      VCORS(JFIELD)%EVALS(:,IM)   = PFGEGVND(:,JMLOC)
      VCORS(JFIELD)%EVECS(:,:,IM) = PFGEDVND(:,:,JMLOC)
    ENDDO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT) THEN
    JFIELD_TU = JFIELD
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      VCORS(JFIELD)%EVALS(:,IM)   = PFGEGVNU(:,JMLOC)
      VCORS(JFIELD)%EVECS(:,:,IM) = PFGEDVNU(:,:,JMLOC)
    ENDDO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBQ) THEN
    JFIELD_Q = JFIELD
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      VCORS(JFIELD)%EVALS(:,IM)   = PFGEGVNQ(:,JMLOC)
      VCORS(JFIELD)%EVECS(:,:,IM) = PFGEDVNQ(:,:,JMLOC)
    ENDDO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
    JFIELD_O3 = JFIELD
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      VCORS(JFIELD)%EVALS(:,IM)   = PFGEGVNO3(:,JMLOC)
      VCORS(JFIELD)%EVECS(:,:,IM) = PFGEDVNO3(:,:,JMLOC)
    ENDDO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE/=NGRBLNSP) THEN
    CALl ABOR1 ('UNRECOGNIZED GRIBCODE')
  ENDIF
ENDDO

IF (JFIELD_VO==HUGE(JFIELD_VO) .OR. JFIELD_D==HUGE(JFIELD_D) .OR. &
  & JFIELD_TU==HUGE(JFIELD_TU) .OR. JFIELD_Q==HUGE(JFIELD_Q) .OR. &
  & (LDOZJB .AND. JFIELD_O3==HUGE(JFIELD_O3))) THEN
  CALL ABOR1 ('SOME EXPECTED VARS MISSING - GATHEREIGMD NOT YET GENERALIZED')
ENDIF

!     ------------------------------------------------------------------

!     2.    Fill and send local spectral arrays to all PE's in the A-set
!           ------------------------------------------------------------

IDIMS =SIZE(PFGEGVNS,1)
IDIMD =SIZE(PFGEGVND,1)
IDIMQ =SIZE(PFGEGVNQ,1)
IDIMU =SIZE(PFGEGVNU,1)
IF (LDOZJB) THEN
  IDIMO3=SIZE(PFGEGVNO3,1)
ELSE
  IDIMO3=0
ENDIF

ICOMSIZE=IDIMS + IDIMD + IDIMQ + IDIMU + IDIMO3 +&
 & IDIMS*IDIMS +IDIMD*IDIMD +IDIMQ*IDIMQ +IDIMU*IDIMU +IDIMO3*IDIMO3  

ISTA1=1
IEND1=ISTA1+IDIMS-1
ISTA2=IEND1+1
IEND2=ISTA2+IDIMD-1
ISTA3=IEND2+1
IEND3=ISTA3+IDIMQ-1
ISTA4=IEND3+1
IEND4=ISTA4+IDIMU-1
ISTA5=IEND4+1
IEND5=ISTA5+IDIMO3-1
ISTA6=IEND5+1
IEND6=ISTA6+IDIMS*IDIMS-1
ISTA7=IEND6+1
IEND7=ISTA7+IDIMD*IDIMD-1
ISTA8=IEND7+1
IEND8=ISTA8+IDIMQ*IDIMQ-1
ISTA9=IEND8+1
IEND9=ISTA9+IDIMU*IDIMU-1
ISTA10=IEND9+1
IEND10=ISTA10+IDIMO3*IDIMO3-1

IF (IEND10 /= ICOMSIZE) CALL ABOR1(&
 & 'GATHEREIGMD: WRONG DIMENSIONING OF COMMUNICATION BUFFER')  

ALLOCATE (ZSENDBUF(ICOMSIZE*NUMP))

!CDIR NODEP
DO JMLOC=1,NUMP
  IOFF = (JMLOC-1)*ICOMSIZE
  ZSENDBUF(ISTA1+IOFF:IEND1+IOFF)=PFGEGVNS(:,JMLOC)
  ZSENDBUF(ISTA2+IOFF:IEND2+IOFF)=PFGEGVND(:,JMLOC)
  ZSENDBUF(ISTA3+IOFF:IEND3+IOFF)=PFGEGVNQ(:,JMLOC)
  ZSENDBUF(ISTA4+IOFF:IEND4+IOFF)=PFGEGVNU(:,JMLOC)

  DO J2=1,IDIMS
    ISTA=ISTA6+(J2-1)*IDIMS
    ZSENDBUF(ISTA+IOFF:ISTA+IDIMS-1+IOFF)=PFGEDVNS(:,J2,JMLOC)
  ENDDO
  DO J2=1,IDIMD
    ISTA=ISTA7+(J2-1)*IDIMD
    ZSENDBUF(ISTA+IOFF:ISTA+IDIMD-1+IOFF)=PFGEDVND(:,J2,JMLOC)
  ENDDO
  DO J2=1,IDIMQ
    ISTA=ISTA8+(J2-1)*IDIMQ
    ZSENDBUF(ISTA+IOFF:ISTA+IDIMQ-1+IOFF)=PFGEDVNQ(:,J2,JMLOC)
  ENDDO
  DO J2=1,IDIMU
    ISTA=ISTA9+(J2-1)*IDIMU
    ZSENDBUF(ISTA+IOFF:ISTA+IDIMU-1+IOFF)=PFGEDVNU(:,J2,JMLOC)
  ENDDO
ENDDO

! Special case - packing O3 fields
IF (LDOZJB) THEN
  DO JMLOC=1,NUMP
    IOFF = (JMLOC-1)*ICOMSIZE
    ZSENDBUF(ISTA5+IOFF:IEND5+IOFF)=PFGEGVNO3(:,JMLOC)
    DO J2=1,IDIMO3
      ISTA=ISTA10+(J2-1)*IDIMO3
      ZSENDBUF(ISTA+IOFF:ISTA+IDIMO3-1+IOFF)=PFGEDVNO3(:,J2,JMLOC)
    ENDDO
  ENDDO
ENDIF

!  Receive buffer over-dimensioned due to laziness

ALLOCATE (ZRECVBUF(ICOMSIZE*(NSMAX+1)))
ITAG=MTAGEIGMD

DO JROCA=1,NPRTRW-1

  CALL GSTATS(523,0)
  ISENDSETA=MYSENDSET(NPRTRW,MYSETW,JROCA)
  CALL SET2PE(ISEND,0,0,ISENDSETA,MYSETV)
  CALL MPL_SEND(ZSENDBUF,KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
   & CDSTRING='GATHEREIGMD:')  

  IRECVSETA=MYRECVSET(NPRTRW,MYSETW,JROCA)
  CALL SET2PE(IRECV,0,0,IRECVSETA,MYSETV)
  CALL MPL_RECV(ZRECVBUF,KSOURCE=NPRCIDS(IRECV),KTAG=ITAG, &
   & KOUNT=ILREC,KFROM=IRECVID,CDSTRING='GATHEREIGMD:')  

  CALL PE2SET(IRECVID,IA,IB,IRECVSETW,IV)
  CALL GSTATS(523,1)
  IRECVNUMP=NUMPP(IRECVSETW)
  IF (IEND10*IRECVNUMP /= ILREC) CALL ABOR1(&
   & 'GATHEREIGMD: WRONG AMOUNT RECEIVED ')  

!OCL NOVREC
!CDIR NODEP
  DO JMLOC=1,IRECVNUMP
    IOFF = (JMLOC-1)*ICOMSIZE
    IM=NALLMS(NPTRMS(IRECVSETW)-1+JMLOC)
    VCORS(JFIELD_VO)%EVALS(:,IM)=ZRECVBUF(ISTA1+IOFF:IEND1+IOFF)
    VCORS(JFIELD_D )%EVALS(:,IM)=ZRECVBUF(ISTA2+IOFF:IEND2+IOFF)
    VCORS(JFIELD_Q )%EVALS(:,IM)=ZRECVBUF(ISTA3+IOFF:IEND3+IOFF)
    VCORS(JFIELD_TU)%EVALS(:,IM)=ZRECVBUF(ISTA4+IOFF:IEND4+IOFF)
    DO J2=1,IDIMS
      ISTA=ISTA6+(J2-1)*IDIMS
      VCORS(JFIELD_VO)%EVECS(:,J2,IM)=ZRECVBUF(ISTA+IOFF:ISTA+IDIMS-1+IOFF)
    ENDDO
    DO J2=1,IDIMD
      ISTA=ISTA7+(J2-1)*IDIMD
      VCORS(JFIELD_D)%EVECS(:,J2,IM)=ZRECVBUF(ISTA+IOFF:ISTA+IDIMD-1+IOFF)
    ENDDO
    DO J2=1,IDIMQ
      ISTA=ISTA8+(J2-1)*IDIMQ
      VCORS(JFIELD_Q)%EVECS(:,J2,IM)=ZRECVBUF(ISTA+IOFF:ISTA+IDIMQ-1+IOFF)
    ENDDO
    DO J2=1,IDIMU
      ISTA=ISTA9+(J2-1)*IDIMU
      VCORS(JFIELD_TU)%EVECS(:,J2,IM)=ZRECVBUF(ISTA+IOFF:ISTA+IDIMU-1+IOFF)
    ENDDO
  ENDDO

  ! Special case - unpacking O3 fields
  IF (LDOZJB) THEN
    DO JMLOC=1,IRECVNUMP
      IOFF = (JMLOC-1)*ICOMSIZE
      IM=NALLMS(NPTRMS(IRECVSETW)-1+JMLOC)
      VCORS(JFIELD_O3)%EVALS(:,IM)=ZRECVBUF(ISTA5+IOFF:IEND5+IOFF)
      DO J2=1,IDIMO3
        ISTA=ISTA10+(J2-1)*IDIMO3
        VCORS(JFIELD_O3)%EVECS(:,J2,IM)=ZRECVBUF(ISTA+IOFF:ISTA+IDIMO3-1+IOFF)
      ENDDO
    ENDDO
  ENDIF
ENDDO

DEALLOCATE (ZSENDBUF, ZRECVBUF)

!     ------------------------------------------------------------------ 
IF (LHOOK) CALL DR_HOOK('GATHEREIGMD',1,ZHOOK_HANDLE)
END SUBROUTINE GATHEREIGMD
