SUBROUTINE GATHFLNM(KIND,KTAG,KNFL,KDOM,CDNOMA,KNOMA,PVALCO,&
 & KLONGA,KMASK,KDIM,PAVE,PMIN,PMAX,LDNORM,KPTR)  

!**** *GATHFLNM*  Communicate arrays of packed fields, Arpege fields names and norms 

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *GATHFLNM(KTAG,KNFL,KDOM,CDNOMA,KNOMA,PVALCO,KLONGA,KDIM,
!                         PAVE,PMIN,PMAX,LDNORM)

!        Explicit arguments :
!        --------------------
!         KIND   : 0 <=> SEND ; 1 <=> RECV
!         KTAG   : message tag
!         KNFL   : number of fields
!         KDOM   : number of subdomains
!         CDNOMA : name of the fields
!         KNOMA  : effective length of CDNOMA
!         PVALCO : packed fields
!         KLONGA : effective length (in REAL 64 bits) of PVALCO for each subdomain
!         KMASK  : binary mask ; =0 to skip the field (no write out) ; else =1
!         KDIM   : Declared dimension of PVALCO in calling routine
!         PAVE   : mean of field
!         PMIN   : min of field
!         PMAX   : max of field
!         LDNORM : .TRUE. to communicate norms
!         KPTR   : subdomains first point adress

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!       All data is collected in a single message as an integer array.

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group Meteo-France

!     Modifications.
!     --------------
!        Original: 98-02-19
!        Optimized : 98-06  : Only one message sent or received,      
!                             Introduce scatmsg and gathmsg
!                             Add norms parameters to message (8 integers)
!        Modified : 98-12-30 R. El Khatib several fields at once + simplifications
!    R. El Khatib : 99-12-27 KPTR necessary if more than 1 subdomain
!        D.Dent   : 00-09-01  MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib 28-Jul-2006 Porting to NEC
!     ---------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMTAG   , ONLY : MTAGPART
USE YOMMP    , ONLY : NPRCIDS
USE MPL_MODULE,ONLY : MPL_SEND ,MPL_RECV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNFL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDOM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTAG 
CHARACTER(LEN=16) ,INTENT(INOUT) :: CDNOMA(KNFL,KDOM) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNOMA(KNFL,KDOM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVALCO(KDIM,KNFL) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLONGA(KNFL,KDOM) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KMASK(KNFL,KDOM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAVE(KNFL,KDOM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMIN(KNFL,KDOM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMAX(KNFL,KDOM) 
LOGICAL           ,INTENT(IN)    :: LDNORM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPTR(KDOM) 
!     ZMESS   : message to send or recieve
!     ILEN    : message lenth
!     ILENMAX : max. message lenth

REAL   (KIND=JPRB), ALLOCATABLE :: ZMESS(:)

INTEGER(KIND=JPIM) ::  ILEN, ILENMAX, ILONGA, ILONGNOM, INOMAX, INORM, IMASK, &
 & ITAG, IWORD, JCHAR, JDOM, JFLD, ILENFLD, ISIZE, ISIZEFLD, &
 & IST, IEND, IZST, IZEND  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
!     ------------------------------------------------------------------

!*     1. PREPARATIONS
!         ------------

! Count the maximum number of words :
IF (LHOOK) CALL DR_HOOK('GATHFLNM',0,ZHOOK_HANDLE)
ISIZE=1 ! Actually should be the number of words a real variable is coded on.
IMASK=1
ILONGNOM=1
INOMAX=16
INORM=6
ILONGA=1
ILENFLD=ISIZE*KDIM
ILENMAX = KNFL*KDOM*(IMASK+ILONGNOM+INOMAX+INORM+ILONGA+ILENFLD)
ALLOCATE(ZMESS(ILENMAX))
ITAG=MTAGPART+KTAG

!*     2. CONCATENATE & SEND, OR RECIEVE & SPLIT
!         --------------------------------------

IF (KIND == 0) THEN

  ZMESS(:)=0
  IWORD=0

  DO JFLD=1,KNFL
    DO JDOM=1,KDOM

!     Write mask
      ZMESS(IWORD+1) = KMASK (JFLD,JDOM)
      IWORD=IWORD+1
      
      IF (KMASK (JFLD,JDOM) == 1) THEN
!       write field name length
        ZMESS(IWORD+1) = KNOMA(JFLD,JDOM)
        IWORD=IWORD+1
!       write field name 
        DO JCHAR=1,KNOMA(JFLD,JDOM)
          ZMESS(IWORD+JCHAR)=ICHAR(CDNOMA(JFLD,JDOM)(JCHAR:JCHAR))
        ENDDO
        IWORD=IWORD+KNOMA(JFLD,JDOM)
!       write norms        
        IF (LDNORM) THEN
          IST=IWORD+1
          IEND=IWORD+ISIZE
          ZMESS(IST:IEND)=TRANSFER(PAVE(JFLD,JDOM),ZMESS,ISIZE)
          IWORD=IWORD+ISIZE
          IST=IWORD+1
          IEND=IWORD+ISIZE
          ZMESS(IST:IEND)=TRANSFER(PMIN(JFLD,JDOM),ZMESS,ISIZE)
          IWORD=IWORD+ISIZE
          IST=IWORD+1
          IEND=IWORD+ISIZE
          ZMESS(IST:IEND)=TRANSFER(PMAX(JFLD,JDOM),ZMESS,ISIZE)
          IWORD=IWORD+ISIZE
        ENDIF
!       write field length
        ZMESS(IWORD+1) = KLONGA(JFLD,JDOM)
        IWORD=IWORD+1
!       write field        
        ISIZEFLD=ISIZE*KLONGA(JFLD,JDOM)
        IST=IWORD+1
        IEND=IWORD+ISIZEFLD
        IZST=KPTR(JDOM)
        IZEND=KPTR(JDOM)+KLONGA(JFLD,JDOM)-1
        CALL ECMWF_TRANSFER( ZMESS(IST:IEND),       JPRB*ISIZEFLD, &
         &                  PVALCO(IZST:IZEND,JFLD),JPRB*ISIZEFLD)
        IWORD=IWORD+ISIZEFLD
      ENDIF

    ENDDO
  ENDDO

  ILEN = IWORD

  CALL MPL_SEND(ZMESS(1:ILEN),KDEST=NPRCIDS(1),KTAG=ITAG, &
   & CDSTRING='GATHFLNM:')  

ELSE

  CALL MPL_RECV(ZMESS(1:ILENMAX),KTAG=ITAG, &
   & KOUNT=ILEN,CDSTRING='GATHFLNM:')  
  IF (ILENMAX < ILEN) THEN
    CALL ABOR1(' GATHFLNM : MESSAGE RECIEVED TOO BIG')
  ENDIF

  IWORD=0

  DO JFLD=1,KNFL
    DO JDOM=1,KDOM

!     Read mask
      KMASK (JFLD,JDOM) = ZMESS(IWORD+1)
      IWORD=IWORD+1

      IF (KMASK (JFLD,JDOM) == 1) THEN
!       Read field name lenght
        KNOMA(JFLD,JDOM) = ZMESS(IWORD+1)
        IWORD=IWORD+1
!       Read field name
        DO JCHAR=1,KNOMA(JFLD,JDOM)
          CDNOMA(JFLD,JDOM)(JCHAR:JCHAR) = CHAR(INT(ZMESS(IWORD+JCHAR)))
        ENDDO
        IWORD=IWORD+KNOMA(JFLD,JDOM)
!       Read norms
        IF (LDNORM) THEN
          IST=IWORD+1
          IEND=IWORD+ISIZE
          PAVE(JFLD,JDOM)=TRANSFER(ZMESS(IST:IEND),PAVE(JFLD,JDOM))
          IWORD=IWORD+ISIZE
          IST=IWORD+1
          IEND=IWORD+ISIZE
          PMIN(JFLD,JDOM)=TRANSFER(ZMESS(IST:IEND),PAVE(JFLD,JDOM))
          IWORD=IWORD+ISIZE
          IST=IWORD+1
          IEND=IWORD+ISIZE
          PMAX(JFLD,JDOM)=TRANSFER(ZMESS(IST:IEND),PAVE(JFLD,JDOM))
          IWORD=IWORD+ISIZE
        ENDIF
!       Read field length
        KLONGA(JFLD,JDOM) = ZMESS(IWORD+1)
        IWORD=IWORD+1
!       Read field 
        ISIZEFLD=ISIZE*KLONGA(JFLD,JDOM)
        IST=IWORD+1
        IEND=IWORD+ISIZEFLD
        IZST=KPTR(JDOM)
        IZEND=KPTR(JDOM)+KLONGA(JFLD,JDOM)-1
        CALL ECMWF_TRANSFER(PVALCO(IZST:IZEND,JFLD),JPRB*KLONGA(JFLD,JDOM),   &
         &                  ZMESS(IST:IEND),        JPRB*KLONGA(JFLD,JDOM))
        IWORD=IWORD+ISIZEFLD
      ENDIF 

    ENDDO
  ENDDO

  IF (IWORD /= ILEN) THEN
    CALL ABOR1('GATHFLNM : MESSAGE RECIEVED WITH WRONG LENGTH')
  ENDIF
  
ENDIF

DEALLOCATE(ZMESS)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GATHFLNM',1,ZHOOK_HANDLE)
END SUBROUTINE GATHFLNM
