subroutine gl2ll(kglatno,kglonno,kpe,klolatno,klolonno)

!****   gl2ll  ****  routine to convert from global lat/lon numbers
!                                          to local lat/lon numbers

!       purpose
!       -------

!**     Interface.
!       ----------
!         *call* *gl2ll(...) 

!        Explicit arguments : 
!        --------------------
!                kglatno  - global latitude number  (input)
!                kglonno  - global longitude number  (input)
!                kpe      - PE to whom the point belongs (output)
!                klolatno - local latitude number (output)
!                klolatno - local latitude number (output)

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group *ECMWF*

!     Modifications.
!     --------------
!         Original   : 96-04-04
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!----------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMP    , ONLY : NSTA     ,NONL     ,NPTRFRSTLAT,NFRSTLAT ,NLSTLAT ,NGPSET2PE
USE YOMCT0   , ONLY : N_REGIONS   ,N_REGIONS_NS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGLATNO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGLONNO 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPE 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLOLATNO 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLOLONNO 

INTEGER(KIND=JPIM) :: iaset, ibset, iend, ifirst, igl1, igl2, igloff, ist, ja, jb, jgl
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!----------------------------------------------------------------------
! find first a-set

IF (LHOOK) CALL DR_HOOK('GL2LL',0,ZHOOK_HANDLE)
iaset=1
do ja=1,N_REGIONS_NS
  igloff=nptrfrstlat(ja)
  if(ja > 1) then
    if(nlstlat(ja-1) == nfrstlat(ja)) then
      ifirst=nlstlat(ja-1)
      if(kglatno > ifirst) then
        iaset=ja
      elseif (kglatno == ifirst) then
        do jb=1,N_REGIONS(ja)
          if(nonl(igloff,jb) > 0.and.nsta(igloff,jb) <= kglonno) iaset=ja
        enddo
      endif
    else
      ifirst=nlstlat(ja-1)+1
      if(kglatno >= ifirst) iaset=ja
    endif
  endif
enddo

!  find the b-set

ibset=-999
igloff=nptrfrstlat(iaset)
igl1  =nfrstlat(iaset)
igl2  =nlstlat(iaset)
do jgl=igl1,igl2
  if(jgl == kglatno) then
    do jb=1,N_REGIONS(iaset)
      ist=nsta(igloff+jgl-igl1,jb)
      iend=nsta(igloff+jgl-igl1,jb)+nonl(igloff+jgl-igl1,jb)-1
      if(kglonno >= ist.and.kglonno <= iend) then
        ibset=jb
        klolonno=kglonno-ist+1
      endif
    enddo
  endif
enddo

if(iaset <= 0.or.iaset > N_REGIONS_NS.or.ibset <= 0 &
 & .or.ibset > N_REGIONS(iaset)) call abor1('gl2ll : error')  
klolatno=kglatno-nfrstlat(iaset)+1

!     find the PE that owns the grid point

kpe=NGPSET2PE(iaset,ibset)

IF (LHOOK) CALL DR_HOOK('GL2LL',1,ZHOOK_HANDLE)
end subroutine gl2ll

