SUBROUTINE ORCVGPF(PRCV,PFLD)

!     Purpose.
!     --------
!     Receive a distributed output grid point field from the required 
!     processors

!        Explicit arguments :
!        --------------------
!        PRCV   : global grid point field to be received
!        PFLD   : if nproc > 1 then PFLD is used as a buffer

!     Author.
!     -------
!        MPP Group *ECMWF*

!     Modifications.
!     --------------
!        Original : 96-03-19
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - PFLD does not contain polar values any longer, and non polar
!            values are stored from its element 1.
!        R. El Khatib : 99-12-28 remove poles rows
!        D.Dent   : 00-09-01  MPE to MPL conversion
!        Y.Tremolet: 01-01-19: some cleaning
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : NPRCIDS  ,NSTA     ,NONL     ,NPTRFRSTLAT,&
 & NFRSTLAT ,NLSTLAT ,NGPSET2PE
USE YOMTAG   , ONLY : MTAGDISTGP
USE YOMCT0   , ONLY : NPROC    ,N_REGIONS   ,N_REGIONS_NS
USE YOMGEM   , ONLY : NLOENG   ,NGPTOTMX ,NGPTOTG,NGPTOT
USE MPL_MODULE

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRCV(NGPTOTG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLD(NGPTOT) 
INTEGER(KIND=JPIM) :: IGL1, IGL2, IGLOFF, ILAST, ILEN, ILENB,&
 & ILOFF, IOFF, IRCV, ITAG, J, JA, JB, JGL, JLON  
REAL(KIND=JPRB) :: ZFLD (NGPTOTMX)
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ORCVGPF',0,ZHOOK_HANDLE)
DO JA=1,N_REGIONS_NS
  IGLOFF = NPTRFRSTLAT(JA)
  IGL1 = NFRSTLAT(JA)
  IGL2 = NLSTLAT(JA)
  IOFF = 0
  IF(JA > 1) THEN
    IF( NLSTLAT(JA-1) == NFRSTLAT(JA) )THEN
      ILAST = NLSTLAT(JA-1)-1
    ELSE
      ILAST = NLSTLAT(JA-1)
    ENDIF
    DO J=NFRSTLAT(1),ILAST
      IOFF = IOFF+NLOENG(J)
    ENDDO
  ENDIF
  DO JB=1,N_REGIONS(JA)
    IF( NPROC > 1 )THEN
      IRCV=NGPSET2PE(JA,JB)
      ITAG = MTAGDISTGP
      ILENB = NGPTOTMX
      CALL GSTATS(692,0)
      CALL MPL_RECV(ZFLD(1:ILENB),KSOURCE=NPRCIDS(IRCV),KTAG=ITAG, &
       & CDSTRING='ORCVGPF:')  
      CALL GSTATS(692,1)
      ILEN = 0
      ILOFF = 0
      CALL GSTATS(1881,0)
      DO JGL=IGL1,IGL2
        DO JLON=1,NONL(IGLOFF+JGL-IGL1,JB)
          PRCV(IOFF+ILOFF+NSTA(IGLOFF+JGL-IGL1,JB)+JLON-1)=ZFLD(ILEN+JLON)
        ENDDO
        ILEN = ILEN + NONL(IGLOFF+JGL-IGL1,JB)
        ILOFF = ILOFF + NLOENG(JGL)
      ENDDO
      CALL GSTATS(1881,1)
    ELSE
      ILEN = 0
      ILOFF = 0
      CALL GSTATS(1881,0)
      DO JGL=IGL1,IGL2
        DO JLON=1,NONL(IGLOFF+JGL-IGL1,JB)
          PRCV(IOFF+ILOFF+NSTA(IGLOFF+JGL-IGL1,JB)+JLON-1)=PFLD(ILEN+JLON)
        ENDDO
        ILEN = ILEN + NONL(IGLOFF+JGL-IGL1,JB)
        ILOFF = ILOFF + NLOENG(JGL)
      ENDDO
      CALL GSTATS(1881,1)
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('ORCVGPF',1,ZHOOK_HANDLE)
END SUBROUTINE ORCVGPF
