SUBROUTINE TRWVTOF(KSETV,KLOC,KFLDG,PSPEC,KFLDSETL,PSPECG,KFLDIOL,KFLDD,&
 & KFLDOFF,KSIZEPK)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NPROC   ,NPRTRW   ,NPRINTLEV   ,NPRTRV   ,LELAM
USE YOMDIM   , ONLY : NSPEC2   ,NMSMAX   ,NSMAX
USE YEMDIM   , ONLY : NISNAX
USE YEMLAP   , ONLY : NCPL4M
USE YOMMP    , ONLY : MYPROC   ,MYSETW   ,MYSETV   ,NPRCIDS
USE YOMMPG   , ONLY : NPOSSP   ,NDIM0G
USE YOMTAG   , ONLY : MTAGDISTSP
USE MPL_MODULE


!**** *TRWVTOF*  - TRANSPOSE A WAVE-DISTRIBUTED SET OF FIELDS 
!                  TO A FIELD-DISTRIBUTED SET OF SPECTRA

!     PURPOSE.
!     --------
!        To communicate a set of global spectral fields from W/V-set processors
!        to the processors in charge of global work prior to writing out fields 
!        (usually : packing).
!        The distribution per fields of global spectra is CONTIGUOUS among
!        a subset of processors. 

!**   INTERFACE.
!     ----------
!       *CALL* *TRWVTOF*

!        EXPLICIT ARGUMENTS
!        ------------------
!          KSETV   : V-set each output field belongs
!          KLOC    : local adress in PSPEC for each output field
!          KFLDG   : total number of fields
!          PSPEC   : wave&set-distributed fields
!          KFLDSETL: total number of fields in the input spectral array
!          PSPECG  : field-distributed spectra
!          KFLDIOL : local number of fields in the output spectral array
!          KFLDD   : number of field in each processor in output
!          KFLDOFF : fields offset after distribution of fields among processors.
!          KSIZEPK : (overdimensionned) size of 1 global spectrum

!        IMPLICIT ARGUMENTS
!        ------------------
!        See #include below.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 99-02-05
!        Modified : 00-09-01 by D.Dent : MPE to MPL conversion
!    R. El Khatib : 02-21-20 Fullpos B-level distribution
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Ouaraini & R. El Khatib: 03-Oct-06: The received global spectral array is ordered in according to JM=0,1,2,3,4,...
!     --------------------------------------------------------------------------

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSETL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDIOL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSIZEPK
INTEGER(KIND=JPIM),INTENT(IN)    :: KSETV(KFLDG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOC(KFLDG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPEC(KFLDSETL,NSPEC2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPECG(KSIZEPK,KFLDIOL) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDD(NPROC) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDOFF(NPROC) 
INTEGER(KIND=JPIM)               :: IDIM0GG1(0:NMSMAX)
INTEGER(KIND=JPIM)               :: IDIM0GG2(0:NSMAX)
REAL(KIND=JPRB)                  :: ZSPEC2(KSIZEPK)

!     IFLDA      : absolute pointer of the field to be sent/received 
!     ZBUF       : message

REAL(KIND=JPRB), ALLOCATABLE :: ZBUF(:)


INTEGER(KIND=JPIM) :: IFLDA, IPROC, ILEN,ILENB, ISENDER, JSETV, ISPEC2, ITAG,&
 & ITAGR, JFLD, JROC, JSETW, JSP, IOFF, IFIELDS  
INTEGER(KIND=JPIM) :: JN, JM, IPOS, ISP, ISPG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "set2pe.intfb.h"

!     ------------------------------------------------------------------

!*    0. PRELIMINAR CONTROL
!        ------------------

IF (LHOOK) CALL DR_HOOK('TRWVTOF',0,ZHOOK_HANDLE)
IF (KFLDD(MYPROC) /= KFLDIOL) THEN
  CALL ABOR1('TRWVTOF : INTERNAL ERROR ON KFLDIOL')
ENDIF

!*    1. SEND SPECTRAL DATA TO "IO" PROCESSORS
!        -------------------------------------

IF (NSPEC2 > 0 .AND. KFLDSETL > 0) THEN ! My set contains waves and fields
  DO JROC=1,NPROC
    IF (KFLDD(JROC) > 0) THEN ! scanning the I/O processors to send them data
      IF (NPRCIDS(JROC) == MYPROC) THEN
        DO JFLD=1,KFLDIOL
          IFLDA=KFLDOFF(MYPROC)+JFLD
          IF (KSETV(IFLDA) == MYSETV) THEN
            IOFF=0
            DO JSP=NPOSSP(MYSETW),NPOSSP(MYSETW)+NSPEC2-1
              IOFF=IOFF+1
              PSPECG(JSP,JFLD)=PSPEC(KLOC(IFLDA),IOFF)
            ENDDO
          ENDIF
        ENDDO
      ELSE
        IFIELDS=0
        DO JFLD=1,KFLDD(JROC)
          IFLDA=KFLDOFF(JROC)+JFLD
          IF (KSETV(IFLDA) == MYSETV) THEN
            IFIELDS=IFIELDS+1
          ENDIF
        ENDDO 
        IF (IFIELDS > 0) THEN
          ALLOCATE(ZBUF(NSPEC2*IFIELDS))
          IOFF=0
          DO JFLD=1,KFLDD(JROC)
            IFLDA=KFLDOFF(JROC)+JFLD
            IF (KSETV(IFLDA) == MYSETV) THEN
              DO JSP=1,NSPEC2
                ZBUF(IOFF+JSP)=PSPEC(KLOC(IFLDA),JSP)
              ENDDO
              IOFF=IOFF+NSPEC2
            ENDIF
          ENDDO
          ITAG=MTAGDISTSP+MYPROC
          CALL MPL_SEND(ZBUF,KDEST=NPRCIDS(JROC),KTAG=ITAG,CDSTRING='TRWVTOF:')
          IF (NPRINTLEV == 2) THEN
            WRITE(NULOUT,*) 'TRWVTOF : send spectral field to ',JROC,&
             & ' with tag ',ITAG  
          ENDIF
          DEALLOCATE(ZBUF)
        ENDIF
      ENDIF
    ENDIF
  ENDDO
ENDIF

!*     2. "IO" PROCESSORS RECEIVE DATA
!         ----------------------------
IF (LELAM)  THEN
  IPOS=1
  DO JM=0,NMSMAX
    IDIM0GG1(JM)=IPOS
    IPOS=IPOS+NCPL4M(JM)
  ENDDO
ELSE
  IPOS=1
  DO JM=0,NSMAX
    IDIM0GG2(JM)=IPOS
    IPOS=IPOS+(NSMAX+1-JM)*2
  ENDDO
ENDIF

IF (KFLDIOL > 0) THEN ! I should receive fields
  DO JSETV=1,NPRTRV
    IFIELDS=0
    DO JFLD=1,KFLDIOL
      IFLDA=KFLDOFF(MYPROC)+JFLD
      IF (KSETV(IFLDA) == JSETV) THEN
        IFIELDS=IFIELDS+1
      ENDIF
    ENDDO
    IF (IFIELDS > 0) THEN
      DO JSETW=1,NPRTRW
        ISPEC2=NPOSSP(JSETW+1)-NPOSSP(JSETW)
        IF (ISPEC2 > 0) THEN
          ILEN=ISPEC2*IFIELDS
          CALL SET2PE(IPROC,0,0,JSETW,JSETV)
          IF (NPRCIDS(IPROC) /= MYPROC) THEN
            ITAG = MTAGDISTSP+IPROC
            ALLOCATE(ZBUF(ILEN))
            CALL MPL_RECV(ZBUF,KSOURCE=NPRCIDS(IPROC),KTAG=ITAG,KFROM=ISENDER,&
             & KRECVTAG=ITAGR,CDSTRING='TRWVTOF:',KOUNT=ILENB)  
            IF (ILENB /= ILEN) THEN
              WRITE(NULOUT,*) 'ILENB = ',ILENB,' ILEN = ',ILEN
              CALL ABOR1('TRWVTOF : INVALID RECEIVE MESSAGE LENGHT')
            ELSEIF (ISENDER /= NPRCIDS(IPROC)) THEN
              WRITE(NULOUT,*) 'KFROM = ',ISENDER, ' KSOURCE = ',NPRCIDS(IPROC)
              CALL ABOR1('TRWVTOF : INVALID RECEIVE MESSAGE PROCESSOR')
            ELSEIF (NPRINTLEV == 2) THEN
              WRITE(NULOUT,*) 'TRWVTOF : recv spectral field from ',ISENDER,&
               & ' with tag ',ITAGR  
            ENDIF
            IOFF=1
            DO JFLD=1,KFLDIOL
              IFLDA=KFLDOFF(MYPROC)+JFLD
              IF (KSETV(IFLDA) == JSETV) THEN
                PSPECG(NPOSSP(JSETW):NPOSSP(JSETW)+ISPEC2-1,JFLD)=&
                 & ZBUF(IOFF:IOFF+ISPEC2-1)  
                IOFF=IOFF+ISPEC2
              ENDIF
            ENDDO
            DEALLOCATE(ZBUF)
          ENDIF
        ENDIF
      ENDDO
    ENDIF
  ENDDO

  IF (LELAM)  THEN
    DO JFLD=1,KFLDIOL
      ZSPEC2=0.0_JPRB   ! Tableau intermediaire pour ne pas ecraser PSPECG
      DO JM=0,NMSMAX
        DO JN=0,NISNAX(JM)
          ISP=NDIM0G(JM)+4*JN
          ISPG=IDIM0GG1(JM)+4*JN
          ZSPEC2(ISPG:ISPG+3)=PSPECG(ISP:ISP+3,JFLD)
        ENDDO
      ENDDO
      PSPECG(:,JFLD)=ZSPEC2(:)
    ENDDO
  ELSE
    DO JFLD=1,KFLDIOL
      ZSPEC2=0.0_JPRB   ! Tableau intermediaire pour ne pas ecraser PSPECG
      DO JM=0,NSMAX
        DO JN=JM,NSMAX
          ISP=NDIM0G(JM)+2*(JN-JM)
          ISPG=IDIM0GG2(JM)+2*(JN-JM)
          ZSPEC2(ISPG)=PSPECG(ISP,JFLD)
          ZSPEC2(ISPG+1)=PSPECG(ISP+1,JFLD)
        ENDDO
      ENDDO
      PSPECG(:,JFLD)=ZSPEC2(:)
    ENDDO
  ENDIF
ENDIF
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRWVTOF',1,ZHOOK_HANDLE)
END SUBROUTINE TRWVTOF
