!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACACON ( KIDIA,KFDIA,KLON,&
 !-----------------------------------------------------------------------
 ! - INPUT LOGICAL
 & LDSIMP,&
 ! - INPUT 1D .
 & PEXPN,PLSCP,PPART,PLSM,PNEIJ,PTS,&
 ! - INPUT/OUTPUT 1D .
 & PQIST,PQLST,PQNST,PQRST,PTST,&
 ! - OUTPUT 1D .
 & PACONI,PACONL,PACORL,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACACON* - CALCULS D'AUTO-CONVERSION.

!     Sujet.
!     ------
!     - ROUTINE DE CALCUL ACTIF (SUR UN NIVEAU).
!       CALCUL DE TROIS INCREMENTS D'AUTO-CONVERSION.
!     - COMPUTATION OF THREE AUTO-CONVERSION INCREMENTS.

!**   Interface.
!     ----------
!        *CALL* *ACACON*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLMPHYS" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.

! - ARGUMENT DE CONTROLE D'UNE VERSION SIMPLIFIEE (TOTALE AUTO-CONVERSION).

! LDSIMP     : IF .TRUE., CAS SIMPLIFIE.

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 1D .

! PEXPN      : FONCTION DE DEPENDANCE EN TEMPERATURE DES PROCESSUS < 0C.
! PLSCP      : CHALEUR DE FUSION SUR CHALEUR MASSIQUE A PRESSION CONSTANTE.
! PPART      : PROPORTION DE LA MAILLE CONCERNEE.
! PLSM       : LAND/SEA MASK.
! PNEIJ      : PROPORTION OF SNOW-COVERED LAND.
! PTS        : SURFACE TEMPERATURE.

!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTRE-SORTIE.
!     --------------------------

! - 1D .

! PQIST      : QUANTITE COURANTE D'EAU SOLIDE NUAGEUSE.
! PQLST      : QUANTITE COURANTE D'EAU LIQUIDE NUAGEUSE.
! PQNST      : QUANTITE COURANTE D'EAU SOLIDE PRECIPITANTE.
! PQRST      : QUANTITE COURANTE D'EAU LIQUIDE PRECIPITANTE.
! PTST       : VALEUR COURANTE DE LA TEMPERATURE A L'ECHELLE DE LA MAILLE.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - 1D .

! PACONI     : INCREMENT D'AUTO-CONVERSION GLACE => NEIGE.
! PACONL     : INCREMENT D'AUTO-CONVERSION EAU LIQUIDE => NEIGE (W-B-F).
! PACORL     : INCREMENT D'AUTO-CONVERSION EAU LIQUIDE => PLUIE.

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY1/
! COMMON/YOMPHY2/

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     Methode.
!     --------

!     Auteur.
!     -------
!        07-02, J.-F. Geleyn (extraction from APLMPHYS).

!     Modifications.
!     --------------
!        07-03, Option of the ARPEGE scheme - J.-F. Geleyn & J.-M. Piriou
!        07-12, Full auto-conversion simplification - R. Brozkova

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : NPHYREP
USE YOMCST   , ONLY : RPI      ,RTT
USE YOMPHY0  , ONLY : RAUTEFR  ,RAUTEFS  ,RQLCR    ,RQICRMAX ,RQICRMIN ,&
 &                    RWBF1    ,RWBF2    ,RAUTSBET ,RQICRT1  ,RQICRT2  ,&
 &                    RQICRSN
USE YOMPHY1  , ONLY : TMERGL
USE YOMPHY2  , ONLY : TSPHY

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
LOGICAL           ,INTENT(IN)    :: LDSIMP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXPN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSCP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPART(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJ(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQIST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQNST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQRST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTST(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PACONI(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PACONL(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PACORL(KLON) 
CHARACTER(LEN=6)  ,INTENT(IN)    :: CDLOCK 

INTEGER(KIND=JPIM) :: JLON

LOGICAL :: LLA0MPS, LLARPSC

REAL(KIND=JPRB) :: ZEPS1, ZHSEFN, ZHSEFL, ZWBFEF, ZALF, ZQLCR, ZQICR, ZPOISS, &
 & ZAUTEFR, ZAUTEFS, ZARG1, ZARG2, ZALPH, ZBETA, ZDELT, ZQCR, ZFACICE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACACON',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'ACACON') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

LLA0MPS=.TRUE.
LLARPSC=.FALSE.

ZEPS1=1.E-10_JPRB

IF (LLA0MPS) THEN

  ZPOISS=2.0_JPRB/SQRT(RPI)

  DO JLON=KIDIA,KFDIA
    IF (.NOT.LDSIMP) THEN
      ZAUTEFR=RAUTEFR
      ZAUTEFS=RAUTEFS*PEXPN(JLON)
      ZQLCR=RQLCR*ZPOISS
      ZQICR=(RQICRMIN+(RQICRMAX-RQICRMIN)*PEXPN(JLON))*ZPOISS
      ZALF=PQIST(JLON)/MAX(ZEPS1,PQLST(JLON)+PQIST(JLON))
      ZHSEFN=ZAUTEFS*TSPHY*(1.0_JPRB-EXP(-(PQIST(JLON)/ZQICR)**2))
      PACONI(JLON)=ZHSEFN*PQIST(JLON)/(1.0_JPRB+ZHSEFN)
      ZHSEFL=ZAUTEFR*TSPHY*(1.0_JPRB-EXP(-(PQLST(JLON)/ZQLCR)**2))
      ZWBFEF=ZAUTEFR*TSPHY*RWBF1*(1.0_JPRB-EXP(-(PQLST(JLON)*PQIST(JLON))&
       &/(ZQLCR*ZQICR*RWBF2**2)))*ZALF*(1.0_JPRB-ZALF)
      PACORL(JLON)=ZHSEFL*(PQLST(JLON)/(1.0_JPRB+ZHSEFL+ZWBFEF))
      PACONL(JLON)=ZWBFEF*(PQLST(JLON)/(1.0_JPRB+ZHSEFL+ZWBFEF))
      PQLST(JLON)=PQLST(JLON)-PACORL(JLON)-PACONL(JLON)
      PQIST(JLON)=PQIST(JLON)-PACONI(JLON)
      PQRST(JLON)=PQRST(JLON)+PACORL(JLON)
      PQNST(JLON)=PQNST(JLON)+PACONL(JLON)+PACONI(JLON)
      PTST(JLON)=PTST(JLON)+PACONL(JLON)*PLSCP(JLON)*PPART(JLON)
    ELSE
      ZQLCR=RQLCR*ZPOISS
      ZQICR=(RQICRMIN+(RQICRMAX-RQICRMIN)*PEXPN(JLON))*ZPOISS
      ZALF=PQIST(JLON)/MAX(ZEPS1,PQLST(JLON)+PQIST(JLON))
      PACONI(JLON)=PQIST(JLON)
      ZHSEFL=(1.0_JPRB-EXP(-(PQLST(JLON)/ZQLCR)**2))
      ZWBFEF=RWBF1*(1.0_JPRB-EXP(-(PQLST(JLON)*PQIST(JLON))&
       &/(ZQLCR*ZQICR*RWBF2**2)))*ZALF*(1.0_JPRB-ZALF)
      PACONL(JLON)=ZWBFEF*(PQLST(JLON)/MAX(ZEPS1,ZHSEFL+ZWBFEF))
      PACORL(JLON)=PQLST(JLON)-PACONL(JLON)
      PQLST(JLON)=PQLST(JLON)-PACORL(JLON)-PACONL(JLON)
      PQIST(JLON)=PQIST(JLON)-PACONI(JLON)
      PQRST(JLON)=PQRST(JLON)+PACORL(JLON)
      PQNST(JLON)=PQNST(JLON)+PACONL(JLON)+PACONI(JLON)
      PTST(JLON)=PTST(JLON)+PACONL(JLON)*PLSCP(JLON)*PPART(JLON)
    ENDIF
  ENDDO

ENDIF

IF (LLARPSC) THEN

  ZARG1=2.0_JPRB*RQICRMAX*(1.0_JPRB-0.999_JPRB)/(RQICRMAX-RQICRMIN)-1.0_JPRB
  ZARG2=2.0_JPRB*(RQICRMAX-1.5_JPRB*RQICRMIN)/(RQICRMAX-RQICRMIN)-1.0_JPRB
  ZARG1=0.5_JPRB*LOG(ABS((1.0_JPRB+ZARG1)/(1.0_JPRB-ZARG1)))
  ZARG2=0.5_JPRB*LOG(ABS((1.0_JPRB+ZARG2)/(1.0_JPRB-ZARG2)))
  ZALPH=(ZARG1-ZARG2)/(RQICRT2-RQICRT1)
  ZBETA=ZARG1-RQICRT2*ZALPH

  DO JLON=KIDIA,KFDIA
    IF (.NOT.LDSIMP) THEN
      ZAUTEFR=RAUTEFR
      ZDELT=PTST(JLON)-RTT
      ZAUTEFS=RAUTEFS*EXP(RAUTSBET*ZDELT)
      ZQCR=RQICRMAX-(RQICRMAX-RQICRMIN)*0.5_JPRB*(1.0_JPRB+TANH(ZALPH*ZDELT&
       &+ZBETA))
      ZFACICE=PLSM(JLON)*PNEIJ(JLON)+(1.0_JPRB-PLSM(JLON))&
       &*MAX(0.0_JPRB,SIGN(1.0_JPRB,TMERGL-PTS(JLON)))
      ZQCR=ZQCR*(1.0_JPRB-ZFACICE*(1.0_JPRB-RQICRSN))
      PACONI(JLON)=MAX(0.0_JPRB,(1.0_JPRB-EXP(-ZAUTEFS*TSPHY))*(PQIST(JLON)&
       &-ZQCR))
      PACORL(JLON)=MAX(0.0_JPRB,(1.0_JPRB-EXP(-ZAUTEFR*TSPHY))*(PQLST(JLON)&
       &-RQLCR))
      PACONL(JLON)=0.0_JPRB
      PQLST(JLON)=PQLST(JLON)-PACORL(JLON)
      PQIST(JLON)=PQIST(JLON)-PACONI(JLON)
      PQRST(JLON)=PQRST(JLON)+PACORL(JLON)
      PQNST(JLON)=PQNST(JLON)+PACONI(JLON)
    ELSE
      PACONI(JLON)=PQIST(JLON)
      PACORL(JLON)=PQLST(JLON)
      PACONL(JLON)=0.0_JPRB
      PQLST(JLON)=PQLST(JLON)-PACORL(JLON)
      PQIST(JLON)=PQIST(JLON)-PACONI(JLON)
      PQRST(JLON)=PQRST(JLON)+PACORL(JLON)
      PQNST(JLON)=PQNST(JLON)+PACONI(JLON)
    ENDIF
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('ACACON',1,ZHOOK_HANDLE)
END SUBROUTINE ACACON



