!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACCDEV ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPRS,PAPRSF,PCP,PDELP,PQ,PQI,PQL,PQN,PQR,PQW,PR,PT,PNEBCOND,PHCRICS,&
 & PRMF,PQSATS,PNCV,&
 ! - INPUT  1D .
 & PLSM,PNEIJ,PTS,&
 ! - OUTPUT 2D .
 & PFASL,PFASN,PFCSQL,PFCSQN,PFESL,PFESN,PFPLSL,PFPLSN,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACCDEV* - CALCUL DES FLUX DE CONDENSATION ET/OU PRECIPITATION
!                STRATIFORME.

!     Sujet.
!     ------
!     - ROUTINE DE CALCUL ACTIF .
!       CALCUL DES FLUX DE CONDENSATION ET/OU PRECIPITATION STRATIFORME
!              (LIQUIDE ET NEIGE) .
!     - COMPUTATION OF STRATIFORM CONDENSATION AND/OR PRECIPITATION FLUXES
!              (WATER AND SNOW) .

!     - CALCUL DES PSEUDO-FLUX LIES AUX PRECIPITATIONS STRATIFORMES
!              (LIQUIDE ET NEIGE) .
!     - COMPUTATION OF PSEUDO-FLUXES LINKED TO STRATIF. PRECIPITATIONS
!              (WATER AND SNOW) .

!**   Interface.
!     ----------
!        *CALL* *ACCDEV*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPRS      : PRESSION AUX DEMI-NIVEAUX.

! - 2D (1:KLEV) .

! PAPRSF     : PRESSION AUX NIVEAUX PLEIN.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE.
! PDELP      : EPAISSEUR EN PRESSION DE LA COUCHE.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PQW        : HUMIDITE SPECIFIQUE DU THERMOMETRE MOUILLE.
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PT         : TEMPERATURE.
! PNEBCOND   : NEBULOSITE DIAGNOSTIQUE POUR CONDENSATION/EVAPORATION.
! PHCRICS    : HUMIDITE RELATIVE CRITIQUE POUR CONDENSATION/EVAPORATION.
! PRMF       : PROPORTION DE LA GLACE.
! PQSATS     : HUMIDITE SPECIFIQUE DE SATURATION (SCHEMA COND./EVAP.).
! PNCV       : CONVECTIVE CLOUDINESS (HISTORIC).

! - 1D 

! PLSM       : INDICE TERRE/MER.
! PNEIJ      : PROPORTION DE SOL ENNEIGE.
! PTS        : TEMPERATURE DE SURFACE.

! ADDITIONAL PROGNOSTIC VARIABLES

! PQI        : RATIO OF SUSPENDED ICE
! PQL        : RATIO OF SUSPENDED LIQUID WATER
! PQN        : RATIO OF SNOW
! PQR        : RATIO OF RAIN WATER

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - 2D (0:KLEV) .

! PFPLSL     : PRECIPITATIONS STRATIFORMES SOUS FORME LIQUIDE.
! PFPLSN     : PRECIPITATIONS STRATIFORMES SOUS FORME NEIGE.

! PROGNOSTIC PSEUDO FLUXES (WITH RESPECT TO VAPOUR)

! PFASL      : STRATIFORM AUTOCONVERSION (LIQUID)
! PFASN      : STRATIFORM AUTOCONVERSION (SOLID)
! PFCSQL     : STRATIFORM CONDENSATION (LIQUID)
! PFCSQN     : STRATIFORM CONDENSATION (SOLID)
! PFESL      : STRATIFORM EVAPORATION OF RAIN
! PFESN      : STRATIFORM EVAPORATION OF SNOW 

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     Methode.
!     --------

!     Auteur.
!     -------
!        07-02, R. Brozkova.

!     Modifications.
!     --------------
!     S. Ivatek-Sahdan 29.03.2007. : passive bug reported by R. Brozkova
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG        ,RDT      ,RD       ,&
 & RV       ,RGAMD    ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    
USE YOMPHY   , ONLY : L3MT, LSTRAPRO, LNEIGE, LSMNIMBT, LSMGCDEV, LXRCDEV
USE YOMPHY0  , ONLY : RSMDTX, RDTFAC
USE YOMPHY2  , ONLY : TSPHY

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEBCOND(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHCRICS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSATS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNCV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJ(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFASL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFASN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCSQL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCSQN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFESL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFESN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLSL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLSN(KLON,0:KLEV) 
CHARACTER(LEN=6) ,INTENT(IN)    :: CDLOCK 

REAL(KIND=JPRB) :: ZPOID(KLON,KLEV),ZIPOI(KLON,KLEV),ZDQ(KLON,KLEV)&
 & ,ZFCSQL(KLON,0:KLEV),ZFCSQN(KLON,0:KLEV),ZQLTMP(KLON,KLEV),ZQITMP(KLON,KLEV)&
 & ,ZQTMP(KLON,KLEV),ZLHV(KLON,KLEV),ZLHS(KLON,KLEV),ZDQM(KLON,KLEV)&
 & ,ZFALLR(KLON,KLEV),ZFALLS(KLON,KLEV),ZRCVOTT(KLON,0:KLEV)&
 & ,ZTCORR(KLON,KLEV),ZMELNET(KLON,KLEV)

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZEPS1, ZGDTI, ZRMLOC, ZCONL, ZCONI, ZTESTB, ZALF, &
 & ZQ, ZQL, ZQI, ZQVN, ZNEBM1, ZDQVN, ZTEST, ZRNEB, ZQVCS, &
 & ZQT, ZQC9, ZICE, ZEPQC, ZNEIGE, ZSQRT6, ZRDSRV, ZDELTA, ZRHC, &
 & ZDQI, ZDQL, ZFRACON, ZLEFF, ZTLIQ, ZSIGS, ZRATQ, ZRAT2, ZQC, &
 & ZQSAT, ZNEBT, ZZ, ZZRHC 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "aplmphys.intfb.h"
#include "fctdoi.h"
#include "fcttrm.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACCDEV',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'ACCDEV') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!     I - CALCUL DE PARAMETRES DERIVES ET CONSTANTES DE SECURITE.

!         COMPUTATION OF DERIVED PARAMETERS AND SECURITY CONSTANTS.

ZGDTI=1.0_JPRB/(RG*TSPHY)
ZEPS1=1.E-10_JPRB

ZEPQC=1.E-14_JPRB
ZSQRT6=SQRT(6._JPRB)
ZRDSRV=RD/RV

IF (LNEIGE) THEN
  ZNEIGE=1.0_JPRB
ELSE
  ZNEIGE=0.0_JPRB
ENDIF

!*
!     ------------------------------------------------------------------
!     II - CALCULS PRELIMINAIRES DE L'EPAISSEUR EN PRESSION DES COUCHES
!     DIVISEE PAR G*DT POUR CHAQUE NIVEAU. INVERSE DE CETTE QUANTITE.

!          PRELIMINARY COMPUTATIONS OF THE LAYER'S PRESSURE THICKNESS
!     DIVIDED BY G*DT FOR ALL LEVELS. INVERSE OF THIS QUANTITY.

! - TEMPORAIRE(S) 2D (1:KLEV) .

! ZPOID     : DP/(RG*DT) POUR UNE COUCHE ET UN PAS DE TEMPS DONNES.
!           : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI     : INVERSE DE ZPOID.
!           : INVERSE OF ZPOID.

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZPOID(JLON,JLEV)=PDELP(JLON,JLEV)*ZGDTI
    ZIPOI(JLON,JLEV)=1.0_JPRB/ZPOID(JLON,JLEV)
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     III - CALCULS DE CONDENSATION-EVAPORATION.

!           CONDENSATION-EVAPORATION COMPUTATIONS.

! - TEMPORAIRE(S) 2D (0:KLEV) .

! ZFCSQL    : PSEUDO-FLUX TEMPORAIRE POUR CONDENSATION MOINS EVAPORATION.
!           : TEMPORARY PSEUDO-FLUX FOR CONDENSATION MINUS EVAPORATION.
! ZFCSQN    : PSEUDO-FLUX TEMPORAIRE POUR CONDENSATION SOLIDE MOINS SUBLIMATION.
!           : TEMPORARY PSEUDO-FLUX FOR SOLID CONDENSATION MINUS SUBLIMATION.
! ZLHS      : CHALEUR LATENTE DE SUBLIMATION.
!           : SUBLIMATION LATENT HEAT.
! ZLHV      : CHALEUR LATENTE DE VAPORISATION.
!           : VAPORISATION LATENT HEAT.
! ZQTMP     : PQ EVOLUTIF PROTEGE CONTRE LES VALEURS NEGATIVES.
!           : NEGATIVE-PROTECTED EVOLUTIVE VALUE OF PQ.
! ZQITMP    : PQI EVOLUTIF PROTEGE CONTRE LES VALEURS NEGATIVES.
!           : NEGATIVE-PROTECTED EVOLUTIVE VALUE OF PQI.
! ZQLTMP    : PQL EVOLUTIF PROTEGE CONTRE LES VALEURS NEGATIVES.
!           : NEGATIVE-PROTECTED EVOLUTIVE VALUE OF PQL.
! ZRCVOTT   : DEGRE D'INHOMOGENEITE SUPPOSE DES PRECIPITATIONS.
!           : INHOMOGENEITY DEGREE OF PRECIPITATIONS.

! - TEMPORAIRE(S) 2D (1:KLEV) .

! ZDQM      : DEFICIT MAXIMUM DE SATURATION.
!           : MAXIMUM OF SATURATION DEFICIT.

DO JLON=KIDIA,KFDIA
  ZFCSQL(JLON,KTDIA-1)=0.0_JPRB
  ZFCSQN(JLON,KTDIA-1)=0.0_JPRB
ENDDO

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZLHV(JLON,JLEV)=FOLH(PT(JLON,JLEV),0.0_JPRB)
    ZLHS(JLON,JLEV)=FOLH(PT(JLON,JLEV),1.0_JPRB)
  ENDDO
ENDDO

ZRCVOTT(:,:)=0.0_JPRB
ZTCORR(:,:)=0.0_JPRB
ZDQM(:,:)=PQW(:,:)

IF(LXRCDEV) THEN

!     ------------------------------------------------------------------
!     IIIa - SCHEMA D'APRES ACPLUIE_PROG. 

!            SCHEME AFTER ACPLUIE_PROG.

! - TEMPORAIRE(S) 2D (1:KLEV) .

! ZDQ       : CORRECTION DE Q TOTAL POUR PASSER A L'EQUILIBRE SATURE.
!           : CORRECTION OF TOTAL Q TO REACH THE SATURATION BALANCE.

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
      ZRNEB=PNEBCOND(JLON,JLEV)/MAX(ZEPS1,PNEBCOND(JLON,JLEV)+PNCV(JLON,JLEV)&
       &-PNEBCOND(JLON,JLEV)*PNCV(JLON,JLEV))
      ZQVCS=MAX(0.0_JPRB,PQ(JLON,JLEV)-PQW(JLON,JLEV)*(PNEBCOND(JLON,JLEV)&
       &+PNCV(JLON,JLEV)-PNEBCOND(JLON,JLEV)*PNCV(JLON,JLEV)))
      ZQ=(PQ(JLON,JLEV)-ZQVCS)*ZRNEB+ZQVCS/MAX(ZEPS1,1.0_JPRB-PNCV(JLON,JLEV))
      ZQL=PQL(JLON,JLEV)*ZRNEB
      ZQI=PQI(JLON,JLEV)*ZRNEB
      ZALF=ZQI/MAX(ZEPS1,ZQL+ZQI)

      ZNEBM1=1.0_JPRB-PNEBCOND(JLON,JLEV)
      ZQVN=PQW(JLON,JLEV)*(1.0_JPRB+(PHCRICS(JLON,JLEV)-1.0_JPRB)*ZNEBM1)

      ZDQVN=ZQ-ZQVN
      ZTEST=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZQVN-(ZQ+ZQL+ZQI)))
      ZTESTB=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZDQVN))
      ZRMLOC=(1.0_JPRB-ZTESTB)*ZALF+ZTESTB*PRMF(JLON,JLEV)
      ZCONL=(1.0_JPRB-ZRMLOC)*(1.0_JPRB-ZTEST)*ZDQVN-ZTEST*ZQL
      ZCONI=ZRMLOC*(1.0_JPRB-ZTEST)*ZDQVN-ZTEST*ZQI
      ZDQ(JLON,JLEV)=PQW(JLON,JLEV)-MIN(ZQVN,ZQ+ZQL+ZQI)

      ZFCSQL(JLON,JLEV)=ZFCSQL(JLON,JLEV-1)+ZPOID(JLON,JLEV)*(1.0_JPRB&
       &-PNCV(JLON,JLEV))*ZCONL
      ZFCSQN(JLON,JLEV)=ZFCSQN(JLON,JLEV-1)+ZPOID(JLON,JLEV)*(1.0_JPRB&
       &-PNCV(JLON,JLEV))*ZCONI

      ZQLTMP(JLON,JLEV)=MAX(0.0_JPRB,ZQL+ZCONL)
      ZQITMP(JLON,JLEV)=MAX(0.0_JPRB,ZQI+ZCONI)
      ZQTMP(JLON,JLEV)=MAX(0.0_JPRB,ZQ-ZCONL-ZCONI)
    ENDDO
  ENDDO

ENDIF ! LXRCDEV


IF(LSMGCDEV) THEN

!     ------------------------------------------------------------------
!     IIIb - SCHEMA SMITH-GERARD.

!            SMITH-GERARD SCHEME.

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

! Total cloud fraction

      ZNEBT=PNEBCOND(JLON,JLEV)&
       &+PNCV(JLON,JLEV)-PNEBCOND(JLON,JLEV)*PNCV(JLON,JLEV)
      ZZ=MAX(0.0_JPRB,SIGN(1.0_JPRB, ZNEBT-ZEPS1))

! Convective proportion

      ZFRACON=ZZ*PNCV(JLON,JLEV)/(ZNEBT+(1.0_JPRB-ZZ))
      ZQC9=PQI(JLON,JLEV)+PQL(JLON,JLEV)
      ZQT=ZQC9+PQ(JLON,JLEV)
      ZRHC=PHCRICS(JLON,JLEV)
      ZICE=PRMF(JLON,JLEV)

!-----------------
! TOTAL CONDENSATE
!-----------------

      ZQSAT=PQSATS(JLON,JLEV)

      ZLEFF=ZLHV(JLON,JLEV)*(1.0_JPRB-ZICE)+ZLHS(JLON,JLEV)*ZICE
      ZTLIQ=PT(JLON,JLEV)-&
     & (ZLHV(JLON,JLEV)*PQL(JLON,JLEV)+ZLHS(JLON,JLEV)*PQI(JLON,JLEV))/&
     & PCP(JLON,JLEV)

      ZSIGS=(1.0_JPRB-ZRHC) *ZQSAT/&
     &      (ZSQRT6 * (1.0_JPRB + (ZLEFF * ZLEFF * ZQSAT * ZRDSRV)/&
     &      (PR(JLON,JLEV) * ZTLIQ * ZTLIQ * PCP(JLON,JLEV))) )

      ZRATQ=ZQT/ZQSAT
      ZZRHC=MIN(ZRHC,1.0_JPRB-ZEPQC)
      ZRATQ=MAX(ZZRHC,MIN(2.0_JPRB-ZZRHC,ZRATQ))
      ZRAT2=(ZRATQ-1.0_JPRB)/(1.0_JPRB-ZZRHC)

      ZTEST=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZRATQ-1.0_JPRB))
      ZQC=(ZTEST*(6.0_JPRB*ZRAT2+(1.0_JPRB-ZRAT2)**3)+&
        &(1.0_JPRB-ZTEST)*(1.0_JPRB+ZRAT2)**3)/ZSQRT6
      ZQC=ZQC*ZSIGS

! Control of warming.

      ZQC=MIN(ZQC, MAX(0.0_JPRB,ZQC9)+RSMDTX*PCP(JLON,JLEV)/ZLEFF)

! Direct protection  of convective condensate against evaporation:

      ZQC=MAX(ZQC, ZQC9*ZFRACON)
      
! Repartition by FONICE.

      ZQL=(1.0_JPRB-ZICE)*ZQC
      ZQI=ZQC-ZQL

!-----------------------------------------
! PREVENT MELTING AT NEGATIVE TEMPERATURE.
!-----------------------------------------

      IF (LSMNIMBT) THEN

! Prevent melting of existing ice below RTT.

        ZDELTA=ZNEIGE*MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT(JLON,JLEV)))
        ZZ=MAX(0.0_JPRB, SIGN(1.0_JPRB,PQI(JLON,JLEV)-ZQI))*ZDELTA

        ZQI=ZQI*(1.0_JPRB-ZZ) + &
          & ZZ* PQI(JLON,JLEV)*MIN(1.0_JPRB,ZQC/MAX(ZEPQC,ZQC9))

      ENDIF

! Additional security required to avoid rounding error problems.

      ZQI=MIN(ZQC,ZQI)
      ZQL=MAX(0.0_JPRB,ZQC-ZQI)

!---------------------------------------------------
! COMPUTE LARGE-SCALE CONDENSATION/EVAPORATION FLUX.
!---------------------------------------------------

      ZDQL=ZQL - PQL(JLON,JLEV)
      ZDQI=ZQI - PQI(JLON,JLEV)

! Stratiform fluxes:

      ZFCSQL(JLON,JLEV) = ZFCSQL(JLON,JLEV-1)+ ZPOID(JLON,JLEV)*ZDQL
      ZFCSQN(JLON,JLEV) = ZFCSQN(JLON,JLEV-1)+ ZPOID(JLON,JLEV)*ZDQI

! Update in case we call microphysics (LSTRAPRO):

      ZQLTMP(JLON,JLEV)=MAX(0.0_JPRB,PQL(JLON,JLEV)+ZDQL)
      ZQITMP(JLON,JLEV)=MAX(0.0_JPRB,PQI(JLON,JLEV)+ZDQI)
      ZQTMP(JLON,JLEV)=MAX(0.0_JPRB,PQ(JLON,JLEV)-ZDQL-ZDQI)
      ZDQ(JLON,JLEV)=MAX(0.0_JPRB,PQW(JLON,JLEV)-ZQTMP(JLON,JLEV))

    ENDDO
  ENDDO

ENDIF ! LSMGCDEV


IF (LSTRAPRO) THEN
!*
!     ------------------------------------------------------------------
!     IV  - CALCULS DE LA MICROPHYSIQUE (HORS 3MT).

!           MICROPHYSIC COMPUTATIONS (OUTSIDE 3MT).

  CALL APLMPHYS (KIDIA,KFDIA,KLON,KTDIA,KLEV,&
              & PAPRS,PAPRSF,PCP,ZQTMP,ZQITMP,ZQLTMP,PQN,PQR,PR,PT,&
              & ZIPOI,ZDQ,ZDQM,ZLHS,ZLHV,PNEBCOND,ZPOID,ZRCVOTT,&
              & ZTCORR,PLSM, PNEIJ, PTS,&
              & ZFCSQL,ZFCSQN,&
              & ZFALLR,ZFALLS,&
              & PFASL,PFASN,PFESL,PFESN,PFPLSL,PFPLSN,&
              & ZMELNET,&
              & 'APLMPHYS')

ENDIF ! LSTRAPRO
 
!*
!     ------------------------------------------------------------------
!     V   - FLUX DE CONDENSATION-EVAPORATION.

!           CONDENSATION-EVAPORATION FLUX.

DO JLEV=KTDIA-1,KLEV
  DO JLON=KIDIA,KFDIA
    PFCSQL(JLON,JLEV)=ZFCSQL(JLON,JLEV)
    PFCSQN(JLON,JLEV)=ZFCSQN(JLON,JLEV)
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('ACCDEV',1,ZHOOK_HANDLE)
END SUBROUTINE ACCDEV
