!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACCOLL ( KIDIA,KFDIA,KLON,&
 !-----------------------------------------------------------------------
 ! - INPUT 1D .
 & PEXPN,PHPLSL,PHPLSN,PLPLSL,PLPLSN,PLSCP,PPART,PRCOLL,PRHOAIR,&
 ! - INPUT/OUTPUT 1D .
 & PQIST,PQLST,PQNST,PQRST,PTST,&
 ! - OUTPUT 1D .
 & PCOLNI,PCOLNL,PCOLRI,PCOLRL,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACCOLL* - CALCULS DE COLLECTION.

!     Sujet.
!     ------
!     - ROUTINE DE CALCUL ACTIF (SUR UN NIVEAU).
!       CALCUL DE QUATRE INCREMENTS DE COLLECTION.
!     - COMPUTATION OF FOUR COLLECTION INCREMENTS.

!**   Interface.
!     ----------
!        *CALL* *ACCOLL*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLMPHYS" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 1D .

! PEXPN      : FONCTION DE DEPENDANCE EN TEMPERATURE DES PROCESSUS < 0C.
! PHPLSL     : FLUX DE PRECIPITATION LIQUIDE AU SOMMET DE LA COUCHE.
! PHPLSN     : FLUX DE PRECIPITATION NEIGEUSE AU SOMMET DE LA COUCHE.
! PLPLSL     : FLUX DE PRECIPITATION LIQUIDE AU BAS DE LA COUCHE.
! PLPLSL     : FLUX DE PRECIPITATION NEIGEUSE AU BAS DE LA COUCHE.
! PLSCP      : CHALEUR DE FUSION SUR CHALEUR MASSIQUE A PRESSION CONSTANTE.
! PPART      : PROPORTION DE LA MAILLE CONCERNEE.
! PRCOLL     : RAPPORT DES EFFICACITES DE COLLECTION NEIGE/PLUIE.
! PRHOAIR    : DENSITE DE L'AIR.

!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTRE-SORTIE.
!     --------------------------

! - 1D .

! PQIST      : QUANTITE COURANTE D'EAU SOLIDE NUAGEUSE.
! PQLST      : QUANTITE COURANTE D'EAU LIQUIDE NUAGEUSE.
! PQNST      : QUANTITE COURANTE D'EAU SOLIDE PRECIPITANTE.
! PQRST      : QUANTITE COURANTE D'EAU LIQUIDE PRECIPITANTE.
! PTST       : VALEUR COURANTE DE LA TEMPERATURE A L'ECHELLE DE LA MAILLE.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - 1D .

! PCOLNI     : INCREMENT DE COLLECTION GLACE => NEIGE.
! PCOLNL     : INCREMENT DE COLLECTION EAU LIQUIDE => NEIGE.
! PCOLRI     : INCREMENT DE COLLECTION GLACE => PLUIE.
! PCOLRL     : INCREMENT DE COLLECTION EAU LIQUIDE => PLUIE.

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST/
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     Methode.
!     --------

!     Auteur.
!     -------
!        07-02, J.-F. Geleyn (extraction from APLMPHYS).

!     Modifications.
!     --------------
!        07-03, Option of the ARPEGE scheme - J.-F. Geleyn & J.-M. Piriou

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : NPHYREP
USE YOMCST   , ONLY : RTT
USE YOMPHY0  , ONLY : EFFCOLL  ,RACCEF   ,RAGGEF   ,RRIMEF
USE YOMPHY2  , ONLY : TSPHY

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXPN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHPLSL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHPLSN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLPLSL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLPLSN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSCP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPART(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCOLL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHOAIR(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQIST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQNST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQRST(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTST(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLNI(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLNL(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLRI(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLRL(KLON) 
CHARACTER(LEN=6)  ,INTENT(IN)    :: CDLOCK 

INTEGER(KIND=JPIM) :: JLON

LOGICAL :: LLA0MPS, LLARPSC

REAL(KIND=JPRB) :: ZEXCOLL, ZCOLLN, ZCOLLI, ZFPLSL, ZFPLSN, ZPOWFL, ZPOWFN, &
 & ZEPS1, ZEXRHO, ZCEXPN, ZCOEFFA, ZCOEFFB, ZFACRHO, ZEXPN, ZCOLNL, ZCOLRL, ZFAC

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACCOLL',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'ACCOLL') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

LLA0MPS=.TRUE.
LLARPSC=.FALSE.

IF (LLA0MPS) THEN

  ZEXCOLL=0.8_JPRB

  DO JLON=KIDIA,KFDIA
    ZCOLLN=EFFCOLL*PRCOLL(JLON)
    ZCOLLI=PEXPN(JLON)
    ZFPLSL=MAX(0.0_JPRB,0.5_JPRB*(PHPLSL(JLON)+PLPLSL(JLON)))
    ZPOWFL=EFFCOLL*TSPHY*ZFPLSL**ZEXCOLL
    ZFPLSN=MAX(0.0_JPRB,0.5_JPRB*(PHPLSN(JLON)+PLPLSN(JLON)))
    ZPOWFN=ZCOLLN*TSPHY*ZFPLSN**ZEXCOLL
    PCOLRL(JLON)=PQLST(JLON)*ZPOWFL/(1.0_JPRB+ZPOWFL+ZPOWFN)
    PCOLNL(JLON)=PQLST(JLON)*ZPOWFN/(1.0_JPRB+ZPOWFL+ZPOWFN)
    ZPOWFL=ZPOWFL*ZCOLLI
    ZPOWFN=ZPOWFN*ZCOLLI
    PCOLRI(JLON)=PQIST(JLON)*ZPOWFL/(1.0_JPRB+ZPOWFL+ZPOWFN)
    PCOLNI(JLON)=PQIST(JLON)*ZPOWFN/(1.0_JPRB+ZPOWFL+ZPOWFN)
    PQLST(JLON)=PQLST(JLON)-PCOLRL(JLON)-PCOLNL(JLON)
    PQIST(JLON)=PQIST(JLON)-PCOLNI(JLON)-PCOLRI(JLON)
    PQRST(JLON)=PQRST(JLON)+PCOLRL(JLON)+PCOLRI(JLON)
    PQNST(JLON)=PQNST(JLON)+PCOLNL(JLON)+PCOLNI(JLON)
    PTST(JLON)=PTST(JLON)+(PCOLNL(JLON)-PCOLRI(JLON))*PLSCP(JLON)*PPART(JLON)
  ENDDO

ENDIF

IF (LLARPSC) THEN

  ZEPS1=1.E-20_JPRB

  ZEXRHO=0.6_JPRB
  ZCEXPN=0.025_JPRB
  ZCOEFFA=5.1748_JPRB
  ZCOEFFB=18.4607_JPRB

  DO JLON=KIDIA,KFDIA
    ZFACRHO=PRHOAIR(JLON)**ZEXRHO
    ZEXPN=EXP(ZCEXPN*(PTST(JLON)-RTT))
    PCOLNI(JLON)=PQIST(JLON)*(1.0_JPRB-EXP(-RAGGEF*ZEXPN*ZFACRHO*TSPHY*ZCOEFFB&
     &*PQNST(JLON)))
    ZCOLNL=PQLST(JLON)*(1.0_JPRB-EXP(-RRIMEF*ZFACRHO*TSPHY*ZCOEFFB*PQNST(JLON))) 
    ZCOLRL=PQLST(JLON)*(1.0_JPRB-EXP(-RACCEF*ZFACRHO*TSPHY*ZCOEFFA*PQRST(JLON)))
    ZFAC=MIN(1.0_JPRB, PQLST(JLON)/MAX(ZEPS1,ZCOLNL+ZCOLRL))
    PCOLNL(JLON)=ZFAC*ZCOLNL
    PCOLRL(JLON)=ZFAC*ZCOLRL
    PCOLRI(JLON)=0.0_JPRB
    PQLST(JLON)=PQLST(JLON)-PCOLRL(JLON)-PCOLNL(JLON)
    PQIST(JLON)=PQIST(JLON)-PCOLNI(JLON)
    PQRST(JLON)=PQRST(JLON)+PCOLRL(JLON)
    PQNST(JLON)=PQNST(JLON)+PCOLNL(JLON)+PCOLNI(JLON)
    PTST(JLON)=PTST(JLON)+PCOLNL(JLON)*PLSCP(JLON)*PPART(JLON)
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('ACCOLL',1,ZHOOK_HANDLE)
END SUBROUTINE ACCOLL


