!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACDIFSPADT (KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI5,PAPHIF5,PDELP5,PRDELP5,PT5,PQ5,&
 & PU5,PV5,PKTROV5,PKUROV5,PXTROV5,PXUROV5,&
 ! - INPUT  1D .
 & PCDROV5,PCHROV5,PXDROV5,PXHROV5,&
 & PLSM,PNEIJ,PQS,PTS,PTS1,PQS1,&
 ! - OUTPUT 2D .
 & PDIFTQ5,PDIFTS5,PSTRTU5,PSTRTV5,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACDIFSPADT * - COMPUTATION OF TRAJECTORIES OF VERTICAL TURBULENT
!                     DIFFUSION - SIMPLIFIED

!     Subject.
!     --------
!     Computation of trajectories for:
!     - COMPUTATION OF VERTICAL TURBULENT DIFFUSION INCLUDING A NEGATIVE
!       HUMIDITY CORRECTION : VERTICAL TURBULENT FLUXES, CALL TO THE
!       CHARNOCK FORMULA, THERMAL RADIATION FLUXES' CORRECTION AND ALL
!       CORRESPONDING FLUXES AT THE SURFACE OR IN THE SOIL .
!       SIMPLIFICATION OF ORIGINAL ACDIFUS IS BASED ON ASSUMPTION
!       OF CONSTANT FIELDS FOR Ts and Qs.

!**   Interface.
!     ----------
!        *CALL* *ACDIFSPADT*

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR INPUT
!     -------------------

! - DIMENSIONAL PARAMETERS OF PHYSICS.

! KIDIA      : START OF HORIZONTAL LOOP (IST in CPG)
! KFDIA      : END OF HORIZONTAL LOOP.
! KLON       : HORIZONTAL DIMENSION.
! KTDIA      : START OF THE VERTICAL LOOP IN THE PHYSICS (1 IN GENERAL)
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION "FULL LEVEL"
!              (NFLEVG in CPG)

! - THE NAME OF THE PHYSICAL VARIABLES - trajectories

! - 2D (0:KLEV) .

! PAPHI5     : GEOPOTENTIAL ON HALF LEVELS.
! PKTROV5    : VERTICAL EXCHANGE COEFFICIENT OF T AND Q IN KG/(M*M*S).
! PKUROV5    : VERTICAL EXCHANGE COEFFICIENT OF U AND V IN KG/(M*M*S).
! PXTROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PKTROV.
! PXUROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PKUROV.

! - 2D (1:KLEV) .

! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.
! PDELP5     : LAYER THICKNESS IN PRESSURE UNITS.
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOR.
! PRDELP5    : INVERSE VALUE OF LAYER THICKNESS (OF PDELP).
! PT5        : TEMPERATURE.
! PU5        : X - COMPONENT OF WIND.
! PV5        : Y - COMPONENT OF WIND.

! ------------------------

! - 1D (GEOGRAPHIQUE) .

! PLSM       : LAND/SEA MASK.

! - 1D (DIAGNOSTIQUE) .

! PNEIJ      : FRACTION OF SOIL COVERED BY SNOW.
! PQS        : SURFACE SPECIFIC HUMIDITY.
! PTS        : SURFACE TEMPERATURE.
! PQS1       : SURFACE SPECIFIC HUMIDITY AT t+dt.
! PTS1       : SURFACE TEMPERATURE AT t+dt.

! The trajectory
! --------------
! PCDROV5    : PCD RENORME EN DENSITE FOIS VITESSE.
! PCHROV5    : PCH RENORME EN DENSITE FOIS VITESSE.
! PXDROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PCDROV.
! PXHROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PCHROV.

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR OUTPUT.
!     ---------------------

! - 2D (0:KLEV) .

! PDIFTQ5    : TURBULENT FLUX (INC. Q NEGATIVE) OF SPECIFIC HUMIDITY.
! PDIFTS5    : TURBULENT FLUX OF ENTHALPY (OR DRY STATIC ENERGY).
! PSTRTU5    : TURBULENT FLUX OF MOMENTUM "U".
! PSTRTV5    : TURBULENT FLUX OF MOMENTUM "V".

!-----------------------------------------------------------------------

! -   IMPLICIT ARGUMENTS
!     -------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY1/
! COMMON/YOMPHY2/
! COMMON/FCTTRM/

!-----------------------------------------------------------------------

!     Externals.
!     ----------

!     Methode.
!     --------

!     Authors.
!     --------
!          96-01-23, M.Janiskova

!     Modifications.
!     --------------
!        97-08, introduce CDLOCK - J.M. Piriou.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND2  ,ONLY : JPRH

USE YOMPHY   , ONLY : LNEIGE
USE YOMCST   , ONLY : RPI      ,RDAY     ,RG       ,RCPD     ,&
 & RCPV     ,RCW      ,RCS  
USE YOMPHY1  , ONLY : TMERGL
USE YOMPHY2  , ONLY : TSPHY

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKTROV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKUROV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTROV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXUROV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCDROV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCHROV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXDROV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXHROV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJ(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS1(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS1(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQ5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRTU5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRTV5(KLON,0:KLEV) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
REAL(KIND=JPRH) :: ZLSM(KLON),ZNEIJ(KLON),ZQS(KLON),ZTS(KLON),ZQS1(KLON),ZTS1(KLON)
REAL(KIND=JPRH) :: ZCPS1(KLON),ZPN(KLON)

REAL(KIND=JPRH) :: ZKTROV5(KLON,0:KLEV),ZKUROV5(KLON,0:KLEV)&
 & ,ZXUROV5(KLON,0:KLEV),ZXTROV5(KLON,0:KLEV)&
 & ,ZSTRTU5(KLON,0:KLEV),ZSTRTV5(KLON,0:KLEV)&
 & ,ZDIFTQ5(KLON,0:KLEV),ZDIFTS5(KLON,0:KLEV)&
 & ,ZAPHI5(KLON,0:KLEV)&
 & ,ZAPHIF5(KLON,KLEV),ZDELP5(KLON,KLEV),ZRDELP5(KLON,KLEV)&
 & ,ZQ5(KLON,KLEV),ZT5(KLON,KLEV),ZU5(KLON,KLEV)&
 & ,ZV5(KLON,KLEV),ZCDROV5(KLON),ZCHROV5(KLON)&
 & ,ZXDROV5(KLON),ZXHROV5(KLON)  

REAL(KIND=JPRH) :: ZKSRV5(KLON,0:KLEV),ZXTRO5(KLON,0:KLEV)&
 & ,ZXURO5(KLON,0:KLEV),ZFCQNG5(KLON,0:KLEV)&
 & ,ZDSE5(KLON,KLEV),ZIPOI5(KLON,KLEV),ZPOID5(KLON,KLEV)&
 & ,ZN15(KLON,KLEV),ZN25(KLON,KLEV),ZN35(KLON,KLEV)&
 & ,ZN45(KLON,KLEV),ZQCOR5(KLON,KLEV),ZSUB15(KLON,KLEV)&
 & ,ZSUB25(KLON,KLEV),ZCP5(KLON,KLEV)&
 & ,ZCHRS5(KLON),ZDLLS5(KLON),ZDLLW5(KLON)&
 & ,ZDLS5(KLON),ZXDRO5(KLON),ZXSRO5(KLON),ZDSES15(KLON)  

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZCPVMD, ZCPVMS, ZCPVMW, ZDLDQ5, ZELIM15,&
 & ZELIM25, ZEPS1, ZFENT5, ZFQ5, ZGDT, ZGDTI, &
 & ZMUL15, ZMUL25, ZPUL, ZQCORP5, ZZ0, ZZ1  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACDIFSPADT',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'DFSPADT') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!     I - AUXILIARY CONSTANTS.

ZCPVMD=RCPV-RCPD
ZCPVMW=RCPV-RCW
ZCPVMS=RCPV-RCS

!*
!     ------------------------------------------------------------------
!     II - COMPUTATION OF DERIVED PARAMETERS AND SECURITY CONSTANT (FOR
!     CSDT).

ZGDT=RG*TSPHY
ZGDTI=1.0_JPRB/ZGDT
ZPUL=2.0_JPRB*RPI/RDAY

ZEPS1=1.E-08_JPRB

ZZ1=1.0_JPRB
ZZ0=0.0_JPRB

!     CONVERSION FROM SINGLE TO DOUBLE PRECISION

DO JLON=KIDIA,KFDIA

  DO JLEV=KTDIA,KLEV
    ZAPHIF5(JLON,JLEV)=PAPHIF5(JLON,JLEV)
    ZDELP5(JLON,JLEV)=PDELP5(JLON,JLEV)
    ZRDELP5(JLON,JLEV)=PRDELP5(JLON,JLEV)
    ZT5(JLON,JLEV)=PT5(JLON,JLEV)
    ZQ5(JLON,JLEV)=PQ5(JLON,JLEV)
    ZU5(JLON,JLEV)=PU5(JLON,JLEV)
    ZV5(JLON,JLEV)=PV5(JLON,JLEV)
  ENDDO

  DO JLEV=0,KLEV
    ZAPHI5(JLON,JLEV)=PAPHI5(JLON,JLEV)
    ZKTROV5(JLON,JLEV)=PKTROV5(JLON,JLEV)
    ZKUROV5(JLON,JLEV)=PKUROV5(JLON,JLEV)
    ZXTROV5(JLON,JLEV)=PXTROV5(JLON,JLEV)
    ZXUROV5(JLON,JLEV)=PXUROV5(JLON,JLEV)
  ENDDO

  ZCDROV5(JLON)=PCDROV5(JLON)
  ZCHROV5(JLON)=PCHROV5(JLON)
  ZXDROV5(JLON)=PXDROV5(JLON)
  ZXHROV5(JLON)=PXHROV5(JLON)

  ZLSM(JLON)=PLSM(JLON)
  ZNEIJ(JLON)=PNEIJ(JLON)
  ZQS(JLON)=PQS(JLON)
  ZTS(JLON)=PTS(JLON)
  ZQS1(JLON)=PQS1(JLON)
  ZTS1(JLON)=PTS1(JLON)
ENDDO

!*
!     ------------------------------------------------------------------
!     III - COMPUTATION OF SPECIFIC HEAT AT CONSTANT PRESSURE FOR AIR
!     Cp AND AT THE SURFACE Cps

! - TEMPORARY 2D.

! For the trajectory
! -------------------
! ZCP5     : VALUE OF ATMOSPHERIC Cp

! - TEMPORARY 1D.

! ZCPS1    : VALUE OF Cp AT THE SURFACE AT t+dt

DO JLON=KIDIA,KFDIA
  ZCPS1(JLON)=RCPD+ZCPVMD*ZQS1(JLON)

  DO JLEV=KTDIA,KLEV
    ZCP5(JLON,JLEV)=RCPD*(1.0_JPRB-ZQ5(JLON,JLEV))+RCPV*ZQ5(JLON,JLEV)
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     IV - PRELIMINARY COMPUTATIONS OF DRY STATIC ENERGY, SPECIFIC
!     HUMIDITY CORRECTED OF ITS NEGATIVE VALUES THROUGH UPWARDS PUMPING
!     (EXCEPT IN THE CASE OF THE LOWEST LEVEL ASSUMED TO POSSES A
!     FICTITIOUS SOURCE AT THE SURFACE) AND OF LAYERS' CHARACTERISTICS
!     IN THE UNITS OF THE NORMALIZED EXCHANGE COEFFICIENTS (KG/M**2/S)
!     FOR "POID" AND ITS INVERSE FOR "IPOI".

! - TEMPORARY(S) 2D (0:KLEV) .

! For the trajectory
! ------------------
! ZXURO5    : ZKUROV MODIFIED TO ATTENUATE FIBRILLATIONS.
! ZFCQNG5   : PSEUDO-FLUX OF WATER TO CORRECT FOR Q<0.

! - TEMPORARY(S) 2D (1:KLEV) .

! For the trajectory
! -------------------
! ZPOID5    : DP/(RG*DT) FOR A GIVEN LAYER AND A GIVEN TIME STEP.
! ZIPOI5    : INVERSE OF ZPOID.
! ZDSE5     : LOCAL ARRAY FOR DRY STATIC ENERGIES.
! ZQCOR5    : LOCAL ARRAY FOR Q AFTER CORRECTION OF NEGATIVE VALUES.

! - TEMPORARY 1D.

! For the trajectory
! -------------------
! ZXDRO5    : ZCDROV MODIFIED TO ATTENUATE FIBRILLATIONS.
! ZDSES15   : LOCAL ARRAY FOR SURFACE DRY STATIC ENERGIE AT t+dt.

DO JLON=KIDIA,KFDIA
  ZFCQNG5(JLON,KTDIA-1)=0.0_JPRB

  ZDSES15(JLON)=ZCPS1(JLON)*ZTS1(JLON)+ZAPHI5(JLON,KLEV)
ENDDO

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZPOID5(JLON,JLEV)=ZDELP5(JLON,JLEV)*ZGDTI
    ZIPOI5(JLON,JLEV)=ZRDELP5(JLON,JLEV)*ZGDT
    ZDSE5(JLON,JLEV)=ZCP5(JLON,JLEV)*ZT5(JLON,JLEV)+ZAPHIF5(JLON,JLEV)

    ZQCORP5=ZQ5(JLON,JLEV)+ZFCQNG5(JLON,JLEV-1)*ZIPOI5(JLON,JLEV)
    ZFCQNG5(JLON,JLEV)=-MAX(ZZ0,-ZQCORP5)*ZPOID5(JLON,JLEV)
    ZQCOR5(JLON,JLEV)=ZQCORP5+MAX(ZZ0,-ZQCORP5)
  ENDDO
ENDDO

!     POTENTIAL "ANTI-FIBRILLATION" AMPLIFICATION OF THE EXCHANGE
!     COEFFICIENTS.

DO JLON=KIDIA,KFDIA
  ZXDRO5(JLON)=ZCDROV5(JLON)*ZXDROV5(JLON)
ENDDO

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZXURO5(JLON,JLEV)=ZKUROV5(JLON,JLEV)*ZXUROV5(JLON,JLEV)

  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     V - COMPUTATIONS FOR MOMENTUM / ELIMINATION PART. IN ALL THE
!     FOLLOWING ONE WILL USE ZELIM1 AND ZELIM2 (1 FOR THE COUPLE U,V
!     AND 2 FOR THE COUPLE S,Q) TO NAME THE SUBDIAGONAL ELEMENT BY
!     WHICH ONE SHOULD MULTIPLY THE PRECEEDING LINE TO ELIMINATE IT BY
!     SUBSTRACTION AND ZMUL1 AND ZMUL2 (1 FOR THE COUPLE U,V AND
!     2 FOR THE COUPLE S,Q) TO NAME THE MULTIPLYING FACTOR TO PUT THE
!     MAIN DIAGONAL TO "1". ZSUB1 AND ZSUB2 WILL CONTAIN THE MODIFIED
!     SUPERDIAGONAL COEFFICIENTS THAT WILL BE USED IN THE SUBSTITUTION
!     (1 FOR THE COUPLE U,V AND 2 FOR Q,S) AND ZN1, ZN2, ZN3 AND ZN4
!     WILL CONTAIN THE RIGHT HAND SIDES DUE TO BECOME THE FINAL VALUES
!     (1 FOR U, 2 FOR V, 3 FOR S AND 4 FOR Q).

! - TEMPORARY(S) 2D (1:KLEV) .

! For the trajectory
! -------------------
! ZSUB15    : ARRAY OF COEFFICIENTS FOR THE U,V SUBSTITUTION.
! ZSUB25    : ARRAY OF COEFFICIENTS FOR THE S,Q SUBSTITUTION.
! ZN15      : ARRAY OF RIGH HAND SIDES FOR U.
! ZN25      : ARRAY OF RIGHT HAND SIDES FOR V.
! ZN35      : ARRAY OF RIGH HAND SIDES FOR S (S=CP*T+PHI).
! ZN45      : ARRAY OF RIGHT HAND SIDES FOR Q.

!     ELIMINATION AT THE TOP.

DO JLON=KIDIA,KFDIA
  ZMUL15=1.0_JPRB/(1.0_JPRB+ZXURO5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA))
  ZSUB15(JLON,KTDIA)=ZMUL15*ZXURO5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA)

  ZN15(JLON,KTDIA)=ZMUL15*ZU5(JLON,KTDIA)
  ZN25(JLON,KTDIA)=ZMUL15*ZV5(JLON,KTDIA)
ENDDO

!     ELIMINATION FOR A STANDART LEVEL.

DO JLEV=KTDIA+1,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZELIM15=ZXURO5(JLON,JLEV-1)*ZIPOI5(JLON,JLEV)
    ZMUL15=1.0_JPRB/(1.0_JPRB+ZELIM15*(1.0_JPRB-ZSUB15(JLON,JLEV-1))+ZXURO5(JLON,&
     & JLEV)&
     & *ZIPOI5(JLON,JLEV))  

    ZSUB15(JLON,JLEV)=ZMUL15*ZXURO5(JLON,JLEV)*ZIPOI5(JLON,JLEV)
    ZN15(JLON,JLEV)=ZMUL15*(ZU5(JLON,JLEV)+ZELIM15*ZN15(JLON,JLEV-1))
    ZN25(JLON,JLEV)=ZMUL15*(ZV5(JLON,JLEV)+ZELIM15*ZN25(JLON,JLEV-1))
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     VI - COMPUTATION FOR MOMENTUM: SURFACE CONDITION, "CHARNOCK-TYPE"
!     CALCULATION THEN SUBSTITUTION WITH FLUXES' COMPUTATION AT ALL
!     LEVELS.

!     SURFACE CALCULATIONS.

DO JLON=KIDIA,KFDIA
  ZELIM15=ZXURO5(JLON,KLEV-1)*ZIPOI5(JLON,KLEV)
  ZMUL15=1.0_JPRB/(1.0_JPRB+ZELIM15*(1.0_JPRB-ZSUB15(JLON,KLEV-1))+ZXDRO5(JLON)&
   & *ZIPOI5(JLON,KLEV))  

  ZN15(JLON,KLEV)=ZMUL15*(ZU5(JLON,KLEV)+ZELIM15*ZN15(JLON,KLEV-1))
  ZN25(JLON,KLEV)=ZMUL15*(ZV5(JLON,KLEV)+ZELIM15*ZN25(JLON,KLEV-1))

  ZSTRTU5(JLON,KLEV)=ZCDROV5(JLON)*ZN15(JLON,KLEV)
  ZSTRTV5(JLON,KLEV)=ZCDROV5(JLON)*ZN25(JLON,KLEV)
ENDDO

!     BACK-SUBSTITUTION FOR A STANDART LAYER AND AT THE TOP.

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZN15(JLON,JLEV)=ZN15(JLON,JLEV)+ZSUB15(JLON,JLEV)*ZN15(JLON,JLEV+1)
    ZN25(JLON,JLEV)=ZN25(JLON,JLEV)+ZSUB15(JLON,JLEV)*ZN25(JLON,JLEV+1)

    ZSTRTU5(JLON,JLEV)=ZKUROV5(JLON,JLEV)*(ZN15(JLON,JLEV)-ZN15(JLON,JLEV+1))
    ZSTRTV5(JLON,JLEV)=ZKUROV5(JLON,JLEV)*(ZN25(JLON,JLEV)-ZN25(JLON,JLEV+1))
  ENDDO
ENDDO

!     UPPER BOUNDARY CONDITION.

DO JLON=KIDIA,KFDIA
  ZSTRTU5(JLON,KTDIA-1)=0.0_JPRB
  ZSTRTV5(JLON,KTDIA-1)=0.0_JPRB
ENDDO

!*
!     ------------------------------------------------------------------
!     VII - PRELIMINARY COMPUTATIONS AT THE SURFACE FOR THE COUPLING OF
!     THE IMPLICIT SOLUTIONS IN S AND Q THROUGH THE TS VALUE (ONE WILL
!     HAVE TWO VALUES, A PRIORI IDENTICAL, OF THE NORMALIZED CH
!     COEFFICIENT TO ALLOW SENSITIVITY STUDIES FOR SENSIBLE/LATENT
!     HEATS).

DO JLON=KIDIA,KFDIA

!     COMPUTATIONS CORRESPONDING TO EVAPORATION/SUBLIMATION.

! - TEMPORARY(S) 1D .

! ZPN       : PROPORTION OF THE SURFACE EVAPORATING IN ICE PHASE.

  IF (LNEIGE) THEN
    ZPN(JLON)=ZLSM(JLON)*ZNEIJ(JLON)+(1.0_JPRB-ZLSM(JLON))&
     & *MAX(ZZ0,SIGN(ZZ1,REAL(TMERGL-ZTS(JLON),KIND(ZZ1))))  
  ELSE
    ZPN(JLON)=0.0_JPRB
  ENDIF

!     DIFFERENTIATION BETWEEN THE SURFACE EXCHANGE COEFFICIENTS FOR
!     LATENT/SENSIBLE HEAT.

!     POTENTIAL "ANTI-FIBRILLATION" AMPLIFICATION OF THE EXCHANGE
!     COEFFICIENTS.

! - TEMPORARY(S) 1D .

! For trajectory
! --------------
! ZCHRS5    : EXCHANGE COEFFICIENT ZCHROV FOR SENSIBLE HEAT.
! ZXSRO5    : ZCHRS MODIFIED TO ATTENUATE FIBRILLATIONS.

  ZCHRS5(JLON)=ZCHROV5(JLON)
  ZXSRO5(JLON)=ZCHRS5(JLON)*ZXHROV5(JLON)

!     CONTRIBUTION TO THE COEFFICIENTS OF THE TRIDIAGONAL MATRIX.

! - TEMPORARY(S) 1D .

! For trajectory
! ---------------
! ZDLS5     : TEMPERATURE DERIVATIVE OF THE TOTAL FLUX ; SENSIBLE PART.
! ZDLLW5    : AS ZDLS ; MINUS LATENT PART, LIQUID PHASE.
! ZDLLS5    : AS ZDLS ; MINUS LATENT PART, ICE PHASE.

  ZDLS5(JLON)=(ZQS(JLON)-ZQ5(JLON,KLEV))*ZCHRS5(JLON)*ZCPVMD
  ZDLLW5(JLON)=(ZQS(JLON)-ZQ5(JLON,KLEV))*ZCHRS5(JLON)*ZCPVMW*(1.0_JPRB &
   & -ZPN(JLON))  
  ZDLLS5(JLON)=(ZQS(JLON)-ZQ5(JLON,KLEV))*ZCHRS5(JLON)*ZCPVMS*ZPN(JLON)
  ZDLDQ5=ZDLLW5(JLON)+ZDLLS5(JLON)-ZDLS5(JLON)*ZTS(JLON)
ENDDO

!*
!     ------------------------------------------------------------------
!     VIII - COMPUTATION FOR TEMPERATURE AND SPECIFIC HUMIDITY :
!     ELIMINATION PART. LIKE AT THE SURFACE ONE DIFFERENTIATES THE
!     (A PRIORI EQUAL) NORMALIZED COEFFICIENTS FOR S AND Q.

! - TEMPORARY(S) 2D (0:KLEV) .

! For trajectory
! ---------------
! ZKSRV5    : VALUE (DUPLICATED FROM ZKTROV) FOR THE S DIFFUSION.
! ZXTRO5    : ZKSRV MODIFIED TO ATTENUATE FIBRILLATIONS.

!     THE VERTICAL EXCHANGE COEFFICIENT FOR S AND Q.

!     POTENTIAL "ANTI-FIBRILLATION" AMPLIFICATION OF THE EXCHANGE
!     COEFFICIENT.

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZKSRV5(JLON,JLEV)=ZKTROV5(JLON,JLEV)
    ZXTRO5(JLON,JLEV)=ZKSRV5(JLON,JLEV)*ZXTROV5(JLON,JLEV)
  ENDDO
ENDDO

!     TOP ELIMINATION FOR THE DRY STATIC ENERGY, S AND FOR THE SPECIFIC
!     HUMIDITY CORRECTED FROM NEGATIVE VALUES, Q.

DO JLON=KIDIA,KFDIA
  ZMUL25=1.0_JPRB/(1.0_JPRB+ZXTRO5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA))
  ZSUB25(JLON,KTDIA)=ZMUL25*ZXTRO5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA)

  ZN35(JLON,KTDIA)=ZMUL25*ZDSE5(JLON,KTDIA)
  ZN45(JLON,KTDIA)=ZMUL25*ZQCOR5(JLON,KTDIA)
ENDDO

!     ELIMINATION FOR A STANDART LAYER FOR S AND Q.

DO JLEV=KTDIA+1,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZELIM25=ZXTRO5(JLON,JLEV-1)*ZIPOI5(JLON,JLEV)
    ZMUL25=1.0_JPRB/(1.0_JPRB+ZELIM25*(1.0_JPRB-ZSUB25(JLON,JLEV-1))+ZXTRO5(JLON,&
     & JLEV)&
     & *ZIPOI5(JLON,JLEV))  
    ZSUB25(JLON,JLEV)=ZMUL25*ZXTRO5(JLON,JLEV)*ZIPOI5(JLON,JLEV)

    ZN35(JLON,JLEV)=ZMUL25*(ZDSE5(JLON,JLEV)+ZELIM25*ZN35(JLON,JLEV-1))
    ZN45(JLON,JLEV)=ZMUL25*(ZQCOR5(JLON,JLEV)+ZELIM25*ZN45(JLON,JLEV-1))
  ENDDO
ENDDO

!     SURFACE ELIMINATION CALCULATION FOR S AND Q.

DO JLON=KIDIA,KFDIA
  ZELIM25=ZXTRO5(JLON,KLEV-1)*ZIPOI5(JLON,KLEV)
  ZMUL25=1.0_JPRB/(1.0_JPRB+ZELIM25*(1.0_JPRB-ZSUB25(JLON,KLEV-1))&
   & +ZXSRO5(JLON)*ZIPOI5(JLON,KLEV))  

  ZN35(JLON,KLEV)=ZMUL25*(ZDSE5(JLON,KLEV)&
   & +ZXSRO5(JLON)*ZIPOI5(JLON,KLEV)*ZDSES15(JLON)&
   & +ZELIM25*ZN35(JLON,KLEV-1))  
  ZN45(JLON,KLEV)=ZMUL25*(ZQCOR5(JLON,KLEV)&
   & +ZXSRO5(JLON)*ZIPOI5(JLON,KLEV)*ZQS1(JLON)&
   & +ZELIM25*ZN45(JLON,KLEV-1))  
ENDDO

!*
!     ------------------------------------------------------------------
!     IX - COMPUTATIONS FOR TEMPERATURE AND SPECIFIC HUMIDITY :
!     BACK-SUBSTITUTION PART AND CALCULATION OF FLUXES AT ALL LEVELS AS
!     WELL AS AT THE SURFACE AND IN THE SOIL.

!     SURFACE AND SOIL FLUXES.

DO JLON=KIDIA,KFDIA
  ZFQ5=ZCHRS5(JLON)*(ZN45(JLON,KLEV)-ZQS1(JLON))
  ZFENT5=ZCHRS5(JLON)*(ZN35(JLON,KLEV)-ZDSES15(JLON))
  ZDIFTQ5(JLON,KLEV)=ZFQ5+ZFCQNG5(JLON,KLEV)
  ZDIFTS5(JLON,KLEV)=ZFENT5

ENDDO

!     BACK-SUBSTITUTION FOR A STANDART LEVEL.
!     ATMOSPHERIC FLUXES.

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZN35(JLON,JLEV)=ZN35(JLON,JLEV)+ZSUB25(JLON,JLEV)*ZN35(JLON,JLEV+1)
    ZN45(JLON,JLEV)=ZN45(JLON,JLEV)+ZSUB25(JLON,JLEV)*ZN45(JLON,JLEV+1)

    ZDIFTQ5(JLON,JLEV)=ZKSRV5(JLON,JLEV)*(ZN45(JLON,JLEV)&
     & -ZN45(JLON,JLEV+1))+ZFCQNG5(JLON,JLEV)  

    ZDIFTS5(JLON,JLEV)=ZKSRV5(JLON,JLEV)*(ZN35(JLON,JLEV)-ZN35(JLON,JLEV+1))
  ENDDO
ENDDO

!     UPPER BOUNDARY CONDITION.

DO JLON=KIDIA,KFDIA
  ZDIFTQ5(JLON,KTDIA-1)=0.0_JPRB
  ZDIFTS5(JLON,KTDIA-1)=0.0_JPRB
ENDDO

!     CONVERSION FROM DOUBLE TO SINGLE PRECISION

DO JLON=KIDIA,KFDIA
  DO JLEV=0,KLEV
    PDIFTQ5(JLON,JLEV)=ZDIFTQ5(JLON,JLEV)
    PDIFTS5(JLON,JLEV)=ZDIFTS5(JLON,JLEV)
    PSTRTU5(JLON,JLEV)=ZSTRTU5(JLON,JLEV)
    PSTRTV5(JLON,JLEV)=ZSTRTV5(JLON,JLEV)
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('ACDIFSPADT',1,ZHOOK_HANDLE)
END SUBROUTINE ACDIFSPADT
