!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACDRAGLAD (KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI5,PAPHIF5,PAPRS5,PAPRSF5,&
 & PDELP5,PRDELP5,&
 & PU5,PV5,PT5,PQ5,&
 & PAPHI,PAPHIF,PAPRS,PAPRSF,PDELP,PRDELP,&
 & PU,PV,PT,PQ,&
 ! - INPUT  1D .
 & PGETRL,PVRLAN,PVRLDI,PTS,PQS,&
 ! - OUTPUT 2D .
 & PSTRDU5,PSTRDV5,&
 & PSTRDU,PSTRDV,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACDRAGLAD * - ADJOINT COMPUTATION OF SIMPLIFIED EFFECT OF OROGRAPHIC
!                    GRAVITY-WAVES.

!     Subject.
!     --------
!     Adjoint computation of:
!     - COMPUTATION OF OROGRAPHIC GRAVITY-WAVES' BREAKING EFFECTS ON
!       MOMENTUM - LINEAR PART OF COMPUTATION

!**   Interface.
!     ----------
!        *CALL* *ACDRAGLAD*

! -   ARGUMENTS FOR INPUT
!     -------------------

! - DIMENSIONAL PARAMETERS OF PHYSICS

! KIDIA      : START OF HORIZONTAL LOOP (IST in CPG).
! KFDIA      : END OF HORIZONTAL LOOP (IEND in CPG).
! KLON       : HORIZONTAL DIMENSIONAL.
! KTDIA      : START OF THE VERTICAL LOOP IN THE PHYSIC (1 IN GENERAL).
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION "FULL LEVEL".
!              (NFLEVG in CPG).

! - THE NAME OF THE PHYSICAL VARIABLES

! - 2D (0:KLEV) .

! PAPRS      : PRESSURE ON HALF LEVELS.
! PAPHI      : GEOPOTENTIAL ON HALF LEVELS.

! The same but for trajectory
! ---------------------------
! PAPRS5     : PRESSURE ON HALF LEVELS.
! PAPHI5     : GEOPOTENTIAL ON HALF LEVELS.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF     : PRESSURE ON FULL LEVELS.
! PDELP      : LAYER THICKNESS IN PRESSURE UNITS.
! PRDELP     : INVERSE OF LAYER THICKNESS IN PRESSURE UNITS.
! PU         : X COMPONENT OF WIND.
! PV         : Y COMPONENT OF WIND.
! PT         : TEMPERATURE.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOUR.

! The same but for trajectory
! ---------------------------
! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF5    : PRESSURE ON FULL LEVELS.
! PDELP5     : LAYER THICKNESS IN PRESSURE UNITS.
! PRDELP5    : INVERSE OF LAYER THICKNESS IN PRESSURE UNITS.
! PU5        : X COMPONENT OF WIND.
! PV5        : Y COMPONENT OF WIND.
! PT5        : TEMPERATURE.
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOUR.

! - 1D (GEOGRAPHIC) .

! PGETRL     : STANDARD DEVIATION OF OROGRAPHY (unit J/KG).
! PVRLAN     : ANISOTROPY OF THE RELIEF (1 = ISOTROPY).
! PVRLDI     : ANGLE OF MAXIMAL GRADIENT OF RELIEF WITH X-AXIS.

! - 1D (PROGNOSTIC) .

! PTS        : SURFACE TEMPERATURE.
! PQS        : SURFACE SPECIFIC HUMIDITY.

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR OUTPUT.
!     ----------------------

! - 2D (0:KLEV) .

! PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".

! The same but for trajectory
! ---------------------------
! PSTRDU5    : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDV5    : FLUX "GRAVITY WAVE DRAG" "V".

!-----------------------------------------------------------------------

! -   IMPLICIT ARGUMENTS.
!     ---------------------

! COMMON/YOMCST /
! COMMON/YOMPHY/
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/
! COMMON/FCTTRM/
! COMMON/FCTTRMAD/

!-----------------------------------------------------------------------

!     Externals.
!     ----------

!     Method.
!     -------

!     Author.
!     -------
!          95-10, M.Janiskova

!     Modifications.
!     --------------
!        96-05, GWDBC,GWDCD according to ACDRAG modif. - M.Janiskova
!        97-08, introduce CDLOCK - J.M. Piriou.
!        99-01, code optimization - M.Janiskova
!        99-10, suppression of USURIC - F. Bouyssel.
!        99-12, form drag set to zero (if LSMOOTHB=.T.) :  - F. Bouyssel.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI      ,RG       ,RD       ,RV       ,&
 & RCPD     ,RCPV     ,RKAPPA   ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMPHY   , ONLY : LCVPP    ,LNEIGE
USE YOMPHY0  , ONLY : GWDBC    ,GWDCD    ,GWDSE    ,HOBST
USE YOMPHY2  , ONLY : TSPHY
USE YOMSIMPHL, ONLY : LSMOOTHB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGETRL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRLAN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRLDI(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDU5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRDU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRDV(KLON,0:KLEV) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
REAL(KIND=JPRB) :: ZNBVNO(KLON,0:KLEV),ZRAPP(KLON,0:KLEV),ZNFNO(KLON,0:KLEV)&
 & ,ZWGHT(KLON,0:KLEV),ZRAPPH(KLON,0:KLEV)&
 & ,ZLSCPE(KLON,KLEV),ZQSAT(KLON,KLEV),ZR(KLON,KLEV)&
 & ,ZLH(KLON,KLEV),ZIPOI(KLON,KLEV),ZPOID(KLON,KLEV)&
 & ,ZU(KLON,KLEV),ZV(KLON,KLEV)  
REAL(KIND=JPRB) :: ZGWDCS(KLON),ZCP(KLON),ZFACS(KLON)&
 & ,ZOBST(KLON),ZRS(KLON),ZSTUS(KLON),ZSTVS(KLON)&
 & ,ZSUMD(KLON),ZSUMF(KLON),ZSUMU(KLON),ZSUMV(KLON)&
 & ,ZSUSR(KLON),ZUSR(KLON),ZUSUR(KLON),ZVSUR(KLON)&
 & ,ZAA(KLON),ZBB(KLON)  

REAL(KIND=JPRB) ::Z_PSTRDU15(KLON,0:KLEV),Z_PSTRDU25(KLON,0:KLEV)&
 & ,Z_PSTRDV15(KLON,0:KLEV),Z_PSTRDV25(KLON,0:KLEV)  

REAL(KIND=JPRB) :: ZNBVNO5(KLON,0:KLEV),ZRAPP5(KLON,0:KLEV)&
 & ,ZRAPP15(KLON,0:KLEV),ZRAPP25(KLON,0:KLEV)&
 & ,ZRAPPH5(KLON,0:KLEV),ZRAPP115(KLON,0:KLEV)&
 & ,ZWGHT5(KLON,0:KLEV),ZNFNO5(KLON,0:KLEV)&
 & ,ZWGHT15(KLON,0:KLEV)&
 & ,ZLSCPE5(KLON,KLEV),ZQSAT5(KLON,KLEV),ZR5(KLON,KLEV)&
 & ,ZLH5(KLON,KLEV),ZIPOI5(KLON,KLEV),ZPOID5(KLON,KLEV)&
 & ,ZU5(KLON,KLEV),ZV5(KLON,KLEV),ZEW5(KLON,KLEV)&
 & ,ZCP5(KLON,KLEV),ZESP5(KLON,KLEV),ZDPHI5(KLON,KLEV)&
 & ,ZDTETA5(KLON,KLEV),ZAUX5(KLON,KLEV)&
 & ,ZZB5(KLON,KLEV),ZZC5(KLON,KLEV),ZZD5(KLON,KLEV)&
 & ,ZRTI5(KLON,KLEV),ZSTA5(KLON,KLEV)&
 & ,ZWA5(KLON,KLEV),ZCIS5(KLON,KLEV)&
 & ,ZSUMU5(KLON,KLEV),ZSUMV5(KLON,KLEV),ZSUMF5(KLON,KLEV)&
 & ,ZPBLF5(KLON,KLEV),ZPBL5(KLON,KLEV),ZAUSR5(KLON,KLEV)&
 & ,ZPRRAP5(KLON,KLEV),ZZP5(KLON,KLEV),ZZPA5(KLON,KLEV)&
 & ,ZSUMD5(KLON,KLEV),ZDU5(KLON,KLEV),ZDV5(KLON,KLEV)&
 & ,ZDX5(KLON,KLEV),ZX5(KLON,KLEV),ZALPHA5(KLON,KLEV)&
 & ,ZRZ5(KLON,KLEV),ZRZ15(KLON,KLEV)  

REAL(KIND=JPRB) :: ZGWDCS5(KLON),ZFACS5(KLON),ZOBST5(KLON),ZVA5(KLON)&
 & ,ZSTUS5(KLON),ZSTVS5(KLON),ZSUSR5(KLON),ZUSR5(KLON)&
 & ,ZUSUR5(KLON),ZVSUR5(KLON),ZSUMD15(KLON),ZSUMD25(KLON)&
 & ,ZUSTAR5(KLON),ZVSTAR5(KLON),ZVB5(KLON),ZVC5(KLON)&
 & ,ZAA5(KLON),ZBB5(KLON),ZTUNE5(KLON),ZBB15(KLON)  

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: Z_XXXX, Z_YYYY, ZA, ZALP1, ZALP2, ZALPHA, ZAUSR,&
 & ZAUX, ZBB1, ZCIS, ZD, ZDEL1, ZDEL2, ZDELTA, &
 & ZDPHI, ZDTETA, ZDU, ZDV, ZDX, ZEPS1, ZEPS2, &
 & ZEPS3, ZEPS5, ZEPS6, ZESP, ZEW, ZGDT, ZGDTI, &
 & ZPBL, ZPBLF, ZPRRAP, ZRTI, ZRVMD, ZRZ, ZRZ1, &
 & ZSTA, ZTUNE, ZUSTAR, ZVA, ZVB, ZVC, ZVSTAR, &
 & ZWA, ZX, ZXX, ZXZX, ZXZX1, ZZB, ZZC, ZZD, &
 & ZZP, ZZPA, ZZX, ZZX1, ZZX2, ZZZB, ZZZR, ZZZRZ  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcttrm.h"
#include "fcttrmad.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACDRAGLAD',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'DRAGLAD') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!     I - COMPUTATION OF DERIVED PARAMETERS, SECURITY CONSTANTS (FOR THE
!     ANISOTROPY FACTOR, THE SQUARE OF THE WIND SPEED, THE NORMALISED
!     STATIC STABILITY AND A PRECISION THRESHOLD FOR RECOMPUTED
!     PRESSURES) AS WELL AS LIMIT ARGUMENT FOR THE SINE FUNCTION.

ZGDT=RG*TSPHY
ZGDTI=1.0_JPRB/ZGDT

!     THE 'ALP' AND 'DEL' VALUES ARE RESULTING FROM ANALYTICAL FITS OF
!     ELLIPTIC INTEGRALS FOR THE ANISOTROPIC CASE.

ZALP1=0.75_JPRB+0.25_JPRB*LOG(16._JPRB)
ZALP2=3.5_JPRB*RPI-LOG(16._JPRB)-8._JPRB
ZDEL1=2.75_JPRB-0.75_JPRB*LOG(16._JPRB)
ZDEL2=-0.25_JPRB*RPI-LOG(16._JPRB)+4._JPRB

ZEPS1=1.E-06_JPRB
ZEPS2=1.E-04_JPRB
ZEPS3=1.E-09_JPRB
ZEPS5=1.E-06_JPRB
ZEPS6=1.E-08_JPRB

!*
!     ----------------------------------------------------------------
!     0. 2 - Computation of specific humidity of saturation and
!     effective ratio of L and Cp for condensation/evaporation,
!     computation of gass constant for air

! For trajectory
! ---------------------------
! - TEMPORARY(S) 2D (1:KLEV) .

! ZLH5      : LATENT HEAT
! ZQSAT5    : SPECIFIC HUMIDITY OF SATURATION
! ZLSCPE5   : EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
! ZR5       : GASS CONSTANT FOR AIR

! ZCP5      : VALUE OF ATMOSPHERIC Cp

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA

    ZR5(JLON,JLEV)=RD*(1.0_JPRB-PQ5(JLON,JLEV))+RV*PQ5(JLON,JLEV)
    ZCP5(JLON,JLEV)=RCPD*(1.0_JPRB-PQ5(JLON,JLEV))+RCPV*PQ5(JLON,JLEV)

!     Snow option dependent computations.

    IF (LNEIGE) THEN
      IF ((RTT-PT5(JLON,JLEV)) < 0.0_JPRB) THEN
        ZDELTA=0.0_JPRB
      ELSE
        ZDELTA=1.0_JPRB
      ENDIF
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF

!     Call to the defined function to compute LH.

    ZLH5(JLON,JLEV)= FOLH (PT5(JLON,JLEV),ZDELTA)

!     Computation of effective ratio of L and Cp for cond./evap.

    ZLSCPE5(JLON,JLEV)=ZLH5(JLON,JLEV)/ZCP5(JLON,JLEV)

!     Computation of Qsat(PT)

    ZEW5(JLON,JLEV)= FOEW (PT5(JLON,JLEV),ZDELTA)
    ZESP5(JLON,JLEV)=ZEW5(JLON,JLEV)/PAPRSF5(JLON,JLEV)
    ZQSAT5(JLON,JLEV)= FOQS (ZESP5(JLON,JLEV))
  ENDDO
ENDDO

!*
!     -----------------------------------------------------------------
!     0. 3 - Computation of one over G time the Brunt-Vaisala frequency
!     divided by density the whole being squared as well as density
!     for G.W.D.

! For trajectory
! --------------
! - TEMPORARY(S) 2D (0:KLEV) .

! ZNBVNO5   : SQUARE OF BRUNT-VAISALA FR. DIVIDED BY G AND DENSITY

! - TEMPORARY 1D

! ZGWDCS5   : DENSITY AT THE SURFACE FOR OROGRAPHIC DRAG.

ZRVMD=RV-RD

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA

    ZDPHI5(JLON,JLEV)=PAPHIF5(JLON,JLEV)-PAPHIF5(JLON,JLEV+1)
    ZCIS5(JLON,JLEV)=MAX(ZEPS2 &
     & ,(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))**2 &
     & +(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))**2)  

!     Preliminary stability computation.

    ZDTETA5(JLON,JLEV)=ZR5(JLON,JLEV)*PT5(JLON,JLEV)&
     & -ZR5(JLON,JLEV+1)*PT5(JLON,JLEV+1)&
     & +RKAPPA*ZDPHI5(JLON,JLEV)  

!     Correction for shallow convection (optional).

    IF (LCVPP) THEN
      ZAUX5(JLON,JLEV)=MIN(0.0_JPRB,PQ5(JLON,JLEV)-ZQSAT5(JLON,JLEV)&
       & -PQ5(JLON,JLEV+1)&
       & +ZQSAT5(JLON,JLEV+1))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))*MAX(0.0_JPRB,SIGN(1.0_JPRB,RD &
       & *(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV)+(ZQSAT5(JLON,JLEV)&
       & -ZQSAT5(JLON,JLEV+1))*PAPRS5(JLON,JLEV)&
       & /PAPRS5(JLON,KLEV))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))-ZDTETA5(JLON,JLEV)))  

      ZZB5(JLON,JLEV)=SIGN(1.0_JPRB,RD &
       & *(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV)+(ZQSAT5(JLON,JLEV)&
       & -ZQSAT5(JLON,JLEV+1))*PAPRS5(JLON,JLEV)&
       & /PAPRS5(JLON,KLEV))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))-ZDTETA5(JLON,JLEV))  

      ZZC5(JLON,JLEV)=PQ5(JLON,JLEV)-ZQSAT5(JLON,JLEV)&
       & -PQ5(JLON,JLEV+1)+ZQSAT5(JLON,JLEV+1)  
      IF (ZZC5(JLON,JLEV) < 0.0_JPRB) THEN
        ZZD5(JLON,JLEV)=ZZC5(JLON,JLEV)
      ELSE
        ZZD5(JLON,JLEV)=0.0_JPRB
      ENDIF

    ELSE
      ZAUX5(JLON,JLEV)=0.0_JPRB
    ENDIF

!     Stability computation.

    ZRTI5(JLON,JLEV)=2.0_JPRB/(ZR5(JLON,JLEV)*PT5(JLON,JLEV)&
     & +ZR5(JLON,JLEV+1)*PT5(JLON,JLEV+1))  
    ZSTA5(JLON,JLEV)=ZDPHI5(JLON,JLEV)*(ZDTETA5(JLON,JLEV)&
     & +RD*ZAUX5(JLON,JLEV))*ZRTI5(JLON,JLEV)  

!     Computation of one over G time the Brunt-Vaisala frequency divided
!     by density the whole being squared.

    ZNBVNO5(JLON,JLEV)=ZSTA5(JLON,JLEV)/(PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)*ZDPHI5(JLON,JLEV))**2  

  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA

  ZDPHI5(JLON,KLEV)=PAPHIF5(JLON,KLEV)-PAPHI5(JLON,KLEV)

  ZRS(JLON)=RD+ZRVMD*PQS(JLON)
  ZCIS5(JLON,KLEV)=MAX(ZEPS2,PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2)

!     Stability computation.

  ZRTI5(JLON,KLEV)=2.0_JPRB/(ZR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)+ZRS(JLON)*PTS(JLON))  

  ZSTA5(JLON,KLEV)=ZDPHI5(JLON,KLEV)*(ZR5(JLON,KLEV)&
   & *PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)&
   & -ZRS(JLON)*PTS(JLON))*ZRTI5(JLON,KLEV)  

!     Computation of one over G time the Brunt-Vaisala frequency divided
!     by density the whole being squared as well as density (for G.W.D.)

  ZNBVNO5(JLON,KLEV)=ZSTA5(JLON,KLEV)/(PAPRS5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZDPHI5(JLON,KLEV))**2  

  ZGWDCS5(JLON)=PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)

ENDDO

!*
!     ------------------------------------------------------------------
!     0. 4 - Preliminary computations for the layer's pressure thickness
!     divided by G*DT and its inverse for all levels as well as setting
!     to zero the weights for the integral over the depth of the
!     orographic obstacle.

! For trajectory
! --------------
! - TEMPORARY(S) 2D (1:KLEV) .

! ZPOID5    : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI5    : INVERSE OF ZPOID.
! ZWGHT5    : WEIGHTS FOR THE INTEGRAL OVER THE DEPTH OF THE OBSTACLE.

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA

    ZPOID5(JLON,JLEV)=PDELP5(JLON,JLEV)*ZGDTI
    ZIPOI5(JLON,JLEV)=PRDELP5(JLON,JLEV)*ZGDT
    ZWGHT15(JLON,JLEV)=0.0_JPRB

  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     0. 5 - Surface computations and initialization for the ascending
!     loop.

! For trajectory
! --------------
! - TEMPORARY(S) 2D (0:KLEV) .

! ZNFNO5    : NORMALISED EFECTIVE BRUNT-VAISALA FREQUENCY (N/RHO/G).
! ZRAPP5    : RATIO OF THE UPPER AIR FLUX TO THE SURFACE ONE.
! ZU5       : X COMPONENT OF THE EFFECTIVE WIND.
! ZV5       : Y COMPONENT OF THE EFFECTIVE WIND.

!     Vertical averaging over the thickness of the orographic obstacle
!     and calculation of the effective values (wind and stability)
!     through linear combinations between the averages (chosen at the
!     surface) and the original values (obtained again at the effective
!     height of the obstacles.

! For trajectory
! --------------
! - TEMPORARY 1D .

! ZOBST5    : EFFECTIVE HEIGHT OF THE OBSTACLE IN PRESSURE COORDINATE.

! - TEMPORARY(S) 2D .
! ZSUMF5    : AVERAGED VALUE FOR A SQUARED "N/RHO/G" (<0 ACCEPTED).
! ZSUMU5    : AVERAGED VALUE FOR THE "U" WIND.
! ZSUMV5    : AVERAGED VALUE FOR THE "V" WIND.

!     Weighted summation.

DO JLON=KIDIA,KFDIA
  ZOBST5(JLON)=MIN(PAPRS5(JLON,KLEV)&
   & -PAPRSF5(JLON,KTDIA),MAX(PAPRS5(JLON,KLEV)&
   & -PAPRSF5(JLON,KLEV)&
   & ,HOBST*PGETRL(JLON)*ZGWDCS5(JLON)))  

  ZVA5(JLON)=PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV)
  ZVB5(JLON)=HOBST*PGETRL(JLON)*ZGWDCS5(JLON)

  IF (ZVA5(JLON) > ZVB5(JLON)) THEN
    ZVC5(JLON)=ZVA5(JLON)
  ELSE
    ZVC5(JLON)=ZVB5(JLON)
  ENDIF

  ZWGHT5(JLON,KLEV)=(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))/ZOBST5(JLON)
  ZSUMU5(JLON,KLEV)=ZWGHT5(JLON,KLEV)*PU5(JLON,KLEV)
  ZSUMV5(JLON,KLEV)=ZWGHT5(JLON,KLEV)*PV5(JLON,KLEV)
  ZSUMF5(JLON,KLEV)=ZWGHT5(JLON,KLEV)*ZNBVNO5(JLON,KLEV)
ENDDO

DO JLEV=KLEV-1,KTDIA,-1

  DO JLON=KIDIA,KFDIA

    ZWGHT5(JLON,JLEV)=MAX(0.0_JPRB,(PAPRSF5(JLON,JLEV+1)&
     & -MAX(PAPRSF5(JLON,JLEV),PAPRS5(JLON,KLEV)&
     & -ZOBST5(JLON)))/ZOBST5(JLON))  

    ZXZX=PAPRS5(JLON,KLEV)-ZOBST5(JLON)
    IF (ZXZX > PAPRSF5(JLON,JLEV)) THEN
      ZWA5(JLON,JLEV)=PAPRS5(JLON,KLEV)-ZOBST5(JLON)
    ELSE
      ZWA5(JLON,JLEV)=PAPRSF5(JLON,JLEV)
    ENDIF

    ZSUMU5(JLON,JLEV)=ZSUMU5(JLON,JLEV+1)+0.5_JPRB*ZWGHT5(JLON,JLEV)&
     & *(PU5(JLON,JLEV)+PU5(JLON,JLEV+1))  
    ZSUMV5(JLON,JLEV)=ZSUMV5(JLON,JLEV+1)+0.5_JPRB*ZWGHT5(JLON,JLEV)&
     & *(PV5(JLON,JLEV)+PV5(JLON,JLEV+1))  
    ZSUMF5(JLON,JLEV)=ZSUMF5(JLON,JLEV+1)&
     & +ZWGHT5(JLON,JLEV)*ZNBVNO5(JLON,JLEV)  
  ENDDO

ENDDO

!     Linear combination.

DO JLON=KIDIA,KFDIA
  ZNFNO5(JLON,KLEV)=SQRT(MAX(0.0_JPRB,ZSUMF5(JLON,KTDIA)))

  ZPBLF5(JLON,KLEV)=1.0_JPRB-MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)&
   & -PAPRSF5(JLON,KLEV))/ZOBST5(JLON))  
  ZU5(JLON,KLEV)=PU5(JLON,KLEV)+ZPBLF5(JLON,KLEV)&
   & *(ZSUMU5(JLON,KTDIA)-PU5(JLON,KLEV))  
  ZV5(JLON,KLEV)=PV5(JLON,KLEV)+ZPBLF5(JLON,KLEV)&
   & *(ZSUMV5(JLON,KTDIA)-PV5(JLON,KLEV))  
ENDDO

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZPBL5(JLON,JLEV)=1.0_JPRB-MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)-PAPRS5(JLON,&
     & JLEV))&
     & /ZOBST5(JLON))  
    ZNFNO5(JLON,JLEV)=SQRT(MAX(0.0_JPRB,ZNBVNO5(JLON,JLEV)+ZPBL5(JLON,&
     & JLEV)&
     & *(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV))))  

    ZPBLF5(JLON,JLEV)=1.0_JPRB-MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)&
     & -PAPRSF5(JLON,JLEV))/ZOBST5(JLON))  
    ZU5(JLON,JLEV)=PU5(JLON,JLEV)+ZPBLF5(JLON,JLEV)&
     & *(ZSUMU5(JLON,KTDIA)-PU5(JLON,JLEV))  
    ZV5(JLON,JLEV)=PV5(JLON,JLEV)+ZPBLF5(JLON,JLEV)&
     & *(ZSUMV5(JLON,KTDIA)-PV5(JLON,JLEV))  
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA

!     Computation of a fictitious wind that would correspond, in the
!     isotropic case, to the anisotropic drag.

! For trajectory
! ---------------
! - TEMPORARY(S) 1D .

! ZUSUR5    : X COMPONENT OF THE FICTITIOUS SURFACE WIND.
! ZVSUR5    : Y COMPONENT OF THE FICTITIOUS SURFACE WIND.

  ZUSTAR5(JLON)=ZSUMU5(JLON,KTDIA)*COS(2.0_JPRB*PVRLDI(JLON))&
   & +ZSUMV5(JLON,KTDIA)*SIN(2.0_JPRB*PVRLDI(JLON))  

  ZVSTAR5(JLON)=ZSUMU5(JLON,KTDIA)*SIN(2.0_JPRB*PVRLDI(JLON))&
   & -ZSUMV5(JLON,KTDIA)*COS(2.0_JPRB*PVRLDI(JLON))  

  ZA=PVRLAN(JLON)**2+0.5_JPRB*(4._JPRB*(1.0_JPRB-PVRLAN(JLON))*(1.0_JPRB+ZALP1 &
   & *PVRLAN(JLON))+PVRLAN(JLON)*LOG(1.0_JPRB/MAX(ZEPS1,PVRLAN(JLON)))&
   & *(1.0_JPRB+ZALP2*PVRLAN(JLON)))/RPI  
  ZD=0.5_JPRB*(4._JPRB*(1.0_JPRB-PVRLAN(JLON))*(1.0_JPRB+ZDEL1*PVRLAN(JLON))-3._JPRB &
   & *PVRLAN(JLON)*LOG(1.0_JPRB/MAX(ZEPS1,PVRLAN(JLON)))*(1.0_JPRB+ZDEL2 &
   & *PVRLAN(JLON)))/RPI  

  ZUSUR5(JLON)=ZA*ZSUMU5(JLON,KTDIA)+ZD*ZUSTAR5(JLON)
  ZVSUR5(JLON)=ZA*ZSUMV5(JLON,KTDIA)+ZD*ZVSTAR5(JLON)

!     "Linear" part of the computation.

! For trajectory
! ---------------
! - TEMPORARY(S) 1D .

! ZUSR5     : SPEED OF FICTITIOUS SURFACE WIND.
! ZFACS5    : SURFACE FACTOR FOR THE COMPUTATION OF ZRAPP.
! ZSTUS5    : "LINEAR" VALUE OF THE "U" SURFACE FLUX.
! ZSTVS5    : "LINEAR" VALUE OF THE "V" SURFACE FLUX.
! ZSUSR5    : SURF. SCALAR PRODUCT OF EFFECTIVE AND FICITIOUS WINDS.

  ZSUSR5(JLON)=MAX(ZEPS2,ZSUMU5(JLON,KTDIA)*ZUSUR5(JLON)&
   & +ZSUMV5(JLON,KTDIA)*ZVSUR5(JLON))  
  ZUSR5(JLON)=MAX(0.0_JPRB,SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2))
  ZFACS5(JLON)=ZNFNO5(JLON,KLEV)/ZSUSR5(JLON)**3

  ZRAPP5(JLON,KLEV)=1.0_JPRB

  ZSTUS5(JLON)=GWDSE*ZGWDCS5(JLON)**2*ZNFNO5(JLON,KLEV)&
   & *PGETRL(JLON)*ZUSUR5(JLON)  
  ZSTVS5(JLON)=GWDSE*ZGWDCS5(JLON)**2*ZNFNO5(JLON,KLEV)&
   & *PGETRL(JLON)*ZVSUR5(JLON)  

  ZTUNE5(JLON)=GWDBC*(ZUSR5(JLON)**2/ZSUSR5(JLON))**2

  ZBB5(JLON)=MAX(0.0_JPRB,1.0_JPRB-ZUSR5(JLON)/MAX(ZEPS5,ZTUNE5(JLON)&
   & *HOBST*PGETRL(JLON)*ZGWDCS5(JLON)&
   & *ZNFNO5(JLON,KLEV)))  
  ZZZB=ZTUNE5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZNFNO5(JLON,KLEV)
  IF (ZZZB > ZEPS5) THEN
    ZBB15(JLON)=ZTUNE5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)&
     & *ZNFNO5(JLON,KLEV)  
  ELSE
    ZBB15(JLON)=ZEPS5
  ENDIF

  ZAA5(JLON)=GWDCD*ZTUNE5(JLON)*ZBB5(JLON)*(1.0_JPRB-ZBB5(JLON))
  IF (LSMOOTHB) ZAA5(JLON)=0.0_JPRB
ENDDO

!*
!     ------------------------------------------------------------------
!     0. 6 - Ascending computations of the linear estimate of the relative
!     flux and checking of the first levels of deposition and of
!     reflexion (if the latter exists).

!     Computations in the vertical loop.

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA

!     "Linear" part of the computation.

    ZAUSR5(JLON,JLEV)=0.5_JPRB*((ZU5(JLON,JLEV)+ZU5(JLON,JLEV+1))&
     & *ZUSUR5(JLON)&
     & +(ZV5(JLON,JLEV)+ZV5(JLON,JLEV+1))*ZVSUR5(JLON))  

    ZPRRAP5(JLON,JLEV)=ZFACS5(JLON)*ZAUSR5(JLON,JLEV)**3 &
     & /MAX(ZEPS3,ZNFNO5(JLON,JLEV))  

    ZZP5(JLON,JLEV)=MIN(ZRAPP5(JLON,JLEV+1),ZPRRAP5(JLON,JLEV))
    ZRAPP5(JLON,JLEV)=MAX(0.0_JPRB,MIN(ZRAPP5(JLON,JLEV+1),ZPRRAP5(JLON,JLEV)))

  ENDDO
ENDDO

!     Upper boundary condition and preparation of the descending loop.

DO JLON=KIDIA,KFDIA

!     "Linear" part of the computation.

  PSTRDU5(JLON,KTDIA-1)=0.0_JPRB
  PSTRDV5(JLON,KTDIA-1)=0.0_JPRB
  ZRAPP5(JLON,KTDIA-1)=0.0_JPRB

  Z_PSTRDU25(JLON,KTDIA-1)=0.0_JPRB
  Z_PSTRDV25(JLON,KTDIA-1)=0.0_JPRB
ENDDO

!*
!     ------------------------------------------------------------------
!     0. 7 - Descending loop for the combination of effects and the
!     numerical securities leading to the final flux calculation.

!     Provisional fluxes taking into account all three effects.

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZZPA5(JLON,JLEV)=MIN(1.0_JPRB,ZRAPP5(JLON,JLEV))
    ZRAPP15(JLON,JLEV)=MAX(0.0_JPRB,ZZPA5(JLON,JLEV))

    ZRZ5(JLON,JLEV)=MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))&
     & /MAX(ZEPS6,ZBB5(JLON)*HOBST*PGETRL(JLON)&
     & *ZGWDCS5(JLON)))  
    IF((ZBB5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)) > ZEPS6)THEN
      ZRZ15(JLON,JLEV)=ZBB5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)
    ELSE
      ZRZ15(JLON,JLEV)=ZEPS6
    ENDIF

    ZZZR=(1.0_JPRB-ZRZ5(JLON,JLEV))**3/(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON))
    IF (ZZZR > ZEPS6) THEN
      ZRAPPH5(JLON,JLEV)=(1.0_JPRB-ZRZ5(JLON,JLEV))**3 &
       & /(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON))  
    ELSE
      ZRAPPH5(JLON,JLEV)=ZEPS6
    ENDIF

    ZRAPP115(JLON,JLEV)=ZRAPP15(JLON,JLEV)*(1.0_JPRB+ZAA5(JLON)&
     & *SQRT(MAX(ZEPS6,(1.0_JPRB-ZRZ5(JLON,JLEV))**3 &
     & /(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON)))))  

    Z_PSTRDU15(JLON,JLEV)=ZRAPP115(JLON,JLEV)*ZSTUS5(JLON)
    Z_PSTRDV15(JLON,JLEV)=ZRAPP115(JLON,JLEV)*ZSTVS5(JLON)
  ENDDO
ENDDO

!     Pseudo-implicit algorithm with respect to the effective surface
!     wind projected on the stress direction.

! For trajectory
! --------------
! - TEMPORARY(S) 2D .

! ZSUMD5    : AVERAGED WIND TENDENCIES PROJECTED ON THE STRESS' DIREC..

!     Summation for a temporary normalised average of tendencies.

DO JLON=KIDIA,KFDIA
  ZSUMD5(JLON,KTDIA)=0.5_JPRB*ZWGHT5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA)&
   & *(ZRAPP115(JLON,KTDIA)-ZRAPP5(JLON,KTDIA-1))  
ENDDO

DO JLEV=KTDIA+1,KLEV
  DO JLON=KIDIA,KFDIA
    ZSUMD5(JLON,JLEV)=ZSUMD5(JLON,JLEV-1)+0.5_JPRB*(ZWGHT5(JLON,JLEV)&
     & +ZWGHT5(JLON,JLEV-1))*ZIPOI5(JLON,JLEV)&
     & *(ZRAPP115(JLON,JLEV)-ZRAPP115(JLON,JLEV-1))  
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZSUMD15(JLON)=ZSUMD5(JLON,KLEV)&
   & +0.5_JPRB*ZWGHT5(JLON,KLEV)*ZIPOI5(JLON,KLEV)&
   & *(ZRAPP115(JLON,KLEV)-ZRAPP115(JLON,KLEV-1))  
ENDDO

!     Final averaged value, linear reduction factor and its use.

DO JLON=KIDIA,KFDIA
  ZSUMD25(JLON)=ZSUMD15(JLON)&
   & *MAX(0.0_JPRB,SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2))  
  ZRAPP25(JLON,KLEV)=1.0_JPRB/(1.0_JPRB+ZSUMD25(JLON)*ZUSR5(JLON)/ZSUSR5(JLON))
ENDDO

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    Z_PSTRDU25(JLON,JLEV)=Z_PSTRDU15(JLON,JLEV)*ZRAPP25(JLON,KLEV)
    Z_PSTRDV25(JLON,JLEV)=Z_PSTRDV15(JLON,JLEV)*ZRAPP25(JLON,KLEV)
  ENDDO
ENDDO

!     Numerical security (use of an algorithm inspired by the one of the
!     mass-flux type deep convection routines).

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    IF (JLEV == KTDIA) THEN
      ZDU5(JLON,JLEV)=ZIPOI5(JLON,JLEV)&
       & *(Z_PSTRDU25(JLON,JLEV-1)-Z_PSTRDU25(JLON,JLEV))  
      ZDV5(JLON,JLEV)=ZIPOI5(JLON,JLEV)&
       & *(Z_PSTRDV25(JLON,JLEV-1)-Z_PSTRDV25(JLON,JLEV))  
    ELSE
      ZDU5(JLON,JLEV)=ZIPOI5(JLON,JLEV)&
       & *(PSTRDU5(JLON,JLEV-1)-Z_PSTRDU25(JLON,JLEV))  
      ZDV5(JLON,JLEV)=ZIPOI5(JLON,JLEV)&
       & *(PSTRDV5(JLON,JLEV-1)-Z_PSTRDV25(JLON,JLEV))  
    ENDIF

    ZDX5(JLON,JLEV)=ZIPOI5(JLON,JLEV)*(ZUSUR5(JLON)&
     & *Z_PSTRDU25(JLON,JLEV)&
     & +ZVSUR5(JLON)*Z_PSTRDV25(JLON,JLEV))  

    ZX5(JLON,JLEV)=ZUSUR5(JLON)*PU5(JLON,JLEV)+ZVSUR5(JLON)*PV5(JLON,JLEV)
    ZALPHA5(JLON,JLEV)=1.0_JPRB/(1.0_JPRB+ZDX5(JLON,JLEV)&
     & /MAX(ZEPS2,ZX5(JLON,JLEV)))  

    IF (JLEV == KTDIA) THEN
      PSTRDU5(JLON,JLEV)=Z_PSTRDU25(JLON,JLEV-1)-ZPOID5(JLON,JLEV)&
       & *ZALPHA5(JLON,JLEV)*ZDU5(JLON,JLEV)  
      PSTRDV5(JLON,JLEV)=Z_PSTRDV25(JLON,JLEV-1)-ZPOID5(JLON,JLEV)&
       & *ZALPHA5(JLON,JLEV)*ZDV5(JLON,JLEV)  
    ELSE
      PSTRDU5(JLON,JLEV)=PSTRDU5(JLON,JLEV-1)-ZPOID5(JLON,JLEV)&
       & *ZALPHA5(JLON,JLEV)*ZDU5(JLON,JLEV)  
      PSTRDV5(JLON,JLEV)=PSTRDV5(JLON,JLEV-1)-ZPOID5(JLON,JLEV)&
       & *ZALPHA5(JLON,JLEV)*ZDV5(JLON,JLEV)  
    ENDIF
  ENDDO
ENDDO
!*

!     INITIALIZATION OF THE LOCAL VARIABLES

ZGWDCS(:)=0.0_JPRB
ZCP(:)=0.0_JPRB
ZFACS(:)=0.0_JPRB
ZOBST(:)=0.0_JPRB
ZSTUS(:)=0.0_JPRB
ZSTVS(:)=0.0_JPRB
ZSUMD(:)=0.0_JPRB
ZSUMF(:)=0.0_JPRB
ZSUMU(:)=0.0_JPRB
ZSUMV(:)=0.0_JPRB
ZSUSR(:)=0.0_JPRB
ZUSR(:)=0.0_JPRB
ZUSUR(:)=0.0_JPRB
ZVSUR(:)=0.0_JPRB
ZAA(:)=0.0_JPRB
ZBB(:)=0.0_JPRB

ZLSCPE(:,:)=0.0_JPRB
ZQSAT(:,:)=0.0_JPRB
ZR(:,:)=0.0_JPRB
ZLH(:,:)=0.0_JPRB
ZIPOI(:,:)=0.0_JPRB
ZPOID(:,:)=0.0_JPRB
ZU(:,:)=0.0_JPRB
ZV(:,:)=0.0_JPRB

ZNBVNO(:,:)=0.0_JPRB
ZRAPP(:,:)=0.0_JPRB
ZNFNO(:,:)=0.0_JPRB
ZWGHT(:,:)=0.0_JPRB
ZRAPPH(:,:)=0.0_JPRB

!*
!     ------------------------------------------------------------------
!     VII - DESCENDING LOOP FOR THE COMBINATION OF EFFECTS AND THE
!     NUMERICAL SECURITIES LEADING TO THE FINAL FLUX CALCULATION.

!     PSEUDO-IMPLICIT ALGORITHM WITH RESPECT TO THE EFFECTIVE SURFACE
!     WIND PROJECTED ON THE STRESS DIRECTION.

! - TEMPORARY(S) 1D .

! ZSUMD     : AVERAGED WIND TENDENCIES PROJECTED ON THE STRESS' DIREC..

!     NUMERICAL SECURITY (USE OF AN ALGORITHM INSPIRED BY THE ONE OF THE
!     MASS-FLUX TYPE DEEP CONVECTION ROUTINES).

DO JLEV=KLEV,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZALPHA=0.0_JPRB
    ZDU=0.0_JPRB
    ZDV=0.0_JPRB
    ZX=0.0_JPRB
    ZDX=0.0_JPRB

    ZPOID(JLON,JLEV)=ZPOID(JLON,JLEV)-ZALPHA5(JLON,JLEV)&
     & *ZDV5(JLON,JLEV)*PSTRDV(JLON,JLEV)  
    ZDV=ZDV-ZPOID5(JLON,JLEV)*ZALPHA5(JLON,JLEV)*PSTRDV(JLON,JLEV)
    ZALPHA=ZALPHA-ZPOID5(JLON,JLEV)*ZDV5(JLON,JLEV)*PSTRDV(JLON,JLEV)
    PSTRDV(JLON,JLEV-1)=PSTRDV(JLON,JLEV-1)+PSTRDV(JLON,JLEV)
    PSTRDV(JLON,JLEV)=0.0_JPRB

    ZPOID(JLON,JLEV)=ZPOID(JLON,JLEV)-ZALPHA5(JLON,JLEV)&
     & *ZDU5(JLON,JLEV)*PSTRDU(JLON,JLEV)  
    ZDU=ZDU-ZPOID5(JLON,JLEV)*ZALPHA5(JLON,JLEV)*PSTRDU(JLON,JLEV)
    ZALPHA=ZALPHA-ZPOID5(JLON,JLEV)*ZDU5(JLON,JLEV)*PSTRDU(JLON,JLEV)
    PSTRDU(JLON,JLEV-1)=PSTRDU(JLON,JLEV-1)+PSTRDU(JLON,JLEV)
    PSTRDU(JLON,JLEV)=0.0_JPRB

    IF (ZX5(JLON,JLEV) > ZEPS2) THEN
      ZX=ZX+ZALPHA/(ZX5(JLON,JLEV)+ZDX5(JLON,JLEV))
      ZX=ZX-(ZX5(JLON,JLEV)/(ZX5(JLON,JLEV)+ZDX5(JLON,JLEV))**2)*ZALPHA
      ZDX=ZDX-(ZX5(JLON,JLEV)/(ZX5(JLON,JLEV)+ZDX5(JLON,JLEV))**2)*ZALPHA
    ELSE
      ZDX=ZDX-ZEPS2*ZALPHA/(ZEPS2+ZDX5(JLON,JLEV))**2
    ENDIF

    PU(JLON,JLEV)=PU(JLON,JLEV)+ZUSUR5(JLON)*ZX
    ZUSUR(JLON)=ZUSUR(JLON)+PU5(JLON,JLEV)*ZX
    PV(JLON,JLEV)=PV(JLON,JLEV)+ZVSUR5(JLON)*ZX
    ZVSUR(JLON)=ZVSUR(JLON)+PV5(JLON,JLEV)*ZX

    ZIPOI(JLON,JLEV)=ZIPOI(JLON,JLEV)&
     & +(ZUSUR5(JLON)*Z_PSTRDU25(JLON,JLEV)&
     & +ZVSUR5(JLON)*Z_PSTRDV25(JLON,JLEV))*ZDX  
    PSTRDU(JLON,JLEV)=PSTRDU(JLON,JLEV)+ZIPOI5(JLON,JLEV)*ZUSUR5(JLON)*ZDX
    ZUSUR(JLON)=ZUSUR(JLON)+ZIPOI5(JLON,JLEV)*Z_PSTRDU25(JLON,JLEV)*ZDX
    PSTRDV(JLON,JLEV)=PSTRDV(JLON,JLEV)+ZIPOI5(JLON,JLEV)*ZVSUR5(JLON)*ZDX
    ZVSUR(JLON)=ZVSUR(JLON)+ZIPOI5(JLON,JLEV)*Z_PSTRDV25(JLON,JLEV)*ZDX

    PSTRDV(JLON,JLEV-1)=PSTRDV(JLON,JLEV-1)+ZIPOI5(JLON,JLEV)*ZDV
    PSTRDV(JLON,JLEV)=PSTRDV(JLON,JLEV)-ZIPOI5(JLON,JLEV)*ZDV
    IF (JLEV == KTDIA) THEN
      ZIPOI(JLON,JLEV)=ZIPOI(JLON,JLEV)&
       & +(Z_PSTRDV25(JLON,JLEV-1)-Z_PSTRDV25(JLON,JLEV))*ZDV  
    ELSE
      ZIPOI(JLON,JLEV)=ZIPOI(JLON,JLEV)&
       & +(PSTRDV5(JLON,JLEV-1)-Z_PSTRDV25(JLON,JLEV))*ZDV  
    ENDIF

    PSTRDU(JLON,JLEV-1)=PSTRDU(JLON,JLEV-1)+ZIPOI5(JLON,JLEV)*ZDU
    PSTRDU(JLON,JLEV)=PSTRDU(JLON,JLEV)-ZIPOI5(JLON,JLEV)*ZDU
    IF (JLEV == KTDIA) THEN
      ZIPOI(JLON,JLEV)=ZIPOI(JLON,JLEV)&
       & +(Z_PSTRDU25(JLON,JLEV-1)-Z_PSTRDU25(JLON,JLEV))*ZDU  
    ELSE
      ZIPOI(JLON,JLEV)=ZIPOI(JLON,JLEV)&
       & +(PSTRDU5(JLON,JLEV-1)-Z_PSTRDU25(JLON,JLEV))*ZDU  
    ENDIF
  ENDDO
ENDDO

!     FINAL AVERAGED VALUE, LINEAR REDUCTION FACTOR AND ITS USE.

DO JLEV=KLEV,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZRAPP(JLON,KLEV)=ZRAPP(JLON,KLEV)+Z_PSTRDV15(JLON,JLEV)*PSTRDV(JLON,JLEV)
    PSTRDV(JLON,JLEV)=PSTRDV(JLON,JLEV)*ZRAPP25(JLON,KLEV)

    ZRAPP(JLON,KLEV)=ZRAPP(JLON,KLEV)+Z_PSTRDU15(JLON,JLEV)*PSTRDU(JLON,JLEV)
    PSTRDU(JLON,JLEV)=PSTRDU(JLON,JLEV)*ZRAPP25(JLON,KLEV)
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZSUSR(JLON)=ZSUSR(JLON)+ZRAPP(JLON,KLEV)&
   & /(ZSUSR5(JLON)+ZSUMD25(JLON)*ZUSR5(JLON))  
  ZSUSR(JLON)=ZSUSR(JLON)-ZSUSR5(JLON)*ZRAPP(JLON,KLEV)&
   & /(ZSUSR5(JLON)+ZSUMD25(JLON)*ZUSR5(JLON))**2  
  ZUSR(JLON)=ZUSR(JLON)&
   & -ZSUMD25(JLON)*ZSUSR5(JLON)*ZRAPP(JLON,KLEV)&
   & /(ZSUSR5(JLON)+ZSUMD25(JLON)*ZUSR5(JLON))**2  
  ZSUMD(JLON)=ZSUMD(JLON)&
   & -ZUSR5(JLON)*ZSUSR5(JLON)*ZRAPP(JLON,KLEV)&
   & /(ZSUSR5(JLON)+ZSUMD25(JLON)*ZUSR5(JLON))**2  
  ZRAPP(JLON,KLEV)=0.0_JPRB

  IF ((SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)) > 0.0_JPRB) THEN
    ZSTUS(JLON)=ZSTUS(JLON)+ZSUMD15(JLON)*ZSTUS5(JLON)*ZSUMD(JLON)&
     & /SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)  
    ZSTVS(JLON)=ZSTVS(JLON)+ZSUMD15(JLON)*ZSTVS5(JLON)*ZSUMD(JLON)&
     & /SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)  
    ZSUMD(JLON)=ZSUMD(JLON)*SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)
  ELSE
    ZSUMD(JLON)=0.0_JPRB
  ENDIF
ENDDO

!     SUMMATION FOR A TEMPORARY NORMALISED AVERAGE OF TENDENCIES.

DO JLON=KIDIA,KFDIA
  ZWGHT(JLON,KLEV)=ZWGHT(JLON,KLEV)&
   & +0.5_JPRB*(ZRAPP115(JLON,KLEV)-ZRAPP115(JLON,KLEV-1))&
   & *ZIPOI5(JLON,KLEV)*ZSUMD(JLON)  
  ZIPOI(JLON,KLEV)=ZIPOI(JLON,KLEV)&
   & +0.5_JPRB*(ZRAPP115(JLON,KLEV)-ZRAPP115(JLON,KLEV-1))&
   & *ZWGHT5(JLON,KLEV)*ZSUMD(JLON)  
  ZRAPP(JLON,KLEV)=ZRAPP(JLON,KLEV)+0.5_JPRB*ZWGHT5(JLON,KLEV)&
   & *ZIPOI5(JLON,KLEV)*ZSUMD(JLON)  
  ZRAPP(JLON,KLEV-1)=ZRAPP(JLON,KLEV-1)-0.5_JPRB*ZWGHT5(JLON,KLEV)&
   & *ZIPOI5(JLON,KLEV)*ZSUMD(JLON)  
  ZSUMD(JLON)=ZSUMD(JLON)
ENDDO

DO JLEV=KLEV,KTDIA+1,-1
  DO JLON=KIDIA,KFDIA
    ZWGHT(JLON,JLEV)=ZWGHT(JLON,JLEV)&
     & +0.5_JPRB*(ZRAPP115(JLON,JLEV)-ZRAPP115(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZSUMD(JLON)  
    ZWGHT(JLON,JLEV-1)=ZWGHT(JLON,JLEV-1)&
     & +0.5_JPRB*(ZRAPP115(JLON,JLEV)-ZRAPP115(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZSUMD(JLON)  
    ZIPOI(JLON,JLEV)=ZIPOI(JLON,JLEV)&
     & +0.5_JPRB*(ZWGHT5(JLON,JLEV)+ZWGHT5(JLON,JLEV-1))&
     & *(ZRAPP115(JLON,JLEV)-ZRAPP115(JLON,JLEV-1))*ZSUMD(JLON)  
    ZRAPP(JLON,JLEV)=ZRAPP(JLON,JLEV)&
     & +0.5_JPRB*(ZWGHT5(JLON,JLEV)+ZWGHT5(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZSUMD(JLON)  
    ZRAPP(JLON,JLEV-1)=ZRAPP(JLON,JLEV-1)&
     & -0.5_JPRB*(ZWGHT5(JLON,JLEV)+ZWGHT5(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZSUMD(JLON)  
    ZSUMD(JLON)=ZSUMD(JLON)
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZWGHT(JLON,KTDIA)=ZWGHT(JLON,KTDIA)+0.5_JPRB*ZIPOI5(JLON,KTDIA)&
   & *(ZRAPP115(JLON,KTDIA)-ZRAPP5(JLON,KTDIA-1))&
   & *ZSUMD(JLON)  
  ZIPOI(JLON,KTDIA)=ZIPOI(JLON,KTDIA)+0.5_JPRB*ZWGHT5(JLON,KTDIA)&
   & *(ZRAPP115(JLON,KTDIA)-ZRAPP5(JLON,KTDIA-1))&
   & *ZSUMD(JLON)  
  ZRAPP(JLON,KTDIA)=ZRAPP(JLON,KTDIA)+0.5_JPRB*ZWGHT5(JLON,KTDIA)&
   & *ZIPOI5(JLON,KTDIA)*ZSUMD(JLON)  
  ZRAPP(JLON,KTDIA-1)=ZRAPP(JLON,KTDIA-1)-0.5_JPRB*ZWGHT5(JLON,KTDIA)&
   & *ZIPOI5(JLON,KTDIA)*ZSUMD(JLON)  
  ZSUMD(JLON)=0.0_JPRB
ENDDO

!     PROVISIONAL FLUXES TAKING INTO ACOUNT ALL THREE EFFECTS.

DO JLEV=KLEV,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZRZ=0.0_JPRB
    ZRZ1=0.0_JPRB
    ZZPA=0.0_JPRB
    ZSTVS(JLON)=ZSTVS(JLON)+ZRAPP115(JLON,JLEV)*PSTRDV(JLON,JLEV)
    ZRAPP(JLON,JLEV)=ZRAPP(JLON,JLEV)+ZSTVS5(JLON)*PSTRDV(JLON,JLEV)
    PSTRDV(JLON,JLEV)=0.0_JPRB

    ZSTUS(JLON)=ZSTUS(JLON)+ZRAPP115(JLON,JLEV)*PSTRDU(JLON,JLEV)
    ZRAPP(JLON,JLEV)=ZRAPP(JLON,JLEV)+ZSTUS5(JLON)*PSTRDU(JLON,JLEV)
    PSTRDU(JLON,JLEV)=0.0_JPRB

    ZAA(JLON)=ZAA(JLON)+ZRAPP15(JLON,JLEV)*SQRT(ZRAPPH5(JLON,&
     & JLEV))&
     & *ZRAPP(JLON,JLEV)  
    ZRAPPH(JLON,JLEV)=ZRAPPH(JLON,JLEV)&
     & +(0.5_JPRB*ZRAPP15(JLON,JLEV)*ZAA5(JLON)&
     & /SQRT(ZRAPPH5(JLON,JLEV)))*ZRAPP(JLON,JLEV)  
    ZRAPP(JLON,JLEV)=&
     & (1.0_JPRB+ZAA5(JLON)*SQRT(ZRAPPH5(JLON,JLEV)))*ZRAPP(JLON,JLEV)  

    ZZZR=(1.0_JPRB-ZRZ5(JLON,JLEV))**3/(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON))
    IF (ZZZR > ZEPS6) THEN
      ZRZ=ZRZ-(3._JPRB*(1.0_JPRB-ZRZ5(JLON,JLEV))**2 &
       & /(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON)))*ZRAPPH(JLON,JLEV)  
      ZBB(JLON)=ZBB(JLON)-((1.0_JPRB-ZRZ5(JLON,JLEV))**3*ZRZ5(JLON,JLEV)&
       & /(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON))**2)*ZRAPPH(JLON,JLEV)  
      ZRZ=ZRZ-((1.0_JPRB-ZRZ5(JLON,JLEV))**3*ZBB5(JLON)&
       & /(1.0_JPRB+ZRZ5(JLON,JLEV)*ZBB5(JLON))**2)*ZRAPPH(JLON,JLEV)  
      ZRAPPH(JLON,JLEV)=0.0_JPRB
    ELSE
      ZRAPPH(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZZZRZ=(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))/ZRZ15(JLON,JLEV)
    IF (ZZZRZ < 1.0_JPRB) THEN
      PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZRZ/ZRZ15(JLON,JLEV)
      PAPRS(JLON,JLEV)=PAPRS(JLON,JLEV)-ZRZ/ZRZ15(JLON,JLEV)
      ZRZ1=ZRZ1-((PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))&
       & /ZRZ15(JLON,JLEV)**2)*ZRZ  
    ENDIF

    IF((ZBB5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)) > ZEPS6)THEN
      ZGWDCS(JLON)=ZGWDCS(JLON)+HOBST*PGETRL(JLON)*ZBB5(JLON)*ZRZ1
      ZBB(JLON)=ZBB(JLON)+HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZRZ1
    ENDIF

    IF (ZZPA5(JLON,JLEV) > 0.0_JPRB) THEN
      ZZPA=ZZPA+ZRAPP(JLON,JLEV)
      ZRAPP(JLON,JLEV)=0.0_JPRB
    ELSE
      ZRAPP(JLON,JLEV)=0.0_JPRB
    ENDIF

    IF (ZRAPP5(JLON,JLEV) < 1.0_JPRB) THEN
      ZRAPP(JLON,JLEV)=ZRAPP(JLON,JLEV)+ZZPA
    ENDIF

  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     VI - ASCENDING COMPUTATIONS OF THE LINEAR ESTIMATE OF THE RELATIVE
!     FLUX AND CHECKING OF THE FIRST LEVELS OF DEPOSITION AND OF
!     REFLEXION (IF THE LATTER EXISTS).

!     UPPER BOUNDARY CONDITION AND PREPARATION OF THE DESCENDING LOOP.

DO JLON=KIDIA,KFDIA

!     "LINEAR" PART OF THE COMPUTATION.

  ZRAPP(JLON,KTDIA-1)=0.0_JPRB
  PSTRDV(JLON,KTDIA-1)=0.0_JPRB
  PSTRDU(JLON,KTDIA-1)=0.0_JPRB
ENDDO

!     COMPUTATIONS IN THE VERTICAL LOOP.

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZZP=0.0_JPRB
    ZPRRAP=0.0_JPRB
    ZAUSR=0.0_JPRB

!     "LINEAR" PART OF THE COMPUTATION.

    IF (ZZP5(JLON,JLEV) > 0.0_JPRB) THEN
      ZZP=ZZP+ZRAPP(JLON,JLEV)
      ZRAPP(JLON,JLEV)=0.0_JPRB
    ELSE
      ZRAPP(JLON,JLEV)=0.0_JPRB
    ENDIF

    IF (ZRAPP5(JLON,JLEV+1) < ZPRRAP5(JLON,JLEV)) THEN
      ZRAPP(JLON,JLEV+1)=ZRAPP(JLON,JLEV+1)+ZZP
    ELSE
      ZPRRAP=ZPRRAP+ZZP
    ENDIF

    IF (ZNFNO5(JLON,JLEV) > ZEPS3) THEN
      ZFACS(JLON)=ZFACS(JLON)+(ZAUSR5(JLON,JLEV)**3/ZNFNO5(JLON,JLEV))*ZPRRAP
      ZAUSR=ZAUSR+(3._JPRB*ZAUSR5(JLON,JLEV)**2*ZFACS5(JLON)&
       & /ZNFNO5(JLON,JLEV))*ZPRRAP  
      ZNFNO(JLON,JLEV)=ZNFNO(JLON,JLEV)&
       & -(ZFACS5(JLON)*ZAUSR5(JLON,JLEV)**3 &
       & /ZNFNO5(JLON,JLEV)**2)*ZPRRAP  
    ELSE
      ZFACS(JLON)=ZFACS(JLON)+(ZAUSR5(JLON,JLEV)**3/ZEPS3)*ZPRRAP
      ZAUSR=ZAUSR+(3._JPRB*ZAUSR5(JLON,JLEV)**2*ZFACS5(JLON)/ZEPS3)*ZPRRAP
    ENDIF

    ZU(JLON,JLEV+1)=ZU(JLON,JLEV+1)+0.5_JPRB*ZUSUR5(JLON)*ZAUSR
    ZU(JLON,JLEV)=ZU(JLON,JLEV)+0.5_JPRB*ZUSUR5(JLON)*ZAUSR
    ZUSUR(JLON)=ZUSUR(JLON)+0.5_JPRB*(ZU5(JLON,JLEV)+ZU5(JLON,JLEV+1))*ZAUSR
    ZV(JLON,JLEV)=ZV(JLON,JLEV)+0.5_JPRB*ZVSUR5(JLON)*ZAUSR
    ZV(JLON,JLEV+1)=ZV(JLON,JLEV+1)+0.5_JPRB*ZVSUR5(JLON)*ZAUSR
    ZVSUR(JLON)=ZVSUR(JLON)+0.5_JPRB*(ZV5(JLON,JLEV)+ZV5(JLON,JLEV+1))*ZAUSR
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     V - SURFACE COMPUTATIONS AND INITIALIZATION FOR THE ASCENDING
!     LOOP.

! - TEMPORARY(S) 2D (0:KLEV) .

! ZNFNO     : NORMALISED EFECTIVE BRUNT-VAISALA FREQUENCY (N/RHO/G).
! ZRAPP     : RATIO OF THE UPPER AIR FLUX TO THE SURFACE ONE.
! ZU        : X COMPONENT OF THE EFFECTIVE WIND.
! ZV        : Y COMPONENT OF THE EFFECTIVE WIND.

!     VERTICAL AVERAGING OVER THE THICKNESS OF THE OROGRAPHIC OBSTACLE
!     AND CALCULATION OF THE EFFECTIVE VALUES (WIND AND STABILITY)
!     THROUGH LINEAR COMBINATIONS BETWEEN THE AVERAGES (CHOSEN AT THE
!     SURFACE) AND THE ORIGINAL VALUES (OBTAINED AGAIN AT THE EFFECTIVE
!     HEIGHT OF THE OBSTACLE).

! - TEMPORARY(S) 1D .

! ZOBST     : EFFECTIVE HEIGHT OF THE OBSTACLE IN PRESSURE COORDINATE.
! ZSUMF     : AVERAGED VALUE FOR A SQUARED "N/RHO/G" (<0 ACCEPTED).
! ZSUMU     : AVERAGED VALUE FOR THE "U" WIND.
! ZSUMV     : AVERAGED VALUE FOR THE "V" WIND.

DO JLON=KIDIA,KFDIA
  ZTUNE=0.0_JPRB
  ZBB1=0.0_JPRB
  ZUSTAR=0.0_JPRB
  ZVSTAR=0.0_JPRB

!     "LINEAR" PART OF THE COMPUTATION.

! - TEMPORARY(S) 1D .

! ZUSR      : SPEED OF FICTITIOUS SURFACE WIND.
! ZFACS     : SURFACE FACTOR FOR THE COMPUTATION OF ZRAPP.
! ZSTUS     : "LINEAR" VALUE OF THE "U" SURFACE FLUX.
! ZSTVS     : "LINEAR" VALUE OF THE "V" SURFACE FLUX.
! ZSUSR     : SURF. SCALAR PRODUCT OF EFFECTIVE AND FICITIOUS WINDS.

  IF (LSMOOTHB) ZAA(JLON)=0.0_JPRB
  ZTUNE=ZTUNE+GWDCD*ZBB5(JLON)*(1.0_JPRB-ZBB5(JLON))*ZAA(JLON)
  ZBB(JLON)=ZBB(JLON)+GWDCD*ZTUNE5(JLON)*(1.0_JPRB-2.0_JPRB*ZBB5(JLON))*ZAA(JLON)
  ZAA(JLON)=0.0_JPRB

  IF ((1.0_JPRB-ZUSR5(JLON)/ZBB15(JLON)) > 0.0_JPRB) THEN
    ZUSR(JLON)=ZUSR(JLON)-ZBB(JLON)/ZBB15(JLON)
    ZBB1=ZBB1+(ZUSR5(JLON)/ZBB15(JLON)**2)*ZBB(JLON)
    ZBB(JLON)=0.0_JPRB
  ELSE
    ZBB(JLON)=0.0_JPRB
  ENDIF

  ZZZB=ZTUNE5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZNFNO5(JLON,KLEV)
  IF (ZZZB > ZEPS5) THEN
    ZTUNE=ZTUNE+HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZNFNO5(JLON,KLEV)*ZBB1
    ZGWDCS(JLON)=ZGWDCS(JLON)+HOBST*PGETRL(JLON)*ZTUNE5(JLON)&
     & *ZNFNO5(JLON,KLEV)*ZBB1  
    ZNFNO(JLON,KLEV)=ZNFNO(JLON,KLEV)+HOBST*PGETRL(JLON)&
     & *ZTUNE5(JLON)*ZGWDCS5(JLON)*ZBB1  
  ENDIF

  ZUSR(JLON)=ZUSR(JLON)+(4._JPRB*GWDBC*ZUSR5(JLON)**3/ZSUSR5(JLON)**2)*ZTUNE
  ZSUSR(JLON)=ZSUSR(JLON)-(2.0_JPRB*GWDBC*ZUSR5(JLON)**4/ZSUSR5(JLON)**3)*ZTUNE

  ZGWDCS(JLON)=ZGWDCS(JLON)+2.0_JPRB*GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)&
   & *ZNFNO5(JLON,KLEV)*ZVSUR5(JLON)*ZSTVS(JLON)  
  ZVSUR(JLON)=ZVSUR(JLON)+GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)**2 &
   & *ZNFNO5(JLON,KLEV)*ZSTVS(JLON)  
  ZNFNO(JLON,KLEV)=ZNFNO(JLON,KLEV)+GWDSE*PGETRL(JLON)&
   & *ZGWDCS5(JLON)**2*ZVSUR5(JLON)*ZSTVS(JLON)  
  ZSTVS(JLON)=0.0_JPRB

  ZGWDCS(JLON)=ZGWDCS(JLON)+2.0_JPRB*GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)&
   & *ZNFNO5(JLON,KLEV)*ZUSUR5(JLON)*ZSTUS(JLON)  
  ZUSUR(JLON)=ZUSUR(JLON)+GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)**2 &
   & *ZNFNO5(JLON,KLEV)*ZSTUS(JLON)  
  ZNFNO(JLON,KLEV)=ZNFNO(JLON,KLEV)+GWDSE*PGETRL(JLON)&
   & *ZGWDCS5(JLON)**2*ZUSUR5(JLON)*ZSTUS(JLON)  
  ZSTUS(JLON)=0.0_JPRB

  ZRAPP(JLON,KLEV)=0.0_JPRB

  ZNFNO(JLON,KLEV)=ZNFNO(JLON,KLEV)+ZFACS(JLON)/ZSUSR5(JLON)**3
  ZSUSR(JLON)=ZSUSR(JLON)-(3._JPRB*ZNFNO5(JLON,KLEV)/ZSUSR5(JLON)**4)&
   & *ZFACS(JLON)  
  ZFACS(JLON)=0.0_JPRB

  IF ((SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2)) > 0) THEN
    ZUSUR(JLON)=ZUSUR(JLON)+ZUSUR5(JLON)*ZUSR(JLON)&
     & /SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2)  
    ZVSUR(JLON)=ZVSUR(JLON)+ZVSUR5(JLON)*ZUSR(JLON)&
     & /SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2)  
    ZUSR(JLON)=0.0_JPRB
  ELSE
    ZUSR(JLON)=0.0_JPRB
  ENDIF

  Z_YYYY=ZSUMU5(JLON,KTDIA)*ZUSUR5(JLON)+ZSUMV5(JLON,KTDIA)*ZVSUR5(JLON)
  IF (Z_YYYY > ZEPS2) THEN
    ZUSUR(JLON)=ZUSUR(JLON)+ZSUMU5(JLON,KTDIA)*ZSUSR(JLON)
    ZSUMU(JLON)=ZSUMU(JLON)+ZUSUR5(JLON)*ZSUSR(JLON)
    ZVSUR(JLON)=ZVSUR(JLON)+ZSUMV5(JLON,KTDIA)*ZSUSR(JLON)
    ZSUMV(JLON)=ZSUMV(JLON)+ZVSUR5(JLON)*ZSUSR(JLON)
    ZSUSR(JLON)=0.0_JPRB
  ELSE
    ZSUSR(JLON)=0.0_JPRB
  ENDIF

!     COMPUTATION OF A FICTITIOUS WIND THAT WOULD CORRESPOND, IN THE
!     ISOTROPIC CASE, TO THE ANISOTROPIC DRAG.

! - TEMPORARY(S) 1D .

! ZUSUR     : X COMPONENT OF THE FICTITIOUS SURFACE WIND.
! ZVSUR     : Y COMPONENT OF THE FICTITIOUS SURFACE WIND.

  ZA=PVRLAN(JLON)**2+0.5_JPRB*(4._JPRB*(1.0_JPRB-PVRLAN(JLON))*(1.0_JPRB+ZALP1 &
   & *PVRLAN(JLON))+PVRLAN(JLON)*LOG(1.0_JPRB/MAX(ZEPS1,PVRLAN(JLON)))&
   & *(1.0_JPRB+ZALP2*PVRLAN(JLON)))/RPI  
  ZD=0.5_JPRB*(4._JPRB*(1.0_JPRB-PVRLAN(JLON))*(1.0_JPRB+ZDEL1*PVRLAN(JLON))-3._JPRB &
   & *PVRLAN(JLON)*LOG(1.0_JPRB/MAX(ZEPS1,PVRLAN(JLON)))*(1.0_JPRB+ZDEL2 &
   & *PVRLAN(JLON)))/RPI  

  ZSUMV(JLON)=ZSUMV(JLON)+ZA*ZVSUR(JLON)
  ZVSTAR=ZVSTAR+ZD*ZVSUR(JLON)
  ZVSUR(JLON)=0.0_JPRB

  ZSUMU(JLON)=ZSUMU(JLON)+ZA*ZUSUR(JLON)
  ZUSTAR=ZUSTAR+ZD*ZUSUR(JLON)
  ZUSUR(JLON)=0.0_JPRB

  ZSUMU(JLON)=ZSUMU(JLON)+ZVSTAR*SIN(2.0_JPRB*PVRLDI(JLON))
  ZSUMV(JLON)=ZSUMV(JLON)-ZVSTAR*COS(2.0_JPRB*PVRLDI(JLON))

  ZSUMU(JLON)=ZSUMU(JLON)+ZUSTAR*COS(2.0_JPRB*PVRLDI(JLON))
  ZSUMV(JLON)=ZSUMV(JLON)+ZUSTAR*SIN(2.0_JPRB*PVRLDI(JLON))

ENDDO

!     LINEAR COMBINATION.

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZPBLF=0.0_JPRB
    ZPBL=0.0_JPRB
    PV(JLON,JLEV)=PV(JLON,JLEV)+ZV(JLON,JLEV)
    ZSUMV(JLON)=ZSUMV(JLON)+ZPBLF5(JLON,JLEV)*ZV(JLON,JLEV)
    PV(JLON,JLEV)=PV(JLON,JLEV)-ZPBLF5(JLON,JLEV)*ZV(JLON,JLEV)
    ZPBLF=ZPBLF+(ZSUMV5(JLON,KTDIA)-PV5(JLON,JLEV))*ZV(JLON,JLEV)
    ZV(JLON,JLEV)=0.0_JPRB

    PU(JLON,JLEV)=PU(JLON,JLEV)+ZU(JLON,JLEV)
    ZSUMU(JLON)=ZSUMU(JLON)+ZPBLF5(JLON,JLEV)*ZU(JLON,JLEV)
    PU(JLON,JLEV)=PU(JLON,JLEV)-ZPBLF5(JLON,JLEV)*ZU(JLON,JLEV)
    ZPBLF=ZPBLF+(ZSUMU5(JLON,KTDIA)-PU5(JLON,JLEV))*ZU(JLON,JLEV)
    ZU(JLON,JLEV)=0.0_JPRB

    ZZX2=(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,JLEV))/ZOBST5(JLON)
    IF (ZZX2 < 1.0_JPRB) THEN
      PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)-ZPBLF/ZOBST5(JLON)
      PAPRSF(JLON,JLEV)=PAPRSF(JLON,JLEV)+ZPBLF/ZOBST5(JLON)
      ZOBST(JLON)=ZOBST(JLON)+(PAPRS5(JLON,KLEV)&
       & -PAPRSF5(JLON,JLEV))*ZPBLF/ZOBST5(JLON)**2  
    ENDIF

    Z_XXXX=ZNBVNO5(JLON,JLEV)&
     & +ZPBL5(JLON,JLEV)*(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV))  
    IF (Z_XXXX > 0.0_JPRB) THEN
      ZNBVNO(JLON,JLEV)=ZNBVNO(JLON,JLEV)+0.5_JPRB*ZNFNO(JLON,JLEV)&
       & /SQRT(ZNBVNO5(JLON,JLEV)+ZPBL5(JLON,JLEV)&
       & *(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV)))  
      ZPBL=ZPBL+0.5_JPRB*(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV))&
       & *ZNFNO(JLON,JLEV)/SQRT(ZNBVNO5(JLON,JLEV)+ZPBL5(JLON,JLEV)&
       & *(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV)))  
      ZSUMF(JLON)=ZSUMF(JLON)+0.5_JPRB*ZPBL5(JLON,JLEV)*ZNFNO(JLON,&
       & JLEV)&
       & /SQRT(ZNBVNO5(JLON,JLEV)+ZPBL5(JLON,JLEV)&
       & *(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV)))  
      ZNBVNO(JLON,JLEV)=ZNBVNO(JLON,JLEV)-0.5_JPRB*ZPBL5(JLON,JLEV)&
       & *ZNFNO(JLON,JLEV)&
       & /SQRT(ZNBVNO5(JLON,JLEV)+ZPBL5(JLON,JLEV)&
       & *(ZSUMF5(JLON,KTDIA)-ZNBVNO5(JLON,JLEV)))  
      ZNFNO(JLON,JLEV)=0.0_JPRB
    ELSE
      ZNFNO(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZZX1=(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))/ZOBST5(JLON)
    IF (ZZX1 < 1.0_JPRB) THEN
      PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)-ZPBL/ZOBST5(JLON)
      PAPRS(JLON,JLEV)=PAPRS(JLON,JLEV)+ZPBL/ZOBST5(JLON)
      ZOBST(JLON)=ZOBST(JLON)+(PAPRS5(JLON,KLEV)-PAPRS5(JLON,&
       & JLEV))&
       & *ZPBL/ZOBST5(JLON)**2  
    ENDIF

  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZPBLF=0.0_JPRB
  PV(JLON,KLEV)=PV(JLON,KLEV)+ZV(JLON,KLEV)
  ZSUMV(JLON)=ZSUMV(JLON)+ZPBLF5(JLON,KLEV)*ZV(JLON,KLEV)
  PV(JLON,KLEV)=PV(JLON,KLEV)-ZPBLF5(JLON,KLEV)*ZV(JLON,KLEV)
  ZPBLF=ZPBLF+(ZSUMV5(JLON,KTDIA)-PV5(JLON,KLEV))*ZV(JLON,KLEV)
  ZV(JLON,KLEV)=0.0_JPRB

  PU(JLON,KLEV)=PU(JLON,KLEV)+ZU(JLON,KLEV)
  ZSUMU(JLON)=ZSUMU(JLON)+ZPBLF5(JLON,KLEV)*ZU(JLON,KLEV)
  PU(JLON,KLEV)=PU(JLON,KLEV)-ZPBLF5(JLON,KLEV)*ZU(JLON,KLEV)
  ZPBLF=ZPBLF+(ZSUMU5(JLON,KTDIA)-PU5(JLON,KLEV))*ZU(JLON,KLEV)
  ZU(JLON,KLEV)=0.0_JPRB

  ZZX=(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))/ZOBST5(JLON)
  IF (ZZX < 1.0_JPRB) THEN
    PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)-ZPBLF/ZOBST5(JLON)
    PAPRSF(JLON,KLEV)=PAPRSF(JLON,KLEV)+ZPBLF/ZOBST5(JLON)
    ZOBST(JLON)=ZOBST(JLON)+(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))&
     & *ZPBLF/ZOBST5(JLON)**2  
  ENDIF

  IF (ZSUMF5(JLON,KTDIA) > 0) THEN
    ZSUMF(JLON)=ZSUMF(JLON)+ZNFNO(JLON,KLEV)/(2*SQRT(ZSUMF5(JLON,KTDIA)))
    ZNFNO(JLON,KLEV)=0.0_JPRB
  ELSE
    ZNFNO(JLON,KLEV)=0.0_JPRB
  ENDIF
ENDDO

!     WEIGHTED SUMMATION.

DO JLEV=KTDIA,KLEV-1

  DO JLON=KIDIA,KFDIA
    ZWA=0.0_JPRB
    ZNBVNO(JLON,JLEV)=ZNBVNO(JLON,JLEV)+ZWGHT5(JLON,JLEV)*ZSUMF(JLON)
    ZWGHT(JLON,JLEV)=ZWGHT(JLON,JLEV)+ZNBVNO5(JLON,JLEV)*ZSUMF(JLON)
    ZSUMF(JLON)=ZSUMF(JLON)

    ZWGHT(JLON,JLEV)=ZWGHT(JLON,JLEV)+0.5_JPRB*ZSUMV(JLON)&
     & *(PV5(JLON,JLEV)+PV5(JLON,JLEV+1))  
    PV(JLON,JLEV)=PV(JLON,JLEV)+0.5_JPRB*ZWGHT5(JLON,JLEV)*ZSUMV(JLON)
    PV(JLON,JLEV+1)=PV(JLON,JLEV+1)+0.5_JPRB*ZWGHT5(JLON,JLEV)*ZSUMV(JLON)
    ZSUMV(JLON)=ZSUMV(JLON)

    ZWGHT(JLON,JLEV)=ZWGHT(JLON,JLEV)+0.5_JPRB*ZSUMU(JLON)&
     & *(PU5(JLON,JLEV)+PU5(JLON,JLEV+1))  
    PU(JLON,JLEV)=PU(JLON,JLEV)+0.5_JPRB*ZWGHT5(JLON,JLEV)*ZSUMU(JLON)
    PU(JLON,JLEV+1)=PU(JLON,JLEV+1)+0.5_JPRB*ZWGHT5(JLON,JLEV)*ZSUMU(JLON)
    ZSUMU(JLON)=ZSUMU(JLON)

    ZXZX1=(PAPRSF5(JLON,JLEV+1)-ZWA5(JLON,JLEV))/ZOBST5(JLON)
    IF (ZXZX1 > 0.0_JPRB) THEN
      PAPRSF(JLON,JLEV+1)=PAPRSF(JLON,JLEV+1)+ZWGHT(JLON,JLEV)/ZOBST5(JLON)
      ZWA=ZWA-ZWGHT(JLON,JLEV)/ZOBST5(JLON)
      ZOBST(JLON)=ZOBST(JLON)-((PAPRSF5(JLON,JLEV+1)-ZWA5(JLON,&
       & JLEV))&
       & /ZOBST5(JLON)**2)*ZWGHT(JLON,JLEV)  
      ZWGHT(JLON,JLEV)=0.0_JPRB
    ELSE
      ZWGHT(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZXZX=PAPRS5(JLON,KLEV)-ZOBST5(JLON)
    IF (ZXZX > PAPRSF5(JLON,JLEV)) THEN
      PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZWA
      ZOBST(JLON)=ZOBST(JLON)-ZWA
    ELSE
      PAPRSF(JLON,JLEV)=PAPRSF(JLON,JLEV)+ZWA
    ENDIF
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZVA=0.0_JPRB
  ZVB=0.0_JPRB
  ZVC=0.0_JPRB
  ZNBVNO(JLON,KLEV)=ZNBVNO(JLON,KLEV)+ZWGHT5(JLON,KLEV)*ZSUMF(JLON)
  ZWGHT(JLON,KLEV)=ZWGHT(JLON,KLEV)+ZNBVNO5(JLON,KLEV)*ZSUMF(JLON)
  ZSUMF(JLON)=0.0_JPRB

  PV(JLON,KLEV)=PV(JLON,KLEV)+ZWGHT5(JLON,KLEV)*ZSUMV(JLON)
  ZWGHT(JLON,KLEV)=ZWGHT(JLON,KLEV)+PV5(JLON,KLEV)*ZSUMV(JLON)
  ZSUMV(JLON)=0.0_JPRB

  PU(JLON,KLEV)=PU(JLON,KLEV)+ZWGHT5(JLON,KLEV)*ZSUMU(JLON)
  ZWGHT(JLON,KLEV)=ZWGHT(JLON,KLEV)+PU5(JLON,KLEV)*ZSUMU(JLON)
  ZSUMU(JLON)=0.0_JPRB

  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZWGHT(JLON,KLEV)/ZOBST5(JLON)
  PAPRSF(JLON,KLEV)=PAPRSF(JLON,KLEV)-ZWGHT(JLON,KLEV)/ZOBST5(JLON)
  ZOBST(JLON)=ZOBST(JLON)-((PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))&
   & /ZOBST5(JLON)**2)*ZWGHT(JLON,KLEV)  
  ZWGHT(JLON,KLEV)=0.0_JPRB

  ZXX=PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KTDIA)
  IF (ZXX < ZVC5(JLON)) THEN
    PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZOBST(JLON)
    PAPRSF(JLON,KTDIA)=PAPRSF(JLON,KTDIA)-ZOBST(JLON)
    ZOBST(JLON)=0.0_JPRB
  ELSE
    ZVC=ZVC+ZOBST(JLON)
    ZOBST(JLON)=0.0_JPRB
  ENDIF

  IF (ZVA5(JLON) > ZVB5(JLON)) THEN
    ZVA=ZVA+ZVC
  ELSE
    ZVB=ZVB+ZVC
  ENDIF

  ZGWDCS(JLON)=ZGWDCS(JLON)+HOBST*PGETRL(JLON)*ZVB

  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZVA
  PAPRSF(JLON,KLEV)=PAPRSF(JLON,KLEV)-ZVA
ENDDO

!*
!     ------------------------------------------------------------------
!     IV - PRELIMINARY COMPUTATIONS FOR THE LAYER'S PRESSURE THICKNESS
!     DIVIDED BY G*DT AND ITS INVERSE FOR ALL LEVELS AS WELL AS SETTING
!     TO ZERO THE WEIGHTS FOR THE INTEGRAL OVER THE DEPTH OF THE
!     OROGRAPHIC OBSTACLE.

! - TEMPORARY(S) 2D (1:KLEV) .

! ZPOID     : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI     : INVERSE OF ZPOID.
! ZWGHT     : WEIGHTS FOR THE INTEGRAL OVER THE DEPTH OF THE OBSTACLE.

DO JLEV=KLEV,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZWGHT(JLON,JLEV)=0.0_JPRB

    PRDELP(JLON,JLEV)=PRDELP(JLON,JLEV)+ZIPOI(JLON,JLEV)*ZGDT
    ZIPOI(JLON,JLEV)=0.0_JPRB

    PDELP(JLON,JLEV)=PDELP(JLON,JLEV)+ZPOID(JLON,JLEV)*ZGDTI
    ZPOID(JLON,JLEV)=0.0_JPRB
  ENDDO
ENDDO

!*
!     -----------------------------------------------------------------
!     III - COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY
!     DIVIDED BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY
!     FOR G.W.D.

! - TEMPORARY(S) 2D (0:KLEV) .

! ZNBVNO    : SQUARE OF BRUNT-VAISALA FR. DIVIDED BY G AND DENSITY

! - TEMPORARY 1D

! ZGWDCS    : DENSITY AT THE SURFACE FOR OROGRAPHIC DRAG.
! ZRS       : GASSCONSTANT FOR AIRAT THE SURFACE

ZRVMD=RV-RD

DO JLON=KIDIA,KFDIA
  ZRTI=0.0_JPRB
  ZSTA=0.0_JPRB
  ZDPHI=0.0_JPRB
  ZCIS=0.0_JPRB

!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY (FOR G.W.D.)

  ZRTI=ZRTI+PAPRS5(JLON,KLEV)*ZGWDCS(JLON)
  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZRTI5(JLON,KLEV)*ZGWDCS(JLON)
  ZGWDCS(JLON)=0.0_JPRB

  ZSTA=ZSTA+ZNBVNO(JLON,KLEV)&
   & /(PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)*ZDPHI5(JLON,KLEV))**2  
  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)-2.0_JPRB*ZSTA5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZDPHI5(JLON,KLEV)*ZNBVNO(JLON,KLEV)&
   & /(PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)*ZDPHI5(JLON,KLEV))**3  
  ZRTI=ZRTI-2.0_JPRB*ZSTA5(JLON,KLEV)*PAPRS5(JLON,KLEV)&
   & *ZDPHI5(JLON,KLEV)*ZNBVNO(JLON,KLEV)&
   & /(PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)*ZDPHI5(JLON,KLEV))**3  
  ZDPHI=ZDPHI-2.0_JPRB*ZSTA5(JLON,KLEV)*PAPRS5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZNBVNO(JLON,KLEV)&
   & /(PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)*ZDPHI5(JLON,KLEV))**3  
  ZNBVNO(JLON,KLEV)=0.0_JPRB

!     STABILITY COMPUTATION.

  ZRS(JLON)=RD+ZRVMD*PQS(JLON)

  ZDPHI=ZDPHI+(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+2*RKAPPA &
   & *ZDPHI5(JLON,KLEV)-ZRS(JLON)*PTS(JLON))*ZRTI5(JLON,KLEV)&
   & *ZSTA  
  PT(JLON,KLEV)=PT(JLON,KLEV)+ZDPHI5(JLON,KLEV)*ZR5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZSTA  
  ZR(JLON,KLEV)=ZR(JLON,KLEV)+ZDPHI5(JLON,KLEV)*PT5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZSTA  
  ZRTI=ZRTI+(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA &
   & *ZDPHI5(JLON,KLEV)-ZRS(JLON)*PTS(JLON))*ZDPHI5(JLON,KLEV)&
   & *ZSTA  

  PT(JLON,KLEV)=PT(JLON,KLEV)-2.0_JPRB*ZR5(JLON,KLEV)*ZRTI &
   & /(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA*ZDPHI5(JLON,KLEV)&
   & +ZRS(JLON)*PTS(JLON))**2  
  ZR(JLON,KLEV)=ZR(JLON,KLEV)-2.0_JPRB*PT5(JLON,KLEV)*ZRTI &
   & /(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA*ZDPHI5(JLON,KLEV)&
   & +ZRS(JLON)*PTS(JLON))**2  
  ZDPHI=ZDPHI-2.0_JPRB*RKAPPA*ZRTI &
   & /(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA*ZDPHI5(JLON,KLEV)&
   & +ZRS(JLON)*PTS(JLON))**2  

  IF ((PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2) > ZEPS2) THEN
    PU(JLON,KLEV)=PU(JLON,KLEV)+2.0_JPRB*PU5(JLON,KLEV)*ZCIS
    PV(JLON,KLEV)=PV(JLON,KLEV)+2.0_JPRB*PV5(JLON,KLEV)*ZCIS
  ENDIF

  PAPHIF(JLON,KLEV)=PAPHIF(JLON,KLEV)+ZDPHI
  PAPHI(JLON,KLEV)=PAPHI(JLON,KLEV)-ZDPHI
ENDDO

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZSTA=0.0_JPRB
    ZRTI=0.0_JPRB
    ZCIS=0.0_JPRB
    ZDPHI=0.0_JPRB
    ZDTETA=0.0_JPRB
    ZAUX=0.0_JPRB
    ZZB=0.0_JPRB
    ZZD=0.0_JPRB
    ZZC=0.0_JPRB

!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED.

    ZSTA=ZSTA+ZNBVNO(JLON,JLEV)&
     & /(PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZDPHI5(JLON,JLEV))**2  
    PAPRS(JLON,JLEV)=PAPRS(JLON,JLEV)-2.0_JPRB*ZSTA5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)*ZDPHI5(JLON,JLEV)*ZNBVNO(JLON,JLEV)&
     & /(PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZDPHI5(JLON,JLEV))**3  
    ZRTI=ZRTI-2.0_JPRB*ZSTA5(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZDPHI5(JLON,JLEV)*ZNBVNO(JLON,JLEV)&
     & /(PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZDPHI5(JLON,JLEV))**3  
    ZDPHI=ZDPHI-2.0_JPRB*ZSTA5(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)*ZNBVNO(JLON,JLEV)&
     & /(PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZDPHI5(JLON,JLEV))**3  
    ZNBVNO(JLON,JLEV)=0.0_JPRB

!     STABILITY COMPUTATION.

    ZDPHI=ZDPHI+(ZDTETA5(JLON,JLEV)+RD*ZAUX5(JLON,JLEV))&
     & *ZRTI5(JLON,JLEV)*ZSTA  
    ZDTETA=ZDTETA+ZDPHI5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZSTA
    ZAUX=ZAUX+RD*ZDPHI5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZSTA
    ZRTI=ZRTI+(ZDTETA5(JLON,JLEV)+RD*ZAUX5(JLON,JLEV))*ZDPHI5(JLON,JLEV)*ZSTA

    PT(JLON,JLEV)=PT(JLON,JLEV)-2.0_JPRB*ZR5(JLON,JLEV)*ZRTI &
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    ZR(JLON,JLEV)=ZR(JLON,JLEV)-2.0_JPRB*PT5(JLON,JLEV)*ZRTI &
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    PT(JLON,JLEV+1)=PT(JLON,JLEV+1)-2.0_JPRB*ZR5(JLON,JLEV+1)*ZRTI &
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    ZR(JLON,JLEV+1)=ZR(JLON,JLEV+1)-2.0_JPRB*PT5(JLON,JLEV+1)*ZRTI &
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  

!     CORRECTION FOR SHALLOW CONVECTION (OPTIONAL).

    IF (LCVPP) THEN
      ZZB=ZZB+ZZD5(JLON,JLEV)*0.5_JPRB &
       & *(ZLSCPE5(JLON,JLEV)+ZLSCPE5(JLON,JLEV+1))*ZAUX  
      ZLSCPE(JLON,JLEV)=ZLSCPE(JLON,JLEV)+ZZD5(JLON,JLEV)*0.5_JPRB &
       & *ZZB5(JLON,JLEV)*ZAUX  
      ZLSCPE(JLON,JLEV+1)=ZLSCPE(JLON,JLEV+1)+ZZD5(JLON,JLEV)*0.5_JPRB &
       & *ZZB5(JLON,JLEV)*ZAUX  
      ZZD=ZZD+0.5_JPRB*ZZB5(JLON,JLEV)&
       & *(ZLSCPE5(JLON,JLEV)+ZLSCPE5(JLON,JLEV+1))*ZAUX  

      IF (ZZC5(JLON,JLEV) < 0.0_JPRB) THEN
        ZZC=ZZC+ZZD
      ENDIF

      PQ(JLON,JLEV)=PQ(JLON,JLEV)+ZZC
      ZQSAT(JLON,JLEV)=ZQSAT(JLON,JLEV)-ZZC
      PQ(JLON,JLEV+1)=PQ(JLON,JLEV+1)-ZZC
      ZQSAT(JLON,JLEV+1)=ZQSAT(JLON,JLEV+1)+ZZC

      ZZB=0.0_JPRB
    ELSE
      ZAUX=0.0_JPRB
    ENDIF

!     PRELIMINARY STABILITY COMPUTATION.

    PT(JLON,JLEV)=PT(JLON,JLEV)+ZR5(JLON,JLEV)*ZDTETA
    ZR(JLON,JLEV)=ZR(JLON,JLEV)+PT5(JLON,JLEV)*ZDTETA
    PT(JLON,JLEV+1)=PT(JLON,JLEV+1)-ZR5(JLON,JLEV+1)*ZDTETA
    ZR(JLON,JLEV+1)=ZR(JLON,JLEV+1)-PT5(JLON,JLEV+1)*ZDTETA
    ZDPHI=ZDPHI+RKAPPA*ZDTETA

    ZXX=(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))**2 &
     & +(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))**2  
    IF (ZXX > ZEPS2) THEN
      PU(JLON,JLEV)=PU(JLON,JLEV)&
       & +2.0_JPRB*(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))*ZCIS  
      PU(JLON,JLEV+1)=PU(JLON,JLEV+1)&
       & -2.0_JPRB*(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))*ZCIS  
      PV(JLON,JLEV)=PV(JLON,JLEV)&
       & +2.0_JPRB*(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))*ZCIS  
      PV(JLON,JLEV+1)=PV(JLON,JLEV+1)&
       & -2.0_JPRB*(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))*ZCIS  
    ENDIF

    PAPHIF(JLON,JLEV)=PAPHIF(JLON,JLEV)+ZDPHI
    PAPHIF(JLON,JLEV+1)=PAPHIF(JLON,JLEV+1)-ZDPHI

  ENDDO
ENDDO

!*
!     ----------------------------------------------------------------
!     II - COMPUTATION OF SPECIFIC HUMIDITY OF SATURATION AND
!     EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION,
!     COMPUTATION OF GASS CONSTANT FOR AIR

! - TEMPORARY(S) 2D (1:KLEV) .

! ZLH       : LATENT HEAT
! ZQSAT     : SPECIFIC HUMIDITY OF SATURATION
! ZLSCPE    : EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
! ZR        : GASS CONSTANT FOR AIR

! - TEMPORARY 1D

! ZCP       : VALUE OF ATMOSPHERIC Cp

DO JLEV=KLEV,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZESP=0.0_JPRB
    ZEW=0.0_JPRB

!     COMPUTATION OF Qsat(PT)

    IF ((1.0_JPRB-ZESP5(JLON,JLEV)) > 0.0_JPRB) THEN
      ZESP=ZESP+FOQSAD1(ZQSAT(JLON,JLEV),ZESP5(JLON,JLEV))
    ELSE
      ZESP=ZESP+FOQSAD2(ZQSAT(JLON,JLEV),ZESP5(JLON,JLEV))
    ENDIF
    ZQSAT(JLON,JLEV)=0.0_JPRB

    ZEW=ZEW+ZESP/PAPRSF5(JLON,JLEV)
    PAPRSF(JLON,JLEV)=PAPRSF(JLON,JLEV)&
     & -(ZEW5(JLON,JLEV)/PAPRSF5(JLON,JLEV)**2)*ZESP  

!     SNOW OPTION DEPENDENT COMPUTATIONS.

    IF (LNEIGE) THEN
      IF ((RTT-PT5(JLON,JLEV)) < 0.0_JPRB) THEN
        ZDELTA=0.0_JPRB
      ELSE
        ZDELTA=1.0_JPRB
      ENDIF
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF

    PT(JLON,JLEV)=PT(JLON,JLEV)+FOEWAD(ZEW,PT5(JLON,JLEV),ZDELTA)

!     COMPUTATION OF EFFECTIVE RATIO OF L AND Cp FOR COND./EVAP.

    ZLH(JLON,JLEV)=ZLH(JLON,JLEV)+ZLSCPE(JLON,JLEV)/ZCP5(JLON,JLEV)
    ZCP(JLON)=ZCP(JLON)-(ZLH5(JLON,JLEV)/ZCP5(JLON,JLEV)**2)&
     & *ZLSCPE(JLON,JLEV)  
    ZLSCPE(JLON,JLEV)=0.0_JPRB

!     CALL TO THE DEFINED FUNCTION TO COMPUTE LH.

    PT(JLON,JLEV)=PT(JLON,JLEV)+FOLHAD(ZLH(JLON,JLEV),ZDELTA)
    ZLH(JLON,JLEV)=0.0_JPRB

    PQ(JLON,JLEV)=PQ(JLON,JLEV)-RCPD*ZCP(JLON)
    PQ(JLON,JLEV)=PQ(JLON,JLEV)+RCPV*ZCP(JLON)
    ZCP(JLON)=0.0_JPRB

    PQ(JLON,JLEV)=PQ(JLON,JLEV)-RD*ZR(JLON,JLEV)
    PQ(JLON,JLEV)=PQ(JLON,JLEV)+RV*ZR(JLON,JLEV)
    ZR(JLON,JLEV)=0.0_JPRB
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('ACDRAGLAD',1,ZHOOK_HANDLE)
END SUBROUTINE ACDRAGLAD
