!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACDRAGLTL (KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI5,PAPHIF5,PAPRS5,PAPRSF5,&
 & PDELP5,PRDELP5,&
 & PU5,PV5,PT5,PQ5,&
 & PAPHI,PAPHIF,PAPRS,PAPRSF,PDELP,PRDELP,&
 & PU,PV,PT,PQ,&
 ! - INPUT  1D .
 & PGETRL,PVRLAN,PVRLDI,PTS,PQS,&
 ! - OUTPUT 2D .
 & PSTRDU5,PSTRDV5,&
 & PSTRDU,PSTRDV,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACDRAGLTL * - TANGENT LINEAR OF SIMPLIFIED EFFECT OF OROGRAPHIC
!                    GRAVITY-WAVES.

!     Subject.
!     --------
!     Tangent linear of:
!     - COMPUTATION OF OROGRAPHIC GRAVITY-WAVES' BREAKING EFFECTS ON
!       MOMENTUM - LINEAR PART OF COMPUTATION

!**   Interface.
!     ----------
!        *CALL* *ACDRAGL*

! -   ARGUMENTS FOR INPUT
!     -------------------

! - DIMENSIONAL PARAMETERS OF PHYSICS

! KIDIA      : START OF HORIZONTAL LOOP (IST in CPG).
! KFDIA      : END OF HORIZONTAL LOOP (IEND in CPG).
! KLON       : HORIZONTAL DIMENSIONAL.
! KTDIA      : START OF THE VERTICAL LOOP IN THE PHYSIC (1 IN GENERAL).
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION "FULL LEVEL".
!              (NFLEVG in CPG).

! - THE NAME OF THE PHYSICAL VARIABLES

! - 2D (0:KLEV) .

! PAPRS      : PRESSURE ON HALF LEVELS.
! PAPHI      : GEOPOTENTIAL ON HALF LEVELS.

! The same but for trajectory
! ---------------------------
! PAPRS5     : PRESSURE ON HALF LEVELS.
! PAPHI5     : GEOPOTENTIAL ON HALF LEVELS.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF     : PRESSURE ON FULL LEVELS.
! PDELP      : LAYER THICKNESS IN PRESSURE UNITS.
! PRDELP     : INVERSE OF LAYER THICKNESS IN PRESSURE UNITS.
! PU         : X COMPONENT OF WIND.
! PV         : Y COMPONENT OF WIND.
! PT         : TEMPERATURE.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOUR.

! The same but for trajectory
! ---------------------------
! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF5    : PRESSURE ON FULL LEVELS.
! PDELP5     : LAYER THICKNESS IN PRESSURE UNITS.
! PRDELP5    : INVERSE OF LAYER THICKNESS IN PRESSURE UNITS.
! PU5        : X COMPONENT OF WIND.
! PV5        : Y COMPONENT OF WIND.
! PT5        : TEMPERATURE.
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOUR.

! - 1D (GEOGRAPHIC) .

! PGETRL     : STANDARD DEVIATION OF OROGRAPHY (unit J/KG).
! PVRLAN     : ANISOTROPY OF THE RELIEF (1 = ISOTROPY).
! PVRLDI     : ANGLE OF MAXIMAL GRADIENT OF RELIEF WITH X-AXIS.

! - 1D (PROGNOSTIC) .

! PTS        : SURFACE TEMPERATURE.
! PQS        : SURFACE SPECIFIC HUMIDITY.

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR OUTPUT.
!     ----------------------

! - 2D (0:KLEV) .

! PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".

! The same but for trajectory
! ---------------------------
! PSTRDU5    : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDV5    : FLUX "GRAVITY WAVE DRAG" "V".

!-----------------------------------------------------------------------

! -   IMPLICIT ARGUMENTS.
!     ---------------------

! COMMON/YOMCST /
! COMMON/YOMPHY/
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/
! COMMON/FCTTRM/
! COMMON/FCTTRMTL/

!-----------------------------------------------------------------------

!     Externals.
!     ----------

!     Method.
!     -------

!     Author.
!     -------
!          95-10, M.Janiskova

!     Modifications.
!     --------------
!        96-05, GWDBC,GWDCD according to ACDRAG modif. - M.Janiskova
!        97-08, introduce CDLOCK - J.M. Piriou.
!        99-10, suppression of USURIC - F. Bouyssel.
!        99-12, form drag set to zero (if LSMOOTHB=.T.) :  - F. Bouyssel.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI      ,RG       ,RD       ,RV       ,&
 & RCPD     ,RCPV     ,RKAPPA   ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMPHY   , ONLY : LCVPP    ,LNEIGE
USE YOMPHY0  , ONLY : GWDBC    ,GWDCD    ,GWDSE    ,HOBST
USE YOMPHY2  , ONLY : TSPHY
USE YOMSIMPHL, ONLY : LSMOOTHB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGETRL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRLAN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRLDI(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDU5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDV(KLON,0:KLEV) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
REAL(KIND=JPRB) :: ZNBVNO(KLON,0:KLEV),ZRAPP(KLON,0:KLEV),ZNFNO(KLON,0:KLEV)&
 & ,ZWGHT(KLON,0:KLEV),ZRAPPH(KLON,0:KLEV)&
 & ,ZLSCPE(KLON,KLEV),ZQSAT(KLON,KLEV),ZR(KLON,KLEV)&
 & ,ZLH(KLON,KLEV),ZIPOI(KLON,KLEV),ZPOID(KLON,KLEV)&
 & ,ZU(KLON,KLEV),ZV(KLON,KLEV)  
REAL(KIND=JPRB) :: ZGWDCS(KLON),ZCP(KLON),ZFACS(KLON)&
 & ,ZOBST(KLON),ZRS(KLON),ZSTUS(KLON),ZSTVS(KLON)&
 & ,ZSUMD(KLON),ZSUMF(KLON),ZSUMU(KLON),ZSUMV(KLON)&
 & ,ZSUSR(KLON),ZUSR(KLON),ZUSUR(KLON),ZVSUR(KLON)&
 & ,ZAA(KLON),ZBB(KLON)  

REAL(KIND=JPRB) :: ZNBVNO5(KLON,0:KLEV),ZRAPP5(KLON,0:KLEV)&
 & ,ZWGHT5(KLON,0:KLEV),ZNFNO5(KLON,0:KLEV)&
 & ,ZRAPPH5(KLON,0:KLEV)&
 & ,ZLSCPE5(KLON,KLEV),ZQSAT5(KLON,KLEV),ZR5(KLON,KLEV)&
 & ,ZLH5(KLON,KLEV),ZIPOI5(KLON,KLEV),ZPOID5(KLON,KLEV)&
 & ,ZU5(KLON,KLEV),ZV5(KLON,KLEV)  
REAL(KIND=JPRB) :: ZGWDCS5(KLON),ZCP5(KLON),ZFACS5(KLON),ZOBST5(KLON)&
 & ,ZSTUS5(KLON),ZSTVS5(KLON),ZSUMD5(KLON),ZSUMF5(KLON)&
 & ,ZSUMU5(KLON),ZSUMV5(KLON),ZSUSR5(KLON)&
 & ,ZUSR5(KLON),ZUSUR5(KLON),ZVSUR5(KLON)&
 & ,ZAA5(KLON),ZBB5(KLON)  

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: Z_XXXX, Z_YYYY, ZA, ZALP1, ZALP2, ZALPHA, ZALPHA5,&
 & ZAUSR, ZAUSR5, ZAUX, ZAUX5, ZBB1, ZBB15, &
 & ZCIS, ZCIS5, ZD, ZDEL1, ZDEL2, ZDELTA, ZDPHI, &
 & ZDPHI5, ZDTETA, ZDTETA5, ZDU, ZDU5, ZDV, &
 & ZDV5, ZDX, ZDX5, ZEPS1, ZEPS2, ZEPS3, ZEPS5, &
 & ZEPS6, ZESP, ZESP5, ZEW, ZEW5, ZGDT, ZGDTI, &
 & ZPBL, ZPBL5, ZPBLF, ZPBLF5, ZPRRAP, ZPRRAP5, &
 & ZRTI, ZRTI5, ZRVMD, ZRZ, ZRZ1, ZRZ15, ZRZ5, &
 & ZSTA, ZSTA5, ZTUNE, ZTUNE5, ZUSTAR, ZUSTAR5, &
 & ZVA, ZVA5, ZVB, ZVB5, ZVC, ZVC5, ZVSTAR, &
 & ZVSTAR5, ZWA, ZWA5, ZX, ZX5, ZXX, ZXZX, ZZB, &
 & ZZB5, ZZC, ZZC5, ZZD, ZZD5, ZZP, ZZP5, ZZPA, &
 & ZZPA5, ZZX, ZZX1, ZZX2, ZZZB  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcttrm.h"
#include "fcttrmtl.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACDRAGLTL',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'DRAGLTL') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!     I - COMPUTATION OF DERIVED PARAMETERS, SECURITY CONSTANTS (FOR THE
!     ANISOTROPY FACTOR, THE SQUARE OF THE WIND SPEED, THE NORMALISED
!     STATIC STABILITY AND A PRECISION THRESHOLD FOR RECOMPUTED
!     PRESSURES) AS WELL AS LIMIT ARGUMENT FOR THE SINE FUNCTION.

ZGDT=RG*TSPHY
ZGDTI=1.0_JPRB/ZGDT

!     THE 'ALP' AND 'DEL' VALUES ARE RESULTING FROM ANALYTICAL FITS OF
!     ELLIPTIC INTEGRALS FOR THE ANISOTROPIC CASE.

ZALP1=0.75_JPRB+0.25_JPRB*LOG(16._JPRB)
ZALP2=3.5_JPRB*RPI-LOG(16._JPRB)-8._JPRB
ZDEL1=2.75_JPRB-0.75_JPRB*LOG(16._JPRB)
ZDEL2=-0.25_JPRB*RPI-LOG(16._JPRB)+4._JPRB

ZEPS1=1.E-06_JPRB
ZEPS2=1.E-04_JPRB
ZEPS3=1.E-09_JPRB
ZEPS5=1.E-06_JPRB
ZEPS6=1.E-08_JPRB

!*
!     ----------------------------------------------------------------
!     II - COMPUTATION OF SPECIFIC HUMIDITY OF SATURATION AND
!     EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION,
!     COMPUTATION OF GASS CONSTANT FOR AIR

! - TEMPORARY(S) 2D (1:KLEV) .

! ZLH       : LATENT HEAT
! ZQSAT     : SPECIFIC HUMIDITY OF SATURATION
! ZLSCPE    : EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
! ZR        : GASS CONSTANT FOR AIR

! - TEMPORARY 1D

! ZCP       : VALUE OF ATMOSPHERIC Cp

! The same but for trajectory
! ---------------------------
! - TEMPORARY(S) 2D (1:KLEV) .

! ZLH5      : LATENT HEAT
! ZQSAT5    : SPECIFIC HUMIDITY OF SATURATION
! ZLSCPE5   : EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
! ZR5       : GASS CONSTANT FOR AIR

! - TEMPORARY 1D

! ZCP5      : VALUE OF ATMOSPHERIC Cp

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA

    ZR5(JLON,JLEV)=RD*(1.0_JPRB-PQ5(JLON,JLEV))+RV*PQ5(JLON,JLEV)
    ZR(JLON,JLEV)=RD*(-PQ(JLON,JLEV))+RV*PQ(JLON,JLEV)

    ZCP5(JLON)=RCPD*(1.0_JPRB-PQ5(JLON,JLEV))+RCPV*PQ5(JLON,JLEV)
    ZCP(JLON)=RCPD*(-PQ(JLON,JLEV))+RCPV*PQ(JLON,JLEV)

!     SNOW OPTION DEPENDENT COMPUTATIONS.

    IF (LNEIGE) THEN
      IF ((RTT-PT5(JLON,JLEV)) < 0.0_JPRB) THEN
        ZDELTA=0.0_JPRB
      ELSE
        ZDELTA=1.0_JPRB
      ENDIF
!         ZDELTA=MAX(0.,SIGN(1.,RTT-PT5(JLON,JLEV)))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF

!     CALL TO THE DEFINED FUNCTION TO COMPUTE LH.

    ZLH5(JLON,JLEV)= FOLH (PT5(JLON,JLEV),ZDELTA)

    ZLH(JLON,JLEV)= FOLHTL (PT(JLON,JLEV),ZDELTA)

!     COMPUTATION OF EFFECTIVE RATIO OF L AND Cp FOR COND./EVAP.

    ZLSCPE5(JLON,JLEV)=ZLH5(JLON,JLEV)/ZCP5(JLON)
    ZLSCPE(JLON,JLEV)=ZLH(JLON,JLEV)/ZCP5(JLON)&
     & -(ZLH5(JLON,JLEV)/ZCP5(JLON)**2)*ZCP(JLON)  

!     COMPUTATION OF Qsat(PT)

    ZEW5= FOEW (PT5(JLON,JLEV),ZDELTA)
    ZEW= FOEWTL (PT(JLON,JLEV),PT5(JLON,JLEV),ZDELTA)

    ZESP5=ZEW5/PAPRSF5(JLON,JLEV)
    ZESP=ZEW/PAPRSF5(JLON,JLEV)&
     & -(ZEW5/PAPRSF5(JLON,JLEV)**2)*PAPRSF(JLON,JLEV)  

    ZQSAT5(JLON,JLEV)= FOQS (ZESP5)
    IF ((1.0_JPRB-ZESP5) > 0.0_JPRB) THEN
      ZQSAT(JLON,JLEV)= FOQSTL1 (ZESP,ZESP5)
    ELSE
      ZQSAT(JLON,JLEV)= FOQSTL2 (ZESP,ZESP5)
    ENDIF

  ENDDO
ENDDO

!*
!     -----------------------------------------------------------------
!     III - COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY
!     DIVIDED BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY
!     FOR G.W.D.

! - TEMPORARY(S) 2D (0:KLEV) .

! ZNBVNO    : SQUARE OF BRUNT-VAISALA FR. DIVIDED BY G AND DENSITY

! - TEMPORARY 1D

! ZGWDCS    : DENSITY AT THE SURFACE FOR OROGRAPHIC DRAG.
! ZRS       : GASSCONSTANT FOR AIRAT THE SURFACE

! The same but for trajectory
! ---------------------------
! - TEMPORARY(S) 2D (0:KLEV) .

! ZNBVNO5   : SQUARE OF BRUNT-VAISALA FR. DIVIDED BY G AND DENSITY

! - TEMPORARY 1D

! ZGWDCS5   : DENSITY AT THE SURFACE FOR OROGRAPHIC DRAG.

ZRVMD=RV-RD

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA

    ZDPHI5=PAPHIF5(JLON,JLEV)-PAPHIF5(JLON,JLEV+1)
    ZDPHI=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)

    ZCIS5=MAX(ZEPS2,(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))**2 &
     & +(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))**2)  
    ZXX=(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))**2 &
     & +(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))**2  
    IF (ZXX > ZEPS2) THEN
      ZCIS=2.0_JPRB*(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))*PU(JLON,JLEV)&
       & -2.0_JPRB*(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))*PU(JLON,JLEV+1)&
       & +2.0_JPRB*(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))*PV(JLON,JLEV)&
       & -2.0_JPRB*(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))*PV(JLON,JLEV+1)  
    ELSE
      ZCIS=0.0_JPRB
    ENDIF

!     PRELIMINARY STABILITY COMPUTATION.

    ZDTETA5=ZR5(JLON,JLEV)*PT5(JLON,JLEV)&
     & -ZR5(JLON,JLEV+1)*PT5(JLON,JLEV+1)&
     & +RKAPPA*ZDPHI5  
    ZDTETA=ZR5(JLON,JLEV)*PT(JLON,JLEV)&
     & +PT5(JLON,JLEV)*ZR(JLON,JLEV)&
     & -ZR5(JLON,JLEV+1)*PT(JLON,JLEV+1)&
     & -PT5(JLON,JLEV+1)*ZR(JLON,JLEV+1)&
     & +RKAPPA*ZDPHI  

!     CORRECTION FOR SHALLOW CONVECTION (OPTIONAL).

    IF (LCVPP) THEN
      ZZB5=SIGN(1.0_JPRB,RD &
       & *(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV)+(ZQSAT5(JLON,JLEV)&
       & -ZQSAT5(JLON,JLEV+1))*PAPRS5(JLON,JLEV)&
       & /PAPRS5(JLON,KLEV))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))-ZDTETA5)  
      ZZB=0.0_JPRB

      ZZC5=PQ5(JLON,JLEV)-ZQSAT5(JLON,JLEV)-PQ5(JLON,JLEV+1)&
       & +ZQSAT5(JLON,JLEV+1)  
      ZZC=PQ(JLON,JLEV)-ZQSAT(JLON,JLEV)-PQ(JLON,JLEV+1)+ZQSAT(JLON,JLEV+1)
      IF (ZZC5 < 0) THEN
        ZZD5=ZZC5
        ZZD=ZZC
      ELSE
        ZZD5=0.0_JPRB
        ZZD=0.0_JPRB
      ENDIF

      ZAUX5=MIN(0.0_JPRB,PQ5(JLON,JLEV)-ZQSAT5(JLON,JLEV)&
       & -PQ5(JLON,JLEV+1)&
       & +ZQSAT5(JLON,JLEV+1))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))*MAX(0.0_JPRB,SIGN(1.0_JPRB,RD &
       & *(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV)+(ZQSAT5(JLON,JLEV)&
       & -ZQSAT5(JLON,JLEV+1))*PAPRS5(JLON,JLEV)&
       & /PAPRS5(JLON,KLEV))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))-ZDTETA5))  

      ZAUX=ZZD5*0.5_JPRB*(ZLSCPE5(JLON,JLEV)+ZLSCPE5(JLON,JLEV+1))*ZZB &
       & +ZZD5*0.5_JPRB*(ZLSCPE(JLON,JLEV)+ZLSCPE(JLON,JLEV+1))*ZZB5 &
       & +ZZD*0.5_JPRB*(ZLSCPE5(JLON,JLEV)+ZLSCPE5(JLON,JLEV+1))*ZZB5  
    ELSE
      ZAUX5=0.0_JPRB
      ZAUX=0.0_JPRB
    ENDIF

!     STABILITY COMPUTATION.

    ZRTI5=2.0_JPRB/(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))  
    ZRTI=(-2.0_JPRB*ZR5(JLON,JLEV)*PT(JLON,JLEV)&
     & -2.0_JPRB*PT5(JLON,JLEV)*ZR(JLON,JLEV)&
     & -2.0_JPRB*ZR5(JLON,JLEV+1)*PT(JLON,JLEV+1)&
     & -2.0_JPRB*PT5(JLON,JLEV+1)*ZR(JLON,JLEV+1))&
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  

    ZSTA5=ZDPHI5*(ZDTETA5+RD*ZAUX5)*ZRTI5
    ZSTA=(ZDTETA5+RD*ZAUX5)*ZRTI5*ZDPHI &
     & +ZDPHI5*ZRTI5*ZDTETA &
     & +RD*ZDPHI5*ZRTI5*ZAUX &
     & +(ZDTETA5+RD*ZAUX5)*ZDPHI5*ZRTI  

!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED.

    ZNBVNO5(JLON,JLEV)=ZSTA5/(PAPRS5(JLON,JLEV)*ZRTI5*ZDPHI5)**2
    ZNBVNO(JLON,JLEV)=ZSTA/(PAPRS5(JLON,JLEV)*ZRTI5*ZDPHI5)**2 &
     & -2.0_JPRB*ZSTA5*ZRTI5*ZDPHI5*PAPRS(JLON,JLEV)&
     & /(PAPRS5(JLON,JLEV)*ZRTI5*ZDPHI5)**3 &
     & -2.0_JPRB*ZSTA5*PAPRS5(JLON,JLEV)*ZDPHI5*ZRTI &
     & /(PAPRS5(JLON,JLEV)*ZRTI5*ZDPHI5)**3 &
     & -2.0_JPRB*ZSTA5*PAPRS5(JLON,JLEV)*ZRTI5*ZDPHI &
     & /(PAPRS5(JLON,JLEV)*ZRTI5*ZDPHI5)**3  

  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA

  ZDPHI5=PAPHIF5(JLON,KLEV)-PAPHI5(JLON,KLEV)
  ZDPHI=PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)

  ZRS(JLON)=RD+ZRVMD*PQS(JLON)
  ZCIS5=MAX(ZEPS2,PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2)
  IF ((PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2) > ZEPS2) THEN
    ZCIS=2.0_JPRB*PU5(JLON,KLEV)*PU(JLON,KLEV)&
     & +2.0_JPRB*PV5(JLON,KLEV)*PV(JLON,KLEV)  
  ELSE
    ZCIS=0.0_JPRB
  ENDIF

!     STABILITY COMPUTATION.

  ZRTI5=2.0_JPRB/(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA*ZDPHI5 &
   & +ZRS(JLON)*PTS(JLON))  
  ZRTI=(-2.0_JPRB*ZR5(JLON,KLEV)*PT(JLON,KLEV)&
   & -2.0_JPRB*PT5(JLON,KLEV)*ZR(JLON,KLEV)&
   & -2.0_JPRB*RKAPPA*ZDPHI)&
   & /(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA*ZDPHI5 &
   & +ZRS(JLON)*PTS(JLON))**2  

  ZSTA5=ZDPHI5*(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA &
   & *ZDPHI5-ZRS(JLON)*PTS(JLON))*ZRTI5  
  ZSTA=(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+2*RKAPPA &
   & *ZDPHI5-ZRS(JLON)*PTS(JLON))*ZRTI5*ZDPHI &
   & +ZDPHI5*ZR5(JLON,KLEV)*ZRTI5*PT(JLON,KLEV)&
   & +ZDPHI5*PT5(JLON,KLEV)*ZRTI5*ZR(JLON,KLEV)&
   & +(ZR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA &
   & *ZDPHI5-ZRS(JLON)*PTS(JLON))*ZDPHI5*ZRTI  

!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY (FOR G.W.D.)

  ZNBVNO5(JLON,KLEV)=ZSTA5/(PAPRS5(JLON,KLEV)*ZRTI5*ZDPHI5)**2
  ZNBVNO(JLON,KLEV)=ZSTA/(PAPRS5(JLON,KLEV)*ZRTI5*ZDPHI5)**2 &
   & -2.0_JPRB*ZSTA5*ZRTI5*ZDPHI5*PAPRS(JLON,KLEV)&
   & /(PAPRS5(JLON,KLEV)*ZRTI5*ZDPHI5)**3 &
   & -2.0_JPRB*ZSTA5*PAPRS5(JLON,KLEV)*ZDPHI5*ZRTI &
   & /(PAPRS5(JLON,KLEV)*ZRTI5*ZDPHI5)**3 &
   & -2.0_JPRB*ZSTA5*PAPRS5(JLON,KLEV)*ZRTI5*ZDPHI &
   & /(PAPRS5(JLON,KLEV)*ZRTI5*ZDPHI5)**3  

  ZGWDCS5(JLON)=PAPRS5(JLON,KLEV)*ZRTI5
  ZGWDCS(JLON)=PAPRS5(JLON,KLEV)*ZRTI+ZRTI5*PAPRS(JLON,KLEV)

ENDDO

!*
!     ------------------------------------------------------------------
!     IV - PRELIMINARY COMPUTATIONS FOR THE LAYER'S PRESSURE THICKNESS
!     DIVIDED BY G*DT AND ITS INVERSE FOR ALL LEVELS AS WELL AS SETTING
!     TO ZERO THE WEIGHTS FOR THE INTEGRAL OVER THE DEPTH OF THE
!     OROGRAPHIC OBSTACLE.

! - TEMPORARY(S) 2D (1:KLEV) .

! ZPOID     : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI     : INVERSE OF ZPOID.
! ZWGHT     : WEIGHTS FOR THE INTEGRAL OVER THE DEPTH OF THE OBSTACLE.

! The same but for trajectory
! ---------------------------
! - TEMPORARY(S) 2D (1:KLEV) .

! ZPOID5    : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI5    : INVERSE OF ZPOID.
! ZWGHT5    : WEIGHTS FOR THE INTEGRAL OVER THE DEPTH OF THE OBSTACLE.

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZPOID5(JLON,JLEV)=PDELP5(JLON,JLEV)*ZGDTI
    ZPOID(JLON,JLEV)=PDELP(JLON,JLEV)*ZGDTI

    ZIPOI5(JLON,JLEV)=PRDELP5(JLON,JLEV)*ZGDT
    ZIPOI(JLON,JLEV)=PRDELP(JLON,JLEV)*ZGDT

    ZWGHT5(JLON,JLEV)=0.0_JPRB
    ZWGHT(JLON,JLEV)=0.0_JPRB
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     V - SURFACE COMPUTATIONS AND INITIALIZATION FOR THE ASCENDING
!     LOOP.

! - TEMPORARY(S) 2D (0:KLEV) .

! ZNFNO     : NORMALISED EFECTIVE BRUNT-VAISALA FREQUENCY (N/RHO/G).
! ZRAPP     : RATIO OF THE UPPER AIR FLUX TO THE SURFACE ONE.
! ZU        : X COMPONENT OF THE EFFECTIVE WIND.
! ZV        : Y COMPONENT OF THE EFFECTIVE WIND.

! The same but for trajectory
! ---------------------------
! - TEMPORARY(S) 2D (0:KLEV) .

! ZNFNO5    : NORMALISED EFECTIVE BRUNT-VAISALA FREQUENCY (N/RHO/G).
! ZRAPP5    : RATIO OF THE UPPER AIR FLUX TO THE SURFACE ONE.
! ZU5       : X COMPONENT OF THE EFFECTIVE WIND.
! ZV5       : Y COMPONENT OF THE EFFECTIVE WIND.

!     VERTICAL AVERAGING OVER THE THICKNESS OF THE OROGRAPHIC OBSTACLE
!     AND CALCULATION OF THE EFFECTIVE VALUES (WIND AND STABILITY)
!     THROUGH LINEAR COMBINATIONS BETWEEN THE AVERAGES (CHOSEN AT THE
!     SURFACE) AND THE ORIGINAL VALUES (OBTAINED AGAIN AT THE EFFECTIVE
!     HEIGHT OF THE OBSTACLE).

! - TEMPORARY(S) 1D .

! ZOBST     : EFFECTIVE HEIGHT OF THE OBSTACLE IN PRESSURE COORDINATE.
! ZSUMF     : AVERAGED VALUE FOR A SQUARED "N/RHO/G" (<0 ACCEPTED).
! ZSUMU     : AVERAGED VALUE FOR THE "U" WIND.
! ZSUMV     : AVERAGED VALUE FOR THE "V" WIND.

! The same but for trajectory
! ----------------------------
! - TEMPORARY(S) 1D .

! ZOBST5    : EFFECTIVE HEIGHT OF THE OBSTACLE IN PRESSURE COORDINATE.
! ZSUMF5    : AVERAGED VALUE FOR A SQUARED "N/RHO/G" (<0 ACCEPTED).
! ZSUMU5    : AVERAGED VALUE FOR THE "U" WIND.
! ZSUMV5    : AVERAGED VALUE FOR THE "V" WIND.

!     WEIGHTED SUMMATION.

DO JLON=KIDIA,KFDIA
  ZOBST5(JLON)=MIN(PAPRS5(JLON,KLEV)&
   & -PAPRSF5(JLON,KTDIA),MAX(PAPRS5(JLON,KLEV)&
   & -PAPRSF5(JLON,KLEV)&
   & ,HOBST*PGETRL(JLON)*ZGWDCS5(JLON)))  

  ZVA5=PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV)
  ZVA=PAPRS(JLON,KLEV)-PAPRSF(JLON,KLEV)
  ZVB5=HOBST*PGETRL(JLON)*ZGWDCS5(JLON)
  ZVB=HOBST*PGETRL(JLON)*ZGWDCS(JLON)

  IF (ZVA5 > ZVB5) THEN
    ZVC5=ZVA5
    ZVC=ZVA
  ELSE
    ZVC5=ZVB5
    ZVC=ZVB
  ENDIF

  IF ((PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KTDIA)) < ZVC5) THEN
    ZOBST(JLON)=PAPRS(JLON,KLEV)-PAPRSF(JLON,KTDIA)
  ELSE
    ZOBST(JLON)=ZVC
  ENDIF

  ZWGHT5(JLON,KLEV)=(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))/ZOBST5(JLON)
  ZWGHT(JLON,KLEV)=PAPRS(JLON,KLEV)/ZOBST5(JLON)&
   & -PAPRSF(JLON,KLEV)/ZOBST5(JLON)&
   & -((PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))&
   & /ZOBST5(JLON)**2)*ZOBST(JLON)  

  ZSUMU5(JLON)=ZWGHT5(JLON,KLEV)*PU5(JLON,KLEV)
  ZSUMU(JLON)=ZWGHT5(JLON,KLEV)*PU(JLON,KLEV)+PU5(JLON,KLEV)*ZWGHT(JLON,KLEV)

  ZSUMV5(JLON)=ZWGHT5(JLON,KLEV)*PV5(JLON,KLEV)
  ZSUMV(JLON)=ZWGHT5(JLON,KLEV)*PV(JLON,KLEV)+PV5(JLON,KLEV)*ZWGHT(JLON,KLEV)

  ZSUMF5(JLON)=ZWGHT5(JLON,KLEV)*ZNBVNO5(JLON,KLEV)
  ZSUMF(JLON)=ZWGHT5(JLON,KLEV)*ZNBVNO(JLON,KLEV)&
   & +ZNBVNO5(JLON,KLEV)*ZWGHT(JLON,KLEV)  
ENDDO

DO JLEV=KLEV-1,KTDIA,-1

  DO JLON=KIDIA,KFDIA
    ZWGHT5(JLON,JLEV)=MAX(0.0_JPRB,(PAPRSF5(JLON,JLEV+1)&
     & -MAX(PAPRSF5(JLON,JLEV),PAPRS5(JLON,KLEV)&
     & -ZOBST5(JLON)))/ZOBST5(JLON))  

    ZXZX=PAPRS5(JLON,KLEV)-ZOBST5(JLON)
    IF (ZXZX > PAPRSF5(JLON,JLEV)) THEN
      ZWA5=PAPRS5(JLON,KLEV)-ZOBST5(JLON)
      ZWA=PAPRS(JLON,KLEV)-ZOBST(JLON)
    ELSE
      ZWA5=PAPRSF5(JLON,JLEV)
      ZWA=PAPRSF(JLON,JLEV)
    ENDIF

    IF (((PAPRSF5(JLON,JLEV+1)-ZWA5)/ZOBST5(JLON)) > 0.0_JPRB) THEN
      ZWGHT(JLON,JLEV)=PAPRSF(JLON,JLEV+1)/ZOBST5(JLON)&
       & -ZWA/ZOBST5(JLON)&
       & -((PAPRSF5(JLON,JLEV+1)-ZWA5)&
       & /ZOBST5(JLON)**2)*ZOBST(JLON)  
    ELSE
      ZWGHT(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZSUMU5(JLON)=ZSUMU5(JLON)+0.5_JPRB*ZWGHT5(JLON,JLEV)&
     & *(PU5(JLON,JLEV)+PU5(JLON,JLEV+1))  
    ZSUMU(JLON)=ZSUMU(JLON)+0.5_JPRB*ZWGHT(JLON,JLEV)&
     & *(PU5(JLON,JLEV)+PU5(JLON,JLEV+1))&
     & +0.5_JPRB*ZWGHT5(JLON,JLEV)&
     & *(PU(JLON,JLEV)+PU(JLON,JLEV+1))  

    ZSUMV5(JLON)=ZSUMV5(JLON)+0.5_JPRB*ZWGHT5(JLON,JLEV)&
     & *(PV5(JLON,JLEV)+PV5(JLON,JLEV+1))  
    ZSUMV(JLON)=ZSUMV(JLON)+0.5_JPRB*ZWGHT(JLON,JLEV)&
     & *(PV5(JLON,JLEV)+PV5(JLON,JLEV+1))&
     & +0.5_JPRB*ZWGHT5(JLON,JLEV)&
     & *(PV(JLON,JLEV)+PV(JLON,JLEV+1))  

    ZSUMF5(JLON)=ZSUMF5(JLON)+ZWGHT5(JLON,JLEV)*ZNBVNO5(JLON,JLEV)
    ZSUMF(JLON)=ZSUMF(JLON)&
     & +ZWGHT5(JLON,JLEV)*ZNBVNO(JLON,JLEV)&
     & +ZNBVNO5(JLON,JLEV)*ZWGHT(JLON,JLEV)  
  ENDDO

ENDDO

!     LINEAR COMBINATION.

DO JLON=KIDIA,KFDIA
  ZNFNO5(JLON,KLEV)=SQRT(MAX(0.0_JPRB,ZSUMF5(JLON)))
  IF (ZSUMF5(JLON) > 0) THEN
    ZNFNO(JLON,KLEV)=ZSUMF(JLON)/(2*SQRT(ZSUMF5(JLON)))
  ELSE
    ZNFNO(JLON,KLEV)=0.0_JPRB
  ENDIF

  ZPBLF5=1.0_JPRB-MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))/ZOBST5(JLON))
  ZZX=(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))/ZOBST5(JLON)
  IF (ZZX < 1.0_JPRB) THEN
    ZPBLF=-PAPRS(JLON,KLEV)/ZOBST5(JLON)&
     & +PAPRSF(JLON,KLEV)/ZOBST5(JLON)&
     & +(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,KLEV))*ZOBST(JLON)&
     & /ZOBST5(JLON)**2  
  ELSE
    ZPBLF=0.0_JPRB
  ENDIF

  ZU5(JLON,KLEV)=PU5(JLON,KLEV)+ZPBLF5*(ZSUMU5(JLON)-PU5(JLON,KLEV))
  ZU(JLON,KLEV)=PU(JLON,KLEV)&
   & +ZPBLF5*ZSUMU(JLON)-ZPBLF5*PU(JLON,KLEV)&
   & +(ZSUMU5(JLON)-PU5(JLON,KLEV))*ZPBLF  

  ZV5(JLON,KLEV)=PV5(JLON,KLEV)+ZPBLF5*(ZSUMV5(JLON)-PV5(JLON,KLEV))
  ZV(JLON,KLEV)=PV(JLON,KLEV)&
   & +ZPBLF5*ZSUMV(JLON)-ZPBLF5*PV(JLON,KLEV)&
   & +(ZSUMV5(JLON)-PV5(JLON,KLEV))*ZPBLF  
ENDDO

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZPBL5=1.0_JPRB-MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))/ZOBST5(JLON))
    ZZX1=(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))/ZOBST5(JLON)
    IF (ZZX1 < 1.0_JPRB) THEN
      ZPBL=-PAPRS(JLON,KLEV)/ZOBST5(JLON)&
       & +PAPRS(JLON,JLEV)/ZOBST5(JLON)&
       & +(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))*ZOBST(JLON)&
       & /ZOBST5(JLON)**2  
    ELSE
      ZPBL=0.0_JPRB
    ENDIF

    ZNFNO5(JLON,JLEV)=SQRT(MAX(0.0_JPRB,ZNBVNO5(JLON,JLEV)+ZPBL5 &
     & *(ZSUMF5(JLON)-ZNBVNO5(JLON,JLEV))))  
    Z_XXXX=ZNBVNO5(JLON,JLEV)+ZPBL5*(ZSUMF5(JLON)-ZNBVNO5(JLON,JLEV))
    IF (Z_XXXX > 0.0_JPRB) THEN
      ZNFNO(JLON,JLEV)=0.5_JPRB/SQRT(ZNBVNO5(JLON,JLEV)+ZPBL5 &
       & *(ZSUMF5(JLON)-ZNBVNO5(JLON,JLEV)))&
       & *(ZNBVNO(JLON,JLEV)&
       & +(ZSUMF5(JLON)-ZNBVNO5(JLON,JLEV))*ZPBL &
       & +ZPBL5*ZSUMF(JLON)&
       & -ZPBL5*ZNBVNO(JLON,JLEV))  
    ELSE
      ZNFNO(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZPBLF5=1.0_JPRB-MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,JLEV))&
     & /ZOBST5(JLON))  
    ZZX2=(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,JLEV))/ZOBST5(JLON)
    IF (ZZX2 < 1.0_JPRB) THEN
      ZPBLF=-PAPRS(JLON,KLEV)/ZOBST5(JLON)&
       & +PAPRSF(JLON,JLEV)/ZOBST5(JLON)&
       & +(PAPRS5(JLON,KLEV)-PAPRSF5(JLON,JLEV))*ZOBST(JLON)&
       & /ZOBST5(JLON)**2  
    ELSE
      ZPBLF=0.0_JPRB
    ENDIF

    ZU5(JLON,JLEV)=PU5(JLON,JLEV)+ZPBLF5*(ZSUMU5(JLON)-PU5(JLON,JLEV))
    ZU(JLON,JLEV)=PU(JLON,JLEV)&
     & +ZPBLF5*ZSUMU(JLON)-ZPBLF5*PU(JLON,JLEV)&
     & +(ZSUMU5(JLON)-PU5(JLON,JLEV))*ZPBLF  

    ZV5(JLON,JLEV)=PV5(JLON,JLEV)+ZPBLF5*(ZSUMV5(JLON)-PV5(JLON,JLEV))
    ZV(JLON,JLEV)=PV(JLON,JLEV)&
     & +ZPBLF5*ZSUMV(JLON)-ZPBLF5*PV(JLON,JLEV)&
     & +(ZSUMV5(JLON)-PV5(JLON,JLEV))*ZPBLF  
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA

!     COMPUTATION OF A FICTITIOUS WIND THAT WOULD CORRESPOND, IN THE
!     ISOTROPIC CASE, TO THE ANISOTROPIC DRAG.

! - TEMPORARY(S) 1D .

! ZUSUR     : X COMPONENT OF THE FICTITIOUS SURFACE WIND.
! ZVSUR     : Y COMPONENT OF THE FICTITIOUS SURFACE WIND.

! The same but for trajectory
! ----------------------------
! - TEMPORARY(S) 1D .

! ZUSUR5    : X COMPONENT OF THE FICTITIOUS SURFACE WIND.
! ZVSUR5    : Y COMPONENT OF THE FICTITIOUS SURFACE WIND.

  ZUSTAR5=ZSUMU5(JLON)*COS(2.0_JPRB*PVRLDI(JLON))+ZSUMV5(JLON)&
   & *SIN(2.0_JPRB*PVRLDI(JLON))  
  ZUSTAR=ZSUMU(JLON)*COS(2.0_JPRB*PVRLDI(JLON))&
   & +ZSUMV(JLON)*SIN(2.0_JPRB*PVRLDI(JLON))  

  ZVSTAR5=ZSUMU5(JLON)*SIN(2.0_JPRB*PVRLDI(JLON))&
   & -ZSUMV5(JLON)*COS(2.0_JPRB*PVRLDI(JLON))  
  ZVSTAR=ZSUMU(JLON)*SIN(2.0_JPRB*PVRLDI(JLON))&
   & -ZSUMV(JLON)*COS(2.0_JPRB*PVRLDI(JLON))  

  ZA=PVRLAN(JLON)**2+0.5_JPRB*(4._JPRB*(1.0_JPRB-PVRLAN(JLON))*(1.0_JPRB+ZALP1 &
   & *PVRLAN(JLON))+PVRLAN(JLON)*LOG(1.0_JPRB/MAX(ZEPS1,PVRLAN(JLON)))&
   & *(1.0_JPRB+ZALP2*PVRLAN(JLON)))/RPI  
  ZD=0.5_JPRB*(4._JPRB*(1.0_JPRB-PVRLAN(JLON))*(1.0_JPRB+ZDEL1*PVRLAN(JLON))-3._JPRB &
   & *PVRLAN(JLON)*LOG(1.0_JPRB/MAX(ZEPS1,PVRLAN(JLON)))*(1.0_JPRB+ZDEL2 &
   & *PVRLAN(JLON)))/RPI  

  ZUSUR5(JLON)=ZA*ZSUMU5(JLON)+ZD*ZUSTAR5
  ZUSUR(JLON)=ZA*ZSUMU(JLON)+ZD*ZUSTAR

  ZVSUR5(JLON)=ZA*ZSUMV5(JLON)+ZD*ZVSTAR5
  ZVSUR(JLON)=ZA*ZSUMV(JLON)+ZD*ZVSTAR

!     "LINEAR" PART OF THE COMPUTATION.

! - TEMPORARY(S) 1D .

! ZUSR      : SPEED OF FICTITIOUS SURFACE WIND.
! ZFACS     : SURFACE FACTOR FOR THE COMPUTATION OF ZRAPP.
! ZSTUS     : "LINEAR" VALUE OF THE "U" SURFACE FLUX.
! ZSTVS     : "LINEAR" VALUE OF THE "V" SURFACE FLUX.
! ZSUSR     : SURF. SCALAR PRODUCT OF EFFECTIVE AND FICITIOUS WINDS.

! The same but for trajectory
! ---------------------------
! - TEMPORARY(S) 1D .

! ZUSR5     : SPEED OF FICTITIOUS SURFACE WIND.
! ZFACS5    : SURFACE FACTOR FOR THE COMPUTATION OF ZRAPP.
! ZSTUS5    : "LINEAR" VALUE OF THE "U" SURFACE FLUX.
! ZSTVS5    : "LINEAR" VALUE OF THE "V" SURFACE FLUX.
! ZSUSR5    : SURF. SCALAR PRODUCT OF EFFECTIVE AND FICITIOUS WINDS.

  ZSUSR5(JLON)=MAX(ZEPS2,ZSUMU5(JLON)*ZUSUR5(JLON)+ZSUMV5(JLON)*ZVSUR5(JLON))
  Z_YYYY=ZSUMU5(JLON)*ZUSUR5(JLON)+ZSUMV5(JLON)*ZVSUR5(JLON)
  IF (Z_YYYY > ZEPS2) THEN
    ZSUSR(JLON)=ZSUMU5(JLON)*ZUSUR(JLON)&
     & +ZUSUR5(JLON)*ZSUMU(JLON)&
     & +ZSUMV5(JLON)*ZVSUR(JLON)+ZVSUR5(JLON)*ZSUMV(JLON)  
  ELSE
    ZSUSR(JLON)=0.0_JPRB
  ENDIF

  ZUSR5(JLON)=MAX(0.0_JPRB,SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2))
  IF ((SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2)) > 0) THEN
    ZUSR(JLON)=ZUSUR5(JLON)*ZUSUR(JLON)&
     & /SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2)&
     & +ZVSUR5(JLON)*ZVSUR(JLON)&
     & /SQRT(ZUSUR5(JLON)**2+ZVSUR5(JLON)**2)  
  ELSE
    ZUSR(JLON)=0.0_JPRB
  ENDIF

  ZFACS5(JLON)=ZNFNO5(JLON,KLEV)/ZSUSR5(JLON)**3
  ZFACS(JLON)=ZNFNO(JLON,KLEV)/ZSUSR5(JLON)**3 &
   & -(3._JPRB*ZNFNO5(JLON,KLEV)/ZSUSR5(JLON)**4)*ZSUSR(JLON)  
  ZRAPP5(JLON,KLEV)=1.0_JPRB
  ZRAPP(JLON,KLEV)=0.0_JPRB

  ZSTUS5(JLON)=GWDSE*ZGWDCS5(JLON)**2*ZNFNO5(JLON,KLEV)&
   & *PGETRL(JLON)*ZUSUR5(JLON)  
  ZSTUS(JLON)=2.0_JPRB*GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)&
   & *ZNFNO5(JLON,KLEV)*ZUSUR5(JLON)*ZGWDCS(JLON)&
   & +GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)**2 &
   & *ZNFNO5(JLON,KLEV)*ZUSUR(JLON)&
   & +GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)**2 &
   & *ZUSUR5(JLON)*ZNFNO(JLON,KLEV)  

  ZSTVS5(JLON)=GWDSE*ZGWDCS5(JLON)**2*ZNFNO5(JLON,KLEV)&
   & *PGETRL(JLON)*ZVSUR5(JLON)  
  ZSTVS(JLON)=2.0_JPRB*GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)&
   & *ZNFNO5(JLON,KLEV)*ZVSUR5(JLON)*ZGWDCS(JLON)&
   & +GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)**2 &
   & *ZNFNO5(JLON,KLEV)*ZVSUR(JLON)&
   & +GWDSE*PGETRL(JLON)*ZGWDCS5(JLON)**2 &
   & *ZVSUR5(JLON)*ZNFNO(JLON,KLEV)  

  ZTUNE5=GWDBC*(ZUSR5(JLON)**2/ZSUSR5(JLON))**2
  ZTUNE=(4._JPRB*GWDBC*ZUSR5(JLON)**3/ZSUSR5(JLON)**2)*ZUSR(JLON)&
   & -(2.0_JPRB*GWDBC*ZUSR5(JLON)**4/ZSUSR5(JLON)**3)*ZSUSR(JLON)  

  ZBB5(JLON)=MAX(0.0_JPRB,1.0_JPRB-ZUSR5(JLON)/MAX(ZEPS5,ZTUNE5 &
   & *HOBST*PGETRL(JLON)*ZGWDCS5(JLON)&
   & *ZNFNO5(JLON,KLEV)))  
  ZZZB=ZTUNE5*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZNFNO5(JLON,KLEV)
  IF (ZZZB > ZEPS5) THEN
    ZBB15=ZTUNE5*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZNFNO5(JLON,KLEV)
    ZBB1=HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZNFNO5(JLON,KLEV)&
     & *ZTUNE &
     & +HOBST*PGETRL(JLON)*ZTUNE5*ZNFNO5(JLON,KLEV)&
     & *ZGWDCS(JLON)&
     & +HOBST*PGETRL(JLON)*ZTUNE5*ZGWDCS5(JLON)&
     & *ZNFNO(JLON,KLEV)  
  ELSE
    ZBB15=ZEPS5
    ZBB1=0.0_JPRB
  ENDIF

  IF ((1.0_JPRB-ZUSR5(JLON)/ZBB15) > 0.0_JPRB) THEN
    ZBB(JLON)=-ZUSR(JLON)/ZBB15+(ZUSR5(JLON)/ZBB15**2)*ZBB1
  ELSE
    ZBB(JLON)=0.0_JPRB
  ENDIF

  ZAA5(JLON)=GWDCD*ZTUNE5*ZBB5(JLON)*(1.0_JPRB-ZBB5(JLON))
  ZAA(JLON)=GWDCD*ZBB5(JLON)*(1.0_JPRB-ZBB5(JLON))*ZTUNE &
   & +GWDCD*ZTUNE5*(1.0_JPRB-2.0_JPRB*ZBB5(JLON))*ZBB(JLON)  

  IF (LSMOOTHB) THEN
    ZAA5(JLON)=0.0_JPRB
    ZAA(JLON)=0.0_JPRB
  ENDIF
ENDDO

!*
!     ------------------------------------------------------------------
!     VI - ASCENDING COMPUTATIONS OF THE LINEAR ESTIMATE OF THE RELATIVE
!     FLUX AND CHECKING OF THE FIRST LEVELS OF DEPOSITION AND OF
!     REFLEXION (IF THE LATTER EXISTS).

!     COMPUTATIONS IN THE VERTICAL LOOP.

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA

!     "LINEAR" PART OF THE COMPUTATION.

    ZAUSR5=0.5_JPRB*((ZU5(JLON,JLEV)+ZU5(JLON,JLEV+1))*ZUSUR5(JLON)&
     & +(ZV5(JLON,JLEV)+ZV5(JLON,JLEV+1))*ZVSUR5(JLON))  
    ZAUSR=0.5_JPRB*(ZUSUR5(JLON)*ZU(JLON,JLEV+1)&
     & +ZUSUR5(JLON)*ZU(JLON,JLEV)&
     & +(ZU5(JLON,JLEV)+ZU5(JLON,JLEV+1))*ZUSUR(JLON)&
     & +ZVSUR5(JLON)*ZV(JLON,JLEV)&
     & +ZVSUR5(JLON)*ZV(JLON,JLEV+1)&
     & +(ZV5(JLON,JLEV)+ZV5(JLON,JLEV+1))*ZVSUR(JLON))  

    ZPRRAP5=ZFACS5(JLON)*ZAUSR5**3/MAX(ZEPS3,ZNFNO5(JLON,JLEV))
    IF(ZNFNO5(JLON,JLEV) > ZEPS3) THEN
      ZPRRAP=(ZAUSR5**3/ZNFNO5(JLON,JLEV))*ZFACS(JLON)&
       & +(3._JPRB*ZAUSR5**2*ZFACS5(JLON)/ZNFNO5(JLON,JLEV))*ZAUSR &
       & -(ZFACS5(JLON)*ZAUSR5**3/ZNFNO5(JLON,JLEV)**2)&
       & *ZNFNO(JLON,JLEV)  
    ELSE
      ZPRRAP=(ZAUSR5**3/ZEPS3)*ZFACS(JLON)&
       & +(3._JPRB*ZAUSR5**2*ZFACS5(JLON)/ZEPS3)*ZAUSR  
    ENDIF

    ZRAPP5(JLON,JLEV)=MAX(0.0_JPRB,MIN(ZRAPP5(JLON,JLEV+1),ZPRRAP5))
    ZZP5=MIN(ZRAPP5(JLON,JLEV+1),ZPRRAP5)
    IF (ZRAPP5(JLON,JLEV+1) < ZPRRAP5) THEN
      ZZP=ZRAPP(JLON,JLEV+1)
    ELSE
      ZZP=ZPRRAP
    ENDIF

    IF (ZZP5 > 0.0_JPRB) THEN
      ZRAPP(JLON,JLEV)=ZZP
    ELSE
      ZRAPP(JLON,JLEV)=0.0_JPRB
    ENDIF

  ENDDO
ENDDO

!     UPPER BOUNDARY CONDITION AND PREPARATION OF THE DESCENDING LOOP.

DO JLON=KIDIA,KFDIA

!     "LINEAR" PART OF THE COMPUTATION.

  PSTRDU5(JLON,KTDIA-1)=0.0_JPRB
  PSTRDU(JLON,KTDIA-1)=0.0_JPRB

  PSTRDV5(JLON,KTDIA-1)=0.0_JPRB
  PSTRDV(JLON,KTDIA-1)=0.0_JPRB

  ZRAPP5(JLON,KTDIA-1)=0.0_JPRB
  ZRAPP(JLON,KTDIA-1)=0.0_JPRB
ENDDO

!*
!     ------------------------------------------------------------------
!     VII - DESCENDING LOOP FOR THE COMBINATION OF EFFECTS AND THE
!     NUMERICAL SECURITIES LEADING TO THE FINAL FLUX CALCULATION.

!     PROVISIONAL FLUXES TAKING INTO ACOUNT ALL THREE EFFECTS.

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA

    ZZPA5=MIN(1.0_JPRB,ZRAPP5(JLON,JLEV))
    IF (ZRAPP5(JLON,JLEV) < 1.0_JPRB) THEN
      ZZPA=ZRAPP(JLON,JLEV)
    ELSE
      ZZPA=0.0_JPRB
    ENDIF

    ZRAPP5(JLON,JLEV)=MAX(0.0_JPRB,ZZPA5)
    IF (ZZPA5 > 0.0_JPRB) THEN
      ZRAPP(JLON,JLEV)=ZZPA
    ELSE
      ZRAPP(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZRZ5=MIN(1.0_JPRB,(PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))&
     & /MAX(ZEPS6,ZBB5(JLON)*HOBST*PGETRL(JLON)&
     & *ZGWDCS5(JLON)))  
    IF((ZBB5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)) > ZEPS6)THEN
      ZRZ15=ZBB5(JLON)*HOBST*PGETRL(JLON)*ZGWDCS5(JLON)
      ZRZ1=HOBST*PGETRL(JLON)*ZBB5(JLON)*ZGWDCS(JLON)&
       & +HOBST*PGETRL(JLON)*ZGWDCS5(JLON)*ZBB(JLON)  
    ELSE
      ZRZ15=ZEPS6
      ZRZ1=0.0_JPRB
    ENDIF

    IF (((PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))/ZRZ15) < 1.0_JPRB) THEN
      ZRZ=PAPRS(JLON,KLEV)/ZRZ15 &
       & -PAPRS(JLON,JLEV)/ZRZ15 &
       & -((PAPRS5(JLON,KLEV)-PAPRS5(JLON,JLEV))/ZRZ15**2)*ZRZ1  
    ELSE
      ZRZ=0.0_JPRB
    ENDIF

    IF (((1.0_JPRB-ZRZ5)**3/(1.0_JPRB+ZRZ5*ZBB5(JLON))) > ZEPS6) THEN
      ZRAPPH5(JLON,JLEV)=(1.0_JPRB-ZRZ5)**3/(1.0_JPRB+ZRZ5*ZBB5(JLON))
      ZRAPPH(JLON,JLEV)=-(3._JPRB*(1.0_JPRB-ZRZ5)**2/(1.0_JPRB+ZRZ5*ZBB5(JLON)))&
       & *ZRZ &
       & -((1.0_JPRB-ZRZ5)**3*ZRZ5/(1.0_JPRB+ZRZ5*ZBB5(JLON))**2)&
       & *ZBB(JLON)&
       & -((1.0_JPRB-ZRZ5)**3*ZBB5(JLON)&
       & /(1.0_JPRB+ZRZ5*ZBB5(JLON))**2)*ZRZ  
    ELSE
      ZRAPPH5(JLON,JLEV)=ZEPS6
      ZRAPPH(JLON,JLEV)=0.0_JPRB
    ENDIF

    ZRAPP(JLON,JLEV)=(1.0_JPRB+ZAA5(JLON)*SQRT(ZRAPPH5(JLON,JLEV)))&
     & *ZRAPP(JLON,JLEV)&
     & +ZRAPP5(JLON,JLEV)*SQRT(ZRAPPH5(JLON,JLEV))&
     & *ZAA(JLON)&
     & +(0.5_JPRB*ZRAPP5(JLON,JLEV)*ZAA5(JLON)&
     & /SQRT(ZRAPPH5(JLON,JLEV)))*ZRAPPH(JLON,JLEV)  
    ZRAPP5(JLON,JLEV)=ZRAPP5(JLON,JLEV)*(1.0_JPRB+ZAA5(JLON)&
     & *SQRT(MAX(ZEPS6,(1.0_JPRB-ZRZ5)**3/(1.0_JPRB+ZRZ5 &
     & *ZBB5(JLON)))))  

    PSTRDU5(JLON,JLEV)=ZRAPP5(JLON,JLEV)*ZSTUS5(JLON)
    PSTRDU(JLON,JLEV)=ZRAPP5(JLON,JLEV)*ZSTUS(JLON)&
     & +ZSTUS5(JLON)*ZRAPP(JLON,JLEV)  

    PSTRDV5(JLON,JLEV)=ZRAPP5(JLON,JLEV)*ZSTVS5(JLON)
    PSTRDV(JLON,JLEV)=ZRAPP5(JLON,JLEV)*ZSTVS(JLON)&
     & +ZSTVS5(JLON)*ZRAPP(JLON,JLEV)  
  ENDDO
ENDDO

!     PSEUDO-IMPLICIT ALGORITHM WITH RESPECT TO THE EFFECTIVE SURFACE
!     WIND PROJECTED ON THE STRESS DIRECTION.

! - TEMPORARY(S) 1D .

! ZSUMD     : AVERAGED WIND TENDENCIES PROJECTED ON THE STRESS' DIREC..

! The same but for trajectory
! ---------------------------
! - TEMPORARY(S) 1D .

! ZSUMD5    : AVERAGED WIND TENDENCIES PROJECTED ON THE STRESS' DIREC..

!     SUMMATION FOR A TEMPORARY NORMALISED AVERAGE OF TENDENCIES.

DO JLON=KIDIA,KFDIA
  ZSUMD5(JLON)=0.5_JPRB*ZWGHT5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA)&
   & *(ZRAPP5(JLON,KTDIA)-ZRAPP5(JLON,KTDIA-1))  
  ZSUMD(JLON)=0.5_JPRB*ZIPOI5(JLON,KTDIA)&
   & *(ZRAPP5(JLON,KTDIA)-ZRAPP5(JLON,KTDIA-1))&
   & *ZWGHT(JLON,KTDIA)&
   & +0.5_JPRB*ZWGHT5(JLON,KTDIA)&
   & *(ZRAPP5(JLON,KTDIA)-ZRAPP5(JLON,KTDIA-1))&
   & *ZIPOI(JLON,KTDIA)&
   & +0.5_JPRB*ZWGHT5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA)&
   & *ZRAPP(JLON,KTDIA)&
   & -0.5_JPRB*ZWGHT5(JLON,KTDIA)*ZIPOI5(JLON,KTDIA)&
   & *ZRAPP(JLON,KTDIA-1)  
ENDDO

DO JLEV=KTDIA+1,KLEV
  DO JLON=KIDIA,KFDIA
    ZSUMD5(JLON)=ZSUMD5(JLON)+0.5_JPRB*(ZWGHT5(JLON,JLEV)&
     & +ZWGHT5(JLON,JLEV-1))*ZIPOI5(JLON,JLEV)&
     & *(ZRAPP5(JLON,JLEV)-ZRAPP5(JLON,JLEV-1))  
    ZSUMD(JLON)=ZSUMD(JLON)&
     & +0.5_JPRB*(ZRAPP5(JLON,JLEV)-ZRAPP5(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZWGHT(JLON,JLEV)&
     & +0.5_JPRB*(ZRAPP5(JLON,JLEV)-ZRAPP5(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZWGHT(JLON,JLEV-1)&
     & +0.5_JPRB*(ZWGHT5(JLON,JLEV)+ZWGHT5(JLON,JLEV-1))&
     & *(ZRAPP5(JLON,JLEV)-ZRAPP5(JLON,JLEV-1))&
     & *ZIPOI(JLON,JLEV)&
     & +0.5_JPRB*(ZWGHT5(JLON,JLEV)+ZWGHT5(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZRAPP(JLON,JLEV)&
     & -0.5_JPRB*(ZWGHT5(JLON,JLEV)+ZWGHT5(JLON,JLEV-1))&
     & *ZIPOI5(JLON,JLEV)*ZRAPP(JLON,JLEV-1)  
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZSUMD5(JLON)=ZSUMD5(JLON)&
   & +0.5_JPRB*ZWGHT5(JLON,KLEV)*ZIPOI5(JLON,KLEV)&
   & *(ZRAPP5(JLON,KLEV)-ZRAPP5(JLON,KLEV-1))  
  ZSUMD(JLON)=ZSUMD(JLON)&
   & +0.5_JPRB*(ZRAPP5(JLON,KLEV)-ZRAPP5(JLON,KLEV-1))&
   & *ZIPOI5(JLON,KLEV)*ZWGHT(JLON,KLEV)&
   & +0.5_JPRB*(ZRAPP5(JLON,KLEV)-ZRAPP5(JLON,KLEV-1))&
   & *ZWGHT5(JLON,KLEV)*ZIPOI(JLON,KLEV)&
   & +0.5_JPRB*ZWGHT5(JLON,KLEV)*ZIPOI5(JLON,KLEV)&
   & *ZRAPP(JLON,KLEV)&
   & -0.5_JPRB*ZWGHT5(JLON,KLEV)*ZIPOI5(JLON,KLEV)&
   & *ZRAPP(JLON,KLEV-1)  
ENDDO

!     FINAL AVERAGED VALUE, LINEAR REDUCTION FACTOR AND ITS USE.

DO JLON=KIDIA,KFDIA
  IF ((SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)) > 0.0_JPRB) THEN
    ZSUMD(JLON)=ZSUMD(JLON)*SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)&
     & +ZSUMD5(JLON)*ZSTUS5(JLON)*ZSTUS(JLON)&
     & /SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)&
     & +ZSUMD5(JLON)*ZSTVS5(JLON)*ZSTVS(JLON)&
     & /SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2)  
  ELSE
    ZSUMD(JLON)=0.0_JPRB
  ENDIF
  ZSUMD5(JLON)=ZSUMD5(JLON)*MAX(0.0_JPRB,SQRT(ZSTUS5(JLON)**2+ZSTVS5(JLON)**2))

  ZRAPP5(JLON,KLEV)=1.0_JPRB/(1.0_JPRB+ZSUMD5(JLON)*ZUSR5(JLON)/ZSUSR5(JLON))
  ZRAPP(JLON,KLEV)=ZSUSR(JLON)&
   & /(ZSUSR5(JLON)+ZSUMD5(JLON)*ZUSR5(JLON))&
   & -ZSUSR5(JLON)*ZSUSR(JLON)&
   & /(ZSUSR5(JLON)+ZSUMD5(JLON)*ZUSR5(JLON))**2 &
   & -ZSUMD5(JLON)*ZSUSR5(JLON)*ZUSR(JLON)&
   & /(ZSUSR5(JLON)+ZSUMD5(JLON)*ZUSR5(JLON))**2 &
   & -ZUSR5(JLON)*ZSUSR5(JLON)*ZSUMD(JLON)&
   & /(ZSUSR5(JLON)+ZSUMD5(JLON)*ZUSR5(JLON))**2  
ENDDO

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    PSTRDU(JLON,JLEV)=PSTRDU5(JLON,JLEV)*ZRAPP(JLON,KLEV)&
     & +ZRAPP5(JLON,KLEV)*PSTRDU(JLON,JLEV)  
    PSTRDU5(JLON,JLEV)=PSTRDU5(JLON,JLEV)*ZRAPP5(JLON,KLEV)

    PSTRDV(JLON,JLEV)=PSTRDV5(JLON,JLEV)*ZRAPP(JLON,KLEV)&
     & +ZRAPP5(JLON,KLEV)*PSTRDV(JLON,JLEV)  
    PSTRDV5(JLON,JLEV)=PSTRDV5(JLON,JLEV)*ZRAPP5(JLON,KLEV)
  ENDDO
ENDDO

!     NUMERICAL SECURITY (USE OF AN ALGORITHM INSPIRED BY THE ONE OF THE
!     MASS-FLUX TYPE DEEP CONVECTION ROUTINES).

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA

    ZDU5=ZIPOI5(JLON,JLEV)*(PSTRDU5(JLON,JLEV-1)-PSTRDU5(JLON,JLEV))
    ZDU=ZIPOI5(JLON,JLEV)*PSTRDU(JLON,JLEV-1)&
     & -ZIPOI5(JLON,JLEV)*PSTRDU(JLON,JLEV)&
     & +(PSTRDU5(JLON,JLEV-1)-PSTRDU5(JLON,JLEV))*ZIPOI(JLON,JLEV)  

    ZDV5=ZIPOI5(JLON,JLEV)*(PSTRDV5(JLON,JLEV-1)-PSTRDV5(JLON,JLEV))
    ZDV=ZIPOI5(JLON,JLEV)*PSTRDV(JLON,JLEV-1)&
     & -ZIPOI5(JLON,JLEV)*PSTRDV(JLON,JLEV)&
     & +(PSTRDV5(JLON,JLEV-1)-PSTRDV5(JLON,JLEV))*ZIPOI(JLON,JLEV)  

    ZDX5=ZIPOI5(JLON,JLEV)*(ZUSUR5(JLON)*PSTRDU5(JLON,JLEV)&
     & +ZVSUR5(JLON)*PSTRDV5(JLON,JLEV))  
    ZDX=(ZUSUR5(JLON)*PSTRDU5(JLON,JLEV)&
     & +ZVSUR5(JLON)*PSTRDV5(JLON,JLEV))*ZIPOI(JLON,JLEV)&
     & +ZIPOI5(JLON,JLEV)*ZUSUR5(JLON)*PSTRDU(JLON,JLEV)&
     & +ZIPOI5(JLON,JLEV)*PSTRDU5(JLON,JLEV)*ZUSUR(JLON)&
     & +ZIPOI5(JLON,JLEV)*ZVSUR5(JLON)*PSTRDV(JLON,JLEV)&
     & +ZIPOI5(JLON,JLEV)*PSTRDV5(JLON,JLEV)*ZVSUR(JLON)  

    ZX5=ZUSUR5(JLON)*PU5(JLON,JLEV)+ZVSUR5(JLON)*PV5(JLON,JLEV)
    ZX=ZUSUR5(JLON)*PU(JLON,JLEV)+PU5(JLON,JLEV)*ZUSUR(JLON)&
     & +ZVSUR5(JLON)*PV(JLON,JLEV)+PV5(JLON,JLEV)*ZVSUR(JLON)  

    ZALPHA5=1.0_JPRB/(1.0_JPRB+ZDX5/MAX(ZEPS2,ZX5))
    IF (ZX5 > ZEPS2) THEN
      ZALPHA=ZX/(ZX5+ZDX5)-(ZX5/(ZX5+ZDX5)**2)*ZX-(ZX5/(ZX5+ZDX5)**2)*ZDX
    ELSE
      ZALPHA=-ZEPS2*ZDX/(ZEPS2+ZDX5)**2
    ENDIF

    PSTRDU5(JLON,JLEV)=PSTRDU5(JLON,JLEV-1)-ZPOID5(JLON,JLEV)*ZALPHA5*ZDU5
    PSTRDU(JLON,JLEV)=PSTRDU(JLON,JLEV-1)&
     & -ZALPHA5*ZDU5*ZPOID(JLON,JLEV)&
     & -ZPOID5(JLON,JLEV)*ZALPHA5*ZDU &
     & -ZPOID5(JLON,JLEV)*ZDU5*ZALPHA  

    PSTRDV5(JLON,JLEV)=PSTRDV5(JLON,JLEV-1)-ZPOID5(JLON,JLEV)*ZALPHA5*ZDV5
    PSTRDV(JLON,JLEV)=PSTRDV(JLON,JLEV-1)&
     & -ZALPHA5*ZDV5*ZPOID(JLON,JLEV)&
     & -ZPOID5(JLON,JLEV)*ZALPHA5*ZDV &
     & -ZPOID5(JLON,JLEV)*ZDV5*ZALPHA  
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('ACDRAGLTL',1,ZHOOK_HANDLE)
END SUBROUTINE ACDRAGLTL
