!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACHMTAD ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - TRAJECTORY .
 & PAPHI5,PAPHIF5,PAPRS5,PAPRSF5,PCP5,PQ5,PR5,PT5,&
 & PU5,PV5,PGZ0F5,PHV5,&
 & PNEIJG5,PNEIJV5,PSNS5,PTS5,PWS5,PWSI5,&
 ! - INPUT  2D .
 & PAPHI,PAPHIF,PAPRS,PAPRSF,PCP,PQ,PR,PT,PU,PV,&
 ! - INPUT  1D .
 & PDPHIT,PDPHIV,PGZ0F,PGZ0HF,PGZ0RLF,PHV,PLSM,&
 & PNEIJG,PNEIJV,PSNS,PTS,PVEG0,PWFC,PWS,PWSI,&
 ! - INPUT  LOGIQUE .
 & LDCLS,LDHMT,LDZ0H,&
 ! - OUTPUT 2D .
 & PNBVNO,&
 ! - OUTPUT 1D .
 & PCD,PCDN,PCDROV,PCH,PCHROV,PCPS,PDQSTS,PGWDCS,&
 & PGZ0,PHQ,PHU,PNEIJ,PQCLS,PQS,PQSATS,&
 & PRHCLS,PRS,PSTAB,PTCLS,PUCLS,PVCLS,PUCLN,PVCLN,PZPCLS,PVEG,&
 & PXDROV,PXHROV,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACHMTAD   * - DETERMINATION DES CARACTERISTIQUES DE SURFACE.
!**** *ACHMTAD   * - INTERP. DES PARAMETRES U,V,T,Q AUX HAUTEURS CHOISIES.

!     Sujet.
!     ------

!     - ROUTINE DE CALCUL ACTIF .
!       DETERMINATION DE CARACTERISTIQUES DE SURFACE CONCERNANT LA
!       NEIGE, L'HUMIDITE DU SOL ET LES COEFFICIENTS D'ECHANGE EN
!       SURFACE PLUS LEUR AMPLIFICATION SI "ANTI-FIBRILLATION" ACTIVEE .
!     - SI L'OPTION 'CLS' EST SELECTIONNEE :
!          DETERMINATION DE PARAMETRES SUPPLEMENTAIRES NECESSAIRES POUR
!          LES CALCULS DE DIFFUSION VERTICALE ET DE TRAINEE DES ONDES DE
!          GRAVITE OROGRAPHIQUES .
!     - SI L'OPTION 'HMT' EST SELECTIONNEE :
!          INTERPOLATION DES TEMPERATURES, HUMIDITES (SPECIFIQUES ET
!          RELATIVES) ET VENTS AUX HAUTEURS "METEO" .

!     - COMPUTATION OF SURFACE CHARACTERISTICS FOR SNOW, SOIL HUMIDITY
!       AND SURFACE EXCHANGE COEFFICIENTS PLUS THEIR AMPLIFICATION
!       FACTOR IF "ANTI-FIBRILLATION" IS ACTIVATED .
!     - IF THE SWITCH 'CLS' IS ON :
!          COMPUTATION OF ADDITIONNAL PARAMETERS NECESSARY FOR THE
!          VERTICAL DIFFUSION AND GRAVITY WAVE DRAG PARAMETERIZATIONS .
!     - IF THE SWITCH 'HMT' IS ON :
!          TEMPERATURE, MOISTURE (SPECIFIC AND RELATIVE) AND WIND
!          INTERPOLATION AT STANDARD "METEOROLOGICAL" LEVELS .

!**   Interface.
!     ----------
!        *CALL* *ACHMTAD*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPHI      : GEOPOTENTIEL AUX DEMI-NIVEAUX.
! PAPRS      : PRESSION AUX DEMI-NIVEAUX.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES.
! PAPRSF     : PRESSION AUX NIVEAUX.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PT         : TEMPERATURE.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.

! - 1D (PROGNOSTIQUE) .

! PSNS       : MASSE PAR UNITE DE SURFACE DE LA COUCHE NEIGEUSE.
! PTS        : TEMPERATURE DE SURFACE.
! PWS        : CONTENU EN EAU DU RESERVOIR DE SURFACE.
! PWSI       : CONTENU EN EAU GELEE DU RESERVOIR DE SURFACE.

! - 1D (SPECIFIQUE SELON L'UTILISATION DE ACHMT) .

! PDPHIT     : HAUTEUR D INTERPOLATION POUR T ET Q (EN GEOPOTENTIEL).
! PDPHIV     : HAUTEUR D INTERPOLATION POUR U ET V (EN GEOPOTENTIEL).
! PHV        : RESISTANCE AU FLUX D'EVAPOTRANSPIRATION.
! PWFC       : TENEUR EN EAU A LA CAPACITE AU CHAMPS.

! - 1D (DIAGNOSTIQUE) .

! PNEIJG     : PROPORTION DE SOL RECOUVERTE DE NEIGE.
! PNEIJV     : PROPORTION DE VEGETATION RECOUVERTE DE NEIGE.

! - 1D (GEOGRAPHIQUE) .

! PGZ0F      : CHAMP TEMPOREL DE G FOIS LA LONGUEUR DE RUGOSITE.
! PGZ0HF     : CHAMP TEMPOREL DE G FOIS LA LONGUEUR DE RUGOSITE THERM.
! PGZ0RLF    : GRAVITE * LONGUEUR DE RUGOSITE DUE AU RELIEF.
! PLSM       : INDICE TERRE/MER.
! PVEG0      : PROPORTION DE SOL COUVERTE DE VEGETATION.

! - LOGIQUES .

! LDCLS      : DETERMINATION COMPLETE DES CARACTERISTIQUES DE SURFACE.
! LDHMT      : CALCULS NECESSAIRES A L'INTERPOLATION AUX HAUTEURS METEO
!              DU VENT, DE LA TEMPERATURE ET DE L'HUMIDITE.
! LDZ0H      : DIFFERENTES LONGUEURS DE RUGOSITE DYN/THER.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PNBVNO     : CARRE DE FR. BRUNT-VAISALA DIVISEE PAR G FOIS LA DENSITE.

! - 1D (DIAGNOSTIQUE) .

! PCD        : COEFFICIENT D'ECHANGE EN SURFACE POUR U ET V.
! PCDN       : COEFFICIENT NEUTRE D'ECHANGE EN SURFACE.
! PCDROV     : PCD RENORME EN DENSITE FOIS VITESSE.
! PCH        : COEFFICIENT D'ECHANGE EN SURFACE POUR T ET Q.
! PCHROV     : PCH RENORME EN DENSITE FOIS VITESSE.
! PCPS       : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR AU SOL.
! PDQSTS     : DERIVEE DE PQSATS PAR RAPPORT A LA TEMPERATURE.
! PGWDCS     : DENSITE EN SURFACE POUR LE DRAG OROGRAPHIQUE.
! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.
! PHQ        : POIDS DE L'HUMIDITE DE L'AIR DANS L'HUMIDITE DE SURFACE.
! PHU        : POIDS DE L'HUMIDITE SATURANTE DANS L'HUMIDITE DE SURFACE.
! PNEIJ      : PROPORTION DE LA MAILLE RECOUVERTE DE NEIGE.
! PQCLS      : SORTIE DIAGNOSTIQUE DE L'HUMIDITE SPECIFIQUE A HTQ METEO.
! PQS        : HUMIDITE SPECIFIQUE DE SURFACE.
! PQSATS     : HUMIDITE SPECIFIQUE DE SATURATION EN SURFACE.
! PRHCLS     : SORTIE DIAGNOSTIQUE DE L'HUMIDITE RELATIVE A HTQ METEO.
! PRS        : CONSTANTE DES GAZ POUR L'AIR AU SOL.
! PSTAB      : INDICE DE STABILITE A LA SURFACE.
! PTCLS      : SORTIE DIAGNOSTIQUE DE LA TEMPERATURE A HTQ METEO.
! PUCLS      : SORTIE DIAGNOSTIQUE DU VENT EN X A HUV METEO.
! PVCLS      : SORTIE DIAGNOSTIQUE DU VENT EN Y A HUV METEO.
! PUCLN      : SORTIE DIAGNOSTIQUE DU VENT NEUTRE EN X A HUV METEO.
! PVCLN      : SORTIE DIAGNOSTIQUE DU VENT NEUTRE EN Y A HUV METEO.
! PZPCLS     : SORTIE DIAGNOSTIQUE DE PRESSION A HTQ METEO.
! PVEG       : PROPORTION DE VEGETATION APPARENTE (NON RECOUVERTE DE NEIGE).
! PXDROV     : MULTIPLICATEUR "ANTI-FIBRILLATION" DE PCDROV.
! PXHROV     : MULTIPLICATEUR "ANTI-FIBRILLATION" DE PCHROV.
!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY1/
! COMMON/FCTTRM /

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     *ACNTCLS*
!     *ACNTCLSAD*

!     Methode.
!     --------

!     Auteur.
!     -------
!        99-07, F. Bouyssel.

!     Modifications
!     -------------
!        2001-01, New use of the Richardson critical number (USURID) - F. Bouyssel.
!        R. El Khatib : 01-08-07 Pruning options
!        04-06, introduction of EDK for Fm and Fh in stable case - F Bouyssel
!      2008-11, Vent Neutre - C Payan
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LNEIGE   ,LSNV     ,LSOLV    ,LFGEL
USE YOMCST   , ONLY : RPI      ,RG       ,RD       ,RV       ,&
 & RCPD     ,RCPV     ,RKAPPA   ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMPHY0  , ONLY : EDB      ,EDC      ,EDD      ,USURIC   ,VKARMN,&
 & USURICL  , USURID  ,EDK, VZIUSTAR0
USE YOMPHY1  , ONLY : GCZ0H    ,ALRCN1   ,ALRCN2   ,GCONV    ,&
 & RD1      ,WCRIN    ,WSMX     ,RZHZ0M  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM)               :: KTDIA ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0F5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJG5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSI5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHIV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0F(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0HF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0RLF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHV(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEIJG(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEIJV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSNS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWFC(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWSI(KLON) 
LOGICAL           ,INTENT(IN)    :: LDCLS 
LOGICAL           ,INTENT(IN)    :: LDHMT 
LOGICAL           ,INTENT(IN)    :: LDZ0H 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNBVNO(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCD(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCH(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCHROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDQSTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGWDCS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHQ(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHU(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEIJ(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSATS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRHCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTAB(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZPCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVEG(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXDROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXHROV(KLON) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
REAL(KIND=JPRB) :: ZCDNH(KLON),ZDELT(KLON),ZDPHI(KLON),ZESP(KLON)&
 & ,ZRTI(KLON),ZSTA(KLON),ZU(KLON),ZWSMX(KLON),ZGZ0H(KLON)&
 & ,ZSTAC(KLON),ZSTABV(KLON),ZSTAH(KLON),ZSTAHC(KLON)  

REAL(KIND=JPRB) ::ZPCD5(KLON),ZPCDN5(KLON),ZPCH5(KLON)&
 & ,ZPCPS5(KLON)&
 & ,ZPGZ05(KLON),ZPHQ5(KLON),ZPHU5(KLON),ZPNEIJ5(KLON)&
 & ,ZPQS5(KLON),ZPQSATS5(KLON),ZPRS5(KLON),ZPVEG5(KLON)
REAL(KIND=JPRB) :: ZCDNH5(KLON),ZDPHI5(KLON),ZESP5(KLON)&
 & ,ZRTI5(KLON),ZSTA5(KLON),ZU5(KLON),ZGZ0H5(KLON)&
 & ,ZSTAC5(KLON),ZSTABV5(KLON),ZWS5(KLON)&
 & ,ZSTAH5(KLON),ZHS5(KLON),ZSTAHC5(KLON)  

REAL(KIND=JPRB) :: ZPHU15(KLON),ZPHQ15(KLON),ZSTA15(KLON)
REAL(KIND=JPRB) :: ZEW5(KLON),ZWFC(KLON),ZDEW(KLON),ZRZD5(KLON),ZRZH5(KLON)&
 & ,ZMU5(KLON),ZCD05(KLON),ZPD5(KLON),ZCH05(KLON),ZPH5(KLON)&
 & ,ZCD5(KLON),ZCH5(KLON),ZCIS5(KLON)&
 & ,ZDS5(KLON),ZDID5(KLON),ZDIH5(KLON),ZLOS15(KLON)&
 & ,ZLOI15(KLON),ZLOS5(KLON),ZLOI5(KLON),ZDLEW5(KLON)

INTEGER(KIND=JPIM) :: JLON

REAL(KIND=JPRB) :: Z0CR, Z2B, Z3B, Z3BC, &
 & ZCD, &
 & ZCD0, ZCH, ZCH0, ZCIS, &
 & ZCPVMD, ZDID, &
 & ZDIH, ZDLEW, ZDS, ZEPS1, ZEW, &
 & ZLOI, ZLOS, &
 & ZMU, ZPD, ZPH, ZRVMD, ZRZD, &
 & ZRZH, ZUZ0CN, ZWS, &
 & ZUSURIC, ZHS, ZIXP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "acntclsad.intfb.h"

#include "fcttrm.h"
#include "fcttrmad.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACHMTAD',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'ACHMTAD') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!                        TRAJECTOIRE
!                        TRAJECTORY
!     ------------------------------------------------------------------
!     I - CONSTANTES AUXILIAIRES.

!         AUXILIARY CONSTANTS.

ZEPS1=1.E-04_JPRB

ZCPVMD=RCPV-RCPD
ZRVMD=RV-RD
ZUSURIC=USURIC*USURICL

!*
!     ------------------------------------------------------------------
!     II - CALCUL DES PARAMETRES DERIVES ET CONSTANTE DE SECURITE (POUR
!     LE CARRE DU CISAILLEMENT DE VENT).

!          COMPUTATION OF DERIVED PARAMETERS AND SECURITY CONSTANT (FOR
!     THE SQUARE OF THE WIND SHEAR).

Z2B=2.0_JPRB*EDB
Z3B=3._JPRB*EDB
Z3BC=3._JPRB*EDB*EDC

!*
!     ------------------------------------------------------------------
!     III - PASSAGE EN GEOPOTENTIEL POUR LES CONSTANTES RELATIVES A LA
!     LONGUEUR DE RUGOSITE EN CAS DE NEIGE.

!           GOING TO GEOPOTENTIAL FOR CONSTANTS RELATIVE TO ROUGHNESS
!     LENGTH IN CASE OF SNOW COVER.

Z0CR=RG*ALRCN1
ZUZ0CN=1.0_JPRB/(RG*ALRCN2)

!*
!     ------------------------------------------------------------------
!     IV - CALCUL DE L EPAISSEUR DE LA CLS (EN GEOPOTENTIEL).

!          COMPUTATION OF THE THICKNESS OF THE SBL (IN GEOPOTENTIAL).

! - TEMPORAIRE(S) 1D .

! ZDPHI     : EPAISSEUR EN GEOPOTENTIEL DU NIVEAU DE SURFACE.
!            : GEOPOTENTIAL THICKNESS OF THE SURFACE LEVEL.

DO JLON=KIDIA,KFDIA
  ZDPHI5(JLON)=PAPHIF5(JLON,KLEV)-PAPHI5(JLON,KLEV)
ENDDO

!*
!     ------------------------------------------------------------------
!     V - MISE A UNE VALEUR CONSTANTE DU CONTENU MAXIMUM EN EAU DU
!     RESERVOIR DE SURFACE (PHYSIQUE DU SOL HYPER-SIMPLIFIEE).

!         SETTING TO A CONSTANT VALUE THE MAXIMUM CONTENT OF THE
!     SURFACE WATER RESERVOIR (HYPER-SIMPLIFIED SOIL PHYSICS).

! - TEMPORAIRE(S) 1D .

! ZWSMX     : CONTENU MAXIMAL EN EAU DU RESERVOIR SUPERFICIEL.
!            : MAXIMUM CONTENT OF THE SURFACE RESERVOIR.

IF (.NOT.LSOLV) THEN
  DO JLON=KIDIA,KFDIA
    ZWSMX(JLON)=WSMX
  ENDDO
ENDIF

!*
!     ------------------------------------------------------------------
!     VI - CALCULS PROPREMENT DITS DES CARACTERISTIQUES DE SURFACE.

!          EFFECTIVE CALCULATIONS OF THE SURFACE CHARACTERISTICS.

IF(USURID == 0.0_JPRB) THEN
  ZIXP=1.0_JPRB
ELSE
  ZIXP=3._JPRB
ENDIF

DO JLON=KIDIA,KFDIA

!     CALCULS CONCERNANT LA PRESENCE OU NON DE NEIGE AU SOL.
!     COMPUTATIONS CONCERNING THE PRESENCE OR ABSENCE OF SNOW ON GROUND.

! - TEMPORAIRE(S) 1D .

! ZDELT     : UN POUR CALCULS DE SATURATION SUR GLACE, ZERO SINON.
!            : ONE FOR SATURATION CALCULATION FOR ICE, ZERO OTHERWISE.

  IF (LNEIGE) THEN
    IF (LSNV) THEN
      ZPNEIJ5(JLON)=PLSM(JLON) *&
       & ( (1.0_JPRB-PVEG0(JLON))*PNEIJG5(JLON)+PVEG0(JLON)*PNEIJV5(JLON) )  
      ZPVEG5(JLON)=(1.0_JPRB-PNEIJV5(JLON))*PVEG0(JLON)
      ZPGZ05(JLON)=SQRT((1.0_JPRB-PNEIJV5(JLON))*PGZ0F5(JLON)**2+&
       & PNEIJV5(JLON)*(Z0CR**2+PGZ0RLF(JLON)**2))  
    ELSE
      ZPNEIJ5(JLON)=PLSM(JLON)*PSNS5(JLON)/(PSNS5(JLON)+WCRIN)
      ZPVEG5(JLON)=PVEG0(JLON)
      ZPGZ05(JLON)=PGZ0F5(JLON)&
       & +(Z0CR-PGZ0F5(JLON))*PLSM(JLON)*PSNS5(JLON)&
       & /(PSNS5(JLON)+WCRIN*(1+ZUZ0CN*PGZ0F5(JLON)))  
    ENDIF
    ZDELT(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PTS5(JLON)))
  ELSE
    ZPNEIJ5(JLON)=0.0_JPRB
    ZPVEG5 (JLON)=PVEG0 (JLON)
    ZPGZ05 (JLON)=PGZ0F5(JLON)
    ZDELT (JLON)=0.0_JPRB
  ENDIF

!     CAS DE DEUX LONGUEURS DE RUGOSITE (SUR CONTINENT NON TOTALEMENT
!     COUVERT DE NEIGE).
!     CASE OF TWO ROUGHNESS LENGTHS (ON PARTIALLY SNOW-FREE LAND ONLY).

! ZGZ0H      : G FOIS LA LONGUEUR DE RUGOSITE THERMIQUE COURANTE.

  IF (LDZ0H) THEN

    IF (LNEIGE) THEN
      IF (LSNV) THEN
        ZGZ0H5(JLON)=SQRT((1.0_JPRB-PNEIJV5(JLON))*PGZ0HF(JLON)**2+&
         & PNEIJV5(JLON)*(Z0CR**2+&
         & (PGZ0RLF(JLON)*PGZ0HF(JLON)/PGZ0F5(JLON))**2))  
      ELSE
        ZGZ0H5(JLON)=PGZ0HF(JLON)+(Z0CR-PGZ0HF(JLON))*PLSM(JLON)&
         & *PSNS5(JLON)/(PSNS5(JLON)+WCRIN*(1+ZUZ0CN &
         & *PGZ0HF(JLON)))  
      ENDIF
    ELSE
      ZGZ0H5(JLON)=PGZ0HF(JLON)
    ENDIF

! ZGZ0H = ZGZ0 OVER SEA UNLESS REQUIRED IN NAMPHY1 (CLIMATE RUNS)

    IF (VZIUSTAR0 > 0) THEN
       ZGZ0H5(JLON)=PLSM(JLON)*ZGZ0H5(JLON)+(1.0_JPRB-PLSM(JLON))*PGZ0HF(JLON)
    ELSE
       ZGZ0H5(JLON)=PLSM(JLON)*ZGZ0H5(JLON)+(1.0_JPRB-PLSM(JLON))*ZPGZ05(JLON)*RZHZ0M
    ENDIF   
       

  ENDIF

!     CALCUL DE SATURATION UTILISANT LES FONCTIONS DEFINIES.
!     SATURATION CALCULATION WITH CALL TO DEFINED FUNCTIONS.

! - TEMPORAIRE(S) 1D .

! ZESP      : PRESSION DE VAPEUR SATURANTE SUR PRESSION.
!            : SATURATION WATER VAPOUR PRESSURE OVER PRESSURE.

  ZEW5(JLON)= FOEW   (PTS5(JLON),ZDELT(JLON))
  ZESP5(JLON)=ZEW5(JLON)/PAPRS5(JLON,KLEV)
  ZPQSATS5(JLON)= FOQS   (ZESP5(JLON))

!     CALCULS RELATIFS A L'HUMIDITE DE SURFACE.
!     COMPUTATIONS CONCERNING SURFACE HUMIDITY.

!     CALCULS OPTIONNELS SUIVANT LE TYPE DE SCHEMA SOL/VEG CHOISI.
!     OPTIONAL COMPUTATIONS ACCORDING TO THE CHOSEN SOIL/VEG SCHEME.

  IF (LSOLV) THEN
    ZWFC(JLON)=PWFC(JLON)*RD1*GCONV
    IF (LFGEL) THEN
      ZWS5(JLON)=PWS5(JLON)+PWSI5(JLON)
    ELSE
      ZWS5(JLON)=PWS5(JLON)
    ENDIF
    IF (LSNV) THEN
      IF (ZWS5(JLON) < ZWFC(JLON)) THEN
        ZPHU15(JLON)=1.0_JPRB+PLSM(JLON)*(1.0_JPRB-PNEIJG5(JLON))*&
         & (0.5_JPRB*(1.0_JPRB-COS(RPI*ZWS5(JLON)/ZWFC(JLON)))-1.0_JPRB)  
      ELSE
        ZPHU15(JLON)=1.0_JPRB
      ENDIF
      ZPHU5(JLON)=ZPHU15(JLON)
      IF (MIN(PQ5(JLON,KLEV)/ZPQSATS5(JLON),1.0_JPRB) > ZPHU15(JLON))THEN
        IF ((PQ5(JLON,KLEV)/ZPQSATS5(JLON)) < (1.0_JPRB)) THEN
          ZPHU5(JLON)=PQ5(JLON,KLEV)/ZPQSATS5(JLON)
        ELSE
          ZPHU5(JLON)=1.0_JPRB
        ENDIF
      ENDIF
      ZPQS5(JLON)= (ZPVEG5(JLON)*PHV5(JLON)&
       & +PVEG0(JLON)*PNEIJV5(JLON)&
       & +(1.0_JPRB-PVEG0(JLON))*ZPHU5(JLON))*ZPQSATS5(JLON)&
       & +(1.0_JPRB-PHV5(JLON))*ZPVEG5(JLON)* PQ5(JLON,KLEV)  
    ELSE
      IF (ZWS5(JLON) < ZWFC(JLON)) THEN
        ZPHU15(JLON)=1.0_JPRB+PLSM(JLON)*(1.0_JPRB-ZPNEIJ5(JLON))*&
         & (0.5_JPRB*(1.0_JPRB-COS(RPI*ZWS5(JLON)/ZWFC(JLON)))-1.0_JPRB)  
      ELSE
        ZPHU15(JLON)=1.0_JPRB
      ENDIF
      ZPHU5(JLON)=ZPHU15(JLON)
      IF (MIN(PQ5(JLON,KLEV)/ZPQSATS5(JLON),1.0_JPRB) > ZPHU15(JLON))THEN
        IF ((PQ5(JLON,KLEV)/ZPQSATS5(JLON)) < (1.0_JPRB)) THEN
          ZPHU5(JLON)=PQ5(JLON,KLEV)/ZPQSATS5(JLON)
        ELSE
          ZPHU5(JLON)=1.0_JPRB
        ENDIF
      ENDIF
      ZPQS5(JLON)=(ZPVEG5(JLON)*PHV5(JLON)&
       & +(1.0_JPRB-ZPVEG5(JLON))*ZPHU5(JLON))&
       & *ZPQSATS5(JLON)&
       & +(1.0_JPRB-PHV5(JLON))*ZPVEG5(JLON)*PQ5(JLON,KLEV)  
    ENDIF
  ELSE
    ZPHU15(JLON)=1.0_JPRB+PLSM(JLON)*(1.0_JPRB-ZPNEIJ5(JLON))&
     & *(0.5_JPRB*(1.0_JPRB-COS(RPI*PWS5(JLON)/ZWSMX(JLON)))-1.0_JPRB)  
    ZPHQ15(JLON)=(1.0_JPRB-ZPHU15(JLON))*ZPVEG5(JLON)
    ZDEW(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,PQ5(JLON,KLEV)-ZPQSATS5(JLON)))
    ZPHU5(JLON)=ZPHU15(JLON)+ZDEW(JLON)*ZPHQ15(JLON)
    ZPHQ5(JLON)=ZPHQ15(JLON)-ZDEW(JLON)*ZPHQ15(JLON)
    ZPQS5(JLON)=ZPHU5(JLON)*ZPQSATS5(JLON)+ZPHQ5(JLON)*PQ5(JLON,KLEV)
  ENDIF

  ZPCPS5(JLON)=RCPD+ZCPVMD*ZPQS5(JLON)
  ZPRS5(JLON)=RD+ZRVMD*ZPQS5(JLON)

!     CALCULS GEOMETRIQUES.
!     GEOMETRIC CALCULATIONS.

  ZRZD5(JLON)=1.0_JPRB+ZDPHI5(JLON)/ZPGZ05(JLON)
  ZPCDN5(JLON)=(VKARMN/LOG(ZRZD5(JLON)))**2

!     CALCUL PRELIMINAIRES NECESSAIRES A LA DETERMINATION DES
!     COEFFICIENTS D'ECHANGE CD ET CH DANS LE CAS DE DEUX RUGOSITES
!     DYNAMIQUE Z0 ET THERMIQUE ZOH.

!     PRELIMINARY COMPUTATIONS NECESSARY FOR THE DETERMINATION OF
!     EXCHANGE COEFFICIENTS CD AND CH IN THE CASE OF TWO ROUGHNESS
!     LENGTHS Z0 DYNAMICAL AND ZOH THERMAL.

! - TEMPORAIRE(S) 1D .

! ZCDNH     : COEFFICIENT THERMIQUE NEUTRE D'ECHANGE EN SURFACE.
!            : NEUTRAL SURFACE THERMAL EXCHANGE COEFFICIENT.

  IF (LDZ0H) THEN
    ZRZH5(JLON)=1.0_JPRB+ZDPHI5(JLON)/ZGZ0H5(JLON)
    ZCDNH5(JLON)=VKARMN**2/(LOG(ZRZH5(JLON))*LOG(ZRZD5(JLON)))
    ZMU5(JLON)=LOG(ZPGZ05(JLON)/ZGZ0H5(JLON))
    ZCD05(JLON)=(GCZ0H(0,1)+ZMU5(JLON)*(GCZ0H(1,1)&
     & +ZMU5(JLON)*(GCZ0H(2,1)+ZMU5(JLON)&
     & *GCZ0H(3,1))))/(1.5_JPRB*EDC)  
    ZPD5(JLON)=(GCZ0H(0,2)+ZMU5(JLON)*(GCZ0H(1,2)&
     & +ZMU5(JLON)*(GCZ0H(2,2)+ZMU5(JLON)&
     & *GCZ0H(3,2))))-0.5_JPRB  
    ZCH05(JLON)=(GCZ0H(0,3)+ZMU5(JLON)*(GCZ0H(1,3)&
     & +ZMU5(JLON)*(GCZ0H(2,3)+ZMU5(JLON)&
     & *GCZ0H(3,3))))/EDC  
    ZPH5(JLON)=(GCZ0H(0,4)+ZMU5(JLON)*(GCZ0H(1,4)&
     & +ZMU5(JLON)*(GCZ0H(2,4)+ZMU5(JLON)&
     & *GCZ0H(3,4))))-0.5_JPRB  
    ZCD5(JLON)=ZCD05(JLON)*ZRZD5(JLON)**ZPD5(JLON)
    ZCH5(JLON)=ZCH05(JLON)*ZRZH5(JLON)**ZPH5(JLON)
    ZCD5(JLON)=1.0_JPRB+PLSM(JLON)*(ZCD5(JLON)-1.0_JPRB)
    ZCH5(JLON)=1.0_JPRB+PLSM(JLON)*(ZCH5(JLON)-1.0_JPRB)
  ELSE
    ZRZH5(JLON)=ZRZD5(JLON)
    ZCDNH5(JLON)=ZPCDN5(JLON)
    ZCD5(JLON)=1.0_JPRB
    ZCH5(JLON)=1.0_JPRB
  ENDIF

!     CISAILLEMENT DE VENT.
!     WIND SHEAR.

! - TEMPORAIRE(S) 1D .

! ZU        : MODULE DU VENT DE SURFACE.
!            : SURFACE WIND SPEED.

  IF ((PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2) <= ZEPS1) THEN
    ZCIS5(JLON)=ZEPS1
  ELSE
    ZCIS5(JLON)=PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2
  ENDIF
  ZU5(JLON)=SQRT(ZCIS5(JLON))

!     CALCUL DE STABILITE.
!     STABILITY COMPUTATION.

! - TEMPORAIRE(S) 1D .

! ZRTI      : INVERSE DE R*T.
!            : INVERSE OF R*T.
! ZSTA      : APPROXIMATION DE DELTA(PHI)*DELTA(LN(THETA)).
!            : APPROXIMATION OF DELTA(PHI)*DELTA(LN(THETA)).

  ZRTI5(JLON)=2.0_JPRB/(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)+ZPRS5(JLON)*PTS5(JLON))  
  ZSTA15(JLON)=ZDPHI5(JLON)*(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)-ZPRS5(JLON)*PTS5(JLON))&
   & *ZRTI5(JLON)  
  ZSTABV5(JLON)=ZSTA15(JLON)
  IF (ZSTA15(JLON) > 0.0_JPRB) THEN
    ZSTAH5(JLON)=ZSTA15(JLON) &
     & /(1.0_JPRB+ZIXP*ZUSURIC*ZSTA15(JLON)/ZCIS5(JLON))**(1.0_JPRB/ZIXP)  
    ZSTA5(JLON)=ZSTA15(JLON)/(1.0_JPRB+ZSTA15(JLON)*ZUSURIC/ZCIS5(JLON))
  ELSE
    ZSTAH5(JLON)=ZSTA15(JLON)
    ZSTA5 (JLON)=ZSTA15(JLON)
  ENDIF
  PSTAB(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZSTA5(JLON)))
  ZSTAC5(JLON)=ABS(ZSTA5(JLON))
  ZSTAHC5(JLON)=ABS(ZSTAH5(JLON))

!     CALCULS COMMUNS POUR QUANTITE DE MOUVEMENT ET ENERGIE.
!     COMMON COMPUTATIONS FOR MOMENTUM AND ENERGY.

  ZDS5(JLON)=SQRT(ZCIS5(JLON)+EDD/EDK*ZSTAC5(JLON))
  ZHS5(JLON)=SQRT(ZCIS5(JLON)+EDD*EDK*ZSTAHC5(JLON))
  ZDID5(JLON)=1.0_JPRB/(ZU5(JLON)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZD5(JLON)))  
  ZDIH5(JLON)=1.0_JPRB/(ZU5(JLON)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))  

!     CALCULS POUR LES COMPOSANTES DU VENT DE CD ET DE SON PRODUIT PAR
!     DENSITE FOIS MODULE DU VENT.
!     COMPUTATIONS FOR MOMENTUM OF CD AND OF ITS PRODUCT BY DENSITY
!     TIME WIND SPEED.

  ZLOS15(JLON)=ZCIS5(JLON)*ZDS5(JLON)/(ZU5(JLON)*ZDS5(JLON)+Z2B*ZSTAC5(JLON))
  ZLOI15(JLON)=ZU5(JLON)-Z2B*ZSTA5(JLON)*ZDID5(JLON)
  ZPCD5(JLON)=(ZLOI15(JLON)+PSTAB(JLON)&
   & *(ZLOS15(JLON)-ZLOI15(JLON)))*ZPCDN5(JLON)/ZU5(JLON)  

!     IDEM POUR LA TEMPERATURE ET L'HUMIDITE.
!     THE SAME FOR TEMPERATURE AND HUMIDITY.

  ZLOS5(JLON)=ZCIS5(JLON)**2/(ZU5(JLON)*ZCIS5(JLON)&
   & +Z3B*ZSTAHC5(JLON)*ZHS5(JLON))  
  ZLOI5(JLON)=ZU5(JLON)-Z3B*ZSTA5(JLON)*ZDIH5(JLON)
  ZPCH5(JLON)=(ZLOI5(JLON)+PSTAB(JLON)*(ZLOS5(JLON)-ZLOI5(JLON)))&
   & *ZCDNH5(JLON)/ZU5(JLON)  

ENDDO

IF (LDCLS) THEN
  DO JLON=KIDIA,KFDIA
    ZDLEW5 (JLON)=FODLEW (PTS5(JLON),ZDELT(JLON))
  ENDDO
ENDIF

!*
!     ------------------------------------------------------------------
!                        INITIALISATION DES VARIABLES LOCALES
!                        INITIALIZATION OF LOCAL VARIABLES
!     ------------------------------------------------------------------

ZCDNH (:) = 0.0_JPRB
ZDPHI (:) = 0.0_JPRB
ZESP  (:) = 0.0_JPRB
ZRTI  (:) = 0.0_JPRB
ZSTA  (:) = 0.0_JPRB
ZSTAH (:) = 0.0_JPRB
ZU    (:) = 0.0_JPRB
ZSTABV(:) = 0.0_JPRB
ZSTAC (:) = 0.0_JPRB
ZSTAHC(:) = 0.0_JPRB
ZGZ0H (:) = 0.0_JPRB

!*
!     ------------------------------------------------------------------
!                        CALCUL DE LA PARTIE ADJOINTE
!                        COMPUTATION OF THE ADJOINT PART
!     ------------------------------------------------------------------

IF (LDHMT) THEN

  CALL ACNTCLSAD ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
   & PAPRS5,PAPRSF5,PCP5,PQ5,PT5,PU5,PV5,&
   & ZPCD5,ZPCDN5,ZCDNH5,ZPCH5,ZPCPS5,ZDPHI5,ZPQS5,PTS5,&
   & PAPRS,PAPRSF,PCP,PQ,PT,PU,PV,&
   & PCD,PCDN,ZCDNH,PCH,PCPS,ZDPHI,PDPHIT,PDPHIV,&
   & PQS,PSTAB,PTS,&
   & LDZ0H,&
   & PQCLS,PRHCLS,PTCLS,PUCLS,PVCLS,PUCLN,PVCLN,PZPCLS,'ACNTCLSAD' )  

ENDIF

IF (LDCLS) THEN
  DO JLON=KIDIA,KFDIA
    ZDLEW=0.0_JPRB

!     CALCULS DE UN SUR G FOIS LA FREQUENCE DE BRUNT-VAISALA DIVISEE PAR
!     LA DENSITE LE TOUT AU CARRE DE MEME QUE LA DENSITE (POUR G.W.D.).
!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY (FOR G.W.D.)

    ZRTI(JLON)=ZRTI(JLON)+PAPRS5(JLON,KLEV)*PGWDCS(JLON)
    PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZRTI5(JLON)*PGWDCS(JLON)
    PGWDCS(JLON)=0.0_JPRB

    ZSTABV(JLON)=ZSTABV(JLON)&
     & +1.0_JPRB/((PAPRS5(JLON,KLEV)*ZRTI5(JLON)&
     & *ZDPHI5(JLON))**2)*PNBVNO(JLON,KLEV)  
    PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)&
     & -2*ZSTABV5(JLON)*ZRTI5(JLON)*ZDPHI5(JLON)&
     & /((PAPRS5(JLON,KLEV)*ZRTI5(JLON)&
     & *ZDPHI5(JLON))**3)*PNBVNO(JLON,KLEV)  
    ZRTI(JLON)=ZRTI(JLON)&
     & -2*ZSTABV5(JLON)*PAPRS5(JLON,KLEV)*ZDPHI5(JLON)&
     & /((PAPRS5(JLON,KLEV)*ZRTI5(JLON)&
     & *ZDPHI5(JLON))**3)*PNBVNO(JLON,KLEV)  
    ZDPHI(JLON)=ZDPHI(JLON)&
     & -2*ZSTABV5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON)&
     & /((PAPRS5(JLON,KLEV)*ZRTI5(JLON)&
     & *ZDPHI5(JLON))**3)*PNBVNO(JLON,KLEV)  
    PNBVNO(JLON,KLEV)=0.0_JPRB

!     CALCUL DE LA DERIVEE PAR RAPPORT A TS DE QSATS ET DES COEFFICIENTS
!     D'ECHANGE MULTIPLIES PAR DENSITE ET MODULE DU VENT.
!     COMPUTATION OF THE DERIVATIVE OF QSATS WITH RESPECT TO TS AND OF
!     THE EXCHANGE COEFFICIENTS MULTIPLIED BY DENSITY AND WIND SPEED.

    PCH(JLON)=PCH(JLON)+ZU5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON)*PCHROV(JLON)
    ZU(JLON)=ZU(JLON)+ZPCH5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON)*PCHROV(JLON)
    PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)&
     & +ZPCH5(JLON)*ZU5(JLON)*ZRTI5(JLON)&
     & *PCHROV(JLON)  
    ZRTI(JLON)=ZRTI(JLON)+ZPCH5(JLON)*ZU5(JLON)*PAPRS5(JLON,KLEV)*PCHROV(JLON)
    PCHROV(JLON)=0.0_JPRB

    PCD(JLON)=PCD(JLON)+ZU5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON)*PCDROV(JLON)
    ZU(JLON)=ZU(JLON)+ZPCD5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON)*PCDROV(JLON)
    PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)&
     & +ZPCD5(JLON)*ZU5(JLON)*ZRTI5(JLON)&
     & *PCDROV(JLON)  
    ZRTI(JLON)=ZRTI(JLON)+ZPCD5(JLON)*ZU5(JLON)*PAPRS5(JLON,KLEV)*PCDROV(JLON)
    PCDROV(JLON)=0.0_JPRB

    PQSATS(JLON)=PQSATS(JLON)&
     & +FODQSAD1(PDQSTS(JLON),ZPQSATS5(JLON),&
     & ZESP5(JLON),ZDLEW5(JLON))  
    ZESP(JLON)=ZESP(JLON)&
     & +FODQSAD2(ZPQSATS5(JLON),PDQSTS(JLON),&
     & ZESP5(JLON),ZDLEW5(JLON))  
    ZDLEW=ZDLEW+FODQSAD3(ZPQSATS5(JLON),ZESP5(JLON),PDQSTS(JLON),ZDLEW5(JLON))
    PDQSTS(JLON)=0.0_JPRB

    PTS(JLON)=PTS(JLON)+FODLEWAD(ZDLEW,PTS5(JLON),ZDELT(JLON))

  ENDDO
ENDIF

!*
!     ------------------------------------------------------------------
!     VI - CALCULS PROPREMENT DITS DES CARACTERISTIQUES DE SURFACE.

!          EFFECTIVE CALCULATIONS OF THE SURFACE CHARACTERISTICS.

DO JLON=KIDIA,KFDIA

  ZEW =0.0_JPRB
  ZWS =0.0_JPRB
  ZRZD=0.0_JPRB
  ZRZH=0.0_JPRB
  ZMU =0.0_JPRB
  ZCD0=0.0_JPRB
  ZPD =0.0_JPRB
  ZCH0=0.0_JPRB
  ZPH =0.0_JPRB
  ZCD =0.0_JPRB
  ZCH =0.0_JPRB
  ZCIS=0.0_JPRB
  ZDS =0.0_JPRB
  ZHS =0.0_JPRB
  ZDID=0.0_JPRB
  ZDIH=0.0_JPRB
  ZLOS=0.0_JPRB
  ZLOI=0.0_JPRB

  PXDROV(JLON)=0.0_JPRB
  PXHROV(JLON)=0.0_JPRB

!     IDEM POUR LA TEMPERATURE ET L'HUMIDITE.
!     THE SAME FOR TEMPERATURE AND HUMIDITY.

  ZLOI=ZLOI+(1.0_JPRB-PSTAB(JLON))*ZCDNH5(JLON)/ZU5(JLON)*PCH(JLON)
  ZLOS=ZLOS+PSTAB(JLON)*ZCDNH5(JLON)/ZU5(JLON)*PCH(JLON)
  ZCDNH(JLON)=ZCDNH(JLON)&
   & +(ZLOI5(JLON)+PSTAB(JLON)*(ZLOS5(JLON)-ZLOI5(JLON)))&
   & /ZU5(JLON)*PCH(JLON)  
  ZU(JLON)=ZU(JLON)&
   & -(ZLOI5(JLON)+PSTAB(JLON)*(ZLOS5(JLON)-ZLOI5(JLON)))&
   & *ZCDNH5(JLON)/ZU5(JLON)**2*PCH(JLON)  
  PCH(JLON)=0.0_JPRB

  ZU(JLON)=ZU(JLON)+ZLOI
  ZDIH=ZDIH-Z3B*ZSTA5(JLON)*ZLOI
  ZSTA(JLON)=ZSTA(JLON)-Z3B*ZDIH5(JLON)*ZLOI
  ZLOI=0.0_JPRB

  ZCIS=ZCIS+ZCIS5(JLON)&
   & *(ZU5(JLON)*ZCIS5(JLON)+2.0_JPRB*Z3B*ZSTAHC5(JLON)*ZHS5(JLON))&
   & /(ZU5(JLON)*ZCIS5(JLON)+Z3B*ZSTAHC5(JLON)*ZHS5(JLON))**2 &
   & *ZLOS  
  ZU(JLON)=ZU(JLON)-ZCIS5(JLON)**3 &
   & /(ZU5(JLON)*ZCIS5(JLON)+Z3B*ZSTAHC5(JLON)*ZHS5(JLON))**2 &
   & *ZLOS  
  ZHS=ZHS &
   & -Z3B*ZSTAHC5(JLON)*ZCIS5(JLON)**2 &
   & /(ZU5(JLON)*ZCIS5(JLON)+Z3B*ZSTAHC5(JLON)*ZHS5(JLON))**2 &
   & *ZLOS  
  ZSTAHC(JLON)=ZSTAHC(JLON)&
   & -Z3B*ZHS5(JLON)*ZCIS5(JLON)**2 &
   & /(ZU5(JLON)*ZCIS5(JLON)+Z3B*ZSTAHC5(JLON)*ZHS5(JLON))**2 &
   & *ZLOS  
  ZLOS=0.0_JPRB

!     CALCULS POUR LES COMPOSANTES DU VENT DE CD ET DE SON PRODUIT PAR
!     DENSITE FOIS MODULE DU VENT.
!     COMPUTATIONS FOR MOMENTUM OF CD AND OF ITS PRODUCT BY DENSITY
!     TIME WIND SPEED.

  ZLOI=ZLOI+(1.0_JPRB-PSTAB(JLON))*ZPCDN5(JLON)/ZU5(JLON)*PCD(JLON)
  ZLOS=ZLOS+PSTAB(JLON)*ZPCDN5(JLON)/ZU5(JLON)*PCD(JLON)
  PCDN(JLON)=PCDN(JLON)&
   & +(ZLOI15(JLON)+PSTAB(JLON)*(ZLOS15(JLON)-ZLOI15(JLON)))&
   & /ZU5(JLON)*PCD(JLON)  
  ZU(JLON)=ZU(JLON)&
   & -(ZLOI15(JLON)+PSTAB(JLON)*(ZLOS15(JLON)-ZLOI15(JLON)))&
   & *ZPCDN5(JLON)/ZU5(JLON)**2*PCD(JLON)  
  PCD(JLON)=0.0_JPRB

  ZU(JLON)=ZU(JLON)+ZLOI
  ZDID=ZDID-Z2B*ZSTA5(JLON)*ZLOI
  ZSTA(JLON)=ZSTA(JLON)-Z2B*ZDID5(JLON)*ZLOI

  ZCIS=ZCIS+ZDS5(JLON)/(ZU5(JLON)*ZDS5(JLON)+Z2B*ZSTAC5(JLON))*ZLOS
  ZDS=ZDS+ZCIS5(JLON)*Z2B*ZSTAC5(JLON)&
   & /(ZU5(JLON)*ZDS5(JLON)+Z2B*ZSTAC5(JLON))**2*ZLOS  
  ZU(JLON)=ZU(JLON)-ZCIS5(JLON)*ZDS5(JLON)**2 &
   & /(ZU5(JLON)*ZDS5(JLON)+Z2B*ZSTAC5(JLON))**2*ZLOS  
  ZSTAC(JLON)=ZSTAC(JLON)-Z2B*ZCIS5(JLON)*ZDS5(JLON)&
   & /(ZU5(JLON)*ZDS5(JLON)+Z2B*ZSTAC5(JLON))**2*ZLOS  

!     CALCULS COMMUNS POUR QUANTITE DE MOUVEMENT ET ENERGIE.
!     COMMON COMPUTATIONS FOR MOMENTUM AND ENERGY.

  ZU(JLON)=ZU(JLON)&
   & -1.0_JPRB/(ZU5(JLON)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))**2*ZDIH  
  ZCH=ZCH &
   & -Z3BC*ZCDNH5(JLON)*SQRT(ZSTAC5(JLON)*ZRZH5(JLON))&
   & /(ZU5(JLON)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))**2*ZDIH  
  ZCDNH(JLON)=ZCDNH(JLON)&
   & -ZCH5(JLON)*Z3BC*SQRT(ZSTAC5(JLON)*ZRZH5(JLON))&
   & /(ZU5(JLON)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))**2*ZDIH  
  ZRZH=ZRZH &
   & -ZCH5(JLON)*Z3BC*ZCDNH5(JLON)*ZSTAC5(JLON)&
   & /(2.0_JPRB*SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))&
   & /(ZU5(JLON)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))**2*ZDIH  
  ZSTAC(JLON)=ZSTAC(JLON)&
   & -ZCH5(JLON)*Z3BC*ZCDNH5(JLON)*ZRZH5(JLON)&
   & /(2.0_JPRB*SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))&
   & /(ZU5(JLON)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZH5(JLON)))**2*ZDIH  

  ZU(JLON)=ZU(JLON)&
   & -1.0_JPRB/(ZU5(JLON)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)&
   & *SQRT(ZSTAC5(JLON)*ZRZD5(JLON)))**2*ZDID  
  ZCD=ZCD &
   & -Z3BC*ZPCDN5(JLON)*SQRT(ZSTAC5(JLON)*ZRZD5(JLON))&
   & /(ZU5(JLON)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)*SQRT(ZSTAC5(JLON)&
   & *ZRZD5(JLON)))**2*ZDID  
  PCDN(JLON)=PCDN(JLON)&
   & -ZCD5(JLON)*Z3BC*SQRT(ZSTAC5(JLON)*ZRZD5(JLON))&
   & /(ZU5(JLON)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)*SQRT(ZSTAC5(JLON)&
   & *ZRZD5(JLON)))**2*ZDID  
  ZRZD=ZRZD &
   & -ZCD5(JLON)*Z3BC*ZPCDN5(JLON)*ZSTAC5(JLON)&
   & /(2.0_JPRB*SQRT(ZSTAC5(JLON)*ZRZD5(JLON)))&
   & /(ZU5(JLON)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)*SQRT(ZSTAC5(JLON)&
   & *ZRZD5(JLON)))**2*ZDID  
  ZSTAC(JLON)=ZSTAC(JLON)&
   & -ZCD5(JLON)*Z3BC*ZPCDN5(JLON)*ZRZD5(JLON)&
   & /(2.0_JPRB*SQRT(ZSTAC5(JLON)*ZRZD5(JLON)))&
   & /(ZU5(JLON)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)*SQRT(ZSTAC5(JLON)&
   & *ZRZD5(JLON)))**2*ZDID  

  ZCIS=ZCIS+1.0_JPRB/(2.0_JPRB*SQRT(ZCIS5(JLON)+EDD*EDK*ZSTAHC5(JLON)))*ZHS
  ZSTAHC(JLON)=ZSTAHC(JLON)+EDD*EDK &
   & /(2.0_JPRB*SQRT(ZCIS5(JLON)+EDD*EDK*ZSTAHC5(JLON)))*ZHS

  ZCIS=ZCIS+1.0_JPRB/(2.0_JPRB*SQRT(ZCIS5(JLON)+EDD/EDK*ZSTAC5(JLON)))*ZDS
  ZSTAC(JLON)=ZSTAC(JLON)+EDD/EDK &
   & /(2.0_JPRB*SQRT(ZCIS5(JLON)+EDD/EDK*ZSTAC5(JLON)))*ZDS

!     CALCUL DE STABILITE.
!     STABILITY COMPUTATION.

! - TEMPORAIRE(S) 1D .

! ZRTI      : INVERSE DE R*T.
!            : INVERSE OF R*T.
! ZSTA      : APPROXIMATION DE DELTA(PHI)*DELTA(LN(THETA)).
!            : APPROXIMATION OF DELTA(PHI)*DELTA(LN(THETA)).

  IF (ZSTAH5(JLON) < 0.0_JPRB) THEN
    ZSTAH(JLON)=ZSTAH(JLON)-ZSTAHC(JLON)
    ZSTAHC(JLON)=0.0_JPRB
  ELSE
    ZSTAH(JLON)=ZSTAH(JLON)+ZSTAHC(JLON)
    ZSTAHC(JLON)=0.0_JPRB
  ENDIF

  IF (ZSTA5(JLON) < 0.0_JPRB) THEN
    ZSTA(JLON)=ZSTA(JLON)-ZSTAC(JLON)
    ZSTAC(JLON)=0.0_JPRB
  ELSE
    ZSTA(JLON)=ZSTA(JLON)+ZSTAC(JLON)
    ZSTAC(JLON)=0.0_JPRB
  ENDIF

  IF (ZSTA15(JLON) > 0.0_JPRB) THEN
    ZCIS=ZCIS &
     & +ZUSURIC*(ZSTA15(JLON)/ZCIS5(JLON))**2 &
     & /(1.0_JPRB+ZSTA15(JLON)*ZUSURIC/ZCIS5(JLON))**2*ZSTA(JLON)  
    ZSTA(JLON)=(1.0_JPRB/(1.0_JPRB+ZSTA15(JLON)*ZUSURIC/ZCIS5(JLON))**2)*ZSTA(JLON)
    
    ZSTA(JLON)=ZSTA(JLON)+ZSTAH(JLON) &
     & /(1.0_JPRB+ZIXP*ZUSURIC*ZSTA15(JLON)/ZCIS5(JLON))**(1.0_JPRB/ZIXP)  
    ZSTA(JLON)=ZSTA(JLON)-ZSTAH(JLON) &
     & *ZUSURIC*ZSTA15(JLON)*ZCIS5(JLON) &
     & /((1.0_JPRB+ZIXP*ZUSURIC*ZSTA15(JLON)/ZCIS5(JLON))*ZCIS5(JLON)**2) &
     & /(1.0_JPRB+ZIXP*ZUSURIC*ZSTA15(JLON)/ZCIS5(JLON))**(1.0_JPRB/ZIXP)  
    ZCIS=ZCIS+ZSTAH(JLON) &
     & *ZUSURIC*ZSTA15(JLON)**2 &
     & /((1.0_JPRB+ZIXP*ZUSURIC*ZSTA15(JLON)/ZCIS5(JLON))*ZCIS5(JLON)**2) &
     & /(1.0_JPRB+ZIXP*ZUSURIC*ZSTA15(JLON)/ZCIS5(JLON))**(1.0_JPRB/ZIXP)  
    ZSTAH(JLON)=0.0_JPRB
  ELSE
    ZSTA (JLON)=ZSTA (JLON)+ZSTAH (JLON)
    ZSTAH(JLON)=0.0_JPRB
  ENDIF

  ZSTA(JLON)=ZSTA(JLON)+ZSTABV(JLON)
  ZSTABV(JLON)=0.0_JPRB

  ZDPHI(JLON)=ZDPHI(JLON)&
   & +(PR5(JLON,KLEV)*PT5(JLON,KLEV)+2.0_JPRB*RKAPPA*ZDPHI5(JLON)&
   & -ZPRS5(JLON)*PTS5(JLON))*ZRTI5(JLON)*ZSTA(JLON)  
  ZRTI(JLON)=ZRTI(JLON)&
   & +ZDPHI5(JLON)*(PR5(JLON,KLEV)*PT5(JLON,KLEV)+RKAPPA &
   & *ZDPHI5(JLON)-ZPRS5(JLON)*PTS5(JLON))*ZSTA(JLON)  
  PT(JLON,KLEV)=PT(JLON,KLEV)&
   & +ZDPHI5(JLON)*PR5(JLON,KLEV)*ZRTI5(JLON)*ZSTA(JLON)  
  PR(JLON,KLEV)=PR(JLON,KLEV)&
   & +ZDPHI5(JLON)*PT5(JLON,KLEV)*ZRTI5(JLON)*ZSTA(JLON)  
  PTS(JLON)=PTS(JLON)-ZDPHI5(JLON)*ZPRS5(JLON)*ZRTI5(JLON)*ZSTA(JLON)
  PRS(JLON)=PRS(JLON)-ZDPHI5(JLON)*PTS5(JLON)*ZRTI5(JLON)*ZSTA(JLON)
  ZSTA(JLON)=0.0_JPRB

  PT(JLON,KLEV)=PT(JLON,KLEV)&
   & -2.0_JPRB*PR5(JLON,KLEV)/(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)+ZPRS5(JLON)*PTS5(JLON))**2*ZRTI(JLON)  
  PR(JLON,KLEV)=PR(JLON,KLEV)&
   & -2.0_JPRB*PT5(JLON,KLEV)/(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)+ZPRS5(JLON)*PTS5(JLON))**2*ZRTI(JLON)  
  ZDPHI(JLON)=ZDPHI(JLON)&
   & -2.0_JPRB*RKAPPA/(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)+ZPRS5(JLON)*PTS5(JLON))**2*ZRTI(JLON)  
  PTS(JLON)=PTS(JLON)&
   & -2.0_JPRB*ZPRS5(JLON)/(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)+ZPRS5(JLON)*PTS5(JLON))**2*ZRTI(JLON)  
  PRS(JLON)=PRS(JLON)&
   & -2.0_JPRB*PTS5(JLON)/(PR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON)+ZPRS5(JLON)*PTS5(JLON))**2*ZRTI(JLON)  
  ZRTI(JLON)=0.0_JPRB

!     CISAILLEMENT DE VENT.
!     WIND SHEAR.

! - TEMPORAIRE(S) 1D .

! ZU        : MODULE DU VENT DE SURFACE.
!            : SURFACE WIND SPEED.

  ZCIS=ZCIS+1.0_JPRB/(2.0_JPRB*SQRT(ZCIS5(JLON)))*ZU(JLON)
  ZU(JLON)=0.0_JPRB
  IF ((PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2) > ZEPS1) THEN
    PU(JLON,KLEV)=PU(JLON,KLEV)+2.0_JPRB*PU5(JLON,KLEV)*ZCIS
    PV(JLON,KLEV)=PV(JLON,KLEV)+2.0_JPRB*PV5(JLON,KLEV)*ZCIS
  ENDIF

!     CALCUL PRELIMINAIRES NECESSAIRES A LA DETERMINATION DES
!     COEFFICIENTS D'ECHANGE CD ET CH DANS LE CAS DE DEUX RUGOSITES
!     DYNAMIQUE Z0 ET THERMIQUE ZOH.

!     PRELIMINARY COMPUTATIONS NECESSARY FOR THE DETERMINATION OF
!     EXCHANGE COEFFICIENTS CD AND CH IN THE CASE OF TWO ROUGHNESS
!     LENGTHS Z0 DYNAMICAL AND ZOH THERMAL.

! - TEMPORAIRE(S) 1D .

! ZCDNH     : COEFFICIENT THERMIQUE NEUTRE D'ECHANGE EN SURFACE.
!            : NEUTRAL SURFACE THERMAL EXCHANGE COEFFICIENT.

  IF (LDZ0H) THEN
    ZCH=PLSM(JLON)*ZCH
    ZCD=PLSM(JLON)*ZCD

    ZCH0=ZCH0+ZRZH5(JLON)**ZPH5(JLON)*ZCH
    ZPH=ZPH+ZCH05(JLON)*LOG(ZRZH5(JLON))*ZRZH5(JLON)**ZPH5(JLON)*ZCH
    ZRZH=ZRZH+ZCH05(JLON)*ZPH5(JLON)*ZRZH5(JLON)**(ZPH5(JLON)-1)*ZCH

    ZCD0=ZCD0+ZRZD5(JLON)**ZPD5(JLON)*ZCD
    ZPD=ZPD+ZCD05(JLON)*LOG(ZRZD5(JLON))*ZRZD5(JLON)**ZPD5(JLON)*ZCD
    ZRZD=ZRZD+ZCD05(JLON)*ZPD5(JLON)*ZRZD5(JLON)**(ZPD5(JLON)-1)*ZCD

    ZMU=ZMU+( GCZ0H(1,4)+2.0_JPRB*ZMU5(JLON)*GCZ0H(2,4)&
     & +3._JPRB*ZMU5(JLON)**2*GCZ0H(3,4) )*ZPH  

    ZMU=ZMU+( GCZ0H(1,3)+2.0_JPRB*ZMU5(JLON)*GCZ0H(2,3)&
     & +3._JPRB*ZMU5(JLON)**2*GCZ0H(3,3) )/EDC*ZCH0  

    ZMU=ZMU+( GCZ0H(1,2)+2.0_JPRB*ZMU5(JLON)*GCZ0H(2,2)&
     & +3._JPRB*ZMU5(JLON)**2*GCZ0H(3,2) )*ZPD  

    ZMU=ZMU+( GCZ0H(1,1)+2.0_JPRB*ZMU5(JLON)*GCZ0H(2,1)&
     & +3._JPRB*ZMU5(JLON)**2*GCZ0H(3,1) )/(1.5_JPRB*EDC)*ZCD0  

    PGZ0(JLON)=PGZ0(JLON)+1.0_JPRB/ZPGZ05(JLON)*ZMU
    ZGZ0H(JLON)=ZGZ0H(JLON)-1.0_JPRB/ZGZ0H5(JLON)*ZMU

    ZRZD=ZRZD &
     & -VKARMN**2*LOG(ZRZH5(JLON))/ZRZD5(JLON)&
     & /(LOG(ZRZH5(JLON))*LOG(ZRZD5(JLON)))**2*ZCDNH(JLON)  
    ZRZH=ZRZH &
     & -VKARMN**2*LOG(ZRZD5(JLON))/ZRZH5(JLON)&
     & /(LOG(ZRZH5(JLON))*LOG(ZRZD5(JLON)))**2*ZCDNH(JLON)  
    ZCDNH(JLON)=0.0_JPRB

    ZDPHI(JLON)=ZDPHI(JLON)+1.0_JPRB/ZGZ0H5(JLON)*ZRZH
    ZGZ0H(JLON)=ZGZ0H(JLON)-ZDPHI5(JLON)/ZGZ0H5(JLON)**2*ZRZH
  ELSE
    PCDN(JLON)=PCDN(JLON)+ZCDNH(JLON)
    ZCDNH(JLON)=0.0_JPRB
    ZRZD=ZRZD+ZRZH
  ENDIF

!     CALCULS GEOMETRIQUES.
!     GEOMETRIC CALCULATIONS.

  ZRZD=ZRZD-2*VKARMN**2/(ZRZD5(JLON)*(LOG(ZRZD5(JLON)))**3)*PCDN(JLON)
  PCDN(JLON)=0.0_JPRB

  ZDPHI(JLON)=ZDPHI(JLON)+1.0_JPRB/ZPGZ05(JLON)*ZRZD
  PGZ0(JLON)=PGZ0(JLON)-ZDPHI5(JLON)/ZPGZ05(JLON)**2*ZRZD

!     CALCULS RELATIFS A L'HUMIDITE DE SURFACE.
!     COMPUTATIONS CONCERNING SURFACE HUMIDITY.

!     CALCULS OPTIONNELS SUIVANT LE TYPE DE SCHEMA SOL/VEG CHOISI.
!     OPTIONAL COMPUTATIONS ACCORDING TO THE CHOSEN SOIL/VEG SCHEME.

  PQS(JLON)=PQS(JLON)+ZRVMD*PRS(JLON)
  PRS(JLON)=0.0_JPRB
  PQS(JLON)=PQS(JLON)+ZCPVMD*PCPS(JLON)
  PCPS(JLON)=0.0_JPRB

  IF (LSOLV) THEN
    IF (LSNV) THEN
      PVEG(JLON)=PVEG(JLON)+(PHV5(JLON)*ZPQSATS5(JLON)&
       & +(1.0_JPRB-PHV5(JLON))*PQ5(JLON,KLEV))*PQS(JLON)  
      PHV(JLON)=PHV(JLON)&
       & +ZPVEG5(JLON)*(ZPQSATS5(JLON)-PQ5(JLON,KLEV))*PQS(JLON)  
      PNEIJV(JLON)=PNEIJV(JLON)+PVEG0(JLON)*ZPQSATS5(JLON)*PQS(JLON)
      PHU(JLON)=PHU(JLON)+(1.0_JPRB-PVEG0(JLON))*ZPQSATS5(JLON)*PQS(JLON)
      PQSATS(JLON)=PQSATS(JLON)&
       & +(ZPVEG5(JLON)*PHV5(JLON)+PVEG0(JLON)*PNEIJV5(JLON)&
       & +(1.0_JPRB-PVEG0(JLON))*ZPHU5(JLON))*PQS(JLON)  
      PQ(JLON,KLEV)=PQ(JLON,KLEV)+(1.0_JPRB-PHV5(JLON))*ZPVEG5(JLON)*PQS(JLON)
      PQS(JLON)=0.0_JPRB
      IF (MIN(PQ5(JLON,KLEV)/ZPQSATS5(JLON),1.0_JPRB) > ZPHU15(JLON))THEN
        IF ((PQ5(JLON,KLEV)/ZPQSATS5(JLON)) < (1.0_JPRB)) THEN
          PQ(JLON,KLEV)=PQ(JLON,KLEV)+1.0_JPRB/ZPQSATS5(JLON)*PHU(JLON)
          PQSATS(JLON)=PQSATS(JLON)&
           & -PQ5(JLON,KLEV)/ZPQSATS5(JLON)**2*PHU(JLON)  
          PHU(JLON)=0.0_JPRB
        ELSE
          PHU(JLON)=0.0_JPRB
        ENDIF
      ENDIF
      IF (ZWS5(JLON) < ZWFC(JLON)) THEN
        PNEIJG(JLON)=PNEIJG(JLON)&
         & -PLSM(JLON)*(0.5_JPRB*(1.0_JPRB-COS(RPI*ZWS5(JLON)&
         & /ZWFC(JLON)))-1.0_JPRB)*PHU(JLON)  
        ZWS=ZWS&
         & +PLSM(JLON)*(1.0_JPRB-PNEIJG5(JLON))*0.5_JPRB*RPI/ZWFC(JLON)&
         & *SIN(RPI*ZWS5(JLON)/ZWFC(JLON))*PHU(JLON)  
        PHU(JLON)=0.0_JPRB
      ELSE
        PHU(JLON)=0.0_JPRB
      ENDIF
    ELSE
      PVEG(JLON)=PVEG(JLON)&
       & +( (PHV5(JLON)-ZPHU5(JLON))*ZPQSATS5(JLON)&
       & +(1.0_JPRB-PHV5(JLON))*PQ5(JLON,KLEV) )*PQS(JLON)  
      PHV(JLON)=PHV(JLON)&
       & +ZPVEG5(JLON)*(ZPQSATS5(JLON)-PQ5(JLON,KLEV))*PQS(JLON)  
      PHU(JLON)=PHU(JLON)+(1.0_JPRB-ZPVEG5(JLON))*ZPQSATS5(JLON)*PQS(JLON)
      PQSATS(JLON)=PQSATS(JLON)&
       & +(ZPVEG5(JLON)*PHV5(JLON)+(1.0_JPRB-ZPVEG5(JLON))*ZPHU5(JLON))&
       & *PQS(JLON)  
      PQ(JLON,KLEV)=PQ(JLON,KLEV)+(1.0_JPRB-PHV5(JLON))*ZPVEG5(JLON)*PQS(JLON)
      PQS(JLON)=0.0_JPRB
      IF (MIN(PQ5(JLON,KLEV)/ZPQSATS5(JLON),1.0_JPRB) > ZPHU15(JLON))THEN
        IF ((PQ5(JLON,KLEV)/ZPQSATS5(JLON)) < (1.0_JPRB)) THEN
          PQ(JLON,KLEV)=PQ(JLON,KLEV)+1.0_JPRB/ZPQSATS5(JLON)*PHU(JLON)
          PQSATS(JLON)=PQSATS(JLON)&
           & -PQ5(JLON,KLEV)/ZPQSATS5(JLON)**2*PHU(JLON)  
          PHU(JLON)=0.0_JPRB
        ELSE
          PHU(JLON)=0.0_JPRB
        ENDIF
      ENDIF
      IF (ZWS5(JLON) < ZWFC(JLON)) THEN
        PNEIJ(JLON)=PNEIJ(JLON)&
         & -PLSM(JLON)*(0.5_JPRB*(1.0_JPRB-COS(RPI*ZWS5(JLON)&
         & /ZWFC(JLON)))-1.0_JPRB)*PHU(JLON)  
        ZWS=ZWS&
         & +PLSM(JLON)*(1.0_JPRB-ZPNEIJ5(JLON))*0.5_JPRB*RPI/ZWFC(JLON)&
         & *SIN(RPI*ZWS5(JLON)/ZWFC(JLON))*PHU(JLON)  
        PHU(JLON)=0.0_JPRB
      ELSE
        PHU(JLON)=0.0_JPRB
      ENDIF
    ENDIF
    IF (LFGEL) THEN
      PWS (JLON)=PWS( JLON)+ZWS
      PWSI(JLON)=PWSI(JLON)+ZWS
      ZWS=0.0_JPRB
    ELSE
      PWS (JLON)=PWS( JLON)+ZWS
      ZWS=0.0_JPRB
    ENDIF
  ELSE
    PQSATS(JLON)=PQSATS(JLON)+ZPHU5(JLON)*PQS(JLON)
    PHU(JLON)=PHU(JLON)+ZPQSATS5(JLON)*PQS(JLON)
    PQ(JLON,KLEV)=PQ(JLON,KLEV)+ZPHQ5(JLON)*PQS(JLON)
    PHQ(JLON)=PHQ(JLON)+PQ5(JLON,KLEV)*PQS(JLON)
    PQS(JLON)=0.0_JPRB

    PHQ(JLON)=(1.0_JPRB-ZDEW(JLON))*PHQ(JLON)

    PHQ(JLON)=PHQ(JLON)+ZDEW(JLON)*PHU(JLON)

    PHU(JLON)=PHU(JLON)-ZPVEG5(JLON)*PHQ(JLON)
    PVEG(JLON)=PVEG(JLON)+(1.0_JPRB-ZPHU15(JLON))*PHQ(JLON)
    PHQ(JLON)=0.0_JPRB

    PNEIJ(JLON)=PNEIJ(JLON)&
     & -PLSM(JLON)*(0.5_JPRB*(1.0_JPRB-COS(RPI*PWS5(JLON)/ZWSMX(JLON)))-1.0_JPRB)&
     & *PHU(JLON)  
    PWS(JLON)=PWS(JLON)&
     & +PLSM(JLON)*(1.0_JPRB-ZPNEIJ5(JLON))*0.5_JPRB*RPI/ZWSMX(JLON)&
     & *SIN(RPI*PWS5(JLON)/ZWSMX(JLON))*PHU(JLON)  
    PHU(JLON)=0.0_JPRB
  ENDIF

!     CALCUL DE SATURATION UTILISANT LES FONCTIONS DEFINIES.
!     SATURATION CALCULATION WITH CALL TO DEFINED FUNCTIONS.

! - TEMPORAIRE(S) 1D .

! ZESP      : PRESSION DE VAPEUR SATURANTE SUR PRESSION.
!            : SATURATION WATER VAPOUR PRESSURE OVER PRESSURE.

  IF ((1.0_JPRB-ZESP5(JLON)) > 0.0_JPRB) THEN
    ZESP(JLON)=ZESP(JLON)+FOQSAD1(PQSATS(JLON),ZESP5(JLON))
  ELSE
    ZESP(JLON)=ZESP(JLON)+FOQSAD2(PQSATS(JLON),ZESP5(JLON))
  ENDIF
  PQSATS(JLON)=0.0_JPRB
  ZEW=ZEW+1.0_JPRB/PAPRS5(JLON,KLEV)*ZESP(JLON)
  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)-ZEW5(JLON)/PAPRS5(JLON,KLEV)**2*ZESP(JLON)
  ZESP(JLON)=0.0_JPRB
  PTS(JLON)=PTS(JLON)+FOEWAD (ZEW,PTS5(JLON),ZDELT(JLON))

!     CAS DE DEUX LONGUEURS DE RUGOSITE (SUR CONTINENT NON TOTALEMENT
!     COUVERT DE NEIGE).
!     CASE OF TWO ROUGHNESS LENGTHS (ON PARTIALLY SNOW-FREE LAND ONLY).

! ZGZ0H      : G FOIS LA LONGUEUR DE RUGOSITE THERMIQUE COURANTE.

  IF (LDZ0H) THEN

! ZGZ0H = ZGZ0 OVER SEA UNLESS REQUIRED IN NAMPHY1 (CLIMATE RUNS)

     IF (VZIUSTAR0 > 0) THEN
        ZGZ0H(JLON)=PLSM(JLON)*ZGZ0H(JLON)
     ELSE   
        PGZ0(JLON)=PGZ0(JLON)+(1.0_JPRB-PLSM(JLON))*RZHZ0M*ZGZ0H(JLON)
        ZGZ0H(JLON)=PLSM(JLON)*ZGZ0H(JLON)
     ENDIF  

    IF (LNEIGE) THEN
      IF (LSNV) THEN
        PNEIJV(JLON)=PNEIJV(JLON)&
         & +( -PGZ0HF(JLON)**2+Z0CR**2 &
         & +(PGZ0RLF(JLON)*PGZ0HF(JLON)/PGZ0F5(JLON))**2 )&
         & /(2.0_JPRB*SQRT((1.0_JPRB-PNEIJV5(JLON))*PGZ0HF(JLON)**2+&
         & PNEIJV5(JLON)*(Z0CR**2+&
         & (PGZ0RLF(JLON)*PGZ0HF(JLON)/PGZ0F5(JLON))**2)))&
         & *ZGZ0H(JLON)  
        PGZ0F(JLON)=PGZ0F(JLON)&
         & -PNEIJV5(JLON)*2.0_JPRB*PGZ0HF(JLON)**2 &
         & *PGZ0RLF(JLON)**2/(PGZ0F5(JLON)**3)&
         & /(2.0_JPRB*SQRT((1.0_JPRB-PNEIJV5(JLON))*PGZ0HF(JLON)**2+&
         & PNEIJV5(JLON)*(Z0CR**2+&
         & (PGZ0RLF(JLON)*PGZ0HF(JLON)/PGZ0F5(JLON))**2)))&
         & *ZGZ0H(JLON)  
        ZGZ0H(JLON)=0.0_JPRB
      ELSE
        PSNS(JLON)=PSNS(JLON)&
         & +(Z0CR-PGZ0HF(JLON))*PLSM(JLON)&
         & *WCRIN*(1+ZUZ0CN*PGZ0HF(JLON))&
         & /(PSNS5(JLON)+WCRIN*(1+ZUZ0CN*PGZ0HF(JLON)))**2 &
         & *ZGZ0H(JLON)  
        ZGZ0H(JLON)=0.0_JPRB
      ENDIF
    ELSE
      ZGZ0H(JLON)=0.0_JPRB
    ENDIF

  ENDIF

!     CALCULS CONCERNANT LA PRESENCE OU NON DE NEIGE AU SOL.
!     COMPUTATIONS CONCERNING THE PRESENCE OR ABSENCE OF SNOW ON GROUND.

! - TEMPORAIRE(S) 1D .

! ZDELT     : UN POUR CALCULS DE SATURATION SUR GLACE, ZERO SINON.
!            : ONE FOR SATURATION CALCULATION FOR ICE, ZERO OTHERWISE.

  IF (LNEIGE) THEN
    IF (LSNV) THEN
      PNEIJV(JLON)=PNEIJV(JLON)&
       & +(-PGZ0F5(JLON)**2+Z0CR**2+PGZ0RLF(JLON)**2)&
       & /(2.0_JPRB*SQRT((1.0_JPRB-PNEIJV5(JLON))*PGZ0F5(JLON)**2+&
       & PNEIJV5(JLON)*(Z0CR**2+PGZ0RLF(JLON)**2)))*PGZ0(JLON)  
      PGZ0F(JLON)=PGZ0F(JLON)&
       & +(1.0_JPRB-PNEIJV5(JLON))*2.0_JPRB*PGZ0F5(JLON)&
       & /(2.0_JPRB*SQRT((1.0_JPRB-PNEIJV5(JLON))*PGZ0F5(JLON)**2+&
       & PNEIJV5(JLON)*(Z0CR**2+PGZ0RLF(JLON)**2)))*PGZ0(JLON)  
      PGZ0(JLON)=0.0_JPRB
      PNEIJV(JLON)=PNEIJV(JLON)-PVEG0(JLON)*PVEG(JLON)
      PVEG(JLON)=0.0_JPRB
      PNEIJG(JLON)=PNEIJG(JLON)+PLSM(JLON)*(1.0_JPRB-PVEG0(JLON))*PNEIJ(JLON)
      PNEIJV(JLON)=PNEIJV(JLON)+PLSM(JLON)*PVEG0(JLON)*PNEIJ(JLON)
      PNEIJ(JLON)=0.0_JPRB
    ELSE
      PGZ0F(JLON)=PGZ0F(JLON)&
       & +(1.0_JPRB &
       & -PLSM(JLON)*PSNS5(JLON)&
       & *(PSNS5(JLON)+WCRIN*(1.0_JPRB+Z0CR*ZUZ0CN))&
       & /(PSNS5(JLON)+WCRIN*(1+ZUZ0CN*PGZ0F5(JLON)))**2)&
       & *PGZ0(JLON)  
      PSNS(JLON)=PSNS(JLON)&
       & +(Z0CR-PGZ0F5(JLON))*PLSM(JLON)&
       & *WCRIN*(1+ZUZ0CN*PGZ0F5(JLON))&
       & /(PSNS5(JLON)+WCRIN*(1+ZUZ0CN*PGZ0F5(JLON)))**2 &
       & *PGZ0(JLON)  
      PGZ0(JLON)=0.0_JPRB
      PVEG(JLON)=0.0_JPRB
      PSNS(JLON)=PSNS(JLON)+PLSM(JLON)*WCRIN &
       & /(PSNS5(JLON)+WCRIN)**2*PNEIJ(JLON)  
      PNEIJ(JLON)=0.0_JPRB
    ENDIF
  ELSE
    PGZ0F(JLON)=PGZ0F(JLON)+PGZ0(JLON)
    PGZ0(JLON)=0.0_JPRB
    PVEG(JLON)=0.0_JPRB
    PNEIJ(JLON)=0.0_JPRB
  ENDIF

ENDDO
!*
!     ------------------------------------------------------------------
!     IV - CALCUL DE L EPAISSEUR DE LA CLS (EN GEOPOTENTIEL).

!          COMPUTATION OF THE THICKNESS OF THE SBL (IN GEOPOTENTIAL).

! - TEMPORAIRE(S) 1D .

! ZDPHI     : EPAISSEUR EN GEOPOTENTIEL DU NIVEAU DE SURFACE.
!            : GEOPOTENTIAL THICKNESS OF THE SURFACE LEVEL.

DO JLON=KIDIA,KFDIA
  PAPHIF(JLON,KLEV)=PAPHIF(JLON,KLEV)+ZDPHI(JLON)
  PAPHI(JLON,KLEV)=PAPHI(JLON,KLEV)-ZDPHI(JLON)
  ZDPHI(JLON)=0.0_JPRB
ENDDO

IF (LHOOK) CALL DR_HOOK('ACHMTAD',1,ZHOOK_HANDLE)
END SUBROUTINE ACHMTAD
