!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACNEBXR ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PQ,PQC,PQSAT,&
 ! - OUTPUT 2D .
 & PDNEBQC,PDNEBQS,PDNEBQV,PNEB)  

!**** *ACNEBXR* - CALCUL DE LA NEBULOSITE XU-RANDALL ET DE SES DERIVEES.
!                 COMPUTATION OF XU-RANDALL CLOUDINESS AND ITS DERIVATIVES.

!     Sujet.
!     ------
!     CALCUL DE LA NEBULOSITE CONFORMEMENT A XU ET RANDALL, JAS 96.
!     CLOUDINESS COMPUTATION, AS IN XU AND RANDALL, JAS 96.

!**   Interface.
!     ----------
!        *CALL* *ACNEBXR*

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA, KFDIA : BORNES BOUCLES HORIZONTALES   (IST,IEND DANS CPG).
! KIDIA, KFDIA : START/END OF HORIZONTAL LOOP  (IST,IEND IN *CPG*).
! KLON : DIMENSION HORIZONTALE                 (NPROMA DANS CPG).
! KLON : HORIZONTAL DIMENSION                  (NPROMA IN *CPG*).
! KTDIA : DEBUT BOUCLE VERTICALE DANS LA PHYSIQUE.
! KTDIA : START OF THE VERTICAL LOOP IN THE PHYSICS (IF SOME LEVELS ARE
!                     SKIPPED AT THE TOP OF THE MODEL).
! KLEV : FIN BOUCLE VERTICE ET DIMENSION VERTICALE (NFLEVG DANS CPG).
! KLEV : END OF VERTICAL LOOP AND VERTICAL DIMENSION(NFLEVG IN *CPG*).

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (1:KLEV) .

! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOUR.
! PQC        : HUMIDITE SPECIFIQUE DE L'EAU CONDENSEE (LIQUIDE + GLACE).
! PQC        : SPECIFIC HUMIDITY OF CONDENSATED WATER.
! PQSAT      : HUMIDITE SPECIFIQUE DE SATURATION.
! PQSAT      : SPECIFIC HUMIDITY AT SATURATION.

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - 2D (1:KLEV) .

! PDNEBQC    : DERIVEE PARTIELLE DE PNEB PAR RAPPORT A PQC.
! PDNEBQC    : PARTIAL DERIVATIVE OF PNEB ON PQC.
! PDNEBQS    : DERIVEE PARTIELLE DE PNEB PAR RAPPORT A PQSAT.
! PDNEBQS    : PARTIAL DERIVATIVE OF PNEB ON PQSAT.
! PDNEBQV    : DERIVEE PARTIELLE DE PNEB PAR RAPPORT A PQ.
! PDNEBQV    : PARTIAL DERIVATIVE OF PNEB ON PQ.
! PNEB       : NEBULOSITE PARTIELLE "RADIATIVE".
! PNEB       : FRACTIONAL CLOUDINESS FOR RADIATION.

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY0/

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     Methode.
!     --------

!     Auteur.
!     -------
!        97-08, J.M. Piriou.

!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY0  , ONLY : QXRAL    ,QXRDEL   ,QXRR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSAT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDNEBQC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDNEBQS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDNEBQV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNEB(KLON,KLEV) 
INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZEPS1, ZEXP, ZEXPDER, ZQSQV, ZRAPP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*
!     ------------------------------------------------------------------
!     I - CALCUL DE LA NEBULOSITE.
!         CLOUDINESS COMPUTATION.

IF (LHOOK) CALL DR_HOOK('ACNEBXR',0,ZHOOK_HANDLE)
ZEPS1=1.E-12_JPRB

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZQSQV=(PQSAT(JLON,JLEV)-PQ(JLON,JLEV))**(-QXRDEL)
    ZRAPP=(PQ(JLON,JLEV)/PQSAT(JLON,JLEV))**QXRR
    ZEXP=EXP(-QXRAL*PQC(JLON,JLEV)*ZQSQV)
    PNEB(JLON,JLEV)=MAX(ZEPS1,MIN(1.0_JPRB-ZEPS1,ZRAPP*(1.0_JPRB-ZEXP)))
    PDNEBQC(JLON,JLEV)=QXRAL*ZQSQV*ZRAPP*ZEXP
    ZEXPDER=ZRAPP*ZEXP*QXRAL*PQC(JLON,JLEV)*QXRDEL &
     & *(PQSAT(JLON,JLEV)-PQ(JLON,JLEV))**(-QXRDEL-1.0_JPRB)  
    PDNEBQS(JLON,JLEV)=-QXRR/PQSAT(JLON,JLEV)*PNEB(JLON,JLEV)-ZEXPDER
    PDNEBQV(JLON,JLEV)=QXRR/PQ(JLON,JLEV)*PNEB(JLON,JLEV)+ZEXPDER
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('ACNEBXR',1,ZHOOK_HANDLE)
END SUBROUTINE ACNEBXR
