!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACNTCLSAD ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - TRAJECTORY .
 & PAPRS5,PAPRSF5,PCP5,PQ5,PT5,PU5,PV5,&
 & PCD5,PCDN5,PCDNH5,PCH5,PCPS5,PDPHI5,PQS5,PTS5,&
 ! - INPUT  2D .
 & PAPRS,PAPRSF,PCP,PQ,PT,PU,PV,&
 ! - INPUT  1D .
 & PCD,PCDN,PCDNH,PCH,PCPS,PDPHI,PDPHIT,PDPHIV,&
 & PQS,PSTAB,PTS,&
 ! - INPUT  LOGIQUE
 & LDZ0H,&
 ! - OUTPUT 1D .
 & PQCLS,PRHCLS,PTCLS,PUCLS,PVCLS,PUCLN,PVCLN,PZPCLS,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACNTCLSAD * - INTERP. DES PARAMETRES U,V,T,Q AUX HAUTEURS CHOISIES.
!                    CODE ADJOINT DE ACNTCLS

!     Sujet.
!     ------

!     - ROUTINE DE CALCUL ACTIF .
!          INTERPOLATION DES TEMPERATURES, HUMIDITES (SPECIFIQUES ET
!          RELATIVES) ET VENTS AUX HAUTEURS "METEO" .

!          TEMPERATURE, MOISTURE (SPECIFIC AND RELATIVE) AND WIND
!          INTERPOLATION AT STANDARD "METEOROLOGICAL" LEVELS .

!**   Interface.
!     ----------
!        *CALL* *ACNTCLSAD*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPRS      : PRESSION AUX DEMI-NIVEAUX.

! - 2D (1:KLEV) .

! PAPRSF     : PRESSION AUX NIVEAUX.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PT         : TEMPERATURE.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.

! - LOGIQUES .

! LDZ0H      : DIFFERENTES LONGUEURS DE RUGOSITE DYN/THER.

! - 1D (DIAGNOSTIQUE) .

! PCD        : COEFFICIENT D'ECHANGE EN SURFACE POUR U ET V.
! PCDN       : COEFFICIENT NEUTRE D'ECHANGE EN SURFACE.
! PCDNH      : COEFFICIENT THERMIQUE NEUTRE D'ECHANGE EN SURFACE.
! PCH        : COEFFICIENT D'ECHANGE EN SURFACE POUR T ET Q.
! PCPS       : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR AU SOL.
! PQS        : HUMIDITE SPECIFIQUE DE SURFACE.
! PSTAB      : INDICE DE STABILITE A LA SURFACE.

! - 1D (PROGNOSTIQUE) .

! PTS        : TEMPERATURE DE SURFACE.

! - 1D (SPECIFIQUE) .

! PDPHIT     : HAUTEUR D INTERPOLATION POUR T ET Q (EN GEOPOTENTIEL).
! PDPHIT     : HAUTEUR D INTERPOLATION POUR T ET Q (EN GEOPOTENTIEL).
! PDPHIV     : HAUTEUR D INTERPOLATION POUR U ET V (EN GEOPOTENTIEL).

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 1D (DIAGNOSTIQUE) .

! PQCLS      : SORTIE DIAGNOSTIQUE DE L'HUMIDITE SPECIFIQUE A HTQ METEO.
! PRHCLS     : SORTIE DIAGNOSTIQUE DE L'HUMIDITE RELATIVE A HTQ METEO.
! PTCLS      : SORTIE DIAGNOSTIQUE DE LA TEMPERATURE A HTQ METEO.
! PUCLS      : SORTIE DIAGNOSTIQUE DU VENT EN X A HUV METEO.
! PVCLS      : SORTIE DIAGNOSTIQUE DU VENT EN Y A HUV METEO.
! PUCLN      : SORTIE DIAGNOSTIQUE DU VENT NEUTRE EN X A HUV METEO.
! PVCLN      : SORTIE DIAGNOSTIQUE DU VENT NEUTRE EN Y A HUV METEO.
! PZPCLS     : SORTIE DIAGNOSTIQUE DE PRESSION A HTQ METEO.

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/FCTTRM /

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     Methode.
!     --------

!     Auteur.
!     -------
!        99-07, F. Bouyssel.

!     Modifications
!     -------------
!      2008-11, C. Payan, Neutral Wind and ZIV25-->ZIVT5.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LNEIGE,  LBLVAR
USE YOMCST   , ONLY : RV       ,RCPD     ,RCPV     ,RETV     ,&
 & RCW      ,RCS      ,RLVTT    ,RLSTT    ,RTT      ,&
 & RALPW    ,RBETW    ,RGAMW    ,RALPS    ,RBETS    ,&
 & RGAMS    ,RALPD    ,RBETD    ,RGAMD  
USE YOMPHY0  , ONLY : VKARMN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM)               :: KTDIA ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCD5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCDN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCDNH5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHI5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCD(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDNH(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCH(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDPHI(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHIV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTAB(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KLON) 
LOGICAL           ,INTENT(IN)    :: LDZ0H 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRHCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZPCLS(KLON) 
CHARACTER(LEN=9)  ,INTENT(IN)    :: CDLOCK 
REAL(KIND=JPRB) :: ZQCLS5(KLON),ZRHCLS5(KLON),ZTCLS5(KLON),ZUCLS5(KLON)&
 & ,ZVCLS5(KLON),ZPCLS5(KLON)  

INTEGER(KIND=JPIM) :: JLON

REAL(KIND=JPRB) :: ZBD, ZBD5, ZBH, ZBH5, ZBN, ZBN5, ZBNH, ZBNH5,&
 & ZCORS, ZCORS5, ZCORU, ZCORU5, ZCPVMD, ZDELTA, &
 & ZEW, ZEW5, ZIV, ZIV5, ZIVN5, ZIVT5, ZLOGS, ZLOGS5, &
 & ZLOGU, ZLOGU5, ZRS, ZRS5, ZRU, ZRU5  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcttrm.h"
#include "fcttrmad.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACNTCLSAD',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'ACNTCLSAD') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!                        TRAJECTOIRE
!                        TRAJECTORY
!     ------------------------------------------------------------------
!     I - CONSTANTES AUXILIAIRES.

!         AUXILIARY CONSTANTS.

ZCPVMD=RCPV-RCPD

!*
!     ------------------------------------------------------------------
!     II - INTERPOLATION.

!          INTERPOLATION.

DO JLON=KIDIA,KFDIA

!     CALCULS PREPARATOIRES.
!     PREPARATORY CALCULATIONS.

  ZBN5=VKARMN/SQRT(PCDN5(JLON))

!     CALCULS DIFFERENTIES SELON QU'ON A UN OU DEUX Z0.
!     DIFFERENT COMPUTATIONS DEPENDING WHETHER WE HAVE ONE OR TWO Z0.

  IF (LDZ0H) THEN
    ZBNH5=VKARMN*SQRT(PCDN5(JLON))/PCDNH5(JLON)
  ELSE
    ZBNH5=ZBN5
  ENDIF

  ZBD5=VKARMN/SQRT(PCD5(JLON))
  ZBH5=VKARMN*SQRT(PCD5(JLON))/PCH5(JLON)
  ZRU5=PDPHIV(JLON)/PDPHI5(JLON)
  ZRS5=PDPHIT(JLON)/PDPHI5(JLON)
  ZLOGU5=LOG(1.0_JPRB+ZRU5*(EXP(ZBN5)-1.0_JPRB))
  ZLOGS5=LOG(1.0_JPRB+ZRS5*(EXP(ZBNH5)-1.0_JPRB))
  ZCORU5=PSTAB(JLON)*ZRU5*(ZBN5-ZBD5)+(1.0_JPRB-PSTAB(JLON))&
   & *LOG(1.0_JPRB+ZRU5*(EXP(MAX(0.0_JPRB,ZBN5-ZBD5))-1.0_JPRB))  
  ZCORS5=PSTAB(JLON)*ZRS5*(ZBNH5-ZBH5)+(1.0_JPRB-PSTAB(JLON))&
   & *LOG(1.0_JPRB+ZRS5*(EXP(MAX(0.0_JPRB,ZBNH5-ZBH5))-1.0_JPRB))  

!     INTERPOLATION DES COMPOSANTES DU VENT.
!     INTERPOLATION OF WIND COMPONENTS.

  ZIV5=MAX(0.0_JPRB,MIN(1.0_JPRB,(ZLOGU5-ZCORU5)/ZBD5))
  ZUCLS5(JLON)=PU5(JLON,KLEV)*ZIV5
  ZVCLS5(JLON)=PV5(JLON,KLEV)*ZIV5

!     INTERPOLATION DES COMPOSANTES DU VENT NEUTRE.
!     INTERPOLATION OF NEUTRAL WIND COMPONENTS.

  ZIVN5=MAX(0.0_JPRB,MIN(1.0_JPRB,ZLOGU5/ZBD5))

!     INTERPOLATION DES VARIABLES THERMODYNAMIQUES.
!     INTERPOLATION OF THERMODYNAMIC VARIABLES.

  ZIVT5=MAX(0.0_JPRB,MIN(1.0_JPRB,(ZLOGS5-ZCORS5)/ZBH5))
  ZQCLS5(JLON)=PQS5(JLON)+ZIVT5*(PQ5(JLON,KLEV)-PQS5(JLON))
  ZTCLS5(JLON)=(PCPS5(JLON)*PTS5(JLON)+ZIVT5*(PCP5(JLON,KLEV)&
   & *PT5(JLON,KLEV)-PCPS5(JLON)*PTS5(JLON)&
   & +PDPHI5(JLON))-PDPHIT(JLON))&
   & /(RCPD+ZCPVMD*ZQCLS5(JLON))  

!     CALCUL DE L'HUMIDITE RELATIVE.
!     RELATIVE HUMIDITY COMPUTATION.

  IF (LNEIGE) THEN
    ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZTCLS5(JLON)))
  ELSE
    ZDELTA=0.0_JPRB
  ENDIF

  ZPCLS5(JLON)=PAPRS5(JLON,KLEV)+ZRS5*(PAPRSF5(JLON,KLEV)-PAPRS5(JLON,KLEV))
  ZEW5= FOEW (ZTCLS5(JLON),ZDELTA)
  ZRHCLS5(JLON)=(ZPCLS5(JLON)*ZQCLS5(JLON)*(RETV+1.0_JPRB))&
   & /(ZEW5*(1.0_JPRB+RETV*ZQCLS5(JLON)))  
!*
!     ------------------------------------------------------------------
!                        INITIALISATION DES VARIABLES LOCALES
!                        INITIALIZATION OF LOCAL VARIABLES
!     ------------------------------------------------------------------

  ZBN  =0.0_JPRB
  ZBNH =0.0_JPRB
  ZBD  =0.0_JPRB
  ZBH  =0.0_JPRB
  ZRU  =0.0_JPRB
  ZRS  =0.0_JPRB
  ZLOGU=0.0_JPRB
  ZLOGS=0.0_JPRB
  ZCORU=0.0_JPRB
  ZCORS=0.0_JPRB
  ZIV  =0.0_JPRB
  ZEW  =0.0_JPRB

!*
!     ------------------------------------------------------------------
!                        CALCUL DE LA PARTIE ADJOINTE
!                        COMPUTATION OF THE ADJOINT PART
!     ------------------------------------------------------------------

!     CALCUL DE L'HUMIDITE RELATIVE.
!     RELATIVE HUMIDITY COMPUTATION.

  PQCLS(JLON)=PQCLS(JLON)&
   & +(ZPCLS5(JLON)*(RETV+1.0_JPRB)/(ZEW5*(1.0_JPRB+RETV*ZQCLS5(JLON)))&
   & -(ZPCLS5(JLON)*ZQCLS5(JLON)*(RETV+1.0_JPRB))*ZEW5*RETV &
   & /(ZEW5*(1.0_JPRB+RETV*ZQCLS5(JLON)))**2 )*PRHCLS(JLON)  
  PZPCLS(JLON)=PZPCLS(JLON)&
   & +ZQCLS5(JLON)*(RETV+1.0_JPRB)/(ZEW5*(1.0_JPRB+RETV*ZQCLS5(JLON)))&
   & *PRHCLS(JLON)  
  ZEW=ZEW &
   & -( (ZPCLS5(JLON)*ZQCLS5(JLON)*(RETV+1.0_JPRB))&
   & *(1.0_JPRB+RETV*ZQCLS5(JLON))&
   & /(ZEW5*(1.0_JPRB+RETV*ZQCLS5(JLON)))**2 )*PRHCLS(JLON)  
  PRHCLS(JLON)=0.0_JPRB

  PTCLS(JLON)=PTCLS(JLON)+FOEWAD (ZEW,ZTCLS5(JLON),ZDELTA)
  ZEW=0.0_JPRB

  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+(1.0_JPRB-ZRS5)*PZPCLS(JLON)
  PAPRSF(JLON,KLEV)=PAPRSF(JLON,KLEV)+ZRS5*PZPCLS(JLON)
  ZRS=ZRS+(PAPRSF5(JLON,KLEV)-PAPRS5(JLON,KLEV))*PZPCLS(JLON)
  PZPCLS(JLON)=0.0_JPRB

!     INTERPOLATION DES VARIABLES THERMODYNAMIQUES.
!     INTERPOLATION OF THERMODYNAMIC VARIABLES.

    IF (LBLVAR) THEN
      PT(JLON,KLEV)=PT(JLON,KLEV) + PTCLS(JLON)
    ELSE
  PTS(JLON)=PTS(JLON)&
   & +(1.0_JPRB-ZIVT5)*PCPS5(JLON)&
   & /(RCPD+ZCPVMD*ZQCLS5(JLON))*PTCLS(JLON)  
  PCPS(JLON)=PCPS(JLON)&
   & +(1.0_JPRB-ZIVT5)*PTS5(JLON)&
   & /(RCPD+ZCPVMD*ZQCLS5(JLON))*PTCLS(JLON)  
  ZIV=ZIV &
   & +(PCP5(JLON,KLEV)*PT5(JLON,KLEV)-PCPS5(JLON)&
   & *PTS5(JLON)+PDPHI5(JLON))&
   & /(RCPD+ZCPVMD*ZQCLS5(JLON))*PTCLS(JLON)  
  PT(JLON,KLEV)=PT(JLON,KLEV)&
   & +ZIVT5*PCP5(JLON,KLEV)&
   & /(RCPD+ZCPVMD*ZQCLS5(JLON))*PTCLS(JLON)  
  PCP(JLON,KLEV)=PCP(JLON,KLEV)&
   & +ZIVT5*PT5(JLON,KLEV)&
   & /(RCPD+ZCPVMD*ZQCLS5(JLON))*PTCLS(JLON)  
  PDPHI(JLON)=PDPHI(JLON)+ZIVT5/(RCPD+ZCPVMD*ZQCLS5(JLON))*PTCLS(JLON)
  PQCLS(JLON)=PQCLS(JLON)&
   & -ZCPVMD*(PCPS5(JLON)*PTS5(JLON)&
   & +ZIVT5*(PCP5(JLON,KLEV)*PT5(JLON,KLEV)&
   & -PCPS5(JLON)*PTS5(JLON)+PDPHI5(JLON))&
   & -PDPHIT(JLON))/((RCPD+ZCPVMD*ZQCLS5(JLON))**2) &
   & *PTCLS(JLON)  
  PTCLS(JLON)=0.0_JPRB
  ENDIF

   IF (LBLVAR) THEN
     PQ(JLON,KLEV)=PQ(JLON,KLEV)+PQCLS(JLON)
   ELSE
  PQS(JLON)=PQS(JLON)+(1.0_JPRB-ZIVT5)*PQCLS(JLON)
  ZIV=ZIV+(PQ5(JLON,KLEV)-PQS5(JLON))*PQCLS(JLON)
  PQ(JLON,KLEV)=PQ(JLON,KLEV)+ZIVT5*PQCLS(JLON)
  PQCLS(JLON)=0.0_JPRB
  ENDIF

  IF ((((ZLOGS5-ZCORS5)/ZBH5) > 0.0_JPRB).AND.&
     & (((ZLOGS5-ZCORS5)/ZBH5) < 1.0_JPRB)) THEN  
    ZLOGS=ZLOGS+ZIV/ZBH5
    ZCORS=ZCORS-ZIV/ZBH5
    ZBH=ZBH-(ZLOGS5-ZCORS5)/(ZBH5**2)*ZIV
  ENDIF
  ZIV=0.0_JPRB

!     INTERPOLATION DES COMPOSANTES DU VENT.
!     INTERPOLATION OF WIND COMPONENTS.

  ZIV=ZIV+PV5(JLON,KLEV)*PVCLS(JLON)
  PV(JLON,KLEV)=PV(JLON,KLEV)+ZIV5*PVCLS(JLON)
  PVCLS(JLON)=0.0_JPRB

  ZIV=ZIV+PU5(JLON,KLEV)*PUCLS(JLON)
  PU(JLON,KLEV)=PU(JLON,KLEV)+ZIV5*PUCLS(JLON)
  PUCLS(JLON)=0.0_JPRB

  IF ((((ZLOGU5-ZCORU5)/ZBD5) > 0.0_JPRB).AND.&
     & (((ZLOGU5-ZCORU5)/ZBD5) < 1.0_JPRB)) THEN  
    ZLOGU=ZLOGU+ZIV/ZBD5
    ZCORU=ZCORU-ZIV/ZBD5
    ZBD=ZBD-(ZLOGU5-ZCORU5)/(ZBD5**2)*ZIV
  ENDIF
  ZIV=0.0_JPRB

!     INTERPOLATION DES COMPOSANTES DU VENT NEUTRE.
!     INTERPOLATION OF NEUTRAL WIND COMPONENTS.

  ZIV=ZIV+PV5(JLON,KLEV)*PVCLN(JLON)
  PV(JLON,KLEV)=PV(JLON,KLEV)+ZIVN5*PVCLN(JLON)
  PVCLN(JLON)=0.0_JPRB

  ZIV=ZIV+PU5(JLON,KLEV)*PUCLN(JLON)
  PU(JLON,KLEV)=PU(JLON,KLEV)+ZIVN5*PUCLN(JLON)
  PUCLN(JLON)=0.0_JPRB

  IF (((ZLOGU5/ZBD5) > 0.0_JPRB).AND.&
     & ((ZLOGU5/ZBD5) < 1.0_JPRB)) THEN  
    ZLOGU=ZLOGU+ZIV/ZBD5
    ZBD=ZBD-ZLOGU5/(ZBD5**2)*ZIV
  ENDIF
  ZIV=0.0_JPRB

  IF ((ZBNH5-ZBH5) <= 0.0_JPRB) THEN
    ZBNH=ZBNH+PSTAB(JLON)*ZRS5*ZCORS
    ZBH=ZBH-PSTAB(JLON)*ZRS5*ZCORS
    ZRS=ZRS+PSTAB(JLON)*(ZBNH5-ZBH5)*ZCORS
  ELSE
    ZBNH=ZBNH &
     & +( PSTAB(JLON)*ZRS5 &
     & +(1.0_JPRB-PSTAB(JLON))*ZRS5*EXP(ZBNH5-ZBH5)&
     & /(1.0_JPRB+ZRS5*(EXP(ZBNH5-ZBH5)-1.0_JPRB)) )*ZCORS  
    ZBH=ZBH &
     & -( PSTAB(JLON)*ZRS5 &
     & +(1.0_JPRB-PSTAB(JLON))*ZRS5*EXP(ZBNH5-ZBH5)&
     & /(1.0_JPRB+ZRS5*(EXP(ZBNH5-ZBH5)-1.0_JPRB)) )*ZCORS  
    ZRS=ZRS &
     & +( PSTAB(JLON)*(ZBNH5-ZBH5)&
     & +(1.0_JPRB-PSTAB(JLON))*(EXP(ZBNH5-ZBH5)-1.0_JPRB)&
     & /(1.0_JPRB+ZRS5*(EXP(ZBNH5-ZBH5)-1.0_JPRB)) )*ZCORS  
  ENDIF
  ZCORS=0.0_JPRB

  IF ((ZBN5-ZBD5) <= 0.0_JPRB) THEN
    ZBN=ZBN+PSTAB(JLON)*ZRU5*ZCORU
    ZBD=ZBD-PSTAB(JLON)*ZRU5*ZCORU
    ZRU=ZRU+PSTAB(JLON)*(ZBN5-ZBD5)*ZCORU
  ELSE
    ZBN=ZBN &
     & +( PSTAB(JLON)*ZRU5 &
     & +(1.0_JPRB-PSTAB(JLON))*ZRU5*EXP(ZBN5-ZBD5)&
     & /(1.0_JPRB+ZRU5*(EXP(ZBN5-ZBD5)-1.0_JPRB)) )*ZCORU  
    ZBD=ZBD &
     & -( PSTAB(JLON)*ZRU5 &
     & +(1.0_JPRB-PSTAB(JLON))*ZRU5*EXP(ZBN5-ZBD5)&
     & /(1.0_JPRB+ZRU5*(EXP(ZBN5-ZBD5)-1.0_JPRB)) )*ZCORU  
    ZRU=ZRU &
     & +( PSTAB(JLON)*(ZBN5-ZBD5)&
     & +(1.0_JPRB-PSTAB(JLON))*(EXP(ZBN5-ZBD5)-1.0_JPRB)&
     & /(1.0_JPRB+ZRU5*(EXP(ZBN5-ZBD5)-1.0_JPRB)) )*ZCORU  
  ENDIF
  ZCORU=0.0_JPRB

  ZRS=ZRS+(EXP(ZBNH5)-1.0_JPRB)/(1.0_JPRB+ZRS5*(EXP(ZBNH5)-1.0_JPRB))*ZLOGS
  ZBNH=ZBNH+ZRS5*EXP(ZBNH5)/(1.0_JPRB+ZRS5*(EXP(ZBNH5)-1.0_JPRB))*ZLOGS
  ZLOGS=0.0_JPRB

  ZRU=ZRU+(EXP(ZBN5)-1.0_JPRB)/(1.0_JPRB+ZRU5*(EXP(ZBN5)-1.0_JPRB))*ZLOGU
  ZBN=ZBN+ZRU5*EXP(ZBN5)/(1.0_JPRB+ZRU5*(EXP(ZBN5)-1.0_JPRB))*ZLOGU
  ZLOGU=0.0_JPRB

  PDPHI(JLON)=PDPHI(JLON)-PDPHIT(JLON)/(PDPHI5(JLON)**2)*ZRS
  ZRS=0.0_JPRB

  PDPHI(JLON)=PDPHI(JLON)-PDPHIV(JLON)/(PDPHI5(JLON)**2)*ZRU
  ZRU=0.0_JPRB

  PCD(JLON)=PCD(JLON)+VKARMN/(2.0_JPRB*PCH5(JLON)*SQRT(PCD5(JLON)))*ZBH
  PCH(JLON)=PCH(JLON)-VKARMN*SQRT(PCD5(JLON))/(PCH5(JLON)**2)*ZBH
  ZBH=0.0_JPRB

  PCD(JLON)=PCD(JLON)-VKARMN/(2.0_JPRB*PCD5(JLON)*SQRT(PCD5(JLON)))*ZBD
  ZBD=0.0_JPRB

!     CALCULS DIFFERENTIES SELON QU'ON A UN OU DEUX Z0.
!     DIFFERENT COMPUTATIONS DEPENDING WHETHER WE HAVE ONE OR TWO Z0.

  IF (LDZ0H) THEN
    PCDN(JLON)=PCDN(JLON)+VKARMN/(2.0_JPRB*PCDNH5(JLON)*SQRT(PCDN5(JLON)))*ZBNH
    PCDNH(JLON)=PCDNH(JLON)-(VKARMN*SQRT(PCDN5(JLON))/PCDNH5(JLON)**2)*ZBNH
  ELSE
    ZBN=ZBN+ZBNH
  ENDIF
  ZBNH=0.0_JPRB

!     CALCULS PREPARATOIRES.
!     PREPARATORY CALCULATIONS.

  PCDN(JLON)=PCDN(JLON)-VKARMN/(2.0_JPRB*PCDN5(JLON)*SQRT(PCDN5(JLON)))*ZBN
  ZBN=0.0_JPRB

ENDDO

IF (LHOOK) CALL DR_HOOK('ACNTCLSAD',1,ZHOOK_HANDLE)
END SUBROUTINE ACNTCLSAD
