!OPTIONS XOPT(NOEVAL)
!-----------------------------------------------------------------------
SUBROUTINE ACRADIN(KIDIA,KFDIA,KLON,KGL1,KGL2,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 !        ENTREES
 & KMODE , KFLUX , KRAD,&
 & PALBD , PALBP , PAPRS , PAPRSF, PCCO2 , PNEB,&
 & PQO3  , PAER,&
 & PDELP , PEMIS , PMU0  , PQ   , PQSAT,&
 & PQICE , PQLI  , PLSM  , PT    , PTS,&
 !        SORTIES
 & PEMTD , PEMTU , PTRSO,&
 & PFRTHC, PFRTH , PFRSOC, PFRSO,PSFSWDIR,PSFSWDIF,&
 & PFSDNN, PFSDNV, PCTRSO, PCEMTR, PTRSOD,&
 ! - LOCK ARGUMENTS .
 & PPIZA_DST,PCGA_DST,PTAUREL_DST,&
 & CDLOCK)

!**** *ACRADIN* - GESTION MEMOIRE POUR COMPATIBILITE AVEC FTN STANDARD.

!     Sujet.
!     ------
!     - ROUTINE TAMPON NON PORTABLE : GESTION DE LA MEMOIRE.
!       ALLOCATION DYNAMIQUE DES TABLEAUX LOCAUX DE RADINT.
!     - NON-STANDARD-FORTRAN BUFFER-ROUTINE:
!       RADINT LOCAL FIELDS DYNAMICAL ALLOCATION.

!**   Interface.
!     ----------
!        *CALL* *ACRADIN*

!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
! -   INPUT ARGUMENTS.
!     -------------------

!-----------------------------------------------------------------------
! NOTA:    LA DOCUMENTATION DES AUTRES VARIABLES EN ENTREE ET(OU) SORTIE
!              EST DANS LE CODE D'APLPAR.
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!              "APLPAR" CODE.
!-----------------------------------------------------------------------

!     Externes.
!     ---------
!        CALL ATRADIN (test d'initialisation des variables)
!        CALL ABOR1 (en cas de variable non initialisee)
!        CALL RADINT (calcul des flux physiques)
!        CALL RADINT15 (calcul des flux physiques : cycle 15 fige)
!        CALL RFMR     (calcul des flux physiques : cycle 15 fige DM)
!        CALL RADDIAG (diagnostics FMR)
!        CALL RADDIAG15 (diagnostics FMR : version figee cycle 15)

!     Methode.
!     --------

!     Auteur.
!     -------
!        94-01, M. Deque.

!     Modifications.
!     --------------
!        94-09, Ph. Dandin - Passage de l'ozone.
!        94-11, Ph. Dandin - Passage des aerosols.
!        95-04, Ph. Dandin - Albedos diffus/direct.
!        95-08, Ph. Dandin - Diagnostics FMR.
!        95-12, Ph. Dandin - Decoupage RADINT/RADDIAG
!        96-06, Th. Lagarde - KIDIA et KFDIA
!        96-08, Ph. Dandin - Les emissivites contiennent les flux LW
!        97-05, M. Deque (d'apres Ph. Dandin) gel de FMR               
!        98-01, M. Deque - mise en place du verrou                     
!        98-07, JPh. Piedelievre - memoire distribuee
!      2003-04, M. Bellus - input argument LDLAND removed
!                    while it is computed locally (not in APLPAR)
!      2003-10-06 Y. Bouteloup : RECMWF interface to ecmwf radiation code
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        04-07, F. Bouyssel - Use 6 aerosols & NSW
!      2004-09-16 Y. Seity : 4 new arguments for AROME 
!        04-07, F. Bouyssel - Use 6 aerosols & NSW
!        05-10, F. Bouyssel - 3 New arguments PCTRSO,PCEMTR,PTRSOD
!      2005-10-10 Y. Seity : 3 optional arguments for dust optical properties 
!      2005-12, A. ALias - Use 6 aerosols in call RFMR
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LRAYFM   ,LRAYFM15
USE YOERAD   , ONLY : NSW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL2 
INTEGER(KIND=JPIM)               :: KTDIA ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
INTEGER(KIND=JPIM)               :: KFLUX ! Argument NOT used
INTEGER(KIND=JPIM)               :: KRAD ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEB(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,KLEV,6) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSAT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQICE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPIZA_DST(KLON,KLEV,NSW)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCGA_DST(KLON,KLEV,NSW)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUREL_DST(KLON,KLEV,NSW)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMTD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMTU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRSO(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTHC(KLON,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOC(KLON,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSO(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSFSWDIR(KLON,NSW)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSFSWDIF(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFSDNN(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFSDNV(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCTRSO(KLON,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCEMTR(KLON,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRSOD(KLON)
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
! - 2D (0:KLEV) FLUXES .

! PFRSO      : SHORTWAVE RADIATIVE FLUX.
! PFRTH      : LONGWAVE RADIATIVE FLUX.
! - 2D (0:1)
! PFRSOC     : SHORTWAVE CLEAR SKY RADIATIVE FLUX
! PFRTHC     : LONGWAVE CLEAR SKY RADIATIVE FLUX

! Arrays passed between RADINT and RADDIAG

! ZTH        : HALF LEVEL TEMPERATURE

REAL(KIND=JPRB) :: ZTH (KLON,KLEV+1)

! Arrays containing PROGNOSTIC OZONE MIXING RATIO for RADINT (ECMWF)
! ZECPO3    : FULL LEVEL PROGNOSTIC OZONE MIXING RATIO 

INTEGER(KIND=JPIM) :: JGL, JLON, JLEV
REAL(KIND=JPRB) :: ZAER(KLON,KLEV,6) 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "recmwf.intfb.h"
#include "rfmr.intfb.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACRADIN',0,ZHOOK_HANDLE)
IF (CDLOCK /= 'ACRADIN') CALL ABOR1('ACRADIN:BAD NUMB OF ARGS!')

!  AEROSOLS FOR LRAYFM15
DO JGL=1,6
  DO JLEV=1,KLEV
    DO JLON=KIDIA,KFDIA
      ZAER(JLON,JLEV,JGL)=PAER(JLON,JLEV,JGL)
    ENDDO
  ENDDO
ENDDO

IF (LRAYFM) THEN
    CALL RECMWF (KIDIA  , KFDIA  , KLON , KTDIA, KLEV,&
     & KMODE,&
     & PALBD  , PALBP , PAPRS  , PAPRSF, PCCO2 , PNEB  ,&
     & PQO3   , PAER  , PDELP  , PEMIS , PMU0  ,&
     & PQ     , PQSAT , PQICE  , PQLI  , PLSM  , PT    , PTS,&
     & PEMTD  , PEMTU , PTRSO  ,&
     & ZTH    , PCTRSO, PCEMTR , PTRSOD,& 
     & PFRTHC , PFRTH , PFRSOC, PFRSO, PSFSWDIR, PSFSWDIF,&
     & PFSDNN, PFSDNV ,&
     & PPIZA_DST,PCGA_DST,PTAUREL_DST)
ELSEIF (LRAYFM15) THEN
  CALL RFMR &
   & ( KIDIA  , KFDIA  , KLON , KTDIA, KLEV,&
   & KMODE  , KFLUX , KRAD,&
   & PALBD  , PALBP , PAPRS , PAPRSF, PCCO2, PNEB,&
   & PQO3   , ZAER,&
   & PDELP  , PEMIS , PMU0  , PQ   , PQSAT,&
   & PQICE  , PQLI  , PLSM  , PT    , PTS,&
   & PEMTD  , PEMTU , PTRSO,&
   & ZTH    , PCTRSO, PCEMTR, PTRSOD,&
   & PFRTHC , PFRTH , PFRSOC, PFRSO)  
ENDIF

IF (LHOOK) CALL DR_HOOK('ACRADIN',1,ZHOOK_HANDLE)
END SUBROUTINE ACRADIN
