!OPTIONS XOPT(NOEVAL)
SUBROUTINE APLMINI ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT 2D .
 & PAPRS,PAPRSF,PCP,PQIMP,PQLMP,PR,PTMP,&
 & PIPOI,PLHS,PLHV,PNEBM,PPOID,PTCORR,&
 ! - OUTPUT 2D .
 & PMELNET,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *APLMINI * - MINI MICROPHYSIC: CALCUL DU GEL/FONTE


!     Sujet.
!     ------
!     - ROUTINE DE CALCUL ACTIF .
!       CALCUL DU GEL/FONTE.
!     - COMPUTATION OF FREEZING/MELTING.

!**   Interface.
!     ----------
!        *CALL* *APLMINI*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPRS      : PRESSION AUX DEMI-NIVEAUX.

! - 2D (1:KLEV) .

! PAPRSF     : PRESSION AUX NIVEAUX PLEIN.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE.
! PQIMP      : RATIO OF SUSPENDED ICE
! PQLMP      : RATIO OF SUSPENDED LIQUID WATER
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PTMP       : TEMPERATURE.
! PIPOI      : FACTEUR DE CONVERSION DE D_P FLUX VERS D_T Q_X.
! PLHS       : CHALEUR LATENTE DE SUBLIMATION.
! PLHV       : CHALEUR LATENTE DE VAPORISATION.
! PNEBM      : NEBULOSITE POUR LA MICROPHYSIQUE.
! PPOID      : FACTEUR DE CONVERSION DE D_T Q_X VERS D_P FLUX.
! PTCORR     : CORRECTION DE TEMPERATURE POUR LE NUAGE CONVECTIF.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - 2D (KLEV) .

! PMELNET    : NET MELTING RATE.
!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/

!-----------------------------------------------------------------------

!     Externes.
!     ---------
!        *ACACON*
!        *ACEVMEL*

!     Methode.
!     --------

!     Auteur.
!     -------
!        07-12, R. Brozkova.

!     Modifications.
!     --------------
!        08-10, Passing q vapour as argument to ACEVMEL - B. Catry

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : NPHYREP
USE YOMCST   , ONLY : RTT
USE YOMPHY0  , ONLY : FSPRAIN  ,TFVR     ,TFVS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIMP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLMP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPOI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEBM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPOID(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCORR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMELNET(KLON,KLEV) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 

REAL(KIND=JPRB) :: ZFPLSL(KLON,0:KLEV), ZFPLSN(KLON,0:KLEV)

REAL(KIND=JPRB) :: ZIPB(KLON),ZIPH(KLON),ZDQ(KLON),ZTST(KLON)&
 & ,ZEXPN(KLON),ZLSCP(KLON)&
 & ,ZACORL(KLON),ZACONI(KLON),ZACONL(KLON),ZEVAR(KLON),ZEVAN(KLON),ZFONT(KLON)&
 & ,ZZACORL(KLON),ZZACONI(KLON),ZZACONL(KLON)&
 & ,ZSTAL3(KLON),ZHPLSL(KLON)&
 & ,ZSTAN3(KLON),ZHPLSN(KLON)&
 & ,ZLPLSL(KLON),ZLPLSN(KLON),ZQRST(KLON),ZQNST(KLON),ZQLST(KLON),ZQIST(KLON)&
 & ,ZIPOI(KLON),ZPOID(KLON),ZPART(KLON),ZTEST(KLON),ZQVST(KLON)&
 & ,ZZRHO(KLON),ZZPRSF(KLON),ZLSM(KLON),ZNEIJ(KLON),ZTS(KLON)

REAL(KIND=JPRB) :: ZIPLSLO(KLON),ZIPLSLE(KLON),ZIPLSNO(KLON),ZIPLSNE(KLON)&
 & ,ZPRPLO(KLON),ZPRPLE(KLON),ZNEBLOC(KLON)&
 & ,ZOPLSLO(KLON),ZOPLSNO(KLON),ZOPLSLE(KLON),ZOPLSNE(KLON)

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZEPS1, ZEXTMP, &
 & ZMUL, ZZNEBLOC, ZW, ZOPRSL, ZOPRSN, &
 & ZZPRPLE, ZZTEST, ZIPLSL, ZIPLSN

LOGICAL :: LLSIMP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "acacon.intfb.h"
#include "acevmel.intfb.h"
#include "abor1.intfb.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('APLMINI',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'APLMINI') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!     I - CONDITION A LA LIMITE SUPERIEURE, SUBSTITUTION P-Z  ET PARAMETRES.

!         UPPER BOUNDARY CONDITION P-Z EXCHANGE AND PARAMETERS.

! - TEMPORAIRE(S) 1D .

! ZIPH      : INVERSE DE LA PRESSION AU SOMMET DE LA COUCHE.
!           : INVERSE PRESSURE AT THE TOP OF THE LAYER.

LLSIMP=.TRUE.

ZEPS1=1.E-10_JPRB

ZEXTMP=0.0231_JPRB

DO JLON=KIDIA,KFDIA
  ZFPLSL(JLON,KTDIA-1)=0.0_JPRB
  ZFPLSN(JLON,KTDIA-1)=0.0_JPRB
  ZIPH(JLON)=1.0_JPRB/MAX(ZEPS1,PAPRS(JLON,KTDIA-1))
ENDDO

DO JLON=KIDIA,KFDIA
  ZNEBLOC(JLON)=PNEBM(JLON,KTDIA)
  ZPRPLO(JLON)=0.0_JPRB
  ZPRPLE(JLON)=0.0_JPRB
  ZIPLSLO(JLON)=0.0_JPRB
  ZIPLSLE(JLON)=0.0_JPRB
  ZIPLSNO(JLON)=0.0_JPRB
  ZIPLSNE(JLON)=0.0_JPRB
ENDDO

!*
!     ------------------------------------------------------------------
!     II - CALCULS PROPREMENTS DITS DANS UNE BOUCLE VERTICALE OU
!     L'INFORMATION SE TRANSMET DE COUCHE A COUCHE.

!          EFFECTIVE CALCULATIONS IN A VERTICAL LOOP WHERE THE
!     INFORMATION IS PASSED FROM LAYER TO LAYER.

DO JLEV=KTDIA,KLEV

!     CALCULS PRELIMINAIRES ET PREPARATION DE LA DEPENDANCE EN TEMPERATURE
!     POUR LES CONSTANTES NEIGEUSES (LES DEPENDANCES, CHOISIES D'APRES
!     LOPEZ (2002), SONT REGROUPEES).

!     PRELIMINARY COMPUTATIONS AND PREPARATIOB OF THE TEMPERATURE DEPENDENCY
!     FOR THE SNOW-RELATED CONSTANTS (ALL DEPENDENCIES, TAKEN FROM
!     LOPEZ (2002), ARE MERGED).

! - TEMPORAIRE(S) 1D.

! ZQIST     : VALEUR COURRANTE DE QI.
!           : RUNNING VALUE OF QI.
! ZQLST     : VALEUR COURRANTE DE QL.
!           : RUNNING VALUE OF QL.
! ZTST      : VALEUR COURRANTE DE T.
!           : RUNNING VALUE OF T.
! ZDQ       : COPIE LOCALE DE PDQ.
!           : LOCAL COPY OF PDQ.
! ZIPOI     : COPIE LOCALE DE PIPOI.
!           : LOCAL COPY OF PIPOI.
! ZPOID     : COPIE LOCALE DE PPOID.
!           : LOCAL COPY OF PPOID.
! ZZPRSF    : COPIE LOCALE DE PAPRSF.
!           : LOCAL COPY OF PAPRSF.

! ZEXPN     : FACTEUR COMMUN DE DEPENDANCE EN TEMPERATURE (CROISSANT AVEC T).
!           : COMMON TEMPERATURE DEPENDENCY FACTOR (INCREASING WITH T).
! ZIPB      : INVERSE DE LA PRESSION A LA BASE DE LA COUCHE.
!           : INVERSE PRESSURE AT THE BOTTOM OF THE LAYER.
! ZLSCP     : CHALEUR LATENTE DE FUSION DIVISEE PAR CP.
!           : MELTING LATENT HEAT DIVIDED BY CP.


  DO JLON=KIDIA,KFDIA

!     COPIES LOCALES.
!     LOCAL COPIES.

    ZMUL=1.0_JPRB/MAX(ZEPS1,ZNEBLOC(JLON))
    ZQLST(JLON)=PQLMP(JLON,JLEV)*ZMUL
    ZQIST(JLON)=PQIMP(JLON,JLEV)*ZMUL
    ZTST(JLON)=PTMP(JLON,JLEV)
    ZDQ(JLON)=0.0_JPRB
    ZIPOI(JLON)=PIPOI(JLON,JLEV)
    ZPOID(JLON)=PPOID(JLON,JLEV)
    ZZPRSF(JLON)=PAPRSF(JLON,JLEV)
    ZZRHO(JLON)=PAPRSF(JLON,JLEV)/(PR(JLON,JLEV)*PTMP(JLON,JLEV))

!     DEPENDANCES EN TEMPERATURE (POUR LA NEIGE ET LA GLACE) & AUTRES CHAMPS 1D.
!     TEMPERATURE DEPENDENCIES (FOR SNOW AND ICE) AND OTHER 1D ARRAYS.

    ZEXPN(JLON)=MIN(1.0_JPRB,EXP(ZEXTMP*(PTMP(JLON,JLEV)-RTT)))
    ZIPB(JLON)=1.0_JPRB/PAPRS(JLON,JLEV)
    ZLSCP(JLON)=(PLHS(JLON,JLEV)-PLHV(JLON,JLEV))/PCP(JLON,JLEV)

  ENDDO

!     AUTOCONVERSION (INCLUANT LE PROCESSUS DE WEGENER-BERGERON-FINDEISEN).
!     AUTOCONVERSION (INCLUDING THE WEGENER-BERGERON-FINDEISEN PROCESS).

! - TEMPORAIRE(S) 1D.

! ZPART     : COPIE LOCALE DE LA NEBULOSITE.
!           : LOCAL COPY OF CLOUDINESS.
! ZQNST     : VALEUR COURRANTE DE QN.
!           : RUNNING VALUE OF QN.
! ZQRST     : VALEUR COURRANTE DE QR.
!           : RUNNING VALUE OF QR.

! ZACONI    : INCREMENT D'AUTOCONVERSION GLACE-NEIGE.
!           : ICE-SNOW AUTOCONVERSION INCREMENT.
! ZACORL    : INCREMENT D'AUTOCONVERSION EAU-PLUIE.
!           : WATER-RAIN AUTOCONVERSION INCREMENT.
! ZACONL    : INCREMENT D'AUTOCONVERSION EAU-NEIGE (PROCESSUS W-B-F).
!           : WATER-SNOW AUTOCONVERSION INCREMENT (W-B-F PROCESS).

  DO JLON=KIDIA,KFDIA
    ZQRST(JLON)=0.0_JPRB
    ZQNST(JLON)=0.0_JPRB
    ZPART(JLON)=ZNEBLOC(JLON)
    ZLSM(JLON)=0.0_JPRB
    ZNEIJ(JLON)=0.0_JPRB
    ZTS(JLON)=PTMP(JLON,KLEV)
  ENDDO

  CALL ACACON (KIDIA,KFDIA,KLON,&
!   INPUT
    &           LLSIMP,&
    &           ZEXPN,ZLSCP,ZPART,ZLSM,ZNEIJ,ZTS,&
!   INPUT/OUTPUT
    &           ZQIST,ZQLST,ZQNST,ZQRST,ZTST,&
!   OUTPUT
    &           ZACONI,ZACONL,ZACORL,&
    &           'ACACON')

!     SEDIMENTATION STATISTIQUE.
!     STATISTICAL SEDIMENTATION.

  DO JLON=KIDIA,KFDIA
    ZZACONI(JLON)=ZACONI(JLON)
    ZZACORL(JLON)=ZACORL(JLON)
    ZZACONL(JLON)=ZACONL(JLON)

    ZACONI(JLON)=ZACONI(JLON)*ZPART(JLON)
    ZACORL(JLON)=ZACORL(JLON)*ZPART(JLON)
    ZACONL(JLON)=ZACONL(JLON)*ZPART(JLON)

    ZFPLSL(JLON,JLEV)=ZFPLSL(JLON,JLEV-1)+PPOID(JLON,JLEV)*ZACORL(JLON)
    ZFPLSN(JLON,JLEV)=ZFPLSN(JLON,JLEV-1)+PPOID(JLON,JLEV)&
     &*(ZACONL(JLON)+ZACONI(JLON))

  ENDDO

!     CALCULS D'EVAPORATION PUIS DE FONTE DES FLUX DE PRECIPITATIONS.

!     EVAPORATION AND MELTING COMPUTATIONS FOR THE PRECIPITATION FLUXES.

! - TEMPORAIRE(S) 1D.

!           : RAIN EVAPORATION INCREMENT.
! ZFONT     : INCREMENT DE FONTE DE LA NEIGE (-GEL DE LA PLUIE).
!           : SNOW MELT (-RAIN FREEZING) INCREMENT.
! ZTEST     : TEST POUR LA PERPETUATION DU FLUX DE PRECIPITATIONS.
!           : TEST FOR THE PERPETUATION OF THE PRECIPITATION FLUX.

! PASS (CLOUDY PART AND SEEDED - FOR THE MELTING)

  DO JLON=KIDIA,KFDIA
    ZPART(JLON)=ZNEBLOC(JLON)+(1.0_JPRB-ZNEBLOC(JLON))*ZPRPLE(JLON)
    ZW=ZNEBLOC(JLON)/MAX(ZEPS1,ZPART(JLON))
    ZHPLSL(JLON)=ZIPLSLO(JLON)*ZPRPLO(JLON)*ZW+ZIPLSLE(JLON)*&
     &(1.0_JPRB-ZW) 
    ZHPLSN(JLON)=ZIPLSNO(JLON)*ZPRPLO(JLON)*ZW+ZIPLSNE(JLON)*&
     &(1.0_JPRB-ZW) 
    ZLPLSL(JLON)=ZHPLSL(JLON)+PPOID(JLON,JLEV)*ZW*ZZACORL(JLON)
    ZLPLSN(JLON)=ZHPLSN(JLON)+PPOID(JLON,JLEV)*ZW&
     &*(ZZACONL(JLON)+ZZACONI(JLON))

    ZDQ(JLON)=0.0_JPRB
    ZQRST(JLON)=0.0_JPRB
    ZQNST(JLON)=0.0_JPRB
    ZQVST(JLON)=0.0_JPRB
    ZTST(JLON)=ZTST(JLON)+PTCORR(JLON,JLEV)
    ZSTAL3(JLON)=1.0_JPRB
    ZSTAN3(JLON)=1.0_JPRB
  ENDDO

  CALL ACEVMEL (KIDIA,KFDIA,KLON,&
!   INPUT
    &            LLSIMP,&
    &            ZDQ,ZHPLSL,ZHPLSN,ZIPB,ZIPH,ZIPOI,ZLPLSL,ZLPLSN,ZPOID,&
    &            ZQNST,ZQRST,ZQVST,ZSTAL3,ZSTAN3,ZTST,ZZPRSF,ZLSCP,ZZRHO,&
!   OUTPUT
    &            ZEVAN,ZEVAR,ZFONT,ZTEST,&
    &            'ACEVMEL')

  DO JLON=KIDIA,KFDIA
    ZOPRSL=MAX(0.0_JPRB,ZLPLSL(JLON)+ZPOID(JLON)*ZFONT(JLON))/&
     &      MAX(ZEPS1,ZLPLSL(JLON))
    ZOPRSN=MAX(0.0_JPRB,ZLPLSN(JLON)-ZPOID(JLON)*ZFONT(JLON))/&
     &      MAX(ZEPS1,ZLPLSN(JLON))
    ZOPLSLE(JLON)=ZOPRSL*ZIPLSLE(JLON)
    ZOPLSNE(JLON)=ZOPRSN*ZIPLSNE(JLON)
    ZOPLSLO(JLON)=ZOPRSL*(ZIPLSLO(JLON)*ZPRPLO(JLON)+PPOID(JLON,JLEV)&
     &      *ZZACORL(JLON))
    ZOPLSNO(JLON)=ZOPRSN*(ZIPLSNO(JLON)*ZPRPLO(JLON)+PPOID(JLON,JLEV)&
     &      *(ZZACONL(JLON)+ZZACONI(JLON)))
    ZFONT(JLON)=ZFONT(JLON)*ZPART(JLON)
  ENDDO

!     VALEURS DES PSEUDO-FLUX, FIN DU CALCUL DE SEDIMENTATION, VALEURS DES
!     FLUX ET ROTATION A LA FIN DU CALCUL DE COUCHE.

!     VALUES OF PSEUDO-FLUXES, END OF THE SEDIMENTATION COMPUTATION, VALUES
!     OF THE FLUXES AND SWAP AT THE END OF THE LAYER'S COMPUTATIONS.

  DO JLON=KIDIA,KFDIA
    PMELNET(JLON,JLEV)=ZFONT(JLON)-ZACONL(JLON)
    ZFPLSL(JLON,JLEV)=MAX(0.0_JPRB,ZFPLSL(JLON,JLEV)+ZFONT(JLON)&
     &*PPOID(JLON,JLEV))
    ZFPLSN(JLON,JLEV)=MAX(0.0_JPRB,ZFPLSN(JLON,JLEV)-ZFONT(JLON)&
     &*PPOID(JLON,JLEV))
    ZIPH(JLON)=ZIPB(JLON)
  ENDDO

!     RECOUVREMENT DES ZONES NUAGEUSES ET PLUVIEUSES.
!     OVERLAP OF CLOUDY AND RAINY AREAS.

  IF(JLEV /= KLEV) THEN
    DO JLON=KIDIA,KFDIA
      ZZPRPLE=ZPRPLE(JLON)
      ZZNEBLOC=ZNEBLOC(JLON)
      ZNEBLOC(JLON)=PNEBM(JLON,JLEV+1)
      ZPRPLO(JLON)=(MIN(ZNEBLOC(JLON),ZZNEBLOC)*(1.0_JPRB-ZZPRPLE)&
       &+ZNEBLOC(JLON)*ZZPRPLE)/MAX(ZEPS1,ZNEBLOC(JLON))
      ZPRPLE(JLON)=(MAX(ZNEBLOC(JLON),ZZNEBLOC)*(1.0_JPRB-ZZPRPLE)&
       &-ZNEBLOC(JLON)+ZZPRPLE)/MAX(ZEPS1,(1.0_JPRB-ZNEBLOC(JLON)))
      ZIPLSL=MAX(0.0_JPRB,ZFPLSL(JLON,JLEV))
      ZIPLSN=MAX(0.0_JPRB,ZFPLSN(JLON,JLEV))
      ZMUL=1.0_JPRB/MAX(ZEPS1,ZNEBLOC(JLON)*ZPRPLO(JLON))
      ZIPLSLO(JLON)=MAX(0.0_JPRB,(ZIPLSL-ZOPLSLE(JLON)*ZPRPLE(JLON)&
       &*(1.0_JPRB-ZNEBLOC(JLON))))*ZMUL
      ZIPLSNO(JLON)=MAX(0.0_JPRB,(ZIPLSN-ZOPLSNE(JLON)*ZPRPLE(JLON)&
       &*(1.0_JPRB-ZNEBLOC(JLON))))*ZMUL
      ZMUL=1.0_JPRB/MAX(ZEPS1,(1.0_JPRB-ZNEBLOC(JLON))*ZPRPLE(JLON))
      ZIPLSLE(JLON)=MAX(0.0_JPRB,(ZIPLSL-ZOPLSLO(JLON)*ZPRPLO(JLON)&
       &*ZNEBLOC(JLON)))*ZMUL
      ZIPLSNE(JLON)=MAX(0.0_JPRB,(ZIPLSN-ZOPLSNO(JLON)*ZPRPLO(JLON)&
       &*ZNEBLOC(JLON)))*ZMUL
      ZZTEST=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZNEBLOC(JLON)-ZZNEBLOC))
      ZIPLSLO(JLON)=ZZTEST*ZIPLSLO(JLON)+(1.0_JPRB-ZZTEST)*ZOPLSLO(JLON)
      ZIPLSNO(JLON)=ZZTEST*ZIPLSNO(JLON)+(1.0_JPRB-ZZTEST)*ZOPLSNO(JLON)
      ZIPLSLE(JLON)=ZZTEST*ZOPLSLE(JLON)+(1.0_JPRB-ZZTEST)*ZIPLSLE(JLON)
      ZIPLSNE(JLON)=ZZTEST*ZOPLSNE(JLON)+(1.0_JPRB-ZZTEST)*ZIPLSNE(JLON)
    ENDDO
  ENDIF

ENDDO

IF (LHOOK) CALL DR_HOOK('APLMINI',1,ZHOOK_HANDLE)
END SUBROUTINE APLMINI 

