SUBROUTINE CPHFLUX2( KLON, KIDIA, KFDIA, KLEV,&
 & PDT,&
 & LD_LCTEMP, LD_LCTOP,&
 !  Variables 2D Input
 & PTEND,PTENDQ,PTENDQI,PTENDQL,PTENDU,PTENDV,&
 & PT, PU, PV, PCP , PDELP,&
 !  Variables 2D In/Output
 & PFLUX )  

! *  ROUTINE COMPUTES FLUXES FROM TENDENCIES

!    PDT    = TIME STEP
!    LCTEMP = LOGICAL = TRUE IF TEMPERATURE TENDENCIES WHICH COMPUTE  FL
!                       OF THE ENTHALPY
!    LCTOP  = LOGICAL = TRUE IF WE BEGIN THE INTEGRATION FROM TOP
!    PTEND  = TENDENCIES WICH CALCULATE THE FLUXES
!    PTENDQ = HUMIDITY ONLY USED IF LCTEMP = TRUE
!    PTENDQI= ICE ONLY USED IF LCTEMP = TRUE
!    PTENDQL= LIQUID WATER ONLY USED IF LCTEMP = TRUE
!    PTENDU = U WIND ONLY USED IF LCTEMP = TRUE
!    PTENDV = V WIND ONLY USED IF LCTEMP = TRUE
!    PT     = TEMPERATURE ONLY USED IF LCTEMP = TRUE
!    PU     = U WIND ONLY USED IF LCTEMP = TRUE
!    PV     = V WIND ONLY USED IF LCTEMP = TRUE
!    PCP    = Cp ONLY USED IF LCTEMP = TRUE
!    PDELP  = DELTA P FULL LEVEL

!    PFLUX  = MUST CONTAIN BOUNDARIE VALUES EITHER AT TOP OR BOTTOM DEPE

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST  , ONLY : RCPD , RCPV , RG, RCS, RCW

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
LOGICAL           ,INTENT(IN)    :: LD_LCTEMP 
LOGICAL           ,INTENT(IN)    :: LD_LCTOP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEND(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDQL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENDV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUX(KLON,0:KLEV) 
INTEGER(KIND=JPIM) :: JLEV, JLON
REAL(KIND=JPRB) ::ZCOEF1,ZCOEF2,ZCP1,ZTDCP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('CPHFLUX2',0,ZHOOK_HANDLE)
IF (LD_LCTEMP) THEN
  ZCOEF1 = RCPV - RCPD
  ZCOEF2 = 1.0_JPRB
ELSE
  ZCOEF1 = 0.0_JPRB
  ZCOEF2 = 0.0_JPRB
ENDIF

IF (LD_LCTOP) THEN
  DO JLEV = 1,KLEV
    DO JLON = KIDIA,KFDIA

      ZTDCP= ZCOEF1 * PTENDQ(JLON,JLEV)  + ZCOEF2 *(&
       & (RCW-RCPD)* PTENDQL(JLON,JLEV) +&
       & (RCS-RCPD)* PTENDQI(JLON,JLEV) )  

      ZCP1= ZCOEF2*(PCP(JLON,JLEV) + PDT * ZTDCP)+ (1.0_JPRB-ZCOEF2)

      PFLUX(JLON,JLEV)=PFLUX(JLON,JLEV-1) - PDELP(JLON,JLEV)/RG &
       & * (   PTEND(JLON,JLEV) * ZCP1 +  ZCOEF2 *&
       & ( 0.5_JPRB*PDT*((PTENDU(JLON,JLEV))**2 + (PTENDV(JLON,JLEV))**2)&
       & +   ( PU(JLON,JLEV)*PTENDU(JLON,JLEV)&
       & +         PV(JLON,JLEV)*PTENDV(JLON,JLEV) )&
       & +   PT(JLON,JLEV) * ZTDCP ) )  
    ENDDO
  ENDDO

ELSE
  DO JLEV = KLEV,1,-1
    DO JLON = KIDIA,KFDIA

      ZTDCP= ZCOEF1 * PTENDQ (JLON,JLEV) + ZCOEF2 *(&
       & (RCW-RCPD)* PTENDQL(JLON,JLEV) +&
       & (RCS-RCPD)* PTENDQI(JLON,JLEV) )  

      ZCP1 = ZCOEF2*(PCP(JLON,JLEV) + PDT * ZTDCP)+ (1.0_JPRB-ZCOEF2)

      PFLUX(JLON,JLEV-1)=PFLUX(JLON,JLEV) + PDELP(JLON,JLEV)/RG &
       & * (   PTEND(JLON,JLEV) * ZCP1 +  ZCOEF2 *&
       & ( 0.5_JPRB*PDT*((PTENDU(JLON,JLEV))**2 + (PTENDV(JLON,JLEV))**2)&
       & +   ( PU(JLON,JLEV)*PTENDU(JLON,JLEV)&
       & +         PV(JLON,JLEV)*PTENDV(JLON,JLEV) )&
       & +   PT(JLON,JLEV) * ZTDCP )   )  
    ENDDO
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('CPHFLUX2',1,ZHOOK_HANDLE)
END SUBROUTINE CPHFLUX2
