! Library:phys $Id: hlnocondcv.F90 5126 2007-05-11 16:21:54Z towil $

!options xopt(noeval)
 subroutine hlnocondcv(                                     &
!    integers and logicals  
     &                       kcall,     khor,     klev,     &
     &                       kstart,    kstop,              &
!    work arrays 
     &                       klabcv,    klabc,    klab      )
!     --------------------------------------------------------------
!     --------   
!     PURPOSE     Set labels 'klabcv','klabc, 'klab' in case that 
!     --------    the convection scheme `hlcondcv' is not activated.
!                 Hence emphasis is on resolved scale motion.  
!     --------------------------------------------------------------
!     INTERFACE :   subroutine 'hlnocondcv' 
!     ---------     is called from subroutine 'hlconds'
!     INPUT:
!     ---------
!     kcall :    call number to identify possible activation 
!     khor :     number of grid points in loop
!     klev :     number of vertical levels.
!     kstart:    start index in horizontal loop
!     kstop :    end index of horizontal loop   
!     ---------     
!     OUTPUT:
!     ---------
!     klabcv:   label for each level identifying convective status 
!     klabc:    convective label 
!     klab:     convective level number  
!     ---------      
!     AUTHOR:     
!     ---------
!     Bent H. Sass (Dec. 2000), ALADIN code December 2005 

!     Modifications: 
!     --------------
!     First version for ALADIN/ARPEGE: 2004-0830
!     ==============================================================

!     1. Declarations. 
!     ==============================================================

!     1.1 MODULES USED.

use parkind1  ,only : jpim,    jprb
use yomhook   ,only : lhook,   dr_hook

!     1.2 OTHER DECLARATIONS 
     
implicit none

integer(kind=jpim),intent(in)     ::  kcall
integer(kind=jpim),intent(in)     ::  khor
integer(kind=jpim),intent(in)     ::  klev 
integer(kind=jpim),intent(in)     ::  kstart
integer(kind=jpim),intent(in)     ::  kstop

integer(kind=jpim),intent(inout)  ::  klabcv(khor,klev)
integer(kind=jpim),intent(inout)  ::  klabc(khor,klev)
integer(kind=jpim),intent(inout)  ::  klab(khor,klev)

integer(kind=jpim)                ::  jl, jk

real(kind=jprb)                   ::  zhook_handle 

if(lhook) call dr_hook('hlnocondcv',0,zhook_handle)

! 1. Initialize arrays for use in other condensation routines

do jk=1,klev
   do jl=kstart,kstop
      klabcv(jl,jk)=4
      klabc(jl,jk)=1
      klab(jl,jk)=1
   enddo
enddo

if (lhook) call dr_hook('hlnocondcv',1,zhook_handle)

end subroutine hlnocondcv
