! Library:phys $Id: hlradia.F90 5752 2008-03-03 14:42:44Z xiaohua $

#ifdef CRAY
!FPP$ NOCONCUR R
!FPP$ NOVECTOR R
#endif
!options xopt(noeval)

SUBROUTINE HLRADIA ( khor    , klev    , kstart  , kstop   , phybf   , &
     &               phybh   , psa     , pscos   , palbedo , pemsurf , &
     &               pfrland , ptsk    , pt      , pq      , pclq    , &
     &               pcice   , pf      , ph      , pdpf    , ptotcov , &
     &               ldstat  , psalbcor,                               &
!    -- INPUT/OUTPUT --
     &               pstrad  , pstdrad , pstclo  , pstdclo , pslwdn  , &
!    -- OUTPUT --
     &               psswdf  ,                                         & 
     &               psswdn  , pslwnet , psswnet , ptswdn  , ptlwnet , &
     &               ptswnet , plwnet  , pswnet  , pdtdt )

!     Purpose:
!     --------
!     *HLRADIA* : HIRLAM's radiation scheme.
!                 1)   compute temperature tendencies "pdtdt(jl,jk)"
!                      due to radiative processes (in K/s).
!                 2)   compute radiative fluxes at
!                      model half levels ( W/m2 ) including sfc and TOA


!     Interface:
!     ----------
!     variable         type         content
!     ========         ====         =======
!     khor             input        Number of horizontal grid points pro-
!                                   cessed at a time.
!     klev             input        Number of vertical levels
!     kstart           input        Starting index for horizontal loops
!     kstop            input        Ending index for horizontal loops
!     ldstat           input        Logical switch for statistics.
!     phybf            input        Hybrid levels (full)
!     phybh            input        Hybrid levels (half)
!     pfrland          input        Fraction of land in grid square
!     palbedo          input        Grid-square albedo without solar
!                                   height correction.
!     pemsurf          input        Grid-square surface emissivity
!     ptsk             input        Grid-square averaged sfc temp (K)
!     pt               input        Layer temperature (K)
!     pq               input        Layer specific humidity (kg/kg)
!     pclq             input        Layer specific cloud water (liquid).(kg/kg)
!     pcice            input        Layer specific cloud ice.(kg/kg)
!     pf               input        Pressure (Pa) at model full levels.
!     ph               input        Pressure (pa) at model half levels.
!     pdpf             input        Pressure difference between model
!                                   half levels.
!     ptotcov          input        Fractional cloud cover for each
!                                   model layer
!     psa              input        Solar radiative flux on top of the atmosphere
!                                   as a funtion of day of the year (Wm-2)
!     pscos            input        Cosine of solar zenith angle.
!     psalbcor         input        Solar height correction for albedo
!     ----------------------------------------------------------------
!     pdtdt            output       Temperature tendency (K/s) as a
!                                   result of radiative processes.
!     pstrad           output       Sum of radiative temperature ten-
!                                   dencies for each layer.
!                                   ( for statistics )
!     pstdrad          output       Sum of squares of radiative tempe-
!                                   rature tendencies for each level.
!                                   ( for statistics )
!     pstclo           output       Sum of cloud covers for every layer.
!                                   ( for statistics )
!     pstdclo          output       Sum of squares of cloud covers
!                                   for each model layer.
!                                   ( for statistics )
!     psswnet          output       Net surface shortwave radiation
!     pslwnet          output       Net surface longwave radiation
!     psswdn           output       Downwelling sfc shortwave radiation
!     psswdf           output       Downwelling sfc shortwave diffuse radiation
!     pslwdn           in-/output   Downwelling sfc longwave radiation
!     ptswnet          output       Net t.o.a shortwave radiation
!     ptlwnet          output       Net t.o.a longwave radiation
!     ptswdn           output       Downwelling t.o.a shortwave radiation
!     plwnet           output       Net radiation flux density at
!                                   model half levels. (W/m2)
!     pswnet           output       Net radiation flux density at
!                                   model half levels. (W/m2)
!     ----------------------------------------------------------------
!     zslwup           work array   Grid average upward
!                                   longwave  radiation from the ground
!     ztmpcov          work array   Temporary cloud cover.
!     zscos03          work array   pscos**0.3
!pr   zscosre          work array   TRUE cosine of solar zenith angle
!pr                                 (allowed to be smaller than 'cosmin')
!     zfstopc          work array   Shortwave downward flux at the top
!                                   of the uppermost cloud layer.
!     ztrans           work array   Transmission (dimensionless)
!                                   for solar radiation.
!     zcloabs          work array   Cloud absorption (dimensionless)
!                                   for solar radiation.
!     zcovmax          work array   Maximum fractional cloud cover
!                                   in a vertical column.
!                                   half levels.
!     zbb              work array   Black body radiation at model half
!                                   levels
!     zqpath           work array   Scaled water vapour path from top
!                                   of the atmosphere to the top of
!                                   layer "jk"
!     zqbpath          work array   Scaled water vapour from bottom of
!                                   the atmosphere to the top of layer
!                                   "jk"
!     zemiqu           work array   Clear air emissivity from top of
!                                   the atmosphere to the top of layer
!                                   "jk"
!     zemiqb           work array   Clear air emissivity corresponding
!                                   to water path "zqbpath"
!     zemtotu          work array   Cloud emissivity over several model
!                                   layers above layer "jk"
!     zemtotb          work array   Cloud emissivity over several model
!                                   layers below layer "jk"
!     zcwin            work array   Vertical integral of cloud water
!                                   ( g/m2 ) in every layer.
!     zdfrac          work array    Dimensionless "fractional emissi-
!                                   vity" for every model layer.
!     zefcov           work array   Effective cloud cover for every
!                                   layer.
!     zefcovu          work array   Effective total cloud cover above
!                                   layer "jk"
!     zsumemi          work array   Vertical 'integral' of grid box
!                                   cloud water amount.
!     zsumcwi          work array   Cloud water (g/m2) in layer scaled
!                                   by factor "ptotcov(jl,jk)/zcovmax(jl)"
!                                   to be valid for cloud cover "zcovmax"
!     ima              work array   Layer number with maximum effective
!                                   cloud cover "zefcov(jl,ima(jl))"
!                                   below layer number "jk".
!     imau             work array   Layer number with maximum effective
!                                   cloud cover "zefcovu(jl,imau(jl))"
!                                   above layer number "jk"
!lrkw zemc1,zfice,zre,zresw,zreice,zreq,zrewat,ibot
!lrkw                 work arrays  for effective radius calculation
!pr   zqepath work array   Scaled path length for H2O e-type continuum
!pr                        absorption from TOA to the top of layer "jk"
!pr   zqcpath work array   Scaled path length for CO2 absorption
!pr                        from TOA to the top of layer "jk"
!pr   zecont  work array   Emissivity for H2O e-type continuum
!pr   zeco2   work array   Emissivity for carbon dioxide
!gjlr zlwdncl work array   LW flux below cloud

!     zfabso3(1:klev) work Prescribe the vertical distribution 
!                          of ozone shortwave absorption
!     zcadd(1:klev)   work Tuning factor for longwave cooling.

!     Externals:
!     ----------

!     The contents of module "YHLRAD" with constants and arrays
!     used specifically in the radiation scheme.

!     Method:
!     -------

!     Reference:
!     ----------
!     Savijrvi, H. (1990): Fast radiation parameterization schemes for 
!     mesoscale and short-range forecast models, J. Appl. Meteor.,
!     29, 437-447.
!     for effective radius: Contr Atm Phys, 1999,72,205-218
!     for longwave modifications: Report 49,
!     Department of Meteorology, University of Helsinki

!     Author:
!     -------
!     Oct-1991    Hannu Savijaervi, University of Helsinki.

!     Modifications:
!     --------------
!     Nov-1992    Bent H.Sass       The Danish Meteorological Institute,
!     Jan-1993                      Modified for operational use.
!     Feb-1994    B.Sass (DMI),
!                 L.Rontu (FMI)     Revisions.
!     Dec-1998    C.Fortelius (FMI) 
!     Jan-2000    K.Wyser (MISU)
!     Oct-2000    P.Raisanen (DMUH)
!     Jun-2002    C.Jones (SMHI)
!     03-Jun-2005 S.Niemela         Version for ARPEGE/IFS/ALADIN

!     03-Jun-2005 S.Niemela       a) zscos(khor) added and used instead of pscos.
!                                    pscos remains as input parameter.
!                                 b) zzq(khor,klev) added and used instead of q.

!     28-Sep-2005 S.Niemela       a) Renaming of subrouties and modules.
!                                 b) csusa(klev),zfabso3(klev),zcadd(klev)
!                                    removed from argument list. 
!                                    These are calculated now inside
!                                    HLRADIA (previously done in SUHLRAD/INIRAD).
!                                 c) Cleaning of argument list. 
!                                    Following removed:
!                                    zslwup, ztmpcov, zscos03, zscosre, 
!                                    zwsalb, zfstopc, zcloabs, ztrans, zcovmax, 
!                                    ima, imau, zsumemi, zsumcwi, zbb, zqpath, 
!                                    zqbpath, zemiqu, zemiqb, zemtotu, zemtotb,
!                                    zcwin, zefcov, zefcovu, icovu, zdfrac, 
!                                    zemc1, zfice, zre, zresw, zreice, zreq, zrewat, 
!                                    ibot, zdtlw, zdtsw, zqepath, zqcpath, zecont,
!                                    zeco2 and zlwdncl. 
!                                 d) zbeta, tlow and npower were not used 
!                                    ---> removed.

!     03-Oct-2005 S.Niemela          Calculation of pscos externalized. In HIRLAM,
!                                    done in SOLAR. In Aladin, taken from YOMPHY3.
!                                    Consequently, some of the variables not needed
!                                    anymore. Following parameters removed:
!                                    hourc,minc,secc,coslat,sinlat,along,
!                                    cosd,sind.

!     04-Oct-2005 S.Niemela          Cloud liquid (pclq) and ice (pcice) as 
!                                    separate input parameters (instead of
!                                    cloud condensate).
!                                    --->
!                                 a) Module YHLCOND not needed.
!                                 b) locw not needed.
!                                 c) csusa,jintt,tmelt not needed.                
!                                    ---> Moved to HLCLDIAG
!                                 d) for cloud water part "zw1" --> "zcw1" 
!     End modifications:
!     -----------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM , JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YHLCONST  ,ONLY : RAIR, CPAIR, GRAVIT, STEBOL
USE YHLRAD    ,ONLY : NTABM, CQPMIN, CPAT, CKILGR,     &
 &             CSUR1, CSUR3, COSMIN, CPATSH,           &
 &             CAADRY, CAAHUM, CBBDRY, CSAC,           &
 &             CFRAC1, CFRAC2, CAAK, CASK, CPF, CPC,   &
 &             CPSREF, CDPSUR, CREFRAD, SHTAB, CQCO2,  &
 &             CEML1, CEML2, CEML3, CEML4, CPATE,      &
 &             CEME1A, CEME2A, CEME1B, CEME2B, CEME3B, &
 &             CEME4B, CEME5B, CPATC, CEMC1A, CEMC2A,  &
 &             CEMC3A, CEMC1B, CEMC2B, CEMC3B, CEMC4B, &
 &             CA1, CA2, CA3, CA4, CA5, CB1, CB2, CB3, &
 &             CB4, CB5, CEMADD, CMMY, REMN, CRELAND,  &
 &             CRESEA, B10A, B10B, B11, B12, B13A,     &
 &             B13B, B14, A1, A2, A3

IMPLICIT NONE

! IN/OUT VARIABLES 

INTEGER(KIND=JPIM), INTENT(IN)    :: khor
INTEGER(KIND=JPIM), INTENT(IN)    :: klev
INTEGER(KIND=JPIM), INTENT(IN)    :: kstart
INTEGER(KIND=JPIM), INTENT(IN)    :: kstop

REAL(KIND=JPRB),    INTENT(IN)    :: phybf(klev)
REAL(KIND=JPRB),    INTENT(IN)    :: phybh(klev+1)
REAL(KIND=JPRB),    INTENT(IN)    :: psa
REAL(KIND=JPRB),    INTENT(IN)    :: pscos(khor)
REAL(KIND=JPRB),    INTENT(IN)    :: palbedo(khor)
REAL(KIND=JPRB),    INTENT(IN)    :: pemsurf(khor)
REAL(KIND=JPRB),    INTENT(IN)    :: pfrland(khor)
REAL(KIND=JPRB),    INTENT(IN)    :: ptsk(khor)
REAL(KIND=JPRB),    INTENT(IN)    :: pt(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: pq(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: pclq(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: pcice(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: pf(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: ph(khor,klev+1)
REAL(KIND=JPRB),    INTENT(IN)    :: pdpf(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: ptotcov(khor,klev)
REAL(KIND=JPRB),    INTENT(IN)    :: psalbcor(khor)

LOGICAL,            INTENT(IN)    :: ldstat

REAL(KIND=JPRB),    INTENT(INOUT) :: pstrad(klev)
REAL(KIND=JPRB),    INTENT(INOUT) :: pstdrad(klev)
REAL(KIND=JPRB),    INTENT(INOUT) :: pstclo(klev)
REAL(KIND=JPRB),    INTENT(INOUT) :: pstdclo(klev)
REAL(KIND=JPRB),    INTENT(INOUT) :: pslwdn(khor)
REAL(KIND=JPRB),    INTENT(INOUT) :: psswdn(khor)
REAL(KIND=JPRB),    INTENT(INOUT) :: psswdf(khor)
REAL(KIND=JPRB),    INTENT(OUT)   :: pslwnet(khor)
REAL(KIND=JPRB),    INTENT(OUT)   :: psswnet(khor)
REAL(KIND=JPRB),    INTENT(OUT)   :: ptswdn(khor)
REAL(KIND=JPRB),    INTENT(OUT)   :: ptlwnet(khor)
REAL(KIND=JPRB),    INTENT(OUT)   :: ptswnet(khor)
REAL(KIND=JPRB),    INTENT(OUT)   :: plwnet(khor,klev+1)
REAL(KIND=JPRB),    INTENT(OUT)   :: pswnet(khor,klev+1)
REAL(KIND=JPRB),    INTENT(OUT)   :: pdtdt(khor,klev)

! WORKING ARRAYS.

INTEGER(KIND=JPIM) :: ibot(khor)
INTEGER(KIND=JPIM) :: ima(khor)
INTEGER(KIND=JPIM) :: imau(khor)
INTEGER(KIND=JPIM) :: icovu(khor,klev+1)

REAL(KIND=JPRB)    :: zslwup(khor)
REAL(KIND=JPRB)    :: ztmpcov(khor)
REAL(KIND=JPRB)    :: zscos03(khor)
REAL(KIND=JPRB)    :: zscosre(khor)
REAL(KIND=JPRB)    :: zwsalb(khor)
REAL(KIND=JPRB)    :: zcloabs(khor)
REAL(KIND=JPRB)    :: zfstopc(khor)
REAL(KIND=JPRB)    :: ztrans(khor)
REAL(KIND=JPRB)    :: zcovmax(khor)
REAL(KIND=JPRB)    :: zsumemi(khor)
REAL(KIND=JPRB)    :: zsumcwi(khor)
REAL(KIND=JPRB)    :: zbb(khor,klev+1)
REAL(KIND=JPRB)    :: zqpath(khor,klev+1)
REAL(KIND=JPRB)    :: zqbpath(khor,klev+1)
REAL(KIND=JPRB)    :: zemiqu(khor,klev+1)
REAL(KIND=JPRB)    :: zemiqb(khor,klev+1)
REAL(KIND=JPRB)    :: zemtotu(khor,klev)
REAL(KIND=JPRB)    :: zemtotb(khor,klev)
REAL(KIND=JPRB)    :: zcwin(khor,klev)
REAL(KIND=JPRB)    :: zefcov(khor,klev)
REAL(KIND=JPRB)    :: zefcovu(khor,klev+1)
REAL(KIND=JPRB)    :: zdfrac(khor,klev)
REAL(KIND=JPRB)    :: zemc1(khor,klev)
REAL(KIND=JPRB)    :: zfice(khor,klev)
REAL(KIND=JPRB)    :: zre(khor,klev)
REAL(KIND=JPRB)    :: zresw(khor,klev)
REAL(KIND=JPRB)    :: zreice(khor,klev)
REAL(KIND=JPRB)    :: zreq(khor,klev)
REAL(KIND=JPRB)    :: zrewat(khor,klev)
REAL(KIND=JPRB)    :: zqepath(khor,klev+1)
REAL(KIND=JPRB)    :: zqcpath(khor,klev+1)
REAL(KIND=JPRB)    :: zecont(khor)
REAL(KIND=JPRB)    :: zeco2(khor)
REAL(KIND=JPRB)    :: zlwdncl(khor)
REAL(KIND=JPRB)    :: zdtsw(khor,klev+1)
REAL(KIND=JPRB)    :: zdtlw(khor,klev+1)

! OTHER VARIABLES

INTEGER(KIND=JPIM) :: ilevp1,ilevm1,jl,jk,jkp1,jkm1,itab,itabr

REAL(KIND=JPRB)    :: ZHOOK_HANDLE

REAL(KIND=JPRB)    :: zgrrcp,zrgrav,zcdpr,zemc,zeps,zeps2,         &
     &                zcos,zpath,zdtcle,ztrc,zuzc,zemi,            &
     &     zusq,zrads,ztrad,ztdrad,ztclo,ztdclo,zdtdt,zcl,zt,zabc, &
     &     zzefcov,zw1,zw2,zw3,zw4,zw5,zw6,zw7,zfls,zq,zabs2,      &
     &     zq1,zq2,zq3,zq4,zf1,zf2,zf3,zf4,zdif1,zdif3,zdif4,      &
     &     zzdtcle,ztrs,zrad,zsur,ztran,zcw1,zdf

REAL(KIND=JPRB)    :: zrpath,zrbpath,zw8,zw9,zetmax,               &
     &                zeline,zeh2o,za,zb,zx,zx0,zcx,zt3,zu,zue,zuc,&
     &                zalbr,ztrf,zabf,zapr,zfl,zarfl,zden1,zden2

REAL(KIND=JPRB)    :: zcsur1_mod,zcsur1_coeff,zcsur1_wv(khor),    &
     &                zzq(khor,klev)

REAL(KIND=JPRB)    :: zemice,zemwat,zfrmod,ztc,                    &
     &                zwei, zweisum, zreqsum, zrewsum

REAL(KIND=JPRB)    :: zscos(khor)
REAL(KIND=JPRB)    :: zfabso3(klev)
REAL(KIND=JPRB)    :: zcadd(klev)
REAL(KIND=JPRB)    :: zcabs1
REAL(KIND=JPRB)    :: zcabs2

!     =========================================================

IF (LHOOK) CALL DR_HOOK('HLRADIA',0,ZHOOK_HANDLE)

!     ------------------------------
!     definition of local constants.
!     ------------------------------

      zeps=0.001_JPRB
      zeps2=0.00000001_JPRB
      zgrrcp=gravit/cpair
      zrgrav=1._JPRB/gravit
      zcdpr=cask/cpsref
      zalbr=0.70_JPRB
      ztrf=40._JPRB
      zabf=0.013_JPRB
      zapr=0.8_JPRB
      zetmax=3.47E-5_JPRB
      ilevp1=klev+1_JPIM
      ilevm1=klev-1_JPIM

      zcabs1 = 0._JPRB

      DO jk=1,klev
        jkp1=jk+1

!       ------------------------------------------------------------------
!       Tuning factor for longwave cooling (now vertically non-constant)
!       ------------------------------------------------------------------

        zcadd(jk)=1.5E-6_JPRB*(3._JPRB*phybf(jk)-1._JPRB)

!       ----------------------------------------------------------------
!       Prescribe the vertical distribution of ozone shortwave absorption
!       ----------------------------------------------------------------

        zx = LOG(1._JPRB+1000._JPRB*phybh(jkp1))
        zcabs2 = 1._JPRB-EXP(zx*(-6.52978e-3_JPRB+zx*(-0.178512_JPRB &
     &         +zx*(7.13869e-2_JPRB + zx*(-1.45003e-2_JPRB+zx*7.36790e-4_JPRB)))))
        zfabso3(jk) = zcabs2-zcabs1
        zcabs1 = zcabs2

      ENDDO

!     ==================
!     Longwave radiation
!     ==================

!     -----------------------------------------------------------------
!     "zslwup(jl)" is the area averaged infrared
!     radiative flux from surface. Blackbody radiation at model
!     half levels is "zbb(jl,jk)". At top jk=1, at bottom jk=ilevp1.
!     -----------------------------------------------------------------

      DO jl=kstart,kstop

!lrtest This is to test consistent formulation of surface emissivity
!lrtest in RADIA and surface scheme. zslwup now contains upward
!lrtest LW flux emitted by the surface plus reflected downwelling LW
!lrtest flux. Downwelling LW flux is taken from previous time step
!lrtest (zero at kstep 1, initialized in PHYS).
        zt=ptsk(jl)*ptsk(jl)*ptsk(jl)*ptsk(jl)
        zslwup(jl)=stebol*pemsurf(jl)*zt+(1._JPRB-pemsurf(jl))*pslwdn(jl)
!lrtest        zslwup(jl)=stebol*zt
        zbb(jl,1)=stebol*pt(jl,1)*pt(jl,1)*pt(jl,1)*pt(jl,1)

! zbb(jl,ilevp1) is used as the effective emission temperature
! of the lowest layer. In order not to overestimate "heating from
! ground" it should be closer to the surface temperature than the layer
! mid-point temperature is. The semi-empirical approxmn below is based
! on an assumption of a logarithmic temperature profile in the lowest
! half-layer with a roughness length of a few cm.

        zt= (2._JPRB*pt(jl,klev) + ptsk(jl))/3._JPRB
        zbb(jl,ilevp1)=stebol*zt*zt*zt*zt
      ENDDO

!     -----------------------------------------------------------------
!     Security check for negative humidity.
!     -----------------------------------------------------------------

      DO jk=1,klev
        DO jl=kstart,kstop
          zzq(jl,jk)=MAX(pq(jl,jk),0._JPRB)
        ENDDO
      ENDDO

!     -----------------------------------------------------------------
!     Atmospheric blackbody radiation "zbb(jl,jk)" at model half levels.
!     -----------------------------------------------------------------

      DO jk=2,klev
        jkm1=jk-1
          DO jl=kstart,kstop
            zt=pt(jl,jkm1) +(pt(jl,jk)-pt(jl,jkm1))* &
     &                     pdpf(jl,jkm1)/(pdpf(jl,jk)+pdpf(jl,jkm1)+zeps)
            zbb(jl,jk)=stebol*zt*zt*zt*zt
          ENDDO
      ENDDO

!     ------------------------------------------------------------------
!     The modification of LW clear-sky emisvities starts here
!     ------------------------------------------------------------------

      DO jl=kstart,kstop
        zqpath(jl,1) = 0._JPRB
        zqepath(jl,1) = 0._JPRB
        zqcpath(jl,1) = 0._JPRB
        zqbpath(jl,ilevp1)=0._JPRB
        zemiqu(jl,1) = 0._JPRB
        zemiqb(jl,ilevp1) =0._JPRB
      ENDDO

      DO jk =2,ilevp1
        jkm1=jk-1
        DO jl=kstart,kstop

! Compute effective absorber amounts from the TOA downwards
! zqpath(jl,jk), zqepath(jl,jk), and zqcpath(jl.jk) are the path lengths
! for H2O line absorption, H2O e-type continuum absorption and CO2
! absorption from the top of the atmosphere to the top of layer 'jk'.

          zcx = pf(jl,jkm1)*pdpf(jl,jkm1)*cpat
          zt3 = pt(jl,jkm1)*pt(jl,jkm1)*pt(jl,jkm1)
          zpath = zzq(jl,jkm1)*zcx

          zqepath(jl,jk)= zqepath(jl,jkm1)+ zpath                        &
     &      *1.6078_JPRB*zzq(jl,jkm1)/(1._JPRB+0.6078_JPRB*zzq(jl,jkm1)) &
     &      *6.0202E+14_JPRB/(zt3*zt3+zeps)
          zqcpath(jl,jk)= zqcpath(jl,jkm1)+ cqco2* zcx *zt3/1.3824e+7_JPRB

! Computation of individual layer emissivity for the the calculation
! of 'zdfrac(jl,jkm1)'.

          zx = SQRT(SQRT(SQRT(zpath)))
          zemc=(zx*zx*zx*zx*(ceml1+zx*(ceml2+zx*(ceml3+zx*ceml4))) &
     &          +cemadd) / (pdpf(jl,jkm1)+zeps)

          zdfrac(jl,jkm1) = cfrac1 / (cfrac1 +cfrac2*zemc)
          zpath = max( zpath, cqpmin )
          zqpath(jl,jk) = zqpath(jl,jkm1) + zpath
        ENDDO

!     -----------------------------------------------------------------
!     Computation of upward clear air emissivities (zemiqu)
!     from level JK to TOA
!     -----------------------------------------------------------------

! H2O e-type continuum
        DO jl=kstart,kstop
          zue= zqepath(jl,jk)
          IF (zue < cpate) THEN
            zecont(jl) = zue*(ceme1a+zue*ceme2a)
          ELSE
            zx = SQRT(zue)
            zecont(jl) = ceme1b+zx*(ceme2b+zx*(ceme3b+zx*(ceme4b+zx*ceme5b)))
          ENDIF
        ENDDO
! CO2
        DO jl=kstart,kstop
          zuc= zqcpath(jl,jk)
          IF (zuc < cpatc) THEN
            zx = SQRT(zuc)
            zeco2(jl) = zx*(cemc1a+zx*(cemc2a+zx*cemc3a))
          ELSE
            zx = LOG(zuc)
            zeco2(jl) = cemc1b+zx*(cemc2b+zx*(cemc3b+zx*cemc4b))
          ENDIF
        ENDDO

        DO JL=KSTART,KSTOP
! H2O lines and p-type continuum
          zu = zqpath(jl,jk)
          zx0 = SQRT(SQRT(zu))
          zx = SQRT(zx0)
          zeline=zx0*zx0*(ceml1+zx*(ceml2+zx*(ceml3+zx*ceml4)))
! a and zb are the "overlap factors" between water vapor line and continuum
! absorption, and between water vapor and CO2 absorption, respectively
          za=1._JPRB+zx0*(ca1+zx0*(ca2+zx0*(ca3+zx0*(ca4+zx0*ca5))))
          zb=1._JPRB+zx0*(cb1+zx0*(cb2+zx0*(cb3+zx0*(cb4+zx0*cb5))))
! Total emissivity for water vapor
          zeh2o= zeline +za*(1._JPRB-zeline)*zecont(jl)
! Total clear air emissivity
          zemiqu(jl,jk) = zeh2o+zb*(1._JPRB-zeh2o)*zeco2(jl)
        ENDDO
      ENDDO

!     -----------------------------------------------------------------
!     Computation of "downward" clear air emissivities (zemiqb) between
!     level JK  and the surface
!     -----------------------------------------------------------------

      DO jk=1,klev
! H2O e-type continuum
        DO jl=kstart,kstop
          zue= zqepath(jl,ilevp1)-zqepath(jl,jk)
          IF (zue < cpate) THEN
            zecont(jl) = zue*(ceme1a+zue*ceme2a)
          ELSE
            zx = SQRT(zue)
            zecont(jl) = ceme1b+zx*(ceme2b+zx*(ceme3b+zx*(ceme4b+zx*ceme5b)))
          ENDIF
        ENDDO
! CO2
        DO jl=kstart,kstop
          zuc= zqcpath(jl,ilevp1)-zqcpath(jl,jk)
          IF (zuc < cpatc) THEN
            zx = SQRT(zuc)
            zeco2(jl) = zx*(cemc1a+zx*(cemc2a+zx*cemc3a))
          ELSE
            zx = LOG(zuc)
            zeco2(jl) = cemc1b+zx*(cemc2b+zx*(cemc3b+zx*cemc4b))
          ENDIF
        ENDDO
! H2O lines and p-type continuum
        DO jl = kstart,kstop
          zu = zqpath(jl,ilevp1)-zqpath(jl,jk)
! zqbpath (the water vapor path length from the sfc upwards) is still
! needed for shortwave calculations
          zqbpath(jl,jk) = zu
          zx0= SQRT(SQRT(zu))
          zx = SQRT(zx0)
          zeline=zx0*zx0*(ceml1+zx*(ceml2+zx*(ceml3+zx*ceml4)))
! a and zb are the "overlap factors" between water vapor line and continuum
! absorption, and between water vapor and CO2 absorption, respectively
          za=1._JPRB+zx0*(ca1+zx0*(ca2+zx0*(ca3+zx0*(ca4+zx0*ca5))))
          zb=1._JPRB+zx0*(cb1+zx0*(cb2+zx0*(cb3+zx0*(cb4+zx0*cb5))))
! Total emissivity for water vapor
          zeh2o= zeline +za*(1._JPRB-zeline)*zecont(jl)
! Total clear air emissivity
          zemiqb(jl,jk) = zeh2o+zb*(1._JPRB-zeh2o)*zeco2(jl)
        ENDDO
      ENDDO

!     ------------------------------------------------------------------
!     Estimate the cloud water content "zcwin(jl,jk)" of each layer
!     ( g/m**2 ), the integral inside clouds, not grid box value. .
!     Compute emissivity "zemc" of clouds and a reduced effective
!     fractional cloud cover "zefcov(jl,jk)"
!     ------------------------------------------------------------------

!  Initialize for layer 1, assuming there are no clouds

      DO jl=kstart,kstop
         zre(jl,1)=0._JPRB
         zresw(jl,1)=0._JPRB
         zcwin(jl,1)=0._JPRB
         zefcov(jl,1)=0._JPRB
      ENDDO

      DO jk=1,klev
        DO jl=kstart,kstop
          IF (ptotcov(jl,jk) > zeps) THEN
            zcw1=(pclq(jl,jk)+pcice(jl,jk))/(ptotcov(jl,jk)+zeps)
          ELSE
            zcw1=0._JPRB
          ENDIF
          zcwin(jl,jk)=zrgrav*ckilgr*pdpf(jl,jk)*zcw1

!  Calculate the effective radius for water and ice
!  relax the droplet number towards the background value at
!  sigma=0.7, the background value is identical with the sea value
!--------------------------------------------
        zfrmod=pfrland(jl)*MAX(0._JPRB,(phybf(jk)-.7_JPRB)/.3_JPRB)
        zrewat(jl,jk)=MAX(remn, cmmy*(creland*zfrmod+cresea*(1._JPRB-zfrmod)) &
     &                  *(zcw1*pf(jl,jk)/(rair*pt(jl,jk)))**.33333333_JPRB)

! Fraction of ice.
        zfice(jl,jk)  = MIN(pcice(jl,jk)/(pclq(jl,jk)+pcice(jl,jk)+zeps2), &
     &                     1._JPRB)

! zre for ice is function of T, after Ou & Liou,1993
        ztc=MIN(-20._JPRB,pt(jl,jk)-273._JPRB)
        zreice(jl,jk)=.5_JPRB*(((.0012_JPRB*ztc+.197_JPRB)*ztc  &
     &               +12.42_JPRB)*ztc+326.3_JPRB)

!  Emissivity is a function of effective radius, different for
!  water and ice.

        zemwat=.0255_JPRB +.0570_JPRB*EXP(-.0890_JPRB*zrewat(jl,jk))
        zemice=.0202_JPRB+.2059_JPRB*EXP(-.0676_JPRB*zreice(jl,jk))
        zemc1(jl,jk)=(1._JPRB-zfice(jl,jk))*zemwat+zfice(jl,jk)*zemice
        zw1=zemc1(jl,jk)*zcwin(jl,jk)

          IF (zw1 < 0.5_JPRB) THEN
             zemc=zw1
!1.-MAX(1.-zw1,0.)
          ELSEIF (zw1 < 5._JPRB) THEN
             zemc=1._JPRB-EXP(-zw1)
          ELSE
             zemc=1._JPRB
          ENDIF

          zefcov(jl,jk)=zemc*ptotcov(jl,jk)
        ENDDO
      ENDDO

!     ---------------------------------------------------------------
!     Determine maximum effective cloud cover "zefcovu(jl,jk)"
!     valid for layers above "jk" , and the associated layer
!     number "icovu(jl,jk)".
!     ---------------------------------------------------------------

      DO jl=kstart,kstop
        imau(jl)=1_JPIM
        zsumemi(jl)=0._JPRB
        ztmpcov(jl)=0._JPRB
      ENDDO

      DO jk=1,klev
        DO jl=kstart,kstop
          zsumemi(jl)=zsumemi(jl) + ptotcov(jl,jk)*zcwin(jl,jk)

          zw1=zemc1(jl,jk)*zsumemi(jl)
          IF (zw1 < 0.5_JPRB) THEN
             zemtotu(jl,jk)=zw1
!1.-MAX(1.-zw1,0.)
          ELSEIF (zw1 < 5._JPRB) THEN
             zemtotu(jl,jk)=1._JPRB-EXP(-zw1)
          ELSE
             zemtotu(jl,jk)=1._JPRB
          ENDIF

          zefcovu(jl,jk)=zefcov(jl,1)
          icovu(jl,jk)=imau(jl)

        ENDDO
      ENDDO

      DO jk=2,klev
        jkm1=jk-1
          DO jl=kstart,kstop
            zefcovu(jl,jk)=ztmpcov(jl)
            icovu(jl,jk)=imau(jl)
            zemi=MAX( ptotcov(jl,jkm1)*zemtotu(jl,jkm1),  &
     &                zefcov(jl,jkm1) )
            IF( zemi >= ztmpcov(jl) ) THEN
               zefcovu(jl,jk)=zemi
               icovu(jl,jk)=jkm1
               ztmpcov(jl)=zemi
               imau(jl)=jkm1
            ENDIF
          ENDDO
      ENDDO

      DO jl=kstart,kstop
        zefcovu(jl,ilevp1)=ztmpcov(jl)
        icovu(jl,ilevp1)=imau(jl)
        zemi=MAX( ptotcov(jl,klev)*zemtotu(jl,klev),  &
     &            zefcov(jl,klev) )
        IF( zemi >= ztmpcov(jl) ) THEN
          zefcovu(jl,ilevp1)=zemi
          icovu(jl,ilevp1)=klev
        ENDIF
      ENDDO

!     --------------------------------------------------------------
!     Estimate a total cloud emissivity "zemtotb" from the bottom of
!     the atmosphere to the top of layer "jk"
!     ---------------------------------------------------------------

      DO jl=kstart,kstop
        zsumemi(jl)=0._JPRB
        ima(jl)=klev
        ztmpcov(jl)=0._JPRB
      ENDDO

      DO jk=klev,1,-1
        DO jl=kstart,kstop
          zsumemi(jl)=zsumemi(jl) + ptotcov(jl,jk)*zcwin(jl,jk)
          zw1=zemc1(jl,jk)*zsumemi(jl)
          IF (zw1 < 0.5_JPRB) THEN
             zemtotb(jl,jk)=zw1
!1.-MAX(1.-zw1,0.)
          ELSEIF (zw1 < 5._JPRB) THEN
             zemtotb(jl,jk)=1._JPRB-EXP(-zw1)
          ELSE
             zemtotb(jl,jk)=1._JPRB
          ENDIF
        ENDDO
      ENDDO

! initialize longwave fluxes
!---------------------------
      DO jl=kstart,kstop
         pslwdn(jl)=0._JPRB
         pslwnet(jl)=0._JPRB
         ptlwnet(jl)=0._JPRB
      ENDDO

      DO jk=1,klev+1
      DO jl=kstart,kstop
         zdtlw(jl,jk)=0._JPRB
         plwnet(jl,jk)=0._JPRB
      ENDDO
      ENDDO

!gj040203 Moved calculation of clear sky surface radiation flux to here
!gj040203 as we wish to use radf (integrated water vapour emission) to
!gj040203 determine a correction that should be applied to clear sky
!gj040203 as the water vapour concentration in the atmosphere goes down
!gj040203 (emission from other gases & aerosols becomes fractionally
!gj040203 more important as water vapour concentrations decrease)
!gj040203 and to account for know deficiencies in the water vapour
!gj040203 emission at low WV concentrations...we make csur1 afunction
!gj040203 of the water vapour emission radf.
!     ---------------------------------------------------------------
!     Determination of sfc downwelling longwave radiation "pslwdn(jl)"
!     At first a cloud free contribution is calculated
!     in a loop over all model layers.The final net radiation is
!     determined by adding a cloudy contribution  with weight
!     "zefcovu(jl,ilevp1)".
!     ---------------------------------------------------------------

      DO jl=kstart,kstop
        zlwdncl(jl)=0._JPRB
      ENDDO

!     To be consistent with the computation of "heating from ground"
!     in the lowest layer, compute the contribution of the lowest
!     layer to the downward flux at the surface separately

      DO jl=kstart,kstop
        pslwdn(jl) = zbb(jl,ilevp1)*(zemiqb(jl,klev)-zemiqb(jl,ilevp1))
        zlwdncl(jl) = pslwdn(jl)
      ENDDO

      DO jk=klev-1,1,-1
         jkp1=jk+1
         DO jl=kstart,kstop
            pslwdn(jl)=pslwdn(jl) + 0.5_JPRB*( zbb(jl,jk)+zbb(jl,jkp1) )  &
     &               *( zemiqb(jl,jk) - zemiqb(jl,jkp1) )

!------------------------------
! calculate clear sky emission below cloud base

           IF(jk >= (icovu(jl,ilevp1)))THEN
            zlwdncl(jl)=zlwdncl(jl)+0.5_JPRB*( zbb(jl,jk)+zbb(jl,jkp1) )  &
     &               *( zemiqb(jl,jk) - zemiqb(jl,jkp1) )
           ENDIF
!------------------------------
          ENDDO
      ENDDO

!lr test:cold temperature increase of PSLWDN not applied 30.9.2003
      zcsur1_coeff=6.92e-4_JPRB
      zcsur1_mod=0._JPRB
      DO jl=kstart,kstop
       zcsur1_wv(jl)= csur1+zcsur1_mod
      ENDDO

      DO jl=kstart,kstop

! The empirical correction for e-type continuum is no longer needed!

!gjlr------------------------------
!gjlr Caluclate clear sky contribution to LW flux at surface
!gjlr it is a combination of clear sky flux form
!gjlr depth of atmosphere in clear portion of grid box plus
!gjlr clear contribution from below cloud base, multiplied
!gjlr by cloud fraction plus zcsur1_wv(jl) (missing gas)
!gjlr contribution, which is assumed to be a full depth of atmos
!gjlr contribution (for want of knowing how better to partition it)
!gjlr
        pslwdn(jl)=( (1._JPRB-zefcovu(jl,ilevp1))*(pslwdn(jl)+zcsur1_wv(jl)) ) &
     &           +( zefcovu(jl,ilevp1)*(zlwdncl(jl)+zcsur1_wv(jl)) )
!gjlr------------------------------
        zsur=pt(jl,klev) -cdpsur*(pt(jl,klev) -pt(jl,ilevm1))    &
     &                   /(pf(jl,klev) -pf(jl,ilevm1)+zeps)
        zrad=stebol*zsur*zsur*zsur*zsur
!gjlr        zemi=MIN( pslwdn(jl)/zrad, 1. )
!gjlr------------------------------
!gjlr zemi normalised cloudy flux contribution as a function
!gjlr clear sky emissivity. this is now calculated from cloud base
!gjlr to surface in zlwdncl array
!gjlr
        zemi=MIN( zlwdncl(jl)/zrad, 1._JPRB )
!gjlr------------------------------
        pslwdn(jl)=pslwdn(jl) +csur3*(1._JPRB-zemi)*zefcovu(jl,ilevp1)  &
     &           *zbb(jl,icovu(jl,ilevp1)+1)
        pslwnet(jl)=pslwdn(jl)-zslwup(jl)
!lrtest        pslwnet(jl)=pemsurf(jl)*(pslwdn(jl)-zslwup(jl))
!lrtest use the form with pemsurf if zslwup=sigma*Tskin**4
!lrtest the form without pemsurf is identical to this in case
!lrtest zslwup(jl)=stebol*pemsurf(jl)*zt+(1.-pemsurf(jl))*pslwdn(jl)
!lrtest both forms correspond the definitions of subroutine RAD2SURF

      ENDDO

!     Compute infrared heating effect of all layers except the upper-
!     most and the lowest one. The heating rate consists of the fol-
!     lowing terms "zq1,zq2,zq3,zq4" described below.
!     ---------------------------------------------------------------

      DO jk=ilevm1,2,-1
        jkm1=jk-1
        jkp1=jk+1
        DO jl=kstart,kstop
            zzefcov=ztmpcov(jl)
            zemi=MAX( ptotcov(jl,jkp1)*zemtotb(jl,jkp1),  &
     &           zefcov(jl,jkp1) )
            IF( zemi >= ztmpcov(jl) ) THEN
              ima(jl)=jkp1
              zzefcov=zemi
              ztmpcov(jl)=zemi
            ENDIF

!           ------------------------------------------------------------
!           "zq1" is the flux difference across the clear fraction of
!           the layer with no overlying clouds.
!           ------------------------------------------------------------
            zw1=1._JPRB-MAX( zefcov(jl,jk),zefcovu(jl,jk) )
            zt=0.5_JPRB*(zbb(jl,jk) +zbb(jl,jkp1))
            zfls=(zslwup(jl)-zt)*(zemiqb(jl,jk)-zemiqb(jl,jkp1))

            zq=zt*(zemiqu(jl,jk)-zemiqu(jl,jkp1))  &
     &        -zcadd(jk)*pdpf(jl,jk)/zgrrcp
            zq1=zw1*( zq +zfls )
!           ------------------------------------------------------------
!           "zq2" is the flux difference across the clear fraction of
!           the layer with overlying clouds.
!           ------------------------------------------------------------
            zw2=MAX( zefcovu(jl,jk) -zefcov(jl,jk),0._JPRB )
            zw3=MIN( MAX( zzefcov-zefcov(jl,jk), 0._JPRB), zw2 )
            zf2=MIN( (ph(jl,jk) -ph(jl,icovu(jl,jk)+1))/cpc, 1._JPRB )
            zq2=zw2*zq*zf2 + (zw2-zw3)*zfls
!           ------------------------------------------------------------
!           "zq3" is the flux difference across the cloudy fraction of
!           the layer with no overlying clouds.
!           ------------------------------------------------------------
            zw4=MAX( zefcov(jl,jk)-zefcovu(jl,jk), 0._JPRB )
            zw5=MIN( zefcov(jl,jkp1),zefcov(jl,jk) )
            zw6=MAX( zw5-zefcovu(jl,jk), 0._JPRB )
            zw7=MIN( zzefcov,zefcov(jl,jk) )
            zw8=MAX( zw7-zefcovu(jl,jk),0._JPRB )

            zf3=MIN( cpf/((ph(jl,ima(jl))-ph(jl,jkp1))+zeps),1._JPRB )
            zdif3=zbb(jl,ima(jl)) -zbb(jl,jkp1)
            zf4=MIN( cpf/(ph(jl,ilevp1)-ph(jl,jkp1)+zeps),1._JPRB )
            zdif4=zslwup(jl) -zbb(jl,jkp1)

! The empirical correction for e-type continuum is no longer needed!
            zq3=zw4*( -zbb(jl,jk) +0.5_JPRB*(zbb(jl,jk) +zbb(jl,jkm1))  &
     &          *zemiqu(jl,jk) +zcsur1_wv(jl)*phybh(jk))
            zq3=zq3 + (zw8-zw6)*zdif3*(zf3+zdfrac(jl,jkp1)*(1.-zf3))
            zq3=zq3 + (zw4-zw8)*zdif4*(zf4+zdfrac(jl,jkp1)*(1.-zf4))
!           ------------------------------------------------------------
!           "zq4" is the flux difference across the cloudy fraction of
!           the layer with overlying clouds.
!           ------------------------------------------------------------
            zw6=MIN( zefcovu(jl,jk), zefcov(jl,jk) )
            zw7=MIN(MIN( zzefcov,zefcov(jl,jk) ),zefcovu(jl,jk) )
            zw8=MAX( zw6 -zefcov(jl,jkm1) ,0._JPRB )
            zw9=MAX( zw7 -zefcov(jl,jkp1), 0._JPRB )
            zf1=MIN( cpf/(ph(jl,jk)-ph(jl,icovu(jl,jk)+1)+zeps),1._JPRB )
            zdif1=zbb(jl,icovu(jl,jk)+1) -zbb(jl,jk)

            zq4=zw8*zdif1*( zf1 +zdfrac(jl,jkm1)*(1._JPRB-zf1) )
            zq4=zq4 + zw9*zdif3*( zf3 +zdfrac(jl,jkp1)*(1._JPRB-zf3) )
            zq4=zq4 + (zw6-zw7)*zdif4*( zf4+zdfrac(jl,jkp1)*(1._JPRB-zf4) )

!           -----------------------------------------------------------
!           Infrared heating rate "zdtlw(jl,jk) in layer "jk" due to
!           all terms zq1, zq2, zq3 and zq4
!           ---------------------------------------------------------

            zdtlw(jl,jk)=zgrrcp*( zq1 +zq2 + zq3 + zq4 )/pdpf(jl,jk)
        ENDDO
      ENDDO

!     ---------------------------------------------------------------
!     Temperature tendencies of uppermost layer and of lowest model
!     layer.
!     ---------------------------------------------------------------

      DO jl=kstart,kstop

! Assume pure "cooling to space" in the uppermost layer,
! instead of setting the temperature tendency to zero
        zdtlw(jl,1)=0.5_JPRB*(zbb(jl,1)+zbb(jl,2))*(zemiqu(jl,1)-zemiqu(jl,2))  &
     &                *zgrrcp/(pdpf(jl,1)+zeps)
!       -------------------------------------------------------------
        zw1=1._JPRB-MAX( zefcov(jl,klev),zefcovu(jl,klev) )
        zt=zbb(jl,ilevp1)
        zfls=(zslwup(jl)-zt)*(zemiqb(jl,klev)-zemiqb(jl,ilevp1))

        zq=zt*(zemiqu(jl,klev)-zemiqu(jl,ilevp1))  &
     &        -zcadd(klev)*pdpf(jl,klev)/zgrrcp
        zq1=zw1*( zq +zfls )
!       -------------------------------------------------------------
        zw2=MAX( zefcovu(jl,klev) -zefcov(jl,klev), 0._JPRB )
        zf2=MIN( (ph(jl,klev) -ph(jl,icovu(jl,klev)+1))/cpc, 1._JPRB )
        zq2=zw2*( zq*zf2 + zfls )
!       -------------------------------------------------------------
        zw4=MAX( zefcov(jl,klev) -zefcovu(jl,klev), 0._JPRB )

! The empirical correction for e-type continuum is no longer needed!
        zq3=zw4*( -zbb(jl,klev) +0.5_JPRB*( zbb(jl,klev) +zbb(jl,ilevm1))  &
     &       *zemiqu(jl,klev) + zcsur1_wv(jl)*phybh(klev) - zbb(jl,ilevp1) &
     &       +zslwup(jl))
!lrcj+zemiqu(jl,klev) +csur1*phybh(klev) -zbb(jl,ilevp1) +zslwup(jl))

!       -------------------------------------------------------------
        zw6=MIN( zefcovu(jl,klev), zefcov(jl,klev) )
        zw9=MAX( zw6 -zefcov(jl,ilevm1), 0._JPRB )
        zf1=MIN( cpf/(ph(jl,klev)-ph(jl,icovu(jl,klev)+1)+zeps),1._JPRB )
        zdif1=zbb(jl,icovu(jl,klev)+1) -zbb(jl,klev)

        zq4=zw9*zdif1*(zf1 +zdfrac(jl,ilevm1)*(1._JPRB-zf1))
        zq4= zq4 + zw6*( zslwup(jl) -zbb(jl,ilevp1) )

!       -------------------------------------------------------------

        zdtlw(jl,klev)=zgrrcp*( zq1 + zq2 + zq3 + zq4 )/pdpf(jl,klev)
      ENDDO

!     ======================================================

!     ===================
!     Shortwave radiation
!     ===================

!     Local solar zenith angle:
!     -------------------------
      DO jl=kstart,kstop
        zcos=pscos(jl)
        zscosre(jl) = zcos
        zcos=MAX( zcos, cosmin )
        zscos(jl)=zcos
        zscos03(jl)=zcos**0.3_JPRB
      ENDDO

      DO jl=kstart,kstop
        zfstopc(jl)=0._JPRB
        psswnet(jl)=0._JPRB
        ptswnet(jl)=0._JPRB
        ptswdn(jl)=0._JPRB
        DO jk=1,klev+1
           pswnet(jl,jk)=0._JPRB
           zdtsw(jl,jk)=0._JPRB
        ENDDO
      ENDDO

!     Initialize horizontal arrays
!     ----------------------------
      DO jl=kstart,kstop
        zcovmax(jl)=0._JPRB
        zsumcwi(jl)=0._JPRB
        ztrans(jl)=1._JPRB
        zcloabs(jl)=0._JPRB
        ima(jl)=klev+1
        ibot(jl)=klev
      ENDDO

! Solar angle correction to surface alb 
! ----------------------------------------------------------------------
      DO jl = kstart,kstop
        zwsalb(jl)=palbedo(jl)+psalbcor(jl) 
      ENDDO
!     ------------------------------------------------------
!     Determine maximum cloud cover "zcovmax" in the vertical
!     column.
!     ------------------------------------------------------
      DO jk=1,klev
        DO jl=kstart,kstop
          IF( ptotcov(jl,jk) > zcovmax(jl) ) THEN
            zcovmax(jl)=ptotcov(jl,jk)
          ENDIF

          IF( (ptotcov(jl,jk) > zeps).AND.(ima(jl) > klev) ) THEN
            ima(jl)=jk
          ENDIF
          jkm1=MAX(1,jk-1)
          IF ( (ptotcov(jl,jk) < zeps).AND.(ptotcov(jl,jkm1) > zeps) ) THEN
             ibot(jl)=jk-1
          ENDIF
        ENDDO
      ENDDO

!     ------------------------------------------------------
!     Define effective radius zresw for SW calculations
!     ------------------------------------------------------
        DO jl=kstart,kstop
           zrewsum=0._JPRB
           zreqsum=0._JPRB
           zweisum=0._JPRB
           DO jk=1,klev
              jkm1=MAX(1,jk-1)
              zreq(jl,jk)=a1*zreice(jl,jk)+a2*zscos(jl)+a3
              zwei=pcice(jl,jk)*pdpf(jl,jk)
              zreqsum=zreqsum+zwei*zreq(jl,jk)
              zweisum=zweisum+zwei
              zwei=pclq(jl,jk)*pdpf(jl,jk)
              zrewsum=zrewsum+zwei*zrewat(jl,jk)
              zweisum=zweisum+zwei
              IF (zweisum > cqpmin) THEN
                 zresw(jl,jk)=(zreqsum+zrewsum)/zweisum
              ELSE
                 zresw(jl,jk)=0._JPRB
              ENDIF
              IF ((pclq(jl,jk)+pcice(jl,jk)) > 1.0E-08_JPRB) THEN
                 zresw(jl,jk)=MAX(remn,zresw(jl,jk))
              ELSE
                 zresw(jl,jk)=MAX(remn,zresw(jl,jkm1))
              ENDIF
           ENDDO
        ENDDO

!     Compute downward solar flux "zfstopc(jl)" at the top of
!     the uppermost cloud layer.
!     ------------------------------------------------------
      DO jl=kstart,kstop
        IF( ima(jl) <= klev ) THEN

!lrkw Define zalbr from cloud properties
!lrkw Calculation copied here from trans/abs calculations below

          DO jk=ima(jl),ibot(jl)
             zsumcwi(jl)=zsumcwi(jl)+zcwin(jl,jk)
          ENDDO
          ztrc=(b13a*zresw(jl,ibot(jl))+b13b)*(zscos(jl)+b14)
          ztrc=MIN(1._JPRB,ztrc/(ztrc+zsumcwi(jl)))
          zabs2=(b10a*zresw(jl,ibot(jl))+b10b)*(zscos(jl)  &
     &         +b11)*LOG(1._JPRB+b12*zsumcwi(jl))
          zalbr=MAX(1._JPRB-ztrc-zabs2,0._JPRB)
          zsumcwi(jl)=0._JPRB

          zuzc=SQRT( zqpath(jl,ima(jl))/zscos(jl))
          zusq=SQRT( zuzc )
          zfstopc(jl)=psa*zscos(jl)*(1._JPRB-0.024_JPRB/SQRT(zscos(jl)) &
     &               -0.11_JPRB*caak*zusq -zcdpr*pf(jl,ima(jl)-1)       &
     &               *( 0.28_JPRB/(1._JPRB+6.43_JPRB*zscos(jl))         &
     &               -0.07_JPRB*zalbr ) )
        ELSE
           zfstopc(jl)=psa*zscosre(jl)
        ENDIF
        zfstopc(jl)=MAX(0._JPRB,zfstopc(jl))
      ENDDO

!     Solar radiative heating by H2O-absorption (gas+cloud droplets);
!     start main loops:
!     ------------------------------------------------------
      DO jk=1,klev
        jkp1=jk+1
          DO jl=kstart,kstop
!           ----------------------------------------------------------
!           Clear sky part as in Savijaervi (J. Appl. Meteor. 1990)
!           ( correction by BHS,i.e. multiplication of "zdtcle"
!           by zscos(jl) to produce correct effect per unit area.
!           ----------------------------------------------------------
            zpath=0.5_JPRB*( zqpath(jl,jkp1) + zqpath(jl,jk) )
            zrbpath=zqpath(jl,ilevp1)/zscos(jl)
            zrpath=0.5_JPRB*crefrad*( zqbpath(jl,jkp1) + zqbpath(jl,jk) ) &
     &             +zrbpath

            IF ( zpath < cpatsh ) THEN
!           --------------------------
!lr In dry case exponents for incoming radiation and 'shtab' for
!lr reflected radiation

               zw1=caadry*(zpath/zscos(jl))**(-cbbdry)
               itabr=NINT( 200._JPRB*zrpath )
               itabr=MAX0( MIN0(itabr,ntabm),1 )
               zdtcle=psa*csac*zzq(jl,jk)*pf(jl,jk)*                    &
     &              (                                                   &
     &              zw1                                                 &
     &              +caahum*palbedo(jl)*crefrad*zscos(jl)*shtab(itabr)  &
     &              )                                                   &
     &              +1.7E-06_JPRB*zscos03(jl)
            ELSE

!lr In humid case 'shtab' for incoming and reflected radiation

              itab=NINT( 200._JPRB*zpath/zscos(jl) )
              itab=MAX0(MIN0( itab,ntabm ),1 )
              itabr=NINT( 200._JPRB*zrpath )
              itabr=MAX0( MIN0(itabr,ntabm),1 )
              zdtcle=psa*csac*zzq(jl,jk)*pf(jl,jk)*caahum*        &
     &             (                                              &
     &              shtab(itab)                                   &
     &              +palbedo(jl)*crefrad*zscos(jl)*shtab(itabr)   &
     &             )                                              &
     &             +1.7E-06_JPRB*zscos03(jl)
            ENDIF

! Add stratospheric ozone absorption to clear-sky SW heating
            zdtcle = zdtcle+0.024_JPRB*psa*SQRT(zscos(jl))*zgrrcp  &
     &           *zfabso3(jk)/(pdpf(jl,jk)+zeps)

!           ----------------------------------------------------------
!           Determine shortwave radiative temperature tendency "zdtsw".
!           Cloud transmission and absorbtion from Slingo, Liou, et al.
!           ----------------------------------------------------------
            zabc=zabf*(1.4_JPRB +zscos(jl))
            ztrc=ztrf*(0.5_JPRB +zscos(jl))

!     Make transmission depend on re
!     ------------------------------
            ztrc=(b13a*zresw(jl,jk)+b13b)*(zscos(jl)+b14)
            IF( jk >= ima(jl) ) THEN
              ztran=ztrans(jl)
            ELSE
              ztran=1.0_JPRB
            ENDIF

!----------------------------------------------------------------------
! An incredible experiment based on an ECMWF idea: multiply zcwin by 0.8
! to simulate the effect of cloud horizontal inhomogeneity, but do not
! touch effective radius calculated above. This probably depends on grid
! size, as does ptotcov already ... Multiplying here only affects SW
! calculations.

            zcwin(jl,jk)=0.8_JPRB*zcwin(jl,jk)
!----------------------------------------------------------------------
!lrkw: make absorption depend on re
!----------------------------------------------------------------------

            zcloabs(jl)=(b10a*zresw(jl,jk)+b10b)*(zscos(jl)+b11) &
     &           *LOG(1._JPRB+b12*zsumcwi(jl))
            zsumcwi(jl)=zsumcwi(jl)+ptotcov(jl,jk)*zcwin(jl,jk)  &
     &           /(zcovmax(jl) +zeps)
            zabs2=(b10a*zresw(jl,jk)+b10b)*(zscos(jl)+b11)       &
     &           *LOG(1._JPRB+b12*zsumcwi(jl))
            zw1=MAX(0._JPRB,zabs2-zcloabs(jl))

            zzdtcle=zdtcle*zcovmax(jl)*ztran

            IF (zscosre(jl) <= 0) THEN
               zdtsw(jl,jk)=0._JPRB
            ELSE
               zdtsw(jl,jk)=zgrrcp/(pdpf(jl,jk)+zeps)      &
     &              *zcovmax(jl)*zfstopc(jl)*zw1           &
     &              +zzdtcle + zdtcle*(1._JPRB-zcovmax(jl))
            ENDIF

            ztrans(jl)=MIN(1._JPRB,ztrc/(ztrc+zsumcwi(jl)))
            zabc=ztran -ztrans(jl)
            zabc=MAX( zabc, ztran*zdtcle*pdpf(jl,jk)/      &
     &           (zgrrcp*zfstopc(jl)+zeps) )
            ztrans(jl)=MIN( 1.0_JPRB, MAX( ztran-zabc, 0.0_JPRB ) )
            ztrans(jl)=MIN(ztrans(jl),ztran)

!           ----------------------------------------------------------
!           Net surface shortwave radiative flux density at the ground
!           ----------------------------------------------------------
            IF( jk == klev ) THEN
              zuzc=SQRT( zqpath(jl,ilevp1)/zscos(jl) )
              zusq=SQRT( zuzc )
              ztrs=psa*zscos(jl)*(1._JPRB -0.024_JPRB/SQRT(zscos(jl))  &
     &             -0.11_JPRB*caak*zusq -zcdpr*pf(jl,klev)            &
     &             *( 0.28_JPRB/(1._JPRB +6.43_JPRB*zscos(jl))         &
     &             -0.07_JPRB*zwsalb(jl) ) )

              zdf=  zfstopc(jl)*zcovmax(jl)*ztrans(jl)/                &
     &              (1._JPRB -zwsalb(jl)*(1._JPRB-ztrans(jl))*zapr)
              zdf=MAX(0._JPRB,zdf)
              zrads=ztrs*(1._JPRB-zcovmax(jl))+zdf                     

              zrads=MAX(0._JPRB,zrads)
              psswdn(jl)=zrads
              psswnet(jl)=(1._JPRB-zwsalb(jl))*psswdn(jl)

!ns Diffuse radiation as a sum of cloudy and clear air contributions,
!ns clear air according to Paltridge-Platt-Savijarvi

              if (pscos(jl) > cosmin) then
                 psswdf(jl) = zdf + &
     &           100._JPRB*(1._JPRB-exp(-2.865_JPRB*asin(pscos(jl))))
              else 
                 psswdf(jl) = zdf
              endif
              psswdf(jl)=min(psswdn(jl),psswdf(jl))
            ENDIF
          ENDDO
      ENDDO

!     ----------------------------------------------------------------
!     Include absorption of radiation from reflected beams
!     in cloudy part of grid box.
!     ----------------------------------------------------------------

      DO jk=1,klev
        DO jl=kstart,kstop
          zfl=zfstopc(jl)*zcovmax(jl)*ztrans(jl)
          zden1=zwsalb(jl)*(1._JPRB-ztrans(jl))
          zden2=zden1*zapr
          IF( jk >= ima(jl) ) THEN
              zarfl=zgrrcp*zfl*( 1._JPRB/(1._JPRB-zden1)  &
     &              -1._JPRB/(1._JPRB-zden2) )/           &
     &              (ph(jl,ilevp1)-ph(jl,ima(jl))+zeps)
          ELSE
              zarfl=0.0_JPRB
          ENDIF

! The absorption of reflected radiation should also be
! accounted for in shortwave heating.
! For computational reasons, a minimum value 'scosmin' has been
! assumed above for the cosine of solar zenith angle. However, if the
! sun is  really below the horizon, 'zarfl' should be set to zero

          IF (zscosre(jl) <= 0.) zarfl=0._JPRB
          zdtsw(jl,jk)=zdtsw(jl,jk) + zarfl

! ---------------------------------------------------
! Total radiative heating as a sum of LW and SW parts
! ---------------------------------------------------

            pdtdt(jl,jk)=zdtlw(jl,jk)+zdtsw(jl,jk)
        ENDDO
      ENDDO

!     ----------------------------------------------------------
!     Predicted temperature changes in the uppermost stra-
!     tosphere ( for "phybf(j) < hybcr" ) are neglected.
!     ----------------------------------------------------------

!     ----------------------------------------------------------------
!     Determine SW and LW net radiative flux densities (W/m2)
!     between layers.
!     ----------------------------------------------------------------

      DO jl=kstart,kstop
         pswnet(jl,ilevp1)=psswnet(jl)
         plwnet(jl,ilevp1)=pslwnet(jl)
      ENDDO

      DO jk=klev,1,-1
        jkp1=jk+1
          DO jl=kstart,kstop
            pswnet(jl,jk)=pswnet(jl,jkp1)     &
     &      + zdtsw(jl,jk)*pdpf(jl,jk)/zgrrcp
            plwnet(jl,jk)=plwnet(jl,jkp1)     &
     &      + zdtlw(jl,jk)*pdpf(jl,jk)/zgrrcp
          ENDDO
      ENDDO

      DO jl=kstart,kstop
         ptswnet(jl)=pswnet(jl,1)
         ptswdn(jl)=MAX(0._JPRB,psa*zscosre(jl))
         ptlwnet(jl)=plwnet(jl,1)
      ENDDO

!     -----------------------------------------------------------
!     Do statistics of clouds and radiation
!     -----------------------------------------------------------

      DO jk=1,klev
        ztrad=pstrad(jk)
        ztdrad=pstdrad(jk)
        ztclo=pstclo(jk)
        ztdclo=pstdclo(jk)

        DO jl=kstart,kstop
          zcl=ptotcov(jl,jk)
          zdtdt=pdtdt(jl,jk)
          ztrad=ztrad + zdtdt
          ztdrad=ztdrad + zdtdt*zdtdt
          ztclo=ztclo + zcl
          ztdclo=ztdclo + zcl*zcl
        ENDDO

        IF (ldstat) THEN
!       ------------------------------------------------------
!       update pstrad(jk),pstdrad(jk),pstclo(jk) and pstdclo(jk)
!       ------------------------------------------------------
          pstrad(jk)=ztrad
          pstdrad(jk)=ztdrad
          pstclo(jk)=ztclo
          pstdclo(jk)=ztdclo
        ENDIF
      ENDDO

IF (LHOOK) CALL DR_HOOK('HLRADIA',1,ZHOOK_HANDLE)
END SUBROUTINE HLRADIA
