! Library:phys $Id: hltend2flx.F90 5126 2007-05-11 16:21:54Z towil $

!options xopt(noeval)
subroutine hltend2flx ( khor,    klev,   kstart,    &
     &                  kstop,   pdpf,   pvar       )

!     ======================================================
!     PURPOSE:
!     Compute conversion of tendencies (1:nlev) 
!     of variable 'pvar(jl,jk)' to fluxes (1:nlev) 
!     remembering that 'pvar' is defined from 0:nlev 
!     and that the flux at the too ('0-level') is zero.
!     -------
!     METHOD: 
!     ------------------------------------------------------
!     Use basic relation to convert from tendency to flux:
!     "Flux" = "tendency" * -pdpf(jl,jk)/gravit 
!     ------------------------------------------------------
!     AUTHOR: Bent H. Sass, DMI.
!     ------------- 
!     MODIFICATIONS: 
!     Original version 2005-03-26.
!     ======================================================
!     1. DECLARATIONS.
 
!     1.1 MODULES USED:

use parkind1  ,only : jpim     ,jprb
use yomhook   ,only : lhook    ,dr_hook 
use yhlconst  ,only : gravit

!     1.2 OTHER DECLARATIONS:
     
implicit none

integer(kind=jpim),intent(in)    ::  khor
integer(kind=jpim),intent(in)    ::  klev
integer(kind=jpim),intent(in)    ::  kstart
integer(kind=jpim),intent(in)    ::  kstop
real(kind=jprb),   intent(in)    ::  pdpf(khor,klev)
real(kind=jprb),   intent(inout) ::  pvar(khor,0:klev)

real(kind=jprb) :: zhook_handle
real(kind=jprb) :: zrgrav

integer(kind=jpim)   ::   jl,  jk

      if (lhook) call dr_hook('hltend2flx',0,zhook_handle)

      zrgrav=1./gravit
     
      do jk=1,klev
        do jl=kstart,kstop
          pvar(jl,jk)=pvar(jl,jk-1) -zrgrav*pdpf(jl,jk)*pvar(jl,jk)
        enddo 
      enddo 

      if (lhook) call dr_hook('hltend2flx',1,zhook_handle)

end subroutine hltend2flx
