SUBROUTINE INITAPLPAR &
 !---------------------------------------------------------------------
 ! - INPUT .
 & (KIDIA , KFDIA  , KLON   , KLEV   , KSGST  , KCSS   ,&
 & PVEG0  ,&
 !---------------------------------------------------------------------
 ! - OUTPUT .
 & PDIFCQ , PDIFCQN, PDIFCQL, PDIFCS , PDIFTQ , PDIFTQN, PDIFTQL,&
 & PDIFTS , PFCCQL , PFCCQN , PFCSQL , PFCSQN , PFCQNG , PFCQING,&
 & PFCQLNG, PFCQRNG, PFCQSNG,&
 & PFPLCL , PFPLCN , PFPLCG , PFPLCH , PFPLSL , PFPLSN , PFPLSG ,&
 & PFPLSH , PFRSO  , PFRSOC ,&
 & PFRTH  , PFRTHC , PSTRCU , PSTRCV , PSTRDU , PSTRDV , PSTRTU ,&
 & PSTRTV , PSTRMU , PSTRMV , PFRMH  , PFCHOZ , PCPS   , PLHS   ,&
 & PRS    , PLH    , PLSCPE , PNEB   , PQICE  , PQLI   , PQSAT  ,&
 & PQW    , PRH    , PTW    , PALB   , PCD    , PCDN   , PCH    ,&
 & PC1    , PC2    , PCT    , PEMIS  , PFCHSP , PFCLL  , PFCLN  ,&
 & PFCS   , PFEVI  , PFEVL  , PFEVN  , PFEVV  , PFTR   , PFLWSP ,&
 & PFONTE , &
 & PFGEL  , PFGELS , PFRSODS, PFRSOPS, PFRSOPT, PFRSOLU, PFRTHDS,&
 & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN, PFPEVPSL,PFPEVPSN,PFPEVPCL,&
 & PFPEVPCN,PFTKE  , PGZ0   , PGZ0H  , PNEIJ  , PVEG   , PQS    , &
 & PQSATS , PRUISL , PRUISP , PRUISS , PUCLS  , PVCLS  , PTCLS  ,&
 & PQCLS  , PRHCLS , PCLCT  , PCLCH  , PCLCM  , PCLCL  , PCLCC  ,&
 & PCAPE  , KCLPH  , PCLPH  , PUGST  , PVGST  )

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0    ,ONLY : LAROME 
#ifdef DOC
!**** *INITAPLPAR*

!     Purpose.
!     --------
!         Initialise outputs of "aplpar" routine.

!**   Interface.
!     ----------
!        *CALL* *INITAPLPAR(...)*

!        Explicit arguments :
!        --------------------

! -   ARGUMENTS D'ENTREE.
! -   INPUT ARGUMENTS.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.
! - DIMENSIONS.

! KIDIA, KFDIA : BORNES BOUCLES HORIZONTALES   (IST,IEND DANS CPG).
! KIDIA, KFDIA : START/END OF HORIZONTAL LOOP  (IST,IEND IN *CPG*).
! KLON : DIMENSION HORIZONTALE                 (NPROMA DANS CPG).
! KLON : HORIZONTAL DIMENSION                  (NPROMA IN *CPG*).
! KLON: IDEM BUT FOR ARRAYS USED SOLELY BY DMN PHYSICS
! KLEV : FIN BOUCLE VERTICE ET DIMENSION VERTICALE (NFLEVG DANS CPG).
! KLEV : END OF VERTICAL LOOP AND VERTICAL DIMENSION(NFLEVG IN *CPG*).
! KSGST      : NOMBRE DE TEMPERATURES ET DE FLUX DE SURFACE SOUS-MAILLE
!                     (NTSSG DANS CPG)
! KSGST      : NUMBER OF SUBGRID SURFACE TEMPERATURES AND FLUXES
!                     (NTSSG IN *CPG*)
! KCSS       : NBRE DE NIVEAUX DANS LE SOL PROFOND
! KCSS       : NBR OF VERTICAL LAYERS IN THE DEEP SOIL

! - 1D (GEOGRAPHIQUE) .
! - 1D (GEOGRAPHICAL DISTRIBUTION) .

! PVEG0      : PROPORTION DE SOL COUVERTE DE VEGETATION.
! PVEG0      : FRACTIONAL COVER BY VEGETATION.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
! -   OUTPUT ARGUMENTS.
!     --------------------

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PDIFCQ     : FLUX CONVECTIF D'HUMIDITE SPECIFIQUE (HORS PLUIE/NEIGE).
! PDIFCQ     : CONVECTIVE FLUX OF SPECIFIC HUMIDITY (NOT RAIN/SNOW).
! PDIFCQN    : FLUX CONVECTIF D'EAU SOLIDE (HORS PLUIE/NEIGE).
! PDIFCQN    : CONVECTIVE FLUX OF SOLID WATER (NOT RAIN/SNOW).
! PDIFCQL    : FLUX CONVECTIF D'EAU LIQUIDE (HORS PLUIE/NEIGE).
! PDIFCQL    : CONVECTIVE FLUX OF LIQUID WATER (NOT RAIN/SNOW).
! PDIFCS     : FLUX CONVECTIF D'ENTHALPIE (HORS PLUIE/NEIGE).
! PDIFCS     : CONVECTIVE FLUX OF ENTHALPY (NOT RAIN/SNOW).
! PDIFTQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
! PDIFTQ     : TURBULENT FLUX (INC. Q NEGATIVE) OF SPECIFIC HUMIDITY.
! PDIFTQN    : FLUX TURBULENT (ET Q NEGATIF) D'EAU GLACE.
! PDIFTQN    : TURBULENT FLUX (INC. Q NEGATIVE) OF SOLID WATER.
! PDIFTQL    : FLUX TURBULENT (ET Q NEGATIF) D'EAU LIQUIDE.
! PDIFTQL    : TURBULENT FLUX (INC. Q NEGATIVE) OF LIQUID WATER.
! PDIFTS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
! PDIFTS     : TURBULENT FLUX OF ENTHALPY (OR DRY STATIC ENERGY).
! PFCHOZ     : FLUX PHOTO-CHIMIQUE D'OZONE.
! PFCHOZ     : OZONE PHOTO-CHEMICAL FLUX.
! PFCCQN     : FLUX DE CONDENSATION LIE AUX RR CONVECTIVES NEIGEUSES.
! PFCCQN     : CONVECTIVE CONDENSATION FLUX FOR ICE.
! PFCCQL     : FLUX DE CONDENSATION LIE AUX RR CONVECTIVES LIQUIDES.
! PFCCQL     : CONVECTIVE CONDENSATION FLUX FOR LIQUID WATER.
! PFCQNG     : FLUX FICTIF DE VAPEUR POUR CORRIGER LES Q<0.
! PFCQNG     : PSEUDO-FLUX OF WATER TO CORRECT FOR Q<0.
! PFCQING    : FLUX FICTIF DE GLACE POUR CORRIGER LES QI<0.
! PFCQING    : PSEUDO-FLUX OF ICE TO CORRECT FOR QI<0.
! PFCQLNG    : FLUX FICTIF D'EAU LIQUIDE POUR CORRIGER LES QL<0.
! PFCQLNG    : PSEUDO-FLUX OF LIQUID WATER TO CORRECT FOR QL<0.
! PFCQRNG    : PSEUDO-FLUX OF RAIN TO CORRECT FOR QL<0.
! PFCQSNG    : PSEUDO-FLUX OF SNOW TO CORRECT FOR QL<0.
! PFPEVPSL   : FLUX DE PRECIPITATIONS DU A L'EVAPORATION RESOLUE.
! PFPEVPSL   : PRECIPITATION FLUX DUE TO RESOLVED EVAPORATION.
! PFPEVPSN   : FLUX DE PRECIPITATIONS DU A LA SUBLIMATION RESOLUE.
! PFPEVPSN   : PRECIPITATION FLUX DUE TO RESOLVED SUBLIMATION.
! PFPEVPCL   : FLUX DE PRECIPITATIONS DU A L'EVAPORATION CONVECTIVE.
! PFPEVPCL   : PRECIPITATION FLUX DUE TO CONVECTIVE EVAPORATION.
! PFPEVPCN   : FLUX DE PRECIPITATIONS DU A LA CONVECTIVE SUBLIMATION.
! PFPEVPCN   : PRECIPITATION FLUX DUE TO SUBLIMATION CONVECTIVE.
! PFTKE      : FLUX DE TKE.
! PFTKE      : TKE FLUX.
! PFPFPSL    : FLUX DE PRECIPITATION RESOLUE: TERME DE GENERATION/FORMATION. 
! PFPFPSL    : FLUX OF RESOLVED PRECIPITATION : THE GENERATION TERM. 
! PFPFPSN    : FLUX DE PRECIPITATION RESOLUE: TERME DE GENERATION/FORMATION.
! PFPFPSN    : FLUX OF RESOLVED PRECIPITATION : THE GENERATION TERM. 
! PFPFPCL    : FLUX DE PRECIPITATION CONVECTIVE: TERME DE GENERATION/FORMATION. 
! PFPFPCL    : FLUX OF CONVECTIVE PRECIPITATION : THE GENERATION TERM. 
! PFPFPCN    : FLUX DE PRECIPITATION CONVECTIVE: TERME DE GENERATION/FORMATION.
! PFPFPCN    : FLUX OF CONVECTIVE PRECIPITATION : THE GENERATION TERM. 
! PFPLCL     : PRECIPITATIONS CONVECTIVES SOUS FORME LIQUIDE.
! PFPLCL     : CONVECTIVE PRECIPITATION AS RAIN.
! PFPLCN     : PRECIPITATIONS CONVECTIVES SOUS FORME NEIGE.
! PFPLCN     : CONVECTIVE PRECIPITATION AS SNOW.
! PFPLCG     : PRECIPITATIONS CONVECTIVES SOUS FORME GRAUPEL.
! PFPLCG     : CONVECTIVE PRECIPITATION AS GRAUPEL.
! PFPLCH     : PRECIPITATIONS CONVECTIVES SOUS FORME GRELE.
! PFPLCH     : CONVECTIVE PRECIPITATION AS HAIL.
! PFPLSL     : PRECIPITATIONS STRATIFORMES SOUS FORME LIQUIDE.
! PFPLSL     : STRATIFORM PRECIPITATION AS RAIN.
! PFPLSN     : PRECIPITATIONS STRATIFORMES SOUS FORME NEIGE.
! PFPLSN     : STRATIFORM PRECIPITATION AS SNOW.
! PFPLSG     : PRECIPITATIONS STRATIFORMES SOUS FORME GRAUPEL.
! PFPLSG     : STRATIFORM PRECIPITATION AS GRAUPEL.
! PFPLSH     : PRECIPITATIONS STRATIFORMES SOUS FORME GRELE.
! PFPLSH     : STRATIFORM PRECIPITATION AS HAIL.
! PFRMH      : FLUX MESOSPHERIQUE D'ENTHALPIE.
! PFRMH      : MESOSPHERIC ENTHALPY FLUX.
! PFRSO      : FLUX DE RAYONNEMENT SOLAIRE.
! PFRSO      : SHORTWAVE RADIATIVE FLUX.
! PFRTH      : FLUX DE RAYONNEMENT THERMIQUE.
! PFRTH      : LONGWAVE RADIATIVE FLUX.
! PFCSQN     : FLUX DE CONDENSATION LIE AUX RR STRATIFORMES NEIGEUSES.
! PFCSQN     : STRATIFORM CONDENSATION FLUX FOR ICE.
! PFCSQL     : FLUX DE CONDENSATION LIE AUX RR STRATIFORMES LIQUIDES.
! PFCSQL     : STRATIFORM CONDENSATION FLUX FOR LIQUID WATER.
! PSTRCU     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "U".
! PSTRCU     : CONVECTIVE FLUX OF MOMENTUM "U".
! PSTRCV     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "V".
! PSTRCV     : CONVECTIVE FLUX OF MOMENTUM "V".
! PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
! PSTRDU     : GRAVITY WAVE DRAG FLUX "U".
! PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".
! PSTRDV     : GRAVITY WAVE DRAG FLUX "V".
! PSTRTU     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "U".
! PSTRTU     : TURBULENT FLUX OF MOMENTUM "U".
! PSTRTV     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "V".
! PSTRTV     : TURBULENT FLUX OF MOMENTUM "V".
! PSTRMU     : FLUX MESOSPHERIQUE DE QUANTITE DE MOUVEMENT "U".
! PSTRMU     : MESOSPHERIC FLUX FOR "U"-MOMENTUM.
! PSTRMV     : FLUX MESOSPHERIQUE DE QUANTITE DE MOUVEMENT "V".
! PSTRMV     : MESOSPHERIC FLUX FOR "V"-MOMENTUM.

! - 2D (1:KLEV) .

! PLH        : CHALEUR LATENTE A LA TEMPERATURE DE L'AIR.
! PLH        : LATENT HEAT AT AIR TEMPERATURE.
! PLSCPE     : RAPPORT EFECTIF DES L ET CP EN CONDENSATION/EVAPORATION.
! PLSCPE     : EFFECTIVE RATIO OF L AND CP FOR CONDENSATION/EVAPORATION.
! PNEB       : NEBULOSITE PARTIELLE "RADIATIVE".
! PNEB       : FRACTIONAL CLOUDINESS FOR RADIATION.
! PQICE      : HUMIDITE SPECIFIQUE SOLIDE "RADIATIVE".
! PQICE      : SPECIFIC HUMIDITY OF SOLID WATER FOR RADIATION.
! PQLI       : HUMIDITE SPECIFIQUE LIQUIDE "RADIATIVE".
! PQLI       : SPECIFIC HUMIDITY OF LIQUID WATER FOR RADIATION.
! PQSAT      : HUMIDITE SPECIFIQUE DE SATURATION.
! PQSAT      : SPECIFIC HUMIDITY AT SATURATION.
! PQW        : HUMIDITE SPECIFIQUE DU THERMOMETRE MOUILLE.
! PQW        : SPECIFIC HUMIDITY OF THE WET THERMOMETER.
! PRH        : HUMIDITE RELATIVE.
! PRH        : RELATIVE HUMIDITY.
! PTW        : TEMPERATURE DU THERMOMETRE MOUILLE.
! PTW        : TEMPERATURE OF THE WET THERMOMETER.

! - 2D (CLOUD AND RADIATION I/O FOR ECMWF PHYSICS)

! PEMTD      : DOWNWARD LONGWAVE EMISSIVITY
! PEMTU      : UPWARD   LONGWAVE EMISSIVITY
! PTRSO      : SHORTWAVE TRANSMISSIVITY
! ZTENT      : TENDENCY OF TEMPERATURE.

! - 2D (0:1)
! PFRSOC     : SHORTWAVE CLEAR SKY RADIATIVE FLUX
! PFRTHC     : LONGWAVE CLEAR SKY RADIATIVE FLUX

! - 1D (DIAGNOSTIQUE) .

! PALB       : ALBEDO DE SURFACE COURANT.
! PALB       : MODEL SURFACE SHORTWAVE ALBEDO.
! PCD        : COEFFICIENT D'ECHANGE EN SURFACE POUR U ET V.
! PCD        : EXCHANGE COEFFICIENT AT SURFACE LEVEL FOR U AND V.
! PCDN       : COEFFICIENT NEUTRE D'ECHANGE EN SURFACE.
! PCDN       : EXCHANGE COEFF. AT SURFACE LEVEL IN NEUTRAL CONDITIONS.
! PCH        : COEFFICIENT D'ECHANGE EN SURFACE POUR T ET Q.
! PCH        : EXCHANGE COEFFICIENT AT SURFACE LEVEL FOR T AND Q.
! PCPS       : CHALEUR MASSIQUE DE L'AIR EN SURFACE.
! PC1        : COEFF. HYDRIQUE REPRESENTANT L'INTENSITE AVEC LAQUELLE
!              LES FLUX DE SURFACE PARTICIPENT A L'EVOLUTION DE WS.
! PC1        : HYDROLOGICAL COEFF. SHOWING THE CONTRIBUTION OF SURFACE
!              FLUXES IN THE WS EVOLUTION.
! PC2        : COEFF. HYDRIQUE TRADUISANT LA RAPIDITE DES TRANSFERTS
!              D'EAU ENTRE LES DEUX RESERVOIRS.
! PC2        : HYDROLOGICAL COEFFICIENT SHOWING THE QUICKNESS OF WATER
!              TRANSFERS BETWEEN BOTH TANKS.
! PCT        : COEFFICIENT THERMIQUE DU MILIEU SOL-VEGETATION.
! PCT        : THERMICAL COEFFICIENT OF SOIL-VEGETATION MIDDEL.
! PEMIS      : EMISSIVITE DE SURFACE COURANTE.
! PEMIS      : MODEL SURFACE LONGWAVE EMISSIVITY.
! PFCHSP     : FLUX DE CHALEUR DE LA SURFACE VERS LE SOL PROFOND.
! PFCHSP     : HEAT FLUX FROM SURFACE TO DEEP SOIL.
! PFCLL      : FLUX DE CHALEUR LATENTE SUR EAU LIQUIDE (OU SOL HUMIDE).
! PFCLL      : LATENT HEAT FLUX OVER LIQUID WATER (OR WET SOIL).
! PFCLN      : FLUX DE CHALEUR LATENTE SUR NEIGE (OU GLACE).
! PFCLN      : LATENT HEAT FLUX OVER SNOW (OR ICE).
! PFCS       : FLUX DE CHALEUR SENSIBLE EN SURFACE.
! PFCS       : SENSIBLE HEAT FLUX AT SURFACE LEVEL.
! PFEVI      : FLUX DE VAPEUR D'EAU SUR SOL GELE.
! PFEVI      : WATER VAPOUR FLUX OVER FROZEN SOIL.
! PFEVL      : FLUX DE VAPEUR D'EAU SUR EAU LIQUIDE (OU SOL HUMIDE).
! PFEVL      : WATER VAPOUR FLUX OVER LIQUID WATER (OR WET SOIL)
! PFEVN      : FLUX DE VAPEUR D'EAU SUR NEIGE (OU GLACE) ET SOL GELE.
! PFEVN      : WATER VAPOUR FLUX OVER SNOW (OR ICE) AND FROZEN SOIL.
! PFEVV      : FLUX D'EVAPOTRANSPIRATION.
! PFEVV      : EVAPOTRANSPIRATION FLUX.
! PFTR       : FLUX DE TRANSPIRATION.
! PFTR       : TRANSPIRATION FLUX.
! PFLWSP     : FLUX D'EAU LIQUIDE DE LA SURFACE VERS LE SOL PROFOND.
! PFLWSP     : WATER FLUX FROM SURFACE TO DEEP SOIL.
! PFONTE     : FLUX D'EAU CORRESPONDANT A LA FONTE DE NEIGE EN SURFACE.
! PFONTE     : WATER FLUX CORRESPONDING TO SURFACE SNOW MELT.
! PFRSODS    : FLUX SOLAIRE DESCENDANT EN SURFACE
! PFRSODS    : SURFACE DOWNWARDS SOLAR FLUX
! PFRSOPS    : FLUX SOLAIRE PARALLELE EN SURFACE
! PFRSOPS    : SURFACE PARALLEL SOLAR FLUX
! PFRTHDS    : FLUX IR DESCENDANT EN SURFACE
! PFRTHDS    : SURFACE DOWNWARDS IR FLUX
! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.
! PGZ0       : G * ROUGHNESS LENGTH (CURRENT).
! PGZ0H      : G*LONGUEUR DE RUGOSITE THERMIQUE COURANTE (SI KVCLIV >= 8)
! PGZ0H      : CURRENT G*THERMAL ROUGHNESS LENGTH (IF KVCLIV >= 8)
! PLHS       : CHALEUR LATENTE EN SURFACE.
! PNEIJ      : PROPORTION DE SOL ENNEIGE.
! PNEIJ      : FRACTION OF SOIL COVERED BY SNOW.
! PVEG       : FRACTION DE VEGETATION APPARENTE.
! PVEG       : FRACTIONAL COVER BY APPARENT VEGETATION.
! PQS        : HUMIDITE SPECIFIQUE DE SURFACE.
! PQS        : SPECIFIC HUMIDITY AT SURFACE LEVEL.
! PQSATS     : HUMIDITE SPECIFIQUE DE SATURATION EN SURFACE.
! PQSATS     : SATURATED SPECIFIC HUMIDITY AT SURFACE LEVEL.
! PRS        : CONSTANTE DES GAZ DE L'AIR EN SURFACE.
! PRUISP     : FLUX DE RUISSELLEMENT EN PROFONDEUR.
! PRUISP     : RUN-OFF FLUX IN SOIL.
! PRUISL     : FLUX DE RUISSELLEMENT DU RESERVOIR D'INTERCEPTION.
! PRUISL     : RUN-OFF FLUX OUT THE INTERCEPTION WATER-TANK.
! PRUISS     : FLUX DE RUISSELLEMENT EN SURFACE.
! PRUISS     : RUN-OFF FLUX AT SURFACE LEVEL.
! PFGEL      : FLUX DE GEL DE L'EAU DU SOL.
! PFGEL      : FREEZING FLUX OF SOIL WATER.
! PFGELS     : FLUX DE GEL DE L'EAU DU RESERVOIR DE SURFACE.
! PFGELS     : FREEZING FLUX OF SOIL WATER AT SURFACE LEVEL.
! PUCLS      : SORTIE DIAGNOSTIQUE DU VENT EN X A HUV METEO.
! PUCLS      : U-COMPONENT OF WIND AT 10 METERS (DIAGNOSTIC).
! PVCLS      : SORTIE DIAGNOSTIQUE DU VENT EN Y A HUV METEO.
! PVCLS      : V-COMPONENT OF WIND AT 10 METERS (DIAGNOSTIC).
! PTCLS      : SORTIE DIAGNOSTIQUE DE LA TEMPERATURE A HTQ METEO.
! PTCLS      : TEMPERATURE AT 2 METERS (DIAGNOSTIC).
! PQCLS      : SORTIE DIAGNOSTIQUE DE L'HUMIDITE SPECIFIQUE A HTQ METEO.
! PQCLS      : SPECIFIC HUMIDITY AT 2 METERS (DIAGNOSTIC).
! PRHCLS     : SORTIE DIAGNOSTIQUE DE L'HUMIDITE RELATIVE A HTQ METEO.
! PRHCLS     : RELATIVE HUMIDITY AT 2 METERS (DIAGNOSTIC).
! PCLCC      : SORTIE DIAGNOSTIQUE DE LA NEBULOSITE CONVECTIVE.
! PCLCC      : CONVECTIVE CLOUD COVER (DIAGNOSTIC).
! PCLCH      : SORTIE DIAGNOSTIQUE DE LA NEBULOSITE HAUTE.
! PCLCH      : HIGH CLOUD COVER (DIAGNOSTIC).
! PCLCL      : SORTIE DIAGNOSTIQUE DE LA NEBULOSITE BASSE.
! PCLCL      : LOW CLOUD COVER (DIAGNOSTIC).
! PCLCM      : SORTIE DIAGNOSTIQUE DE LA NEBULOSITE MOYENNE.
! PCLCM      : MEDIUM CLOUD COVER (DIAGNOSTIC).
! PCLCT      : SORTIE DIAGNOSTIQUE DE LA NEBULOSITE TOTALE.
! PCLCT      : TOTAL CLOUD COVER (DIAGNOSTIC).
! PCAPE      : CAPE.
! PCLPH      : HAUTEUR (EN METRES) DE LA COUCHE LIMITE PLANETAIRE
! PCLPH      : HEIGHT (IN METERS) OF THE PBL
! PUGST      : SORTIE DIAGNOSTIQUE DES RAFALES EN X
! PUGST      : U-COMPONENT OF GUSTS (DIAGNOSTIC).
! PVGST      : SORTIE DIAGNOSTIQUE DES RAFALES EN Y
! PVGST      : V-COMPONENT OF GUSTS (DIAGNOSTIC).

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        2004-09-15: F. Bouyssel

!   Modifications
!   -------------
!        2007-02-25: R. Brozkova, split of autoconversion and prec. evaporation
!                                 fluxes.
!        2007-05-07: Y. Seity, initialisation of Graupel and Hail fluxes

!     ----------------------------------------------------------------------------------
#endif

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSGST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCSS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG0(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCQ(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCQN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCQL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQ(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCCQL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCCQN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCSQL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCSQN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQNG(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQING(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQLNG(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQRNG(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQSNG(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCG(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCH(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSG(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSH(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSO(KLON,0:KLEV,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOC(KLON,0:1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTH(KLON,0:KLEV,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTHC(KLON,0:1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRCU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRCV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRDV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRTU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRTV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRMU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRMV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRMH(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCHOZ(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPFPSL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPFPSN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPFPCL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPFPCN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPEVPSL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPEVPSN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPEVPCL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPEVPCN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFTKE(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCPS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLSCPE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNEB(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQICE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQSAT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALB(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCD(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCDN(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCH(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PC1(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PC2(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCT(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCHSP(KLON,KCSS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCLL(KLON,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCLN(KLON,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCS(KLON,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEVI(KLON,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEVL(KLON,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEVN(KLON,KSGST+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEVV(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFTR(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLWSP(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFONTE(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFGEL(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFGELS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSODS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOPS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOPT(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOLU(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTHDS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGZ0(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGZ0H(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNEIJ(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVEG(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQSATS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRUISL(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRUISP(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRUISS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRHCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCT(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCH(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCM(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCL(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCC(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCAPE(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLPH(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUGST(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVGST(KLON) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCLPH(KLON) 

INTEGER(KIND=JPIM) :: JLON, JLEV, JCSS, JSG
REAL(KIND=JPRB) :: ZEPS0
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('INITAPLPAR',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

ZEPS0=1.E-12_JPRB

DO JLEV = 0, KLEV
  DO JLON = KIDIA,KFDIA
    PDIFCQ  (JLON,JLEV) = 0.0_JPRB
    PDIFCQN (JLON,JLEV) = 0.0_JPRB
    PDIFCQL (JLON,JLEV) = 0.0_JPRB
    PDIFCS  (JLON,JLEV) = 0.0_JPRB
    PDIFTQ  (JLON,JLEV) = 0.0_JPRB
    PDIFTQN (JLON,JLEV) = 0.0_JPRB
    PDIFTQL (JLON,JLEV) = 0.0_JPRB
    PDIFTS  (JLON,JLEV) = 0.0_JPRB
    PFCQNG  (JLON,JLEV) = 0.0_JPRB
    PFCQING (JLON,JLEV) = 0.0_JPRB
    PFCQLNG (JLON,JLEV) = 0.0_JPRB
    PFCQRNG (JLON,JLEV) = 0.0_JPRB
    PFCQSNG (JLON,JLEV) = 0.0_JPRB
    PFPLCL  (JLON,JLEV) = 0.0_JPRB
    PFPLCN  (JLON,JLEV) = 0.0_JPRB
    PFPLCG  (JLON,JLEV) = 0.0_JPRB
    PFPLCH  (JLON,JLEV) = 0.0_JPRB
    PFPLSL  (JLON,JLEV) = 0.0_JPRB
    PFPLSN  (JLON,JLEV) = 0.0_JPRB
    PFPLSG  (JLON,JLEV) = 0.0_JPRB
    PFPLSH  (JLON,JLEV) = 0.0_JPRB
    PFCCQN  (JLON,JLEV) = 0.0_JPRB
    PFCCQL  (JLON,JLEV) = 0.0_JPRB
    PFCSQN  (JLON,JLEV) = 0.0_JPRB
    PFCSQL  (JLON,JLEV) = 0.0_JPRB
    PSTRCU  (JLON,JLEV) = 0.0_JPRB
    PSTRCV  (JLON,JLEV) = 0.0_JPRB
    PSTRDU  (JLON,JLEV) = 0.0_JPRB
    PSTRDV  (JLON,JLEV) = 0.0_JPRB
    PSTRTU  (JLON,JLEV) = 0.0_JPRB
    PSTRTV  (JLON,JLEV) = 0.0_JPRB
    PSTRMU  (JLON,JLEV) = 0.0_JPRB
    PSTRMV  (JLON,JLEV) = 0.0_JPRB
    PFRMH   (JLON,JLEV) = 0.0_JPRB
    PFCHOZ  (JLON,JLEV) = 0.0_JPRB
    PFPFPSL (JLON,JLEV) = 0.0_JPRB
    PFPFPSN (JLON,JLEV) = 0.0_JPRB
    PFPFPCL (JLON,JLEV) = 0.0_JPRB
    PFPFPCN (JLON,JLEV) = 0.0_JPRB
    PFPEVPSL(JLON,JLEV) = 0.0_JPRB
    PFPEVPSN(JLON,JLEV) = 0.0_JPRB
    PFPEVPCL(JLON,JLEV) = 0.0_JPRB
    PFPEVPCN(JLON,JLEV) = 0.0_JPRB
    PFTKE (JLON,JLEV) = 0.0_JPRB
  ENDDO
ENDDO

DO JSG  = 1, KSGST+1
  DO JLEV = 0, KLEV
    DO JLON = KIDIA,KFDIA
      PFRSO (JLON,JLEV,JSG) = 0.0_JPRB
      PFRTH (JLON,JLEV,JSG) = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

DO JLEV = 1, KLEV
  DO JLON = KIDIA,KFDIA
    PNEB   (JLON,JLEV) = ZEPS0
    PQICE  (JLON,JLEV) = 0.0_JPRB
    PQLI   (JLON,JLEV) = 0.0_JPRB
    PQSAT  (JLON,JLEV) = 0.0_JPRB
    PRH    (JLON,JLEV) = 0.0_JPRB
    PLSCPE (JLON,JLEV) = 0.0_JPRB
    PQW    (JLON,JLEV) = 0.0_JPRB
    PTW    (JLON,JLEV) = 0.0_JPRB
    PLH    (JLON,JLEV) = 0.0_JPRB
  ENDDO
ENDDO

IF (.NOT.LAROME) THEN
  DO JLON = KIDIA,KFDIA
    PVEG    (JLON) = PVEG0(JLON)
  ENDDO
ELSE
  DO JLON = KIDIA,KFDIA
    PVEG    (JLON) = 0.0_JPRB
  ENDDO
ENDIF
   
DO JLON = KIDIA,KFDIA
  PALB    (JLON) = 0.0_JPRB
  PCD     (JLON) = 0.0_JPRB
  PCDN    (JLON) = 0.0_JPRB
  PCH     (JLON) = 0.0_JPRB
  PEMIS   (JLON) = 0.0_JPRB
  PFEVV   (JLON) = 0.0_JPRB
  PFTR    (JLON) = 0.0_JPRB
  PFLWSP  (JLON) = 0.0_JPRB
  PFONTE  (JLON) = 0.0_JPRB
  PFRSODS (JLON) = 0.0_JPRB
  PFRSOPS (JLON) = 0.0_JPRB
  PFRSOPT (JLON) = 0.0_JPRB
  PFRSOLU (JLON) = 0.0_JPRB
  PFRTHDS (JLON) = 0.0_JPRB
  PGZ0    (JLON) = 0.0_JPRB
  PGZ0H   (JLON) = 0.0_JPRB
  PNEIJ   (JLON) = 0.0_JPRB
  PQS     (JLON) = 0.0_JPRB
  PQSATS  (JLON) = 0.0_JPRB
  PRUISP  (JLON) = 0.0_JPRB
  PRUISL  (JLON) = 0.0_JPRB
  PRUISS  (JLON) = 0.0_JPRB
  PFGEL   (JLON) = 0.0_JPRB
  PFGELS  (JLON) = 0.0_JPRB
  PUCLS   (JLON) = 0.0_JPRB
  PVCLS   (JLON) = 0.0_JPRB
  PTCLS   (JLON) = 0.0_JPRB
  PQCLS   (JLON) = 0.0_JPRB
  PRHCLS  (JLON) = 0.0_JPRB
  PCLCT   (JLON) = 0.0_JPRB
  PCLCH   (JLON) = 0.0_JPRB
  PCLCM   (JLON) = 0.0_JPRB
  PCLCL   (JLON) = 0.0_JPRB
  PCLCC   (JLON) = 0.0_JPRB
  PCPS    (JLON) = 0.0_JPRB
  PC1     (JLON) = 0.0_JPRB
  PC2     (JLON) = 0.0_JPRB
  PCT     (JLON) = 0.0_JPRB
  PLHS    (JLON) = 0.0_JPRB
  PRS     (JLON) = 0.0_JPRB
  PCAPE   (JLON) = 0.0_JPRB
  PCLPH   (JLON) = 0.0_JPRB
  PUGST   (JLON) = 0.0_JPRB
  PVGST   (JLON) = 0.0_JPRB
  KCLPH   (JLON) = KLEV
ENDDO

DO JSG  = 1, KSGST+1
  DO JLON = KIDIA,KFDIA
    PFCLL (JLON,JSG) = 0.0_JPRB
    PFCLN (JLON,JSG) = 0.0_JPRB
    PFCS  (JLON,JSG) = 0.0_JPRB
    PFEVI (JLON,JSG) = 0.0_JPRB
    PFEVL (JLON,JSG) = 0.0_JPRB
    PFEVN (JLON,JSG) = 0.0_JPRB
  ENDDO
ENDDO

DO JSG  = 0,1
  DO JLON = KIDIA,KFDIA
    PFRSOC(JLON,JSG) = 0.0_JPRB
    PFRTHC(JLON,JSG) = 0.0_JPRB
  ENDDO
ENDDO

DO JCSS = 1, KCSS
  DO JLON = KIDIA,KFDIA
    PFCHSP(JLON,JCSS) = 0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('INITAPLPAR',1,ZHOOK_HANDLE)
END SUBROUTINE INITAPLPAR
