SUBROUTINE MF_PHYSAD &
 !------------------------------------------------------------
 ! -INPUT in TL and AD
 & (KST,KEND,KGL3,KGL4,KOFF,KSTGLO,LDRETCFOU,PDTPHY, &
 & PGELAM,PGEMU,PGM,POROG, &
 !------------------------------------------------------------
 ! -INPUT in TL, INPUT/OUTPUT in AD
 & PUT0,PVT0,PTT0,PQT0,PQT0L,PQT0M, &
 & PCP0,PDELP0,PRDELP0,PHI0,PHIF0,PRE0,PRE0F,PEVEL0, &
 & PUT9,PVT9,PTT9,PQT9, &
 & PCP9,PDELP9,PRDELP9,PHI9,PHIF9,PRE9,PRE9F, &
 !-----------------------------------------------------------
 ! -INPUT in TL and AD
 & PQS,PQS1,PSNS,PTS,PTS1, &
 !----------------------------------------------------------
 ! -INPUT/OUTPUT in TL and AD
 & PB1,PGMVT1,PGFLT1, &
 !-----------------------------------------------------------
 ! -OUTPUT in TL and AD
 & KFIELDS, &
 !-----------------------------------------------------------
 ! -INPUT for TRAJECTORY in TL and AD
 & PUT5,PVT5,PTT5,PQT5,PQT5L,PQT5M, &
 & PCP5,PDELP5,PRDELP5,PHI5,PHIF5,PRE5,PRE5F,PEVEL5, &
 & PUT95,PVT95,PTT95,PQT95, &
 & PCP95,PDELP95,PRDELP95,PHI95,PHIF95,PRE95,PRE95F, &
 & PSD_VF5)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LSLAG    ,LTWOTL
USE YOMPHY2  , ONLY : TSPHY
USE YOMDIM   , ONLY : NFLSA    ,NFLEVG   ,NFLSUL   ,NPROMA   ,NPROMM
USE YOMDIMT  , ONLY : NPROMT
USE YOMCT3   , ONLY : NSTEP
USE YOMSC2   , ONLY : NFLDSLB1
USE YOMSIMPHL, ONLY : LTRAJPST ,LRAYSP
USE YOMRCOEF , ONLY : LRCOEF   ,NG3SR    ,NGMTR
USE YOMIOP   , ONLY : NG3DT95  ,NG3PR95  ,NG2DSF5
USE YOMDPHY  , ONLY : NTSSG
USE SURFACE_FIELDS_MIX, ONLY : YSD_VFD, YSD_VF
USE YOMFFTTC , ONLY : RBASCOTC ,RBASSITC
USE YOMFOUTC , ONLY : RFORAB3C ,RFORAB4C ,RFORAB6C ,RFORAB3N ,&
 & RFORAB4N ,RFORAB6N ,RFORAT1C ,RFORAT2C ,RFORAT3C ,&
 & RFORAT4C ,RFORAT5C ,RFORAT1N ,RFORAT2N ,RFORAT3N ,&
 & RFORAT4N ,RFORAT5N ,RFOCOR ,RFOAC  
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1T9  ,MSLB1GFL9
USE YOMGMV   , ONLY : YT1
USE YOM_YGFL , ONLY : YGFL, YQ

!**** *MF_PHYSAD* - METEO-FRANCE physics: adjoint grid point calculations.

!     Purpose.
!     --------
!        METEO-FRANCE physics: adjoint grid point calculations.

!**   Interface.
!     ----------
!        *CALL* *MF_PHYSAD(...)*

!        Explicit arguments :
!        --------------------

!     INPUT in TL and AD:
!     -------------------
!        KST       : first element of work.
!        KEND      : last element of work.
!        KGL3,KGL4 : DM-local first and last latitude of computations
!                    - bounds NPROMA-packets for calculations.
!        KOFF      : index offset between NPROMM-arrays and NGPTOT-arrays.
!        KSTGLO    : global offset.
!        LDRETCFOU : .T. -> CPPFTTCINV is called.
!        PDTPHY    : timestep used in the physics.
!        PGELAM    : geographic longitude.
!        PGEMU     : sine of geographic latitude.
!        PGM       : mapping factor.
!        POROG     : surface orography.

!     INPUT in TL, INPUT/OUTPUT in AD:
!     --------------------------------
!        PUT0      : zonal wind time t.
!        PVT0      : meridian wind time t.
!        PTT0      : temperature at time t.
!        PQT0      : humidity at time t.
!        PQT0L     : zonal derivative of humidity at time t.
!        PQT0M     : meridian derivative of humidity at time t.
!        PCP0      : "cp" at time t.
!        PDELP0    : pressure depth "delta prehyd" of layers at time t.
!        PRDELP0   : inverse of "delta prehyd" of layers at time t.
!        PHI0      : geopotential height on interlayers at time t.
!        PHIF0     : geopotential height on layers at time t.
!        PRE0      : hydrostatic pressure "prehyd" on interlayers at time t.
!        PRE0F     : hydrostatic pressure "prehyd" on layers at time t.
!        PEVEL0    : "etadot d prehyd / d eta" on interlayers at time t.
!        PUT9      : zonal wind at time t-dt.
!        PVT9      : meridian wind at time t-dt.
!        PTT9      : temperature at time t-dt.
!        PQT9      : humidity at time t-dt.
!        PCP9      : "cp" at time t-dt.
!        PDELP9    : pressure depth "delta prehyd" of layers at time t-dt.
!        PRDELP9   : inverse of "delta prehyd" of layers at time t-dt.
!        PHI9      : geopotential height on interlayers at time t-dt.
!        PHIF9     : geopotential height on layers at time t-dt.
!        PRE9      : hydrostatic pressure "prehyd" on interlayers at time t-dt.
!        PRE9F     : hydrostatic pressure "prehyd" on layers at time t-dt.

!     INPUT in TL and AD:
!     -------------------
!        PQS       : surface specific humidity at t or t-dt accord. to ltwotl.
!        PQS1      : surface specific humidity at t+dt.
!        PSNS      : mass of snow per unit surf at t or t-dt accord. to ltwotl.
!        PTS       : surface temperature at t or t-dt accord. to ltwotl.
!        PTS1      : surface temperature at t+dt.

!     INPUT/OUTPUT in TL and AD:
!     --------------------------
!        PB1       : "SLB1"-buffer, used for interpolations in the SL scheme.
!        PGMVT1    : GMV t+dt
!        PGFLT1    : GFL t+dt

!     OUTPUT in TL and AD:
!     --------------------
!        KFIELDS   : number of "physical tendencies" fields to be stored
!                    on trajectory buffer.

!     INPUT FOR TRAJECTORY ("5" ARRAYS) in TL and AD:
!     -----------------------------------------------
!        PUT5      : zonal wind time t.
!        PVT5      : meridian wind time t.
!        PTT5      : temperature time t.
!        PQT5      : humidity at time t.
!        PQT5L     : zonal derivative of humidity at time t.
!        PQT5M     : meridian derivative of humidity at time t.
!        PCP5      : "cp" at time t.
!        PDELP5    : pressure depth "delta prehyd" of layers at time t.
!        PRDELP5   : inverse of "delta prehyd" of layers at time t.
!        PHI5      : geopotential height on interlayers at time t.
!        PHIF5     : geopotential height on layers at time t.
!        PRE5      : hydrostatic pressure "prehyd" on interlayers at time t.
!        PRE5F     : hydrostatic pressure "prehyd" on layers at time t.
!        PEVEL5    : "etadot d prehyd / d eta" on interlayers at time t.
!        PUT95     : zonal wind at time t-dt.
!        PVT95     : meridian wind at time t-dt.
!        PTT95     : temperature at time t-dt.
!        PQT95     : humidity at time t-dt.
!        PCP95     : "cp" at time t-dt.
!        PDELP95   : pressure depth "delta prehyd" of layers at time t-dt.
!        PRDELP95  : inverse of "delta prehyd" of layers at time t-dt.
!        PHI95     : geopotential height on interlayers at time t-dt.
!        PHIF95    : geopotential height on layers at time t-dt.
!        PRE95     : hydrostatic pressure "prehyd" on interlayers at time t-dt.
!        PRE95F    : hydrostatic pressure "prehyd" on layers at time t-dt.
!        PSD_VF5   : surface grid-point fields.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        2000-12-04: F. Bouyssel & J.M. Piriou

!     Modifications.
!     --------------
!      Modified :
!       K. YESSAD and F. BOUYSSEL (FEB 2001): adapt code to SL2TL + cleaning.
!       01-03, mesospheric drag - F. Bouyssel
!       M.Hamrud      01-Oct-2003 CY28 Cleaning
!       K. YESSAD (Jan 2004): physics interpolated at O in the AD code.
!       K. YESSAD (Jan 2004): make the code valid for any value of NFLSA.
!       K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL3 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL4 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOFF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDRETCFOU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP0(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHI0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHIF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRE0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRE0F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVEL0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP9(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHI9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHIF9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRE9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRE9F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS1(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS1(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KFIELDS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5L(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT5M(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(NPROMT,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI5(NPROMT,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIF5(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE5(NPROMT,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE5F(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMT,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP95(NPROMT,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI95(NPROMT,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIF95(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE95(NPROMT,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE95F(NPROMT,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSD_VF5(NPROMT,YSD_VFD%NDIM5) 

!     ------------------------------------------------------------------

!     ==== COMPUTED BY CPPHINPAD ====
! ZMU0 : (NPROMM)           ; SOLAR ANGLE
! ZMU0M: (NPROMM)           ; SOLAR ANGLE (mean between radiation steps)
! ZCVGQ: (NPROMM,NFLEVG)    ; CONVERGENCE OF HUMIDITY

REAL(KIND=JPRB) :: ZMU0(NPROMM),ZCVGQ(NPROMM,NFLEVG)
REAL(KIND=JPRB) :: ZMU0M(NPROMM)

!     --- 1D DIAGNOSTIC FIELDS, MOON RADIATION.
! ZMU0LU   : (NPROMM)         ; LUNAR ANGLE.
REAL(KIND=JPRB) :: ZMU0LU(NPROMM)

! ZCVGQ5: (NPROMM,NFLEVG)   ; CONVERGENCE OF HUMIDITY

REAL(KIND=JPRB) :: ZCVGQ5(NPROMM,NFLEVG)

!     ==== FIELDS FOR SIMPL.PHYS.PARAM. ====

REAL(KIND=JPRB) :: ZDIFCQ(NPROMM,0:NFLEVG), ZDIFCS(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZDIFTQ(NPROMM,0:NFLEVG), ZDIFTS(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFCCQN(NPROMM,0:NFLEVG), ZFCCQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFCSQN(NPROMM,0:NFLEVG), ZFCSQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFPLCL(NPROMM,0:NFLEVG), ZFPLCN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFPLSL(NPROMM,0:NFLEVG), ZFPLSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFRSO (NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB) :: ZFRTH (NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB) :: ZSTRCU(NPROMM,0:NFLEVG), ZSTRCV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRDU(NPROMM,0:NFLEVG), ZSTRDV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRTU(NPROMM,0:NFLEVG), ZSTRTV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRMU(NPROMM,0:NFLEVG), ZSTRMV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFRMH (NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFDIS (NPROMA,0:NFLEVG)

REAL(KIND=JPRB) :: ZDIFCQ5(NPROMM,0:NFLEVG), ZDIFCS5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZDIFTQ5(NPROMM,0:NFLEVG), ZDIFTS5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFCCQN5(NPROMM,0:NFLEVG), ZFCCQL5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFCSQN5(NPROMM,0:NFLEVG), ZFCSQL5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFPLCL5(NPROMM,0:NFLEVG), ZFPLCN5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFPLSL5(NPROMM,0:NFLEVG), ZFPLSN5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFRSO5 (NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB) :: ZFRTH5 (NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB) :: ZSTRCU5(NPROMM,0:NFLEVG), ZSTRCV5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRDU5(NPROMM,0:NFLEVG), ZSTRDV5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRTU5(NPROMM,0:NFLEVG), ZSTRTV5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRMU5(NPROMM,0:NFLEVG), ZSTRMV5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZFRMH5 (NPROMM,0:NFLEVG)

REAL(KIND=JPRB) :: ZKTROV5(NPROMM,0:NFLEVG),ZKUROV5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZXTROV5(NPROMM,0:NFLEVG),ZXUROV5(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZCDROV5(NPROMM),ZCHROV5(NPROMM),ZXDROV5(NPROMM),ZXHROV5(NPROMM)

REAL(KIND=JPRB) :: ZTENDU(NPROMM,NFLEVG), ZTENDV(NPROMM,NFLEVG)
REAL(KIND=JPRB) :: ZTENDH(NPROMM,NFLEVG), ZTENDQ(NPROMM,NFLEVG)

REAL(KIND=JPRB) :: ZTENDU5(NPROMM,NFLEVG), ZTENDV5(NPROMM,NFLEVG)
REAL(KIND=JPRB) :: ZTENDH5(NPROMM,NFLEVG), ZTENDQ5(NPROMM,NFLEVG)

REAL(KIND=JPRB) :: ZPOUB(NPROMM,NFLEVG)


!     RADIATION COEFFICIENTS (for preliminary solution)
REAL(KIND=JPRB) :: ZRAB3C(NPROMM,NFLEVG)   ! clear-sky parallel/diffuse reflectivity
REAL(KIND=JPRB) :: ZRAB4C(NPROMM,NFLEVG)   ! clear-sky diffuse transmissivity
REAL(KIND=JPRB) :: ZRAB6C(NPROMM,NFLEVG)   ! clear-sky diffuse reflectivity
REAL(KIND=JPRB) :: ZRAB3N(NPROMM,NFLEVG)   ! cloudy parallel/diffuse reflectivity
REAL(KIND=JPRB) :: ZRAB4N(NPROMM,NFLEVG)   ! cloudy diffuse transmissivity
REAL(KIND=JPRB) :: ZRAB6N(NPROMM,NFLEVG)   ! cloudy diffuse reflectivity
REAL(KIND=JPRB) :: ZRAT1C(NPROMM,NFLEVG)   ! clear-sky parallel transmissivity
REAL(KIND=JPRB) :: ZRAT2C(NPROMM,NFLEVG)   ! clear-sky parallel/diffuse transmissivity
REAL(KIND=JPRB) :: ZRAT3C(NPROMM,NFLEVG)   ! clear-sky parallel/diffuse reflectivity
REAL(KIND=JPRB) :: ZRAT4C(NPROMM,NFLEVG)   ! clear-sky diffuse transmissivity
REAL(KIND=JPRB) :: ZRAT5C(NPROMM,NFLEVG)   ! clear-sky diffuse reflectivity
REAL(KIND=JPRB) :: ZRAT1N(NPROMM,NFLEVG)   ! cloudy parallel transmissivity
REAL(KIND=JPRB) :: ZRAT2N(NPROMM,NFLEVG)   ! cloudy parallel/diffuse transmissivity
REAL(KIND=JPRB) :: ZRAT3N(NPROMM,NFLEVG)   ! cloudy parallel/diffuse reflectivity
REAL(KIND=JPRB) :: ZRAT4N(NPROMM,NFLEVG)   ! cloudy diffuse transmissivity
REAL(KIND=JPRB) :: ZRAT5N(NPROMM,NFLEVG)   ! cloudy diffuse reflectivity
REAL(KIND=JPRB) :: ZCOR(NPROMM,NFLEVG)     ! correction for cloudiness for thermal rad.
REAL(KIND=JPRB) :: ZAC(NPROMM, (NFLEVG+1)*(NFLEVG+1))  ! Curtis matrix
REAL(KIND=JPRB) :: ZAC_HC(NFLEVG+1,NFLEVG+1)   ! horizontally-constant field for ZAC

INTEGER(KIND=JPIM) :: IFIELDSS, IFIELDST, IJL, IPQ, JL1, JL2, JLEV, JROF, JSG

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "aplparsad.intfb.h"
#include "aplparsadt.intfb.h"
#include "cppfttcinv.intfb.h"
#include "cpphinp.intfb.h"
#include "cpphinpad.intfb.h"
#include "cptendsmad.intfb.h"
#include "cptendsmat.intfb.h"
#include "cputqysad.intfb.h"
#include "rdphtrajtm.intfb.h"
#include "rdradcoef.intfb.h"

!     ------------------------------------------------------------------

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('MF_PHYSAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!        1.0 Preliminary calculations.
!            -------------------------

TSPHY = MAX(PDTPHY,1.0_JPRB)

DO JLEV = 0, NFLEVG
  DO JROF = KST, KEND
    ZDIFCQ(JROF,JLEV) = 0.0_JPRB
    ZDIFCS(JROF,JLEV) = 0.0_JPRB
    ZDIFTQ(JROF,JLEV) = 0.0_JPRB
    ZDIFTS(JROF,JLEV) = 0.0_JPRB
    ZFPLCL(JROF,JLEV) = 0.0_JPRB
    ZFPLCN(JROF,JLEV) = 0.0_JPRB
    ZFPLSL(JROF,JLEV) = 0.0_JPRB
    ZFPLSN(JROF,JLEV) = 0.0_JPRB
    ZFCCQN(JROF,JLEV) = 0.0_JPRB
    ZFCCQL(JROF,JLEV) = 0.0_JPRB
    ZFCSQN(JROF,JLEV) = 0.0_JPRB
    ZFCSQL(JROF,JLEV) = 0.0_JPRB
    ZSTRCU(JROF,JLEV) = 0.0_JPRB
    ZSTRCV(JROF,JLEV) = 0.0_JPRB
    ZSTRDU(JROF,JLEV) = 0.0_JPRB
    ZSTRDV(JROF,JLEV) = 0.0_JPRB
    ZSTRTU(JROF,JLEV) = 0.0_JPRB
    ZSTRTV(JROF,JLEV) = 0.0_JPRB
    ZSTRMU(JROF,JLEV) = 0.0_JPRB
    ZSTRMV(JROF,JLEV) = 0.0_JPRB
    ZFRMH (JROF,JLEV) = 0.0_JPRB
    ZFDIS (JROF,JLEV) = 0.0_JPRB
  ENDDO
ENDDO

DO JSG  = 1, NTSSG+1
  DO JLEV = 0, NFLEVG
    DO JROF = KST, KEND
      ZFRSO(JROF,JLEV,JSG) = 0.0_JPRB
      ZFRTH(JROF,JLEV,JSG) = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

DO JLEV = 1, NFLEVG
  DO JROF = KST, KEND
    ZTENDU(JROF,JLEV) = 0.0_JPRB
    ZTENDQ(JROF,JLEV) = 0.0_JPRB
    ZTENDV(JROF,JLEV) = 0.0_JPRB
    ZTENDH(JROF,JLEV) = 0.0_JPRB
    ZCVGQ (JROF,JLEV) = 0.0_JPRB
  ENDDO
ENDDO

CALL CPPHINP (KST,KEND,&
 & PGEMU,PGELAM,&
 & PUT5,PVT5,PQT5,PQT5L,PQT5M,ZPOUB,ZPOUB,&
 & PRDELP5,PEVEL5,ZPOUB,&
 & ZMU0,ZMU0LU,ZMU0M,ZCVGQ5)  

! * transforms Fourier into grid-point transmission coefficients
!   for simplified physics.

IF (LDRETCFOU) THEN
  CALL CPPFTTCINV(KGL3,KGL4,KOFF,&
   & RBASCOTC(1,KOFF),RBASSITC(1,KOFF),&
   & RFORAB3C(1,0,KGL3),RFORAB4C(1,0,KGL3),RFORAB6C(1,0,KGL3),&
   & RFORAB3N(1,0,KGL3),RFORAB4N(1,0,KGL3),RFORAB6N(1,0,KGL3),&
   & RFORAT1C(1,0,KGL3),RFORAT2C(1,0,KGL3),RFORAT3C(1,0,KGL3),&
   & RFORAT4C(1,0,KGL3),RFORAT5C(1,0,KGL3),&
   & RFORAT1N(1,0,KGL3),RFORAT2N(1,0,KGL3),RFORAT3N(1,0,KGL3),&
   & RFORAT4N(1,0,KGL3),RFORAT5N(1,0,KGL3),&
   & RFOCOR(1,0,KGL3),RFOAC(1,0,KGL3),&
   & ZRAB3C,ZRAB4C,ZRAB6C,ZRAB3N,ZRAB4N,ZRAB6N,&
   & ZRAT1C,ZRAT2C,ZRAT3C,ZRAT4C,ZRAT5C,&
   & ZRAT1N,ZRAT2N,ZRAT3N,ZRAT4N,ZRAT5N,&
   & ZCOR,ZAC)  
! * Simplified physics now reads a horizontally-constant field for ZAC:
!   take RFOAC(..,0,kgl3) to initialize ZAC.
  DO JL2=1,NFLEVG+1
    DO JL1=1,NFLEVG+1
      IJL=JL1+(JL2-1)*(NFLEVG+1)
      ZAC_HC(JL2,JL1)=RFOAC(IJL,0,KGL3)
    ENDDO
  ENDDO
ENDIF

IF (LRAYSP.AND.LRCOEF) THEN
  IFIELDSS=NG3SR*NFLEVG
  IFIELDST=NGMTR*(NFLEVG+1)
  CALL RDRADCOEF(KST,KEND,KSTGLO,IFIELDSS,&
   & ZRAB3C,ZRAB3N,ZRAB4C,ZRAB4N,ZRAB6C,ZRAB6N,&
   & ZRAT1C,ZRAT1N,ZRAT2C,ZRAT2N,ZRAT3C,ZRAT3N,&
   & ZRAT4C,ZRAT4N,ZRAT5C,ZRAT5N,ZCOR,ZAC_HC)  
ENDIF

!     ------------------------------------------------------------------

!        1.4 Compute the trajectories for phys.fluxes and tendencies
!            --------------------------------------------------------

IF (LTRAJPST) THEN

  KFIELDS=NG3DT95*NFLEVG+NG3PR95*(NFLEVG+1)+NG2DSF5
  CALL RDPHTRAJTM(NSTEP,KST,KEND,KSTGLO,KFIELDS,&
   & ZCVGQ5 ,&
   & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
   & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
   & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,ZFRSO5 ,ZFRTH5 ,&
   & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,ZSTRTU5,ZSTRTV5,&
   & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
   & ZKTROV5,ZKUROV5,ZXTROV5,ZXUROV5,&
   & ZCDROV5,ZCHROV5,ZXDROV5,ZXHROV5,&
   & ZTENDU5,ZTENDV5,ZTENDH5,ZTENDQ5 )  

ELSE

  IF (LTWOTL) THEN

    CALL APLPARSADT (KST,KEND,NPROMA,1,NFLEVG,NTSSG,NSTEP,&
     & PHI5,PRE5,PHIF5,PRE5F,PDELP5(1,1),PRDELP5,&
     & PUT5,PVT5,PTT5,PQT5,PCP5,ZCVGQ5,&
     & PSNS,PTS,PQS,PTS1,PQS1,&
     & PSD_VF5(1,YSD_VF%YGETRL%MP5),PSD_VF5(1,YSD_VF%YLSM%MP5),PSD_VF5(1,YSD_VF%YVEG%MP5),&
     & PSD_VF5(1,YSD_VF%YZ0F%MP5),PSD_VF5(1,YSD_VF%YVRLAN%MP5),PSD_VF5(1,YSD_VF%YVRLDI%MP5),&
     & PSD_VF5(1,YSD_VF%YALBF%MP5),PSD_VF5(1,YSD_VF%YEMISF%MP5),ZMU0,PGM,&
     & ZAC_HC,ZCOR,&
     & ZRAB3C,ZRAB3N,ZRAB4C,ZRAB4N,ZRAB6C,ZRAB6N,&
     & ZRAT1C,ZRAT1N,ZRAT2C,ZRAT2N,ZRAT3C,ZRAT3N,&
     & ZRAT4C,ZRAT4N,ZRAT5C,ZRAT5N,&
     & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
     & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
     & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,ZFRSO5,ZFRTH5,&
     & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,ZSTRTU5,ZSTRTV5,&
     & ZKTROV5,ZKUROV5,ZXTROV5,ZXUROV5,&
     & ZCDROV5,ZCHROV5,ZXDROV5,ZXHROV5,&
     & ZSTRMU5,ZSTRMV5,ZFRMH5 ,'PARSADT')  

    CALL CPTENDSMAT (NPROMA,KST,KEND,NFLEVG,&
     & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
     & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
     & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,&
     & ZFRSO5,ZFRTH5,&
     & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,&
     & ZSTRTU5,ZSTRTV5,&
     & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
     & PRDELP5,PHIF5,&
     & PUT5,PVT5,PTT5,PQT5,&
     & PQS,PTS,POROG,&
     & ZTENDU5,ZTENDV5,ZTENDH5,ZTENDQ5)  

  ELSE

    CALL APLPARSADT (KST,KEND,NPROMA,1,NFLEVG,NTSSG,NSTEP,&
     & PHI95,PRE95,PHIF95,PRE95F,PDELP95(1,1),PRDELP95,&
     & PUT95,PVT95,PTT95,PQT95,PCP95,ZCVGQ5,&
     & PSNS,PTS,PQS,PTS1,PQS1,&
     & PSD_VF5(1,YSD_VF%YGETRL%MP5),PSD_VF5(1,YSD_VF%YLSM%MP5),PSD_VF5(1,YSD_VF%YVEG%MP5),&
     & PSD_VF5(1,YSD_VF%YZ0F%MP5),PSD_VF5(1,YSD_VF%YVRLAN%MP5),PSD_VF5(1,YSD_VF%YVRLDI%MP5),&
     & PSD_VF5(1,YSD_VF%YALBF%MP5),PSD_VF5(1,YSD_VF%YEMISF%MP5),ZMU0,PGM,&
     & ZAC_HC,ZCOR,&
     & ZRAB3C,ZRAB3N,ZRAB4C,ZRAB4N,ZRAB6C,ZRAB6N,&
     & ZRAT1C,ZRAT1N,ZRAT2C,ZRAT2N,ZRAT3C,ZRAT3N,&
     & ZRAT4C,ZRAT4N,ZRAT5C,ZRAT5N,&
     & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
     & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
     & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,ZFRSO5,ZFRTH5,&
     & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,ZSTRTU5,ZSTRTV5,&
     & ZKTROV5,ZKUROV5,ZXTROV5,ZXUROV5,&
     & ZCDROV5,ZCHROV5,ZXDROV5,ZXHROV5,&
     & ZSTRMU5,ZSTRMV5,ZFRMH5 ,'PARSADT')  

    CALL CPTENDSMAT (NPROMA,KST,KEND,NFLEVG,&
     & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
     & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
     & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,&
     & ZFRSO5,ZFRTH5,&
     & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,&
     & ZSTRTU5,ZSTRTV5,&
     & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
     & PRDELP95,PHIF95,&
     & PUT95,PVT95,PTT95,PQT95,&
     & PQS,PTS,POROG,&
     & ZTENDU5,ZTENDV5,ZTENDH5,ZTENDQ5)  

  ENDIF

ENDIF

!        1.3 Computation of evolution of T, u, v and Q
!            -----------------------------------------

! * Calculation of IPQ, since the old pointer
!   MSLB1[X]9 (=MSLB1GFL9+IP[X]) do not exist any longer in PTRSLB1.
!   (In the future, CPTENDSMAD and CPUTQYSAD must be
!   rewritten in order to treat the GFL globally and not
!   individually).

IPQ=(YQ%MP_SL1-1)*(NFLEVG+2*NFLSUL)

IF (LSLAG.AND.LTWOTL) THEN
  ! * SL2TL
  CALL CPUTQYSAD (NPROMA,KST,KEND,NFLEVG,PDTPHY,&
   & ZTENDH5,ZTENDQ5,ZTENDU5,ZTENDV5,&
   & PCP5,PDELP5(1,1),PTT5,PUT5,PVT5,&
   & ZTENDH,ZTENDQ,ZTENDU,ZTENDV,&
   & PCP0,PDELP0(1,1),PTT0,PUT0,PVT0,&
   & ZFDIS,PB1(1,MSLB1T9+1-NFLSA),PB1(1,MSLB1GFL9+IPQ+1-NFLSA),&
   & PB1(1,MSLB1U9+1-NFLSA),PB1(1,MSLB1V9+1-NFLSA))
ELSEIF (LSLAG.AND.(.NOT.LTWOTL)) THEN
  ! * SL3TL
  CALL CPUTQYSAD (NPROMA,KST,KEND,NFLEVG,PDTPHY,&
   & ZTENDH5,ZTENDQ5,ZTENDU5,ZTENDV5,&
   & PCP95,PDELP95(1,1),PTT95,PUT95,PVT95,&
   & ZTENDH,ZTENDQ,ZTENDU,ZTENDV,&
   & PCP9,PDELP9(1,1),PTT9,PUT9,PVT9,&
   & ZFDIS,PB1(1,MSLB1T9+1-NFLSA),PB1(1,MSLB1GFL9+IPQ+1-NFLSA),&
   & PB1(1,MSLB1U9+1-NFLSA),PB1(1,MSLB1V9+1-NFLSA))
ELSE
  ! * EUL
  CALL CPUTQYSAD (NPROMA,KST,KEND,NFLEVG,PDTPHY,&
   & ZTENDH5,ZTENDQ5,ZTENDU5,ZTENDV5,&
   & PCP95,PDELP95(1,1),PTT95,PUT95,PVT95,&
   & ZTENDH,ZTENDQ,ZTENDU,ZTENDV,&
   & PCP9,PDELP9(1,1),PTT9,PUT9,PVT9,&
   & ZFDIS,PGMVT1(1,1,YT1%MT),PGFLT1(1,1,YQ%MP1),&
   & PGMVT1(1,1,YT1%MU),PGMVT1(1,1,YT1%MV))  
ENDIF

!        1.2 Computation of tendencies T,u,v and Q
!            -------------------------------------

IF (LTWOTL) THEN

  CALL CPTENDSMAD (NPROMA,KST,KEND,NFLEVG,&
   & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
   & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
   & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,&
   & ZFRSO5,ZFRTH5,&
   & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,&
   & ZSTRTU5,ZSTRTV5,&
   & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
   & PRDELP5,PHIF5,&
   & PUT5,PVT5,PTT5,PQT5,&
   & ZDIFCQ,ZDIFCS,ZDIFTQ,ZDIFTS,&
   & ZFCCQL,ZFCCQN,ZFCSQL,ZFCSQN,&
   & ZFPLCL,ZFPLCN,ZFPLSL,ZFPLSN,&
   & ZFRSO,ZFRTH,&
   & ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,&
   & ZSTRTU,ZSTRTV,&
   & ZSTRMU,ZSTRMV,ZFRMH,&
   & PRDELP0,PHIF0,&
   & PUT0,PVT0,PTT0,PQT0,&
   & PQS,PTS,POROG,&
   & ZTENDU5,ZTENDV5,ZTENDH5,ZTENDQ5,&
   & ZTENDU,ZTENDV,ZTENDH,ZTENDQ)  

ELSE

  CALL CPTENDSMAD (NPROMA,KST,KEND,NFLEVG,&
   & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
   & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
   & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,&
   & ZFRSO5,ZFRTH5,&
   & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,&
   & ZSTRTU5,ZSTRTV5,&
   & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
   & PRDELP95,PHIF95,&
   & PUT95,PVT95,PTT95,PQT95,&
   & ZDIFCQ,ZDIFCS,ZDIFTQ,ZDIFTS,&
   & ZFCCQL,ZFCCQN,ZFCSQL,ZFCSQN,&
   & ZFPLCL,ZFPLCN,ZFPLSL,ZFPLSN,&
   & ZFRSO,ZFRTH,&
   & ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,&
   & ZSTRTU,ZSTRTV,&
   & ZSTRMU,ZSTRMV,ZFRMH,&
   & PRDELP9,PHIF9,&
   & PUT9,PVT9,PTT9,PQT9,&
   & PQS,PTS,POROG,&
   & ZTENDU5,ZTENDV5,ZTENDH5,ZTENDQ5,&
   & ZTENDU,ZTENDV,ZTENDH,ZTENDQ)  

ENDIF

!        1.1 Call parametrization
!            --------------------

IF (LTWOTL) THEN

  CALL APLPARSAD (KST,KEND,NPROMA,1,NFLEVG,NTSSG,NSTEP,&
   & PHI5,PRE5,PHIF5,PRE5F,PDELP5(1,1),PRDELP5,&
   & PUT5,PVT5,PTT5,PQT5,PCP5,ZCVGQ5,&
   & PHI0,PRE0,PHIF0,PRE0F,PDELP0(1,1),PRDELP0,&
   & PUT0,PVT0,PTT0,PQT0,PCP0,ZCVGQ,&
   & PSNS,PTS,PQS,PTS1,PQS1,&
   & PSD_VF5(1,YSD_VF%YGETRL%MP5),PSD_VF5(1,YSD_VF%YLSM%MP5),PSD_VF5(1,YSD_VF%YVEG%MP5),&
   & PSD_VF5(1,YSD_VF%YZ0F%MP5),PSD_VF5(1,YSD_VF%YVRLAN%MP5),PSD_VF5(1,YSD_VF%YVRLDI%MP5),&
   & PSD_VF5(1,YSD_VF%YALBF%MP5),PSD_VF5(1,YSD_VF%YEMISF%MP5),ZMU0,PGM,&
   & ZAC_HC,ZCOR,&
   & ZRAB3C,ZRAB3N,ZRAB4C,ZRAB4N,ZRAB6C,ZRAB6N,&
   & ZRAT1C,ZRAT1N,ZRAT2C,ZRAT2N,ZRAT3C,ZRAT3N,&
   & ZRAT4C,ZRAT4N,ZRAT5C,ZRAT5N,&
   & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
   & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
   & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,ZFRSO5,ZFRTH5,&
   & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,ZSTRTU5,ZSTRTV5,&
   & ZKTROV5,ZKUROV5,ZXTROV5,ZXUROV5,&
   & ZCDROV5,ZCHROV5,ZXDROV5,ZXHROV5,&
   & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
   & ZDIFCQ,ZDIFCS,ZDIFTQ,ZDIFTS,&
   & ZFCCQL,ZFCCQN,ZFCSQL,ZFCSQN,&
   & ZFPLCL,ZFPLCN,ZFPLSL,ZFPLSN,ZFRSO,ZFRTH,&
   & ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,ZSTRTU,ZSTRTV,&
   & ZSTRMU,ZSTRMV,ZFRMH,'PARSAD ')  

ELSE

  CALL APLPARSAD (KST,KEND,NPROMA,1,NFLEVG,NTSSG,NSTEP,&
   & PHI95,PRE95,PHIF95,PRE95F,PDELP95(1,1),PRDELP95,&
   & PUT95,PVT95,PTT95,PQT95,PCP95,ZCVGQ5,&
   & PHI9,PRE9,PHIF9,PRE9F,PDELP9(1,1),PRDELP9,&
   & PUT9,PVT9,PTT9,PQT9,PCP9,ZCVGQ,&
   & PSNS,PTS,PQS,PTS1,PQS1,&
   & PSD_VF5(1,YSD_VF%YGETRL%MP5),PSD_VF5(1,YSD_VF%YLSM%MP5),PSD_VF5(1,YSD_VF%YVEG%MP5),&
   & PSD_VF5(1,YSD_VF%YZ0F%MP5),PSD_VF5(1,YSD_VF%YVRLAN%MP5),PSD_VF5(1,YSD_VF%YVRLDI%MP5),&
   & PSD_VF5(1,YSD_VF%YALBF%MP5),PSD_VF5(1,YSD_VF%YEMISF%MP5),ZMU0,PGM,&
   & ZAC_HC,ZCOR,&
   & ZRAB3C,ZRAB3N,ZRAB4C,ZRAB4N,ZRAB6C,ZRAB6N,&
   & ZRAT1C,ZRAT1N,ZRAT2C,ZRAT2N,ZRAT3C,ZRAT3N,&
   & ZRAT4C,ZRAT4N,ZRAT5C,ZRAT5N,&
   & ZDIFCQ5,ZDIFCS5,ZDIFTQ5,ZDIFTS5,&
   & ZFCCQL5,ZFCCQN5,ZFCSQL5,ZFCSQN5,&
   & ZFPLCL5,ZFPLCN5,ZFPLSL5,ZFPLSN5,ZFRSO5,ZFRTH5,&
   & ZSTRCU5,ZSTRCV5,ZSTRDU5,ZSTRDV5,ZSTRTU5,ZSTRTV5,&
   & ZKTROV5,ZKUROV5,ZXTROV5,ZXUROV5,&
   & ZCDROV5,ZCHROV5,ZXDROV5,ZXHROV5,&
   & ZSTRMU5,ZSTRMV5,ZFRMH5 ,&
   & ZDIFCQ,ZDIFCS,ZDIFTQ,ZDIFTS,&
   & ZFCCQL,ZFCCQN,ZFCSQL,ZFCSQN,&
   & ZFPLCL,ZFPLCN,ZFPLSL,ZFPLSN,ZFRSO,ZFRTH,&
   & ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,ZSTRTU,ZSTRTV,&
   & ZSTRMU,ZSTRMV,ZFRMH,'PARSAD ')  

ENDIF

CALL CPPHINPAD (KST,KEND,&
 & PUT5,PVT5,PQT5,PQT5L,PQT5M,PRDELP5,PEVEL5,&
 & PUT0,PVT0,PQT0,PQT0L,PQT0M,PRDELP0,PEVEL0,&
 & ZCVGQ5,ZCVGQ)  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('MF_PHYSAD',1,ZHOOK_HANDLE)
END SUBROUTINE MF_PHYSAD
