SUBROUTINE MTS_PHYS(KIDIA,KFDIA,KLON,KLEV,PGEMU,PGELAM,POROG,PAPRS,PAPRSF,&
     &            PLNPR,PALPH,PU,PV,PT,PQ,PQO3,PQLI,PQICE,PNEB,PTS,PLSM,&
     &            PUCLS,PVCLS,PTCLS,PQCLS,&
     &            PTBMSAT7,PTBMSAT8,PTBMSAT9,PTBGOES11,PTBGOES12,PTBMTSAT1)

  !**** *MTS_PHYS*

  !     PURPOSE
  !     -------
  !**   INTERFACE
  !     ---------
  !     *CALL* *MTS_PHYS*
  
  ! - INPUT .
  !   ------
  ! KIDIA, KFDIA : START/END OF HORIZONTAL LOOP  (IST,ILONLOC IN *CPG*).
  ! KLON         : HORIZONTAL DIMENSION          (NPROMA IN *CPG*).
  ! KLEV         : END OF VERTICAL LOOP AND VERTICAL DIMENSION(NFLEVG IN *CPG*).
  ! PGEMU        : SINE OF GEOGRAPHICAL LATITUDE.
  ! PGELAM       : GEOGRAPHICAL LONGITUDE.
  ! POROG        : SURFACE OROGRAPHY .
  ! PAPRS        : PRESSURE ON HALF-LEVELS.
  ! PAPRSF       : PRESSURE ON FULL LEVELS.
  ! PLNPR        : LOG(PAPRS(JLEV)/PAPRS(JLEV-1)) (FOR HYDROSTATICS).
  ! PALPH        : LOG(PAPRS(JLEV)/PAPRSF(JLEV)) (FOR HYDROSTATICS).
  ! PU           : X-COMPONENT OF WIND.
  ! PV           : Y-COMPONENT OF WIND.
  ! PT           : TEMPERATURE.
  ! PQ           : SPECIFIC HUMIDITY OF WATER VAPOUR.
  ! PQO3         : OZONE MIXING RATIO (MASS).
  ! PQLI         : SPECIFIC HUMIDITY OF LIQUID WATER FOR RADIATION.
  ! PQICE        : SPECIFIC HUMIDITY OF SOLID WATER FOR RADIATION.
  ! PNEB         : FRACTIONAL CLOUDINESS FOR RADIATION.
  ! PTS          : SURFACE LAYER TEMPERATURE.
  ! PLSM         : LAND/SEA MASK.
  ! PUCLS        : U-COMPONENT OF WIND AT 10 METERS (DIAGNOSTIC).
  ! PVCLS        : V-COMPONENT OF WIND AT 10 METERS (DIAGNOSTIC).
  ! PTCLS        : TEMPERATURE AT 2 METERS (DIAGNOSTIC).
  ! PQCLS        : SPECIFIC HUMIDITY AT 2 METERS (DIAGNOSTIC).
  
  ! - OUTPUT .
  !   ------
  ! PTB          : BRIGHTNESS TEMPERATURE (K)
  ! PTBCLD       : CLOUD-AFFECTED BRIGHTNES TEMPERATURE (K)
  
  
  USE PARKIND1  ,ONLY : JPIM     ,JPRB
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
  
  USE YOMCST   , ONLY : RPI,RA
  USE YOMCT0   , ONLY : NPRINTLEV
  USE YOMDIM   , ONLY : NPPM
  USE YOMMTS   , ONLY : NSAT   ,NTYPE
  USE YOMDIMO  , ONLY : NLSAT , NMXTCHU, NPNSAV   ,NPNSSV
  
  USE YOMTVRAD , ONLY : NJPPF   ,RADPRE  &
       &                    ,NVALCHAN,NVALCHAN_MTS,O3CLRT &
       &                    ,SATGRP_TABLE, LRTTOV_INTERPOL
  USE YOMRIP   , ONLY : RCODEC, RSIDEC, RCOVSR, RSIVSR
  IMPLICIT NONE
  
  !-------------------------------------------------
  ! DUMMY VARIABLES.
  !-------------------------------------------------

  INTEGER(KIND=JPIM), INTENT(IN) :: KIDIA
  INTEGER(KIND=JPIM), INTENT(IN) :: KFDIA
  INTEGER(KIND=JPIM), INTENT(IN) :: KLON
  INTEGER(KIND=JPIM), INTENT(IN) :: KLEV

  REAL(KIND=JPRB) ,INTENT(IN) :: PGEMU(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PGELAM(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: POROG(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PTS(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PLSM(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PTCLS(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PQCLS(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PUCLS(KLON)
  REAL(KIND=JPRB) ,INTENT(IN) :: PVCLS(KLON)

  REAL(KIND=JPRB) ,INTENT(IN) :: PAPRS(KLON,0:KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PQO3(KLON,0:KLEV)

  REAL(KIND=JPRB) ,INTENT(IN) :: PAPRSF(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PLNPR(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PALPH(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PU(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PV(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PT(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PQ(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PQLI(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PQICE(KLON,KLEV)
  REAL(KIND=JPRB) ,INTENT(IN) :: PNEB(KLON,KLEV)

  REAL(KIND=JPRB) ,INTENT(OUT) :: PTBMSAT7(KLON,2), PTBMSAT8(KLON,8), PTBMSAT9(KLON,8)
  REAL(KIND=JPRB) ,INTENT(OUT) :: PTBGOES11(KLON,4), PTBGOES12(KLON,4), PTBMTSAT1(KLON,4)

  !-------------------------------------------------
  ! LOCAL VARIABLES.
  !-------------------------------------------------

  ! ZTSTAR5      : Surface temperature (see CTSTAR).
  ! ZT05         : Surface temperature (see CTSTAR).
  ! ZPXP5,ZPXPD5 : Intermediate variables (see PPINIT)
  REAL(KIND=JPRB) :: ZTSTAR5(KLON)
  REAL(KIND=JPRB) :: ZT05(KLON)
  REAL(KIND=JPRB) :: ZPXP5(KLON,0:KLEV,NPPM)
  REAL(KIND=JPRB) :: ZPXPD5(KLON,0:KLEV,NPPM)


  ! ZANGL        : Satellite local zenith angle (deg) .
  ! ZANGA        : Satellite local azimuth angle (deg)
  ! ZANGS        : Solar zenith angle at surface (deg)
  ! ZANGSA       : Relative sat. solar azimuth angle.
  ! ZGRODY       : Grody type MW emissivity coeffs
  ! ISAT         : Satellite index
  REAL(KIND=JPRB) :: ZANGL(NJPPF)
  REAL(KIND=JPRB) :: ZANGS(NJPPF)
  REAL(KIND=JPRB) :: ZANGA(NJPPF)
  REAL(KIND=JPRB) :: ZANGSA(NJPPF)

  CHARACTER(LEN=3) :: CLPP
  INTEGER(KIND=JPIM) :: ICOUNT(KLON)
  REAL(KIND=JPRB), ALLOCATABLE :: ZPRES(:,:),ZCV(:,:)

  REAL(KIND=JPRB), ALLOCATABLE :: ZTQOPP5(:,:,:)
  REAL(KIND=JPRB), ALLOCATABLE :: ZHPPH(:,:,:)
  REAL(KIND=JPRB), ALLOCATABLE :: ZROFIN(:,:)
  REAL(KIND=JPRB), ALLOCATABLE :: ZROFOUT(:,:)

  REAL(KIND=JPRB) :: ZQLI0(KLON,KLEV)
  REAL(KIND=JPRB) :: ZQICE0(KLON,KLEV)
  REAL(KIND=JPRB) :: ZNEB0(KLON,KLEV)

  REAL(KIND=JPRB) :: ZU1(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZV1(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZT1(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZQO31(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZQ1(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZQLI1(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZQICE1(KLON,0:KLEV)
  REAL(KIND=JPRB) :: ZNEB1(KLON,0:KLEV)

  INTEGER(KIND=JPIM)   :: IPMOTOV
  INTEGER(KIND=JPIM)   :: JLEV

  INTEGER(KIND=JPIM) :: ILON,IPACK,ILONFIN,IDONE,JJPACK
  INTEGER(KIND=JPIM) :: INSAV,INSSV,ISAT,ICHPF
  REAL(KIND=JPRB) :: ZPPRES(NLSAT)
  REAL(KIND=JPRB) :: ZGRODY(6,NJPPF)
  INTEGER(KIND=JPIM) :: ISURF(NJPPF)
  REAL(KIND=JPRB) :: ZSAV(NPNSAV,NJPPF)
  REAL(KIND=JPRB) :: ZSSV(NPNSSV,NJPPF)
  REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU)  :: ZTAUSFC,ZEMIS
  INTEGER(KIND=JPIM) :: ICHID(NJPPF*NMXTCHU)
  INTEGER(KIND=JPIM) :: IPFID(NJPPF*NMXTCHU)

  REAL(KIND=JPRB)                              :: ZTBCLD(NJPPF*NMXTCHU),ZTBCLD_INT(KLON,NMXTCHU)

  REAL(KIND=JPRB), ALLOCATABLE                 :: ZCVM(:,:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZPAP(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZPAPH(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZCLDEMIS(:,:)
  INTEGER(KIND=JPIM), ALLOCATABLE              :: IFAIL(:)
  INTEGER(KIND=JPIM), ALLOCATABLE              :: INCH(:) ! <== number of channels
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZTBCLR(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZRADCLR(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZTBTOT(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZRADTOT(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZRADCLD(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZRADCLDLEV(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZOBS(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZVAROUT(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZAIT(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZAIS(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZTAU(:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZAV(:,:,:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZZLAT(:)
  REAL(KIND=JPRB), ALLOCATABLE                 :: ZOROG(:)

  INTEGER(KIND=JPIM) :: JSERIES,ILONLOC,JCH,JFIELD,INLAMTOP,JLON
  INTEGER(KIND=JPIM) :: INAV, ICHANMAX, IC, IJPNSAT
  REAL(KIND=JPRB) :: ZDG2RAD
  REAL(KIND=JPRB) :: ZLAT,ZLON,ZLONSAT,ZGAMMA,ZCOSGAMMA,ZHSAT,ZANGLRAD
  REAL(KIND=JPRB) :: ZANGSRAD,ZMU0
  REAL(KIND=JPRB) :: ZMASK(KLON)

  LOGICAL :: LL_LINTERP, LLCLOUD, LL_RETR_EMIS_TSKIN

  !INTEGER(KIND=JPIM) :: ISTYPE(NJPPF)
  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  INTERFACE
#include "getcparam.h"
  END INTERFACE

#include "ppinit.intfb.h"
#include "ctstar.intfb.h"
#include "abor1.intfb.h"
#include "ppobsa.intfb.h"
#include "extrap.intfb.h"


  IF (LHOOK) CALL DR_HOOK('MTS_PHYS',0,ZHOOK_HANDLE)


  !-------------------------------------------------
  ! Default initialization.
  !-------------------------------------------------


  CALL GETCPARAM(KNAV=INAV,KJPNSAT=IJPNSAT)
  ALLOCATE(ZAV(NLSAT,INAV+2,NJPPF))
  ZDG2RAD=RPI/180._JPRB
  ZHSAT=35800000._JPRB

  PTBMSAT7(:,:)=0._JPRB
  PTBMSAT8(:,:)=0._JPRB
  PTBMSAT9(:,:)=0._JPRB
  PTBGOES11(:,:)=0._JPRB
  PTBGOES12(:,:)=0._JPRB
  PTBMTSAT1(:,:)=0._JPRB

  !                 1. PREPARATION OF INTERPOLATION
  !                 -------------------------------

  CALL PPINIT (KLON,KIDIA,KFDIA,KLEV,PAPRS,PAPRSF,ZPXP5,ZPXPD5)

  !*                  COMPUTE T* AND T0

  CALL CTSTAR(KLON,KIDIA,KFDIA,                            &
       &           PT(KIDIA:KFDIA,KLEV),PAPRS(KIDIA:KFDIA,KLEV),&
       &           PAPRSF(KIDIA:KFDIA,KLEV),POROG,ZTSTAR5,ZT05)

  !               2. Postprocess T, q and O3 to fix pressure levels
  !               -------------------------------------------------

  !*             Switches

  !*              Types of TOVS/RAD1C/SSMI/REO3 processing
  !                 LLTOVCC=Cloud cleared radiances, with 1D-Var
  !                 LLRAD1C=Calibrated 'raw' radiances, no 1D-Var
  !                 LLREO3=Retrieved ozone layers
  !                 SSMI is LLTOVCC. METEOSAT is LLRAD1C

  ALLOCATE(ZPRES(KLON,NLSAT))
  ALLOCATE(ZHPPH(KLON,NLSAT,INAV+2))
  ALLOCATE(ZTQOPP5(KLON,NLSAT,INAV+2))

  ICOUNT(:)=NLSAT
  DO JLEV=1,NLSAT
     ZPRES(KIDIA:KFDIA,JLEV)=RADPRE(JLEV)
  ENDDO

  ILON=KFDIA-KIDIA+1
  IF (KIDIA /= 1) THEN

     !-------------------------------------------------
     ! The physics is designed to be run in the horizontal from KIDIA to KFDIA.
     ! The RTTOV code is designed to be run in the horizontal from 1 to KNPF.
     ! If it happens on some architecture that the physics is run with KIDIA /= 1,
     ! some code will have to be added here to shift the arrays coming from physics.
     !-------------------------------------------------

     CALL ABOR1('MTS_PHYS/ABOR1: KIDIA /= 1. ARRAYS FROM PHYSICS NEED TO BE SHIFTED!')
  ENDIF

  !-------------------------------------------------
  ! Copy NFLEVG arrays on (NFLEVG+1) arrays.
  !-------------------------------------------------

  DO JLEV=1,KLEV
     DO JLON=KIDIA,KFDIA
        ZQLI0(JLON,JLEV)= PQLI(JLON,JLEV)
        ZQICE0(JLON,JLEV)=PQICE(JLON,JLEV)
        ZNEB0(JLON,JLEV)= PNEB(JLON,JLEV)

        ZU1(JLON,JLEV)=PU(JLON,JLEV)
        ZV1(JLON,JLEV)=PV(JLON,JLEV)
        ZT1(JLON,JLEV)=PT(JLON,JLEV)
        ZQ1(JLON,JLEV)=PQ(JLON,JLEV)
        ZQO31(JLON,JLEV)=PQO3(JLON,JLEV)
        ZQLI1(JLON,JLEV)=ZQLI0(JLON,JLEV)
        ZQICE1(JLON,JLEV)=ZQICE0(JLON,JLEV)
        ZNEB1(JLON,JLEV)=ZNEB0(JLON,JLEV)
     ENDDO
  ENDDO
  DO JLON=KIDIA,KFDIA
     ZU1(JLON,0)=ZU1(JLON,1)
     ZV1(JLON,0)=ZV1(JLON,1)
     ZT1(JLON,0)=ZT1(JLON,1)
     ZQ1(JLON,0)=ZQ1(JLON,1)
     ZQO31(JLON,0)=ZQO31(JLON,1)
     ZQLI1(JLON,0)=ZQLI1(JLON,1)
     ZQICE1(JLON,0)=ZQICE1(JLON,1)
     ZNEB1(JLON,0)=ZNEB1(JLON,1)
  ENDDO

  !-------------------------------------------------
  ! Interpolate from the (NFLEVG+1) levels to the RTTOV NLSAT ad hoc levels.
  ! In input: individual arrays: ZU1,ZV1,ZT1,ZQ1,ZQO3,...
  ! In output: the array ZHPPH.
  !-------------------------------------------------

  CLPP='TVC'
  CALL PPOBSA(KLON,ILON,NLSAT,CLPP,NTYPE,ICOUNT,ZPRES,POROG,&
       &           PAPRS,PAPRSF,ZPXP5,ZPXPD5,                    &
       &           ZU1,ZV1,ZT1,ZQ1,ZQO31,ZQLI1,ZQICE1,ZNEB1,     &
       &           ZTSTAR5,ZT05,PLNPR,PALPH,                     &
       &           INAV+2,ZHPPH)
  ZPRES = ZPRES/100.

  DO JLEV=1,NLSAT
    DO JLON=KIDIA,KFDIA
      ! Temperature
      ZTQOPP5(JLON,JLEV,1)=ZHPPH(JLON,JLEV,1)
      ! specific humidity
      ZTQOPP5(JLON,JLEV,2)=ZHPPH(JLON,JLEV,2)
      IF (ZHPPH(JLON,JLEV,2) < 0.0000000001) ZTQOPP5(JLON,JLEV,2)=0.0_JPRB
      ! Ozone: output from PPOBSA not used,
      ! amelioration temporaire pour l'O3 en prenant la climato fournie par RTTOV
      ! stockee dans le module YOMTVRAD
      ZTQOPP5(JLON,JLEV,3)=O3CLRT(JLEV)
      ! Cloud liquid water
      ZTQOPP5(JLON,JLEV,4)=ZHPPH(JLON,JLEV,5)
      IF (ZHPPH(JLON,JLEV,5) < 0.0000000001) ZTQOPP5(JLON,JLEV,4)=0.0_JPRB
      ! Cloud ice water
      ZTQOPP5(JLON,JLEV,5)=ZHPPH(JLON,JLEV,6)
      IF (ZHPPH(JLON,JLEV,6) < 0.0000000001) ZTQOPP5(JLON,JLEV,5)=0.0_JPRB
      ! Cloud content
      ZTQOPP5(JLON,JLEV,6)=ZHPPH(JLON,JLEV,4)
    ENDDO
  ENDDO

  !-------------------------------------------------
  ! Extrapolate temperature above model's top.
  !-------------------------------------------------

  INLAMTOP=0
  DO JLEV=1,NLSAT
     IF(NINT(RADPRE(JLEV)) < NINT(PAPRSF(1,1))) INLAMTOP=JLEV
  ENDDO

  IF (INLAMTOP > 0) THEN
     IPMOTOV=NLSAT-INLAMTOP+1
     ALLOCATE(ZROFIN(KLON,IPMOTOV))
     ALLOCATE(ZROFOUT(KLON,NLSAT))
     ZROFIN(KIDIA:KFDIA,1:IPMOTOV)=ZHPPH(KIDIA:KFDIA,INLAMTOP:NLSAT, 1)
     CALL EXTRAP(ZROFIN,ZROFOUT,KLON)
     DO JLEV=1,INLAMTOP
        ZTQOPP5(KIDIA:KFDIA,JLEV,1)=ZROFOUT(KIDIA:KFDIA,JLEV)
     ENDDO
     DEALLOCATE(ZROFIN,ZROFOUT)
     DEALLOCATE(ZHPPH)
  ENDIF

  !              Cloudy radiance computation
  !              ------------------------------

  ISAT=NSAT

  !-------------------------------------------------
  ! Deal with horizontal dimensionning: the physics has horizontal dimension ILON.
  ! The RTTOV code has  has horizontal dimension NJPPF.
  ! One casts the ILON arrays into packets of NJPPF.
  !-------------------------------------------------

  IPACK=ILON/NJPPF+1
  ILONFIN=MOD(ILON,NJPPF)
  IF(ILONFIN == 0) THEN
     IPACK=IPACK-1
     ILONFIN=NJPPF
  ENDIF

  DO JSERIES=1,SIZE(SATGRP_TABLE) ! Loop over satellites
     ! compute the number of channels
     ICHANMAX=0
     DO IC=1,SIZE(NVALCHAN_MTS,1)
        IF ( NVALCHAN_MTS(IC,JSERIES) /= 0) ICHANMAX=ICHANMAX+1
     ENDDO

     ZTBCLD_INT=0._JPRB

     ! *      mask initialisation for each satellite
     ! *       -------------------------------------
     ZMASK=1._JPRB
     ! *       Pressure Pa--> hPa
     ! *       -------------------------
     ZPPRES(1:NLSAT)=RADPRE(1:NLSAT)/100._JPRB

     ! *       Make packets, Call RT-code.
     ! *       --------------------------------

     IDONE=0
     DO JJPACK=1,IPACK
        !-------------------------------------------------
        ! The width of the current packet is ILONLOC.
        !-------------------------------------------------

        IF(JJPACK == IPACK) THEN
           ILONLOC=ILONFIN
        ELSE
           ILONLOC=NJPPF
        ENDIF

        ! *      Set indices of TOVS channels.
        ! *      ----------------------------------

        ICHPF=0
        DO JLON=1,ILONLOC
           DO JCH=1,ICHANMAX
              ICHPF=ICHPF+1
              ICHID(ICHPF)=NVALCHAN_MTS(JCH,JSERIES)
              IPFID(ICHPF)=JLON
              ZEMIS(ICHPF)=1._JPRB
           ENDDO
        ENDDO

        !-------------------------------------------------
        ! Allocate arrays.
        !-------------------------------------------------

        ALLOCATE (ZCVM(ILONLOC,KLEV,INAV))
        ALLOCATE (ZCV(2,ILONLOC))
        ALLOCATE (ZPAP(ILONLOC,KLEV))
        ALLOCATE (ZPAPH(ILONLOC,KLEV+1))
        ALLOCATE (IFAIL(ILONLOC))
        ALLOCATE (ZTAU(NLSAT,ICHPF))
        ALLOCATE (ZCLDEMIS(ICHPF,KLEV))
        ALLOCATE (ZAIT(ICHPF,KLEV+1))
        ALLOCATE (ZAIS(ICHPF,KLEV+1))
        ALLOCATE (INCH(ILONLOC))
        ALLOCATE (ZZLAT(ILONLOC))
        ALLOCATE (ZOROG(ILONLOC))
        ALLOCATE (ZTBCLR(ICHPF))
        ALLOCATE (ZTBTOT(ICHPF))
        ALLOCATE (ZRADCLR(ICHPF))
        ALLOCATE (ZRADTOT(ICHPF))
        ALLOCATE (ZRADCLD(ICHPF))
        ALLOCATE (ZRADCLDLEV(NLSAT,ICHPF))
        ALLOCATE (ZOBS(ICHPF))
        ALLOCATE (ZVAROUT(4,ICHPF))

        ! *    Temperature, WV and O3 - RTTOV levels
        ! *    ------------------------------------------

        DO JFIELD=1,INAV+2
           DO JLON=1,ILONLOC
              ZAV(:,JFIELD,JLON)=ZTQOPP5(JLON+IDONE,:,JFIELD)
           ENDDO
        ENDDO

        ! *    T, CC, CLW, CIW - model levels
        ! *    -----------------------------------

        DO JLEV=1,KLEV
           DO JLON=1,ILONLOC
              ZCVM(JLON,JLEV,1)=PT(JLON+IDONE,JLEV)
              ZCVM(JLON,JLEV,2)=PNEB(JLON+IDONE,JLEV)

              ZCVM(JLON,JLEV,3)=PQLI(JLON+IDONE,JLEV)
              ZCVM(JLON,JLEV,4)=PQICE(JLON+IDONE,JLEV)

              ZPAP(JLON,JLEV)=PAPRSF(JLON+IDONE,JLEV)/100._JPRB
              ZPAPH(JLON,JLEV+1)=PAPRS(JLON+IDONE,JLEV)/100._JPRB
           ENDDO
        ENDDO

        DO JLON=1,ILONLOC

           !-------------------------------------------------
           ! First half level pressure.
           !-------------------------------------------------

           ZPAPH(JLON,1)=PAPRS(JLON+IDONE,0)/100._JPRB

           !-------------------------------------------------
           ! 2m and 10m values.
           !-------------------------------------------------

           ZSAV(1,JLON)=PTCLS(JLON+IDONE)
           ZSAV(2,JLON)=MAX(1.E-07_JPRB,PQCLS(JLON+IDONE))
           ZSAV(3,JLON)=PAPRS(JLON+IDONE,KLEV)/100._JPRB
           ZSAV(4,JLON)=PUCLS(JLON+IDONE)
           ZSAV(5,JLON)=PVCLS(JLON+IDONE)

           !-------------------------------------------------
           ! Surface properties.
           !-------------------------------------------------

           ISURF(JLON)=1-NINT(PLSM(JLON+IDONE)) ! Surface type (0=land, 1=sea, 2=ice/snow)
           !                       ISTYPE(JLON)=NINT(PLSM(JLON+IDONE))+1 ! Surface type index; 1=sea, 2=land

           ZOROG(JLON)=POROG(JLON+IDONE)
           ZZLAT(JLON)=ASIN(PGEMU(JLON+IDONE))

           ZSSV(1,JLON)=PTS(JLON+IDONE) ! Radiative skin temperature.
           ZSSV(2,JLON)=0._JPRB ! FASTEM-2 land coef s.
           ZSSV(3,JLON)=0._JPRB ! FASTEM-2 land coef inf.
           ZSSV(4,JLON)=0._JPRB ! FASTEM-2 land coef r.
           ZSSV(5,JLON)=0._JPRB ! FASTEM-2 land coef small.
           ZSSV(6,JLON)=0._JPRB ! FASTEM-2 land coef large or Q.

           !-------------------------------------------------------------
           ! choice of the satellite and computation ofthe zenithal angle
           !-------------------------------------------------------------
           IF(     SATGRP_TABLE(ISAT)%rtseries==3 .OR. SATGRP_TABLE(ISAT)%rtseries==4 &
                & .OR. SATGRP_TABLE(ISAT)%rtseries==15 ) THEN


              SELECT CASE(SATGRP_TABLE(JSERIES)%rtseries)
              CASE(3)
                 SELECT CASE(SATGRP_TABLE(JSERIES)%rtid)
                 CASE(7)
                    ZLONSAT=57.5
                 CASE(8)
                    ZLONSAT=3.4
                 CASE(9)
                    ZLONSAT=0.
                 END SELECT
              CASE(4)
                 SELECT CASE(SATGRP_TABLE(JSERIES)%rtid)
                 CASE(11)
                    ZLONSAT=-135.
                 CASE(12)
                    ZLONSAT=-75.
                 END SELECT
              CASE(15)
                 SELECT CASE(SATGRP_TABLE(JSERIES)%rtid)
                 CASE(140)
                    ZLONSAT=140.
                 END SELECT
              END SELECT

              ZLAT=ASIN(PGEMU(JLON+IDONE))
              ZLON=PGELAM(JLON+IDONE)

              ZCOSGAMMA=COS(ZLAT)*COS(ZLON-ZLONSAT)
              ZGAMMA=ACOS(ZCOSGAMMA)
              ZANGLRAD=ZGAMMA+ATAN(RA*SIN(ZGAMMA)/(RA+ZHSAT-RA*ZCOSGAMMA))
              ZANGL(JLON)=ZANGLRAD/ZDG2RAD
              ! the maximum value for the zenital angle allowed for RTTOV is 75 degrees
              IF( ZANGL(JLON) >= 74._JPRB) THEN
                 ZMASK(JLON+IDONE)=0._JPRB
                 ZANGL(JLON)=0._JPRB
              ENDIF
           ELSE
              ZANGL(JLON)=0._JPRB
           ENDIF

           ! calcul de l'angle zenithal solaire
           ZMU0=MAX( RSIDEC*PGEMU(JLON+IDONE)&
                &  -RCODEC*RCOVSR*SQRT(1.0_JPRB-PGEMU(JLON+IDONE)**2)*COS(ZLON)&
                &  +RCODEC*RSIVSR*SQRT(1.0_JPRB-PGEMU(JLON+IDONE)**2)*SIN(ZLON)&
                &  ,0.0_JPRB)
           ZANGSRAD=ACOS(ZMU0)
           ZANGS(JLON)=ZANGSRAD/ZDG2RAD

           ZANGA(JLON)=0._JPRB
           ZANGSA(JLON)=0._JPRB

           !-------------------------------------------------
           ! Grody type MW emissivity coeffs.
           !-------------------------------------------------

           ZGRODY(:,JLON)=1._JPRB
        ENDDO

        ! *   Perform radiative transfer calculations
        ! *    ---------------------------------------------

        INSAV=5
        INSSV=6

        DO JFIELD=1,ILONLOC
           INCH(JFIELD) = ICHANMAX
        ENDDO


        IF( LRTTOV_INTERPOL )THEN
           ! WARNING ! NOT PHASED YET !
           CALL ABOR1('LRTTOV_INTERPOL NOT SUPPORTED YET IN MTS_PHYS')
        ELSE

           ! WARNING ! Preliminary version of code using RTTOV 9
           ! Not yet validated - Vincent Guidard

           LL_LINTERP = .FALSE.
           LLCLOUD = .FALSE.
           LL_RETR_EMIS_TSKIN = .FALSE.
           
           ZCV(1,:)=500.0_JPRB
           ZCV(2,:)=0.0_JPRB

           IF (NPRINTLEV >= 2) THEN
             DO JLON=1,ILONLOC
               DO JLEV=1,NLSAT
                 print*,'MTS_PHYS: ZAV(JLEV,1:INAV+2,JLON)=',ZAV(JLEV,1:INAV+2,JLON)
               ENDDO
             ENDDO
           ENDIF

           CALL RTTOV_EC(IFAIL, ISAT, ICHPF, ILONLOC, NLSAT,              &
                &    ICHID, IPFID,                                         &
                &    TRANSPOSE(ZPRES), ZAV(1:NLSAT,1:INAV+2,1:ILONLOC),ZSAV,ZSSV, ISURF, ZEMIS, ZCV,  &
                &    ZANGL, ZANGA, ZANGS, ZANGSA, ZZLAT, ZOROG,&
                &    LLCLOUD, LL_LINTERP,                                  &
                &    ZTBCLR, ZRADCLR, ZTBTOT, ZRADTOT, ZRADCLD, ZRADCLDLEV,&
                &    ZTAU, ZTAUSFC, LL_RETR_EMIS_TSKIN, ZOBS, ZVAROUT)

        ENDIF

        ZEMIS=1.
        DO JLON=1,ILONLOC
           IF(IFAIL(JLON) /= 0) THEN
              print*,'MTS_PHYS/ATTENTION PROBLEME DE RTTOV : IFAIL('&
                   &                                       ,JLON,')=', IFAIL(JLON)
              print*,'MTS_PHYS: JLON,ILONLOC,IDONE,JLON,KIDIA,KFDIA,ILON='&
                   &                                      ,JLON,ILONLOC,IDONE,JLON,KIDIA,KFDIA,ILON
              DO JLEV=1,NLSAT
                print*,'MTS_PHYS: ZAV(JLEV,1:INAV+2,JLON)=',ZAV(JLEV,1:INAV+2,JLON)
              ENDDO
              print*,'MTS_PHYS: ZSAV(:,JLON)=',ZSAV(:,JLON)
              print*,'MTS_PHYS: ZTBCLD',ZTBCLD(JLON)!!,'  ZTB',ZTB(JLON)
              CALL ABOR1('MTS_PHYS/ABOR1: le tableau IFAIL est non nul')
           ENDIF
        ENDDO


        ! *    Construct output array containing model brigthness temperatures
        ! *    --------------------------------------------------------------------

        ICHPF=0
        DO JLON=1,ILONLOC
           DO  JCH=1,ICHANMAX
              ICHPF=ICHPF+1
              ZTBCLD_INT(JLON+IDONE,NVALCHAN_MTS(JCH,JSERIES))=ZTBTOT(ICHPF)*ZMASK(JLON+IDONE)
           ENDDO
        ENDDO

        ! *    Deallocate arrays
        ! *    ----------------------

        DEALLOCATE (ZCVM)
        DEALLOCATE (ZCV)
        DEALLOCATE (ZPAP)
        DEALLOCATE (ZPAPH)
        DEALLOCATE (IFAIL)
        DEALLOCATE (ZTAU)
        DEALLOCATE (ZCLDEMIS)
        DEALLOCATE (ZAIT)
        DEALLOCATE (ZAIS)
        DEALLOCATE (INCH)
        DEALLOCATE (ZZLAT)
        DEALLOCATE (ZOROG)
        DEALLOCATE (ZTBCLR)
        DEALLOCATE (ZTBTOT)
        DEALLOCATE (ZRADCLR)
        DEALLOCATE (ZRADTOT)
        DEALLOCATE (ZRADCLD)
        DEALLOCATE (ZRADCLDLEV)
        DEALLOCATE (ZOBS)
        DEALLOCATE (ZVAROUT)

        IDONE=IDONE+ILONLOC

     ENDDO ! DO JJPACK

     SELECT CASE(SATGRP_TABLE(JSERIES)%rtseries)
     CASE(3)
        SELECT CASE(SATGRP_TABLE(JSERIES)%rtid)
        CASE(7)
           DO JCH=1,ICHANMAX
              SELECT CASE (NVALCHAN_MTS(JCH,JSERIES))
              CASE(1)
                 PTBMSAT7(:,1)=ZTBCLD_INT(:,1)
              CASE(2)
                 PTBMSAT7(:,2)=ZTBCLD_INT(:,2)
              END SELECT
           ENDDO
        CASE(8)
           DO JCH=1,ICHANMAX
              SELECT CASE (NVALCHAN_MTS(JCH,JSERIES))
              CASE(1)
                 PTBMSAT8(:,1)=ZTBCLD_INT(:,1)
              CASE(2)
                 PTBMSAT8(:,2)=ZTBCLD_INT(:,2)
              CASE(3)
                 PTBMSAT8(:,3)=ZTBCLD_INT(:,3)
              CASE(4)
                 PTBMSAT8(:,4)=ZTBCLD_INT(:,4)
              CASE(5)
                 PTBMSAT8(:,5)=ZTBCLD_INT(:,5)
              CASE(6)
                 PTBMSAT8(:,6)=ZTBCLD_INT(:,6)
              CASE(7)
                 PTBMSAT8(:,7)=ZTBCLD_INT(:,7)
              CASE(8)
                 PTBMSAT8(:,8)=ZTBCLD_INT(:,8)
              END SELECT
           ENDDO
        CASE(9)
           DO JCH=1,ICHANMAX
              SELECT CASE (NVALCHAN_MTS(JCH,JSERIES))
              CASE(1)
                 PTBMSAT9(:,1)=ZTBCLD_INT(:,1)
              CASE(2)
                 PTBMSAT9(:,2)=ZTBCLD_INT(:,2)
              CASE(3)
                 PTBMSAT9(:,3)=ZTBCLD_INT(:,3)
              CASE(4)
                 PTBMSAT9(:,4)=ZTBCLD_INT(:,4)
              CASE(5)
                 PTBMSAT9(:,5)=ZTBCLD_INT(:,5)
              CASE(6)
                 PTBMSAT9(:,6)=ZTBCLD_INT(:,6)
              CASE(7)
                 PTBMSAT9(:,7)=ZTBCLD_INT(:,7)
              CASE(8)
                 PTBMSAT9(:,8)=ZTBCLD_INT(:,8)
              END SELECT
           ENDDO
        END SELECT
     CASE(4)
        SELECT CASE(SATGRP_TABLE(JSERIES)%rtid)
        CASE(11)
           DO JCH=1,ICHANMAX
              SELECT CASE (NVALCHAN_MTS(JCH,JSERIES))
              CASE(1)
                 PTBGOES11(:,1)=ZTBCLD_INT(:,1)
              CASE(2)
                 PTBGOES11(:,2)=ZTBCLD_INT(:,2)
              CASE(3)
                 PTBGOES11(:,3)=ZTBCLD_INT(:,3)
              CASE(4)
                 PTBGOES11(:,4)=ZTBCLD_INT(:,4)
              END SELECT
           ENDDO
        CASE(12)
           DO JCH=1,ICHANMAX
              SELECT CASE (NVALCHAN_MTS(JCH,JSERIES))
              CASE(1)
                 PTBGOES12(:,1)=ZTBCLD_INT(:,1)
              CASE(2)
                 PTBGOES12(:,2)=ZTBCLD_INT(:,2)
              CASE(3)
                 PTBGOES12(:,3)=ZTBCLD_INT(:,3)
              CASE(4)
                 PTBGOES12(:,4)=ZTBCLD_INT(:,4)
              END SELECT
           ENDDO
        END SELECT
     CASE(15)
        SELECT CASE(SATGRP_TABLE(JSERIES)%rtid)
        CASE(140)
           DO JCH=1,ICHANMAX
              SELECT CASE (NVALCHAN_MTS(JCH,JSERIES))
              CASE(1)
                 PTBMTSAT1(:,1)=ZTBCLD_INT(:,1)
              CASE(2)
                 PTBMTSAT1(:,2)=ZTBCLD_INT(:,2)
              CASE(3)
                 PTBMTSAT1(:,3)=ZTBCLD_INT(:,3)
              CASE(4)
                 PTBMTSAT1(:,4)=ZTBCLD_INT(:,4)
              END SELECT
           ENDDO
        END SELECT
     END SELECT

  ENDDO

  DEALLOCATE (ZPRES)
  DEALLOCATE (ZAV)
  IF (LHOOK) CALL DR_HOOK('MTS_PHYS',1,ZHOOK_HANDLE)
END SUBROUTINE MTS_PHYS
