!OPTIONS XOPT(NOEVAL)
SUBROUTINE SUPHMF(KULOUT)

!**** *SUPHMF*   - Calls initialization of commons controlling physics
!                  in the Meteo-France version.

!     Purpose.
!     --------
!           Organise the setup of physical constants for Meteo-France
!           physics package.

!**   Interface.
!     ----------
!        *CALL* *SUPHMF(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        None.

!     Method.
!     -------
!        Irrelevant.

!     Externals.
!     ----------

!     SUPHY0
!     SUPHY1
!     SUPHY2
!     SUPHY3
!     SUTOPH

!     Reference.
!     ----------

!     Author.
!     -------
!        J.-F. Geleyn.

!     Modifications.
!     --------------
!        Original : 91-06-15
!        Modified 91-06-10 by A. Lasserre-Bigorry (call to SUTOPH)
!        Modified 99-03-01 by D. Giard (call to VAL923 for 923 and 927)
!        Modified 01-04-02 R. El Khatib setup for CAPE diagnostic
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified 04-11-16 Y. Seity : call suphmnh for AROME physics
!        R. Zaaboul 28-Feb-2006: call suparar, suphmpa and suphmse (ex suphmnh)
!        Y. Seity 06-07-10: nfpsurfex and lfpart2 in call suphmse (prepsurfex)  
!        E . Bazile 06-09-01 : call to suphmpa if LCVPPKF for ARPEGE/ALADIN.
!     ------------------------------------------------------------------

!*       1.    Call routines for specific physics' commons setup.
!              --------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LSOLV, LCVPPKF
USE YOMARPHY , ONLY : LMPA, LMSE
USE YOMCT0   , ONLY : LSFORC, LFPART2
USE YOMFPC   , ONLY : NFPSURFEX

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sucape.intfb.h"
#include "suphy0.intfb.h"
#include "suphy1.intfb.h"
#include "suphy2.intfb.h"
#include "suphy3.intfb.h"
#include "sutoph.intfb.h"
#include "val923.intfb.h"
#include "suparar.intfb.h"
#include "suphmpa.intfb.h"
#include "suphmse.intfb.h"

IF (LHOOK) CALL DR_HOOK('SUPHMF',0,ZHOOK_HANDLE)
CALL SUPHY0(KULOUT)
CALL SUPHY1(KULOUT)
CALL SUPHY2(KULOUT)
CALL SUPHY3(KULOUT)
CALL SUTOPH(KULOUT)

CALL VAL923(LSOLV)

CALL SUCAPE(KULOUT)

! setup for AROME physics and SURFEX
CALL SUPARAR(KULOUT)
IF (LMPA.OR.LCVPPKF) CALL SUPHMPA(KULOUT)
IF (LMSE.AND.NFPSURFEX==0.AND.(.NOT.LFPART2)) CALL SUPHMSE(KULOUT)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPHMF',1,ZHOOK_HANDLE)
END SUBROUTINE SUPHMF
