SUBROUTINE SUPHMPA(KULOUT)

!**** *SUPHMPA*   - Initialize common meso_NH MODD_ used in physics for AROME

!     Purpose.
!     --------
!           Initialize MODD_PARAMETERS, MODD_CST, MODD_CONF,
!           MODD_RAIN_ICE_DESCR, MODD_RAIN_ICE_PARAM, MODD_BUDGET  
!           parameters used in meso_NH Physics and aladin/meso_NH physics
!           interface 

!**   Interface.
!     ----------
!        *CALL* *SUPHMPA(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMPHY2

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Documentation AROME 

!     Author.
!     -------
!       R. Zaaboul

!     Modifications.
!     --------------
!        Original : 28-Feb-2006
!           E. BAZILE : 01-09-2006 : Modified for LCVPPKF.
!        Y. Seity : 28-March-2007 Move chemistry setup under suphmse       
!        O.Riviere: 01/10/2008 removal of call to now obsolete aro_iniapft
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDYN, ONLY : TSTEP
USE YOMARAR, ONLY : NSPLITR,NSV,LOWARM,LAROBU_ENABLE,NRR

USE YOMARPHY, ONLY : LMICRO,LTURB,LKFBCONV,LMFSHAL,LUSECHEM, LRDUST, &
          & LORILAM,LMPA
USE YOMPHY, ONLY :LPHCDPI, LCVPPKF
USE YOMLDDH   , ONLY : LSDDH, LONLYVAR
USE YOMRIP, ONLY : NINDAT
USE YOMCT0 ,ONLY : LTWOTL
USE YEMGEO, ONLY : LMAP
USE YOMDIM    ,ONLY : NPROMA   ,NFLEVG   


IMPLICIT NONE
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: KULOUT ! UNDETERMINED INTENT
INTEGER(KIND=JPIM)   :: ILON
INTEGER(KIND=JPIM)   :: ILEV
INTEGER(KIND=JPIM)   :: ISV
INTEGER(KIND=JPIM)   :: IRR

REAL(KIND=JPRB) :: ZTSTEP
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!-- Next five strings are kept in modules in MNH, not in AROME !!!
!   ( here they are hardcoded for use in AROINI_BUDGET )

CHARACTER (LEN=4)   :: CLUOUT     ! name of output listing
CHARACTER (LEN=4)   :: CLTURB     ! Kind of turbulence parameterization
CHARACTER (LEN=4)   :: CLTURBDIM  ! dimensionality of the turbulence scheme
CHARACTER (LEN=4)   :: CLRAD      ! Kind of radiation scheme
CHARACTER (LEN=4)   :: CLDCONV    ! Kind of deep convection scheme
CHARACTER (LEN=4)   :: CLCLOUD    ! Kind of microphysical scheme

#include "sucvmnh.intfb.h"
#include "aroini_cstmnh.h"
#include "aroini_micro.h"
#include "aro_subudget.h"
#include "aroini_budget.h"
#include "aroini_turb.h"
#include "aroini_mfshal.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPHMPA',0,ZHOOK_HANDLE)

!       1. Initialisation of MesoNH constantes

CALL AROINI_CSTMNH (KULOUT,LTWOTL,.NOT.LMAP)

!       2. Initialisation for microphysics scheme

ZTSTEP=TSTEP
CALL AROINI_MICRO (KULOUT,ZTSTEP,LOWARM,NSPLITR) 

!       3. Initialisation of Budget

ILON = NPROMA
ILEV = NFLEVG
ISV  = NSV
IRR  = NRR

IF (LMPA.AND.LSDDH) THEN
!LAROBU_ENABLE=LAROBU_ENABLE.OR.(LSDDH.AND..NOT.LONLYVAR).OR.LPHCDPI
LAROBU_ENABLE=.TRUE.
CALL ARO_SUBUDGET(ILON,ILEV,ZTSTEP)
ENDIF
 
!! WARNING:
!!    Values of CLRAD,CLDCONV,CLTURB,CLTURBDIM, CLCLOUD
!!    are hard coded here. In MNH they are saved
!!    in modules.
 
CLUOUT     = 'AIBUO'
CLRAD      = 'NONE'
 
IF(LKFBCONV.OR.LCVPPKF) THEN
  CLDCONV    = 'KAFR'
ELSE
  CLDCONV    = 'NONE'
ENDIF
 
IF(LTURB) THEN
  CLTURB    = 'XXXX'
  CLTURBDIM = '1DIM'
ELSE
  CLTURB    = 'NONE'
  CLTURBDIM = 'NONE'
ENDIF
 
IF(LMICRO) THEN
  CLCLOUD = 'ICE3'
ELSE
  CLCLOUD = 'NONE'
ENDIF
 
IF (LMPA.AND.LSDDH) THEN
CALL AROINI_BUDGET(LAROBU_ENABLE,CLUOUT,KULOUT,ZTSTEP,ISV,IRR, &
                   & CLRAD,CLDCONV,CLTURB,CLTURBDIM, CLCLOUD)


ENDIF


!       4. Initialisation of Turbulence scheme

CALL AROINI_TURB

!       5. Initialisation of Mass Flux Shallow convection scheme

IF(LMFSHAL) CALL AROINI_MFSHAL

!       6. Initialisation of Convection scheme

IF(LKFBCONV.OR.LCVPPKF) THEN
  CALL SUCVMNH(KULOUT)
ENDIF


! -----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPHMPA',1,ZHOOK_HANDLE)
END SUBROUTINE SUPHMPA
