SUBROUTINE SUPHMSE(KULOUT)

!**** *SUPHMSE*   - Initialize common meso_NH MODD_ used in physics for AROME

!     Purpose.
!     --------
!           Initialize MODD_PARAMETERS, MODD_CST, MODD_CONF,
!           MODD_RAIN_ICE_DESCR, MODD_RAIN_ICE_PARAM, MODD_BUDGET  
!           parameters used in meso_NH Physics and aladin/meso_NH physics
!           interface 

!**   Interface.
!     ----------
!        *CALL* *SUPHMSE(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMPHY2

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Documentation AROME 

!     Author.
!     -------
!        R. Zaaboul

!     Modifications.
!     --------------
!        Original : 28-Feb-2006
!        Y. Seity : 28-March-2007 Move chemistry setup from suphmpa
!        A. Alias : 13-June-2007 Set IIEXT and IJEXT for LELAM=.F. and 
!                                new call to aroini_surf when ARPEGE
!        A. Alias : 26-Sept-2007 Call to aroini_surf modified
!        Y. Seity : 11-Jan-2008 Call to aroini_surf modified to check date
!        Y. Seity : 30-01-2008 Add arguments to aroini_surf  
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMARAR, ONLY : NSV,NSWB_MNH,NGPAR,&
                  &MALBDIR,MALBSCA,MVTS,MVEMIS,XSW_BANDS
USE YOMCST,ONLY : RMD,RDAY,REA,REPSM 
USE YOMDIM,ONLY : NPROMA,NDGLG,NDGUXG,NDLON,NDLUXG
USE YOMARPHY, ONLY :  LMPA,LUSECHEM, LRDUST, LORILAM, LRDEPOS,&
                 &   LINITCHEM, LINITDUST, LINITORILAM   
USE YOMGPPB, ONLY : GPARBUF
USE YOMRIP, ONLY : RSOVR, RSIDEC, RCODEC,NINDAT,RTIMST,RHGMT,NSSSSS,RDECLI,&
                &  REQTIM
USE YOMGC, ONLY : GELAM, GEMU
USE YOMGEM, ONLY : NGPTOT, NGPTOT_CAP, NGPTOTG, RLOCEN, RSTRET
USE YOMCT0 ,ONLY : LELAM    ,LTWOTL
USE YOM_YGFL, ONLY : NGFL_EXT
USE YEMGEO, ONLY : LMAP
USE YOMPHY, ONLY : LRAYFM15
USE YOMNSV   , ONLY : NSV_CHEMBEG, NSV_CHEMEND, NSV_DSTBEG, NSV_DSTEND, &
 &                    NSV_AERBEG, NSV_AEREND, NSV_CO2,NSV_DSTDEPBEG, &
 &                    NSV_DSTDEPEND

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)   :: KULOUT

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZCO2 ,ZTETA
REAL(KIND=JPRB), DIMENSION (:,:),ALLOCATABLE :: ZGPAR   
REAL(KIND=JPRB), DIMENSION (:,:),ALLOCATABLE :: ZGPAR2   
INTEGER(KIND=JPIM) :: IIEXT,IJEXT,IENDC,JKGLO,JF,JLON
INTEGER(KIND=JPIM) :: IDAY, IMONTH, IYEAR, IMYPROC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sc2wrg.intfb.h"
#include "abor1.intfb.h"
#include "fctast.h"
#include "aroini_mnhc.h"
#include "aroini_nsv.h"
#include "aroini_surf.h"
#include "whichproc.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPHMSE',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

IF (LRAYFM15) CALL ABOR1('SUPHMSE: FM15 NOT SUPPORTED WITH SURFEX!')

! Initialize MNH constants if not LMPA

IF (.NOT.LMPA) THEN
  CALL AROINI_CSTMNH(KULOUT,LTWOTL,.NOT.LMAP)
ENDIF


!     initialisation of chemistry, aerosols and dust scheme
! ANNEE

WRITE(KULOUT,*)'NINDAT =',NINDAT
IYEAR = NINDAT / 10000
! MOIS

IMONTH = (NINDAT - 10000*IYEAR ) / 100
! JOUR DU MOIS

IDAY = NINDAT - 10000*IYEAR - 100*IMONTH

 CALL WHICHPROC(IMYPROC)

 CALL AROINI_MNHC(LUSECHEM, LORILAM, LRDUST, LRDEPOS,  &
  &                LINITCHEM, LINITDUST, LINITORILAM,   &
  &                IDAY, IMONTH, IYEAR, KULOUT,IMYPROC)




!       initialisation nsv
 CALL AROINI_NSV(NSV_CHEMBEG, NSV_CHEMEND, NSV_AERBEG, NSV_AEREND,    &
   &             NSV_DSTBEG, NSV_DSTEND, NSV_DSTDEPBEG, NSV_DSTDEPEND,&
   &             NSV_CO2)
   WRITE(UNIT=KULOUT,FMT='('' NSV_CHEMBEG = '',I3, &
 & '' NSV_CHEMEND = '',I3,'' NSV_AERBEG = '',I3,'' NSV_AEREND = '',I3, &
 & ''NSV_DSTBEG = '',I3,''NSV_DSTEND = '',I3,                          &
 & ''NSV_DSTDEPBEG = '',I3,''NSV_DSTDEPEND = '',I3,''NSV_CO2 = '',I3)')&
 & NSV_CHEMBEG,NSV_CHEMEND,NSV_AERBEG,NSV_AEREND,NSV_DSTBEG,NSV_DSTEND,&
 & NSV_DSTDEPBEG,NSV_DSTDEPEND,NSV_CO2

!     Initialisation of surface scheme

! for surface
NSV=NGFL_EXT

!initialisation de ZCO2 pour la surface (Il sera cras ensuite sous suecrad/surdi)
ZCO2=360.E-06_JPRB*44.0/RMD


IF (.NOT.ALLOCATED(XSW_BANDS)) ALLOCATE (XSW_BANDS(NSWB_MNH))

!  DO J=1,NSWB_MNH
!    XSW_BANDS(J)=((RLIMIT(J)+RLIMIT(J+1))/2.)*10E-6
!  ENDDO
!On initialise en dur pour l'instant xsw_bands, mais il serait mieux de le
!faire dans le setup du rayonnement (suswn)

SELECT CASE (NSWB_MNH)
  CASE (1)
    XSW_BANDS(1)=999.
  CASE (2)
    XSW_BANDS(1)=0.47_JPRB
    XSW_BANDS(2)=2.345_JPRB
    XSW_BANDS=XSW_BANDS*0.000001_JPRB
  CASE (4)
    XSW_BANDS(1)=0.47_JPRB
    XSW_BANDS(2)=0.94_JPRB
    XSW_BANDS(3)=1.785_JPRB
    XSW_BANDS(4)=3.19_JPRB
    XSW_BANDS=XSW_BANDS*0.000001_JPRB
  CASE (6)
    XSW_BANDS(1)=0.215_JPRB
    XSW_BANDS(2)=0.345_JPRB
    XSW_BANDS(3)=0.565_JPRB
    XSW_BANDS(4)=0.94_JPRB
    XSW_BANDS(5)=1.785_JPRB
    XSW_BANDS(6)=3.19_JPRB
    XSW_BANDS=XSW_BANDS*0.000001_JPRB
  CASE DEFAULT
    CALL ABOR1('SUPHMSE: NSWB_MNH /= 1,2,4,6 not coded')
ENDSELECT

! calculation of astronomical parameters needed by ini_surf
! as it is done in updtim.f90

RHGMT=REAL(MOD(NSSSSS,NINT(RDAY)),JPRB) 
ZTETA=RTETA(RTIMST)
RDECLI=RDS(ZTETA)
REQTIM=RET(ZTETA)
RSOVR =REQTIM+RHGMT
RCODEC=COS(RDECLI)
RSIDEC=SIN(RDECLI)
IF (LELAM) THEN
   IJEXT=NDGLG-NDGUXG
   IIEXT=NDLON-NDLUXG
ELSE
   IJEXT=0
   IIEXT=0
ENDIF

! allocation du buffer
IF (NGPAR /= 0) THEN
   ALLOCATE (ZGPAR(NGPTOT,NGPAR))
   ALLOCATE (ZGPAR2(NPROMA,NGPAR))
   ZGPAR=0.0_JPRB
   ZGPAR2=0.0_JPRB
ENDIF
IF(LELAM) THEN
  CALL AROINI_SURF (KULOUT,NGPTOT,NGPTOT_CAP,NPROMA,NGFL_EXT,NSWB_MNH,&
 & IYEAR, IMONTH, IDAY, NSSSSS,&
 & IIEXT, IJEXT,ZCO2,GEMU,GELAM,RSOVR,RSIDEC,RCODEC,&
 & ZGPAR(:,MALBDIR:MALBDIR+NSWB_MNH-1),&
 & ZGPAR(:,MALBSCA:MALBSCA+NSWB_MNH-1),&
 & ZGPAR(:,MVEMIS),ZGPAR(:,MVTS),XSW_BANDS)  
ELSE
! ARPEGE (gaussian Grid)
  CALL AROINI_SURF (KULOUT,NGPTOT,NGPTOT_CAP,NPROMA,NGFL_EXT,NSWB_MNH,&
 & IYEAR, IMONTH, IDAY, NSSSSS,&
 & IIEXT, IJEXT,ZCO2,GEMU,GELAM,RSOVR,RSIDEC,RCODEC,&
 & ZGPAR(:,MALBDIR:MALBDIR+NSWB_MNH-1),&
 & ZGPAR(:,MALBSCA:MALBSCA+NSWB_MNH-1),&
 & ZGPAR(:,MVEMIS),ZGPAR(:,MVTS),XSW_BANDS,KGPGLO=NGPTOTG,PLOCEN=RLOCEN,PSTRET=RSTRET)  
ENDIF

DO JKGLO=1,NGPTOT,NPROMA
  IENDC=MIN(NPROMA,NGPTOT-JKGLO+1)
  DO JF=1,NGPAR
    DO JLON=1,IENDC
      ZGPAR2(JLON,JF)=ZGPAR(JKGLO+JLON-1,JF)
    ENDDO
  ENDDO
 
  CALL SC2WRG (GPARBUF,JKGLO,1,IENDC,ZGPAR2,1,NGPAR )
ENDDO
DEALLOCATE(ZGPAR)
DEALLOCATE(ZGPAR2)
  
! -----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPHMSE',1,ZHOOK_HANDLE)
END SUBROUTINE SUPHMSE
