!OPTIONS XOPT(NOEVAL)
SUBROUTINE UPDTIER15(KSTEP,KRADFR,PTSTEP)

!**** *UPDTIER15* - UPDATE TIME FOR ECMWF FULL RADIATION COMPUTATIONS
!****               FROZEN VERSION (CYCLE 15) OF UPDTIER

!     Purpose.
!     --------
!     UPDATE TIME OF THE MODEL + 1/2 TIME BETWEEN 2 FULL RADIATION STEPS

!**   Interface.
!     ----------
!        *CALL* *UPDTIER15(KSTEP,KRADFR,PTSTEP)      from *ECRADFR15*

!        Explicit arguments :
!        --------------------
!        KSTEP : TIME STEP INDEX
!        KRADFR: FREQUENCY OF FULL RADIATION COMPUTATIONS
!        PTSTEP: TIME STEP LENGTH

!        Implicit arguments :
!        --------------------
!        YOMRIP15

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        Original : 93-02-22 by Jean-Jacques Morcrette *ECMWF*

!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LTWOTL
USE YOMRIP15 , ONLY : RCODECM15,RSIDECM15,RCOVSRM15,RSIVSRM15
USE YOMRIP   , ONLY : NSSSSS   ,RTIMST   ,RTIMTR
USE YOMCST   , ONLY : RPI      ,RDAY     ,REA      ,REPSM    ,&
 & RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KRADFR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTEP 
INTEGER(KIND=JPIM) :: IHGMT, ISTADD, ISTASS, ITIME, IZT

REAL(KIND=JPRB) :: ZANGOZC, ZCOTHOZ, ZDEASOM, ZDECLIM, ZEQTIMM,&
 & ZHGMT, ZSITHOZ, ZSOVRM, ZSTATI, ZTETA, ZTHETOZ, &
 & ZTIMTR, ZWSOVRM  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "suecozo.intfb.h"

#include "fctast.h"
#include "fcttim.h"
#include "fcttrm.h"
!     ------------------------------------------------------------------

!          1.  TIME EQUATION
!              -------------

IF (LHOOK) CALL DR_HOOK('UPDTIER15',0,ZHOOK_HANDLE)
ITIME=NINT(PTSTEP)
IF (LTWOTL) THEN
  IZT=NINT(PTSTEP*(REAL(KSTEP,JPRB)+0.5_JPRB))
ELSE
  IZT=ITIME*KSTEP
ENDIF
IZT=ITIME*KSTEP

ZSTATI=REAL(IZT,JPRB)+0.5_JPRB*KRADFR*ITIME
ISTADD=IZT/NINT(RDAY)
ISTASS=MOD(IZT,NINT(RDAY))
ZTIMTR=RTIMST+ZSTATI
IHGMT=MOD(NINT(ZSTATI)+NSSSSS,NINT(RDAY))
ZHGMT=REAL(IHGMT,JPRB)

ZTETA=RTETA(ZTIMTR)
ZDEASOM=RRS(ZTETA)
ZDECLIM=RDS(ZTETA)
ZEQTIMM=RET(ZTETA)
ZSOVRM =ZEQTIMM+ZHGMT
ZWSOVRM=ZSOVRM*2.0_JPRB*RPI/RDAY

RCODECM15=COS(ZDECLIM)
RSIDECM15=SIN(ZDECLIM)

RCOVSRM15=COS(ZWSOVRM)
RSIVSRM15=SIN(ZWSOVRM)
ZTHETOZ=RTETA(RTIMTR)
ZANGOZC=REL(ZTHETOZ)-1.7535_JPRB
ZCOTHOZ=COS(ZANGOZC)
ZSITHOZ=SIN(ZANGOZC)

!     ------------------------------------------------------------------

!          2.  COMPUTES TIME-DEPENDENT PARAMETERS FOR OZONE CLIMATOLOGY
!              --------------------------------------------------------

CALL SUECOZO (ZANGOZC)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('UPDTIER15',1,ZHOOK_HANDLE)
END SUBROUTINE UPDTIER15
