SUBROUTINE VDFHGHTHL (KSTEP, KIDIA , KFDIA  , KLON  , KLEV   , KDRAFT, PTMST,&
                   & PUM1, PVM1, PTM1    , PQM1  , PLM1   , PIM1  , PAM1,&
                   & PAPHM1  , PAPM1 , PGEOM1 , PGEOH , &
                   & PKHFL   , PKQFL , PKUFL  , PKVFL, &
! OUTPUT
                   & PMFLX  , &        
                   & PTHLUH  , PQTUH   , PTHTVUH, & 
                   & PQCUH, PQIUH, PUUH, PVUH,&
                   & KGFL_EZDIAG, PEZDIAG, tendqvup,tendtup,surfprep,surfsnow)
                   
                     
!     ------------------------------------------------------------------

!**   *VDFHGHTHL* - INTERFACE TO VDFHGHTHLN to be called from 
!                                          the main physics interface of Arome APL_AROME

!     A.P. SIEBESMA and S. MALARDEL   May 2007   

!     Wim de Rooy                     August 2008
!                include tendencies due to updraft rain/snow, and export
!                surface precipitation/snow

!     PURPOSE
!     -------

!     PREPARE VARIABLE AND CALL VDFHGHTNHL

!     INTERFACE
!     ---------

!     *VDFHGHTN* IS CALLED BY *VDFMAIN*

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLEV*         NUMBER OF LEVELS
!     *KSTEP*        TIME STEP
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KDRAFT*       NUMBER OF EXPLICITLY MODELED DRAFTS - CURRENTLY 3:
!                    1: test parcel
!                    2: rising dry thermals which stop at cloud base or inversion
!                    3: rising dry thermals which become cloudy
!                    (4: downdrafts .. to be done?)

!     INPUT PARAMETERS (REAL):

!     *PTMST*        DOUBLE TIME STEP (SINGLE AT 1TH STEP)        S
!     *PTM1*         TEMPERATURE AT T-1                           K
!     *PUM1*         X_VELOCITY COMPONENT AT T-1                  M/S
!     *PVM1*         V-VELOCITY COMPONENT AT T-1                  M/S
!     *PQM1*         SPECIFIC HUMUDITY AT T-1                     KG/KG
!     *PLM1*         SPECIFIC CLOUD LIQUID WATER AT T-1           KG/KG
!     *PIM1*         SPECIFIC CLOUD ICE AT T-1                    KG/KG
!     *PAM1*         CLOUD FRACTION AT T-1                        KG/KG
!     *PAPHM1*       PRESSURE AT HALF LEVEL AT T-1                PA
!     *PAPM1*        PRESSURE AT FULL LEVEL AT T-1                PA
!     *PGEOM1*       GEOPOTENTIAL AT T-1                          M2/S2
!     *PGEOH*        GEOPOTENTIAL AT HALF LEVEL                   M2/S2
 
!     *PKHFL*        SURFACE KINEMATIC HEAT FLUX                  K*M/S
!     *PKQFL*        SURFACE KINEMATIC MOISTURE FLUX              M/S
!     *PKUFL*        SURFACE KINEMATIC ZONAL WIND FLUX     
!     *PKVFL*        SURFACE KINEMATIC MERIDIONAL WIND FLUX

!     OUTPUT PARAMETERS (REAL):
!     *PMFLX*        PBL MASS FLUX                                M/S
!     *PTHLUH*       UPDRAFT GENERALIZED LIQUID STATIC ENERGY (SLG)
!                    AT HALF LEVEL                                M2/S2
!     *PQTUH*        UPDRAFT SPECIFIC TOTAL WATER AT HALF LEVEL   KG/KG
!     *PTHTVUH*      UPDRAFT virt pot. temp. (for TKE scheme) AT HALF LEVEL K 
!     *PQCUH*        UPDRAFT SPECIFIC TOTAL WATER AT HALF LEVEL   KG/KG
!     *PQIUH*        UPDRAFT SPECIFIC TOTAL WATER AT HALF LEVEL   KG/KG
!     *PUUH*         UPDRAFT X-MOMENTUM
!     *PVUH*         UPDRAFT Y-MOMENTUM
!     tendtup,tendqvup,surfprep   tendencies for non-conserved AROME
!                   variables due to updraft precipitation (and its evaporation)

!     METHOD
!     ------

!     SEE DOCUMENTATION

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RCPD     ,RETV     ,RLVTT    ,&
                    & RLSTT    ,RATM

IMPLICIT NONE


!*         0.1    GLOBAL VARIABLES

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDRAFT
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKHFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKQFL(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKUFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKVFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLX(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTHLUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQTUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTHTVUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQCUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQIUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVUH(KLON,0:KLEV,KDRAFT) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGFL_EZDIAG
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEZDIAG(KLON,KLEV,KGFL_EZDIAG) 

!*         0.2    LOCAL VARIABLES

REAL(KIND=JPRB)      :: PFPLVN(KLON,0:KLEV) 
REAL(KIND=JPRB)      :: PFPLVL(KLON,0:KLEV) 
REAL(KIND=JPRB)      ::  ZKMFL(KLON)
REAL(KIND=JPRB), ALLOCATABLE :: ZEXTR2(:,:), ZEXTRA(:,:,:)
INTEGER(KIND=JPIM)   :: IFLDX2, ILEVX, IFLDX,itje
REAL(KIND=JPRB)      :: ZSLGUH(KLON,0:KLEV,KDRAFT),ZFRACB(KLON,KDRAFT), &
  & ZZPTOP(KLON,KDRAFT),ZZPLCL(KLON,KDRAFT)
INTEGER(KIND=JPIM)   :: IPLCL(KLON,KDRAFT),  IPTOP(KLON,KDRAFT) , IPLZB(KLON,KDRAFT)
REAL(KIND=JPRB)      :: ZWUAVG(KLON),ZRICUI(KLON),ZMCU(KLON) , &
  & ZPGUSUM(KLON,KLEV), ZPEUSUM(KLON,KLEV) , ZBIR(KLON), &
  & ZWQT(KLON,0:KLEV), ZWQT2(KLON,0:KLEV)
LOGICAL              :: ODNODECP(KLON) ,ODRUNDRY(KLON) 
INTEGER(KIND=JPIM)   :: IPBLTYPE(KLON) 

REAL(KIND=JPRB)       :: ZRSCP,ZINVATM
REAL(KIND=JPRB)       :: INVDZRHO              !   -1/(dz*rho)
REAL(KIND=JPRB)       :: TENDQVUP(KLON,KLEV) 
REAL(KIND=JPRB)       :: SURFPREP(KLON) 
REAL(KIND=JPRB)       :: SURFSNOW(KLON) 
REAL(KIND=JPRB)       :: TENDTUP(KLON,KLEV) 

REAL(KIND=JPRB)       :: ZHOOK_HANDLE
INTEGER(KIND=JPIM)    :: JLON,JLEV,JDRAFT

#include "vdfhghtnhl.intfb.h"

IF (LHOOK) CALL DR_HOOK('VDFHGHTHL',0,ZHOOK_HANDLE)

! Compute surface momentum flux
ZKMFL(:) = SQRT(PKUFL(:)**2 + PKVFL(:)**2)

! Non used input arguments
ZBIR = 0.
ODNODECP = .FALSE.
ODRUNDRY = .FALSE.


! ALLOCATE OUTPUT
IFLDX2=60
IFLDX=60
ILEVX=KLEV+1
ALLOCATE(ZEXTR2(KLON,IFLDX2))
ALLOCATE(ZEXTRA(KLON,ILEVX,IFLDX))

PMFLX=0._JPRB
ZSLGUH=0._JPRB
PQTUH=0._JPRB
PTHLUH=0._JPRB
PFPLVL=0._JPRB
PFPLVN=0._JPRB
SURFPREP=0._JPRB
SURFSNOW=0._JPRB
TENDQVUP=0._JPRB
TENDTUP=0._JPRB

CALL VDFHGHTNHL ( &
  & KIDIA , KFDIA, KLON, KLEV, KDRAFT, PTMST, KSTEP,&
  & PUM1, PVM1, PTM1, PQM1, PLM1, PIM1, PAM1,&
  & PAPHM1, PAPM1, PGEOM1, PGEOH  , &
  & ZKMFL   , PKHFL   , PKQFL   , PMFLX  , &
  & ZEXTR2  , IFLDX2  , ZEXTRA  , ILEVX  , IFLDX , &                  
  & PUUH, PVUH, ZSLGUH  , PQTUH   , PTHTVUH, ZFRACB  , &
  & ZZPTOP  , IPTOP   , ZZPLCL  , IPLCL  , IPLZB   , &
  & ZWUAVG, ZRICUI, ZMCU, &
  & PFPLVL, PFPLVN, &
  & ZBIR, ODNODECP, ODRUNDRY, IPBLTYPE, ZWQT, ZWQT2, &
  & KGFL_EZDIAG, PEZDIAG )


! Convert SLGUH to THLUH
!!!! The transformation from sl to thetal is approximate because in MesoNH we use
!!!!  cp moist and the latente corresponding the T of the env,
!!!!  but in VDFHGHT we use dry cp and constant latent heat.
ZRSCP=RD/RCPD
ZINVATM=1/RATM

DO JDRAFT=2,KDRAFT
  DO JLEV=0,KLEV
    DO JLON=1,KLON
      PTHLUH(JLON,JLEV,JDRAFT) = MAX(0._JPRB,(ZSLGUH (JLON,JLEV,JDRAFT) &
 &     -  PGEOH(JLON,JLEV)) )  / ((max(PAPHM1(JLON,JLEV),1._JPRB)*ZINVATM)**ZRSCP) / RCPD 
    END DO
  END DO
END DO

PQCUH=0._JPRB
PQIUH=0._JPRB

!  Calculate tendencies in qv, and T due to vertical flux
!  divergence of updraft precipitation (snow and rain), i.e. PFPLVL and PFPLVN

DO JLEV=2,KLEV
  DO JLON=1,KLON
    INVDZRHO=RG/(PAPHM1(JLON,JLEV)-PAPHM1(JLON,JLEV-1))
    
    TENDQVUP(JLON,JLEV)= - (PFPLVL(JLON,JLEV) - PFPLVL(JLON,JLEV-1))*INVDZRHO &
 &                       - (PFPLVN(JLON,JLEV) - PFPLVN(JLON,JLEV-1))*INVDZRHO
    TENDTUP(JLON,JLEV)=  (PFPLVL(JLON,JLEV) - PFPLVL(JLON,JLEV-1))*INVDZRHO &
 &                      *RLVTT/RCPD  & 
 &                       +  (PFPLVN(JLON,JLEV) - PFPLVN(JLON,JLEV-1))*INVDZRHO &
 &                      *RLSTT/RCPD  
  ENDDO
ENDDO
 
DO JLON=1,KLON
  SURFPREP(JLON)=PFPLVL(JLON,KLEV)
  SURFSNOW(JLON)=PFPLVN(JLON,KLEV)
ENDDO

! DEALLOCATE LOCAL OUTPUT ARRAYS
DEALLOCATE(ZEXTR2,ZEXTRA)

IF (LHOOK) CALL DR_HOOK('VDFHGHTHL',1,ZHOOK_HANDLE)
END SUBROUTINE VDFHGHTHL
