SUBROUTINE WRITEPHYSIO &
 & (KEND, KST, KGL1, KGL2, KSTGLO, &
 & PGELAM, PGEMU, &
 & PGM, PMU0, POROG, POROGL, POROGM, PRCORI, PRATATH, PRATATX, &
 & PLSM, PARG,  PSAB, PALBF, PALBSF, PEMISF, PD2, PIVEG, PLAI, PCT, &
 & PGZ0F, PGZ0HF, PGZ0RLF, PGETRL, PVRLAN, PVRLDI, PRSMIN, PVEG0, PHV, &
 & PAESEA, PAELAN, PAESOO, PAEDES, PVO3ABC, &
 & PSPT0, PSPT0L, PSPT0M, PTT0, PTT0L, PTT0M, PQT0, PQT0L, PQT0M, &
 & PUT0, PVT0, PVORT0, PDIVT0, PCVGQ, PLCVQ, &
 & PTS0, PTP0, PWL0, PWS0, PWSI0, PWP0, PWPI0, PSNS0, &
 & PDIFCQ, PDIFCQN, PDIFCQL, PDIFCS, PDIFTQ, PDIFTQN, PDIFTQL, &
 & PDIFTS, PFCCQL, PFCCQN, PFCSQL, PFCSQN, PFCQNG, PFCQNNG, &
 & PFCQLNG, PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH, &
 & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV, PSTRMU, PSTRMV, &
 & PFRMH, PFCHOZ, PNEB, PQICE, PQLI, PRH, &
 & PFCS, PFCLL, PFCLN, PFEVL, PFEVN, PFEVV, PFTR, PFLWSP, PFONTE, &
 & PFGEL, PFGELS, PFRSODS, PFRSOPS, PFRSOPT, PFRTHDS, &
 & PQS, PRUISL, PRUISP, PRUISS, &
 & PUCLS, PVCLS, PTCLS, PQCLS, PRHCLS, &
 & PCLCT, PCLCH, PCLCM, PCLCL, PCLCC, &
 & PFPLCH, PFPLSH, PTCCH, PSCCH, PBCCH, PBLH )

! --------------------------------------------------------------
! **** *writephysio* Write SCM input/output of physics on file.
! --------------------------------------------------------------
! Subject:
!     Write on files all inputs and some outputs from MF_PHYS
!     for profiles matching a given criteria.

! Method:
! Authors:  1999-03, J.M. Piriou / F. Bouyssel.
! Modifications:
!   2000-10, J.M. Piriou: use of a namelist for SCM extractions.
!   2002-03, J.M. Piriou: same arguments for writephysio as for MF_PHYS.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   2004-03: T. Kovacic : extracted subroutine WRITEPROFILE
!   2004-10, F. Bouyssel : cleaning
!   2007-05, F. Bouyssel : add PFCLL, PFCLN
! --------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM    ,JPRB
USE YOMHOOK   ,ONLY : LHOOK   ,DR_HOOK
USE YOMLUN    ,ONLY : NULOUT
USE YOMRIP    ,ONLY : RSTATI
USE YOMDYN    ,ONLY : TSTEP
USE YOMSCM    ,ONLY : NFRSCM, NSCMTS, NSCM_SPACE_S, GSCM_LON1, GSCM_LON2 &
 & , GSCM_LAT1, GSCM_LAT2, GSCM_RADIUS, NSCM_ADD_SAMPL  
USE YOMDPHY   ,ONLY : NTSSG  
USE YOMDIM    ,ONLY : NFLEVG  ,NPROMA  ,NPROMM
USE YOMCST    ,ONLY : RA  
USE YOMCT3    ,ONLY : NSTEP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL1
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL2
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO
REAL(KIND=JPRB),INTENT(IN)    :: PGELAM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGEMU(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PMU0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: POROG(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: POROGL(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: POROGM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRCORI(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRATATH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRATATX(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PLSM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PARG(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSAB(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PALBF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PALBSF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PEMISF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PD2(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PIVEG(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PLAI(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PCT(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PGZ0F(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGZ0HF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGZ0RLF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGETRL(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVRLAN(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVRLDI(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRSMIN(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVEG0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PHV(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAESEA(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAELAN(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAESOO(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAEDES(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVO3ABC(NPROMA,3)
REAL(KIND=JPRB),INTENT(IN)    :: PSPT0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSPT0L(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSPT0M(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PTT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTT0L(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTT0M(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQT0L(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQT0M(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PUT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PVT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PVORT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIVT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PCVGQ(NPROMM,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PLCVQ(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTS0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PTP0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWL0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWS0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWSI0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWP0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWPI0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSNS0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCQ(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCS(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTQ(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTS(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCCQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCCQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCSQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCSQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCQNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCQNNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCQLNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLCL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLCN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLSL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSO(NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFRTH(NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRCU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRCV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRDU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRDV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRTU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRTV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRMU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRMV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFRMH(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCHOZ(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PNEB(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQICE(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQLI(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PRH(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCS(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFCLL(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFCLN(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFEVL(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFEVN(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFEVV(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFTR(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFLWSP(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFONTE(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFGEL(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFGELS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSODS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSOPS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSOPT(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRTHDS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PQS(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRUISL(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PRUISP(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PRUISS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PUCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PVCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PTCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PQCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PRHCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCT(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCH(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCM(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCL(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCC(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLCH(NPROMA,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLSH(NPROMA,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTCCH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSCCH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PBCCH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PBLH(NPROMA)

INTEGER(KIND=JPIM) :: JSTEP,JLON
LOGICAL :: LLOK, LLTIME_OK
REAL(KIND=JPRB) :: ZLAT_MODELE,ZLON_MODELE,ZD,ZHOUR,ZRR,ZPI,ZREQUIRED
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "writeprofile.intfb.h"

!-------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRITEPHYSIO',0,ZHOOK_HANDLE)

! -------------------------------------------------
! Define criteria for choosing profiles.
! -------------------------------------------------

!-------------------------------------------------
! 1. Time sampling.
!-------------------------------------------------

ZHOUR=RSTATI/3600._JPRB ! time since the beginning of the prediction in hours.
IF(NSCMTS(0) == 0) THEN
  !
  ! -------------------------------------------------
  ! One time step over nfrscm.
  ! -------------------------------------------------
  !
  LLTIME_OK=MOD(NSTEP,NFRSCM) == 0
  WRITE(NULOUT,*) 'writephysio: nscmts(0) == 0: lltime_ok=',LLTIME_OK
ELSEIF(NSCMTS(0) > 0) THEN
  !
  ! -------------------------------------------------
  ! Explicit list of time steps.
  ! -------------------------------------------------
  !
  LLTIME_OK=.FALSE.
  DO JSTEP=1,NSCMTS(0)
    IF(NFRSCM*NSCMTS(JSTEP) == NSTEP) LLTIME_OK=.TRUE.
  ENDDO
  WRITE(NULOUT,*) 'writephysio: nscmts(0) > 0: NSTEP=',NSTEP,' lltime_ok=',LLTIME_OK
ELSE
  !
  ! -------------------------------------------------
  ! Explicit list of hours.
  ! -------------------------------------------------
  !
  LLTIME_OK=.FALSE.
  DO JSTEP=1,-NSCMTS(0)
    ZREQUIRED=-REAL(NFRSCM,JPRB)*NSCMTS(JSTEP) ! required time in hours.
    IF(ABS(ZHOUR-ZREQUIRED) <= TSTEP) LLTIME_OK=.TRUE.
  ENDDO
  WRITE(NULOUT,*) 'writephysio: expl list hours: lltime_ok=',LLTIME_OK
ENDIF

!-------------------------------------------------
! 2. Space sampling.
!-------------------------------------------------

! RAPPEL: en t042: 8000 pdg, en T199: 180000 pdg.

IF (LLTIME_OK) THEN
  DO JLON=KST,KEND
    !
    ! -------------------------------------------------
    ! Compute model grid-point lat/lon coordinates.
    ! -------------------------------------------------
    !
    ZLAT_MODELE=ASIN(PGEMU(JLON))
    ZPI=4._JPRB*ATAN(1.0_JPRB)
    ZLON_MODELE=MODULO(PGELAM(JLON)+ZPI,2.0_JPRB*ZPI)-ZPI
    !
    ! -------------------------------------------------
    ! Is this profile required by the user?
    ! -------------------------------------------------
    !
    LLOK=.FALSE.
    IF(NSCM_SPACE_S == 0) THEN
      !
      ! -------------------------------------------------
      ! Choose profiles close to a given location.
      ! -------------------------------------------------
      !
      ! Compute othodromic distance (in meters) between searched point and model grid-point.
      !
      ZD=COS(ZLAT_MODELE)*COS(GSCM_LAT1)*COS(ZLON_MODELE-GSCM_LON1) &
       & + SIN(ZLAT_MODELE)*SIN(GSCM_LAT1)  
      ZD=ACOS(ZD)*RA
      LLOK=ZD < GSCM_RADIUS

    ELSEIF(NSCM_SPACE_S == 1) THEN
      !
      ! -------------------------------------------------
      ! Choose profiles inside a given box (in (lon, lat)),
      ! with additional regular sampling inside that box.
      ! -------------------------------------------------
      !
      LLOK=ZLON_MODELE > GSCM_LON1 &
       & .AND. ZLON_MODELE < GSCM_LON2 &
       & .AND. ZLAT_MODELE > GSCM_LAT1 &
       & .AND. ZLAT_MODELE < GSCM_LAT2 &
       & .AND.MOD(JLON,NSCM_ADD_SAMPL) == 0  
    ELSEIF(NSCM_SPACE_S == 10) THEN
      !
      ! -------------------------------------------------
      ! Choose profiles depending on physical output.
      ! -------------------------------------------------
      !
      ZRR=PFPLCL(JLON,NFLEVG)+PFPLCN(JLON,NFLEVG)
      LLOK=ZRR > 0.0_JPRB
    ELSEIF(NSCM_SPACE_S < 0) THEN
      !
      ! -------------------------------------------------
      ! Choose one profile over -NSCM_SPACE_S.
      ! -------------------------------------------------
      !
      LLOK=MOD(JLON,-NSCM_SPACE_S) == 0
    ELSE
      PRINT*,'writephysio/ERROR: NSCM_SPACE_S unexpected!...'
      PRINT*,NSCM_SPACE_S
      CALL ABOR1('writephysio/ERROR: NSCM_SPACE_S unexpected!')
    ENDIF
    IF (LLOK) THEN
      !
      ! -------------------------------------------------
      ! The grid-point is a desired profile.
      ! -------------------------------------------------
      !
      CALL WRITEPROFILE &
       & ('Profile.', &
       & JLON, ZLAT_MODELE, ZLON_MODELE, ZD, PGELAM, PGEMU, &
       & PGM, PMU0, POROG, POROGL, POROGM, PRCORI, PRATATH, PRATATX, &
       & PLSM, PARG,  PSAB, PALBF, PALBSF, PEMISF, PD2, PIVEG, PLAI, PCT, &
       & PGZ0F, PGZ0HF, PGZ0RLF, PGETRL, PVRLAN, PVRLDI, PRSMIN, PVEG0, PHV, &
       & PAESEA, PAELAN, PAESOO, PAEDES, PVO3ABC, &
       & PSPT0, PSPT0L, PSPT0M, PTT0, PTT0L, PTT0M, PQT0, PQT0L, PQT0M, &
       & PUT0, PVT0, PVORT0, PDIVT0, PCVGQ, PLCVQ, &
       & PTS0, PTP0, PWL0, PWS0, PWSI0, PWP0, PWPI0, PSNS0, &
       & PDIFCQ, PDIFCQN, PDIFCQL, PDIFCS, PDIFTQ, PDIFTQN, PDIFTQL, &
       & PDIFTS, PFCCQL, PFCCQN, PFCSQL, PFCSQN, PFCQNG, PFCQNNG, &
       & PFCQLNG, PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH, &
       & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV, PSTRMU, PSTRMV, &
       & PFRMH, PFCHOZ, PNEB, PQICE, PQLI, PRH, &
       & PFCS, PFCLL, PFCLN, PFEVL, PFEVN, PFEVV, PFTR, PFLWSP, PFONTE, &
       & PFGEL, PFGELS, PFRSODS, PFRSOPS, PFRSOPT, PFRTHDS, &
       & PQS, PRUISL, PRUISP, PRUISS, &
       & PUCLS, PVCLS, PTCLS, PQCLS, PRHCLS, &
       & PCLCT, PCLCH, PCLCM, PCLCL, PCLCC, &
       & PFPLCH, PFPLSH, PTCCH, PSCCH, PBCCH, PBLH )
    ENDIF
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('WRITEPHYSIO',1,ZHOOK_HANDLE)

END SUBROUTINE WRITEPHYSIO
