SUBROUTINE WRITEPROFILE &
 & (CDNAME, &
 & KLON, PLAT_MODELE, PLON_MODELE, PD, PGELAM, PGEMU, &
 & PGM, PMU0, POROG, POROGL, POROGM, PRCORI, PRATATH, PRATATX, &
 & PLSM, PARG,  PSAB, PALBF, PALBSF, PEMISF, PD2, PIVEG, PLAI, PCT, &
 & PGZ0F, PGZ0HF, PGZ0RLF, PGETRL, PVRLAN, PVRLDI, PRSMIN, PVEG0, PHV, &
 & PAESEA, PAELAN, PAESOO, PAEDES, PVO3ABC, &
 & PSPT0, PSPT0L, PSPT0M, PTT0, PTT0L, PTT0M, PQT0, PQT0L, PQT0M, &
 & PUT0, PVT0, PVORT0, PDIVT0, PCVGQ, PLCVQ, &
 & PTS0, PTP0, PWL0, PWS0, PWSI0, PWP0, PWPI0, PSNS0, &
 & PDIFCQ, PDIFCQN, PDIFCQL, PDIFCS, PDIFTQ, PDIFTQN, PDIFTQL, &
 & PDIFTS, PFCCQL, PFCCQN, PFCSQL, PFCSQN, PFCQNG, PFCQNNG, &
 & PFCQLNG, PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH, &
 & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV, PSTRMU, PSTRMV, &
 & PFRMH, PFCHOZ, PNEB, PQICE, PQLI, PRH, &
 & PFCS, PFCLL, PFCLN, PFEVL, PFEVN, PFEVV, PFTR, PFLWSP, PFONTE, &
 & PFGEL, PFGELS, PFRSODS, PFRSOPS, PFRSOPT, PFRTHDS, &
 & PQS, PRUISL, PRUISP, PRUISS, &
 & PUCLS, PVCLS, PTCLS, PQCLS, PRHCLS, &
 & PCLCT, PCLCH, PCLCM, PCLCL, PCLCC, &
 & PFPLCH, PFPLSH, PTCCH, PSCCH, PBCCH, PBLH )

! --------------------------------------------------------------
! **** *WRITEPROFILE* Write SCM input/output of physics on file.
! --------------------------------------------------------------
! Subject:
!     Write on files all inputs and some outputs from MF_PHYS
!     for one point.

! Method:
! Authors:  1999-03, J.M. Piriou / F. Bouyssel.
! Modifications:
!   2000-10, J.M. Piriou: use of a namelist for SCM extractions.
!   2002-03, J.M. Piriou: same arguments for writephysio as for MF_PHYS.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   2004-03, T. Kovacic : extracted this subroutine from WRITEPHYSIO
!   2004-10, F. Bouyssel : cleaning
!   2005-03, F. Bouyssel : TEQH
!   2007-05, F. Bouyssel : add PFCLL, PFCLN
! --------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM    ,JPRB
USE YOMHOOK   ,ONLY : LHOOK   ,DR_HOOK
USE YOMLUN   , ONLY : NULFP15
USE YOMRIP   , ONLY : RSTATI  ,NINDAT  ,NSSSSS
USE YOMSCM   , ONLY : NSCM_SPACE_S   
USE YOMDPHY  , ONLY : NTSSG  
USE SURFACE_FIELDS_MIX,ONLY : YSD_VAD, YSP_SBD, YSD_VVD, YSD_VND
USE YOMGEM   , ONLY : VBH     ,VAH     ,TEQC    ,TEQK    ,TEQH
USE YOMARPHY , ONLY : LMSE
USE YOMDIM   , ONLY : NFLEVG  ,NPROMA  ,NPROMM 
USE YOMPHY   , ONLY : LGWDC   ,LNEBCO  ,LNEBR   ,LSNV    ,LSOLV   ,LO3ABC 
USE YOMPHY2  , ONLY : TSPHY   
USE YOMCT3   , ONLY : NSTEP

IMPLICIT NONE

!-------------------------------------------------

CHARACTER(LEN=*),INTENT(IN)   :: CDNAME
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
REAL(KIND=JPRB),INTENT(IN)    :: PLAT_MODELE
REAL(KIND=JPRB),INTENT(IN)    :: PLON_MODELE
REAL(KIND=JPRB),INTENT(IN)    :: PD
REAL(KIND=JPRB),INTENT(IN)    :: PGELAM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGEMU(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PMU0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: POROG(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: POROGL(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: POROGM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRCORI(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRATATH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRATATX(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PLSM(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PARG(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSAB(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PALBF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PALBSF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PEMISF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PD2(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PIVEG(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PLAI(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PCT(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PGZ0F(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGZ0HF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGZ0RLF(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PGETRL(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVRLAN(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVRLDI(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRSMIN(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVEG0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PHV(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAESEA(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAELAN(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAESOO(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PAEDES(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PVO3ABC(NPROMA,3)
REAL(KIND=JPRB),INTENT(IN)    :: PSPT0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSPT0L(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSPT0M(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PTT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTT0L(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTT0M(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQT0L(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQT0M(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PUT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PVT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PVORT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIVT0(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PCVGQ(NPROMM,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PLCVQ(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTS0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PTP0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWL0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWS0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWSI0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWP0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PWPI0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSNS0(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCQ(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFCS(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTQ(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PDIFTS(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCCQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCCQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCSQL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCSQN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCQNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCQNNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCQLNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLCL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLCN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLSL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSO(NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFRTH(NPROMM,0:NFLEVG,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRCU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRCV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRDU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRDV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRTU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRTV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRMU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PSTRMV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFRMH(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCHOZ(NPROMM,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PNEB(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQICE(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PQLI(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PRH(NPROMA,NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFCS(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFCLL(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFCLN(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFEVL(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFEVN(NPROMM,NTSSG+1)
REAL(KIND=JPRB),INTENT(IN)    :: PFEVV(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFTR(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFLWSP(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFONTE(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFGEL(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFGELS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSODS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSOPS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRSOPT(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFRTHDS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PQS(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PRUISL(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PRUISP(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PRUISS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PUCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PVCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PTCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PQCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PRHCLS(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCT(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCH(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCM(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCL(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCC(NPROMM)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLCH(NPROMA,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PFPLSH(NPROMA,0:NFLEVG)
REAL(KIND=JPRB),INTENT(IN)    :: PTCCH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PSCCH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PBCCH(NPROMA)
REAL(KIND=JPRB),INTENT(IN)    :: PBLH(NPROMA)

CHARACTER (LEN = 200) ::  CLDURH,CLFS,CLNOMP
INTEGER(KIND=JPIM) :: IULLFA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ecr1d.intfb.h"
#include "ecr2df.intfb.h"
#include "ecr2dv.intfb.h"
#include "ecrpnebh.intfb.h"
#include "nomfi.intfb.h"

!-------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRITEPROFILE',0,ZHOOK_HANDLE)

! -------------------------------------------------

WRITE(CLDURH,FMT='(I4.4)') NSTEP
WRITE(CLFS,FMT='(2A)') '+',CLDURH(1:4)
      !
      ! -------------------------------------------------
      ! Look for a free file name.
      ! -------------------------------------------------
      !
CALL NOMFI(CDNAME,CLFS,CLNOMP)
      !
      ! -------------------------------------------------
      ! Open SCM file.
      ! -------------------------------------------------
      !
IULLFA=NULFP15
CALL LFAOUV(IULLFA,CLNOMP,'W')
      !
      ! -------------------------------------------------
      ! Write input to physics.
      ! Write integers from physics.
      ! -------------------------------------------------
      !
CALL LFAECRR(IULLFA,'LONGITUDE',PLON_MODELE,1)
CALL LFAECRR(IULLFA,'LATITUDE',PLAT_MODELE,1)
CALL ECR1D  (IULLFA,'PGM',PGM,KLON,NPROMA)
CALL LFAECRR(IULLFA,'TEQK',TEQK,1)
CALL LFAECRR(IULLFA,'TEQC',TEQC,1)
CALL LFAECRR(IULLFA,'TEQH',TEQH,1)
CALL LFAECRI(IULLFA,'KINDAT',NINDAT,1)
CALL LFAECRI(IULLFA,'KSSSSS',NSSSSS,1)
CALL LFAECRR(IULLFA,'RSTATI',RSTATI,1)
CALL LFAECRI(IULLFA,'NSTEP',NSTEP,1)
CALL LFAECRI(IULLFA,'KLEV',NFLEVG,1)
CALL LFAECRR(IULLFA,'TSPHY',TSPHY,1)
CALL LFAECRI(IULLFA,'KVCLIV',YSD_VVD%NUMFLDS,1)
CALL LFAECRI(IULLFA,'KNEBH',YSD_VND%NUMFLDS,1)
CALL LFAECRI(IULLFA,'KCSS',YSP_SBD%NLEVS,1)
CALL LFAECRI(IULLFA,'KSGST',NTSSG,1)
IF(NSCM_SPACE_S == 0) THEN
  CALL LFAECRR(IULLFA,'DISTANCE',PD,1)
ENDIF
      !
      ! -------------------------------------------------
      ! Write fields from time 0.
      ! -------------------------------------------------
      !
CALL ECR2DV(IULLFA,'PVORT0',PVORT0,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PDIVT0',PDIVT0,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PUT0',PUT0,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PVT0',PVT0,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PTT0',PTT0,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PQT0',PQT0,KLON,NPROMA,NFLEVG)
CALL ECR1D (IULLFA,'PSPT0',PSPT0,KLON,NPROMA)
      !
      !
CALL ECR2DV(IULLFA,'PTT0L',PTT0L,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PQT0L',PQT0L,KLON,NPROMA,NFLEVG)
CALL ECR1D (IULLFA,'PSPT0L',PSPT0L,KLON,NPROMA)
      !
      !
CALL ECR2DV(IULLFA,'PTT0M',PTT0M,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PQT0M',PQT0M,KLON,NPROMA,NFLEVG)
CALL ECR1D (IULLFA,'PSPT0M',PSPT0M,KLON,NPROMA)
      !
      !
CALL ECR1D(IULLFA,'PTS0',PTS0,KLON,NPROMA)
CALL ECRPNEBH(IULLFA,'PTP0',PTP0,KLON,NPROMA,1)
IF (.NOT.LMSE) THEN
  CALL ECR1D(IULLFA,'PWL0',PWL0,KLON,NPROMA)
ENDIF
CALL ECR1D(IULLFA,'PWS0',PWS0,KLON,NPROMA)
CALL ECR1D(IULLFA,'PWSI0',PWSI0,KLON,NPROMA)
CALL ECR1D(IULLFA,'PWP0',PWP0,KLON,NPROMA)
CALL ECR1D(IULLFA,'PWPI0',PWPI0,KLON,NPROMA)
CALL ECR1D(IULLFA,'PSNS0',PSNS0,KLON,NPROMA)
      !
      ! -------------------------------------------------
      ! Write constant fields.
      ! -------------------------------------------------
      !
CALL ECR1D(IULLFA,'PRCORI',PRCORI,KLON,NPROMA)
CALL ECR1D(IULLFA,'PGEMU',PGEMU,KLON,NPROMA)
CALL ECR1D(IULLFA,'PGELAM',PGELAM,KLON,NPROMA)
CALL ECR1D(IULLFA,'PRATATH',PRATATH,KLON,NPROMA)
CALL ECR1D(IULLFA,'PRATATX',PRATATX,KLON,NPROMA)
CALL ECR1D(IULLFA,'POROG',POROG,KLON,NPROMA)
CALL ECR1D(IULLFA,'POROGL',POROGL,KLON,NPROMA)
CALL ECR1D(IULLFA,'POROGM',POROGM,KLON,NPROMA)
CALL LFAECRR(IULLFA,'VAH',VAH,NFLEVG+1)
CALL LFAECRR(IULLFA,'VBH',VBH,NFLEVG+1)
      !
      ! -------------------------------------------------
      ! Write arrays in input to physics.
      ! -------------------------------------------------
      !
IF (LSOLV) THEN
  CALL ECR1D(IULLFA,'PARG',PARG,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PD2',PD2,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PIVEG',PIVEG,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PLAI',PLAI,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PRSMIN',PRSMIN,KLON,NPROMA)
  IF (YSD_VVD%NUMFLDS >= 8) THEN
    CALL ECR1D(IULLFA,'PGZ0HF',PGZ0HF,KLON,NPROMA)
  ENDIF
  CALL ECR1D(IULLFA,'PSAB',PSAB,KLON,NPROMA)
ENDIF
IF (LSOLV) THEN
  CALL ECR1D(IULLFA,'PHV',PHV,KLON,NPROMA)
ENDIF
IF (.NOT.LMSE) THEN
  CALL ECR1D(IULLFA,'PALBF',PALBF,KLON,NPROMA)
  IF (LSNV) THEN
    CALL ECR1D(IULLFA,'PALBSF',PALBSF,KLON,NPROMA)
  ENDIF
  CALL ECR1D(IULLFA,'PEMISF',PEMISF,KLON,NPROMA)
ENDIF
CALL ECR1D(IULLFA,'PGETRL',PGETRL,KLON,NPROMA)
CALL ECR1D(IULLFA,'PLSM',PLSM,KLON,NPROMA)
CALL ECR1D(IULLFA,'PVEG0',PVEG0,KLON,NPROMA)
IF (.NOT.LMSE) THEN
  CALL ECR1D(IULLFA,'PGZ0F',PGZ0F,KLON,NPROMA)
  IF (LSOLV) THEN
    IF (LSNV) THEN
      CALL ECR1D(IULLFA,'PGZ0RLF',PGZ0RLF,KLON,NPROMA)
    ENDIF
  ENDIF
ENDIF
CALL ECR1D(IULLFA,'PVRLAN',PVRLAN,KLON,NPROMA)
CALL ECR1D(IULLFA,'PVRLDI',PVRLDI,KLON,NPROMA)
IF (YSD_VAD%NUMFLDS >= 4) THEN
  CALL ECR1D(IULLFA,'PAESEA',PAESEA,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PAELAN',PAELAN,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PAESOO',PAESOO,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PAEDES',PAEDES,KLON,NPROMA)
ENDIF
IF (LO3ABC) THEN
  CALL ECRPNEBH(IULLFA,'PVO3ABC',PVO3ABC,KLON,NPROMA,3)
ENDIF
      !
      ! -------------------------------------------------
      ! Write input to physics, redundant
      ! with variables that will be computed by SCM.
      ! -------------------------------------------------
      !
CALL ECR2DV(IULLFA,'PCVGQ',PCVGQ,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PCVGQDY',PLCVQ,KLON,NPROMA,NFLEVG)
      !
      ! -------------------------------------------------
      ! Write output from physics.
      ! -------------------------------------------------
      !
CALL ECR2DF(IULLFA,'PDIFCQ',PDIFCQ,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFCS',PDIFCS,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFCQN',PDIFCQN,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFCQL',PDIFCQL,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFTQ',PDIFTQ,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFTQN',PDIFTQN,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFTQL',PDIFTQL,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PDIFTS',PDIFTS,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCCQL',PFCCQL,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCCQN',PFCCQN,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCSQL',PFCSQL,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCSQN',PFCSQN,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCQNG',PFCQNG,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCQNNG',PFCQNNG,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCQLNG',PFCQLNG,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFPLCL',PFPLCL,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFPLCN',PFPLCN,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFPLSL',PFPLSL,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFPLSN',PFPLSN,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFRSO',PFRSO,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFRTH',PFRTH,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRCU',PSTRCU,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRCV',PSTRCV,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRDU',PSTRDU,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRDV',PSTRDV,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRTU',PSTRTU,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRTV',PSTRTV,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRMU',PSTRMU,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PSTRMV',PSTRMV,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFRMH',PFRMH,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFCHOZ',PFCHOZ,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PNEB',PNEB,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PQICE',PQICE,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PQLI',PQLI,KLON,NPROMA,NFLEVG)
!     CALL ECR2DV(IULLFA,'PQSAT',PQSAT,KLON,NPROMA,NFLEVG)
CALL ECR2DV(IULLFA,'PRH',PRH,KLON,NPROMA,NFLEVG)

!     Split of old PNEBH      
CALL ECR2DF(IULLFA,'PFPLCH',PFPLCH,KLON,NPROMA,NFLEVG)
CALL ECR2DF(IULLFA,'PFPLSH',PFPLSH,KLON,NPROMA,NFLEVG)
IF (LNEBCO.OR.LGWDC) THEN
  CALL ECR1D(IULLFA,'PTCCH',PTCCH,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PSCCH',PSCCH,KLON,NPROMA)
  CALL ECR1D(IULLFA,'PBCCH',PBCCH,KLON,NPROMA)
ENDIF
IF ( LNEBR ) CALL ECR1D(IULLFA,'PPBLH' ,PBLH ,KLON,NPROMA)

CALL ECR1D(IULLFA,'PCT',PCT,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFCS',PFCS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFCLL',PFCLL,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFCLN',PFCLN,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFEVL',PFEVL,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFEVN',PFEVN,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFEVV',PFEVV,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFTR',PFTR,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFLWSP',PFLWSP,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFONTE',PFONTE,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFGEL',PFGEL,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFGELS',PFGELS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFRSODS',PFRSODS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFRSOPS',PFRSOPS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFRSOPT',PFRSOPT,KLON,NPROMA)
CALL ECR1D(IULLFA,'PFRTHDS',PFRTHDS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PQS',PQS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PRUISL',PRUISL,KLON,NPROMA)
CALL ECR1D(IULLFA,'PRUISP',PRUISP,KLON,NPROMA)
CALL ECR1D(IULLFA,'PRUISS',PRUISS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PUCLS',PUCLS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PVCLS',PVCLS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PTCLS',PTCLS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PQCLS',PQCLS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PRHCLS',PRHCLS,KLON,NPROMA)
CALL ECR1D(IULLFA,'PCLCT',PCLCT,KLON,NPROMA)
CALL ECR1D(IULLFA,'PCLCH',PCLCH,KLON,NPROMA)
CALL ECR1D(IULLFA,'PCLCM',PCLCM,KLON,NPROMA)
CALL ECR1D(IULLFA,'PCLCL',PCLCL,KLON,NPROMA)
CALL ECR1D(IULLFA,'PCLCC',PCLCC,KLON,NPROMA)
CALL ECR1D(IULLFA,'PMU0',PMU0,KLON,NPROMA)
      !
      ! -------------------------------------------------
      ! Close SCM file.
      ! -------------------------------------------------
      !
CALL LFAFER(IULLFA)
IF (LHOOK) CALL DR_HOOK('WRITEPROFILE',1,ZHOOK_HANDLE)

END SUBROUTINE WRITEPROFILE
