SUBROUTINE AER_CGROWTH &
  &( KIDIA   , KFDIA   , KLON     , KLEV  , &
  &  PAERPHO , &
  &  PTAERPHI, PTAERPHO & 
  &)

!*** * AER_CGROWTH* - GROWTH OF O.M. AND B.C. AEROSOLS

!**   INTERFACE.
!     ----------
!          *AER_CGROWTH* IS CALLED FROM *AER_PHY3*.

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*
!        FROM O.BOUCHER's cgrowth 

!     SOURCE.
!     -------

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2006-11-21
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAERATM ,ONLY : RGRATE  

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV

REAL(KIND=JPRB)   ,INTENT(IN)    :: PAERPHO(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTAERPHI(KLON,KLEV), PTAERPHO(KLON,KLEV)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JL, JK
REAL(KIND=JPRB) :: ZCOEF, ZAERCONV

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_CGROWTH',0,ZHOOK_HANDLE)

ZCOEF= RGRATE

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZAERCONV = PAERPHO(JL,JK) * ZCOEF
    PTAERPHI(JL,JK) =  ZAERCONV
    PTAERPHO(JL,JK) = -ZAERCONV    
  ENDDO
ENDDO
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_CGROWTH',1,ZHOOK_HANDLE)
END SUBROUTINE AER_CGROWTH
