SUBROUTINE AER_LOSS &
  &( KIDIA, KFDIA, KLON, KLEV, KAERO, &
  &  PAERO, PTSPHY, &
  &  PTAERO & 
  &)

!*** * AER_LOSS* - SINK TERMS FOR AEROSOLS IN ANALYSIS

!**   INTERFACE.
!     ----------
!          *AER_LOSS* IS CALLED FROM *AER_xxx*.

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     SOURCE.
!     -------

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2007-11-17

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAERSNK ,ONLY : RAERTS

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN) :: KLON 
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KAERO

REAL(KIND=JPRB),INTENT(IN)    :: PAERO(KLON,KLEV,KAERO)
REAL(KIND=JPRB),INTENT(IN)    :: PTSPHY
REAL(KIND=JPRB),INTENT(INOUT) :: PTAERO(KLON,KLEV,KAERO)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JAER, JK, JL
REAL(KIND=JPRB) :: ZEXP(KAERO), ZAERN(KLON,KLEV,KAERO)

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_LOSS',0,ZHOOK_HANDLE)

DO JAER=1,KAERO
  ZEXP(JAER)=EXP(-PTSPHY*RAERTS(JAER))
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZAERN(JL,JK,JAER)=PAERO(JL,JK,JAER)*ZEXP(JAER)
      PTAERO(JL,JK,JAER)=(ZAERN(JL,JK,JAER)-PAERO(JL,JK,JAER))/PTSPHY
    ENDDO
  ENDDO
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_LOSS',1,ZHOOK_HANDLE)
END SUBROUTINE AER_LOSS
