SUBROUTINE AER_PHY1( KIDIA, KFDIA, KLON , KTDIA  , KLEV , KSTART , KSTEP, &
   &  PGFL, PRS1 , PRSF1, PAPRSF, PTP, PTL , PQ, &
   &  PCLON, PSLON, PGEMU, &
   &  PAEROP , &
   &  PAERMSS, PAERMSST, PAERTAU, PAERTAUT, PAERTAUI, PAERTAUB, PAERTAUA, PAERTAUF, PAERTAUO )

!**   INTERFACE.
!     ----------
!          *AER_PHY1* IS CALLED FROM *CALLPAR*.

! INPUTS:
!  -------
! PRS1 (KLON,0:KLEV)  : HALF-LEVEL PRESSURE           (Pa)
! PRSF1(KLON,KLEV)    : PROVISIONAL FULL-LEVEL PRESSURE           (Pa)
! PAPRSF              : PRESSURE ON FULL LEVELS.
! PTP(KLON,KLEV)      : FULL-LEVEL TEMPERATURE (W. DYN.TEND.) (K)
! PQ                  : SPECIFIC HUMIDITY
! PTL(KLON)           : SURFACE TEMPERATURE
! PCLON, PSLON(KLON)  : COSINE & SINE OF LONGITUDE
! PGEMU(KLON)         : SINE OF LATITUDE

! PAERMSS(KLON,KLEV,NACTAERO) :
! PAERMSST(KLON,NACTAERO) :
! PAERTAU(KLON,KLEV,NACTAERO) : 
! PAERTAUT(KLON,NACTAERO)     :
! PAERTAUI(KLON,17)           :

! INPUTS/OUTPUTS:
!---------------
! PGFL                : GFL FIELDS

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAERSRC, ONLY : LEPAERO 
USE YOEAERATM, ONLY : LAERCLIMG, LAERCLIMZ, LAERCLIST
USE YOEAERLID ,ONLY : NWLID
USE YOM_YGFL , ONLY : YGFL, YAERO, NAERO, NACTAERO 
!USE YOEDBUG  , ONLY : KSTPDBG
USE YOMCST   , ONLY : RD, RG
!USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA, KFDIA, KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KTDIA, KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KSTART, KSTEP

REAL(KIND=JPRB),INTENT(INOUT) :: PGFL(KLON,KLEV,YGFL%NDIM)
REAL(KIND=JPRB),INTENT(IN)    :: PRS1(KLON,0:KLEV), PRSF1(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PAPRSF(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PTP(KLON,KLEV)   
REAL(KIND=JPRB),INTENT(IN)    :: PCLON(KLON), PSLON(KLON), PGEMU(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PTL(KLON)  
REAL(KIND=JPRB),INTENT(IN)    :: PQ(KLON,KLEV)

REAL(KIND=JPRB),INTENT(INOUT) :: PAEROP(KLON,KLEV,NACTAERO)    

REAL(KIND=JPRB),INTENT(INOUT) :: PAERMSS(KLON,KLEV,NACTAERO)
REAL(KIND=JPRB),INTENT(INOUT) :: PAERMSST(KLON,NACTAERO) 
REAL(KIND=JPRB),INTENT(INOUT) :: PAERTAU(KLON,KLEV,NACTAERO)
REAL(KIND=JPRB),INTENT(INOUT) :: PAERTAUT(KLON,NACTAERO,19), PAERTAUB(KLON,NACTAERO,19),PAERTAUO(KLON,KLEV,3)
REAL(KIND=JPRB),INTENT(INOUT) :: PAERTAUI(KLON,19)         , PAERTAUA(KLON,19)         ,PAERTAUF(KLON,19)

! Local variables
INTEGER(KIND=JPIM) :: IFLAG, JK, JL, JGFL, IWAVL , IWHAT, INWAVL, ITWAVL(19)  
REAL(KIND=JPRB) :: ZAEROCLIM(KLON,KLEV,NAERO), ZAEROCLIS(KLON,KLEV,2)
REAL(KIND=JPRB) :: ZDP(KLON,KLEV), ZDZ(KLON,KLEV), ZTH(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZRHCL(KLON,KLEV), ZRHO(KLON,KLEV) 
REAL(KIND=JPRB) :: ZQSAT(KLON,KLEV)
REAL(KIND=JPRB) :: ZAEROMGI(KLON,19), ZAERASYI(KLON,19), ZDUM(KLON,KLEV), ZDUM3(KLON,NWLID,0:KLEV)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "aer_bdgtmss.intfb.h"
#include "aer_climz.intfb.h"
#include "aer_climg.intfb.h"
#include "aer_clist.intfb.h"
#include "aer_tau2mixr.intfb.h"
#include "satur.intfb.h"

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_PHY1',0,ZHOOK_HANDLE)

!-- Aerosols are started from either a zonal annual mean or monthly
!   geographical mean climatology, 
!   set on the first time-step then let to evolve with dynamics 
!   (and maybe vdiff and conv, depending on LVDFTRAC and LMFTRAC)
 
IFLAG=2
CALL SATUR (KIDIA , KFDIA , KLON  , KTDIA , KLEV,&
  & PAPRSF, PTP    , ZQSAT , IFLAG)  

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZRHCL(JL,JK)=PQ(JL,JK)/ZQSAT(JL,JK)
  ENDDO
ENDDO

  DO JK=1,KLEV-1
    DO JL=KIDIA,KFDIA
      ZTH(JL,JK)=(PTP(JL,JK)*PRSF1(JL,JK)&
       & *(PRSF1(JL,JK+1)-PRS1(JL,JK+1))&
       & +PTP(JL,JK+1)*PRSF1(JL,JK+1)*(PRS1(JL,JK+1)-PRSF1(JL,JK)))&
       & *(1.0_JPRB/(PRS1(JL,JK+1)*(PRSF1(JL,JK+1)-PRSF1(JL,JK))))  
    ENDDO
  ENDDO
  DO JL=KIDIA,KFDIA
    ZTH(JL,0)=PTP(JL,1)-PRSF1(JL,1)*(PTP(JL,1)-ZTH(JL,1))&
     & /(PRSF1(JL,1)-PRS1(JL,1)) 
    ZTH(JL,KLEV)=PTL(JL)  
  ENDDO

  IF (LAERCLIMZ .OR. LAERCLIMG) THEN
    IF (LAERCLIMZ) THEN
      CALL AER_CLIMZ ( KIDIA, KFDIA, KLON, KLEV, NACTAERO, &
         & PRS1, PRSF1, ZTH, &
         & ZAEROCLIM )
    ELSEIF (LAERCLIMG) THEN
      CALL AER_CLIMG ( KIDIA, KFDIA, KLON, KLEV, NACTAERO, &
         & PRS1, PGEMU, PCLON, PSLON, ZTH, &
         & ZAEROCLIM )
    ENDIF
! BEN - call AER_BDGTMSS routine to go from optical depth to mixing ratio
    LEPAERO=.FALSE.
    IWHAT=2
    INWAVL=1
    ITWAVL(1)=8
!-- Caution: if LEPAERO=.T., PAEROP is in kg kg-1 and  ZCAERO is in kg cm-3
!-- Caution: if LEPAERO=.F., PAEROP is in kg kg-1 and  ZCAERO is unitless
! optical thicknesses can be computed over the 19 following wavelengths:

  CALL AER_BDGTMSS &
   & ( KIDIA, KFDIA , KLON  , KLEV, NACTAERO, IWHAT, INWAVL, ITWAVL, KSTART, KSTEP, &
   &   PRS1 , PAEROP, ZAEROCLIM, ZDUM, ZDUM, ZRHCL, ZDUM, &
   &   PAERMSS, PAERMSST, PAERTAU , PAERTAUT, PAERTAUI, PAERTAUB, PAERTAUA, PAERTAUF, PAERTAUO, &
   &   ZAEROMGI,ZAERASYI, ZDUM3, ZDUM3 )

  IF (NACTAERO > 0 .AND. KSTEP == KSTART) THEN
    DO JGFL=1,NACTAERO
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
!          PGFL(JL,JK,YAERO(JGFL)%MP9)=ZAEROCLIM(JL,JK,JGFL)
!BEN
           IF(JK > 10 ) THEN
            PGFL(JL,JK,YAERO(JGFL)%MP9)=PAERMSS(JL,JK,1)
           ELSE
            PGFL(JL,JK,YAERO(JGFL)%MP9)= 0.0_JPRB
           ENDIF
! initialize total aerosol mixing ratio from climatology
          PGFL(JL,JK,YAERO(NAERO)%MP9)= PGFL(JL,JK,YAERO(NAERO)%MP9)+ PAERMSS(JL,JK,1)
        ENDDO
      ENDDO
    ENDDO
   ENDIF

  ENDIF

  IF (LAERCLIST) THEN
    CALL AER_CLIST ( KIDIA, KFDIA, KLON, KLEV, NACTAERO, &
      & PRS1, PGEMU, PCLON, PSLON, ZTH, &
      & ZAEROCLIS )
    LEPAERO=.TRUE.

    IWHAT=2
    IWAVL=8

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZRHO(JL,JK)=PRSF1(JL,JK)/(RD*PTP(JL,JK))
        ZDP(JL,JK)= PRS1(JL,JK) - PRS1(JL,JK-1)
        ZDZ(JL,JK)= ZDP(JL,JK) / (ZRHO(JL,JK)*RG)  
      ENDDO
    ENDDO

    CALL AER_TAU2MIXR &
      & ( KIDIA , KFDIA , KLON   , KLEV, 2, IWHAT, IWAVL, &
      &   PRS1  , ZAEROCLIS, ZRHCL, &
      &   PAERMSS &
      & )

! BEN
! Comment out for now - need to find a better way to identify aerosol type since now
! there is an extra variable in the PGFL array (i.e. total aerosol mixing ratio)
!    IF (NTYPAER(7) /= 0) THEN
!      JGFL=NAERO-1
!      DO JK=1,KLEV
!        DO JL=KIDIA,KFDIA
!          PGFL(JL,JK,YAERO(JGFL)%MP9)=PAERMASS(JL,JK,1)
!        ENDDO
!      ENDDO
!    ENDIF
!    IF (NTYPAER(8) /= 0) THEN
!      JGFL=NAERO
!      DO JK=1,KLEV
!        DO JL=KIDIA,KFDIA
!          PGFL(JL,JK,YAERO(JGFL)%MP9)=PAERMASS(JL,JK,2)
!        ENDDO
!      ENDDO
!    ENDIF
   ENDIF

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_PHY1',1,ZHOOK_HANDLE)
END SUBROUTINE AER_PHY1

