 SUBROUTINE AER_RADON &
   & ( KIDIA , KFDIA , KLON  , KLEV, &
   &   PCAERO, &
   &   PTAERO &
   & )  

!**** *AER_RADON* RADON DECAY WITH A GIVEN HALF-LIFE

!     PURPOSE.
!     -------
!     - CALCULATION OF RADON DECAY 
!**   Interface.
!     ----------
!        *CALL* *AER_RADON*

!-----------------------------------------------------------------------

! -   INPUT ARGUMENTS.
!     -------------------

! - DIMENSIONS ETC.

!       KIDIA     : START OF HORIZONTAL LOOP
!       KFDIA     : END   OF HORIZONTAL LOOP
!       KLON      : HORIZONTAL DIMENSION
!       KLEV      : END OF VERTICAL LOOP AND VERTICAL DIMENSION

! - 2D (1:KLEV) .

!       PCAERAO   : CONCENTRATION OF RADON

! -   UPDATED ARGUMENTS.
!     -----------------
! - 2D (1:KLEV) TENDENCY .
!       PTAERO    :  TENDENCY FOR RADON CONCENTRATION

!-----------------------------------------------------------------------

!     Externals.
!     ---------

!     Method.
!     -------

!      D(PRN) / DT = PPRN * EXP(-Dt*2.11E-06)

!     Reference.
!    -----------

!     Author
!    --------
!         20050408  JJMorcrette

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY2  , ONLY : TSPHY

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PCAERO(KLON,KLEV) 

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAERO(KLON,KLEV) 

REAL(KIND=JPRB) :: ZCAERO(KLON,KLEV)

INTEGER(KIND=JPIM) :: JK, JL

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*
!     ------------------------------------------------------------------

!         PRELIMINARY COMPUTATIONS AND VARIOUS INITIALIZATIONS

IF (LHOOK) CALL DR_HOOK('AER_RADON',0,ZHOOK_HANDLE)

!*- Updated concentration

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZCAERO(JL,JK) = PCAERO(JL,JK) * EXP(-TSPHY * 2.11E-06_JPRB)
  ENDDO
ENDDO

!*- Tendency

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PTAERO(JL,JK) = (ZCAERO(JL,JK)-PCAERO(JL,JK)) / TSPHY
  ENDDO
ENDDO

!------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_RADON',1,ZHOOK_HANDLE)
END SUBROUTINE AER_RADON

