SUBROUTINE AER_SO2SO4 &
  &( KIDIA, KFDIA , KLON  , KLEV  , &
  &  PSO2 , PGELAT, PTSPHY, &
  &  PTSO2, PTSO4 )

!*** * AER_SO2SO4* - GAS-TO-PARTICLE (SULPHATE AEROSOLS)

!**   INTERFACE.
!     ----------
!          *AER_SO2SO4* IS CALLED FROM *AER_PHY3*.

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*
!        FROM O.BOUCHER and N.HUNNEUS's gastoparticle 

!     SOURCE.
!     -------

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 20080508
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RDAY , RMSO2, RMSO4

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV

REAL(KIND=JPRB)   ,INTENT(IN)    :: PSO2(KLON,KLEV), PGELAT(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSO2(KLON,KLEV), PTSO4(KLON,KLEV)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JL, JK
REAL(KIND=JPRB) :: ZSO4SO2       , ZFACT(KLON)
REAL(KIND=JPRB) :: ZAERCONV(KLON), ZTAUCHEM0(KLON)

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SO2SO4',0,ZHOOK_HANDLE)

ZSO4SO2=RMSO4/RMSO2

DO JL=KIDIA,KFDIA
  ZTAUCHEM0(JL) = RDAY * (8._JPRB - 5._JPRB* COS(PGELAT(JL)) )
  ZFACT(JL)=(1._JPRB-EXP(-PTSPHY/ZTAUCHEM0(JL))) / PTSPHY
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZAERCONV(JL) =  PSO2(JL,JK) * ZFACT(JL)

    PTSO4(JL,JK) =  ZAERCONV(JL) * ZSO4SO2
    PTSO2(JL,JK) = -ZAERCONV(JL)    
  ENDDO
ENDDO
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SO2SO4',1,ZHOOK_HANDLE)
END SUBROUTINE AER_SO2SO4
