SUBROUTINE AER_SSALT &
  &( KIDIA, KFDIA, KLON, KBINSS, &
  &  PCI  , PLSM , PWIND, &
  &  PFLXSS & 
  &)

!*** * AER_SSALT* - SOURCE TERMS FOR SEA SALT AEROSOLS

!**   INTERFACE.
!     ----------
!          *AER_SSALT* IS CALLED FROM *AER_SRC*.

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*
!        FROM O.BOUCHER's seasalt 

!     SOURCE.
!     -------
!     GONG et al., 1997

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2004-05-11

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAERSRC ,ONLY : RSSFLX  

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBINSS

REAL(KIND=JPRB)   ,INTENT(IN)    :: PCI(KLON), PLSM(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWIND(KLON) 

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLXSS(KLON,9)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JL, JBIN
REAL(KIND=JPRB) :: ZFROC, ZOCEA

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SSALT',0,ZHOOK_HANDLE)

! N.B.: RSSFLX in mg m-2 s-1 so PFLXSS is in g cm-2 s-1 after x by 1.E-3 x 1.E-4
! N.B.: RSSFLX in mg m-2 s-1 so PFLXSS is in kg m-2 s-1 after x by         1.E-6

DO JBIN=1,KBINSS
  DO JL=KIDIA,KFDIA
    PFLXSS(JL,JBIN)= 0._JPRB
    ZOCEA= 1._JPRB-PLSM(JL)
    ZFROC= ZOCEA*(1._JPRB-PCI(JL))
!-- flux is considered only over full ocean grids, and for their open ocean fraction
    IF (PLSM(JL) == 0._JPRB) THEN
      PFLXSS(JL,JBIN) = ZFROC * RSSFLX(JBIN) * (PWIND(JL)**3.41_JPRB) * 1.E-06_JPRB
    ENDIF
  ENDDO
ENDDO
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SSALT',1,ZHOOK_HANDLE)
END SUBROUTINE AER_SSALT

