SUBROUTINE AER_STRAACT &
 &( KLAT, KLEV, &
 &  PR2SO4, &
 &  PACTSO4 &
 &)

!*** *AER_STRAACT* - H2SO4 ACTIVITY (GIAUQUE) AS A FUNCTION OF H2SO4 WP

!**   INTERFACE.
!     ----------
!          *AER_STRAACT* IS CALLED FROM *AER_PHY3*.

!     AUTHOR.
!     -------
!        FROM S.BEKKI's *straact*
!        JJMorcrette  ECMWF 20080423 rewritten for IFS

! INPUTS 
! ------
!     PR2SO4: VMR of H2SO4
!     KLAT: number of latitude bands in the model domain
!     KLEV: number of altitude bands in the model domain
!     for IFS: perhaps add another dimension for longitude

! OUTPUTS 
! -------
!     PACTSO4: H2SO4 activity (percent)

!------------------------------------------------------------------------------
 
USE PARKIND1  ,ONLY : JPRB , JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAERST1 ,ONLY : NSAM, RX, RXC

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)  :: KLAT
INTEGER(KIND=JPIM),INTENT(IN)  :: KLEV

REAL(KIND=JPRB)   ,INTENT(IN)  :: PR2SO4(KLAT,KLEV)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PACTSO4(KLAT,KLEV)

 
!*       0.5   LOCAL VARIABLES
!              ---------------

!INTEGER(KIND=JPIM) :: NN,JI,JK,JX,JX1
!REAL(KIND=JPRB) :: ZTC, ZTB, ZTA

INTEGER(KIND=JPIM) :: JI, JK, JN
REAL(KIND=JPRB)    :: ZADD, ZDEN, ZNUM, ZXT

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRAACT',0,ZHOOK_HANDLE)

DO JI=1,KLAT
  DO JK=1,KLEV
! HERE LINEAR INTERPOLATIONS
    ZXT = PR2SO4(JI,JK)

    DO JN=2,NSAM
      IF (ZXT > RXC(JN)) THEN
        ZDEN = (RXC(JN-1)-RXC(JN))
        ZNUM = (RX(JN-1) -RX(JN))
        ZADD = (ZXT - RXC(JN))
        PACTSO4(JI,JK) = RX(JN) + ZADD*ZNUM/ZDEN
      ENDIF
    ENDDO
    IF (ZXT >  RXC(1)   ) PACTSO4(JI,JK)=0.0_JPRB
    IF (ZXT <= RXC(NSAM)) PACTSO4(JI,JK)=15811.0_JPRB
  ENDDO
ENDDO

!DO JI=1,KLAT
!  DO JK=1,KLEV
!! HERE LINEAR INTERPOLATIONS
!    ZXT = PR2SO4(JI,JK)

!    CALL AER_STRPACT(NSAM, JX, ZXT, RXC )

!    JX1=JX+1
!    IF (JX == 0) THEN
!      PACTSO4(JI,JK)=0.0 _JPRB
!    ELSEIF(JX >= NN) THEN
!      PACTSO4(JI,JK)=15811.0_JPRB 
!    ELSE 
!      ZTC=ZXT      - RXC(JX)
!      ZTB=RX(JX1)  - RX(JX)
!      ZTA=RXC(JX1) - RXC(JX)
!      ZTA=ZTB/ZTA
!      PACTSO4(JI,JK) = RX(JX) + ZTA*ZTC
!    ENDIF
!  ENDDO
!ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRAACT',1,ZHOOK_HANDLE)
END SUBROUTINE AER_STRAACT
