SUBROUTINE AER_STRCOMP &
 &( KLAT, KLEV, &
 &  PH2O , PT   , PPMB, &
 &  PR2SO4 &
 &)

!*** *AER-STRCOMP* -AEROSOL H2SO4 WEIGHT FRACTION AS F ( PH2O, TEMP)

!**   INTERFACE.
!     ----------
!          *AER_STRCOMP* IS CALLED FROM *AER_STRAERO*.

!     AUTHOR.
!     -------
!        FROM S.BEKKI's *straact*
!        JJMorcrette  ECMWF 20080423 rewritten for IFS

! INPUTS
! ------
!   PH2O  : VMR of H2O
!   PT    : temperature (K)
!   PPMB  : pressure (mb)
!   KLAT  : number of latitude bands in the model domain
!   KLEV  : number of altitude bands in the model domain
!   for IFS: perhaps add another dimension for longitude

! OUTPUT
! ------  
!   PR2SO4: aerosol H2SO4 weight fraction (percent)

!------------------------------------------------------------------------------
 
USE PARKIND1  ,ONLY : JPRB , JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE
   
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------
INTEGER(KIND=JPIM),INTENT(IN)  :: KLAT
INTEGER(KIND=JPIM),INTENT(IN)  :: KLEV

REAL(KIND=JPRB),INTENT(IN)  :: PPMB(KLAT,KLEV), PH2O(KLAT,KLEV), PT(KLAT,KLEV)

REAL(KIND=JPRB),INTENT(OUT) :: PR2SO4(KLAT,KLEV)

  
!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: I,J, IK, JI, JK
REAL(KIND=JPRB) :: ZTP, ZH2O, ZVAL, ZA, ZB

!     local variables to be saved on exit
INTEGER(KIND=JPIM) :: INSTEP, IM, IN
DATA INSTEP/0/
REAL(KIND=JPRB) :: ZF(16,28)
REAL(KIND=JPRB) :: ZXC(16)
REAL(KIND=JPRB) :: ZYC(28)
REAL(KIND=JPRB) :: ZXC1, ZXC16, ZYC1, ZYC28

SAVE INSTEP,IM,IN,ZF,ZXC,ZYC,ZXC1,ZXC16,ZYC1,ZYC28

#include "aer_strfind.intfb.h"

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRCOMP',0,ZHOOK_HANDLE)

IF (INSTEP == 0) THEN
  
  INSTEP=1
  ZXC(1) =0.01_JPRB
  ZXC(2) =0.1_JPRB
  ZXC(3) =0.5_JPRB
  ZXC(4) =1.0_JPRB
  ZXC(5) =1.5_JPRB
  ZXC(6) =2.0_JPRB
  ZXC(7) =3.0_JPRB
  ZXC(8) =5.0_JPRB
  ZXC(9) =6.0_JPRB
  ZXC(10)=8.0_JPRB
  ZXC(11)=10.0_JPRB
  ZXC(12)=12.0_JPRB
  ZXC(13)=15.0_JPRB
  ZXC(14)=20.0_JPRB
  ZXC(15)=30.0_JPRB
  ZXC(16)=100.0_JPRB

  IN=16
  IM=28
  ZYC(1)=175.0_JPRB

  DO JI=2,IM
    ZYC(JI)=ZYC(JI-1)+5.0_JPRB
  ENDDO

!  CONVERSION mb IN 1.0E-4mB
  DO JI=1,IN
    ZXC(JI)=ZXC(JI)*1.0E-4_JPRB
  ENDDO 

  ZXC1=ZXC(1)  +1.E-10_JPRB
  ZXC16=ZXC(16)-1.E-8_JPRB
  ZYC1=ZYC(1)  +1.E-5_JPRB
  ZYC28=ZYC(28)-1.E-5_JPRB

  ZF(6,4)=43.45_JPRB
  ZF(6,5)=53.96_JPRB
  ZF(6,6)=60.62_JPRB
  ZF(6,7)=65.57_JPRB
  ZF(6,8)=69.42_JPRB
  ZF(6,9)=72.56_JPRB
  ZF(6,10)=75.17_JPRB
  ZF(6,11)=77.38_JPRB
  ZF(6,12)=79.30_JPRB
  ZF(6,13)=80.99_JPRB
  ZF(6,14)=82.50_JPRB
  ZF(6,15)=83.92_JPRB
  ZF(6,16)=85.32_JPRB
  ZF(6,17)=86.79_JPRB
  ZF(6,18)=88.32_JPRB

! ADD FACTOR BECAUSE THE SLOPE IS TOO IMPORTANT
!   NOT FOR THIS ONE BUT THE REST
!      LOG DOESN'T WORK
  ZA=(ZF(6,5)-ZF(6,4))/( (ZYC(5)-ZYC(4))*2.0_JPRB)
  ZB=-ZA*ZYC(4) + ZF(6,4)
  ZF(6,1)=ZA*ZYC(1) + ZB
  ZF(6,2)=ZA*ZYC(2) + ZB
  ZF(6,3)=ZA*ZYC(3) + ZB


  ZF(7,4)=37.02_JPRB
  ZF(7,5)=49.46_JPRB
  ZF(7,6)=57.51_JPRB
  ZF(7,7)=63.12_JPRB
  ZF(7,8)=67.42_JPRB
  ZF(7,9)=70.85_JPRB
  ZF(7,10)=73.70_JPRB
  ZF(7,11)=76.09_JPRB
  ZF(7,12)=78.15_JPRB
  ZF(7,13)=79.96_JPRB
  ZF(7,14)=81.56_JPRB
  ZF(7,15)=83.02_JPRB
  ZF(7,16)=84.43_JPRB
  ZF(7,17)=85.85_JPRB
  ZF(7,18)=87.33_JPRB

  ZA=(ZF(7,5)-ZF(7,4))/( (ZYC(5)-ZYC(4))*2.0_JPRB)
  ZB=-ZA*ZYC(4) + ZF(7,4)
  ZF(7,1)=ZA*ZYC(1) + ZB
  ZF(7,2)=ZA*ZYC(2) + ZB
  ZF(7,3)=ZA*ZYC(3) + ZB

  ZF(8,4)=25.85_JPRB
  ZF(8,5)=42.26_JPRB
  ZF(8,6)=52.78_JPRB
  ZF(8,7)=59.55_JPRB
  ZF(8,8)=64.55_JPRB
  ZF(8,9)=68.45_JPRB
  ZF(8,10)=71.63_JPRB
  ZF(8,11)=74.29_JPRB
  ZF(8,12)=76.56_JPRB
  ZF(8,13)=78.53_JPRB
  ZF(8,14)=80.27_JPRB
  ZF(8,15)=81.83_JPRB
  ZF(8,16)=83.27_JPRB
  ZF(8,17)=84.67_JPRB
  ZF(8,18)=86.10_JPRB

  ZA=(ZF(8,5)-ZF(8,4))/( (ZYC(5)-ZYC(4))*2.5_JPRB )
  ZB=-ZA*ZYC(4) + ZF(8,4)
  ZF(8,1)=ZA*ZYC(1) + ZB
  ZF(8,2)=ZA*ZYC(2) + ZB
  ZF(8,3)=ZA*ZYC(3) + ZB

  ZF(9,4)=15.38_JPRB
  ZF(9,5)=39.35_JPRB
  ZF(9,6)=50.73_JPRB
  ZF(9,7)=58.11_JPRB
  ZF(9,8)=63.41_JPRB
  ZF(9,9)=67.52_JPRB
  ZF(9,10)=70.83_JPRB
  ZF(9,11)=73.60_JPRB
  ZF(9,12)=75.95_JPRB
  ZF(9,13)=77.98_JPRB
  ZF(9,14)=79.77_JPRB
  ZF(9,15)=81.38_JPRB
  ZF(9,16)=82.84_JPRB
  ZF(9,17)=84.25_JPRB
  ZF(9,18)=85.66_JPRB

  ZA=(ZF(9,5)-ZF(9,4))/( (ZYC(5)-ZYC(4))*7.0_JPRB)
  ZB=-ZA*ZYC(4) + ZF(9,4)
  ZF(9,1)=ZA*ZYC(1) + ZB
  ZF(9,2)=ZA*ZYC(2) + ZB
  ZF(9,3)=ZA*ZYC(3) + ZB

  ZF(10,4)=0.0_JPRB
  ZF(10,5)=34.02_JPRB
  ZF(10,6)=46.93_JPRB
  ZF(10,7)=55.61_JPRB
  ZF(10,8)=61.47_JPRB
  ZF(10,9)=65.94_JPRB
  ZF(10,10)=69.49_JPRB
  ZF(10,11)=72.44_JPRB
  ZF(10,12)=74.93_JPRB
  ZF(10,13)=77.08_JPRB
  ZF(10,14)=78.96_JPRB
  ZF(10,15)=80.63_JPRB
  ZF(10,16)=82.15_JPRB
  ZF(10,17)=83.57_JPRB
  ZF(10,18)=84.97_JPRB

  ZA=(ZF(10,6)-ZF(10,5))/( (ZYC(6)-ZYC(5))*1.5_JPRB)
  ZB=-ZA*ZYC(5) + ZF(10,5)
  ZF(10,1)=ZA*ZYC(1) + ZB
  ZF(10,2)=ZA*ZYC(2) + ZB
  ZF(10,3)=ZA*ZYC(3) + ZB
  ZF(10,4)=ZA*ZYC(4) + ZB

  ZF(11,4)=0.0_JPRB
  ZF(11,5)=29.02_JPRB
  ZF(11,6)=43.69_JPRB
  ZF(11,7)=53.44_JPRB
  ZF(11,8)=59.83_JPRB
  ZF(11,9)=64.62_JPRB
  ZF(11,10)=68.39_JPRB
  ZF(11,11)=71.48_JPRB
  ZF(11,12)=74.10_JPRB
  ZF(11,13)=76.33_JPRB
  ZF(11,14)=78.29_JPRB
  ZF(11,15)=80.02_JPRB
  ZF(11,16)=81.58_JPRB
  ZF(11,17)=83.03_JPRB
  ZF(11,18)=84.44_JPRB

  ZA=(ZF(11,6)-ZF(11,5))/( (ZYC(6)-ZYC(5))*2.5_JPRB )
  ZB=-ZA*ZYC(5) + ZF(11,5)
  ZF(11,1)=ZA*ZYC(1) + ZB
  ZF(11,2)=ZA*ZYC(2) + ZB
  ZF(11,3)=ZA*ZYC(3) + ZB
  ZF(11,4)=ZA*ZYC(4) + ZB

  ZF(12,4)=0.0_JPRB
  ZF(12,5)=23.13_JPRB
  ZF(12,6)=40.86_JPRB
  ZF(12,7)=51.44_JPRB
  ZF(12,8)=58.38_JPRB
  ZF(12,9)=63.47_JPRB
  ZF(12,10)=67.43_JPRB
  ZF(12,11)=70.66_JPRB
  ZF(12,12)=73.38_JPRB
  ZF(12,13)=75.70_JPRB
  ZF(12,14)=77.72_JPRB
  ZF(12,15)=79.51_JPRB
  ZF(12,16)=81.11_JPRB
  ZF(12,17)=82.58_JPRB
  ZF(12,18)=83.99_JPRB

  ZA=(ZF(12,6)-ZF(12,5))/( (ZYC(6)-ZYC(5))*3.5_JPRB )
  ZB=-ZA*ZYC(5) + ZF(12,5)
  ZF(12,1)=ZA*ZYC(1) + ZB
  ZF(12,2)=ZA*ZYC(2) + ZB
  ZF(12,3)=ZA*ZYC(3) + ZB
  ZF(12,4)=ZA*ZYC(4) + ZB

  ZF(13,4)=0.0_JPRB
  ZF(13,5)=0.0_JPRB
  ZF(13,6)=36.89_JPRB
  ZF(13,7)=48.63_JPRB
  ZF(13,8)=56.46_JPRB
  ZF(13,9)=61.96_JPRB
  ZF(13,10)=66.19_JPRB
  ZF(13,11)=69.60_JPRB
  ZF(13,12)=72.45_JPRB
  ZF(13,13)=74.89_JPRB
  ZF(13,14)=76.99_JPRB
  ZF(13,15)=78.85_JPRB
  ZF(13,16)=80.50_JPRB
  ZF(13,17)=82.02_JPRB
  ZF(13,18)=83.44_JPRB

  ZA=(ZF(13,7)-ZF(13,6))/( (ZYC(7)-ZYC(6))*2.0_JPRB)
  ZB=-ZA*ZYC(6) + ZF(13,6)
  ZF(13,1)=ZA*ZYC(1) + ZB
  ZF(13,2)=ZA*ZYC(2) + ZB
  ZF(13,3)=ZA*ZYC(3) + ZB
  ZF(13,4)=ZA*ZYC(4) + ZB
  ZF(13,5)=ZA*ZYC(5) + ZB

  ZF(14,4)=0.0_JPRB
  ZF(14,5)=0.0_JPRB
  ZF(14,6)=30.82_JPRB
  ZF(14,7)=44.49_JPRB
  ZF(14,8)=53.69_JPRB
  ZF(14,9)=59.83_JPRB
  ZF(14,10)=64.47_JPRB
  ZF(14,11)=68.15_JPRB
  ZF(14,12)=71.19_JPRB
  ZF(14,13)=73.77_JPRB
  ZF(14,14)=76.00_JPRB
  ZF(14,15)=77.95_JPRB
  ZF(14,16)=79.69_JPRB
  ZF(14,17)=81.26_JPRB
  ZF(14,18)=82.72_JPRB

  ZA=(ZF(14,7)-ZF(14,6))/( (ZYC(7)-ZYC(6))*2.5_JPRB )
  ZB=-ZA*ZYC(6) + ZF(14,6)
  ZF(14,1)=ZA*ZYC(1) + ZB
  ZF(14,2)=ZA*ZYC(2) + ZB
  ZF(14,3)=ZA*ZYC(3) + ZB
  ZF(14,4)=ZA*ZYC(4) + ZB
  ZF(14,5)=ZA*ZYC(5) + ZB

  ZF(15,4)=0.0_JPRB
  ZF(15,5)=0.0_JPRB
  ZF(15,6)=0.0_JPRB
  ZF(15,7)=37.71_JPRB
  ZF(15,8)=48.49_JPRB
  ZF(15,9)=56.40_JPRB
  ZF(15,10)=61.75_JPRB
  ZF(15,11)=65.89_JPRB
  ZF(15,12)=69.25_JPRB
  ZF(15,13)=72.07_JPRB
  ZF(15,14)=74.49_JPRB
  ZF(15,15)=76.59_JPRB
  ZF(15,16)=78.45_JPRB
  ZF(15,17)=80.12_JPRB
  ZF(15,18)=81.64_JPRB

  ZA=(ZF(15,8)-ZF(15,7))/( (ZYC(8)-ZYC(7))*1.5_JPRB)
  ZB=-ZA*ZYC(7) + ZF(15,7)
  ZF(15,1)=ZA*ZYC(1) + ZB
  ZF(15,2)=ZA*ZYC(2) + ZB
  ZF(15,3)=ZA*ZYC(3) + ZB
  ZF(15,4)=ZA*ZYC(4) + ZB
  ZF(15,5)=ZA*ZYC(5) + ZB
  ZF(15,6)=ZA*ZYC(6) + ZB

! SUPPOSE THAT AT GIVEN  AND PH2O<2mB,
!  %H2SO4 = A *LOG(PH2O) +B
!  ZXC(1-5) :EXTENSION LEFT (LOW H2O)

  DO J=1,18
    ZA=(ZF(6,J)-ZF(7,J))/(LOG(ZXC(6))-LOG(ZXC(7)))
    ZB=-ZA*LOG(ZXC(6)) + ZF(6,J)
    DO IK=1,5
      ZF(IK,J)=ZA*LOG(ZXC(IK)) + ZB
    ENDDO
  ENDDO

! ZXC(16) :EXTENSION RIGHT (HIGH H2O)
  DO J=1,18
    ZA=(ZF(15,J)-ZF(14,J))/(ZXC(15)-ZXC(14))
    ZB=-ZA*ZXC(15) + ZF(15,J)
    ZF(16,J)=ZA*ZXC(16) + ZB
  ENDDO
!   ZF(16,2)=1.0

!   ZYC(16-25) :EXTENSION DOWN (HIGH T)
  DO I=1,IN
    ZA=(ZF(I,18)-ZF(I,17))/(ZYC(18)-ZYC(17))
    ZB=-ZA*ZYC(18) + ZF(I,18)
    DO IK=19,28
      ZF(I,IK)=ZA*ZYC(IK) + ZB
    ENDDO
  ENDDO

! MANUAL CORRECTIONS
  DO J=1,10
    ZF(1,J)=94.0_JPRB
  ENDDO

  DO J=1,6
    ZF(2,J)=77.0_JPRB +REAL(J)
  ENDDO

  DO J=1,7
    ZF(16,J)=9.0_JPRB
  ENDDO

  DO I=1,IN
    DO J=1,IM
      IF (ZF(I,J) < 9.0_JPRB)  ZF(I,J)=30.0_JPRB
      IF (ZF(I,J) > 99.99_JPRB) ZF(I,J)=99.99_JPRB
    ENDDO
  ENDDO
      
ENDIF

DO JI=1,KLAT
  DO JK=1,KLEV
    ZTP=PT(JI,JK)
    IF (ZTP < 175.1_JPRB) ZTP=175.1_JPRB

! Partial pressure of H2O (mb)
    ZH2O =PPMB(JI,JK)*PH2O(JI,JK)
    IF(ZH2O < ZXC1) THEN 
      PR2SO4(JI,JK)=99.99_JPRB
!      ZH2O=ZXC(1)+1.0E-10_JPRB
    ELSE 
      IF(ZH2O > ZXC16) ZH2O=ZXC16 
! SIMPLE LINEAR INTERPOLATIONS

      CALL AER_STRFIND ( IN, IM, ZH2O, ZTP, ZXC, ZYC, ZF, ZVAL )

      IF(PPMB(JI,JK) >= 10.0_JPRB .AND. ZVAL < 60.0_JPRB) ZVAL=60.0_JPRB
      PR2SO4(JI,JK)=ZVAL
    ENDIF
  ENDDO
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRCOMP',1,ZHOOK_HANDLE)
END SUBROUTINE AER_STRCOMP
