SUBROUTINE AER_STRDENS &
  &( KLAT, KLEV, &
  &  PR2SO4, PT, &
  &  PDENSO4)

!*** *AER_STRDENS* - DERIVE AEROSOL DENSITY AS F (H2SO4 WEIGHT PERCENT AND TEMP.)

!     VERY ROUGH APPROXIMATION (SEE FOR WATER IN HANDBOOK
!     LINEAR 2% FOR 30 DEGREES with RESPECT TO WATER)

!     AUTHOR.
!     -------
!        FROM S.BEKKI's *densh2sa*
!        JJMorcrette  ECMWF 20080423 rewritten for IFS

! INPUTS
! ------
!   PR2SO4 : aerosol H2SO4 weight fraction (percent)
!   PT     : temperature (K)
!   KLAT   : number of latitude bands in the model domain
!   KLEV   : number of altitude bands in the model domain
!   for IFS: perhaps add another dimension for longitude

! OUTPUT
! ------
!   PDENSO4: aerosol mass density (gr/cm3 = aerosol mass/aerosol volume)
!------------------------------------------------------------------------------
 
USE PARKIND1  ,ONLY : JPRB , JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE
   
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN) :: KLAT,KLEV

REAL(KIND=JPRB),INTENT(IN) :: PR2SO4(KLAT,KLEV), PT(KLAT,KLEV)

REAL(KIND=JPRB),INTENT(OUT) :: PDENSO4(KLAT,KLEV)
   

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JI, JK

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRDENS',0,ZHOOK_HANDLE)

! Loop on model domain (2 dimension for UPMC model; 3 for IFS)
DO JI=1,KLAT
  DO JK=1,KLEV
!   RO AT 20C
    PDENSO4(JI,JK)=0.78681252E-5_JPRB *PR2SO4(JI,JK)*PR2SO4(JI,JK) &
      & + 0.82185978E-2_JPRB * PR2SO4(JI,JK) + 0.97968381_JPRB
    PDENSO4(JI,JK)=PDENSO4(JI,JK) &
      & * ( 1.0_JPRB - (PT(JI,JK)-293.0_JPRB)*0.02_JPRB/30.0_JPRB )
!   RO=1.4
  ENDDO
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRDENS',1,ZHOOK_HANDLE)
END SUBROUTINE AER_STRDENS
