SUBROUTINE CALLPARAD (KIDIA,KFDIA,KLON,KSMAX,KLEVS,KTILES,&
 & KSTGLO,&
 & KTDIA,KLEV,KSTART,KSTEP,LDRAIN1D,&
 !-----------------------------------------------------------------------
 ! - INPUT (TRAJECTORY) .
 & PT5    , PQ5, PGFL5,&
 & PU5    , PV5    , PVERVEL5,&
 & PRS15  , PRSF15,&
 & PAPRS5 , PAPRSF5, PDELP5,&
 & PAPHI5 , PAPHIF5,&
 & PEMTD5 , PTRSO5 , PMU0,   PMU0M,&
 & PSNS5  , PTSA5  , PWL5,   PTL5,&
 & PWSA5  , PTIA5,&
 & PASN5  , PRSN5  , PTSN5,&
 & PHSTD  , PHSDFOR, PGAMMA,&
 & PTHETA , PSIG,&
 & PLSM   , PALBF  , &
 & PALUVP , PALUVD , PALNIP, PALNID,&
 & PEMISF,&
 & PCO2O, PCO2B, PCO2A,PCO2F,PCH4,PCH4F,PSF6,PGFLFLX,&
 & PCVL   , PCVH   , PTVL  , PTVH ,&
 & PLAILC , PLAIHC ,&
 & PSOTY , PSOALUVP, PSOALUVD, PSOALNIP, PSOALNID,&
 & PCI , PSST,&
 & PGELAM , PCLON  , PSLON,&
 & PGZ0MCL  ,PLZ0HCL, PGEMU, POROG, &
 & PGZ0M5 , PLZ0H5,&
 & PU10N5 , PV10N5,&
 ! - UPDATED (TRAJECTORY)
 & PTENT5 , PTENQ5, PTENGFL5,&
 & PTENU5 , PTENV5,&
 & PNTOP  , PNBAS  , PACPR5,&
 & PUSTRTI5, PVSTRTI5, PAHFSTI5,PEVAPTI5 ,PTSKTI5,&
 !====================================================================     
 ! - INPUT .
 & PT     , PQ     , PGFL,&
 & PU     , PV     , PVERVEL,&
 & PL     , PI     , PA     ,&
 & PRS1   , PRSF1,&
 & PAPRS  , PAPRSF , PDELP,&
 & PAPHI  , PAPHIF,&
 & PTSA   , PTL,&
 & PWSA,&
 ! - UPDATED
 & PTENT  , PTENQ, PTENGFL,&
 & PTENU  , PTENV,&
 & PACPR,&
 & PUSTRTI, PVSTRTI, PAHFSTI,PEVAPTI ,PTSKTI,&
 ! - OUTPUT
 & PDIFTQ , PDIFTS , PSTRTU , PSTRTV,&
 & PGZ0M  , PLZ0H,&
 & PU10N  , PV10N, &
 & PTLE1,YD_MWAVE_RAD)        

!**** *CALLPARAD * - CALL ECMWF PHYSICS (Adjoint)

!     PURPOSE.
!     --------
!     - CALL THE SUBROUTINES OF THE E.C.M.W.F. PHYSICS PACKAGE.

!**   Interface.
!     ----------
!        *CALL* *CALLPARAD*

!-----------------------------------------------------------------------

! -   INPUT ARGUMENTS.
!     -------------------

! - DIMENSIONS ETC.

! KIDIA   : START OF HORIZONTAL LOOP
! KFDIA   : START OF HORIZONTAL LOOP
! KLON    : HORIZONTAL DIMENSION
! KSMAX   : HORIZONTAL SPECTRAL TRUNCATION
! KLEVS   : NUMBER OF LEVELS IN SOIL
! KTILES  : NUMBER OF vegetation (surface cover) types
! KTDIA   : START OF THE VERTICAL LOOP IN THE PHYSICS
!            (IF SOME LEVELS ARE SKIPPED AT THE TOP OF THE MODEL).
! KLEV    : END OF VERTICAL LOOP AND VERTICAL DIMENSION
! KSTART  : FIRST STEP OF MODEL
! KSTEP   : CURRENT TIME STEP

!-----------------------------------------------------------------------

! - 2D (0:KLEV) .

! PAPHI5     : GEOPOTENTIAL ON HALF-LEVELS.             (Trajectory)
! PAPRS5     : PRESSURE ON HALF-LEVELS.                 (Trajectory)
! PRS15      : PROVISIONAL T+DT PRESSURE ON HALF LEVELS (Trajectory)
! PEMTD5     : DOWNWARD LONGWAVE EMISSIVITY             (Trajectory)
! PTRSO5     : SHORTWAVE TRANSMISSIVITY                 (Trajectory)

! - 2D (1:KLEV) .

! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.             (Trajectory)
! PAPRSF5    : PRESSURE ON FULL LEVELS.                 (Trajectory)
! PDELP5     : LAYER THICKNESS IN PRESSURE UNITS.       (Trajectory)
! PT5        : TEMPERATURE.                             (Trajectory)
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOUR.       (Trajectory)
! PU5        : X-COMPONENT OF WIND                      (Trajectory)
! PV5        : Y-COMPONENT OF WIND                      (Trajectory)
! PVERVEL5   : VERTICAL VELOCITY                        (Trajectory)
! PRSF15     : PROVISIONAL T+DT PRESSURE ON FULL LEVELS (Trajectory)
! PGFL5      : GFL FIELDS                               (Trajectory)

! - 1D (PROGNOSTIC QUANTITIES) .

! PSNS5      : MASS OF SNOW PER UNIT SURFACE.           (Trajectory)
! PTSA5      : MULTI-LAYER SOIL TEMPERATURE             (Trajectory)
! PWSA5      : MULTI-LAYER SOIL WETNESS                 (Trajectory)
! PTIA5      : MULTI-LAYER ICE TEMPERATURE              (Trajectory)
! PWL5       : SKIN RESERVOIR WATER CONTENT             (Trajectory) 
! PTL5       : SKIN (BRIGHTNESS) TEMPERATURE            (Trajectory)
! PASN5      : SNOW ALBEDO                              (Trajectory)
! PRSN5      : SNOW DENSITY                             (Trajectory)
! PTSN5      : SNOW TEMPERATURE                         (Trajectory)


! - 1D (GEOGRAPHICAL DISTRIBUTION) .

! PCLON      : COSINE OF LONGITUDE
! PSLON      : SINE   OF LONGITUDE
! PALBF      : BACKGROUND SURFACE SHORTWAVE ALBEDO (SNOW-FREE).
! PALUVP     : MODIS ALBEDO UV-VIS PARALLEL
! PALUVD     : MODIS ALBEDO UV-VIS DIFFUSE
! PALNIP     : MODIS ALBEDO NEAR IR PARALLEL
! PALNID     : MODIS ALBEDO NEAR IR DIFFUSE
! PCVL       : LOW VEGETATION COVER
! PCVH       : HIGH VEGETATION COVER
! PTVL       : LOW VEGETATION DOMINANT TYPE
! PTVH       : HIGH VEGETATION DOMINANT TYPE
! PLAILC     : LOW VEGETATION LAI (VALUE FROM CLIMATE FILE)
! PLAIHC     : HIGH VEGETATION LAI (VALUE FROM CLIMATE FILE)
! PSOTY      : SOIL TYPE
! PSOALUVP    : SOIL ALBEDO VIS/UV DIRECT
! PSOALUVD    : SOIL ALBEDO VIS/UV DIFFUSE
! PSOALNIP    : SOIL ALBEDO NIR DIRECT
! PSOALNID    : SOIL ALBEDO NIR DIFFUSE
! PCI        : SEA-ICE FRACTION
! PSST       : (OPEN) SEA SURFACE TEMPERATURE
! PEMISF     : BACKGROUND SURFACE LONGWAVE EMISSIVITY (SNOW-FREE).
! PLSM       : LAND/SEA MASK.
! PCO2O      : Ocean CO2 flux
! PCO2B      : Biosphere CO2 flux
! PCO2A      : Anthropogenic CO2 flux
! PCO2F      : Biomass burning CO2 flux
! PCH4       : Agrregated CH4 surface fluxes
! PCH4F      : CH4 fire emissions
! PSF6       : SF6 emissions
! PGFLFLX    : flux for GRG and TRAC species
! PGZ0MCL    : GRAVITY * ROUGHNESS LENGTH
! PMU0       : LOCAL COSINE OF INSTANTANEOUS SOLAR ZENITH ANGLE.
! PMU0M      : LOCAL COSINE OF INSTANTANEOUS MEAN SOLAR ZENITH ANGLE.
! PSIG       : MEAN SUBGRID SLOPE
! PGAMMA     : ANISOTROPY OF SUBGRID OROGRAPHY ("ASPECT RATIO")
! PTHETA     : DIRECTION (ANGLE) WHERE MEAN SQUARE GRADIENT IS LARGEST
! PHSTD      : STANDARD DEVIATION OF SUBGRID OROGRAPHY
! PHSDFOR    : NEW STANDARD DEVIATION OF SUBGRID OROGRAPHY
! PLZ0HCL    : LOGARITHM OF ROUGHNESS LENGTH FOR HEAT
! PGEMU      : SINE OF LATITUDE
! POROG      : OROGRAPHY

!-----------------------------------------------------------------------

! -   UPDATED ARGUMENTS.
!     --------------------

! - 2D (1:KLEV) .

! PTENT5     : TENDENCY OF TEMPERATURE.                 (Trajectory)
! PTENQ5     : TENDENCY OF HUMIDITY                     (Trajectory)
! PTENU5     : TENDENCY OF U-COMP. OF WIND.             (Trajectory)
! PTENV5     : TENDENCY OF U-COMP. OF WIND.             (Trajectory)
! PTENGFL5   : TENDENCY OF ALL GFL FIELDS               (Trajectory)
! PNTOP      : CONVECTIVE CLOUD TOP INDEX (TO BE CONVERTED TO INTEGER)
! PNBAS      : CONVECTIVE CLOUD BASE INDEX (TO BE CONVERTED TO INTEGER)
! PACPR5     : CONVECTIVE RAINFALL ACCUMULATED BETWEEN TWO
!              (FULL) RADIATION TIME STEPS
! PUSTRTI5   : E-W (INSTANTANEOUS) SURFACE STRESS 
!              FOR EACH TILE                            (Trajectory)
! PVSTRTI5   : N-S (INSTANTANEOUS) SURFACE STRESS 
!              FOR EACH TILE                            (Trajectory)
! PAHFSTI5   : (INSTANTANEOUS) SURFACE SENSIBLE HEAT FLUX 
!              FOR EACH TILE                            (Trajectory)
! PEVAPTI5   : (INSTANTANEOUS) EVAPORATION 
!              FOR EACH TILE                            (Trajectory)
! PTSKTI5    : SKIN TEMPERATURE FOR EACH TILE           (Trajectory)

!-----------------------------------------------------------------------

! - 2D (0:KLEV) .

! PAPHI      : GEOPOTENTIAL ON HALF-LEVELS.
! PAPRS      : PRESSURE ON HALF-LEVELS.
! PRS1       : PROVISIONAL T+DT PRESSURE ON HALF LEVELS

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF     : PRESSURE ON FULL LEVELS.
! PDELP      : LAYER THICKNESS IN PRESSURE UNITS.  
! PT         : TEMPERATURE.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOUR.
! PU         : X-COMPONENT OF WIND
! PV         : Y-COMPONENT OF WIND
! PVERVEL    : VERTICAL VELOCITY
! PRSF1      : PROVISIONAL T+DT PRESSURE ON FULL LEVELS.
! PGFL       : GFL FIELDS

! - 1D (PROGNOSTIC QUANTITIES) .

! PTSA       : MULTI-LAYER SOIL TEMPERATURE     
! PWSA       : MULTI-LAYER SOIL WETNESS         
! PTL        : SKIN (BRIGHTNESS) TEMPERATURE    

! - 1D (DIAGNOSTIC QUANTITIES) .

!-----------------------------------------------------------------------

! -   UPDATED ARGUMENTS.
!     --------------------

! - 2D (1:KLEV) .

! PTENT      : TENDENCY OF TEMPERATURE.
! PTENQ      : TENDENCY OF HUMIDITY
! PTENU      : TENDENCY OF U-COMP. OF WIND.     
! PTENV      : TENDENCY OF U-COMP. OF WIND.
! PTENGFL    : TENDENCY OF ALL GFL FIELDS  
! PACPR      : CONVECTIVE RAINFALL ACCUMULATED BETWEEN TWO
!              (FULL) RADIATION TIME STEPS
! PUSTRTI   : E-W (INSTANTANEOUS) SURFACE STRESS FOR EACH TILE    
! PVSTRTI   : N-S (INSTANTANEOUS) SURFACE STRESS FOR EACH TILE    
! PAHFSTI   : (INSTANTANEOUS) SURFACE SENSIBLE HEAT FLUX FOR EACH TILE        
! PEVAPTI   : (INSTANTANEOUS) EVAPORATION FOR EACH TILE                       
! PTSKTI    : SKIN TEMPERATURE FOR EACH TILE

!-----------------------------------------------------------------------

! -   OUTPUT ARGUMENTS.
!     --------------------

! - 2D (0:KLEV) FLUXES .

! PDIFTQ     : TURBULENT FLUX (INC. Q NEGATIVE) OF SPECIFIC HUMIDITY.
! PDIFTS     : TURBULENT FLUX OF ENTHALPY (OR DRY STATIC ENERGY).
! PSTRTU     : TURBULENT FLUX OF MOMENTUM "U".
! PSTRTV     : TURBULENT FLUX OF MOMENTUM "V".

! - 1D (TENDENCIES FROM SURFACE SCHEME)

! PTLE1      : OF SKIN TEMPERATURE

! - 1D (DIAGNOSTIC) .

! PGZ0M      : G * ROUGHNESS LENGTH (CURRENT).
! PLZ0H      : LOGARITHM OF ROUGHNESS LENGTH FOR HEAT (CURRENT)

!-----------------------------------------------------------------------

!     Externals.  
!     ----------

!     SURFRAD   -
!     SATUR     -
!     CLOUD     -
!     RADHEAT   -
!     VDFMAINS  -
!     GWDRAG    -
!     CUCALLN   -
!     CUCALLN2  -
!     COND      - 
!     CONDAD    - 
!     GWDRAG_WMSS -
!     GWDRAG_WMSSAD -
!     CUCALLNAD - 
!     CUCALLN2AD - 
!     GWDRAGAD  -
!     VDFMAINSAD -
!     RADHEATAD -
!     CLOUDAD   -
!     SATURAD   -

!     Method. See documentaion.
!     -------
!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     J.F. MAHFOUF        E.C.M.W.F.          13/12/95

!     Adapted from CALLPAR for TL and AD versions

!     ORIGINAL 93-10-04 M.HAMRUD/P.VITERBO (FROM APLPAR)

!     Adapted to tile approach : 14/06/1999 (JFM)
!     Adapted for sw radation  :    06/1999 (M. Janiskova)
!             for new lw radiation: 02/2000 (M. Janiskova)  

!     MODIFIED 2000-01-21 M. Janiskova (reading the trajectory
!                         (for phys.tendecies from file insted
!                          of re-computing them)
!        D.Salmond (15-10-01) FULLIMP mods
!              2002-10-31 M. Janiskova (new cloud scheme - cloudst)
!     Modified 2003-12-15 P. Lopez: separate call to simplified convection
!                                   scheme.
!     P. Viterbo 2004-05-24 Change surface units
!     M. Ko"hler 2004-12-03 Adoptation to moist advection-diffusion PBL
!        D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!     P. Lopez + P. Bechtold 2006-02-11  Add Tracers
!     JJMorcrette 20060625 MODIS albedo
!     G. Balsamo 20070115 Soil type
!     A. Geer     25-04-2008  Rainy 4D-Var for multiple sensors
!     G. Balsamo  10-10-2008  Lake tile
!     A. Geer     01-10-2008  Rainy 4D-Var: tidying, orography, compatability
!     M. Janiskova 2008-04-10 Frequency of calling LW radiation
!     H. Hersbach   04-Dec-2009 10-m neutral wind
!     M.Janiskova 18-09-2009  Call to non-orographic gwd
!     S. Boussetta/G.Balsamo 05-2009 Added variable LAI
!     M.Janiskova 18-05-2010  Modified use of saved trajectory for LEMWAVE
!                             to avoid re-computation 
!     A. Geer    07-Apr-2010 cloud / precip fraction for all-sky
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMPHY2  , ONLY : TSPHY
USE YOERAD   , ONLY : NSW      ,NTSW      ,NMODE    , RCCNLND, RCCNSEA
USE YOERDI   , ONLY : RCARDI
USE YOMCST   , ONLY : RG       ,RLVTT    ,RLSTT    ,RTT  ,RCPD, RDAY

USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOPHNC   , ONLY : LETRAJPT ,LERADI2  ,LERADS2  ,LERADSW2 ,&
 & LERADN2  ,LERADFL2 ,LERADLW2 ,LEDCLD2  ,LENCLD2 ,LEVDIF2  ,&
 & LEGWDG2  ,LECUMF2  ,LECOND2  ,LEGWWMS2 ,LEQNGT2 ,&
 & LESURF2  ,LTRACLNPH
USE YOMIOP   , ONLY : NG3DT95   ,NG3PR95, NG2DSF5
USE YOMNCL   , ONLY : LNCLIN
USE YOELWRAD , ONLY : RADLWM, NLWFR
USE YOEGWDWMS, ONLY : TGWDWMS, LREGWWMS
USE YOMCT0   , ONLY : NSTOP
USE YOMCUMFS , ONLY : LECUMFS2
USE YOEGWWMS , ONLY : GTPHYGWWMS
USE YOEPHY   , ONLY : LEMWAVE
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS   ,RSTATI
USE YOMMWAVE , ONLY : MWAVE_PHYS_TYPE, MWAVE_RAD_TYPE, LEMWAVE_ACTIVE, N_SATSENS
USE PARMWAVE , ONLY : NMAX_CHAN
USE YOM_YGFL , ONLY : YGFL, YAERO, NAERO,&
 & YGHG, NGHG, YGRG, NGRG, LFIRE, LCH4SFC, YLRCH4, NTRAC
USE TYPE_GEMS_PROFILES, ONLY : NGEMS
USE YOMECEARTH , ONLY : LECEARTH

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KSMAX
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
LOGICAL           ,INTENT(IN)    :: LDRAIN1D
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL5(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL5(KLON,KLEV,YGFL%NDIM)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRS15(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSF15(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMTD5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTRSO5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0M(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSA5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSA5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTIA5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PASN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSN5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHSTD(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHSDFOR(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGAMMA(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTHETA(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSIG(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALBF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALUVP(KLON), PALUVD(KLON), PALNIP(KLON), PALNID(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMISF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2O(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2B(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2A(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2F(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH4(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH4F(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSF6(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLFLX(KLON,NGRG+NTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVH(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTVL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTVH(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAILC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAIHC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOTY(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALUVP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALUVD(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALNIP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALNID(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCI(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSST(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KLON), PCLON(KLON), PSLON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0MCL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLZ0HCL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLZ0H5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV5(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENGFL5(KLON,KLEV,YGFL%NDIM1)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNTOP(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNBAS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PACPR5(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSTRTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSTRTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAHFSTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVAPTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N5(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N5(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PL(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PI(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PA(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVERVEL(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(KLON,KLEV,YGFL%NDIM)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRS1(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRSF1(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRSF(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSA(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWSA(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENGFL(KLON,KLEV,YGFL%NDIM1)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PACPR(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSTRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSTRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAHFSTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVAPTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTQ(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTU(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTV(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLZ0H(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTLE1(KLON) 

!     Common arrays for rain/cloud MWave assimilation
TYPE (MWAVE_RAD_TYPE), INTENT (INOUT) :: YD_MWAVE_RAD (KLON, N_SATSENS)
!*
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: Z_PVDISG5(KLON), Z_PUSTRG5(KLON),Z_PVSTRG5(KLON)

REAL(KIND=JPRB) :: Z_PVDIS (KLON)
REAL(KIND=JPRB) :: Z_PVDISG(KLON), Z_PUSTRG(KLON),Z_PVSTRG(KLON)

REAL(KIND=JPRB) :: Z_PDIFTQ5(KLON,0:KLEV),Z_PDIFTS5(KLON,0:KLEV)

REAL(KIND=JPRB) :: Z_PACPR15(KLON)
REAL(KIND=JPRB) :: Z_PALB5(KLON),Z_PEMIS5(KLON)

REAL(KIND=JPRB) :: Z_PFTG125(KLON),&
 & Z_PFWEV5(KLON) ,&
 & Z_PFWG125(KLON),Z_PFWMLT5(KLON),&
 & Z_PFWROD5(KLON),Z_PFWRO15(KLON)  

REAL(KIND=JPRB) :: Z_PFTG12(KLON),Z_PFTLHEV(KLON),Z_PFTLHSB(KLON),&
 & Z_PFWEV(KLON) ,&
 & Z_PFWG12(KLON),Z_PFWMLT(KLON),Z_PFWSB(KLON),&
 & Z_PFWROD(KLON),Z_PFWRO1(KLON)  

REAL(KIND=JPRB) :: Z_PSNSE15(KLON),Z_PWLE15(KLON),Z_PTLE15(KLON)
REAL(KIND=JPRB) :: Z_PTSAE15(KLON,KLEVS),Z_PWSAE15(KLON,KLEVS)

REAL(KIND=JPRB) :: Z_PSNSE1(KLON),Z_PWLE1(KLON)
REAL(KIND=JPRB) :: Z_PTSAE1(KLON,KLEVS),Z_PWSAE1(KLON,KLEVS)

REAL(KIND=JPRB) :: Z_PDIFCQ5(KLON,0:KLEV),&
 & Z_PDIFCS5(KLON,0:KLEV),&
 & Z_PFCQNG5(KLON,0:KLEV),Z_PFPLCL5(KLON,0:KLEV),&
 & Z_PFPLCN5(KLON,0:KLEV),&
 & Z_PFPLSL5(KLON,0:KLEV),Z_PFPLSN5(KLON,0:KLEV),&
 & Z_PFHPCL5(KLON,0:KLEV),&
 & Z_PFHPCN5(KLON,0:KLEV),Z_PFHPSL5(KLON,0:KLEV),&
 & Z_PFHPSN5(KLON,0:KLEV),&
 & Z_PFRSO5 (KLON,0:KLEV),Z_PFRTH5 (KLON,0:KLEV),&
 & Z_PSRSWDCS5(KLON),Z_PSRSWD5(KLON),&
 & Z_PFRSOD5(KLON)       ,Z_PFRTHD5(KLON),&
 & Z_PSTRCU5(KLON,0:KLEV),&
 & Z_PSTRCV5(KLON,0:KLEV),Z_PSTRDU5(KLON,0:KLEV),&
 & Z_PSTRDV5(KLON,0:KLEV),&
 & Z_PSTRTU5(KLON,0:KLEV),Z_PSTRTV5(KLON,0:KLEV)  
REAL(KIND=JPRB) :: Z_PFCCQN5(KLON,0:KLEV), Z_PFCCQL5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZFLXWGWU5(KLON,0:KLEV), ZFLXWGWV5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTENU5(KLON,KLEV), ZTENV5(KLON,KLEV), ZTFIELD5(KLON,KLEV)

REAL(KIND=JPRB) :: Z_PDIFCQ(KLON,0:KLEV),&
 & Z_PDIFCS(KLON,0:KLEV),&
 & Z_PFCQNG(KLON,0:KLEV),Z_PFPLCL(KLON,0:KLEV),Z_PFPLCN(KLON,0:KLEV),&
 & Z_PFPLSL(KLON,0:KLEV),Z_PFPLSN(KLON,0:KLEV),Z_PFHPCL(KLON,0:KLEV),&
 & Z_PFHPCN(KLON,0:KLEV),Z_PFHPSL(KLON,0:KLEV),Z_PFHPSN(KLON,0:KLEV),&
 & Z_PFRSO (KLON,0:KLEV),Z_PFRTH (KLON,0:KLEV),Z_PSRSWD(KLON),&
 & Z_PFRSOD(KLON)       ,Z_PFRTHD(KLON),&
 & Z_PSTRCU(KLON,0:KLEV),&
 & Z_PSTRCV(KLON,0:KLEV),Z_PSTRDU(KLON,0:KLEV),Z_PSTRDV(KLON,0:KLEV)  
REAL(KIND=JPRB) :: Z_PFCCQN(KLON,0:KLEV), Z_PFCCQL(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZFLXWGWU(KLON,0:KLEV), ZFLXWGWV(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTENU(KLON,KLEV), ZTENV(KLON,KLEV), ZTFIELD(KLON,KLEV)

REAL(KIND=JPRB) :: Z_PNEB5(KLON,KLEV), Z_PQLI5(KLON,KLEV),&
 & Z_PQSAT5(KLON,KLEV),Z_PQICE5(KLON,KLEV),Z_PTCC5 (KLON)  
REAL(KIND=JPRB) :: Z_PFRSOC5(KLON,0:1),Z_PFRTHC5(KLON,0:1)

REAL(KIND=JPRB) :: Z_PNEB(KLON,KLEV), Z_PQLI(KLON,KLEV),&
 & Z_PQSAT(KLON,KLEV),Z_PQICE(KLON,KLEV),Z_PTCC  (KLON)  
REAL(KIND=JPRB) :: Z_PFRSOC(KLON,0:1),Z_PFRTHC(KLON,0:1)

REAL(KIND=JPRB) :: Z_PEMTD (KLON,0:KLEV),Z_PTRSO (KLON,0:KLEV)
!            0.2  LOCAL ARRAYS FOR ECMWF PHYSICS PACKAGE
!                 --------------------------------------

INTEGER(KIND=JPIM) :: ITOPC(KLON), IBASC(KLON) 
INTEGER(KIND=JPIM) :: ITOPC1(KLON), IBASC1(KLON)
INTEGER(KIND=JPIM) :: ITYPE(KLON), ICBOT(KLON), ICTOP(KLON), IBOTSC(KLON)
INTEGER(KIND=JPIM) :: ID0, IM0, IY0, IDINCR,ISEC,IDD,IMM,IYY,ILMON(12)

LOGICAL :: LLLAND(KLON),LLSICE(KLON),LLNH(KLON),LLCUM(KLON)
LOGICAL :: LLSC(KLON)
LOGICAL :: LLLAKE(KLON)
LOGICAL :: LLOCN_KPP(KLON)
!--to store variable LAI
REAL(KIND=JPRB) :: ZLAIL(KLON),ZLAIH(KLON)

REAL(KIND=JPRB) :: ZALBD5(KLON,NTSW), ZALBP5(KLON,NTSW)
REAL(KIND=JPRB) :: ZAZ0H5(KLON)  , ZAZ0M5 (KLON) , ZHSTD(KLON), ZHSDFOR(KLON)
REAL(KIND=JPRB) :: ZSNM1M5(KLON) , ZRSNM1M5(KLON)
REAL(KIND=JPRB) :: ZWLM1M5(KLON)
REAL(KIND=JPRB) :: ZGEOM15(KLON,KLEV), ZGEOMH5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZLU5(KLON,KLEV)   , ZLUDE5(KLON,KLEV), ZMFU5(KLON,KLEV)
REAL(KIND=JPRB) :: ZMFD5(KLON,KLEV)  
REAL(KIND=JPRB) :: ZPSTRTU5(KLON,0:KLEV),ZPSTRTV5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTRSOD5(KLON)
REAL(KIND=JPRB) :: ZCEMTR5(KLON,0:1) , ZCTRSO5(KLON,0:1)
REAL(KIND=JPRB) :: ZEMIR5(KLON),       ZEMIW5(KLON)
REAL(KIND=JPRB) :: ZQRAIN5(KLON,KLEV), ZRAINT5(KLON,KLEV)
REAL(KIND=JPRB) :: ZSUDU5(KLON)
REAL(KIND=JPRB) :: ZLITOT5(KLON,KLEV) 
REAL(KIND=JPRB) :: ZDSRP5(KLON), ZISUND5(KLON)
REAL(KIND=JPRB) :: ZPA5(KLON,KLEV), ZPL5(KLON,KLEV), ZPI5(KLON,KLEV)
REAL(KIND=JPRB) :: ZPA55(KLON,KLEV), ZPL55(KLON,KLEV), ZPI55(KLON,KLEV)
REAL(KIND=JPRB) :: ZCOVPTOT5(KLON,KLEV)

!---- added for convenience                    
REAL(KIND=JPRB) :: ZTHT5(KLON,KLEV+1)

REAL(KIND=JPRB) :: ZALBD(KLON,NTSW), ZALBP(KLON,NTSW)
REAL(KIND=JPRB) :: ZAZ0H(KLON)  , ZAZ0M (KLON)
REAL(KIND=JPRB) :: ZCTLWP(KLON)
REAL(KIND=JPRB) :: ZSNM1M(KLON)
REAL(KIND=JPRB) :: ZWLM1M(KLON)
REAL(KIND=JPRB) :: ZWLMX (KLON)
REAL(KIND=JPRB) :: ZGEOM1(KLON,KLEV), ZGEOMH(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZLU(KLON,KLEV)   , ZLUDE(KLON,KLEV), ZMFU(KLON,KLEV)
REAL(KIND=JPRB) :: ZMFD(KLON,KLEV)  , ZHRSW(KLON,KLEV), ZHRLW(KLON,KLEV)
REAL(KIND=JPRB) :: ZTRSOD(KLON)
REAL(KIND=JPRB) :: ZT(KLON,KLEV)
REAL(KIND=JPRB) :: ZQRAIN(KLON,KLEV), ZRAINT(KLON,KLEV)
REAL(KIND=JPRB) :: ZCEMTR(KLON,0:1) , ZCTRSO(KLON,0:1)
REAL(KIND=JPRB) :: ZEMIR(KLON),       ZEMIW(KLON)
REAL(KIND=JPRB) :: ZCHAR(KLON)
REAL(KIND=JPRB) :: ZTSKRAD5(KLON)
REAL(KIND=JPRB) :: ZTSKRAD(KLON)
REAL(KIND=JPRB) :: ZPA(KLON,KLEV), ZPL(KLON,KLEV), ZPI(KLON,KLEV)

!---- added for convenience                    
REAL(KIND=JPRB) :: ZTHT(KLON,KLEV+1), ZEMIT(KLON)
REAL(KIND=JPRB) :: ZCCC5(KLON) , ZHCC5(KLON)  , ZLCC5(KLON)  , ZMCC5(KLON)

REAL(KIND=JPRB) :: ZFRTI(KLON,KTILES),ZALBTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZALBTI(KLON,KTILES)
INTEGER(KIND=JPIM) :: ITVL(KLON),ITVH(KLON),ISOTY(KLON)
REAL(KIND=JPRB) ::    ZCVL(KLON),ZCVH(KLON)

REAL(KIND=JPRB) ::    ZCCNL(KLON),ZCCNO(KLON)
REAL(KIND=JPRB) ::    ZWND5(KLON),ZCCNL5(KLON),ZCCNO5(KLON)

REAL(KIND=JPRB) :: ZUCFL5 (KLON), ZVCFL5 (KLON), ZTCFL5 (KLON), ZQCFL5 (KLON), ZDCFL5 (KLON)
REAL(KIND=JPRB) :: ZUCFL  (KLON), ZVCFL  (KLON), ZTCFL  (KLON), ZQCFL  (KLON), ZDCFL  (KLON)

REAL(KIND=JPRB) :: ZLRCH4(KLON,KLEV)

REAL(KIND=JPRB) :: ZCOVPTOT(KLON,KLEV)

!          EXTRA LOCAL STORAGE FOR TRAJECTORY
!          ----- ----- ------- --- ----------

REAL(KIND=JPRB) :: ZTLE55   (KLON) , ZAZ0M55(KLON),&
 & ZAZ0H55  (KLON) , ZLZ0H55(KLON)   
REAL(KIND=JPRB) :: ZTENT5(KLON,KLEV),ZTENQ5(KLON,KLEV)
REAL(KIND=JPRB) :: ZTENT55A (KLON,KLEV),ZTENQ55A (KLON,KLEV),&
 & ZTENU55A (KLON,KLEV),ZTENV55A (KLON,KLEV)     
REAL(KIND=JPRB) :: ZTENT55B (KLON,KLEV),ZTENQ55B (KLON,KLEV),&
 & ZTENU55B (KLON,KLEV),ZTENV55B (KLON,KLEV)     
REAL(KIND=JPRB) :: ZTENT55C (KLON,KLEV),ZTENQ55C (KLON,KLEV),&
 & ZTENU55C (KLON,KLEV),ZTENV55C (KLON,KLEV)     
REAL(KIND=JPRB) :: ZTENT55D (KLON,KLEV),ZTENQ55D (KLON,KLEV),&
 & ZTENU55D (KLON,KLEV),ZTENV55D (KLON,KLEV)     
REAL(KIND=JPRB) :: ZTENT55E (KLON,KLEV),ZTENQ55E (KLON,KLEV),&
 & ZTENU55E (KLON,KLEV),ZTENV55E (KLON,KLEV)     
REAL(KIND=JPRB) :: ZTENT55F (KLON,KLEV),ZTENQ55F (KLON,KLEV),&
 & ZTENU55F (KLON,KLEV),ZTENV55F (KLON,KLEV)  
REAL(KIND=JPRB) :: ZTENT55G (KLON,KLEV),&
 & ZTENU55G (KLON,KLEV),ZTENV55G (KLON,KLEV)
REAL(KIND=JPRB) :: ZUSTRTI55(KLON,KTILES),ZVSTRTI55(KLON,KTILES)
REAL(KIND=JPRB) :: ZAHFSTI55(KLON,KTILES),ZEVAPTI55(KLON,KTILES),&
 & ZTSKTI55(KLON,KTILES)  
REAL(KIND=JPRB) :: ZPEMTD5(KLON,0:KLEV)

 REAL(KIND=JPRB) :: ZTENTX5 (KLON,KLEV),ZTENQX5 (KLON,KLEV),&
 & ZTENUX5 (KLON,KLEV),ZTENVX5 (KLON,KLEV)  
REAL(KIND=JPRB) :: ZTENTVD5 (KLON,KLEV),ZTENQVD5 (KLON,KLEV),&
 & ZTENUVD5 (KLON,KLEV),ZTENVVD5 (KLON,KLEV) 
REAL(KIND=JPRB) :: ZTENTGW5 (KLON,KLEV),&
 & ZTENUGW5 (KLON,KLEV),ZTENVGW5 (KLON,KLEV)  
REAL(KIND=JPRB) :: ZTENTCO5 (KLON,KLEV),ZTENQCO5 (KLON,KLEV),&
 & ZTENUCO5 (KLON,KLEV),ZTENVCO5 (KLON,KLEV) 
REAL(KIND=JPRB) :: ZTENTCL5 (KLON,KLEV)
REAL(KIND=JPRB) :: ZTENUWM5 (KLON,KLEV),ZTENVWM5 (KLON,KLEV)

INTEGER(KIND=JPIM) :: IFLAG, JK, JL, JSW, IFIELDS, ITRC
INTEGER(KIND=JPIM) :: IBLK
INTEGER(KIND=JPIM) :: ISTEP, IFREQWMS, ISTEPWMS

LOGICAL :: LLSLPHY

!*     Local arrays for rain/cloud MWave assimilation
REAL (KIND=JPRB), TARGET :: ZPHYS_P_FW    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PH_FW   (KLON,KLEV+1)
REAL (KIND=JPRB), TARGET :: ZPHYS_T_FW    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_Q_FW    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CC_FW   (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CWC_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CIC_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_RFL_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_SFL_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CRFL_FW (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CSFL_FW (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PFRC_FW (KLON,KLEV) 
    
REAL (KIND=JPRB), TARGET :: ZPHYS_P_AD    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PH_AD   (KLON,KLEV+1)
REAL (KIND=JPRB), TARGET :: ZPHYS_T_AD    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_Q_AD    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CC_AD   (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CWC_AD  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CIC_AD  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_RFL_AD  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_SFL_AD  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CRFL_AD (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CSFL_AD (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PFRC_AD (KLON,KLEV)
    
TYPE (MWAVE_PHYS_TYPE) :: YL_MWAVE_PHYS_FW (KLON)
TYPE (MWAVE_PHYS_TYPE) :: YL_MWAVE_PHYS_AD (KLON)

REAL(KIND=JPRB) :: ZCARDI,ZRCHAR,ZRCPD,ZTSGWD

CHARACTER (LEN = 1) ::  CL_CDCONF

!Dummy arrays for convection
REAL(KIND=JPRB) :: zmfude_rate(klon,klev)     ! UD detrainmnet rate
REAL(KIND=JPRB) :: zmfdde_rate(klon,klev)     ! DD detrainmnet rate
REAL(KIND=JPRB) :: zcape(klon)                ! CAPE (J/kg)
REAL(KIND=JPRB) :: zwmean(klon)               ! vertically averaged updraught speed (M/S)

! FLAKE: Lake model (Local for PASSIVE introduction)
REAL(KIND=JPRB) :: ZCLAKE5(KLON)
REAL(KIND=JPRB) :: ZHLICE5(KLON) 
REAL(KIND=JPRB) :: ZTLICE5 (KLON)
REAL(KIND=JPRB) :: ZTLWML5 (KLON)

!----------------------------------------------------------------------------
! For convective/diffusive tracer Transport:

INTEGER(KIND=JPIM)           :: ITRAC
INTEGER(KIND=JPIM)           :: IAERO(NAERO) ! Index array for aerosol tracers
INTEGER(KIND=JPIM)           :: IGRG(NGRG)   ! Index array for reactive gas tracers
INTEGER(KIND=JPIM)           :: ITRACER(NTRAC)   ! Index array for flexible tracers 
REAL(KIND=JPRB), POINTER :: ZCEN5(:,:,:),ZTENC5(:,:,:),ZCEN(:,:,:) ,ZTENC (:,:,:),&
                           &ZTENC5_SKF(:,:,:), ZSCAV5(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZTENC55D(:,:,:),ZTENC55B(:,:,:)
REAL(KIND=JPRB), POINTER :: ZCFLX5(:,:), ZCFLX(:,:)
!!REAL(KIND=JPRB), ALLOCATABLE :: ZTENCX5(:,:,:)
!!REAL(KIND=JPRB), ALLOCATABLE :: ZTENCVD5(:,:,:)
! for GRG in coupled mode
REAL(KIND=JPRB), POINTER :: ZGRGTEND(:,:,:)

!----------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZALBICE(KLON)
REAL(KIND=JPRB) :: ZTHKICE(KLON)
REAL(KIND=JPRB) :: ZSNTICE(KLON)

!Dummy arrays for vdfmain
LOGICAL :: LLSHCV(KLON)

LOGICAL :: LLCLDCOVER

REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "surfrad.h"
#include "surfbc.h"
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cldpp.intfb.h"
#include "cldppad.intfb.h"
#include "cloud.intfb.h"
#include "cloudad.intfb.h"
#include "cloudst.intfb.h"
#include "cloudstad.intfb.h"
#include "cond.intfb.h"
#include "condad.intfb.h"
#include "cucalln.intfb.h"
#include "cucallnad.intfb.h"
#include "cucalln2.intfb.h"
#include "cucalln2ad.intfb.h"
#include "gems_dealloc.intfb.h"
#include "gems_dealloc_ad.intfb.h"
#include "gems_init.intfb.h"
#include "gems_init_ad.intfb.h"
#include "gems_tend.intfb.h"
#include "gems_tend_ad.intfb.h"
#include "gwdrags.intfb.h"
#include "gwdragad.intfb.h"
!#include "icestateoasis3.intfb.h"
!#include "gwdrag_wmss.intfb.h"
#include "gwdrag_wmssad.intfb.h"
#include "radheat.intfb.h"
#include "radheatad.intfb.h"
#include "radinaad.intfb.h"
#include "rdphtrajt.intfb.h"
#include "satur.intfb.h"
#include "saturad.intfb.h"
#include "updcal.intfb.h"
#include "vdfmains.intfb.h"
#include "vdfmainsad.intfb.h"
#include "mwave_obsop_ad.intfb.h"

!     ------------------------------------------------------------------
#include "fcttre.h"
#include "fcttim.h"

!-----------------------------------------------------------------------

!*                  ----  TRAJECTORY COMPUTATIONS  ----

!-----------------------------------------------------------------------

!*         1.     INITIAL COMPUTATIONS.
!                 ---------------------

IF (LHOOK) CALL DR_HOOK('CALLPARAD',0,ZHOOK_HANDLE)

! Set up constants required
ZRCPD=1.0_JPRB/RCPD

! Initialize Ocean and Lake variable for passive introduction  
LLOCN_KPP(:)=.FALSE.
LLLAKE(:)=.FALSE.
ZCLAKE5(:)=0.0_JPRB
ZHLICE5(:)=0.0_JPRB
ZTLICE5(:)=0.0_JPRB
ZTLWML5(:)=0.0_JPRB

!*  Rain/cloud MWave assimilation
IF (LEMWAVE_ACTIVE) THEN
    DO JL = KIDIA, KFDIA
       YL_MWAVE_PHYS_FW (JL) % NLEVELS   = KLEV    
       
       YL_MWAVE_PHYS_FW (JL) % P   => ZPHYS_P_FW   (JL,:)
       YL_MWAVE_PHYS_FW (JL) % PH  => ZPHYS_PH_FW  (JL,:)
       YL_MWAVE_PHYS_FW (JL) % T   => ZPHYS_T_FW   (JL,:)
       YL_MWAVE_PHYS_FW (JL) % Q   => ZPHYS_Q_FW   (JL,:)
       YL_MWAVE_PHYS_FW (JL) % RFL => ZPHYS_RFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % SFL => ZPHYS_SFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CRFL=> ZPHYS_CRFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CSFL=> ZPHYS_CSFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CWC => ZPHYS_CWC_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CIC => ZPHYS_CIC_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CC  => ZPHYS_CC_FW  (JL,:)
       YL_MWAVE_PHYS_FW (JL) % PFRC=> ZPHYS_PFRC_FW(JL,:)
       
       YL_MWAVE_PHYS_AD (JL) % NLEVELS   = KLEV   
       
       YL_MWAVE_PHYS_AD (JL) % P   => ZPHYS_P_AD   (JL,:)
       YL_MWAVE_PHYS_AD (JL) % PH  => ZPHYS_PH_AD  (JL,:)
       YL_MWAVE_PHYS_AD (JL) % T   => ZPHYS_T_AD   (JL,:)
       YL_MWAVE_PHYS_AD (JL) % Q   => ZPHYS_Q_AD   (JL,:)
       YL_MWAVE_PHYS_AD (JL) % RFL => ZPHYS_RFL_AD (JL,:)
       YL_MWAVE_PHYS_AD (JL) % SFL => ZPHYS_SFL_AD (JL,:)
       YL_MWAVE_PHYS_AD (JL) % CRFL=> ZPHYS_CRFL_AD (JL,:)
       YL_MWAVE_PHYS_AD (JL) % CSFL=> ZPHYS_CSFL_AD (JL,:)
       YL_MWAVE_PHYS_AD (JL) % CWC => ZPHYS_CWC_AD (JL,:)
       YL_MWAVE_PHYS_AD (JL) % CIC => ZPHYS_CIC_AD (JL,:)
       YL_MWAVE_PHYS_AD (JL) % CC  => ZPHYS_CC_AD  (JL,:)
       YL_MWAVE_PHYS_AD (JL) % PFRC=> ZPHYS_PFRC_AD(JL,:)
    ENDDO
ENDIF

ITRAC=0
IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_INIT( KIDIA, KFDIA, KLEV, KLON, ITRAC, PCO2O, PCO2B, PCO2A, PCO2F, &
     &  PCH4, PCH4F, PSF6, PGFLFLX, PGEMU, PLSM, IGRG, ITRACER, IAERO, ZLRCH4, PGFL5, &
     &  PTENGFL5, ZCEN5, ZTENC5, ZTENC5_SKF, ZCFLX5, ZSCAV5, ZGRGTEND)
ELSE
! ALLOCATE arrays so they can be passed to VDFOUTER etc. in the non-GEMS case
   ALLOCATE(ZCEN5(1,1,1))
   ALLOCATE(ZTENC5(1,1,1))
   ALLOCATE(ZCFLX5(1,1))
   ALLOCATE(ZSCAV5(1))
ENDIF



IF (LTRACLNPH .AND. NGEMS > 0) THEN
  ALLOCATE( ZTENC55B(KLON,KLEV,ITRAC) )
  ALLOCATE( ZTENC55D(KLON,KLEV,ITRAC) )  
  ZTENC55B(:,:,:)  =0.0_JPRB
  ZTENC55D(:,:,:) =0.0_JPRB
ENDIF

CL_CDCONF='A'

CALL SURF_INQ(PRCHAR=ZRCHAR)

ZCARDI = 0.0_JPRB
ZCHAR(:)=ZRCHAR
ZQRAIN5 (:,:) = 0.0_JPRB
ZRAINT5 (:,:) = 0.0_JPRB
ZLITOT5(:,:)=0.0_JPRB
LLSLPHY=.FALSE.

ZPA55(:,:) = 0.0_JPRB
ZPL55(:,:) = 0.0_JPRB
ZPI55(:,:) = 0.0_JPRB

!*       1.1   FIRST TIME-STEP
IF (KSTEP == KSTART) THEN
  DO JL=KIDIA,KFDIA
    PNTOP(JL)=1.0_JPRB
    PNBAS(JL)=1.0_JPRB
    PACPR5(JL)=0.0_JPRB

    PGZ0M5(JL)=PGZ0MCL(JL)
    PLZ0H5(JL)=PLZ0HCL(JL)
  ENDDO
ENDIF

!*       1.2   CHANGE UNITS/SCALINGS

DO JL=KIDIA,KFDIA
  ZAZ0M5(JL)  = PGZ0M5(JL) /RG
  ZAZ0H5(JL)  = EXP(PLZ0H5(JL))
  ZLZ0H55(JL) = PLZ0H5(JL)
  ZSNM1M5(JL) = PSNS5(JL)
  ZRSNM1M5(JL)= PRSN5(JL)
  ZWLM1M5(JL) = PWL5 (JL)
  IF (PLSM(JL) > 0.5_JPRB) THEN
    ZHSTD (JL) = PHSTD(JL) /RG
    ZHSDFOR(JL) = PHSDFOR(JL)  
  ELSE
    ZHSTD (JL) = 0._JPRB
    PSIG  (JL) = 0._JPRB
    ZHSDFOR(JL) = 0._JPRB 
  ENDIF
ENDDO

!*      1.3   GLOBAL AUXILLIARY VARIABLES

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZGEOM15(JL,JK)  = PAPHIF5(JL,JK)-PAPHI5(JL,KLEV)
  ENDDO
ENDDO
DO JK=0,KLEV
  DO JL=KIDIA,KFDIA
    ZGEOMH5(JL,JK)=PAPHI5(JL,JK)-PAPHI5(JL,KLEV)
  ENDDO
ENDDO

!*      1.3a   AUXILLIARY VARIABLES FOR VDF, SRF AND SURFRAD

CALL SURFBC (KIDIA=KIDIA,KFDIA=KFDIA,KLON=KLON,KTILES=KTILES,&
 & PTVL=PTVL,PTVH=PTVH,PSOTY=PSOTY,&
 & PSOALUVP=PSOALUVP,PSOALUVD=PSOALUVD,PSOALNIP=PSOALNIP,PSOALNID=PSOALNID,&
 & PSDOR=ZHSTD,PCVLC=PCVL,PCVHC=PCVH,&
 & PLAILC=PLAILC,PLAIHC=PLAIHC,&
 & PLSM=PLSM,PCI=PCI,PCLAKE=ZCLAKE5,PHLICE=ZHLICE5,&
 & PGEMU=PGEMU,PSNM1M=ZSNM1M5,PWLM1M=ZWLM1M5,PRSNM1M=ZRSNM1M5,&
 & LDLAND=LLLAND,LDSICE=LLSICE,LDLAKE=LLLAKE,LDNH=LLNH,LDOCN_KPP=LLOCN_KPP,&
 & KTVL=ITVL,KTVH=ITVH,KSOTY=ISOTY,&
 & PCVL=ZCVL,PCVH=ZCVH,PLAIL=ZLAIL,PLAIH=ZLAIH,&
 & PWLMX=ZWLMX,PFRTI=ZFRTI,&
 & PALBF=PALBF,PALUVP=PALUVP,PALUVD=PALUVD,PALNIP=PALNIP,PALNID=PALNID)

DO JL=KIDIA,KFDIA
  ITOPC(JL)=INT(PNTOP(JL)+0.01_JPRB)
  IBASC(JL)=INT(PNBAS(JL)+0.01_JPRB)
ENDDO

!* SEA-ICE FOR EC-EARTH
ZALBICE(KIDIA:KFDIA) = 0.0_JPRB
ZTHKICE(KIDIA:KFDIA) = 0.0_JPRB
ZSNTICE(KIDIA:KFDIA) = 0.0_JPRB
!IF (LECEARTH) THEN
!   CALL ICESTATEOASIS3(KSTGLO,KIDIA,KFDIA, &
!   &  PALBICE=ZALBICE(KIDIA:KFDIA),PTHKICE=ZTHKICE(KIDIA:KFDIA),PSNTICE=ZSNTICE(KIDIA:KFDIA))
!END IF


!*       1.4  SET SURFACE TENDENCIES TO ZERO
DO JK=1,KLEVS
  DO JL=KIDIA,KFDIA
    Z_PTSAE15(JL,JK)=0.0_JPRB
    Z_PWSAE15(JL,JK)=0.0_JPRB
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  Z_PTLE15(JL)=0.0_JPRB
  Z_PWLE15(JL)=0.0_JPRB
  Z_PSNSE15(JL)=0.0_JPRB
ENDDO

!*       1.5  SET FLUXES TO ZERO

! radiation

IF (.NOT. LERADI2) THEN
  Z_PFRSOC5(:,:) = 0.0_JPRB
  Z_PFRTHC5(:,:) = 0.0_JPRB
  Z_PFRSOD5(:)   = 0.0_JPRB
  Z_PFRTHD5(:)   = 0.0_JPRB
  Z_PFRSO5 (:,:) = 0.0_JPRB
! Set surface solar flux to a minimum value for stability in vertical diffusion
  Z_PFRSO5 (:,KLEV) = 1.0E-15_JPRB
  Z_PFRTH5 (:,:) = 0.0_JPRB
ENDIF

! vertical diffusion

IF (.NOT.LEVDIF2) THEN
  Z_PDIFTS5(:,:) = 0.0_JPRB
  Z_PDIFTQ5(:,:) = 0.0_JPRB
  Z_PSTRTU5(:,:) = 0.0_JPRB
  Z_PSTRTV5(:,:) = 0.0_JPRB
  Z_PDIFTS5(:,:) = 0.0_JPRB
  Z_PDIFTQ5(:,:) = 0.0_JPRB
ENDIF

!gravity wave drag

IF (.NOT. LEGWDG2) THEN
  Z_PUSTRG5(:)   = 0.0_JPRB
  Z_PVSTRG5(:)   = 0.0_JPRB
  Z_PVDISG5(:)   = 0.0_JPRB
  Z_PSTRDU5(:,:) = 0.0_JPRB
  Z_PSTRDV5(:,:) = 0.0_JPRB
ENDIF

!convection

IF (LECUMF2) THEN
  ZPSTRTU5(:,:) = 0.0_JPRB
  ZPSTRTV5(:,:) = 0.0_JPRB
  zmfude_rate(:,:) = 0.0_JPRB
  zmfdde_rate(:,:) = 0.0_JPRB
ENDIF
IF (.NOT.LECUMF2) THEN
  Z_PDIFCS5(:,:) = 0.0_JPRB
  Z_PDIFCQ5(:,:) = 0.0_JPRB
  Z_PFHPCL5(:,:) = 0.0_JPRB
  Z_PFHPCN5(:,:) = 0.0_JPRB
  Z_PFPLCL5(:,:) = 0.0_JPRB
  Z_PFPLCN5(:,:) = 0.0_JPRB
  Z_PSTRCU5(:,:) = 0.0_JPRB
  Z_PSTRCV5(:,:) = 0.0_JPRB
  Z_PFCCQL5(:,:) = 0.0_JPRB
  Z_PFCCQN5(:,:) = 0.0_JPRB
  ZLUDE5 (:,:) = 0.0_JPRB
  ZLU5   (:,:) = 0.0_JPRB
ENDIF

!large scale precipitation

IF (.NOT.LECOND2 .AND. .NOT.LENCLD2) THEN
  Z_PFHPSL5(:,:) = 0.0_JPRB
  Z_PFHPSN5(:,:) = 0.0_JPRB
  Z_PFPLSL5(:,:) = 0.0_JPRB
  Z_PFPLSN5(:,:) = 0.0_JPRB
  ZCOVPTOT5(:,:) = 0.0_JPRB
ENDIF

!cloudiness

IF (.NOT.LEDCLD2 .AND. .NOT.LENCLD2) THEN
  Z_PNEB5(:,:)  = 0.0_JPRB
  Z_PQLI5(:,:)  = 0.0_JPRB
  Z_PQICE5(:,:) = 0.0_JPRB
  Z_PTCC5(:)    = 0.0_JPRB
ENDIF

IF (.NOT.LEGWWMS2) THEN
  ZFLXWGWU5(:,:) = 0.0_JPRB
  ZFLXWGWV5(:,:) = 0.0_JPRB
ENDIF

!*       1.6  SET SURFACE SOLAR TRANSMISSIVITY

Z_PSRSWDCS5(:) = 0.0_JPRB
Z_PSRSWD5(:) = 0.0_JPRB
DO JL=KIDIA,KFDIA
  ZSUDU5 (JL) = Z_PSRSWDCS5(JL)
  ZTRSOD5(JL) = Z_PSRSWD5(JL)
ENDDO

!*       1.7 SET PHYSICAL TENDENCIES

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZTENTX5(JL,JK) = PTENT5(JL,JK)
    ZTENQX5(JL,JK) = PTENQ5(JL,JK)
    ZTENUX5(JL,JK) = PTENU5(JL,JK)
    ZTENVX5(JL,JK) = PTENV5(JL,JK)
  ENDDO
ENDDO

!*       1.8 READ TRAJECTORY FOR PHYS.TENDENCIES AND RADIATION

IF ((LERADN2.OR.LERADSW2) .OR. LENCLD2 .OR. LETRAJPT) THEN
  IFIELDS = NG3DT95*KLEV + NG3PR95*(KLEV+1) + NG2DSF5
  CALL RDPHTRAJT(KSTEP,KIDIA,KFDIA,KSTGLO,IFIELDS,&
   & ZTENTVD5,ZTENQVD5,ZTENUVD5,ZTENVVD5,&
   & ZTENTGW5,ZTENUGW5,ZTENVGW5,&
   & ZTENTCO5,ZTENQCO5,ZTENUCO5,ZTENVCO5,&
   & ZTENTCL5,ZTENUWM5,ZTENVWM5,&
   & Z_PDIFTQ5 ,Z_PDIFTS5 ,ZPEMTD5 ,PTRSO5,&
   & ZPA55, ZPL55, ZPI55,&
   & ZLUDE5, ZLU5,&
   & Z_PFPLCL5, Z_PFPLCN5,ZUCFL5,ZVCFL5,ZTCFL5,ZQCFL5 )  
ENDIF

!     ------------------------------------------------------------------

!*         2.     RADIATION TRANSFER
!                 ------------------

IF ( LERADI2 ) THEN

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENT55A(JL,JK) = ZTENTX5(JL,JK)
      ZTENQ55A(JL,JK) = ZTENQX5(JL,JK)
      ZTENU55A(JL,JK) = ZTENUX5(JL,JK)
      ZTENV55A(JL,JK) = ZTENVX5(JL,JK)
!     IF(LETRAJPT)ZTENTX5(JL,JK) = ZTENTRA5(JL,JK)
    ENDDO
  ENDDO

!*         2.1  FULL RADIATION COMPUTATIONS

!*         2.2 SURFACE BOUNDARY CONDITIONS
  IF (LERADS2) THEN

    DO JL=KIDIA,KFDIA
      ZWND5(JL)=SQRT(PU5(JL,KLEV)*PU5(JL,KLEV)+PV5(JL,KLEV)*PV5(JL,KLEV))
    ENDDO  

    IY0=NCCAA(NINDAT)
    IM0=NMM(NINDAT)
    ID0=NDD(NINDAT)
    IDINCR=(NSSSSS+NINT(RSTATI))/NINT(RDAY)
    ISEC=MOD(NSSSSS+NINT(RSTATI),NINT(RDAY))
    CALL UPDCAL(ID0,IM0,IY0,IDINCR,IDD,IMM,IYY,ILMON,-1)
!    CALL SURFRAD(KDD=IDD,KMM=IMM,KMON=ILMON,KSECO=ISEC,&
!     & KIDIA=KIDIA,KFDIA=KFDIA,KLON=KLON,KTILES=KTILES,KSW=NSW,&
!     & LDNH=LLNH,&
!     & PALBF=PALBF,PALBICEF=ZALBICE,PTVH=PTVH,&
!     & PALUVP=PALUVP,PALUVD=PALUVD,PALNIP=PALNIP,PALNID=PALNID,&
!     & PASN=PASN5,PMU0=PMU0M,PTS=PTL5,PWND=ZWND5,&
!     & PWS1=PWSA5(:,1),KSOTY=ISOTY,PFRTI=ZFRTI,PHLICE=ZHLICE5,PTLICE=ZTLICE5,&
!     & PALBD=ZALBD5(:,1:NSW),PALBP=ZALBP5(:,1:NSW),PALB=Z_PALB5,&
!     & PEMIR=ZEMIR5,PEMIW=ZEMIW5,PEMIT=Z_PEMIS5,&
!     & PALBTI=ZALBTI5,PCCNL=ZCCNL5,PCCNO=ZCCNO5 )  

  ELSE
    DO JSW=1,NTSW
      ZALBD5(KIDIA:KFDIA,JSW)=PALBF(KIDIA:KFDIA)
      ZALBP5(KIDIA:KFDIA,JSW)=PALBF(KIDIA:KFDIA)
    ENDDO
    DO JL=KIDIA,KFDIA
      Z_PALB5(JL)=PALBF(JL)
      Z_PEMIS5(JL)=PEMISF(JL)
      ZEMIR5(JL)=PEMISF(JL)
      ZEMIW5(JL)=PEMISF(JL)
      ZCCNL5(JL)=RCCNLND
      ZCCNO5(JL)=RCCNSEA
    ENDDO
    DO JK=1,KTILES
      DO JL=KIDIA,KFDIA
        ZALBTI5(JL,JK)=PALBF(JL)
      ENDDO
    ENDDO
  ENDIF

  IFLAG=2
  CALL SATUR  ( KIDIA  , KFDIA  , KLON  , KTDIA , KLEV,&
   & PAPRSF5, PT5    , Z_PQSAT5, IFLAG)   

!*         2.3 CLOUD
  IF (LEDCLD2 .AND. .NOT. LENCLD2) THEN

    ITOPC1(:) = ITOPC(:)
    IBASC1(:) = IBASC(:)
    Z_PACPR15(:) = PACPR5(:)

    CALL  CLOUD ( KIDIA  , KFDIA  , KLON  , KLEV,&
     & IBASC  , ITOPC,&
     & PAPRSF5, PACPR5,&
     & PQ5    , Z_PQSAT5 , PT5    , PVERVEL5,&
     & ZCCC5  , ZHCC5  , ZLCC5  , ZMCC5,&
     & Z_PNEB5  , Z_PTCC5  , Z_PQICE5 , Z_PQLI5   )  
  ELSEIF (LENCLD2) THEN
    IF (KSTEP == KSTART .AND. .NOT.LNCLIN) THEN
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZPA55(JL,JK) = 0.0_JPRB
          ZPL55(JL,JK) = 0.0_JPRB
          ZPI55(JL,JK) = 0.0_JPRB
        ENDDO
      ENDDO
    ENDIF
    ! The only part of CLDPP relevant here is the numerics 
    ! security checks
    LLCLDCOVER=.FALSE.
    CALL  CLDPP &
     & ( KIDIA , KFDIA , KLON   , KLEV,&
     & PAPRS5 , PAPRSF5 ,ZPA55  , ZPL55   , ZPI55, PT5,&
! Following line should be PGELAT,PGELAM, but not available in this 
! routine and is not relevant here anyway. 
     & PSLON, PSLON,&
     & LLCLDCOVER , ZCCC5  , ZHCC5  , ZLCC5   , ZMCC5  , Z_PTCC5,&
     & Z_PNEB5  , Z_PQICE5 , Z_PQLI5 )  
  ENDIF

  IF ((LERADN2 .AND. LERADFL2) .OR. (LERADN2 .AND. LERADLW2)) THEN
    DO JK=0,KLEV
      DO JL=KIDIA,KFDIA
        PEMTD5(JL,JK) = ZPEMTD5(JL,JK)
      ENDDO
    ENDDO
  ENDIF

  ZTSKRAD5(KIDIA:KFDIA)=PTL5(KIDIA:KFDIA)

  ZCEMTR5(:,:) = 0.0_JPRB
  ZCTRSO5(:,:) = 0.0_JPRB

  CALL RADHEAT &
   & (  KIDIA  , KFDIA  , KLON   , KLEV,&
   & PAPRS5 ,&
   & Z_PEMIS5 , PEMTD5 , PMU0,&
   & PQ5,&
   & PTENT5 , PTRSO5 , ZTRSOD5, PTL5  , TSPHY,&
   & Z_PFRSO5 , Z_PFRTH5 , Z_PFRSOD5, Z_PFRTHD5,&
   & ZCEMTR5, ZCTRSO5, Z_PFRSOC5, Z_PFRTHC5,&
   & ZSUDU5 , ZISUND5, ZDSRP5  &
   & )  

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENTX5(JL,JK) = PTENT5(JL,JK)
    ENDDO
  ENDDO

ELSE
  DO JK=1,KTILES
    DO JL=KIDIA,KFDIA
      ZALBTI5(JL,JK)=PALBF(JL)
    ENDDO
  ENDDO    
  DO JL=KIDIA,KFDIA
    Z_PEMIS5(JL)=PEMISF(JL)
  ENDDO
  ZTSKRAD5(KIDIA:KFDIA)=PTL5(KIDIA:KFDIA)
ENDIF

IF (LETRAJPT) THEN

! -----------------------------------
! using stored variables from TL     |
! -----------------------------------

!     ------------------------------------------------------------------

!*         3.A    VERTICAL EXCHANGE OF U,V,T,Q BY TURBULENCE
!                 ------------------------------------------

  DO JL=KIDIA,KFDIA
    ZTLE55(JL)=Z_PTLE15(JL)
    ZAZ0M55(JL)=ZAZ0M5(JL)
    ZAZ0H55(JL)=ZAZ0H5(JL)
  ENDDO

  DO JK=1,KTILES
    DO JL=KIDIA,KFDIA
      ZUSTRTI55(JL,JK)=PUSTRTI5(JL,JK)
      ZVSTRTI55(JL,JK)=PVSTRTI5(JL,JK)
      ZAHFSTI55(JL,JK)=PAHFSTI5(JL,JK)
      ZEVAPTI55(JL,JK)=PEVAPTI5(JL,JK)
      ZTSKTI55(JL,JK)=PTSKTI5(JL,JK)
    ENDDO
  ENDDO

  IF ( LEVDIF2 ) THEN

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENT55B(JL,JK) = ZTENTX5(JL,JK)
        ZTENQ55B(JL,JK) = ZTENQX5(JL,JK)
        ZTENU55B(JL,JK) = ZTENUX5(JL,JK)
        ZTENV55B(JL,JK) = ZTENVX5(JL,JK)

        ZTENTX5(JL,JK) = ZTENTVD5(JL,JK)
        ZTENQX5(JL,JK) = ZTENQVD5(JL,JK)
        ZTENUX5(JL,JK) = ZTENUVD5(JL,JK)
        ZTENVX5(JL,JK) = ZTENVVD5(JL,JK)
      ENDDO
    ENDDO

!LOP Tracer tendencies (not used for the moment)
!    DO ITRC=1,ITRAC
!      DO JK=1,KLEV
!        DO JL=KIDIA,KFDIA
!          ZTENC55B(JL,JK,ITRC) = ZTENCX5(JL,JK,ITRC)
!          ZTENCX5(JL,JK,ITRC) = ZTENCVD5(JL,JK,ITRC)
!        ENDDO
!      ENDDO
!    ENDDO

  ENDIF

!     ------------------------------------------------------------------

!*         4.A     GRAVITY WAVE DRAG PARAMETRIZATION
!                  ---------------------------------

  IF ( LEGWDG2 ) THEN

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENT55C(JL,JK) = ZTENTX5(JL,JK)
        ZTENQ55C(JL,JK) = ZTENQX5(JL,JK)
        ZTENU55C(JL,JK) = ZTENUX5(JL,JK)
        ZTENV55C(JL,JK) = ZTENVX5(JL,JK)

        ZTENTX5(JL,JK) = ZTENTGW5(JL,JK)
        ZTENUX5(JL,JK) = ZTENUGW5(JL,JK)
        ZTENVX5(JL,JK) = ZTENVGW5(JL,JK)
      ENDDO
    ENDDO

  ENDIF

!     ------------------------------------------------------------------

!*         5.A    CONVECTION PARAMETRIZATION
!                 --------------------------

  IF (LECUMF2) THEN

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENT55D(JL,JK) = ZTENTX5(JL,JK)
        ZTENQ55D(JL,JK) = ZTENQX5(JL,JK)
        ZTENU55D(JL,JK) = ZTENUX5(JL,JK)
        ZTENV55D(JL,JK) = ZTENVX5(JL,JK)

        ZTENTX5(JL,JK) = ZTENTCO5(JL,JK)
        ZTENQX5(JL,JK) = ZTENQCO5(JL,JK)
        ZTENUX5(JL,JK) = ZTENUCO5(JL,JK)
        ZTENVX5(JL,JK) = ZTENVCO5(JL,JK)

        ZTENT55F(JL,JK) = ZTENTX5(JL,JK)
        ZTENQ55F(JL,JK) = ZTENQX5(JL,JK)
        ZTENU55F(JL,JK) = ZTENUX5(JL,JK)
        ZTENV55F(JL,JK) = ZTENVX5(JL,JK)
      ENDDO
    ENDDO

!LOP Tracer tendencies (not used for the moment)
!    DO ITRC=1,ITRAC
!      DO JK=1,KLEV
!        DO JL=KIDIA,KFDIA
!          ZTENC55D(JL,JK,ITRC) = ZTENCX5(JL,JK,ITRC)
!        ENDDO
!      ENDDO
!    ENDDO

  ENDIF

!    ------------------------------------------------------------------

!*         6.A    LARGE SCALE WATER PHASE CHANGES
!                 -------------------------------

  IF ( LECOND2 .OR. LENCLD2 ) THEN

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENT55E(JL,JK) = ZTENTX5(JL,JK)
        ZTENQ55E(JL,JK) = ZTENQX5(JL,JK)
        ZTENU55E(JL,JK) = ZTENUX5(JL,JK)
        ZTENV55E(JL,JK) = ZTENVX5(JL,JK)
      ENDDO
    ENDDO
    IF (LEMWAVE) THEN
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZTENT5(JL,JK) = ZTENTX5(JL,JK)
          ZTENQ5(JL,JK) = ZTENQX5(JL,JK)
        ENDDO
      ENDDO
      CALL CLOUDST &
       & (KIDIA  , KFDIA , KLON , KTDIA , KLEV, LLSLPHY, LDRAIN1D,&
       & TSPHY  , &
       & PAPRS5 , PAPRSF5, PQ5  , Z_PQSAT5  , PT5    , &
       & ZLUDE5 , ZLU5   , &
       & ZTENT5 , ZTENQ5 , &
       & ZPA5   , ZPI5   , ZPL5 , Z_PFPLSL5 , Z_PFPLSN5, &
       & Z_PFHPSL5, Z_PFHPSN5, ZCOVPTOT5 )  

      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZTENTX5(JL,JK) = ZTENT5(JL,JK)
        ENDDO
      ENDDO
    ELSE
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZTENTX5(JL,JK) = ZTENTCL5(JL,JK) 
        ENDDO
      ENDDO
    ENDIF
  ENDIF

!     ------------------------------------------------------------------

!          7.A   WARNER-McINTYRE-SCINOCCA NON-OROGRAPHIC GRAVITY WAVE SCHEME
!                -----------------------------------------------------------

  IF ( LEGWWMS2 ) THEN

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENT55G(JL,JK) = ZTENTX5(JL,JK) 
        ZTENU55G(JL,JK) = ZTENUX5(JL,JK)
        ZTENU55G(JL,JK) = ZTENUX5(JL,JK)

        ZTENU5(JL,JK) = ZTENUWM5(JL,JK)
        ZTENV5(JL,JK) = ZTENVWM5(JL,JK)
      ENDDO
    ENDDO
  ENDIF

ELSE

! ------------------------------------------------------------
! re-computation of NL routines to get necessary trajectory  |
! ------------------------------------------------------------

!     ------------------------------------------------------------------

!*         3.     VERTICAL EXCHANGE OF U,V,T,Q BY TURBULENCE
!                 ------------------------------------------

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENT55B(JL,JK) = PTENT5(JL,JK)
      ZTENQ55B(JL,JK) = PTENQ5(JL,JK)
      ZTENU55B(JL,JK) = PTENU5(JL,JK)
      ZTENV55B(JL,JK) = PTENV5(JL,JK)
    ENDDO
  ENDDO
  IF (NGEMS > 0) THEN
    DO ITRC=1,ITRAC
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZTENC55B(JL,JK,ITRC) = ZTENC5(JL,JK,ITRC)  
        ENDDO
      ENDDO
    ENDDO
  ENDIF  

!     ***  LOCAL STORAGE OF UPDATED ARRAYS ***

  DO JL=KIDIA,KFDIA
    ZAZ0M55(JL)=ZAZ0M5(JL)
    ZAZ0H55(JL)=ZAZ0H5(JL)
    ZTLE55(JL)=Z_PTLE15(JL)
  ENDDO

  DO JK=1,KTILES
    DO JL=KIDIA,KFDIA
      ZUSTRTI55(JL,JK)=PUSTRTI5(JL,JK)
      ZVSTRTI55(JL,JK)=PVSTRTI5(JL,JK)
      ZAHFSTI55(JL,JK)=PAHFSTI5(JL,JK)
      ZEVAPTI55(JL,JK)=PEVAPTI5(JL,JK)
      ZTSKTI55(JL,JK)=PTSKTI5(JL,JK)
    ENDDO
  ENDDO

  IF ( LEVDIF2 ) THEN
    
    CALL VDFMAINS (KIDIA,KFDIA,KLON,KLEV,KLEVS,KSTEP,KTILES,&
     & ITRAC , TSPHY  , ITVL  , ITVH  ,ZCVL ,ZCVH ,&
     & ZLAIL , ZLAIH  , ZHSDFOR,&
     & PU5   , PV5    , PT5   , PQ5   , ZCEN5, PAPRS5, ZGEOM15, ZGEOMH5,&
     & PTL5  , PTSA5  , PWSA5,&
     & Z_PFRSO5(1,KLEV) , Z_PFRTH5(1,KLEV), Z_PEMIS5,&
     & ZTHKICE, ZSNTICE, &
     & PTSN5 ,PTIA5(1,1),&
     ! & ZHLICE5 , ZTLICE5 , ZTLWML5 , & !lake passive
     & PSST  , ISOTY  , ZFRTI , ZALBTI5 , ZWLMX,&
     & ZCHAR ,ZTSKRAD5, ZCFLX5,&
     ! OUTPUT (TRAJECTORY)
     & ZAZ0M5 , ZAZ0H5,&
     & ZUCFL5, ZVCFL5, ZTCFL5, ZDCFL5, ZQCFL5, PU10N5, PV10N5,& 
     ! OUTPUT TENDENDCIES
     & PTENT5 , PTENQ5 , PTENU5 , PTENV5 , ZTENC5, Z_PTLE15,&
     !-UPDATED FIELDS FOR TILES (TRAJECTORY)
     & PUSTRTI5 ,PVSTRTI5 ,PAHFSTI5,PEVAPTI5,PTSKTI5,&
     ! FLUX OUTPUTS (TRAJECTORY)
     & Z_PDIFTS5, Z_PDIFTQ5, Z_PSTRTU5 , Z_PSTRTV5 )

  ENDIF

! do not turn off shallow convection

  DO JL=KIDIA,KFDIA
    LLSHCV(JL) = .TRUE.
  ENDDO

!     ------------------------------------------------------------------

!*         4.     GRAVITY WAVE DRAG PARAMETRIZATION
!                  ---------------------------------

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENT55C(JL,JK) = PTENT5(JL,JK)
      ZTENQ55C(JL,JK) = PTENQ5(JL,JK)
      ZTENU55C(JL,JK) = PTENU5(JL,JK)
      ZTENV55C(JL,JK) = PTENV5(JL,JK)
    ENDDO
  ENDDO

  IF ( LEGWDG2 ) THEN

!*         4.1     CALL GWD

    CALL GWDRAGS &
     & ( KIDIA   , KFDIA , KLON , KLEV,&
     & TSPHY   , KSTEP  , KSTART,&
     & PAPRS5  , PAPRSF5, ZGEOM15,&
     & PT5   , PU5 , PV5,&
     & ZHSTD  , PGAMMA , PTHETA, PSIG,&
     ! OUTPUTS
     & Z_PUSTRG5  , Z_PVSTRG5 , Z_PVDISG5, PTENV5 , PTENU5 ,PTENT5,&
     ! FLUX OUTPUTS
     & Z_PSTRDU5  , Z_PSTRDV5                        )  

  ENDIF

!     ------------------------------------------------------------------

!*         5.     CONVECTION PARAMETRIZATION
!                 --------------------------

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENT55D(JL,JK) = PTENT5(JL,JK)
      ZTENQ55D(JL,JK) = PTENQ5(JL,JK)
      ZTENU55D(JL,JK) = PTENU5(JL,JK)
      ZTENV55D(JL,JK) = PTENV5(JL,JK)
    ENDDO
  ENDDO
  IF (NGEMS > 0) THEN
    DO ITRC=1,ITRAC
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZTENC55D(JL,JK,ITRC) = ZTENC5(JL,JK,ITRC)  
        ENDDO
      ENDDO  
    ENDDO
  ENDIF
  
  IF (LECUMF2) THEN

!*         5.1    CALL CUCALLN

    IF (.NOT.LECUMFS2) THEN

! TIEDTKE'S MASS FLUX CONVECTION SCHEME

      CALL CUCALLN &
       & ( KIDIA   , KFDIA , KLON  , KSMAX , KLEV,&
       & KSTEP   , LLLAND,LLSLPHY,&
       & TSPHY,&
       & PT5     , PQ5   ,  PU5    , PV5 , ZLITOT5,&
       & PVERVEL5, Z_PDIFTQ5, Z_PDIFTS5,   PAPRS5,&
       & PRSF15  , PRS15  , ZGEOM15, ZGEOMH5,&
       & PTENT5  , PTENQ5 , PTENU5 , PTENV5 , PACPR5,&
       & ITOPC   , IBASC  , ITYPE,&
       & ICBOT   , ICTOP  , IBOTSC , LLCUM  , LLSC,&
       & LLSHCV  ,&
       & ZLU5    , ZLUDE5 , ZMFU5  , ZMFD5,&
       & Z_PDIFCQ5 , Z_PDIFCS5, Z_PFHPCL5, Z_PFHPCN5,&
       & Z_PFPLCL5 , Z_PFPLCN5, Z_PSTRCU5, Z_PSTRCV5,&
       & Z_PFCCQL5 , Z_PFCCQN5,&
       & zmfude_rate ,    zmfdde_rate,   zcape,   zwmean,&
       & ITRAC   , ZCEN5   , ZTENC5, ZSCAV5 )

    ELSE

! SIMPLIFIED MASS FLUX CONVECTION SCHEME (P. Lopez)

      CALL CUCALLN2 &
       & ( KIDIA   , KFDIA , KLON  , KLEV,&
       & KSTEP   , KSTART, LLLAND,LLSLPHY,LDRAIN1D, &
       & TSPHY,&
       & PT5     , PQ5   ,  PU5    , PV5,&
       & PVERVEL5, Z_PDIFTQ5, Z_PDIFTS5, PAPRS5,&
       & PRSF15  , PRS15  , ZGEOM15, ZGEOMH5,&
       & PTENT5  , PTENQ5 , PTENU5 , PTENV5 , PACPR5,&
       & ITOPC   , IBASC  , ITYPE,&
       & ICBOT   , ICTOP  , IBOTSC , LLCUM  , LLSC,&
       & ZLU5    , ZLUDE5 , ZMFU5  , ZMFD5,&
       & Z_PDIFCQ5 , Z_PDIFCS5, Z_PFHPCL5, Z_PFHPCN5,&
       & Z_PFPLCL5 , Z_PFPLCN5, Z_PSTRCU5, Z_PSTRCV5,&
       & Z_PFCCQL5 , Z_PFCCQN5,&
       & zmfude_rate ,    zmfdde_rate,   zcape,&
       & ITRAC   , ZCEN5   , ZTENC5, ZSCAV5 )

    ENDIF

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENT55F(JL,JK) = PTENT5(JL,JK)
        ZTENQ55F(JL,JK) = PTENQ5(JL,JK)
        ZTENU55F(JL,JK) = PTENU5(JL,JK)
        ZTENV55F(JL,JK) = PTENV5(JL,JK)
      ENDDO
    ENDDO

  ENDIF

!     ------------------------------------------------------------------

!*         6.     LARGE SCALE WATER PHASE CHANGES
!                 -------------------------------

!     ***  LOCAL STORAGE OF UPDATED TENDENCIES ***

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENT55E(JL,JK) = PTENT5(JL,JK)
      ZTENQ55E(JL,JK) = PTENQ5(JL,JK)
      ZTENU55E(JL,JK) = PTENU5(JL,JK)
      ZTENV55E(JL,JK) = PTENV5(JL,JK)
    ENDDO
  ENDDO

  IF ( LECOND2 .AND. .NOT. LENCLD2 ) THEN

!*         6.1    CALL COND

    CALL COND &
     & ( KIDIA , KFDIA , KLON  , KLEV , LLSLPHY,&
     & TSPHY,&
     & PT5    , PQ5    , PRS15,   PRSF15,&
     & PTENT5,  PTENQ5,&
     & Z_PFHPSL5, Z_PFHPSN5, Z_PFPLSL5, Z_PFPLSN5          )  

  ELSEIF( LENCLD2 ) THEN

!*         6.2     CALL CLOUDST

    CALL CLOUDST &
     & (KIDIA  , KFDIA , KLON , KTDIA , KLEV, LLSLPHY, LDRAIN1D,&
     & TSPHY  , &
     & PAPRS5 , PAPRSF5, PQ5  , Z_PQSAT5  , PT5    , &
     & ZLUDE5 , ZLU5   , &
     & PTENT5 , PTENQ5 , &
     & ZPA5   , ZPI5   , ZPL5 , Z_PFPLSL5 , Z_PFPLSN5, &
     & Z_PFHPSL5, Z_PFHPSN5, ZCOVPTOT5 )  
  ENDIF

!*        7.  WARNER-McINTYRE-SCINOCCA NON-OROGRAPHIC GRAVITY WAVE SCHEME
!             -----------------------------------------------------------

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENT55G(JL,JK) = PTENT5(JL,JK)
      ZTENU55G(JL,JK) = PTENU5(JL,JK)
      ZTENV55G(JL,JK) = PTENV5(JL,JK)

      ZTENU5(JL,JK) = ZTENUWM5(JL,JK)
      ZTENV5(JL,JK) = ZTENVWM5(JL,JK)
    ENDDO
  ENDDO

  IF ( LEGWWMS2 ) THEN
!    ZTSGWD=GTPHYGWWMS
!    ZTSGWD=TSPHY
!    IF (MOD(KSTEP*TSPHY,ZTSGWD)==0.0_JPRB.OR.KSTEP==KSTART) THEN
!      CALL GWDRAG_WMSS(KIDIA , KFDIA , KLON   , KLEV   , ZTSGWD , &
!       & PT5 , PU5 , PV5 , PAPRSF5  , PAPRS5, PAPHIF5, &
!       & ZTENU5 , ZTENV5 , ZFLXWGWU5, ZFLXWGWV5)
!    ENDIF

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        PTENU5(JL,JK) = PTENU5(JL,JK)+ZTENU5(JL,JK)
        PTENV5(JL,JK) = PTENV5(JL,JK)+ZTENV5(JL,JK)

        ZTFIELD5(JL,JK) = (PU5(JL,JK)*ZTENU5(JL,JK) &
         & + PV5(JL,JK)*ZTENV5(JL,JK))*ZRCPD
        PTENT5(JL,JK) = PTENT5(JL,JK)-ZTFIELD5(JL,JK)
      ENDDO
    ENDDO

  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*         8.     ELIMINATION OF NEGATIVE SPECIFIC HUMIDITIES
!                 -------------------------------------------

IF ( LEQNGT2 ) THEN

  CALL ABOR1 ('AD OF NEGATIVE HUMIDITIES NOT CODED')

ELSE

!*         8.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PFCQNG5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         9.     TRACER TRANSPORT TENDENCIES BACK INTO EXTRA FIELDS
!                 --------------------------------------------------
 
IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_TEND( KIDIA, KFDIA, KLEV, KLON, PTENGFL5, ZTENC5 )
ENDIF


!     ------------------------------------------------------------------

!*        10.      COMPUTATION OF NEW SURFACE VALUES
!                  ---------------------------------

IF ( LESURF2) THEN

!*        10.1   CALL SURFTSTP

  CALL ABOR1 ('AD OF LAND SURFACE SCHEME NOT CODED')

ELSE
  DO JL=KIDIA,KFDIA
    Z_PFTG125(JL)=0.0_JPRB
    Z_PFWROD5(JL)=0.0_JPRB
    Z_PFWRO15(JL)=0.0_JPRB
    Z_PFWMLT5(JL)=0.0_JPRB
    Z_PFWG125(JL)=0.0_JPRB
    Z_PFWEV5(JL)=0.0_JPRB
  ENDDO
ENDIF
!     ------------------------------------------------------------------

!*              11.     END OF E.C.M.W.F. PHYSICS
!                      -------------------------

!*              11.1    UNIT CHANGES/SCALINGS
DO JL=KIDIA,KFDIA
  PGZ0M5 (JL)=ZAZ0M5(JL)*RG
  PLZ0H5 (JL)=LOG(ZAZ0H5(JL))
  Z_PFWG125(JL)=Z_PFWG125(JL)*1.E+03_JPRB
  Z_PFWMLT5(JL)=Z_PFWMLT5(JL)*1.E+03_JPRB
  Z_PFWRO15(JL)=Z_PFWRO15(JL)*1.E+03_JPRB
  Z_PFWROD5(JL)=Z_PFWROD5(JL)*1.E+03_JPRB
  Z_PFWEV5 (JL)=Z_PFWEV5 (JL)*1.E+03_JPRB
ENDDO

!-----------------------------------------------------------------------

!*                    ----  ADJOINT COMPUTATIONS  ----

!-----------------------------------------------------------------------

!*         7.     INITIALISATIONS
!                 ---------------

Z_PUSTRG (:)=0.0_JPRB
Z_PVSTRG (:)=0.0_JPRB

ZUCFL (:)=0.0_JPRB
ZVCFL (:)=0.0_JPRB
ZTCFL (:)=0.0_JPRB
ZQCFL (:)=0.0_JPRB
ZDCFL (:)=0.0_JPRB

ZALBD(:,:)   = 0.0_JPRB
ZALBP(:,:)   = 0.0_JPRB
ZALBTI(:,:)  = 0.0_JPRB
ZTSKRAD(:)   = 0.0_JPRB
ZEMIR(:)     = 0.0_JPRB
ZEMIW(:)     = 0.0_JPRB
ZAZ0H(:)     = 0.0_JPRB
ZAZ0M(:)     = 0.0_JPRB
ZCTLWP(:)    = 0.0_JPRB
ZSNM1M(:)    = 0.0_JPRB
ZWLM1M(:)    = 0.0_JPRB
ZGEOM1(:,:)  = 0.0_JPRB
ZGEOMH(:,:)  = 0.0_JPRB
ZLU(:,:)     = 0.0_JPRB
ZLUDE(:,:)   = 0.0_JPRB
ZMFU(:,:)    = 0.0_JPRB
ZMFD(:,:)    = 0.0_JPRB
ZHRSW(:,:)   = 0.0_JPRB
ZHRLW(:,:)   = 0.0_JPRB
ZTRSOD(:)    = 0.0_JPRB
ZT(:,:)      = 0.0_JPRB
ZTHT(:,:)    = 0.0_JPRB
ZEMIT(:)     = 0.0_JPRB

ZPA(:,:)     = 0.0_JPRB
ZPL(:,:)     = 0.0_JPRB
ZPI(:,:)     = 0.0_JPRB
ZQRAIN(:,:)  = 0.0_JPRB
ZRAINT(:,:)  = 0.0_JPRB
ZCTRSO(:,:)  = 0.0_JPRB
ZCEMTR(:,:)  = 0.0_JPRB
Z_PQICE(:,:)   = 0.0_JPRB
Z_PQLI(:,:)    = 0.0_JPRB
Z_PNEB(:,:)    = 0.0_JPRB
Z_PTCC(:)      = 0.0_JPRB
Z_PEMTD(:,:)   = 0.0_JPRB

Z_PFWROD (:)   = 0.0_JPRB
Z_PFWRO1 (:)   = 0.0_JPRB
Z_PFWMLT (:)   = 0.0_JPRB
Z_PFWG12 (:)   = 0.0_JPRB
Z_PFWEV  (:)   = 0.0_JPRB
Z_PFRSO(:,:)   = 0.0_JPRB
Z_PFRTH(:,:)   = 0.0_JPRB
Z_PQSAT(:,:)   = 0.0_JPRB

Z_PTRSO(:,:) = 0.0_JPRB
Z_PFRSOD(:)  = 0.0_JPRB
Z_PFRTHD(:)  = 0.0_JPRB

Z_PFPLSN (:,:) = 0.0_JPRB
Z_PFPLCN (:,:) = 0.0_JPRB
Z_PFPLSL (:,:) = 0.0_JPRB
Z_PFPLCL (:,:) = 0.0_JPRB
Z_PFHPSL (:,:) = 0.0_JPRB
Z_PFHPSN (:,:) = 0.0_JPRB
Z_PFHPCL (:,:) = 0.0_JPRB
Z_PFHPCN (:,:) = 0.0_JPRB
ZCOVPTOT (:,:) = 0.0_JPRB

ZTENU   (:,:) = 0.0_JPRB
ZTENV   (:,:) = 0.0_JPRB
ZTFIELD (:,:) = 0.0_JPRB
ZFLXWGWU(:,:) = 0.0_JPRB
ZFLXWGWV(:,:) = 0.0_JPRB

!    REMARK :

!    Prognostic variables of the surface scheme need to be initialised
!    to zero as long as the TL/AD versions are not available

Z_PTSAE1(:,:)  = 0.0_JPRB
Z_PWSAE1(:,:)  = 0.0_JPRB
Z_PWLE1(:)     = 0.0_JPRB
Z_PSNSE1(:)    = 0.0_JPRB


!*      AD-OBSERVATION OPERATOR FOR MW-RADIANCE ASSIMILATION

IF (LEMWAVE_ACTIVE) THEN

   DO JL = KIDIA, KFDIA
      YL_MWAVE_PHYS_FW (JL) % P   (1:KLEV)   = PAPRSF5 (JL,1:KLEV)  
      YL_MWAVE_PHYS_FW (JL) % PH  (1:KLEV+1) = PAPRS5  (JL,0:KLEV)
      YL_MWAVE_PHYS_FW (JL) % T   (1:KLEV)   = PT5     (JL,1:KLEV)
      YL_MWAVE_PHYS_FW (JL) % Q   (1:KLEV)   = PQ5     (JL,1:KLEV)
    
      DO JK = 1, KLEV
         YL_MWAVE_PHYS_FW (JL) % RFL (JK)  = 0.5_JPRB * (Z_PFPLSL5 (JL,JK) + Z_PFPLSL5 (JL,JK-1))
         YL_MWAVE_PHYS_FW (JL) % SFL (JK)  = 0.5_JPRB * (Z_PFPLSN5 (JL,JK) + Z_PFPLSN5 (JL,JK-1))
         YL_MWAVE_PHYS_FW (JL) % CRFL (JK) = 0.5_JPRB * (Z_PFPLCL5 (JL,JK) + Z_PFPLCL5 (JL,JK-1))
         YL_MWAVE_PHYS_FW (JL) % CSFL (JK) = 0.5_JPRB * (Z_PFPLCN5 (JL,JK) + Z_PFPLCN5 (JL,JK-1))
      ENDDO
      
      IF (LENCLD2) THEN  
        YL_MWAVE_PHYS_FW (JL) % CWC (1:KLEV) = ZPL5 (JL,1:KLEV)
        YL_MWAVE_PHYS_FW (JL) % CIC (1:KLEV) = ZPI5 (JL,1:KLEV)
        YL_MWAVE_PHYS_FW (JL) % CC  (1:KLEV) = ZPA5 (JL,1:KLEV)
        YL_MWAVE_PHYS_FW (JL) % PFRC(1:KLEV) = ZCOVPTOT5 (JL,1:KLEV)
      ELSE
        YL_MWAVE_PHYS_FW (JL) % CWC (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_FW (JL) % CIC (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_FW (JL) % CC  (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_FW (JL) % PFRC(1:KLEV) = 0.0_JPRB
      ENDIF

      YL_MWAVE_PHYS_FW (JL) % U10  = ZUCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % V10  = ZVCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % T2M  = ZTCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % Q2M  = ZQCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % TSK  = PTL5   (JL)
      
      YL_MWAVE_PHYS_FW (JL) % LSM  = PLSM   (JL)
      YL_MWAVE_PHYS_FW (JL) % SIM  = PCI    (JL)
      
      YL_MWAVE_PHYS_FW (JL) % SLAT = PGEMU  (JL)
      YL_MWAVE_PHYS_FW (JL) % SLON = PSLON  (JL)
      YL_MWAVE_PHYS_FW (JL) % CLON = PCLON  (JL)
      YL_MWAVE_PHYS_FW (JL) % OROG = POROG  (JL) / RG ! Convert to metres
            
      YL_MWAVE_PHYS_AD (JL) % P  (1:KLEV)   = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % PH (1:KLEV+1) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % T  (1:KLEV)   = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % Q  (1:KLEV)   = 0.0_JPRB
    
      YL_MWAVE_PHYS_AD (JL) % RFL (1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % SFL (1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CRFL(1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CSFL(1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CWC (1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CIC (1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CC  (1:KLEV) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % PFRC(1:KLEV) = 0.0_JPRB
    
      YL_MWAVE_PHYS_AD (JL) % U10  = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % V10  = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % T2M  = 0.0_JPRB 
      YL_MWAVE_PHYS_AD (JL) % Q2M  = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % TSK  = 0.0_JPRB
      
      YL_MWAVE_PHYS_AD (JL) % LSM  = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % SIM  = 0.0_JPRB
      
      YL_MWAVE_PHYS_AD (JL) % SLAT = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % SLON = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % CLON = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % OROG = 0.0_JPRB  
   ENDDO

   CALL MWAVE_OBSOP_AD (KIDIA, KFDIA, KLON, KLEV, NMAX_CHAN, YL_MWAVE_PHYS_FW, YL_MWAVE_PHYS_AD, YD_MWAVE_RAD)

   DO JL = KIDIA, KFDIA
      PAPRSF  (JL,1:KLEV) = PAPRSF (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % P  (1:KLEV)
      PAPRS   (JL,0:KLEV) = PAPRS  (JL,0:KLEV) + YL_MWAVE_PHYS_AD (JL) % PH (1:KLEV+1)
      PT      (JL,1:KLEV) = PT     (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % T  (1:KLEV)
      PQ      (JL,1:KLEV) = PQ     (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % Q  (1:KLEV)
      
      YL_MWAVE_PHYS_AD (JL) % P  (:) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % PH (:) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % T  (:) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % Q  (:) = 0.0_JPRB      

      IF (LENCLD2) THEN  
        ZPA (JL,1:KLEV) = ZPA (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % CC  (1:KLEV)
        ZPL (JL,1:KLEV) = ZPL (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % CWC (1:KLEV)
        ZPI (JL,1:KLEV) = ZPI (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % CIC (1:KLEV)
        ZCOVPTOT (JL,1:KLEV) = ZCOVPTOT (JL,1:KLEV) + YL_MWAVE_PHYS_AD (JL) % PFRC (1:KLEV) 
      ENDIF

      YL_MWAVE_PHYS_AD (JL) % PFRC(:) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CC  (:) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CWC (:) = 0.0_JPRB
      YL_MWAVE_PHYS_AD (JL) % CIC (:) = 0.0_JPRB
   
      DO JK = KLEV, 1, -1
         Z_PFPLSN (JL,JK-1) = Z_PFPLSN (JL,JK-1) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % SFL (JK)
         Z_PFPLCN (JL,JK-1) = Z_PFPLCN (JL,JK-1) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % CSFL(JK)
         Z_PFPLSN (JL,JK  ) = Z_PFPLSN (JL,JK  ) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % SFL (JK)
         Z_PFPLCN (JL,JK  ) = Z_PFPLCN (JL,JK  ) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % CSFL(JK)
         YL_MWAVE_PHYS_AD (JL) % SFL (JK) = 0.0_JPRB
         YL_MWAVE_PHYS_AD (JL) % CSFL (JK) = 0.0_JPRB

         Z_PFPLSL (JL,JK-1) = Z_PFPLSL (JL,JK-1) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % RFL (JK)
         Z_PFPLCL (JL,JK-1) = Z_PFPLCL (JL,JK-1) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % CRFL(JK)
         Z_PFPLSL (JL,JK  ) = Z_PFPLSL (JL,JK  ) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % RFL (JK)
         Z_PFPLCL (JL,JK  ) = Z_PFPLCL (JL,JK  ) + 0.5_JPRB * YL_MWAVE_PHYS_AD (JL) % CRFL(JK)
         YL_MWAVE_PHYS_AD (JL) % RFL (JK) = 0.0_JPRB
         YL_MWAVE_PHYS_AD (JL) % CRFL (JK) = 0.0_JPRB
      ENDDO

      ZUCFL (JL) = ZUCFL (JL) + YL_MWAVE_PHYS_AD (JL) % U10
      ZVCFL (JL) = ZVCFL (JL) + YL_MWAVE_PHYS_AD (JL) % V10
      ZTCFL (JL) = ZTCFL (JL) + YL_MWAVE_PHYS_AD (JL) % T2M
      ZQCFL (JL) = ZQCFL (JL) + YL_MWAVE_PHYS_AD (JL) % Q2M
      PTL   (JL) = PTL   (JL) + YL_MWAVE_PHYS_AD (JL) % TSK
      
      YL_MWAVE_PHYS_AD (JL) % U10 = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % V10 = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % T2M = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % Q2M = 0.0_JPRB  
      YL_MWAVE_PHYS_AD (JL) % TSK = 0.0_JPRB  
   ENDDO
ENDIF
!     ------------------------------------------------------------------

!*              10.     END OF E.C.M.W.F. PHYSICS
!                      -------------------------

!*              10.1    UNIT CHANGES/SCALINGS
DO JL=KIDIA,KFDIA
  Z_PFWEV  (JL)=Z_PFWEV  (JL)*1.E+03_JPRB
  Z_PFWROD (JL)=Z_PFWROD (JL)*1.E+03_JPRB
  Z_PFWRO1 (JL)=Z_PFWRO1 (JL)*1.E+03_JPRB
  Z_PFWMLT (JL)=Z_PFWMLT (JL)*1.E+03_JPRB
  Z_PFWG12 (JL)=Z_PFWG12 (JL)*1.E+03_JPRB
  ZAZ0H  (JL)=ZAZ0H(JL) + PLZ0H(JL)/ZAZ0H55(JL)
  PLZ0H  (JL)=0.0_JPRB
  ZAZ0M  (JL)=ZAZ0M(JL) + PGZ0M(JL)*RG
  PGZ0M  (JL)=0.0_JPRB
ENDDO
!     ------------------------------------------------------------------

!*        10.      COMPUTATION OF NEW SURFACE VALUES
!                  ---------------------------------

IF ( LESURF2) THEN

!*        10.1   CALL SURFTSTP

  CALL ABOR1 ('AD OF LAND SURFACE SCHEME NOT CODED')

ELSE
  DO JL=KIDIA,KFDIA
    Z_PFTG12 (JL)=0.0_JPRB
    Z_PFWROD (JL)=0.0_JPRB
    Z_PFWRO1 (JL)=0.0_JPRB
    Z_PFWMLT (JL)=0.0_JPRB
    Z_PFWG12 (JL)=0.0_JPRB
    Z_PFWEV (JL)=0.0_JPRB
  ENDDO
ENDIF

!*         9.      TRACER TRANSPORT TENDENCIES BACK INTO EXTRA FIELDS
!                  --------------------------------------------------

!    Initialize Tracers Adjoint Variables

IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_INIT_AD( KIDIA, KFDIA, KLEV, KLON, ITRAC, &
   &  PTENGFL, ZCEN, ZTENC, ZCFLX )
ELSE
! ALLOCATE arrays so they can be passed to VDFOUTER etc. in the non-GEMS case
   ALLOCATE(ZCEN(1,1,1))
   ALLOCATE(ZTENC(1,1,1))
   ALLOCATE(ZCFLX(1,1))
ENDIF

!     ------------------------------------------------------------------

!*         8.     ELIMINATION OF NEGATIVE SPECIFIC HUMIDITIES
!                 -------------------------------------------

IF ( LEQNGT2 ) THEN

  CALL ABOR1 ('AD OF NEGATIVE HUMIDITIES NOT CODED')

ELSE

!*         8.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PFCQNG (JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         7.   WARNER-McINTYRE-SCINOCCA NON-OROGRAPHIC GRAVITY WAVE SCHEME
!               -----------------------------------------------------------

IF ( LEGWWMS2 ) THEN
 
  ZTSGWD=GTPHYGWWMS
!  ZTSGWD=TSPHY

  IFREQWMS = INT(ZTSGWD/TSPHY)
  ISTEPWMS = MOD(KSTEP,IFREQWMS)

  DO JK=KLEV,1,-1
    DO JL=KIDIA,KFDIA
      ZTFIELD(JL,JK) = ZTFIELD(JL,JK)-PTENT(JL,JK)
      ZTENU(JL,JK) = ZTENU(JL,JK)+ZRCPD*PU5(JL,JK)*ZTFIELD(JL,JK)
      PU(JL,JK) = PU(JL,JK)+ZRCPD*ZTENU5(JL,JK)*ZTFIELD(JL,JK)
      ZTENV(JL,JK) = ZTENV(JL,JK)+ZRCPD*PV5(JL,JK)*ZTFIELD(JL,JK)
      PV(JL,JK) = PV(JL,JK)+ZRCPD*ZTENV5(JL,JK)*ZTFIELD(JL,JK)
      ZTFIELD(JL,JK) = 0.0_JPRB

      ZTENV(JL,JK) = ZTENV(JL,JK)+PTENV(JL,JK)
      ZTENU(JL,JK) = ZTENU(JL,JK)+PTENU(JL,JK)

      IF (LREGWWMS) THEN
        IF (PAPRSF5(JL,JK) < 50.0_JPRB .AND. ABS(PV5(JL,JK)) > 30.0_JPRB) THEN
          ZTENU (JL,JK) = 0.0_JPRB
        ENDIF
      ENDIF

    ENDDO
  ENDDO

  IF (ISTEPWMS == 0. .OR. KSTEP==KSTART .OR. KSTEP >= (NSTOP-1)) THEN
!  IF (MOD(KSTEP*TSPHY,ZTSGWD)==0.0_JPRB.OR.KSTEP==KSTART) THEN

    IBLK=(KSTGLO-1)/KLON + 1
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENU(JL,JK) = ZTENU(JL,JK)+TGWDWMS(JL,JK,IBLK)
        TGWDWMS(JL,JK,IBLK) = 0.0_JPRB
        ZTENV(JL,JK) = ZTENV(JL,JK)+TGWDWMS(JL,KLEV+JK,IBLK)
        TGWDWMS(JL,KLEV+JK,IBLK) = 0.0_JPRB 
      ENDDO
    ENDDO
        
    CALL GWDRAG_WMSSAD(KIDIA , KFDIA , KLON   , KLEV   , ZTSGWD , &
     & PT5 , PU5 , PV5 , PAPRSF5  , PAPRS5, PAPHIF5, &
     & ZTENU55G  , ZTENV55G , ZFLXWGWU5, ZFLXWGWV5,  &
     & PT  , PU  , PV  , PAPRSF   , PAPRS , PAPHIF , &
     & ZTENU  , ZTENV  , ZFLXWGWU , ZFLXWGWV )

  ELSE

    IBLK=(KSTGLO-1)/KLON + 1
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        TGWDWMS(JL,JK,IBLK) = TGWDWMS(JL,JK,IBLK)+ZTENU(JL,JK)
        ZTENU(JL,JK) = 0.0_JPRB 
        TGWDWMS(JL,KLEV+JK,IBLK) = TGWDWMS(JL,KLEV+JK,IBLK)+ZTENV(JL,JK)
        ZTENV(JL,JK) = 0.0_JPRB
      ENDDO
    ENDDO
  ENDIF

ELSE

  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA      
      ZFLXWGWU (JL,JK) = 0.0_JPRB
      ZFLXWGWV (JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO
ENDIF


!*         6.     LARGE SCALE WATER PHASE CHANGES
!                 -------------------------------

!*    SET UP INPUT FLUXES TO ZERO

IF ( LECOND2 .AND. .NOT. LENCLD2 ) THEN

!*         6.1    CALL CONDAD

  CALL CONDAD &
   & ( KIDIA , KFDIA , KLON  , KLEV,&
   & TSPHY,&
   & PT5    , PQ5    , PRS15,   PRSF15,&
   & ZTENT55E, ZTENQ55E,&
   & Z_PFHPSL5, Z_PFHPSN5, Z_PFPLSL5, Z_PFPLSN5,&
   & PT    , PQ    , PRS1,   PRSF1,&
   & PTENT,  PTENQ,&
   & Z_PFHPSL, Z_PFHPSN, Z_PFPLSL, Z_PFPLSN          )  

ELSEIF (LENCLD2) THEN

!*         6.2     CALL CLOUDSTAD

  IBLK=(KSTGLO-1)/KLON + 1
  IF (KSTEP >= (NSTOP-1)) THEN
    PA(:,:)  = 0.0_JPRB
    PL(:,:)  = 0.0_JPRB
    PI(:,:)  = 0.0_JPRB
  ELSE
    DO JK=KLEV,1,-1
      DO JL=KIDIA,KFDIA
        ZPI (JL,JK) = ZPI (JL,JK) + PI(JL,JK)
        PI(JL,JK) = 0.0_JPRB
        ZPL (JL,JK) = ZPL (JL,JK) + PL(JL,JK)
        PL(JL,JK) = 0.0_JPRB
        ZPA (JL,JK) = ZPA (JL,JK) + PA(JL,JK)
        PA(JL,JK) = 0.0_JPRB
      ENDDO
    ENDDO
  ENDIF

  CALL CLOUDSTAD &
   & (KIDIA  , KFDIA , KLON , KTDIA , KLEV, LDRAIN1D, &
   & TSPHY  , &
   & PAPRS5 , PAPRSF5, PQ5  , Z_PQSAT5  , PT5    , &
   & ZLUDE5 , ZLU5   , &
   & ZTENT55E , ZTENQ55E , &
   & ZPA5   , ZPI5   , ZPL5 , Z_PFPLSL5 , Z_PFPLSN5, &
   & Z_PFHPSL5, Z_PFHPSN5, ZCOVPTOT5, &
   & PAPRS  , PAPRSF , PQ   , Z_PQSAT   , PT     , &
   & ZLUDE  , ZLU    , &
   & PTENT  , PTENQ  , &
   & ZPA    , ZPI    , ZPL  ,Z_PFPLSL   , Z_PFPLSN , &
   & Z_PFHPSL , Z_PFHPSN, ZCOVPTOT )  

ENDIF

!     ------------------------------------------------------------------

!*         5.     CONVECTION PARAMETRIZATION
!                 --------------------------

IF (LECUMF2) THEN

  IF (.NOT.LECUMFS2) THEN
  

! TIEDTKE'S MASS FLUX CONVECTION SCHEME

    CALL CUCALLNAD &
     & (  KIDIA,    KFDIA,    KLON,   KLEV,&
     & LLLAND,&
     & TSPHY,&
     & PT5,      PQ5,      PU5,      PV5  ,   ZLITOT5,&
     & PVERVEL5, Z_PDIFTQ5,  Z_PDIFTS5,&
     & PRSF15,   PRS15,    ZGEOM15,  ZGEOMH5,&
     & ZTENT55D, ZTENQ55D, ZTENU55D, ZTENV55D,&
     & ITYPE,&
     & ICBOT,    ICTOP,    IBOTSC,   LLCUM,   LLSC,&
     & ZLU5,     ZLUDE5,   ZMFU5,    ZMFD5,&
     & Z_PFPLCL5,  Z_PFPLCN5,&
     & ITRAC,    ZCEN5,    ZTENC55D, ZSCAV5,&
     & PT,       PQ,       PU,       PV,&
     & PVERVEL,  PDIFTQ,   PDIFTS,&
     & PRSF1,    PRS1,     ZGEOM1,   ZGEOMH,&
     & PTENT,    PTENQ,    PTENU,    PTENV,&
     & ZLU,      ZLUDE,    ZMFU,     ZMFD,&
     & ZCEN  , ZTENC )  

  ELSE

! SIMPLIFIED MASS FLUX CONVECTION SCHEME (P. Lopez)

    CALL CUCALLN2AD &
     & (  KIDIA,    KFDIA,    KLON,   KLEV,&
     & KSTEP,    KSTART,   LLLAND, LDRAIN1D, &
     & TSPHY,&
     & PT5,      PQ5,      PU5,      PV5,&
     & PVERVEL5, Z_PDIFTQ5,  Z_PDIFTS5,  PAPRS5,&
     & PRSF15,   PRS15,    ZGEOM15,  ZGEOMH5,&
     & ZTENT55D, ZTENQ55D, ZTENU55D, ZTENV55D,&
     & ZTENT55F, ZTENQ55F, ZTENU55F, ZTENV55F,&
     & ITYPE,&
     & ICBOT,    ICTOP,    IBOTSC,   LLCUM,   LLSC,&
     & ZLU5,     ZLUDE5,   ZMFU5,    ZMFD5,&
     & Z_PFPLCL5,  Z_PFPLCN5,&
     & ITRAC,    ZCEN5,    ZTENC55D, ZSCAV5,&
     & PT,       PQ,       PU,       PV,&
     & PVERVEL,  PDIFTQ,   PDIFTS,   PAPRS,&
     & PRSF1,    PRS1,     ZGEOM1,   ZGEOMH,&
     & PTENT,    PTENQ,    PTENU,    PTENV,&
     & ZLU,      ZLUDE,    ZMFU,     ZMFD,&
     & Z_PDIFCQ,   Z_PDIFCS,   Z_PFHPCL,   Z_PFHPCN,&
     & Z_PFPLCL,   Z_PFPLCN,   Z_PSTRCU,   Z_PSTRCV,  Z_PFCCQL ,Z_PFCCQN,&
     & ZCEN  , ZTENC )  

  ENDIF
  
!*         5.2    SET CONVECTIVE CLOUD PARAMETERS FOR NEXT TIME-STEP


  DO JL=KIDIA,KFDIA
    PNTOP(JL)=REAL ( ITOPC(JL),JPRB)  + 0.01_JPRB
    PNBAS(JL)=REAL ( IBASC(JL),JPRB)  + 0.01_JPRB
  ENDDO

ELSE

!*         5.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PDIFCS (JL,JK)=0.0_JPRB
      Z_PDIFCQ (JL,JK)=0.0_JPRB
      Z_PFHPCL (JL,JK)=0.0_JPRB
      Z_PFHPCN (JL,JK)=0.0_JPRB
      Z_PFPLCL (JL,JK)=0.0_JPRB
      Z_PFPLCN (JL,JK)=0.0_JPRB
      Z_PSTRCU (JL,JK)=0.0_JPRB
      Z_PSTRCV (JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF


!*         4.     GRAVITY WAVE DRAG PARAMETRIZATION
!                  ---------------------------------

!*         4.1     SETUP INPUT FLUXES TO ZERO.

!Z_PUSTRG (:)=0.0_JPRB
!Z_PVSTRG (:)=0.0_JPRB
Z_PVDISG (:)=0.0_JPRB

Z_PSTRDU (:,:)=0.0_JPRB
Z_PSTRDV (:,:)=0.0_JPRB

IF ( LEGWDG2 ) THEN

!*         4.2     CALL GWDAD

  CALL GWDRAGAD &
   & ( KIDIA    , KFDIA  , KLON  , KLEV,&
   & TSPHY    , KSTEP   , KSTART,&
   & PAPRS5   , PAPRSF5 , ZGEOM15, PT5    , PU5,&
   & PV5      , ZHSTD   , PGAMMA,  PTHETA , PSIG,&
   & PAPRS    , PAPRSF  , ZGEOM1 , PT     , PU,&
   & PV,&
   ! OUTPUTS
   & Z_PUSTRG5  , Z_PVSTRG5 , Z_PVDISG5,&
   & ZTENV55C , ZTENU55C, ZTENT55C,&
   & Z_PUSTRG   , Z_PVSTRG  , Z_PVDISG , PTENV  , PTENU  , PTENT,&
   ! FLUX OUTPUTS
   & Z_PSTRDU5  , Z_PSTRDV5,&
   & Z_PSTRDU   , Z_PSTRDV                                )  

ENDIF

!     ------------------------------------------------------------------

!*         3.     VERTICAL EXCHANGE OF U,V,T,Q BY TURBULENCE
!                 ------------------------------------------

!*         3.1     SETUP INPUT FLUXES TO ZERO.

DO JL=KIDIA,KFDIA
  Z_PVDIS (JL)=0.0_JPRB
  Z_PFTLHEV (JL)=0.0_JPRB
  Z_PFTLHSB (JL)=0.0_JPRB
  Z_PFWSB (JL)=0.0_JPRB
ENDDO

IF ( LEVDIF2 ) THEN

  CALL VDFMAINSAD (KIDIA,KFDIA,KLON,KLEV,KLEVS,KSTEP,KTILES,&
   & ITRAC , TSPHY  , ITVL  , ITVH  ,ZCVL ,ZCVH ,&
   & ZLAIL, ZLAIH   , ZHSDFOR,&
   & PU5   , PV5    , PT5   , PQ5   , ZCEN5, PAPRS5, ZGEOM15, ZGEOMH5,&
   & PTL5  , PTSA5  , PWSA5,&
   & Z_PFRSO5(1,KLEV), Z_PFRTH5(1,KLEV), Z_PEMIS5,&
   & ZTHKICE, ZSNTICE, &
   & PTSN5 ,PTIA5(1,1),&
   ! & ZHLICE5 , ZTLICE5 , ZTLWML5 , & !lake passive
   & PSST  , ISOTY  , ZFRTI , ZALBTI5 , ZWLMX,&
   & ZCHAR ,ZTSKRAD5,ZCFLX5,&
   ! OUTPUT (TRAJECTORY)
   & ZAZ0M55 , ZAZ0H55,&
   & ZUCFL5, ZVCFL5, ZTCFL5, ZDCFL5, ZQCFL5, PU10N5, PV10N5,& 
   ! OUTPUT TENDENDCIES
   & ZTENT55B , ZTENQ55B , ZTENU55B , ZTENV55B , ZTENC55B, ZTLE55,&
   !-UPDATED FIELDS FOR TILES (TRAJECTORY)
   & ZUSTRTI55 ,ZVSTRTI55 ,ZAHFSTI55,ZEVAPTI55,ZTSKTI55,&
   ! FLUX OUTPUTS (TRAJECTORY)
   & Z_PDIFTS5, Z_PDIFTQ5, Z_PSTRTU5 , Z_PSTRTV5,&
   !=========  
   & PU    , PV    , PT    , PQ   , ZCEN, PAPRS , ZGEOM1, ZGEOMH,&
   & PTL   , PTSA  , PWSA,&
   & Z_PFRSO(1,KLEV) , Z_PFRTH(1,KLEV),&
   & ZALBTI ,ZTSKRAD,ZCFLX,&
   ! OUTPUT           
   & ZAZ0M , ZAZ0H,&
   & ZUCFL, ZVCFL, ZTCFL, ZDCFL, ZQCFL, PU10N, PV10N,& 
   ! OUTPUT TENDENCIES           
   & PTENT  , PTENQ , PTENU , PTENV , ZTENC, PTLE1,&
   !-UPDATED FIELDS FOR TILES 
   & PUSTRTI , PVSTRTI , PAHFSTI , PEVAPTI , PTSKTI,&
   ! FLUX OUTPUTS 
   & PDIFTS , PDIFTQ  , PSTRTU  , PSTRTV  )  
     
ELSE
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      PDIFTQ (JL,JK)=0.0_JPRB
      PDIFTS (JL,JK)=0.0_JPRB
      PSTRTU (JL,JK)=0.0_JPRB
      PSTRTV (JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
  PU10N(:) = 0.0_JPRB
  PV10N(:) = 0.0_JPRB
ENDIF


!     ------------------------------------------------------------------

!*         2.     RADIATION TRANSFER
!                 ------------------

IF ( LERADI2 ) THEN

!*         2.1  FULL RADIATION COMPUTATIONS
!*         2.5  RADIATIVE FLUXES AT EVERY TIME-STEP

  CALL RADHEATAD &
   & (  KIDIA   , KFDIA   , KLON    , KLEV,&
   & PAPRS5  , PAPRSF5,&
   & Z_PEMIS5  , PEMTD5  , PMU0,&
   & PQ5,&
   & ZTENT55A , PT5     , PTRSO5  , ZTRSOD5 , PTL5,&
   & Z_PNEB5  , Z_PQLI5 , Z_PQICE5,&
   & Z_PFRSO5  ,Z_PFRTH5  , Z_PFRSOD5 , Z_PFRTHD5,&
   & PAPRS   , PAPRSF,&
   & Z_PEMTD,&
   & PQ,&
   & PTENT   , PT      , Z_PTRSO   , ZTRSOD  , PTL,&
   & Z_PFRSO   , Z_PFRTH   , Z_PFRSOD  , Z_PFRTHD )  

!*         2.4  CALL RADIATION AT EVERY GRID POINT
  PTL(KIDIA:KFDIA)=PTL(KIDIA:KFDIA)+ZTSKRAD(KIDIA:KFDIA)
  ZTSKRAD(KIDIA:KFDIA)=0.0_JPRB

  IF (LERADSW2 .OR. LERADN2) THEN

    IF (LERADN2 .AND. LERADLW2) THEN
      ISTEP = MOD(KSTEP,NLWFR)
      IF (ISTEP /= 0 .AND. KSTEP /= 0 .AND. KSTEP < (NSTOP-1)) THEN
        IBLK=(KSTGLO-1)/KLON + 1
        DO JK=0,KLEV
          DO JL=KIDIA,KFDIA
            RADLWM (JL,JK,IBLK) = RADLWM (JL,JK,IBLK)+Z_PEMTD(JL,JK)
            Z_PEMTD(JL,JK) = 0.0_JPRB
          ENDDO
        ENDDO
      ELSE
        IBLK=(KSTGLO-1)/KLON + 1
        DO JK=0,KLEV
          DO JL=KIDIA,KFDIA
            Z_PEMTD(JL,JK) = Z_PEMTD(JL,JK)+RADLWM (JL,JK,IBLK)
            RADLWM (JL,JK,IBLK) = 0.0_JPRB
          ENDDO
        ENDDO      
      ENDIF
    ENDIF

    CALL RADINAAD &
     & ( KIDIA , KFDIA , KLON , KLEV   , NMODE,&
     & ZALBD5, ZALBP5, PAPRS5 , PAPRSF5,&
     & ZCCNL5, ZCCNO5, RCARDI , Z_PNEB5,&
     & PGELAM, PCLON , PSLON  ,&
     & PDELP5, ZEMIR5, ZEMIW5,&
     & PGEMU , PMU0  ,  PQ5  , Z_PQSAT5,&
     & Z_PQICE5, Z_PQLI5,&
     & PLSM  , PT5   , PTL5,&
     & ZTHT5 ,&
     & ZALBD , ZALBP  , PAPRS  , PAPRSF ,&
     & ZCCNL , ZCCNO  , ZCARDI , Z_PNEB ,&
     & PDELP , ZEMIR  , ZEMIW  ,&
     & PQ    , Z_PQSAT,&
     & Z_PQICE , Z_PQLI,&
     & PT    , PTL    ,&
     & Z_PEMTD , Z_PTRSO,&
     & ZTHT  , ZTRSOD)  
  ENDIF

!*         2.3 CLOUD
  IF (LEDCLD2 .AND. .NOT. LENCLD2) THEN
    CALL CLOUDAD ( KIDIA    , KFDIA , KLON   , KLEV,&
     & IBASC1  , ITOPC1  ,&
     & PQ5     , Z_PQSAT5, PT5     , Z_PACPR15,&
     & Z_PNEB5 , Z_PTCC5 , Z_PQICE5, Z_PQLI5,&
     & PQ      , Z_PQSAT , PT      , PACPR  ,&
     & Z_PNEB  , Z_PTCC  , Z_PQICE , Z_PQLI  )  
  ELSEIF (LENCLD2) THEN
    CALL  CLDPPAD &
     & ( KIDIA , KFDIA    , KLON    , KLEV,&
     & ZPA55   , ZPL55    , ZPI55   ,&
     & Z_PNEB5 , Z_PQICE5 , Z_PQLI5 ,&
     & ZPA     , ZPL      , ZPI     ,&
     & Z_PNEB  , Z_PQICE  , Z_PQLI  )      
    
    IBLK=(KSTGLO-1)/KLON + 1
    IF (KSTEP == KSTART) THEN
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZPA  (JL,JK) = 0.0_JPRB
          ZPL  (JL,JK) = 0.0_JPRB
          ZPI  (JL,JK) = 0.0_JPRB
        ENDDO
      ENDDO
    ELSE
      DO JK=KLEV,1,-1
        DO JL=KIDIA,KFDIA
          PI(JL,JK) = PI(JL,JK) + ZPI(JL,JK)  
          ZPI  (JL,JK) = 0.0_JPRB
          PL(JL,JK) = PL(JL,JK) + ZPL(JL,JK)  
          ZPL  (JL,JK) = 0.0_JPRB
          PA(JL,JK) = PA(JL,JK) + ZPA(JL,JK)
          ZPA  (JL,JK) = 0.0_JPRB
        ENDDO
      ENDDO
    ENDIF

  ELSE
    Z_PNEB  (:,:)=0.0_JPRB
    Z_PQLI  (:,:)=0.0_JPRB
    Z_PQICE (:,:)=0.0_JPRB
    Z_PTCC  (:)  =0.0_JPRB
  ENDIF

  CALL SATURAD (KIDIA , KFDIA , KLON  , KTDIA , KLEV,&
   & PAPRSF5, PT5    , Z_PQSAT5,&
   & PAPRSF , PT     , Z_PQSAT  )  

ELSE
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      Z_PNEB (JL,JK)=0.0_JPRB
      Z_PQLI (JL,JK)=0.0_JPRB
      Z_PQICE (JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
  DO JL=KIDIA,KFDIA
    Z_PFRSOC (JL,0)=0.0_JPRB
    Z_PFRTHC (JL,0)=0.0_JPRB
    Z_PFRSOC (JL,1)=0.0_JPRB
    Z_PFRTHC (JL,1)=0.0_JPRB
    Z_PFRSOD (JL)=0.0_JPRB
    Z_PFRTHD (JL)=0.0_JPRB
  ENDDO
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PFRSO (JL,JK)=0.0_JPRB
      Z_PFRTH (JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
  PTL(KIDIA:KFDIA)=PTL(KIDIA:KFDIA)+ZTSKRAD(KIDIA:KFDIA)
  ZTSKRAD(KIDIA:KFDIA)=0.0_JPRB
ENDIF

!*         1.     INITIAL COMPUTATIONS.
!                 ---------------------

!*       1.6  SET SURFACE SOLAR TRANSMISSIVITY
Z_PSRSWD (:) = 0.0_JPRB
DO JL=KIDIA,KFDIA
  Z_PSRSWD(JL) = Z_PSRSWD(JL) + ZTRSOD (JL)
  ZTRSOD (JL) = 0.0_JPRB
ENDDO

!*       1.5  SET FLUXES TO ZERO
DO JK=0,KLEV
  DO JL=KIDIA,KFDIA
    Z_PFCCQL (JL,JK) = 0.0_JPRB
    Z_PFCCQN (JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!*       1.4  SET SURFACE TENDENCIES TO ZERO
DO JK=1,KLEVS
  DO JL=KIDIA,KFDIA
    Z_PTSAE1 (JL,JK)=0.0_JPRB
    Z_PWSAE1 (JL,JK)=0.0_JPRB
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  PTLE1 (JL)=0.0_JPRB
  Z_PWLE1 (JL)=0.0_JPRB
  Z_PSNSE1 (JL)=0.0_JPRB
ENDDO

!*      1.3   GLOBAL AUXILLIARY VARIABLES

DO JK=0,KLEV
  DO JL=KIDIA,KFDIA
    PAPHI (JL,JK) = PAPHI (JL,JK) + ZGEOMH (JL,JK)
    ZGEOMH (JL,JK)=0.0_JPRB
  ENDDO
ENDDO
DO JK=KLEV,1,-1
  DO JL=KIDIA,KFDIA
    PAPHIF (JL,JK) = PAPHIF (JL,JK) + ZGEOM1 (JL,JK)
    ZGEOM1 (JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!*       1.2   CHANGE UNITS/SCALINGS
DO JL=KIDIA,KFDIA
  PLZ0H(JL) = PLZ0H(JL) + ZAZ0H (JL)*EXP(ZLZ0H55(JL))
  ZAZ0H (JL)  = 0.0_JPRB
  PGZ0M (JL) = PGZ0M (JL) + ZAZ0M (JL)/RG
  ZAZ0M (JL)  = 0.0_JPRB
ENDDO

!*       1.1   FIRST TIME-STEP
IF (KSTEP == KSTART) THEN
  DO JL=KIDIA,KFDIA
    PACPR (JL)=0.0_JPRB
    PGZ0M (JL)=0.0_JPRB
    PLZ0H (JL)=0.0_JPRB
  ENDDO
ENDIF

IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_TEND_AD( KIDIA, KFDIA, KLEV, KLON, &
   &  ZLRCH4, PGFL, PTENGFL, ZCEN, ZTENC )
ENDIF


IF (NGEMS > 0) THEN
  DEALLOCATE( ZTENC55D )
  DEALLOCATE( ZTENC55B )
ENDIF

IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_DEALLOC( ZCEN5, ZTENC5, ZTENC5_SKF, ZCFLX5, ZSCAV5, ZGRGTEND)
  CALL GEMS_DEALLOC_AD( ZCEN, ZTENC, ZCFLX)
ELSE
  DEALLOCATE(ZCEN5,ZTENC5,ZCFLX5)
  DEALLOCATE(ZCEN,ZTENC,ZCFLX)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CALLPARAD',1,ZHOOK_HANDLE)
END SUBROUTINE CALLPARAD
