SUBROUTINE CALLPARTL (KIDIA,KFDIA,KLON,KLEVS,KTILES,&
 & KSTGLO,&
 & KTDIA,KLEV,KSTART,KSTEP,LDRAIN1D,&
 !-----------------------------------------------------------------------
 ! - INPUT (TRAJECTORY) .
 & PT5    , PQ5, PGFL5, &
 & PU5    , PV5    , PVERVEL5,&
 & PRS15  , PRSF15,&
 & PAPRS5 , PAPRSF5, PDELP5,&
 & PAPHI5 , PAPHIF5,&
 & PEMTD5 , PTRSO5 , PMU0,   PMU0M,&
 & PSNS5  , PTSA5  , PWL5,   PTL5,&
 & PWSA5  , PTIA5,&
 & PASN5  , PRSN5  , PTSN5,&
 & PHSTD  , PHSDFOR, PGAMMA,&
 & PTHETA , PSIG,&
 & PLSM   , PALBF  , &
 & PALUVP , PALUVD , PALNIP, PALNID,&
 & PEMISF,&
 & PCO2O, PCO2B, PCO2A,PCO2F,PCH4,PCH4F,PSF6,PGFLFLX,&
 & PCVL   , PCVH   , PTVL  , PTVH , PLAILC,  PLAIHC,&
 & PSOTY , PSOALUVP, PSOALUVD, PSOALNIP, PSOALNID,&
 & PCI , PSST,&
 & PGELAM , PCLON  , PSLON ,&
 & PGZ0MCL  ,PLZ0HCL, PGEMU, POROG, &
 & PGZ0M5 , PLZ0H5,&
 & PU10N5 ,  PV10N5,&
 ! - UPDATED (TRAJECTORY)
 & PTENT5 , PTENQ5, PTENGFL5,&
 & PTENU5 , PTENV5,&
 & PNTOP  , PNBAS  , PACPR5,&
 & PUSTRTI5, PVSTRTI5, PAHFSTI5,PEVAPTI5 ,PTSKTI5,&
 !====================================================================     
 ! - INPUT .
 & PT     , PQ, PGFL,&
 & PU     , PV     , PVERVEL,&
 & PL     , PI     , PA     ,&
 & PRS1   , PRSF1,&
 & PAPRS  , PAPRSF , PDELP,&
 & PAPHI  , PAPHIF,&
 & PSNS   , PTSA   , PWL  , PTL,&
 & PWSA,&
 ! - UPDATED
 & PTENT  , PTENQ, PTENGFL,&
 & PTENU  , PTENV,&
 & PACPR,&
 & PUSTRTI, PVSTRTI, PAHFSTI,PEVAPTI ,PTSKTI,&
 ! - OUTPUT
 & PDIFTQ , PDIFTS , PSTRTU , PSTRTV,&
 & PGZ0M  , PLZ0H,&
 & PU10N  ,  PV10N ,&
 & PTLE1,YD_MWAVE_RAD) 

!**** *CALLPARTL * - CALL ECMWF PHYSICS (Tangent linear)

!     PURPOSE.
!     --------
!     - CALL THE SUBROUTINES OF THE E.C.M.W.F. PHYSICS PACKAGE.

!**   Interface.
!     ----------
!        *CALL* *CALLPARTL*

!-----------------------------------------------------------------------

! -   INPUT ARGUMENTS.
!     -------------------

! - DIMENSIONS ETC.

! KIDIA   : START OF HORIZONTAL LOOP
! KFDIA   : START OF HORIZONTAL LOOP
! KLON    : HORIZONTAL DIMENSION
! KLEVS   : NUMBER OF LEVELS IN SOIL
! KTILES  : NUMBER OF vegetation (surface cover) types
! KTDIA   : START OF THE VERTICAL LOOP IN THE PHYSICS
!            (IF SOME LEVELS ARE SKIPPED AT THE TOP OF THE MODEL).
! KLEV    : END OF VERTICAL LOOP AND VERTICAL DIMENSION
! KSTART  : FIRST STEP OF MODEL
! KSTEP   : CURRENT TIME STEP

!-----------------------------------------------------------------------

! - 2D (0:KLEV) .

! PAPHI5     : GEOPOTENTIAL ON HALF-LEVELS.             (Trajectory)
! PAPRS5     : PRESSURE ON HALF-LEVELS.                 (Trajectory)
! PRS15      : PROVISIONAL T+DT PRESSURE ON HALF LEVELS (Trajectory)
! PEMTD5     : DOWNWARD LONGWAVE EMISSIVITY             (Trajectory)
! PTRSO5     : SHORTWAVE TRANSMISSIVITY                 (Trajectory)

! - 2D (1:KLEV) .

! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.             (Trajectory)
! PAPRSF5    : PRESSURE ON FULL LEVELS.                 (Trajectory)
! PDELP5     : LAYER THICKNESS IN PRESSURE UNITS.       (Trajectory)
! PT5        : TEMPERATURE.                             (Trajectory)
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOUR.       (Trajectory)
! PU5        : X-COMPONENT OF WIND                      (Trajectory)
! PV5        : Y-COMPONENT OF WIND                      (Trajectory)
! PVERVEL5   : VERTICAL VELOCITY                        (Trajectory)
! PRSF15     : PROVISIONAL T+DT PRESSURE ON FULL LEVELS (Trajectory)
! PGFL5      : GFL FIELDS                               (Trajectory)

! - 1D (PROGNOSTIC QUANTITIES) .

! PSNS5      : MASS OF SNOW PER UNIT SURFACE.           (Trajectory)
! PTSA5      : MULTI-LAYER SOIL TEMPERATURE             (Trajectory)
! PWSA5      : MULTI-LAYER SOIL WETNESS                 (Trajectory)
! PTIA5      : MULTI-LAYER ICE TEMPERATURE              (Trajectory)
! PWL5       : SKIN RESERVOIR WATER CONTENT             (Trajectory) 
! PTL5       : SKIN (BRIGHTNESS) TEMPERATURE            (Trajectory)
! PASN5      : SNOW ALBEDO                              (Trajectory)
! PRSN5      : SNOW DENSITY                             (Trajectory)
! PTSN5      : SNOW TEMPERATURE                         (Trajectory)

! - 1D (GEOGRAPHICAL DISTRIBUTION) .

! PCLON      : COSINE OF LONGITUDE
! PSLON      : SINE   OF LONGITUDE
! PALBF      : BACKGROUND SURFACE SHORTWAVE ALBEDO (SNOW-FREE).
! PALUVP     : MODIS ALBEDO UV-VIS PARALLEL
! PALUVD     : MODIS ALBEDO UV-VIS DIFFUSE
! PALNIP     : MODIS ALBEDO NEAR IR PARALLEL
! PALNID     : MODIS ALBEDO NEAR IR DIFFUSE
! PCVL       : LOW VEGETATION COVER
! PCVH       : HIGH VEGETATION COVER
! PTVL       : LOW VEGETATION DOMINANT TYPE
! PTVH       : HIGH VEGETATION DOMINANT TYPE
! PLAILC     : LOW VEGETATION LAI (VALUE FROM CLIMATE FILE)
! PLAIHC     : HIGH VEGETATION LAI (VALUE FROM CLIMATE FILE)
! PSOTY      : SOIL TYPE
! PSOALUVP    : SOIL ALBEDO UV/VIS DIRECT
! PSOALUVD    : SOIL ALBEDO UV/VIS DIFFUSE
! PSOALNIP    : SOIL ALBEDO NIR DIRECT
! PSOALNID    : SOIL ALBEDO NIR DIFFUSE
! PCI        : SEA-ICE FRACTION
! PSST       : (OPEN) SEA SURFACE TEMPERATURE
! PRCORI     : CORIOLIS PARAMETER.
! PEMISF     : BACKGROUND SURFACE LONGWAVE EMISSIVITY (SNOW-FREE).
! PLSM       : LAND/SEA MASK.
! PCO2O      : Ocean CO2 flux
! PCO2B      : Biosphere CO2 flux
! PCO2A      : Anthropogenic CO2 flux
! PCO2F      : Biomass burning CO2 flux
! PCH4       : Agrregated CH4 surface fluxes
! PCH4F      : CH4 fire emissions
! PSF6       : SF6 emissions
! PGFLFLX    : flux for GRG and TRAC species
! PGZ0MCL    : GRAVITY * ROUGHNESS LENGTH
! PMU0       : LOCAL COSINE OF INSTANTANEOUS SOLAR ZENITH ANGLE.
! PMU0M      : LOCAL COSINE OF INSTANTANEOUS MEAN SOLAR ZENITH ANGLE.
! PSIG       : MEAN SUBGRID SLOPE
! PGAMMA     : ANISOTROPY OF SUBGRID OROGRAPHY ("ASPECT RATIO")
! PTHETA     : DIRECTION (ANGLE) WHERE MEAN SQUARE GRADIENT IS LARGEST
! PHSTD      : STANDARD DEVIATION OF SUBGRID OROGRAPHY
! PHSDFOR    : NEW STANDARD DEVIATION OF SUBGRID OROGRAPHY
! PLZ0HCL    : LOGARITHM OF ROUGHNESS LENGTH FOR HEAT
! PGEMU      : SINE OF LATITUDE
! POROG      : OROGRAPHY

!-----------------------------------------------------------------------

! -   UPDATED ARGUMENTS.
!     --------------------

! - 2D (1:KLEV) .

! PTENT5     : TENDENCY OF TEMPERATURE.                 (Trajectory)
! PTENQ5     : TENDENCY OF HUMIDITY                     (Trajectory)
! PTENU5     : TENDENCY OF U-COMP. OF WIND.             (Trajectory)
! PTENV5     : TENDENCY OF U-COMP. OF WIND.             (Trajectory)
! PTENGFL5   : TENDENCY OF ALL GFL FIELDS               (Trajectory)
! PNTOP      : CONVECTIVE CLOUD TOP INDEX (TO BE CONVERTED TO INTEGER)
! PNBAS      : CONVECTIVE CLOUD BASE INDEX (TO BE CONVERTED TO INTEGER)
! PACPR5     : CONVECTIVE RAINFALL ACCUMULATED BETWEEN TWO
!              (FULL) RADIATION TIME STEPS
! PUSTRTI5   : E-W (INSTANTANEOUS) SURFACE STRESS 
!              FOR EACH TILE                            (Trajectory)
! PVSTRTI5   : N-S (INSTANTANEOUS) SURFACE STRESS 
!              FOR EACH TILE                            (Trajectory)
! PAHFSTI5   : (INSTANTANEOUS) SURFACE SENSIBLE HEAT FLUX 
!              FOR EACH TILE                            (Trajectory)
! PEVAPTI5   : (INSTANTANEOUS) EVAPORATION 
!              FOR EACH TILE                            (Trajectory)
! PTSKTI5    : SKIN TEMPERATURE FOR EACH TILE           (Trajectory)

!-----------------------------------------------------------------------

! - 2D (0:KLEV) .

! PAPHI      : GEOPOTENTIAL ON HALF-LEVELS.
! PAPRS      : PRESSURE ON HALF-LEVELS.
! PRS1       : PROVISIONAL T+DT PRESSURE ON HALF LEVELS

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF     : PRESSURE ON FULL LEVELS.
! PDELP      : LAYER THICKNESS IN PRESSURE UNITS.
! PT         : TEMPERATURE.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOUR.
! PU         : X-COMPONENT OF WIND
! PV         : Y-COMPONENT OF WIND
! PVERVEL    : VERTICAL VELOCITY
! PRSF1      : PROVISIONAL T+DT PRESSURE ON FULL LEVELS.
! PGFL       : GFL FIELDS

! - 1D (PROGNOSTIC QUANTITIES) .

! PSNS       : MASS OF SNOW PER UNIT SURFACE.   
! PTSA       : MULTI-LAYER SOIL TEMPERATURE     
! PWSA       : MULTI-LAYER SOIL WETNESS         
! PWL        : SKIN RESERVOIR WATER CONTENT     
! PTL        : SKIN (BRIGHTNESS) TEMPERATURE    

! - 1D (DIAGNOSTIC QUANTITIES) .

!-----------------------------------------------------------------------

! -   UPDATED ARGUMENTS.
!     --------------------

! - 2D (1:KLEV) .

! PTENT      : TENDENCY OF TEMPERATURE.
! PTENQ      : TENDENCY OF HUMIDITY
! PTENU      : TENDENCY OF U-COMP. OF WIND.     
! PTENV      : TENDENCY OF U-COMP. OF WIND.     
! PTENGFL    : TENDENCY OF ALL GFL FIELDS 
! PACPR      : CONVECTIVE RAINFALL ACCUMULATED BETWEEN TWO
!              (FULL) RADIATION TIME STEPS
! PUSTRTI   : E-W (INSTANTANEOUS) SURFACE STRESS FOR EACH TILE    
! PVSTRTI   : N-S (INSTANTANEOUS) SURFACE STRESS FOR EACH TILE    
! PAHFSTI   : (INSTANTANEOUS) SURFACE SENSIBLE HEAT FLUX FOR EACH TILE        
! PEVAPTI   : (INSTANTANEOUS) EVAPORATION FOR EACH TILE                       
! PTSKTI    : SKIN TEMPERATURE FOR EACH TILE

!-----------------------------------------------------------------------

! -   OUTPUT ARGUMENTS.
!     --------------------

! - 2D (0:KLEV) FLUXES .

! PDIFTQ     : TURBULENT FLUX (INC. Q NEGATIVE) OF SPECIFIC HUMIDITY.
! PDIFTS     : TURBULENT FLUX OF ENTHALPY (OR DRY STATIC ENERGY).
! PSTRTU     : TURBULENT FLUX OF MOMENTUM "U".
! PSTRTV     : TURBULENT FLUX OF MOMENTUM "V".

! - 1D (TENDENCIES FROM SURFACE SCHEME)

! PTLE1      : OF SKIN TEMPERATURE

! - 1D (DIAGNOSTIC) .

! PGZ0M      : G * ROUGHNESS LENGTH (CURRENT).
! PLZ0H      : LOGARITHM OF ROUGHNESS LENGTH FOR HEAT (CURRENT)

!-----------------------------------------------------------------------

!     Externals.  
!     ----------

!     SURFRAD   -
!     SATURTL   -
!     CLOUDTL   -
!     RADHEATTL -
!     VDFMAINSTL -
!     GWDRAGTL  -
!     CUCALLNTL -
!     CUCALLN2TL -
!     CONDTL    - 
!     GWDRAG_WMSSTL

!     Method. See documentaion.
!     -------
!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     J.F. MAHFOUF        E.C.M.W.F.          13/12/95

!     Adapted from CALLPAR for TL and AD versions

!     ORIGINAL 93-10-04 M.HAMRUD/P.VITERBO (FROM APLPAR)

!     MODIFIED 2000-01-21 M. Janiskova (writting the trajectory
!                            for phys.tendecies and radiation)

!     Adapted to tile approach : 01/06/1999 (JFM)
!              2002-10-31 M. Janiskova (new cloud scheme - cloudst)
!        P. Viterbo  24-05-2004  Change surface units
!        M.Janiskova 14-12-2005  Modified initialization of cloud variables
!        P. Lopez + P. Bechtold 11-02-2006  Include different Tracers
!        JJMorcrette 20060625 MODIS albedo
!        G. Balsamo 20070115 Soil type 
!        A. Geer     25-04-2008  Rainy 4D-Var for multiple sensors
!        G. Balsamo  10-10-2008  Lake tile
!        A. Geer     01-10-2008  Rainy 4D-Var: tidying, orography, compatability
!        M.Janiskova 10-04-2008 Frequency of calling LW radiation
!        H. Hersbach   04-Dec-2009 10-m neutral wind
!        S. Boussetta/G.Balsamo  05-2009   Add variable LAI
!        M.Janiskova 16-09-2009 Call to non-orographic gravity wave
!        M.Janiskova 18-05-2010 Modified trajectory saving for LEMWAVE to avoid
!                               re-computation in adjoint
!        A. Geer     07-Apr-2010 cloud / precip fraction for all-sky
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY2  , ONLY : TSPHY
USE YOERAD   , ONLY : NSW      ,NTSW     ,NMODE    , RCCNLND, RCCNSEA
USE YOERDI   , ONLY : RCARDI
USE YOMCST   , ONLY : RG       ,RLVTT    ,RLSTT    ,RTT  ,RCPD, RDAY
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOPHNC   , ONLY : LETRAJPT ,LERADI2  ,LERADS2 ,LERADSW2 ,&
 & LERADN2  ,LERADFL2 ,LERADLW2 ,LEDCLD2 ,LENCLD2 ,LEVDIF2  ,&
 & LEGWDG2  ,LECUMF2  ,LECUBM2  ,LECOND2 ,LEGWWMS2,LEQNGT2  ,&
 & LESURF2  ,LTRACLNPH 
USE YOMIOP   , ONLY : NG3DT95   ,NG3PR95, NG2DSF5
USE YOMCT3   , ONLY : NSTEP
USE YOMNCL   , ONLY : GPCLOUD5, GPCLOUDIN, LNCLIN
USE YOELWRAD , ONLY : RADLWM  , RADLWM5  , NLWFR
USE YOEGWDWMS, ONLY : TGWDWMS  ,TGWDWMS5 , LREGWWMS
USE YOMCT0   , ONLY : NSTOP
USE YOMCUMFS , ONLY : LECUMFS2
USE YOEGWWMS , ONLY : GTPHYGWWMS
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS   ,RSTATI
USE YOMMWAVE , ONLY : MWAVE_PHYS_TYPE, MWAVE_RAD_TYPE, LEMWAVE_ACTIVE, &
                    & LDMWAVE_TEST, N_SATSENS
USE PARMWAVE , ONLY : NMAX_CHAN
USE YOM_YGFL , ONLY : YGFL, YAERO, NAERO, &
 & YGHG, NGHG, YGRG, NGRG, NTRAC, LFIRE, LCH4SFC, YLRCH4, LCO2SFC
USE TYPE_GEMS_PROFILES, ONLY : NGEMS
USE YOMECEARTH , ONLY : LECEARTH

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
LOGICAL           ,INTENT(IN)    :: LDRAIN1D 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL5(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL5(KLON,KLEV,YGFL%NDIM)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRS15(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSF15(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMTD5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTRSO5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0M(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSA5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSA5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTIA5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PASN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSN5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHSTD(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHSDFOR(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGAMMA(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTHETA(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSIG(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALBF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALUVP(KLON), PALUVD(KLON), PALNIP(KLON), PALNID(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMISF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2O(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2B(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2A(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2F(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH4(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH4F(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSF6(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLFLX(KLON,NGRG+NTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVH(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTVL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTVH(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAILC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAIHC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOTY(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALUVP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALUVD(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALNIP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALNID(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCI(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSST(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KLON), PCLON(KLON), PSLON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0MCL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLZ0HCL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLZ0H5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV5(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENGFL5(KLON,KLEV,YGFL%NDIM1)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNTOP(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNBAS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PACPR5(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSTRTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSTRTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAHFSTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVAPTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PL(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PI(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PA(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(KLON,KLEV,YGFL%NDIM)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRS1(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSF1(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSA(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSA(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENGFL(KLON,KLEV,YGFL%NDIM1)
!*GEMSuncomm
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PACPR(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSTRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSTRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAHFSTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVAPTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTQ(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTU(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTV(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10N(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10N(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLZ0H(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTLE1(KLON) 

!     Common arrays for rain/cloud MWave assimilation
TYPE (MWAVE_RAD_TYPE), INTENT (INOUT) :: YD_MWAVE_RAD (KLON, N_SATSENS)

REAL(KIND=JPRB) :: ZDIFTQ5(KLON,0:KLEV),ZDIFTS5(KLON,0:KLEV)

REAL(KIND=JPRB) :: ZUSTRG5(KLON),ZVSTRG5(KLON)
REAL(KIND=JPRB) :: ZUSTRG(KLON),ZVSTRG(KLON)

REAL(KIND=JPRB) :: ZUCFL5(KLON),ZVCFL5(KLON),ZTCFL5(KLON),ZQCFL5(KLON),ZDCFL5(KLON)
REAL(KIND=JPRB) :: ZUCFL(KLON),ZVCFL(KLON),ZTCFL(KLON),ZQCFL(KLON),ZDCFL(KLON)

!*
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: Z_PVDIS5 (KLON)
REAL(KIND=JPRB) :: Z_PVDISG5(KLON)

REAL(KIND=JPRB) :: Z_PVDIS (KLON)
REAL(KIND=JPRB) :: Z_PVDISG(KLON)

REAL(KIND=JPRB) :: Z_PALB5(KLON),Z_PEMIS5(KLON)
REAL(KIND=JPRB) :: Z_PALB (KLON),Z_PEMIS (KLON)
REAL(KIND=JPRB) :: Z_PFTG125(KLON),Z_PFTLHEV5(KLON),Z_PFTLHSB5(KLON),&
 & Z_PFWEV5(KLON) ,&
 & Z_PFWG125(KLON),Z_PFWMLT5(KLON),Z_PFWSB5(KLON),&
 & Z_PFWROD5(KLON),Z_PFWRO15(KLON)  

REAL(KIND=JPRB) :: Z_PBLH5(KLON) 

REAL(KIND=JPRB) :: Z_PFTG12(KLON),Z_PFTLHEV(KLON),Z_PFTLHSB(KLON),&
 & Z_PFWEV(KLON) ,&
 & Z_PFWG12(KLON),Z_PFWMLT(KLON),Z_PFWSB(KLON),&
 & Z_PFWROD(KLON),Z_PFWRO1(KLON)  

REAL(KIND=JPRB) :: Z_PSNSE15(KLON),Z_PWLE15(KLON),Z_PTLE15(KLON)
REAL(KIND=JPRB) :: Z_PTSAE15(KLON,KLEVS),Z_PWSAE15(KLON,KLEVS)

REAL(KIND=JPRB) :: Z_PSNSE1(KLON),Z_PWLE1(KLON)
REAL(KIND=JPRB) :: Z_PTSAE1(KLON,KLEVS),Z_PWSAE1(KLON,KLEVS)

REAL(KIND=JPRB) :: Z_PDIFCQ5(KLON,0:KLEV),&
 & Z_PDIFCS5(KLON,0:KLEV),&
 & Z_PFCQNG5(KLON,0:KLEV),Z_PFPLCL5(KLON,0:KLEV),&
 & Z_PFPLCN5(KLON,0:KLEV),&
 & Z_PFPLSL5(KLON,0:KLEV),Z_PFPLSN5(KLON,0:KLEV),&
 & Z_PFHPCL5(KLON,0:KLEV),&
 & Z_PFHPCN5(KLON,0:KLEV),Z_PFHPSL5(KLON,0:KLEV),&
 & Z_PFHPSN5(KLON,0:KLEV),&
 & Z_PFRSO5 (KLON,0:KLEV),Z_PFRTH5 (KLON,0:KLEV),&
 & Z_PFRSOD5(KLON)       ,Z_PFRTHD5(KLON),&
 & Z_PSTRCU5(KLON,0:KLEV),&
 & Z_PSTRCV5(KLON,0:KLEV),Z_PSTRDU5(KLON,0:KLEV),&
 & Z_PSTRDV5(KLON,0:KLEV),&
 & Z_PSTRTU5(KLON,0:KLEV),Z_PSTRTV5(KLON,0:KLEV)  
REAL(KIND=JPRB) :: Z_PFCCQN5(KLON,0:KLEV), Z_PFCCQL5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZFLXWGWU5(KLON,0:KLEV), ZFLXWGWV5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTENU5(KLON,KLEV), ZTENV5(KLON,KLEV), ZTFIELD5(KLON,KLEV)

REAL(KIND=JPRB) :: Z_PDIFCQ(KLON,0:KLEV),&
 & Z_PDIFCS(KLON,0:KLEV),&
 & Z_PFCQNG(KLON,0:KLEV),Z_PFPLCL(KLON,0:KLEV),Z_PFPLCN(KLON,0:KLEV),&
 & Z_PFPLSL(KLON,0:KLEV),Z_PFPLSN(KLON,0:KLEV),Z_PFHPCL(KLON,0:KLEV),&
 & Z_PFHPCN(KLON,0:KLEV),Z_PFHPSL(KLON,0:KLEV),Z_PFHPSN(KLON,0:KLEV),&
 & Z_PFRSO (KLON,0:KLEV),Z_PFRTH (KLON,0:KLEV),&
 & Z_PFRSOD(KLON)       ,Z_PFRTHD(KLON),&
 & Z_PSTRCU(KLON,0:KLEV),&
 & Z_PSTRCV(KLON,0:KLEV),Z_PSTRDU(KLON,0:KLEV),Z_PSTRDV(KLON,0:KLEV)  
REAL(KIND=JPRB) :: Z_PFCCQN(KLON,0:KLEV), Z_PFCCQL(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZFLXWGWU(KLON,0:KLEV), ZFLXWGWV(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTENU(KLON,KLEV), ZTENV(KLON,KLEV), ZTFIELD(KLON,KLEV)

REAL(KIND=JPRB) :: Z_PNEB5(KLON,KLEV), Z_PQLI5(KLON,KLEV),&
 & Z_PQSAT5(KLON,KLEV),Z_PQICE5(KLON,KLEV),Z_PTCC5 (KLON)  
REAL(KIND=JPRB) :: Z_PFRSOC5(KLON,0:1),Z_PFRTHC5(KLON,0:1)

REAL(KIND=JPRB) :: Z_PNEB(KLON,KLEV), Z_PQLI(KLON,KLEV),&
 & Z_PQSAT(KLON,KLEV),Z_PQICE(KLON,KLEV),Z_PTCC  (KLON)  
REAL(KIND=JPRB) :: Z_PFRSOC(KLON,0:1),Z_PFRTHC(KLON,0:1)

REAL(KIND=JPRB) :: Z_PEMTD (KLON,0:KLEV),Z_PTRSO (KLON,0:KLEV),Z_PSRSWD(KLON)
REAL(KIND=JPRB) ::&
 & Z_PSRSWDCS5(KLON),Z_PSRSWD5(KLON)  

!            0.2  LOCAL ARRAYS FOR ECMWF PHYSICS PACKAGE
!                 --------------------------------------

INTEGER(KIND=JPIM) :: ITOPC(KLON), IBASC(KLON)
INTEGER(KIND=JPIM) :: ITYPE(KLON), ICBOT(KLON), ICTOP(KLON), IBOTSC(KLON)
INTEGER(KIND=JPIM) :: ID0, IM0, IY0, IDINCR,ISEC,IDD,IMM,IYY,ILMON(12)

LOGICAL :: LLLAND(KLON),LLSICE(KLON),LLNH(KLON),LLCUM(KLON)
LOGICAL :: LLSC(KLON)
LOGICAL :: LLLAKE(KLON)
LOGICAL :: LLOCN_KPP(KLON)
!--to store variable LAI--
REAL(KIND=JPRB) :: ZLAIL(KLON),ZLAIH(KLON)

REAL(KIND=JPRB) :: ZALBD5(KLON,NTSW), ZALBP5(KLON,NTSW)
REAL(KIND=JPRB) :: ZAZ0H5(KLON)  , ZAZ0M5 (KLON) , ZHSTD(KLON), ZHSDFOR(KLON)
REAL(KIND=JPRB) :: ZSNM1M5(KLON) , ZRSNM1M5(KLON)
REAL(KIND=JPRB) :: ZWLM1M5(KLON)
REAL(KIND=JPRB) :: ZGEOM15(KLON,KLEV), ZGEOMH5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZLU5(KLON,KLEV)   , ZLUDE5(KLON,KLEV), ZMFU5(KLON,KLEV)
REAL(KIND=JPRB) :: ZMFD5(KLON,KLEV)  
REAL(KIND=JPRB) :: ZPSTRTU5(KLON,0:KLEV),ZPSTRTV5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTRSOD5(KLON)
REAL(KIND=JPRB) :: ZEMIR5(KLON),       ZEMIW5(KLON)
REAL(KIND=JPRB) :: ZLITOT5(KLON,KLEV)    
REAL(KIND=JPRB) :: ZQRAIN5(KLON,KLEV), ZRAINT5(KLON,KLEV)
REAL(KIND=JPRB) :: ZSUDU5(KLON)
REAL(KIND=JPRB) :: ZPA5(KLON,KLEV), ZPL5(KLON,KLEV), ZPI5(KLON,KLEV)
REAL(KIND=JPRB) :: ZPA55(KLON,KLEV), ZPL55(KLON,KLEV), ZPI55(KLON,KLEV)

!---- added for convenience                    
REAL(KIND=JPRB) :: ZTHT5(KLON,KLEV+1)

REAL(KIND=JPRB) :: ZALBD(KLON,NTSW), ZALBP(KLON,NTSW)
REAL(KIND=JPRB) :: ZAZ0H(KLON)  , ZAZ0M (KLON)
REAL(KIND=JPRB) :: ZSNM1M(KLON)
REAL(KIND=JPRB) :: ZWLM1M(KLON)
REAL(KIND=JPRB) :: ZWLMX (KLON)
REAL(KIND=JPRB) :: ZGEOM1(KLON,KLEV), ZGEOMH(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZLU(KLON,KLEV)   , ZLUDE(KLON,KLEV), ZMFU(KLON,KLEV)
REAL(KIND=JPRB) :: ZMFD(KLON,KLEV)  
REAL(KIND=JPRB) :: ZTRSOD(KLON)
REAL(KIND=JPRB) :: ZEMIR(KLON),       ZEMIW(KLON)
REAL(KIND=JPRB) :: ZQRAIN(KLON,KLEV), ZRAINT(KLON,KLEV)
REAL(KIND=JPRB) :: ZPA(KLON,KLEV), ZPL(KLON,KLEV), ZPI(KLON,KLEV)

REAL(KIND=JPRB) :: ZPEMTD5(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTENTVD5 (KLON,KLEV),ZTENQVD5 (KLON,KLEV),&
 & ZTENUVD5 (KLON,KLEV),ZTENVVD5 (KLON,KLEV) 
REAL(KIND=JPRB) :: ZTENTGW5 (KLON,KLEV),&
 & ZTENUGW5 (KLON,KLEV),ZTENVGW5 (KLON,KLEV)  
REAL(KIND=JPRB) :: ZTENTCO5 (KLON,KLEV),ZTENQCO5 (KLON,KLEV),&
 & ZTENUCO5 (KLON,KLEV),ZTENVCO5 (KLON,KLEV)
REAL(KIND=JPRB) :: ZTENTCL5 (KLON,KLEV)
REAL(KIND=JPRB) :: ZTENUWM5 (KLON,KLEV),ZTENVWM5 (KLON,KLEV)
REAL(KIND=JPRB) :: ZCHAR(KLON)
REAL(KIND=JPRB) :: ZTSKRAD5(KLON)
REAL(KIND=JPRB) :: ZTSKRAD(KLON)
!---- added for convenience                    
REAL(KIND=JPRB) :: ZTHT(KLON,KLEV+1)

REAL(KIND=JPRB) :: ZFRTI(KLON,KTILES),ZALBTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZALBTI(KLON,KTILES)
INTEGER(KIND=JPIM) :: ITVL(KLON),ITVH(KLON),ISOTY(KLON)
REAL(KIND=JPRB) ::    ZCVL(KLON),ZCVH(KLON)

REAL(KIND=JPRB) ::    ZCCNL(KLON),ZCCNO(KLON)
REAL(KIND=JPRB) ::    ZWND5(KLON),ZCCNL5(KLON),ZCCNO5(KLON)

REAL(KIND=JPRB) :: ZLRCH4(KLON,KLEV)
REAL(KIND=JPRB) :: ZCOVPTOT5(KLON,KLEV), ZCOVPTOT(KLON,KLEV)
REAL(KIND=JPRB) :: ZALBICE(KLON)

!----------------------------------------------------------------------------------
! For convective/diffusive tracer Transport:

INTEGER(KIND=JPIM)           :: ITRAC
INTEGER(KIND=JPIM)           :: IAERO(NAERO) ! Index array for aerosol tracers
INTEGER(KIND=JPIM)           :: IGRG(NGRG)   ! Index array for reactive gas tracers
INTEGER(KIND=JPIM)           :: ITRACER(NTRAC)   ! Index array for flexible tracers 
REAL(KIND=JPRB), POINTER :: ZCEN5(:,:,:),ZTENC5(:,:,:),ZCEN(:,:,:),&
                          & ZTENC(:,:,:),ZTENC5_SKF(:,:,:), ZSCAV5(:)

REAL(KIND=JPRB), POINTER :: ZCFLX5(:,:), ZCFLX(:,:)
! for GRG in coupled mode
REAL(KIND=JPRB), POINTER :: ZGRGTEND(:,:,:)
!----------------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JK, JL, JSW, IFIELDS
INTEGER(KIND=JPIM) :: IBLK
INTEGER(KIND=JPIM) :: ISTEP, IFREQWMS, ISTEPWMS

!*     Local arrays for rain/cloud MWave assimilation
REAL (KIND=JPRB), TARGET :: ZPHYS_P_FW    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PH_FW   (KLON,KLEV+1)
REAL (KIND=JPRB), TARGET :: ZPHYS_T_FW    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_Q_FW    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CC_FW   (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CWC_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CIC_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_RFL_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_SFL_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CRFL_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CSFL_FW  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PFRC_FW (KLON,KLEV)
    
REAL (KIND=JPRB), TARGET :: ZPHYS_P_TL    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PH_TL   (KLON,KLEV+1)
REAL (KIND=JPRB), TARGET :: ZPHYS_T_TL    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_Q_TL    (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CC_TL   (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CWC_TL  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CIC_TL  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_RFL_TL  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_SFL_TL  (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CRFL_TL (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_CSFL_TL (KLON,KLEV)
REAL (KIND=JPRB), TARGET :: ZPHYS_PFRC_TL (KLON,KLEV)
   
TYPE (MWAVE_PHYS_TYPE) :: YL_MWAVE_PHYS_FW (KLON)
TYPE (MWAVE_PHYS_TYPE) :: YL_MWAVE_PHYS_TL (KLON)

REAL(KIND=JPRB) :: ZCARDI,ZRCHAR,ZRCPD,ZTSGWD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! FLAKE: Lake model (Local for PASSIVE introduction)
REAL(KIND=JPRB) :: ZCLAKE5(KLON)
REAL(KIND=JPRB) :: ZHLICE5(KLON)
REAL(KIND=JPRB) :: ZTLICE5(KLON)
REAL(KIND=JPRB) :: ZTLWML5(KLON)


INTERFACE
#include "surfrad.h"
#include "surfbc.h"
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cldpptl.intfb.h"
#include "cloudtl.intfb.h"
#include "cloudsttl.intfb.h"
#include "condtl.intfb.h"
#include "cucallntl.intfb.h"
#include "cucalln2tl.intfb.h"
#include "gems_dealloc_tl.intfb.h"
#include "gems_dealloc.intfb.h"
#include "gems_init.intfb.h"
#include "gems_init_tl.intfb.h"
#include "gems_tend.intfb.h"
#include "gems_tend_tl.intfb.h"
#include "gwdragtl.intfb.h"
#include "gwdrag_wmsstl.intfb.h"
#include "radheattl.intfb.h"
#include "radinatl.intfb.h"
#include "saturtl.intfb.h"
#include "updcal.intfb.h"
#include "vdfmainstl.intfb.h"
#include "wrphtrajt.intfb.h"
#include "mwave_obsop_tl.intfb.h"
#include "mwave_obsop_test.intfb.h"
!#include "icestateoasis3.intfb.h"

!     ------------------------------------------------------------------
#include "fcttre.h"
#include "fcttim.h"

!     ------------------------------------------------------------------

!*         1.     INITIAL COMPUTATIONS.
!                 ---------------------

IF (LHOOK) CALL DR_HOOK('CALLPARTL',0,ZHOOK_HANDLE)

! Set up constants required
ZRCPD=1.0_JPRB/RCPD

! Initialize Ocean and Lake variable for passive introduction  
LLOCN_KPP(:)=.FALSE.
LLLAKE(:)=.FALSE.
ZCLAKE5(:)=0.0_JPRB
ZHLICE5(:)=0.0_JPRB
ZTLICE5(:)=0.0_JPRB
ZTLWML5(:)=0.0_JPRB

ZCOVPTOT5(:,:)=0.0_JPRB
ZCOVPTOT (:,:)=0.0_JPRB

!*  Rain/cloud MWave assimilation
IF (LEMWAVE_ACTIVE) THEN
    DO JL = KIDIA, KFDIA
       YL_MWAVE_PHYS_FW (JL) % NLEVELS   = KLEV   
       
       YL_MWAVE_PHYS_FW (JL) % P   => ZPHYS_P_FW   (JL,:)
       YL_MWAVE_PHYS_FW (JL) % PH  => ZPHYS_PH_FW  (JL,:)
       YL_MWAVE_PHYS_FW (JL) % T   => ZPHYS_T_FW   (JL,:)
       YL_MWAVE_PHYS_FW (JL) % Q   => ZPHYS_Q_FW   (JL,:)
       YL_MWAVE_PHYS_FW (JL) % RFL => ZPHYS_RFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % SFL => ZPHYS_SFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CRFL => ZPHYS_CRFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CSFL => ZPHYS_CSFL_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CWC => ZPHYS_CWC_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CIC => ZPHYS_CIC_FW (JL,:)
       YL_MWAVE_PHYS_FW (JL) % CC  => ZPHYS_CC_FW  (JL,:)
       YL_MWAVE_PHYS_FW (JL) % PFRC=> ZPHYS_PFRC_FW (JL,:)
       
       YL_MWAVE_PHYS_TL (JL) % NLEVELS   = KLEV 
       
       YL_MWAVE_PHYS_TL (JL) % P   => ZPHYS_P_TL   (JL,:)
       YL_MWAVE_PHYS_TL (JL) % PH  => ZPHYS_PH_TL  (JL,:)
       YL_MWAVE_PHYS_TL (JL) % T   => ZPHYS_T_TL   (JL,:)
       YL_MWAVE_PHYS_TL (JL) % Q   => ZPHYS_Q_TL   (JL,:)
       YL_MWAVE_PHYS_TL (JL) % RFL => ZPHYS_RFL_TL (JL,:)
       YL_MWAVE_PHYS_TL (JL) % SFL => ZPHYS_SFL_TL (JL,:)
       YL_MWAVE_PHYS_TL (JL) % CRFL => ZPHYS_CRFL_TL (JL,:)
       YL_MWAVE_PHYS_TL (JL) % CSFL => ZPHYS_CSFL_TL (JL,:)
       YL_MWAVE_PHYS_TL (JL) % CWC => ZPHYS_CWC_TL (JL,:)
       YL_MWAVE_PHYS_TL (JL) % CIC => ZPHYS_CIC_TL (JL,:)
       YL_MWAVE_PHYS_TL (JL) % CC  => ZPHYS_CC_TL  (JL,:)
       YL_MWAVE_PHYS_TL (JL) % PFRC=> ZPHYS_PFRC_TL(JL,:)
    ENDDO
ENDIF

ITRAC=0
IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_INIT( KIDIA, KFDIA, KLEV, KLON, ITRAC, PCO2O, PCO2B, PCO2A, PCO2F, &
     &  PCH4, PCH4F, PSF6, PGFLFLX, PGEMU, PLSM, IGRG, ITRACER, IAERO, ZLRCH4, PGFL5, &
     &  PTENGFL5, ZCEN5, ZTENC5, ZTENC5_SKF, ZCFLX5, ZSCAV5, ZGRGTEND )
  CALL GEMS_INIT_TL( KIDIA, KFDIA, KLEV, KLON, ITRAC, &
   &  ZLRCH4, PGFL, PTENGFL, ZCEN, ZTENC, ZCFLX )
ELSE
! ALLOCATE arrays so they can be passed to VDFOUTER etc. in the non-GEMS case
   ALLOCATE(ZCEN5(1,1,1))
   ALLOCATE(ZCEN(1,1,1))
   ALLOCATE(ZTENC5(1,1,1))
   ALLOCATE(ZTENC(1,1,1))
   ALLOCATE(ZCFLX5(1,1))
   ALLOCATE(ZCFLX(1,1))
   ALLOCATE(ZSCAV5(1))
ENDIF


CALL SURF_INQ(PRCHAR=ZRCHAR)

ZCARDI        = 0.0_JPRB
ZCHAR   (:)   = ZRCHAR
ZQRAIN  (:,:) = 0.0_JPRB
ZQRAIN5 (:,:) = 0.0_JPRB
ZRAINT  (:,:) = 0.0_JPRB
ZRAINT5 (:,:) = 0.0_JPRB
ZLITOT5 (:,:) = 0.0_JPRB

ZLUDE  (:,:) = 0.0_JPRB
ZLUDE5 (:,:) = 0.0_JPRB
ZLU    (:,:) = 0.0_JPRB
ZLU5   (:,:) = 0.0_JPRB

ZPA55  (:,:) = 0.0_JPRB
ZPL55  (:,:) = 0.0_JPRB
ZPI55  (:,:) = 0.0_JPRB

!  Initialization of radiation perturbation

Z_PTRSO (:,:) = 0.0_JPRB
Z_PEMTD (:,:) = 0.0_JPRB
Z_PFRSO(:,:) = 0.0_JPRB
Z_PFRTH(:,:) = 0.0_JPRB
Z_PFRSOD(:)  = 0.0_JPRB
Z_PFRTHD(:)  = 0.0_JPRB

! Initialization of phys. tendencies

ZTENTVD5 (:,:) = 0.0_JPRB
ZTENQVD5 (:,:) = 0.0_JPRB
ZTENUVD5 (:,:) = 0.0_JPRB
ZTENVVD5 (:,:) = 0.0_JPRB
ZTENTGW5 (:,:) = 0.0_JPRB
ZTENUGW5 (:,:) = 0.0_JPRB
ZTENVGW5 (:,:) = 0.0_JPRB
ZTENTCO5 (:,:) = 0.0_JPRB
ZTENQCO5 (:,:) = 0.0_JPRB
ZTENUCO5 (:,:) = 0.0_JPRB
ZTENVCO5 (:,:) = 0.0_JPRB
ZTENTCL5 (:,:) = 0.0_JPRB

ZPEMTD5  (:,:) = 0.0_JPRB

!*       1.1   FIRST TIME-STEP
IF (KSTEP == KSTART) THEN
  DO JL=KIDIA,KFDIA
    PNTOP(JL)=1.0_JPRB
    PNBAS(JL)=1.0_JPRB
    PACPR (JL)=0.0_JPRB
    PACPR5(JL)=0.0_JPRB

    PGZ0M (JL)=0.0_JPRB
    PGZ0M5(JL)=PGZ0MCL(JL)
    PLZ0H (JL)=0.0_JPRB
    PLZ0H5(JL)=PLZ0HCL(JL)
  ENDDO
ENDIF

!*       1.2   CHANGE UNITS/SCALINGS
DO JL=KIDIA,KFDIA
  ZAZ0M (JL)  = PGZ0M (JL) /RG
  ZAZ0M5(JL)  = PGZ0M5(JL) /RG
  ZAZ0H (JL)  = PLZ0H(JL)*EXP(PLZ0H5(JL))
  ZAZ0H5(JL)  = EXP(PLZ0H5(JL))
  ZSNM1M (JL) = PSNS (JL) !present but not used since PSNS is not perturbed
  ZSNM1M5(JL) = PSNS5(JL)
!  ZRSNM1M(JL) = PRSN (JL) !not present since PRSN is not perturbed
  ZRSNM1M5(JL)= PRSN5(JL)
  ZWLM1M (JL) = PWL  (JL)
  ZWLM1M5(JL) = PWL5 (JL)
  IF (PLSM(JL) > 0.5_JPRB) THEN
    ZHSTD (JL) = PHSTD(JL) /RG
    ZHSDFOR(JL) = PHSDFOR(JL)  
  ELSE
    ZHSTD (JL) = 0._JPRB
    PSIG  (JL) = 0._JPRB
    ZHSDFOR(JL) = 0._JPRB 
  ENDIF
ENDDO

!*      1.3   GLOBAL AUXILLIARY VARIABLES

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZGEOM1 (JL,JK)=PAPHIF (JL,JK)
    ZGEOM15(JL,JK)=PAPHIF5(JL,JK)-PAPHI5(JL,KLEV)
  ENDDO
ENDDO
DO JK=0,KLEV
  DO JL=KIDIA,KFDIA
    ZGEOMH (JL,JK)=PAPHI (JL,JK)
    ZGEOMH5(JL,JK)=PAPHI5(JL,JK)-PAPHI5(JL,KLEV)
  ENDDO
ENDDO

!*      1.3a   AUXILLIARY VARIABLES FOR VDF, SRF AND SURFRAD

CALL SURFBC (KIDIA=KIDIA,KFDIA=KFDIA,KLON=KLON,KTILES=KTILES,&
 & PTVL=PTVL,PTVH=PTVH,PSOTY=PSOTY,&
 & PSOALUVP=PSOALUVP,PSOALUVD=PSOALUVD,PSOALNIP=PSOALNIP,PSOALNID=PSOALNID,&
 & PSDOR=ZHSTD,PCVLC=PCVL,PCVHC=PCVH,&
 & PLAILC=PLAILC, PLAIHC=PLAIHC,&
 & PLSM=PLSM,PCI=PCI,PCLAKE=ZCLAKE5,PHLICE=ZHLICE5,&
 & PGEMU=PGEMU,PSNM1M=ZSNM1M5,PWLM1M=ZWLM1M5,PRSNM1M=ZRSNM1M5,&
 & LDLAND=LLLAND,LDSICE=LLSICE,LDLAKE=LLLAKE,LDNH=LLNH,LDOCN_KPP=LLOCN_KPP,&
 & KTVL=ITVL,KTVH=ITVH,KSOTY=ISOTY,&
 & PCVL=ZCVL,PCVH=ZCVH,PLAIL=ZLAIL,PLAIH=ZLAIH,&
 & PWLMX=ZWLMX,PFRTI=ZFRTI,&
 & PALBF=PALBF,PALUVP=PALUVP,PALUVD=PALUVD,PALNIP=PALNIP,PALNID=PALNID)

DO JL=KIDIA,KFDIA
  ITOPC(JL)=INT(PNTOP(JL)+0.01_JPRB)
  IBASC(JL)=INT(PNBAS(JL)+0.01_JPRB)
ENDDO

ZALBICE(KIDIA:KFDIA) = 0.0_JPRB
!IF (LECEARTH) THEN
!   CALL ICESTATEOASIS3(KSTGLO,KIDIA,KFDIA,PALBICE=ZALBICE(KIDIA:KFDIA))
!END IF

!*       1.4  SET SURFACE TENDENCIES TO ZERO
DO JK=1,KLEVS
  DO JL=KIDIA,KFDIA
    Z_PTSAE1 (JL,JK)=0.0_JPRB
    Z_PTSAE15(JL,JK)=0.0_JPRB
    Z_PWSAE1 (JL,JK)=0.0_JPRB
    Z_PWSAE15(JL,JK)=0.0_JPRB
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  PTLE1 (JL)=0.0_JPRB
  Z_PTLE15(JL)=0.0_JPRB
  Z_PWLE1 (JL)=0.0_JPRB
  Z_PWLE15(JL)=0.0_JPRB
  Z_PSNSE1 (JL)=0.0_JPRB
  Z_PSNSE15(JL)=0.0_JPRB
ENDDO

!*       1.5  SET FLUXES TO ZERO
DO JK=0,KLEV
  DO JL=KIDIA,KFDIA
    Z_PFCCQL (JL,JK) = 0.0_JPRB
    Z_PFCCQL5(JL,JK) = 0.0_JPRB
    Z_PFCCQN (JL,JK) = 0.0_JPRB
    Z_PFCCQN5(JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!*       1.6  SET SURFACE SOLAR TRANSMISSIVITY

Z_PSRSWDCS5(:) = 0.0_JPRB
Z_PSRSWD5(:) = 0.0_JPRB
Z_PSRSWD(:) = 0.0_JPRB
DO JL=KIDIA,KFDIA
  ZSUDU5 (JL)  = Z_PSRSWDCS5(JL)
  ZTRSOD (JL) = Z_PSRSWD(JL)
  ZTRSOD5(JL) = Z_PSRSWD5(JL)
ENDDO

!     ------------------------------------------------------------------

!*         2.     RADIATION TRANSFER
!                 ------------------

IF ( LERADI2 ) THEN

!*         2.1  FULL RADIATION COMPUTATIONS

!*         2.2 SURFACE BOUNDARY CONDITIONS
  IF (LERADS2) THEN

    DO JL=KIDIA,KFDIA
      ZWND5(JL)=SQRT(PU5(JL,KLEV)*PU5(JL,KLEV)+PV5(JL,KLEV)*PV5(JL,KLEV))
    ENDDO  

    IY0=NCCAA(NINDAT)
    IM0=NMM(NINDAT)
    ID0=NDD(NINDAT)
    IDINCR=(NSSSSS+NINT(RSTATI))/NINT(RDAY)
    ISEC=MOD(NSSSSS+NINT(RSTATI),NINT(RDAY))
    CALL UPDCAL(ID0,IM0,IY0,IDINCR,IDD,IMM,IYY,ILMON,-1)
!    CALL SURFRAD(KDD=IDD,KMM=IMM,KMON=ILMON,KSECO=ISEC,&
!     & KIDIA=KIDIA,KFDIA=KFDIA,KLON=KLON,KTILES=KTILES,KSW=NSW,&
!     & LDNH=LLNH,&
!     & PALBF=PALBF,PALBICEF=ZALBICE,PTVH=PTVH,&
!     & PALUVP=PALUVP,PALUVD=PALUVD,PALNIP=PALNIP,PALNID=PALNID,&
!     & PASN=PASN5,PMU0=PMU0M,PTS=PTL5,PWND=ZWND5,&
!     & PWS1=PWSA5(:,1),KSOTY=ISOTY,PFRTI=ZFRTI,PHLICE=ZHLICE5,PTLICE=ZTLICE5,&
!     & PALBD=ZALBD5(:,1:NSW),PALBP=ZALBP5(:,1:NSW),PALB=Z_PALB5,&
!     & PEMIR=ZEMIR5,PEMIW=ZEMIW5,PEMIT=Z_PEMIS5,&
!     & PALBTI=ZALBTI5,PCCNL=ZCCNL5,PCCNO=ZCCNO5 )  

!*      Set perturbations to zero (no TL version of SURFRAD)
!       --------------------------------------------------

    Z_PALB  (:)  =0.0_JPRB
    ZALBD (:,:)=0.0_JPRB
    ZALBP (:,:)=0.0_JPRB
    ZALBTI(:,:)=0.0_JPRB
    Z_PEMIS (:)  =0.0_JPRB
    ZEMIR (:)  =0.0_JPRB
    ZEMIW (:)  =0.0_JPRB
    ZCCNL (:)  =0.0_JPRB
    ZCCNO (:)  =0.0_JPRB

  ELSE
    DO JSW=1,NTSW
      ZALBD (KIDIA:KFDIA,JSW)=0.0_JPRB
      ZALBD5(KIDIA:KFDIA,JSW)=PALBF(KIDIA:KFDIA)
      ZALBP (KIDIA:KFDIA,JSW)=0.0_JPRB  
      ZALBP5(KIDIA:KFDIA,JSW)=PALBF(KIDIA:KFDIA)
    ENDDO
    DO JL=KIDIA,KFDIA
      Z_PALB (JL)=0.0_JPRB
      Z_PALB5(JL)=PALBF(JL)
      Z_PEMIS (JL)=0.0_JPRB
      Z_PEMIS5(JL)=PEMISF(JL)
      ZEMIR (JL)=0.0_JPRB
      ZEMIR5(JL)=PEMISF(JL)
      ZEMIW (JL)=0.0_JPRB
      ZEMIW5(JL)=PEMISF(JL)
      ZCCNL (JL)=0.0_JPRB
      ZCCNL5(JL)=RCCNLND
      ZCCNO (JL)=0.0_JPRB
      ZCCNO5(JL)=RCCNSEA
    ENDDO
    DO JK=1,KTILES
      DO JL=KIDIA,KFDIA
        ZALBTI (JL,JK)=0.0_JPRB
        ZALBTI5(JL,JK)=PALBF(JL)
      ENDDO
    ENDDO
  ENDIF

  CALL SATURTL (KIDIA , KFDIA , KLON  , KTDIA , KLEV,&
   & PAPRSF5, PT5    , Z_PQSAT5,&
   & PAPRSF , PT     , Z_PQSAT  )  

!*         2.3 CLOUD
  IF (LEDCLD2 .AND. .NOT. LENCLD2) THEN
    CALL CLOUDTL ( KIDIA    , KFDIA , KLON   , KLEV,&
     & IBASC    , ITOPC,&
     & PQ5      , Z_PQSAT5, PT5    , PACPR5,&
     & Z_PNEB5    , Z_PTCC5 , Z_PQICE5 , Z_PQLI5,&
     & PQ       , Z_PQSAT , PT     , PACPR,&
     & Z_PNEB     , Z_PTCC  , Z_PQICE  , Z_PQLI    )  

  ELSEIF (LENCLD2) THEN

! read variables from buffers
    IBLK=(KSTGLO-1)/KLON + 1
    IF (KSTEP == KSTART) THEN
      IF (.NOT. LNCLIN) THEN
        DO JK=1,KLEV
          DO JL=KIDIA,KFDIA
            ZPA55(JL,JK) = 0.0_JPRB
            ZPL55(JL,JK) = 0.0_JPRB
            ZPI55(JL,JK) = 0.0_JPRB
            ZPA  (JL,JK) = 0.0_JPRB
            ZPL  (JL,JK) = 0.0_JPRB
            ZPI  (JL,JK) = 0.0_JPRB
          ENDDO
        ENDDO
      ELSE
        DO JK=1,KLEV
          DO JL=KIDIA,KFDIA
            ZPA55(JL,JK) = GPCLOUDIN(JL,JK,IBLK) 
            ZPL55(JL,JK) = GPCLOUDIN(JL,KLEV+JK,IBLK)
            ZPI55(JL,JK) = GPCLOUDIN(JL,2*KLEV+JK,IBLK)
            ZPA  (JL,JK) = 0.0_JPRB
            ZPL  (JL,JK) = 0.0_JPRB
            ZPI  (JL,JK) = 0.0_JPRB
          ENDDO
        ENDDO
      ENDIF
    ELSE
      DO JK=1,KLEV
        DO JL=KIDIA,KFDIA
          ZPA55(JL,JK) = GPCLOUD5(JL,JK,IBLK)
          ZPL55(JL,JK) = GPCLOUD5(JL,KLEV+JK,IBLK)
          ZPI55(JL,JK) = GPCLOUD5(JL,2*KLEV+JK,IBLK)
          ZPA  (JL,JK) = PA(JL,JK)
          ZPL  (JL,JK) = PL(JL,JK)
          ZPI  (JL,JK) = PI(JL,JK)
        ENDDO
      ENDDO
    ENDIF
! end of read variables

    CALL  CLDPPTL &
     & ( KIDIA  , KFDIA  , KLON  , KLEV,&
     & ZPA55  , ZPL55  , ZPI55,&
     & Z_PNEB5  , Z_PQICE5 , Z_PQLI5,&
     & ZPA    , ZPL    , ZPI,&
     & Z_PNEB   , Z_PQICE  , Z_PQLI  )        

  ELSE
    Z_PNEB  (:,:)=0.0_JPRB
    Z_PNEB5 (:,:)=0.0_JPRB
    Z_PQLI  (:,:)=0.0_JPRB
    Z_PQLI5 (:,:)=0.0_JPRB
    Z_PQICE (:,:)=0.0_JPRB
    Z_PQICE5(:,:)=0.0_JPRB
    Z_PTCC  (:)  =0.0_JPRB
    Z_PTCC5 (:)  =0.0_JPRB
  ENDIF

!*         2.4  CALL RADIATION AT EVERY GRID POINT

  IF (LERADSW2 .OR. LERADN2) THEN
    CALL RADINATL &
     & ( KIDIA , KFDIA , KLON , KLEV   , NMODE,&
     & ZALBD5, ZALBP5, PAPRS5 , PAPRSF5,&
     & ZCCNL5, ZCCNO5, RCARDI, Z_PNEB5,&
     & PGELAM, PCLON , PSLON ,&
     & PDELP5, ZEMIR5, ZEMIW5,&
     & PGEMU , PMU0 ,  PQ5    , Z_PQSAT5 ,&
     & Z_PQICE5, Z_PQLI5,&
     & PLSM  , PT5   , PTL5,&
     & ZPEMTD5, PTRSO5,&
     & ZTHT5 , ZTRSOD5,&
     & ZALBD , ZALBP , PAPRS  , PAPRSF,&
     & ZCCNL , ZCCNO , ZCARDI , Z_PNEB,&
     & PDELP , ZEMIR , ZEMIW,&
     & PQ    , Z_PQSAT,&
     & Z_PQICE , Z_PQLI,&
     & PT    , PTL,&
     & Z_PEMTD , Z_PTRSO,&
     & ZTHT  , ZTRSOD)  

    IF (LERADN2) THEN

      IF (LERADLW2) THEN
        ISTEP = MOD(KSTEP,NLWFR)
        IF (ISTEP /= 0 .AND. KSTEP /= 0 .AND. KSTEP < (NSTOP-1)) THEN
          IBLK=(KSTGLO-1)/KLON + 1
          DO JK=0,KLEV
            DO JL=KIDIA,KFDIA
              Z_PEMTD(JL,JK) = RADLWM (JL,JK,IBLK)
              ZPEMTD5 (JL,JK) = RADLWM5(JL,JK,IBLK)
            ENDDO
          ENDDO
        ELSE
          IBLK=(KSTGLO-1)/KLON + 1
          DO JK=0,KLEV
            DO JL=KIDIA,KFDIA
              IF (KSTEP >= (NSTOP-1)) THEN
                RADLWM (JL,JK,IBLK) = 0.
              ELSE
                RADLWM (JL,JK,IBLK) = Z_PEMTD(JL,JK)
              ENDIF
              RADLWM5(JL,JK,IBLK) = ZPEMTD5 (JL,JK)
            ENDDO
          ENDDO      
        ENDIF
      ENDIF

      IF (LERADFL2 .OR. LERADLW2) THEN
        DO JK=0,KLEV
          DO JL=KIDIA,KFDIA
            PEMTD5(JL,JK) = ZPEMTD5(JL,JK)
          ENDDO
        ENDDO
      ENDIF
    ENDIF

  ENDIF

  ZTSKRAD5(KIDIA:KFDIA)=PTL5(KIDIA:KFDIA)
  ZTSKRAD(KIDIA:KFDIA)=PTL(KIDIA:KFDIA)

!*         2.5  RADIATIVE FLUXES AT EVERY TIME-STEP

  CALL RADHEATTL &
   & (  KIDIA   , KFDIA   , KLON    , KLEV,&
   & PAPRS5  , PAPRSF5,&
   & Z_PEMIS5  , PEMTD5  , PMU0,&
   & PQ5,&
   & PTENT5  , PT5     , PTRSO5  , ZTRSOD5 , PTL5,&
   & Z_PNEB5 , Z_PQLI5 , Z_PQICE5,&
   & Z_PFRSO5  , Z_PFRTH5  , Z_PFRSOD5 , Z_PFRTHD5,&
   & PAPRS   , PAPRSF,&
   & Z_PEMTD,&
   & PQ,&
   & PTENT   , PT      , Z_PTRSO   , ZTRSOD  , PTL,&
   & Z_PFRSO   , Z_PFRTH   , Z_PFRSOD  , Z_PFRTHD )  

ELSE
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      Z_PNEB (JL,JK)=0.0_JPRB
      Z_PNEB5(JL,JK)=0.0_JPRB
      Z_PQLI (JL,JK)=0.0_JPRB
      Z_PQLI5(JL,JK)=0.0_JPRB
      Z_PQICE (JL,JK)=0.0_JPRB
      Z_PQICE5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
  DO JL=KIDIA,KFDIA
    Z_PFRSOC (JL,0)=0.0_JPRB
    Z_PFRSOC5(JL,0)=0.0_JPRB
    Z_PFRTHC (JL,0)=0.0_JPRB
    Z_PFRTHC5(JL,0)=0.0_JPRB
    Z_PFRSOC (JL,1)=0.0_JPRB
    Z_PFRSOC5(JL,1)=0.0_JPRB
    Z_PFRTHC (JL,1)=0.0_JPRB
    Z_PFRTHC5(JL,1)=0.0_JPRB
    Z_PFRSOD (JL)=0.0_JPRB
    Z_PFRSOD5(JL)=0.0_JPRB
    Z_PFRTHD (JL)=0.0_JPRB
    Z_PFRTHD5(JL)=0.0_JPRB
  ENDDO
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PFRSO (JL,JK)=0.0_JPRB
      Z_PFRSO5(JL,JK)=0.0_JPRB
      Z_PFRTH (JL,JK)=0.0_JPRB
      Z_PFRTH5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
! Set surface solar flux to a minimum value for stability in vertical diffusion
  DO JL=KIDIA,KFDIA
    Z_PFRSO5(JL,KLEV)=1.0E-15_JPRB
  ENDDO
  DO JK=1,KTILES
    DO JL=KIDIA,KFDIA
      ZALBTI (JL,JK)=0.0_JPRB
      ZALBTI5(JL,JK)=PALBF(JL)
    ENDDO
  ENDDO    
  DO JL=KIDIA,KFDIA
    Z_PEMIS(JL) =0.0_JPRB
    Z_PEMIS5(JL)=PEMISF(JL)
  ENDDO
  ZTSKRAD5(KIDIA:KFDIA)=PTL5(KIDIA:KFDIA)
  ZTSKRAD(KIDIA:KFDIA)=PTL(KIDIA:KFDIA)
ENDIF

!     ------------------------------------------------------------------

!*         3.     VERTICAL EXCHANGE OF U,V,T,Q BY TURBULENCE
!                 ------------------------------------------

IF ( LEVDIF2 ) THEN

  CALL VDFMAINSTL (KIDIA,KFDIA,KLON,KLEV,KLEVS,KSTEP,KTILES,&
   & ITRAC , TSPHY, ITVL, ITVH, ZCVL, ZCVH,&
   & ZLAIL, ZLAIH, ZHSDFOR,&
   & PU5   , PV5    , PT5   , PQ5  , ZCEN5, PAPRS5, ZGEOM15, ZGEOMH5 ,&
   & PTL5  , PTSA5  , PWSA5,&
   & Z_PFRSO5(1,KLEV) , Z_PFRTH5(1,KLEV), Z_PEMIS5,&
   & PTSN5 ,PTIA5(1,1),&
   ! & ZHLICE5 , ZTLICE5 , ZTLWML5 , & !lake passive
   & PSST  , ISOTY  , ZFRTI , ZALBTI5 , ZWLMX,&
   & ZCHAR ,ZTSKRAD5, ZCFLX5 ,&
   ! OUTPUT (TRAJECTORY)
   & ZAZ0M5 , ZAZ0H5,&
   & ZUCFL5, ZVCFL5, ZTCFL5, ZDCFL5, ZQCFL5, PU10N5, PV10N5,& 
   ! OUTPUT TENDENDCIES
   & PTENT5 , PTENQ5 , PTENU5 , PTENV5 , ZTENC5, Z_PTLE15,&
   !-UPDATED FIELDS FOR TILES (TRAJECTORY)
   & PUSTRTI5 ,PVSTRTI5 ,PAHFSTI5,PEVAPTI5,PTSKTI5,&
   ! FLUX OUTPUTS (TRAJECTORY)
   & ZDIFTS5, ZDIFTQ5, Z_PSTRTU5 , Z_PSTRTV5,&
   !=========
   & PU    , PV    , PT    , PQ    , ZCEN , PAPRS , ZGEOM1, ZGEOMH ,&
   & PTL   , PTSA  , PWSA,&
   & Z_PFRSO(1,KLEV) , Z_PFRTH(1,KLEV),&
   & ZALBTI, ZTSKRAD, ZCFLX,&
   ! OUTPUT           
   & ZAZ0M , ZAZ0H,&
   & ZUCFL, ZVCFL, ZTCFL, ZDCFL, ZQCFL, PU10N, PV10N, & 
   ! OUTPUT TENDENCIES           
   & PTENT  , PTENQ , PTENU , PTENV , ZTENC, PTLE1,&
   !-UPDATED FIELDS FOR TILES 
   & PUSTRTI , PVSTRTI , PAHFSTI , PEVAPTI , PTSKTI,&
   ! FLUX OUTPUTS 
   & PDIFTS , PDIFTQ  , PSTRTU  , PSTRTV  )  


  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENTVD5(JL,JK) = PTENT5(JL,JK)
      ZTENQVD5(JL,JK) = PTENQ5(JL,JK)
      ZTENUVD5(JL,JK) = PTENU5(JL,JK)
      ZTENVVD5(JL,JK) = PTENV5(JL,JK)
    ENDDO
  ENDDO

ELSE

!*         3.1     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.          
  DO JL=KIDIA,KFDIA
    Z_PVDIS (JL)=0.0_JPRB
    Z_PVDIS5(JL)=0.0_JPRB
    Z_PFTLHEV (JL)=0.0_JPRB
    Z_PFTLHEV5(JL)=0.0_JPRB
    Z_PFTLHSB (JL)=0.0_JPRB
    Z_PFTLHSB5(JL)=0.0_JPRB
    ZUCFL (JL)=0.0_JPRB
    ZUCFL5(JL)=0.0_JPRB
    ZVCFL (JL)=0.0_JPRB
    ZVCFL5(JL)=0.0_JPRB
    ZTCFL (JL)=0.0_JPRB
    ZTCFL5(JL)=0.0_JPRB
    ZDCFL (JL)=0.0_JPRB
    ZDCFL5(JL)=0.0_JPRB
    PU10N (JL)=0.0_JPRB
    PV10N (JL)=0.0_JPRB
    Z_PFWSB (JL)=0.0_JPRB
    Z_PFWSB5(JL)=0.0_JPRB
    Z_PBLH5(JL)=0.0_JPRB
  ENDDO
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      PDIFTQ (JL,JK)=0.0_JPRB
      ZDIFTQ5(JL,JK)=0.0_JPRB
      PDIFTS (JL,JK)=0.0_JPRB
      ZDIFTS5(JL,JK)=0.0_JPRB
      PSTRTU (JL,JK)=0.0_JPRB
      Z_PSTRTU5(JL,JK)=0.0_JPRB
      PSTRTV (JL,JK)=0.0_JPRB
      Z_PSTRTV5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF
!     ------------------------------------------------------------------

!*         4.     GRAVITY WAVE DRAG PARAMETRIZATION
!                 ---------------------------------

IF ( LEGWDG2 ) THEN

!*         4.1     CALL GWDTL

  CALL GWDRAGTL &
   & ( KIDIA    , KFDIA  , KLON  , KLEV,&
   & TSPHY    , KSTEP   , KSTART,&
   & PAPRS5   , PAPRSF5 , ZGEOM15, PT5    , PU5,&
   & PV5      , ZHSTD   , PGAMMA,  PTHETA , PSIG,&
   & PAPRS    , PAPRSF  , ZGEOM1 , PT     , PU,&
   & PV,&
   ! OUTPUTS
   & ZUSTRG5  , ZVSTRG5 , Z_PVDISG5, PTENV5 , PTENU5 , PTENT5,&
   & ZUSTRG   , ZVSTRG  , Z_PVDISG , PTENV  , PTENU  , PTENT,&
   ! FLUX OUTPUTS
   & Z_PSTRDU5  , Z_PSTRDV5,&
   & Z_PSTRDU   , Z_PSTRDV                                )  

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENTGW5(JL,JK) = PTENT5(JL,JK)
      ZTENUGW5(JL,JK) = PTENU5(JL,JK)
      ZTENVGW5(JL,JK) = PTENV5(JL,JK)
    ENDDO
  ENDDO
ELSE

!*         4.2     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.

  ZUSTRG5(:)=0.0_JPRB
  ZVSTRG5(:)=0.0_JPRB
  Z_PVDISG5(:)=0.0_JPRB
  ZUSTRG (:)=0.0_JPRB
  ZVSTRG (:)=0.0_JPRB
  Z_PVDISG (:)=0.0_JPRB

  Z_PSTRDU5(:,:)=0.0_JPRB
  Z_PSTRDV5(:,:)=0.0_JPRB
  Z_PSTRDU (:,:)=0.0_JPRB
  Z_PSTRDV (:,:)=0.0_JPRB

ENDIF


!     ------------------------------------------------------------------

!*         5.     CONVECTION PARAMETRIZATION
!                 --------------------------

IF (LECUMF2) THEN

  IF (LECUBM2) THEN

!            CALL CUCALLJ
!     I     (  KIDIA   , KFDIA  , KLON   , KLEV
!     I     ,  KSTEP   , KSTART , LLLAND
!     R     ,  TSPHY
!     R     ,  PT5     , PQ5    , PU5    , PV5
!     R     ,  PVERVEL5, ZDIFTQ5, ZDIFTS5, PAPRS5
!     R     ,  PRSF15  , PRS15  , ZGEOM15
!     R     ,  PTENT5  , PTENQ5 , PTENU5 , PTENV5 , PACPR5
!     I     ,  ITOPC   , IBASC  , ITYPE
!     R     ,  PT      , PQ     , PU     , PV
!     R     ,  PVERVEL , PDIFTQ , PDIFTS , PAPRS
!     R     ,  PRSF1   , PRS1   , ZGEOM1
!     R     ,  PTENT   , PTENQ  , PTENU  , PTENV  , PACPR
!     R     ,  'TLM'                                       )

  ELSEIF (LECUMFS2) THEN

! SIMPLIFIED MASS FLUX CONVECTION SCHEME (P. Lopez)

    DO JK=0,KLEV
      DO JL=KIDIA,KFDIA
        ZPSTRTU5(JL,JK)=0.0_JPRB
        ZPSTRTV5(JL,JK)=0.0_JPRB
      ENDDO
    ENDDO

    CALL CUCALLN2TL &
     & (  KIDIA,    KFDIA,    KLON,   KLEV,&
     & KSTEP,   LLLAND, LDRAIN1D,&
     & TSPHY,&
     & PT5,      PQ5,      PU5,      PV5,&
     & PVERVEL5, ZDIFTQ5,  ZDIFTS5,  PAPRS5,&
     & PRSF15,   PRS15,    ZGEOM15,  ZGEOMH5,&
     & PTENT5,   PTENQ5,   PTENU5,   PTENV5,  PACPR5,&
     & ITOPC,    IBASC,    ITYPE,&
     & ICBOT,    ICTOP,    IBOTSC,   LLCUM,   LLSC,&
     & ZLU5,     ZLUDE5,   ZMFU5,    ZMFD5,&
     & Z_PDIFCQ5,  Z_PDIFCS5,  Z_PFHPCL5,  Z_PFHPCN5,&
     & Z_PFPLCL5,  Z_PFPLCN5,  Z_PSTRCU5,  Z_PSTRCV5, Z_PFCCQL5,Z_PFCCQN5,&
     & ITRAC,    ZCEN5,    ZTENC5,   ZSCAV5,&
     & PT,       PQ,       PU,       PV,&
     & PVERVEL,  PDIFTQ,   PDIFTS,   PAPRS,&
     & PRSF1,    PRS1,     ZGEOM1,   ZGEOMH,&
     & PTENT,    PTENQ,    PTENU,    PTENV,&
     & ZLU,      ZLUDE,    ZMFU,     ZMFD,&
     & Z_PDIFCQ,   Z_PDIFCS,   Z_PFHPCL,   Z_PFHPCN,&
     & Z_PFPLCL,   Z_PFPLCN,   Z_PSTRCU,   Z_PSTRCV,  Z_PFCCQL ,Z_PFCCQN,&
     & ZCEN  , ZTENC )

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENTCO5(JL,JK) = PTENT5(JL,JK)
        ZTENQCO5(JL,JK) = PTENQ5(JL,JK)
        ZTENUCO5(JL,JK) = PTENU5(JL,JK)
        ZTENVCO5(JL,JK) = PTENV5(JL,JK)
      ENDDO
    ENDDO

  ELSE

! TIEDTKE'S MASS FLUX CONVECTION SCHEME

    DO JK=0,KLEV
      DO JL=KIDIA,KFDIA
        ZPSTRTU5(JL,JK)=0.0_JPRB
        ZPSTRTV5(JL,JK)=0.0_JPRB
      ENDDO
    ENDDO

    CALL CUCALLNTL &
     & (  KIDIA,    KFDIA,    KLON,   KLEV,&
     & KSTEP,   LLLAND,&
     & TSPHY,&
     & PT5,      PQ5,      PU5,      PV5,     ZLITOT5,&
     & PVERVEL5, ZDIFTQ5,  ZDIFTS5,  PAPRS5,&
     & PRSF15,   PRS15,    ZGEOM15,  ZGEOMH5,&
     & PTENT5,   PTENQ5,   PTENU5,   PTENV5,  PACPR5,&
     & ITOPC,    IBASC,    ITYPE,&
     & ICBOT,    ICTOP,    IBOTSC,   LLCUM,   LLSC,&
     & ZLU5,     ZLUDE5,   ZMFU5,    ZMFD5,&
     & Z_PDIFCQ5,  Z_PDIFCS5,  Z_PFHPCL5,  Z_PFHPCN5,&
     & Z_PFPLCL5,  Z_PFPLCN5,  Z_PSTRCU5,  Z_PSTRCV5, Z_PFCCQL5,Z_PFCCQN5,&
     & ITRAC,    ZCEN5,    ZTENC5,   ZSCAV5,&
     & PT,       PQ,       PU,       PV,&
     & PVERVEL,  PDIFTQ,   PDIFTS,&
     & PRSF1,    PRS1,     ZGEOM1,   ZGEOMH,&
     & PTENT,    PTENQ,    PTENU,    PTENV,   PACPR,&
     & ZLU,      ZLUDE,    ZMFU,     ZMFD,&
     & Z_PFPLCL, Z_PFPLCN,&
     & ZCEN,     ZTENC )

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENTCO5(JL,JK) = PTENT5(JL,JK)
        ZTENQCO5(JL,JK) = PTENQ5(JL,JK)
        ZTENUCO5(JL,JK) = PTENU5(JL,JK)
        ZTENVCO5(JL,JK) = PTENV5(JL,JK)
      ENDDO
    ENDDO

  ENDIF

!*         5.2    SET CONVECTIVE CLOUD PARAMETERS FOR NEXT TIME-STEP
  DO JL=KIDIA,KFDIA
    PNTOP(JL)=REAL ( ITOPC(JL),JPRB)  + 0.01_JPRB
    PNBAS(JL)=REAL ( IBASC(JL),JPRB)  + 0.01_JPRB
  ENDDO
ELSE

!*         5.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PDIFCS (JL,JK)=0.0_JPRB
      Z_PDIFCS5(JL,JK)=0.0_JPRB
      Z_PDIFCQ (JL,JK)=0.0_JPRB
      Z_PDIFCQ5(JL,JK)=0.0_JPRB
      Z_PFHPCL (JL,JK)=0.0_JPRB
      Z_PFHPCL5(JL,JK)=0.0_JPRB
      Z_PFHPCN (JL,JK)=0.0_JPRB
      Z_PFHPCN5(JL,JK)=0.0_JPRB
      Z_PFPLCL (JL,JK)=0.0_JPRB
      Z_PFPLCL5(JL,JK)=0.0_JPRB
      Z_PFPLCN (JL,JK)=0.0_JPRB
      Z_PFPLCN5(JL,JK)=0.0_JPRB
      Z_PSTRCU (JL,JK)=0.0_JPRB
      Z_PSTRCU5(JL,JK)=0.0_JPRB
      Z_PSTRCV (JL,JK)=0.0_JPRB
      Z_PSTRCV5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         6.     LARGE SCALE WATER PHASE CHANGES
!                 -------------------------------

IF ( LECOND2 .AND. .NOT. LENCLD2 ) THEN

!*         6.1    CALL CONDTL

  CALL CONDTL &
   & ( KIDIA , KFDIA , KLON , KLEV,&
   & TSPHY,&
   & PT5    , PQ5    , PRS15,   PRSF15,&
   & PTENT5,  PTENQ5,&
   & Z_PFHPSL5, Z_PFHPSN5, Z_PFPLSL5, Z_PFPLSN5,&
   & PT    , PQ    , PRS1,   PRSF1,&
   & PTENT,  PTENQ,&
   & Z_PFHPSL, Z_PFHPSN, Z_PFPLSL, Z_PFPLSN          )  

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENTCL5(JL,JK) = PTENT5(JL,JK)
    ENDDO
  ENDDO

ELSEIF( LENCLD2 ) THEN

!*         6.2     CALL CLOUDSTTL

  CALL CLOUDSTTL &
   & (KIDIA  , KFDIA , KLON , KTDIA , KLEV, LDRAIN1D, &
   & TSPHY  , &
   & PAPRS5 , PAPRSF5, PQ5  , Z_PQSAT5  , PT5    , &
   & ZLUDE5 , ZLU5   , &
   & PTENT5 , PTENQ5 , &
   & ZPA5   , ZPI5   , ZPL5 , Z_PFPLSL5 , Z_PFPLSN5, &
   & Z_PFHPSL5, Z_PFHPSN5, ZCOVPTOT5, &
   & PAPRS  , PAPRSF , PQ   , Z_PQSAT   , PT     , &
   & ZLUDE  , ZLU    , &
   & PTENT  , PTENQ  , &
   & ZPA    , ZPI    , ZPL  ,Z_PFPLSL   , Z_PFPLSN , &
   & Z_PFHPSL , Z_PFHPSN, ZCOVPTOT )  

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENTCL5(JL,JK) = PTENT5(JL,JK)
    ENDDO
  ENDDO

  IBLK=(KSTGLO-1)/KLON + 1
  IF (KSTEP >= (NSTOP-1)) THEN
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        PA (JL,JK) = 0.0_JPRB
        PL (JL,JK) = 0.0_JPRB
        PI (JL,JK) = 0.0_JPRB
        GPCLOUD5(JL,JK,IBLK)        = 0.0_JPRB
        GPCLOUD5(JL,KLEV+JK,IBLK)   = 0.0_JPRB
        GPCLOUD5(JL,2*KLEV+JK,IBLK) = 0.0_JPRB
      ENDDO
    ENDDO
  ELSE
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        PA(JL,JK)  = ZPA (JL,JK)
        PL(JL,JK)  = ZPL (JL,JK)
        PI(JL,JK)  = ZPI (JL,JK)
        GPCLOUD5(JL,JK,IBLK)        = ZPA5(JL,JK)
        GPCLOUD5(JL,KLEV+JK,IBLK)   = ZPL5(JL,JK)
        GPCLOUD5(JL,2*KLEV+JK,IBLK) = ZPI5(JL,JK)
      ENDDO
    ENDDO
  ENDIF

ELSE

!*         6.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PFHPSL (JL,JK)=0.0_JPRB
      Z_PFHPSL5(JL,JK)=0.0_JPRB
      Z_PFHPSN (JL,JK)=0.0_JPRB
      Z_PFHPSN5(JL,JK)=0.0_JPRB
      Z_PFPLSL (JL,JK)=0.0_JPRB
      Z_PFPLSL5(JL,JK)=0.0_JPRB
      Z_PFPLSN (JL,JK)=0.0_JPRB
      Z_PFPLSN5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!          7.0 WARNER-McINTYRE-SCINOCCA NON-OROGRAPHIC GRAVITY WAVE SCHEME

!  Note call could possibly already be placed after diffusion if coupling with 
!  coupling with  precip or CAPE not necessary, but calling at end of processes
!  preferable

IF ( LEGWWMS2 ) THEN
 
  ZTSGWD=GTPHYGWWMS
!  ZTSGWD=TSPHY

  IFREQWMS = INT(ZTSGWD/TSPHY)
  ISTEPWMS = MOD(KSTEP,IFREQWMS)

  IF (ISTEPWMS == 0. .OR. KSTEP==KSTART .OR. KSTEP >= (NSTOP-1)) THEN
!  IF (MOD(KSTEP*TSPHY,ZTSGWD)==0.0_JPRB.OR.KSTEP==KSTART) THEN
    
    CALL GWDRAG_WMSSTL(KIDIA , KFDIA , KLON   , KLEV   , ZTSGWD , &
     & PT5 , PU5 , PV5 , PAPRSF5  , PAPRS5, PAPHIF5, &
     & ZTENU5 , ZTENV5 , ZFLXWGWU5, ZFLXWGWV5, &
     & PT  , PU  , PV  , PAPRSF   , PAPRS , PAPHIF , &
     & ZTENU  , ZTENV  , ZFLXWGWU , ZFLXWGWV ) 

    IBLK=(KSTGLO-1)/KLON + 1
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        IF (KSTEP >= (NSTOP-1)) THEN
          TGWDWMS(JL,JK,IBLK) = 0.
          TGWDWMS(JL,KLEV+JK,IBLK) = 0.
        ELSE
          TGWDWMS(JL,JK,IBLK)      = ZTENU(JL,JK)
          TGWDWMS(JL,KLEV+JK,IBLK) = ZTENV(JL,JK)
        ENDIF
        TGWDWMS5(JL,JK,IBLK)      = ZTENU5(JL,JK)
        TGWDWMS5(JL,KLEV+JK,IBLK) = ZTENV5(JL,JK)
      ENDDO
    ENDDO

  ELSE

    IBLK=(KSTGLO-1)/KLON + 1
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTENU(JL,JK) = TGWDWMS(JL,JK,IBLK)
        ZTENV(JL,JK) = TGWDWMS(JL,KLEV+JK,IBLK)
        ZTENU5(JL,JK) = TGWDWMS5(JL,JK,IBLK)
        ZTENV5(JL,JK) = TGWDWMS5(JL,KLEV+JK,IBLK)
      ENDDO
    ENDDO
  ENDIF

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENUWM5(JL,JK) = ZTENU5(JL,JK)
      ZTENVWM5(JL,JK) = ZTENV5(JL,JK)

      IF (LREGWWMS) THEN
        IF (PAPRSF5(JL,JK) < 50.0_JPRB .AND. ABS(PV5(JL,JK)) > 30.0_JPRB) THEN
          ZTENU (JL,JK) = 0.0_JPRB
        ENDIF
      ENDIF

      PTENU (JL,JK) = PTENU (JL,JK)+ZTENU (JL,JK)
      PTENU5(JL,JK) = PTENU5(JL,JK)+ZTENU5(JL,JK)
      PTENV (JL,JK) = PTENV (JL,JK)+ZTENV (JL,JK)
      PTENV5(JL,JK) = PTENV5(JL,JK)+ZTENV5(JL,JK)

      ZTFIELD (JL,JK) = ZRCPD*(PU5(JL,JK)*ZTENU(JL,JK)+ZTENU5(JL,JK)*PU(JL,JK) &
       & + PV5(JL,JK)*ZTENV(JL,JK)+ZTENV5(JL,JK)*PV(JL,JK))
      ZTFIELD5(JL,JK) = (PU5(JL,JK)*ZTENU5(JL,JK) &
       & + PV5(JL,JK)*ZTENV5(JL,JK))*ZRCPD
      PTENT (JL,JK) = PTENT (JL,JK)-ZTFIELD (JL,JK)
      PTENT5(JL,JK) = PTENT5(JL,JK)-ZTFIELD5(JL,JK)
    ENDDO
  ENDDO

ELSE

!*        7.1     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA      
      ZFLXWGWU (JL,JK) = 0.0_JPRB
      ZFLXWGWU5(JL,JK) = 0.0_JPRB
      ZFLXWGWV (JL,JK) = 0.0_JPRB
      ZFLXWGWV5(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         8.     ELIMINATION OF NEGATIVE SPECIFIC HUMIDITIES
!                 -------------------------------------------

IF ( LEQNGT2 ) THEN

  CALL ABOR1 ('TL OF NEGATIVE HUMIDITIES NOT CODED')

ELSE

!*         8.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      Z_PFCQNG (JL,JK)=0.0_JPRB
      Z_PFCQNG5(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         9.     TRACER TRANSPORT TENDENCIES BACK INTO EXTRA FIELDS
!                 --------------------------------------------------

IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_TEND( KIDIA, KFDIA, KLEV, KLON, PTENGFL5, ZTENC5 )
  CALL GEMS_TEND_TL( KIDIA, KFDIA, KLEV, KLON, PTENGFL, ZTENC )
ENDIF

!     ------------------------------------------------------------------

!*        10.      COMPUTATION OF NEW SURFACE VALUES
!                  ---------------------------------

IF ( LESURF2) THEN

!*        10.1   CALL SURFTSTP

  CALL ABOR1 ('TL OF LAND SURFACE SCHEME NOT CODED')

ELSE
  DO JL=KIDIA,KFDIA
    Z_PFTG12 (JL)=0.0_JPRB
    Z_PFTG125(JL)=0.0_JPRB
    Z_PFWROD (JL)=0.0_JPRB
    Z_PFWROD5(JL)=0.0_JPRB
    Z_PFWRO1 (JL)=0.0_JPRB
    Z_PFWRO15(JL)=0.0_JPRB
    Z_PFWMLT (JL)=0.0_JPRB
    Z_PFWMLT5(JL)=0.0_JPRB
    Z_PFWG12 (JL)=0.0_JPRB
    Z_PFWG125(JL)=0.0_JPRB
    Z_PFWEV (JL)=0.0_JPRB
    Z_PFWEV5(JL)=0.0_JPRB
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*              11.     END OF E.C.M.W.F. PHYSICS
!                      -------------------------

!*              11.1    UNIT CHANGES/SCALINGS
DO JL=KIDIA,KFDIA
  PGZ0M  (JL)=ZAZ0M (JL)*RG
  PGZ0M5 (JL)=ZAZ0M5(JL)*RG
  PLZ0H  (JL)=ZAZ0H(JL)/ZAZ0H5(JL)
  PLZ0H5 (JL)=LOG(ZAZ0H5(JL))
  Z_PFWG12 (JL)=Z_PFWG12 (JL)*1.E+03_JPRB
  Z_PFWG125(JL)=Z_PFWG125(JL)*1.E+03_JPRB
  Z_PFWMLT (JL)=Z_PFWMLT (JL)*1.E+03_JPRB
  Z_PFWMLT5(JL)=Z_PFWMLT5(JL)*1.E+03_JPRB
  Z_PFWRO1 (JL)=Z_PFWRO1 (JL)*1.E+03_JPRB
  Z_PFWRO15(JL)=Z_PFWRO15(JL)*1.E+03_JPRB
  Z_PFWROD (JL)=Z_PFWROD (JL)*1.E+03_JPRB
  Z_PFWROD5(JL)=Z_PFWROD5(JL)*1.E+03_JPRB
  Z_PFWEV  (JL)=Z_PFWEV  (JL)*1.E+03_JPRB
  Z_PFWEV5 (JL)=Z_PFWEV5 (JL)*1.E+03_JPRB
ENDDO

!*      12. WRITE TRAJECTORY FOR PHYS.TENDENCIES, RADIATION AND
!           NEW CLOUD SCHEME

IF ((LERADN2.OR.LERADSW2) .OR. LENCLD2 .OR. LETRAJPT) THEN
  IFIELDS = NG3DT95*KLEV + NG3PR95*(KLEV+1) + NG2DSF5
  CALL WRPHTRAJT(NSTEP,KIDIA,KFDIA,KSTGLO,IFIELDS,&
   & ZTENTVD5,ZTENQVD5,ZTENUVD5,ZTENVVD5,&
   & ZTENTGW5,ZTENUGW5,ZTENVGW5,&
   & ZTENTCO5,ZTENQCO5,ZTENUCO5,ZTENVCO5,&
   & ZTENTCL5,ZTENUWM5,ZTENVWM5,&
   & ZDIFTQ5 ,ZDIFTS5 ,ZPEMTD5 ,PTRSO5,&
   & ZPA55, ZPL55, ZPI55,&
   & ZLUDE5, ZLU5,&
   & Z_PFPLCL5, Z_PFPLCN5,ZUCFL5,ZVCFL5,ZTCFL5,ZQCFL5 )  
ENDIF

!*      13. OBSERVATION OPERATOR FOR MW-RADIANCE ASSIMILATION


IF (LEMWAVE_ACTIVE) THEN
   DO JL=KIDIA,KFDIA
      YL_MWAVE_PHYS_FW (JL) % P   (1:KLEV)   = PAPRSF5 (JL,1:KLEV)  
      YL_MWAVE_PHYS_FW (JL) % PH  (1:KLEV+1) = PAPRS5  (JL,0:KLEV)
      YL_MWAVE_PHYS_FW (JL) % T   (1:KLEV)   = PT5     (JL,1:KLEV)
      YL_MWAVE_PHYS_FW (JL) % Q   (1:KLEV)   = PQ5     (JL,1:KLEV)
    
      DO JK = 1, KLEV
         YL_MWAVE_PHYS_FW (JL) % RFL (JK)  = 0.5_JPRB * (Z_PFPLSL5 (JL,JK) + Z_PFPLSL5 (JL,JK-1))
         YL_MWAVE_PHYS_FW (JL) % SFL (JK)  = 0.5_JPRB * (Z_PFPLSN5 (JL,JK) + Z_PFPLSN5 (JL,JK-1))
         YL_MWAVE_PHYS_FW (JL) % CRFL (JK) = 0.5_JPRB * (Z_PFPLCL5 (JL,JK) + Z_PFPLCL5 (JL,JK-1))
         YL_MWAVE_PHYS_FW (JL) % CSFL (JK) = 0.5_JPRB * (Z_PFPLCN5 (JL,JK) + Z_PFPLCN5 (JL,JK-1))
      ENDDO

      IF (LENCLD2) THEN     
        YL_MWAVE_PHYS_FW (JL) % CWC (1:KLEV) = ZPL5 (JL,1:KLEV)
        YL_MWAVE_PHYS_FW (JL) % CIC (1:KLEV) = ZPI5 (JL,1:KLEV)
        YL_MWAVE_PHYS_FW (JL) % CC  (1:KLEV) = ZPA5 (JL,1:KLEV)
        YL_MWAVE_PHYS_FW (JL) % PFRC(1:KLEV) = ZCOVPTOT5(JL,1:KLEV) 
      ELSE
        YL_MWAVE_PHYS_FW (JL) % CWC (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_FW (JL) % CIC (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_FW (JL) % CC  (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_FW (JL) % PFRC(1:KLEV) = 0.0_JPRB
      ENDIF

      YL_MWAVE_PHYS_FW (JL) % U10  = ZUCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % V10  = ZVCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % T2M  = ZTCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % Q2M  = ZQCFL5 (JL)
      YL_MWAVE_PHYS_FW (JL) % TSK  = PTL5   (JL)
      
      YL_MWAVE_PHYS_FW (JL) % LSM  = PLSM   (JL)
      YL_MWAVE_PHYS_FW (JL) % SIM  = PCI    (JL)
      
      YL_MWAVE_PHYS_FW (JL) % SLAT = PGEMU  (JL)
      YL_MWAVE_PHYS_FW (JL) % SLON = PSLON  (JL)
      YL_MWAVE_PHYS_FW (JL) % CLON = PCLON  (JL)
      YL_MWAVE_PHYS_FW (JL) % OROG = POROG  (JL) / RG ! Convert to metres
            

      YL_MWAVE_PHYS_TL (JL) % P   (1:KLEV)   = PAPRSF (JL,1:KLEV)  
      YL_MWAVE_PHYS_TL (JL) % PH  (1:KLEV+1) = PAPRS  (JL,0:KLEV)
      YL_MWAVE_PHYS_TL (JL) % T   (1:KLEV)   = PT     (JL,1:KLEV)
      YL_MWAVE_PHYS_TL (JL) % Q   (1:KLEV)   = PQ     (JL,1:KLEV)
    
      DO JK = 1, KLEV
         YL_MWAVE_PHYS_TL (JL) % RFL (JK)  = 0.5_JPRB * (Z_PFPLSL (JL,JK) + Z_PFPLSL (JL,JK-1))
         YL_MWAVE_PHYS_TL (JL) % SFL (JK)  = 0.5_JPRB * (Z_PFPLSN (JL,JK) + Z_PFPLSN (JL,JK-1))
         YL_MWAVE_PHYS_TL (JL) % CRFL (JK) = 0.5_JPRB * (Z_PFPLCL (JL,JK) + Z_PFPLCL (JL,JK-1))
         YL_MWAVE_PHYS_TL (JL) % CSFL (JK) = 0.5_JPRB * (Z_PFPLCN (JL,JK) + Z_PFPLCN (JL,JK-1))
      ENDDO

      IF (LENCLD2) THEN     
        YL_MWAVE_PHYS_TL (JL) % CWC (1:KLEV) = ZPL (JL,1:KLEV)
        YL_MWAVE_PHYS_TL (JL) % CIC (1:KLEV) = ZPI (JL,1:KLEV)
        YL_MWAVE_PHYS_TL (JL) % CC  (1:KLEV) = ZPA (JL,1:KLEV)
        YL_MWAVE_PHYS_TL (JL) % PFRC(1:KLEV) = ZCOVPTOT(JL,1:KLEV)
      ELSE
        YL_MWAVE_PHYS_TL (JL) % CWC (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_TL (JL) % CIC (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_TL (JL) % CC  (1:KLEV) = 0.0_JPRB
        YL_MWAVE_PHYS_TL (JL) % PFRC(1:KLEV) = 0.0_JPRB
      ENDIF

      YL_MWAVE_PHYS_TL (JL) % U10  = ZUCFL (JL)
      YL_MWAVE_PHYS_TL (JL) % V10  = ZVCFL (JL)
      YL_MWAVE_PHYS_TL (JL) % T2M  = ZTCFL (JL)
      YL_MWAVE_PHYS_TL (JL) % Q2M  = ZQCFL (JL)
      YL_MWAVE_PHYS_TL (JL) % TSK  = PTL   (JL)
      
      YL_MWAVE_PHYS_TL (JL) % LSM  = 0.0_JPRB
      YL_MWAVE_PHYS_TL (JL) % SIM  = 0.0_JPRB
      
      YL_MWAVE_PHYS_TL (JL) % SLAT = 0.0_JPRB            
      YL_MWAVE_PHYS_TL (JL) % SLON = 0.0_JPRB            
      YL_MWAVE_PHYS_TL (JL) % CLON = 0.0_JPRB          
      YL_MWAVE_PHYS_TL (JL) % OROG = 0.0_JPRB        
   ENDDO

   CALL MWAVE_OBSOP_TL     (KIDIA, KFDIA, KLON, KLEV, NMAX_CHAN, YL_MWAVE_PHYS_FW, YL_MWAVE_PHYS_TL, YD_MWAVE_RAD)
     
   IF (LDMWAVE_TEST) & 
   & CALL MWAVE_OBSOP_TEST (KIDIA, KFDIA, KLON, KLEV, NMAX_CHAN, YL_MWAVE_PHYS_FW, YL_MWAVE_PHYS_TL, YD_MWAVE_RAD)   
ENDIF

IF (NGEMS > 0 .AND. LTRACLNPH) THEN
  CALL GEMS_DEALLOC( ZCEN5, ZTENC5, ZTENC5_SKF, ZCFLX5, ZSCAV5, ZGRGTEND)
  CALL GEMS_DEALLOC_TL( ZCEN, ZTENC,ZCFLX )
ELSE
  DEALLOCATE(ZCEN5,ZTENC5,ZCFLX5)
  DEALLOCATE(ZCEN,ZTENC,ZCFLX)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CALLPARTL',1,ZHOOK_HANDLE)
END SUBROUTINE CALLPARTL
