  SUBROUTINE CMIP6_PI_AEROSOLS

    !--- inherited types, functions and data
    USE PARKIND1  ,ONLY : JPIM     ,JPRB
    USE YOMHOOK, ONLY : LHOOK, DR_HOOK
    USE YOMLUN,  ONLY : NULOUT
    USE YOMRIP,  ONLY : NINDAT, NSTADD

    USE YOMCST,  ONLY : RPI    
    USE YOMMP,   ONLY : MYPROC

    USE YOMDIM,  ONLY: NPROMA, NFLEVG, NGPBLKS
    USE YOMGEM, ONLY : NGPTOT
    USE YOERAD, ONLY : NRPROMA
    USE YOERAD, ONLY : TM5_NLON , TM5_NLAT, TM5_NLEV , TM5_NMON , NWVL, &
&                      TM5_LON, TM5_LAT, TM5_LEV, TM5_HYA, TM5_HYB, TM5_HYAI,  &
&                      TM5_HYBI, TM5_P0, TM5_PS, TM5_PPS, TM5_PPSF,         &
&                      TM5_NUM_KS, TM5_NUM_AS, TM5_NUM_CS, TM5_MSO4_KS,        &
&                      TM5_MSO4_AS, TM5_MSO4_CS, TM5_MBC_KS,TM5_MBC_AS, TM5_MBC_CS,     &
&                      TM5_MBC_KI, TM5_MOC_KS, TM5_MOC_AS, TM5_MOC_CS, TM5_MOC_KI,         &
&                      TM5_MSS_AS, TM5_MSS_CS, TM5_MDU_AS, TM5_MDU_CS,    &
&                      TM5_MDU_AI, TM5_MDU_CI,                                 &
&                      TM5_MNO3,   TM5_MMSA, &
&                      TM5_AOD, TM5_ABS, TM5_ASY
    USE YOE_PI_AERO, ONLY: LPI_AERO_UPDATED
    USE YOECLDP, ONLY: NCLOUDACT

    IMPLICIT NONE

    REAL(KIND=JPRB) :: ZHOOK_HANDLE

    INTEGER(KIND=JPIM), PARAMETER :: RPRC = 1
    INTEGER(KIND=JPIM), PARAMETER :: ITAG = 34567
    INTEGER(KIND=JPIM), SAVE :: IYROLD=-999,  IMONOLD=-999

    INTEGER(KIND=JPIM) :: ILAT, ILON, IMONTH, ILEV
    INTEGER(KIND=JPIM) :: IJ0, IM0, IA0, IDAY, IMON, IMODYR, ILMOIS(12)
    INTEGER(KIND=JPIM) :: JN
    INTEGER(KIND=JPIM) :: JLAT, JLON, JLEV    

    !---included functions
#include "updcal.intfb.h"
#include "fcttim.h"

    !---executable procedure
    IF (LHOOK) CALL DR_HOOK('CMIP6_PI_AEROSOLS',0,ZHOOK_HANDLE)
    
    WRITE(NULOUT,*)'Entering Subroutine CMIP6_PI_AEROSOLS'

    IF (.NOT.ALLOCATED(TM5_LEV)) CALL ALLOCATE_TM5_VARS

    IJ0=NDD(NINDAT)                 ! see ../functions/fcttim.h
    IM0=NMM(NINDAT)
    IA0=NCCAA(NINDAT)
    CALL UPDCAL(IJ0,IM0,IA0,NSTADD,IDAY,IMON,IMODYR,ILMOIS,NULOUT)

    IF (IMON/=IMONOLD) THEN
       IMONOLD=IMON

       IF (MYPROC==RPRC) THEN

          WRITE(NULOUT,*) 'Reading optical properties from file' 
          WRITE(NULOUT,*) 'IMON= ',IMON
          CALL READ_TM5_OPT_PROP(IMON, TM5_LEV, TM5_LON, TM5_LAT, &
                              &  TM5_HYA, TM5_HYB, TM5_HYAI, TM5_HYBI, &
                              &  TM5_P0,  TM5_PS,  TM5_AOD, TM5_ABS, TM5_ASY, TM5_PPS, TM5_PPSF)


          WRITE(NULOUT,*) 'Reading aerosol data from file' 
          CALL READ_TM5_AEROSOLS(IMON,   &
                          &  TM5_NUM_KS,  TM5_NUM_AS,  TM5_NUM_CS,   &
                          &  TM5_MSO4_KS, TM5_MSO4_AS, TM5_MSO4_CS,   &
                          &  TM5_MBC_KS,  TM5_MBC_AS,  TM5_MBC_CS,    &
                          &  TM5_MOC_KS,  TM5_MOC_AS,  TM5_MOC_CS,    &
                          &               TM5_MSS_AS,  TM5_MSS_CS,    &
                          &               TM5_MDU_AS,  TM5_MDU_CS,    &
                          &               TM5_MBC_KI,  TM5_MOC_KI,    &
                          &               TM5_MDU_AI,  TM5_MDU_CI,    &
                          &  TM5_MNO3,    TM5_MMSA)


! DIAGNOSTIC OUTPUTS FOR DEBUGGING
!    DO JLEV=1,TM5_NLEV
!        IF (IMON == 1) THEN
! print *, '==================================================================================================================================='
! print *,  'HYAI(1)= ',TM5_HYAI(1,JLEV),' HYAI(2)= ',TM5_HYAI(2,JLEV), ' HYBI(1)= ',TM5_HYBI(1,JLEV), 'HYBI(2)= ',TM5_HYBI(2,JLEV)
! print *, '==================================================================================================================================='
!        ENDIF
!    END DO

!       DO JLAT=1,TM5_NLAT
!          DO JLON=1,TM5_NLON
!!        IF (NSTEP == 4 .and. ZGLON(JL) <= 22.5 .and. ZGLON(JL) > 22.4 .and. ZGLAT(JL) <= 45.7 .and. ZGLAT(JL) >= 45.6) THEN
!        IF (IMON == 1) THEN
!          WRITE(NULOUT,'(A,2(1X,F8.3),3(1X,E12.5))') 'XXXXXXXX', TM5_LON(JLON), TM5_LAT(JLAT), TM5_AOD(JLON, JLAT, 3, 4), TM5_ABS(JLON, JLAT, 3, 4), TM5_ASY(JLON, JLAT, 3, 4)
!          WRITE(NULOUT,'(A,2(1X,F8.3),3(1X,E12.5))') 'GGGGGGGG', TM5_LON(JLON), TM5_LAT(JLAT), TM5_MSO4_KS(JLON, JLAT, 2) , TM5_MBC_KS(JLON, JLAT, 2), TM5_MOC_KI(JLON, JLAT, 2)
!          WRITE(NULOUT,'(A,2(1X,F8.3),3(1X,E12.5))') 'DDDDDDDDD', TM5_LON(JLON), TM5_LAT(JLAT), TM5_PPSF(JLON, JLAT, 2), (TM5_PPSF(JLON, JLAT, 2)-TM5_PPSF(JLON, JLAT, 3)) , (TM5_PPS(JLON, JLAT, 2)-TM5_PPS(JLON, JLAT, 3))
!          WRITE(NULOUT,'(A,2(1X,F8.3),3(1X,E12.5))') 'ZZZZZZZZ', TM5_LON(JLON), TM5_LAT(JLAT), TM5_AOD(JLON, JLAT, 16, 4), TM5_ABS(JLON, JLAT, 16, 4), TM5_ASY(JLON, JLAT, 16, 4)
!        ENDIF
!        IF (IMON == 1 .and. ZGLON(JL) <= 22.5 .and. ZGLON(JL) > 22.4 .and. ZGLAT(JL) <= 45.7 .and. ZGLAT(JL) >= 45.6) THEN
!          WRITE(6,'(A,2(1X,F8.3),3(1X,E12.5))') 'VVVVVVVV', ZGLON(JL), ZGLAT(JL), TM5_PPS(JLON, JLAT,JLEV), TM5_AOD(JLON, JLAT, JLEV, 4)
!        ENDIF
!!
!!        IF (ZMAC2SP_AOD(JL,KLEV,1) > 10. .or. ZMAC2SP_AOD(JL,KLEV,1) < 0.) THEN
!!           print *, '============================================================================================================================================'
!!           print *, 'ZMAC2SP_AOD= ',ZMAC2SP_AOD(JL,KLEV,1), ' JL= ',JL, ' Lon= ',ZGLON(JL) ,' Lat= ',ZGLAT(JL),'PGEOH= ',PGEOH (JL,KLEV+1),' YEAR_FR= ',YEAR_FR
!!           print *, '============================================================================================================================================'
!!        ENDIF
!       END DO   
!     END DO
!!    END DO
! END DIAGNOSTIC OUTPUTS FOR DEBUGGING


       END IF
       
       IF (MYPROC==RPRC) WRITE(NULOUT,*) 'Broadcast aerosol optical properties'
       CALL BROADCAST_TM5_OPT_PROP(TM5_LEV, TM5_LON, TM5_LAT, &
                             & TM5_HYA, TM5_HYB, TM5_HYAI, TM5_HYBI, &
                             & TM5_P0,  TM5_PS,  TM5_AOD, TM5_ABS,  TM5_ASY, TM5_PPS, TM5_PPSF )

       
       IF (MYPROC==RPRC) WRITE(NULOUT,*) 'Broadcast aerosol data'
       CALL BROADCAST_TM5_AEROSOLS(TM5_NUM_KS,  TM5_NUM_AS,  TM5_NUM_CS,    &
                                &  TM5_MSO4_KS, TM5_MSO4_AS, TM5_MSO4_CS,   &
                                &  TM5_MBC_KS,  TM5_MBC_AS,  TM5_MBC_CS,    &
                                &  TM5_MOC_KS,  TM5_MOC_AS,  TM5_MOC_CS,    &
                                &               TM5_MSS_AS,  TM5_MSS_CS,    &
                                &               TM5_MDU_AS,  TM5_MDU_CS,    & 
                                &               TM5_MBC_KI,  TM5_MOC_KI,    &
                                &               TM5_MDU_AI,  TM5_MDU_CI,    &
                                &  TM5_MNO3,    TM5_MMSA)

       LPI_AERO_UPDATED = .TRUE.
    ELSE
       WRITE(NULOUT,*)'Not new month, nothing for cmip6_pi_aerosols to do'
       LPI_AERO_UPDATED = .FALSE.
       
    END IF

  
    IF (LHOOK) CALL DR_HOOK('CMIP6_PI_AEROSOLS',1,ZHOOK_HANDLE)


CONTAINS

  !--------------------------------------------------------------------------------------------------------------

  !--------------------------------------------------------------------------------------------------------------
  SUBROUTINE ALLOCATE_TM5_VARS

    !---array dimensions for TM5 data
    TM5_NLON = 120
    TM5_NLAT = 90
    TM5_NLEV = NFLEVG
    TM5_NMON = 12
    NWVL = 14

    ! allocate variables needed for TM5
    IF (MYPROC==RPRC) WRITE(NULOUT,*) 'Allocate TM5 variables with nlev=',TM5_NLEV

    ! TM5 dimensions and coordinates
    ALLOCATE(TM5_LON(TM5_NLON))
    ALLOCATE(TM5_LAT(TM5_NLAT))
    ALLOCATE(TM5_LEV(TM5_NLEV))
    ALLOCATE(TM5_HYA(TM5_NLEV))
    ALLOCATE(TM5_HYB(TM5_NLEV))
    ALLOCATE(TM5_HYAI(2,TM5_NLEV))
    ALLOCATE(TM5_HYBI(2,TM5_NLEV))

    ALLOCATE(TM5_PS(TM5_NLON, TM5_NLAT))
    ! TM5 pressure field in 3 dimensions at full levels and interface
    ! levels
    ALLOCATE(TM5_PPS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_PPSF(TM5_NLON, TM5_NLAT, 0:TM5_NLEV))

    ! aerosol number concentrations from TM5 (soluble modes only)
    ALLOCATE(TM5_NUM_KS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_NUM_AS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_NUM_CS(TM5_NLON, TM5_NLAT, TM5_NLEV))

    ! aerosol mass concentrations from TM5 (soluble modes only)
    ALLOCATE(TM5_MSO4_KS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MSO4_AS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MSO4_CS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MBC_KS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MBC_AS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MBC_CS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MBC_KI(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MOC_KS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MOC_AS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MOC_CS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MOC_KI(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MSS_AS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MSS_CS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MDU_AS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MDU_CS(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MDU_AI(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MDU_CI(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MNO3(TM5_NLON, TM5_NLAT, TM5_NLEV))
    ALLOCATE(TM5_MMSA(TM5_NLON, TM5_NLAT, TM5_NLEV))

    ! aerosol optical properties
    ALLOCATE(TM5_AOD(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL))
    ALLOCATE(TM5_ABS(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL))
    ALLOCATE(TM5_ASY(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL))

  END SUBROUTINE ALLOCATE_TM5_VARS
  !--------------------------------------------------------------------------------------------------------------

  !--------------------------------------------------------------------------------------------------------------
  SUBROUTINE BROADCAST_TM5_OPT_PROP(PTM5_LEV,  PTM5_LON,  PTM5_LAT,  &
                                &  PTM5_HYA,  PTM5_HYB,  PTM5_HYAI, PTM5_HYBI, &
                                &  PTM5_P0,   PTM5_PS,   PTM5_AOD,  PTM5_ABS,  PTM5_ASY, PTM5_PPS, PTM5_PPSF )

    !--- inherited types, functions and data
    USE YOMHOOK, ONLY    : LHOOK, DR_HOOK
    USE MPL_MODULE, ONLY : MPL_BROADCAST

    IMPLICIT NONE

    !---subroutine interface
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_LEV(TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_LON(TM5_NLON)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_LAT(TM5_NLAT)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_HYA(TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_HYB(TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_HYAI(2,TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_HYBI(2,TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_P0
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_PS(TM5_NLON,TM5_NLAT)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_AOD(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_ABS(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_ASY(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_PPS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PTM5_PPSF(TM5_NLON, TM5_NLAT, 0:TM5_NLEV)

    !---local data
    REAL(KIND=JPRB) :: ZHOOK_HANDLE
    REAL(KIND=JPRB) :: ZP0(1)        ! needed because MPL_BROADCAST does not support scalar reals

    !---executable procedure
    IF (LHOOK) CALL DR_HOOK('BROADCAST_TM5_OPT_PROP',0,ZHOOK_HANDLE)
    ZP0(1) = PTM5_P0

    CALL MPL_BROADCAST(PTM5_LEV,KTAG=ITAG+1,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-1: ')
    CALL MPL_BROADCAST(PTM5_LON,KTAG=ITAG+2,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-2: ')
    CALL MPL_BROADCAST(PTM5_LAT,KTAG=ITAG+3,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-3: ')
    CALL MPL_BROADCAST(PTM5_HYA,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-4: ')
    CALL MPL_BROADCAST(PTM5_HYB,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-5: ')
    CALL MPL_BROADCAST(PTM5_HYAI,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-6: ')
    CALL MPL_BROADCAST(PTM5_HYBI,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-7: ')
    CALL MPL_BROADCAST(ZP0,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-8: ')
    CALL MPL_BROADCAST(PTM5_PS,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-9: ')
    CALL MPL_BROADCAST(PTM5_AOD,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-10: ')
    CALL MPL_BROADCAST(PTM5_ABS,KTAG=ITAG+5,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-11: ')
    CALL MPL_BROADCAST(PTM5_ASY,KTAG=ITAG+6,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-12: ')
    CALL MPL_BROADCAST(PTM5_PPS,KTAG=ITAG+7,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-13: ')
    CALL MPL_BROADCAST(PTM5_PPSF,KTAG=ITAG+8,KROOT=RPRC,CDSTRING='TM5_PI_OPT_PROP-14: ')

    IF (LHOOK) CALL DR_HOOK('BROADCAST_TM5_OPT_PROP',1,ZHOOK_HANDLE)
  END SUBROUTINE BROADCAST_TM5_OPT_PROP
  !--------------------------------------------------------------------------------------------------------------

  !--------------------------------------------------------------------------------------------------------------
  SUBROUTINE BROADCAST_TM5_AEROSOLS(PNUM_KS,  PNUM_AS,  PNUM_CS,    &
                                 &  PMSO4_KS, PMSO4_AS, PMSO4_CS,  &
                                 &  PMBC_KS,  PMBC_AS,  PMBC_CS,   &
                                 &  PMOC_KS,  PMOC_AS,  PMOC_CS,   &
                                 &            PMSS_AS,  PMSS_CS,   &
                                 &            PMDU_AS,  PMDU_CS,   &
                                 &            PMBC_KI,  PMOC_KI,   &
                                 &            PMDU_AI,  PMDU_CI,   &
                                 &  PMNO3,    PMMSA)

    !--- inherited types, functions and data
    USE YOMHOOK, ONLY    : LHOOK, DR_HOOK
    USE MPL_MODULE, ONLY : MPL_BROADCAST

    IMPLICIT NONE

    !---subroutine interface
    REAL(KIND=JPRB), INTENT(INOUT) :: PNUM_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PNUM_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PNUM_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMSO4_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMSO4_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMSO4_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMBC_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMBC_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMBC_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMBC_KI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMOC_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMOC_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMOC_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMOC_KI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMSS_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMSS_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMDU_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMDU_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMDU_AI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMDU_CI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMNO3(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(INOUT) :: PMMSA(TM5_NLON, TM5_NLAT, TM5_NLEV)

    !---local data
    REAL(KIND=JPRB) :: ZHOOK_HANDLE

    !---executable procedure
    IF (LHOOK) CALL DR_HOOK('BROADCAST_TM5_AEROSOLS',0,ZHOOK_HANDLE)

    CALL MPL_BROADCAST(PNUM_KS,KTAG=ITAG+4,KROOT=RPRC,CDSTRING='TM5_PI_AERO-1: ')
    CALL MPL_BROADCAST(PNUM_AS,KTAG=ITAG+5,KROOT=RPRC,CDSTRING='TM5_PI_AERO-2: ')
    CALL MPL_BROADCAST(PNUM_CS,KTAG=ITAG+6,KROOT=RPRC,CDSTRING='TM5_PI_AERO-3: ')
    CALL MPL_BROADCAST(PMSO4_KS,KTAG=ITAG+7,KROOT=RPRC,CDSTRING='TM5_PI_AERO-4: ')
    CALL MPL_BROADCAST(PMSO4_AS,KTAG=ITAG+8,KROOT=RPRC,CDSTRING='TM5_PI_AERO-5: ')
    CALL MPL_BROADCAST(PMSO4_CS,KTAG=ITAG+9,KROOT=RPRC,CDSTRING='TM5_PI_AERO-6: ')
    CALL MPL_BROADCAST(PMBC_KS,KTAG=ITAG+10,KROOT=RPRC,CDSTRING='TM5_PI_AERO-7: ')
    CALL MPL_BROADCAST(PMBC_AS,KTAG=ITAG+11,KROOT=RPRC,CDSTRING='TM5_PI_AERO-8: ')
    CALL MPL_BROADCAST(PMBC_CS,KTAG=ITAG+12,KROOT=RPRC,CDSTRING='TM5_PI_AERO-9: ')
    CALL MPL_BROADCAST(PMBC_KI,KTAG=ITAG+13,KROOT=RPRC,CDSTRING='TM5_PI_AERO-10: ')
    CALL MPL_BROADCAST(PMOC_KS,KTAG=ITAG+14,KROOT=RPRC,CDSTRING='TM5_PI_AERO-11: ')
    CALL MPL_BROADCAST(PMOC_AS,KTAG=ITAG+15,KROOT=RPRC,CDSTRING='TM5_PI_AERO-12: ')
    CALL MPL_BROADCAST(PMOC_CS,KTAG=ITAG+16,KROOT=RPRC,CDSTRING='TM5_PI_AERO-13: ')
    CALL MPL_BROADCAST(PMOC_KI,KTAG=ITAG+17,KROOT=RPRC,CDSTRING='TM5_PI_AERO-14: ')
    CALL MPL_BROADCAST(PMSS_AS,KTAG=ITAG+18,KROOT=RPRC,CDSTRING='TM5_PI_AERO-15: ')
    CALL MPL_BROADCAST(PMSS_CS,KTAG=ITAG+19,KROOT=RPRC,CDSTRING='TM5_PI_AERO-16: ')
    CALL MPL_BROADCAST(PMDU_AS,KTAG=ITAG+20,KROOT=RPRC,CDSTRING='TM5_PI_AERO-17: ')
    CALL MPL_BROADCAST(PMDU_CS,KTAG=ITAG+21,KROOT=RPRC,CDSTRING='TM5_PI_AERO-18: ')
    CALL MPL_BROADCAST(PMDU_AI,KTAG=ITAG+22,KROOT=RPRC,CDSTRING='TM5_PI_AERO-19: ')
    CALL MPL_BROADCAST(PMDU_CI,KTAG=ITAG+23,KROOT=RPRC,CDSTRING='TM5_PI_AERO-20: ')
    ! NO3 and MSA only used in cloud activation scheme
    IF (NCLOUDACT > 0) THEN
      CALL MPL_BROADCAST(PMNO3,KTAG=ITAG+24,KROOT=RPRC,CDSTRING='TM5_PI_AERO-21: ')
      CALL MPL_BROADCAST(PMMSA,KTAG=ITAG+25,KROOT=RPRC,CDSTRING='TM5_PI_AERO-22: ')
    ENDIF

    IF (LHOOK) CALL DR_HOOK('BROADCAST_TM5_AEROSOLS',1,ZHOOK_HANDLE)
  END SUBROUTINE BROADCAST_TM5_AEROSOLS
  !--------------------------------------------------------------------------------------------------------------

  !--------------------------------------------------------------------------------------------------------------
  SUBROUTINE READ_TM5_OPT_PROP(IMON, PTM5_LEV, PTM5_LON, PTM5_LAT,  &
                            &  PTM5_HYA, PTM5_HYB, &
                            &  PTM5_HYAI, PTM5_HYBI, PTM5_P0, PTM5_PS, &
                            &  PTM5_AOD, PTM5_ABS, PTM5_ASY, PTM5_PPS, PTM5_PPSF )

    !--- inherited types, functions and data
    USE YOMHOOK, ONLY : LHOOK, DR_HOOK
    USE YOERAD   , ONLY : AEROPIFIL_OPT
    USE NETCDF

    IMPLICIT NONE

    !---subroutine interface
    INTEGER(KIND=JPIM), INTENT(IN) :: IMON
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_LEV(TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_LON(TM5_NLON)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_LAT(TM5_NLAT)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_HYA(TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_HYB(TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_HYAI(2,TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_HYBI(2,TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_P0
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_PS(TM5_NLON, TM5_NLAT)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_PPS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_PPSF(TM5_NLON, TM5_NLAT, 0:TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_AOD(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_ABS(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL)
    REAL(KIND=JPRB), INTENT(OUT) :: PTM5_ASY(TM5_NLON, TM5_NLAT, TM5_NLEV, NWVL)


!    CHARACTER(LEN=*), PARAMETER  :: CAERINPUT='/scratch/rd/pa1/ecearth/crayrundir/tm5_clim_pi_aerosol.nc'
    INTEGER                      :: INCID
    INTEGER                      :: INCRESULT

    CHARACTER (LEN=*), PARAMETER :: LAT_NAME='lat'
    CHARACTER (LEN=*), PARAMETER :: LON_NAME='lon'
    CHARACTER (LEN=*), PARAMETER :: LEV_NAME='lev'
    CHARACTER (LEN=*), PARAMETER :: WAV_NAME='wav'
    INTEGER :: LAT_VARID, LON_VARID, LEV_VARID, WAV_VARID

    CHARACTER (LEN=*), PARAMETER :: HYAM_NAME='a'
    CHARACTER (LEN=*), PARAMETER :: HYBM_NAME='b'
    INTEGER :: HYAM_VARID, HYBM_VARID

    CHARACTER (LEN=*), PARAMETER :: HYAI_NAME='a_bnds'
    CHARACTER (LEN=*), PARAMETER :: HYBI_NAME='b_bnds'
    INTEGER :: HYAI_VARID, HYBI_VARID

    CHARACTER (LEN=*), PARAMETER :: PS_NAME='ps'
    INTEGER :: PS_VARID

    CHARACTER (LEN=*), PARAMETER :: P0_NAME='p0'
    INTEGER :: P0_VARID

    CHARACTER (LEN=*), PARAMETER :: TM5_AOD_NAME='od4daer'
    CHARACTER (LEN=*), PARAMETER :: TM5_ABS_NAME='abs4daer'
    CHARACTER (LEN=*), PARAMETER :: TM5_ASY_NAME='sasy4daer'
    INTEGER :: TM5_AOD_VARID
    INTEGER :: TM5_ABS_VARID
    INTEGER :: TM5_ASY_VARID

    CHARACTER (LEN = 300) ::  CAERINPUT
    INTEGER               ::  IFL
    REAL(KIND=JPRB) :: ZHOOK_HANDLE 

    IFL=LEN_TRIM(AEROPIFIL_OPT)
    CAERINPUT=AEROPIFIL_OPT(1:IFL) 
    !---executable procedure
    IF (LHOOK) CALL DR_HOOK('READ_TM5_OPT_PROP',0,ZHOOK_HANDLE)
    
    !---open netcdf file
    INCRESULT = NF90_OPEN(CAERINPUT,NF90_NOWRITE,INCID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Open file', INCRESULT) 

    !---read lat/lon/lev
    INCRESULT = NF90_INQ_VARID(INCID, LAT_NAME, LAT_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get LAT ID', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, LON_NAME, LON_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get LON ID', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, LEV_NAME, LEV_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get LEV ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, LAT_VARID, PTM5_LAT)
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read LAT', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, LON_VARID, PTM5_LON) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read LON', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, LEV_VARID, PTM5_LEV) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read LEV', INCRESULT) 

    !--- read P0
    INCRESULT = NF90_INQ_VARID(INCID, P0_NAME, P0_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get P0 ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, P0_VARID, PTM5_P0) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read P0', INCRESULT) 

    !---read hybrid level coordinates a/b for full levels 
    INCRESULT = NF90_INQ_VARID(INCID, HYAM_NAME, HYAM_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get A ID', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, HYBM_NAME, HYBM_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get B ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, HYAM_VARID, PTM5_HYA) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read A', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, HYBM_VARID, PTM5_HYB) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read B', INCRESULT) 

    !---read hybrid level coordinates a/b for half levels 
    INCRESULT = NF90_INQ_VARID(INCID, HYAI_NAME, HYAI_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get A_BNDS ID', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, HYBI_NAME, HYBI_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get B_BNDS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, HYAI_VARID, PTM5_HYAI, & 
              & START=(/1,1/), COUNT=(/2,TM5_NLEV/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read A_BNDS', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, HYBI_VARID, PTM5_HYBI, & 
              & START=(/1,1/), COUNT=(/2,TM5_NLEV/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read B_BNDS', INCRESULT) 

    !---read surface pressure
    INCRESULT = NF90_INQ_VARID(INCID, PS_NAME, PS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get PS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, PS_VARID,  PTM5_PS, &
                 & START=(/1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read PS', INCRESULT) 

    !---read optical properties
    INCRESULT = NF90_INQ_VARID(INCID, TM5_AOD_NAME, TM5_AOD_VARID)  
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get AOD ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, TM5_AOD_VARID,  PTM5_AOD, &
                 & START=(/1,1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,NWVL,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read AOD', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, TM5_ABS_NAME, TM5_ABS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get ABS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, TM5_ABS_VARID,  PTM5_ABS, &
                 & START=(/1,1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,NWVL,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read ABS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, TM5_ASY_NAME, TM5_ASY_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get SASY ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, TM5_ASY_VARID,  PTM5_ASY, &
                 & START=(/1,1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,NWVL,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read SASY', INCRESULT) 

    !---close netcdf file
    INCRESULT = ( NF90_CLOSE(INCID) )
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Close file', INCRESULT) 


    !---surface pressure
     PTM5_PPSF(:,:,0) = PTM5_PS(:,:)
    !---other levels
    DO JLEV=1,TM5_NLEV
       DO JLAT=1,TM5_NLAT
          DO JLON=1,TM5_NLON

             PTM5_PPS(JLON,JLAT,JLEV) = &
                   &   PTM5_HYA(JLEV)*PTM5_P0 + PTM5_HYB(JLEV)*PTM5_PS(JLON,JLAT)

             PTM5_PPSF(JLON,JLAT,JLEV) = &
                   &   PTM5_HYAI(2,JLEV)*PTM5_P0 + PTM5_HYBI(2,JLEV)*PTM5_PS(JLON,JLAT)
          END DO
       END DO
    END DO


    IF (LHOOK) CALL DR_HOOK('READ_TM5_OPT_PROP',1,ZHOOK_HANDLE)
  END SUBROUTINE READ_TM5_OPT_PROP
  !--------------------------------------------------------------------------------------------------------------

  !--------------------------------------------------------------------------------------------------------------
  SUBROUTINE READ_TM5_AEROSOLS(IMON,   &
                        &  PNUM_KS,  PNUM_AS,  PNUM_CS,   &
                        &  PMSO4_KS, PMSO4_AS, PMSO4_CS,  &
                        &  PMBC_KS,  PMBC_AS,  PMBC_CS,   &
                        &  PMOC_KS,  PMOC_AS,  PMOC_CS,   &
                        &            PMSS_AS,  PMSS_CS,   &
                        &            PMDU_AS,  PMDU_CS,   &
                        &            PMBC_KI,  PMOC_KI,   &
                        &            PMDU_AI,  PMDU_CI,   &
                        &  PMNO3,    PMMSA)

    !--- inherited types, functions and data
    USE YOMHOOK, ONLY : LHOOK, DR_HOOK
    USE YOERAD   , ONLY : AEROPIFIL_CONC
    USE NETCDF

    IMPLICIT NONE

    !---subroutine interface
    INTEGER(KIND=JPIM), INTENT(IN) :: IMON
    REAL(KIND=JPRB), INTENT(OUT) :: PNUM_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PNUM_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PNUM_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMSO4_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMSO4_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMSO4_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMBC_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMBC_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMBC_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMBC_KI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMOC_KS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMOC_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMOC_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMOC_KI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMSS_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMSS_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMDU_AS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMDU_CS(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMDU_AI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMDU_CI(TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMNO3  (TM5_NLON, TM5_NLAT, TM5_NLEV)
    REAL(KIND=JPRB), INTENT(OUT) :: PMMSA  (TM5_NLON, TM5_NLAT, TM5_NLEV)

!    CHARACTER(LEN=*), PARAMETER  :: CAERINPUT='/scratch/rd/pa1/ecearth/crayrundir/tm5_clim_pi_aerosol.nc'
    INTEGER                      :: INCID
    INTEGER                      :: INCRESULT

    CHARACTER (LEN=*), PARAMETER :: NUM_KS_NAME='nconc_ais'
    CHARACTER (LEN=*), PARAMETER :: NUM_AS_NAME='nconc_acs'
    CHARACTER (LEN=*), PARAMETER :: NUM_CS_NAME='nconc_cos'
    CHARACTER (LEN=*), PARAMETER :: MSO4_KS_NAME='mmrso4_ais'
    CHARACTER (LEN=*), PARAMETER :: MSO4_AS_NAME='mmrso4_acs'
    CHARACTER (LEN=*), PARAMETER :: MSO4_CS_NAME='mmrso4_cos'
    CHARACTER (LEN=*), PARAMETER :: MBC_KS_NAME='mmrbc_ais'
    CHARACTER (LEN=*), PARAMETER :: MBC_AS_NAME='mmrbc_acs'
    CHARACTER (LEN=*), PARAMETER :: MBC_CS_NAME='mmrbc_cos'
    CHARACTER (LEN=*), PARAMETER :: MBC_KI_NAME='mmrbc_aii'
    CHARACTER (LEN=*), PARAMETER :: MOC_KS_NAME='mmroa_ais'
    CHARACTER (LEN=*), PARAMETER :: MOC_AS_NAME='mmroa_acs'
    CHARACTER (LEN=*), PARAMETER :: MOC_CS_NAME='mmroa_cos'
    CHARACTER (LEN=*), PARAMETER :: MOC_KI_NAME='mmroa_aii'
    CHARACTER (LEN=*), PARAMETER :: MSS_AS_NAME='mmrss_acs'
    CHARACTER (LEN=*), PARAMETER :: MSS_CS_NAME='mmrss_cos'
    CHARACTER (LEN=*), PARAMETER :: MDU_AS_NAME='mmrdust_acs'
    CHARACTER (LEN=*), PARAMETER :: MDU_CS_NAME='mmrdust_cos'
    CHARACTER (LEN=*), PARAMETER :: MDU_AI_NAME='mmrdust_aci'
    CHARACTER (LEN=*), PARAMETER :: MDU_CI_NAME='mmrdust_coi'
    CHARACTER (LEN=*), PARAMETER :: MNO3_NAME='mmr_no3'
    CHARACTER (LEN=*), PARAMETER :: MMSA_NAME='mmr_msa'
    INTEGER :: NUM_KS_VARID
    INTEGER :: NUM_AS_VARID
    INTEGER :: NUM_CS_VARID
    INTEGER :: MSO4_KS_VARID
    INTEGER :: MSO4_AS_VARID
    INTEGER :: MSO4_CS_VARID
    INTEGER :: MBC_KS_VARID
    INTEGER :: MBC_AS_VARID
    INTEGER :: MBC_CS_VARID
    INTEGER :: MBC_KI_VARID
    INTEGER :: MOC_KS_VARID
    INTEGER :: MOC_AS_VARID
    INTEGER :: MOC_CS_VARID
    INTEGER :: MOC_KI_VARID
    INTEGER :: MSS_AS_VARID
    INTEGER :: MSS_CS_VARID
    INTEGER :: MDU_AS_VARID
    INTEGER :: MDU_CS_VARID
    INTEGER :: MDU_AI_VARID
    INTEGER :: MDU_CI_VARID
    INTEGER :: MNO3_VARID
    INTEGER :: MMSA_VARID


    CHARACTER (LEN = 300) ::  CAERINPUT
    INTEGER               ::  IFL
    REAL(KIND=JPRB) :: ZHOOK_HANDLE 

    IFL=LEN_TRIM(AEROPIFIL_CONC)
    CAERINPUT=AEROPIFIL_CONC(1:IFL) 

    !---executable procedure
    IF (LHOOK) CALL DR_HOOK('READ_TM5_AEROSOLS',0,ZHOOK_HANDLE)
    
    !---open netcdf file
    INCRESULT = NF90_OPEN(CAERINPUT,NF90_NOWRITE,INCID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Open file(2)', INCRESULT) 

    !---read aerosol fields
    INCRESULT = NF90_INQ_VARID(INCID, NUM_KS_NAME, NUM_KS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get NUM_KS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, NUM_KS_VARID,  PNUM_KS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read NUM_KS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, NUM_AS_NAME, NUM_AS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get NUM_AS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, NUM_AS_VARID,  PNUM_AS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read NUM_AS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, NUM_CS_NAME, NUM_CS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get NUM_CS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, NUM_CS_VARID,  PNUM_CS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read NUM_CS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MSO4_KS_NAME, MSO4_KS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MSO4_KS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MSO4_KS_VARID,  PMSO4_KS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MSO4_KS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MSO4_AS_NAME, MSO4_AS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MSO4_AS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MSO4_AS_VARID,  PMSO4_AS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MSO4_AS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MSO4_CS_NAME, MSO4_CS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MSO4_CS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MSO4_CS_VARID,  PMSO4_CS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MSO4_CS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MBC_KS_NAME, MBC_KS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MBC_KS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MBC_KS_VARID,  PMBC_KS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MBC_KS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MBC_KI_NAME, MBC_KI_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MBC_KI ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MBC_KI_VARID,  PMBC_KI, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MBC_KI', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MBC_AS_NAME, MBC_AS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MBC_AS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MBC_AS_VARID,  PMBC_AS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MBC_AS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MBC_CS_NAME, MBC_CS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MBC_CS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MBC_CS_VARID,  PMBC_CS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MBC_CS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MOC_KS_NAME, MOC_KS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MOC_KS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MOC_KS_VARID,  PMOC_KS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MOC_KS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MOC_KI_NAME, MOC_KI_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MOC_KI ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MOC_KI_VARID,  PMOC_KI, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MOC_KI', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MOC_AS_NAME, MOC_AS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MOC_ASS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MOC_AS_VARID,  PMOC_AS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MOC_AS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MOC_CS_NAME, MOC_CS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MOC_CS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MOC_CS_VARID,  PMOC_CS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MOC_CS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MSS_AS_NAME, MSS_AS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MSS_AS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MSS_AS_VARID,  PMSS_AS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MSS_AS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MSS_CS_NAME, MSS_CS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MSS_CS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MSS_CS_VARID,  PMSS_CS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MSS_CS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MDU_AS_NAME, MDU_AS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MDU_AS ID', INCRESULT) 

    INCRESULT =  NF90_GET_VAR(INCID, MDU_AS_VARID,  PMDU_AS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MDU_AS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MDU_CS_NAME, MDU_CS_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MDU_CS ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MDU_CS_VARID,  PMDU_CS, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MDU_CS', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MDU_AI_NAME, MDU_AI_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MDU_AI ID', INCRESULT) 

    INCRESULT =  NF90_GET_VAR(INCID, MDU_AI_VARID,  PMDU_AI, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MDU_AI', INCRESULT) 

    INCRESULT = NF90_INQ_VARID(INCID, MDU_CI_NAME, MDU_CI_VARID) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MDU_CI ID', INCRESULT) 

    INCRESULT = NF90_GET_VAR(INCID, MDU_CI_VARID,  PMDU_CI, &
         & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) ) 
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MDU_CI', INCRESULT) 

    ! NO3 and MSA only used in cloud activation scheme
    IF (NCLOUDACT > 0) THEN
      INCRESULT = NF90_INQ_VARID(INCID, MNO3_NAME, MNO3_VARID)
      IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MNO3 ID', INCRESULT)

      INCRESULT = NF90_GET_VAR(INCID, MNO3_VARID,  PMNO3, &
           & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) )
      IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MNO3', INCRESULT)

      INCRESULT = NF90_INQ_VARID(INCID, MMSA_NAME, MMSA_VARID)
      IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Get MMSA ID', INCRESULT)

      INCRESULT = NF90_GET_VAR(INCID, MMSA_VARID,  PMMSA, &
           & START=(/1,1,1,IMON/), COUNT=(/TM5_NLON,TM5_NLAT,TM5_NLEV,1/) )
      IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Read MMSA', INCRESULT)
    ENDIF

    !---close netcdf file
    INCRESULT = ( NF90_CLOSE(INCID) )
    IF (INCRESULT /= NF90_NOERR) CALL NF_ERROR(CAERINPUT, 'Close file(2)', INCRESULT) 

    IF (LHOOK) CALL DR_HOOK('READ_TM5_AEROSOLS',1,ZHOOK_HANDLE)
  END SUBROUTINE READ_TM5_AEROSOLS
  !--------------------------------------------------------------------------------------------------------------

  !--------------------------------------------------------------------------------------------------------------
  SUBROUTINE NF_ERROR(FILE_NAME, DOING_WHAT, INCRESULT)

    USE NETCDF

    CHARACTER(LEN=*), INTENT(IN) :: FILE_NAME
    CHARACTER(LEN=*), INTENT(IN) :: DOING_WHAT
    INTEGER, INTENT(IN) :: INCRESULT

    CHARACTER(LEN=*), PARAMETER :: WS=" "   ! whitespace

    !---included functions
#include "abor1.intfb.h"

    CALL ABOR1('CMIP6_PI_AEROSOLS: File:'//FILE_NAME//WS//DOING_WHAT//WS//TRIM(NF90_STRERROR(INCRESULT)))
  END SUBROUTINE NF_ERROR
  !--------------------------------------------------------------------------------------------------------------

END SUBROUTINE CMIP6_PI_AEROSOLS
