!OPTIONS XOPT(HSFUN)
SUBROUTINE COVER &
 !---input
 & (KIDIA, KFDIA, KLON, KLEV, KTOP, KBOT, &
 & PQ, PSAT, PLSUM, PVAR, &
 & PLNEW, PCCNEW)  

! *COVER* -  

!     A.TOMPKINS     E.C.M.W.F.     2003 

!     PURPOSE
!     -------

!       THIS ROUTINE TAKE VARIANCE AND TOTAL WATER AND DIAGNOSES
!       CC AND QV AND QL.

!**   INTERFACE.
!     ----------

!     *COVER* IS CALLED FROM *CALLPAR*

!     EXTERNALS.
!     ----------
!          NONE

!     MODIFICATIONS.
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     SWITCHES.
!     --------

!     MODEL PARAMETERS
!     ----------------

!     REFERENCE.
!     ----------
!          Tompkins 2002, J. Atmos. Sci.

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOECLDP,   ONLY : NSHAPEP, NSHAPEQ, NBETA, RBETA, RBETAP1

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTOP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBOT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSAT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSUM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNEW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCCNEW(KLON,KLEV) 
!REAL_B :: PTSPHY

!---input variables
!---output variables
!---local fields
REAL(KIND=JPRB) ::             &
 & ZAF(KLON),  &!function minimum
 & ZBF(KLON),  &!function maximum
 & ZQT(KLON)  

INTEGER(KIND=JPIM) :: JK, JL, JX 
REAL(KIND=JPRB) :: ZPQ, ZX, ZFAC, ZBETAP, ZBETAPP1
REAL(KIND=JPRB) :: ZEPSEC=1.e-12,ZSIGMA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!DIR$ VFUNCTION EXPHF
!#include "fcttre.h"

!-----------------------------------------------------------------------
! 0.0 Master setup
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COVER',0,ZHOOK_HANDLE)
ZPQ=NSHAPEP+NSHAPEQ

! Master loop in Z.

DO JK=KTOP,KBOT

!-----------------------------------------------------------------------
! 1.0 setup
!-----------------------------------------------------------------------
  DO JL=KIDIA,KFDIA
    ZQT(JL)=PQ(JL,JK)+PLSUM(JL,JK) !total water
  ENDDO

!-----------------------------------------------------------------------
! 2.0 calculate A and B
!-----------------------------------------------------------------------
  DO JL=KIDIA,KFDIA
    ZSIGMA=SQRT(MAX(PVAR(JL,JK),0.0_JPRB))
    ZAF(JL)=ZQT(JL)-ZSIGMA*SQRT((ZPQ+1.0_JPRB)*NSHAPEP/NSHAPEQ)
    ZBF(JL)=(ZQT(JL)-ZAF(JL))*ZPQ/NSHAPEP+ZAF(JL)
  ENDDO

!   calculate

  DO JL=KIDIA,KFDIA
    IF (ZBF(JL)<=PSAT(JL,JK)) THEN ! clear sky points
      PCCNEW(JL,JK)=0.0_JPRB
      PLNEW(JL,JK)=0.0_JPRB
    ELSEIF (ZAF(JL)>=PSAT(JL,JK)) THEN ! overcast points
      PCCNEW(JL,JK)=1.0_JPRB
      PLNEW(JL,JK)=ZQT(JL)-PSAT(JL,JK)
    ELSE ! partially cloudy points
      ZX=(PSAT(JL,JK)-ZAF(JL))/MAX(ZBF(JL)-ZAF(JL),ZEPSEC)
      ZX=MAX(MIN(ZX,1.0_JPRB),0.0_JPRB)
      JX=INT(ZX*NBETA)
      ZFAC=ZX*NBETA-JX
      IF (JX < NBETA) THEN 
        ZBETAP  =(1.0_JPRB-ZFAC)*RBETA(JX)+ZFAC*RBETA(JX+1)
        ZBETAPP1=(1.0_JPRB-ZFAC)*RBETAP1(JX)+ZFAC*RBETAP1(JX+1)

        PCCNEW(JL,JK)=1.0_JPRB-ZBETAP
        PLNEW(JL,JK)=(ZBF(JL)-ZAF(JL))*NSHAPEP/ZPQ*(1.0_JPRB-ZBETAPP1)+ &
         & (ZAF(JL)-PSAT(JL,JK))*(1.0_JPRB-ZBETAP)  
      ELSE  
        PCCNEW(JL,JK)=0.0_JPRB
        PLNEW(JL,JK)=0.0_JPRB
      ENDIF
    ENDIF
    PLNEW(JL,JK)=MAX(PLNEW(JL,JK),0.0_JPRB)
    PCCNEW(JL,JK)=MIN(MAX(PCCNEW(JL,JK),0.0_JPRB),1.0_JPRB)
  ENDDO

ENDDO ! Vertical loop in K

IF (LHOOK) CALL DR_HOOK('COVER',1,ZHOOK_HANDLE)
END SUBROUTINE COVER
