SUBROUTINE CPSPE(&
 & KPROMA ,KSTART ,KPROF  ,KLEVS  ,KLEVXP , KLEVO, &
 & KFLDXP ,KFLDXP2,PDT,&
 & PTSAE1 ,PTLE1  ,PWSAE1,&
 & PISAE1,&
 & PWLE1  ,PSNSE1,&
 & PASNE1 ,PRSNE1 ,PTSNE1,&
 & PEXTRE1,PXTR2E1,&
 & PUOE1  ,PVOE1  ,PTOE1  ,PSOE1, & !KPP
 & PTSA1  ,PTL1   ,PWSA1,&
 & PISA1,&
 & PWL1   ,PSNS1,&
 & PASN1  ,PRSN1  ,PTSN1,&
 & PEXTRP1,PXTRP21,&
 & PUO1  ,PVO1  ,PTO1  ,PSO1 &     !KPP
 & )  

!**** *CPSPE* - COMPUTE NEW T+DT VALUES FOR SURFACE FIELDS

!     PURPOSE.                                                                  
!     --------                                                        

!     COMPUTE THE NEW T+DT VALUES FOR THE SURFACE FIELDS
!     FROM T-DT VALUES AND TENDENCIES FOR ECMWF PHYSICS

!**   INTERFACE. CALL CPSPE(...)
!     ----------
!                CPSPE IS CALLED FROM POSTPHY

!     INPUT PARAMETERS :

!     KPROMA   : HORISONTAL DIMENSION
!     KSTART   : START POINT
!     KPROF    : END POINT
!     KFLEVS   : NUMBER OF SOIL LAYERS
!     KFLEVO   : NUMBER OF OCEAN MIXED LAYERS  !KPP
!     PDT      : TIME STEP FOR PHYSICS                                                  
!     PSNSE1   : TENDENCY OF SNOW MASS PER UNIT SURFACE
!     PWLE1    : TENDENCY OF SKIN RESERVOIR WATER CONTENT
!     PTLE1    : TENDENCY OF SKIN TEMPERATURE
!     PTSAE1   : TENDENCY OF MULTI-LAYER SOIL TEMPERATURE
!     PWSAE1   : TENDENCY OF MULTI-LAYER SOIL WETNESS
!     PEXTRE1  : TENDENCY OF MULTI-LAYER EXTRA FIELDS
!     PXTR2E1  : TENDENCY OF SINGLE-LAYER EXTRA FIELDS
!     PUOE1    : TENDENCY OF OCEAN VELOCITY (U)  
!     PVOE1    : TENDENCY OF OCEAN VELOCITY (V)  
!     PTOE1    : TENDENCY OF OCEAN TEMPERATURE   
!     PSOE1    : TENDENCY OF OCEAN SALINITY      

!     OUTPUT PARAMETERS: (T+DT VALUES)

!     PSNS1    : SNOW MASS PER UNIT SURFACE
!     PWL1     : SKIN RESERVOIR WATER CONTENT
!     PTL1     : SKIN TEMPERATURE
!     PTSA1    : MULTI-LAYER SOIL TEMPERATURE
!     PWSA1    : MULTI-LAYER SOIL WETNESS
!     PEXTRP1  : MULTI-LAYER EXTRA FIELDS   
!     PXTRP21  : SINGLE-LAYER EXTRA FIELDS   
!     PUO1     : OCEAN VELOCITY (U)  
!     PVO1     : OCEAN VELOCITY (V) 
!     PTO1     : OCEAN TEMPERATURE 
!     PSO1     : OCEAN SALINITY    

!     MODIFICATION
!     ------------
!     ORIGINAL     J.-J. Morcrette
!     MODIFIED     M.HAMRUD  93-10-05 (NEW SURFACE SCHEME)
!     Modified     P Viterbo 99-03-26 Tiling of the land-surface
!     Modified     D.Salmond 15-10-01 FULLIMP mods
!     Modified     Y.Takaya  02-02-09 Ocean mixed layer model

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVXP
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVO  !KPP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDXP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDXP2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSAE1(KPROMA,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLE1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSAE1(KPROMA,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PISAE1(KPROMA,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWLE1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNSE1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PASNE1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSNE1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSNE1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXTRE1(KPROMA,KLEVXP,KFLDXP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTR2E1(KPROMA,KFLDXP2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUOE1(KPROMA,KLEVO)  !KPP
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOE1(KPROMA,KLEVO)  !KPP
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOE1(KPROMA,KLEVO)  !KPP
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOE1(KPROMA,KLEVO)  !KPP
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSA1(KPROMA,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTL1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWSA1(KPROMA,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PISA1(KPROMA,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWL1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSNS1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PASN1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRSN1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSN1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEXTRP1(KPROMA,KLEVXP,KFLDXP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXTRP21(KPROMA,KFLDXP2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUO1(KPROMA,KLEVO)   !KPP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVO1(KPROMA,KLEVO)   !KPP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTO1(KPROMA,KLEVO)   !KPP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSO1(KPROMA,KLEVO)   !KPP 
INTEGER(KIND=JPIM) :: JLEV, JROF, JVAR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*        1.   COMPUTE T+DT VALUES
!              -------------------
IF (LHOOK) CALL DR_HOOK('CPSPE',0,ZHOOK_HANDLE)
DO JLEV=1,KLEVS
  DO JROF = KSTART, KPROF
    PTSA1(JROF,JLEV) = PTSA1(JROF,JLEV) + PDT*PTSAE1(JROF,JLEV)
    PWSA1(JROF,JLEV) = PWSA1(JROF,JLEV) + PDT*PWSAE1(JROF,JLEV)
    PISA1(JROF,JLEV) = PISA1(JROF,JLEV) + PDT*PISAE1(JROF,JLEV)
  ENDDO
ENDDO
DO JROF=KSTART,KPROF
  PWL1(JROF) = PWL1(JROF)  + PDT*PWLE1(JROF)
  PTL1(JROF) = PTL1(JROF)  + PDT*PTLE1(JROF)
  PSNS1(JROF)= PSNS1(JROF) + PDT*PSNSE1(JROF)
  PASN1(JROF)= PASN1(JROF) + PDT*PASNE1(JROF)
  PRSN1(JROF)= PRSN1(JROF) + PDT*PRSNE1(JROF)
  PTSN1(JROF)= PTSN1(JROF) + PDT*PTSNE1(JROF)
ENDDO
DO JVAR=1,KFLDXP
  DO JLEV=1,KLEVXP
    DO JROF=KSTART,KPROF
      PEXTRP1(JROF,JLEV,JVAR)=PEXTRP1(JROF,JLEV,JVAR)&
       & + PDT*PEXTRE1(JROF,JLEV,JVAR)  
    ENDDO
  ENDDO
ENDDO
DO JVAR=1,KFLDXP2
  DO JROF=KSTART,KPROF
    PXTRP21(JROF,JVAR)=PXTRP21(JROF,JVAR)+PDT*PXTR2E1(JROF,JVAR)
  ENDDO
ENDDO

!-- KPP
DO JLEV=1,KLEVO
  DO JROF = KSTART, KPROF
    PUO1(JROF,JLEV) = PUO1(JROF,JLEV) + PDT*PUOE1(JROF,JLEV)   
    PVO1(JROF,JLEV) = PVO1(JROF,JLEV) + PDT*PVOE1(JROF,JLEV)   
    PTO1(JROF,JLEV) = PTO1(JROF,JLEV) + PDT*PTOE1(JROF,JLEV)   
    PSO1(JROF,JLEV) = PSO1(JROF,JLEV) + PDT*PSOE1(JROF,JLEV)   
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPSPE',1,ZHOOK_HANDLE)
END SUBROUTINE CPSPE

