SUBROUTINE CUBASEN2TL &
 & ( KIDIA,    KFDIA,    KLON,     KLEV,&
 & PTENH5,   PQENH5,   PGEOH5,   PAP5,     PAPH5,&
 & PQHFL5,   PAHFS5,&
 & PTEN5,    PQEN5,    PGEO5,&
 & PTU5,     PQU5,     PLU5,&
 & PBUOH5,   PWUBASE5, PLGLAC5,  PQPRCV5,&
 & KTYPE,    KLAB,     LDCUM,    LDSC,&
 & KCBOT,    KBOTSC,   KCTOP,    KSTUP,&
 & PTENH,    PQENH,    PGEOH,    PAP,      PAPH,&
 & PQHFL,    PAHFS,&
 & PTEN,     PQEN,     PGEO,&
 & PTU,      PQU,      PLU,&
 & PBUOH,    PWUBASE,  PLGLAC,   PQPRCV )  

!          THIS ROUTINE CALCULATES CLOUD BASE FIELDS
!          CLOUD BASE HEIGHT AND CLOUD TOP HEIGHT
!          (TANGENT LINEAR VERSION)

!          P. LOPEZ    ECMWF   (05/2002) 
!          Inspired from A. Pier Siebesma  (KNMI)
!          and C. Jakob (ECMWF) (01/2001)

!          PURPOSE.
!          --------
!          TO PRODUCE CLOUD BASE AND CLOUD TOP VALUES FOR 
!          SIMPLIFIED CONVECTIVE PARAMETRIZATION

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTRN2TL*.
!          INPUT ARE ENVIRONM. VALUES OF T,Q,P,PHI AT HALF LEVELS.
!          IT RETURNS CLOUD FIELDS VALUES AND FLAGS AS FOLLOWS;
!                 KLAB=0 FOR STABLE LAYERS
!                 KLAB=1 FOR SUBCLOUD LEVELS
!                 KLAB=2 FOR CLOUD LEVELS LEVEL

!          METHOD.
!          --------
!          LIFT SURFACE AIR DRY-ADIABATICALLY TO CLOUD TOP.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS

!    INPUT PARAMETERS (REAL):

!    *PTENH5*       ENV. TEMPERATURE (T+1) ON HALF LEVELS         (Trajectory)
!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS      (Trajectory)                
!    *PQHFL5*       MOISTURE FLUX (EXCEPT FROM SNOW EVAP.)        (Trajectory)
!    *PAHFS5*       SENSIBLE HEAT FLUX                            (Trajectory)
!    *PSSTRU5*      KINEMATIC surface U-MOMENTUM FLUX             (Trajectory)
!    *PSSTRV5*      KINEMATIC surface V-MOMENTUM FLUX             (Trajectory)
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                   (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS           (Trajectory)
!    *PTEN5*        PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)     (Trajectory)
!    *PQEN5*        PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1)  (Trajectory)
!    *PGEO5*        GEOPOTENTIAL                                  (Trajectory)
!    *PQHFL5*       MOISTURE FLUX (EXCEPT FROM SNOW EVAP.)        (Trajectory)
!    *PAHFS5*       SENSIBLE HEAT FLUX                            (Trajectory)

!    *PTENH*        ENV. TEMPERATURE (T+1) ON HALF LEVELS           K
!    *PQENH*        ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS      KG/KG
!    *PQHFL*        MOISTURE FLUX (EXCEPT FROM SNOW EVAP.)        KG/(SM2)
!    *PAHFS*        SENSIBLE HEAT FLUX                            W/M2
!    *PSSTRU*       KINEMATIC surface U-MOMENTUM FLUX             (M/S)^2
!    *PSSTRV*       KINEMATIC surface V-MOMENTUM FLUX             (M/S)^2
!    *PGEOH*        GEOPOTENTIAL ON HALF LEVELS                   M2/S2
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS             PA
!    *PTEN*         PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)       K
!    *PQEN*         PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1)  KG/KG
!    *PGEO*         GEOPOTENTIAL                                  M2/S2
!    *PQHFL*        MOISTURE FLUX (EXCEPT FROM SNOW EVAP.)        KG/(SM2)
!    *PAHFS*        SENSIBLE HEAT FLUX                            W/M2

!    UPDATED PARAMETERS (REAL):

!    *PTU5*         TEMPERATURE IN UPDRAFTS                       (Trajectory)
!    *PQU5*         SPEC. HUMIDITY IN UPDRAFTS                    (Trajectory)
!    *PLU5*         LIQUID WATER CONTENT IN UPDRAFTS              (Trajectory)

!    *PTU*          TEMPERATURE IN UPDRAFTS                         K
!    *PQU*          SPEC. HUMIDITY IN UPDRAFTS                    KG/KG
!    *PLU*          LIQUID WATER CONTENT IN UPDRAFTS              KG/KG

!    UPDATED PARAMETERS (INTEGER):

!    *KLAB*         FLAG KLAB=1 FOR SUBCLOUD LEVELS
!                        KLAB=2 FOR CLOUD LEVELS

!    OUTPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS 
!    *LDSC*         FLAG: .TRUE. IF BL-CLOUDS EXIST

!    OUTPUT PARAMETERS (INTEGER):

!    *KTYPE*       TYPE OF CONVECTION    
!    *KSTUP*       LEVEL AT WHICH THE UPDRAFT STARTS
!    *KCBOT*       CLOUD BASE LEVEL !    
!    *KCTOP*       CLOUD TOP LEVEL = HEIGHEST HALF LEVEL 
!                     WITH A NON-ZERO CLOUD UPDRAFT.
!    *KBOTSC*      CLOUD BASE LEVEL OF BL-CLOUDS

!    OUTPUT PARAMETERS (REAL):

!    *PBUOH5*      BUOYANCY ON HALF LEVELS                      (Trajectory)
!    *PLGLAC5*     FROZEN CLOUD WATER CONTENT                   (Trajectory)
!    *PQPRCV5*     CONVECTIVE PRECIPITATION CONTENT             (Trajectory)

!    *PBUOH*       BUOYANCY ON HALF LEVELS                       M/S2
!    *PLGLAC*      FROZEN CLOUD WATER CONTENT                   KG/KG 
!    *PQPRCV*      CONVECTIVE PRECIPITATION CONTENT             KG/KG 

!          EXTERNALS
!          ---------
!          *CUPDRATL*   FOR COMPUTING CHARACTERISTICS OF UPDRAFT

!          MODIFICATIONS
!          -------------
!          P.Lopez      01-Oct-2005 New version of linearized convection
!          P.Lopez      19-Oct-2007 Revised version to improve match to Tiedtke scheme

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RCPD, RETV, RD, RG
USE YOEVDF   , ONLY : RKAP
USE YOECUMF2 , ONLY : LMFMID2, RDEPTHS2, NJKT12
USE YOMCUMFS , ONLY : LREGCV

IMPLICIT NONE

INTEGER(KIND=JPIM) ,INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM) ,INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM) ,INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM) ,INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PTENH5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PAP5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PQHFL5(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PAHFS5(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PTEN5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PQEN5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PGEO5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PTU5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PQU5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PLU5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PBUOH5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PWUBASE5(KLON) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PLGLAC5(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PQPRCV5(KLON,KLEV) 
INTEGER(KIND=JPIM) ,INTENT(OUT)   :: KTYPE(KLON) 
INTEGER(KIND=JPIM) ,INTENT(OUT)   :: KLAB(KLON,KLEV) 
LOGICAL            ,INTENT(OUT)   :: LDCUM(KLON) 
LOGICAL            ,INTENT(OUT)   :: LDSC(KLON) 
INTEGER(KIND=JPIM) ,INTENT(OUT)   :: KCBOT(KLON) 
INTEGER(KIND=JPIM) ,INTENT(OUT)   :: KBOTSC(KLON) 
INTEGER(KIND=JPIM) ,INTENT(OUT)   :: KCTOP(KLON) 
INTEGER(KIND=JPIM) ,INTENT(OUT)   :: KSTUP(KLON) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PTENH(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PQENH(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PGEOH(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PQHFL(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PAHFS(KLON,KLEV+1) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PTEN(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PQEN(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(IN)    :: PGEO(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PTU(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PQU(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(INOUT) :: PLU(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PBUOH(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PWUBASE(KLON) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PLGLAC(KLON,KLEV) 
REAL(KIND=JPRB)    ,INTENT(OUT)   :: PQPRCV(KLON,KLEV) 

!             LOCAL STORAGE
!             ----- -------

LOGICAL :: LLGO_ON(KLON),  LLMID(KLON)

INTEGER(KIND=JPIM) :: IKSTART, JK, JL, IKB, JKK, JKT1

REAL(KIND=JPRB)    :: ZSENH5(KLON,KLEV+1),  ZSUH5(KLON,KLEV),&
                    & ZWU2H5(KLON,KLEV)  
REAL(KIND=JPRB)    :: ZSENH(KLON,KLEV+1),   ZSUH(KLON,KLEV),&
                    & ZWU2H(KLON,KLEV) 
 
REAL(KIND=JPRB)     :: ZTU5(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZQU5(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZLU5(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZBUOH5(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZLGLAC5(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZQPRCV5(KLON,KLEV)
REAL(KIND=JPRB)     :: ZZWU2H5(KLON,KLEV)
REAL(KIND=JPRB)     :: ZZSUH5(KLON,KLEV)  

REAL(KIND=JPRB)     :: ZTU05(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZQU05(KLON,KLEV) 

INTEGER(KIND=JPIM)  :: ILAB(KLON,KLEV) 
INTEGER(KIND=JPIM)  :: ICBOT(KLON) 
INTEGER(KIND=JPIM)  :: IBOTSC(KLON) 
INTEGER(KIND=JPIM)  :: ICTOP(KLON) 
INTEGER(KIND=JPIM)  :: ISTUP(KLON)
LOGICAL             :: LLCUM(KLON) 
LOGICAL             :: LLSC(KLON) 
LOGICAL             :: LLFOUND_CV(KLON) 

REAL(KIND=JPRB)     :: ZTU(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZQU(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZLU(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZBUOH(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZLGLAC(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZQPRCV(KLON,KLEV)
REAL(KIND=JPRB)     :: ZZWU2H(KLON,KLEV)
REAL(KIND=JPRB)     :: ZZSUH(KLON,KLEV) 

REAL(KIND=JPRB)     :: ZTU0(KLON,KLEV) 
REAL(KIND=JPRB)     :: ZQU0(KLON,KLEV) 

REAL(KIND=JPRB) :: ZRHO        ! DENSITY AT SURFACE            (KG/M^3) 
REAL(KIND=JPRB) :: ZUST        ! U_STAR                  (M/S)
REAL(KIND=JPRB) :: ZKHVFL    ! SURFACE BUOYANCY FLUX            (K M/S)
REAL(KIND=JPRB) :: ZWS        ! SIGMA_W            AT LOWEST MODEL HALFLEVEL    (M/S)
REAL(KIND=JPRB) :: ZQEXC        ! HUMIDITY    EXCESS AT LOWEST MODEL HALFLEVEL    (#)
REAL(KIND=JPRB) :: ZTEXC        ! TEMPERATURE EXCESS AT LOWEST MODEL HALFLEVEL    (K)
REAL(KIND=JPRB) :: ZTVENH    ! ENVIRONMENT VIRTUAL TEMPERATURE AT HALF LEVELS (K)  
REAL(KIND=JPRB) :: ZTVUH     ! UPDRAFT VIRTUAL TEMPERATURE AT HALF LEVELS     (K)
REAL(KIND=JPRB) :: ZDEPTH5, ZRHO5, ZUST5, ZKHVFL5, ZWS5, ZQEXC5, ZTEXC5,&
                 & ZTVENH5,ZTVUH5,ZFACT3,ZFACT4,ZFACT5,ZFACT6,ZFACT7
REAL(KIND=JPRB) :: ZWORK15, ZWORK25
REAL(KIND=JPRB) :: ZWORK1, ZWORK2
REAL(KIND=JPRB) :: ZRCPD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include  "cupdratl.intfb.h"

!----------------------------------------------------------------------
!     0.           INITIALIZE CONSTANTS AND FIELDS
!                  -------------------------------
!----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CUBASEN2TL',0,ZHOOK_HANDLE)

JKT1=NJKT12
ZRCPD=1.0_JPRB/RCPD

DO JL=KIDIA,KFDIA
  LLGO_ON(JL)=.FALSE.
  LLMID(JL)=.FALSE.
ENDDO
 
DO JL=KIDIA,KFDIA
  PWUBASE5(JL)=0.0_JPRB
  PWUBASE(JL)=0.0_JPRB
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PBUOH5 (JL,JK)=0.0_JPRB
    PBUOH  (JL,JK)=0.0_JPRB
    PQPRCV5(JL,JK)=0.0_JPRB
    PQPRCV (JL,JK)=0.0_JPRB
    PLGLAC5(JL,JK)=0.0_JPRB
    PLGLAC (JL,JK)=0.0_JPRB  
  ENDDO
ENDDO

!----------------------------------------------------------------------
!       -----------------------------------------------------------
!       1.1  PREPARE FIELDS ON HALF LEVELS BY LINEAR INTERPOLATION
!             OF SPECIFIC HUMIDITY AND STATIC ENERGY
!       -----------------------------------------------------------

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZSUH5(JL,JK)=0.0_JPRB
    ZSUH (JL,JK)=0.0_JPRB
    ZWU2H5(JL,JK)=0.0_JPRB
    ZWU2H (JL,JK)=0.0_JPRB
    ZSENH5(JL,JK)=RCPD*PTENH5(JL,JK)+PGEOH5(JL,JK)
    ZSENH (JL,JK)=RCPD*PTENH (JL,JK)+PGEOH (JL,JK)
    ZTU05(JL,JK)=PTU5(JL,JK)
    ZTU0 (JL,JK)=PTU (JL,JK)
    ZQU05(JL,JK)=PQU5(JL,JK)
    ZQU0 (JL,JK)=PQU (JL,JK)
  ENDDO
ENDDO

KCBOT  (:)  =-1      ! cloud base level for convection, (-1 if not found)
KSTUP  (:)  =-1      ! level at which the updraft starts (-1 if not found)
KBOTSC (:)  =-1      ! sc    base level for sc-clouds , (-1 if not found)
KCTOP  (:)  =-1      ! cloud top for convection (-1 if not found)
KTYPE  (:)  =0       ! convection type (0 if no convection)
LDCUM  (:)  =.FALSE. ! on exit: true if cloudbase=found
LDSC   (:)  =.FALSE. ! on exit: true if cloudbase=found

LLFOUND_CV(:)=.FALSE.

DO JK=KLEV-1,JKT1,-1

! Initialize local arrays
  ICBOT(:)=-1
  ISTUP(:)=-1
  IBOTSC(:)=-1
  ICTOP(:)=-1
  LLCUM(:)=.FALSE.
  LLSC(:)=.FALSE.

  DO JKK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZBUOH5 (JL,JKK)=0.0_JPRB
      ZBUOH  (JL,JKK)=0.0_JPRB
      ZQPRCV5(JL,JKK)=0.0_JPRB
    ENDDO
  ENDDO
  DO JKK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZQPRCV (JL,JKK)=0.0_JPRB
      ZLGLAC5(JL,JKK)=0.0_JPRB  
      ZLGLAC (JL,JKK)=0.0_JPRB  
    ENDDO
  ENDDO
  DO JKK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTU5   (JL,JKK)=ZTU05(JL,JKK)
      ZTU    (JL,JKK)=ZTU0 (JL,JKK)
      ZQU5   (JL,JKK)=ZQU05(JL,JKK)
      ZQU    (JL,JKK)=ZQU0 (JL,JKK)
    ENDDO
  ENDDO
  DO JKK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZLU5   (JL,JKK)=0.0_JPRB
      ZLU    (JL,JKK)=0.0_JPRB
      ZZWU2H5(JL,JKK)=0.0_JPRB
    ENDDO
  ENDDO
  DO JKK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZZWU2H (JL,JKK)=0.0_JPRB    
      ZZSUH5 (JL,JKK)=0.0_JPRB
      ZZSUH  (JL,JKK)=0.0_JPRB   
    ENDDO
  ENDDO

  IF (JK == KLEV-1) THEN

!        ---------------------------------------------------------
!        1.2    INITIALISE FIELDS AT LOWEST HALF MODEL LEVEL
!        ---------------------------------------------------------

!LOP     LL_LDO_ASC=.TRUE.

    DO JL=KIDIA,KFDIA
  
      ZRHO5 = PAPH5(JL,KLEV+1)/(RD*(PTEN5(JL,KLEV)*(1.0_JPRB+RETV*PQEN5(JL,KLEV))))
      ZRHO  = ZRHO5 * ( PAPH(JL,KLEV+1)/PAPH5(JL,KLEV+1) &
       & - PTEN(JL,KLEV)/PTEN5(JL,KLEV) &
       & - RETV*PQEN(JL,KLEV)/(1.0_JPRB+RETV*PQEN5(JL,KLEV)) )  

!LOP      ZSQRTUV5=SQRT(PSSTRU5(JL)**2 + PSSTRV5(JL)**2)
!LOP      IF (ZSQRTUV5 > ZREPUST) THEN
!LOP        ZUST5 = ZSQRTUV5
!LOP        ZUST = (PSSTRU(JL)*PSSTRU5(JL)+PSSTRV(JL)*PSSTRV5(JL))/ZUST5
!LOP      ELSE
!LOP        ZUST5 = ZREPUST
!LOP        ZUST = 0.0_JPRB
!LOP      ENDIF
      ZUST5 = 0.1_JPRB
      ZUST = 0.0_JPRB

      ZKHVFL5 = (PAHFS5(JL,KLEV+1)*ZRCPD &
       & + RETV*PTEN5(JL,KLEV)*PQHFL5(JL,KLEV+1))/ZRHO5  
      ZKHVFL  = (PAHFS(JL,KLEV+1)*ZRCPD &
       & + RETV*(PTEN(JL,KLEV)*PQHFL5(JL,KLEV+1) &
       & + PTEN5(JL,KLEV)*PQHFL(JL,KLEV+1)) &
       & - ZKHVFL5*ZRHO) / ZRHO5  

      ZFACT3 = ZUST5**3 &
       & - 1.5_JPRB*RKAP*ZKHVFL5*PGEOH5(JL,KLEV)/PTEN5(JL,KLEV)  
      ZWS5  = 1.2_JPRB * MAX(ZFACT3,0.0_JPRB)**(1.0_JPRB/3._JPRB)
      IF (ZFACT3 > 0.0_JPRB) THEN
        ZFACT7=1.0_JPRB/(ZWS5*ZWS5)
        ZFACT6=1.728_JPRB*0.5_JPRB*RKAP/PTEN5(JL,KLEV)*ZFACT7
        ZWS = 1.728_JPRB * ZFACT7 * ( ZUST*ZUST5*ZUST5 ) &
         & - ZFACT6 * (ZKHVFL*PGEOH5(JL,KLEV) &
         & + ZKHVFL5*PGEOH(JL,KLEV)-ZKHVFL5*PGEOH5(JL,KLEV)*PTEN(JL,KLEV) &
         & / PTEN5(JL,KLEV))  
! Reduce perturbation of w* if required
        IF (LREGCV .AND. ABS(ZWS5) < 0.5_JPRB) THEN
          ZWS = 0.0_JPRB
        ENDIF
      ELSE
        ZWS = 0.0_JPRB
      ENDIF

      IF (ZKHVFL5 < 0.0_JPRB) THEN
        LLGO_ON(JL)  = .TRUE.
        ILAB(JL,KLEV)= 1
        ZFACT4=-1.5_JPRB*PAHFS5(JL,KLEV+1)/(ZRHO5*ZWS5*RCPD)
        ZTEXC5 = MAX(ZFACT4,0.0_JPRB)
        IF (ZFACT4 > 0.0_JPRB) THEN
          ZTEXC = ZFACT4*(PAHFS(JL,KLEV+1)/PAHFS5(JL,KLEV+1) &
           & - ZRHO/ZRHO5 - ZWS/ZWS5)   
        ELSE
          ZTEXC = 0.0_JPRB
        ENDIF
        ZFACT5=-1.5_JPRB*PQHFL5(JL,KLEV+1)/(ZRHO5*ZWS5)
        ZQEXC5 = MAX(ZFACT5,0.0_JPRB)
        IF (ZFACT5 > 0.0_JPRB) THEN
          ZQEXC = ZFACT5*(PQHFL(JL,KLEV+1)/PQHFL5(JL,KLEV+1) &
           & - ZRHO/ZRHO5 - ZWS/ZWS5)   
        ELSE
          ZQEXC = 0.0_JPRB
        ENDIF
        ZQU5(JL,KLEV) = PQENH5(JL,KLEV) + ZQEXC5
        ZQU (JL,KLEV) = PQENH(JL,KLEV)  + ZQEXC
        ZZSUH5(JL,KLEV)= ZSENH5(JL,KLEV) + RCPD*ZTEXC5
        ZZSUH (JL,KLEV)= ZSENH(JL,KLEV)  + RCPD*ZTEXC
        ZTU5(JL,KLEV) = (ZSENH5(JL,KLEV)-PGEOH5(JL,KLEV))*ZRCPD + ZTEXC5
        ZTU (JL,KLEV) = (ZSENH(JL,KLEV)-PGEOH(JL,KLEV))*ZRCPD + ZTEXC
        ZZWU2H5(JL,KLEV)= ZWS5*ZWS5
        ZZWU2H(JL,KLEV) = 2.0_JPRB*ZWS*ZWS5

!  determine buoyancy at lowest half level

        ZTVENH5=(1.0_JPRB+RETV*PQENH5(JL,KLEV)) &
         & * (ZSENH5(JL,KLEV)-PGEOH5(JL,KLEV))*ZRCPD  
        ZTVENH=(RETV*PQENH(JL,KLEV)*(ZSENH5(JL,KLEV)-PGEOH5(JL,KLEV)) &
         & + (1.0_JPRB+RETV*PQENH5(JL,KLEV))*(ZSENH(JL,KLEV)-PGEOH(JL,KLEV))) &
         & * ZRCPD  
        ZTVUH5 = (1.0_JPRB+RETV*ZQU5(JL,KLEV))*ZTU5(JL,KLEV)
        ZTVUH  = RETV*ZQU(JL,KLEV)*ZTU5(JL,KLEV) &
         & + (1.0_JPRB+RETV*ZQU5(JL,KLEV))*ZTU(JL,KLEV)  
        ZBUOH5(JL,KLEV) = (ZTVUH5-ZTVENH5)*RG/ZTVENH5
        ZBUOH(JL,KLEV) = RG*(ZTVUH*ZTVENH5-ZTVUH5*ZTVENH)/(ZTVENH5**2)

! Reduce buoyancy perturbation if required
        IF (LREGCV) THEN
          ZBUOH(JL,KLEV) = ZBUOH(JL,KLEV)*0.35_JPRB
        ENDIF
      ELSE
        LLGO_ON(JL)=.FALSE.    ! non-convective point
      ENDIF
    ENDDO

  ELSE

!        -------------------------------------------------
!        1.3    CHECK FOR "MID-LEVEL" CONVECTION 
!               AND INITIALISE FIELDS BEFORE ASCENT 
!               (SIMPLIFIED SCHEME)
!        -------------------------------------------------
 
!LOP    LL_LDO_ASC=.TRUE.

    DO JL=KIDIA,KFDIA
 
      IF (.NOT.LLFOUND_CV(JL) .AND. PGEOH5(JL,JK+1) < 1.5E+05_JPRB) THEN
 
        LLGO_ON(JL)=.TRUE.
 
        ZTEXC5=0.2_JPRB
        ZTEXC =0.0_JPRB
        ZQEXC5=1.E-04_JPRB
        ZQEXC =0.0_JPRB
        ZQU5(JL,JK+1)=PQENH5(JL,JK+1) + ZQEXC5
        ZQU (JL,JK+1)=PQENH (JL,JK+1) + ZQEXC
        ZZSUH5(JL,JK+1)=ZSENH5(JL,JK+1) + RCPD*ZTEXC5
        ZZSUH (JL,JK+1)=ZSENH (JL,JK+1) + RCPD*ZTEXC
        ZTU5(JL,JK+1)=(ZSENH5(JL,JK+1)-PGEOH5(JL,JK+1))*ZRCPD + ZTEXC5
        ZTU (JL,JK+1)=(ZSENH (JL,JK+1)-PGEOH (JL,JK+1))*ZRCPD + ZTEXC
        ZLU5(JL,JK+1)=0.0_JPRB
        ZLU (JL,JK+1)=0.0_JPRB
        ZZWU2H5(JL,JK+1)=1.0_JPRB
        ZZWU2H (JL,JK+1)=0.0_JPRB
        ILAB(JL,JK+1)=1

! construct mixed layer for parcels emanating in lowest 60 hPa
        IF (PAPH5(JL,KLEV+1)-PAPH5(JL,JK)<60.E2_JPRB) THEN
          ZQU5(JL,JK+1)  =0.0_JPRB
          ZQU (JL,JK+1)  =0.0_JPRB
          ZZSUH5(JL,JK+1)=0.0_JPRB
          ZZSUH (JL,JK+1)=0.0_JPRB
          ZWORK15        =0.0_JPRB
          ZWORK1         =0.0_JPRB
          DO JKK=JK+2,JK,-1
            IF( ZWORK15 < 50.E2_JPRB ) THEN
              ZWORK25=PAPH5(JL,JKK)-PAPH5(JL,JKK-1)
              ZWORK2 =PAPH (JL,JKK)-PAPH (JL,JKK-1)
              ZWORK15      =ZWORK15+ZWORK25
              ZWORK1       =ZWORK1 +ZWORK2
              ZQU5(JL,JK+1)=ZQU5(JL,JK+1) +PQENH5(JL,JKK)*ZWORK25
              ZQU (JL,JK+1)=ZQU (JL,JK+1) +PQENH (JL,JKK)*ZWORK25 &
                                        & +PQENH5(JL,JKK)*ZWORK2
              ZZSUH5(JL,JK+1)=ZZSUH5(JL,JK+1)+ZSENH5(JL,JKK)*ZWORK25
              ZZSUH (JL,JK+1)=ZZSUH (JL,JK+1)+ZSENH (JL,JKK)*ZWORK25 &
                                        &    +ZSENH5(JL,JKK)*ZWORK2
            ENDIF
          ENDDO
          ZQU (JL,JK+1)  =(ZQU(JL,JK+1)-ZWORK1*ZQU5(JL,JK+1)/ZWORK15)/ZWORK15 &
                       & + ZQEXC
          ZQU5(JL,JK+1)  = ZQU5(JL,JK+1)/ZWORK15 + ZQEXC5
          ZZSUH (JL,JK+1)=(ZZSUH (JL,JK+1)-ZWORK1*ZZSUH5(JL,JK+1)/ZWORK15)/ZWORK15 &
                       & + RCPD*ZTEXC
          ZZSUH5(JL,JK+1)= ZZSUH5(JL,JK+1)/ZWORK15 + RCPD*ZTEXC5
          ZTU5(JL,JK+1)  =(ZZSUH5(JL,JK+1)-PGEOH5(JL,JK+1))*ZRCPD + ZTEXC5
          ZTU (JL,JK+1)  =(ZZSUH (JL,JK+1)-PGEOH (JL,JK+1))*ZRCPD + ZTEXC
        ENDIF
 
!  determine buoyancy 
 
        ZTVENH5 = (1.0_JPRB+RETV*PQENH5(JL,JK+1)) &
         & *(ZSENH5(JL,JK+1)-PGEOH5(JL,JK+1))*ZRCPD  
        ZTVENH = ( RETV*PQENH(JL,JK+1)*(ZSENH5(JL,JK+1)-PGEOH5(JL,JK+1)) &
         & + (1.0_JPRB+RETV*PQENH5(JL,JK+1)) &
         & * (ZSENH(JL,JK+1)-PGEOH(JL,JK+1)) )*ZRCPD  
        ZTVUH5 = (1.0_JPRB+RETV*ZQU5(JL,JK+1))*ZTU5(JL,JK+1)
        ZTVUH = RETV*ZQU(JL,JK+1)*ZTU5(JL,JK+1) &
         & + (1.0_JPRB+RETV*ZQU5(JL,JK+1))*ZTU(JL,JK+1)  
        ZBUOH5(JL,JK+1) = (ZTVUH5-ZTVENH5)*RG/ZTVENH5
        ZBUOH(JL,JK+1) = RG*(ZTVUH*ZTVENH5-ZTVUH5*ZTVENH)/(ZTVENH5**2)

! Reduce buoyancy perturbation if required
        IF (LREGCV) THEN
          ZBUOH(JL,JK+1) = ZBUOH(JL,JK+1)*0.35_JPRB
        ENDIF

      ELSE
        LLGO_ON(JL)=.FALSE.
      ENDIF

    ENDDO

  ENDIF

!----------------------------------------------------------------------

!     2.0          DO ASCENT IN SUBCLOUD AND LAYER,
!                  CHECK FOR EXISTENCE OF CONDENSATION LEVEL,
!                  ADJUST T,Q AND L ACCORDINGLY IN *CUADJTQ*,
!                  CHECK FOR BUOYANCY AND SET FLAGS
!                  -------------------------------------
!       ------------------------------------------------------------
!        2.1  DO THE VERTICAL ASCENT UNTIL VELOCITY BECOMES NEGATIVE
!       ------------------------------------------------------------

!LOP  IF (LL_LDO_ASC) THEN

    IKSTART=JK

    CALL CUPDRATL &
     & ( KIDIA,    KFDIA,    KLON,   KLEV,&
     & IKSTART,  LLGO_ON,&
     & PQENH5,   PGEOH5,   PAPH5,    PGEO5,&
     & ZTU5,     ZQU5,     ZLU5,&
     & ZSENH5,   ZZWU2H5,  ZZSUH5,&
     & ZBUOH5,   ZLGLAC5,  ZQPRCV5,&
     & ILAB,     LLCUM,    LLSC,     ICBOT,&
     & IBOTSC,   ICTOP,    ISTUP,&
     & PQENH,    PGEOH,    PAPH,     PGEO,&
     & ZTU,      ZQU,      ZLU,&
     & ZSENH,    ZZWU2H,   ZZSUH,&
     & ZBUOH,    ZLGLAC,   ZQPRCV ) 
 
    DO JL=KIDIA,KFDIA
      IF (LLCUM(JL)) THEN
        ZDEPTH5=(PAPH5(JL,ICBOT(JL))-PAPH5(JL,ICTOP(JL)))
        IF ((IKSTART == KLEV-1 .AND. ZDEPTH5 <  RDEPTHS2) .OR. &
          & (IKSTART  < KLEV-1 .AND. ZDEPTH5 >= RDEPTHS2)) THEN
          DO JKK=1,KLEV
            PTU5(JL,JKK)=ZTU5(JL,JKK)
            PTU (JL,JKK)=ZTU (JL,JKK)
            PQU5(JL,JKK)=ZQU5(JL,JKK)
            PQU (JL,JKK)=ZQU (JL,JKK)
            PLU5(JL,JKK)=ZLU5(JL,JKK)
            PLU (JL,JKK)=ZLU (JL,JKK)
            ZWU2H5(JL,JKK)=ZZWU2H5(JL,JKK)
            ZWU2H (JL,JKK)=ZZWU2H (JL,JKK)
            ZSUH5(JL,JKK)=ZZSUH5(JL,JKK)
            ZSUH (JL,JKK)=ZZSUH (JL,JKK)
            PBUOH5(JL,JKK)=ZBUOH5(JL,JKK)
            PBUOH (JL,JKK)=ZBUOH (JL,JKK)
            PLGLAC5(JL,JKK)=ZLGLAC5(JL,JKK)
            PLGLAC (JL,JKK)=ZLGLAC (JL,JKK)
            PQPRCV5(JL,JKK)=ZQPRCV5(JL,JKK)
            PQPRCV (JL,JKK)=ZQPRCV (JL,JKK)
            KLAB(JL,JKK)=ILAB(JL,JKK)
          ENDDO
          LDCUM(JL)=LLCUM(JL)
          LDSC(JL)=LLSC(JL)
          KCBOT(JL)=ICBOT(JL)
          KBOTSC(JL)=IBOTSC(JL)
          KCTOP(JL)=ICTOP(JL)
          KSTUP(JL)=ISTUP(JL)
          IF (IKSTART < KLEV-1) LLFOUND_CV(JL)=.TRUE.
        ENDIF
      ENDIF
    ENDDO

!LOP     LL_LDO_ASC=.FALSE.

!LOP  ENDIF

ENDDO

!----------------------------------------------------------------------

! Set flag for mid-level convection in simplified scheme

IF (LMFMID2) THEN
  DO JL=KIDIA,KFDIA
    IF (LDCUM(JL).AND.LLMID(JL)) KTYPE(JL)=3
  ENDDO
ENDIF

! Set updraft vertical velocity at cloud base 

DO JL=KIDIA,KFDIA
  IF (LDCUM(JL)) THEN
    IKB=KCBOT(JL)
    IF (ZWU2H5(JL,IKB) > 0.0_JPRB) THEN 
      PWUBASE5(JL)=SQRT(ZWU2H5(JL,IKB))
      PWUBASE (JL)=0.5_JPRB*ZWU2H(JL,IKB)/PWUBASE5(JL)
    ELSE
      PWUBASE5(JL)=0.0_JPRB
      PWUBASE (JL)=0.0_JPRB
    ENDIF 
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('CUBASEN2TL',1,ZHOOK_HANDLE)
END SUBROUTINE CUBASEN2TL
