SUBROUTINE CUCTRACERTL &
 & ( KIDIA,    KFDIA,    KLON,    KLEV,  KTRAC,&
 & KCTOP,    KDTOP,&
 & LDCUM,    LDDRAF,   PTSPHY,&
 & PAPH5,    PMFU5,    PMFD5,&
 & PUDRATE5, PDDRATE5,&
 & PDMFUP5,  PDMFDP5,&
 & PCEN5,    PTENC5,   PSCAV5,&
 & PAPH,     PMFU,     PMFD,&
 & PUDRATE,  PDDRATE,&
 & PDMFUP,   PDMFDP,&
 & PCEN,     PTENC  )  

!**** *CUCTRACERTL* - COMPUTE CONVECTIVE TRANSPORT OF CHEM. TRACERS
!                     IMPORTANT: ROUTINE IS FOR POSITIVE DEFINIT QUANTITIES
!                     TANGENT-LINEAR VERSION

!          P.BECHTOLD        E.C.M.W.F.              11/02/2004

!**   INTERFACE.
!     ----------

!          *CUTRACERTL* IS CALLED FROM *CUMASTRNTL*

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KTRAC*        NUMBER OF CHEMICAL TRACERS

!    *KCTOP*        CLOUD TOP  LEVEL
!    *KDTOP*        DOWNDRAFT TOP LEVEL

!    INPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS 
!    *LDDRAF*       FLAG: .TRUE. IF DOWNDRAFTS EXIST

!    INPUT PARAMETERS (REAL):

!    *PTSPHY*       PHYSICS TIME-STEP                              S
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS            PA

!    Trajectory arrays

!    *PCEN5*        PROVISIONAL ENVIRONMENT TRACER CONCENTRATION
!    *PMFU5*        MASSFLUX UPDRAFTS                             KG/(M2*S)
!    *PMFD5*        MASSFLUX DOWNDRAFTS                           KG/(M2*S)
!    *PUDRATE5*     UPDRAFT DETRAINMENT                           KG/(M2*S)
!    *PDDRATE5*     DOWNDRAFT DETRAINMENT                         KG/(M2*S)
!    *PDMFUP5*      UPDRAUGT PRECIP PRODUCTION IN LAYER           KG/(M2*S)
!    *PDMFDP5*      DDRAUGHT PRECIP PRODUCTION IN LAYER           KG/(M2*S)
!    *PSCAV5*       SCAVENGING COEFFICIENT                        UNITLESS

!    TL perturbation arrays

!    *PCEN*         PROVISIONAL ENVIRONMENT TRACER CONCENTRATION
!    *PMFU*         MASSFLUX UPDRAFTS                             KG/(M2*S)
!    *PMFD*         MASSFLUX DOWNDRAFTS                           KG/(M2*S)
!    *PUDRATE       UPDRAFT DETRAINMENT                           KG/(M2*S)
!    *PDDRATE       DOWNDRAFT DETRAINMENT                         KG/(M2*S)
!    *PDMFUP*       UPDRAUGT PRECIP PRODUCTION IN LAYER           KG/(M2*S)
!    *PDMFDP*       DDRAUGHT PRECIP PRODUCTION IN LAYER           KG/(M2*S)

!    UPDATED PARAMETERS (REAL):

!    Trajectory arrays

!    *PTENC5*       UPDATED TENDENCY OF CHEM. TRACERS              1/S

!    TL perturbation arrays

!    *PTENC*        UPDATED TENDENCY OF CHEM. TRACERS              1/S

!          METHOD
!          -------
!     EXPLICIT CENTRED-UPSTREAM AND IMPLICIT SOLUTION OF VERTICAL ADVECTION
!     DEPENDING ON VALUE OF RMFSOLCT: 0=EXPLICIT 1=IMPLICIT 0-1 SEMI-IMPLICIT

!     FOR EXPLICIT SOLUTION: ONLY ONE SINGLE ITERATION
!     FOR IMPLICIT SOLUTION: FIRST IMPLICIT SOLVER, THEN EXPLICIT SOLVER
!                            TO CORRECT TENDENCIES BELOW CLOUD BASE

!          EXTERNALS
!          ---------
!          CUBIDIAGTL

!          MODIFICATIONS
!          -------------
!          P.LOPEZ,   E.C.M.W.F.  11/02/2004 : Tangent-linear version
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG
USE YOECUMF  , ONLY : RMFSOLCT,  RMFCMIN
USE YOECUMF2 , ONLY : RMFSOLCT2, RMFCMIN2
USE YOMCUMFS , ONLY : LECUMFS2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDTOP(KLON) 
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON) 
LOGICAL           ,INTENT(IN)    :: LDDRAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUDRATE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDDRATE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDMFUP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDMFDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCEN5(KLON,KLEV,KTRAC) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENC5(KLON,KLEV,KTRAC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSCAV5(KTRAC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUDRATE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDDRATE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDMFUP(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDMFDP(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCEN(KLON,KLEV,KTRAC) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENC(KLON,KLEV,KTRAC) 
! Trajectory arrays

! Tangent-linear perturbation arrays

INTEGER(KIND=JPIM) :: IK, JK, JL, JN

REAL(KIND=JPRB) :: ZZP5, ZMFA5, ZERATE5, ZPOSI5, ZIMP, ZTSPHY
REAL(KIND=JPRB) :: ZZP , ZMFA , ZERATE , ZFACT
REAL(KIND=JPRB) :: ZRMFCMIN, ZRMFSOLCT

!     ALLOCATABLE ARAYS
REAL(KIND=JPRB), DIMENSION(:,:,:), ALLOCATABLE :: ZCEN5, ZCU5, ZCD5, ZTENC5, ZMFC5
REAL(KIND=JPRB), DIMENSION(:,:),   ALLOCATABLE :: ZDP5, ZB5,  ZR15
REAL(KIND=JPRB), DIMENSION(:,:,:), ALLOCATABLE :: ZCEN, ZCU, ZCD, ZTENC, ZMFC
REAL(KIND=JPRB), DIMENSION(:,:),   ALLOCATABLE :: ZDP, ZB,  ZR1
LOGICAL, DIMENSION(:,:),  ALLOCATABLE :: LLCUMASK, LLCUMBAS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cubidiagtl.intfb.h"
!----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CUCTRACERTL',0,ZHOOK_HANDLE)

IF (LECUMFS2) THEN
  ZRMFSOLCT=RMFSOLCT2
  ZRMFCMIN=RMFCMIN2
ELSE
  ZRMFSOLCT=RMFSOLCT
  ZRMFCMIN=RMFCMIN
ENDIF

ZIMP=1.0_JPRB-ZRMFSOLCT
ZTSPHY=1._JPRB/PTSPHY

! Trajectory arrays

ALLOCATE(ZCEN5(KLON,KLEV,KTRAC)) !Half-level environmental values
ALLOCATE(ZCU5(KLON,KLEV,KTRAC))  !Updraft values
ALLOCATE(ZCD5(KLON,KLEV,KTRAC))  !Downdraft values
ALLOCATE(ZTENC5(KLON,KLEV,KTRAC))!Tendency
ALLOCATE(ZMFC5(KLON,KLEV,KTRAC)) !Fluxes
ALLOCATE(ZDP5(KLON,KLEV))        !Pressure difference
ALLOCATE(LLCUMASK(KLON,KLEV))    !Mask for convection

! Tangent-linear perturbation arrays

ALLOCATE(ZCEN(KLON,KLEV,KTRAC)) !Half-level environmental values
ALLOCATE(ZCU(KLON,KLEV,KTRAC))  !Updraft values
ALLOCATE(ZCD(KLON,KLEV,KTRAC))  !Downdraft values
ALLOCATE(ZTENC(KLON,KLEV,KTRAC))!Tendency
ALLOCATE(ZMFC(KLON,KLEV,KTRAC)) !Fluxes
ALLOCATE(ZDP(KLON,KLEV))        !Pressure difference

! Initialize Cumulus mask + some setups

DO JK=2,KLEV
   DO JL=KIDIA,KFDIA
      LLCUMASK(JL,JK)=.FALSE.
      IF( LDCUM(JL) ) THEN
          ZDP5(JL,JK)=RG/(PAPH5(JL,JK+1)-PAPH5(JL,JK))
          ZFACT=ZDP5(JL,JK)/(PAPH5(JL,JK+1)-PAPH5(JL,JK))
          ZDP (JL,JK)=-(PAPH(JL,JK+1)-PAPH(JL,JK))*ZFACT
          IF(JK>=KCTOP(JL)-1) THEN
             LLCUMASK(JL,JK)=.TRUE.
          ENDIF 
      ENDIF 
   ENDDO 
ENDDO 
!----------------------------------------------------------------------

DO JN=1,KTRAC 

!*    1.0          DEFINE TRACERS AT HALF LEVELS
!                  -----------------------------

   DO JK=2,KLEV
      IK=JK-1
      DO JL=KIDIA,KFDIA
         ZCEN5(JL,JK,JN)=PCEN5(JL,JK,JN)
         ZCEN (JL,JK,JN)=PCEN (JL,JK,JN)
         ZCU5(JL,JK,JN) =PCEN5(JL,IK,JN)
         ZCU (JL,JK,JN) =PCEN (JL,IK,JN)
         ZCD5(JL,JK,JN) =PCEN5(JL,IK,JN)
         ZCD (JL,JK,JN) =PCEN (JL,IK,JN)
         ZTENC5(JL,JK,JN)=0._JPRB
         ZTENC(JL,JK,JN) =0._JPRB
      ENDDO
   ENDDO
   DO JL=KIDIA,KFDIA
         ZCU5(JL,KLEV,JN) =PCEN5(JL,KLEV,JN)
         ZCU (JL,KLEV,JN) =PCEN (JL,KLEV,JN)
   ENDDO


!*    2.0          COMPUTE UPDRAFT VALUES
!                  ----------------------

    DO JK=KLEV-1,3,-1
       IK=JK+1
       DO JL=KIDIA,KFDIA
        IF ( LLCUMASK(JL,JK) ) THEN
          ZERATE5=PMFU5(JL,JK)-PMFU5(JL,IK)+PUDRATE5(JL,JK)
          ZERATE =PMFU (JL,JK)-PMFU (JL,IK)+PUDRATE (JL,JK)
          ZMFA5=1._JPRB/MAX(ZRMFCMIN,PMFU5(JL,JK))
          IF (PMFU5(JL,JK) > ZRMFCMIN) THEN
            ZMFA=-PMFU(JL,JK)/(PMFU5(JL,JK)*PMFU5(JL,JK))
          ELSE
            ZMFA=0._JPRB
          ENDIF

          IF ( JK>=KCTOP(JL) )  THEN
             ZCU5(JL,JK,JN)=( PMFU5(JL,IK)*ZCU5(JL,IK,JN)+ZERATE5*PCEN5(JL,JK,JN) &
                   &       - (PUDRATE5(JL,JK)+PDMFUP5(JL,JK)*PSCAV5(JN))*ZCU5(JL,IK,JN) )*ZMFA5

             ZCU(JL,JK,JN)= ZMFA*( PMFU5(JL,IK)*ZCU5(JL,IK,JN)+ZERATE5*PCEN5(JL,JK,JN) &
                   &      - (PUDRATE5(JL,JK)+PDMFUP5(JL,JK)*PSCAV5(JN))*ZCU5(JL,IK,JN) ) & 
                   &      + ( PMFU(JL,IK)*ZCU5(JL,IK,JN) &
                   &      +  (PMFU5(JL,IK)-PUDRATE5(JL,JK))*ZCU(JL,IK,JN) &
                   &      +  ZERATE*PCEN5(JL,JK,JN) + ZERATE5*PCEN(JL,JK,JN) &
                   &      -  (PUDRATE(JL,JK)+PDMFUP(JL,JK)*PSCAV5(JN))*ZCU5(JL,IK,JN) )*ZMFA5
          ENDIF
        ENDIF
       ENDDO
    ENDDO


!*    3.0          COMPUTE DOWNDRAFT VALUES
!                  ------------------------

    DO JK=3,KLEV
       IK=JK-1
       DO JL=KIDIA,KFDIA
         IF ( LDDRAF(JL).AND.JK==KDTOP(JL) ) THEN
        !Nota: in order to avoid final negative Tracer values at the LFS the allowed value of ZCD
        !      depends on the jump in mass flux at the LFS
          !ZCD5(JL,JK,JN)=.5_JPRB*(ZCU5(JL,JK,JN)+PCEN5(JL,IK,JN))
          !ZCD(JL,JK,JN) =.5_JPRB*(ZCU(JL,JK,JN)+ZCEN(JL,IK,JN))
           ZCD5(JL,JK,JN)=.1_JPRB*ZCU5(JL,JK,JN)+.9_JPRB*PCEN5(JL,IK,JN)
           ZCD(JL,JK,JN) =.1_JPRB*ZCU(JL,JK,JN)+.9_JPRB*PCEN(JL,IK,JN)
         ELSEIF ( LDDRAF(JL).AND.JK>KDTOP(JL) ) THEN
           ZERATE5=-PMFD5(JL,JK)+PMFD5(JL,IK)+PDDRATE5(JL,JK)  
           ZERATE=-PMFD(JL,JK)+PMFD(JL,IK)+PDDRATE(JL,JK)
           ZMFA5=1._JPRB/MIN(-ZRMFCMIN,PMFD5(JL,JK))
           IF (PMFD5(JL,JK) < -ZRMFCMIN) THEN
             ZMFA=-PMFD(JL,JK)/(PMFD5(JL,JK)*PMFD5(JL,JK))
           ELSE
             ZMFA=0._JPRB
           ENDIF
           ZCD5(JL,JK,JN)=( PMFD5(JL,IK)*ZCD5(JL,IK,JN)-ZERATE5*PCEN5(JL,IK,JN) &
                       & + (PDDRATE5(JL,JK)+PDMFDP5(JL,JK)*PSCAV5(JN))*ZCD5(JL,IK,JN) )*ZMFA5 
           ZCD (JL,JK,JN)= ZMFA*( PMFD5(JL,IK)*ZCD5(JL,IK,JN)-ZERATE5*PCEN5(JL,IK,JN) &
                  &      + (PDDRATE5(JL,JK)+PDMFDP5(JL,JK)*PSCAV5(JN))*ZCD5(JL,IK,JN) ) & 
                  &      + ( PMFD (JL,IK)*ZCD5(JL,IK,JN) &
                  &      +  (PMFD5(JL,IK)+PDDRATE5(JL,JK))*ZCD (JL,IK,JN) &
                  &      -  ZERATE *PCEN5(JL,IK,JN) - ZERATE5*PCEN (JL,IK,JN) &
                  &      +  (PDDRATE(JL,JK)+PDMFDP(JL,JK)*PSCAV5(JN))*ZCD5(JL,IK,JN) )*ZMFA5
         ENDIF
       ENDDO
    ENDDO

! In order to avoid negative Tracer at KLEV adjust ZCD
  JK=KLEV
  IK=JK-1
  DO JL=KIDIA,KFDIA
    IF( LDDRAF(JL) ) THEN
     ZPOSI5=-ZDP5(JL,JK)*(PMFU5(JL,JK)*ZCU5(JL,JK,JN)+PMFD5(JL,JK)*ZCD5(JL,JK,JN)&
                    & -(PMFU5(JL,JK)+PMFD5(JL,JK))*PCEN5(JL,IK,JN))
     IF( PCEN5(JL,JK,JN)+ZPOSI5*PTSPHY<0.0_JPRB ) THEN
        ZMFA5=1._JPRB/MIN(-ZRMFCMIN,PMFD5(JL,JK))
        IF (PMFD5(JL,JK) < -ZRMFCMIN) THEN
           ZMFA=-PMFD(JL,JK)/(PMFD5(JL,JK)*PMFD5(JL,JK))
        ELSE
           ZMFA=0._JPRB
        ENDIF
        ZFACT=(PMFU5(JL,JK)+PMFD5(JL,JK))*PCEN5(JL,IK,JN)-PMFU5(JL,JK)*ZCU5(JL,JK,JN)&
                    &+PCEN5(JL,JK,JN)/(PTSPHY*ZDP5(JL,JK)) 
        ZCD5(JL,JK,JN)=ZFACT*ZMFA5
        ZCD(JL,JK,JN)=ZMFA*ZFACT + ( &
          & (PMFU(JL,JK)+PMFD(JL,JK))*PCEN5(JL,IK,JN) &
          & +(PMFU5(JL,JK)+PMFD5(JL,JK))*PCEN(JL,IK,JN)&
          & -PMFU(JL,JK)*ZCU5(JL,JK,JN)-PMFU5(JL,JK)*ZCU(JL,JK,JN)&
          & +PCEN(JL,JK,JN)/(PTSPHY*ZDP5(JL,JK))-PCEN5(JL,JK,JN)*ZDP(JL,JK) &
          & /(PTSPHY*ZDP5(JL,JK)*ZDP5(JL,JK)) )*ZMFA5
     ENDIF
    ENDIF
  ENDDO

ENDDO

ZMFC5(:,:,:)=0._JPRB


!----------------------------------------------------------------------
 
  DO JN=1,KTRAC

!*    3.0          COMPUTE FLUXES
!                  --------------

    DO JK=2,KLEV
      IK=JK-1
      DO JL=KIDIA,KFDIA
        IF(LLCUMASK(JL,JK)) THEN
          ZMFA5=PMFU5(JL,JK)+PMFD5(JL,JK)
          ZMFA =PMFU (JL,JK)+PMFD (JL,JK)

          ZMFC5(JL,JK,JN)=PMFU5(JL,JK)*ZCU5(JL,JK,JN)+PMFD5(JL,JK)*ZCD5(JL,JK,JN)&
                       & -ZIMP*ZMFA5*ZCEN5(JL,IK,JN) 
          ZMFC(JL,JK,JN)=PMFU (JL,JK)*ZCU5(JL,JK,JN)+PMFU5(JL,JK)*ZCU (JL,JK,JN) &
                      & +PMFD (JL,JK)*ZCD5(JL,JK,JN)+PMFD5(JL,JK)*ZCD (JL,JK,JN) &
                      & -ZIMP*(ZMFA5*ZCEN(JL,IK,JN)+ZMFA*ZCEN5(JL,IK,JN)) 

        ENDIF
      ENDDO
    ENDDO


!*    4.0          COMPUTE TENDENCIES = RHS
!                  ------------------------

       
    DO JK=2,KLEV-1
       IK=JK+1
       DO JL=KIDIA,KFDIA
         IF(LLCUMASK(JL,JK)) THEN
            ZTENC5(JL,JK,JN)=ZDP5(JL,JK)*(ZMFC5(JL,IK,JN)-ZMFC5(JL,JK,JN))
            ZTENC (JL,JK,JN)=ZDP (JL,JK)*(ZMFC5(JL,IK,JN)-ZMFC5(JL,JK,JN)) &
                          & +ZDP5(JL,JK)*(ZMFC (JL,IK,JN)-ZMFC (JL,JK,JN))
         !  ZTENC5(JL,JK,JN)=ZTENC5(JL,JK,JN)-PSCAV5(JN)*ZDP5(JL,JK)*PDMFUP5(JL,JK)*ZCU5(JL,JK,JN)
         !  ZTENC (JL,JK,JN)=ZTENC (JL,JK,JN)-PSCAV5(JN)*(&
         ! &                 ZDP (JL,JK)*PDMFUP5(JL,JK)*ZCU5(JL,JK,JN)+&
         ! &                 ZDP5(JL,JK)*PDMFUP (JL,JK)*ZCU5(JL,JK,JN)+&
         ! &                 ZDP5(JL,JK)*PDMFUP5(JL,JK)*ZCU (JL,JK,JN) )
         ENDIF
       ENDDO
    ENDDO
    
    JK=KLEV
       DO JL=KIDIA,KFDIA
         IF(LDCUM(JL)) THEN
            ZTENC5(JL,JK,JN)=-ZDP5(JL,JK)*ZMFC5(JL,JK,JN)
            ZTENC (JL,JK,JN)=-ZDP5(JL,JK)*ZMFC (JL,JK,JN) &
                           & -ZDP (JL,JK)*ZMFC5(JL,JK,JN)
         !  ZTENC5(JL,JK,JN)=ZTENC5(JL,JK,JN)-PSCAV5(JN)*ZDP5(JL,JK)*PDMFUP5(JL,JK)*ZCU5(JL,JK,JN)
         !  ZTENC (JL,JK,JN)=ZTENC (JL,JK,JN)-PSCAV5(JN)*(&
         ! &                 ZDP (JL,JK)*PDMFUP5(JL,JK)*ZCU5(JL,JK,JN)+&
         ! &                 ZDP5(JL,JK)*PDMFUP (JL,JK)*ZCU5(JL,JK,JN)+&
         ! &                 ZDP5(JL,JK)*PDMFUP5(JL,JK)*ZCU (JL,JK,JN) )
         ENDIF
       ENDDO

  ENDDO


 IF ( ZRMFSOLCT==0.0_JPRB ) THEN


!*    4.0          UPDATE TENDENCIES
!                  -----------------

   DO JN=1,KTRAC
     DO JK=2,KLEV
        DO JL=KIDIA,KFDIA
          IF(LLCUMASK(JL,JK)) THEN
             PTENC5(JL,JK,JN)=PTENC5(JL,JK,JN)+ZTENC5(JL,JK,JN)
             PTENC (JL,JK,JN)=PTENC (JL,JK,JN)+ZTENC (JL,JK,JN)
          ENDIF
        ENDDO
     ENDDO
   ENDDO

 ELSE

!---------------------------------------------------------------------------

!*    5.0          IMPLICIT SOLUTION
!                  -----------------
 
   ! Fill bi-diagonal Matrix vectors A=k-1, B=k;
   ! reuse ZMFC=A and ZB=B;
   ! ZTENC corresponds to the RHS ("constants") of the equation
   ! The solution is in ZR1
 
    ALLOCATE(ZB5(KLON,KLEV))
    ALLOCATE(ZR15(KLON,KLEV))   
    ALLOCATE(LLCUMBAS(KLON,KLEV))
    LLCUMBAS(:,:)=.FALSE.
    ZB5(:,:)=1._JPRB

    ALLOCATE(ZB(KLON,KLEV))
    ALLOCATE(ZR1(KLON,KLEV))   
    ZB(:,:)=0._JPRB
 
  DO JN=1,KTRAC

   ! Fill vectors A, B and RHS
 
    DO JK=2,KLEV
       IK=JK+1
       DO JL=KIDIA,KFDIA
         LLCUMBAS(JL,JK)=LLCUMASK(JL,JK)
         IF(LLCUMBAS(JL,JK)) THEN
           ZZP5=ZRMFSOLCT*ZDP5(JL,JK)*PTSPHY
           ZZP =ZRMFSOLCT*ZDP (JL,JK)*PTSPHY

           ZMFC5(JL,JK,JN)=-ZZP5*(PMFU5(JL,JK)+PMFD5(JL,JK))
           ZMFC (JL,JK,JN)=-ZZP *(PMFU5(JL,JK)+PMFD5(JL,JK)) &
                        &  -ZZP5*(PMFU (JL,JK)+PMFD (JL,JK)) 

           ZTENC5(JL,JK,JN) = ZTENC5(JL,JK,JN)*PTSPHY+PCEN5(JL,JK,JN)
           ZTENC (JL,JK,JN) = ZTENC (JL,JK,JN)*PTSPHY+PCEN (JL,JK,JN)
         ! ZTENC5(JL,JK,JN) = (PTENC5(JL,JK,JN)+ZTENC5(JL,JK,JN))*PTSPHY+PCEN5(JL,JK,JN)
         ! ZTENC (JL,JK,JN) = (PTENC (JL,JK,JN)+ZTENC (JL,JK,JN))*PTSPHY+PCEN (JL,JK,JN)

           IF(JK<KLEV) THEN
             ZB5(JL,JK)=1._JPRB+ZZP5*(PMFU5(JL,IK)+PMFD5(JL,IK))
             ZB(JL,JK)=ZZP5*(PMFU (JL,IK)+PMFD (JL,IK)) &
                    & +ZZP *(PMFU5(JL,IK)+PMFD5(JL,IK))
           ELSE
             ZB5(JL,JK)=1._JPRB
             ZB(JL,JK)=0._JPRB
           ENDIF
         ENDIF
       ENDDO
    ENDDO
 
    CALL CUBIDIAGTL&
       &( KIDIA, KFDIA, KLON, KLEV, &
       &  KCTOP, LLCUMBAS, &
       &  ZMFC5(:,:,JN),  ZB5,   ZTENC5(:,:,JN),   ZR15, &
       &  ZMFC (:,:,JN),  ZB ,   ZTENC (:,:,JN),   ZR1  )
 
     ! Compute tendencies
 
    DO JK=2,KLEV
       DO JL=KIDIA,KFDIA
         IF(LLCUMBAS(JL,JK)) THEN
           PTENC5(JL,JK,JN)=PTENC5(JL,JK,JN)+(ZR15(JL,JK)-PCEN5(JL,JK,JN))*ZTSPHY
           PTENC (JL,JK,JN)=PTENC (JL,JK,JN)+(ZR1 (JL,JK)-PCEN (JL,JK,JN))*ZTSPHY
         ! PTENC5(JL,JK,JN)=(ZR15(JL,JK)-PCEN5(JL,JK,JN))*ZTSPHY
         ! PTENC (JL,JK,JN)=(ZR1 (JL,JK)-PCEN (JL,JK,JN))*ZTSPHY
         ENDIF
       ENDDO
    ENDDO

  ENDDO
 
 
  DEALLOCATE(ZB5)
  DEALLOCATE(ZR15)
  DEALLOCATE(LLCUMBAS)
  DEALLOCATE(ZB)
  DEALLOCATE(ZR1)

 ENDIF
!---------------------------------------------------------------------------

DEALLOCATE(LLCUMASK)  

DEALLOCATE(ZDP5)
DEALLOCATE(ZMFC5)
DEALLOCATE(ZTENC5)
DEALLOCATE(ZCD5)
DEALLOCATE(ZCU5)
DEALLOCATE(ZCEN5)

DEALLOCATE(ZDP)
DEALLOCATE(ZMFC)
DEALLOCATE(ZTENC)
DEALLOCATE(ZCD)
DEALLOCATE(ZCU)
DEALLOCATE(ZCEN)

IF (LHOOK) CALL DR_HOOK('CUCTRACERTL',1,ZHOOK_HANDLE)
END SUBROUTINE CUCTRACERTL
