!OPTIONS XOPT(HSFUN)
SUBROUTINE CUFLX2 &
 & (  KIDIA,    KFDIA,    KLON,   KLEV,&
 & LDRAIN1D, LDLAND,   PTSPHY,&
 & PTEN,     PQEN,     PQSEN,    PTENH,    PQENH,&
 & PAPH,     PAP,    LDCUM,&
 & KCBOT,    KCTOP,    KDTOP,    KTOPM2,&
 & KTYPE,    LDDRAF,&
 & PMFU,     PMFD,     PMFUS,    PMFDS,&
 & PMFUQ,    PMFDQ,    PMFUL,    PLUDE,&
 & PDMFUP,   PDMFDP,   PDPMEL,   PLGLAC,&
 & PMFLXR,   PMFLXS,   PRAIN )  

!          M.TIEDTKE         E.C.M.W.F.     7/86 MODIF. 12/89

!          PURPOSE
!          -------

!          THIS ROUTINE DOES THE FINAL CALCULATION OF CONVECTIVE
!          FLUXES IN THE CLOUD LAYER AND IN THE SUBCLOUD LAYER
!          IN CASE OF THE SIMPLIFIED CONVECTION SCHEME. 

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTR*.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KCBOT*        CLOUD BASE LEVEL
!    *KCTOP*        CLOUD TOP LEVEL
!    *KDTOP*        TOP LEVEL OF DOWNDRAFTS

!    INPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS

!    INPUT PARAMETERS (REAL):

!    *PTSPHY*       TIME STEP FOR THE PHYSICS                       S
!    *PTEN*         PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)       K
!    *PQEN*         PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1)  KG/KG
!    *PQSEN*        ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)   KG/KG
!    *PTENH*        ENV. TEMPERATURE (T+1) ON HALF LEVELS           K
!    *PQENH*        ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS      KG/KG
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS            PA
!    *PAP*          PROVISIONAL PRESSURE ON FULL LEVELS            PA

!    UPDATED PARAMETERS (INTEGER):

!    *KTYPE*        SET TO ZERO IF LDCUM=.FALSE.

!    UPDATED PARAMETERS (LOGICAL):

!    *LDDRAF*       SET TO .FALSE. IF LDCUM=.FALSE. OR KDTOP<KCTOP   

!    UPDATED PARAMETERS (REAL):

!    *PMFU*         MASSFLUX IN UPDRAFTS                          KG/(M2*S)
!    *PMFD*         MASSFLUX IN DOWNDRAFTS                        KG/(M2*S)
!    *PMFUS*        FLUX OF DRY STATIC ENERGY IN UPDRAFTS          J/(M2*S)
!    *PMFDS*        FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS        J/(M2*S)
!    *PMFUQ*        FLUX OF SPEC. HUMIDITY IN UPDRAFTS            KG/(M2*S)
!    *PMFDQ*        FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS          KG/(M2*S)
!    *PMFUL*        FLUX OF LIQUID WATER IN UPDRAFTS              KG/(M2*S)
!    *PLUDE*        DETRAINED LIQUID WATER                        KG/(M3*S)
!    *PDMFUP*       FLUX DIFFERENCE OF PRECIP. IN UPDRAFTS        KG/(M2*S)
!    *PDMFDP*       FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS      KG/(M2*S)

!    OUTPUT PARAMETERS (REAL):

!    *PDPMEL*       CHANGE IN PRECIP.-FLUXES DUE TO MELTING       KG/(M2*S)
!    *PLGLAC*       FLUX OF FROZEN CLOUD WATER IN UPDRAFTS        KG/(M2*S)
!    *PMFLXR*       CONVECTIVE RAIN FLUX                          KG/(M2*S)
!    *PMFLXS*       CONVECTIVE SNOW FLUX                          KG/(M2*S)
!    *PRAIN*        TOTAL PRECIP. PRODUCED IN CONV. UPDRAFTS      KG/(M2*S)
!                   (NO EVAPORATION IN DOWNDRAFTS)

!          EXTERNALS
!          ---------
!          NONE

!          MODIFICATIONS
!          -------------
!             99-06-14 : Optimisation        D.SALMOND
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Lopez      11-01-2007  Added switch LDRAIN
!        P. Lopez      04-09-2007  Revision to match Tiedtke scheme
!        A. Geer       01-10-2008  LDRAIN1D name change to relfect usage

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RCPD     ,RLVTT    ,RLSTT    ,RLMLT    ,RTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOEPHLI  , ONLY : LPHYLIN  ,RLPTRC
USE YOECUMF2 , ONLY : RCUCOV2  ,RCPECONS2 ,RTAUMEL2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
LOGICAL           ,INTENT(IN)    :: LDRAIN1D 
LOGICAL           ,INTENT(IN)    :: LDLAND(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQEN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSEN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV) 
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCBOT(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTOPM2 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KTYPE(KLON) 
LOGICAL           ,INTENT(INOUT) :: LDDRAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLUDE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFUP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDPMEL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLGLAC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXR(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAIN(KLON) 
INTEGER(KIND=JPIM) :: IK, IKB, ITOP, JK, JL

REAL(KIND=JPRB) :: ZRHEBC(KLON)

REAL(KIND=JPRB) :: ZALFAW, ZCONS1, ZCONS1A, ZCONS2, &
 & ZDENOM, ZDRFL, ZDRFL1, ZFAC, ZFOEEWI, &
 & ZFOEEWL, ZOEALFA, ZOEEWM, ZOELHM, ZPDR, ZPDS, &
 & ZRFL, ZRFLN, ZRMIN, ZRNEW, ZSNMLT, ZTARG, &
 & ZTMST, ZZP, ZFACT1
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!DIR$ VFUNCTION EXPHF
#include "fcttre.h"

!--------------------------------------------------------------------
!*             SPECIFY CONSTANTS

IF (LHOOK) CALL DR_HOOK('CUFLX2',0,ZHOOK_HANDLE)
ZTMST=PTSPHY
ZCONS1A=RCPD/(RLMLT*RG*RTAUMEL2)
ZCONS2=1.0_JPRB/(RG*ZTMST)

!*    1.0          DETERMINE FINAL CONVECTIVE FLUXES
!                  ---------------------------------

ITOP=KLEV
DO JL=KIDIA,KFDIA
  PRAIN(JL)=0.0_JPRB
  ITOP=MIN(ITOP,KCTOP(JL))
  IF(.NOT.LDCUM(JL).OR.KDTOP(JL) < KCTOP(JL)) LDDRAF(JL)=.FALSE.
  IF(.NOT.LDCUM(JL)) KTYPE(JL)=0
  IF(LDLAND(JL)) THEN
    ZRHEBC(JL)=0.7_JPRB
  ELSE
    ZRHEBC(JL)=0.9_JPRB
  ENDIF
ENDDO
KTOPM2=MAX(2,ITOP-2)
! TO GET IDENTICAL RESULTS FOR DIFFERENT NPROMA FORCE KTOPM2 TO 2
KTOPM2=2
DO JK=KTOPM2,KLEV
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    PMFLXR(JL,JK)=0.0_JPRB
    PMFLXS(JL,JK)=0.0_JPRB
    PDPMEL(JL,JK)=0.0_JPRB
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN
      PMFUS(JL,JK)=RCPD*PMFU(JL,JK)*(PMFUS(JL,JK)-PTENH(JL,JK))
      PMFUQ(JL,JK)=PMFU(JL,JK)*(PMFUQ(JL,JK)-PQENH(JL,JK))
      PMFUL(JL,JK)=PMFU(JL,JK)*PMFUL(JL,JK)
      PLGLAC(JL,JK)=PMFU(JL,JK)*PLGLAC(JL,JK)
      IF (JK < KLEV) THEN
        PLUDE(JL,JK)=PMFU(JL,JK+1)*PLUDE(JL,JK)
      ELSE
        PLUDE(JL,JK)=0.0_JPRB
      ENDIF
      IF(LDDRAF(JL).AND.JK >= KDTOP(JL)) THEN
        PMFDS(JL,JK)=RCPD*PMFD(JL,JK)*(PMFDS(JL,JK)-PTENH(JL,JK))
        PMFDQ(JL,JK)=PMFD(JL,JK)*(PMFDQ(JL,JK)-PQENH(JL,JK))
      ELSE
        PMFD(JL,JK)=0.0_JPRB
        PMFDS(JL,JK)=0.0_JPRB
        PMFDQ(JL,JK)=0.0_JPRB
        PDMFDP(JL,JK-1)=0.0_JPRB
      ENDIF
    ELSE
      PMFU(JL,JK)=0.0_JPRB
      PMFD(JL,JK)=0.0_JPRB
      PMFUS(JL,JK)=0.0_JPRB
      PMFDS(JL,JK)=0.0_JPRB
      PMFUQ(JL,JK)=0.0_JPRB
      PMFDQ(JL,JK)=0.0_JPRB
      PMFUL(JL,JK)=0.0_JPRB
      PLGLAC(JL,JK)=0.0_JPRB
      PDMFUP(JL,JK-1)=0.0_JPRB
      PDMFDP(JL,JK-1)=0.0_JPRB
      PLUDE(JL,JK-1)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO
PMFLXR(:,KLEV+1)=0.0_JPRB
PMFLXS(:,KLEV+1)=0.0_JPRB

!*    1.5          SCALE FLUXES BELOW CLOUD BASE
!                  LINEAR DCREASE
!                  -----------------------------

!DIR$ IVDEP
!OCL NOVREC
DO JL=KIDIA,KFDIA
  IF(LDCUM(JL)) THEN
    IKB=KCBOT(JL)
    IK=IKB+1
    ZZP=((PAPH(JL,KLEV+1)-PAPH(JL,IK))/(PAPH(JL,KLEV+1)-PAPH(JL,IKB)))
    IF(KTYPE(JL) == 3) ZZP=ZZP*ZZP 
    PMFU(JL,IK)=PMFU(JL,IKB)*ZZP
    IF (LPHYLIN .OR. LDRAIN1D) THEN
      ZTARG=PTENH(JL,IKB)
      ZOEALFA=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG-RLPTRC))+1.0_JPRB)
      ZOELHM=ZOEALFA*RLVTT+(1.0_JPRB-ZOEALFA)*RLSTT
      PMFUS(JL,IK)=(PMFUS(JL,IKB)-ZOELHM*PMFUL(JL,IKB))*ZZP
    ELSE
      PMFUS(JL,IK)=(PMFUS(JL,IKB)-FOELHMCU(PTENH(JL,IKB))*PMFUL(JL,IKB))*ZZP
    ENDIF
    PMFUQ(JL,IK)=(PMFUQ(JL,IKB)+PMFUL(JL,IKB))*ZZP
    PMFUL(JL,IK)=0.0_JPRB
  ENDIF
ENDDO

DO JK=KTOPM2,KLEV
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK > KCBOT(JL)+1) THEN
      IKB=KCBOT(JL)+1
      ZZP=((PAPH(JL,KLEV+1)-PAPH(JL,JK))/(PAPH(JL,KLEV+1)-PAPH(JL,IKB)))
      IF(KTYPE(JL) == 3) THEN
        ZZP=ZZP**2
      ENDIF
      PMFU(JL,JK)=PMFU(JL,IKB)*ZZP
      PMFUS(JL,JK)=PMFUS(JL,IKB)*ZZP
      PMFUQ(JL,JK)=PMFUQ(JL,IKB)*ZZP
      PMFUL(JL,JK)=0.0_JPRB
      IF(LDDRAF(JL).AND.PMFD(JL,IKB+1) == 0.0_JPRB) THEN
        PMFD(JL,JK)=PMFD(JL,IKB)*ZZP
        PMFDS(JL,JK)=PMFDS(JL,IKB)*ZZP
        PMFDQ(JL,JK)=PMFDQ(JL,IKB)*ZZP
      ENDIF
    ENDIF
  ENDDO
ENDDO

!*    2.            CALCULATE RAIN/SNOW FALL RATES
!*                  CALCULATE MELTING OF SNOW
!*                  CALCULATE EVAPORATION OF PRECIP
!                   -------------------------------

DO JK=KTOPM2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN
      PRAIN(JL)=PRAIN(JL)+PDMFUP(JL,JK)
      IF(PMFLXS(JL,JK) > 0.0_JPRB.AND.PTEN(JL,JK) > RTT) THEN
        ZCONS1=ZCONS1A*(1.0_JPRB+0.5_JPRB*(PTEN(JL,JK)-RTT))
        ZFAC=ZCONS1*(PAPH(JL,JK+1)-PAPH(JL,JK))
        ZSNMLT=MIN(PMFLXS(JL,JK),ZFAC*(PTEN(JL,JK)-RTT))
        PDPMEL(JL,JK)=ZSNMLT
        IF (LPHYLIN .OR. LDRAIN1D) THEN
          ZTARG=PTEN(JL,JK)-ZSNMLT/ZFAC
          ZOEALFA=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG-RLPTRC))+1.0_JPRB)
          ZFOEEWL=R2ES*EXP(R3LES*(ZTARG-RTT)/(ZTARG-R4LES))
          ZFOEEWI=R2ES*EXP(R3IES*(ZTARG-RTT)/(ZTARG-R4IES))
          ZOEEWM=ZOEALFA*ZFOEEWL + (1.0_JPRB-ZOEALFA)*ZFOEEWI
          PQSEN(JL,JK)=ZOEEWM/PAP(JL,JK)
        ELSE
          PQSEN(JL,JK)=FOEEWMCU(PTEN(JL,JK)-ZSNMLT/ZFAC)/PAP(JL,JK)
        ENDIF
      ENDIF
      IF (LPHYLIN .OR. LDRAIN1D) THEN
        ZALFAW=0.545_JPRB*(TANH(0.17_JPRB*(PTEN(JL,JK)-RLPTRC))+1.0_JPRB)
      ELSE
        ZALFAW=FOEALFCU(PTEN(JL,JK))
      ENDIF
!no liquid precipitation above melting level
      IF(PTEN(JL,JK) < RTT .AND. ZALFAW > 0.0_JPRB) THEN
        PLGLAC(JL,JK)=PLGLAC(JL,JK)+ZALFAW*(PDMFUP(JL,JK)+PDMFDP(JL,JK))
        ZALFAW=0.0_JPRB
      ENDIF

      PMFLXR(JL,JK+1)=PMFLXR(JL,JK)+ZALFAW*&
       & (PDMFUP(JL,JK)+PDMFDP(JL,JK))+PDPMEL(JL,JK)  
      PMFLXS(JL,JK+1)=PMFLXS(JL,JK)+(1.0_JPRB-ZALFAW)*&
       & (PDMFUP(JL,JK)+PDMFDP(JL,JK))-PDPMEL(JL,JK)  
      IF(PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1) < 0.0_JPRB) THEN
        PDMFDP(JL,JK)=-(PMFLXR(JL,JK)+PMFLXS(JL,JK)+PDMFUP(JL,JK))
        PMFLXR(JL,JK+1)=0.0_JPRB
        PMFLXS(JL,JK+1)=0.0_JPRB
        PDPMEL(JL,JK)  =0.0_JPRB
      ELSEIF(PMFLXR(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXS(JL,JK+1)=PMFLXS(JL,JK+1)+PMFLXR(JL,JK+1)
        PMFLXR(JL,JK+1)=0.0_JPRB
      ELSEIF(PMFLXS(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXR(JL,JK+1)=PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1)
        PMFLXS(JL,JK+1)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO

! Reminder for conservation:
!    pdmfup(jl,jk)+pdmfdp(jl,jk)=pmflxr(jl,jk+1)+pmflxs(jl,jk+1)-pmflxr(jl,jk)-pmflxs(jl,jk)

DO JK=KTOPM2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCBOT(JL)) THEN 
      ZRFL=PMFLXR(JL,JK)+PMFLXS(JL,JK)
      IF(ZRFL > 1.E-20_JPRB) THEN
        ZFACT1=1.0_JPRB/(5.09E-3_JPRB*RCUCOV2)
        ZDRFL1=RCPECONS2*RCUCOV2*MAX(0.0_JPRB,PQSEN(JL,JK)-PQEN(JL,JK))*&
         & (SQRT(PAPH(JL,JK)/PAPH(JL,KLEV+1))*ZRFL*ZFACT1)**0.5777_JPRB*&
         & (PAPH(JL,JK+1)-PAPH(JL,JK))  
        ZRNEW=ZRFL-ZDRFL1
        ZRMIN=ZRFL-RCUCOV2*MAX(0.0_JPRB,ZRHEBC(JL)*PQSEN(JL,JK)-PQEN(JL,JK))&
         & *ZCONS2*(PAPH(JL,JK+1)-PAPH(JL,JK))  
        ZRNEW=MAX(ZRNEW,ZRMIN)
        ZRFLN=MAX(ZRNEW,0.0_JPRB)
        ZDRFL=MIN(0.0_JPRB,ZRFLN-ZRFL)
        IF (LPHYLIN .OR. LDRAIN1D) THEN
          ZALFAW=0.545_JPRB*(TANH(0.17_JPRB*(PTEN(JL,JK)-RLPTRC))+1.0_JPRB)
        ELSE
          ZALFAW=FOEALFCU(PTEN(JL,JK))
        ENDIF
        IF(PTEN(JL,JK) < RTT) ZALFAW=0.0_JPRB
        ZPDR=ZALFAW*PDMFDP(JL,JK)
        ZPDS=(1.0_JPRB-ZALFAW)*PDMFDP(JL,JK)
        ZDENOM=1.0_JPRB/MAX(1.E-20_JPRB,PMFLXR(JL,JK)+PMFLXS(JL,JK))
        PMFLXR(JL,JK+1)=PMFLXR(JL,JK)+ZPDR &
         & +PDPMEL(JL,JK)+ZDRFL*PMFLXR(JL,JK)*ZDENOM  
        PMFLXS(JL,JK+1)=PMFLXS(JL,JK)+ZPDS &
         & -PDPMEL(JL,JK)+ZDRFL*PMFLXS(JL,JK)*ZDENOM  
        PDMFUP(JL,JK)=PDMFUP(JL,JK)+ZDRFL
        IF(PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1) < 0.0_JPRB) THEN
          PDMFUP(JL,JK)=PDMFUP(JL,JK)-(PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1))
          PMFLXR(JL,JK+1)=0.0_JPRB
          PMFLXS(JL,JK+1)=0.0_JPRB
          PDPMEL(JL,JK)  =0.0_JPRB
        ELSEIF(PMFLXR(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXS(JL,JK+1)=PMFLXS(JL,JK+1)+PMFLXR(JL,JK+1)
          PMFLXR(JL,JK+1)=0.0_JPRB
        ELSEIF(PMFLXS(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXR(JL,JK+1)=PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1)
          PMFLXS(JL,JK+1)=0.0_JPRB
        ENDIF
      ELSE
        PMFLXR(JL,JK+1)=0.0_JPRB
        PMFLXS(JL,JK+1)=0.0_JPRB
        PDMFDP(JL,JK)=0.0_JPRB
        PDPMEL(JL,JK)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('CUFLX2',1,ZHOOK_HANDLE)
END SUBROUTINE CUFLX2
