!OPTIONS XOPT(HSFUN)
SUBROUTINE CUFLX2AD &
 & (  KIDIA,    KFDIA,    KLON,   KLEV,&
 & PTSPHY,&
 & PTEN5,    PQEN5,    PQSEN5,   PTENH5,   PQENH5,&
 & PAPH5,    PAP5,   LDLAND,   LDCUM, &
 & KCBOT,    KCTOP,    KDTOP,    KTOPM2,&
 & KTYPE,    LDDRAF,&
 & PMFU5,    PMFD5,    PMFUS5,   PMFDS5,&
 & PMFUQ5,   PMFDQ5,   PMFUL5,   PLUDE5,&
 & PDMFUP5,  PDMFDP5,  PDPMEL5,  PLGLAC5,&
 & PMFLXR5,  PMFLXS5,  PRAIN5,&
 & PTEN,     PQEN,     PQSEN,    PTENH,    PQENH,&
 & PAPH,     PAP,&
 & PMFU,     PMFD,     PMFUS,    PMFDS,&
 & PMFUQ,    PMFDQ,    PMFUL,    PLUDE,&
 & PDMFUP,   PDMFDP,   PDPMEL,   PLGLAC,&
 & PMFLXR,   PMFLXS,   PRAIN )  

!          Ph LOPEZ          E.C.M.W.F.     13/08/2002

!          Adapted from

!          M.TIEDTKE         E.C.M.W.F.     7/86 MODIF. 12/89

!          PURPOSE
!          -------

!          THIS ROUTINE DOES THE FINAL CALCULATION OF CONVECTIVE
!          FLUXES IN THE CLOUD LAYER AND IN THE SUBCLOUD LAYER
!          IN THE CASE OF THE SIMPLIFIED CONVECTION SCHEME
!          (Adjoint)

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTRNAD*.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KCBOT*        CLOUD BASE LEVEL
!    *KCTOP*        CLOUD TOP LEVEL
!    *KDTOP*        TOP LEVEL OF DOWNDRAFTS

!    INPUT PARAMETERS (LOGICAL):

!    *LDLAND*       LAND SEA MASK (.TRUE. FOR LAND)
!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS

!    INPUT PARAMETERS (REAL):

!    *PTSPHY*       TIME STEP FOR THE PHYSICS                       S
!    *PTEN5*        PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)    (Trajectory)
!    *PQEN5*        PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1) (Trajectory)
!    *PQSEN5*       ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)  (Trajectory)
!    *PTENH5*       ENV. TEMPERATURE (T+1) ON HALF LEVELS        (Trajectory)
!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS     (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS          (Trajectory)
!    *PAP5*         PROVISIONAL PRESSURE ON FULL LEVELS          (Trajectory)

!    *PTEN*         PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)       K
!    *PQEN*         PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1)  KG/KG
!    *PQSEN*        ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)   KG/KG
!    *PTENH*        ENV. TEMPERATURE (T+1) ON HALF LEVELS           K
!    *PQENH*        ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS      KG/KG
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS            PA
!    *PAP*          PROVISIONAL PRESSURE ON FULL LEVELS            PA

!    UPDATED PARAMETERS (INTEGER):

!    *KTYPE*        SET TO ZERO IF LDCUM=.FALSE.

!    UPDATED PARAMETERS (LOGICAL):

!    *LDDRAF*       SET TO .FALSE. IF LDCUM=.FALSE. OR KDTOP<KCTOP   

!    UPDATED PARAMETERS (REAL):

!    *PMFU5*        MASSFLUX IN UPDRAFTS                        (Trajectory)  
!    *PMFD5*        MASSFLUX IN DOWNDRAFTS                      (Trajectory)  
!    *PMFUS5*       FLUX OF DRY STATIC ENERGY IN UPDRAFTS       (Trajectory)  
!    *PMFDS5*       FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS     (Trajectory)
!    *PMFUQ5*       FLUX OF SPEC. HUMIDITY IN UPDRAFTS          (Trajectory)  
!    *PMFDQ5*       FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS        (Trajectory)  
!    *PMFUL5*       FLUX OF LIQUID WATER IN UPDRAFTS            (Trajectory)  
!    *PLUDE5*       DETRAINED LIQUID WATER                      (Trajectory)  
!    *PDMFUP5*      FLUX DIFFERENCE OF PRECIP. IN UPDRAFTS      (Trajectory)  
!    *PDMFDP5*      FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS    (Trajectory)  

!    *PMFU*         MASSFLUX IN UPDRAFTS                          KG/(M2*S)
!    *PMFD*         MASSFLUX IN DOWNDRAFTS                        KG/(M2*S)
!    *PMFUS*        FLUX OF DRY STATIC ENERGY IN UPDRAFTS          J/(M2*S)
!    *PMFDS*        FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS        J/(M2*S)
!    *PMFUQ*        FLUX OF SPEC. HUMIDITY IN UPDRAFTS            KG/(M2*S)
!    *PMFDQ*        FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS          KG/(M2*S)
!    *PMFUL*        FLUX OF LIQUID WATER IN UPDRAFTS              KG/(M2*S)
!    *PLUDE*        DETRAINED LIQUID WATER                        KG/(M3*S)
!    *PDMFUP*       FLUX DIFFERENCE OF PRECIP. IN UPDRAFTS        KG/(M2*S)
!    *PDMFDP*       FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS      KG/(M2*S)

!    OUTPUT PARAMETERS (REAL):

!    *PDPMEL5*      CHANGE IN PRECIP.-FLUXES DUE TO MELTING     (Trajectory) 
!    *PLGLAC5*      FLUX OF FROZEN CLOUD WATER IN UPDRAFTS      (Trajectory) 
!    *PMFLXR5*      CONVECTIVE RAIN FLUX                        (Trajectory) 
!    *PMFLXS5*      CONVECTIVE SNOW FLUX                        (Trajectory) 
!    *PRAIN5*       TOTAL PRECIP. PRODUCED IN CONV. UPDRAFTS    (Trajectory) 
!                   (NO EVAPORATION IN DOWNDRAFTS)       

!    *PDPMEL*       CHANGE IN PRECIP.-FLUXES DUE TO MELTING       KG/(M2*S)
!    *PLGLAC*       FLUX OF FROZEN CLOUD WATER IN UPDRAFTS        KG/(M2*S)
!    *PMFLXR*       CONVECTIVE RAIN FLUX                          KG/(M2*S)
!    *PMFLXS*       CONVECTIVE SNOW FLUX                          KG/(M2*S)
!    *PRAIN*        TOTAL PRECIP. PRODUCED IN CONV. UPDRAFTS      KG/(M2*S)
!                   (NO EVAPORATION IN DOWNDRAFTS)

!          EXTERNALS
!          ---------
!          NONE

!       Modifications
!       -------------
!        P. Lopez      11-01-2007  Removed switch LPHYLIN
!        P. Lopez      04-09-2007  Revision to match Tiedtke scheme

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RCPD     ,RLVTT    ,RLSTT    ,RLMLT    ,RTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE   ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOEPHLI  , ONLY : RLPTRC
USE YOECUMF2 , ONLY : RCUCOV2   ,RCPECONS2 ,RTAUMEL2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP5(KLON,KLEV) 
LOGICAL           ,INTENT(IN)    :: LDLAND(KLON)
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCBOT(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTOPM2 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KTYPE(KLON) 
LOGICAL           ,INTENT(INOUT) :: LDDRAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLUDE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFUP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDPMEL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLGLAC5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXR5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXS5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAIN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTEN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQEN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSEN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLUDE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFUP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDPMEL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLGLAC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXR(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAIN(KLON) 

INTEGER(KIND=JPIM) :: IK, IKB, ITOP, JK, JL

REAL(KIND=JPRB) :: Z1S5(KLON,KLEV),        Z2S5(KLON,KLEV),&
                 & ZALFAW5(KLON,KLEV),     ZDENOM5(KLON,KLEV),&
                 & ZFAC5(KLON,KLEV),       ZFOEEWI5(KLON,KLEV),&
                 & ZFOEEWL5(KLON,KLEV),    ZOEALFA5(KLON,KLEV),&
                 & ZOEEWM5(KLON,KLEV),     ZOELHM5(KLON,KLEV),&
                 & ZRFL5(KLON,KLEV),       ZRFLN5(KLON,KLEV),&
                 & ZRMIN5(KLON,KLEV),      ZRNEW15(KLON,KLEV),&
                 & ZSNMLT5(KLON,KLEV),     ZTARG15(KLON),&
                 & ZZP15(KLON,KLEV),       ZZP25(KLON,KLEV),&
                 & ZP15(KLON,KLEV),        ZP25(KLON,KLEV),&
                 & ZFACT25(KLON,KLEV),     ZFACT35(KLON,KLEV),&
                 & ZCONS15(KLON,KLEV),     ZLGLAC15(KLON,KLEV),&
                 & ZRNEW25(KLON,KLEV),     ZMFLXS25(KLON,KLEV),&
                 & ZMFLXR15(KLON,KLEV+1),  ZMFLXS15(KLON,KLEV+1),&
                 & ZMFLXR35(KLON,KLEV+1),  ZMFLXS35(KLON,KLEV+1),&
                 & ZDMFUP15(KLON,KLEV),    ZDMFDP15(KLON,KLEV),&
                 & ZDRFL5(KLON,KLEV),      ZTARG25(KLON,KLEV),& 
                 & ZALFAW15(KLON,KLEV),    ZALFAW25(KLON,KLEV),&
                 & ZMFUS15(KLON,KLEV),     ZMFUQ15(KLON,KLEV),&
                 & ZMFUL15(KLON,KLEV),     ZMFDS15(KLON,KLEV),&
                 & ZMFDQ15(KLON,KLEV),     ZMFU15(KLON,KLEV),&
                 & ZMFD15(KLON,KLEV),      ZMFD25(KLON,KLEV),&
                 & ZLUDE15(KLON,KLEV),     ZMFU25(KLON,KLEV)

REAL(KIND=JPRB) :: ZRHEBC(KLON)

REAL(KIND=JPRB) :: ZCONS1A, ZCONS2, ZTMST,  ZCONS3,   ZSQRP5, &
                 & ZDRFL15, ZPDR5,  ZPDS5  

REAL(KIND=JPRB) :: Z1S, Z2S, ZALFAW, ZDENOM, ZDRFL1, ZCONS1, &
                 & ZDRFL, ZFAC, ZFOEEWI, ZFOEEWL, ZOEALFA, &
                 & ZOEEWM, ZOELHM, ZPDR, ZPDS, ZRFL, ZRFLN, &
                 & ZRMIN, ZRNEW, ZSNMLT, ZTARG, ZZP, ZP1, ZP2, &
                 & ZFACT1, ZFACT2, ZFACT3
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!DIR$ VFUNCTION EXPHF
#include "fcttre.h"
#include "fcttread.h"

!--------------------------------------------------------------------
!*             SPECIFY CONSTANTS

IF (LHOOK) CALL DR_HOOK('CUFLX2AD',0,ZHOOK_HANDLE)
ZTMST=PTSPHY
ZCONS1A=RCPD/(RLMLT*RG*RTAUMEL2)
ZCONS2=1.0_JPRB/(RG*ZTMST)

!--------------------------------------------------------------------
!*             INITIALIZE LOCAL ARRAYS (TRAJECTORY STORAGE)

ZMFLXR15(:,:)=0.0_JPRB
ZMFLXS15(:,:)=0.0_JPRB
ZMFLXS25(:,:)=0.0_JPRB
ZMFLXR35(:,:)=0.0_JPRB
ZMFLXS35(:,:)=0.0_JPRB

!--------------------------------------------------------------------

!*    1.0          DETERMINE FINAL CONVECTIVE FLUXES
!                  ---------------------------------

ITOP=KLEV
DO JL=KIDIA,KFDIA
  PRAIN5(JL)=0.0_JPRB
  ITOP=MIN(ITOP,KCTOP(JL))
  IF(.NOT.LDCUM(JL).OR.KDTOP(JL) < KCTOP(JL)) LDDRAF(JL)=.FALSE.
  IF(.NOT.LDCUM(JL)) KTYPE(JL)=0
  IF(LDLAND(JL)) THEN
    ZRHEBC(JL)=0.7_JPRB
  ELSE
    ZRHEBC(JL)=0.9_JPRB
  ENDIF
ENDDO
KTOPM2=MAX(2,ITOP-2)
! TO GET IDENTICAL RESULTS FOR DIFFERENT NPROMA FORCE KTOPM2 TO 2
KTOPM2=2
DO JK=KTOPM2,KLEV
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    PMFLXR5(JL,JK)=0.0_JPRB
    PMFLXS5(JL,JK)=0.0_JPRB
    PDPMEL5(JL,JK)=0.0_JPRB
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN

! Store trajectory array for adjoint
      ZLGLAC15(JL,JK)=PLGLAC5(JL,JK)
      ZMFU15(JL,JK)=PMFU5(JL,JK)
      ZMFUS15(JL,JK)=PMFUS5(JL,JK)
      ZMFUQ15(JL,JK)=PMFUQ5(JL,JK)
      ZMFUL15(JL,JK)=PMFUL5(JL,JK)
      ZLUDE15(JL,JK)=PLUDE5(JL,JK)

      PMFUS5(JL,JK)=RCPD*PMFU5(JL,JK)*(PMFUS5(JL,JK)-PTENH5(JL,JK))
      PMFUQ5(JL,JK)=PMFU5(JL,JK)*(PMFUQ5(JL,JK)-PQENH5(JL,JK))
      PMFUL5(JL,JK)=PMFU5(JL,JK)*PMFUL5(JL,JK)
      PLGLAC5(JL,JK)=PMFU5(JL,JK)*PLGLAC5(JL,JK)
      IF (JK < KLEV) THEN
! Store trajectory array for adjoint
        ZMFU25(JL,JK)=PMFU5(JL,JK+1)
        PLUDE5(JL,JK)=PMFU5(JL,JK+1)*PLUDE5(JL,JK)
      ELSE
        PLUDE5(JL,JK)=0.0_JPRB
      ENDIF

      IF(LDDRAF(JL).AND.JK >= KDTOP(JL)) THEN
! Store trajectory array for adjoint
        ZMFD15(JL,JK)=PMFD5(JL,JK)
        ZMFDS15(JL,JK)=PMFDS5(JL,JK)
        ZMFDQ15(JL,JK)=PMFDQ5(JL,JK)

        PMFDS5(JL,JK)=RCPD*PMFD5(JL,JK)*(PMFDS5(JL,JK)-PTENH5(JL,JK))
        PMFDQ5(JL,JK)=PMFD5(JL,JK)*(PMFDQ5(JL,JK)-PQENH5(JL,JK))
      ELSE
        PMFD5(JL,JK)=0.0_JPRB
        PMFDS5(JL,JK)=0.0_JPRB
        PMFDQ5(JL,JK)=0.0_JPRB
        PDMFDP5(JL,JK-1)=0.0_JPRB
      ENDIF
    ELSE
      PMFU5(JL,JK)=0.0_JPRB
      PMFD5(JL,JK)=0.0_JPRB
      PMFUS5(JL,JK)=0.0_JPRB
      PMFDS5(JL,JK)=0.0_JPRB
      PMFUQ5(JL,JK)=0.0_JPRB
      PMFDQ5(JL,JK)=0.0_JPRB
      PMFUL5(JL,JK)=0.0_JPRB
      PLGLAC5(JL,JK)=0.0_JPRB
      PDMFUP5(JL,JK-1)=0.0_JPRB
      PDMFDP5(JL,JK-1)=0.0_JPRB
      PLUDE5(JL,JK-1)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO
PMFLXR5(:,KLEV+1)=0.0_JPRB
PMFLXS5(:,KLEV+1)=0.0_JPRB

!*    1.5          SCALE FLUXES BELOW CLOUD BASE
!                  LINEAR DCREASE
!                  -----------------------------

!DIR$ IVDEP
!OCL NOVREC
DO JL=KIDIA,KFDIA
  IF(LDCUM(JL)) THEN
    IKB=KCBOT(JL)
    IK=IKB+1
    ZP15(JL,IK)=PAPH5(JL,KLEV+1)-PAPH5(JL,IK)
    ZP25(JL,IK)=PAPH5(JL,KLEV+1)-PAPH5(JL,IKB)
    ZZP15(JL,IK)=ZP15(JL,IK)/ZP25(JL,IK)

    IF(KTYPE(JL) == 3) THEN
      ZZP25(JL,IK)=ZZP15(JL,IK)*ZZP15(JL,IK)
    ELSE
      ZZP25(JL,IK)=ZZP15(JL,IK)
    ENDIF

    PMFU5(JL,IK)=PMFU5(JL,IKB)*ZZP25(JL,IK)

    ZTARG15(JL)=PTENH5(JL,IKB)
    ZOEALFA5(JL,IK)=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG15(JL)-RLPTRC))+1.0_JPRB)
    ZOELHM5(JL,IK)=ZOEALFA5(JL,IK)*RLVTT+(1.0_JPRB-ZOEALFA5(JL,IK))*RLSTT
    PMFUS5(JL,IK)=(PMFUS5(JL,IKB)-ZOELHM5(JL,IK) &
               & *PMFUL5(JL,IKB))*ZZP25(JL,IK)  
    PMFUQ5(JL,IK)=(PMFUQ5(JL,IKB)+PMFUL5(JL,IKB))*ZZP25(JL,IK)
    PMFUL5(JL,IK)=0.0_JPRB
  ENDIF
ENDDO

DO JK=KTOPM2,KLEV
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK > KCBOT(JL)+1) THEN
      IKB=KCBOT(JL)+1
      ZP15(JL,JK)=PAPH5(JL,KLEV+1)-PAPH5(JL,JK)
      ZP25(JL,JK)=PAPH5(JL,KLEV+1)-PAPH5(JL,IKB)
      ZZP15(JL,JK)=ZP15(JL,JK)/ZP25(JL,JK)

      IF(KTYPE(JL) == 3) THEN
        ZZP25(JL,JK)=ZZP15(JL,JK)**2
      ELSE
        ZZP25(JL,JK)=ZZP15(JL,JK)
      ENDIF

      PMFU5(JL,JK)=PMFU5(JL,IKB)*ZZP25(JL,JK)
      PMFUS5(JL,JK)=PMFUS5(JL,IKB)*ZZP25(JL,JK)
      PMFUQ5(JL,JK)=PMFUQ5(JL,IKB)*ZZP25(JL,JK)
      PMFUL5(JL,JK)=0.0_JPRB

! Store trajectory array for adjoint
      ZMFD25(JL,JK)=PMFD5(JL,IKB+1)

      IF(LDDRAF(JL).AND.PMFD5(JL,IKB+1) == 0.0_JPRB) THEN
        PMFD5(JL,JK)=PMFD5(JL,IKB)*ZZP25(JL,JK)
        PMFDS5(JL,JK)=PMFDS5(JL,IKB)*ZZP25(JL,JK)
        PMFDQ5(JL,JK)=PMFDQ5(JL,IKB)*ZZP25(JL,JK)
      ENDIF
    ENDIF
  ENDDO
ENDDO

!*    2.            CALCULATE RAIN/SNOW FALL RATES
!*                  CALCULATE MELTING OF SNOW
!*                  CALCULATE EVAPORATION OF PRECIP
!                   -------------------------------

DO JK=KTOPM2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN
      PRAIN5(JL)=PRAIN5(JL)+PDMFUP5(JL,JK)
      ZMFLXS25(JL,JK)=PMFLXS5(JL,JK)
      IF(PMFLXS5(JL,JK) > 0.0_JPRB.AND.PTEN5(JL,JK) > RTT) THEN
        ZCONS15(JL,JK)=ZCONS1A*(1.0_JPRB+0.5_JPRB*(PTEN5(JL,JK)-RTT))
        ZFAC5(JL,JK)=ZCONS15(JL,JK)*(PAPH5(JL,JK+1)-PAPH5(JL,JK))
        IF (PMFLXS5(JL,JK) < ZFAC5(JL,JK)*(PTEN5(JL,JK)-RTT)) THEN
          ZSNMLT5(JL,JK) = PMFLXS5(JL,JK)
        ELSE
          ZSNMLT5(JL,JK) = ZFAC5(JL,JK)*(PTEN5(JL,JK)-RTT)
        ENDIF
        PDPMEL5(JL,JK)=ZSNMLT5(JL,JK)

        ZTARG25(JL,JK)=PTEN5(JL,JK)-ZSNMLT5(JL,JK)/ZFAC5(JL,JK)

        ZOEALFA5(JL,JK)=0.545_JPRB &
         & *(TANH(0.17_JPRB*(ZTARG25(JL,JK)-RLPTRC))+1.0_JPRB)  
        ZFOEEWL5(JL,JK)=R2ES*EXP(R3LES*(ZTARG25(JL,JK)-RTT) &
         & /(ZTARG25(JL,JK)-R4LES))  
        ZFOEEWI5(JL,JK)=R2ES*EXP(R3IES*(ZTARG25(JL,JK)-RTT) &
         & /(ZTARG25(JL,JK)-R4IES))  
        ZOEEWM5(JL,JK)=ZOEALFA5(JL,JK)*ZFOEEWL5(JL,JK) &
         & +(1.0_JPRB-ZOEALFA5(JL,JK))*ZFOEEWI5(JL,JK)  
        PQSEN5(JL,JK)=ZOEEWM5(JL,JK)/PAP5(JL,JK)
      ENDIF
      ZALFAW5(JL,JK)=0.545_JPRB*(TANH(0.17_JPRB*(PTEN5(JL,JK)-RLPTRC))+1.0_JPRB)
      ZDMFUP15(JL,JK)=PDMFUP5(JL,JK)
      ZDMFDP15(JL,JK)=PDMFDP5(JL,JK)
      ZALFAW15(JL,JK)=ZALFAW5(JL,JK)
!no liquid precipitation above melting level
      IF(PTEN5(JL,JK) < RTT .AND. ZALFAW5(JL,JK) > 0.0_JPRB) THEN
        PLGLAC5(JL,JK)=PLGLAC5(JL,JK) &
                    & +ZALFAW5(JL,JK)*(PDMFUP5(JL,JK)+PDMFDP5(JL,JK))
        ZALFAW5(JL,JK)=0.0_JPRB
      ENDIF
      ZALFAW25(JL,JK)=ZALFAW5(JL,JK)
      PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK)+ZALFAW5(JL,JK)*&
       & (PDMFUP5(JL,JK)+PDMFDP5(JL,JK))+PDPMEL5(JL,JK)  
      PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK)+(1.0_JPRB-ZALFAW5(JL,JK))*&
       & (PDMFUP5(JL,JK)+PDMFDP5(JL,JK))-PDPMEL5(JL,JK)  
      ZMFLXR15(JL,JK+1)=PMFLXR5(JL,JK+1)
      ZMFLXS15(JL,JK+1)=PMFLXS5(JL,JK+1)
      IF(PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1) < 0.0_JPRB) THEN
        PDMFDP5(JL,JK)=-(PMFLXR5(JL,JK)+PMFLXS5(JL,JK)+PDMFUP5(JL,JK))
        PMFLXR5(JL,JK+1)=0.0_JPRB
        PMFLXS5(JL,JK+1)=0.0_JPRB
        PDPMEL5(JL,JK)  =0.0_JPRB
      ELSEIF(PMFLXR5(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK+1)+PMFLXR5(JL,JK+1)
        PMFLXR5(JL,JK+1)=0.0_JPRB
      ELSEIF(PMFLXS5(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1)
        PMFLXS5(JL,JK+1)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO

! Reminder for conservation:
!    pdmfup(jl,jk)+pdmfdp(jl,jk)=pmflxr(jl,jk+1)+pmflxs(jl,jk+1)-pmflxr(jl,jk)-pmflxs(jl,jk)

DO JK=KTOPM2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCBOT(JL)) THEN 
      ZRFL5(JL,JK)=PMFLXR5(JL,JK)+PMFLXS5(JL,JK)
      IF(ZRFL5(JL,JK) > 1.E-20_JPRB) THEN
        Z1S5(JL,JK) = MAX(0.0_JPRB,PQSEN5(JL,JK)-PQEN5(JL,JK))
        ZCONS3=RCPECONS2*RCUCOV2
        ZSQRP5=SQRT(PAPH5(JL,JK)/PAPH5(JL,KLEV+1))
        ZFACT1=1.0_JPRB/(5.09E-3_JPRB*RCUCOV2)
        ZFACT25(JL,JK)=ZSQRP5*ZRFL5(JL,JK)*ZFACT1
        ZFACT35(JL,JK)=ZFACT25(JL,JK)**0.5777_JPRB
        ZDRFL15=ZCONS3*Z1S5(JL,JK)*ZFACT35(JL,JK) &
         & *(PAPH5(JL,JK+1)-PAPH5(JL,JK))  
        ZRNEW15(JL,JK)=ZRFL5(JL,JK)-ZDRFL15
        Z2S5(JL,JK) = MAX(0.0_JPRB,ZRHEBC(JL)*PQSEN5(JL,JK)-PQEN5(JL,JK))
        ZRMIN5(JL,JK)=ZRFL5(JL,JK)-RCUCOV2*Z2S5(JL,JK)*ZCONS2 &
         & *(PAPH5(JL,JK+1)-PAPH5(JL,JK))  
        ZRNEW25(JL,JK)=MAX(ZRNEW15(JL,JK),ZRMIN5(JL,JK))
        ZRFLN5(JL,JK)=MAX(ZRNEW25(JL,JK),0.0_JPRB)
        ZDRFL5(JL,JK)=MIN(ZRFLN5(JL,JK)-ZRFL5(JL,JK),0.0_JPRB)
        ZALFAW5(JL,JK)=0.545_JPRB*(TANH(0.17_JPRB*(PTEN5(JL,JK)-RLPTRC))+1.0_JPRB)
        IF(PTEN5(JL,JK) < RTT) THEN
          ZALFAW5(JL,JK)=0.0_JPRB
        ENDIF
        ZPDR5=ZALFAW5(JL,JK)*PDMFDP5(JL,JK)
        ZPDS5=(1.0_JPRB-ZALFAW5(JL,JK))*PDMFDP5(JL,JK)
        ZDENOM5(JL,JK)=1.0_JPRB/MAX(PMFLXR5(JL,JK)+PMFLXS5(JL,JK),1.E-20_JPRB)
        PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK)+ZPDR5 &
         & +PDPMEL5(JL,JK)+ZDRFL5(JL,JK)*PMFLXR5(JL,JK)*ZDENOM5(JL,JK)  
        PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK)+ZPDS5 &
         & -PDPMEL5(JL,JK)+ZDRFL5(JL,JK)*PMFLXS5(JL,JK)*ZDENOM5(JL,JK)  
        PDMFUP5(JL,JK)=PDMFUP5(JL,JK)+ZDRFL5(JL,JK)
        ZMFLXR35(JL,JK+1)=PMFLXR5(JL,JK+1)
        ZMFLXS35(JL,JK+1)=PMFLXS5(JL,JK+1)
        IF(PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1) < 0.0_JPRB) THEN
          PDMFUP5(JL,JK)=PDMFUP5(JL,JK)-(PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1))
          PMFLXR5(JL,JK+1)=0.0_JPRB
          PMFLXS5(JL,JK+1)=0.0_JPRB
          PDPMEL5(JL,JK)  =0.0_JPRB
        ELSEIF(PMFLXR5(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK+1)+PMFLXR5(JL,JK+1)
          PMFLXR5(JL,JK+1)=0.0_JPRB
        ELSEIF(PMFLXS5(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1)
          PMFLXS5(JL,JK+1)=0.0_JPRB
        ENDIF
      ELSE
        PMFLXR5(JL,JK+1)=0.0_JPRB
        PMFLXS5(JL,JK+1)=0.0_JPRB
        PDMFDP5(JL,JK)=0.0_JPRB
        PDPMEL5(JL,JK)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO

!     -----------------------------------------------------------
!     -----------------------------------------------------------

!     *****               ADJOINT CALCULATIONS             ******

!     -----------------------------------------------------------
!     -----------------------------------------------------------

!*    0.          INITIALIZATIONS
!                 ---------------

PRAIN(:)=0.0_JPRB

!     -----------------------------------------------------------

!*    1.0          DETERMINE FINAL CONVECTIVE FLUXES
!                  ---------------------------------

DO JK=KLEV,KTOPM2,-1
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCBOT(JL)) THEN

      ZRFL=0.0_JPRB

      IF(ZRFL5(JL,JK) > 1.E-20_JPRB) THEN

        ZDRFL=0.0_JPRB
        ZDENOM=0.0_JPRB
        ZALFAW=0.0_JPRB
        ZPDS=0.0_JPRB
        ZPDR=0.0_JPRB
        ZRFLN=0.0_JPRB
        ZRNEW=0.0_JPRB
        ZRMIN=0.0_JPRB
        Z2S=0.0_JPRB
        ZDRFL1=0.0_JPRB
        Z1S=0.0_JPRB
        ZFACT3=0.0_JPRB
        ZFACT2=0.0_JPRB

        ZCONS3=RCPECONS2*RCUCOV2

        IF(ZMFLXR35(JL,JK+1)+ZMFLXS35(JL,JK+1) < 0.0_JPRB) THEN
          PDPMEL(JL,JK)  =0.0_JPRB
          PMFLXS(JL,JK+1)=0.0_JPRB
          PMFLXR(JL,JK+1)=0.0_JPRB
          PMFLXR(JL,JK+1)=PMFLXR(JL,JK+1)-PDMFUP(JL,JK)
          PMFLXS(JL,JK+1)=PMFLXS(JL,JK+1)-PDMFUP(JL,JK)
        ELSEIF(ZMFLXR35(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXR(JL,JK+1)=0.0_JPRB
          PMFLXR(JL,JK+1)=PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1)
        ELSEIF(ZMFLXS35(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXS(JL,JK+1)=0.0_JPRB
          PMFLXS(JL,JK+1)=PMFLXS(JL,JK+1)+PMFLXR(JL,JK+1)
        ENDIF

        ZDRFL=ZDRFL+PDMFUP(JL,JK)
 
        PMFLXS(JL,JK)=PMFLXS(JL,JK)+PMFLXS(JL,JK+1) &
         & *(1.0_JPRB+ZDRFL5(JL,JK)*ZDENOM5(JL,JK))  
        ZPDS=ZPDS+PMFLXS(JL,JK+1)
        PDPMEL(JL,JK)=PDPMEL(JL,JK)-PMFLXS(JL,JK+1)
        ZDRFL=ZDRFL+PMFLXS(JL,JK+1)*PMFLXS5(JL,JK)*ZDENOM5(JL,JK)
        ZDENOM=ZDENOM+PMFLXS(JL,JK+1)*ZDRFL5(JL,JK)*PMFLXS5(JL,JK)
        PMFLXS(JL,JK+1)=0.0_JPRB

        PMFLXR(JL,JK)=PMFLXR(JL,JK)+PMFLXR(JL,JK+1) &
         & *(1.0_JPRB+ZDRFL5(JL,JK)*ZDENOM5(JL,JK))  
        ZPDR=ZPDR+PMFLXR(JL,JK+1)
        PDPMEL(JL,JK)=PDPMEL(JL,JK)+PMFLXR(JL,JK+1)
        ZDRFL=ZDRFL+PMFLXR(JL,JK+1)*PMFLXR5(JL,JK)*ZDENOM5(JL,JK)
        ZDENOM=ZDENOM+PMFLXR(JL,JK+1)*ZDRFL5(JL,JK)*PMFLXR5(JL,JK)
        PMFLXR(JL,JK+1)=0.0_JPRB

        IF ((PMFLXR5(JL,JK)+PMFLXS5(JL,JK)) > 1.E-20_JPRB) THEN
          PMFLXR(JL,JK)=PMFLXR(JL,JK)-ZDENOM*(ZDENOM5(JL,JK)**2)
          PMFLXS(JL,JK)=PMFLXS(JL,JK)-ZDENOM*(ZDENOM5(JL,JK)**2)
        ENDIF
        ZDENOM=0.0_JPRB

        PDMFDP(JL,JK)=PDMFDP(JL,JK)+ZPDS*(1.0_JPRB-ZALFAW5(JL,JK))
        ZALFAW=ZALFAW-ZPDS*PDMFDP5(JL,JK)
        ZPDS=0.0_JPRB

        PDMFDP(JL,JK)=PDMFDP(JL,JK)+ZPDR*ZALFAW5(JL,JK)
        ZALFAW=ZALFAW+ZPDR*PDMFDP5(JL,JK)
        ZPDR=0.0_JPRB
        
        IF(PTEN5(JL,JK) < RTT) ZALFAW=0.0_JPRB

        PTEN(JL,JK)=PTEN(JL,JK)+ZALFAW*0.17_JPRB*0.545_JPRB &
           & *(1.0_JPRB-TANH(0.17_JPRB*(PTEN5(JL,JK)-RLPTRC))**2)  
        ZALFAW=0.0_JPRB
        
        IF ((ZRFLN5(JL,JK)-ZRFL5(JL,JK)) < 0.0_JPRB) THEN
          ZRFLN=ZRFLN+ZDRFL
          ZRFL=ZRFL-ZDRFL
        ENDIF
        ZDRFL=0.0_JPRB

        IF (ZRNEW25(JL,JK) > 0.0_JPRB) THEN
          ZRNEW=ZRNEW+ZRFLN
        ENDIF
        ZRFLN=0.0_JPRB
  
        IF (ZRNEW15(JL,JK) < ZRMIN5(JL,JK)) THEN
          ZRMIN=ZRMIN+ZRNEW
          ZRNEW=0.0_JPRB
        ENDIF

        ZRFL=ZRFL+ZRMIN
        Z2S=Z2S-ZRMIN*RCUCOV2*ZCONS2*(PAPH5(JL,JK+1)-PAPH5(JL,JK))
        PAPH(JL,JK+1)=PAPH(JL,JK+1)-ZRMIN*RCUCOV2*ZCONS2*Z2S5(JL,JK)
        PAPH(JL,JK  )=PAPH(JL,JK  )+ZRMIN*RCUCOV2*ZCONS2*Z2S5(JL,JK)
        ZRMIN=0.0_JPRB
 
        IF (ZRHEBC(JL)*PQSEN5(JL,JK)-PQEN5(JL,JK) > 0.0_JPRB) THEN
          PQSEN(JL,JK)=PQSEN(JL,JK)+ZRHEBC(JL)*Z2S
          PQEN (JL,JK)=PQEN (JL,JK)-Z2S
        ENDIF
        Z2S=0.0_JPRB
                
        ZRFL=ZRFL+ZRNEW
        ZDRFL1=ZDRFL1-ZRNEW
        ZRNEW=0.0_JPRB
        
        Z1S=Z1S+ZDRFL1*ZCONS3*ZFACT35(JL,JK) &
         & *(PAPH5(JL,JK+1)-PAPH5(JL,JK))  
        ZFACT3=ZFACT3+ZDRFL1*ZCONS3*Z1S5(JL,JK) &
         & *(PAPH5(JL,JK+1)-PAPH5(JL,JK))  
        PAPH(JL,JK+1)=PAPH(JL,JK+1)+ZDRFL1*ZCONS3*Z1S5(JL,JK) &
         & *ZFACT35(JL,JK)  
        PAPH(JL,JK  )=PAPH(JL,JK  )-ZDRFL1*ZCONS3*Z1S5(JL,JK) &
         & *ZFACT35(JL,JK)  
        ZDRFL1=0.0_JPRB
        
        ZFACT2=ZFACT2+ZFACT3*0.5777_JPRB &
         & *ZFACT35(JL,JK)/ZFACT25(JL,JK)  
        ZFACT3=0.0_JPRB
 
! Redefine constant
        ZFACT1=1.0_JPRB/(5.09E-3_JPRB*RCUCOV2)

        ZSQRP5=SQRT(PAPH5(JL,JK)/PAPH5(JL,KLEV+1))
        ZRFL=ZRFL+ZFACT2*ZSQRP5*ZFACT1
        PAPH(JL,JK)=PAPH(JL,JK)+ZFACT2*ZSQRP5 &
         & *ZRFL5(JL,JK)/PAPH5(JL,JK)*ZFACT1  
        PAPH(JL,KLEV+1)=PAPH(JL,KLEV+1)-ZFACT2*ZSQRP5 &
         & *ZRFL5(JL,JK)/PAPH5(JL,KLEV+1)*ZFACT1  
        ZFACT2=0.0_JPRB

        IF (PQSEN5(JL,JK)-PQEN5(JL,JK) > 0.0_JPRB) THEN
          PQSEN(JL,JK)=PQSEN(JL,JK)+Z1S
          PQEN (JL,JK)=PQEN (JL,JK)-Z1S
        ENDIF
        Z1S=0.0_JPRB

      ELSE
        PDPMEL(JL,JK)=0.0_JPRB
        PDMFDP(JL,JK)=0.0_JPRB
        PMFLXS(JL,JK+1)=0.0_JPRB
        PMFLXR(JL,JK+1)=0.0_JPRB
      ENDIF

      PMFLXR(JL,JK)=PMFLXR(JL,JK)+ZRFL
      PMFLXS(JL,JK)=PMFLXS(JL,JK)+ZRFL
      ZRFL=0.0_JPRB

    ENDIF
  ENDDO
ENDDO

!*    2.            CALCULATE RAIN/SNOW FALL RATES
!*                  CALCULATE MELTING OF SNOW
!*                  CALCULATE EVAPORATION OF PRECIP
!                   -------------------------------

DO JK=KLEV,KTOPM2,-1
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN
     
      ZALFAW=0.0_JPRB

      IF(ZMFLXR15(JL,JK+1)+ZMFLXS15(JL,JK+1) < 0.0_JPRB) THEN
        PDPMEL(JL,JK)=0.0_JPRB
        PMFLXS(JL,JK+1)=0.0_JPRB
        PMFLXR(JL,JK+1)=0.0_JPRB
        PMFLXR(JL,JK)=PMFLXR(JL,JK)-PDMFDP(JL,JK)
        PMFLXS(JL,JK)=PMFLXS(JL,JK)-PDMFDP(JL,JK)
        PDMFUP(JL,JK)=PDMFUP(JL,JK)-PDMFDP(JL,JK)
        PDMFDP(JL,JK)=0.0_JPRB
      ELSEIF(ZMFLXR15(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXR(JL,JK+1)=0.0_JPRB
        PMFLXR(JL,JK+1)=PMFLXR(JL,JK+1)+PMFLXS(JL,JK+1)
      ELSEIF(ZMFLXS15(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXS(JL,JK+1)=0.0_JPRB
        PMFLXS(JL,JK+1)=PMFLXS(JL,JK+1)+PMFLXR(JL,JK+1)
      ENDIF
     
      PMFLXS(JL,JK)=PMFLXS(JL,JK)+PMFLXS(JL,JK+1)
      ZALFAW=ZALFAW-PMFLXS(JL,JK+1)*(ZDMFUP15(JL,JK)+ZDMFDP15(JL,JK))
      PDMFUP(JL,JK)=PDMFUP(JL,JK)+PMFLXS(JL,JK+1)*(1.0_JPRB-ZALFAW25(JL,JK))
      PDMFDP(JL,JK)=PDMFDP(JL,JK)+PMFLXS(JL,JK+1)*(1.0_JPRB-ZALFAW25(JL,JK))
      PDPMEL(JL,JK)=PDPMEL(JL,JK)-PMFLXS(JL,JK+1)
      PMFLXS(JL,JK+1)=0.0_JPRB

      PMFLXR(JL,JK)=PMFLXR(JL,JK)+PMFLXR(JL,JK+1)
      ZALFAW=ZALFAW+PMFLXR(JL,JK+1)*(ZDMFUP15(JL,JK)+ZDMFDP15(JL,JK))
      PDMFUP(JL,JK)=PDMFUP(JL,JK)+PMFLXR(JL,JK+1)*ZALFAW25(JL,JK)
      PDMFDP(JL,JK)=PDMFDP(JL,JK)+PMFLXR(JL,JK+1)*ZALFAW25(JL,JK)
      PDPMEL(JL,JK)=PDPMEL(JL,JK)+PMFLXR(JL,JK+1)
      PMFLXR(JL,JK+1)=0.0_JPRB

!no liquid precipitation above melting level
      IF(PTEN5(JL,JK) < RTT .AND. ZALFAW15(JL,JK) > 0.0_JPRB) THEN
        ZALFAW=0.0_JPRB

        ZALFAW = ZALFAW + (ZDMFUP15(JL,JK)+ZDMFDP15(JL,JK))*PLGLAC(JL,JK)
        PDMFUP(JL,JK) = PDMFUP(JL,JK) + ZALFAW15(JL,JK)*PLGLAC(JL,JK)
        PDMFDP(JL,JK) = PDMFDP(JL,JK) + ZALFAW15(JL,JK)*PLGLAC(JL,JK)
      ENDIF

      PTEN(JL,JK)=PTEN(JL,JK)+ZALFAW*0.545_JPRB*0.17_JPRB &
         & *(1.0_JPRB-TANH(0.17_JPRB*(PTEN5(JL,JK)-RLPTRC))**2)  
      ZALFAW=0.0_JPRB

      IF(ZMFLXS25(JL,JK) > 0.0_JPRB.AND.PTEN5(JL,JK) > RTT) THEN
        ZOEEWM=0.0_JPRB
        ZFOEEWI=0.0_JPRB
        ZFOEEWL=0.0_JPRB
        ZOEALFA=0.0_JPRB
        ZTARG=0.0_JPRB
        ZSNMLT=0.0_JPRB
        ZFAC=0.0_JPRB
        ZCONS1=0.0_JPRB

        ZOEEWM=ZOEEWM+PQSEN(JL,JK)/PAP5(JL,JK)
        PAP(JL,JK)=PAP(JL,JK)-PQSEN(JL,JK)*ZOEEWM5(JL,JK) &
         & /(PAP5(JL,JK)**2)  
        PQSEN(JL,JK)=0.0_JPRB
        
        ZFOEEWL=ZFOEEWL+ZOEEWM*ZOEALFA5(JL,JK)
        ZFOEEWI=ZFOEEWI+ZOEEWM*(1.0_JPRB-ZOEALFA5(JL,JK))
        ZOEALFA=ZOEALFA+ZOEEWM*(ZFOEEWL5(JL,JK)-ZFOEEWI5(JL,JK))  
        ZOEEWM=0.0_JPRB

        ZTARG=ZTARG+ZFOEEWI*ZFOEEWI5(JL,JK)*R3IES &
         & *(RTT-R4IES)/(ZTARG25(JL,JK)-R4IES)**2  
        ZFOEEWI=0.0_JPRB

        ZTARG=ZTARG+ZFOEEWL*ZFOEEWL5(JL,JK)*R3LES &
         & *(RTT-R4LES)/(ZTARG25(JL,JK)-R4LES)**2  
        ZFOEEWL=0.0_JPRB
        
        ZTARG=ZTARG+ZOEALFA*0.545_JPRB*0.17_JPRB &
         & *(1.0_JPRB-TANH(0.17_JPRB*(ZTARG25(JL,JK)-RLPTRC))**2)  
        ZOEALFA=0.0_JPRB

        PTEN(JL,JK)=PTEN(JL,JK)+ZTARG
        ZSNMLT=ZSNMLT-ZTARG/ZFAC5(JL,JK)
        ZFAC=ZFAC+ZTARG*ZSNMLT5(JL,JK)/(ZFAC5(JL,JK)**2)
        ZTARG=0.0_JPRB

        ZSNMLT=ZSNMLT+PDPMEL(JL,JK)
        PDPMEL(JL,JK)=0.0_JPRB

        IF (ZMFLXS25(JL,JK) < ZFAC5(JL,JK)*(PTEN5(JL,JK)-RTT)) THEN
          PMFLXS(JL,JK)=PMFLXS(JL,JK)+ZSNMLT
        ELSE
          ZFAC=ZFAC+ZSNMLT*(PTEN5(JL,JK)-RTT) 
          PTEN(JL,JK)=PTEN(JL,JK)+ZSNMLT*ZFAC5(JL,JK)
        ENDIF
        ZSNMLT=0.0_JPRB

        ZCONS1=ZCONS1+ZFAC*(PAPH5(JL,JK+1)-PAPH5(JL,JK))
        PAPH(JL,JK+1)=PAPH(JL,JK+1)+ZFAC*ZCONS15(JL,JK) 
        PAPH(JL,JK  )=PAPH(JL,JK  )-ZFAC*ZCONS15(JL,JK) 
        ZFAC=0.0_JPRB

        PTEN(JL,JK)=PTEN(JL,JK)+ZCONS1*ZCONS1A*0.5_JPRB
        ZCONS1=0.0_JPRB
      ENDIF

      PDMFUP(JL,JK)=PDMFUP(JL,JK)+PRAIN(JL)
    ENDIF
  ENDDO
ENDDO

DO JK=KLEV,KTOPM2,-1
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK > KCBOT(JL)+1) THEN
      IKB=KCBOT(JL)+1
 
      ZZP=0.0_JPRB
      ZP1=0.0_JPRB
      ZP2=0.0_JPRB

      IF(LDDRAF(JL).AND.ZMFD25(JL,JK) == 0.0_JPRB) THEN
        PMFDQ(JL,IKB)=PMFDQ(JL,IKB)+PMFDQ(JL,JK)*ZZP25(JL,JK)
        ZZP=ZZP+PMFDQ(JL,JK)*PMFDQ5(JL,IKB)
        PMFDQ(JL,JK)=0.0_JPRB

        PMFDS(JL,IKB)=PMFDS(JL,IKB)+PMFDS(JL,JK)*ZZP25(JL,JK)
        ZZP=ZZP+PMFDS(JL,JK)*PMFDS5(JL,IKB)
        PMFDS(JL,JK)=0.0_JPRB

        PMFD(JL,IKB)=PMFD(JL,IKB)+PMFD(JL,JK)*ZZP25(JL,JK)
        ZZP=ZZP+PMFD(JL,JK)*PMFD5(JL,IKB)
        PMFD(JL,JK)=0.0_JPRB
      ENDIF

      PMFUL(JL,JK)=0.0_JPRB
   
      PMFUQ(JL,IKB)=PMFUQ(JL,IKB)+PMFUQ(JL,JK)*ZZP25(JL,JK)
      ZZP=ZZP+PMFUQ(JL,JK)*PMFUQ5(JL,IKB)
      PMFUQ(JL,JK)=0.0_JPRB

      PMFUS(JL,IKB)=PMFUS(JL,IKB)+PMFUS(JL,JK)*ZZP25(JL,JK)
      ZZP=ZZP+PMFUS(JL,JK)*PMFUS5(JL,IKB)
      PMFUS(JL,JK)=0.0_JPRB

      PMFU(JL,IKB)=PMFU(JL,IKB)+PMFU(JL,JK)*ZZP25(JL,JK)
      ZZP=ZZP+PMFU(JL,JK)*PMFU5(JL,IKB)
      PMFU(JL,JK)=0.0_JPRB
      
      IF(KTYPE(JL) == 3) THEN
        ZZP=2.0_JPRB*ZZP*ZZP15(JL,JK)
      ENDIF

      ZP1=ZP1+ZZP/ZP25(JL,JK)
      ZP2=ZP2-ZZP*ZP15(JL,JK)/(ZP25(JL,JK)**2)
      ZZP=0.0_JPRB

      PAPH(JL,KLEV+1)=PAPH(JL,KLEV+1)+ZP2
      PAPH(JL,IKB)=PAPH(JL,IKB)-ZP2
      ZP2=0.0_JPRB

      PAPH(JL,KLEV+1)=PAPH(JL,KLEV+1)+ZP1
      PAPH(JL,JK)=PAPH(JL,JK)-ZP1
      ZP1=0.0_JPRB
    ENDIF
  ENDDO
ENDDO

!DIR$ IVDEP
!OCL NOVREC
DO JL=KIDIA,KFDIA
  IF(LDCUM(JL)) THEN
    IKB=KCBOT(JL)
    IK=IKB+1
    
    ZZP=0.0_JPRB
    ZOELHM=0.0_JPRB
    ZP1=0.0_JPRB
    ZP2=0.0_JPRB

    PMFUL(JL,IK)=0.0_JPRB
 
    PMFUQ(JL,IKB)=PMFUQ(JL,IKB)+PMFUQ(JL,IK)*ZZP25(JL,IK) 
    PMFUL(JL,IKB)=PMFUL(JL,IKB)+PMFUQ(JL,IK)*ZZP25(JL,IK) 
    ZZP=ZZP+PMFUQ(JL,IK)*(PMFUQ5(JL,IKB)+PMFUL5(JL,IKB))
    PMFUQ(JL,IK)=0.0_JPRB

    ZOEALFA=0.0_JPRB
    ZTARG=0.0_JPRB
 
    PMFUS(JL,IKB)=PMFUS(JL,IKB)+PMFUS(JL,IK)*ZZP25(JL,IK)
    PMFUL(JL,IKB)=PMFUL(JL,IKB)-PMFUS(JL,IK)*ZOELHM5(JL,IK) &
     & *ZZP25(JL,IK)  
    ZOELHM=ZOELHM-PMFUS(JL,IK)*PMFUL5(JL,IKB)*ZZP25(JL,IK)
    ZZP=ZZP+PMFUS(JL,IK)*(PMFUS5(JL,IKB) &
     & -ZOELHM5(JL,IK)*PMFUL5(JL,IKB))  
    PMFUS(JL,IK)=0.0_JPRB
 
    ZOEALFA=ZOEALFA+ZOELHM*(RLVTT-RLSTT)
    ZOELHM=0.0_JPRB 
      
    ZTARG=ZTARG+ZOEALFA*0.545_JPRB*0.17_JPRB &
     & *(1.0_JPRB-TANH(0.17_JPRB*(ZTARG15(JL)-RLPTRC))**2)  
    ZOEALFA=0.0_JPRB

    PTENH(JL,IKB)=PTENH(JL,IKB)+ZTARG
    ZTARG=0.0_JPRB

    PMFU(JL,IKB)=PMFU(JL,IKB)+PMFU(JL,IK)*ZZP25(JL,IK)
    ZZP=ZZP+PMFU(JL,IK)*PMFU5(JL,IKB)
    PMFU(JL,IK)=0.0_JPRB

    IF(KTYPE(JL) == 3) THEN
      ZZP=2.0_JPRB*ZZP*ZZP15(JL,IK)
    ENDIF

    ZP1=ZP1+ZZP/ZP25(JL,IK)
    ZP2=ZP2-ZZP*ZP15(JL,IK)/(ZP25(JL,IK)**2)
    ZZP=0.0_JPRB

    PAPH(JL,KLEV+1)=PAPH(JL,KLEV+1)+ZP2
    PAPH(JL,IKB)=PAPH(JL,IKB)-ZP2
    ZP2=0.0_JPRB

    PAPH(JL,KLEV+1)=PAPH(JL,KLEV+1)+ZP1
    PAPH(JL,IK)=PAPH(JL,IK)-ZP1
    ZP1=0.0_JPRB
  ENDIF
ENDDO

PMFLXS(:,KLEV+1)=0.0_JPRB
PMFLXR(:,KLEV+1)=0.0_JPRB

DO JK=KLEV,KTOPM2,-1
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN
      IF(LDDRAF(JL).AND.JK >= KDTOP(JL)) THEN
        PMFD(JL,JK) = PMFD(JL,JK) +PMFDQ(JL,JK)*(ZMFDQ15(JL,JK)-PQENH5(JL,JK))
        PQENH(JL,JK)= PQENH(JL,JK)-PMFDQ(JL,JK)*ZMFD15(JL,JK)
        PMFDQ(JL,JK)=              PMFDQ(JL,JK)*ZMFD15(JL,JK)

        PMFD(JL,JK) = PMFD(JL,JK) +PMFDS(JL,JK)*RCPD*(ZMFDS15(JL,JK)-PTENH5(JL,JK))
        PTENH(JL,JK)= PTENH(JL,JK)-PMFDS(JL,JK)*RCPD*ZMFD15(JL,JK)
        PMFDS(JL,JK)=              PMFDS(JL,JK)*RCPD*ZMFD15(JL,JK)            
      ELSE
        PDMFDP (JL,JK-1)=0.0_JPRB
        PMFDQ (JL,JK)=0.0_JPRB
        PMFDS (JL,JK)=0.0_JPRB
        PMFD (JL,JK)=0.0_JPRB
      ENDIF

      IF (JK < KLEV) THEN
        PMFU(JL,JK+1)= PMFU(JL,JK+1)+PLUDE(JL,JK)*ZLUDE15(JL,JK)
        PLUDE(JL,JK) =               PLUDE(JL,JK)*ZMFU25(JL,JK)
      ELSE
        PLUDE(JL,JK)=0.0_JPRB
      ENDIF

      PMFU(JL,JK)  = PMFU(JL,JK)+PLGLAC(JL,JK)*ZLGLAC15(JL,JK)
      PLGLAC(JL,JK)=             PLGLAC(JL,JK)*ZMFU15(JL,JK)

      PMFU(JL,JK) = PMFU(JL,JK)+PMFUL(JL,JK)*ZMFUL15(JL,JK)
      PMFUL(JL,JK)=             PMFUL(JL,JK)*ZMFU15(JL,JK)

      PMFU(JL,JK) = PMFU(JL,JK) +PMFUQ(JL,JK)*(ZMFUQ15(JL,JK)-PQENH5(JL,JK))
      PQENH(JL,JK)= PQENH(JL,JK)-PMFUQ(JL,JK)*ZMFU15(JL,JK)
      PMFUQ(JL,JK)=              PMFUQ(JL,JK)*ZMFU15(JL,JK)

      PMFU(JL,JK) = PMFU(JL,JK) +PMFUS(JL,JK)*RCPD*(ZMFUS15(JL,JK)-PTENH5(JL,JK))
      PTENH(JL,JK)= PTENH(JL,JK)-PMFUS(JL,JK)*RCPD*ZMFU15(JL,JK)
      PMFUS(JL,JK)=              PMFUS(JL,JK)*RCPD*ZMFU15(JL,JK)            

      PDPMEL(JL,JK)=0.0_JPRB
      PMFLXS(JL,JK)=0.0_JPRB
      PMFLXR(JL,JK)=0.0_JPRB
    ELSE      
      PMFU (JL,JK)=0.0_JPRB
      PMFD (JL,JK)=0.0_JPRB
      PMFUS (JL,JK)=0.0_JPRB
      PMFDS (JL,JK)=0.0_JPRB
      PMFUQ (JL,JK)=0.0_JPRB
      PMFDQ (JL,JK)=0.0_JPRB
      PMFUL (JL,JK)=0.0_JPRB
      PLGLAC (JL,JK)=0.0_JPRB
      PDMFUP (JL,JK-1)=0.0_JPRB
      PDMFDP (JL,JK-1)=0.0_JPRB
      PLUDE (JL,JK-1)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  PRAIN(JL)=0.0_JPRB
ENDDO

IF (LHOOK) CALL DR_HOOK('CUFLX2AD',1,ZHOOK_HANDLE)
END SUBROUTINE CUFLX2AD
