#ifdef RS6K
@PROCESS NOCHECK
#endif

SUBROUTINE FIXSPPT( PGMVS, DPTENQ, PGFLT1, PGMVT1 )
!
!     Purpose.   Correct SPPT tendencies to guarantee that average does not change
!     --------   
!
!*   Interface.
!     ----------
!
!        *CALL* *FIXSPPT (PGMVS, DPTENQ, PGFLT1)
!
!     Explicit arguments :
!        --------------------
!
!     INPUT:
!     -------------
!        PGMVS      : surface GMV variables 
!        DPTENQ     : tendency change in Q due to SPPT (before and after), computed in SPPTEN
!        PGFLT1     : GFL tendencies
!        PGMVT1     : GMV tendencies
!
!     OUTPUT:
!     -------
!        PGFLT1      : GFL tendencies
!        PGMVT1      : GMV tendencies
!
!     Implicit arguments :  None.
!     --------------------
!
!     Author:
!     -------
!     June 2015 - Jost von Hardenberg *ISAC-CNR* 


USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK

USE YOM_YGFL , ONLY : YGFL, YQ

USE YOMGMV   , ONLY : YPH9, NDIMGMVS, YT1
USE YOMGEM   , ONLY : NGPTOT, VAH, VBH  

USE YOMDIM   , ONLY : NPROMA, NGPBLKS, NFLEVG
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

! arguments

REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVS(NPROMA,NDIMGMVS,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: DPTENQ(NPROMA,NFLEVG,4,2,NGPBLKS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1,NGPBLKS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM,NGPBLKS)

! Local variables

INTEGER(KIND=JPIM) :: IBL,ICEND,JKGLO,JROF
INTEGER(KIND=JPIM) :: IST, IEND, JLEV , JGFL, JFLD
REAL(KIND=JPRB) :: ZPRE9(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZPRES9(NPROMA,0:NFLEVG,NGPBLKS)
REAL(KIND=JPRB) :: ZSPT9(NPROMA)
 
REAL(KIND=JPRB) :: ZFIELD3DT1(NPROMA,NFLEVG),ZFIELD3DT0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZFIELD2DT1(NPROMA,0:NFLEVG),ZFIELD2DT0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZFIELD2DT2(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZFIELD3DT2(NPROMA,NFLEVG)

REAL(KIND=JPRB) :: ZFIELD1D(NPROMA,12,NGPBLKS) ,ZNORMS(3,12)
REAL(KIND=JPRB) :: ZWEIGHT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnorm1.intfb.h"
#include "gppwc.intfb.h"
#include "gppreh.intfb.h"

IF (LHOOK) CALL DR_HOOK('FIXSPPT',0,ZHOOK_HANDLE)

! calculate pressure 
DO JKGLO=1,NGPTOT,NPROMA
   ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
   IBL=(JKGLO-1)/NPROMA+1
   IST=1
   IEND=ICEND
   DO JROF=IST,IEND
        ZSPT9(JROF)=PGMVS(JROF,YPH9%MSP,IBL)
        ZPRE9 (JROF,NFLEVG) = EXP(ZSPT9(JROF))
   ENDDO
   CALL GPPREH(NPROMA,IST,IEND,NFLEVG, VAH,VBH,ZPRE9)
   DO JLEV=0,NFLEVG
     DO JROF=IST,IEND
       ZPRES9(JROF,JLEV,IBL) = ZPRE9(JROF,JLEV)
     ENDDO
  ENDDO 
ENDDO

! Calculate total average mass of Q, energy and momentum tendencies before and after advection
! NB: we use GPPWC also to compute the mass-weighted vertical integrals of T,U and V tendencies 
! e.g. int(C_p T(p) dp)/g. We can neglect the constants C_p and g.

DO JKGLO=1,NGPTOT,NPROMA
   ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
   IBL=(JKGLO-1)/NPROMA+1
   IST=1
   IEND=ICEND
   DO JFLD=1,4
      DO JLEV=1,NFLEVG
         DO JROF=IST,IEND
            ZFIELD3DT0(JROF,JLEV)=DPTENQ(JROF,JLEV,JFLD,1,IBL)
            ZFIELD3DT1(JROF,JLEV)=DPTENQ(JROF,JLEV,JFLD,2,IBL)
            ZFIELD3DT2(JROF,JLEV)=ABS(DPTENQ(JROF,JLEV,JFLD,2,IBL)-DPTENQ(JROF,JLEV,JFLD,1,IBL))
            ZPRE9 (JROF,JLEV) =  ZPRES9(JROF,JLEV,IBL) 
         ENDDO
      ENDDO
      CALL GPPWC(NPROMA,IST,IEND,NFLEVG,ZFIELD2DT0,ZFIELD3DT0,ZPRE9)
      CALL GPPWC(NPROMA,IST,IEND,NFLEVG,ZFIELD2DT1,ZFIELD3DT1,ZPRE9)
      CALL GPPWC(NPROMA,IST,IEND,NFLEVG,ZFIELD2DT2,ZFIELD3DT2,ZPRE9)
      DO JROF=IST,IEND
         ZFIELD1D(JROF,1+(JFLD-1)*3,IBL)=ZFIELD2DT0(JROF,NFLEVG)
         ZFIELD1D(JROF,2+(JFLD-1)*3,IBL)=ZFIELD2DT1(JROF,NFLEVG)
         ZFIELD1D(JROF,3+(JFLD-1)*3,IBL)=ZFIELD2DT2(JROF,NFLEVG)
      ENDDO
   ENDDO
ENDDO

! norms are broadcast if PNORMS=ZNORMS specified
CALL GPNORM1(ZFIELD1D,12,.FALSE.,PNORMS=ZNORMS)

! Diagnostics
!WRITE(NULOUT,'("FIXSPPTQ1:",3e16.7)') ZNORMS(1,1),ZNORMS(2,1),ZNORMS(3,1)
!WRITE(NULOUT,'("FIXSPPTQ2:",3e16.7)') ZNORMS(1,2),ZNORMS(2,2),ZNORMS(3,2)
!WRITE(NULOUT,'("FIXSPPTQ3:",3e16.7)') ZNORMS(1,3),ZNORMS(2,3),ZNORMS(3,3)
!WRITE(NULOUT,'("FIXSPPTT1:",3e16.7)') ZNORMS(1,4),ZNORMS(2,4),ZNORMS(3,4)
!WRITE(NULOUT,'("FIXSPPTT2:",3e16.7)') ZNORMS(1,5),ZNORMS(2,5),ZNORMS(3,5)
!WRITE(NULOUT,'("FIXSPPTT3:",3e16.7)') ZNORMS(1,6),ZNORMS(2,6),ZNORMS(3,6)
!WRITE(NULOUT,'("FIXSPPTU1:",3e16.7)') ZNORMS(1,7),ZNORMS(2,7),ZNORMS(3,7)
!WRITE(NULOUT,'("FIXSPPTU2:",3e16.7)') ZNORMS(1,8),ZNORMS(2,8),ZNORMS(3,8)
!WRITE(NULOUT,'("FIXSPPTU3:",3e16.7)') ZNORMS(1,9),ZNORMS(2,9),ZNORMS(3,9)
!WRITE(NULOUT,'("FIXSPPTV1:",3e16.7)') ZNORMS(1,10),ZNORMS(2,10),ZNORMS(3,10)
!WRITE(NULOUT,'("FIXSPPTV2:",3e16.7)') ZNORMS(1,11),ZNORMS(2,11),ZNORMS(3,11)
!WRITE(NULOUT,'("FIXSPPTV3:",3e16.7)') ZNORMS(1,12),ZNORMS(2,12),ZNORMS(3,12)
    
! Q,T,U,V tendency after SPPT minus tendency before SPPT 
! p=>p +<p0>*w -<p1>*w 
! w=abs(p0-p1)/<abs(p0-p1)> 
! This way we distribute the excess/defect based on the amplitude of the local change in p due to SPPT
    
! Calculate corrected tendencies PGFLT1 and PGMVT1
DO JKGLO=1,NGPTOT,NPROMA
   ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
   IBL=(JKGLO-1)/NPROMA+1
   IST=1
   IEND=ICEND
   DO JLEV=1,NFLEVG
      DO JROF=IST,IEND
         ZWEIGHT=ABS(DPTENQ(JROF,JLEV,1,2,IBL)-DPTENQ(JROF,JLEV,1,1,IBL))/ZNORMS(1,3)
         PGFLT1(JROF,JLEV,YQ%MP1,IBL)=PGFLT1(JROF,JLEV,YQ%MP1,IBL) + ZWEIGHT*ZNORMS(1,1) - ZWEIGHT*ZNORMS(1,2) 
         ZWEIGHT=ABS(DPTENQ(JROF,JLEV,2,2,IBL)-DPTENQ(JROF,JLEV,2,1,IBL))/ZNORMS(1,6)
         PGMVT1(JROF,JLEV,YT1%MT,IBL)=PGMVT1(JROF,JLEV,YT1%MT,IBL) + ZWEIGHT*ZNORMS(1,4) - ZWEIGHT*ZNORMS(1,5) 
         ZWEIGHT=ABS(DPTENQ(JROF,JLEV,3,2,IBL)-DPTENQ(JROF,JLEV,3,1,IBL))/ZNORMS(1,9)
         PGMVT1(JROF,JLEV,YT1%MU,IBL)=PGMVT1(JROF,JLEV,YT1%MU,IBL) + ZWEIGHT*ZNORMS(1,7) - ZWEIGHT*ZNORMS(1,8) 
         ZWEIGHT=ABS(DPTENQ(JROF,JLEV,4,2,IBL)-DPTENQ(JROF,JLEV,4,1,IBL))/ZNORMS(1,12)
         PGMVT1(JROF,JLEV,YT1%MV,IBL)=PGMVT1(JROF,JLEV,YT1%MV,IBL) + ZWEIGHT*ZNORMS(1,10) - ZWEIGHT*ZNORMS(1,11) 
      ENDDO
   ENDDO   
ENDDO

! Sanity check
!DO JKGLO=1,NGPTOT,NPROMA
!   ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
!   IBL=(JKGLO-1)/NPROMA+1
!   IST=1
!   IEND=ICEND
!   DO JFLD=1,4
!      DO JLEV=1,NFLEVG
!         DO JROF=IST,IEND
!            ZWEIGHT=ABS(DPTENQ(JROF,JLEV,JFLD,2,IBL)-DPTENQ(JROF,JLEV,JFLD,1,IBL))/ZNORMS(1,3+(JFLD-1)*3)
!            ZFIELD3DT1(JROF,JLEV)=DPTENQ(JROF,JLEV,JFLD,2,IBL) + ZWEIGHT*ZNORMS(1,1+(JFLD-1)*3) - ZWEIGHT*ZNORMS(1,2+(JFLD-1)*3)
!            ZPRE9 (JROF,JLEV) =  ZPRES9(JROF,JLEV,IBL)
!         ENDDO
!      ENDDO
!      CALL GPPWC(NPROMA,IST,IEND,NFLEVG,ZFIELD2DT1,ZFIELD3DT1,ZPRE9)
!      DO JROF=IST,IEND
!         ZFIELD1D(JROF,JFLD,IBL)=ZFIELD2DT1(JROF,NFLEVG)
!      ENDDO
!   ENDDO
!ENDDO

! norms are broadcast if PNORMS=ZNORMS specified
!CALL GPNORM1(ZFIELD1D,12,.FALSE.,PNORMS=ZNORMS)

!WRITE(NULOUT,'("FIXSPPTQ1CTRL",3e16.7)') ZNORMS(1,1),ZNORMS(2,1),ZNORMS(3,1)
!WRITE(NULOUT,'("FIXSPPTT1CTRL",3e16.7)') ZNORMS(1,2),ZNORMS(2,2),ZNORMS(3,2)
!WRITE(NULOUT,'("FIXSPPTU1CTRL",3e16.7)') ZNORMS(1,3),ZNORMS(2,3),ZNORMS(3,3)
!WRITE(NULOUT,'("FIXSPPTV1CTRL",3e16.7)') ZNORMS(1,4),ZNORMS(2,4),ZNORMS(3,4)

IF (LHOOK) CALL DR_HOOK('FIXSPPT',1,ZHOOK_HANDLE)
END SUBROUTINE FIXSPPT
