SUBROUTINE GP_SSTAQUA(PGEMU,PGELAM,PSP_SB,PSP_SG,PSP_RR,PSD_VF)

!**** *GP_SSTAQUA* - ECMWF PHYSICS

!     Purpose.
!     --------
!           Aquaplanet SST distributions

!**   Interface.
!     ----------
!        *CALL* *GP_SSTAQUA*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by EC_PHYS

!     Reference.
!     ----------

!     Author.
!     -------
!        Nils Wedi *ECMWF*

!     Modifications.
!     --------------
!     -----------------------------------------------------------------------------

USE PARKIND1  ,ONLY  : JPIM     ,JPRB
USE YOMHOOK   ,ONLY  : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY  : NPROMA
USE YOMGEM   , ONLY  : NGPTOT
USE YOMDYNCORE, ONLY : MSSTSCHEME, RLAMBDA0, RLAMBDAD, RPHID, RCHI
USE SURFACE_FIELDS_MIX  , ONLY : YSP_SB,  YSP_SG,  YSP_RR, YSD_VF

IMPLICIT NONE

REAL(KIND=JPRB),INTENT(IN)    :: PGEMU(NGPTOT)
REAL(KIND=JPRB),INTENT(IN)    :: PGELAM(NGPTOT)
REAL(KIND=JPRB),INTENT(INOUT) :: PSP_SB(:,:,:,:)
REAL(KIND=JPRB),INTENT(INOUT) :: PSP_SG(:,:,:)
REAL(KIND=JPRB),INTENT(INOUT) :: PSP_RR(:,:,:)
REAL(KIND=JPRB),INTENT(INOUT) :: PSD_VF(:,:,:)
INTEGER(KIND=JPIM) :: JROF, IST, IEND, JKGLO, IBL, IOFF
REAL(KIND=JPRB) :: ZPI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GP_SSTAQUA',0,ZHOOK_HANDLE)

ZPI=ASIN(1.0_JPRB)*2.0_JPRB

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JKGLO,IST,IEND,IBL,IOFF,JROF)
DO JKGLO=1,NGPTOT,NPROMA
  IST=1
  IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBL=(JKGLO-1)/NPROMA+1
  IOFF=JKGLO  

  DO JROF=IST,IEND
    IF( MSSTSCHEME == 1 ) THEN
      ! Control
      IF(ABS(PGEMU(IOFF+JROF-1)) >  SIN(ZPI/3._JPRB)) THEN
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ELSE
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*&
         &(1._JPRB-SIN(ASIN(PGEMU(IOFF+JROF-1))*3._JPRB/2._JPRB)**2)+273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ENDIF
    ELSEIF( MSSTSCHEME == 9 ) THEN
      ! Constant SST reference 30 degrees C
      PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=303.15_JPRB
      PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
    ELSEIF( MSSTSCHEME == 2 ) THEN
      ! Peaked
      IF(ABS(PGEMU(IOFF+JROF-1)) >  SIN(ZPI/3._JPRB)) THEN
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ELSE
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*&
         &(1._JPRB-ABS(ASIN(PGEMU(IOFF+JROF-1)))*3._JPRB/ZPI)+273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ENDIF
    ELSEIF( MSSTSCHEME == 3 ) THEN
      ! Flat
      IF(ABS(PGEMU(IOFF+JROF-1)) >  SIN(ZPI/3._JPRB)) THEN
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ELSE
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*&
         &(1._JPRB-SIN(ASIN(PGEMU(IOFF+JROF-1))*3._JPRB/2._JPRB)**4)+273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ENDIF
    ELSEIF( MSSTSCHEME == 4 ) THEN
      ! Qobs
      IF(ABS(PGEMU(IOFF+JROF-1)) >  SIN(ZPI/3._JPRB)) THEN
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ELSE
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*0.5_JPRB*&
         &((1._JPRB-SIN(ASIN(PGEMU(IOFF+JROF-1))*3._JPRB/2._JPRB)**2)+&
         &(1._JPRB-SIN(ASIN(PGEMU(IOFF+JROF-1))*3._JPRB/2._JPRB)**4))+273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ENDIF
    ELSEIF( MSSTSCHEME == 5 ) THEN
      ! Control 5N
      IF(ABS(PGEMU(IOFF+JROF-1)) >  SIN(ZPI/3._JPRB)) THEN
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=273.15_JPRB
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ELSE
        IF( ASIN(PGEMU(IOFF+JROF-1)) >  ZPI/36._JPRB ) THEN
          PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*&
           &(1._JPRB-SIN((ASIN(PGEMU(IOFF+JROF-1))-ZPI/36._JPRB)*&
           &90._JPRB/55._JPRB)**2)+273.15_JPRB
        ELSE
          PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*&
           &(1._JPRB-SIN((ASIN(PGEMU(IOFF+JROF-1))-ZPI/36._JPRB)*&
           &90._JPRB/65._JPRB)**2)+273.15_JPRB
        ENDIF
        PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
        PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      ENDIF
    ELSE
      ! LAT/LON ANOMALIES ADDED TO THE CONTROL
      RLAMBDA0 = 0._JPRB
      RLAMBDAD = 30._JPRB/180._JPRB*ZPI
      RPHID    = 15._JPRB/180._JPRB*ZPI
      ! Control
      IF(ABS(PGEMU(IOFF+JROF-1)) >  SIN(ZPI/3._JPRB)) THEN
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=273.15_JPRB
      ELSE
        PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=27._JPRB*&
         &(1._JPRB-SIN(ASIN(PGEMU(IOFF+JROF-1))*3._JPRB/2._JPRB)**2)+273.15_JPRB
      ENDIF
      
      IF( MSSTSCHEME == 6 ) THEN
        ! 1KEQ
        RCHI=1.0_JPRB
        IF(((PGELAM(IOFF+JROF-1) > (RLAMBDA0 - RLAMBDAD)).AND.&
         &(PGELAM(IOFF+JROF-1) < (RLAMBDA0 + RLAMBDAD))).AND.&
         &(ABS(PGEMU(IOFF+JROF-1)) <= SIN(RPHID))) THEN
          PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)+&
           &RCHI*COS(0.5_JPRB*ZPI*(PGELAM(IOFF+JROF-1)-RLAMBDA0)/RLAMBDAD)**2*&
           &COS(0.5_JPRB*ZPI*(ASIN(PGEMU(IOFF+JROF-1))/RPHID))**2
        ENDIF
      ELSEIF( MSSTSCHEME == 7 ) THEN
        ! 3KEQ
        RCHI=3.0_JPRB
        IF(((PGELAM(IOFF+JROF-1) > (RLAMBDA0 - RLAMBDAD)).AND.&
         &(PGELAM(IOFF+JROF-1) < (RLAMBDA0 + RLAMBDAD))).AND.&
         &(ABS(PGEMU(IOFF+JROF-1)) <= SIN(RPHID))) THEN
          PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)+&
           &RCHI*COS(0.5_JPRB*ZPI*(PGELAM(IOFF+JROF-1)-RLAMBDA0)/RLAMBDAD)**2*&
           &COS(0.5_JPRB*ZPI*(ASIN(PGEMU(IOFF+JROF-1))/RPHID))**2
        ENDIF
      ELSEIF( MSSTSCHEME == 8 ) THEN
        ! 3KW1
        RCHI=3.0_JPRB
        IF( ABS(PGEMU(IOFF+JROF-1)) <= SIN(RPHID) ) THEN
          PSD_VF(JROF,YSD_VF%YSST%MP,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)+&
           &RCHI*COS(PGELAM(IOFF+JROF-1)-RLAMBDA0)*&
           &COS(0.5_JPRB*ZPI*(ASIN(PGEMU(IOFF+JROF-1))/RPHID))**2
        ENDIF
      ENDIF
      PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,1,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,2,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,3,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SB(JROF,4,YSP_SB%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_RR(JROF,YSP_RR%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
      PSP_SG(JROF,YSP_SG%YT%MP0,IBL)=PSD_VF(JROF,YSD_VF%YSST%MP,IBL)
    ENDIF
    PSD_VF(JROF,YSD_VF%YCI%MP,IBL)=0.0_JPRB
    PSP_SB(JROF,1,YSP_SB%YQ%MP0,IBL)=0.0_JPRB
    PSP_SB(JROF,2,YSP_SB%YQ%MP0,IBL)=0.0_JPRB
    PSP_SB(JROF,3,YSP_SB%YQ%MP0,IBL)=0.0_JPRB
    PSP_SB(JROF,4,YSP_SB%YQ%MP0,IBL)=0.0_JPRB
    PSP_SG(JROF,YSP_SG%YF%MP0,IBL)=0.0_JPRB
    PSP_SG(JROF,YSP_SG%YR%MP0,IBL)=100._JPRB
    PSP_SG(JROF,YSP_SG%YA%MP0,IBL)=0.85_JPRB
    PSP_RR(JROF,YSP_RR%YW%MP0,IBL)=0.0_JPRB
    PSP_SB(JROF,1,YSP_SB%YTL%MP0,IBL)=271.46_JPRB
    PSP_SB(JROF,2,YSP_SB%YTL%MP0,IBL)=271.46_JPRB
    PSP_SB(JROF,3,YSP_SB%YTL%MP0,IBL)=271.46_JPRB
    PSP_SB(JROF,4,YSP_SB%YTL%MP0,IBL)=271.46_JPRB
    
    PSD_VF(JROF,YSD_VF%YSOTY%MP,IBL)=1._JPRB
    PSD_VF(JROF,YSD_VF%YZ0F%MP,IBL) =1.E-04_JPRB
    PSD_VF(JROF,YSD_VF%YLZ0H%MP,IBL)=1.E-04_JPRB
  ENDDO
ENDDO
!$OMP END PARALLEL DO

IF (LHOOK) CALL DR_HOOK('GP_SSTAQUA',1,ZHOOK_HANDLE)
END SUBROUTINE GP_SSTAQUA

