 SUBROUTINE GRG_NOX2NO2 (KIDIA  , KFDIA  , KLON  , KLEV,  KTRAC, KGRG, KKTRAC, PRS1,  PRSF1, PQP, PTP, & 
&   PLP, PIP, PAP, PSZA, PALB, PLSM, PCEN, PGFL )

! PURPOSE
! -------
!  Diagnostic interconversion nox to no2 

! INTERFACE.
! ----------
! GRG_NOX2NO2 CALLED FROM *CALLPAR.

! INPUTS:
! -------
! PCEN    (KLON,KLEV,KTRAC)   : CONCENTRATION OF GEMS    (kg/kg) / OUTPUT
! PSZA    (KLON)              : COS OF SZA                
! PALB    (KLON)              : surface albedo!
! PLSM    (KLON)              : land-sea-mask
! PRS1    (KLON,0:KLEV)       : FULL-LEVEL PRESSURE           (Pa)
! PRSF1    (KLON,KLEV)        : HALF-LEVEL PRESSURE           (Pa)
! PQP     (KLON,KLEV)         :  SPECIFIC HUMIDITY            (K)
! PTP     (KLON,KLEV)         :  TEMPERATURE                  (K)
! PLP     (KLON,KLEV)         :  LCWC     
! PIP     (KLON,KLEV)         :  ICWC     
! PAP     (KLON,KLEV)         :  CLOD FRACTION  

! OUTPUTS
! -------
!! PGFL     (KLON,KLEV,YGFL%NDIM)   : CONCENTRATION OF NO2 in GFL ARRAY     (kg/kg)

!     AUTHOR.
!     -------
!        JOHANNES FLEMMING based on Antje Dethof & TM5 

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2006-12-15


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RG, RPI
!USE YOERDI   , ONLY : RRAE
USE YOMDIM   , ONLY : NPPM
USE YOM_YGFL , ONLY : YGFL, NGRG, NTRAC, YGRG, YTRAC 
USE YOMCOUPLO4, ONLY : LNOX2NO2
USE YOECLD   , ONLY : REPSEC
USE YOERDI   , ONLY : REPCLC

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1  ARGUMENTS
!             ---------

INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA , KFDIA , KLON , KLEV

REAL(KIND=JPRB),INTENT(IN)    :: PRSF1(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PRS1(KLON,0:KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PQP(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PTP(KLON,KLEV)  
REAL(KIND=JPRB),INTENT(IN)    :: PLP(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PIP(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PAP(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PSZA(KLON)          
REAL(KIND=JPRB),INTENT(IN)    :: PALB(KLON)          
REAL(KIND=JPRB),INTENT(IN)    :: PLSM(KLON)

INTEGER(KIND=JPIM),INTENT(IN) :: KTRAC
INTEGER(KIND=JPIM),INTENT(IN) :: KGRG(NGRG), KKTRAC(NTRAC)
REAL(KIND=JPRB),INTENT(IN)    :: PCEN(KLON,KLEV,KTRAC)
REAL(KIND=JPRB), INTENT(OUT)  :: PGFL(KLON,KLEV,YGFL%NDIM)
!*       0.2  Local Variables
!             ---------

REAL(KIND=JPRB)     :: ZSOL(KLON)
!REAL(KIND=JPRB)     :: ZSZA(KLON)
REAL(KIND=JPRB)     :: ZCOD(KLON)       ! Cloud Optical depth 

REAL(KIND=JPRB)     :: ZT(KLON,0:KLEV)  ! Temp
REAL(KIND=JPRB)     :: ZL(KLON,0:KLEV)  ! LCWC
REAL(KIND=JPRB)     :: ZI(KLON,0:KLEV)  ! ICWC
REAL(KIND=JPRB)     :: ZQ(KLON,0:KLEV)  ! q

REAL(KIND=JPRB)     :: ZJNO2(KLON,KLEV) ! NO2 photolysis
!REAL(KIND=JPRB)     :: ZJNO2_CS(KLON,KLEV) ! NO2 photolysis
REAL(KIND=JPRB)     :: ZTAUC(KLON,KLEV) ! Cloud optical depth per layer
REAL(KIND=JPRB)     :: ZCC(KLON,KLEV)   ! Cloud cover above 
REAL(KIND=JPRB)     :: ZNO2(KLON,KLEV)  ! diagnostic NO2
REAL(KIND=JPRB)     :: ZRGI,ZCRAE 
REAL(KIND=JPRB)     :: ZCOLO3(KLON,0:KLEV) 
REAL(KIND=JPRB)     :: ZRXP(KLON,0:KLEV,NPPM)
INTEGER(KIND=JPIM)  :: JL, JLEV, IO3, INOX, INO2, ITR, IGR1, IGR2, IGR3 , IGR4

!REAL(KIND=JPRB) , PARAMETER :: ZC1=1.7E-12_JPRB,ZC2=-1310.0_JPRB
! new constants from std_CY32R1_coupled_bufr2odb/ifs/pp_obs/gpnox.F90
REAL(KIND=JPRB) ,  PARAMETER :: ZC1=3.0E-12_JPRB,ZC2=-1500.0_JPRB   !From Sander et al. (2003)
REAL(KIND=JPRB) , PARAMETER :: ZC3=4.5E19_JPRB

!REAL(KIND=JPRB) :: ZC4, ZRHO
!REAL(KIND=JPRB) :: ZRATIO
!REAL(KIND=JPRB) :: ZZN,ZZK,ZO3mol,ZZB
REAL(KIND=JPRB) :: ZDELP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "grg_jno2_cloud.intfb.h"
!#include "grg_jno2.intfb.h"
#include "cod_op.intfb.h"
#include "abor1.intfb.h"
#include "gpnox.intfb.h"

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRG_NOX2NO2',0,ZHOOK_HANDLE)
! search pointer for O3, NOx, NO2

IF (LNOX2NO2) THEN

 IO3=-1
INOX=-1
INO2=-1
IGR1=-1
IGR2=-1
IGR3=-1
IGR4=-1

 DO ITR=1,NGRG
  IF (TRIM(YGRG(ITR)%CNAME)=='GO3') IO3=KGRG(ITR)
  IF (TRIM(YGRG(ITR)%CNAME)=='NOX') INOX=KGRG(ITR)
 ENDDO 
 
 DO ITR=1,NTRAC
  IF (TRIM(YTRAC(ITR)%CNAME)=='NO2') INO2=ITR
  IF (TRIM(YTRAC(ITR)%CNAME)=='GR1') IGR1=ITR
  IF (TRIM(YTRAC(ITR)%CNAME)=='GR2') IGR2=ITR
  IF (TRIM(YTRAC(ITR)%CNAME)=='GR3') IGR3=ITR
  IF (TRIM(YTRAC(ITR)%CNAME)=='GR4') IGR4=ITR
 ENDDO 

 IF (ANY( (/IO3,INOX,INO2/)  == -1) ) THEN
     CALL ABOR1('UNDEFINED NO2,NOX or GO3') 
 ENDIF


!*       1.   COMPUTE photolysis frequency for NO2

!*       1.1  Convert  COS(SZA) to solar elevation 

DO JL=KIDIA,KFDIA
  ZSOL(JL) = 90.0_JPRB -  ACOS(PSZA(JL)) / RPI* 180_JPRB
ENDDO

!*       1.2  Compute full level O3 TC 

ZRGI=1.0_JPRB/RG

DO JL=KIDIA,KFDIA
  ZCOLO3(JL,0)=0.0_JPRB
  ZT(JL,0)=PTP(JL,1)
  ZQ(JL,0)=PQP(JL,1)
  ZL(JL,0)=PLP(JL,1)
  ZI(JL,0)=PIP(JL,1)
ENDDO

DO JLEV=1,KLEV
  DO JL=KIDIA,KFDIA
    ZDELP=PRS1(JL,JLEV)-PRS1(JL,JLEV-1)
    ZCOLO3(JL,JLEV)=ZCOLO3(JL,JLEV-1)+MAX(0.0_JPRB,PCEN(JL,JLEV,IO3))*ZDELP*ZRGI
  ZT(JL,JLEV)=PTP(JL,JLEV)
  ZQ(JL,JLEV)=PQP(JL,JLEV)
  ZL(JL,JLEV)=PLP(JL,JLEV)
  ZI(JL,JLEV)=PIP(JL,JLEV)
  ENDDO
ENDDO

! 1.3 Calculate J(NO2)

! vom radintg.F90  correct for earth curvature
!ZCRAE=RRAE*(RRAE+2.0_JPRB)

!! SET UP SOLAR ZENITH ANGLE, CORRECTED FOR EARTH'S CURVATURE
!DO JL=KIDIA,KFDIA
!  IF (PSZA(JL) > 1.E-10_JPRB) THEN
!    ZSZA(JL)=RRAE/(SQRT(PSZA(JL)**2+ZCRAE)-PSZA(JL))
!  ELSE
!    ZSZA(JL)= RRAE/SQRT(ZCRAE)
!  ENDIF 
!ENDDO

DO JLEV=1,KLEV
  DO JL=KIDIA,KFDIA
    ZRXP(JL,JLEV,1)=PRS1(JL,JLEV)
    ZRXP(JL,JLEV,2)=PRSF1(JL,JLEV)
    ZRXP(JL,JLEV,3)=LOG(PRS1(JL,JLEV))
    ZRXP(JL,JLEV,4)=LOG(PRSF1(JL,JLEV))
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  ZRXP(JL,0,1)=PRS1(JL,0)
  ZRXP(JL,0,2)=0.1_JPRB
  ZRXP(JL,0,3)=LOG(0.1_JPRB)
  ZRXP(JL,0,4)=LOG(0.1_JPRB)
ENDDO


! calculate integrated cloud cover above level - adapted from cldpp.f90
ZCC=0.0_JPRB

DO JL=KIDIA,KFDIA
  ZCC(JL,1)=1.0_JPRB-MIN(MAX(PAP(JL,1),REPCLC),1.0_JPRB-REPCLC)
ENDDO

DO JLEV=2,KLEV
  DO JL=KIDIA,KFDIA
    IF(PAP(JL,JLEV-1) < 1.0_JPRB-REPSEC) THEN
      ZCC(JL,JLEV)=ZCC(JL,JLEV-1)* &
       & (1.0_JPRB-MAX(PAP(JL,JLEV),PAP(JL,JLEV-1)))&
       & /(1.0_JPRB-MIN(PAP(JL,JLEV-1),1.0_JPRB-REPSEC))
    ELSE
     ZCC(JL,JLEV) = 0.0_JPRB 
    ENDIF
  ENDDO
ENDDO

DO JLEV=1,KLEV
  DO JL=KIDIA,KFDIA
    ZCC(JL,JLEV)=MAX(0.0_JPRB,1.0_JPRB-ZCC(JL,JLEV))
  ENDDO
ENDDO

 CALL COD_OP(KLON,KFDIA,1, ZQ, ZT, PRS1, PRSF1 , PLSM, ZL, ZI, ZCOD ,ZTAUC)

!CALL PPNOX((KIDIA  , KFDIA  , KLON , KLEV, KLEV, 1 ,KLEV,&
! & PRES,PLNPRES,LDBELS,LDBLES,&
! & PRXP,PRXPD,PTF,PO3F,PNOXF,PSOL,PAL,PCC,PTAUC,PCOD,&
! & ZNO2)

! CALL GRG_JNO2(KLON,   KIDIA,KFDIA,KLEV, ZCOLO3(1,1), PTP(1,1),ZSOL, PALB,                    ZJNO2_CS, ZRXP)

 CALL GRG_JNO2_CLOUD( KLON, KIDIA, KFDIA, KLEV, ZCOLO3(1,1), PTP(1,1), ZSOL, PALB, ZCC, ZTAUC, ZCOD, ZJNO2, ZRXP)

 CALL GPNOX(KLON, KIDIA, KFDIA, KLEV, ZNO2, PTP(1,1), PCEN(1,1,IO3), PCEN(1,1,INOX), ZSOL, ZJNO2, ZRXP)

!! 1.4 Calculate K(O3), and NOX/NO2 ratio 

!  ZC4= RNAVO / (1000000.0_JPRB * 0.048_JPRB) 

DO JLEV=1,KLEV
  DO JL=KIDIA,KFDIA
!!! Determine ZRATIO=NO2/NOx
! IF (PSZA(JL) >= 0.01_JPRB) THEN 
!    ZZK=ZC1*EXP(ZC2/PTP(JL,JLEV))
!    ZZN=ZJNO2(JL,JLEV)/ZZK
!    ZRHO = PRSF1(JL,JLEV)/(RD*(PTP(JL,JLEV)))
!    ZO3mol = (PCEN(JL,JLEV,IO3) +0.5 * PSZA(JL) * 1.3E-7 )* ZRHO * ZC4
!    ZZB=1.0_JPRB + ZZN/ZO3mol
!    ZRATIO=1.0_JPRB/ZZB
!  ELSE
!    ZRATIO=1.0_JPRB
!  ENDIF
!  PGFL(JL,JLEV,YTRAC(INO2)%MP9_PH) = PCEN(JL,JLEV,INOX)*ZRATIO 
!  IF (IEEE_IS_NAN(ZNO2(JL,JLEV))) THEN
!   WRITE(*,*) ' IS NAN ZNO2 ' , JL, JLEV, PCEN(JL,JLEV,INOX), PCEN(JL,JLEV,IO3), &
!  &   ZJNO2(JL,JLEV), ZCC(JL, JLEV) ,  ZTAUC(JL, JLEV) 
!  ENDIF
!  IF (ZNO2(JL,JLEV) < 0.0_JPRB ) THEN
!   WRITE(*,*) ' NEGATIVE ZNO2 ' , JL, JLEV, PCEN(JL,JLEV,INOX), PCEN(JL,JLEV,IO3), &
!  &     ZJNO2(JL,JLEV), ZCC(JL, JLEV) ,  ZTAUC(JL, JLEV) 
!  ENDIF
!  IF (IEEE_IS_NAN(ZNO2(JL,JLEV))) THEN
!   WRITE(*,*) ' IS NAN ZJNO2 ' , JL, JLEV, PCEN(JL,JLEV,INOX), PCEN(JL,JLEV,IO3), &
!  &   ZJNO2(JL,JLEV), ZCC(JL, JLEV) ,  ZTAUC(JL, JLEV) 
!  ENDIF
  IF(INO2 > 0 ) PGFL(JL,JLEV,YTRAC(INO2)%MP9_PH) = ZNO2(JL,JLEV) 
  IF(IGR1 > 0 ) PGFL(JL,JLEV,YTRAC(IGR1)%MP9_PH) = ZJNO2(JL,JLEV) 
  IF(IGR2 > 0 ) PGFL(JL,JLEV,YTRAC(IGR2)%MP9_PH) = ZCC(JL,JLEV) 
  IF(IGR3 > 0 ) PGFL(JL,JLEV,YTRAC(IGR3)%MP9_PH) = ZTAUC(JL,JLEV) 
  IF(IGR4 > 0 ) PGFL(JL,JLEV,YTRAC(IGR4)%MP9_PH) = ZNO2(JL,JLEV)/PCEN(JL,JLEV,INOX)  
  ENDDO
ENDDO



ENDIF ! LNOX2NO2
IF (LHOOK) CALL DR_HOOK('GRG_NOX2NO2',1,ZHOOK_HANDLE )
END SUBROUTINE GRG_NOX2NO2
