SUBROUTINE IDISGPF(PGLOBAL,PLOCAL)

!**** *isndgpf * - Routine to distribute an input grid point field

!     Purpose.
!     --------
! distribute an input global grid point field to the current processor 

!**   Interface.
!     ----------
!        *CALL* *isndgpf *

!        Explicit arguments :
!        --------------------
!        PGLOBAL   : grid point field to distribute
!        PLOCAL   : if nproc is 1 then PLOCAL is the distributed field

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.   NONE.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group *ECMWF*

!     Modifications.
!     --------------
!        Original : 96-03-19
!    R. El Khatib : 99-12-28 remove poles rows
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MY_REGION_NS   ,MY_REGION_EW   ,NSTA     ,NONL     ,&
 & NPTRFRSTLAT,NFRSTLAT ,NLSTLAT  
USE YOMGEM   , ONLY : NLOENG   ,NGPTOT   ,NGPTOTG

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PGLOBAL(NGPTOTG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLOCAL(NGPTOT) 
INTEGER(KIND=JPIM) :: IA, IB, IGL1, IGL2, IGLOFF, ILAST, ILEN,&
 & ILOFF, IOFF, J, JGL, JLON  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('IDISGPF',0,ZHOOK_HANDLE)
IA=MY_REGION_NS
IB=MY_REGION_EW

IGLOFF=NPTRFRSTLAT(IA)
IGL1 = NFRSTLAT(IA)
IGL2 = NLSTLAT(IA)

IOFF=0
IF(IA > 1) THEN
  IF( NLSTLAT(IA-1) == NFRSTLAT(IA) )THEN
    ILAST=NLSTLAT(IA-1)-1
  ELSE
    ILAST=NLSTLAT(IA-1)
  ENDIF
  DO J=NFRSTLAT(1),ILAST
    IOFF=IOFF+NLOENG(J)
  ENDDO
ENDIF

ILEN = 0
ILOFF = 0
DO JGL=IGL1,IGL2
  DO JLON=1,NONL(IGLOFF+JGL-IGL1,IB)
    PLOCAL(ILEN+JLON)=PGLOBAL(IOFF+ILOFF+NSTA(IGLOFF+JGL-IGL1,IB)+JLON-1)
  ENDDO
  ILEN = ILEN + NONL(IGLOFF+JGL-IGL1,IB)
  ILOFF = ILOFF + NLOENG(JGL)
ENDDO

IF (LHOOK) CALL DR_HOOK('IDISGPF',1,ZHOOK_HANDLE)
END SUBROUTINE IDISGPF
