!OPTIONS XOPT(HSFUN)
SUBROUTINE LUBKSB(PARRAY1,KLON,KIDIA,KFDIA,KN,KNP,KINDX,PARRAY2) 
!-------------------------------------------------------------
! A.TOMPKINS       E.C.M.W.F.      2006
! 
! SOLVES THE SET OF N LINEAR EQUATIONS A  X = B. 
! HERE A IS INPUT, NOT AS THE MATRIX A BUT RATHER AS 
! ITS LU DECOMPOSITION, DETERMINED BY THE ROUTINE LUDCMP. 
! KINDX IS INPUT AS THE PERMUTATION VECTOR RETURNED BY LUDCMP. 
! PARRAY2(1:N) IS INPUT AS THE RIGHT-HAND SIDE VECTOR, 
! AND RETURNS WITH THE SOLUTION VECTOR X. PARRAY1, KN, KNP, 
! AND KINDX ARE NOT MODIFIED BY THIS ROUTINE AND CAN BE 
! LEFT IN PLACE FOR SUCCESSIVE CALLS WITH DI
!
! Reference: Adapted from Numerical Recipes in Fortran
!  
!-------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KN,KNP,KLON,KIDIA,KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KINDX(KLON,KN) 
REAL(KIND=JPRB),INTENT(IN)    :: PARRAY1(KLON,KNP,KNP)
REAL(KIND=JPRB),INTENT(INOUT) :: PARRAY2(KLON,KN) 

INTEGER(KIND=JPIM) :: I,II,J,IL,JL
REAL(KIND=JPRB) ::  ZSUM 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('LUBKSB',0,ZHOOK_HANDLE)

DO JL=KIDIA,KFDIA

II=0 

! WHEN II IS SET TO A POSITIVE VALUE, IT WILL BECOME 
! THE INDEX OF THE  RST NONVANISHING ELEMENT OF B. 
! WE NOW DO THE FORWARD SUBSTITUTION, EQUATION (2.3.6). 
! THE ONLY NEW WRINKLE IS TO UNSCRAMBLE THE PERMUTATION AS WE GO. 
DO I=1,KN 
  IL=KINDX(JL,I) 
  ZSUM=PARRAY2(JL,IL) 
  PARRAY2(JL,IL)=PARRAY2(JL,I) 
  IF (II /= 0)THEN 
    DO J=II,I-1 
      ZSUM=ZSUM-PARRAY1(JL,I,J)*PARRAY2(JL,J) 
    ENDDO
  ELSEIF (ZSUM /= 0.) THEN 
    II=I 
  ENDIF 
  PARRAY2(JL,I)=ZSUM 
ENDDO
DO I=KN,1,-1 ! NOW WE DO THE BACKSUBSTITUTION, EQUATION (2.3.7). 
  ZSUM=PARRAY2(JL,I) 
  DO J=I+1,KN 
    ZSUM=ZSUM-PARRAY1(JL,I,J)*PARRAY2(JL,J) 
  ENDDO
  PARRAY2(JL,I)=ZSUM/PARRAY1(JL,I,I) ! STORE A COMPONENT OF THE SOLUTION VECTOR X. 
ENDDO

ENDDO !JL 

IF (LHOOK) CALL DR_HOOK('LUBKSB',1,ZHOOK_HANDLE)
END SUBROUTINE LUBKSB
