SUBROUTINE m7_aero_prop &
  &( KIDIA, KFDIA, KLON, KLEV, & ! ECMWF indices
  &  pxtm1       , pxtte     , & !   "   tracers
  &  pdensaer  	 , prwet     , & 
  &  PTSPHY                    &
  &)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM  , JPRB
USE YOMHOOK   ,ONLY : LHOOK , DR_HOOK

!  USE mo_kind,         ONLY: dp
!  USE mo_time_control, ONLY: time_step_len
!  USE mo_tracer,       ONLY: AEROSOLNUMBER, AEROSOLMASS, ntrac, &
!                             trlist,   flag
!  USE mo_aero,         ONLY: cmode
!  USE mo_aero_m7,      ONLY: ram2cmr, nmod
!  USE mo_constants,    ONLY: api
!  USE mo_aero_mem,     ONLY: rwet, densaer

USE YOMCST    ,ONLY : RPI
USE YO_AERO_M7,ONLY : ntrac  , nmod , nmode, &
  &                   ram2cmr, rdens,        & 
  &                   cphas, ctrac


  IMPLICIT NONE

  !--- Parameters:

INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA, KFDIA, KLON, KLEV

REAL(KIND=JPRB),INTENT(IN)    :: pxtm1(klon ,klev,ntrac)

REAL(KIND=JPRB), INTENT(IN)   :: PTSPHY

REAL(KIND=JPRB),INTENT(INOUT) :: pxtte(klon ,klev,ntrac)

REAL(KIND=JPRB),INTENT(OUT)   :: pdensaer(klon ,klev,nmod), prwet(klon ,klev,nmod)

  !--- Streams and stream elements:

!  REAL(KIND=JPRB),      POINTER :: rwet_p(:,:,:)
!  REAL(KIND=JPRB),      POINTER :: densaer_p(:,:,:)

  !--- Local Variables:

INTEGER(KIND=JPIM) :: imod, jt, jmod

REAL(KIND=JPRB)    :: zeps

REAL(KIND=JPRB)    :: zmasssum(klon ,klev,nmod),  zdenssum(klon ,klev,nmod)

REAL(KIND=JPRB)    :: zxtp1(klon ,klev,ntrac)

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_AERO_PROP',0,ZHOOK_HANDLE)

  !--- 0) Initializations:

!!  zeps=EPSILON(1._JPRB)
!  zeps=100._JPRB*TINY(1._JPRB)
  zeps=1.E-32_JPRB 

  zmasssum(:,:,:)=0._JPRB
  zdenssum(:,:,:)=0._JPRB

  !--- Calculate tracer mixing ratios at t=t+1
  !    Convert sulfuric species from kg(S)/kg(air) to kg(tracer)/kg(air):

  DO jt=1, ntrac

!     SELECT CASE (trlist%ti(jt)%basename)
     IF (ctrac(jt) == 'SO2') THEN
!     CASE('SO2')
        zxtp1(kidia:kfdia,:,jt)=pxtm1(kidia:kfdia,:,jt)+pxtte(kidia:kfdia,:,jt)*PTSPHY * 2.0_JPRB   ! * M(SO2)/M(S)=64/32
     ELSEIF (ctrac(jt) == 'SO4') THEN
!     CASE('SO4')
        zxtp1(kidia:kfdia,:,jt)=pxtm1(kidia:kfdia,:,jt)+pxtte(kidia:kfdia,:,jt)*PTSPHY * 3.06_JPRB  ! * M(H2SO4)/M(S)
     ELSE
!     CASE DEFAULT
        zxtp1(kidia:kfdia,:,jt)=pxtm1(kidia:kfdia,:,jt)+pxtte(kidia:kfdia,:,jt)*PTSPHY
!     END SELECT
     ENDIF

  ENDDO

  !--- 1) Calculate accumulated properties:

  DO  jt=1, ntrac

!     IF (trlist%ti(jt)%nphase==AEROSOLMASS) THEN
     IF (cphas(jt) == 'AMS') THEN 

!        imod=trlist%ti(jt)%mode
        imod=nmode(jt)

        !--- Sum mass mixing ratio for each mode:

        zmasssum(kidia:kfdia,:,imod)=zmasssum(kidia:kfdia,:,imod)+zxtp1(kidia:kfdia,:,jt)

        !--- Average density of each compound weighted with the mass mixing ratio:

        zdenssum(kidia:kfdia,:,imod)=zdenssum(kidia:kfdia,:,imod)+zxtp1(kidia:kfdia,:,jt)*rdens(jt)

     ENDIF

  ENDDO

  !--- 2) Compute average density for each mode:

  DO jmod=1, nmod

!     densaer_p => densaer(jmod)%ptr

     WHERE(zmasssum(kidia:kfdia,:,jmod) > zeps) 

        pdensaer(kidia:kfdia,:,jmod)=zdenssum(kidia:kfdia,:,jmod)/zmasssum(kidia:kfdia,:,jmod)

     ELSEWHERE

        pdensaer(kidia:kfdia,:,jmod)=0._JPRB

     ENDWHERE

  ENDDO

  !--- 3) Computation of the number median diameter of the modes and store in stream:

  DO jt=1, ntrac
!     IF(trlist%ti(jt)%nphase==AEROSOLNUMBER) THEN
     IF (cphas(jt) == 'ANU') THEN

!        imod=trlist%ti(jt)%mode
        imod=nmode(jt)

!        rwet_p    => rwet(imod)%ptr
!        densaer_p => densaer(imod)%ptr

        WHERE (zxtp1(kidia:kfdia,:,jt) > 1.E-30_JPRB .AND. pdensaer(kidia:kfdia,:,imod) > 1.E-30_JPRB)

           prwet(kidia:kfdia,:,imod)=((3._JPRB/(4._JPRB*RPI))*(zmasssum(kidia:kfdia,:,imod)/zxtp1(kidia:kfdia,:,jt))/ &
             &                       pdensaer(kidia:kfdia,:,imod) )**(1._JPRB/3._JPRB) * ram2cmr(imod)

        ELSEWHERE

           prwet(kidia:kfdia,:,imod)=0._JPRB

        ENDWHERE

     ENDIF
  ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_AERO_PROP',1,ZHOOK_HANDLE)
END SUBROUTINE m7_aero_prop
