SUBROUTINE m7_coat (pm6rp_lon_lev, pcrtcst)
        
  ! Purpose:
  ! ---------
  ! *m7_coat* calculates the number of sulfate 
  !           molecules required to coat a particle
  !           with cLayerThickness of sulfate
  !
  ! Author:
  ! ---------
  ! Philip Stier, MPI                          2001
  !
  ! Interface:
  ! ---------
  ! *m7_coat* is called from *m7_concoag*
  !
  !  Modifications:
  !  --------------
  !  20091112: JJMorcrette, Adaptation to ECMWF IFS 
  !

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YO_AERO_M7,ONLY : nmod     ,cLayerThickness      ,cmr2ras
USE YOMCST    ,ONLY : RPI

IMPLICIT NONE

!- Inputs
!  ------
REAL(KIND=JPRB),INTENT(IN) :: pm6rp_lon_lev(nmod)    ! Ambient radii for current
!                                                    ! longitude and level [cm]
!- Outputs
!  -------
REAL(KIND=JPRB),INTENT(OUT):: pcrtcst(nmod)          ! Critical constant, i.e. number of
!                                                    ! sulfate to cover an average particle
!                                                    ! of the mode with a layer of the 
!                                                    ! thickness determined by cLayerThickness.

!- Local arrays
INTEGER(KIND=JPIM) :: jmod 

REAL(KIND=JPRB)    :: zras(nmod)             ! Radius of average surface 
!                                            ! for a single particle [cm]
REAL(KIND=JPRB)    :: zas(nmod)              ! Average surface 
!                                            ! for single particle [cm+2]
        
REAL(KIND=JPRB)    :: zsurf_molec  

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_COAT',0,ZHOOK_HANDLE)

zsurf_molec = 2.39E-15_JPRB ! Average cross-section 
!                           ! of a single H2SO4 molecule [cm+2]

!--- 1) Calculate the radii of average surface for modes 5-7:

zras(5) = pm6rp_lon_lev(5) * cmr2ras(5)
zras(6) = pm6rp_lon_lev(6) * cmr2ras(6)
zras(7) = pm6rp_lon_lev(7) * cmr2ras(7)

DO jmod=5, 7
           
!--- 2) Calculate the average surface of an particle for modes 5-7:

  zas(jmod)    = 4._JPRB * zras(jmod)**2 * RPI

!--- 3) Determine the number of sulfate molecules needed to form
!       a cLayerThickness thick layer of sulfate on the particles
!       in modes 5-7:
           
  pcrtcst(jmod) = (zas(jmod) / zsurf_molec) * cLayerThickness

ENDDO

!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_COAT',1,ZHOOK_HANDLE)
END SUBROUTINE m7_coat
