SUBROUTINE M7_EMI_DMS   &
  &( KIDIA, KFDIA, KLON, &
  &  PCI  , PDMS , PLSM, PTS , PWIND, &
  &  PFLXM7 & 
  &)

!*** * M7_EMI_DMS* - SOURCE TERMS FOR DMS

!**   INTERFACE.
!     ----------
!          *M7_EMI_DMS* IS CALLED FROM *  *.

!     AUTHOR.
!     -------
!       S. Kloster , MPI 20020802
!       P. Stier   , MPI 20020601

!     MODIFICATIONS.
!     --------------
!       FROM original M7 *piston_velocity* and *dms_emissions* 
!       JJMorcrette 20100106 Adaptation to ECMWF IFS

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YO_AERO_M7  ,ONLY : nm7aer, npist
USE YO_AERO_TRAC,ONLY : idt_dms
USE YOMCST    ,ONLY : RTT

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PCI(KLON), PDMS(KLON), PLSM(KLON), PTS(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWIND(KLON) 

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLXM7(KLON,nm7aer)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZVDMS(KLON), ZDMS_TER(KLON)
REAL(KIND=JPRB) :: ZCONS1, ZDMSCON, ZDMSEMISS &
  &             ,  ZKW   , ZSCHMIDT_DMS, ZSPEED, ZSST

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_EMI_DMS',0,ZHOOK_HANDLE)

zcons1=32.064E-11_JPRB / 3600._JPRB

!----- for development and initial testing
  zdmscon=0._JPRB
  zdms_ter(:)=0._JPRB
!-----

DO JL=KIDIA,KFDIA
  zvdms(JL)= 0._JPRB

  IF (PLSM(JL) == 0._JPRB) THEN
!    zdmscon=PDMS(JL)   

!- 1. Schmidt number
    zsst=PTS(JL)-RTT

!- 1.1  DMS Schmidt number after Andreae

!- limit SST to valid range as the polynomial gets negative for T > 35 C.
    zsst=MIN(35._JPRB, zsst)
  
    zschmidt_dms=3652.047271_JPRB &
      &  +zsst*(-246.99_JPRB + zsst*(8.536397_JPRB -0.124397_JPRB*zsst))
!
!- 2. piston velocity [cm/hr]
    zspeed=PWIND(JL)

    if (npist == 1) then     ! Liss & Merlivat, 1986
      if (zspeed > 3.6_JPRB .and. zspeed <= 13._JPRB) then
        zkw=2.85_JPRB*zspeed-9.65_JPRB
        zvdms(JL)=zkw*(zschmidt_dms/600._JPRB)**(-0.5_JPRB)
      elseif (zspeed <= 3.6_JPRB) then
        zkw=0.17_JPRB*zspeed
        zvdms(JL)=zkw*(zschmidt_dms/600._JPRB)**(-0.6667_JPRB)
      else
        zkw=5.9_JPRB*zspeed-49.3_JPRB
        zvdms(JL)=zkw*(zschmidt_dms/600._JPRB)**(-0.5_JPRB)
      endif

    elseif (npist == 2) then ! Wanninkhof, 1992
        zkw=0.31_JPRB*zspeed*zspeed
        zvdms(JL)=zkw*(zschmidt_dms/660._JPRB)**(-0.5_JPRB)

    elseif (npist == 3) then ! Nightingale, 2000
        zkw=zspeed * (0.333_JPRB + zspeed*0.222_JPRB)
        zvdms(JL)=zkw*(zschmidt_dms/660._JPRB)**(-0.5_JPRB)

    endif

    zdmsemiss=zdmscon*zvdms(JL)*zcons1
    PFLXM7(jl,idt_dms) = PFLXM7(jl,idt_dms) + zdmsemiss*(1._JPRB-PCI(JL))

  ENDIF

  PFLXM7(jl,idt_dms) = PFLXM7(jl,idt_dms) + zdms_ter(jl)

ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_EMI_DMS',1,ZHOOK_HANDLE)
END SUBROUTINE M7_EMI_DMS
