SUBROUTINE M7_EMI_DU &
  &( KIDIA, KFDIA, KLON, KSW  , KTILES , KAERWND, &
  &  PALB , PALBP, PDEP, PFRTI, PGUST, PHSDFOR, PLSM, PSNS, PWIND, PWS1, &
  &  PFLXM7 & 
  &)

!*** * M7_EMI_DU* - SOURCE TERMS FOR DUST A LA MORCRETTE ...

!**   INTERFACE.
!     ----------
!          *M7_EMI_DU* IS CALLED FROM *  *.

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*


!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 20100125

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YO_AERO_M7  ,ONLY : nm7aer, rfcon_ai, rfcon_ci, rmmr_ai, rmmr_ci
USE YO_AERO_TRAC,ONLY : idt_mduai, idt_mduci, idt_nai, idt_nci

USE YOEPHY    ,ONLY : LE4ALB
USE YOS_SOIL  ,ONLY : RWSAT, RWPWP

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON , KIDIA, KFDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSW 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES
INTEGER(KIND=JPIM),INTENT(IN)    :: KAERWND

REAL(KIND=JPRB)   ,INTENT(IN)    :: PALB(KLON) , PALBP(KLON,KSW), PDEP(KLON), PFRTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGUST(KLON), PHSDFOR(KLON)  , PLSM(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(KLON) , PWIND(KLON)    , PWS1(KLON)


REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLXM7(KLON,nm7aer)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JL

LOGICAL :: LLDUST(KLON)

REAL(KIND=JPRB) :: ZDEP(KLON), ZLTS_AI(KLON), ZLTS_CI(KLON), ZWNDDU(KLON)
REAL(KIND=JPRB) :: ZEMI_AI, ZEMI_CI, ZFSWET, ZMMR_AI, ZMMR_CI, ZSWETN, ZWND_AI, ZWND_CI
REAL(KIND=JPRB) :: ZRWPWP, ZRWSAT

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_EMI_DU',0,ZHOOK_HANDLE)

ZRWPWP=RWPWP
ZRWSAT=RWSAT
ZMMR_AI = rmmr_ai*1.E+06_JPRB
ZMMR_CI = rmmr_ci*1.E+06_JPRB

IF (KAERWND < 2) THEN
  ZWNDDU(KIDIA:KFDIA)=PWIND(KIDIA:KFDIA)
ELSE
  ZWNDDU(KIDIA:KFDIA)=PGUST(KIDIA:KFDIA)
ENDIF

LLDUST(:)=.FALSE.
DO JL=KIDIA,KFDIA
  IF (PLSM(JL) >= 0.99_JPRB .AND. PSNS(JL) == 0.0_JPRB .AND. PWS1(JL) <= 2._JPRB * ZRWPWP &
    & .AND. (LE4ALB .AND. (PALB(JL) < 0.5_JPRB .AND. PALBP(JL,1) >= 0.08_JPRB .AND. PALBP(JL,1) < 0.55_JPRB)) &
    & .AND. PFRTI(JL,8) >= 0.1_JPRB .AND. PHSDFOR(JL) <= 50._JPRB ) THEN
    LLDUST(JL)=.TRUE.
    ZDEP(JL)  = PDEP(JL)
  ENDIF
ENDDO


DO JL=KIDIA,KFDIA
  ZEMI_ai=0._JPRB
  ZEMI_ci=0._JPRB 
  IF (LLDUST(JL)) THEN
    ZSWETN = MIN(1._JPRB, MAX(0.001_JPRB, (PWS1(JL)-ZRWPWP)/(ZRWSAT-ZRWPWP) ) )
    ZFSWET = 1.2_JPRB+0.2_JPRB*LOG10(ZSWETN)
    ZLTS_ai(JL) = 1.5_JPRB * ZFSWET * (ZMMR_AI/0.15_JPRB)**0.25_JPRB
    ZLTS_ci(JL) = 1.5_JPRB * ZFSWET * (ZMMR_CI/0.15_JPRB)**0.25_JPRB

    IF (ZWNDDU(JL) > ZLTS_ai(JL)) THEN
      ZWND_ai = MAX(0._JPRB, (ZWNDDU(JL)-ZLTS_ai(JL)) *ZWNDDU(JL)*ZWNDDU(JL) )
      ZEMI_ai = ZDEP(JL) * ZWND_ai
    ENDIF
    IF (ZWNDDU(JL) > ZLTS_ai(JL)) THEN
      ZWND_ci = MAX(0._JPRB, (ZWNDDU(JL)-ZLTS_ci(JL)) *ZWNDDU(JL)*ZWNDDU(JL) )
      ZEMI_ci = ZDEP(JL) * ZWND_ci
    ENDIF
  ENDIF

!*       1.    EMITTED MASS-FLUX
!              -----------------

  PFLXM7(JL,idt_mduai)= ZEMI_ai
  PFLXM7(JL,idt_mduci)= ZEMI_ci


!*       2.    EMITTED NUMBER-FLUX
!              -------------------

! accumulation soluble
  PFLXM7(JL,idt_nai)=PFLXM7(JL,idt_nai)+PFLXM7(JL,idt_mduai) * rfcon_ai
! coarse soluble
  PFLXM7(JL,idt_nci)=PFLXM7(JL,idt_nci)+PFLXM7(JL,idt_mduci) * rfcon_ci

ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_EMI_DU',1,ZHOOK_HANDLE)
END SUBROUTINE M7_EMI_DU
