!OPTIONS XOPT(HSFUN)
SUBROUTINE RADACT &
 & ( KIDIA , KFDIA , KLON , KLEV,&
 & KRINT , KDLON , KDLONP2 , KSHIFT, KCF,&
 & PAPRS , PGELAM, PSIN  , PCLON, PSLON , PTH,&
 & PQ    , PQS   , PECPO3,&
 & PAER  , POZON ,&
 & PSU2TM, PSU3TM, PSU4TM,&
 & PBC2TM, PBC3TM, PBC4TM, PBC5TM,&
 & POM2TM, POM3TM, POM4TM, POM5TM,&
 & PDD3TM, PDD4TM, PDD6TM, PDD7TM,&
 & PSS3TM, PSS4TM &
 & )  

!***********************************************************************
! CAUTION: THIS ROUTINE WORKS ONLY ON A NON-ROTATED, UNSTRETCHED GRID
!***********************************************************************

!**** *RADACT  - COMPUTES DISTRIBUTION OF AEROSOLS AND OZONE

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *RADACT* FROM *RADINT*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    91/03/15

!     MODIFICATIONS.
!     --------------
!     J.-J. MORCRETTE  E.C.M.W.F.    93/03/15   OPERATIONAL CLIMATOLOGY
!     JJMorcrette 98-12-21 GISS volcanic aerosol climatology
!     JJMorcrette 99-09    monthly climatology of tropospheric aerosols
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     JJMorcrette 20080206 revised use of GISS history of volcanic aerosols
!     JJMorcrette 20080325 RH effect on new SO4 aerosol
!     T. van Noije Jan-2014 Included option for using aerosol mixing ratios from TM5
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY :         RPI, RG
USE YOEAERD  , ONLY : CVDAES   ,CVDAEL   ,CVDAEU   ,CVDAED   ,&
 & RCAEOPS  ,RCAEOPL  ,RCAEOPU  ,RCAEOPD  ,RCTRBGA  ,&
 & RCVOBGA  ,RCSTBGA  ,RCTRPT   ,RAESC    ,RAESS    ,&
 & RAELC    ,RAELS    ,RAEUC    ,RAEUS    ,RAEDC    ,&
 & RAEDS  
USE YOEOZOC  , ONLY : COZQC    ,COZQS    ,COZHC    ,COZHS
USE YOERAD   , ONLY : LHVOLCA  ,LNEWAER  ,LESO4HIS ,NVOLCVERT,&
 & LVOLCSPEC,RVOLCSPEC, LCMIP5, &
 & IPCC_NLON, IPCC_NLAT, IPCC_NLEV, IPCC_NMON,&
 & IPCC_SO4,  IPCC_LEV,  IPCC_LON,  IPCC_LAT, &
 & IPCC_SOA,  IPCC_OC1,  IPCC_OC2,  IPCC_BC1,  IPCC_BC2, &
 & IPCC_DST1, IPCC_DST2, IPCC_DST3, IPCC_DST4, IPCC_SSLT1, &
 & IPCC_SSLT2,IPCC_SSLT3,IPCC_SSLT4
USE YOERAD   , ONLY : LTM5AER, LCMIP6_PI_AEROSOLS
USE YOEAERC  , ONLY :       REPAER   ,&
 & RTAEBC  ,RTAEOR   ,RTAESD   ,RTAESS   ,RTAESU   , &
 & RTAEVO  ,RTAESO4, RVOLCDEC
USE YOEAEROP , ONLY : ALF_SU, ALF_OM, ALF_DD, ALF_SS, ALF_BC
USE YOEAERSNK, ONLY : RRHTAB
USE YOMLUN   , ONLY : NULOUT, NULERR

! KW testing
USE YOMMP , ONLY : MYPROC
USE YOMRIP, ONLY : NSTADD,NSTASS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDLONP2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KRINT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSHIFT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTH(KDLONP2,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KDLONP2,KLEV), PQS(KDLONP2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PECPO3(KDLONP2,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAER(KDLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POZON(KDLON,KLEV) 
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PSU2TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PSU3TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PSU4TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PBC2TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PBC3TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PBC4TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PBC5TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: POM2TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: POM3TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: POM4TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: POM5TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PDD3TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PDD4TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PDD6TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PDD7TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PSS3TM(KLON,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL   , INTENT(IN)    :: PSS4TM(KLON,KLEV)
!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!     -----------------------------------------------------------------

!*       0.2   LOCAL ARRAYS.
!              -------------

INTEGER(KIND=JPIM) :: IINLA1(KDLON), IINLA2(KDLON)
INTEGER(KIND=JPIM) :: IINLO1(KDLON), IINLO2(KDLON)

REAL(KIND=JPRB) :: ZAED  (KDLON), ZAEL  (KDLON), ZAES  (KDLON), ZAEU  (KDLON)
REAL(KIND=JPRB) :: ZAEQDN(KDLON), ZAEQDO(KDLON), ZAEQLN(KDLON), ZAEQLO(KDLON)
REAL(KIND=JPRB) :: ZAEQSN(KDLON), ZAEQSO(KDLON), ZAEQUN(KDLON), ZAEQUO(KDLON)

REAL(KIND=JPRB) :: ZAERBC(KDLON), ZAEROR(KDLON), ZAERSD(KDLON)
REAL(KIND=JPRB) :: ZAERSS(KDLON), ZAERSU(KDLON), ZAERVO(KDLON), ZAERVOBG(KDLON)

REAL(KIND=JPRB) :: ZAETRN(KDLON), ZAETRO(KDLON)

REAL(KIND=JPRB) :: ZALP(66)
REAL(KIND=JPRB) :: ZDPN(KDLON)  , ZDPO(KDLON)
REAL(KIND=JPRB) :: ZFAED(21)    , ZFAEL(21)    , ZFAES(21)    , ZFAEU(21)
REAL(KIND=JPRB) :: ZFOZQ(11)    , ZFOZH(11)
REAL(KIND=JPRB) :: ZGRTH(KDLON)
REAL(KIND=JPRB) :: ZLON(KDLON)  , ZLONR(96)    , ZNLO1(KDLON) , ZNLO2(KDLON)

REAL(KIND=JPRB) :: ZOZH (KDLON) , ZOZQ (KDLON)
REAL(KIND=JPRB) :: ZQOFN(KDLON) , ZQOFO(KDLON)
REAL(KIND=JPRB) :: ZSILAT(KDLON), ZSINR(73), ZSTRDEP(KDLON)
REAL(KIND=JPRB) :: ZAERSO4(KDLON),ZRH(KDLON)
INTEGER(KIND=JPIM) :: IRH(KDLON)

INTEGER(KIND=JPIM) :: IL, IMM, IMNC, IMNS, INLA, INLA1, INLA2, INLO1, INLO2, &
 & ITOTPT, JAER, JK, JL, JLR, JMM, JNN, JTAB, &
 & INLATR, INLONR, ITOT  

REAL(KIND=JPRB) :: ZAETR, ZAETRV, ZCOS1, ZCOS10, ZCOS2, ZCOS3, ZCOS4,&
 & ZCOS5, ZCOS6, ZCOS7, ZCOS8, ZCOS9, ZCPHN3, &
 & ZCPHO3, ZDPNMO, ZGRIDR, ZLATR, ZSDPN3, ZSDPO3, &
 & ZSIN, ZSIN1, ZSIN10, ZSIN2, ZSIN3, ZSIN4, &
 & ZSIN5, ZSIN6, ZSIN7, ZSIN8, ZSIN9  
REAL(KIND=JPRB) :: ZAERBC1, ZAERBC2, ZAEROR1, ZAEROR2, ZAERSD1, ZAERSD2, &
 & ZAERSS1, ZAERSS2, ZAERSU1, ZAERSU2  
REAL(KIND=JPRB) :: ZDLONR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-- used if LCMIP5=true
REAL(KIND=JPRB) :: ZSINR_IPCC(IPCC_NLAT)
REAL(KIND=JPRB), DIMENSION(KDLON,0:IPCC_NLEV) :: &
 &    IPCCLEV, IPCCSULFATE, IPCCSOATMP &
 & ,  IPCCOC1TMP,IPCCOC2TMP,IPCCBC1TMP,IPCCBC2TMP,IPCCDST1TMP &
 & ,  IPCCDST2TMP,IPCCDST3TMP,IPCCDST4TMP,IPCCSSLT1TMP,IPCCSSLT2TMP &
 & ,  IPCCSSLT3TMP,IPCCSSLT4TMP
REAL(KIND=JPRB), DIMENSION(KDLON,KLEV) :: &
 & IPCCSULF,IPCCSOA,IPCCOC1,IPCCOC2, &
 & IPCCBC1,IPCCBC2,IPCCDST1,IPCCDST2,IPCCDST3,IPCCDST4,IPCCSSLT1, &
 & IPCCSSLT2,IPCCSSLT3,IPCCSSLT4
REAL(KIND=JPRB) :: ZAER1, ZAER2, ZAERLEV1, ZAERLEV2
REAL(KIND=JPRB) :: PAERTAU(KDLON,KLEV,6),AEROMASS(KDLON,12)
REAL(KIND=JPRB) :: ZRRR(KDLON),PRE(KDLON),ZAERDP(KDLON)
INTEGER(KIND=JPIM) :: KWAVL=9 ! (corresponds to 555 nm, see su_aerop.F90)
INTEGER(KIND=JPIM) :: ILEV

! KW testing
INTEGER :: I,IUNIT
CHARACTER (LEN=24) :: FNAME
LOGICAL :: IS_OPEN

#include "abor1.intfb.h"
#include "legtri.intfb.h"

!     ------------------------------------------------------------------

!*         1.     "NEW AEROSOL DISTRIBUTION" PARAMETERS COMPUTATIONS
!                 --------------------------------------------------

!*         1.1    VOLCANIC AEROSOL DISTRIBUTION PARAMETERS
!                 ----------------------------------------

!                 GISS CLIMATOLOGY, APPLIED TO SULPHATE AEROSOLS
!                 ----------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADACT',0,ZHOOK_HANDLE)
IF (LHVOLCA) THEN
  INLATR=46
  ZGRIDR=180._JPRB/(INLATR-1)
  DO JLR=1,INLATR
    ZLATR=90._JPRB-(JLR-1)*ZGRIDR
    ZSINR(JLR)=SIN(ZLATR*RPI/180._JPRB)
  ENDDO

  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    INLA=0
    ZSILAT(IL)=-9999._JPRB
    ZSIN=PSIN(JL)
    DO JLR=1,INLATR-1
      IF (ZSIN <= ZSINR(JLR) .AND. ZSIN > ZSINR(JLR+1)) THEN
        INLA=JLR
        ZSILAT(IL)=(ZSIN-ZSINR(INLA))/(ZSINR(INLA+1)-ZSINR(INLA))
        ZAERVO(IL)=RTAEVO(INLA)+ZSILAT(IL)*(RTAEVO(INLA+1)-RTAEVO(INLA))
      ENDIF
    ENDDO
    IF (ZSIN <= ZSINR(INLATR-1) .AND. ZSIN >= ZSINR(INLATR))THEN
      INLA=INLATR
      ZSILAT(IL)=(ZSIN-ZSINR(INLA-1))/(ZSINR(INLA)-ZSINR(INLA-1))
      ZAERVO(IL)=RTAEVO(INLA-1)&
       & +ZSILAT(IL)*(RTAEVO(INLA)-RTAEVO(INLA-1))  
    ENDIF
    IF (INLA == 0) THEN
      CALL ABOR1(' Problem with lat. interpolation in radact!')
    ENDIF
    IF(LVOLCSPEC) THEN
      IF(ZSIN > 0.64_JPRB) THEN
        ZAERVO(IL)=RVOLCSPEC(1)
      ELSEIF(ZSIN > 0.34_JPRB) THEN
        ZSILAT(IL)=(ZSIN-0.34_JPRB)/(0.3_JPRB)
        ZAERVO(IL)=RVOLCSPEC(2)+ZSILAT(IL)*(RVOLCSPEC(1)-RVOLCSPEC(2))
      ELSEIF(ZSIN > -0.34_JPRB) THEN
        ZAERVO(IL)=RVOLCSPEC(2)
      ELSEIF(ZSIN > -0.64_JPRB) THEN
        ZSILAT(IL)=(-ZSIN-0.34_JPRB)/(0.3_JPRB)
        ZAERVO(IL)=RVOLCSPEC(2)+ZSILAT(IL)*(RVOLCSPEC(3)-RVOLCSPEC(2))
      ELSE
        ZAERVO(IL)=RVOLCSPEC(3)
      ENDIF
    ENDIF
    ZDPO(IL)=PAPRS(JL,1)
    ZAETRO(IL)=1.0_JPRB
    ZSTRDEP(IL)=0.0_JPRB
  ENDDO


!-- define how thick the stratosphere is in pressure units
  DO JK=1,KLEV
    IL=KSHIFT
    IF (KCF == 0) THEN
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1
        ZGRTH(IL)= PTH(JL,JK)/PTH(JL,JK+1)
      ENDDO
    ELSEIF (KCF == 1) THEN
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1
        ZGRTH(IL)= PTH(IL,JK)/PTH(IL,JK+1)
      ENDDO
    ENDIF
    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      ZDPN(IL)=PAPRS(JL,JK+1)
      IF (0.5_JPRB*(PAPRS(JL,JK+1)+PAPRS(JL,JK)) < 999._JPRB) THEN
        ZAETRN(IL)=1.0_JPRB
      ELSE
        ZAETRN(IL)=ZAETRO(IL)*(MIN(1.0_JPRB, ZGRTH(IL )))**RCTRPT
      ENDIF
      ZAETR=SQRT(ZAETRN(IL)*ZAETRO(IL))

      IF(NVOLCVERT==0) THEN
        ZAETRV=ZAETR
      ELSEIF(NVOLCVERT==1) THEN
! Use temperature profile, but no volcanic aerosol above 10hPa
        IF (0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 999._JPRB) THEN
          ZAETRV=0.0_JPRB
        ELSE
          ZAETRV=ZAETR
        ENDIF
      ELSEIF(NVOLCVERT==2) THEN
! Use prognostic ozone to define lower edge of aerosol distribution
        IF (0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 999._JPRB) THEN
          ZAETRV=0.0_JPRB
        ELSEIF(0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 7000._JPRB) THEN
          ZAETRV=1.0_JPRB
        ELSEIF(PAPRS(JL,JK) > 40000._JPRB) THEN
          ZAETRV=0.0_JPRB
        ELSE
          IF(PECPO3(JL,JK) > 0.4E-6_JPRB) THEN
            ZAETRV=1.0_JPRB
          ELSEIF(PECPO3(JL,JK) > 0.2E-6_JPRB) THEN
            ZAETRV=(PECPO3(JL,JK)-0.2E-6_JPRB)/0.2E-6_JPRB
          ELSE
            ZAETRV=0.0_JPRB
          ENDIF
        ENDIF
      ENDIF

      ZDPNMO=ZDPN(IL)-ZDPO(IL)
      ZSTRDEP(IL)=ZSTRDEP(IL)+ZAETRV*ZDPNMO
    ENDDO
    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL = IL+1
      ZDPO(IL)  =ZDPN(IL)
      ZAETRO(IL)=ZAETRN(IL)
    ENDDO
  ENDDO
!-- 
  IL = KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL = IL+1
    ZAERVO(IL)=ZAERVO(IL)/ZSTRDEP(IL)
  ENDDO

  IF (LCMIP5 .OR. LTM5AER .OR. LCMIP6_PI_AEROSOLS) THEN
    IL =KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL = IL+1
!-- SUAERV still divides VOBGA and STBGA with 19330, correct this here
      ZAERVOBG(IL)=RCVOBGA*19330.0_JPRB/ZSTRDEP(IL)
!-- avoid double-counting of background
!-- exponential decay (RVOLCDEC is set in suecaec)
      ZAERVO(IL)=MAX(0.0_JPRB,ZAERVO(IL)-ZAERVOBG(IL))*RVOLCDEC
    END DO
  ENDIF

!                 TANRE ET AL. CLIMATOLOGY 
!                 ------------------------

ELSE
  IL = KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL = IL+1
    ZAERVO(IL)=RCSTBGA
  ENDDO
ENDIF
ITOTPT=IL

!*         1.2    TROPOSPHERIC AEROSOL DISTRIBUTION PARAMETERS
!                 --------------------------------------------

TRANSIENT: IF (.NOT.LCMIP5 .AND. .NOT.LTM5AER .AND. .NOT.LCMIP6_PI_AEROSOLS) THEN
IF (LNEWAER) THEN
!  print *,'LNEWAER= ',LNEWAER
!  print *,' inputs        SINLAT            LONGITUDE'
!  DO JL=KIDIA,KFDIA,KRINT
!    print 9001,JL,PSIN(JL),PGELAM(JL)
  9001    format(1x,'RADACT ',1x,I5,1x,2E15.8)
!  ENDDO         

!-- latitude       
  INLATR=46
  ZGRIDR=180._JPRB/(INLATR-1)
  DO JLR=1,INLATR
    ZLATR=90._JPRB-(JLR-1)*ZGRIDR
    ZSINR(JLR)=SIN(ZLATR*RPI/180._JPRB)
  ENDDO
  INLONR=72
  ZDLONR=2._JPRB*RPI/INLONR
  DO JLR=1,INLONR
    ZLONR(JLR)=(JLR-1)*ZDLONR
  ENDDO

!  print *,'Reference grid for Tegen climatology'        
!  print 9121,(ZSINR(JLR),JLR=1,NLATR)
  9121     format(1x,'ZSINR ',8E15.7)
!  print 9122,(ZLONR(JLR),JLR=1,NLONR)
  9122     format(1x,'ZLONR ',8E15.7)
           
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT         
    IL=IL+1
    IINLA1(IL)=0
    IINLA2(IL)=0
    ZSILAT(IL)=-9999._JPRB
    ZSIN=PSIN(JL)
    DO JLR=1,INLATR-1
      IF (ZSIN <= ZSINR(JLR) .AND. ZSIN > ZSINR(JLR+1)) THEN
        INLA=JLR
        IINLA1(IL)=JLR
        IINLA2(IL)=JLR+1
        ZSILAT(IL)=(ZSIN-ZSINR(INLA))/(ZSINR(INLA+1)-ZSINR(INLA))
      ENDIF
    ENDDO
    IF (ZSIN <= ZSINR(INLATR-1) .AND. ZSIN >= ZSINR(INLATR))THEN
      INLA=INLATR
      IINLA1(IL)=INLATR-1
      IINLA2(IL)=INLATR
      ZSILAT(IL)=(ZSIN-ZSINR(INLA-1))/(ZSINR(INLA)-ZSINR(INLA-1))
    ENDIF    
    IF (INLA == 0) THEN 
      CALL ABOR1(' Problem with lat. interpolation in radact!')
    ENDIF
!    print 9123,JL,IL,PSIN(JL),INLA,ZSINR(INLA),ZSILAT(IL)
    9123     format(1x,'Interp.Latit.',2I4,F10.7,I4,2F10.7)          
  ENDDO  

!-- longitude
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IINLO1(IL)=0
    IINLO2(IL)=0
    ZLON(IL)=-9999.
    DO JLR=1,71
      IF (PGELAM(JL) < ZLONR(JLR+1) .AND. PGELAM(JL) >= ZLONR(JLR)) &
         & THEN  
        IINLO1(IL)=JLR
        IINLO2(IL)=JLR+1
        ZNLO1(IL)=ZLONR(JLR)
        ZNLO2(IL)=ZLONR(JLR+1)
      ENDIF
    ENDDO      
    IF (PGELAM(JL) >= ZLONR(72)) THEN
      IINLO1(IL)=72
      IINLO2(IL)= 1
      ZNLO1(IL)=ZLONR(72)
!      ZNLO2(IL)=ZLONR(72)+2.*RPI
      ZNLO2(IL)=ZLONR(72)+ZDLONR
    ENDIF
!    print 9124,JL,IL,PGELAM(JL),IINLO1(IL),IINLO2(IL),ZNLO1(IL),ZNLO2(IL)
    9124     format(1x,'Interp.Longit0.',2I4,F10.7,2I5,2F10.7)          
  ENDDO  
 
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT        
    IL=IL+1
    IF (IINLO1(IL) == 0 .OR. IINLO2(IL) == 0) THEN 
      CALL ABOR1(' Problem with long. interpolation in radact!')
    ENDIF
    ZLON(IL)=(PGELAM(JL)-ZNLO1(IL))/(ZNLO2(IL)-ZNLO1(IL))
    INLO1=IINLO1(IL)
    INLO2=IINLO2(IL)
    INLA1=IINLA1(IL)
    INLA2=IINLA2(IL)
    
    ZAERBC1=RTAEBC(INLO1,INLA1) &
     & +ZSILAT(IL)*(RTAEBC(INLO1,INLA2)-RTAEBC(INLO1,INLA1))    
    ZAERBC2=RTAEBC(INLO2,INLA1) &
     & +ZSILAT(IL)*(RTAEBC(INLO2,INLA2)-RTAEBC(INLO2,INLA1))    
    ZAERBC(IL)=ZAERBC1+ZLON(IL)*(ZAERBC2-ZAERBC1)
        
    ZAEROR1=RTAEOR(INLO1,INLA1) &
     & +ZSILAT(IL)*(RTAEOR(INLO1,INLA2)-RTAEOR(INLO1,INLA1))    
    ZAEROR2=RTAEOR(INLO2,INLA1) &
     & +ZSILAT(IL)*(RTAEOR(INLO2,INLA2)-RTAEOR(INLO2,INLA1))    
    ZAEROR(IL)=ZAEROR1+ZLON(IL)*(ZAEROR2-ZAEROR1)
        
    ZAERSD1=RTAESD(INLO1,INLA1) &
     & +ZSILAT(IL)*(RTAESD(INLO1,INLA2)-RTAESD(INLO1,INLA1))    
    ZAERSD2=RTAESD(INLO2,INLA1) &
     & +ZSILAT(IL)*(RTAESD(INLO2,INLA2)-RTAESD(INLO2,INLA1))    
    ZAERSD(IL)=ZAERSD1+ZLON(IL)*(ZAERSD2-ZAERSD1)
        
    ZAERSS1=RTAESS(INLO1,INLA1) &
     & +ZSILAT(IL)*(RTAESS(INLO1,INLA2)-RTAESS(INLO1,INLA1))    
    ZAERSS2=RTAESS(INLO2,INLA1) &
     & +ZSILAT(IL)*(RTAESS(INLO2,INLA2)-RTAESS(INLO2,INLA1))    
    ZAERSS(IL)=ZAERSS1+ZLON(IL)*(ZAERSS2-ZAERSS1)
        
    ZAERSU1=RTAESU(INLO1,INLA1) &
     & +ZSILAT(IL)*(RTAESU(INLO1,INLA2)-RTAESU(INLO1,INLA1))    
    ZAERSU2=RTAESU(INLO2,INLA1) &
     & +ZSILAT(IL)*(RTAESU(INLO2,INLA2)-RTAESU(INLO2,INLA1))    
    ZAERSU(IL)=ZAERSU1+ZLON(IL)*(ZAERSU2-ZAERSU1)
          
!    print 9125,JL,IL,PSIN(JL),PGELAM(JL),ZSILAT(IL) &
!     &      ,RTAESU(INLO1,INLA2),RTAESU(INLO1,INLA1),ZAERSU1 &
!     &      ,RTAESU(INLO2,INLA2),RTAESU(INLO2,INLA1),ZAERSU2 &
!     &                    ,INLA1,INLO1,INLO2,INLA2
9125     format(1x,'Interp.Longit1.',2I4,9F10.7,4I5)          
!    print 9126,JL,IL,PSIN(JL),PGELAM(JL),ZSILAT(IL),ZLON(IL) &
!     &      ,ZNLO1(IL),ZNLO2(IL),INLA1,INLO1,INLO2,INLA2
9126     format(1x,'Interp.Longit2.',2I4,6F10.7,4I5)          
!    print 9127,JL,IL,ZAERBC(IL),ZAEROR(IL),ZAERSD(IL),ZAERSS(IL) &
!     &      ,ZAERSU(IL)
9127     format(1x,'Interp.Longit3.',2I4,5F10.7)          
  ENDDO       
ENDIF


!*         1.3    TROPOSPHERIC SULPHATE AEROSOL DISTRIBUTION PARAMETERS
!                 -----------------------------------------------------

IF (LNEWAER .AND. LESO4HIS) THEN
!  print *,'LNEWAER= ',LNEWAER
!  print *,' inputs        SINLAT            LONGITUDE'
!  DO JL=KIDIA,KFDIA,KRINT
!    print 9001,JL,PSIN(JL),PGELAM(JL)
!  ENDDO         

!-- latitude       
  INLATR=73
  ZGRIDR=180._JPRB/(INLATR-1)
  DO JLR=1,INLATR
    ZLATR=90._JPRB-(JLR-1)*ZGRIDR
    ZSINR(JLR)=SIN(ZLATR*RPI/180._JPRB)
  ENDDO
  INLONR=96
  ZDLONR=2._JPRB*RPI/INLONR
  DO JLR=1,INLONR
    ZLONR(JLR)=(JLR-1)*ZDLONR
  ENDDO

!  print *,'Reference grid for O.Boucher climatology'        
!  print 9121,(ZSINR(JLR),JLR=1,NLATR)
!  print 9122,(ZLONR(JLR),JLR=1,NLONR)
           
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT         
    IL=IL+1
    IINLA1(IL)=0
    IINLA2(IL)=0
    ZSILAT(IL)=-9999._JPRB
    ZSIN=PSIN(JL)
    DO JLR=1,INLATR-1
      IF (ZSIN <= ZSINR(JLR) .AND. ZSIN > ZSINR(JLR+1)) THEN
        INLA=JLR
        IINLA1(IL)=JLR
        IINLA2(IL)=JLR+1
        ZSILAT(IL)=(ZSIN-ZSINR(INLA))/(ZSINR(INLA+1)-ZSINR(INLA))
      ENDIF
    ENDDO
    IF (ZSIN <= ZSINR(INLATR-1) .AND. ZSIN >= ZSINR(INLATR))THEN
      INLA=INLATR
      IINLA1(IL)=INLATR-1
      IINLA2(IL)=INLATR
      ZSILAT(IL)=(ZSIN-ZSINR(INLA-1))/(ZSINR(INLA)-ZSINR(INLA-1))
    ENDIF    
    IF (INLA == 0) THEN 
      CALL ABOR1(' Problem with lat. interpolation in radact!')
    ENDIF
!    print 9123,JL,IL,PSIN(JL),INLA,ZSINR(INLA),ZSILAT(IL) 
  ENDDO  

!-- longitude
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IINLO1(IL)=0
    IINLO2(IL)=0
    ZLON(IL)=-9999.
    DO JLR=1,95
      IF (PGELAM(JL) < ZLONR(JLR+1) .AND. PGELAM(JL) >= ZLONR(JLR)) &
         & THEN  
        IINLO1(IL)=JLR
        IINLO2(IL)=JLR+1
        ZNLO1(IL)=ZLONR(JLR)
        ZNLO2(IL)=ZLONR(JLR+1)
      ENDIF
    ENDDO      
    IF (PGELAM(JL) >= ZLONR(72)) THEN
      IINLO1(IL)=96
      IINLO2(IL)= 1
      ZNLO1(IL)=ZLONR(96)
!      ZNLO2(IL)=ZLONR(96)+2.*RPI
      ZNLO2(IL)=ZLONR(96)+ZDLONR
    ENDIF
!    print 9124,JL,IL,PGELAM(JL),IINLO1(IL),IINLO2(IL),ZNLO1(IL),ZNLO2(IL)      
  ENDDO  
 
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT        
    IL=IL+1
    IF (IINLO1(IL) == 0 .OR. IINLO2(IL) == 0) THEN 
      CALL ABOR1(' Problem with long. interpolation in radact!')
    ENDIF
    ZLON(IL)=(PGELAM(JL)-ZNLO1(IL))/(ZNLO2(IL)-ZNLO1(IL))
    INLO1=IINLO1(IL)
    INLO2=IINLO2(IL)
    INLA1=IINLA1(IL)
    INLA2=IINLA2(IL)
    
    ZAERSU1=RTAESO4(INLO1,INLA1) &
     & +ZSILAT(IL)*(RTAESO4(INLO1,INLA2)-RTAESO4(INLO1,INLA1))    
    ZAERSU2=RTAESO4(INLO2,INLA1) &
     & +ZSILAT(IL)*(RTAESO4(INLO2,INLA2)-RTAESO4(INLO2,INLA1))    
    ZAERSU(IL)=ZAERSU1+ZLON(IL)*(ZAERSU2-ZAERSU1)

!-- Finally transforming the vertical integrated amount into a total optical depth
! NB: climatol. amount in mg S m-2  
!     absorption coefficient in m2 g-1 and corresponds to 80% relative humidity

!    ZAERSU(IL)=ZAERSU(IL)*11.882_JPRB*1.E-03_JPRB
          
!    print 9125,JL,IL,PSIN(JL),PGELAM(JL),ZSILAT(IL) &
!     &      ,RTAESO4(INLO1,INLA2),RTAESO4(INLO1,INLA1),ZAERSU1 &
!     &      ,RTAESO4(INLO2,INLA2),RTAESO4(INLO2,INLA1),ZAERSU2 &
!     &                    ,INLA1,INLO1,INLO2,INLA2  
!    print 9126,JL,IL,PSIN(JL),PGELAM(JL),ZSILAT(IL),ZLON(IL) &
!     &      ,ZNLO1(IL),ZNLO2(IL),INLA1,INLO1,INLO2,INLA2
!    print 9127,JL,IL,ZAERSU(IL)
  ENDDO       
ENDIF

ELSE TRANSIENT ! either LCMIP5 or LTM5AER, or both
   
   CMIP5: IF (LCMIP5 .AND. (.NOT.LTM5AER) .AND. (.NOT.LCMIP6_PI_AEROSOLS)) THEN ! LCMIP5 only
      
  !-- latitudinal interpolation
  ZSINR_IPCC(:)=SIN(IPCC_LAT(:))
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IINLA1(IL)=0
    IINLA2(IL)=0
    ZSILAT(IL)=-9999._JPRB
    ZSIN=PSIN(JL)
    IF (ZSIN >= ZSINR_IPCC(IPCC_NLAT))THEN
      IINLA1(IL)=IPCC_NLAT
      IINLA2(IL)=IPCC_NLAT
      ZSILAT(IL)=0.0_JPRB
    ELSE IF (ZSIN <= ZSINR_IPCC(1))THEN
      IINLA1(IL)=1
      IINLA2(IL)=1
      ZSILAT(IL)=0.0_JPRB
    ELSE
      DO JLR=2,IPCC_NLAT
        IF (ZSIN < ZSINR_IPCC(JLR)) THEN
          IINLA1(IL)=JLR
          IINLA2(IL)=JLR-1
          ZSILAT(IL)=(ZSIN-ZSINR_IPCC(JLR))/(ZSINR_IPCC(JLR-1)-ZSINR_IPCC(JLR))
          EXIT
        ENDIF
      ENDDO
    ENDIF
    IF (IINLA1(IL) == 0 .OR. IINLA2(IL) == 0) THEN
      CALL ABOR1(' Problem with lat. interpolation in radact!')
    ENDIF
    !WRITE(NULOUT,*)'JL=',JL,' PSIN=',PSIN(JL),' ZSILAT=',ZSILAT(IL), &
    !     & ' ZSINR_IPCC=',ZSINR_IPCC(IINLA1(IL)),ZSINR_IPCC(IINLA2(IL))
  ENDDO

  !-- longitudinal interpolation
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IINLO1(IL)=0
    IINLO2(IL)=0
    ZLON(IL)=-9999.
    IF (PGELAM(JL) >= IPCC_LON(IPCC_NLON)) THEN
      IINLO1(IL)=IPCC_NLON
      IINLO2(IL)=1
      ZNLO1(IL)=IPCC_LON(IPCC_NLON)
      ZNLO2(IL)=2.*IPCC_LON(IPCC_NLON)-IPCC_LON(IPCC_NLON-1)
    ELSE
      DO JLR=1,IPCC_NLON-1
        IF (PGELAM(JL) < IPCC_LON(JLR+1) .AND. PGELAM(JL) >= IPCC_LON(JLR)) THEN
          IINLO1(IL)=JLR
          IINLO2(IL)=JLR+1
          ZNLO1(IL)=IPCC_LON(JLR)
          ZNLO2(IL)=IPCC_LON(JLR+1)
        ENDIF
      ENDDO
    ENDIF
    IF (IINLO1(IL) == 0 .OR. IINLO2(IL) == 0) THEN
      CALL ABOR1(' Problem with long. interpolation in radact! Line 1')
    ENDIF
    ZLON(IL)=(PGELAM(JL)-ZNLO1(IL))/(ZNLO2(IL)-ZNLO1(IL))
    !WRITE(NULOUT,*)'JL=',JL,' PGELAM=',PGELAM(JL), &
    !     & ' IPCC_LON=',IPCC_LON(IINLO1(IL)),IPCC_LON(IINLO2(IL))
  ENDDO

  !-- compute aerosol concentrations
  DO ILEV=1,IPCC_NLEV
    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      INLO1=IINLO1(IL)
      INLO2=IINLO2(IL)
      INLA1=IINLA1(IL)
      INLA2=IINLA2(IL)

      ZAER1=IPCC_SO4(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_SO4(ILEV,INLO1,INLA2)-IPCC_SO4(ILEV,INLO1,INLA1))
      ZAER2=IPCC_SO4(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_SO4(ILEV,INLO2,INLA2)-IPCC_SO4(ILEV,INLO2,INLA1))
      IPCCSULFATE(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_SOA(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_SOA(ILEV,INLO1,INLA2)-IPCC_SOA(ILEV,INLO1,INLA1))
      ZAER2=IPCC_SOA(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_SOA(ILEV,INLO2,INLA2)-IPCC_SOA(ILEV,INLO2,INLA1))
      IPCCSOATMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_OC1(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_OC1(ILEV,INLO1,INLA2)-IPCC_OC1(ILEV,INLO1,INLA1))
      ZAER2=IPCC_OC1(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_OC1(ILEV,INLO2,INLA2)-IPCC_OC1(ILEV,INLO2,INLA1))
      IPCCOC1TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_OC2(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_OC2(ILEV,INLO1,INLA2)-IPCC_OC2(ILEV,INLO1,INLA1))
      ZAER2=IPCC_OC2(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_OC2(ILEV,INLO2,INLA2)-IPCC_OC2(ILEV,INLO2,INLA1))
      IPCCOC2TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_BC1(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_BC1(ILEV,INLO1,INLA2)-IPCC_BC1(ILEV,INLO1,INLA1))
      ZAER2=IPCC_BC1(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_BC1(ILEV,INLO2,INLA2)-IPCC_BC1(ILEV,INLO2,INLA1))
      IPCCBC1TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_BC2(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_BC2(ILEV,INLO1,INLA2)-IPCC_BC2(ILEV,INLO1,INLA1))
      ZAER2=IPCC_BC2(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_BC2(ILEV,INLO2,INLA2)-IPCC_BC2(ILEV,INLO2,INLA1))
      IPCCBC2TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_DST1(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST1(ILEV,INLO1,INLA2)-IPCC_DST1(ILEV,INLO1,INLA1))
      ZAER2=IPCC_DST1(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST1(ILEV,INLO2,INLA2)-IPCC_DST1(ILEV,INLO2,INLA1))
      IPCCDST1TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_DST2(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST2(ILEV,INLO1,INLA2)-IPCC_DST2(ILEV,INLO1,INLA1))
      ZAER2=IPCC_DST2(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST2(ILEV,INLO2,INLA2)-IPCC_DST2(ILEV,INLO2,INLA1))
      IPCCDST2TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_DST3(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST3(ILEV,INLO1,INLA2)-IPCC_DST3(ILEV,INLO1,INLA1))
      ZAER2=IPCC_DST3(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST3(ILEV,INLO2,INLA2)-IPCC_DST3(ILEV,INLO2,INLA1))
      IPCCDST3TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_DST4(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST4(ILEV,INLO1,INLA2)-IPCC_DST4(ILEV,INLO1,INLA1))
      ZAER2=IPCC_DST4(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_DST4(ILEV,INLO2,INLA2)-IPCC_DST4(ILEV,INLO2,INLA1))
      IPCCDST4TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_SSLT1(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT1(ILEV,INLO1,INLA2)-IPCC_SSLT1(ILEV,INLO1,INLA1))
      ZAER2=IPCC_SSLT1(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT1(ILEV,INLO2,INLA2)-IPCC_SSLT1(ILEV,INLO2,INLA1))
      IPCCSSLT1TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_SSLT2(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT2(ILEV,INLO1,INLA2)-IPCC_SSLT2(ILEV,INLO1,INLA1))
      ZAER2=IPCC_SSLT2(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT2(ILEV,INLO2,INLA2)-IPCC_SSLT2(ILEV,INLO2,INLA1))
      IPCCSSLT2TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_SSLT3(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT3(ILEV,INLO1,INLA2)-IPCC_SSLT3(ILEV,INLO1,INLA1))
      ZAER2=IPCC_SSLT3(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT3(ILEV,INLO2,INLA2)-IPCC_SSLT3(ILEV,INLO2,INLA1))
      IPCCSSLT3TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAER1=IPCC_SSLT4(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT4(ILEV,INLO1,INLA2)-IPCC_SSLT4(ILEV,INLO1,INLA1))
      ZAER2=IPCC_SSLT4(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_SSLT4(ILEV,INLO2,INLA2)-IPCC_SSLT4(ILEV,INLO2,INLA1))
      IPCCSSLT4TMP(IL,ILEV)=ZAER1+ZLON(IL)*(ZAER2-ZAER1)

      ZAERLEV1=IPCC_LEV(ILEV,INLO1,INLA1) &
        & +ZSILAT(IL)*(IPCC_LEV(ILEV,INLO1,INLA2)-IPCC_LEV(ILEV,INLO1,INLA1))
      ZAERLEV2=IPCC_LEV(ILEV,INLO2,INLA1) &
        & +ZSILAT(IL)*(IPCC_LEV(ILEV,INLO2,INLA2)-IPCC_LEV(ILEV,INLO2,INLA1))
      IPCCLEV(IL,ILEV)=ZAERLEV1+ZLON(IL)*(ZAERLEV2-ZAERLEV1)
    ENDDO
  ENDDO

  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IPCCSULFATE(IL,0)=0.0_JPRB
    IPCCSOATMP(IL,0)=0.0_JPRB
    IPCCOC1TMP(IL,0)=0.0_JPRB
    IPCCOC2TMP(IL,0)=0.0_JPRB
    IPCCBC1TMP(IL,0)=0.0_JPRB
    IPCCBC2TMP(IL,0)=0.0_JPRB
    IPCCDST1TMP(IL,0)=0.0_JPRB
    IPCCDST2TMP(IL,0)=0.0_JPRB
    IPCCDST3TMP(IL,0)=0.0_JPRB
    IPCCDST4TMP(IL,0)=0.0_JPRB
    IPCCSSLT1TMP(IL,0)=0.0_JPRB
    IPCCSSLT2TMP(IL,0)=0.0_JPRB
    IPCCSSLT3TMP(IL,0)=0.0_JPRB
    IPCCSSLT4TMP(IL,0)=0.0_JPRB
    IPCCLEV(IL,0)=0.0_JPRB
  ENDDO

!    IL=KSHIFT
!    DO JL=KIDIA,KFDIA,KRINT
!      IL=IL+1
!      WRITE(NULOUT,*)'IL = ',IL
!      WRITE(NULOUT,*)'IPCCLEV(IL,:) = ',IPCCLEV(IL,:)
!    ENDDO   

! Some sanity checks on inverpolated IPCCLEV

  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IF (.NOT.All(IPCCLEV(IL,0:IPCC_NLEV-1) < IPCCLEV(IL,1:IPCC_NLEV))) THEN
      WRITE(NULOUT,*)'IPCCLEV is not monotonic ascending: '
      WRITE(NULOUT,'(a,5i4)')'IL = ' &
         & ,IL,IINLO1(IL),IINLO2(IL),IINLA1(IL),IINLA2(IL)
      WRITE(NULOUT,'(A16,5F16.3,/,(6F16.3))')'IPCCLEV(IL,:) = ',IPCCLEV(IL,:)
      CALL ABOR1('IPCCLEV is not monotonic ascending')
    ENDIF
 ENDDO
 
ENDIF CMIP5

!-- main loop over levels
  DO JK=1,KLEV

!-- mid-level variables (pressure, RH)
    IL=KSHIFT
    IF (KCF == 0) THEN
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1
        PRE(IL)=0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1))
        ZAERDP(IL)=(PAPRS(JL,JK+1)-PAPRS(JL,JK))/RG
        ZRH(IL) = PQ(JL,JK)/PQS(JL,JK)
      ENDDO
    ELSEIF (KCF == 1) THEN
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1
        PRE(IL)=0.5_JPRB*(PAPRS(IL,JK)+PAPRS(IL,JK+1))
        ZAERDP(IL)=(PAPRS(IL,JK+1)-PAPRS(IL,JK))/RG
        ZRH(IL) = PQ(IL,JK)/PQS(IL,JK)
      ENDDO
    ENDIF
!   ZRH(:)=0.0_JPRB ! for dry aerosols

   IF (LCMIP5 .AND. (.NOT.LTM5AER) .AND. (.NOT.LCMIP6_PI_AEROSOLS)) THEN ! LCMIP5 only

!-- vertical interpolation
    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      DO ILEV=0,IPCC_NLEV-1
        IF(PRE(IL)>=IPCCLEV(IL,ILEV) .AND. PRE(IL)<IPCCLEV(IL,ILEV+1)) THEN
          ZRRR(IL)=1.0_JPRB/(IPCCLEV(IL,ILEV)-IPCCLEV(IL,ILEV+1))
          IPCCSULF(IL,JK)=IPCCSULFATE(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCSULFATE(IL,ILEV)-IPCCSULFATE(IL,ILEV+1))
          IPCCSOA(IL,JK)=IPCCSOATMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCSOATMP(IL,ILEV)-IPCCSOATMP(IL,ILEV+1))
          IPCCOC1(IL,JK)=IPCCOC1TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCOC1TMP(IL,ILEV)-IPCCOC1TMP(IL,ILEV+1))
          IPCCOC2(IL,JK)=IPCCOC2TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCOC2TMP(IL,ILEV)-IPCCOC2TMP(IL,ILEV+1))
          IPCCBC1(IL,JK)=IPCCBC1TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCBC1TMP(IL,ILEV)-IPCCBC1TMP(IL,ILEV+1))
          IPCCBC2(IL,JK)=IPCCBC2TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCBC2TMP(IL,ILEV)-IPCCBC2TMP(IL,ILEV+1))
          IPCCDST1(IL,JK)=IPCCDST1TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCDST1TMP(IL,ILEV)-IPCCDST1TMP(IL,ILEV+1))
          IPCCDST2(IL,JK)=IPCCDST2TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCDST2TMP(IL,ILEV)-IPCCDST2TMP(IL,ILEV+1))
          IPCCDST3(IL,JK)=IPCCDST3TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCDST3TMP(IL,ILEV)-IPCCDST3TMP(IL,ILEV+1))
          IPCCDST4(IL,JK)=IPCCDST4TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCDST4TMP(IL,ILEV)-IPCCDST4TMP(IL,ILEV+1))
          IPCCSSLT1(IL,JK)=IPCCSSLT1TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCSSLT1TMP(IL,ILEV)-IPCCSSLT1TMP(IL,ILEV+1))
          IPCCSSLT2(IL,JK)=IPCCSSLT2TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCSSLT2TMP(IL,ILEV)-IPCCSSLT2TMP(IL,ILEV+1))
          IPCCSSLT3(IL,JK)=IPCCSSLT3TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCSSLT3TMP(IL,ILEV)-IPCCSSLT3TMP(IL,ILEV+1))
          IPCCSSLT4(IL,JK)=IPCCSSLT4TMP(IL,ILEV+1)+(PRE(IL)-IPCCLEV(IL,ILEV+1))*ZRRR(IL)*(IPCCSSLT4TMP(IL,ILEV)-IPCCSSLT4TMP(IL,ILEV+1))
        ELSE IF(PRE(IL) >= IPCCLEV(IL,IPCC_NLEV)) THEN
          IPCCSULF(IL,JK)=IPCCSULFATE(IL,IPCC_NLEV)
          IPCCSOA(IL,JK)=IPCCSOATMP(IL,IPCC_NLEV)
          IPCCOC1(IL,JK)=IPCCOC1TMP(IL,IPCC_NLEV)
          IPCCOC2(IL,JK)=IPCCOC2TMP(IL,IPCC_NLEV)
          IPCCBC1(IL,JK)=IPCCBC1TMP(IL,IPCC_NLEV)
          IPCCBC2(IL,JK)=IPCCBC2TMP(IL,IPCC_NLEV)
          IPCCDST1(IL,JK)=IPCCDST1TMP(IL,IPCC_NLEV)
          IPCCDST2(IL,JK)=IPCCDST2TMP(IL,IPCC_NLEV)
          IPCCDST3(IL,JK)=IPCCDST3TMP(IL,IPCC_NLEV)
          IPCCDST4(IL,JK)=IPCCDST4TMP(IL,IPCC_NLEV)
          IPCCSSLT1(IL,JK)=IPCCSSLT1TMP(IL,IPCC_NLEV)
          IPCCSSLT2(IL,JK)=IPCCSSLT2TMP(IL,IPCC_NLEV)
          IPCCSSLT3(IL,JK)=IPCCSSLT3TMP(IL,IPCC_NLEV)
          IPCCSSLT4(IL,JK)=IPCCSSLT4TMP(IL,IPCC_NLEV)
        ENDIF
      ENDDO
    ENDDO

!    IL=KSHIFT
!    DO JL=KIDIA,KFDIA,KRINT
!      IL=IL+1
!      WRITE(NULOUT,*)'IL = ',IL
!      WRITE(NULOUT,*)'ZRRR(IL,:) = ',ZRRR(IL,:)
!    ENDDO

!    IL=KSHIFT
!    DO JL=KIDIA,KFDIA,KRINT
!      IL=IL+1
!      WRITE(NULOUT,*)'IL = ',IL
!      WRITE(NULOUT,*)'IPCCSULF(IL,:) = ',IL,IPCCSULF(IL,:)
!    ENDDO

!-- End of interpolation of CMIP5 aerosol distributions

    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
! Tido Semmler, 16.03.2011: remapping of CAM bins into IFS
!                           bins according to Twan van Noije
!
! Dry radius boundaries (microns): 
! SS IFS: 0.03      - 0.5 -       5.0 -      20.0 
! SS CAM:      0.1  - 0.5 - 1.5 - 5.0 - 10.0
! DD IFS: 0.03      -    0.55 - 0.9 -                  20.0 
! DD CAM:      0.05 - 0.5           - 1.25 - 2.5 - 5.0
!
      ! hydrophylic
      AEROMASS(IL,1) =IPCCSSLT1(IL,JK)
      AEROMASS(IL,9) =IPCCSSLT2(IL,JK)+IPCCSSLT3(IL,JK)
      AEROMASS(IL,10)=IPCCSSLT4(IL,JK)
      AEROMASS(IL,2) =IPCCSULF(IL,JK)
      AEROMASS(IL,3) =IPCCBC2(IL,JK)
      AEROMASS(IL,4) =IPCCDST1(IL,JK)+0.04355_JPRB*IPCCDST2(IL,JK)
      AEROMASS(IL,11)=0.43550_JPRB*IPCCDST2(IL,JK)
      AEROMASS(IL,12)=0.52095_JPRB*IPCCDST2(IL,JK)+IPCCDST3(IL,JK)+IPCCDST4(IL,JK)
      AEROMASS(IL,5) =IPCCSOA(IL,JK)+1.4_JPRB*IPCCOC2(IL,JK)
      ! hydrophobic
      AEROMASS(IL,7) =IPCCBC1(IL,JK)
      AEROMASS(IL,8) =1.4_JPRB*IPCCOC1(IL,JK)
    ENDDO

   ELSE IF (LTM5AER .OR. LCMIP6_PI_AEROSOLS) THEN ! LTM5AER or LCMIP6_PI_AEROSOLS
! For LW aerosol radiative effects:
! Calculate AEROMASS based on TM5 aerosol mass mixing ratios. 
! This is done because the LW absorption coefficients RAER are specified for the IFS types 
! (with respect to the component-specific AOD contribution at 0.55 nm).
! This information is currently not available from TM5.
!
! LW effects are most important for coarse-mode particles:
! - stratospheric aerosols resulting from large volcanic eruptions (currently
!   added to the TM5 tropospheric fields);
! - tropospheric aerosols containing mineral dust or sea salt
!
! Dry radius boundaries (microns): 
! SS IFS: 0.03      - 0.5 - 5.0 - 20.0 
! SS TM5:      0.05 - 0.5 -            Inf
! DD IFS: 0.03      -    0.55 - 0.9 - 20.0     
! DD TM5:      0.05 - 0.5 -                Inf
! In TM5 the mode median radii are varying over time. 
! This would lead to varying conversion factors for mapping of SS and DD from
! TM5 to IFS.To avoid this, we make the following assumptions:
! - We include the mass of particles > 20.0 microns
! - We neglect the difference between the boundaries at 0.55 and 0.5 microns
! - According to Morcrette et al. (JGR, 2009) roughly 10, 20 and 70% of the
!   total mass of SS and DD is in the bins 1,2 and 3, respectively. We use this
!   to split to contribution from the coarse TM5 mode over bins 2 and 3
!
! Contributions from ammonium-nitrate and MSA are currently neglected.
!
    IL=KSHIFT
    IF (KCF == 0) THEN
    ! call to RADACT in RADINTG uses KCF=1 but KCF=0 would give the same here
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1 
        ! IL=JL since KSHIFT=KDIA-1 and KRINT=1 in call to RADACT in RADINTG
        !hydrophylic
        AEROMASS(IL,1) = PSS3TM(JL,JK)
        AEROMASS(IL,9) = 0.22222_JPRB*PSS4TM(JL,JK)
        AEROMASS(IL,10)= (1.0_JPRB-0.22222_JPRB)*PSS4TM(JL,JK)
        AEROMASS(IL,2) = PSU2TM(JL,JK)+ &
                       &  PSU3TM(JL,JK)+PSU4TM(JL,JK) 
        AEROMASS(IL,3) = PBC2TM(JL,JK)+PBC3TM(JL,JK)
        ! for desert dust also include insoluble modes here:
        AEROMASS(IL,4) = PDD3TM(JL,JK)+PDD6TM(JL,JK)
        AEROMASS(IL,11)= 0.22222_JPRB*(PDD4TM(JL,JK)+PDD7TM(JL,JK))
        AEROMASS(IL,12)= (1.0_JPRB-0.22222_JPRB)*(PDD4TM(JL,JK)+PDD7TM(JL,JK))
        AEROMASS(IL,5) = POM2TM(JL,JK)+POM3TM(JL,JK) 
        ! hydrophobic
        AEROMASS(IL,7) = PBC4TM(JL,JK)+PBC5TM(JL,JK)
        AEROMASS(IL,8) = POM4TM(JL,JK)+POM5TM(JL,JK)
      ENDDO
    ELSEIF (KCF == 1) THEN
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1
        !hydrophylic
        AEROMASS(IL,1) = PSS3TM(IL,JK)
        AEROMASS(IL,9) = 0.22222_JPRB*PSS4TM(IL,JK)
        AEROMASS(IL,10)= (1.0_JPRB-0.22222_JPRB)*PSS4TM(IL,JK)
        AEROMASS(IL,2) = PSU2TM(IL,JK)+ &
                       &  PSU3TM(IL,JK)+PSU4TM(IL,JK)
        AEROMASS(IL,3) = PBC2TM(IL,JK)+PBC3TM(IL,JK)
        ! for desert dust also include insoluble modes here:
        AEROMASS(IL,4) = PDD3TM(IL,JK)+PDD6TM(IL,JK)
        AEROMASS(IL,11)= 0.22222_JPRB*(PDD4TM(IL,JK)+PDD7TM(IL,JK))
        AEROMASS(IL,12)= (1.0_JPRB-0.22222_JPRB)*(PDD4TM(IL,JK)+PDD7TM(IL,JK))
        AEROMASS(IL,5) = POM2TM(IL,JK)+POM3TM(IL,JK)
        ! hydrophobic
        AEROMASS(IL,7) = PBC4TM(IL,JK)+PBC5TM(IL,JK)
        AEROMASS(IL,8) = POM4TM(IL,JK)+POM5TM(IL,JK)
      ENDDO
    ENDIF
   ENDIF ! TM5AER or CMIP6_PI_AEROSOLS

    !-- RH dependency of aerosol extinction
    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      DO JTAB=1,12
        IF (ZRH(IL)*100._JPRB > RRHTAB(JTAB)) THEN
          IRH(IL)=JTAB
        ENDIF
      ENDDO
    ENDDO
   
    !-- aerosol optical thickness
    !-- aerosol type 1:SS,   2:SU,   3:BC,   4:DD,   5:OC
    !   N.B.: extinction coefficients are in m2 g-1
    !   mapping of size bins for SS and DD (Twan van Noije)
    DO JAER=1,5
      IL=KSHIFT
      DO JL=KIDIA,KFDIA,KRINT
        IL=IL+1
        SELECT CASE (JAER)
        CASE (1)
          PAERTAU(JL,JK,JAER)=AEROMASS(IL,1)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_SS(IRH(IL),KWAVL,1)+ &
                         & AEROMASS(IL,9)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_SS(IRH(IL),KWAVL,2)+ &
                         & AEROMASS(IL,10)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_SS(IRH(IL),KWAVL,3)
        CASE (2)
          PAERTAU(JL,JK,JAER)=AEROMASS(IL,2)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_SU(IRH(IL),KWAVL)
        CASE (3)
          PAERTAU(JL,JK,JAER)=AEROMASS(IL,3)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_BC(KWAVL)
          ! hydrophobic contribution
          ! no dependency of ALF_BC on RH yet
          PAERTAU(JL,JK,JAER)=PAERTAU(JL,JK,JAER)+&
                         & AEROMASS(IL,7)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_BC(KWAVL)
        CASE (4)
          PAERTAU(JL,JK,JAER)=AEROMASS(IL,4)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_DD(1,KWAVL)+ &
                         & AEROMASS(IL,11)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_DD(2,KWAVL)+ &
                         & AEROMASS(IL,12)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_DD(3,KWAVL)
        CASE (5)
          PAERTAU(JL,JK,JAER)=AEROMASS(IL,5)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_OM(IRH(IL),KWAVL)
          ! hydrophobic contribution
          PAERTAU(JL,JK,JAER)=PAERTAU(JL,JK,JAER)+&
                         & AEROMASS(IL,8)*ZAERDP(IL)* &
                         & 1.E+03_JPRB*ALF_OM(1,KWAVL)
        END SELECT
      ENDDO ! JL
    ENDDO !JAER
  ENDDO ! JK
  
ENDIF TRANSIENT

!     ------------------------------------------------------------------   

!*       2.     OZONE
!               -----

ZSIN=PSIN(KIDIA)

!*       2.1     CALL TO LEGTRI.
!                ---------------
!***
CALL LEGTRI (ZSIN,6,66,ZALP)
!***

!*       2.2     LEGENDRE TRANSFORM FOR OZONE.
!                -----------------------------

DO JMM=1,11
  ZFOZQ(JMM)=0.0_JPRB
  ZFOZH(JMM)=0.0_JPRB
ENDDO
IMM=0
IMNC=0
IMNS=0
DO JMM=1,6
  IMM=IMM+1
  DO JNN=JMM,6
    IMNC=IMNC+1
    ZFOZQ(IMM)=ZFOZQ(IMM)+ZALP(IMNC)*COZQC(IMNC)
    ZFOZH(IMM)=ZFOZH(IMM)+ZALP(IMNC)*COZHC(IMNC)
  ENDDO
  IF(JMM /= 1) THEN
    IMM=IMM+1
    DO JNN=JMM,6
      IMNS=IMNS+1
      ZFOZQ(IMM)=ZFOZQ(IMM)+ZALP(IMNS+6)*COZQS(IMNS)
      ZFOZH(IMM)=ZFOZH(IMM)+ZALP(IMNS+6)*COZHS(IMNS)
    ENDDO
  ENDIF
ENDDO

!*       2.3     FOURIER TRANSFORM FOR OZONE.
!                ----------------------------

IL=KSHIFT
DO JL=KIDIA,KFDIA,KRINT
  IL=IL+1
  ZCOS1=PCLON(JL)
  ZSIN1=PSLON(JL)
  ZCOS2=ZCOS1*ZCOS1-ZSIN1*ZSIN1
  ZSIN2=ZSIN1*ZCOS1+ZCOS1*ZSIN1
  ZCOS3=ZCOS2*ZCOS1-ZSIN2*ZSIN1
  ZSIN3=ZSIN2*ZCOS1+ZCOS2*ZSIN1
  ZCOS4=ZCOS3*ZCOS1-ZSIN3*ZSIN1
  ZSIN4=ZSIN3*ZCOS1+ZCOS3*ZSIN1
  ZCOS5=ZCOS4*ZCOS1-ZSIN4*ZSIN1
  ZSIN5=ZSIN4*ZCOS1+ZCOS4*ZSIN1
  ZOZQ(IL)=&
   & ZFOZQ(1)+2.0_JPRB*(ZFOZQ(2)*ZCOS1+ZFOZQ(3)*ZSIN1+ZFOZQ(4)*ZCOS2 &
   & +ZFOZQ(5)*ZSIN2+ZFOZQ(6)*ZCOS3+ZFOZQ(7)*ZSIN3+ZFOZQ(8)&
   & *ZCOS4+ZFOZQ(9)*ZSIN4+ZFOZQ(10)*ZCOS5+ZFOZQ(11)*ZSIN5)  
  ZOZH(IL)=&
   & ZFOZH(1)+2.0_JPRB*(ZFOZH(2)*ZCOS1+ZFOZH(3)*ZSIN1+ZFOZH(4)*ZCOS2 &
   & +ZFOZH(5)*ZSIN2+ZFOZH(6)*ZCOS3+ZFOZH(7)*ZSIN3+ZFOZH(8)&
   & *ZCOS4+ZFOZH(9)*ZSIN4+ZFOZH(10)*ZCOS5+ZFOZH(11)*ZSIN5)  
  ZOZH(IL)=SQRT(ZOZH(IL))**3
ENDDO

!     ------------------------------------------------------------------

!       3.     AEROSOLS
!              --------
!***
!       3.1     CALL TO LEGTRI

!***
CALL LEGTRI (ZSIN,11,66,ZALP)
!***

!       3.2     LEGENDRE TRANSFORM FOR AEROSOLS
!               -------------------------------

DO JMM=1,21
  ZFAES(JMM) = 0.0_JPRB
  ZFAEL(JMM) = 0.0_JPRB
  ZFAEU(JMM) = 0.0_JPRB
  ZFAED(JMM) = 0.0_JPRB
ENDDO
IMM  = 0
IMNC = 0
IMNS = 0
DO JMM=1,11
  IMM  = IMM+1
  DO JNN=JMM,11
    IMNC = IMNC+1
    ZFAES(IMM) = ZFAES(IMM)+ZALP(IMNC)*RAESC(IMNC)
    ZFAEL(IMM) = ZFAEL(IMM)+ZALP(IMNC)*RAELC(IMNC)
    ZFAEU(IMM) = ZFAEU(IMM)+ZALP(IMNC)*RAEUC(IMNC)
    ZFAED(IMM) = ZFAED(IMM)+ZALP(IMNC)*RAEDC(IMNC)
  ENDDO
  IF(JMM /= 1) THEN
    IMM  = IMM+1
    DO JNN=JMM,11
      IMNS = IMNS+1
      ZFAES(IMM) = ZFAES(IMM)+ZALP(IMNS+11)*RAESS(IMNS)
      ZFAEL(IMM) = ZFAEL(IMM)+ZALP(IMNS+11)*RAELS(IMNS)
      ZFAEU(IMM) = ZFAEU(IMM)+ZALP(IMNS+11)*RAEUS(IMNS)
      ZFAED(IMM) = ZFAED(IMM)+ZALP(IMNS+11)*RAEDS(IMNS)
    ENDDO
  ENDIF
ENDDO

!       3.3     FOURIER TRANSFORM FOR AEROSOLS
!               ------------------------------

IL = KSHIFT
DO JL=KIDIA,KFDIA,KRINT
  IL = IL+1
  ZCOS1    = PCLON(JL)
  ZSIN1    = PSLON(JL)
  ZCOS2    = ZCOS1*ZCOS1-ZSIN1*ZSIN1
  ZSIN2    = ZSIN1*ZCOS1+ZCOS1*ZSIN1
  ZCOS3    = ZCOS2*ZCOS1-ZSIN2*ZSIN1
  ZSIN3    = ZSIN2*ZCOS1+ZCOS2*ZSIN1
  ZCOS4    = ZCOS3*ZCOS1-ZSIN3*ZSIN1
  ZSIN4    = ZSIN3*ZCOS1+ZCOS3*ZSIN1
  ZCOS5    = ZCOS4*ZCOS1-ZSIN4*ZSIN1
  ZSIN5    = ZSIN4*ZCOS1+ZCOS4*ZSIN1
  ZCOS6    = ZCOS5*ZCOS1-ZSIN5*ZSIN1
  ZSIN6    = ZSIN5*ZCOS1+ZCOS5*ZSIN1
  ZCOS7    = ZCOS6*ZCOS1-ZSIN6*ZSIN1
  ZSIN7    = ZSIN6*ZCOS1+ZCOS6*ZSIN1
  ZCOS8    = ZCOS7*ZCOS1-ZSIN7*ZSIN1
  ZSIN8    = ZSIN7*ZCOS1+ZCOS7*ZSIN1
  ZCOS9    = ZCOS8*ZCOS1-ZSIN8*ZSIN1
  ZSIN9    = ZSIN8*ZCOS1+ZCOS8*ZSIN1
  ZCOS10   = ZCOS9*ZCOS1-ZSIN9*ZSIN1
  ZSIN10   = ZSIN9*ZCOS1+ZCOS9*ZSIN1
  ZAES(IL) = ZFAES(1) + 2.0_JPRB*&
   & ( ZFAES(2)*ZCOS1  + ZFAES(3)*ZSIN1  + ZFAES(4)*ZCOS2 &
   & + ZFAES(5)*ZSIN2  + ZFAES(6)*ZCOS3  + ZFAES(7)*ZSIN3 &
   & + ZFAES(8)*ZCOS4  + ZFAES(9)*ZSIN4  + ZFAES(10)*ZCOS5 &
   & + ZFAES(11)*ZSIN5 + ZFAES(12)*ZCOS6 + ZFAES(13)*ZSIN6 &
   & + ZFAES(14)*ZCOS7 + ZFAES(15)*ZSIN7 + ZFAES(16)*ZCOS8 &
   & + ZFAES(17)*ZSIN8 + ZFAES(18)*ZCOS9 + ZFAES(19)*ZSIN9 &
   & + ZFAES(20)*ZCOS10+ ZFAES(21)*ZSIN10                 )  
  ZAEL(IL) = ZFAEL(1) + 2.0_JPRB*&
   & ( ZFAEL(2)*ZCOS1  + ZFAEL(3)*ZSIN1  + ZFAEL(4)*ZCOS2 &
   & + ZFAEL(5)*ZSIN2  + ZFAEL(6)*ZCOS3  + ZFAEL(7)*ZSIN3 &
   & + ZFAEL(8)*ZCOS4  + ZFAEL(9)*ZSIN4  + ZFAEL(10)*ZCOS5 &
   & + ZFAEL(11)*ZSIN5 + ZFAEL(12)*ZCOS6 + ZFAEL(13)*ZSIN6 &
   & + ZFAEL(14)*ZCOS7 + ZFAEL(15)*ZSIN7 + ZFAEL(16)*ZCOS8 &
   & + ZFAEL(17)*ZSIN8 + ZFAEL(18)*ZCOS9 + ZFAEL(19)*ZSIN9 &
   & + ZFAEL(20)*ZCOS10+ ZFAEL(21)*ZSIN10                 )  
  ZAEU(IL) = ZFAEU(1) + 2.0_JPRB*&
   & ( ZFAEU(2)*ZCOS1  + ZFAEU(3)*ZSIN1  + ZFAEU(4)*ZCOS2 &
   & + ZFAEU(5)*ZSIN2  + ZFAEU(6)*ZCOS3  + ZFAEU(7)*ZSIN3 &
   & + ZFAEU(8)*ZCOS4  + ZFAEU(9)*ZSIN4  + ZFAEU(10)*ZCOS5 &
   & + ZFAEU(11)*ZSIN5 + ZFAEU(12)*ZCOS6 + ZFAEU(13)*ZSIN6 &
   & + ZFAEU(14)*ZCOS7 + ZFAEU(15)*ZSIN7 + ZFAEU(16)*ZCOS8 &
   & + ZFAEU(17)*ZSIN8 + ZFAEU(18)*ZCOS9 + ZFAEU(19)*ZSIN9 &
   & + ZFAEU(20)*ZCOS10+ ZFAEU(21)*ZSIN10                 )  
  ZAED(IL) = ZFAED(1) + 2.0_JPRB*&
   & ( ZFAED(2)*ZCOS1  + ZFAED(3)*ZSIN1  + ZFAED(4)*ZCOS2 &
   & + ZFAED(5)*ZSIN2  + ZFAED(6)*ZCOS3  + ZFAED(7)*ZSIN3 &
   & + ZFAED(8)*ZCOS4  + ZFAED(9)*ZSIN4  + ZFAED(10)*ZCOS5 &
   & + ZFAED(11)*ZSIN5 + ZFAED(12)*ZCOS6 + ZFAED(13)*ZSIN6 &
   & + ZFAED(14)*ZCOS7 + ZFAED(15)*ZSIN7 + ZFAED(16)*ZCOS8 &
   & + ZFAED(17)*ZSIN8 + ZFAED(18)*ZCOS9 + ZFAED(19)*ZSIN9 &
   & + ZFAED(20)*ZCOS10+ ZFAED(21)*ZSIN10                 )  
ENDDO

!     ------------------------------------------------------------------

!*       4.      VERTICAL DISTRIBUTION
!*               ---------------------

IL=KSHIFT
DO JL=KIDIA,KFDIA,KRINT
  IL=IL+1
  ZDPO(IL)=PAPRS (JL,1)
  ZCPHO3=PAPRS (JL,1)**3
  ZSDPO3=SQRT  (ZCPHO3)
!  IF(LESO4HIS) THEN
!    ZAERSO4(IL)=0._JPRB
!  ELSE
    ZAERSO4(IL)=ZAERSU(IL)*CVDAEL(1)
!  ENDIF
  IF (LNEWAER) THEN
    ZAEQSO(IL)= ZAERSS(IL)*CVDAES(1)
    ZAEQLO(IL)= ZAEROR(IL)*CVDAEL(1) + ZAERSO4(IL)
    ZAEQUO(IL)= ZAERBC(IL)*CVDAEU(1)
    ZAEQDO(IL)= ZAERSD(IL)*CVDAED(1)
  ELSE
    ZAEQSO(IL)=RCAEOPS*ZAES(IL)*CVDAES(1)
    ZAEQLO(IL)=RCAEOPL*ZAEL(IL)*CVDAEL(1)
    ZAEQUO(IL)=RCAEOPU*ZAEU(IL)*CVDAEU(1)
    ZAEQDO(IL)=RCAEOPD*ZAED(IL)*CVDAED(1)
  ENDIF  
  ZAETRO(IL)=1.0_JPRB
  ZQOFO(IL)=ZOZQ(IL)*ZSDPO3 / (ZSDPO3 + ZOZH(IL))
ENDDO

DO JK=1,KLEV
  IL=KSHIFT
  IF (KCF == 0) THEN
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      ZGRTH(IL)= PTH(JL,JK)/PTH(JL,JK+1)
      ZRH(IL)  = PQS(JL,JK)/PQ (JL,JK)
    ENDDO
  ELSEIF (KCF == 1) THEN
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      ZGRTH(IL)= PTH(IL,JK)/PTH(IL,JK+1)
      ZRH(IL)  = PQS(IL,JK)/PQ (IL,JK)
    ENDDO
  ENDIF
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    DO JTAB=1,12
      IF (ZRH(IL)*100._JPRB > RRHTAB(JTAB)) THEN
        IRH(IL)=JTAB
      ENDIF
    ENDDO
  ENDDO

  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    IF(LESO4HIS) THEN
      ZAERSO4(IL)=ZAERSU(IL)*CVDAEL(JK+1)*1.E-03_JPRB*ALF_SU(IRH(IL),9)
    ELSE
      ZAERSO4(IL)=ZAERSU(IL)*CVDAEL(JK+1)
    ENDIF
  ENDDO

  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    ZDPN(IL)=PAPRS (JL,JK+1)
    ZCPHN3=PAPRS (JL,JK+1)**3
    ZSDPN3=SQRT  (ZCPHN3)
    IF (LNEWAER) THEN
      ZAEQSN(IL)= ZAERSS(IL)*CVDAES(JK+1)
      ZAEQLN(IL)= ZAEROR(IL)*CVDAEL(JK+1) + ZAERSO4(IL)
      ZAEQUN(IL)= ZAERBC(IL)*CVDAEU(JK+1)
      ZAEQDN(IL)= ZAERSD(IL)*CVDAED(JK+1)
    ELSE
      ZAEQSN(IL)=RCAEOPS*ZAES(IL)*CVDAES(JK+1)
      ZAEQLN(IL)=RCAEOPL*ZAEL(IL)*CVDAEL(JK+1)
      ZAEQUN(IL)=RCAEOPU*ZAEU(IL)*CVDAEU(JK+1)
      ZAEQDN(IL)=RCAEOPD*ZAED(IL)*CVDAED(JK+1)
    ENDIF  

    IF (0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 999._JPRB) THEN
      ZAETRN(IL)=1.0_JPRB
    ELSE
      ZAETRN(IL)=ZAETRO(IL)*(MIN(1.0_JPRB,     ZGRTH(IL)         ))**RCTRPT
    ENDIF
    ZAETR=SQRT  (ZAETRN(IL)*ZAETRO(IL))
    IF(NVOLCVERT==0) THEN
      ZAETRV=ZAETR
    ELSEIF(NVOLCVERT==1) THEN
! Use temperature profile, but no volcanic aerosol above 10hPa
      IF (0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 999._JPRB) THEN
        ZAETRV=0.0_JPRB
      ELSE
        ZAETRV=ZAETR
      ENDIF
    ELSEIF(NVOLCVERT==2) THEN
! Use prognostic ozone to define lower edge of aerosol distribution
      IF (0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 999._JPRB) THEN
        ZAETRV=0.0_JPRB
      ELSEIF(0.5_JPRB*(PAPRS(JL,JK)+PAPRS(JL,JK+1)) < 7000._JPRB) THEN
        ZAETRV=1.0_JPRB
      ELSEIF(PAPRS(JL,JK) > 40000._JPRB) THEN
        ZAETRV=0.0_JPRB
      ELSE
        IF(PECPO3(JL,JK) > 0.4E-6_JPRB) THEN
          ZAETRV=1.0_JPRB
        ELSEIF(PECPO3(JL,JK) > 0.2E-6_JPRB) THEN
          ZAETRV=(PECPO3(JL,JK)-0.2E-6_JPRB)/0.2E-6_JPRB
        ELSE
          ZAETRV=0.0_JPRB
        ENDIF
      ENDIF
    ENDIF

    ZQOFN(IL)=ZOZQ(IL)*ZSDPN3/(ZSDPN3+ZOZH(IL))
    ZDPNMO    =ZDPN(IL)-ZDPO(IL)
    
    IF (.NOT.LCMIP5 .AND. .NOT.LTM5AER .AND. .NOT.LCMIP6_PI_AEROSOLS) THEN
       
      PAER(IL,1,JK)=(1.0_JPRB-ZAETR)*(RCTRBGA*ZDPNMO+ ZAEQLN(IL)-ZAEQLO(IL))
      PAER(IL,2,JK)=(1.0_JPRB-ZAETR)*(ZAEQSN(IL)-ZAEQSO(IL))
      PAER(IL,3,JK)=(1.0_JPRB-ZAETR)*(ZAEQDN(IL)-ZAEQDO(IL))
      PAER(IL,4,JK)=(1.0_JPRB-ZAETR)*(ZAEQUN(IL)-ZAEQUO(IL))
! volcanic ash at fixed (low) background value
      PAER(IL,5,JK)=   ZAETRV  * RCVOBGA*ZDPNMO
! volcanic sulphate aerosol either fixed or varying according to GISS 
      PAER(IL,6,JK)=   ZAETRV  * ZAERVO(IL) * ZDPNMO
    ELSE
      PAER(IL,1,JK)=PAERTAU(JL,JK,2)+PAERTAU(JL,JK,5)
      PAER(IL,2,JK)=PAERTAU(JL,JK,1)
      PAER(IL,3,JK)=PAERTAU(JL,JK,4)
      PAER(IL,4,JK)=PAERTAU(JL,JK,3)
! stratospheric aerosols from GISS plus constant background (RCVOBGA)
      PAER(IL,5,JK)=   ZAETRV  * ZAERVOBG(IL)*ZDPNMO
      PAER(IL,6,JK)=   ZAETRV  * ZAERVO(IL)*ZDPNMO
    ENDIF
    POZON(IL,JK)=ZQOFN(IL)-ZQOFO(IL)
!**** **************************************************
!**** **************************************************
  ENDDO
  IL=KSHIFT
  DO JL=KIDIA,KFDIA,KRINT
    IL=IL+1
    ZDPO(IL)=ZDPN(IL)
    ZQOFO(IL)=ZQOFN(IL)

    ZAEQSO(IL)=ZAEQSN(IL)
    ZAEQLO(IL)=ZAEQLN(IL)
    ZAEQUO(IL)=ZAEQUN(IL)
    ZAEQDO(IL)=ZAEQDN(IL)
    ZAETRO(IL)=ZAETRN(IL)
  ENDDO
  
!-- diagnostics in case of problem  
  DO JAER=1,6
    IL=KSHIFT
    DO JL=KIDIA,KFDIA,KRINT
      IL=IL+1
      PAER(IL,JAER,JK)=MAX(PAER(IL,JAER,JK),REPAER)
    ENDDO
    itot=il
  ENDDO   
!--
ENDDO

!IL=KSHIFT+1
!WRITE(NULOUT,*) ' RADACT volc:',PSIN(KIDIA),ZSTRDEP(IL),(PAER(IL,6,jk),jk=35,60,5)

!IF (LNEWAER) THEN
!  print 9100,kshift+1,itot,ZSIN,REPAER
!9100   format(1x,'RADACT kshift/itot ',2I4,F10.6,E12.4)      
!  do jk=1,klev
!    do jl=kshift+1,itot,3
!      jend=min(jl+2,itot)
!      iprint=0
!      do jjl=jl,jend
!        do JAER=1,6
!          if (paer(jjl,JAER,jk).gt.1000.) then
!            iprint=1
!          endif
!        enddo
!      enddo    
!      if (iprint.eq.1) then
!        print 9102,JK,JL,((PAER(JIL,JAER,JK),JAER=1,6),JIL=JL,JEND)
9102    format(1x,'radact ',I2,I4,18E9.2)
!      endif
!    enddo
!  enddo
!ENDIF  
!IF (IPRINT.EQ.1) THEN
!  CALL ABOR1(' Problem with aerosols in RADACT')
!ENDIF  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADACT',1,ZHOOK_HANDLE)
END SUBROUTINE RADACT
