SUBROUTINE RADDRV(KGPTOT ,KFLDS ,KBLOCKS,&
 & PGMV,PGMVS,&
 & PGELAM ,PGEMU ,PGESLO ,PGECLO ,PGPIN,PGFL)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!USE YOMCT0   , ONLY : NSTART
USE YOMDIM   , ONLY : NDGENL   ,NFLEVG   ,NPROMA
USE YOMGEM   , ONLY : NGPTOT
!USE YOEAERD  , ONLY : RCAEROS
USE YOERDI   , ONLY : RCARDI
USE YOERAD   , ONLY : NMODE    ,NRPROMA  ,NTSW     ,LEPO3RA  ,&
 & LCMIP6_PI_AEROSOLS, LMAC2SP, L2RAD, L2RADTEND, L2RADACCU, NRADFR
USE YOMPHY   , ONLY : LRAYFM
USE YOMRADF  , ONLY : EMTD     ,TRSW     ,EMTC     ,TRSC     ,&
 & SRSWD    ,SRSWDCS  ,SRSWPAR  ,SRSWUVB, SRSWPARC, SRSWTINC ,&
 & SRFDIR   ,SRCDIR, EMTD2, TRSW2, EMTC2, TRSC2   
USE SURFACE_FIELDS_MIX , ONLY : SP_SB,SP_SG,SP_RR,&
 & SD_VF,SD_VN,SD_VD, &
 & SD_XA, SD_X2, YSD_XAD, &
 & YSP_RR, YSD_VF, YSD_VN
USE YOM_YGFL , ONLY : YGFL     ,YQ ,YL ,YI ,YR, YS ,YO3 ,YAERO   ,&
 & NACTAERO, NAERO, YCDNC, YICNC, YRE_LIQ, YRE_ICE        
USE YOMGMV   , ONLY : YPH9     ,NDIMGMV  ,NDIMGMVS
USE YOMGC    , ONLY : OROG
USE YOECLDP,   ONLY : NCLOUDACT
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE YOMCT0   , ONLY : NFRPOS, NSTOP
USE PHY_DIAG_MOD, ONLY : NRAD2D, NRAD3D, RAD_EXTRA_DIAG, RESET_ACCUM_DIAG

!**** *RADDRV* - Radiation driver routine

!     Purpose.
!     --------
!           The motivation for this routine is primarily to call RADINTG.
!        RADINTG has been transplanted from CPGLAG/CALLPAR, so as to be 
!        able to resolve the conflict of latitudinal dependency implicit
!        in the interpolation scheme in RADINTG and support of any value
!        for NPROMA in CPGLAG.

!**   Interface.
!     ----------
!        *CALL* *RADDRV*

!        Explicit arguments :
!        --------------------
!        PGPIN     : input grid point array
!        PGFL     : input grid point array for unified_treatment grid-point fields

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by SCAN2M and SCAN2M.

!     Reference.
!     ----------
!        see documentation for CPGLAG and CALLPAR

!     Author.
!     -------
!        George Mozdzynski  *ECMWF*

!     Modifications.
!     --------------
!      Original : 94-12-5
!      95-08,   Ph. Dandin  : Ozone and aerosols for FMR (Meteo-France) 
!      96-06-07 JJ Morcrette: Surf. LW and SW radiative properties
!      96-08,   Ph. Dandin  : I*DIA added for RADDIAG
!      990128   JJMorcrette : sunshine duration
!      99-01-28 JJMorcrette : sunshine duration
!      99-05-25 JJMorcrette : Revised aerosols
!      99-09  JJMorcrette: clear-sky radiation fluxes
!      01-06-29 JJMorcrette : reordering aerosols
!      01-10-10 JJMorcrette : CCNs
!      01-11    GMozdzynski : support new radiation grid
!      02-09-20 JJMorcrette : P/P UV, PAR
!      03-08-01 M.Hamrud    : GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      JJMorcrette 20060721 PP of clear-sky OAR and TOA incident solar radiation
!        M.Hamrud      01-Jul-2006 Revised surface fields
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!      JJMorcrette 20081230 : Prognostic aerosols in rad and clouds
!      JJMorcrette 20091201 : Total and clear-sky downward direct SW flux at surface
!      GMozdzynski 20100226 : Remove old radiation interpolation interface (RADINT)
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGPTOT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOCKS 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV,KBLOCKS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVS(NPROMA,NDIMGMVS,KBLOCKS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(MAX(NPROMA,KGPTOT)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(MAX(NPROMA,KGPTOT)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(MAX(NPROMA,KGPTOT)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(MAX(NPROMA,KGPTOT)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPIN(NPROMA,KFLDS,KBLOCKS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM,KBLOCKS) 

!     ------------------------------------------------------------------

! ZPRE9  : (NPROMA,0:NFLEVG)     ; HALF LEVEL PRESSURE
! ZPRE9F : (NPROMA,  NFLEVG)     ; FULL LEVEL PRESSURE
! ZDELP9 : (NPROMA,0:NFLEVG+1)   ; PRESSURE DIFFERENCE ACROSS LAYERS
! ZGEOH      : HALF LEVEL GEOPOTENTIAL HEIGHT

REAL(KIND=JPRB), ALLOCATABLE :: ZPRE9  (:,:),ZPRE9F (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDELP9 (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGEOH(:,:)
LOGICAL :: LLLAND (NPROMA)
LOGICAL :: LLLANDT(KGPTOT)

! ZMU0M : (NPROMA)            ; SOLAR ANGLE (between radiation steps)

REAL(KIND=JPRB) :: ZMU0M (NPROMA)
REAL(KIND=JPRB) :: ZMU0MT(KGPTOT)

! - 1D (GEOGRAPHICAL DISTRIBUTION) .

! ZALBD      : DIFFUSE ALBEDO IN THE NTSW SW INTERVALS
! ZALBP      : PARALLEL ALBEDO IN THE NTSW SW INTERVALS

REAL(KIND=JPRB) :: ZALBD (NPROMA,NTSW)
REAL(KIND=JPRB) :: ZALBP (NPROMA,NTSW)
REAL(KIND=JPRB) :: ZALBDT(KGPTOT,NTSW)
REAL(KIND=JPRB) :: ZALBPT(KGPTOT,NTSW)

! ZCCNL      : CCN CONCENTRATION OVER LAND
! ZCCNO      : CCN CONCENTRATION OVER OCEAN 

REAL(KIND=JPRB) :: ZCCNL (NPROMA), ZCCNO (NPROMA)
REAL(KIND=JPRB) :: ZCCNLT(KGPTOT), ZCCNOT(KGPTOT)

! - 2D (1:KLEV) .

! ZNEB       : FRACTIONAL CLOUDINESS FOR RADIATION.
! ZQICE      : SPECIFIC HUMIDITY OF SOLID WATER FOR RADIATION.
! ZQLI       : SPECIFIC HUMIDITY OF LIQUID WATER FOR RADIATION.
! ZQRAI      : SPECIFIC HUMIDITY OF RAIN WATER FOR RADIATION.
! ZRAIT      : RAIN RATE FOR RADIATION.
! ZQSAT      : SPECIFIC HUMIDITY AT SATURATION.
! ZAEROCL    : CLIMATOLOGICAL AEROSOL OPTICAL DEPTH
! ZOZONE     : CLIMATOLOGICAL OZONE

! ZGEOH      : HALF LEVEL GEOPOTENTIAL HEIGHT

REAL(KIND=JPRB) :: ZNEB  (NPROMA,NFLEVG),ZQLI  (NPROMA,NFLEVG)&
 & ,  ZQSAT (NPROMA,NFLEVG),ZQICE (NPROMA,NFLEVG)&
 & ,  ZQRAI (NPROMA,NFLEVG),ZRAIT (NPROMA,NFLEVG)  
!REAL(KIND=JPRB) :: ZAERO(NPROMA,NFLEVG,NAERO)

! - 1D (DIAGNOSTIC) .

! ZEMIR      : MODEL SURFACE LONGWAVE EMISSIVITY OUTSIDE LW WINDOW. 
! ZEMIW      : MODEL SURFACE LONGWAVE WINDOW EMISSIVITY.
! ZEMIT      : MODEL SURFACE LONGWAVE TOTAL EMISSIVITY.
! ZCCC       : CONVECTIVE CLOUD COVER
! ZLCC       : LOW CLOUD COVER
! ZMCC       : MEDIUM CLOUD COVER
! ZHCC       : HIGH CLOUD COVER
! ZTCC       : TOTAL CLOUD COVER

REAL(KIND=JPRB) :: ZEMIR (NPROMA),ZEMIW (NPROMA),ZEMIT (NPROMA)
REAL(KIND=JPRB) :: ZEMIRT(KGPTOT),ZEMIWT(KGPTOT),ZEMITT(KGPTOT)

REAL(KIND=JPRB) :: ZTCC (NPROMA), ZCCC (NPROMA), ZLCC (NPROMA),&
 & ZMCC (NPROMA), ZHCC (NPROMA)  
REAL(KIND=JPRB) :: ZTCCT(KGPTOT), ZCCCT(KGPTOT), ZLCCT(KGPTOT),&
 & ZMCCT(KGPTOT), ZHCCT(KGPTOT)  

! - ADDITIONAL DIAGNOSTIC FIELDS
REAL(KIND=JPRB) :: ZEXTR2(KGPTOT,NRAD2D)
REAL(KIND=JPRB) :: ZEXTRA(KGPTOT,YSD_XAD%NLEVS,NRAD3D)

! - 2D (0:KLEV) FLUXES .

! ZFRSO      : SHORTWAVE RADIATIVE FLUX.
! ZFRTH      : LONGWAVE RADIATIVE FLUX.

REAL(KIND=JPRB), ALLOCATABLE :: ZFRSO (:,:) ,ZFRTH (:,:)

! - 2D (0:1)
! ZFRSOC     : SHORTWAVE CLEAR SKY RADIATIVE FLUX
! ZFRTHC     : LONGWAVE CLEAR SKY RADIATIVE FLUX

REAL(KIND=JPRB), ALLOCATABLE :: ZFRSOC (:,:),ZFRTHC (:,:)
REAL(KIND=JPRB) :: ZFRSOCT(KGPTOT,0:1),ZFRTHCT(KGPTOT,0:1)

! Arrays passed between RADINT and RADDIAG

! ZTH        : HALF LEVEL TEMPERATURE
! ZCTRSO     : CLEAR-SKY SHORTWAVE TRANSMISSIVITY
! ZCEMTR     : CLEAR-SKY NET LONGWAVE EMISSIVITY

REAL(KIND=JPRB), ALLOCATABLE :: ZTH (:,:)

! ZTRENU9T   : SURFACE TEMPERATURE (PTS)
! ZVFLSMT    : LAND-SEA MASK (PSLM)
! ZVREMTD    : TOTAL DOWNWARD LONGWAVE EMISSIVITY (EMTD)
! ZVREMTU    : TOTAL UPWARD   LONGWAVE EMISSIVITY (EMTU)
! ZVRTRSW    : TOTAL SHORTWAVE TRANSMISSIVITY (PTRSO)

! ZAEROT     : MIXING RATIO OF PROGNOSTIC AEROSOLS

!-- clear-sky radiation
! ZVRTRSCT   : CLEAR-SKY SHORTWAVE TRANSMISSIVITY (PTRSC)
! ZVREMTCT   : CLEAR-SKY NET LONGWAVE EMISSIVITY  (EMTC)

! Z2Q9       : SPECIFIC HUMIDITY PA/PA (PQ)
! Z2T9       : FULL LEVEL TEMPERATURE (PT)
! Z2O39      : FULL LEVEL PROGNOSTIC OZONE MIXING RATIO (ECMWF)

! ZTRSODT    : SURFACE DOWNWARD SW RADIATION
! ZVNTOPT    : INDEX FOR TOP OF CONVECTIVE LAYER
! ZVNBAST    : INDEX FOR BOTTOM OF CONVECTIVE LAYER

REAL(KIND=JPRB) :: ZTRENU9T(KGPTOT)
REAL(KIND=JPRB) :: ZVFLSMT (KGPTOT)
REAL(KIND=JPRB), ALLOCATABLE :: ZVREMTD (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVREMTU (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVRTRSW (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCDNCT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZICNCT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRE_LIQT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRE_ICET(:,:)
!--      
REAL(KIND=JPRB) :: ZTRSODT (KGPTOT)
REAL(KIND=JPRB) :: ZSUDUT  (KGPTOT)
REAL(KIND=JPRB) :: ZUVDFT  (KGPTOT)
REAL(KIND=JPRB) :: ZPARFT  (KGPTOT)
REAL(KIND=JPRB) :: ZPARCFT (KGPTOT)
REAL(KIND=JPRB) :: ZTINCFT (KGPTOT)
REAL(KIND=JPRB) :: ZFDIRT  (KGPTOT), ZCDIRT(KGPTOT)

REAL(KIND=JPRB) :: ZVNTOPT (KGPTOT)
REAL(KIND=JPRB) :: ZVNBAST (KGPTOT)

!                 LOCAL ARRAYS FROM METEO-FRANCE PHYSICS PACKAGE
!                 ----------------------------------------------

! ZQO3   ; OZONE MIXING RATIO (MASS)
! ZAER   ; AEROSOLS DISTRIBUTIONS
REAL(KIND=JPRB), ALLOCATABLE :: ZQO3(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZAER(:,:,:)

REAL(KIND=JPRB), ALLOCATABLE :: ZPRE9T (:,:),ZPRE9FT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZDELP9T(:,:), ZGEOHT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZNEBT (:,:),ZQLIT(:,:)&
 & ,  ZQSATT(:,:),ZQICET(:,:),ZQSNOWT(:,:)&
 & ,  ZQRAIT(:,:),ZRAITT(:,:)  
REAL(KIND=JPRB), ALLOCATABLE :: ZFRSOT(:,:) ,ZFRTHT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZTHT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVREMTDT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVREMTUT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVRTRSWT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVREMTCT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVRTRSCT(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: Z2Q9T   (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: Z2T9T   (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: Z2O39T  (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZAEROT(:,:,:)

INTEGER(KIND=JPIM) :: IBLOCK, IEND, IFDIA, IGL1,&
 & IGL2, IIDIA, &
 & IST, JAER, JCHA, JK, JL, JLEV, JN, &
 & JLON, JSTGLO, JSW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! Double radiation
REAL(KIND=JPRB), ALLOCATABLE :: ZTRSC2(:,:), ZTRSO2(:,:), ZEMTD2(:,:), ZEMTC2(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRADDUMMY1D(:), ZRADDUMMY2D(:,:)

!     ------------------------------------------------------------------

#include "radcfg.intfb.h"
!#include "aer_rad.intfb.h"
#include "raddiag.intfb.h"
#include "radintg.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADDRV',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!  Initialisation

ALLOCATE(ZPRE9T (KGPTOT,0:NFLEVG))
ALLOCATE(ZGEOHT (KGPTOT,0:NFLEVG))
ALLOCATE(ZPRE9FT(KGPTOT,NFLEVG))
ALLOCATE(ZDELP9T(KGPTOT,0:NFLEVG+1))
ALLOCATE(ZNEBT(KGPTOT,NFLEVG))
ALLOCATE(ZQLIT(KGPTOT,NFLEVG))
ALLOCATE(ZQSATT(KGPTOT,NFLEVG))
ALLOCATE(ZQICET(KGPTOT,NFLEVG))
ALLOCATE(ZQSNOWT(KGPTOT,NFLEVG))
ALLOCATE(ZQRAIT(KGPTOT,NFLEVG))
ALLOCATE(ZRAITT(KGPTOT,NFLEVG))
ALLOCATE(ZFRSOT(KGPTOT,0:NFLEVG))
ALLOCATE(ZFRTHT(KGPTOT,0:NFLEVG))
ALLOCATE(ZTHT(KGPTOT,NFLEVG+1))
ALLOCATE(ZVREMTDT(KGPTOT,NFLEVG+1))
ALLOCATE(ZVREMTUT(KGPTOT,NFLEVG+1))
ALLOCATE(ZVRTRSWT(KGPTOT,NFLEVG+1))
ALLOCATE(ZVREMTCT(KGPTOT,NFLEVG+1))
ALLOCATE(ZVRTRSCT(KGPTOT,NFLEVG+1))
ALLOCATE(Z2Q9T(KGPTOT,NFLEVG))
ALLOCATE(Z2T9T(KGPTOT,NFLEVG))
IF(YO3%LGP.AND.LEPO3RA) THEN
  ALLOCATE(Z2O39T(KGPTOT,NFLEVG))
ELSE
  ALLOCATE(Z2O39T(1,1))
ENDIF
IF(NACTAERO > 0) THEN
  ALLOCATE(ZAEROT(KGPTOT,NFLEVG,NAERO))
ELSE
  ALLOCATE(ZAEROT(1,1,1))
ENDIF
IF (NCLOUDACT > 0) THEN
  ALLOCATE(ZCDNCT(KGPTOT,NFLEVG))
  ALLOCATE(ZICNCT(KGPTOT,NFLEVG))
  ALLOCATE(ZRE_LIQT(KGPTOT,NFLEVG))
  ALLOCATE(ZRE_ICET(KGPTOT,NFLEVG))
ELSE
  ALLOCATE(ZCDNCT(1,1))
  ALLOCATE(ZICNCT(1,1))
  ALLOCATE(ZRE_LIQT(1,1))
  ALLOCATE(ZRE_ICET(1,1))
END IF

IF( LRAYFM )THEN
  CALL GSTATS(1201,0)
! METEO FRANCE REQUIRE THESE ARRAYS
  ALLOCATE(ZQO3(NPROMA,NFLEVG))
  ALLOCATE(ZAER(NPROMA,6,NFLEVG))
!$OMP PARALLEL PRIVATE(JLEV,JLON,JCHA)
!$OMP DO SCHEDULE(STATIC)
  DO JLEV=1,NFLEVG
    DO JLON=1,NPROMA
      ZQO3(JLON,JLEV) = 0.0_JPRB
    ENDDO
  ENDDO
!$OMP END DO NOWAIT
!$OMP DO SCHEDULE(STATIC)
  DO JLEV=1,NFLEVG
    DO JCHA=1,6
      DO JLON=1,NPROMA
        ZAER(JLON,JCHA,JLEV)=0.0_JPRB
      ENDDO
    ENDDO
  ENDDO
!$OMP END DO
!$OMP END PARALLEL
  CALL GSTATS(1201,1)
ELSE
! SO FOR ECMWF RADIATION WE MAKE THEM AS SMALL AS POSSIBLE
! (WITHOUT VIOLATING RUNTIME SUBSCRIPT CHECKING ON CALLS)
  ALLOCATE(ZQO3(1,1))
  ALLOCATE(ZAER(1,1,1))
ENDIF


!  Perform Computations on Fine Grid

CALL GSTATS(1202,0)

!$OMP PARALLEL &
!$OMP&PRIVATE(JSTGLO,IST,IEND,IBLOCK,JAER,JL,JSW,JK,&
!$OMP&        LLLAND,ZALBD,ZALBP,ZCCNL,ZCCNO,ZGEOH,ZPRE9,ZPRE9F,ZNEB,ZCCC,ZHCC,ZLCC,&
!$OMP&        ZMCC,ZTCC,ZDELP9,ZEMIR,ZEMIW,ZEMIT,ZMU0M,ZQSAT,ZQICE,ZQLI,&
!$OMP&        ZQRAI,ZRAIT)

IF(.NOT.ALLOCATED(ZPRE9 )) ALLOCATE(ZPRE9(NPROMA,0:NFLEVG))
IF(.NOT.ALLOCATED(ZPRE9F)) ALLOCATE(ZPRE9F(NPROMA,NFLEVG))
IF(.NOT.ALLOCATED(ZDELP9)) ALLOCATE(ZDELP9(NPROMA,0:NFLEVG+1))
IF(.NOT.ALLOCATED(ZGEOH )) ALLOCATE(ZGEOH(NPROMA,0:NFLEVG))

!$OMP DO SCHEDULE(DYNAMIC,1)
DO JSTGLO=1,NGPTOT,NPROMA
  IST =1
  IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
  IBLOCK=(JSTGLO-1)/NPROMA+1

  ! Reset accumulated diagnostics if needed
  CALL RESET_ACCUM_DIAG(IST, IEND, NFLEVG, IBLOCK, .TRUE.)

  CALL RADCFG(IST    ,IEND   ,&
   & JSTGLO ,KFLDS,&
   & LLLAND,&
   & OROG(JSTGLO),&
   & PGPIN(1,1,IBLOCK),PGFL(1,1,1,IBLOCK),&
   & PGMV(1,1,YPH9%MU,IBLOCK),PGMV(1,1,YPH9%MV,IBLOCK),&
   & PGMV(1,1,YPH9%MT,IBLOCK),PGMVS(1,YPH9%MSP,IBLOCK),&
   & SP_SB(1,1,1,IBLOCK),SP_SG(1,1,IBLOCK),SP_RR(1,1,IBLOCK),&
   & SD_VF(1,1,IBLOCK),SD_VN(1,1,IBLOCK),SD_VD(1,1,IBLOCK),&
   & SD_XA(1,1,1,IBLOCK), SD_X2(1,1,IBLOCK), &
   & ZALBD  ,ZALBP  ,ZCCNL  ,ZCCNO  ,ZPRE9  ,ZPRE9F ,ZNEB,&
   & ZCCC   ,ZHCC   ,ZLCC   ,ZMCC   ,ZTCC,&
   & PGELAM(JSTGLO),&
   & ZDELP9 ,ZEMIR  ,ZEMIW ,ZEMIT,&
   & ZGEOH,PGEMU(JSTGLO)  ,ZMU0M ,ZQSAT,&
   & ZQICE  ,ZQLI   ,ZQRAI ,ZRAIT &
   & )
       
  DO JL=IST,IEND
    LLLANDT (JSTGLO+JL-1)  =LLLAND(JL)
    ZCCCT   (JSTGLO+JL-1)  =ZCCC  (JL)
    ZHCCT   (JSTGLO+JL-1)  =ZHCC  (JL)
    ZLCCT   (JSTGLO+JL-1)  =ZLCC  (JL)
    ZMCCT   (JSTGLO+JL-1)  =ZMCC  (JL)
    ZTCCT   (JSTGLO+JL-1)  =ZTCC  (JL)
    ZEMIRT  (JSTGLO+JL-1)  =ZEMIR (JL)
    ZEMIWT  (JSTGLO+JL-1)  =ZEMIW (JL)
    ZEMITT  (JSTGLO+JL-1)  =ZEMIT (JL)
    ZMU0MT  (JSTGLO+JL-1)  =ZMU0M (JL)
    ZTRENU9T(JSTGLO+JL-1)  =SP_RR (JL,YSP_RR%YT%MP9,IBLOCK )
    ZVFLSMT (JSTGLO+JL-1)  =SD_VF (JL,YSD_VF%YLSM%MP,IBLOCK)
    
    ZVNTOPT (JSTGLO+JL-1)  =SD_VN (JL,YSD_VN%YTOP%MP,IBLOCK)
    ZVNBAST (JSTGLO+JL-1)  =SD_VN (JL,YSD_VN%YBAS%MP,IBLOCK)
    ZCCNLT  (JSTGLO+JL-1)  =ZCCNL (JL)
    ZCCNOT  (JSTGLO+JL-1)  =ZCCNO (JL)
  ENDDO
  DO JSW=1,NTSW
    DO JL=IST,IEND
      ZALBDT  (JSTGLO+JL-1,JSW)  =ZALBD (JL,JSW)
      ZALBPT  (JSTGLO+JL-1,JSW)  =ZALBP (JL,JSW)
    ENDDO
  ENDDO
  DO JK=0,NFLEVG
    DO JL=IST,IEND
      ZPRE9T  (JSTGLO+JL-1,JK)=ZPRE9 (JL,JK)
      ZGEOHT  (JSTGLO+JL-1,JK)=ZGEOH (JL,JK)
    ENDDO
  ENDDO
  DO JK=1,NFLEVG
    DO JL=IST,IEND
      ZPRE9FT (JSTGLO+JL-1,JK)=ZPRE9F(JL,JK)
      ZNEBT   (JSTGLO+JL-1,JK)=ZNEB  (JL,JK)
      ZQSATT  (JSTGLO+JL-1,JK)=ZQSAT (JL,JK)
!      ZQICET  (JSTGLO+JL-1,JK)=ZQICE (JL,JK)
!      ZQSNOWT (JSTGLO+JL-1,JK)=ZQSNOW (JL,JK)
!      ZQLIT   (JSTGLO+JL-1,JK)=ZQLI  (JL,JK)
      ZQICET  (JSTGLO+JL-1,JK)=PGFL(JL,JK,YI%MP9_PH,IBLOCK)
      ZQSNOWT (JSTGLO+JL-1,JK)=PGFL(JL,JK,YS%MP9_PH,IBLOCK)
      ZQLIT   (JSTGLO+JL-1,JK)=PGFL(JL,JK,YL%MP9_PH,IBLOCK)
!      ZQRAIT  (JSTGLO+JL-1,JK)=ZQRAI (JL,JK)
      ZQRAIT  (JSTGLO+JL-1,JK)=PGFL(JL,JK,YR%MP9_PH,IBLOCK)
      ZRAITT  (JSTGLO+JL-1,JK)=ZRAIT (JL,JK)
      Z2Q9T   (JSTGLO+JL-1,JK)=PGFL(JL,JK,YQ%MP9_PH,IBLOCK)
      Z2T9T   (JSTGLO+JL-1,JK)=PGMV(JL,JK,YPH9%MT,IBLOCK)
      IF(YO3%LGP.AND.LEPO3RA) THEN
        Z2O39T  (JSTGLO+JL-1,JK)=PGFL(JL,JK,YO3%MP9_PH,IBLOCK)
      ENDIF
    ENDDO
  ENDDO

  IF (NACTAERO > 0) THEN
    DO JAER=1,NAERO
      DO JK=1,NFLEVG
        DO JL=IST,IEND
          ZAEROT(JSTGLO+JL-1,JK,JAER)=PGFL(JL,JK,YAERO(JAER)%MP9_PH,IBLOCK)
        ENDDO
      ENDDO
    ENDDO 
  ENDIF

  IF (NCLOUDACT > 0) THEN
    DO JK=1,NFLEVG
      DO JL=IST,IEND
        ZCDNCT(JSTGLO+JL-1,JK) = PGFL(JL,JK,YCDNC%MP9_PH,IBLOCK)
        ZICNCT(JSTGLO+JL-1,JK) = PGFL(JL,JK,YICNC%MP9_PH,IBLOCK)
        ZRE_LIQT(JSTGLO+JL-1,JK) = PGFL(JL,JK,YRE_LIQ%MP9_PH,IBLOCK)
        ZRE_ICET(JSTGLO+JL-1,JK) = PGFL(JL,JK,YRE_ICE%MP9_PH,IBLOCK)
      END DO
    END DO
  END IF

  DO JK=0,NFLEVG+1
    DO JL=IST,IEND
      ZDELP9T (JSTGLO+JL-1,JK)=ZDELP9(JL,JK)
    ENDDO
  ENDDO
  DO JK=1,NFLEVG+1
    DO JL=IST,IEND
      ZVREMTDT(JSTGLO+JL-1,JK)=EMTD(JL,JK,IBLOCK)
      ZVREMTUT(JSTGLO+JL-1,JK)=0.0
      ZVRTRSWT(JSTGLO+JL-1,JK)=TRSW(JL,JK,IBLOCK)
!--clear-sky
      ZVREMTCT(JSTGLO+JL-1,JK)=EMTC(JL,JK,IBLOCK)
      ZVRTRSCT(JSTGLO+JL-1,JK)=TRSC(JL,JK,IBLOCK)
!--              
    ENDDO
  ENDDO

ENDDO
!$OMP END DO

IF(ALLOCATED(ZPRE9 )) DEALLOCATE(ZPRE9)
IF(ALLOCATED(ZPRE9F)) DEALLOCATE(ZPRE9F)
IF(ALLOCATED(ZDELP9)) DEALLOCATE(ZDELP9)
IF(ALLOCATED(ZGEOH)) DEALLOCATE(ZGEOH)

!$OMP END PARALLEL
CALL GSTATS(1202,1)

!  FULL RADIATION COMPUTATIONS


CALL RADINTG &
 & (NFLEVG  ,NMODE,&
 & ZALBDT  ,ZALBPT  ,ZCCNLT  ,ZCCNOT  ,ZGEOHT  ,ZPRE9T  ,ZPRE9FT ,RCARDI ,ZNEBT,&
 & ZQO3    ,ZAER    ,ZAEROT  ,&
 & PGECLO  ,PGESLO,&
 & ZDELP9T(1,1)     ,ZEMIRT  ,ZEMIWT,&
 & PGELAM  ,PGEMU   ,ZMU0MT  ,Z2Q9T,&
 & ZQSATT  ,ZQICET  ,ZQSNOWT ,ZQLIT   ,ZQRAIT  ,ZRAITT,&
 & ZVFLSMT ,Z2T9T   ,Z2O39T  ,ZTRENU9T,&
 & ZVREMTDT,ZVREMTUT,ZVRTRSWT,ZVRTRSCT,ZVREMTCT,ZVNTOPT ,ZVNBAST,&
 ! FLUX OUTPUTS
 & ZEMITT  ,ZTHT    ,ZTRSODT ,ZSUDUT  ,ZUVDFT  ,ZPARFT  ,ZPARCFT, ZTINCFT,&
 & ZFDIRT  ,ZCDIRT  , &
 & YSD_XAD%NLEVS,  ZEXTRA,  ZEXTR2,&
 & ZCDNCT, ZICNCT, ZRE_LIQT, ZRE_ICET )

! Double radiation
! Second call to RADINTG
!
IF (L2RAD) THEN
   ! Allocate arrays for the wanted fields
   ALLOCATE(ZTRSC2(KGPTOT,NFLEVG+1) ,ZTRSO2(KGPTOT,NFLEVG+1)  ,&
        &   ZEMTD2(KGPTOT,NFLEVG+1) ,ZEMTC2(KGPTOT,NFLEVG+1))
   
   ! Allocate dummy variables so that the second call does not mess
   ! up the correct values of OUT variables:
   ! RADDRV   - RADINTG
   ! ZVREMTUT - PEMTU(NGPTOT,KLEV+1)
   ! ZEMITT   - PEMIT(NGPTOT)
   ! ZTHT     - PTH(NGPTOT,KLEV+1)
   ! ZTRSODT  - PTRSOD(NGPTOT)
   ! ZSUDUT   - PSUDU(NGPTOT)
   ! ZUVDFT   - PUVDF(NGPTOT)
   ! ZPARFT   - PPARF(NGPTOT)
   ! ZPARCFT  - PPARCF(NGPTOT)
   ! ZTINCFT  - PTINCF(NGPTOT)
   ! ZFDIRT   - PFDIR(NGPTOT)
   ! ZCDIRT   - PCDIR(NGPTOT)
   ALLOCATE(ZRADDUMMY1D(KGPTOT),ZRADDUMMY2D(KGPTOT,NFLEVG+1))
   
   CALL RADINTG &
        & (NFLEVG  ,NMODE,&
        & ZALBDT  ,ZALBPT  ,ZCCNLT  ,ZCCNOT  ,ZGEOHT  ,ZPRE9T  ,ZPRE9FT ,RCARDI ,ZNEBT,&
        & ZQO3    ,ZAER    ,ZAEROT  ,&
        & PGECLO  ,PGESLO,&
        & ZDELP9T(1,1)     ,ZEMIRT  ,ZEMIWT,&
        & PGELAM  ,PGEMU   ,ZMU0MT  ,Z2Q9T,&
        & ZQSATT  ,ZQICET  ,ZQSNOWT ,ZQLIT   ,ZQRAIT  ,ZRAITT,&
        & ZVFLSMT ,Z2T9T   ,Z2O39T  ,ZTRENU9T,&
        & ZEMTD2,ZRADDUMMY2D,ZTRSO2,ZTRSC2,ZEMTC2,ZVNTOPT ,ZVNBAST,&
        ! FLUX OUTPUTS
        & ZRADDUMMY1D, ZRADDUMMY2D, ZRADDUMMY1D, ZRADDUMMY1D, ZRADDUMMY1D, ZRADDUMMY1D, ZRADDUMMY1D, ZRADDUMMY1D,&
        & ZRADDUMMY1D  ,ZRADDUMMY1D ,&
        & YSD_XAD%NLEVS ,ZEXTRA, ZEXTR2     , &
        & ZCDNCT, ZICNCT, ZRE_LIQT, ZRE_ICET, &
        & LZEROAERO=.TRUE.)

   DEALLOCATE(ZRADDUMMY1D,ZRADDUMMY2D)

END IF

!  PERFORM RADIATION DIAGNOSTICS

IIDIA=0
IFDIA=0

IGL1=1
IGL2=NDGENL
CALL GSTATS(1906,0)
CALL RADDIAG &
 & (IGL1    ,IGL2    ,NGPTOT  ,NFLEVG,&
 & LLLANDT,&
 & ZALBDT  ,ZALBPT  ,ZPRE9T  ,ZNEBT,&
 & ZCCCT   ,ZHCCT   ,ZLCCT   ,ZMCCT   ,ZTCCT,&
 & ZEMIRT  ,ZEMIWT  ,ZEMITT,&
 & PGEMU   ,ZMU0MT  ,ZVFLSMT,&
 & Z2T9T   ,ZTRENU9T,&
 & ZVREMTDT,ZVRTRSWT,&
 & ZFRTHCT ,ZFRTHT  ,ZFRSOCT ,ZFRSOT  ,ZTHT  , ZVRTRSCT, ZVREMTCT &
 & )  
CALL GSTATS(1906,1)

IF(ALLOCATED(ZFRSO))  DEALLOCATE(ZFRSO)
IF(ALLOCATED(ZFRTH))  DEALLOCATE(ZFRTH)
IF(ALLOCATED(ZFRSOC)) DEALLOCATE(ZFRSOC)
IF(ALLOCATED(ZFRTHC)) DEALLOCATE(ZFRTHC)
IF(ALLOCATED(ZTH))    DEALLOCATE(ZTH) 

!  WRITE OUT GRIDPOINT ARRAY TO WORK FILE

CALL GSTATS(1203,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,IST,IEND,JK,JL,IBLOCK)
DO JSTGLO=1,NGPTOT,NPROMA
  IST =1
  IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
  IBLOCK=(JSTGLO-1)/NPROMA+1

  DO JK=1,NFLEVG+1
    DO JL=IST,IEND
!-- clear-sky              
      EMTD(JL,JK,IBLOCK) = ZVREMTDT(JSTGLO+JL-1,JK)
      TRSW(JL,JK,IBLOCK) = ZVRTRSWT(JSTGLO+JL-1,JK)
      EMTC(JL,JK,IBLOCK) = ZVREMTCT(JSTGLO+JL-1,JK)
      TRSC(JL,JK,IBLOCK) = ZVRTRSCT(JSTGLO+JL-1,JK)
    ENDDO
  ENDDO

  ! Double radiation
  ! Copy local arrays to a global array (used in callpar.F90)
  IF (L2RAD) THEN
    DO JK=1,NFLEVG+1
      DO JL=IST,IEND
!-- clear-sky              
        EMTD2(JL,JK,IBLOCK) = ZEMTD2(JSTGLO+JL-1,JK)
        TRSW2(JL,JK,IBLOCK) = ZTRSO2(JSTGLO+JL-1,JK)
        EMTC2(JL,JK,IBLOCK) = ZEMTC2(JSTGLO+JL-1,JK)
        TRSC2(JL,JK,IBLOCK) = ZTRSC2(JSTGLO+JL-1,JK)
      ENDDO
    ENDDO
  END IF
  
  DO JL=IST,IEND
    SRSWDCS(JL,IBLOCK) = ZSUDUT (JSTGLO+JL-1)
    SRSWD  (JL,IBLOCK) = ZTRSODT(JSTGLO+JL-1)
    SRSWUVB(JL,IBLOCK) = ZUVDFT (JSTGLO+JL-1)
    SRSWPAR(JL,IBLOCK) = ZPARFT (JSTGLO+JL-1)
    SRSWPARC(JL,IBLOCK)= ZPARCFT(JSTGLO+JL-1)
    SRSWTINC(JL,IBLOCK)= ZTINCFT(JSTGLO+JL-1)
    SRFDIR (JL,IBLOCK) = ZFDIRT (JSTGLO+JL-1)
    SRCDIR (JL,IBLOCK) = ZCDIRT (JSTGLO+JL-1)
  ENDDO

  IF (NRAD2D > 0 .OR. NRAD3D > 0) &
     & CALL RAD_EXTRA_DIAG(KGPTOT, NPROMA, JSTGLO, IST, IEND, IBLOCK, ZEXTRA, ZEXTR2, ZMU0M)
               
ENDDO
!$OMP END PARALLEL DO

CALL GSTATS(1203,1)

IF(ALLOCATED(ZVREMTD)) DEALLOCATE(ZVREMTD)
IF(ALLOCATED(ZVREMTU)) DEALLOCATE(ZVREMTU)
IF(ALLOCATED(ZVRTRSW)) DEALLOCATE(ZVRTRSW)

! Double radiation
IF(ALLOCATED(ZTRSC2)) DEALLOCATE(ZTRSC2)
IF(ALLOCATED(ZTRSO2)) DEALLOCATE(ZTRSO2)
IF(ALLOCATED(ZEMTD2)) DEALLOCATE(ZEMTD2)
IF(ALLOCATED(ZEMTC2)) DEALLOCATE(ZEMTC2)
     
DEALLOCATE(ZPRE9T, ZPRE9FT, ZDELP9T, ZNEBT, ZQLIT, ZQSATT, ZQICET, ZQSNOWT)
DEALLOCATE(ZQRAIT, ZRAITT,ZGEOHT, ZFRSOT, ZFRTHT, ZTHT, ZVREMTDT, ZVREMTUT)
DEALLOCATE(ZVRTRSWT, ZVREMTCT, ZVRTRSCT, Z2Q9T, Z2T9T, Z2O39T)
DEALLOCATE(ZQO3, ZAER)
DEALLOCATE(ZAEROT)
DEALLOCATE(ZCDNCT, ZICNCT, ZRE_LIQT, ZRE_ICET)


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADDRV',1,ZHOOK_HANDLE)
END SUBROUTINE RADDRV
