SUBROUTINE RADINAAD &
 & ( KIDIA , KFDIA , KLON   , KLEV   , KMODE,&
 & PALBD5, PALBP5, PAPRS5 , PAPRSF5,&
 & PCCNL5, PCCNO5, PCCO25 , PCLFR5,&
 & PGELAM, PCLON,  PSLON,&
 & PDP5,   PEMIR5, PEMIW5,&
 & PGEMU,  PMU0 ,  PQ5    , PQS5,&
 & PQIWP5, PQLWP5,&
 & PSLM5 , PT5   , PTS5,&
 & PTH5  ,&
 & PALBD , PALBP , PAPRS  , PAPRSF,&
 & PCCNL , PCCNO , PCCO2  , PCLFR,&
 & PDP   , PEMIR , PEMIW,&
 & PQ    , PQS   ,&
 & PQIWP , PQLWP,&
 & PT    , PTS,&
 & PEMTD , PTRSO,&
 & PTH   , PTRSOD     &
 & )  

!**** *RADINAAD* - RADIATION INTERFACE TO RADIATION SCHEME USED
!                  DURING ASSIMILATION
!                  --> ADJOINT <--

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! KLEV   : NUMBER OF LEVELS

! PAER5  : (KLON,6,KLEV)        ; OPTICAL THICKNESS OF THE AEROSOLS
! PALBD  : (KLON,NSW)           ; DIFFUSE ALBEDO IN THE NSW SW INTERVALS
! PALBP  : (KLON,NSW)           ; PARALLEL ALBEDO IN THE NSW SW INTERVALS
! PAPRS  : (KLON,KLEV+1)        ; HALF LEVEL PRESSURE
! PAPRSF : (KLON,KLEV )         ; FULL LEVEL PRESSURE
! PCCNL  : (KLON)               ; CCN CONCENTRATION OVER LAND
! PCCNO  : (KLON)               ; CCN CONCENTRATION OVER THE OCEAN
! PCCO2  :                      ; CONCENTRATION IN CO2 (PA/PA)
! PCLFR  : (KLON,KLEV )         ; CLOUD FRACTIONAL COVER
! PQO35  : (KLON,KLEV )         ; OZONE MIXING RATIO (MASS)
! PDP    : (KLON,KLEV)          ; LAYER PRESSURE THICKNESS
! PEMIR  : (KLON)               ; SURFACE EMISSIVITY OUTSIDE LW WINDOW
! PEMIW  : (KLON)               ; SURFACE LW WINDOW EMISSIVITY
! PGEMU  : (KLON)               ; SINE OF LATITUDE ON THE REAL EARTH
! PMU0   : (KLON)               ; SOLAR ANGLE
! PQ     : (KLON,KLEV )         ; SPECIFIC HUMIDITY PA/PA
! PQS    : (KLON,KLEV )         ; SATURATION SPECIFIC HUMIDITY PA/PA
! PQIWP  : (KLON,KLEV )         ; ICE    WATER KG/KG
! PQLWP  : (KLON,KLEV )         ; LIQUID WATER KG/KG
! PSLM5  : (KLON)               ; LAND-SEA MASK
! PT     : (KLON,KLEV)          ; FULL LEVEL TEMPERATURE
! PTS    : (KLON)               ; SURFACE TEMPERATURE
!     ==== OUTPUTS ===
! PEMTD (KLON,KLEV+1)           ; TOTAL DOWNWARD LONGWAVE EMISSIVITY
! PTRSO (KLON,KLEV+1)           ; TOTAL SHORTWAVE TRANSMISSIVITY
! PTH   (KLON,KLEV+1)           ; HALF LEVEL TEMPERATURE
! PCTRSO(KLON,2)                ; CLEAR-SKY SHORTWAVE TRANSMISSIVITY
! PCEMTR(KLON,2)                ; CLEAR-SKY NET LONGWAVE EMISSIVITY
! PTRSOD(KLON)                  ; TOTAL-SKY SURFACE SW TRANSMISSITY

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHORS.
!     --------
!        M. JANISKOVA           

!        adapted from RADINA and RADINATL

!        J.-J. MORCRETTE        *ECMWF*
!        J.-F. MAHFOUF          *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        ORIGINAL       : 99-04-15
!        Modified : 2000-02-28 M.Janiskova - adopted for revised
!                                            aerosols as in radina
!                   2000-11-21 M.Janiskova - safety checks modified
!                   2008-03-18 M.Janiskova - commenting the computation
!                                            of diagnostic quantities
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY3  , ONLY : RII0
USE YOERAD   , ONLY : NAER     , NSW
USE YOEAERD  , ONLY : RCAEROS
USE YOERDU   , ONLY : NIMP     ,NOUT
USE YOERDI   , ONLY : RRAE     ,REPH2O
USE YOMPHY   , ONLY : LRAYFM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNO5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO25 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KLON), PCLON(KLON), PSLON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIR5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNO(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCCO2 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLFR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIR(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQIWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMTD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTRSO(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTRSOD(KLON) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!- trajectory values

!     ==== COMPUTED IN RADLSW ===

!- perturbations

!     ==== COMPUTED IN RADLSW ===

!     ------------------------------------------------------------------

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZQAER5(KLON,6,KLEV) , ZQOZ5(KLON,KLEV)

REAL(KIND=JPRB) :: ZFRSOD5(KLON)
REAL(KIND=JPRB) :: ZFRTH5 (KLON,KLEV+1), ZFRSO5 (KLON,KLEV+1)
REAL(KIND=JPRB) :: ZAMU05(KLON)
REAL(KIND=JPRB) :: ZCLFR5(KLON,KLEV)
REAL(KIND=JPRB) :: ZQ5(KLON,KLEV)    , ZQS5(KLON,KLEV)
REAL(KIND=JPRB) :: ZQIWP5(KLON,KLEV) , ZQLWP5(KLON,KLEV)
REAL(KIND=JPRB) :: ZQAER(KLON,6,KLEV), ZQOZ(KLON,KLEV)

REAL(KIND=JPRB) :: ZFRTHC(KLON,KLEV+1), ZFRSOC(KLON,KLEV+1), ZFRSOD(KLON)
REAL(KIND=JPRB) :: ZFRTH (KLON,KLEV+1), ZFRSO (KLON,KLEV+1)
REAL(KIND=JPRB) :: ZCLFR(KLON,KLEV)
REAL(KIND=JPRB) :: ZQ(KLON,KLEV)    , ZQS(KLON,KLEV)
REAL(KIND=JPRB) :: ZQIWP(KLON,KLEV) , ZQLWP(KLON,KLEV)

!     ==== EXTRA STORAGE FOR TRAJECTORY ===

REAL(KIND=JPRB) :: Z1S5(KLON,KLEV)   , Z2S5(KLON,KLEV)
REAL(KIND=JPRB) :: Z3S5(KLON,KLEV)

INTEGER(KIND=JPIM) :: JAER, JK, JL

REAL(KIND=JPRB) :: Z1S, Z2S, Z3S, ZCRAE, ZRII0, ZRII05
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "radaca.intfb.h"
#include "radlswad.intfb.h"

!     ------------------------------------------------------------------

!*       1.    PREPARATORY WORK
!              ----------------

IF (LHOOK) CALL DR_HOOK('RADINAAD',0,ZHOOK_HANDLE)
NIMP=0
NOUT=0
ZRII0  =0.0_JPRB
ZRII05 =RII0

!- various constants      

ZCRAE=RRAE*(RRAE+2.0_JPRB)

!     ------------------------------------------------------------------

!                 ----- TRAJECTORY COMPUTATIONS ------

!     ------------------------------------------------------------------

!*         2.     HALF-LEVEL QUANTITIES

DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    Z1S5(JL,JK) = PAPRSF5(JL,JK-1)*(PAPRSF5(JL,JK)-PAPRS5(JL,JK))
    Z2S5(JL,JK) = PAPRSF5(JL,JK)*(PAPRS5(JL,JK)-PAPRSF5(JL,JK-1))
    Z3S5(JL,JK) = PAPRS5(JL,JK)*(PAPRSF5(JL,JK)-PAPRSF5(JL,JK-1))
    PTH5(JL,JK)=(PT5(JL,JK-1)*Z1S5(JL,JK)+PT5(JL,JK)*Z2S5(JL,JK))/Z3S5(JL,JK)
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  PTH5(JL,1) = PT5(JL,1)-PAPRSF5(JL,1)&
   & *(PT5(JL,1)-PTH5(JL,2))/(PAPRSF5(JL,1)-PAPRS5(JL,2))    

  PTH5(JL,KLEV+1)= PTS5(JL)
ENDDO

!     ------------------------------------------------------------------

!*         3.     SOLAR ZENITH ANGLE IS SAMPLED AND EARTH'S CURVATURE
!                                     CORRECTED

! 300 CONTINUE

DO JL=KIDIA,KFDIA
  IF (PMU0(JL) > 1.E-10_JPRB) THEN
    ZAMU05(JL)=RRAE/(SQRT(PMU0(JL)**2+ZCRAE)-PMU0(JL))
  ELSE
    ZAMU05(JL)= RRAE/SQRT(ZCRAE)
  ENDIF  
ENDDO

!*           3.3     SECURITY COMPUTATIONS FOR CRITICAL VARIABLES.

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (PCLFR5(JL,JK) <= 0.0_JPRB) THEN
      ZCLFR5(JL,JK) = 0.0_JPRB
    ELSEIF (PCLFR5(JL,JK) >= 1.0_JPRB) THEN
      ZCLFR5(JL,JK) = 1.0_JPRB
    ELSE
      ZCLFR5(JL,JK)=PCLFR5(JL,JK)
    ENDIF
    IF (PQS5(JL,JK) <= 2.0_JPRB*REPH2O) THEN
      ZQS5(JL,JK) = 2.0_JPRB*REPH2O
    ELSE
      ZQS5(JL,JK)=PQS5(JL,JK)
    ENDIF
    IF (PQ5(JL,JK) <= 2.0_JPRB*REPH2O) THEN
      ZQ5(JL,JK) = 2.0_JPRB*REPH2O
    ELSEIF (PQ5(JL,JK) >= ZQS5(JL,JK)*(1.0_JPRB-REPH2O)) THEN
      ZQ5(JL,JK) = ZQS5(JL,JK)*(1.0_JPRB-REPH2O)
    ELSE
      ZQ5(JL,JK)=PQ5(JL,JK)
    ENDIF
    IF (PQLWP5(JL,JK) <= REPH2O) THEN
      ZQLWP5(JL,JK) = REPH2O
    ELSE
      ZQLWP5(JL,JK)=PQLWP5(JL,JK)
    ENDIF
    IF (PQIWP5(JL,JK) <= REPH2O) THEN
      ZQIWP5(JL,JK) = REPH2O
    ELSE
      ZQIWP5(JL,JK)=PQIWP5(JL,JK)
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         4.     OZONE AND AEROSOLS

IF(.NOT.LRAYFM) THEN
  CALL  RADACA ( KIDIA , KFDIA , KLON , KLEV,&
   & PAPRS5, PGELAM, PGEMU, PCLON, PSLON, PTH5,&
   & ZQAER5, ZQOZ5 )  

!  IF (NOZOCL == 1.AND..NOT.LEPO3RA) THEN
!    CALL RADOZC ( KIDIA , KFDIA , KLON, KTDIA, KLEV &
!     &, 1     , KLON , 0  &
!     &, PAPRS5, PGEMU &
!     &, ZQOZ5 )
!  ENDIF
ENDIF

IF (NAER == 0) THEN
  DO JAER = 1 , 6
    DO JK = 1 , KLEV
      DO JL = KIDIA,KFDIA
        ZQAER5(JL,JAER,JK) = RCAEROS
      ENDDO
    ENDDO
  ENDDO
ELSE
  DO JK = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZQAER5(JL,5,JK) = RCAEROS
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!                 ----- ADJOINT COMPUTATION ------

!     ------------------------------------------------------------------

!*                        INITIALIZATIONS
!                         ---------------
!        OZONE AND AEOROSOLS - since they are constant, the perturbation
!                              should be equal to zero

ZQAER (:,:,:) = 0.0_JPRB
ZQOZ (:,:) = 0.0_JPRB

!     other arrays

ZFRTHC(:,:)=0.0_JPRB
ZFRTH(:,:)=0.0_JPRB
ZFRSOC(:,:)=0.0_JPRB
ZFRSO(:,:)=0.0_JPRB
ZFRSOD(:)=0.0_JPRB
ZCLFR(:,:)=0.0_JPRB
ZQS  (:,:)=0.0_JPRB
ZQ   (:,:)=0.0_JPRB
ZQLWP(:,:)=0.0_JPRB
ZQIWP(:,:)=0.0_JPRB


!*         6.     TRANSFORM FLUXES TO MODEL HISTORICAL VARIABLES

! diagnostic clear sky quantities
!DO JL=KIDIA,KFDIA
!  ZFRTHC (JL,KLEV+1)=ZFRTHC (JL,KLEV+1)+PCEMTR (JL,2)
!  PCEMTR (JL,2)=0.0_JPRB

!  ZFRTHC (JL,1)=ZFRTHC (JL,1)+PCEMTR (JL,1)
!  PCEMTR (JL,1)=0.0_JPRB

!  ZFRSOC (JL,KLEV+1)=ZFRSOC (JL,KLEV+1)+PCTRSO (JL,2)/(ZRII05*ZAMU05(JL))
!  PCTRSO (JL,2)=0.0_JPRB

!  ZFRSOC (JL,1)=ZFRSOC (JL,1)+PCTRSO (JL,1)/(ZRII05*ZAMU05(JL))
!  PCTRSO (JL,1)=0.0_JPRB
!ENDDO

DO JL=KIDIA,KFDIA
  ZFRSOD(JL)=ZFRSOD(JL)+PTRSOD(JL)/(ZRII05*ZAMU05(JL))
  PTRSOD(JL)=0.0_JPRB
ENDDO

DO JK=1,KLEV+1
  DO JL=KIDIA,KFDIA

    ZFRTH (JL,JK)=ZFRTH (JL,JK)+PEMTD (JL,JK)
    PEMTD (JL,JK)=0.0_JPRB

    ZFRSO (JL,JK)=ZFRSO (JL,JK)+PTRSO (JL,JK)/(ZRII05*ZAMU05(JL))
    PTRSO (JL,JK)=0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         5.      CALL TO ACTUAL RADIATION SCHEME (Adjoint)

CALL RADLSWAD &
 & ( KIDIA , KFDIA , KLON , KLEV ,  KMODE , NAER,&
 & ZRII05,&
 & ZQAER5, PALBD5, PALBP5, PAPRS5, PAPRSF5,&
 & PCCNL5, PCCNO5,&
 & PCCO25, ZCLFR5, PDP5  , PEMIR5, PEMIW5,  ZQOZ5,&
 & PSLM5 , ZAMU05,&
 & ZQ5   , ZQIWP5, ZQLWP5, ZQS5,&
 & PTH5  , PT5   , PTS5,&
 & ZFRTH5, ZFRSO5, ZFRSOD5,&
 & ZQAER , PALBD , PALBP , PAPRS  , PAPRSF,&
 & PCCO2 , ZCLFR , PDP   , PEMIR  , PEMIW  , ZQOZ,&
 & ZQ    , ZQIWP , ZQLWP , ZQS    ,&
 & PTH   , PT    , PTS,&
 & ZFRTH , ZFRSO , ZFRSOD )  

!     ------------------------------------------------------------------

!*         3.     SOLAR ZENITH ANGLE IS SAMPLED AND EARTH'S CURVATURE

!*         3.3     SECURITY COMPUTATIONS FOR CRITICAL VARIABLES.

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (PQIWP5(JL,JK) > REPH2O) THEN
      PQIWP (JL,JK) = PQIWP (JL,JK) + ZQIWP (JL,JK)
    ENDIF
    ZQIWP (JL,JK) = 0.0_JPRB

    IF (PQLWP5(JL,JK) > REPH2O) THEN
      PQLWP (JL,JK) = PQLWP (JL,JK) + ZQLWP (JL,JK)
    ENDIF
    ZQLWP (JL,JK) = 0.0_JPRB
      
    IF (PQ5(JL,JK) >= ZQS5(JL,JK)*(1.0_JPRB-REPH2O)) THEN
      ZQS (JL,JK) = ZQS (JL,JK) + ZQ (JL,JK)*(1.0_JPRB-REPH2O)
    ELSEIF (PQ5(JL,JK) > 2.0_JPRB*REPH2O) THEN
      PQ (JL,JK) = PQ (JL,JK) + ZQ (JL,JK)
    ENDIF
    ZQ (JL,JK) = 0.0_JPRB

    IF (PQS5(JL,JK) > 2.0_JPRB*REPH2O) THEN
      PQS (JL,JK) = PQS (JL,JK) + ZQS (JL,JK)
    ENDIF
    ZQS (JL,JK) = 0.0_JPRB
    
    IF (PCLFR5(JL,JK) > 0.0_JPRB .OR. PCLFR5(JL,JK) < 1.0_JPRB) THEN
      PCLFR (JL,JK) = PCLFR (JL,JK) + ZCLFR (JL,JK)
    ENDIF
    ZCLFR (JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.     HALF-LEVEL QUANTITIES

DO JL=KIDIA,KFDIA
  PTS(JL)=PTS(JL)+PTH (JL,KLEV+1)
  PTH (JL,KLEV+1)=0.0_JPRB

  PT (JL,1)=PT (JL,1)+PTH (JL,1)
  PAPRSF (JL,1)=PAPRSF (JL,1)&
   & -PTH (JL,1)*(PT5(JL,1)-PTH5(JL,2))         &
   & /(PAPRSF5(JL,1)-PAPRS5(JL,2))    
  PT (JL,1)=PT (JL,1)-PTH (JL,1)*PAPRSF5(JL,1)/(PAPRSF5(JL,1)-PAPRS5(JL,2))
  PTH (JL,2)=PTH (JL,2)+PTH (JL,1)*PAPRSF5(JL,1)/(PAPRSF5(JL,1)-PAPRS5(JL,2))
  PAPRSF (JL,1)=PAPRSF (JL,1)+PTH (JL,1)&
   & *PAPRSF5(JL,1)*(PT5(JL,1)-PTH5(JL,2))&
   & /((PAPRSF5(JL,1)-PAPRS5(JL,2))**2)  
  PAPRS (JL,2)=PAPRS (JL,2)-PTH (JL,1)&
   & *PAPRSF5(JL,1)*(PT5(JL,1)-PTH5(JL,2))&
   & /((PAPRSF5(JL,1)-PAPRS5(JL,2))**2)  
  PTH (JL,1)=0.0_JPRB
ENDDO

DO JK=KLEV,2,-1
  DO JL=KIDIA,KFDIA
    Z1S=0.0_JPRB
    Z2S=0.0_JPRB
    Z3S=0.0_JPRB

    PT (JL,JK-1)=PT (JL,JK-1)+PTH (JL,JK)*Z1S5(JL,JK)/Z3S5(JL,JK)
    PT (JL,JK)=PT (JL,JK)+PTH (JL,JK)*Z2S5(JL,JK)/Z3S5(JL,JK)
    Z1S=Z1S+PTH (JL,JK)*PT5(JL,JK-1)/Z3S5(JL,JK)
    Z2S=Z2S+PTH (JL,JK)*PT5(JL,JK)/Z3S5(JL,JK)
    Z3S=Z3S-PTH (JL,JK)*(PT5(JL,JK-1)*Z1S5(JL,JK)&
     & +PT5(JL,JK)*Z2S5(JL,JK))/Z3S5(JL,JK)**2  
    PTH (JL,JK)=0.0_JPRB

    PAPRS (JL,JK)=PAPRS (JL,JK)+Z3S*(PAPRSF5(JL,JK)-PAPRSF5(JL,JK-1))
    PAPRSF (JL,JK)=PAPRSF (JL,JK)+Z3S*PAPRS5(JL,JK)
    PAPRSF (JL,JK-1)=PAPRSF (JL,JK-1)-Z3S*PAPRS5(JL,JK)

    PAPRSF (JL,JK)=PAPRSF (JL,JK)+Z2S*(PAPRS5(JL,JK)-PAPRSF5(JL,JK-1))
    PAPRS (JL,JK)=PAPRS (JL,JK)+Z2S*PAPRSF5(JL,JK)
    PAPRSF (JL,JK-1)=PAPRSF (JL,JK-1)-Z2S*PAPRSF5(JL,JK)

    PAPRSF (JL,JK-1)=PAPRSF (JL,JK-1)+Z1S*(PAPRSF5(JL,JK)-PAPRS5(JL,JK))
    PAPRSF (JL,JK)=PAPRSF (JL,JK)+Z1S*PAPRSF5(JL,JK-1)
    PAPRS (JL,JK)=PAPRS (JL,JK)-Z1S*PAPRSF5(JL,JK-1)
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('RADINAAD',1,ZHOOK_HANDLE)
END SUBROUTINE RADINAAD
