SUBROUTINE RADOZCMF ( KIDIA , KFDIA , KLON , KLEV,&
 & PAPRS , PGEMU, POZON )

!**** *RADOZCMF* - COMPUTES DISTRIBUTION OF OZONE FROM CLIMATOLOGY

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     F. BOUYSSEL  Meteo-France    2008/10/01
!     Adaptation of radozc to work on rotated and stretched geometry

!     MODIFICATIONS.
!     --------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEOZOC  , ONLY : RSINC    ,ROZT     ,RPROC

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POZON(KLON,KLEV) 
!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZOZLT(KLON,0:35), ZOZON(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZRRR(KLON,0:34), ZSILAT(KLON), ZSIN(KLON)

INTEGER(KIND=JPIM) :: INLA(KLON)
INTEGER(KIND=JPIM) :: JC, JK, JL, I1, I2

REAL(KIND=JPRB) :: ZPMR, Z1, Z2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------
!     ------------------------------------------------------------------
!     ------------------------------------------------------------------

!*         1.     LATITUDE INDEX WITHIN OZONE CLIMATOLOGY
!                 ---------------------------------------

IF (LHOOK) CALL DR_HOOK('RADOZCMF',0,ZHOOK_HANDLE)

DO JL=KIDIA,KFDIA
  ZSIN(JL)=PGEMU(JL)
  INLA(JL)=0
  ZSILAT(JL)=0._JPRB
  DO JC=18,1,-1
    I1=INT(MAX(0._JPRB,SIGN(1._JPRB,RSINC(JC+1)-ZSIN(JL))))
    I2=INT(MAX(0._JPRB,SIGN(1._JPRB,ZSIN(JL)-RSINC(JC))))
    INLA(JL)=MAX(INLA(JL),I1*I2*JC)
  ENDDO
  ZSILAT(JL)=(ZSIN(JL)-RSINC(INLA(JL)))/(RSINC(INLA(JL)+1)-RSINC(INLA(JL)))
  IF (INLA(JL) == 0) CALL ABOR1(' Problem with lat. interpolation in radozcmf!')
ENDDO

!     ------------------------------------------------------------------

!*         2.     LATITUDE INTERPOLATED FIELD
!                 ---------------------------

DO JL=KIDIA,KFDIA
  Z1=REAL(INLA(JL))
  Z2=MIN(ABS((Z1-1._JPRB)*(Z1-18._JPRB)),1.0_JPRB)
  I1=INLA(JL)
  DO JC=0,35
    ZOZLT(JL,JC)=(1.0_JPRB-Z2)*ROZT(I1,JC) &
     & + Z2*(ROZT(I1,JC)+ZSILAT(JL)*(ROZT(I1+1,JC)-ROZT(I1,JC)))
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         3.     VERTICAL INTERPOLATION 
!                 ----------------------

DO JL=KIDIA,KFDIA
  DO JC=0,34
    ZRRR(JL,JC)=(1.0_JPRB/(RPROC(JC)-RPROC(JC+1)))*(ZOZLT(JL,JC)-ZOZLT(JL,JC+1))
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  DO JC=0,34
    DO JK=1,KLEV+1
      ZPMR=PAPRS(JL,JK)
      IF(ZPMR >= RPROC(JC).AND.ZPMR < RPROC(JC+1)) &
       & ZOZON(JL,JK)=ZOZLT(JL,JC+1)+(ZPMR-RPROC(JC+1))*ZRRR(JL,JC)  
    ENDDO
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  DO JK=1,KLEV+1
    ZPMR=PAPRS(JL,JK)
    ZPMR=PAPRS(JL,JK)
    IF(ZPMR >= RPROC(35)) ZOZON(JL,JK)=ZOZLT(JL,35)
  ENDDO
ENDDO

! INTEGRATION IN THE VERTICAL:
DO JL=KIDIA,KFDIA
  DO JK=1,KLEV
    POZON(JL,JK)=(PAPRS(JL,JK+1)-PAPRS(JL,JK))&
     & *(ZOZON(JL,JK)+ZOZON(JL,JK+1))*0.5_JPRB  
  ENDDO
ENDDO

!     -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADOZCMF',1,ZHOOK_HANDLE)
END SUBROUTINE RADOZCMF
