SUBROUTINE RESTORE_VDFOUT(KJ, KJREF, KSTART,KSTEP,&
           & KIDIA,KFDIA,KLON,KLEV,KLEVS,KTILES,KSTGLO,&
           & PWSA,PTSA, PAZ0M,PAZ0H, &
           & PVDIS,PVDISG, PDISSGW, PFTLHEV, PFTLHSB, PFWSB,PBIR,PVAR, &
           & PUCFL,PVCFL,PTCFL,PDCFL,PQCFL,PZINV,PBLH,KHPBLN, &
           & PU10N,PV10N,PUST, &
           & PFRSOTI,PEVAPSNW,PGUST, PWUAVG,LDNODECP,KPBLTYPE, &
           & PTENT,PTENQ,PTENL,PTENI,PTENA,PTENU,PTENV, &
           & PTLE1, PTL, PT,PQ,&
           & PUSTRTI,PVSTRTI,PAHFSTI,PEVAPTI,PTSKTI, &
           & PDIFTS,PDIFTQ,PDIFTL,PDIFTI,PSTRTU,PSTRTV,PTOFDU,PTOFDV, &
           & PSTRDU,PSTRDV,PKH_VDF)
!     ------------------------------------------------------------------

!**   *RESTORE_VDFOUT* - FOR PERTURBED RUN OF THE SEKF SURFACE ANALYSIS 

!     P. DE ROSNAY   E.C.M.W.F.    02 FEBRUARY 2009

!     PURPOSE
!     -------

!     RESTORE SAVED VARIABLES TO BE USED BY VDFMAIN WHEN PERFORMING PERTURBED
!     RUNS FOR THE EKF SURFACE ANALYSIS

!     INTERFACE
!     ---------

!     *RESTORE_VDFOUT* IS CALLED BY *CALLPAR*

!     INPUT PARAMETERS (INTEGER):

!
! KSTART  : FIRST STEP OF MODEL
! KSTEP   : CURRENT TIME STEP
!


!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KLEV*         NUMBER OF LEVELS
!     *KLEVS*        NUMBER OF LEVELS IN SOIL
!     *KLEV*         END OF VERTICAL LOOP AND VERTICAL DIMENSION
!     *KTILES*       NUMBER OF VEGETATION (SURFACE COVER) TYPES
!     *KSTGLO*       GLOBAL OFFSET (USED IN MESSAGE PASSING VERSION)
!! KDHVTLS : Number of variables for individual tiles
!! KDHFTLS : Number of fluxes for individual tiles
!! KDHVTSS : Number of variables for snow energy budget
!! KDHFTSS : Number of fluxes for snow energy budget
!! KDHVTTS : Number of variables for soil energy budget
!! KDHFTTS : Number of fluxes for soil energy budget
!! KDHVTIS : Number of variables for sea ice energy budget
!! KDHFTIS : Number of fluxes for sea ice energy budget


!     OUTPUT PARAMETERS (REAL):
! PWSA       : MULTI-LAYER SOIL WETNESS
! PTSA       : MULTI-LAYER SOIL TEMPERATURE

! PVDIS      : TURBULENT DISSIPATION
! PVDISG     : GRAVITY WAVE DRAG DISSIPATION
! PFTLHEV    : SURFACE LATENT HEAT FLUX (SNOW FREE FRACTION)
! PFTLHSB    : SURFACE LATENT HEAT FLUX (SNOW COVERED FRACTION)
! PFWSB      : SURFACE WATER VAPOUR FLUX (SUBLIMATION)
! PUCFL      : U-COMPONENT OF WIND AT 10 METERS (DIAGNOSTIC).
! PVCFL      : V-COMPONENT OF WIND AT 10 METERS (DIAGNOSTIC).
! PTCFL      : TEMPERATURE AT 2 METERS (DIAGNOSTIC).
! PDCFL      : DEW POINT TEMPERATURE AT 2 METERS (DIAGNOSTIC).
! PQCFL      : SPECIFIC HUMIDITY AT 2 METERS (DIAGNOSTIC).
! PBLH       : BOUNDARY LAYER HEIGHT
! PU10N      : U-COMPONENT OF NEUTRAL WIND AT 10 METERS (DIAGNOSTIC).
! PV10N      : V-COMPONENT OF NEUTRAL WIND AT 10 METERS (DIAGNOSTIC).
! PUST       : FRICTION VELOCITY (DIAGNOSTIC).
! PTENT      : TENDENCY OF TEMPERATURE.
! PTENQ      : TENDENCY OF HUMIDITY
! PTENU      : TENDENCY OF U-COMP. OF WIND.
! PTENV      : TENDENCY OF V-COMP. OF WIND.
! PTLE1      : OF SKIN TEMPERATURE
! PTL        : SKIN TEMPERATURE
! PT         : ATMOSPHERIC TEMPERATURE PROFILE
! PQ         : ATMOSPHERIC SPECIFIC HUMIDITY PROFILE
! PUSTRTI    : E-W (INSTANTANEOUS) SURFACE STRESS FOR EACH TILE
! PVSTRTI    : N-S (INSTANTANEOUS) SURFACE STRESS FOR EACH TILE
! PAHFSTI    : (INSTANTANEOUS) SURFACE SENSIBLE HEAT FLUX FOR EACH TILE
! PEVAPTI       : (INSTANTANEOUS) EVAPORATION FOR EACH TILE
! PTSKTI     : SKIN TEMPERATURE FOR EACH TILE
! PDIFTS     : TURBULENT FLUX OF ENTHALPY (OR DRY STATIC ENERGY).
! PDIFTQ     : TURBULENT FLUX (INC. Q NEGATIVE) OF SPECIFIC HUMIDITY.
! PDIFTL     : TURBULENT FLUX (INC. Q NEGATIVE) OF LIQUID WATER.
! PDIFTI     : TURBULENT FLUX (INC. Q NEGATIVE) OF SOLID WATER.
! PSTRTU     : TURBULENT FLUX OF MOMENTUM "U".
! PSTRTV     : TURBULENT FLUX OF MOMENTUM "V".
! PSTRDU     : GRAVITY WAVE DRAG FLUX "U".
! PSTRDV     : GRAVITY WAVE DRAG FLUX "V".
! ZKH_VDF    : TURBULENT DIFFUSION COEFFICIENT FOR HEAT
!! PDHTLS     : Diagnostic array for tiles (see module yomcdh)
!! PDHTSS     : Diagnostic array for snow T (see module yomcdh)
!! PDHTTS     : Diagnostic array for soil T (see module yomcdh)
!! PDHTIS     : Diagnostic array for ice T (see module yomcdh)





!     INTPUT PARAMETERS (REAL):


!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMPHY2  , ONLY : TSPHY


USE YOMSEKF  , ONLY : SKF_ZWSA,SKF_ZTSA, SKF_ZAZ0M,SKF_ZAZ0H, &
!   & SKF_PVDIS,SKF_PVDISG, SKF_ZDISSGW, SKF_PFTLHEV, SKF_PFTLHSB,&
!   & SKF_PFWSB,SKF_ZBIR,SKF_ZVAR, &
!   & SKF_PUCFL,SKF_PVCFL,SKF_PTCFL,SKF_PDCFL,&
!   & SKF_PQCFL,SKF_ZZINV,SKF_PBLH,ISKF_IHPBLN, &
!   & SKF_PU10N,SKF_PV10N,SKF_PUST,&
   & SKF_ZFRSOTI,&
!  & SKF_ZEVAPSNW,SKF_ZGUST,&
!   & SKF_ZWUAVG,LSKF_LLNODECP,ISKF_IPBLTYPE, &
   & SKF_PTENT,SKF_PTENQ,SKF_ZTENL,SKF_ZTENI,&
   & SKF_ZTENA,SKF_PTENU,SKF_PTENV, SKF_PTLE1, SKF_PTL,&
   & SKF_PTENT_CUM,SKF_PTENQ_CUM, &
   & SKF_PUSTRTI,SKF_PVSTRTI,SKF_PAHFSTI,SKF_PEVAPTI,SKF_PTSKTI, &
!   & SKF_PDIFTS,SKF_PDIFTQ,SKF_PDIFTL,SKF_PDIFTI,SKF_PSTRTU,&
!   & SKF_PSTRTV,&
   & SKF_PTOFDU,SKF_PTOFDV
!   & SKF_PSTRDU,SKF_PSTRDV,SKF_zkh_vdf 



IMPLICIT NONE

! INPUT PARAMETERS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KJ
INTEGER(KIND=JPIM),INTENT(IN)    :: KJREF
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO


! OUTPUT VARIABLES (RESTORED)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PWSA(KLON,KLEVS)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PTSA(KLON,KLEVS)

INTEGER(KIND=JPIM),INTENT(INOUT)    :: KHPBLN(KLON),KPBLTYPE(KLON)
LOGICAL,INTENT(INOUT)               :: LDNODECP(KLON)

REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PAZ0M (KLON),PAZ0H(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PVDIS(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PVDISG(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PDISSGW(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PFTLHEV(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PFTLHSB(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PFWSB(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PBIR(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PVAR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PUCFL(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PVCFL(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTCFL(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PDCFL(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PQCFL(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZINV(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PBLH(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PU10N(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PV10N(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PUST(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PFRSOTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PEVAPSNW(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PGUST(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PWUAVG(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENA(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENI(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENL(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTENV(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTLE1(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTL(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PUSTRTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PVSTRTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PAHFSTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PEVAPTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTSKTI(KLON,KTILES)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PDIFTS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PDIFTQ(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PDIFTL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PDIFTI(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PSTRTU(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PSTRTV(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTOFDU(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTOFDV(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PSTRDU(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PSTRDV(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PKH_VDF(KLON,KLEV)   


! LOCAL VARIABLE 
REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTEGER(KIND=JPIM)  :: JL, IBLK
INTEGER(KIND=JPIM)  :: JK_nopert,JK_nopert_previous_step
INTEGER(KIND=JPIM)  :: IBLKpt,JLpt

IF (LHOOK) CALL DR_HOOK('RESTORE_VDFOUT',0,ZHOOK_HANDLE)

JK_nopert = 1_JPIM  ! from JKSTEP in callpar
JK_nopert_previous_step = 5_JPIM  ! stored in previous time step
IBLKpt = 52_JPIM
JLpt = 53_JPIM

! RESTORE VARIABLES of the KJth run
! KJ=1 : Coupled / non perturbed
! KJ=2 : Offline / perturbe SM layer1
! KJ=3 : Offline / perturbe SM layer2
! KJ=4 : Offline / perturbe SM layer3
! KJ=5 : Offline / non perturbed 

IBLK=(KSTGLO-1)/KLON + 1

!IF(IBLK.eq.52) THEN
!write(34,*) 'RESTORE_VDF: Pt 53-52 input',KJ
!write(34,*) 'PWSA ', PWSA(JLpt,:),PTCFL(JLpt),PDCFL(JLpt)
!write(34,*) 'SKF_WSA ', SKF_ZWSA(JLpt,IBLK,:,KJ),SKF_PTCFL(JLpt,IBLK,KJ),SKF_PTL(JLpt,IBLK,KJ)
!ENDIF
! vdfmain's output variables
! Restore when they exist (not for KSTART)
! vdfmain's out variables and in/out perturbed variables
! in: PWSA,PTSA, PTL
! out: PTCFL, PDCFL
! inout: PTLE1

IF(KSTEP /= KSTART) THEN
  DO JL=KIDIA,KFDIA

! In perturbed
    PWSA(JL,1:KLEVS) = SKF_ZWSA(JL,IBLK,1:KLEVS,KJ)
    PTSA(JL,1:KLEVS) = SKF_ZTSA(JL,IBLK,1:KLEVS,KJ)
! Vertical atmospheric profiles of T and Q:
! propagate perturb from previous step as done in sltend2.F90 for non perturbed case:
IF(KJ /= JK_nopert) THEN
    PT(JL,:) = PT(JL,:) + &
     &    TSPHY*(-SKF_PTENT_CUM(JL,IBLK,1:KLEV,JK_nopert_previous_step)+SKF_PTENT_CUM(JL,IBLK,1:KLEV,KJ))
    PQ(JL,:) = PQ(JL,:) + &
     &  TSPHY*(-SKF_PTENQ_CUM(JL,IBLK,1:KLEV,JK_nopert_previous_step)+SKF_PTENQ_CUM(JL,IBLK,1:KLEV,KJ))

    PTENT(JL,1:KLEV) = SKF_PTENT(JL,IBLK,1:KLEV,KJREF) + & 
!                !      PTENT(JL,1:KLEV) + &
               &   (-SKF_PTENT(JL,IBLK,1:KLEV,JK_nopert_previous_step)+SKF_PTENT(JL,IBLK,1:KLEV,KJ))
!
    PTENQ(JL,1:KLEV) = SKF_PTENQ(JL,IBLK,1:KLEV,KJREF) + & 
!                !  PTENQ(JL,1:KLEV) + &
               &  (-SKF_PTENQ(JL,IBLK,1:KLEV,JK_nopert_previous_step)+SKF_PTENQ(JL,IBLK,1:KLEV,KJ))
ENDIF
IF(KJ == JK_nopert) THEN
    PTENT(JL,1:KLEV) = SKF_PTENT(JL,IBLK,1:KLEV,KJREF) 
    PTENQ(JL,1:KLEV) = SKF_PTENQ(JL,IBLK,1:KLEV,KJREF) 
ENDIF
! out perturbed
! not necessary for out variables (provided by last perturbed loop to continue callpar)
!    PTCFL(JL) = SKF_PTCFL(JL,IBLK,KJ)
!    PDCFL(JL) = SKF_PDCFL(JL,IBLK,KJ)

! out others 
!    KHPBLN(JL) = ISKF_IHPBLN(JL,IBLK,KJREF)
!    KPBLTYPE(JL) = ISKF_IPBLTYPE(JL,IBLK,KJREF)
!    LDNODECP(JL) = LSKF_LLNODECP(JL,IBLK,KJREF)
!    PVDIS(JL)  = SKF_PVDIS(JL,IBLK,KJREF)
!    PVDISG(JL)  = SKF_PVDISG(JL,IBLK,KJREF)
!    PDISSGW(JL,KLEV)= SKF_ZDISSGW(JL,IBLK,KLEV,KJREF)
!    PFTLHEV(JL)    = SKF_PFTLHEV(JL,IBLK,KJREF)
!    PFTLHSB(JL)    = SKF_PFTLHSB(JL,IBLK,KJREF)
!    PFWSB(JL) = SKF_PFWSB(JL,IBLK,KJREF)
!    PBIR(JL) = SKF_ZBIR(JL,IBLK,KJREF)
!    PVAR(JL,1:KLEV) = SKF_ZVAR(JL,IBLK,1:KLEV,KJREF)
!    PUCFL(JL) = SKF_PUCFL(JL,IBLK,KJREF)
!    PVCFL(JL) = SKF_PVCFL(JL,IBLK,KJREF)
!    PQCFL(JL) = SKF_PQCFL(JL,IBLK,KJREF)
!    PZINV(JL) = SKF_ZZINV(JL,IBLK,KJREF)
!    PBLH(JL) = SKF_PBLH(JL,IBLK,KJREF)
!    PU10N(JL) = SKF_PU10N(JL,IBLK,KJREF) = PU10N(JL) 
!    PV10N(JL) = SKF_PV10N(JL,IBLK,KJREF) = PV10N(JL) 
!    PUST(JL) = SKF_PUST(JL,IBLK,KJREF) = PUST(JL) 
!    PEVAPSNW(JL) = SKF_ZEVAPSNW(JL,IBLK,KJREF)
!    PGUST(JL) = SKF_ZGUST(JL,IBLK,KJREF)
!    PWUAVG(JL) = SKF_ZWUAVG(JL,IBLK,KJREF)
!    PDIFTS(JL,0:KLEV) = SKF_PDIFTS(JL,IBLK,0:KLEV,KJREF)
!    PDIFTQ(JL,0:KLEV) = SKF_PDIFTQ(JL,IBLK,0:KLEV,KJREF)
!    PDIFTL(JL,0:KLEV) = SKF_PDIFTL(JL,IBLK,0:KLEV,KJREF)
!    PDIFTI(JL,0:KLEV) = SKF_PDIFTI(JL,IBLK,0:KLEV,KJREF)
!    PSTRTU(JL,0:KLEV) = SKF_PSTRTU(JL,IBLK,0:KLEV,KJREF)
!    PSTRTV(JL,0:KLEV) = SKF_PSTRTV(JL,IBLK,0:KLEV,KJREF)
!    PSTRDU(JL,0:KLEV) = SKF_PSTRDU(JL,IBLK,0:KLEV,KJREF)
!    PSTRDV(JL,0:KLEV) = SKF_PSTRDV(JL,IBLK,0:KLEV,KJREF)
!    PKH_VDF(JL,1:KLEV) = SKF_ZKH_VDF(JL,IBLK,1:KLEV,KJREF)
  ENDDO
ENDIF


! vdfmain's inout variables
! Allways restore inout variables from the reference
DO JL=KIDIA,KFDIA
  PAZ0M(JL)  = SKF_ZAZ0M(JL,IBLK,KJREF)
  PAZ0H(JL)  = SKF_ZAZ0H(JL,IBLK,KJREF)
  PTENA(JL,1:KLEV) = SKF_ZTENA(JL,IBLK,1:KLEV,KJREF)
  PTENI(JL,1:KLEV) = SKF_ZTENI(JL,IBLK,1:KLEV,KJREF)
  PTENL(JL,1:KLEV) = SKF_ZTENL(JL,IBLK,1:KLEV,KJREF)
  PTENU(JL,1:KLEV) = SKF_PTENU(JL,IBLK,1:KLEV,KJREF)
  PTENV(JL,1:KLEV) = SKF_PTENV(JL,IBLK,1:KLEV,KJREF)
IF(KSTEP == KSTART) THEN
  PTENQ(JL,1:KLEV) = SKF_PTENQ(JL,IBLK,1:KLEV,KJREF)
  PTENT(JL,1:KLEV) = SKF_PTENT(JL,IBLK,1:KLEV,KJREF)
ENDIF
  PTOFDU(JL) = SKF_PTOFDU(JL,IBLK,KJREF)
  PTOFDV(JL) = SKF_PTOFDV(JL,IBLK,KJREF)
!inout that could be just out:
    PFRSOTI(JL,1:KTILES) = SKF_ZFRSOTI(JL,IBLK,1:KTILES,KJREF)
  ! for perturbed variables restore ref value only for KSTART 
! inout perturbed
  IF(KSTEP /= KSTART) THEN
    PTSKTI(JL,1:KTILES) = SKF_PTSKTI(JL,IBLK,1:KTILES,KJ)
    PTL(JL) = SKF_PTL(JL,IBLK,KJ)
    PEVAPTI(JL,1:KTILES) = SKF_PEVAPTI(JL,IBLK,1:KTILES,KJ)
    PUSTRTI(JL,1:KTILES) = SKF_PUSTRTI(JL,IBLK,1:KTILES,KJ)
    PVSTRTI(JL,1:KTILES) = SKF_PVSTRTI(JL,IBLK,1:KTILES,KJ)
    PAHFSTI(JL,1:KTILES) = SKF_PAHFSTI(JL,IBLK,1:KTILES,KJ)
    PTLE1(JL) = SKF_PTLE1(JL,IBLK,KJ)
    PEVAPTI(JL,1:KTILES) = SKF_PEVAPTI(JL,IBLK,1:KTILES,KJ)
  ELSE
    PTSKTI(JL,1:KTILES) = SKF_PTSKTI(JL,IBLK,1:KTILES,KJREF)
    PTL(JL) = SKF_PTL(JL,IBLK,KJREF)
    PEVAPTI(JL,1:KTILES) = SKF_PEVAPTI(JL,IBLK,1:KTILES,KJREF)
    PUSTRTI(JL,1:KTILES) = SKF_PUSTRTI(JL,IBLK,1:KTILES,KJREF)
    PVSTRTI(JL,1:KTILES) = SKF_PVSTRTI(JL,IBLK,1:KTILES,KJREF)
    PAHFSTI(JL,1:KTILES) = SKF_PAHFSTI(JL,IBLK,1:KTILES,KJREF)
    PTLE1(JL) = SKF_PTLE1(JL,IBLK,KJREF)
    PEVAPTI(JL,1:KTILES) = SKF_PEVAPTI(JL,IBLK,1:KTILES,KJREF)

  ENDIF

ENDDO



IF (LHOOK) CALL DR_HOOK('RESTORE_VDFOUT',1,ZHOOK_HANDLE)
END SUBROUTINE RESTORE_VDFOUT
