SUBROUTINE RNDECAY ( KIDIA,KFDIA,KLON,KTDIA,KLEV,PGHG,PTENGHG)  

!**** *RNDECAY* RADIOACTIVE DECAY OF RADON

!     PURPOSE.
!     -------
!     - CALCULATION OF Radon FLUX AND TENDENCY
!       DUE TO RADIOACTIVE DECAY
!       (MIXING RATIO RELATIVE TO MASS)

!**   Interface.
!     ----------
!        *CALL* *RNDECAY*

!-----------------------------------------------------------------------

! -   INPUT ARGUMENTS.
!     -------------------

! - DIMENSIONS ETC.

!       KIDIA     : START OF HORIZONTAL LOOP
!       KFDIA     : END   OF HORIZONTAL LOOP
!       KLON      : HORIZONTAL DIMENSION
!       KTDIA     : START OF THE VERTICAL LOOP IN THE PHYSICS
!                   (IF SOME LEVELS ARE SKIPPED AT THE TOP OF THE MODEL).
!       KLEV      : END OF VERTICAL LOOP AND VERTICAL DIMENSION

! - 2D (1:KLEV) .

!       PGHG      : MASS MIXING RATIO OF RADON

! -   UPDATED ARGUMENTS.
!     -----------------
! - 2D (1:KLEV) TENDENCY .
!       PTENGHG    :  RADIOACTIVE TENDENCY FOR RADON

!-----------------------------------------------------------------------

!     Externals.
!     ---------


!     Author
!    --------
!         05-04-08, R. Engelen 

!     Modifications :
!    ----------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY2  , ONLY : TSPHY
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGHG(KLON,KLEV,1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENGHG(KLON,KLEV,1) 
REAL(KIND=JPRB) :: ZANEX(KLON,KLEV)

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZDECAY
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*
!     ------------------------------------------------------------------

!         PRELIMINARY COMPUTATIONS AND VARIOUS INITIALIZATIONS

IF (LHOOK) CALL DR_HOOK('RNDECAY',0,ZHOOK_HANDLE)

ZDECAY=2.11E-06_JPRB

!*   COMPUTATION OF MIXING RATIO AT TIME T+DT

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZANEX(JLON,JLEV)=EXP(-TSPHY*ZDECAY)*PGHG(JLON,JLEV,1)  
  ENDDO
ENDDO

!*   COMPUTATION OF THE ACTUAL TENDENCY

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZANEX(JLON,JLEV)=(ZANEX(JLON,JLEV)-PGHG(JLON,JLEV,1))/TSPHY
!    UPDATE RADON TENDENCY            
    PTENGHG(JLON,JLEV,1)=PTENGHG(JLON,JLEV,1)+ZANEX(JLON,JLEV)
!    PTENGHG(JLON,JLEV,1)=-MIN(-PTENGHG(JLON,JLEV,1)*TSPHY,PGHG(JLON,JLEV,1))/TSPHY
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('RNDECAY',1,ZHOOK_HANDLE)

END SUBROUTINE RNDECAY
