SUBROUTINE STORE_SURFTSTP(KJ, KJREF,KSTART,KSTEP,&
           & KIDIA,KFDIA,KLON,KLEV,KLEVS,KLEVO,KSTGLO,LDOCN_KPP,&
           & PWSA,PTSA,PZO,PHO,PHO_INV,PDO,PADVT,PADVS,PTRI0,PTRI1,PSWDK_SAVE,&
           & PUSTRC,PVSTRC,&
           & PUO0,PVO0,PUOC,PVOC,PTO0,PSO0,&
           & PFTG12,PFWROD,PFWRO1,PFWG12,PFWMLT,PFWEV,PENES,&
           & PDIFM, PDIFT, PDIFS,&
           & PSNSE1,PTSNE1,PASNE1,PRSNE1,PTSAE1,PTIAE1,&
           & PWLE1,PWSAE1,PTLICEE1,PTLMNWE1,PTLWMLE1,PTLBOTE1,PTLSFE1,&
           & PHLICEE1,PHLMLE1,PUOE1,PVOE1,PTOE1,PSOE1)

!     ------------------------------------------------------------------

!**   *STORE_SURFTSTP* - FOR PERTURBED RUN OF THE SEKF SURFACE ANALYSIS 

!     P. DE ROSNAY   E.C.M.W.F.    12 FEBRUARY 2009

!     PURPOSE
!     -------

!     STORE VARIABLES OF THE CURRENT KJth perturbed RUN, 
!     IN ORDER TO USE THEM IN SURFTSTP WHEN PERFORMING PERTURBED
!     RUNS FOR THE EKF SURFACE ANALYSIS
!     VARIABLES WILL BE RESTORED AT THE NEXT TIME STEP BY RESTORE_SURFTSTP

!     INTERFACE
!     ---------

!     *STORE_SURFTSTP* IS CALLED BY *CALLPAR*

!     INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KLEV*         NUMBER OF LEVELS
!     *KLEVS*        NUMBER OF LEVELS IN SOIL
!     *KLEVO*        Number of ocean vertical layers
!     *KLEV*         END OF VERTICAL LOOP AND VERTICAL DIMENSION
!     *KSTGLO*       GLOBAL OFFSET (USED IN MESSAGE PASSING VERSION)
!     *LDOCN_KPP*    ocean coupling (cf surf/module/surfbc_ctl_mod.F90)


!     INPUT PARAMETERS (REAL):
! KSTART  : FIRST STEP OF MODEL
! KSTEP   : CURRENT TIME STEP
! PWSA       : MULTI-LAYER SOIL WETNESS
! PTSA       : MULTI-LAYER SOIL TEMPERATURE

! PZO,PHO,PHO_INV,PDO,PADVT,PADVS,PTRI0,PTRI1,PSWDK_SAVE,PUO0,PVO0,PTO0,PSO0 : Ocean model

! PFTG12     UPWARD FLUX BETWEEN SURFACE AND DEEP LAYER   W/M**2
! PFWROD      DEEP LAYER RUN-OFF                          kg/m**2/s
! PFWRO1      SURFACE RUN-OFF                             kg/m**2/s
! PFWG12      WATER FLUX BETWEEN LAYER 1 AND 2            kg/m**2/s
! PFWMLT      WATER FLUX CORRESPONDING TO SNOW MELT       kg/m**2/s
! PFWEV      EVAPORATION OVER LAND SURFACE               kg/m**2/s
! PENES      SOIL ENERGY per unit area                   J/M**2
! KPP OCEAN MODEL and FLAKE
! PDIFM
! PDIFT
! PDIFS
! PTLICEE1,PTLMNWE1,PTLWMLE1,PTLBOTE1,PTLSFE1&
! PHLICEE1,PHLMLE1,PUOE1,PVOE1,PTOE1,PSOE1
! PDHTSS     Diagnostic array for snow T (see module yomcdh)
! PDHTTS     Diagnostic array for soil T (see module yomcdh)
! PDHTIS     Diagnostic array for ice T (see module yomcdh)
! PDHSSS     Diagnostic array for snow mass (see module yomcdh)
! PDHIIS     Diagnostic array for interception layer (see module yomcdh)
! PDHWLS     Diagnostic array for soil water (see module yomcdh)



! OUTPUT PARAMETERS ARE STORED IN YOMSEKF FOR THE NEXT TIME STEP, IN THE SKF_* VARIABLES

!     Modifications.
!     --------------
!     P de Rosnay 20-10-2009 separate out and inout storage and different cases 
!                            of initialization from ocean_ml_driver_mod.F90


!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
!USE YOEPHY    ,ONLY : LEOCML

USE YOMSEKF  , ONLY : &
!   & SKF_ZWSA,SKF_ZTSA, &
   & SKF_PZO,SKF_PHO,SKF_PHO_INV,SKF_PDO,SKF_PADVT,&
   & SKF_PADVS,SKF_PTRI0,SKF_PTRI1,SKF_PSWDK_SAVE,&
   & SKF_PUO0,SKF_PVO0,SKF_PTO0,SKF_PSO0,&
   & SKF_PUSTRC, SKF_PVSTRC,SKF_PUOC,SKF_PVOC, &
!   & SKF_PFTG12,SKF_PFWROD,&
    & SKF_PFWRO1,&
!   & SKF_PFWG12,SKF_PFWMLT,SKF_PFWEV,SKF_PENES,&
!   & SKF_PDIFM, SKF_PDIFT, SKF_PDIFS,&
   & SKF_PSNSE1,SKF_PTSNE1,SKF_PASNE1,SKF_PRSNE1,SKF_PTSAE1,SKF_PTIAE1,&
   & SKF_PWLE1,SKF_PWSAE1, & 
!  & SKF_PTLICEE1 & 
!   & ,SKF_PTLMNWE1,&
!   & SKF_PTLWMLE1,SKF_PTLBOTE1,SKF_PTLSFE1,&
!   & SKF_PHLICEE1,SKF_PHLMLE1,&
   & SKF_PUOE1,SKF_PVOE1,SKF_PTOE1,SKF_PSOE1



IMPLICIT NONE

! INPUT PARAMETERS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KJ
INTEGER(KIND=JPIM),INTENT(IN)    :: KJREF
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVO
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO
LOGICAL           ,INTENT(IN)    :: LDOCN_KPP(KLON) !KPP


! INPUT VARIABLES (TO STORE)

REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSA(KLON,KLEVS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSA(KLON,KLEVS)
REAL(KIND=JPRB), INTENT(IN)      :: PZO(KLON,KLEVO+1)            
REAL(KIND=JPRB), INTENT(IN)      :: PHO(KLON,KLEVO+1)           
REAL(KIND=JPRB), INTENT(IN)      :: PHO_INV(KLON,KLEVO+1)      
REAL(KIND=JPRB), INTENT(IN)      :: PDO(KLON,0:KLEVO)         
REAL(KIND=JPRB), INTENT(IN)      :: PADVT(KLON,KLEVO+1)      
REAL(KIND=JPRB), INTENT(IN)      :: PADVS(KLON,KLEVO+1)     
REAL(KIND=JPRB), INTENT(IN)      :: PTRI0(KLON,0:KLEVO)    
REAL(KIND=JPRB), INTENT(IN)      :: PTRI1(KLON,0:KLEVO)         
REAL(KIND=JPRB), INTENT(IN)      :: PSWDK_SAVE(KLON,0:KLEVO)     
REAL(KIND=JPRB), INTENT(IN)      :: PUO0(KLON,KLEVO+1)         
REAL(KIND=JPRB), INTENT(IN)      :: PVO0(KLON,KLEVO+1)        
REAL(KIND=JPRB), INTENT(IN)      :: PTO0(KLON,KLEVO+1)       
REAL(KIND=JPRB), INTENT(IN)      :: PSO0(KLON,KLEVO+1)      
REAL(KIND=JPRB),INTENT(IN)       :: PUOC(KLON,KLEVO+1)
REAL(KIND=JPRB),INTENT(IN)       :: PVOC(KLON,KLEVO+1)
REAL(KIND=JPRB),INTENT(IN)       :: PUSTRC(KLON)
REAL(KIND=JPRB),INTENT(IN)       :: PVSTRC(KLON)

REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTG12(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFWROD(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFWRO1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFWG12(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFWMLT(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFWEV(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENES(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFM(KLON,0:KLEVO+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFT(KLON,0:KLEVO+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFS(KLON,0:KLEVO+1)     
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNSE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSNE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PASNE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSNE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSAE1(KLON,KLEVS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTIAE1(KLON,KLEVS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWLE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSAE1(KLON,KLEVS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLICEE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLMNWE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLWMLE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLBOTE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLSFE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHLICEE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHLMLE1(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUOE1(KLON,KLEVO+1)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOE1(KLON,KLEVO+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOE1(KLON,KLEVO+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOE1(KLON,KLEVO+1)  



! LOCAL VARIABLE 

REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTEGER(KIND=JPIM)  :: JL, IBLK

IF (LHOOK) CALL DR_HOOK('STORE_SURFTSTP',0,ZHOOK_HANDLE)


! STORE VARIABLES of the KJth run
! KJ=1 : store/restore reference inout only
! KJ=2 : Offline / perturbe SM layer1
! KJ=3 : Offline / perturbe SM layer2
! KJ=4 : Offline / perturbe SM layer3
! KJ=5 : Offline / non perturbed


IBLK=(KSTGLO-1)/KLON + 1

! 1. Store surftstp's out variables:
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! Store after Kstep or for perturbed loops
! ie store in all case except for (KSTART.and.kj=1)

IF(KSTEP /= KSTART .OR. KJ > 1) THEN
  DO JL=KIDIA,KFDIA
! not out   SKF_ZWSA(JL,IBLK,1:KLEVS,KJ) = PWSA(JL,1:KLEVS)
!    SKF_ZTSA(JL,IBLK,1:KLEVS,KJ) = PTSA(JL,1:KLEVS)

!    SKF_PFTG12(JL,IBLK,KJREF) = PFTG12(JL)
!    SKF_PFWROD(JL,IBLK,KJREF) = PFWROD(JL)
!    SKF_PFWG12(JL,IBLK,KJREF) = PFWG12(JL)
!    SKF_PFWMLT(JL,IBLK,KJREF) = PFWMLT(JL)
!    SKF_PFWEV(JL,IBLK,KJREF) = PFWEV(JL)
!    SKF_PENES(JL,IBLK,:,KJREF) = PENES(JL,:)
!    SKF_PDIFM(JL,IBLK,:,KJREF) = PDIFM(JL,:)
!    SKF_PDIFT(JL,IBLK,:,KJREF) = PDIFT(JL,:)
!    SKF_PDIFS(JL,IBLK,:,KJREF) = PDIFS(JL,:)
!    SKF_PTLICEE1(JL,IBLK,KJREF) = PTLICEE1(JL)
!    SKF_PTLMNWE1(JL,IBLK,KJREF) = PTLMNWE1(JL)
!    SKF_PTLWMLE1(JL,IBLK,KJREF) = PTLWMLE1(JL)
!    SKF_PTLBOTE1(JL,IBLK,KJREF) = PTLBOTE1(JL)
!    SKF_PTLSFE1(JL,IBLK,KJREF) = PTLSFE1(JL)
!    SKF_PHLICEE1(JL,IBLK,KJREF) = PHLICEE1(JL)
!    SKF_PHLMLE1(JL,IBLK,KJREF) = PHLMLE1(JL)
!
! include inout variables that could be just out variables (diagnosed below surftstp):
!    SKF_PFWRO1(JL,IBLK,KJREF) = PFWRO1(JL)

  ENDDO
ENDIF


! 2. Store surftstp's inout variables already initialized in callpar:
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
DO JL=KIDIA,KFDIA
  SKF_PSNSE1(JL,IBLK,KJREF) = PSNSE1(JL)
  SKF_PTSNE1(JL,IBLK,KJREF) = PTSNE1(JL)
  SKF_PASNE1(JL,IBLK,KJREF) = PASNE1(JL)
  SKF_PRSNE1(JL,IBLK,KJREF) = PRSNE1(JL)
  SKF_PTIAE1(JL,IBLK,1:KLEVS,KJREF) = PTIAE1(JL,1:KLEVS)
  SKF_PWLE1(JL,IBLK,KJREF) = PWLE1(JL)
  SKF_PFWRO1(JL,IBLK,KJREF) = PFWRO1(JL)

  SKF_PTSAE1(JL,IBLK,1:KLEVS,KJREF) = PTSAE1(JL,1:KLEVS)
  SKF_PWSAE1(JL,IBLK,1:KLEVS,KJREF) = PWSAE1(JL,1:KLEVS)
ENDDO


! 3. Store surftstp's inout variables initialized below surftstp:
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! Skip the store of this variables for KSTART

IF(KSTEP /= KSTART .OR. KJ > 1) THEN
DO JL=KIDIA,KFDIA
! Take care not to store not computed variables (KPP not coupled points)
  IF( LDOCN_KPP(JL) ) THEN
    SKF_PZO(JL,IBLK,1:KLEVO+1,KJREF) = PZO(JL,1:KLEVO+1)
    SKF_PDO(JL,IBLK,1:KLEVO,KJREF) = PDO(JL,1:KLEVO)
    SKF_PHO(JL,IBLK,1:KLEVO+1,KJREF) = PHO(JL,1:KLEVO+1)
    SKF_PHO_INV(JL,IBLK,1:KLEVO+1,KJREF) = PHO_INV(JL,1:KLEVO+1)
    SKF_PUO0(JL,IBLK,1:KLEVO+1,KJREF) = PUO0(JL,1:KLEVO+1)
    SKF_PVO0(JL,IBLK,1:KLEVO+1,KJREF) = PVO0(JL,1:KLEVO+1)
    SKF_PTO0(JL,IBLK,1:KLEVO+1,KJREF) = PTO0(JL,1:KLEVO+1)
    SKF_PSO0(JL,IBLK,1:KLEVO+1,KJREF) = PSO0(JL,1:KLEVO+1)
    SKF_PUOE1(JL,IBLK,1:KLEVO+1,KJREF) = PUOE1(JL,1:KLEVO+1)
    SKF_PVOE1(JL,IBLK,1:KLEVO+1,KJREF) = PVOE1(JL,1:KLEVO+1)
    SKF_PTOE1(JL,IBLK,1:KLEVO+1,KJREF) = PTOE1(JL,1:KLEVO+1)
    SKF_PSOE1(JL,IBLK,1:KLEVO+1,KJREF) = PSOE1(JL,1:KLEVO+1)
    SKF_PADVT(JL,IBLK,1:KLEVO+1,KJREF) = PADVT(JL,1:KLEVO+1)
    SKF_PADVS(JL,IBLK,1:KLEVO+1,KJREF) = PADVS(JL,1:KLEVO+1)
    SKF_PTRI0(JL,IBLK,0:KLEVO,KJREF) = PTRI0(JL,0:KLEVO) 
    SKF_PTRI1(JL,IBLK,0:KLEVO,KJREF) = PTRI1(JL,0:KLEVO) 
    SKF_PSWDK_SAVE(JL,IBLK,0:KLEVO,KJREF) = PSWDK_SAVE(JL,0:KLEVO)
    SKF_PUOC(JL,IBLK,0:KLEVO,KJREF) = PUOC(JL,0:KLEVO) 
    SKF_PVOC(JL,IBLK,0:KLEVO,KJREF) = PVOC(JL,0:KLEVO) 
    SKF_PUSTRC(JL,IBLK,KJREF) = PUSTRC(JL) 
    SKF_PVSTRC(JL,IBLK,KJREF) = PVSTRC(JL) 
!  ELSE
!    SKF_PUOE1(JL,IBLK,1:KLEVO+1,KJREF) = 0.0_JPRB
!    SKF_PVOE1(JL,IBLK,1:KLEVO+1,KJREF) = 0.0_JPRB 
!    SKF_PTOE1(JL,IBLK,1:KLEVO+1,KJREF) = 0.0_JPRB 
!    SKF_PSOE1(JL,IBLK,1:KLEVO+1,KJREF) = 0.0_JPRB 
!    SKF_PSWDK_SAVE(JL,IBLK,0:KLEVO,KJREF) = 0.0_JPRB
  ENDIF
ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('STORE_SURFTSTP',1,ZHOOK_HANDLE)
END SUBROUTINE STORE_SURFTSTP
