SUBROUTINE SU_CLOP550 

!**** *SU_CLOP550*  - INITIALIZE COMMON YOECLOP550

!     PURPOSE.
!     --------
!           INITIALIZE YOECLOP550, WITH CLOUD OPTICAL PARAMETERS

!**   INTERFACE.
!     ----------
!        *CALL*  SUCLOP550

!        EXPLICIT ARGUMENTS :
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS :
!        --------------------
!        COMMON YOECLOP550

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE
!     "INTEGRATED FORECASTING SYSTEM"

!     A. Slingo, 1989: J. Atmos. Sci., 46, 1419-1427
!     Fu, 1996: J. Climate, 9, 2058-2082

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 20060324

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOECLOP550, ONLY : RSA55, RSB55, RSC55, RSD55, RSE55, RSF55, RFA55, RFB55, RFC55, RFD55

IMPLICIT NONE

!     -----------------------------------------------------------------
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SU_CLOP550',0,ZHOOK_HANDLE)

!*          1.    SHORTWAVE CLOUD OPTICAL PROPERTIES AT 550 NM
!                 --------------------------------------------

!-- WATER CLOUDS FROM SLINGO (1989)

RSA55=2.383E-02_JPRB
RSB55=1.300_JPRB
RSC55=0._JPRB
RSD55=0._JPRB
RSE55=0.825_JPRB
RSF55=2.776E-03_JPRB

!-- ICE CLOUDS FROM FU (1996)

RFA55 = (/-0.303108E-04_JPRB, 0.251805E+01_JPRB /)
RFB55 = (/ 0.989368E-07_JPRB, 0.544286E-07_JPRB,-0.100877E-09_JPRB, 0.456410E-12_JPRB /)
RFC55 = (/ 0.751135E+00_JPRB, 0.909208E-03_JPRB,-0.972292E-06_JPRB,-0.862063E-08_JPRB /)
RFD55 = (/ 0.115399E+00_JPRB, 0.254148E-03_JPRB, 0.195306E-05_JPRB,-0.126367E-07_JPRB /)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU_CLOP550',1,ZHOOK_HANDLE)
END SUBROUTINE SU_CLOP550
