SUBROUTINE SUGWWMS(KSMAX)

! INITIALZE YOEGWWMS, THE MODULE THAT CONTROLS THE WARNER MCINTYRE GW PARAMETRIZATION

!          A.ORR            ECMWF     August 2008

!          INTERFACE
!          ---------
!          CALLED FROM *SUPHEC*

!          MODIFICATIONS
!          -------------
!          P. Bechtold, ECMWF (October 2008) Redefine computation of launch level


!--------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB 
USE YOMDIM    ,ONLY : NFLEVG
USE YOMSTA    ,ONLY : STPRE
USE YOMCT0    ,ONLY : NCONF
USE YOMDYN    ,ONLY : TSTEP
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN    ,ONLY : NULOUT,NULNAM
USE YOEGWWMS  ,ONLY : NLAUNCH, GFLUXLAUN, GCSTAR, GPTWO, NSLOPE, &
                    & LOZPR, GGAUSSA, GGAUSSB, NGAUSS, GCOEFF, GTPHYGWWMS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN):: KSMAX ! horizontal (spectral) resolution of host model

! internal
INTEGER(KIND=JPIM) :: JK
REAL(KIND=JPRB)    :: ZLAUNCHP        ! launch height of gw spectrum in Pa
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!--------------------------------------------------------------------------------

#include "posnam.intfb.h"
#include "namgwwms.h"

!--------------------------------------------------------------------------------

!             Set values of parameters/switches
!              --------------------------------

    
IF (LHOOK) CALL DR_HOOK('SUGWWMS',0,ZHOOK_HANDLE)

!*  SPECIFICATION OF SOURCE SPECTRUM
!*  --------------------------------

ZLAUNCHP=45000.0_JPRB   !launch height (Pa)
GFLUXLAUN=3.75E-3_JPRB  !total launch momentum flux in each azimuth (rho_o x F_o)

! resolution scaling for wavenumber>700
GFLUXLAUN=GFLUXLAUN*(1.0_JPRB-MIN(1.0_JPRB,atan((MAX(KSMAX,700)-700)/REAL(6000-700))))

NSLOPE=1                !s (1,0,-1 are valid values) s is the slope at small-m 
                        !end of the launch spectrum 
GPTWO=2.0_JPRB          !2*p (3 or 2 are valid values) p is the exponent of omega 
                        !in the expression of the launch spectrum
GCSTAR=1.0_JPRB         !C* (see McLandress and Scinocca 2005)

!* Extra parameters to introduce some latitudinal/seasonal variation of the launch spectrum.   
!* If LOZPR=TRUE and GGAUSS=1 then launch momentum flux is proportional to total precipitation  
!* If LOZPR=TRUE and GGAUSS=2 then launch momentum flux has a gaussian distribution

LOZPR=.FALSE.           !If .TRUE. then variable launch momemtum flux
NGAUSS=1
GGAUSSA=10.0_JPRB       !gaussian distribution half-width in degrees (used if GGAUSS=2)
GGAUSSB=0.5_JPRB        !height of gaussian distribution (used if GGAUSS=2)
GCOEFF=2000.0_JPRB      !multiplicative factor (used if GGAUSS=1) 


!*  COMPUTE MODEL LEVEL LAUNCH HEIGHT OF GW SPECTRUM
!*  ------------------------------------------------

NLAUNCH=NFLEVG-1
DO JK=NFLEVG,2,-1
  IF(STPRE(JK) > ZLAUNCHP)NLAUNCH=JK
ENDDO

!*  SETUP TIME FREQUENCY CALL OF SCHEME AS FUNCTION OF MODEL RESOLUTION
!*  -------------------------------------------------------------------

GTPHYGWWMS=3600.0_JPRB
IF (KSMAX == 255) THEN
   GTPHYGWWMS=5400.0_JPRB
ELSEIF (KSMAX < 255) THEN
   GTPHYGWWMS=7200.0_JPRB
ENDIF

IF (NCONF == 131.OR.NCONF == 401.OR.NCONF == 501.OR.NCONF == 601 &
 & .OR.NCONF == 801) THEN
!  GTPHYGWWMS=3600.0_JPRB
  GTPHYGWWMS=2.0_JPRB*TSTEP
ENDIF

CALL POSNAM(NULNAM,'NAMGWWMS')
READ (NULNAM,NAMGWWMS)
GTPHYGWWMS=INT(GTPHYGWWMS/TSTEP)*TSTEP

WRITE(UNIT=NULOUT,FMT='('' SUGWWMS: ZLAUNCHP = '',F7.1,'' GFLUXLAUN = '',F7.5,&
      & '' GCSTAR = '',F5.1,'' GTPHYGWWMS = '',F6.0)') ZLAUNCHP,GFLUXLAUN,GCSTAR,GTPHYGWWMS
IF (LOZPR) THEN
   WRITE(UNIT=NULOUT,FMT='('' SUGWWMS: LOZPR = '',L1,'' NGAUSS = '',I1,&
      & '' GGAUSSA = '',F7.1,'' GGAUSSB = '',F7.1)') LOZPR,NGAUSS,GGAUSSA,GGAUSSB
ENDIF
!--------------------------------------------------------------------------- 

IF (LHOOK) CALL DR_HOOK('SUGWWMS',1,ZHOOK_HANDLE)

END SUBROUTINE SUGWWMS

