SUBROUTINE UPDTIER(KSTEP,KRADFR,PTSTEP)

!**** *UPDTIER* - UPDATE TIME FOR ECMWF FULL RADIATION COMPUTATIONS

!     Purpose.
!     --------
!     UPDATE TIME OF THE MODEL + 1/2 TIME BETWEEN 2 FULL RADIATION STEPS

!**   Interface.
!     ----------
!        *CALL* *UPDTIER(KSTEP,KRADFR,PTSTEP)      from *ECRADFR*

!        Explicit arguments :
!        --------------------
!        KSTEP : TIME STEP INDEX
!        KRADFR: FREQUENCY OF FULL RADIATION COMPUTATIONS
!        PTSTEP: TIME STEP LENGTH

!        Implicit arguments :
!        --------------------
!        YOERIP

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Jean-Jacques Morcrette             *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-02-22
!        Modified J-J Morcrette 95-06-29  2time-levels for radiation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LTWOTL
USE YOERIP   , ONLY : RCODECM  ,RSIDECM  ,RCOVSRM  ,RSIVSRM
USE YOMRIP   , ONLY : NSSSSS   ,RTIMST   ,RTIMTR
USE YOMCST   , ONLY : RPI      ,RDAY     ,REA      ,REPSM    ,&
 & RI0      ,RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOERAD   , ONLY : LPERPET, NHINCSOL
USE YOERDI   , ONLY : RSOLINC
USE YOMDYNCORE, ONLY : LAQUA
USE YOMORB, ONLY : LCORBMD, ORBCALDAYM, SHR_ORB_UPD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KRADFR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTEP 
INTEGER(KIND=JPIM) :: ISTADD, ISTASS, ITIME, IZT, ISEC

REAL(KIND=JPRB) :: ZANGOZC, ZCOTHOZ, ZDEASOM, ZDECLIM, ZEQTIMM,&
 & ZHGMT, ZI0, ZSITHOZ, ZSOVRM, ZSTATI, ZTETA, ZTHETOZ, &
 & ZTIMTR, ZWSOVRM  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "suecozo.intfb.h"

#include "fctast.h"
#include "fcttim.h"
#include "fcttrm.h"
!     ------------------------------------------------------------------

!          1.  TIME EQUATION
!              -------------

IF (LHOOK) CALL DR_HOOK('UPDTIER',0,ZHOOK_HANDLE)
ITIME=NINT(PTSTEP)
IF (LTWOTL) THEN
  IZT=NINT(PTSTEP*(REAL(KSTEP,JPRB)+0.5_JPRB))
ELSE
  IZT=ITIME*KSTEP
ENDIF
IZT=ITIME*KSTEP

!--
IF (LPERPET) THEN
  ISEC=IZT/NINT(RDAY)
  IZT=IZT-ISEC*NINT(RDAY)
ENDIF
!--

ZSTATI=REAL(IZT,JPRB)+0.5_JPRB*KRADFR*ITIME
ISTADD=IZT/NINT(RDAY)
ISTASS=MOD(IZT,NINT(RDAY))
ZTIMTR=RTIMST+ZSTATI
ZHGMT=REAL(MOD(NINT(ZSTATI)+NSSSSS,NINT(RDAY)),JPRB)

ZTETA=RTETA(ZTIMTR)
IF( LAQUA ) THEN
  ZDEASOM=RRSAQUA(ZTETA)
  ZDECLIM=RDSAQUA(ZTETA)
  ZEQTIMM=RETAQUA(ZTETA)
ELSE
  ZDEASOM=RRS(ZTETA)
  ZDECLIM=RDS(ZTETA)
  ZEQTIMM=RET(ZTETA)
ENDIF

IF(LCORBMD) THEN
  CALL SHR_ORB_UPD(ZSTATI, ORBCALDAYM, ZDECLIM)
ENDIF

ZSOVRM =ZEQTIMM+ZHGMT
ZWSOVRM=ZSOVRM*2.0_JPRB*RPI/RDAY

IF (NHINCSOL /= 0) THEN
  ZI0=RSOLINC
ELSE
  ZI0=RI0
ENDIF

RCODECM=COS(ZDECLIM)
RSIDECM=SIN(ZDECLIM)

RCOVSRM=COS(ZWSOVRM)
RSIVSRM=SIN(ZWSOVRM)
ZTHETOZ=RTETA(RTIMTR)
ZANGOZC=REL(ZTHETOZ)-1.7535_JPRB
ZCOTHOZ=COS(ZANGOZC)
ZSITHOZ=SIN(ZANGOZC)

!     ------------------------------------------------------------------

!          2.  COMPUTES TIME-DEPENDENT PARAMETERS FOR OZONE CLIMATOLOGY
!              --------------------------------------------------------

CALL SUECOZO (ZANGOZC)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('UPDTIER',1,ZHOOK_HANDLE)
END SUBROUTINE UPDTIER
