SUBROUTINE VDFDIFMSTL (KIDIA, KFDIA, KLON, KLEV, KTOP,&
 & PTMST ,PUM15 ,PVM15  ,PAPHM15 , PCFM5, PTOFDC5,&
 & PVOM5 ,PVOL5 ,PUDIF5 ,PVDIF5  ,&
 & PUM1  ,PVM1  ,PAPHM1 ,PCFM    , PTOFDC,&
 & PVOM  ,PVOL  ,PUDIF  ,PVDIF)  
!     ------------------------------------------------------------------

!**   *VDFDIFMSTL* - DOES THE IMPLICIT CALCULATION FOR MOMENTUM DIFFUSION
!                    (Tangent linear)

!     J.F. MAHFOUF          E.C.M.W.F.    02/10/95

!     adapted from

!     DERIVED FROM VDIFF (CY34) BY
!     A.C.M. BELJAARS       E.C.M.W.F.    10-11-89
!     MODIFIED
!     --------
!     M. Janiskova  - 29/07/2004 - TL of Turb. Orogr. Drag + cleaning

!     PURPOSE
!     -------

!     SOLVE TRIDIAGONAL MATRICES FOR MOMENTUM DIFFUSION

!     INTERFACE
!     ---------

!     *VDFDIFMSTL* IS CALLED BY *VDFMAINSTL*

!*      INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLEV*         NUMBER OF LEVELS
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KTOP*         INDEX FOR BOUNDARY LAYER TOP

!*      INPUT PARAMETERS (REAL):

!     *PTMST*        DOUBLE TIME STEP (SINGLE AT 1TH STEP)

!*      INPUT PARAMETERS (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PUM15       PUM1          X-VELOCITY COMPONENT AT T-1               m/s
!  PVM15       PVM1          Y-VELOCITY COMPONENT AT T-1               m/s
!  PAPHM15     PAPHM1        PRESSURE AT T-1                           Pa
!  PCFM5       PCFM          PROP.TO EXCH.COEFF. FOR MOMENTUM 
!                                          (C,K-STAR IN DOC.)
!  PTOFDC5     PTOFDC        TURB. OROGR. DRAG (IMPLICIT) COEFFICIENTS 
!  PVOM5       PVOM          U-TENDENCY                                m/s2
!  PVOL5       PVOL          V-TENDENCY                                m/s2 

!*      OUTPUT PARAMETERS (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PUDIF5      PUDIF         U-DOUBLE-TILDE DEVIDED BY ALFA
!  PVDIF5      PVDIF         V-DOUBLE-TILDE DEVIDED BY ALFA


!     METHOD
!     ------

!     *LU*-DECOMPOSITION AND BACK SUBSTITUTION IN ONE DOWNWARD SCAN
!     AND ONE UPWARD SCAN.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEVDF   , ONLY : RVDIFTS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTOP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM15(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCFM5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOFDC5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOM5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUDIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVDIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCFM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOFDC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUDIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVDIF(KLON,KLEV) 
!*    LOCAL STORAGE
!     ----- -------

REAL(KIND=JPRB) ::  ZTCOE (KLON), ZEBSM (KLON,KLEV)
REAL(KIND=JPRB) ::  ZTCOE5(KLON), ZEBSM5(KLON,KLEV)

INTEGER(KIND=JPIM) :: ILEVM1, ITOPP1, JK, JL

REAL(KIND=JPRB) :: ZDISC, ZDISC5, ZFAC, ZFAC5, ZQDP, ZQDP5, ZTPFAC2
REAL(KIND=JPRB) :: ZDISCI, ZDISCI5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('VDFDIFMSTL',0,ZHOOK_HANDLE)
ZTPFAC2 = 1.0_JPRB/RVDIFTS
ILEVM1 = -1+KLEV
ITOPP1 = 1+KTOP

!*         1.1     SETTING OF RIGHT HAND SIDES.

DO JK = KTOP, KLEV
  DO JL = KIDIA, KFDIA
    PUDIF (JL,JK) = ZTPFAC2*PUM1 (JL,JK)+PTMST*PVOM (JL,JK)
    PUDIF5(JL,JK) = ZTPFAC2*PUM15(JL,JK)+PTMST*PVOM5(JL,JK)
    PVDIF (JL,JK) = ZTPFAC2*PVM1 (JL,JK)+PTMST*PVOL (JL,JK)
    PVDIF5(JL,JK) = ZTPFAC2*PVM15(JL,JK)+PTMST*PVOL5(JL,JK)
  ENDDO
ENDDO

!*         1.2     TOP LAYER ELIMINATION.

DO JL = KIDIA, KFDIA
  ZTCOE (JL) = PCFM (JL,KTOP)
  ZTCOE5(JL) = PCFM5(JL,KTOP)
  ZQDP  = -(PAPHM1 (JL,ITOPP1)-PAPHM1 (JL,KTOP))/&
   & (PAPHM15(JL,ITOPP1)-PAPHM15(JL,KTOP))**2  
  ZQDP5 = 1.0_JPRB/(PAPHM15(JL,ITOPP1)-PAPHM15(JL,KTOP))
  ZDISCI  = PCFM5(JL,KTOP)*ZQDP+ZQDP5*PCFM(JL,KTOP)+PTOFDC(JL,KTOP)
  ZDISCI5 = 1.0_JPRB+PCFM5(JL,KTOP)*ZQDP5+PTOFDC5(JL,KTOP)
  ZDISC  = -ZDISCI/ZDISCI5**2
  ZDISC5 = 1.0_JPRB/ZDISCI5
  ZEBSM (JL,KTOP) =(ZQDP*PCFM5(JL,KTOP)+ZQDP5*PCFM(JL,KTOP))*&
   & ZDISC5+(PCFM5(JL,KTOP)*ZQDP5)*ZDISC  
  ZEBSM5(JL,KTOP) = ZDISC5*(PCFM5(JL,KTOP)*ZQDP5)
  PUDIF (JL,KTOP) = PUDIF(JL,KTOP)*ZDISC5+PUDIF5(JL,KTOP)*ZDISC
  PUDIF5(JL,KTOP) = ZDISC5*PUDIF5(JL,KTOP)
  PVDIF (JL,KTOP) = PVDIF(JL,KTOP)*ZDISC5+PVDIF5(JL,KTOP)*ZDISC
  PVDIF5(JL,KTOP) = ZDISC5*PVDIF5(JL,KTOP)
ENDDO

!*         1.3     ELIMINATION FOR MIDDLE LAYERS.

DO JK = ITOPP1, ILEVM1
  DO JL = KIDIA, KFDIA
    ZQDP  = -(PAPHM1 (JL,JK+1)-PAPHM1 (JL,JK))/&
     & (PAPHM15(JL,JK+1)-PAPHM15(JL,JK))**2  
    ZQDP5 = 1.0_JPRB/(PAPHM15(JL,JK+1)-PAPHM15(JL,JK))
    ZFAC  = ZQDP*ZTCOE5(JL)+ZQDP5*ZTCOE(JL)
    ZFAC5 = ZTCOE5(JL)*ZQDP5
    ZTCOE (JL) = PCFM (JL,JK)
    ZTCOE5(JL) = PCFM5(JL,JK)
    ZDISCI  = (1.0_JPRB-ZEBSM5(JL,JK-1))*ZFAC-ZFAC5*ZEBSM (JL,JK-1) &
     &      + PCFM5(JL,JK)*ZQDP+ZQDP5*PCFM(JL,JK)+PTOFDC(JL,JK)
    ZDISCI5 = 1.0_JPRB+ZFAC5*(1.0_JPRB-ZEBSM5(JL,JK-1)) &
     &      + PCFM5(JL,JK)*ZQDP5+PTOFDC5(JL,JK)
    ZDISC  = -ZDISCI/ZDISCI5**2
    ZDISC5 = 1.0_JPRB/ZDISCI5
    ZEBSM (JL,JK) = ZDISC *(PCFM5(JL,JK)*ZQDP5) +&
     & ZDISC5*(PCFM (JL,JK)*ZQDP5) +&
     & ZDISC5*(PCFM5(JL,JK)*ZQDP )  
    ZEBSM5(JL,JK) = ZDISC5*(PCFM5(JL,JK)*ZQDP5)
    PUDIF (JL,JK) = (PUDIF(JL,JK)+(PUDIF(JL,JK-1)*ZFAC5+&
     & PUDIF5(JL,JK-1)*ZFAC))*ZDISC5+&
     & (PUDIF5(JL,JK)+ZFAC5*PUDIF5(JL,JK-1))*ZDISC  
    PUDIF5(JL,JK) =  ZDISC5*(PUDIF5(JL,JK)+ZFAC5*PUDIF5(JL,JK-1))
    PVDIF (JL,JK) = (PVDIF(JL,JK)+(PVDIF(JL,JK-1)*ZFAC5+&
     & PVDIF5(JL,JK-1)*ZFAC))*ZDISC5+&
     & (PVDIF5(JL,JK)+ZFAC5*PVDIF5(JL,JK-1))*ZDISC  
    PVDIF5(JL,JK) =  ZDISC5*(PVDIF5(JL,JK)+ZFAC5*PVDIF5(JL,JK-1))
  ENDDO
ENDDO

!*         1.4     BOTTOM LAYER ELIMINATION.

DO JL = KIDIA, KFDIA
  ZQDP  = -(PAPHM1 (JL,KLEV+1)-PAPHM1 (JL,KLEV))/&
   & (PAPHM15(JL,KLEV+1)-PAPHM15(JL,KLEV))**2  
  ZQDP5 = 1.0_JPRB/(PAPHM15(JL,KLEV+1)-PAPHM15(JL,KLEV))
  ZFAC  = ZQDP*ZTCOE5(JL)+ZQDP5*ZTCOE(JL)
  ZFAC5 = ZTCOE5(JL)*ZQDP5
  ZDISCI  = (1.0_JPRB-ZEBSM5(JL,ILEVM1))*ZFAC-ZFAC5*ZEBSM(JL,ILEVM1) &
   &      + PCFM5(JL,KLEV)*ZQDP+ZQDP5*PCFM(JL,KLEV)+PTOFDC(JL,KLEV)
  ZDISCI5 = 1.0_JPRB+ZFAC5*(1.0_JPRB-ZEBSM5(JL,ILEVM1)) &
   &      + PCFM5(JL,KLEV)*ZQDP5+PTOFDC5(JL,KLEV)
  ZDISC  = -ZDISCI/ZDISCI5**2
  ZDISC5 = 1.0_JPRB/ZDISCI5
  PUDIF (JL,KLEV) = (PUDIF(JL,KLEV)+(PUDIF(JL,ILEVM1)*ZFAC5+&
   & PUDIF5(JL,ILEVM1)*ZFAC))*ZDISC5+&
   & (PUDIF5(JL,KLEV)+ZFAC5*PUDIF5(JL,ILEVM1))*&
   & ZDISC  
  PUDIF5(JL,KLEV) =  ZDISC5*(PUDIF5(JL,KLEV)+ZFAC5*PUDIF5(JL,ILEVM1))
  PVDIF (JL,KLEV) = (PVDIF(JL,KLEV)+(PVDIF(JL,ILEVM1)*ZFAC5+&
   & PVDIF5(JL,ILEVM1)*ZFAC))*ZDISC5+&
   & (PVDIF5(JL,KLEV)+ZFAC5*PVDIF5(JL,ILEVM1))*&
   & ZDISC  
  PVDIF5(JL,KLEV) =  ZDISC5*(PVDIF5(JL,KLEV)+ZFAC5*PVDIF5(JL,ILEVM1))
ENDDO

!*         1.5     BACK-SUBSTITUTION.

DO JK = ILEVM1, KTOP, -1
  DO JL = KIDIA, KFDIA
    PUDIF (JL,JK) = PUDIF(JL,JK)+(PUDIF(JL,JK+1)*&
     & ZEBSM5(JL,JK)+PUDIF5(JL,JK+1)*ZEBSM(JL,JK))  
    PUDIF5(JL,JK) = PUDIF5(JL,JK)+ZEBSM5(JL,JK)*PUDIF5(JL,JK+1)
    PVDIF (JL,JK) = PVDIF(JL,JK)+(PVDIF(JL,JK+1)*&
     & ZEBSM5(JL,JK)+PVDIF5(JL,JK+1)*ZEBSM(JL,JK))  
    PVDIF5(JL,JK) = PVDIF5(JL,JK)+ZEBSM5(JL,JK)*PVDIF5(JL,JK+1)
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('VDFDIFMSTL',1,ZHOOK_HANDLE)
END SUBROUTINE VDFDIFMSTL

