!OPTIONS XOPT(HSFUN)
SUBROUTINE VDFEXCUSAD (KIDIA, KFDIA, KLON, KLEV, PTMST,&
 & PZ0MM5 ,&
 & PUM15  ,PVM15  ,PTM15  ,PQM15 , &
 & PAPHM15,PGEOM15,PGEOH5 ,PCPTGZ5,&
 & PKMFL5 ,PKHFL5 ,PKQFL5 ,PZINV5,&
 & PCFM5  ,PCFH5, &
 & PZ0MM  , &
 & PUM1   ,PVM1   ,PTM1   ,PQM1 , &
 & PAPHM1 ,PGEOM1 ,PGEOH  ,PCPTGZ ,PCFM ,PCFH)  
!     ------------------------------------------------------------------

!**   *VDFEXCUSAD* - DETERMINES THE EXCHANGE COEFFICIENTS BETWEEN THE
!                    UPPER MODEL LEVELS WITH STABILITY AS A FUNCTION OF
!                    RICHARDSON NUMBER
!                    (Adjoint)

!     J.F. MAHFOUF         E.C.M.W.F.    09/02/96

!     MODIFIED BY

!     A.C.M.BELJAARS    E.C.M.W.F.    26/03/99
!     M. Janiskova      15/10/07  add mixed layer
!                                 reduce K towards MO higher up (as in NL code)
!     M. Janiskova      19/12/07  modified wind shear for Ri computation
!     M. Janiskova      08/08/08  bug correction
!     M. Janiskova      15/09/09  perturbation of K not used above 0.1 hPa

!     PURPOSE
!     -------

!     DETERMINE EXCHANGE COEFFICIENTS BETWEEN THE UPPER MODEL LEVELS

!     INTERFACE
!     ---------

!     *VDFEXCUSAD* IS CALLED BY *VDFMAINSAD*

!*      INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLEV*         NUMBER OF LEVELS
!     *KLON*         NUMBER OF GRID POINTS PER PACKET

!*      INPUT PARAMETERS (REAL):

!     *PTMST*        DOUBLE TIME STEP (SINGLE AT 1TH STEP)

!*      INPUT PARAMETERS (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PUM15       PUM1          X-VELOCITY COMPONENT AT T-1               m/s
!  PVM15       PVM1          Y-VELOCITY COMPONENT AT T-1               m/s
!  PTM15       PTM1          TEMPERATURE AT T-1                        K
!  PQM15       PQM1          SPECIFIC HUMUDITY AT T-1                  kg/kg
!  PAPHM15     PAPHM1        PRESSURE AT T-1                           Pa
!  PGEOM15     PGEOM1        GEOPOTENTIAL AT T-1                       m2/s2
!  PGEOH5      PGEOH         GEOPOTENTIAL AT HALF LEVELS, T-1          m2/s2
!  PCPTGZ5     PCPTGZ        DRY STATIC ENERGY                         J/kg
!  PKMFL5      ---           KINEMATIC MOMENTUM FLUX
!  PKHFL5      ---           KINEMATIC HEAT FLUX
!  PKQFL5      ---           KINEMATIC MOISTURE FLUX
!  PZINV5      ---           INVERSION HEIGHT

!*      OUTPUT PARAMETERS (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PCFM5       PCFM          PROP. TO EXCH. COEFF. FOR MOMENTUM 
!                                              (C-STAR IN DOC.)       
!  PCFH5       PCFH          PROP. TO EXCH. COEFF. FOR HEAT  
!                                              (C-STAR IN DOC.)       
!                            (ONLY PCFM(*,1:KLEV-1) AND
!                             PCFH(*,1:KLEV-1) ARE COMPUTED)

!     METHOD
!     ------

!     SEE DOCUMENTATION

!     THE EXCHANGE COEFFICIENTS ARE EXPRESSED ACCORDING TO 
!     LOUIS et al. (1982) PROPOSAL NEAR THE SURFACE.  HIGHER
!     UP MONIN-OBUKOV VALUES ARE USED WITH A LENGTH SCALE OF
!     300 M (150M).
!     POSSIBILITY TO USE LOUIS et al. (1982) PROPOSAL IN STABLE
!     SITUATIONS STILL KEPT.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RCPD     ,RETV
USE YOEVDF   , ONLY : RLAM     ,RKAP     ,RVDIFTS  ,REPDU2
USE YOEVDFS  , ONLY : JPRITBL  ,RITBL    ,ARITBL   ,&
                    & RCHETA   ,RCHETB   ,RIMAX    ,&
                    & DRITBL   ,DRI26
USE YOEPHLI  , ONLY : RLPCC    ,RLPMIXL  ,RLPBETA
USE YOPHNC   , ONLY : LEKPERT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ0MM5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM15(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTGZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKMFL5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKHFL5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKQFL5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZINV5(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCFM5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCFH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0MM(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHM1(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOH(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPTGZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCFM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCFH(KLON,KLEV) 

!*            LOCAL STORAGE
!             ----- -------

INTEGER(KIND=JPIM) :: JK, JL, IRIB

REAL(KIND=JPRB) ::    ZUST5(KLON),ZKHVFL5(KLON)
REAL(KIND=JPRB) ::    ZL5(KLON),ZLI5(KLON)
REAL(KIND=JPRB) ::    ZRI (KLON),ZDU1 (KLON),ZDU2 (KLON),ZMGEOM (KLON)
REAL(KIND=JPRB) ::    ZDTV(KLON),ZDTVR(KLON),ZPHIM(KLON),ZPHIH(KLON)

REAL(KIND=JPRB) :: Z1S, Z2S, Z3S, Z4S, Z5S, Z6S, &
 &                 ZDRORO, ZETA, ZA, ZB, ZIFMOM, ZIFMOH, &
 &                 ZDUDZ, ZDUDZ1, ZDUDZ2, &
 &                 ZZH, ZSCF, ZIFLTGM, ZIFLTGH, &
 &                 ZBM, ZBH, ZCM, ZCH, Z2GEOMF, ZLIM, ZLIM2, &
 &                 ZALH2, ZALM2, ZCFNC, ZZI, ZZI2, &
 &                 ZZ, ZDH, ZDL, ZPHIKH, ZPHIKM, ZPHIKHI, ZPHIKMI, &
 &                 ZDHZZ, ZKH, ZCFHNEW, ZCFNC1

REAL(KIND=JPRB) :: ZENTRSFC, ZCB, ZCD, ZEPS, ZLENMO, ZX2, ZRIMIN
REAL(KIND=JPRB) :: ZCONS2, ZCONS3, ZCONS23, ZCONS8, ZCONS9, &
 &                 ZCONS10, ZCONS11, ZCONS13, ZCC, ZRG, &
 &                 ZUMBETA, ZCONSHG, ZHLM, ZHLM2, ZREPUST
REAL(KIND=JPRB) :: ZAAA , ZCCC , ZDDD , ZAAA1, ZCCC1, ZDDD1
REAL(KIND=JPRB) :: ZAAA2, ZCCC2, ZDDD2

!            EXTRA LOCAL STORAGE FOR TRAJECTORY
!            ----------------------------------

REAL(KIND=JPRB) :: ZZW5(KLON), ZWTVENTR5(KLON)

REAL(KIND=JPRB) :: ZDU15(KLON,KLEV)  , ZDU25(KLON,KLEV), ZDRORO5(KLON,KLEV)
REAL(KIND=JPRB) :: ZMGEOM5(KLON,KLEV), ZDTV5(KLON,KLEV), ZDTVR5(KLON,KLEV)
REAL(KIND=JPRB) :: ZRI5(KLON,KLEV), ZETA5(KLON,KLEV), ZA5(KLON,KLEV)
REAL(KIND=JPRB) :: ZB5(KLON,KLEV), ZPHIM5(KLON,KLEV), ZPHIH5(KLON,KLEV)
REAL(KIND=JPRB) :: ZIFMOM5(KLON,KLEV), ZIFMOH5(KLON,KLEV)
REAL(KIND=JPRB) :: ZDUDZ15(KLON,KLEV), ZDUDZ25(KLON,KLEV)
REAL(KIND=JPRB) :: ZDUDZ5(KLON,KLEV), ZZH5(KLON,KLEV), ZSCF5(KLON,KLEV)
REAL(KIND=JPRB) :: ZIFLTGM5(KLON,KLEV), ZIFLTGH5(KLON,KLEV)
REAL(KIND=JPRB) :: ZBM5(KLON,KLEV), ZBH5(KLON,KLEV), ZCM5(KLON,KLEV)
REAL(KIND=JPRB) :: ZCH5(KLON,KLEV), Z2GEOMF5(KLON,KLEV)
REAL(KIND=JPRB) :: ZLIM5(KLON,KLEV), ZLIM25(KLON,KLEV)
REAL(KIND=JPRB) :: ZALM25(KLON,KLEV), ZALH25(KLON,KLEV)
REAL(KIND=JPRB) :: ZCFNC5(KLON,KLEV), ZZI5(KLON,KLEV), ZZI25(KLON,KLEV)
REAL(KIND=JPRB) :: ZZ5(KLON,KLEV), ZDH5(KLON,KLEV), ZDL5(KLON,KLEV)
REAL(KIND=JPRB) :: ZPHIKH5(KLON,KLEV), ZPHIKM5(KLON,KLEV)
REAL(KIND=JPRB) :: ZDHZZ5(KLON,KLEV), ZPHIKHI5(KLON,KLEV)
REAL(KIND=JPRB) :: ZPHIKMI5(KLON,KLEV), ZKH5(KLON,KLEV)
REAL(KIND=JPRB) :: ZCFHNEW5(KLON,KLEV), ZCFNC15(KLON,KLEV)
REAL(KIND=JPRB) :: ZPCFM15(KLON,KLEV), ZPCFH15(KLON,KLEV)
REAL(KIND=JPRB) :: ZPCFM25(KLON,KLEV), ZPCFH25(KLON,KLEV)

REAL(KIND=JPRB) :: Z1S5(KLON,KLEV), Z2S5(KLON,KLEV), Z3S5(KLON,KLEV)
REAL(KIND=JPRB) :: Z4S5(KLON,KLEV), Z5S5(KLON,KLEV), Z6S5(KLON,KLEV)
REAL(KIND=JPRB) :: ZSQRT15(KLON,KLEV), ZSQRT25(KLON,KLEV)
REAL(KIND=JPRB) :: ZSQRT35(KLON,KLEV), ZSQRT45(KLON,KLEV)
REAL(KIND=JPRB) :: ZSQRT55(KLON,KLEV), ZSQRT65(KLON,KLEV)

REAL(KIND=JPRB) :: ZZD5
REAL(KIND=JPRB) :: ZPAP5

LOGICAL :: LLMONOB

REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "surf_inq.h"
END INTERFACE

!     ------------------------------------------------------------------

!*       1.     INITIALIZE CONSTANTS
!               ---------- ----------

IF (LHOOK) CALL DR_HOOK('VDFEXCUSAD',0,ZHOOK_HANDLE)

! logical switch for using either Monin-Obuchov (true) or Louis (false)
! for stable conditions

LLMONOB = .TRUE.
!LLMONOB = .FALSE.

! different constants

ZENTRSFC  = 0.2_JPRB       ! factor for surface based top entrainment
ZCD       = 1.0_JPRB
ZCB       = 5.0_JPRB
ZEPS      = 1.E-10_JPRB
!or ZLENMO    = 150.0_JPRB
ZLENMO    = 300.0_JPRB
ZRIMIN    = 1.E-3_JPRB

ZCONS2    = (0.5_JPRB*RKAP)/RG
ZCONS3    = 1.0_JPRB/RLAM
ZCONS23   = ZCONS2*ZCONS3

ZCC       = RLPCC
ZCONS8    = 2.0_JPRB*ZCB
ZCONS9    = 3._JPRB*ZCB
ZCONS10   = (RVDIFTS*PTMST*RG**2)/(0.5_JPRB*RD)
ZCONS11   = (3._JPRB*ZCB*ZCC)/SQRT(27._JPRB)
ZUMBETA   = 1.0_JPRB - RLPBETA
ZCONSHG   = 2.0_JPRB*RG
ZHLM      = ZCONSHG*RLPMIXL
ZHLM2     = ZHLM*ZHLM

CALL SURF_INQ(PREPUST=ZREPUST)

! optimization
ZRG       = 1.0_JPRB/RG
ZCONS13   = 1.0_JPRB/3._JPRB

!     CONSTANTS FOR REGULARIZATION

ZAAA=10.0_JPRB
ZCCC=0.0416277_JPRB
ZAAA1=30.0_JPRB
ZCCC1=0.031838_JPRB
ZAAA2=5.0_JPRB
ZCCC2=0.075871_JPRB
!ZAAA3=20.0_JPRB
!ZCCC3=0.0588705_JPRB

!     ------------------------------------------------------------------

IF (LEKPERT) THEN
  DO JL = KIDIA, KFDIA
    ZZW5(JL)=SQRT(PUM15(JL,KLEV)**2+PVM15(JL,KLEV)**2)
  ENDDO
ENDIF

!*         2.     PREPARE SCALING COEFFICIENTS FOR MIXED LAYER
!                 --------------------------------------------

DO JL=KIDIA,KFDIA
  ZUST5 (JL)=MAX(SQRT(PKMFL5(JL)),ZREPUST)
  ZKHVFL5(JL)=PKHFL5(JL)+RETV*PTM15(JL,KLEV)*PKQFL5(JL)
  IF (ZKHVFL5(JL)  <  0.0_JPRB) THEN
    ZL5 (JL) = ZUST5(JL)**3*PTM15(JL,KLEV)/(RKAP*RG*(ZKHVFL5(JL)-ZEPS))
    ZLI5(JL) = 1.0_JPRB/ZL5(JL)
  ENDIF
ENDDO

!        3. VERTICAL LOOP
!           -------- ----

!--------------------------------------------------------------------------

!*                 ------ TRAJECTORY COMPUTATIONS --------

!--------------------------------------------------------------------------  

!***  
DO JK = KLEV-1, 1, -1
!***

  DO JL = KIDIA, KFDIA
    PCFM5(JL,JK) = 0.0_JPRB
    PCFH5(JL,JK) = 0.0_JPRB
  ENDDO

  DO JL = KIDIA, KFDIA

!*          COMPUTE RI-NUMBER

    ZMGEOM5(JL,JK) = PGEOM15(JL,JK)-PGEOM15(JL,JK+1)
    Z1S5(JL,JK) = (PUM15(JL,JK)-PUM15(JL,JK+1))**2 &
     & +(PVM15(JL,JK)-PVM15(JL,JK+1))**2
    IF (REPDU2 >= Z1S5(JL,JK)) THEN
      ZDU15(JL,JK) = REPDU2
    ELSE
      ZDU15(JL,JK) = Z1S5(JL,JK)
    ENDIF
    ZSQRT15(JL,JK) = SQRT(ZDU15(JL,JK))
    ZDUDZ15(JL,JK) = ZSQRT15(JL,JK)/ZMGEOM5(JL,JK)*RG            !shear

!     shear factor: f=27.2 * (1-p/ps) * exp(-10*(1-p/ps))
!                   f=0 at p/ps=0 and 1
!                   max(f)=1.0 at p/ps=0.9

    ZDUDZ25(JL,JK) = ZDUDZ15(JL,JK) + 0.005_JPRB  &
     & * 27.2_JPRB*(1.0_JPRB - PAPHM15(JL,JK)/PAPHM15(JL,KLEV)) &
     & * EXP(-10.0_JPRB*(1.0_JPRB - PAPHM15(JL,JK)/PAPHM15(JL,KLEV)))
    ZDU25(JL,JK) = (ZDUDZ25(JL,JK)*ZMGEOM5(JL,JK)*ZRG) **2

    ZDRORO5(JL,JK) = 2.0_JPRB*(PCPTGZ5(JL,JK)-PCPTGZ5(JL,JK+1))/&
     & (PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1)-PGEOM15(JL,JK)-&
     & PGEOM15(JL,JK+1))+&
     & RETV*(PQM15(JL,JK)-PQM15(JL,JK+1))  
    ZDTV5(JL,JK)= PCPTGZ5(JL,JK)-PCPTGZ5(JL,JK+1) &
     & + RETV * 0.5_JPRB * (PQM15(JL,JK)-PQM15(JL,JK+1))&
     & * (PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1))
    ZDTVR5(JL,JK) = RCPD/ZDTV5(JL,JK)
    ZRI5(JL,JK) = ZMGEOM5(JL,JK)*ZDRORO5(JL,JK)/ZDU25(JL,JK)
  ENDDO

  DO JL = KIDIA, KFDIA

!           COMPUTE STABILITY FUNCTIONS

    IF (ZRI5(JL,JK) > 0.0_JPRB .AND. LLMONOB) THEN

      IRIB=INT(ZRI5(JL,JK)*(1.0_JPRB/DRITBL))+1
      IF (IRIB  >=  JPRITBL) THEN
!           LINEAR EXTENSION OF LOOK-UP TABLE
        ZETA5(JL,JK) = RITBL(JPRITBL)*(ZRI5(JL,JK)*(1.0_JPRB/RIMAX))
      ELSE
        ZX2 = IRIB*DRITBL
        ZA5(JL,JK) = (ZX2-ZRI5(JL,JK))*(1.0_JPRB/DRITBL)
        ZB5(JL,JK) = 1.0_JPRB-ZA5(JL,JK)
        ZETA5(JL,JK) = ZA5(JL,JK)*RITBL(IRIB) + ZB5(JL,JK)*RITBL(IRIB+1)&
         & +( (ZA5(JL,JK)**3-ZA5(JL,JK))*ARITBL(IRIB)&
         & +(  ZB5(JL,JK)**3-ZB5(JL,JK))*ARITBL(IRIB+1) )*DRI26
      ENDIF

!           STABLE PHI-FUNCTIONS

      ZPHIM5(JL,JK) = 1.0_JPRB+RCHETA*ZETA5(JL,JK)
      ZPHIH5(JL,JK) =  (1.0_JPRB+RCHETB*ZETA5(JL,JK))**2
    ENDIF

  ENDDO

  DO JL = KIDIA, KFDIA

    IF (ZRI5(JL,JK) > 0.0_JPRB) THEN  ! statically stable

      IF (LLMONOB) THEN  ! Monin-Obuchov

!           COMMON FACTORS FOR STABLE AND UNSTABLE
!           new K: K,LTG scales with l=kappa*z
!                  K,MO  scales with l=150m above surface layer

        ZIFMOM5(JL,JK) = 1.0_JPRB / (ZPHIM5(JL,JK)**2)              !F(MO),M
        ZIFMOH5(JL,JK) = 1.0_JPRB / (ZPHIM5(JL,JK)*ZPHIH5(JL,JK))   !F(MO),H
        ZSQRT25(JL,JK) = SQRT(ZDU25(JL,JK))
        ZDUDZ5(JL,JK) = ZSQRT25(JL,JK)/ZMGEOM5(JL,JK)*RG            !shear
        ZZH5(JL,JK) = 0.5_JPRB*ZRG*(PGEOM15(JL,JK)+PGEOM15(JL,JK+1))+PZ0MM5(JL)
        ZSCF5(JL,JK) = SQRT(1.0_JPRB+ZCD*ZRI5(JL,JK))

!        ZIFLTGM5 = 1.0_JPRB/(1.0_JPRB+2.0_JPRB*ZCB*ZRI5(JL)/ZSCF5) !F(LTG),M
!F(LTG),M
        ZIFLTGM5(JL,JK) = ZSCF5(JL,JK)/(ZSCF5(JL,JK) &
         & +2.0_JPRB*ZCB*ZRI5(JL,JK))
!F(LTG),H
        ZIFLTGH5(JL,JK) = &
         & 1.0_JPRB/(1.0_JPRB+2.0_JPRB*ZCB*ZRI5(JL,JK)*ZSCF5(JL,JK))

        ZSQRT35(JL,JK) = SQRT(ZIFLTGM5(JL,JK))
        ZBM5(JL,JK) = RKAP*ZZH5(JL,JK)*ZSQRT35(JL,JK)
        ZSQRT45(JL,JK) = SQRT(ZIFLTGH5(JL,JK))
        ZBH5(JL,JK) = RKAP*ZZH5(JL,JK)*ZSQRT45(JL,JK)

        ZSQRT55(JL,JK) = SQRT(ZIFMOM5(JL,JK))
        ZCM5(JL,JK) = ZLENMO*ZSQRT55(JL,JK)
        ZSQRT65(JL,JK) = SQRT(ZIFMOH5(JL,JK))
        ZCH5(JL,JK) = ZLENMO*ZSQRT65(JL,JK)

        PCFM5(JL,JK) = ZDUDZ5(JL,JK)*(ZBM5(JL,JK)*ZCM5(JL,JK) &
         & /(ZBM5(JL,JK)+ZCM5(JL,JK)))**2
        PCFH5(JL,JK) = ZDUDZ5(JL,JK)*(ZBH5(JL,JK)*ZCH5(JL,JK) &
         & /(ZBH5(JL,JK)+ZCH5(JL,JK)))**2

      ELSE  ! Louis
        Z2GEOMF5(JL,JK) =PGEOM15(JL,JK)+PGEOM15(JL,JK+1)+ZCONSHG*PZ0MM5(JL)
        ZLIM5(JL,JK) = RLPBETA &
        & +ZUMBETA/(1.0_JPRB+(Z2GEOMF5(JL,JK)*Z2GEOMF5(JL,JK))/ZHLM2)
        ZLIM25(JL,JK) = ZLIM5(JL,JK)*ZLIM5(JL,JK)
        ZALM25(JL,JK) = ZLIM25(JL,JK)*((ZCONS2*Z2GEOMF5(JL,JK)) &
         & /(1.0_JPRB+ZCONS23*Z2GEOMF5(JL,JK)))**2
        ZALH25(JL,JK) =ZALM25(JL,JK)

        ZSQRT25(JL,JK) = SQRT(ZDU25(JL,JK))
        ZCFNC5(JL,JK) = RG*ZSQRT25(JL,JK)/ZMGEOM5(JL,JK)
        ZSCF5(JL,JK) = SQRT(1.0_JPRB+ZCD*ZRI5(JL,JK))


!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

        PCFM5(JL,JK) = (ZCFNC5(JL,JK)*ZALM25(JL,JK)) &
         & /(1.0_JPRB+ZCONS8*ZRI5(JL,JK)/ZSCF5(JL,JK))
        PCFH5(JL,JK) = (ZCFNC5(JL,JK)*ZALH25(JL,JK)) &
         & /(1.0_JPRB+ZCONS9*ZRI5(JL,JK)*ZSCF5(JL,JK))
      ENDIF

    ELSE  ! statically unstable

      Z2GEOMF5(JL,JK) =PGEOM15(JL,JK)+PGEOM15(JL,JK+1)+ZCONSHG*PZ0MM5(JL)
      ZLIM5(JL,JK) = RLPBETA &
       & +ZUMBETA/(1.0_JPRB+(Z2GEOMF5(JL,JK)*Z2GEOMF5(JL,JK))/ZHLM2)
      ZLIM25(JL,JK) = ZLIM5(JL,JK)*ZLIM5(JL,JK)
      ZALM25(JL,JK) =ZLIM25(JL,JK)*((ZCONS2*Z2GEOMF5(JL,JK)) &
       & /(1.0_JPRB+ZCONS23*Z2GEOMF5(JL,JK)))**2
      ZALH25(JL,JK) = ZALM25(JL,JK)

      ZSQRT25(JL,JK) = SQRT(ZDU25(JL,JK))
      ZCFNC5(JL,JK) = RG*ZSQRT25(JL,JK)/ZMGEOM5(JL,JK)

!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

      ZZI5(JL,JK) = ZCONSHG/Z2GEOMF5(JL,JK)
      ZZI25(JL,JK) = ZZI5(JL,JK)*ZZI5(JL,JK)
      Z2S5(JL,JK) = SQRT(-ZRI5(JL,JK))
      Z3S5(JL,JK) = ZCONS11*ZALM25(JL,JK)*ZZI25(JL,JK)
      Z4S5(JL,JK) = ZCONS11*ZALH25(JL,JK)*ZZI25(JL,JK)
      Z5S5(JL,JK) = ZCFNC5(JL,JK)*ZALM25(JL,JK)
      PCFM5(JL,JK) = Z5S5(JL,JK)*(1.0_JPRB-(ZCONS8*ZRI5(JL,JK)) &
       & /(1.0_JPRB+Z3S5(JL,JK)*Z2S5(JL,JK)))
      Z6S5(JL,JK) = ZCFNC5(JL,JK)*ZALH25(JL,JK)
      PCFH5(JL,JK) = Z6S5(JL,JK)*(1.0_JPRB-(ZCONS9*ZRI5(JL,JK)) &
       & /(1.0_JPRB+Z4S5(JL,JK)*Z2S5(JL,JK)))

    ENDIF

!           ADD MIXED LAYER PARAMETRIZATION (SURF. AND OUTER LAYER)

    IF (PGEOH5(JL,JK-1)*ZRG <= PZINV5(JL) ) THEN  ! up to level below entr.lev.
      ZZ5(JL,JK)  = PGEOH5(JL,JK)*ZRG
      ZDH5(JL,JK) = ZZ5(JL,JK)/PZINV5(JL)
      ZDL5(JL,JK) = ZZ5(JL,JK)*ZLI5(JL)
      ZPHIKH5(JL,JK) = (1.0_JPRB-39._JPRB*ZDL5(JL,JK))**(-ZCONS13)
      ZPHIKM5(JL,JK) = (1.0_JPRB-15._JPRB*ZDL5(JL,JK))**(-ZCONS13)

!           K,surface

      ZDHZZ5(JL,JK) = ZUST5(JL) * ZZ5(JL,JK) * (1.0_JPRB-ZDH5(JL,JK))**2
      ZPHIKHI5(JL,JK) = RKAP / ZPHIKH5(JL,JK)
      PCFH5(JL,JK) = ZPHIKHI5(JL,JK) * ZDHZZ5(JL,JK)
      ZPHIKMI5(JL,JK) = RKAP / ZPHIKM5(JL,JK)
      PCFM5(JL,JK) = ZPHIKMI5(JL,JK) * ZDHZZ5(JL,JK)
    ENDIF

! Storage for backward computation
    ZPCFH15(JL,JK) = PCFH5(JL,JK)
    ZPCFM15(JL,JK) = PCFM5(JL,JK)

!           ADD ENTRAINMENT
!           entrainment velocity * T,v jump due to sfc flux

   IF (PGEOH5(JL,JK)*ZRG<=PZINV5(JL).AND.PZINV5(JL)<PGEOH5(JL,JK-1)*ZRG) THEN

      ZWTVENTR5(JL) = - ZENTRSFC * ZKHVFL5(JL)              !sfc flux
      ZWTVENTR5(JL) = MAX(0.0_JPRB,ZWTVENTR5(JL))

      ZKH5(JL,JK) = ZWTVENTR5(JL) * ZRG * ZMGEOM5(JL,JK) * ZDTVR5(JL,JK)
      IF (ZKH5(JL,JK) < 0.0_JPRB) THEN
        ZCFHNEW5(JL,JK) = 0.0_JPRB
      ELSE
        ZCFHNEW5(JL,JK) = ZKH5(JL,JK)
      ENDIF
      IF (ZCFHNEW5(JL,JK) > PCFH5(JL,JK)) THEN !protection against K=0
        PCFH5(JL,JK) = ZCFHNEW5(JL,JK)
      ENDIF
      IF (0.75_JPRB*ZCFHNEW5(JL,JK) > PCFM5(JL,JK)) THEN
        PCFM5(JL,JK) = 0.75_JPRB*ZCFHNEW5(JL,JK)
      ENDIF
    ENDIF

! Storage for backward computation
    ZPCFH25(JL,JK) = PCFH5(JL,JK)
    ZPCFM25(JL,JK) = PCFM5(JL,JK)

!           SCALE DIFFUSION COEFFICIENTS FOR USE IN VDFDIFH/M

    ZCFNC15(JL,JK) =(ZCONS10*PAPHM15(JL,JK))/(ZMGEOM5(JL,JK) &
     & *(PTM15(JL,JK)*(1.0_JPRB+RETV*PQM15(JL,JK)) &
     & + PTM15(JL,JK+1)*(1.0_JPRB+RETV*PQM15(JL,JK+1))))
    PCFH5(JL,JK) = PCFH5(JL,JK) * ZCFNC15(JL,JK)
    PCFM5(JL,JK) = PCFM5(JL,JK) * ZCFNC15(JL,JK)
    
  ENDDO

!***  
ENDDO
!***  
!--------------------------------------------------------------------------

!*                 ------ ADJOINT COMPUTATIONS --------

!--------------------------------------------------------------------------  

DO JL=KIDIA,KFDIA
  ZRI (JL) = 0.0_JPRB
  ZMGEOM (JL) = 0.0_JPRB
  ZDU1 (JL) = 0.0_JPRB
  ZDU2 (JL) = 0.0_JPRB
  ZDTV (JL) = 0.0_JPRB
  ZPHIM(JL) = 0.0_JPRB
  ZPHIH(JL) = 0.0_JPRB
  ZDTVR(JL) = 0.0_JPRB
ENDDO

!        3. VERTICAL LOOP
!           -------- ----

!***  
DO JK = 1,KLEV-1
!***
  DO JL = KIDIA, KFDIA

!         INITIALISATIONS OF LOCAL VARIABLES

    ZIFMOM  = 0.0_JPRB
    ZIFMOH  = 0.0_JPRB
    ZDUDZ   = 0.0_JPRB
    ZZH     = 0.0_JPRB
    ZSCF    = 0.0_JPRB
    ZIFLTGM = 0.0_JPRB
    ZIFLTGH = 0.0_JPRB
    ZBM     = 0.0_JPRB
    ZBH     = 0.0_JPRB
    ZCM     = 0.0_JPRB
    ZCH     = 0.0_JPRB
    Z2GEOMF = 0.0_JPRB
    ZLIM    = 0.0_JPRB
    ZLIM2   = 0.0_JPRB
    ZALH2   = 0.0_JPRB
    ZALM2   = 0.0_JPRB
    ZCFNC   = 0.0_JPRB
    ZZI     = 0.0_JPRB
    ZZI2    = 0.0_JPRB
    Z2S     = 0.0_JPRB
    Z3S     = 0.0_JPRB
    Z4S     = 0.0_JPRB
    Z5S     = 0.0_JPRB
    Z6S     = 0.0_JPRB
    ZZ      = 0.0_JPRB
    ZDH     = 0.0_JPRB
    ZDL     = 0.0_JPRB
    ZPHIKH  = 0.0_JPRB
    ZPHIKM  = 0.0_JPRB
    ZDHZZ   = 0.0_JPRB
    ZPHIKHI = 0.0_JPRB
    ZPHIKMI = 0.0_JPRB
    ZKH     = 0.0_JPRB
    ZCFHNEW = 0.0_JPRB
    ZCFNC1  = 0.0_JPRB

    IF (.NOT.LEKPERT) THEN
      PCFM (JL,JK) = 0.0_JPRB
      PCFH (JL,JK) = 0.0_JPRB
    ELSE
      ZDDD  = MAX(ZAAA*EXP(-(ZCCC*ZRI5(JL,JK))**2),1.0_JPRB)
      ZDDD  = 1.0_JPRB/ZDDD
      ZDDD1 = MAX(ZAAA1*EXP(-(ZCCC1*ZRI5(JL,JK))**2),1.0_JPRB)
      ZDDD1 = 1.0_JPRB/ZDDD1

! perturbation of exchange coefficients not used above 0.1 hPa
! (to avoid negative temperature increments when T in trajectory
! is already too low)
      ZPAP5 = 0.5_JPRB*(PAPHM15(JL,JK) + PAPHM15(JL,JK-1))
      IF (ZPAP5 < 10.0_JPRB) THEN
        PCFM(JL,JK) = 0.0_JPRB
        PCFH(JL,JK) = 0.0_JPRB
      ENDIF

      IF (PTM15(JL,KLEV) < 253.15_JPRB) THEN
        PCFM(JL,JK) = PCFM(JL,JK)*ZDDD
        PCFH(JL,JK) = PCFH(JL,JK)*ZDDD1

      ELSE
        IF (PGEOM15(JL,JK) < 30000.0_JPRB) THEN
          PCFM(JL,JK) = PCFM(JL,JK)*ZDDD

          IF (ZZW5(JL) < 1.0_JPRB) THEN
            PCFH(JL,JK) = 0.0_JPRB
          ELSE
            PCFH(JL,JK) = PCFH(JL,JK)*ZDDD1
          ENDIF
!or        ELSEIF (PAPHM15(JL,JK) > 15000.0_JPRB) THEN
!or2        ELSEIF (PAPHM15(JL,JK) > 10000.0_JPRB) THEN
        ELSE
          ZDDD2 = MAX(ZAAA2*EXP(-(ZCCC2*ZRI5(JL,JK))**2),1.0_JPRB)
          ZDDD2 = 1.0_JPRB/ZDDD2

          PCFM(JL,JK) = PCFM(JL,JK)*ZDDD2
!or          PCFH(JL,JK) = PCFH(JL,JK)*ZDDD

          PCFH(JL,JK) = PCFH(JL,JK)*ZDDD1

        ENDIF
      ENDIF
    ENDIF

!           SCALE DIFFUSION COEFFICIENTS FOR USE IN VDFDIFH/M

    ZCFNC1 = ZCFNC1+ZPCFM25(JL,JK)*PCFM(JL,JK)
    PCFM(JL,JK) = ZCFNC15(JL,JK)*PCFM(JL,JK)
    ZCFNC1 = ZCFNC1+ZPCFH25(JL,JK)*PCFH(JL,JK)
    PCFH(JL,JK) = ZCFNC15(JL,JK)*PCFH(JL,JK)

    ZZD5 =  PTM15(JL,JK)*(1.0_JPRB+RETV*PQM15(JL,JK  )) &
     & +PTM15(JL,JK+1)*(1.0_JPRB+RETV*PQM15(JL,JK+1))  
    PAPHM1 (JL,JK) = PAPHM1(JL,JK) &
     & + ZCONS10*ZCFNC1/(ZMGEOM5(JL,JK)*ZZD5)  
    ZMGEOM(JL) = ZMGEOM(JL) - (ZCONS10*PAPHM15(JL,JK)*ZCFNC1)&
     & /(ZMGEOM5(JL,JK)**2*ZZD5)  
    PTM1(JL,JK) = PTM1(JL,JK) - (ZCONS10*PAPHM15(JL,JK) &
     & *(1.0_JPRB+RETV*PQM15(JL,JK))*ZCFNC1)/(ZMGEOM5(JL,JK)*ZZD5**2)  
    PQM1(JL,JK) = PQM1(JL,JK) - (ZCONS10*PAPHM15(JL,JK) &
     & *ZCFNC1*RETV*PTM15(JL,JK))/(ZMGEOM5(JL,JK)*ZZD5**2)  
    PTM1(JL,JK+1) = PTM1(JL,JK+1) - (ZCONS10*PAPHM15(JL,JK) &
     & *(1.0_JPRB+RETV*PQM15(JL,JK+1))*ZCFNC1) &
     & /(ZMGEOM5(JL,JK)*ZZD5**2)  
    PQM1(JL,JK+1) = PQM1(JL,JK+1) - (ZCONS10*PAPHM15(JL,JK) &
     & *ZCFNC1*RETV*PTM15(JL,JK+1))/(ZMGEOM5(JL,JK)*ZZD5**2) 

!           ADD ENTRAINMENT
!           entrainment velocity * T,v jump due to sfc flux

   IF (PGEOH5(JL,JK)*ZRG<=PZINV5(JL).AND.PZINV5(JL)<PGEOH5(JL,JK-1)*ZRG) THEN
      ZWTVENTR5(JL) = - ZENTRSFC * ZKHVFL5(JL)
      ZWTVENTR5(JL) = MAX(0.0_JPRB,ZWTVENTR5(JL))

      IF (0.75_JPRB*ZCFHNEW5(JL,JK) > ZPCFM15(JL,JK)) THEN
        ZCFHNEW = ZCFHNEW+0.75_JPRB*PCFM(JL,JK)
        PCFM(JL,JK) = 0.0_JPRB
      ENDIF
      IF (ZCFHNEW5(JL,JK) > ZPCFH15(JL,JK)) THEN !protection against K=0
        ZCFHNEW = ZCFHNEW+PCFH(JL,JK)
        PCFH(JL,JK) = 0.0_JPRB
      ENDIF

      IF (ZKH5(JL,JK) >= 0.0_JPRB) THEN
        ZKH = ZKH+ZCFHNEW
      ENDIF

      ZDTVR(JL) = ZDTVR(JL)+ZWTVENTR5(JL)*ZRG*ZMGEOM5(JL,JK)*ZKH
      ZMGEOM(JL) = ZMGEOM(JL)+ZWTVENTR5(JL)*ZRG*ZDTVR5(JL,JK)*ZKH
    ENDIF

!           ADD MIXED LAYER PARAMETRIZATION (SURF. AND OUTER LAYER)

    IF (PGEOH5(JL,JK-1)*ZRG <= PZINV5(JL) ) THEN  ! up to level below entr.lev.

!           K,surface
      ZDHZZ = ZDHZZ+ZPHIKMI5(JL,JK)*PCFM(JL,JK)
      ZPHIKMI = ZPHIKMI+ZDHZZ5(JL,JK)*PCFM(JL,JK)
      PCFM(JL,JK) = 0.0_JPRB
      ZPHIKM = ZPHIKM-RKAP*ZPHIKMI/ZPHIKM5(JL,JK)**2
      ZDHZZ = ZDHZZ+ZPHIKHI5(JL,JK)*PCFH(JL,JK)
      ZPHIKHI = ZPHIKHI+ZDHZZ5(JL,JK)*PCFH(JL,JK)
      PCFH(JL,JK) = 0.0_JPRB
      ZPHIKH = ZPHIKH-RKAP*ZPHIKHI/ZPHIKH5(JL,JK)**2
      ZZ = ZZ+ZUST5(JL)*ZDHZZ*(1.0_JPRB-ZDH5(JL,JK))**2
      ZDH = ZDH-2.0_JPRB*ZUST5(JL)*ZZ5(JL,JK)*(1.0_JPRB-ZDH5(JL,JK))*ZDHZZ

      ZDL = ZDL+15._JPRB*ZCONS13*ZPHIKM &
       & *(1.0_JPRB-15._JPRB*ZDL5(JL,JK))**(-ZCONS13-1.0_JPRB)
      ZDL = ZDL+39._JPRB*ZCONS13*ZPHIKH &
       & *(1.0_JPRB-39._JPRB*ZDL5(JL,JK))**(-ZCONS13-1.0_JPRB)
      ZZ = ZZ+ZLI5(JL)*ZDL
      ZZ = ZZ+ZDH/PZINV5(JL)
      PGEOH(JL,JK) = PGEOH(JL,JK)+ZRG*ZZ
    ENDIF

    IF (ZRI5(JL,JK) > 0.0_JPRB) THEN  ! statically stable

      IF (LLMONOB) THEN  ! Monin-Obuchov 

!           COMMON FACTORS FOR STABLE AND UNSTABLE
!           new K: K,LTG scales with l=kappa*z 
!                  K,MO  scales with l=150m above surface layer

        ZDUDZ = ZDUDZ+PCFH(JL,JK) &
         & *(ZBH5(JL,JK)*ZCH5(JL,JK)/(ZBH5(JL,JK)+ZCH5(JL,JK)))**2
        ZCH = ZCH+2.0_JPRB*ZDUDZ5(JL,JK)*ZBH5(JL,JK)*ZCH5(JL,JK) &
         & *(PCFH(JL,JK)*ZBH5(JL,JK)**2)/(ZBH5(JL,JK)+ZCH5(JL,JK))**3
        ZBH = ZBH+2.0_JPRB*ZDUDZ5(JL,JK)*ZBH5(JL,JK)*ZCH5(JL,JK) &
         & *(PCFH(JL,JK)*ZCH5(JL,JK)**2)/(ZBH5(JL,JK)+ZCH5(JL,JK))**3
        PCFH(JL,JK) = 0.0_JPRB
        ZDUDZ = ZDUDZ+PCFM(JL,JK) &
         & *(ZBM5(JL,JK)*ZCM5(JL,JK)/(ZBM5(JL,JK)+ZCM5(JL,JK)))**2
        ZCM = ZCM+2.0_JPRB*ZDUDZ5(JL,JK)*ZBM5(JL,JK)*ZCM5(JL,JK) &
         & *(PCFM(JL,JK)*ZBM5(JL,JK)**2)/(ZBM5(JL,JK)+ZCM5(JL,JK))**3
        ZBM = ZBM+2.0_JPRB*ZDUDZ5(JL,JK)*ZBM5(JL,JK)*ZCM5(JL,JK) &
         & *(PCFM(JL,JK)*ZCM5(JL,JK)**2)/(ZBM5(JL,JK)+ZCM5(JL,JK))**3
        PCFM(JL,JK) = 0.0_JPRB

        ZIFMOH = ZIFMOH+0.5_JPRB*ZLENMO*ZCH/ZSQRT65(JL,JK)
        ZIFMOM = ZIFMOM+0.5_JPRB*ZLENMO*ZCM/ZSQRT55(JL,JK)

        ZZH = ZZH+RKAP*ZSQRT45(JL,JK)*ZBH
        ZIFLTGH = ZIFLTGH+0.5_JPRB*RKAP*ZZH5(JL,JK)*ZBH/ZSQRT45(JL,JK)
        ZZH = ZZH+RKAP*ZSQRT35(JL,JK)*ZBM
        ZIFLTGM = ZIFLTGM+0.5_JPRB*RKAP*ZZH5(JL,JK)*ZBM/ZSQRT35(JL,JK)

        ZSCF = ZSCF-2.0_JPRB*ZCB*ZIFLTGH5(JL,JK)*ZIFLTGH5(JL,JK) &
         & *ZRI5(JL,JK)*ZIFLTGH
        ZRI(JL) = ZRI(JL)-2.0_JPRB*ZCB*ZIFLTGH5(JL,JK)*ZIFLTGH5(JL,JK) &
         & *ZSCF5(JL,JK)*ZIFLTGH
        ZSCF = ZSCF+2.0_JPRB*ZCB*ZRI5(JL,JK)*ZIFLTGM &
         & /(ZSCF5(JL,JK)+2.0_JPRB*ZCB*ZRI5(JL,JK))**2
!or        ZRI(JL) = ZRI(JL)+2.0_JPRB*ZCB*ZSCF5(JL,JK)*ZIFLTGM &
        ZRI(JL) = ZRI(JL)-2.0_JPRB*ZCB*ZSCF5(JL,JK)*ZIFLTGM &
         & /(ZSCF5(JL,JK)+2.0_JPRB*ZCB*ZRI5(JL,JK))**2
        ZRI(JL) = ZRI(JL)+ZCD*ZSCF/(2.0_JPRB*ZSCF5(JL,JK))
        PGEOM1(JL,JK) = PGEOM1(JL,JK)+0.5_JPRB*ZRG*ZZH
        PGEOM1(JL,JK+1) = PGEOM1(JL,JK+1)+0.5_JPRB*ZRG*ZZH
        PZ0MM(JL) = PZ0MM(JL)+ZZH

        ZDU2(JL) = ZDU2(JL)+0.5_JPRB*RG*ZDUDZ/(ZSQRT25(JL,JK)*ZMGEOM5(JL,JK))
        ZMGEOM(JL) = ZMGEOM(JL)-RG*ZSQRT25(JL,JK)*ZDUDZ/ZMGEOM5(JL,JK)**2
        ZPHIH(JL) = ZPHIH(JL) &
         & -ZPHIM5(JL,JK)*ZIFMOH5(JL,JK)*ZIFMOH5(JL,JK)*ZIFMOH
        ZPHIM(JL) = ZPHIM(JL) &
         & -ZPHIH5(JL,JK)*ZIFMOH5(JL,JK)*ZIFMOH5(JL,JK)*ZIFMOH
        ZPHIM(JL) = ZPHIM(JL)-2.0_JPRB*ZIFMOM/(ZPHIM5(JL,JK)**3)

      ELSE  ! Louis

!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

        ZCFNC = ZCFNC + PCFH(JL,JK)*ZALH25(JL,JK) &
         & /(1.0_JPRB + ZCONS9*ZRI5(JL,JK)*ZSCF5(JL,JK))
        ZALH2 = ZALH2 + PCFH(JL,JK)*ZCFNC5(JL,JK) &
         & /(1.0_JPRB + ZCONS9*ZRI5(JL,JK)*ZSCF5(JL,JK))
        ZRI(JL) = ZRI(JL) - PCFH(JL,JK)*ZCFNC5(JL,JK) &
         & *ZALH25(JL,JK)*ZCONS9*ZSCF5(JL,JK) &
         & /(1.0_JPRB+ZCONS9*ZRI5(JL,JK)*ZSCF5(JL,JK))**2
        ZSCF = ZSCF - PCFH(JL,JK)*ZCFNC5(JL,JK)*ZALH25(JL,JK) &
         & *ZCONS9*ZRI5(JL,JK) &
         & /(1.0_JPRB + ZCONS9*ZRI5(JL,JK)*ZSCF5(JL,JK))**2
        PCFH(JL,JK) = 0.0_JPRB
        ZCFNC = ZCFNC + PCFM(JL,JK)*ZALM25(JL,JK) &
         & /(1.0_JPRB+ZCONS8*ZRI5(JL,JK)/ZSCF5(JL,JK))
        ZALM2 = ZALM2 + PCFM(JL,JK)*ZCFNC5(JL,JK) &
         & /(1.0_JPRB+ZCONS8*ZRI5(JL,JK)/ZSCF5(JL,JK))
        ZRI (JL) = ZRI (JL) - PCFM(JL,JK)*ZCFNC5(JL,JK) &
         & *ZALM25(JL,JK)*ZCONS8/ZSCF5(JL,JK) &
         & /(1.0_JPRB+ZCONS8*ZRI5(JL,JK)/ZSCF5(JL,JK))**2
        ZSCF = ZSCF + PCFM(JL,JK)*ZCFNC5(JL,JK) &
         & *ZALM25(JL,JK)*ZCONS8*ZRI5(JL,JK) &
         & /(ZSCF5(JL,JK)+ZCONS8*ZRI5(JL,JK))**2
        PCFM(JL,JK) = 0.0_JPRB

        ZRI(JL) = ZRI(JL) + ZSCF*ZCD/(2.0_JPRB*ZSCF5(JL,JK))
        ZDU2(JL) = ZDU2(JL)+0.5_JPRB*RG*ZCFNC/(ZSQRT25(JL,JK)*ZMGEOM5(JL,JK))
        ZMGEOM(JL) = ZMGEOM(JL)-RG*ZSQRT25(JL,JK)*ZCFNC/ZMGEOM5(JL,JK)**2

        ZALM2 = ZALM2 + ZALH2
        ZLIM2 = ZLIM2 + ZALM2*((ZCONS2*Z2GEOMF5(JL,JK)) &
         & /(1.0_JPRB+ZCONS23*Z2GEOMF5(JL,JK)))**2
        Z2GEOMF = Z2GEOMF +  2.0_JPRB*ZALM2*ZLIM25(JL,JK) &
         & *ZCONS2*ZCONS2*Z2GEOMF5(JL,JK) &
         & /(1.0_JPRB+ZCONS23*Z2GEOMF5(JL,JK))**3
        ZLIM = ZLIM + 2.0_JPRB*ZLIM2*ZLIM5(JL,JK)
        Z2GEOMF = Z2GEOMF - 2.0_JPRB*ZLIM*ZUMBETA*Z2GEOMF5(JL,JK)/ZHLM2 &
         & /(1.0_JPRB+Z2GEOMF5(JL,JK)*Z2GEOMF5(JL,JK)/ZHLM2)**2
        PGEOM1 (JL,JK  ) = PGEOM1 (JL,JK  ) + Z2GEOMF
        PGEOM1 (JL,JK+1) = PGEOM1 (JL,JK+1) + Z2GEOMF
        PZ0MM  (JL)      = PZ0MM  (JL)      + Z2GEOMF*ZCONSHG

      ENDIF

    ELSE  ! statically unstable

!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

      Z6S = Z6S + PCFH(JL,JK)*(1.0_JPRB-(ZCONS9*ZRI5(JL,JK)) &
       & /(1.0_JPRB+Z4S5(JL,JK)*Z2S5(JL,JK)))
      ZRI(JL) = ZRI(JL) - PCFH(JL,JK)*Z6S5(JL,JK)*ZCONS9 &
       & /(1.0_JPRB+Z4S5(JL,JK)*Z2S5(JL,JK))
      Z4S = Z4S + PCFH(JL,JK)*Z6S5(JL,JK)*ZCONS9*Z2S5(JL,JK) &
       & *ZRI5(JL,JK)/(1.0_JPRB+Z4S5(JL,JK)*Z2S5(JL,JK))**2
      Z2S = Z2S + PCFH(JL,JK)*Z6S5(JL,JK)*ZCONS9*Z4S5(JL,JK) &
       & *ZRI5(JL,JK)/(1.0_JPRB+Z4S5(JL,JK)*Z2S5(JL,JK))**2
      PCFH(JL,JK) = 0.0_JPRB
      ZALH2 = ZALH2 + ZCFNC5(JL,JK)*Z6S
      ZCFNC = ZCFNC + ZALH25(JL,JK)*Z6S
      Z5S = Z5S + PCFM(JL,JK)*(1.0_JPRB-(ZCONS8*ZRI5(JL,JK)) &
       & /(1.0_JPRB+Z3S5(JL,JK)*Z2S5(JL,JK)))
      ZRI (JL) = ZRI (JL) - PCFM(JL,JK)*Z5S5(JL,JK)*ZCONS8 &
       & /(1.0_JPRB+Z3S5(JL,JK)*Z2S5(JL,JK))
      Z3S = Z3S + PCFM(JL,JK)*Z5S5(JL,JK)*ZCONS8*Z2S5(JL,JK) &
       & *ZRI5(JL,JK)/(1.0_JPRB+Z3S5(JL,JK)*Z2S5(JL,JK))**2
      Z2S = Z2S + PCFM(JL,JK)*Z5S5(JL,JK)*ZCONS8*Z3S5(JL,JK) &
       & *ZRI5(JL,JK)/(1.0_JPRB+Z3S5(JL,JK)*Z2S5(JL,JK))**2
      PCFM(JL,JK) = 0.0_JPRB
      ZALM2 = ZALM2 + ZCFNC5(JL,JK)*Z5S
      ZCFNC = ZCFNC + ZALM25(JL,JK)*Z5S
      ZALH2 = ZALH2 + Z4S*ZCONS11*ZZI25(JL,JK)
      ZZI2  = ZZI2 +  Z4S*ZCONS11*ZALH25(JL,JK)
      ZALM2 = ZALM2 + Z3S*ZCONS11*ZZI25(JL,JK)
      ZZI2  = ZZI2 +  Z3S*ZCONS11*ZALM25(JL,JK)

!   Threshold for minimum Richardson number

      ZRI(JL) = ZRI(JL) - Z2S/(2.0_JPRB*SQRT(MAX(-ZRI5(JL,JK),ZRIMIN)))
      ZZI  = ZZI + 2.0_JPRB*ZZI2*ZZI5(JL,JK)
      Z2GEOMF = Z2GEOMF - ZZI*ZCONSHG/Z2GEOMF5(JL,JK)**2

!           COMMON FACTORS FOR STABLE AND UNSTABLE

      ZDU2(JL) = ZDU2(JL)+0.5_JPRB*RG*ZCFNC/(ZSQRT25(JL,JK)*ZMGEOM5(JL,JK))
      ZMGEOM(JL) = ZMGEOM(JL)-RG*ZSQRT25(JL,JK)*ZCFNC/ZMGEOM5(JL,JK)**2
      ZALM2 = ZALM2 + ZALH2
      ZLIM2 = ZLIM2 + ZALM2*((ZCONS2*Z2GEOMF5(JL,JK)) &
       & /(1.0_JPRB+ZCONS23*Z2GEOMF5(JL,JK)))**2
      Z2GEOMF = Z2GEOMF +  2.0_JPRB*ZALM2*ZLIM25(JL,JK) &
       & *ZCONS2*ZCONS2*Z2GEOMF5(JL,JK) &
       & /(1.0_JPRB+ZCONS23*Z2GEOMF5(JL,JK))**3
      ZLIM = ZLIM + 2.0_JPRB*ZLIM2*ZLIM5(JL,JK)
      Z2GEOMF = Z2GEOMF - 2.0_JPRB*ZLIM*ZUMBETA*Z2GEOMF5(JL,JK)/ZHLM2 &
       & /(1.0_JPRB+Z2GEOMF5(JL,JK)*Z2GEOMF5(JL,JK)/ZHLM2)**2
      PGEOM1 (JL,JK  ) = PGEOM1 (JL,JK  ) + Z2GEOMF
      PGEOM1 (JL,JK+1) = PGEOM1 (JL,JK+1) + Z2GEOMF
      PZ0MM  (JL)      = PZ0MM  (JL)      + Z2GEOMF*ZCONSHG

    ENDIF
  ENDDO

  DO JL = KIDIA, KFDIA
    ZETA = 0.0_JPRB
    ZA = 0.0_JPRB
    ZB = 0.0_JPRB

    IF (ZRI5(JL,JK) > 0.0_JPRB .AND. LLMONOB) THEN

!           STABLE PHI-FUNCTIONS

!err      ZETA = ZETA+2.0_JPRB*RCHETB*ZPHIH(JL) &
!err       & *(1.0_JPRB+RCHETB*ZETA5(JL,JK))**2
      ZETA = ZETA+2.0_JPRB*RCHETB*ZPHIH(JL) &
       & *(1.0_JPRB+RCHETB*ZETA5(JL,JK))
      ZPHIH(JL) = 0.0_JPRB
      ZETA = ZETA+RCHETA*ZPHIM(JL)
      ZPHIM(JL) = 0.0_JPRB
    
!           COMPUTE STABILITY FUNCTIONS

      IRIB=INT(ZRI5(JL,JK)*(1.0_JPRB/DRITBL))+1
      IF (IRIB  >=  JPRITBL) THEN
!           LINEAR EXTENSION OF LOOK-UP TABLE

        ZRI(JL) = ZRI(JL)+RITBL(JPRITBL)*(ZETA*(1.0_JPRB/RIMAX))
      ELSE
        ZA = ZA+(RITBL(IRIB) &
         & +DRI26*(3.0_JPRB*ZA5(JL,JK)**2-1.0_JPRB)*ARITBL(IRIB))*ZETA
        ZB = ZB+(RITBL(IRIB+1) &
         & +DRI26*(3.0_JPRB*ZB5(JL,JK)**2-1.0_JPRB)*ARITBL(IRIB+1))*ZETA
        ZA = ZA-ZB
        ZRI(JL) =ZRI(JL)-ZA*(1.0_JPRB/DRITBL)
      ENDIF
    ENDIF

  ENDDO

  DO JL = KIDIA, KFDIA

!         INITIALISATIONS OF LOCAL VARIABLES

    ZDRORO   = 0.0_JPRB
    Z1S      = 0.0_JPRB
    ZDUDZ1   = 0.0_JPRB
    ZDUDZ2   = 0.0_JPRB

!*          COMPUTE RI-NUMBER

    ZDRORO = ZDRORO + ZRI(JL)*ZMGEOM5(JL,JK)/ZDU25(JL,JK)
    ZMGEOM(JL) = ZMGEOM(JL) + ZRI(JL)*ZDRORO5(JL,JK)/ZDU25(JL,JK)
    ZDU2(JL) = ZDU2(JL) - ZRI(JL)*ZMGEOM5(JL,JK) &
     & *ZDRORO5(JL,JK)/ZDU25(JL,JK)**2
    ZRI (JL) = 0.0_JPRB

! for regularization
    ZDTVR(JL) = 0.0_JPRB

    ZDTV(JL) = ZDTV(JL)-RCPD*ZDTVR(JL)/ZDTV5(JL,JK)**2
    ZDTVR(JL) = 0.0_JPRB
    PCPTGZ(JL,JK) = PCPTGZ(JL,JK) &
     & +(1.0_JPRB+0.5_JPRB*RETV*(PQM15(JL,JK)-PQM15(JL,JK+1)))*ZDTV(JL)
    PCPTGZ(JL,JK+1) = PCPTGZ(JL,JK+1) &
     & +(0.5_JPRB*RETV*(PQM15(JL,JK)-PQM15(JL,JK+1))-1.0_JPRB)*ZDTV(JL)
    PQM1(JL,JK) = PQM1(JL,JK) &
     & +0.5_JPRB*RETV*(PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1))*ZDTV(JL)
    PQM1(JL,JK+1) = PQM1(JL,JK+1) &
     & -0.5_JPRB*RETV*(PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1))*ZDTV(JL)
    ZDTV(JL) = 0.0_JPRB

    PQM1 (JL,JK) = PQM1 (JL,JK) + RETV*ZDRORO
    PQM1 (JL,JK+1) = PQM1(JL,JK+1) - RETV*ZDRORO
    PGEOM1(JL,JK) = PGEOM1(JL,JK) + ZDRORO*&
     & 2.0_JPRB*(PCPTGZ5(JL,JK)-PCPTGZ5(JL,JK+1))/&
     & (PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1)-&
     & PGEOM15(JL,JK)-PGEOM15(JL,JK+1))**2  
    PGEOM1(JL,JK+1) = PGEOM1(JL,JK+1) + ZDRORO*&
     & 2.0_JPRB*(PCPTGZ5(JL,JK)-PCPTGZ5(JL,JK+1))/&
     & (PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1)-&
     & PGEOM15(JL,JK)-PGEOM15(JL,JK+1))**2       
    PCPTGZ(JL,JK) = PCPTGZ(JL,JK) + ZDRORO*&
     & 2.0_JPRB*(2.0_JPRB*PCPTGZ5(JL,JK+1)-PGEOM15(JL,JK)-&
     & PGEOM15(JL,JK+1))/&
     & (PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1)-&
     & PGEOM15(JL,JK)-PGEOM15(JL,JK+1))**2                
    PCPTGZ(JL,JK+1) = PCPTGZ(JL,JK+1) - ZDRORO*&
     & 2.0_JPRB*(2.0_JPRB*PCPTGZ5(JL,JK)-PGEOM15(JL,JK)-&
     & PGEOM15(JL,JK+1))/&
     & (PCPTGZ5(JL,JK)+PCPTGZ5(JL,JK+1)-&
     & PGEOM15(JL,JK)-PGEOM15(JL,JK+1))**2

    ZMGEOM(JL) = ZMGEOM(JL)+2.0_JPRB*(ZDUDZ25(JL,JK) &
     & *ZMGEOM5(JL,JK)*ZRG**2)*ZDUDZ25(JL,JK)*ZDU2(JL)
    ZDUDZ2 = ZDUDZ2+2.0_JPRB*(ZDUDZ25(JL,JK) &
     & *ZMGEOM5(JL,JK)*ZRG**2)*ZMGEOM5(JL,JK)*ZDU2(JL)
    ZDU2(JL) = 0.0_JPRB

    ZDUDZ1 = ZDUDZ1+ZDUDZ2
!    PAPHM1(JL,JK) = PAPHM1(JL,JK)+0.005_JPRB*ZDUDZ2/PAPHM15(JL,KLEV)
!    PAPHM1(JL,KLEV) = PAPHM1(JL,KLEV)-0.005_JPRB*PAPHM15(JL,JK)*ZDUDZ2 &
!     & /PAPHM15(JL,KLEV)**2
    ZDU1(JL) = ZDU1(JL)+0.5_JPRB*RG*ZDUDZ1/(ZSQRT15(JL,JK)*ZMGEOM5(JL,JK))
    ZMGEOM(JL) = ZMGEOM(JL)-RG*ZSQRT15(JL,JK)*ZDUDZ1/ZMGEOM5(JL,JK)**2
    IF (REPDU2 >= Z1S5(JL,JK)) THEN
      ZDU1(JL) = 0.0_JPRB
    ELSE
      Z1S = Z1S + ZDU1(JL)
      ZDU1(JL) = 0.0_JPRB
    ENDIF
    PUM1(JL,JK) = PUM1(JL,JK) + 2.0_JPRB*Z1S*(PUM15(JL,JK)-PUM15(JL,JK+1))
    PUM1(JL,JK+1) = PUM1(JL,JK+1) - 2.0_JPRB*Z1S*(PUM15(JL,JK)-PUM15(JL,JK+1))
    PVM1(JL,JK) = PVM1(JL,JK) + 2.0_JPRB*Z1S*(PVM15(JL,JK)-PVM15(JL,JK+1))
    PVM1 (JL,JK+1) = PVM1(JL,JK+1) - 2.0_JPRB*Z1S*(PVM15(JL,JK)-PVM15(JL,JK+1))

    PGEOM1 (JL,JK) = PGEOM1 (JL,JK) + ZMGEOM (JL)
    PGEOM1 (JL,JK+1) = PGEOM1 (JL,JK+1) - ZMGEOM (JL)
    ZMGEOM (JL) = 0.0_JPRB
  ENDDO

  DO JL = KIDIA, KFDIA
    PCFM (JL,JK) = 0.0_JPRB
    PCFH (JL,JK) = 0.0_JPRB
  ENDDO

ENDDO

IF (LHOOK) CALL DR_HOOK('VDFEXCUSAD',1,ZHOOK_HANDLE)
END SUBROUTINE VDFEXCUSAD

