!OPTIONS XOPT(HSFUN)
SUBROUTINE VDFMAINSTL (KIDIA,KFDIA,KLON,KLEV,KLEVS,KSTEP,KTILES,&
 & KTRAC,PTSPHY,KTVL,KTVH,PCVL,PCVH,&
 & PLAIL, PLAIH, PSIGFLT,&
 & PUM15  ,PVM15  ,PTM15  ,PQM15  ,PCM15  ,PAPHM15  ,PGEOM15, PGEOH5 ,&
 & PTSKM1M5,PTSAM1M5,PWSAM1M5, &
 & PSSRFL5,PSLRFL5 ,PEMIS5 ,&
 & PTSNOW5, PTICE5,&
 ! & PHLICE5, PTLICE5, PTLWML5, &  !lake passive
 & PSST   ,KSOTY  , PFRTI ,PALBTI5 ,PWLMX5 ,&
 & PCHAR ,PTSKRAD5, PCFLX5,&
 ! OUTPUT (TRAJECTORY)           
 & PZ0M5   ,PZ0H5 , &
 & PU10M5  , PV10M5  , PT2M5   , PD2M5   , PQ2M5 ,&
 & P10NU5  , P10NV5  ,&
 ! OUTPUT TENDENCIES (TRAJECTORY)           
 & PTE5   ,PQE5   ,PVOM5  ,PVOL5 ,PTENC5  ,PTSKE15,&
 !-UPDATED FIELDS FOR TILES (TRAJECTORY)
 & PUSTRTI5,PVSTRTI5,PAHFSTI5,PEVAPTI5,PTSKTI5,&
 ! OUTPUT FLUXES (TRAJECTORY)           
 & PDIFTS5,PDIFTQ5,PSTRTU5,PSTRTV5, &
 ! ========         
 & PUM1   ,PVM1   ,PTM1   ,PQM1  ,PCM1   ,PAPHM1   ,PGEOM1, PGEOH ,&
 & PTSKM1M,PTSAM1M,PWSAM1M, &
 & PSSRFL,PSLRFL, &
 & PALBTI,PTSKRAD,PCFLX, &
 ! OUTPUT           
 & PZ0M   ,PZ0H   , &
 & PU10M  , PV10M  , PT2M   , PD2M   , PQ2M ,&
 & P10NU  , P10NV  ,&
 ! OUTPUT TENDENIES           
 & PTE    ,PQE    ,PVOM   ,PVOL  ,PTENC  ,PTSKE1, &
 !-UPDATED FIELDS FOR TILES
 & PUSTRTI,PVSTRTI,PAHFSTI,PEVAPTI,PTSKTI,&
 ! OUTPUT FLUXES
 & PDIFTS ,PDIFTQ ,PSTRTU ,PSTRTV)  
!-----------------------------------------------------------------------     

!***

!**   *VDFMAINSTL* - DOES THE VERTICAL EXCHANGE OF U,V,T,Q BY TURBULENCE.
!                   (Tangent linear)

!     J.F. MAHFOUF     E.C.M.W.F.         05/10/95

!     Modified TL code

!     M. Janiskova                        11/06/99 (TL for surface tile)
!     M. Janiskova                        03/09/03 (TL of new tile coupling)
!     P. Viterbo                          24/05/2004 (Change surface units)
!     P. LOPEZ                            06/06/05 (Cleaning )
!     M. Janiskova                        27/06/05 (Rewritten for externalized
!                                                   surface)
!     A. Beljaars                         10/12/05 TOFD
!     P. Lopez                            13/02/2006 (Added tracer diffusion)
!     G. Balsamo                          15/01/2007 (Added soil type)
!     M. Janiskova                        02/10/2007 (PBL height computation)
!     G. Balsamo                          20/10/2008 (Added lake tile)
!     H. Hersbach                         04/12/2009 (10-m neutral wind)
!     S. Boussetta/G.Balsamo              May 2009   (Add variable LAI)

! --------------------

!     adapted from

!     J.F.GELEYN       E.C.M.W.F.         20/04/82.
!     MODIFIED BY
!     C.A.BLONDIN      E.C.M.W.F.         18/12/86.
!     IFS-VERSION (TECHNICAL REVISION OF *CY34*) BY
!     A.C.M. BELJAARS  E.C.M.W.F.         20/10/89.

!     OBUKHOV-L UPDATE      ACMB          26/03/90.

!     MODIFIED BY

!     A.C.M. BELJAARS  E.C.M.W.F.         30/09/98 (SURFACE TILES)

!     PURPOSE.
!     --------

!          THIS ROUTINE COMPUTES THE PHYSICAL TENDENCIES OF THE FOUR
!     PROGNOSTIC VARIABLES U,V,T AND Q DUE TO THE VERTICAL EXCHANGE BY
!     TURBULENT (= NON-MOIST CONVECTIVE) PROCESSES. THESE TENDENCIES ARE
!     OBTAINED AS THE DIFFERENCE BETWEEN THE RESULT OF AN IMPLICIT
!     TIME-STEP STARTING FROM VALUES AT T-1 AND THESE T-1 VALUES. ALL
!     THE DIAGNOSTIC COMPUTATIONS (EXCHANGE COEFFICIENTS, ...) ARE DONE
!      FROM THE T-1 VALUES. AS A BY-PRODUCT THE ROUGHNESS LENGTH OVER SEA
!     IS UPDATED ACCORDINGLY TO THE *CHARNOCK FORMULA. HEAT AND MOISTURE
!     SURFACE FLUXES AND THEIR DERIVATIVES AGAINST TS, WS AND WL
!     (THE LATTER WILL BE LATER WEIGHTED WITH THE SNOW FACTOR IN
!     *VDIFF*), LATER TO BE USED FOR SOIL PROCESSES TREATMENT, ARE ALSO
!     COMPUTED AS WELL AS A STABILITY VALUE TO BE USED AS A DIAGNOSTIC
!     OF THE DEPTH OF THE WELL MIXED LAYER IN CONVECTIVE COMPUTATIONS.

!     INTERFACE.
!     ----------
!          *VDIFF* TAKES THE MODEL VARIABLES AT T-1 AND RETURNS THE VALUES
!     FOR THE PROGNOSTIC TIME T+1 DUE TO VERTICAL DIFFUSION.
!     THE MODEL VARIABLES, THE MODEL DIMENSIONS AND THE DIAGNOSTICS DATA
!     ARE PASSED AS SUBROUTINE ARGUMENTS. CONSTANTS THAT DO NOT CHANGE
!     DURING A MODEL RUN (E.G. PHYSICAL CONSTANTS, SWITCHES ETC.) ARE
!     STORED IN A SINGLE COMMON BLOCK *YOMVDF*, WHICH IS INITIALIZED
!     BY SET-UP ROUTINE *SUVDF*.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLEV*         NUMBER OF LEVELS
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEVS*         NUMBER OF SOIL LAYERS
!    *KSTEP*        CURRENT TIME STEP INDEX
!    *KTILES*       NUMBER OF TILES (I.E. SUBGRID AREAS WITH DIFFERENT
!                   OF SURFACE BOUNDARY CONDITION)
!    *KTRAC*        NUMBER OF TRACERS

!    *KTVL*         VEGETATION TYPE FOR LOW VEGETATION FRACTION
!    *KTVH*         VEGETATION TYPE FOR HIGH VEGETATION FRACTION
!    *KSOTY*        SOIL TYPE                                         (1-7)

!     INPUT PARAMETERS (LOGICAL)

!     INPUT PARAMETERS (REAL)
!    *PCVL*         LOW VEGETATION COVER                          -  
!    *PCVH*         HIGH VEGETATION COVER                         -  
!    *PLAIL*        LOW VEGETATION LAI                            m2/m2
!    *PLAIH*        HIGH VEGETATION LAI                           m2/m2
!    *PSIGFLT*      STANDARD DEVIATION OF FILTERED OROGRAPHY      M

!    *PTSPHY*       TIME STEP FOR THE PHYSICS

!*      INPUT PARAMETERS AT T-1 OR CONSTANT IN TIME (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PUM15       PUM1          X-VELOCITY COMPONENT                      m/s
!  PVM15       PVM1          Y-VELOCITY COMPONENT                      m/s
!  PTM15       PTM1          TEMPERATURE                               K
!  PQM15       PQM1          SPECIFIC HUMIDITY                         kg/kg
!  PCM15       PCM1          TRACER CONCENTRATION                      kg/kg
!  PAPHM15     PAPHM1        PRESSURE                                  Pa
!  PGEOM15     PGEOM1        GEOPOTENTIAL                             m**2/s**2
!  PGEOH5      PGEOH         GEOPOTENTIAL AT HALF LEVELS              m**2/s**2
!  PTSKM1M5    PTSKM1M       SKIN TEMPERATURE                          K
!  PTSKRAD5    PTSKRAD       SKIN TEMPERATURE at last ful radiation    K
!                            time step
!  PTSAM1M5    PTSAM1M       SURFACE TEMPERATURE                       K
!  PWSAM1M5    PWSAM1M       SOIL MOISTURE ALL LAYERS                 m**3/m**3
!  PSSRFL5     PSSRFL        NET SHORTWAVE RADIATION FLUX AT SURFACE   W/m2
!  PSLRFL5     PSLRFL        NET LONGWAVE RADIATION FLUX AT SURFACE    W/M2
!  PEMIS5      ---           MODEL SURFACE LONGWAVE EMISSIVITY         
!  PTSNOW5     ---           SNOW TEMPERATURE                          K
!  PTICE5      ---           ICE TEMPERATURE (TOP SLAB)                K
!  PHLICE5     ---           LAKE ICE THICKNESS                        m
!  PTLICE5     ---           LAKE ICE TEMPERATURE                      K
!  PTLWML5     ---           LAKE MIXED LAYER TEMPERATURE              K
!  PSST        ---           (OPEN) SEA SURFACE TEMPERATURE            K
!  PFRTI       ---           TILE FRACTIONS                            (0-1)
!                            1: WATER         5: SNOW ON LOW-VEG+BARE-SOIL
!                            2: ICE           6: DRY SNOW-FREE HIGH-VEG
!                            3: WET SKIN      7: SNOW UNDER HIGH-VEG
!                            4: DRY SNOW-FREE 8: BARE SOIL
!                               LOW-VEG
!  PALBTI5     PALBTI        BROADBAND ALBEDO FOR TILE FRACTIONS       (0-1)
!  PWLMX5      ---           MAXIMUM SKIN RESERVOIR CAPACITY           kg/m**2
!  PCHAR       ---           "EQUIVALENT" CHARNOCK PARAMETER
!  PCFLX5      PCFLX         TRACER SURFACE FLUX                       kg/(m2 s)

!*      UPDATED PARAMETERS (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PTE5        PTE           TEMPERATURE TENDENCY                      K/s
!  PQE5        PQE           MOISTURE TENDENCY                        kg/(kg s)
!  PVOM5       PVOM          MERIODINAL VELOCITY TENDENCY (DU/DT)      m/s2
!  PVOL5       PVOL          LATITUDE TENDENCY            (DV/DT)      m/s2
!  PTENC5      PTENC         TRACER TENDENCY                           kg/(kg s)
!  PTSKE15     PTSKE1        SKIN TEMPERATURE TENDENCY                 K/s
!  PZ0M5       PZ0M          AERODYNAMIC ROUGHNESS LENGTH              m
!  PZ0H5       PZ0H          ROUGHNESS LENGTH FOR HEAT                 m

!*      UDATED PARAMETERS FOR TILES (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PUSTRTI5    PUSTRTI       SURFACE U-STRESS                          N/m2
!  PVSTRTI5    PVSTRTI       SURFACE V-STRESS                          N/m2
!  PAHFSTI5    PAHFSTI       SURFACE SENSIBLE HEAT FLUX                W/m2
!  PEVAPTI5    PEVAPTI       SURFACE MOISTURE FLUX                     kg/m2/s
!  PTSKTI5     PTSKTI        SKIN TEMPERATURE                          K

!*      OUTPUT PARAMETERS (REAL):
!  Trajectory  Perturbation  Description                               Unit
!  PSTRTU5     PSTRTU        TURBULENT FLUX OF U-MOMEMTUM       kg*(m/s)/(m2*s)
!  PSTRTV5     PSTRTV        TURBULENT FLUX OF V-MOMEMTUM       kg*(m/s)/(m2*s)
!  PDIFTS5     PDIFTS        TURBULENT FLUX OF HEAT                    J/(m2*s)
!  PDIFTQ5     PDIFTQ        TRUBULENT FLUX OF SPECIFIC HUMIDITY      kg/(m2*s)

!     METHOD.
!     -------

!     SEE VDFMAIN

!     --------------------------------------------------------
!     THIS TANGENT LINEAR VERSION IS ONLY COMPATIBLE WITH THE
!     VERTICAL DIFFUSION SCHEME VDFMAINS
!     --------------------------------------------------------

!     EXTERNALS.
!     ----------

!     *VDFMAINSTL* CALLS SUCESSIVELY:
!         *SURFEXCDRIVERSTL* -----> *VUPDZ0*
!                              |--> *VSURF*
!                              |--> *VEXCSS*
!                              |--> *VEVAP*
!         *VDFDPBLS*
!         *VDFEXCUSTL*
!         *VDFDIFMSTL*
!         *VDFDIFHSTL*
!         *VDFINCRSTL*
!         *VDFDIFCSTL*
!         *SURFPPSTL* -----> *SPPCFLSTL*

!     REFERENCE.
!     ----------

!          SEE VERTICAL DIFFUSION'S PART OF THE MODEL'S DOCUMENTATION
!     FOR DETAILS ABOUT THE MATHEMATICS OF THIS ROUTINE.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEVDF   , ONLY : RVDIFTS
USE YOMCST   , ONLY : RG       ,RCPD     ,RETV     ,RLVTT    ,&
 & RLSTT    ,RTT  
USE YOETHF   , ONLY : RVTMP2   ,R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,&
 & R5ALSCP  ,RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOEPHLI  , ONLY : LENOPERT, LEPPCFLS
USE YOEPHY   , ONLY : LVDFTRAC
USE YOPHNC   , ONLY : LEKPERT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAC
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTVL(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTVH(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSOTY(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVH(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAIL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAIH(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIGFLT(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCM15(KLON,KLEV,KTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM15(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSKM1M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSAM1M5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSAM1M5(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSSRFL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLRFL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSNOW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTICE5(KLON) 
!REAL(KIND=JPRB)   ,INTENT(IN)    :: PHLICE5(KLON) !lake passive
!REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLICE5(KLON) !lake passive
!REAL(KIND=JPRB)   ,INTENT(IN)    :: PTLWML5(KLON) !lake passive
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSST(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWLMX5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCHAR(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSKRAD5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCFLX5(KLON,KTRAC)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0H5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P10NU5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P10NV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT2M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PD2M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ2M5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVOM5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVOL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENC5(KLON,KLEV,KTRAC)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKE15(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSTRTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSTRTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAHFSTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVAPTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKTI5(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQ5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTU5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCM1(KLON,KLEV,KTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSKM1M(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSAM1M(KLON,KLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSAM1M(KLON,KLEVS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSSRFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLRFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSKRAD(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCFLX(KLON,KTRAC)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0H(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10M (KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P10NU (KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P10NV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT2M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PD2M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ2M(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVOM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVOL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENC(KLON,KLEV,KTRAC)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKE1(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSTRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSTRTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAHFSTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVAPTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSKTI(KLON,KTILES) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQ(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTV(KLON,0:KLEV) 


!*         0.     LOCAL VARIABLES.
!                 ----- ----------

REAL(KIND=JPRB) :: ZVDIS5(KLON)
REAL(KIND=JPRB) :: ZCPTGZ5(KLON,KLEV), ZCFM5(KLON,KLEV)  , ZCFH5(KLON,KLEV)
REAL(KIND=JPRB) :: ZUDIF5(KLON,KLEV) , ZVDIF5(KLON,KLEV) 
REAL(KIND=JPRB) :: ZTDIF5(KLON,KLEV) , ZQDIF5(KLON,KLEV) , ZTOFDC5(KLON,KLEV)   
REAL(KIND=JPRB) :: ZKHFL5(KLON)      , ZKQFL5(KLON)      , ZKMFL5(KLON) 
REAL(KIND=JPRB) :: ZQEA5(KLON,KLEV), ZTEA5(KLON,KLEV), ZTEWODIS5(KLON,KLEV)
REAL(KIND=JPRB) :: ZUEA5(KLON,KLEV), ZVEA5(KLON,KLEV)
REAL(KIND=JPRB) :: ZZ0MW5(KLON) , ZZ0HW5(KLON), ZZ0QW5(KLON)
REAL(KIND=JPRB) :: ZZCPTS5(KLON), ZZQSA5(KLON), ZZBUOM5(KLON)

REAL(KIND=JPRB) :: ZCPTSTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZQSTI5(KLON,KTILES) , ZDQSTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZCSATTI5(KLON,KTILES), ZCAIRTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZCFHTI5(KLON,KTILES) , ZCFQTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZAHFLTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZTSKTIP15(KLON,KTILES),ZQSTIP15(KLON,KTILES)
REAL(KIND=JPRB) :: ZCPTSTIP15(KLON,KTILES),ZSSRFLTI5(KLON,KTILES)
REAL(KIND=JPRB) :: ZSTR5(KLON,KTILES),ZG05(KLON,KTILES)
REAL(KIND=JPRB) :: ZPEVAPSNW5(KLON)
REAL(KIND=JPRB) :: ZPBLH5(KLON), ZZINV5(KLON), ZKHVFL5(KLON)

REAL(KIND=JPRB) :: ZVDIS(KLON)
REAL(KIND=JPRB) :: ZCPTGZ(KLON,KLEV), ZCFM(KLON,KLEV)  , ZCFH(KLON,KLEV)
REAL(KIND=JPRB) :: ZUDIF(KLON,KLEV) , ZVDIF(KLON,KLEV)
REAL(KIND=JPRB) :: ZTDIF(KLON,KLEV) , ZQDIF(KLON,KLEV) , ZTOFDC(KLON,KLEV)   
REAL(KIND=JPRB) :: ZQEA(KLON,KLEV), ZTEA(KLON,KLEV), ZTEWODIS(KLON,KLEV)
REAL(KIND=JPRB) :: ZUEA(KLON,KLEV), ZVEA(KLON,KLEV)
REAL(KIND=JPRB) :: ZZ0MW(KLON) , ZZ0HW(KLON), ZZ0QW(KLON)
REAL(KIND=JPRB) :: ZZCPTS(KLON), ZZQSA(KLON), ZZBUOM(KLON)

REAL(KIND=JPRB) :: ZCPTSTI(KLON,KTILES)
REAL(KIND=JPRB) :: ZQSTI(KLON,KTILES) , ZDQSTI(KLON,KTILES)
REAL(KIND=JPRB) :: ZCSATTI(KLON,KTILES), ZCAIRTI(KLON,KTILES)
REAL(KIND=JPRB) :: ZCFHTI(KLON,KTILES) , ZCFQTI(KLON,KTILES)
REAL(KIND=JPRB) :: ZAHFLTI(KLON,KTILES)
REAL(KIND=JPRB) :: ZTSKTIP1(KLON,KTILES),ZQSTIP1(KLON,KTILES)
REAL(KIND=JPRB) :: ZCPTSTIP1(KLON,KTILES),ZSSRFLTI(KLON,KTILES)
REAL(KIND=JPRB) :: ZSTR(KLON,KTILES),ZG0(KLON,KTILES)
REAL(KIND=JPRB) :: ZPEVAPSNW(KLON)

INTEGER(KIND=JPIM) :: ITOP, JK, JL

REAL(KIND=JPRB) :: ZCP, ZCP5, ZGDPH, ZGDPH5, ZTMST

REAL(KIND=JPRB) :: ZRG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "surfexcdriverstl.h"
#include "surfppstl.h"
END INTERFACE

#include "vdfdifhstl.intfb.h"
#include "vdfdifmstl.intfb.h"
#include "vdfexcustl.intfb.h"
#include "vdftofdcs.intfb.h"
#include "vdfincrstl.intfb.h"
#include "vdfdifcstl.intfb.h"
#include "vdfdpbls.intfb.h"

!     ------------------------------------------------------------------

#include "fcttre.h"

!     ------------------------------------------------------------------

!*         1.     INITIALIZE CONSTANTS
!                 ---------- ---------

IF (LHOOK) CALL DR_HOOK('VDFMAINSTL',0,ZHOOK_HANDLE)
ZTMST=PTSPHY
ZRG=-1.0_JPRB/RG

!          1.1  GET PROVISONAL T+DT WINDS

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZQEA (JL,JK)=PQE (JL,JK)
    ZQEA5(JL,JK)=PQE5(JL,JK)
    ZTEA (JL,JK)=PTE (JL,JK)
    ZTEA5(JL,JK)=PTE5(JL,JK)
    ZUEA (JL,JK)=PVOM (JL,JK)
    ZUEA5(JL,JK)=PVOM5(JL,JK)
    ZVEA (JL,JK)=PVOL (JL,JK)
    ZVEA5(JL,JK)=PVOL5(JL,JK)
  ENDDO
ENDDO

!*         1.2  dry static energy cp(q)*T + gz

DO JK = 1, KLEV
  DO JL = KIDIA, KFDIA
    ZCPTGZ(JL,JK)=&
     & PGEOM1 (JL,JK)+PTM1 (JL,JK)*RCPD*(1.0_JPRB+RVTMP2*PQM15(JL,JK)) +&
     & PTM15(JL,JK)*RCPD*    RVTMP2*PQM1 (JL,JK)  
    ZCPTGZ5(JL,JK)=&
     & PGEOM15(JL,JK)+PTM15(JL,JK)*RCPD*(1.0_JPRB+RVTMP2*PQM15(JL,JK)) 
  ENDDO
ENDDO


CALL SURFEXCDRIVERSTL( &
 & KIDIA=KIDIA, KFDIA=KFDIA, KLON=KLON, KLEVS=KLEVS, KTILES=KTILES, &
 & KSTEP=KSTEP, &
 & PTSTEP=PTSPHY, PRVDIFTS=RVDIFTS, LDNOPERT=LENOPERT, LDKPERT=LEKPERT, &
! input data, non-tiled - trajectory
 & KTVL=KTVL, KTVH=KTVH, PCVL=PCVL, PCVH=PCVH, &
 & PLAIL=PLAIL, PLAIH=PLAIH,&
 & PUMLEV5=PUM15(:,KLEV), PVMLEV5=PVM15(:,KLEV), PTMLEV5=PTM15(:,KLEV), &
 & PQMLEV5=PQM15(:,KLEV), PAPHMS5=PAPHM15(:,KLEV), &
 & PGEOMLEV5=PGEOM15(:,KLEV), PCPTGZLEV5=ZCPTGZ5(:,KLEV), &
 & PSST=PSST, PTSKM1M5=PTSKM1M5, PCHAR=PCHAR, &
 & PSSRFL5=PSSRFL5, PTICE5=PTICE5, PTSNOW5=PTSNOW5, PWLMX5=PWLMX5, &
! input data, soil - trajectory
 & PTSAM1M5=PTSAM1M5, PWSAM1M5=PWSAM1M5, KSOTY=KSOTY, &
! input data, tiled - trajectory
 & PFRTI=PFRTI, PALBTI5=PALBTI5, &
! updated data, tiled - trajectory
 & PUSTRTI5=PUSTRTI5, PVSTRTI5=PVSTRTI5, PAHFSTI5=PAHFSTI5, &
 & PEVAPTI5=PEVAPTI5, PTSKTI5=PTSKTI5, &
! updated data, non-tiled - trajectory
 & PZ0M5=PZ0M5, PZ0H5=PZ0H5, &
! output data, tiled - trajectory
 & PSSRFLTI5=ZSSRFLTI5, PQSTI5=ZQSTI5, PDQSTI5=ZDQSTI5, PCPTSTI5=ZCPTSTI5, &
 & PCFHTI5=ZCFHTI5, PCFQTI5=ZCFQTI5, PCSATTI5=ZCSATTI5, PCAIRTI5=ZCAIRTI5, &
! output data, non-tiled - trajectory
 & PCFMLEV5=ZCFM5(:,KLEV), &
 & PKMFL5=ZKMFL5, PKHFL5=ZKHFL5, PKQFL5=ZKQFL5, &
 & PEVAPSNW5=ZPEVAPSNW5, &
 & PZ0MW5=ZZ0MW5, PZ0HW5=ZZ0HW5, PZ0QW5=ZZ0QW5, &
 & PCPTSPP5=ZZCPTS5, PQSAPP5=ZZQSA5, PBUOMPP5=ZZBUOM5, &
! input data, non-tiled
 & PUMLEV=PUM1(:,KLEV), PVMLEV=PVM1(:,KLEV), PTMLEV=PTM1(:,KLEV), &
 & PQMLEV=PQM1(:,KLEV), PAPHMS=PAPHM1(:,KLEV), &
 & PGEOMLEV=PGEOM1(:,KLEV), PCPTGZLEV=ZCPTGZ(:,KLEV), &
 & PTSKM1M=PTSKM1M, PSSRFL=PSSRFL, &
! input data, soil
 & PTSAM1M=PTSAM1M, PWSAM1M=PWSAM1M, &
! input data, tiled
 & PALBTI=PALBTI, &
! updated data, tiled
 & PUSTRTI=PUSTRTI, PVSTRTI=PVSTRTI, PAHFSTI=PAHFSTI, &
 & PEVAPTI=PEVAPTI, PTSKTI=PTSKTI, &
! updated data, non-tiled
 & PZ0M=PZ0M, PZ0H=PZ0H, &
! output data, tiled
 & PSSRFLTI=ZSSRFLTI, PQSTI=ZQSTI, PDQSTI=ZDQSTI, PCPTSTI=ZCPTSTI, &
 & PCFHTI=ZCFHTI, PCFQTI=ZCFQTI, PCSATTI=ZCSATTI, PCAIRTI=ZCAIRTI, &
! output data, non-tiled
 & PCFMLEV=ZCFM(:,KLEV), PEVAPSNW=ZPEVAPSNW, &
 & PZ0MW=ZZ0MW, PZ0HW=ZZ0HW, PZ0QW=ZZ0QW, &
 & PCPTSPP=ZZCPTS, PQSAPP=ZZQSA, PBUOMPP=ZZBUOM &
 & )

!*         2.     BOUNDARY LAYER HEIGHT FOR DIANOSTICS ONLY
!          ------------------------------------------------

CALL VDFDPBLS(KIDIA,KFDIA,KLON,KLEV,&
 & PTM15,PQM15,PGEOM15,ZKMFL5,ZKHFL5,ZKQFL5,ZPBLH5)

DO JL=KIDIA,KFDIA
  ZKHVFL5(JL)  = ZKHFL5(JL) + RETV * PTM15(JL,KLEV) * ZKQFL5(JL) !w'theta,v'
  IF ( ZKHVFL5(JL) >= 0.0_JPRB ) THEN
    ZZINV5(JL) = 0.0_JPRB   ! stable, therefore 0 PBL depth
  ELSE
    ZZINV5(JL) = ZPBLH5(JL)
  ENDIF
ENDDO


!*         3.     EXCHANGE COEFFICIENTS ABOVE THE SURFACE LAYER
!          ----------------------------------------------------

CALL VDFEXCUSTL (KIDIA, KFDIA, KLON, KLEV, ZTMST,&
 & PZ0M5  ,&
 & PUM15  ,PVM15  ,PTM15  ,PQM15 , &
 & PAPHM15,PGEOM15,PGEOH5 ,ZCPTGZ5, &
 & ZKMFL5 , ZKHFL5,ZKQFL5 ,ZZINV5 , &
 & ZCFM5  , ZCFH5 , &
 & PZ0M   , &
 & PUM1   ,PVM1   ,PTM1   ,PQM1  , &
 & PAPHM1 ,PGEOM1 ,PGEOH  ,ZCPTGZ ,ZCFM  ,ZCFH)  

CALL VDFTOFDCS (KIDIA,KFDIA,KLON,KLEV,ZTMST,&
 & PUM15,PVM15,PGEOM15,PSIGFLT,&
 & ZTOFDC5)  

!    No perturbations 
DO JK = 1,KLEV
  DO JL = KIDIA, KFDIA
    ZTOFDC(JL,JK)=0.0_JPRB
  ENDDO
ENDDO
!     ------------------------------------------------------------------

!*         4.     SOLVE DIFFUSION EQUATION.
!                 ----- --------- ---------

!          4.1    MOMENTUM

ITOP=1

CALL VDFDIFMSTL (KIDIA, KFDIA, KLON, KLEV, ITOP,&
 & ZTMST ,PUM15 ,PVM15  ,PAPHM15 , ZCFM5, ZTOFDC5, &
 & PVOM5 ,PVOL5 ,ZUDIF5 ,ZVDIF5  ,&
 & PUM1  ,PVM1  ,PAPHM1 ,ZCFM    , ZTOFDC, &
 & PVOM  ,PVOL  ,ZUDIF  ,ZVDIF)  

!          4.2    DRY STATIC ENERGY AND MOISTURE

CALL VDFDIFHSTL(KIDIA,KFDIA,KLON,KLEV,ITOP,KTILES,&
 & ZTMST  ,&
 & PFRTI  ,ZSSRFLTI5 ,PSLRFL5 ,PEMIS5  ,ZPEVAPSNW5,&
! & PHLICE5  ,PTLICE5 ,PTLWML5 ,& !lake passive
 & ZCPTGZ5,PTM15     ,PQM15   ,PAPHM15 ,&
 & ZCFH5  ,ZCFHTI5   ,ZCFQTI5 ,&
 & ZTDIF5 ,ZQDIF5    ,ZCPTSTI5,ZQSTI5  ,ZCAIRTI5  ,ZCSATTI5,&
 & ZDQSTI5,PTSKTI5   ,PTSKRAD5,&
 & PTSAM1M5(1,1), PTSNOW5, PTICE5, PSST ,&
 & ZTSKTIP15, ZQSTIP15, ZCPTSTIP15, PTE5, PQE5,&
 & PEVAPTI5 , PAHFSTI5, ZAHFLTI5  ,ZSTR5, ZG05,&
 & ZSSRFLTI , PSLRFL  , ZPEVAPSNW,&
 & ZCPTGZ ,PTM1  ,PQM1  , PAPHM1 ,&
 & ZCFH   ,ZCFHTI,ZCFQTI ,&
 & ZTDIF  ,ZQDIF ,ZCPTSTI, ZQSTI , ZCAIRTI, ZCSATTI,&
 & ZDQSTI ,PTSKTI,PTSKRAD,&
 & PTSAM1M(1,1),&
 & ZTSKTIP1,ZQSTIP1,ZCPTSTIP1, PTE , PQE,&
 & PEVAPTI ,PAHFSTI,ZAHFLTI  , ZSTR, ZG0)  

!          4.3     INCREMENTATION OF U AND V TENDENCIES, STORAGE OF
!                  THE DISSIPATION, COMPUTATION OF MULTILEVEL FLUXES.

CALL VDFINCRSTL (KIDIA,KFDIA,KLON,KLEV,ITOP,ZTMST,&
 & PUM15    ,PVM15 ,PTM15  ,PQM15  , PAPHM15, &
 & PGEOM15  ,ZCFM5 ,ZTOFDC5,ZCPTGZ5,&
 & ZUDIF5   ,ZVDIF5,ZTDIF5 ,ZQDIF5 , &
 & PVOM5    ,PVOL5 ,PTE5   ,PQE5   ,&
 & ZTEWODIS5,ZVDIS5,PSTRTU5,PSTRTV5, &
 & PUM1     ,PVM1  ,PTM1   ,PQM1   ,PAPHM1, &
 & PGEOM1   ,ZCFM  ,ZTOFDC ,ZCPTGZ ,&
 & ZUDIF    ,ZVDIF ,ZTDIF  ,ZQDIF  , &
 & PVOM     ,PVOL  ,PTE    ,PQE    , &
 & ZTEWODIS ,ZVDIS ,PSTRTU ,PSTRTV)  

!          4.4  Solve for tracers

IF (LVDFTRAC .AND. KTRAC > 0) THEN  
  CALL VDFDIFCSTL(KIDIA,KFDIA,KLON,KLEV,ITOP,KTRAC,ZTMST,&
                & PCM15,PTENC5,PAPHM15,ZCFH5,PCFLX5,&
                & PCM1 ,PTENC ,PAPHM1 ,ZCFH ,PCFLX )
ENDIF

!     ------------------------------------------------------------------

!*       5.    Wrap-up computations for the surface,
!               compute weather parameters, and skin T tendency

CALL SURFPPSTL( KIDIA=KIDIA,KFDIA=KFDIA,KLON=KLON,KTILES=KTILES, &
 & LDPPCFLS=LEPPCFLS,PTSTEP=PTSPHY, &
! input - trajectory
 & PFRTI=PFRTI, PTSKTIP15=ZTSKTIP15, PAHFLTI5=ZAHFLTI5, PG0TI5=ZG05, &
 & PSTRTULEV5=PSTRTU5(:,KLEV), PSTRTVLEV5=PSTRTV5(:,KLEV), &
 & PTSKM1M5=PTSKM1M5, PUMLEV5=PUM15(:,KLEV), PVMLEV5=PVM15(:,KLEV), &
 & PQMLEV5=PQM15(:,KLEV), PGEOMLEV5=PGEOM15(:,KLEV), &
 & PCPTSPP5=ZZCPTS5, PCPTGZLEV5=ZCPTGZ5(:,KLEV), &
 & PAPHMS5=PAPHM15(:,KLEV), PZ0MW5=ZZ0MW5, PZ0HW5=ZZ0HW5, PZ0QW5=ZZ0QW5, &
 & PQSAPP5=ZZQSA5, PBUOM5=ZZBUOM5, &
! updated - trajectory
 & PAHFSTI5=PAHFSTI5, PEVAPTI5=PEVAPTI5, PTSKE15=PTSKE15, &
! output - trajectory
 & PDIFTSLEV5=PDIFTS5(:,KLEV), PDIFTQLEV5=PDIFTQ5(:,KLEV), &
 & PUSTRTI5=PUSTRTI5, PVSTRTI5=PVSTRTI5,  PTSKTI5=PTSKTI5, &
 & PU10M5=PU10M5, PV10M5=PV10M5, PT2M5=PT2M5, PD2M5=PD2M5, PQ2M5=PQ2M5, &
 & P10NU5=P10NU5, P10NV5=P10NV5, &
! input
 & PTSKTIP1=ZTSKTIP1, PAHFLTI=ZAHFLTI, PG0TI=ZG0, &
 & PSTRTULEV=PSTRTU(:,KLEV), PSTRTVLEV=PSTRTV(:,KLEV), &
 & PTSKM1M=PTSKM1M , PUMLEV=PUM1(:,KLEV), PVMLEV=PVM1(:,KLEV), &
 & PQMLEV=PQM1(:,KLEV), PGEOMLEV=PGEOM1(:,KLEV), &
 & PCPTSPP=ZZCPTS, PCPTGZLEV=ZCPTGZ(:,KLEV), &
 & PAPHMS=PAPHM1(:,KLEV), PZ0MW=ZZ0MW, PZ0HW=ZZ0HW, PZ0QW=ZZ0QW, &
 & PQSAPP=ZZQSA, PBUOM=ZZBUOM, &
! updated
 & PAHFSTI=PAHFSTI, PEVAPTI=PEVAPTI, PTSKE1=PTSKE1, &
! output
 & PDIFTSLEV=PDIFTS(:,KLEV), PDIFTQLEV=PDIFTQ(:,KLEV), PUSTRTI=PUSTRTI, &
 & PVSTRTI=PVSTRTI,  PTSKTI=PTSKTI, &
 & PU10M=PU10M, PV10M=PV10M, PT2M=PT2M, PD2M=PD2M, PQ2M=PQ2M, &
 & P10NU=P10NU, P10NV=P10NV &
 & )

!     ------------------------------------------------------------------

!*         6.     FLUX COMPUTATIONS
!                 -----------------

DO JL = KIDIA, KFDIA
  PDIFTQ (JL,0) = 0.0_JPRB
  PDIFTQ5(JL,0) = 0.0_JPRB
  PDIFTS (JL,0) = 0.0_JPRB
  PDIFTS5(JL,0) = 0.0_JPRB
  PSTRTU (JL,0) = 0.0_JPRB
  PSTRTU5(JL,0) = 0.0_JPRB
  PSTRTV (JL,0) = 0.0_JPRB
  PSTRTV5(JL,0) = 0.0_JPRB
ENDDO

DO JK = KLEV, 2, -1
  DO JL = KIDIA, KFDIA
    ZGDPH = (PAPHM1 (JL,JK-1)-PAPHM1 (JL,JK))*ZRG
    ZGDPH5=(PAPHM15(JL,JK-1)-PAPHM15(JL,JK))*ZRG
!...change of dry static energy is converted to flux of d.s.e.
    ZCP =RCPD*RVTMP2*PQM1(JL,JK)
    ZCP5=RCPD*(1.0_JPRB+RVTMP2*PQM15(JL,JK))
    PDIFTS (JL,JK-1)=&
     & ZCP *(ZTEWODIS5(JL,JK)-ZTEA5(JL,JK))*ZGDPH5+ &
     & ZCP5*(ZTEWODIS (JL,JK)-ZTEA (JL,JK))*ZGDPH5+ &
     & ZCP5*(ZTEWODIS5(JL,JK)-ZTEA5(JL,JK))*ZGDPH + &
     & PDIFTS (JL,JK)  
    PDIFTS5(JL,JK-1)=ZCP5*(ZTEWODIS5(JL,JK)-ZTEA5(JL,JK))*&
     & ZGDPH5+PDIFTS5(JL,JK)  
!...changes in Q,U,V tendencies are converted to fluxes
    PDIFTQ (JL,JK-1)=&
     & (PQE (JL,JK)-ZQEA (JL,JK))*ZGDPH5+ &
     & (PQE5(JL,JK)-ZQEA5(JL,JK))*ZGDPH + &
     & PDIFTQ (JL,JK)  
    PDIFTQ5(JL,JK-1)=(PQE5 (JL,JK)-ZQEA5(JL,JK))*ZGDPH5+PDIFTQ5(JL,JK)
    PSTRTU (JL,JK-1)=&
     & (PVOM (JL,JK)-ZUEA (JL,JK))*ZGDPH5+ &
     & (PVOM5(JL,JK)-ZUEA5(JL,JK))*ZGDPH + &
     & PSTRTU (JL,JK)  
    PSTRTU5(JL,JK-1)=(PVOM5(JL,JK)-ZUEA5(JL,JK))*ZGDPH5+PSTRTU5(JL,JK)
    PSTRTV (JL,JK-1)=&
     & (PVOL (JL,JK)-ZVEA (JL,JK))*ZGDPH5+ &
     & (PVOL5(JL,JK)-ZVEA5(JL,JK))*ZGDPH + &
     & PSTRTV (JL,JK)  
    PSTRTV5(JL,JK-1)=(PVOL5(JL,JK)-ZVEA5(JL,JK))*ZGDPH5+PSTRTV5(JL,JK)
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('VDFMAINSTL',1,ZHOOK_HANDLE)
END SUBROUTINE VDFMAINSTL
