SUBROUTINE LWBVAD &
 & ( KIDIA, KFDIA, KLON , KLEV  , KMODE, KTRAER,&
 & PDT05 , PEMIS5, PEMIW5,&
 & PTL5  , PTAVE5,&
 & PEMIT5, PFLUC5,&
 & PABCU5, PBINT5, PBSUI5, PCNTRB5, &
! saved in NL coputation - lwbv
 & PB5, PBSUR5, PBTOP5, PDBSL5, &
 & PGA, PGB, PGASUR, PGBSUR, PGATOP, PGBTOP, &
 & PADJD5, PADJU5, PDBDT5, PDISD5, PDISU5, PDWFSU5, &
! saved in NL coputation - lwvdr
 & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
 & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
 & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
 & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15, &
! perturbation
 & PDT0  , PEMIS , PEMIW ,&
 & PTL   , PTAVE ,&
 & PEMIT , PFLUC ,&
 & PABCU , PBINT , PBSUI , PCNTRB &
 & )

!**** *LWBVAD*   - COMPUTE PLANCK FUNC., PERF. VERT. INTEGRATION
!                      (adjoint)

!     PURPOSE.
!     --------
!           TO COMPUTE THE PLANCK FUNCTION AND PERFORM THE
!           VERTICAL INTEGRATION. SPLIT OUT FROM LW FOR MEMORY
!           SAVING

!**   INTERFACE.
!     ----------

!        *LWVBAD* IS CALLED FROM *LWAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!  Trajectory  Perturbation  Description                         Dimension
!  PDT05       PDT0          SURFACE TEMPERATURE DISCONTINUITY   (KLON)
!  PEMIS5      PEMIS         SURFACE LW EMISSIVITY               (KLON)
!  PEMIW5      PEMIW         SURFACE LW WINDOW EMISSIVITY        (KLON)
!  PTAVE5      PTAVE         TEMPERATURE                         (KLON,KLEV)
!  PTL5        PTL           HALF LEVEL TEMPERATURE              (KLON,KLEV+1)

!     ==== SAVED FROM NL COMPUTATION (if LWLOPT.eq.T) ===
!  PGA, PGB    ---           dB/dT-weighted LAYER PADE APPROXIMANTS
!                                                              (KLON,8,2,KLEV)
!  PGA(B)SUR   ---           SURFACE PADE APPROXIMANTS           (KLON,8,2)
!  PGA(B)TOP   ---           T.O.A. PADE APPROXIMANTS            (KLON,8,2)
!  PB5         ---           SPECTRAL HALF LEVEL PLANCK FUNC.  (KLON,NSIL,KLEV+1)
!  PBSUR5      ---           SURFACE SPECTRAL PLANCK FUNCTION    (KLON,NSIL)
!  PBTOP5      ---           TOP SPECTRAL PLANCK FUNCTION        (KLON,NSIL)
!  PDBSL5      ---           SUB-LAYER PLANCK FUNCTION GRADIENT (KLON,NSIL,KLEV*2)
!  PADJD(U)5   ---           CONTRIBUTION OF ADJACENT LAYERS     (KLON,KLEV+1)
!  PDBDT5      ---           LAYER PLANCK FUNCTION GRADIENT      (KLON,NUA,KLEV)
!  PDISD(U)5   ---           CONTRIBUTION BY DISTANT LAYERS      (KLON,KLEV+1)
!  PDWFSU5     ---           SPECTRAL DOWNWARD FLUX AT SURFACE   (KLON,NSIL)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PABCU5      PABCU         ABSORBER AMOUNTS               (KLON,NUA,3*KLEV+1)
!  PBINT5      PBINT         HALF-LEVEL PLANCK FUNCTIONS          (KLON,KLEV+1)
!  PBSUI5      PBSUI                                              (KLON)
!  PCNTRB5     PCNTRB        CLEAR-SKY ENERGY EXCHANGE 
!                            MATRIX                       (KLON,KLEV+1,KLEV+1) 
!  PEMIT5      PEMIT         SURFACE TOTAL LW EMISSIVITY          (KLON)
!  PFLUC5      PFLUC         RADIATIVE FLUXES CLEAR-SKY:          (KLON,2,KLEV)
!                    1  ==>  UPWARD   FLUX TOTAL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE PLANCK FUNCTIONS ON THE INTERFACES AND THE
!     GRADIENT OF PLANCK FUNCTIONS IN THE LAYERS.
!          2. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING THE CON-
!     TRIBUTIONS OF THE ADJACENT AND DISTANT LAYERS AND THOSE FROM THE
!     BOUNDARIES.
!          3. COMPUTES THE CLEAR-SKY COOLING RATES.

!     EXTERNALS.
!     ----------

!          *LWBAD*, *LWVAD*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*        7-Dec-2006

!     MODIFICATIONS.
!     --------------
!        M. Janiskova   4-Apr-2007 additional arguments passed from
!                                  the routine for AD computation
!        M. Janiskova  19-May-2008 using shorter NPROMALW instead of KLON
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NSIL     ,NIPD     ,NUA
USE YOERDU   , ONLY : NUAER    ,NTRAER
USE YOPHNC   , ONLY : LH2OCO2  ,LWLOPT
USE YOELWRAD , ONLY : NPROMALW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT05(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMIT5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUC5(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU5(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBINT5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBSUI5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB5(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDT0(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUC(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PABCU(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBSUI(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCNTRB(KLON,KLEV+1,KLEV+1)
! from NL computation - lwbv if LWLOPT.eq.T
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB5(KLON,NSIL,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBSUR5(KLON,NSIL)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBTOP5(KLON,NSIL)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDBSL5(KLON,NSIL,KLEV*2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGA(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGB(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGASUR(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGBSUR(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGATOP(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGBTOP(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PADJD5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PADJU5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDBDT5(KLON,NSIL,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU5(KLON,NSIL)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDISD5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDISU5(KLON,KLEV+1)
! from NL computation - lwvdr
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PCOND5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPA5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPB5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPA15(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPB15(KLON,KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-------------------------------------------------------------------------

!              ------------
REAL(KIND=JPRB) :: ZB(KLON,NSIL,KLEV+1), ZBSUR(KLON,NSIL), ZBTOP(KLON,NSIL)
REAL(KIND=JPRB) :: ZDBSL(KLON,NSIL,KLEV*2)
REAL(KIND=JPRB) :: ZDIV5(KLON),ZPEMIT5(KLON)

INTEGER(KIND=JPIM) :: JL, JLW
INTEGER(KIND=JPIM) :: IUAER, ITRAER
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwb.intfb.h"
#include "lwv.intfb.h"
#include "lwbad.intfb.h"
#include "lwvad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWBVAD',0,ZHOOK_HANDLE)

!*         1.    INITIALIZATION
!                --------------

IF (LH2OCO2) THEN
  IUAER  = 9
  ITRAER = 9
ELSE
  IUAER  = NUAER
  ITRAER = NTRAER
ENDIF

!*         2.    COMPUTES PLANCK FUNCTIONS
!                -------------------------

IF (.NOT. LWLOPT) THEN
  CALL LWB &
   & ( KIDIA, KFDIA, KLON  , KLEV  , KMODE,&
   & PDT05 , PTAVE5, PTL5   ,&
   & PB5   , PBINT5, PBSUR5 , PBTOP5 , PDBSL5,&
   & PGA   , PGB   , PGASUR, PGBSUR, PGATOP, PGBTOP )

!     ------------------------------------------------------------------

!*         3.    PERFORMS THE VERTICAL INTEGRATION
!                ---------------------------------

  CALL LWV &
   & ( KIDIA , KFDIA, KLON  , KLEV,&
! & NUAER , NTRAER,&
   & IUAER , ITRAER,&
   & PABCU5 , PB5   , PBINT5 , PBSUR5 , PBTOP5 , PDBSL5,&
   & PEMIS5 , PEMIW5,&
   & PGA    , PGB   , PGASUR , PGBSUR , PGATOP , PGBTOP,&
   & PCNTRB5, PFLUC5,&
! for adjoint computation
   & PADJD5, PADJU5, PDBDT5, PDISD5, PDISU5, PDWFSU5,&
   & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
   & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
   & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
   & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15 )
ENDIF

DO JL=KIDIA,KFDIA
  PEMIT5(JL) = 0.0_JPRB
  PBSUI5(JL) = 0.0_JPRB
ENDDO
DO JLW=1,NSIL
  DO JL=KIDIA,KFDIA
    PBSUI5(JL) = PBSUI5(JL)+PBSUR5(JL,JLW)
    IF (JLW >= 3.AND. JLW <= 4) THEN
      PEMIT5(JL) = PEMIT5(JL)+PBSUR5(JL,JLW)*PEMIW5(JL)
    ELSE
      PEMIT5(JL) = PEMIT5(JL)+PBSUR5(JL,JLW)*PEMIS5(JL)
    ENDIF
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  ZDIV5(JL) = 1.0_JPRB/PBSUI5(JL)
  ZPEMIT5(JL) = PEMIT5(JL)
  PEMIT5(JL) = PEMIT5(JL)*ZDIV5(JL)
ENDDO

!     ------------------------------------------------------------------

!*         0.    ADJOINT CALCULATIONS
!                --------------------

!* Set local variables to zero

ZDBSL(:,:,:) = 0.0_JPRB
ZB (:,:,:) = 0.0_JPRB
ZBSUR(:,:) = 0.0_JPRB
ZBTOP(:,:) = 0.0_JPRB

DO JL=KIDIA,KFDIA
  PBSUI(JL) = PBSUI(JL)-ZPEMIT5(JL)*ZDIV5(JL)*ZDIV5(JL)*PEMIT(JL)
  PEMIT(JL) = PEMIT(JL)*ZDIV5(JL)
ENDDO
DO JLW=NSIL,1,-1
  DO JL=KIDIA,KFDIA
    IF (JLW >= 3.AND. JLW <= 4) THEN
      PEMIW(JL) = PEMIW(JL)+PBSUR5(JL,JLW)*PEMIT(JL)
      ZBSUR(JL,JLW) = ZBSUR(JL,JLW)+PEMIW5(JL)*PEMIT(JL)
    ELSE
      PEMIS(JL) = PEMIS(JL)+PBSUR5(JL,JLW)*PEMIT(JL)
      ZBSUR(JL,JLW) = ZBSUR(JL,JLW)+PEMIS5(JL)*PEMIT(JL)
    ENDIF
    ZBSUR(JL,JLW) = ZBSUR(JL,JLW)+PBSUI(JL)
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  PEMIT (JL) = 0.0_JPRB
  PBSUI (JL) = 0.0_JPRB
ENDDO

!*         1.3.  PERFORMS THE VERTICAL INTEGRATION
!                ---------------------------------

CALL LWVAD &
 & ( KIDIA , KFDIA, KLON  , KLEV,&
! & NUAER , NTRAER,&
 & IUAER , ITRAER,&
 & PGA    , PGB   , PGASUR , PGBSUR , PGATOP , PGBTOP,&
 & PABCU5 , PB5   , PBINT5 , PBSUR5 , PBTOP5 , PDBSL5,&
 & PEMIS5 , PEMIW5,&
 & PCNTRB5, PFLUC5,&
! saved in NL computation - lwv
 & PADJD5, PADJU5, PDBDT5, PDISD5, PDISU5, PDWFSU5, &
! saved in NL coputation - lwvdr
 & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
 & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
 & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
 & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15, &
! perturbation
 & PABCU  , ZB    , PBINT  , ZBSUR  , ZBTOP  , ZDBSL ,&
 & PEMIS  , PEMIW ,&
 & PCNTRB , PFLUC &
 & )

!*         1.2.  COMPUTES PLANCK FUNCTIONS
!                -------------------------

CALL LWBAD &
 & ( KIDIA, KFDIA, KLON  , KLEV  , KMODE,&
 & PGA   , PGB   , PGASUR, PGBSUR, PGATOP, PGBTOP,& 
 & PDT05 , PTAVE5, PTL5   ,&
 & PB5   , PBINT5, PBSUR5 , PBTOP5 , PDBSL5,&
 & PDT0  , PTAVE , PTL    ,&
 & ZB    , PBINT , ZBSUR  , ZBTOP  , ZDBSL &
 & )

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWBVAD',1,ZHOOK_HANDLE)
END SUBROUTINE LWBVAD
