SUBROUTINE LWINTERF &
 & (KLON,KLEV,KIDIA,KFDIA, &
 & PENTREE,PTL, &
 & PCLDLU,PCLDLUP, &
 & PFUPNE1,PFDNNE1)  

!**** *LWINTERF*   - COMPUTATION OF LW FLUXES

!     PURPOSE.
!     --------
!-- Calcul .des flux ciels clair
!--        .des flux ciels nuageux
!--        .des taux de refroidissement
!-- par une methode basee sur les reseaux de neurones
!-- ARA/LMD 
!--          07/97 ---> LMDz
!--          02/98 ---> CEPMMT

!**   INTERFACE.
!     ----------

!        *LWINTERF* IS CALLED FROM LWNEUR

!        EXPLICIT ARGUMENTS :
!        --------------------

!     INPUT PARAMETERS (REAL):

! PENTREE  : (KLON,6+3*KLEV)
! PCLDLU   : (KLON,KLEV)       ; UPWARD EFFECTIVE CLOUD FRACTION
! PTL      : (KLON,KLEV+1)     ; HALF LEVEL TEMPERATURE

!     OUTPUT PARAMETERS (REAL):

! PFUPNE1(KLON,KLEV+1,KLEV+1)
! PFDNNE1(KLON,KLEV+1,KLEV+1)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!        *LWPRCLAIRE*, *LWPRNUAGE*

!     REFERENCE.
!     ----------

!     AUTHORS.
!     --------
!        F. CHEVALLIER          *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 99-05
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOENEUR  , ONLY : NBNEUR,WNELN,RAPIN,ENTREEMOY,&
 & RAPOUT,SORTIEMOY,NCOU,NBR,NBNEURMAX  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENTREE(KLON,6+3*KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLUP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFUPNE1(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFDNNE1(KLON,KLEV+1,KLEV+1) 
!-------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

REAL(KIND=JPRB) :: Z_SIGMA, ZTTM
REAL(KIND=JPRB) :: ZYNXN1T(KLON,0:NBNEURMAX,0:NCOU)
REAL(KIND=JPRB) :: ZYNXN1N(KLON,0:NBNEURMAX,0:NCOU)
REAL(KIND=JPRB) :: ZFUPC(KLON,KLEV+1),ZFDNC(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZSORTIENO1(KLON,NBNEURMAX)
REAL(KIND=JPRB) :: ZSORTIENO2(KLON,NBNEURMAX)
INTEGER(KIND=JPIM) :: IATM,JLEV,IENT,JLEV1,JLEV2,IR
INTEGER(KIND=JPIM) :: JNU
INTEGER(KIND=JPIM) :: ICORR(KLON,KLEV),I_NBNU(KLEV)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwprclair.intfb.h"
#include "lwprnuage.intfb.h"

DATA Z_SIGMA / 5.6698e-8_JPRB/

!     ------------------------------------------------------------------

!*         1.     INITIALIZATION.
!                 ---------------

IF (LHOOK) CALL DR_HOOK('LWINTERF',0,ZHOOK_HANDLE)
IR = 1
I_NBNU = 0
ZYNXN1T = 1.0_JPRB
ZYNXN1N = 1.0_JPRB

!     -----------------------------------------------------------------

!*         2.  1     CALCUL CIEL CLAIR - debut
!                    -------------------------

DO IENT=1,NBNEUR(0,IR)
  DO IATM = KIDIA,KFDIA
    ZYNXN1T(IATM,IENT,0)=RAPIN(IENT,IR) &
     & *(PENTREE(IATM,IENT)-ENTREEMOY(IENT,IR))  
    ZTTM = 0.9_JPRB
    IF (ABS(ZYNXN1T(IATM,IENT,0)) > ZTTM) THEN
      ZYNXN1T(IATM,IENT,0)=MAX(-1.0_JPRB*ZTTM,ZYNXN1T(IATM,IENT,0))
      ZYNXN1T(IATM,IENT,0)=MIN(ZTTM,ZYNXN1T(IATM,IENT,0))
    ENDIF
  ENDDO
ENDDO

CALL LWPRCLAIR &
 & (1,2,KIDIA,KFDIA,KLON,NCOU,NBNEURMAX,NBR, &
 & NBNEUR,WNELN,ZYNXN1T, &
 & ZSORTIENO1,ZSORTIENO2)  

DO JLEV = 1,KLEV+1
  DO IATM = KIDIA,KFDIA
    ZFUPC(IATM,JLEV) = ZSORTIENO1(IATM,JLEV)*RAPOUT(JLEV,1) &
     & + SORTIEMOY(JLEV,1)  
    ZFDNC(IATM,JLEV) = ZSORTIENO2(IATM,JLEV)*RAPOUT(JLEV,2) &
     & + SORTIEMOY(JLEV,2)  
  ENDDO
ENDDO
DO IATM = KIDIA,KFDIA
  ZFUPC(IATM,KLEV+1) = Z_SIGMA * PENTREE(IATM,1)
ENDDO

!-------------------------------------------------------------------
!*       2.  2    CALCUL CIEL CLAIR - fin
!                 -----------------------

!-------------------------------------------------------------------
!*       3.  1    CAS CIEL NUAGEUX - debut
!                 ------------------------

!-- Localisation des couches nuageuses

DO IATM = KIDIA,KFDIA

  DO JLEV = 1,KLEV
    IF (PCLDLU(IATM,KLEV+1-JLEV) > 0.0_JPRB &
       & .OR.ABS(PCLDLUP(IATM,KLEV+1-JLEV)) > 0.0_JPRB) THEN  
      I_NBNU(JLEV) = I_NBNU(JLEV) + 1
      ICORR(I_NBNU(JLEV),JLEV) = IATM
    ENDIF
  ENDDO

ENDDO

!-- Initialisation des flux unicouches

DO JLEV1=1,KLEV+1
  DO JLEV2 = 1,KLEV+1
    DO IATM = KIDIA,KFDIA
      PFUPNE1(IATM,JLEV2,JLEV1) = ZFUPC(IATM,KLEV + 2 - JLEV1)
      PFDNNE1(IATM,JLEV2,JLEV1) = ZFDNC(IATM,KLEV + 2 - JLEV1)
    ENDDO
  ENDDO
ENDDO

!-- Calcul, une couche de nuages, supposes opaques, a la fois

DO JNU = 1,KLEV
  IF (I_NBNU(JNU) > 0) THEN

    DO IATM = 1,I_NBNU(JNU)
      ZYNXN1N(IATM,1,0) = ZYNXN1T(ICORR(IATM,JNU),3,0)
      ZYNXN1N(IATM,2,0) = ZYNXN1T(ICORR(IATM,JNU),4,0)
      ZYNXN1N(IATM,3,0) = ZYNXN1T(ICORR(IATM,JNU),5,0)
      ZYNXN1N(IATM,4,0) = ZYNXN1T(ICORR(IATM,JNU),6,0)
    ENDDO
    
!-- Flux montants

    IF (JNU == 1) THEN
      DO IATM = 1,I_NBNU(JNU)
        PFUPNE1(ICORR(IATM,JNU),KLEV + 2 - JNU,KLEV+1) =&
         & Z_SIGMA*(PTL(ICORR(IATM,JNU),KLEV+1)**4)  
      ENDDO
    ELSE
      DO IENT=1,3*JNU
        DO IATM = 1,I_NBNU(JNU)
          ZYNXN1N(IATM,4+IENT,0) = ZYNXN1T(ICORR(IATM,JNU),6+IENT,0)
        ENDDO
      ENDDO
      IR = 2 + 2*(JNU-1)+1
      
      CALL LWPRNUAGE &
       & (KLON,IR,I_NBNU(JNU),NBNEUR,NCOU,NBNEURMAX,NBR, &
       & WNELN,ZYNXN1N,ZSORTIENO1 )  

      DO JLEV = 1,JNU
        DO IATM = 1,I_NBNU(JNU)
          PFUPNE1(ICORR(IATM,JNU),KLEV + 2 - JNU,KLEV + 2 - JLEV) =&
           & ZSORTIENO1(IATM,JLEV)*RAPOUT(JLEV,IR)&
           & +SORTIEMOY(JLEV,IR)  
        ENDDO
      ENDDO
    ENDIF
    
!-- Flux descendants

    IF (JNU == KLEV) THEN
      DO IATM = 1,I_NBNU(JNU)
        PFDNNE1(ICORR(IATM,JNU),KLEV + 2 - JNU,1) =&
         & - Z_SIGMA*(PTL(ICORR(IATM,JNU),1)**4)  
      ENDDO
    ELSE
      DO IENT=(6+3*(JNU-1) + 1),NBNEUR(0,1)
        DO IATM = 1,I_NBNU(JNU)
          ZYNXN1N(IATM,4 + IENT - 6 - 3*(JNU-1),0) =&
           & ZYNXN1T(ICORR(IATM,JNU),IENT,0)  
        ENDDO
      ENDDO
      IR = 2 + 2*JNU
      
      CALL LWPRNUAGE &
       & (KLON,IR,I_NBNU(JNU),NBNEUR,NCOU,NBNEURMAX,NBR,  &
       & WNELN,ZYNXN1N,ZSORTIENO1 )  

      DO JLEV = (JNU+1),KLEV+1
        DO IATM = 1,I_NBNU(JNU)
          PFDNNE1(ICORR(IATM,JNU),KLEV + 2 - JNU,KLEV + 2 - JLEV) =&
           & ZSORTIENO1(IATM,JLEV-JNU)*RAPOUT(JLEV-JNU,IR)&
           & +SORTIEMOY(JLEV-JNU,IR)  
        ENDDO
      ENDDO
    ENDIF
  ENDIF

ENDDO

!     --------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWINTERF',1,ZHOOK_HANDLE)
END SUBROUTINE LWINTERF
