SUBROUTINE LWVAD &
 & ( KIDIA, KFDIA, KLON , KLEV , KUAER , KTRAER,&
 & PGA    , PGB   , PGASUR , PGBSUR, PGATOP , PGBTOP,& 
 & PABCU5 , PB5   , PBINT5 , PBSUR5, PBTOP5 , PDBSL5,&
 & PEMIS5 , PEMIW5,&
 & PCNTRB5, PFLUC5,&
! saved in NL computation - lwv
 & PADJD5, PADJU5, PDBDT5, PDISD5, PDISU5, PDWFSU5, &
! saved in NL coputation - lwvdr
 & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
 & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
 & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
 & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15, &
! perturbation
 & PABCU  , PB    , PBINT  , PBSUR , PBTOP  , PDBSL,&
 & PEMIS  , PEMIW ,&
 & PCNTRB , PFLUC &
 & )  

!**** *LWVAD*   - LONGWAVE RADIATION, VERTICAL INTEGRATION
!                      (adjoint)

!     PURPOSE.
!     --------
!           CARRIES OUT THE VERTICAL INTEGRATION TO GIVE LONGWAVE
!           FLUXES OR RADIANCES

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PABCU5      PABCU         ABSORBER AMOUNTS               (KLON,NUA,3*KLEV+1)
!  PB5         PB            SPECTRAL HALF-LEVEL PLANCK
!                            FUNCTIONS                       (KLON,NSIL,KLEV+1)
!  PBINT5      PBINT         HALF-LEVEL PLANCK FUNCTIONS          (KLON,KLEV+1)
!  PBSUR5      PBSUR         SPECTRAL SURFACE PLANCK FUNCTION     (KLON,NSIL)
!  PBTOP5      PBTOP         SPECTRAL T.O.A. PLANCK FUNCTION      (KLON,NSIL)
!  PDBSL5      PDBSL         SUB-LAYER PLANCK FUNCTION GRADIENT   (KLON,KLEV*2)
!  PEMIS5      PEMIS         SURFACE LW EMISSIVITY                (KLON)
!  PEMIW5      PEMIW         SURFACE LW WINDOW EMISSIVITY         (KLON)
!  PGA         ---           PADE APPROXIMANTS               (KLON,NIPD,2,KLEV)
!  PGB         ---           PADE APPROXIMANTS               (KLON,NIPD,2,KLEV)
!  PGASUR      ---           SURFACE PADE APPROXIMANTS            (KLON,NIPD,2)
!  PGBSUR      ---           SURFACE PADE APPROXIMANTS            (KLON,NIPD,2)
!  PGATOP      ---           T.O.A. PADE APPROXIMANTS             (KLON,NIPD,2)
!  PGBTOP      ---           T.O.A. PADE APPROXIMANTS             (KLON,NIPD,2)

!     ==== SAVED FROM NL COMPUTATION (if LWLOPT.eq.T) ===
!  PADJD(U)5   ---           CONTRIBUTION OF ADJACENT LAYERS     (KLON,KLEV+1)
!  PDBDT5      ---           LAYER PLANCK FUNCTION GRADIENT      (KLON,NUA,KLEV)
!  PDISD(U)5   ---           CONTRIBUTION BY DISTANT LAYERS      (KLON,KLEV+1)
!  PDWFSU5     ---           SPECTRAL DOWNWARD FLUX AT SURFACE   (KLON,NSIL)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                          Dimension
!  PCNTRB5     PCNTRB        CLEAR-SKY ENERGY EXCHANGE 
!                            MATRIX                        (KLON,KLEV+1,KLEV+1)
!  PFLUC5      PFLUC         RADIATIVE FLUXES CLEAR-SKY           (KLON,2,KLEV)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING BETWEEN
!     CONTRIBUTIONS BY -  THE NEARBY LAYERS
!                      -  THE DISTANT LAYERS
!                      -  THE BOUNDARY TERMS
!          2. COMPUTES THE CLEAR-SKY DOWNWARD AND UPWARD EMISSIVITIES.

!     EXTERNALS.
!     ----------

!          *LWVNAD*, *LWVDAD*, *LWVBAD*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*        7-Dec-2006

!     MODIFICATIONS.
!     --------------
!        M. Janiskova   4-Apr-2007 skipping some computation saved in NL,
!                                  additional arguments passed from
!                                  the routine for AD computation
!        M. Janiskova  19-May-2008 using shorter NPROMALW instead of KLON
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NSIL     ,NIPD     ,NUA
USE YOPHNC   , ONLY : LH2OCO2  ,LWLOPT
USE YOELWRAD , ONLY : NPROMALW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUAER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGASUR(KLON,NIPD,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGBSUR(KLON,NIPD,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGATOP(KLON,NIPD,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGBTOP(KLON,NIPD,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU5(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB5(KLON,NSIL,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBINT5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUR5(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBTOP5(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBSL5(KLON,NSIL,KLEV*2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB5(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUC5(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PABCU(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB(KLON,NSIL,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBSUR(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBTOP(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDBSL(KLON,NSIL,KLEV*2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUC(KLON,2,KLEV+1) 
! from NL computation - lwv
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PADJD5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PADJU5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PDBDT5(KLON,NSIL,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PDWFSU5(KLON,NSIL)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PDISD5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PDISU5(KLON,KLEV+1)
! from NL computation - lwvdr
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PCOND5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPA5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPB5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPA15(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPB15(KLON,KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZADJD(KLON,KLEV+1), ZADJU(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZDBDT(KLON,NSIL,KLEV), ZDISD(KLON,KLEV+1) 
REAL(KIND=JPRB) :: ZDISU(KLON,KLEV+1), ZDWFSU(KLON,NSIL)  

REAL(KIND=JPRB) :: ZDWFSU5(KLON,NSIL)
REAL(KIND=JPRB) :: ZDWFSU15(KLON,NSIL), ZDWFSU25(KLON,NSIL)
REAL(KIND=JPRB) :: ZDWFSU35(KLON,NSIL)

INTEGER(KIND=JPIM) :: JA, JK, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE


#include "lwvb.intfb.h"
#include "lwvbr.intfb.h"
#include "lwvd.intfb.h"
#include "lwvdr.intfb.h"
#include "lwvn.intfb.h"
#include "lwvnr.intfb.h"
#include "lwvbad.intfb.h"
#include "lwvbrad.intfb.h"
#include "lwvdad.intfb.h"
#include "lwvdrad.intfb.h"
#include "lwvnad.intfb.h"
#include "lwvnrad.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.1     INITIALIZE LAYER CONTRIBUTIONS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWVAD',0,ZHOOK_HANDLE)

IF (.NOT.LWLOPT) THEN
  DO JK=1,KLEV+1
    DO JL=KIDIA,KFDIA
      PADJD5(JL,JK) = 0.0_JPRB
      PADJU5(JL,JK) = 0.0_JPRB
      PDISD5(JL,JK) = 0.0_JPRB
      PDISU5(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO
ENDIF

DO JA=1,NSIL
  DO JL=KIDIA,KFDIA
    ZDWFSU5 (JL,JA) = 0.0_JPRB
    ZDWFSU15(JL,JA) = 0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

IF (.NOT.LWLOPT) THEN
!     ------------------------------------------------------------------

!*         2.1     CONTRIBUTION FROM ADJACENT LAYERS
!                  ---------------------------------

  IF (LH2OCO2) THEN
    CALL LWVNR &
     & ( KIDIA, KFDIA, KLON  , KLEV , KUAER,&
     & PABCU5, PDBSL5, PGA    , PGB   ,&
     & PADJD5, PADJU5, PCNTRB5, PDBDT5, ZDWFSU5 )
  ELSE
    CALL LWVN &
     & ( KIDIA, KFDIA, KLON  , KLEV , KUAER,&
     & PABCU5, PDBSL5, PGA    , PGB  ,&
     & PADJD5, PADJU5, PCNTRB5, PDBDT5, ZDWFSU5 )
  ENDIF

!     ------------------------------------------------------------------

!*         2.2     CONTRIBUTION FROM DISTANT LAYERS
!                  ---------------------------------

  DO JA=1,NSIL
    DO JL=KIDIA,KFDIA
      ZDWFSU25(JL,JA) = ZDWFSU5(JL,JA)
    ENDDO
  ENDDO
  IF (LH2OCO2) THEN
    CALL LWVDR &
     & ( KIDIA , KFDIA, KLON , KLEV  , KTRAER,&
     & PABCU5 , PDBDT5, PGA   , PGB   ,&
     & PCNTRB5, PDISD5, PDISU5, ZDWFSU5,&
! for adjoint computation
     & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
     & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
     & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
     & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15 )
  ELSE
    CALL LWVD &
     & ( KIDIA , KFDIA, KLON , KLEV  , KTRAER,&
     & PABCU5 , PDBDT5, PGA   , PGB   ,&
     & PCNTRB5, PDISD5, PDISU5, ZDWFSU5 ,&
! for adjoint computation
     & PTTA5  , PTTB5 )
  ENDIF

!     ------------------------------------------------------------------

!*         2.3     EXCHANGE WITH THE BOUNDARIES
!                  ----------------------------

  DO JA=1,NSIL
    DO JL=KIDIA,KFDIA
      ZDWFSU35(JL,JA) = ZDWFSU5(JL,JA)
    ENDDO
  ENDDO
  IF (LH2OCO2) THEN
    CALL LWVBR &
     & ( KIDIA , KFDIA , KLON  , KLEV  , KUAER,&
     & PABCU5 , PADJD5 , PADJU5 ,&
     & PB5    , PBINT5 , PBSUR5 , PBTOP5,&
     & PDISD5 , PDISU5 , PEMIS5 , PEMIW5,&
     & PGASUR , PGBSUR , PGATOP , PGBTOP,&
     & ZDWFSU5, PFLUC5 )
  ELSE
    CALL LWVB &
     & ( KIDIA , KFDIA , KLON  , KLEV  , KUAER,&
     & PABCU5 , PADJD5 , PADJU5 ,&
     & PB5    , PBINT5 , PBSUR5 , PBTOP5,&
     & PDISD5 , PDISU5 , PEMIS5 , PEMIW5,&
     & PGASUR , PGBSUR , PGATOP , PGBTOP,&
     & ZDWFSU5, PFLUC5 )
  ENDIF

ELSE
  DO JA=1,NSIL
    DO JL=KIDIA,KFDIA
      ZDWFSU25(JL,JA) = ZDWFSU5(JL,JA)
      ZDWFSU35(JL,JA) = PDWFSU5(JL,JA)
    ENDDO
  ENDDO
ENDIF


!     ==================================================================

!*         0.    ADJOINT CALCULATIONS
!                --------------------

!* Set local variables to zero

ZADJD(:,:) = 0.0_JPRB
ZADJU(:,:) = 0.0_JPRB
ZDISD(:,:) = 0.0_JPRB
ZDISU(:,:) = 0.0_JPRB
ZDBDT(:,:,:) = 0.0_JPRB
ZDWFSU(:,:)  = 0.0_JPRB

!     ------------------------------------------------------------------

!*         1.2.3   EXCHANGE WITH THE BOUNDARIES
!                  ----------------------------

IF (LH2OCO2) THEN
  CALL LWVBRAD &
   & ( KIDIA , KFDIA , KLON  , KLEV  , KUAER,&
   & PGASUR , PGBSUR , PGATOP , PGBTOP,&
   & PABCU5 , PADJD5 , PADJU5 ,&
   & PB5    , PBINT5 , PBSUR5 , PBTOP5,&
   & PDISD5 , PDISU5 , PEMIS5 , PEMIW5,&
   & ZDWFSU35, PFLUC5 ,&
   & PABCU  , ZADJD  , ZADJU  ,&
   & PB     , PBINT  , PBSUR  , PBTOP ,&
   & ZDISD  , ZDISU  , PEMIS  , PEMIW ,&
   & ZDWFSU , PFLUC  &
   & )
ELSE
  CALL LWVBAD &
   & ( KIDIA , KFDIA , KLON  , KLEV  , KUAER,&
   & PGASUR , PGBSUR , PGATOP , PGBTOP,&
   & PABCU5 , PADJD5 , PADJU5 ,&
   & PB5    , PBINT5 , PBSUR5 , PBTOP5,&
   & PDISD5 , PDISU5 , PEMIS5 , PEMIW5,&
   & ZDWFSU35, PFLUC5 ,&
   & PABCU  , ZADJD  , ZADJU  ,&
   & PB     , PBINT  , PBSUR  , PBTOP ,&
   & ZDISD  , ZDISU  , PEMIS  , PEMIW ,&
   & ZDWFSU , PFLUC  &
   & )
ENDIF

!     ------------------------------------------------------------------

!*         1.2.2   CONTRIBUTION FROM DISTANT LAYERS
!                  ---------------------------------

IF (LH2OCO2) THEN
  CALL LWVDRAD &
   & ( KIDIA , KFDIA, KLON , KLEV  , KTRAER,&
   & PGA    , PGB   , PABCU5, PDBDT5 ,&
   & PCNTRB5, PDISD5, PDISU5, ZDWFSU25,&
! saved in NL coputation - lwvdr
   & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
   & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
   & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
   & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15, &
! perturbation   
   & PABCU  , ZDBDT ,&
   & PCNTRB , ZDISD , ZDISU , ZDWFSU &
   & )
ELSE
  CALL LWVDAD &
   & ( KIDIA , KFDIA, KLON , KLEV  , KTRAER,&
   & PGA    , PGB   , PABCU5, PDBDT5 ,&
   & PCNTRB5, PDISD5, PDISU5, ZDWFSU25,&
! saved in NL coputation - lwvd
   & PTTA5  , PTTB5 ,&
! perturbation
   & PABCU  , ZDBDT ,&
   & PCNTRB , ZDISD , ZDISU , ZDWFSU &
   & )
ENDIF

!     ------------------------------------------------------------------

!*         1.2.1   CONTRIBUTION FROM ADJACENT LAYERS
!                  ---------------------------------

IF (LH2OCO2) THEN
  CALL LWVNRAD &
   & ( KIDIA, KFDIA, KLON  , KLEV , KUAER,&
   & PGA   , PGB   , PABCU5 , PDBSL5, &
   & PADJD5, PADJU5, PCNTRB5, PDBDT5, ZDWFSU15 ,&
   & PABCU , PDBSL , &
   & ZADJD , ZADJU , PCNTRB , ZDBDT , ZDWFSU &
   & )
ELSE
  CALL LWVNAD &
   & ( KIDIA, KFDIA, KLON  , KLEV , KUAER,&
   & PGA   , PGB   , PABCU5 , PDBSL5, &
   & PADJD5, PADJU5, PCNTRB5, PDBDT5, ZDWFSU15 ,&
   & PABCU , PDBSL , &
   & ZADJD , ZADJU , PCNTRB , ZDBDT , ZDWFSU &
   & )  
ENDIF

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWVAD',1,ZHOOK_HANDLE)
END SUBROUTINE LWVAD
