#ifdef RS6K
@PROCESS HOT
#endif
SUBROUTINE LWVDRAD &
 & ( KIDIA, KFDIA, KLON , KLEV  , KTRAER,&
 & PGA    , PGB    ,&
 & PABCU5 , PDBDT5 ,&
 & PCNTRB5, PDISD5 , PDISU5, PDWFSU5, &
! saved in NL coputation - lwvdr
 & PCOND5  , PTTPA5  , PTTPB5 , PTTA5 , PTTB5  ,&
 & PZZA5   , PZZB5   , PXNA5  , PXNB5 , PXDIVA5,&
 & PXDIVB5 , PZZA15  , PZZB15 , PXNA15, PXNB15 ,&
 & PXDIVA15, PXDIVB15, PTTPA15, PTTPB15, &
! perturbation
 & PABCU  , PDBDT  ,&
 & PCNTRB , PDISD  , PDISU , PDWFSU &
 & )

!**** *LWVDRAD*   - L.W., VERTICAL INTEGRATION, DISTANT LAYERS
!                      (adjoint)

!     PURPOSE.
!     --------
!           CARRIES OUT THE VERTICAL INTEGRATION ON THE DISTANT LAYERS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                    Dimension
!  PABCU5      PABCU         ABSORBER AMOUNTS               (KLON,NUA,3*KLEV+1)
!  PDBDT5      PDBDT         LAYER PLANCK FUNCTION GRADIENT (KLON,KLEV)
!  PGA         ---           PADE APPROXIMANTS              (KLON,NIPD,2)
!  PGB         ---           PADE APPROXIMANTS              (KLON,NIPD,2)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                       Dimension
!  PCNTRB5     PCNTRB        ENERGY EXCHANGE MATRIX        (KLON,KLEV+1,KLEV+1)
!  PDISD5      PDISD         CONTRIBUTION BY DISTANT LAYERS    (KLON,KLEV+1)
!  PDISU5      PDISU         CONTRIBUTION BY DISTANT LAYERS    (KLON,KLEV+1)
!  PDWFSU5     PDWFSU        SPECTRAL DOWNWARD FLUX AT SURFACE (KLON,NSIL)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. PERFORMS THE VERTICAL INTEGRATION CORRESPONDING TO THE
!     CONTRIBUTIONS OF THE DISTANT LAYERS USING TRAPEZOIDAL RULE

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       5-Dec-2006
!                       reduced version for H2O and CO2 based on lwvdad

!     MODIFICATIONS.
!     --------------
!        M.Janiskova   23-Jan-2008 condition PCOND for reduced loops
!        J.Hague       08-Feb-2008 code optimization 
!        M.Janiskova   21-May-2008 using shorter NPROMALW instead of KLON
!        D.Salmond     02-Jun-2008 code optimization

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NSIL     ,NIPD     ,NUA      ,NG1P1
USE YOPHNC   , ONLY : LWLOPT
USE YOELWRAD , ONLY : NPROMALW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU5(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBDT5(KLON,NSIL,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCNTRB5(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDISD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDISU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU5(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PABCU(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDBDT(KLON,NSIL,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDISD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDISU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU(KLON,NSIL)
! saved in NL coputation - lwvdr
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PCOND5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPA5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPB5(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVA5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVB5(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PZZB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXNB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVA15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PXDIVB15(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPA15(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PTTPB15(KLON,KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZTTP5(KIDIA:KFDIA,KTRAER,KLEV+1)
REAL(KIND=JPRB) :: ZTT(KIDIA:KFDIA,KTRAER), ZTTP(KIDIA:KFDIA,KTRAER,KLEV+1)
REAL(KIND=JPRB) :: ZCNTRB(KIDIA:KFDIA,KLEV+1,KLEV+1) 

REAL(KIND=JPRB) :: ZWW1, ZWW2, ZWW3, ZWW4, ZWW5, ZWW6
REAL(KIND=JPRB) :: ZWW15, ZWW25, ZWW35, ZWW45, ZWW55, ZWW65
REAL(KIND=JPRB) :: ZWD15, ZWD25, ZWD35, ZWD45, ZWD55, ZWD65
REAL(KIND=JPRB) :: ZXN, ZXD, ZZ
REAL(KIND=JPRB) :: ZXD5

INTEGER(KIND=JPIM) :: IJKL, IKD1, IKD2, IKJ, IKJP1, IKM1, IKN,&
 & IKP1, IKU1, IKU2, JA, JK, JKJ, JL, JLK, IKJ1
INTEGER(KIND=JPIM) :: IJA,I1,I2,I3,I4,I5,I6,I7,I8,I9
INTEGER(KIND=JPIM) :: IJA1

INTEGER(KIND=JPIM) :: JJ, JN, JX(KLON)

INTEGER(KIND=JPIM) :: IC, ILOOP, IJL
LOGICAL :: LL_DO(KIDIA:KFDIA) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!#include "lwttmtl.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.1     INITIALIZE LAYER CONTRIBUTIONS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWVDRAD',0,ZHOOK_HANDLE)

!ILOOP = (KFDIA/(KFDIA-KIDIA+1)-1)*NPROMALW
ILOOP = KIDIA-1

IF (.NOT. LWLOPT) THEN
  DO JK = 1, KLEV+1
    DO JL = KIDIA,KFDIA
      PDISD5(JL,JK) = 0.0_JPRB
      PDISU5(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO

!*         1.2     INITIALIZE TRANSMISSION FUNCTIONS
!                  ---------------------------------

!DO JA = 1, NTRA
  DO JA = 1,KTRAER
    DO JL = KIDIA,KFDIA
      IJL = JL - ILOOP

      ZTTP5(JL,JA,:) = 1.0_JPRB

      PTTA5(IJL,JA,:) = 1.0_JPRB
      PTTB5(IJL,JA,:) = 1.0_JPRB
    ENDDO
  ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

!*         2.2     CONTRIBUTION FROM DISTANT LAYERS
!                  ---------------------------------

!*         2.2.1   DISTANT AND ABOVE LAYERS
!                  ------------------------

!*         2.2.2   FIRST UPPER LEVEL
!                  -----------------

  DO JK = KLEV-1, 1, -1
    IKP1=JK+1
    IKN=(JK-1)*NG1P1+1
    IKD1= JK  *NG1P1+1

    DO JA = 1 , 8
      DO JL = KIDIA,KFDIA
        IJL = JL - ILOOP

        PZZA15(IJL,JA,JK) = &
         & SQRT(PABCU5(JL,JA,IKN)-PABCU5(JL,JA,IKD1)+1E-25_JPRB)
        ZXD5 = PGB(JL,JA,1,JK) &
         & + PZZA15(IJL,JA,JK)*(PGB(JL,JA,2,JK)+PZZA15(IJL,JA,JK) )
        PXNA15(IJL,JA,JK) = PGA(JL,JA,1,JK) &
         & + PZZA15(IJL,JA,JK)*PGA(JL,JA,2,JK)
        PXDIVA15(IJL,JA,JK) = 1.0_JPRB/ZXD5
        ZTTP5(JL,JA,1) = PXNA15(IJL,JA,JK)*PXDIVA15(IJL,JA,JK)
      ENDDO
    ENDDO

    DO JL = KIDIA,KFDIA
      PTTPA15(JL,JK) = ZTTP5(JL,3,1)
      IF (PTTPA15(JL,JK) < 0.0_JPRB) THEN
        ZTTP5(JL,3,1) = 0.0_JPRB
      ENDIF
      ZTTP5(JL,9,1) = ZTTP5(JL,8,1)
    ENDDO

!*         2.2.3   HIGHER UP
!                  ---------

    DO JKJ=IKP1,KLEV
      IKJP1=JKJ+1
      IKD2= JKJ  *NG1P1+1
      IKJ1=JKJ-JK

      I1 = JKJ
      I2 = KLEV+JKJ
      I3 = 2*KLEV+JKJ
      I4 = 3*KLEV+JKJ
      I5 = 4*KLEV+JKJ
      I6 = 5*KLEV+JKJ
      I7 = 6*KLEV+JKJ
      I8 = 7*KLEV+JKJ
      I9 = 8*KLEV+JKJ   

      DO JL = KIDIA,KFDIA
        IJL = JL - ILOOP
        IF (JK > 1 .AND. (IKJP1-JK) > 3) THEN
          PCOND5(IJL,JKJ,JK) = &
           & ABS(PCNTRB5(JL,JK+2,IKJP1)-PCNTRB5(JL,JK+1,IKJP1))
        ELSE
          PCOND5(IJL,JKJ,JK) = 9999._JPRB
        ENDIF
      ENDDO

      IC=0
      LL_DO(:)=.TRUE.
      DO JL = KIDIA,KFDIA
        IJL = JL - ILOOP

!exp_A
!        IF (PCOND5(IJL,JKJ,JK) < 0.0001_JPRB) THEN
!exp_B
!        IF (PCOND5(IJL,JKJ,JK) < 0.00001_JPRB) THEN
!exp_C
        IF (PCOND5(IJL,JKJ,JK) < 0.001_JPRB) THEN
          PCNTRB5(JL,JK,IKJP1) = PCNTRB5(JL,JK+1,IKJP1)
          PDISD5(JL,JK) = PDISD5(JL,JK)+PCNTRB5(JL,JK,IKJP1)
          IC=IC+1
          LL_DO(JL)=.FALSE.
        ENDIF
      ENDDO

    JJ=0
    DO JL = KIDIA,KFDIA
      IF (LL_DO(JL)) THEN
        JJ=JJ+1
        JX(JJ)=JL
      ENDIF
    ENDDO
    JN=JJ

      IF (IC /= KFDIA-KIDIA+1) THEN

        DO JA = 1 , 8
          IJA = (JA-1)*KLEV+JKJ
!         IJA1= (JA  )*KLEV+JKJ
          DO JJ=1,JN
            JL=JX(JJ)
              IJL = JL - ILOOP

              PZZA5(IJL,IJA,JK) = &
               & SQRT(PABCU5(JL,JA,IKN)-PABCU5(JL,JA,IKD2)+1E-25_JPRB)
              ZXD5 = PGB(JL,JA,1,JKJ) &
               & + PZZA5(IJL,IJA,JK)*(PGB(JL,JA,2,JKJ)+PZZA5(IJL,IJA,JK))
              PXNA5(IJL,IJA,JK) = PGA(JL,JA,1,JKJ) &
               & + PZZA5(IJL,IJA,JK)*PGA( JL,JA,2,JKJ)
              PXDIVA5(IJL,IJA,JK) = 1.0_JPRB/ZXD5
              ZTTP5(JL,JA,IKJ1+1) = PXNA5(IJL,IJA,JK)*PXDIVA5(IJL,IJA,JK)

!             PZZA5(IJL,IJA1,JK) = &
!              & SQRT(PABCU5(JL,JA+1,IKN)-PABCU5(JL,JA+1,IKD2)+1.d-25)
!             ZXD5 = PGB(JL,JA+1,1,JKJ) &
!              & + PZZA5(IJL,IJA1,JK)*(PGB(JL,JA+1,2,JKJ)+PZZA5(IJL,IJA1,JK))
!             PXNA5(IJL,IJA1,JK) = PGA(JL,JA+1,1,JKJ) &
!              & + PZZA5(IJL,IJA1,JK)*PGA( JL,JA+1,2,JKJ)
!             PXDIVA5(IJL,IJA1,JK) = 1.0_JPRB/ZXD5
!             ZTTP5(JL,JA+1,IKJ1+1) = PXNA5(IJL,IJA1,JK)*PXDIVA5(IJL,IJA1,JK)
          ENDDO
        ENDDO

        DO JJ=1,JN
          JL=JX(JJ)
            IJL = JL - ILOOP

            PTTPA5(IJL,IKJ1+1,JK) = ZTTP5(JL,3,IKJ1+1)
            IF (PTTPA5(IJL,IKJ1+1,JK) < 0.0_JPRB) THEN
              ZTTP5(JL,3,IKJ1+1) = 0.0_JPRB
            ENDIF
            ZTTP5(JL,9,IKJ1+1) = ZTTP5(JL,8,IKJ1+1)
        ENDDO

!        DO JA = 1, KTRAER
!          IJA = (JA-1)*KLEV+JKJ
!          DO JL = KIDIA,KFDIA
!            IF (LL_DO(JL)) THEN
!              IJL = JL - ILOOP
!              PTTA5(IJL,IJA,JK) = &
!             & (ZTTP5(JL,JA,IKJ1)+ZTTP5(JL,JA,IKJ1+1))*0.5_JPRB
!            ENDIF
!          ENDDO
!        ENDDO

        DO JA = 1, KTRAER-1,2
          DO JJ=1,JN
            JL=JX(JJ)
              IJL = JL - ILOOP
              IJA = (JA-1)*KLEV+JKJ
              IJA1= (JA  )*KLEV+JKJ

              PTTA5(IJL,IJA ,JK) = &
               & (ZTTP5(JL,JA,IKJ1)+ZTTP5(JL,JA,IKJ1+1))*0.5_JPRB
              PTTA5(IJL,IJA1,JK) = &
               & (ZTTP5(JL,JA+1,IKJ1)+ZTTP5(JL,JA+1,IKJ1+1))*0.5_JPRB
          ENDDO
        ENDDO
        IF (MOD(KTRAER,2) /= 0) THEN
          DO JJ=1,JN
            JL=JX(JJ)
              IJL = JL - ILOOP
              IJA = (KTRAER-1)*KLEV+JKJ

              PTTA5(IJL,IJA ,JK) = &
               & (ZTTP5(JL,KTRAER,IKJ1)+ZTTP5(JL,KTRAER,IKJ1+1))*0.5_JPRB
          ENDDO
        ENDIF

        DO JJ=1,JN
          JL=JX(JJ)

            IJL = JL - ILOOP
            ZWW15 = PDBDT5(JL,1,JKJ)*PTTA5(IJL,I1,JK)
            ZWW25 = PDBDT5(JL,2,JKJ)*PTTA5(IJL,I2,JK)*PTTA5(IJL,I7,JK)
            ZWW35 = PDBDT5(JL,3,JKJ)*PTTA5(IJL,I4,JK)*PTTA5(IJL,I8,JK)
            ZWW45 = PDBDT5(JL,4,JKJ)*PTTA5(IJL,I5,JK)*PTTA5(IJL,I9,JK)
            ZWW55 = PDBDT5(JL,5,JKJ)*PTTA5(IJL,I3,JK)
            ZWW65 = PDBDT5(JL,6,JKJ)*PTTA5(IJL,I6,JK)
            PCNTRB5(JL,JK,IKJP1) = ZWW15+ZWW25+ZWW35+ZWW45+ZWW55+ZWW65
            PDISD5(JL,JK) = PDISD5(JL,JK)+PCNTRB5(JL,JK,IKJP1)
            IF (JK == 1) THEN
              PDWFSU5(JL,1) = PDWFSU5(JL,1)+ZWW15
              PDWFSU5(JL,2) = PDWFSU5(JL,2)+ZWW25
              PDWFSU5(JL,3) = PDWFSU5(JL,3)+ZWW35
              PDWFSU5(JL,4) = PDWFSU5(JL,4)+ZWW45
              PDWFSU5(JL,5) = PDWFSU5(JL,5)+ZWW55
              PDWFSU5(JL,6) = PDWFSU5(JL,6)+ZWW65
            ENDIF
        ENDDO

      ENDIF
    ENDDO
  ENDDO


!*         2.2.4   DISTANT AND BELOW LAYERS
!                  ------------------------

!*         2.2.5   FIRST LOWER LEVEL
!                  -----------------

  DO JK=3,KLEV+1
    IKN=(JK-1)*NG1P1+1
    IKM1=JK-1
    IKJ=JK-2
    IKU1= IKJ  *NG1P1+1

    DO JA = 1 , 8
      DO JL = KIDIA,KFDIA
        IJL = JL - ILOOP

        PZZB15(IJL,JA,JK) = &
         & SQRT(PABCU5(JL,JA,IKU1)-PABCU5(JL,JA,IKN)+1E-25_JPRB)
        ZXD5 = PGB(JL,JA,1,IKJ) &
         & + PZZB15(IJL,JA,JK)*(PGB(JL,JA,2,IKJ)+PZZB15(IJL,JA,JK) )
        PXNB15(IJL,JA,JK) = PGA(JL,JA,1,IKJ) &
         & + PZZB15(IJL,JA,JK)*PGA( JL,JA,2,IKJ)
        PXDIVB15(IJL,JA,JK) = 1.0_JPRB/ZXD5
        ZTTP5(JL,JA,1) = PXNB15(IJL,JA,JK)*PXDIVB15(IJL,JA,JK)
      ENDDO
    ENDDO

    DO JL = KIDIA,KFDIA
      PTTPB15(JL,JK) = ZTTP5(JL,3,1)
      IF (PTTPB15(JL,JK) < 0.0_JPRB) THEN
        ZTTP5(JL,3,1) = 0.0_JPRB
      ENDIF
      ZTTP5(JL,9,1) = ZTTP5(JL,8,1)
    ENDDO
  
!*         2.2.6   DOWN BELOW
!                  ----------

    DO JLK=1,IKJ
      IJKL=IKM1-JLK
      IKU2=(IJKL-1)*NG1P1+1

      I1 = JLK
      I2 = KLEV+JLK
      I3 = 2*KLEV+JLK
      I4 = 3*KLEV+JLK
      I5 = 4*KLEV+JLK
      I6 = 5*KLEV+JLK
      I7 = 6*KLEV+JLK
      I8 = 7*KLEV+JLK
      I9 = 8*KLEV+JLK

      DO JL = KIDIA,KFDIA
        IJL = JL - ILOOP

        IF ((JK-IJKL) > 3) THEN
          PCOND5(IJL,JLK,JK) = &
           & ABS(PCNTRB5(JL,JK-1,IJKL)-PCNTRB5(JL,JK-2,IJKL))
        ELSE
          PCOND5(IJL,JLK,JK) = 9999._JPRB
        ENDIF
      ENDDO

      IC=0
      LL_DO(:)=.TRUE.
      DO JL = KIDIA,KFDIA
        IJL = JL - ILOOP

!exp_A
!        IF (PCOND5(IJL,JLK,JK) < 0.0001_JPRB) THEN
!exp_B
!        IF (PCOND5(IJL,JLK,JK) < 0.00001_JPRB) THEN
!exp_C
        IF (PCOND5(IJL,JLK,JK) < 0.001_JPRB) THEN 
          IC=IC+1
          LL_DO(JL)=.FALSE. 
          PCNTRB5(JL,JK,IJKL) = PCNTRB5(JL,JK-1,IJKL)
          PDISU5(JL,JK) = PDISU5(JL,JK)+PCNTRB5(JL,JK,IJKL)
        ENDIF
      ENDDO

    JJ=0
    DO JL = KIDIA,KFDIA
      IF (LL_DO(JL)) THEN
        JJ=JJ+1
        JX(JJ)=JL
      ENDIF
    ENDDO
    JN=JJ


      IF (IC /= KFDIA-KIDIA+1) THEN
      
        DO JA = 1 , 8
          IJA = (JA-1)*KLEV+JLK
!         IJA1= (JA  )*KLEV+JLK
          DO JJ=1,JN
            JL=JX(JJ)
              IJL = JL - ILOOP

              PZZB5(IJL,IJA,JK) = &
               & SQRT(PABCU5(JL,JA,IKU2)-PABCU5(JL,JA,IKN)+1E-25_JPRB)
              ZXD5 = PGB(JL,JA,1,IJKL) &
               & + PZZB5(IJL,IJA,JK)*(PGB(JL,JA,2,IJKL)+PZZB5(IJL,IJA,JK) )
              PXNB5(IJL,IJA,JK) = PGA(JL,JA,1,IJKL) &
               & + PZZB5(IJL,IJA,JK)*PGA(JL,JA,2,IJKL)
              PXDIVB5(IJL,IJA,JK) = 1.0_JPRB/ZXD5
              ZTTP5(JL,JA,JLK+1) = PXNB5(IJL,IJA,JK)*PXDIVB5(IJL,IJA,JK)

!             PZZB5(IJL,IJA1,JK) = &
!              & SQRT(PABCU5(JL,JA+1,IKU2)-PABCU5(JL,JA+1,IKN)+1.d-25)
!             ZXD5 = PGB(JL,JA+1,1,IJKL) &
!              & + PZZB5(IJL,IJA1,JK)*(PGB(JL,JA+1,2,IJKL)+PZZB5(IJL,IJA1,JK) )
!             PXNB5(IJL,IJA1,JK) = PGA(JL,JA+1,1,IJKL) &
!              & + PZZB5(IJL,IJA1,JK)*PGA(JL,JA+1,2,IJKL)
!             PXDIVB5(IJL,IJA1,JK) = 1.0_JPRB/ZXD5
!             ZTTP5(JL,JA+1,JLK+1) = PXNB5(IJL,IJA1,JK)*PXDIVB5(IJL,IJA1,JK)
          ENDDO
        ENDDO

        DO JJ=1,JN
          JL=JX(JJ)
            IJL = JL - ILOOP

            PTTPB5(IJL,JLK+1,JK) = ZTTP5(JL,3,JLK+1)
            IF (PTTPB5(IJL,JLK+1,JK) < 0.0_JPRB) THEN
              ZTTP5(JL,3,JLK+1) = 0.0_JPRB
            ENDIF
            ZTTP5(JL,9,JLK+1) = ZTTP5(JL,8,JLK+1)
        ENDDO

!        DO JA = 1, KTRAER
!          DO JL = KIDIA,KFDIA
!            IF (LL_DO(JL)) THEN
!              IJL = JL - ILOOP
!              IJA = (JA-1)*KLEV+JLK
!              PTTB5(IJL,IJA,JK) = &
!               & (ZTTP5(JL,JA,JLK)+ZTTP5(JL,JA,JLK+1))*0.5_JPRB
!            ENDIF
!          ENDDO
!        ENDDO

        DO JA = 1, KTRAER-1,2
          DO JJ=1,JN
            JL=JX(JJ)
              IJL = JL - ILOOP
              IJA = (JA-1)*KLEV+JLK
              IJA1= (JA  )*KLEV+JLK

              PTTB5(IJL,IJA ,JK) = &
               & (ZTTP5(JL,JA,JLK)+ZTTP5(JL,JA,JLK+1))*0.5_JPRB
              PTTB5(IJL,IJA1,JK) = &
               & (ZTTP5(JL,JA+1,JLK)+ZTTP5(JL,JA+1,JLK+1))*0.5_JPRB
          ENDDO
        ENDDO
        IF (MOD(KTRAER,2) /= 0) THEN
          DO JJ=1,JN
            JL=JX(JJ)
              IJL = JL - ILOOP
              IJA = (KTRAER-1)*KLEV+JLK

              PTTB5(IJL,IJA ,JK) = &
               & (ZTTP5(JL,KTRAER,JLK)+ZTTP5(JL,KTRAER,JLK+1))*0.5_JPRB
          ENDDO
        ENDIF

        DO JJ=1,JN
          JL=JX(JJ)
            IJL = JL - ILOOP
            ZWD15 = PDBDT5(JL,1,IJKL)*PTTB5(IJL,I1,JK)
            ZWD25 = PDBDT5(JL,2,IJKL)*PTTB5(IJL,I2,JK)*PTTB5(IJL,I7,JK)
            ZWD35 = PDBDT5(JL,3,IJKL)*PTTB5(IJL,I4,JK)*PTTB5(IJL,I8,JK)
            ZWD45 = PDBDT5(JL,4,IJKL)*PTTB5(IJL,I5,JK)*PTTB5(IJL,I9,JK)
            ZWD55 = PDBDT5(JL,5,IJKL)*PTTB5(IJL,I3,JK)
            ZWD65 = PDBDT5(JL,6,IJKL)*PTTB5(IJL,I6,JK)
            PCNTRB5(JL,JK,IJKL) = ZWD15+ZWD25+ZWD35+ZWD45+ZWD55+ZWD65
            PDISU5(JL,JK) = PDISU5(JL,JK)+PCNTRB5(JL,JK,IJKL)
        ENDDO
      ENDIF

    ENDDO
  ENDDO

ENDIF

!* -------------------------------------------------------------------

!*         0.    ADJOINT CALCULATIONS
!                --------------------

!* Set local variables to zero

ZCNTRB(KIDIA:KFDIA,:,:) =PCNTRB(KIDIA:KFDIA,:,:)

ZTT(:,:) = 0.0_JPRB
ZTTP(:,:,:) = 0.0_JPRB

!* -------------------------------------------------------------------

!-----------------------
!* BACKWARD COMPUTATION
!-----------------------

!*         1.2.    VERTICAL INTEGRATION
!                  --------------------

!*         1.2.2     CONTRIBUTION FROM DISTANT LAYERS
!                    ---------------------------------

!*         1.2.2.4   DISTANT AND BELOW LAYERS
!                    ------------------------

DO JK=KLEV+1,3,-1
  IKN=(JK-1)*NG1P1+1
  IKM1=JK-1
  IKJ=JK-2
  IKU1= IKJ  *NG1P1+1

!*         1.2.2.6   DOWN BELOW
!                    ----------

  DO JLK=IKJ,1,-1
    IJKL=IKM1-JLK
    IKU2=(IJKL-1)*NG1P1+1

    I1 = JLK
    I2 = KLEV+JLK
    I3 = 2*KLEV+JLK
    I4 = 3*KLEV+JLK
    I5 = 4*KLEV+JLK
    I6 = 5*KLEV+JLK
    I7 = 6*KLEV+JLK
    I8 = 7*KLEV+JLK
    I9 = 8*KLEV+JLK

    LL_DO(:)=.TRUE.
    IC=0
    DO JL = KIDIA,KFDIA
      IJL = JL - ILOOP
      IF (PCOND5(IJL,JLK,JK) < 0.001_JPRB) THEN
        ZCNTRB(JL,JK,IJKL) = ZCNTRB(JL,JK,IJKL)+PDISU(JL,JK)
        ZCNTRB(JL,JK-1,IJKL) = ZCNTRB(JL,JK-1,IJKL)+ZCNTRB(JL,JK,IJKL)
        ZCNTRB(JL,JK,IJKL) = 0.0_JPRB
        IC=IC+1
        LL_DO(JL)=.FALSE.
      ENDIF
    ENDDO

    JJ=0
    DO JL = KIDIA,KFDIA
      IF (LL_DO(JL)) THEN
        JJ=JJ+1
        JX(JJ)=JL
      ENDIF
    ENDDO
    JN=JJ


    IF (IC /= KFDIA-KIDIA+1) THEN

        DO JJ=1,JN
          JL=JX(JJ)
          IJL = JL - ILOOP

          ZCNTRB(JL,JK,IJKL) = ZCNTRB(JL,JK,IJKL)+PDISU(JL,JK)

          ZTT(JL,6) = ZTT(JL,6)+PDBDT5(JL,6,IJKL)*ZCNTRB(JL,JK,IJKL)
          PDBDT(JL,6,IJKL) = PDBDT(JL,6,IJKL) &
           & + PTTB5(IJL,I6,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,3) = ZTT(JL,3)+PDBDT5(JL,5,IJKL)*ZCNTRB(JL,JK,IJKL)
          PDBDT(JL,5,IJKL) = PDBDT(JL,5,IJKL) &
           & + PTTB5(IJL,I3,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,9) = ZTT(JL,9) &
           & + PDBDT5(JL,4,IJKL)*PTTB5(IJL,I5,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,5) = ZTT(JL,5) &
           & + PDBDT5(JL,4,IJKL)*PTTB5(IJL,I9,JK)*ZCNTRB(JL,JK,IJKL)
          PDBDT(JL,4,IJKL) = PDBDT(JL,4,IJKL) &
           & + PTTB5(IJL,I5,JK)*PTTB5(IJL,I9,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,8) = ZTT(JL,8) &
           & + PDBDT5(JL,3,IJKL)*PTTB5(IJL,I4,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,4) = ZTT(JL,4) &
           & + PDBDT5(JL,3,IJKL)*PTTB5(IJL,I8,JK)*ZCNTRB(JL,JK,IJKL)
          PDBDT(JL,3,IJKL) = PDBDT(JL,3,IJKL) &
           & + PTTB5(IJL,I4,JK)*PTTB5(IJL,I8,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,7) = ZTT(JL,7) &
           & + PDBDT5(JL,2,IJKL)*PTTB5(IJL,I2,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,2) = ZTT(JL,2) &
           & + PDBDT5(JL,2,IJKL)*PTTB5(IJL,I7,JK)*ZCNTRB(JL,JK,IJKL)
          PDBDT(JL,2,IJKL) = PDBDT(JL,2,IJKL) &
           & + PTTB5(IJL,I2,JK)*PTTB5(IJL,I7,JK)*ZCNTRB(JL,JK,IJKL)
          ZTT(JL,1) = ZTT(JL,1)+PDBDT5(JL,1,IJKL)*ZCNTRB(JL,JK,IJKL)
          PDBDT(JL,1,IJKL) = PDBDT(JL,1,IJKL) &
           & + PTTB5(IJL,I1,JK)*ZCNTRB(JL,JK,IJKL)
          ZCNTRB(JL,JK,IJKL) = 0.0_JPRB
      ENDDO

!      DO JA = 1, KTRAER
!        DO JL = KIDIA,KFDIA
!          IF (LL_DO(JL)) THEN
!            ZTTP(JL,JA,JLK)   = ZTTP(JL,JA,JLK)+0.5_JPRB*ZTT(JL,JA)
!            ZTTP(JL,JA,JLK+1) = ZTTP(JL,JA,JLK+1)+0.5_JPRB*ZTT(JL,JA)
!            ZTT(JL,JA) = 0.0_JPRB
!          ENDIF
!        ENDDO
!      ENDDO

      IF (MOD(KTRAER,2) /= 0) THEN
        DO JJ=1,JN
          JL=JX(JJ)
            ZTTP(JL,KTRAER,JLK)   = &
             & ZTTP(JL,KTRAER,JLK)+0.5_JPRB*ZTT(JL,KTRAER)
            ZTTP(JL,KTRAER,JLK+1) = &
             & ZTTP(JL,KTRAER,JLK+1)+0.5_JPRB*ZTT(JL,KTRAER)
            ZTT(JL,KTRAER) = 0.0_JPRB
        ENDDO
      ENDIF

      DO JA = 1, KTRAER-1,2
        DO JJ=1,JN
          JL=JX(JJ)
            ZTTP(JL,JA+1,JLK)   = ZTTP(JL,JA+1,JLK)+0.5_JPRB*ZTT(JL,JA+1)
            ZTTP(JL,JA+1,JLK+1) = ZTTP(JL,JA+1,JLK+1)+0.5_JPRB*ZTT(JL,JA+1)
            ZTT(JL,JA+1) = 0.0_JPRB

            ZTTP(JL,JA,JLK)   = ZTTP(JL,JA,JLK)+0.5_JPRB*ZTT(JL,JA)
            ZTTP(JL,JA,JLK+1) = ZTTP(JL,JA,JLK+1)+0.5_JPRB*ZTT(JL,JA)
            ZTT(JL,JA) = 0.0_JPRB
        ENDDO
      ENDDO

      DO JJ=1,JN
        JL=JX(JJ)
          IJL = JL - ILOOP

          ZTTP(JL,8,JLK+1) = ZTTP(JL,8,JLK+1)+ZTTP(JL,9,JLK+1)
          ZTTP(JL,9,JLK+1) = 0.0_JPRB
          IF (PTTPB5(IJL,JLK+1,JK) < 0.0_JPRB) THEN
            ZTTP(JL,3,JLK+1) = 0.0_JPRB
          ENDIF
      ENDDO

!     DO JA = 1 , 8, 2
      DO JA = 7 , 1, -2
        IJA = (JA-1)*KLEV+JLK
        IJA1 = JA*KLEV+JLK
!       DO JJ=1,JN
        DO JJ=JN,1,-1
          JL=JX(JJ)
            IJL = JL - ILOOP

            ZXN = PXDIVB5(IJL,IJA,JK)*ZTTP(JL,JA,JLK+1)
            ZXD = -PXNB5(IJL,IJA,JK)*PXDIVB5(IJL,IJA,JK) * ZXN
!           ZTTP(JL,JA,JLK+1) = 0.0_JPRB
            ZZ = 0.5*(PGA(JL,JA,2,IJKL)*ZXN &
             & +(PGB(JL,JA,2,IJKL)+2.0_JPRB*PZZB5(IJL,IJA,JK))*ZXD)
!           IF (PZZB5(IJL,IJA,JK) > 0.0_JPRB) THEN
              PABCU(JL,JA,IKU2) = PABCU(JL,JA,IKU2)+ZZ/PZZB5(IJL,IJA,JK)
              PABCU(JL,JA,IKN) = PABCU(JL,JA,IKN)-ZZ/PZZB5(IJL,IJA,JK)
!           ENDIF

            ZXN = PXDIVB5(IJL,IJA1,JK)*ZTTP(JL,JA+1,JLK+1)
            ZXD = -PXNB5(IJL,IJA1,JK)*PXDIVB5(IJL,IJA1,JK) * ZXN
!           ZTTP(JL,JA+1,JLK+1) = 0.0_JPRB
            ZZ = 0.5*(PGA(JL,JA+1,2,IJKL)*ZXN &
             & +(PGB(JL,JA+1,2,IJKL)+2.0_JPRB*PZZB5(IJL,IJA1,JK))*ZXD)
!           IF (PZZB5(IJL,IJA1,JK) > 0.0_JPRB) THEN
              PABCU(JL,JA+1,IKU2) = PABCU(JL,JA+1,IKU2)+ZZ/PZZB5(IJL,IJA1,JK)
              PABCU(JL,JA+1,IKN) = PABCU(JL,JA+1,IKN)-ZZ/PZZB5(IJL,IJA1,JK)
!           ENDIF

        ENDDO
      ENDDO
!     DO JA = 1 , 8, 2
      DO JA = 7 , 1, -2
        IJA = (JA-1)*KLEV+JLK
        IJA1 = JA*KLEV+JLK
!       DO JJ=1,JN
        DO JJ=JN,1,-1
          JL=JX(JJ)
            ZTTP(JL,JA,JLK+1) = 0.0_JPRB
            ZTTP(JL,JA+1,JLK+1) = 0.0_JPRB
        ENDDO
      ENDDO

    ENDIF

  ENDDO

!*         1.2.2.5   FIRST LOWER LEVEL
!                    -----------------

  DO JL = KIDIA,KFDIA
    ZTTP(JL,8,1) = ZTTP(JL,8,1)+ZTTP(JL,9,1)
    ZTTP(JL,9,1) = 0.0_JPRB
    IF (PTTPB15(JL,JK) < 0.0_JPRB) THEN
      ZTTP(JL,3,1) = 0.0_JPRB
    ENDIF
  ENDDO

  DO JA = 1 , 8
    DO JL = KIDIA,KFDIA
      IJL = JL - ILOOP

      ZXN = PXDIVB15(IJL,JA,JK)*ZTTP(JL,JA,1)
      ZXD = -PXNB15(IJL,JA,JK)*PXDIVB15(IJL,JA,JK) &
       & * PXDIVB15(IJL,JA,JK)*ZTTP(JL,JA,1)
      ZTTP(JL,JA,1) = 0.0_JPRB
      ZZ = PGA(JL,JA,2,IKJ)*ZXN
      ZZ = ZZ+(PGB( JL,JA,2,IKJ)+2.0_JPRB*PZZB15(IJL,JA,JK))*ZXD
!      IF (PZZB15(IJL,JA,JK) > 0.0_JPRB) THEN
        PABCU(JL,JA,IKU1) = PABCU(JL,JA,IKU1)+0.5_JPRB*ZZ/PZZB15(IJL,JA,JK)
        PABCU(JL,JA,IKN) = PABCU(JL,JA,IKN)-0.5_JPRB*ZZ/PZZB15(IJL,JA,JK)
!      ENDIF
    ENDDO
  ENDDO

ENDDO

!*         1.2.2.1   DISTANT AND ABOVE LAYERS 
!                    ------------------------

DO JK = 1, KLEV-1
  IKP1=JK+1
  IKN=(JK-1)*NG1P1+1
  IKD1= JK  *NG1P1+1

!*         1.2.2.3   HIGHER UP 
!                    ---------

  DO JKJ = KLEV , IKP1 , -1
    IKJP1=JKJ+1
    IKD2= JKJ  *NG1P1+1
    IKJ1=JKJ-JK

    I1 = JKJ
    I2 = KLEV+JKJ
    I3 = 2*KLEV+JKJ
    I4 = 3*KLEV+JKJ
    I5 = 4*KLEV+JKJ
    I6 = 5*KLEV+JKJ
    I7 = 6*KLEV+JKJ
    I8 = 7*KLEV+JKJ
    I9 = 8*KLEV+JKJ

    LL_DO(:)=.TRUE.
    IC=0
    DO JL = KIDIA,KFDIA
      IJL = JL - ILOOP

      IF (PCOND5(IJL,JKJ,JK) < 0.001_JPRB) THEN
        ZCNTRB(JL,JK,IKJP1) = ZCNTRB(JL,JK,IKJP1)+PDISD(JL,JK)
        ZCNTRB(JL,JK+1,IKJP1) = ZCNTRB(JL,JK+1,IKJP1)+ZCNTRB(JL,JK,IKJP1)
        ZCNTRB(JL,JK,IKJP1) = 0.0_JPRB
        IC=IC+1
        LL_DO(JL)=.FALSE.
      ENDIF
    ENDDO

    JJ=0
    DO JL = KIDIA,KFDIA
      IF (LL_DO(JL)) THEN
        JJ=JJ+1
        JX(JJ)=JL
      ENDIF
    ENDDO
    JN=JJ

    IF(IC /= KFDIA-KIDIA+1) THEN

      IF (JK == 1) THEN
        DO JJ=1,JN
          JL=JX(JJ)
            IJL = JL - ILOOP

            ZCNTRB(JL,JK,IKJP1) = ZCNTRB(JL,JK,IKJP1)+PDISD(JL,JK)
            ZWW6 = PDWFSU(JL,6)
            ZWW5 = PDWFSU(JL,5)
            ZWW4 = PDWFSU(JL,4)
            ZWW3 = PDWFSU(JL,3)
            ZWW2 = PDWFSU(JL,2)
            ZWW1 = PDWFSU(JL,1)
            ZWW6 = ZWW6+ZCNTRB(JL,JK,IKJP1)
            ZWW5 = ZWW5+ZCNTRB(JL,JK,IKJP1)
            ZWW4 = ZWW4+ZCNTRB(JL,JK,IKJP1)
            ZWW3 = ZWW3+ZCNTRB(JL,JK,IKJP1)
            ZWW2 = ZWW2+ZCNTRB(JL,JK,IKJP1)
            ZWW1 = ZWW1+ZCNTRB(JL,JK,IKJP1)
            ZCNTRB(JL,JK,IKJP1) = 0.0_JPRB
      
            ZTT(JL,6) = ZTT(JL,6)+PDBDT5(JL,6,JKJ)*ZWW6
            PDBDT(JL,6,JKJ) = PDBDT(JL,6,JKJ)+PTTA5(IJL,I6,JK)*ZWW6
            ZTT(JL,3) = ZTT(JL,3)+PDBDT5(JL,5,JKJ)*ZWW5
            PDBDT(JL,5,JKJ) = PDBDT(JL,5,JKJ)+PTTA5(IJL,I3,JK)*ZWW5
            ZTT(JL,9) = ZTT(JL,9)+PDBDT5(JL,4,JKJ)*PTTA5(IJL,I5,JK)*ZWW4
            ZTT(JL,5) = ZTT(JL,5)+PDBDT5(JL,4,JKJ)*PTTA5(IJL,I9,JK)*ZWW4
            PDBDT(JL,4,JKJ) = PDBDT(JL,4,JKJ) &
             & + PTTA5(IJL,I5,JK)*PTTA5(IJL,I9,JK)*ZWW4
            ZTT(JL,8) = ZTT(JL,8)+PDBDT5(JL,3,JKJ)*PTTA5(IJL,I4,JK)*ZWW3
            ZTT(JL,4) = ZTT(JL,4)+PDBDT5(JL,3,JKJ)*PTTA5(IJL,I8,JK)*ZWW3
            PDBDT(JL,3,JKJ) = PDBDT(JL,3,JKJ) &
             & + PTTA5(IJL,I4,JK)*PTTA5(IJL,I8,JK)*ZWW3
            ZTT(JL,7) = ZTT(JL,7)+PDBDT5(JL,2,JKJ)*PTTA5(IJL,I2,JK)*ZWW2
            ZTT(JL,2) = ZTT(JL,2)+PDBDT5(JL,2,JKJ)*PTTA5(IJL,I7,JK)*ZWW2
            PDBDT(JL,2,JKJ) = PDBDT(JL,2,JKJ) &
             & + PTTA5(IJL,I2,JK)*PTTA5(IJL,I7,JK)*ZWW2
            ZTT(JL,1) = ZTT(JL,1)+PDBDT5(JL,1,JKJ)*ZWW1
            PDBDT(JL,1,JKJ) = PDBDT(JL,1,JKJ)+PTTA5(IJL,I1,JK)*ZWW1
        ENDDO
      ELSE
        DO JJ=1,JN
          JL=JX(JJ)
            IJL = JL - ILOOP

            ZCNTRB(JL,JK,IKJP1) = ZCNTRB(JL,JK,IKJP1)+PDISD(JL,JK)
      
            ZTT(JL,6) = ZTT(JL,6)+PDBDT5(JL,6,JKJ)*ZCNTRB(JL,JK,IKJP1)
            PDBDT(JL,6,JKJ) = PDBDT(JL,6,JKJ) &
             & +PTTA5(IJL,I6,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,3) = ZTT(JL,3)+PDBDT5(JL,5,JKJ)*ZCNTRB(JL,JK,IKJP1)
            PDBDT(JL,5,JKJ) = PDBDT(JL,5,JKJ) &
             & +PTTA5(IJL,I3,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,9) = ZTT(JL,9) &
             & +PDBDT5(JL,4,JKJ)*PTTA5(IJL,I5,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,5) = ZTT(JL,5) &
             & +PDBDT5(JL,4,JKJ)*PTTA5(IJL,I9,JK)*ZCNTRB(JL,JK,IKJP1)
            PDBDT(JL,4,JKJ) = PDBDT(JL,4,JKJ) &
             & + PTTA5(IJL,I5,JK)*PTTA5(IJL,I9,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,8) = ZTT(JL,8) &
             & +PDBDT5(JL,3,JKJ)*PTTA5(IJL,I4,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,4) = ZTT(JL,4) &
             & +PDBDT5(JL,3,JKJ)*PTTA5(IJL,I8,JK)*ZCNTRB(JL,JK,IKJP1)
            PDBDT(JL,3,JKJ) = PDBDT(JL,3,JKJ) &
             & + PTTA5(IJL,I4,JK)*PTTA5(IJL,I8,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,7) = ZTT(JL,7) &
             & +PDBDT5(JL,2,JKJ)*PTTA5(IJL,I2,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,2) = ZTT(JL,2) &
             & +PDBDT5(JL,2,JKJ)*PTTA5(IJL,I7,JK)*ZCNTRB(JL,JK,IKJP1)
            PDBDT(JL,2,JKJ) = PDBDT(JL,2,JKJ) &
             & + PTTA5(IJL,I2,JK)*PTTA5(IJL,I7,JK)*ZCNTRB(JL,JK,IKJP1)
            ZTT(JL,1) = ZTT(JL,1)+PDBDT5(JL,1,JKJ)*ZCNTRB(JL,JK,IKJP1)
            PDBDT(JL,1,JKJ) = PDBDT(JL,1,JKJ) &
             & +PTTA5(IJL,I1,JK)*ZCNTRB(JL,JK,IKJP1)

            ZCNTRB(JL,JK,IKJP1) = 0.0_JPRB 
        ENDDO
      ENDIF 

!      DO JA = 1, KTRAER
!        DO JL = KIDIA,KFDIA
!          IF (LL_DO(JL)) THEN
!            ZTTP(JL,JA,IKJ1)   = ZTTP(JL,JA,IKJ1)+0.5_JPRB*ZTT(JL,JA)
!            ZTTP(JL,JA,IKJ1+1) = ZTTP(JL,JA,IKJ1+1)+0.5_JPRB*ZTT(JL,JA)
!            ZTT(JL,JA) = 0.0_JPRB
!          ENDIF
!        ENDDO
!      ENDDO

      IF (MOD(KTRAER,2) /= 0) THEN
        DO JJ=1,JN
          JL=JX(JJ)
            ZTTP(JL,KTRAER,IKJ1)   = &
             & ZTTP(JL,KTRAER,IKJ1)+0.5_JPRB*ZTT(JL,KTRAER)
            ZTTP(JL,KTRAER,IKJ1+1) = &
             & ZTTP(JL,KTRAER,IKJ1+1)+0.5_JPRB*ZTT(JL,KTRAER)
            ZTT(JL,KTRAER) = 0.0_JPRB
        ENDDO
      ENDIF
      DO JA = 1, KTRAER-1,2
        DO JJ=1,JN
          JL=JX(JJ)
            ZTTP(JL,JA+1,IKJ1)   = ZTTP(JL,JA+1,IKJ1)+0.5_JPRB*ZTT(JL,JA+1)
            ZTTP(JL,JA+1,IKJ1+1) = ZTTP(JL,JA+1,IKJ1+1)+0.5_JPRB*ZTT(JL,JA+1)
            ZTT(JL,JA+1) = 0.0_JPRB

            ZTTP(JL,JA,IKJ1)   = ZTTP(JL,JA,IKJ1)+0.5_JPRB*ZTT(JL,JA)
            ZTTP(JL,JA,IKJ1+1) = ZTTP(JL,JA,IKJ1+1)+0.5_JPRB*ZTT(JL,JA)
            ZTT(JL,JA) = 0.0_JPRB
        ENDDO
      ENDDO

      DO JJ=1,JN
        JL=JX(JJ)
          IJL = JL - ILOOP

          ZTTP(JL,8,IKJ1+1) = ZTTP(JL,8,IKJ1+1)+ZTTP(JL,9,IKJ1+1)
          ZTTP(JL,9,IKJ1+1) = 0.0_JPRB
          IF (PTTPA5(IJL,IKJ1+1,JK) < 0.0_JPRB) THEN
            ZTTP(JL,3,IKJ1+1) = 0.0_JPRB
          ENDIF
      ENDDO

!     DO JA = 1 , 8, 2
      DO JA = 7 , 1, -2
        IJA = (JA-1)*KLEV+JKJ
        IJA1 = JA*KLEV+JKJ
!       DO JJ=1,JN
        DO JJ=JN,1,-1
          JL=JX(JJ)
            IJL = JL - ILOOP

            ZXN = PXDIVA5(IJL,IJA,JK)*ZTTP(JL,JA,IKJ1+1)
            ZXD = -PXNA5(IJL,IJA,JK)*PXDIVA5(IJL,IJA,JK)* ZXN
!           ZTTP(JL,JA,IKJ1+1) = 0.0_JPRB
            ZZ = 0.5*(PGA(JL,JA,2,JKJ)*ZXN &
             & +(PGB(JL,JA,2,JKJ)+2.0_JPRB*PZZA5(IJL,IJA,JK))*ZXD)
!           IF (PZZA5(IJL,IJA,JK) > 0.0_JPRB) THEN
              PABCU(JL,JA,IKN) = PABCU(JL,JA,IKN)+ZZ/PZZA5(IJL,IJA,JK)
              PABCU(JL,JA,IKD2) = PABCU(JL,JA,IKD2)-ZZ/PZZA5(IJL,IJA,JK)
!           ENDIF
            ZXN = PXDIVA5(IJL,IJA1,JK)*ZTTP(JL,JA+1,IKJ1+1)
            ZXD = -PXNA5(IJL,IJA1,JK)*PXDIVA5(IJL,IJA1,JK)* ZXN
!           ZTTP(JL,JA+1,IKJ1+1) = 0.0_JPRB
            ZZ = 0.5*(PGA(JL,JA+1,2,JKJ)*ZXN &
             & +(PGB(JL,JA+1,2,JKJ)+2.0_JPRB*PZZA5(IJL,IJA1,JK))*ZXD)
!           IF (PZZA5(IJL,IJA1,JK) > 0.0_JPRB) THEN
              PABCU(JL,JA+1,IKN) = PABCU(JL,JA+1,IKN)+ZZ/PZZA5(IJL,IJA1,JK)
              PABCU(JL,JA+1,IKD2) = PABCU(JL,JA+1,IKD2)-ZZ/PZZA5(IJL,IJA1,JK)
!           ENDIF
        ENDDO
      ENDDO
!     DO JA = 1 , 8, 2
      DO JA = 7 , 1, -2
        IJA = (JA-1)*KLEV+JKJ
        IJA1 = JA*KLEV+JKJ
!       DO JJ=1,JN
        DO JJ=JN,1,-1
          JL=JX(JJ)
            ZTTP(JL,JA,IKJ1+1) = 0.0_JPRB
            ZTTP(JL,JA+1,IKJ1+1) = 0.0_JPRB
        ENDDO
      ENDDO

    ENDIF

  ENDDO

!*         1.2.2.2   FIRST UPPER LEVEL
!                    -----------------

  DO JL = KIDIA,KFDIA
    ZTTP(JL,8,1) = ZTTP(JL,8,1)+ZTTP(JL,9,1)
    ZTTP(JL,9,1) = 0.0_JPRB
    IF (PTTPA15(JL,JK) < 0.0_JPRB) THEN
      ZTTP(JL,3,1) = 0.0_JPRB
    ENDIF
  ENDDO

  DO JA = 1 , 8
    DO JL = KIDIA,KFDIA
      IJL = JL - ILOOP

      ZXN = PXDIVA15(IJL,JA,JK)*ZTTP(JL,JA,1)
      ZXD = -PXNA15(IJL,JA,JK)*PXDIVA15(IJL,JA,JK) &
       & * PXDIVA15(IJL,JA,JK)*ZTTP(JL,JA,1)
      ZTTP(JL,JA,1) = 0.0_JPRB
      ZZ = PGA(JL,JA,2,JK)*ZXN
      ZZ = ZZ+(PGB(JL,JA,2,JK)+2.0_JPRB*PZZA15(IJL,JA,JK))*ZXD
!      IF (PZZA15(IJL,JA,JK) > 0.0_JPRB) THEN
        PABCU(JL,JA,IKN) = PABCU(JL,JA,IKN)+0.5_JPRB*ZZ/PZZA15(IJL,JA,JK)
        PABCU(JL,JA,IKD1) = PABCU(JL,JA,IKD1)-0.5_JPRB*ZZ/PZZA15(IJL,JA,JK)
!      ENDIF
    ENDDO
  ENDDO

ENDDO

!*         1.1.1     INITIALIZE LAYER CONTRIBUTIONS
!                    ------------------------------

DO JK = 1, KLEV+1
  DO JL = KIDIA,KFDIA
    PDISD (JL,JK) = 0.0_JPRB
    PDISU (JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWVDRAD',1,ZHOOK_HANDLE)
END SUBROUTINE LWVDRAD
