SUBROUTINE SRTM_SRTM_224GP_MCICA &
 & ( KIDIA,  KFDIA,  KLON , KLEV , KSW  , KCOLS , KCLDLY ,&
 &   PAER ,  PALBD,  PALBP, PAPH , PAP  , &
 &   PTS  ,  PTH  ,  PT   ,&
 &   PQ   ,  PCO2 ,  PCH4 , PN2O , PNO2 , POZN  , PRMU0 ,&
 &   PFRCL,  PTAUC,  PASYC, POMGC,&
 &   PFSUX,  PFSUC,  PFUVF, PFUVC, PPARF, PPARCF, PSUDU ,&
 &   PFDIR,  PCDIR,  PFDIF, PCDIF,&
 &   PAODSTRAT, PAAODSTRAT, PREFAODSTRAT,&
 &   PAODTM, PSSATM, PASFTM,&
 &   PAODPI, PSSAPI, PASFPI,&
 &   PMAC2SPAOD, PMAC2SPSSA, PMAC2SPASY)

!-- interface to RRTM_SW
!     JJMorcrette 030225
!     JJMorcrette 20050110  McICA version
!     JJMorcrette 20070614  bug-fix for solar duration
!     JJMorcrette 20071015  3D fields of CO2, CH4, N2O and NO2
!     D.Salmond  31-Oct-2007 Vector version in the style of RRTM from Meteo France & NEC
!     JJMorcrette 20091201  Total and clear-sky downward direct flux
!     T. van Noije Jan-2014 Included option for using aerosol optical properties
!                           from TM5 for EC-Earth
!     S. Boussetta March-2016 added option to use the anthropogenic aerosol MACv2 SP
!     S. Boussetta July-2016 added option to use prescribed pre-industrial aerosol from TM5 Climatology
!     M. Menegoz added option to use the CMIP6 stratospheric aerosol

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARSRTM  , ONLY : JPLAY
USE YOERAD   , ONLY : NAER,    LTM5AER
USE YOERAD   , ONLY : LMAC2SP, LCMIP6_PI_AEROSOLS, LCMIP6_STRATAER_FULL

USE YOESRTAER, ONLY : RSRTAUA, RSRPIZA, RSRASYA
USE YOMPHY3  , ONLY : RII0
USE YOMCST   , ONLY : RI0 
USE YOESRTWN , ONLY : RI0_SRTM

IMPLICIT NONE

!-- Input arguments

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KSW  
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOLS
INTEGER(KIND=JPIM),INTENT(IN)    :: KCLDLY(KCOLS) 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,6,KLEV)    ! top to bottom
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,KSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,KSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCO2(KLON,KLEV), PCH4(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PN2O(KLON,KLEV), PNO2(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMU0(KLON)

REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRCL(KLON,KCOLS,KLEV) ! bottom to top
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUC(KLON,KCOLS,KLEV) ! bottom to top
REAL(KIND=JPRB)   ,INTENT(IN)    :: PASYC(KLON,KCOLS,KLEV) ! bottom to top
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMGC(KLON,KCOLS,KLEV) ! bottom to top

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFSUX(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFSUC(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFUVF(KLON), PFUVC(KLON), PPARF(KLON), PPARCF(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSUDU(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFDIF(KLON,KLEV+1), PCDIF(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFDIR(KLON,KLEV+1), PCDIR(KLON,KLEV+1)

!-- CMIP6 prescribed stratospheric aerosols
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PAODSTRAT(KLON,KLEV,KSW)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PAAODSTRAT(KLON,KLEV,KSW)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PREFAODSTRAT(KLON,KLEV,KSW)

!-- TM5 
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PAODTM(KLON,KSW,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PSSATM(KLON,KSW,KLEV)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PASFTM(KLON,KSW,KLEV)

!-- from MAC2SP anthropogenic aerosol
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PMAC2SPAOD(KLON,KLEV,KSW)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PMAC2SPSSA(KLON,KLEV,KSW)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PMAC2SPASY(KLON,KLEV,KSW)
!REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PMAC2SPXRE(KLON)

!-- from CMIP6 prescribed pre-industrial aerosol from TM5 climatology
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PAODPI(KLON,KLEV,KSW)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PSSAPI(KLON,KLEV,KSW)
REAL(KIND=JPRB)   ,OPTIONAL, INTENT(IN)    :: PASFPI(KLON,KLEV,KSW)

!-- Output arguments

!-----------------------------------------------------------------------

!-- dummy integers

INTEGER(KIND=JPIM) :: ICLDATM, INFLAG, ICEFLAG, I_LIQFLAG, ITMOL, I_NSTR
INTEGER(KIND=JPIM) :: IK, IMOL, J1, J2, JAE, JL, JK, JSW
INTEGER(KIND=JPIM) :: JSW_ORDERED, JSW_ORDERED_BIS

!-- dummy reals

REAL(KIND=JPRB) :: ZPZ(KIDIA:KFDIA,0:JPLAY)   , ZPAVEL(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZTAVEL(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZCOLDRY(KIDIA:KFDIA,JPLAY) , ZCOLMOL(KIDIA:KFDIA,JPLAY) , ZWKL(KIDIA:KFDIA,35,JPLAY)
REAL(KIND=JPRB) :: ZCOLCH4(KIDIA:KFDIA,JPLAY) , ZCOLCO2(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZCOLH2O(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZCOLO2(KIDIA:KFDIA,JPLAY)  , ZCOLO3(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZFORFAC(KIDIA:KFDIA,JPLAY) , ZFORFRAC(KIDIA:KFDIA,JPLAY), ZSELFFAC(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZSELFFRAC(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZFAC00(KIDIA:KFDIA,JPLAY)  , ZFAC01(KIDIA:KFDIA,JPLAY)  , ZFAC10(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZFAC11(KIDIA:KFDIA,JPLAY)
REAL(KIND=JPRB) :: ZONEMINUS(KIDIA:KFDIA)    , ZRMU0(KIDIA:KFDIA) , ZADJI0
REAL(KIND=JPRB) :: ZALBD(KIDIA:KFDIA,KSW)    , ZALBP(KIDIA:KFDIA,KSW)    

REAL(KIND=JPRB) :: ZFRCL(KIDIA:KFDIA,KCOLS,JPLAY), ZTAUC(KIDIA:KFDIA,JPLAY,KCOLS), ZASYC(KIDIA:KFDIA,JPLAY,KCOLS)
REAL(KIND=JPRB) :: ZOMGC(KIDIA:KFDIA,JPLAY,KCOLS)
REAL(KIND=JPRB) :: ZTAUA(KIDIA:KFDIA,JPLAY,KSW), ZASYA(KIDIA:KFDIA,JPLAY,KSW), ZOMGA(KIDIA:KFDIA,JPLAY,KSW)
REAL(KIND=JPRB) :: ZFUVF(KIDIA:KFDIA), ZFUVC(KIDIA:KFDIA), ZPARF(KIDIA:KFDIA), ZPARCF(KIDIA:KFDIA), ZSUDU(KIDIA:KFDIA)

REAL(KIND=JPRB) :: ZBBCD(KIDIA:KFDIA,JPLAY+1), ZBBCU(KIDIA:KFDIA,JPLAY+1), ZBBFD(KIDIA:KFDIA,JPLAY+1), ZBBFU(KIDIA:KFDIA,JPLAY+1)
REAL(KIND=JPRB) :: ZBBFDIR(KIDIA:KFDIA,JPLAY+1),ZBBCDIR(KIDIA:KFDIA,JPLAY+1)

INTEGER(KIND=JPIM) :: ILAYTROP(KIDIA:KFDIA)
INTEGER(KIND=JPIM) :: INDFOR(KIDIA:KFDIA,JPLAY), INDSELF(KIDIA:KFDIA,JPLAY)
INTEGER(KIND=JPIM) :: JP(KIDIA:KFDIA,JPLAY), JT(KIDIA:KFDIA,JPLAY), JT1(KIDIA:KFDIA,JPLAY)

REAL(KIND=JPRB) :: ZAMD                  ! Effective molecular weight of dry air (g/mol)
REAL(KIND=JPRB) :: ZAMW                  ! Molecular weight of water vapor (g/mol)
REAL(KIND=JPRB) :: ZAMCO2                ! Molecular weight of carbon dioxide (g/mol)
REAL(KIND=JPRB) :: ZAMO                  ! Molecular weight of ozone (g/mol)
REAL(KIND=JPRB) :: ZAMCH4                ! Molecular weight of methane (g/mol)
REAL(KIND=JPRB) :: ZAMN2O                ! Molecular weight of nitrous oxide (g/mol)
REAL(KIND=JPRB) :: ZAMC11                ! Molecular weight of CFC11 (g/mol) - CFCL3
REAL(KIND=JPRB) :: ZAMC12                ! Molecular weight of CFC12 (g/mol) - CF2CL2
REAL(KIND=JPRB) :: ZAVGDRO               ! Avogadro's number (molecules/mole)
REAL(KIND=JPRB) :: ZGRAVIT               ! Gravitational acceleration (cm/sec2)
REAL(KIND=JPRB) :: ZAMM(KIDIA:KFDIA)

REAL(KIND=JPRB) :: ZRAMW                  ! Molecular weight of water vapor (g/mol)
REAL(KIND=JPRB) :: ZRAMCO2                ! Molecular weight of carbon dioxide (g/mol)
REAL(KIND=JPRB) :: ZRAMO                  ! Molecular weight of ozone (g/mol)
REAL(KIND=JPRB) :: ZRAMCH4                ! Molecular weight of methane (g/mol)
REAL(KIND=JPRB) :: ZRAMN2O                ! Molecular weight of nitrous oxide (g/mol)

! Atomic weights for conversion from mass to volume mixing ratios; these
!  are the same values used in ECRT to assure accurate conversion to vmr
data ZAMD   /  28.970_JPRB    /
data ZAMW   /  18.0154_JPRB   /
data ZAMCO2 /  44.011_JPRB    /
data ZAMO   /  47.9982_JPRB   /
data ZAMCH4 /  16.043_JPRB    /
data ZAMN2O /  44.013_JPRB    /
data ZAMC11 / 137.3686_JPRB   /
data ZAMC12 / 120.9140_JPRB   /
data ZAVGDRO/ 6.02214E23_JPRB /
data ZGRAVIT/ 9.80665E02_JPRB /
data ZRAMW   /  0.05550_JPRB   /
data ZRAMCO2 /  0.02272_JPRB   /
data ZRAMO   /  0.02083_JPRB   /
data ZRAMCH4 /  0.06233_JPRB    /
data ZRAMN2O /  0.02272_JPRB    /


!REAL(KIND=JPRB) :: ZCLEAR, ZCLOUD, ZTOTCC
REAL(KIND=JPRB) :: ZEPSEC

INTEGER(KIND=JPIM) :: IOVLP, IC, ICOUNT, INDEX(KIDIA:KFDIA)
REAL(KIND=JPRB) :: ZHOOK_HANDLE


#include "srtm_setcoef.intfb.h"
#include "srtm_spcvrt_mcica.intfb.h"


!-----------------------------------------------------------------------
!-- calculate information needed ny the radiative transfer routine 

IF (LHOOK) CALL DR_HOOK('SRTM_SRTM_224GP_MCICA',0,ZHOOK_HANDLE)
ZEPSEC  = 1.E-06_JPRB
ZONEMINUS=1.0_JPRB -  ZEPSEC
ZADJI0 = RII0 / RI0_SRTM
!-- overlap: 1=max-ran, 2=maximum, 3=random   N.B.: irrelevant in McICA version
IOVLP=3

IC=0
DO JL = KIDIA, KFDIA
  IF (PRMU0(JL) > 0.0_JPRB) THEN
    IC=IC+1
    INDEX(IC)=JL
  ENDIF
ENDDO
ICOUNT=IC

ICLDATM  = 1
INFLAG   = 2
ICEFLAG  = 3
I_LIQFLAG= 1
ITMOL    = 7
I_NSTR   = 2

DO JSW=1,KCOLS
  DO JK=1,KLEV
    DO JL = KIDIA, KFDIA
      ZFRCL(JL,JSW,JK) = PFRCL(JL,JSW,JK)
      ZTAUC(JL,JK,JSW) = PTAUC(JL,JSW,JK)
      ZASYC(JL,JK,JSW) = PASYC(JL,JSW,JK)
      ZOMGC(JL,JK,JSW) = POMGC(JL,JSW,JK)
    ENDDO
  ENDDO
ENDDO

ZRMU0(KIDIA:KFDIA)=PRMU0(KIDIA:KFDIA)
PFUVF(KIDIA:KFDIA)=0._JPRB
PFUVC(KIDIA:KFDIA)=0._JPRB
PPARF(KIDIA:KFDIA)=0._JPRB
PPARCF(KIDIA:KFDIA)=0._JPRB

!- coefficients related to the cloud optical properties (original RRTM_SW)

!- coefficients for the temperature and pressure dependence of the 
! molecular absorption coefficients

DO J1=1,35
  DO J2=1,KLEV
    DO IC=1,ICOUNT
      JL=INDEX(IC)
      ZWKL(JL,J1,J2)=0.0_JPRB 
    ENDDO
  ENDDO
ENDDO

DO IC=1,ICOUNT
  JL=INDEX(IC)
  ZPZ(JL,0) = paph(JL,klev+1)*0.01_JPRB
ENDDO

!ZCLEAR=1.0_JPRB
!ZCLOUD=0.0_JPRB
!ZTOTCC=0.0_JPRB

DO JK = 1, KLEV
  DO IC=1,ICOUNT
    JL=INDEX(IC)
    ZPAVEL(JL,JK) = pap(JL,KLEV-JK+1) *0.01_JPRB
    ZTAVEL(JL,JK) = pt (JL,KLEV-JK+1)
    ZPZ(JL,JK)    = paph(JL,KLEV-JK+1) *0.01_JPRB
    ZWKL(JL,1,JK) = pq(JL,KLEV-JK+1)  *ZAMD*ZRAMW
    ZWKL(JL,2,JK) = PCO2(JL,KLEV-JK+1)*ZAMD*ZRAMCO2
    ZWKL(JL,3,JK) = pozn(JL,KLEV-JK+1)*ZAMD*ZRAMO
    ZWKL(JL,4,JK) = PN2O(JL,KLEV-JK+1)*ZAMD*ZRAMN2O
    ZWKL(JL,6,JK) = PCH4(JL,KLEV-JK+1)*ZAMD*ZRAMCH4
    ZWKL(JL,7,JK) = 0.20944_JPRB
    ZAMM(JL) = (1-ZWKL(JL,1,JK))*ZAMD + ZWKL(JL,1,JK)*ZAMW
    ZCOLDRY(JL,JK) = (ZPZ(JL,JK-1)-ZPZ(JL,JK))*1.E3_JPRB*ZAVGDRO/(ZGRAVIT*ZAMM(JL)*(1+ZWKL(JL,1,JK)))
  ENDDO
ENDDO

DO IMOL=1,ITMOL
  DO JK=1,KLEV
    DO IC=1,ICOUNT
      JL=INDEX(IC)
      ZWKL(JL,IMOL,JK)=ZCOLDRY(JL,JK)* ZWKL(JL,IMOL,JK)
    ENDDO
  ENDDO
ENDDO

CALL SRTM_SETCOEF &
 & ( KIDIA , KFDIA    , KLEV,&
 & ZPAVEL  , ZTAVEL,&
 & ZCOLDRY , ZWKL,&
 & ILAYTROP,&
 & ZCOLCH4  , ZCOLCO2 , ZCOLH2O , ZCOLMOL  , ZCOLO2 , ZCOLO3,&
 & ZFORFAC , ZFORFRAC , INDFOR  , ZSELFFAC, ZSELFFRAC, INDSELF, &
 & ZFAC00  , ZFAC01   , ZFAC10  , ZFAC11,&
 & JP      , JT       , JT1     , ZRMU0  &
 & )  

!- call the radiation transfer routine

DO JSW=1,KSW
  DO IC=1,ICOUNT
    JL=INDEX(IC)
    ZALBD(JL,JSW)=PALBD(JL,JSW)
    ZALBP(JL,JSW)=PALBP(JL,JSW)
  ENDDO
ENDDO

!- mixing of aerosols

IF (NAER == 0) THEN
  DO JSW=1,KSW
    DO JK=1,KLEV
      DO IC=1,ICOUNT
        JL=INDEX(IC)
        ZTAUA(JL,JK,JSW)= 0.0_JPRB
        ZASYA(JL,JK,JSW)= 0.0_JPRB
        ZOMGA(JL,JK,JSW)= 1.0_JPRB
      ENDDO
    ENDDO
  ENDDO
ELSE
  DO JSW=1,KSW
    DO JK=1,KLEV
      DO IC=1,ICOUNT
        JL=INDEX(IC)
        IK=KLEV+1-JK
        ! IFS wavelength intervals 1..14 as defined in SUSRTAER correspond to 13..1,14 in TM5
        JSW_ORDERED=KSW-JSW 
        IF (JSW==14) THEN 
          JSW_ORDERED=14
        ENDIF
        
        ! IFS wavelength intervals 1..14 as defined in SUSRTAER correspond to 2..14,1 in the CMIP6 forcing
        JSW_ORDERED_BIS=JSW+1 
        IF (JSW==14) THEN 
          JSW_ORDERED_BIS=1
        ENDIF
        
        IF (.NOT.LTM5AER) THEN
          ZTAUA(JL,JK,JSW)=0.0_JPRB
          ZASYA(JL,JK,JSW)=0.0_JPRB
          ZOMGA(JL,JK,JSW)=0.0_JPRB
 
          IF (LCMIP6_PI_AEROSOLS) THEN ! prescribed pre-industrial TM5 clim
             
            ! -- previous option
            ! ZTAUA(JL,JK,JSW)=PAODPI(JL,IK,JSW_ORDERED)
            ! ZOMGA(JL,JK,JSW)=PAODPI(JL,IK,JSW_ORDERED)*PSSAPI(JL,IK,JSW_ORDERED)
            ! ZASYA(JL,JK,JSW)=PAODPI(JL,IK,JSW_ORDERED) &
            !  & *PSSAPI(JL,IK,JSW_ORDERED)*PASFPI(JL,IK,JSW_ORDERED)

            ! Adjusted according to what is read in the  pre-industrial TM5 clim file
            !---CMIP6 preindustrial aerosols
            !   extensive variable ABS = AAOD is provided for preindustrial aerosols
            !   single-scattering albedo SSA = 1-ABS/AOD 
            !   for combining contributions of different aerosol populations, the quantity of interest
            !   is SSA*AOD = AOD-ABS   
            ZTAUA(JL,JK,JSW)=PAODPI(JL,IK,JSW_ORDERED)
            ZOMGA(JL,JK,JSW)=PAODPI(JL,IK,JSW_ORDERED)-PSSAPI(JL,IK,JSW_ORDERED)
            ZASYA(JL,JK,JSW)=PASFPI(JL,IK,JSW_ORDERED)

            ! Eventually Add contributions from stratospheric aerosols
            IF (LCMIP6_STRATAER_FULL) THEN
              ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+PAODSTRAT(JL,IK,JSW_ORDERED_BIS)
              ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+PAODSTRAT(JL,IK,JSW_ORDERED_BIS)-PAAODSTRAT(JL,IK,JSW_ORDERED_BIS)
              ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+PREFAODSTRAT(JL,IK,JSW_ORDERED_BIS)
              ! ZOMGA is the diffusion, not the absorption
             ELSE
              DO JAE=5,6
                ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK)
                ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK) &
                   & *RSRPIZA(JSW,JAE)
                ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK) &
                   & *RSRPIZA(JSW,JAE)*RSRASYA(JSW,JAE)
              ENDDO
            ENDIF
           
            ! Eventually Add anthropogenic aerosols from MAC2SP
            IF (LMAC2SP) THEN
              ! Here MAC2SP OP are added on top of the TM5 but other configurations could be also possible 
              ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)
              ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)*PMAC2SPSSA(JL,IK,JSW)  
              ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)*PMAC2SPSSA(JL,IK,JSW)*PMAC2SPASY(JL,IK,JSW) 
            ENDIF

            IF (ZOMGA(JL,JK,JSW) /= 0.0_JPRB) THEN
              ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)/ZOMGA(JL,JK,JSW)
            ENDIF
            IF (ZTAUA(JL,JK,JSW) /= 0.0_JPRB) THEN
              ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)/ZTAUA(JL,JK,JSW)
            ENDIF

          ELSE !  default aerosol climatology (Tegen or CMIP5)
!CDIR UNROLL=6
            DO JAE=1,6
              ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK)
              ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK) &
                   & *RSRPIZA(JSW,JAE)  
              ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK) &
                   & *RSRPIZA(JSW,JAE)*RSRASYA(JSW,JAE) 
            ENDDO
             
            ! -- other options
            !IF (LMAC2SP) THEN
            ! ! Here MAC2SP OP replace the urban/black carbon part of the default aerosol climatology (Tegen or CMIP5) but other configurations could be also possible 
            ! ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)-(RSRTAUA(JSW,4)*PAER(JL,4,IK))+PMAC2SPAOD(JL,IK,JSW)
            ! ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)-(RSRTAUA(JSW,4)*PAER(JL,4,IK)*RSRPIZA(JSW,4)) &
            !  & +PMAC2SPAOD(JL,IK,JSW)* PMAC2SPSSA(JL,IK,JSW)
            ! ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)-(RSRTAUA(JSW,4)*PAER(JL,4,IK) &
            !  & *RSRPIZA(JSW,4)*RSRASYA(JSW,4))+PMAC2SPAOD(JL,IK,JSW)* PMAC2SPSSA(JL,IK,JSW)*PMAC2SPASY(JL,IK,JSW) 
            !ENDIF
            !IF (LMAC2SP) THEN
            ! ! Here MAC2SP OP is on top of the default aerosol climatology (Tegen or CMIP5) but other configurations could be also possible 
            ! ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)
            ! ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)* PMAC2SPSSA(JL,IK,JSW)
            ! ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)* PMAC2SPSSA(JL,IK,JSW)*PMAC2SPASY(JL,IK,JSW) 
            !ENDIF

            IF (ZOMGA(JL,JK,JSW) /= 0.0_JPRB) THEN
              ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)/ZOMGA(JL,JK,JSW)
            ENDIF
            IF (ZTAUA(JL,JK,JSW) /= 0.0_JPRB) THEN
              ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)/ZTAUA(JL,JK,JSW)
            ENDIF

          ENDIF !end Tegen or prescribed pre-industrial TM5_clim 
        ELSE 
          ! Use aerosol optical property fields from TM5

          ZTAUA(JL,JK,JSW)=PAODTM(JL,JSW_ORDERED,IK)
          ZOMGA(JL,JK,JSW)=PAODTM(JL,JSW_ORDERED,IK)*PSSATM(JL,JSW_ORDERED,IK)
          ZASYA(JL,JK,JSW)=PAODTM(JL,JSW_ORDERED,IK) &
                        & *PSSATM(JL,JSW_ORDERED,IK)*PASFTM(JL,JSW_ORDERED,IK)
          ! Add contributions from stratospheric aerosols

          IF (LCMIP6_STRATAER_FULL) THEN
              ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+PAODSTRAT(JL,IK,JSW_ORDERED_BIS)
              ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+PAODSTRAT(JL,IK,JSW_ORDERED_BIS)-PAAODSTRAT(JL,IK,JSW_ORDERED_BIS)
              ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+PREFAODSTRAT(JL,IK,JSW_ORDERED_BIS)
             ELSE
              DO JAE=5,6
                ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK)
                ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK) &
                 & *RSRPIZA(JSW,JAE)
                ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+RSRTAUA(JSW,JAE)*PAER(JL,JAE,IK) &
                 & *RSRPIZA(JSW,JAE)*RSRASYA(JSW,JAE)
              ENDDO
          ENDIF

          ! Eventually Add anthropogenic aerosols from MAC2SP
          ! Normally TM5 simulates both the pre-industrial and anthropogenic parts
          ! and the contributions from MAC2SP shouldn't be added.

          !IF (LMAC2SP) THEN
          ! Here MAC2SP OP are added on top of the TM5 but other configurations could be also possible 
          ! ZTAUA(JL,JK,JSW)=ZTAUA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)
          ! ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)*PMAC2SPSSA(JL,IK,JSW)  
          ! ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)+PMAC2SPAOD(JL,IK,JSW)*PMAC2SPSSA(JL,IK,JSW)*PMAC2SPASY(JL,IK,JSW) 
          !ENDIF
 
          IF (ZOMGA(JL,JK,JSW) /= 0.0_JPRB) THEN
            ZASYA(JL,JK,JSW)=ZASYA(JL,JK,JSW)/ZOMGA(JL,JK,JSW)
          ENDIF
          IF (ZTAUA(JL,JK,JSW) /= 0.0_JPRB) THEN
            ZOMGA(JL,JK,JSW)=ZOMGA(JL,JK,JSW)/ZTAUA(JL,JK,JSW)
          ENDIF
        ENDIF
        
      ENDDO
    ENDDO
  ENDDO
ENDIF

DO JK=1,KLEV+1
  DO IC=1,ICOUNT
    JL=INDEX(IC)
    ZBBCU(JL,JK)=0.0_JPRB
    ZBBCD(JL,JK)=0.0_JPRB
    ZBBFU(JL,JK)=0.0_JPRB
    ZBBFD(JL,JK)=0.0_JPRB
    ZBBFDIR(JL,JK)=0.0_JPRB
    ZBBCDIR(JL,JK)=0.0_JPRB
  ENDDO
ENDDO

DO IC=1,ICOUNT
  JL=INDEX(IC)
  ZFUVF(JL)=0.0_JPRB
  ZFUVC(JL)=0.0_JPRB
  ZPARF(JL)=0.0_JPRB
  ZPARCF(JL)=0.0_JPRB     
  ZSUDU(JL)=0.0_JPRB
ENDDO

CALL SRTM_SPCVRT_MCICA &
 &( KIDIA  , KFDIA    , KLEV   , KSW    , KCOLS  , ZONEMINUS,&
 & ZALBD   , ZALBP,&
 & ZFRCL   , ZTAUC    , ZASYC  , ZOMGC  ,&
 & ZTAUA   , ZASYA   , ZOMGA , ZRMU0,&
 & ILAYTROP,&
 & ZCOLCH4 , ZCOLCO2  , ZCOLH2O, ZCOLMOL , ZCOLO2   , ZCOLO3,&
 & ZFORFAC , ZFORFRAC , INDFOR , ZSELFFAC, ZSELFFRAC, INDSELF,&
 & ZFAC00  , ZFAC01   , ZFAC10 , ZFAC11  ,&
 & JP      , JT       , JT1    ,&
 & ZBBFD   , ZBBFU    , ZBBCD  , ZBBCU , ZFUVF , ZFUVC, ZPARF, ZPARCF, ZSUDU,& 
 & ZBBFDIR , ZBBCDIR  )

DO JK=1,KLEV+1
  DO IC=1,ICOUNT
    JL=INDEX(IC)
    PFSUC(JL,1,JK)=ZADJI0 * ZBBCU(JL,JK)
    PFSUC(JL,2,JK)=ZADJI0 * ZBBCD(JL,JK)
    PFSUX(JL,1,JK)=ZADJI0 * ZBBFU(JL,JK)
    PFSUX(JL,2,JK)=ZADJI0 * ZBBFD(JL,JK)
    PFDIR(JL,JK)  =ZADJI0 * ZBBFDIR(JL,JK)
    PCDIR(JL,JK)  =ZADJI0 * ZBBCDIR(JL,JK)
    PFDIF(JL,JK)  =PFSUX(JL,2,JK)-PFDIR(JL,JK)
    PCDIF(JL,JK)  =PFSUC(JL,2,JK)-PCDIR(JL,JK)
  ENDDO
ENDDO
DO IC=1,ICOUNT
  JL=INDEX(IC)
  PFUVF(JL) =ZADJI0 * ZFUVF(JL)
  PFUVC(JL) =ZADJI0 * ZFUVC(JL)
  PPARF(JL) =ZADJI0 * ZPARF(JL)
  PPARCF(JL)=ZADJI0 * ZPARCF(JL)
  PSUDU(JL) =ZADJI0 * ZSUDU(JL)
ENDDO

DO JK=1,KLEV+1
  DO IC=1,ICOUNT
    JL=INDEX(IC)
    IF (PRMU0(JL) <= 0.0_JPRB) THEN
      PFSUC(JL,1,JK)=0.0_JPRB
      PFSUC(JL,2,JK)=0.0_JPRB
      PFSUX(JL,1,JK)=0.0_JPRB
      PFSUX(JL,2,JK)=0.0_JPRB
      PFDIR(JL,JK)  =0.0_JPRB
      PCDIR(JL,JK)  =0.0_JPRB
    ENDIF
  ENDDO
ENDDO
DO IC=1,ICOUNT
  JL=INDEX(IC)
  IF (PRMU0(JL) <= 0.0_JPRB) THEN
    PFUVF(JL) =0.0_JPRB
    PFUVC(JL) =0.0_JPRB
    PPARF(JL) =0.0_JPRB
    PPARCF(JL)=0.0_JPRB
    PSUDU(JL)=0.0_JPRB
  ENDIF
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SRTM_SRTM_224GP_MCICA',1,ZHOOK_HANDLE)
END SUBROUTINE SRTM_SRTM_224GP_MCICA
