#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SUECOZCAQUA ( KINDAT, KMINUT )

!**** *SUECOZCAQUA* - DEFINES A ZONALLY SYMMETRIC LAT-HEIGHT OZONE DISTRIBUTION
!                 SYMMETRIZED ABOUT THE EQUATOR, CORRESPONDING TO THE ANNUAL MEAN
!                 CLIMATOLOGY USED IN AMIP II.

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SUECOZCAQUA* FROM *UPDTIM*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===
! ROZT    :                : AMOUNT OF OZONE (KG/KG) 

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     Nils Wedi 08/01/08 following modifications by J.-J. MORCRETTE  E.C.M.W.F.

!     MODIFICATIONS.
!     --------------
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RPI
USE YOERDI   , ONLY : RO3
USE YOEOZOC  , ONLY : RSINC    ,ROZT     ,RPROC
USE YOERAD   , ONLY : RPERTOZ  ,NPERTOZ

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KINDAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMINUT 
!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB),SAVE :: ZOZCL(64,59), ZLAT(64)
REAL(KIND=JPRB),SAVE :: ZYTIME(12), ZMDAY(12)

INTEGER(KIND=JPIM) :: IDY, IM, IM1, IM2, IMN, JK, JL

REAL(KIND=JPRB) :: ZTIMI, ZXTIME, ZINCH, ZINCL, ZINCROZ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.
#include "abor1.intfb.h"

#include "fcttim.h"      
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECOZCAQUA',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN
ZMDAY = (/&
 & 31._JPRB,    59.25_JPRB,  90.25_JPRB, 120.25_JPRB, 151.25_JPRB, 181.25_JPRB &
 & ,  212.25_JPRB, 243.25_JPRB, 273.25_JPRB, 304.25_JPRB, 334.25_JPRB, 365.25_JPRB &
 & /)  
ZYTIME= (/&
 & 22320._JPRB,  64980._JPRB, 107640._JPRB, 151560._JPRB, 195480._JPRB, 239400._JPRB &
 & , 283320._JPRB, 327960._JPRB, 371880._JPRB, 415800._JPRB, 459720._JPRB, 503640._JPRB &
 & /)

ZLAT = (/ -8.78638E+01_JPRB, -8.50965E+01_JPRB, -8.23129E+01_JPRB, -7.95256E+01_JPRB, -7.67369E+01_JPRB &
    &,-7.39475E+01_JPRB, -7.11577E+01_JPRB, -6.83678E+01_JPRB, -6.55776E+01_JPRB, -6.27873E+01_JPRB &
    &,-5.99970E+01_JPRB, -5.72066E+01_JPRB, -5.44162E+01_JPRB, -5.16257E+01_JPRB, -4.88352E+01_JPRB &
    &,-4.60447E+01_JPRB, -4.32542E+01_JPRB, -4.04636E+01_JPRB, -3.76731E+01_JPRB, -3.48825E+01_JPRB &
    &,-3.20919E+01_JPRB, -2.93014E+01_JPRB, -2.65108E+01_JPRB, -2.37202E+01_JPRB, -2.09296E+01_JPRB &
    &,-1.81390E+01_JPRB, -1.53484E+01_JPRB, -1.25578E+01_JPRB, -9.76710E+00_JPRB, -6.97650E+00_JPRB &
    &,-4.18590E+00_JPRB, -1.39530E+00_JPRB,  1.39530E+00_JPRB,  4.18590E+00_JPRB,  6.97650E+00_JPRB &
    &, 9.76710E+00_JPRB,  1.25578E+01_JPRB,  1.53484E+01_JPRB,  1.81390E+01_JPRB,  2.09296E+01_JPRB &
    &, 2.37202E+01_JPRB,  2.65108E+01_JPRB,  2.93014E+01_JPRB,  3.20919E+01_JPRB,  3.48825E+01_JPRB &
    &, 3.76731E+01_JPRB,  4.04636E+01_JPRB,  4.32542E+01_JPRB,  4.60447E+01_JPRB,  4.88352E+01_JPRB &
    &, 5.16257E+01_JPRB,  5.44162E+01_JPRB,  5.72066E+01_JPRB,  5.99970E+01_JPRB,  6.27873E+01_JPRB &
    &, 6.55776E+01_JPRB,  6.83678E+01_JPRB,  7.11577E+01_JPRB,  7.39475E+01_JPRB,  7.67369E+01_JPRB &
    &, 7.95256E+01_JPRB,  8.23129E+01_JPRB,  8.50965E+01_JPRB,  8.78638E+01_JPRB /)

!* Ozone climatological mixing ratio (ppmv)
!        from Langematz and Fortuin 

! Month:  Year average
ZOZCL(:, 1) = (/&
 &1.31668E+00_JPRB, 1.30596E+00_JPRB, 1.29668E+00_JPRB, 1.29177E+00_JPRB, 1.28737E+00_JPRB, &
 &1.30641E+00_JPRB, 1.33538E+00_JPRB, 1.35974E+00_JPRB, 1.38974E+00_JPRB, 1.41538E+00_JPRB, &
 &1.43075E+00_JPRB, 1.44026E+00_JPRB, 1.46139E+00_JPRB, 1.47967E+00_JPRB, 1.50586E+00_JPRB, &
 &1.52349E+00_JPRB, 1.53176E+00_JPRB, 1.55057E+00_JPRB, 1.58628E+00_JPRB, 1.59813E+00_JPRB, &
 &1.60358E+00_JPRB, 1.60978E+00_JPRB, 1.60622E+00_JPRB, 1.59170E+00_JPRB, 1.57947E+00_JPRB, &
 &1.56989E+00_JPRB, 1.55544E+00_JPRB, 1.53811E+00_JPRB, 1.53066E+00_JPRB, 1.52499E+00_JPRB, &
 &1.51674E+00_JPRB, 1.51308E+00_JPRB, 1.51308E+00_JPRB, 1.51674E+00_JPRB, 1.52499E+00_JPRB, &
 &1.53066E+00_JPRB, 1.53811E+00_JPRB, 1.55544E+00_JPRB, 1.56989E+00_JPRB, 1.57947E+00_JPRB, &
 &1.59170E+00_JPRB, 1.60622E+00_JPRB, 1.60978E+00_JPRB, 1.60358E+00_JPRB, 1.59813E+00_JPRB, &
 &1.58628E+00_JPRB, 1.55057E+00_JPRB, 1.53176E+00_JPRB, 1.52349E+00_JPRB, 1.50586E+00_JPRB, &
 &1.47967E+00_JPRB, 1.46139E+00_JPRB, 1.44026E+00_JPRB, 1.43075E+00_JPRB, 1.41538E+00_JPRB, &
 &1.38974E+00_JPRB, 1.35974E+00_JPRB, 1.33538E+00_JPRB, 1.30641E+00_JPRB, 1.28737E+00_JPRB, &
 &1.29177E+00_JPRB, 1.29668E+00_JPRB, 1.30596E+00_JPRB, 1.31668E+00_JPRB /)   
ZOZCL(:, 2) = (/&
 &1.45025E+00_JPRB, 1.44087E+00_JPRB, 1.43303E+00_JPRB, 1.42959E+00_JPRB, 1.42488E+00_JPRB, &
 &1.44540E+00_JPRB, 1.47710E+00_JPRB, 1.50423E+00_JPRB, 1.53788E+00_JPRB, 1.56596E+00_JPRB, &
 &1.58591E+00_JPRB, 1.59658E+00_JPRB, 1.61652E+00_JPRB, 1.63325E+00_JPRB, 1.65872E+00_JPRB, &
 &1.67512E+00_JPRB, 1.68092E+00_JPRB, 1.69656E+00_JPRB, 1.72982E+00_JPRB, 1.74287E+00_JPRB, &
 &1.74941E+00_JPRB, 1.75808E+00_JPRB, 1.75535E+00_JPRB, 1.73927E+00_JPRB, 1.72471E+00_JPRB, &
 &1.71235E+00_JPRB, 1.69413E+00_JPRB, 1.67266E+00_JPRB, 1.66154E+00_JPRB, 1.65297E+00_JPRB, &
 &1.64427E+00_JPRB, 1.64041E+00_JPRB, 1.64041E+00_JPRB, 1.64427E+00_JPRB, 1.65297E+00_JPRB, &
 &1.66154E+00_JPRB, 1.67266E+00_JPRB, 1.69413E+00_JPRB, 1.71235E+00_JPRB, 1.72471E+00_JPRB, &
 &1.73927E+00_JPRB, 1.75535E+00_JPRB, 1.75808E+00_JPRB, 1.74941E+00_JPRB, 1.74287E+00_JPRB, &
 &1.72982E+00_JPRB, 1.69656E+00_JPRB, 1.68092E+00_JPRB, 1.67512E+00_JPRB, 1.65872E+00_JPRB, &
 &1.63325E+00_JPRB, 1.61652E+00_JPRB, 1.59658E+00_JPRB, 1.58591E+00_JPRB, 1.56596E+00_JPRB, &
 &1.53788E+00_JPRB, 1.50423E+00_JPRB, 1.47710E+00_JPRB, 1.44540E+00_JPRB, 1.42488E+00_JPRB, &
 &1.42959E+00_JPRB, 1.43303E+00_JPRB, 1.44087E+00_JPRB, 1.45025E+00_JPRB /)   
ZOZCL(:, 3) = (/&
 &1.59199E+00_JPRB, 1.58210E+00_JPRB, 1.57378E+00_JPRB, 1.56975E+00_JPRB, 1.56279E+00_JPRB, &
 &1.58424E+00_JPRB, 1.61825E+00_JPRB, 1.64742E+00_JPRB, 1.68389E+00_JPRB, 1.71494E+00_JPRB, &
 &1.74039E+00_JPRB, 1.75448E+00_JPRB, 1.77472E+00_JPRB, 1.79032E+00_JPRB, 1.81243E+00_JPRB, &
 &1.82584E+00_JPRB, 1.82916E+00_JPRB, 1.84333E+00_JPRB, 1.87660E+00_JPRB, 1.89029E+00_JPRB, &
 &1.89722E+00_JPRB, 1.90757E+00_JPRB, 1.90540E+00_JPRB, 1.88846E+00_JPRB, 1.87143E+00_JPRB, &
 &1.85548E+00_JPRB, 1.83412E+00_JPRB, 1.80957E+00_JPRB, 1.79536E+00_JPRB, 1.78418E+00_JPRB, &
 &1.77389E+00_JPRB, 1.76916E+00_JPRB, 1.76916E+00_JPRB, 1.77389E+00_JPRB, 1.78418E+00_JPRB, &
 &1.79536E+00_JPRB, 1.80957E+00_JPRB, 1.83412E+00_JPRB, 1.85548E+00_JPRB, 1.87143E+00_JPRB, &
 &1.88846E+00_JPRB, 1.90540E+00_JPRB, 1.90757E+00_JPRB, 1.89722E+00_JPRB, 1.89029E+00_JPRB, &
 &1.87660E+00_JPRB, 1.84333E+00_JPRB, 1.82916E+00_JPRB, 1.82584E+00_JPRB, 1.81243E+00_JPRB, &
 &1.79032E+00_JPRB, 1.77472E+00_JPRB, 1.75448E+00_JPRB, 1.74039E+00_JPRB, 1.71494E+00_JPRB, &
 &1.68389E+00_JPRB, 1.64742E+00_JPRB, 1.61825E+00_JPRB, 1.58424E+00_JPRB, 1.56279E+00_JPRB, &
 &1.56975E+00_JPRB, 1.57378E+00_JPRB, 1.58210E+00_JPRB, 1.59199E+00_JPRB /)   
ZOZCL(:, 4) = (/&
 &1.73873E+00_JPRB, 1.72675E+00_JPRB, 1.71635E+00_JPRB, 1.70989E+00_JPRB, 1.69837E+00_JPRB, &
 &1.72009E+00_JPRB, 1.75579E+00_JPRB, 1.78717E+00_JPRB, 1.82816E+00_JPRB, 1.86445E+00_JPRB, &
 &1.89603E+00_JPRB, 1.91466E+00_JPRB, 1.93718E+00_JPRB, 1.95305E+00_JPRB, 1.97166E+00_JPRB, &
 &1.98130E+00_JPRB, 1.98136E+00_JPRB, 1.99426E+00_JPRB, 2.02815E+00_JPRB, 2.04175E+00_JPRB, &
 &2.04729E+00_JPRB, 2.05206E+00_JPRB, 2.04552E+00_JPRB, 2.02710E+00_JPRB, 2.00810E+00_JPRB, &
 &1.98990E+00_JPRB, 1.96692E+00_JPRB, 1.94081E+00_JPRB, 1.92289E+00_JPRB, 1.90873E+00_JPRB, &
 &1.89778E+00_JPRB, 1.89273E+00_JPRB, 1.89273E+00_JPRB, 1.89778E+00_JPRB, 1.90873E+00_JPRB, &
 &1.92289E+00_JPRB, 1.94081E+00_JPRB, 1.96692E+00_JPRB, 1.98990E+00_JPRB, 2.00810E+00_JPRB, &
 &2.02710E+00_JPRB, 2.04552E+00_JPRB, 2.05206E+00_JPRB, 2.04729E+00_JPRB, 2.04175E+00_JPRB, &
 &2.02815E+00_JPRB, 1.99426E+00_JPRB, 1.98136E+00_JPRB, 1.98130E+00_JPRB, 1.97166E+00_JPRB, &
 &1.95305E+00_JPRB, 1.93718E+00_JPRB, 1.91466E+00_JPRB, 1.89603E+00_JPRB, 1.86445E+00_JPRB, &
 &1.82816E+00_JPRB, 1.78717E+00_JPRB, 1.75579E+00_JPRB, 1.72009E+00_JPRB, 1.69837E+00_JPRB, &
 &1.70989E+00_JPRB, 1.71635E+00_JPRB, 1.72675E+00_JPRB, 1.73873E+00_JPRB /)   
ZOZCL(:, 5) = (/&
 &1.89371E+00_JPRB, 1.87942E+00_JPRB, 1.86668E+00_JPRB, 1.85849E+00_JPRB, 1.84676E+00_JPRB, &
 &1.87006E+00_JPRB, 1.90897E+00_JPRB, 1.94277E+00_JPRB, 1.98587E+00_JPRB, 2.02391E+00_JPRB, &
 &2.05916E+00_JPRB, 2.08152E+00_JPRB, 2.10579E+00_JPRB, 2.12175E+00_JPRB, 2.13772E+00_JPRB, &
 &2.14408E+00_JPRB, 2.14073E+00_JPRB, 2.15151E+00_JPRB, 2.18491E+00_JPRB, 2.19889E+00_JPRB, &
 &2.20341E+00_JPRB, 2.20220E+00_JPRB, 2.19166E+00_JPRB, 2.17348E+00_JPRB, 2.15386E+00_JPRB, &
 &2.13448E+00_JPRB, 2.11041E+00_JPRB, 2.08310E+00_JPRB, 2.06324E+00_JPRB, 2.04748E+00_JPRB, &
 &2.03523E+00_JPRB, 2.02950E+00_JPRB, 2.02950E+00_JPRB, 2.03523E+00_JPRB, 2.04748E+00_JPRB, &
 &2.06324E+00_JPRB, 2.08310E+00_JPRB, 2.11041E+00_JPRB, 2.13448E+00_JPRB, 2.15386E+00_JPRB, &
 &2.17348E+00_JPRB, 2.19166E+00_JPRB, 2.20220E+00_JPRB, 2.20341E+00_JPRB, 2.19889E+00_JPRB, &
 &2.18491E+00_JPRB, 2.15151E+00_JPRB, 2.14073E+00_JPRB, 2.14408E+00_JPRB, 2.13772E+00_JPRB, &
 &2.12175E+00_JPRB, 2.10579E+00_JPRB, 2.08152E+00_JPRB, 2.05916E+00_JPRB, 2.02391E+00_JPRB, &
 &1.98587E+00_JPRB, 1.94277E+00_JPRB, 1.90897E+00_JPRB, 1.87006E+00_JPRB, 1.84676E+00_JPRB, &
 &1.85849E+00_JPRB, 1.86668E+00_JPRB, 1.87942E+00_JPRB, 1.89371E+00_JPRB /)   
ZOZCL(:, 6) = (/&
 &2.05308E+00_JPRB, 2.03824E+00_JPRB, 2.02494E+00_JPRB, 2.01650E+00_JPRB, 2.00464E+00_JPRB, &
 &2.03026E+00_JPRB, 2.07332E+00_JPRB, 2.11021E+00_JPRB, 2.15556E+00_JPRB, 2.19540E+00_JPRB, &
 &2.23411E+00_JPRB, 2.25832E+00_JPRB, 2.28300E+00_JPRB, 2.29856E+00_JPRB, 2.31374E+00_JPRB, &
 &2.31866E+00_JPRB, 2.31308E+00_JPRB, 2.32194E+00_JPRB, 2.35482E+00_JPRB, 2.36999E+00_JPRB, &
 &2.37439E+00_JPRB, 2.36832E+00_JPRB, 2.35455E+00_JPRB, 2.33669E+00_JPRB, 2.31654E+00_JPRB, &
 &2.29605E+00_JPRB, 2.27080E+00_JPRB, 2.24210E+00_JPRB, 2.22140E+00_JPRB, 2.20507E+00_JPRB, &
 &2.19234E+00_JPRB, 2.18645E+00_JPRB, 2.18645E+00_JPRB, 2.19234E+00_JPRB, 2.20507E+00_JPRB, &
 &2.22140E+00_JPRB, 2.24210E+00_JPRB, 2.27080E+00_JPRB, 2.29605E+00_JPRB, 2.31654E+00_JPRB, &
 &2.33669E+00_JPRB, 2.35455E+00_JPRB, 2.36832E+00_JPRB, 2.37439E+00_JPRB, 2.36999E+00_JPRB, &
 &2.35482E+00_JPRB, 2.32194E+00_JPRB, 2.31308E+00_JPRB, 2.31866E+00_JPRB, 2.31374E+00_JPRB, &
 &2.29856E+00_JPRB, 2.28300E+00_JPRB, 2.25832E+00_JPRB, 2.23411E+00_JPRB, 2.19540E+00_JPRB, &
 &2.15556E+00_JPRB, 2.11021E+00_JPRB, 2.07332E+00_JPRB, 2.03026E+00_JPRB, 2.00464E+00_JPRB, &
 &2.01650E+00_JPRB, 2.02494E+00_JPRB, 2.03824E+00_JPRB, 2.05308E+00_JPRB /)   
ZOZCL(:, 7) = (/&
 &2.22090E+00_JPRB, 2.20708E+00_JPRB, 2.19482E+00_JPRB, 2.18711E+00_JPRB, 2.17381E+00_JPRB, &
 &2.20323E+00_JPRB, 2.25172E+00_JPRB, 2.29348E+00_JPRB, 2.34260E+00_JPRB, 2.38409E+00_JPRB, &
 &2.42750E+00_JPRB, 2.45297E+00_JPRB, 2.47672E+00_JPRB, 2.49132E+00_JPRB, 2.50714E+00_JPRB, &
 &2.51187E+00_JPRB, 2.50494E+00_JPRB, 2.51210E+00_JPRB, 2.54460E+00_JPRB, 2.56123E+00_JPRB, &
 &2.56552E+00_JPRB, 2.55347E+00_JPRB, 2.53546E+00_JPRB, 2.51723E+00_JPRB, 2.49600E+00_JPRB, &
 &2.47402E+00_JPRB, 2.44811E+00_JPRB, 2.41879E+00_JPRB, 2.39859E+00_JPRB, 2.38291E+00_JPRB, &
 &2.37038E+00_JPRB, 2.36475E+00_JPRB, 2.36475E+00_JPRB, 2.37038E+00_JPRB, 2.38291E+00_JPRB, &
 &2.39859E+00_JPRB, 2.41879E+00_JPRB, 2.44811E+00_JPRB, 2.47402E+00_JPRB, 2.49600E+00_JPRB, &
 &2.51723E+00_JPRB, 2.53546E+00_JPRB, 2.55347E+00_JPRB, 2.56552E+00_JPRB, 2.56123E+00_JPRB, &
 &2.54460E+00_JPRB, 2.51210E+00_JPRB, 2.50494E+00_JPRB, 2.51187E+00_JPRB, 2.50714E+00_JPRB, &
 &2.49132E+00_JPRB, 2.47672E+00_JPRB, 2.45297E+00_JPRB, 2.42750E+00_JPRB, 2.38409E+00_JPRB, &
 &2.34260E+00_JPRB, 2.29348E+00_JPRB, 2.25172E+00_JPRB, 2.20323E+00_JPRB, 2.17381E+00_JPRB, &
 &2.18711E+00_JPRB, 2.19482E+00_JPRB, 2.20708E+00_JPRB, 2.22090E+00_JPRB /)
ZOZCL(:, 8) = (/&
 &2.39747E+00_JPRB, 2.38735E+00_JPRB, 2.37877E+00_JPRB, 2.37460E+00_JPRB, 2.36180E+00_JPRB, &
 &2.39362E+00_JPRB, 2.44738E+00_JPRB, 2.49526E+00_JPRB, 2.54901E+00_JPRB, 2.59291E+00_JPRB, &
 &2.64127E+00_JPRB, 2.67064E+00_JPRB, 2.69749E+00_JPRB, 2.71370E+00_JPRB, 2.72947E+00_JPRB, &
 &2.73397E+00_JPRB, 2.72635E+00_JPRB, 2.73005E+00_JPRB, 2.75964E+00_JPRB, 2.77781E+00_JPRB, &
 &2.78240E+00_JPRB, 2.76603E+00_JPRB, 2.74519E+00_JPRB, 2.72770E+00_JPRB, 2.70539E+00_JPRB, &
 &2.68108E+00_JPRB, 2.65322E+00_JPRB, 2.62186E+00_JPRB, 2.60236E+00_JPRB, 2.58786E+00_JPRB, &
 &2.57689E+00_JPRB, 2.57240E+00_JPRB, 2.57240E+00_JPRB, 2.57689E+00_JPRB, 2.58786E+00_JPRB, &
 &2.60236E+00_JPRB, 2.62186E+00_JPRB, 2.65322E+00_JPRB, 2.68108E+00_JPRB, 2.70539E+00_JPRB, &
 &2.72770E+00_JPRB, 2.74519E+00_JPRB, 2.76603E+00_JPRB, 2.78240E+00_JPRB, 2.77781E+00_JPRB, &
 &2.75964E+00_JPRB, 2.73005E+00_JPRB, 2.72635E+00_JPRB, 2.73397E+00_JPRB, 2.72947E+00_JPRB, &
 &2.71370E+00_JPRB, 2.69749E+00_JPRB, 2.67064E+00_JPRB, 2.64127E+00_JPRB, 2.59291E+00_JPRB, &
 &2.54901E+00_JPRB, 2.49526E+00_JPRB, 2.44738E+00_JPRB, 2.39362E+00_JPRB, 2.36180E+00_JPRB, &
 &2.37460E+00_JPRB, 2.37877E+00_JPRB, 2.38735E+00_JPRB, 2.39747E+00_JPRB /)   
ZOZCL(:, 9) = (/&
 &2.59040E+00_JPRB, 2.58383E+00_JPRB, 2.57882E+00_JPRB, 2.57786E+00_JPRB, 2.56552E+00_JPRB, &
 &2.60426E+00_JPRB, 2.66668E+00_JPRB, 2.72224E+00_JPRB, 2.78077E+00_JPRB, 2.82653E+00_JPRB, &
 &2.87930E+00_JPRB, 2.90710E+00_JPRB, 2.92929E+00_JPRB, 2.94329E+00_JPRB, 2.96439E+00_JPRB, &
 &2.97066E+00_JPRB, 2.96132E+00_JPRB, 2.96297E+00_JPRB, 2.99224E+00_JPRB, 3.00835E+00_JPRB, &
 &3.00922E+00_JPRB, 2.98514E+00_JPRB, 2.95766E+00_JPRB, 2.93616E+00_JPRB, 2.91070E+00_JPRB, &
 &2.88408E+00_JPRB, 2.85935E+00_JPRB, 2.83233E+00_JPRB, 2.81441E+00_JPRB, 2.80173E+00_JPRB, &
 &2.79304E+00_JPRB, 2.79011E+00_JPRB, 2.79011E+00_JPRB, 2.79304E+00_JPRB, 2.80173E+00_JPRB, &
 &2.81441E+00_JPRB, 2.83233E+00_JPRB, 2.85935E+00_JPRB, 2.88408E+00_JPRB, 2.91070E+00_JPRB, &
 &2.93616E+00_JPRB, 2.95766E+00_JPRB, 2.98514E+00_JPRB, 3.00922E+00_JPRB, 3.00835E+00_JPRB, &
 &2.99224E+00_JPRB, 2.96297E+00_JPRB, 2.96132E+00_JPRB, 2.97066E+00_JPRB, 2.96439E+00_JPRB, &
 &2.94329E+00_JPRB, 2.92929E+00_JPRB, 2.90710E+00_JPRB, 2.87930E+00_JPRB, 2.82653E+00_JPRB, &
 &2.78077E+00_JPRB, 2.72224E+00_JPRB, 2.66668E+00_JPRB, 2.60426E+00_JPRB, 2.56552E+00_JPRB, &
 &2.57786E+00_JPRB, 2.57882E+00_JPRB, 2.58383E+00_JPRB, 2.59040E+00_JPRB /)   
ZOZCL(:,10) = (/&
 &2.79666E+00_JPRB, 2.79598E+00_JPRB, 2.79695E+00_JPRB, 2.80350E+00_JPRB, 2.79945E+00_JPRB, &
 &2.83957E+00_JPRB, 2.90828E+00_JPRB, 2.97269E+00_JPRB, 3.03613E+00_JPRB, 3.08233E+00_JPRB, &
 &3.14211E+00_JPRB, 3.17481E+00_JPRB, 3.20831E+00_JPRB, 3.22879E+00_JPRB, 3.24683E+00_JPRB, &
 &3.25035E+00_JPRB, 3.23874E+00_JPRB, 3.23857E+00_JPRB, 3.26815E+00_JPRB, 3.28020E+00_JPRB, &
 &3.27696E+00_JPRB, 3.25539E+00_JPRB, 3.22831E+00_JPRB, 3.20464E+00_JPRB, 3.17386E+00_JPRB, &
 &3.13971E+00_JPRB, 3.11771E+00_JPRB, 3.09619E+00_JPRB, 3.07544E+00_JPRB, 3.06161E+00_JPRB, &
 &3.05815E+00_JPRB, 3.05868E+00_JPRB, 3.05868E+00_JPRB, 3.05815E+00_JPRB, 3.06161E+00_JPRB, &
 &3.07544E+00_JPRB, 3.09619E+00_JPRB, 3.11771E+00_JPRB, 3.13971E+00_JPRB, 3.17386E+00_JPRB, &
 &3.20464E+00_JPRB, 3.22831E+00_JPRB, 3.25539E+00_JPRB, 3.27696E+00_JPRB, 3.28020E+00_JPRB, &
 &3.26815E+00_JPRB, 3.23857E+00_JPRB, 3.23874E+00_JPRB, 3.25035E+00_JPRB, 3.24683E+00_JPRB, &
 &3.22879E+00_JPRB, 3.20831E+00_JPRB, 3.17481E+00_JPRB, 3.14211E+00_JPRB, 3.08233E+00_JPRB, &
 &3.03613E+00_JPRB, 2.97269E+00_JPRB, 2.90828E+00_JPRB, 2.83957E+00_JPRB, 2.79945E+00_JPRB, &
 &2.80350E+00_JPRB, 2.79695E+00_JPRB, 2.79598E+00_JPRB, 2.79666E+00_JPRB /)   
ZOZCL(:,11) = (/&
 &3.02288E+00_JPRB, 3.03065E+00_JPRB, 3.04022E+00_JPRB, 3.05304E+00_JPRB, 3.04518E+00_JPRB, &
 &3.09716E+00_JPRB, 3.17864E+00_JPRB, 3.25211E+00_JPRB, 3.32398E+00_JPRB, 3.37774E+00_JPRB, &
 &3.44323E+00_JPRB, 3.47757E+00_JPRB, 3.51576E+00_JPRB, 3.54222E+00_JPRB, 3.57061E+00_JPRB, &
 &3.57270E+00_JPRB, 3.54968E+00_JPRB, 3.54293E+00_JPRB, 3.57001E+00_JPRB, 3.58286E+00_JPRB, &
 &3.57859E+00_JPRB, 3.55026E+00_JPRB, 3.51518E+00_JPRB, 3.48233E+00_JPRB, 3.44824E+00_JPRB, &
 &3.41531E+00_JPRB, 3.38936E+00_JPRB, 3.36211E+00_JPRB, 3.34698E+00_JPRB, 3.33700E+00_JPRB, &
 &3.32765E+00_JPRB, 3.32493E+00_JPRB, 3.32493E+00_JPRB, 3.32765E+00_JPRB, 3.33700E+00_JPRB, &
 &3.34698E+00_JPRB, 3.36211E+00_JPRB, 3.38936E+00_JPRB, 3.41531E+00_JPRB, 3.44824E+00_JPRB, &
 &3.48233E+00_JPRB, 3.51518E+00_JPRB, 3.55026E+00_JPRB, 3.57859E+00_JPRB, 3.58286E+00_JPRB, &
 &3.57001E+00_JPRB, 3.54293E+00_JPRB, 3.54968E+00_JPRB, 3.57270E+00_JPRB, 3.57061E+00_JPRB, &
 &3.54222E+00_JPRB, 3.51576E+00_JPRB, 3.47757E+00_JPRB, 3.44323E+00_JPRB, 3.37774E+00_JPRB, &
 &3.32398E+00_JPRB, 3.25211E+00_JPRB, 3.17864E+00_JPRB, 3.09716E+00_JPRB, 3.04518E+00_JPRB, &
 &3.05304E+00_JPRB, 3.04022E+00_JPRB, 3.03065E+00_JPRB, 3.02288E+00_JPRB /)   
ZOZCL(:,12) = (/&
 &3.26701E+00_JPRB, 3.28644E+00_JPRB, 3.30793E+00_JPRB, 3.33354E+00_JPRB, 3.33767E+00_JPRB, &
 &3.40303E+00_JPRB, 3.49753E+00_JPRB, 3.57803E+00_JPRB, 3.65918E+00_JPRB, 3.72309E+00_JPRB, &
 &3.79847E+00_JPRB, 3.83798E+00_JPRB, 3.87661E+00_JPRB, 3.90054E+00_JPRB, 3.92552E+00_JPRB, &
 &3.92688E+00_JPRB, 3.90262E+00_JPRB, 3.89158E+00_JPRB, 3.91592E+00_JPRB, 3.92034E+00_JPRB, &
 &3.90677E+00_JPRB, 3.87363E+00_JPRB, 3.83552E+00_JPRB, 3.80395E+00_JPRB, 3.76449E+00_JPRB, &
 &3.72127E+00_JPRB, 3.69052E+00_JPRB, 3.65984E+00_JPRB, 3.64925E+00_JPRB, 3.64284E+00_JPRB, &
 &3.63055E+00_JPRB, 3.62628E+00_JPRB, 3.62628E+00_JPRB, 3.63055E+00_JPRB, 3.64284E+00_JPRB, &
 &3.64925E+00_JPRB, 3.65984E+00_JPRB, 3.69052E+00_JPRB, 3.72127E+00_JPRB, 3.76449E+00_JPRB, &
 &3.80395E+00_JPRB, 3.83552E+00_JPRB, 3.87363E+00_JPRB, 3.90677E+00_JPRB, 3.92034E+00_JPRB, &
 &3.91592E+00_JPRB, 3.89158E+00_JPRB, 3.90262E+00_JPRB, 3.92688E+00_JPRB, 3.92552E+00_JPRB, &
 &3.90054E+00_JPRB, 3.87661E+00_JPRB, 3.83798E+00_JPRB, 3.79847E+00_JPRB, 3.72309E+00_JPRB, &
 &3.65918E+00_JPRB, 3.57803E+00_JPRB, 3.49753E+00_JPRB, 3.40303E+00_JPRB, 3.33767E+00_JPRB, &
 &3.33354E+00_JPRB, 3.30793E+00_JPRB, 3.28644E+00_JPRB, 3.26701E+00_JPRB /)   
ZOZCL(:,13) = (/&
 &3.53291E+00_JPRB, 3.56681E+00_JPRB, 3.60326E+00_JPRB, 3.64425E+00_JPRB, 3.65917E+00_JPRB, &
 &3.73690E+00_JPRB, 3.84607E+00_JPRB, 3.93966E+00_JPRB, 4.03311E+00_JPRB, 4.10622E+00_JPRB, &
 &4.19000E+00_JPRB, 4.23052E+00_JPRB, 4.27292E+00_JPRB, 4.29921E+00_JPRB, 4.32808E+00_JPRB, &
 &4.32791E+00_JPRB, 4.29475E+00_JPRB, 4.27706E+00_JPRB, 4.29849E+00_JPRB, 4.29521E+00_JPRB, &
 &4.27359E+00_JPRB, 4.23900E+00_JPRB, 4.19801E+00_JPRB, 4.16226E+00_JPRB, 4.12389E+00_JPRB, &
 &4.08601E+00_JPRB, 4.04411E+00_JPRB, 3.99725E+00_JPRB, 3.98422E+00_JPRB, 3.97505E+00_JPRB, &
 &3.95559E+00_JPRB, 3.94722E+00_JPRB, 3.94722E+00_JPRB, 3.95559E+00_JPRB, 3.97505E+00_JPRB, &
 &3.98422E+00_JPRB, 3.99725E+00_JPRB, 4.04411E+00_JPRB, 4.08601E+00_JPRB, 4.12389E+00_JPRB, &
 &4.16226E+00_JPRB, 4.19801E+00_JPRB, 4.23900E+00_JPRB, 4.27359E+00_JPRB, 4.29521E+00_JPRB, &
 &4.29849E+00_JPRB, 4.27706E+00_JPRB, 4.29475E+00_JPRB, 4.32791E+00_JPRB, 4.32808E+00_JPRB, &
 &4.29921E+00_JPRB, 4.27292E+00_JPRB, 4.23052E+00_JPRB, 4.19000E+00_JPRB, 4.10622E+00_JPRB, &
 &4.03311E+00_JPRB, 3.93966E+00_JPRB, 3.84607E+00_JPRB, 3.73690E+00_JPRB, 3.65917E+00_JPRB, &
 &3.64425E+00_JPRB, 3.60326E+00_JPRB, 3.56681E+00_JPRB, 3.53291E+00_JPRB /)   
ZOZCL(:,14) = (/&
 &3.80641E+00_JPRB, 3.84936E+00_JPRB, 3.89485E+00_JPRB, 3.94713E+00_JPRB, 3.97942E+00_JPRB, &
 &4.07415E+00_JPRB, 4.20091E+00_JPRB, 4.31014E+00_JPRB, 4.42206E+00_JPRB, 4.51187E+00_JPRB, &
 &4.60942E+00_JPRB, 4.66396E+00_JPRB, 4.71776E+00_JPRB, 4.75278E+00_JPRB, 4.78670E+00_JPRB, &
 &4.78478E+00_JPRB, 4.74608E+00_JPRB, 4.72398E+00_JPRB, 4.74479E+00_JPRB, 4.74117E+00_JPRB, &
 &4.71783E+00_JPRB, 4.68291E+00_JPRB, 4.63866E+00_JPRB, 4.59614E+00_JPRB, 4.54876E+00_JPRB, &
 &4.50042E+00_JPRB, 4.45438E+00_JPRB, 4.40497E+00_JPRB, 4.38101E+00_JPRB, 4.36396E+00_JPRB, &
 &4.34748E+00_JPRB, 4.34108E+00_JPRB, 4.34108E+00_JPRB, 4.34748E+00_JPRB, 4.36396E+00_JPRB, &
 &4.38101E+00_JPRB, 4.40497E+00_JPRB, 4.45438E+00_JPRB, 4.50042E+00_JPRB, 4.54876E+00_JPRB, &
 &4.59614E+00_JPRB, 4.63866E+00_JPRB, 4.68291E+00_JPRB, 4.71783E+00_JPRB, 4.74117E+00_JPRB, &
 &4.74479E+00_JPRB, 4.72398E+00_JPRB, 4.74608E+00_JPRB, 4.78478E+00_JPRB, 4.78670E+00_JPRB, &
 &4.75278E+00_JPRB, 4.71776E+00_JPRB, 4.66396E+00_JPRB, 4.60942E+00_JPRB, 4.51187E+00_JPRB, &
 &4.42206E+00_JPRB, 4.31014E+00_JPRB, 4.20091E+00_JPRB, 4.07415E+00_JPRB, 3.97942E+00_JPRB, &
 &3.94713E+00_JPRB, 3.89485E+00_JPRB, 3.84936E+00_JPRB, 3.80641E+00_JPRB /)   
ZOZCL(:,15) = (/&
 &4.05448E+00_JPRB, 4.11298E+00_JPRB, 4.17421E+00_JPRB, 4.24587E+00_JPRB, 4.30657E+00_JPRB, &
 &4.42108E+00_JPRB, 4.57234E+00_JPRB, 4.70747E+00_JPRB, 4.84043E+00_JPRB, 4.94629E+00_JPRB, &
 &5.05962E+00_JPRB, 5.12602E+00_JPRB, 5.18849E+00_JPRB, 5.23126E+00_JPRB, 5.27690E+00_JPRB, &
 &5.28098E+00_JPRB, 5.24008E+00_JPRB, 5.22288E+00_JPRB, 5.25513E+00_JPRB, 5.25148E+00_JPRB, &
 &5.22528E+00_JPRB, 5.18480E+00_JPRB, 5.13556E+00_JPRB, 5.09148E+00_JPRB, 5.03579E+00_JPRB, &
 &4.97437E+00_JPRB, 4.93259E+00_JPRB, 4.89189E+00_JPRB, 4.84660E+00_JPRB, 4.81244E+00_JPRB, &
 &4.79545E+00_JPRB, 4.78897E+00_JPRB, 4.78897E+00_JPRB, 4.79545E+00_JPRB, 4.81244E+00_JPRB, &
 &4.84660E+00_JPRB, 4.89189E+00_JPRB, 4.93259E+00_JPRB, 4.97437E+00_JPRB, 5.03579E+00_JPRB, &
 &5.09148E+00_JPRB, 5.13556E+00_JPRB, 5.18480E+00_JPRB, 5.22528E+00_JPRB, 5.25148E+00_JPRB, &
 &5.25513E+00_JPRB, 5.22288E+00_JPRB, 5.24008E+00_JPRB, 5.28098E+00_JPRB, 5.27690E+00_JPRB, &
 &5.23126E+00_JPRB, 5.18849E+00_JPRB, 5.12602E+00_JPRB, 5.05962E+00_JPRB, 4.94629E+00_JPRB, &
 &4.84043E+00_JPRB, 4.70747E+00_JPRB, 4.57234E+00_JPRB, 4.42108E+00_JPRB, 4.30657E+00_JPRB, &
 &4.24587E+00_JPRB, 4.17421E+00_JPRB, 4.11298E+00_JPRB, 4.05448E+00_JPRB /)   
ZOZCL(:,16) = (/&
 &4.26202E+00_JPRB, 4.34096E+00_JPRB, 4.42322E+00_JPRB, 4.52075E+00_JPRB, 4.61802E+00_JPRB, &
 &4.75326E+00_JPRB, 4.93114E+00_JPRB, 5.09486E+00_JPRB, 5.25071E+00_JPRB, 5.37588E+00_JPRB, &
 &5.50375E+00_JPRB, 5.58983E+00_JPRB, 5.66574E+00_JPRB, 5.72184E+00_JPRB, 5.78828E+00_JPRB, &
 &5.80911E+00_JPRB, 5.77506E+00_JPRB, 5.76080E+00_JPRB, 5.79864E+00_JPRB, 5.79871E+00_JPRB, &
 &5.77457E+00_JPRB, 5.73811E+00_JPRB, 5.68938E+00_JPRB, 5.64174E+00_JPRB, 5.58136E+00_JPRB, &
 &5.51469E+00_JPRB, 5.47356E+00_JPRB, 5.43465E+00_JPRB, 5.37123E+00_JPRB, 5.32407E+00_JPRB, &
 &5.31259E+00_JPRB, 5.30976E+00_JPRB, 5.30976E+00_JPRB, 5.31259E+00_JPRB, 5.32407E+00_JPRB, &
 &5.37123E+00_JPRB, 5.43465E+00_JPRB, 5.47356E+00_JPRB, 5.51469E+00_JPRB, 5.58136E+00_JPRB, &
 &5.64174E+00_JPRB, 5.68938E+00_JPRB, 5.73811E+00_JPRB, 5.77457E+00_JPRB, 5.79871E+00_JPRB, &
 &5.79864E+00_JPRB, 5.76080E+00_JPRB, 5.77506E+00_JPRB, 5.80911E+00_JPRB, 5.78828E+00_JPRB, &
 &5.72184E+00_JPRB, 5.66574E+00_JPRB, 5.58983E+00_JPRB, 5.50375E+00_JPRB, 5.37588E+00_JPRB, &
 &5.25071E+00_JPRB, 5.09486E+00_JPRB, 4.93114E+00_JPRB, 4.75326E+00_JPRB, 4.61802E+00_JPRB, &
 &4.52075E+00_JPRB, 4.42322E+00_JPRB, 4.34096E+00_JPRB, 4.26202E+00_JPRB /)   
ZOZCL(:,17) = (/&
 &4.44969E+00_JPRB, 4.54304E+00_JPRB, 4.64002E+00_JPRB, 4.75694E+00_JPRB, 4.88904E+00_JPRB, &
 &5.05484E+00_JPRB, 5.26356E+00_JPRB, 5.45512E+00_JPRB, 5.63335E+00_JPRB, 5.77665E+00_JPRB, &
 &5.92432E+00_JPRB, 6.03705E+00_JPRB, 6.12695E+00_JPRB, 6.19554E+00_JPRB, 6.28120E+00_JPRB, &
 &6.32144E+00_JPRB, 6.29808E+00_JPRB, 6.30071E+00_JPRB, 6.36210E+00_JPRB, 6.36860E+00_JPRB, &
 &6.34934E+00_JPRB, 6.32772E+00_JPRB, 6.28678E+00_JPRB, 6.23711E+00_JPRB, 6.17610E+00_JPRB, &
 &6.11009E+00_JPRB, 6.06261E+00_JPRB, 6.01470E+00_JPRB, 5.94683E+00_JPRB, 5.89427E+00_JPRB, &
 &5.86906E+00_JPRB, 5.85835E+00_JPRB, 5.85835E+00_JPRB, 5.86906E+00_JPRB, 5.89427E+00_JPRB, &
 &5.94683E+00_JPRB, 6.01470E+00_JPRB, 6.06261E+00_JPRB, 6.11009E+00_JPRB, 6.17610E+00_JPRB, &
 &6.23711E+00_JPRB, 6.28678E+00_JPRB, 6.32772E+00_JPRB, 6.34934E+00_JPRB, 6.36860E+00_JPRB, &
 &6.36210E+00_JPRB, 6.30071E+00_JPRB, 6.29808E+00_JPRB, 6.32144E+00_JPRB, 6.28120E+00_JPRB, &
 &6.19554E+00_JPRB, 6.12695E+00_JPRB, 6.03705E+00_JPRB, 5.92432E+00_JPRB, 5.77665E+00_JPRB, &
 &5.63335E+00_JPRB, 5.45512E+00_JPRB, 5.26356E+00_JPRB, 5.05484E+00_JPRB, 4.88904E+00_JPRB, &
 &4.75694E+00_JPRB, 4.64002E+00_JPRB, 4.54304E+00_JPRB, 4.44969E+00_JPRB /)   
ZOZCL(:,18) = (/&
 &4.58695E+00_JPRB, 4.70061E+00_JPRB, 4.81874E+00_JPRB, 4.95661E+00_JPRB, 5.10316E+00_JPRB, &
 &5.28898E+00_JPRB, 5.52349E+00_JPRB, 5.74338E+00_JPRB, 5.94835E+00_JPRB, 6.12908E+00_JPRB, &
 &6.30540E+00_JPRB, 6.44553E+00_JPRB, 6.55253E+00_JPRB, 6.63480E+00_JPRB, 6.73481E+00_JPRB, &
 &6.79356E+00_JPRB, 6.78322E+00_JPRB, 6.80965E+00_JPRB, 6.90321E+00_JPRB, 6.92805E+00_JPRB, &
 &6.92323E+00_JPRB, 6.90999E+00_JPRB, 6.87748E+00_JPRB, 6.83857E+00_JPRB, 6.78632E+00_JPRB, &
 &6.72776E+00_JPRB, 6.67384E+00_JPRB, 6.61478E+00_JPRB, 6.55163E+00_JPRB, 6.50241E+00_JPRB, &
 &6.47032E+00_JPRB, 6.45603E+00_JPRB, 6.45603E+00_JPRB, 6.47032E+00_JPRB, 6.50241E+00_JPRB, &
 &6.55163E+00_JPRB, 6.61478E+00_JPRB, 6.67384E+00_JPRB, 6.72776E+00_JPRB, 6.78632E+00_JPRB, &
 &6.83857E+00_JPRB, 6.87748E+00_JPRB, 6.90999E+00_JPRB, 6.92323E+00_JPRB, 6.92805E+00_JPRB, &
 &6.90321E+00_JPRB, 6.80965E+00_JPRB, 6.78322E+00_JPRB, 6.79356E+00_JPRB, 6.73481E+00_JPRB, &
 &6.63480E+00_JPRB, 6.55253E+00_JPRB, 6.44553E+00_JPRB, 6.30540E+00_JPRB, 6.12908E+00_JPRB, &
 &5.94835E+00_JPRB, 5.74338E+00_JPRB, 5.52349E+00_JPRB, 5.28898E+00_JPRB, 5.10316E+00_JPRB, &
 &4.95661E+00_JPRB, 4.81874E+00_JPRB, 4.70061E+00_JPRB, 4.58695E+00_JPRB /)   
ZOZCL(:,19) = (/&
 &4.66323E+00_JPRB, 4.78984E+00_JPRB, 4.92148E+00_JPRB, 5.07556E+00_JPRB, 5.24552E+00_JPRB, &
 &5.44571E+00_JPRB, 5.69596E+00_JPRB, 5.94358E+00_JPRB, 6.17857E+00_JPRB, 6.39401E+00_JPRB, &
 &6.59404E+00_JPRB, 6.75696E+00_JPRB, 6.89566E+00_JPRB, 7.00444E+00_JPRB, 7.12081E+00_JPRB, &
 &7.20030E+00_JPRB, 7.21206E+00_JPRB, 7.25604E+00_JPRB, 7.36941E+00_JPRB, 7.42498E+00_JPRB, &
 &7.44859E+00_JPRB, 7.45545E+00_JPRB, 7.43972E+00_JPRB, 7.41363E+00_JPRB, 7.37226E+00_JPRB, &
 &7.32340E+00_JPRB, 7.27498E+00_JPRB, 7.21961E+00_JPRB, 7.17249E+00_JPRB, 7.13884E+00_JPRB, &
 &7.11588E+00_JPRB, 7.10756E+00_JPRB, 7.10756E+00_JPRB, 7.11588E+00_JPRB, 7.13884E+00_JPRB, &
 &7.17249E+00_JPRB, 7.21961E+00_JPRB, 7.27498E+00_JPRB, 7.32340E+00_JPRB, 7.37226E+00_JPRB, &
 &7.41363E+00_JPRB, 7.43972E+00_JPRB, 7.45545E+00_JPRB, 7.44859E+00_JPRB, 7.42498E+00_JPRB, &
 &7.36941E+00_JPRB, 7.25604E+00_JPRB, 7.21206E+00_JPRB, 7.20030E+00_JPRB, 7.12081E+00_JPRB, &
 &7.00444E+00_JPRB, 6.89566E+00_JPRB, 6.75696E+00_JPRB, 6.59404E+00_JPRB, 6.39401E+00_JPRB, &
 &6.17857E+00_JPRB, 5.94358E+00_JPRB, 5.69596E+00_JPRB, 5.44571E+00_JPRB, 5.24552E+00_JPRB, &
 &5.07556E+00_JPRB, 4.92148E+00_JPRB, 4.78984E+00_JPRB, 4.66323E+00_JPRB /)   
ZOZCL(:,20) = (/&
 &4.70915E+00_JPRB, 4.84121E+00_JPRB, 4.97871E+00_JPRB, 5.13840E+00_JPRB, 5.31183E+00_JPRB, &
 &5.51790E+00_JPRB, 5.78768E+00_JPRB, 6.08066E+00_JPRB, 6.34816E+00_JPRB, 6.59108E+00_JPRB, &
 &6.81553E+00_JPRB, 6.99819E+00_JPRB, 7.14569E+00_JPRB, 7.26410E+00_JPRB, 7.40125E+00_JPRB, &
 &7.50653E+00_JPRB, 7.53834E+00_JPRB, 7.60563E+00_JPRB, 7.74719E+00_JPRB, 7.83706E+00_JPRB, &
 &7.89311E+00_JPRB, 7.92480E+00_JPRB, 7.93487E+00_JPRB, 7.93819E+00_JPRB, 7.91414E+00_JPRB, &
 &7.87354E+00_JPRB, 7.83974E+00_JPRB, 7.79981E+00_JPRB, 7.77299E+00_JPRB, 7.75738E+00_JPRB, &
 &7.73924E+00_JPRB, 7.73428E+00_JPRB, 7.73428E+00_JPRB, 7.73924E+00_JPRB, 7.75738E+00_JPRB, &
 &7.77299E+00_JPRB, 7.79981E+00_JPRB, 7.83974E+00_JPRB, 7.87354E+00_JPRB, 7.91414E+00_JPRB, &
 &7.93819E+00_JPRB, 7.93487E+00_JPRB, 7.92480E+00_JPRB, 7.89311E+00_JPRB, 7.83706E+00_JPRB, &
 &7.74719E+00_JPRB, 7.60563E+00_JPRB, 7.53834E+00_JPRB, 7.50653E+00_JPRB, 7.40125E+00_JPRB, &
 &7.26410E+00_JPRB, 7.14569E+00_JPRB, 6.99819E+00_JPRB, 6.81553E+00_JPRB, 6.59108E+00_JPRB, &
 &6.34816E+00_JPRB, 6.08066E+00_JPRB, 5.78768E+00_JPRB, 5.51790E+00_JPRB, 5.31183E+00_JPRB, &
 &5.13840E+00_JPRB, 4.97871E+00_JPRB, 4.84121E+00_JPRB, 4.70915E+00_JPRB /)   
ZOZCL(:,21) = (/&
 &4.72722E+00_JPRB, 4.85793E+00_JPRB, 4.99429E+00_JPRB, 5.15768E+00_JPRB, 5.35530E+00_JPRB, &
 &5.57761E+00_JPRB, 5.86410E+00_JPRB, 6.17049E+00_JPRB, 6.45368E+00_JPRB, 6.71482E+00_JPRB, &
 &6.95711E+00_JPRB, 7.14888E+00_JPRB, 7.30471E+00_JPRB, 7.42996E+00_JPRB, 7.57460E+00_JPRB, &
 &7.69364E+00_JPRB, 7.74333E+00_JPRB, 7.83694E+00_JPRB, 8.01082E+00_JPRB, 8.12495E+00_JPRB, &
 &8.20721E+00_JPRB, 8.28226E+00_JPRB, 8.33049E+00_JPRB, 8.36331E+00_JPRB, 8.36843E+00_JPRB, &
 &8.35702E+00_JPRB, 8.35113E+00_JPRB, 8.33787E+00_JPRB, 8.30778E+00_JPRB, 8.29334E+00_JPRB, &
 &8.29464E+00_JPRB, 8.30191E+00_JPRB, 8.30191E+00_JPRB, 8.29464E+00_JPRB, 8.29334E+00_JPRB, &
 &8.30778E+00_JPRB, 8.33787E+00_JPRB, 8.35113E+00_JPRB, 8.35702E+00_JPRB, 8.36843E+00_JPRB, &
 &8.36331E+00_JPRB, 8.33049E+00_JPRB, 8.28226E+00_JPRB, 8.20721E+00_JPRB, 8.12495E+00_JPRB, &
 &8.01082E+00_JPRB, 7.83694E+00_JPRB, 7.74333E+00_JPRB, 7.69364E+00_JPRB, 7.57460E+00_JPRB, &
 &7.42996E+00_JPRB, 7.30471E+00_JPRB, 7.14888E+00_JPRB, 6.95711E+00_JPRB, 6.71482E+00_JPRB, &
 &6.45368E+00_JPRB, 6.17049E+00_JPRB, 5.86410E+00_JPRB, 5.57761E+00_JPRB, 5.35530E+00_JPRB, &
 &5.15768E+00_JPRB, 4.99429E+00_JPRB, 4.85793E+00_JPRB, 4.72722E+00_JPRB /)   
ZOZCL(:,22) = (/&
 &4.71333E+00_JPRB, 4.84359E+00_JPRB, 4.97978E+00_JPRB, 5.14390E+00_JPRB, 5.34622E+00_JPRB, &
 &5.57382E+00_JPRB, 5.86777E+00_JPRB, 6.17821E+00_JPRB, 6.47155E+00_JPRB, 6.74496E+00_JPRB, &
 &7.00084E+00_JPRB, 7.19848E+00_JPRB, 7.36234E+00_JPRB, 7.49498E+00_JPRB, 7.64143E+00_JPRB, &
 &7.77328E+00_JPRB, 7.85141E+00_JPRB, 7.96438E+00_JPRB, 8.15615E+00_JPRB, 8.31301E+00_JPRB, &
 &8.43854E+00_JPRB, 8.54656E+00_JPRB, 8.63008E+00_JPRB, 8.70272E+00_JPRB, 8.74599E+00_JPRB, &
 &8.77159E+00_JPRB, 8.78994E+00_JPRB, 8.79630E+00_JPRB, 8.76793E+00_JPRB, 8.75608E+00_JPRB, &
 &8.76239E+00_JPRB, 8.77303E+00_JPRB, 8.77303E+00_JPRB, 8.76239E+00_JPRB, 8.75608E+00_JPRB, &
 &8.76793E+00_JPRB, 8.79630E+00_JPRB, 8.78994E+00_JPRB, 8.77159E+00_JPRB, 8.74599E+00_JPRB, &
 &8.70272E+00_JPRB, 8.63008E+00_JPRB, 8.54656E+00_JPRB, 8.43854E+00_JPRB, 8.31301E+00_JPRB, &
 &8.15615E+00_JPRB, 7.96438E+00_JPRB, 7.85141E+00_JPRB, 7.77328E+00_JPRB, 7.64143E+00_JPRB, &
 &7.49498E+00_JPRB, 7.36234E+00_JPRB, 7.19848E+00_JPRB, 7.00084E+00_JPRB, 6.74496E+00_JPRB, &
 &6.47155E+00_JPRB, 6.17821E+00_JPRB, 5.86777E+00_JPRB, 5.57382E+00_JPRB, 5.34622E+00_JPRB, &
 &5.14390E+00_JPRB, 4.97978E+00_JPRB, 4.84359E+00_JPRB, 4.71333E+00_JPRB /)   
ZOZCL(:,23) = (/&
 &4.67912E+00_JPRB, 4.79764E+00_JPRB, 4.92160E+00_JPRB, 5.07457E+00_JPRB, 5.27695E+00_JPRB, &
 &5.51058E+00_JPRB, 5.81288E+00_JPRB, 6.12986E+00_JPRB, 6.43090E+00_JPRB, 6.70737E+00_JPRB, &
 &6.97063E+00_JPRB, 7.17993E+00_JPRB, 7.36027E+00_JPRB, 7.50778E+00_JPRB, 7.65802E+00_JPRB, &
 &7.79243E+00_JPRB, 7.88299E+00_JPRB, 8.02608E+00_JPRB, 8.25668E+00_JPRB, 8.44587E+00_JPRB, &
 &8.60415E+00_JPRB, 8.74549E+00_JPRB, 8.86606E+00_JPRB, 8.98280E+00_JPRB, 9.06624E+00_JPRB, &
 &9.12905E+00_JPRB, 9.16728E+00_JPRB, 9.18795E+00_JPRB, 9.17360E+00_JPRB, 9.17340E+00_JPRB, &
 &9.17901E+00_JPRB, 9.18968E+00_JPRB, 9.18968E+00_JPRB, 9.17901E+00_JPRB, 9.17340E+00_JPRB, &
 &9.17360E+00_JPRB, 9.18795E+00_JPRB, 9.16728E+00_JPRB, 9.12905E+00_JPRB, 9.06624E+00_JPRB, &
 &8.98280E+00_JPRB, 8.86606E+00_JPRB, 8.74549E+00_JPRB, 8.60415E+00_JPRB, 8.44587E+00_JPRB, &
 &8.25668E+00_JPRB, 8.02608E+00_JPRB, 7.88299E+00_JPRB, 7.79243E+00_JPRB, 7.65802E+00_JPRB, &
 &7.50778E+00_JPRB, 7.36027E+00_JPRB, 7.17993E+00_JPRB, 6.97063E+00_JPRB, 6.70737E+00_JPRB, &
 &6.43090E+00_JPRB, 6.12986E+00_JPRB, 5.81288E+00_JPRB, 5.51058E+00_JPRB, 5.27695E+00_JPRB, &
 &5.07457E+00_JPRB, 4.92160E+00_JPRB, 4.79764E+00_JPRB, 4.67912E+00_JPRB /)   
ZOZCL(:,24) = (/&
 &4.63197E+00_JPRB, 4.73726E+00_JPRB, 4.84746E+00_JPRB, 4.98410E+00_JPRB, 5.16663E+00_JPRB, &
 &5.39558E+00_JPRB, 5.69325E+00_JPRB, 6.01992E+00_JPRB, 6.33758E+00_JPRB, 6.61680E+00_JPRB, &
 &6.89206E+00_JPRB, 7.11509E+00_JPRB, 7.30384E+00_JPRB, 7.46269E+00_JPRB, 7.62325E+00_JPRB, &
 &7.77080E+00_JPRB, 7.88784E+00_JPRB, 8.06008E+00_JPRB, 8.32255E+00_JPRB, 8.53345E+00_JPRB, &
 &8.71588E+00_JPRB, 8.89587E+00_JPRB, 9.05804E+00_JPRB, 9.22151E+00_JPRB, 9.35652E+00_JPRB, &
 &9.47492E+00_JPRB, 9.54587E+00_JPRB, 9.59178E+00_JPRB, 9.57128E+00_JPRB, 9.55942E+00_JPRB, &
 &9.52847E+00_JPRB, 9.51743E+00_JPRB, 9.51743E+00_JPRB, 9.52847E+00_JPRB, 9.55942E+00_JPRB, &
 &9.57128E+00_JPRB, 9.59178E+00_JPRB, 9.54587E+00_JPRB, 9.47492E+00_JPRB, 9.35652E+00_JPRB, &
 &9.22151E+00_JPRB, 9.05804E+00_JPRB, 8.89587E+00_JPRB, 8.71588E+00_JPRB, 8.53345E+00_JPRB, &
 &8.32255E+00_JPRB, 8.06008E+00_JPRB, 7.88784E+00_JPRB, 7.77080E+00_JPRB, 7.62325E+00_JPRB, &
 &7.46269E+00_JPRB, 7.30384E+00_JPRB, 7.11509E+00_JPRB, 6.89206E+00_JPRB, 6.61680E+00_JPRB, &
 &6.33758E+00_JPRB, 6.01992E+00_JPRB, 5.69325E+00_JPRB, 5.39558E+00_JPRB, 5.16663E+00_JPRB, &
 &4.98410E+00_JPRB, 4.84746E+00_JPRB, 4.73726E+00_JPRB, 4.63197E+00_JPRB /)   
ZOZCL(:,25) = (/&
 &4.55738E+00_JPRB, 4.65200E+00_JPRB, 4.75109E+00_JPRB, 4.87275E+00_JPRB, 5.03066E+00_JPRB, &
 &5.25133E+00_JPRB, 5.54212E+00_JPRB, 5.87313E+00_JPRB, 6.19917E+00_JPRB, 6.48973E+00_JPRB, &
 &6.76777E+00_JPRB, 6.99703E+00_JPRB, 7.20024E+00_JPRB, 7.37248E+00_JPRB, 7.54341E+00_JPRB, &
 &7.71100E+00_JPRB, 7.85624E+00_JPRB, 8.04804E+00_JPRB, 8.32757E+00_JPRB, 8.56905E+00_JPRB, &
 &8.78336E+00_JPRB, 8.98978E+00_JPRB, 9.18605E+00_JPRB, 9.39751E+00_JPRB, 9.57433E+00_JPRB, &
 &9.72945E+00_JPRB, 9.83621E+00_JPRB, 9.91754E+00_JPRB, 9.89808E+00_JPRB, 9.88418E+00_JPRB, &
 &9.83926E+00_JPRB, 9.82015E+00_JPRB, 9.82015E+00_JPRB, 9.83926E+00_JPRB, 9.88418E+00_JPRB, &
 &9.89808E+00_JPRB, 9.91754E+00_JPRB, 9.83621E+00_JPRB, 9.72945E+00_JPRB, 9.57433E+00_JPRB, &
 &9.39751E+00_JPRB, 9.18605E+00_JPRB, 8.98978E+00_JPRB, 8.78336E+00_JPRB, 8.56905E+00_JPRB, &
 &8.32757E+00_JPRB, 8.04804E+00_JPRB, 7.85624E+00_JPRB, 7.71100E+00_JPRB, 7.54341E+00_JPRB, &
 &7.37248E+00_JPRB, 7.20024E+00_JPRB, 6.99703E+00_JPRB, 6.76777E+00_JPRB, 6.48973E+00_JPRB, &
 &6.19917E+00_JPRB, 5.87313E+00_JPRB, 5.54212E+00_JPRB, 5.25133E+00_JPRB, 5.03066E+00_JPRB, &
 &4.87275E+00_JPRB, 4.75109E+00_JPRB, 4.65200E+00_JPRB, 4.55738E+00_JPRB /)   
ZOZCL(:,26) = (/&
 &4.46737E+00_JPRB, 4.55306E+00_JPRB, 4.64275E+00_JPRB, 4.74888E+00_JPRB, 4.87538E+00_JPRB, &
 &5.09632E+00_JPRB, 5.38816E+00_JPRB, 5.72586E+00_JPRB, 6.05227E+00_JPRB, 6.34382E+00_JPRB, &
 &6.62906E+00_JPRB, 6.86787E+00_JPRB, 7.06938E+00_JPRB, 7.24461E+00_JPRB, 7.43089E+00_JPRB, &
 &7.61465E+00_JPRB, 7.77368E+00_JPRB, 7.97999E+00_JPRB, 8.27469E+00_JPRB, 8.54114E+00_JPRB, &
 &8.78206E+00_JPRB, 9.01210E+00_JPRB, 9.23563E+00_JPRB, 9.48045E+00_JPRB, 9.69058E+00_JPRB, &
 &9.87902E+00_JPRB, 1.00275E+01_JPRB, 1.01536E+01_JPRB, 1.01607E+01_JPRB, 1.01696E+01_JPRB, &
 &1.01314E+01_JPRB, 1.01166E+01_JPRB, 1.01166E+01_JPRB, 1.01314E+01_JPRB, 1.01696E+01_JPRB, &
 &1.01607E+01_JPRB, 1.01536E+01_JPRB, 1.00275E+01_JPRB, 9.87902E+00_JPRB, 9.69058E+00_JPRB, &
 &9.48045E+00_JPRB, 9.23563E+00_JPRB, 9.01210E+00_JPRB, 8.78206E+00_JPRB, 8.54114E+00_JPRB, &
 &8.27469E+00_JPRB, 7.97999E+00_JPRB, 7.77368E+00_JPRB, 7.61465E+00_JPRB, 7.43089E+00_JPRB, &
 &7.24461E+00_JPRB, 7.06938E+00_JPRB, 6.86787E+00_JPRB, 6.62906E+00_JPRB, 6.34382E+00_JPRB, &
 &6.05227E+00_JPRB, 5.72586E+00_JPRB, 5.38816E+00_JPRB, 5.09632E+00_JPRB, 4.87538E+00_JPRB, &
 &4.74888E+00_JPRB, 4.64275E+00_JPRB, 4.55306E+00_JPRB, 4.46737E+00_JPRB /)   
ZOZCL(:,27) = (/&
 &4.36882E+00_JPRB, 4.44878E+00_JPRB, 4.53227E+00_JPRB, 4.63006E+00_JPRB, 4.74235E+00_JPRB, &
 &4.95221E+00_JPRB, 5.23354E+00_JPRB, 5.56581E+00_JPRB, 5.88807E+00_JPRB, 6.18085E+00_JPRB, &
 &6.47150E+00_JPRB, 6.71959E+00_JPRB, 6.92773E+00_JPRB, 7.11015E+00_JPRB, 7.30585E+00_JPRB, &
 &7.49745E+00_JPRB, 7.65902E+00_JPRB, 7.87789E+00_JPRB, 8.18885E+00_JPRB, 8.47273E+00_JPRB, &
 &8.73086E+00_JPRB, 8.96884E+00_JPRB, 9.20141E+00_JPRB, 9.45704E+00_JPRB, 9.68719E+00_JPRB, &
 &9.90336E+00_JPRB, 1.00956E+01_JPRB, 1.02712E+01_JPRB, 1.03276E+01_JPRB, 1.03791E+01_JPRB, &
 &1.03532E+01_JPRB, 1.03461E+01_JPRB, 1.03461E+01_JPRB, 1.03532E+01_JPRB, 1.03791E+01_JPRB, &
 &1.03276E+01_JPRB, 1.02712E+01_JPRB, 1.00956E+01_JPRB, 9.90336E+00_JPRB, 9.68719E+00_JPRB, &
 &9.45704E+00_JPRB, 9.20141E+00_JPRB, 8.96884E+00_JPRB, 8.73086E+00_JPRB, 8.47273E+00_JPRB, &
 &8.18885E+00_JPRB, 7.87789E+00_JPRB, 7.65902E+00_JPRB, 7.49745E+00_JPRB, 7.30585E+00_JPRB, &
 &7.11015E+00_JPRB, 6.92773E+00_JPRB, 6.71959E+00_JPRB, 6.47150E+00_JPRB, 6.18085E+00_JPRB, &
 &5.88807E+00_JPRB, 5.56581E+00_JPRB, 5.23354E+00_JPRB, 4.95221E+00_JPRB, 4.74235E+00_JPRB, &
 &4.63006E+00_JPRB, 4.53227E+00_JPRB, 4.44878E+00_JPRB, 4.36882E+00_JPRB /)   
ZOZCL(:,28) = (/&
 &4.26235E+00_JPRB, 4.33831E+00_JPRB, 4.41740E+00_JPRB, 4.51158E+00_JPRB, 4.62265E+00_JPRB, &
 &4.81678E+00_JPRB, 5.08391E+00_JPRB, 5.40195E+00_JPRB, 5.70985E+00_JPRB, 5.99577E+00_JPRB, &
 &6.28338E+00_JPRB, 6.53423E+00_JPRB, 6.74123E+00_JPRB, 6.92431E+00_JPRB, 7.12135E+00_JPRB, &
 &7.31801E+00_JPRB, 7.49046E+00_JPRB, 7.71409E+00_JPRB, 8.02617E+00_JPRB, 8.31288E+00_JPRB, &
 &8.57620E+00_JPRB, 8.82669E+00_JPRB, 9.07309E+00_JPRB, 9.34398E+00_JPRB, 9.59013E+00_JPRB, &
 &9.82264E+00_JPRB, 1.00262E+01_JPRB, 1.02122E+01_JPRB, 1.03041E+01_JPRB, 1.03951E+01_JPRB, &
 &1.04242E+01_JPRB, 1.04504E+01_JPRB, 1.04504E+01_JPRB, 1.04242E+01_JPRB, 1.03951E+01_JPRB, &
 &1.03041E+01_JPRB, 1.02122E+01_JPRB, 1.00262E+01_JPRB, 9.82264E+00_JPRB, 9.59013E+00_JPRB, &
 &9.34398E+00_JPRB, 9.07309E+00_JPRB, 8.82669E+00_JPRB, 8.57620E+00_JPRB, 8.31288E+00_JPRB, &
 &8.02617E+00_JPRB, 7.71409E+00_JPRB, 7.49046E+00_JPRB, 7.31801E+00_JPRB, 7.12135E+00_JPRB, &
 &6.92431E+00_JPRB, 6.74123E+00_JPRB, 6.53423E+00_JPRB, 6.28338E+00_JPRB, 5.99577E+00_JPRB, &
 &5.70985E+00_JPRB, 5.40195E+00_JPRB, 5.08391E+00_JPRB, 4.81678E+00_JPRB, 4.62265E+00_JPRB, &
 &4.51158E+00_JPRB, 4.41740E+00_JPRB, 4.33831E+00_JPRB, 4.26235E+00_JPRB /)   
ZOZCL(:,29) = (/&
 &4.15160E+00_JPRB, 4.22500E+00_JPRB, 4.30139E+00_JPRB, 4.39270E+00_JPRB, 4.50209E+00_JPRB, &
 &4.69393E+00_JPRB, 4.94888E+00_JPRB, 5.24183E+00_JPRB, 5.53129E+00_JPRB, 5.80518E+00_JPRB, &
 &6.08307E+00_JPRB, 6.32793E+00_JPRB, 6.53472E+00_JPRB, 6.71791E+00_JPRB, 6.91121E+00_JPRB, &
 &7.09585E+00_JPRB, 7.25162E+00_JPRB, 7.45713E+00_JPRB, 7.74731E+00_JPRB, 8.01454E+00_JPRB, &
 &8.25796E+00_JPRB, 8.48377E+00_JPRB, 8.70315E+00_JPRB, 8.94174E+00_JPRB, 9.17005E+00_JPRB, &
 &9.39713E+00_JPRB, 9.59122E+00_JPRB, 9.76712E+00_JPRB, 9.89084E+00_JPRB, 1.00074E+01_JPRB, &
 &1.00310E+01_JPRB, 1.00533E+01_JPRB, 1.00533E+01_JPRB, 1.00310E+01_JPRB, 1.00074E+01_JPRB, &
 &9.89084E+00_JPRB, 9.76712E+00_JPRB, 9.59122E+00_JPRB, 9.39713E+00_JPRB, 9.17005E+00_JPRB, &
 &8.94174E+00_JPRB, 8.70315E+00_JPRB, 8.48377E+00_JPRB, 8.25796E+00_JPRB, 8.01454E+00_JPRB, &
 &7.74731E+00_JPRB, 7.45713E+00_JPRB, 7.25162E+00_JPRB, 7.09585E+00_JPRB, 6.91121E+00_JPRB, &
 &6.71791E+00_JPRB, 6.53472E+00_JPRB, 6.32793E+00_JPRB, 6.08307E+00_JPRB, 5.80518E+00_JPRB, &
 &5.53129E+00_JPRB, 5.24183E+00_JPRB, 4.94888E+00_JPRB, 4.69393E+00_JPRB, 4.50209E+00_JPRB, &
 &4.39270E+00_JPRB, 4.30139E+00_JPRB, 4.22500E+00_JPRB, 4.15160E+00_JPRB /)   
ZOZCL(:,30) = (/&
 &4.02550E+00_JPRB, 4.09676E+00_JPRB, 4.17093E+00_JPRB, 4.25964E+00_JPRB, 4.36236E+00_JPRB, &
 &4.53261E+00_JPRB, 4.76532E+00_JPRB, 5.03610E+00_JPRB, 5.30936E+00_JPRB, 5.57354E+00_JPRB, &
 &5.84450E+00_JPRB, 6.08420E+00_JPRB, 6.27261E+00_JPRB, 6.44120E+00_JPRB, 6.62256E+00_JPRB, &
 &6.79302E+00_JPRB, 6.94008E+00_JPRB, 7.12418E+00_JPRB, 7.38402E+00_JPRB, 7.62355E+00_JPRB, &
 &7.84035E+00_JPRB, 8.04398E+00_JPRB, 8.23635E+00_JPRB, 8.43785E+00_JPRB, 8.63638E+00_JPRB, &
 &8.83974E+00_JPRB, 9.00596E+00_JPRB, 9.15261E+00_JPRB, 9.25188E+00_JPRB, 9.34608E+00_JPRB, &
 &9.36226E+00_JPRB, 9.37964E+00_JPRB, 9.37964E+00_JPRB, 9.36226E+00_JPRB, 9.34608E+00_JPRB, &
 &9.25188E+00_JPRB, 9.15261E+00_JPRB, 9.00596E+00_JPRB, 8.83974E+00_JPRB, 8.63638E+00_JPRB, &
 &8.43785E+00_JPRB, 8.23635E+00_JPRB, 8.04398E+00_JPRB, 7.84035E+00_JPRB, 7.62355E+00_JPRB, &
 &7.38402E+00_JPRB, 7.12418E+00_JPRB, 6.94008E+00_JPRB, 6.79302E+00_JPRB, 6.62256E+00_JPRB, &
 &6.44120E+00_JPRB, 6.27261E+00_JPRB, 6.08420E+00_JPRB, 5.84450E+00_JPRB, 5.57354E+00_JPRB, &
 &5.30936E+00_JPRB, 5.03610E+00_JPRB, 4.76532E+00_JPRB, 4.53261E+00_JPRB, 4.36236E+00_JPRB, &
 &4.25964E+00_JPRB, 4.17093E+00_JPRB, 4.09676E+00_JPRB, 4.02550E+00_JPRB /)   
ZOZCL(:,31) = (/&
 &3.88440E+00_JPRB, 3.95650E+00_JPRB, 4.03160E+00_JPRB, 4.12184E+00_JPRB, 4.22540E+00_JPRB, &
 &4.37983E+00_JPRB, 4.59330E+00_JPRB, 4.84209E+00_JPRB, 5.09485E+00_JPRB, 5.34154E+00_JPRB, &
 &5.59465E+00_JPRB, 5.81792E+00_JPRB, 5.99288E+00_JPRB, 6.14883E+00_JPRB, 6.32017E+00_JPRB, &
 &6.47304E+00_JPRB, 6.59279E+00_JPRB, 6.74248E+00_JPRB, 6.96047E+00_JPRB, 7.15585E+00_JPRB, &
 &7.32836E+00_JPRB, 7.48982E+00_JPRB, 7.63834E+00_JPRB, 7.79184E+00_JPRB, 7.94031E+00_JPRB, &
 &8.09063E+00_JPRB, 8.20904E+00_JPRB, 8.30961E+00_JPRB, 8.36028E+00_JPRB, 8.41371E+00_JPRB, &
 &8.43205E+00_JPRB, 8.45006E+00_JPRB, 8.45006E+00_JPRB, 8.43205E+00_JPRB, 8.41371E+00_JPRB, &
 &8.36028E+00_JPRB, 8.30961E+00_JPRB, 8.20904E+00_JPRB, 8.09063E+00_JPRB, 7.94031E+00_JPRB, &
 &7.79184E+00_JPRB, 7.63834E+00_JPRB, 7.48982E+00_JPRB, 7.32836E+00_JPRB, 7.15585E+00_JPRB, &
 &6.96047E+00_JPRB, 6.74248E+00_JPRB, 6.59279E+00_JPRB, 6.47304E+00_JPRB, 6.32017E+00_JPRB, &
 &6.14883E+00_JPRB, 5.99288E+00_JPRB, 5.81792E+00_JPRB, 5.59465E+00_JPRB, 5.34154E+00_JPRB, &
 &5.09485E+00_JPRB, 4.84209E+00_JPRB, 4.59330E+00_JPRB, 4.37983E+00_JPRB, 4.22540E+00_JPRB, &
 &4.12184E+00_JPRB, 4.03160E+00_JPRB, 3.95650E+00_JPRB, 3.88440E+00_JPRB /)   
ZOZCL(:,32) = (/&
 &3.72632E+00_JPRB, 3.79719E+00_JPRB, 3.87096E+00_JPRB, 3.96019E+00_JPRB, 4.06381E+00_JPRB, &
 &4.20882E+00_JPRB, 4.40453E+00_JPRB, 4.62546E+00_JPRB, 4.85893E+00_JPRB, 5.08595E+00_JPRB, &
 &5.31392E+00_JPRB, 5.50994E+00_JPRB, 5.66583E+00_JPRB, 5.80017E+00_JPRB, 5.94296E+00_JPRB, &
 &6.06540E+00_JPRB, 6.15216E+00_JPRB, 6.25556E+00_JPRB, 6.41664E+00_JPRB, 6.56867E+00_JPRB, &
 &6.69660E+00_JPRB, 6.79894E+00_JPRB, 6.88705E+00_JPRB, 6.97678E+00_JPRB, 7.05915E+00_JPRB, &
 &7.14022E+00_JPRB, 7.19384E+00_JPRB, 7.23135E+00_JPRB, 7.24114E+00_JPRB, 7.25790E+00_JPRB, &
 &7.26455E+00_JPRB, 7.27460E+00_JPRB, 7.27460E+00_JPRB, 7.26455E+00_JPRB, 7.25790E+00_JPRB, &
 &7.24114E+00_JPRB, 7.23135E+00_JPRB, 7.19384E+00_JPRB, 7.14022E+00_JPRB, 7.05915E+00_JPRB, &
 &6.97678E+00_JPRB, 6.88705E+00_JPRB, 6.79894E+00_JPRB, 6.69660E+00_JPRB, 6.56867E+00_JPRB, &
 &6.41664E+00_JPRB, 6.25556E+00_JPRB, 6.15216E+00_JPRB, 6.06540E+00_JPRB, 5.94296E+00_JPRB, &
 &5.80017E+00_JPRB, 5.66583E+00_JPRB, 5.50994E+00_JPRB, 5.31392E+00_JPRB, 5.08595E+00_JPRB, &
 &4.85893E+00_JPRB, 4.62546E+00_JPRB, 4.40453E+00_JPRB, 4.20882E+00_JPRB, 4.06381E+00_JPRB, &
 &3.96019E+00_JPRB, 3.87096E+00_JPRB, 3.79719E+00_JPRB, 3.72632E+00_JPRB /)   
ZOZCL(:,33) = (/&
 &3.55839E+00_JPRB, 3.62594E+00_JPRB, 3.69632E+00_JPRB, 3.78119E+00_JPRB, 3.87769E+00_JPRB, &
 &4.01295E+00_JPRB, 4.19113E+00_JPRB, 4.38376E+00_JPRB, 4.59531E+00_JPRB, 4.79405E+00_JPRB, &
 &4.99195E+00_JPRB, 5.16279E+00_JPRB, 5.29421E+00_JPRB, 5.40277E+00_JPRB, 5.50958E+00_JPRB, &
 &5.59121E+00_JPRB, 5.63876E+00_JPRB, 5.69835E+00_JPRB, 5.80838E+00_JPRB, 5.90968E+00_JPRB, &
 &5.98629E+00_JPRB, 6.03133E+00_JPRB, 6.06060E+00_JPRB, 6.08648E+00_JPRB, 6.10101E+00_JPRB, &
 &6.11010E+00_JPRB, 6.09862E+00_JPRB, 6.07441E+00_JPRB, 6.05843E+00_JPRB, 6.04733E+00_JPRB, &
 &6.02100E+00_JPRB, 6.01003E+00_JPRB, 6.01003E+00_JPRB, 6.02100E+00_JPRB, 6.04733E+00_JPRB, &
 &6.05843E+00_JPRB, 6.07441E+00_JPRB, 6.09862E+00_JPRB, 6.11010E+00_JPRB, 6.10101E+00_JPRB, &
 &6.08648E+00_JPRB, 6.06060E+00_JPRB, 6.03133E+00_JPRB, 5.98629E+00_JPRB, 5.90968E+00_JPRB, &
 &5.80838E+00_JPRB, 5.69835E+00_JPRB, 5.63876E+00_JPRB, 5.59121E+00_JPRB, 5.50958E+00_JPRB, &
 &5.40277E+00_JPRB, 5.29421E+00_JPRB, 5.16279E+00_JPRB, 4.99195E+00_JPRB, 4.79405E+00_JPRB, &
 &4.59531E+00_JPRB, 4.38376E+00_JPRB, 4.19113E+00_JPRB, 4.01295E+00_JPRB, 3.87769E+00_JPRB, &
 &3.78119E+00_JPRB, 3.69632E+00_JPRB, 3.62594E+00_JPRB, 3.55839E+00_JPRB /)   
ZOZCL(:,34) = (/&
 &3.38607E+00_JPRB, 3.44237E+00_JPRB, 3.50101E+00_JPRB, 3.57053E+00_JPRB, 3.64964E+00_JPRB, &
 &3.79562E+00_JPRB, 3.96840E+00_JPRB, 4.12875E+00_JPRB, 4.30634E+00_JPRB, 4.47161E+00_JPRB, &
 &4.63204E+00_JPRB, 4.76472E+00_JPRB, 4.87413E+00_JPRB, 4.95648E+00_JPRB, 5.02032E+00_JPRB, &
 &5.05873E+00_JPRB, 5.06888E+00_JPRB, 5.09097E+00_JPRB, 5.15813E+00_JPRB, 5.20198E+00_JPRB, &
 &5.22138E+00_JPRB, 5.21126E+00_JPRB, 5.18598E+00_JPRB, 5.15541E+00_JPRB, 5.11018E+00_JPRB, &
 &5.05672E+00_JPRB, 5.00681E+00_JPRB, 4.95301E+00_JPRB, 4.91336E+00_JPRB, 4.88144E+00_JPRB, &
 &4.85223E+00_JPRB, 4.83868E+00_JPRB, 4.83868E+00_JPRB, 4.85223E+00_JPRB, 4.88144E+00_JPRB, &
 &4.91336E+00_JPRB, 4.95301E+00_JPRB, 5.00681E+00_JPRB, 5.05672E+00_JPRB, 5.11018E+00_JPRB, &
 &5.15541E+00_JPRB, 5.18598E+00_JPRB, 5.21126E+00_JPRB, 5.22138E+00_JPRB, 5.20198E+00_JPRB, &
 &5.15813E+00_JPRB, 5.09097E+00_JPRB, 5.06888E+00_JPRB, 5.05873E+00_JPRB, 5.02032E+00_JPRB, &
 &4.95648E+00_JPRB, 4.87413E+00_JPRB, 4.76472E+00_JPRB, 4.63204E+00_JPRB, 4.47161E+00_JPRB, &
 &4.30634E+00_JPRB, 4.12875E+00_JPRB, 3.96840E+00_JPRB, 3.79562E+00_JPRB, 3.64964E+00_JPRB, &
 &3.57053E+00_JPRB, 3.50101E+00_JPRB, 3.44237E+00_JPRB, 3.38607E+00_JPRB /)   
ZOZCL(:,35) = (/&
 &3.19025E+00_JPRB, 3.23814E+00_JPRB, 3.28781E+00_JPRB, 3.34363E+00_JPRB, 3.39743E+00_JPRB, &
 &3.53331E+00_JPRB, 3.68686E+00_JPRB, 3.82229E+00_JPRB, 3.97588E+00_JPRB, 4.11943E+00_JPRB, &
 &4.25483E+00_JPRB, 4.35022E+00_JPRB, 4.42932E+00_JPRB, 4.48168E+00_JPRB, 4.50995E+00_JPRB, &
 &4.50574E+00_JPRB, 4.47157E+00_JPRB, 4.44884E+00_JPRB, 4.46575E+00_JPRB, 4.46194E+00_JPRB, &
 &4.43580E+00_JPRB, 4.37958E+00_JPRB, 4.30809E+00_JPRB, 4.22732E+00_JPRB, 4.13683E+00_JPRB, &
 &4.04255E+00_JPRB, 3.97562E+00_JPRB, 3.91318E+00_JPRB, 3.86111E+00_JPRB, 3.81884E+00_JPRB, &
 &3.79339E+00_JPRB, 3.78124E+00_JPRB, 3.78124E+00_JPRB, 3.79339E+00_JPRB, 3.81884E+00_JPRB, &
 &3.86111E+00_JPRB, 3.91318E+00_JPRB, 3.97562E+00_JPRB, 4.04255E+00_JPRB, 4.13683E+00_JPRB, &
 &4.22732E+00_JPRB, 4.30809E+00_JPRB, 4.37958E+00_JPRB, 4.43580E+00_JPRB, 4.46194E+00_JPRB, &
 &4.46575E+00_JPRB, 4.44884E+00_JPRB, 4.47157E+00_JPRB, 4.50574E+00_JPRB, 4.50995E+00_JPRB, &
 &4.48168E+00_JPRB, 4.42932E+00_JPRB, 4.35022E+00_JPRB, 4.25483E+00_JPRB, 4.11943E+00_JPRB, &
 &3.97588E+00_JPRB, 3.82229E+00_JPRB, 3.68686E+00_JPRB, 3.53331E+00_JPRB, 3.39743E+00_JPRB, &
 &3.34363E+00_JPRB, 3.28781E+00_JPRB, 3.23814E+00_JPRB, 3.19025E+00_JPRB /)   
ZOZCL(:,36) = (/&
 &2.98017E+00_JPRB, 3.01760E+00_JPRB, 3.05630E+00_JPRB, 3.10324E+00_JPRB, 3.15594E+00_JPRB, &
 &3.27107E+00_JPRB, 3.40117E+00_JPRB, 3.52109E+00_JPRB, 3.65488E+00_JPRB, 3.77450E+00_JPRB, &
 &3.87887E+00_JPRB, 3.93426E+00_JPRB, 3.97878E+00_JPRB, 3.99569E+00_JPRB, 3.98280E+00_JPRB, &
 &3.93843E+00_JPRB, 3.86680E+00_JPRB, 3.80076E+00_JPRB, 3.76759E+00_JPRB, 3.72135E+00_JPRB, &
 &3.65604E+00_JPRB, 3.56014E+00_JPRB, 3.45321E+00_JPRB, 3.33984E+00_JPRB, 3.22586E+00_JPRB, & 
 &3.11543E+00_JPRB, 3.04133E+00_JPRB, 2.97553E+00_JPRB, 2.92876E+00_JPRB, 2.89068E+00_JPRB, &
 &2.86870E+00_JPRB, 2.85794E+00_JPRB, 2.85794E+00_JPRB, 2.86870E+00_JPRB, 2.89068E+00_JPRB, &
 &2.92876E+00_JPRB, 2.97553E+00_JPRB, 3.04133E+00_JPRB, 3.11543E+00_JPRB, 3.22586E+00_JPRB, &
 &3.33984E+00_JPRB, 3.45321E+00_JPRB, 3.56014E+00_JPRB, 3.65604E+00_JPRB, 3.72135E+00_JPRB, &
 &3.76759E+00_JPRB, 3.80076E+00_JPRB, 3.86680E+00_JPRB, 3.93843E+00_JPRB, 3.98280E+00_JPRB, &
 &3.99569E+00_JPRB, 3.97878E+00_JPRB, 3.93426E+00_JPRB, 3.87887E+00_JPRB, 3.77450E+00_JPRB, &
 &3.65488E+00_JPRB, 3.52109E+00_JPRB, 3.40117E+00_JPRB, 3.27107E+00_JPRB, 3.15594E+00_JPRB, &
 &3.10324E+00_JPRB, 3.05630E+00_JPRB, 3.01760E+00_JPRB, 2.98017E+00_JPRB /)   
ZOZCL(:,37) = (/&
 &2.73384E+00_JPRB, 2.77057E+00_JPRB, 2.80851E+00_JPRB, 2.85674E+00_JPRB, 2.91161E+00_JPRB, &
 &2.99487E+00_JPRB, 3.09444E+00_JPRB, 3.18872E+00_JPRB, 3.28931E+00_JPRB, 3.37821E+00_JPRB, &
 &3.44982E+00_JPRB, 3.49037E+00_JPRB, 3.50900E+00_JPRB, 3.49717E+00_JPRB, 3.44330E+00_JPRB, &
 &3.36066E+00_JPRB, 3.25795E+00_JPRB, 3.15562E+00_JPRB, 3.08056E+00_JPRB, 2.99538E+00_JPRB, &
 &2.89722E+00_JPRB, 2.78037E+00_JPRB, 2.65600E+00_JPRB, 2.52691E+00_JPRB, 2.40847E+00_JPRB, &
 &2.30135E+00_JPRB, 2.22892E+00_JPRB, 2.16556E+00_JPRB, 2.12357E+00_JPRB, 2.09141E+00_JPRB, &
 &2.08356E+00_JPRB, 2.08058E+00_JPRB, 2.08058E+00_JPRB, 2.08356E+00_JPRB, 2.09141E+00_JPRB, &
 &2.12357E+00_JPRB, 2.16556E+00_JPRB, 2.22892E+00_JPRB, 2.30135E+00_JPRB, 2.40847E+00_JPRB, &
 &2.52691E+00_JPRB, 2.65600E+00_JPRB, 2.78037E+00_JPRB, 2.89722E+00_JPRB, 2.99538E+00_JPRB, &
 &3.08056E+00_JPRB, 3.15562E+00_JPRB, 3.25795E+00_JPRB, 3.36066E+00_JPRB, 3.44330E+00_JPRB, &
 &3.49717E+00_JPRB, 3.50900E+00_JPRB, 3.49037E+00_JPRB, 3.44982E+00_JPRB, 3.37821E+00_JPRB, &
 &3.28931E+00_JPRB, 3.18872E+00_JPRB, 3.09444E+00_JPRB, 2.99487E+00_JPRB, 2.91161E+00_JPRB, &
 &2.85674E+00_JPRB, 2.80851E+00_JPRB, 2.77057E+00_JPRB, 2.73384E+00_JPRB /)   
ZOZCL(:,38) = (/&
 &2.44889E+00_JPRB, 2.48330E+00_JPRB, 2.51896E+00_JPRB, 2.56553E+00_JPRB, 2.61879E+00_JPRB, &
 &2.68163E+00_JPRB, 2.76010E+00_JPRB, 2.83264E+00_JPRB, 2.90458E+00_JPRB, 2.96659E+00_JPRB, &
 &3.00385E+00_JPRB, 3.01988E+00_JPRB, 3.00743E+00_JPRB, 2.96513E+00_JPRB, 2.88232E+00_JPRB, &
 &2.77495E+00_JPRB, 2.64752E+00_JPRB, 2.51849E+00_JPRB, 2.41295E+00_JPRB, 2.29635E+00_JPRB, &
 &2.17503E+00_JPRB, 2.05188E+00_JPRB, 1.92685E+00_JPRB, 1.80231E+00_JPRB, 1.69754E+00_JPRB, &
 &1.60903E+00_JPRB, 1.54285E+00_JPRB, 1.48343E+00_JPRB, 1.45088E+00_JPRB, 1.42758E+00_JPRB, &
 &1.42942E+00_JPRB, 1.43177E+00_JPRB, 1.43177E+00_JPRB, 1.42942E+00_JPRB, 1.42758E+00_JPRB, &
 &1.45088E+00_JPRB, 1.48343E+00_JPRB, 1.54285E+00_JPRB, 1.60903E+00_JPRB, 1.69754E+00_JPRB, &
 &1.80231E+00_JPRB, 1.92685E+00_JPRB, 2.05188E+00_JPRB, 2.17503E+00_JPRB, 2.29635E+00_JPRB, &
 &2.41295E+00_JPRB, 2.51849E+00_JPRB, 2.64752E+00_JPRB, 2.77495E+00_JPRB, 2.88232E+00_JPRB, &
 &2.96513E+00_JPRB, 3.00743E+00_JPRB, 3.01988E+00_JPRB, 3.00385E+00_JPRB, 2.96659E+00_JPRB, &
 &2.90458E+00_JPRB, 2.83264E+00_JPRB, 2.76010E+00_JPRB, 2.68163E+00_JPRB, 2.61879E+00_JPRB, &
 &2.56553E+00_JPRB, 2.51896E+00_JPRB, 2.48330E+00_JPRB, 2.44889E+00_JPRB /)   
ZOZCL(:,39) = (/&
 &2.15250E+00_JPRB, 2.17620E+00_JPRB, 2.20047E+00_JPRB, 2.24048E+00_JPRB, 2.30814E+00_JPRB, &
 &2.35205E+00_JPRB, 2.40840E+00_JPRB, 2.45830E+00_JPRB, 2.50406E+00_JPRB, 2.53921E+00_JPRB, &
 &2.55267E+00_JPRB, 2.54663E+00_JPRB, 2.50161E+00_JPRB, 2.43071E+00_JPRB, 2.32565E+00_JPRB, &
 &2.20257E+00_JPRB, 2.06376E+00_JPRB, 1.92101E+00_JPRB, 1.79879E+00_JPRB, 1.66058E+00_JPRB, &
 &1.52608E+00_JPRB, 1.40196E+00_JPRB, 1.28507E+00_JPRB, 1.17799E+00_JPRB, 1.09044E+00_JPRB, &
 &1.01769E+00_JPRB, 9.63303E-01_JPRB, 9.15154E-01_JPRB, 8.90068E-01_JPRB, 8.73254E-01_JPRB, &
 &8.80721E-01_JPRB, 8.86165E-01_JPRB, 8.86165E-01_JPRB, 8.80721E-01_JPRB, 8.73254E-01_JPRB, &
 &8.90068E-01_JPRB, 9.15154E-01_JPRB, 9.63303E-01_JPRB, 1.01769E+00_JPRB, 1.09044E+00_JPRB, &
 &1.17799E+00_JPRB, 1.28507E+00_JPRB, 1.40196E+00_JPRB, 1.52608E+00_JPRB, 1.66058E+00_JPRB, &
 &1.79879E+00_JPRB, 1.92101E+00_JPRB, 2.06376E+00_JPRB, 2.20257E+00_JPRB, 2.32565E+00_JPRB, &
 &2.43071E+00_JPRB, 2.50161E+00_JPRB, 2.54663E+00_JPRB, 2.55267E+00_JPRB, 2.53921E+00_JPRB, &
 &2.50406E+00_JPRB, 2.45830E+00_JPRB, 2.40840E+00_JPRB, 2.35205E+00_JPRB, 2.30814E+00_JPRB, &
 &2.24048E+00_JPRB, 2.20047E+00_JPRB, 2.17620E+00_JPRB, 2.15250E+00_JPRB /)   
ZOZCL(:,40) = (/&
 &1.83693E+00_JPRB, 1.85524E+00_JPRB, 1.87380E+00_JPRB, 1.90611E+00_JPRB, 1.96046E+00_JPRB, &
 &1.98430E+00_JPRB, 2.01845E+00_JPRB, 2.04475E+00_JPRB, 2.06795E+00_JPRB, 2.07918E+00_JPRB, &
 &2.07489E+00_JPRB, 2.05836E+00_JPRB, 2.00633E+00_JPRB, 1.93071E+00_JPRB, 1.81760E+00_JPRB, &
 &1.68782E+00_JPRB, 1.54985E+00_JPRB, 1.40315E+00_JPRB, 1.27448E+00_JPRB, 1.13005E+00_JPRB, &
 &9.97243E-01_JPRB, 8.80875E-01_JPRB, 7.77177E-01_JPRB, 6.86904E-01_JPRB, 6.17787E-01_JPRB, &
 &5.63218E-01_JPRB, 5.20198E-01_JPRB, 4.82263E-01_JPRB, 4.64661E-01_JPRB, 4.53623E-01_JPRB, &
 &4.62148E-01_JPRB, 4.68077E-01_JPRB, 4.68077E-01_JPRB, 4.62148E-01_JPRB, 4.53623E-01_JPRB, &
 &4.64661E-01_JPRB, 4.82263E-01_JPRB, 5.20198E-01_JPRB, 5.63218E-01_JPRB, 6.17787E-01_JPRB, &
 &6.86904E-01_JPRB, 7.77177E-01_JPRB, 8.80875E-01_JPRB, 9.97243E-01_JPRB, 1.13005E+00_JPRB, &
 &1.27448E+00_JPRB, 1.40315E+00_JPRB, 1.54985E+00_JPRB, 1.68782E+00_JPRB, 1.81760E+00_JPRB, &
 &1.93071E+00_JPRB, 2.00633E+00_JPRB, 2.05836E+00_JPRB, 2.07489E+00_JPRB, 2.07918E+00_JPRB, &
 &2.06795E+00_JPRB, 2.04475E+00_JPRB, 2.01845E+00_JPRB, 1.98430E+00_JPRB, 1.96046E+00_JPRB, &
 &1.90611E+00_JPRB, 1.87380E+00_JPRB, 1.85524E+00_JPRB, 1.83693E+00_JPRB /)   
ZOZCL(:,41) = (/&
 &1.50752E+00_JPRB, 1.52394E+00_JPRB, 1.54064E+00_JPRB, 1.56750E+00_JPRB, 1.60579E+00_JPRB, &
 &1.61865E+00_JPRB, 1.63840E+00_JPRB, 1.64520E+00_JPRB, 1.65247E+00_JPRB, 1.65234E+00_JPRB, &
 &1.64195E+00_JPRB, 1.61867E+00_JPRB, 1.56163E+00_JPRB, 1.48612E+00_JPRB, 1.37628E+00_JPRB, &
 &1.25041E+00_JPRB, 1.12248E+00_JPRB, 9.87718E-01_JPRB, 8.70360E-01_JPRB, 7.37185E-01_JPRB, &
 &6.21238E-01_JPRB, 5.25204E-01_JPRB, 4.44070E-01_JPRB, 3.76293E-01_JPRB, 3.26848E-01_JPRB, &
 &2.89422E-01_JPRB, 2.60936E-01_JPRB, 2.36583E-01_JPRB, 2.24418E-01_JPRB, 2.16572E-01_JPRB, &
 &2.20830E-01_JPRB, 2.23846E-01_JPRB, 2.23846E-01_JPRB, 2.20830E-01_JPRB, 2.16572E-01_JPRB, &
 &2.24418E-01_JPRB, 2.36583E-01_JPRB, 2.60936E-01_JPRB, 2.89422E-01_JPRB, 3.26848E-01_JPRB, &
 &3.76293E-01_JPRB, 4.44070E-01_JPRB, 5.25204E-01_JPRB, 6.21238E-01_JPRB, 7.37185E-01_JPRB, &
 &8.70360E-01_JPRB, 9.87718E-01_JPRB, 1.12248E+00_JPRB, 1.25041E+00_JPRB, 1.37628E+00_JPRB, &
 &1.48612E+00_JPRB, 1.56163E+00_JPRB, 1.61867E+00_JPRB, 1.64195E+00_JPRB, 1.65234E+00_JPRB, &
 &1.65247E+00_JPRB, 1.64520E+00_JPRB, 1.63840E+00_JPRB, 1.61865E+00_JPRB, 1.60579E+00_JPRB, &
 &1.56750E+00_JPRB, 1.54064E+00_JPRB, 1.52394E+00_JPRB, 1.50752E+00_JPRB /)   
ZOZCL(:,42) = (/&
 &1.27945E+00_JPRB, 1.28479E+00_JPRB, 1.28990E+00_JPRB, 1.30137E+00_JPRB, 1.31880E+00_JPRB, &
 &1.31862E+00_JPRB, 1.30715E+00_JPRB, 1.28822E+00_JPRB, 1.27017E+00_JPRB, 1.24669E+00_JPRB, &
 &1.21805E+00_JPRB, 1.18032E+00_JPRB, 1.12824E+00_JPRB, 1.06610E+00_JPRB, 9.86858E-01_JPRB, &
 &9.01532E-01_JPRB, 8.16254E-01_JPRB, 7.19605E-01_JPRB, 6.12891E-01_JPRB, 5.08153E-01_JPRB, &
 &4.19557E-01_JPRB, 3.49238E-01_JPRB, 2.96092E-01_JPRB, 2.51434E-01_JPRB, 2.18232E-01_JPRB, &
 &1.92741E-01_JPRB, 1.72459E-01_JPRB, 1.58579E-01_JPRB, 1.50857E-01_JPRB, 1.45233E-01_JPRB, &
 &1.45128E-01_JPRB, 1.44607E-01_JPRB, 1.44607E-01_JPRB, 1.45128E-01_JPRB, 1.45233E-01_JPRB, &
 &1.50857E-01_JPRB, 1.58579E-01_JPRB, 1.72459E-01_JPRB, 1.92741E-01_JPRB, 2.18232E-01_JPRB, &
 &2.51434E-01_JPRB, 2.96092E-01_JPRB, 3.49238E-01_JPRB, 4.19557E-01_JPRB, 5.08153E-01_JPRB, &
 &6.12891E-01_JPRB, 7.19605E-01_JPRB, 8.16254E-01_JPRB, 9.01532E-01_JPRB, 9.86858E-01_JPRB, &
 &1.06610E+00_JPRB, 1.12824E+00_JPRB, 1.18032E+00_JPRB, 1.21805E+00_JPRB, 1.24669E+00_JPRB, &
 &1.27017E+00_JPRB, 1.28822E+00_JPRB, 1.30715E+00_JPRB, 1.31862E+00_JPRB, 1.31880E+00_JPRB, &
 &1.30137E+00_JPRB, 1.28990E+00_JPRB, 1.28479E+00_JPRB, 1.27945E+00_JPRB /)   
ZOZCL(:,43) = (/&
 &1.08757E+00_JPRB, 1.08504E+00_JPRB, 1.08212E+00_JPRB, 1.08306E+00_JPRB, 1.08648E+00_JPRB, &
 &1.07705E+00_JPRB, 1.04544E+00_JPRB, 1.01100E+00_JPRB, 9.78591E-01_JPRB, 9.43120E-01_JPRB, &
 &9.06581E-01_JPRB, 8.64193E-01_JPRB, 8.18677E-01_JPRB, 7.68376E-01_JPRB, 7.11086E-01_JPRB, &
 &6.53146E-01_JPRB, 5.96320E-01_JPRB, 5.26418E-01_JPRB, 4.32725E-01_JPRB, 3.51107E-01_JPRB, &
 &2.84097E-01_JPRB, 2.32855E-01_JPRB, 1.97915E-01_JPRB, 1.68695E-01_JPRB, 1.46487E-01_JPRB, &
 &1.29095E-01_JPRB, 1.14701E-01_JPRB, 1.06872E-01_JPRB, 1.01920E-01_JPRB, 9.79130E-02_JPRB, &
 &9.58961E-02_JPRB, 9.39386E-02_JPRB, 9.39386E-02_JPRB, 9.58961E-02_JPRB, 9.79130E-02_JPRB, &
 &1.01920E-01_JPRB, 1.06872E-01_JPRB, 1.14701E-01_JPRB, 1.29095E-01_JPRB, 1.46487E-01_JPRB, &
 &1.68695E-01_JPRB, 1.97915E-01_JPRB, 2.32855E-01_JPRB, 2.84097E-01_JPRB, 3.51107E-01_JPRB, &
 &4.32725E-01_JPRB, 5.26418E-01_JPRB, 5.96320E-01_JPRB, 6.53146E-01_JPRB, 7.11086E-01_JPRB, &
 &7.68376E-01_JPRB, 8.18677E-01_JPRB, 8.64193E-01_JPRB, 9.06581E-01_JPRB, 9.43120E-01_JPRB, &
 &9.78591E-01_JPRB, 1.01100E+00_JPRB, 1.04544E+00_JPRB, 1.07705E+00_JPRB, 1.08648E+00_JPRB, &
 &1.08306E+00_JPRB, 1.08212E+00_JPRB, 1.08504E+00_JPRB, 1.08757E+00_JPRB /)   
ZOZCL(:,44) = (/&
 &9.25922E-01_JPRB, 9.17938E-01_JPRB, 9.09625E-01_JPRB, 9.03568E-01_JPRB, 8.97835E-01_JPRB, &
 &8.82048E-01_JPRB, 8.38158E-01_JPRB, 7.95214E-01_JPRB, 7.55658E-01_JPRB, 7.15325E-01_JPRB, &
 &6.76977E-01_JPRB, 6.35310E-01_JPRB, 5.96643E-01_JPRB, 5.56440E-01_JPRB, 5.14939E-01_JPRB, &
 &4.75526E-01_JPRB, 4.37684E-01_JPRB, 3.86684E-01_JPRB, 3.06332E-01_JPRB, 2.43166E-01_JPRB, &
 &1.92864E-01_JPRB, 1.55662E-01_JPRB, 1.32617E-01_JPRB, 1.13653E-01_JPRB, 9.88656E-02_JPRB, &
 &8.69834E-02_JPRB, 7.67827E-02_JPRB, 7.24225E-02_JPRB, 6.92112E-02_JPRB, 6.63655E-02_JPRB, &
 &6.37103E-02_JPRB, 6.13636E-02_JPRB, 6.13636E-02_JPRB, 6.37103E-02_JPRB, 6.63655E-02_JPRB, &
 &6.92112E-02_JPRB, 7.24225E-02_JPRB, 7.67827E-02_JPRB, 8.69834E-02_JPRB, 9.88656E-02_JPRB, &
 &1.13653E-01_JPRB, 1.32617E-01_JPRB, 1.55662E-01_JPRB, 1.92864E-01_JPRB, 2.43166E-01_JPRB, &
 &3.06332E-01_JPRB, 3.86684E-01_JPRB, 4.37684E-01_JPRB, 4.75526E-01_JPRB, 5.14939E-01_JPRB, &
 &5.56440E-01_JPRB, 5.96643E-01_JPRB, 6.35310E-01_JPRB, 6.76977E-01_JPRB, 7.15325E-01_JPRB, &
 &7.55658E-01_JPRB, 7.95214E-01_JPRB, 8.38158E-01_JPRB, 8.82048E-01_JPRB, 8.97835E-01_JPRB, &
 &9.03568E-01_JPRB, 9.09625E-01_JPRB, 9.17938E-01_JPRB, 9.25922E-01_JPRB /)   
ZOZCL(:,45) = (/&
 &6.82010E-01_JPRB, 6.75983E-01_JPRB, 6.69549E-01_JPRB, 6.64811E-01_JPRB, 6.60316E-01_JPRB, &
 &6.49172E-01_JPRB, 6.18773E-01_JPRB, 5.88958E-01_JPRB, 5.61617E-01_JPRB, 5.33667E-01_JPRB, &
 &5.07121E-01_JPRB, 4.76253E-01_JPRB, 4.47535E-01_JPRB, 4.17777E-01_JPRB, 3.87311E-01_JPRB, &
 &3.58359E-01_JPRB, 3.30508E-01_JPRB, 2.92239E-01_JPRB, 2.31035E-01_JPRB, 1.83196E-01_JPRB, &
 &1.45280E-01_JPRB, 1.17270E-01_JPRB, 9.96461E-02_JPRB, 8.50357E-02_JPRB, 7.40739E-02_JPRB, &
 &6.56345E-02_JPRB, 5.83767E-02_JPRB, 5.56308E-02_JPRB, 5.37354E-02_JPRB, 5.20787E-02_JPRB, &
 &5.05269E-02_JPRB, 4.91805E-02_JPRB, 4.91805E-02_JPRB, 5.05269E-02_JPRB, 5.20787E-02_JPRB, &
 &5.37354E-02_JPRB, 5.56308E-02_JPRB, 5.83767E-02_JPRB, 6.56345E-02_JPRB, 7.40739E-02_JPRB, &
 &8.50357E-02_JPRB, 9.96461E-02_JPRB, 1.17270E-01_JPRB, 1.45280E-01_JPRB, 1.83196E-01_JPRB, &
 &2.31035E-01_JPRB, 2.92239E-01_JPRB, 3.30508E-01_JPRB, 3.58359E-01_JPRB, 3.87311E-01_JPRB, &
 &4.17777E-01_JPRB, 4.47535E-01_JPRB, 4.76253E-01_JPRB, 5.07121E-01_JPRB, 5.33667E-01_JPRB, &
 &5.61617E-01_JPRB, 5.88958E-01_JPRB, 6.18773E-01_JPRB, 6.49172E-01_JPRB, 6.60316E-01_JPRB, &
 &6.64811E-01_JPRB, 6.69549E-01_JPRB, 6.75983E-01_JPRB, 6.82010E-01_JPRB /)   
ZOZCL(:,46) = (/&
 &5.11782E-01_JPRB, 5.07170E-01_JPRB, 5.02027E-01_JPRB, 4.98172E-01_JPRB, 4.94385E-01_JPRB, &
 &4.86653E-01_JPRB, 4.66123E-01_JPRB, 4.45914E-01_JPRB, 4.27570E-01_JPRB, 4.08742E-01_JPRB, &
 &3.90935E-01_JPRB, 3.67216E-01_JPRB, 3.45173E-01_JPRB, 3.22615E-01_JPRB, 3.00003E-01_JPRB, &
 &2.78571E-01_JPRB, 2.57984E-01_JPRB, 2.29604E-01_JPRB, 1.83622E-01_JPRB, 1.47313E-01_JPRB, &
 &1.18215E-01_JPRB, 9.64565E-02_JPRB, 8.24708E-02_JPRB, 7.07373E-02_JPRB, 6.20778E-02_JPRB, &
 &5.55481E-02_JPRB, 4.98871E-02_JPRB, 4.78651E-02_JPRB, 4.65092E-02_JPRB, 4.53239E-02_JPRB, &
 &4.41960E-02_JPRB, 4.32189E-02_JPRB, 4.32189E-02_JPRB, 4.41960E-02_JPRB, 4.53239E-02_JPRB, &
 &4.65092E-02_JPRB, 4.78651E-02_JPRB, 4.98871E-02_JPRB, 5.55481E-02_JPRB, 6.20778E-02_JPRB, &
 &7.07373E-02_JPRB, 8.24708E-02_JPRB, 9.64565E-02_JPRB, 1.18215E-01_JPRB, 1.47313E-01_JPRB, &
 &1.83622E-01_JPRB, 2.29604E-01_JPRB, 2.57984E-01_JPRB, 2.78571E-01_JPRB, 3.00003E-01_JPRB, &
 &3.22615E-01_JPRB, 3.45173E-01_JPRB, 3.67216E-01_JPRB, 3.90935E-01_JPRB, 4.08742E-01_JPRB, &
 &4.27570E-01_JPRB, 4.45914E-01_JPRB, 4.66123E-01_JPRB, 4.86653E-01_JPRB, 4.94385E-01_JPRB, &
 &4.98172E-01_JPRB, 5.02027E-01_JPRB, 5.07170E-01_JPRB, 5.11782E-01_JPRB /)   
ZOZCL(:,47) = (/&
 &3.63765E-01_JPRB, 3.60429E-01_JPRB, 3.56585E-01_JPRB, 3.53686E-01_JPRB, 3.50802E-01_JPRB, &
 &3.45210E-01_JPRB, 3.30805E-01_JPRB, 3.16695E-01_JPRB, 3.04031E-01_JPRB, 2.91112E-01_JPRB, &
 &2.78967E-01_JPRB, 2.63001E-01_JPRB, 2.48166E-01_JPRB, 2.33134E-01_JPRB, 2.18416E-01_JPRB, &
 &2.04392E-01_JPRB, 1.90835E-01_JPRB, 1.72208E-01_JPRB, 1.41525E-01_JPRB, 1.16615E-01_JPRB, &
 &9.60598E-02_JPRB, 8.01891E-02_JPRB, 6.95306E-02_JPRB, 6.04548E-02_JPRB, 5.37852E-02_JPRB, &
 &4.87839E-02_JPRB, 4.43809E-02_JPRB, 4.28447E-02_JPRB, 4.18395E-02_JPRB, 4.09670E-02_JPRB, &
 &4.01268E-02_JPRB, 3.94065E-02_JPRB, 3.94065E-02_JPRB, 4.01268E-02_JPRB, 4.09670E-02_JPRB, &
 &4.18395E-02_JPRB, 4.28447E-02_JPRB, 4.43809E-02_JPRB, 4.87839E-02_JPRB, 5.37852E-02_JPRB, &
 &6.04548E-02_JPRB, 6.95306E-02_JPRB, 8.01891E-02_JPRB, 9.60598E-02_JPRB, 1.16615E-01_JPRB, &
 &1.41525E-01_JPRB, 1.72208E-01_JPRB, 1.90835E-01_JPRB, 2.04392E-01_JPRB, 2.18416E-01_JPRB, &
 &2.33134E-01_JPRB, 2.48166E-01_JPRB, 2.63001E-01_JPRB, 2.78967E-01_JPRB, 2.91112E-01_JPRB, &
 &3.04031E-01_JPRB, 3.16695E-01_JPRB, 3.30805E-01_JPRB, 3.45210E-01_JPRB, 3.50802E-01_JPRB, &
 &3.53686E-01_JPRB, 3.56585E-01_JPRB, 3.60429E-01_JPRB, 3.63765E-01_JPRB /)   
ZOZCL(:,48) = (/&
 &2.29434E-01_JPRB, 2.27275E-01_JPRB, 2.24755E-01_JPRB, 2.22870E-01_JPRB, 2.21029E-01_JPRB, &
 &2.16942E-01_JPRB, 2.06652E-01_JPRB, 1.96768E-01_JPRB, 1.87990E-01_JPRB, 1.79202E-01_JPRB, &
 &1.70985E-01_JPRB, 1.63083E-01_JPRB, 1.55752E-01_JPRB, 1.48377E-01_JPRB, 1.41421E-01_JPRB, &
 &1.34556E-01_JPRB, 1.27676E-01_JPRB, 1.18441E-01_JPRB, 1.03035E-01_JPRB, 8.96772E-02_JPRB, &
 &7.78627E-02_JPRB, 6.79781E-02_JPRB, 6.05699E-02_JPRB, 5.41094E-02_JPRB, 4.93300E-02_JPRB, &
 &4.57137E-02_JPRB, 4.24286E-02_JPRB, 4.12259E-02_JPRB, 4.04401E-02_JPRB, 3.97648E-02_JPRB, &
 &3.91041E-02_JPRB, 3.85464E-02_JPRB, 3.85464E-02_JPRB, 3.91041E-02_JPRB, 3.97648E-02_JPRB, &
 &4.04401E-02_JPRB, 4.12259E-02_JPRB, 4.24286E-02_JPRB, 4.57137E-02_JPRB, 4.93300E-02_JPRB, &
 &5.41094E-02_JPRB, 6.05699E-02_JPRB, 6.79781E-02_JPRB, 7.78627E-02_JPRB, 8.96772E-02_JPRB, &
 &1.03035E-01_JPRB, 1.18441E-01_JPRB, 1.27676E-01_JPRB, 1.34556E-01_JPRB, 1.41421E-01_JPRB, &
 &1.48377E-01_JPRB, 1.55752E-01_JPRB, 1.63083E-01_JPRB, 1.70985E-01_JPRB, 1.79202E-01_JPRB, &
 &1.87990E-01_JPRB, 1.96768E-01_JPRB, 2.06652E-01_JPRB, 2.16942E-01_JPRB, 2.21029E-01_JPRB, &
 &2.22870E-01_JPRB, 2.24755E-01_JPRB, 2.27275E-01_JPRB, 2.29434E-01_JPRB /)   
ZOZCL(:,49) = (/&
 &1.45805E-01_JPRB, 1.44396E-01_JPRB, 1.42736E-01_JPRB, 1.41505E-01_JPRB, 1.40326E-01_JPRB, &
 &1.37281E-01_JPRB, 1.29804E-01_JPRB, 1.22831E-01_JPRB, 1.16769E-01_JPRB, 1.10868E-01_JPRB, &
 &1.05440E-01_JPRB, 1.01676E-01_JPRB, 9.83351E-02_JPRB, 9.51024E-02_JPRB, 9.22746E-02_JPRB, &
 &8.93572E-02_JPRB, 8.62899E-02_JPRB, 8.23209E-02_JPRB, 7.56287E-02_JPRB, 6.94452E-02_JPRB, &
 &6.35446E-02_JPRB, 5.79995E-02_JPRB, 5.29706E-02_JPRB, 4.85668E-02_JPRB, 4.53460E-02_JPRB, &
 &4.29105E-02_JPRB, 4.06314E-02_JPRB, 3.97173E-02_JPRB, 3.91171E-02_JPRB, 3.86129E-02_JPRB, &
 &3.81128E-02_JPRB, 3.77057E-02_JPRB, 3.77057E-02_JPRB, 3.81128E-02_JPRB, 3.86129E-02_JPRB, &
 &3.91171E-02_JPRB, 3.97173E-02_JPRB, 4.06314E-02_JPRB, 4.29105E-02_JPRB, 4.53460E-02_JPRB, &
 &4.85668E-02_JPRB, 5.29706E-02_JPRB, 5.79995E-02_JPRB, 6.35446E-02_JPRB, 6.94452E-02_JPRB, &
 &7.56287E-02_JPRB, 8.23209E-02_JPRB, 8.62899E-02_JPRB, 8.93572E-02_JPRB, 9.22746E-02_JPRB, &
 &9.51024E-02_JPRB, 9.83351E-02_JPRB, 1.01676E-01_JPRB, 1.05440E-01_JPRB, 1.10868E-01_JPRB, &
 &1.16769E-01_JPRB, 1.22831E-01_JPRB, 1.29804E-01_JPRB, 1.37281E-01_JPRB, 1.40326E-01_JPRB, &
 &1.41505E-01_JPRB, 1.42736E-01_JPRB, 1.44396E-01_JPRB, 1.45805E-01_JPRB /)   
ZOZCL(:,50) = (/&
 &1.02111E-01_JPRB, 1.01121E-01_JPRB, 9.99752E-02_JPRB, 9.91327E-02_JPRB, 9.83368E-02_JPRB, &
 &9.66579E-02_JPRB, 9.24560E-02_JPRB, 8.84785E-02_JPRB, 8.50321E-02_JPRB, 8.15801E-02_JPRB, &
 &7.83596E-02_JPRB, 7.66252E-02_JPRB, 7.51168E-02_JPRB, 7.36208E-02_JPRB, 7.24192E-02_JPRB, &
 &7.10784E-02_JPRB, 6.95502E-02_JPRB, 6.75136E-02_JPRB, 6.38220E-02_JPRB, 6.02876E-02_JPRB, &
 &5.67336E-02_JPRB, 5.31224E-02_JPRB, 4.94627E-02_JPRB, 4.62381E-02_JPRB, 4.38110E-02_JPRB, &
 &4.19064E-02_JPRB, 4.00936E-02_JPRB, 3.92147E-02_JPRB, 3.85899E-02_JPRB, 3.80565E-02_JPRB, &
 &3.75245E-02_JPRB, 3.70813E-02_JPRB, 3.70813E-02_JPRB, 3.75245E-02_JPRB, 3.80565E-02_JPRB, &
 &3.85899E-02_JPRB, 3.92147E-02_JPRB, 4.00936E-02_JPRB, 4.19064E-02_JPRB, 4.38110E-02_JPRB, &
 &4.62381E-02_JPRB, 4.94627E-02_JPRB, 5.31224E-02_JPRB, 5.67336E-02_JPRB, 6.02876E-02_JPRB, &
 &6.38220E-02_JPRB, 6.75136E-02_JPRB, 6.95502E-02_JPRB, 7.10784E-02_JPRB, 7.24192E-02_JPRB, &
 &7.36208E-02_JPRB, 7.51168E-02_JPRB, 7.66252E-02_JPRB, 7.83596E-02_JPRB, 8.15801E-02_JPRB, &
 &8.50321E-02_JPRB, 8.84785E-02_JPRB, 9.24560E-02_JPRB, 9.66579E-02_JPRB, 9.83368E-02_JPRB, &
 &9.91327E-02_JPRB, 9.99752E-02_JPRB, 1.01121E-01_JPRB, 1.02111E-01_JPRB /)   
ZOZCL(:,51) = (/&
 &7.53064E-02_JPRB, 7.45857E-02_JPRB, 7.37752E-02_JPRB, 7.31865E-02_JPRB, 7.26370E-02_JPRB, &
 &7.19888E-02_JPRB, 7.02443E-02_JPRB, 6.85071E-02_JPRB, 6.70258E-02_JPRB, 6.53944E-02_JPRB, &
 &6.38124E-02_JPRB, 6.32647E-02_JPRB, 6.28041E-02_JPRB, 6.22691E-02_JPRB, 6.19641E-02_JPRB, &
 &6.15024E-02_JPRB, 6.08398E-02_JPRB, 5.97895E-02_JPRB, 5.74628E-02_JPRB, 5.51812E-02_JPRB, &
 &5.27836E-02_JPRB, 5.02533E-02_JPRB, 4.76230E-02_JPRB, 4.52699E-02_JPRB, 4.33442E-02_JPRB, &
 &4.17048E-02_JPRB, 4.01297E-02_JPRB, 3.91724E-02_JPRB, 3.84275E-02_JPRB, 3.77742E-02_JPRB, &
 &3.71232E-02_JPRB, 3.65602E-02_JPRB, 3.65602E-02_JPRB, 3.71232E-02_JPRB, 3.77742E-02_JPRB, &
 &3.84275E-02_JPRB, 3.91724E-02_JPRB, 4.01297E-02_JPRB, 4.17048E-02_JPRB, 4.33442E-02_JPRB, &
 &4.52699E-02_JPRB, 4.76230E-02_JPRB, 5.02533E-02_JPRB, 5.27836E-02_JPRB, 5.51812E-02_JPRB, &
 &5.74628E-02_JPRB, 5.97895E-02_JPRB, 6.08398E-02_JPRB, 6.15024E-02_JPRB, 6.19641E-02_JPRB, &
 &6.22691E-02_JPRB, 6.28041E-02_JPRB, 6.32647E-02_JPRB, 6.38124E-02_JPRB, 6.53944E-02_JPRB, &
 &6.70258E-02_JPRB, 6.85071E-02_JPRB, 7.02443E-02_JPRB, 7.19888E-02_JPRB, 7.26370E-02_JPRB, &
 &7.31865E-02_JPRB, 7.37752E-02_JPRB, 7.45857E-02_JPRB, 7.53064E-02_JPRB /)   
ZOZCL(:,52) = (/&
 &5.60640E-02_JPRB, 5.55360E-02_JPRB, 5.49609E-02_JPRB, 5.45502E-02_JPRB, 5.41725E-02_JPRB, &
 &5.41036E-02_JPRB, 5.37837E-02_JPRB, 5.34018E-02_JPRB, 5.31513E-02_JPRB, 5.27156E-02_JPRB, &
 &5.22549E-02_JPRB, 5.24169E-02_JPRB, 5.26187E-02_JPRB, 5.27417E-02_JPRB, 5.30904E-02_JPRB, &
 &5.32953E-02_JPRB, 5.33141E-02_JPRB, 5.30501E-02_JPRB, 5.18097E-02_JPRB, 5.05604E-02_JPRB, &
 &4.91508E-02_JPRB, 4.75769E-02_JPRB, 4.58898E-02_JPRB, 4.43676E-02_JPRB, 4.29228E-02_JPRB, &
 &4.15335E-02_JPRB, 4.01924E-02_JPRB, 3.91487E-02_JPRB, 3.82768E-02_JPRB, 3.74997E-02_JPRB, &
 &3.67282E-02_JPRB, 3.60467E-02_JPRB, 3.60467E-02_JPRB, 3.67282E-02_JPRB, 3.74997E-02_JPRB, &
 &3.82768E-02_JPRB, 3.91487E-02_JPRB, 4.01924E-02_JPRB, 4.15335E-02_JPRB, 4.29228E-02_JPRB, &
 &4.43676E-02_JPRB, 4.58898E-02_JPRB, 4.75769E-02_JPRB, 4.91508E-02_JPRB, 5.05604E-02_JPRB, &
 &5.18097E-02_JPRB, 5.30501E-02_JPRB, 5.33141E-02_JPRB, 5.32953E-02_JPRB, 5.30904E-02_JPRB, &
 &5.27417E-02_JPRB, 5.26187E-02_JPRB, 5.24169E-02_JPRB, 5.22549E-02_JPRB, 5.27156E-02_JPRB, &
 &5.31513E-02_JPRB, 5.34018E-02_JPRB, 5.37837E-02_JPRB, 5.41036E-02_JPRB, 5.41725E-02_JPRB, &
 &5.45502E-02_JPRB, 5.49609E-02_JPRB, 5.55360E-02_JPRB, 5.60640E-02_JPRB /)   
ZOZCL(:,53) = (/&
 &4.27770E-02_JPRB, 4.23818E-02_JPRB, 4.19647E-02_JPRB, 4.16731E-02_JPRB, 4.14099E-02_JPRB, &
 &4.16336E-02_JPRB, 4.20615E-02_JPRB, 4.24272E-02_JPRB, 4.28817E-02_JPRB, 4.31715E-02_JPRB, &
 &4.34241E-02_JPRB, 4.39614E-02_JPRB, 4.45367E-02_JPRB, 4.50705E-02_JPRB, 4.58601E-02_JPRB, &
 &4.65351E-02_JPRB, 4.70555E-02_JPRB, 4.73719E-02_JPRB, 4.69210E-02_JPRB, 4.64493E-02_JPRB, &
 &4.58131E-02_JPRB, 4.50264E-02_JPRB, 4.41679E-02_JPRB, 4.34018E-02_JPRB, 4.23882E-02_JPRB, &
 &4.12097E-02_JPRB, 4.00751E-02_JPRB, 3.89489E-02_JPRB, 3.79593E-02_JPRB, 3.70691E-02_JPRB, &
 &3.61902E-02_JPRB, 3.54045E-02_JPRB, 3.54045E-02_JPRB, 3.61902E-02_JPRB, 3.70691E-02_JPRB, &
 &3.79593E-02_JPRB, 3.89489E-02_JPRB, 4.00751E-02_JPRB, 4.12097E-02_JPRB, 4.23882E-02_JPRB, &
 &4.34018E-02_JPRB, 4.41679E-02_JPRB, 4.50264E-02_JPRB, 4.58131E-02_JPRB, 4.64493E-02_JPRB, &
 &4.69210E-02_JPRB, 4.73719E-02_JPRB, 4.70555E-02_JPRB, 4.65351E-02_JPRB, 4.58601E-02_JPRB, &
 &4.50705E-02_JPRB, 4.45367E-02_JPRB, 4.39614E-02_JPRB, 4.34241E-02_JPRB, 4.31715E-02_JPRB, &
 &4.28817E-02_JPRB, 4.24272E-02_JPRB, 4.20615E-02_JPRB, 4.16336E-02_JPRB, 4.14099E-02_JPRB, &
 &4.16731E-02_JPRB, 4.19647E-02_JPRB, 4.23818E-02_JPRB, 4.27770E-02_JPRB /)   
ZOZCL(:,54) = (/&
 &3.90617E-02_JPRB, 3.87032E-02_JPRB, 3.83250E-02_JPRB, 3.80608E-02_JPRB, 3.78280E-02_JPRB, &
 &3.80795E-02_JPRB, 3.85863E-02_JPRB, 3.90372E-02_JPRB, 3.95705E-02_JPRB, 3.99537E-02_JPRB, &
 &4.03037E-02_JPRB, 4.08533E-02_JPRB, 4.14369E-02_JPRB, 4.20061E-02_JPRB, 4.28662E-02_JPRB, &
 &4.36243E-02_JPRB, 4.42404E-02_JPRB, 4.45967E-02_JPRB, 4.40603E-02_JPRB, 4.35128E-02_JPRB, &
 &4.28190E-02_JPRB, 4.20989E-02_JPRB, 4.15914E-02_JPRB, 4.11163E-02_JPRB, 4.01684E-02_JPRB, &
 &3.89148E-02_JPRB, 3.77108E-02_JPRB, 3.66273E-02_JPRB, 3.56884E-02_JPRB, 3.48453E-02_JPRB, &
 &3.40150E-02_JPRB, 3.32735E-02_JPRB, 3.32735E-02_JPRB, 3.40150E-02_JPRB, 3.48453E-02_JPRB, &
 &3.56884E-02_JPRB, 3.66273E-02_JPRB, 3.77108E-02_JPRB, 3.89148E-02_JPRB, 4.01684E-02_JPRB, &
 &4.11163E-02_JPRB, 4.15914E-02_JPRB, 4.20989E-02_JPRB, 4.28190E-02_JPRB, 4.35128E-02_JPRB, &
 &4.40603E-02_JPRB, 4.45967E-02_JPRB, 4.42404E-02_JPRB, 4.36243E-02_JPRB, 4.28662E-02_JPRB, &
 &4.20061E-02_JPRB, 4.14369E-02_JPRB, 4.08533E-02_JPRB, 4.03037E-02_JPRB, 3.99537E-02_JPRB, &
 &3.95705E-02_JPRB, 3.90372E-02_JPRB, 3.85863E-02_JPRB, 3.80795E-02_JPRB, 3.78280E-02_JPRB, &
 &3.80608E-02_JPRB, 3.83250E-02_JPRB, 3.87032E-02_JPRB, 3.90617E-02_JPRB /)   
ZOZCL(:,55) = (/&
 &3.57076E-02_JPRB, 3.53822E-02_JPRB, 3.50390E-02_JPRB, 3.47995E-02_JPRB, 3.45935E-02_JPRB, &
 &3.48646E-02_JPRB, 3.54299E-02_JPRB, 3.59468E-02_JPRB, 3.65418E-02_JPRB, 3.70016E-02_JPRB, &
 &3.74336E-02_JPRB, 3.79894E-02_JPRB, 3.85777E-02_JPRB, 3.91751E-02_JPRB, 4.00897E-02_JPRB, &
 &4.09149E-02_JPRB, 4.16112E-02_JPRB, 4.20022E-02_JPRB, 4.13996E-02_JPRB, 4.07993E-02_JPRB, &
 &4.00734E-02_JPRB, 3.94179E-02_JPRB, 3.92029E-02_JPRB, 3.90051E-02_JPRB, 3.81334E-02_JPRB, &
 &3.68136E-02_JPRB, 3.55508E-02_JPRB, 3.44894E-02_JPRB, 3.35806E-02_JPRB, 3.27687E-02_JPRB, &
 &3.19756E-02_JPRB, 3.12713E-02_JPRB, 3.12713E-02_JPRB, 3.19756E-02_JPRB, 3.27687E-02_JPRB, &
 &3.35806E-02_JPRB, 3.44894E-02_JPRB, 3.55508E-02_JPRB, 3.68136E-02_JPRB, 3.81334E-02_JPRB, &
 &3.90051E-02_JPRB, 3.92029E-02_JPRB, 3.94179E-02_JPRB, 4.00734E-02_JPRB, 4.07993E-02_JPRB, &
 &4.13996E-02_JPRB, 4.20022E-02_JPRB, 4.16112E-02_JPRB, 4.09149E-02_JPRB, 4.00897E-02_JPRB, &
 &3.91751E-02_JPRB, 3.85777E-02_JPRB, 3.79894E-02_JPRB, 3.74336E-02_JPRB, 3.70016E-02_JPRB, &
 &3.65418E-02_JPRB, 3.59468E-02_JPRB, 3.54299E-02_JPRB, 3.48646E-02_JPRB, 3.45935E-02_JPRB, &
 &3.47995E-02_JPRB, 3.50390E-02_JPRB, 3.53822E-02_JPRB, 3.57076E-02_JPRB /)   
ZOZCL(:,56) = (/&
 &3.15005E-02_JPRB, 3.12120E-02_JPRB, 3.09114E-02_JPRB, 3.06993E-02_JPRB, 3.05211E-02_JPRB, &
 &3.08150E-02_JPRB, 3.14477E-02_JPRB, 3.20426E-02_JPRB, 3.27124E-02_JPRB, 3.32680E-02_JPRB, &
 &3.38053E-02_JPRB, 3.42814E-02_JPRB, 3.47794E-02_JPRB, 3.53413E-02_JPRB, 3.63252E-02_JPRB, &
 &3.72359E-02_JPRB, 3.80332E-02_JPRB, 3.84204E-02_JPRB, 3.75651E-02_JPRB, 3.67267E-02_JPRB, &
 &3.57916E-02_JPRB, 3.48936E-02_JPRB, 3.42635E-02_JPRB, 3.36521E-02_JPRB, 3.27063E-02_JPRB, &
 &3.15573E-02_JPRB, 3.04502E-02_JPRB, 2.98209E-02_JPRB, 2.93617E-02_JPRB, 2.89833E-02_JPRB, &
 &2.86170E-02_JPRB, 2.83278E-02_JPRB, 2.83278E-02_JPRB, 2.86170E-02_JPRB, 2.89833E-02_JPRB, &
 &2.93617E-02_JPRB, 2.98209E-02_JPRB, 3.04502E-02_JPRB, 3.15573E-02_JPRB, 3.27063E-02_JPRB, &
 &3.36521E-02_JPRB, 3.42635E-02_JPRB, 3.48936E-02_JPRB, 3.57916E-02_JPRB, 3.67267E-02_JPRB, &
 &3.75651E-02_JPRB, 3.84204E-02_JPRB, 3.80332E-02_JPRB, 3.72359E-02_JPRB, 3.63252E-02_JPRB, &
 &3.53413E-02_JPRB, 3.47794E-02_JPRB, 3.42814E-02_JPRB, 3.38053E-02_JPRB, 3.32680E-02_JPRB, &
 &3.27124E-02_JPRB, 3.20426E-02_JPRB, 3.14477E-02_JPRB, 3.08150E-02_JPRB, 3.05211E-02_JPRB, &
 &3.06993E-02_JPRB, 3.09114E-02_JPRB, 3.12120E-02_JPRB, 3.15005E-02_JPRB /)   
ZOZCL(:,57) = (/&
 &2.70169E-02_JPRB, 2.67651E-02_JPRB, 2.65080E-02_JPRB, 2.63226E-02_JPRB, 2.61704E-02_JPRB, &
 &2.64665E-02_JPRB, 2.71203E-02_JPRB, 2.77576E-02_JPRB, 2.84775E-02_JPRB, 2.91183E-02_JPRB, &
 &2.97650E-02_JPRB, 3.01093E-02_JPRB, 3.04623E-02_JPRB, 3.09496E-02_JPRB, 3.20035E-02_JPRB, &
 &3.30092E-02_JPRB, 3.39258E-02_JPRB, 3.43217E-02_JPRB, 3.31986E-02_JPRB, 3.21145E-02_JPRB, &
 &3.09715E-02_JPRB, 2.98018E-02_JPRB, 2.86482E-02_JPRB, 2.75629E-02_JPRB, 2.65034E-02_JPRB, &
 &2.54868E-02_JPRB, 2.45105E-02_JPRB, 2.39904E-02_JPRB, 2.36234E-02_JPRB, 2.33292E-02_JPRB, &
 &2.30501E-02_JPRB, 2.28414E-02_JPRB, 2.28414E-02_JPRB, 2.30501E-02_JPRB, 2.33292E-02_JPRB, &
 &2.36234E-02_JPRB, 2.39904E-02_JPRB, 2.45105E-02_JPRB, 2.54868E-02_JPRB, 2.65034E-02_JPRB, &
 &2.75629E-02_JPRB, 2.86482E-02_JPRB, 2.98018E-02_JPRB, 3.09715E-02_JPRB, 3.21145E-02_JPRB, &
 &3.31986E-02_JPRB, 3.43217E-02_JPRB, 3.39258E-02_JPRB, 3.30092E-02_JPRB, 3.20035E-02_JPRB, &
 &3.09496E-02_JPRB, 3.04623E-02_JPRB, 3.01093E-02_JPRB, 2.97650E-02_JPRB, 2.91183E-02_JPRB, &
 &2.84775E-02_JPRB, 2.77576E-02_JPRB, 2.71203E-02_JPRB, 2.64665E-02_JPRB, 2.61704E-02_JPRB, &
 &2.63226E-02_JPRB, 2.65080E-02_JPRB, 2.67651E-02_JPRB, 2.70169E-02_JPRB /)   
ZOZCL(:,58) = (/&
 &2.33352E-02_JPRB, 2.31132E-02_JPRB, 2.28908E-02_JPRB, 2.27269E-02_JPRB, 2.25953E-02_JPRB, &
 &2.28689E-02_JPRB, 2.34844E-02_JPRB, 2.41103E-02_JPRB, 2.48347E-02_JPRB, 2.55210E-02_JPRB, &
 &2.62465E-02_JPRB, 2.64854E-02_JPRB, 2.67270E-02_JPRB, 2.71561E-02_JPRB, 2.82546E-02_JPRB, &
 &2.93343E-02_JPRB, 3.03552E-02_JPRB, 3.07580E-02_JPRB, 2.93864E-02_JPRB, 2.80863E-02_JPRB, &
 &2.67716E-02_JPRB, 2.54210E-02_JPRB, 2.39857E-02_JPRB, 2.26660E-02_JPRB, 2.14604E-02_JPRB, &
 &2.03560E-02_JPRB, 1.93110E-02_JPRB, 1.85950E-02_JPRB, 1.80120E-02_JPRB, 1.75034E-02_JPRB, &
 &1.70226E-02_JPRB, 1.66092E-02_JPRB, 1.66092E-02_JPRB, 1.70226E-02_JPRB, 1.75034E-02_JPRB, &
 &1.80120E-02_JPRB, 1.85950E-02_JPRB, 1.93110E-02_JPRB, 2.03560E-02_JPRB, 2.14604E-02_JPRB, &
 &2.26660E-02_JPRB, 2.39857E-02_JPRB, 2.54210E-02_JPRB, 2.67716E-02_JPRB, 2.80863E-02_JPRB, &
 &2.93864E-02_JPRB, 3.07580E-02_JPRB, 3.03552E-02_JPRB, 2.93343E-02_JPRB, 2.82546E-02_JPRB, &
 &2.71561E-02_JPRB, 2.67270E-02_JPRB, 2.64854E-02_JPRB, 2.62465E-02_JPRB, 2.55210E-02_JPRB, &
 &2.48347E-02_JPRB, 2.41103E-02_JPRB, 2.34844E-02_JPRB, 2.28689E-02_JPRB, 2.25953E-02_JPRB, &
 &2.27269E-02_JPRB, 2.28908E-02_JPRB, 2.31132E-02_JPRB, 2.33352E-02_JPRB /)   
ZOZCL(:,59) = (/&
 &2.21047E-02_JPRB, 2.18926E-02_JPRB, 2.16817E-02_JPRB, 2.15248E-02_JPRB, 2.13999E-02_JPRB, &
 &2.16608E-02_JPRB, 2.22515E-02_JPRB, 2.28630E-02_JPRB, 2.35804E-02_JPRB, 2.42759E-02_JPRB, &
 &2.50245E-02_JPRB, 2.52290E-02_JPRB, 2.54354E-02_JPRB, 2.58457E-02_JPRB, 2.69556E-02_JPRB, &
 &2.80585E-02_JPRB, 2.91151E-02_JPRB, 2.95016E-02_JPRB, 2.79805E-02_JPRB, 2.65527E-02_JPRB, &
 &2.51354E-02_JPRB, 2.36494E-02_JPRB, 2.19556E-02_JPRB, 2.04275E-02_JPRB, 1.91529E-02_JPRB, &
 &1.80668E-02_JPRB, 1.70456E-02_JPRB, 1.63900E-02_JPRB, 1.58689E-02_JPRB, 1.54207E-02_JPRB, &
 &1.50039E-02_JPRB, 1.46534E-02_JPRB, 1.46534E-02_JPRB, 1.50039E-02_JPRB, 1.54207E-02_JPRB, &
 &1.58689E-02_JPRB, 1.63900E-02_JPRB, 1.70456E-02_JPRB, 1.80668E-02_JPRB, 1.91529E-02_JPRB, &
 &2.04275E-02_JPRB, 2.19556E-02_JPRB, 2.36494E-02_JPRB, 2.51354E-02_JPRB, 2.65527E-02_JPRB, &
 &2.79805E-02_JPRB, 2.95016E-02_JPRB, 2.91151E-02_JPRB, 2.80585E-02_JPRB, 2.69556E-02_JPRB, &
 &2.58457E-02_JPRB, 2.54354E-02_JPRB, 2.52290E-02_JPRB, 2.50245E-02_JPRB, 2.42759E-02_JPRB, &
 &2.35804E-02_JPRB, 2.28630E-02_JPRB, 2.22515E-02_JPRB, 2.16608E-02_JPRB, 2.13999E-02_JPRB, &
 &2.15248E-02_JPRB, 2.16817E-02_JPRB, 2.18926E-02_JPRB, 2.21047E-02_JPRB /)   


! PRESSURE LEVELS FOR O3 CLIMATOLOGY IN PASCAL !!!!!!!!!!!

RPROC = (/&
 &    0.0_JPRB &
 &,   2.84191E+01_JPRB,   3.25350E+01_JPRB, 3.71936E+01_JPRB,   4.24615E+01_JPRB &
 &,   4.84153E+01_JPRB,   5.51432E+01_JPRB, 6.27490E+01_JPRB,   7.13542E+01_JPRB &
 &,   8.11039E+01_JPRB,   9.21699E+01_JPRB, 1.04757E+02_JPRB,   1.19108E+02_JPRB &
 &,   1.35511E+02_JPRB,   1.54305E+02_JPRB, 1.75891E+02_JPRB,   2.00739E+02_JPRB &
 &,   2.29403E+02_JPRB,   2.62529E+02_JPRB, 3.00884E+02_JPRB,   3.45369E+02_JPRB &
 &,   3.97050E+02_JPRB,   4.57190E+02_JPRB, 5.27278E+02_JPRB,   6.09073E+02_JPRB &
 &,   7.04636E+02_JPRB,   8.16371E+02_JPRB, 9.47075E+02_JPRB,   1.10000E+03_JPRB &
 &,   1.27894E+03_JPRB,   1.48832E+03_JPRB, 1.73342E+03_JPRB,   2.02039E+03_JPRB &
 &,   2.35659E+03_JPRB,   2.75066E+03_JPRB, 3.21283E+03_JPRB,   3.75509E+03_JPRB &
 &,   4.39150E+03_JPRB,   5.13840E+03_JPRB, 6.01483E+03_JPRB,   7.04289E+03_JPRB &
 &,   8.24830E+03_JPRB,   9.66096E+03_JPRB, 1.13155E+04_JPRB,   1.32514E+04_JPRB &
 &,   1.55122E+04_JPRB,   1.81445E+04_JPRB, 2.11947E+04_JPRB,   2.47079E+04_JPRB &
 &,   2.87273E+04_JPRB,   3.32956E+04_JPRB, 3.84573E+04_JPRB,   4.42610E+04_JPRB &
 &,   5.07601E+04_JPRB,   5.80132E+04_JPRB, 6.60837E+04_JPRB,   7.50393E+04_JPRB &
 &,   8.49521E+04_JPRB,   9.58981E+04_JPRB, 1.00369E+05_JPRB &
 &,   110000._JPRB &
 &/)

ENDIF

!     ------------------------------------------------------------------

!*         1.     TIME INDEX WITHIN OZONE CLIMATOLOGY
!                 -----------------------------------

IDY=NDD(KINDAT)-1
IMN=NMM(KINDAT)
IF (IMN == 1) THEN
  ZXTIME=REAL(IDY,KIND(ZXTIME))*1440._JPRB + KMINUT
ELSEIF (IMN == 2) THEN
  IF(IDY == 28) IDY=IDY-1
! A DAY IN FEB. IS 28.25*24*60/28=1452.8571min LONG.
  ZXTIME=44640._JPRB+REAL(IDY,KIND(ZXTIME))*1452.8571_JPRB+KMINUT
ELSE
  ZXTIME=(ZMDAY(IMN-1)+REAL(IDY,KIND(ZXTIME)))*1440._JPRB+KMINUT
ENDIF
! 525960=MINUTES IN A SIDERAL YEAR (365.25d)
ZXTIME=MOD(ZXTIME,525960._JPRB)

IM1=0
IM2=0
IF (ZXTIME <= ZYTIME(1)) THEN
  IM1=12
  IM2=1
  ZTIMI=(ZYTIME(1)-ZXTIME)/44640._JPRB
ELSEIF(ZXTIME > ZYTIME(12)) THEN
  IM1=12
  IM2=1
  ZTIMI=(548280._JPRB-ZXTIME)/44640._JPRB
! 548280.=(365.25d + 15.5d)*24*60
ELSE
  DO IM=1,11
    IF (ZXTIME > ZYTIME(IM) .AND. ZXTIME <= ZYTIME(IM+1)) THEN
      IM1=IM
      IM2=IM+1
      ZTIMI=(ZXTIME-ZYTIME(IM2))/(ZYTIME(IM1)-ZYTIME(IM2))
    ENDIF
  ENDDO
  IF (IM1 == 0.OR. IM2 == 0 ) THEN
    CALL ABOR1('Problem with time interpolation in suecozc!')
  ENDIF
ENDIF

!*         2.0    TIME INTERPOLATED FIELD
!                 -----------------------

!*( Field is also transformed in kg/kg! )

DO JL=1,64
  DO JK=1,59
    ROZT(JL,JK)=RO3 * ZOZCL(JL,JK)
  ENDDO
  ROZT(JL, 0)=RO3 * 5.21105E-02_JPRB
  ROZT(JL,60)=ROZT(JL,59)
ENDDO
ZINCROZ=RPERTOZ
IF (RPERTOZ /= 0.0_JPRB) THEN
  DO JL=6,14
    ZINCL=SIN( FLOAT(JL-6)/4._JPRB*RPI/2._JPRB )
!-- perturbation of Ozone climatology above tropopause (10-100 hPa)
    IF (NPERTOZ == 0) THEN
      DO JK=23,29
        ZINCH=SIN( FLOAT(JK-23)/3.0_JPRB*RPI/2.0_JPRB )
        ROZT(JL,JK)=ROZT(JL,JK) * (1.0_JPRB + ZINCROZ*ZINCL*ZINCH)
      ENDDO
!-- perturbation of Ozone climatology below tropopause (50-500 hPa)
    ELSEIF (NPERTOZ == 1) THEN
      DO JK=27,33
        ZINCH=SIN( FLOAT(JK-27)/3.0_JPRB*RPI/2.0_JPRB )
        ROZT(JL,JK)=ROZT(JL,JK) * (1.0_JPRB + ZINCROZ*ZINCL*ZINCH)
      ENDDO
    ENDIF
  ENDDO
ENDIF


!     VECTOR OF LATITUDES FOR OZONE CLIMATOLOGY:

DO JL=1,64
  RSINC(JL)=SIN(ZLAT(JL)*RPI/180._JPRB)
ENDDO

LLFIRSTCALL = .FALSE.
IF (LHOOK) CALL DR_HOOK('SUECOZCAQUA',1,ZHOOK_HANDLE)
END SUBROUTINE SUECOZCAQUA
