#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SUECSO4 ( KINDAT, KMINUT )

!**** *SUECSO4* - DEFINES NEW CLIMATOLOGICAL DISTRIBUTION OF AEROSOLS

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SUECSO4* FROM *UPDTIM*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===
!     AEROSOL TOTAL OPTICAL THICKNESS (0.55 MICRON)

!-- O.BOUCHER MONTHLY/DECENNIAL CLIMATOLOGY OF TROPOSPHERIC SULPHATE AEROSOLS

! RTAESO4 : SULFATE AEROSOL


!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    98/12/21

!     MODIFICATIONS.
!     --------------
!     990115: JJMorcrette, Volcanic aerosols
!     990525: JJMorcrette, Monthly Climatology of Tropospheric Aerosols
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOEAERC   ,ONLY : RTAESO4

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KINDAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMINUT 
!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB),SAVE :: ZYTIME(12), ZMDAY(12) 

INTEGER(KIND=JPIM) :: IDY, IM, IM1, IM2, IMN, IYR, JI, JL, JM

REAL(KIND=JPRB) :: ZTIMI, ZXTIME
REAL(KIND=JPRB) :: ZDISYR
REAL(KIND=JPRB) :: ZAERSU(96,73,12), ZAER1(96,73,12), ZAER2(96,73,12)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

#include "abor1.intfb.h"

#include "su_so4_obs1920.intfb.h"
#include "su_so4_obs1930.intfb.h"
#include "su_so4_obs1940.intfb.h"
#include "su_so4_obs1950.intfb.h"
#include "su_so4_obs1960.intfb.h"
#include "su_so4_obs1970.intfb.h"
#include "su_so4_obs1980.intfb.h"
#include "su_so4_obs1990.intfb.h"

#include "su_so4_A1B2000.intfb.h"
#include "su_so4_A1B2010.intfb.h"
#include "su_so4_A1B2020.intfb.h"
#include "su_so4_A1B2030.intfb.h"
#include "su_so4_A1B2040.intfb.h"
#include "su_so4_A1B2050.intfb.h"
#include "su_so4_A1B2060.intfb.h"
#include "su_so4_A1B2070.intfb.h"
#include "su_so4_A1B2080.intfb.h"
#include "su_so4_A1B2090.intfb.h"
#include "su_so4_A1B2100.intfb.h"

#include "fcttim.h"      
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECSO4',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  CALL PART1

ENDIF
!     ------------------------------------------------------------------

!*         1.     TIME INDEX WITHIN AEROSOL CLIMATOLOGY
!                 -------------------------------------

IYR=NCCAA(KINDAT)
IMN=NMM(KINDAT)

IF (IYR < 1920) THEN
  CALL SU_SO4_OBS1920 ( ZAER1 )
  CALL SU_SO4_OBS1920 ( ZAER2 )
  ZDISYR=0._JPRB
ELSEIF (IYR >= 1920 .AND. IYR < 1930) THEN
  CALL SU_SO4_OBS1920 ( ZAER1 )
  CALL SU_SO4_OBS1930 ( ZAER2 )
  ZDISYR= (IYR - 1920) / 10._JPRB
ELSEIF (IYR >= 1930 .AND. IYR < 1940) THEN
  CALL SU_SO4_OBS1930 ( ZAER1 )
  CALL SU_SO4_OBS1940 ( ZAER2 )
  ZDISYR= (IYR - 1930) / 10._JPRB
ELSEIF (IYR >= 1940 .AND. IYR < 1950) THEN
  CALL SU_SO4_OBS1940 ( ZAER1 )
  CALL SU_SO4_OBS1950 ( ZAER2 )
  ZDISYR= (IYR - 1940) / 10._JPRB
ELSEIF (IYR >= 1950 .AND. IYR < 1960) THEN
  CALL SU_SO4_OBS1950 ( ZAER1 )
  CALL SU_SO4_OBS1960 ( ZAER2 )
  ZDISYR= (IYR - 1950) / 10._JPRB
ELSEIF (IYR >= 1960 .AND. IYR < 1970) THEN
  CALL SU_SO4_OBS1960 ( ZAER1 )
  CALL SU_SO4_OBS1970 ( ZAER2 )
  ZDISYR= (IYR - 1960) / 10._JPRB
ELSEIF (IYR >= 1970 .AND. IYR < 1980) THEN
  CALL SU_SO4_OBS1970 ( ZAER1 )
  CALL SU_SO4_OBS1980 ( ZAER2 )
  ZDISYR= (IYR - 1970) / 10._JPRB
ELSEIF (IYR >= 1980 .AND. IYR < 1990) THEN
  CALL SU_SO4_OBS1980 ( ZAER1 )
  CALL SU_SO4_OBS1990 ( ZAER2 )
  ZDISYR= (IYR - 1980) / 10._JPRB
ELSEIF (IYR >= 1990 .AND. IYR < 2000) THEN
  CALL SU_SO4_OBS1990 ( ZAER1 )
  CALL SU_SO4_A1B2000 ( ZAER2 )
  ZDISYR= (IYR - 1990) / 10._JPRB
ELSEIF (IYR >= 2000 .AND. IYR < 2010) THEN
  CALL SU_SO4_A1B2000 ( ZAER1 )
  CALL SU_SO4_A1B2010 ( ZAER2 )
  ZDISYR= (IYR - 2000) / 10._JPRB
ELSEIF (IYR >= 2010 .AND. IYR < 2020) THEN
  CALL SU_SO4_A1B2010 ( ZAER1 )
  CALL SU_SO4_A1B2020 ( ZAER2 )
  ZDISYR= (IYR - 2010) / 10._JPRB
ELSEIF (IYR >= 2020 .AND. IYR < 2030) THEN
  CALL SU_SO4_A1B2020 ( ZAER1 )
  CALL SU_SO4_A1B2030 ( ZAER2 )
  ZDISYR= (IYR - 2020) / 10._JPRB
ELSEIF (IYR >= 2030 .AND. IYR < 2040) THEN
  CALL SU_SO4_A1B2030 ( ZAER1 )
  CALL SU_SO4_A1B2040 ( ZAER2 )
  ZDISYR= (IYR - 2030) / 10._JPRB
ELSEIF (IYR >= 2040 .AND. IYR < 2050) THEN
  CALL SU_SO4_A1B2040 ( ZAER1 )
  CALL SU_SO4_A1B2050 ( ZAER2 )
  ZDISYR= (IYR - 2040) / 10._JPRB
ELSEIF (IYR >= 2050 .AND. IYR < 2060) THEN
  CALL SU_SO4_A1B2050 ( ZAER1 )
  CALL SU_SO4_A1B2060 ( ZAER2 )
  ZDISYR= (IYR - 2050) / 10._JPRB
ELSEIF (IYR >= 2060 .AND. IYR < 2070) THEN
  CALL SU_SO4_A1B2060 ( ZAER1 )
  CALL SU_SO4_A1B2070 ( ZAER2 )
  ZDISYR= (IYR - 2060) / 10._JPRB
ELSEIF (IYR >= 2070 .AND. IYR < 2080) THEN
  CALL SU_SO4_A1B2070 ( ZAER1 )
  CALL SU_SO4_A1B2080 ( ZAER2 )
  ZDISYR= (IYR - 2070) / 10._JPRB
ELSEIF (IYR >= 2080 .AND. IYR < 2090) THEN
  CALL SU_SO4_A1B2080 ( ZAER1 )
  CALL SU_SO4_A1B2090 ( ZAER2 )
  ZDISYR= (IYR - 2080) / 10._JPRB
ELSEIF (IYR >= 2090 .AND. IYR < 2100) THEN
  CALL SU_SO4_A1B2090 ( ZAER1 )
  CALL SU_SO4_A1B2100 ( ZAER2 )
  ZDISYR= (IYR - 2090) / 10._JPRB
ELSEIF (IYR >= 2100) THEN
  CALL SU_SO4_A1B2100 ( ZAER1 )
  CALL SU_SO4_A1B2100 ( ZAER2 )
  ZDISYR=0._JPRB
ENDIF

DO JM=1,12
  DO JI=1,73
    DO JL=1,96
      ZAERSU(JL,JI,JM)=(1._JPRB-ZDISYR)*ZAER1(JL,JI,JM)+ZDISYR*ZAER2(JL,JI,JM)
    ENDDO
  ENDDO
ENDDO

!*         1.1    SEASONAL TIME INDEX WITHIN O.BOUCHER'S AEROSOL CLIMATOLOGY
!                 ----------------------------------------------------------

IDY=NDD(KINDAT)-1
IMN=NMM(KINDAT)
IF (IMN == 1) THEN
  ZXTIME=REAL(IDY)*1440._JPRB + KMINUT
ELSEIF (IMN == 2) THEN
  IF(IDY == 28) IDY=IDY-1
! A DAY IN FEB. IS 28.25*24*60/28=1452.8571min LONG.
  ZXTIME=44640._JPRB+REAL(IDY)*1452.8571_JPRB+KMINUT
ELSE
  ZXTIME=(ZMDAY(IMN-1)+REAL(IDY))*1440._JPRB+KMINUT
ENDIF
! 525960=MINUTES IN A SIDERAL YEAR (365.25d)
ZXTIME=MOD(ZXTIME,525960._JPRB)

IM1=0
IM2=0
IF (ZXTIME <= ZYTIME(1)) THEN
  IM1=12
  IM2=1
  ZTIMI=(ZYTIME(1)-ZXTIME)/44640._JPRB
ELSEIF(ZXTIME > ZYTIME(12)) THEN
  IM1=12
  IM2=1
  ZTIMI=(548280._JPRB-ZXTIME)/44640._JPRB
! 548280.=(365.25d + 15.5d)*24*60
ELSE
  DO IM=1,11
    IF (ZXTIME > ZYTIME(IM) .AND. ZXTIME <= ZYTIME(IM+1)) THEN
      IM1=IM
      IM2=IM+1
      ZTIMI=(ZXTIME-ZYTIME(IM2))/(ZYTIME(IM1)-ZYTIME(IM2))
    ENDIF
  ENDDO
  IF (IM1 == 0.OR. IM2 == 0 ) THEN
    CALL ABOR1('Problem with time interpolation in suecso4!')
  ENDIF
ENDIF

!*         2.0    TOTAL OPTICAL THICKNESS
!                 -----------------------

!*         2.1    TROPOSPHERIC AEROSOL MONTHLY CLIMATOLOGY
!                 ----------------------------------------

!-- VECTOR OF LATITUDES FOR O.BOUCHER'S SULPHATE AEROSOL CLIMATOLOGY:

DO JI=1,73
  DO JL=1,96
    RTAESO4(JL,JI)=ZAERSU(JL,JI,IM2)&
     & +ZTIMI*(ZAERSU(JL,JI,IM1)-ZAERSU(JL,JI,IM2))  
  ENDDO
ENDDO

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SUECSO4',1,ZHOOK_HANDLE)
!-----------------------------------------------------------------------------

CONTAINS


SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SUECSO4:PART1',0,ZHOOK_HANDLE)


ZMDAY = (/&
 & 31._JPRB,    59.25_JPRB,  90.25_JPRB, 120.25_JPRB, 151.25_JPRB, 181.25_JPRB &
 & ,  212.25_JPRB, 243.25_JPRB, 273.25_JPRB, 304.25_JPRB, 334.25_JPRB, 365.25_JPRB &
 & /)  
ZYTIME= (/&
 & 22320._JPRB,  64980._JPRB, 107640._JPRB, 151560._JPRB, 195480._JPRB, 239400._JPRB &
 & , 283320._JPRB, 327960._JPRB, 371880._JPRB, 415800._JPRB, 459720._JPRB, 503640._JPRB &
 & /)  

!-- the ALF_SU, ASY_SU, OMG_SU and RRHTAB are already part of YOEAEROP

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUECSO4:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1

!-----------------------------------------------------------------------------
END SUBROUTINE SUECSO4
