SUBROUTINE SURADDIAG

  USE PARKIND1, ONLY: JPIM, JPRB
  USE YOMHOOK, ONLY: LHOOK, DR_HOOK
  USE PHY_DIAG_MOD, ONLY: NEW_DIAG_SET, NEW_DIAGNOSTIC
  USE YOERAD, ONLY: L2RAD, L2RADACCU, L2RADTEND
  USE YOERADDIAG
  USE YOM_GRIB_CODES, ONLY: &
    & NGRB_SW_CS_TOA_CLN, NGRB_SW_ALL_TOA_CLN, NGRB_SW_CS_SRF_CLN, NGRB_SW_ALL_SRF_CLN, &
    & NGRB_LW_CS_TOA_CLN, NGRB_LW_ALL_TOA_CLN, NGRB_LW_CS_SRF_CLN, NGRB_LW_ALL_SRF_CLN, &
    & NGRB_HR, NGRB_HR_CLN

  IMPLICIT NONE

  INTEGER(KIND=JPIM) :: ISET
  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  IF (LHOOK) CALL DR_HOOK('SURADDIAG',0,ZHOOK_HANDLE)

   IF (L2RAD .OR. L2RADTEND) THEN

     CALL NEW_DIAG_SET('Double call', ISET)

   END IF

   IF (L2RAD) THEN
     ! Double-call diagnostics have been moved to CALLPAR, so LRAD is now set to 
     ! .FALSE.
     CALL NEW_DIAGNOSTIC(ISET, 'SW clear-sky ToA no aerosol', 2, NGRB_SW_CS_TOA_CLN, & 
                      &  L2RADACCU, .FALSE., D_SW_CS_TOA_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'SW all-sky ToA no aerosol', 2, NGRB_SW_ALL_TOA_CLN, & 
                      &  L2RADACCU, .FALSE., D_SW_ALL_TOA_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'SW clear-sky at surface no aerosol', 2, NGRB_SW_CS_SRF_CLN, &
                      &  L2RADACCU, .FALSE., D_SW_CS_SRF_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'SW all-sky at surface no aerosol', 2, NGRB_SW_ALL_SRF_CLN, & 
                      &  L2RADACCU, .FALSE., D_SW_ALL_SRF_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'LW clear-sky ToA no aerosol', 2, NGRB_LW_CS_TOA_CLN, &
                      &  L2RADACCU, .FALSE., D_LW_CS_TOA_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'LW all-sky ToA no aerosol', 2, NGRB_LW_ALL_TOA_CLN, &
                      &  L2RADACCU, .FALSE., D_LW_ALL_TOA_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'LW clear-sky at surface no aerosol', 2, NGRB_LW_CS_SRF_CLN, &
                      &  L2RADACCU, .FALSE., D_LW_CS_SRF_CLN)

     CALL NEW_DIAGNOSTIC(ISET, 'LW all-sky at surface no aerosol', 2, NGRB_LW_ALL_SRF_CLN, &
                      &  L2RADACCU, .FALSE., D_LW_ALL_SRF_CLN)

     ! Additional for CMIP6 data request...not too clear why the data request is for the clean case only, 
     ! so added diagnostics with aerosols, too
     !CALL NEW_DIAGNOSTIC(ISET, 'SW upwelling flux at ToA, all-sky, no aerosol', 2, NGRB_SW_UPW_ALL_TOA_CLN, &
     !                 &  L2RADACCU, .FALSE., D_SW_UPW_ALL_TOA_CLN)

     !CALL NEW_DIAGNOSTIC(ISET, 'SW upwelling flux at ToA, clear-sky, no aerosol', 2, NGRB_SW_UPW_CS_TOA_CLN, &
     !                 &  L2RADACCU, .FALSE., D_SW_UPW_CS_TOA_CLN)

     !CALL NEW_DIAGNOSTIC(ISET, 'SW downwelling flux at surface, clear-sky, no aerosol', 2, NGRB_SW_DOWNW_CS_SRF_CLN, &
     !                 &  L2RADACCU, .FALSE., D_SW_DOWNW_CS_SRF_CLN)

     !CALL NEW_DIAGNOSTIC(ISET, 'SW upwelling flux at ToA, all-sky', 2, NGRB_SW_UPW_ALL_TOA, &
     !                 &  L2RADACCU, .FALSE., D_SW_UPW_ALL_TOA)

     !CALL NEW_DIAGNOSTIC(ISET, 'SW upwelling flux at ToA, clear-sky', 2, NGRB_SW_UPW_CS_TOA, &
     !                 &  L2RADACCU, .FALSE., D_SW_UPW_CS_TOA)

     !CALL NEW_DIAGNOSTIC(ISET, 'SW downwelling flux at surface, clear-sky', 2, NGRB_SW_DOWNW_CS_SRF, &
     !                 &  L2RADACCU, .FALSE., D_SW_DOWNW_CS_SRF)
  END IF

  IF (L2RADTEND) THEN
     ! NB These are implemented in callpar.F90, so F here.
     CALL NEW_DIAGNOSTIC(ISET, 'Radiative heating rate', 3, NGRB_HR, &
                      &  L2RADACCU, .FALSE., D_RAD_HR)

     CALL NEW_DIAGNOSTIC(ISET, 'Radiative heating rate no aerosol', 3, NGRB_HR_CLN, &
                      &  L2RADACCU, .FALSE., D_RAD_HR_CLN)
  END IF

  IF (LHOOK) CALL DR_HOOK('SURADIAG',1,ZHOOK_HANDLE)
END SUBROUTINE SURADDIAG

