SUBROUTINE SW1SAD &
 & ( KIDIA  , KFDIA  , KLON  , KLEV  , KAER  , KNU,&
 & PAER5  , PALBD5 , PALBP5, PCG5  , PCLD5 , PCLEAR5,&
 & PDSIG5 , POMEGA5, POZ5  , PRMU5 , PSEC5 , PTAU5  , PUD5,&
 & PFD5   , PFU5   , PCD5  , PCU5,&
 & PAER  , PALBD , PALBP, PCG  , PCLD , PCLEAR,&
 & PDSIG , POMEGA, POZ  , PRMU , PSEC , PTAU  , PUD,&
 & PFD   , PFU   , PCD  , PCU &
 & )  

!**** *SW1SAD* - SHORTWAVE RADIATION, FIRST SPECTRAL INTERVAL
!                -> ADJOINT <-

!     PURPOSE.
!     --------

!          THIS ROUTINE COMPUTES THE SHORTWAVE RADIATION FLUXES IN TWO
!     SPECTRAL INTERVALS FOLLOWING FOUQUART AND BONNEL (1980).

!**   INTERFACE.
!     ----------

!          *SW1SAD* IS CALLED FROM *SWAD*.

!        IMPLICIT ARGUMENTS :
!        --------------------

!     ==== INPUTS ===
!     ==== OUTPUTS ===

!     METHOD.
!     -------

!          1. COMPUTES UPWARD AND DOWNWARD FLUXES CORRESPONDING TO
!     CONTINUUM SCATTERING
!          2. MULTIPLY BY OZONE TRANSMISSION FUNCTION

!     EXTERNALS.
!     ----------

!          *SWCLR*  , *SWR*  , *SWTT*, *SWUVO3*,
!          *SWCLRAD*, *SWRAD*, *SWTTAD*, *SWUVOAD3*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT
!        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-10-21
!        Modified : 99-05-17 M.Janiskova - code correction
!                 2000-02-24 M.Janiskova - adopted for revised aerosols
!                                          as in sw1s
!                 2002-05-07 M.Janiskova - adopted for 6 spectral intervals
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!                 2008-03-18 M.Janiskova - code optimization
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOESW    , ONLY : RRAY     ,RSUN
USE YOERAD   , ONLY : NSW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER5(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCG5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLEAR5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSIG5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMU5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUD5(KLON,5,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAER(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCG(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLEAR(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDSIG(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POMEGA(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSEC(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAU(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUD(KLON,5,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCU(KLON,KLEV+1) 
!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!     ------------------------------------------------------------------

!              ------------

INTEGER(KIND=JPIM) :: IIND(6)

REAL(KIND=JPRB) :: ZCGAZ5(KLON,KLEV)&
 & ,  ZPIZAZ5(KLON,KLEV)&
 & ,  ZRAYL5(KLON)&
 & ,  ZRAY15(KLON,KLEV+1)  , ZRAY015(KLON,KLEV+1)&
 & ,  ZRAY25(KLON,KLEV+1)  , ZRAY025(KLON,KLEV+1)&
 & ,  ZREFZ5(KLON,2,KLEV+1), ZREFZ05(KLON,2,KLEV+1)&
 & ,  ZRJ5(KLON,6,KLEV+1)  , ZRJ05(KLON,6,KLEV+1)&
 & ,  ZRK5(KLON,6,KLEV+1)  , ZRK05(KLON,6,KLEV+1)&
 & ,  ZRMUE5(KLON,KLEV+1)  , ZRMU05(KLON,KLEV+1)&
 & ,  ZTAUAZ5(KLON,KLEV)&
 & ,  ZTRA15(KLON,KLEV+1)  , ZTRA015(KLON,KLEV+1)&
 & ,  ZTRA25(KLON,KLEV+1)  , ZTRA025(KLON,KLEV+1)&
 & ,  ZTRCLD5(KLON)        , ZTRCLR5(KLON)&
 & ,  ZDUMMY(KLON,KLEV)  

REAL(KIND=JPRB) ::&
 & ZDW5(KLON,6,KLEV+1)  , ZDR5(KLON,6,KLEV+1) &
 & ,  ZUW5(KLON,6,KLEV+1)  , ZUR5(KLON,6,KLEV+1) &
 & ,  ZDO5(KLON,2,KLEV+1)  , ZDT5(KLON,2,KLEV+1) &
 & ,  ZUO5(KLON,2,KLEV+1)  , ZUT5(KLON,2,KLEV+1) &
 & ,  ZDDIFF5(KLON,KLEV+1) , ZDDIRF5(KLON,KLEV+1)&
 & ,  ZUDIFF5(KLON,KLEV+1) , ZUDIRF5(KLON,KLEV+1)  
REAL(KIND=JPRB) :: ZRE5(KLON,KLEV), ZR05(KLON,KLEV)

REAL(KIND=JPRB) :: ZCGAZ(KLON,KLEV)&
 & ,  ZDIFF(KLON)          , ZDIRF(KLON)        &
 & ,  ZPIZAZ(KLON,KLEV)&
 & ,  ZRAYL(KLON)&
 & ,  ZRAY1(KLON,KLEV+1)   , ZRAY01(KLON,KLEV+1)&
 & ,  ZRAY2(KLON,KLEV+1)   , ZRAY02(KLON,KLEV+1)&
 & ,  ZREFZ(KLON,2,KLEV+1) , ZREFZ0(KLON,2,KLEV+1)&
 & ,  ZRJ(KLON,6,KLEV+1)   , ZRJ0(KLON,6,KLEV+1)&
 & ,  ZRK(KLON,6,KLEV+1)   , ZRK0(KLON,6,KLEV+1)&
 & ,  ZRMUE(KLON,KLEV+1)   , ZRMU0(KLON,KLEV+1)&
 & ,  ZR(KLON,6)&
 & ,  ZTAUAZ(KLON,KLEV)&
 & ,  ZTRA1(KLON,KLEV+1)   , ZTRA01(KLON,KLEV+1)&
 & ,  ZTRA2(KLON,KLEV+1)   , ZTRA02(KLON,KLEV+1)&
 & ,  ZW(KLON,6), ZO(KLON,2), ZT(KLON,2)
REAL(KIND=JPRB) :: ZRE, ZR0

! fields for adjoint computation
REAL(KIND=JPRB) :: ZRESWR15(KLON,KLEV), ZRESWR25(KLON,KLEV)
REAL(KIND=JPRB) :: ZTRSWR15(KLON,KLEV), ZTRSWR25(KLON,KLEV)

INTEGER(KIND=JPIM) :: IKL, IKM1, JAJ, JK, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "swclr.intfb.h"
#include "swclrad.intfb.h"
#include "swr.intfb.h"
#include "swrad.intfb.h"
#include "swtt1.intfb.h"
#include "swtt1ad.intfb.h"
#include "swuvo3.intfb.h"
#include "swuvo3ad.intfb.h"

!     ------------------------------------------------------------------

!*         1.     FIRST SPECTRAL INTERVAL (0.25-0.68 MICRON)
!                 ----------------------- ------------------

! 100 CONTINUE

!*         1.1    OPTICAL THICKNESS FOR RAYLEIGH SCATTERING
!                 -----------------------------------------

! 110 CONTINUE

IF (LHOOK) CALL DR_HOOK('SW1SAD',0,ZHOOK_HANDLE)
DO JL = KIDIA,KFDIA
  ZRAYL5(JL) =  RRAY(KNU,1) + PRMU5(JL) * (RRAY(KNU,2) + PRMU5(JL)&
   & * (RRAY(KNU,3) + PRMU5(JL) * (RRAY(KNU,4) + PRMU5(JL)&
   & * (RRAY(KNU,5) + PRMU5(JL) *  RRAY(KNU,6)        ))))  
ENDDO

!     ------------------------------------------------------------------

!*         2.    CONTINUUM SCATTERING CALCULATIONS
!                ---------------------------------

! 200 CONTINUE

!*         2.1   CLEAR-SKY FRACTION OF THE COLUMN
!                --------------------------------

! 210 CONTINUE

CALL SWCLR &
 & ( KIDIA , KFDIA  , KLON   , KLEV   , KAER   , KNU,&
 & PAER5 , PALBP5 , PDSIG5 , ZRAYL5 , PSEC5,&
 & ZCGAZ5, ZPIZAZ5, ZRAY015, ZRAY025, ZREFZ05, ZRJ05,&
 & ZRK05 , ZRMU05 , ZTAUAZ5, ZTRA015, ZTRA025, ZTRCLR5, &
 & .FALSE.,ZDUMMY,ZDUMMY,ZDUMMY)  

!*         2.2   CLOUDY FRACTION OF THE COLUMN
!                -----------------------------

! 220 CONTINUE

CALL SWR &
 & ( KIDIA  ,KFDIA  ,KLON   ,KLEV   , KNU,&
 & PALBD5 ,PCG5   ,PCLD5  ,POMEGA5, PSEC5 ,PTAU5,&
 & ZCGAZ5 ,ZPIZAZ5,ZRAY15 ,ZRAY25 ,ZREFZ5 , ZRJ5  ,ZRK5 , ZRMUE5,&
 & ZTAUAZ5,ZTRA15 ,ZTRA25 ,ZTRCLD5,&
 & ZRESWR15,ZRESWR25,ZTRSWR15,ZTRSWR25 &
 & )  

!     ------------------------------------------------------------------

!*         3.    OZONE ABSORPTION
!                ----------------

! 300 CONTINUE

IF (NSW <= 4) THEN

!*         3.1   TWO OR FOUR SPECTRAL INTERVALS
!                ------------------------------

  IIND(1)=1
  IIND(2)=2
  IIND(3)=3
  IIND(4)=1
  IIND(5)=2
  IIND(6)=3

!*         3.1.1  DOWNWARD FLUXES
!                 ---------------

! 310 CONTINUE

  JAJ = 2

  DO JL = KIDIA,KFDIA
    ZDW5(JL,1,KLEV+1)=0.0_JPRB
    ZDW5(JL,2,KLEV+1)=0.0_JPRB
    ZDW5(JL,3,KLEV+1)=0.0_JPRB
    ZDW5(JL,4,KLEV+1)=0.0_JPRB
    ZDW5(JL,5,KLEV+1)=0.0_JPRB
    ZDW5(JL,6,KLEV+1)=0.0_JPRB
    PFD5(JL,KLEV+1)=((1.0_JPRB-PCLEAR5(JL))*ZRJ5(JL,JAJ,KLEV+1)&
     & + PCLEAR5(JL) *ZRJ05(JL,JAJ,KLEV+1)) * RSUN(KNU)  
    PCD5(JL,KLEV+1)= ZRJ05(JL,JAJ,KLEV+1) * RSUN(KNU)
  ENDDO
  DO JK = 1 , KLEV
    IKL = KLEV+1-JK
    IKM1 = IKL+1
    DO JL = KIDIA,KFDIA
      ZRE5(JL,IKL) = 1.0_JPRB/ZRMUE5(JL,IKL)
      ZR05(JL,IKL) = 1.0_JPRB/ZRMU05(JL,IKL)

      ZDW5(JL,1,IKL)=ZDW5(JL,1,IKM1)+PUD5(JL,1,IKL)*ZRE5(JL,IKL)
      ZDW5(JL,2,IKL)=ZDW5(JL,2,IKM1)+PUD5(JL,2,IKL)*ZRE5(JL,IKL)
      ZDW5(JL,3,IKL)=ZDW5(JL,3,IKM1)+POZ5(JL,  IKL)*ZRE5(JL,IKL)
      ZDW5(JL,4,IKL)=ZDW5(JL,4,IKM1)+PUD5(JL,1,IKL)*ZR05(JL,IKL)
      ZDW5(JL,5,IKL)=ZDW5(JL,5,IKM1)+PUD5(JL,2,IKL)*ZR05(JL,IKL)
      ZDW5(JL,6,IKL)=ZDW5(JL,6,IKM1)+POZ5(JL,  IKL)*ZR05(JL,IKL)
    ENDDO

    CALL SWTT1( KIDIA, KFDIA, KLON, KNU, 6, IIND,&
     & ZDW5(1,1,IKL), ZDR5(1,1,IKL) )

    DO JL = KIDIA,KFDIA
      ZDDIFF5(JL,IKL) = ZDR5(JL,1,IKL)*ZDR5(JL,2,IKL)*ZDR5(JL,3,IKL) &
       & *ZRJ5(JL,JAJ,IKL)
      ZDDIRF5(JL,IKL) = ZDR5(JL,4,IKL)*ZDR5(JL,5,IKL)*ZDR5(JL,6,IKL) &
       & *ZRJ05(JL,JAJ,IKL)

      PFD5(JL,IKL) = ((1.0_JPRB-PCLEAR5(JL)) * ZDDIFF5(JL,IKL)&
       & +PCLEAR5(JL)  * ZDDIRF5(JL,IKL)) * RSUN(KNU)  
      PCD5(JL,IKL) = ZDDIRF5(JL,IKL) * RSUN(KNU)
    ENDDO
  ENDDO

!*         3.2.1  UPWARD FLUXES
!                 -------------

! 320 CONTINUE

  DO JL = KIDIA,KFDIA
    PFU5(JL,1) = ((1.0_JPRB-PCLEAR5(JL))*ZDDIFF5(JL,1)*PALBD5(JL,KNU)&
     & + PCLEAR5(JL) *ZDDIRF5(JL,1)*PALBP5(JL,KNU))&
     & * RSUN(KNU)  
    PCU5(JL,1) = ZDDIRF5(JL,1) * PALBP5(JL,KNU) * RSUN(KNU)
  ENDDO

  DO JL = KIDIA,KFDIA
    ZUW5(JL,1,1) = ZDW5(JL,1,1)
    ZUW5(JL,2,1) = ZDW5(JL,2,1) 
    ZUW5(JL,3,1) = ZDW5(JL,3,1)
    ZUW5(JL,4,1) = ZDW5(JL,4,1)
    ZUW5(JL,5,1) = ZDW5(JL,5,1)
    ZUW5(JL,6,1) = ZDW5(JL,6,1)
  ENDDO

  DO JK = 2 , KLEV+1
    IKM1=JK-1
    DO JL = KIDIA,KFDIA
      ZUW5(JL,1,JK)=ZUW5(JL,1,IKM1)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZUW5(JL,2,JK)=ZUW5(JL,2,IKM1)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZUW5(JL,3,JK)=ZUW5(JL,3,IKM1)+POZ5(JL,  IKM1)*1.66_JPRB
      ZUW5(JL,4,JK)=ZUW5(JL,4,IKM1)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZUW5(JL,5,JK)=ZUW5(JL,5,IKM1)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZUW5(JL,6,JK)=ZUW5(JL,6,IKM1)+POZ5(JL,  IKM1)*1.66_JPRB
    ENDDO

    CALL SWTT1( KIDIA, KFDIA, KLON, KNU, 6, IIND,&
     & ZUW5(1,1,JK), ZUR5(1,1,JK) )

    DO JL = KIDIA,KFDIA
      ZUDIFF5(JL,JK) = ZUR5(JL,1,JK)*ZUR5(JL,2,JK)*ZUR5(JL,3,JK) &
       & *ZRK5(JL,JAJ,JK)
      ZUDIRF5(JL,JK) = ZUR5(JL,4,JK)*ZUR5(JL,5,JK)*ZUR5(JL,6,JK) &
       & *ZRK05(JL,JAJ,JK)
    
      PFU5(JL,JK) = ((1.0_JPRB-PCLEAR5(JL)) * ZUDIFF5(JL,JK)&
       & +PCLEAR5(JL)  * ZUDIRF5(JL,JK)) * RSUN(KNU)  
      PCU5(JL,JK) = ZUDIRF5(JL,JK) * RSUN(KNU)
    ENDDO
  ENDDO
  
ELSEIF (NSW == 6) THEN

!*         3.2   SIX SPECTRAL INTERVALS
!                ----------------------

  IIND(1)=1
  IIND(2)=2
  IIND(3)=1
  IIND(4)=2

!*         3.2.1  DOWNWARD FLUXES
!                 ---------------

  JAJ = 2

  DO JL = KIDIA,KFDIA
    ZDW5(JL,1,KLEV+1)=0.0_JPRB
    ZDW5(JL,2,KLEV+1)=0.0_JPRB
    ZDW5(JL,3,KLEV+1)=0.0_JPRB
    ZDW5(JL,4,KLEV+1)=0.0_JPRB
    ZDO5(JL,1,KLEV+1)=0.0_JPRB
    ZDO5(JL,2,KLEV+1)=0.0_JPRB
    PFD5(JL,KLEV+1)=((1.0_JPRB-PCLEAR5(JL))*ZRJ5(JL,JAJ,KLEV+1)&
     & + PCLEAR5(JL) *ZRJ05(JL,JAJ,KLEV+1)) * RSUN(KNU)  
    PCD5(JL,KLEV+1)= ZRJ05(JL,JAJ,KLEV+1) * RSUN(KNU)
  ENDDO

  DO JK = 1 , KLEV
    IKL = KLEV+1-JK
    IKM1 = IKL+1
    DO JL = KIDIA,KFDIA
      ZRE5(JL,IKL) = 1.0_JPRB/ZRMUE5(JL,IKL)
      ZR05(JL,IKL) = 1.0_JPRB/ZRMU05(JL,IKL)

      ZDW5(JL,1,IKL)=ZDW5(JL,1,IKM1)+PUD5(JL,1,IKL)*ZRE5(JL,IKL)
      ZDW5(JL,2,IKL)=ZDW5(JL,2,IKM1)+PUD5(JL,2,IKL)*ZRE5(JL,IKL)
      ZDW5(JL,3,IKL)=ZDW5(JL,3,IKM1)+PUD5(JL,1,IKL)*ZR05(JL,IKL)
      ZDW5(JL,4,IKL)=ZDW5(JL,4,IKM1)+PUD5(JL,2,IKL)*ZR05(JL,IKL)
      
      ZDO5(JL,1,IKL)=ZDO5(JL,1,IKM1)+POZ5(JL,  IKL)*ZRE5(JL,IKL)
      ZDO5(JL,2,IKL)=ZDO5(JL,2,IKM1)+POZ5(JL,  IKL)*ZR05(JL,IKL)
    ENDDO

    CALL SWTT1( KIDIA, KFDIA, KLON, KNU, 4, IIND,&
     & ZDW5(1,1,IKL), ZDR5(1,1,IKL) )
    
    CALL SWUVO3 ( KIDIA, KFDIA, KLON, KNU, 2,&
     & ZDO5(1,1,IKL), ZDT5(1,1,IKL) )
    
    DO JL = KIDIA,KFDIA
      ZDDIFF5(JL,IKL) = ZDR5(JL,1,IKL)*ZDR5(JL,2,IKL)*ZDT5(JL,1,IKL) &
       & *ZRJ5(JL,JAJ,IKL)
      ZDDIRF5(JL,IKL) = ZDR5(JL,3,IKL)*ZDR5(JL,4,IKL)*ZDT5(JL,2,IKL) &
       & *ZRJ05(JL,JAJ,IKL)

      PFD5(JL,IKL) = ((1.0_JPRB-PCLEAR5(JL)) * ZDDIFF5(JL,IKL)&
       & +PCLEAR5(JL)  * ZDDIRF5(JL,IKL)) * RSUN(KNU)  
      PCD5(JL,IKL) = ZDDIRF5(JL,IKL) * RSUN(KNU)
    ENDDO
  ENDDO

!*         3.2.2  UPWARD FLUXES
!                 -------------

  DO JL = KIDIA,KFDIA
    PFU5(JL,1) = ((1.0_JPRB-PCLEAR5(JL))*ZDDIFF5(JL,1)*PALBD5(JL,KNU)&
     & + PCLEAR5(JL) *ZDDIRF5(JL,1)*PALBP5(JL,KNU))&
     & * RSUN(KNU)  
    PCU5(JL,1) = ZDDIRF5(JL,1) * PALBP5(JL,KNU) * RSUN(KNU)
  ENDDO

  DO JL = KIDIA,KFDIA
    ZUW5(JL,1,1) = ZDW5(JL,1,1)
    ZUW5(JL,2,1) = ZDW5(JL,2,1) 
    ZUW5(JL,3,1) = ZDW5(JL,3,1)
    ZUW5(JL,4,1) = ZDW5(JL,4,1)
    ZUO5(JL,1,1) = ZDO5(JL,1,1)
    ZUO5(JL,2,1) = ZDO5(JL,2,1)
  ENDDO

  DO JK = 2 , KLEV+1
    IKM1=JK-1
    DO JL = KIDIA,KFDIA
      ZUW5(JL,1,JK)=ZUW5(JL,1,IKM1)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZUW5(JL,2,JK)=ZUW5(JL,2,IKM1)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZUW5(JL,3,JK)=ZUW5(JL,3,IKM1)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZUW5(JL,4,JK)=ZUW5(JL,4,IKM1)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZUO5(JL,1,JK)=ZUO5(JL,1,IKM1)+POZ5(JL,  IKM1)*1.66_JPRB
      ZUO5(JL,2,JK)=ZUO5(JL,2,IKM1)+POZ5(JL,  IKM1)*1.66_JPRB
    ENDDO

    CALL SWTT1( KIDIA, KFDIA, KLON, KNU, 4, IIND,&
     & ZUW5(1,1,JK), ZUR5(1,1,JK) )
    
    CALL SWUVO3 ( KIDIA, KFDIA, KLON, KNU, 2,&
     & ZUO5(1,1,JK), ZUT5(1,1,JK) )

    DO JL = KIDIA,KFDIA
      ZUDIFF5(JL,JK) = ZUR5(JL,1,JK)*ZUR5(JL,2,JK)*ZUT5(JL,1,JK) &
       & *ZRK5(JL,JAJ,JK)
      ZUDIRF5(JL,JK) = ZUR5(JL,3,JK)*ZUR5(JL,4,JK)*ZUT5(JL,2,JK) &
       & *ZRK05(JL,JAJ,JK)

      PFU5(JL,JK) = ((1.0_JPRB-PCLEAR5(JL)) * ZUDIFF5(JL,JK)&
       & +PCLEAR5(JL)  * ZUDIRF5(JL,JK)) * RSUN(KNU)  
      PCU5(JL,JK) = ZUDIRF5(JL,JK) * RSUN(KNU)
    ENDDO
  ENDDO

ENDIF

!     ==================================================================

!          10.     ADJOINT CALCULATIONS
!                  --------------------

!1000 CONTINUE

ZRAYL(:)    = 0.0_JPRB
ZDIRF(:)    = 0.0_JPRB
ZDIFF(:)    = 0.0_JPRB

ZR(:,:)     = 0.0_JPRB
ZW(:,:)     = 0.0_JPRB
ZT(:,:)     = 0.0_JPRB
ZO(:,:)     = 0.0_JPRB
ZRMU0(:,:)  = 0.0_JPRB
ZRMUE(:,:)  = 0.0_JPRB
ZCGAZ(:,:)  = 0.0_JPRB
ZPIZAZ(:,:) = 0.0_JPRB
ZRAY1(:,:)  = 0.0_JPRB
ZRAY2(:,:)  = 0.0_JPRB
ZTAUAZ(:,:) = 0.0_JPRB
ZTRA1(:,:)  = 0.0_JPRB
ZTRA2(:,:)  = 0.0_JPRB
ZRAY01(:,:) = 0.0_JPRB
ZRAY02(:,:) = 0.0_JPRB
ZTRA01(:,:) = 0.0_JPRB
ZTRA02(:,:) = 0.0_JPRB

ZRJ(:,:,:)  = 0.0_JPRB
ZRJ0(:,:,:) = 0.0_JPRB
ZRK(:,:,:)  = 0.0_JPRB
ZRK0(:,:,:) = 0.0_JPRB
ZREFZ(:,:,:)= 0.0_JPRB
ZREFZ0(:,:,:)= 0.0_JPRB

JAJ = 2

IF (NSW <= 4) THEN

!*         3.1   TWO OR FOUR SPECTRAL INTERVALS
!                ------------------------------

  IIND(1)=1
  IIND(2)=2
  IIND(3)=3
  IIND(4)=1
  IIND(5)=2
  IIND(6)=3

  DO JK=KLEV+1,2,-1
    IKM1 = JK-1

    DO JL=KIDIA,KFDIA
      ZDIRF(JL)  = ZDIRF(JL) + PCU(JL,JK) * RSUN(KNU)
      PCU(JL,JK) = 0.0_JPRB
 
      PCLEAR(JL) = PCLEAR(JL) - PFU(JL,JK) *ZUDIFF5(JL,JK)*RSUN(KNU)
      ZDIFF(JL)  = ZDIFF(JL) + PFU(JL,JK) *(1.0_JPRB-PCLEAR5(JL))*RSUN(KNU)
      PCLEAR(JL) = PCLEAR(JL) + PFU(JL,JK) *ZUDIRF5(JL,JK)*RSUN(KNU)
      ZDIRF(JL)  = ZDIRF(JL) + PFU(JL,JK) *PCLEAR5(JL)*RSUN(KNU)
      PFU(JL,JK) = 0.0_JPRB

      ZR(JL,4)   = ZR(JL,4)&
       & + ZDIRF(JL) *ZUR5(JL,5,JK)*ZUR5(JL,6,JK)*ZRK05(JL,JAJ,JK)  
      ZR(JL,5)   = ZR(JL,5)&
       & + ZDIRF(JL) *ZUR5(JL,4,JK)*ZUR5(JL,6,JK)*ZRK05(JL,JAJ,JK)  
      ZR(JL,6)   = ZR(JL,6)&
       & + ZDIRF(JL) *ZUR5(JL,4,JK)*ZUR5(JL,5,JK)*ZRK05(JL,JAJ,JK)  
      ZRK0(JL,JAJ,JK) =  ZRK0(JL,JAJ,JK)&
       & + ZDIRF(JL) *ZUR5(JL,4,JK)*ZUR5(JL,5,JK)*ZUR5(JL,6,JK)  
      ZDIRF(JL)  = 0.0_JPRB

      ZR(JL,1)   = ZR(JL,1)&
       & + ZDIFF(JL) *ZUR5(JL,2,JK)*ZUR5(JL,3,JK)*ZRK5(JL,JAJ,JK)  
      ZR(JL,2)   = ZR(JL,2)&
       & + ZDIFF(JL) *ZUR5(JL,1,JK)*ZUR5(JL,3,JK)*ZRK5(JL,JAJ,JK)  
      ZR(JL,3)   = ZR(JL,3)&
       & + ZDIFF(JL) *ZUR5(JL,1,JK)*ZUR5(JL,2,JK)*ZRK5(JL,JAJ,JK)  
      ZRK(JL,JAJ,JK) = ZRK(JL,JAJ,JK)&
       & + ZDIFF(JL) *ZUR5(JL,1,JK)*ZUR5(JL,2,JK)*ZUR5(JL,3,JK)  
      ZDIFF(JL)  = 0.0_JPRB
    ENDDO

    CALL SWTT1AD( KIDIA, KFDIA, KLON, KNU, 6, IIND,&
     & ZUW5(1,1,JK), ZUR5(1,1,JK), ZW, ZR)

    DO JL=KIDIA,KFDIA
      POZ(JL,IKM1) = POZ(JL,IKM1) + ZW(JL,6)*1.66_JPRB
      PUD(JL,2,IKM1) = PUD(JL,2,IKM1) + ZW(JL,5)*1.66_JPRB
      PUD(JL,1,IKM1) = PUD(JL,1,IKM1) + ZW(JL,4)*1.66_JPRB
      POZ(JL,IKM1) = POZ(JL,IKM1) + ZW(JL,3)*1.66_JPRB
      PUD(JL,2,IKM1) = PUD(JL,2,IKM1) + ZW(JL,2)*1.66_JPRB
      PUD(JL,1,IKM1) = PUD(JL,1,IKM1) + ZW(JL,1)*1.66_JPRB
    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    ZDIRF(JL) = ZDIRF(JL) + PCU(JL,1) *PALBP5(JL,KNU)*RSUN(KNU)
    PALBP(JL,KNU) = PALBP(JL,KNU) + PCU(JL,1) *ZDDIRF5(JL,1)*RSUN(KNU)
    PCU(JL,1) = 0.0_JPRB

    PCLEAR(JL) = PCLEAR(JL) - PFU(JL,1) *ZDDIFF5(JL,1)*PALBD5(JL,KNU)&
     & *RSUN(KNU)  
    ZDIFF(JL) =  ZDIFF(JL)&
     & + PFU(JL,1) *(1.0_JPRB-PCLEAR5(JL))*PALBD5(JL,KNU)*RSUN(KNU)  
    PALBD(JL,KNU) = PALBD(JL,KNU)&
     & + PFU(JL,1) *(1.0_JPRB-PCLEAR5(JL))*ZDDIFF5(JL,1)*RSUN(KNU)  
    PCLEAR(JL) = PCLEAR(JL)+ PFU(JL,1) *ZDDIRF5(JL,1)*PALBP5(JL,KNU)&
     & *RSUN(KNU)  
    ZDIRF(JL) = ZDIRF(JL)+ PFU(JL,1) *PCLEAR5(JL)*PALBP5(JL,KNU)&
     & *RSUN(KNU)  
    PALBP(JL,KNU) = PALBP(JL,KNU)&
     & + PFU(JL,1) *PCLEAR5(JL)*ZDDIRF5(JL,1)*RSUN(KNU)  
    PFU(JL,1) = 0.0_JPRB
  ENDDO

!1320 CONTINUE

  DO JK=KLEV,1,-1
    IKL=KLEV+1-JK

    DO JL=KIDIA,KFDIA

      ZDIRF(JL) = ZDIRF(JL) + PCD(JL,IKL) *RSUN(KNU)
      PCD(JL,IKL) = 0.0_JPRB

      PCLEAR(JL) = PCLEAR(JL) - PFD(JL,IKL) *ZDDIFF5(JL,IKL)*RSUN(KNU)
      ZDIFF(JL) = ZDIFF(JL)+ PFD(JL,IKL) *(1.0_JPRB-PCLEAR5(JL))*RSUN(KNU)
      PCLEAR(JL) = PCLEAR(JL)+ PFD(JL,IKL) *ZDDIRF5(JL,IKL)*RSUN(KNU)
      ZDIRF(JL) = ZDIRF(JL)+ PFD(JL,IKL) * PCLEAR5(JL)*RSUN(KNU)
      PFD(JL,IKL) = 0.0_JPRB

      ZR(JL,4)   = ZR(JL,4)&
       & + ZDIRF(JL) *ZDR5(JL,5,IKL)*ZDR5(JL,6,IKL)*ZRJ05(JL,JAJ,IKL)  
      ZR(JL,5)   = ZR(JL,5)&
       & + ZDIRF(JL) *ZDR5(JL,4,IKL)*ZDR5(JL,6,IKL)*ZRJ05(JL,JAJ,IKL)  
      ZR(JL,6)   = ZR(JL,6)&
       & + ZDIRF(JL) *ZDR5(JL,4,IKL)*ZDR5(JL,5,IKL)*ZRJ05(JL,JAJ,IKL)  
      ZRJ0(JL,JAJ,IKL) =  ZRJ0(JL,JAJ,IKL)&
       & + ZDIRF(JL) *ZDR5(JL,4,IKL)*ZDR5(JL,5,IKL)*ZDR5(JL,6,IKL)  
      ZDIRF(JL)  = 0.0_JPRB

      ZR(JL,1)   = ZR(JL,1)&
       & + ZDIFF(JL) *ZDR5(JL,2,IKL)*ZDR5(JL,3,IKL)*ZRJ5(JL,JAJ,IKL)  
      ZR(JL,2)   = ZR(JL,2)&
       & + ZDIFF(JL) *ZDR5(JL,1,IKL)*ZDR5(JL,3,IKL)*ZRJ5(JL,JAJ,IKL)  
      ZR(JL,3)   = ZR(JL,3)&
       & + ZDIFF(JL) *ZDR5(JL,1,IKL)*ZDR5(JL,2,IKL)*ZRJ5(JL,JAJ,IKL)  
      ZRJ(JL,JAJ,IKL) = ZRJ(JL,JAJ,IKL)&
       & + ZDIFF(JL) *ZDR5(JL,1,IKL)*ZDR5(JL,2,IKL)*ZDR5(JL,3,IKL)  
      ZDIFF(JL)  = 0.0_JPRB
    ENDDO

    CALL SWTT1AD( KIDIA, KFDIA, KLON, KNU, 6, IIND,&
     & ZDW5(1,1,IKL), ZDR5(1,1,IKL), ZW, ZR)

    DO JL=KIDIA,KFDIA
      ZRE = 0.0_JPRB
      ZR0 = 0.0_JPRB

      POZ(JL,IKL) = POZ(JL,IKL) + ZR05(JL,IKL)*ZW(JL,6)
      ZR0 = ZR0 + POZ5(JL,IKL)*ZW(JL,6)

      PUD(JL,2,IKL) = PUD(JL,2,IKL) + ZR05(JL,IKL)*ZW(JL,5)
      ZR0 = ZR0 + PUD5(JL,2,IKL)*ZW(JL,5)

      PUD(JL,1,IKL) = PUD(JL,1,IKL) + ZR05(JL,IKL)*ZW(JL,4)
      ZR0 = ZR0 + PUD5(JL,1,IKL)*ZW(JL,4)

      POZ(JL,IKL) = POZ(JL,IKL) + ZRE5(JL,IKL)*ZW(JL,3)
      ZRE = ZRE + POZ5(JL,IKL)*ZW(JL,3)

      PUD(JL,2,IKL) = PUD(JL,2,IKL) + ZRE5(JL,IKL)*ZW(JL,2)
      ZRE = ZRE + PUD5(JL,2,IKL)*ZW(JL,2)

      PUD(JL,1,IKL) = PUD(JL,1,IKL) + ZRE5(JL,IKL)*ZW(JL,1)
      ZRE = ZRE + PUD5(JL,1,IKL)*ZW(JL,1)

      ZRMU0(JL,IKL) = ZRMU0(JL,IKL)-ZR0*ZR05(JL,IKL)**2
      ZRMUE(JL,IKL) = ZRMUE(JL,IKL)-ZRE*ZRE5(JL,IKL)**2
    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    ZRJ0(JL,JAJ,KLEV+1) = ZRJ0(JL,JAJ,KLEV+1) + PCD(JL,KLEV+1)*RSUN(KNU)
    PCD(JL,KLEV+1) = 0.0_JPRB

    PCLEAR(JL) = PCLEAR(JL)- PFD(JL,KLEV+1) *ZRJ5(JL,JAJ,KLEV+1)*RSUN(KNU)
    ZRJ(JL,JAJ,KLEV+1) = ZRJ(JL,JAJ,KLEV+1)&
     & + PFD(JL,KLEV+1) *(1.0_JPRB-PCLEAR5(JL))*RSUN(KNU)  
    PCLEAR(JL) = PCLEAR(JL)+ PFD(JL,KLEV+1) *ZRJ05(JL,JAJ,KLEV+1)&
     & *RSUN(KNU)  
    ZRJ0(JL,JAJ,KLEV+1) = ZRJ0(JL,JAJ,KLEV+1)&
     & + PFD(JL,KLEV+1) *PCLEAR5(JL)*RSUN(KNU)  
    PFD(JL,KLEV+1) = 0.0_JPRB

    ZW(JL,6) = 0.0_JPRB
    ZW(JL,5) = 0.0_JPRB
    ZW(JL,4) = 0.0_JPRB
    ZW(JL,3) = 0.0_JPRB
    ZW(JL,2) = 0.0_JPRB
    ZW(JL,1) = 0.0_JPRB
  ENDDO

!1310 CONTINUE

!1300 CONTINUE

ELSEIF (NSW == 6) THEN

!*         3.2   SIX SPECTRAL INTERVALS
!                ----------------------

  IIND(1)=1
  IIND(2)=2
  IIND(3)=1
  IIND(4)=2

  DO JK=KLEV+1,2,-1
    IKM1 = JK-1

    DO JL=KIDIA,KFDIA
      ZDIRF(JL)  = ZDIRF(JL) + PCU(JL,JK) * RSUN(KNU)
      PCU(JL,JK) = 0.0_JPRB

      PCLEAR(JL) = PCLEAR(JL) - PFU(JL,JK) *ZUDIFF5(JL,JK)*RSUN(KNU)
      ZDIFF(JL)  = ZDIFF(JL) + PFU(JL,JK) *(1.0_JPRB-PCLEAR5(JL))*RSUN(KNU)
      PCLEAR(JL) = PCLEAR(JL) + PFU(JL,JK) *ZUDIRF5(JL,JK)*RSUN(KNU)
      ZDIRF(JL)  = ZDIRF(JL) + PFU(JL,JK) *PCLEAR5(JL)*RSUN(KNU)
      PFU(JL,JK) = 0.0_JPRB

      ZR(JL,3)   = ZR(JL,3)&
       & + ZDIRF(JL) *ZUR5(JL,4,JK)*ZUT5(JL,2,JK)*ZRK05(JL,JAJ,JK)  
      ZR(JL,4)   = ZR(JL,4)&
       & + ZDIRF(JL) *ZUR5(JL,3,JK)*ZUT5(JL,2,JK)*ZRK05(JL,JAJ,JK)  
      ZT(JL,2)   = ZT(JL,2)&
       & + ZDIRF(JL) *ZUR5(JL,3,JK)*ZUR5(JL,4,JK)*ZRK05(JL,JAJ,JK)  
      ZRK0(JL,JAJ,JK) =  ZRK0(JL,JAJ,JK)&
       & + ZDIRF(JL) *ZUR5(JL,3,JK)*ZUR5(JL,4,JK)*ZUT5(JL,2,JK)  
      ZDIRF(JL)  = 0.0_JPRB

      ZR(JL,1)   = ZR(JL,1)&
       & + ZDIFF(JL) *ZUR5(JL,2,JK)*ZUT5(JL,1,JK)*ZRK5(JL,JAJ,JK)  
      ZR(JL,2)   = ZR(JL,2)&
       & + ZDIFF(JL) *ZUR5(JL,1,JK)*ZUT5(JL,1,JK)*ZRK5(JL,JAJ,JK)  
      ZT(JL,1)   = ZT(JL,1)&
       & + ZDIFF(JL) *ZUR5(JL,1,JK)*ZUR5(JL,2,JK)*ZRK5(JL,JAJ,JK)  
      ZRK(JL,JAJ,JK) = ZRK(JL,JAJ,JK)&
       & + ZDIFF(JL) *ZUR5(JL,1,JK)*ZUR5(JL,2,JK)*ZUT5(JL,1,JK)  
      ZDIFF(JL)  = 0.0_JPRB
    ENDDO

    CALL SWUVO3AD ( KIDIA, KFDIA, KLON, KNU, 2,&
     & ZUO5(1,1,JK), ZUT5(1,1,JK), ZO, ZT)
    
    CALL SWTT1AD( KIDIA, KFDIA, KLON, KNU, 4, IIND,&
     & ZUW5(1,1,JK), ZUR5(1,1,JK), ZW, ZR)

    DO JL=KIDIA,KFDIA
      POZ(JL,IKM1) = POZ(JL,IKM1) + ZO(JL,2)*1.66_JPRB
      POZ(JL,IKM1) = POZ(JL,IKM1) + ZO(JL,1)*1.66_JPRB
      PUD(JL,2,IKM1) = PUD(JL,2,IKM1) + ZW(JL,4)*1.66_JPRB
      PUD(JL,1,IKM1) = PUD(JL,1,IKM1) + ZW(JL,3)*1.66_JPRB
      PUD(JL,2,IKM1) = PUD(JL,2,IKM1) + ZW(JL,2)*1.66_JPRB
      PUD(JL,1,IKM1) = PUD(JL,1,IKM1) + ZW(JL,1)*1.66_JPRB
    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    ZDIRF(JL) = ZDIRF(JL) + PCU(JL,1) *PALBP5(JL,KNU)*RSUN(KNU)
    PALBP(JL,KNU) = PALBP(JL,KNU) + PCU(JL,1) *ZDDIRF5(JL,1)*RSUN(KNU)
    PCU(JL,1) = 0.0_JPRB

    PCLEAR(JL) = PCLEAR(JL) - PFU(JL,1) *ZDDIFF5(JL,1)*PALBD5(JL,KNU)&
     & *RSUN(KNU)  
    ZDIFF(JL) =  ZDIFF(JL)&
     & + PFU(JL,1) *(1.0_JPRB-PCLEAR5(JL))*PALBD5(JL,KNU)*RSUN(KNU)  
    PALBD(JL,KNU) = PALBD(JL,KNU)&
     & + PFU(JL,1) *(1.0_JPRB-PCLEAR5(JL))*ZDDIFF5(JL,1)*RSUN(KNU)  
    PCLEAR(JL) = PCLEAR(JL)+ PFU(JL,1) *ZDDIRF5(JL,1)*PALBP5(JL,KNU)&
     & *RSUN(KNU)  
    ZDIRF(JL) = ZDIRF(JL)+ PFU(JL,1) *PCLEAR5(JL)*PALBP5(JL,KNU)&
     & *RSUN(KNU)  
    PALBP(JL,KNU) = PALBP(JL,KNU)&
     & + PFU(JL,1) *PCLEAR5(JL)*ZDDIRF5(JL,1)*RSUN(KNU)  
    PFU(JL,1) = 0.0_JPRB
  ENDDO

  DO JK=KLEV,1,-1
    IKL=KLEV+1-JK

    DO JL=KIDIA,KFDIA

      ZDIRF(JL) = ZDIRF(JL) + PCD(JL,IKL) *RSUN(KNU)
      PCD(JL,IKL) = 0.0_JPRB

      PCLEAR(JL) = PCLEAR(JL) - PFD(JL,IKL) *ZDDIFF5(JL,IKL)*RSUN(KNU)
      ZDIFF(JL) = ZDIFF(JL)+ PFD(JL,IKL) *(1.0_JPRB-PCLEAR5(JL))*RSUN(KNU)
      PCLEAR(JL) = PCLEAR(JL)+ PFD(JL,IKL) *ZDDIRF5(JL,IKL)*RSUN(KNU)
      ZDIRF(JL) = ZDIRF(JL)+ PFD(JL,IKL) * PCLEAR5(JL)*RSUN(KNU)
      PFD(JL,IKL) = 0.0_JPRB

      ZR(JL,3)   = ZR(JL,3)&
       & + ZDIRF(JL) *ZDR5(JL,4,IKL)*ZDT5(JL,2,IKL)*ZRJ05(JL,JAJ,IKL)  
      ZR(JL,4)   = ZR(JL,4)&
       & + ZDIRF(JL) *ZDR5(JL,3,IKL)*ZDT5(JL,2,IKL)*ZRJ05(JL,JAJ,IKL)  
      ZT(JL,2)   = ZT(JL,2)&
       & + ZDIRF(JL) *ZDR5(JL,3,IKL)*ZDR5(JL,4,IKL)*ZRJ05(JL,JAJ,IKL)  
      ZRJ0(JL,JAJ,IKL) =  ZRJ0(JL,JAJ,IKL)&
       & + ZDIRF(JL) *ZDR5(JL,3,IKL)*ZDR5(JL,4,IKL)*ZDT5(JL,2,IKL)  
      ZDIRF(JL)  = 0.0_JPRB

      ZR(JL,1)   = ZR(JL,1)&
       & + ZDIFF(JL) *ZDR5(JL,2,IKL)*ZDT5(JL,1,IKL)*ZRJ5(JL,JAJ,IKL)  
      ZR(JL,2)   = ZR(JL,2)&
       & + ZDIFF(JL) *ZDR5(JL,1,IKL)*ZDT5(JL,1,IKL)*ZRJ5(JL,JAJ,IKL)  
      ZT(JL,1)   = ZT(JL,1)&
       & + ZDIFF(JL) *ZDR5(JL,1,IKL)*ZDR5(JL,2,IKL)*ZRJ5(JL,JAJ,IKL)  
      ZRJ(JL,JAJ,IKL) = ZRJ(JL,JAJ,IKL)&
       & + ZDIFF(JL) *ZDR5(JL,1,IKL)*ZDR5(JL,2,IKL)*ZDT5(JL,1,IKL)  
      ZDIFF(JL)  = 0.0_JPRB
    ENDDO

    CALL SWUVO3AD ( KIDIA, KFDIA, KLON, KNU, 2,&
     & ZDO5(1,1,IKL), ZDT5(1,1,IKL), ZO, ZT )
    
    CALL SWTT1AD ( KIDIA, KFDIA, KLON, KNU, 4, IIND,&
     & ZDW5(1,1,IKL), ZDR5(1,1,IKL), ZW, ZR )

    DO JL=KIDIA,KFDIA
      ZRE = 0.0_JPRB
      ZR0 = 0.0_JPRB

      POZ(JL,IKL) = POZ(JL,IKL) + ZR05(JL,IKL)*ZO(JL,2)
      ZR0 = ZR0 + POZ5(JL,IKL)*ZO(JL,2)

      POZ(JL,IKL) = POZ(JL,IKL) + ZRE5(JL,IKL)*ZO(JL,1)
      ZRE = ZRE + POZ5(JL,IKL)*ZO(JL,1)

      PUD(JL,2,IKL) = PUD(JL,2,IKL) + ZR05(JL,IKL)*ZW(JL,4)
      ZR0 = ZR0 + PUD5(JL,2,IKL)*ZW(JL,4)

      PUD(JL,1,IKL) = PUD(JL,1,IKL) + ZR05(JL,IKL)*ZW(JL,3)
      ZR0 = ZR0 + PUD5(JL,1,IKL)*ZW(JL,3)

      PUD(JL,2,IKL) = PUD(JL,2,IKL) + ZRE5(JL,IKL)*ZW(JL,2)
      ZRE = ZRE + PUD5(JL,2,IKL)*ZW(JL,2)

      PUD(JL,1,IKL) = PUD(JL,1,IKL) + ZRE5(JL,IKL)*ZW(JL,1)
      ZRE = ZRE + PUD5(JL,1,IKL)*ZW(JL,1)

      ZRMU0(JL,IKL) = ZRMU0(JL,IKL)-ZR0*ZR05(JL,IKL)**2
      ZRMUE(JL,IKL) = ZRMUE(JL,IKL)-ZRE*ZRE5(JL,IKL)**2
    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    ZRJ0(JL,JAJ,KLEV+1) = ZRJ0(JL,JAJ,KLEV+1) + PCD(JL,KLEV+1)*RSUN(KNU)
    PCD(JL,KLEV+1) = 0.0_JPRB

    PCLEAR(JL) = PCLEAR(JL)- PFD(JL,KLEV+1) *ZRJ5(JL,JAJ,KLEV+1)*RSUN(KNU)
    ZRJ(JL,JAJ,KLEV+1) = ZRJ(JL,JAJ,KLEV+1)&
     & + PFD(JL,KLEV+1) *(1.0_JPRB-PCLEAR5(JL))*RSUN(KNU)  
    PCLEAR(JL) = PCLEAR(JL)+ PFD(JL,KLEV+1) *ZRJ05(JL,JAJ,KLEV+1)&
     & *RSUN(KNU)  
    ZRJ0(JL,JAJ,KLEV+1) = ZRJ0(JL,JAJ,KLEV+1)&
     & + PFD(JL,KLEV+1) *PCLEAR5(JL)*RSUN(KNU)  
    PFD(JL,KLEV+1) = 0.0_JPRB

    ZO(JL,2) = 0.0_JPRB
    ZO(JL,1) = 0.0_JPRB
    ZW(JL,4) = 0.0_JPRB
    ZW(JL,3) = 0.0_JPRB
    ZW(JL,2) = 0.0_JPRB
    ZW(JL,1) = 0.0_JPRB
  ENDDO

ENDIF

!     ------------------------------------------------------------------            

CALL SWRAD &
 & ( KIDIA  ,KFDIA  ,KLON   ,KLEV   , KNU,&
 & PALBD5 ,PCG5   ,PCLD5  ,POMEGA5, PSEC5 ,PTAU5,&
 & ZCGAZ5 ,ZPIZAZ5,ZRAY15 ,ZRAY25 ,ZREFZ5 , ZRJ5  ,ZRK5 , ZRMUE5,&
 & ZTAUAZ5,ZTRA15 ,ZTRA25,&
! saved in NL computation - swr
 & ZRESWR15,ZRESWR25,ZTRSWR15,ZTRSWR25,&
! perturbation
 & PALBD  ,PCG    ,PCLD   ,POMEGA , PSEC  ,PTAU,&
 & ZCGAZ  ,ZPIZAZ ,ZRAY1  ,ZRAY2  ,ZREFZ  , ZRJ   ,ZRK  , ZRMUE,&
 & ZTAUAZ ,ZTRA1  ,ZTRA2  &
 & )  

!1220 CONTINUE     

!     ------------------------------------------------------------------            

CALL SWCLRAD &
 & ( KIDIA , KFDIA  , KLON   , KLEV   , KAER   , KNU,&
 & PAER5 , PALBP5 , PDSIG5 , ZRAYL5 , PSEC5,&
 & ZCGAZ5, ZPIZAZ5, ZRAY015, ZRAY025, ZREFZ05, ZRJ05,&
 & ZRK05 , ZRMU05 , ZTAUAZ5, ZTRA015, ZTRA025,&
 & PAER  , PALBP  , PDSIG  , ZRAYL  , PSEC,&
 & ZCGAZ , ZPIZAZ , ZRAY01 , ZRAY02 , ZREFZ0 , ZRJ0,&
 & ZRK0  , ZRMU0  , ZTAUAZ , ZTRA01 , ZTRA02 &
 & )  

!1210 CONTINUE

!1200 CONTINUE     

!     ------------------------------------------------------------------

DO JL=KIDIA,KFDIA
  PRMU(JL) = PRMU(JL) + ZRAYL(JL)      *(1*RRAY(KNU,2) + PRMU5(JL)&
   & * (2*RRAY(KNU,3) + PRMU5(JL)*(3*RRAY(KNU,4) + PRMU5(JL) &
   & * (4*RRAY(KNU,5) + PRMU5(JL)* 5*RRAY(KNU,6)        ))))  
ENDDO

!1110 CONTINUE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SW1SAD',1,ZHOOK_HANDLE)
END SUBROUTINE SW1SAD
