SUBROUTINE SW1STL &
 & ( KIDIA  , KFDIA  , KLON  , KLEV  , KAER  , KNU,&
 & PAER5  , PALBD5 , PALBP5, PCG5  , PCLD5 , PCLEAR5,&
 & PDSIG5 , POMEGA5, POZ5  , PRMU5 , PSEC5 , PTAU5  , PUD5,&
 & PFD5   , PFU5   , PCD5  , PCU5,&
 & PAER  , PALBD , PALBP, PCG  , PCLD , PCLEAR,&
 & PDSIG , POMEGA, POZ  , PRMU , PSEC , PTAU  , PUD,&
 & PFD   , PFU   , PCD  , PCU &
 & )  

!**** *SW1STL* - SHORTWAVE RADIATION, FIRST SPECTRAL INTERVAL
!                -> TANGENT LINEAR <-

!     PURPOSE.
!     --------

!          THIS ROUTINE COMPUTES THE SHORTWAVE RADIATION FLUXES IN TWO
!     SPECTRAL INTERVALS FOLLOWING FOUQUART AND BONNEL (1980).

!**   INTERFACE.
!     ----------

!          *SW1STL* IS CALLED FROM *SWTL*.

!        IMPLICIT ARGUMENTS :
!        --------------------

!     ==== INPUTS ===
!     ==== OUTPUTS ===

!     METHOD.
!     -------

!          1. COMPUTES UPWARD AND DOWNWARD FLUXES CORRESPONDING TO
!     CONTINUUM SCATTERING
!          2. MULTIPLY BY OZONE TRANSMISSION FUNCTION

!     EXTERNALS.
!     ----------

!          *SWCLRTL*, *SWRTL*, *SWTTTL*, *SWUVO3TL*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT
!        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-09-26
!        Modified : 99-05-17 M.Janiskova - correction of dimension
!                                          for zrj0 and zrk0
!                 2000-02-24 M.Janiskova - adopted for revised aerosols
!                                          as in sw1s
!                 2002-05-07 M.Janiskova - adopted for 6 spectral intervals
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!                 2008-03-18 M.Janiskova - code optimization
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOESW    , ONLY : RRAY     ,RSUN
USE YOERAD   , ONLY : NSW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER5(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCG5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLEAR5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSIG5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMU5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUD5(KLON,5,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCG(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLEAR(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSIG(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUD(KLON,5,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCU(KLON,KLEV+1) 
!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!     ------------------------------------------------------------------

!              ------------

INTEGER(KIND=JPIM) :: IIND(6)

REAL(KIND=JPRB) :: ZCGAZ5(KLON,KLEV)&
 & ,  ZDIFF5(KLON)         , ZDIRF5(KLON)        &
 & ,  ZPIZAZ5(KLON,KLEV)&
 & ,  ZRAYL5(KLON)&
 & ,  ZRAY15(KLON,KLEV+1)  , ZRAY015(KLON,KLEV+1)&
 & ,  ZRAY25(KLON,KLEV+1)  , ZRAY025(KLON,KLEV+1)&
 & ,  ZREFZ5(KLON,2,KLEV+1), ZREFZ05(KLON,2,KLEV+1)&
 & ,  ZRJ5(KLON,6,KLEV+1)  , ZRJ05(KLON,6,KLEV+1)&
 & ,  ZRK5(KLON,6,KLEV+1)  , ZRK05(KLON,6,KLEV+1)&
 & ,  ZRMUE5(KLON,KLEV+1)  , ZRMU05(KLON,KLEV+1)&
 & ,  ZR5(KLON,6)&
 & ,  ZTAUAZ5(KLON,KLEV)&
 & ,  ZTRA15(KLON,KLEV+1)  , ZTRA015(KLON,KLEV+1)&
 & ,  ZTRA25(KLON,KLEV+1)  , ZTRA025(KLON,KLEV+1)&
 & ,  ZW5(KLON,6), ZO5(KLON,2), ZT5(KLON,2)  
REAL(KIND=JPRB) :: ZRE5, ZR05

REAL(KIND=JPRB) :: ZCGAZ(KLON,KLEV)&
 & ,  ZDIFF(KLON)          , ZDIRF(KLON)        &
 & ,  ZPIZAZ(KLON,KLEV)&
 & ,  ZRAYL(KLON)&
 & ,  ZRAY1(KLON,KLEV+1)   , ZRAY01(KLON,KLEV+1)&
 & ,  ZRAY2(KLON,KLEV+1)   , ZRAY02(KLON,KLEV+1)&
 & ,  ZREFZ(KLON,2,KLEV+1) , ZREFZ0(KLON,2,KLEV+1)&
 & ,  ZRJ(KLON,6,KLEV+1)   , ZRJ0(KLON,6,KLEV+1)&
 & ,  ZRK(KLON,6,KLEV+1)   , ZRK0(KLON,6,KLEV+1)&
 & ,  ZRMUE(KLON,KLEV+1)   , ZRMU0(KLON,KLEV+1)&
 & ,  ZR(KLON,6)&
 & ,  ZTAUAZ(KLON,KLEV)&
 & ,  ZTRA1(KLON,KLEV+1)   , ZTRA01(KLON,KLEV+1)&
 & ,  ZTRA2(KLON,KLEV+1)   , ZTRA02(KLON,KLEV+1)&
 & ,  ZW(KLON,6), ZO(KLON,2), ZT(KLON,2)  
REAL(KIND=JPRB) :: ZRE, ZR0

INTEGER(KIND=JPIM) :: IKL, IKM1, JAJ, JK, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "swclrtl.intfb.h"
#include "swrtl.intfb.h"
#include "swtt1tl.intfb.h"
#include "swuvo3tl.intfb.h"

!     ------------------------------------------------------------------

!*         1.     FIRST SPECTRAL INTERVAL (0.25-0.68 MICRON)
!                 ----------------------- ------------------

! 100 CONTINUE

!*         1.1    OPTICAL THICKNESS FOR RAYLEIGH SCATTERING
!                 -----------------------------------------

! 110 CONTINUE

IF (LHOOK) CALL DR_HOOK('SW1STL',0,ZHOOK_HANDLE)
DO JL = KIDIA,KFDIA
  ZRAYL (JL) =  PRMU(JL)                 *(1*RRAY(KNU,2) +&
   & PRMU5(JL)&
   & * (2*RRAY(KNU,3) + PRMU5(JL)*(3*RRAY(KNU,4) + PRMU5(JL)&
   & * (4*RRAY(KNU,5) + PRMU5(JL)* 5*RRAY(KNU,6)        ))))  

  ZRAYL5(JL) =  RRAY(KNU,1) + PRMU5(JL) * (RRAY(KNU,2) + PRMU5(JL)&
   & * (RRAY(KNU,3) + PRMU5(JL) * (RRAY(KNU,4) + PRMU5(JL)&
   & * (RRAY(KNU,5) + PRMU5(JL) *  RRAY(KNU,6)        ))))  
ENDDO

!     ------------------------------------------------------------------

!*         2.    CONTINUUM SCATTERING CALCULATIONS
!                ---------------------------------

! 200 CONTINUE

!*         2.1   CLEAR-SKY FRACTION OF THE COLUMN
!                --------------------------------

! 210 CONTINUE

CALL SWCLRTL &
 & ( KIDIA , KFDIA  , KLON   , KLEV   , KAER   , KNU,&
 & PAER5 , PALBP5 , PDSIG5 , ZRAYL5 , PSEC5,&
 & ZCGAZ5, ZPIZAZ5, ZRAY015, ZRAY025, ZREFZ05, ZRJ05,&
 & ZRK05 , ZRMU05 , ZTAUAZ5, ZTRA015, ZTRA025,&
 & PAER  , PALBP  , PDSIG  , ZRAYL  , PSEC,&
 & ZCGAZ , ZPIZAZ , ZRAY01  , ZRAY02, ZREFZ0 , ZRJ0,&
 & ZRK0  , ZRMU0  , ZTAUAZ  , ZTRA01, ZTRA02 &
 & )  

!*         2.2   CLOUDY FRACTION OF THE COLUMN
!                -----------------------------

! 220 CONTINUE

CALL SWRTL &
 & ( KIDIA  ,KFDIA  ,KLON   ,KLEV   , KNU,&
 & PALBD5 ,PCG5   ,PCLD5  ,POMEGA5, PSEC5 ,PTAU5,&
 & ZCGAZ5 ,ZPIZAZ5,ZRAY15 ,ZRAY25 ,ZREFZ5 , ZRJ5  ,ZRK5 , ZRMUE5,&
 & ZTAUAZ5,ZTRA15 ,ZTRA25,&
 & PALBD  ,PCG    ,PCLD   ,POMEGA , PSEC  ,PTAU,&
 & ZCGAZ  ,ZPIZAZ ,ZRAY1  ,ZRAY2  ,ZREFZ  , ZRJ   ,ZRK  , ZRMUE,&
 & ZTAUAZ ,ZTRA1  ,ZTRA2 &
 & )  

!     ------------------------------------------------------------------

!*         3.    OZONE ABSORPTION
!                ----------------

! 300 CONTINUE

IF (NSW <= 4) THEN

!*         3.1   TWO OR FOUR SPECTRAL INTERVALS
!                ------------------------------

  IIND(1)=1
  IIND(2)=2
  IIND(3)=3
  IIND(4)=1
  IIND(5)=2
  IIND(6)=3

!*         3.1.1  DOWNWARD FLUXES
!                 ---------------

! 310 CONTINUE

  JAJ = 2

  DO JL = KIDIA,KFDIA
    ZW (JL,1)=0.0_JPRB
    ZW (JL,2)=0.0_JPRB
    ZW (JL,3)=0.0_JPRB
    ZW (JL,4)=0.0_JPRB
    ZW (JL,5)=0.0_JPRB
    ZW (JL,6)=0.0_JPRB
    ZW5(JL,1)=0.0_JPRB
    ZW5(JL,2)=0.0_JPRB
    ZW5(JL,3)=0.0_JPRB
    ZW5(JL,4)=0.0_JPRB
    ZW5(JL,5)=0.0_JPRB
    ZW5(JL,6)=0.0_JPRB
    PFD (JL,KLEV+1)=((  -PCLEAR (JL))*ZRJ5(JL,JAJ,KLEV+1)&
     & +(1.0_JPRB-PCLEAR5(JL))*ZRJ (JL,JAJ,KLEV+1)&
     & + PCLEAR(JL)  *ZRJ05(JL,JAJ,KLEV+1)&
     & + PCLEAR5(JL) *ZRJ0 (JL,JAJ,KLEV+1)) * RSUN(KNU)  
    PFD5(JL,KLEV+1)=((1.0_JPRB-PCLEAR5(JL))*ZRJ5(JL,JAJ,KLEV+1)&
     & + PCLEAR5(JL) *ZRJ05(JL,JAJ,KLEV+1)) * RSUN(KNU)  
    PCD (JL,KLEV+1)= ZRJ0 (JL,JAJ,KLEV+1) * RSUN(KNU)
    PCD5(JL,KLEV+1)= ZRJ05(JL,JAJ,KLEV+1) * RSUN(KNU)
  ENDDO
  DO JK = 1 , KLEV
    IKL = KLEV+1-JK
    DO JL = KIDIA,KFDIA
      ZRE5 = 1.0_JPRB/ZRMUE5(JL,IKL)
      ZRE  = -ZRMUE(JL,IKL)*ZRE5**2
      ZR05 = 1.0_JPRB/ZRMU05(JL,IKL)
      ZR0  = -ZRMU0(JL,IKL)*ZR05**2

      ZW (JL,1)= ZW (JL,1) + PUD(JL,1,IKL)*ZRE5 + PUD5(JL,1,IKL)*ZRE
      ZW (JL,2)= ZW (JL,2) + PUD(JL,2,IKL)*ZRE5 + PUD5(JL,2,IKL)*ZRE 
      ZW (JL,3)= ZW (JL,3) + POZ(JL,  IKL)*ZRE5 + POZ5(JL,  IKL)*ZRE
      ZW (JL,4)= ZW (JL,4) + PUD(JL,1,IKL)*ZR05 + PUD5(JL,1,IKL)*ZR0 
      ZW (JL,5)= ZW (JL,5) + PUD(JL,2,IKL)*ZR05 + PUD5(JL,2,IKL)*ZR0
      ZW (JL,6)= ZW (JL,6) + POZ(JL,  IKL)*ZR05 + POZ5(JL,  IKL)*ZR0 

      ZW5(JL,1)=ZW5(JL,1)+PUD5(JL,1,IKL)*ZRE5
      ZW5(JL,2)=ZW5(JL,2)+PUD5(JL,2,IKL)*ZRE5
      ZW5(JL,3)=ZW5(JL,3)+POZ5(JL,  IKL)*ZRE5
      ZW5(JL,4)=ZW5(JL,4)+PUD5(JL,1,IKL)*ZR05
      ZW5(JL,5)=ZW5(JL,5)+PUD5(JL,2,IKL)*ZR05
      ZW5(JL,6)=ZW5(JL,6)+POZ5(JL,  IKL)*ZR05
    ENDDO

    CALL SWTT1TL ( KIDIA, KFDIA, KLON, KNU, 6, IIND , ZW5  , ZR5 , ZW  , ZR )

    DO JL = KIDIA,KFDIA
      ZDIFF (JL) = ZR (JL,1)*ZR5(JL,2)*ZR5(JL,3)*ZRJ5(JL,JAJ,IKL)&
       & + ZR5(JL,1)*ZR (JL,2)*ZR5(JL,3)*ZRJ5(JL,JAJ,IKL)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZR (JL,3)*ZRJ5(JL,JAJ,IKL)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZR5(JL,3)*ZRJ (JL,JAJ,IKL)  
      ZDIFF5(JL) = ZR5(JL,1)*ZR5(JL,2)*ZR5(JL,3)*ZRJ5(JL,JAJ,IKL)
      ZDIRF (JL) = ZR (JL,4)*ZR5(JL,5)*ZR5(JL,6)*ZRJ05(JL,JAJ,IKL)&
       & + ZR5(JL,4)*ZR (JL,5)*ZR5(JL,6)*ZRJ05(JL,JAJ,IKL)&
       & + ZR5(JL,4)*ZR5(JL,5)*ZR (JL,6)*ZRJ05(JL,JAJ,IKL)&
       & + ZR5(JL,4)*ZR5(JL,5)*ZR5(JL,6)*ZRJ0 (JL,JAJ,IKL)  
      ZDIRF5(JL) = ZR5(JL,4)*ZR5(JL,5)*ZR5(JL,6)*ZRJ05(JL,JAJ,IKL)
      PFD (JL,IKL) = ((  -PCLEAR (JL)) * ZDIFF5(JL)&
       & +(1.0_JPRB-PCLEAR5(JL)) * ZDIFF (JL)&
       & +PCLEAR (JL)  * ZDIRF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF (JL)) * RSUN(KNU)  
      PFD5(JL,IKL) = ((1.0_JPRB-PCLEAR5(JL)) * ZDIFF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF5(JL)) * RSUN(KNU)  
      PCD (JL,IKL) = ZDIRF (JL) * RSUN(KNU)
      PCD5(JL,IKL) = ZDIRF5(JL) * RSUN(KNU)
    ENDDO
  ENDDO

!*         3.2.1  UPWARD FLUXES
!                 -------------

! 320 CONTINUE

  DO JL = KIDIA,KFDIA
    PFU (JL,1) = ((  -PCLEAR (JL))*ZDIFF5(JL)*PALBD5(JL,KNU)&
     & +  (1.0_JPRB-PCLEAR5(JL))*ZDIFF (JL)*PALBD5(JL,KNU)&
     & +  (1.0_JPRB-PCLEAR5(JL))*ZDIFF5(JL)*PALBD (JL,KNU)&
     & + PCLEAR (JL) *ZDIRF5(JL)*PALBP5(JL,KNU)&
     & + PCLEAR5(JL) *ZDIRF (JL)*PALBP5(JL,KNU)&
     & + PCLEAR5(JL) *ZDIRF5(JL)*PALBP (JL,KNU))&
     & * RSUN(KNU)  
    PFU5(JL,1) = ((1.0_JPRB-PCLEAR5(JL))*ZDIFF5(JL)*PALBD5(JL,KNU)&
     & + PCLEAR5(JL) *ZDIRF5(JL)*PALBP5(JL,KNU))&
     & * RSUN(KNU)  
    PCU (JL,1) = ZDIRF (JL) * PALBP5(JL,KNU) * RSUN(KNU)&
     & + ZDIRF5(JL) * PALBP (JL,KNU) * RSUN(KNU)  
    PCU5(JL,1) = ZDIRF5(JL) * PALBP5(JL,KNU) * RSUN(KNU)
  ENDDO

  DO JK = 2 , KLEV+1
    IKM1=JK-1
    DO JL = KIDIA,KFDIA
      ZW (JL,1)=ZW (JL,1)+PUD (JL,1,IKM1)*1.66_JPRB
      ZW (JL,2)=ZW (JL,2)+PUD (JL,2,IKM1)*1.66_JPRB
      ZW (JL,3)=ZW (JL,3)+POZ (JL,  IKM1)*1.66_JPRB
      ZW (JL,4)=ZW (JL,4)+PUD (JL,1,IKM1)*1.66_JPRB
      ZW (JL,5)=ZW (JL,5)+PUD (JL,2,IKM1)*1.66_JPRB
      ZW (JL,6)=ZW (JL,6)+POZ (JL,  IKM1)*1.66_JPRB
      ZW5(JL,1)=ZW5(JL,1)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZW5(JL,2)=ZW5(JL,2)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZW5(JL,3)=ZW5(JL,3)+POZ5(JL,  IKM1)*1.66_JPRB
      ZW5(JL,4)=ZW5(JL,4)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZW5(JL,5)=ZW5(JL,5)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZW5(JL,6)=ZW5(JL,6)+POZ5(JL,  IKM1)*1.66_JPRB
    ENDDO

    CALL SWTT1TL( KIDIA, KFDIA, KLON, KNU, 6, IIND, ZW5, ZR5, ZW, ZR)

    DO JL = KIDIA,KFDIA
      ZDIFF (JL) = ZR (JL,1)*ZR5(JL,2)*ZR5(JL,3)*ZRK5(JL,JAJ,JK)&
       & + ZR5(JL,1)*ZR (JL,2)*ZR5(JL,3)*ZRK5(JL,JAJ,JK)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZR (JL,3)*ZRK5(JL,JAJ,JK)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZR5(JL,3)*ZRK (JL,JAJ,JK)  
      ZDIFF5(JL) = ZR5(JL,1)*ZR5(JL,2)*ZR5(JL,3)*ZRK5(JL,JAJ,JK)
      ZDIRF (JL) = ZR (JL,4)*ZR5(JL,5)*ZR5(JL,6)*ZRK05(JL,JAJ,JK)&
       & + ZR5(JL,4)*ZR (JL,5)*ZR5(JL,6)*ZRK05(JL,JAJ,JK)&
       & + ZR5(JL,4)*ZR5(JL,5)*ZR (JL,6)*ZRK05(JL,JAJ,JK)&
       & + ZR5(JL,4)*ZR5(JL,5)*ZR5(JL,6)*ZRK0 (JL,JAJ,JK)  
      ZDIRF5(JL) = ZR5(JL,4)*ZR5(JL,5)*ZR5(JL,6)*ZRK05(JL,JAJ,JK)
      PFU (JL,JK) = ((  -PCLEAR (JL)) * ZDIFF5(JL)&
       & +(1.0_JPRB-PCLEAR5(JL)) * ZDIFF (JL)&
       & +PCLEAR (JL)  * ZDIRF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF (JL)) * RSUN(KNU)  
      PFU5(JL,JK) = ((1.0_JPRB-PCLEAR5(JL)) * ZDIFF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF5(JL)) * RSUN(KNU)  
      PCU (JL,JK) = ZDIRF (JL) * RSUN(KNU)
      PCU5(JL,JK) = ZDIRF5(JL) * RSUN(KNU)
    ENDDO
  ENDDO
  
ELSEIF (NSW == 6) THEN

!*         3.2   SIX SPECTRAL INTERVALS
!                ----------------------

  IIND(1)=1
  IIND(2)=2
  IIND(3)=1
  IIND(4)=2

!*         3.2.1  DOWNWARD FLUXES
!                 ---------------

  JAJ = 2

  DO JL = KIDIA,KFDIA
    ZW (JL,1)=0.0_JPRB
    ZW (JL,2)=0.0_JPRB
    ZW (JL,3)=0.0_JPRB
    ZW (JL,4)=0.0_JPRB
    ZO (JL,1)=0.0_JPRB
    ZO (JL,2)=0.0_JPRB

    ZW5(JL,1)=0.0_JPRB
    ZW5(JL,2)=0.0_JPRB
    ZW5(JL,3)=0.0_JPRB
    ZW5(JL,4)=0.0_JPRB
    ZO5(JL,1)=0.0_JPRB
    ZO5(JL,2)=0.0_JPRB
    
    PFD (JL,KLEV+1)=((  -PCLEAR (JL))*ZRJ5(JL,JAJ,KLEV+1)&
     & +(1.0_JPRB-PCLEAR5(JL))*ZRJ (JL,JAJ,KLEV+1)&
     & + PCLEAR(JL)  *ZRJ05(JL,JAJ,KLEV+1)&
     & + PCLEAR5(JL) *ZRJ0 (JL,JAJ,KLEV+1)) * RSUN(KNU)  
    PFD5(JL,KLEV+1)=((1.0_JPRB-PCLEAR5(JL))*ZRJ5(JL,JAJ,KLEV+1)&
     & + PCLEAR5(JL) *ZRJ05(JL,JAJ,KLEV+1)) * RSUN(KNU)  
    PCD (JL,KLEV+1)= ZRJ0 (JL,JAJ,KLEV+1) * RSUN(KNU)
    PCD5(JL,KLEV+1)= ZRJ05(JL,JAJ,KLEV+1) * RSUN(KNU)
  ENDDO
  
  DO JK = 1 , KLEV
    IKL = KLEV+1-JK
    DO JL = KIDIA,KFDIA
      ZRE5 = 1.0_JPRB/ZRMUE5(JL,IKL)
      ZRE  = -ZRMUE(JL,IKL)*ZRE5**2
      ZR05 = 1.0_JPRB/ZRMU05(JL,IKL)
      ZR0  = -ZRMU0(JL,IKL)*ZR05**2

      ZW (JL,1)= ZW (JL,1) + PUD(JL,1,IKL)*ZRE5 + PUD5(JL,1,IKL)*ZRE 
      ZW (JL,2)= ZW (JL,2) + PUD(JL,2,IKL)*ZRE5 + PUD5(JL,2,IKL)*ZRE 
      ZW (JL,3)= ZW (JL,3) + PUD(JL,1,IKL)*ZR05 + PUD5(JL,1,IKL)*ZR0 
      ZW (JL,4)= ZW (JL,4) + PUD(JL,2,IKL)*ZR05 + PUD5(JL,2,IKL)*ZR0 
       
      ZO (JL,1)= ZO (JL,1) + POZ(JL,  IKL)*ZRE5 + POZ5(JL,  IKL)*ZRE 
      ZO (JL,2)= ZO (JL,2) + POZ(JL,  IKL)*ZR05 + POZ5(JL,  IKL)*ZR0
       
      ZW5(JL,1)=ZW5(JL,1)+PUD5(JL,1,IKL)*ZRE5
      ZW5(JL,2)=ZW5(JL,2)+PUD5(JL,2,IKL)*ZRE5
      ZW5(JL,3)=ZW5(JL,3)+PUD5(JL,1,IKL)*ZR05
      ZW5(JL,4)=ZW5(JL,4)+PUD5(JL,2,IKL)*ZR05
      
      ZO5(JL,1)=ZO5(JL,1)+POZ5(JL,  IKL)*ZRE5
      ZO5(JL,2)=ZO5(JL,2)+POZ5(JL,  IKL)*ZR05
    ENDDO

    CALL SWTT1TL ( KIDIA, KFDIA, KLON, KNU, 4, IIND, ZW5, ZR5, ZW, ZR )

    CALL SWUVO3TL ( KIDIA, KFDIA, KLON, KNU, 2, ZO5, ZT5, ZO, ZT )
    
    DO JL = KIDIA,KFDIA
      ZDIFF (JL) = ZR (JL,1)*ZR5(JL,2)*ZT5(JL,1)*ZRJ5(JL,JAJ,IKL)&
       & + ZR5(JL,1)*ZR (JL,2)*ZT5(JL,1)*ZRJ5(JL,JAJ,IKL)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZT (JL,1)*ZRJ5(JL,JAJ,IKL)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZT5(JL,1)*ZRJ (JL,JAJ,IKL)  
      ZDIFF5(JL) = ZR5(JL,1)*ZR5(JL,2)*ZT5(JL,1)*ZRJ5(JL,JAJ,IKL)
      ZDIRF (JL) = ZR (JL,3)*ZR5(JL,4)*ZT5(JL,2)*ZRJ05(JL,JAJ,IKL)&
       & + ZR5(JL,3)*ZR (JL,4)*ZT5(JL,2)*ZRJ05(JL,JAJ,IKL)&
       & + ZR5(JL,3)*ZR5(JL,4)*ZT (JL,2)*ZRJ05(JL,JAJ,IKL)&
       & + ZR5(JL,3)*ZR5(JL,4)*ZT5(JL,2)*ZRJ0 (JL,JAJ,IKL)  
      ZDIRF5(JL) = ZR5(JL,3)*ZR5(JL,4)*ZT5(JL,2)*ZRJ05(JL,JAJ,IKL)
      PFD (JL,IKL) = ((  -PCLEAR (JL)) * ZDIFF5(JL)&
       & +(1.0_JPRB-PCLEAR5(JL)) * ZDIFF (JL)&
       & +PCLEAR (JL)  * ZDIRF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF (JL)) * RSUN(KNU)  
      PFD5(JL,IKL) = ((1.0_JPRB-PCLEAR5(JL)) * ZDIFF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF5(JL)) * RSUN(KNU)  
      PCD (JL,IKL) = ZDIRF (JL) * RSUN(KNU)
      PCD5(JL,IKL) = ZDIRF5(JL) * RSUN(KNU)
    ENDDO
  ENDDO
  
!*         3.2.2  UPWARD FLUXES
!                 -------------

  DO JL = KIDIA,KFDIA
    PFU (JL,1) = ((  -PCLEAR (JL))*ZDIFF5(JL)*PALBD5(JL,KNU)&
     & +  (1.0_JPRB-PCLEAR5(JL))*ZDIFF (JL)*PALBD5(JL,KNU)&
     & +  (1.0_JPRB-PCLEAR5(JL))*ZDIFF5(JL)*PALBD (JL,KNU)&
     & + PCLEAR (JL) *ZDIRF5(JL)*PALBP5(JL,KNU)&
     & + PCLEAR5(JL) *ZDIRF (JL)*PALBP5(JL,KNU)&
     & + PCLEAR5(JL) *ZDIRF5(JL)*PALBP (JL,KNU))&
     & * RSUN(KNU)  
    PFU5(JL,1) = ((1.0_JPRB-PCLEAR5(JL))*ZDIFF5(JL)*PALBD5(JL,KNU)&
     & + PCLEAR5(JL) *ZDIRF5(JL)*PALBP5(JL,KNU))&
     & * RSUN(KNU)  
    PCU (JL,1) = ZDIRF (JL) * PALBP5(JL,KNU) * RSUN(KNU)&
     & + ZDIRF5(JL) * PALBP (JL,KNU) * RSUN(KNU)  
    PCU5(JL,1) = ZDIRF5(JL) * PALBP5(JL,KNU) * RSUN(KNU)
  ENDDO

  DO JK = 2 , KLEV+1
    IKM1=JK-1
    DO JL = KIDIA,KFDIA
      ZW (JL,1)=ZW (JL,1)+PUD (JL,1,IKM1)*1.66_JPRB
      ZW (JL,2)=ZW (JL,2)+PUD (JL,2,IKM1)*1.66_JPRB
      ZW (JL,3)=ZW (JL,3)+PUD (JL,1,IKM1)*1.66_JPRB
      ZW (JL,4)=ZW (JL,4)+PUD (JL,2,IKM1)*1.66_JPRB
      ZO (JL,1)=ZO (JL,1)+POZ (JL,  IKM1)*1.66_JPRB
      ZO (JL,2)=ZO (JL,2)+POZ (JL,  IKM1)*1.66_JPRB
            
      ZW5(JL,1)=ZW5(JL,1)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZW5(JL,2)=ZW5(JL,2)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZW5(JL,3)=ZW5(JL,3)+PUD5(JL,1,IKM1)*1.66_JPRB
      ZW5(JL,4)=ZW5(JL,4)+PUD5(JL,2,IKM1)*1.66_JPRB
      ZO5(JL,1)=ZO5(JL,1)+POZ5(JL,  IKM1)*1.66_JPRB
      ZO5(JL,2)=ZO5(JL,2)+POZ5(JL,  IKM1)*1.66_JPRB
    ENDDO

    CALL SWTT1TL( KIDIA, KFDIA, KLON, KNU, 4, IIND, ZW5, ZR5, ZW, ZR)
    
    CALL SWUVO3TL ( KIDIA, KFDIA, KLON, KNU, 2, ZO5, ZT5, ZO, ZT)
    
    DO JL = KIDIA,KFDIA
      ZDIFF (JL) = ZR (JL,1)*ZR5(JL,2)*ZT5(JL,1)*ZRK5(JL,JAJ,JK)&
       & + ZR5(JL,1)*ZR (JL,2)*ZT5(JL,1)*ZRK5(JL,JAJ,JK)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZT (JL,1)*ZRK5(JL,JAJ,JK)&
       & + ZR5(JL,1)*ZR5(JL,2)*ZT5(JL,1)*ZRK (JL,JAJ,JK)  
      ZDIFF5(JL) = ZR5(JL,1)*ZR5(JL,2)*ZT5(JL,1)*ZRK5(JL,JAJ,JK)
      ZDIRF (JL) = ZR (JL,3)*ZR5(JL,4)*ZT5(JL,2)*ZRK05(JL,JAJ,JK)&
       & + ZR5(JL,3)*ZR (JL,4)*ZT5(JL,2)*ZRK05(JL,JAJ,JK)&
       & + ZR5(JL,3)*ZR5(JL,4)*ZT (JL,2)*ZRK05(JL,JAJ,JK)&
       & + ZR5(JL,3)*ZR5(JL,4)*ZT5(JL,2)*ZRK0 (JL,JAJ,JK)  
      ZDIRF5(JL) = ZR5(JL,3)*ZR5(JL,4)*ZT5(JL,2)*ZRK05(JL,JAJ,JK)
      PFU (JL,JK) = ((  -PCLEAR (JL)) * ZDIFF5(JL)&
       & +(1.0_JPRB-PCLEAR5(JL)) * ZDIFF (JL)&
       & +PCLEAR (JL)  * ZDIRF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF (JL)) * RSUN(KNU)  
      PFU5(JL,JK) = ((1.0_JPRB-PCLEAR5(JL)) * ZDIFF5(JL)&
       & +PCLEAR5(JL)  * ZDIRF5(JL)) * RSUN(KNU)  
      PCU (JL,JK) = ZDIRF (JL) * RSUN(KNU)
      PCU5(JL,JK) = ZDIRF5(JL) * RSUN(KNU)
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SW1STL',1,ZHOOK_HANDLE)
END SUBROUTINE SW1STL
