SUBROUTINE SWCLRTL &
 & ( KIDIA  , KFDIA  , KLON   , KLEV   , KAER   , KNU,&
 & PAER5  , PALBP5 , PDSIG5 , PRAYL5 , PSEC5,&
 & PCGAZ5 , PPIZAZ5, PRAY15 , PRAY25 , PREFZ5 , PRJ05,&
 & PRK05  , PRMU05 , PTAUAZ5, PTRA15 , PTRA25,&
 & PAER   , PALBP  , PDSIG  , PRAYL  , PSEC,&
 & PCGAZ  , PPIZAZ , PRAY1  , PRAY2  , PREFZ  , PRJ0,&
 & PRK0   , PRMU0  , PTAUAZ , PTRA1  , PTRA2 &
 & )  

!**** *SWCLRTL* - CLEAR-SKY COLUMN COMPUTATIONS
!                 -> TANGENT LINEAR <-  

!     PURPOSE.
!     --------
!           COMPUTES THE REFLECTIVITY AND TRANSMISSIVITY IN CASE OF
!     CLEAR-SKY COLUMN

!**   INTERFACE.
!     ----------

!          *SWCLRTL* IS CALLED EITHER FROM *SW1STL*, *SWNITL*

!        IMPLICIT ARGUMENTS :
!        --------------------

!     ==== INPUTS ===
!     ==== OUTPUTS ===

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT
!        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-09-26
!        Modified : 99-05-17 M.Janiskova - correction of dimension
!                                          for prj0 and prk0
!                 2000-02-24 M.Janiskova - adopted for revised aerosols
!                                          as in swclr
!                 2002-05-06 M.Janiskova - adopted for 6 spectral intervals
!        03-10-10 Deborah Salmond and Marta Janiskova Optimisation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        JJMorcrette 20051111 bug-fix to swclr from Alf Grini (Meteo-France)
!        M.Janiskova 20080318 code optimization + cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOESW    , ONLY : RTAUA    ,RPIZA    ,RCGA
USE YOERAD   , ONLY : NOVLP    ,NSW
USE YOERDI   , ONLY : REPCLC
USE YOERDU   , ONLY : REPSCT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER5(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSIG5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRAYL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCGAZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPIZAZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY15(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY25(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PREFZ5(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRJ05(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRK05(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRMU05(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTAUAZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA15(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA25(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSIG(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRAYL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCGAZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPIZAZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY2(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PREFZ(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRJ0(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRK0(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRMU0(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTAUAZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA2(KLON,KLEV+1) 
!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!     ------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZC0I5(KLON,KLEV+1)&
 & ,  ZCLEAR5(KLON) &
 & ,  ZR215(KLON) ,  ZSS05(KLON) , ZSCAT5(KLON)&
 & ,  ZTR5(KLON,2,KLEV+1)  

REAL(KIND=JPRB) :: ZC0I(KLON,KLEV+1)&
 & ,  ZCLEAR(KLON) &
 & ,  ZR21(KLON) , ZSS0(KLON) , ZSCAT(KLON)&
 & ,  ZTR(KLON,2,KLEV+1)  

INTEGER(KIND=JPIM) :: IKL, IKLP1, IKM1, JA, JAE, JAJ, JK, JL, INU1

REAL(KIND=JPRB) :: ZBMU0, ZBMU05, ZBMU1, ZBMU15, ZCORAE, ZCORAE5,&
 & ZDEN, ZDEN0, ZDEN05, ZDEN1, ZDEN15, ZDEN5, ZDENB, ZDENB5, &
 & ZFACOA, ZFACOA5, ZFF, ZFF5, &
 & ZMUE, ZMUE5, ZNUM, ZNUM5, ZRATIO, ZRATIO5, ZRE11, &
 & ZRE115, ZTRAY, ZTRAY5
REAL(KIND=JPRB) :: ZIPIZAZ5, ZIPTAUZ5, ZIDENB5, ZICLEAR5, ZDIV15, ZDIV25
REAL(KIND=JPRB) :: ZRR5,ZRR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1.    OPTICAL PARAMETERS FOR AEROSOLS AND RAYLEIGH
!                --------------------------------------------

! 100 CONTINUE

IF (LHOOK) CALL DR_HOOK('SWCLRTL',0,ZHOOK_HANDLE)
DO JK = 1 , KLEV+1
  DO JA = 1 , 6
    DO JL = KIDIA,KFDIA
      PRJ0 (JL,JA,JK) = 0.0_JPRB
      PRJ05(JL,JA,JK) = 0.0_JPRB
      PRK0 (JL,JA,JK) = 0.0_JPRB
      PRK05(JL,JA,JK) = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

! ------   NB: 'PAER' AEROSOLS ARE ENTERED FROM TOP TO BOTTOM

DO JK = 1 , KLEV
  IKL=KLEV+1-JK
  DO JL = KIDIA,KFDIA
    PCGAZ  (JL,JK) = 0.0_JPRB
    PCGAZ5 (JL,JK) = 0.0_JPRB
    PPIZAZ (JL,JK) = 0.0_JPRB
    PPIZAZ5(JL,JK) = 0.0_JPRB
    PTAUAZ (JL,JK) = 0.0_JPRB
    PTAUAZ5(JL,JK) = 0.0_JPRB
  ENDDO
  DO JAE=1,6
    DO JL = KIDIA,KFDIA
      PTAUAZ (JL,JK)=PTAUAZ (JL,JK)+PAER (JL,JAE,IKL)*RTAUA(KNU,JAE)
      PTAUAZ5(JL,JK)=PTAUAZ5(JL,JK)+PAER5(JL,JAE,IKL)*RTAUA(KNU,JAE)
      PPIZAZ (JL,JK)=PPIZAZ (JL,JK)+PAER (JL,JAE,IKL)&
       & * RTAUA(KNU,JAE)*RPIZA(KNU,JAE)  
      PPIZAZ5(JL,JK)=PPIZAZ5(JL,JK)+PAER5(JL,JAE,IKL)&
       & * RTAUA(KNU,JAE)*RPIZA(KNU,JAE)  
      PCGAZ (JL,JK) =  PCGAZ (JL,JK) +PAER (JL,JAE,IKL)&
       & * RTAUA(KNU,JAE)*RPIZA(KNU,JAE)*RCGA(KNU,JAE)  
      PCGAZ5(JL,JK) =  PCGAZ5(JL,JK) +PAER5(JL,JAE,IKL)&
       & * RTAUA(KNU,JAE)*RPIZA(KNU,JAE)*RCGA(KNU,JAE)  
    ENDDO
  ENDDO

  DO JL = KIDIA,KFDIA
    IF (KAER /= 0) THEN
      ZIPIZAZ5 = 1.0_JPRB/PPIZAZ5(JL,JK)
      PCGAZ (JL,JK)=PCGAZ (JL,JK)*ZIPIZAZ5 &
       & -PCGAZ5(JL,JK)*PPIZAZ (JL,JK)*(ZIPIZAZ5**2)  
      PCGAZ5(JL,JK)=PCGAZ5(JL,JK)*ZIPIZAZ5
      ZIPTAUZ5 = 1.0_JPRB/PTAUAZ5(JL,JK)
      PPIZAZ (JL,JK)=PPIZAZ (JL,JK)*ZIPTAUZ5 &
       & -PPIZAZ5(JL,JK)*PTAUAZ(JL,JK)*(ZIPTAUZ5**2)  
      PPIZAZ5(JL,JK)=PPIZAZ5(JL,JK)*ZIPTAUZ5
      ZTRAY  = PRAYL (JL) * PDSIG5(JL,JK)+ PRAYL5(JL) * PDSIG (JL,JK)
      ZTRAY5 = PRAYL5(JL) * PDSIG5(JL,JK)

!      ZRATIO = ZTRAY / (ZTRAY5 + PTAUAZ5(JL,JK))&
!       & - ZTRAY5* (ZTRAY  + PTAUAZ (JL,JK))&
!       & /((ZTRAY5 + PTAUAZ5(JL,JK))**2)  
!      ZRATIO5= ZTRAY5 / (ZTRAY5 + PTAUAZ5(JL,JK))

      ZFF  = 2 * PCGAZ (JL,JK) * PCGAZ5(JL,JK)
      ZFF5 = PCGAZ5(JL,JK) * PCGAZ5(JL,JK)
!--
      ZDENB = ZTRAY +PTAUAZ(JL,JK)*(1.0_JPRB-PPIZAZ5(JL,JK)*ZFF5) &
        &   +PTAUAZ5(JL,JK)*( -PPIZAZ(JL,JK)*ZFF5 -PPIZAZ5(JL,JK)*ZFF)
      ZDENB5= ZTRAY5+PTAUAZ5(JL,JK)*(1.0_JPRB-PPIZAZ5(JL,JK)*ZFF5)
      ZIDENB5 = 1.0_JPRB/ZDENB5
      ZRATIO= ZTRAY*ZIDENB5 - ZTRAY5*ZDENB*(ZIDENB5*ZIDENB5)
      ZRATIO5=ZTRAY5 * ZIDENB5

      PTAUAZ (JL,JK)=ZTRAY +PTAUAZ (JL,JK)*(1.0_JPRB-PPIZAZ5(JL,JK)*&
       & ZFF5)&
       & +PTAUAZ5(JL,JK)*(  -PPIZAZ (JL,JK)*ZFF5-PPIZAZ5(JL,JK)*ZFF )  
      PTAUAZ5(JL,JK)=ZTRAY5+PTAUAZ5(JL,JK)*(1.0_JPRB-PPIZAZ5(JL,JK)*ZFF5)
      ZDIV15 = 1.0_JPRB/(1.0_JPRB + PCGAZ5(JL,JK))
      PCGAZ (JL,JK) = PCGAZ (JL,JK) *(1.0_JPRB - ZRATIO5) * ZDIV15 &
       & - PCGAZ5(JL,JK) * ZRATIO * ZDIV15 &
       & - PCGAZ5(JL,JK)*(1.0_JPRB - ZRATIO5) *PCGAZ (JL,JK) * (ZDIV15**2)
      PCGAZ5(JL,JK) = PCGAZ5(JL,JK)*(1.0_JPRB - ZRATIO5) * ZDIV15
      ZDIV25 = 1.0_JPRB/(1.0_JPRB - PPIZAZ5(JL,JK) * ZFF5)
      PPIZAZ (JL,JK) =ZRATIO &
       & - ( ZRATIO*PPIZAZ5(JL,JK)*(1.0_JPRB-ZFF5)&
       & +(1.0_JPRB-ZRATIO5)*PPIZAZ (JL,JK)*(1.0_JPRB-ZFF5)&
       & -(1.0_JPRB-ZRATIO5)*PPIZAZ5(JL,JK)*ZFF ) * ZDIV25 &
       & -(1.0_JPRB-ZRATIO5)*PPIZAZ5(JL,JK)*(1.0_JPRB-ZFF5)&
       & *( - PPIZAZ (JL,JK) * ZFF5 &
       & - PPIZAZ5(JL,JK) * ZFF  ) * (ZDIV25**2)  
      PPIZAZ5(JL,JK)= ZRATIO5 &
       & + (1.0_JPRB-ZRATIO5)*PPIZAZ5(JL,JK)*(1.0_JPRB-ZFF5) * ZDIV25  
    ELSE
      ZTRAY  = PRAYL (JL) * PDSIG5(JL,JK)+ PRAYL5(JL) * PDSIG (JL,JK)
      ZTRAY5 = PRAYL5(JL) * PDSIG5(JL,JK)
      PTAUAZ (JL,JK) = ZTRAY
      PTAUAZ5(JL,JK) = ZTRAY5
      PCGAZ  (JL,JK) = 0.0_JPRB
      PCGAZ5 (JL,JK) = 0.0_JPRB
      PPIZAZ (JL,JK) = 0.0_JPRB
      PPIZAZ5(JL,JK) = 1.0_JPRB-REPSCT
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.    TOTAL EFFECTIVE CLOUDINESS ABOVE A GIVEN LEVEL
!                ----------------------------------------------

! 200 CONTINUE

DO JL = KIDIA,KFDIA
  ZC0I (JL,KLEV+1)= 0.0_JPRB
  ZC0I5(JL,KLEV+1)= 0.0_JPRB
  ZCLEAR (JL)     = 0.0_JPRB
  ZCLEAR5(JL)     = 1.0_JPRB
  ZSCAT (JL)      = 0.0_JPRB
  ZSCAT5(JL)      = 0.0_JPRB
ENDDO

DO JK = 1 , KLEV
  IKL = KLEV+1 - JK
  IKLP1 = IKL + 1
  DO JL = KIDIA,KFDIA
    ZFACOA =     - PPIZAZ (JL,IKL)*  PCGAZ5(JL,IKL)**2 &
     & - PPIZAZ5(JL,IKL)*2*PCGAZ (JL,IKL)*PCGAZ5(JL,IKL)  
    ZFACOA5 = 1.0_JPRB - PPIZAZ5(JL,IKL)*  PCGAZ5(JL,IKL)**2
    ZCORAE  = ZFACOA  * PTAUAZ5(JL,IKL) * PSEC5(JL)&
     & + ZFACOA5 * PTAUAZ (JL,IKL) * PSEC5(JL)&
     & + ZFACOA5 * PTAUAZ5(JL,IKL) * PSEC (JL)  
    ZCORAE5 = ZFACOA5 * PTAUAZ5(JL,IKL) * PSEC5(JL)
    ZR215(JL) = EXP(-ZCORAE5 )
    ZR21 (JL) = -ZCORAE * ZR215(JL)
    ZSS0 (JL) =   -ZR21 (JL)
    ZSS05(JL) = 1.0_JPRB-ZR215(JL)
    IF (NOVLP == 1 .OR. NOVLP == 4) THEN
!* maximum-random      
      IF (ZSS05(JL) >= ZSCAT5(JL)) THEN
        ZNUM = ZSS0 (JL)
        ZNUM5= ZSS05(JL)
      ELSE
        ZNUM  = ZSCAT (JL)
        ZNUM5 = ZSCAT5(JL)
      ENDIF
      IF (ZSCAT5(JL) <= 1.0_JPRB-REPCLC) THEN
        ZDEN  = ZSCAT (JL)
        ZDEN5 = ZSCAT5(JL)
      ELSE
        ZDEN  = 0.0_JPRB
        ZDEN5 = 1.0_JPRB-REPCLC
      ENDIF
      ZICLEAR5 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
      ZCLEAR(JL) = ZCLEAR (JL) * (1.0_JPRB-ZNUM5) * ZICLEAR5&
       & - ZCLEAR5(JL) * ZNUM * ZICLEAR5&
       & + ZCLEAR5(JL) * (1.0_JPRB-ZNUM5) * ZDEN *(ZICLEAR5**2)  
      ZCLEAR5(JL)= ZCLEAR5(JL) * (1.0_JPRB-ZNUM5) * ZICLEAR5
      ZC0I (JL,IKL) =     - ZCLEAR (JL)
      ZC0I5(JL,IKL) = 1.0_JPRB - ZCLEAR5(JL)
      ZSCAT (JL) = ZSS0 (JL)
      ZSCAT5(JL) = ZSS05(JL)
    ELSEIF (NOVLP == 2) THEN
!* maximum
      IF (ZSS05(JL) >= ZSCAT5(JL)) THEN
        ZSCAT (JL) = ZSS0 (JL)
        ZSCAT5(JL) = ZSS05(JL)
      ENDIF
      ZC0I (JL,IKL) = ZSCAT (JL)
      ZC0I5(JL,IKL) = ZSCAT5(JL)
    ELSEIF (NOVLP == 3) THEN
!* random
      ZCLEAR (JL)=ZCLEAR (JL)*(1.0_JPRB-ZSS05(JL))-ZCLEAR5(JL)*ZSS0 (JL)
      ZCLEAR5(JL)=ZCLEAR5(JL)*(1.0_JPRB-ZSS05(JL))
      ZSCAT (JL) =     - ZCLEAR (JL)
      ZSCAT5(JL) = 1.0_JPRB - ZCLEAR5(JL)
      ZC0I (JL,IKL) = ZSCAT (JL)
      ZC0I5(JL,IKL) = ZSCAT5(JL)
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         3.    REFLECTIVITY/TRANSMISSIVITY FOR PURE SCATTERING
!                -----------------------------------------------

! 300 CONTINUE

DO JL = KIDIA,KFDIA
  PRAY1 (JL,KLEV+1) = 0.0_JPRB
  PRAY15(JL,KLEV+1) = 0.0_JPRB
  PRAY2 (JL,KLEV+1) = 0.0_JPRB
  PRAY25(JL,KLEV+1) = 0.0_JPRB
  PREFZ (JL,2,1) = PALBP (JL,KNU)
  PREFZ5(JL,2,1) = PALBP5(JL,KNU)
  PREFZ (JL,1,1) = PALBP (JL,KNU)
  PREFZ5(JL,1,1) = PALBP5(JL,KNU)
  PTRA1 (JL,KLEV+1) = 0.0_JPRB
  PTRA15(JL,KLEV+1) = 1.0_JPRB
  PTRA2 (JL,KLEV+1) = 0.0_JPRB
  PTRA25(JL,KLEV+1) = 1.0_JPRB
ENDDO

DO JK = 2 , KLEV+1
  IKM1 = JK-1
  DO JL = KIDIA,KFDIA

!     ------------------------------------------------------------------

!*         3.1  EQUIVALENT ZENITH ANGLE
!               -----------------------

! 310 CONTINUE

    ZMUE  = (  -ZC0I (JL,JK)) * PSEC5(JL)&
     & + (1.0_JPRB-ZC0I5(JL,JK)) * PSEC (JL)&
     & + ZC0I (JL,JK) * 1.66_JPRB  
    ZMUE5 = (1.0_JPRB-ZC0I5(JL,JK)) * PSEC5(JL)+ ZC0I5(JL,JK) * 1.66_JPRB
    PRMU05(JL,JK) = 1.0_JPRB/ZMUE5
    PRMU0 (JL,JK) = -ZMUE*(PRMU05(JL,JK)**2)

!     ------------------------------------------------------------------

!*         3.2  REFLECT./TRANSMISSIVITY DUE TO RAYLEIGH AND AEROSOLS
!               ----------------------------------------------------

! 320 CONTINUE

    ZBMU0 =      - 0.75_JPRB * PCGAZ (JL,IKM1)  * PRMU05(JL,JK) &
     & - 0.75_JPRB * PCGAZ5(JL,IKM1) * PRMU0(JL,JK)
    ZBMU05 = 0.5_JPRB - 0.75_JPRB * PCGAZ5(JL,IKM1) * PRMU05(JL,JK)
    ZDEN0  = ( - PPIZAZ (JL,IKM1) + ZBMU0*PPIZAZ5(JL,IKM1) &
     & + ZBMU05*PPIZAZ (JL,IKM1) ) * PTAUAZ5(JL,IKM1) * ZMUE5 &
     & + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU05 * PPIZAZ5(JL,IKM1)) * PTAUAZ (JL,IKM1) * ZMUE5  &
     & + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU05 * PPIZAZ5(JL,IKM1)) * PTAUAZ5(JL,IKM1) * ZMUE   &
     & + ( -PPIZAZ (JL,IKM1) ) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 * ZMUE5**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (- PPIZAZ (JL,IKM1)  &
     & + 2.0_JPRB*ZBMU0 *PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ (JL,IKM1)) * PTAUAZ5(JL,IKM1)**2 * ZMUE5**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1)) &
     & * (2*PTAUAZ (JL,IKM1)*PTAUAZ5(JL,IKM1))*ZMUE5**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 &
     & * (2*ZMUE*ZMUE5)  
    ZDEN05= 1.0_JPRB + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU05 * PPIZAZ5(JL,IKM1)) * PTAUAZ5(JL,IKM1) * ZMUE5   &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 * ZMUE5**2 
    PTRA15(JL,IKM1) = 1.0_JPRB/ZDEN05
    PTRA1 (JL,IKM1) = -ZDEN0*(PTRA15(JL,IKM1)**2)
    PRAY1 (JL,IKM1) = ZBMU0 * PPIZAZ5(JL,IKM1) * PTAUAZ5(JL,IKM1) &
     & * ZMUE5 * PTRA15(JL,IKM1) &
     & + ZBMU05* PPIZAZ (JL,IKM1)*PTAUAZ5(JL,IKM1)*ZMUE5 * PTRA15(JL,IKM1) &
     & + ZBMU05* PPIZAZ5(JL,IKM1)*PTAUAZ (JL,IKM1)*ZMUE5 * PTRA15(JL,IKM1) &
     & + ZBMU05* PPIZAZ5(JL,IKM1)*PTAUAZ5(JL,IKM1)*ZMUE  * PTRA15(JL,IKM1) &
     & + ZBMU05* PPIZAZ5(JL,IKM1)*PTAUAZ5(JL,IKM1)*ZMUE5 * PTRA1 (JL,IKM1)  
    PRAY15(JL,IKM1) = ZBMU05 * PPIZAZ5(JL,IKM1) * PTAUAZ5(JL,IKM1) &
     & * ZMUE5  * PTRA15(JL,IKM1)

    ZBMU1  =     - 0.75_JPRB * PCGAZ (JL,IKM1)  * 0.5_JPRB
    ZBMU15 = 0.5_JPRB - 0.75_JPRB * PCGAZ5(JL,IKM1) * 0.5_JPRB
    ZDEN1  = (( - PPIZAZ (JL,IKM1) + ZBMU1*PPIZAZ5(JL,IKM1) &
     & + ZBMU15*PPIZAZ (JL,IKM1) ) * PTAUAZ5(JL,IKM1)  &
     & + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU15 * PPIZAZ5(JL,IKM1)) * PTAUAZ (JL,IKM1) ) * 2.0_JPRB &
     & + (( -PPIZAZ (JL,IKM1) ) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2  &
     & + (1.0_JPRB-PPIZAZ5(JL,IKM1)) * (- PPIZAZ (JL,IKM1)  &
     & + 2.0_JPRB*ZBMU1 *PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ (JL,IKM1) )*PTAUAZ5(JL,IKM1)**2 &
     & + (1.0_JPRB-PPIZAZ5(JL,IKM1))*(1.0_JPRB- PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1))*2.0_JPRB*PTAUAZ (JL,IKM1) &
     & * PTAUAZ5(JL,IKM1))* 4.0_JPRB  
    ZDEN15= 1.0_JPRB + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU15 * PPIZAZ5(JL,IKM1)) * PTAUAZ5(JL,IKM1) *2.0_JPRB    &
     & + (1.0_JPRB-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 * 4.0_JPRB    
    PTRA25(JL,IKM1) = 1.0_JPRB/ZDEN15
    PTRA2 (JL,IKM1) = -ZDEN1*(PTRA25(JL,IKM1)**2)
    PRAY2 (JL,IKM1) = 2.0_JPRB*(ZBMU1 * PPIZAZ5(JL,IKM1) * PTAUAZ5(JL,IKM1) &
     & * PTRA25(JL,IKM1) &
     & + ZBMU15 * PPIZAZ (JL,IKM1) * PTAUAZ5(JL,IKM1) * PTRA25(JL,IKM1)&
     & + ZBMU15 * PPIZAZ5(JL,IKM1) * PTAUAZ (JL,IKM1) * PTRA25(JL,IKM1)&
     & + ZBMU15 * PPIZAZ5(JL,IKM1) * PTAUAZ5(JL,IKM1) * PTRA2 (JL,IKM1))  
    PRAY25(JL,IKM1) = 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1)* PTAUAZ5(JL,IKM1) &
     & * PTRA25(JL,IKM1)

    ZRR5=1.0_JPRB/ (1.0_JPRB-PRAY25(JL,IKM1)*PREFZ5(JL,1,IKM1))
    ZRR=(PRAY2(JL,IKM1)*PREFZ5(JL,1,IKM1)+PRAY25(JL,IKM1)*PREFZ(JL,1,IKM1)) &
     & *ZRR5**2  
    PREFZ (JL,1,JK) =  PRAY1 (JL,IKM1)&
     & + ( PREFZ (JL,1,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)&
     & + PREFZ5(JL,1,IKM1) * PTRA1 (JL,IKM1) * PTRA25(JL,IKM1)&
     & + PREFZ5(JL,1,IKM1) * PTRA15(JL,IKM1) * PTRA2 (JL,IKM1) )*ZRR5&
     & + PREFZ5(JL,1,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)* ZRR  
    PREFZ5(JL,1,JK) =  PRAY15(JL,IKM1)+  &
     & PREFZ5(JL,1,IKM1) * PTRA15(JL,IKM1)* PTRA25(JL,IKM1)*ZRR5  

    ZTR (JL,1,IKM1) =  PTRA1 (JL,IKM1)*ZRR5+ PTRA15(JL,IKM1)*ZRR
    ZTR5(JL,1,IKM1) =  PTRA15(JL,IKM1)*ZRR5

    PREFZ (JL,2,JK) =  PRAY1 (JL,IKM1)&
     & + PREFZ (JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA1 (JL,IKM1) * PTRA25(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA2 (JL,IKM1)  
    PREFZ5(JL,2,JK) =  PRAY15(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)   

    ZTR (JL,2,IKM1) = PTRA1 (JL,IKM1)
    ZTR5(JL,2,IKM1) = PTRA15(JL,IKM1)

  ENDDO
ENDDO

DO JL = KIDIA,KFDIA
  ZMUE  = (  -ZC0I (JL,1))*PSEC5(JL)&
   & + (1.0_JPRB-ZC0I5(JL,1))*PSEC (JL)+ZC0I (JL,1)*1.66_JPRB  
  ZMUE5 = (1.0_JPRB-ZC0I5(JL,1))*PSEC5(JL)+ZC0I5(JL,1)*1.66_JPRB
  PRMU0 (JL,1)=-ZMUE/(ZMUE5**2)
  PRMU05(JL,1)=1.0_JPRB/ZMUE5
ENDDO

!     ------------------------------------------------------------------

!*         3.5    REFLECT./TRANSMISSIVITY BETWEEN SURFACE AND LEVEL
!                 -------------------------------------------------

! 350 CONTINUE

IF (NSW <= 4) THEN
  INU1=1
ELSEIF (NSW == 6) THEN
  INU1=3
ENDIF

IF (KNU <= INU1) THEN
  JAJ = 2
  DO JL = KIDIA,KFDIA
    PRJ0 (JL,JAJ,KLEV+1) = 0.0_JPRB
    PRJ05(JL,JAJ,KLEV+1) = 1.0_JPRB
    PRK0 (JL,JAJ,KLEV+1) = PREFZ (JL, 1,KLEV+1)
    PRK05(JL,JAJ,KLEV+1) = PREFZ5(JL, 1,KLEV+1)
  ENDDO

  DO JK = 1 , KLEV
    IKL = KLEV+1 - JK
    IKLP1 = IKL + 1
    DO JL = KIDIA,KFDIA
      ZRE11 = PRJ0 (JL,JAJ,IKLP1) * ZTR5(JL,  1,IKL)&
       & + PRJ05(JL,JAJ,IKLP1) * ZTR (JL,  1,IKL)  
      ZRE115= PRJ05(JL,JAJ,IKLP1) * ZTR5(JL,  1,IKL)
      PRJ0 (JL,JAJ,IKL) = ZRE11
      PRJ05(JL,JAJ,IKL) = ZRE115
      PRK0 (JL,JAJ,IKL) = ZRE11  * PREFZ5(JL,  1,IKL)&
       & + ZRE115 * PREFZ (JL,  1,IKL)  
      PRK05(JL,JAJ,IKL) = ZRE115 * PREFZ5(JL,  1,IKL)
    ENDDO
  ENDDO

ELSE

  DO JAJ = 1 , 2
    DO JL = KIDIA,KFDIA
      PRJ0 (JL,JAJ,KLEV+1) = 0.0_JPRB
      PRJ05(JL,JAJ,KLEV+1) = 1.0_JPRB
      PRK0 (JL,JAJ,KLEV+1) = PREFZ (JL,JAJ,KLEV+1)
      PRK05(JL,JAJ,KLEV+1) = PREFZ5(JL,JAJ,KLEV+1)
    ENDDO

    DO JK = 1 , KLEV
      IKL = KLEV+1 - JK
      IKLP1 = IKL + 1
      DO JL = KIDIA,KFDIA
        ZRE11 = PRJ0 (JL,JAJ,IKLP1) * ZTR5(JL,JAJ,IKL)&
         & + PRJ05(JL,JAJ,IKLP1) * ZTR (JL,JAJ,IKL)  
        ZRE115= PRJ05(JL,JAJ,IKLP1) * ZTR5(JL,JAJ,IKL)
        PRJ0 (JL,JAJ,IKL) = ZRE11
        PRJ05(JL,JAJ,IKL) = ZRE115
        PRK0 (JL,JAJ,IKL) = ZRE11  * PREFZ5(JL,JAJ,IKL)&
         & + ZRE115 * PREFZ (JL,JAJ,IKL)  
        PRK05(JL,JAJ,IKL) = ZRE115 * PREFZ5(JL,JAJ,IKL)
      ENDDO
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SWCLRTL',1,ZHOOK_HANDLE)
END SUBROUTINE SWCLRTL
