SUBROUTINE SWRTL &
 & ( KIDIA  , KFDIA  , KLON  , KLEV   , KNU,&
 & PALBD5 , PCG5   , PCLD5 , POMEGA5, PSEC5 , PTAU5,&
 & PCGAZ5 , PPIZAZ5, PRAY15, PRAY25 , PREFZ5, PRJ5  , PRK5,&
 & PRMUE5 , PTAUAZ5, PTRA15, PTRA25,&
 & PALBD  , PCG    , PCLD  , POMEGA , PSEC  , PTAU,&
 & PCGAZ  , PPIZAZ , PRAY1 , PRAY2  , PREFZ , PRJ   , PRK,&
 & PRMUE  , PTAUAZ , PTRA1 , PTRA2 &
 & )  

!**** *SWRTL* - CONTINUUM SCATTERING COMPUTATIONS
!               -> TANGENT LINEAR <-

!     PURPOSE.
!     --------
!           COMPUTES THE REFLECTIVITY AND TRANSMISSIVITY IN CASE OF
!     CONTINUUM SCATTERING

!**   INTERFACE.
!     ----------

!          *SWRTL* IS CALLED EITHER FROM *SW1STL*, *SWNITL*

!        IMPLICIT ARGUMENTS :
!        --------------------

!     ==== INPUTS ===
!     ==== OUTPUTS ===

!     METHOD.
!     -------

!          1. COMPUTES CONTINUUM FLUXES CORRESPONDING TO AEROSOL
!     OR/AND RAYLEIGH SCATTERING (NO MOLECULAR GAS ABSORPTION)

!     EXTERNALS.
!     ----------

!          *SWDE*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT
!        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-09-27
!        Modified : 2002-05-07 M.Janiskova - 6 spectral intervals and
!                                            additional overlap assumption
!        03-10-10 Deborah Salmond and Marta Janiskova Optimisation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        08-03-27 M.Janiskova - code optimization and cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOERAD   , ONLY : NOVLP    ,NSW
USE YOECLD   , ONLY : REPSEC
USE YOEOVLP  , ONLY : RA1OVLP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCG5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU5(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCGAZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPIZAZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY15(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY25(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PREFZ5(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRJ5(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRK5(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRMUE5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUAZ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA15(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA25(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCG(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU(KLON,NSW,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCGAZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPIZAZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAY2(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PREFZ(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRJ(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRK(KLON,6,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRMUE(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUAZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRA2(KLON,KLEV+1) 
!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!     ------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZC1I5(KLON,KLEV+1)&
 & ,  ZCLEAR5(KLON)         , ZCLOUD5(KLON) &
 & ,  ZGG5(KLON)            , ZRE15(KLON)   , ZRE25(KLON)&
 & ,  ZR215(KLON)           , ZR225(KLON)&
 & ,  ZSS15(KLON)           , ZTO15(KLON)   , ZTR5(KLON,2,KLEV+1)&
 & ,  ZTR15(KLON)           , ZTR25(KLON)&
 & ,  ZW5(KLON)  

REAL(KIND=JPRB) :: ZC1I(KLON,KLEV+1)&
 & ,  ZCLEAR(KLON)         , ZCLOUD(KLON) &
 & ,  ZGG(KLON)            , ZRE1(KLON)   , ZRE2(KLON)&
 & ,  ZR21(KLON)           , ZR22(KLON)&
 & ,  ZSS1(KLON)           , ZTO1(KLON)   , ZTR(KLON,2,KLEV+1)&
 & ,  ZTR1(KLON)           , ZTR2(KLON)&
 & ,  ZW(KLON)  

INTEGER(KIND=JPIM) :: IKL, IKLP1, IKM1, JA, JAJ, JK, JL, INU1

REAL(KIND=JPRB) :: ZBMU0, ZBMU05, ZBMU1, ZBMU15, ZCORAE, ZCORAE5,&
 & ZCORCD, ZCORCD5, ZDEN, ZDEN1, ZDEN15, ZDEN5, &
 & ZFACOA, ZFACOA5, ZFACOC, ZFACOC5, &
 & ZMUE, ZMUE5, ZNUM, ZNUM5, ZRE11, ZRE115, &
 & ZALPHA1, ZCHKAE, ZCHKAE5, ZCHKCD, ZCHKCD5  
REAL(KIND=JPRB) :: ZRR,ZRR5
REAL(KIND=JPRB) :: ZICLEAR5, ZDIV15, ZDIV25, ZDIV35, ZDIV45
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "swdetl.intfb.h"

!     ------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

! 100 CONTINUE

IF (LHOOK) CALL DR_HOOK('SWRTL',0,ZHOOK_HANDLE)
DO JK = 1 , KLEV+1
  DO JA = 1 , 6
    DO JL = KIDIA,KFDIA
      PRJ (JL,JA,JK) = 0.0_JPRB
      PRJ5(JL,JA,JK) = 0.0_JPRB
      PRK (JL,JA,JK) = 0.0_JPRB
      PRK5(JL,JA,JK) = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.    TOTAL EFFECTIVE CLOUDINESS ABOVE A GIVEN LEVEL
!                ----------------------------------------------

! 200 CONTINUE

DO JL = KIDIA,KFDIA
  ZC1I (JL,KLEV+1) = 0.0_JPRB
  ZC1I5(JL,KLEV+1) = 0.0_JPRB
  ZCLEAR (JL) = 0
  ZCLEAR5(JL) = 1.0_JPRB
  ZCLOUD (JL) = 0.0_JPRB
  ZCLOUD5(JL) = 0.0_JPRB
ENDDO

DO JK = 1 , KLEV
  IKL = KLEV+1 - JK
  IKLP1 = IKL + 1
  ZALPHA1=RA1OVLP( IKL )
  DO JL = KIDIA,KFDIA
    ZFACOA  =    - PPIZAZ (JL,IKL)*PCGAZ5(JL,IKL)**2 &
     & - PPIZAZ5(JL,IKL)*2*PCGAZ(JL,IKL)*PCGAZ5(JL,IKL)  
    ZFACOA5 = 1.0_JPRB - PPIZAZ5(JL,IKL)*PCGAZ5(JL,IKL)**2
    ZFACOC  =    - POMEGA (JL,KNU,IKL) * PCG5(JL,KNU,IKL)**2 &
     & - POMEGA5(JL,KNU,IKL) * 2*PCG (JL,KNU,IKL)&
     & * PCG5(JL,KNU,IKL)  
    ZFACOC5 = 1.0_JPRB - POMEGA5(JL,KNU,IKL) * PCG5(JL,KNU,IKL)&
     & * PCG5(JL,KNU,IKL)  
    ZCORAE  = ZFACOA  * PTAUAZ5(JL,IKL) * PSEC5(JL)&
     & + ZFACOA5 * PTAUAZ (JL,IKL) * PSEC5(JL)&
     & + ZFACOA5 * PTAUAZ5(JL,IKL) * PSEC (JL)  
    ZCORAE5 = ZFACOA5 * PTAUAZ5(JL,IKL) * PSEC5(JL)
    ZCORCD  = ZFACOC  * PTAU5(JL,KNU,IKL) * PSEC5(JL)&
     & + ZFACOC5 * PTAU (JL,KNU,IKL) * PSEC5(JL)&
     & + ZFACOC5 * PTAU5(JL,KNU,IKL) * PSEC (JL)  
    ZCORCD5 = ZFACOC5 * PTAU5(JL,KNU,IKL) * PSEC5(JL)

    IF (ZCORAE5 <= 200._JPRB) THEN
      ZCHKAE  =  ZCORAE
      ZCHKAE5 =  ZCORAE5
    ELSE
      ZCHKAE  =  0.0_JPRB
      ZCHKAE5 =  200._JPRB
    ENDIF
    
    IF (ZCORCD5 <= 200._JPRB) THEN
      ZCHKCD  = ZCORCD
      ZCHKCD5 = ZCORCD5
    ELSE
      ZCHKCD  = 0.0_JPRB
      ZCHKCD5 = 200._JPRB
    ENDIF
    
    ZR215(JL) = EXP( - ZCHKAE5 )
    ZR21 (JL) = -ZCHKAE*ZR215(JL)
    ZR225(JL) = EXP( - ZCHKCD5 )
    ZR22 (JL) = -ZCHKCD*ZR225(JL)
        
    ZSS1 (JL) = PCLD (JL,IKL)*(1.0_JPRB-ZR215(JL)*ZR225(JL))&
     & + PCLD5(JL,IKL)*(   -ZR21 (JL)*ZR225(JL)&
     & -ZR215(JL)*ZR22 (JL))&
     & + (   -PCLD (JL,IKL))*(1.0_JPRB-ZR215(JL))&
     & + (1.0_JPRB-PCLD5(JL,IKL))*(   -ZR21 (JL))  
    ZSS15(JL) = PCLD5(JL,IKL)*(1.0_JPRB-ZR215(JL)*ZR225(JL))&
     & + (1.0_JPRB-PCLD5(JL,IKL))*(1.0_JPRB-ZR215(JL))  

    IF (NOVLP == 1) THEN
!* maximum-random      
      IF (ZSS15(JL) >= ZCLOUD5(JL)) THEN
        ZNUM = ZSS1 (JL)
        ZNUM5= ZSS15(JL)
      ELSE
        ZNUM = ZCLOUD (JL)
        ZNUM5= ZCLOUD5(JL)
      ENDIF
      IF (ZCLOUD5(JL) <= 1.0_JPRB-REPSEC) THEN
        ZDEN = ZCLOUD (JL)
        ZDEN5= ZCLOUD5(JL)
      ELSE
        ZDEN = 0.0_JPRB
        ZDEN5= 1.0_JPRB-REPSEC
      ENDIF
      ZICLEAR5 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
!      ZICLEAR  = ZDEN*(ZICLEAR5**2)  ! directly imported in ZCLEAR
      ZCLEAR(JL) = ZCLEAR (JL) * (1.0_JPRB-ZNUM5) * ZICLEAR5&
       & - ZCLEAR5(JL) * ZNUM * ZICLEAR5&
       & + ZCLEAR5(JL) * (1.0_JPRB-ZNUM5) * ZDEN *(ZICLEAR5**2)  
      ZCLEAR5(JL)= ZCLEAR5(JL) * (1.0_JPRB-ZNUM5) * ZICLEAR5
      ZC1I (JL,IKL) =     - ZCLEAR (JL)
      ZC1I5(JL,IKL) = 1.0_JPRB - ZCLEAR5(JL)
      ZCLOUD (JL) = ZSS1 (JL)
      ZCLOUD5(JL) = ZSS15(JL)
    ELSEIF (NOVLP == 2) THEN
!* maximum
      IF (ZSS15(JL) >= ZCLOUD5(JL)) THEN
        ZCLOUD (JL) =  ZSS1 (JL)
        ZCLOUD5(JL) =  ZSS15(JL)
      ENDIF
      ZC1I (JL,IKL) = ZCLOUD (JL)
      ZC1I5(JL,IKL) = ZCLOUD5(JL)
    ELSEIF (NOVLP == 3) THEN
!* random
      ZCLEAR (JL) = ZCLEAR (JL)*(1.0_JPRB - ZSS15(JL))&
       & + ZCLEAR5(JL)*(    - ZSS1(JL) )  
      ZCLEAR5(JL) = ZCLEAR5(JL)*(1.0_JPRB - ZSS15(JL))
      ZCLOUD (JL) =     - ZCLEAR (JL)
      ZCLOUD5(JL) = 1.0_JPRB - ZCLEAR5(JL)
      ZC1I (JL,IKL) = ZCLOUD (JL)
      ZC1I5(JL,IKL) = ZCLOUD5(JL)
    ELSEIF (NOVLP == 4) THEN
!* Hogan & Illingworth (2001)
      IF (ZSS15(JL) >= ZCLOUD5(JL)) THEN
        ZNUM  = ZSS1 (JL)
        ZNUM5 = ZSS15(JL)
      ELSE
        ZNUM  = ZCLOUD (JL)
        ZNUM5 = ZCLOUD5(JL)
      ENDIF
      IF (ZCLOUD5(JL) <= 1.0_JPRB-REPSEC) THEN
        ZDEN = ZCLOUD (JL)
        ZDEN5= ZCLOUD5(JL)
      ELSE
        ZDEN = 0.0_JPRB
        ZDEN5= 1.0_JPRB-REPSEC
      ENDIF
      ZICLEAR5 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
!      ZICLEAR  = ZDEN*(ZICLEAR5**2)  ! directly imported in ZCLEAR
      ZCLEAR (JL) = -(ZALPHA1*ZCLEAR5(JL)*ZICLEAR5)*ZNUM&
       & + (ZALPHA1*ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*ZICLEAR5**2) * ZDEN &
       & - (1.0_JPRB-ZALPHA1)*ZCLEAR5(JL)*ZSS1(JL) &
       & + (ZALPHA1*(1.0_JPRB-ZNUM5)*ZICLEAR5 &
       & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-ZSS15(JL)))*ZCLEAR(JL)  
      ZCLEAR5(JL)=ZCLEAR5(JL) * (ZALPHA1*(1.0_JPRB-ZNUM5)*ZICLEAR5 &
       & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-ZSS15(JL)) )  
      ZC1I (JL,IKL) = - ZCLEAR(JL)
      ZC1I5(JL,IKL) = 1.0_JPRB - ZCLEAR5(JL)
      ZCLOUD (JL) = ZSS1 (JL)
      ZCLOUD5(JL) = ZSS15(JL)
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         3.    REFLECTIVITY/TRANSMISSIVITY FOR PURE SCATTERING
!                -----------------------------------------------

! 300 CONTINUE

DO JL = KIDIA,KFDIA
  PRAY1 (JL,KLEV+1) = 0.0_JPRB
  PRAY15(JL,KLEV+1) = 0.0_JPRB
  PRAY2 (JL,KLEV+1) = 0.0_JPRB
  PRAY25(JL,KLEV+1) = 0.0_JPRB
  PREFZ (JL,2,1) = PALBD (JL,KNU)
  PREFZ5(JL,2,1) = PALBD5(JL,KNU)
  PREFZ (JL,1,1) = PALBD (JL,KNU)
  PREFZ5(JL,1,1) = PALBD5(JL,KNU)
  PTRA1 (JL,KLEV+1) = 0.0_JPRB
  PTRA15(JL,KLEV+1) = 1.0_JPRB
  PTRA2 (JL,KLEV+1) = 0.0_JPRB
  PTRA25(JL,KLEV+1) = 1.0_JPRB
ENDDO

DO JK = 2 , KLEV+1
  IKM1 = JK-1
  DO JL = KIDIA,KFDIA

!     ------------------------------------------------------------------

!*         3.1  EQUIVALENT ZENITH ANGLE
!               -----------------------

! 310 CONTINUE

    ZMUE = (  -ZC1I (JL,JK)) * PSEC5(JL)&
     & + (1.0_JPRB-ZC1I5(JL,JK)) * PSEC (JL)&
     & + ZC1I (JL,JK) * 1.66_JPRB  
    ZMUE5 = (1.0_JPRB-ZC1I5(JL,JK)) * PSEC5(JL)+ ZC1I5(JL,JK) * 1.66_JPRB
    PRMUE5(JL,JK) = 1.0_JPRB/ZMUE5
    PRMUE (JL,JK) = -ZMUE*PRMUE5(JL,JK)**2

!     ------------------------------------------------------------------

!*         3.2  REFLECT./TRANSMISSIVITY DUE TO RAYLEIGH AND AEROSOLS
!               ----------------------------------------------------

! 320 CONTINUE

    ZBMU0  = - 0.75_JPRB * PCGAZ5(JL,IKM1) * PRMUE (JL,JK) &
     & - 0.75_JPRB * PCGAZ (JL,IKM1) * PRMUE5(JL,JK)
    ZBMU05 = 0.5_JPRB - 0.75_JPRB * PCGAZ5(JL,IKM1) * PRMUE5(JL,JK)

    ZDEN =( - PPIZAZ(JL,IKM1) + ZBMU0*PPIZAZ5(JL,IKM1) &
     & + ZBMU05*PPIZAZ(JL,IKM1) ) * PTAUAZ5(JL,IKM1) * ZMUE5 &
     & + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU05*PPIZAZ5(JL,IKM1)) * PTAUAZ(JL,IKM1)  * ZMUE5 &
     & + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU05*PPIZAZ5(JL,IKM1)) * PTAUAZ5(JL,IKM1) * ZMUE &
     & + ( -PPIZAZ(JL,IKM1) ) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 *ZMUE5**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (   - PPIZAZ(JL,IKM1)  &
     & + 2.0_JPRB*ZBMU0 *PPIZAZ5(JL,IKM1)+2.0_JPRB*ZBMU05*PPIZAZ(JL,IKM1) )&
     & * PTAUAZ5(JL,IKM1)**2 *ZMUE5**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*2*PTAUAZ(JL,IKM1) &
     & * PTAUAZ5(JL,IKM1)*ZMUE5**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & +2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 *2*ZMUE *ZMUE5  
    ZDEN5 = 1.0_JPRB + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU05 * PPIZAZ5(JL,IKM1)) * PTAUAZ5(JL,IKM1) * ZMUE5 &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU05*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 *ZMUE5**2 
    PTRA15(JL,IKM1) = 1.0_JPRB/ZDEN5
    PTRA1 (JL,IKM1) = -ZDEN*(PTRA15(JL,IKM1)**2)
    PRAY1 (JL,IKM1) = ZBMU0*PPIZAZ5(JL,IKM1)*PTAUAZ5(JL,IKM1) &
     & * ZMUE5*PTRA15(JL,IKM1) &
     & + ZBMU05*PPIZAZ (JL,IKM1)*PTAUAZ5(JL,IKM1)*ZMUE5*PTRA15(JL,IKM1) &
     & + ZBMU05*PPIZAZ5(JL,IKM1)*PTAUAZ (JL,IKM1)*ZMUE5*PTRA15(JL,IKM1) &
     & + ZBMU05*PPIZAZ5(JL,IKM1)*PTAUAZ5(JL,IKM1)*ZMUE *PTRA15(JL,IKM1) &
     & + ZBMU05*PPIZAZ5(JL,IKM1)*PTAUAZ5(JL,IKM1)*ZMUE5*PTRA1(JL,IKM1)  
    PRAY15(JL,IKM1) = ZBMU05*PPIZAZ5(JL,IKM1)*PTAUAZ5(JL,IKM1) &
     & * ZMUE5*PTRA15(JL,IKM1)

    ZBMU1  =  - 0.75_JPRB * PCGAZ(JL,IKM1)  * 0.5_JPRB
    ZBMU15 = 0.5_JPRB - 0.75_JPRB * PCGAZ5(JL,IKM1) * 0.5_JPRB
    ZDEN1  =(( - PPIZAZ(JL,IKM1) + ZBMU1*PPIZAZ5(JL,IKM1) &
     & + ZBMU15*PPIZAZ(JL,IKM1) ) * PTAUAZ5(JL,IKM1)  &
     & + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU15 * PPIZAZ5(JL,IKM1)) * PTAUAZ(JL,IKM1) ) * 2.0_JPRB   &
     & + (( -PPIZAZ(JL,IKM1) ) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (   - PPIZAZ(JL,IKM1)  &
     & + 2.0_JPRB*ZBMU1 *PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ(JL,IKM1) )*PTAUAZ5(JL,IKM1)**2 &
     & + (1-PPIZAZ5(JL,IKM1)) * (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1))*2*PTAUAZ(JL,IKM1) &
     & * PTAUAZ5(JL,IKM1))*4.0_JPRB  
    ZDEN15= 1.0_JPRB + (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + ZBMU15 * PPIZAZ5(JL,IKM1)) * PTAUAZ5(JL,IKM1) * 2.0_JPRB &
     & + (1-PPIZAZ5(JL,IKM1))* (1.0_JPRB - PPIZAZ5(JL,IKM1) &
     & + 2.0_JPRB*ZBMU15*PPIZAZ5(JL,IKM1))*PTAUAZ5(JL,IKM1)**2 * 4.0_JPRB  
    PTRA25(JL,IKM1) = 1.0_JPRB/ZDEN15
    PTRA2 (JL,IKM1) = -ZDEN1*(PTRA25(JL,IKM1)**2)
    PRAY2 (JL,IKM1) = 2.0_JPRB*(ZBMU1 * PPIZAZ5(JL,IKM1) &
     & * PTAUAZ5(JL,IKM1) * PTRA25(JL,IKM1)&
     & + ZBMU15 * PPIZAZ(JL,IKM1)  * PTAUAZ5(JL,IKM1) *  PTRA25(JL,IKM1) &
     & + ZBMU15 * PPIZAZ5(JL,IKM1) * PTAUAZ(JL,IKM1)  *  PTRA25(JL,IKM1) &
     & + ZBMU15 * PPIZAZ5(JL,IKM1) * PTAUAZ5(JL,IKM1) *  PTRA2(JL,IKM1))  
    PRAY25(JL,IKM1) = 2.0_JPRB * ZBMU15 * PPIZAZ5(JL,IKM1) &
     & * PTAUAZ5(JL,IKM1) * PTRA25(JL,IKM1)

!     ------------------------------------------------------------------

!*         3.3  EFFECT OF CLOUD LAYER
!               ---------------------

! 330 CONTINUE

    ZDIV15 = 1.0_JPRB/POMEGA5(JL,KNU,IKM1)
    ZDIV25 = 1.0_JPRB/PPIZAZ5(JL,IKM1)
    ZTO1 (JL) = PTAU (JL,KNU,IKM1)*ZDIV15&
     & - PTAU5(JL,KNU,IKM1)*POMEGA(JL,KNU,IKM1)*(ZDIV15**2)&
     & + PTAUAZ (JL,IKM1)*ZDIV25&
     & - PTAUAZ5(JL,IKM1)*PPIZAZ(JL,IKM1)*(ZDIV25**2)  
    ZTO15(JL) = PTAU5(JL,KNU,IKM1)*ZDIV15 + PTAUAZ5(JL,IKM1)*ZDIV25
    ZR21 (JL) = PTAU (JL,KNU,IKM1) + PTAUAZ (JL,IKM1)
    ZR215(JL) = PTAU5(JL,KNU,IKM1) + PTAUAZ5(JL,IKM1)
    ZDIV35 = 1.0_JPRB/ZR215(JL)
    ZR22 (JL) = PTAU (JL,KNU,IKM1) * ZDIV35&
     & - PTAU5(JL,KNU,IKM1) * ZR21(JL) *(ZDIV35**2)  
    ZR225(JL) = PTAU5(JL,KNU,IKM1) * ZDIV35
    ZGG (JL) = ZR22 (JL) * PCG5(JL,KNU,IKM1)&
     & + ZR225(JL) * PCG (JL,KNU,IKM1)&
     & + (   - ZR22 (JL)) * PCGAZ5(JL,IKM1)&
     & + (1.0_JPRB - ZR225(JL)) * PCGAZ (JL,IKM1)  
    ZGG5(JL) = ZR225(JL) * PCG5(JL,KNU,IKM1)&
     & + (1.0_JPRB - ZR225(JL)) * PCGAZ5(JL,IKM1)  
    IF (POMEGA5(JL,KNU,IKM1)==1.0_JPRB .AND. PPIZAZ5(JL,IKM1)==1.0_JPRB) THEN
      ZW (JL)=0.0_JPRB
      ZW5(JL)=1.0_JPRB
    ELSE
      ZDIV45 = 1.0_JPRB/ZTO15(JL)
      ZW (JL) = ZR21 (JL) * ZDIV45 - ZR215(JL) * ZTO1(JL)*(ZDIV45**2)
      ZW5(JL) = ZR215(JL) * ZDIV45
    ENDIF

  ENDDO

  CALL SWDETL &
   & ( KIDIA, KFDIA , KLON,&
   & ZGG5  , PREFZ5(1,1,IKM1), PRMUE5(1,JK) , ZTO15 , ZW5,&
   & ZRE15 , ZRE25  , ZTR15  , ZTR25,&
   & ZGG   , PREFZ (1,1,IKM1), PRMUE (1,JK) , ZTO1  , ZW,&
   & ZRE1  , ZRE2   , ZTR1   , ZTR2 &
   & )  

  DO JL = KIDIA,KFDIA

    ZRR5=1.0_JPRB/(1.0_JPRB-PRAY25(JL,IKM1)*PREFZ5(JL,1,IKM1))
    ZRR=(PRAY2(JL,IKM1)*PREFZ5(JL,1,IKM1)+PRAY25(JL,IKM1)*PREFZ(JL,1,IKM1)) &
     & *ZRR5**2  

    PREFZ (JL,1,JK) =&
     & PCLD(JL,IKM1) * ZRE25(JL)+ PCLD5(JL,IKM1) * ZRE2 (JL)&
     & - (PRAY15(JL,IKM1)&
     & + PREFZ5(JL,1,IKM1)  * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)*ZRR5)&
     & * PCLD(JL,IKM1) &
     & +(PRAY1 (JL,IKM1)&
     & + PREFZ (JL,1,IKM1)  * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)*ZRR5 &
     & + PREFZ5(JL,1,IKM1)  * PTRA1 (JL,IKM1) * PTRA25(JL,IKM1)*ZRR5 &
     & + PREFZ5(JL,1,IKM1)  * PTRA15(JL,IKM1) * PTRA2 (JL,IKM1)*ZRR5 &
     & + PREFZ5(JL,1,IKM1)  * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)*ZRR) &
     & * (1.0_JPRB-PCLD5(JL,IKM1))   

    PREFZ5(JL,1,JK) =&
     & PCLD5(JL,IKM1) * ZRE25(JL)&
     & +(PRAY15(JL,IKM1)+ &
     & PREFZ5(JL,1,IKM1)*PTRA15(JL,IKM1)* PTRA25(JL,IKM1)*ZRR5)* &
     & (1.0_JPRB-PCLD5(JL,IKM1))  

    ZTR (JL,1,IKM1) =&
     & PCLD(JL,IKM1) * ZTR25(JL) + PCLD5(JL,IKM1) * ZTR2 (JL) &
     & + PTRA1 (JL,IKM1)*ZRR5* (1.0_JPRB-PCLD5(JL,IKM1))&
     & + PTRA15(JL,IKM1)*ZRR * (1.0_JPRB-PCLD5(JL,IKM1))&
     & - PTRA15(JL,IKM1)*ZRR5* PCLD(JL,IKM1)  

    ZTR5(JL,1,IKM1) = PCLD5(JL,IKM1) * ZTR25(JL) &
     & + PTRA15(JL,IKM1)*ZRR5 * (1.0_JPRB-PCLD5(JL,IKM1))  

    PREFZ (JL,2,JK) =&
     & (  -PCLD(JL,IKM1)) * (PRAY15(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1) )&
     & +(1.0_JPRB-PCLD5(JL,IKM1)) * (PRAY1 (JL,IKM1)&
     & + PREFZ (JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA1 (JL,IKM1) * PTRA25(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA2 (JL,IKM1) )&
     & + PCLD(JL,IKM1) * ZRE15(JL)&
     & + PCLD5(JL,IKM1) * ZRE1 (JL)  

    PREFZ5(JL,2,JK) =&
     & (1.0_JPRB-PCLD5(JL,IKM1)) * (PRAY15(JL,IKM1)&
     & + PREFZ5(JL,2,IKM1) * PTRA15(JL,IKM1) * PTRA25(JL,IKM1) )&
     & + PCLD5(JL,IKM1) * ZRE15(JL)  

    ZTR (JL,2,IKM1) = PCLD(JL,IKM1) * ZTR15(JL)&
     & + PCLD5(JL,IKM1) * ZTR1 (JL)&
     & + PTRA1 (JL,IKM1) * (1.0_JPRB-PCLD5(JL,IKM1))&
     & + PTRA15(JL,IKM1) * (  -PCLD(JL,IKM1))  

    ZTR5(JL,2,IKM1) = PCLD5(JL,IKM1) * ZTR15(JL)&
     & + PTRA15(JL,IKM1) * (1.0_JPRB-PCLD5(JL,IKM1))  

  ENDDO
ENDDO
DO JL = KIDIA,KFDIA
  ZMUE  = (  -ZC1I (JL,1))*PSEC5(JL)&
   & + (1.0_JPRB-ZC1I5(JL,1))*PSEC (JL)+ZC1I (JL,1)*1.66_JPRB  
  ZMUE5 = (1.0_JPRB-ZC1I5(JL,1))*PSEC5(JL)+ZC1I5(JL,1)*1.66_JPRB
  PRMUE5(JL,1)=1.0_JPRB/ZMUE5
  PRMUE (JL,1)=-ZMUE*PRMUE5(JL,1)**2
ENDDO

!     ------------------------------------------------------------------

!*         3.5    REFLECT./TRANSMISSIVITY BETWEEN SURFACE AND LEVEL
!                 -------------------------------------------------

! 350 CONTINUE

IF (NSW <= 4) THEN
  INU1=1
ELSEIF (NSW == 6) THEN
  INU1=3
ENDIF

IF (KNU <= INU1) THEN
  JAJ = 2
  DO JL = KIDIA,KFDIA
    PRJ (JL,JAJ,KLEV+1) = 0.0_JPRB
    PRJ5(JL,JAJ,KLEV+1) = 1.0_JPRB
    PRK (JL,JAJ,KLEV+1) = PREFZ (JL, 1,KLEV+1)
    PRK5(JL,JAJ,KLEV+1) = PREFZ5(JL, 1,KLEV+1)
  ENDDO

  DO JK = 1 , KLEV
    IKL = KLEV+1 - JK
    IKLP1 = IKL + 1
    DO JL = KIDIA,KFDIA
      ZRE11 = PRJ (JL,JAJ,IKLP1) * ZTR5(JL,  1,IKL)&
       & + PRJ5(JL,JAJ,IKLP1) * ZTR (JL,  1,IKL)  
      ZRE115= PRJ5(JL,JAJ,IKLP1) * ZTR5(JL,  1,IKL)
      PRJ (JL,JAJ,IKL) = ZRE11
      PRJ5(JL,JAJ,IKL) = ZRE115
      PRK (JL,JAJ,IKL) = ZRE11  * PREFZ5(JL,  1,IKL)&
       & + ZRE115 * PREFZ (JL,  1,IKL)  
      PRK5(JL,JAJ,IKL) = ZRE115 * PREFZ5(JL,  1,IKL)
    ENDDO
  ENDDO

ELSE

  DO JAJ = 1 , 2
    DO JL = KIDIA,KFDIA
      PRJ (JL,JAJ,KLEV+1) = 0.0_JPRB
      PRJ5(JL,JAJ,KLEV+1) = 1.0_JPRB
      PRK (JL,JAJ,KLEV+1) = PREFZ (JL,JAJ,KLEV+1)
      PRK5(JL,JAJ,KLEV+1) = PREFZ5(JL,JAJ,KLEV+1)
    ENDDO

    DO JK = 1 , KLEV
      IKL = KLEV+1 - JK
      IKLP1 = IKL + 1
      DO JL = KIDIA,KFDIA
        ZRE11 = PRJ (JL,JAJ,IKLP1) * ZTR5(JL,JAJ,IKL)&
         & + PRJ5(JL,JAJ,IKLP1) * ZTR (JL,JAJ,IKL)  
        ZRE115= PRJ5(JL,JAJ,IKLP1) * ZTR5(JL,JAJ,IKL)
        PRJ (JL,JAJ,IKL) = ZRE11
        PRJ5(JL,JAJ,IKL) = ZRE115
        PRK (JL,JAJ,IKL) = ZRE11  * PREFZ5(JL,JAJ,IKL)&
         & + ZRE115 * PREFZ (JL,JAJ,IKL)  
        PRK5(JL,JAJ,IKL) = ZRE115 * PREFZ5(JL,JAJ,IKL)
      ENDDO
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SWRTL',1,ZHOOK_HANDLE)
END SUBROUTINE SWRTL
