!OPTIONS XOPT(HSFUN)
SUBROUTINE SWUAD &
 & ( KIDIA , KFDIA  , KLON   , KLEV,&
 !- trajectory inputs     
 & PSCT5 , PCARDI5, PCLDSW5, PPMB5 , PPSOL5,&
 & PRMU05, PTAVE5 , PWV5,&
 !- trajectory outputs     
 & PAKI5 , PCLD5  , PCLEAR5, PDSIG5, PFACT5, PRMU5,&
 & PSEC5 , PUD5,&
 !- perturbation inputs     
 & PSCT  , PCARDI , PCLDSW , PPMB  , PPSOL,&
 & PRMU0 , PTAVE  , PWV,&
 !- perturbation outputs     
 & PAKI  , PCLD   , PCLEAR , PDSIG , PFACT , PRMU,&
 & PSEC  , PUD &
 & )  

!**** *SWUAD* - SHORTWAVE RADIATION, ABSORBER AMOUNTS
!               -> ADJOINT <-

!     PURPOSE.
!     --------
!           COMPUTES THE ABSORBER AMOUNTS USED IN SHORTWAVE RADIATION
!     CALCULATIONS

!**   INTERFACE.
!     ----------
!          *SWUAD* IS CALLED BY *SWAD*

!        IMPLICIT ARGUMENTS :
!        --------------------

!     ==== INPUTS ===
!     ==== OUTPUTS ===

!     METHOD.
!     -------

!          1. COMPUTES ABSORBER AMOUNTS WITH TEMPERATURE AND PRESSURE
!     SCALING.

!     EXTERNALS.
!     ----------

!          *SWTT1*
!          *SWTT1AD*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT
!        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-10-18
!        Modified : 99-05-26 M.Janiskova - corrections of AD code
!        Modified : 2002-05-06 M. Janiskova - including Hogan&Illingworth
!                                             overlap assumption
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        2008-03-27 M.Janiskova - code optimization
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOECLD   , ONLY : REPSEC
USE YOERAD   , ONLY : NOVLP    ,NSW
USE YOERDU   , ONLY : REPSCQ
USE YOESW    , ONLY : RPDH1    ,RPDU1    ,RPNH     ,RPNU     ,&
 & RTDH2O   ,RTDUMG   ,RTH2O    ,RTUMG  
USE YOEOVLP  , ONLY : RA1OVLP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSCT5 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCARDI5 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDSW5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPMB5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPSOL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMU05(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAKI5(KLON,2,NSW) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLEAR5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDSIG5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFACT5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRMU5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSEC5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUD5(KLON,5,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSCT 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCARDI 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLDSW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPMB(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPSOL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAKI(KLON,2,NSW) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLEAR(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDSIG(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFACT(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSEC(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUD(KLON,5,KLEV+1) 
!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!- trajectory inputs

!- trajectory outputs

!- perturbation inputs

!- perturbation outputs

INTEGER(KIND=JPIM) :: INUIR

!     ------------------------------------------------------------------

!              ------------

INTEGER(KIND=JPIM) :: IIND(2)

REAL(KIND=JPRB) :: ZC1J5(KLON,KLEV+1)&
 & ,  ZCLEAR5(KLON,KLEV+1), ZCLOUD5(KLON,KLEV+1)&
 & ,  Z1755(KLON,KLEV+1)  , Z1905(KLON,KLEV+1)&
 & ,  ZSIG5(KLON,KLEV+1)&
 & ,  ZUD5(KLON,2)  

REAL(KIND=JPRB) ::&
 & ZDEN5(KLON,KLEV)    , ZNUM5(KLON,KLEV)&
 & ,  ZDSH2O5(KLON,KLEV)  , ZDSCO25(KLON,KLEV)&
 & ,  ZFPPW5(KLON,KLEV)&
 & ,  ZRTH5(KLON,KLEV)    , ZRTU5(KLON,KLEV)&
 & ,  ZWH2O5(KLON,KLEV)&
 & ,  ZR5(KLON,2,NSW)     ,  ZUD15(KLON,2)  
REAL(KIND=JPRB) :: ZITAVE5 (KLON,KLEV), ZIWH2O5(KLON,KLEV)&
 & ,  ZICLEAR5(KLON,KLEV+1), ZLOG15(KLON,2,NSW)
REAL(KIND=JPRB) :: ZIPSOL5(KLON), ZICLOUD5(KLON)

REAL(KIND=JPRB) :: ZC1J(KLON,KLEV+1)&
 & ,  ZCLEAR(KLON,KLEV+1) , ZCLOUD(KLON,KLEV+1)&
 & ,  Z175(KLON,KLEV+1)   , Z190(KLON,KLEV+1)&
 & ,  ZSIG(KLON,KLEV+1)&
 & ,  ZR(KLON,2)          , ZUD(KLON,2)  

INTEGER(KIND=JPIM) :: IKL, IKLP1, IKP1, JA, JK, JL, JNU

REAL(KIND=JPRB) :: ZNUM, ZDEN, ZFPPW, ZWH2O, ZDSCO2, ZRTU, ZRTH&
 & ,  ZDSH2O, ZALPHA1
REAL(KIND=JPRB) :: ZITAVE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "swtt1.intfb.h"
#include "swtt1ad.intfb.h"

!     ------------------------------------------------------------------

!*         1.     COMPUTES AMOUNTS OF ABSORBERS
!                 -----------------------------

! 100 CONTINUE

IF (LHOOK) CALL DR_HOOK('SWUAD',0,ZHOOK_HANDLE)
IIND(1)=1
IIND(2)=2

!*         1.1    INITIALIZES QUANTITIES
!                 ----------------------

! 110 CONTINUE

DO JL = KIDIA,KFDIA
  PUD5(JL,1,KLEV+1)=0.0_JPRB
  PUD5(JL,2,KLEV+1)=0.0_JPRB
  PUD5(JL,3,KLEV+1)=0.0_JPRB
  PUD5(JL,4,KLEV+1)=0.0_JPRB
  PUD5(JL,5,KLEV+1)=0.0_JPRB
  PFACT5(JL)= PRMU05(JL)*PSCT5
  PRMU5(JL) = PRMU05(JL)
  PSEC5(JL) = 1.0_JPRB/PRMU5(JL)
  ZC1J5(JL,KLEV+1)=0.0_JPRB
ENDDO

!*          1.3    AMOUNTS OF ABSORBERS
!                  --------------------

! 130 CONTINUE

DO JL= KIDIA,KFDIA
  ZUD15(JL,1) = 0.0_JPRB
  ZUD15(JL,2) = 0.0_JPRB
  Z1755(JL,1) = PPSOL5(JL)**RPDU1
  Z1905(JL,1) = PPSOL5(JL)**RPDH1
  ZSIG5(JL,1) = PPSOL5(JL)
  ZCLEAR5(JL,KLEV+1)=1.0_JPRB
  ZCLOUD5(JL,KLEV+1)=0.0_JPRB
ENDDO

DO JK = 1 , KLEV
  IKP1 = JK + 1
  IKL = KLEV+1 - JK
  IKLP1 = IKL+1
  ZALPHA1=RA1OVLP(KLEV+1-JK)
  
  DO JL = KIDIA,KFDIA
    ZITAVE5(JL,JK) = 1.0_JPRB/PTAVE5(JL,JK)
    ZRTH5(JL,JK)=(RTH2O*ZITAVE5(JL,JK))**RTDH2O
    ZRTU5(JL,JK)=(RTUMG*ZITAVE5(JL,JK))**RTDUMG
    IF (PWV5(JL,IKL) >= REPSCQ) THEN
      ZWH2O5(JL,JK) = PWV5(JL,IKL)
    ELSE
      ZWH2O5(JL,JK) = REPSCQ
    ENDIF
    ZSIG5(JL,IKP1) = 100._JPRB * PPMB5(JL,IKP1)

    ZIPSOL5(JL) = 1.0_JPRB/PPSOL5(JL)
    PDSIG5(JL,JK) = (ZSIG5(JL,JK) - ZSIG5(JL,IKP1))*ZIPSOL5(JL)
    Z1755(JL,IKP1) = ZSIG5(JL,IKP1) ** RPDU1
    Z1905(JL,IKP1) = ZSIG5(JL,IKP1) ** RPDH1

    ZDSCO25(JL,JK) = Z1755(JL,JK) - Z1755(JL,IKP1)
    ZDSH2O5(JL,JK) = Z1905(JL,JK) - Z1905(JL,IKP1)
    PUD5(JL,1,JK) = RPNH*ZDSH2O5(JL,JK)*ZWH2O5(JL,JK)*ZRTH5(JL,JK)
    PUD5(JL,2,JK) = RPNU*ZDSCO25(JL,JK)*PCARDI5*ZRTU5(JL,JK)
    PUD5(JL,3,JK) = 0.0_JPRB

    ZIWH2O5(JL,JK) = 1.0_JPRB/(1.0_JPRB+0.608_JPRB*ZWH2O5(JL,JK))
    ZFPPW5(JL,JK)=1.6078_JPRB*ZWH2O5(JL,JK)*ZIWH2O5(JL,JK)
    PUD5(JL,4,JK)=PUD5(JL,1,JK)*ZFPPW5(JL,JK)
    PUD5(JL,5,JK)=PUD5(JL,1,JK)*(1.0_JPRB-ZFPPW5(JL,JK))

    ZUD15(JL,1) = ZUD15(JL,1) + PUD5(JL,1,JK)
    ZUD15(JL,2) = ZUD15(JL,2) + PUD5(JL,2,JK)

!-

    IF (NOVLP == 1) THEN
      IF (PCLDSW5(JL,IKL) >= ZCLOUD5(JL,IKLP1)) THEN
        ZNUM5(JL,IKL)= PCLDSW5(JL,IKL)
      ELSE
        ZNUM5(JL,IKL)= ZCLOUD5(JL,IKLP1)
      ENDIF
      IF (ZCLOUD5(JL,IKLP1) <= 1.0_JPRB-REPSEC) THEN
        ZDEN5(JL,IKL)= ZCLOUD5(JL,IKLP1)
      ELSE
        ZDEN5(JL,IKL)= 1.0_JPRB-REPSEC
      ENDIF
      ZICLEAR5(JL,IKL) = 1.0_JPRB/(1.0_JPRB-ZDEN5(JL,IKL))
      ZCLEAR5(JL,IKL) = ZCLEAR5(JL,IKLP1)&
       & *(1.0_JPRB-ZNUM5(JL,IKL))*ZICLEAR5(JL,IKL)  
      ZC1J5(JL,IKL)= 1.0_JPRB - ZCLEAR5(JL,IKL)
      ZCLOUD5(JL,IKL) = PCLDSW5(JL,IKL)
    ELSEIF (NOVLP == 2) THEN
      IF (PCLDSW5(JL,IKL) >= ZCLOUD5(JL,IKLP1)) THEN
        ZCLOUD5(JL,IKL) = PCLDSW5(JL,IKL)
      ELSE
        ZCLOUD5(JL,IKL) = ZCLOUD5(JL,IKLP1)
      ENDIF
      ZC1J5(JL,IKL) = ZCLOUD5(JL,IKL)
    ELSEIF (NOVLP == 3) THEN
      ZCLEAR5(JL,IKL)   = ZCLEAR5(JL,IKLP1)*(1.0_JPRB-PCLDSW5(JL,IKL))
      ZCLOUD5(JL,IKL)   = 1.0_JPRB - ZCLEAR5(JL,IKL)
      ZC1J5(JL,IKL) = ZCLOUD5(JL,IKL)
    ELSEIF (NOVLP == 4) THEN
      IF (PCLDSW5(JL,IKL) >= ZCLOUD5(JL,IKLP1)) THEN
        ZNUM5(JL,IKL) = PCLDSW5(JL,IKL)
      ELSE
        ZNUM5(JL,IKL) = ZCLOUD5(JL,IKLP1)
      ENDIF
      IF (ZCLOUD5(JL,IKLP1) <= 1.0_JPRB-REPSEC) THEN
        ZDEN5(JL,IKL) = ZCLOUD5(JL,IKLP1)
      ELSE
        ZDEN5(JL,IKL) = 1.0_JPRB-REPSEC
      ENDIF
      ZICLEAR5(JL,IKL) = 1.0_JPRB/(1.0_JPRB-ZDEN5(JL,IKL))
      ZCLEAR5(JL,IKL)=ZCLEAR5(JL,IKLP1) &
       & * (ZALPHA1*(1.0_JPRB-ZNUM5(JL,IKL))*ZICLEAR5(JL,IKL) &
       & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-PCLDSW5(JL,IKL)) )  
      ZC1J5(JL,IKL) = 1.0_JPRB - ZCLEAR5(JL,IKL)
      ZCLOUD5(JL,IKL) = PCLDSW5(JL,IKL) 
    ENDIF
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  PCLEAR5(JL)=1.0_JPRB-ZC1J5(JL,1)
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (PCLEAR5(JL) < 1.0_JPRB) THEN
      ZICLOUD5(JL) = 1.0_JPRB/(1.0_JPRB-PCLEAR5(JL))
      PCLD5(JL,JK)=PCLDSW5(JL,JK)*ZICLOUD5(JL)
    ELSE
      PCLD5(JL,JK)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO

!*         1.4    COMPUTES CLEAR-SKY GREY ABSORPTION COEFFICIENTS
!                 -----------------------------------------------

! 140 CONTINUE

DO JA = 1, 2
  DO JL=KIDIA,KFDIA
    ZUD5(JL,JA) = ZUD15(JL,JA) * PSEC5(JL)
  ENDDO
ENDDO

!mj DO JNU= 1,NSW

IF (NSW <= 4) THEN
  INUIR=2
ELSEIF (NSW == 6) THEN
  INUIR=4
ENDIF

DO JNU= INUIR,NSW

  CALL SWTT1( KIDIA, KFDIA, KLON, JNU, 2, IIND, ZUD5, ZR5(1,1,JNU) )

  DO JA = 1,2
    DO JL = KIDIA,KFDIA
      ZLOG15(JL,JA,JNU) = LOG(ZR5(JL,JA,JNU))
      PAKI5(JL,JA,JNU) = - ZLOG15(JL,JA,JNU) / ZUD5(JL,JA)
    ENDDO
  ENDDO

ENDDO

!     ==================================================================

!          10.   ADJOINT CALCULATIONS
!                --------------------

ZC1J(:,:)   = 0.0_JPRB
ZCLEAR(:,:) = 0.0_JPRB
ZCLOUD(:,:) = 0.0_JPRB
Z175(:,:)   = 0.0_JPRB
Z190(:,:)   = 0.0_JPRB
ZSIG(:,:)   = 0.0_JPRB
ZR(:,:)     = 0.0_JPRB
ZUD(:,:)    = 0.0_JPRB

!mj DO JNU=NSW,1,-1

DO JNU=NSW,INUIR,-1

  DO JA=2,1,-1
    DO JL=KIDIA,KFDIA
      ZR(JL,JA)  = ZR(JL,JA) - PAKI(JL,JA,JNU)/(ZR5(JL,JA,JNU)*ZUD5(JL,JA))
      ZUD(JL,JA)= ZUD(JL,JA)&
       & + PAKI(JL,JA,JNU)*ZLOG15(JL,JA,JNU)/(ZUD5(JL,JA)**2)  
      PAKI(JL,JA,JNU) = 0.0_JPRB
    ENDDO
  ENDDO

  CALL SWTT1AD( KIDIA, KFDIA, KLON, JNU, 2, IIND,&
   & ZUD5, ZR5(1,1,JNU) , ZUD , ZR  )

ENDDO

DO JA=2,1,-1
  DO JL=KIDIA,KFDIA
    PSEC(JL) = PSEC(JL) + ZUD(JL,JA) * ZUD15(JL,JA)
    ZUD(JL,JA) = ZUD(JL,JA) * PSEC5(JL) 
  ENDDO
ENDDO

!1140 CONTINUE

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (PCLEAR5(JL) < 1.0_JPRB) THEN
      PCLDSW(JL,JK) = PCLDSW(JL,JK) + PCLD(JL,JK)*ZICLOUD5(JL)
      PCLEAR(JL) = PCLEAR(JL)&
       & + PCLD(JL,JK) *PCLDSW5(JL,JK)*(ZICLOUD5(JL)**2)  
    ENDIF
    PCLD(JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  ZC1J(JL,1) = ZC1J(JL,1) - PCLEAR(JL)
  PCLEAR(JL) = 0.0_JPRB
ENDDO

DO JK=KLEV,1,-1
  IKP1=JK+1
  IKL=KLEV+1-JK
  IKLP1=IKL+1
  ZALPHA1=RA1OVLP(KLEV+1-JK)

  DO JL=KIDIA,KFDIA
    ZNUM = 0.0_JPRB
    ZDEN = 0.0_JPRB
    ZFPPW = 0.0_JPRB
    ZWH2O = 0.0_JPRB
    ZDSCO2 = 0.0_JPRB
    ZDSH2O = 0.0_JPRB
    ZRTU = 0.0_JPRB
    ZRTH = 0.0_JPRB
    ZITAVE = 0.0_JPRB

    IF (NOVLP == 4) THEN
      PCLDSW (JL,IKL) = PCLDSW (JL,IKL) + ZCLOUD (JL,IKL)
      ZCLOUD (JL,IKL) = 0.0_JPRB
      
      ZCLEAR(JL,IKL) = ZCLEAR(JL,IKL) - ZC1J (JL,IKL)
      ZC1J (JL,IKL) = 0.0_JPRB

      ZNUM = ZNUM - (ZALPHA1*ZCLEAR5(JL,IKLP1)*ZICLEAR5(JL,IKL)) &
       & * ZCLEAR (JL,IKL)  
      ZDEN = ZDEN + (ZALPHA1*ZCLEAR5(JL,IKLP1) &
       & * (1.0_JPRB-ZNUM5(JL,IKL))*ZICLEAR5(JL,IKL)**2) * ZCLEAR (JL,IKL)  
      PCLDSW(JL,IKL) = PCLDSW(JL,IKL) - (1.0_JPRB-ZALPHA1) &
       & * ZCLEAR5(JL,IKLP1)*ZCLEAR (JL,IKL)  
      ZCLEAR(JL,IKLP1) = ZCLEAR(JL,IKLP1) &
       & + (ZALPHA1*(1.0_JPRB-ZNUM5(JL,IKL))*ZICLEAR5(JL,IKL) &
       & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-PCLDSW5(JL,IKL)))*ZCLEAR (JL,IKL)  
      ZCLEAR (JL,IKL) = 0.0_JPRB
      
      IF (ZCLOUD5(JL,IKLP1) <= 1.0_JPRB-REPSEC) THEN
        ZCLOUD (JL,IKLP1) = ZCLOUD(JL,IKLP1) + ZDEN
      ENDIF
!      ZDEN = 0.0_JPRB

      IF (PCLDSW5(JL,IKL) >= ZCLOUD5(JL,IKLP1)) THEN
        PCLDSW (JL,IKL) = PCLDSW(JL,IKL) + ZNUM
      ELSE
        ZCLOUD (JL,IKLP1) = ZCLOUD(JL,IKLP1) + ZNUM
      ENDIF
!      ZNUM = 0.0_JPRB
      
    ELSEIF (NOVLP == 3) THEN
      ZCLOUD(JL,IKL) = ZCLOUD(JL,IKL) + ZC1J(JL,IKL)
      ZC1J(JL,IKL)=0.0_JPRB

      ZCLEAR(JL,IKL) = ZCLEAR(JL,IKL) - ZCLOUD(JL,IKL)
      ZCLOUD(JL,IKL) = 0.0_JPRB

      ZCLEAR(JL,IKLP1) = ZCLEAR(JL,IKLP1)&
       & + ZCLEAR(JL,IKL)*(1.0_JPRB-PCLDSW5(JL,IKL))  
      PCLDSW(JL,IKL) = PCLDSW(JL,IKL)- ZCLEAR(JL,IKL) * ZCLEAR5(JL,IKLP1)
      ZCLEAR(JL,IKL) = 0.0_JPRB

    ELSEIF (NOVLP == 2) THEN
      ZCLOUD(JL,IKL) = ZCLOUD(JL,IKL) + ZC1J(JL,IKL)
      ZC1J(JL,IKL)   = 0.0_JPRB

      IF (PCLDSW5(JL,IKL) >= ZCLOUD5(JL,IKLP1)) THEN
        PCLDSW(JL,IKL) = PCLDSW(JL,IKL) + ZCLOUD(JL,IKL)
        ZCLOUD(JL,IKL) = 0.0_JPRB
      ELSE
        ZCLOUD(JL,IKLP1) = ZCLOUD(JL,IKLP1)+ ZCLOUD(JL,IKL)
        ZCLOUD(JL,IKL)   = 0.0_JPRB
      ENDIF

    ELSEIF (NOVLP == 1) THEN
      PCLDSW(JL,IKL) = PCLDSW(JL,IKL) + ZCLOUD(JL,IKL)
      ZCLOUD(JL,IKL) = 0.0_JPRB

      ZCLEAR(JL,IKL) = ZCLEAR(JL,IKL) - ZC1J(JL,IKL)
      ZC1J(JL,IKL)   = 0.0_JPRB

      ZCLEAR(JL,IKLP1) = ZCLEAR(JL,IKLP1)&
       & + ZCLEAR(JL,IKL)*(1.0_JPRB-ZNUM5(JL,IKL))*ZICLEAR5(JL,IKL)  
      ZNUM = ZNUM - ZCLEAR(JL,IKL) * ZCLEAR5(JL,IKLP1)*ZICLEAR5(JL,IKL)  
      ZDEN = ZDEN& 
       & + ZCLEAR(JL,IKL) * ZCLEAR5(JL,IKLP1)*(1.0_JPRB-ZNUM5(JL,IKL))&
       & *(ZICLEAR5(JL,IKL)**2)      
      ZCLEAR(JL,IKL) = 0.0_JPRB

      IF (ZCLOUD5(JL,IKLP1) <= 1.0_JPRB-REPSEC) THEN
        ZCLOUD (JL,IKLP1) = ZCLOUD(JL,IKLP1) + ZDEN
      ENDIF
!      ZDEN = 0.0_JPRB

      IF (PCLDSW5(JL,IKL) >= ZCLOUD5(JL,IKLP1)) THEN
        PCLDSW (JL,IKL) = PCLDSW(JL,IKL) + ZNUM
      ELSE
        ZCLOUD (JL,IKLP1) = ZCLOUD(JL,IKLP1) + ZNUM
      ENDIF
!      ZNUM = 0.0_JPRB

    ENDIF

    PUD(JL,1,JK) = PUD(JL,1,JK) + ZUD(JL,1)
    PUD(JL,2,JK) = PUD(JL,2,JK) + ZUD(JL,2)

    PUD(JL,1,JK) = PUD(JL,1,JK) + PUD(JL,5,JK) * (1.0_JPRB-ZFPPW5(JL,JK))
    ZFPPW = ZFPPW - PUD(JL,5,JK) * PUD5(JL,1,JK)
    PUD(JL,5,JK) = 0.0_JPRB

    PUD(JL,1,JK) = PUD(JL,1,JK) + PUD(JL,4,JK) * ZFPPW5(JL,JK)
    ZFPPW = ZFPPW + PUD(JL,4,JK) * PUD5(JL,1,JK)
    PUD(JL,4,JK) = 0.0_JPRB

    ZWH2O = ZWH2O + ZFPPW *1.6078_JPRB*ZIWH2O5(JL,JK)  
    ZWH2O = ZWH2O -1.6078_JPRB*ZFPPW*ZWH2O5(JL,JK)*0.608_JPRB &
     & *(ZIWH2O5(JL,JK)**2)  
!    ZFPPW = 0.0_JPRB

    PUD(JL,3,JK) = 0.0_JPRB

    ZDSCO2 = ZDSCO2 + PUD(JL,2,JK)*RPNU*PCARDI5*ZRTU5(JL,JK)
    PCARDI        = PCARDI+ PUD(JL,2,JK)*RPNU*ZDSCO25(JL,JK)*ZRTU5(JL,JK)
    ZRTU   = ZRTU + PUD(JL,2,JK)*RPNU*ZDSCO25(JL,JK)*PCARDI5
    PUD(JL,2,JK)  = 0.0_JPRB

    ZDSH2O = ZDSH2O&
     & + PUD(JL,1,JK)*RPNH*ZWH2O5(JL,JK)*ZRTH5(JL,JK)          
    ZWH2O  = ZWH2O&
     & + PUD(JL,1,JK)*RPNH*ZDSH2O5(JL,JK)*ZRTH5(JL,JK)  
    ZRTH   = ZRTH&
     & + PUD(JL,1,JK)*RPNH*ZDSH2O5(JL,JK)*ZWH2O5(JL,JK)  
    PUD(JL,1,JK)  = 0.0_JPRB

    Z190(JL,JK)   = Z190(JL,JK) + ZDSH2O
    Z190(JL,IKP1) = Z190(JL,IKP1) - ZDSH2O
    ZDSH2O = 0.0_JPRB

    Z175(JL,JK)   = Z175(JL,JK) + ZDSCO2
    Z175(JL,IKP1) = Z175(JL,IKP1) - ZDSCO2
!    ZDSCO2 = 0.0_JPRB

    ZSIG(JL,IKP1) = ZSIG(JL,IKP1)&
     & + Z190(JL,IKP1)*RPDH1*ZSIG5(JL,IKP1)**(RPDH1-1.0_JPRB)  
    Z190(JL,IKP1) = 0.0_JPRB

    ZSIG(JL,IKP1) = ZSIG(JL,IKP1)&
     & + Z175(JL,IKP1)*RPDU1*ZSIG5(JL,IKP1)**(RPDU1-1.0_JPRB)  
    Z175(JL,IKP1) = 0.0_JPRB

    ZSIG(JL,JK)   = ZSIG(JL,JK) + PDSIG(JL,JK) * ZIPSOL5(JL)
    ZSIG(JL,IKP1) = ZSIG(JL,IKP1) - PDSIG(JL,JK) * ZIPSOL5(JL)
    PPSOL(JL)     = PPSOL(JL) - PDSIG(JL,JK)&
     & *(ZSIG5(JL,JK)-ZSIG5(JL,IKP1))*(ZIPSOL5(JL)**2)  
    PDSIG(JL,JK)  = 0.0_JPRB

    PPMB(JL,IKP1) = PPMB(JL,IKP1) + 100._JPRB * ZSIG(JL,IKP1)
    ZSIG(JL,IKP1) = 0.0_JPRB

    IF (PWV5(JL,IKL) >= REPSCQ) THEN
      PWV(JL,IKL) = PWV(JL,IKL) + ZWH2O
    ENDIF
!    ZWH2O = 0.0_JPRB

    ZITAVE = ZITAVE + ZRTU &
     & * (RTUMG**RTDUMG)*RTDUMG*ZITAVE5(JL,JK)**(RTDUMG-1.0_JPRB)
!    ZRTU  = 0.0_JPRB

    ZITAVE = ZITAVE + ZRTH &
     & * (RTH2O**RTDH2O)*RTDH2O*ZITAVE5(JL,JK)**(RTDH2O-1.0_JPRB)
!    ZRTH  = 0.0_JPRB

    PTAVE(JL,JK) = PTAVE(JL,JK)-ZITAVE*(ZITAVE5(JL,JK)**2)
!   ZTAVE = 0.0_JPRB
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  ZCLOUD(JL,KLEV+1) = 0.0_JPRB
  ZCLEAR(JL,KLEV+1) = 0.0_JPRB

  PPSOL(JL)  = PPSOL(JL) + ZSIG(JL,1)
  ZSIG(JL,1) = 0.0_JPRB

  PPSOL(JL)  = PPSOL(JL) + Z190(JL,1)*RPDH1*PPSOL5(JL)**(RPDH1-1.0_JPRB)
  Z190(JL,1) = 0.0_JPRB

  PPSOL(JL)  = PPSOL(JL) + Z175(JL,1)*RPDU1*PPSOL5(JL)**(RPDU1-1.0_JPRB)
  Z175(JL,1) = 0.0_JPRB

  ZUD(JL,2) = 0.0_JPRB
  ZUD(JL,1) = 0.0_JPRB
ENDDO

!1130 CONTINUE

DO JL=KIDIA,KFDIA
  ZC1J(JL,KLEV+1) = 0.0_JPRB

  PRMU(JL) = PRMU(JL) - PSEC(JL) * (PSEC5(JL)**2)
  PSEC(JL) = 0.0_JPRB

  PRMU0(JL) = PRMU0(JL) + PRMU(JL)
  PRMU(JL)  = 0.0_JPRB

  PRMU0(JL) = PRMU0(JL) + PFACT(JL) * PSCT5
  PSCT      = PSCT + PFACT(JL) * PRMU05(JL)
  PFACT(JL) = 0.0_JPRB

  PUD(JL,5,KLEV+1) = 0.0_JPRB
  PUD(JL,4,KLEV+1) = 0.0_JPRB
  PUD(JL,3,KLEV+1) = 0.0_JPRB
  PUD(JL,2,KLEV+1) = 0.0_JPRB
  PUD(JL,1,KLEV+1) = 0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SWUAD',1,ZHOOK_HANDLE)
END SUBROUTINE SWUAD
