SUBROUTINE AEROD_AD(KDLEN,KLEN,KMXCOUNT,KDF,PTF5,PTF,PAEROD_HAT)  

!**** *AEROD_AD* - Adjoint operator routine for aerosol optical depth observations.

!     A. Benedetti             ECMWF          2005/03/31
!     Based on J.-J. Morcrette's developments for aersol optical depth forward modelling.  

!     PURPOSE
!     -------
!     Simulates total aerosol optical depth :

!**   INTERFACE
!     ---------

!    CALL AEROD_AD(KDLEN,ILEN,IMXCOUNT,IDF,ZTF5,ZTF,ZXPP)  
!     WHERE KDLEN            = First dimension: Max number of observations, this set (INPUT)
!           KLEN             = Number of aerosol optcal depth observations (INPUT)
!           PTF5             = Temperature on full levels (INPUT)
!           PTF              = Temperature perturbation (INOUT)
!           PAEROD_HAT       = Aerosol optical depth perturbation (INOUT)  

!     AEROD_AD is called from HOPAD

!**   METHOD
!     ------
!     Dummy sample program for aerosol optical depth!!!  For now the observational operator is a constant
!     times the surface temperature!!!!  

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     A. Benedetti             ECMWF          2005/03/31            *original*
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN, ONLY : NULOUT
USE YOMDIM, ONLY : NFLEVG
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT, KDF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) ! TEMPERATURE (TRAJECTORY)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG)  ! TEMPERATURE PERTURBATION
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAEROD_HAT(KDLEN,KMXCOUNT,KDF) ! AEROSOL OPTICAL DEPTH

! local variables           
           
INTEGER(KIND=JPIM) :: I, J                   ! loops
REAL(KIND=JPRB) :: ZTSFC                     ! surface temperature perturbation
REAL(KIND=JPRB), PARAMETER :: ALPHA=0.0033   ! multiplicative constant 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('AEROD_AD',0,ZHOOK_HANDLE)

! Initialize surface temperature perturbation
ZTSFC= 0.0_JPRB

! loop through each obs 
DO I = 1, KLEN

  DO J=1, KMXCOUNT

    WRITE(NULOUT,*) 'AEROD_AD: OPTICAL DEPTH PERTURBATION', PAEROD_HAT(I,J,1)
    
    ZTSFC = ZTSFC + ALPHA * PAEROD_HAT(I,J,1)
 
    PAEROD_HAT(I,J,1) = 0.0_JPRB

    PTF(I,NFLEVG)= PTF(I,NFLEVG)+ ZTSFC
    WRITE(NULOUT,*) 'AEROD_AD: PERTURBATION', ZTSFC, PTF(I,NFLEVG)

    ZTSFC=0.0_JPRB

  ENDDO
   
ENDDO

PAEROD_HAT(:,:,:) = 0.0_JPRB

IF (LHOOK) CALL DR_HOOK('AEROD_AD',1,ZHOOK_HANDLE)

END SUBROUTINE AEROD_AD
