SUBROUTINE AEROD_OP(KDLEN,KLEN,KMXCOUNT,KDF,PTF5,PAEROD)  

!*** *AEROD_OP* Operator routine for aerosol optical depth observations.

!     A. Benedetti             ECMWF          2005/03/30
!     Based on J.-J. Morcrette's developments for aersol optical depth forward modelling.  

!     PURPOSE
!     -------
!     Simulates total aerosol optical depth :

!**   INTERFACE
!     ---------

!    CALL AEROD_OP(KDLEN,ILEN,IMXCOUNT,IDF,ZTF5,ZXPP)  
!     WHERE KDLEN  = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol optical depth obs (INPUT)
!           PTF5       = Temperature on full levels (INPUT)
!           PAEROD     = Aerosol optical depth (OUTPUT)  

!     
!     AEROD_OP is called from HOP

!**   METHOD
!     ------
!     Dummy sample program for aerosol optical depth!!!  For now the observational operator is a constant
!     times the surface temperature!!!!  

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     A. Benedetti             ECMWF          2005/03/30            *original*
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM, ONLY : NFLEVG
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT, KDF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) ! TEMPERATURE TRAJ
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAEROD(KDLEN,KMXCOUNT,KDF) ! AEROSOL OPT DEPTH

! local variables           
           
INTEGER(KIND=JPIM) :: I, J                   ! loops
REAL(KIND=JPRB) :: ZTSFC5                    ! surface temperature
REAL(KIND=JPRB), PARAMETER :: ALPHA=0.0033   ! multiplicative constant 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('AEROD_OP',0,ZHOOK_HANDLE)

! initialise zxpp
PAEROD(:,:,:) = 0.0_JPRB

! loop through each obs 
DO I = 1, KLEN

  ZTSFC5 = PTF5(I,NFLEVG) 
      
! Now store the "aerosol optical depth" 
  DO J=1, KMXCOUNT
    PAEROD(I,J,1) = ALPHA * ZTSFC5          
  ENDDO
  
ENDDO

IF (LHOOK) CALL DR_HOOK('AEROD_OP',1,ZHOOK_HANDLE)

END SUBROUTINE AEROD_OP
