SUBROUTINE AEROD_TL(KDLEN,KLEN,KMXCOUNT,KDF,PTF5,PTF,PAEROD_PRIME)  

!**** *aerod_tl* - Tangent linear operator routine for aerosol optical depth observations.

!     A. Benedetti             ECMWF          2005/03/31
!     Based on J.-J. Morcrette's developments for aersol optical depth forward modelling.  

!     PURPOSE
!     -------
!     Simulates total aerosol optical depth :

!**   INTERFACE
!     ---------

!    CALL AEROD_TL(KDLEN,ILEN,IMXCOUNT,IDF,ZTF5,ZTF,ZXPP)  
!     WHERE KDLEN            = First dimension: Max number of observations, this set (INPUT)
!           KLEN             = Number of aerosol optcal depth observations (INPUT)
!           PTF5             = Temperature on full levels (INPUT)
!           PTF              = Temperature perturbation (INPUT)
!           PAEROD_PRIME     = Aerosol optical depth perturbation (OUTPUT)  

!     AEROD_TL is called from HOPTL

!**   METHOD
!     ------
!     Dummy sample program for aerosol optical depth!!!  For now the observational operator is a constant
!     times the surface temperature!!!!  

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     A. Benedetti             ECMWF          2005/03/31            *original*
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM, ONLY : NFLEVG
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT, KDF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) ! TEMPERATURE (TRAJECTORY)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,0:NFLEVG)  ! TEMPERATURE PERTURBATION
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAEROD_PRIME(KDLEN,KMXCOUNT,KDF) ! AEROSOL OPTICAL DEPTH

! local variables           
           
INTEGER(KIND=JPIM) :: I, J                   ! loops
REAL(KIND=JPRB) :: ZTSFC                     ! surface temperature perturbation
REAL(KIND=JPRB), PARAMETER :: ALPHA=0.0033   ! multiplicative constant 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('AEROD_TL',0,ZHOOK_HANDLE)

! initialise zxpp
PAEROD_PRIME(:,:,:) = 0.0_JPRB

! loop through each obs 
DO I = 1, KLEN

  ZTSFC = PTF(I,NFLEVG) 
      
! Now store the "aerosol optical depth" 

  DO J= 1, KMXCOUNT
    PAEROD_PRIME(I,J,1) = ALPHA * ZTSFC           
  ENDDO
   
ENDDO

IF (LHOOK) CALL DR_HOOK('AEROD_TL',1,ZHOOK_HANDLE)

END SUBROUTINE AEROD_TL
