SUBROUTINE APACHE (LDNIVP,KPROMA,KSTART,KPROF,KLEV,KOPLEV,&
 & PT1,PQ1,PU1,PV1,PEPS1,PORO1,PVAH1,PVBH1,&
 & PQL1,PQI1,PQA1,PO31,PSPD1,PWW1,PSN1,PRR1,PGR1,PHL1,&
 & PTKE1,LDPDEP,LDWHALF,LDWFULL,&
 & LDWAT,LDICE,LDCFR,LDO3,LDSN,LDRR,LDGR,LDHL,&
 & LDTKE,PT2,PQ2,PU2,PV2,PEPS2,PORO2,PVAH2,PVBH2,&
 & PQL2,PQI2,PQA2,PO32,PSN2,PRR2,PGR2,PHL2,PTKE2,PSPD2,&
 & PWH2,PWF2,PFI2,PRH2,PEORG,LDOBS,&
 & PEPR2F,PPR2H,PTS1,PTS2,PDPBLC,PV00,LDSPCH,&
 & PDTDZSG)  

!**** *APACHE* - Arpege Pour Arpege CHamps Echange.

!     PURPOSE.
!     --------
!           INTERPOLATION VERTICAL.

!**   INTERFACE.
!     ----------
!        *CALL* *APACHE (LDNIVP,KPROMA,KSTART,KPROF,KLEV,KOPLEV,
!    *           PT1,PQ1,PU1,PV1,PEPS1,PORO1,PVAH1,PVBH1,
!    *           PQL1,PQI1,PQA1,PO31,PSPD1,PWW1,
!    *           PSN1,PRR1,PGR1,PHL1,PTKE1,
!    *           LDPDEP,LDWHALF,LDWFULL,
!    *           LDWAT,LDICE,LDCFR,LDO3,
!    *           LDSN,LDRR,LDGR,LDHL,LDTKE,
!    *           PT2,PQ2,PU2,PV2,PEPS2,PORO2,PVAH2,PVBH2,
!    *           PQL2,PQI2,PQA2,PO32,PSN2,PRR2,PGR2,PHL2,PTKE2,
!    *           PSPD2,PWH2,PWF2,
!    *           PFI2,PRH2,PEORG,LDOBS,
!    *           PEPR2F,PPR2H,PTS1,PTS2,PDPBLC,PV00,LDSPCH,
!    *           PDTDZSG)

!        EXPLICIT ARGUMENTS :
!        --------------------
!        LDNIVP    : Key = .TRUE. Output Pressure System in PEPR2F
!                        = .FALSE.    Computed in VEINE
!        KPROMA    : Horizontal Dimension
!        KSTART    : Starting value for Horizontal Work
!        KPROF     : Depth of Horizontal Work
!        KLEV      : Number of Levels in System 1
!        KOPLEV    : Number of Levels in System 2
!        PT1       : Temperature 1
!        PQ1       : Specific Humidity 1
!        PU1,PV1   : Wind Components 1
!        PEPS1     : Surface Pressure 1
!        PORO1     : Orography 1
!        PVAH1     : *A* Coefficients 1
!        PVBH1     : *B* Coefficients 1
!        PQL1      : Liquid water 1
!        PQI1      : Ice 1
!        PQA1      : CLOUD COVER 1
!        PO31      : ozone mixing ratio 1
!        PSN1      : snow 1
!        PRR1      : rain 1
!        PGR1      : graupel 1
!        PHL1      : hail 1
!        PTKE1     : tke 1
!        PSPD1     : Pressure Departure 1 (unscaled)
!        PWW1      : Vertical Divergence 1 at half levels
!        PORO2     : Orography 2
!        PVAH2     : *A* Coefficients 2  (depending on LDNIVP)
!        PVBH2     : *B* Coefficients 2  (depending on LDNIVP)
!        PEPR2F    : "FULL" Pressures 2  (depending on LDNIVP)
!        PTS1      : Surface Temperature 1
!        PTS2      : Surface Temperature 2
!        PDPBLC    : Critical Thickness of PBL
!        PV00      : Standard Reference Pressure
!        LDSPCH    : .TRUE. : Specific Humidity interpolated
!                    .FALSE.: Relative Humidity interpolated
!        PDTDZSG   : - RDTDZ1/RG
!        LDOBS     : key for outputs needed for obs-operator
!                    .TRUE. : Additional outputs: PHI2, PEORG
!                    .FALSE.: Standard output.
!         !!!        NECESSARY: IF(LDOBS): ALSO LDNIVP       !!!
!         !!!        NECESSARY: IF(LDOBS): ALSO .NOT. LDSPCH !!!
!        LDWAT     : pp of liquid water requested
!        LDICE     : pp of ice requested
!        LDCFR     : pp of cloud fraction requested
!        LDO3      : pp of ozone mixing ratio requested
!        LDSN      : pp of snow requested
!        LDRR      : pp of rain requested
!        LDGR      : pp of graupel requested
!        LDHL      : pp of hail requested
!        LDTKE     : pp of turbulent kinetic energy requested
!        LDPDEP    : pp of NH pressure departure requested
!        LDWHALF   : pp of NH vertic. velocity at half levels requested
!        LDWFULL   : pp of NH vertic. velocity at full levels requested

!  output:

!        PT2       : Temperature 2
!        PQ2       : Specific Humidity 2
!        PRH2      : Relative Humidity 2
!        PU2,PV2   : Wind Components 2
!        PQL2      : Liquid water 2
!        PQI2      : Ice 2
!        PQA2      : CLOUD COVER 2
!        PO32      : ozone mixing ratio 2
!        PSN2      : snow 2
!        PRR2      : rain 2
!        PGR2      : graupel 2
!        PHL2      : hail 2
!        PTKE2     : turbulent kinetic energy 2
!        PSPD2     : Pressure Departure 2 (unscaled)
!        PWH2,PWF2 : Vertical velocity w half and full levels 2
!        PEPS2     : Surface Pressure 2
!        PFI2      : Geopotential 2  (if LDOBS)
!        PEORG     : Pressures of data origin (if LDOBS)

!        IMPLICIT ARGUMENTS
!        --------------------
!                YOMLUN
!                YOMCST

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!          GPPRE
!          GPXYB
!          GPRCP
!          PPINIT
!          VEINE
!          AVAL

!     REFERENCE.
!     ----------
!        ARPEGE/ALADIN DOCUMENTATION.

!     AUTHOR.
!     -------
!        RADMILA BUBNOVA *GMAP/COMPAS - STAGE MICECO*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-10-24
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        Modification :

!           92-02-20:   by Radmila Bubnova (change of interface)
!           92-07-08:   by Radmila Bubnova (CALL GPXYB)
!           92-09-17:   by Radmila Bubnova (Obs-Operator tools)
!           94-05-13:   by Eric Bazile (Liquid Water and Ice)
!           95-02-13:   by Radmila Bubnova (NH variables)
!    R. El Khatib : 97-06-02  3 NH fields
!    R. El Khatib : 98-09-01 Remove unused arg. PTS00 from AVAL
!    R. El Khatib : 98-12-11 Cleanup pp of liquid water
!    R. El Khatib : 00-08-16 VEINE replaced by FPPS - PPLETA - FPVIEW
!    R. El Khatib : 01-08-07 Pruning options
!    Y. Bouteloup : 28-01-05 Introduction of precipitation
!    G. Hello : 05-01-27 snow, rain, graupel and tke
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!    Y. Seity : 07-12-20 hail
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDYNA  , ONLY : LRWSDLR
USE YOMLUN   , ONLY : NULOUT

!     -----------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOPLEV 
LOGICAL           ,INTENT(IN)    :: LDNIVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEPS1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PORO1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAH1(0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH1(0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQL1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQI1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQA1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO31(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRR1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGR1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHL1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPD1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW1(KPROMA,0:KLEV) 
LOGICAL           ,INTENT(IN)    :: LDPDEP 
LOGICAL           ,INTENT(IN)    :: LDWHALF 
LOGICAL           ,INTENT(IN)    :: LDWFULL 
LOGICAL           ,INTENT(IN)    :: LDWAT 
LOGICAL           ,INTENT(IN)    :: LDICE 
LOGICAL           ,INTENT(IN)    :: LDCFR 
LOGICAL           ,INTENT(IN)    :: LDO3 
LOGICAL           ,INTENT(IN)    :: LDSN 
LOGICAL           ,INTENT(IN)    :: LDRR 
LOGICAL           ,INTENT(IN)    :: LDGR 
LOGICAL           ,INTENT(IN)    :: LDHL 
LOGICAL           ,INTENT(IN)    :: LDTKE
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEPS2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PORO2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAH2(0:KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH2(0:KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQL2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQI2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQA2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO32(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRR2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGR2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHL2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPD2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWH2(KPROMA,KOPLEV,0:1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWF2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFI2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRH2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEORG(KPROMA,KOPLEV) 
LOGICAL           ,INTENT(IN)    :: LDOBS 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEPR2F(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPR2H(KPROMA,KOPLEV,0:1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPBLC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV00 
LOGICAL           ,INTENT(IN)    :: LDSPCH 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTDZSG 

!     -----------------------------------------------------------------

!  SYSTEM 1 :

REAL(KIND=JPRB) ::    ZPRESH(KPROMA,0:KLEV),  ZPRESF(KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZGRHL1(KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZLNPR (KPROMA,KLEV)  ,  ZALPH (KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZDELP (KPROMA,KLEV)  ,  ZRDELP(KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZRTGR (KPROMA,KLEV)  ,  ZRPRES(KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZRPP  (KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZPXPD (KPROMA,0:KLEV,4),ZPXP(KPROMA,0:KLEV,4)
REAL(KIND=JPRB) ::    ZR    (KPROMA,KLEV),    ZCP(KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZKAPA (KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZVDELB(KLEV),           ZVC(KLEV)

! SYSTEM ESCALE     :

REAL(KIND=JPRB) ::    ZPESCH(KPROMA,0:KLEV),  ZPESCF(KPROMA,KLEV)
REAL(KIND=JPRB) ::    ZPXDES(KPROMA,0:KLEV,4),ZPXES (KPROMA,0:KLEV,4)

! OUTPUT - SYSTEM 2 :

REAL(KIND=JPRB) ::    ZALFAF(KPROMA,KOPLEV), ZALFAH(KPROMA,KOPLEV,0:1)

REAL(KIND=JPRB) :: ZXLEV(KOPLEV)

INTEGER(KIND=JPIM) :: IPROF, ISTART, JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -----------------------------------------------------------------

#include "abor1.intfb.h"
#include "aval.intfb.h"
#include "fpps.intfb.h"
#include "fpview.intfb.h"
#include "gppre.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "ppinit.intfb.h"
#include "ppleta.intfb.h"

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('APACHE',0,ZHOOK_HANDLE)

!     -----------------------------------------------------------------

!*       1.    PRELIMINARY CALCULATION.
!              ------------------------

!*    1.1   PREPARATIONS



IF(LDOBS.AND.(.NOT.LDNIVP)) THEN
  WRITE(NULOUT,*) 'OBS-OPERATOR OPTION REQUIRES LDNIVP=.TRUE.!'
  CALL ABOR1('APACHE')
ENDIF
IF(LDOBS.AND.LDSPCH) THEN
  WRITE(NULOUT,*) 'OBS-OPERATOR OPTION REQUIRES LDSPCH=.FALSE.!'
  CALL ABOR1('APACHE')
ENDIF
IF(LRWSDLR) THEN
  WRITE(NULOUT,*) 'APACHE CALLED WITH LRWSDLR=T (IMPROPER CALC PRES)!'
ENDIF
ISTART = KSTART
IPROF = KPROF

!*       1.3   COMPUTE MODEL LEVEL PRESSURES - SYSTEM 1.

DO JROF=ISTART,IPROF
  ZPRESH(JROF,KLEV)=PEPS1(JROF)
ENDDO

CALL GPPREH(KPROMA,ISTART,IPROF,KLEV,PVAH1,PVBH1,ZPRESH)

DO JLEV=1,KLEV
  ZVC(JLEV)=PVAH1(JLEV)*PVBH1(JLEV-1)-PVAH1(JLEV-1)*PVBH1(JLEV)
  ZVDELB(JLEV)=PVBH1(JLEV)-PVBH1(JLEV-1)
ENDDO

CALL GPXYB(KPROMA,ISTART,IPROF,KLEV,ZVDELB,ZVC,ZPRESH,ZDELP,&
 & ZRDELP,ZLNPR,ZALPH,ZRTGR,ZRPRES,ZRPP)  

CALL GPPREF(KPROMA,ISTART,IPROF,KLEV,ZALPH,ZPRESH,ZPRESF)

!*       1.4   PREPARE FOR INTERPOLATIONS.


CALL PPINIT(KPROMA,ISTART,IPROF,KLEV,ZPRESH,ZPRESF,ZPXP,ZPXPD)

!*       1.5   COMPUTE HAIL+GRAUPEL

ZGRHL1(ISTART:IPROF,1:KLEV)=PGR1(ISTART:IPROF,1:KLEV)+PHL1(ISTART:IPROF,1:KLEV)

!*       1.6   COMPUTE R, CP AND KAPA.

CALL GPRCP(KPROMA,ISTART,IPROF,KLEV,PQ=PQ1(1,1),PQI=PQI1(1,1),PQL=PQL1(1,1),&
 & PQG=ZGRHL1(1,1),PCP=ZCP,PR=ZR,PKAP=ZKAPA)  


!     -----------------------------------------------------------------

!*       2.    COMPUTE PRESSURES IN SYSTEM 2 AND WEIGHTS ALPHA.
!              ------------------------------------------------

!*       2.1 Output surface pressure

CALL FPPS(KPROMA,ISTART,IPROF,1,PORO1,PORO2,PT1(1,1),PQ1(1,1),PQI1(1,1),&
 & PQL1(1,1),PRR1(1,1),PSN1(1,1),ZGRHL1(1,1),PEPS1,PTS1,PEPS2)  

!*       2.2 Ouput eta levels

IF (.NOT.LDNIVP) THEN
  DO JLEV=1,KOPLEV
    ZXLEV(JLEV)=REAL(JLEV,JPRB)
  ENDDO
  CALL PPLETA(KPROMA,ISTART,IPROF,KOPLEV,KOPLEV,ZXLEV,PVAH2,PVBH2,&
   & PEPS2,PEPR2F,PPR2H(1,1,1),PPR2H(1,1,0))  
ENDIF

!*      2.3 Weights

CALL FPVIEW(KPROMA,ISTART,IPROF,KOPLEV,PEPS1,PEPS2,PEPR2F,PDPBLC,ZALFAF)
IF (LDWHALF) THEN
  DO JLEV=0,1
    CALL FPVIEW(KPROMA,ISTART,IPROF,KOPLEV,PEPS1,PEPS2,PPR2H(1,1,JLEV),PDPBLC,&
     & ZALFAH(1,1,JLEV))  
  ENDDO
ENDIF

!     -----------------------------------------------------------------

!*       3     PREPARE ESCALE SYSTEM.
!              ----------------------

!*       3.1   COMPUTE PRESSURES AT ESCALE LEVELS.

DO JROF=ISTART,IPROF
  ZPESCH(JROF,KLEV) = PEPS2(JROF)
ENDDO


CALL GPPRE(KPROMA,ISTART,IPROF,KLEV,PVAH1,PVBH1,ZPESCH,ZPESCF)

!*       3.2   INITIALIZATION OF ESCALE SYSTEM.

CALL PPINIT(KPROMA,ISTART,IPROF,KLEV,ZPESCH,ZPESCF,ZPXES,ZPXDES)

!     -----------------------------------------------------------------

!*       4     VERTICAL INTERPOLATION.
!              -----------------------


CALL AVAL(KPROMA,ISTART,IPROF,KLEV,KOPLEV,&
 & ZPRESH,ZPRESF,ZPXP,ZPXPD,PTS1,PTS2,PORO1,&
 & ZPESCH,ZPESCF,ZPXES,ZPXDES,PVAH1,PVBH1,PEORG,&
 & PPR2H,PEPR2F,PORO2,ZALFAF,ZALFAH,ZLNPR,ZALPH,ZDELP,&
 & ZRDELP,ZRTGR,ZRPRES,ZRPP,ZVDELB,ZVC,&
 & PU1,PV1,PU2,PV2,PFI2,PT1,PT2,PQ1,PQ2,PRH2,ZR,&
 & PQL1,PQI1,PQA1,PO31,PSPD1,PWW1,PSN1,PRR1,PGR1,PHL1,PTKE1,&
 & PQL2,PQI2,PQA2,PO32,PSPD2,PWH2,PWF2,&
 & PSN2,PRR2,PGR2,PHL2,PTKE2,ZCP,ZKAPA,PV00,PDTDZSG,LDSPCH,&
 & LDOBS,LDWAT,LDICE,LDCFR,LDO3,LDSN,LDRR,LDGR,LDHL,&
 & LDTKE,LDPDEP,LDWHALF,LDWFULL,&
 & PEPS2)  

!     -----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('APACHE',1,ZHOOK_HANDLE)
END SUBROUTINE APACHE
