SUBROUTINE AVAL(KPROMA,KSTART,KPROF,KLEV,KOPLEV,&
 & PEPRSH,PEPRSF,PEPXP,PEPXPD,PTS1,PTS2,PORO1,&
 & PEPESH,PEPESF,PEXES,PEXESO,PVAH1,PVBH1,PEORG,&
 & PEPR2H,PEPR2F,PORO2,PALFA,PALFAH,PLNPR,PALPH,PDELP,&
 & PRDELP,PRTGR,PRPRES,PRPP,PVDELB,PVC,&
 & PU1,PV1,PU2,PV2,PFI2,PT1,PT2,PQ1,PQ2,PRH2,PR,&
 & PQL1,PQI1,PQA1,PO31,PSPD1,PWW1,&
 & PSN1,PRR1,PGR1,PHL1,PTKE1,&
 & PQL2,PQI2,PQA2,PO32,PSPD2,PWH2,PWF2,&
 & PSN2,PRR2,PGR2,PHL2,PTKE2,&
 & PCP,PKAPA,PV00,PDTDZSG,LDSPCH,LDOBS,&
 & LDWAT,LDICE,LDCFR,LDO3,&
 & LDSN,LDRR,LDGR,LDHL,LDTKE,LDPDEP,LDWHALF,LDWFULL,PEPS2)  

!**** *AVAL* - Arpege - Vertical - Arpege - Liasion.

!     PURPOSE.
!     --------
!           VERTICAL INTERPOLATION WITH PBL DISPLACEMENT TAKEN IN.

!**   INTERFACE.
!     ----------

!        *CALL* *AVAL(KPROMA,KSTART,KPROF,KLEV,KOPLEV,
!     *           PEPRSH,PEPRSF,PEPXP,PEPXPD,PTS1,PTS2,PORO1,
!     *           PEPESH,PEPESF,PEXES,PEXESO,PVAH1,PVBH1,PEORG,
!     *           PEPR2H,PEPR2F,PORO2,PALFA,PALFAH,PLNPR,PALPH,PDELP,
!     *           PRDELP,PRTGR,PRPRES,PRPP,PVDELB,PVC,
!     *           PU1,PV1,PU2,PV2,PFI2,PT1,PT2,PQ1,PQ2,PRH2,PR,
!     *           PQL1,PQI1,PQA1,PO31,PSPD1,PWW1,
!     *           PSN1,PRR1,PGR1,PHL1,PTKE1,
!     *           PQL2,PQI2,PQA2,PO32,PSPD2,PWH2,PWF2,
!     *           PSN2,PRR2,PGR2,PHL2,PTKE2,
!     *           PCP,PKAPA,PV00,PDTDZSG,LDSPCH,LDOBS,
!     *           LDWAT,LDICE,LDCFR,LDO3,
!     *           LDSN,LDRR,LDGR,LDTKE,LDPDEP,LDWHALF,LDWFULL,PEPS2)

!        EXPLICIT ARGUMENTS :   KPROMA    : HORIZONTAL DIMENSIONING.
!        --------------------   KSTART    : STARTING VALUE FOR WORK
!                               KPROF     : DEPTH OF WORK.
!                               KLEV     : NUMBER OF LEVELS S - 1.
!                               KOPLEV    : NUMBER OF LEVELS S - 2.
!                               KLEVB     : FLAGS OF LEVELS
!                               PEPRSH    : HALF PRESSURES SYSTEM 1.
!                               PEPRSF    : FULL PRESSURES SYSTEM 1.
!                               PEPXP      : HALF, FULL AND LN PRES.
!       Input :                 PEPXPD     : 1./DP AND 1./D(LN P).
!                               PTS1       : SURF. TEMP. SYSTEM 1.
!                               PTS2       : SURF. TEMP. SYSTEM 2.
!                               PEPS2      : SURF. PRES. SYSTEM 2.
!                               PORO1     : OROGRAPHIE SYSTEM 1.
!                               PORO2     : OROGRAPHIE SYSTEM 2.
!                               PEPESH    : HALF PRES. SYSTEM ESC.
!                               PEPESF    : FULL PRES. SYSTEM ESC.
!                               PEXES     : HALF, FULL AND LN P ESC.
!                               PEXESO    : 1./DP AND 1./D(LN P)ESC.
!                               PVAH1     : A-COEFFICIENTS SYSTEM 1.
!                               PVBH1     : B-COEFFICIENTS SYSTEM 1.
!                               PEPR2H     : PRESSURES HALF LEV. S-2.
!                               PEPR2F     : PRESSURES FULL LEV. S-2.
!                               PALFA     : WEIGHTS FOR PROFILES.
!                               PALFAH    : WEIGHTS FOR PROFILES. (half levels)
!                               PLNPR     : LOGARITHM OF RATIO PRESS.
!                               PALPH     : COEFF. OF THE HYDROSTATIC
!                               PT1       : TEMPERATURE SYSTEM 1.
!                               PQ1       : SPECIFIC. HUM. SYSTEM 1.
!                               PU1       : U - WIND       SYSTEM 1.
!                               PV1       : V - WIND       SYSTEM 1.
!                               PQL1      : Liquid water 1
!                               PQI1      : Ice 1
!                               PQA1      : Cloud Cover 1           
!                               PO31      : Ozone mixing ratio 1  
!                               PSN1      : Snow 1
!                               PRR1      : Rain 1
!                               PGR1      : Graupel 1
!                               PHL1      : Hail 1
!                               PTKE1     : turbulent kinetic energy 1         
!                               PSPD1     : Pressure Departure 1
!                               PWW1      : Vert. velocity w at half levels 1
!                               PR        : GAS CONSTANT.( ACC. TO Q).
!                               PCP       : SPEC. HEAT.  ( ACC. TO Q).
!                               PKAPA     : R/CP         ( ACC. TO Q).
!                               PV00      : Standard Ref. Pressure
!                               PDTDZSG   : - RDTDZ1/RG
!                               LDSPCH    : .TRUE. : Specific Hum. kept
!                                           .FALSE.: Relative Hum. kept
!                               LDOBS     : key for obs-operator outputs:
!                                         : .TRUE. : PFI2, PEORG outputs,
!                                           .FALSE.: Standard output,
!                                           PFI2, PEORG filled by 0.
!                               LDWAT     : pp of liquid water requested
!                               LDICE     : pp of ice requested
!                               LDCFR     : pp of cloud fraction requested
!                               LDO3      : pp of ozone mixing ratio requested
!                               LDSN      : pp of snow requested
!                               LDRR      : pp of rain requested
!                               LDGR      : pp of graupel requested
!                               LDHL      : pp of hail requested
!                               LDTKE     : pp of tke requested
!                               LDPDEP    : pp of NH pressure departure requested
!                               LDWHALF   : pp of NH vertic. velocity at half levels requested
!                               LDWFULL   : pp of NH vertic. velocity at full levels requested
! -----------------------------------------------------------------------
!                               PT2       : TEMPERATURE SYSTEM 2.
!                               PQ2       : SPECIFIC. HUM. SYSTEM 2.
!       Output :                PU2       : U - WIND       SYSTEM 2.
!                               PV2       : V - WIND       SYSTEM 2.
!                               PQL2      : Liquid water 2
!                               PQI2      : Ice 2
!                               PQA2      : Cloud Cover 2
!                               PO32      : Ozone mixing ratio 2
!                               PSN2      : Snow 2
!                               PRR2      : Rain 2
!                               PGR2      : Graupel 2
!                               PHL2      : Hail 2
!                               PTKE2     : Turbulent kinetic energy 2
!                               PSPD2     : Pressure Departure 2
!                               PWH2,PWF2 : Vert. velocity half/full lev. 2
!                               PRH2      : RELATIVE HUMID.SYSTEM 2.
!         (if LDOBS)            PFI2      : GEOPOTENCIAL   SYSTEM 2.
!         (if LDOBS)            PEORG     : PRESSURES OF DATA ORIGIN.

!        IMPLICIT ARGUMENTS
!        --------------------
!                 YOMCST
!                 YOMPHY
!                 YOMLUN

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!          GPRH
!          GPXYB
!          GPPRE
!          GPRCP
!          PPFLEV
!          PPUV
!          PPT
!          PPQ
!          PPRH
!          PPGEOP
!          BOB

!     REFERENCE.
!     ----------
!        ARPEGE/ALADIN DOCUMENTATION.

!     AUTHOR.
!     -------
!        RADMILA BUBNOVA *GMAP/COMPAS - STAGE MICECO*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-10-24
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        Modified : 92-09-17 by R. Bubnova: Obs-operator option
!        Modified : 94-05-13 by Eric Bazile ( Liquid Water and Ice )
!        Modified : 95-02-13 by Radmila Bubnova (NH variables)
!    R. El Khatib : 97-06-02  3 NH fields
!    R. El Khatib : 98-09-01 Remove unused arg. PTS00
!    R. El Khatib : 98-12-11 Cleanup pp of liquid water
!    R. El Khatib : 01-08-07 Pruning options
!       N. Wedi   : 01-11-08 Correct extrapolation problem
!        Modified 04-02-06: Y. Seity : new arguments to GPRCP
!    R. El Khatib : 04-08-10 PPSTA optimisation
!         G.Hello : 05-01-27 snow, rain, graupel, tke
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMCT0B  , ONLY : LECMWF
USE YOMPHY   , ONLY : LNEIGE

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPRSH(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPRSF(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPXP(KPROMA,0:KLEV,4) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPXPD(KPROMA,0:KLEV,4) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PORO1(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEPESH(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEPESF(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXES(KPROMA,0:KLEV,4) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXESO(KPROMA,0:KLEV,4) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAH1(0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH1(0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEORG(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPR2H(KPROMA,KOPLEV,0:1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPR2F(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PORO2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALFA(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALFAH(KPROMA,KOPLEV,0:1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLNPR(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALPH(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTGR(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPRES(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRPP(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVDELB(KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVC(KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFI2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRH2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PR(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQL1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQI1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQA1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO31(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRR1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGR1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHL1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPD1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW1(KPROMA,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQL2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQI2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQA2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO32(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSN2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRR2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGR2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHL2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPD2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWH2(KPROMA,KOPLEV,0:1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWF2(KPROMA,KOPLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKAPA(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV00 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTDZSG 
LOGICAL           ,INTENT(IN)    :: LDSPCH 
LOGICAL           ,INTENT(IN)    :: LDOBS 
LOGICAL           ,INTENT(IN)    :: LDWAT 
LOGICAL           ,INTENT(IN)    :: LDICE 
LOGICAL           ,INTENT(IN)    :: LDCFR 
LOGICAL           ,INTENT(IN)    :: LDO3 
LOGICAL           ,INTENT(IN)    :: LDSN 
LOGICAL           ,INTENT(IN)    :: LDRR 
LOGICAL           ,INTENT(IN)    :: LDGR 
LOGICAL           ,INTENT(IN)    :: LDHL 
LOGICAL           ,INTENT(IN)    :: LDTKE 
LOGICAL           ,INTENT(IN)    :: LDPDEP 
LOGICAL           ,INTENT(IN)    :: LDWHALF 
LOGICAL           ,INTENT(IN)    :: LDWFULL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPS2(KPROMA) 

!     ------------------------------------------------------------------

!                         LLBELO  :  .TRUE. IF GO UNDER LOWEST
!                                    FULL LEVEL OF SYSTEM 1.
!                         LLBELS  :  .TRUE. IF GO UNDER PORO1.
!                         LLBLO   :  TABLE OF OCCURENCE OF LLBELO.
!                         LLBLS   :  TABLE OF OCCURENCE OF LLBELS.
!                         ZLNP2F  :  LOG(PRESSURES 2 AT FULL LEVELS)
!                         ZVBH1   :  B-COEFFICIENTS 1 FILLED TO STRING.
!                         ZBORG   :  B-COEFFICIENTS 1 INT. TO OUTPUT L.
!                         ZRH1    :  RELATIVE HUMIDITY 1
!                         ZTHETA  :  POTENTIAL TEMPERATURE    1
!                         ZTHETS  :  SURFACE POTENT. TEMPER.  1
!                         ZTHES2  :  SURFACE POTENT. TEMPER.  2
!                         ZTSEA   :  STAND. SURF. TEMPER. AT SEA LEV.
!                         ZCINT   :  GEOPOTENTIAL INTEGRATION CONST.
!                         ZTE     :  TEMP. - ESCALE
!                         ZQE     :  SPEC. HUM. - ESCALE
!                         ZUE     :  U - WIND   - ESCALE
!                         ZVE     :  V - WIND   - ESCALE
!                         ZQLE    :  LIQ. WATER - ESCALE
!                         ZQIE    :  ICE        - ESCALE
!                         ZRRE    :  RAIN       - ESCALE
!                         ZSNE    :  SNOW       - ESCALE
!                         ZGRE    :  GRAUPEL    - ESCALE
!                         ZHLE    :  HAIL       - ESCALE
!                         ZTKEE   :  TKE        - ESCALE
!                         ZSPDE   :  PRESS. DEP.- ESCALE
!                         ZWWDE   :  VERT. velocity. - ESCALE
!                         ZWORK   :  WORKING ARRAY.
!                         ZWORK1  :  WORKING ARRAY.

LOGICAL :: LLBELO(KPROMA,KOPLEV), LLBELOH(KPROMA,KOPLEV,0:1)
LOGICAL :: LLBELS(KPROMA,KOPLEV), LLBELSH(KPROMA,KOPLEV,0:1)
LOGICAL :: LLBLO (KOPLEV)       , LLBLOH(KOPLEV,0:1)
LOGICAL :: LLBLS (KOPLEV)       , LLBLSH(KOPLEV,0:1)

INTEGER(KIND=JPIM) :: ILEVB (KPROMA,KOPLEV,4), ILEVBH(KPROMA,KOPLEV,4,0:1)

REAL(KIND=JPRB) ::    ZLNP2F(KPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZVBH1 (KPROMA,0:KLEV),   ZBORG (KPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZRH1  (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZTHETA(KPROMA,  KLEV)
REAL(KIND=JPRB) ::    ZTHETS(KPROMA)       ,   ZTHES2(KPROMA)
REAL(KIND=JPRB) ::    ZTSEA (KPROMA)       ,   ZCINT (KPROMA)
REAL(KIND=JPRB) ::    ZTSTAR (KPROMA)
REAL(KIND=JPRB) ::    ZSUM  (KPROMA)
REAL(KIND=JPRB) ::    ZTE   (KPROMA,0:KLEV),   ZQE   (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZUE   (KPROMA,0:KLEV),   ZVE   (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZQLE  (KPROMA,0:KLEV),   ZQIE  (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZSPDE (KPROMA,0:KLEV),   ZWWDE (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZQAE  (KPROMA,0:KLEV),   ZO3E  (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZSNE  (KPROMA,0:KLEV),   ZRRE  (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZGRE  (KPROMA,0:KLEV),   ZTKEE  (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZHLE  (KPROMA,0:KLEV)
REAL(KIND=JPRB) ::    ZWORK (KPROMA,KOPLEV),   ZWORK1(KPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZSTTF (KPROMA,0:KLEV),   ZSTZF (KPROMA,0:KLEV)

INTEGER(KIND=JPIM) :: ISLCT, JH, JLEV, JROF

LOGICAL :: LLEXTR

REAL(KIND=JPRB) :: ZDELTA, ZES, ZRELH, ZRHMAX, ZRHMIN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "bob.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "pp2dint.intfb.h"
#include "ppflev.intfb.h"
#include "ppgeop.intfb.h"
#include "ppq.intfb.h"
#include "pprh.intfb.h"
#include "ppt.intfb.h"
#include "ppuv.intfb.h"
#include "ppsta.intfb.h"

#include "fcttrm.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('AVAL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    TRANSFER SYSTEM 1 TO ESCALE SYSTEM.
!              -----------------------------------

DO JLEV = 0, KLEV
  DO JROF = KSTART,KPROF
    ZVBH1(JROF,JLEV) = PVBH1(JLEV)
  ENDDO
ENDDO

IF (LDOBS) THEN
  DO JLEV = 1, KOPLEV
    DO JROF = KSTART,KPROF
      ZBORG(JROF,JLEV) = 0.0_JPRB
      PFI2 (JROF,JLEV) = 0.0_JPRB
      PEORG(JROF,JLEV) = 0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!*       1.1   PREPARATION FOR TEMPERATURE -  COMPUTE THETA,
!              STANDARD SEA LEVEL TEMPERATURE, SCALE VD (NHDYN)

! NO USE OF ESCALE SYSTEM IN ECMWF MODE
IF( LECMWF ) THEN
  DO JROF = KSTART,KPROF
    ZTSEA (JROF) = PTS1(JROF)+PDTDZSG*PORO1(JROF)
    ZTSTAR(JROF) = PTS1(JROF)
  ENDDO
ELSE
! USE OF ESCALE SYSTEM IN LARPEGE MODE
  DO JLEV = 1,KLEV
    DO JROF = KSTART,KPROF
      ZTHETA(JROF,JLEV) = PT1(JROF,JLEV)*(PV00/PEPRSF(JROF,JLEV))&
       & **PKAPA(JROF,JLEV)  
    ENDDO
  ENDDO

  DO JROF = KSTART,KPROF
    ZTHETS(JROF) = PTS1(JROF)*(PV00/PEPRSH(JROF,KLEV))**PKAPA(JROF,KLEV)
    ZTHES2(JROF) = PTS2(JROF)*(PV00/PEPESH(JROF,KLEV))**PKAPA(JROF,KLEV)
    ZTSEA (JROF) = PTS1(JROF)+PDTDZSG*PORO1(JROF)
    ZTSTAR(JROF) = PTS1(JROF)
  ENDDO

!*       1.2   TRANSFER TEMPERATURE.

  DO JLEV = 1,KLEV
    DO JROF = KSTART,KPROF
      ZTE(JROF,JLEV) = (ZTHETA(JROF,JLEV)-ZTHETS(JROF)+&
       & ZTHES2(JROF))&
       & *(PEPESF(JROF,JLEV)/PV00)**PKAPA(JROF,JLEV)  
    ENDDO
  ENDDO

  IF(LDSPCH) THEN

!*       1.3   TRANSFER SPECIFIC HUMIDITY (KEPT) .

    DO JLEV = 1,KLEV
      DO JROF = KSTART,KPROF
        ZQE(JROF,JLEV) = PQ1(JROF,JLEV)
      ENDDO
    ENDDO

  ELSE

!*       1.4   TRANSFER RELATIVE HUMIDITY (KEPT) .

    DO JLEV = 1,KLEV
      DO JROF = KSTART,KPROF
        IF(LNEIGE) THEN
          ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT1(JROF,JLEV)))
        ELSE
          ZDELTA=0.0_JPRB
        ENDIF
        ZES=FOEW(PT1(JROF,JLEV),ZDELTA)
        ZRELH=(PEPRSF(JROF,JLEV)*PQ1(JROF,JLEV)*(RETV+1.0_JPRB))&
         & /((1.0_JPRB+RETV*PQ1(JROF,JLEV))*ZES)  
        ZRH1(JROF,JLEV)=MAX(0.0_JPRB,MIN(ZRELH,1.0_JPRB))
      ENDDO
    ENDDO

    DO JLEV = 1,KLEV
      DO JROF = KSTART,KPROF
        IF(LNEIGE) THEN
          ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZTE(JROF,JLEV)))
        ELSE
          ZDELTA=0.0_JPRB
        ENDIF
        ZES=FOEW(ZTE(JROF,JLEV),ZDELTA)
        ZQE(JROF,JLEV)=-ZRH1(JROF,JLEV)*ZES/(RETV*&
         & ZRH1(JROF,JLEV)*ZES-PEPESF(JROF,JLEV)*(RETV+1.0_JPRB))  
      ENDDO
    ENDDO

  ENDIF

!*       1.5   TRANSFER WIND COMPONENTS, LIQUID WATER AND ICE (KEPT).

  DO JLEV = 1,KLEV
    DO JROF = KSTART,KPROF
      ZUE(JROF,JLEV) = PU1(JROF,JLEV)
      ZVE(JROF,JLEV) = PV1(JROF,JLEV)
      IF(LDWAT) THEN
        ZQLE(JROF,JLEV) = PQL1(JROF,JLEV)
      ENDIF
      IF(LDICE) THEN
        ZQIE(JROF,JLEV) = PQI1(JROF,JLEV)
      ENDIF     
      IF (LDCFR) THEN
        ZQAE(JROF,JLEV) = PQA1(JROF,JLEV)
      ENDIF
      IF (LDO3) THEN
        ZO3E(JROF,JLEV) = PO31(JROF,JLEV)
      ENDIF
      IF (LDSN) THEN
        ZSNE(JROF,JLEV) = PSN1(JROF,JLEV)
      ENDIF
      IF (LDRR) THEN
        ZRRE(JROF,JLEV) = PRR1(JROF,JLEV)
      ENDIF
      IF (LDGR) THEN
        ZGRE(JROF,JLEV) = PGR1(JROF,JLEV)
      ENDIF
      IF (LDHL) THEN
        ZHLE(JROF,JLEV) = PHL1(JROF,JLEV)
      ENDIF
      IF (LDTKE) THEN
        ZTKEE(JROF,JLEV) = PTKE1(JROF,JLEV)
      ENDIF
      IF(LDPDEP) THEN
        ZSPDE(JROF,JLEV) = PSPD1(JROF,JLEV)
      ENDIF
    ENDDO
  ENDDO
  IF(LDWHALF.OR.LDWFULL) THEN
    DO JLEV =0,KLEV
      DO JROF = KSTART,KPROF
        ZWWDE(JROF,JLEV) = PWW1(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

ENDIF

!*       2.    LANCE A L'AVEUGLETTE.
!              ---------------------

!*       2.1   INITIALIZE FLAGS FOR SYSTEM 1 AND LOG(PEPR2F).

CALL PPFLEV(KPROMA,KSTART,KPROF,KLEV,KOPLEV,&
 & PEPR2F,PEPRSH,PEPRSF,&
 & ILEVB,&
 & LLBELO,LLBELS,LLBLO,LLBLS)  
IF (LDWHALF) THEN
  DO JH=0,1
    CALL PPFLEV(KPROMA,KSTART,KPROF,KLEV,KOPLEV,&
     & PEPR2H(1,1,JH),PEPRSH,PEPRSF,&
     & ILEVBH(1,1,1,JH),LLBELOH(1,1,JH),&
     & LLBELSH(1,1,JH),LLBLOH(1,JH),LLBLSH(1,JH))  
  ENDDO
ENDIF

DO JLEV = 1,KOPLEV
  DO JROF = KSTART,KPROF
    ZLNP2F(JROF,JLEV) = LOG(PEPR2F(JROF,JLEV))
  ENDDO
ENDDO

CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KLEV+1,1,PEPXP(1,0,2),PEPXP(1,0,4),&
 & ZSTTF,ZSTZF)  

!*       2.2   TEMPERATURE 1 ---> POST PROC. ---> 2

LLEXTR = .TRUE.
CALL PPT(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
 & PEPR2F,ZLNP2F,LLBELO,LLBELS,LLBLO,LLBLS,LLEXTR,PORO1,&
 & PEPXP,PEPXPD,ZTSTAR,ZTSEA,PR,&
 & PT1,PT2,ZSTTF)  

!*       2.3   SPECIFIC HUMIDITY 1 ---> POST PROC. ---> 2

CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
 & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
 & PQ1,PQ2)  

!*       2.4   RELAT. HUM. 1 ---> POST PROC. ---> 2

ZRHMAX=1.0_JPRB
ZRHMIN=0.0_JPRB
CALL PPRH(.FALSE.,KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
 & PEPR2F,LLBELO,LLBLO,ZRHMAX,ZRHMIN,PEPXP,PEPXPD,&
 & PQ1,PT1,PRH2)  

!*       2.5   WINDS       1 ---> POST PROC. ---> 2

CALL PPUV(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
 & LLBELO,LLBLO,ZLNP2F,PEPXP,PEPXPD,&
 & PU1,PV1,PU2,PV2)  

IF(LDOBS) THEN

!*          2.6   GEOPOTENTIAL 1 ---> POST PROC. ---> 2

  LLEXTR = .TRUE.
  CALL PPGEOP(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,ZLNP2F,LLBELO,LLBELS,LLBLO,LLBLS,LLEXTR,PORO1,&
   & PEPXP,PEPXPD,ZTSTAR,PR,PLNPR,PALPH,&
   & PT1,PFI2,ZSTTF)  

ENDIF

IF (LDWAT) THEN

!*          2.7   LIQUID WATER 1 ---> POST PROC. ---> 2

  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PQL1,PQL2)  

ENDIF

IF (LDICE) THEN

!*          2.8   ICE 1 ---> POST PROC. ---> 2

  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PQI1,PQI2)  

ENDIF


IF (LDCFR) THEN

!*          2.9   CLOUD COVER 1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PQA1,PQA2)  

ENDIF

IF (LDO3) THEN

!*          2.10   OZONE 1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PO31,PO32)  

ENDIF

IF (LDSN) THEN

!*          2.11   SNOW 1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PSN1,PSN2)  

ENDIF

IF (LDRR) THEN

!*          2.12   RAIN 1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PRR1,PRR2)  

ENDIF

IF (LDGR) THEN

!*          2.12   GRAUPEL 1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PGR1,PGR2)  

ENDIF

IF (LDHL) THEN

!*          2.12   HAIL 1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PHL1,PHL2)  

ENDIF

IF (LDTKE) THEN

!*          2.12   TKE     1 ---> POST PROC. ---> 2    
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PTKE1,PTKE2)  

ENDIF
IF(LDPDEP) THEN

!*          2.13   PRESSURE DEPARTURE 1 ---> POST PROC. ---> 2
  CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,LLBELO,LLBLO,PEPXP,PEPXPD,&
   & PSPD1,PSPD2)  

ENDIF

IF(LDWFULL) THEN
  ISLCT=1
  CALL PP2DINT(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,KOPLEV,ILEVB,&
   & ISLCT,LLBELS,LLBLS,PEPR2F,PEPXP,PEPXPD,PWW1,PWF2)  

ENDIF

IF(LDWHALF) THEN
  ISLCT=1
  DO JH=0,1
    CALL PP2DINT(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,KOPLEV,&
     & ILEVBH(1,1,1,JH),ISLCT,LLBELSH(1,1,JH),&
     & LLBLSH(1,JH),PEPR2H(1,1,JH),PEPXP,PEPXPD,&
     & PWW1,PWH2(1,1,JH))  
  ENDDO

ENDIF

IF( .NOT. LECMWF ) THEN

!*       3.    INTERPOLATE THE SYSTEM ESCALE AND MAKE FINAL PROFILE.
!              -----------------------------------------------------

!*       3.1   INITIALIZE FLAGS FOR SYSTEM ESCALE.

  CALL PPFLEV(KPROMA,KSTART,KPROF,KLEV,KOPLEV,&
   & PEPR2F,PEPESH,PEPESF,&
   & ILEVB,&
   & LLBELO,LLBELS,LLBLO,LLBLS)  

  CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KLEV+1,1,PEXES(1,0,2),PEXES(1,0,4),&
   & ZSTTF,ZSTZF)  

  IF(LDOBS) THEN

!*          3.1.1   INTERPOLATE B - COEFFICIENTS  ----> 2

    CALL BOB(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZVBH1,ZBORG)  

    DO JLEV = 1, KOPLEV
      DO JROF = KSTART, KPROF
        PEORG(JROF,JLEV) = PEPR2F(JROF,JLEV)+(1.0_JPRB-PALFA(JROF,&
         & JLEV))*&
         & ZBORG(JROF,JLEV)*(PEPRSH(JROF,KLEV) -&
         & PEPS2(JROF))  
      ENDDO
    ENDDO

  ENDIF

!*       3.2   TEMPERATURE E ---> POST PROC. ---> ZWORK

  LLEXTR = .TRUE.
  CALL GPRCP(KPROMA,KSTART,KPROF,KLEV,PQ=ZQE(1,1),&
   & PCP=PCP,PR=PR,PKAP=PKAPA)  
  CALL PPT(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & PEPR2F,ZLNP2F,LLBELO,LLBELS,LLBLO,LLBLS,LLEXTR,PORO2,&
   & PEXES,PEXESO,PTS2,ZTSEA,PR,ZTE,ZWORK,ZSTTF)  

!*       3.3   COMBINE TWO TEMPERATURE PROFILES.

  DO JLEV = 1,KOPLEV
    DO JROF = KSTART,KPROF
      PT2(JROF,JLEV) = PALFA(JROF,JLEV)*PT2(JROF,JLEV)&
       & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
    ENDDO
  ENDDO

  IF(LDSPCH) THEN

!*       3.4   SPECIFIC HUMIDITY E ---> POST PROC. ---> ZWORK

    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZQE,ZWORK)  

!*       3.5   COMBINE TWO MIXING RATIO PROFILES.

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PQ2(JROF,JLEV) = PALFA(JROF,JLEV)*PQ2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        IF(LNEIGE) THEN
          ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT2(JROF,JLEV)))
        ELSE
          ZDELTA=0.0_JPRB
        ENDIF
        ZES=FOEW(PT2(JROF,JLEV),ZDELTA)
        ZRELH=(PEPR2F(JROF,JLEV)*PQ2(JROF,JLEV)*(RETV+1.0_JPRB))&
         & /((1.0_JPRB+RETV*PQ2(JROF,JLEV))*ZES)  
        PRH2(JROF,JLEV)=MAX(0.0_JPRB,MIN(ZRELH,1.0_JPRB))
      ENDDO
    ENDDO

  ELSE

!*       3.6   RELAT. HUM. E ---> POST PROC. ---> ZWORK

    ZRHMAX=1.0_JPRB
    ZRHMIN=0.0_JPRB
    CALL PPRH(.FALSE.,KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,ZRHMAX,ZRHMIN,PEXES,PEXESO,&
     & ZQE,ZTE,ZWORK)  

!*       3.7   COMBINE TWO RELATIVE HUMIDITY PROFILES.

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PRH2(JROF,JLEV) = PALFA(JROF,JLEV)*PRH2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        IF(LNEIGE) THEN
          ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT2(JROF,JLEV)))
        ELSE
          ZDELTA=0.0_JPRB
        ENDIF
        ZES=FOEW(PT2(JROF,JLEV),ZDELTA)
        PQ2(JROF,JLEV)=-PRH2(JROF,JLEV)*ZES/(RETV*&
         & PRH2(JROF,JLEV)*ZES-PEPR2F(JROF,JLEV)*(RETV+1.0_JPRB))  
      ENDDO
    ENDDO

  ENDIF

!*       3.8   WINDS       E ---> POST PROC. ---> ZWORK,ZWORK1

  CALL PPUV(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
   & LLBELO,LLBLO,ZLNP2F,PEXES,PEXESO,&
   & ZUE,ZVE,ZWORK,ZWORK1)  

!*       3.9   COMBINE TWO WINDS PROFILES.

  DO JLEV = 1,KOPLEV
    DO JROF = KSTART,KPROF
      PU2(JROF,JLEV) = PALFA(JROF,JLEV)*PU2(JROF,JLEV)&
       & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      PV2(JROF,JLEV) = PALFA(JROF,JLEV)*PV2(JROF,JLEV)&
       & + (1.0_JPRB-PALFA(JROF,JLEV))*ZWORK1(JROF,JLEV)  
    ENDDO
  ENDDO

  IF(LDOBS) THEN

    CALL GPPREH(KPROMA,KSTART,KPROF,KLEV,PVAH1,PVBH1,PEPESH)
    CALL GPXYB(KPROMA,KSTART,KPROF,KLEV,PVDELB,PVC,PEPESH,PDELP,&
     & PRDELP,PLNPR,PALPH,PRTGR,PRPRES,PRPP)  
    CALL GPPREF(KPROMA,KSTART,KPROF,KLEV,PALPH,PEPESH,PEPESF)
!  PR is calculated yet (via GPRCP, called before previous call to PPT).

!*          3.9.1   GEOPOTENTIAL E ---> POST PROC. ---> ZWORK

    LLEXTR = .TRUE.
    CALL PPGEOP(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,ZLNP2F,LLBELO,LLBELS,LLBLO,LLBLS,LLEXTR,PORO2,&
     & PEXES,PEXESO,PTS2,PR,PLNPR,PALPH,&
     & ZTE,ZWORK,ZSTTF)  

!*          3.9.2   COMBINE TWO GEOPOTENTIAL PROFILES.

    DO JROF = KSTART,KPROF
      ZCINT(JROF)  = 0.0_JPRB
      ZSUM (JROF)  = 0.0_JPRB
      DO JLEV = 1,KOPLEV
        ZCINT(JROF)  = ZCINT(JROF) + (PFI2(JROF,JLEV)-&
         & ZWORK(JROF,JLEV))*(1.0_JPRB-PALFA(JROF,JLEV))*&
         & (1.0_JPRB -PALFA(JROF,JLEV))  
        ZSUM (JROF)  = ZSUM (JROF) + (1.0_JPRB-PALFA(JROF,JLEV))*&
         & (1.0_JPRB -PALFA(JROF,JLEV))  
      ENDDO
    ENDDO

    DO JROF = KSTART,KPROF
      ZCINT(JROF)  = ZCINT(JROF)/MAX(ZSUM(JROF),1.E-12_JPRB)
    ENDDO

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PFI2(JROF,JLEV) = PALFA(JROF,JLEV)*PFI2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*(ZWORK(JROF,JLEV)+&
         & ZCINT(JROF))  
      ENDDO
    ENDDO

  ENDIF

  IF (LDWAT) THEN

!*          3.9.3   LIQUID WATER E ---> POST PROC. ---> ZWORK

    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZQLE,ZWORK)  

!*          3.9.4   COMBINE TWO LIQUID WATER PROFILES.

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PQL2(JROF,JLEV) = PALFA(JROF,JLEV)*PQL2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDICE) THEN

!*          3.9.5   ICE  E ---> POST PROC. ---> ZWORK

    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZQIE,ZWORK)  

!*          3.9.6   COMBINE TWO ICE PROFILES.

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PQI2(JROF,JLEV) = PALFA(JROF,JLEV)*PQI2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF


  IF (LDCFR) THEN

!*          3.9.7   CLOUD COVER E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZQAE,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PQA2(JROF,JLEV) = PALFA(JROF,JLEV)*PQA2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDO3) THEN

!*          3.9.8   OZONE E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZO3E,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PO32(JROF,JLEV) = PALFA(JROF,JLEV)*PO32(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDSN) THEN

!*          3.9.9   SNOW  E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZSNE,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PSN2(JROF,JLEV) = PALFA(JROF,JLEV)*PSN2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDRR) THEN

!*          3.9.10   RAIN  E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZRRE,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PRR2(JROF,JLEV) = PALFA(JROF,JLEV)*PRR2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDGR) THEN

!*          3.9.11   GRAUPEL E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZGRE,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PGR2(JROF,JLEV) = PALFA(JROF,JLEV)*PGR2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDHL) THEN

!*          3.9.11   HAIL E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZHLE,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PHL2(JROF,JLEV) = PALFA(JROF,JLEV)*PHL2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF (LDTKE) THEN

!*          3.9.12   TKE E ---> POST PROC. ---> ZWORK
    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZTKEE,ZWORK)  

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PTKE2(JROF,JLEV) = PALFA(JROF,JLEV)*PTKE2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF(LDPDEP) THEN

!*          4.1     PRESS. DEP. E ---> POST PROC. ---> ZWORK

    CALL PPQ(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,ILEVB,&
     & PEPR2F,LLBELO,LLBLO,PEXES,PEXESO,&
     & ZSPDE,ZWORK)  

!*          4.2     COMBINE TWO PRESS. DEP. PROFILES.

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PSPD2(JROF,JLEV) = PALFA(JROF,JLEV)*PSPD2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO

  ENDIF

  IF(LDWFULL) THEN

!*          4.3     VERT. VEL. E ---> POST PROC. ---> ZWORK

    ISLCT=1
    CALL PP2DINT(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,&
     & KOPLEV,ILEVB,&
     & ISLCT,LLBELS,LLBLS,PEPR2F,PEXES,PEXESO,&
     & ZWWDE,ZWORK)  

!*          4.4     COMBINE TWO VERT. VEL. PROFILES

    DO JLEV = 1,KOPLEV
      DO JROF = KSTART,KPROF
        PWF2(JROF,JLEV) =PALFA(JROF,JLEV)*PWF2(JROF,JLEV)&
         & +(1.0_JPRB-PALFA(JROF,JLEV))*ZWORK(JROF,JLEV)  
      ENDDO
    ENDDO
  ENDIF

  IF(LDWHALF) THEN
    DO JH=0,1

!*          4.3     VERT. VEL. E ---> POST PROC. ---> ZWORK

      ISLCT=1
      CALL PP2DINT(KPROMA,KSTART,KPROF,KLEV,KOPLEV,1,KOPLEV,&
       & ILEVBH(1,1,1,JH),ISLCT,LLBELSH(1,1,JH),&
       & LLBLSH(1,JH),PEPR2H(1,1,JH),PEXES,PEXESO,&
       & ZWWDE,ZWORK)  

!*          4.4     COMBINE TWO VERT. VEL. PROFILES

      DO JLEV = 1,KOPLEV
        DO JROF = KSTART,KPROF
          PWH2(JROF,JLEV,JH)=&
           & PALFAH(JROF,JLEV,JH)*PWH2(JROF,JLEV,JH)&
           & +(1.0_JPRB-PALFAH(JROF,JLEV,JH))*ZWORK(JROF,JLEV)  
        ENDDO
      ENDDO

    ENDDO
  ENDIF

ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AVAL',1,ZHOOK_HANDLE)
END SUBROUTINE AVAL
