SUBROUTINE CTSTARAD(KPROMA,KSTART,KPROF,PTB,PRESBH,PRESBF,&
 & PTSTAR,PT0,PTB5,PRESBH5,PRESBF5)  

!**** *CTSTARAD* - COMPUTES STANDARD SURFACE TEMPERATURE - ADJOINT
!                              AND SURFACE TEMPERATURE.

!     PURPOSE.
!     --------
!           COMPUTES THE STANDARD SURFACE TEMPERATURE AND THE SURFACE
!           TEMPERATURE TO BE USED FOR EXTRAPOLATIONS OF TEMPERATURE
!           AND GEOPOTENTIEL.

!**   INTERFACE.
!     ----------
!        *CALL* *CTSTARAD(KPROMA,KSTART,KPROF,PTB,PRESBH,PRESBF,
!                        ,PTSTAR,PT0,PTB5,PRESBH5,PRESBF5)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA         - HORIZONTAL DIMENSIONS.             (INPUT-C)
!        KSTART         - START OF WORK                      (INPUT-C)
!        KPROF          - DEPTH OF WORK                      (INPUT-C)
!        PTB(KPROMA)    - TEMPERATURE AT LOWEST MODEL LEVEL  (INPUT)
!        PRESBH(KPROMA) - LOWEST MODEL HALF LEVEL PRESSURES  (INPUT)
!        PRESBF(KPROMA) - LOWEST MODEL FULL LEVEL PRESSURES  (INPUT)

!        PTSTAR(KPROMA) - SURFACE TEMPERATURE                (OUTPUT)
!        PT0(KPROMA)    - STANDARD SURFACE TEMPERATURE       (OUTPUT)
!        PTB5(KPROMA)    - TEMPERATURE AT LOWEST MODEL LEVEL  (INPUT-T)
!        PRESBH5(KPROMA) - LOWEST MODEL HALF LEVEL PRESSURES  (INPUT-T)
!        PRESBF5(KPROMA) - LOWEST MODEL FULL LEVEL PRESSURES  (INPUT-T)

!        IMPLICIT ARGUMENTS :    CONSTANTS FROM YOMSTA,YOMCST.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.   NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-01-17
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESBH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESBF(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSTAR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESBH5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESBF5(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZALPHA, ZDTDZSG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    COMPUTES SURFACE TEMPERATURE
!*             THEN STANDARD SURFACE TEMPERATURE.

IF (LHOOK) CALL DR_HOOK('CTSTARAD',0,ZHOOK_HANDLE)
ZDTDZSG=-RDTDZ1/RG
ZALPHA=ZDTDZSG*RD
DO JL=KSTART,KPROF
  PTSTAR(JL)=PTSTAR(JL)+PT0(JL)
  PTB(JL)=PTB(JL)+(1.0_JPRB+ZALPHA*(PRESBH5(JL)/PRESBF5(JL)-1.0_JPRB))*PTSTAR(JL)
  PRESBH(JL)=PRESBH(JL)+PTB5(JL)*ZALPHA/PRESBF5(JL)*PTSTAR(JL)
  PRESBF(JL)=PRESBF(JL)-PTB5(JL)*ZALPHA*PRESBH5(JL)/PRESBF5(JL)**2*PTSTAR(JL)
  PTSTAR(JL)=0.0_JPRB
  PT0(JL)=0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CTSTARAD',1,ZHOOK_HANDLE)
END SUBROUTINE CTSTARAD

