SUBROUTINE EXPBESUAD(KPROMA,KSTART,KPROF,KLEVP,KLOLEV,&
 & PZO,LDBELS,LDBLES,POROG,&
 & PRESH,PT,&
 & PPPP,&
 & PRESH5,PT5)  

!**** *EXPBESUAD* - EXTRAPOLATE P BELOW SURFACE  (ADJOINT)

!     PURPOSE.
!     --------

!        COMPUTES SURFACE PRESSURE BELOW MODEL'S SURFACE (ADJOINT)

!**   INTERFACE.
!     ----------

!        *CALL* *EXPBESUAD(KPROMA,KSTART,KPROF,KLEVP,KLOLEV,
!                         PZO,LDBELS,LDBLES,POROG,
!                         PRESH,PT,
!                         PPPP,
!                         PRESH5,PT5)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.
!        KSTART                    - START OF WORK.
!        KPROF                     - DEPTH OF WORK.
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION
!        PZO                       - POST-PROCESSING LEVEL HEIGHTS
!        LDBELS                    - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE
!        LDBLES                    - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.
!        POROG                     - MODEL OROGRAPHY.
!        PRESH                     - HALF LEVEL PRESSURE
!        PT                        - TEMPERATURE
!        PPPP                      - POST-PROCESSED PRESSURE
!        PRESH5                    - HALF LEVEL PRESSURE
!        PT5                       - TEMPERATURE

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS
!     ---------

!        NONE

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC     *ECMWF*

!     MODIFICATIONS.
!     --------------
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KPROMA) 
REAL(KIND=JPRB) ::ZT5    (KPROMA             ),ZT     (KPROMA             )&
 & ,ZALPHA5(KPROMA             ),ZALPHA (KPROMA             )  
INTEGER(KIND=JPIM) :: J0, JL, JLEVP

REAL(KIND=JPRB) :: Z1, Z15, Z2, Z25, Z3, Z35, Z4, Z45, ZDTDZSG,&
 & ZOROG, ZT0, ZT05, ZTX, ZTY  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    EXTRAPOLATE PRESSURE BELOW MODEL'S SURFACE
!              ------------------------------------------

!*       1.0   PRE-SET

IF (LHOOK) CALL DR_HOOK('EXPBESUAD',0,ZHOOK_HANDLE)
DO J0=KSTART,KPROF
  ZALPHA(J0)     =0.0_JPRB
  ZT    (J0)     =0.0_JPRB
ENDDO

!*       1.1   PRE-CALCULATION

ZTX=290.5_JPRB
ZTY=255.0_JPRB
ZDTDZSG=-RDTDZ1/RG
DO JL=KSTART,KPROF
  IF(PT5   (JL) < ZTY) THEN
    ZT5   (JL)=0.5_JPRB*(ZTY+PT5   (JL))
  ELSEIF(PT5   (JL) < ZTX) THEN
    ZT5   (JL)=PT5   (JL)
  ELSE
    ZT5   (JL)=0.5_JPRB*(ZTX+PT5   (JL))
  ENDIF

  ZT05=ZT5   (JL)+ZDTDZSG*POROG(JL)
  IF(ZTX > ZT5   (JL) .AND. ZT05 > ZTX) THEN
    ZT05=ZTX
  ELSEIF(ZTX <= ZT5   (JL) .AND. ZT05 > ZT5   (JL)) THEN
    ZT05=ZT5   (JL)+0.0001_JPRB
  ENDIF

  ZOROG=SIGN(MAX(1.0_JPRB,ABS(POROG(JL))),POROG(JL))
  ZALPHA5(JL)=RD*(ZT05-ZT5   (JL))/ZOROG
ENDDO

!*       1.2   EXTRAPOLATION

DO JLEVP=KLOLEV,KLEVP
  IF(LDBLES(JLEVP)) THEN
    DO JL=KSTART,KPROF
      IF(LDBELS(JL,JLEVP)) THEN
        Z15            = (POROG(JL)-PZO(JL,JLEVP))*ZALPHA5(JL)
        Z25            = Z15/(RD*ZT5   (JL))
        Z35            = Z25 + 1.0_JPRB
        Z45            = LOG(Z35)/ZALPHA5(JL) +LOG(PRESH5(JL,NFLEVG))

        Z4             =0.0_JPRB
        Z3             =0.0_JPRB
        Z2             =0.0_JPRB
        Z1             =0.0_JPRB

        Z4             =Z4 +PPPP(JL,JLEVP)*EXP(Z45)

        Z3             =Z3 +Z4*(1.0_JPRB/(ZALPHA5(JL)*Z35))

        ZALPHA(JL)     =ZALPHA(JL) +Z4*(-LOG(Z35)/ZALPHA5(JL)**2)

        PRESH(JL,NFLEVG)=PRESH(JL,NFLEVG) +Z4*(1.0_JPRB/PRESH5(JL,NFLEVG))

        Z2             =Z2 +Z3

        Z1             =Z1 +Z2*(1.0_JPRB/(RD*ZT5   (JL)))

        ZT(JL)         =ZT(JL) +Z2*(-Z15/(RD*ZT5   (JL)**2))

        ZALPHA(JL)     =ZALPHA(JL) +Z1*(POROG(JL)-PZO(JL,JLEVP))
      ENDIF
    ENDDO
  ENDIF
ENDDO

DO JL=KSTART,KPROF
  ZT0=0.0_JPRB

  ZOROG=SIGN(MAX(1.0_JPRB,ABS(POROG(JL))),POROG(JL))
  ZT0=ZT0+(RD/ZOROG)*ZALPHA(JL)
  ZT(JL)=ZT(JL)-(RD/ZOROG)*ZALPHA(JL)

  ZT05=ZT5(JL)+ZDTDZSG*POROG(JL)
  IF(ZTX > ZT5(JL) .AND. ZT05 > ZTX) THEN
    ZT0=0.0_JPRB
  ELSEIF(ZTX <= ZT5(JL) .AND. ZT05 > ZT5(JL)) THEN
    ZT(JL)=ZT(JL)+ZT0
  ELSE
    ZT(JL)=ZT(JL)+ZT0
  ENDIF

  IF(PT5(JL) < ZTY)          THEN
    PT(JL)=PT(JL)+0.5_JPRB*ZT(JL)
  ELSEIF(PT5(JL) < ZTX)       THEN
    PT(JL)=PT(JL)+ZT(JL)
  ELSE
    PT(JL)=PT(JL)+0.5_JPRB*ZT(JL)
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EXPBESUAD',1,ZHOOK_HANDLE)
END SUBROUTINE EXPBESUAD

