SUBROUTINE EXPBESUTL (KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & PZO,LDBELS,LDBLES,POROG,&
 & PRESH,PT,&
 & PPPP,&
 & PRESH5,PT5)  

!**** *EXPBESU* - EXTRAPOLATE P BELOW SURFACE, TANGENT LINEAR VERSION

!     PURPOSE
!     -------

!        COMPUTES SURFACE PRESSURE BELOW MODEL'S SURFACE

!**   INTERFACE.
!     ----------

!        *CALL* *EXPBESU(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,
!                        PZO,LDBELS,LDBLES,POROG,
!                        PRESH,PT,
!                        PPPP)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.
!        KSTART                    - START OF WORK.
!        KPROF                     - DEPTH OF WORK.
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION
!        PZO                       - POST-PROCESSING LEVEL HEIGHTS
!        LDBELS                    - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE
!        LDBLES                    - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.
!        POROG                     - MODEL OROGRAPHY.
!        PRESH                     - HALF LEVEL PRESSURE
!        PT                        - TEMPERATURE
!        PPPP                      - POST-PROCESSED PRESSURE

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS
!     ---------

!        NONE

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC   *ECMWF*
!        TANGENT LINEAR VERSION BY MIKE FISHER *ECMWF*

!     MODIFICATIONS.
!     --------------
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KPROMA) 
REAL(KIND=JPRB) ::ZT    (KPROMA             )&
 & ,ZALPHA(KPROMA             )  
REAL(KIND=JPRB) ::ZT5    (KPROMA             )&
 & ,ZALPHA5(KPROMA             )  
INTEGER(KIND=JPIM) :: JL, JLEVP

REAL(KIND=JPRB) :: Z1, Z15, Z2, Z25, Z3, Z35, Z4, Z45, ZDTDZSG,&
 & ZOROG, ZT0, ZT05, ZTX, ZTY  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    EXTRAPOLATE PRESSURE BELOW MODEL'S SURFACE
!              ------------------------------------------

!*       1.1   PRE-CALCULATION

IF (LHOOK) CALL DR_HOOK('EXPBESUTL',0,ZHOOK_HANDLE)
ZTX=290.5_JPRB
ZTY=255.0_JPRB
ZDTDZSG=-RDTDZ1/RG
DO JL=KSTART,KPROF
  IF(PT5(JL) < ZTY) THEN
    ZT5(JL)=0.5_JPRB*(ZTY+PT5(JL))
    ZT (JL)=0.5_JPRB*(    PT (JL))
  ELSEIF(PT5(JL) < ZTX) THEN
    ZT5(JL)=PT5    (JL)
    ZT (JL)=PT     (JL)
  ELSE
    ZT5(JL)=0.5_JPRB*(ZTX+PT5(JL))
    ZT (JL)=0.5_JPRB*(    PT (JL))
  ENDIF

  ZT05=ZT5(JL)+ZDTDZSG*POROG(JL)
  ZT0 =ZT (JL)
  IF(ZTX > ZT5(JL) .AND. ZT05 > ZTX) THEN
    ZT05=ZTX
    ZT0 =0.0_JPRB
  ELSEIF(ZTX <= ZT5(JL) .AND. ZT05 > ZT5(JL)) THEN
    ZT05=ZT5(JL)+0.0001_JPRB
    ZT0 =ZT (JL)
  ENDIF

  ZOROG=SIGN(MAX(1.0_JPRB,ABS(POROG(JL))),POROG(JL))
  ZALPHA5(JL)=RD*(ZT05-ZT5(JL))/ZOROG
  ZALPHA (JL)=RD*(ZT0 -ZT (JL))/ZOROG
ENDDO

!*       1.2   EXTRAPOLATION

DO JLEVP=KLOLEV,KLEVP
  IF(LDBLES(JLEVP)) THEN
    DO JL=KSTART,KPROF
      IF(LDBELS(JL,JLEVP)) THEN
        Z15           = (POROG(JL)-PZO(JL,JLEVP))*ZALPHA5(JL)
        Z1            = (POROG(JL)-PZO(JL,JLEVP))*ZALPHA(JL)

        Z25           = Z15/(RD*ZT5    (JL))
        Z2            =  Z1/(RD*ZT5    (JL))&
         & - ZT(JL)*(Z15/(RD*ZT5(JL)*ZT5(JL)))  

        Z35           = Z25 + 1.0_JPRB
        Z3            = Z2

        Z45           = LOG(Z35)/ZALPHA5(JL) +LOG(PRESH5(JL,NFLEVG))
        Z4            =  Z3/(Z35*ZALPHA5(JL))&
         & - ZALPHA(JL)*LOG(Z35)&
         & /(ZALPHA5(JL)*ZALPHA5(JL))&
         & + PRESH(JL,NFLEVG)/PRESH5(JL,NFLEVG)  

        PPPP(JL,JLEVP)= EXP(Z45)*Z4
      ENDIF
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EXPBESUTL',1,ZHOOK_HANDLE)
END SUBROUTINE EXPBESUTL
