SUBROUTINE POS(&
 & CDCONF,KST,KND,PXLEV,KOPLEV,KSTGLO,PVAH,PVBH,KFPLEV,PXORO,PTSI,PXTSO, &
 & PU,PUL,PDIV,PV,PVL,PVOR,PT,PTL,PTM,LDQ,PQ,LDQDER,PQL,PQM,LDL,PL,LDI,PI, &
 & LDS,PS,LDR,PR,LDGR,PGR,LDHL,PHL,LDTKE,PTKE,&
 & PSP,PSPL,PSPM,LDEXT,PEXT,LDA,PA,LDO3,PO3,&
 & LDUD, PUAL, PUOM, LDDD, PDAL, PDOM, LDUEN, PUEN, LDUNEBH, PUNEBH,&
 & LDAERO,PAERO,LDGHG,PGHG,LDTRAC,PTRAC,LDGRG,PGRG,&
 & PSPD,PSVD,PEV,PLRCH4,PCH4S,PERA40,POROG,PRCORI,PGM,POROGL,POROGM,&
 & PGPP,KFLDSPP,PAUX,KFLDAUX,&
 & PEXTRA,PRMCUFGP1,PRMCUFGP2,PRMCUFGP3,PRMCUFGP4,PRMCUFGP5)  

!**** *POS* - post-processing

!     PURPOSE.
!     -------
!           Post-processing calculations.

!**   INTERFACE.
!     ---------
!        *CALL* *POS(...)

!        EXPLICIT ARGUMENTS :
!        ------------------
!            INPUT :
!   --  control variables ---
!        CDCONF    : configuration of work
!                    'B' <=> Pressure                 level pp.
!                    'H' <=> Height (above orography) level pp.
!                    'T' <=> Potential temperature    level pp.
!                    'V' <=> Potential vorticity      level pp.
!                    'M' <=> Model                    level pp.
!                    'S' <=> Eta                      level pp.
!        KST       : first point to post-process 
!        KND       : last point to post-process 
!        PXLEV     : array containing the pp levels (in P,H,PV,TH or s)
!        KOPLEV    : number of post-processing levels
!        KFLDSPP   : number of fields to be spectrally fitted
!        KFLDAUX   : number of fields to remain grid point
!            For APACHE :
!        PVAH      : output vertical function *A*
!        PVBH      : output vertical function *B*
!        KFPLEV    : number of output vertical levels
!        PXORO     : external orography ("interpolated")
!        PTSI      : input surface temperature
!        PXTSO     : output surface temperature
!             -----------
!   --  model variables ---
!        PU      : zonal wind time t
!        PV      : meridional wind time t
!        PT      : temperature time t
!        PQ      : humidity time t
!        PDIV    : divergence  time t
!        PVOR    : vorticity  time t
!        PEXT     : passiv scalars time t
!        PSP     : surface pressure variable time t
!        PRMCUFGP: 
!        PL      : cloud liquid water time t
!        PI      : cloud ice time t
!        PA      : cloud fraction time t
!        PS      : snow time t
!        PR      : rain time t
!        PGR     : graupel time t
!        PHL     : hail time t
!        PTKE    : turbulent kinetic energy time t
!        PO3     : ozone mixing ratio time t
!        PUAL    : updraught mesh fraction
!        PUOM    : updraught vert velocity
!        PDAL    : downdraught mesh fraction
!        PDOM    : downdraught vert velocity
!        PUEN    : updraught entrainment
!        PUNEBH  : pseudo-historic convective cloud fraction
!        PAERO   : aerosols time t
!        PGHG    : greenhouse gases time t
!        PTRAC   : greenhouse gases time t
!        PGRG    : GRG fields time t
!        PSPD     : Scaled Pressure Departure time t
!        PSVD     : Scaled Vertical Divergence time t
!        PEV     : <eta dot * dp/deta> on top of the atmosphere time t
!        PEXTRA    : extra input fields
!        PERA40  : ERA40 diagnostic fields
!        PLRCH4    : CH4 loss rate
!        PCH4S     : CH4 atmospheric sink
!   --  Grid point help arrays
!        POROG     : Orography
!        PRCORI    : Coriolis force
!        PGM       : Map factor
!   --  longitudinal derivatives ---
!        PTL     : temperature time t
!        PSPL    : surface pressure variable time t
!        PUL     : longitudl gradient of u compenent
!        PVL     : longitudl gradient of v component
!        POROGL    : orography
!   --  meridional derivatives ---
!        PTM     : temperature time t
!        PSPM    : surface pressure variable time t
!        POROGM    : orography
!            OUTPUT (post-processed fields) :
!        PGPP      : fields to be spectrally fitted
!        PAUX     : fields to remain gridpoint

!     EXTERNALS.
!     ---------

! Modifications
! -------------
!   ORIGINAL : 88-02-04
!   Modified 91-05-21 by A.Lasserre-Bigorry (scalar variables)
!   Modified 92-02-01 by M.Hamrud (IO scheme)
!   Modified spring 92, C.Periard and A.Joly : 'modern fields'
!   Modified 93-02-24 by E.Andersson (GPPREF/H)
!   Modified 93-06-01 M.Hamrud (CTSTAR called with T(.,NFLEV-1)
!   Modified 94-07-19 R. El Khatib (Flexibility for Full-POS,
!            cleanings and configurations H, V, T, S and M)
!   Modified 95-10-01 MPP Group Distributed Memory version
!   Modified 95-11-09 R. El Khatib (pp. of equivalent potential temperature)
!   Modified 96-02-08 R. El Khatib (Model variable Ps or Log(Ps) instead of Ps)
!   Modified 97-02-25 R. El Khatib (call PP2DINT for pp of T on PV levels)
!   Modified 97-06-10 N.Wedi extended the upper air fields used in FPOS
!   R. El Khatib : 97-06-02  3 NH fields
!   R. El Khatib : 97-12-23 ZEVT0 ,LRUBC in GPCTY
!   R. El Khatib : 98_12-10 cleanup liquid water & ice
!   D.Dent       : 99-05-06 Support for log surface pressure 
!   R. El Khatib : 99-10-25 More flexibility for Ps/Log(Ps), LFPRH100
!   K. YESSAD    : 99-11-15 Add dummy "zpsdvbc" in "gpcty".
!   R. El Khatib : 99-12-09 Vertical velocity at the bottom of the 
!              atmosphere computed from the wind at the lowest full level
!   R. El Khatib : 00-08-16 modified PPLETA ; PPZHLEV+PPPS replaced by FPPS
!   R. El Khatib : 00-09-28 derived types + total water vapour
!   D. Dent      : 00-04-12 Computation of Montgomery geopotential
!   R. El Khatib : 01-01-05 iso t=0 + iso t'w=0
!   R. El Khatib : 01-03-28 Redefinition of QFPTYPE%ISKP + secondary pointer
!                           %IGT1
!   R. El Khatib : 01-08-07 Pruning options
!   R. El Khatib : 03-01-06 Computations savings and protections
!                           - happy new year
!   R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!   F. Vana      : 02-07-03 Virtual potential temperature
!   R. El Khatib : 02-09-04 Iso -10 Celsius
!   R. El Khatib : 03-08-12 gfl
!   R. El Khatib : 03-04-17 Fullpos improvemnts
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   20-Jun-2004 J. Masek   (NPDVAR, NVDVAR) = (2, 3)
!   R. El Khatib : 04-04-14 Optimization
!   14-Mar-2005 R. Engelen    Greenhouse Gase
!   14-Mar-2005 A. Untch    Aerosols
!   11-Apr-2005 J Flemming Aerosol replaced with GEMS GRG 
!   Y. Bouteloup : 28-Jan-2005 Add PR (precipitation)
!   K. Yessad    : 04-01-24 EVERDIA renamed into GPVERDIA
!   G. Hello     : 05-01-27 snow, rain, graupel, tke
!   K. Yessad    : 04-01-25 obsolete EGPHI replaced by GPGRXYB+GPGRGEO
!   R. El Ouaraini & Ryad El Khatib : 05-07-28 Full-pos CUF fields
!   M. Bellus    : 29-Sep-2006 prognostic convection and MAC scheme (ALARO-0)
!   S. Serrar    : 07-09-06 include tracers used for diagnostics only (GEMS)
!   G. Hello     : 07-02 thetav
!   S. Serrar    : 22-03-2007 post-process new GFL ERA40 fields
!   S. Serrar    : 17-07-2007 two new GFL fields methane related
!   K. Yessad    : 10-10-2007 (provisional) adaptations to VFE for NH variables
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme + etadot
!    S. Serrar   02-05-2008 test on LERA40 removed
!   K. Yessad (Feb 2008): paragraph renumbering, specific paragraphes for GFL 
!   K. Yessad (Aug 2008): (partial) more concise rewriting.
!   JJMorcrette : 20090217 PP of aerosol and UV processor outputs
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSFSU  ,JPOSVX2
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPPM     ,LVOR     ,LUVDER  
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : LFPART2  ,LNHDYN   ,LSPRT
USE YOMDYNA  , ONLY : NVDVAR   ,NPDVAR   ,LNH_GEOGW,LRWSDLR
USE YOMDYN   , ONLY : RC_PD1   ,RCORPDF
USE YOMGEM   , ONLY : VBH      ,VAH      ,VC       ,VDELB    ,VFE_RDETAH, VETAH
USE YOMCST   , ONLY : RG       ,RATM     ,RD       ,RV       ,RCPD     ,&
 & RKAPPA   ,RTT  
USE YOMSTA   , ONLY : RDTDZ1   ,NLEXTRAP
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_DERIB,LVFE_X_TERM
USE YOMFPC   , ONLY : LFPLOSP  ,LFPRH100 ,CFPFMT   ,FPBL     ,LFPQ
USE YOMFPG   , ONLY : LFPOSVER
USE YOMVPOS  , ONLY : QFPTYPE
USE YOMAFN   , ONLY : TFP_ABS  ,TFP_CLF  ,TFP_DIV  ,TFP_ETH  ,TFP_FIS  ,&
 & TFP_FOL  ,TFP_FSU  ,TFP_FUA  ,TFP_GM   ,TFP_KHI  ,TFP_LNSP ,&
 & TFP_MG   ,TFP_MOCO ,TFP_MSL  ,TFP_O3MX ,TFP_PCAO ,TFP_PD   ,&
 & TFP_PJET ,TFP_PSI  ,TFP_PV   ,TFP_P    ,TFP_Q    ,TFP_HU   ,&
 & TFP_GFL  ,TFP_SHD  ,TFP_SP   ,TFP_STD  ,TFP_I    ,TFP_TCAO ,&
 & TFP_TH   ,TFP_THPW ,TFP_T    ,TFP_UJET ,TFP_U    ,TFP_VD   ,&
 & TFP_VJET ,TFP_VOR  ,TFP_VV   ,TFP_VW   ,TFP_V    ,TFP_WND  ,&
 & TFP_TWV  ,TFP_WWS  ,TFP_L    ,TFP_Z    ,TFP_PT0B ,TFP_HT0B ,&
 & TFP_HTPW ,TFP_IET  ,TFP_PT1B ,TFP_HT1B ,TFP_SN   ,TFP_RR   ,&
 & TFP_GR   ,TFP_TKE  ,TFP_AERO ,TFP_GHG  ,TFP_TRAC ,TFP_GRG  ,&
 & TFP_CUF1 ,TFP_CUF2 ,TFP_CUF3 ,TFP_CUF4 ,TFP_CUF5 ,TFP_SRE  ,&
 & TFP_DAL  ,TFP_DOM  ,TFP_UAL  ,TFP_UOM  ,TFP_UEN  ,TFP_UNEBH,&
 & TFP_ERA40,TFP_THV  ,TFP_ETAD,TFP_LRCH4,TFP_CH4S  ,TFP_HL   ,&
 & TFP_NOGW ,TFP_AEROUT,TFP_UVP
USE SURFACE_FIELDS_MIX  , ONLY : YSD_XA,YSD_XAD
USE YOM_YGFL , ONLY : NGFL_EXT, NAERO, NGHG, NGRG, NTRAC, NERA40,&
 &  YLRCH4, YCH4S, YNOGW, NNOGW, YAEROUT, NAEROUT, YUVP, NUVP
USE YOMGFL   , ONLY : GFL

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSPP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDAUX 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXLEV(KOPLEV)
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAH(0:KFPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH(0:KFPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXORO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTSO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOR(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDQ 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDQDER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PS(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDGR
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGR(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDHL
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHL(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDTKE
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTKE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMCUFGP1(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMCUFGP2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMCUFGP3(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMCUFGP4(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMCUFGP5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPM(NPROMA) 
LOGICAL           ,INTENT(IN)    :: LDEXT(NGFL_EXT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXT(NPROMA,NFLEVG,NGFL_EXT) 
LOGICAL           ,INTENT(IN)    :: LDA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDO3 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3(NPROMA,NFLEVG) 
LOGICAL           ,INTENT(IN)    :: LDUD
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUAL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUOM(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDDD
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDAL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDOM(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDUEN
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUEN(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDUNEBH
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUNEBH(NPROMA,NFLEVG)
LOGICAL           ,INTENT(IN)    :: LDAERO(NAERO)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAERO(NPROMA,NFLEVG,NAERO)
LOGICAL           ,INTENT(IN)    :: LDGHG(NGHG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGHG(NPROMA,NFLEVG,NGHG) 
LOGICAL           ,INTENT(IN)    :: LDTRAC(NTRAC)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTRAC(NPROMA,NFLEVG,NTRAC)
LOGICAL           ,INTENT(IN)    :: LDGRG(NGRG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGRG(NPROMA,NFLEVG,NGRG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPD(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSVD(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEV(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPP(NPROMA,KFLDSPP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAUX(NPROMA,KFLDAUX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXTRA(NPROMA,YSD_XAD%NLEVS,YSD_XAD%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PERA40(NPROMA,NFLEVG,NERA40)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLRCH4(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH4S(NPROMA,NFLEVG)

!     ------------------------------------------------------------------

!     ==== WORKING ARRAYS FOR INTERPOLATIONS ===

!     ZPRESH, ZPRESF : HALF LEVEL, FULL LEVEL HYD PRESSURES
!     ZGEOPH, ZGEOPF : HALF LEVEL, FULL LEVEL GEOPOTENTIALS
!     ZPRES, ZLNPRES : PP. PRESSURES, LN OF PP.PRESSURES
!     ZGEOP          : PP. GEOPOTENTIALS
!     ZPXP : HALF,FULL AND LN HALF,FULL LEVEL PRESSURES or GEOPOTENTIALS
!     ZPXPD: 1./D(P) AND 1./D(LN(P)) or 1./D(Z) AND 1./D(LN(Z))

REAL(KIND=JPRB) ::    ZPRESH(NPROMA,0:NFLEVG),  ZPRESF (NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZNHPREF(NPROMA,NFLEVG) ! "pre" at full levels.
REAL(KIND=JPRB) ::    ZNHPPI(NPROMA,NFLEVG)  ! "pre/prehyd" at full levels.
REAL(KIND=JPRB) ::    ZRNHPPI(NPROMA,NFLEVG) ! "prehyd/pre" at full levels.
REAL(KIND=JPRB) ::    ZGEOPH(NPROMA,0:NFLEVG),  ZGEOPF (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZPRES (NPROMA,KOPLEV),   ZLNPRES(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZGEOP (NPROMA,KOPLEV),    ZRPRESF(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZPXPD (NPROMA,0:NFLEVG,NPPM), ZPXP(NPROMA,0:NFLEVG,NPPM)
REAL(KIND=JPRB) ::    ZQCHAL(NPROMA,NFLEVG)  ! zon comp of grad(log(pre/prehyd))
REAL(KIND=JPRB) ::    ZQCHAM(NPROMA,NFLEVG)  ! mer comp of grad(log(pre/prehyd))

REAL(KIND=JPRB) ::    ZRNHPPI_HYD(NPROMA,NFLEVG) ! equal to 1.
REAL(KIND=JPRB) ::    ZQCHAL_HYD(NPROMA,NFLEVG)  ! equal to 0.
REAL(KIND=JPRB) ::    ZQCHAM_HYD(NPROMA,NFLEVG)  ! equal to 0.

REAL(KIND=JPRB) ::    ZDELP (NPROMA,NFLEVG), ZRDELP(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZLNPR (NPROMA,NFLEVG), ZRPRE (NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZRPP  (NPROMA,NFLEVG), ZALPH (NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZRTGR (NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZLNPRL(NPROMA,NFLEVG), ZLNPRM(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZALPHL(NPROMA,NFLEVG), ZALPHM(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZALPHPLL(NPROMA,NFLEVG),ZALPHPLM(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZOROGL(NPROMA),        ZOROGM(NPROMA)
REAL(KIND=JPRB) ::    ZGMAPP(NPROMA)

INTEGER(KIND=JPIM) :: ILEVB(NPROMA,KOPLEV,NPPM)
INTEGER(KIND=JPIM) :: ILEVBH(NPROMA,KOPLEV,NPPM,0:1)
INTEGER(KIND=JPIM) :: IPTRLEV(KOPLEV) ! IPTRLEV : absolute indexes of levels
LOGICAL :: LLBELO(NPROMA,KOPLEV),   LLBLOW(KOPLEV)
LOGICAL :: LLBELS(NPROMA,KOPLEV),   LLBLES(KOPLEV)
LOGICAL :: LLBELOH(NPROMA,KOPLEV,0:1), LLBLOWH(KOPLEV,0:1)
LOGICAL :: LLBELSH(NPROMA,KOPLEV,0:1), LLBLESH(KOPLEV,0:1)

!     === LOCAL MODEL VARIABLES (full levels, except ZWWT0 : half levels) ==

REAL(KIND=JPRB) ::    ZUT0  (NPROMA,0:NFLEVG),    ZVT0  (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZUH   (NPROMA,0:NFLEVG),    ZVH   (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZX0   (NPROMA,NFLEVG),      ZWUV  (NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZTT0  (NPROMA,0:NFLEVG),    ZQT0  (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZLT0  (NPROMA,0:NFLEVG),    ZIT0  (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZPDT0 (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZWWT0 (NPROMA,0:NFLEVG) ! "w" at half levels.
REAL(KIND=JPRB) ::    ZGWT0 (NPROMA,0:NFLEVG) ! "gw" at half levels.
REAL(KIND=JPRB) ::    ZDIVT0(NPROMA,0:NFLEVG),    ZVORT0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZAT0  (NPROMA,0:NFLEVG),    ZO3T0 (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZDALT0  (NPROMA,0:NFLEVG),  ZDOMT0 (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZUALT0  (NPROMA,0:NFLEVG),  ZUOMT0 (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZUENT0  (NPROMA,0:NFLEVG),  ZUNEBHT0 (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZGHGT0(NPROMA,0:NFLEVG,NGHG)
REAL(KIND=JPRB) ::    ZTRACT0(NPROMA,0:NFLEVG,NTRAC)
REAL(KIND=JPRB) ::    ZAEROT0(NPROMA,0:NFLEVG,NAERO)
REAL(KIND=JPRB) ::    ZGRGT0(NPROMA,0:NFLEVG,NGRG)
REAL(KIND=JPRB) ::    ZSNT0 (NPROMA,0:NFLEVG),    ZRRT0 (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZGRT0 (NPROMA,0:NFLEVG),    ZTKET0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZHLT0 (NPROMA,0:NFLEVG),    ZGRHLT0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZEXTT0(NPROMA,0:NFLEVG,NGFL_EXT)

REAL(KIND=JPRB) ::    ZUT0L(NPROMA,0:NFLEVG), ZVT0L(NPROMA,0:NFLEVG)

REAL(KIND=JPRB) ::    ZTT0L(NPROMA,NFLEVG),     ZTT0M(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZQT0L(NPROMA,NFLEVG),     ZQT0M(NPROMA,NFLEVG)

REAL(KIND=JPRB) ::    ZSPL(NPROMA),            ZSPM(NPROMA)

REAL(KIND=JPRB) ::    ZTSTAR(NPROMA),          ZT0(NPROMA)

REAL(KIND=JPRB) ::    ZUS0(NPROMA) ! U_surf
REAL(KIND=JPRB) ::    ZVS0(NPROMA) ! V_surf

!     ZTOUP : potential vorticity
!     ZTETA : potential temperature
!     ZEPTH : equivalent potential temperature
!     ZIBET : virtual potential temperature
!     ZSTDF : stretching deformation
!     ZSHDF : shearing deformation
!     ZSIMR : simulated reflectivity
!     ZVIRT : virtual t or theta (different from what is computed by ZIBET)

REAL(KIND=JPRB) ::    ZTOUP(NPROMA,0:NFLEVG),   ZTETA(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZSTDF(NPROMA,0:NFLEVG),   ZSHDF(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZEPTH(NPROMA,0:NFLEVG),   ZIBET(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZSIMR(NPROMA,0:NFLEVG),   ZVIRT(NPROMA,0:NFLEVG)

REAL(KIND=JPRB) ::    ZKAP0(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZVDT0(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZDVER0(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZR0  (NPROMA,NFLEVG),     ZCP0 (NPROMA,NFLEVG)

!     ==== LOCAL POST-PROCESSED FIELDS  ===

REAL(KIND=JPRB) ::    ZPPU (NPROMA,KOPLEV),   ZPPV (NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPT (NPROMA,KOPLEV),   ZPPQ (NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPHI  (NPROMA,KOPLEV),   ZPPHRL (NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPVVE (NPROMA,KOPLEV),   ZPPEXT(NPROMA,KOPLEV,NGFL_EXT)
REAL(KIND=JPRB) ::    ZPPL (NPROMA,KOPLEV),   ZPPTHPW(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPI (NPROMA,KOPLEV),   ZPPA (NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPPDEP(NPROMA,KOPLEV),   ZPPVDIV(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPWIND(NPROMA,KOPLEV),   ZPPTETA(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPEPTH(NPROMA,KOPLEV),   ZPPSTDF(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPSHDF(NPROMA,KOPLEV),   ZPPPOTV(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPUAP (NPROMA,KOPLEV),   ZPPIBET(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPPDIV(NPROMA,KOPLEV),   ZPPPVOR(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPWVEL(NPROMA,KOPLEV),   ZPPO3(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPDAL(NPROMA,KOPLEV),   ZPPDOM(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPUAL(NPROMA,KOPLEV),   ZPPUOM(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPUEN(NPROMA,KOPLEV),   ZPPUNEBH(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPMG   (NPROMA,KOPLEV),   ZPPPABS(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPGHG(NPROMA,KOPLEV,NGHG)
REAL(KIND=JPRB) ::    ZPPTRAC(NPROMA,KOPLEV,NTRAC)
REAL(KIND=JPRB) ::    ZPPAERO(NPROMA,KOPLEV,NAERO)
REAL(KIND=JPRB) ::    ZPPGRG(NPROMA,KOPLEV,NGRG)
REAL(KIND=JPRB) ::    ZPPSN(NPROMA,KOPLEV),   ZPPRR(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPGR(NPROMA,KOPLEV),   ZPPTKE(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPHL(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPSIMR(NPROMA,KOPLEV),   ZPPVIRT(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPEVEL(NPROMA,KOPLEV)
REAL(KIND=JPRB) ::    ZPPIN(NPROMA,KOPLEV),ZPPIN1(NPROMA)


REAL(KIND=JPRB),ALLOCATABLE :: ZPPERA(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZPPFUA(:,:,:), ZPPFSU(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZEXTRA(:,:),ZERA40(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZLRCH4T0(:,:),ZPPLRCH4(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZCH4ST0(:,:),ZPPCH4S(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZNOGWT0(:,:,:),ZPPNOGW(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZAEROUT0(:,:,:),ZPPAEROUT(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZUVPT0(:,:,:)  ,ZPPUVP(:,:,:)

REAL(KIND=JPRB) ::    ZPPPMER(NPROMA),          ZFOLDIN(NPROMA)
REAL(KIND=JPRB) ::    ZPPCUF1(NPROMA)
REAL(KIND=JPRB) ::    ZPPCUF2(NPROMA)
REAL(KIND=JPRB) ::    ZPPCUF3(NPROMA)
REAL(KIND=JPRB) ::    ZPPCUF4(NPROMA)
REAL(KIND=JPRB) ::    ZPPCUF5(NPROMA)
REAL(KIND=JPRB) ::    ZUJET  (NPROMA),          ZVJET  (NPROMA)
REAL(KIND=JPRB) ::    ZPJET  (NPROMA),          ZPICAO (NPROMA)
REAL(KIND=JPRB) ::    ZTICAO (NPROMA),ZPPSP(NPROMA),ZLNPPSP(NPROMA)
REAL(KIND=JPRB) ::    ZMOCO  (NPROMA),          ZPPTWV (NPROMA)
REAL(KIND=JPRB) ::    ZPT0B  (NPROMA),          ZHT0B  (NPROMA)
REAL(KIND=JPRB) ::    ZPT1B  (NPROMA),          ZHT1B  (NPROMA)
REAL(KIND=JPRB) ::    ZHTPW  (NPROMA)

!     ==== OTHER VARIABLES  ===

REAL(KIND=JPRB) ::    ZWORK(NPROMA), ZLNWORK(NPROMA)
REAL(KIND=JPRB) ::    ZEVEL (NPROMA,0:NFLEVG), ZVVEL (NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZETADOT(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZPSDIV(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZPSDVBC(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) ::    ZDIVDP(NPROMA,NFLEVG)
REAL(KIND=JPRB) ::    ZES   (NPROMA,NFLEVG), ZRHF (NPROMA,NFLEVG)

!     ZPPWVELH : true vertical velocity post-processed at half-levels of pp.
!                ZPPWVELH(.,JLEV,0) : half level above full level JLEV
!                ZPPWVELH(.,JLEV,1) : half level below full level JLEV
!     ZPH      : half level pressures of pp. (same rule as above)
REAL(KIND=JPRB) ::    ZPPWVELH(NPROMA,KOPLEV,0:1)
REAL(KIND=JPRB) ::    ZPH     (NPROMA,KOPLEV,0:1)

REAL(KIND=JPRB) :: ZGPHL(NPROMA,0:NFLEVG) ! zon comp of grad(gz) at half levels
REAL(KIND=JPRB) :: ZGPHM(NPROMA,0:NFLEVG) ! mer comp of grad(gz) at half levels
REAL(KIND=JPRB) :: ZGPFL(NPROMA,  NFLEVG) ! zon comp of grad(gz) at full levels
REAL(KIND=JPRB) :: ZGPFM(NPROMA,  NFLEVG) ! mer comp of grad(gz) at full levels
REAL(KIND=JPRB) :: ZRT(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRTL(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRTM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTPW0(NPROMA,NFLEVG) ! Model level T'w
REAL(KIND=JPRB) :: ZSTTF(NPROMA,0:NFLEVG) ! ICAO temperature
REAL(KIND=JPRB) :: ZSTZF(NPROMA,0:NFLEVG) ! ICAO geopotential
REAL(KIND=JPRB) :: ZGWFT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZWWS(NPROMA) 
REAL(KIND=JPRB) :: ZLNPRESF(NPROMA,NFLEVG),ZSPD(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZIN(NPROMA,0:NFLEVG+1),ZOU(NPROMA,0:NFLEVG+1)

INTEGER(KIND=JPIM) :: IHILEV, ILEVM, ILOLEV, IPOL2,INFUA,ILOC,&
 & ILOCU,ILOCV, IPTR, ISLCT, JH, JL, JLEV, JLEVP, JROF, JVAR, INFSU, I, ILPT,&
 & IPROMVC, JVEXTR, JFLD, IBL 

LOGICAL :: LL, LLAERO, LLDIV0, LLVERCOR, &
 & LLICE, LLLGFL, LLNIVP, LLTDER, LLUVDER, &
 & LLVOR0, LLWAT, LLTOP  ,LLEXTR, LLVV
LOGICAL :: LLQDER
LOGICAL :: LLDER
LOGICAL :: LLGWF,LLGDWI
LOGICAL :: LLUNIT
LOGICAL :: LLVFE

LOGICAL :: LL_SP      ! surface pressure
LOGICAL :: LL_LNSP    ! log. surface pressure
LOGICAL :: LL_MSL     ! mean sea level pressure
LOGICAL :: LL_CUF1     ! Filtred ln(Ps),NCUFNR=1 for monitoring Coupling Updates Frequency
LOGICAL :: LL_CUF2     ! Filtred ln(Ps),NCUFNR=2 for monitoring CUP
LOGICAL :: LL_CUF3     ! Filtred ln(Ps),NCUFNR=3 for monitoring CUP
LOGICAL :: LL_CUF4     ! Filtred ln(Ps),NCUFNR=4 for monitoring CUF
LOGICAL :: LL_CUF5     ! Filtred ln(Ps),NCUFNR=5 for monitoring CUF

LOGICAL :: LL_FOL     ! tropopause folding ind.
LOGICAL :: LL_UJET    ! ICAO U component of the jet
LOGICAL :: LL_VJET    ! ICAO V component of the jet
LOGICAL :: LL_PJET    ! ICAO pressure at jet level
LOGICAL :: LL_PCAO    ! ICAO tropopause pressure
LOGICAL :: LL_TCAO    ! ICAO tropopause temperature
LOGICAL :: LL_WWS     ! surface vertical velocity
LOGICAL :: LL_PT0B    ! Pressure of iso-T=0 celsius, from bottom
LOGICAL :: LL_HT0B    ! height of iso-T=0 celsius, from bottom
LOGICAL :: LL_PT1B    ! Pressure of iso-T=-10 celsius, from bottom
LOGICAL :: LL_HT1B    ! height of iso-T=-10 celsius, from bottom
LOGICAL :: LL_HTPW    ! height of iso-T'w=0 Kelvin, from bottom
LOGICAL :: LL_PPUV    ! Wind vector
LOGICAL :: LL_T       ! temperature
LOGICAL :: LL_Z       ! geopotential
LOGICAL :: LL_VV      ! vertical velocity omega
LOGICAL :: LL_ETAD    ! vertical velocity etadot
LOGICAL :: LL_Q       ! specific humidity
LOGICAL :: LL_HU      ! relative humidity
LOGICAL :: LL_L       ! cloud liquid water
LOGICAL :: LL_I       ! cloud ice
LOGICAL :: LL_CLF     ! cloud fraction
LOGICAL :: LL_O3MX    ! ozone mixing ratio
LOGICAL :: LL_DAL     ! DD mesh fraction
LOGICAL :: LL_DOM     ! DD omega*dt
LOGICAL :: LL_UAL     ! UD mesh fraction
LOGICAL :: LL_UOM     ! UD omega*dt
LOGICAL :: LL_UEN     ! UD entrainment
LOGICAL :: LL_UNEBH   ! Convective pseudo historic cloud fraction
LOGICAL :: LL_SN      ! snow
LOGICAL :: LL_RR      ! rain
LOGICAL :: LL_GR      ! graupel
LOGICAL :: LL_HL      ! hail
LOGICAL :: LL_TKE     ! turbulent kinetic energy
LOGICAL :: LL_THPW    ! theta'w
LOGICAL :: LL_WND     ! wind velocity
LOGICAL :: LL_DIV     ! Divergence
LOGICAL :: LL_VOR     ! Relative Vorticity
LOGICAL :: LL_ABS     ! Absolute Vorticity
LOGICAL :: LL_TH      ! Potential temperature
LOGICAL :: LL_ETH     ! Equivalent pot. temperature
LOGICAL :: LL_IET     ! Isobaric equivalent temperature
LOGICAL :: LL_SRE     ! Simulated reflectivity
LOGICAL :: LL_THV     ! virtual t or theta 
LOGICAL :: LL_STD     ! Stretching Deformation
LOGICAL :: LL_SHD     ! Shearing Deformation
LOGICAL :: LL_PV      ! Potential Vorticity
LOGICAL :: LL_TWV     ! Total water vapour
LOGICAL :: LL_P       ! Pressure
LOGICAL :: LL_MG      ! Montgomery potential
LOGICAL :: LL_MOCO    ! Moisture convergence
LOGICAL :: LL_PD      ! n.-h. pressure departure 
LOGICAL :: LL_VD      ! vertical divergence 
LOGICAL :: LL_VW      ! true vertical velocity w
LOGICAL :: LL_ERA40   ! ERA40 diagnostic fields (one logical for all fields)
LOGICAL :: LL_LRCH4   ! CH4 loss rate
LOGICAL :: LL_CH4S    ! CH4 atmospheric sink
LOGICAL :: LL_GHG(NGHG)  ! Greenhouse Gases
LOGICAL :: LL_TRAC(NTRAC)  ! Tracers for diagnostics only
LOGICAL :: LL_AERO(NAERO)  ! Aerosols
LOGICAL :: LL_GRG(NGRG)  ! GEMS GRG 
LOGICAL :: LL_GFL(NGFL_EXT) ! passive scalar
LOGICAL :: LL_NOGW(NNOGW)   ! noro gwd scheme fields
LOGICAL :: LL_AEROUT(NAEROUT)! Aerosol outputs
LOGICAL :: LL_UVP(NUVP)! UV-processor outputs
LOGICAL :: LL_FUA(JPOSVX2) ! Free upper air fields
LOGICAL :: LL_FSU(JPOSFSU) ! Free surface fields

REAL(KIND=JPRB) :: Z1SRG, ZDUMARR(1), ZMDTDZSG, ZRHMAX, ZRHMIN, ZTROPO, ZXTEMP
! * useless dummy arguments if "lvercor=.f.":
REAL(KIND=JPRB) :: ZVCRSSA0F(1),ZVCRSSA0H(1),ZVCASRS0F(1)
REAL(KIND=JPRB) :: ZVCASRS0H(1),ZVCRSSA0FL(1),ZVCRSSA0FM(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "apache.intfb.h"
#include "ctstar.intfb.h"
#include "fpps.intfb.h"
#include "gnhdlrb.intfb.h"
#include "gnhpre.intfb.h"
#include "gpcty.intfb.h"
#include "gpept.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgrgeo.intfb.h"
#include "gpxx.intfb.h"
#include "gphlwi.intfb.h"
#include "gphluv.intfb.h"
#include "gpgrxyb.intfb.h"
#include "gpgw.intfb.h"
#include "gpiet.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gpprs0d.intfb.h"
#include "gppvo.intfb.h"
#include "gprcp.intfb.h"
#include "gprh.intfb.h"
#include "gprt.intfb.h"
#include "gpuvs.intfb.h"
#include "gpverdia.intfb.h"
#include "gpxyb.intfb.h"
#include "poaero.intfb.h"
#include "pp2dint.intfb.h"
#include "ppflev.intfb.h"
#include "ppgeop.intfb.h"
#include "ppinit.intfb.h"
#include "ppleta.intfb.h"
#include "ppltemp.intfb.h"
#include "pplteta.intfb.h"
#include "ppltp.intfb.h"
#include "pppmer.intfb.h"
#include "ppq.intfb.h"
#include "pprh.intfb.h"
#include "ppt.intfb.h"
#include "ppthpw.intfb.h"
#include "ppuv.intfb.h"
#include "ppvvel.intfb.h"
#include "ppwetpoint.intfb.h"
#include "ppsta.intfb.h"
#include "ppcvirt.intfb.h"
#include "verder.intfb.h"
#include "verint.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('POS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INITIALIZATIONS.
!              ---------------

!*    1.1  LOGICAL AND OTHER SCALAR INITIALIZATIONS

!     1.1.1  LL_[X] for GFL

LL_Q    = QFPTYPE%LL(TFP_Q%ICOD)
LL_L    = QFPTYPE%LL(TFP_L%ICOD)
LL_I    = QFPTYPE%LL(TFP_I%ICOD)
LL_CLF  = QFPTYPE%LL(TFP_CLF%ICOD)
LL_O3MX = QFPTYPE%LL(TFP_O3MX%ICOD)
LL_DAL = QFPTYPE%LL(TFP_DAL%ICOD)
LL_DOM = QFPTYPE%LL(TFP_DOM%ICOD)
LL_UAL = QFPTYPE%LL(TFP_UAL%ICOD)
LL_UOM = QFPTYPE%LL(TFP_UOM%ICOD)
LL_UEN = QFPTYPE%LL(TFP_UEN%ICOD)
LL_UNEBH = QFPTYPE%LL(TFP_UNEBH%ICOD)
LL_SN   = QFPTYPE%LL(TFP_SN%ICOD)
LL_RR   = QFPTYPE%LL(TFP_RR%ICOD)
LL_GR   = QFPTYPE%LL(TFP_GR%ICOD)
LL_HL   = QFPTYPE%LL(TFP_HL%ICOD)
LL_TKE  = QFPTYPE%LL(TFP_TKE%ICOD)
LL_LRCH4 = YLRCH4%LACTIVE
LL_CH4S = YCH4S%LACTIVE
DO JVAR=1,NGHG
  LL_GHG(JVAR)=QFPTYPE%LL(TFP_GHG(JVAR)%ICOD)
ENDDO
DO JVAR=1,NTRAC
  LL_TRAC(JVAR)=QFPTYPE%LL(TFP_TRAC(JVAR)%ICOD)
ENDDO
DO JVAR=1,NAERO
  LL_AERO(JVAR)=QFPTYPE%LL(TFP_AERO(JVAR)%ICOD)
ENDDO
DO JVAR=1,NGRG
  LL_GRG(JVAR)=QFPTYPE%LL(TFP_GRG(JVAR)%ICOD)
ENDDO
DO JVAR=1,NGFL_EXT
  LL_GFL(JVAR) = QFPTYPE%LL(TFP_GFL(JVAR)%ICOD)
ENDDO
DO JVAR=1,NNOGW
  LL_NOGW(JVAR) = QFPTYPE%LL(TFP_NOGW(JVAR)%ICOD)
ENDDO
DO JVAR=1,NAEROUT
  LL_AEROUT(JVAR)=QFPTYPE%LL(TFP_AEROUT(JVAR)%ICOD)
ENDDO
DO JVAR=1,NUVP
  LL_UVP(JVAR)=QFPTYPE%LL(TFP_UVP(JVAR)%ICOD)
ENDDO
LL_ERA40 = .FALSE.
! LL_ERA40 true if one of tendencies is requested
DO JVAR=1,NERA40
  IF(QFPTYPE%LL(TFP_ERA40(JVAR)%ICOD)) LL_ERA40 = .TRUE.
ENDDO

!     1.1.2  LL_[X] for other variables 

LL_SP   = QFPTYPE%LL(TFP_SP%ICOD)
LL_LNSP = QFPTYPE%LL(TFP_LNSP%ICOD)
LL_MSL  = QFPTYPE%LL(TFP_MSL%ICOD)
LL_CUF1 = QFPTYPE%LL(TFP_CUF1%ICOD)
LL_CUF2 = QFPTYPE%LL(TFP_CUF2%ICOD)
LL_CUF3 = QFPTYPE%LL(TFP_CUF3%ICOD)
LL_CUF4 = QFPTYPE%LL(TFP_CUF4%ICOD)
LL_CUF5 = QFPTYPE%LL(TFP_CUF5%ICOD)
LL_FOL  = QFPTYPE%LL(TFP_FOL%ICOD)
LL_UJET = QFPTYPE%LL(TFP_UJET%ICOD)
LL_VJET = QFPTYPE%LL(TFP_VJET%ICOD)
LL_PJET = QFPTYPE%LL(TFP_PJET%ICOD)
LL_PCAO = QFPTYPE%LL(TFP_PCAO%ICOD)
LL_TCAO = QFPTYPE%LL(TFP_TCAO%ICOD)
LL_WWS  = QFPTYPE%LL(TFP_WWS%ICOD)
LL_PT0B = QFPTYPE%LL(TFP_PT0B%ICOD)
LL_HT0B = QFPTYPE%LL(TFP_HT0B%ICOD)
LL_PT1B = QFPTYPE%LL(TFP_PT1B%ICOD)
LL_HT1B = QFPTYPE%LL(TFP_HT1B%ICOD)
LL_HTPW = QFPTYPE%LL(TFP_HTPW%ICOD)
LL_PPUV = QFPTYPE%LL(TFP_U%ICOD) .OR. QFPTYPE%LL(TFP_V%ICOD) .OR. &
 & (QFPTYPE%LL(TFP_VOR%ICOD).AND.(QFPTYPE%ISKP(TFP_VOR%ICOD) == 2)) .OR. &
 & (QFPTYPE%LL(TFP_DIV%ICOD).AND.(QFPTYPE%ISKP(TFP_DIV%ICOD) == 2)) .OR. &
 & (QFPTYPE%LL(TFP_ABS%ICOD).AND.(QFPTYPE%ISKP(TFP_ABS%ICOD) == 2)) .OR. &
 & QFPTYPE%LL(TFP_KHI%ICOD) .OR. QFPTYPE%LL(TFP_PSI%ICOD)  
LL_T    = QFPTYPE%LL(TFP_T%ICOD)
LL_Z    = QFPTYPE%LL(TFP_Z%ICOD)
LL_VV   = QFPTYPE%LL(TFP_VV%ICOD)
LL_ETAD = QFPTYPE%LL(TFP_ETAD%ICOD)
LL_HU   = QFPTYPE%LL(TFP_HU%ICOD)
LL_THPW = QFPTYPE%LL(TFP_THPW%ICOD)
LL_WND  = QFPTYPE%LL(TFP_WND%ICOD)
LL_DIV  = QFPTYPE%LL(TFP_DIV%ICOD).AND.(QFPTYPE%ISKP(TFP_DIV%ICOD) == 1)
LL_VOR  = QFPTYPE%LL(TFP_VOR%ICOD).AND.(QFPTYPE%ISKP(TFP_VOR%ICOD) == 1) .OR. &
 & QFPTYPE%LL(TFP_ABS%ICOD).AND.(QFPTYPE%ISKP(TFP_ABS%ICOD) == 1)   
LL_ABS  = QFPTYPE%LL(TFP_ABS%ICOD).AND.(QFPTYPE%ISKP(TFP_ABS%ICOD) == 1)
LL_TH   = QFPTYPE%LL(TFP_TH%ICOD)
LL_ETH  = QFPTYPE%LL(TFP_ETH%ICOD)
LL_IET  = QFPTYPE%LL(TFP_IET%ICOD)
LL_SRE  = QFPTYPE%LL(TFP_SRE%ICOD)
LL_STD  = QFPTYPE%LL(TFP_STD%ICOD)
LL_SHD  = QFPTYPE%LL(TFP_SHD%ICOD)
LL_PV   = QFPTYPE%LL(TFP_PV%ICOD)
LL_TWV  = QFPTYPE%LL(TFP_TWV%ICOD)
LL_P    = QFPTYPE%LL(TFP_P%ICOD)
LL_MG   = QFPTYPE%LL(TFP_MG%ICOD)
LL_MOCO = QFPTYPE%LL(TFP_MOCO%ICOD)
LL_PD   = QFPTYPE%LL(TFP_PD%ICOD)
LL_VD   = QFPTYPE%LL(TFP_VD%ICOD)
LL_VW   = QFPTYPE%LL(TFP_VW%ICOD)
LL_THV  = QFPTYPE%LL(TFP_THV%ICOD)
DO JVAR=1,JPOSVX2
  LL_FUA(JVAR) = QFPTYPE%LL(TFP_FUA(JVAR)%ICOD)
ENDDO
DO JVAR=1,JPOSFSU
  LL_FSU(JVAR) = QFPTYPE%LL(TFP_FSU(JVAR)%ICOD)
ENDDO

!     1.1.3  Other variables.

LLVV = LL_VV.OR.LL_ETAD

LLTDER = LL_FOL .OR. LL_PV .OR. LL_TH .OR. LL_ETH .OR. LL_IET .OR.&
 & LL_VD .OR. LL_VW .OR. LL_WWS .OR. LL_THV .OR. &
 & CDCONF == 'V' .OR. CDCONF == 'T'  
LLUVDER= LL_STD .OR. LL_SHD
LLDIV0 = LL_DIV .OR. LL_STD .OR. LL_SHD .OR. LLVV .OR. LL_VW .OR. LL_VD .OR.&
 & LL_MOCO  
LLVOR0 = LL_VOR .OR. LLTDER .OR. LL_STD .OR. LL_SHD
LLAERO = LL_UJET .OR. LL_VJET .OR. LL_PJET .OR. LL_PCAO .OR. LL_TCAO
LLQDER = LL_VD .OR. LL_VW .OR. LL_WWS .OR. LL_MOCO

! --- Relaxation of thin layer hypothesis not yet coded ----------------------
LLVERCOR=.FALSE.
IPROMVC=1
! ----------------------------------------------------------------------------

ILOLEV=1
IHILEV=KOPLEV
IBL=(KSTGLO-1)/NPROMA+1

IF (LLUVDER .AND. .NOT.LUVDER) THEN
  CALL ABOR1 ('POS INTERNAL ERROR ON LUVDER')
ENDIF
IF (LLVOR0.AND. .NOT.LVOR) THEN
  CALL ABOR1 ('POS INTERNAL ERROR ON LVOR')
ENDIF
IF (LLQDER .AND. .NOT.LDQDER .AND. .NOT.LSPRT) THEN
  CALL ABOR1 ('POS : NEEDS Q DERIVATIVES')
ENDIF

!*    1.2  ARRAYS INITIALIZATIONS

!*      1.2.0   QUANTITIES WHICH DO NOT REQUIRE 'GP..' ROUTINES

!*      1.2.0a  Constants

DO JROF=KST,KND
  ZGMAPP(JROF)=PGM(JROF)*PGM(JROF)
ENDDO

!*      1.2.0a  GFL 

DO JLEV=1,NFLEVG
  IF (LDQ) THEN
    ZQT0(KST:KND,JLEV)=PQ(KST:KND,JLEV)
  ELSE
    ZQT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDQDER) THEN
    ZQT0L(KST:KND,JLEV)=PQL(KST:KND,JLEV)*PGM(KST:KND)
    ZQT0M(KST:KND,JLEV)=PQM(KST:KND,JLEV)*PGM(KST:KND)
  ENDIF
  IF (LDL) THEN
    ZLT0(KST:KND,JLEV)=PL(KST:KND,JLEV)
  ELSE
    ZLT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDI) THEN
    ZIT0(KST:KND,JLEV)=PI(KST:KND,JLEV)
  ELSE
    ZIT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDA) THEN
    ZAT0(KST:KND,JLEV)=PA(KST:KND,JLEV)
  ELSE
    ZAT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDO3) THEN
    ZO3T0(KST:KND,JLEV)=PO3(KST:KND,JLEV)
  ELSE
    ZO3T0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDUD) THEN
    ZUALT0(KST:KND,JLEV)=PUAL(KST:KND,JLEV)
    ZUOMT0(KST:KND,JLEV)=PUOM(KST:KND,JLEV)
  ELSE
    ZUALT0(KST:KND,JLEV)=0.0_JPRB
    ZUOMT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDDD) THEN
    ZDALT0(KST:KND,JLEV)=PDAL(KST:KND,JLEV)
    ZDOMT0(KST:KND,JLEV)=PDOM(KST:KND,JLEV)
  ELSE
    ZDALT0(KST:KND,JLEV)=0.0_JPRB
    ZDOMT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDUEN) THEN
    ZUENT0(KST:KND,JLEV)=PUEN(KST:KND,JLEV)
  ELSE
    ZUENT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDUNEBH) THEN
    ZUNEBHT0(KST:KND,JLEV)=PUNEBH(KST:KND,JLEV)
  ELSE
    ZUNEBHT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDS) THEN
    ZSNT0(KST:KND,JLEV)=PS(KST:KND,JLEV)
  ELSE
    ZSNT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDR) THEN
    ZRRT0(KST:KND,JLEV)=PR(KST:KND,JLEV)
  ELSE
    ZRRT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDGR) THEN
    ZGRT0(KST:KND,JLEV)=PGR(KST:KND,JLEV)
  ELSE
    ZGRT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  IF (LDHL) THEN
    ZHLT0(KST:KND,JLEV)=PHL(KST:KND,JLEV)
  ELSE
    ZHLT0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
  ZGRHLT0(KST:KND,JLEV)=ZGRT0(KST:KND,JLEV)+ZHLT0(KST:KND,JLEV)
  IF (LDTKE) THEN
    ZTKET0(KST:KND,JLEV)=PTKE(KST:KND,JLEV)
  ELSE
    ZTKET0(KST:KND,JLEV)=0.0_JPRB
  ENDIF
ENDDO
IF(NGHG > 0 .AND. LL_LRCH4) THEN
  ALLOCATE(ZLRCH4T0(NPROMA,0:NFLEVG))
  ZLRCH4T0(KST:KND,1:NFLEVG)=PLRCH4(KST:KND,1:NFLEVG)
ENDIF
IF(NGHG > 0 .AND. LL_CH4S) THEN
  ALLOCATE(ZCH4ST0(NPROMA,0:NFLEVG))
  ZCH4ST0(KST:KND,1:NFLEVG)=PCH4S(KST:KND,1:NFLEVG)
ENDIF
IF(NNOGW > 0) THEN
  ALLOCATE(ZNOGWT0(NPROMA,0:NFLEVG,NNOGW))
  DO JVAR=1,NNOGW
    ZNOGWT0(KST:KND,1:NFLEVG,JVAR)=GFL(KST:KND,1:NFLEVG,YNOGW(JVAR)%MP,IBL)
  ENDDO
ENDIF 
IF(NUVP > 0) THEN
  ALLOCATE(ZUVPT0(NPROMA,0:NFLEVG,NUVP))
  DO JVAR=1,NUVP
    ZUVPT0(KST:KND,1:NFLEVG,JVAR)=GFL(KST:KND,1:NFLEVG,YUVP(JVAR)%MP,IBL)
  ENDDO
ENDIF 
IF(NAEROUT > 0) THEN
  ALLOCATE(ZAEROUT0(NPROMA,0:NFLEVG,NAEROUT))
  DO JVAR=1,NAEROUT
    ZAEROUT0(KST:KND,1:NFLEVG,JVAR)=GFL(KST:KND,1:NFLEVG,YAEROUT(JVAR)%MP,IBL)
  ENDDO
ENDIF 

DO JVAR=1,NGHG
  IF (LDGHG(JVAR)) THEN
    ZGHGT0(KST:KND,1:NFLEVG,JVAR)=PGHG(KST:KND,1:NFLEVG,JVAR)
  ELSE
    ZGHGT0(KST:KND,1:NFLEVG,JVAR)=0.0_JPRB
  ENDIF
ENDDO

DO JVAR=1,NTRAC
  IF (LDTRAC(JVAR)) THEN
    ZTRACT0(KST:KND,1:NFLEVG,JVAR)=PTRAC(KST:KND,1:NFLEVG,JVAR)
  ELSE
    ZTRACT0(KST:KND,1:NFLEVG,JVAR)=0.0_JPRB
  ENDIF
ENDDO

DO JVAR=1,NAERO
  IF (LDAERO(JVAR)) THEN
    ZAEROT0(KST:KND,1:NFLEVG,JVAR)=PAERO(KST:KND,1:NFLEVG,JVAR)
  ELSE
    ZAEROT0(KST:KND,1:NFLEVG,JVAR)=0.0_JPRB
  ENDIF
ENDDO

DO JVAR=1,NGRG
  IF (LDGRG(JVAR)) THEN
    ZGRGT0(KST:KND,1:NFLEVG,JVAR)=PGRG(KST:KND,1:NFLEVG,JVAR)
  ELSE
    ZGRGT0(KST:KND,1:NFLEVG,JVAR)=0.0_JPRB
  ENDIF
ENDDO

DO JVAR=1,NGFL_EXT
  ! ky: must this test be on LDEXT or LL_GFL or (LDEXT.OR.LL_GFL)?
  IF (LDEXT(JVAR)) THEN
    ZEXTT0(KST:KND,1:NFLEVG,JVAR)=PEXT(KST:KND,1:NFLEVG,JVAR)
  ELSE
    ZEXTT0(KST:KND,1:NFLEVG,JVAR)=0.0_JPRB
  ENDIF
ENDDO

! ky: seems to miss something here about ERA40

!*      1.2.0b  Other than GFL 

DO JLEV=1,NFLEVG
  ZUT0(KST:KND,JLEV)=PU(KST:KND,JLEV)*PGM(KST:KND)
  ZVT0(KST:KND,JLEV)=PV(KST:KND,JLEV)*PGM(KST:KND)
  IF (LLUVDER) THEN
    ZUT0L(KST:KND,JLEV)=PUL(KST:KND,JLEV)*ZGMAPP(KST:KND)
    ZVT0L(KST:KND,JLEV)=PVL(KST:KND,JLEV)*ZGMAPP(KST:KND)
  ENDIF
  IF (LLDIV0) THEN
    ZDIVT0(KST:KND,JLEV)=PDIV(KST:KND,JLEV)*ZGMAPP(KST:KND)
  ENDIF
  IF (LLVOR0) THEN
    ZVORT0(KST:KND,JLEV)=PVOR(KST:KND,JLEV)*ZGMAPP(KST:KND)
  ENDIF
  ZTT0(KST:KND,JLEV)=PT(KST:KND,JLEV)
  IF (LLTDER) THEN
    ZTT0L (KST:KND,JLEV)=PTL(KST:KND,JLEV)*PGM(KST:KND)
    ZTT0M (KST:KND,JLEV)=PTM(KST:KND,JLEV)*PGM(KST:KND)
  ENDIF
ENDDO

DO JROF=KST,KND
  ZOROGL(JROF)=POROGL(JROF)*PGM(JROF)
  ZOROGM(JROF)=POROGM(JROF)*PGM(JROF)
ENDDO

!*      1.2.1a  PRELIMINAR COMPUTATION OF Cp, R AND R/Cp FROM Q

IF (LL_MG .OR. LL_Z .OR. CDCONF == 'H' .OR. LLTDER .OR.&
   & LLAERO .OR. LL_T .OR. LL_THPW .OR. LL_MOCO .OR. LRWSDLR) THEN  
  CALL GPRCP (NPROMA,KST,KND,NFLEVG,PQ=ZQT0(1,1),PQI=ZIT0(1,1),&
   &  PQL=ZLT0(1,1),PQR=ZRRT0(1,1),PQS=ZSNT0(1,1),PQG=ZGRHLT0(1,1),&
   &  PCP=ZCP0,PR=ZR0,PKAP=ZKAP0)  
ENDIF

!*      1.2.1b  PRELIMINAR COMPUTATION OF RT and its horizontal derivatives

IF (LL_VD.OR.LL_VW.OR.LL_WWS) THEN
  CALL GPRT (LSPRT,NPROMA,KST,KND,NFLEVG,RD,RV,&
   & ZR0,ZTT0(1,1),ZTT0L(1,1),ZTT0M(1,1),ZQT0L(1,1),ZQT0M(1,1),&
   & ZRT,ZRTL,ZRTM)  
ENDIF

!*      1.2.2   MODEL LEVEL PRESSURES

DO JROF = KST , KND
  ZPRESH(JROF,NFLEVG) = EXP(PSP(JROF))
  IF (LLVV .OR. LLTDER) THEN
    ZSPL(JROF) = ZPRESH(JROF,NFLEVG)*PSPL(JROF)*PGM(JROF)
    ZSPM(JROF) = ZPRESH(JROF,NFLEVG)*PSPM(JROF)*PGM(JROF)
  ENDIF
ENDDO
CALL GPPREH (NPROMA,KST,KND,NFLEVG,VAH,VBH,ZPRESH)
CALL GPXYB (NPROMA,KST,KND,NFLEVG,VDELB,VC,ZPRESH,&
 & ZDELP,ZRDELP,ZLNPR,ZALPH,ZRTGR,ZRPRE,ZRPP)  
CALL GPPREF (NPROMA,KST,KND,NFLEVG,ZALPH,ZPRESH,ZPRESF)

IF(LVERTFE) THEN
  DO JLEV = 1, NFLEVG
    DO JROF = KST , KND
      ZRPRESF(JROF,JLEV)=1.0_JPRB/ZPRESF(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

IF (LL_VD.OR.LL_VW.OR.LL_WWS) THEN
  CALL GPGRXYB(NPROMA,KST,KND,NFLEVG,VC,VBH,&
   & ZSPL,ZSPM,ZRDELP,ZRPRE,ZRPP,ZLNPR,ZALPH,ZRTGR,&
   & ZLNPRL,ZLNPRM,ZALPHL,ZALPHM,ZALPHPLL,ZALPHPLM)
ENDIF

! Save PSPD in ZSPD (after that, PSPD should not be used any more in POS).
IF (LNHDYN) ZSPD(KST:KND,1:NFLEVG)=PSPD(KST:KND,1:NFLEVG)

! Provisional treatment of the deep-layer NH equations.
! POS is not properly coded for the time being to deal properly with that
!  (dynamics using GP routines and externalisable interpolator should be
!  well separated, dynamics should use "presim" and NH deep-layer equations,
!  externalisable interpolator should see only the hydrostatic pressure
!  "prehyd" and should not call any GP.. routine; there are also parts not
!  yet updated for NH dynamics). 
! Provisionally, all the calculations will be done with thin layer equations,
!  we simply transform the "presim" based variables into "prehyd" based
!  variables. We recall that (d prehyd/d presim) = a**2/r**2.
! Transformation of ZRTGR and derivatives computed in GPGRXYB is not easy to
!  do, so we assume that the difference between derivatives based on "presim"
!  and derivatives based on "prehyd" is negligible (we assume for example that
!  grad(presim) is roughly equal to grad(prehyd)).
IF (LRWSDLR) THEN
  ! Save log(presim).
  DO JLEV=1,NFLEVG
    DO JROF=KST,KND
      ZLNPRESF(JROF,JLEV)=LOG(ZPRESF(JROF,JLEV))
    ENDDO
  ENDDO
  ! Transform "presim" quantities into "prehyd" quantities.
  CALL GNHDLRB(NPROMA,KST,KND,.TRUE.,POROG,PT,ZR0,PSPD, &
   & ZPRESF,ZPRESH,ZDELP(1,1),ZRDELP,ZLNPR,ZALPH,PRPREHO=ZRPRE,PRPPO=ZRPP)
  ! Convert PSPD from log(pre/presim)+rc_pd1*rcorpdf to log(pre/prehyd)
  !  log(pre/prehyd)=[log(pre/presim)+(rc_pd1*rcorpdf)]
  !   +[log(presim)-log(prehyd)-(rc_pd1*rcorpdf)]
  !  No conversion is done on horizontal derivatives.
  IF (NPDVAR==2) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KND
        ZSPD(JROF,JLEV)=ZSPD(JROF,JLEV) &
         & +ZLNPRESF(JROF,JLEV)-LOG(ZPRESF(JROF,JLEV))-RC_PD1*RCORPDF(JLEV)
      ENDDO
    ENDDO
  ENDIF
  ! From now on PSP should not be used any more in POS. Use ZPRESH instead.
ENDIF

!*      1.2.3   NH PRESSURE DEPARTURE

IF (LL_PD.OR.LL_VD.OR.LL_VW) THEN
  IF (LNHDYN) THEN
    IF (LFPART2) THEN
      ZPDT0(KST:KND,1:NFLEVG)=ZSPD(KST:KND,1:NFLEVG)
    ELSE
      IF (LNH_GEOGW) CALL ABOR1('POS 1.2.3: lnh_geogw not yet coded')
      CALL GNHPRE(NPROMA,NFLEVG,KST,KND,ZSPD,ZPRESF, &
       & ZNHPREF,ZNHPPI,ZRNHPPI,ZPDT0(1,1))
      !nyc -- not yet coded (need PSPDL,PSPDM as input data)
      !nyc CALL GNHGRPRE(NPROMA,NFLEVG,KST,KND,ZRTGR,ZSPL,ZSPM, &
      !nyc  & ZNHPREF,PSPDL,PSPDM, &
      !nyc  & ZNHPREL,ZNHPREM,ZLNNHPREFL,ZLNNHPREFM,ZQCHAL,ZQCHAM)
    ENDIF
  ELSE
    ZPDT0(KST:KND,1:NFLEVG)=0.0_JPRB
    ZNHPPI(KST:KND,1:NFLEVG)=1.0_JPRB
    ZRNHPPI(KST:KND,1:NFLEVG)=1.0_JPRB
    ZNHPREF(KST:KND,1:NFLEVG)=ZPRESF(KST:KND,1:NFLEVG)
    ZQCHAL(KST:KND,1:NFLEVG)=0.0_JPRB
    ZQCHAM(KST:KND,1:NFLEVG)=0.0_JPRB
  ENDIF

ENDIF

!*      1.2.4   GRADIENT OF GEOPOTENTIAL AT HALF LEVELS

IF (LL_VD.OR.LL_VW.OR.LL_WWS) THEN

  ! ky: caution! input data ZQCHAL,ZQCHAM are not yet coded for NH model.
  !  (need to have PSPDL and PSPDM as dummy input data, and to call
  !  GNHGRPRE to compute ZQCHAL and ZQCHAM)
  !  If LNHDYN=F, ZQCHAL and ZQCHAM are not used in GPGRGEO.
  !  (the call of GPGEO later in this routine is correctly coded for the
  !  hydrostatic model only).

  !nyc -- not yet coded (need ZRNHPPI,ZQCHAL,ZQCHAM as input data)
  !nyc -- this is normally the good way of computing grad(gz) in a NH model
  !nyc -- in a NH model, requires to call GNHGRPRE before
  !nyc CALL GPGRGEO(LNHDYN,&
  !nyc  & NPROMA,KST,KND,NFLEVG,&
  !nyc  & ZRNHPPI,PT,ZR0,ZRTL,ZRTM,&
  !nyc  & ZLNPR,ZLNPRL,ZLNPRM,ZALPH,ZALPHL,ZALPHM,&
  !nyc  & ZQCHAL,ZQCHAM,&
  !nyc  & ZOROGL,ZOROGM,&
  !nyc  & ZGPFL,ZGPFM,ZGPHL,ZGPHM)

  ! ky: provisionally one computes grad(gz) in a hydrostatic way
  !     even if the model is NH, in order get reproducibility with
  !     the old routine EGPHI.
  !     The same remark can be done about the call of GPGEO in part 1.3.1
  !      ((gz) is currently computed in a hydrostatic way even in the NH model)
  ZRNHPPI_HYD(KST:KND,1:NFLEVG)=1.0_JPRB
  ZQCHAL_HYD(KST:KND,1:NFLEVG)=0.0_JPRB
  ZQCHAM_HYD(KST:KND,1:NFLEVG)=0.0_JPRB
  CALL GPGRGEO(LNHDYN,&
   & NPROMA,KST,KND,NFLEVG,&
   & ZRNHPPI_HYD,PT,ZR0,ZRTL,ZRTM,&
   & ZLNPR,ZLNPRL,ZLNPRM,ZALPH,ZALPHL,ZALPHM,&
   & ZQCHAL_HYD,ZQCHAM_HYD,&
   & ZOROGL,ZOROGM,&
   & ZGPFL,ZGPFM,ZGPHL,ZGPHM)

ENDIF

!*      1.2.5   INTEGRATION OF HYDROSTATIC

IF (LL_VD.OR.LL_VW.OR.(LLVV.AND.(CDCONF == 'M'))) THEN
  CALL GPCTY (LLVERCOR,NPROMA,IPROMVC,KST,KND,NFLEVG,VDELB,VBH,ZUT0(1,1),&
   & ZVT0(1,1),ZDIVT0(1,1),PEV,.FALSE.,ZDELP,ZRDELP,ZLNPR,ZALPH,&
   & ZRTGR,ZSPL,ZSPM,&
   & ZVCRSSA0F,ZVCRSSA0H,ZVCASRS0F,ZVCASRS0H,ZVCRSSA0FL,ZVCRSSA0FM,&
   & ZEVEL,ZVVEL,ZPSDIV,ZPSDVBC,ZDIVDP,ZRPRESF)
  DO JLEV=1,NFLEVG
    IF(LVERTFE) THEN
      ZETADOT(KST:KND,JLEV)=ZEVEL(KST:KND,JLEV) &
       & *(VETAH(JLEV)-VETAH(JLEV-1))*ZRDELP(KST:KND,JLEV)
    ELSE
      ZETADOT(KST:KND,JLEV)=&
       & 0.5_JPRB*(ZEVEL(KST:KND,JLEV)+ZEVEL(KST:KND,JLEV-1)) &
       & *(VETAH(JLEV)-VETAH(JLEV-1))*ZRDELP(KST:KND,JLEV)
    ENDIF
  ENDDO
ENDIF

!*      1.2.6   VERTICAL DIVERGENCE

IF (LL_VD.OR.LL_VW.AND.(.NOT.LFPART2)) THEN
  IF (LNHDYN) THEN
    IF (NVDVAR == 3) THEN
      ZDVER0(KST:KND,1:NFLEVG)=PSVD(KST:KND,1:NFLEVG)
    ELSEIF (NVDVAR == 4) THEN
      IF (.NOT.(LVERTFE.AND.LVFE_X_TERM)) THEN
        ! compute interpolation weights
        CALL GPHLWI(NPROMA,KST,KND,ZLNPR,ZALPH,ZWUV)  
        ! interpolate wind into half levels
        CALL GPHLUV(NPROMA,KST,KND,ZWUV,ZUT0(1,1),ZVT0(1,1),ZUH,ZVH)
      ENDIF
      CALL GPXX(NFLEVG,NPROMA,KST,KND,ZGPHL,ZGPHM,ZGPFL,ZGPFM,ZLNPR, &
       & ZRNHPPI_HYD,PT,ZR0,ZUT0(1,1),ZVT0(1,1),ZUH,ZVH,ZX0)
      ZDVER0(KST:KND,1:NFLEVG)= &
       & PSVD(KST:KND,1:NFLEVG)-ZX0(KST:KND,1:NFLEVG)
      ! see above  
      !      CALL ABOR1(' POS: computation of dver not ready for NVDVAR = 4')
      ! * in this case, compute X (array ZX0) by calling routine GPXX,
      !   then use the NVDVAR == 3 formula with replacing PSVD by (PSVD-X).
      !nyc ZDVER0(KST:KND,1:NFLEVG)= &
      !nyc & PSVD(KST:KND,1:NFLEVG)-ZX0(KST:KND,1:NFLEVG)
    ENDIF
    IF (LNH_GEOGW) CALL ABOR1('POS 1.2.6: lnh_geogw not yet coded')
  ELSE
    ! dver_hyd=-(R/Rd)*(D+X+(cv/cp)(omega/prehyd))
    CALL GPVERDIA(NPROMA,KST,KND,ZUT0(1,1),ZVT0(1,1),&
     & ZDIVT0(1,1),ZVVEL,ZGPHL,ZGPHM,ZGPFL,ZGPFM,ZLNPR,&
     & ZALPH,ZRT,ZKAP0,ZVDT0)
    DO JLEV=1,NFLEVG
      DO JROF=KST,KND
        ZDVER0(JROF,JLEV)=(ZR0(JROF,JLEV)/RD)*ZVDT0(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      1.2.7   VERTICAL VELOCITY

Z1SRG=1.0_JPRB/RG
IF (LL_VD.OR.LL_VW.OR.LL_WWS) THEN
  ! * first compute (U,V) at the surface.
  LLDER=.FALSE.
  CALL GPUVS(NFLEVG,NPROMA,KST,KND,LLDER,ZUT0(1,1),ZVT0(1,1),ZUS0,ZVS0)
  IF(LL_WWS.AND.(.NOT.LL_VD).AND.(.NOT.LL_VW)) THEN
    ! * Simply compute (gw)_surf and w_surf.
    DO JROF=KST,KND
      ZGWT0(JROF,NFLEVG)=(ZUS0(JROF)*ZGPHL(JROF,NFLEVG)+&
       & ZVS0(JROF)*ZGPHM(JROF,NFLEVG))
      ZWWT0(JROF,NFLEVG)=ZGWT0(JROF,NFLEVG)*Z1SRG
    ENDDO
  ELSE
    ! ky: for the time being, LLVFE is set to .F. anytime because half level
    !     values of ZWWT0 may be required in parts 2 and 3, and because
    !     hard coded FD code for conversions [Delta(w)] <-> w is also present
    !     in routines like SPECFITA and SPECFITG; the VFE case
    !     must be properly studied and coded later. Provisionally, a FD 
    !     calculation of "w" at half levels is kept, even in a FULL-POS task
    !     running with LVERTFE.AND.LVFE_GW
    !! LLVFE=LVERTFE.AND.LVFE_GW
    LLVFE=.FALSE.
    IF (LFPART2) THEN
      IF (LLVFE) THEN
        ! * Compute [Delta(w)] at full levels (in ZVDT0) and "w" at full
        !   levels (in ZWWT0).
        ! ky: VFE code to be checked and validated.
        DO JLEV=1,NFLEVG
          DO JROF=KST,KND
            ZVDT0(JROF,JLEV)=PSVD(JROF,JLEV)*VFE_RDETAH(JLEV)*Z1SRG
          ENDDO
        ENDDO
        CALL VERINT(NPROMA,KST,KND,NFLEVG,ZVDT0(1,1),ZWWT0(1,1),1)
        ZVDT0(KST:KND,1:NFLEVG)=PSVD(KST:KND,1:NFLEVG)
        DO JROF=KST,KND
          ZWWS(JROF)=(ZUS0(JROF)*ZGPHL(JROF,NFLEVG)+&
           & ZVS0(JROF)*ZGPHM(JROF,NFLEVG))*Z1SRG
        ENDDO
        DO JLEV=1,NFLEVG
          DO JROF=KST,KND
            ZWWT0(JROF,JLEV)=ZWWT0(JROF,JLEV)+ZWWS(JROF)
          ENDDO
        ENDDO
      ELSE
        ! * Compute [Delta(w)] at full levels (in ZVDT0) and "w" at half
        !   levels (in ZWWT0).
        ZVDT0(KST:KND,1:NFLEVG)=PSVD(KST:KND,1:NFLEVG)
        DO JROF=KST,KND
          ZWWT0(JROF,NFLEVG)=(ZUS0(JROF)*ZGPHL(JROF,NFLEVG)+&
           & ZVS0(JROF)*ZGPHM(JROF,NFLEVG))*Z1SRG
        ENDDO
        DO JLEV=NFLEVG,1,-1
          DO JROF=KST,KND
            ZWWT0(JROF,JLEV-1)=ZWWT0(JROF,JLEV)+ZVDT0(JROF,JLEV)*Z1SRG
          ENDDO
        ENDDO
      ENDIF
    ELSE
      IF (LLVFE) THEN
        IF (LNH_GEOGW) CALL ABOR1('POS 1.2.7: lnh_geogw not yet coded')
        ! * Compute (gw) at full levels.
        LLGWF=.FALSE.
        LLGDWI=.FALSE.
        CALL GPGW( &
         & NFLEVG,NPROMA,KST,KND,LLGWF,LLGDWI, &
         & ZOROGL,ZOROGM,ZLNPR,ZALPH,ZUS0,ZVS0, &
         & PT,ZDVER0,ZRNHPPI, &
         & ZGWT0,ZGWFT0 &
         & )
        ! * Compute [Delta(w)] at full levels (in ZVDT0) and "w" at full
        !   levels (in ZWWT0).
        ! ky: VFE code to be checked and validated.
        ZWWT0(KST:KND,1:NFLEVG)=ZGWFT0(KST:KND,1:NFLEVG)*Z1SRG
        IF (LVFE_DERIB) THEN
          ZIN(KST:KND,1:NFLEVG)=ZWWT0(KST:KND,1:NFLEVG)
          ! Constant upper BC:
          ZIN(KST:KND,0)=ZIN(KST:KND,1)
          ! Lower BC: (w)_surf = (w_s)
          ZIN(KST:KND,NFLEVG+1)=ZGWT0(KST:KND,NFLEVG)*Z1SRG
          CALL VERDER(NPROMA,KST,KND,NFLEVG+2,ZIN(1,0),ZOU(1,0),.TRUE.)
          ZVDT0(KST:KND,1:NFLEVG)=ZOU(KST:KND,1:NFLEVG)
        ELSE
          CALL VERDER(NPROMA,KST,KND,NFLEVG,ZWWT0(1,1),ZVDT0(1,1))
        ENDIF
        DO JLEV=1,NFLEVG
          DO JROF=KST,KND
            ZVDT0(JROF,JLEV)=ZVDT0(JROF,JLEV)/VFE_RDETAH(JLEV)
          ENDDO
        ENDDO
      ELSE
        ! * Compute (gw) at half levels.
        !   ky: FD discretisation is provisionally kept for this call of GPGW,
        !       even if LVFE_GW, to provide half levels of 'gw' and for
        !       consistency with FD-remaining transformations dver <-> gw.
        LLGWF=.FALSE.
        LLGDWI=.FALSE.
        CALL GPGW( &
         & NFLEVG,NPROMA,KST,KND,LLGWF,LLGDWI, &
         & ZOROGL,ZOROGM,ZLNPR,ZALPH,ZUS0,ZVS0, &
         & PT,ZDVER0,ZRNHPPI, &
         & ZGWT0,ZGWFT0,LDVFE=.FALSE. &
         & )
        ! * Compute [Delta(w)] at full levels (in ZVDT0) and "w" at half
        !   levels (in ZWWT0).
        ZWWT0(KST:KND,0:NFLEVG)=ZGWT0(KST:KND,0:NFLEVG)*Z1SRG
        DO JLEV=1,NFLEVG
          ZVDT0(KST:KND,JLEV)=ZGWT0(KST:KND,JLEV-1)-ZGWT0(KST:KND,JLEV)
        ENDDO
      ENDIF
    ENDIF
  ENDIF
ENDIF

!*      1.2.8 INITIALIZATION OF THE LOCAL PP FIELD ARRAYS

IF(YSD_XAD%NUMFLDS > 0) ALLOCATE(ZEXTRA(NPROMA,0:NFLEVG))
IF(NERA40 > 0) THEN
  ALLOCATE(ZERA40(NPROMA,0:NFLEVG))
  ALLOCATE(ZPPERA(NPROMA,KOPLEV,NERA40))
ENDIF
IF(NGHG > 0 .AND. LL_LRCH4) ALLOCATE(ZPPLRCH4(NPROMA,KOPLEV))
IF(NGHG > 0 .AND. LL_CH4S) ALLOCATE(ZPPCH4S(NPROMA,KOPLEV))
IF(NNOGW > 0) ALLOCATE(ZPPNOGW(NPROMA,KOPLEV,NNOGW))
IF(NUVP > 0) ALLOCATE(ZPPUVP(NPROMA,KOPLEV,NUVP))
IF(NAEROUT > 0) ALLOCATE(ZPPAEROUT(NPROMA,KOPLEV,NAEROUT))

INFUA=COUNT(LL_FUA)
INFSU=COUNT(LL_FSU)
ALLOCATE(ZPPFUA(NPROMA,KOPLEV,INFUA))
ALLOCATE(ZPPFSU(NPROMA,INFSU))

! * GFL:
ZPPQ (:,:)=0.0_JPRB
ZPPL (:,:)=0.0_JPRB
ZPPI (:,:)=0.0_JPRB
ZPPA (:,:)=0.0_JPRB
ZPPO3(:,:)=0.0_JPRB
ZPPDAL (:,:)=0.0_JPRB
ZPPDOM (:,:)=0.0_JPRB
ZPPUAL (:,:)=0.0_JPRB
ZPPUOM (:,:)=0.0_JPRB
ZPPUEN (:,:)=0.0_JPRB
ZPPUNEBH (:,:)=0.0_JPRB
ZPPSN(:,:)=0.0_JPRB
ZPPRR(:,:)=0.0_JPRB
ZPPGR(:,:)=0.0_JPRB
ZPPHL(:,:)=0.0_JPRB
ZPPTKE(:,:)=0.0_JPRB
IF(NGHG > 0 .AND. LL_LRCH4) ZPPLRCH4(:,:)=0.0_JPRB
IF(NGHG > 0 .AND. LL_CH4S) ZPPCH4S(:,:)=0.0_JPRB
IF(NNOGW > 0) ZPPNOGW(:,:,:)=0.0_JPRB
IF(NAEROUT > 0) ZPPAEROUT(:,:,:)=0.0_JPRB
IF(NUVP > 0) ZPPUVP(:,:,:)=0.0_JPRB
ZPPGHG(:,:,:)=0.0_JPRB
ZPPTRAC(:,:,:)=0.0_JPRB
ZPPAERO(:,:,:)=0.0_JPRB
ZPPGRG(:,:,:)=0.0_JPRB
ZPPEXT(:,:,:)=0.0_JPRB

! * GMV and other non-GFL variables:
ZPPPMER(:)  =0.0_JPRB
ZPPCUF1(:)  =0.0_JPRB
ZPPCUF2(:)  =0.0_JPRB
ZPPCUF3(:)  =0.0_JPRB
ZPPCUF4(:)  =0.0_JPRB
ZPPCUF5(:)  =0.0_JPRB
ZFOLDIN(:)  =0.0_JPRB
ZUJET  (:)  =0.0_JPRB
ZVJET  (:)  =0.0_JPRB
ZPJET  (:)  =0.0_JPRB
ZPICAO (:)  =0.0_JPRB
ZTICAO (:)  =0.0_JPRB
ZPPSP  (:)  =0.0_JPRB
ZLNPPSP(:)  =0.0_JPRB
ZMOCO  (:)  =0.0_JPRB
ZPPTWV (:)  =0.0_JPRB
ZPT0B  (:)  =0.0_JPRB
ZHT0B  (:)  =0.0_JPRB
ZPT1B  (:)  =0.0_JPRB
ZHT1B  (:)  =0.0_JPRB
ZHTPW  (:)  =0.0_JPRB
ZPPU (:,:)=0.0_JPRB
ZPPV (:,:)=0.0_JPRB
ZPPT (:,:)=0.0_JPRB
ZPPHI  (:,:)=0.0_JPRB
ZPMG   (:,:)=0.0_JPRB
ZPPHRL (:,:)=0.0_JPRB
ZPPVVE (:,:)=0.0_JPRB
ZPPTHPW(:,:)=0.0_JPRB
ZPPPDEP(:,:)=0.0_JPRB
ZPPVDIV(:,:)=0.0_JPRB
ZPPWVEL(:,:)=0.0_JPRB
ZPPWIND(:,:)=0.0_JPRB
ZPPTETA(:,:)=0.0_JPRB
ZPPEPTH(:,:)=0.0_JPRB
ZPPIBET(:,:)=0.0_JPRB
ZPPSTDF(:,:)=0.0_JPRB
ZPPSHDF(:,:)=0.0_JPRB
ZPPPOTV(:,:)=0.0_JPRB
ZPPUAP (:,:)=0.0_JPRB
ZPPPDIV(:,:)=0.0_JPRB
ZPPPVOR(:,:)=0.0_JPRB
ZPPPABS(:,:)=0.0_JPRB
ZPPFSU (:,:)=0.0_JPRB
ZPPFUA (:,:,:)=0.0_JPRB
ZPPSIMR(:,:)=0.0_JPRB

!*    1.3   COMPUTE MODEL VARIABLES

!*      1.3.1    MODEL LEVEL GEOPOTENTIALS

IF ( CDCONF == 'H' .OR. LLAERO .OR. LL_MOCO .OR. LL_HT0B .OR. LL_PT0B .OR.  &
   & LL_HT1B .OR. LL_PT1B .OR. ( (CDCONF == 'V'.OR.CDCONF == 'M').AND.LL_Z ) )&
   & THEN  
  ! ky: valid for the hydrostatic model only!
  DO JROF = KST , KND
    ZGEOPH(JROF,NFLEVG)=POROG(JROF)
  ENDDO
  CALL GPGEO (NPROMA,KST,KND,NFLEVG,ZGEOPH,ZGEOPF(1,1),&
   & ZTT0(1,1),ZR0,ZLNPR,ZALPH)  
ENDIF

!*      1.3.2   MODEL LEVEL POTENTIAL VORTICITY AND POTENTIAL TEMPERATURE

IF (CDCONF == 'V' .OR. CDCONF == 'T' .OR. LLTDER) THEN
  ! ky: valid for thin layer models.
  CALL GPPVO (NPROMA,IPROMVC,KST,KND,NFLEVG,&
   & ZPRESF,ZRDELP,ZKAP0,PRCORI,&
   & ZVORT0(1,1),ZUT0(1,1),ZVT0(1,1),ZTT0(1,1),&
   & ZTT0M,ZTT0L,ZSPM,ZSPL,ZTOUP(1,1),ZTETA(1,1))  
ENDIF

!*      1.3.3   MODEL LEVEL DEFORMATIONS

IF ( LLUVDER ) THEN
  DO JLEV = 1 , NFLEVG
    DO JROF = KST , KND
      ZSTDF(JROF,JLEV) = ZUT0L(JROF,JLEV)- 0.5_JPRB*ZDIVT0(JROF,JLEV)
      ZSHDF(JROF,JLEV) = ZVT0L(JROF,JLEV)- 0.5_JPRB*ZVORT0(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!*      1.3.4   MODEL LEVEL EQUIVALENT POTENTIAL TEPERATURE

IF (LL_ETH) THEN
  CALL GPEPT (NPROMA,KST,KND,NFLEVG,ZTETA(1,1),ZTT0(1,1),&
   & ZPRESF,ZEPTH(1,1))  
ENDIF

!*      1.3.5   MODEL LEVEL ISOBARIC EQUIVALENT TEMPERATURE

IF (LL_IET) THEN
  CALL GPIET (NPROMA,KST,KND,NFLEVG,ZQT0(1,1),ZTT0(1,1),ZCP0,&
   & ZPRESF,ZIBET(1,1))  
ENDIF

!*      1.3.6    MODEL LEVEL SIMULATED REFLECTIVITIES
IF (LL_SRE) THEN
  LLUNIT=.TRUE.
  CALL GPPRS0D(NPROMA,KST,KND,NFLEVG,ZTT0(1,1),ZQT0(1,1),&
   & ZLT0(1,1),ZRRT0(1,1),ZIT0(1,1),ZSNT0(1,1),ZGRHLT0(1,1),&
   & LLUNIT,ZSIMR(1,1))
ENDIF

!*      1.3.7    MODEL LEVEL thetav
IF (LL_THV) THEN
  CALL PPCVIRT(NPROMA,KST,KND,NFLEVG,ZTETA(1,1),ZQT0(1,1),&
   & ZLT0(1,1),ZRRT0(1,1),ZSNT0(1,1),ZGRHLT0(1,1),ZIT0(1,1),&
   & ZVIRT(1,1))
ENDIF

!*    1.4   PREPARE FOR INTERPOLATIONS.

!*      1.4.1    COMPUTE SURFACE TEMPERATURE AND LAPSE RATE.

IF (LL_MOCO.OR.LL_MG.OR.LL_T.OR.LL_THPW.OR.LL_Z.OR.LL_MSL.OR.CDCONF == 'H') &
   & THEN  
  CALL CTSTAR (NPROMA,KST,KND,ZTT0(1,NLEXTRAP),&
   & ZPRESH(1,NFLEVG),ZPRESF(1,NLEXTRAP),&
   & POROG,ZTSTAR,ZT0)  
ENDIF

!*      1.4.2   COMPUTE POST-PROCESSING LEVELS

LLNIVP=.TRUE.
IF (CDCONF == 'B') THEN
  DO JLEVP = 1, KOPLEV
    DO JROF = KST, KND
      ZPRES(JROF,JLEVP) = PXLEV(JLEVP)
      ZLNPRES(JROF,JLEVP) = LOG(PXLEV(JLEVP))
    ENDDO
  ENDDO
ELSEIF (CDCONF == 'H') THEN
  DO JLEVP = 1 , KOPLEV
    DO JROF = KST , KND
      ZGEOP(JROF,JLEVP) = RG*PXLEV(JLEVP)+POROG(JROF)
    ENDDO
  ENDDO
  CALL FPPS(NPROMA,KST,KND,KOPLEV,POROG,ZGEOP,ZTT0(1,1),ZQT0(1,1),&
   & ZIT0(1,1),ZLT0(1,1),ZRRT0(1,1),ZSNT0(1,1),ZGRHLT0(1,1),&
   & ZPRESH(1,NFLEVG),PTSI,ZPRES,PRESH=ZPRESH,PLNPR=ZLNPR,PALPH=ZALPH)  
  DO JLEVP = 1 , KOPLEV
    DO JROF = KST , KND
      ZLNPRES(JROF,JLEVP) = LOG(ZPRES(JROF,JLEVP))
    ENDDO
  ENDDO
ELSEIF (CDCONF == 'V') THEN
  DO JLEVP = 1 , KOPLEV
    CALL PPLTP (NPROMA,KST,KND,NFLEVG,ZPRESF,&
     & ZTOUP(1,1),PXLEV(JLEVP),.FALSE.,PRCORI,&
     & ZPRES(1,JLEVP),ZLNPRES(1,JLEVP),ZWORK)  
  ENDDO
ELSEIF (CDCONF == 'T') THEN
  DO JLEVP = 1 , KOPLEV
    CALL PPLTETA (NPROMA,KST,KND,NFLEVG,ZPRESF,ZTETA(1,1),&
     & PXLEV(JLEVP),ZPRES(1,JLEVP),ZLNPRES(1,JLEVP))  
  ENDDO
ELSEIF (CDCONF == 'S') THEN
  IF (LFPART2) THEN
    IF (KFPLEV /= KOPLEV) THEN
      CALL ABOR1('ETA LEVELS SELECTION IMPOSSIBLE')
    ENDIF
    LLNIVP=.FALSE.
  ELSE
    IF (LFPOSVER) THEN
      ! cf. PPLETA but without (useless) call to GPPREH+GPXYB+GPPREF.
      DO JLEVP=1,KOPLEV
        IPTRLEV(JLEVP)=INT(PXLEV(JLEVP),JPIM)
        ZPRES(KST:KND,JLEVP)=ZPRESF(KST:KND,IPTRLEV(JLEVP))
        ZPH(KST:KND,JLEVP,1)=ZPRESH(KST:KND,IPTRLEV(JLEVP))
        ZPH(KST:KND,JLEVP,0)=ZPRESH(KST:KND,IPTRLEV(JLEVP)-1)
      ENDDO
    ELSE
      IF (LRWSDLR) THEN
        ! ky: if LRWSDLR, pressure calculations inside PPLETA are currently
        !     not correct, and not easy to adapt.
        !     GPPREH+GPXYB+GPPREF is only valid for "presim".
        !     But the output data ZPRES and ZPH should be "prehyd" pressures. 
        !     Not easy to manage.
        CALL ABOR1(' POS: PPLETA not properly updated for LRWSDLR')
      ENDIF
      CALL PPLETA(NPROMA,KST,KND,KFPLEV,KOPLEV,PXLEV,PVAH,PVBH,&
       & ZPRESH(1,NFLEVG),ZPRES,ZPH(1,1,1),ZPH(1,1,0))  
    ENDIF
    DO JLEVP = 1 , KOPLEV
      DO JROF = KST , KND
        ZLNPRES(JROF,JLEVP) = LOG(ZPRES(JROF,JLEVP))
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      1.4.3   SET UP HELP-ARRAYS FOR VERTICAL INTERPOLATION

IF (CDCONF /= 'M') THEN
  CALL PPINIT (NPROMA,KST,KND,NFLEVG,ZPRESH,ZPRESF,ZPXP,ZPXPD)
ENDIF

!*      1.4.4   FIND MODEL LEVEL UNDER SPECIFIED OUTPUT LEVELS

IF ( CDCONF /= 'M' .AND.&
   & ((CDCONF /= 'H'.AND.CDCONF /= 'S').OR..NOT.LFPART2) ) THEN  
  CALL PPFLEV (NPROMA,KST,KND,NFLEVG,KOPLEV,ZPRES,ZPRESH,&
   & ZPRESF,ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)  
  IF (LL_VD.AND.CDCONF == 'S') THEN
    DO JH=0,1
      CALL PPFLEV(NPROMA,KST,KND,NFLEVG,KOPLEV,&
       & ZPH(1,1,JH),ZPRESH,ZPRESF,&
       & ILEVBH(1,1,1,JH),LLBELOH(1,1,JH),&
       & LLBELSH(1,1,JH),LLBLOWH(1,JH),LLBLESH(1,JH))  
    ENDDO
  ENDIF
ENDIF

!*      1.4.5   COMPUTE ICAO ATMOSPHERE

IF (CDCONF /= 'M'.AND.CDCONF /= 'V'.AND.(LL_T.OR.LL_THPW.OR.LL_Z.OR.LL_MG)) THEN
  CALL PPSTA('PPREF',NPROMA,KST,KND,NFLEVG+1,1,ZPXP(1,0,2),ZPXP(1,0,4),ZSTTF,&
   & ZSTZF)
ENDIF

!     ------------------------------------------------------------------
!*       2.    POST-PROCESSING.
!              ---------------

!*    2.1   TRADITIONAL GFL FIELDS

!     (k.y.: expected reorganisation of GFL treatment in the future in
!     order to treat them globally and not GFL by GFL).

IF ((CDCONF /= 'H'.AND.CDCONF /= 'S').OR..NOT.LFPART2) THEN

!*      2.1.1   Configurations requiring a call to PPQ

  ! Moisture
  IF (LL_Q.OR.LL_THPW) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZQT0,ZPPQ)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPQ(KST:KND,JLEVP)=ZQT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Liquid water
  IF (LL_L.OR.LL_THPW) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZLT0,ZPPL)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPL(KST:KND,JLEVP)=ZLT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Ice
  IF (LL_I.OR.LL_THPW) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZIT0,ZPPI)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPI(KST:KND,JLEVP)=ZIT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Cloud fraction
  IF (LL_CLF) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZAT0,ZPPA)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPA(KST:KND,JLEVP)=ZAT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Ozone
  IF (LL_O3MX) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZO3T0,ZPPO3)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPO3(KST:KND,JLEVP)=ZO3T0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! MAC SCHEME variables
  IF (LL_DAL) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZDALT0,ZPPDAL)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPDAL(KST:KND,JLEVP)=ZDALT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF
  IF (LL_DOM) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZDOMT0,ZPPDOM)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPDOM(KST:KND,JLEVP)=ZDOMT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF
  IF (LL_UAL) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZUALT0,ZPPUAL)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPUAL(KST:KND,JLEVP)=ZUALT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF
  IF (LL_UOM) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZUOMT0,ZPPUOM)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPUOM(KST:KND,JLEVP)=ZUOMT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF
  IF (LL_UEN) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZUENT0,ZPPUEN)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPUEN(KST:KND,JLEVP)=ZUENT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF
  IF (LL_UNEBH) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZUNEBHT0,ZPPUNEBH)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPUNEBH(KST:KND,JLEVP)=ZUNEBHT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Greenhouse gases
  DO JVAR = 1 , NGHG
    IF (LL_GHG(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZGHGT0(1,0,JVAR),ZPPGHG(1,1,JVAR))  
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPGHG(KST:KND,JLEVP,JVAR)=ZGHGT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  ! Tracers 
  DO JVAR = 1 , NTRAC
    IF (LL_TRAC(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZTRACT0(1,0,JVAR),ZPPTRAC(1,1,JVAR))
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPTRAC(KST:KND,JLEVP,JVAR)=ZTRACT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  ! aerosols
  DO JVAR = 1 , NAERO
    IF (LL_AERO(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZAEROT0(1,0,JVAR),ZPPAERO(1,1,JVAR))  
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPAERO(KST:KND,JLEVP,JVAR)=ZAEROT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  ! GRG fields
  DO JVAR = 1 , NGRG
    IF (LL_GRG(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZGRGT0(1,0,JVAR),ZPPGRG(1,1,JVAR))  
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPGRG(KST:KND,JLEVP,JVAR)=ZGRGT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  ! Snow
  IF (LL_SN) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZSNT0,ZPPSN)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPSN(KST:KND,JLEVP)=ZSNT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Rain
  IF (LL_RR) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZRRT0,ZPPRR)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPRR(KST:KND,JLEVP)=ZRRT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Graupels
  IF (LL_GR) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZGRT0,ZPPGR)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPGR(KST:KND,JLEVP)=ZGRT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Hail
  IF (LL_HL) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZHLT0,ZPPHL)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPHL(KST:KND,JLEVP)=ZHLT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! TKE
  IF (LL_TKE) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZTKET0,ZPPTKE)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPTKE(KST:KND,JLEVP)=ZTKET0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! LRCH4 
  IF(NGHG > 0 .AND. LL_LRCH4) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZLRCH4T0,ZPPLRCH4)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPLRCH4(KST:KND,JLEVP)=ZLRCH4T0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! CH4S
  IF(NGHG > 0 .AND. LL_CH4S) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
        & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZCH4ST0,ZPPCH4S)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPCH4S(KST:KND,JLEVP)=ZCH4ST0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

  ! Extra-GFL
  DO JVAR = 1 , NGFL_EXT
    IF (LL_GFL(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,1,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZEXTT0(1,0,JVAR),ZPPEXT(1,1,JVAR))  
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPEXT(KST:KND,JLEVP,JVAR)=ZEXTT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  ! NORO GWD FIELDS
  DO JVAR=1,NNOGW
    IF (LL_NOGW(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,1,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZNOGWT0(1,0,JVAR),ZPPNOGW(1,1,JVAR))
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPNOGW(KST:KND,JLEVP,JVAR)=ZNOGWT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO
 
  ! UV-PROCESSOR OUTPUT FIELDS
  DO JVAR=1,NUVP
    IF (LL_UVP(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,1,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZUVPT0(1,0,JVAR),ZPPUVP(1,1,JVAR))
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPUVP(KST:KND,JLEVP,JVAR)=ZUVPT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO 

  ! AEROSOL OUTPUT FIELDS
  DO JVAR=1,NAEROUT
    IF (LL_AEROUT(JVAR).AND.(KOPLEV>0)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,1,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
         & ZAEROUT0(1,0,JVAR),ZPPAEROUT(1,1,JVAR))
      ELSE
        DO JLEVP = 1 , KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPAEROUT(KST:KND,JLEVP,JVAR)=ZAEROUT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

ENDIF

! ERA40 fields
IF (LL_ERA40) THEN
  IPTR=1
  DO JFLD=1,NERA40
    DO JLEV = 1 , NFLEVG
      ZERA40(KST:KND,JLEV)=PERA40(KST:KND,JLEV,JFLD)
    ENDDO
    CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
      & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZERA40,ZPPERA(1,1,IPTR))
    IPTR=IPTR+1
  ENDDO
ENDIF

!*    2.2   TRADITIONAL NON-GFL FIELDS

IF ((CDCONF /= 'H'.AND.CDCONF /= 'S').OR..NOT.LFPART2) THEN

!*      2.2.1   WINDS :

  IF (LL_PPUV.OR.LL_WND) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPUV (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,ILEVB,LLBELO,LLBLOW,&
       & ZLNPRES,ZPXP,ZPXPD,ZUT0,ZVT0,ZPPU,ZPPV)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPU(KST:KND,JLEVP)=ZUT0(KST:KND,ILEVM)
        ZPPV(KST:KND,JLEVP)=ZVT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      2.2.2    TEMPERATURE :

  IF (LL_T.OR.LL_THPW) THEN
    IF (CDCONF == 'V') THEN
      ! Do not use PPT because it contains an implicit definition of
      ! the tropopause
      ISLCT=2
      CALL PP2DINT(NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZTT0,ZPPT)  
    ELSEIF (CDCONF /= 'M') THEN
      LL=.TRUE.
      CALL PPT (NPROMA,KST,KND,NFLEVG,KOPLEV,1,ILEVB,&
       & ZPRES,ZLNPRES,LLBELO,LLBELS,LLBLOW,LLBLES,LL,&
       & POROG,ZPXP,ZPXPD,ZTSTAR,ZT0,ZR0,ZTT0,ZPPT,ZSTTF)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPT(KST:KND,JLEVP)=ZTT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      2.2.3    UPPER AIR PRESSURE :

  IF (LL_P.AND.CDCONF /= 'B') THEN
    IF (CDCONF /= 'M') THEN
      ZPPUAP(KST:KND,1:KOPLEV)=ZPRES(KST:KND,1:KOPLEV)
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPUAP(KST:KND,JLEVP)=ZPRESF(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      2.2.4    SURFACE PRESSURE :

  IF ( LL_SP.OR.LL_LNSP ) THEN
    ZPPSP(KST:KND)=ZPRESH(KST:KND,NFLEVG)
  ENDIF

!*      2.2.5   PRESSURE DEPARTURE :

  IF (LL_PD) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZPDT0,ZPPPDEP)  
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPPDEP(KST:KND,JLEVP)=ZPDT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      2.2.6   TRUE VERTICAL VELOCITY W :

  IF (LL_VW) THEN
    ISLCT=1
    IF (CDCONF /= 'M') THEN
      CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,&
       & ILOLEV,IHILEV,ILEVB,ISLCT,LLBELS,LLBLES,&
       & ZPRES,ZPXP,ZPXPD,ZWWT0,ZPPWVEL)  
    ELSE
      !      ****** this is an internal configuration *********
      !      ****** ZPPWVEL is filled with upper half level values ******* 
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPWVEL(KST:KND,JLEVP)=ZWWT0(KST:KND,ILEVM-1)
      ENDDO
    ENDIF
  ENDIF

!*      2.2.7   VERTICAL DIVERGENCE : 

  IF (LL_VD) THEN
    IF (CDCONF == 'S') THEN
      ISLCT=1
      DO JH=0,1
        CALL PP2DINT(NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
         & IHILEV,ILEVBH(1,1,1,JH),ISLCT,&
         & LLBELSH(1,1,JH),LLBLESH(1,JH),ZPH(1,1,JH),&
         & ZPXP,ZPXPD,ZWWT0,ZPPWVELH(1,1,JH))  
      ENDDO
      DO JLEVP=1,KOPLEV
        DO JROF = KST, KND
          ZPPVDIV(JROF,JLEVP)=-RG*(ZPPWVELH(JROF,JLEVP,1)&
           & -ZPPWVELH(JROF,JLEVP,0))  
        ENDDO
      ENDDO
    ELSEIF (CDCONF /= 'M') THEN
      CALL ABOR1('VERT. DIVERGENCE IS COMPUTABLE ONLY ON ETA LEVELS')
    ELSE
      DO JLEVP = 1 , KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPVDIV(KST:KND,JLEVP)=ZVDT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

ENDIF

!*      2.2.8   GEOPOTENTIAL & MONTGOMERY POTENTIAL

IF (LL_Z .OR. LL_MG) THEN
  IF (CDCONF == 'B'.OR.CDCONF == 'T'.OR.CDCONF == 'S') THEN
    LLEXTR=.TRUE.
    CALL PPGEOP (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,ILEVB,ZPRES,ZLNPRES,&
     & LLBELO,LLBELS,LLBLOW,LLBLES,LLEXTR,POROG,&
     & ZPXP,ZPXPD,ZTSTAR,ZR0,ZLNPR,ZALPH,ZTT0,ZPPHI,ZSTTF)
  ELSEIF (CDCONF == 'V') THEN
    ! Do not use PPGEOP because it contains an implicit definition of
    ! the tropopause
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZGEOPF,ZPPHI)  
  ELSEIF (CDCONF == 'M') THEN
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPHI(KST:KND,JLEVP)=ZGEOPF(KST:KND,ILEVM)
    ENDDO
  ENDIF
  IF(LL_MG) THEN
    DO JLEVP = 1 , KOPLEV
      DO JL = KST , KND
        ZPMG(JL,JLEVP)=ZPPHI(JL,JLEVP)+&
         & RCPD*PXLEV(JLEVP)*((ZPRES(JL,JLEVP)/RATM)**RKAPPA)  
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      2.2.9   RELATIVE HUMIDITY :

IF (LL_HU) THEN
  IF (LECMWF) THEN
    ZRHMAX=2.0_JPRB
  ELSE
    ZRHMAX=1.0_JPRB
  ENDIF  
  ZRHMIN=0.0_JPRB
  IF (CDCONF /= 'M') THEN
    CALL PPRH(.FALSE.,NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & ILEVB,ZPRES,LLBELO,LLBLOW,ZRHMAX,ZRHMIN,ZPXP,ZPXPD,ZQT0,ZTT0,ZPPHRL)  
  ELSE
    CALL GPRH (.FALSE.,NPROMA,KST,KND,NFLEVG,ZRHMAX,ZRHMIN,&
     & ZQT0(1,1),ZTT0(1,1),ZPRESF,ZES,ZRHF)  
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPHRL(KST:KND,JLEVP)=ZRHF(KST:KND,ILEVM)
    ENDDO
  ENDIF
  IF (LFPRH100) THEN
    ! convert relative humidity field to percent
    DO JLEVP = 1 , KOPLEV
      DO JL = KST , KND
        ZPPHRL(JL,JLEVP)=ZPPHRL(JL,JLEVP)*100.0_JPRB
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      2.2.10   VERTICAL VELOCITY :

IF (LL_VV) THEN
  IF (CDCONF /= 'M') THEN
    CALL PPVVEL (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & ILEVB,ZPRES,LLBELO,LLBLOW,&
     & ZPXP,ZPXPD,ZDELP,ZRDELP,ZLNPR,ZALPH,ZRTGR,&
     & ZUT0,ZVT0,ZDIVT0(1,1),ZSPL,ZSPM,PEV,&
     & ZPPVVE,.FALSE.,ZRPRESF,.FALSE.)
  ELSE
    ! Remark N Wedi and K Yessad: for the case (LL_VV.AND.CDCONF=='M'), this
    ! call to GPCTY is useless because it has already been done in part 1.2.5
    ! CALL GPCTY (LLVERCOR,NPROMA,IPROMVC,KST,KND,NFLEVG,VDELB,VBH,ZUT0(1,1),&
    !  & ZVT0(1,1),ZDIVT0(1,1),PEV,.FALSE.,ZDELP,ZRDELP,ZLNPR,ZALPH,&
    !  & ZRTGR,ZSPL,ZSPM,&
    !  & ZVCRSSA0F,ZVCRSSA0H,ZVCASRS0F,ZVCASRS0H,ZVCRSSA0FL,ZVCRSSA0FM,&
    !  & ZEVEL,ZVVEL,ZPSDIV,ZPSDVBC,ZDIVDP,ZRPRESF)
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      DO JL = KST , KND
        ZPPVVE(JL,JLEVP)=ZPXP(JL,ILEVM,2)*ZVVEL(JL,ILEVM)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      2.2.11   ETADOT

IF (LL_ETAD) THEN
  IF (CDCONF /= 'M') THEN
    CALL PPVVEL (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & ILEVB,ZPRES,LLBELO,LLBLOW,&
     & ZPXP,ZPXPD,ZDELP,ZRDELP,ZLNPR,ZALPH,ZRTGR,&
     & ZUT0,ZVT0,ZDIVT0(1,1),ZSPL,ZSPM,PEV,&
     & ZPPEVEL,.FALSE.,ZRPRESF,.TRUE.)  
  ELSE
    ! Remark N Wedi and K Yessad: as above GPCTY obsolete
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPEVEL(KST:KND,JLEVP)=ZETADOT(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.2.12   MOISTURE CONVERGENCE :

IF (LL_MOCO) THEN
!  CALL MOCONI (KST,KND,ZPRESH(1,NFLEVG),ZPRESF,ZDELP,ZUT0(1,1),ZVT0(1,1),&
!   &ZDIVT0(1,1),ZQT0(1,1),ZQT0L,ZQT0M,ZGEOPF(1,1),ZGEOPH,ZR0,ZTT0(1,1),&
!   &ZTSTAR,PGCLP,ZMOCO)
ENDIF

!*      2.2.13   MSL PRESSURE :

IF (LL_MSL) THEN
  CALL PPPMER (NPROMA,KST,KND,ZPRESH(1,NFLEVG),POROG,ZTSTAR,ZT0,ZPPPMER)
ENDIF

!*      2.2.14  Filtred ln(Ps) for monitoring Couplinf Update Frequency :

IF (LL_CUF1) ZPPCUF1(KST:KND)=PRMCUFGP1(KST:KND)
IF (LL_CUF2) ZPPCUF2(KST:KND)=PRMCUFGP2(KST:KND)
IF (LL_CUF3) ZPPCUF3(KST:KND)=PRMCUFGP3(KST:KND)
IF (LL_CUF4) ZPPCUF4(KST:KND)=PRMCUFGP4(KST:KND)
IF (LL_CUF5) ZPPCUF5(KST:KND)=PRMCUFGP5(KST:KND)

!*    2.3   TRADITIONAL FIELDS FOR CONFIGURATIONS CALLING 'APACHE'

IF ( LFPART2 .AND. (CDCONF == 'H'.OR.CDCONF == 'S') ) THEN

  ! Individual GFL and other variables
  LLLGFL=ANY(LL_GFL)
  IF (LL_PPUV.OR.LL_WND.OR.LL_T.OR.LL_THPW.OR.&
     & LL_PD.OR.LL_VD.OR.LL_VW.OR.LL_Q.OR.LL_L.OR.LL_I.OR.&
     & LL_CLF.OR.LL_O3MX.OR.LL_P.OR.LLLGFL.OR.&
     & LL_SN.OR.LL_RR.OR.LL_GR.OR.LL_HL.OR.LL_TKE) THEN  
    ZMDTDZSG = -RDTDZ1/RG
    LLWAT=LL_L.OR.LL_THPW
    LLICE=LL_I.OR.LL_THPW
    CALL APACHE(LLNIVP,NPROMA,KST,KND,NFLEVG,KOPLEV,ZTT0,&
     & ZQT0,ZUT0,ZVT0,ZPRESH(1,NFLEVG),PXORO,VAH,VBH,&
     & ZLT0,ZIT0,ZAT0,ZO3T0,ZPDT0,ZWWT0,&
     & ZSNT0,ZRRT0,ZGRT0,ZHLT0,ZTKET0,&
     & LL_PD,LL_VD,LL_VW,&
     & LLWAT,LLICE,LL_CLF,LL_O3MX,&
     & LL_SN,LL_RR,LL_GR,LL_HL,LL_TKE,&
     & ZPPT,ZPPQ,ZPPU,ZPPV,ZPPSP,POROG,PVAH,&
     & PVBH,ZPPL,ZPPI,ZPPA,ZPPO3,ZPPSN,&
     & ZPPRR,ZPPGR,ZPPHL,ZPPTKE,&
     & ZPPPDEP,ZPPWVELH,&
     & ZPPWVEL,ZPPHI,ZPPHRL,ZDUMARR,.FALSE.,ZPPUAP,ZPH,&
     & PTSI,PXTSO,FPBL,RATM,LFPQ,ZMDTDZSG)  
    !    APACHE returns ZPPSP=surface pressure (NOT log surface pressure)
    !    (of course, like POS itself)
    IF (LL_VD) THEN
      DO JLEVP=1,KOPLEV
        DO JROF = KST, KND
          ZPPVDIV(JROF,JLEVP)=-RG*(ZPPWVELH(JROF,JLEVP,1)&
           & -ZPPWVELH(JROF,JLEVP,0))  
        ENDDO
      ENDDO
    ENDIF
  ENDIF

  ! Greenhouse Gases (work around)
  DO JVAR=1,NGHG
    LL_VD=.FALSE.
    LL_VW=.FALSE.
    LLWAT=.FALSE.
    LLICE=.FALSE.
    LL_CLF=.FALSE.
    LL_O3MX=.FALSE.
    LL_SN=.FALSE.
    LL_RR=.FALSE.
    LL_GR=.FALSE.
    LL_HL=.FALSE.
    LL_TKE=.FALSE.
    LFPQ=.TRUE.
    IF (LL_GHG(JVAR)) THEN
      CALL APACHE(LLNIVP,NPROMA,KST,KND,NFLEVG,KOPLEV,ZTT0,&
       & ZQT0,ZUT0,ZVT0,ZPRESH(1,NFLEVG),PXORO,VAH,VBH,&
       & ZLT0,ZIT0,ZAT0,ZO3T0,ZGHGT0(1,0,JVAR),ZWWT0,&
       & ZSNT0,ZRRT0,ZGRT0,ZHLT0,ZTKET0,&
       & LL_GHG(JVAR),LL_VD,LL_VW,&
       & LLWAT,LLICE,LL_CLF,LL_O3MX,&
       & LL_SN,LL_RR,LL_GR,LL_HL,LL_TKE,&
       & ZPPT,ZPPQ,ZPPU,ZPPV,ZPPSP,POROG,PVAH,&
       & PVBH,ZPPL,ZPPI,ZPPA,ZPPO3,ZPPSN,&
       & ZPPRR,ZPPGR,ZPPHL,ZPPTKE,&
       & ZPPGHG(1,1,JVAR),ZPPWVELH,&
       & ZPPWVEL,ZPPHI,ZPPHRL,ZDUMARR,.FALSE.,ZPPUAP,ZPH,&
       & PTSI,PXTSO,FPBL,RATM,LFPQ,ZMDTDZSG)
    ENDIF 
  ENDDO

  ! Diagnostic Tracers (work around)
  DO JVAR=1,NTRAC
    LL_VD=.FALSE.
    LL_VW=.FALSE.
    LLWAT=.FALSE.
    LLICE=.FALSE.
    LL_CLF=.FALSE.
    LL_O3MX=.FALSE.
    LL_SN=.FALSE.
    LL_RR=.FALSE.
    LL_GR=.FALSE.
    LL_HL=.FALSE.
    LL_TKE=.FALSE.
    LFPQ=.TRUE.
    IF (LL_TRAC(JVAR)) THEN
      CALL APACHE(LLNIVP,NPROMA,KST,KND,NFLEVG,KOPLEV,ZTT0,&
       & ZQT0,ZUT0,ZVT0,ZPRESH(1,NFLEVG),PXORO,VAH,VBH,&
       & ZLT0,ZIT0,ZAT0,ZO3T0,ZTRACT0(1,0,JVAR),ZWWT0,&
       & ZSNT0,ZRRT0,ZGRT0,ZHLT0,ZTKET0,&
       & LL_TRAC(JVAR),LL_VD,LL_VW,&
       & LLWAT,LLICE,LL_CLF,LL_O3MX,&
       & LL_SN,LL_RR,LL_GR,LL_HL,LL_TKE,&
       & ZPPT,ZPPQ,ZPPU,ZPPV,ZPPSP,POROG,PVAH,&
       & PVBH,ZPPL,ZPPI,ZPPA,ZPPO3,ZPPSN,&
       & ZPPRR,ZPPGR,ZPPHL,ZPPTKE,&
       & ZPPTRAC(1,1,JVAR),ZPPWVELH,&
       & ZPPWVEL,ZPPHI,ZPPHRL,ZDUMARR,.FALSE.,ZPPUAP,ZPH,&
       & PTSI,PXTSO,FPBL,RATM,LFPQ,ZMDTDZSG)
    ENDIF
  ENDDO

  ! Reactive Gases (work around)
  DO JVAR=1,NGRG
    LL_VD=.FALSE.
    LL_VW=.FALSE.
    LLWAT=.FALSE.
    LLICE=.FALSE.
    LL_CLF=.FALSE.
    LL_O3MX=.FALSE.
    LL_SN=.FALSE.
    LL_RR=.FALSE.
    LL_GR=.FALSE.
    LL_HL=.FALSE.
    LL_TKE=.FALSE.
    LFPQ=.TRUE.
    IF (LL_GRG(JVAR)) THEN
      CALL APACHE(LLNIVP,NPROMA,KST,KND,NFLEVG,KOPLEV,ZTT0,&
       & ZQT0,ZUT0,ZVT0,ZPRESH(1,NFLEVG),PXORO,VAH,VBH,&
       & ZLT0,ZIT0,ZAT0,ZO3T0,ZGRGT0(1,0,JVAR),ZWWT0,&
       & ZSNT0,ZRRT0,ZGRT0,ZHLT0,ZTKET0,&
       & LL_GRG(JVAR),LL_VD,LL_VW,&
       & LLWAT,LLICE,LL_CLF,LL_O3MX,&
       & LL_SN,LL_RR,LL_GR,LL_HL,LL_TKE,&
       & ZPPT,ZPPQ,ZPPU,ZPPV,ZPPSP,POROG,PVAH,&
       & PVBH,ZPPL,ZPPI,ZPPA,ZPPO3,ZPPSN,&
       & ZPPRR,ZPPGR,ZPPHL,ZPPTKE,&
       & ZPPGRG(1,1,JVAR),ZPPWVELH,&
       & ZPPWVEL,ZPPHI,ZPPHRL,ZDUMARR,.FALSE.,ZPPUAP,ZPH,&
       & PTSI,PXTSO,FPBL,RATM,LFPQ,ZMDTDZSG)
    ENDIF
  ENDDO

  ! Aerosol (work around)
  DO JVAR=1,NAERO
    LL_VD=.FALSE.
    LL_VW=.FALSE.
    LLWAT=.FALSE.
    LLICE=.FALSE.
    LL_CLF=.FALSE.
    LL_O3MX=.FALSE.
    LL_SN=.FALSE.
    LL_RR=.FALSE.
    LL_GR=.FALSE.
    LL_HL=.FALSE.
    LL_TKE=.FALSE.
    LFPQ=.TRUE.
    IF (LL_AERO(JVAR)) THEN
      CALL APACHE(LLNIVP,NPROMA,KST,KND,NFLEVG,KOPLEV,ZTT0,&
       & ZQT0,ZUT0,ZVT0,ZPRESH(1,NFLEVG),PXORO,VAH,VBH,&
       & ZLT0,ZIT0,ZAT0,ZO3T0,ZAEROT0(1,0,JVAR),ZWWT0,&
       & ZSNT0,ZRRT0,ZGRT0,ZHLT0,ZTKET0,&
       & LL_AERO(JVAR),LL_VD,LL_VW,&
       & LLWAT,LLICE,LL_CLF,LL_O3MX,&
       & LL_SN,LL_RR,LL_GR,LL_HL,LL_TKE,&
       & ZPPT,ZPPQ,ZPPU,ZPPV,ZPPSP,POROG,PVAH,&
       & PVBH,ZPPL,ZPPI,ZPPA,ZPPO3,ZPPSN,&
       & ZPPRR,ZPPGR,ZPPHL,ZPPTKE,&
       & ZPPAERO(1,1,JVAR),ZPPWVELH,&
       & ZPPWVEL,ZPPHI,ZPPHRL,ZDUMARR,.FALSE.,ZPPUAP,ZPH,&
       & PTSI,PXTSO,FPBL,RATM,LFPQ,ZMDTDZSG)
    ENDIF
  ENDDO

ENDIF

!*    2.4   MODERN DYNAMICAL METEOROLOGY FIELDS

!*      2.4.1   POTENTIAL VORTICITY

IF (LL_PV.AND.CDCONF /= 'V') THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZTOUP,ZPPPOTV)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPPOTV(KST:KND,JLEVP)=ZTOUP(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.2   POTENTIAL TEMPERATURE

IF (LL_TH.AND.CDCONF /= 'T') THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZTETA,ZPPTETA)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPTETA(KST:KND,JLEVP)=ZTETA(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.3   EQUIVALENT POTENTIAL TEMPERATURE

IF (LL_ETH) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZEPTH,ZPPEPTH)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPEPTH(KST:KND,JLEVP)=ZEPTH(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.4   STRETCHING DEFORMATION

IF (LL_STD) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZSTDF,ZPPSTDF)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPSTDF(KST:KND,JLEVP)=ZSTDF(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.5   SHEARING DEFORMATION

IF (LL_SHD) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZSHDF,ZPPSHDF)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPSHDF(KST:KND,JLEVP)=ZSHDF(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.6   TOTAL WATER VAPOUR

IF (LL_TWV) THEN
  Z1SRG=1.0_JPRB/RG
  DO JLEVP=1,NFLEVG
    DO JL  = KST, KND
      ZPPTWV(JL)=ZPPTWV(JL)+ZQT0(JL,JLEVP)*ZDELP(JL,JLEVP)*Z1SRG
    ENDDO
  ENDDO
ENDIF

!*      2.4.7   TROPOPAUSE FOLDING INDICATOR

IF (LL_FOL) THEN
  ZTROPO=2.E-6_JPRB ! THIS IS ANOTHER INTERNAL DEFINITION OF THE TROPOPAUSE !
  CALL PPLTP (NPROMA,KST,KND,NFLEVG,ZPRESF,ZTOUP(1,1),ZTROPO,.TRUE.,PRCORI,&
   & ZWORK,ZLNWORK,ZFOLDIN)  
ENDIF

!*      2.4.8   ISOBARIC EQUIVALENT TEMPERATURE

IF (LL_IET) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZIBET,ZPPIBET)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPIBET(KST:KND,JLEVP)=ZIBET(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.9   SIMULATED REFLECTIVITIES

IF (LL_SRE) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZSIMR,ZPPSIMR)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPSIMR(KST:KND,JLEVP)=ZSIMR(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.4.10  THETAv

IF (LL_THV) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZVIRT,ZPPVIRT)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPVIRT(KST:KND,JLEVP)=ZVIRT(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*    2.5   OTHER FIELDS

!*      2.5.1   MOIST IRREVERSIBLE ADIABATIC POTENTIAL TEMPERATURE

IF (LL_THPW) THEN
  I=TFP_THPW%ICOD
  DO JL = 1, QFPTYPE%ILEV(I)
    ILPT=QFPTYPE%ILVP(JL,I)
    IF (CDCONF /= 'M') THEN
      CALL PPTHPW(KST,KND,NPROMA,ZPRES(1,ILPT),ZPPT(1,ILPT),ZPPQ(1,ILPT),&
       & ZPPL(1,ILPT),ZPPI(1,ILPT),ZPPTHPW(1,ILPT))  
    ELSE
      ILEVM=INT(PXLEV(ILPT))
      CALL PPTHPW (KST,KND,NPROMA,ZPRESF(1,ILEVM),ZPPT(1,ILPT),&
       & ZPPQ(1,ILPT),ZPPL(1,ILPT),ZPPI(1,ILPT),ZPPTHPW(1,ILPT))  
    ENDIF
  ENDDO
ENDIF

!*      2.5.2   WIND VELOCITY

IF (LL_WND) THEN
  DO JLEVP = 1 , KOPLEV
    DO JROF  = KST , KND
      ZPPWIND(JROF,JLEVP)=SQRT( &
       & ZPPU(JROF,JLEVP)*ZPPU(JROF,JLEVP)+ZPPV(JROF,JLEVP)*ZPPV(JROF,JLEVP) )
    ENDDO
  ENDDO
ENDIF

!*      2.5.3   DIVERGENCE

IF (LL_DIV) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZDIVT0,ZPPPDIV)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPPDIV(KST:KND,JLEVP)=ZDIVT0(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      2.5.4   VORTICITY

IF (LL_VOR.OR.LL_ABS) THEN
  IF (CDCONF /= 'M') THEN
    ISLCT=2
    CALL PP2DINT (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZVORT0,ZPPPVOR)  
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPPVOR(KST:KND,JLEVP)=ZVORT0(KST:KND,ILEVM)
    ENDDO
  ENDIF
  IF (LL_ABS) THEN
    IF (CFPFMT == 'MODEL') THEN
      DO JLEVP = 1 , KOPLEV
        ZPPPABS(KST:KND,JLEVP)=ZPPPVOR(KST:KND,JLEVP)+PRCORI(KST:KND)
      ENDDO
    ELSE
      ! Coriolis parameter will be added during horizontal interpolations
      DO JLEVP = 1 , KOPLEV
        ZPPPABS(KST:KND,JLEVP)=ZPPPVOR(KST:KND,JLEVP)
      ENDDO
    ENDIF
  ENDIF
ENDIF

!*      2.5.5   ICAO FIELDS

IF (LLAERO) THEN
  IPOL2=4
  CALL POAERO(ZGEOPF(1,1),ZTT0(1,1),ZUT0(1,1),ZVT0(1,1),&
   & ZPICAO,ZTICAO,ZUJET,ZVJET,ZPJET,ZPRESF,NPROMA,KST,KND,NFLEVG,IPOL2)
ENDIF

!*      2.5.6 ISO-TEMPERATURE FIELDS

IF (LL_PT0B .OR. LL_HT0B) THEN
  LLTOP=.FALSE.
  ZXTEMP=RTT
  CALL PPLTEMP(NPROMA,KST,KND,NFLEVG,ZGEOPF(1,1),ZTT0(1,1),ZXTEMP,LLTOP,ZHT0B)
  IF (LL_PT0B) THEN
    CALL FPPS(NPROMA,KST,KND,1,POROG,ZHT0B,ZTT0(1,1),ZQT0(1,1),ZIT0(1,1), &
     & ZLT0(1,1),ZRRT0(1,1),ZSNT0(1,1),ZGRHLT0(1,1),ZPRESH(1,NFLEVG),&
     & PTSI,ZPT0B,PRESH=ZPRESH,PLNPR=ZLNPR,PALPH=ZALPH)  
  ENDIF
  IF (LL_HT0B) THEN
    ! Convert from geopotentials to meters 
    Z1SRG=1.0_JPRB/RG
    ZHT0B(KST:KND)=ZHT0B(KST:KND)*Z1SRG
  ENDIF
ENDIF
IF (LL_PT1B .OR. LL_HT1B) THEN
  LLTOP=.FALSE.
  ZXTEMP=RTT-10._JPRB
  CALL PPLTEMP(NPROMA,KST,KND,NFLEVG,ZGEOPF(1,1),ZTT0(1,1),ZXTEMP,LLTOP,ZHT1B)
  IF (LL_PT1B) THEN
    CALL FPPS(NPROMA,KST,KND,1,POROG,ZHT1B,ZTT0(1,1),ZQT0(1,1),ZIT0(1,1), &
     & ZLT0(1,1),ZRRT0(1,1),ZSNT0(1,1),ZGRHLT0(1,1),ZPRESH(1,NFLEVG),&
     & PTSI,ZPT1B,PRESH=ZPRESH,PLNPR=ZLNPR,PALPH=ZALPH)  
  ENDIF
  IF (LL_HT1B) THEN
    ! Convert from geopotentials to meters
    Z1SRG=1.0_JPRB/RG
    ZHT1B(KST:KND)=ZHT1B(KST:KND)*Z1SRG
  ENDIF
ENDIF
IF (LL_HTPW) THEN
  DO JL=1,NFLEVG
    CALL PPWETPOINT(KST,KND,NPROMA,ZPRESF(1,JL),ZTT0(1,JL),ZQT0(1,JL),&
     & ZLT0(1,JL),ZIT0(1,JL),ZTPW0(1,JL))  
  ENDDO
  LLTOP=.FALSE.
  ZXTEMP=RTT
  CALL PPLTEMP(NPROMA,KST,KND,NFLEVG,ZGEOPF(1,1),ZTPW0,ZXTEMP,LLTOP,ZHTPW)
  ! Convert from geopotentials to meters
  Z1SRG=1.0_JPRB/RG
  ZHTPW(KST:KND)=ZHTPW(KST:KND)*Z1SRG
ENDIF

!*    2.6   FREE-USE FIELDS

IF (ANY(LL_FUA)) THEN

  ! provide vertical post-processing here in pressure with result ZPPFUA
  ! in general PPQ is used for interpolation in the vertical in pressure

  IF (YSD_XAD%NUMFLDS > 0) THEN
    IF(YSD_XAD%NLEVS /= NFLEVG) THEN
      CALL ABOR1 ('POS: YSD_XAD%NLEVS /= NFLEVG not allowed with LFPOS=.T.')
    ENDIF
    IF(YSD_XAD%NUMFLDS > JPOSVX2) THEN
      CALL ABOR1 ('POS: JPOSVX2 too small for YSD_XAD%NUMFLDS')
    ENDIF
  ENDIF
  IPTR=1
  DO JVAR=1,JPOSVX2
    IF (LL_FUA(JVAR)) THEN
      DO JVEXTR=1,YSD_XAD%NUMFLDS
        IF (TFP_FUA(JVAR)%IGRIB == YSD_XA%YXA(JVEXTR)%IGRBCODE(1)) THEN
          DO JLEV = 1 , YSD_XAD%NLEVS
            ZEXTRA(KST:KND,JLEV) = PEXTRA(KST:KND,JLEV,JVEXTR)
          ENDDO
          CALL PPQ (NPROMA,KST,KND,NFLEVG,KOPLEV,ILOLEV,&
           & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZEXTRA,ZPPFUA(1,1,IPTR))  
          IPTR=IPTR+1
          EXIT
        ENDIF
      ENDDO
    ENDIF
  ENDDO

ENDIF

IF (ANY(LL_FSU)) THEN
ENDIF

!     ------------------------------------------------------------------

!*       3.     STORE POST-PROCESSED FIELDS.
!               ----------------------------

!*    3.1   GFL FIELDS
!     (k.y.: expected reorganisation of GFL treatment in the future in
!     order to treat them globally and not GFL by GFL).

I=TFP_Q%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPQ,PGPP,PAUX)

I=TFP_L%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPL,PGPP,PAUX)

I=TFP_I%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPI,PGPP,PAUX)

I=TFP_CLF%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPA,PGPP,PAUX)

I=TFP_O3MX%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPO3,PGPP,PAUX)

I=TFP_DAL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPDAL,PGPP,PAUX)

I=TFP_DOM%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPDOM,PGPP,PAUX)

I=TFP_UAL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPUAL,PGPP,PAUX)

I=TFP_UOM%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPUOM,PGPP,PAUX)

I=TFP_UEN%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPUEN,PGPP,PAUX)

I=TFP_UNEBH%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPUNEBH,PGPP,PAUX)

I=TFP_SN%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPSN,PGPP,PAUX)

I=TFP_RR%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPRR,PGPP,PAUX)

I=TFP_GR%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPGR,PGPP,PAUX)

I=TFP_HL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPHL,PGPP,PAUX)

I=TFP_TKE%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPTKE,PGPP,PAUX)

IF(NGHG > 0 .AND. LL_LRCH4) THEN
  I=TFP_LRCH4%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPLRCH4,PGPP,PAUX)
ENDIF

IF(NGHG > 0 .AND. LL_CH4S) THEN
  I=TFP_CH4S%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPCH4S,PGPP,PAUX)
ENDIF

DO JVAR = 1 , NGHG
  I=TFP_GHG(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPGHG(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NTRAC
  I=TFP_TRAC(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPTRAC(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NAERO
  I=TFP_AERO(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPAERO(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NGRG
  I=TFP_GRG(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPGRG(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NGFL_EXT
  I=TFP_GFL(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPEXT(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NNOGW
  I=TFP_NOGW(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPNOGW(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NAEROUT
  I=TFP_AEROUT(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPAEROUT(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1 , NUVP
  I=TFP_UVP(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPUVP(1,1,JVAR),PGPP,PAUX)
ENDDO

DO JVAR = 1,NERA40
  I=TFP_ERA40(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPERA(1,1,JVAR),PGPP,PAUX)
ENDDO

!*    3.2   OTHER FIELDS

I=TFP_U%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,KOPLEV
    ZPPIN(KST:KND,JL)=ZPPU(KST:KND,JL)/PGM(KST:KND)
  ENDDO
  CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_V%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,KOPLEV
    ZPPIN(KST:KND,JL)=ZPPV(KST:KND,JL)/PGM(KST:KND)
  ENDDO
  CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_T%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPT,PGPP,PAUX)

I=TFP_Z%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPHI,PGPP,PAUX)

I=TFP_HU%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPHRL,PGPP,PAUX)

I=TFP_VV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPVVE,PGPP,PAUX)

I=TFP_ETAD%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPEVEL,PGPP,PAUX)

I=TFP_MSL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPPMER,PGPP,PAUX)

I=TFP_CUF1%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPCUF1,PGPP,PAUX)
I=TFP_CUF2%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPCUF2,PGPP,PAUX)
I=TFP_CUF3%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPCUF3,PGPP,PAUX)
I=TFP_CUF4%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPCUF4,PGPP,PAUX)
I=TFP_CUF5%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPCUF5,PGPP,PAUX)

I=TFP_PV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPPOTV,PGPP,PAUX)

I=TFP_TH%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPTETA,PGPP,PAUX)

I=TFP_ETH%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPEPTH,PGPP,PAUX)

I=TFP_IET%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIBET,PGPP,PAUX)

I=TFP_SRE%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPSIMR,PGPP,PAUX)

I=TFP_THV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPVIRT,PGPP,PAUX)

I=TFP_STD%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,KOPLEV
    ZPPIN(KST:KND,JL)=ZPPSTDF(KST:KND,JL)/ZGMAPP(KST:KND)
  ENDDO
  CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_SHD%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,KOPLEV
    ZPPIN(KST:KND,JL)=ZPPSHDF(KST:KND,JL)/ZGMAPP(KST:KND)
  ENDDO
  CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_TWV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPTWV,PGPP,PAUX)

I=TFP_FOL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZFOLDIN,PGPP,PAUX)

I=TFP_THPW%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPTHPW,PGPP,PAUX)

I=TFP_WND%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPWIND,PGPP,PAUX)

I=TFP_PSI%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,QFPTYPE%ILEV(I)
    ILPT=QFPTYPE%ILVP(JL,I)
    ILOC=QFPTYPE%IGT1(I,1)+JL-1
    PGPP(KST:KND,ILOC) = ZPPV(KST:KND,ILPT)/PGM(KST:KND)
  ENDDO
ENDIF

I=TFP_KHI%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,QFPTYPE%ILEV(I)
    ILPT=QFPTYPE%ILVP(JL,I)
    ILOC=QFPTYPE%IGT1(I,1)+JL-1
    PGPP(KST:KND,ILOC) = ZPPU(KST:KND,ILPT)/PGM(KST:KND)
  ENDDO
ENDIF

I=TFP_DIV%ICOD
IF (QFPTYPE%LL(I)) THEN
  IF (QFPTYPE%ISKP(I) == 2) THEN
    IF (QFPTYPE%ISF(I) >= 1) THEN
      DO JL=1,QFPTYPE%ILEV(I)
        ILPT=QFPTYPE%ILVP(JL,I)
        ILOCU=QFPTYPE%IGT1(I,1)+JL-1
        ILOCV=QFPTYPE%IGT1(I,2)+JL-1
        PGPP(KST:KND,ILOCU) = ZPPU(KST:KND,ILPT)/PGM(KST:KND)
        PGPP(KST:KND,ILOCV) = ZPPV(KST:KND,ILPT)/PGM(KST:KND)
      ENDDO
    ELSE
      CALL ABOR1('POS : INTERNAL ERROR ON DIV')
    ENDIF
  ELSE
    DO JL=1,KOPLEV
      ZPPIN(KST:KND,JL)=ZPPPDIV(KST:KND,JL)/ZGMAPP(KST:KND)
    ENDDO
    CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDIF

I=TFP_VOR%ICOD
IF (QFPTYPE%LL(I)) THEN
  IF (QFPTYPE%ISKP(I) == 2) THEN
    IF (QFPTYPE%ISF(I) >= 1) THEN
      DO JL=1,QFPTYPE%ILEV(I)
        ILPT=QFPTYPE%ILVP(JL,I)
        ILOCU=QFPTYPE%IGT1(I,1)+JL-1
        ILOCV=QFPTYPE%IGT1(I,2)+JL-1
        PGPP(KST:KND,ILOCU) = ZPPU(KST:KND,ILPT)/PGM(KST:KND)
        PGPP(KST:KND,ILOCV) = ZPPV(KST:KND,ILPT)/PGM(KST:KND)
      ENDDO
    ELSE
      CALL ABOR1('POS : INTERNAL ERROR ON DIV')
    ENDIF
  ELSE
    DO JL=1,KOPLEV
      ZPPIN(KST:KND,JL)=ZPPPVOR(KST:KND,JL)/ZGMAPP(KST:KND)
    ENDDO
    CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDIF

I=TFP_ABS%ICOD
IF (QFPTYPE%LL(I)) THEN
  IF (QFPTYPE%ISKP(I) == 2) THEN
    IF (QFPTYPE%ISF(I) >= 1) THEN
      DO JL=1,QFPTYPE%ILEV(I)
        ILPT=QFPTYPE%ILVP(JL,I)
        ILOCU=QFPTYPE%IGT1(I,1)+JL-1
        ILOCV=QFPTYPE%IGT1(I,2)+JL-1
        PGPP(KST:KND,ILOCU) = ZPPU(KST:KND,ILPT)/PGM(KST:KND)
        PGPP(KST:KND,ILOCV) = ZPPV(KST:KND,ILPT)/PGM(KST:KND)
      ENDDO
    ELSE
      CALL ABOR1('POS : INTERNAL ERROR ON DIV')
    ENDIF
  ELSE
    DO JL=1,KOPLEV
      ZPPIN(KST:KND,JL)=ZPPPABS(KST:KND,JL)/ZGMAPP(KST:KND)
    ENDDO
    CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDIF

I=TFP_P%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPUAP,PGPP,PAUX)

I=TFP_MG%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPMG,PGPP,PAUX)

IPTR=1
DO JVAR = 1,JPOSVX2
  I=TFP_FUA(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) THEN
    CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPFUA(1,1,IPTR),PGPP,PAUX)
    IPTR=IPTR+1
  ENDIF
ENDDO

I=TFP_PD%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPPDEP,PGPP,PAUX)

I=TFP_VD%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPVDIV,PGPP,PAUX)

I=TFP_VW%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KST,KND,KOPLEV,I,ZPPWVEL,PGPP,PAUX)

I=TFP_UJET%ICOD
IF (QFPTYPE%LL(I)) THEN
  ZPPIN1(KST:KND)=ZUJET(KST:KND)/PGM(KST:KND)
  CALL FILL_PP2(KST,KND,I,ZPPIN1,PGPP,PAUX)
ENDIF

I=TFP_VJET%ICOD
IF (QFPTYPE%LL(I)) THEN
  ZPPIN1(KST:KND)=ZVJET(KST:KND)/PGM(KST:KND)
  CALL FILL_PP2(KST,KND,I,ZPPIN1,PGPP,PAUX)
ENDIF

I=TFP_PJET%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPJET,PGPP,PAUX)

I=TFP_PCAO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPICAO,PGPP,PAUX)

I=TFP_TCAO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZTICAO,PGPP,PAUX)

DO JVAR = 1,JPOSFSU
  I=TFP_FSU(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPFSU(1,JVAR),PGPP,PAUX)
ENDDO

IF ((LFPLOSP.AND.QFPTYPE%LL(TFP_SP%ICOD)).OR.QFPTYPE%LL(TFP_LNSP%ICOD)) THEN
  DO JROF = KST , KND
    ZLNPPSP(JROF) = LOG(ZPPSP(JROF))
  ENDDO
ENDIF
I=TFP_SP%ICOD
IF (LFPLOSP) THEN
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZLNPPSP,PGPP,PAUX)
ELSE
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPPSP,PGPP,PAUX)
ENDIF
I=TFP_LNSP%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZLNPPSP,PGPP,PAUX)

I=TFP_WWS%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZWWT0(1,NFLEVG),PGPP,PAUX)

I=TFP_FIS%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,POROG,PGPP,PAUX)

I=TFP_GM%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,PGM,PGPP,PAUX)

I=TFP_MOCO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZMOCO,PGPP,PAUX)

I=TFP_PT0B%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPT0B,PGPP,PAUX)

I=TFP_HT0B%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZHT0B,PGPP,PAUX)

I=TFP_PT1B%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZPT1B,PGPP,PAUX)

I=TFP_HT1B%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZHT1B,PGPP,PAUX)

I=TFP_HTPW%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KST,KND,I,ZHTPW,PGPP,PAUX)


!*    3.3   DEALLOCATIONS

DEALLOCATE(ZPPFUA)
DEALLOCATE(ZPPFSU)
IF(NGHG > 0 .AND. LL_LRCH4) THEN
  DEALLOCATE(ZPPLRCH4)
  DEALLOCATE(ZLRCH4T0)
ENDIF
IF(NGHG > 0 .AND. LL_CH4S) THEN
  DEALLOCATE(ZPPCH4S)
  DEALLOCATE(ZCH4ST0)
ENDIF
IF(NNOGW > 0) THEN
  DEALLOCATE(ZPPNOGW)
  DEALLOCATE(ZNOGWT0)
ENDIF
IF(NAEROUT > 0) THEN
  DEALLOCATE(ZPPAEROUT)
  DEALLOCATE(ZAEROUT0)
ENDIF
IF(NUVP > 0) THEN
  DEALLOCATE(ZPPUVP)
  DEALLOCATE(ZUVPT0)
ENDIF
IF(NERA40 > 0) THEN
  DEALLOCATE(ZERA40)
  DEALLOCATE(ZPPERA)
ENDIF
IF(YSD_XAD%NUMFLDS > 0) THEN
  DEALLOCATE(ZEXTRA)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('POS',1,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

CONTAINS

SUBROUTINE FILL_PP2(KST,KND,KCOD,PIN,PGPP,PAUX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KST
INTEGER(KIND=JPIM), INTENT(IN)  :: KND
INTEGER(KIND=JPIM), INTENT(IN)  :: KCOD
REAL(KIND=JPRB)   , INTENT(IN)  :: PIN(NPROMA)
REAL(KIND=JPRB)   , INTENT(OUT) :: PGPP(NPROMA,KFLDSPP)
REAL(KIND=JPRB)   , INTENT(OUT) :: PAUX(NPROMA,KFLDAUX)

INTEGER(KIND=JPIM) :: ILOC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP2',0,ZHOOK_HANDLE)

IF (QFPTYPE%ISF(KCOD) >= 1) THEN
  ILOC=QFPTYPE%IGT1(KCOD,1)
  PGPP(KST:KND,ILOC) = PIN(KST:KND)
ELSE
  ILOC=QFPTYPE%IGPX(KCOD)
  PAUX(KST:KND,ILOC) = PIN(KST:KND)
ENDIF

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP2',1,ZHOOK_HANDLE)

END SUBROUTINE FILL_PP2

!     ------------------------------------------------------------------

SUBROUTINE FILL_PP3(KST,KND,KOPLEV,KCOD,PIN,PGPP,PAUX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KST
INTEGER(KIND=JPIM), INTENT(IN)  :: KND
INTEGER(KIND=JPIM), INTENT(IN)  :: KOPLEV
INTEGER(KIND=JPIM), INTENT(IN)  :: KCOD
REAL(KIND=JPRB)   , INTENT(IN)  :: PIN(NPROMA,KOPLEV)
REAL(KIND=JPRB)   , INTENT(OUT) :: PGPP(NPROMA,KFLDSPP)
REAL(KIND=JPRB)   , INTENT(OUT) :: PAUX(NPROMA,KFLDAUX)

INTEGER(KIND=JPIM) :: ILOC,ILPT,JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP3',0,ZHOOK_HANDLE)

DO JL=1,QFPTYPE%ILEV(KCOD)
  ILPT=QFPTYPE%ILVP(JL,KCOD)
  IF (QFPTYPE%ISF(KCOD) >= 1) THEN
    ILOC=QFPTYPE%IGT1(KCOD,1)+JL-1
    PGPP(KST:KND,ILOC) = PIN(KST:KND,ILPT)
  ELSE
    ILOC=QFPTYPE%IGPX(KCOD)+JL-1
    PAUX(KST:KND,ILOC) = PIN(KST:KND,ILPT)
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP3',1,ZHOOK_HANDLE)

END SUBROUTINE FILL_PP3

!     ------------------------------------------------------------------
END SUBROUTINE POS
