SUBROUTINE PPAKAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEV_AK,KLEVB,&
 & PRES,PLNPRES,LDBELS,LDBLES,PWEIGHT, &
 & PRXP,PRXPD,PGRGF,&
 & PGRGPP,&
 & PRXP5,PRXPD5,PGRGF5,cdpar)  

!**** *PPAKAD* - POST-PROCESS GRG fields using AVERAGING KERNELS (AK), ADJOINT.

!     PURPOSE.
!     --------
!       PERFORMS THE VERTICAL INTERPOLATION TO THE GIVEN
!       averaging kernel PRESSURE LEVELS. 
!       COMPUTES model equivalent of obs as SUM(zgrg*pweight)

!**   INTERFACE.
!     ----------
!        *CALL* *PPAKAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,PWEIGHT,KLOLEV,KLEVB,
!                      PRES,PLNPRES,LDBELS,LDBLES,PRXP,PRXPD,PGRGF,
!                      PGRGPP,
!                      PRXP5,PRXPD5,PGRGF5)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT)
!        KSTART                    - START OF WORK.                    (INPUT)
!        KPROF                     - DEPTH OF WORK.                    (INPUT)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT)
!        KLEV_AK                  - NUMBER OF averaging kernel
!                                    PRESSURE LEVELS,                  (INPUT)

!        KLEVB(KPROMA,KLEV_AK,NPPM,KLEVP)  - INPUT LEVEL BELOW PRES            (INPUT)
!                                    (SEE PPFLEV)
!        PRES(KPROMA,KLEVP,KLEV_AK)        - POST-PROCESSING LEVEL PRESSURES   (INPUT)
!        PLNPRES(KPROMA,KLEV_AK,KLEVP)     - LOG(PRES)                         (INPUT)
!        LDBELS(KPROMA,KLEV_AK,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE                    (INPUT)
!        LDBLES(KLEV_AK,KLEVP)             - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT)
!        PWEIGHT(KPROMA,KLEV_AK,KLEVP)     - Weights for averaging kernel      (INPUT)
!        PRXP(KPROMA,0:KFLEV,NPPM) - Gradient wrt HALF,FULL AND LN HALF LEVEL
!                                    PRESSURES (SEE PPINIT)            (INOUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- Gradient wrt 1./D(P) AND 1./D(LN(P))(INOUT)
!        PGRGF(KPROMA,0:KFLEV)     - Gradient wrt GRG field            (INOUT)

!        PGRGPP(KPROMA,KLEVP)      - POST-PROCESSED GRG column field   (INOUT)

!        PRXP5(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF LEVEL
!                                    PRESSURES                         (INPUT)
!        PRXPD5(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))          (INPUT)
!        PGRGF5(KPROMA,0:KFLEV)     - GRG field ON FULL INPUT LEVELS.  (INOUT)
!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMDIM.
!        --------------------

!     METHOD.
!     -------
!        CALCULATE model fields on averaging kernel pressure levels and
!        model equivalent of observation as PGRGPP=SUM(ZGRG*PWEIGHT)

!     REFERENCE.
!     ----------

!     AUTHOR.
!     -------
!        ANTJE DETHOF, BASED ON PP-ROUTINES BY
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 27-Jul-2005
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV_AK
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEV_AK,NPPM,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,KLEVP,KLEV_AK) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRES(KPROMA,KLEV_AK,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEV_AK,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEV_AK,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWEIGHT(KPROMA,KLEVP,KLEV_AK) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGRGF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: PGRGPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PGRGF5(KPROMA,0:KFLEV) 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR

INTEGER(KIND=JPIM) :: ISLCT, JL, JLEV, JLEVP,IAK
INTEGER(KIND=JPIM) :: IAB, IBL

REAL(KIND=JPRB) :: ZGRG(KPROMA,KLEVP,0:KLEV_AK),ZRGI
REAL(KIND=JPRB) :: ZCOL(KPROMA,0:KFLEV),ZCOL5(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gppwc.intfb.h"
#include "gppwcad.intfb.h"

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS GRG field
!              -----------------------
IF (LHOOK) CALL DR_HOOK('PPAKAD',0,ZHOOK_HANDLE)

CALL GPPWC(KPROMA,KSTART,KPROF,KFLEV,ZCOL5,PGRGF5(1,1),PRXP5)
DO JLEV=0,KFLEV
  DO JL=KSTART,KPROF
    ZCOL(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO


!*       1.4   COMPUTE SUM(ZGRG*PWEIGHT)

ZGRG(:,:,:)=0.0_JPRB

DO IAK=KLEV_AK,1,-1
  DO JLEVP=KLOLEV,KLEVP
    DO JL=KSTART,KPROF
      ZGRG(JL,JLEVP,IAK)=ZGRG(JL,JLEVP,IAK) + PGRGPP(JL,JLEVP)*PWEIGHT(JL,JLEVP,IAK)
      ZGRG(JL,JLEVP,IAK-1)=ZGRG(JL,JLEVP,IAK-1) - PGRGPP(JL,JLEVP)*PWEIGHT(JL,JLEVP,IAK)
    ENDDO
  ENDDO
ENDDO
PGRGPP(:,:)=0.0_JPRB

!*       1.3   EXTRAPOLATE GRG FIELD BELOW surface
!*             ASSUME GRG=0 below surface

ISLCT=3
DO JLEVP=KLOLEV,KLEVP

!*       1.3   EXTRAPOLATE GRG column DOWNWARDS.
!*             ASSUME CONSTANT GRG, AS IN ROUTINE PPQ.

ZRGI=1.0_JPRB/RG
  DO IAK=1,KLEV_AK
    IF(LDBLES(IAK,JLEVP)) THEN
      DO JL=KSTART,KPROF
        IF(LDBELS(JL,IAK,JLEVP)) THEN
          PGRGF(JL,KFLEV)=PGRGF(JL,KFLEV)&
           & +ZRGI*(PRES(JL,JLEVP,IAK)-PRXP5(JL,KFLEV,1))*ZGRG(JL,JLEVP,IAK)
          PRXP(JL,KFLEV,1)=PRXP(JL,KFLEV,1)-ZRGI*PGRGF5(JL,KFLEV)*ZGRG(JL,JLEVP,IAK)
          ZCOL(JL,KFLEV)=ZCOL(JL,KFLEV)+ZGRG(JL,JLEVP,IAK)
          ZGRG(JL,JLEVP,IAK)=0.0_JPRB
        ENDIF
      ENDDO
    ENDIF
  ENDDO
ENDDO

!*       1.2    LINEAR INTERPOLATION AND UPWARD EXTRAPOLATION of GRG column.

DO JLEVP=KLOLEV,KLEVP
  DO IAK=1,KLEV_AK
    IF (.NOT. LDBLES(IAK,JLEVP)) THEN
      DO JL=KSTART,KPROF
        IBL=KLEVB(JL,IAK,ISLCT,JLEVP)
        IAB=IBL-1
        ZCOL(JL,IAB)=ZCOL(JL,IAB)+(1.0_JPRB-((PLNPRES(JL,IAK,JLEVP)-&
         & PRXP5(JL,IAB,ISLCT))*PRXPD5(JL,IAB,ISLCT)))*ZGRG(JL,JLEVP,IAK)
        ZCOL(JL,IBL)=ZCOL(JL,IBL)+((PLNPRES(JL,IAK,JLEVP)&
         & -PRXP5(JL,IAB,ISLCT))*PRXPD5(JL,IAB,ISLCT))*ZGRG(JL,JLEVP,IAK)
        PRXP(JL,IAB,ISLCT)=PRXP(JL,IAB,ISLCT)-(ZCOL5(JL,IBL)-&
         & ZCOL5(JL,IAB))*PRXPD5(JL,IAB,ISLCT)*ZGRG(JL,JLEVP,IAK)
        PRXPD(JL,IAB,ISLCT)=PRXPD(JL,IAB,ISLCT)+(ZCOL5(JL,IBL)-&
         & ZCOL5(JL,IAB))*(PLNPRES(JL,IAK,JLEVP)-PRXP5(JL,IAB,ISLCT))*&
         & ZGRG(JL,JLEVP,IAK)
        ZGRG(JL,JLEVP,IAK)=0.0_JPRB
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        IF(.NOT.LDBELS(JL,IAK,JLEVP)) THEN
          IBL=KLEVB(JL,IAK,ISLCT,JLEVP)
          IAB=IBL-1
          ZCOL(JL,IAB)=ZCOL(JL,IAB)+(1.0_JPRB-((PLNPRES(JL,IAK,JLEVP)-&
           & PRXP5(JL,IAB,ISLCT))*PRXPD5(JL,IAB,ISLCT)))*ZGRG(JL,JLEVP,IAK)
          ZCOL(JL,IBL)=ZCOL(JL,IBL)+((PLNPRES(JL,IAK,JLEVP)&
           & -PRXP5(JL,IAB,ISLCT))*PRXPD5(JL,IAB,ISLCT))*ZGRG(JL,JLEVP,IAK)
          PRXP(JL,IAB,ISLCT)=PRXP(JL,IAB,ISLCT)-(ZCOL5(JL,IBL)-&
           & ZCOL5(JL,IAB))*PRXPD5(JL,IAB,ISLCT)*ZGRG(JL,JLEVP,IAK)
          PRXPD(JL,IAB,ISLCT)=PRXPD(JL,IAB,ISLCT)+(ZCOL5(JL,IBL)-&
           & ZCOL5(JL,IAB))*(PLNPRES(JL,IAK,JLEVP)-PRXP5(JL,IAB,ISLCT))*&
           & ZGRG(JL,JLEVP,IAK)
          ZGRG(JL,JLEVP,IAK)=0.0_JPRB
        ENDIF
      ENDDO
    ENDIF
  ENDDO

ENDDO

!*       1.1   COMPUTE column GRG ON MODEL HALF LEVELS.
!*             Column GRG=0 AT TOP OF MODEL.

CALL GPPWCAD(KPROMA,KSTART,KPROF,KFLEV,ZCOL,PGRGF(1,1),PRXP,PGRGF5(1,1),PRXP5)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPAKAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPAKAD
