SUBROUTINE PPCLWAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & PCLWF,&
 & PCLWPP)  

!**** *PPCLWAD * - POST-PROCESS CLOUD LIQUID WATER (AD)

!     PURPOSE.
!     --------
!        PERFORMS THE VERTICAL INTERPOLATION OF CLOUD LIQUID WATER
!     TO A GIVEN LEVEL.

!**   INTERFACE.
!     ----------
!        *CALL* *PPCLWAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                        PCLWF,
!                        PCLWPP)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRPRES          (INPUT-C)
!                                    (SEE PPFLEV)
!        PCLWF(KPROMA,0:KFLEV)     - CLOUD LW ON FULL INPUT LEVELS     (INPUT)

!        PCLWPP(KPROMA,KLEVP)      - POST-PROCESSED CLOUD LW           (OUTPUT)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST AND YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  PPINTP - LINEAR INTERPOLATION
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 93-01-12
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLWF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLWPP(KPROMA,KLEVP) 
INTEGER(KIND=JPIM) :: IPNT, JL, JLEVP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPCLWAD',0,ZHOOK_HANDLE)
DO JLEVP=KLOLEV,KLEVP
  DO JL   =KSTART,KPROF
    IPNT =MIN(MAX(KLEVB(JL,JLEVP,1),0),KFLEV)
    PCLWF(JL,IPNT)   = PCLWF(JL,IPNT) +  PCLWPP(JL,JLEVP)
  ENDDO
ENDDO

!*       1.1   PREPARE FOR EXTRAPOLATION ABOVE TOP

DO JL=KSTART,KPROF
  PCLWF(JL,1)=PCLWF(JL,1)+PCLWF(JL,0)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPCLWAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPCLWAD

