SUBROUTINE PPCVIRT(KPROMA,KSTART,KPROF,KFLEV,PT,PQ,PQL,PQR,PQS,&
 & PQG,PQI,PTV)

!**** *PPCVIRT* - Computes virtual coefficient for T or Theta

!     Purpose.
!     --------
!          COMPUTE THE NEDDED COEFFICIENT FOR Tv or Thetav

!**   INTERFACE.
!     ----------
!       *CALL* *PPCVIRT*

!        EXPLICIT ARGUMENTS
!        --------------------
!            INPUT :
!        KPROMA    : Horizontal dimension
!        KSTART    : start of work
!        KPROF     : depth of work
!        KFLEV     : number of vertical levels
!        PT        : t or theta   (KPROMA,KFLEV)
!        PQ        : specific humidity (KPROMA,KFLEV)
!        PQL      : cloud water (KPROMA,KFLEV)
!        PQL      : cloud rain (KPROMA,KFLEV)
!        PQI      : cloud ice (KPROMA,KFLEV)
!        PQS      : cloud snow (KPROMA,KFLEV)
!        PQG      : cloud graupel (KRPOMA,KFLEV)
!            OUTPUT:
!        PTV    :  Tv or Thetav (KRPOMA,KLEV)

!        IMPLICIT ARGUMENTS
!        --------------------
!           NONE

!     METHOD.
!     -------
!       The virtual Temperature is defined as the temperature that will have
!       a dry parcel of the same P and Rho that the humid parcel

!     EXTERNALS.
!     ----------
   
!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        Gwenaelle Hello *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 07-02-01
!     ------------------------------------------------------------------
USE PARKIND1 ,ONLY : JPIM, JPRB
USE YOMHOOK  ,ONLY : LHOOK, DR_HOOK
USE YOMCST   ,ONLY : RD, RV
!-----------------------------------------------------------------------
IMPLICIT NONE
 
INTEGER(KIND=JPIM),INTENT(IN) :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN) :: KSTART
INTEGER(KIND=JPIM),INTENT(IN) :: KPROF
INTEGER(KIND=JPIM),INTENT(IN) :: KFLEV
REAL(KIND=JPRB),INTENT(IN) :: PQ(KPROMA,KFLEV)
REAL(KIND=JPRB),INTENT(IN) :: PT(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(IN) :: PQL(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(IN) :: PQR(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(IN) :: PQS(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(IN) :: PQG(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(IN) :: PQI(KPROMA,KFLEV)
REAL(KIND=JPRB),INTENT(OUT) :: PTV(KPROMA,KFLEV)
!------------------------------------------------------------------------
REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTEGER(KIND=JPIM) :: JLEV,JLON
REAL(KIND=JPRB) :: ZALPHA
!------------------------------------------------------------------------
IF(LHOOK) CALL DR_HOOK('PPCVIRT',0,ZHOOK_HANDLE)

ZALPHA= RV/RD - 1._JPRB
DO JLEV=1,KFLEV
   DO JLON=KSTART,KPROF
      PTV(JLON,JLEV)=( 1 + PQ(JLON,JLEV)*ZALPHA)*PT(JLON,JLEV)
   ENDDO
   IF(PRESENT(PQL)) THEN
      DO JLON=KSTART,KPROF
         PTV(JLON,JLEV)=PTV(JLON,JLEV)-PQL(JLON,JLEV)*PT(JLON,JLEV)
      ENDDO
   ENDIF
   IF(PRESENT(PQR)) THEN
      DO JLON=KSTART,KPROF
         PTV(JLON,JLEV)=PTV(JLON,JLEV)-PQR(JLON,JLEV)*PT(JLON,JLEV)
      ENDDO
   ENDIF
  IF(PRESENT(PQS)) THEN
     DO JLON=KSTART,KPROF
        PTV(JLON,JLEV)=PTV(JLON,JLEV)-PQS(JLON,JLEV)*PT(JLON,JLEV)
     ENDDO
  ENDIF
  IF(PRESENT(PQG)) THEN
     DO JLON=KSTART,KPROF
        PTV(JLON,JLEV)=PTV(JLON,JLEV)-PQG(JLON,JLEV)*PT(JLON,JLEV)
     ENDDO
  ENDIF
  IF(PRESENT(PQI)) THEN
     DO JLON=KSTART,KPROF
        PTV(JLON,JLEV)=PTV(JLON,JLEV)-PQI(JLON,JLEV)*PT(JLON,JLEV)
     ENDDO
  ENDIF
ENDDO
!------------------------------------------------------------
IF(LHOOK) CALL DR_HOOK('PPCVIRT',1,ZHOOK_HANDLE)
END SUBROUTINE PPCVIRT
