SUBROUTINE PPGEOP_OLD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & KLEVB,&
 & PRES,PLNPRES,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,POROG,&
 & PRXP,PRXPD,PTSTAR,PR0,PLNPR,PALPH,&
 & PTF,PZPP)  

!**** *PPGEOP* - POST-PROCESS GEOPOTENTIAL.

!     PURPOSE.
!     --------
!           COMPUTES GEOPOTENTIAL AT FULL LEVELS AND
!       PERFORMS THE VERTICAL INTERPOLATION TO A GIVEN
!       MODEL CO-ORDINATE LEVEL. ALSO EXTRAPOLATES GEOPOTENTIAL
!       IF SO REQUESTED. THE INTERPOLATION IS DONE IN TERMS OF
!       GEOPOTENTIAL DEPARTURES FROM THE ICAO STANDARD ATMOSPHERE.

!**   INTERFACE.
!     ----------
!        *CALL* *PPGEOP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                PRES,PLNPRES,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,POROG,
!                       PRXP,PRXPD,PTSTAR,PR0,PLNPR,PALPH,
!                       PTF,PZPP)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRES (PLNPRES)  (INPUT-C)
!                                    (SEE PPFLEV)
!        PRES(KPROMA,KLEVP)        - POST-PROCESSING LEVEL PRESSURE    (INPUT-C)
!        PLNPRES(KPROMA,KLEVP)     - LOG(POST-PROCESSING LEVEL PRES.)  (INPUT-C)
!        POROG(KPROMA)             - MODEL OROGRAPHY.                  (INPUT-C)
!        LDBELO(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     LOWEST (FULL) MODEL LEVEL        (INPUT-C)
!        LDBELS(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE                    (INPUT-C)
!        LDBLOW(KLEVP)             - .TRUE. IF LDBELO(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        LDBLES(KLEVP)             - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        LDEXTR                    - .TRUE. IF EXTRAPOLATION REQUESTED (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!        PTSTAR(KPROMA)            - SURFACE TEMPERATURE               (INPUT)
!        PR0(KPROMA,KFLEV)         - R                                 (INPUT)
!        PLNPR(KPROMA,KFLEV)       - LOGARITHM OF RATIO OF PRESSURE
!                                    (PREVIOUSLY COMPUTED IN GPXYB)    (INPUT)
!        PALPH(KPROMA,KFLEV)       - COEFFICIENTS OF THE HYDROSTATICS
!                                    (PREVIOUSLY COMPUTED IN GPXYB)    (INPUT)
!        PTF(KPROMA,0:KFLEV)       - TEMPERATURE ON FULL INPUT LEVELS. (INPUT)

!        PZPP(KPROMA,KLEVP)        - POST-PROCESSED GEOPOTENTIAL       (OUTPUT)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  GPGEO  - COMPUTE GEOPOTENTIAL ON HALF LEVELS
!     ----------  PPINTP - LINEAR INTERPOLATION
!                 PPSTA  - TEMP. AND GEOPOTENTIAL OF THE STANDARD ATM.
!                 PPINTP - LINEAR INTERPOLATION
!                 PPITPQ - QUADRATIC INTERPOLATION FOR TOP LEVELS

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-01-26
!        Erik Andersson  920409: PP geop. on full levels, in ln(p).
!                                Integrate and interpolate dep. fr. ICAO
!          "  , J-F. Geleyn      Mod. T*, T0 and alpha below surface.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRES(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLOW(KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDEXTR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZPP(KPROMA,KLEVP) 
REAL(KIND=JPRB) :: ZFLDH(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFLDF(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZTSTAR(KPROMA)
REAL(KIND=JPRB) :: ZALPHA(KPROMA)

REAL(KIND=JPRB) :: ZSTTP(KPROMA,KLEVP),ZSTZP(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZSTTF(KPROMA,0:KFLEV),ZSTZF(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZSTTH(KPROMA),ZSTZH(KPROMA)

INTEGER(KIND=JPIM) :: ISLCT, JL, JLEV, JLEVP

REAL(KIND=JPRB) :: ZALFLP2, ZALFLPR, ZCOEF, ZDTDZSG, ZLNPRT,&
 & ZOROG, ZS, ZT0, ZTX, ZTY, ZZ  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpgeo.intfb.h"
#include "ppintp.intfb.h"
#include "ppitpq.intfb.h"
#include "ppsta.intfb.h"

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS GEOPOTENTIAL.
!              --------------------------

!*       1.1   COMPUTE GEOPOTENTIAL ON MODEL LEVELS. DEP FROM ICAO.
!           ZSTTF FIRST CONTAINS ICAO TEMPERATURES ON FULL LEVELS, THEN
!           IT IS CHANGED TO CONTAIN 'MOIST' DEPARTURES FROM ICAO.

IF (LHOOK) CALL DR_HOOK('PPGEOP_OLD',0,ZHOOK_HANDLE)
CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KFLEV+1,1,&
 & PRXP(1,0,2),PRXP(1,0,4),&
 & ZSTTF,ZSTZF)  

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    ZS=PTF(JL,JLEV)-ZSTTF(JL,JLEV)*RD/PR0(JL,JLEV)
    ZSTTF(JL,JLEV)=ZS
  ENDDO
ENDDO

DO JL=KSTART,KPROF
  ZFLDH(JL,KFLEV)=0.0_JPRB
ENDDO
CALL GPGEO(KPROMA,KSTART,KPROF,KFLEV,ZFLDH,ZFLDF(1,1),ZSTTF(1,1),&
 & PR0,PLNPR,PALPH)  

!     PROVISIONAL DEFINITION OF ZFLDF(-,0) FOR HIGH LEVEL INTERPOLATION
DO JL = KSTART, KPROF
  ZFLDF(JL,0) = ZFLDH(JL,0)
ENDDO

!*       1.2   INTERPOLATE GEOPOTENTIAL DEPARTURE FROM ICAO.
!              ISLCT=3 GOES WITH ZFLDH (HALF LEVELS)
!              ISLCT=4 GOES WITH ZFLDF (FULL LEVELS)

ISLCT=4
CALL PPINTP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PLNPRES,PRXP,PRXPD,&
 & ZFLDF,PZPP)  

!*       1.3   INTERPOLATE BETWEEN LOWEST LEVEL AND SURFACE.

IF(MOD(ISLCT,2) == 0) THEN
  DO JLEVP=KLOLEV,KLEVP
    IF (LDBLOW(JLEVP)) THEN
      DO JL=KSTART,KPROF
        IF (LDBELO(JL,JLEVP).AND..NOT.LDBELS(JL,JLEVP) ) THEN
          ZCOEF=(PRXP(JL,KFLEV,ISLCT-1)-PLNPRES(JL,JLEVP))/&
           & (PRXP(JL,KFLEV,ISLCT-1)-PRXP(JL,KFLEV,ISLCT))  
          PZPP(JL,JLEVP)=ZCOEF*ZFLDF(JL,KFLEV)
        ENDIF
      ENDDO
    ENDIF
  ENDDO
ENDIF

!*       1.4   MODIFY INTERPOLATION/EXTRAPOLATION ABOVE SECOND
!*          FULL MODEL LEVEL. FIT QUADRATIC POLYNOMIAL IN LN(P).

CALL PPITPQ(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,ISLCT,&
 & PLNPRES,PRXP,ZFLDF,&
 & PZPP)  

!*       1.5   CALCULATE ICAO GEOPOTENTIAL ON PRESSURE LEVELS.
!*          ADD ICAO GEOPOTENTIAL TO THE POST-PROCESSED DEPARTURES.
!*          ALSO CALCULATE ICAO HEIGHTS AT MODEL SURFACE AND
!*          ADD TO POST-PROCESSED GEOPOTENTIAL.

CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KLEVP,KLOLEV,PRES,PLNPRES,ZSTTP,ZSTZP)

CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,1,1,&
 & PRXP(1,KFLEV,1),PRXP(1,KFLEV,3),ZSTTH,ZSTZH)  

DO JLEVP=KLOLEV,KLEVP
  DO JL=KSTART,KPROF
    IF(.NOT.LDBELS(JL,JLEVP)) THEN
      ZZ=POROG(JL)+PZPP(JL,JLEVP)+ZSTZP(JL,JLEVP)-ZSTZH(JL)
      PZPP(JL,JLEVP)=ZZ
    ENDIF
  ENDDO
ENDDO

!*       1.6   EXTRAPOLATE GEOPOTENTIAL DOWNWARDS.

IF(LDEXTR) THEN
  ZTX=290.5_JPRB
  ZTY=255.0_JPRB
  ZDTDZSG=-RDTDZ1/RG
  DO JL=KSTART,KPROF
    IF(PTSTAR(JL) < ZTY) THEN
      ZTSTAR(JL)=0.5_JPRB*(ZTY+PTSTAR(JL))
    ELSEIF(PTSTAR(JL) < ZTX) THEN
      ZTSTAR(JL)=PTSTAR(JL)
    ELSE
      ZTSTAR(JL)=0.5_JPRB*(ZTX+PTSTAR(JL))
    ENDIF

    ZT0=ZTSTAR(JL)+ZDTDZSG*POROG(JL)
    IF(ZTX > ZTSTAR(JL) .AND. ZT0 > ZTX) THEN
      ZT0=ZTX
    ELSEIF(ZTX <= ZTSTAR(JL) .AND. ZT0 > ZTSTAR(JL)) THEN
      ZT0=ZTSTAR(JL)
    ENDIF

    ZOROG=SIGN(MAX(1.0_JPRB,ABS(POROG(JL))),POROG(JL))
    ZALPHA(JL)=RD*(ZT0-ZTSTAR(JL))/ZOROG
  ENDDO

  DO JLEVP=KLOLEV,KLEVP
    IF (LDBLES(JLEVP)) THEN
      DO JL=KSTART,KPROF
        IF(LDBELS(JL,JLEVP)) THEN
          ZLNPRT=PLNPRES(JL,JLEVP)-PRXP(JL,KFLEV,3)
          ZALFLPR=ZALPHA(JL)*ZLNPRT
          ZALFLP2=ZALFLPR*ZALFLPR
          PZPP(JL,JLEVP)=&
           & POROG(JL)-RD*ZTSTAR(JL)*ZLNPRT*&
           & (1.0_JPRB+0.5_JPRB*ZALFLPR+1.0_JPRB/6._JPRB*ZALFLP2)  
        ENDIF
      ENDDO
    ENDIF
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPGEOP_OLD',1,ZHOOK_HANDLE)
END SUBROUTINE PPGEOP_OLD
