SUBROUTINE PPINITZA(KPROMA,KSTART,KPROF,KFLEV,&
 & PRGEOPH,PRGEOPF,&
 & PRXP,PRXPD,&
 & PRXP5)  

!**** *PPINITZA* - SET UP HELP-ARRAYS FOR VERTICAL (Z) INTERPOLATION
!                  (ADJOINT)

!     PURPOSE.
!     --------
!        SETS UP HELP-ARRAYS FOR Z LEVEL VERTICAL INTERPOLATION.
!     MUST BE CALLED BEFORE ANY INTERPOLATION ROUTINES.

!**   INTERFACE.
!     ----------
!        *CALL* *PPINITZA(KPROMA,KSTART,KPROF,KFLEV
!                        PRPRESH,PRPRESF,
!                        PRXP,PRXPD
!                        PRPRESH5,PRPRESF5,PRXP5)*

!        EXPLICIT ARGUMENTS :   NONE.
!        --------------------
!        KPROMA                    - FIRST DIMENSION FOR ARRAYS
!        KSTART                    - START OF WORK
!        KPROF                     - NO. OF COLOMNS FOR INTERPOLATAION
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS
!        PRGEOPH(KPROMA,0:KFLEV)   - INPUT HALF LEVEL GEOP.
!        PRPRESF(KPROMA,KFLEV)     - INPUT FULL LEVEL GEOP.
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    GEOPOTENTIALS
!        PRXPD(KPROMA,0KFLEV,NPPM) - 1./D(Z) AND 1./D(LN(Z))
!        PRXP5(KPROMA,0:KFLEV,NPPM)- HALF,FULL AND LN HALF,FULL LEVEL Z

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     EXTERNALS.   NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRGEOPH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRGEOPF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 
INTEGER(KIND=JPIM) :: JFLEV, JL, JSLCT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    COMPUTE HELP VARIABLES RELATED TO INPUT GRID.
!              --------------------------------------------

!*       1.2   1/D(Z) and 1/D(Ln(Z))

IF (LHOOK) CALL DR_HOOK('PPINITZA',0,ZHOOK_HANDLE)
DO JSLCT=1,NPPM
  DO JFLEV=0,KFLEV-1
    DO JL=KSTART,KPROF
      PRXP(JL,JFLEV,JSLCT)=PRXP(JL,JFLEV,JSLCT)-&
       & PRXPD(JL,JFLEV,JSLCT)/&
       & (PRXP5(JL,JFLEV  ,JSLCT)-PRXP5(JL,JFLEV+1,JSLCT))**2  

      PRXP(JL,JFLEV+1,JSLCT)=PRXP(JL,JFLEV+1,JSLCT)+&
       & PRXPD(JL,JFLEV,JSLCT)/&
       & (PRXP5(JL,JFLEV  ,JSLCT)-PRXP5(JL,JFLEV+1,JSLCT))**2  
    ENDDO
  ENDDO
ENDDO

!*       1.1   GEOP. AT HALF/FULL LEVELS AND LN GEOP.AT HALF/FULL LEVELS

DO JL=KSTART,KPROF
  PRGEOPH(JL,0)=PRGEOPH(JL,0)+PRXP(JL,0,1)
  PRGEOPH(JL,0)=PRGEOPH(JL,0)+PRXP(JL,0,3)
ENDDO

DO JFLEV=1,KFLEV
  DO JL=KSTART,KPROF
    PRGEOPH(JL,JFLEV)=PRGEOPH(JL,JFLEV)+PRXP(JL,JFLEV,1)
    PRGEOPF(JL,JFLEV)=PRGEOPF(JL,JFLEV)+PRXP(JL,JFLEV,2)
    PRGEOPH(JL,JFLEV)=PRGEOPH(JL,JFLEV)+PRXP(JL,JFLEV,3)
    PRGEOPF(JL,JFLEV)=PRGEOPF(JL,JFLEV)+PRXP(JL,JFLEV,4)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPINITZA',1,ZHOOK_HANDLE)
END SUBROUTINE PPINITZA

