SUBROUTINE PPNOX(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & PRES,PLNPRES,LDBELS,LDBLES,&
 & PRXP,PRXPD,PTF,PO3F,PNOXF,PSOL,PAL,PCC,PTAUC,PCOD,&
 & PNO2)  

!**** *PPNOX* - POST-PROCESS NOx

!     PURPOSE.
!     --------
!       COMPUTES layer NO2.
!       COMPUTE LAYER NOX AND CONVERT INTO LAYER NO2.
!       ALSO EXTRAPOLATES layer NO2 IF SO REQUESTED.

!**   INTERFACE.
!     ----------
!        *CALL* *PPNOX(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                      PRES,PLNPRES,LDBELS,LDBLES,
!                      PRXP,PRXPD,PTF,PO3F,
!                      PNOXF,PSOL,PAL,PCC,PTAUC,PCOD,PNO2)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRES            (INPUT-C)
!                                    (SEE PPFLEV)
!        PRES(KPROMA,KLEVP)        - POST-PROCESSING LEVEL PRESSURES   (INPUT-C)
!        PLNPRES(KPROMA,KLEVP)     - LOG(PRES)                         (INPUT-C)
!        LDBELS(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE                    (INPUT-C)
!        LDBLES(KLEVP)             - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!        PTF(KPROMA,0:KFLEV)       - Temperature ON FULL INPUT LEVELS. (INPUT)
!        PO3F(KPROMA,0:KFLEV)      - Ozone ON FULL INPUT LEVELS.       (INPUT)
!        PNOXF(KPROMA,0:KFLEV)     - NOX ON FULL INPUT LEVEL           (INPUT)
!        PSOL(KPROMA)              - SOLAR ELEVATION                   (INPUT)
!        PAL(KPROMA)               - Albedo                            (INPUT)
!        PCC(KPROMA,KFLEV)         - Cloud cover                       (INPUT)
!        PTAUC(KPROMA,KFLEV)       - Optical depth per layer above level (INPUT)
!        PCOD(KPROMA)              - Total optical depth               (INPUT)


!        PNO2(KPROMA,0:KFLEV)       - POST-PROCESSED NO2                (OUTPUT)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  GPNOX  - COMPUTE layer NO2 ON HALF LEVELS
!     ----------  GPPWC  - COMPUTE PWC ON HALF LEVELS
!                 PPINTP - LINEAR INTERPOLATION

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        ERIK ANDERSSON, BASED ON PP-ROUTINES BY
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 92-06-08
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Dethof : 04-Jan-2007 Adapt for NOx operator
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRES(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3F(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNOXF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCC(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUC(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOD(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNO2(KPROMA,0:KFLEV) 
REAL(KIND=JPRB) :: ZPCO3(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZJNO2(KPROMA,KFLEV)

INTEGER(KIND=JPIM) :: JK

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnox.intfb.h"
#include "gppwc.intfb.h"
#include "grg_jno2_cloud.intfb.h"

!     ------------------------------------------------------------------

!*       1.    Compute NO2 on model levels at observation location
!              ----------------

IF (LHOOK) CALL DR_HOOK('PPNOX',0,ZHOOK_HANDLE)

!*       1   COMPUTE layer O3 ON MODEL HALF LEVELS for use in GRG_JNO2. ZPCO3=0 AT TOP OF MODEL.

CALL GPPWC(KPROMA,KSTART,KPROF,KFLEV,ZPCO3,PO3F(1,1),PRXP)

!*       2   COMPUTE photolysis frequency for NO2

!CALL GRG_JNO2(KPROMA,KSTART,KPROF,KFLEV,ZPCO3(1,1),PTF(1,1),PSOL,PAL,ZJNO2_test,PRXP)

CALL GRG_JNO2_CLOUD(KPROMA,KSTART,KPROF,KFLEV,ZPCO3(1,1),PTF(1,1),PSOL,PAL,&
 &PCC,PTAUC,PCOD,ZJNO2,PRXP)

!*       3 Convert model level NOX to NO2 

CALL GPNOX(KPROMA,KSTART,KPROF,KFLEV,PNO2(1,1),PTF(1,1),PO3F(1,1),PNOXF(1,1),PSOL,ZJNO2,PRXP)

DO JK=1,KPROMA
  PNO2(JK,0)=PNO2(JK,1)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPNOX',1,ZHOOK_HANDLE)
END SUBROUTINE PPNOX
