SUBROUTINE PPNOXAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & PRES,PLNPRES,LDBELS,LDBLES,&
 & PRXP,PRXPD,PTF,PO3F,PNOXF,PSOL,PAL,&
 & PNO2,&
 & PRXP5,PRXPD5,PTF5,PO3F5,PNOXF5,PCC5,PTAUC5,PCOD5)  

!**** *PPNOXAD*  - POST-PROCESS NOx, ADJOINT.

!     PURPOSE.
!     --------
!       COMPUTES layer NO2.
!       COMPUTE LAYER NOX AND CONVERT INTO LAYER NO2.
!       ALSO EXTRAPOLATES layer NO2 IF SO REQUESTED.

!**   INTERFACE.
!     ----------
!        *CALL* *PPNOXAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                        PRES,PLNPRES,LDBELS,LDBLES,
!                        PRXP,PRXPD,PTF,PO3F,PNOXF,PPCF5,PSOL,PAL,
!                        PNO2,
!                        PRXP5,PRXPD5,PTF5,PO3F5,PNOXF5,PCC5,PTAUC5,PCOD5)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRES            (INPUT-C)
!                                    (SEE PPFLEV)
!        PRES(KPROMA,KLEVP)        - POST-PROCESSING LEVEL PRESSURES   (INPUT-C)
!        PLNPRES(KPROMA,KLEVP)     - LOG(PRES)                        (INPUT-C)
!        LDBELS(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE                    (INPUT-C)
!        LDBLES(KLEVP)             - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!        PTF(KPROMA,0:KFLEV)       - TEMPERATURE PERT ON FULL
!                                    INPUT LEVELS.                     (INPUT)
!        PO3F(KPROMA,0:KFLEV)      - OZONE PERT ON FULL
!                                    INPUT LEVELS.                     (INPUT)
!        PNOXF(KPROMA,0:KFLEV)     - NOX PERT ON FULL INPUT LEVELS.    (INPUT)
!        PSOL(KPROMA)              - SOLAR ELEVATION                   (INPUT)
!        PAL(KPROMA)               - Albedo                            (INPUT)
!        PNO2(KPROMA,0:KFLEV)       - POST-PROCESSED NO2                (OUTPUT)
!        PRXP5(KPROMA,0:KFLEV,NPPM)- HALF,FULL AND LN HALF LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT-T)
!        PRXPD5(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))          (INPUT-T)
!        PTF5(KPROMA,0:KFLEV)      - TEMPERATURE ON FULL
!                                    INPUT LEVELS.                     (INPUT-T)
!        PO3F5(KPROMA,0:KFLEV)     - OZONE ON FULL
!                                    INPUT LEVELS.                     (INPUT-T)
!        PNOXF5(KPROMA,0:KFLEV)    - NOX ON FULL INPUT LEVELS.          (INPUT-T)
!        PCC5(KPROMA,KFLEV)        - Cloud cover                       (INPUT)
!        PTAUC5(KPROMA,KFLEV)      - Optical depth per layer above level (INPUT)
!        PCOD5(KPROMA)             - Total optical depth               (INPUT)


!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  GPNOX  - COMPUTE COLUMN NO2 ON HALF LEVELS
!     ----------  GPPWC  - COMPUTE PWC ON HALF LEVELS 
!                 PPINTP - LINEAR INTERPOLATION

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        ERIK ANDERSSON, BASED ON PP-ROUTINES BY
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 92-06-08
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Dethof : 04-Jan-2007 Adapt for NOx operator
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRES(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO3F(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNOXF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNO2(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3F5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNOXF5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCC5(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUC5(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOD5(KPROMA)

REAL(KIND=JPRB) :: ZPCO3(KPROMA,0:KFLEV),ZPCO35(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZJNO2(KPROMA,KFLEV),ZJNO25(KPROMA,KFLEV)

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnoxad.intfb.h"
#include "gppwc.intfb.h" 
#include "gppwcad.intfb.h"
#include "grg_jno2_cloud.intfb.h"
#include "grg_jno2_cloudad.intfb.h"


IF (LHOOK) CALL DR_HOOK('PPNOXAD',0,ZHOOK_HANDLE)

!*    ------------------------------------------------------------------
!*       0.1   COMPUTE GENERALIZED TRAJECTORY

!*       0.2   Compute NO2 on model levels at observation location

!*       0.2.1   COMPUTE layer O3 ON MODEL HALF LEVELS for use in GRG_JNO2. ZPCO3=0 AT TOP OF MODEL.

CALL GPPWC(KPROMA,KSTART,KPROF,KFLEV,ZPCO35,PO3F5(1,1),PRXP5)

!*       0.2.2   Compute photolysis frequency for NO2
CALL GRG_JNO2_CLOUD(KPROMA,KSTART,KPROF,KFLEV,ZPCO35(1,1),PTF5(1,1),PSOL,PAL,&
 & PCC5,PTAUC5,PCOD5,ZJNO25,PRXP5)

!*    ------------------------------------------------------------------

!*       1.    Compute NO2 on model levels at observation location
!              ----------------------------------------------------

DO JLEV=0,KFLEV
  DO JL=KSTART,KPROF
    ZPCO3(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO

ZJNO2(:,:)=0.0_JPRB

!*       1.1.3 Convert model level NOX to NO2
CALL GPNOXAD(KPROMA,KSTART,KPROF,KFLEV,PNO2, &
 & PTF(1,1),PO3F(1,1),PNOXF(1,1),PSOL,ZJNO2,PRXP,&
 & PTF5(1,1),PO3F5(1,1),PNOXF5(1,1),ZJNO25,PRXP5)

!*       1.1.2 Compute photolysis frequency for NO2

CALL GRG_JNO2_CLOUDAD(KPROMA,KSTART,KPROF,KFLEV,ZPCO3(1,1),PTF(1,1),PSOL,PAL,ZJNO2,PRXP,&
 & ZPCO35(1,1),PTF5(1,1),PCC5,PTAUC5,PCOD5,PRXP5)

!*       1.1.1 COMPUTE layer O3 ON MODEL HALF LEVELS. ZPCO3=0 AT TOP OF MODEL.

CALL GPPWCAD(KPROMA,KSTART,KPROF,KFLEV,ZPCO3,PO3F(1,1),PRXP,PO3F5(1,1),PRXP5)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPNOXAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPNOXAD
