SUBROUTINE PPOBSA(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,PRESO,POROG,&
 & PRESH5,PRESF5,PXP5,PXPD5,&
 & PUF5,PVF5,PTF5,PQF5,PO3F5,&
 & PCLWF5,PCIWF5,PCCF5,PTSTAR5,PT05,PLNPR5,PALPH5,&
 & KDPP,PXPP,YDGEMSPROF5,PNO25)

!**** PPOBSA - POST-PROCESSING

!     D. VASILJEVIC          ECMWF           91-01-03

!     PURPOSE
!     -------

!        FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBSERVED.

!**   INTERFACE
!     ---------

!        CALL PPOBSA(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,PRESO,POROG,
!    I               PRESH5,PRESF5,PXP5,PXPD5,PUF5,PVF5,PTF5,PQF5,PO3F5,
!    I               PCLWF5,PCIWF5,PCCF5,
!    I               PTSTAR5,PT05,PLNPR5,PALPH5,
!    O               KDPP,PXPP,YDGEMSPROF5,PNO25)
!     WHERE KDLEN   : FIRST DIMENSION (Observations)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS
!           KDCOUNT : SECOND DIMENSION (Data)
!           CDPAR   : PARAMETER FOR POST-PROCESSING
!           KOBSTYPE: OBS. TYPE
!           KCOUNT  : COUNTER FOR THE PARTICULAR OBS. PARAMETER
!           PRESO   : OBSERVED PRESSURES
!           POROG   : OROGRAPHY
!           PRESH5  : HALF LEVEL PRESSURES
!           PRESF5  : FULL LEVEL PRESSURES
!           PXP5    : INTERMEDIATE ARRAY
!           PXPD5   : ..................
!           PUF5    : U AT MODEL FULL LEVELS
!           PVF5    : V AT MODEL FULL LEVELS
!           PTF5    : T AT MODEL FULL LEVELS
!           PQF5    : Q AT MODEL FULL LEVELS
!           PO3F5   : OZON AT MODEL FULL LEVELS
!           PCLWF5  : CLOUD LIQUID WATER AT MODEL FULL LEVELS
!           PCIWF5  : CLOUD ICE AT MODEL FULL LEVELS
!           PCCF5   : CLOUD COVER AT MODEL FULL LEVELS
!           PTSTAR5 : T*
!           PT05    : T0
!           PLNPR5  : LOG. OF THE RATIO OF PRESSURE
!           PALPH5  : COEFFICIENTS OF THE HYDROSTATICS
!           KDPP    : Number of fields, dimension of PXPP.
!           PXPP    : POST-PROCESSED VARIABLES
!           YDGEMSPROF5   : GEMS PROFILES (OPTIONAL)
!           PNO25   : NO2 (optional)  

!        PPOBSA IS CALLED FROM HOP

!**   METHOD
!     ------

!        CALL TO AN APPROPIRIATE POST-PROCESSING ROUTINE

!**   EXTERNALS
!     ---------

!        PPFLEV  - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES
!        PPUV    - U,V POST-PROCESSING
!        PPT     - T         "
!        PPRH    - RH        "
!        PPRQ    - Q         "
!        PPPWC   - PWC       "
!        PPGEOP  - Z         "
!        GPRCP   - COMPUTE R, CP AND KAPPA

!**   MODIFICATIONS
!     -------------
!     910103 D.VASILJEVIC ORIGINAL
!     E.Andersson 92-06-22 : Introduce PWC
!     VC          96-01    : Cleaning
!     E.Andersson 96-02-07 : Remove 'V'-option
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Holm      98-05-08 : Introduce integrated ozone option (TO3)
!     E.Andersson 98-06-11 : Pass dimension as argument
!     F.Chevallier00-03    : Install cloud switch 'TVC'
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     J.M. Piriou 04-02    : Install cloud switch 'TQL'
!     N.Bormann   04-12-23 : New CDPAR-flavour 'RTL' for limb radiances
!     M.Hamrud  15-Jan-2006  Revised GPRCP
!     D.Tan       06-03-13 : Introduce HLoS (horizontal line-of-sight wind component)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMLUN   , ONLY : NULERR
USE GOMS_MIX     , ONLY : LGOMUA,YGOMV
USE YOMCOCTP , ONLY : NSATEM, NSCATT
USE TYPE_GEMS_PROFILES
USE YOM_YGFL , ONLY : NGRG,YGRG_NL

!     ------------------------------------------------------------------
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSTYPE
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESO(KDLEN,KDCOUNT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO3F5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCIWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCCF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXPP(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB)   ,INTENT(IN), OPTIONAL:: PNO25(KDLEN,0:NFLEVG)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVB (:,:,:)
LOGICAL, ALLOCATABLE :: LLBELO(:,:),LLBLOW(:),LLBELS(:,:),LLBLES(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZLNPRESO(:,:)
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG) ::ZR05
INTEGER(KIND=JPIM) :: ILEN,IMXCNT,INPP,JLEN,IND_NOX
INTEGER(KIND=JPIM) :: JLEVP
LOGICAL :: LLEXTR,LLU,LLT,LLQ,LLO3,LLH,LLZ,LLPWC,LLTO3,LL_CHKGOM,LLGHG
LOGICAL :: LLCC,LLCLW,LLCIW,LLTCC,LLNO2
LOGICAL, DIMENSION(NGRG) :: LLGRG
REAL(KIND=JPRB) :: ZRHMAX,ZRHMIN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB), ALLOCATABLE :: ZGEMSMODEL(:,:),ZGEMSRTTOV(:,:)
TYPE (TYPE_GEMSPROF), INTENT(INOUT), OPTIONAL :: YDGEMSPROF5(KDLEN,NGEMS)
INTEGER(KIND=JPIM) :: JGEMS,JGRG

#include "abor1.intfb.h"
#include "gprcp.intfb.h"
#include "ppcc.intfb.h"
#include "ppclw.intfb.h"
#include "ppflev.intfb.h"
#include "ppgeop.intfb.h"
#include "pppwc.intfb.h"
#include "ppq.intfb.h"
#include "pprh.intfb.h"
#include "ppt.intfb.h"
#include "pptcc.intfb.h"
#include "ppuv.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPOBSA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*    1.  PREPARATION
!*    1.1 MAX. NO. OF LEVELS

ILEN   = KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0) GOTO 9

LLU=  CDPAR == 'U'   .OR. CDPAR == 'FF'  .OR. CDPAR == 'BG1' &
 &                   .OR. CDPAR == 'HLS'
LLT=  CDPAR == 'T'   .OR. CDPAR == 'BG1' .OR. CDPAR == 'TV1' &
 &                   .OR. CDPAR == 'RTL'
LLQ=  CDPAR == 'Q'   .OR. CDPAR == 'BG1' .OR. CDPAR == 'TV1' &
 &                   .OR. CDPAR == 'RTL'
LLO3= CDPAR == 'O3'  .OR. CDPAR == 'BG1' .OR. CDPAR == 'TV1' &
 &                   .OR. CDPAR == 'RTL'
LLTO3=CDPAR == 'TO3' .OR. CDPAR == 'BG1'
LLGHG=CDPAR == 'BG1' .OR. CDPAR == 'TV1'
LLH=  CDPAR == 'H'   .OR. CDPAR == 'LH'
LLZ=  CDPAR == 'Z'   .OR. CDPAR == 'DZ'  .OR. CDPAR == 'BG1' &
 &                   .OR. CDPAR == 'RTL'
LLPWC=CDPAR == 'PWC' .OR. CDPAR == 'BG1'
LLCC =CDPAR == 'CC'
LLCLW=CDPAR == 'CLW'
LLCIW=CDPAR == 'CIW'
LLTCC=CDPAR == 'TCC'
IF ( CDPAR == 'TVC' ) THEN
  LLQ = .true.
  LLT = .true.
  LLO3 = .true.
  LLCC = .true.
  LLCLW = .true.
  LLCIW = .true.
ELSEIF ( CDPAR == 'TQL' ) THEN
  LLQ = .true.
  LLT = .true.
  LLO3 = .true.
  LLCC = .false.
  LLCLW = .true.
  LLCIW = .false.
ENDIF
DO JGRG=1,NGRG
  LLGRG(JGRG)=CDPAR == TRIM(YGRG_NL(JGRG)%cname) .OR. CDPAR == 'BG1'
ENDDO

LLNO2=.FALSE.
IND_NOX=-1
IF(PRESENT(PNO25)) THEN
  LLNO2=.TRUE.
  DO JGRG=1,NGRG
    IF(TRIM(YGRG_NL(JGRG)%cname) =='NOX') IND_NOX=JGRG
    IF (IND_NOX==-1) THEN
      WRITE(NULERR,*) 'PPOBSA: PROBLEM WITH NOX OPERATOR. NOX FIELD NEEDED BUT DOES NOT EXIST.'
      CALL ABOR1('PPOBSA: PROBLEM WITH NOX OPERATOR. NOX FIELD DOES NOT EXIST')
    ENDIF
  ENDDO
ENDIF


LL_CHKGOM=(KOBSTYPE /= NSCATT) .AND.&
 &((KOBSTYPE /= NSATEM .AND. CDPAR /= 'BG1' .AND. CDPAR /= 'TV1' &
 & .AND. CDPAR /= 'TVC'.AND. CDPAR /= 'TQL'  ) .OR. &
 &(KOBSTYPE == NSATEM .AND. CDPAR /= 'BG1'))
 

IF(.NOT.ANY((/LLU,LLT,LLQ,LLO3,LLH,LLZ,LLPWC,LLTO3,&
   & LLCC,LLCLW,LLCIW,LLTCC,LLGHG,LLGRG/))) THEN  
  WRITE(NULERR,'('' *** REQUESTED PARAMETER='',A)') CDPAR
  CALL ABOR1('PPOBSA: WRONG REQUESTED PARAMETER')
ENDIF

INPP=0

!*    1.2 FIND OUT PRESSURE LEVEL BELOW SPECIFIED (OBSERVED) PRESSURES

ALLOCATE(ILEVB(KDLEN,IMXCNT,NPPM))
ALLOCATE(LLBELO(KDLEN,IMXCNT))
ALLOCATE(LLBELS(KDLEN,IMXCNT))
ALLOCATE(LLBLOW(IMXCNT))
ALLOCATE(LLBLES(IMXCNT))
CALL PPFLEV(KDLEN,1,ILEN,NFLEVG,IMXCNT,&
 & PRESO,PRESH5,PRESF5,&
 & ILEVB,&
 & LLBELO,LLBELS,LLBLOW,LLBLES)  

!*    1.3 LN(P)

IF (LLZ .OR. LLPWC .OR. LLU .OR. LLT .OR. LLTO3 .OR. LLGHG .OR. ANY(LLGRG) ) THEN
  ALLOCATE(ZLNPRESO(KDLEN,IMXCNT))
  DO JLEVP=1,IMXCNT
    IF(CDPAR == 'TV1' .OR. CDPAR == 'TVC' .OR. CDPAR == 'TQL') THEN
      ZLNPRESO(1:ILEN,JLEVP) = LOG(PRESO(1,JLEVP))
    ELSE
      ZLNPRESO(1:ILEN,JLEVP) = LOG(PRESO(1:ILEN,JLEVP))
    ENDIF
  ENDDO
ENDIF


!*    2.  U AND V COMPONENTS

IF(LLU) THEN
  IF(LL_CHKGOM .AND. .NOT.(LGOMUA(KOBSTYPE,YGOMV%MU) .AND. LGOMUA(KOBSTYPE,YGOMV%MV)))THEN
    CALL ABORX('PPOBSA: U REQUIRES U AND V GOM ARRAY') 
  ENDIF
  IF(INPP+2 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, UV')
  INPP=INPP+2

  CALL PPUV(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,LLBELO,&
   & LLBLOW,ZLNPRESO,&
   & PXP5,PXPD5,PUF5,PVF5,&
   & PXPP(1,1,INPP-1),PXPP(1,1,INPP))  
ENDIF

!*    4.  TEMPERATURE

IF(LLT) THEN
  IF(LL_CHKGOM .AND. .NOT.(LGOMUA(KOBSTYPE,YGOMV%MT) .AND. LGOMUA(KOBSTYPE,YGOMV%MQ)))&
   & CALL ABORX('PPOBSA: T REQUIRES T AND Q GOM ARRAY')  
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, T')
  INPP=INPP+1

  CALL GPRCP(KDLEN,1,ILEN,NFLEVG,PQ=PQF5(1,1),PR=ZR05)
  LLEXTR = .TRUE.
  CALL PPT(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELO,LLBELS,LLBLOW,LLBLES,LLEXTR,POROG,&
   & PXP5,PXPD5,PTSTAR5,PT05,ZR05,PTF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    5.  HUMIDITY
!*    5.1 REL. HUMIDITY

IF(LLH) THEN
  IF(LL_CHKGOM .AND. .NOT. LGOMUA(KOBSTYPE,YGOMV%MQ)) CALL ABORX('PPOBSA: H REQUIRES Q GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, RH')
  INPP=INPP+1

  ZRHMAX=1.2_JPRB
  ZRHMIN=-0.2_JPRB
  CALL PPRH(.TRUE.,KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBLOW,ZRHMAX,ZRHMIN,&
   & PXP5,PXPD5,PQF5,PTF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    5.2 Q

IF(LLQ) THEN
  IF(LL_CHKGOM .AND. .NOT. LGOMUA(KOBSTYPE,YGOMV%MQ)) CALL ABORX('PPOBSA: Q REQUIRES Q GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, Q')
  INPP=INPP+1

  CALL PPQ(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBLOW,&
   & PXP5,PXPD5,PQF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    5.3 HUMIDITY, PWC. GIVES PWC FROM P=PRESO TO P=0.

IF(LLPWC) THEN
  IF(LL_CHKGOM .AND. .NOT. LGOMUA(KOBSTYPE,YGOMV%MQ)) CALL ABORX('PPOBSA: PWC REQUIRES Q GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, PWC')
  INPP=INPP+1

  CALL PPPWC(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELS,LLBLES,&
   & PXP5,PXPD5,PQF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    6.  GEOPOTENTIAL

IF(LLZ) THEN
  IF(LL_CHKGOM .AND. .NOT.(LGOMUA(KOBSTYPE,YGOMV%MT) .AND. LGOMUA(KOBSTYPE,YGOMV%MQ)))&
   & CALL ABORX('PPOBSA: Z REQUIRES T AND Q GOM')  
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, Z')
  INPP=INPP+1

  CALL GPRCP(KDLEN,1,ILEN,NFLEVG,PQ=PQF5(1,1),PR=ZR05)
  LLEXTR = .TRUE.
  CALL PPGEOP(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELO,LLBELS,LLBLOW,LLBLES,LLEXTR,POROG,&
   & PXP5,PXPD5,PTSTAR5,ZR05,PLNPR5,PALPH5,PTF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    7.  OZON 
!*    7.1 INTERPOLATED OZONE
!*        For now, use PPQ awaiting a dedicated routine PPO3

IF( (LLO3 .AND. .NOT. (CDPAR == 'TV1' .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MO3)) ) &
          &      .OR. (CDPAR == 'TVC' .OR. CDPAR == 'TQL') ) THEN
  IF (CDPAR /= 'TVC' .AND. CDPAR /= 'TQL') THEN
    IF(LL_CHKGOM .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MO3)) CALL ABORX('PPOBSA: O3 REQUIRES O3 GOM')
  ENDIF
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, O3')
  INPP=INPP+1

  CALL PPQ(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBLOW,&
   & PXP5,PXPD5,PO3F5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    7.2 INTEGRATED OZONE
!*        For now, use PPPWC awaiting a dedicated routine PPTO3

IF(LLTO3) THEN
  IF(LL_CHKGOM .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MO3)) CALL ABORX('PPOBSA: TO3 REQUIRES O3 GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, TO3')
  INPP=INPP+1

  CALL PPPWC(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELS,LLBLES,&
   & PXP5,PXPD5,PO3F5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    8.  CLOUDS
!*    8.1 CLOUD COVER

IF(LLCC) THEN
  IF(LL_CHKGOM .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MCC)) CALL ABORX('PPOBSA: CC REQUIRES CC GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, CC')
  INPP=INPP+1

  CALL PPCC(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PCCF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    8.2 CLOUD LIQUID WATER

IF(LLCLW) THEN
  IF(LL_CHKGOM .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MCLW)) CALL ABORX('PPOBSA: CLW REQUIRES CLW GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, CLW')
  INPP=INPP+1

  CALL PPCLW(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PCLWF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    8.3 CLOUD ICE (Use PPCLW for now)

IF(LLCIW) THEN
  IF(LL_CHKGOM .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MCIW)) CALL ABORX('PPOBSA: CIW REQUIRES CIW GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, CIW')
  INPP=INPP+1

  CALL PPCLW(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PCIWF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    8.4 TOTAL CLOUD COVER

IF(LLTCC) THEN
  IF(LL_CHKGOM .AND. .NOT.LGOMUA(KOBSTYPE,YGOMV%MCC)) CALL ABORX('PPOBSA: TCC REQUIRES CC GOM')
  IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, CIW')
  INPP=INPP+1

  CALL PPTCC(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,&
   & PCCF5,&
   & PXPP(1,1,INPP))  
ENDIF

!*    9.  GREENHOUSE GASES
!*        For now, use PPQ awaiting a dedicated routine PPO3

IF(PRESENT(YDGEMSPROF5)) THEN

IF( (LLGHG .AND. .NOT. (CDPAR == 'TV1' .AND. .NOT.ANY(LGOMUA(KOBSTYPE,YGOMV%MGHG(:)))) ) &
           &      .OR. (CDPAR == 'TVC' .OR. CDPAR == 'TQL') ) THEN
   IF (CDPAR /= 'TVC' .AND. CDPAR /= 'TQL') THEN
     IF(LL_CHKGOM .AND. .NOT.ANY(LGOMUA(KOBSTYPE,YGOMV%MGHG(:)))) CALL ABORX('PPOBSA: GHG REQUIRES GHG GOM')
   ENDIF

  DO JGEMS=1,NGEMS
    IF(YDGEMSPROF5(1,JGEMS)%LRTTOV) THEN
      ALLOCATE(ZGEMSMODEL(KDLEN,0:NFLEVG),ZGEMSRTTOV(KDLEN,IMXCNT))
      DO JLEN=1,ILEN
        ZGEMSMODEL(JLEN,1:NFLEVG)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(:)
        ZGEMSMODEL(JLEN,0)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)
      ENDDO
      CALL PPQ(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
       & PRESO,LLBELO,LLBLOW,&
       & PXP5,PXPD5,ZGEMSMODEL,&
       & ZGEMSRTTOV)  

      DO JLEN=1,ILEN
        YDGEMSPROF5(JLEN,JGEMS)%RTTOVPROF(:)=ZGEMSRTTOV(JLEN,:)
      ENDDO
      DEALLOCATE(ZGEMSMODEL,ZGEMSRTTOV)
    ENDIF
  ENDDO
ENDIF

!*    11. GRG reactive gases
!*        For now, use PPPWC awaiting a dedicated routine PPTO3

IF(ANY(LLGRG) .AND. .NOT. (CDPAR == 'BG1')) THEN

  JGRG=0
  DO JGEMS=1,NGEMS
    IF(YDGEMSPROF5(1,JGEMS)%CSPECIES=='GRG') THEN
      JGRG=JGRG+1
      IF(LLGRG(JGRG)) THEN
        IF(INPP+1 > KDPP) CALL ABORX('PPOBSA: KDPP EXCEEDED, GRG')
        INPP=INPP+1
        ALLOCATE(ZGEMSMODEL(KDLEN,0:NFLEVG))
        IF(LLNO2.AND.JGRG==IND_NOX) THEN
          DO JLEN=1,ILEN
            ZGEMSMODEL(JLEN,0:NFLEVG)= &
             & PNO25(JLEN,0:NFLEVG)
          ENDDO
        ELSE
          DO JLEN=1,ILEN
            ZGEMSMODEL(JLEN,1:NFLEVG)= &
             & YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1:NFLEVG)
            ZGEMSMODEL(JLEN,0)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)
          ENDDO
        ENDIF

        CALL PPPWC(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
         & PRESO,ZLNPRESO,LLBELS,LLBLES,&
         & PXP5,PXPD5,ZGEMSMODEL,&
         & PXPP(1,1,INPP))  
        DEALLOCATE(ZGEMSMODEL)

      ENDIF
    ENDIF
  ENDDO
ENDIF  !end llgrg loop

ENDIF  !end ydgemsprof5 loop

!*    10.  FINALLY, DEALLOCATE AND RETURN

DEALLOCATE(ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)
IF(ALLOCATED(ZLNPRESO)) DEALLOCATE(ZLNPRESO)

9 CONTINUE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPOBSA',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CONTAINS
SUBROUTINE ABORX(CDERROR)
CHARACTER (LEN = *),INTENT(IN) ::  CDERROR
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('PPOBSA:ABORX',0,ZHOOK_HANDLE)
WRITE(NULERR,*) 'KOBSTYPE = ',KOBSTYPE,' CDPAR=',CDPAR
WRITE(NULERR,*) 'INPP = ',INPP,' KDPP=',KDPP
CALL ABOR1(CDERROR)
IF (LHOOK) CALL DR_HOOK('PPOBSA:ABORX',1,ZHOOK_HANDLE)
END SUBROUTINE ABORX

!     ------------------------------------------------------------------
END SUBROUTINE PPOBSA
