SUBROUTINE PPOBSAC(KDLEN,KLEN,KDCOUNT,CDPAR,PINTH,KOBSTYPE,KCOUNT,&
 & PALTI,POROG,PRESH5,PRESF5,PGEOPH5,PGEOPF5,&
 & PCP5,PRF5,PUF5,PVF5,PTF5,PQF5,&
 & PTS5,PSST5,PWS5,PVEG5,PSN5,PSLM5,PZ05,&
 & KDPP,PXPP,PTCLS5,PHUCLS5)  
!****-------------------------------------------------------------------
!****  PPOBSAC : CANARI SURFACE POST-PROCESSING
!****            FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBS
!****  called from SURFACEO
!****-------------------------------------------------------------------

!     WHERE KDLEN   : FIRST DIMENSION (OBSERVATIONS)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS
!           KDCOUNT : SECOND DIMENSION (Data)
!           CDPAR   : PARAMETER FOR POST-PROCESSING
!           PINTH   : INTERPOLATION HEIGHT
!           KOBSTYPE: OBS. TYPE
!           KCOUNT  : COUNTER OF DATA PER OBS
!           PALTI   : OBS. OROGRAPHY
!           POROG   : MODEL OROGRAPHY
!           PRESH5  : HALF LEVEL PRESS. AT OBS. POINTS
!           PRESF5  : FULL LEVEL PRESS. AT OBS. POINTS
!           PGEOPH5 : HALF LEVEL GEOP. AT OBS. POINTS
!           PGEOPF5 : FULL LEVEL GEOP. AT OBS. POINTS
!           PCP5    : Cp
!           PRF5    : R
!           PUF5    : U VALUES (MODEL) AT OB. POINS
!           PVF5    : V VALUES (MODEL) AT OB. POINS
!           PTF5    : T VALUES (MODEL) AT OB. POINS
!           PQF5    : Q VALUES (MODEL) AT OB. POINS
!           PTS5    : SURFACE TEMPERATURE
!           PSST5   : SURFACE TEMPERATURE OVER SEA (duplicated obs)
!           PWS5    : SOIL WETNESS
!           PVEG5   : VEGETATION
!           PSN5    : SNOW DEPTH
!           PSLM5   : SEE/LAND MASK
!           PZ05    : ROUGHNESS LENGTH
!           KDPP    : Number of fields, dimension of PXPP.
!           PXPP    : POST-PROCESSED VALUES

!**   EXTERNALS : ACHMT  ACSOLW
!     ---------

!     MODIFIED
!     --------
!     L. Isaksen   96-02-20 : NFLEV --> NFLEVG
!     E. Andersson 97-11-14 : GOM-variables more flexible
!     E. Andersson 98-06-11 : Pass dimension as argument
!     V. Lorant    98-08-05 : ZPCLS in ACHMT call
!     D. Giard     99-03-15 : ACSOLW called instead of ACSOL
!     D. Puech     99-06-30 : Add KCOUNT and init for NSCATT
!     F. Bouyssel  98-11-25 : Simplified ro run with PREINTS
!     P. Moll      99-08-03 : implementation in cy21t1  
!     P. Moll      99/10/15 : Put Apache and ACHMT in cy22 (for MF)
!     R. El Khatib 00-08-21 : Turbulent gusts out of ACHMT
!     F. Taillefer 00/10/20 : Land/sea mask for surface obs rejection
!     R. El Khatib 01-08-07 : Pruning options
!     J.M. Piriou / J.F Geleyn  01-08-23 : New definition and use of the thermal mixing length.
!     M. Bellus  2003-04-24 : Change of arguments in ACHMT calling
!                             + removing LSOLV and some cleaning
!     M. Tudor   2003-10-21 : Change args in ACHMT, reintroduce LSOLV
!        M.Hamrud      01-Jul-2006 Revised surface fields
!     A. Alias   2007-10-11 : Change args in ACHMT, PRTI is introduced
!     C. Payan   2008-11    : Neutral Wind
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMDPHY  , ONLY : LDIRCLSMOD
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LCANARI
USE YOMCST   , ONLY : RG
USE YOMSTA   , ONLY : RDTDZ1
USE YOMOBS   , ONLY : LSLREJ   ,LSCATT_NEUTRAL
USE YOMANCS  , ONLY : RMDI
USE YOMPHY1  , ONLY : EWFC     ,GWFC
USE YOMPHY2  , ONLY : HTCLS    ,HVCLS
USE SURFACE_FIELDS_MIX, ONLY : YSD_VVD
USE YOMPHY   , ONLY : LSOLV    ,LFGEL
USE YOMCOCTP , ONLY : NSYNOP   ,NSCATT
USE YOMDB
USE GOMS_MIX     , ONLY : YGOMEC, LGOMUA, YGOMV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINTH(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSTYPE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALTI(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOPH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOPF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCP5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN),OPTIONAL    :: PTCLS5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(IN),OPTIONAL    :: PHUCLS5(KDLEN)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSST5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSN5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ05(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXPP(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB) :: ZDPHIT(KDLEN),                ZDPHIV(KDLEN)
REAL(KIND=JPRB) :: ZARG(KDLEN),                  ZHV(KDLEN)
REAL(KIND=JPRB) :: ZZ0H(KDLEN),                  ZWFC(KDLEN)
REAL(KIND=JPRB) :: ZGZ0R(KDLEN),                 ZNEIJG(KDLEN)
REAL(KIND=JPRB) :: ZNEIJV(KDLEN)
REAL(KIND=JPRB) :: ZNBVNO(KDLEN,0:NFLEVG)       
REAL(KIND=JPRB) :: ZFPLSH(KDLEN,0:NFLEVG),       ZFPLCH(KDLEN,0:NFLEVG)
REAL(KIND=JPRB) :: ZCD(KDLEN),                   ZCDN(KDLEN)
REAL(KIND=JPRB) :: ZCDROV(KDLEN),                ZCH(KDLEN)
REAL(KIND=JPRB) :: ZCHROV(KDLEN),                ZCPS(KDLEN)
REAL(KIND=JPRB) :: ZDQSTS(KDLEN),                ZGWDCS(KDLEN)
REAL(KIND=JPRB) :: ZGZ0(KDLEN),                  ZHQ(KDLEN)
REAL(KIND=JPRB) :: ZGZ0H(KDLEN)
REAL(KIND=JPRB) :: ZHU(KDLEN),                   ZNEIJ(KDLEN)
REAL(KIND=JPRB) :: ZQCLS(KDLEN),                 ZQS(KDLEN)
REAL(KIND=JPRB) :: ZQSATS(KDLEN),                ZRHCLS(KDLEN)
REAL(KIND=JPRB) :: ZRS(KDLEN), ZRTI(KDLEN),      ZSTAB(KDLEN)
REAL(KIND=JPRB) :: ZTCLS(KDLEN),                 ZUCLS(KDLEN)
REAL(KIND=JPRB) :: ZVCLS(KDLEN),                 ZVEG(KDLEN)
REAL(KIND=JPRB) :: ZUCLN(KDLEN),                 ZVCLN(KDLEN)
REAL(KIND=JPRB) :: ZXDROV(KDLEN),                ZXHROV(KDLEN)
REAL(KIND=JPRB) :: ZURAF(KDLEN),                 ZVRAF(KDLEN)
REAL(KIND=JPRB) :: ZPCLS(KDLEN),                 ZWSI(KDLEN)
REAL(KIND=JPRB) :: ZRHMAX,ZRHMIN,ZINCR(KDLEN),ZREJ(KDLEN),ZZARG,ZEPS,ZS1,ZS2
INTEGER(KIND=JPIM) :: JROF,IPOSM,IMXCNT,JC,ILEN
LOGICAL :: LLDCLS,LLDHMT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "achmt.intfb.h"

#include "openmp_obs.h"

!**---------------------------------------------------------------------
!**  1.  PREPARATION DU CALCUL DES PARAMETRES AUX HAUTEURS METEOS.
!**      --------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPOBSAC',0,ZHOOK_HANDLE)
ILEN=KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0 .AND. LHOOK) CALL DR_HOOK('PPOBSAC',1,ZHOOK_HANDLE)
IF(IMXCNT == 0) RETURN

IF (CDPAR == 'U10' .OR. CDPAR == 'FFS') THEN
  IF(KOBSTYPE /= NSCATT .AND. &
   & .NOT.(LGOMUA(KOBSTYPE,YGOMV%MU) .AND. LGOMUA(KOBSTYPE,YGOMV%Mv)))&
   & CALL ABOR1('ERROR PPOBSAC: U10 REQUIRES GOMU AND GOMV')  
ELSEIF (CDPAR == 'T2') THEN
  IF(.NOT.(LGOMUA(KOBSTYPE,YGOMV%MT) .AND. LGOMUA(KOBSTYPE,YGOMV%MQ)))&
   & CALL ABOR1('ERROR PPOBSAC: T2 REQUIRES GOMT AND GOMQ')  
ELSEIF (CDPAR == 'H2') THEN
  IF(.NOT.LGOMUA(KOBSTYPE,YGOMV%MQ))&
   & CALL ABOR1('ERROR PPOBSAC: H2 REQUIRES GOMQ')  
ELSEIF (CDPAR == 'TS') THEN
  IF(.NOT.(LGOMUA(KOBSTYPE,YGOMV%MT) .AND. LGOMUA(KOBSTYPE,YGOMV%MQ)))&
   & CALL ABOR1('ERROR PPOBSAC: TS REQUIRES GOMT AND GOMQ')  
ELSE
  WRITE(NULOUT,'('' *** REQUESTED PARAMETER='',A)') CDPAR
  CALL ABOR1('ERROR PPOBSAC: REQUESTED PARAMETER')
ENDIF

!  REMPLISSAGE DES TABLEAUX LOCAUX.

ZDPHIT(1:ILEN) = RG*HTCLS
ZDPHIV(1:ILEN) = RG*HVCLS

IF(KOBSTYPE == NSYNOP)THEN
  DO JROF = 1 , ILEN
    ZDPHIV(JROF)=PINTH(JROF)
  ENDDO
ENDIF 

ZEPS=1.E-1_JPRB
ZS1=0.5_JPRB
ZS2=1.5_JPRB

LLDCLS=.FALSE.
LLDHMT=.TRUE.

! Calculs preliminaires pour le schema sol-vegetation (WFC,HV)
IF ( LSOLV ) THEN
  DO JROF = 1 , ILEN
    IPOSM = ROBHDR(JROF,MDB_MAPOMM_AT_INDEX)
    ZARG(JROF) = YGOMEC(IPOSM)%ECARG
    ZHV(JROF)  = YGOMEC(IPOSM)%ECHV
    IF ((YSD_VVD%NUMFLDS >= 8).AND.LSOLV) ZZ0H(JROF) = YGOMEC(IPOSM)%ECZ0H
    IF (LFGEL) ZWSI(JROF) = YGOMEC(IPOSM)%ECSAB
    IF ( PSLM5(JROF) <= 0.5_JPRB ) THEN
      ZWFC(JROF) = 1.0_JPRB
    ELSE
      ZZARG = MAX(ZEPS,ZARG(JROF))
      ZWFC(JROF) = GWFC*(ZZARG**EWFC)
    ENDIF
  ENDDO
ENDIF

! Prise en compte du masque terre/mer pour le rejet des T2m et des Hu2m des
! observations terrestres entourees par 4 points de mer et des Ts entourees de terre.
ZREJ(:)=0.0_JPRB
IF ( CDPAR == 'H2' .OR. CDPAR == 'T2' .OR. CDPAR == 'TS' ) THEN
  IF ( LCANARI .AND. LSLREJ ) THEN
    DO JROF = 1 , ILEN
      IPOSM = ROBHDR(JROF,MDB_MAPOMM_AT_INDEX)
      ZREJ(JROF)=YGOMEC(IPOSM)%ECEMIS*0.01_JPRB
    ENDDO
  ENDIF
ENDIF

!**---------------------------------------------------------------------
!**  - 2 - Appel pour le calcul des parametres aux hauteurs meteo.
!**        ------------------------------------------------------

CALL ACHMT(1,ILEN,KDLEN,0,NFLEVG,YSD_VVD%NUMFLDS,&
 & PGEOPH5,PGEOPF5,PRESH5,PRESF5,PCP5,PQF5(1,1),PRF5,PTF5(1,1),&
 & PUF5(1,1),PVF5(1,1),ZFPLSH,ZFPLCH,ZDPHIT,ZDPHIV,PZ05,ZZ0H,ZGZ0R,&
 & ZHV,PSLM5,ZNEIJG,ZNEIJV,PSN5,PTS5,PVEG5,ZWFC,PWS5,ZWSI,&
 & LLDCLS,LLDHMT,&
 & ZNBVNO,ZCD,ZCDN,ZCDROV,ZCH,ZCHROV,ZCPS,ZDQSTS,ZGWDCS,&
 & ZGZ0,ZGZ0H,ZHQ,ZHU,ZNEIJ,ZQCLS,ZQS,ZQSATS,ZRHCLS,ZRS,ZRTI,&
 & ZSTAB,ZTCLS,ZUCLS,ZVCLS,ZUCLN,ZVCLN,ZPCLS,ZVEG,ZXDROV,ZXHROV,ZURAF,ZVRAF,&
 & 'ACHMT  ')  

IF (LDIRCLSMOD) THEN
   ZTCLS(1:ILEN)=PTCLS5(1:ILEN)
   ZRHCLS(1:ILEN)=PHUCLS5(1:ILEN)
ENDIF


!**---------------------------------------------------------------------
!**  - 3 - Stockage/initialisation des champs diagnostics
!**        necessaires a l'analyse.
!**        ----------------------------------------------

ZRHMAX=1.0_JPRB
ZRHMIN=0.0_JPRB

IF (CDPAR == 'H2') THEN
  PXPP(1:ILEN,1,1) = MAX(ZRHMIN,MIN(ZRHMAX,ZRHCLS(1:ILEN)))
  DO JC = 2,IMXCNT
    WHERE(KCOUNT(1:ILEN) >= JC) PXPP(1:ILEN,JC,1)=PXPP(1:ILEN,1,1)
  ENDDO
  WHERE(ZREJ(1:ILEN) > ZS2) PXPP(1:ILEN,1,1)=RMDI
ELSEIF (CDPAR == 'U10' .OR. CDPAR == 'FFS')     THEN
  IF(KOBSTYPE == NSCATT .AND. LSCATT_NEUTRAL) THEN
    PXPP(1:ILEN,1,1) = ZUCLN(1:ILEN)
    PXPP(1:ILEN,1,2) = ZVCLN(1:ILEN)
  ELSE
    PXPP(1:ILEN,1,1) = ZUCLS(1:ILEN)
    PXPP(1:ILEN,1,2) = ZVCLS(1:ILEN)
  ENDIF
  DO JC = 2,IMXCNT
    WHERE(KCOUNT(1:ILEN) >= JC) PXPP(1:ILEN,JC,1)=PXPP(1:ILEN,1,1)
    WHERE(KCOUNT(1:ILEN) >= JC) PXPP(1:ILEN,JC,2)=PXPP(1:ILEN,1,2)
  ENDDO
ELSEIF (CDPAR == 'T2') THEN
  ZINCR(1:ILEN) = RDTDZ1*(POROG(1:ILEN)-PALTI(1:ILEN))/RG
  WHERE(ABS(PALTI(1:ILEN)) > 90000._JPRB) ZINCR(1:ILEN)=0.0_JPRB
  PXPP(1:ILEN,1,1) = ZTCLS(1:ILEN)-ZINCR(1:ILEN)
  DO JC = 2,IMXCNT
    WHERE(KCOUNT(1:ILEN) >= JC) PXPP(1:ILEN,JC,1)=PXPP(1:ILEN,1,1)
  ENDDO
  WHERE(ZREJ(1:ILEN) > ZS2) PXPP(1:ILEN,1,1)=RMDI
ELSEIF (CDPAR == 'TS') THEN
!  cas des SST : l'altitude de l'obs est consideree a 0 metre.
  ZINCR(1:ILEN)=RDTDZ1*POROG(1:ILEN)/RG
  PXPP(1:ILEN,1,1) = PSST5(1:ILEN)-ZINCR(1:ILEN)
  DO JC = 2,IMXCNT
    WHERE(KCOUNT(1:ILEN) >= JC) PXPP(1:ILEN,JC,1)=PXPP(1:ILEN,1,1)
  ENDDO
  WHERE(ZREJ(1:ILEN) > ZS1) PXPP(1:ILEN,1,1)=RMDI
ELSE
  WRITE(NULOUT,'('' *** ERROR IN PPOBSAC'')')
  WRITE(NULOUT,'('' *** REQUESTED PARAMETER='',A)') CDPAR
ENDIF

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PPOBSAC',1,ZHOOK_HANDLE)
END SUBROUTINE PPOBSAC
