SUBROUTINE PPOBSATL(KDLEN,KLEN,KDCOUNT,CDPAR,KCOUNT,PRESO,POROG,&
 & PXP,PXPD,PUF,PVF,PTF,PQF,PO3F,&
 & PCLWF,PCIWF,PCCF,PTSTAR,PT0,PLNPR,PALPH,&
 & KDPP,PXPP,&
 & PRESH5,PRESF5,PXP5,PXPD5,PUF5,PVF5,PTF5,PQF5,PO3F5,&
 & PCLWF5,PCIWF5,PCCF5,PTSTAR5,PT05,PLNPR5,PALPH5,YDGEMSPROF,YDGEMSPROF5,&
 & PNO2,PNO25)

!**** PPOBSATL - POST-PROCESSING

!     D. VASILJEVIC          ECMWF           91-01-03
!     (TANGENT LINEAR VERSION - MIKE FISHER ECMWF)

!     PURPOSE
!     -------

!        FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBSERVED.

!**   INTERFACE
!     ---------

!        CALL PPOBSATL(KDLEN,KLEN,KDCOUNT,CDPAR,KCOUNT,PRESO,POROG,
!    I                 PXP,PXPD,PUF,PVF,PTF,PQF,PO3F,
!    I                 PCLWF,PCIWF,PCCF,PTSTAR,PT0,PLNPR,PALPH,
!    O                 KDPP,PXPP,
!    R                 PRESH5,PRESF5,PXP5,PXPD5,PUF5,PVF5,PTF5,PQF5,PO3F5,
!    R                 PCLWF5,PCIWF5,PCCF5,PTSTAR5,PT05,PLNPR5,PALPH5,
!                      YDGEMSPROF,YDGEMSPROF5,PNO2,PNO25)

!     WHERE KDLEN   : FIRST DIMENSION (Observations)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS
!           KDCOUNT : SECOND DIMENSION (Data)
!           CDPAR   : PARAMETER FOR POST-PROCESSING
!           KCOUNT  : COUNTER FOR THE PARTICULAR OBS. PARAMETER
!           PRESO   : OBSERVED PRESSURES
!           POROG   : OROGRAPHY
!           PRESH   : HALF LEVEL PRESSURES
!           PRESF   : FULL LEVEL PRESSURES
!           PXP     : INTERMEDIATE ARRAY
!           PXPD    : ..................
!           PUF     : U AT MODEL FULL LEVELS
!           PVF     : V AT MODEL FULL LEVELS
!           PTF     : T AT MODEL FULL LEVELS
!           PQF     : Q AT MODEL FULL LEVELS
!           PO3F    : O3 AT MODEL FULL LEVELS
!           PCLWF   : CLOUD LIQUID WATER AT MODEL FULL LEVELS
!           PCIWF   : CLOUD ICE AT MODEL FULL LEVELS
!           PCCF    : CLOUD COVER AT MODEL FULL LEVELS
!           PTSTAR  : T*
!           PT0     : T0
!           PLNPR   : LOG. OF THE RATIO OF PRESSURE
!           PALPH   : COEFFICIENTS OF THE HYDROSTATICS
!           KDPP    : Number of fields, dimension of PXPP.
!           PXPP    : POST-PROCESSED VARIABLES
!           YDGEMSPROF   : GEMS PROFILES (OPTIONAL)
!           PNO2    : NO2 (optional)
!           PNO25   : NO2 (optional)

!        PPOBSATL IS CALLED FROM HOPTL

!**   METHOD
!     ------

!        CALL TO AN APPROPIRIATE POST-PROCESSING ROUTINE

!**   EXTERNALS
!     ---------

!        PPFLEV   - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES
!        PPUVTL   - U,V POST-PROCESSING
!        PPTTL    - T         "
!        PPRHTL   - RH        "
!        PPRQTL   - Q         "
!        PPPWCTL  - PWC       "
!        PPGEOPTL - Z         "
!        GPRCPTL  - COMPUTE R, CP AND KAPPA

!**   MODIFICATIONS
!     -------------
!     E.Andersson 92-06-22 : Introduce PWC
!     E.Andersson 96-02-07 : Remove 'V'-option
!     FV971113 - Correct PXPD,PXPD5 dimensions
!     FV          97-04-27 :  Call to PPRHTL corrected
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Holm      98-05-08 : Introduce intrgrated ozone option (TO3)
!     E.Andersson 98-06-11 : Pass dimension as argument
!     E.Andersson 99-07-02 : Incorporate clouds
!     N.Bormann   04-12-23 : New CDPAR-flavour 'RTL' for limb radiances
!     M.Hamrud    15-Jan-2006  Revised GPRCP
!     D.Tan       02-Mar-2007  Introduce HLoS (horizontal line-of-sight wind component)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMLUN   , ONLY : NULERR
USE YOMCOCTP , ONLY : NSATEM
USE GOMS_MIX     , ONLY : LGOMUA ,YGOMV
USE TYPE_GEMS_PROFILES
USE YOM_YGFL , ONLY : NGRG,YGRG_NL
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT
CHARACTER(LEN=3)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESO(KDLEN,KDCOUNT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3F(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLWF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCIWF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT0(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KDLEN,NFLEVG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXPP(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PO3F5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCIWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCCF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN), OPTIONAL:: PNO2(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN), OPTIONAL:: PNO25(KDLEN,0:NFLEVG)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVB (:,:,:)
LOGICAL, ALLOCATABLE :: LLBELO(:,:),LLBLOW(:),LLBELS(:,:),LLBLES(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZLNPRESO(:,:)
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG) :: ZR05,ZR0
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG) ::ZPERT1,ZPERT2
INTEGER(KIND=JPIM) :: ILEN,IMXCNT,INPP,JLEN,IND_NOX
INTEGER(KIND=JPIM) :: JLEVP
LOGICAL :: LLEXTR,LLU,LLT,LLQ,LLO3,LLH,LLZ,LLPWC,LLTO3,LLGHG
LOGICAL :: LLCC,LLCLW,LLCIW,LLTCC,LLNO2
LOGICAL, DIMENSION(NGRG) :: LLGRG
REAL(KIND=JPRB) :: ZRHMAX,ZRHMIN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB), ALLOCATABLE :: ZGEMSMODEL(:,:),ZGEMSRTTOV(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGEMSMODEL5(:,:),ZGEMSRTTOV5(:,:)
TYPE (TYPE_GEMSPROF), INTENT(INOUT), OPTIONAL :: YDGEMSPROF(KDLEN,NGEMS)
TYPE (TYPE_GEMSPROF), INTENT(IN), OPTIONAL :: YDGEMSPROF5(KDLEN,NGEMS)
INTEGER(KIND=JPIM) :: JGEMS,JGRG

#include "abor1.intfb.h"
#include "gprcp.intfb.h"
#include "gprcptl.intfb.h"
#include "ppcctl.intfb.h"
#include "ppclwtl.intfb.h"
#include "ppflev.intfb.h"
#include "ppgeoptl.intfb.h"
#include "pppwctl.intfb.h"
#include "ppqtl.intfb.h"
#include "pprhtl.intfb.h"
#include "pptcctl.intfb.h"
#include "ppttl.intfb.h"
#include "ppuvtl.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPOBSATL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*    1.  PREPARATION
!*    1.1 MAX. NO. OF LEVELS

ILEN   = KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0) GOTO 9

LLU=  CDPAR == 'U'   .OR. CDPAR == 'FF'  .OR. CDPAR == 'BG1' &
 &                   .OR. CDPAR == 'HLS'
LLT=  CDPAR == 'T'   .OR. CDPAR == 'BG1' .OR. CDPAR == 'TV1' &
 &                   .OR. CDPAR == 'RTL'
LLQ=  CDPAR == 'Q'   .OR. CDPAR == 'BG1' .OR. CDPAR == 'TV1' &
 &                   .OR. CDPAR == 'RTL'
LLO3= CDPAR == 'O3'  .OR. CDPAR == 'BG1' .OR. CDPAR == 'TV1' &
 &                   .OR. CDPAR == 'RTL'
LLTO3=CDPAR == 'TO3' .OR. CDPAR == 'BG1'
LLGHG=CDPAR == 'BG1' .OR. CDPAR == 'TV1'
LLH=  CDPAR == 'H'   .OR. CDPAR == 'LH'  .OR. CDPAR == 'BG1'
LLZ=  CDPAR == 'Z'   .OR. CDPAR == 'DZ'  .OR. CDPAR == 'BG1' &
 &                   .OR. CDPAR == 'RTL'
LLPWC=CDPAR == 'PWC' .OR. CDPAR == 'BG1'
LLCC =CDPAR == 'CC'
LLCLW=CDPAR == 'CLW'
LLCIW=CDPAR == 'CIW'
LLTCC=CDPAR == 'TCC'

DO JGRG=1,NGRG
  LLGRG(JGRG)=CDPAR == TRIM(YGRG_NL(JGRG)%cname) .OR. CDPAR == 'BG1'
ENDDO


LLNO2=.FALSE.
IND_NOX=-1
IF(PRESENT(PNO25)) THEN
  LLNO2=.TRUE.
  DO JGRG=1,NGRG
    IF(TRIM(YGRG_NL(JGRG)%cname) =='NOX') IND_NOX=JGRG
    IF (IND_NOX==-1) THEN
      WRITE(NULERR,*) 'PPOBSATL: PROBLEM WITH NOX OPERATOR. NOX FIELD NEEDED BUT DOES NOT EXIST.'
      CALL ABOR1('PPOBSATL: PROBLEM WITH NOX OPERATOR. NOX FIELD DOES NOT EXIST')
    ENDIF
  ENDDO
ENDIF

IF(.NOT.ANY((/LLU,LLT,LLQ,LLO3,LLH,LLZ,LLPWC,LLTO3,&
   & LLCC,LLCLW,LLCIW,LLTCC,LLGHG,LLGRG/))) THEN  
  WRITE(NULERR,'('' *** REQUESTED PARAMETER='',A)') CDPAR
  CALL ABOR1('ERROR PPOBSATL: WRONG REQUESTED PARAMETER')
ENDIF

!*    1.2 FIND OUT PRESSURE LEVEL BELOW SPECIFIED (OBSERVED) PRESSURES

ALLOCATE(ILEVB(KDLEN,IMXCNT,NPPM))
ALLOCATE(LLBELO(KDLEN,IMXCNT))
ALLOCATE(LLBELS(KDLEN,IMXCNT))
ALLOCATE(LLBLOW(IMXCNT))
ALLOCATE(LLBLES(IMXCNT))
CALL PPFLEV(KDLEN,1,ILEN,NFLEVG,IMXCNT,&
 & PRESO,PRESH5,PRESF5,&
 & ILEVB,&
 & LLBELO,LLBELS,LLBLOW,LLBLES)  

!*    1.3 LN(P)

IF(LLZ .OR. LLPWC .OR. LLU .OR. LLT .OR. LLTO3 .OR. LLGHG .OR. ANY(LLGRG) ) THEN
  ALLOCATE(ZLNPRESO(KDLEN,IMXCNT))
  DO JLEVP=1,IMXCNT
    IF(CDPAR == 'TV1') THEN
      ZLNPRESO(1:ILEN,JLEVP) = LOG(PRESO(1,JLEVP))
    ELSE
      ZLNPRESO(1:ILEN,JLEVP) = LOG(PRESO(1:ILEN,JLEVP))
    ENDIF
  ENDDO
ENDIF

!*    1.5 ZERO OVERDIMENSIONED PART OF PERTURBATION ARRAYS

ZPERT1(1:ILEN,0)=0.0_JPRB
ZPERT2(1:ILEN,0)=0.0_JPRB
INPP=0

!*    2.  U AND V COMPONENTS

IF(LLU) THEN
  IF(INPP+2 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, UV')
  INPP=INPP+2

  ZPERT1(1:ILEN,1:NFLEVG)=PUF(1:ILEN,1:NFLEVG)
  ZPERT2(1:ILEN,1:NFLEVG)=PVF(1:ILEN,1:NFLEVG)
  CALL PPUVTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,LLBELO,&
   & LLBLOW,ZLNPRESO,&
   & PXP,PXPD,ZPERT1,ZPERT2,&
   & PXPP(1,1,INPP-1),PXPP(1,1,INPP),&
   & PXP5,PXPD5,PUF5,PVF5)  
ENDIF

!*    4.  TEMPERATURE

IF(LLT) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, T')
  INPP=INPP+1

  CALL GPRCP(KDLEN,1,ILEN,NFLEVG,PQ=PQF5(1,1),PR=ZR05)
  CALL GPRCPTL (KDLEN,1,ILEN,NFLEVG,PQ=PQF(1,1),PR=ZR0)
  ZPERT1(1:ILEN,1:NFLEVG)=PTF(1:ILEN,1:NFLEVG)
  LLEXTR = .TRUE.
  CALL PPTTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELO,LLBELS,LLBLOW,LLBLES,LLEXTR,POROG,&
   & PXP,PXPD,PTSTAR,PT0,ZR0,ZPERT1,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PTSTAR5,PT05,ZR05,PTF5)  
ENDIF

!*    5.  HUMIDITY
!*    5.1 REL. HUMIDITY

IF(LLH) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, RH')
  INPP=INPP+1

  ZRHMAX=1.2_JPRB
  ZRHMIN=-0.2_JPRB
  ZPERT1(1:ILEN,1:NFLEVG)=PTF(1:ILEN,1:NFLEVG)
  ZPERT2(1:ILEN,1:NFLEVG)=PQF(1:ILEN,1:NFLEVG)
  CALL PPRHTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBLOW,ZRHMAX,ZRHMIN,&
   & PXP,PXPD,ZPERT2,ZPERT1,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PTF5,PQF5)  
ENDIF

!*    5.2 Q

IF(LLQ) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, Q')
  INPP=INPP+1

  ZPERT2(1:ILEN,1:NFLEVG)=PQF(1:ILEN,1:NFLEVG)
  CALL PPQTL(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBLOW,&
   & PXP,PXPD,ZPERT2,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PQF5)  
ENDIF

!*    5.3 HUMIDITY, PWC. GIVES PWC FROM P=PRESO TO P=0.

IF(LLPWC) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, PWC')
  INPP=INPP+1

  ZPERT2(1:ILEN,1:NFLEVG)=PQF(1:ILEN,1:NFLEVG)
  CALL PPPWCTL(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELS,LLBLES,&
   & PXP,PXPD,ZPERT2,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PQF5)  
ENDIF

!*    6.  GEOPOTENTIAL

IF(LLZ) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, Z')
  INPP=INPP+1

  CALL GPRCP(KDLEN,1,ILEN,NFLEVG,PQ=PQF5(1,1),PR=ZR05)
  CALL GPRCPTL (KDLEN,1,ILEN,NFLEVG,PQ=PQF(1,1),PR=ZR0)
  LLEXTR = .TRUE.
  ZPERT1(1:ILEN,1:NFLEVG)=PTF(1:ILEN,1:NFLEVG)
  CALL PPGEOPTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & ZLNPRESO,LLBELO,LLBELS,LLBLOW,LLBLES,LLEXTR,POROG,&
   & PXP,PXPD,PTSTAR,ZR0,PLNPR,PALPH,ZPERT1,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PTSTAR5,ZR05,PLNPR5,PALPH5,PTF5)  
ENDIF

!*    7.  OZON
!*    7.1 INTERPOLATED OZONE
!*        For now, use PPQTL awaiting a dedicated routine PPO3TL

IF(LLO3 .AND. .NOT.(CDPAR == 'TV1' .AND. .NOT.LGOMUA(NSATEM,YGOMV%MO3))&
   & .AND. .NOT.(CDPAR == 'BG1' .AND. .NOT.LGOMUA(NSATEM,YGOMV%MO3))  ) THEN  
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, O3')
  INPP=INPP+1

  ZPERT2(1:ILEN,1:NFLEVG)=PO3F(1:ILEN,1:NFLEVG)
  CALL PPQTL(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBLOW,&
   & PXP,PXPD,ZPERT2,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PO3F5)  
ENDIF

!*    7.2 INTEGRATED OZONE
!*        For now, use PPPWCTL awaiting a dedicated routine PPTO3TL

IF(LLTO3 .AND. .NOT.(CDPAR == 'BG1' .AND. .NOT.LGOMUA(NSATEM,YGOMV%MO3)) ) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, TO3')
  INPP=INPP+1

  ZPERT2(1:ILEN,1:NFLEVG)=PO3F(1:ILEN,1:NFLEVG)
  CALL PPPWCTL(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,ZLNPRESO,LLBELS,LLBLES,&
   & PXP,PXPD,ZPERT2,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PO3F5)  
ENDIF

!*    8.  CLOUDS
!*    8.1 CLOUD COVER

IF(LLCC) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, CC')
  INPP=INPP+1

  ZPERT1(:,0)=0.0_JPRB
  ZPERT1(:,1:NFLEVG)=PCCF(:,:)
  CALL PPCCTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBELS,LLBLOW,&
   & PXP,PXPD,ZPERT1,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PCCF5)  
ENDIF

!*    8.2 CLOUD LIQUID WATER

IF(LLCLW) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, CLW')
  INPP=INPP+1

  ZPERT1(:,0)=0.0_JPRB
  ZPERT1(:,1:NFLEVG)=PCLWF(:,:)
  CALL PPCLWTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBELS,LLBLOW,&
   & PXP,PXPD,ZPERT1,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PCLWF5)  
ENDIF

!*    8.3 CLOUD ICE (Use PPCLWTL for now)

IF(LLCIW) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, CIW')
  INPP=INPP+1

  ZPERT1(:,0)=0.0_JPRB
  ZPERT1(:,1:NFLEVG)=PCIWF(:,:)
  CALL PPCLWTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
   & PRESO,LLBELO,LLBELS,LLBLOW,&
   & PXP,PXPD,ZPERT1,&
   & PXPP(1,1,INPP),&
   & PXP5,PXPD5,PCIWF5)  
ENDIF

!*    8.4 TOTAL CLOUD COVER

IF(LLTCC) THEN
  IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, CIW')
  INPP=INPP+1

  ZPERT1(:,0)=0.0_JPRB
  ZPERT1(:,1:NFLEVG)=PCCF(:,:)
  CALL PPTCCTL (KDLEN,1,ILEN,NFLEVG,IMXCNT,1,&
   & ZPERT1,&
   & PXPP(1,1,INPP),&
   & PCCF5)  
ENDIF

!*    9.  GREENHOUSE GASES
!*        For now, use PPQ awaiting a dedicated routine PPO3
IF(PRESENT(YDGEMSPROF5) .AND. PRESENT(YDGEMSPROF)) THEN

IF(LLGHG .AND. .NOT.(CDPAR == 'TV1' .AND. .NOT.ANY(LGOMUA(NSATEM,YGOMV%MGHG(:))))&
   & .AND. .NOT.(CDPAR == 'BG1' .AND. .NOT.ANY(LGOMUA(NSATEM,YGOMV%MGHG(:))))  ) THEN  

  DO JGEMS=1,NGEMS
    IF(YDGEMSPROF5(1,JGEMS)%LRTTOV) THEN
      ALLOCATE(ZGEMSMODEL(KDLEN,0:NFLEVG),ZGEMSRTTOV(KDLEN,IMXCNT))
      ALLOCATE(ZGEMSMODEL5(KDLEN,0:NFLEVG),ZGEMSRTTOV5(KDLEN,IMXCNT))
      DO JLEN=1,ILEN
        ZGEMSMODEL(JLEN,0)=0.0_JPRB
        ZGEMSMODEL(JLEN,1:NFLEVG)=YDGEMSPROF(JLEN,JGEMS)%MODELPROF(:)
        ZGEMSMODEL5(JLEN,0)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)
        ZGEMSMODEL5(JLEN,1:NFLEVG)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(:)
      ENDDO
     CALL PPQTL(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
      & PRESO,LLBELO,LLBLOW,&
      & PXP,PXPD,ZGEMSMODEL,&
      & ZGEMSRTTOV,&
      & PXP5,PXPD5,ZGEMSMODEL5)  

      DO JLEN=1,ILEN
        YDGEMSPROF(JLEN,JGEMS)%RTTOVPROF(:)=ZGEMSRTTOV(JLEN,:)
      ENDDO
      DEALLOCATE(ZGEMSMODEL,ZGEMSRTTOV)
      DEALLOCATE(ZGEMSMODEL5,ZGEMSRTTOV5)
    ENDIF
  ENDDO
ENDIF

!*    10. GRG reactive gases
!*        For now, use PPPWCTL awaiting a dedicated routine PPTO3TL

IF(ANY(LLGRG) .AND. .NOT.(CDPAR == 'BG1')) THEN

  JGRG=0
  DO JGEMS=1,NGEMS
    IF(YDGEMSPROF5(1,JGEMS)%CSPECIES=='GRG') THEN
      JGRG=JGRG+1
      IF(LLGRG(JGRG)) THEN
        IF(INPP+1 > KDPP) CALL ABOR1('PPOBSATL: KDPP EXCEEDED, GRG')
        INPP=INPP+1
        ALLOCATE(ZGEMSMODEL(KDLEN,0:NFLEVG))
        ALLOCATE(ZGEMSMODEL5(KDLEN,0:NFLEVG))
        IF(LLNO2.AND.JGRG==IND_NOX) THEN
          DO JLEN=1,ILEN
            ZGEMSMODEL(JLEN,0)=0.0_JPRB
            ZGEMSMODEL(JLEN,1:NFLEVG)=PNO2(JLEN,1:NFLEVG)
            ZGEMSMODEL5(JLEN,0:NFLEVG)=PNO25(JLEN,0:NFLEVG)
          ENDDO
        ELSE
          DO JLEN=1,ILEN
            ZGEMSMODEL(JLEN,0)=0.0_JPRB
            ZGEMSMODEL(JLEN,1:NFLEVG)=YDGEMSPROF(JLEN,JGEMS)%MODELPROF(:)
            ZGEMSMODEL5(JLEN,0)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)
            ZGEMSMODEL5(JLEN,1:NFLEVG)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(:)
          ENDDO
        ENDIF

        CALL PPPWCTL(KDLEN,1,ILEN,NFLEVG,IMXCNT,1,ILEVB,&
         & PRESO,ZLNPRESO,LLBELS,LLBLES,&
         & PXP,PXPD,ZGEMSMODEL,&
         & PXPP(1,1,INPP),&
         & PXP5,PXPD5,ZGEMSMODEL5)  
        DEALLOCATE(ZGEMSMODEL)
        DEALLOCATE(ZGEMSMODEL5)
      ENDIF
    ENDIF
  ENDDO
ENDIF

ENDIF
!*    10.  FINALLY, DEALLOCATE AND RETURN

DEALLOCATE(ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)
IF(ALLOCATED(ZLNPRESO)) DEALLOCATE(ZLNPRESO)

9 CONTINUE

IF (LHOOK) CALL DR_HOOK('PPOBSATL',1,ZHOOK_HANDLE)
END SUBROUTINE PPOBSATL
