SUBROUTINE PPOBSN(KDLEN,KLEN,KDCOUNT,CDPAR,KCOUNT,PALTI,POROG,&
 & PRESF5,PTF5,PTSF5,KDPP,PXPP)  
!****-------------------------------------------------------------------
!****   PPOBSN : CANARI SURFACE POST-PROCESSING FOR SNOW
!****   ------
!****   AUTHOR :      B. URBAN 97-02-20
!****-------------------------------------------------------------------
!***  PURPOSE : FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBSER
!***  -------
!**   INTERFACE
!**   ---------
!**   CALL PPOBSN(KDLEN,CDPAR,KSET,PALTI,POROG,
!**  &            PRESF5,PTF5,PTSF5,
!**  &            KDPP,PXPP)

!     WHERE KDLEN   : FIRST DIMENSION (OBSERVATIONS)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS
!           KDCOUNT : SECOND DIMENSION (Data)
!           CDPAR   : PARAMETER FOR POST-PROCESSING
!           PINTH   : INTERPOLATION HEIGHT
!           KCOUNT  : COUNTER OF DATA PER OBS
!           PALTI   : OBS. OROGRAPHY
!           POROG   : MODEL OROGRAPHY
!           PRESF5  : FULL LEVEL PRESSURES
!           PTF5    : T AT MODEL FULL LEVELS
!           PTSF5   : MODEL'S SURFACE TEMPERATURE
!           KDPP    : Number of fields, dimension of PXPP.
!           PXPP    : POST-PROCESSED VALUES

!     PPOBSN IS CALLED FOR SYNOP FROM SURFACEO

!**   METHOD :  TAKES INTO ACCOUNT THE DIFFERENCE BETWEEN REAL SURFACE
!     ------    AND MODEL SURFACE

!**   EXTERNALS : NONE
!     ---------

!**   MODIFIED :   
!     --------
!        P. Moll         99/10/15  Put Apache and ACHMT in cy22 (for MF)
!        L.Gaytandjieva  00/03/01  PXPP modif
!        L.Gaytandjieva  00/11/01  Add PTSF5 as input param., ZTOBS - modified,
!                                  PXPP rewritten as PXPP=ZPXPP1+ZPXPP2
!        R. El Khatib : 01-08-07 Pruning options
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : RG       ,RD       ,RTT
USE YOMCT0   , ONLY : LCANARI
USE YOMSTA   , ONLY : RDTDZ1
USE YOMANCS  , ONLY : RMDI
USE QACLIM   , ONLY : LCLIM
USE YOMDB
USE GOMS_MIX     , ONLY : YGOMS    ,YGOMCAN
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALTI(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSF5(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXPP(KDLEN,KDCOUNT,KDPP) 
!     ZSNS    : HAUTEUR DE NEIGE.
!     ZRRRR   : PRECIPITATIONS LIQUIDES
!     ZRRSN   : PRECIPITATIONS NEIGEUSES
!     ZCCLITS : CLIMATOLOGIE DE TEMPERATURE DE SURFACE
!     ZCCLISN : CLIMATOLOGIE DE NEIGE

REAL(KIND=JPRB) :: ZSNS(KDLEN)
REAL(KIND=JPRB) :: ZRRSN(KDLEN),        ZRRRR(KDLEN)
REAL(KIND=JPRB) :: ZCCLITS(KDLEN),      ZCCLISN(KDLEN)

INTEGER(KIND=JPIM) :: ILEN, IPOSM, IST, JROF, IMXCNT

REAL(KIND=JPRB) :: ZDTDP, ZDTDZ, ZTCLIMA, ZTOBS, ZPXPP1, ZPXPP2
REAL(KIND=JPRB) :: ZHOOK_HANDLE
#include "openmp_obs.h"

!**---------------------------------------------------------------------
!**  1.  REMPLISSAGE DES TABLEAUX LOCAUX
!**      -------------------------------

IF (LHOOK) CALL DR_HOOK('PPOBSN',0,ZHOOK_HANDLE)
IST = 1
ILEN=KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN))
IF(IMXCNT == 0 .AND. LHOOK) CALL DR_HOOK('PPOBSN',1,ZHOOK_HANDLE)
IF(IMXCNT == 0) RETURN

IF ( .NOT. LCANARI ) THEN
  DO JROF = IST , ILEN
    PXPP(JROF,1,1) = 0.0_JPRB
  ENDDO
  IF (LHOOK) CALL DR_HOOK('PPOBSN',1,ZHOOK_HANDLE)
  RETURN
ENDIF

DO JROF = IST , ILEN
  IPOSM = ROBHDR(JROF,MDB_MAPOMM_AT_INDEX)
  ZSNS(JROF)= YGOMS(IPOSM)%SN
  ZRRRR(JROF) = YGOMCAN(IPOSM)%ECPRWA
  ZRRSN(JROF) = YGOMCAN(IPOSM)%ECPRSN
  ZCCLITS(JROF) = YGOMCAN(IPOSM)%ECCLITS
  ZCCLISN(JROF) = YGOMCAN(IPOSM)%ECCLISN
ENDDO

!**---------------------------------------------------------------------
!**  - 2 - ANALYSE PROPREMENT DITE.
!**        -----------------------

DO JROF = IST , ILEN

  IF (CDPAR == 'SN') THEN
    IF (LCLIM) THEN

!  Calcul gradient de temperature pres de la surface
!  On suppose que le modele a bien 3 niveaux au moins...

      ZDTDP = (PTF5(JROF,NFLEVG) - PTF5(JROF,NFLEVG-2))&
       & / (PRESF5(JROF,NFLEVG) - PRESF5(JROF,NFLEVG-2))  
      ZDTDZ = -ZDTDP*RG/RD/PTF5(JROF,NFLEVG-1)

!  Calcul temperature a l'altitude obs

      IF(PALTI(JROF) == RMDI .OR. PALTI(JROF) > 90000._JPRB) THEN
        ZTOBS = PTF5(JROF,NFLEVG)
      ELSE
        ZTOBS = PTSF5(JROF)+ RDTDZ1*(PALTI(JROF)-POROG(JROF))/RG
!        ZTOBS = PTF5(JROF,NFLEVG)+ ZDTDZ*(PALTI(JROF)-POROG(JROF))/RG
      ENDIF

!  Calcul valeur climatologique de la hauteur de neige au point d'observation

      IF(PALTI(JROF) == RMDI .OR. PALTI(JROF) > 90000._JPRB) THEN
        ZTCLIMA = ZCCLITS(JROF)
      ELSE
        ZTCLIMA = ZCCLITS(JROF)+ RDTDZ1*(PALTI(JROF)-POROG(JROF))/RG
      ENDIF

!  Calcul valeur de la hauteur de neige au point d'observation

      ZPXPP1 = 0.5_JPRB * MAX(0.0_JPRB, RTT + 3.0_JPRB - ZTCLIMA)
      ZPXPP2 = (ZSNS(JROF)-ZCCLISN(JROF)) * MAX(0.0_JPRB,RTT+3.0_JPRB-ZTOBS) / 3.0_JPRB
      PXPP(JROF,1,1) = ZPXPP1 + ZPXPP2

    ELSE
      PXPP(JROF,1,1) = ZSNS(JROF)
    ENDIF

  ELSEIF (CDPAR == 'RR')     THEN

!  ZRRRR est le flux de precipitations non-neigeuses cumulees sur la meme
!  duree que l'observation
!  ZRRSN est le flux de precipitations neigeuses cumulees sur la meme
!  duree que l'observation

    PXPP(JROF,1,1) = ZRRRR(JROF) + ZRRSN(JROF)

!CCCCC  Pb a resoudre  (cf BU)
!               IF(ZRRPP(JOBS,JNOB) + ZRNPP(JOBS,JNOB).NE.0) THEN
!C S'il y a des RR totales dans le modele, la version modele de RR
!C total est la version modele du RR neigeux
!C incrementee par l'observation RR total ponderee par la proportion
!C de RR non neigeux du modele
!                  ZRRPP(JOBS,JNOB) = ZRNPP(JOBS,JNOB)/
!     S             (ZRRPP(JOBS,JNOB) + ZRNPP(JOBS,JNOB))*
!     S             ROBSAR(IPOSRR(JOBS,JNOB)+NCMVAR)+ZRRPP(JOBS,JNOB)
!                ELSE
!C S'il n'y a pas de RR totales dans le modele,
!C la version modele de RR total est egale a l'observation RR total
!                 ZRRPP(JOBS,JNOB) = ROBSAR(IPOSRR(JOBS,JNOB)+NCMVAR)
!              ENDIF
!CCCCC

  ELSE
    WRITE(NULOUT,'('' *** ERROR IN PPOBSN'')')
    WRITE(NULOUT,'('' *** REQUESTED PARAMETER='',A)') CDPAR
  ENDIF

ENDDO

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PPOBSN',1,ZHOOK_HANDLE)
END SUBROUTINE PPOBSN
