SUBROUTINE PPPMERTL(KPROMA,KSTART,KPROF,PRPRESS,POROG,PTSTAR,PT0,&
 & PMSLPPP,&
 & PRPRESS5,PTSTAR5,PT05)  

!**** *PPPMERTL* - POST-PROCESS MSL PRESSURE.

!     PURPOSE.
!     --------
!           COMPUTES MSL PRESSURE.

!**   INTERFACE.
!     ----------
!        *CALL* *PPPMERTL(KPROMA,KSTART,KPROF,PRPRESS,POROG,PTSTAR,PT0
!                                      ,PMSLPPP
!                                      ,PRPRESS5,PTSTAR5,PT05)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        PRPRESS(KPROMA)           - SURFACE PRESSURE                  (INPUT)
!        POROG(KPROMA)             - MODEL OROGRAPHY.                  (INPUT-C)
!        PTSTAR(KPROMA)            - SURFACE TEMPERATURE               (INPUT)
!        PT0(KPROMA)               - STANDARD SURFACE TEMPERATURE      (INPUT)

!        PMSLPPP(KPROMA)           - POST-PROCESSED MSL PRESSURE       (OUTPUT)

!        PRPRESS5(KPROMA)            - SURFACE PRESSURE                 (INPUT-T)
!        PTSTAR5(KPROMA)            - SURFACE TEMPERATURE              (INPUT-T)
!        PT05(KPROMA)               - STANDARD SURFACE TEMPERATURE     (INPUT-T)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  NONE
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        ERIK ANDERSSON, MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-10-08
!     E. A-son, J-F Geleyn 920409 Mod. T*, T0 and alpha below surface.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRESS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMSLPPP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRESS5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KPROMA) 
REAL(KIND=JPRB) :: ZTSTAR(KPROMA),  ZALPHA(KPROMA)
REAL(KIND=JPRB) :: ZTSTAR5(KPROMA), ZALPHA5(KPROMA)

INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZOROG, ZT0, ZT05, ZTX, ZTY, ZX, ZX5, ZY, ZY2, ZY25, ZY5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS MSL PRESSURE.
!              --------------------------

!*       1.1   COMPUTATION OF MODIFIED ALPHA AND TSTAR.

IF (LHOOK) CALL DR_HOOK('PPPMERTL',0,ZHOOK_HANDLE)
ZTX=290.5_JPRB
ZTY=255.0_JPRB
DO JL=KSTART,KPROF

  IF(PTSTAR5(JL) < ZTY) THEN
    ZTSTAR5(JL)=0.5_JPRB*(ZTY+PTSTAR5(JL))
    ZTSTAR(JL)=0.5_JPRB*PTSTAR(JL)
  ELSEIF(PTSTAR5(JL) < ZTX) THEN
    ZTSTAR5(JL)=PTSTAR5(JL)
    ZTSTAR(JL)=PTSTAR(JL)
  ELSE
    ZTSTAR5(JL)=0.5_JPRB*(ZTX+PTSTAR5(JL))
    ZTSTAR(JL)=0.5_JPRB*PTSTAR(JL)
  ENDIF

  IF(ZTX > ZTSTAR5(JL) .AND. PT05(JL) > ZTX) THEN
    ZT05=ZTX
    ZT0=0.0_JPRB
  ELSEIF(ZTX <= ZTSTAR5(JL) .AND. PT05(JL) > ZTSTAR5(JL)) THEN
    ZT05=ZTSTAR5(JL)
    ZT0=ZTSTAR(JL)
  ELSE
    ZT05=PT05(JL)
    ZT0=PT0(JL)
  ENDIF

  ZOROG=SIGN(MAX(1.0_JPRB,ABS(POROG(JL))),POROG(JL))
  ZALPHA5(JL)=RD*(ZT05-ZTSTAR5(JL))/ZOROG
  ZALPHA(JL)=RD*(ZT0-ZTSTAR(JL))/ZOROG
ENDDO

!*       1.2   COMPUTATION OF MSL PRESSURE.

DO JL=KSTART,KPROF
  IF (ABS(POROG(JL)) >= 0.001_JPRB) THEN
    ZX5=POROG(JL)/(RD*ZTSTAR5(JL))
    ZX=-POROG(JL)/(RD*ZTSTAR5(JL)*ZTSTAR5(JL))*ZTSTAR(JL)
    ZY5=ZALPHA5(JL)*ZX5
    ZY=ZALPHA5(JL)*ZX+ZX5*ZALPHA(JL)
    ZY25=ZY5*ZY5
    ZY2=2.0_JPRB*ZY5*ZY
    PMSLPPP(JL)=EXP(ZX5*(1.0_JPRB-0.5_JPRB*ZY5+1.0_JPRB/3._JPRB*ZY25))&
     & *(PRPRESS(JL)&
     & +PRPRESS5(JL)*((1.0_JPRB-0.5_JPRB*ZY5+1.0_JPRB/3._JPRB*ZY25)*ZX &
     & +(-0.5_JPRB*ZX5)*ZY &
     & +(1.0_JPRB/3._JPRB*ZX5)*ZY2 ))  
  ELSE
    PMSLPPP(JL)=PRPRESS(JL)
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPPMERTL',1,ZHOOK_HANDLE)
END SUBROUTINE PPPMERTL
