SUBROUTINE PPRESET(KTIME,LDNOPPFIL,CDYFILE)

!**** *PPRESET*  - Reset post-processing setup

!     PURPOSE.
!     --------
!        To reset the post-processing dimensions and arrays

!**   INTERFACE.
!     ----------
!       *CALL* *PPRESET*

!        EXPLICIT ARGUMENTS
!        --------------------
!        KTIME  : time in model

!        IMPLICIT ARGUMENTS
!        --------------------
!        See modules

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE* & DAVID DENT *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 99-10-14 
!        Modified 00-01-10 : D.Dent : remove CALL SUDIM
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMFPC   , ONLY : LFPCNT
USE YOMAUXB  , ONLY : GAUXBUF
USE GRIDPOINT_BUFFERS_MIX, ONLY : ALLOCATED_GRIDPOINT_BUFFER, &
 &                            DEALLOCATE_GRIDPOINT_BUFFER

IMPLICIT NONE

INTEGER(KIND=JPIB),INTENT(IN)    :: KTIME 
LOGICAL           ,INTENT(OUT)   :: LDNOPPFIL 
CHARACTER(LEN=*)  ,INTENT(INOUT) :: CDYFILE 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "dealfpos.intfb.h"
#include "ppreq.intfb.h"
#include "subfpos.intfb.h"
#include "sufpc.intfb.h"
#include "sufpd.intfb.h"

!*       1. RESET CONTROL VARIABLES
!           -----------------------

IF (LHOOK) CALL DR_HOOK('PPRESET',0,ZHOOK_HANDLE)
IF (LFPCNT) THEN
  CALL DEALFPOS
  CALL PPREQ(KTIME,LDNOPPFIL,CDYFILE)
  LDNOPPFIL=.TRUE.
  CALL SUFPC(CDYFILE)
  CALL SUFPD
  CALL SUBFPOS
ENDIF

!*       2. RESET GP BUFFERS DIMENSIONS
!           ---------------------------

IF(ALLOCATED_GRIDPOINT_BUFFER(GAUXBUF)) THEN
  CALL DEALLOCATE_GRIDPOINT_BUFFER(GAUXBUF)
ENDIF

IF (LHOOK) CALL DR_HOOK('PPRESET',1,ZHOOK_HANDLE)
END SUBROUTINE PPRESET
