SUBROUTINE PPRH2M(&
 & KPROMA,KIDIA,KFDIA,PRHMAX,PRHMIN,&
 & PAPF,PQ,PT,&
 & PES,&
 & PRH2)  

!**** *PPRH2M* - POST-PROCESS RELATIVE HUMIDITY AT 2 METERS

!     PURPOSE
!     -------
!          COMPUTE AND PERFORMS THE VERTICAL INTERPOLATION OF
!          2 METRES RELATIVE HUMIDITY GIVEN THE LOWER AND THE
!          SURFACE MODEL CO-ORDINATE LEVELS.

!**   INTERFACE
!     ---------
!       *CALL* *PRH2M(
!     C               KPROMA,KIDIA,KFDIA,NFLEVG,
!     I               ZPRESF5(1,NFLEVG),ZQF5(1,NFLEVG),ZTF5(1,NFLEVG),
!     O               PRH2)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KIDIA                     - INDEX OF START POINT              (INPUT-C)
!        KFDIA                     - INDEX OF END   POINT              (INPUT-C)
!        PRHMAX               - MAXIMUM RELATIVE HUMIDITY              (INPUT-C)
!        PRHMIN               - MINIMUM RELATIVE HUMIDITY              (INPUT-C)
!        PAPF(KPROMA)              - FULL PRESSURE  ON THE LOWER
!                                    LEVEL                             (INPUT)
!        PQ(KPROMA)                - SPECIFIC HUMIDITY ON THE LOWER
!                                    LEVEL.                            (INPUT)
!        PT(KPROMA)                - TEMPERATURE ON THE LOWER LEVEL    (INPUT)

!        PRH2(KPROMA)              - POST-PROCESSED 2 M RELATIVE
!                                    HUMIDITY                          (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        *GPRH*

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL   : 92-05-22
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMIN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPF(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PES(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRH2(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZEPHUM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gprh.intfb.h"

!*    1.   CONSTANT DEFINITIONS
!          --------------------

!     1.1    ZEPHUM IS A MINIMUM THRESHOLD FOR THE
!            RELATIVE HUMIDITY IN THE SURFACE LAYER
!            (TO PREVENT UNREALISTIC DEW POINT TEMPERATURE)

IF (LHOOK) CALL DR_HOOK('PPRH2M',0,ZHOOK_HANDLE)
ZEPHUM=5.E-2_JPRB

!*    1.2       COMPUTE RELATIVE HUMIDITY LOWER MODEL LAYER

CALL GPRH(.TRUE.,KPROMA,KIDIA,KFDIA, 1 ,PRHMAX,PRHMIN,PQ,PT,PAPF,PES,PRH2 )

!*    1.3       COMPUTE RELATIVE HUMIDITY IN SURFACE LAYER

DO JL=KIDIA,KFDIA
  PRH2(JL)=MAX(ZEPHUM,PRH2(JL))
ENDDO

!     ----------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPRH2M',1,ZHOOK_HANDLE)
END SUBROUTINE PPRH2M
