SUBROUTINE PPTCCAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,&
 & PCCF,&
 & PTCCPP,&
 & PCCF5)  

!**** *PPTCCAD* - POST-PROCESS TOTAL CLOUD COVER

!     PURPOSE.
!     --------

!        PERFORMS THE VERTICAL INTEGRATION OF CLOUD COVER

!**   INTERFACE.
!     ----------
!        *CALL* *PPTCCAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,
!                        PCCF,
!                        PTCCPP,
!                        PCCF5)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        PCCF(KPROMA,0:KFLEV)      - CLOUD COVER ON FULL INPUT LEVELS  (INPUT)

!        PTCCPP(KPROMA,KLEVP)      - POST-PROCESSED CLOUD COVER       (OUTPUT)
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PCCF5(KPROMA,0:KFLEV)      - CLOUD COVER ON FULL INPUT LEVELS  (INPUT)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST AND YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  PPINTP - LINEAR INTERPOLATION
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 93-01-12
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCCF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTCCPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB) :: ZTCCPP5(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZMAXTCC(KPROMA)
REAL(KIND=JPRB) :: ZMAXTCC5(KPROMA)

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZX, ZX5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS CLOUD COVER
!              ------------------------

!*       1.0   PRESET

IF (LHOOK) CALL DR_HOOK('PPTCCAD',0,ZHOOK_HANDLE)
DO JL  =KSTART,KPROF
  ZTCCPP5(JL,klevp) = 0.0_JPRB
  ZMAXTCC5(JL) = PCCF5(JL,1)
ENDDO

DO JLEV=2,KFLEV
  DO JL  =KSTART,KPROF
    IF(PCCF5(JL,JLEV-1) < 0.0_JPRB)         THEN
      ZX5         = ZTCCPP5(JL,klevp)
      ZTCCPP5 (JL,klevp) = ZX5 + (1.0_JPRB-ZX5)*ZMAXTCC5(JL)
      ZMAXTCC5(JL) = PCCF5(JL,JLEV)
    ELSE
      IF(ZMAXTCC5(JL) < PCCF5(JL,JLEV))           THEN
        ZMAXTCC5(JL) = PCCF5(JL,JLEV)
      ENDIF
    ENDIF
  ENDDO
ENDDO

DO JL  =KSTART,KPROF
  ZX5        = ZTCCPP5(JL,klevp)
  ZTCCPP5(JL,klevp) = ZX5 + (1.0_JPRB-ZX5)*ZMAXTCC5(JL)
ENDDO

!*       1.1   INTEGRATE CLOUD COVER

DO JL  =KSTART,KPROF
  ZX5          = ZTCCPP5(JL,klevp)

  ZX           = (1.0_JPRB-ZMAXTCC5(JL))*PTCCPP(JL,klevp)
  ZMAXTCC (JL) = (1.0_JPRB-ZX5)*PTCCPP(JL,klevp)
  PTCCPP  (JL,klevp) = PTCCPP(JL,klevp) + ZX
ENDDO

DO JLEV=2,KFLEV
  DO JL  =KSTART,KPROF
    IF(PCCF5(JL,JLEV-1) < 0.0_JPRB)         THEN
      PCCF(JL,JLEV)= PCCF(JL,JLEV) + ZMAXTCC(JL)
      ZX = (1.0_JPRB-ZMAXTCC5(JL))*PTCCPP(JL,klevp)
      ZMAXTCC (JL) = (1.0_JPRB-PCCF5(JL,KFLEV))*PTCCPP(JL,klevp)
      PTCCPP  (JL,klevp) = PTCCPP(JL,klevp) + ZX
    ELSE
      IF(ZMAXTCC5(JL) < PCCF5(JL,JLEV))           THEN
        PCCF(JL,JLEV) = PCCF(JL,JLEV) + ZMAXTCC(JL)
      ENDIF
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPTCCAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPTCCAD

