SUBROUTINE PPTCCTL (KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & PCCF,&
 & PTCCPP,&
 & PCCF5)  

!**** *PPTCC* - POST-PROCESS TOTAL CLOUD COVER

!     PURPOSE.
!     --------

!        PERFORMS THE VERTICAL INTEGRATION OF CLOUD COVER

!**   INTERFACE.
!     ----------
!        *CALL* *PPTCC(KPROMA,KSTART,KPROF,KFLEV,KLEVP,
!                      PCCF,
!                      PTCCPP,
!                      PCCF5)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        PCCF(KPROMA,0:KFLEV)      - CLOUD COVER ON FULL INPUT LEVELS  (INPUT)

!        PTCCPP(KPROMA,KLEVP)      - POST-PROCESSED CLOUD COVER       (OUTPUT)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST AND YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  PPINTP - LINEAR INTERPOLATION
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC  *ECMWF*
!       (Tangent Linear Version - Mike Fisher *ECMWF*)

!     MODIFICATIONS.
!     --------------
!        Ad Stoffelen   Friday 13/08/93  Debugging!
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTCCPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB) ::Z_PTCCPP5(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZMAXTCC(KPROMA),ZMAXTCC5(KPROMA)

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZX, ZX5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS CLOUD COVER
!              ------------------------

!*       1.0   PRESET

IF (LHOOK) CALL DR_HOOK('PPTCCTL',0,ZHOOK_HANDLE)
IF( KLEVP /= KLOLEV )THEN
  WRITE(NULOUT,*) ' PPTCC: NOT ONE TCC PER PROFILE!', KLEVP, KLOLEV
  CALL ABOR1('SUBROUTINE PPTCCTL')
ENDIF
DO JL = KSTART, KPROF
  Z_PTCCPP5(JL,KLEVP) = 0.0_JPRB
  PTCCPP(JL,KLEVP)  = 0.0_JPRB
  ZMAXTCC(JL)  = PCCF(JL,1)
  ZMAXTCC5(JL) = PCCF5(JL,1)
ENDDO

!*       1.1   INTEGRATE CLOUD COVER

DO JLEV = 2, KFLEV
  DO JL = KSTART, KPROF
    IF( PCCF5(JL,JLEV-1) < 0.0_JPRB )THEN
      ZX5 = Z_PTCCPP5(JL,KLEVP)
      ZX  = PTCCPP(JL,KLEVP)
      Z_PTCCPP5 (JL,KLEVP) = ZX5 + (1.0_JPRB-ZX5)*ZMAXTCC5(JL)
      PTCCPP (JL,KLEVP) = ZX + (1.0_JPRB-ZX)*ZMAXTCC5(JL)&
       & + (1.0_JPRB-ZX5)*ZMAXTCC(JL)  
      ZMAXTCC(JL)  = PCCF(JL,JLEV)
      ZMAXTCC5(JL) = PCCF5(JL,JLEV)
    ELSE
      IF( ZMAXTCC5(JL) < PCCF5(JL,JLEV) )THEN
        ZMAXTCC(JL)  = PCCF(JL,JLEV)
        ZMAXTCC5(JL) = PCCF5(JL,JLEV)
      ENDIF
    ENDIF
  ENDDO
ENDDO

DO JL = KSTART, KPROF
  ZX  = PTCCPP(JL,KLEVP)
  ZX5 = Z_PTCCPP5(JL,KLEVP)
  PTCCPP(JL,KLEVP) = ZX + (1.0_JPRB-ZX)*ZMAXTCC5(JL)+ (1.0_JPRB-ZX5)*ZMAXTCC(JL)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPTCCTL',1,ZHOOK_HANDLE)
END SUBROUTINE PPTCCTL
