SUBROUTINE PPTTL(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & PRES,PLNPRES,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,&
 & POROG,PRXP,PRXPD,PTSTAR,PT0,PR0,PTF,PTPP,&
 & PRXP5,PRXPD5,PTSTAR5,PT05,PR05,PTF5)  

!**** *PPTTL* - POST-PROCESS TEMPERATURE - TANGENT LINEAR.

!     PURPOSE.
!     --------
!           PERFORMS THE VERTICAL INTERPOLATION OF TEMPERATURETO A GIVEN
!       MODEL CO-ORDINATE LEVEL. ALSO EXTRAPOLATES TEMPERATURE
!       BELOW SURFACE AND BOTTOM MODEL LEVEL IF SO REQUESTED.

!**   INTERFACE.
!     ----------
!        *CALL* *PPTTL(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                    PRPRES,POROG,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,
!                    PRXP,PRXPD,PTSTAR,PT0,
!                    PTF,PTPP,PRXP5,PRXPD5,PTSTAR5,PT05,PTF5)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRPRES          (INPUT-C)
!                                    (SEE PPFLEV)
!        PRPRES(KPROMA,KFLEV)       - POST-PROCESS   LEVEL PRESSURES.   (INPUT-C)
!        PLNPRT(KPROMA,KLEVP)      - PLNPRT(.,JLEVP)=
!                                     LOG(PRPRES(.,JLEVP)/PSOL)        (INPUT-C)
!        POROG(KPROMA)             - MODEL OROGRAPHY.                  (INPUT-C)
!        LDBELO(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     LOWEST (FULL) MODEL LEVEL        (INPUT-C)
!        LDBELS(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE                    (INPUT-C)
!        LDBLOW(KLEVP)             - FIRST LEVEL OF LDBELO CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        LDBLES(KLEVP)             - FIRST LEVEL OF LDBELS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        LDEXTR                    - .TRUE. IF EXTRAPOLATION REQUESTED (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!        PTSTAR(KPROMA)            - SURFACE TEMPERATURE               (INPUT)
!        PT0(KPROMA)               - STANDARD SURFACE TEMPERATURE      (INPUT)
!        PTF(KPROMA,0:KFLEV)       - TEMPERATURE ON FULL INPUT LEVELS. (INPUT)

!        PTPP(KPROMA,KLEVP)        - POST-PROCESSED TEMPERATURE        (OUTPUT)

!        PRXP5(KPROMA,0:KFLEV,NPPM)- HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT-T)
!        PRXPD5(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))          (INPUT-T)
!        PTSTAR5(KPROMA)           - SURFACE TEMPERATURE               (INPUT-T)
!        PT05(KPROMA)              - STANDARD SURFACE TEMPERATURE      (INPUT-T)
!        PTF5(KPROMA,0:KFLEV)      - TEMPERATURE ON FULL INPUT LEVELS. (INPUT-T)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST AND YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  PPINTP - LINEAR INTERPOLATION
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-01-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1
USE YOMCT0   , ONLY : LOLDPP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRES(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLOW(KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDEXTR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR05(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KPROMA,0:KFLEV) 

REAL(KIND=JPRB) ::ZTPP5(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZTF(KPROMA,0:KFLEV),       ZSTZF(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZALPHA(KPROMA)
REAL(KIND=JPRB) :: ZALPHA5(KPROMA),           ZTF5(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZSTZP(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZR0(KPROMA,0:KFLEV),       ZRP(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZR05(KPROMA,0:KFLEV),      ZRP5(KPROMA,KLEVP)

INTEGER(KIND=JPIM) :: IBL, ISLCT, JL, JLEV, JLEVP

REAL(KIND=JPRB) :: ZALP, ZALP5, ZALPC, ZFI2000, ZFI2500, ZGAM,&
 & ZGAM5, ZTXX, ZUSDFI, ZXX, ZXX5, ZXXX  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ppintp.intfb.h"
#include "ppintptl.intfb.h"
#include "ppsta.intfb.h"
#include "ppstatl.intfb.h"
#include "ppttl_old.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PPTTL',0,ZHOOK_HANDLE)
IF(LOLDPP) THEN
  CALL PPTTL_OLD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
   & PRES,POROG,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,&
   & PRXP,PRXPD,PTSTAR,PT0,PTF,PTPP,&
   & PRXP5,PRXPD5,PTSTAR5,PT05,PTF5)  
  IF (LHOOK) CALL DR_HOOK('PPTTL',1,ZHOOK_HANDLE)
  RETURN
ENDIF

!     ------------------------------------------------------------------
!*    1.    POST-PROCESS TEMPERATURE.
!           -------------------------
CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KFLEV+1,1,&
 & PRXP5(1,0,2),PRXP5(1,0,4),ZTF5,ZSTZF)  
CALL PPSTATL('PPREF',KPROMA,KSTART,KPROF,KFLEV+1,1,&
 & PRXP(1,0,2),PRXP(1,0,4),ZTF,ZSTZF,&
 & PRXP5(1,0,2))  

CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KLEVP,KLOLEV,PRES,PLNPRES,ZTPP5,ZSTZP)

DO JLEVP = KLOLEV, KLEVP
  DO JL = KSTART,KPROF
    PTPP(JL,JLEVP) = 0.0_JPRB
  ENDDO
ENDDO

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    ZTF(JL,JLEV)=PTF(JL,JLEV)-RD/PR05(JL,JLEV)*ZTF(JL,JLEV)&
     & + ZTF5(JL,JLEV)*RD/(PR05(JL,JLEV)**2)*PR0(JL,JLEV)  
    ZTF5(JL,JLEV)=PTF5(JL,JLEV)-ZTF5(JL,JLEV)*RD/PR05(JL,JLEV)
    ZR0(JL,JLEV) = PR0(JL,JLEV)
    ZR05(JL,JLEV) = PR05(JL,JLEV)
  ENDDO
ENDDO

!*    1.1   PREPARE FOR EXTRAPOLATION ABOVE TOP

DO JL = KSTART,KPROF
  ZTF(JL,0) = ZTF(JL,1)
  ZTF5(JL,0) = ZTF5(JL,1)
  ZR0(JL,0) = ZR0(JL,1)
  ZR05(JL,0) = ZR05(JL,1)
ENDDO

!*    1.2   INTERPOLATE TEMPERATURE QUADRATICALY IN LOG(P) TO KEEP GEOPOTENTIAL
!*          IN UPPERAIR AND BETWEEN LOWEST LEVEL AND SURFACE.

ISLCT=2
CALL PPINTP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRES,PRXP5,PRXPD5,ZR05,ZRP5)  
CALL PPINTPTL(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRES,PRXP,PRXPD,ZR0,ZRP,&
 & PRXP5,PRXPD5,ZR05)  

DO JLEVP = KLOLEV, KLEVP
  IF (.NOT. LDBLOW(JLEVP)) THEN
    DO JL = KSTART,KPROF
      IBL = KLEVB(JL,JLEVP,4)
      ZXX5 = (PRXP5(JL,IBL,4)-PLNPRES(JL,JLEVP))&
       & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      ZGAM5 = (PRXP5(JL,IBL-1,4)+PRXP5(JL,IBL,4)-2.0_JPRB*&
       & PRXP5(JL,IBL-1,3))/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  

      ZXX =(PRXP(JL,IBL,4)-ZXX5*(PRXP(JL,IBL,4)-PRXP(JL,IBL-1,4)))&
       & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      ZGAM= ( PRXP(JL,IBL-1,4)+PRXP(JL,IBL,4)&
       & -2.0_JPRB*PRXP(JL,IBL-1,3) -&
       & ZGAM5*(PRXP(JL,IBL,4)-PRXP(JL,IBL-1,4)) )&
       & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  

      PTPP(JL,JLEVP) = PTPP(JL,JLEVP) * RD/ZRP5(JL,JLEVP)&
       & - ZTPP5(JL,JLEVP) * (RD/(ZRP5(JL,JLEVP)**2))&
       & * ZRP(JL,JLEVP)&
       & + ZTF(JL,IBL)&
       & + (ZTF(JL,IBL-1)-ZTF(JL,IBL))&
       & * (ZXX5 + 3._JPRB * ZGAM5 * ZXX5 * (ZXX5 - 1.0_JPRB))&
       & + (ZTF5(JL,IBL-1)-ZTF5(JL,IBL))&
       & * (ZXX + 3._JPRB * ZGAM * ZXX5 * (ZXX5 - 1.0_JPRB)&
       & + 3._JPRB * ZGAM5 * (2.0_JPRB*ZXX*ZXX5-ZXX) )  
    ENDDO
  ELSE
    DO JL = KSTART,KPROF
      IF (.NOT.LDBELO(JL,JLEVP)) THEN
        IBL = KLEVB(JL,JLEVP,4)
        ZXX5 = (PRXP5(JL,IBL,4)-PLNPRES(JL,JLEVP))&
         & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        ZGAM5 = (PRXP5(JL,IBL-1,4)+PRXP5(JL,IBL,4)-2.0_JPRB*&
         & PRXP5(JL,IBL-1,3))/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  

        ZXX =(PRXP(JL,IBL,4)-ZXX5*(PRXP(JL,IBL,4)-PRXP(JL,IBL-1,&
         & 4)))&
         & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        ZGAM= ( PRXP(JL,IBL-1,4)+PRXP(JL,IBL,4)&
         & -2.0_JPRB*PRXP(JL,IBL-1,3) -&
         & ZGAM5*(PRXP(JL,IBL,4)-PRXP(JL,IBL-1,4)) )&
         & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  

        PTPP(JL,JLEVP) = PTPP(JL,JLEVP) * RD/ZRP5(JL,JLEVP)&
         & - ZTPP5(JL,JLEVP) * (RD/(ZRP5(JL,JLEVP)**2))&
         & * ZRP(JL,JLEVP)&
         & + ZTF(JL,IBL)&
         & + (ZTF(JL,IBL-1)-ZTF(JL,IBL))&
         & * (ZXX5 + 3._JPRB * ZGAM5 * ZXX5 * (ZXX5 - 1.0_JPRB))&
         & + (ZTF5(JL,IBL-1)-ZTF5(JL,IBL))&
         & * (ZXX + 3._JPRB * ZGAM * ZXX5 * (ZXX5 - 1.0_JPRB)&
         & + 3._JPRB * ZGAM5 * (2.0_JPRB*ZXX*ZXX5-ZXX) )  
      ELSEIF (LDBELO(JL,JLEVP) .AND. .NOT.LDBELS(JL,JLEVP)) THEN
        ZXX5 = (PRXP5(JL,KFLEV,3) - PLNPRES(JL,JLEVP))&
         & / (PRXP5(JL,KFLEV,3) - PRXP5(JL,KFLEV,4))  
        ZXX = ( PRXP(JL,KFLEV,3) -&
         & ZXX5*( PRXP(JL,KFLEV,3) - PRXP(JL,KFLEV,4) ) )&
         & / (PRXP5(JL,KFLEV,3) - PRXP5(JL,KFLEV,4))  
        PTPP(JL,JLEVP)=PTSTAR(JL)+ZXX5*(PTF(JL,KFLEV)-PTSTAR(JL))&
         & +ZXX*(PTF5(JL,KFLEV)-PTSTAR5(JL))  
      ENDIF
    ENDDO
  ENDIF
ENDDO

!*       1.4   EXTRAPOLATE TEMPERATURE BELOW SURFACE.

IF (LDEXTR) THEN
  ZTXX    = 298.0_JPRB
  ZFI2000 = 2000.0_JPRB * RG
  ZFI2500 = 2500.0_JPRB * RG
  ZUSDFI  = 1.0_JPRB / (ZFI2500 - ZFI2000)
  ZALPC   = -RDTDZ1 * RD / RG
  DO JL = KSTART, KPROF
    ZXXX = RD * MIN(1.0_JPRB, MAX(0.0_JPRB, (POROG(JL)-ZFI2000) * ZUSDFI))
    IF(ZTXX - PT05(JL) <= 0.0_JPRB)THEN
      IF( ZALPC + ZXXX*MIN(0.0_JPRB, ZTXX - PT05(JL)) /&
         & MAX(POROG(JL),ZFI2000)  >=  0.0_JPRB) THEN  
        ZALPHA(JL)=-ZXXX*PT0(JL)/MAX(POROG(JL),ZFI2000)
      ELSE
        ZALPHA(JL)=0.0_JPRB
      ENDIF
    ELSE
      ZALPHA(JL)=0.0_JPRB
    ENDIF
    ZALPHA5(JL) = MAX(0.0_JPRB, ZALPC + ZXXX * MIN(0.0_JPRB, ZTXX - PT05(JL))&
     & / MAX(POROG(JL), ZFI2000) )  
  ENDDO

  DO JLEVP = KLOLEV, KLEVP
    IF (LDBLES(JLEVP)) THEN
      DO JL = KSTART, KPROF
        IF (LDBELS(JL,JLEVP)) THEN
          ZALP5 = ZALPHA5(JL) * (PLNPRES(JL,JLEVP)-PRXP5(JL,KFLEV,3))
          ZALP = ZALPHA(JL)*(PLNPRES(JL,JLEVP)-PRXP5(JL,KFLEV,3))&
           & - ZALPHA5(JL) * PRXP(JL,KFLEV,3)  
          PTPP(JL,JLEVP) = PTSTAR(JL) *&
           & (1.0_JPRB + (1.0_JPRB + (0.5_JPRB + ZALP5/6._JPRB) * ZALP5) * ZALP5)&
           & + PTSTAR5(JL) *&
           & (1.0_JPRB + ( 1.0_JPRB + 0.5_JPRB * ZALP5) * ZALP5) * ZALP  
        ENDIF
      ENDDO
    ENDIF
  ENDDO
ELSE
  DO JLEVP = KLOLEV, KLEVP
    IF (LDBLES(JLEVP)) THEN
      DO JL = KSTART, KPROF
        IF (LDBELS(JL,JLEVP)) THEN
          PTPP(JL,JLEVP) = PTF(JL,KFLEV)
        ENDIF
      ENDDO
    ENDIF
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPTTL',1,ZHOOK_HANDLE)
END SUBROUTINE PPTTL

