SUBROUTINE TJCUBI (PX1,PY1,PX2,PY2,PX3,PY3,PX4,PY4,PXM,PYM,PXN,PYN,PENT,PEPS)

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PX1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX3 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY3 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX4 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY4 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PYM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXN 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PYN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPS 
REAL(KIND=JPRB) :: Z_PX0, Z_PY0, ZA, ZA1, ZA2, ZA3, ZA4, ZB, ZB1,&
 & ZB2, ZB3, ZB4, ZC, ZC1, ZC2, ZC3, ZC4, ZD, &
 & ZD1, ZD2, ZD3, ZD4, ZDD, ZDELTA  
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!**** *TJCUBI*  - POST-PROCESSING FOR AERONAUTICS

!     PURPOSE.
!     --------
!        To post-process jet and ICAO tropopause
!     THIS SUBROUTINE COMPUTES THE PARAMETERS A,B,C,D OF A CUBIC CURVE
!     PY=APX3+BPX2+CPX+D  PASSANT PAR LES 4 POINTS (PX1,PY1)(PX2,PY2)(PX3,PY3)(PX4, PY4)
!     THIS IS A  4 EQUATION SYSTEM WITH  4 UNKNOWNS
!     IN ORDER TO HAVE A CRAMER SYSTEM THE DETERMINANT HAS TO BE NONZERO
!     LET PX1 PX2 PX3 ET PX4 TBE DIFFERENT?
!     IF THIS CONDITION IS FULFILLED THERE EXISTS AN UNIQUE SOLUTION

!     WE SEEK THE COORDINATES OF THE POINTS FOR WHICH
!     THE DERIVATIVE Y'(PX)=PENT WHICH LEADS TO SOLVE THE EQUATION
!     3APX2+2BPX+C=PENT
!                   --->     PXM,PYM=MAXI     PXN,PYN=MINI
!     IF THE POINT ARE ON A STRIGHT LINE?, CHOOSE THE POINT PX3,PY3 AS
!     MAXI MUM AND MINIMUM
!     BEEING MOST OF THE CASES WITH AN INTERPOLATION
!     FROM THE MODEL LEVELS WE MAKE THE COMPUTATIONS WITH PX=NIVEAU
!     WHICH ENSURES THE EXISTENCE OF A SOLUTION

!**   INTERFACE.
!     ----------
!       *CALL* *TJCUBI*

!        EXPLICIT ARGUMENTS
!        --------------------
!               INPUT :
!               ------

!               OUTPUT :
!               ------

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        F. Autones and V. Martin *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-03-01
!        Modified : 93-01-01 J.-Marie Carriere
!        Modified : 94-11-01 X. RAGOT, R. El Khatib (adaptation to ARPEGE/IFS)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

!                    CALCULATION OF THE PARAMETERS

IF (LHOOK) CALL DR_HOOK('TJCUBI',0,ZHOOK_HANDLE)
ZA1=PX2**2*(PX3-PX4)-PX2*(PX3**2-PX4**2)+PX4*PX3**2-PX3*PX4**2
ZA2=PY2*(PX3-PX4)-PX2*(PY3-PY4)+PX4*PY3-PX3*PY4
ZA3=PY2*(PX3**2-PX4**2)-PX2**2*(PY3-PY4)+PX4**2*PY3-PX3**2*PY4
ZA4=PY2*(PX4*PX3**2-PX3*PX4**2)-PX2**2*(PY3*PX4-PX3*PY4)+&
 & PX2*(PY3*PX4**2-PY4*PX3**2)  

ZB1=PY2*(PX3-PX4)-PX2*(PY3-PY4)+PY3*PX4-PY4*PX3
ZB2=PX2**3*(PX3-PX4)-PX2*(PX3**3-PX4**3)+PX4*PX3**3-PX3*PX4**3
ZB3=PX2**3*(PY3-PY4)-PY2*(PX3**3-PX4**3)+PY4*PX3**3-PY3*PX4**3
ZB4=PX2**3*(PY3*PX4-PY4*PX3)-PY2*(PX4*PX3**3-PX3*PX4**3)+&
 & PX2*(PY4*PX3**3-PY3*PX4**3)  

ZC1=PX2**2*(PY3-PY4)-PY2*(PX3**2-PX4**2)+PY4*PX3**2-PY3*PX4**2
ZC2=PX2**3*(PY3-PY4)-PY2*(PX3**3-PX4**3)+PY4*PX3**3-PY3*PX4**3
ZC3=PX2**3*(PX3**2-PX4**2)-PX2**2*(PX3**3-PX4**3)+PX3**3*PX4**2-PX4**3*PX3**2
ZC4=PX2**3*(PY4*PX3**2-PY3*PX4**2)-PX2**2*(PY4*PX3**3-PY3*PX4**3)+&
 & PY2*(PX3**3*PX4**2-PX4**3*PX3**2)  

ZD1=PX2**2*(PX3*PY4-PX4*PY3)-PX2*(PY4*PX3**2-PY3*PX4**2)+&
 & PY2*(PX4*PX3**2-PX3*PX4**2)  
ZD2=PX2**3*(PX3*PY4-PX4*PY3)-PX2*(PY4*PX3**3-PY3*PX4**3)+&
 & PY2*(PX4*PX3**3-PX3*PX4**3)  
ZD3=PX2**3*(PY4*PX3**2-PY3*PX4**2)-PX2**2*(PY4*PX3**3-PY3*PX4**3)+&
 & PY2*(PX3**3*PX4**2-PX4**3*PX3**2)  
ZD4=PX2**3*(PX4*PX3**2-PX3*PX4**2)-PX2**2*(PX4*PX3**3-PX3*PX4**3)+&
 & PX2*(PX3**3*PX4**2-PX3**2*PX4**3)  

ZDELTA=PX1**3*ZA1-PX1**2*ZB2+PX1*ZC3-ZD4

ZA=(PY1*ZA1-PX1**2*ZA2+PX1*ZA3-ZA4)/ZDELTA
ZB=(PX1**3*ZB1-PY1*ZB2+PX1*ZB3-ZB4)/ZDELTA
ZC=(PX1**3*ZC1-PX1**2*ZC2+PY1*ZC3-ZC4)/ZDELTA
ZD=(PX1**3*ZD1-PX1**2*ZD2+PX1*ZD3-PY1*ZD4)/ZDELTA

!                    PARTICULAR CASE
!                    THE POINTS ARE ON A STRIGHT LINE

IF(ABS(ZA) >= PEPS.AND.ABS(ZB) >= PEPS) THEN

!                    PARTICULAR CASE
!                    QUADRATIC CASE

!         IF (ABS(ZA).LT.PEPS) THEN
!            PXM=(PENT-ZC)/(2*ZB)
!            PXN=PXM
!            PYM=ZB*PXM**2+ZC*PXM+ZD
!            PYN=PYM
!         ELSE

!                    CALCULATION OF THE  SOLUTIONS OF
!                    Y'(PX)=3ZAPX2+2ZBPX+ZC=PENT

  ZDD=4._JPRB*ZB**2-12._JPRB*ZA*(ZC-PENT)

  IF (ZDD < 0) THEN
    PXM=0.0_JPRB
    PYM=0.0_JPRB
    PXN=0.0_JPRB
    PYN=0.0_JPRB
  ELSE
    PXM=(-2.0_JPRB*ZB+SQRT(ZDD))/(6._JPRB*ZA)
    PXN=(-2.0_JPRB*ZB-SQRT(ZDD))/(6._JPRB*ZA)
    PYM=ZA*PXM**3+ZB*PXM**2+ZC*PXM+ZD
    PYN=ZA*PXN**3+ZB*PXN**2+ZC*PXN+ZD
  ENDIF

  IF (PYM < PYN) THEN
    Z_PY0=PYM
    PYM=PYN
    PYN=Z_PY0
    Z_PX0=PXM
    PXM=PXN
    PXN=Z_PX0
  ENDIF

!         ENDIF

ELSE
  PXM=PX3
  PYM=PY3
  PXN=PX3
  PYN=PY3
ENDIF

IF (LHOOK) CALL DR_HOOK('TJCUBI',1,ZHOOK_HANDLE)
END SUBROUTINE TJCUBI

