SUBROUTINE TJQUD (PX1,PY1,PX2,PY2,PX3,PY3,PXM,PYM,PENT,PEPS)

!**** *TJQUD*  - POST-PROCESSING FOR AERONAUTICS

!     PURPOSE.
!     --------
!        To post-process jet and ICAO tropopause
!     THIS SUBROUTINE COMPUTES THE PARAMETERS A,B,C OF A QUADRATIC FUNCTION
!     Y=APX2+BX+C FITTING EXACTLY THE  POINTS (PX1,PY1)(PX2,PY2)(PX3,PY3)
!     THIS IS A  3 EQUATION SYSTEM WITH  A 3 UNKNOWNS
!     IN ORDER TO ENSURE TO HAVE A CRAMER SYSTEM
!     THE DETERMINANT OF THE SYSTEM HAS TO BE  NONZERO
!     LET  PX1 PX2 AND PX3 BE DIFFERENT
!     IF THIS CONDITION IS FULFILLED, THERE EXISTS AN UNIQUE SOLUTION

!     THEN WE DETERMINE  THE COORDINATES OF THE POINT WHERE  ??????
!     Y'(X)=PENT----> PXM,PYM
!     IF THE POINTS ARE  ON A STRIGHT LINE WE CHOOSE THE POINT PX2,PY2

!     BEEING MOST OF THE TIME IN THE CASE OF AN INTERPOLATION BETWEEN
!     THE MODEL LEVELS, MAKE THE CALCULATION BUY TAKING X= NIVEAU
!     THIS ENSURES THE EXISTENCE OF THE SOLUTION

!**   INTERFACE.
!     ----------
!       *CALL* *TJQUD*

!        EXPLICIT ARGUMENTS
!        --------------------
!               INPUT :
!               ------

!               OUTPUT :
!               ------

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        F. Autones and V. Martin *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-03-01
!        Modified : 93-01-01 J.-Marie Carriere
!        Modified : 94-11-01 X. RAGOT, R. El Khatib (adaptation to ARPEGE/IFS)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PX1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX3 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PY3 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXM 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PYM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEPS 
REAL(KIND=JPRB) :: ZA, ZB, ZC, ZDELTA
REAL(KIND=JPRB) :: ZHOOK_HANDLE


IF (LHOOK) CALL DR_HOOK('TJQUD',0,ZHOOK_HANDLE)
ZDELTA=(PX2-PX3)*(PX1-PX2)*(PX1-PX3)

ZA=(PY1*(PX2-PX3)-PX1*(PY2-PY3)+PY2*PX3-PY3*PX2)/ZDELTA
ZB=(PX1**2*(PY2-PY3)-PY1*(PX2**2-PX3**2)+PY3*PX2**2-PY2*PX3**2)/ZDELTA
ZC=(PX1**2*(PX2*PY3-PX3*PY2)-PX1*(PY3*PX2**2-PY2*PX3**2)+&
 & PY1*(PX3*PX2**2-PX2*PX3**2))/ZDELTA  

IF (ABS(ZA) < PEPS) THEN
  PXM=PX2
  PYM=PY2
ELSE
  PXM=(PENT-ZB)/(2*ZA)
  PYM=ZA*PXM**2+ZB*PXM+ZC
ENDIF

IF (LHOOK) CALL DR_HOOK('TJQUD',1,ZHOOK_HANDLE)
END SUBROUTINE TJQUD

