SUBROUTINE COUPLO4_DEFINITIONS 
! * COUPL04_DEFINITIONS * - ASSIGNS AND PROVIDES .... 
!                            VARIABLE DEFINITION FOR OASIS4 COUPLED RUNS 

!     PURPOSE
!     --------
!     This routine computes & extracts local grid and spectral information 
!     and transient definitions 
!     and feeds this information to the coupler oasis4 by the calls 
!     of routines from the psmile library

!     IFS running in gaussian grid is treated as the reduced gaussian grid  

!     A Multi level and a single levele grid point grid and spectral grid
!     are defined

!     IFS meteo fields are definied as output to OASIS4
!     Production and loss rates of CTM are definied as input from OASIS4

!     The exchange is based on all local MPI_tasks
!     An option is implemented for exchange on one processor only  
!     LGLOBALE_EXCHANGEGP/SP 

!**   INTERFACE.
!     ----------
!              *COUPLEO4_definitions is called from CNT4 without arguments

!!**   EXTERNALS 
!     ----------
!              *prism_initialized 
!              *prism_init_comp    
!              *prism_reduced_grid_map
!              *prism_def_grid
!              *prism_def_partioning
!              *prism_set_points
!              *prism_set_coorners
!              *prism_def_var
!              *prism_calc_newdat
!              *prism_end_def

!              *GATH_GRID
!              *POSNAM   

!     REFERENCE.
!     ----------
!                   OASIS4 User guide
!     AUTHOR.
!     -------
!     2005-22-10  J. Flemming

!     MODIFICATIONS.
!     --------------

  USE PARKIND1 , ONLY : JPIM     ,JPRB 

  USE COUPLO4_MIX, ONLY : JP_NDIM, YIFS_GRID, & 
  &   JP_FIELDS, NBR_FIELDS, YFIELD_CPL, &
  &   YMODEL_TIME, YMODEL_TIME_BOUNDS,  EINCR1, EINCR2, EINCRMODEL, &
  &   LGLOBALEXCHANGEGP,  MEX_PROCGP, LGLOBALEXCHANGESP,  MEX_PROCSP, &
  &   ICOMP_ID, COMP_NAME, INDEX_RPREVACC, RPREVACC, TRANSIENT

  USE YOMCOUPLO4, ONLY : LCOUPLO4_CTM, NFRCOUPLO4, LOUT_SPC, &
    &  LOUT_FLUX, LGLOBALGP, LGLOBALSP,  MPROCGPG, MPROCSPG, &
    &  CTM, LDIFF_GRG, LCONV_GRG, LDIA_GRG, &
    &  LNOX2NO2

  USE YOMCST   , ONLY : RPI

!  USE COUPLO4_COM_MOD, ONLY : ICOMP_ID, COMP_NAME

  USE YOMGC    , ONLY : GELAM  ,GELAT, NGPLAT      

  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  USE YOMMP    , ONLY : LSPLIT, MYPROC, MYSETW, MYSETV, NPTRLL, NPSP  

  USE YOMDIM   , ONLY :       NDGLG ,&
       & NDGENL   ,NFLEVL ,NFLEVG , NSPEC2, NSPEC2G, &
       & NSPEC2MX

  USE YOMLUN   , ONLY : NULERR, NULNAM

  USE YOMGEM   , ONLY : NLOENG, NGPTOT, NGPTOTG    

  USE YOMDYN   , ONLY : TSTEP

  USE YOMGMV   , ONLY : YT0

  USE YOM_YGFL , ONLY :  YQ,  NGRG, YGRG, YGRG_NL, YI, YL, YA , YERA40

  USE MPL_MODULE   , ONLY : MPL_BROADCAST, MPL_BARRIER

  USE SURFACE_FIELDS_MIX  , ONLY :YSD_VD , YSD_VF, YSP_RR, YSP_SG, YSP_SB

  USE YOMLEG   , ONLY : RMU

!  USE YOM_GRGCTM, ONLY : QLOSSRGRG,  QPROGRG , QCONCTMGRG

  USE YOMCT0   , ONLY :  NPRTRW, NPROC, LCOUPLO4, LCOUPLO4_ENV

  USE GRG_PHOTOLYSIS_MIX , ONLY : FINT

  USE YOMTAG, ONLY : MTAGRAD

  USE PRISM_DUMMY_MOD

  IMPLICIT NONE

! VARIABLES FOR  GRIDS  DEFINITION
  REAL (KIND=JPRB), ALLOCATABLE :: ZCOORDS_1 (:)
  REAL (KIND=JPRB), ALLOCATABLE :: ZCOORDS_2 (:)
  REAL (KIND=JPRB), ALLOCATABLE :: ZCOORDS_3 (:)

! SET CORNERS
  REAL (KIND=JPRB), ALLOCATABLE :: ZCORNERS_1 (:,:)
  REAL (KIND=JPRB), ALLOCATABLE :: ZCORNERS_2 (:,:)
  REAL (KIND=JPRB), ALLOCATABLE :: ZCORNERS_3 (:,:)

  REAL (KIND=JPRB), ALLOCATABLE :: ZRGAUSLAT (:)

  REAL(KIND=JPRB)            :: ZDELTAX, ZDELTAYUP, ZDELTAYDW  
  REAL(KIND=JPRB)            :: ZPI, ZRND, ZIOFFSET     

! variables for prism_def_partioning 
  INTEGER(KIND=JPIM)                ::  ILATS_LOCAL ! number of latitudes per TASK
  INTEGER(KIND=JPIM),  ALLOCATABLE  ::  IPARTITION(:,:)  ! number of point per latitide band
  INTEGER(KIND=JPIM),  ALLOCATABLE  ::  IEXTENT(:,:)   ! index of start in global array 
  INTEGER(KIND=JPIM)                ::  IOFFSET, IOFFSETSPEC, IOFFSETSPECV   ! offset in global index 
  INTEGER(KIND=JPIM)                ::  ILOCALLAT   ! local  lat index 
  INTEGER(KIND=JPIM)                ::  IGLOBALLAT  ! global lat index 

  INTEGER(KIND=JPIM)   :: IVAR_NODIMS(2)
  INTEGER(KIND=JPIM)   :: IVAR_TYPE

! LOCAL VARIABLES
  INTEGER(KIND=JPIM) :: IERROR, I_ERRORS = 0
  INTEGER(KIND=JPIM) :: I, IK, IKK, IKE, IKS,ILATOLD 

  LOGICAL :: LLPRISMFLAG
  INTEGER(KIND=JPIM), PARAMETER :: JPIOMASTER=1
  REAL (KIND=JPRB), ALLOCATABLE :: ZFINT (:)

! DEFINE GRID LOCAL
  INTEGER(KIND=JPIM)            :: I_GRIDS
  INTEGER(KIND=JPIM)            :: INBR_CORNERS
  INTEGER(KIND=JPIM)            :: INBR_CORNERS_HORI
  INTEGER(KIND=JPIM)            :: INBR_CORNERS_VERT

  INTEGER(KIND=JPIM), DIMENSION(2,JP_NDIM)  :: ISHAPE_GP
  INTEGER(KIND=JPIM), DIMENSION(2,JP_NDIM)  :: ISHAPE_SP
  INTEGER(KIND=JPIM), DIMENSION(2,JP_NDIM+1)  :: ISHAPE_D 

! Variables for gathering global fields 
  REAL (KIND=JPRB), ALLOCATABLE :: ZLOCAL(:,:,:)
  REAL (KIND=JPRB), ALLOCATABLE :: ZGLOBAL(:,:)
  INTEGER(KIND=JPIM)            :: IFGATHG

  INTEGER(KIND=JPIM), ALLOCATABLE  :: ITO(:)
  LOGICAL                          :: LLCALLPSMILEGP
  LOGICAL                          :: LLCALLPSMILESP
  INTEGER(KIND=JPIM)    :: IK11, IK12,IK1, IGR, JGFL, ICOUNT_ACC, ICOUNT_ORDER, IORDERMET
  CHARACTER(LEN=4)      :: CLSPEZ_O4

  INTEGER(KIND=JPIM), DIMENSION(5)  :: IORDERCTM

!GRG photolysis info file
  CHARACTER(LEN=10)  :: clgrg_fint='grg_fintwb'
  LOGICAL            :: llfileok
  INTEGER(KIND=JPIM), PARAMETER  :: IUTMP=737

  TYPE(TRANSIENT) :: YLFIELD_CPL_D
  

 REAL(KIND=JPRB) :: ZHOOK_HANDLE


  INTERFACE
#include "gath_grid.h"
  END INTERFACE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "namcouplo4.h"


IF (LHOOK) CALL DR_HOOK('COUPLO4_DEFINITIONS',0,ZHOOK_HANDLE)

!----------------------------------------------------------------------
!                  0.5  Configure based on Namelist  NAMCOUPLO4
! ----------------------------------------------------------------------


  LCOUPLO4_CTM=.FALSE.  

  NFRCOUPLO4 = -1 ! Coupling frequency in hour

  LOUT_SPC=.FALSE.        ! output of spectral vor, div, sp
  LOUT_FLUX=.FALSE.       ! output of surface fluxes
 

  LGLOBALGP=.FALSE.       ! GP exchange via one mpi tasks only
  LGLOBALSP=.FALSE.       ! SP exchange via one mpi tasks only  
  MPROCGPG=-1             ! number of mpi tasks if  LGLOBALGP
  MPROCSPG=-1             ! number of mpi tasks if  LGLOBALSP
  CTM='TOY'               ! default Name of coupled CTM assume toy
  LDIFF_GRG = .TRUE.      ! IFS with Diffusion for GRG coupled 
  LCONV_GRG = .TRUE.      ! IFS with Convection for GRG coupled
  LNOX2NO2 = .FALSE.

  IF (LCOUPLO4 .NEQV. LCOUPLO4_ENV) THEN
     WRITE(NULERR,*) 'LCOUPLO4 /=  LCOUPLO4_ENV ', LCOUPLO4, LCOUPLO4_ENV  
!     CALL ABOR1(' LCOUPLO4 and LCOUPLO4_ENV MISMATCH')   
  ENDIF 

! action only in  prism run 
  IF( .NOT. LCOUPLO4 ) THEN 
     IF (LHOOK) CALL DR_HOOK('COUPLO4_DEFINITIONS',1,ZHOOK_HANDLE)
     RETURN
  ENDIF

! Read in NAMELIST 
 
  CALL POSNAM (NULNAM,'NAMCOUPLO4')
  READ (NULNAM,NAMCOUPLO4)    

  WRITE(NULERR,*) 'AFTER READING NAMELIST ', LOUT_SPC, LOUT_FLUX, & 
  &                 LGLOBALSP, LGLOBALGP


  WRITE(NULERR,*) 'IFS is coupled to ',CTM
  WRITE(NULERR,*) 'IFS applied CTM tendencies ', LCOUPLO4_CTM 
  WRITE(NULERR,*) 'IFS Diffusion  for GRG fields ',  LDIFF_GRG 
  WRITE(NULERR,*) 'IFS Convection for GRG fields ',  LCONV_GRG 
  WRITE(NULERR,*) 'GRG Diagnostics ',  LDIA_GRG 
  WRITE(NULERR,*) 'DIAGNOSTIC NO2 ',  LNOX2NO2
   
  LGLOBALEXCHANGEGP = LGLOBALGP
  MEX_PROCGP = MPROCGPG 
  LGLOBALEXCHANGESP = LGLOBALSP  
  MEX_PROCSP = MPROCSPG 

  LLCALLPSMILEGP = .TRUE.
  LLCALLPSMILESP = .TRUE.

  IF ( LGLOBALEXCHANGEGP .AND. MYPROC /= MEX_PROCGP ) LLCALLPSMILEGP=.FALSE.
  IF ( LGLOBALEXCHANGESP .AND. MYPROC /= MEX_PROCSP ) LLCALLPSMILESP=.FALSE.

! no action

  WRITE( NULERR , * ) '---------------------------------------------------------------------'
  WRITE ( NULERR , * ) 'I AM COMPONENT ', TRIM(COMP_NAME), ' WITH ID ', ICOMP_ID
  WRITE ( NULERR , * ) '---------------------------------------------------------------------'
  WRITE ( NULERR , * )

  CALL PRISM_INITIALIZED (LLPRISMFLAG, IERROR)

  IF (IERROR /= 0) I_ERRORS = I_ERRORS + 1
  IF (.NOT. LLPRISMFLAG) THEN
      WRITE (NULERR,*) TRIM(COMP_NAME), " PRISM IS NOT INITIALIZED"
      CALL prism_init_comp (icomp_id, comp_name, ierror )
  ENDIF

!----------------------------------------------------------------------
!                  1.   TIME SETTINGS 
! ----------------------------------------------------------------------

! Time setting from xml file only !!!!
! has to be changed to actual model time 
! NFRCOUPLO4 is namelist parameter, coupling frequncy in hours 

  EINCRMODEL = 3600.0 * FLOAT(NFRCOUPLO4)  

! Time bounds  !!
  EINCR1 = - 0.5 * EINCRMODEL
  EINCR2 =   0.5 * EINCRMODEL  

  IF ( EINCRMODEL <  0 ) THEN 
    WRITE(NULERR,*) ' NFRCOUPLO4 not defined '
    WRITE(NULERR,*) ' NFRCOUPLO4 =  ', NFRCOUPLO4,' i.e. ',  EINCRMODEL 
    CALL ABOR1(' WRONG NFRCOUPLO4 IN COUPLO4_DEFINITIONS')                   
  ENDIF 

! Check if coupling frequency and time step do not contradict
  IF ( MOD (EINCRMODEL, TSTEP) /= 0 ) THEN 
    WRITE(NULERR,*) ' TSTEP and NFRCOUPLO4 do not synchronise'
    WRITE(NULERR,*) ' TSTEP = ', TSTEP  
    WRITE(NULERR,*) ' NFRCOUPLO4 =  ', NFRCOUPLO4,' i.e. ',  EINCRMODEL 
    CALL ABOR1('WRONG NFRCOUPLO4 OR TSTEP IN COUPLO4_DEFINITIONS ')            
  ENDIF 

  YMODEL_TIME           = PRISM_JOBSTART_DATE
  YMODEL_TIME_BOUNDS(1) = PRISM_JOBSTART_DATE
  YMODEL_TIME_BOUNDS(2) = PRISM_JOBSTART_DATE

  CALL PRISM_CALC_NEWDATE ( YMODEL_TIME_BOUNDS(1), EINCR1, IERROR )
  CALL PRISM_CALC_NEWDATE ( YMODEL_TIME_BOUNDS(2), EINCR2, IERROR )

! ----------------------------------------------------------------------
!                     2.   GRID DEFINITION 
! ----------------------------------------------------------------------

  WRITE(NULERR,*) 'W AND V SET', MYSETW,MYSETV , MYPROC, NDGLG  
  WRITE(NULERR,*) ' NPSP ',  NPSP 
  ZPI = RPI

! ----------------------------------------------------------------------
!                     2.1 Assign GRID Dimensions  
! ----------------------------------------------------------------------

! for reduced gaussian grids and spectral grid (1 dimension horizontaly)
  WRITE ( NULERR , '(17X,3(5X,A5))' ) 'I-DIM', 'K-DIM' ,'I-DIMSP', 'K-DIMSP'
  WRITE ( NULERR , '(17X,A30)')      '------------------------------'
  WRITE ( NULERR , '(A17,6I5)' )   'GLOBAL DIMS ARE: ', NGPTOTG,NFLEVG,NSPEC2G,NFLEVG
  WRITE ( NULERR , '(A17,6I5)' )   'LOCAL  DIMS ARE: ',NGPTOT,NFLEVG,NSPEC2,NFLEVL  

! ASSIGNS SHAPE OF ARRAYS
  ISHAPE_GP = 1
  ISHAPE_GP(2,1) = NGPTOT
  ISHAPE_GP(2,2) = NFLEVG   ! VERTICAL MIN  

! gridless grids require global index space index in def grid
  IOFFSETSPEC = NSPEC2MX * (MYSETW-1)  ! very crude index space  
  IOFFSETSPECV = NPTRLL(MYSETV)   - 1

  ISHAPE_SP      = 1
  ISHAPE_SP(2,1) = IOFFSETSPEC  + NSPEC2MX  
  ISHAPE_SP(1,1) = IOFFSETSPEC  + 1
  ISHAPE_SP(1,2) = IOFFSETSPECV + 1
  ISHAPE_SP(2,2) = IOFFSETSPECV + NFLEVL    

! Get global Gaussian latitudes , including poles, neede for corner coordinates
  ALLOCATE (ZRGAUSLAT(0:NDGLG+1))
  ZRGAUSLAT(0)      = 90.0_JPRB
  ZRGAUSLAT(NDGLG+1)=-90.0_JPRB
  ZRGAUSLAT(1:NDGLG)= RMU(1:NDGLG)
  ZRGAUSLAT(1:NDGLG)= ASIN( ZRGAUSLAT(1:NDGLG))*(360.0_JPRB/(2.0_JPRB*ZPI))

! Define Number of lats and extend per lat band
  ILATS_LOCAL= NGPLAT(NGPTOT) - NGPLAT(1) +1  

  ALLOCATE (IPARTITION(ILATS_LOCAL,2))
  ALLOCATE (IEXTENT(ILATS_LOCAL,2))

!  WRITE ( NULERR , *  )   'LOCAL NUMBER OF LATITUDES IS: ',ILATS_LOCAL 
  IF (ILATS_LOCAL /= NDGENL) THEN
     WRITE ( NULERR , * ) 'LOCAL NUMBER OF LATITUDES PER A-SET IS :'&
          &      ,NDGENL,LSPLIT
!   IF (ILATS_LOCAL /= NDGENL-1) CALL ABOR1('Something wrong with ILATS_LOCAL')
  ENDIF

! ----------------------------------------------------------------------
!                2.2 Determine lat and lon of local points for PRISM Grids  
! ----------------------------------------------------------------------

! 2.2.1 allocation of zcoords1/2/3 =>  argument for prism_setpoints/corners 
!-----------------------------------------------------------------------

  ALLOCATE ( ZCOORDS_1(ISHAPE_GP(1,1):ISHAPE_GP(2,1)) &
       &, STAT=IERROR )
  IF ( IERROR /= 0 ) PRINT *, 'ERROR ALLOCATING  ZCOORDS_1'
  IF ( IERROR /= 0 ) I_ERRORS = I_ERRORS + 1

  ALLOCATE ( ZCOORDS_2(ISHAPE_GP(1,1):ISHAPE_GP(2,1)) &
       &, STAT=IERROR )
  IF ( IERROR /= 0 ) PRINT *, 'ERROR ALLOCATING  ZCOORDS_2'
  IF ( IERROR /= 0 ) I_ERRORS = I_ERRORS + 1

  ALLOCATE ( ZCOORDS_3(ISHAPE_GP(1,2)-1:ISHAPE_GP(2,2)), STAT=IERROR )
  IF ( IERROR /= 0 ) PRINT *, 'ERROR ALLOCATING  ZCOORDS_3'
  IF ( IERROR /= 0 ) I_ERRORS = I_ERRORS + 1

! DEFINE LOCAL GRID CORNER POINTS
  INBR_CORNERS = 8
  INBR_CORNERS_VERT=2
  INBR_CORNERS_HORI = (INBR_CORNERS / INBR_CORNERS_VERT) / 2 

  ALLOCATE ( ZCORNERS_1(ISHAPE_GP(1,1):ISHAPE_GP(2,1),INBR_CORNERS_HORI), STAT=IERROR )
  IF ( IERROR /= 0 ) PRINT *, 'ERROR ALLOCATING ZCORNERS_1'
  IF ( IERROR /= 0 ) I_ERRORS = I_ERRORS + 1

  ALLOCATE ( ZCORNERS_2(ISHAPE_GP(1,1):ISHAPE_GP(2,1),INBR_CORNERS_HORI), STAT=IERROR )
  IF ( IERROR /= 0 ) PRINT *, 'ERROR ALLOCATING ZCORNERS_1'
  IF ( IERROR /= 0 ) I_ERRORS = I_ERRORS + 1

  ALLOCATE ( ZCORNERS_3(ISHAPE_GP(1,2)-1:ISHAPE_GP(2,2),INBR_CORNERS_VERT), STAT=IERROR )
  IF ( IERROR /= 0 ) PRINT *, 'ERROR ALLOCATING ZCORNERS_3'
  IF ( IERROR /= 0 ) I_ERRORS = I_ERRORS + 1


! 2.2.2 Set longitudes and latitudes for points
!-----------------------------------------------------------------------

  ZCOORDS_1 = 0.0_JPRB
  ZCOORDS_2 = 0.0_JPRB
  ZCOORDS_3 = 0.0_JPRB

  IK = 0    
  DO I = ISHAPE_GP(1,1), ISHAPE_GP(2,1)
     IK = IK + 1  

! Degrees east 0...360

! rounding test
!     ZCOORDS_1(IK) = FLOAT(INT(0.5+ (360.0_JPRB/(2.0_JPRB*ZPI))*GELAM(IK)*100000.0_JPRB))*0.00001_JPRB 

     ZCOORDS_1(IK) = (360.0_JPRB/(2.0_JPRB*ZPI))*GELAM(IK) 

! latitude 90 > ZCOORDS_2  > -90
     ZCOORDS_2(IK) = (360.0_JPRB/(2.0_JPRB*ZPI))*GELAT(IK) 
  ENDDO

! SET VERTICAL AXIS
  DO IK = ISHAPE_GP(1,2), ISHAPE_GP(2,2)
     ZCOORDS_3(IK) = FLOAT (IK)                         !ONLY LAYER INDEX
  ENDDO


! 2.2.3 Assign longitude and latitude  of corners
!-----------------------------------------------------------------------

! we simple take the average of the latitude as corners in ns direction
! we assume constant delta x on each latidue band

  ZDELTAX=-9999.0_JPRB
  ZDELTAYDW=-9999.0_JPRB
  ZDELTAYUP=-9999.0_JPRB
  ZCORNERS_1=0.0_JPRB
  ZCORNERS_2=0.0_JPRB  
  ZCORNERS_3=0.0_JPRB

  ZRND=100000.0_JPRB
  IK=0
  ILATOLD=-1
  DO I = ISHAPE_GP(1,1), ISHAPE_GP(2,1)
     IK=IK+1
     ZDELTAX =   0.5_JPRB * 360.0_JPRB/ FLOAT( NLOENG( NGPLAT(IK) ))

     ZDELTAYUP = 0.5_JPRB * ( ZRGAUSLAT(NGPLAT(IK)-1) - ZRGAUSLAT (NGPLAT(IK)) )
     ZDELTAYDW = 0.5_JPRB * ( ZRGAUSLAT(NGPLAT(IK)+1) - ZRGAUSLAT (NGPLAT(IK)) ) 

     IF( ZRGAUSLAT(NGPLAT(IK)-1) ==  90.0_JPRB)  ZDELTAYUP = 2.0_JPRB*ZDELTAYUP
     IF( ZRGAUSLAT(NGPLAT(IK)+1) == -90.0_JPRB)  ZDELTAYDW = 2.0_JPRB*ZDELTAYDW

     IF( ZRGAUSLAT(NGPLAT(IK)) /=  ZCOORDS_2(IK) ) THEN
        WRITE(NULERR,*) ' WRONG ',ZRGAUSLAT(NGPLAT(IK)),ZCOORDS_2(IK)      
        CALL ABOR1('Something wrong with ZRGAUSLAT ')
     ENDIF

! to indicate periodicity we define longitude of corners < 0 and > 360       

     ZCORNERS_1(IK,1) = ZCOORDS_1(IK)-ZDELTAX
     ZCORNERS_1(IK,2) = ZCOORDS_1(IK)+ZDELTAX

     ZCORNERS_2(IK,1) = ZCOORDS_2(IK)+ZDELTAYUP
     ZCORNERS_2(IK,2) = ZCOORDS_2(IK)+ZDELTAYDW

!     IF (NGPLAT(IK) /= ILATOLD )THEN  
!      IF (IK > 0) THEN 
!      write(NULERR,*) 'LOC COORDS 1 first ',NGPLAT(IK), ZCOORDS_1(IK),ZCORNERS_1(IK,1:2)
!      write(NULERR,*) 'LOC COORDS 2 first ',NGPLAT(IK), ZCOORDS_2(IK),ZCORNERS_2(IK,1:2)
!        write(NULERR,*) 'LOC COORDS 1  last',NGPLAT(IK), ZCOORDS_1(IK-1),ZCORNERS_1(IK-1,1:2)
!        write(NULERR,*) 'LOC COORDS 2  last',NGPLAT(IK), ZCOORDS_2(IK-1),ZCORNERS_2(IK-1,1:2)

!     ENDIF

!     ILATOLD= NGPLAT(IK)
!     ENDIF


  ENDDO

! corner in the vertical 
! we assume model level number as unit of vertical dimension
  DO IK = ISHAPE_GP(1,2), ISHAPE_GP(2,2)
     ZCORNERS_3(IK,1) = ZCOORDS_3(IK)-0.5
     ZCORNERS_3(IK,2) = ZCOORDS_3(IK)+0.5
  ENDDO

! corner in the vertical for 1d grid
     ZCORNERS_3(0,1) = -0.5
     ZCORNERS_3(0,2) =  0.5


! 2.2.4 DEFINIG IEXTENT and IPARTITION
!-----------------------------------------------------------------------
! ipartition refers to a global index space 
! we define the global index space as going from most northern latidude in a 
! circular manner down to the  south pole , latitude band starts at 0 lon. 

  IEXTENT=0
  IPARTITION=-999

  IK=0
  DO I = ISHAPE_GP(1,1), ISHAPE_GP(2,1)
     IK=IK+1
     IGLOBALLAT=NGPLAT(IK)
     ILOCALLAT=IGLOBALLAT-NGPLAT(1)+1
 ! number of points per local lat-band
     IEXTENT(ILOCALLAT,1) = IEXTENT(ILOCALLAT,1) + 1

   ! first point on latitude band
     IF (IPARTITION(ILOCALLAT,1)==-999) THEN
      ! all full lat bands north of domain
        IOFFSET=0
        DO IKK=1, IGLOBALLAT-1,1
           IOFFSET=NLOENG(IKK)+IOFFSET
        ENDDO
! index on the actual band
        ZIOFFSET= (ZCOORDS_1(IK)/360.0_JPRB) * FLOAT(NLOENG(IGLOBALLAT))

! check ZIOFFSET 
        IF ( FLOOR (ZIOFFSET) /= ZIOFFSET ) THEN
         WRITE(NULERR,*) ' ZIOFFSET , ZCOORDS_1 ', ZIOFFSET, ZCOORDS_1(IK) 
!         CALL ABOR1('Something wrong with ZIOFFSET  ')
        ENDIF

        IOFFSET=IOFFSET + INT (ZIOFFSET + 0.5_JPRB) 

        IPARTITION(ILOCALLAT,1)=IOFFSET
     ENDIF
  ENDDO

! check IEXTENT
   IF (SUM(IEXTENT(:,1)) /= NGPTOT) THEN
     WRITE(NULERR,*) ' IEXTENT ', SUM(IEXTENT(:,1)),NGPTOT       
     CALL ABOR1('Something wrong with IEXTENT  ')
   ENDIF

! check that boxes left and right border fit
 IK12=0
 DO ILOCALLAT = 1, NGPLAT(NGPTOT)  - NGPLAT(1) + 1
    IK11=IK12+1
    IK12=IK11+IEXTENT(ILOCALLAT,1)-1
    DO IKK=IK11,IK12-1
       IF (ZCORNERS_1(IKK,2) /= ZCORNERS_1(IKK+1,1)) THEN
          ZCORNERS_1(IKK,2) = ZCORNERS_1(IKK+1,1)
       ENDIF
    ENDDO
 ENDDO


! ----------------------------------------------------------------------
!  2.3 IF(LGLOBALEXCHANGEGP)  gather gird information to for global grid
! --------------------------------------------------------------------

 IF (LGLOBALEXCHANGEGP) THEN

     WRITE(NULERR,*) 'OASIS4 exchange vi Master MPI task only on PROC', MEX_PROCGP 

!  2.3.1 GATHER Corner and point coordinates to global GRID
!-----------------------------------------------------------------------

     IFGATHG=6 ! lat lon for points and for 2 corners = 6 fields

     ALLOCATE (ZLOCAL(NGPTOT,IFGATHG,1))
     ALLOCATE (ZGLOBAL(NGPTOTG,IFGATHG))

   ! assign local array with grid information 
     IOFFSET=0
     ZLOCAL(1:NGPTOT,1,1) = ZCOORDS_1(1:NGPTOT)
     IOFFSET=IOFFSET+NGPTOT
     ZLOCAL(1:NGPTOT,2,1) = ZCOORDS_2(1:NGPTOT) 
     IOFFSET=IOFFSET+NGPTOT
     ZLOCAL(1:NGPTOT,3,1) = ZCORNERS_1(1:NGPTOT,1)
     IOFFSET=IOFFSET+NGPTOT
     ZLOCAL(1:NGPTOT,4,1) = ZCORNERS_1(1:NGPTOT,2)
     IOFFSET=IOFFSET+NGPTOT
     ZLOCAL(1:NGPTOT,5,1) = ZCORNERS_2(1:NGPTOT,1)
     IOFFSET=IOFFSET+NGPTOT
     ZLOCAL(1:NGPTOT,6,1) = ZCORNERS_2(1:NGPTOT,2)

! Get global array  
     ALLOCATE (ITO(IFGATHG)) 
     ITO= MEX_PROCGP

     CALL GATH_GRID(PGP=ZLOCAL,PGPG=ZGLOBAL,KTO=ITO,KFGATHG=IFGATHG)

     DEALLOCATE (ZLOCAL)
     DEALLOCATE (ZCOORDS_1)
     DEALLOCATE (ZCOORDS_2)
     DEALLOCATE (ZCORNERS_1)
     DEALLOCATE (ZCORNERS_2)

     IF (LLCALLPSMILEGP) THEN

       ALLOCATE ( ZCOORDS_1(1:NGPTOTG), STAT = IERROR )
       ALLOCATE ( ZCOORDS_2(1:NGPTOTG), STAT = IERROR )
       ALLOCATE ( ZCORNERS_1(1:NGPTOTG,2), STAT = IERROR )
       ALLOCATE ( ZCORNERS_2(1:NGPTOTG,2), STAT = IERROR )

       ZCOORDS_1(1:NGPTOTG) = ZGLOBAL(1:NGPTOTG,1)
       ZCOORDS_2(1:NGPTOTG) = ZGLOBAL(1:NGPTOTG,2)
       ZCORNERS_1(1:NGPTOTG,1) = ZGLOBAL(1:NGPTOTG,3)
       ZCORNERS_1(1:NGPTOTG,2) = ZGLOBAL(1:NGPTOTG,4)
       ZCORNERS_2(1:NGPTOTG,1) = ZGLOBAL(1:NGPTOTG,5)
       ZCORNERS_2(1:NGPTOTG,2) = ZGLOBAL(1:NGPTOTG,6)

! assign shape of global arrays
       ISHAPE_GP=1
       ISHAPE_GP(2,1) = NGPTOTG
       ISHAPE_GP(2,2) = NFLEVG

!     2.3.2 Re-determine extent and "partition" for global grid 
!     on exchange proc  
!-----------------------------------------------------------------------

        IF (ALLOCATED (IPARTITION)) DEALLOCATE(IPARTITION)
        IF (ALLOCATED (IEXTENT)   ) DEALLOCATE(IEXTENT)

        ALLOCATE(IPARTITION(NDGLG,2))
        ALLOCATE(IEXTENT(NDGLG,2))

        IPARTITION(1,1)=0
        ILATS_LOCAL = NDGLG
        IOFFSET = 0
         DO IKK=1, NDGLG,1
           IEXTENT(IKK,1)    = NLOENG(IKK)
           IPARTITION(IKK,1) = IOFFSET
           IOFFSET = IOFFSET + NLOENG(IKK)
          ENDDO

!    check that boxes upper and lower as well as left an right border fit
        IK12=0
        DO ILOCALLAT = 1, NDGLG 
          IK11=IK12+1
          IK12=IK11+IEXTENT(ILOCALLAT,1)-1
          DO IKK=IK11,IK12-1
             IF (ZCORNERS_1(IKK,2) /= ZCORNERS_1(IKK+1,1)) THEN
               ZCORNERS_1(IKK,2) = ZCORNERS_1(IKK+1,1)
             ENDIF
           ENDDO
         IF ((ZCORNERS_1(IK12,2) - 360.0_JPRB)/= ZCORNERS_1(IK11,1)) THEN
           ZCORNERS_1(IK12,2) = ZCORNERS_1(IK11,1) + 360.0_JPRB 
        ENDIF
      ENDDO

    ENDIF ! llcalpsmilegp

  ENDIF ! endif lglobalexchange


  IF (LGLOBALEXCHANGESP .AND. LLCALLPSMILESP) THEN
       ISHAPE_SP(1,1) = 1
       ISHAPE_SP(2,1) = NSPEC2G
       ISHAPE_SP(1,2) = 1
       ISHAPE_SP(2,2) = NFLEVG
   ENDIF
  

! ----------------------------------------------------------------------
!    3. Define four PRISM Grids (mulit & single level /gridpoint & spectral
! ----------------------------------------------------------------------

! YIFS_GRID(1)  Multi level grid fields
! YIFS_GRID(2)  single level grid fields 
! YIFS_GRID(3)  Multi spectral coefficent fields
! YIFS_GRID(4)  single level spectral coefficinet fields 

  YIFS_GRID(1:2)%ITYPE  = PRISM_Gaussreduced_regvrt  ! 1 ARRAY FOR HIRIZONTAL DOMAIN AND 1 FOR VERTICAL

  YIFS_GRID(3:4)%ITYPE  = PRISM_gridless  ! 1 ARRAY FOR HIRIZONTAL DOMAIN AND 1 FOR VERTICAL

  YIFS_GRID(1:4)%CNAME_POINTS = 'center'
  YIFS_GRID(1:4)%LNEW_POINTS = .TRUE.
  YIFS_GRID(1:4)%ID   = -999_JPIM

  YIFS_GRID(1)%CNAME  = 'ifs_grid_3d'
  YIFS_GRID(2)%CNAME  = 'ifs_grid_2d'
 

  YIFS_GRID(1)%ISHAPE =  ISHAPE_GP
  YIFS_GRID(2)%ISHAPE =  ISHAPE_GP
  YIFS_GRID(2)%ISHAPE(1:2,2) =  ISHAPE_GP(1,2)   ! no vertical extention

  YIFS_GRID(3)%CNAME  = 'ifs_spec_3d'
  YIFS_GRID(4)%CNAME  = 'ifs_spec_2d'

  YIFS_GRID(3:4)%ISHAPE(1,1) =  ISHAPE_SP(1,1)
  YIFS_GRID(3:4)%ISHAPE(2,1) =  ISHAPE_SP(2,1)

  YIFS_GRID(3)%ISHAPE(1,2) =    ISHAPE_SP(1,2)
  YIFS_GRID(3)%ISHAPE(2,2) =    ISHAPE_SP(2,2)

  YIFS_GRID(4)%ISHAPE(1,2) =   1 
  YIFS_GRID(4)%ISHAPE(2,2) =   1

! ----------------------------------------------------------------------
!                3.1  Loop over grid types  
! ----------------------------------------------------------------------

IF (LLCALLPSMILEGP) THEN
  DO I_GRIDS=1,2  ! reduced gaussian grids 

     IKS = YIFS_GRID(I_GRIDS)%ISHAPE(1,2)
     IKE = YIFS_GRID(I_GRIDS)%ISHAPE(2,2)

     IF (I_GRIDS == 2) THEN
        IKS = 0
        IKE = 0
     ENDIF


     CALL PRISM_DEF_GRID (YIFS_GRID(I_GRIDS)%ID, &
         & YIFS_GRID(I_GRIDS)%CNAME, ICOMP_ID, & 
         & YIFS_GRID(I_GRIDS)%ISHAPE, YIFS_GRID(I_GRIDS)%ITYPE, IERROR )

     WRITE (NULERR,*)' AFTER DEF_GRID ', YIFS_GRID(I_GRIDS)%ID, &
          & YIFS_GRID(I_GRIDS)%CNAME, YIFS_GRID(I_GRIDS)%ISHAPE

!  Corners 
     WRITE(NULERR,*)' BEFORE SET_CORNER GRID TYPE ', YIFS_GRID(I_GRIDS)%ID, &
          & INBR_CORNERS,  YIFS_GRID(I_GRIDS)%ISHAPE

     CALL PRISM_SET_CORNERS ( YIFS_GRID(I_GRIDS)%ID, &
          &  INBR_CORNERS,  YIFS_GRID(I_GRIDS)%ISHAPE, &
          &  ZCORNERS_1, ZCORNERS_2, ZCORNERS_3(IKS:IKE,1:2), IERROR )

     WRITE(NULERR,*)' BEFORE SET_POINTS ', YIFS_GRID(I_GRIDS)%ID_POINTS ,&
          &  YIFS_GRID(I_GRIDS)%CNAME_POINTS, YIFS_GRID(I_GRIDS)%ID, & 
          & YIFS_GRID(I_GRIDS)%ISHAPE,YIFS_GRID(I_GRIDS)%LNEW_POINTS, IERROR
! Points 
     CALL PRISM_SET_POINTS ( YIFS_GRID(I_GRIDS)%ID_POINTS , &
          &  YIFS_GRID(I_GRIDS)%CNAME_POINTS, & 
          & YIFS_GRID(I_GRIDS)%ID,YIFS_GRID(I_GRIDS)%ISHAPE,ZCOORDS_1, &
          &  ZCOORDS_2, ZCOORDS_3(IKS:IKE),& 
          & YIFS_GRID(I_GRIDS)%LNEW_POINTS, IERROR)

! Setting of global data of reduced Gaussian Grid
     CALL PRISM_REDUCEDGRID_MAP(YIFS_GRID(I_GRIDS)%ID, &
          & NDGLG, NLOENG(1: NDGLG) , IERROR )

!     WRITE(nulerr,*) 'mlevel ', ilats_local, IPARTITION,iextent

! Describe partioning 
! vertical part is not supported but has to be specified as :

     IEXTENT(:,2)=IKE-IKS+1
     IPARTITION(:,2)=0 

     CALL PRISM_DEF_PARTITION ( YIFS_GRID(I_GRIDS)%ID, &
          & ILATS_LOCAL , IPARTITION, IEXTENT, IERROR )

  ENDDO ! reduced gaussian grid
ENDIF  ! LCALPSMILEGP

IF(LOUT_SPC .AND. LLCALLPSMILESP) THEN 
   DO I_GRIDS=3,4 ! spectral grids 

!  no grid if no surface spectral data and multi pe exchange
        IF (NPSP == 0 .AND. I_GRIDS ==4 .AND. .NOT. LGLOBALEXCHANGESP )CYCLE 
        YIFS_GRID(4)%ISHAPE(2,1) = NPRTRW * NSPEC2MX 
!      IF (NPSP /= 1 .AND. I_GRIDS ==4 ) THEN
!         IF (LGLOBALEXCHANGESP ) THEN
!! gathering in exchnage of 2d sp field with send recv with NSPEC2MX
!           YIFS_GRID(I_GRIDS)%ISHAPE(2,1) = NPRTRV * NSPEC2MX 
!         ELSE
!             CYCLE ! no grid if no surface spectral data and multi pe exchange
!          ENDIF
!      ENDIF
     ISHAPE_D(1:2,3)  = 1 ! needed since PRISM_gridless always requires 3d 
     ISHAPE_D(1:2,1:2)= YIFS_GRID(I_GRIDS)%ISHAPE(1:2,1:2)

      CALL PRISM_DEF_GRID ( YIFS_GRID(I_GRIDS)%ID, &
          & YIFS_GRID(I_GRIDS)%CNAME, ICOMP_ID, & 
          & ISHAPE_D, YIFS_GRID(I_GRIDS)%ITYPE, IERROR )

      WRITE (NULERR,*)' AFTER SPECTRAL GRID DEF_GRID ',YIFS_GRID(I_GRIDS)%ID, &
          & YIFS_GRID(I_GRIDS)%CNAME, ISHAPE_D

! Points in gridless case 
      CALL PRISM_SET_POINTS ( YIFS_GRID(I_GRIDS)%ID_POINTS , &
          & YIFS_GRID(I_GRIDS)%CNAME_POINTS, & 
          & YIFS_GRID(I_GRIDS)%ID, & 
          & YIFS_GRID(I_GRIDS)%LNEW_POINTS, IERROR)
   ENDDO
ENDIF  ! LCALLPSMILESP


! ----------------------------------------------------------------------
!                   4.   TRANSIENT VARIABLES DEFINITION
! ----------------------------------------------------------------------

! Clear SEETINGS FOR ALL TRANSIENTS
  YFIELD_CPL(1:JP_FIELDS)%CNAME     = ''
  YFIELD_CPL(1:JP_FIELDS)%ID        = 999_JPIM
  YFIELD_CPL(1:JP_FIELDS)%L_IN      = .FALSE.
  YFIELD_CPL(1:JP_FIELDS)%L_OUT     = .FALSE.
  YFIELD_CPL(1:JP_FIELDS)%T_CALL    =  EINCRMODEL
  YFIELD_CPL(1:JP_FIELDS)%IPOINTER  =  HUGE(JPIM)
  YFIELD_CPL(1:JP_FIELDS)%IGRID     =  PRISM_undefined 
  YFIELD_CPL(1:JP_FIELDS)%MASK      =  PRISM_undefined 
  YFIELD_CPL(1:JP_FIELDS)%CMEM      =  ''   
  YFIELD_CPL(1:JP_FIELDS)%L_SPEC    =  .FALSE. 
  YFIELD_CPL(1:JP_FIELDS)%L_IC      =  .FALSE. 
  IF( CTM == 'MOC') YFIELD_CPL(1:JP_FIELDS)%L_IC =  .TRUE. 
  YFIELD_CPL(1:JP_FIELDS)%L_ACC     =  .FALSE. 
  YFIELD_CPL(1:JP_FIELDS)%IORDER    =  -1_JPIM 

! output fields 
  I=0

! spectral fields
 IF (LOUT_SPC) THEN 
  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_spinf2d' ! spectral info 2d 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  4 ! single level spec
  YFIELD_CPL(I)%CMEM      =  'SPA'
  YFIELD_CPL(I)%L_SPEC    =  .TRUE.
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=1

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_spinf3d'  ! spectral info 3D 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  3 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'SPA'
  YFIELD_CPL(I)%L_SPEC   =  .TRUE.
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=2
  
  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_spsp' ! spectral surface pressure 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  4  ! single level spec
  YFIELD_CPL(I)%CMEM      =  'SPA'
  YFIELD_CPL(I)%L_SPEC    =  .TRUE.
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=3

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_spdiv' ! spectral divergence 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  3 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'SPA'
  YFIELD_CPL(I)%L_SPEC    =  .TRUE.
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=5


  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_spvor' ! spectral divergence 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  3 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'SPA'
  YFIELD_CPL(I)%L_SPEC    =  .TRUE.
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=4

 ENDIF

IF ( .NOT. LOUT_SPC) THEN
  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_u' ! u wind 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YT0%MU
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GMV'  

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_v' ! v wind
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YT0%MV
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GMV'  
ENDIF
 
  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_t' ! temperature
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YT0%MT
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM     =  'GMV'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=7

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_sp' ! surface pressure 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YT0%MSP
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM     =   'GMS'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=8

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_q' ! humidity
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YQ%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=9


 IF (CTM /= 'MOC' ) THEN 
  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_oro' ! level
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  2 ! single 
  YFIELD_CPL(I)%CMEM      =  'COR'
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=6

! land sea mask is cmem GPP
  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_lsm' ! level
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVF'
  YFIELD_CPL(I)%IPOINTER  =  YSD_VF%YLSM%MP 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=18
 ENDIF

IF (CTM == 'TM5' ) THEN
 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_clwc' ! cloud liquid water content
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YL%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=10

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_ciwc' !  cloud ice water content 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YI%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=11

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_cc'  !  cloud cover
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YA%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=12

I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_mfu'  !  mass flux updraft 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YERA40(1)%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=13

I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_mfd'  !  mass flux downdraft 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YERA40(2)%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=14

I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_udr'  !  detrainment updraft 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YERA40(3)%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=15

I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_ddr'  !  detrainment downdraft 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YERA40(4)%MP
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GFL'  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=16

ENDIF


 IF (LOUT_FLUX) THEN

IF (CTM == 'TM5'  ) THEN
 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_u10m' ! 10m wind u  
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%Y10U%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=22

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_v10m' ! 10m wind v  
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%Y10V%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=23

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_d2m' ! 2m dew point   
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%Y2D%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=25

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_t2m' ! 2m temperature   
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%Y2T%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=26

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_ssr' ! surface solar radiation  
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YSSR%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  YFIELD_CPL(I)%L_ACC     =  .TRUE. 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=33

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_skt' ! skin temperature on  
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSP_RR%YT%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SRR' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=34

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_src' ! skin water content n  
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSP_RR%YW%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SRR' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=24

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_cp' ! convecive precipitation   
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YCP%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  YFIELD_CPL(I)%L_ACC     =  .TRUE. 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=31

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_lsp' ! large scale  precipitation   
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YLSP%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  YFIELD_CPL(I)%L_ACC     =  .TRUE. 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=32

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_alb' ! albedo   
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YALB%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=17

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_ci' !sea ice 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVF'
  YFIELD_CPL(I)%IPOINTER  =  YSD_VF%YCI%MP 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=20

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_sst' ! sea surface temperature    
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VF%YSST%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVF' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=21


  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_fsr' ! forecast rougness length    
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YZ0F%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=19

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_sw1' ! soil water layer 1    
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSP_SB%YQ%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SSB' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=36


ENDIF

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_sd' ! snow depth    
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSP_SG%YF%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SSG' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=35

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_qflx' ! latent heat flux 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YSLHF%MP ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  YFIELD_CPL(I)%L_ACC     =  .TRUE. 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=27

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_shflx' ! sensible  heat flux 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YSSHF%MP ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  YFIELD_CPL(I)%L_ACC     =  .TRUE. 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=28

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_ewss' ! x component surface stress 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YIEWSS%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=29

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_nsss' ! y component surface stress 
  YFIELD_CPL(I)%L_OUT     = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  YSD_VD%YINSSS%MP  ! does this work ??
  YFIELD_CPL(I)%IGRID     =  2 ! single level
  YFIELD_CPL(I)%CMEM      =  'SVD' 
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=30

 ENDIF
 ! >>> new transient containing lat/lon/lev/oro/lsm information


   IORDERCTM=(/1,2,3,4,5/) 
   IF (CTM=='TM5') IORDERCTM=(/3,5,1,4,2/)
   IORDERMET=I

! Chemistry fields
 DO JGFL = 1, NGRG 

  IF ( YGRG_NL(JGFL)%NCOUPLO4 >= 1 ) THEN

      YGRG_NL(JGFL)%LDIFF = LDIFF_GRG
      YGRG(JGFL)%LDIFF = LDIFF_GRG
      YGRG_NL(JGFL)%LCONV = LCONV_GRG
      YGRG(JGFL)%LCONV = LCONV_GRG

     CLSPEZ_O4 ='XXXX'

     IF (YGRG_NL(JGFL)%IGRBCODE == 210121 ) CLSPEZ_O4 ='no2  '
     IF (YGRG_NL(JGFL)%IGRBCODE == 210123 ) CLSPEZ_O4 ='co  '
     IF (YGRG_NL(JGFL)%IGRBCODE == 210129 ) CLSPEZ_O4 ='nox '
     IF (YGRG_NL(JGFL)%IGRBCODE == 210122 ) CLSPEZ_O4 ='so2 '
     IF (YGRG_NL(JGFL)%IGRBCODE == 210203 ) CLSPEZ_O4 ='o3  '
     IF (YGRG_NL(JGFL)%IGRBCODE == 210124 ) CLSPEZ_O4 ='ch2o'

     IF (  CLSPEZ_O4 == 'XXXX' ) THEN
            CALL ABOR1('ERROR IN PRISM_DEF, NOT IN SMIOC'//TRIM(YGRG_NL(JGFL)%CNAME) )
     ENDIF

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_conc_'//TRIM(CLSPEZ_O4)   ! ctm concentartion
  YFIELD_CPL(I)%L_IN      = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  HUGE(JPIM)
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GRG'
  YFIELD_CPL(I)%L_IC      =  .TRUE.  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=IORDERMET+(IORDERCTM(JGFL)-1)*3+1

 I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_pchem_'//TRIM(CLSPEZ_O4)   ! ctm production chem
  YFIELD_CPL(I)%L_IN      = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  HUGE(JPIM)
  YFIELD_CPL(I)%IGRID     =  1 ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GRG'
  YFIELD_CPL(I)%L_IC      =  .TRUE.  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=IORDERMET+(IORDERCTM(JGFL)-1)*3+2

  I=I+1
  YFIELD_CPL(I)%CNAME = 'ifs_lchem_'//TRIM(CLSPEZ_O4)   ! ctm loss rates chem
  YFIELD_CPL(I)%L_IN      = .TRUE.
  YFIELD_CPL(I)%IPOINTER  =  HUGE(JPIM)
  YFIELD_CPL(I)%IGRID     =  1  ! Multi level
  YFIELD_CPL(I)%CMEM      =  'GRG'
  YFIELD_CPL(I)%L_IC      =  .TRUE.  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=IORDERMET+(IORDERCTM(JGFL)-1)*3+3

  ENDIF

 ENDDO

! IF (LNOX2NO2) THEN
!  I=I+1
!  YFIELD_CPL(I)%CNAME = 'ifs_conc_no2'   ! ctm conc
!  YFIELD_CPL(I)%L_IN      = .TRUE.
!  YFIELD_CPL(I)%IPOINTER  =  HUGE(JPIM)
!  YFIELD_CPL(I)%IGRID     =  1           ! Multi level
!  YFIELD_CPL(I)%CMEM      =  'GRG'
!  YFIELD_CPL(I)%L_IC      =  .TRUE.  
!  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=YFIELD_CPL(I-1)%IORDER+1
! ENDIF




DO JGFL=1,NGRG   
! only for fields with IC from FDB ! see grg_input
  IF(YGRG_NL(JGFL)%NREQIN  == 1 .AND. YGRG_NL(JGFL)%NCOUPLO4 == 2 ) THEN 
 
   IF (YGRG_NL(JGFL)%IGRBCODE == 210123 ) CLSPEZ_O4 ='co  '
   IF (YGRG_NL(JGFL)%IGRBCODE == 210129 ) CLSPEZ_O4 ='nox '
   IF (YGRG_NL(JGFL)%IGRBCODE == 210121 ) CLSPEZ_O4 ='no2 '
   IF (YGRG_NL(JGFL)%IGRBCODE == 210122 ) CLSPEZ_O4 ='so2 '
   IF (YGRG_NL(JGFL)%IGRBCODE == 210203 ) CLSPEZ_O4 ='o3  '
   IF (YGRG_NL(JGFL)%IGRBCODE == 210124 ) CLSPEZ_O4 ='ch2o'

   I =I+1
   YFIELD_CPL(I)%CNAME = 'ifs_out_'//TRIM(CLSPEZ_O4)
   YFIELD_CPL(I)%L_OUT     = .TRUE.
   YFIELD_CPL(I)%IPOINTER  =  YGRG(JGFL)%MP
   YFIELD_CPL(I)%IGRID     =  1 ! Multi level
   YFIELD_CPL(I)%CMEM      =  'GFL'  
   YFIELD_CPL(I)%L_IC      =  .TRUE.  
  IF (CTM=='TM5')  YFIELD_CPL(I)%IORDER=MAXVAL(YFIELD_CPL(1:I)%IORDER)+1
  ENDIF

ENDDO  

  NBR_FIELDS=I

! determine order if undefined
 IF (CTM/='TM5') THEN 
  DO IK=1,NBR_FIELDS
   YFIELD_CPL(IK)%IORDER = IK  
  ENDDO 
 ENDIF

!re-arange order
 ICOUNT_ORDER=0
 DO IK=1,NBR_FIELDS
  IF (YFIELD_CPL(IK)%IORDER < 1 .OR. YFIELD_CPL(IK)%IORDER > NBR_FIELDS) CALL ABOR1(' Re-arrange mismatch') 
  DO IK1=1,NBR_FIELDS
   IF (YFIELD_CPL(IK1)%IORDER==IK) THEN 
    YLFIELD_CPL_D=YFIELD_CPL(IK)
    YFIELD_CPL(IK)=YFIELD_CPL(IK1)
    YFIELD_CPL(IK1)=YLFIELD_CPL_D
    ICOUNT_ORDER=ICOUNT_ORDER+1
    EXIT 
   ENDIF  
  ENDDO 
 ENDDO 

 DO IK=1,NBR_FIELDS
  WRITE(NULERR,*) ' IORDER AFTER ', IK , YFIELD_CPL(IK)%CNAME 
 ENDDO

  IF (ICOUNT_ORDER /=  NBR_FIELDS) CALL ABOR1(' Re-arrange mismatch total') 

! Number of fields wich need to de-accumulated
  ALLOCATE (INDEX_RPREVACC(COUNT(YFIELD_CPL(:)%L_ACC)))
  ICOUNT_ACC=0


  IVAR_TYPE = PRISM_Double_Precision

  DO IK=1,NBR_FIELDS

  CALL MPL_BARRIER(CDSTRING='COUPLO4_DEFINITIONS LOOP FIELDS ')

! Test for invalid  pointer 
     IF(YFIELD_CPL(IK)%IPOINTER < 0 ) THEN
       YFIELD_CPL(IK)%L_IN      = .FALSE.
       
       WRITE(NULERR,*) 'NOT AVAILAIBLE IN RUN ', TRIM(YFIELD_CPL(IK)%CNAME) 
       CYCLE      
      ENDIF  

! Number of fields wich need to de-accumulated
   IF ( YFIELD_CPL(IK)%L_ACC ) THEN 
    ICOUNT_ACC=ICOUNT_ACC+1
    
   ENDIF
 
! Test if process is included in exchange
    IF  (.NOT.LLCALLPSMILEGP  .AND. (.NOT. YFIELD_CPL(IK)%L_SPEC)) CYCLE
    IF  (.NOT.LLCALLPSMILESP  .AND.        YFIELD_CPL(IK)%L_SPEC)  CYCLE


! Only MPI tasks  with surface spectral fields  
    IF  ( .NOT. LGLOBALEXCHANGESP .AND. YFIELD_CPL(IK)%IGRID == 4  .AND.  NPSP /= 1) THEN
         YFIELD_CPL(IK)%L_IN      = .FALSE.
         YFIELD_CPL(IK)%L_OUT     = .FALSE.
         CYCLE
    ENDIF

    IVAR_NODIMS(1) = SIZE(ISHAPE_GP,2)  !  no bundles or vectors
    IF (YFIELD_CPL(IK)%L_SPEC) IVAR_NODIMS(1)=  SIZE(ISHAPE_D,2) !
    IVAR_NODIMS(2) = 0

    ISHAPE_D = 1  
    ISHAPE_D(1:2,1:2) = YIFS_GRID(YFIELD_CPL(IK)%IGRID)%ISHAPE(1:2,1:2)
    
    IGR = YFIELD_CPL(IK)%IGRID

     CALL PRISM_DEF_VAR ( YFIELD_CPL(IK)%ID,  &
          & YFIELD_CPL(IK)%CNAME,  YIFS_GRID(IGR)%ID, & 
          & YIFS_GRID(IGR)%ID_POINTS, YFIELD_CPL(IK)%MASK, &
          & IVAR_NODIMS, ISHAPE_D(1:2,1:IVAR_NODIMS(1)), IVAR_TYPE, IERROR )

      WRITE(NULERR,*) ' AFTER CALL PRISM_DEF_VAR ID ', IK, IK1,&
     &   TRIM(YFIELD_CPL(IK)%CNAME),YFIELD_CPL(IK)%ID,YFIELD_CPL(IK)%L_ACC, IERROR

     IF (IERROR /= 0 .OR. YFIELD_CPL(IK)%ID < 0 ) THEN
      WRITE(NULERR,*) ' VARIABLE ',TRIM(YFIELD_CPL(IK)%CNAME), & 
&     ' NOT IN SMIOC - NO EXCHANGE'
          YFIELD_CPL(IK)%L_IN      = .FALSE.
          YFIELD_CPL(IK)%L_OUT     = .FALSE.
          IF  (LGLOBALEXCHANGEGP .OR. LGLOBALEXCHANGESP ) THEN 
            CALL ABOR1('ERROR IN PRISM_DEF, GLOBALEXCHANGE but NOT IN SMIOC')
          ENDIF
     ENDIF 
  ENDDO

!#IFDEF TIMING
!   CALL MPI_BARRIER ( COMM_PSMILE, IERROR )
!  TIC=MPI_WTIME()
!#ENDIF

! ----------------------------------------------------------------------
!                    5.  END OF DEFINITIONS 
! ----------------------------------------------------------------------

  CALL GSTATS(261,0)
  CALL PRISM_ENDDEF ( IERROR )
  CALL GSTATS(261,1)

  IF (IERROR /= 0) I_ERRORS = I_ERRORS + 1

  WRITE(NULERR,*) " PRISM_ENDDEF FINISHED "

  IF (I_ERRORS > 0) CALL ABOR1('ERROR IN PRISM_SET_GRIDDEF')

!#IFDEF TIMING
!  TOC=MPI_WTIME()
!  WRITE ( * , 9930 ) TRIM(COMP_NAME), MYPE, TOC-TIC
!#ENDIF#

IF (ALLOCATED(ZCOORDS_1)) DEALLOCATE(ZCOORDS_1) 
IF (ALLOCATED(ZCOORDS_2)) DEALLOCATE(ZCOORDS_2)
IF (ALLOCATED(ZCOORDS_3)) DEALLOCATE(ZCOORDS_3)

IF (ALLOCATED(ZCORNERS_1)) DEALLOCATE(ZCORNERS_1)
IF (ALLOCATED(ZCORNERS_2)) DEALLOCATE(ZCORNERS_2)
IF (ALLOCATED(ZCORNERS_3)) DEALLOCATE(ZCORNERS_3)

IF (ALLOCATED(IPARTITION))  DEALLOCATE(ZRGAUSLAT)
IF (ALLOCATED(IEXTENT))  DEALLOCATE(IEXTENT)
IF (ALLOCATED(ZLOCAL))  DEALLOCATE(ZLOCAL)
IF (ALLOCATED(ZGLOBAL))  DEALLOCATE(ZGLOBAL)
IF (ALLOCATED(ITO))  DEALLOCATE(ITO)

IF (ICOUNT_ACC > 0 ) THEN 
 ALLOCATE( RPREVACC ( NGPTOT, ICOUNT_ACC)) 
 RPREVACC = 0.0_JPRB
ENDIF


!----------------------------------------------------------------------
!                  6.   Appendix 2 Read file with infor for GRG photolysis calculations
! ----------------------------------------------------------------------

IF (NGRG > 0 .AND. LNOX2NO2 ) THEN
fint=999.99
  ALLOCATE (ZFINT(PRODUCT(SHAPE(FINT))))

 IF( MYPROC == JPIOMASTER )THEN
   llfileok=.false.
  INQUIRE(FILE=TRIM(clgrg_fint),EXIST=llfileok )

  IF(.NOT. llfileok)THEN
    WRITE(NULERR,*) 'ERROR couplo4_definitions: Cannot read GRG file '//TRIM(clgrg_fint)
    CALL ABOR1(': FILE couplo4_definitions: grg_fintbw does not exist.')
  ELSE
    WRITE(NULERR,*) 'INFO couplo4_definitions : Opening GRG file '//TRIM(clgrg_fint)
    OPEN(IUTMP,FORM='FORMATTED',ACTION='READ',STATUS='OLD',   &
     & FILE=TRIM(clgrg_fint))
    READ(IUTMP,*) fint
    CLOSE(IUTMP)
    WRITE(NULERR,*) 'INFO  couplo4_definitions: Finished reading GRG file '//TRIM(clgrg_fint)
  ENDIF
 ENDIF
 IF (NPROC > 0) THEN
  ZFINT=RESHAPE(FINT,SHAPE(ZFINT))
  CALL MPL_BROADCAST (ZFINT,MTAGRAD,JPIOMASTER,CDSTRING='COUPLO4_DEFINITIONS FINT')
  FINT=RESHAPE(ZFINT,SHAPE(FINT))
 ENDIF

 DEALLOCATE (ZFINT)

!write(*,*)' fint stats'
!do i=0,102
!write(*,*)i,fint(i,10,2),fint(i,18,2),fint(i,10,6),fint(i,18,6) 
!enddo

ENDIF

IF (LHOOK) CALL DR_HOOK('COUPLO4_DEFINITIONS',1,ZHOOK_HANDLE)
END SUBROUTINE COUPLO4_DEFINITIONS
