SUBROUTINE COUPLO4_GRG_INPUT( CNAME, PLOCGP, LDFIRST)
!***** CUOPL04_GRG_INPUT 

!**   PURPOSE.
!     --------
!      Put CTM input from Coupler Oasis4 in to GRG auxilary array of
!      module YOM_GRGCTM 

!**   INTERFACE.
!     ----------
!              *COUPLEO4_exchange (CNAME,PLOCGP, LDFIRST) 

!        Explicit arguments :
!        --------------------
!              CNAME  (INPUT)         :  transient name 
!              PLOCGP (INPUT)         :  data array 
!              LDFIRST (INPUT)         :  FIRST call 
!     AUTHOR.
!     -------
!     2006-05-24  J. Flemming

 

  USE PARKIND1  ,ONLY : JPIM, JPRB

  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  USE YOMLUN   , ONLY : NULERR

  USE YOMGEM   , ONLY : NGPTOT, VAH, VBH    

  USE YOM_YGFL , ONLY : YGFL, YGFLC, YGRG_NL, NGRG, NGRG_CPLO4, LGRGSFC

  USE YOMDIM   , ONLY : NFLEVG, NPROMA,  NGPBLKS

  USE YOMGFL   , ONLY : GFL, GFLT1
 
  USE YOMCOUPLO4, ONLY : CTM, LDIA_GRG

  USE YOMGMV   , ONLY : GMVS, YT0

  USE SURFACE_FIELDS_MIX, ONLY : SD_XA, YSD_XAD, YSD_XA, YSD_VF, SD_VF

  IMPLICIT NONE 

!-----------------------------------------------------------------------

REAL (KIND=JPRB), INTENT(IN)      :: PLOCGP(NPROMA,NFLEVG,NGPBLKS)  
CHARACTER(LEN=128), INTENT(IN)    :: CNAME
LOGICAL, INTENT(IN)               :: LDFIRST 

LOGICAL           :: LLCUT_TOP 
 
INTEGER(KIND=JPIM):: IGRIB, IL, IGRG, I, IGFL, IGFL_TEND, JGFL, JSTGLO, IEXTRA
INTEGER(KIND=JPIM):: ICEND, ISTC, IBL, JLEV, J, ICUTTOP

REAL (KIND=JPRB), ALLOCATABLE ::  ZPRESH(:,:,:)  ! (NPROMA,NFLEVG+1,IBLK) 
REAL (KIND=JPRB), ALLOCATABLE ::  ZPRESF(:,:,:)  ! (NPROMA,NFLEVG,IBLK)  
REAL (KIND=JPRB), ALLOCATABLE ::  ZDELP(:,:,:)  ! (NPROMA,NFLEVG+1,IBLK) 
REAL (KIND=JPRB), ALLOCATABLE ::  ZTC(:,:)  ! (NPROMA,IBLK) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZFACTOR

#include "gppre.intfb.h"
#include "gptco3.intfb.h"


!################  subroutine start ################

IF (LHOOK) CALL DR_HOOK('COUPLO4_GRG_INPUT',0,ZHOOK_HANDLE)


  LLCUT_TOP=.TRUE. ! cut of tope levels until ICUTTOP
  ICUTTOP=5

! find IGRG, i.e. index  of GRG species in current run
  IL     =   LEN(TRIM(CNAME))
  IGRIB=-999
  IF (CNAME(IL-2:IL) == '_co')  IGRIB = 210123
  IF (CNAME(IL-2:IL) == '_o3')  IGRIB = 210203
  IF (CNAME(IL-2:IL) == 'nox')  IGRIB = 210129
  IF (CNAME(IL-2:IL) == 'no2')  IGRIB = 210121 
  IF (CNAME(IL-2:IL) == 'so2')  IGRIB = 210122
  IF (CNAME(IL-3:IL) == 'ch2o') IGRIB = 210124

! Find species within NGRG range   
 IGRG=0
 DO I=1,NGRG   
   IF (YGRG_NL(I)%IGRBCODE == IGRIB ) IGRG=I 
 ENDDO

! Find species within GFL range 
 IGFL=0 
 DO JGFL=1,YGFL%NUMFLDS
   IF(YGFLC(JGFL)%IGRBCODE == IGRIB  )  IGFL=JGFL
 ENDDO
   
! Find itendency within GFL range 
 IGFL_TEND=0 
 DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%CNAME == TRIM(YGRG_NL(IGRG)%CNAME)//'_tend'  ) IGFL_TEND=JGFL
 ENDDO
  
IF (IGRG == 0 ) THEN
    WRITE(NULERR,*) 'NO IFS GFL SPECIES FOR CTM input  ', CNAME  

!  put no2 in extra field 116 
!  IF (LDIA_GRG .AND. YSD_XAD%NDIM >= 16 .AND. TRIM(CNAME) == 'ifs_conc_no2')THEN
!    WRITE(NULERR,*) ' put no2 in extra field 116 '
!    DO JSTGLO=1,NGPTOT,NPROMA
!      ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
!      ISTC=1
!      IBL=(JSTGLO-1)/NPROMA+1
!      DO JLEV=1,NFLEVG
!        DO J=ISTC,ICEND
!          SD_XA(J,JLEV,16, IBL)   =  PLOCGP (J,JLEV,IBL) 
!        ENDDO
!      ENDDO
!    ENDDO
!  ENDIF

   IF (LHOOK) CALL DR_HOOK('COUPLO4_GRG_INPUT',1,ZHOOK_HANDLE)
   RETURN
 ENDIF
    
!  WRITE(NULERR,*) 'FOUND IFS SPECIES FOR CTM input  ', CNAME

! Put CTM input in GRG auxilliary arrays 

 SELECT CASE ( CNAME(1:9))
 
 CASE('ifs_pchem' ) 

   IEXTRA  =  IGRG + 2 *NGRG_CPLO4 

! calculate total column of pchem field as surface flux of coupled species
! allocate half and full level pressures for convertions

 IF( YGRG_NL(IGRG)%LDIFF ) THEN

  IF (.NOT. LGRGSFC) CALL ABOR1(' LGRGSFC MUST BE TRUE IF LDIFF')

  IF(.NOT.(ALLOCATED(ZPRESH)))ALLOCATE(ZPRESH(NPROMA,0:NFLEVG,NGPBLKS))
  IF(.NOT.(ALLOCATED(ZPRESF)))ALLOCATE(ZPRESF(NPROMA,1:NFLEVG,NGPBLKS))
  IF(.NOT.(ALLOCATED(ZDELP)))ALLOCATE(ZDELP(NPROMA,0:NFLEVG+1,NGPBLKS))
  IF(.NOT.(ALLOCATED(ZTC)))  ALLOCATE( ZTC(NPROMA,NGPBLKS))

   ZPRESH(:,:,:)  = 0.0_JPRB
   ZTC(:,:) = 0.0_JPRB
   ZDELP(:,:,:) = 0.0_JPRB
   ZPRESH(:,:,:) = 0.0_JPRB
   ZPRESF(:,:,:) = 0.0_JPRB
 
! calculate p and delta p
   DO JSTGLO=1,NGPTOT,NPROMA
       ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
       ISTC=1
       IBL=(JSTGLO-1)/NPROMA+1
       ZPRESH(ISTC:ICEND,NFLEVG,IBL)  =  EXP(GMVS(ISTC:ICEND, YT0%MSP,IBL)) 
       CALL GPPRE(NPROMA,ISTC,ICEND, NFLEVG, VAH, VBH,  ZPRESH(:,:,IBL), ZPRESF(:,:,IBL))
       DO I=1, NFLEVG
         ZDELP(:,I,IBL)=  ZPRESH(:,I,IBL) -  ZPRESH(:,I-1,IBL)
       ENDDO
   ENDDO

     ZTC(:,:)=0.0_JPRB
! caluclate total col
       DO JSTGLO=1,NGPTOT,NPROMA
        ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        ISTC=1
        IBL=(JSTGLO-1)/NPROMA+1
         CALL GPTCO3(NPROMA,ISTC,ICEND, NFLEVG,ZDELP(:,:,IBL), & 
           & PLOCGP(:,:,IBL),ZTC(:,IBL))
       ENDDO

      SD_VF(:,YSD_VF%YGRGFLX(IGRG)%MP,:) = ZTC(:,:) 

  WRITE(NULERR,*) ' GRG FLUX ', IGFL_TEND, MINVAL(ZTC),MAXVAL(ZTC)
  IF((ALLOCATED(ZPRESH)))DEALLOCATE(ZPRESH)
  IF((ALLOCATED(ZPRESF)))DEALLOCATE(ZPRESF)
  IF((ALLOCATED(ZDELP)))DEALLOCATE(ZDELP)
  IF((ALLOCATED(ZTC)))  DEALLOCATE(ZTC)

  IF(IGFL_TEND > 0 ) THEN 
! tendency in in glf field to be substracted from lchem  
!   IF (CTM /= 'MOZ' ) THEN
      DO JSTGLO=1,NGPTOT,NPROMA
        ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        ISTC=1
        IBL=(JSTGLO-1)/NPROMA+1
         DO JLEV=1,NFLEVG
          DO J=ISTC,ICEND
           GFL(J,JLEV,YGFLC(IGFL_TEND)%MP,IBL) = -1.0_JPRB * PLOCGP(J,JLEV,IBL)
          ENDDO
         ENDDO
      ENDDO
   ENDIF
 ELSE ! LDIFF
  IF (LGRGSFC) SD_VF(:,YSD_VF%YGRGFLX(IGRG)%MP,:) = 0.0_JPRB
 ENDIF 

 CASE('ifs_lchem' )

 IEXTRA=   IGRG +NGRG_CPLO4 
  
 IF(IGFL_TEND > 0 ) THEN 
! tendency total in gfl field 
     DO JSTGLO=1,NGPTOT,NPROMA
      ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
      ISTC=1
      IBL=(JSTGLO-1)/NPROMA+1
      DO JLEV=1,NFLEVG
          DO J=ISTC,ICEND
           IF( YGRG_NL(IGRG)%LDIFF ) THEN
           GFL(J,JLEV,YGFLC(IGFL_TEND)%MP,IBL) = GFL(J,JLEV,YGFLC(IGFL_TEND)%MP,IBL) + PLOCGP(J,JLEV,IBL)
           ELSE
           GFL(J,JLEV,YGFLC(IGFL_TEND)%MP,IBL) = PLOCGP(J,JLEV,IBL)
           ENDIF
          ENDDO
      ENDDO
    ENDDO
 
   WRITE(NULERR,*) 'TEND IN GFL', IGFL_TEND, MINVAL(PLOCGP),MAXVAL(PLOCGP)

  ENDIF 

 CASE ('ifs_conc_') 

   IEXTRA=IGRG

 ! Initial conditions from CTM into GFL fields if not from MARS  
      IF (LDFIRST.AND.( YGRG_NL(IGRG)%NREQIN == 0 ) ) THEN
       DO JSTGLO=1,NGPTOT,NPROMA
        ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        ISTC=1
        IBL=(JSTGLO-1)/NPROMA+1
          DO JLEV=1,NFLEVG
           DO J=ISTC,ICEND
           GFL(J,JLEV,YGFLC(IGFL)%MP, IBL)     = MAX(PLOCGP(J,JLEV,IBL),0.0_JPRB) 
           GFL(J,JLEV,YGFLC(IGFL)%MP9,IBL)     = MAX(PLOCGP(J,JLEV,IBL),0.0_JPRB) 
           GFLT1(J,JLEV,YGFLC(IGFL)%MP1,IBL)   = MAX(PLOCGP(J,JLEV,IBL),0.0_JPRB)
           ENDDO
         ENDDO
       ENDDO
    
     ENDIF   !  lfirst

 END SELECT 

! Put into extra arrays
  IF (YSD_XAD%NDIM >= IEXTRA .AND. LDIA_GRG ) THEN 
! additionally in extra array
   ZFACTOR= 1.0E10
   IF (CNAME(1:9) == 'ifs_conc_' ) ZFACTOR= 1.0_JPRB
   WRITE(NULERR,*)  CNAME, ' IN ',IEXTRA,' GRIB ',YSD_XA%YXA(IEXTRA)%IGRBCODE,' FACTOR ', ZFACTOR
   DO JSTGLO=1,NGPTOT,NPROMA
      ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
      ISTC=1
      IBL=(JSTGLO-1)/NPROMA+1
      DO JLEV=1,NFLEVG
          DO J=ISTC,ICEND
          SD_XA(J,JLEV,IEXTRA, IBL)   =  PLOCGP (J,JLEV,IBL) * ZFACTOR 
        ENDDO
      ENDDO
    ENDDO
!   IF (LLCUT_TOP) THEN
!       DO JLEV = 1, ICUTTOP-1  
!         SD_XA(:,JLEV,IEXTRA,:) = SD_XA(:,ICUTTOP,IEXTRA,:)
!       ENDDO 
!   ENDIF 
  ENDIF


IF (LHOOK) CALL DR_HOOK('COUPLO4_GRG_INPUT',1,ZHOOK_HANDLE)

END SUBROUTINE COUPLO4_GRG_INPUT
