SUBROUTINE COUPLO4_INIMPI

!**** *INIOASIS4*  - Initialize coupled mode communication

!     Purpose.
!     --------
!     Initialize coupler to get the MPI communicator

!**   Interface.
!     ----------
!       *CALL*  *INIOASIS4*

!     Input:
!     -----

!     Output:
!     ------
!       MPLUSERCOMM in MPL_MODULE is updated.

!     Method:
!     ------
!       OASIS4 usage is controlled by environment variables
!     Externals:
!     ---------
!       GETENV - Get enviroment variables

!     Reference:
!     ---------
!       S. Valcke, 2006: OASIS3 User Guide (prism_2-5), 
!       PRISM Support Initiative Report No 3,
!       CERFACS, Toulouse, France, 64 pp.

!     Author:
!     -------
!       Johannes Flemming - based on K. Mogensen, INOASI3

!     Modifications.
!     --------------

!     -----------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE MPL_MODULE  
USE YOMCT0 , ONLY : LCOUPLO4_ENV
USE COUPLO4_MIX, ONLY : COMP_NAME ,  ICOMP_ID 
 

USE PRISM_DUMMY_MOD

IMPLICIT NONE

INTEGER(KIND=JPIM) :: ILOCAL_COMM 
INTEGER(KIND=JPIM) :: IERROR

CHARACTER(LEN=128) :: CLOASIS4

REAL(KIND=JPRB) :: ZHOOK_HANDLE


!################  subroutine start ################

! Get the enviroment variables


CALL GETENV('OASIS4RUN',CLOASIS4)

WRITE(*,'(2A)')'OASIS4 environment variable           : ',TRIM(CLOASIS4)

LCOUPLO4_ENV=.FALSE.

LCOUPLO4_ENV = (CLOASIS4 == 'yes') 

IF ( LCOUPLO4_ENV ) THEN 
! to allow multiple openMP thread number for mpmd jobs
!call jfh_bind()

  WRITE ( 0 , * ) 'BEFORE PRISM_INIT in IFS '
  CALL PRISM_INIT (COMP_NAME, IERROR )
  CALL PRISM_INIT_COMP (ICOMP_ID, COMP_NAME, IERROR )
  WRITE ( 0 , * ) 'AFTER PRISM_II 0N IFS '
  WRITE ( 0 , * )
  WRITE ( 0 , * ) '---------------------------------------------------------------------'
  WRITE ( 0 , * ) 'I AM COMPONENT ', TRIM(COMP_NAME), ' IN APPLICATION IFS'
  WRITE ( 0 , * ) '---------------------------------------------------------------------'
  WRITE ( 0 , * ) 'COMPONENT ID ',ICOMP_ID,' LOCALCOM ', ILOCAL_COMM
  CALL PRISM_GET_LOCALCOMM ( ICOMP_ID, ILOCAL_COMM, IERROR )
  IF (IERROR /= 0) CALL ABOR1('Error getting local communicator from OASIS3')
  LMPLUSERCOMM=.TRUE.
  MPLUSERCOMM=ILOCAL_COMM

  IF (LHOOK) CALL DR_HOOK('COUPLO4_INIMPI',0,ZHOOK_HANDLE)
  IF (LHOOK) CALL DR_HOOK('COUPLO4_INIMPI',1,ZHOOK_HANDLE)

ENDIF

END SUBROUTINE COUPLO4_INIMPI
