SUBROUTINE PERTSEKF_V2

!**** *PERTSEKF*  - extracts soil moisture fields from working file

!     Purpose.
!     --------
!     (1) Extract the initial soil moisture fields from
!     gridpoint buffer.

!**   Interface.
!     ----------
!        *CALL* *PERTSEKF(...)

!        Explicit arguments :      None
!        --------------------

!        Implicit arguments :      None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M. Drusch  *ECMWF*

!     Modifications.
!     --------------
!         2008-08-03 - K.Scipal - prepared chess type pertubation 
!                    (this only works with two pert. runs, which is
!                     currently not implemented)
!         2009-02-10 - P de Rosnay - coding standard

!     ------------------------------------------------------------------

USE PARKIND1 , ONLY      : JPIM, JPRB
USE YOMHOOK  , ONLY      : LHOOK, DR_HOOK
USE YOMLUN   , ONLY      : NULOUT
USE YOMDIM   , ONLY      : NPROMA
USE YOMGEM   , ONLY      : NGPTOT
USE SURFACE_FIELDS_MIX, ONLY : SP_SB, YSP_SB
USE YOMSEKF  , ONLY      : N_SEKF_PT, VSM_PERT_INC, PERT_CHESS

IMPLICIT NONE

REAL(KIND=JPRB)     ::  ZSM_PERT_INC
REAL(KIND=JPRB)     ::  ZHOOK_HANDLE

INTEGER(KIND=JPIM)  ::  IEND, IST, IBL, I_FLD_SM
INTEGER(KIND=JPIM)  ::  JKGLO, JROF

! LOCAL VARIABLE:
INTEGER(KIND=JPIM)  ::  IZSIGN

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PERTSEKF_V2',0,ZHOOK_HANDLE)

IF (N_SEKF_PT==0) THEN
  WRITE(NULOUT,*) 'No perturbations, reference run.'
  ZSM_PERT_INC = 0.0000
  WRITE(NULOUT,*) 'Soil moisture increment set to: ', ZSM_PERT_INC
  I_FLD_SM = 0_JPIM
ENDIF
IF (N_SEKF_PT==1) THEN
  WRITE(NULOUT,*) 'Perturb soil moisture layer 1'
  ZSM_PERT_INC = VSM_PERT_INC(1)
  WRITE(NULOUT,*) 'Soil moisture increment set to: ', ZSM_PERT_INC
  I_FLD_SM = 1_JPIM
ENDIF
IF (N_SEKF_PT==2) THEN
  WRITE(NULOUT,*) 'Perturb soil moisture layer 2'
  ZSM_PERT_INC = VSM_PERT_INC(2)
  WRITE(NULOUT,*) 'Soil moisture increment set to: ', ZSM_PERT_INC
  I_FLD_SM = 2_JPIM
ENDIF
IF (N_SEKF_PT==3) THEN
  WRITE(NULOUT,*) 'Perturb soil moisture layer 3'
  ZSM_PERT_INC = VSM_PERT_INC(3)
  WRITE(NULOUT,*) 'Soil moisture increment set to: ', ZSM_PERT_INC
  I_FLD_SM = 3_JPIM
ENDIF

!P. de Rosnay for the  future 
! Simplify the 25 lines above by:
! ZSM_PERT_INC = 0.0_JPRM
! IF(N_SEKF_PT>0) ZSM_PERT_INC = VSM_PERT_INC(N_SEKF_PT)
! WRITE(NULOUT,*) 'Soil moisture increment set to: ', ZSM_PERT_INC

IF (I_FLD_SM > 0_JPIM) THEN
  IST=1
  DO JKGLO=1,NGPTOT,NPROMA
    IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    DO JROF = IST, IEND
      IF (SP_SB(JROF,I_FLD_SM,YSP_SB%YQ%MP0,IBL)>0.0) THEN
         IZSIGN=1.0_JPRB
         IF (PERT_CHESS) THEN
           IZSIGN=SIGN(1.0,-0.5+MOD(JROF,2))
         ENDIF
         SP_SB(JROF,I_FLD_SM,YSP_SB%YQ%MP0,IBL)=SP_SB(JROF,I_FLD_SM,YSP_SB%YQ%MP0,IBL)+IZSIGN*ZSM_PERT_INC
      ENDIF
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PERTSEKF_V2',1,ZHOOK_HANDLE)
END SUBROUTINE PERTSEKF_V2
