SUBROUTINE SEKF_MAGN_RH(PTD2M,PT2M)

!**** *SEKF_MAGN_RH*  - compute rH from the Magnus equation

!     Input parameters.
!     -----------------

!     ZTD2M      :  2 m dewpoint temperature
!     ZT2M       :  2 m temperature

!     Purpose.
!     --------
!     (1) Compute rH from the magnus equation

!**   Interface.
!     ----------
!        *CALL* *SEKF_MAGN_RH(...)

!        Explicit arguments :      None
!        --------------------

!        Implicit arguments :      None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M. Drusch  *ECMWF*

!     Modifications.
!     --------------
!      2009-02-10 - P. de Rosnay - ECMWF coding standard

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK

IMPLICIT NONE

REAL(KIND=JPRB)     ::  ZHOOK_HANDLE
REAL(KIND=JPRB), INTENT(INOUT)  ::  PTD2M
REAL(KIND=JPRB), INTENT(IN)     ::  PT2M
REAL(KIND=JPRB)     ::  ZC1, ZC2, ZC3, ZC4
REAL(KIND=JPRB)     ::  ZE1, ZE2

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SEKF_MAGN_RH',0,ZHOOK_HANDLE)

! Set constants for Magnus Equation

ZC1 = 611.21_JPRB
ZC2 = 17.502_JPRB
ZC3 = 273.16_JPRB
ZC4 =  32.19_JPRB

! Compute relative humidity

ZE1 = ZC1 * EXP(ZC2*(PTD2M-ZC3)/(PTD2M-ZC4))
ZE2 = ZC1 * EXP(ZC2*(PT2M-ZC3)/(PT2M-ZC4))

PTD2M = 100._JPRB * ZE1 / ZE2

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SEKF_MAGN_RH',1,ZHOOK_HANDLE)

END SUBROUTINE SEKF_MAGN_RH
