SUBROUTINE STORE_SEKF_CV(KIDIA,KFDIA,KLON,&
 & KSTGLO, KSTEP,&
 & PVSM_ML, PT2M_TEMP, PTD2M_TEMP)

!**** *STORE_SEKF_CV*  - store soil moisture and 2 m temperature fields from 
!                        the fg and perturbed SEKF model runs

!     Input parameters.
!     -----------------

!     KIDIA      :  Start of horizontal loop
!     KFDIA      :  End of horizontal loop
!     KLON       :  Horizontal dimension
!     KSTEP      :  Actual model time step
!     KSTGLO     :  Block number
!     PVSM_ML    :  Multi layer volumetric soil moisture at time t
!     PT2M_TEMP  :  Temperature at 2 m at time t
!     PTD2M_TEMP :  Dew point temperature at 2 m at time t

!     Purpose.
!     --------
!     (1) Store soil moisture and screen level parameter fields from the SEKF runs

!**   Interface.
!     ----------
!        *CALL* *STORE_SEKF_CV(...)

!        Explicit arguments :      None
!        --------------------

!        Implicit arguments :      None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M. Drusch  *ECMWF*

!     Modifications.
!     --------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK
USE YOMDIM   , ONLY : NPROMA
USE YOMSEKF  , ONLY : T2M_FG, TD2M_FG, VOL_SM_FG, NSLAY, N_SEKF_PT

IMPLICIT NONE

REAL(KIND=JPRB)     ::  ZHOOK_HANDLE
REAL(KIND=JPRB), INTENT(IN)     ::  PVSM_ML(NPROMA,NSLAY)
REAL(KIND=JPRB), INTENT(IN)     ::  PT2M_TEMP(NPROMA), PTD2M_TEMP(NPROMA)

INTEGER(KIND=JPIM), INTENT(IN)  ::  KLON, KIDIA, KFDIA, KSTEP
INTEGER(KIND=JPIM), INTENT(IN)  ::  KSTGLO
INTEGER(KIND=JPIM)              ::  IBLK, JL, JK

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STORE_SEKF_CV',0,ZHOOK_HANDLE)

! store volumetric soil moisture
IBLK=(KSTGLO-1)/KLON + 1
DO JK=1,NSLAY
  DO JL=KIDIA,KFDIA
    VOL_SM_FG(JL,JK,IBLK,KSTEP,N_SEKF_PT+1) = PVSM_ML(JL,JK)
  ENDDO
ENDDO

! store screen level parameters
IBLK=(KSTGLO-1)/KLON + 1
DO JL=KIDIA,KFDIA
  T2M_FG(JL,IBLK,KSTEP,N_SEKF_PT+1) = PT2M_TEMP(JL)
  TD2M_FG(JL,IBLK,KSTEP,N_SEKF_PT+1) = PTD2M_TEMP(JL)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STORE_SEKF_CV',1,ZHOOK_HANDLE)
END SUBROUTINE STORE_SEKF_CV
