SUBROUTINE SUSEKF

USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK

USE YOMDIM    ,ONLY : NPROMA, NFLEVG, NGPBLKS
USE YOMCT0    ,ONLY : NSTART, NSTOP
USE YOMGEM    ,ONLY : NGPTOT
USE YOMLUN    ,ONLY : NULNAM, NULOUT
USE SURFACE_FIELDS_MIX  , ONLY : YSP_SBD
USE YOMDPHY  , ONLY : NTILES, NCOM !,NCSNEC

USE YOMSEKF   ,ONLY : LUSE_JATM, N_SEKF_CV, N_SEKF_PT, &
 & N_SEKF_OUPT, NSLAY, FKF_TENT_1, &
 & FKF_TENT_2, FKF_TENT_3, FKF_TENT_4, FKF_TENT, FKF_TENQ, FKF_TENU, &
 & FKF_TENV, FKF_SURF_SO, FKF_SURF_TH, FKF_SURF_CR, FKF_SURF_LR, &
 & VOL_SM_FG, T2M_FG, TD2M_FG, &
 & VSM_PERT_INC, VSM_PERT_RES, LUSEKF_REF, LUSE_T2M, &
 & LUSE_RH2M, LUSE_ASCAT, NOBS_SCREEN, NOBS_SEKF_MAX, &
 & NSCREENTIME, NOBS_TSTEP, NANATIME, NANA_TSTEP, &
 & NSCREENDATE, NANADATE, &
 & VSM_ANAL_INC, SEKF_INC, SEKF_F, VSM_INC_L1, VSM_INC_L2, VSM_INC_L3, &
 & NREF_BLOCK, NREF_GP, SEKF_Q, &
 & SEKF_DEBUG_3D, SEKF_DEBUG, &
 & SEKF_A, SEKF_B, SEKF_G, SEKF_J, NANAL, &
 & MOD_ERR, BACK_ERR, ANA_ERR, T2M_ERR, RH2M_ERR, ASCAT_ERR, &
 & PERT_CHESS, &
 ! temporary variables for offline jacobians
 ! for vdfouter
 & SKF_ZWSA,SKF_ZTSA, SKF_ZAZ0M,SKF_ZAZ0H, &
 & SKF_PTENT_CUM,SKF_PTENQ_CUM,&
 & SKF_PVDIS,SKF_PVDISG, SKF_ZDISSGW, SKF_PFTLHEV, SKF_PFTLHSB,&
 & SKF_PFWSB,SKF_ZBIR,SKF_ZVAR, &
 & SKF_PUCFL,SKF_PVCFL,SKF_PTCFL,SKF_PDCFL,&
 & SKF_PQCFL,SKF_ZZINV,SKF_PBLH,ISKF_IHPBLN, &
 & SKF_PU10N,SKF_PV10N,SKF_PUST,&
 & SKF_ZFRSOTI,SKF_ZEVAPSNW,SKF_ZGUST,&
 & SKF_ZWUAVG,LSKF_LLNODECP,ISKF_IPBLTYPE, &
 & SKF_PTENT,SKF_PTENQ,SKF_ZTENL,SKF_ZTENI,SKF_ZTENA,SKF_PTENU,SKF_PTENV, &
 & SKF_PTLE1, SKF_PTL, &
 & SKF_PUSTRTI,SKF_PVSTRTI,SKF_PAHFSTI,SKF_PEVAPTI,SKF_PTSKTI, &
 & SKF_PDIFTS,SKF_PDIFTQ,SKF_PDIFTL,SKF_PDIFTI,SKF_PSTRTU,&
 & SKF_PSTRTV,SKF_PTOFDU,SKF_PTOFDV,SKF_PSTRDU,SKF_PSTRDV,SKF_zkh_vdf, &
 !for surftstp
 & SKF_PZO,SKF_PHO,SKF_PHO_INV,SKF_PDO,SKF_PADVT,&
 & SKF_PADVS,SKF_PTRI0,SKF_PTRI1,SKF_PSWDK_SAVE,&
 & SKF_PUO0,SKF_PVO0,SKF_PTO0,SKF_PSO0,&
 & SKF_PUSTRC,SKF_PVSTRC,SKF_PUOC,SKF_PVOC,&
 & SKF_PFTG12,SKF_PFWROD,SKF_PFWRO1,SKF_PFWG12,SKF_PFWMLT,SKF_PFWEV,SKF_PENES,&
 & SKF_PDIFM, SKF_PDIFT,SKF_PDIFS, &
 & SKF_PSNSE1,SKF_PTSNE1,SKF_PASNE1,SKF_PRSNE1,SKF_PTSAE1,SKF_PTIAE1,&
 & SKF_PWLE1,SKF_PWSAE1,SKF_PTLICEE1,SKF_PTLMNWE1,&
 & SKF_PTLWMLE1,SKF_PTLBOTE1,SKF_PTLSFE1,&
 & SKF_PHLICEE1,SKF_PHLMLE1,SKF_PUOE1,SKF_PVOE1,SKF_PTOE1,SKF_PSOE1



#ifdef DOC

!**** *SUSEKF*  - Routine to setup SEKF

!     Purpose.
!     --------
!           Setup the discriptors for initializing the SEKF

!**   Interface.
!     ----------
!        *CALL* *SUSEKF*

!        Explicit arguments : None
!        --------------------

!        Implicit arguments : Common YOMSEKF
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.     None
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Matthias Drusch  *ECMWF*

!     Modifications.
!     --------------
!        Original : 2007-07-05
!        2008-06-12 Extended Namelist
!        2009-10-11 Patricia de Rosnay - Cleaning and structuration of the routine
!        2009-10-13 Patricia de Rosnay - Offline Jacobians (add SKF_* variables)

!     ------------------------------------------------------------------
#endif

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"
#include "suanct.intfb.h"

#include "namsekf.h"

!     ------------------------------------------------------------------
!*       1.    SET DEFAULTS.
!              ------------

!*        set-up analysis constants

IF (LHOOK) CALL DR_HOOK('SUSEKF',0,ZHOOK_HANDLE)

CALL SUANCT

ALLOCATE(FKF_TENT_1(NPROMA,NFLEVG,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_TENT_2(NPROMA,NFLEVG,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_TENT_3(NPROMA,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_TENT_4(NPROMA,NGPBLKS,NSTART:NSTOP))

ALLOCATE(FKF_TENT(NPROMA,NFLEVG,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_TENQ(NPROMA,NFLEVG,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_TENU(NPROMA,NFLEVG,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_TENV(NPROMA,NFLEVG,NGPBLKS,NSTART:NSTOP))

ALLOCATE(FKF_SURF_SO(NPROMA,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_SURF_TH(NPROMA,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_SURF_CR(NPROMA,NGPBLKS,NSTART:NSTOP))
ALLOCATE(FKF_SURF_LR(NPROMA,NGPBLKS,NSTART:NSTOP))

ALLOCATE(VSM_PERT_RES(2))

ALLOCATE(VSM_INC_L1(NPROMA,NGPBLKS))
ALLOCATE(VSM_INC_L2(NPROMA,NGPBLKS))
ALLOCATE(VSM_INC_L3(NPROMA,NGPBLKS))
ALLOCATE(SEKF_Q(NGPTOT,1,1))
ALLOCATE(SEKF_DEBUG_3D(NGPTOT,20,1))
ALLOCATE(SEKF_DEBUG(20))

!     ------------------------------------------------------------------
!*       2.    READ NAMELIST.
!              -------------

CALL POSNAM(NULNAM,'NAMSEKF')
READ(NULNAM,NAMSEKF)

WRITE(NULOUT,*) '-------------------------------'
WRITE(NULOUT,*) 'SUSEKF: Namelist read:'
WRITE(NULOUT,*) '-------------------------------'
WRITE(NULOUT,*) '                               '
WRITE(NULOUT,*) 'SUSEKF: Logical Switches'
WRITE(NULOUT,*) 'LUSE_T2M:', LUSE_T2M
WRITE(NULOUT,*) 'LUSE_RH2M:', LUSE_RH2M
WRITE(NULOUT,*) 'LUSE_ASCAT:', LUSE_ASCAT
WRITE(NULOUT,*) 'LUSE_JATM:', LUSE_JATM
WRITE(NULOUT,*) '                               '
WRITE(NULOUT,*) 'SUSEKF: Variables'
WRITE(NULOUT,*) 'N_SEKF_CV:', N_SEKF_CV
WRITE(NULOUT,*) 'NSLAY:', NSLAY
WRITE(NULOUT,*) 'PATDEBUG NLEVS (from SURFACE_FIELDS_MIX):',YSP_SBD%NLEVS 
WRITE(NULOUT,*) 'PATDEBUG NFLEVG from YOMDIM:',NFLEVG
WRITE(NULOUT,*) 'PATDEBUG NTILES from YOMDPHY:',NTILES
WRITE(NULOUT,*) 'PATDEBUG NCOM (ie KLEV0) from YOMDPHY:',NCOM
WRITE(NULOUT,*) 'NOBS_SCREEN:', NOBS_SCREEN
WRITE(NULOUT,*) 'NANAL:', NANAL
WRITE(NULOUT,*) 'NSCREENTIME:', NSCREENTIME
WRITE(NULOUT,*) 'NANATIME:', NANATIME
WRITE(NULOUT,*) 'NSCREENDATE:', NSCREENDATE
WRITE(NULOUT,*) 'NANADATE:', NANADATE
WRITE(NULOUT,*) '                               '
WRITE(NULOUT,*) 'VSM_PERT_INC:', VSM_PERT_INC
WRITE(NULOUT,*) 'CHESS Type Pert.:', PERT_CHESS
WRITE(NULOUT,*) 'Model Error:', MOD_ERR
WRITE(NULOUT,*) 'Background Error:', BACK_ERR
WRITE(NULOUT,*) 'Analysis Error:', ANA_ERR
WRITE(NULOUT,*) 'T2M Error:', T2M_ERR
WRITE(NULOUT,*) 'RH2M Error:', RH2M_ERR
WRITE(NULOUT,*) 'ASCAT Error:', ASCAT_ERR


VSM_PERT_RES(1) = 1.0_JPRB
VSM_PERT_RES(2) = 10.0_JPRB
N_SEKF_PT = 0_JPIM
N_SEKF_OUPT=2_JPIM
!LUSE_JATM = .False.  ! Switch to offline jacobians
!LUSE_JATM = .True.  ! Switch to coupled jacobians
WRITE(NULOUT,*) 'LUSE_JATM in setup',LUSE_JATM

WRITE(NULOUT,*) '-------------------------------'
WRITE(NULOUT,*) 'SUSEKF: Hardcoded'
WRITE(NULOUT,*) '-------------------------------'
WRITE(NULOUT,*) '                               '
WRITE(NULOUT,*) 'SUSEKF: H-Checks'
WRITE(NULOUT,*) 'VSM_PERT_RES(1) / Temperature:', VSM_PERT_RES(1)
WRITE(NULOUT,*) 'VSM_PERT_RES(2) / Relative Hum.:', VSM_PERT_RES(2)
WRITE(NULOUT,*) 'SUSEKF: Initialized values'
WRITE(NULOUT,*) 'N_SEKF_PT:', N_SEKF_PT
WRITE(NULOUT,*) 'N_SEKF_OUPT:', N_SEKF_OUPT

!     ------------------------------------------------------------------
!*       3.    Consistency checks for SEKF set up
!              -----------------------------------
IF (N_SEKF_CV==0) THEN
  LUSEKF_REF=.FALSE.
  WRITE(NULOUT,*) '---------------------------------------------'
  WRITE(NULOUT,*) 'N_SEKF_CV equals 0, no analysis will be performed:'
  WRITE(NULOUT,*) 'LUSEKF_REF set to false ... analysis returns 0 increments'
  WRITE(NULOUT,*) '---------------------------------------------'
ENDIF
IF (NSLAY > N_SEKF_CV) THEN
  NSLAY = N_SEKF_CV
  WRITE(NULOUT,*) '---------------------------------------------'
  WRITE(NULOUT,*) 'NSLAY larger than N_SEKF_CV:'
  WRITE(NULOUT,*) 'NSLAY set equal N_SEKF_CV'
  WRITE(NULOUT,*) '---------------------------------------------'
ENDIF

!     ------------------------------------------------------------------
!*       4.    Consistency checks for observations
!              -----------------------------------
NOBS_SEKF_MAX = 0

IF (LUSE_T2M .AND. LUSE_RH2M) THEN
  NOBS_SEKF_MAX = NOBS_SCREEN * 2
ELSEIF ((LUSE_T2M .AND. .NOT.LUSE_RH2M) .OR. &
       & (.NOT.LUSE_T2M .AND. LUSE_RH2M)) THEN
  NOBS_SEKF_MAX = NOBS_SCREEN
ELSEIF (.NOT.LUSE_T2M .AND. .NOT.LUSE_RH2M) THEN
  NOBS_SEKF_MAX = 0
ENDIF

IF (LUSE_ASCAT) THEN
  ! currently we assume that we have a maximum of 6 observations within an analysis cycle
  NOBS_SEKF_MAX = NOBS_SEKF_MAX + 6  
ENDIF

WRITE(NULOUT,*) 'SUSEKF: Dimensions'
WRITE(NULOUT,*) 'NOBS_SEKF_MAX:', NOBS_SEKF_MAX

IF (NOBS_SEKF_MAX==0) THEN

  LUSEKF_REF=.FALSE.
  WRITE(NULOUT,*) '---------------------------------------------'
  WRITE(NULOUT,*) 'SUSEKF: Number of observations for SEKF is 0!'
  WRITE(NULOUT,*) 'LUSEKF_REF set to false ... analysis returns 0 increments'
  WRITE(NULOUT,*) '---------------------------------------------'

ELSEIF (NOBS_SEKF_MAX>0) THEN

  ALLOCATE(NOBS_TSTEP(NOBS_SCREEN))
  ALLOCATE(NANA_TSTEP(NANAL))

  ALLOCATE(SEKF_A(NGPTOT,N_SEKF_CV*N_SEKF_CV,1))
  ALLOCATE(SEKF_B(NGPTOT,N_SEKF_CV*N_SEKF_CV,1))
  ALLOCATE(SEKF_G(NGPTOT,N_SEKF_CV*NOBS_SEKF_MAX,1))
  ALLOCATE(SEKF_J(NGPTOT,NOBS_SEKF_MAX*N_SEKF_CV,1))
  ALLOCATE(SEKF_F(NGPTOT,NSLAY*NANAL,1))

  NREF_BLOCK = 45
  NREF_GP = 16

ENDIF

!     ------------------------------------------------------------------
!*       5.    Variable allocation once all dimensions are known
!              ---------------------------------------------------------

ALLOCATE(VSM_ANAL_INC(NSLAY))
ALLOCATE(SEKF_INC(NGPTOT,NSLAY,1))

ALLOCATE(VOL_SM_FG(NPROMA,NSLAY,NGPBLKS,NSTART:NSTOP,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(T2M_FG(NPROMA,NGPBLKS,NSTART:NSTOP,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(TD2M_FG(NPROMA,NGPBLKS,NSTART:NSTOP,N_SEKF_CV+N_SEKF_OUPT))

IF(.NOT.LUSE_JATM) THEN
! Offline Jacobians variables (temporarly saved from callpar)
! VDFOUTER BACKUP
ALLOCATE(ISKF_IHPBLN(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(ISKF_IPBLTYPE(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(LSKF_LLNODECP(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))

ALLOCATE(SKF_ZTSA (NPROMA,NGPBLKS,YSP_SBD%NLEVS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZWSA(NPROMA,NGPBLKS,YSP_SBD%NLEVS,N_SEKF_CV+N_SEKF_OUPT))

ALLOCATE(SKF_ZAZ0M (NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZAZ0H(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVDIS(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVDISG(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZDISSGW(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFTLHEV(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFTLHSB(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFWSB(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZBIR(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZVAR(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUCFL(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVCFL(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTCFL(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDCFL(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PQCFL(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTENT_CUM(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTENQ_CUM(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZZINV(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PBLH(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PU10N(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PV10N(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUST(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZFRSOTI(NPROMA,NGPBLKS,NTILES,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZEVAPSNW(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZGUST(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZWUAVG(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTENT(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTENQ(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZTENA(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZTENI(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZTENL(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTENU(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTENV(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTLE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTL(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUSTRTI(NPROMA,NGPBLKS,NTILES,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVSTRTI(NPROMA,NGPBLKS,NTILES,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PAHFSTI(NPROMA,NGPBLKS,NTILES,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PEVAPTI(NPROMA,NGPBLKS,NTILES,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTSKTI(NPROMA,NGPBLKS,NTILES,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFTS(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFTQ(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFTL(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFTI(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSTRTU(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSTRTV(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTOFDU(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTOFDV(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSTRDU(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSTRDV(NPROMA,NGPBLKS,0:NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_ZKH_VDF(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
!ALLOCATE(SKF_PDHTLS(NPROMA,NGPBLKS,NTILES,NDHVTLS+NDHFTLS,N_SEKF_CV+N_SEKF_OUPT))
!ALLOCATE(SKF_PDHTSS(NPROMA,NGPBLKS,NCSNEC,NDHVTSS+NDHFTSS,N_SEKF_CV+N_SEKF_OUPT))
!ALLOCATE(SKF_PDHTTS(NPROMA,NGPBLKS,YSP_SBD%NLEVS,NDHVTTS+NDHFTTS,N_SEKF_CV+N_SEKF_OUPT))
!ALLOCATE(SKF_PDHTIS(NPROMA,NGPBLKS,KLEVI,NDHVTIS+NDHFTIS,N_SEKF_CV+N_SEKF_OUPT))

!SURFTSTP BACKUP
ALLOCATE(SKF_PZO(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PHO(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PHO_INV(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDO(NPROMA,NGPBLKS,0:NCOM,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PADVT(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PADVS(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTRI0(NPROMA,NGPBLKS,0:NCOM,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTRI1(NPROMA,NGPBLKS,0:NCOM,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSWDK_SAVE(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUO0(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVO0(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTO0(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSO0(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUOC(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVOC(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUSTRC(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVSTRC(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))

ALLOCATE(SKF_PFTG12(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFWROD(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFWRO1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFWG12(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFWMLT(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PFWEV(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PENES(NPROMA,NGPBLKS,NFLEVG,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFM(NPROMA,NGPBLKS,0:NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFT(NPROMA,NGPBLKS,0:NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PDIFS(NPROMA,NGPBLKS,0:NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSNSE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTSNE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PASNE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PRSNE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTSAE1(NPROMA,NGPBLKS,YSP_SBD%NLEVS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTIAE1(NPROMA,NGPBLKS,YSP_SBD%NLEVS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PWLE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PWSAE1(NPROMA,NGPBLKS,YSP_SBD%NLEVS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTLICEE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTLMNWE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTLWMLE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTLBOTE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTLSFE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PHLICEE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PHLMLE1(NPROMA,NGPBLKS,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PUOE1(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PVOE1(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PTOE1(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ALLOCATE(SKF_PSOE1(NPROMA,NGPBLKS,NCOM+1,N_SEKF_CV+N_SEKF_OUPT))
ENDIF


!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUSEKF',1,ZHOOK_HANDLE)
END SUBROUTINE SUSEKF
