SUBROUTINE CMOCTMAP(CDMAP  ,KOUT   ,KOBTYP ,KCDTYP ,KOTSQNO,KCDSQNO )

!**** SUBROUTINE CMOCTMAP - MAP CMA OBSERVATION AND CODE TYPES

!        D. VASILJEVIC    ECMWF    01/09/1994

!     PURPOSE
!     -------

!        MAP CMA OBSERVATION AND CODE TYPE INTO SEQUENCE NOS.

!**   INTERFACE
!     ---------

!        CALL CMOCTMAP(CDMAP  ,KOUT   ,
!                      KOBTYP ,KCDTYP ,
!                      KOTSQNO,KCDSQNO )
!     WHERE:
!           CDMAP   = MAP EITHER CMA/BUFR INTO SEQUENCE NO. OR THE
!                     OTHER WAY ROUND
!           KOUT    = OUTPUT UNIT
!           KOBTYP  = OBS. TYPE
!           KCDTYP  = OBS. CODE TYPE
!           KOTSQNO = OBS. TYPE SEQUENCE NO.
!           KCDSQNO = OBS. CODE TYPE SEQUENCE NO.

!        CMOCTMAP IS CALLED BY

!**   METHOD
!     ------

!        FOR EACH OBS. AND CODE TYPE WITHIN AN OBS. TYPE A SIMPLE 
!     SEQUENCE NO. IS ASSIGNED.

!**   EXTERNALS
!     ---------

!        ENDRUN - ABORT

!**   MODIFICATIONS
!     -------------
!        Niels Bormann  23/12/2004  RARAD becomes LIMB; new codetypes for LIMB
!        Anne Fouilloux 31/03/2010 Add a new codetype for MERIS data (NTCWC)
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCOCTP , ONLY : NSYNOP   ,NSYNOPSQ ,NSRSCD   ,NSRSCDSQ ,&
 & NATSCD   ,NATSCDSQ ,NSHSCD   ,NSHSCDSQ ,NABSCD   ,&
 & NABSCDSQ ,NSHRED   ,NSHREDSQ ,NATSHS   ,NATSHSSQ ,&
 & NAIREP   ,NAIREPSQ ,NAIRCD   ,NAIRCDSQ ,NCODAR   ,&
 & NCODARSQ ,NCOLBA   ,NCOLBASQ ,NAMDAR   ,NAMDARSQ ,&
 & NACARS   ,NACARSSQ ,NSIMAI   ,NSIMAISQ ,NSATOB   ,&
 & NSATOBSQ ,NSTBCD   ,NSTBCDSQ ,NSTB86   ,NSTB86SQ ,&
 & NSTB87   ,NSTB87SQ ,&
 & NSST     ,NSSTSQ   ,NDRIBU   ,NDRIBUSQ ,NDRBCD   ,&
 & NDRBCDSQ ,NBATHY   ,NBATHYSQ ,NTESAC   ,NTESACSQ ,&
 & NDERS1   ,NDERS1SQ ,NTEMP    ,NTEMPSQ  ,NLDTCD   ,&
 & NLDTCDSQ ,NSHTCD   ,NSHTCDSQ ,NTDROP   ,NTDROPSQ ,&
 & NROCOB   ,NROCOBSQ ,NROCSH   ,NROCSHSQ ,NMBTMP   ,&
 & NMBTMPSQ ,NSIMTE   ,NSIMTESQ ,NPILOT   ,NPILOTSQ ,&
 & NLDPCD   ,NLDPCDSQ ,NSHPCD   ,NSHPCDSQ ,NWPPCD   ,&
 & NWPPCDSQ ,NDWLPCD  ,NDWLPCDSQ,&
 & NSATEM   ,NSATEMSQ ,NSTMCD   ,NSTMCDSQ ,&
 & NSTOVS   ,NSTOVSSQ ,NSTRO3   ,NSTRO3SQ ,&
 & NSTDWL,&
 & NSTDWLSQ ,NSTTOV   ,NSTTOVSQ ,NGTSTB   ,NGTSTBSQ ,&
 & NGTST1   ,NGTST1SQ ,NGTST2   ,NGTST2SQ ,NGTHRB   ,&
 & NGTHRBSQ ,NGTHR1   ,NGTHR1SQ ,NGTHR2   ,NGTHR2SQ ,&
 & NSSMI    ,NSSMISQ  ,&
 & NPAOB    ,NPAOBSQ  ,NPABCD   ,NPABCDSQ ,NSCATT   ,&
 & NSCATTSQ ,NSCAT1   ,NSCAT1SQ ,NSCAT2   ,NSCAT2SQ ,&
 & NSCAT3   ,NSCAT3SQ ,NLIMB    ,NLIMBSQ  ,&
 & NSYRCD   ,NSYRCDSQ ,NSCAT4   ,NSCAT4SQ ,&
 & NSCAT5   ,NSCAT5SQ ,NSCAT6   ,NSCAT6SQ ,&
 & NEWPPCD  ,NEWPPCDSQ,NJWPPCD  ,NJWPPCDSQ          ,&
 & NRADOH   ,NRADOHSQ ,NTYPHO   ,NTYPHOSQ           ,&
 & NMETAR   ,NMETARSQ ,NPGPS    ,NPGPSSQ            ,&
 & NGPSRO   ,NGPSROSQ ,NLRAD    ,NLRADSQ            ,&
 & NRADAR   ,NRADARSQ ,NRADA1   ,NRADA1SQ           ,&
 & NSMOS    ,NSMOSSQ, NTCWC, NTCWCSQ

IMPLICIT NONE

CHARACTER(LEN=*)  ,INTENT(IN)    :: CDMAP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOUT 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOBTYP 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KCDTYP 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOTSQNO 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KCDSQNO 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "abortdb.h"
END INTERFACE

!*
!-----------------------------------------------------------------------

!        1.        MAP CMA OBS. AND CODE TYPE INTO SEQUENCE NO.
!                  --------------------------------------------

IF (LHOOK) CALL DR_HOOK('CMOCTMAP',0,ZHOOK_HANDLE)
IF(CDMAP == 'CODETOSQ' )       THEN

!*          1.1    SYNOP

  IF(KOBTYP == NSYNOP)           THEN
    KOTSQNO = NSYNOPSQ

!*          1.1.1  SYNOP LAND REPORT

    IF(KCDTYP == NSRSCD)           THEN
      KCDSQNO = NSRSCDSQ

!*          1.1.2  SYNOP LAND AUTOMATIC REPORT

    ELSEIF(KCDTYP == NATSCD)      THEN
      KCDSQNO = NATSCDSQ

!*          1.1.3  SYNOP-SHIP REPORT

    ELSEIF(KCDTYP == NSHSCD)      THEN
      KCDSQNO = NSHSCDSQ

!*          1.1.4  SYNOP-SHIP ABBREVIATED REPORT

    ELSEIF(KCDTYP == NABSCD)      THEN
      KCDSQNO = NABSCDSQ

!*          1.1.5  SYNOP SHRED REPORT

    ELSEIF(KCDTYP == NSHRED)      THEN
      KCDSQNO = NSHREDSQ

!*          1.1.6  SYNOP AUTOMATIC SHIP REPORT

    ELSEIF(KCDTYP == NATSHS)      THEN
      KCDSQNO = NATSHSSQ

!*          1.1.7  FRENCH SYNOP AUTOMATIC LAND REPORT

    ELSEIF(KCDTYP == NSYRCD)      THEN
      KCDSQNO = NSYRCDSQ

!*          1.1.8  FRENCH radome

    ELSEIF(KCDTYP == NRADOH)      THEN
      KCDSQNO = NRADOHSQ

!*          1.1.9  METAR

    ELSEIF(KCDTYP == NMETAR)      THEN
      KCDSQNO = NMETARSQ

!*          1.1.10 Ground-based GPS

    ELSEIF(KCDTYP == NPGPS)      THEN
      KCDSQNO = NPGPSSQ

!*          1.1.11  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.2    AIREP

  ELSEIF(KOBTYP == NAIREP)      THEN
    KOTSQNO = NAIREPSQ

!*          1.2.1  AIRCRAFT REPORT

    IF(KCDTYP == NAIRCD)           THEN
      KCDSQNO = NAIRCDSQ

!*          1.2.2  CODAR REPORT

    ELSEIF(KCDTYP == NCODAR)       THEN
      KCDSQNO = NCODARSQ

!*          1.2.3  COLBA REPORT

    ELSEIF(KCDTYP == NCOLBA)       THEN
      KCDSQNO = NCOLBASQ

!*          1.2.4  AMDAR AIRCRAFT REPORT

    ELSEIF(KCDTYP == NAMDAR)       THEN
      KCDSQNO = NAMDARSQ

!*          1.2.5  ACARS AIRCRAFT REPORT

    ELSEIF(KCDTYP == NACARS)       THEN
      KCDSQNO = NACARSSQ

!*          1.2.6  SIMULATED AIREP REPORT

    ELSEIF(KCDTYP == NSIMAI)       THEN
      KCDSQNO = NSIMAISQ

!*          1.2.9  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.3    SATOB

  ELSEIF(KOBTYP == NSATOB)       THEN
    KOTSQNO = NSATOBSQ

!*          1.3.1  SATOB REPORT

    IF(KCDTYP == NSTBCD)            THEN
      KCDSQNO = NSTBCDSQ

!*          1.3.2  HIGH RES. VIS WIND SATOB REPORT

    ELSEIF(KCDTYP == NSTB86)            THEN
      KCDSQNO = NSTB86SQ

!*          1.3.3  SST REPORT

    ELSEIF(KCDTYP == NSST  )       THEN
      KCDSQNO = NSSTSQ

!*          1.3.4  AMV SATOB REPORT

    ELSEIF(KCDTYP == NSTB87)            THEN
      KCDSQNO = NSTB87SQ

!*          1.3.9  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.4    DRIBU

  ELSEIF(KOBTYP == NDRIBU)       THEN
    KOTSQNO = NDRIBUSQ

!*          1.4.1  DRIBU REPORT

    IF(KCDTYP == NDRBCD)            THEN
      KCDSQNO = NDRBCDSQ

!*          1.4.2  BATHY REPORT

    ELSEIF(KCDTYP == NBATHY)       THEN
      KCDSQNO = NBATHYSQ

!*          1.4.3  TESAC REPORT

    ELSEIF(KCDTYP == NTESAC)       THEN
      KCDSQNO = NTESACSQ

!*          1.4.4  ERS1 AS DRIBU

    ELSEIF(KCDTYP == NDERS1)       THEN
      KCDSQNO = NDERS1SQ

!*          1.4.9  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE:'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.5    TEMP

  ELSEIF(KOBTYP == NTEMP )       THEN
    KOTSQNO = NTEMPSQ

!*          1.5.1  LAND TEMP REPORT

    IF(KCDTYP == NLDTCD)            THEN
      KCDSQNO = NLDTCDSQ

!*          1.5.2  TEMP SHIP REPORT

    ELSEIF(KCDTYP == NSHTCD)       THEN
      KCDSQNO = NSHTCDSQ

!*          1.5.3  TEMP-DROP REPORT

    ELSEIF(KCDTYP == NTDROP)       THEN
      KCDSQNO = NTDROPSQ

!*          1.5.4  ROCOB REPORT

    ELSEIF(KCDTYP == NROCOB)       THEN
      KCDSQNO = NROCOBSQ

!*          1.5.5  ROCOB SHIP REPORT

    ELSEIF(KCDTYP == NROCSH)       THEN
      KCDSQNO = NROCSHSQ

!*          1.5.6  MOBILE TEMP REPORT

    ELSEIF(KCDTYP == NMBTMP)       THEN
      KCDSQNO = NMBTMPSQ

!*          1.5.7  SIMULATED TEMP STATION CODE TYPE

    ELSEIF(KCDTYP == NSIMTE)       THEN
      KCDSQNO = NSIMTESQ

!*          1.5.9  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE:'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.6    PILOT

  ELSEIF(KOBTYP == NPILOT)       THEN
    KOTSQNO = NPILOTSQ

!*          1.6.1  LAND PILOT REPORT

    IF(KCDTYP == NLDPCD)            THEN
      KCDSQNO = NLDPCDSQ

!*          1.6.2  SHIP PILOT REPORT

    ELSEIF(KCDTYP == NSHPCD)       THEN
      KCDSQNO = NSHPCDSQ

!*          1.6.3  AMERICAN WIND PROFILERS

    ELSEIF(KCDTYP == NWPPCD)       THEN
      KCDSQNO = NWPPCDSQ

!*          1.6.4  EUROPEAN WIND PROFILERS

    ELSEIF(KCDTYP == NEWPPCD)       THEN
      KCDSQNO = NEWPPCDSQ

!*          1.6.5  JAPANESE WIND PROFILERS

    ELSEIF(KCDTYP == NJWPPCD)       THEN
      KCDSQNO = NJWPPCDSQ

!*          1.6.6a  MOBILE WIND PILOT - OBSOLETE

!    ELSEIF(KCDTYP == NMBPLT)       THEN
!      KCDSQNO = NMBPLTSQ

!*          1.6.6b  DOPPLER WIND LIDAR

    ELSEIF(KCDTYP == NDWLPCD)       THEN
      KCDSQNO = NDWLPCDSQ

!*          1.6.9  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.7    SATEM

  ELSEIF(KOBTYP == NSATEM)       THEN
    KOTSQNO = NSATEMSQ

!*          1.7.1  SATEM REPORT

    IF(KCDTYP == NSTMCD)            THEN
      KCDSQNO = NSTMCDSQ

!*          1.7.2  HIGH RESOLUTION SATTELITE REPORT

    ELSEIF(KCDTYP == NSTOVS)       THEN
      KCDSQNO = NSTOVSSQ

!*          1.7.3  HIGH RESOLUTION SIMULATED DWL SATELLITE REPORT

    ELSEIF(KCDTYP == NSTDWL)       THEN
      KCDSQNO = NSTDWLSQ

!*          1.7.4  HIGH RESOLUTION SIMULATED TOVS SATELLITE REPORT

    ELSEIF(KCDTYP == NSTTOV)       THEN
      KCDSQNO = NSTTOVSQ

!*          1.7.5  GTS BUFR SATEM REPORT
!                  (250 KM TEMP AND HUM PROFILES)

    ELSEIF(KCDTYP == NGTSTB)       THEN
      KCDSQNO = NGTSTBSQ

!*          1.7.6  GTS BUFR SATEM CLEAR RADIANCE REPORT

    ELSEIF(KCDTYP == NGTST1)       THEN
      KCDSQNO = NGTST1SQ

!*          1.7.7  GTS BUFR SATEM RETR PROFILES AND
!                  CLEAR RADIANCES REPORT

    ELSEIF(KCDTYP == NGTST2)       THEN
      KCDSQNO = NGTST2SQ

!*          1.7.8  HIGH RES 80 KM BUFR NON GTS SATEM RETR PROFILES

    ELSEIF(KCDTYP == NGTHRB)       THEN
      KCDSQNO = NGTHRBSQ

!*          1.7.9  HIGH RES 80 KM BUFR NON GTS SATEM CLEAR
!                  RADIANCE REPORT

    ELSEIF(KCDTYP == NGTHR1)       THEN
      KCDSQNO = NGTHR1SQ

!*          1.7.10 HIGH RES 80 KM BUFR NON GTS SATEM PROFILES AND
!                  CLEAR RADIANCE REPORT

    ELSEIF(KCDTYP == NGTHR2)       THEN
      KCDSQNO = NGTHR2SQ

!*          1.7.11 SSMI/TRMM REPORT

    ELSEIF(KCDTYP == NSSMI )       THEN
      KCDSQNO = NSSMISQ

!*          1.7.12 RETRIEVED ATMOSPHERIC PARAMETER REPORT
!*                 INCLUDES: Ozone, GRG variables,
!*                 AEROSOL observations (AOD, RAO, RFA, ODA) and
!*                 CLOUD OPTICAL DEPTH 

    ELSEIF(KCDTYP == NSTRO3 )       THEN
      KCDSQNO = NSTRO3SQ

!*          1.7.13 SMOS

    ELSEIF(KCDTYP == NSMOS )       THEN
      KCDSQNO = NSMOSSQ

!*          1.7.14 MERIS

    ELSEIF(KCDTYP == NTCWC )       THEN
      KCDSQNO = NTCWCSQ

!*          1.7.15 UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.8    PAOB

  ELSEIF(KOBTYP == NPAOB )       THEN
    KOTSQNO = NPAOBSQ

!*          1.8.1  PAOB REPORT

    IF(KCDTYP == NPABCD)            THEN
      KCDSQNO = NPABCDSQ

!*          1.8.2  donnees cyclone

    ELSEIF(KCDTYP == NTYPHO)            THEN
      KCDSQNO = NTYPHOSQ

!*          1.8.9  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.9    SCATTEROMETER

  ELSEIF(KOBTYP == NSCATT)       THEN
    KOTSQNO = NSCATTSQ

!*          1.9.1  SCATTEROMETER CODE 1

    IF(KCDTYP == NSCAT1)            THEN
      KCDSQNO = NSCAT1SQ

!*          1.9.2  SCATTEROMETER CODE 2

    ELSEIF(KCDTYP == NSCAT2)            THEN
      KCDSQNO = NSCAT2SQ

!*          1.9.3  SCATTEROMETER CODE 3

    ELSEIF(KCDTYP == NSCAT3)            THEN
      KCDSQNO = NSCAT3SQ

!*          1.9.4  SCATTEROMETER CODE 4

    ELSEIF(KCDTYP == NSCAT4)            THEN
      KCDSQNO = NSCAT4SQ

!*          1.9.5  SCATTEROMETER CODE 5

    ELSEIF(KCDTYP == NSCAT5)            THEN
      KCDSQNO = NSCAT5SQ

!*          1.9.6  SCATTEROMETER CODE 6

    ELSEIF(KCDTYP == NSCAT6)            THEN
      KCDSQNO = NSCAT6SQ

!*          1.9.7  UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.10   LIMB OBSERVATIONS

  ELSEIF(KOBTYP == NLIMB)       THEN
    KOTSQNO = NLIMBSQ

!*          1.10.1 GPS RO

    IF(KCDTYP == NGPSRO)            THEN
      KCDSQNO = NGPSROSQ

!*          1.10.2 Limb radiances

    ELSEIF(KCDTYP == NLRAD)            THEN
      KCDSQNO = NLRADSQ

!*          1.10.9 UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.11 

    ELSEIF(KOBTYP == 11) THEN
      KOTSQNO  = 11

!*          1.12 

    ELSEIF(KOBTYP == 12) THEN
      KOTSQNO = 12

!*          1.13 RADAR

    ELSEIF(KOBTYP == NRADAR) THEN 
      KOTSQNO = NRADARSQ

!*          1.13.1 BUFR RADAR REFLECTIVITY 1

    IF(KCDTYP == NRADA1)  THEN 
      KCDSQNO = NRADA1SQ

!*          1.13.9 UNKNOWN CMA CODE TYPE

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN CMA CODE TYPE'')')
      WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
      WRITE(KOUT,'('' CMA CODE TYPE:'',I10)') KCDTYP
      CALL ABORTDB('CMOCTMAP','UNKNOWN CMA CODE TYPE',KCDTYP)
    ENDIF

!*          1.14 
 
    ELSEIF(KOBTYP == 14) THEN
      KOTSQNO = 14

!*         1.15

   ELSEIF(KOBTYP == 15) THEN 
      KOTSQNO = 15

!*          1.16   UNKNOWN CMA TYPE

  ELSE
    WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
    WRITE(KOUT,'('' UNKNOWN CMA OBS. TYPE'')')
    WRITE(KOUT,'('' CMA OBS. TYPE:'',I10)') KOBTYP
    CALL ABORTDB('CMOCTMAP','UNKNOWN CMA OBS. TYPE',KOBTYP)
  ENDIF
!*
!-----------------------------------------------------------------------

!        3.        MAP CMA SEQUENCE NO. INTO OBS. AND CODE TYPE
!                  --------------------------------------------

ELSEIF(CDMAP == 'SQTOCODE' )     THEN

!*          3.1    SYNOP

  IF(KOTSQNO == NSYNOPSQ)           THEN
    KOBTYP  = NSYNOP

!*          3.1.1  SYNOP LAND REPORT

    IF(KCDSQNO == NSRSCDSQ)           THEN
      KCDTYP  = NSRSCD

!*          3.1.2  SYNOP LAND AUTOMATIC REPORT

    ELSEIF(KCDSQNO == NATSCDSQ)      THEN
      KCDTYP  = NATSCD

!*          3.1.3  SYNOP-SHIP REPORT

    ELSEIF(KCDSQNO == NSHSCDSQ)      THEN
      KCDTYP  = NSHSCD

!*          3.1.4  SYNOP-SHIP ABBREVIATED REPORT

    ELSEIF(KCDSQNO == NABSCDSQ)      THEN
      KCDTYP  = NABSCD

!*          3.1.5  SYNOP SHRED REPORT

    ELSEIF(KCDSQNO == NSHREDSQ)      THEN
      KCDTYP  = NSHRED

!*          3.1.6  SYNOP AUTOMATIC SHIP REPORT

    ELSEIF(KCDSQNO == NATSHSSQ)      THEN
      KCDTYP  = NATSHS

!*          3.1.7  FRENCH SYNOP AUTOMATIC LAND REPORT

    ELSEIF(KCDSQNO == NSYRCDSQ)      THEN
      KCDTYP  = NSYRCD

!*          3.1.8  FRENCH radome

    ELSEIF(KCDSQNO == NRADOHSQ)      THEN
      KCDTYP  = NRADOH

!*          3.1.9  METAR

    ELSEIF(KCDSQNO == NMETARSQ)      THEN
      KCDTYP  = NMETAR

!*          3.1.10 Ground-based GPS

    ELSEIF(KCDSQNO == NPGPSSQ)      THEN
      KCDTYP  = NPGPS

!*          3.1.11 UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN SYNOP CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN SYNOP CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.2    AIREP

  ELSEIF(KOTSQNO == NAIREPSQ)      THEN
    KOBTYP  = NAIREP

!*          3.2.1  AIRCRAFT REPORT

    IF(KCDSQNO == NAIRCDSQ)           THEN
      KCDTYP  = NAIRCD

!*          3.2.2  CODAR REPORT

    ELSEIF(KCDSQNO == NCODARSQ)       THEN
      KCDTYP  = NCODAR

!*          3.2.3  COLBA REPORT

    ELSEIF(KCDSQNO == NCOLBASQ)       THEN
      KCDTYP  = NCOLBA

!*          3.2.4  AMDAR AIRCRAFT REPORT

    ELSEIF(KCDSQNO == NAMDARSQ)       THEN
      KCDTYP  = NAMDAR

!*          3.2.5  ACARS AIRCRAFT REPORT

    ELSEIF(KCDSQNO == NACARSSQ)       THEN
      KCDTYP  = NACARS

!*          3.2.6  SIMULATED AIREP REPORT

    ELSEIF(KCDSQNO == NSIMAISQ)       THEN
      KCDTYP  = NSIMAI

!*          3.2.9  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN AIREP CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN AIREP CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.3    SATOB

  ELSEIF(KOTSQNO == NSATOBSQ)       THEN
    KOBTYP  = NSATOB

!*          3.3.1  SATOB REPORT

    IF(KCDSQNO == NSTBCDSQ)            THEN
      KCDTYP  = NSTBCD

!*          3.3.2  HIGH RES. VIS WIND SATOB REPORT

    ELSEIF(KCDSQNO == NSTB86SQ)            THEN
      KCDTYP  = NSTB86

!*          3.3.3  SST REPORT

    ELSEIF(KCDSQNO == NSSTSQ  )       THEN
      KCDTYP  = NSST

!*          3.3.4  AMV SATOB REPORT

    ELSEIF(KCDSQNO == NSTB87SQ)            THEN
      KCDTYP  = NSTB87

!*          3.3.9  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN SATOB CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN SATOB CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.4    DRIBU

  ELSEIF(KOTSQNO == NDRIBUSQ)       THEN
    KOBTYP  = NDRIBU

!*          3.4.1  DRIBU REPORT

    IF(KCDSQNO == NDRBCDSQ)            THEN
      KCDTYP  = NDRBCD

!*          3.4.2  BATHY REPORT

    ELSEIF(KCDSQNO == NBATHYSQ)       THEN
      KCDTYP  = NBATHY

!*          3.4.3  TESAC REPORT

    ELSEIF(KCDSQNO == NTESACSQ)       THEN
      KCDTYP  = NTESAC

!*          3.4.4  ERS1 AS DRIBU

    ELSEIF(KCDSQNO == NDERS1SQ)       THEN
      KCDTYP  = NDERS1

!*          3.4.9  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN DRIBU CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN DRIBU CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.5    TEMP

  ELSEIF(KOTSQNO == NTEMPSQ )       THEN
    KOBTYP  = NTEMP

!*          3.5.1  LAND TEMP REPORT

    IF(KCDSQNO == NLDTCDSQ)            THEN
      KCDTYP  = NLDTCD

!*          3.5.2  TEMP SHIP REPORT

    ELSEIF(KCDSQNO == NSHTCDSQ)       THEN
      KCDTYP  = NSHTCD

!*          3.5.3  TEMP-DROP REPORT

    ELSEIF(KCDSQNO == NTDROPSQ)       THEN
      KCDTYP  = NTDROP

!*          3.5.4  ROCOB REPORT

    ELSEIF(KCDSQNO == NROCOBSQ)       THEN
      KCDTYP  = NROCOB

!*          3.5.5  ROCOB SHIP REPORT

    ELSEIF(KCDSQNO == NROCSHSQ)       THEN
      KCDTYP  = NROCSH

!*          3.5.6  MOBILE TEMP REPORT

    ELSEIF(KCDSQNO == NMBTMPSQ)       THEN
      KCDTYP  = NMBTMP

!*          3.5.7  SIMULATED TEMP STATION CODE TYPE

    ELSEIF(KCDSQNO == NSIMTESQ)       THEN
      KCDTYP  = NSIMTE

!*          3.5.9  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN TEMP CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN TEMP CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.6    PILOT

  ELSEIF(KOTSQNO == NPILOTSQ)       THEN
    KOBTYP  = NPILOT

!*          3.6.1  LAND PILOT REPORT

    IF(KCDSQNO == NLDPCDSQ)            THEN
      KCDTYP  = NLDPCD

!*          3.6.2  SHIP PILOT REPORT

    ELSEIF(KCDSQNO == NSHPCDSQ)       THEN
      KCDTYP  = NSHPCD

!*          3.6.3  AMERICAN WIND PROFILERS

    ELSEIF(KCDSQNO == NWPPCDSQ)       THEN
      KCDTYP  = NWPPCD

!*          3.6.4  EUROPEAN WIND PROFILERS

    ELSEIF(KCDSQNO == NEWPPCDSQ)       THEN
      KCDTYP  = NEWPPCD

!*          3.6.5  JAPANESE WIND PROFILERS

    ELSEIF(KCDSQNO == NJWPPCDSQ)       THEN
      KCDTYP  = NJWPPCD

!*          3.6.6a  MOBILE WIND PILOT - OBSOLETE

!*          3.6.6b  DOPPLER WIND LIDAR

    ELSEIF(KCDSQNO == NDWLPCDSQ)       THEN
      KCDTYP  = NDWLPCD

!*          3.6.9  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN PILOT CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN PILOT CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.7    SATEM

  ELSEIF(KOTSQNO == NSATEMSQ)       THEN
    KOBTYP  = NSATEM

!*          3.7.1  SATEM REPORT

    IF(KCDSQNO == NSTMCDSQ)            THEN
      KCDTYP  = NSTMCD

!*          3.7.2  HIGH RESOLUTION SATTELITE REPORT

    ELSEIF(KCDSQNO == NSTOVSSQ)       THEN
      KCDTYP  = NSTOVS

!*          3.7.3  HIGH RESOLUTION SIMULATED DWL SATELLITE REPORT

    ELSEIF(KCDSQNO == NSTDWLSQ)       THEN
      KCDTYP  = NSTDWL

!*          3.7.4  HIGH RESOLUTION SIMULATED TOVS SATELLITE REPORT

    ELSEIF(KCDSQNO == NSTTOVSQ)       THEN
      KCDTYP  = NSTTOV

!*          3.7.5  GTS BUFR SATEM REPORT
!                  (250 KM TEMP AND HUM PROFILES)

    ELSEIF(KCDSQNO == NGTSTBSQ)       THEN
      KCDTYP  = NGTSTB

!*          3.7.6  GTS BUFR SATEM CLEAR RADIANCE REPORT

    ELSEIF(KCDSQNO == NGTST1SQ)       THEN
      KCDTYP  = NGTST1

!*          3.7.7  GTS BUFR SATEM RETR PROFILES AND
!                  CLEAR RADIANCES REPORT

    ELSEIF(KCDSQNO == NGTST2SQ)       THEN
      KCDTYP  = NGTST2

!*          3.7.8  HIGH RES 80 KM BUFR NON GTS SATEM RETR PROFILES

    ELSEIF(KCDSQNO == NGTHRBSQ)       THEN
      KCDTYP  = NGTHRB

!*          3.7.9  HIGH RES 80 KM BUFR NON GTS SATEM CLEAR
!                  RADIANCE REPORT

    ELSEIF(KCDSQNO == NGTHR1SQ)       THEN
      KCDTYP  = NGTHR1

!*          3.7.10 HIGH RES 80 KM BUFR NON GTS SATEM PROFILES AND
!                  CLEAR RADIANCE REPORT

    ELSEIF(KCDSQNO == NGTHR2SQ)       THEN
      KCDTYP  = NGTHR2

!*          3.7.11 SSMI/TRMM REPORT

    ELSEIF(KCDSQNO == NSSMISQ )       THEN
      KCDTYP  = NSSMI

!*          3.7.12 RETRIEVED OZONE LAYERS REPORT
!BEN- NOW INCLUDING ALSO RETRIEVED AEROSOL OPTICAL DEPTH!!!!!

    ELSEIF(KCDSQNO == NSTRO3SQ )       THEN
      KCDTYP  = NSTRO3

!*          3.7.13 SMOS REPORT

    ELSEIF(KCDSQNO == NSMOSSQ )       THEN
      KCDTYP  = NSMOS

!*          3.7.14 UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN SATEM CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN SATEM CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.8    PAOB

  ELSEIF(KOTSQNO == NPAOBSQ )       THEN
    KOBTYP  = NPAOB

!*          3.8.1  PAOB REPORT

    IF(KCDSQNO == NPABCDSQ)            THEN
      KCDTYP  = NPABCD

!*          3.8.2  donnees cyclone

    ELSEIF(KCDSQNO == NTYPHOSQ)            THEN
      KCDTYP  = NTYPHO

!*          3.8.9  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN PAOB CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN PAOB CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.9    SCATTEROMETER

  ELSEIF(KOTSQNO == NSCATTSQ)       THEN
    KOBTYP  = NSCATT

!*          3.9.1  SCATT. CODE TYPE 1

    IF(KCDSQNO == NSCAT1SQ)            THEN
      KCDTYP  = NSCAT1

!*          3.9.2  SCATT. CODE TYPE 2

    ELSEIF(KCDSQNO == NSCAT2SQ)            THEN
      KCDTYP  = NSCAT2

!*          3.9.3  SCATT. CODE TYPE 3

    ELSEIF(KCDSQNO == NSCAT3SQ)            THEN
      KCDTYP  = NSCAT3

!*          3.9.4  SCATT. CODE TYPE 4

    ELSEIF(KCDSQNO == NSCAT4SQ)            THEN
      KCDTYP  = NSCAT4

!*          3.9.5  SCATT. CODE TYPE 5

    ELSEIF(KCDSQNO == NSCAT5SQ)            THEN
      KCDTYP  = NSCAT5

!*          3.9.6  SCATT. CODE TYPE 6

    ELSEIF(KCDSQNO == NSCAT6SQ)            THEN
      KCDTYP  = NSCAT6

!*          3.9.7  UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN SCATT. CMA CODE TYPE SEQUENCE NUMBER'')')
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN SCATT. CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*          3.10   LIMB OBSERVATIONS

  ELSEIF(KOTSQNO == NLIMBSQ)       THEN
    KOBTYP  = NLIMB

!*          3.10.1 GPS RO

    IF(KCDSQNO == NGPSROSQ)            THEN
      KCDTYP = NGPSRO

!*          3.10.2 Limb radiances

    ELSEIF(KCDSQNO == NLRADSQ)            THEN
      KCDTYP = NLRAD

!*          3.10.9 UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

    ELSE
      WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
      WRITE(KOUT,'('' UNKNOWN LIMB CMA CODE TYPE SEQUENCE NUMBER'')')  
      WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
      CALL ABORTDB('CMOCTMAP','UNKNOWN LIMB CMA CODE TYPE SEQUENCE NUMBER',KCDSQNO)
    ENDIF

!*              3.11

  ELSEIF(KOTSQNO == 11) THEN
    KOBTYP  = 11

!*              3.12

  ELSEIF(KOTSQNO == 12) THEN
    KOBTYP  = 12

!*          3.13  RADAR
  ELSEIF(KOTSQNO == NRADARSQ)   THEN
    KOBTYP  = NRADAR


!*          3.13.1 BUFR RADAR REFLECTIVITY 1

     IF(KCDSQNO == NRADA1SQ)            THEN
       KCDTYP  = NRADA1

!*          3.13.9 UNKNOWN CMA CODE TYPE SEQUENCE NUMBER

     ELSE
       WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
       WRITE(KOUT,'('' UNKNOWN RADAR CMA CODE TYPE'',&
         &'' SEQUENCE NUMBER'')')
       WRITE(KOUT,'('' CMA CODE TYPE SEQ. NO.:'',I10)') KCDSQNO
       CALL ABORTDB('CMOCTMAP','UNKNOWN RADAR CMA CODE TYPE SEQUENCE NUMBER',&
        & KCDSQNO)
     ENDIF

!*          3.14

    ELSEIF(KOTSQNO == 14) THEN
      KOBTYP  = 14

!*           3.15

    ELSEIF(KOTSQNO == 15) THEN
      KOBTYP = 15

!*          3.16   UNKNOWN CMA OBS. TYPE SEQUENCE NO.

  ELSE
    WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
    WRITE(KOUT,'('' UNKNOWN CMA OBS. TYPE SEQUENCE NUMBER'')')
    WRITE(KOUT,'('' CMA OBS. TYPE SEQ. NO.:'',I10)') KOTSQNO
    CALL ABORTDB('CMOCTMAP','UNKNOWN CMA OBS. TYPE SEQUENCE NUMBER',KOTSQNO)
  ENDIF
!*
!-----------------------------------------------------------------------

!        5.       UNKNOWN MAPPING TYPE
!                 --------------------

ELSE
  WRITE(KOUT,'(''0ERROR IN SUBROUTINE CMOCTMAP'')')
  WRITE(KOUT,'('' UNKNOWN MAPPING TYPE'')')
  WRITE(KOUT,'('' MAPPING TYPE:'',A)') CDMAP
  CALL ABORTDB('CMOCTMAP','UNKNOWN MAPPING TYPE',0)
ENDIF
!*
!-----------------------------------------------------------------------

!        6.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('CMOCTMAP',1,ZHOOK_HANDLE)
END SUBROUTINE CMOCTMAP
