SUBROUTINE MODGRIN(KFIELDS,KPAR,PFPDGG)

!**** *MODGRIN*  - Modify the gridpoint fields retrieved from MARS

!     Purpose.
!     --------
!          To apply some modifications to the grid-point fields
!          retrieved from MARS to adapt to IFS conventions.
!**   Interface.
!     ----------
!        *CALL* *MODGRIN(...)*

!        Explicit arguments : KFIELDS - number of fields
!        -------------------- KPAR    - GRIB codes for fields
!                             PFPDGG  - array containing fields

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.     None
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-05-15 (From CPREP1)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - removal of dummy argument LDCPOLE and parts under LDCPOLE.
!         - removal of dummy argument KIND.
!        Modified : 99-03-26 by P. Viterbo (Surface tiling)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Viterbo    24-05-2004   Change surface units
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        H.Hersbach    03-Apr-2008 reset ocean current to zero over land and sea ice
!        H.Hersbach    04-Dec-2009 rescale forecast surface roughness
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0B  , ONLY : LECMWF
USE YOMCST   , ONLY : RG       ,RTT
USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY : NGPTOT
USE YOMGRB   , ONLY : NGRBSTL1 ,NGRBSWVL1 ,NGRBSD   ,NGRBSDOR ,&
 & NGRBSTL2 ,NGRBSWVL2 ,NGRBLSM  ,NGRBSR   ,NGRBLIT, NGRBSTL3 ,&
 & NGRBSWVL3 ,NGRBSRC  ,NGRBSKT  ,NGRBSTL4 ,NGRBSWVL4 ,&
 & NGRBTSN  ,NGRBSSTK  ,NGRBTVL  ,NGRBTVH  ,NGRBCVL  ,&
 & NGRBCVH  ,NGRBSLT ,NGRBCI     ,NGRBUCUR ,NGRBVCUR, NGRBFSR
USE YOETHF   , ONLY : RHOH2O
USE YOEVDF   , ONLY : NVTYPES
USE YOEPHY   , ONLY : LEOCWA    ,LEOCCO   ,LECURR
USE YOMMCC   , ONLY : LMCC04
USE YOMECEARTH, ONLY : LRDLANDICE, LLANDICE

USE YOMDYNCORE, ONLY : LDYNCORE

IMPLICIT NONE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"

!     EXTERNAL INTEGER FUNCTIONS
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPAR(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPDGG(:,:) 
REAL(KIND=JPRB) :: ZSCALE(KFIELDS+1)

INTEGER(KIND=JPIM) :: ICI, ICVH, ICVL, IITVH, IITVL, ILSM, ILIT, IPARAM, ISD,&
 & ISST, ITSA1, ITSA2, ITSA3, ITSA4, ITSK, ITSN, &
 & ITVH, ITVL, IWSA1, IWSA2, IWSA3, IWSA4, IWSK, &
 & ISOTY,JFLD, JROF , IUCUR, IVCUR

LOGICAL         :: LLCURRENT

REAL(KIND=JPRB) :: ZCVH, ZCVL, ZWLMX
REAL(KIND=JPRB) :: ZRWLMAX,ZRCIMIN,ZRVCOV(0:NVTYPES),ZRVLAI(0:NVTYPES)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    MODIFICATIONS TO FIELDS
!              -----------------------

IF (LHOOK) CALL DR_HOOK('MODGRIN',0,ZHOOK_HANDLE)
CALL SURF_INQ(PRWLMAX=ZRWLMAX,PRCIMIN=ZRCIMIN,PRVCOV=ZRVCOV,PRVLAI=ZRVLAI)

!*    1.2   SETUP LOCAL SWITCHES
LLCURRENT= LECMWF .AND. LECURR .AND. (.NOT. LMCC04)

!*    1.3   ODD SCALINGS
DO JFLD=1,KFIELDS
  IPARAM=KPAR(JFLD)
  IF    (IPARAM == NGRBSD  ) THEN
    ZSCALE(JFLD)=RHOH2O
  ELSEIF(IPARAM == NGRBSR .OR. IPARAM == NGRBFSR ) THEN
    ZSCALE(JFLD)=RG
  ELSEIF(IPARAM == NGRBSRC ) THEN
    ZSCALE(JFLD)=RHOH2O
  ELSEIF(IPARAM == NGRBSDOR) THEN
    ZSCALE(JFLD)=RG
  ELSE
    ZSCALE(JFLD)=1.0_JPRB
  ENDIF
ENDDO

CALL GSTATS(1403,0)
!$OMP PARALLEL DO SCHEDULE(STATIC,1) PRIVATE(JROF,JFLD)
DO JFLD=1,KFIELDS
  DO JROF=1,NGPTOT
    PFPDGG(JROF,JFLD)=ZSCALE(JFLD)*PFPDGG(JROF,JFLD)
  ENDDO
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1403,1)

!*    1.4   MODIFICATIONS DEPENDENT ON LAND SEA MASK
ILSM=ISRCHEQ(KFIELDS,KPAR,1,NGRBLSM)
IF(ILSM > KFIELDS) THEN
  WRITE(NULOUT,*) ' LAND-SEA MASK NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
If (LLANDICE) THEN
  ILIT=ISRCHEQ(KFIELDS,KPAR,1,NGRBLIT)
  IF(ILIT > KFIELDS) THEN
    WRITE(NULOUT,*) ' ICE-SHEET MASK NOT FOUND ON FILE '
    CALL ABOR1(' ERROR in MODGRIN')
  ENDIF
ENDIF
ITSK=ISRCHEQ(KFIELDS,KPAR,1,NGRBSKT)
IF(ITSK > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SKIN TEMPERATURE NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITSA1=ISRCHEQ(KFIELDS,KPAR,1,NGRBSTL1)
IF(ITSA1 > KFIELDS) THEN
  WRITE(NULOUT,*) ' SOIL TEMPERATURE 1 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITSA2=ISRCHEQ(KFIELDS,KPAR,1,NGRBSTL2)
IF(ITSA2 > KFIELDS) THEN
  WRITE(NULOUT,*) ' SOIL TEMPERATURE 2 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITSA3=ISRCHEQ(KFIELDS,KPAR,1,NGRBSTL3)
IF(ITSA3 > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SOIL TEMPERATURE 3 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITSA4=ISRCHEQ(KFIELDS,KPAR,1,NGRBSTL4)
IF(ITSA4 > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SOIL TEMPERATURE 4 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IWSK=ISRCHEQ(KFIELDS,KPAR,1,NGRBSRC)
IF(IWSK > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SKIN WETNESS NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IWSA1=ISRCHEQ(KFIELDS,KPAR,1,NGRBSWVL1)
IF(IWSA1 > KFIELDS) THEN
  WRITE(NULOUT,*) ' SOIL WETNESS 1 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IWSA2=ISRCHEQ(KFIELDS,KPAR,1,NGRBSWVL2)
IF(IWSA2 > KFIELDS) THEN
  WRITE(NULOUT,*) ' SOIL WETNESS 2 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IWSA3=ISRCHEQ(KFIELDS,KPAR,1,NGRBSWVL3)
IF(IWSA3 > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SOIL WETNESS 3 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IWSA4=ISRCHEQ(KFIELDS,KPAR,1,NGRBSWVL4)
IF(IWSA4 > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SOIL WETNESS 4 NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ISD=ISRCHEQ(KFIELDS,KPAR,1,NGRBSD)
IF(ISD > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SNOW DEPTH NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITSN=ISRCHEQ(KFIELDS,KPAR,1,NGRBTSN)
IF(ITSN > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SNOW TEMPERATURE NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ISST=ISRCHEQ(KFIELDS,KPAR,1,NGRBSSTK)
IF(ISST > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SST NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITVL=ISRCHEQ(KFIELDS,KPAR,1,NGRBTVL)
IF(ITVL > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' TVL NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ITVH=ISRCHEQ(KFIELDS,KPAR,1,NGRBTVH)
IF(ITVH > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' TVH NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ICVL=ISRCHEQ(KFIELDS,KPAR,1,NGRBCVL)
IF(ICVL > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' CVL NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ICVH=ISRCHEQ(KFIELDS,KPAR,1,NGRBCVH)
IF(ICVH > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' CVH NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ISOTY=ISRCHEQ(KFIELDS,KPAR,1,NGRBSLT)
IF(ISOTY > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' SOTY NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
ICI=ISRCHEQ(KFIELDS,KPAR,1,NGRBCI)
IF(ICI > KFIELDS.AND.LECMWF) THEN
  WRITE(NULOUT,*) ' CI NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IUCUR=ISRCHEQ(KFIELDS,KPAR,1,NGRBUCUR)
IF(IUCUR > KFIELDS.AND.LECMWF.AND.LLCURRENT) THEN
  WRITE(NULOUT,*) ' U-COMP OF OCEAN CURRENT NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
IVCUR=ISRCHEQ(KFIELDS,KPAR,1,NGRBVCUR)
IF(IVCUR > KFIELDS.AND.LECMWF.AND.LLCURRENT) THEN
  WRITE(NULOUT,*) ' V-COMP OF OCEAN CURRENT NOT FOUND ON FILE '
  CALL ABOR1(' ERROR in MODGRIN')
ENDIF
CALL GSTATS(1404,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JROF,IITVL,IITVH,ZCVL,ZCVH,ZWLMX)
DO JROF=1,NGPTOT
  IF (LECMWF .AND. PFPDGG(JROF,ISST) == -9999.) THEN
    PFPDGG(JROF,ISST)=RTT
  ENDIF
  IF (LECMWF .AND. PFPDGG(JROF,ICI) == -9999.) THEN
    PFPDGG(JROF,ICI)=0.0_JPRB
  ENDIF
  IF(PFPDGG(JROF,ILSM) <= 0.5_JPRB) THEN
    IF(LECMWF) THEN
!      Ocean skin is kept from initial data and not copied from SST field
      IF (.NOT. LEOCWA .AND. .NOT. LEOCCO) THEN
        PFPDGG(JROF,ITSK) =PFPDGG(JROF,ITSA1)
      ENDIF
      PFPDGG(JROF,ITSA3)=PFPDGG(JROF,ITSA1)  !PV Delete ASAP; This line is WRONG, as from the 4-layer ice model
      PFPDGG(JROF,ITSA4)=PFPDGG(JROF,ITSA1)  !PV Delete ASAP; This line is WRONG, as from the 4-layer ice model
      PFPDGG(JROF,IWSK) =0.0_JPRB
      PFPDGG(JROF,IWSA3)=0.0_JPRB
      PFPDGG(JROF,IWSA4)=0.0_JPRB
      PFPDGG(JROF,ISD)  =0.0_JPRB
    ENDIF
    PFPDGG(JROF,ITSA2)=PFPDGG(JROF,ITSA1)
    PFPDGG(JROF,IWSA1)=0.0_JPRB
    PFPDGG(JROF,IWSA2)=0.0_JPRB
  ENDIF

! Define landice mask based on snow depth
  IF (LLANDICE .AND. .NOT. LRDLANDICE) THEN
    IF (PFPDGG(JROF,ISD) >=5.0_JPRB ) THEN
      PFPDGG(JROF,ILIT)=MIN(1.0_JPRB, PFPDGG(JROF,ISD)/10._JPRB)
    ELSE IF (PFPDGG(JROF,ISD) <5.0_JPRB ) THEN
      PFPDGG(JROF,ILIT)=0.0_JPRB
    ENDIF
  ENDIF

  IF (LECMWF) THEN
    PFPDGG(JROF,ITSN)=MIN(PFPDGG(JROF,ITSN),RTT)
    IITVL=NINT(PFPDGG(JROF,ITVL))
    IITVH=NINT(PFPDGG(JROF,ITVH))
    ZCVL=PFPDGG(JROF,ICVL)*ZRVCOV(IITVL)
    ZCVH=PFPDGG(JROF,ICVH)*ZRVCOV(IITVH)
    ZWLMX=ZRWLMAX*(1.0_JPRB-ZCVL-ZCVH+ZCVL*ZRVLAI(IITVL)+ZCVH*ZRVLAI(IITVH))
    PFPDGG(JROF,IWSK)=MAX(0.0_JPRB,MIN(ZWLMX,PFPDGG(JROF,IWSK)))
    IF (PFPDGG(JROF,ICI) <= ZRCIMIN) PFPDGG(JROF,ICI)=0.
  ENDIF
  IF (LLCURRENT) THEN
!    Reset missing values (product land, ice) to zero
     IF (PFPDGG(JROF,IUCUR) == -9999.) THEN
       PFPDGG(JROF,IUCUR)=0.0_JPRB
     ENDIF
     IF (PFPDGG(JROF,IVCUR) == -9999.) THEN
       PFPDGG(JROF,IVCUR)=0.0_JPRB
     ENDIF
!    Reset to zero over model land
     IF(PFPDGG(JROF,ILSM) > 0.5_JPRB) THEN
       PFPDGG(JROF,IUCUR)=0.0_JPRB
       PFPDGG(JROF,IVCUR)=0.0_JPRB
     ENDIF
  ENDIF
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1404,1)

!           if you run the dynamical core only, all initial sfc data
!           read from a file will be overwritten by zero
IF (LDYNCORE) THEN
  WRITE (NULOUT,*) 'DYNAMICAL CORE, ',&
   &'GG fields are set to 0. (subroutine subgridg/modgrin), '
  DO JFLD=1,KFIELDS
    IPARAM=KPAR(JFLD)
    IF    (IPARAM == NGRBLSM  ) THEN
      DO JROF=1,NGPTOT
        PFPDGG(JROF,JFLD)=0.02_JPRB
      ENDDO
    ELSE
      DO JROF=1,NGPTOT
        PFPDGG(JROF,JFLD)=0._JPRB
      ENDDO
    ENDIF
  ENDDO
ENDIF
  
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('MODGRIN',1,ZHOOK_HANDLE)
END SUBROUTINE MODGRIN
