SUBROUTINE SP2LNSP(PSPEC)

!**** *SP2LNSP* -Transform spectral surface pressure into spectral Ln(Ps)

!     Purpose.
!     --------
!       Transform spectral surface pressure into spectral Ln(Ps)

!**   Interface.
!     ----------
!        *CALL* *SP2LNSP(...)*

!        Explicit arguments :
!        --------------------
!           PSPEC    : spectral input/output array

!        Implicit arguments :
!        ------------------

!     Method.
!     -------
!        See documentation

!     Externals.  INV_TRANS, DIR_TRANS - transforms (TRANS lib)
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 02-02-19
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSPEC2 
USE YOMGEM   , ONLY : NGPTOT
USE YOMMP    , ONLY : MYSETV   ,NBSETSP

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPEC(NSPEC2) 
INTEGER(KIND=JPIM) :: ILF, J
INTEGER(KIND=JPIM) :: IVSETSC(1)

REAL(KIND=JPRB) :: ZREEL(NGPTOT,1,1), ZSPEC(1,NSPEC2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "inv_trans.h"
#include "dir_trans.h"
END INTERFACE

IF (LHOOK) CALL DR_HOOK('SP2LNSP',0,ZHOOK_HANDLE)
IVSETSC(1) = NBSETSP

IF(NBSETSP == MYSETV) THEN
  ILF = 1
  DO J=1,NSPEC2
    ZSPEC(1,J)=PSPEC(J)
  ENDDO
ELSE
  ILF = 0
ENDIF

! All processors performs transforms because they all get gridpoints
CALL INV_TRANS(PSPSCALAR=ZSPEC(1:ILF,:),KPROMA=NGPTOT,KVSETSC=IVSETSC,&
 & PGP=ZREEL)  
DO J=1,NGPTOT
  ZREEL(J,1,1)=LOG(ZREEL(J,1,1))
ENDDO
CALL DIR_TRANS(PSPSCALAR=ZSPEC(1:ILF,:),KPROMA=NGPTOT,KVSETSC=IVSETSC,&
 & PGP=ZREEL)  

IF(NBSETSP == MYSETV) THEN
  DO J=1,NSPEC2
    PSPEC(J)=ZSPEC(1,J)
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('SP2LNSP',1,ZHOOK_HANDLE)
END SUBROUTINE SP2LNSP
