SUBROUTINE SU0YOMB

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARKIND2  ,ONLY : JPRH
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG     ,NDGNH    ,NSMAX    ,NSMIN    ,&
 & NCMAX    ,NTMAX    ,NUNDEFLD,&
 & NDLSUR   ,NDGSAG   ,NDGENG  ,&
 &NPROMA,  NFLEVG, NGPBLKS
USE YOMLUN   , ONLY : NULOUT   ,NULDILA  ,NULCONT  ,NULROTC
USE YOMCT0   , ONLY : NCONF    ,LBACKG   ,LSPBSBAL, LFPART2  ,LELAM,&
 & LRETCFOU ,LWRTCFOU ,LOBS, LOBSC1, LSCREEN, LIFSTRAJ, LSFORC, LIFSMIN
USE YOMGEM   , ONLY :    RMUCEN   ,RSTRET, NGPTOT
USE YOMNMIA  , ONLY : NVMODM   ,LNOINI
USE YOMLDDH  , ONLY : LSDDH
USE YOMVAR   , ONLY : LMODERR  ,LVARBC, LJCDFI
USE YOMJG    , ONLY : NJBVERB  ,LJBENER  ,LSPFCE
USE YOMJQ    , ONLY : LSTATMERR
USE YOMWAVELET , ONLY : LJBWAVELET, N_WAVELET_SCALES
USE YOMSIMPHL, ONLY : LRAYSP
USE YOMLCZ   , ONLY : L_SUBSPACE_SVS, LFORCEWR
USE VARBC_SETUP, ONLY : VARBC_SETUP_MIN
USE YOMCVA   , ONLY : NVAPARAM
USE TRAJECTORY_MOD, ONLY : LTRAJRESET, LTRAJHR, allocate_trajectory,&
 & READ_TRAJECTORY, GET_TRAJ_GRID, &
 & LREADGPTRAJ, LTRAJHR_ALTI

USE YOMGFL     , ONLY : GFL5
USE YOM_YGFL , ONLY : YGFL, YGFLC, NGRG
USE YOMGMV , ONLY : YT5

USE YEMJK    , ONLY : LEJK
USE YOMCOAPHY, ONLY : NPHYINT
USE YOMINI   , ONLY : NEINI
USE STOPH_MIX, ONLY : LSTOPH
USE YOMSPSDT,  ONLY : LSPSDT
USE YOMSPJB  , ONLY : SPJB
USE YOE_CUCONVCA, ONLY : INI_CUCONVCA

!**** *SU0YOMB*  - INITIALIZE LEVEL 0 COMMONS AND SOME HIGHER (PART 2)

!     PURPOSE.
!     --------
!           INITIALIZE LEVEL 0 COMMONS (CONSTANT ALONG ALL THE JOB)
!        AND SOME HIGHER LEVEL COMMONS. CALLS ROUTINES THAT PERFORM
!        ALL THE PREPARATIONS NEEDED TO EXECUTE MODEL. THE TASK OF
!        INITIALIZING THE COMMONS IS DIVIDED BETWEEN TWO ROUTINES
!        (SU0YOMA AND SU0YOMB) TO AVOID PROBLEMS WITH USING POINTER
!        ARRAYS WHOSE DIMENSIONS ARE NOT DEFINED UNTIL AFTER CALLING
!        SUDIM.

!     INTERFACE.
!     ----------
!        *CALL* *SU0YOMB*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS
!        --------------------
!        NONE

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!      SUPP   - INITIALIZE POST-PROCESSING
!      SUIOS  - INITIALIZE IO SCHEME
!      SUMP   - INITIALIZE MESSAGE PASIING DISTRIBUTION
!      SUALSPA- ALLOCATE SPECTRAL SPA3 ARRAYS
!      SUALSPA1-ALLOCATE SPECTRAL SPA2 ARRAYS
!      SUALSPAJB-ALLOCATE SPECTRAL SPAxJB ARRAYS
!      SUALGES- ALLOCATE SPACE FOR JB COMPUTATIONS
!      SUALCAN- ALLOCATE SPACE FOR CANARI GRID POINTS COMPUTATIONS
!      SULAP  - INITIALIZE LAPLACE SPACE CONSTANTS COMMONS
!      SUVERT - INITIALIZE VERTICAL GEOMETRY
!      SUGEM1A- INITIALIZE GEOMETRY PART 1-a
!      SUTRANS- INITIALIZE TRANSFORM PACKAGE
!      SUETRANS -INITIALIZE ALADIN TRANSPACK
!      SUGEM  - INITIALIZE GEOMETRY
!      SUGEM2 - INITIALIZE GEOMETRY
!      SULFI  - READ SOME FEATURES OF LFI FILES ( FOR SUDIL OR SUROT).
!      SUDIL  - INITIALIZE DILATATION AND CONTRACTION MATRIX.
!      SUROT  - INITIALIZE ROTATION MATRIX.
!      MATOPEN- OPEN ROTATION, DILATATION AND CONTRACTION MATRIXES
!      SUMDDH - INITIALIZE DOMAINE AND MASKS FOR DDH
!      SUSIMPR- INITIALIZE COMPUTATION OF THE WEIGHTS AS FOR DDH
!      SUGRIB - INITIALIZE GRIB CODING PARAMETERS
!      SUOPH  - INITIALISE INPUT/OUTPUT HANDLING
!      SUDYN  - INITIALISE DYNAMICS
!      SULSFORC  - INITIALISE LARGE SCALE FORCINGS
!      SUMCC  - INITIALIZE SPECIAL KEYS FOR THE CLIMATE VERSION
!      SUPHY  - INITIALISE PHYSICS
!      SURES  - Initialize restart
!      SUSC2A  - INITIALIZE Multitasking.
!      SUSC2B  - INITIALIZE Multitasking.
!      SUBFPOS- INITIALIZE FULL POST-PROCESSING (2nd part)
!      SUALTDH- MEMORY ALLOCATION FOR CUMULATED DDH ARRAYS
!      SUMODE3 - INITIALIZE HORIZONTAL NORMAL MODES
!      SULCZ  - INITIALIZE LANCZOS VARIABLES
!      SUJBCOV - INITIALIZE Jb ERROR COVARIANCE MODEL
!      SUJBWAVELET - INITIALIZE Jb ERROR COVARIANCE MODEL FOR 'WAVELET JB'
!      SUJBBAL - INITIALIZE Jb Balance operators
!      SUINFCE - INITIALIZE Jb gridpoint background error standard deviations
!      SUJBCHVAR - INITIALIZE coefficients for humidity change of variable
!      SUSCAL - INITIALIZE SCALAR PRODUCT
!      SUCFU  - INITIALIZE Cumulated diagnostics switches
!      SUXFU  - INITIALIZE Instantaneous diagnostics switches
!      SU_SUBSPACE - INITIALIZE subspace for singular vector computation
!      SUEGEO1 - INITIALIZE HORIZONTAL GEOMETRY FOR ALADIN( first part)
!      SUEMP - INITIALIZE MESSAGE PASSING DISTRIBUTION (for Aladin)
!      SUEGEO2 - INITIALIZE HORIZONTAL GEOMETRY FOR ALADIN( second part)
!      SUELAP  -INITIALIZE LAPLACE SPACE CONSTANTS COMMONS FOR ALADIN
!      SUEFFT - INITIALIZE FFT'S (MERIDIONAL -FOR ALADIN)
!      SUEBIG - INITIALIZE BIPERIODOCIZATION AND SPECTRAL FIT
!      SUECUV - INITIALIZE CURVATURE TERMS FOR ALADIN
!      SUEBICU- INITIALIZE RELAXATION COEFFICIENTS
!      SUEOPH - INITIALISE INPUT/OUTPUT HANDLING FOR ALADIN
!      SUEDYN - INITIALIZE DYNAMICS FOR ALADIN
!      SUESC2 - INITIALIZE MULTITASKING OF SCAN2MSM (ALADIN PART)
!      SUELCZ - INITIALIZE LANCZOS PARAMETERS AND CHECK WITH ALADIN GRID
!      SUELGES - ALLOCATE SPACE FOR JB COMPUTATIONS (ALADIN PART)
!      SUVAREPS - INITIALIZE VARIABLES FOR VAREPS SYSTEM         
!      SUELLJK - ALLOCATE SPACE FOR JK COMPUTATIONS
!      SUEJK    - INITIALIZE JK HANDLING
!      SUEJKNORM- INITIALIZE JK NORM




!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL      : 87-10-15
!        BILL HECKLEY  : 88-12-07  INTRODUCTION OF OPTION OF COMPUTING NORMAL
!                        MODES LCONMO
!        ALAIN  JOLY   : 10-08-90  1/91, INTRODUCTION DIAGNOSTICS DDH
!        PASCAL MARQUET: 90-12-15  INTRODUCTION OF UNIT "NPINMI" IN SUMODE3
!        PASCAL MARQUET: 90-04-01  CALL SUMODE3 without argument.
!        BILL HECKLEY  : 91-02-20  COMPUTATION OF FIRST GUESS ERRORS
!        BILL HECKLEY  : 91-06-05  NORMAL MODES COMPUTED ACCORDING TO LCONMO
!        Roberto Buizza: 92-02-26  Unstable modes computation using Lanczos
!        D. GIARD      : 92-01-16  tests of LCONMO suppressed, since :
!                           - these tests are done in SUNMI or SUMODE3
!                           - normal modes CAN be read from files in SUMODE3
!        D. GIARD      : 92-03-09  SUMODE3 is not called if LNOINI
!        Y. BOUTELOUP  : 92-04-15  SUMUL and SUSC2 are not called
!                        if LMLTSK = FALSE
!        J.M. PIRIOU   : 92-05-05  CALL SUALTDH moved after CALL SUSC2.
!        P.Courtier    : 92-12-18  move sumul to allow multitasking of suleg
!        D. Giard      : 92-12-08  call SULFI before SUROT or SUDIL
!        D. Giard      : 92-12-08  call SULFI before SUROT or SUDIL
!        Bill Heckley  : 93-05-19  call SUECGES after SUGERCO
!        R. El Khatib  : 94-02-03  SULEG and SUGEM1 moved inside SUGEM
!        K. YESSAD     : 93-07-30  SULEG not called in conf 911.
!       Erik Andersson : 06:07:93  Non-separable structure functions
!        Roberto Buizza: 93-10-27  Covariance matrix constraint when
!                                  computing optimal perturbations
!        R. El Khatib  : 94-03-03  Full-POS implementation
!        R. El Khatib  : 94-04-18  More Full-POS implementation
!        D. Giard      : 94-07-26  call to MATOPEN; SULFI changed
!        L. Isaksen    : 95-06-06  Set-up for distributed memory version
!        F. Bouttier   : 96-02-02  New Jb setup, Improve readability
!        M.Hamrud      : 96-03-10  Remove call to SUMNFCE
!        P. Le Moigne  : 96-03-18  SUSCAL not called if conf 9*
!        M. Siroka     : 98-05-12  call SUELGES for ALADIN
!        C. Fischer    : 98-04-22 call SUELCZ for aladin conf e601
!        M. Janiskova  : 99-02-21  call SUSIMPR for simp.radiation
!        K. YESSAD     : 99-02-17  options LRETCFOU, LWRTCFOU.
!        K. YESSAD     : 99-04-13  adequate initialisation of granul. factors 
!        R. El Khatib  : 99-04-22  NFPVT0 moved inside SUBFPOS ;
!                                  SUFPG after SUOPH
!        R. El Khatib  : 99-08-06  SUFPG moved inside SUBFPOS
!        R. El Khatib  : 99-11-10  SUSC2 => SUSC2A + SUSC2B
!        C. Temperton  : 00-04-12  SUPHY etc. avoided for shallow water model
!        M. Fisher     : 00-10-26  Reorganize Jb setup
!        D. Giard      : 00-09-15  SUSC2A/B called by configuration 923
!        R. El Khatib  : 01-02-01  SUCFU/XFU moved from su0yoma to su0yomb
!                        after calling susta
!        P. Saez       : 00-12-12  SUSC2B, SUALSPA  not called by conf 901
!        R. El Khatib  : 01-08-07  Pruning options
!        J. Masek      : 01-05-25 modified interface to SUEGEO2
!        F. Taillefer  : 02-03-30  SUSCAL not called if conf 7*
!        Y. Tremolet   : 02-08-14 Moved SUALCTV here.
!        C. Fischer    : 03-09-30 Remove call to suellr
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        E .Holm       : 03-12-08  Include humidity change of variable
!                                  coefficient setup (SUJBCHVAR)
!        M.Fisher      17-03-2004  Added calls to SUJBWAVET0 and SUJBWAVTRANS
!        M.Leutbecher  07-05-2004  Added calls to SU_SUBSPACE
!        C. Fischer    : 04-02-27  Merge Aladin into Arpege/IFS su0yomb:
!                        - moved up call to susc2a (susc2 split for LAM)
!                        - all sue* routines now in the IFS setup
!                        - historical header of the file aladinized
!        V.Guidard     : 04-04-01  Add Jk computation for ALADIN
!        K. Yessad     : 04-07-06  Merge Aladin SUEIOS into Arpege/IFS SUIOS.
!        P. Termonia   : 04-08-31  - add call to SUMCUF
!        R. Buizza     : 09-03-05 Add call to SUVAREPS to initialize VAREPS
!        C. Temperton  : 06-02-07  SUALCTV avoided for shallow water model
!        S. Malardel   : 06-06-06 Add call to SULSFORC
!        R. El Khatib  : 31-Aug-2007 Initialise uninitialised variable
!        B. Strajnar   : 06-02-08 Allow call to SUINFCE for LSPFCE=.F.
!        B. Chapnik    : 15-06-08 Add setup of Jk modulation wrt/ level and parameter
!        R. El Khatib  : 24-Oct-2008 Merge sueoph and suoph
!        K. Yessad     : 09-09-08 No recalc of Gaussian lat and weights in SUDIL
!        K. Yessad     : 15-09-08 Prune conf 951.
!        A. Deckmyn    : 01-10-08 LAM wavelets
!        P. Termonia   : 22-12-08 move the call to SUDFI here (from SUINI)
!        A. Dethof     : 17-11-08 LOG NOX JB
!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZGM   (0:NDLSUR,NDGSAG:NDGENG)
REAL(KIND=JPRB) :: ZGNORX(0:NDLSUR,NDGSAG:NDGENG)
REAL(KIND=JPRB) :: ZGNORY(0:NDLSUR,NDGSAG:NDGENG)
REAL(KIND=JPRB) :: ZGELAM(0:NDLSUR,NDGSAG:NDGENG)
REAL(KIND=JPRB) :: ZGELAT(0:NDLSUR,NDGSAG:NDGENG)
REAL(KIND=JPRH) :: ZDLMU(NDGLG)
REAL(KIND=JPRB) :: ZZW(NDGLG)

CHARACTER (LEN = 35) ::  CLINE

INTEGER(KIND=JPIM) :: ICONF, IFACTD, IFACTR, INIMES

LOGICAL :: LLIMST, LLSHW, LLVEQ, LL2D

INTEGER(KIND=JPIM) :: JGFL, ISTEP, JSTGLO, ICEND, IBL
LOGICAL :: LLASTRAJ
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL5(:,:,:,:)

REAL(KIND=JPRB) :: ZLATI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "inifger.intfb.h"
#include "sualcan.intfb.h"
#include "sualcos.intfb.h"
#include "sualctv.intfb.h"
#include "sualfoutc.intfb.h"
#include "sualges.intfb.h"
#include "suallr.intfb.h"
#include "suallt.intfb.h"
#include "sualmdh.intfb.h"
#include "sualspa.intfb.h"
#include "sualspa1.intfb.h"
#include "sualspajb.intfb.h"
#include "sualtdh.intfb.h"
#include "subfpos.intfb.h"
#include "sucfu.intfb.h"
#include "sudfi.intfb.h"
#include "sudil.intfb.h"
#include "sudyn.intfb.h"
#include "suebicu.intfb.h"
#include "suebig.intfb.h"
#include "suecges.intfb.h"
#include "suecuv.intfb.h"
#include "suedyn.intfb.h"
#include "suegeo1.intfb.h"
#include "suegeo2.intfb.h"
#include "suejbbal.intfb.h"
#include "suejbcov.intfb.h"
#include "suejk.intfb.h"
#include "suejknorm.intfb.h"
#include "suelap.intfb.h"
#include "suelcz.intfb.h"
#include "suelges.intfb.h"
#include "suelljk.intfb.h"
#include "suemodjk.intfb.h"
#include "suemp.intfb.h"
#include "suesc2.intfb.h"
#include "suescal.intfb.h"
#include "suetrans.intfb.h"
#include "suevargp.intfb.h"
#include "sugem.intfb.h"
#include "sugem1a.intfb.h"
#include "sugem2.intfb.h"
#include "sugrib.intfb.h"
#include "su_grib_api.intfb.h"
#include "suinfce.intfb.h"
#include "sufw.intfb.h"
#include "suios.intfb.h"
#include "sujbbal.intfb.h"
#include "sujbcov.intfb.h"
#include "sujbwavelet.intfb.h"
#include "sujbwavelet0.intfb.h"
#include "sujbwavtrans.intfb.h"
#include "sujq.intfb.h"
#include "sujbchvar.intfb.h"
#include "sulap.intfb.h"
#include "sulcz.intfb.h"
#include "sulfi.intfb.h"
#include "sulsforc.intfb.h"
#include "sumcc.intfb.h"
#include "sumddh.intfb.h"
#include "sumode3.intfb.h"
#include "sumoderr.intfb.h"
#include "sump.intfb.h"
#include "suoph.intfb.h"
#include "suphy.intfb.h"
#include "supp.intfb.h"
#include "surand1.intfb.h"
#include "surcoftc.intfb.h"
#include "sures.intfb.h"
#include "surinc.intfb.h"
#include "surot.intfb.h"
#include "susc2a.intfb.h"
#include "susc2b.intfb.h"
#include "suscal.intfb.h"
#include "susimpr.intfb.h"
#include "suskf.intfb.h"
#include "suspsdt.intfb.h"
#include "sutrans.intfb.h"
#include "suvareps.intfb.h"
#include "suvert.intfb.h"
#include "suxfu.intfb.h"
#include "su_subspace.intfb.h"
#include "sumcuf.intfb.h"
#include "sucoaphy.intfb.h"
#include "sudimo.intfb.h"
#include "suoaf.intfb.h"
#include "sualobs.intfb.h"
#include "suleg.intfb.h"
#include "suejbwavelet.intfb.h"
#include "suejbwavelet_bmatrix.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU0YOMB',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CALL GSTATS(39,0)
CLINE='----------------------------------'

ICONF=NCONF/100

LLSHW=NCONF == 201.OR.NCONF == 421.OR.NCONF == 521
LLVEQ=NCONF == 202.OR.NCONF == 422.OR.NCONF == 522
LL2D=LLSHW.OR.LLVEQ

!*    Initialize post-processing
WRITE(NULOUT,*) '---- Set up post-processing ---------',CLINE
CALL SUPP(' ')

!*    Initialize I/O-scheme
WRITE(NULOUT,*) '---- Set up I/O scheme --------------',CLINE
CALL SUIOS

!*    Initialize geometry (very first part)
IF (.NOT.LELAM) THEN
  WRITE(NULOUT,*) '---- Set up geometry, part 1a -------',CLINE
  CALL SUGEM1A(ZDLMU,ZZW)
ENDIF

!*    Initialize LFI files parameters
WRITE(NULOUT,*) '---- Set up lfi files ----',CLINE
IF (NCONF == 911.OR.NCONF == 912) THEN
  CALL SULFI(NULOUT,NSMIN,NSMAX,RSTRET,LLIMST,INIMES,IFACTD,IFACTR)
ELSE
  CALL SULFI(NULOUT,NTMAX,NCMAX,RSTRET,LLIMST,INIMES,IFACTD,IFACTR)
ENDIF

!*    Initialize dilatation or rotation matrix
IF (.NOT.LELAM) THEN
  IF (NCONF == 911) THEN
    WRITE(NULOUT,*) '---- Set up dilatation matrix -----',CLINE
    CALL SUDIL(NSMAX,NSMIN,NDGLG,NDGNH,NULDILA,NULCONT,NULOUT,RSTRET,LLIMST,&
     & INIMES,IFACTD,ZDLMU,ZZW)  
    STOP
  ENDIF
  IF (NCONF == 912) THEN
    WRITE(NULOUT,*) '---- Set up rotation matrix -------',CLINE
    ZLATI=ASIN(RMUCEN)
    CALL SUROT(NSMIN,NSMAX,NULROTC,NULOUT,ZLATI,'MATROTC',&
     & LLIMST,INIMES,IFACTR)  
    STOP
  ENDIF
ENDIF

!*    Initialize Aladin geometry (very first part)
IF (LELAM) THEN
  WRITE(NULOUT,*) '---- Set up Aladin geometry, part 1 -',CLINE
  CALL SUEGEO1(ZGM,ZGNORX,ZGNORY,ZGELAM,ZGELAT)
ENDIF

!*    Initialize Aladin Jk handling
IF (LELAM.AND.(ICONF == 1)) THEN
  WRITE(NULOUT,*) '---- Set up Aladin Jk handling -',CLINE
  CALL SUEJK
ELSE
  LEJK=.FALSE.
ENDIF


!*    Initialize transform package
WRITE(NULOUT,*) '------ Initialize transform package --',CLINE
IF (LELAM) THEN
  CALL SUETRANS
ELSE
  CALL SUTRANS('SETUP')
ENDIF

!     Initialize distributed memory partitioning parameters
WRITE(NULOUT,*) '-- Set up distributed mem partition -',CLINE
IF (LELAM) THEN
  CALL SUEMP
ELSE
  CALL SUMP
ENDIF

!*AF
IF (NCONF /= 911 .AND. NCONF /= 912) CALL SULEG

!*       1.4   DIMENSIONS OF OBS. PROCESSING, JO AND JG RELATED ARRAYS
CALL SUDIMO(NULOUT)

!*    Setup observation array format
IF (LOBS) THEN
  WRITE(NULOUT,*) '------ Set up observation format ---',CLINE
  CALL SUOAF
  CALL SUALOBS ! was originally in ./ifs/setup/suallo.F90
ENDIF

! Set up for variational bias correction. For now, do not allow
! any other parameters to be part of the control vector.

NVAPARAM=0
IF (LOBS .AND. LVARBC) THEN
  IF (IABS(NCONF/100)==1) THEN
    CALL VARBC_SETUP_MIN (NVAPARAM)
    IF (NVAPARAM==0) LVARBC = .false.
  ENDIF
ENDIF



!*AF END

!     Allocate spectral arrays
WRITE(NULOUT,*) '-- Set up spectral arrays allocation-',CLINE
IF(NCONF /= 901.AND.NCONF /= 911.AND.NCONF /= 912.AND.NCONF /= 923) THEN
  CALL SUALSPA
ENDIF
CALL SUALSPA1
IF (ICONF == 1.OR.ICONF == 4.OR.ICONF == 5.OR.ICONF == 6.OR.ICONF == 8) THEN
  CALL SUALLR
  CALL SUALSPAJB(SPJB)
ENDIF
!     Setup cost function arrays
CALL SUALCOS

!     Setup dimensioning for wavelet Jb
IF (LJBWAVELET) THEN
  IF (LELAM) THEN
    WRITE(NULOUT,*) '-- Setup Aladin wavelet Jb: preliminaries'
    WRITE(NULOUT,*) '(I have to do this before getting to the Control Vector setup!)'
    CALL SUEJBWAVELET
    N_WAVELET_SCALES=0 ! We don't use it, so for safety, set it to 0
  ELSE
  WRITE(NULOUT,*) '-- Setup spectral filters for wavelet Jb'
  CALL SUJBWAVELET0
  WRITE(NULOUT,*) '-- Define multiple-resolution transforms for wavelet Jb'
  CALL SUJBWAVTRANS
  ENDIF
ELSE
  N_WAVELET_SCALES=0
ENDIF

!     Allocate Fourier arrays containing transmission coefficients.
IF (LRETCFOU.OR.LWRTCFOU.OR.(NUNDEFLD > 0)) THEN
  WRITE(NULOUT,*) '-- Set up Fourier arrays allocation-',CLINE
  CALL SUALFOUTC
ENDIF

!*    Initialize Aladin geometry (second part)
IF (LELAM) THEN
  WRITE(NULOUT,*) '---- Set up Aladin geometry, part 2 -',CLINE
  CALL SUEGEO2
  IF (ICONF == 1) CALL SUEVARGP
ENDIF

!     Initialize Laplace space constants commons
WRITE(NULOUT,*) '---- Set up Laplace space constants -',CLINE
CALL SULAP
IF (LELAM) CALL SUELAP

!*    Initialize geometry (first part) and Legendre polynomials
WRITE(NULOUT,*) '---- Set up vertical geometry -------',CLINE
CALL SUVERT
WRITE(NULOUT,*) '---- Set up geometry, part 1b -------',CLINE
IF (.NOT.LELAM) THEN
  WRITE(NULOUT,*) '---- Set up Legendre polynomials ----',CLINE
  CALL SUGEM
ENDIF

!*    Set up DFI (we need geometry for SSDFI)
IF (LELAM) THEN
  IF (NEINI == 2.OR.NEINI == 4) CALL SUDFI
!       IF (NEINI.EQ.1.OR.NEINI.EQ.3.OR.NEINI.EQ.4) CALL SUENMI
  IF (NEINI == 1.OR.NEINI == 3.OR.NEINI == 4)&
   & CALL ABOR1(' SUINI : NMI NOT CODED IN ALADIN !')  
ELSE
  IF (NEINI == 2.OR.NEINI == 4) CALL SUDFI
ENDIF

!*    Allocate background constraint arrays
WRITE(NULOUT,*) '------- Allocate Jb arrays ----------',CLINE
CALL SUALGES
IF (LELAM) CALL SUELGES

!*    Allocate OI CANARI grid points arrays
IF(ICONF == 7) THEN
  WRITE(NULOUT,*) '------- Allocate CANARI arrays ----',CLINE
  CALL SUALCAN
ENDIF


!*    Initialize geometry (second part)
IF (.NOT.LELAM) THEN
  WRITE(NULOUT,*) '---- Set up geometry, part 2 --------',CLINE
  CALL SUGEM2
ENDIF

!*    Initialize basis of Fourier coefficients for 'TC' coefficients.
IF (LRETCFOU.OR.LWRTCFOU.OR.NUNDEFLD >= 1) THEN
  WRITE(NULOUT,*) '---- Set up basis of Fourier for TC -',CLINE
  CALL SURCOFTC
ENDIF

!*    Initialize buffers for gridpoint scanning, part1
WRITE(NULOUT,*) '---- Set up gridpoint scanning, part A ----',CLINE
CALL SUSC2A
! no call to susc2b at this place because of Aladin mismatch
! for LAM duplicatum the call to susc2b is moved down

!*    Initialize biper. and spectral fit of yomgc in Aladin
IF (LELAM) THEN
  WRITE(NULOUT,*) '---- Set up biper. and fit of gm ----',CLINE
  CALL SUEBIG(ZGM,ZGNORX,ZGNORY,ZGELAM,ZGELAT)
ENDIF

!*    Initialize curvature terms in Aladin
IF (LELAM) THEN
  WRITE(NULOUT,*) '-- Set up curvature terms in Aladin -',CLINE
  CALL SUECUV
ENDIF

!*    Initialize relaxation coefficients for coupling in Aladin
IF (LELAM) THEN
  WRITE(NULOUT,*) '-- Set up relax. coeff. in Aladin ---',CLINE
  CALL SUEBICU
ENDIF


!     Initialize model:

!*    Initialize Dynamics
WRITE(NULOUT,*) '---- Set up model dynamics ----------',CLINE
CALL SUDYN(NULOUT)
IF (LELAM) CALL SUEDYN

!*    Initialize control of DFI: SUFW
WRITE(NULOUT,*) '---- Set up DFI initialization: SUFW ',CLINE
!IF (NEINI == 2 .OR. NEINI == 4 .OR. (LJCDFI .AND. (NCONF==131))) THEN
IF (NEINI == 2 .OR. NEINI == 4 .OR. LJCDFI ) THEN
  CALL SUFW
ENDIF

!*    Initialize Large Scale Forcings
WRITE(NULOUT,*) '---- Set up large scale forcings ----------',CLINE
IF (LSFORC) CALL SULSFORC(NULOUT)

IF(NPHYINT > 0)CALL SUCOAPHY


IF (.NOT.LL2D) THEN
!*    Initialize Physics
  WRITE(NULOUT,*) '---- Set up model physics -----------',CLINE
  CALL SUPHY(NULOUT)
!*    Initialize special keys for the climate version
  WRITE(NULOUT,*) '---- Set up MCC climate model keys --',CLINE
  CALL SUMCC(NULOUT)
ENDIF

!*    Initialize cumulated fluxes requests
WRITE(NULOUT,*) '------ Set up cumulated fluxes diags ---',CLINE
CALL SUCFU(NULOUT)

!*    Initialize instantaneous fluxes requests
WRITE(NULOUT,*) '------ Set up instantaneous fluxes diags ',CLINE
CALL SUXFU(NULOUT)

!*    Initialize variables for VAREPS (NB: must be called before SUGRIB)
WRITE(NULOUT,*) '- Set up VAREPS configuration',CLINE
CALL SUVAREPS

!*    Initialize filter for monitoring the coupling updates
WRITE(NULOUT,*) '------ Set up monitoring coupling-updates',CLINE
CALL SUMCUF

!*    Initialise restart mechanism
WRITE(NULOUT,*) '---- Set up restart mechanism ------',CLINE
CALL SURES(NULOUT)

!*    Initialize buffers for gridpoint scanning, part2
WRITE(NULOUT,*) '---- Set up gridpoint scanning, part B ----',CLINE
IF (NCONF /= 901) CALL SUSC2B
IF (LELAM) CALL SUESC2

!*    Initialize domains and masks for DDH

CALL SUALMDH

IF ( LSDDH ) THEN
  WRITE(NULOUT,*) '---- Set up DDH diagnostic domains ',CLINE
  CALL SUMDDH
ENDIF

IF (.NOT.LSDDH.AND.LRAYSP) THEN
  WRITE(NULOUT,*) '---- Set up for simp.rad.if not ddh',CLINE
  CALL SUSIMPR
ENDIF

!*    Initialize GRIB coding parameters
WRITE(NULOUT,*) '---- Set up files : GRIB parameters -',CLINE
CALL SUGRIB

!*    Initialize file handling
WRITE(NULOUT,*) '---- Set up files : names, FA, FDB --',CLINE
CALL SUOPH

!*    Full Post-processing (2nd part)
WRITE(NULOUT,*) '- Set up F-post processing, part 2',CLINE
CALL SUBFPOS

!*    GRIB API
WRITE(NULOUT,*) '- Set up GRIB API usage',CLINE
CALL SU_GRIB_API

!*    Memory allocation for cumulated DDH arrays (horizontal domains diags)
WRITE(NULOUT,*) '---- Set up DDH diags allocation --',CLINE
CALL SUALTDH

!*    Initialize normal modes if initialization code is actually used
IF (.NOT.LNOINI.AND.(.NOT.LELAM)) THEN
  WRITE(NULOUT,*) '----- Set up normal modes --------',CLINE
  CALL SUMODE3(NVMODM)
ENDIF

!     Setup model error arrays
IF (LMODERR.OR.LSTATMERR) THEN
  WRITE(NULOUT,*) '-- Setup model error arrays-',CLINE
  CALL SUMODERR
ENDIF

!     Allocate space for control variable
IF (NCONF/100 /= 9 .AND. NCONF /= 1 .AND. NCONF /= 302 .AND. NCONF /= 201) THEN
  WRITE(NULOUT,*) '-- Allocate control variable-',CLINE
  CALL SUALCTV
ENDIF

!*    Initialize incremental 3D/4D-var
IF(.NOT.(ICONF == 7.OR.ICONF == 9)) THEN
  WRITE(NULOUT,*) '--- Set up incremental assimilation',CLINE
  CALL SURINC
ENDIF

!*    Initialize control of the Lanczos algorithm
WRITE(NULOUT,*) '---- Set up Lanczos algorithm -------',CLINE
LFORCEWR=.FALSE.
IF(.NOT.(ICONF == 0.OR.ICONF == 2.OR.NCONF == 302)) THEN
  IF (LELAM) THEN
    CALL SUELCZ
  ELSE
    CALL SULCZ
  ENDIF
ENDIF

!*    Allocate the trajectory
IF(NCONF/100 == 1.OR.NCONF == 801.OR.NCONF == 601) THEN
  CALL SUALLT
  LTRAJRESET=.true.
ENDIF

IF (LTRAJHR .AND. LIFSTRAJ) THEN
  CALL allocate_trajectory
ENDIF

!*    Get trajectory values for NOX LOG variable
IF(NGRG > 0) THEN
  IF (LIFSMIN.AND.(LTRAJHR.AND.LTRAJHR_ALTI) ) THEN

    ISTEP    = 0
    LLASTRAJ = .FALSE.
    CALL READ_TRAJECTORY(ISTEP,LLASTRAJ,LREADGPTRAJ)

    IF (LREADGPTRAJ) THEN

      ALLOCATE (ZGMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
      ALLOCATE (ZGMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
      ALLOCATE (ZGFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))

      ZGMV5=0._JPRB
      ZGMV5S=0._JPRB
      ZGFL5=0._JPRB

      CALL GET_TRAJ_GRID(ZGMV5,ZGMV5S,ZGFL5,KSTEP=0)

      DO JGFL=1,YGFL%NDIM5
        IF (YGFLC(JGFL)%LTRAJIO) THEN
          DO JSTGLO=1,NGPTOT,NPROMA
            ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
            IBL=(JSTGLO-1)/NPROMA+1
            GFL5(1:ICEND,1:NFLEVG,YGFLC(JGFL)%MP,IBL)=ZGFL5(1:ICEND,1:NFLEVG,YGFLC(JGFL)%MP,IBL)
            GFL5(ICEND+1:NPROMA,1:NFLEVG,YGFLC(JGFL)%MP,IBL)=0.0_JPRB
          ENDDO
        ENDIF
      ENDDO

      DEALLOCATE(ZGFL5)
      DEALLOCATE(ZGMV5)
      DEALLOCATE(ZGMV5S)

    ENDIF

  ENDIF
ENDIF

!*    Initialize Jb error covariance model
!*    Set up coefficients for humidity change of variable
IF (LBACKG .OR. LOBSC1) THEN
  WRITE(NULOUT,*) '---- Set up coefficients for humidity change of variable --',CLINE
  CALL SUJBCHVAR
ENDIF

IF (LBACKG.OR.LSPBSBAL) THEN
  IF (LSPBSBAL.OR..NOT.LJBENER) THEN
    WRITE(NULOUT,*) '---- Set up Jb balance operators --',CLINE
    IF(LELAM) THEN
      CALL SUEJBBAL('STABAL96  ',NJBVERB)
    ELSE
      CALL SUJBBAL('STABAL96  ')
    ENDIF
  ENDIF
ENDIF

IF (LBACKG) THEN
  WRITE(NULOUT,*) '---- Set up Jb error covariances --',CLINE
  IF(LELAM) THEN
    IF (LJBWAVELET) THEN
      WRITE(NULOUT,*) 'Wavelet Jb in Aladin: good luck!'
      CALL SUEJBWAVELET_BMATRIX
    ELSE
      CALL SUEJBCOV
    ENDIF
  ELSE
    IF (LJBWAVELET) THEN
      CALL SUJBWAVELET
    ELSE
      CALL SUJBCOV
    ENDIF
  ENDIF
  
  IF (LELAM) THEN
    IF (LEJK) THEN
      WRITE(NULOUT,*) '---- Set up Jk error covariances --',CLINE
      CALL SUELLJK(NULOUT)
      CALL SUEJKNORM
      WRITE(NULOUT,*) '---- Set up Jk modulation wrt level and param --',CLINE
      CALL SUEMODJK
    ENDIF
  ENDIF

  IF(.NOT.(ICONF == 8) .AND..NOT.(ICONF == 6)) THEN
    WRITE(NULOUT,*) '---- Set up Background fields --',CLINE
    CALL SUECGES

     IF(.NOT.LELAM .OR. .NOT.LSPFCE) THEN
       WRITE(NULOUT,*) '---- Set up Jb gridpoint background error stdev --',CLINE
       CALL SUINFCE
     ENDIF
  ENDIF

!*    Initialize Jq error covariances
  IF (LMODERR) THEN
    WRITE(NULOUT,*) '---- Set up Jq error covariances --',CLINE
    CALL SUJQ
  ENDIF

ENDIF

!*    Initialize scalar product
IF(.NOT.(NCONF == 1.OR.NCONF == 302.OR.ICONF == 2.OR.ICONF == 9.OR.ICONF == 7 )) THEN
  WRITE(NULOUT,*) '---- Set up scalar product --------',CLINE
  IF (LELAM) THEN
    CALL SUESCAL
  ELSE
    CALL SUSCAL
  ENDIF
ENDIF

!*    Initialize subspace
IF(.NOT.(ICONF == 0.OR.ICONF == 2.OR.NCONF == 302).AND.L_SUBSPACE_SVS) THEN
  WRITE(NULOUT,*) '---- Set up subspace for SV computation -------',CLINE
  CALL SU_SUBSPACE
ENDIF

!     Set up Simplified Kalman Filter
IF (LBACKG) THEN
  WRITE(NULOUT,*) '-- Set up Simplified Kalman Filter --',CLINE
  CALL SUSKF
ENDIF

!     Read in forecast errors
IF (LSCREEN) THEN
  WRITE(NULOUT,*) '---- Read in forecast errors ---',CLINE
  CALL INIFGER
ENDIF

!     Diagnose background error setup
!EA   IF (LBACKG .AND. (LAVCGL .OR. NBGVECS>0) ) THEN
!EA     WRITE(NULOUT,*) '-- Set up Jb : diagnose variances ---',CLINE
!EA     CALL BGVECS
!EA     IF (LWRISIGB .and. .not.LFDBOP)  CALL WRITESD (CFNSIGB,'eb')
!EA   ENDIF

!     Set up Stochastic Physics
IF (.NOT.LFPART2) THEN
  WRITE(NULOUT,'(A72)') '--- Set up stochastic physics, SPBS, CABS '//CLINE
  CALL INI_CUCONVCA
  CALL SURAND1
ENDIF
!     Set up spectral stochastic diabatic tendencies
IF (.NOT.LFPART2) THEN
  WRITE(NULOUT,'(A72)') '--- Set up stochastically perturbed parametrization tendencies '//CLINE
  WRITE(NULOUT,*)       '      SPPT a.k.a. stochastic physics with spectral pattern'
  CALL SUSPSDT
ENDIF
IF (LSPSDT.AND.LSTOPH) THEN
  CALL ABOR1('LSPSDT.AND.LSTOPH==T is not allowed!')
ENDIF

WRITE(NULOUT,*) '-------------------------------------',CLINE
WRITE(NULOUT,*) '------ END OF SETUPS at level 0 -----',CLINE
WRITE(NULOUT,*) '-------------------------------------',CLINE
WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) ' '
CALL FLUSH(NULOUT)
CALL GSTATS(39,1)

!*    Initialize Relaxation
WRITE(NULOUT,*) '------ Set up Relaxation ',CLINE
CALL SURLX(NULOUT)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU0YOMB',1,ZHOOK_HANDLE)
END SUBROUTINE SU0YOMB
