SUBROUTINE SU_SURF_FLDS

!**** *SU_SURF_FLDS*  - Initialize definition of surface fields

!     Purpose.
!     --------
!           Initialize definition of surface fields (physics fields)

!**   Interface.
!     ----------
!        *CALL* SU_SURF_FLDS

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------
!        MODULE SURFACE_FIELDS_MIX

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 01-Jul-2006
!          E. BAZILE : 26-Oct-2006 Corrections for LVGSN,LSOLV, etc ...
!          A. Alias  : 30-Nov-2006 Corrections to have several vertical layers in
!                                  deep SOIL (LTPROF)
!          R. El Khatib 09-Feb-2007 backfixing
!          A. Trojakova 27-Jun-2007 backfixing number of EXTRP
!          A. Alias  : 22-Mar-2007 Corrections Pour SURFEX 
!          JJMorcrette 20070223 DU, BC, OM, SU, VOL, SOaerosol climatological fields
!          J. Flemming   12-April-2008 GFL attribute extension for GEMS
!          Y. Seity : 27-Jan-2008 : modifications to run AROME without 
!                                   ALADIN-Isba fields LLASSIMAROME
!          R. El Khatib 17-Apr-2008 Fix a memory corruption
!          P. Lopez   : 26-Jun-2009 Added 100m wind components
!          P. Lopez   : 22-Oct-2008 Added ducting diagnostic fields
!          Y. Takaya : 29-Jan-2009  add ocean mixed layer fields
!          JJMorcrette: 20091201 Total and clear-sky direct SW radiation flux at surface 
!          H. Hersbach:04-Dec-2009 10-m neutral wind, friction velocity, log forecast z0h
!                                  Move Charnock from constant to time-dependent trajectory
!          S. Boussetta/G.Balsamo      May 2009   Added variable LAI field
!          R. Forbese  01-Mar-2010 Added TCRW,TCSW and added 2 to IVDIAG
!          JJMorcrette 20100212 PP of CBASE, 0DEGL and VISIH
!          T. Wilhelmsson : 25-Mar-2010 add 6 hourly min/max fields
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : N3DINI, LFPART2, NCONF, LCANARI, LFPOS, LAROME, LSCREEN, LIFSTRAJ, N6BINS
USE YOMLUN   , ONLY : NULOUT, NULNAM
USE YOM_YGFL, ONLY : YGHG, NGHG, YGRG, NGRG, LCO2SFC, LAEROSFC, LGRGSFC, YTRAC, LSF6SFC,&
 & NTRAC, LCH4SFC, LFIRE 
USE YOEWCOU  , ONLY : LWCOU  ,LWCOU2W, LWCUR
USE YOMFPC   , ONLY : NFPCLI
USE YOMGRB    ,ONLY : &
 & NGRBZ    ,NGRBTCW  ,NGRBTCWV ,NGRBSTL1 ,          &
 & NGRBSWVL1 ,NGRBSD   ,NGRBLSP  ,NGRBCP   ,NGRBSF   ,&
 & NGRBBLD  ,NGRBSSHF ,NGRBSLHF ,NGRBCHNK ,NGRBMSL  ,&
 & NGRBSDOR ,NGRBISOR ,NGRBANOR ,NGRBSLOR ,NGRBTCC  ,&
 & NGRB10U  ,NGRB10V  ,NGRB2T   ,NGRB2D   ,NGRBSSRD ,&
 & NGRBSTL2 ,NGRBSWVL2 ,NGRBLSM  ,NGRBSR   ,NGRBLIT, NGRBAL   ,&
 & NGRBSTRD ,NGRBSSR  ,NGRBSTR  ,NGRBTSR  ,NGRBTTR  ,&
 & NGRBEWSS ,NGRBNSSS ,NGRBE    ,NGRBSTL3 ,NGRBSWVL3 ,&
 & NGRBCCC  ,NGRBLCC  ,NGRBMCC  ,NGRBHCC  ,NGRBLGWS ,&
 & NGRBMGWS ,NGRBGWD  ,NGRBSRC  ,NGRBMX2T ,NGRBMN2T ,&
 & NGRBMX2T6,NGRBMN2T6,NGRBRO   ,NGRBSRO  ,NGRBSSRO ,&  
 & NGRBTCO3 ,NGRBIEWS ,NGRBINSS ,&
 & NGRBISHF ,NGRBIE   ,NGRBLSRH ,NGRBSKT  ,NGRBSP   ,&
 & NGRBSTL4 ,NGRBSWVL4 ,NGRBCSF  ,NGRBLSF  ,NGRBFAL  ,&
 & NGRBFSR  ,NGRBFLSR ,NGRBVIMD ,NGRBPARCS,NGRBTISR,&
 & NGRBCBASE,NGRB0DEGL,NGRBVISIH                    ,&
 & NGRBBLH  ,NGRBSUND ,NGRBPAR ,NGRBUVB ,NGRBCAPE ,NGRBFDIR, NGRBCDIR,&
 & NGRBCVL  ,NGRBCVH  ,NGRBTVL  ,NGRBTVH  ,&
 & NGRBLAIL ,NGRBLAIH ,NGRBSLT ,&
 & NGRBISTL1,NGRBISTL2,NGRBISTL3,NGRBISTL4,&
 & NGRBASN  ,NGRBRSN  ,NGRBTSN  ,&
 & NGRBCI   ,NGRBSSTK  ,NGRBTCLW ,NGRBTCIW ,NGRBTCRW ,NGRBTCSW,&
 & NGRBTSRC ,NGRBTTRC ,NGRBSSRC ,NGRBSTRC ,&
 & NGRBES   ,NGRBSMLT ,NGRB10FG ,NGRB10FG6,NGRBLSPF ,&   
 & NGRBALUVP,NGRBALUVD,NGRBALNIP,NGRBALNID,&
 & NGRBAERDEP,NGRBAERLTS,NGRBAERSCC       ,&
 & NGRBAERLS01,NGRBAERLS02,NGRBAERLS10,NGRBAERLS03,NGRBAERLS04,NGRBAERLS11,&
 & NGRBAERLS05,NGRBAERLS06,NGRBAERLS12,NGRBAERLS07,NGRBAERLS08,NGRBAERLS09 ,&
 & NGRBCO2NBF ,NGRBCO2OF ,NGRBCO2APF ,NGRBCO2FIRE ,&
 & NGRBSDFOR,NGRBSF6APF,NGRBCH4F,NGRBCH4FIRE,NGRBUCUR, NGRBVCUR,&
 & NGRB100U, NGRB100V ,NGRB10NU ,NGRB10NV ,NGRBZUST,&
 & NGRBDNDZN, NGRBDNDZA, NGRBDCTB, NGRBTPLB, NGRBTPLT, NGRBSSS, &
 & NGRBOCT  ,NGRBOCS  ,NGRBOCU  ,NGRBOCV  ,NGRBOCVVS,&        !KPP
 & NGRBOCVDF,NGRBOCDEP,NGRBOCLDP,NGRBOCLZ ,NGRBADVT ,&        !KPP
 & NGRBADVS ,NGRBOCUC ,NGRBOCVC ,NGRBUSTRC,NGRBVSTRC,& !KPP
 & NGRB117, NGRB118, NGRB119, NGRB120         
USE YOMDPHY  , ONLY : NVXP     ,NVXP2    ,NCXP     , &
 & NVEXTR   ,NVXTR2   ,NCEXTR   ,NCOM    ,LTPROF  ,LDIRCLSMOD !KPP 
USE TRAJECTORY_MOD, ONLY : NGP5, NTRAJ_CST
USE YOMPHY   , ONLY : LMPHYS   ,LNEIGE   ,LGWDC    ,LNEBCO   ,LAEROSEA ,&
 & LAEROLAN ,LAEROSOO ,LAERODES ,LAEROVOL,LAEROSUL ,LRELAXT  ,LRELAXW  ,&
 & LSNV     ,LSOLV    ,LFGEL    ,LNEBR   ,LVGSN    ,LO3ABC   ,CGMIXLEN ,&
 & LPHSPSH
USE YOMNUD   , ONLY : LNDRIV
USE YOMARPHY , ONLY : LMSE
USE YOMMCC   , ONLY : LMCC04
USE YOEPHY   , ONLY : LEPHYS, LE4ALB, LECURR, LEOCML, LELAIV, LESMOSS
USE YOMECEARTH, ONLY : LRDALB, LLANDICE
USE YOMGEMS  , ONLY : NCO2SFC
USE SURFACE_FIELDS_MIX, ONLY : JPMAXSFLDS, NOFFTRAJ, NOFFTRAJ_CST,NSTRAJGRIB,&
 & NPROGSURF, NPROGSURFL, NSURF, NSURFL, NDIMSURF, NDIMSURFL,&
 & INI_SFLP3, SETUP_SFLP3, INI_SFLP2, SETUP_SFLP2, &
 & YSP_SBD, YSP_SB, YSP_SGD, YSP_SG, &
 & YSP_RRD, YSP_RR, YSD_WSD, YSD_WS, &
 & YSD_WWD, YSD_WW, &
 & YSP_EPD, YSP_EP, YSP_X2D, YSP_X2, &
 & YSP_CID, YSP_CI, YSD_VFD, YSD_VF, &
 & YSD_VPD, YSD_VP, YSD_VVD, YSD_VV, &
 & YSD_VND, YSD_VN, YSD_VHD, YSD_VH, &
 & YSD_VAD, YSD_VA, YSD_VCD, YSD_VC, &
 & YSD_VDD, YSD_VD, YSD_VXD, YSD_VX, &
 & YSD_XAD, YSD_XA, YSD_X2D, YSD_X2, &
 & YSP_CLD, YSP_CL, YSP_OMD, YSP_OM, &                         !KPP
 & YSD_V2D, YSD_V2, YSD_V3D, YSD_V3                            !KPP
USE YOMPHYDS, ONLY : CEXTRP, NEXTRPGB, EXTRPDF, CXTRP2, NXTRP2GB, XTRP2DF,&
 & CVEXTRA, NVEXTRAGB, VEXTRADF, CVEXTR2, NVEXTR2GB, VEXTR2DF
USE YOPHLC   , ONLY : LSPHLC
USE YOMOBS   , ONLY : LVDFTRAJ


!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IVCLIN,IVO3ABC,IVCAN,IVWV,IVWW,IVSF,IVCLIH,IVDIAG,&
 & IVCLIX,IVCLIA,IVCLIP,IVCLIV,ICSS,IVSO,IVSG,IVRS,ICLS, ICOM,IVOM,IVO2,IVO3          !KPP  
INTEGER(KIND=JPIM) :: IGRBCODES(100)
INTEGER(KIND=JPIM) :: JGHG, JGRG, JTRAC,JFLD, JBIN
INTEGER(KIND=JPIM) :: IREQIN,IREQINDF,ITRAJ
INTEGER(KIND=JPIM) :: INOGRIB=-999
CHARACTER(LEN=16)  :: CLNAMES(100)
CHARACTER(LEN=16)  :: CLBLANK='                '
REAL(KIND=JPRB)    :: ZDEFAULT(100),ZTDFLT
LOGICAL            :: LL2D, LLNOSURF, LLCURRENT
LOGICAL            :: LLFP_CLASSIC, LLFP_SURFEX
LOGICAL            :: LL_CALL_VDIFLCZ ! VDIFLCZ is called under CPG_DYN

! * LLACTIVE_[X] for group VARSF (means: variable [X] is used and allocated):
LOGICAL            :: LLACTIVE_LSM
LOGICAL            :: LLACTIVE_Z0F
LOGICAL            :: LLACTIVE_ALBF
LOGICAL            :: LLACTIVE_EMISF
LOGICAL            :: LLACTIVE_GETRL
LOGICAL            :: LLACTIVE_VEG
LOGICAL            :: LLACTIVE_VRLAN
LOGICAL            :: LLACTIVE_VRLDI
LOGICAL            :: LLACTIVE_Z0RLF
LOGICAL            :: LLACTIVE_ALBSF

LOGICAL :: LL_FIRST_CALL=.TRUE.
LOGICAL :: LLASSIMAROME      ! T if LAROME except if AROME Assim (F)


REAL(KIND=JPRB)    :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"

!     ------------------------------------------------------------------

#include "namphyds.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU_SURF_FLDS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*      1.  SETUP SURFACE PROGNOSTIC VAR.
!           -----------------------------

LL2D = ANY(SPREAD(NCONF,1,6) ==(/201,202,421,422,521,522/))
LLNOSURF = LL2D .OR. ANY(SPREAD(NCONF,1,2) == (/911,912/))
LLASSIMAROME = LAROME.AND.(.NOT.(LSCREEN .OR. (NCONF == 131)))
LLCURRENT= LECMWF .AND. LECURR .AND. (.NOT. LMCC04)
NOFFTRAJ = 0
NOFFTRAJ_CST = 0
NPROGSURF = 0
NPROGSURFL = 0
NSURF = 0
NSURFL = 0
NDIMSURF = 0
NDIMSURFL = 0

IF(N3DINI == 1) THEN
  IREQINDF=-1
  ZTDFLT = 250._JPRB
ELSE
  IREQINDF=1
  ZTDFLT = 0._JPRB
ENDIF

! Defaults for extra fields

CEXTRP  (:,:) = CLBLANK
NEXTRPGB(:,:) = INOGRIB
EXTRPDF (:,:) = 0.0_JPRB

CXTRP2  (:)   = CLBLANK
NXTRP2GB(:)   = INOGRIB
XTRP2DF (:)   = 0.0_JPRB

CVEXTRA  (:)  = CLBLANK
NVEXTRAGB(:)  = INOGRIB
VEXTRADF (:)  = 0.0_JPRB

CVEXTR2  (:)  = CLBLANK
NVEXTR2GB(:)  = INOGRIB
VEXTR2DF (:)  = 0.0_JPRB


! This is a very strange use of extra fields /MH
IF (LCANARI) THEN
  CXTRP2(1) = 'SURFPREC.EAU.CON'
  CXTRP2(2) = 'SURFPREC.EAU.GEC'
  CXTRP2(3) = 'SURFPREC.NEI.CON'
  CXTRP2(4) = 'SURFPREC.NEI.GEC'
  IF(LSOLV) THEN
    CXTRP2(5) = 'ATMONEBUL.BASSE '
    CXTRP2(6) = 'SURFXFLU.MEVAP.E'
    CXTRP2(7) = 'SURFXEVAPOTRANSP'
  ENDIF
ENDIF

!    READ NAMELIST.

CALL POSNAM(NULNAM,'NAMPHYDS')
READ(NULNAM,NAMPHYDS)

!*      SOILB

IVSO = 0
ICSS = 0
IF( .NOT. LLNOSURF) THEN
  IF(LECMWF) THEN
    IVSO = 3
    ICSS = 4
  ELSE
    IF (LMPHYS.AND.(.NOT.LLASSIMAROME)) THEN
      IVSO = 2
      ICSS = 1
      IF (LTPROF) THEN
         ICSS=3
      ENDIF
      IF((LSOLV.AND.LFGEL).OR.LMSE) THEN
        IVSO = IVSO + 1
      ENDIF
    ENDIF
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSP_SB%YSB)
IF(ASSOCIATED(YSP_SB%YSB)) DEALLOCATE(YSP_SB%YSB)
ALLOCATE(YSP_SB%YSB(JPMAXSFLDS))
CALL INI_SFLP3(YSP_SBD,YSP_SB%YSB,IVSO,ICSS,.TRUE.,'SOILB - SP_SB   ')
YSP_SB%YT  => YSP_SB%YSB(JPMAXSFLDS)
YSP_SB%YQ  => YSP_SB%YSB(JPMAXSFLDS)
YSP_SB%YTL => YSP_SB%YSB(JPMAXSFLDS)

IF( .NOT. LLNOSURF) THEN
  IF(LECMWF) THEN
    IGRBCODES(1:ICSS) = (/NGRBSTL1,NGRBSTL2,NGRBSTL3,NGRBSTL4/)
    CLNAMES  (1:ICSS) = (/'LEV1TEMPERATURE','LEV2TEMPERATURE', &
     & 'LEV3TEMPERATURE','LEV4TEMPERATURE'/)
    ZDEFAULT(1:ICSS) = ZTDFLT
    YSP_SB%YT => YSP_SB%YSB(YSP_SBD%IPTR)
    CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YT,KGRIB=IGRBCODES(1:ICSS),&
     & CDNAME=CLNAMES(1:ICSS),PDEFAULT=ZDEFAULT(1:ICSS),KTRAJ=1,&
     & KREQIN=SPREAD(IREQINDF,1,4))
    IGRBCODES(1:ICSS) = (/NGRBSWVL1,NGRBSWVL2,NGRBSWVL3,NGRBSWVL4/)
    CLNAMES  (1:ICSS) = (/'LEV1RESERV.EAU  ','LEV2RESERV.EAU  ', &
     & 'LEV3RESERV.EAU  ','LEV4RESERV.EAU  '/)
    YSP_SB%YQ => YSP_SB%YSB(YSP_SBD%IPTR)
    CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YQ,KGRIB=IGRBCODES(1:ICSS),&
    & CDNAME=CLNAMES(1:ICSS),PDEFAULT=ZDEFAULT(1:ICSS),KTRAJ=1,&
    & KREQIN=SPREAD(IREQINDF,1,4))
    IGRBCODES(1:ICSS) = (/NGRBISTL1,NGRBISTL2,NGRBISTL3,NGRBISTL4/)
    CLNAMES  (1:ICSS) = (/'LEV1ICETEMP  ','LEV2ICETEMP  ', &
     & 'LEV3ICETEMP  ','LEV4ICETEMP  '/)
    YSP_SB%YTL => YSP_SB%YSB(YSP_SBD%IPTR)
    CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YTL,KGRIB=IGRBCODES(1:ICSS),&
     & CDNAME=CLNAMES(1:ICSS),PDEFAULT=ZDEFAULT(1:ICSS),&
     & KREQIN=SPREAD(IREQINDF,1,4))
  ELSE
    IF (LMPHYS) THEN
      IF(LSOLV.AND.LTPROF)THEN
        YSP_SB%YT => YSP_SB%YSB(YSP_SBD%IPTR)
        ZDEFAULT(1:ICSS) = ZTDFLT
        CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YT,KGRIB=(/NGRBSTL2,NGRBSTL3,NGRBSTL4/),&
       & CDNAME=(/'PROFTEMPERATURE ','LEV3TEMPERATURE ','LEV4TEMPERATURE '/),&
       & PDEFAULT=ZDEFAULT(1:ICSS),KTRAJ=0,KREQIN=SPREAD(1,1,3))
        YSP_SB%YQ => YSP_SB%YSB(YSP_SBD%IPTR)
        CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YQ,KGRIB=(/NGRBSWVL2,NGRBSWVL3,NGRBSWVL4/),&
         & CDNAME=(/'PROFRESERV.EAU  ','LEV3RESERV.EAU  ','LEV4RESERV.EAU  '/), &
         & PDEFAULT=ZDEFAULT(1:ICSS), KTRAJ=0,KREQIN=SPREAD(1,1,3))
        IF((LSOLV.AND.LFGEL).OR.LMSE) THEN
          YSP_SB%YTL => YSP_SB%YSB(YSP_SBD%IPTR)
          CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YTL, &
         & CDNAME=(/'PROFRESERV.GLACE','LEV3RESERV.GLACE','LEV4RESERV.GLACE'/),&
         & PDEFAULT=ZDEFAULT(1:ICSS), KTRAJ=0,KREQIN=SPREAD(1,1,3))
        ENDIF
      ELSEIF (.NOT.LLASSIMAROME) THEN
        YSP_SB%YT => YSP_SB%YSB(YSP_SBD%IPTR)
        ZDEFAULT(1:ICSS) = ZTDFLT
        CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YT,KGRIB=(/NGRBSTL2/),&
         & CDNAME=(/'PROFTEMPERATURE '/),PDEFAULT=ZDEFAULT(1:ICSS), KTRAJ=1)
        YSP_SB%YQ => YSP_SB%YSB(YSP_SBD%IPTR)
        CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YQ,KGRIB=(/NGRBSWVL2/),CDNAME=(/'PROFRESERV.EAU  '/),&
         & KTRAJ=1)
        IF((LSOLV.AND.LFGEL).OR.LMSE) THEN
          YSP_SB%YTL => YSP_SB%YSB(YSP_SBD%IPTR)
          CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YTL,CDNAME=(/'PROFRESERV.GLACE'/), KTRAJ=1)
        ENDIF
      ENDIF
    ENDIF
  ENDIF
ENDIF

! SNOWG
IVSG = 0
IF (LECMWF) THEN
  IVSG = 4
ELSE
  IF (LMPHYS.AND.(.NOT.LLASSIMAROME)) THEN
    IVSG = 1
    IF (LSOLV.AND.(LSNV.OR.LVGSN)) THEN
      IVSG=IVSG+2
      IF (LVGSN) IVSG=IVSG+1
    ENDIF
  ENDIF
ENDIF
IF (LL_FIRST_CALL) NULLIFY(YSP_SG%YSG)
IF(ASSOCIATED(YSP_SG%YSG)) DEALLOCATE(YSP_SG%YSG)
ALLOCATE(YSP_SG%YSG(JPMAXSFLDS))
CALL INI_SFLP2(YSP_SGD,YSP_SG%YSG,IVSG,.TRUE.,'SNOWG - SP_SG   ')
YSP_SG%YF => YSP_SG%YSG(JPMAXSFLDS)
YSP_SG%YA => YSP_SG%YSG(JPMAXSFLDS)
YSP_SG%YR => YSP_SG%YSG(JPMAXSFLDS)
YSP_SG%YT => YSP_SG%YSG(JPMAXSFLDS)

IF(LECMWF) THEN
  YSP_SG%YF => YSP_SG%YSG(YSP_SGD%IPTR)
  CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YF,PDEFAULT=ZTDFLT,&
   &KGRIB=NGRBSD ,CDNAME='SURFRESERV.NEIGE',KTRAJ=1,KREQIN=IREQINDF)
  YSP_SG%YA => YSP_SG%YSG(YSP_SGD%IPTR)
  CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YA,PDEFAULT=ZTDFLT,&
   & KGRIB=NGRBASN,CDNAME='SNOW ALBEDO     ',KTRAJ=1,KREQIN=IREQINDF)
  YSP_SG%YR => YSP_SG%YSG(YSP_SGD%IPTR)
  CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YR,PDEFAULT=ZTDFLT,&
   & KGRIB=NGRBRSN,CDNAME='SNOW DENSITY    ',KTRAJ=1,KREQIN=IREQINDF)
  YSP_SG%YT => YSP_SG%YSG(YSP_SGD%IPTR)
  CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YT,PDEFAULT=ZTDFLT,&
   & KGRIB=NGRBTSN,CDNAME='SNOW TEMPERATURE',KTRAJ=1,KREQIN=IREQINDF)
ELSE
  IF (LMPHYS.AND.(.NOT.LLASSIMAROME)) THEN
    YSP_SG%YF => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YF,KGRIB=NGRBSD ,CDNAME='SURFRESERV.NEIGE',KTRAJ=1,KREQIN=1)
    IF (LSOLV.AND.(LSNV.OR.LVGSN)) THEN
      YSP_SG%YA => YSP_SG%YSG(YSP_SGD%IPTR)
      CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YA,KGRIB=NGRBASN, CDNAME='SURFALBEDO NEIGE', KTRAJ=1,KREQIN=1)
      YSP_SG%YR => YSP_SG%YSG(YSP_SGD%IPTR)
      CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YR,KGRIB=NGRBRSN, CDNAME='SURFDENSIT.NEIGE',KTRAJ=1,KREQIN=1)
      IF (LVGSN) THEN
        YSP_SG%YT => YSP_SG%YSG(YSP_SGD%IPTR)
        CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YT,CDNAME='SURFALBEDO HISTO')
      ENDIF
    ENDIF
  ENDIF
ENDIF

! RESVR

IVRS = 0
IF(LECMWF) THEN
  IVRS = 2
ELSE
  IF (LMPHYS.AND.(.NOT.LLASSIMAROME)) THEN
    IVRS = 2
    IF (LSOLV) THEN
      IVRS=IVRS+1
    ENDIF
    IF ((LSOLV.AND.LFGEL).OR.LMSE) THEN
      IVRS=IVRS+1
    ENDIF
  ELSEIF (LLASSIMAROME) THEN
    IVRS=1
  ENDIF
ENDIF
IF(LFPART2.AND.(LMPHYS.OR.LEPHYS)) IVRS = IVRS+1

IF (LL_FIRST_CALL) NULLIFY(YSP_RR%YRR)
IF(ASSOCIATED(YSP_RR%YRR)) DEALLOCATE(YSP_RR%YRR)
ALLOCATE(YSP_RR%YRR(JPMAXSFLDS))
CALL INI_SFLP2(YSP_RRD,YSP_RR%YRR,IVRS,.TRUE.,'RESVR - SP_RR   ')
YSP_RR%YT => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YW => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YFC => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YIC => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YFP1 => YSP_RR%YRR(JPMAXSFLDS)

IF(LECMWF) THEN
  YSP_RR%YT => YSP_RR%YRR(YSP_RRD%IPTR)
  CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YT,PDEFAULT=ZTDFLT,&
   & KGRIB=NGRBSKT ,CDNAME='SKINTEMPERATURE ',KTRAJ=1,KREQIN=IREQINDF)
  YSP_RR%YW => YSP_RR%YRR(YSP_RRD%IPTR)
  CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YW,PDEFAULT=ZTDFLT,&
   & KGRIB=NGRBSRC ,CDNAME='SKINRESERV.EAU  ',KTRAJ=1,KREQIN=IREQINDF)
ELSE
  IF (LMPHYS) THEN
    YSP_RR%YT => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YT,KGRIB=NGRBSTL1,CDNAME='SURFTEMPERATURE ',KTRAJ=1, KREQIN=1)
    IF (.NOT.LLASSIMAROME) THEN
      YSP_RR%YW => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YW,KGRIB=NGRBSWVL1,CDNAME='SURFRESERV.EAU  ',KTRAJ=1, KREQIN=1)
      IF (LSOLV) THEN
        YSP_RR%YFC => YSP_RR%YRR(YSP_RRD%IPTR)
        CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YFC,KGRIB=NGRBSRC,CDNAME='SURFRESERV.INTERC', KTRAJ=1, KREQIN=1)
      ENDIF
      IF ((LSOLV.AND.LFGEL).OR.LMSE) THEN
        YSP_RR%YIC => YSP_RR%YRR(YSP_RRD%IPTR)
        CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YIC,CDNAME='SURFRESERV.GLACE', KTRAJ=1, KREQIN=1)
      ENDIF
    ENDIF
  ENDIF
ENDIF
IF(LFPART2.AND.(LMPHYS.OR.LEPHYS)) THEN
  YSP_RR%YFP1 => YSP_RR%YRR(YSP_RRD%IPTR)
  CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YFP1,KGRIB=NGRBSTL1 ,CDNAME='INTSURFTEMPERATURE')
ENDIF  

  
! CLS

ICLS=0
IF(LECMWF) THEN
  ICLS = 0
ELSE
  IF (LMPHYS.AND.LDIRCLSMOD) THEN
    ICLS = 2
  ELSE
    ICLS = 0
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSP_CL%YCL)
IF(ASSOCIATED(YSP_CL%YCL)) DEALLOCATE(YSP_CL%YCL)
ALLOCATE(YSP_CL%YCL(JPMAXSFLDS))
CALL INI_SFLP2(YSP_CLD,YSP_CL%YCL,ICLS,.TRUE.,'CLS   - SP_CL   ')
YSP_CL%YTCLS => YSP_CL%YCL(JPMAXSFLDS)
YSP_CL%YHUCLS => YSP_CL%YCL(JPMAXSFLDS)

IF(LECMWF) THEN
ELSE
  IF (LMPHYS.AND.LDIRCLSMOD) THEN
    YSP_CL%YTCLS => YSP_CL%YCL(YSP_CLD%IPTR)
    CALL SETUP_SFLP2(YSP_CLD,YSP_CL%YTCLS,KGRIB=NGRBSTL1,CDNAME='CLSTEMPERATURE  ',KTRAJ=1, KREQIN=1)
    YSP_CL%YHUCLS => YSP_CL%YCL(YSP_CLD%IPTR)
    CALL SETUP_SFLP2(YSP_CLD,YSP_CL%YHUCLS,KGRIB=NGRBSTL1,CDNAME='CLSHUMI.RELATIVE',KTRAJ=1, KREQIN=1)
  ENDIF
ENDIF


! OML (KPP)

IVOM = 0
ICOM = 0
IF(LECMWF) THEN
  IF(LEOCML) THEN
    IVOM = 4
    ICOM = NCOM !+1
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSP_OM%YOM)
IF(ASSOCIATED(YSP_OM%YOM)) DEALLOCATE(YSP_OM%YOM)
ALLOCATE(YSP_OM%YOM(JPMAXSFLDS))
CALL INI_SFLP3(YSP_OMD,YSP_OM%YOM,IVOM,ICOM,.TRUE.,'OML   - SP_OM   ')
YSP_OM%YTO  => YSP_OM%YOM(JPMAXSFLDS)
YSP_OM%YSO  => YSP_OM%YOM(JPMAXSFLDS)
YSP_OM%YUO  => YSP_OM%YOM(JPMAXSFLDS)
YSP_OM%YVO  => YSP_OM%YOM(JPMAXSFLDS)

IF(LECMWF) THEN
  IF(LEOCML) THEN
    ! For ocean mixed layer model, NREQIN=2 was introduced to read
    ! initial condition in SUGRIBO.F90, not in SUGRIBG.F90 
    ! NREQIN = -1 : not read I.C.
    !        =  1 : read ICGG????INIT file (KFILE=0)
    !        =  2 : read ICGG????INIOC file

    IGRBCODES(1:ICOM) = NGRBOCT  
    CLNAMES  (1:ICOM) = 'OCEAN TEMP     '
    YSP_OM%YTO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YTO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM)) 

    IGRBCODES(1:ICOM) = NGRBOCS  
    CLNAMES  (1:ICOM) = 'OCEAN SAL      '
    YSP_OM%YSO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YSO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))

    IGRBCODES(1:ICOM) = NGRBOCU  
    CLNAMES  (1:ICOM) = 'OCEAN VEL U    '
    YSP_OM%YUO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YUO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))

    IGRBCODES(1:ICOM) = NGRBOCV  
    CLNAMES  (1:ICOM) = 'OCEAN VEL V    '
    YSP_OM%YVO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YVO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))
  ENDIF
ENDIF


! EXTRP

IF (LL_FIRST_CALL) NULLIFY(YSP_EP%YEP)
IF(ASSOCIATED(YSP_EP%YEP)) DEALLOCATE(YSP_EP%YEP)
ALLOCATE(YSP_EP%YEP(JPMAXSFLDS))
CALL INI_SFLP3(YSP_EPD,YSP_EP%YEP,NVXP,NCXP,.TRUE.,'EXTRP - SP_EP   ')

IF (YSP_EPD%NUMFLDS > 0) THEN
  DO JFLD=1,YSP_EPD%NUMFLDS
    CALL SETUP_SFLP3(YSP_EPD,YSP_EP%YEP(JFLD),&
     & KGRIB=NEXTRPGB(1:YSP_EPD%NLEVS,JFLD),CDNAME=CEXTRP(1:YSP_EPD%NLEVS,JFLD),&
     & PDEFAULT=EXTRPDF(1:YSP_EPD%NLEVS,JFLD))
  ENDDO
ENDIF
    

! XTRP2

IF (LL_FIRST_CALL) NULLIFY(YSP_X2%YX2)
IF(ASSOCIATED(YSP_X2%YX2)) DEALLOCATE(YSP_X2%YX2)
ALLOCATE(YSP_X2%YX2(JPMAXSFLDS))
CALL INI_SFLP2(YSP_X2D,YSP_X2%YX2,NVXP2,.TRUE.,'XTRP2 - SP_X2   ')
IF (YSP_X2D%NUMFLDS > 0) THEN
  DO JFLD=1,YSP_X2D%NUMFLDS
    CALL SETUP_SFLP2(YSP_X2D,YSP_X2%YX2(JFLD),&
     &KGRIB=NXTRP2GB(JFLD),CDNAME=CXTRP2(JFLD),PDEFAULT=XTRP2DF(JFLD))
  ENDDO
ENDIF


! CANRI
IVCAN = 0

IF(LECMWF) THEN
ELSE
  IF(LCANARI) THEN
    IVCAN = 8
    IF(LSOLV) THEN
      IVCAN = IVCAN+5
    ENDIF
  ENDIF
ENDIF
    
IF (LL_FIRST_CALL) NULLIFY(YSP_CI%YCI)
IF(ASSOCIATED(YSP_CI%YCI)) DEALLOCATE(YSP_CI%YCI)
ALLOCATE(YSP_CI%YCI(JPMAXSFLDS))
CALL INI_SFLP2(YSP_CID,YSP_CI%YCI,IVCAN,.TRUE.,'CANRI - SP_CI   ')

IF(LECMWF) THEN
ELSE
  IF(LCANARI) THEN
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(1),CDNAME='SURFETP.GEOPOTEN')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(2),CDNAME='SURFETA.GEOPOTEN')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(3),CDNAME='SURFANALYSISMASK')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(4),CDNAME='CLSTEMPERATURE  ')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(5),CDNAME='CLSHUMI.RELATIVE')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(6),CDNAME='CLSVENT.ZONAL   ')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(7),CDNAME='CLSVENT.MERIDIEN')
    CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(8),CDNAME='SURFSST.CLIM.   ')
    IF (LSOLV) THEN
      CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(9) ,CDNAME='PROFINC.RESERV.1')
      CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(10),CDNAME='PROFINC.RESERV.2')
      CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(11),CDNAME='PROFINC.RESERV.3')
      CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(12),CDNAME='SURFINC.TEMPERAT')
      CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(13),CDNAME='SURFINC.HUMIDITE')
    ENDIF
  ENDIF
ENDIF

!-------------------------------------------------------------------------

!*      2.  SETUP SURFACE DIAGNOSTIC VAR.
!           -----------------------------

! VARSF

IVSF = 0
IF(LECMWF) THEN
!  IVSF = 19
!  IVSF = 21   
  IVSF=23
  IF (LCO2SFC) IVSF = IVSF+NCO2SFC
  IF (LCO2SFC .AND. LFIRE)   IVSF = IVSF+1
  IF (LCH4SFC)    IVSF = IVSF+1
  IF (LCH4SFC .AND. LFIRE)   IVSF = IVSF+1
  IF (LAEROSFC)IVSF = IVSF+12
  IF (LAEROSFC .AND. LFIRE)IVSF = IVSF+3
  IF (LGRGSFC) IVSF = IVSF+NGRG
               IVSF = IVSF+NTRAC
  IF(LE4ALB)   IVSF = IVSF+4
  IF(LSF6SFC)  IVSF = IVSF+1
  IF(LELAIV)   IVSF = IVSF+2
  IF(.not.LRDALB)   IVSF = IVSF+4
  IF(LESMOSS)  IVSF = IVSF+1
ELSE
  IVSF=0
  ! * Define LLACTIVE.. and increment IVSF for 'LSM' (land sea mask)
  LL_CALL_VDIFLCZ=(.NOT.(LMPHYS.OR.LEPHYS)).AND.LSPHLC
  LLACTIVE_LSM= LL_CALL_VDIFLCZ .OR.LMPHYS
  IF (LLACTIVE_LSM) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'Z0F'
  LLACTIVE_Z0F=LMPHYS.AND.(.NOT.(LMSE.OR.LLASSIMAROME))
  IF (LLACTIVE_Z0F) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'ALBF'
  LLACTIVE_ALBF=LMPHYS.AND.(.NOT.(LMSE.OR.LLASSIMAROME))
  IF (LLACTIVE_ALBF) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'EMISF'
  LLACTIVE_EMISF=LMPHYS.AND.(.NOT.(LMSE.OR.LLASSIMAROME))
  IF (LLACTIVE_EMISF) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'GETRL'
  LLACTIVE_GETRL=LMPHYS.AND.(.NOT.((LFPART2.AND.LMSE).OR.LLASSIMAROME))
  IF (LLACTIVE_GETRL) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'VEG'
  LLACTIVE_VEG=LMPHYS.AND.(.NOT.((LMSE.AND.LFPART2).OR.LLASSIMAROME))
  IF (LLACTIVE_VEG) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'VRLAN'
  LLACTIVE_VRLAN=LMPHYS.AND.(.NOT.((LFPART2.AND.LMSE).OR.LLASSIMAROME))
  IF (LLACTIVE_VRLAN) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'VRLDI'
  LLACTIVE_VRLDI=LMPHYS.AND.(.NOT.((LFPART2.AND.LMSE).OR.LLASSIMAROME))
  IF (LLACTIVE_VRLDI) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'Z0RLF'
  LLACTIVE_Z0RLF=LMPHYS.AND.(.NOT.LMSE).AND.LSOLV.AND.LSNV
  IF (LLACTIVE_Z0RLF) IVSF=IVSF+1
  ! * Define LLACTIVE.. and increment IVSF for 'ALBSF'
  LLACTIVE_ALBSF=LMPHYS.AND.(.NOT.LMSE).AND.LSOLV.AND.LSNV
  IF (LLACTIVE_ALBSF) IVSF=IVSF+1
ENDIF
IF (LFPART2) IVSF = IVSF+1

IF (LL_FIRST_CALL) NULLIFY(YSD_VF%YVF)
IF(ASSOCIATED(YSD_VF%YVF)) DEALLOCATE(YSD_VF%YVF)
ALLOCATE(YSD_VF%YVF(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VFD,YSD_VF%YVF,IVSF,.FALSE.,'VARSF - SD_VF   ')
YSD_VF%YZ0F   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALBF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YEMISF => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YGETRL => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLSM   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLIT   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVEG   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVRLAN => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVRLDI => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSIG   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALBSF => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCONT  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSST   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLZ0H  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCVL   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCVH   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YTVL   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YTVH   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLAIL  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLAIH  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOTY  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOALUVP  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOALUVD  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOALNIP  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOALNID  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCI    => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YUCUR  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVCUR  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YZ0RLF => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSDFOR => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALUVP => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALUVD => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALNIP => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALNID => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YAERDEP=> YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YAERLTS=> YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YAERSCC=> YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YBCBF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YBCFF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YBCGF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YOMBF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YOMFF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YOMGF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSO2L  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSO2H  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOGF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVOLC  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVOLE  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOA   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YGRGFLX  => YSD_VF%YVF(JPMAXSFLDS:JPMAXSFLDS)
YSD_VF%YTRACFLX  => YSD_VF%YVF(JPMAXSFLDS:JPMAXSFLDS)
YSD_VF%YCO2O  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCO2B  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCO2A  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCO2F  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCH4AG => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCH4F  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSF6   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YFP1   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSSS   => YSD_VF%YVF(JPMAXSFLDS)

IF(LECMWF) THEN
  YSD_VF%YZ0F => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YZ0F  ,PDEFAULT=0.01_JPRB,&
   & KGRIB=NGRBSR  ,CDNAME='SURFZ0.FOIS.G   ',KTRAJ=2,KREQIN=IREQINDF)
  YSD_VF%YALBF => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALBF ,PDEFAULT=0.5_JPRB,&
   & KGRIB=NGRBAL  ,CDNAME='SURFALBEDO      ',KTRAJ=2,KREQIN=IREQINDF)
  YSD_VF%YEMISF => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YEMISF,PDEFAULT=0.995_JPRB,&
   & CDNAME='SURFEMISSIVITE  ',KTRAJ=2,KREQIN=-1)
  YSD_VF%YGETRL => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YGETRL,&
   &KGRIB=NGRBSDOR,CDNAME='SURFET.GEOPOTENT',KTRAJ=2,KREQIN=1)
  YSD_VF%YLSM => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLSM  ,&
   & KGRIB=NGRBLSM ,CDNAME='SURFIND.TERREMER',KTRAJ=2,KREQIN=1)
  IF (LLANDICE) THEN
    YSD_VF%YLIT => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLIT  ,&
   & KGRIB=NGRBLIT ,CDNAME='ICESHEETMASK    ',KTRAJ=2,KREQIN=1)
  ENDIF
  YSD_VF%YVRLAN => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLAN,&
   & KGRIB=NGRBISOR,CDNAME='SURFVAR.GEOP.ANI',KTRAJ=2,KREQIN=1)
  YSD_VF%YVRLDI => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLDI,&
   & KGRIB=NGRBANOR,CDNAME='SURFVAR.GEOP.DIR',KTRAJ=2,KREQIN=1)
  YSD_VF%YSIG => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSIG  ,&
   & KGRIB=NGRBSLOR,CDNAME='SURFMEAN SLOPE  ',KTRAJ=2,KREQIN=1)
  YSD_VF%YLZ0H => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLZ0H ,&
   & KGRIB=NGRBLSRH,CDNAME='SURFLOGZ0H      ',KTRAJ=2,KREQIN=1)
  YSD_VF%YCVL => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCVL  ,&
   & KGRIB=NGRBCVL ,CDNAME='LOW VEG. COVER  ',KTRAJ=2,KREQIN=1)
  YSD_VF%YCVH => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCVH  ,PDEFAULT=9.21_JPRB,&
   & KGRIB=NGRBCVH ,CDNAME='HIGH VEG. COVER ',KTRAJ=2,KREQIN=IREQINDF)
  YSD_VF%YTVL => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YTVL  ,&
   & KGRIB=NGRBTVL ,CDNAME='LOW VEG. TYPE   ',KTRAJ=2,KREQIN=1)
  YSD_VF%YTVH => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YTVH  ,&
   & KGRIB=NGRBTVH ,CDNAME='HIGH VEG. TYPE  ',KTRAJ=2,KREQIN=1)

 IF (LELAIV) THEN
  YSD_VF%YLAIL => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLAIL  ,&
   & KGRIB=NGRBLAIL ,CDNAME='LOW VEG. LAI    ',KTRAJ=2,KREQIN=1)
  YSD_VF%YLAIH => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLAIH  ,&
   & KGRIB=NGRBLAIH ,CDNAME='HIGH VEG. LAI   ',KTRAJ=2,KREQIN=1)
  ENDIF

 IF (.not.LRDALB) THEN
  YSD_VF%YSOALUVP => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOALUVP ,&
   & KGRIB=NGRB117 ,CDNAME='SOIL ALBEDO UVP ',KTRAJ=2,KREQIN=1)
  YSD_VF%YSOALUVD => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOALUVD ,&
   & KGRIB=NGRB118 ,CDNAME='SOIL ALBEDO UVD ',KTRAJ=2,KREQIN=1)
  YSD_VF%YSOALNIP => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOALNIP ,&
   & KGRIB=NGRB119 ,CDNAME='SOIL ALBEDO NIP ',KTRAJ=2,KREQIN=1)
  YSD_VF%YSOALNID => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOALNID ,&
   & KGRIB=NGRB120 ,CDNAME='SOIL ALBEDO NID ',KTRAJ=2,KREQIN=1)
 ENDIF

  YSD_VF%YSOTY => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOTY ,&
   & KGRIB=NGRBSLT ,CDNAME='SOIL TYPE       ',KTRAJ=2,KREQIN=1)
  YSD_VF%YCI => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCI   ,&
   & KGRIB=NGRBCI  ,CDNAME='SEA ICE COVER   ',KTRAJ=2,KREQIN=1)
  YSD_VF%YSST => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSST  ,PDEFAULT=295.0_JPRB,&
   & KGRIB=NGRBSSTK ,CDNAME='SEA SURFACE TEMP',KTRAJ=2,KREQIN=IREQINDF)
  IF (LLCURRENT) THEN
     IREQIN=1
  ELSE
     IREQIN=-1
  ENDIF
  YSD_VF%YUCUR => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YUCUR ,PDEFAULT=0.0_JPRB,&
   & KGRIB=NGRBUCUR,CDNAME='UCOM OCEAN CURRENT',KTRAJ=2,KREQIN=IREQIN)
  YSD_VF%YVCUR => YSD_VF%YVF(YSD_VFD%IPTR)
     CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVCUR ,PDEFAULT=0.0_JPRB,&
   & KGRIB=NGRBVCUR,CDNAME='VCOM OCEAN CURRENT',KTRAJ=2,KREQIN=IREQIN)
  YSD_VF%YSDFOR => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSDFOR,&
   & KGRIB=NGRBSDFOR,CDNAME='SD FILTRED OROG ',KTRAJ=2,KREQIN=1)
  IF (LE4ALB) THEN
    YSD_VF%YALUVP => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALUVP,&
     & KGRIB=NGRBALUVP,CDNAME='M_ALB.UVis PARAL',KTRAJ=2,KREQIN=1)
    YSD_VF%YALUVD => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALUVD,&
     & KGRIB=NGRBALUVD,CDNAME='M_ALB.UVis DIFFU',KTRAJ=2,KREQIN=1)
    YSD_VF%YALNIP => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALNIP,&
     & KGRIB=NGRBALNIP,CDNAME='M_ALB.N-IR PARAL',KTRAJ=2,KREQIN=1)
    YSD_VF%YALNID => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALNID,&
     & KGRIB=NGRBALNID,CDNAME='M_ALB.N-IR DIFFU',KTRAJ=2,KREQIN=1)
  ENDIF  
  IF (LAEROSFC) THEN
    YSD_VF%YAERDEP => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YAERDEP,&
     & KGRIB=NGRBAERDEP,CDNAME='Dust emiss poten',KTRAJ=2,KREQIN=1)
    YSD_VF%YAERLTS => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YAERLTS,&
     & KGRIB=NGRBAERLTS,CDNAME='Lift thresh sped',KTRAJ=2,KREQIN=1)
    YSD_VF%YAERSCC => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YAERSCC,&
     & KGRIB=NGRBAERSCC,CDNAME='Soil clay contnt',KTRAJ=2,KREQIN=1)

    YSD_VF%YBCBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YBCBF,&
     & KGRIB=NGRBAERLS01,CDNAME='BlckCarbn biogen',KTRAJ=2,KREQIN=1)
    YSD_VF%YBCFF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YBCFF,&
     & KGRIB=NGRBAERLS02,CDNAME='BlckCarbn fossil',KTRAJ=2,KREQIN=1)
    IF (LFIRE) THEN
      YSD_VF%YBCGF => YSD_VF%YVF(YSD_VFD%IPTR)
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YBCGF,&
       & KGRIB=NGRBAERLS10,CDNAME='BlackCarbon GFED',KTRAJ=2,KREQIN=1)
    ENDIF
    YSD_VF%YOMBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YOMBF,&
     & KGRIB=NGRBAERLS03,CDNAME='Organic biogenic',KTRAJ=2,KREQIN=1)
    YSD_VF%YOMFF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YOMFF,&
     & KGRIB=NGRBAERLS04,CDNAME='Organic fossil  ',KTRAJ=2,KREQIN=1)
    IF (LFIRE) THEN
      YSD_VF%YOMGF => YSD_VF%YVF(YSD_VFD%IPTR)
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YOMGF,&
       & KGRIB=NGRBAERLS11,CDNAME='Organic GFED    ',KTRAJ=2,KREQIN=1)
    ENDIF
    YSD_VF%YSO2L => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSO2L,&
     & KGRIB=NGRBAERLS05,CDNAME='Sulphate low-lvl',KTRAJ=2,KREQIN=1)
    YSD_VF%YSO2H => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSO2H,&
     & KGRIB=NGRBAERLS06,CDNAME='Sulphate highlvl',KTRAJ=2,KREQIN=1)
    IF (LFIRE) THEN
      YSD_VF%YSOGF => YSD_VF%YVF(YSD_VFD%IPTR)
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOGF,&
       & KGRIB=NGRBAERLS12,CDNAME='Sulphate GFED   ',KTRAJ=2,KREQIN=1)
    ENDIF
    YSD_VF%YVOLC => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVOLC,&
     & KGRIB=NGRBAERLS07,CDNAME='Volcanic continu',KTRAJ=2,KREQIN=1)
    YSD_VF%YVOLE => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVOLE,&
     & KGRIB=NGRBAERLS08,CDNAME='Volcanic explosi',KTRAJ=2,KREQIN=1)
    YSD_VF%YSOA => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOA,&
     & KGRIB=NGRBAERLS09,CDNAME='SecondOrganicMatt',KTRAJ=2,KREQIN=1)
  ENDIF

  IF (LGRGSFC) THEN 
    IF(NGRG > 0) THEN
    YSD_VF%YGRGFLX => YSD_VF%YVF(YSD_VFD%IPTR:YSD_VFD%IPTR+NGRG-1)
    DO JGRG=1,NGRG
    IREQIN=1
    IF (YGRG(JGRG)%IGRIBSFC < 0 .OR. YGRG(JGRG)%NCOUPLO4 > 0 ) IREQIN=0
!nl ext
        CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YGRGFLX(JGRG),KGRIB=YGRG(JGRG)%IGRIBSFC, &
    &    CDNAME=TRIM(YGRG(JGRG)%CNAME)//'  SURFACE FLX ', KTRAJ=2,KREQIN=IREQIN) 
     ENDDO
   ENDIF
  ENDIF

  IF(NTRAC > 0) THEN
    YSD_VF%YTRACFLX => YSD_VF%YVF(YSD_VFD%IPTR:YSD_VFD%IPTR+NTRAC-1)
    DO JTRAC=1,NTRAC
    IREQIN=1
    IF (YTRAC(JTRAC)%IGRIBSFC < 0 ) IREQIN=0
!nl ext
        CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YTRACFLX(JTRAC),KGRIB=YTRAC(JTRAC)%IGRIBSFC, &
    &    CDNAME=TRIM(YTRAC(JTRAC)%CNAME)//'  SURFACE FLX ', KTRAJ=2,KREQIN=IREQIN) 
     ENDDO
   ENDIF

  IF (LCO2SFC) THEN
    YSD_VF%YCO2O => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2O ,&
     & KGRIB=NGRBCO2OF ,CDNAME='CO2 SURF - OCEAN',KTRAJ=2,KREQIN=1)
    YSD_VF%YCO2B => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2B ,&
     & KGRIB=NGRBCO2NBF ,CDNAME='CO2 SURF - BIOSF',KTRAJ=2,KREQIN=1)
    YSD_VF%YCO2A => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2A ,&
     & KGRIB=NGRBCO2APF ,CDNAME='CO2 SURF - ANTH ',KTRAJ=2,KREQIN=1)
    IF (LFIRE) THEN
      YSD_VF%YCO2F => YSD_VF%YVF(YSD_VFD%IPTR)
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2F ,&
       & KGRIB=NGRBCO2FIRE ,CDNAME='CO2 FIRE EMISS. ',KTRAJ=2,KREQIN=1)
    ENDIF
  ENDIF
  IF(LCH4SFC) THEN
    YSD_VF%YCH4AG => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCH4AG ,&
     & KGRIB=NGRBCH4F ,CDNAME='CH4 SURF. FLUXES',KTRAJ=2,KREQIN=1)
    IF (LFIRE) THEN
      YSD_VF%YCH4F  => YSD_VF%YVF(YSD_VFD%IPTR)
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCH4F ,&
       & KGRIB=NGRBCH4FIRE ,CDNAME='CH4 FIRE EMISS.  ',KTRAJ=2,KREQIN=1)
    ENDIF
  ENDIF
  
  IF(LSF6SFC) THEN
    YSD_VF%YSF6 => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSF6 ,&
     & KGRIB=NGRBSF6APF ,CDNAME='SF6 SURF - ANTH ',KTRAJ=2,KREQIN=1)
  ENDIF

  IF(LESMOSS) THEN
    YSD_VF%YSSS => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSSS ,KGRIB=NGRBSSS ,CDNAME='SEA SFC SALINITY',KREQIN=1)
  ENDIF

ELSE
  IF (LLACTIVE_LSM) THEN
    YSD_VF%YLSM => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLSM  ,KGRIB=NGRBLSM ,CDNAME='SURFIND.TERREMER',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_Z0F) THEN
    YSD_VF%YZ0F => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YZ0F  ,KGRIB=NGRBSR  ,CDNAME='SURFZ0.FOIS.G   ',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_ALBF) THEN
    YSD_VF%YALBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALBF ,KGRIB=NGRBAL  ,CDNAME='SURFALBEDO      ',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_EMISF) THEN
    YSD_VF%YEMISF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YEMISF,17      ,CDNAME='SURFEMISSIVITE  ',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_GETRL) THEN
    YSD_VF%YGETRL => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YGETRL,KGRIB=NGRBSDOR,CDNAME='SURFET.GEOPOTENT',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_VEG) THEN
    YSD_VF%YVEG => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVEG  ,CDNAME='SURFPROP.VEGETAT',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_VRLAN) THEN
    YSD_VF%YVRLAN => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLAN  ,CDNAME='SURFVAR.GEOP.ANI',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_VRLDI) THEN
    YSD_VF%YVRLDI => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLDI  ,CDNAME='SURFVAR.GEOP.DIR',KTRAJ=2)
  ENDIF
  IF (LLACTIVE_Z0RLF) THEN
    YSD_VF%YZ0RLF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YZ0RLF  ,CDNAME='SURFZ0REL.FOIS.G')
  ENDIF
  IF (LLACTIVE_ALBSF) THEN
    YSD_VF%YALBSF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALBSF  ,CDNAME='SURFALBEDO.COMPL')
  ENDIF
ENDIF
IF(LFPART2) THEN
  YSD_VF%YFP1 => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YFP1,KGRIB=NGRBZ,CDNAME='SURFGEOPOTENTIEL',KTRAJ=0)
ENDIF

! VCLIP
IVCLIP = 0
IF(LECMWF) THEN
ELSE
  IF (LRELAXT) IVCLIP=IVCLIP+1
  IF (LRELAXW) IVCLIP=IVCLIP+1
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VP%YVP)
IF(ASSOCIATED(YSD_VP%YVP)) DEALLOCATE(YSD_VP%YVP)
ALLOCATE(YSD_VP%YVP(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VPD,YSD_VP%YVP,IVCLIP,.FALSE.,'VCLIP - SD_VP   ')
YSD_VP%YTPC => YSD_VP%YVP(JPMAXSFLDS)
YSD_VP%YWPC => YSD_VP%YVP(JPMAXSFLDS)

IF(LECMWF) THEN
ELSE
  IF(LRELAXT) THEN
    YSD_VP%YTPC => YSD_VP%YVP(YSD_VPD%IPTR)
    CALL SETUP_SFLP2(YSD_VPD,YSD_VP%YTPC,CDNAME='RELATEMPERATURE ')
  ENDIF
  IF(LRELAXW) THEN
    YSD_VP%YWPC => YSD_VP%YVP(YSD_VPD%IPTR)
    CALL SETUP_SFLP2(YSD_VPD,YSD_VP%YWPC,CDNAME='RELARESERV.EAU  ')
  ENDIF
ENDIF

! VCLIV

IVCLIV = 0
IF(LECMWF) THEN
ELSE
  IF (LMPHYS.AND.(LSOLV.OR.LMSE).AND.(.NOT.LLASSIMAROME)) THEN
    IVCLIV=3
    IF (LSOLV) THEN
      IVCLIV=IVCLIV+5
      IF (LVGSN) THEN
        IVCLIV=IVCLIV+2
      ENDIF
    ENDIF
  ELSE
    IVCLIV=0
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VV%YVV)
IF(ASSOCIATED(YSD_VV%YVV)) DEALLOCATE(YSD_VV%YVV)
ALLOCATE(YSD_VV%YVV(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VVD,YSD_VV%YVV,IVCLIV,.FALSE.,'VCLIV - SD_VV   ')
YSD_VV%YARG   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YSAB   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YD2    => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YIVEG  => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YRSMIN => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YLAI   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YHV    => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YZ0H   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YALS   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YALV   => YSD_VV%YVV(JPMAXSFLDS)

IF(LECMWF) THEN
ELSE
  IF (LMPHYS.AND.(LSOLV.OR.LMSE).AND.(.NOT.LLASSIMAROME)) THEN
    YSD_VV%YARG   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YARG,CDNAME='SURFPROP.ARGILE ',KTRAJ=2)
    YSD_VV%YSAB   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YSAB,CDNAME='SURFPROP.SABLE  ',KTRAJ=2)
    YSD_VV%YD2    => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YD2,CDNAME='SURFEPAIS.SOL   ')
    IF (LSOLV) THEN
      YSD_VV%YIVEG  => YSD_VV%YVV(YSD_VVD%IPTR)
      CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YIVEG,CDNAME='SURFIND.VEG.DOMI')
      YSD_VV%YRSMIN => YSD_VV%YVV(YSD_VVD%IPTR)
      CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YRSMIN,CDNAME='SURFRESI.STO.MIN')
      YSD_VV%YLAI   => YSD_VV%YVV(YSD_VVD%IPTR)
      CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YLAI,CDNAME='SURFIND.FOLIAIRE')
      YSD_VV%YHV    => YSD_VV%YVV(YSD_VVD%IPTR)
      CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YHV,CDNAME='SURFRES.EVAPOTRA',KTRAJ=2)
      YSD_VV%YZ0H   => YSD_VV%YVV(YSD_VVD%IPTR)
      CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YZ0H,CDNAME='SURFGZ0.THERM   ',KTRAJ=2)
      IF (LVGSN) THEN
        YSD_VV%YALS   => YSD_VV%YVV(YSD_VVD%IPTR)
        CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YALS,CDNAME='SURFALBEDO.SOLNU')
        YSD_VV%YALV   => YSD_VV%YVV(YSD_VVD%IPTR)
        CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YALV,CDNAME='SURFALBEDO.VEG  ')
      ENDIF
    ENDIF
  ENDIF
ENDIF

! VCLIN

IVCLIN = 0
IF(LECMWF) THEN
  IVCLIN = 3
ELSE
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VN%YVN)
IF(ASSOCIATED(YSD_VN%YVN)) DEALLOCATE(YSD_VN%YVN)
ALLOCATE(YSD_VN%YVN(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VND,YSD_VN%YVN,IVCLIN,.FALSE.,'VCLIN - SD_VN   ')
YSD_VN%YTOP  => YSD_VN%YVN(JPMAXSFLDS)
YSD_VN%YBAS  => YSD_VN%YVN(JPMAXSFLDS)
YSD_VN%YACPR => YSD_VN%YVN(JPMAXSFLDS)

IF(LECMWF) THEN
  YSD_VN%YTOP => YSD_VN%YVN(YSD_VND%IPTR)
  CALL SETUP_SFLP2(YSD_VND,YSD_VN%YTOP)
  YSD_VN%YBAS => YSD_VN%YVN(YSD_VND%IPTR)
  CALL SETUP_SFLP2(YSD_VND,YSD_VN%YBAS)
  YSD_VN%YACPR => YSD_VN%YVN(YSD_VND%IPTR)
  CALL SETUP_SFLP2(YSD_VND,YSD_VN%YACPR)
ELSE
ENDIF

! VCLIH
IVCLIH = 0
IF(LECMWF) THEN
ELSE
  IF(LMPHYS) THEN
    IF(LNEBCO.OR.LGWDC)  IVCLIH=IVCLIH+3 ! PTCCH,PSCCH,PBCCH
    IF(LNEBR.OR.(TRIM(CGMIXLEN) == 'TM'))   IVCLIH=IVCLIH+1 ! PPBLH
    IF(LPHSPSH) IVCLIH=IVCLIH+1 ! PSPSH
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VH%YVH)
IF(ASSOCIATED(YSD_VH%YVH)) DEALLOCATE(YSD_VH%YVH)
ALLOCATE(YSD_VH%YVH(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VHD,YSD_VH%YVH,IVCLIH,.FALSE.,'VCLIH - SD_VH   ')
YSD_VH%YTCCH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YSCCH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YBCCH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YPBLH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YSPSH => YSD_VH%YVH(JPMAXSFLDS)

IF(LECMWF) THEN
ELSE
  IF(LMPHYS) THEN
    IF(LNEBCO.OR.LGWDC) THEN
      YSD_VH%YTCCH => YSD_VH%YVH(YSD_VHD%IPTR)
      CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YTCCH)
      YSD_VH%YSCCH => YSD_VH%YVH(YSD_VHD%IPTR)
      CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YSCCH)
      YSD_VH%YBCCH => YSD_VH%YVH(YSD_VHD%IPTR)
      CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YBCCH)
    ENDIF
    IF(LNEBR.OR.(TRIM(CGMIXLEN) == 'TM')) THEN
      YSD_VH%YPBLH => YSD_VH%YVH(YSD_VHD%IPTR)
      CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YPBLH)
    ENDIF
    IF(LPHSPSH) THEN
      YSD_VH%YSPSH => YSD_VH%YVH(YSD_VHD%IPTR)
      CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YSPSH)
    ENDIF
  ENDIF
ENDIF

! VCLIA

IVCLIA = 0
IF(LECMWF) THEN
ELSE
  IF (LMPHYS) THEN
    IF (LAEROSEA) IVCLIA=IVCLIA+1
    IF (LAEROLAN) IVCLIA=IVCLIA+1
    IF (LAEROSOO) IVCLIA=IVCLIA+1
    IF (LAERODES) IVCLIA=IVCLIA+1
    IF (LAEROSUL) IVCLIA=IVCLIA+1
    IF (LAEROVOL) IVCLIA=IVCLIA+1
    IF (LNDRIV)   IVCLIA=IVCLIA+1
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VA%YVA)
IF(ASSOCIATED(YSD_VA%YVA)) DEALLOCATE(YSD_VA%YVA)
ALLOCATE(YSD_VA%YVA(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VAD,YSD_VA%YVA,IVCLIA,.FALSE.,'VCLIA - SD_VA   ')
YSD_VA%YSEA => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YLAN => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YSOO => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YDES => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YSUL => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YVOL => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YNUD => YSD_VA%YVA(JPMAXSFLDS)

IF(LECMWF) THEN
ELSE
  IF (LAEROSEA) THEN
    YSD_VA%YSEA => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YSEA,CDNAME='SURFAEROS.SEA   ')
  ENDIF
  IF (LAEROLAN) THEN
    YSD_VA%YLAN => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YLAN,CDNAME='SURFAEROS.LAND  ')
  ENDIF
  IF (LAEROSOO) THEN
    YSD_VA%YSOO => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YSOO,CDNAME='SURFAEROS.SOOT  ')
  ENDIF
  IF (LAERODES) THEN
    YSD_VA%YDES => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YDES,CDNAME='SURFAEROS.DESERT')
  ENDIF
  IF (LAEROSUL) THEN
    YSD_VA%YSUL => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YSUL,CDNAME='SURFAEROS.SULFAT')
  ENDIF
  IF (LAEROVOL) THEN
    YSD_VA%YVOL => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YVOL,CDNAME='SURFAEROS.VOLCAN')
  ENDIF
  IF (LNDRIV) THEN
    YSD_VA%YNUD => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YVOL,CDNAME='SURFAEROS.NUDG  ')
  ENDIF
ENDIF

! VO3ABC

IVO3ABC = 0
IF(LECMWF) THEN
ELSE
  IF (LMPHYS) THEN
    IF (LO3ABC) THEN
      IVO3ABC=3
    ENDIF
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VC%YVC)
IF(ASSOCIATED(YSD_VC%YVC)) DEALLOCATE(YSD_VC%YVC)
ALLOCATE(YSD_VC%YVC(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VCD,YSD_VC%YVC,IVO3ABC,.FALSE.,'VO3ABC - SD_VC  ')
YSD_VC%YA => YSD_VC%YVC(JPMAXSFLDS)
YSD_VC%YB => YSD_VC%YVC(JPMAXSFLDS)
YSD_VC%YC => YSD_VC%YVC(JPMAXSFLDS)

IF(LECMWF) THEN
ELSE
  IF (LMPHYS) THEN
    IF (LO3ABC) THEN
      YSD_VC%YA => YSD_VC%YVC(YSD_VCD%IPTR)
      CALL SETUP_SFLP2(YSD_VCD,YSD_VC%YA,CDNAME='SURFA.OF.OZONE  ')
      YSD_VC%YB => YSD_VC%YVC(YSD_VCD%IPTR)
      CALL SETUP_SFLP2(YSD_VCD,YSD_VC%YB,CDNAME='SURFB.OF.OZONE  ')
      YSD_VC%YC => YSD_VC%YVC(YSD_VCD%IPTR)
      CALL SETUP_SFLP2(YSD_VCD,YSD_VC%YC,CDNAME='SURFC.OF.OZONE  ')
    ENDIF
  ENDIF
ENDIF


! VDIAGO2 (KPP)

IVO2 = 0
IF(LECMWF) THEN
  IF (LEOCML) THEN
    IVO2 = 3
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_V2%YV2)
IF(ASSOCIATED(YSD_V2%YV2)) DEALLOCATE(YSD_V2%YV2)
ALLOCATE(YSD_V2%YV2(JPMAXSFLDS))
CALL INI_SFLP2(YSD_V2D,YSD_V2%YV2,IVO2,.FALSE.,'VDIAGO2 - SD_V2 ')
YSD_V2%YOCDEP => YSD_V2%YV2(JPMAXSFLDS)
YSD_V2%YUSTRC => YSD_V2%YV2(JPMAXSFLDS)
YSD_V2%YVSTRC => YSD_V2%YV2(JPMAXSFLDS)

IF(LECMWF) THEN
  IF (LEOCML) THEN
    YSD_V2%YOCDEP => YSD_V2%YV2(YSD_V2D%IPTR)
    CALL SETUP_SFLP2(YSD_V2D,YSD_V2%YOCDEP,KGRIB=NGRBOCDEP,&
                    &CDNAME='OCEAN DEP       ',KREQIN=2 )
    YSD_V2%YUSTRC => YSD_V2%YV2(YSD_V2D%IPTR)
    CALL SETUP_SFLP2(YSD_V2D,YSD_V2%YUSTRC,KGRIB=NGRBUSTRC,&
                    &CDNAME='CLIM TAUX       ',KREQIN=2 )
    YSD_V2%YVSTRC => YSD_V2%YV2(YSD_V2D%IPTR)
    CALL SETUP_SFLP2(YSD_V2D,YSD_V2%YVSTRC,KGRIB=NGRBVSTRC,&
                    &CDNAME='CLIM TAUY       ',KREQIN=2 )
  ENDIF
ENDIF


! VDIAGO3 (KPP)

! OML (KPP)

IVO3 = 0
ICOM = 0
IF( .NOT. LLNOSURF) THEN
  IF(LECMWF) THEN
    IF(LEOCML) THEN
      IVO3 = 14
      ICOM = NCOM !+1
    ENDIF
  ENDIF
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_V3%YV3)
IF(ASSOCIATED(YSD_V3%YV3)) DEALLOCATE(YSD_V3%YV3)
ALLOCATE(YSD_V3%YV3(JPMAXSFLDS))
CALL INI_SFLP3(YSD_V3D,YSD_V3%YV3,IVO3,ICOM,.FALSE.,'VDIAGO3 - SD_V3 ')
YSD_V3%YDIFM   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YDIFT   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YDIFS   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YADVT   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YADVS   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YTRI0   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YTRI1   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YSWDK   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YZO     => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YHO     => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YDO     => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YHO_INV => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YUOC    => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YVOC    => YSD_V3%YV3(JPMAXSFLDS)

IF(LECMWF) THEN
  IF(LEOCML) THEN
    IGRBCODES(1:ICOM) = NGRBOCVVS
    CLNAMES  (1:ICOM) = 'OCEAN VISC     '
    YSD_V3%YDIFM => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDIFM,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))

    IGRBCODES(1:ICOM) = NGRBOCVDF
    CLNAMES  (1:ICOM) = 'OCEAN DIFF     '
    YSD_V3%YDIFT => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDIFT,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))

    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YDIFS => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDIFT,KGRIB=IGRBCODES(1:ICOM))

    IGRBCODES(1:ICOM) = NGRBOCLZ
    CLNAMES  (1:ICOM) = 'OCEAN LAYER DEP'  
    YSD_V3%YZO => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YZO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))

    IGRBCODES(1:ICOM) = NGRBOCLDP
    CLNAMES  (1:ICOM) = 'OCEAN THICKNESS'  
    YSD_V3%YDO => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))

    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YHO => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YHO,KGRIB=IGRBCODES(1:ICOM))

    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YHO_INV => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YHO_INV,KGRIB=IGRBCODES(1:ICOM))

    IGRBCODES(1:ICOM) = NGRBADVT
    CLNAMES  (1:ICOM) = 'OCEAN ADVT     '
    YSD_V3%YADVT => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YADVT,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))

    IGRBCODES(1:ICOM) = NGRBADVS
    CLNAMES  (1:ICOM) = 'OCEAN ADVS     '
    YSD_V3%YADVS => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YADVS,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))

    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YTRI0 => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YTRI0,KGRIB=IGRBCODES(1:ICOM))

    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YTRI1 => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YTRI1,KGRIB=IGRBCODES(1:ICOM))

    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YSWDK => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YSWDK,KGRIB=IGRBCODES(1:ICOM))

    IGRBCODES(1:ICOM) = NGRBOCUC
    CLNAMES  (1:ICOM) = 'OCN U-VEL CLIM '
    YSD_V3%YUOC => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YUOC,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))

    IGRBCODES(1:ICOM) = NGRBOCVC
    CLNAMES  (1:ICOM) = 'OCN V-VEL CLIM '
    YSD_V3%YVOC => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YVOC,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))

  ENDIF
ENDIF


! EPD (VDIAG)
IF(LECMWF) THEN
  IVDIAG = 80+3+NGHG+NGRG+NTRAC+3*N6BINS
ELSE
  IVDIAG = 0
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_VD%YVD)
IF(ASSOCIATED(YSD_VD%YVD)) DEALLOCATE(YSD_VD%YVD)
ALLOCATE(YSD_VD%YVD(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VDD,YSD_VD%YVD,IVDIAG,.FALSE.,'VDIAG - SD_VD   ')

YSD_VD%YLSP   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCP    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSF    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YBLD   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSHF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSLHF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMSL   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSP    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10U   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10V   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y2T    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y2D    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTSR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTTR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YEWSS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YNSSS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YE     => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YHCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLGWS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMGWS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YGWD   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMX2T  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMN2T  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YRO    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSRO   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSRO  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YALB   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YIEWSS => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YINSSS => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YISSHF => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YIE    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCSF   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLSSF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YZ0F   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLZ0H  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCW   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCWV  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCLW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCIW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCRW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCSW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSRD  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTRD  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YBLH   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSUND  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSPAR  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSUVB  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCAPE  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTSRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTTRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YES    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSMLT  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10FG  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLSPF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCO3  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YVIMD  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSPARC => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTINC => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSFDIR => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSCDIR => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCBASE => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y0DEGL => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YVISIH => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y100U  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y100V  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YZUST  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10NU  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10NV  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YDNDZN => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YDNDZA => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YDCTB  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTPLB  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTPLT  => YSD_VD%YVD(JPMAXSFLDS)

IF(LECMWF) THEN
  YSD_VD%YLSP => YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLSP,KGRIB=NGRBLSP)
  YSD_VD%YCP=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCP,KGRIB=NGRBCP)
  YSD_VD%YSF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSF,KGRIB=NGRBSF)
  YSD_VD%YBLD=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YBLD,KGRIB=NGRBBLD)
  YSD_VD%YSSHF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSHF,KGRIB=NGRBSSHF)
  YSD_VD%YSLHF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSLHF,KGRIB=NGRBSLHF)
  YSD_VD%YMSL=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMSL,KGRIB=NGRBMSL)
  YSD_VD%YSP => YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSP,KGRIB=NGRBSP)
  YSD_VD%YTCC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCC,KGRIB=NGRBTCC)
  YSD_VD%Y10U=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10U,KGRIB=NGRB10U)
  YSD_VD%Y10V=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10V,KGRIB=NGRB10V)
  YSD_VD%Y2T=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y2T,KGRIB=NGRB2T)
  YSD_VD%Y2D=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y2D,KGRIB=NGRB2D)
  YSD_VD%YSSR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSR,KGRIB=NGRBSSR)
  YSD_VD%YSTR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTR,KGRIB=NGRBSTR)
  YSD_VD%YTSR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTSR,KGRIB=NGRBTSR)
  YSD_VD%YTTR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTTR,KGRIB=NGRBTTR)
  YSD_VD%YEWSS=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YEWSS,KGRIB=NGRBEWSS)
  YSD_VD%YNSSS=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YNSSS,KGRIB=NGRBNSSS)
  YSD_VD%YE=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YE,KGRIB=NGRBE)
  YSD_VD%YCCC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCCC,KGRIB=NGRBCCC)
  YSD_VD%YLCC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLCC,KGRIB=NGRBLCC)
  YSD_VD%YMCC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMCC,KGRIB=NGRBMCC)
  YSD_VD%YHCC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YHCC,KGRIB=NGRBHCC)
  YSD_VD%YLGWS=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLGWS,KGRIB=NGRBLGWS)
  YSD_VD%YMGWS=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMGWS,KGRIB=NGRBMGWS)
  YSD_VD%YGWD=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YGWD,KGRIB=NGRBGWD)
  YSD_VD%YMX2T=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMX2T,KGRIB=NGRBMX2T)
  YSD_VD%YMN2T=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMN2T,KGRIB=NGRBMN2T)
  YSD_VD%YRO=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YRO,KGRIB=NGRBRO)
  YSD_VD%YSRO=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSRO,KGRIB=NGRBSRO)
  YSD_VD%YSSRO=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSRO,KGRIB=NGRBSSRO)
  YSD_VD%YALB=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YALB,KGRIB=NGRBFAL)
  YSD_VD%YIEWSS=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YIEWSS,KGRIB=NGRBIEWS)
  YSD_VD%YINSSS=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YINSSS,KGRIB=NGRBINSS)
  YSD_VD%YISSHF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YISSHF,KGRIB=NGRBISHF)
  YSD_VD%YIE=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YIE  ,KGRIB=NGRBIE  )
  YSD_VD%YCSF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCSF ,KGRIB=NGRBCSF )
  YSD_VD%YLSSF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLSSF,KGRIB=NGRBLSF )
  IREQIN=0
  IF(LECMWF .AND. LIFSTRAJ)   IREQIN=1
  IF(LECMWF .AND. (NCONF==401.OR.NCONF==501)) IREQIN=-1
  ITRAJ=0
  IF(LECMWF) ITRAJ=1
  YSD_VD%Y10NU=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10NU,KGRIB=NGRB10NU, CDNAME='10M NEUTRL UWIND',KTRAJ=ITRAJ,KREQIN=IREQIN)
  YSD_VD%Y10NV=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10NV,KGRIB=NGRB10NV, CDNAME='10M NEUTRL VWIND',KTRAJ=ITRAJ,KREQIN=IREQIN)
  YSD_VD%YZ0F=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YZ0F  ,KGRIB=NGRBFSR, CDNAME='FC SURFACE Z0   ',KTRAJ=ITRAJ,KREQIN=IREQIN)
  YSD_VD%YLZ0H=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLZ0H,KGRIB=NGRBFLSR, CDNAME='LOG FC SURF Z0H ',KTRAJ=ITRAJ,KREQIN=IREQIN)
  YSD_VD%YTCW=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCW ,KGRIB=NGRBTCW )
  YSD_VD%YTCWV=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCWV,KGRIB=NGRBTCWV)
  YSD_VD%YTCLW=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCLW,KGRIB=NGRBTCLW)
  YSD_VD%YTCIW=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCIW,KGRIB=NGRBTCIW)
  YSD_VD%YTCRW=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCRW,KGRIB=NGRBTCRW)
  YSD_VD%YTCSW=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCSW,KGRIB=NGRBTCSW)
  YSD_VD%YSSRD=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSRD,KGRIB=NGRBSSRD)
  YSD_VD%YSTRD=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTRD,KGRIB=NGRBSTRD)
  YSD_VD%YBLH=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YBLH ,KGRIB=NGRBBLH )
  YSD_VD%YSUND=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSUND,KGRIB=NGRBSUND)
  YSD_VD%YSPAR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSPAR,KGRIB=NGRBPAR)
  YSD_VD%YSUVB=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSUVB,KGRIB=NGRBUVB)
  YSD_VD%YCAPE=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCAPE,KGRIB=NGRBCAPE)
  YSD_VD%YTSRC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTSRC,KGRIB=NGRBTSRC)
  YSD_VD%YTTRC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTTRC,KGRIB=NGRBTTRC)
  YSD_VD%YSSRC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSRC,KGRIB=NGRBSSRC)
  YSD_VD%YSTRC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTRC,KGRIB=NGRBSTRC)
  YSD_VD%YES=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YES  ,KGRIB=NGRBES  )
  YSD_VD%YSMLT=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSMLT,KGRIB=NGRBSMLT)
  YSD_VD%Y10FG=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10FG,KGRIB=NGRB10FG)
  YSD_VD%YLSPF=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLSPF,KGRIB=NGRBLSPF)
  YSD_VD%YTCO3=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCO3,KGRIB=NGRBTCO3)
  YSD_VD%YVIMD=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YVIMD,KGRIB=NGRBVIMD)
  YSD_VD%YSPARC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSPARC,KGRIB=NGRBPARCS)
  YSD_VD%YSTINC=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTINC,KGRIB=NGRBTISR)

  YSD_VD%YSFDIR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSFDIR,KGRIB=NGRBFDIR)
  YSD_VD%YSCDIR=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSCDIR,KGRIB=NGRBCDIR)

  YSD_VD%YCBASE=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCBASE,KGRIB=NGRBCBASE)
  YSD_VD%Y0DEGL=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y0DEGL,KGRIB=NGRB0DEGL)
  YSD_VD%YVISIH=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YVISIH,KGRIB=NGRBVISIH)

  YSD_VD%Y100U=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y100U,KGRIB=NGRB100U)
  YSD_VD%Y100V=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y100V,KGRIB=NGRB100V)
  YSD_VD%YZUST=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YZUST,KGRIB=NGRBZUST)
  YSD_VD%YDNDZN=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YDNDZN,KGRIB=NGRBDNDZN)
  YSD_VD%YDNDZA=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YDNDZA,KGRIB=NGRBDNDZA)
  YSD_VD%YDCTB=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YDCTB,KGRIB=NGRBDCTB)
  YSD_VD%YTPLB=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTPLB,KGRIB=NGRBTPLB)
  YSD_VD%YTPLT=> YSD_VD%YVD(YSD_VDD%IPTR)
  CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTPLT,KGRIB=NGRBTPLT)
  IF(NGHG > 0) THEN
    YSD_VD%YTCGHG => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+NGHG-1)
    DO JGHG=1,NGHG
      IF(YGHG(JGHG)%LGP) THEN
        CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCGHG(JGHG),YGHG(JGHG)%IGRBCODE)
      ELSE
        YSD_VDD%IPTR = YSD_VDD%IPTR+1
      ENDIF
    ENDDO
  ENDIF
  IF(NGRG > 0) THEN
    YSD_VD%YTCGRG => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+NGRG-1)
    DO JGRG=1,NGRG
      IF(YGRG(JGRG)%LGP) THEN
! nl ext
        CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCGRG(JGRG),YGRG(JGRG)%IGRIBTC)
      ELSE
        YSD_VDD%IPTR = YSD_VDD%IPTR+1
      ENDIF
    ENDDO
  ENDIF
  IF(NTRAC > 0) THEN
    YSD_VD%YTCTRAC => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+NTRAC-1)
    DO JTRAC=1,NTRAC
      IF(YTRAC(JTRAC)%LGP) THEN
        CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCTRAC(JTRAC),YTRAC(JTRAC)%IGRBCODE)
      ELSE
        YSD_VDD%IPTR = YSD_VDD%IPTR+1
      ENDIF
    ENDDO
  ENDIF
  IF(N6BINS > 0) THEN
    YSD_VD%YMX2T6 => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+N6BINS-1)
    DO JBIN=1,N6BINS
      CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMX2T6(JBIN),KGRIB=NGRBMX2T6)
    ENDDO
    YSD_VD%YMN2T6 => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+N6BINS-1)
    DO JBIN=1,N6BINS
      CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMN2T6(JBIN),KGRIB=NGRBMN2T6)
    ENDDO
    YSD_VD%Y10FG6 => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+N6BINS-1)
    DO JBIN=1,N6BINS
      CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10FG6(JBIN),KGRIB=NGRB10FG6)
    ENDDO
  ENDIF
ELSE
ENDIF

! WAVES (for IFS)

IVWV = 0
IF(LECMWF) THEN
  IVWV = 3 ! Charnock, U-, V-STOKES
ENDIF

IF (LL_FIRST_CALL) NULLIFY(YSD_WS%YWS)
IF(ASSOCIATED(YSD_WS%YWS)) DEALLOCATE(YSD_WS%YWS)
ALLOCATE(YSD_WS%YWS(JPMAXSFLDS))
CALL INI_SFLP2(YSD_WSD,YSD_WS%YWS,IVWV,.FALSE.,'WAVES - SD_WS   ')

YSD_WS%YCHAR   => YSD_WS%YWS(JPMAXSFLDS)
YSD_WS%YUSTOKES => YSD_WS%YWS(JPMAXSFLDS)
YSD_WS%YVSTOKES => YSD_WS%YWS(JPMAXSFLDS)

IF(LECMWF) THEN
  IREQIN=-1 ; IF(LWCOU.AND.LWCOU2W) IREQIN=1
  ITRAJ = 2 ; IF(LVDFTRAJ) ITRAJ = 1 ! PUT CHARNOCK IN traj0XX FILES
  YSD_WS%YCHAR => YSD_WS%YWS(YSD_WSD%IPTR)
  CALL SETUP_SFLP2(YSD_WSD,YSD_WS%YCHAR,KGRIB=NGRBCHNK,&
   & PDEFAULT=0.018_JPRB,KTRAJ=ITRAJ,KREQIN=IREQIN, CDNAME='CHARNOCK_PARAM  ')

  YSD_WS%YUSTOKES => YSD_WS%YWS(YSD_WSD%IPTR)
  CALL SETUP_SFLP2(YSD_WSD,YSD_WS%YUSTOKES,CDNAME='U-STOKES_DRIFT  ')
  YSD_WS%YVSTOKES => YSD_WS%YWS(YSD_WSD%IPTR)
  CALL SETUP_SFLP2(YSD_WSD,YSD_WS%YVSTOKES,CDNAME='V-STOKES_DRIFT  ')
ENDIF

! WAVES (for WAM)
    
IVWW = 0
IF(LECMWF) THEN
  IVWW = 5
  IF(LWCUR) IVWW = IVWW + 2
ENDIF

IF(LL_FIRST_CALL) NULLIFY(YSD_WW%YWW)
IF(ASSOCIATED(YSD_WW%YWW)) DEALLOCATE(YSD_WW%YWW)
ALLOCATE(YSD_WW%YWW(JPMAXSFLDS))
CALL INI_SFLP2(YSD_WWD,YSD_WW%YWW,IVWW,.FALSE.,'WAVES - SD_WW   ')

YSD_WW%YU10N   => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YV10N   => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YRHO    => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YZIL    => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YCIF    => YSD_WW%YWW(JPMAXSFLDS)
IF (LWCUR) THEN
 YSD_WW%YUCURW => YSD_WW%YWW(JPMAXSFLDS)
 YSD_WW%YVCURW => YSD_WW%YWW(JPMAXSFLDS)
ENDIF

IF(LECMWF) THEN
  YSD_WW%YU10N => YSD_WW%YWW(YSD_WWD%IPTR)
  CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YU10N, CDNAME='10m_NEUTRAL_U   ')
  YSD_WW%YV10N => YSD_WW%YWW(YSD_WWD%IPTR)
  CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YV10N, CDNAME='10m_NEUTRAL_V   ')
  YSD_WW%YRHO => YSD_WW%YWW(YSD_WWD%IPTR)
  CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YRHO,  CDNAME='SURFACE_DENSITY ')
  YSD_WW%YZIL => YSD_WW%YWW(YSD_WWD%IPTR)
  CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YZIL,  CDNAME='Zi/L            ')
  YSD_WW%YCIF => YSD_WW%YWW(YSD_WWD%IPTR)
  CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YCIF,  CDNAME='SEA_ICE_FRACTION')
  IF (LWCUR) THEN
    YSD_WW%YUCURW=> YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YUCURW, CDNAME='OCEAN U-CURRENT ')
    YSD_WW%YVCURW=> YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YVCURW, CDNAME='OCEAN V-CURRENT ')
  ENDIF
ENDIF

! VCLIX

IVCLIX = 0
IF(LECMWF)THEN
ELSE
  LLFP_CLASSIC = LFPOS.AND.(NFPCLI == 3)
  LLFP_SURFEX  = LFPOS.AND..NOT.LFPART2.AND.LMSE
  IF (LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX) THEN
    IF (LLASSIMAROME) THEN
      IVCLIX=2
    ELSE
      IVCLIX=6
    ENDIF
    IF (LLFP_CLASSIC.OR.LLFP_SURFEX.AND.(.NOT.LLASSIMAROME)) THEN
      IVCLIX=IVCLIX+2
      IF (LLFP_SURFEX) THEN
        IVCLIX=IVCLIX+2
      ENDIF
    ENDIF
  ENDIF
ENDIF
IF (LL_FIRST_CALL) NULLIFY(YSD_VX%YVX)
IF(ASSOCIATED(YSD_VX%YVX)) DEALLOCATE(YSD_VX%YVX)
ALLOCATE(YSD_VX%YVX(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VXD,YSD_VX%YVX,IVCLIX,.FALSE.,'VCLIX - SD_VX   ')
YSD_VX%YORO   => YSD_VX%YVX(JPMAXSFLDS) 
YSD_VX%YTSC   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YPWS   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YPWP   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YSNO   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YTPC   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YSAB   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YXD2   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YLSM   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YIVEG  => YSD_VX%YVX(JPMAXSFLDS)
IF (LECMWF) THEN
ELSE
  IF (LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX) THEN
    YSD_VX%YORO   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YORO, CDNAME='SURFGEOPOTENTIEL ')
    YSD_VX%YTSC   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YTSC, CDNAME='SURFTEMPERATURE  ')
    YSD_VX%YPWS   => YSD_VX%YVX(YSD_VXD%IPTR)
    IF (.NOT.LLASSIMAROME) THEN
      CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YPWS, CDNAME='SURFPROP.RMAX.EAU')
      YSD_VX%YPWP   => YSD_VX%YVX(YSD_VXD%IPTR)
      CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YPWP, CDNAME='PROFPROP.RMAX.EAU')
      YSD_VX%YSNO   => YSD_VX%YVX(YSD_VXD%IPTR)
      CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YSNO ,CDNAME='SURFRESERV.NEIGE ')
      YSD_VX%YTPC   => YSD_VX%YVX(YSD_VXD%IPTR)
      CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YTPC ,CDNAME='PROFTEMPERATURE  ')
      IF (LLFP_CLASSIC.OR.LLFP_SURFEX) THEN
        YSD_VX%YSAB   => YSD_VX%YVX(YSD_VXD%IPTR)
        CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YSAB ,CDNAME='SURFPROP.SABLE   ')
        YSD_VX%YXD2   => YSD_VX%YVX(YSD_VXD%IPTR)
        CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YXD2 ,CDNAME='SURFEPAIS.SOL    ')
        IF (LLFP_SURFEX) THEN
          YSD_VX%YLSM   => YSD_VX%YVX(YSD_VXD%IPTR)
          CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YLSM ,CDNAME='SURFIND.TERREMER ')
          YSD_VX%YIVEG  => YSD_VX%YVX(YSD_VXD%IPTR)
          CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YIVEG,CDNAME='SURFIND.VEG.DOMI ')
        ENDIF
      ENDIF
    ENDIF
  ENDIF
ENDIF

IF (LECMWF) THEN
ELSE
  IF ((LMPHYS.AND.LSOLV).AND.NCONF /= 923) THEN
    IF ( YSD_VVD%NUMFLDS < 7.OR.YSP_SBD%NLEVS < 1.OR.&
     & YSP_SBD%NUMFLDS < 2.OR.YSP_RRD%NUMFLDS < 3 ) THEN
      CALL ABOR1('SU_SURF_FLDS: LSOLV INCONSISTENT WITH YOMDPHY ')
    ENDIF
    IF (LFGEL.AND.(YSP_SBD%NUMFLDS < 3)) THEN
      CALL ABOR1('SU_SURF_FLDS: LFGEL INCONSISTENT WITH YOMDPHY ')
    ENDIF
    IF (LFGEL.AND.(YSP_RRD%NUMFLDS < 4)) THEN
      CALL ABOR1('SU_SURF_FLDS: LFGEL INCONSISTENT WITH YSP_RRD%NUMFLDS   ')
    ENDIF
    IF (LSNV) THEN
      IF (YSP_SGD%NUMFLDS < 3.OR.YSD_VFD%NUMFLDS < 10) THEN
        CALL ABOR1('SU_SURF_FLDS: LSNV INCONSISTENT WITH YOMDPHY ')
      ENDIF
      IF(.NOT.LSOLV .OR. .NOT.LNEIGE ) THEN
        CALL ABOR1('SU_SURF_FLDS: LSNV INCONSISTENT WITH LSOLV OR LNEIGE')
      ENDIF
    ENDIF
    IF (LVGSN.AND.(YSD_VVD%NUMFLDS < 10)) THEN
      CALL ABOR1('SU_SURF_FLDS: LVGSN INCONSISTENT WITH YSD_VVD%NUMFLDS  ')
    ENDIF
  ENDIF
ENDIF

! VEXTRA
IF (LL_FIRST_CALL) NULLIFY(YSD_XA%YXA)
IF(ASSOCIATED(YSD_XA%YXA)) DEALLOCATE(YSD_XA%YXA)
ALLOCATE(YSD_XA%YXA(NVEXTR))
CALL INI_SFLP3(YSD_XAD,YSD_XA%YXA,NVEXTR,NCEXTR,.FALSE.,'VEXTRA - SD_XA  ')
IF(YSD_XAD%NUMFLDS > 0) THEN
  DO JFLD=1,YSD_XAD%NUMFLDS
    CALL SETUP_SFLP3(YSD_XAD,YSD_XA%YXA(JFLD),&
     & KGRIB   =SPREAD(NVEXTRAGB(JFLD),1,YSD_XAD%NLEVS),&
     & CDNAME  =SPREAD(CVEXTRA(JFLD)  ,1,YSD_XAD%NLEVS),&
     & PDEFAULT=SPREAD(VEXTRADF(JFLD) ,1,YSD_XAD%NLEVS))
  ENDDO
ENDIF

! VEXTR2
IF (LL_FIRST_CALL) NULLIFY(YSD_X2%YX2)
IF(ASSOCIATED(YSD_X2%YX2)) DEALLOCATE(YSD_X2%YX2)
ALLOCATE(YSD_X2%YX2(NVXTR2))
CALL INI_SFLP2(YSD_X2D,YSD_X2%YX2,NVXTR2,.FALSE.,'VEXTR2 - SD_X2  ')
IF(YSD_X2D%NUMFLDS > 0) THEN
  DO JFLD=1,YSD_X2D%NUMFLDS
    CALL SETUP_SFLP2(YSD_X2D,YSD_X2%YX2(JFLD),&
     & KGRIB   =NVEXTR2GB(JFLD),&
     & CDNAME  =CVEXTR2(JFLD)  ,&
     & PDEFAULT=VEXTR2DF(JFLD) )
  ENDDO
ENDIF

WRITE(NULOUT,*) 'NSURF=',NSURF,' NSURFL=',NSURFL,' NDIMSURF=',NDIMSURF,&
 & ' NDIMSURFL=',NDIMSURFL
NGP5 = NOFFTRAJ
NTRAJ_CST = NOFFTRAJ_CST
WRITE(NULOUT,*) 'NGP5=',NGP5,' NTRAJ_CST=',NTRAJ_CST
WRITE(NULOUT,'(A,(10I4))') ' NSTRAJGRIB ',NSTRAJGRIB(1:NGP5)

LL_FIRST_CALL=.FALSE.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU_SURF_FLDS',1,ZHOOK_HANDLE)

END SUBROUTINE SU_SURF_FLDS
