SUBROUTINE SUADMI(KNMAXX,LDADMIS,KNMADMI)

!**** *SUADMI* - Says if a truncation is admissible for a quadratic Gaussian
!                grid or not, when stretching c>1.

!     Purpose.
!     --------

!     Says if a truncation is admissible or not for a list of truncations
!     between 1 and KNMAXX.
!     An admissible truncation is a truncation, the number of longitudes
!     of its Gaussian grid writes as 2**(1+r1) * 3**r2 * 5**r3.
!     For a not admissible truncation, returns the immediately lower
!     and upper admissible truncations.

!**   Interface.
!     ----------
!        *CALL* *SUNCMAX(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KNMAXX    - maximum truncation.

!        OUTPUT:
!          LDADMIS   - indicator of admissibility.
!          KNMADMI   - KNMADMI(.,1): immediately lower admissible truncation.
!                      KNMADMI(.,2): immediately upper admissible truncation.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about conf 911.

!     Externals.
!     ----------
!       none.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP). 

!     Modifications.
!     --------------
!        Original : APRIL 1999
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAXX 
LOGICAL           ,INTENT(OUT)   :: LDADMIS(KNMAXX) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNMADMI(KNMAXX,2) 
INTEGER(KIND=JPIM) :: INMAXN
INTEGER(KIND=JPIM) :: INMAXX
INTEGER(KIND=JPIM) :: JNMAX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUADMI',0,ZHOOK_HANDLE)
LDADMIS=.FALSE.

LDADMIS(    1)=.TRUE.
LDADMIS(    3)=.TRUE.
LDADMIS(    4)=.TRUE.
LDADMIS(    5)=.TRUE.
LDADMIS(    7)=.TRUE.
LDADMIS(    9)=.TRUE.
LDADMIS(   11)=.TRUE.
LDADMIS(   12)=.TRUE.
LDADMIS(   15)=.TRUE.
LDADMIS(   19)=.TRUE.
LDADMIS(   20)=.TRUE.
LDADMIS(   23)=.TRUE.
LDADMIS(   25)=.TRUE.
LDADMIS(   29)=.TRUE.
LDADMIS(   31)=.TRUE.
LDADMIS(   32)=.TRUE.
LDADMIS(   35)=.TRUE.
LDADMIS(   39)=.TRUE.
LDADMIS(   41)=.TRUE.
LDADMIS(   47)=.TRUE.
LDADMIS(   49)=.TRUE.
LDADMIS(   52)=.TRUE.
LDADMIS(   53)=.TRUE.
LDADMIS(   59)=.TRUE.
LDADMIS(   63)=.TRUE.
LDADMIS(   65)=.TRUE.
LDADMIS(   71)=.TRUE.
LDADMIS(   79)=.TRUE.
LDADMIS(   83)=.TRUE.
LDADMIS(   84)=.TRUE.
LDADMIS(   89)=.TRUE.
LDADMIS(   95)=.TRUE.
LDADMIS(   99)=.TRUE.
LDADMIS(  105)=.TRUE.
LDADMIS(  107)=.TRUE.
LDADMIS(  119)=.TRUE.
LDADMIS(  127)=.TRUE.
LDADMIS(  132)=.TRUE.
LDADMIS(  143)=.TRUE.
LDADMIS(  149)=.TRUE.
LDADMIS(  159)=.TRUE.
LDADMIS(  161)=.TRUE.
LDADMIS(  165)=.TRUE.
LDADMIS(  169)=.TRUE.
LDADMIS(  179)=.TRUE.
LDADMIS(  191)=.TRUE.
LDADMIS(  199)=.TRUE.
LDADMIS(  212)=.TRUE.
LDADMIS(  215)=.TRUE.
LDADMIS(  239)=.TRUE.
LDADMIS(  249)=.TRUE.
LDADMIS(  255)=.TRUE.
LDADMIS(  265)=.TRUE.
LDADMIS(  269)=.TRUE.
LDADMIS(  287)=.TRUE.
LDADMIS(  299)=.TRUE.
LDADMIS(  319)=.TRUE.
LDADMIS(  323)=.TRUE.
LDADMIS(  332)=.TRUE.
LDADMIS(  340)=.TRUE.
LDADMIS(  359)=.TRUE.
LDADMIS(  383)=.TRUE.
LDADMIS(  399)=.TRUE.
LDADMIS(  416)=.TRUE.
LDADMIS(  425)=.TRUE.
LDADMIS(  431)=.TRUE.
LDADMIS(  449)=.TRUE.
LDADMIS(  479)=.TRUE.
LDADMIS(  485)=.TRUE.
LDADMIS(  499)=.TRUE.
LDADMIS(  511)=.TRUE.
LDADMIS(  532)=.TRUE.
LDADMIS(  539)=.TRUE.
LDADMIS(  575)=.TRUE.
LDADMIS(  599)=.TRUE.
LDADMIS(  639)=.TRUE.
LDADMIS(  647)=.TRUE.
LDADMIS(  665)=.TRUE.
LDADMIS(  681)=.TRUE.
LDADMIS(  719)=.TRUE.
LDADMIS(  749)=.TRUE.
LDADMIS(  767)=.TRUE.
LDADMIS(  799)=.TRUE.
LDADMIS(  809)=.TRUE.
LDADMIS(  832)=.TRUE.
LDADMIS(  852)=.TRUE.
LDADMIS(  863)=.TRUE.
LDADMIS(  899)=.TRUE.
LDADMIS(  959)=.TRUE.
LDADMIS(  971)=.TRUE.
LDADMIS(  999)=.TRUE.
LDADMIS( 1023)=.TRUE.
LDADMIS( 1065)=.TRUE.
LDADMIS( 1079)=.TRUE.
LDADMIS( 1151)=.TRUE.
LDADMIS( 1199)=.TRUE.
LDADMIS( 1249)=.TRUE.
LDADMIS( 1279)=.TRUE.
LDADMIS( 1295)=.TRUE.
LDADMIS( 1332)=.TRUE.
LDADMIS( 1349)=.TRUE.
LDADMIS( 1364)=.TRUE.
LDADMIS( 1439)=.TRUE.
LDADMIS( 1457)=.TRUE.
LDADMIS( 1499)=.TRUE.
LDADMIS( 1535)=.TRUE.
LDADMIS( 1599)=.TRUE.
LDADMIS( 1619)=.TRUE.
LDADMIS( 1665)=.TRUE.
LDADMIS( 1705)=.TRUE.
LDADMIS( 1727)=.TRUE.
LDADMIS( 1799)=.TRUE.
LDADMIS( 1919)=.TRUE.
LDADMIS( 1943)=.TRUE.
LDADMIS( 1999)=.TRUE.
LDADMIS( 2047)=.TRUE.
LDADMIS( 2083)=.TRUE.
LDADMIS( 2132)=.TRUE.
LDADMIS( 2159)=.TRUE.
LDADMIS( 2249)=.TRUE.
LDADMIS( 2303)=.TRUE.
LDADMIS( 2399)=.TRUE.
LDADMIS( 2429)=.TRUE.
LDADMIS( 2499)=.TRUE.
LDADMIS( 2559)=.TRUE.
LDADMIS( 2591)=.TRUE.
LDADMIS( 2665)=.TRUE.
LDADMIS( 2699)=.TRUE.
LDADMIS( 2729)=.TRUE.
LDADMIS( 2879)=.TRUE.
LDADMIS( 2915)=.TRUE.
LDADMIS( 2999)=.TRUE.
LDADMIS( 3071)=.TRUE.
LDADMIS( 3199)=.TRUE.
LDADMIS( 3239)=.TRUE.
LDADMIS( 3332)=.TRUE.
LDADMIS( 3412)=.TRUE.
LDADMIS( 3455)=.TRUE.
LDADMIS( 3599)=.TRUE.
LDADMIS( 3749)=.TRUE.
LDADMIS( 3839)=.TRUE.
LDADMIS( 3887)=.TRUE.
LDADMIS( 3999)=.TRUE.
LDADMIS( 4049)=.TRUE.
LDADMIS( 4095)=.TRUE.
LDADMIS( 4165)=.TRUE.
LDADMIS( 4265)=.TRUE.
LDADMIS( 4319)=.TRUE.
LDADMIS( 4373)=.TRUE.
LDADMIS( 4499)=.TRUE.
LDADMIS( 4607)=.TRUE.
LDADMIS( 4799)=.TRUE.
LDADMIS( 4859)=.TRUE.
LDADMIS( 4999)=.TRUE.
LDADMIS( 5119)=.TRUE.
LDADMIS( 5183)=.TRUE.
LDADMIS( 5332)=.TRUE.
LDADMIS( 5399)=.TRUE.
LDADMIS( 5460)=.TRUE.
LDADMIS( 5759)=.TRUE.
LDADMIS( 5831)=.TRUE.
LDADMIS( 5999)=.TRUE.
LDADMIS( 6143)=.TRUE.
LDADMIS( 6249)=.TRUE.
LDADMIS( 6399)=.TRUE.
LDADMIS( 6479)=.TRUE.
LDADMIS( 6665)=.TRUE.
LDADMIS( 6749)=.TRUE.
LDADMIS( 6825)=.TRUE.
LDADMIS( 6911)=.TRUE.
LDADMIS( 7199)=.TRUE.
LDADMIS( 7289)=.TRUE.
LDADMIS( 7499)=.TRUE.
LDADMIS( 7679)=.TRUE.
LDADMIS( 7775)=.TRUE.
LDADMIS( 7999)=.TRUE.
LDADMIS( 8099)=.TRUE.
LDADMIS( 8191)=.TRUE.
LDADMIS( 8332)=.TRUE.
LDADMIS( 8532)=.TRUE.
LDADMIS( 8639)=.TRUE.
LDADMIS( 8747)=.TRUE.
LDADMIS( 8999)=.TRUE.
LDADMIS( 9215)=.TRUE.
LDADMIS( 9599)=.TRUE.
LDADMIS( 9719)=.TRUE.
LDADMIS( 9999)=.TRUE.
LDADMIS(10239)=.TRUE.
LDADMIS(10367)=.TRUE.
LDADMIS(10416)=.TRUE.
LDADMIS(10665)=.TRUE.
LDADMIS(10799)=.TRUE.
LDADMIS(10921)=.TRUE.
LDADMIS(11249)=.TRUE.
LDADMIS(11519)=.TRUE.
LDADMIS(11663)=.TRUE.
LDADMIS(11999)=.TRUE.
LDADMIS(12149)=.TRUE.
LDADMIS(12287)=.TRUE.
LDADMIS(12499)=.TRUE.
LDADMIS(12799)=.TRUE.
LDADMIS(12959)=.TRUE.
LDADMIS(13121)=.TRUE.
LDADMIS(13332)=.TRUE.
LDADMIS(13499)=.TRUE.
LDADMIS(13652)=.TRUE.
LDADMIS(13823)=.TRUE.
LDADMIS(14399)=.TRUE.
LDADMIS(14579)=.TRUE.
LDADMIS(14999)=.TRUE.
LDADMIS(15359)=.TRUE.
LDADMIS(15551)=.TRUE.
LDADMIS(15999)=.TRUE.

DO JNMAX=1,KNMAXX
  KNMADMI(JNMAX,1:2)=JNMAX
  INMAXN=JNMAX
  INMAXX=JNMAX
  DO WHILE(.NOT.LDADMIS(INMAXN))
    INMAXN=INMAXN-1
    KNMADMI(JNMAX,1)=INMAXN
  ENDDO
  DO WHILE(.NOT.LDADMIS(INMAXX))
    INMAXX=INMAXX+1
    KNMADMI(JNMAX,2)=INMAXX
  ENDDO
ENDDO

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUADMI',1,ZHOOK_HANDLE)
END SUBROUTINE SUADMI
