SUBROUTINE SUALMP1

!**** *SUALMP1 * - Routine to allocate space for some distributed memory arrays

!     Purpose.
!     --------
!           Allocate space for most of the distributed memory control arrays

!**   Interface.
!     ----------
!        *CALL* *SUALMP1*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-10-01
!        Modified : 96-06-03 L.D. : ->AL06
!        =99-05-04: Deborah Salmond and Bob Carruthers
!                  ALLOCATES fixed to enable NPRTRV .gt. NPRTRW
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF    ,&
 & NPRINTLEV,NPRTRW   ,NPRTRNS  ,&
 & NPRTRV   ,LALLOPR  ,N_REGIONS_NS,N_REGIONS_EW
USE YOMDIM   , ONLY : NDLUNL   ,NDLUXL   ,NDGLG    ,&
 & NDGSAG   ,NDGENG   ,NFLEVL   ,NFLEVG   ,&
 & NMSMAX   ,NSMAX    ,NCMAX  
USE YOMINI   , ONLY : NEINI
USE YOMMP    , ONLY : NUMPP    ,NPROCM   ,NPTRMS   ,NALLMS   ,&
 & NPTRLS   ,NPTRSV   ,&
 & NPTRCV   ,NPTRTV   ,NPTRSVF  ,NPTRMF   ,NSPSTAF  ,&
 & NPTRLL   ,NUMVMO   ,NUMVMOJB ,MYLEVS   ,&
 & NSTA     ,NONL     ,NPTRFRSTLAT,&
 & NPTRLSTLAT,NFRSTLAT ,NLSTLAT  ,NPTRLAT ,&
 & NBSETLEV ,LSPLITLAT, &
 & NUMXPP   ,NGPSET2PE
USE YOMMPG   , ONLY : NPOSSP   ,NDIM0G
USE YOMGEM   , ONLY : NGPTOTL
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

INTEGER(KIND=JPIM) :: ISMAX, IU, JSMAX

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUALMP1',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
ALLOCATE(NUMPP(NPRTRW))
IF(LLP)WRITE(IU,9) 'NUMPP    ',SIZE(NUMPP    ),SHAPE(NUMPP    )
ALLOCATE(NUMXPP(NPRTRW))
IF(LLP)WRITE(IU,9) 'NUMXPP   ',SIZE(NUMXPP   ),SHAPE(NUMXPP   )
ALLOCATE(NPROCM(0:NCMAX))
IF(LLP)WRITE(IU,9) 'NPROCM   ',SIZE(NPROCM   ),SHAPE(NPROCM   )
ALLOCATE(NPTRMS(NPRTRW))
IF(LLP)WRITE(IU,9) 'NPTRMS   ',SIZE(NPTRMS   ),SHAPE(NPTRMS   )
ALLOCATE(NPOSSP(NPRTRW+1))
IF(LLP)WRITE(IU,9) 'NPOSSP   ',SIZE(NPOSSP   ),SHAPE(NPOSSP   )
ISMAX=MAX(NSMAX,NMSMAX)
ALLOCATE(NALLMS(ISMAX+1))
IF(LLP)WRITE(IU,9) 'NALLMS   ',SIZE(NALLMS   ),SHAPE(NALLMS   )
ALLOCATE(NDIM0G(0:ISMAX))
IF(LLP)WRITE(IU,9) 'NDIM0G   ',SIZE(NDIM0G   ),SHAPE(NDIM0G   )

ALLOCATE(NPTRLS(NPRTRNS))
IF(LLP)WRITE(IU,9) 'NPTRLS   ',SIZE(NPTRLS   ),SHAPE(NPTRLS   )
ALLOCATE(NPTRSV(NPRTRV+1))
IF(LLP)WRITE(IU,9) 'NPTRSV   ',SIZE(NPTRSV   ),SHAPE(NPTRSV   )
ALLOCATE(NPTRCV(NPRTRV+1))
IF(LLP)WRITE(IU,9) 'NPTRCV   ',SIZE(NPTRCV   ),SHAPE(NPTRCV   )
ALLOCATE(NPTRTV(NPRTRV+1))
IF(LLP)WRITE(IU,9) 'NPTRTV   ',SIZE(NPTRTV   ),SHAPE(NPTRTV   )
ALLOCATE(NPTRSVF(NPRTRV+1))
IF(LLP)WRITE(IU,9) 'NPTRSVF  ',SIZE(NPTRSVF  ),SHAPE(NPTRSVF  )
ALLOCATE(NPTRMF(NPRTRV+1))
IF(LLP)WRITE(IU,9) 'NPTRMF   ',SIZE(NPTRMF   ),SHAPE(NPTRMF   )
JSMAX=MAX(NSMAX,NMSMAX)
ALLOCATE(NSPSTAF(0:JSMAX))
IF(LLP)WRITE(IU,9) 'NSPSTAF  ',SIZE(NSPSTAF  ),SHAPE(NSPSTAF  )
ALLOCATE(NPTRLL(NPRTRV+1))
IF(LLP)WRITE(IU,9) 'NPTRLL   ',SIZE(NPTRLL   ),SHAPE(NPTRLL   )
IF (NEINI /= 0.AND.NEINI /= 2) THEN
  ALLOCATE(NUMVMO(NPRTRV))
  IF(LLP)WRITE(IU,9) 'NUMVMO   ',SIZE(NUMVMO   ),SHAPE(NUMVMO   )
  IF (NCONF/100 /= 0) THEN
    ALLOCATE(NUMVMOJB(NPRTRV))
    IF(LLP)WRITE(IU,9) 'NUMVMOJB ',SIZE(NUMVMOJB),SHAPE(NUMVMOJB)
  ENDIF
ENDIF
ALLOCATE(MYLEVS(NFLEVL))
IF(LLP)WRITE(IU,9) 'MYLEVS   ',SIZE(MYLEVS   ),SHAPE(MYLEVS   )
ALLOCATE(NSTA(NDGSAG:NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
IF(LLP)WRITE(IU,9) 'NSTA     ',SIZE(NSTA     ),SHAPE(NSTA     )
ALLOCATE(NONL(NDGSAG:NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
IF(LLP)WRITE(IU,9) 'NONL     ',SIZE(NONL     ),SHAPE(NONL     )
ALLOCATE(NPTRFRSTLAT(N_REGIONS_NS))
IF(LLP)WRITE(IU,9) 'NPTRFRSTLAT',SIZE(NPTRFRSTLAT),SHAPE(NPTRFRSTLAT)
ALLOCATE(NPTRLSTLAT(N_REGIONS_NS))
IF(LLP)WRITE(IU,9)'NPTRLSTLAT',SIZE(NPTRLSTLAT),SHAPE(NPTRLSTLAT)
ALLOCATE(NPTRLAT(NDGLG))
IF(LLP)WRITE(IU,9) 'NPTRLAT  ',SIZE(NPTRLAT  ),SHAPE(NPTRLAT  )
ALLOCATE(LSPLITLAT(NDGLG))
IF(LLP)WRITE(IU,9) 'LSPLITLAT',SIZE(LSPLITLAT),SHAPE(LSPLITLAT)
ALLOCATE(NFRSTLAT(N_REGIONS_NS))
IF(LLP)WRITE(IU,9) 'NFRSTLAT ',SIZE(NFRSTLAT ),SHAPE(NFRSTLAT )
ALLOCATE(NLSTLAT(N_REGIONS_NS))
IF(LLP)WRITE(IU,9) 'NLSTLAT  ',SIZE(NLSTLAT  ),SHAPE(NLSTLAT  )
ALLOCATE(NBSETLEV(NFLEVG))
IF(LLP)WRITE(IU,9) 'NBSETLEV ',SIZE(NBSETLEV ),SHAPE(NBSETLEV )
ALLOCATE(NDLUNL(NDGSAG:NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
IF(LLP)WRITE(IU,9) 'NDLUNL   ',SIZE(NDLUNL   ),SHAPE(NDLUNL   )
ALLOCATE(NDLUXL(NDGSAG:NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
IF(LLP)WRITE(IU,9) 'NDLUXL   ',SIZE(NDLUXL   ),SHAPE(NDLUXL   )
ALLOCATE(NGPTOTL(N_REGIONS_NS,N_REGIONS_EW))
IF(LLP)WRITE(IU,9) 'NGPTOTL  ',SIZE(NGPTOTL  ),SHAPE(NGPTOTL  )
ALLOCATE(NGPSET2PE(N_REGIONS_NS,N_REGIONS_EW))
IF(LLP)WRITE(IU,9) 'NGPSET2PE',SIZE(NGPSET2PE),SHAPE(NGPSET2PE)
!     ------------------------------------------------------------------
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUALMP1',1,ZHOOK_HANDLE)
END SUBROUTINE SUALMP1
