SUBROUTINE SUALMP2

!**** *SUALMP2 * - Routine to allocate space for some distributed memory arrays

!     Purpose.
!     --------
!           Allocate space for MYLATS, MYNVMO, and MYMS

!**   Interface.
!     ----------
!        *CALL* *SUALMP2*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        MPP Group *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-10-01
!        Modified by K. YESSAD (MARCH 1997): parallelisation of FULL-POS,
!         allocation of NFPCORE.
!        K. YESSAD (APRIL 1997): add NUMPROCGP.
!        Modified : 98-10-06 R. El Khatib - Remove NUMPROCGP
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF    ,NPRINTLEV,&
 & NPRTRV   ,LALLOPR    
USE YOMDIM   , ONLY : NDGSAL   ,NDGENL   ,NUMCP    
USE YOMMP    , ONLY : NUMVMO   ,NUMVMOJB ,&
 & MYLATS   ,NVMODIST,NGLOBALINDEX ,NGLOBALPROC ,NLOCALINDEX
USE YOMLAP   , ONLY : MYMS
USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG
USE YOMINI   , ONLY : NEINI

IMPLICIT NONE

INTEGER(KIND=JPIM) :: INVMODMXPP, IU

LOGICAL :: LLDONMI, LLP, LLVAR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUALMP2',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
ALLOCATE(MYLATS(NDGSAL:NDGENL))
IF(LLP)WRITE(IU,9) 'MYLATS   ',SIZE(MYLATS   ),SHAPE(MYLATS   )

LLVAR=NCONF/100 /= 0.AND.NCONF/100 /= 2
LLDONMI=NEINI /= 0.AND.NEINI /= 2
IF (LLDONMI) THEN
  IF (LLVAR) THEN
    INVMODMXPP=MAX(MAXVAL(NUMVMO),MAXVAL(NUMVMOJB))
  ELSE
    INVMODMXPP=MAXVAL(NUMVMO)
  ENDIF
  ALLOCATE(NVMODIST(INVMODMXPP,NPRTRV))
  IF(LLP)WRITE(IU,9) 'NVMODIST ',SIZE(NVMODIST ),SHAPE(NVMODIST )
ENDIF
ALLOCATE(MYMS(NUMCP))
IF(LLP)WRITE(IU,9) 'MYMS     ',SIZE(MYMS     ),SHAPE(MYMS     )

ALLOCATE(NGLOBALINDEX(NGPTOT))
IF(LLP)WRITE(IU,9) 'NGLOBALIND',SIZE(NGLOBALINDEX),SHAPE(NGLOBALINDEX)
ALLOCATE(NGLOBALPROC(NGPTOTG))
IF(LLP)WRITE(IU,9) 'NGLOBALPRO',SIZE(NGLOBALPROC),SHAPE(NGLOBALPROC)
ALLOCATE(NLOCALINDEX(NGPTOTG))
IF(LLP)WRITE(IU,9) 'NLOCALINDE',SIZE(NLOCALINDEX),SHAPE(NLOCALINDEX)

!     ------------------------------------------------------------------
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUALMP2',1,ZHOOK_HANDLE)
END SUBROUTINE SUALMP2
