SUBROUTINE SUCFU(KULOUT)

!**** *SUCFU  * - ROUTINE TO INITIALIZE THE CONTROL OF CUMULATED FLUXES

!     PURPOSE.
!     --------
!        SET DEFAULT VALUES, THEN READS NAMELIST NAMCFU AND
!        INITIALIZE POINTERS

!**   INTERFACE.
!     ----------
!        *CALL* *SUCFU(KULOUT)*

!     EXPLICIT ARGUMENTS :  KULOUT
!     --------------------

!     IMPLICIT ARGUMENTS :
!     --------------------
!        COMMON  YOMCFU

!     METHOD.
!     -------
!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-05-23
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled
!                                   by LECMWF
!        Modified: 93-03-29 by C. Dreveton (liquid and solid humidity)
!        Modified: 93-08-18 by R. El Khatib (partial cloudiness)
!        Modified: 94-03-03 by R. El Khatib (LREACFU)
!        Modified BY E. BAZILE   11/02/94 (Surface Fluxes)
!        Modified: 94-06-16 by R. El Khatib (Modify default values for
!                  Full-POS)
!        Modified: 95-04 by Ph. Dandin - FMR scheme with MF physics.
!        Modified: 97-05 by M. Deque   - frozen FMR
!        Modified: 97-08 by D. Giard - case LFSOIL and LSOLV
!        Modified: 99-02 by E. Bazile - Soil freezing.
!        Modified: 99-04 by J.M. Piriou - Moon radiation.
!        Modified: 00-03 by R. El Khatib - Indexes of convection
!        Modified: 00-03 by R. El Khatib - derived types
!                              + indefined pointer set to 0
!                              + filtered duration of precipitations
!        Modified: 01-08-07 by R. El Khatib - pruning options
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified: 04-09-15 by F.Bouyssel - LCUMFU forced to false if LMPHYS=F
!        Y.Seity      15-Jan-2007 add Graupel and Hail in CFU 
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULNAM
USE YOMDIM   , ONLY : NFLEVG
USE YOMCFU   , ONLY : JPFUST   ,NCFUTS   ,NFRRC    ,NFRCFU   ,TYPE_CFU ,&
 & NTYPCFU  ,NFDCFU   ,LCUMFU   ,LREACFU  ,LSTRD    ,LSTRC    ,&
 & LSTRT    ,LFPLC    ,LFPLCG   ,LFPLCH   ,LFPLS    ,LFPLSG   ,&
 & LFPLSH   ,LFR      ,LAMIP    ,&
 & LRAYS    ,LNEBTT   ,LFSF     ,LFSOIL   ,LNEBPAR  ,LTSTRD   ,&
 & LTSTRC   ,LTSTRT   ,LTFPLC   ,LTFPLS   ,LTFR     ,&
 & LTNEB    ,LFDUTP   ,LMOON    ,LFRRC    ,LRAYD
USE YOMPHY   , ONLY : LCONDWT  ,LNEIGE   ,LOZONE   ,LRAYFM15 ,LSOLV    ,&
 & LFGEL    ,LMPHYS
USE YOERAD   , ONLY : NRADFR
USE YOMRAD15 , ONLY : NRADFR15
USE PTRGFU   , ONLY : MSTRDU   ,MSTRDV   ,MSTRCU   ,MSTRCV   ,MFDICQ   ,&
 & MFDICS   ,MSTRTU   ,MSTRTV   ,MFDITQ   ,MFDITS   ,MFPLCL   ,&
 & MFPLSL   ,MFPLCN   ,MFPLSN   ,MFPLCG   ,MFPLSG   ,MFPLCH   ,&
 & MFPLSH   ,MFCCQL   ,MFCCQN   ,MFCSQL   ,&
 & MFCSQN   ,MFRSO    ,MFRTH    ,MFNEB    ,MWS      ,MSNS     ,&
 & MQTOT    ,MFNEBT   ,MOZONT   ,MSPRES   ,MFCL     ,MFCS     ,&
 & MFDISH   ,MFTOPH   ,MQICE    ,MQLI     ,MFRSOC0  ,MFRTHC0  ,&
 & MFRSOPS  ,MFRSOPT  ,MFRSOLU  ,MFRSODS  ,MFRTHDS  ,MFCLL    ,&
 & MFCLN    ,MFEVL    ,MFEVN    ,MFONTE   ,MFCHSP   ,MFEVV    ,&
 & MFLWSP   ,MFTR     ,MRUISL   ,MRUISP   ,MRUISS   ,MNEBCON  ,&
 & MNEBHAU  ,MNEBMOY  ,MNEBBAS  ,MFGEL    ,&
 & MFGELS   ,MFRSOC1  ,MFRTHC1  ,MFDUTP   
USE YOMCT0   , ONLY : LFPOS
USE YOMCT0B  , ONLY : LECMWF
USE YOMSTA   , ONLY : STPRE
USE YOMAFN   , ONLY : GFP_CBLD ,GFP_CCCC ,GFP_CCHS ,GFP_CWS  ,GFP_CVSS ,&
 & GFP_CCSG ,GFP_CLSG ,GFP_CCSH,GFP_CLSH ,GFP_CCP  ,GFP_CCSF ,&
 & GFP_CCVQ ,GFP_CCVS ,&
 & GFP_CCVU ,GFP_CCVV ,GFP_CDRU ,GFP_CE   ,GFP_CEAS ,GFP_CETP ,&
 & GFP_CFON ,GFP_CHCC ,GFP_CICE ,GFP_CIRU ,GFP_CLCC ,GFP_CLHE ,&
 & GFP_CLHS ,GFP_CLI  ,GFP_CLSP ,GFP_CLSS ,GFP_CMCC ,GFP_CQTO ,&
 & GFP_CS   ,GFP_CDUTP,GFP_CSLH ,GFP_CSMR ,GFP_CSNS ,GFP_CSOC ,&
 & GFP_CSOD ,GFP_CSOP ,GFP_CSRU ,GFP_CSSH ,GFP_CSSR ,GFP_CSTR ,&
 & GFP_CTCC ,GFP_CTHC ,GFP_CTHD ,GFP_CTME ,GFP_CTO3 ,GFP_CTOP ,&
 & GFP_CTP  ,GFP_CTSP ,GFP_CTUQ ,GFP_CTUS ,GFP_CUGW ,GFP_CUSS ,&
 & GFP_CVGW  
  
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
!     PARAMETERS
INTEGER(KIND=JPIM), PARAMETER :: JPKIND=6

INTEGER(KIND=JPIM) :: IP, J, IPAR, IK, INF550, INF850, JF, JL, IADD

INTEGER(KIND=JPIM) :: INLEV(NFLEVG+1,JPKIND), ILEV(JPKIND)

CHARACTER(LEN=4) ::  CLPRE(NFLEVG+1,JPKIND), CLEVS(1)
CHARACTER(LEN=16) ::  CLSU
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"
#include "seapre.intfb.h"
#include "sufpcfu.intfb.h"

#include "namcfu.h"
!      ----------------------------------------------------------------

!*       1.    SET DEFAULT VALUES.
!              -------------------

!        1.1 Set implicit default values

IF (LHOOK) CALL DR_HOOK('SUCFU',0,ZHOOK_HANDLE)
LCUMFU = .FALSE.
LREACFU= .FALSE.
LSTRD  = .FALSE.
LTSTRD = .FALSE.
LSTRC  = .FALSE.
LTSTRC = .FALSE.
LSTRT  = .FALSE.
LTSTRT = .FALSE.
LFPLC  = .FALSE.
LFPLCG  = .FALSE.
LFPLCH  = .FALSE.
LTFPLC = .FALSE.
LFPLS  = .FALSE.
LFPLSG = .FALSE.
LFPLSH = .FALSE.
LTFPLS = .FALSE.
LFR    = .FALSE.
LTFR   = .FALSE.
LTNEB  = .FALSE.
LNEBTT = .FALSE.
LNEBPAR= .FALSE.
LFSF   = .FALSE.
LAMIP  = .FALSE.
LRAYS  = .FALSE.
LRAYD  = .FALSE.
LMOON  = .FALSE.
LFSOIL = .FALSE.
LFDUTP = .FALSE.
LFRRC  = .FALSE.
DO J=0,JPFUST
  NCFUTS(J)=0
ENDDO
NFRCFU   =20

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
  NFRCFU=9999999
  NCFUTS(0)=1
  NCFUTS(1)=1
ENDIF

!      ----------------------------------------------------------------

!*       2.    MODIFIES DEFAULT VALUES.
!              ------------------------

CALL POSNAM(NULNAM,'NAMCFU')
READ(NULNAM,NAMCFU)

IF (LFPOS) THEN
  CALL SUFPCFU
ENDIF
IF (LCUMFU .AND. .NOT.LMPHYS) THEN
  LCUMFU=.FALSE.
ENDIF

LSTRD  =LSTRD  .AND.LCUMFU
LSTRC  =LSTRC  .AND.LCUMFU
LSTRT  =LSTRT  .AND.LCUMFU
LFPLC  =LFPLC  .AND.LCUMFU
LFPLS  =LFPLS  .AND.LCUMFU
LFR    =LFR    .AND.LCUMFU
LTNEB  =LTNEB  .AND.LCUMFU
LNEBTT =LNEBTT .AND.LCUMFU
LNEBPAR=LNEBPAR.AND.LCUMFU
LFSF   =LFSF   .AND.LCUMFU
LFSOIL =LFSOIL .AND.LCUMFU
LFRRC  =LFRRC  .AND.LCUMFU
LAMIP  =LAMIP  .AND.LCUMFU
LFDUTP =LFDUTP .AND.LCUMFU
LMOON  =LMOON  .AND.LCUMFU
LRAYD  =LRAYD  .AND.LCUMFU

IF (LFRRC) THEN
  IF (LRAYFM15) THEN
    NFRRC=NRADFR15
  ELSE
    NFRRC=NRADFR
  ENDIF
ELSE
  NFRRC=0
ENDIF
!      ----------------------------------------------------------------

!*       3.    INITIALIZE DESCRIPTORS
!              ----------------------

!        3.1 Preinitialize the vertical levels and prefixes for *FA* files

CLEVS(1) ='S   '

! On each half level :
IK=1
ILEV(IK)=NFLEVG+1
DO J=1,ILEV(IK)
  CLPRE(J,IK)=CLEVS(1)
  INLEV(J,IK)=J-1
ENDDO
! At the surface
IK=2
ILEV(IK)=1
CLPRE(1,IK)='SURF'
INLEV(1,IK)=NFLEVG
! On each full level :
IK=3
ILEV(IK)=NFLEVG
DO J=1,ILEV(IK)
  CLPRE(J,IK)=CLEVS(1)
  INLEV(J,IK)=J
ENDDO
! At the top and the surface :
IK=4
ILEV(IK)=2
CLPRE(1,IK)='SOMM'
CLPRE(2,IK)='SURF'
INLEV(1,IK)=0
INLEV(2,IK)=NFLEVG
! At a given model level
IK=5
ILEV(IK)=1
CLPRE(1,IK)=CLEVS(1)

!        3.2 Setup descripors

IP=0
IADD=1

!        3.2.1 STRess of gravity wave Drag :

MSTRDU=0
MSTRDV=0
IF (LSTRD.OR.LAMIP) THEN
  IF (LTSTRD) THEN
!   On each half level : 
    IK=1 
  ELSE
!   At the surface :
    IK=2 
  ENDIF
  IPAR=1
  CLSU=GFP_CUGW%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MSTRDU)
  CLSU=GFP_CVGW%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MSTRDV)
ENDIF

!        3.2.2 STRess of Convection :

MSTRCU=0
MSTRCV=0
MFDICQ=0
MFDICS=0
IF (LSTRC) THEN
  IF (LTSTRC) THEN
!   On each half level :
    IK=1
  ELSE
!   At the model level the closest to 550 hPa :
    IK=5
    CALL SEAPRE(55000._JPRB,INF550,STPRE,NFLEVG)
    INLEV(1,IK)=INF550-1
  ENDIF
  CLSU=GFP_CCVU%CLNAME(5:)
  IPAR=1
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MSTRCU)
  CLSU=GFP_CCVV%CLNAME(5:)
  IPAR=1
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MSTRCV)
  IF (LTSTRC) THEN
!   On each half level :
    IK=1
  ELSE
!   At the model level the closest to 850 hPa :
    IK=5
    CALL SEAPRE(85000._JPRB,INF850,STPRE,NFLEVG)
    INLEV(1,IK)=INF850-1
  ENDIF
  CLSU=GFP_CCVQ%CLNAME(5:)
  IPAR=0
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MFDICQ)
  CLSU=GFP_CCVS%CLNAME(5:)
  IPAR=0
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MFDICS)
ENDIF

!        3.2.3 STRess of Turbulence, or Surface Fluxes

MSTRTU=0
MSTRTV=0
MFDITQ=0
MFDITS=0
IF (LFSF.OR.LSTRT.OR.LAMIP) THEN
  IF (LTSTRT) THEN
!   On each half level :
    IK=1
  ELSE
!   At the surface :
    IK=2
  ENDIF
  CLSU=GFP_CUSS%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),1,MSTRTU)
  CLSU=GFP_CVSS%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),1,MSTRTV)
  IF (LSTRT.OR.LAMIP) THEN
    CLSU=GFP_CTUQ%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFDITQ)
    CLSU=GFP_CTUS%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFDITS)
  ENDIF
ENDIF

!        3.2.4 Flux de PLuie neige graupel et grele Convective 
!               (Convective rain snow graupel and hail)

MFPLCL=0
MFPLCN=0
MFCCQL=0
MFCCQN=0
MFPLCG=0
MFPLCH=0
IF (LFPLC.OR.LAMIP.OR.LFDUTP) THEN
  IF (LTFPLC) THEN
!   On each half level :
    IK=1
  ELSE
!   At the surface :
    IK=2
  ENDIF
  CLSU=GFP_CCP%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLCL)
  IF (LCONDWT) THEN
    CLSU='FL.COND.L.CONV  '
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFCCQL)
  ENDIF
  IF (LNEIGE) THEN
    CLSU=GFP_CCSF%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLCN)
    IF (LCONDWT) THEN
      CLSU='FL.COND.N.CONV  '
      CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFCCQN)
    ENDIF
  ENDIF
  IF (LFPLCG) THEN
    CLSU=GFP_CCSG%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLCG)
  ENDIF
  IF (LFPLCH) THEN
    CLSU=GFP_CCSH%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLCH)
  ENDIF
ENDIF

!        3.2.5 Flux de PLuie et neige Stratiforme (Stratiform rain and snow)

MFPLSL=0
MFPLSN=0
MFCSQL=0
MFCSQN=0
MFPLSG=0
MFPLSH=0
IF (LFPLS.OR.LAMIP.OR.LFDUTP) THEN
  IF (LTFPLS) THEN
!   On each half level :
    IK=1
  ELSE
!   At the surface :
    IK=2
  ENDIF
  CLSU=GFP_CLSP%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLSL)
  IF (LCONDWT) THEN
    CLSU='FL.COND.L.GECH  '
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFCSQL)
  ENDIF
  IF (LNEIGE) THEN
    CLSU=GFP_CLSS%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLSN)
    IF (LCONDWT) THEN
      CLSU='FL.COND.N.GECH  ' 
      CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFCSQN)
    ENDIF
  ENDIF
  IF (LFPLSG) THEN
    CLSU=GFP_CLSG%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLSG)
  ENDIF
  IF (LFPLSH) THEN
    CLSU=GFP_CLSH%CLNAME(5:)
    CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),0,MFPLSH)
  ENDIF
ENDIF

!        3.2.6 Fluxes of Radiation

MFRSO=0
MFRTH=0
IF (LFR.OR.LAMIP) THEN
  IF (LTFR) THEN
!   On each half level :
    IK=1
  ELSE
    IK=4
  ENDIF
  IPAR=0
  CLSU=GFP_CSSR%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MFRSO)
  CLSU=GFP_CSTR%CLNAME(5:)
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MFRTH)
ENDIF

!        3.2.7 NEBulosite (Cloudiness)  : as a 3D variable

MFNEB=0
IF (LTNEB) THEN
! On each full level :
  IK=3
  IPAR=0
  CLSU='NEBULOSI.CFU'
  CALL NEXTYPE3(CLPRE(1,IK),CLSU,INLEV(1,IK),ILEV(IK),IPAR,MFNEB)
ENDIF

!        3.2.8 NEBulosite Totale (Total Cloudiness) : as seen from bottom

MFNEBT=0
IF (LNEBTT.OR.LAMIP) THEN
  CALL NEXTYPE1(GFP_CTCC%CLNAME(1:4),GFP_CTCC%CLNAME(5:),0,MFNEBT)
ENDIF

!        3.2.9 Surface Fluxes

MFCLL=0
MFCLN=0
MFEVL=0
MFEVN=0
IF (LFSF) THEN
  CALL NEXTYPE1(GFP_CLHE%CLNAME(1:4),GFP_CLHE%CLNAME(5:),0,MFCLL)
  CALL NEXTYPE1(GFP_CLHS%CLNAME(1:4),GFP_CLHS%CLNAME(5:),0,MFCLN)
  CALL NEXTYPE1(GFP_CE%CLNAME(1:4),GFP_CE%CLNAME(5:),0,MFEVL)
  CALL NEXTYPE1(GFP_CS%CLNAME(1:4),GFP_CS%CLNAME(5:),0,MFEVN)
ENDIF

!        3.2.10 SOIL fluxes

MFONTE=0
MFCHSP=0
MFLWSP=0
MRUISS=0
MRUISP=0
MFEVV =0
MFTR  =0
MRUISL=0
MFGEL =0
MFGELS=0
IF (LFSOIL) THEN
  CALL NEXTYPE1(GFP_CFON%CLNAME(1:4),GFP_CFON%CLNAME(5:),0,MFONTE)
  CALL NEXTYPE1(GFP_CCHS%CLNAME(1:4),GFP_CCHS%CLNAME(5:),0,MFCHSP)
  CALL NEXTYPE1(GFP_CEAS%CLNAME(1:4),GFP_CEAS%CLNAME(5:),0,MFLWSP)
  CALL NEXTYPE1(GFP_CSRU%CLNAME(1:4),GFP_CSRU%CLNAME(5:),0,MRUISS)
  CALL NEXTYPE1(GFP_CDRU%CLNAME(1:4),GFP_CDRU%CLNAME(5:),0,MRUISP)
  IF (LSOLV) THEN
    CALL NEXTYPE1(GFP_CETP%CLNAME(1:4),GFP_CETP%CLNAME(5:),0,MFEVV )
    CALL NEXTYPE1(GFP_CTP%CLNAME(1:4),GFP_CTP%CLNAME(5:),0,MFTR  )
    CALL NEXTYPE1(GFP_CIRU%CLNAME(1:4),GFP_CIRU%CLNAME(5:),0,MRUISL)
    IF (LFGEL) THEN
      CALL NEXTYPE1('PROF','FLUX DE GEL     ',0,MFGEL)
      CALL NEXTYPE1('SURF','FLUX DE GEL     ',0,MFGELS)
    ENDIF
  ENDIF
ENDIF

!        3.2.11 Surface Fluxes or AMIP specific fluxes

MFCS=0
MWS   =0
MSNS  =0
MQTOT =0
MSPRES=0
MFCL  =0
MFDISH=0
MFTOPH=0
MQICE =0
MQLI  =0
MOZONT=0
IF (LFSF.OR.LAMIP) THEN
  CALL NEXTYPE1(GFP_CSSH%CLNAME(1:4),GFP_CSSH%CLNAME(5:),0,MFCS)
  IF (LAMIP) THEN
    CALL NEXTYPE1(GFP_CWS%CLNAME(1:4),GFP_CWS%CLNAME(5:),0,MWS   )
    CALL NEXTYPE1(GFP_CSNS%CLNAME(1:4),GFP_CSNS%CLNAME(5:),0,MSNS  )
    CALL NEXTYPE1(GFP_CQTO%CLNAME(1:4),GFP_CQTO%CLNAME(5:),0,MQTOT )
    CALL NEXTYPE1(GFP_CTSP%CLNAME(1:4),GFP_CTSP%CLNAME(5:),0,MSPRES)
    CALL NEXTYPE1(GFP_CSLH%CLNAME(1:4),GFP_CSLH%CLNAME(5:),0,MFCL  )
    CALL NEXTYPE1(GFP_CBLD%CLNAME(1:4),GFP_CBLD%CLNAME(5:),0,MFDISH)
    CALL NEXTYPE1(GFP_CTME%CLNAME(1:4),GFP_CTME%CLNAME(5:),0,MFTOPH)
    CALL NEXTYPE1(GFP_CICE%CLNAME(1:4),GFP_CICE%CLNAME(5:),0,MQICE )
    CALL NEXTYPE1(GFP_CLI%CLNAME(1:4),GFP_CLI%CLNAME(5:),0,MQLI  )
    IF (LOZONE) THEN
      CALL NEXTYPE1(GFP_CTO3%CLNAME(1:4),GFP_CTO3%CLNAME(5:),0,MOZONT)
    ENDIF
  ENDIF
ENDIF

!        3.2.12 FRequency for Radiation of Clear sky (top and bottom)

MFRSOC0=0
MFRTHC0=0
MFRSOC1=0
MFRTHC1=0
IF (LFRRC) THEN
  CALL NEXTYPE1(CLEVS(1),GFP_CSOC%CLNAME(5:),0,MFRSOC0)
  CALL NEXTYPE1B(CLEVS(1),GFP_CSOC%CLNAME(5:),0,MFRSOC1)
  CALL NEXTYPE1(CLEVS(1),GFP_CTHC%CLNAME(5:),0,MFRTHC0)
  CALL NEXTYPE1B(CLEVS(1),GFP_CTHC%CLNAME(5:),0,MFRTHC1)
ENDIF

!        3.2.13 RAYonnements (more dariative fluxes)

MFRSOPS=0
MFRSOPT=0
MFRSODS=0
MFRTHDS=0
IF (LRAYS) THEN
  CALL NEXTYPE1(GFP_CSOP%CLNAME(1:4),GFP_CSOP%CLNAME(5:),0,MFRSOPS)
  CALL NEXTYPE1(GFP_CTOP%CLNAME(1:3),GFP_CTOP%CLNAME(4:),0,MFRSOPT)
ENDIF
IF (LRAYD) THEN
  CALL NEXTYPE1(GFP_CSOD%CLNAME(1:4),GFP_CSOD%CLNAME(5:),0,MFRSODS)
  CALL NEXTYPE1(GFP_CTHD%CLNAME(1:4),GFP_CTHD%CLNAME(5:),0,MFRTHDS)
ENDIF

MFRSOLU=0
IF (LMOON) THEN
  CALL NEXTYPE1(GFP_CSMR%CLNAME(1:4),GFP_CSMR%CLNAME(5:),0,MFRSOLU)
ENDIF

!        3.2.14 NEBulosite PARtielles (partial cloudiness)

MNEBCON=0
MNEBHAU=0
MNEBMOY=0
MNEBBAS=0
IF (LNEBPAR) THEN
  CALL NEXTYPE1(GFP_CCCC%CLNAME(1:4),GFP_CCCC%CLNAME(5:),0,MNEBCON)
  CALL NEXTYPE1(GFP_CHCC%CLNAME(1:4),GFP_CHCC%CLNAME(5:),0,MNEBHAU)
  CALL NEXTYPE1(GFP_CMCC%CLNAME(1:4),GFP_CMCC%CLNAME(5:),0,MNEBMOY)
  CALL NEXTYPE1(GFP_CLCC%CLNAME(1:4),GFP_CLCC%CLNAME(5:),0,MNEBBAS)
ENDIF

!        3.2.16 Filtered duration of total precipitations

MFDUTP=0
IF (LFDUTP) THEN
  CALL NEXTYPE1(GFP_CDUTP%CLNAME(1:4),GFP_CDUTP%CLNAME(5:),0,MFDUTP)
ENDIF

!        3.4 Count fields

NTYPCFU=IP
NFDCFU=IADD-1
LCUMFU=(NFDCFU > 0)

!      -----------------------------------------------------------

!*       4.    PRINT FINAL VALUES.
!              -------------------

WRITE(UNIT=KULOUT,FMT='('' COMMON YOMCFU '')')
WRITE(UNIT=KULOUT,FMT='('' LCUMFU = '',L2,'' LREACFU = '',L2)') LCUMFU, LREACFU
WRITE(UNIT=KULOUT,FMT='('' NFDCFU = '',I6,'' NTYPCFU = '',I6)') NFDCFU, NTYPCFU
WRITE(UNIT=KULOUT,FMT='('' NFRCFU = '',I7)')NFRCFU
WRITE(KULOUT,*) ' NCFUTS =  ',NCFUTS(0),(NCFUTS(J),J=1,ABS(NCFUTS(0)))
IF (LCUMFU) THEN
  WRITE(UNIT=KULOUT,FMT='('' LSTRD  = '',L2,'' LSTRC  = '',L2 &
   & ,'' LSTRT  = '',L2,'' LFPLC  = '',L2,'' LFPLCG  = '',L2 &
   & ,'' LFPLCH = '',L2,'' LFPLS  = '',L2,'' LFPLSG  = '',L2 &
   & ,'' LFPLSH = '',L2,'' LFR    = '',L2,'' LFSOIL = '',L2 &
   & ,'' LNEBTT = '',L2,'' LFSF   = '',L2,'' LNEBPAR= '',L2 &
   & ,'' LAMIP  = '',L2,'' LRAYS  = '',L2,'' LFRRC  = '',L2 &
   & ,'' LFDUTP = '',L2,'' LMOON  = '',L2,'' LRAYD  = '',L2 &
   & )')&
   & LSTRD,LSTRC,LSTRT,LFPLC,LFPLCG,LFPLCH,LFPLS,LFPLSG,LFPLSH,&
   & LFR,LFSOIL,LNEBTT,LFSF,LNEBPAR,LAMIP,LRAYS,LFRRC,LFDUTP,LMOON,LRAYD
  WRITE(UNIT=KULOUT,FMT='('' LTSTRD  = '',L2,'' LTSTRC  = '',L2 &
   & ,'' LTSTRT  = '',L2,'' LTFPLC  = '',L2,'' LTFPLS  = '',L2 &
   & ,'' LTFR    = '',L2,'' LTNEB   = '',L2)')&
   & LTSTRD,LTSTRC,LTSTRT,LTFPLC,LTFPLS,LTFR,LTNEB  
  WRITE(UNIT=KULOUT,FMT='('' NFRRC = '',I8)') NFRRC
  WRITE(UNIT=KULOUT,FMT='('' MSTRDU = '',I2,'' MSTRDV = '',I2 &
   & ,'' MSTRCU = '',I2,'' MSTRCV = '',I2,'' MFDICQ = '',I2,'' MFDICS = '',I2 &
   & ,'' MSTRTU = '',I2,'' MSTRTV = '',I2,'' MFDITQ = '',I2,'' MFDITS = '',I2 &
   & ,'' MFPLCL = '',I2,'' MFPLSL = '',I2,'' MFPLCN = '',I2,'' MFPLSN = '',I2 &
   & ,'' MFCCQL = '',I2,'' MFCCQN = '',I2,'' MFCSQL = '',I2,'' MFCSQN = '',I2 &
   & ,'' MFRSO  = '',I2,'' MFRTH  = '',I2,'' MFNEB  = '',I2,'' MWS    = '',I2 &
   & ,'' MSNS   = '',I2,'' MQTOT  = '',I2,'' MFNEBT = '',I2,'' MOZONT = '',I2 &
   & ,'' MSPRES = '',I2,'' MFCL   = '',I2,'' MFCS   = '',I2,'' MFDISH = '',I2 &
   & ,'' MFTOPH = '',I2,'' MQICE  = '',I2,'' MQLI   = '',I2,'' MFRSOC0= '',I2 &
   & ,'' MFRTHC0= '',I2,'' MFRSOPS= '',I2,'' MFRSOPT= '',I2,'' MFRSOLU= '',I2 &
   & ,'' MFRSODS= '',I2,'' MFRTHDS= '',I2,'' MFCLL  = '',I2,'' MFCLN  = '',I2 &
   & ,'' MFEVL  = '',I2,'' MFEVN  = '',I2,'' MFONTE = '',I2,'' MFCHSP = '',I2 &
   & ,'' MFEVV  = '',I2,'' MFLWSP = '',I2,'' MFTR   = '',I2,'' MRUISL = '',I2 &
   & ,'' MRUISP = '',I2,'' MRUISS = '',I2,'' MNEBCON= '',I2,'' MNEBHAU= '',I2 &
   & ,'' MNEBMOY= '',I2,'' MFGEL  = '',I2,'' MFGELS = '',I2,'' MNEBBAS= '',I2 &
   & ,'' MFRSOC1= '',I2,'' MFRTHC1= '',I2,'' MFDUTP = '',I2 &
   & )') &
   & MSTRDU ,MSTRDV ,MSTRCU ,MSTRCV ,MFDICQ ,MFDICS ,MSTRTU ,MSTRTV ,&
   & MFDITQ ,MFDITS ,MFPLCL ,MFPLSL ,MFPLCN ,MFPLSN ,MFCCQL ,MFCCQN ,&
   & MFCSQL ,MFCSQN ,MFRSO  ,MFRTH  ,MFNEB  ,MWS    ,MSNS   ,MQTOT  ,&
   & MFNEBT ,MOZONT ,MSPRES ,MFCL   ,MFCS   ,MFDISH ,MFTOPH ,MQICE  ,&
   & MQLI   ,MFRSOC0,MFRTHC0,MFRSOPS,MFRSOPT,MFRSOLU,MFRSODS,MFRTHDS,&
   & MFCLL  ,MFCLN  ,MFEVL  ,MFEVN  ,MFONTE ,MFCHSP ,MFEVV  ,MFLWSP ,&
   & MFTR   ,MRUISL ,MRUISP ,MRUISS ,MNEBCON,MNEBHAU,MNEBMOY,MFGEL  ,&
   & MFGELS ,MNEBBAS,MFRSOC1,MFRTHC1,MFDUTP
  WRITE(UNIT=KULOUT,&
   & FMT='('' THE FOLLOWING CUMULATED FLUXES WILL BE HANDLED : '')')  
  DO JF=1,NTYPCFU
    DO JL=1,TYPE_CFU(JF)%ILEV
      IF (TYPE_CFU(JF)%CLPREF(JL) == CLEVS(1)) THEN
        WRITE(UNIT=KULOUT,FMT='(A1,I3.3,A12,10X, ''PARITY = '',I1)') &
         & TRIM(TYPE_CFU(JF)%CLPREF(JL)),TYPE_CFU(JF)%INUMLEV(JL),&
         & TYPE_CFU(JF)%CLSUFF,TYPE_CFU(JF)%IPARITY  
      ELSE
        WRITE(UNIT=KULOUT,FMT='(A16,10X, ''PARITY = '',I1)') &
         & TRIM(TYPE_CFU(JF)%CLPREF(JL))//TYPE_CFU(JF)%CLSUFF,&
         & TYPE_CFU(JF)%IPARITY  
      ENDIF
    ENDDO
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('SUCFU',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CONTAINS

SUBROUTINE NEXTYPE3(CDPREF,CDSUFF,KNLEV,KLEV,KPAR,KX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KLEV,KPAR
INTEGER(KIND=JPIM), INTENT(IN)  :: KNLEV(KLEV)
CHARACTER(LEN=*),   INTENT(IN)  :: CDPREF(KLEV)
CHARACTER(LEN=*),   INTENT(IN)  :: CDSUFF
INTEGER(KIND=JPIM), INTENT(OUT) :: KX

INTEGER(KIND=JPIM) :: J, ILEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUCFU:NEXTYPE3',0,ZHOOK_HANDLE)

IP=IP+1
DO J=1,KLEV
  ILEN=MIN(4,LEN(CDPREF(J)))
  TYPE_CFU(IP)%CLPREF(J)=CDPREF(J)(1:ILEN)
  TYPE_CFU(IP)%INUMLEV(J)=KNLEV(J)
ENDDO
ILEN=MIN(16,LEN(CDSUFF))
TYPE_CFU(IP)%CLSUFF=CDSUFF(1:ILEN)
TYPE_CFU(IP)%ILEV=KLEV
TYPE_CFU(IP)%IPARITY=KPAR
KX=IADD
IADD=IADD+KLEV

IF (LHOOK) CALL DR_HOOK('SUCFU:NEXTYPE3',1,ZHOOK_HANDLE)

END SUBROUTINE NEXTYPE3

SUBROUTINE NEXTYPE1(CDPREF,CDSUFF,KPAR,KX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KPAR
CHARACTER(LEN=*),   INTENT(IN)  :: CDPREF
CHARACTER(LEN=*),   INTENT(IN)  :: CDSUFF
INTEGER(KIND=JPIM), INTENT(OUT) :: KX

INTEGER(KIND=JPIM) :: ILEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUCFU:NEXTYPE1',0,ZHOOK_HANDLE)

IP=IP+1
ILEN=MIN(4,LEN(CDPREF))
TYPE_CFU(IP)%CLPREF(1)=CDPREF(1:ILEN)
TYPE_CFU(IP)%INUMLEV(1)=1
ILEN=MIN(16,LEN(CDSUFF))
TYPE_CFU(IP)%CLSUFF=CDSUFF(1:ILEN)
TYPE_CFU(IP)%ILEV=1
TYPE_CFU(IP)%IPARITY=KPAR
KX=IADD
IADD=IADD+1

IF (LHOOK) CALL DR_HOOK('SUCFU:NEXTYPE1',1,ZHOOK_HANDLE)

END SUBROUTINE NEXTYPE1

SUBROUTINE NEXTYPE1B(CDPREF,CDSUFF,KPAR,KX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KPAR
CHARACTER(LEN=*),   INTENT(IN)  :: CDPREF
CHARACTER(LEN=*),   INTENT(IN)  :: CDSUFF
INTEGER(KIND=JPIM), INTENT(OUT) :: KX

INTEGER(KIND=JPIM) :: ILEN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUCFU:NEXTYPE1B',0,ZHOOK_HANDLE)

IP=IP+1
ILEN=MIN(4,LEN(CDPREF))
TYPE_CFU(IP)%CLPREF(1)=CDPREF(1:ILEN)
TYPE_CFU(IP)%INUMLEV(1)=NFLEVG
ILEN=MIN(16,LEN(CDSUFF))
TYPE_CFU(IP)%CLSUFF=CDSUFF(1:ILEN)
TYPE_CFU(IP)%ILEV=1
TYPE_CFU(IP)%IPARITY=KPAR
KX=IADD
IADD=IADD+1

IF (LHOOK) CALL DR_HOOK('SUCFU:NEXTYPE1B',1,ZHOOK_HANDLE)

END SUBROUTINE NEXTYPE1B

END SUBROUTINE SUCFU
