SUBROUTINE SUCMAHFP

!**** SUBROUTINE SUCMAHFP - CMA HEADER (FIXED PART) 
!                           PACKED INFO. FORMAT

!        D. VASILJEVIC     ECMWF    30/08/1994

!**   PURPOSE
!     -------

!        SET CMA HEADER (FIXED PART) PACKED INFO. FORMAT

!**   INTERFACE
!     ---------

!        CALL SUCMAHFP

!        SUCMAHFP IS CALLED BY SUBROUTINE SUCMAHF

!**   METHOD
!     ------

!        CMA HEADER (FIXED PART) PACKED INFO. FORMAT IS DEFINED.

!**   EXTERNALS
!     ---------

!        NONE

!**   MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCMHDR , ONLY : NCDTBP   ,NCDTOC   ,NINSTBP  ,NINSTOC  ,&
 & NRETTBP  ,NRETTOC  ,NARETBP  ,NARETOC  ,NYYOFF   ,&
 & NMMOFF   ,NDDOFF   ,NHHOFF   ,NMIOFF   ,NSSOFF   ,&
 & NLTFLBP  ,NLTFLOC  ,NLNFLBP  ,NLNFLOC  ,NDTFLBP  ,&
 & NDTFLOC  ,NTMFLBP  ,NTMFLOC  ,NALFLBP  ,NALFLOC  ,&
 & NHMSBP   ,NHMSOC   ,NQCSBP   ,NQCSOC   ,NORFBP   ,&
 & NORFOC   ,NFLGBP   ,NFLGOC   ,NHFSBP   ,NHFSOC   ,&
 & NCMRACBP ,NCMRACOC ,NCMRPABP ,NCMRPAOC ,NCMRREBP ,&
 & NCMRREOC ,NCMRBLBP ,NCMRBLOC  

IMPLICIT NONE
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------
!*

!        1.        OBSERVATION CHARACTERISTICS
!                  ---------------------------
!                  THIS IS PACKED WORD WITH THE FOLLOWING FORMAT:
!                     BITS  0- 9 OBS. CODE TYPE (10 BITS)
!                     BITS 10-19 OBS. INSTRUMENT TYPE (10 BITS)
!                     BITS 20-25 RETRIEVAL TYPE (6 BITS)
!                     BITS 26-31 GEOGRAPHICAL AREA (6 BITS)

!*          1.1    OBS. CODE TYPE PACKING BIT PATTERN

IF (LHOOK) CALL DR_HOOK('SUCMAHFP',0,ZHOOK_HANDLE)
NCDTBP  =   0
NCDTOC  =  10

!*          1.2    OBS. INSTRUMENT TYPE PACKING BIT PATTERN

NINSTBP =  10
NINSTOC =  10

!*          1.3    OBS. RETRIEVAL TYPE PACKING BIT PATTERN

NRETTBP =  20
NRETTOC =   6

!*          1.4    OBS. GEOGRAPHICAL AREA TYPE PACKING BIT PATTERN

NARETBP =  26
NARETOC =   6
!*
!-----------------------------------------------------------------------

!        2.       SYNOPTIC DATE/TIME AND EXACT TIME
!                 ---------------------------------
!                 THE FOLLOWING FORMAT IS USED FOR DATE:
!                 YYYYMMDD = YYYY*10**4+MM*10**2+DD
!                 YYYY - YEAR
!                 MM   - MONTH
!                 DD   - DAY
!                 THE FOLLOWING FORMAT IS USED FOR TIME:
!                 HHMMSS = HH*10**4+MM*10**2+SS
!                 HH - HOUR
!                 MM - MINUTE
!                 SS - SECONDE

!*          2.1    YEAR OFFSET

NYYOFF  =  10**4

!*          2.2    MONTH OFFSET

NMMOFF  =  10**2

!*          2.3    DAY OFFSET

NDDOFF  =  10**0

!*          2.4    EXACT HOUR OFFSET

NHHOFF =  10**4

!*          2.5    EXACT MINUTE OFFSET

NMIOFF =  10**2

!*          2.6    EXACT SECONDE OFFSET

NSSOFF =  10**0
!*
!-----------------------------------------------------------------------

!        3.       REPORT'S FLAGS
!                 --------------
!                  FLAGS ARE ON LAT, LON, DATE, TIME AND ALT. FOR
!                  PARAMETER THERE IS 6 BIT STRUCTURE WITH THE 
!                  FOLLOWING FORMAT:
!                      BIT    0 - 1 HUMAN MONITOR SUBSTITUTION
!                               - 0 NO HUMAN MONITOR SUBSTITUTION
!                      BIT    1 - 1 Q/C SUBSTITUTION
!                               - 0 NO Q/C SUBSTITUTION
!                      BIT    2 - 1 OVER-RIDE FLAG IS SET
!                               - 0 OVER-RIDE FLAG IS NOT SET
!                      BITS 3-4 - 0 PARAMETER IS CORRECT
!                               - 1 PARAM IS PROBABLY CORRECT
!                               - 2 PARAM IS PROBABLY INCORRECT
!                               - 3 PARAM IS INCORRECT
!                      BIT    5 - 1 PARM FLAG SET BY HUMAN MONITOR
!                               - 0 PARAM FLAG SET BY Q/C PROGRAM
!                                   OR NOT CHECKED

!*          3.1    LATITUDE FLAGS BIT POS. AND NO. OF BITS TAKEN

NLTFLBP =   0
NLTFLOC =   6

!*          3.2    LONGITUDE FLAGS BIT POS. AND NO. OF BITS TAKEN

NLNFLBP =   6
NLNFLOC =   6

!*          3.3    DATE FLAGS BIT POS. AND NO. OF BITS TAKEN

NDTFLBP =  12
NDTFLOC =   6

!*          3.4    TIME FLAGS BIT POS. AND NO. OF BITS TAKEN

NTMFLBP =  18
NTMFLOC =   6

!*          3.5    ALTITUDE FLAGS BIT POS. AND NO. OF BITS TAKEN

NALFLBP =  24
NALFLOC =   6

!*          3.6    FLAGS BIT PATTERN

!                      BIT    0 - 1 HUMAN MONITOR SUBSTITUTION
!                               - 0 NO HUMAN MONITOR SUBSTITUTION
!                      BIT    1 - 1 Q/C SUBSTITUTION
!                               - 0 NO Q/C SUBSTITUTION
!                      BIT    2 - 1 OVER-RIDE FLAG IS SET
!                               - 0 OVER-RIDE FLAG IS NOT SET
!                      BITS 3-4 - 0 PARAMETER IS CORRECT
!                               - 1 PARAM IS PROBABLY CORRECT
!                               - 2 PARAM IS PROBABLY INCORRECT
!                               - 3 PARAM IS INCORRECT
!                      BIT    5 - 1 PARM FLAG SET BY HUMAN MONITOR
!                               - 0 PARAM FLAG SET BY Q/C PROGRAM
!                                   OR NOT CHECKED

!*          3.6.1  HUMAN MONITOR SUBSTITUTION BIT POS. AND BITS OCC.

NHMSBP  =   0
NHMSOC  =   1

!*          3.6.2  Q/C SUBSTITUTION BIT POS. AND BITS OCC.

NQCSBP  =   1
NQCSOC  =   1

!*          3.6.3  OVER-RIDE FLAG BIT POS. AND BITS OCC.

NORFBP  =   2
NORFOC  =   1

!*          3.6.4  PARAMETER FLAG BIT POS. AND BITS OCC.

NFLGBP  =   3
NFLGOC  =   2

!*          3.6.5  PARAMETER FLAG FLAG SET BY H/QC POS. AND BITS OCC.

NHFSBP  =   5
NHFSOC  =   1
!*
!-----------------------------------------------------------------------

!        4.       REPORT'S STATUS
!                 ---------------
!                  BIT STRING STRUCTURE:
!                      BIT  0 - IF SET TO 1 REPORT IS ACTIVE
!                      BIT  1 - IF SET TO 1 REPORT IS PASSIVE
!                      BIT  2 - IF SET TO 1 REPORT IS REJECTED
!                      BIT  3 - IF SET TO 1 REPORT IS BLACKLISTED

!*          4.1    ACTIVE REPORT BIT POS. AND NO. OF BITS TAKEN

NCMRACBP=   0
NCMRACOC=   1

!*          4.2    PASSIVE REPORT BIT POS. AND NO. OF BITS TAKEN

NCMRPABP=   1
NCMRPAOC=   1

!*          4.3    REJECTED REPORT BIT POS. AND NO. OF BITS TAKEN

NCMRREBP=   2
NCMRREOC=   1

!*          4.4    BLACKLISTED REPORT BIT POS. AND NO. OF BITS TAKEN

NCMRBLBP=   3
NCMRBLOC=   1
!*
!-----------------------------------------------------------------------

!        5.       REPORT'S EVENTS (PART 1)
!                 ------------------------
!                 STRUCTURE OF EVENTS WORD IS SUCH THAT A PARTICULAR
!                 EVENT IS GIVEN BY ITS POSITION IN THE BIT STRING

!*
!-----------------------------------------------------------------------

!        6.       REPORT'S BLACKLIST EVENTS
!                 -------------------------
!                 STRUCTURE OF BLACKLIST EVENTS WORD IS SUCH THAT A
!                 PARTICULAR EVENT IS GIVEN BY ITS POSITION IN THE BIT
!                 STRING

!*
!-----------------------------------------------------------------------

!        7.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SUCMAHFP',1,ZHOOK_HANDLE)
END SUBROUTINE SUCMAHFP
