SUBROUTINE SUCMOCTP(KOUT)

!**** SUBROUTINE SUCMOCTP - CMA OBSERVATION AND CODE TYPE

!        D. VASILJEVIC    ECMWF    01/09/1994

!**   PURPOSE
!     -------

!        SET CMA OBSERVATION AND CODE TYPES

!**   INTERFACE
!     ---------

!        CALL SUCMOCTP(KOUT)
!     WHERE:
!            KOUT = OUTPUT UNIT

!        SUCMOCTP IS CALLED BY

!**   METHOD
!     ------

!        CMA OBS. AND CODE TYPES ARE DEFINED.
!        NOTE that CODE TYPES have to be no bigger than 511.

!**   EXTERNALS
!     ---------

!        NONE

!**   MODIFICATIONS
!     -------------
!     10-09-1998 : JN Thepaut : initialisation of the french SYNOR
!     02-11-2000 : E Andersson: Descriptive char. strings for Jo-table.
!     14-Mar-05  : D. Tan: Introduce Aeolus information
!     23-12-2004 : RARAD becomes LIMB; new codetypes for LIMB
!     27-04-2009 : J Muñoz Sabater: new obs and code types for SMOS data
!     31-03-2010 : A Fouilloux: new codetype for MERIS
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARCMA   , ONLY : JPMXOTP
USE YOMCMA   , ONLY : NMXOTP   ,NMXOCT
USE YOMCOCTP , ONLY : NOOBST   ,NOCODT   ,NSYNOP   ,NSYNOPSQ ,&
 & NSYNCT   ,NSRSCD   ,NSRSCDSQ ,NATSCD   ,NATSCDSQ ,&
 & NSHSCD   ,NSHSCDSQ ,NABSCD   ,NABSCDSQ ,NSHRED   ,&
 & NSHREDSQ ,NATSHS   ,NATSHSSQ ,&
 & NMETAR   ,NMETARSQ ,NPGPS    ,NPGPSSQ  ,NAIREP   ,NAIREPSQ ,&
 & NAIRCT   ,NAIRCD   ,NAIRCDSQ ,NCODAR   ,NCODARSQ ,&
 & NCOLBA   ,NCOLBASQ ,NAMDAR   ,NAMDARSQ ,NACARS   ,&
 & NACARSSQ ,NSIMAI   ,NSIMAISQ ,NSATOB   ,NSATOBSQ ,&
 & NSABCT   ,NSTBCD   ,NSTBCDSQ ,NSTB86   ,NSTB86SQ ,&
 & NSTB87   ,NSTB87SQ ,NSYRCD   ,NSYRCDSQ ,&
 & NSST     ,NSSTSQ   ,NDRIBU   ,NDRIBUSQ ,NDRICT   ,&
 & NDRBCD   ,NDRBCDSQ ,NBATHY   ,NBATHYSQ ,NTESAC   ,&
 & NTESACSQ ,NDERS1   ,NDERS1SQ ,NTEMP    ,NTEMPSQ  ,&
 & NTEMCT   ,NLDTCD   ,NLDTCDSQ ,NSHTCD   ,NSHTCDSQ ,&
 & NTDROP   ,NTDROPSQ ,NROCOB   ,NROCOBSQ ,NROCSH   ,&
 & NROCSHSQ ,NMBTMP   ,NMBTMPSQ ,NSIMTE   ,NSIMTESQ ,&
 & NPILOT   ,NPILOTSQ ,NPILCT   ,NLDPCD   ,NLDPCDSQ ,&
 & NSHPCD   ,NSHPCDSQ ,NWPPCD   ,NWPPCDSQ ,&
 & NDWLPCD  ,NDWLPCDSQ,NSATEM   ,&
 & NSATEMSQ ,NSAMCT   ,NSTMCD   ,NSTMCDSQ ,NSTOVS   ,&
 & NSTOVSSQ ,NSTRO3   ,NSTRO3SQ ,&
 & NSTDWL   ,NSTDWLSQ ,&
 & NSTTOV   ,NSTTOVSQ ,NGTSTB   ,NGTSTBSQ ,NGTST1   ,&
 & NGTST1SQ ,NGTST2   ,   &
 & NGTST2SQ ,NGTHRB   ,NGTHRBSQ ,NGTHR1   ,NGTHR1SQ ,&
 & NGTHR2   ,NGTHR2SQ ,NSSMI    ,NSSMISQ  ,NPAOB    ,&
 & NPAOBSQ  ,NPAOCT   ,NPABCD   ,NPABCDSQ ,NSCATT   ,&
 & NSCATTSQ ,NSCACT   ,NSCAT1   ,NSCAT1SQ ,NSCAT2   ,&
 & NSCAT2SQ ,NSCAT3   ,NSCAT3SQ ,NLIMB    ,NLIMBSQ  ,&
 & NLIMBCT  ,NSCAT4   ,NSCAT4SQ ,&
 & NSCAT5   ,NSCAT5SQ ,NSCAT6   ,NSCAT6SQ ,NEWPPCD  ,&
 & NEWPPCDSQ,NJWPPCD  ,NJWPPCDSQ,&
 & CCMOOTP  ,CCMOCTP  ,MSQBYCTP ,MNOCTP   ,&
 & NRADOH   ,NRADOHSQ ,NTYPHO   ,NTYPHOSQ ,          &
 & NGPSRO   ,NGPSROSQ ,NLRAD    ,NLRADSQ            ,&
 & NRADAR   ,NRADARSQ ,NRDRCT   ,NRADA1   ,NRADA1SQ ,&
 & NSMOS    ,NSMOSSQ, NTCWC, NTCWCSQ

!*

!        1.        CMA OBSERVATION AND CODE TYPES AND ASSOCIATED
!                  ---------------------------------------------
!                  SEQUENCE NO.
!                  ------------

!*          0.1    PRESET

!*          0.1.1  NO. OF CMA OBS. TYPES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOUT 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "abortdb.h"
END INTERFACE

IF (LHOOK) CALL DR_HOOK('SUCMOCTP',0,ZHOOK_HANDLE)
NOOBST   =  JPMXOTP
IF(NOOBST > NMXOTP)           THEN
  WRITE(KOUT,'(''0ERROR IN SUBROUTINE SUCMOCTP'')')
  WRITE(KOUT,'('' NO. OF CMA OBS. TYPES TOO LARGE'')')
  WRITE(KOUT,'('' NO. OF CMA OBS. TYPES    :'',I10)') NOOBST
  WRITE(KOUT,'('' MAX NO. OF CMA OBS. TYPES:'',I10)') NMXOTP
  CALL ABORTDB('SUCMOCTP','NO. OF CMA OBS. TYPES TOO LARGE',NOOBST)
ENDIF

!*          0.1.2  NO. OF CMA OBS. CODE TYPES

NOCODT   =  12
IF(NOCODT > NMXOCT)           THEN
  WRITE(KOUT,'(''0ERROR IN SUBROUTINE SUCMOCTP'')')
  WRITE(KOUT,'('' NO. OF CMA CODE TYPES TOO LARGE'')')
  WRITE(KOUT,'('' NO. OF CMA CODE TYPES    :'',I10)') NOCODT
  WRITE(KOUT,'('' MAX NO. OF CMA CODE TYPES:'',I10)') NMXOCT
  CALL ABORTDB('SUCMOCTP','NO. OF CMA CODE TYPES TOO LARGE',NOCODT)
ENDIF

MSQBYCTP(:)=0
CCMOOTP(:)='                                '
CCMOCTP(:)='                                '
MNOCTP(:)=1
!*          1.1    SYNOP

NSYNOP   =   1
NSYNOPSQ =   1
NSYNCT   =   10
MNOCTP(NSYNOP)=NSYNCT
CCMOOTP(NSYNOP)='SYNOP, Land stations and ships  '

!*          1.1.1  SYNOP LAND REPORT

NSRSCD   =  11
NSRSCDSQ =   1
MSQBYCTP(NSRSCD)=NSRSCDSQ
CCMOCTP(NSRSCD)='SYNOP Land Manual Report               '

!*          1.1.2  SYNOP LAND AUTOMATIC REPORT

NATSCD   =  14
NATSCDSQ =   2
MSQBYCTP(NATSCD)=NATSCDSQ
CCMOCTP(NATSCD)='SYNOP Land Automatic Report     '

!*          1.1.3  SYNOP-SHIP REPORT

NSHSCD   =  21
NSHSCDSQ =   3
MSQBYCTP(NSHSCD)=NSHSCDSQ
CCMOCTP(NSHSCD)='SYNOP-SHIP Report               '

!*          1.1.4  SYNOP-SHIP ABBREVIATED REPORT

NABSCD   =  22
NABSCDSQ =   4
MSQBYCTP(NABSCD)=NABSCDSQ
CCMOCTP(NABSCD)='SYNOP-SHIP Abbreviated Report   '

!*          1.1.5  SYNOP SHRED REPORT

NSHRED   =  23
NSHREDSQ =   5
MSQBYCTP(NSHRED)=NSHREDSQ
CCMOCTP(NSHRED)='SYNOP SHRED Report              '

!*          1.1.6  SYNOP AUTOMATIC SHIP REPORT

NATSHS   =  24
NATSHSSQ =   6
MSQBYCTP(NATSHS)=NATSHSSQ
CCMOCTP(NATSHS)='SYNOP Automatic SHIP Report     '

!*          1.1.7  FRENCH SYNOP AUTOMATIC LAND REPORT

NSYRCD   =  15
NSYRCDSQ =  7
MSQBYCTP(NSYRCD)=NSYRCDSQ
CCMOCTP(NSYRCD)='French SYNOP Automatic Land Repo'

!*          1.1.8  FRENCH RADOME

NRADOH   =  16
NRADOHSQ =  8
MSQBYCTP(NRADOH)=NRADOHSQ
CCMOCTP(NRADOH)='French RADOME                   '

!*          1.1.9  SYNOP METAR

NMETAR   =  140
NMETARSQ =   9
MSQBYCTP(NMETAR)=NMETARSQ
CCMOCTP(NMETAR)='SYNOP METAR                     '

!*          1.1.10 Ground-based GPS

NPGPS    =  110
NPGPSSQ  =   10
MSQBYCTP(NPGPS)=NPGPSSQ
CCMOCTP(NPGPS)='Ground-based GPS                '

!*          1.2    AIREP

NAIREP   =   2
NAIREPSQ =   2
NAIRCT   =   6
MNOCTP(NAIREP)=NAIRCT
CCMOOTP(NAIREP)='AIREP, Aircraft data            '

!*          1.2.1  AIRCRAFT REPORT

NAIRCD   = 141
NAIRCDSQ =   1
MSQBYCTP(NAIRCD)=NAIRCDSQ
CCMOCTP(NAIRCD)='AIREP Aircraft Report           '

!*          1.2.2  CODAR REPORT

NCODAR   =  41
NCODARSQ =   2
MSQBYCTP(NCODAR)=NCODARSQ
CCMOCTP(NCODAR)='CODAR Aircraft Report           '

!*          1.2.3  COLBA REPORT

NCOLBA   = 241
NCOLBASQ =   3
MSQBYCTP(NCOLBA)=NCOLBASQ
CCMOCTP(NCOLBA)='COLBA Aircraft Report           '

!*          1.2.4  AMDAR AIRCRAFT REPORT

NAMDAR   = 144
NAMDARSQ =   4
MSQBYCTP(NAMDAR)=NAMDARSQ
CCMOCTP(NAMDAR)='AMDAR Aircraft Report           '

!*          1.2.5  ACARS AIRCRAFT REPORT

NACARS   = 145
NACARSSQ =   5
MSQBYCTP(NACARS)=NACARSSQ
CCMOCTP(NACARS)='ACARS Aircraft Report           '

!*          1.2.6  SIMULATED AIREP REPORT

NSIMAI   = 142
NSIMAISQ =   6
MSQBYCTP(NSIMAI)=NSIMAISQ
CCMOCTP(NSIMAI)='Simulated AIREP Report          '

!*          1.3    SATOB

NSATOB   =   3
NSATOBSQ =   3
NSABCT   =   4
MNOCTP(NSATOB)=NSABCT
CCMOOTP(NSATOB)='SATOB, Atmospheric motion winds '

!*          1.3.1  SATOB REPORT

NSTBCD   =  88
NSTBCDSQ =   1
MSQBYCTP(NSTBCD)=NSTBCDSQ
CCMOCTP(NSTBCD)='SATOB REPORT                    '

!*          1.3.2  HIGH RES. VIS WIND SATOB REPORT

NSTB86   =  89
NSTB86SQ =   2
MSQBYCTP(NSTB86)=NSTB86SQ
CCMOCTP(NSTB86)='High Res. Vis Wind SATOB Report '

!*          1.3.3  SST REPORT

NSST     = 188
NSSTSQ   =   3
MSQBYCTP(NSST)=NSSTSQ
CCMOCTP(NSST)='SST Report                      '

!*          1.3.4  AMV SATOB REPORT

NSTB87   =  90
NSTB87SQ =   4
MSQBYCTP(NSTB87)=NSTB87SQ
CCMOCTP(NSTB87)='SATOB Atmospheric Motion Vector '

!*          1.4    DRIBU

NDRIBU   =   4
NDRIBUSQ =   4
NDRICT   =   4
MNOCTP(NDRIBU)=NDRICT
CCMOOTP(NDRIBU)='DRIBU, Drifing Buoys            '

!*          1.4.1  DRIBU REPORT

NDRBCD   = 165
NDRBCDSQ =   1
MSQBYCTP(NDRBCD)=NDRBCDSQ
CCMOCTP(NDRBCD)='DRIBU Buoy Report               '

!*          1.4.2  BATHY REPORT

NBATHY   =  63
NBATHYSQ =   2
MSQBYCTP(NBATHY)=NBATHYSQ
CCMOCTP(NBATHY)='BATHY Buoy Report               '

!*          1.4.3  TESAC REPORT

NTESAC   =  64
NTESACSQ =   3
MSQBYCTP(NTESAC)=NTESACSQ
CCMOCTP(NTESAC)='TESAC Buoy Report               '

!*          1.4.4  ERS1 AS DRIBU

NDERS1   = 160
NDERS1SQ =   4
MSQBYCTP(NDERS1)=NDERS1SQ
CCMOCTP(NDERS1)='ERS1 as DRIBU                   '

!*          1.5    TEMP

NTEMP    =   5
NTEMPSQ  =   5
NTEMCT   =   7
MNOCTP(NTEMP)=NTEMCT
CCMOOTP(NTEMP )='TEMP,  Radiosondes              '

!*          1.5.1  LAND TEMP REPORT

NLDTCD   =  35
NLDTCDSQ =   1
MSQBYCTP(NLDTCD)=NLDTCDSQ
CCMOCTP(NLDTCD)='TEMP Land Report                '

!*          1.5.2  TEMP SHIP REPORT

NSHTCD   =  36
NSHTCDSQ =   2
MSQBYCTP(NSHTCD)=NSHTCDSQ
CCMOCTP(NSHTCD)='TEMP SHIP Report                '

!*          1.5.3  TEMP-DROP REPORT

NTDROP   = 135
NTDROPSQ =   3
MSQBYCTP(NTDROP)=NTDROPSQ
CCMOCTP(NTDROP)='TEMP Dropsonde Report           '

!*          1.5.4  ROCOB REPORT

NROCOB   =  39
NROCOBSQ =   4
MSQBYCTP(NROCOB)=NROCOBSQ
CCMOCTP(NROCOB)='ROCOB TEMP Rocket Report        '

!*          1.5.5  ROCOB SHIP REPORT

NROCSH   =  40
NROCSHSQ =   5
MSQBYCTP(NROCSH)=NROCSHSQ
CCMOCTP(NROCSH)='ROCOB SHIP Rocket Report        '

!*          1.5.6  MOBILE TEMP REPORT

NMBTMP   =  37
NMBTMPSQ =   6
MSQBYCTP(NMBTMP)=NMBTMPSQ
CCMOCTP(NMBTMP)='Mobile TEMP Report              '

!*          1.5.7  SIMULATED TEMP STATION CODE TYPE

NSIMTE   = 137
NSIMTESQ =   7
MSQBYCTP(NSIMTE)=NSIMTESQ
CCMOCTP(NSIMTE)='Simulated TEMP Station Report   '

!*          1.6    PILOT

NPILOT   =   6
NPILOTSQ =   6
NPILCT   =   6
MNOCTP(NPILOT)=NPILCT
CCMOOTP(NPILOT)='PILOT, Balloons and profilers   '

!*          1.6.1  LAND PILOT REPORT

NLDPCD   =  32
NLDPCDSQ =   1
MSQBYCTP(NLDPCD)=NLDPCDSQ
CCMOCTP(NLDPCD)='PILOT Land Report               '

!*          1.6.2  SHIP PILOT REPORT

NSHPCD   =  33
NSHPCDSQ =   2
MSQBYCTP(NSHPCD)=NSHPCDSQ
CCMOCTP(NSHPCD)='PILOT SHIP Report               '

!*          1.6.3  AMERICAN WIND PROFILERS

NWPPCD   =  34
NWPPCDSQ =   3
MSQBYCTP(NWPPCD)=NWPPCDSQ
CCMOCTP(NWPPCD)='American Wind Profiler          '

!*          1.6.4  EUROPEAN WIND PROFILERS

NEWPPCD  =  134
NEWPPCDSQ=   4
MSQBYCTP(NEWPPCD)=NEWPPCDSQ
CCMOCTP(NEWPPCD)='European Wind Profiler          '

!*          1.6.5  JAPANESE WIND PROFILERS

NJWPPCD  =  131
NJWPPCDSQ=   5
MSQBYCTP(NJWPPCD)=NJWPPCDSQ
CCMOCTP(NJWPPCD)='Japanese Wind Profiler          '

!*          1.6.6a  MOBILE WIND PILOT - OBSOLETE

!NMBPLT  =  132
!NMBPLTSQ=   6

!*          1.6.6b  DOPPLER WIND LIDAR

NDWLPCD  =  187
NDWLPCDSQ=   6
MSQBYCTP(NDWLPCD)=NDWLPCDSQ
CCMOCTP(NDWLPCD)='Doppler Wind Lidar              '

!*          1.7    SATEM

NSATEM   =   7
NSATEMSQ =   7
NSAMCT   =  13
MNOCTP(NSATEM)=NSAMCT
CCMOOTP(NSATEM)='SATEM, Satellite sounding data  '

!*          1.7.1  SATEM REPORT

NSTMCD   =  86
NSTMCDSQ =   1
MSQBYCTP(NSTMCD)=NSTMCDSQ
CCMOCTP(NSTMCD)='SATEM REPORT                    '

!*          1.7.2  HIGH RESOLUTION SATTELITE REPORT

NSTOVS   = 186
NSTOVSSQ =   2
MSQBYCTP(NSTOVS)=NSTOVSSQ
CCMOCTP(NSTOVS)='High Resolution Sattelite Report'

!*          1.7.3  HIGH RESOLUTION SIMULATED DWL SATELLITE REPORT

NSTDWL   = 185
NSTDWLSQ =   3
MSQBYCTP(NSTDWL)=NSTDWLSQ
CCMOCTP(NSTDWL)='High Resolution Simulated DWL Re'

!*          1.7.4  HIGH RESOLUTION SIMULATED TOVS SATELLITE REPORT

NSTTOV   = 184
NSTTOVSQ =   4
MSQBYCTP(NSTTOV)=NSTTOVSQ
CCMOCTP(NSTTOV)='High Resolution Simulated TOVS R'

!*          1.7.5  GTS BUFR SATEM REPORT
!                  (250 KM TEMP AND HUM PROFILES)

NGTSTB   = 200
NGTSTBSQ =   5
MSQBYCTP(NGTSTB)=NGTSTBSQ
CCMOCTP(NGTSTB)='GTS BUFR 250 KM SATEM Report    '

!*          1.7.6  GTS BUFR SATEM CLEAR RADIANCE REPORT

NGTST1   = 201
NGTST1SQ =   6
MSQBYCTP(NGTST1)=NGTST1SQ
CCMOCTP(NGTST1)='GTS BUFR SATEM Clear Radiance Re'

!*          1.7.7  GTS BUFR SATEM RETR PROFILES AND
!                  CLEAR RADIANCES REPORT

NGTST2   = 202
NGTST2SQ =   7
MSQBYCTP(NGTST2)=NGTST2SQ
CCMOCTP(NGTST2)='GTS BUFR Retr. Prof. and Radianc'

!*          1.7.8  HIGH RES 80 KM BUFR NON GTS SATEM RETR PROFILES

NGTHRB   = 210
NGTHRBSQ =   8
MSQBYCTP(NGTHRB)=NGTHRBSQ
CCMOCTP(NGTHRB)='Level 1C Calibrated Radiances   '

!*          1.7.9  HIGH RES 80 KM BUFR NON GTS SATEM CLEAR
!                  RADIANCE REPORT

NGTHR1   = 211
NGTHR1SQ =   9
MSQBYCTP(NGTHR1)=NGTHR1SQ
CCMOCTP(NGTHR1)='Rtovs Clear Radiances and Retr. '

!*          1.7.10 HIGH RES 80 KM BUFR NON GTS SATEM PROFILES AND
!                  CLEAR RADIANCE REPORT

NGTHR2   = 212
NGTHR2SQ =  10
MSQBYCTP(NGTHR2)=NGTHR2SQ
CCMOCTP(NGTHR2)='TOVS Clear Radiances and Retr.  '

!*          1.7.11 SSMI/TRMM REPORT

NSSMI    = 215
NSSMISQ  =  11
MSQBYCTP(NSSMI)=NSSMISQ
CCMOCTP(NSSMI)='SSMI or TRMM Report             '

!*          1.7.12 RETRIEVED OZONE LAYERS REPORT

NSTRO3   = 206
NSTRO3SQ =  12
MSQBYCTP(NSTRO3)=NSTRO3SQ
CCMOCTP(NSTRO3)='Retrieved ozone layers report  '

!*          1.7.13 SMOS L-band radiances 

NSMOS   = 400
NSMOSSQ =  13
MSQBYCTP(NSMOS)=NSMOSSQ
CCMOCTP(NSMOS)='SMOS level 1C L-band radiances  '

!*          1.7.14 MERIS REPORT

NTCWC = 214
NTCWCSQ = 14
MSQBYCTP(NTCWC)= NTCWCSQ
CCMOCTP(NTCWC)='TCWC (MERIS) Report                   '

!*          1.8    PAOB

NPAOB    =   8
NPAOBSQ  =   8
NPAOCT   =   2
MNOCTP(NPAOB)=NPAOCT
CCMOOTP(NPAOB )='PAOB,  Australian pseudo obs    '

!*          1.8.1  PAOB REPORT

NPABCD   = 180
NPABCDSQ =   1
MSQBYCTP(NPABCD)=NPABCDSQ
CCMOCTP(NPABCD)='PAOB Report                     '

!*          1.8.2  Donnees cyclone reunion

NTYPHO   = 181
NTYPHOSQ =   2
MSQBYCTP(NTYPHO)=NTYPHOSQ
CCMOCTP(NTYPHO)='TYPHO Report                     '

!*          1.9    SCATTEROMETER

NSCATT   =   9
NSCATTSQ =   9
NSCACT   =   6
MNOCTP(NSCATT)=NSCACT
CCMOOTP(NSCATT)='SCATT, Scatterometer            '

!*          1.9.1  SCATTEROMETER 1 (Obsolete)

NSCAT1   =   8
NSCAT1SQ =   1
MSQBYCTP(NSCAT1)=NSCAT1SQ
CCMOCTP(NSCAT1)='ERS Scatterometer 1 (old)       '

!*          1.9.2  SCATTEROMETER 2

NSCAT2   = 122
NSCAT2SQ =   2
MSQBYCTP(NSCAT2)=NSCAT2SQ
CCMOCTP(NSCAT2)='ERS Scatterometer               '

!*          1.9.3  SCATTEROMETER 3

NSCAT3   = 139
NSCAT3SQ =   3
MSQBYCTP(NSCAT3)=NSCAT3SQ
CCMOCTP(NSCAT3)='ASCAT Scatterometer             '

!*          1.9.4  SCATTEROMETER 4

NSCAT4   = 300
NSCAT4SQ =   4
MSQBYCTP(NSCAT4)=NSCAT4SQ
CCMOCTP(NSCAT4)='NSCAT ADEOS-1                   '

!*          1.9.5  SCATTEROMETER 5

NSCAT5   = 301
NSCAT5SQ =   5
MSQBYCTP(NSCAT5)=NSCAT5SQ
CCMOCTP(NSCAT5)='QuikSCAT                        '

!*          1.9.6  SCATTEROMETER 6

NSCAT6   = 302
NSCAT6SQ =   6
MSQBYCTP(NSCAT6)=NSCAT6SQ
CCMOCTP(NSCAT6)='OCEANSAT-2 Scatterometer        '

!*          1.10   Limb observations

NLIMB    =  10
NLIMBSQ  =  10
NLIMBCT  =   2
MNOCTP(NLIMB)=NLIMBCT
CCMOOTP(NLIMB)='LIMB, Limb observations        '

!*          1.10.1 GPS Radio occultation

NGPSRO   =   250
NGPSROSQ =   1
MSQBYCTP(NGPSRO)=NGPSROSQ
CCMOCTP(NGPSRO)='GPS Radio occultation           '

!*          1.10.2 Limb radiances

NLRAD    =   251
NLRADSQ  =   2
MSQBYCTP(NLRAD)=NLRADSQ
CCMOCTP(NLRAD)='Limb radiances                  '


!*          1.13   RADAR

NRADAR   =  13
NRADARSQ =  13
NRDRCT   =   1
MNOCTP(NRADAR)=NRDRCT
CCMOOTP(NRADAR)='RADAR, meteorological radar     '

!*          1.13.1 BUFR RADAR REFLECTIVITY 1

NRADA1   =   2
NRADA1SQ =   1
MSQBYCTP(NRADA1)=NRADA1SQ
CCMOCTP(NRADA1)='RARAD 1                         '

!*
!-----------------------------------------------------------------------

!        2.       SETUP DONE
!                 ----------

!     WRITE(KOUT,'(''0CMA OBS./CODE TYPES SETUP'')')
!*
!-----------------------------------------------------------------------

!        3.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SUCMOCTP',1,ZHOOK_HANDLE)
END SUBROUTINE SUCMOCTP
