SUBROUTINE SUCTRL_GFLATTR

!     ----------------------------------------------------------------------
!**** *SUCTRL_GFLATTR * - Checking of the consistency of the GFL attributes.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SUCTRL_GFLATTR*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad, after
!        - old part 1.2.2 of SUDIM1 (current parts 1. to 1.4)
!        - old part 1.2.4 of SUDIM1 (current part 2)
!        - old part 1.3.3 of SUDIM1 (current part 3)
!        - old parts 1.9.1 to 1.9.4 of SUDIM1 (current parts 4.1 to 4.4)
!        - old parts 1.10.1 to 1.10.6 of SUDIM1 (current parts 5.1 to 5.6)
!        - some SUDYN checkings (ex: current part 1.5)

!     Modifications.
!     --------------
!      Original : 31-01-2007
!      28-Aug-2007 F. Vana: removing LRSPLINE attribute
!      J.Haseler     27-Feb-2007 Modify defaults for LTRAJIO and LGPINGP
!      JJMorcrette   20060925    DU, BC, OM, SO2, VOL, SOA climatological flds
!      JJMorcrette   20061020    diagnostic of aerosol physical fluxes
!      22-Mar-2007 S.Serrar  : defaults for ERA40 GFL fields
!      27-Jun-2007 E.Holm    : default option for GFL SL TL/AD LADV5 attribute
!      12-Dec-2007 Y.Seity   : add GFL Hail
!      30-Jun-2008 J. Masek  : Setup of SLHD and LSHDQUAD
!      16-Mar-2009 K. Yessad : Prevent forbidden case (LPT,LADV)=(T,F).
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR
USE YOMCT0   , ONLY : NCONF    ,NSTOP    ,LARPEGEF ,LRPLANE  ,LSLAG    ,&
 & LSPRT    ,LCANARI  ,LSFORC   ,LPC_FULL
USE YOMCT0B  , ONLY : LECMWF
USE YOMPHY   , ONLY : LMPHYS   ,LNEBN    ,LNEBR    ,LRRGUST  ,LCVRA    ,&
 & LPROCLD  ,LCVPRO   ,LCDDPRO  ,LENTCH   ,L3MT     ,LSTRAPRO
USE YOMARPHY , ONLY : LMPA
USE YOEPHY   , ONLY : LEPHYS   ,LEPCLD
USE YOMINI   , ONLY : NEINI
USE YOMDYNA  , ONLY : LSLHD    ,LSLHDQUAD,LSLHD_OLD,SLHDKMIN ,SLHDKMAX
USE YOMCVER  , ONLY : LVSPLIP
USE YOMSIMPHL, ONLY : LSIMPH
USE YOM_YGFL , ONLY : YQ_NL    ,YI_NL    ,YL_NL    ,YS_NL    ,YR_NL    , &
 & YG_NL    ,YH_NL  ,YTKE_NL   ,YA_NL    ,YO3_NL   ,YEXT_NL  ,YCPF_NL  , &
 & YSPF_NL  ,YSRC_NL,YCVGQ_NL  ,YAERO_NL ,YGHG_NL  ,YGRG_NL  ,YTRAC_NL , &
 & YSDSAT_NL,YCVV_NL,YFORC_NL  ,YQVA_NL  ,YERA40_NL,YEZDIAG_NL, &
 & YUOM_NL  ,YUAL_NL  ,YDOM_NL  ,YDAL_NL  ,YUEN_NL  ,YUNEBH_NL, &
 & YNOGW_NL ,YUVP_NL  ,YAEROUT_NL,YSLDIA_NL, &
 & JPAERO   ,JPGHG    ,JPGRG    ,JPTRAC   ,JPFORC   ,JPEZDIAG ,JPERA40  , &
 & JPNOGW   ,JPUVP    ,JPAEROUT ,JPSLDIA  , &
 & NGFL_EXT ,NAERO    ,NGHG     ,NGRG     ,NTRAC    ,NERA40   ,NGFL_FORC, &
 & NGFL_EZDIAG, NSLDIA, NNOGW, NUVP, NAEROUT, YGRGTEND_NL, YLRCH4_NL, YCH4S_NL

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JGFL
LOGICAL :: LLCOMPUTE_CVGQ,LLACTIVE
 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUCTRL_GFLATTR',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.     Some consistency checks for GFL attributes.
!               -------------------------------------------

!        1.1    Attribute LVSPLIP:

! * Attribute LVSPLIP:
!   - Is currently coded only for ozone O3.
!   - Excludes some semi-Lagrangian interpolators.
!   - Variable LVSPLIP is set to .TRUE. if the attribute LVSPLIP
!     is .TRUE. for at least one of the GFL variables.

IF (YQ_NL%LVSPLIP &
 & .OR. YA_NL%LVSPLIP .OR. YI_NL%LVSPLIP .OR. YL_NL%LVSPLIP &
 & .OR. YR_NL%LVSPLIP .OR. YS_NL%LVSPLIP .OR. YG_NL%LVSPLIP &
 & .OR. YH_NL%LVSPLIP &
 & .OR. YTKE_NL%LVSPLIP .OR. YSRC_NL%LVSPLIP .OR. YCPF_NL%LVSPLIP &
 & .OR. YSPF_NL%LVSPLIP .OR. YCVGQ_NL%LVSPLIP.OR. YQVA_NL%LVSPLIP &
 & .OR. YSDSAT_NL%LVSPLIP .OR. YCVV_NL%LVSPLIP &
 & .OR. YUOM_NL%LVSPLIP .OR. YUAL_NL%LVSPLIP &
 & .OR. YDOM_NL%LVSPLIP .OR. YDAL_NL%LVSPLIP &
 & .OR. YUEN_NL%LVSPLIP .OR. YUNEBH_NL%LVSPLIP &
 & .OR. YLRCH4_NL%LVSPLIP .OR. YCH4S_NL%LVSPLIP &
 & .OR. ANY(YGHG_NL(:)%LVSPLIP) &
 & .OR. ANY(YTRAC_NL(:)%LVSPLIP) &
 & .OR. ANY(YGRG_NL(:)%LVSPLIP) &
 & .OR. ANY(YGRGTEND_NL(:)%LVSPLIP) &
 & .OR. ANY(YAERO_NL(:)%LVSPLIP) &
 & .OR. ANY(YERA40_NL(:)%LVSPLIP) &
 & .OR. ANY(YNOGW_NL(:)%LVSPLIP) &
 & .OR. ANY(YUVP_NL(:)%LVSPLIP) &
 & .OR. ANY(YAEROUT_NL(:)%LVSPLIP) &
 & .OR. ANY(YSLDIA_NL(:)%LVSPLIP) &
 & .OR. ANY(YFORC_NL(:)%LVSPLIP) &
 & .OR. ANY(YEZDIAG_NL(:)%LVSPLIP) &
 & .OR. ANY(YEXT_NL(:)%LVSPLIP) &
 & ) THEN
  CALL ABOR1('SUCTRL_GFLATTR, CALL ABOR1: LVSPLIP can only be used with YO3')
ENDIF

IF (YO3_NL%LVSPLIP .AND. (YO3_NL%LSLHD.OR.YO3_NL%LQM.OR.YO3_NL%LQMH &
 & .OR. YO3_NL%LHV) ) THEN
  CALL ABOR1('SUCTRL_GFLATTR: if YO3_NL%LVSPLIP == .T., all others YO3_NL interpolation descriptors must be .F.!')
ENDIF

LVSPLIP = LVSPLIP.OR. (YQ_NL%LVSPLIP .OR. &
 & YA_NL%LVSPLIP .OR. &
 & YI_NL%LVSPLIP .OR. &
 & YL_NL%LVSPLIP .OR. &
 & YR_NL%LVSPLIP .OR. &
 & YS_NL%LVSPLIP .OR. &
 & YG_NL%LVSPLIP .OR. &
 & YH_NL%LVSPLIP .OR. &
 & YTKE_NL%LVSPLIP .OR. &
 & YSRC_NL%LVSPLIP .OR. &
 & YO3_NL%LVSPLIP .OR. &
 & YCPF_NL%LVSPLIP .OR. &
 & YSPF_NL%LVSPLIP .OR. &
 & YCVGQ_NL%LVSPLIP.OR. &
 & YQVA_NL%LVSPLIP.OR. &
 & YSDSAT_NL%LVSPLIP.OR. &
 & YCVV_NL%LVSPLIP.OR. &
 & YUOM_NL%LVSPLIP.OR. &
 & YUAL_NL%LVSPLIP.OR. &
 & YDOM_NL%LVSPLIP.OR. &
 & YDAL_NL%LVSPLIP.OR. &
 & YUEN_NL%LVSPLIP.OR. &
 & YUNEBH_NL%LVSPLIP.OR. &
 & YLRCH4_NL%LVSPLIP.OR. &
 & YCH4S_NL%LVSPLIP)

DO JGFL=1,JPGHG
  LVSPLIP = LVSPLIP.OR.(YGHG_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPTRAC
  LVSPLIP = LVSPLIP.OR.(YTRAC_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPGRG
  LVSPLIP = LVSPLIP.OR.(YGRG_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPGRG
  LVSPLIP = LVSPLIP.OR.(YGRGTEND_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPAERO
  LVSPLIP = LVSPLIP.OR.(YAERO_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPERA40
  LVSPLIP = LVSPLIP.OR.(YERA40_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPNOGW
  LVSPLIP = LVSPLIP.OR.(YNOGW_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPUVP
  LVSPLIP = LVSPLIP.OR.(YUVP_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPAEROUT
  LVSPLIP = LVSPLIP.OR.(YAEROUT_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPSLDIA
  LVSPLIP = LVSPLIP.OR.(YSLDIA_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPFORC
  LVSPLIP = LVSPLIP.OR.(YFORC_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,JPEZDIAG
  LVSPLIP = LVSPLIP.OR.(YEZDIAG_NL(JGFL)%LVSPLIP)
ENDDO

DO JGFL=1,SIZE(YEXT_NL)
  LVSPLIP = LVSPLIP.OR.(YEXT_NL(JGFL)%LVSPLIP)
ENDDO

!        1.2    Attribute LHV:

! * Attribute LHV: is currently coded only for ozone O3.
!   If LHV=T, semi-Lagrangian interpolations cannot be spline or SLHD interp.
!   LHV=T is currently not coded for plane geometry.

IF (YQ_NL%LHV .OR. YA_NL%LHV .OR. YI_NL%LHV .OR. YL_NL%LHV &
 & .OR. YR_NL%LHV .OR. YS_NL%LHV .OR. YG_NL%LHV .OR. YH_NL%LHV  &
 & .OR. YTKE_NL%LHV .OR. YSRC_NL%LHV .OR. YCPF_NL%LHV &
 & .OR. YSPF_NL%LHV .OR. YCVGQ_NL%LHV .OR. YQVA_NL%LHV &
 & .OR. YSDSAT_NL%LHV .OR. YCVV_NL%LHV &
 & .OR. YUOM_NL%LHV .OR. YUAL_NL%LHV &
 & .OR. YDOM_NL%LHV .OR. YDAL_NL%LHV &
 & .OR. YUEN_NL%LHV .OR. YUNEBH_NL%LHV &
 & .OR. YLRCH4_NL%LHV .OR. YCH4S_NL%LHV &
 & .OR. ANY(YGHG_NL(:)%LHV) &
 & .OR. ANY(YTRAC_NL(:)%LHV) &
 & .OR. ANY(YGRG_NL(:)%LHV) &
 & .OR. ANY(YGRGTEND_NL(:)%LHV) &
 & .OR. ANY(YAERO_NL(:)%LHV) &
 & .OR. ANY(YERA40_NL(:)%LHV) &
 & .OR. ANY(YNOGW_NL(:)%LHV) &
 & .OR. ANY(YUVP_NL(:)%LHV) &
 & .OR. ANY(YAEROUT_NL(:)%LHV) &
 & .OR. ANY(YSLDIA_NL(:)%LHV) &
 & .OR. ANY(YFORC_NL(:)%LHV) &
 & .OR. ANY(YEZDIAG_NL(:)%LHV) &
 & .OR. ANY(YEXT_NL(:)%LHV) &
 & ) THEN
  CALL ABOR1('SUCTRL_GFLATTR, CALL ABOR1: LHV can only be used with YO3')
ENDIF

IF (YO3_NL%LHV .AND. YO3_NL%LSLHD ) THEN
  CALL ABOR1('SUCTRL_GFLATTR CALL ABOR1: if YO3_NL%LHV == .T., YO3_NL%LSLHD can not be .T.!')
ENDIF

IF (YO3_NL%LHV.AND.LRPLANE) THEN
  WRITE(NULERR,*) 'YO3_NL%LHV=T NOT CODED FOR PLANE GEOMETRY.'
  CALL ABOR1(' SUCTRL_GFLATTR 1.2 (no YO3_NL%LHV for LRPLANE)')
ENDIF

!        1.3    Attribute LSLHD:

! * Attribute LSLHD: variable LSLHD is set to .TRUE. if the attribute LSLHD
!   is .TRUE. for at least one of the GFL variables.

LSLHD = LSLHD.OR. (YQ_NL%LSLHD .OR. &
 & YA_NL%LSLHD .OR. &
 & YI_NL%LSLHD .OR. &
 & YL_NL%LSLHD .OR. &
 & YR_NL%LSLHD .OR. & 
 & YS_NL%LSLHD .OR. &
 & YG_NL%LSLHD .OR. &
 & YH_NL%LSLHD .OR. &
 & YTKE_NL%LSLHD .OR. &
 & YSRC_NL%LSLHD .OR. &
 & YO3_NL%LSLHD .OR. &
 & YCPF_NL%LSLHD .OR. &
 & YSPF_NL%LSLHD .OR. &
 & YCVGQ_NL%LSLHD.OR. &
 & YQVA_NL%LSLHD.OR. &
 & YSDSAT_NL%LSLHD.OR. &
 & YCVV_NL%LSLHD.OR. &
 & YUOM_NL%LSLHD.OR. &
 & YUAL_NL%LSLHD.OR. &
 & YDOM_NL%LSLHD.OR. &
 & YDAL_NL%LSLHD.OR. &
 & YUEN_NL%LSLHD.OR. &
 & YUNEBH_NL%LSLHD.OR. &
 & YLRCH4_NL%LSLHD.OR. &
 & YCH4S_NL%LSLHD)

DO JGFL=1,JPGHG
  LSLHD = LSLHD.OR.(YGHG_NL(JGFL)%LSLHD) 
ENDDO

DO JGFL=1,JPTRAC
  LSLHD = LSLHD.OR.(YTRAC_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,JPGRG
  LSLHD = LSLHD.OR.(YGRG_NL(JGFL)%LSLHD) 
ENDDO

DO JGFL=1,JPGRG
  LSLHD = LSLHD.OR.(YGRGTEND_NL(JGFL)%LSLHD) 
ENDDO

DO JGFL=1,JPAERO
  LSLHD = LSLHD.OR.(YAERO_NL(JGFL)%LSLHD) 
ENDDO

DO JGFL=1,JPERA40
  LSLHD = LSLHD.OR.(YERA40_NL(JGFL)%LSLHD) 
ENDDO

DO JGFL=1,JPNOGW
  LSLHD = LSLHD.OR.(YNOGW_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,JPUVP
  LSLHD = LSLHD.OR.(YUVP_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,JPAEROUT
  LSLHD = LSLHD.OR.(YAEROUT_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,JPSLDIA
  LSLHD = LSLHD.OR.(YSLDIA_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,JPFORC
  LSLHD = LSLHD.OR.(YFORC_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,JPEZDIAG
  LSLHD = LSLHD.OR.(YEZDIAG_NL(JGFL)%LSLHD)
ENDDO

DO JGFL=1,SIZE(YEXT_NL)
  LSLHD = LSLHD.OR.(YEXT_NL(JGFL)%LSLHD)
ENDDO

! * Set internal key LSLHDQUAD (indicates necessity to compute quadratic
!   interpolation weights)

LSLHDQUAD=(SLHDKMIN /= 0.0_JPRB) .OR. &
 & (LSLHD.AND.(.NOT.LSLHD_OLD).AND.(SLHDKMAX /= 0.0_JPRB))


!        1.4    Attribute LGP:

! * Attribute LGP should be reset to .T. for CPF and SPF in some cases:
IF (.NOT.LECMWF .AND. (NCONF == 1 .OR. NCONF == 801 .OR. NCONF == 701 &
   & .OR. NCONF == 401 .OR. NCONF == 501 &
   & .OR. NCONF == 601 .OR. NCONF == 131 .OR. NCONF == 302) .AND. LMPHYS) THEN

  IF(LRRGUST) THEN
    YCPF_NL%LGP=.TRUE.
    YSPF_NL%LGP=.TRUE.
    WRITE(NULERR,*) ' SUCTRL_GFLATTR: YCPF_NL%LGP and YSPF_NL%LGP reset to T!'
  ELSEIF((LNEBN.OR.LNEBR).AND.(.NOT.LRRGUST)) THEN
    YCPF_NL%LGP=.TRUE.
    WRITE(NULERR,*) ' SUCTRL_GFLATTR: YCPF_NL%LGP reset to T!'
  ENDIF
ENDIF

! * For some of the GFL, and if NSTOP > 0, the attribute LGP can be .T.
!   only if LSLAG=T.
!   (but there are probably some other GFL variables which are also problematic
!   about this topic, ABOR1 to be added later when relevant).
IF(YO3_NL%LGP.AND..NOT.LSLAG.AND.NSTOP > 0) THEN
  WRITE(NULERR,*) ' YO3_NL%LGP=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.4_O3')
ENDIF
DO JGFL=1,NAERO
  IF(YAERO_NL(JGFL)%LGP.AND..NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT AEROSOLS NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_AERO')
  ENDIF
ENDDO
DO JGFL=1,NGHG
  IF(YGHG_NL(JGFL)%LGP.AND..NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT GREENHOUSE GASES NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_GHG')
  ENDIF
ENDDO
DO JGFL=1,NTRAC
  IF(YTRAC_NL(JGFL)%LGP.AND..NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT TRACERS NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_TRAC')
  ENDIF
ENDDO
DO JGFL=1,NGRG
  IF(YGRG_NL(JGFL)%LGP.AND..NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT GEMS-GRG NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_GRG')
  ENDIF
ENDDO
IF(NGHG > 0) THEN
  IF(YLRCH4_NL%LGP .AND. .NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT METHANE LOSS RATE NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_LRCH4')
  ENDIF

  IF(YCH4S_NL%LGP .AND. .NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT METHANE SINK NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_CH4S')
  ENDIF
ENDIF
DO JGFL=1,NSLDIA
  IF(YSLDIA_NL(JGFL)%LGP.AND..NOT.LSLAG.AND.NSTOP > 0) THEN
    WRITE(NULERR,*) ' GRIDPOINT LSLDIA DIAGNOSTICS NOT ALLOWED IN EULERIAN.'
    CALL ABOR1(' SUCTRL_GFLATTR: 1.4_SLDIA')
  ENDIF
ENDDO

!        1.5    Attribute LADV:

! * Attribute LADV:
!   For some of the GFL variables, LADV can be set to .T. only if LSLAG=T.
!   (but there are probably some other GFL variables which are also problematic
!   about this topic, ABOR1 to be added later when relevant).

IF(YI_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YI_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_I')
ENDIF
IF(YL_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YL_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_L')
ENDIF
IF(YA_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YA_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_A')
ENDIF
IF(YS_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YS_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_S')
ENDIF
IF(YG_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YG_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_G')
ENDIF
IF(YH_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YH_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_G')
ENDIF
IF(YR_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YR_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_R')
ENDIF
IF(YTKE_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YTKE_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_TKE')
ENDIF
IF(YSRC_NL%LADV.AND..NOT.LSLAG) THEN
  WRITE(NULERR,*) ' YSRC_NL%LADV=TRUE NOT ALLOWED IN EULERIAN.'
  CALL ABOR1(' SUCTRL_GFLATTR: 1.5_SRC')
ENDIF

!     ------------------------------------------------------------------

!*       2.     Some consistency checks for GFL GRIB codes.
!               -------------------------------------------

IF (.NOT.LARPEGEF) THEN

  ! Set default values for grib codes and names of GFL extra fields 
  ! in case they were not given in the namelist.

  DO JGFL=1,NGFL_EXT
    IF(YEXT_NL(JGFL)%IGRBCODE == 999) CALL ABOR1&
     & (' SUCTRL_GFLATTR: THE GRIB CODES OF THE GFL EXTRA FIELDS MUST BE GIVEN')
    IF(YEXT_NL(JGFL)%CNAME == 'XXXXXXXXXXXXXXX') CALL ABOR1&
     & (' SUCTRL_GFLATTR: THE NAMES OF THE GFL EXTRA FIELDS MUST BE GIVEN')
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*       3.     Reset the GFL attributes LSP and LGP for configurations
!               where there is no GFL at all.
!               -------------------------------------------------------

IF(ANY(SPREAD(NCONF,1,11) ==&
 & (/201,202,421,422,521,522,911,912,923,931,932/))) THEN  
  ! * reset the LSP attribute to .FALSE. for all GFL variables.
  !   (there are no GFL variables for such configurations).
  YQ_NL%LSP = .FALSE.
  YL_NL%LSP = .FALSE.
  YI_NL%LSP = .FALSE.
  YS_NL%LSP = .FALSE.
  YR_NL%LSP = .FALSE.
  YG_NL%LSP = .FALSE.
  YH_NL%LSP = .FALSE.
  YTKE_NL%LSP = .FALSE.
  YA_NL%LSP = .FALSE.
  YSRC_NL%LSP= .FALSE.
  YO3_NL%LSP= .FALSE.
  YCPF_NL%LSP= .FALSE.
  YSPF_NL%LSP= .FALSE.
  YCVGQ_NL%LSP= .FALSE.
  YQVA_NL%LSP= .FALSE.
  YSDSAT_NL%LSP= .FALSE.
  YCVV_NL%LSP= .FALSE.
  YUOM_NL%LSP= .FALSE.
  YUAL_NL%LSP= .FALSE.
  YDOM_NL%LSP= .FALSE.
  YDAL_NL%LSP= .FALSE.
  YUEN_NL%LSP= .FALSE.
  YUNEBH_NL%LSP= .FALSE.
  YGHG_NL(:)%LSP= .FALSE.
  YTRAC_NL(:)%LSP= .FALSE.
  YGRG_NL(:)%LSP= .FALSE.
  YAERO_NL(:)%LSP= .FALSE.
  YLRCH4_NL%LSP= .FALSE.
  YCH4S_NL%LSP= .FALSE.
  YERA40_NL(:)%LSP= .FALSE.
  YNOGW_NL(:)%LSP= .FALSE.
  YUVP_NL(:)%LSP= .FALSE.
  YAEROUT_NL(:)%LSP= .FALSE.
  YSLDIA_NL(:)%LSP= .FALSE.
  YFORC_NL(:)%LSP= .FALSE.
  YEZDIAG_NL(:)%LSP= .FALSE.
  YEXT_NL(:)%LSP= .FALSE.
  ! * reset the LGP attribute for all GFL variables.
  !   (there are no GFL variables for such configurations).
  YQ_NL%LGP = .FALSE.
  YL_NL%LGP = .FALSE.
  YI_NL%LGP = .FALSE.
  YS_NL%LGP = .FALSE.
  YR_NL%LGP = .FALSE.
  YG_NL%LGP = .FALSE.
  YH_NL%LGP = .FALSE.
  YTKE_NL%LGP = .FALSE.
  YA_NL%LGP = .FALSE.
  YSRC_NL%LGP= .FALSE.
  YO3_NL%LGP= .FALSE.
  YCPF_NL%LGP= .FALSE.
  YSPF_NL%LGP= .FALSE.
  YCVGQ_NL%LGP= .FALSE.
  YQVA_NL%LGP= .FALSE.
  YSDSAT_NL%LGP= .FALSE.
  YCVV_NL%LGP= .FALSE.
  YUOM_NL%LGP= .FALSE.
  YUAL_NL%LGP= .FALSE.
  YDOM_NL%LGP= .FALSE.
  YDAL_NL%LGP= .FALSE.
  YUEN_NL%LGP= .FALSE.
  YUNEBH_NL%LGP= .FALSE.
  YGHG_NL(:)%LGP= .FALSE.
  YTRAC_NL(:)%LGP= .FALSE.
  YGRG_NL(:)%LGP= .FALSE.
  YAERO_NL(:)%LGP= .FALSE.
  YLRCH4_NL%LGP= .FALSE.
  YCH4S_NL%LGP= .FALSE.
  YERA40_NL(:)%LGP= .FALSE.
  YNOGW_NL(:)%LGP= .FALSE.
  YUVP_NL(:)%LGP= .FALSE.
  YAEROUT_NL(:)%LGP= .FALSE.
  YSLDIA_NL(:)%LGP= .FALSE.
  YFORC_NL(:)%LGP= .FALSE.
  YEZDIAG_NL(:)%LGP= .FALSE.
  YEXT_NL(:)%LGP= .FALSE.
ENDIF

!     ------------------------------------------------------------------

!*       4.     Check representation for GFL fields.
!               ------------------------------------

!*       4.1    check for all GFL that LGP and LSP are not both TRUE.

IF(YQ_NL%LGP.AND.YQ_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YQ_NL%LGP AND YQ_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_Q ')
ENDIF
IF(YO3_NL%LGP.AND.YO3_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YO3_NL%LGP AND YO3_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_O3')
ENDIF
IF(YL_NL%LGP.AND.YL_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YL_NL%LGP AND YL_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_L')
ENDIF
IF(YI_NL%LGP.AND.YI_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YI_NL%LGP AND YI_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_I')
ENDIF
IF(YA_NL%LGP.AND.YA_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YA_NL%LGP AND YA_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_A')
ENDIF
IF(YR_NL%LGP.AND.YR_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YR_NL%LGP AND YR_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_R')
ENDIF
IF(YS_NL%LGP.AND.YS_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YS_NL%LGP AND YS_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_S')
ENDIF
IF(YG_NL%LGP.AND.YG_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YG_NL%LGP AND YG_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_G')
ENDIF
IF(YH_NL%LGP.AND.YH_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YH_NL%LGP AND YH_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_H')
ENDIF
IF(YTKE_NL%LGP.AND.YTKE_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YTKE_NL%LGP AND YTKE_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_TKE')
ENDIF
IF(YSRC_NL%LGP.AND.YSRC_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YSRC_NL%LGP AND YSRC_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_SRC')
ENDIF
IF(YCPF_NL%LGP.AND.YCPF_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YCPF_NL%LGP AND YCPF_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_CPF')
ENDIF
IF(YSPF_NL%LGP.AND.YSPF_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YSPF_NL%LGP AND YSPF_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_SPF')
ENDIF
IF(YCVGQ_NL%LGP.AND.YCVGQ_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YCVGQ_NL%LGP AND YCVGQ_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_CVGQ')
ENDIF
IF(YQVA_NL%LGP.AND.YQVA_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YQVA_NL%LGP AND YQVA_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_QVA')
ENDIF
IF(YSDSAT_NL%LGP.AND.YSDSAT_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YSDSAT_NL%LGP AND YSDSAT_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_SDSAT')
ENDIF
IF(YCVV_NL%LGP.AND.YCVV_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YCVV_NL%LGP AND YCVV_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_CVV')
ENDIF
IF(YUOM_NL%LGP.AND.YUOM_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YUOM_NL%LGP AND YUOM_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_UOM')
ENDIF
IF(YUAL_NL%LGP.AND.YUAL_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YUAL_NL%LGP AND YUAL_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_UAL')
ENDIF
IF(YDOM_NL%LGP.AND.YDOM_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YDOM_NL%LGP AND YDOM_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_DOM')
ENDIF
IF(YDAL_NL%LGP.AND.YDAL_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YDAL_NL%LGP AND YDAL_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_DAL')
ENDIF
IF(YUEN_NL%LGP.AND.YUEN_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YUEN_NL%LGP AND YUEN_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_UEN')
ENDIF
IF(YUNEBH_NL%LGP.AND.YUNEBH_NL%LSP) THEN
  WRITE(NULERR,'(''BOTH YUNEBH_NL%LGP AND YUNEBH_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_UNEBH')
ENDIF

DO JGFL=1,JPGHG
  IF(YGHG_NL(JGFL)%LGP.AND.YGHG_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For greenhouse gas number',JGFL
    WRITE(NULERR,'(''BOTH YGHG_NL%LGP AND YGHG_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_GHG')
  ENDIF
ENDDO

DO JGFL=1,JPTRAC
  IF(YTRAC_NL(JGFL)%LGP.AND.YTRAC_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For diagnostic tracer number',JGFL
    WRITE(NULERR,'(''BOTH YTRAC_NL%LGP AND YTRAC_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_TRAC')
  ENDIF
ENDDO

IF(YLRCH4_NL%LGP.AND.YLRCH4_NL%LSP) THEN
  WRITE(NULERR,'(A,I3)') ' For CH4 loss rate'
  WRITE(NULERR,'(''BOTH YLRCH4_NL%LGP AND YLRCH4_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_LRCH4')
ENDIF

IF(YCH4S_NL%LGP.AND.YCH4S_NL%LSP) THEN
  WRITE(NULERR,'(A,I3)') ' For CH4 atm. sink'
  WRITE(NULERR,'(''BOTH YCH4S_NL%LGP AND YCH4S_NL%LSP TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.1_CH4S')
ENDIF

DO JGFL=1,JPGRG
  IF(YGRG_NL(JGFL)%LGP.AND.YGRG_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For reactive gas number',JGFL
    WRITE(NULERR,'(''BOTH YGRG_NL%LGP AND YGRG_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_GRG')
  ENDIF
ENDDO

DO JGFL=1,JPAERO
  IF(YAERO_NL(JGFL)%LGP.AND.YAERO_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For aerosol number',JGFL
    WRITE(NULERR,'(''BOTH YAERO_NL%LGP AND YAERO_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_AERO')
  ENDIF
ENDDO

DO JGFL=1,JPERA40
  IF(YERA40_NL(JGFL)%LGP.AND.YERA40_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For ERA40 field number',JGFL
    WRITE(NULERR,'(''BOTH YERA40_NL%LGP AND YERA40_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_ERA40')
  ENDIF
ENDDO

DO JGFL=1,JPNOGW
  IF(YNOGW_NL(JGFL)%LGP.AND.YNOGW_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For NOGW field number',JGFL
    WRITE(NULERR,'(''BOTH YNOGW_NL%LGP AND YNOGW_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_NOGW')
  ENDIF
ENDDO

DO JGFL=1,JPUVP
  IF(YUVP_NL(JGFL)%LGP.AND.YUVP_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For UVP field number',JGFL
    WRITE(NULERR,'(''BOTH YUVP_NL%LGP AND YUVP_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_UVP')
  ENDIF
ENDDO

DO JGFL=1,JPAEROUT
  IF(YAEROUT_NL(JGFL)%LGP.AND.YAEROUT_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For AEROUT field number',JGFL
    WRITE(NULERR,'(''BOTH YAEROUT_NL%LGP AND YAEROUT_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_AEROUT')
  ENDIF
ENDDO

DO JGFL=1,JPSLDIA
  IF(YSLDIA_NL(JGFL)%LGP.AND.YSLDIA_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For SLDIA field number',JGFL
    WRITE(NULERR,'(''BOTH YSLDIA_NL%LGP AND YSLDIA_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_SLDIA')
  ENDIF
ENDDO

DO JGFL=1,JPFORC
  IF(YFORC_NL(JGFL)%LGP.AND.YFORC_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For forcing number',JGFL
    WRITE(NULERR,'(''BOTH YFORC_NL%LGP AND YFORC_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_FORC')
  ENDIF
ENDDO

DO JGFL=1,JPEZDIAG
  IF(YEZDIAG_NL(JGFL)%LGP.AND.YEZDIAG_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For easy diagnostic number',JGFL
    WRITE(NULERR,'(''BOTH YEZDIAG_NL%LGP AND YEZDIAG_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_EZDIAG')
  ENDIF
ENDDO

DO JGFL=1,SIZE(YEXT_NL)
  IF(YEXT_NL(JGFL)%LGP.AND.YEXT_NL(JGFL)%LSP) THEN
    WRITE(NULERR,'(A,I3)') ' For extra-GFL number',JGFL
    WRITE(NULERR,'(''BOTH YEXT_NL%LGP AND YEXT_NL%LSP TRUE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.1_EXT')
  ENDIF
ENDDO

!*       4.2    Check that LGP is TRUE for ql,qi,qs,qr,qg,TKE when the AROME
!               physics is activated.

IF(LMPA) THEN
  IF(.NOT.(YL_NL%LGP.AND.YI_NL%LGP.AND.YS_NL%LGP.AND.YR_NL%LGP.AND.&
   & YG_NL%LGP.AND.YTKE_NL%LGP)) THEN
    WRITE(NULERR,'('' AROME REQUIRES YL YI YS YR YG YTKE_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.2')
  ENDIF
ENDIF

!*       4.3    Checkings for ALARO prognostic convection variables

! * UAL and UOM should be both spectral or both grid-point when required.
IF ((YUAL_NL%LGP.AND..NOT.YUOM_NL%LGP).OR.&
 &(YUOM_NL%LGP.AND..NOT.YUAL_NL%LGP)) THEN
  WRITE(NULERR,'(''YUOM_NL%LGP AND YUAL_NL%LGP MUST BE EQUAL'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.3_UAL_UOM_LGP')
ENDIF
IF ((YUAL_NL%LSP.AND..NOT.YUOM_NL%LSP).OR.&
 &(YUOM_NL%LSP.AND..NOT.YUAL_NL%LSP)) THEN
  WRITE(NULERR,'(''YUOM_NL%LSP AND YUAL_NL%LSP MUST BE EQUAL'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.3_UAL_UOM_LSP')
ENDIF

! * DAL and DOM should be both spectral or both grid-point when required.
IF ((YDAL_NL%LGP.AND..NOT.YDOM_NL%LGP).OR.&
 &(YDOM_NL%LGP.AND..NOT.YDAL_NL%LGP)) THEN
  WRITE(NULERR,'(''YDOM_NL%LGP AND YDAL_NL%LGP MUST BE EQUAL'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.3_DAL_DOM_LGP')
ENDIF
IF ((YDAL_NL%LSP.AND..NOT.YDOM_NL%LSP).OR.&
 &(YDOM_NL%LSP.AND..NOT.YDAL_NL%LSP)) THEN
  WRITE(NULERR,'(''YDOM_NL%LSP AND YDAL_NL%LSP MUST BE EQUAL'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.3_DAL_DOM_LSP')
ENDIF

! * If LCVRA, DOM and UOM should be both spectral or both grid-point.
IF (LCVRA) THEN ! Prognostic dd but not PLIC
  IF (YDOM_NL%LGP.AND. .NOT. YUOM_NL%LGP) THEN
    WRITE(NULERR,'(''UNDER LCVRA, YDOM_NL%LGP REQUIRES YUOM_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_LCVRA_LGP')
  ENDIF
  IF (YDOM_NL%LSP.AND. .NOT. YUOM_NL%LSP) THEN
    WRITE(NULERR,'(''UNDER LCVRA, YDOM_NL%LSP REQUIRES YUOM_NL%LSP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_LCVRA_LSP')
  ENDIF
ENDIF

! * If L3MT or LSTRAPRO, LGP should be .TRUE. for ql,qi,qr,qs.
IF (L3MT.OR.LSTRAPRO) THEN
  IF (.NOT. (YL_NL%LGP.AND.YI_NL%LGP.AND.YR_NL%LGP.AND.YS_NL%LGP)) THEN
    WRITE(NULERR,'(''L3MT AND LSTRAPRO REQUIRE ALL YL_NL%LGP AND&
     & YI_NL%LGP AND YR_NL%LGP AND YS_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_L3MT_LGP_1 ')
  ENDIF
ENDIF

! * If L3MT, LGP should be .TRUE. for historic cloudiness UNEBH.
IF (L3MT) THEN
  IF (.NOT. YUNEBH_NL%LGP) THEN
    WRITE(NULERR,'(''L3MT REQUIRES YUNEBH_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_L3MT_LGP_2 ')
  ENDIF
ENDIF

! * If LCVPRO, LGP should be .TRUE. for updraft variables UOM and UAL.
IF (LCVPRO) THEN
  IF (.NOT. (YUOM_NL%LGP.AND.YUAL_NL%LGP)) THEN
    WRITE(NULERR,'(''LCVPRO REQUIRES ALL YUOM_NL%LGP AND YUAL_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_LCVPRO_LGP ')
  ENDIF
ENDIF

! * If LCDDPRO, LGP should be .TRUE. for downdraft variables DOM and DAL.
IF (LCDDPRO) THEN
  IF (.NOT. (YDOM_NL%LGP.AND.YDAL_NL%LGP)) THEN
    WRITE(NULERR,'(''LCVPRO REQUIRES ALL YDOM_NL%LGP AND YDAL_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_LCDDPRO_LGP ')
  ENDIF
ENDIF

! * If LENTCH, LGP should be .TRUE. for historic entrainment UEN.
IF (LENTCH) THEN
  IF (.NOT.YUEN_NL%LGP) THEN
    WRITE(NULERR,'(''LENTCH REQUIRES YUEN_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.3_LENTCH_LGP ')
  ENDIF
ENDIF

!*       4.4    check that, when LPC_FULL=T and unlagged physics is called,
!               we have not LPT=T and LADV=F (relevant for ARPEGE MF physics,
!               ALARO physics, AROME physics and HIRLAM physics).
!               Case LPT=T and LADV=F is currently not properly coded.

IF ( LPC_FULL.AND.LSLAG.AND.(LMPHYS.OR.LSIMPH) ) THEN

  LLACTIVE=YQ_NL%LGP.OR.YQ_NL%LSP
  IF(LLACTIVE.AND.YQ_NL%LPT.AND.(.NOT.YQ_NL%LADV)) THEN
    WRITE(NULERR,'(''YQ_NL%LPT=T and YQ_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_Q ')
  ENDIF
  LLACTIVE=YO3_NL%LGP.OR.YO3_NL%LSP
  IF(LLACTIVE.AND.YO3_NL%LPT.AND.(.NOT.YO3_NL%LADV)) THEN
    WRITE(NULERR,'(''YO3_NL%LPT=T and YO3_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_O3')
  ENDIF
  LLACTIVE=YL_NL%LGP.OR.YL_NL%LSP
  IF(LLACTIVE.AND.YL_NL%LPT.AND.(.NOT.YL_NL%LADV)) THEN
    WRITE(NULERR,'(''YL_NL%LPT=T and YL_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_L')
  ENDIF
  LLACTIVE=YI_NL%LGP.OR.YI_NL%LSP
  IF(LLACTIVE.AND.YI_NL%LPT.AND.(.NOT.YI_NL%LADV)) THEN
    WRITE(NULERR,'(''YI_NL%LPT=T and YI_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_I')
  ENDIF
  LLACTIVE=YA_NL%LGP.OR.YA_NL%LSP
  IF(LLACTIVE.AND.YA_NL%LPT.AND.(.NOT.YA_NL%LADV)) THEN
    WRITE(NULERR,'(''YA_NL%LPT=T and YA_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_A')
  ENDIF
  LLACTIVE=YR_NL%LGP.OR.YR_NL%LSP
  IF(LLACTIVE.AND.YR_NL%LPT.AND.(.NOT.YR_NL%LADV)) THEN
    WRITE(NULERR,'(''YR_NL%LPT=T and YR_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_R')
  ENDIF
  LLACTIVE=YS_NL%LGP.OR.YS_NL%LSP
  IF(LLACTIVE.AND.YS_NL%LPT.AND.(.NOT.YS_NL%LADV)) THEN
    WRITE(NULERR,'(''YS_NL%LPT=T and YS_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_S')
  ENDIF
  LLACTIVE=YG_NL%LGP.OR.YG_NL%LSP
  IF(LLACTIVE.AND.YG_NL%LPT.AND.(.NOT.YG_NL%LADV)) THEN
    WRITE(NULERR,'(''YG_NL%LPT=T and YG_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_G')
  ENDIF
  LLACTIVE=YH_NL%LGP.OR.YH_NL%LSP
  IF(LLACTIVE.AND.YH_NL%LPT.AND.(.NOT.YH_NL%LADV)) THEN
    WRITE(NULERR,'(''YH_NL%LPT=T and YH_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_H')
  ENDIF
  LLACTIVE=YTKE_NL%LGP.OR.YTKE_NL%LSP
  IF(LLACTIVE.AND.YTKE_NL%LPT.AND.(.NOT.YTKE_NL%LADV)) THEN
    WRITE(NULERR,'(''YTKE_NL%LPT=T and YTKE_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_TKE')
  ENDIF
  LLACTIVE=YSRC_NL%LGP.OR.YSRC_NL%LSP
  IF(LLACTIVE.AND.YSRC_NL%LPT.AND.(.NOT.YSRC_NL%LADV)) THEN
    WRITE(NULERR,'(''YSRC_NL%LPT=T and YSRC_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_SRC')
  ENDIF
  LLACTIVE=YCPF_NL%LGP.OR.YCPF_NL%LSP
  IF(LLACTIVE.AND.YCPF_NL%LPT.AND.(.NOT.YCPF_NL%LADV)) THEN
    WRITE(NULERR,'(''YCPF_NL%LPT=T and YCPF_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_CPF')
  ENDIF
  LLACTIVE=YSPF_NL%LGP.OR.YSPF_NL%LSP
  IF(LLACTIVE.AND.YSPF_NL%LPT.AND.(.NOT.YSPF_NL%LADV)) THEN
    WRITE(NULERR,'(''YSPF_NL%LPT=T and YSPF_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_SPF')
  ENDIF
  LLACTIVE=YCVGQ_NL%LGP.OR.YCVGQ_NL%LSP
  IF(LLACTIVE.AND.YCVGQ_NL%LPT.AND.(.NOT.YCVGQ_NL%LADV)) THEN
    WRITE(NULERR,'(''YCVGQ_NL%LPT=T and YCVGQ_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_CVGQ')
  ENDIF
  LLACTIVE=YQVA_NL%LGP.OR.YQVA_NL%LSP
  IF(LLACTIVE.AND.YQVA_NL%LPT.AND.(.NOT.YQVA_NL%LADV)) THEN
    WRITE(NULERR,'(''YQVA_NL%LPT=T and YQVA_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_QVA')
  ENDIF
  LLACTIVE=YSDSAT_NL%LGP.OR.YSDSAT_NL%LSP
  IF(LLACTIVE.AND.YSDSAT_NL%LPT.AND.(.NOT.YSDSAT_NL%LADV)) THEN
    WRITE(NULERR,'(''YSDSAT_NL%LPT=T and YSDSAT_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_SDSAT')
  ENDIF
  LLACTIVE=YCVV_NL%LGP.OR.YCVV_NL%LSP
  IF(LLACTIVE.AND.YCVV_NL%LPT.AND.(.NOT.YCVV_NL%LADV)) THEN
    WRITE(NULERR,'(''YCVV_NL%LPT=T and YCVV_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_CVV')
  ENDIF
  LLACTIVE=YUOM_NL%LGP.OR.YUOM_NL%LSP
  IF(LLACTIVE.AND.YUOM_NL%LPT.AND.(.NOT.YUOM_NL%LADV)) THEN
    WRITE(NULERR,'(''YUOM_NL%LPT=T and YUOM_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_UOM')
  ENDIF
  LLACTIVE=YUAL_NL%LGP.OR.YUAL_NL%LSP
  IF(LLACTIVE.AND.YUAL_NL%LPT.AND.(.NOT.YUAL_NL%LADV)) THEN
    WRITE(NULERR,'(''YUAL_NL%LPT=T and YUAL_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_UAL')
  ENDIF
  LLACTIVE=YDOM_NL%LGP.OR.YDOM_NL%LSP
  IF(LLACTIVE.AND.YDOM_NL%LPT.AND.(.NOT.YDOM_NL%LADV)) THEN
    WRITE(NULERR,'(''YDOM_NL%LPT=T and YDOM_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_DOM')
  ENDIF
  LLACTIVE=YDAL_NL%LGP.OR.YDAL_NL%LSP
  IF(LLACTIVE.AND.YDAL_NL%LPT.AND.(.NOT.YDAL_NL%LADV)) THEN
    WRITE(NULERR,'(''YDAL_NL%LPT=T and YDAL_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_DAL')
  ENDIF
  LLACTIVE=YUEN_NL%LGP.OR.YUEN_NL%LSP
  IF(LLACTIVE.AND.YUEN_NL%LPT.AND.(.NOT.YUEN_NL%LADV)) THEN
    WRITE(NULERR,'(''YUEN_NL%LPT=T and YUEN_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_UEN')
  ENDIF
  LLACTIVE=YUNEBH_NL%LGP.OR.YUNEBH_NL%LSP
  IF(LLACTIVE.AND.YUNEBH_NL%LPT.AND.(.NOT.YUNEBH_NL%LADV)) THEN
    WRITE(NULERR,'(''YUNEBH_NL%LPT=T and YUNEBH_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_UNEBH')
  ENDIF

  DO JGFL=1,JPGHG
    LLACTIVE=YGHG_NL(JGFL)%LGP.OR.YGHG_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YGHG_NL(JGFL)%LPT.AND.(.NOT.YGHG_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For greenhouse gas number',JGFL
      WRITE(NULERR,'(''YGHG_NL%LPT=T and YGHG_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_GHG')
    ENDIF
  ENDDO

  DO JGFL=1,JPTRAC
    LLACTIVE=YTRAC_NL(JGFL)%LGP.OR.YTRAC_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YTRAC_NL(JGFL)%LPT.AND.(.NOT.YTRAC_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For diagnostic tracer number',JGFL
      WRITE(NULERR,'(''YTRAC_NL%LPT=T and YTRAC_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_TRAC')
    ENDIF
  ENDDO

  LLACTIVE=YLRCH4_NL%LGP.OR.YLRCH4_NL%LSP
  IF(LLACTIVE.AND.YLRCH4_NL%LPT.AND.(.NOT.YLRCH4_NL%LADV)) THEN
    WRITE(NULERR,'(A,I3)') ' For CH4 loss rate'
    WRITE(NULERR,'(''YLRCH4_NL%LPT=T and YLRCH4_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_LRCH4')
  ENDIF

  LLACTIVE=YCH4S_NL%LGP.OR.YCH4S_NL%LSP
  IF(LLACTIVE.AND.YCH4S_NL%LPT.AND.(.NOT.YCH4S_NL%LADV)) THEN
    WRITE(NULERR,'(A,I3)') ' For CH4 atm. sink'
    WRITE(NULERR,'(''YCH4S_NL%LPT=T and YCH4S_NL%LADV=F '')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.4_CH4S')
  ENDIF

  DO JGFL=1,JPGRG
    LLACTIVE=YGRG_NL(JGFL)%LGP.OR.YGRG_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YGRG_NL(JGFL)%LPT.AND.(.NOT.YGRG_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For reactive gas number',JGFL
      WRITE(NULERR,'(''YGRG_NL%LPT=T and YGRG_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_GRG')
    ENDIF
  ENDDO

  DO JGFL=1,JPAERO
    LLACTIVE=YAERO_NL(JGFL)%LGP.OR.YAERO_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YAERO_NL(JGFL)%LPT.AND.(.NOT.YAERO_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For aerosol number',JGFL
      WRITE(NULERR,'(''YAERO_NL%LPT=T and YAERO_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_AERO')
    ENDIF
  ENDDO

  DO JGFL=1,JPERA40
    LLACTIVE=YERA40_NL(JGFL)%LGP.OR.YERA40_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YERA40_NL(JGFL)%LPT.AND.(.NOT.YERA40_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For ERA40 field number',JGFL
      WRITE(NULERR,'(''YERA40_NL%LPT=T and YERA40_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_ERA40')
    ENDIF
  ENDDO

  DO JGFL=1,JPNOGW
    LLACTIVE=YNOGW_NL(JGFL)%LGP.OR.YNOGW_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YNOGW_NL(JGFL)%LPT.AND.(.NOT.YNOGW_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For NOGW field number',JGFL
      WRITE(NULERR,'(''YNOGW_NL%LPT=T and YNOGW_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_NOGW')
    ENDIF
  ENDDO

  DO JGFL=1,JPUVP
    LLACTIVE=YUVP_NL(JGFL)%LGP.OR.YUVP_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YUVP_NL(JGFL)%LPT.AND.(.NOT.YUVP_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For UVP field number',JGFL
      WRITE(NULERR,'(''YUVP_NL%LPT=T and YUVP_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_UVP')
    ENDIF
  ENDDO

  DO JGFL=1,JPAEROUT
    LLACTIVE=YAEROUT_NL(JGFL)%LGP.OR.YAEROUT_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YAEROUT_NL(JGFL)%LPT.AND.(.NOT.YAEROUT_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For AEROUT field number',JGFL
      WRITE(NULERR,'(''YAEROUT_NL%LPT=T and YAEROUT_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_AEROUT')
    ENDIF
  ENDDO

  DO JGFL=1,JPSLDIA
    LLACTIVE=YSLDIA_NL(JGFL)%LGP.OR.YSLDIA_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YSLDIA_NL(JGFL)%LPT.AND.(.NOT.YSLDIA_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For SLDIA field number',JGFL
      WRITE(NULERR,'(''YSLDIA_NL%LPT=T and YSLDIA_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_SLDIA')
    ENDIF
  ENDDO

  DO JGFL=1,JPFORC
    LLACTIVE=YFORC_NL(JGFL)%LGP.OR.YFORC_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YFORC_NL(JGFL)%LPT.AND.(.NOT.YFORC_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For forcing number',JGFL
      WRITE(NULERR,'(''YFORC_NL%LPT=T and YFORC_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_FORC')
    ENDIF
  ENDDO

  DO JGFL=1,JPEZDIAG
    LLACTIVE=YEZDIAG_NL(JGFL)%LGP.OR.YEZDIAG_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YEZDIAG_NL(JGFL)%LPT.AND.(.NOT.YEZDIAG_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For easy diagnostic number',JGFL
      WRITE(NULERR,'(''YEZDIAG_NL%LPT=T and YEZDIAG_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_EZDIAG')
    ENDIF
  ENDDO

  DO JGFL=1,SIZE(YEXT_NL)
    LLACTIVE=YEXT_NL(JGFL)%LGP.OR.YEXT_NL(JGFL)%LSP
    IF(LLACTIVE.AND.YEXT_NL(JGFL)%LPT.AND.(.NOT.YEXT_NL(JGFL)%LADV)) THEN
      WRITE(NULERR,'(A,I3)') ' For extra-GFL number',JGFL
      WRITE(NULERR,'(''YEXT_NL%LPT=T and YEXT_NL%LADV=F '')')
      CALL ABOR1(' SUCTRL_GFLATTR 4.4_EXT')
    ENDIF
  ENDDO

ENDIF

!*       4.5    additional checkings

IF(YQ_NL%LGP.AND..NOT.LSPRT.AND..NOT.LCANARI) THEN
  WRITE(NULERR,'(''YQ_NL%LGP TRUE ONLY WORKS WITH LSPRT TRUE'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.5_Q')
ENDIF
IF(LMPHYS.AND.(LNEBN.OR.LNEBR.OR.LRRGUST).AND.(.NOT.YCPF_NL%LGP)) THEN
  WRITE(NULERR,'(''WHEN MF PHYSICS IS ACTIVATED'')')
  WRITE(NULERR,'(''LNEBN.OR.LNEBR.OR.LRRGUST REQUIRES YCPF_NL%LGP'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.5_CPF')
ENDIF
IF(LMPHYS.AND.LRRGUST.AND.(.NOT.YSPF_NL%LGP)) THEN
  WRITE(NULERR,'(''WHEN MF PHYSICS IS ACTIVATED'')')
  WRITE(NULERR,'(''LRRGUST REQUIRES YSPF_NL%LGP'')')
  CALL ABOR1(' SUCTRL_GFLATTR 4.5_SPF')
ENDIF
IF(LEPHYS.AND.LEPCLD) THEN
  IF(.NOT.(YL_NL%LGP.AND.YI_NL%LGP.AND.YA_NL%LGP)) THEN
    WRITE(NULERR,'(''WHEN ECMWF PHYSICS IS ACTIVATED'')')
    WRITE(NULERR,'(''LEPCLD REQUIRES YL_NL%LGP.AND.YI_NL%LGP.AND.YA_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.5_A')
  ENDIF
ENDIF

IF(LMPHYS.AND.LPROCLD) THEN
  IF(.NOT.(YL_NL%LGP.AND.YI_NL%LGP.AND.YR_NL%LGP)) THEN
    WRITE(NULERR,'(''WHEN FRENCH PHYSICS IS ACTIVATED'')')
    WRITE(NULERR,'(''LPROCLD REQUIRES YL_NL%LGP.AND.YI_NL%LGP.AND.YR_NL%LGP'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.5_L_I_R')
  ENDIF
ENDIF

! ky: definition of this local variable must be the same everywhere in the code.
LLCOMPUTE_CVGQ=LMPHYS.AND.(.NOT.LMPA)
IF (LLCOMPUTE_CVGQ) THEN
  IF (YQ_NL%LGP .AND. .NOT. (YCVGQ_NL%LGP .OR. YCVGQ_NL%LSP) &
   & .AND. (NSTOP>0.OR.NEINI>0)) THEN
    WRITE(NULERR,'(''WHEN NON-AROME FRENCH PHYSICS IS ACTIVATED'')')
    WRITE(NULERR,'(''YQ GRID POINT REQUIRES YCVGQ ACTIVE'')')
    CALL ABOR1(' SUCTRL_GFLATTR 4.5_CVGQ')
  ENDIF
ENDIF      

! * Use LSPRT=T and grid-point "q" with an Eulerian advection scheme?
!   LSPRT=T with LSLAG=F can be used only if one of the following
!    conditions is realized:
!    - LSFORC=T (in this case horizontal advections are zero).
!    - NCONF >=900
!    - NSTOP=0
!    - no advected grid-point humidity
IF (LSPRT.AND.(.NOT.LSLAG)) THEN
  IF((.NOT.LSFORC).AND.(NCONF<900).AND.(NSTOP>0) &
   & .AND.(YQ_NL%LGP).AND.(YQ_NL%LADV)) THEN
    WRITE (NULERR,*) &
     & 'SUCTRL_GFLATTR: case where LSPRT=T cannot be used with an EUL scheme'
    CALL ABOR1('SUCTRL_GFLATTR: 4.5 No LSPRT in EUL')
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       5.     CHECK for some GFL that N[GFL class] < or = JP[GFL class]
!               ---------------------------------------------------------

!*       5.1    Check aerosols.

IF(NAERO > JPAERO) THEN
  WRITE(NULERR,*)' Too many aerosols have been asked for!',&
   &' Increase parameter JPAERO in yom_ygfl.F90',&
   &' and JPOSAERO in parfpos.F90 to at least NAERO.' 
  CALL ABOR1('SUCTRL_GFLATTR 5.1')
ENDIF

!*       5.2    Check greenhouse gases.

IF(NGHG > JPGHG) THEN
  WRITE(NULERR,*)' Too many greenhouse gases have been asked for!',&
   &' Increase parameter JPGHG in yom_ygfl.F90',&
   &' and JPOSGHG in parfpos.F90 to at least NGHG.' 
  CALL ABOR1('SUCTRL_GFLATTR 5.2')
ENDIF

!*       5.3    Check tracers.

IF(NTRAC > JPTRAC) THEN
  WRITE(NULERR,*)' Too many tracers have been asked for!',&
   &' Increase parameter JPTRAC in yom_ygfl.F90',&
   &' and JPOSTRAC in parfpos.F90 to at least NTRAC.'
  CALL ABOR1('SUCTRL_GFLATTR 5.3')
ENDIF

!*       5.4    Check reactive gases.

IF(NGRG > JPGRG) THEN
  WRITE(NULERR,*)' Too many reactive gases have been asked for!',&
   &' Increase parameter JPGRG in yom_ygfl.F90',&
   &' and JPOSGRG in parfpos.F90 to at least NGRG.' 
  CALL ABOR1('SUCTRL_GFLATTR 5.4')
ENDIF

!*       5.5    Check forcings.

IF(NGFL_FORC > JPFORC) THEN
  WRITE(NULERR,*)' Too many forcings have been asked for!',&
   &' Increase parameter JPFORC in yom_ygfl.F90',&
   &' and JPOSFORC in parfpos.F90 to at least NGFL_FORC.'
  CALL ABOR1('SUCTRL_GFLATTR 5.5')
ENDIF

!*       5.6    Check easy diagnostics.

IF(NGFL_EZDIAG > JPEZDIAG) THEN
  WRITE(NULERR,*)' Too many easy diagnostics have been asked for!',&
   &' Increase parameter JPEZDIAG in yom_ygfl.F90',&
   &' and JPOSEZDIAG in parfpos.F90 to at least NGFL_EZDIAG.'
  CALL ABOR1('SUCTRL_GFLATTR 5.6')
ENDIF

!*       5.7    Check ERA40 fields.

IF(NERA40 > JPERA40) THEN
  WRITE(NULERR,*)' Too many ERA40 GFL fields have been asked for!',&
   &' Increase parameter JPERA40 in yom_ygfl.F90',&
   &' and JPOSERA40 in parfpos.F90 to at least NERA40.'
  CALL ABOR1('SUCTRL_GFLATTR 5.7')
ENDIF
! Additional checking for this class of GFL: currently only NERA40=0 or
!  NERA40=14 is allowed.
IF(NERA40 /= 0 .AND. NERA40 /= 14 .AND. NERA40 /= 4 ) THEN
  WRITE(NULERR,*) ' Allowed values for NERA40 are 0,4,14 '
  CALL ABOR1('SUCTRL_GFLATTR 5.7b')
ENDIF

!*       5.8    Check NORO GWD SCHEME fields.

IF(NNOGW > JPNOGW) THEN
  WRITE(NULERR,*)' Too many NORO GWD SCHEME fields have been asked for!',&
   &' Increase parameter JPNOGW in yom_ygfl.F90',&
   &' and JPOSNOGW in parfpos.F90 to at least NNOGW.'
  CALL ABOR1('SUCTRL_GFLATTR 5.8')
ENDIF

!*       5.9    Check aerosol output fields.

IF(NUVP > JPUVP) THEN
  WRITE(NULERR,*)' Too many aerosol output fields have been asked for!',&
   &' Increase parameter JPUVP in yom_ygfl.F90',&
   &' and JPOSUVP in parfpos.F90 to at least NUVP.'
  CALL ABOR1('SUCTRL_GFLATTR 5.9')
ENDIF

!*       5.10   Check UV processor output fields.

IF(NAEROUT > JPAEROUT) THEN
  WRITE(NULERR,*)' Too many UV processor output fields have been asked for!',&
   &' Increase parameter JPAEROUT in yom_ygfl.F90',&
   &' and JPOSAEROUT in parfpos.F90 to at least NAEROUT.'
  CALL ABOR1('SUCTRL_GFLATTR 5.10')
ENDIF

!*       5.11   Check semi-Lagrangian diagnostics fields.

IF(NSLDIA > JPSLDIA) THEN
  WRITE(NULERR,*)' Too many semi-Lagrangian diagnostics fields have been asked for!',&
   &' Increase parameter JPSLDIA in yom_ygfl.F90',&
   &' and JPOSSLDIA in parfpos.F90 to at least NSLDIA.'
  CALL ABOR1('SUCTRL_GFLATTR 5.11')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUCTRL_GFLATTR',1,ZHOOK_HANDLE)
END SUBROUTINE SUCTRL_GFLATTR
