SUBROUTINE SUCUCONV_CA(KSTEP)

!**** *SUCUCONV_CA*  - Control Cellular Automaton for convective forcing

!     Purpose.
!     --------
!           Cellular automaton for convection control routine

!**   Interface.
!     ----------
!        *CALL* *SUCUCONV_CA

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        COMMON YOE_CUCONVCA

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------
!        

!     Author.
!     -------
!        P. Bechtold Feb. 2009 copied on J Berner previous routines surand* 

!     Modifications.
!     --------------
!        M. Steinheimer Mar-Aug 2009   changed to probabilistic CA, added optional
!                                      wind dependency, bilinear interpolation, ...
!     ------------------------------------------------------------------

  USE PARKIND1  ,ONLY : JPIM     ,JPRB
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
  USE YOMDIM   , ONLY : NPROMA
  USE YOMGEM   , ONLY : NGPTOT, NGPTOTG
  USE RANDOM_NUMBERS_MIX, ONLY : UNIFORM_DISTRIBUTION
  USE YOE_CUCONVCA , ONLY :  NLIVES,NIJH,&
   & RCUCONVCA, RNLCONVCA, RWASALIVE,&
   & NCELLCU, NFERTCU, RWGHTCU,yd_random_stream_CA,&
   & LCA_ADVECT, CA_WIND, LCA_TEST, NFRCASEED, LCA_GLOBAL, RCA_SEEDPROB, &
   & INITIALIZE_CELLS,write_field, &
   & UPDCELAUT_RGG, VISUALIZE_NEIGHBOURS, VISUALIZE_NEIGHBOURS_LS, &
   & VISUALIZE_NRNEIGHBOURS, VISUALIZE_NRNEIGHBOURS_LS, NTESTPROC, NTESTGP, &
   & NDXUNREAL, NDYUNREAL, NSPINUP, LCA_RANTROP
  USE YOMGMV   , ONLY : YPH9 , GMV
  USE YOECUMF  , ONLY : NJKT4, NJKT5
  USE STOPH_MIX , ONLY : LEXTRAFIELDS, RSTOPHCA, LSTOPH_CASBS
!stuff for SL-halo
  USE YOMMP , ONLY : MYPROC, NASLB1, NSLPROCS  ,&
                   & NSLRPT, NSLSPT, NSLSENDPOS,&
                   & NSLRECVPOS, NSENDPTR  ,NRECVPTR  ,NSLCOMM, &
                   & NSLCORE
  IMPLICIT NONE

!     ------------------------------------------------------------------
  INTEGER(KIND=JPIM),INTENT(IN)   :: KSTEP ! model time step

  INTEGER(KIND=JPIM)              :: IT, ISPINUP
  INTEGER(KIND=JPIM)              :: JROF

  INTEGER(KIND=JPIM), ALLOCATABLE :: ILIVES(:),IINI(:)
  REAL(KIND=JPRB), ALLOCATABLE    :: ZU(:,:), ZV(:,:)
  INTEGER(KIND=JPIM), ALLOCATABLE :: IDX(:), IDY(:)
  REAL(KIND=JPRB), ALLOCATABLE    :: ZRAND1DLS(:),ZRAND1DSS(:)
  REAL(KIND=JPRB)                 :: ZHOOK_HANDLE

  INTEGER(KIND=JPIM)              :: imainproc
  character(LEN=25)                  :: clfname

  INTEGER(KIND=JPIM)              :: JSS, JLS

  INTEGER(KIND=JPIM)              :: JK, JI, ICEND, IBL, IOFF, IGP


!stuff for SL-halo
  LOGICAL                         :: LLINC
  INTEGER(KIND=JPIM)              :: IDUMARR(2)
  INTEGER(KIND=JPIM)              :: IFIXSFLD(2)
  INTEGER(KIND=JPIM)              :: IFLDSLB1
  REAL(KIND=JPRB), ALLOCATABLE    :: ZPB1(:,:)
  REAL(KIND=JPRB), ALLOCATABLE    :: ZFERT(:,:),ZCELL(:,:)
  REAL(KIND=JPRB), ALLOCATABLE    :: ZCUCONVCAG(:,:),ZCUCONVCA(:,:),ZTMP(:,:)

#include "gathergpf.intfb.h"
#include "slcomm.intfb.h"

  IF (LHOOK) CALL DR_HOOK('SUCUCONV_CA',0,ZHOOK_HANDLE)

!  WRITE(NULOUT,'("xxxx in SUCUCONV_CA at step ",I4)') kstep
!  CALL FLUSH(NULOUT)
!     ------------------------------------------------------
!*       1.0  Setup constants and allocate arrays
!     ------------------------------------------------------


  ALLOCATE(ILIVES(NGPTOT),IINI(NGPTOT))

  ALLOCATE(IDX(NGPTOT),IDY(NGPTOT))

  ALLOCATE(ZRAND1DSS(NIJH*NIJH*NGPTOT),ZRAND1DLS(NGPTOT))

  ILIVES=0
  IINI=0

!stuff for SL halo
  LLINC=.FALSE.
  IFIXSFLD(:)=0
  IFLDSLB1=2*NIJH*NIJH
  ALLOCATE(ZPB1(NASLB1,IFLDSLB1))
  ALLOCATE(ZFERT(NIJH*NIJH,NASLB1))
  ALLOCATE(ZCELL(NIJH*NIJH,NASLB1))



!     ------------------------------------------------------
!*       2.0  prepare data
!     ------------------------------------------------------

!*       2.1  get U and V from GMV array to local array
!     ------------------------------------------------------
  ALLOCATE(ZU(NGPTOT,1), ZV(NGPTOT,1))


  IF( CA_WIND == "REAL" ) THEN
    ! calculate mean wind of 850hPa (model level NJKT4) and 500hPa (ml=NJKT5)
    ! and reshape wind arrays from GMV(NPROMA,NFLEVG,NDIMGMV,NGPBLKS) to ZU(NGPTOT,1)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JK,ICEND,IBL,IOFF,JI,IGP)
    DO JK=1,NGPTOT,NPROMA
      ICEND=MIN(NPROMA,NGPTOT-JK+1)
      IBL=(JK-1)/NPROMA+1
      IOFF=JK
      DO JI=1,ICEND
        IGP=IOFF+JI-1
        ZU(IGP,1)=  0.5*(GMV(JI,NJKT4,YPH9%MU,IBL) + GMV(JI,NJKT5,YPH9%MU,IBL) )
        ZV(IGP,1)=  0.5*(GMV(JI,NJKT4,YPH9%MV,IBL) + GMV(JI,NJKT5,YPH9%MV,IBL) )

        IDX(IGP)=NINT(SIGN(1._JPRB,ZU(IGP,1)))
        IDY(IGP)=NINT(SIGN(1._JPRB,ZV(IGP,1)))
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
  ELSE
    !option of idealized wind for testing
    IDX= NDXUNREAL
    IDY= NDYUNREAL
  ENDIF


!     ------------------------------------------------------
!*       3.0  Evolve the CA
!     ------------------------------------------------------

  IF (LCA_GLOBAL) THEN
    IF (KSTEP==1) THEN
      ISPINUP=NSPINUP
    ELSE
      ISPINUP=1
    ENDIF
  ELSE
    ISPINUP=1           !spin-up currently not used for convective CA
    IF (LCA_RANTROP .AND. KSTEP==1) ISPINUP=NSPINUP
  ENDIF

  DO IT=1,ISPINUP
      IF (LCA_GLOBAL) THEN
        IF(MOD(KSTEP,NFRCASEED) == 0 .AND. KSTEP > 0 .OR. KSTEP == 1) THEN
          CALL UNIFORM_DISTRIBUTION (ZRAND1DLS,YD_RANDOM_STREAM_CA)
        !$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JROF)
          DO JROF=1,NGPTOT
              IF (ZRAND1DLS(JROF) >= RCA_SEEDPROB ) THEN
                IINI(JROF)= 1
              ELSE
                IINI(JROF)=0
              ENDIF
          ENDDO
        !$OMP END PARALLEL DO
        ENDIF
        ILIVES(:)=NLIVES
      ELSE
          IF (LCA_TEST) THEN
            RCUCONVCA(:)=0         !these 3 lines can be used to initialize only a single
            IF (MYPROC == NTESTPROC ) RCUCONVCA(NTESTGP)=1     !gaussian gridpoint for testing
            RNLCONVCA(:)=NLIVES
          ENDIF
          IF (LCA_RANTROP) THEN
            CALL UNIFORM_DISTRIBUTION (ZRAND1DLS,YD_RANDOM_STREAM_CA)
           !$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JROF)
            DO JROF=1,NGPTOT
              ILIVES(JROF)= INT(RNLCONVCA(JROF))
              IF (ZRAND1DLS(JROF) >= RCA_SEEDPROB .AND. RCUCONVCA(JROF)==1._JPRB) THEN
                IINI(JROF)= 1
              ELSE
                IINI(JROF)=0
              ENDIF
            ENDDO
           !$OMP END PARALLEL DO
          ELSE
           !$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JROF)
            DO JROF=1,NGPTOT
              ILIVES(JROF)= INT(RNLCONVCA(JROF))
              IINI(JROF)= INT(RCUCONVCA(JROF))
            ENDDO
           !$OMP END PARALLEL DO
          ENDIF
      ENDIF

!     ------------------------------------------------------
!*       3.1  Initialise the cellular automaton from convection
!     ------------------------------------------------------
    IF (LCA_GLOBAL) THEN
      IF(KSTEP==0) CALL INITIALIZE_CELLS(IINI,ILIVES,NCELLCU,NFERTCU)
    ELSE
      !Initialize CA at first step (at step 0 physics field not yet filled)
      IF(KSTEP==1) THEN
        CALL INITIALIZE_CELLS(IINI,ILIVES,NCELLCU,NFERTCU)
      ENDIF
    ENDIF
  !Initialize CA at each NFRCASEED'th step
    IF(MOD(KSTEP,NFRCASEED) == 0 .AND. KSTEP > 0) THEN
      CALL INITIALIZE_CELLS(IINI,ILIVES,NCELLCU,NFERTCU)
    ENDIF

!  WRITE(NULOUT,'("xxxx CA initialized")')
!  CALL FLUSH(NULOUT)


!*       3.2  get halo for CA
!     ------------------------------------------------------


  ZPB1=0
  ZFERT=0
  ZCELL=0


!fill buffer
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JLS,JSS)
  DO JLS=1,NGPTOT
!    WRITE(NULOUT,'("xxxx ",3I8)') NGPTOT,NASLB1,NSLCORE(JLS)
    DO JSS=1,NIJH*NIJH
      ZPB1(NSLCORE(JLS),JSS)=NCELLCU(JSS,JLS)
      ZPB1(NSLCORE(JLS),JSS+NIJH*NIJH)=NFERTCU(JSS,JLS)
    ENDDO
  ENDDO
!$OMP END PARALLEL DO

!  WRITE(NULOUT,'("xxxx SL buffer set up")')
!  CALL FLUSH(NULOUT)

!get halo
  CALL SLCOMM(NASLB1,NSLPROCS,NSLRPT,&
       & NSLSPT,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM,&
       & IDUMARR,&
       & IFLDSLB1,LLINC,0,ZPB1)

!  WRITE(NULOUT,'("xxxx SL-Halo retrieved")')
!  CALL FLUSH(NULOUT)

!extract from buffer
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JLS,JSS)
  DO JLS=1,NASLB1
    DO JSS=1,NIJH*NIJH
      ZCELL(JSS,JLS)=ZPB1(JLS,JSS)
      ZFERT(JSS,JLS)=ZPB1(JLS,JSS+NIJH*NIJH)
    ENDDO
  ENDDO
!$OMP END PARALLEL DO

!  WRITE(NULOUT,'("xxxx buffer extracted")')
!  CALL FLUSH(NULOUT)


!     ------------------------------------------------------
!*       3.3  Update cellular automaton 
!             and transform fields back to reduced Gaussian grid
!     ------------------------------------------------------


   ! create random numbers
    CALL UNIFORM_DISTRIBUTION (ZRAND1DSS,YD_RANDOM_STREAM_CA)

    CALL UPDCELAUT_RGG(ILIVES,IDX,IDY,NFERTCU,NCELLCU,ZFERT,ZCELL,RWGHTCU,ZRAND1DSS,LCA_ADVECT)
  ENDDO  !end spin-up loop

!  WRITE(NULOUT,'("xxxx CA updated")')
!  CALL FLUSH(NULOUT)

  !set WASALIVE field to one at gridpoints where CA was active
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JROF)
  DO JROF=1,NGPTOT
    RCUCONVCA(JROF)=RWGHTCU(JROF)
    IF (LSTOPH_CASBS) RSTOPHCA(JROF)=RWGHTCU(JROF)
    IF (RCUCONVCA(JROF) > 0) RWASALIVE(JROF)=1
    RNLCONVCA(JROF)=RWASALIVE(JROF)
  ENDDO
!$OMP END PARALLEL DO



IF (LEXTRAFIELDS) THEN
  ALLOCATE(ZCUCONVCA(NGPTOT,1),ZCUCONVCAG(NGPTOTG,1))
  imainproc=1

  ZCUCONVCA(:,1)=RCUCONVCA

!get whole fields on 1 processor

  CALL GATHERGPF(ZCUCONVCA(:,1),ZCUCONVCAG,1,imainproc)
!binary output of wind for debugging
  IF (imainproc == MYPROC) THEN
     write(clfname,'(A,I4.4,A)') 'CA',KSTEP,'.dat'
     call write_field(clfname, REAL(ZCUCONVCAG(:,1),JPRB), NGPTOTG, 1)
  ENDIF




  CALL VISUALIZE_NEIGHBOURS_LS(853,ZCUCONVCA(:,1))
  CALL GATHERGPF(ZCUCONVCA(:,1),ZCUCONVCAG,1,imainproc)
  IF (imainproc == MYPROC) THEN
     write(clfname,'(A,I4.4,A)') 'NBLS',KSTEP,'.dat'
     call write_field(clfname, REAL(ZCUCONVCAG(:,1),JPRB), NGPTOTG, 1)
  ENDIF

  CALL VISUALIZE_NRNEIGHBOURS_LS(ZCUCONVCA(:,1))
  CALL GATHERGPF(ZCUCONVCA(:,1),ZCUCONVCAG,1,imainproc)
  IF (imainproc == MYPROC) THEN
     write(clfname,'(A,I4.4,A)') 'NRNBLS',KSTEP,'.dat'
     call write_field(clfname, REAL(ZCUCONVCAG(:,1),JPRB), NGPTOTG, 1)
  ENDIF

  DEALLOCATE(ZCUCONVCA,ZCUCONVCAG)

  ALLOCATE(ZCUCONVCA(NGPTOT,NIJH*NIJH),ZCUCONVCAG(NGPTOTG,NIJH*NIJH),ZTMP(NGPTOTG,1))

  DO JK=1,NIJH*NIJH
    ZCUCONVCA(:,JK)=NCELLCU(JK,:)
    CALL GATHERGPF(ZCUCONVCA(:,JK),ZTMP,1,imainproc)
    ZCUCONVCAG(:,JK)=ZTMP(:,1)
  ENDDO
  IF (imainproc == MYPROC) THEN
     write(clfname,'(A,I4.4,A)') 'CASS',KSTEP,'.dat'
     call write_field(clfname, REAL(ZCUCONVCAG(:,:),JPRB), NGPTOTG,NIJH*NIJH)
  ENDIF

  CALL VISUALIZE_NEIGHBOURS(853,1,ZCUCONVCA)
  DO JK=1,NIJH*NIJH
    CALL GATHERGPF(ZCUCONVCA(:,JK),ZTMP,1,imainproc)
    ZCUCONVCAG(:,JK)=ZTMP(:,1)
  ENDDO
  IF (imainproc == MYPROC) THEN
     write(clfname,'(A,I4.4,A)') 'NBSS',KSTEP,'.dat'
     call write_field(clfname, REAL(ZCUCONVCAG(:,:),JPRB), NGPTOTG,NIJH*NIJH)
  ENDIF

  CALL VISUALIZE_NRNEIGHBOURS(ZCUCONVCA)
  DO JK=1,NIJH*NIJH
    CALL GATHERGPF(ZCUCONVCA(:,JK),ZTMP,1,imainproc)
    ZCUCONVCAG(:,JK)=ZTMP(:,1)
  ENDDO
  IF (imainproc == MYPROC) THEN
     write(clfname,'(A,I4.4,A)') 'NRNBSS',KSTEP,'.dat'
     call write_field(clfname, REAL(ZCUCONVCAG(:,:),JPRB), NGPTOTG,NIJH*NIJH)
  ENDIF
  DEALLOCATE(ZCUCONVCA,ZCUCONVCAG,ZTMP)
ENDIF

!     ------------------------------------------------------
!*       4.0 Cleanup
!     ------------------------------------------------------

  DEALLOCATE(ILIVES)
  DEALLOCATE(IINI)

  DEALLOCATE(ZU, ZV)
  DEALLOCATE(IDX, IDY)
  DEALLOCATE(ZRAND1DSS,ZRAND1DLS)

  DEALLOCATE(ZPB1)
  DEALLOCATE(ZFERT)
  DEALLOCATE(ZCELL)
!     ------------------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('SUCUCONV_CA',1,ZHOOK_HANDLE)
END SUBROUTINE SUCUCONV_CA
