SUBROUTINE SUDEFO_GFLATTR

!     ----------------------------------------------------------------------
!**** *SUDEFO_GFLATTR * - Setting up of the default values for GFL attributes

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SUDEFO_GFLATTR*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad (CNRM/GMAP), after old parts 1.1.2a and 1.1.2b of SUDIM1.

!     Modifications.
!     --------------
!      Original : 31-01-2007 (CY32T2)
!      Modified by P. Tulet : 12-04-07 add CO2
!      28-Aug-2007 F. Vana: removing LRSPLINE attribute
!      J.Haseler     27-Feb-2007 Modify defaults for LTRAJIO and LGPINGP
!      JJMorcrette   20060925    DU, BC, OM, SO2, VOL, SOA climatological flds
!      JJMorcrette   20061020    diagnostic of aerosol physical fluxes
!      22-Mar-2007 S.Serrar  : defaults for ERA40 GFL fields
!      27-Jun-2007 E.Holm    : default option for GFL SL TL/AD LADV5 attribute
!      18-Dec-2007 Y.Seity   : add Hail
!      28-Sep-2008 Y.Seity   : add LRDEPOS,NSV_DSTDEPBEG,NSV_DSTDEPEND
!      JJMorcrette   20090217  PP of aerosol and UV processor outputs 
!                              (default changed by A. Benedetti on 9-3-2009).
!      01-Oct-2008 R.Forbes  : modified defaults for YR and YS
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LSPRT   ,LELAM    ,LSFORC
USE YOMCT0B  , ONLY : LECMWF
USE YOMARPHY , ONLY : LMPA , LUSECHEM, LRDUST, LORILAM, LRCO2, LRDEPOS
USE YOEPHY   , ONLY : LEPCLD
USE YHLOPTION, ONLY : LHL
USE YOPHNC   , ONLY : LENCLD2
USE YOM_YGFL , ONLY : YQ_NL    ,YI_NL    ,YL_NL    ,YS_NL    ,YR_NL    , &
 & YG_NL    ,YH_NL   ,YTKE_NL  ,YA_NL    ,YO3_NL   ,YEXT_NL  ,YCPF_NL  , &
 & YSPF_NL  , YSRC_NL,YCVGQ_NL ,YAERO_NL ,YGHG_NL  ,YGRG_NL  ,YTRAC_NL , &
 & YSDSAT_NL, YCVV_NL,YFORC_NL ,YQVA_NL  ,YERA40_NL,YNOGW_NL,YEZDIAG_NL,YSLDIA_NL, &
 & YUOM_NL  ,YUAL_NL  ,YDOM_NL ,YDAL_NL  ,YUEN_NL  ,YUNEBH_NL,YGFLC  ,&
 & NGFL_EXT ,JPGFL    ,JPAERO  ,JPGHG    ,JPGRG    ,JPTRAC   , &
 & YCH4S_NL ,YLRCH4_NL,YGRGTEND_NL , YAEROUT_NL, YUVP_NL, &
 & JPFORC, JPEZDIAG ,JPERA40, JPNOGW, JPAEROUT, JPUVP, NSLDIA, NSLDIAGP, &
!>>dod
 & YCDNC_NL, YICNC_NL, YRE_LIQ_NL, YRE_ICE_NL
!<<dod
USE YOMNSV   , ONLY : NSV_CHEMBEG, NSV_CHEMEND, NSV_DSTBEG, NSV_DSTEND, &
 & NSV_DSTDEPBEG, NSV_DSTDEPEND, NSV_AERBEG, NSV_AEREND, NSV_CO2
USE TRAJECTORY_MOD   , ONLY : LTRAJHR
!>>dod
USE YOMECEARTH, ONLY: LECEARTH, LECEARTH_UNCOUP
!<<dod
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JGFL, JGFLN
CHARACTER(LEN=15), DIMENSION(JPGFL) :: CLNAME
REAL(KIND=JPRB) :: ZREFVALC_USELESS,ZREFVALI_USELESS
LOGICAL :: LLEC_GPINGP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "aroini_nsv0.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDEFO_GFLATTR',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!        1a     Set implicit default values for GFL attributes
!               (other than "GHG", "GRG", "TRAC", "AERO", "FORCING",
!               "EASY DIAG" and "EXT").

! * Default for attribute LGP
!   (some of them will be readjusted after some namelists reading).
IF (LECMWF) THEN
  YQ_NL%LGP = .TRUE.
  YL_NL%LGP = .TRUE.
  YI_NL%LGP = .TRUE.
  YS_NL%LGP = .TRUE.
  YR_NL%LGP = .TRUE.
  YG_NL%LGP = .FALSE.
  YH_NL%LGP = .FALSE.
  YTKE_NL%LGP = .FALSE.
  YA_NL%LGP = .TRUE.
  YSRC_NL%LGP= .FALSE.
  IF(ANY(SPREAD(NCONF,1,4) == (/1,302,601,801/))) THEN
    YO3_NL%LGP= .FALSE.
  ELSE
    YO3_NL%LGP= .TRUE.
  ENDIF 
  YCPF_NL%LGP= .FALSE.
  YSPF_NL%LGP= .FALSE.
  YCVGQ_NL%LGP= .FALSE.
  YQVA_NL%LGP= .FALSE.
  YSDSAT_NL%LGP= .FALSE.
  YCVV_NL%LGP= .FALSE.
  YUOM_NL%LGP=.FALSE.
  YUAL_NL%LGP=.FALSE.
  YDOM_NL%LGP=.FALSE.
  YDAL_NL%LGP=.FALSE.
  YUEN_NL%LGP=.FALSE.
  YUNEBH_NL%LGP=.FALSE.
ELSEIF (LHL) THEN
  YQ_NL%LGP = .FALSE.
  YL_NL%LGP = .FALSE.
  YI_NL%LGP = .FALSE.
  YS_NL%LGP = .FALSE.
  YR_NL%LGP = .FALSE.
  YG_NL%LGP = .FALSE.
  YH_NL%LGP = .FALSE.
  YTKE_NL%LGP = .FALSE.
  YA_NL%LGP = .FALSE.
  YSRC_NL%LGP= .FALSE.
  YO3_NL%LGP= .FALSE.
  YCPF_NL%LGP= .FALSE.
  YSPF_NL%LGP= .FALSE.
  YCVGQ_NL%LGP= .FALSE.
  YQVA_NL%LGP= .FALSE.
  YSDSAT_NL%LGP= .FALSE.
  YCVV_NL%LGP= .FALSE.
  YUOM_NL%LGP=.FALSE.
  YUAL_NL%LGP=.FALSE.
  YDOM_NL%LGP=.FALSE.
  YDAL_NL%LGP=.FALSE.
  YUEN_NL%LGP=.FALSE.
  YUNEBH_NL%LGP=.FALSE.
ELSEIF ((.NOT.LECMWF).AND.(.NOT.LHL).AND.LMPA) THEN
  YQ_NL%LGP = .TRUE.
  YL_NL%LGP = .TRUE.
  YI_NL%LGP = .TRUE.
  YS_NL%LGP = .TRUE.
  YR_NL%LGP = .TRUE.
  YG_NL%LGP = .TRUE.
  YH_NL%LGP = .FALSE.
  YTKE_NL%LGP = .TRUE.
  YA_NL%LGP = .TRUE.
  YSRC_NL%LGP= .TRUE.
  YO3_NL%LGP= .FALSE.
  YCPF_NL%LGP= .FALSE.
  YSPF_NL%LGP= .FALSE.
  YCVGQ_NL%LGP= .FALSE.
  YQVA_NL%LGP= .FALSE.
  YSDSAT_NL%LGP= .FALSE.
  YCVV_NL%LGP= .FALSE.
  YUOM_NL%LGP=.FALSE.
  YUAL_NL%LGP=.FALSE.
  YDOM_NL%LGP=.FALSE.
  YDAL_NL%LGP=.FALSE.
  YUEN_NL%LGP=.FALSE.
  YUNEBH_NL%LGP=.FALSE.
ELSE
  YQ_NL%LGP = .FALSE.
  YL_NL%LGP=.FALSE.
  YI_NL%LGP=.FALSE.
  YS_NL%LGP = .FALSE.
  YR_NL%LGP = .FALSE.
  YG_NL%LGP = .FALSE.
  YH_NL%LGP = .FALSE.
  YTKE_NL%LGP = .FALSE.
  YA_NL%LGP = .FALSE.
  YSRC_NL%LGP= .FALSE.
  YO3_NL%LGP= .FALSE.
  YCPF_NL%LGP= .FALSE.
  YSPF_NL%LGP= .FALSE.
  YCVGQ_NL%LGP= .FALSE.
  YQVA_NL%LGP= .FALSE.
  YSDSAT_NL%LGP= .FALSE.
  YCVV_NL%LGP= .FALSE.
  YUOM_NL%LGP=.FALSE.
  YUAL_NL%LGP=.FALSE.
  YDOM_NL%LGP=.FALSE.
  YDAL_NL%LGP=.FALSE.
  YUEN_NL%LGP=.FALSE.
  YUNEBH_NL%LGP=.FALSE.
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LGP=.TRUE.
  YICNC_NL%LGP=.TRUE.
  YRE_LIQ_NL%LGP=.TRUE.
  YRE_ICE_NL%LGP=.TRUE.
END IF
!<<dod

! * Default for attribute LSP
!   (some of them will be readjusted after some namelists reading).
IF (LECMWF) THEN
  YQ_NL%LSP = .FALSE.
  YL_NL%LSP = .FALSE.
  YI_NL%LSP = .FALSE.
  YS_NL%LSP = .FALSE.
  YR_NL%LSP = .FALSE.
  YG_NL%LSP = .FALSE.
  YH_NL%LSP = .FALSE.
  YTKE_NL%LSP = .FALSE.
  YA_NL%LSP = .FALSE.
  YSRC_NL%LSP= .FALSE.
  YO3_NL%LSP= .FALSE.
  YCPF_NL%LSP= .FALSE.
  YSPF_NL%LSP= .FALSE.
  YCVGQ_NL%LSP= .FALSE.
  YQVA_NL%LSP= .FALSE.
  YSDSAT_NL%LSP= .FALSE.
  YCVV_NL%LSP= .FALSE.
  YUOM_NL%LSP=.FALSE.
  YUAL_NL%LSP=.FALSE.
  YDOM_NL%LSP=.FALSE.
  YDAL_NL%LSP=.FALSE.
  YUEN_NL%LSP=.FALSE.
  YUNEBH_NL%LSP=.FALSE.
ELSEIF (LHL) THEN
  YQ_NL%LSP = .TRUE.
  YL_NL%LSP = .TRUE.
  YI_NL%LSP = .TRUE.
  YS_NL%LSP = .FALSE.
  YR_NL%LSP = .FALSE.
  YG_NL%LSP = .FALSE.
  YH_NL%LSP = .FALSE.
  YTKE_NL%LSP = .FALSE.
  YA_NL%LSP = .TRUE.
  YSRC_NL%LSP= .FALSE.
  YO3_NL%LSP= .FALSE.
  YCPF_NL%LSP= .FALSE.
  YSPF_NL%LSP= .FALSE.
  YCVGQ_NL%LSP= .FALSE.
  YQVA_NL%LSP= .TRUE.
  YSDSAT_NL%LSP= .FALSE.
  YCVV_NL%LSP= .FALSE.
  YUOM_NL%LSP=.FALSE.
  YUAL_NL%LSP=.FALSE.
  YDOM_NL%LSP=.FALSE.
  YDAL_NL%LSP=.FALSE.
  YUEN_NL%LSP=.FALSE.
  YUNEBH_NL%LSP=.FALSE.
ELSEIF ((.NOT.LECMWF).AND.(.NOT.LHL).AND.LMPA) THEN
  YQ_NL%LSP = .FALSE.
  YL_NL%LSP = .FALSE.
  YI_NL%LSP = .FALSE.
  YS_NL%LSP = .FALSE.
  YR_NL%LSP = .FALSE.
  YG_NL%LSP = .FALSE.
  YH_NL%LSP = .FALSE.
  YTKE_NL%LSP = .FALSE.
  YA_NL%LSP = .FALSE.
  YSRC_NL%LSP= .FALSE.
  YO3_NL%LSP= .FALSE.
  YCPF_NL%LSP= .FALSE.
  YSPF_NL%LSP= .FALSE.
  YCVGQ_NL%LSP= .FALSE.
  YQVA_NL%LSP= .FALSE.
  YSDSAT_NL%LSP= .FALSE.
  YCVV_NL%LSP= .FALSE.
  YUOM_NL%LSP=.FALSE.
  YUAL_NL%LSP=.FALSE.
  YDOM_NL%LSP=.FALSE.
  YDAL_NL%LSP=.FALSE.
  YUEN_NL%LSP=.FALSE.
  YUNEBH_NL%LSP=.FALSE.
ELSE
  YQ_NL%LSP = .TRUE.
  YL_NL%LSP = .FALSE.
  YI_NL%LSP = .FALSE.
  YS_NL%LSP = .FALSE.
  YR_NL%LSP = .FALSE.
  YG_NL%LSP = .FALSE.
  YH_NL%LSP = .FALSE.
  YTKE_NL%LSP = .FALSE.
  YA_NL%LSP = .FALSE.
  YSRC_NL%LSP= .FALSE.
  YO3_NL%LSP= .FALSE.
  YCPF_NL%LSP= .FALSE.
  YSPF_NL%LSP= .FALSE.
  YCVGQ_NL%LSP= .FALSE.
  YQVA_NL%LSP= .FALSE.
  YSDSAT_NL%LSP= .FALSE.
  YCVV_NL%LSP= .FALSE.
  YUOM_NL%LSP=.FALSE.
  YUAL_NL%LSP=.FALSE.
  YDOM_NL%LSP=.FALSE.
  YDAL_NL%LSP=.FALSE.
  YUEN_NL%LSP=.FALSE.
  YUNEBH_NL%LSP=.FALSE.
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LSP=.FALSE.
  YICNC_NL%LSP=.FALSE.
  YRE_LIQ_NL%LSP=.FALSE.
  YRE_ICE_NL%LSP=.FALSE.
END IF
!<<dod

! * Default for attribute NREQIN
IF (ANY(SPREAD(NCONF,1,4) == (/1,302,601,801/))) THEN
  IF (LECMWF) THEN
    ! ky: this must be checked by ECMWF
    YQ_NL%NREQIN = 1
    YL_NL%NREQIN = 1
    YI_NL%NREQIN = 1
    YS_NL%NREQIN = 1
    YR_NL%NREQIN = 1
    YG_NL%NREQIN = 0
    YH_NL%NREQIN = 0
    YTKE_NL%NREQIN = 0
    YA_NL%NREQIN = 1
    YSRC_NL%NREQIN= 0
    YO3_NL%NREQIN= 1
    YCPF_NL%NREQIN= 0
    YSPF_NL%NREQIN= 0
    YCVGQ_NL%NREQIN= 0
    YQVA_NL%NREQIN= 0
    YSDSAT_NL%NREQIN= 0
    YCVV_NL%NREQIN= 0
    YUOM_NL%NREQIN=0
    YUAL_NL%NREQIN=0
    YDOM_NL%NREQIN=0
    YDAL_NL%NREQIN=0
    YUEN_NL%NREQIN=0
    YUNEBH_NL%NREQIN=0
  ELSEIF ((.NOT.LECMWF).AND. LMPA) THEN
    YQ_NL%NREQIN = 1
    YL_NL%NREQIN = 1
    YI_NL%NREQIN = 1
    YS_NL%NREQIN = 1
    YR_NL%NREQIN = 1
    YG_NL%NREQIN = 1
    YH_NL%NREQIN = 1
    YTKE_NL%NREQIN = -1
    YA_NL%NREQIN = -1
    YSRC_NL%NREQIN= -1
    YO3_NL%NREQIN= 0
    YCPF_NL%NREQIN= 0
    YSPF_NL%NREQIN= 0
    YCVGQ_NL%NREQIN= 0
    YQVA_NL%NREQIN= 0
    YSDSAT_NL%NREQIN= 0
    YCVV_NL%NREQIN= 0
    YUOM_NL%NREQIN=0
    YUAL_NL%NREQIN=0
    YDOM_NL%NREQIN=0
    YDAL_NL%NREQIN=0
    YUEN_NL%NREQIN=0
    YUNEBH_NL%NREQIN=0
  ELSE
    YQ_NL%NREQIN = 1
    YL_NL%NREQIN = 0
    YI_NL%NREQIN = 0
    YS_NL%NREQIN = 0
    YR_NL%NREQIN = 0
    YG_NL%NREQIN = 0
    YH_NL%NREQIN = 0
    YTKE_NL%NREQIN = 0
    YA_NL%NREQIN = 0
    YSRC_NL%NREQIN= 0
    YO3_NL%NREQIN= 0
    YCPF_NL%NREQIN= 0
    YSPF_NL%NREQIN= 0
    YCVGQ_NL%NREQIN= 0
    YQVA_NL%NREQIN= 0
    YSDSAT_NL%NREQIN= 0
    YCVV_NL%NREQIN= 0
    YUOM_NL%NREQIN=0
    YUAL_NL%NREQIN=0
    YDOM_NL%NREQIN=0
    YDAL_NL%NREQIN=0
    YUEN_NL%NREQIN=0
    YUNEBH_NL%NREQIN=0
  ENDIF
ELSE
  YQ_NL%NREQIN = 1
  YL_NL%NREQIN = 0
  YI_NL%NREQIN = 0
  YS_NL%NREQIN = 0
  YR_NL%NREQIN = 0
  YG_NL%NREQIN = 0
  YH_NL%NREQIN = 0
  YTKE_NL%NREQIN = 0
  YA_NL%NREQIN = 0
  YSRC_NL%NREQIN= 0
  YO3_NL%NREQIN= 1 
  YCPF_NL%NREQIN= 0
  YSPF_NL%NREQIN= 0
  YCVGQ_NL%NREQIN= 0
  YQVA_NL%NREQIN= 0
  YSDSAT_NL%NREQIN= 0
  YCVV_NL%NREQIN= 0
  YUOM_NL%NREQIN=0
  YUAL_NL%NREQIN=0
  YDOM_NL%NREQIN=0
  YDAL_NL%NREQIN=0
  YUEN_NL%NREQIN=0
  YUNEBH_NL%NREQIN=0
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%NREQIN=0
  YICNC_NL%NREQIN=0
  YRE_LIQ_NL%NREQIN=-1
  YRE_ICE_NL%NREQIN=-1
END IF
!<<dod

! * Default for attribute REFVALI
!   REFVALI is useful only when NREQIN=-1,
!    but to avoid unitialised values, REFVALI is set to -999 in the other cases.
ZREFVALI_USELESS=-999._JPRB
IF (ANY(SPREAD(NCONF,1,4) == (/1,302,601,801/))) THEN
  IF (LECMWF) THEN
    YQ_NL%REFVALI = ZREFVALI_USELESS
    YL_NL%REFVALI = ZREFVALI_USELESS
    YI_NL%REFVALI = ZREFVALI_USELESS
    YS_NL%REFVALI = ZREFVALI_USELESS
    YR_NL%REFVALI = ZREFVALI_USELESS
    YG_NL%REFVALI = ZREFVALI_USELESS
    YH_NL%REFVALI = ZREFVALI_USELESS
    YTKE_NL%REFVALI = ZREFVALI_USELESS
    YA_NL%REFVALI = ZREFVALI_USELESS
    YSRC_NL%REFVALI= ZREFVALI_USELESS
    YO3_NL%REFVALI= ZREFVALI_USELESS
    YCPF_NL%REFVALI= ZREFVALI_USELESS
    YSPF_NL%REFVALI= ZREFVALI_USELESS
    YCVGQ_NL%REFVALI= ZREFVALI_USELESS
    YQVA_NL%REFVALI= ZREFVALI_USELESS
    YSDSAT_NL%REFVALI= ZREFVALI_USELESS
    YCVV_NL%REFVALI= ZREFVALI_USELESS
    YUOM_NL%REFVALI= ZREFVALI_USELESS
    YUAL_NL%REFVALI= ZREFVALI_USELESS
    YDOM_NL%REFVALI= ZREFVALI_USELESS
    YDAL_NL%REFVALI= ZREFVALI_USELESS
    YUEN_NL%REFVALI= ZREFVALI_USELESS
    YUNEBH_NL%REFVALI= ZREFVALI_USELESS
  ELSEIF ((.NOT.LECMWF).AND. LMPA) THEN
    YQ_NL%REFVALI = ZREFVALI_USELESS
    YL_NL%REFVALI = ZREFVALI_USELESS
    YI_NL%REFVALI = ZREFVALI_USELESS
    YS_NL%REFVALI = ZREFVALI_USELESS
    YR_NL%REFVALI = ZREFVALI_USELESS
    YG_NL%REFVALI = ZREFVALI_USELESS
    YH_NL%REFVALI = ZREFVALI_USELESS
    YTKE_NL%REFVALI = 0.000001_JPRB
    YA_NL%REFVALI = 0.0_JPRB
    YSRC_NL%REFVALI = 0.0_JPRB
    YO3_NL%REFVALI= ZREFVALI_USELESS
    YCPF_NL%REFVALI= ZREFVALI_USELESS
    YSPF_NL%REFVALI= ZREFVALI_USELESS
    YCVGQ_NL%REFVALI= ZREFVALI_USELESS
    YQVA_NL%REFVALI= ZREFVALI_USELESS
    YSDSAT_NL%REFVALI= ZREFVALI_USELESS
    YCVV_NL%REFVALI= ZREFVALI_USELESS
    YUOM_NL%REFVALI= ZREFVALI_USELESS
    YUAL_NL%REFVALI= ZREFVALI_USELESS
    YDOM_NL%REFVALI= ZREFVALI_USELESS
    YDAL_NL%REFVALI= ZREFVALI_USELESS
    YUEN_NL%REFVALI= ZREFVALI_USELESS
    YUNEBH_NL%REFVALI= ZREFVALI_USELESS
  ELSE
    YQ_NL%REFVALI = ZREFVALI_USELESS
    YL_NL%REFVALI = ZREFVALI_USELESS
    YI_NL%REFVALI = ZREFVALI_USELESS
    YS_NL%REFVALI = ZREFVALI_USELESS
    YR_NL%REFVALI = ZREFVALI_USELESS
    YG_NL%REFVALI = ZREFVALI_USELESS
    YH_NL%REFVALI = ZREFVALI_USELESS
    YTKE_NL%REFVALI = ZREFVALI_USELESS
    YA_NL%REFVALI = ZREFVALI_USELESS
    YSRC_NL%REFVALI= ZREFVALI_USELESS
    YO3_NL%REFVALI= ZREFVALI_USELESS
    YCPF_NL%REFVALI= ZREFVALI_USELESS
    YSPF_NL%REFVALI= ZREFVALI_USELESS
    YCVGQ_NL%REFVALI= ZREFVALI_USELESS
    YQVA_NL%REFVALI= ZREFVALI_USELESS
    YSDSAT_NL%REFVALI= ZREFVALI_USELESS
    YCVV_NL%REFVALI= ZREFVALI_USELESS
    YUOM_NL%REFVALI= ZREFVALI_USELESS
    YUAL_NL%REFVALI= ZREFVALI_USELESS
    YDOM_NL%REFVALI= ZREFVALI_USELESS
    YDAL_NL%REFVALI= ZREFVALI_USELESS
    YUEN_NL%REFVALI= ZREFVALI_USELESS
    YUNEBH_NL%REFVALI= ZREFVALI_USELESS
  ENDIF
ELSE
  YQ_NL%REFVALI = ZREFVALI_USELESS
  YL_NL%REFVALI = ZREFVALI_USELESS
  YI_NL%REFVALI = ZREFVALI_USELESS
  YS_NL%REFVALI = ZREFVALI_USELESS
  YR_NL%REFVALI = ZREFVALI_USELESS
  YG_NL%REFVALI = ZREFVALI_USELESS
  YH_NL%REFVALI = ZREFVALI_USELESS
  YTKE_NL%REFVALI = ZREFVALI_USELESS
  YA_NL%REFVALI = ZREFVALI_USELESS
  YSRC_NL%REFVALI= ZREFVALI_USELESS
  YO3_NL%REFVALI= ZREFVALI_USELESS
  YCPF_NL%REFVALI= ZREFVALI_USELESS
  YSPF_NL%REFVALI= ZREFVALI_USELESS
  YCVGQ_NL%REFVALI= ZREFVALI_USELESS
  YQVA_NL%REFVALI= ZREFVALI_USELESS
  YSDSAT_NL%REFVALI= ZREFVALI_USELESS
  YCVV_NL%REFVALI= ZREFVALI_USELESS
  YUOM_NL%REFVALI= ZREFVALI_USELESS
  YUAL_NL%REFVALI= ZREFVALI_USELESS
  YDOM_NL%REFVALI= ZREFVALI_USELESS
  YDAL_NL%REFVALI= ZREFVALI_USELESS
  YUEN_NL%REFVALI= ZREFVALI_USELESS
  YUNEBH_NL%REFVALI= ZREFVALI_USELESS
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%REFVALI = ZREFVALI_USELESS
  YICNC_NL%REFVALI = ZREFVALI_USELESS
  YRE_LIQ_NL%REFVALI = 4.0_JPRB
  YRE_ICE_NL%REFVALI = 20.0_JPRB
END IF
!<<dod

! * Default for attribute LREQOUT
!   ky: I don't know exactly what to do for conf other than 1,601,801.
!       Currently I assume that we write on files the same GFL as
!       for conf 1, even if there are not read on initial files.
IF (LECMWF) THEN
  ! ky: this must be checked by ECMWF
  YQ_NL%LREQOUT = .TRUE.
  YL_NL%LREQOUT = .TRUE.
  YI_NL%LREQOUT = .TRUE.
  YS_NL%LREQOUT = .TRUE.
  YR_NL%LREQOUT = .TRUE.
  YG_NL%LREQOUT = .FALSE.
  YH_NL%LREQOUT = .FALSE.
  YTKE_NL%LREQOUT = .FALSE.
  YA_NL%LREQOUT = .TRUE.
  YSRC_NL%LREQOUT= .FALSE.
  YO3_NL%LREQOUT= .TRUE.
  YCPF_NL%LREQOUT= .FALSE.
  YSPF_NL%LREQOUT= .FALSE.
  YCVGQ_NL%LREQOUT= .FALSE.
  YQVA_NL%LREQOUT= .FALSE.
  YSDSAT_NL%LREQOUT= .FALSE.
  YCVV_NL%LREQOUT= .FALSE.
  YUOM_NL%LREQOUT= .FALSE.
  YUAL_NL%LREQOUT= .FALSE.
  YDOM_NL%LREQOUT= .FALSE.
  YDAL_NL%LREQOUT= .FALSE.
  YUEN_NL%LREQOUT= .FALSE.
  YUNEBH_NL%LREQOUT= .FALSE.
ELSEIF ((.NOT.LECMWF).AND. LMPA) THEN
  YQ_NL%LREQOUT = .TRUE.
  YL_NL%LREQOUT = .TRUE.
  YI_NL%LREQOUT = .TRUE.
  YS_NL%LREQOUT = .TRUE.
  YR_NL%LREQOUT = .TRUE.
  YG_NL%LREQOUT = .TRUE.
  YH_NL%LREQOUT = .TRUE.
  YTKE_NL%LREQOUT = .TRUE.
  YA_NL%LREQOUT = .TRUE.
  YSRC_NL%LREQOUT= .FALSE.
  YO3_NL%LREQOUT= .FALSE.
  YCPF_NL%LREQOUT= .FALSE.
  YSPF_NL%LREQOUT= .FALSE.
  YCVGQ_NL%LREQOUT= .FALSE.
  YQVA_NL%LREQOUT= .FALSE.
  YSDSAT_NL%LREQOUT= .FALSE.
  YCVV_NL%LREQOUT= .FALSE.
  YUOM_NL%LREQOUT= .FALSE.
  YUAL_NL%LREQOUT= .FALSE.
  YDOM_NL%LREQOUT= .FALSE.
  YDAL_NL%LREQOUT= .FALSE.
  YUEN_NL%LREQOUT= .FALSE.
  YUNEBH_NL%LREQOUT= .FALSE.
ELSE
  YQ_NL%LREQOUT = .TRUE.
  YL_NL%LREQOUT = .FALSE.
  YI_NL%LREQOUT = .FALSE.
  YS_NL%LREQOUT = .FALSE.
  YR_NL%LREQOUT = .FALSE.
  YG_NL%LREQOUT = .FALSE.
  YH_NL%LREQOUT = .FALSE.
  YTKE_NL%LREQOUT = .FALSE.
  YA_NL%LREQOUT = .FALSE.
  YSRC_NL%LREQOUT= .FALSE.
  YO3_NL%LREQOUT= .FALSE.
  YCPF_NL%LREQOUT= .FALSE.
  YSPF_NL%LREQOUT= .FALSE.
  YCVGQ_NL%LREQOUT= .FALSE.
  YQVA_NL%LREQOUT= .FALSE.
  YSDSAT_NL%LREQOUT= .FALSE.
  YCVV_NL%LREQOUT= .FALSE.
  YUOM_NL%LREQOUT= .FALSE.
  YUAL_NL%LREQOUT= .FALSE.
  YDOM_NL%LREQOUT= .FALSE.
  YDAL_NL%LREQOUT= .FALSE.
  YUEN_NL%LREQOUT= .FALSE.
  YUNEBH_NL%LREQOUT= .FALSE.
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LREQOUT = .FALSE.
  YICNC_NL%LREQOUT = .FALSE.
  YRE_LIQ_NL%LREQOUT = .FALSE.
  YRE_ICE_NL%LREQOUT = .FALSE.
END IF
!<<dod

! * Default for attribute LGPINGP
IF (LECMWF) THEN
  LLEC_GPINGP=ANY(SPREAD(NCONF,1,4) == (/1,302,601,801/)) .OR. &
   & ((NCONF == 131) .AND. .NOT.LTRAJHR)
  YQ_NL%LGPINGP = LLEC_GPINGP
  YL_NL%LGPINGP = LLEC_GPINGP
  YI_NL%LGPINGP = LLEC_GPINGP
  YS_NL%LGPINGP = LLEC_GPINGP
  YR_NL%LGPINGP = LLEC_GPINGP
  YG_NL%LGPINGP = .FALSE.
  YH_NL%LGPINGP = .FALSE.
  YTKE_NL%LGPINGP = .FALSE.
  YA_NL%LGPINGP = LLEC_GPINGP
  YSRC_NL%LGPINGP= .FALSE.
  YO3_NL%LGPINGP = LLEC_GPINGP
  YCPF_NL%LGPINGP= .FALSE.
  YSPF_NL%LGPINGP= .FALSE.
  YCVGQ_NL%LGPINGP= .FALSE.
  YQVA_NL%LGPINGP= .FALSE.
  YSDSAT_NL%LGPINGP= .FALSE.
  YCVV_NL%LGPINGP= .FALSE.
  YUOM_NL%LGPINGP= .TRUE.
  YUAL_NL%LGPINGP= .TRUE.
  YDOM_NL%LGPINGP= .TRUE.
  YDAL_NL%LGPINGP= .TRUE.
  YUEN_NL%LGPINGP= .TRUE.
  YUNEBH_NL%LGPINGP= .TRUE.
ELSE
  LLEC_GPINGP=.FALSE.
  IF (ANY(SPREAD(NCONF,1,4) == (/1,302,601,801/))) THEN
    YQ_NL%LGPINGP = .TRUE.
    YL_NL%LGPINGP = .FALSE.
    YI_NL%LGPINGP = .FALSE.
    YS_NL%LGPINGP = .FALSE.
    YR_NL%LGPINGP = .FALSE.
    YG_NL%LGPINGP = .FALSE.
    YH_NL%LGPINGP = .FALSE.
    YTKE_NL%LGPINGP = .FALSE.
    YA_NL%LGPINGP = .FALSE.
    YSRC_NL%LGPINGP= .FALSE.
    YO3_NL%LGPINGP = .TRUE.
    YCPF_NL%LGPINGP= .FALSE.
    YSPF_NL%LGPINGP= .FALSE.
    YCVGQ_NL%LGPINGP= .FALSE.
    YQVA_NL%LGPINGP= .FALSE.
    YSDSAT_NL%LGPINGP= .FALSE.
    YCVV_NL%LGPINGP= .FALSE.
    YUOM_NL%LGPINGP= .TRUE.
    YUAL_NL%LGPINGP= .TRUE.
    YDOM_NL%LGPINGP= .TRUE.
    YDAL_NL%LGPINGP= .TRUE.
    YUEN_NL%LGPINGP= .TRUE.
    YUNEBH_NL%LGPINGP= .TRUE.
  ELSE
    YQ_NL%LGPINGP = .FALSE.
    YL_NL%LGPINGP = .FALSE.
    YI_NL%LGPINGP = .FALSE.
    YS_NL%LGPINGP = .FALSE.
    YR_NL%LGPINGP = .FALSE.
    YG_NL%LGPINGP = .FALSE.
    YH_NL%LGPINGP = .FALSE.
    YTKE_NL%LGPINGP = .FALSE.
    YA_NL%LGPINGP = .FALSE.
    YSRC_NL%LGPINGP= .FALSE.
    YO3_NL%LGPINGP = .FALSE.
    YCPF_NL%LGPINGP= .FALSE.
    YSPF_NL%LGPINGP= .FALSE.
    YCVGQ_NL%LGPINGP= .FALSE.
    YQVA_NL%LGPINGP= .FALSE.
    YSDSAT_NL%LGPINGP= .FALSE.
    YCVV_NL%LGPINGP= .FALSE.
    YUOM_NL%LGPINGP= .TRUE.
    YUAL_NL%LGPINGP= .TRUE.
    YDOM_NL%LGPINGP= .TRUE.
    YDAL_NL%LGPINGP= .TRUE.
    YUEN_NL%LGPINGP= .TRUE.
    YUNEBH_NL%LGPINGP= .TRUE.
  ENDIF
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LGPINGP = .FALSE.
  YICNC_NL%LGPINGP = .FALSE.
  YRE_LIQ_NL%LGPINGP = .FALSE.
  YRE_ICE_NL%LGPINGP = .FALSE.
END IF
!<<dod

! * Default for attribute LTRAJIO
IF (LECMWF) THEN
  IF (LTRAJHR) THEN
    ! ky: this is probably LTRAJHR.AND.LTRAJHR_ALTI here rather than LTRAJHR
    YQ_NL%LTRAJIO = .TRUE.
    YL_NL%LTRAJIO = .TRUE.
    YI_NL%LTRAJIO = .TRUE.
    YS_NL%LTRAJIO = .TRUE.
    YR_NL%LTRAJIO = .TRUE.
    YG_NL%LTRAJIO = .FALSE.
    YH_NL%LTRAJIO = .FALSE.
    YTKE_NL%LTRAJIO = .FALSE.
    YA_NL%LTRAJIO = .TRUE.
    YSRC_NL%LTRAJIO = .FALSE.
    YO3_NL%LTRAJIO= .TRUE.
  ELSE
    YQ_NL%LTRAJIO = .FALSE.
    YL_NL%LTRAJIO = .FALSE.
    YI_NL%LTRAJIO = .FALSE.
    YS_NL%LTRAJIO = .FALSE.
    YR_NL%LTRAJIO = .FALSE.
    YG_NL%LTRAJIO = .FALSE.
    YH_NL%LTRAJIO = .FALSE.
    YTKE_NL%LTRAJIO = .FALSE.
    YA_NL%LTRAJIO = .FALSE.
    YSRC_NL%LTRAJIO = .FALSE.
    YO3_NL%LTRAJIO= .FALSE.
  ENDIF
  YCPF_NL%LTRAJIO= .FALSE.
  YSPF_NL%LTRAJIO= .FALSE.
  YCVGQ_NL%LTRAJIO= .FALSE.
  YQVA_NL%LTRAJIO= .FALSE.
  YSDSAT_NL%LTRAJIO= .FALSE.
  YCVV_NL%LTRAJIO= .FALSE.
  YUOM_NL%LTRAJIO= .FALSE.
  YUAL_NL%LTRAJIO= .FALSE.
  YDOM_NL%LTRAJIO= .FALSE.
  YDAL_NL%LTRAJIO= .FALSE.
  YUEN_NL%LTRAJIO= .FALSE.
  YUNEBH_NL%LTRAJIO= .FALSE.
ELSE
  YQ_NL%LTRAJIO = .FALSE.
  YL_NL%LTRAJIO = .FALSE.
  YI_NL%LTRAJIO = .FALSE.
  YS_NL%LTRAJIO = .FALSE.
  YR_NL%LTRAJIO = .FALSE.
  YG_NL%LTRAJIO = .FALSE.
  YH_NL%LTRAJIO = .FALSE.
  YTKE_NL%LTRAJIO = .FALSE.
  YA_NL%LTRAJIO = .FALSE.
  YSRC_NL%LTRAJIO = .FALSE.
  YO3_NL%LTRAJIO= .FALSE.
  YCPF_NL%LTRAJIO= .FALSE.
  YSPF_NL%LTRAJIO= .FALSE.
  YCVGQ_NL%LTRAJIO= .FALSE.
  YQVA_NL%LTRAJIO= .FALSE.
  YSDSAT_NL%LTRAJIO= .FALSE.
  YCVV_NL%LTRAJIO= .FALSE.
  YUOM_NL%LTRAJIO= .FALSE.
  YUAL_NL%LTRAJIO= .FALSE.
  YDOM_NL%LTRAJIO= .FALSE.
  YDAL_NL%LTRAJIO= .FALSE.
  YUEN_NL%LTRAJIO= .FALSE.
  YUNEBH_NL%LTRAJIO= .FALSE.
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LTRAJIO = .FALSE.
  YICNC_NL%LTRAJIO = .FALSE.
  YRE_LIQ_NL%LTRAJIO = .FALSE.
  YRE_ICE_NL%LTRAJIO = .FALSE.
END IF
!<<dod

! * Default for attribute LADV
IF (LECMWF) THEN
  IF (LSLAG) THEN
    YQ_NL%LADV = .TRUE.
    YL_NL%LADV = .TRUE.
    YI_NL%LADV = .TRUE.
    YS_NL%LADV = .TRUE.
    YR_NL%LADV = .TRUE.
    YG_NL%LADV = .FALSE.
    YH_NL%LADV = .FALSE.
    YTKE_NL%LADV = .FALSE.
    YA_NL%LADV = .TRUE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = .TRUE.
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ELSE
    YQ_NL%LADV = .TRUE.
    YL_NL%LADV = .FALSE.
    YI_NL%LADV = .FALSE.
    YS_NL%LADV = .FALSE.
    YR_NL%LADV = .FALSE.
    YG_NL%LADV = .FALSE.
    YH_NL%LADV = .FALSE.
    YTKE_NL%LADV = .FALSE.
    YA_NL%LADV = .FALSE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = .FALSE.
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ENDIF
  IF (LENCLD2 .AND. .NOT.LEPCLD) THEN
    YQ_NL%LADV = .TRUE.
    YL_NL%LADV = .FALSE.
    YI_NL%LADV = .FALSE.
    YS_NL%LADV = .FALSE.
    YR_NL%LADV = .FALSE.
    YG_NL%LADV = .FALSE.
    YH_NL%LADV = .FALSE.
    YTKE_NL%LADV = .FALSE.
    YA_NL%LADV = .FALSE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = LSLAG
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ENDIF
ELSEIF ((.NOT.LECMWF).AND. LMPA) THEN
  IF (LSLAG) THEN
    YQ_NL%LADV = .TRUE.
    YL_NL%LADV = .TRUE.
    YI_NL%LADV = .TRUE.
    YS_NL%LADV = .TRUE.
    YR_NL%LADV = .TRUE.
    YG_NL%LADV = .TRUE.
    YH_NL%LADV = .TRUE.
    YTKE_NL%LADV = .TRUE.
    YA_NL%LADV = .FALSE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = .FALSE.
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ELSE
    IF (LSFORC) THEN
      ! * no advection if LSFORC=T
      YQ_NL%LADV = .FALSE.
    ELSEIF((.NOT.LSFORC).AND.LSPRT.AND.YQ_NL%LGP) THEN
      ! * no advection
      YQ_NL%LADV = .FALSE.
    ELSE
      YQ_NL%LADV = .TRUE.
    ENDIF
    YL_NL%LADV = .FALSE.
    YI_NL%LADV = .FALSE.
    YS_NL%LADV = .FALSE.
    YR_NL%LADV = .FALSE.
    YG_NL%LADV = .FALSE.
    YH_NL%LADV = .FALSE.
    YTKE_NL%LADV = .FALSE.
    YA_NL%LADV = .FALSE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = .FALSE.
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ENDIF
ELSE
  IF (LSLAG) THEN
    ! * for the time being attribute is let to .F. for ql,qi,qr,qs
    !   but the diabatic forecast in the MF operational suite now requires .T.
    YQ_NL%LADV = .TRUE.
    YL_NL%LADV = .FALSE.
    YI_NL%LADV = .FALSE.
    YS_NL%LADV = .FALSE.
    YR_NL%LADV = .FALSE.
    YG_NL%LADV = .FALSE.
    YH_NL%LADV = .FALSE.
    YTKE_NL%LADV = .FALSE.
    YA_NL%LADV = .FALSE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = .FALSE.
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ELSE
    IF (LSFORC) THEN
      ! * no advection if LSFORC=T
      YQ_NL%LADV = .FALSE.
    ELSEIF((.NOT.LSFORC).AND.LSPRT.AND.YQ_NL%LGP) THEN
      ! * no advection
      YQ_NL%LADV = .FALSE.
    ELSE
      YQ_NL%LADV = .TRUE.
    ENDIF
    YL_NL%LADV = .FALSE.
    YI_NL%LADV = .FALSE.
    YS_NL%LADV = .FALSE.
    YR_NL%LADV = .FALSE.
    YG_NL%LADV = .FALSE.
    YH_NL%LADV = .FALSE.
    YTKE_NL%LADV = .FALSE.
    YA_NL%LADV = .FALSE.
    YSRC_NL%LADV= .FALSE.
    YO3_NL%LADV = .FALSE.
    YCPF_NL%LADV= .FALSE.
    YSPF_NL%LADV= .FALSE.
    YCVGQ_NL%LADV= .FALSE.
    YQVA_NL%LADV= .FALSE.
    YSDSAT_NL%LADV= .FALSE.
    YCVV_NL%LADV= .FALSE.
    YUOM_NL%LADV= .FALSE.
    YUAL_NL%LADV= .FALSE.
    YDOM_NL%LADV= .FALSE.
    YDAL_NL%LADV= .FALSE.
    YUEN_NL%LADV= .FALSE.
    YUNEBH_NL%LADV= .FALSE.
  ENDIF
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LADV = .FALSE.
  YICNC_NL%LADV = .FALSE.
  YRE_LIQ_NL%LADV = .FALSE.
  YRE_ICE_NL%LADV = .FALSE.
END IF
!<<dod

! * Default attributes for LADV5
!   True means TL/AD SL advection of GFL fields uses 
!   trajectory wind only, making 4D-VAR univariate
!   apart from the observation operators

IF (LECMWF) THEN
  IF (LSLAG) THEN
    YGFLC(:)%LADV5 = .FALSE.
  ELSE
    YGFLC(:)%LADV5 = .FALSE.
  ENDIF
ELSE
  IF (LSLAG) THEN
    YGFLC(:)%LADV5 = .FALSE.
  ELSE
    YGFLC(:)%LADV5 = .FALSE.
  ENDIF
ENDIF

! * Default for attribute LCDERS,LT9,LT1,LT5
!   This value is automatically computed in SUGFL.

! * Default for attribute LPT
YQ_NL%LPT = .TRUE.
YL_NL%LPT = .TRUE.
YI_NL%LPT = .TRUE.
YS_NL%LPT = .TRUE.
YR_NL%LPT = .TRUE.
YG_NL%LPT = .TRUE.
YH_NL%LPT = .TRUE.
YTKE_NL%LPT = .TRUE.
YA_NL%LPT = .TRUE.
YSRC_NL%LPT = .FALSE.
YO3_NL%LPT= .TRUE.
YCPF_NL%LPT= .FALSE.
YSPF_NL%LPT= .FALSE.
YCVGQ_NL%LPT= .FALSE.
YQVA_NL%LPT= .FALSE.
YSDSAT_NL%LPT= .FALSE.
YCVV_NL%LPT= .FALSE.
YUOM_NL%LPT= .FALSE.
YUAL_NL%LPT= .FALSE.
YDOM_NL%LPT= .FALSE.
YDAL_NL%LPT= .FALSE.
YUEN_NL%LPT= .FALSE.
YUNEBH_NL%LPT= .FALSE.
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LPT = .FALSE.
  YICNC_NL%LPT = .FALSE.
  YRE_LIQ_NL%LPT = .FALSE.
  YRE_ICE_NL%LPT = .FALSE.
END IF
!<<dod

! * Default for attribute LPC
YQ_NL%LPC = .TRUE.
YL_NL%LPC = .TRUE.
YI_NL%LPC = .TRUE.
YS_NL%LPC = .TRUE.
YR_NL%LPC = .TRUE.
YG_NL%LPC = .TRUE.
YH_NL%LPC = .TRUE.
YTKE_NL%LPC = .TRUE.
YA_NL%LPC = .TRUE.
YSRC_NL%LPC = .TRUE.
YO3_NL%LPC= .TRUE.
YCPF_NL%LPC= .TRUE.
YSPF_NL%LPC= .TRUE.
YCVGQ_NL%LPC= .TRUE.
YSDSAT_NL%LPC= .FALSE.
YCVV_NL%LPC= .FALSE.
YQVA_NL%LPC= .FALSE.
YUOM_NL%LPC= .FALSE.
YUAL_NL%LPC= .FALSE.
YDOM_NL%LPC= .FALSE.
YDAL_NL%LPC= .FALSE.
YUEN_NL%LPC= .FALSE.
YUNEBH_NL%LPC= .FALSE.
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LPC = .FALSE.
  YICNC_NL%LPC = .FALSE.
  YRE_LIQ_NL%LPC = .FALSE.
  YRE_ICE_NL%LPC = .FALSE.
END IF
!<<dod
! * Default for attribute CNAME
!   This value is automatically computed in SUGFL.

! * Default for attribute IGRBCODE
!   This value is automatically computed in SUGFL.
!   A provisional value has been given for prognostic convection variables
!   used in the ALARO physics (unattributed GRIB codes in the ECMWF table).
YUOM_NL%IGRBCODE=81
YUAL_NL%IGRBCODE=82
YDOM_NL%IGRBCODE=84
YDAL_NL%IGRBCODE=85
YUEN_NL%IGRBCODE=86
YUNEBH_NL%IGRBCODE=79

! * Default for attribute NCOUPLING
IF ((.NOT.LECMWF).AND. LMPA) THEN
  IF(LELAM) THEN
    YQ_NL%NCOUPLING = 1
  ELSE
    YQ_NL%NCOUPLING = 0
  ENDIF
  YL_NL%NCOUPLING = 1
  YI_NL%NCOUPLING = 1
  YS_NL%NCOUPLING = 1
  YR_NL%NCOUPLING = 1
  YG_NL%NCOUPLING = 1
  YH_NL%NCOUPLING = 1
  YTKE_NL%NCOUPLING = -1
  YA_NL%NCOUPLING = 0
  YSRC_NL%NCOUPLING = 0
  YO3_NL%NCOUPLING = 0
  YCPF_NL%NCOUPLING = 0
  YSPF_NL%NCOUPLING = 0
  YCVGQ_NL%NCOUPLING = 0
  YQVA_NL%NCOUPLING = 0
  YSDSAT_NL%NCOUPLING= 0
  YCVV_NL%NCOUPLING= 0
  YUOM_NL%NCOUPLING= 0
  YUAL_NL%NCOUPLING= 0
  YDOM_NL%NCOUPLING= 0
  YDAL_NL%NCOUPLING= 0
  YUEN_NL%NCOUPLING= 0
  YUNEBH_NL%NCOUPLING= 0
ELSE
  IF(LELAM) THEN
    YQ_NL%NCOUPLING = 1
  ELSE
    YQ_NL%NCOUPLING = 0
  ENDIF
  YL_NL%NCOUPLING = 0
  YI_NL%NCOUPLING = 0
  YS_NL%NCOUPLING = 0
  YR_NL%NCOUPLING = 0
  YG_NL%NCOUPLING = 0
  YH_NL%NCOUPLING = 0
  YTKE_NL%NCOUPLING = 0
  YA_NL%NCOUPLING = 0
  YSRC_NL%NCOUPLING = 0
  YO3_NL%NCOUPLING = 0
  YCPF_NL%NCOUPLING = 0
  YSPF_NL%NCOUPLING = 0
  YCVGQ_NL%NCOUPLING = 0
  YQVA_NL%NCOUPLING = 0
  YSDSAT_NL%NCOUPLING= 0
  YCVV_NL%NCOUPLING= 0
  YUOM_NL%NCOUPLING= 0
  YUAL_NL%NCOUPLING= 0
  YDOM_NL%NCOUPLING= 0
  YDAL_NL%NCOUPLING= 0
  YUEN_NL%NCOUPLING= 0
  YUNEBH_NL%NCOUPLING= 0
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%NCOUPLING = 0
  YICNC_NL%NCOUPLING = 0
  YRE_LIQ_NL%NCOUPLING = 0
  YRE_ICE_NL%NCOUPLING = 0
END IF
!<<dod

! * Default for attribute REFVALC
!   REFVALC is useful only when NCOUPLING=-1,
!    but to avoid unitialised values, REFVALC is set to -999 in the other cases.
ZREFVALC_USELESS=-999._JPRB
IF ((.NOT.LECMWF).AND. LMPA) THEN
  YQ_NL%REFVALC = ZREFVALC_USELESS
  YL_NL%REFVALC = ZREFVALC_USELESS
  YI_NL%REFVALC = ZREFVALC_USELESS
  YS_NL%REFVALC = ZREFVALC_USELESS
  YR_NL%REFVALC = ZREFVALC_USELESS
  YG_NL%REFVALC = ZREFVALC_USELESS
  YH_NL%REFVALC = ZREFVALC_USELESS
  YTKE_NL%REFVALC = 0.000001_JPRB
  YA_NL%REFVALC = ZREFVALC_USELESS
  YSRC_NL%REFVALC = ZREFVALC_USELESS
  YO3_NL%REFVALC = ZREFVALC_USELESS
  YCPF_NL%REFVALC = ZREFVALC_USELESS
  YSPF_NL%REFVALC = ZREFVALC_USELESS
  YCVGQ_NL%REFVALC = ZREFVALC_USELESS
  YQVA_NL%REFVALC = ZREFVALC_USELESS
  YSDSAT_NL%REFVALC= ZREFVALC_USELESS
  YCVV_NL%REFVALC= ZREFVALC_USELESS
  YUOM_NL%REFVALC= ZREFVALC_USELESS
  YUAL_NL%REFVALC= ZREFVALC_USELESS
  YDOM_NL%REFVALC= ZREFVALC_USELESS
  YDAL_NL%REFVALC= ZREFVALC_USELESS
  YUEN_NL%REFVALC= ZREFVALC_USELESS
  YUNEBH_NL%REFVALC= ZREFVALC_USELESS
ELSE
  YQ_NL%REFVALC = ZREFVALC_USELESS
  YL_NL%REFVALC = ZREFVALC_USELESS
  YI_NL%REFVALC = ZREFVALC_USELESS
  YS_NL%REFVALC = ZREFVALC_USELESS
  YR_NL%REFVALC = ZREFVALC_USELESS
  YG_NL%REFVALC = ZREFVALC_USELESS
  YH_NL%REFVALC = ZREFVALC_USELESS
  YTKE_NL%REFVALC = ZREFVALC_USELESS
  YA_NL%REFVALC = ZREFVALC_USELESS
  YSRC_NL%REFVALC = ZREFVALC_USELESS
  YO3_NL%REFVALC = ZREFVALC_USELESS
  YCPF_NL%REFVALC = ZREFVALC_USELESS
  YSPF_NL%REFVALC = ZREFVALC_USELESS
  YCVGQ_NL%REFVALC = ZREFVALC_USELESS
  YQVA_NL%REFVALC = ZREFVALC_USELESS
  YSDSAT_NL%REFVALC= ZREFVALC_USELESS
  YCVV_NL%REFVALC= ZREFVALC_USELESS
  YUOM_NL%REFVALC= ZREFVALC_USELESS
  YUAL_NL%REFVALC= ZREFVALC_USELESS
  YDOM_NL%REFVALC= ZREFVALC_USELESS
  YDAL_NL%REFVALC= ZREFVALC_USELESS
  YUEN_NL%REFVALC= ZREFVALC_USELESS
  YUNEBH_NL%REFVALC= ZREFVALC_USELESS
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%REFVALC= ZREFVALC_USELESS
  YICNC_NL%REFVALC= ZREFVALC_USELESS
  YRE_LIQ_NL%REFVALC= ZREFVALC_USELESS
  YRE_ICE_NL%REFVALC= ZREFVALC_USELESS
END IF
!<<dod

! * Default for attribute LSLHD
YQ_NL%LSLHD=.FALSE.
YL_NL%LSLHD=.FALSE.
YI_NL%LSLHD=.FALSE.
YS_NL%LSLHD=.FALSE.
YR_NL%LSLHD=.FALSE.
YG_NL%LSLHD=.FALSE.
YH_NL%LSLHD=.FALSE.
YTKE_NL%LSLHD=.FALSE.
YA_NL%LSLHD=.FALSE.
YSRC_NL%LSLHD=.FALSE.
YO3_NL%LSLHD=.FALSE.
YCPF_NL%LSLHD=.FALSE.
YSPF_NL%LSLHD=.FALSE.
YCVGQ_NL%LSLHD=.FALSE.
YQVA_NL%LSLHD=.FALSE.
YSDSAT_NL%LSLHD=.FALSE.
YCVV_NL%LSLHD=.FALSE.
YUOM_NL%LSLHD=.FALSE.
YUAL_NL%LSLHD=.FALSE.
YDOM_NL%LSLHD=.FALSE.
YDAL_NL%LSLHD=.FALSE.
YUEN_NL%LSLHD=.FALSE.
YUNEBH_NL%LSLHD=.FALSE.
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LSLHD = .FALSE.
  YICNC_NL%LSLHD = .FALSE.
  YRE_LIQ_NL%LSLHD = .FALSE.
  YRE_ICE_NL%LSLHD = .FALSE.
END IF
!<<dod

! * Default for attribute LVSPLIP
YQ_NL%LVSPLIP=.FALSE.
YL_NL%LVSPLIP=.FALSE.
YI_NL%LVSPLIP=.FALSE.
YS_NL%LVSPLIP=.FALSE.
YR_NL%LVSPLIP=.FALSE.
YG_NL%LVSPLIP=.FALSE.
YH_NL%LVSPLIP=.FALSE.
YTKE_NL%LVSPLIP=.FALSE.
YA_NL%LVSPLIP=.FALSE.
YSRC_NL%LVSPLIP=.FALSE.
YO3_NL%LVSPLIP=.FALSE.
YCPF_NL%LVSPLIP=.FALSE.
YSPF_NL%LVSPLIP=.FALSE.
YCVGQ_NL%LVSPLIP=.FALSE.
YQVA_NL%LVSPLIP=.FALSE.
YSDSAT_NL%LVSPLIP=.FALSE.
YCVV_NL%LVSPLIP=.FALSE.
YUOM_NL%LVSPLIP=.FALSE.
YUAL_NL%LVSPLIP=.FALSE.
YDOM_NL%LVSPLIP=.FALSE.
YDAL_NL%LVSPLIP=.FALSE.
YUEN_NL%LVSPLIP=.FALSE.
YUNEBH_NL%LVSPLIP=.FALSE.
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LVSPLIP = .FALSE.
  YICNC_NL%LVSPLIP = .FALSE.
  YRE_LIQ_NL%LVSPLIP = .FALSE.
  YRE_ICE_NL%LVSPLIP = .FALSE.
END IF
!<<dod

! * Default for attribute LQM
IF (LECMWF) THEN
  ! ky: this must be checked by ECMWF
  YQ_NL%LQM=LSLAG.AND.(NCONF == 1.OR.NCONF == 302)
  YL_NL%LQM=LSLAG
  YI_NL%LQM=LSLAG
  YS_NL%LQM=LSLAG
  YR_NL%LQM=LSLAG
  YG_NL%LQM=.FALSE.
  YH_NL%LQM=.FALSE.
  YTKE_NL%LQM=.FALSE.
  YA_NL%LQM=LSLAG
  YSRC_NL%LQM=.FALSE.
  YO3_NL%LQM=LSLAG
  YCPF_NL%LQM=.FALSE.
  YSPF_NL%LQM=.FALSE.
  YCVGQ_NL%LQM=.FALSE.
  YQVA_NL%LQM=.FALSE.
  YSDSAT_NL%LQM=.FALSE.
  YCVV_NL%LQM=.FALSE.
  YUOM_NL%LQM=.FALSE.
  YUAL_NL%LQM=.FALSE.
  YDOM_NL%LQM=.FALSE.
  YDAL_NL%LQM=.FALSE.
  YUEN_NL%LQM=.FALSE.
  YUNEBH_NL%LQM=.FALSE.
ELSEIF ((.NOT.LECMWF).AND.LMPA) THEN
  ! * for the time being attribute is let to .F. for active GFL other than q and TKE 
  !   but SL2TL tests done with this case probably require .T. for some of them.
  YQ_NL%LQM=LSLAG
  YL_NL%LQM=.FALSE.
  YI_NL%LQM=.FALSE.
  YS_NL%LQM=.FALSE.
  YR_NL%LQM=.FALSE.
  YG_NL%LQM=.FALSE.
  YH_NL%LQM=.FALSE.
  YTKE_NL%LQM=LSLAG
  YA_NL%LQM=.FALSE.
  YSRC_NL%LQM=.FALSE.
  YO3_NL%LQM=.FALSE.
  YCPF_NL%LQM=.FALSE.
  YSPF_NL%LQM=.FALSE.
  YCVGQ_NL%LQM=.FALSE.
  YQVA_NL%LQM=.FALSE.
  YSDSAT_NL%LQM=.FALSE.
  YCVV_NL%LQM=.FALSE.
  YUOM_NL%LQM=.FALSE.
  YUAL_NL%LQM=.FALSE.
  YDOM_NL%LQM=.FALSE.
  YDAL_NL%LQM=.FALSE.
  YUEN_NL%LQM=.FALSE.
  YUNEBH_NL%LQM=.FALSE.
ELSE
  ! * for the time being attribute is let to .F. for ql,qi,qr,qs
  !   but the diabatic SL2TL forecast in the MF operational suite now requires .T.
  YQ_NL%LQM=LSLAG.AND.(NCONF == 1 .OR. NCONF == 131 .OR. NCONF == 302)
  YL_NL%LQM=.FALSE.
  YI_NL%LQM=.FALSE.
  YS_NL%LQM=.FALSE.
  YR_NL%LQM=.FALSE.
  YG_NL%LQM=.FALSE.
  YH_NL%LQM=.FALSE.
  YTKE_NL%LQM=.FALSE.
  YA_NL%LQM=.FALSE.
  YSRC_NL%LQM=.FALSE.
  YO3_NL%LQM=.FALSE.
  YCPF_NL%LQM=.FALSE.
  YSPF_NL%LQM=.FALSE.
  YCVGQ_NL%LQM=.FALSE.
  YQVA_NL%LQM=.FALSE.
  YSDSAT_NL%LQM=.FALSE.
  YCVV_NL%LQM=.FALSE.
  YUOM_NL%LQM=.FALSE.
  YUAL_NL%LQM=.FALSE.
  YDOM_NL%LQM=.FALSE.
  YDAL_NL%LQM=.FALSE.
  YUEN_NL%LQM=.FALSE.
  YUNEBH_NL%LQM=.FALSE.
ENDIF
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LQM = .FALSE.
  YICNC_NL%LQM = .FALSE.
  YRE_LIQ_NL%LQM = .FALSE.
  YRE_ICE_NL%LQM = .FALSE.
END IF
!<<dod
! * Default for attribute LQMH
YQ_NL%LQMH=.FALSE.
YL_NL%LQMH=.FALSE.
YI_NL%LQMH=.FALSE.
YS_NL%LQMH=.FALSE.
YR_NL%LQMH=.FALSE.
YG_NL%LQMH=.FALSE.
YH_NL%LQMH=.FALSE.
YTKE_NL%LQMH=.FALSE.
YA_NL%LQMH=.FALSE.
YSRC_NL%LQMH=.FALSE.
YO3_NL%LQMH=.FALSE.
YCPF_NL%LQMH=.FALSE.
YSPF_NL%LQMH=.FALSE.
YCVGQ_NL%LQMH=.FALSE.
YQVA_NL%LQMH=.FALSE.
YSDSAT_NL%LQMH=.FALSE.
YCVV_NL%LQMH=.FALSE.
YUOM_NL%LQMH=.FALSE.
YUAL_NL%LQMH=.FALSE.
YDOM_NL%LQMH=.FALSE.
YDAL_NL%LQMH=.FALSE.
YUEN_NL%LQMH=.FALSE.
YUNEBH_NL%LQMH=.FALSE.
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LQMH = .FALSE.
  YICNC_NL%LQMH = .FALSE.
  YRE_LIQ_NL%LQMH = .FALSE.
  YRE_ICE_NL%LQMH = .FALSE.
END IF
!<<dod

! * Default for attribute LHV
YQ_NL%LHV=.FALSE.
YL_NL%LHV=.FALSE.
YI_NL%LHV=.FALSE.
YS_NL%LHV=.FALSE.
YR_NL%LHV=.FALSE.
YG_NL%LHV=.FALSE.
YH_NL%LHV=.FALSE.
YTKE_NL%LHV=.FALSE.
YA_NL%LHV=.FALSE.
YSRC_NL%LHV=.FALSE.
YO3_NL%LHV=.FALSE.
YCPF_NL%LHV=.FALSE.
YSPF_NL%LHV=.FALSE.
YCVGQ_NL%LHV=.FALSE.
YQVA_NL%LHV=.FALSE.
YSDSAT_NL%LHV=.FALSE.
YCVV_NL%LHV=.FALSE.
YUOM_NL%LHV=.FALSE.
YUAL_NL%LHV=.FALSE.
YDOM_NL%LHV=.FALSE.
YDAL_NL%LHV=.FALSE.
YUEN_NL%LHV=.FALSE.
YUNEBH_NL%LHV=.FALSE.
!>>dod
IF (LECEARTH .OR. LECEARTH_UNCOUP) THEN
  YCDNC_NL%LHV = .FALSE.
  YICNC_NL%LHV = .FALSE.
  YRE_LIQ_NL%LHV = .FALSE.
  YRE_ICE_NL%LHV = .FALSE.
END IF
!<<dod

!     ------------------------------------------------------------------

!        1b     Set implicit default values for GFL attributes
!               (GHG, GRG, AERO, CH4, TRAC, FORCING, EASY DIAG and EXT).

! * Defaults for greenhouse gases (GHG) attributes
DO JGFL=1,JPGHG
  IF (LECMWF) THEN
    YGHG_NL(JGFL)%LGP    = .TRUE.
  ELSE
    YGHG_NL(JGFL)%LGP    = .FALSE.
  ENDIF
  YGHG_NL(JGFL)%LSP    = .FALSE.
  YGHG_NL(JGFL)%NREQIN = 1
  YGHG_NL(JGFL)%LREQOUT= .TRUE.
  IF (LECMWF) THEN
    YGHG_NL(JGFL)%LGPINGP= LLEC_GPINGP
  ELSE
    YGHG_NL(JGFL)%LGPINGP= .TRUE.
  ENDIF
  IF (LECMWF .AND. LTRAJHR) THEN
    YGHG_NL(JGFL)%LTRAJIO= .TRUE.
  ELSE
    YGHG_NL(JGFL)%LTRAJIO= .FALSE.
  ENDIF
  IF (LECMWF.OR.(LENCLD2 .AND. .NOT.LEPCLD)) THEN
    YGHG_NL(JGFL)%LADV=LSLAG
  ELSE
    YGHG_NL(JGFL)%LADV=.FALSE. 
  ENDIF
  YGHG_NL(JGFL)%LCDERS = .FALSE.
  YGHG_NL(JGFL)%LT9 =.FALSE.
  YGHG_NL(JGFL)%LT1 =.FALSE.
  YGHG_NL(JGFL)%LT5 =.FALSE.
  YGHG_NL(JGFL)%LPT =.FALSE.
  YGHG_NL(JGFL)%LPC =.FALSE.
  YGHG_NL(JGFL)%CNAME ='XXXXXXXXXXXXXXX'
  YGHG_NL(JGFL)%IGRBCODE = 999
  YGHG_NL(JGFL)%NCOUPLING= 0
  YGHG_NL(JGFL)%LSLHD    =.FALSE.
  YGHG_NL(JGFL)%LVSPLIP  =.FALSE.
  IF (LECMWF) THEN
    YGHG_NL(JGFL)%LQM =.TRUE.
  ELSE
    YGHG_NL(JGFL)%LQM =.FALSE.
  ENDIF
  YGHG_NL(JGFL)%LQMH     =.FALSE.
  YGHG_NL(JGFL)%LHV      =.FALSE.
ENDDO

! * Defaults for diagnostic tracers (TRAC, as SF6 or Radon) attributes
DO JGFL=1,JPTRAC
  IF (LECMWF) THEN
    YTRAC_NL(JGFL)%LGP    = .TRUE.
  ELSE
    YTRAC_NL(JGFL)%LGP    = .FALSE.
  ENDIF
  YTRAC_NL(JGFL)%LSP    = .FALSE.
  YTRAC_NL(JGFL)%NREQIN = 1
  YTRAC_NL(JGFL)%LREQOUT= .TRUE.
  IF (LECMWF) THEN
    YTRAC_NL(JGFL)%LGPINGP= LLEC_GPINGP
  ELSE
    YTRAC_NL(JGFL)%LGPINGP= .TRUE.
  ENDIF
  YTRAC_NL(JGFL)%LTRAJIO= .FALSE.
  IF (LECMWF.OR.(LENCLD2 .AND. .NOT.LEPCLD)) THEN
    YTRAC_NL(JGFL)%LADV=LSLAG
  ELSE
    YTRAC_NL(JGFL)%LADV=.FALSE.
  ENDIF
  YTRAC_NL(JGFL)%LCDERS = .FALSE.
  YTRAC_NL(JGFL)%LT9 =.FALSE.
  YTRAC_NL(JGFL)%LT1 =.FALSE.
  YTRAC_NL(JGFL)%LT5 =.FALSE.
  YTRAC_NL(JGFL)%LPT =.FALSE.
  YTRAC_NL(JGFL)%LPC =.FALSE.
  YTRAC_NL(JGFL)%CNAME ='XXXXXXXXXXXXXXX'
  YTRAC_NL(JGFL)%IGRBCODE = 999
  YTRAC_NL(JGFL)%NCOUPLING= 0
  YTRAC_NL(JGFL)%LSLHD    =.FALSE.
  YTRAC_NL(JGFL)%LVSPLIP  =.FALSE.
  IF (LECMWF) THEN
    YTRAC_NL(JGFL)%LQM =.TRUE.
  ELSE
    YTRAC_NL(JGFL)%LQM =.FALSE.
  ENDIF
  YTRAC_NL(JGFL)%LQMH     =.FALSE.
  YTRAC_NL(JGFL)%LHV      =.FALSE.
  YTRAC_NL(JGFL)%IGRIBTC   =  -9999
  YTRAC_NL(JGFL)%IGRIBSFC  =  -9999
  YTRAC_NL(JGFL)%LASSIM    = .FALSE.
  YTRAC_NL(JGFL)%RMOLMASS  = -99999.9 
  YTRAC_NL(JGFL)%NCOUPLO4  = -1 
  YTRAC_NL(JGFL)%LDIFF     = .TRUE.
  YTRAC_NL(JGFL)%LCONV     = .TRUE.
ENDDO

! * Defaults for reactive gases (GRG) attributes
DO JGFL=1,JPGRG
  IF (LECMWF) THEN
    YGRG_NL(JGFL)%LGP    = .TRUE.
  ELSE
    YGRG_NL(JGFL)%LGP    = .FALSE.
  ENDIF
  YGRG_NL(JGFL)%LSP    = .FALSE.
  YGRG_NL(JGFL)%NREQIN = 1
  YGRG_NL(JGFL)%LREQOUT= .TRUE.
  IF (LECMWF) THEN
    YGRG_NL(JGFL)%LGPINGP= LLEC_GPINGP
  ELSE
    YGRG_NL(JGFL)%LGPINGP= .TRUE.
  ENDIF
  IF (LECMWF .AND. LTRAJHR) THEN
    YGRG_NL(JGFL)%LTRAJIO= .TRUE.
  ELSE
    YGRG_NL(JGFL)%LTRAJIO= .FALSE.
  ENDIF
  IF (LECMWF.OR.(LENCLD2 .AND. .NOT.LEPCLD)) THEN
    YGRG_NL(JGFL)%LADV=LSLAG
  ELSE
    YGRG_NL(JGFL)%LADV=.FALSE.
  ENDIF
  YGRG_NL(JGFL)%LCDERS = .FALSE.
  YGRG_NL(JGFL)%LT9 =.FALSE.
  YGRG_NL(JGFL)%LT1 =.FALSE.
  YGRG_NL(JGFL)%LT5 =.FALSE.
  YGRG_NL(JGFL)%LPT =.FALSE.
  YGRG_NL(JGFL)%LPC =.FALSE.
  YGRG_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YGRG_NL(JGFL)%IGRBCODE = 999
  YGRG_NL(JGFL)%NCOUPLING= 0
  YGRG_NL(JGFL)%LSLHD    =.FALSE.
  YGRG_NL(JGFL)%LVSPLIP  =.FALSE.
  IF (LECMWF) THEN
    YGRG_NL(JGFL)%LQM =.TRUE.
  ELSE
    YGRG_NL(JGFL)%LQM =.FALSE.
  ENDIF
  YGRG_NL(JGFL)%LQMH     =.FALSE.
  YGRG_NL(JGFL)%LHV      =.FALSE.
  ! gems nl ext
  YGRG_NL(JGFL)%IGRIBTC   =  -9999
  YGRG_NL(JGFL)%IGRIBSFC  =  -9999
  YGRG_NL(JGFL)%LASSIM    = .FALSE.
  YGRG_NL(JGFL)%RMOLMASS  = -99999.9 
  YGRG_NL(JGFL)%NCOUPLO4  = -1 
  YGRG_NL(JGFL)%LDIFF     = .TRUE.
  YGRG_NL(JGFL)%LCONV     = .TRUE.
ENDDO

! * Defaults for GRG tendency fields 
DO JGFL=1,JPGRG
  IF (LECMWF) THEN
    YGRGTEND_NL(JGFL)%LGP    = .TRUE.
  ELSE
    YGRGTEND_NL(JGFL)%LGP    = .FALSE.
  ENDIF
  YGRGTEND_NL(JGFL)%LSP    = .FALSE.
  YGRGTEND_NL(JGFL)%NREQIN = 0
  YGRGTEND_NL(JGFL)%LREQOUT= .FALSE.
  IF (LECMWF) THEN
    YGRGTEND_NL(JGFL)%LGPINGP= LLEC_GPINGP
  ELSE
    YGRGTEND_NL(JGFL)%LGPINGP= .TRUE.
  ENDIF
  YGRGTEND_NL(JGFL)%LTRAJIO= .FALSE.
  YGRGTEND_NL(JGFL)%LADV   = .FALSE.
  YGRGTEND_NL(JGFL)%LCDERS = .FALSE.
  YGRGTEND_NL(JGFL)%LT9 = .FALSE.
  YGRGTEND_NL(JGFL)%LT1 = .FALSE.
  YGRGTEND_NL(JGFL)%LT5 = .FALSE.
  YGRGTEND_NL(JGFL)%LPT = .FALSE.
  YGRGTEND_NL(JGFL)%LPC = .FALSE.  
  YGRGTEND_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YGRGTEND_NL(JGFL)%IGRBCODE = 999
  YGRGTEND_NL(JGFL)%NCOUPLING= 0
  YGRGTEND_NL(JGFL)%LSLHD    =.FALSE. 
  YGRGTEND_NL(JGFL)%LVSPLIP  =.FALSE.  
  YGRGTEND_NL(JGFL)%LQM      =.FALSE.
  YGRGTEND_NL(JGFL)%LQMH     =.FALSE.
  YGRGTEND_NL(JGFL)%LHV      =.FALSE.
ENDDO

! * Defaults for aerosols attributes
DO JGFL=1,JPAERO
  IF (LECMWF) THEN
    YAERO_NL(JGFL)%LGP    = .TRUE.
  ELSE
    YAERO_NL(JGFL)%LGP    = .FALSE.
  ENDIF
  YAERO_NL(JGFL)%LSP    = .FALSE.
  YAERO_NL(JGFL)%NREQIN = 1
  YAERO_NL(JGFL)%LREQOUT= .TRUE.
  IF (LECMWF) THEN
    YAERO_NL(JGFL)%LGPINGP= LLEC_GPINGP
  ELSE
    YAERO_NL(JGFL)%LGPINGP= .TRUE.
  ENDIF
  IF (LECMWF .AND. LTRAJHR) THEN
    YAERO_NL(JGFL)%LTRAJIO= .TRUE.
  ELSE
    YAERO_NL(JGFL)%LTRAJIO= .FALSE.
  ENDIF
  IF (LECMWF.OR.(LENCLD2 .AND. .NOT.LEPCLD)) THEN
    YAERO_NL(JGFL)%LADV=LSLAG
  ELSE
    YAERO_NL(JGFL)%LADV=.FALSE.
  ENDIF
  YAERO_NL(JGFL)%LCDERS = .FALSE.
  YAERO_NL(JGFL)%LT9 = .FALSE.
  YAERO_NL(JGFL)%LT1 = .FALSE.
  YAERO_NL(JGFL)%LT5 = .FALSE.
  YAERO_NL(JGFL)%LPT = .FALSE.
  YAERO_NL(JGFL)%LPC = .FALSE.
  YAERO_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YAERO_NL(JGFL)%IGRBCODE = 999
  YAERO_NL(JGFL)%NCOUPLING= 0
  YAERO_NL(JGFL)%LSLHD    =.FALSE.
  YAERO_NL(JGFL)%LVSPLIP  =.FALSE.
  IF (LECMWF) THEN
    YAERO_NL(JGFL)%LQM =.TRUE.
  ELSE
    YAERO_NL(JGFL)%LQM =.FALSE.
  ENDIF
  YAERO_NL(JGFL)%LQMH     =.FALSE.
  YAERO_NL(JGFL)%LHV      =.FALSE.
ENDDO

! * Defaults for methane related GFL attributes: LRCH4
YLRCH4_NL%LGP  = .FALSE.
YLRCH4_NL%LSP  = .FALSE.
YLRCH4_NL%NREQIN = 1
YLRCH4_NL%LREQOUT= .TRUE.
IF (LECMWF) THEN
  YLRCH4_NL%LGPINGP=  LLEC_GPINGP
ELSE
  YLRCH4_NL%LGPINGP=.TRUE. 
ENDIF
IF (LECMWF .AND. LTRAJHR) THEN
  YLRCH4_NL%LTRAJIO= .TRUE.
ELSE
  YLRCH4_NL%LTRAJIO= .FALSE.
ENDIF
IF (LECMWF.OR.(LENCLD2 .AND. .NOT.LEPCLD)) THEN
  ! YLRCH4_NL%LADV=LSLAG
  YLRCH4_NL%LADV=.FALSE.
ELSE
  YLRCH4_NL%LADV=.FALSE.
ENDIF
YLRCH4_NL%LCDERS = .FALSE.
YLRCH4_NL%LT9 = .FALSE.
YLRCH4_NL%LT1 = .FALSE.
YLRCH4_NL%LT5 = .FALSE.
YLRCH4_NL%LPT = .FALSE.
YLRCH4_NL%LPC = .FALSE.
YLRCH4_NL%CNAME = 'XXXXXXXXXXXXXXX'
YLRCH4_NL%IGRBCODE = 999 
YLRCH4_NL%NCOUPLING= 0
YLRCH4_NL%LSLHD    =.FALSE.
YLRCH4_NL%LVSPLIP  =.FALSE.
IF (LECMWF) THEN
  YLRCH4_NL%LQM      =.TRUE.
ELSE
  YLRCH4_NL%LQM      =.FALSE.
ENDIF
YLRCH4_NL%LQMH     =.FALSE.
YLRCH4_NL%LHV      =.FALSE.

! * Defaults for methane related GFL attributes: CH4S
YCH4S_NL%LGP    = .FALSE.
YCH4S_NL%LSP    = .FALSE.
YCH4S_NL%NREQIN = 1
YCH4S_NL%LREQOUT= .TRUE.
IF (LECMWF) THEN
  YCH4S_NL%LGPINGP= LLEC_GPINGP
ELSE
  YCH4S_NL%LGPINGP=.TRUE. 
ENDIF
IF (LECMWF .AND. LTRAJHR) THEN
  YCH4S_NL%LTRAJIO= .TRUE.
ELSE
  YCH4S_NL%LTRAJIO= .FALSE.
ENDIF
IF (LECMWF.OR.(LENCLD2 .AND. .NOT.LEPCLD)) THEN
  YCH4S_NL%LADV=LSLAG
ELSE
  YCH4S_NL%LADV=.FALSE.
ENDIF
YCH4S_NL%LCDERS = .FALSE.
YCH4S_NL%LT9 = .FALSE.
YCH4S_NL%LT1 = .FALSE.
YCH4S_NL%LT5 = .FALSE.
YCH4S_NL%LPT = .FALSE.
YCH4S_NL%LPC = .FALSE.
YCH4S_NL%CNAME = 'XXXXXXXXXXXXXXX'
YCH4S_NL%IGRBCODE = 999
YCH4S_NL%NCOUPLING= 0
YCH4S_NL%LSLHD    =.FALSE.
YCH4S_NL%LVSPLIP  =.FALSE.
IF (LECMWF) THEN
  YCH4S_NL%LQM      =.TRUE.
ELSE
  YCH4S_NL%LQM      =.FALSE.
ENDIF
YCH4S_NL%LQMH     =.FALSE.
YCH4S_NL%LHV      =.FALSE.

! * Defaults for ERA40 GFL fields
DO JGFL=1,JPERA40
  YERA40_NL(JGFL)%LGP    = .TRUE.
  YERA40_NL(JGFL)%LSP    = .FALSE.
  YERA40_NL(JGFL)%NREQIN = 0
  YERA40_NL(JGFL)%LREQOUT= .TRUE.
  YERA40_NL(JGFL)%LGPINGP= .TRUE.
  YERA40_NL(JGFL)%LTRAJIO= .FALSE.
  YERA40_NL(JGFL)%LADV   = .FALSE.
  YERA40_NL(JGFL)%LCDERS = .FALSE.
  YERA40_NL(JGFL)%LT9 = .FALSE.
  YERA40_NL(JGFL)%LT1 = .FALSE.
  YERA40_NL(JGFL)%LT5 = .FALSE.
  YERA40_NL(JGFL)%LPT = .FALSE.
  YERA40_NL(JGFL)%LPC = .FALSE.
  YERA40_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YERA40_NL(JGFL)%IGRBCODE = 162100 + JGFL - 1
  YERA40_NL(JGFL)%NCOUPLING= 0
  YERA40_NL(JGFL)%LSLHD    =.FALSE.
  YERA40_NL(JGFL)%LVSPLIP  =.FALSE.
  YERA40_NL(JGFL)%LQM      =.FALSE.
  YERA40_NL(JGFL)%LQMH     =.FALSE.
  YERA40_NL(JGFL)%LHV      =.FALSE.
ENDDO

YERA40_NL(1)%CNAME='TSWR'
YERA40_NL(2)%CNAME='TLWR'
YERA40_NL(3)%CNAME='TCSSWR'
YERA40_NL(4)%CNAME='TCSLWR'
YERA40_NL(5)%CNAME='UDMF'
YERA40_NL(6)%CNAME='DDMF'
YERA40_NL(7)%CNAME='UDDR'
YERA40_NL(8)%CNAME='DDDR'
YERA40_NL(9)%CNAME='TPF'
YERA40_NL(10)%CNAME='TDCH'
YERA40_NL(11)%CNAME='TT'
YERA40_NL(12)%CNAME='TSH'
YERA40_NL(13)%CNAME='TU'
YERA40_NL(14)%CNAME='TV'

! * Defaults for noro gwd scheme fields
DO JGFL=1,JPNOGW
  JGFLN=228134+(JGFL-1)*2
  YNOGW_NL(JGFL)%LGP    = .TRUE.
  YNOGW_NL(JGFL)%LSP    = .FALSE.
  YNOGW_NL(JGFL)%NREQIN = 0
  YNOGW_NL(JGFL)%LREQOUT= .TRUE.
  YNOGW_NL(JGFL)%LGPINGP= .FALSE.
  YNOGW_NL(JGFL)%LTRAJIO= .FALSE.
  YNOGW_NL(JGFL)%LADV   = .FALSE.
  YNOGW_NL(JGFL)%LCDERS = .FALSE.
  YNOGW_NL(JGFL)%LT9 = .FALSE.
  YNOGW_NL(JGFL)%LT1 = .FALSE.
  YNOGW_NL(JGFL)%LT5 = .TRUE.
  YNOGW_NL(JGFL)%LPT = .FALSE.
  YNOGW_NL(JGFL)%LPC = .FALSE.
  YNOGW_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
!------- WATCH OUT: this is temporary waiting for a proper gribcode (PBe,JJM)
!  YNOGW_NL(JGFL)%IGRBCODE = 210077 + JGFL - 1
  YNOGW_NL(JGFL)%IGRBCODE = JGFLN
  YNOGW_NL(JGFL)%NCOUPLING= 0
  YNOGW_NL(JGFL)%LSLHD    =.FALSE.
  YNOGW_NL(JGFL)%LVSPLIP  =.FALSE.
  YNOGW_NL(JGFL)%LQM      =.FALSE.
  YNOGW_NL(JGFL)%LQMH     =.FALSE.
  YNOGW_NL(JGFL)%LHV      =.FALSE.
ENDDO

YNOGW_NL(1)%CNAME='TENUNOGW'
YNOGW_NL(2)%CNAME='TENVNOGW'

! * Defaults for UV processor output fields
DO JGFL=1,JPUVP
  YUVP_NL(JGFL)%LGP    = .TRUE.
  YUVP_NL(JGFL)%LSP    = .FALSE.
  YUVP_NL(JGFL)%NREQIN = 0
  YUVP_NL(JGFL)%LREQOUT= .TRUE.
  YUVP_NL(JGFL)%LGPINGP= .FALSE.
  YUVP_NL(JGFL)%LTRAJIO= .FALSE.
  YUVP_NL(JGFL)%LADV   = .FALSE.
  YUVP_NL(JGFL)%LCDERS = .FALSE.
  YUVP_NL(JGFL)%LPHY  = .FALSE.
  YUVP_NL(JGFL)%LT9 = .FALSE.
  YUVP_NL(JGFL)%LT1 = .FALSE.
  YUVP_NL(JGFL)%LT5 = .FALSE.
  YUVP_NL(JGFL)%LPT = .FALSE.
  YUVP_NL(JGFL)%LPC = .FALSE.
  YUVP_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
!------- WATCH OUT: this is temporary waiting for a proper gribcode (JJM)
  YUVP_NL(JGFL)%IGRBCODE = 210055 + JGFL - 1
  YUVP_NL(JGFL)%NCOUPLING= 0
  YUVP_NL(JGFL)%LSLHD    =.FALSE.
  YUVP_NL(JGFL)%LVSPLIP  =.FALSE.
  YUVP_NL(JGFL)%LQM      =.FALSE.
  YUVP_NL(JGFL)%LQMH     =.FALSE.
  YUVP_NL(JGFL)%LHV      =.FALSE.
ENDDO

YUVP_NL(1)%CNAME='UVP2DRAD'
YUVP_NL(2)%CNAME='TOAER340'



! * Defaults for aerosol output fields
DO JGFL=1,JPAEROUT
  YAEROUT_NL(JGFL)%LGP    = .TRUE.
  YAEROUT_NL(JGFL)%LSP    = .FALSE.
  YAEROUT_NL(JGFL)%NREQIN = 0
  YAEROUT_NL(JGFL)%LREQOUT= .TRUE.
  YAEROUT_NL(JGFL)%LGPINGP= .TRUE.
  YAEROUT_NL(JGFL)%LTRAJIO= .FALSE.
  YAEROUT_NL(JGFL)%LADV   = .FALSE.
  YAEROUT_NL(JGFL)%LCDERS = .FALSE.
  YAEROUT_NL(JGFL)%LPHY   = .FALSE.
  YAEROUT_NL(JGFL)%LT9 = .FALSE.
  YAEROUT_NL(JGFL)%LT1 = .FALSE.
  YAEROUT_NL(JGFL)%LT5 = .FALSE.
  YAEROUT_NL(JGFL)%LPT = .FALSE.
  YAEROUT_NL(JGFL)%LPC = .FALSE.
  YAEROUT_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
!------- WATCH OUT: this is temporary waiting for a proper gribcode (JJM)
  YAEROUT_NL(JGFL)%IGRBCODE = 210016 + JGFL - 1
  YAEROUT_NL(JGFL)%NCOUPLING= 0
  YAEROUT_NL(JGFL)%LSLHD    =.FALSE.
  YAEROUT_NL(JGFL)%LVSPLIP  =.FALSE.
  YAEROUT_NL(JGFL)%LQM      =.FALSE.
  YAEROUT_NL(JGFL)%LQMH     =.FALSE.
  YAEROUT_NL(JGFL)%LHV      =.FALSE.
ENDDO

YAEROUT_NL(1)%CNAME='AEROUT-1'
YAEROUT_NL(2)%CNAME='AEROUT-2'
YAEROUT_NL(3)%CNAME='AEROUT-3'
YAEROUT_NL(4)%CNAME='AEROUT-4'
YAEROUT_NL(5)%CNAME='AEROUT-5'
YAEROUT_NL(6)%CNAME='AEROUT-6'
YAEROUT_NL(7)%CNAME='AEROUT-7'
YAEROUT_NL(8)%CNAME='AEROUT-8'



! * Defaults for SLDIA GFL fields GP
DO JGFL=1,NSLDIAGP
  YSLDIA_NL(JGFL)%LGP    = .TRUE.
  YSLDIA_NL(JGFL)%LSP    = .FALSE.
  YSLDIA_NL(JGFL)%NREQIN = 0
  YSLDIA_NL(JGFL)%LREQOUT= .FALSE.
  YSLDIA_NL(JGFL)%LGPINGP= .TRUE.
  YSLDIA_NL(JGFL)%LTRAJIO= .FALSE.
  YSLDIA_NL(JGFL)%LADV   = .FALSE.
  YSLDIA_NL(JGFL)%LCDERS = .FALSE.
  YSLDIA_NL(JGFL)%LT9 = .FALSE.
  YSLDIA_NL(JGFL)%LT1 = .TRUE.
  YSLDIA_NL(JGFL)%LT5 = .FALSE.
  YSLDIA_NL(JGFL)%LPT = .FALSE.
  YSLDIA_NL(JGFL)%LPC = .TRUE.
  YSLDIA_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YSLDIA_NL(JGFL)%IGRBCODE = 162100 + JGFL - 1
  YSLDIA_NL(JGFL)%NCOUPLING= 0
  YSLDIA_NL(JGFL)%LSLHD    =.FALSE.
  YSLDIA_NL(JGFL)%LVSPLIP  =.FALSE.
  YSLDIA_NL(JGFL)%LQM      = LSLAG
  YSLDIA_NL(JGFL)%LQMH     =.FALSE.
  YSLDIA_NL(JGFL)%LHV      =.FALSE.
ENDDO

YSLDIA_NL(1)%CNAME='DPDETA0         '

! * Defaults for SLDIA GFL fields SP
DO JGFL=NSLDIAGP+1,NSLDIA
  YSLDIA_NL(JGFL)%LGP    = .FALSE.
  YSLDIA_NL(JGFL)%LSP    = .TRUE.
  YSLDIA_NL(JGFL)%NREQIN = 0
  YSLDIA_NL(JGFL)%LREQOUT= .FALSE.
  YSLDIA_NL(JGFL)%LGPINGP= .FALSE.
  YSLDIA_NL(JGFL)%LTRAJIO= .FALSE.
  YSLDIA_NL(JGFL)%LADV   = .FALSE.
  YSLDIA_NL(JGFL)%LCDERS = .TRUE.
  YSLDIA_NL(JGFL)%LT9 = .FALSE.
  YSLDIA_NL(JGFL)%LT1 = .TRUE.
  YSLDIA_NL(JGFL)%LT5 = .FALSE.
  YSLDIA_NL(JGFL)%LPT = .FALSE.
  YSLDIA_NL(JGFL)%LPC = .TRUE.
  YSLDIA_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YSLDIA_NL(JGFL)%IGRBCODE = 162100 + JGFL - 1
  YSLDIA_NL(JGFL)%NCOUPLING= 0
  YSLDIA_NL(JGFL)%LSLHD    =.FALSE.
  YSLDIA_NL(JGFL)%LVSPLIP  =.FALSE.
  YSLDIA_NL(JGFL)%LQM      = LSLAG
  YSLDIA_NL(JGFL)%LQMH     =.FALSE.
  YSLDIA_NL(JGFL)%LHV      =.FALSE.
ENDDO


! Lipschitz
YSLDIA_NL(2)%CNAME    ='UTRAJ           '
YSLDIA_NL(3)%CNAME    ='VTRAJ           '
YSLDIA_NL(4)%CNAME    ='WTRAJ           '
! Jacobian
YSLDIA_NL(5)%CNAME    ='LAMBDA0         '
YSLDIA_NL(6)%CNAME    ='THETA0          '
YSLDIA_NL(7)%CNAME    ='ETA0            '

! * Defaults for forcing attributes
DO JGFL=1,JPFORC
  YFORC_NL(JGFL)%LGP = .TRUE.
  YFORC_NL(JGFL)%LSP = .FALSE.
  YFORC_NL(JGFL)%NREQIN = 1
  YFORC_NL(JGFL)%LREQOUT = .FALSE.
  YFORC_NL(JGFL)%LGPINGP = .TRUE.
  YFORC_NL(JGFL)%LTRAJIO = .FALSE.
  YFORC_NL(JGFL)%LADV = .FALSE.
  YFORC_NL(JGFL)%LCDERS = .FALSE.
  YFORC_NL(JGFL)%LT9 = .FALSE.
  YFORC_NL(JGFL)%LT1 = .FALSE.
  YFORC_NL(JGFL)%LT5 = .FALSE.
  YFORC_NL(JGFL)%LPT = .FALSE.
  YFORC_NL(JGFL)%LPC = .FALSE.
  YFORC_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YFORC_NL(JGFL)%IGRBCODE = 999
  YFORC_NL(JGFL)%NCOUPLING = 0
  YFORC_NL(JGFL)%LSLHD=.FALSE.
  YFORC_NL(JGFL)%LVSPLIP=.FALSE.
  YFORC_NL(JGFL)%LQM=.FALSE.
  YFORC_NL(JGFL)%LQMH=.FALSE.
  YFORC_NL(JGFL)%LHV=.FALSE.
ENDDO

! * Defaults for easy diag attributes
DO JGFL=1,JPEZDIAG
  YEZDIAG_NL(JGFL)%LGP = .TRUE.
  YEZDIAG_NL(JGFL)%LSP = .FALSE.
  YEZDIAG_NL(JGFL)%NREQIN = 0
  YEZDIAG_NL(JGFL)%LREQOUT = .TRUE.
  YEZDIAG_NL(JGFL)%LGPINGP = .TRUE.
  YEZDIAG_NL(JGFL)%LTRAJIO = .FALSE.
  YEZDIAG_NL(JGFL)%LADV = .FALSE.
  YEZDIAG_NL(JGFL)%LCDERS = .FALSE.
  YEZDIAG_NL(JGFL)%LT9 = .FALSE.
  YEZDIAG_NL(JGFL)%LT1 = .FALSE.
  YEZDIAG_NL(JGFL)%LT5 = .FALSE.
  YEZDIAG_NL(JGFL)%LPT = .FALSE.
  YEZDIAG_NL(JGFL)%LPC = .FALSE.
  YEZDIAG_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YEZDIAG_NL(JGFL)%IGRBCODE = 999
  YEZDIAG_NL(JGFL)%NCOUPLING = 0
  YEZDIAG_NL(JGFL)%LSLHD=.FALSE.
  YEZDIAG_NL(JGFL)%LVSPLIP=.FALSE.
  YEZDIAG_NL(JGFL)%LQM=.FALSE.
  YEZDIAG_NL(JGFL)%LQMH=.FALSE.
  YEZDIAG_NL(JGFL)%LHV=.FALSE.
ENDDO

! * Defaults for extra-GFL attributes
DO JGFL=1,SIZE(YEXT_NL)
  YEXT_NL(JGFL)%LGP    = .FALSE.
  YEXT_NL(JGFL)%LSP    = .FALSE.
  YEXT_NL(JGFL)%NREQIN = 0
  YEXT_NL(JGFL)%LREQOUT= .FALSE.
  YEXT_NL(JGFL)%LGPINGP= .FALSE.
  YEXT_NL(JGFL)%LTRAJIO= .FALSE.
  YEXT_NL(JGFL)%LADV   = .FALSE.
  YEXT_NL(JGFL)%LCDERS = .FALSE.
  YEXT_NL(JGFL)%LT9 = .FALSE.
  YEXT_NL(JGFL)%LT1 = .FALSE.
  YEXT_NL(JGFL)%LT5 = .FALSE.
  YEXT_NL(JGFL)%LPT = .FALSE.
  YEXT_NL(JGFL)%LPC = .FALSE.  
  YEXT_NL(JGFL)%CNAME = 'XXXXXXXXXXXXXXX'
  YEXT_NL(JGFL)%IGRBCODE = 999
  YEXT_NL(JGFL)%NCOUPLING= 0
  YEXT_NL(JGFL)%LSLHD    =.FALSE. 
  YEXT_NL(JGFL)%LVSPLIP  =.FALSE.  
  YEXT_NL(JGFL)%LQM      =.FALSE.
  YEXT_NL(JGFL)%LQMH     =.FALSE.
  YEXT_NL(JGFL)%LHV      =.FALSE.
ENDDO

! initialize pointers for AROME Chemistry
! ky: it would be better to design a specific GFL structure 'CHEM'
!     for that purpose in the future
!     (NCHEM variables, JPCHEM = max number of chemical variables),
!     or to dispatch these GFL on some already existing GFL groups like
!     AERO, GHG, GRG, TRAC.
NSV_CHEMBEG=0
NSV_CHEMEND=0
NSV_DSTBEG=0
NSV_DSTEND=0
NSV_DSTDEPBEG=0
NSV_DSTDEPEND=0
NSV_AERBEG=0
NSV_AEREND=0
IF (LMPA.AND.(LUSECHEM).OR.(LRDUST).OR.(LRCO2)) THEN
  !      initialise scalar (gas chemistry, aerosols,..)  NGFL_EXT
  WRITE(NULOUT,*)'LUSECHEM,LORILAM,LRDUST,LRCO2,JPGFL,NGFL_EXT =',&
               &  LUSECHEM,LORILAM,LRDUST,LRCO2,JPGFL,NGFL_EXT
  CALL AROINI_NSV0(LUSECHEM,LORILAM,LRDUST,LRCO2,LRDEPOS,JPGFL,NGFL_EXT,&
   &              NSV_CHEMBEG, NSV_CHEMEND, NSV_DSTBEG, NSV_DSTEND,     &
   &              NSV_DSTDEPBEG, NSV_DSTDEPEND, NSV_AERBEG, NSV_AEREND, &
   &              NSV_CO2,CLNAME)
  
  WRITE(NULOUT,*)'NSV_CHEMBEG, NSV_CHEMEND =',NSV_CHEMBEG, NSV_CHEMEND
  WRITE(NULOUT,*)'NSV_AERBEG, NSV_AEREND =',NSV_AERBEG, NSV_AEREND
  WRITE(NULOUT,*)'NSV_DSTBEG, NSV_DSTEND =',NSV_DSTBEG, NSV_DSTEND
  WRITE(NULOUT,*)'NSV_DSTDEPBEG, NSV_DSTDEPEND =',NSV_DSTDEPBEG, NSV_DSTDEPEND
  WRITE(NULOUT,*)'LUSECHEM,LORILAM,LRDUST,LRDEPOS =',LUSECHEM,LORILAM,&
   &              LRDUST,LRDEPOS
  WRITE(NULOUT,*)'CLNAME =',CLNAME(1:NGFL_EXT)
  IF (LUSECHEM) THEN
    YEXT_NL(NSV_CHEMBEG:NSV_CHEMEND)%CNAME(1:15) = &
             &CLNAME(NSV_CHEMBEG:NSV_CHEMEND)(1:15)
  ENDIF
  IF (LRDUST)  THEN
    YEXT_NL(NSV_DSTBEG:NSV_DSTEND)%CNAME(1:15) = &
           &CLNAME(NSV_DSTBEG:NSV_DSTEND)(1:15)
    IF (LRDEPOS) THEN
      YEXT_NL(NSV_DSTDEPBEG:NSV_DSTDEPEND)%CNAME(1:15) = &
           &CLNAME(NSV_DSTDEPBEG:NSV_DSTDEPEND)(1:15)
        WRITE(NULOUT,*)'SET UP YEXT_NL =',&
           & YEXT_NL(NSV_DSTDEPBEG:NSV_DSTDEPEND)%CNAME(1:15)
    ENDIF

  ENDIF
  IF (LORILAM) THEN
    YEXT_NL(NSV_AERBEG:NSV_AEREND)%CNAME(1:15) = &
          & CLNAME(NSV_AERBEG:NSV_AEREND)(1:15)
  ENDIF
  IF (LRCO2) THEN
    YEXT_NL(NSV_CO2)%CNAME(1:15) = &
          & CLNAME(NSV_CO2)(1:15)
  ENDIF
  WRITE(NULOUT,*)'YEXT_NL%CNAME =',YEXT_NL(1:NGFL_EXT)%CNAME
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDEFO_GFLATTR',1,ZHOOK_HANDLE)
END SUBROUTINE SUDEFO_GFLATTR
