SUBROUTINE SUDEFO_VV1(LDECMWF,KFLEV,PVALH,PVBH,PVP00)

!**** *SUDEFO_VV1*  - Routine to initialize default values of vertical coordinate

!     Purpose.
!     --------
!           Initialize default values of the hybrid-cordinate system of the
!           model.

!**   Interface.
!     ----------

!     *CALL* SUDEFO_VV1

!        Explicit arguments :
!        --------------------
!          LDECMWF: .TRUE. if set -up of ECMWF, .FALSE. if METEO-FRANCE
!          KFLEV  : Number of vertical levels
!          PVALH  : Vertical function *A*
!          PVBH   : Vertical function *B*
!          PVP00  : Reference pressure

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib  *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original   : 95-01-05
!        L. Isaksen : 96-04-04  KFLEV=1 option added
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDYNCORE, ONLY: LDYNCORE, RP00_DYN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
LOGICAL           ,INTENT(IN)    :: LDECMWF 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVALH(0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVBH(0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVP00 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1. READ NAMELISTS.
!          ------------------

IF (LHOOK) CALL DR_HOOK('SUDEFO_VV1',0,ZHOOK_HANDLE)
IF (LDECMWF) THEN
  IF (KFLEV == 19) THEN
    PVALH(0)=0.0_JPRB
    PVALH(1)= 2000.0_JPRB
    PVALH(2)= 4000.0_JPRB
    PVALH(3)= 6046.110595_JPRB
    PVALH(4)= 8267.92756_JPRB
    PVALH(5)= 10609.513232_JPRB
    PVALH(6)= 12851.100169_JPRB
    PVALH(7)= 14698.498086_JPRB
    PVALH(8)= 15861.125180_JPRB
    PVALH(9)= 16116.236610_JPRB
    PVALH(10)= 15356.924115_JPRB
    PVALH(11)= 13621.460403_JPRB
    PVALH(12)= 11101.561987_JPRB
    PVALH(13)= 8127.144155_JPRB
    PVALH(14)= 5125.141747_JPRB
    PVALH(15)= 2549.969411_JPRB
    PVALH(16)= 783.195032_JPRB
    PVALH(17)= 0.0_JPRB
    PVALH(18)= 0.0_JPRB
    PVALH(19)= 0.0_JPRB
    PVBH(0)= 0.0000_JPRB
    PVBH(1)= 0.0000_JPRB
    PVBH(2)= 0.0000_JPRB
    PVBH(3)= 0.0003389933_JPRB
    PVBH(4)= 0.0033571866_JPRB
    PVBH(5)= 0.0130700434_JPRB
    PVBH(6)= 0.0340771467_JPRB
    PVBH(7)= 0.0706498323_JPRB
    PVBH(8)= 0.1259166826_JPRB
    PVBH(9)= 0.2011954093_JPRB
    PVBH(10)= 0.2955196487_JPRB
    PVBH(11)= 0.4054091989_JPRB
    PVBH(12)= 0.5249322235_JPRB
    PVBH(13)= 0.6461079479_JPRB
    PVBH(14)= 0.7596983769_JPRB
    PVBH(15)= 0.8564375573_JPRB
    PVBH(16)= 0.9287469142_JPRB
    PVBH(17)= 0.9729851852_JPRB
    PVBH(18)= 0.9922814815_JPRB
    PVBH(19)= 1.0_JPRB
  ELSEIF (KFLEV == 31) THEN
    PVALH(0)= 0.0_JPRB
    PVALH(1)= 2000.000000_JPRB
    PVALH(2)= 4000.000000_JPRB
    PVALH(3)= 6000.000000_JPRB
    PVALH(4)= 8000.000000_JPRB
    PVALH(5)= 9976.135361_JPRB
    PVALH(6)= 11820.539617_JPRB
    PVALH(7)= 13431.393926_JPRB
    PVALH(8)= 14736.356909_JPRB
    PVALH(9)= 15689.207458_JPRB
    PVALH(10)= 16266.610500_JPRB
    PVALH(11)= 16465.005734_JPRB
    PVALH(12)= 16297.619332_JPRB
    PVALH(13)= 15791.598604_JPRB
    PVALH(14)= 14985.269630_JPRB
    PVALH(15)= 13925.517858_JPRB
    PVALH(16)= 12665.291662_JPRB
    PVALH(17)= 11261.228878_JPRB
    PVALH(18)= 9771.406290_JPRB
    PVALH(19)= 8253.212096_JPRB
    PVALH(20)= 6761.341326_JPRB
    PVALH(21)= 5345.914240_JPRB
    PVALH(22)= 4050.717678_JPRB
    PVALH(23)= 2911.569385_JPRB
    PVALH(24)= 1954.805296_JPRB
    PVALH(25)= 1195.889791_JPRB
    PVALH(26)= 638.148911_JPRB
    PVALH(27)= 271.626545_JPRB
    PVALH(28)= 72.063577_JPRB
    PVALH(29)= 0.000000_JPRB
    PVALH(30)= 0.000000_JPRB
    PVALH(31)= 0.000000_JPRB
    PVBH(0)= 0.0000000000_JPRB
    PVBH(1)= 0.0000000000_JPRB
    PVBH(2)= 0.0000000000_JPRB
    PVBH(3)= 0.0000000000_JPRB
    PVBH(4)= 0.0000000000_JPRB
    PVBH(5)= 0.0003908582_JPRB
    PVBH(6)= 0.0029197006_JPRB
    PVBH(7)= 0.0091941320_JPRB
    PVBH(8)= 0.0203191555_JPRB
    PVBH(9)= 0.0369748598_JPRB
    PVBH(10)= 0.0594876397_JPRB
    PVBH(11)= 0.0878949492_JPRB
    PVBH(12)= 0.1220035886_JPRB
    PVBH(13)= 0.1614415235_JPRB
    PVBH(14)= 0.2057032385_JPRB
    PVBH(15)= 0.2541886223_JPRB
    PVBH(16)= 0.3062353873_JPRB
    PVBH(17)= 0.3611450218_JPRB
    PVBH(18)= 0.4182022749_JPRB
    PVBH(19)= 0.4766881754_JPRB
    PVBH(20)= 0.5358865832_JPRB
    PVBH(21)= 0.5950842740_JPRB
    PVBH(22)= 0.6535645569_JPRB
    PVBH(23)= 0.7105944258_JPRB
    PVBH(24)= 0.7654052430_JPRB
    PVBH(25)= 0.8171669567_JPRB
    PVBH(26)= 0.8649558510_JPRB
    PVBH(27)= 0.9077158297_JPRB
    PVBH(28)= 0.9442132326_JPRB
    PVBH(29)= 0.9729851852_JPRB
    PVBH(30)= 0.9922814815_JPRB
    PVBH(31)= 1.0000000000_JPRB
  ENDIF
ELSE
  IF (KFLEV == 15) THEN
    PVALH(0) =0.0_JPRB
    PVALH(1) =5000.000000_JPRB
    PVALH(2) =8571.428570_JPRB
    PVALH(3) =10879.120880_JPRB
    PVALH(4) =12087.912090_JPRB
    PVALH(5) =12362.637360_JPRB
    PVALH(6) =11868.131870_JPRB
    PVALH(7) =10769.230770_JPRB
    PVALH(8) =9230.769230_JPRB
    PVALH(9) =7417.582420_JPRB
    PVALH(10)=5494.505490_JPRB
    PVALH(11)=3626.373630_JPRB
    PVALH(12)=1978.021980_JPRB
    PVALH(13)=714.285710_JPRB
    PVALH(14)=0.000000_JPRB
    PVALH(15)=0.000000_JPRB
    PVBH(0) =0.0_JPRB
    PVBH(1) =0.0000000000_JPRB
    PVBH(2) =0.0271314286_JPRB
    PVBH(3) =0.0773309890_JPRB
    PVBH(4) =0.1465353846_JPRB
    PVBH(5) =0.2306813187_JPRB
    PVBH(6) =0.3257054945_JPRB
    PVBH(7) =0.4275446154_JPRB
    PVBH(8) =0.5321353846_JPRB
    PVBH(9) =0.6354145055_JPRB
    PVBH(10)=0.7333186813_JPRB
    PVBH(11)=0.8217846154_JPRB
    PVBH(12)=0.8967490110_JPRB
    PVBH(13)=0.9541485714_JPRB
    PVBH(14)=0.9899200000_JPRB
    PVBH(15)=1.0000000000_JPRB
  ELSEIF (KFLEV == 21) THEN
    PVALH(0) =0.0_JPRB
    PVALH(1) =2718.281828_JPRB
    PVALH(2) =4892.907290_JPRB
    PVALH(3) =6566.796626_JPRB
    PVALH(4) =7782.870076_JPRB
    PVALH(5) =8584.047877_JPRB
    PVALH(6) =9013.250271_JPRB
    PVALH(7) =9113.397497_JPRB
    PVALH(8) =8927.409793_JPRB
    PVALH(9) =8498.207399_JPRB
    PVALH(10)=7868.710554_JPRB
    PVALH(11)=7081.839499_JPRB
    PVALH(12)=6180.514472_JPRB
    PVALH(13)=5207.655712_JPRB
    PVALH(14)=4206.183460_JPRB
    PVALH(15)=3219.017954_JPRB
    PVALH(16)=2289.079434_JPRB
    PVALH(17)=1459.288139_JPRB
    PVALH(18)= 772.564309_JPRB
    PVALH(19)= 271.828183_JPRB
    PVALH(20)=    .000000_JPRB
    PVALH(21)=   0.000000_JPRB
    PVBH(0) =0.0_JPRB
    PVBH(1) =.0000000000_JPRB
    PVBH(2) =.0137314671_JPRB
    PVBH(3) =.0397781545_JPRB
    PVBH(4) =.0767238156_JPRB
    PVBH(5) =.1231522038_JPRB
    PVBH(6) =.1776470723_JPRB
    PVBH(7) =.2387921744_JPRB
    PVBH(8) =.3051712636_JPRB
    PVBH(9) =.3753680930_JPRB
    PVBH(10)=.4479664161_JPRB
    PVBH(11)=.5215499862_JPRB
    PVBH(12)=.5947025565_JPRB
    PVBH(13)=.6660078804_JPRB
    PVBH(14)=.7340497113_JPRB
    PVBH(15)=.7974118024_JPRB
    PVBH(16)=.8546779070_JPRB
    PVBH(17)=.9044317785_JPRB
    PVBH(18)=.9452571703_JPRB
    PVBH(19)=.9757378356_JPRB
    PVBH(20)=.9944575277_JPRB
    PVBH(21)=1.0000000000_JPRB
  ELSEIF (KFLEV == 24) THEN
    PVALH( 0) =     0.000000_JPRB
    PVALH( 1) =  2718.281828_JPRB
    PVALH( 2) =  4963.818991_JPRB
    PVALH( 3) =  6768.844079_JPRB
    PVALH( 4) =  8165.589682_JPRB
    PVALH( 5) =  9186.288393_JPRB
    PVALH( 6) =  9863.172800_JPRB
    PVALH( 7) = 10228.475497_JPRB
    PVALH( 8) = 10314.429072_JPRB
    PVALH( 9) = 10153.266118_JPRB
    PVALH(10) =  9777.219225_JPRB
    PVALH(11) =  9218.520983_JPRB
    PVALH(12) =  8509.403985_JPRB
    PVALH(13) =  7682.100820_JPRB
    PVALH(14) =  6768.844079_JPRB
    PVALH(15) =  5801.866353_JPRB
    PVALH(16) =  4813.400234_JPRB
    PVALH(17) =  3835.678311_JPRB
    PVALH(18) =  2900.933177_JPRB
    PVALH(19) =  2041.397421_JPRB
    PVALH(20) =  1289.303634_JPRB
    PVALH(21) =   676.884408_JPRB
    PVALH(22) =   236.372333_JPRB
    PVALH(23) =     0.000000_JPRB
    PVALH(24) =     0.000000_JPRB
    PVBH( 0) = 0.0000000000_JPRB
    PVBH( 1) = 0.0000000000_JPRB
    PVBH( 2) = 0.0105139689_JPRB
    PVBH( 3) = 0.0306022548_JPRB
    PVBH( 4) = 0.0593252058_JPRB
    PVBH( 5) = 0.0957431701_JPRB
    PVBH( 6) = 0.1389164957_JPRB
    PVBH( 7) = 0.1879055308_JPRB
    PVBH( 8) = 0.2417706234_JPRB
    PVBH( 9) = 0.2995721218_JPRB
    PVBH(10) = 0.3603703740_JPRB
    PVBH(11) = 0.4232257281_JPRB
    PVBH(12) = 0.4871985322_JPRB
    PVBH(13) = 0.5513491346_JPRB
    PVBH(14) = 0.6147378832_JPRB
    PVBH(15) = 0.6764251262_JPRB
    PVBH(16) = 0.7354712117_JPRB
    PVBH(17) = 0.7909364878_JPRB
    PVBH(18) = 0.8418813027_JPRB
    PVBH(19) = 0.8873660044_JPRB
    PVBH(20) = 0.9264509411_JPRB
    PVBH(21) = 0.9581964609_JPRB
    PVBH(22) = 0.9816629119_JPRB
    PVBH(23) = 0.9959106422_JPRB
    PVBH(24) = 1.0000000000_JPRB
  ENDIF
ENDIF
IF (KFLEV == 1) THEN
  PVALH( 0) = 0.0000000000_JPRB
  PVALH( 1) = 0.0000000000_JPRB
  PVBH( 0)  = 0.0000000000_JPRB
  PVBH( 1)  = 1.0000000000_JPRB
ENDIF

IF( LDYNCORE ) THEN
  PVP00 = RP00_DYN
ELSE
  PVP00 = 101325._JPRB
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDEFO_VV1',1,ZHOOK_HANDLE)
END SUBROUTINE SUDEFO_VV1

