SUBROUTINE SUDIM1

!     ----------------------------------------------------------------------
!**** *SUDIM1 * - Setting up of the dimensioning of the model

!     Purpose.
!     --------
!           Initialization of YOMDIM, and some prints

!**   Interface.
!     ----------
!        *CALL* *SUDIM1*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        D. VASILJEVIC  : 90-06-19 DIMENSIONS FOR OBS. RELATED ARRAYS
!        Christian Blondin : 90-09-19 Dimensioning for O.I job.
!        D. VASILJEVIC  : 90-11-20 REORGANISTION REGARDING OBS./GUESS
!      Modified 91-02-28 by Michel Deque (Relaxation of deep soil values
!      Modified 91-03-15 by Michel Deque (Ozone parameterization)
!      Modified 91-04-01 by Michel Deque (pseudo-historical cloudiness)
!      Modified 91-05-21 by A.Lasserre-Bigorry (scalar variables)
!      Modified 91-07-02 by Philippe Courtier (u-v formulation)
!      Modified 91-07-02 by Philippe Courtier (cleaning)
!      Modified 91-07-02 by Philippe Courtier (Legendre transforms)
!      Modified 91-07-07 by Pascal Marquet (check NMI values)
!      Modified 91-10-01 by M. Deque (anisotropic orography variance)
!      Modified 91-05-11 YOMNMI --> (YOMNMIA and YOMNMIB)
!      Modified 91-11-06 by P. Courtier and M.Hamrud security JPARITY
!      Modified 91-11-20 by P. Moll (Trajectory array positioning)
!      Modified 92-02-16 by M. Deque (2d  and 3d fields for ozone,
!                                             2d fields for vegetation)
!      Modified 92-01-19 by D. Giard (NMI tests)
!      Modified 92-04-28 by Bill Heckley (Initialization of First-guess)
!      Modified 30/5/92 by A. Joly (Introduction of dyn.met. pp)
!      Modified 92-07-15 by D. Giard (NG3DTN suppressed)
!      Modified 93-04-13 by L. Labbe (Dim. for inverse Legendre transfor
!      Modified 93-03-19 by D. Giard : NTMAX,NTEFRE added
!       (NTMAX=truncation for tendencies on the transformed sphere)
!                                    : direct Legendre transf. modified
!      Modified 93-??-?? by L. Labbe : Dim. for GT0    (add orography)
!                                      Dim. for GT0BUF (add orography)
!                                      Dim. for YOMGC5 (trajectory orogra
!      Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by
!      Modified by R. El Khatib : 93-05-06 Set-up through command line
!      Modified by M. Hamrud    : 93-07-05 Make use of above for ECMWF
!      Modified by C. Temperton : 93-11-01 Specification of NXMAX
!      Modified by R. Buizza    : 93-12-01 Set up to run conf 601
!      Modified by C. Temperton : 94-01-13 Fix for T21 diabatic NMI
!      Modified by D. Giard     : 94-02-08 corrections, set-up of NXTIDE
!      Modified by R. EL Khatib : 94-03-03 Full-POS implementation
!      Modified by M. Deque     : 94-09-05 Aerosols
!      K. YESSAD (APRIL 1994)   : Unified horizontal diffusion scheme.
!      Modified by C. Temperton : 94-05-18 Spectral RT option
!      Modified by M. Hamrud    : 94-04-09 Upper air gridpoint fields
!      Modified by R. EL Khatib : 94-03-03 More Full-POS implementation
!      Modified by C. Temperton : 94-05-18 Spectral RT option
!      Modified by M. Hamrud    : 94-04-09 Upper air gridpoint fields
!      Modified by J. N Thepaut : 94-07-14 number of surf fields for TL
!                                 incremental.
!      Modified by R. EL Khatib : 94-11-10 Non-hydrostatic dynamics
!      Modified by M. Tolstykh  : 94-08-31 CUD interpolation in
!                                 semi-Lag scheme
!      Modified by Ph. Dandin   : 94-11-17 FMR scheme with MF physics
!      Modified by C. Temperton : 95-02-06 Two-time-level scheme
!      Modified by G. Radnoti   : 95-01-23 NDGSUR for ALADIN full-pos
!                                 as well
!      Modified by M. Deque     : 95-04-26 ice coupler
!      Modified by K. Yessad    : 95-05-05 Add new variable NOSIHEG.
!      Modified by L. Isaksen   : 95-06-06 Reordering of spectral arrays
!      Modified by D. Giard     : 95-08-30 NXEFRE, NTEFRE suppressed
!      Modified by J. N Thepaut : 95-01-10 disabling of obs dimensioning
!                                 during Full-Pos.
!      Modified by C. Temperton : 95-10-13 Bugfix for two-time-level scheme.
!      Modified by K. YESSAD (AUGUST 1995):
!        - change value of NG3D9 for 2TL SL scheme.
!        - rewritting according to F90 norms.
!      Modified by R. EL Khatib : 95-11-16 Fullpos LFPART2 without command line
!      Modified by M. Deque     : 95-11-27 2nd call to APLPAR (NPROMB)
!      Modified by N. Wedi      : 95-12-10 FULL_POS in LECMWF mode
!      Modified L. Isaksen      : 96-02-20 Remove NFLSURL,NFLSURG
!                                          Introduce NFLEVLMX
!      Modified by A. Untch     : 96-03 grid-point prognostic ozone (EC)
!      Modified by C. Temperton : 96-10-23 Check semi-Lag is being used
!                                          on linear grid
!      Modified by F. Rabier    : 96-10 dimensioning for conf 801
!      Modified by K. YESSAD    : 96-04-03 NDGSUR=5 instead of 6 for CUD5.
!      Modified by R. EL Khatib : 96-05-13 NDLSUR/NDSUR1 for under-linear grid
!      Modified by T. Lagarde   : 96-06-01 2nd call to APLPAR (NPROMC)
!      Modified by M. Deque     : 96-07-01 Nudging (NVCLIU)
!      Modified by R. EL Khatib : 96-07-30 NFGPNH : number of gridpoint NH
!        fields moved from aladin to Arpege/Ifs
!      Modified by J.M. Piriou  : 97-02-28 initialization of NVCLIN in case of
!                                 LNEBN
!      Modified OCT 1996 by K. YESSAD: removal of variable uncentering.
!      Modified MAY 1997 by M. DEQUE : frozen FMR
!      Modified by M. Janiskova : 95-12-12 set physics dimensioning for
!                                 simpl.phys.par.
!      Modified SEPT 1996 by K. YESSAD:
!          - Spectral transforms for transmission coefficients.
!      Modified by G. Hello : 97-08 dimensioning for conf 801 MF.
!      Modified by J. Bidlot    : 97-06-12 implement wave fields
!      Modified by E.Andersson  : 97-11-17 Flexible GOM arrays.
!      Modified by E.Holm       : 98-01-08 Include spectral ozone switch
!                                          for data assimilation
!      Modified by C. Temperton : 98-01-19 2TL SL scheme for SWE.
!      Modified by C. Temperton : 98-03-04 Fix for TL/AD of SWE.
!      Modified by C. Madeira, A. Dziedzic : 97-07-23 ISBA Fields in FULL-POS
!      Modified by E. Bazile    : 97-11-18 Soil freezing (LFGEL)
!      Modified by M. Janiskova : 97-11-14 dimensioning NVCLIN for conf.131 MF
!      Modified by J.M. Piriou  : 97-12-22
!       - dimensioning NVCLIN after NAMDIM read.
!      Modified by J.M. Piriou  : 97-12-23
!       - dimensioning NVCLIN independent from LNOINI.
!      Modified by M. Deque     : 98-02-20 NVCLIN when LNEBR AND LNEBCO
!      Modified by M. Janiskova : 98-11-18 dimensioning for MF conf.131,401,501
!                               : 99-02-21 specified variables for rad.coef.
!      Modified by V. Lorant    : 98-08-05 NVCLIN whith mixing length
!                                 NVCLIS variables stored as 1d fields (NTOZ1D)
!      Modified by T. Bergot    : 98-08 dimensioning for conf 601 MF.
!      Modified by K. YESSAD    : 98-08-07 rewrite the computation of NG3D9 and
!        NG2D9 in a more readable way for 3D and 2D models, make this part of
!        code consistent with the number of t-dt variables read/written in
!        SC2RDT9/SC2WRT9 (and probably correct some bugs).
!      Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!        - removal of LRPOLE in YOMCT0.
!        - removal of code under LRPOLE.
!      Modified by R. El Khatib : 98-08-31 Adiabatic pp.
!      Modified by Z. Bencetic Klaic: 98-10-20
!       - switch for ozone coefficients (LOZSP);
!      Modified by R. El Khatib : 98-11-03
!       - IPROM2 in namelist replaces NPROMA if fullpos LFPART2
!      Modified by J. Teixeira  : 99-03-26 Surface tiling
!      Modified by C. Temperton : 99-08-31 Fix for TL/AD of SL (3D model)
!      Modified by E. Bazile    : 99-01-04 Surface soil freezing (NVRS=4)
!      Modified by M. Charron   : 99-02-01 non-hydrostatic NPROMT
!      Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!      Modified by K. YESSAD : 98-12-04 New option 'lradvtoro'.
!      Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!      Modified 99-06-04 by K. YESSAD: correct default for (NSMIN,NSMAX,NDGL)
!        in configurations 911 and 912. 
!        - if command line, correct bugs (false and fancy previous code).
!        - if not command line, allow options letting SUDIM to compute
!          some adequate default values for NSMAX, NDGL.
!      Modified by C. Fischer   : 99-06-30 nvarmax for cost-functions
!      Modified 00-05-15 by C. Jakob: new EC surface dimensions
!      Modified NOV 1999 by K. YESSAD:
!       - in 'lacudw', compute PDLAT and ILA as in 'lascaw' =>
!         'nrlatx', 'nindlat' and 'rdllat' become useless.
!      R. El Khatib : 00-01-28
!       - Fix to enable in-line fullpos + any initialisation witout integration
!      Modified by K. YESSAD : 00-08-21 relaxation of thin layer hypothesis.
!      R. El Khatib : 00-0-31 NAFPOS initialization moved from SUAFPDS.
!      Modified by D. Giard     : 00-09-12 cleaning old 923
!      Modified by D. Giard     : 00-09-25 bugfix for 923-DM (NFAUX)
!      Modified by D. Giard     : 00-10-25 Snow & Vegetation (NVCLIV=10,LVGSN)
!      Modified by E. Holm      : 00-11-28
!       - Heterogeneous chemistry coefficient added (NVCLIS=8)
!      Modified by E. Bazile    : 00-12-12 NVRS=4 by default.
!      Modified 01-07-10 by K. YESSAD: relaxation of thin layer hyp for SL2TL.
!      R. El Khatib : 01-08-07 Pruning options
!      Modified by M. Hamrud    : 01-03-25 Split in two
!      Modified 01-11-27 by S. ABDALLA: NVWV increased from 3 to 8
!                               (Charnock, U10, V10, Q2m, MSL, SKT, ice?, Zi/L)
!      Modified by R. El khatib : 01-10-23
!       - quick&dirty fix for fullpos after split in two
!      Modified by Y.Bouteloup  : 02-03-29
!       - xxO3ABC => climatological ozone profiles
!      Modified by F. Bouyssel  : 02-06-25 introduction of NVCLIH
!      Modified by K. YESSAD    : 02-10-23
!       - Allow NTCMAX=NDLON/2.
!       - replace SUDIM by SUDIM1 in all prints.
!       - print all error diagnostics on NULERR.
!      Modified by P.Smolikova  : 02-09-30 option NVDVAR=4 for d4 in NH
!      R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!      Modified by JJMorcrette 02-09-30 ECMWF PAR, UV, CAPE
!      Modified by C. Moussy   03-10-01:
!       - Initialisation YxNL%LCOUPLING, YEXT_NL(JGFL)%LCOUPLING
!      Modified by F. Vana      : 03-08-20 LUVDER = .true. when SLHD
!      Modified by M. Tudor     : 03-10-06 
!       - MF part of NVCLIN removed, NVCLIH modified
!      Modified 03-11-01 by J. Bidlot: NVWV decreased from 8 to 5
!                                      (Charnock, U10, V10, air density, Zi/L)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      Modified by M. Tudor     : 03-10-17 
!       - moved 3D variables from NVCLIH to GFL
!      Modified by P. Marquet   : 02-07-17 add VCLIA for aerosol files
!      Modified by Y. Seity     : 04-11-18 add AROME GFL 
!      Modified by C. Temperton : 04-01-30 Fix for shallow water equations
!        M.Hortal      01-Dec-2003 NVEXTRDYN: Extra-fields coming from dynamics
!      Modified by A. Tompkins: 11-02-04 Added 2 to NVDIAG for new TCLW,TCIW
!        G. Radnoti    12-Aug-2004 LUVDER=T in nconf=131 when LJC
!        Y.Tremolet    27-Jul-2004 Model error setup
!        R. Engelen    17-Mar-2005 Greenhouse gases in GFL
!        D.Tan         14-Mar-2005 Aeolus setup (under PILOT)
!        A. Untch      March 2005: Aerosols in GFL 
!         20-Feb-2005 J. Vivoda     3TL Eul PC scheme, GWADV scheme for PC_FULL
!      Modified by G. Hello : 05-04-25 lground option (ee927 for arome + surfex)
!        May-2005    C. Payan   : default (and if LECMWF) NUSE_ECCI values
!                                  modification by namdim
!      Modified by A. Alias : 31-08-05 GMGEC/EAC modif. (M. Deque grid point nudging)
!        Oct-2005    Y. Bouteloup : Introduce YCVGQ  
!                                  (Moisture convergence for French physics)
!        S. Serrar    : 05-06-23 CO2 surface fluxes added (NVSF)
!        JJMorcrette   10-05-2006  MODIS albedo
!        @%4@"$!!! ???????? add +5 to NVDIAG 
!        JJMorcrette 20060510 MODIS albedo
!        JJMorcrette 20060726 Add 2 to NVDIAG for clear-sky PAR and TOA inc.solar radiation
!        JJMorcrette   10-05-2006  MODIS albedo
!        S. Serrar     07-09-2006  tracers used for diagnostics only (GEMS)
!        M.Hamrud      01-Jul-2006 Revised surface fields
!      Modified by A. Alias : 10-03-06 correct value of NVCLIU if LNUDPG
!      Modified by A. Alias : 20-03-06 default for YSDSAT and YCVV
!      Modified by K. Yessad (Oct 2006): do some cleanings and a lot of
!       'hidden bugs' fixes (GFL attr.) during the merge of CY31T2 and CY31R2.
!      28-Sep-2006 M. Bellus : defaults for prognostic covection fields (ALARO0)
!      Modified by A. Alias : 30-11-06 Introduction of severals vertical layers 
!                                      in deep SOIL (LTPROF)
!      Modified by K. Yessad: 31-01-2007: modularisation and big cleanings.
!      J.Haseler     27-Feb-2007 Modify defaults for LTRAJIO and LGPINGP
!      JJMorcrette   20060925    DU, BC, OM, SO2, VOL, SOA climatological flds
!      JJMorcrette   20061020    diagnostic of aerosol physical fluxes
!      22-Mar-2007 S.Serrar  : defaults for ERA40 GFL fields
!      02-MAY-2008 S.Serrar  : check non consistent values for NERA40
!      18-Dec-2007   Y. Seity : add Hail
!      Modified by A. Alias : 19-10-07 Setup of NTOTFNUDG
!      G.Balsamo     20081016    add lake and ocean dimensions
!      JJMorcrette   20090217    PP of aerosol and UV processor output fields
!      Y.Tremolet    27-Jan-2009 Model error for stratosphere
!   30-Jun-2008 J. Masek   New SLHD interpolators.
!      Modified by K Yessad : 15-Sep-08 Prune conf 951.
!      A. Fouilloux  09/09/2009: remove call to sudimo
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIM   , ONLY : JPFPPYX
USE YOMARG   , ONLY : NULIM    ,NARGS    ,NUTMAX   ,NUCMAX   ,NUDGL    ,&
 & NUDLON   ,NUFLEV   ,NUSMAX   ,NUSTTYP  ,USTRET  
USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NFLEVG   ,NFAUX    ,NFC2D    ,&
 & NPROMA   ,NPPM     ,NPROMB   ,NPROMC   ,NVARMAX  ,NMSMAX   ,NSMAX    ,&
 & NSMIN    ,NCMAX    ,NTCMAX   ,NMTCMAX  ,NDGUNG   ,NDGUXG   ,NDLUNG   ,&
 & NDLUXG   ,NXMAX    ,NTMAX    ,NFPPYX   ,NFPPYE   ,NRLEVX   ,LVOR     ,&
 & LADER    ,LUVDER   ,LSPT     ,NUNDEFLD ,LOPTPROMA,NTOTFNUDG 
USE YOMPLDSW , ONLY : LOPT_SCALAR
USE YOMLUN   , ONLY : NULOUT   ,NULNAM   ,NULERR
USE YOMSLPHY , ONLY : LSLPHY   ,NVTEND
USE YOMCT0   , ONLY : NCONF    ,NSTOP    ,LSLAG    ,LFPART2  ,LNHDYN   ,&
 & LELAM    ,LVERCOR  ,LPC_FULL
USE YOMCT0B  , ONLY : LECMWF
USE YOMNMIA  , ONLY : NLEX     ,NVMOD    ,LNMIRQ   ,LRPIMP   ,LNPROJ   ,&
 & LCONMO   ,LCOIMP   ,LCOTYGR  
USE YOMDPHY  , ONLY : NVXP     ,NVXP2    ,NCXP     ,NCSI     ,NCOM      ,&
 & NCSNEC   ,&
 & NTILES   ,NVEXTR   ,NVXTR2   ,NCEXTR   ,NVCLIS   ,NTOZ1D   ,NTOZ2D   ,&
 & NTOZ3D   ,NTSSG    ,NTVG     ,NLOA     ,NLOE     ,NVEXTRDYN,LTPROF   ,&
 & LDIRCLSMOD
USE YOMGEMS  , ONLY : NCO2SFC
USE YOMVAR   , ONLY : LJC      ,LJCNMI   ,LJCNMTL  ,LMODERR  ,LREPRO4DVAR
USE YOMMCC   , ONLY : N923     ,LMCC01   ,LMCC02   ,LMCC03   ,LMCC04   ,&
 & LMCCEC   ,LMCCIEC  ,LMCCICEIC,  LMCC12   ,LMCC19   ,LN923    ,NOACOMM  ,&
 & LMCC05   ,NFRCPL   ,NFRCPL_CHE, LRDISTL1
USE YOMPHY   , ONLY : LMPHYS   ,LGWDC    ,LNEBCO   ,LNEBN    ,LOZONE   ,&
 & LSOLV
USE YOMPHYDS , ONLY : JPVXP    ,JPCXP
USE YOEPHY   , ONLY : LEO3CH
USE YOMINI   , ONLY : NEINI
USE YOMFPC
USE YOMFPG   , ONLY : NFPMAX
USE YOMGEM   , ONLY : NSTTYP   ,NHTYP    ,REFLCAPE ,REFLKUO  ,REFLRHC  ,&
 & RMUCEN   ,RLOCEN   ,RSTRET   ,RNLGINC
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, NDIM_MODERR, NSERR, LFGMODERR, &
 & LBGMODERR, ALPHAQ, WEAK4D_INTERV, TSTEP_ERR, NORDER_MODERR, NPRTMODERR, &
 & NPERIOD_4DVAR, L_WARM_START_MODERR, &
 & ALPHAQ_T,ALPHAQ_D,ALPHAQ_V,ALPHAQ_Q,ALPHAQ_O
USE YOMJQ    , ONLY : LQEQB, LJQBAL, LJQVCOR, LJQHCOR, LJQZERO, NLEVERR0, NLEVERR1
USE YOMDYNA  , ONLY : LSLHD, LSLDIA, LSLHD_STATIC
USE YOM_YGFL , ONLY : YQ_NL    ,YI_NL    ,YL_NL    ,YS_NL    ,YR_NL    , &
 & YG_NL    ,YH_NL   ,YTKE_NL  ,YA_NL    ,YO3_NL   ,YEXT_NL  ,YCPF_NL  , &
!>>dod
 & YCDNC_NL, YICNC_NL, YRE_LIQ_NL, YRE_ICE_NL, &
!<<dod
 & YSPF_NL  , YSRC_NL,YCVGQ_NL ,YAERO_NL ,YGHG_NL  ,YGRG_NL  ,YTRAC_NL , &
 & YSDSAT_NL, YCVV_NL,YFORC_NL ,YQVA_NL  ,YEZDIAG_NL, YUOM_NL  ,YUAL_NL, &
 & YDOM_NL  , YDAL_NL,YUEN_NL  ,YUNEBH_NL,YERA40_NL ,NGFL_EXT  ,YNOGW_NL, NACTAERO,&
 & NAERO    , NGHG   ,NGRG     ,NTRAC    ,NGFL_FORC ,NGFL_EZDIAG,NERA40, &
 & LAEROSFC ,LAERODIU,LFIRE    ,LSF6SFC, LGRGSFC, LCO2SFC, YGRGTEND_NL , &
 & LCH4SFC, YLRCH4_NL, YCH4S_NL, LTRCMFIX, NGRG_ASSIM, NGRG_CPLO4 , &
 & NSLDIA, NSLDIAGP, YSLDIA_NL, YAEROUT_NL, YUVP_NL, NAEROUT, NUVP, LAEROUT, LUVPOUT
USE YOMNUD   , ONLY : NFNUDG   ,LNUDST   ,LNUDG    ,LNUDSM   ,&
 & LNUDRM   ,LNUDSD   ,LNUDTG   ,LNUDQG   ,LNUDUG   ,LNUDVG   ,LNUDPG
USE GOMS_MIX     , ONLY : NUSE_ECCI
USE TYPE_GEMS_PROFILES, ONLY : NGEMS
USE YOMGRB, ONLY : NGRBGRG, NGRBGRGSFC, NGRBTCGRG, NGRBTRAC, NGRBTRACSFC, NGRBTCTRAC

USE YOMECEARTH, ONLY : LRDALB, LRDVEG, LRDLANDICE, LPISM, LLANDICE
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IUNDGLG
INTEGER(KIND=JPIM),SAVE :: IPROM2
INTEGER(KIND=JPIM) :: INS_ZON
INTEGER(KIND=JPIM) :: IPROMA
INTEGER(KIND=JPIM) :: JGFL

LOGICAL :: LLCON, LLNMI
 
REAL(KIND=JPRB) :: ZDGL,ZDLON,ZEPS,ZNCS,ZSEUIL

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namdim.h"
#include "namgfl.h"
#include "namdphy.h"
#include "nammcc.h"
#include "namgem.h"
#include "namfpc.h"
#include "nammoderr.h"

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "suctrl_gflattr.intfb.h"
#include "sudefo_gflattr.intfb.h"
#include "suncmax.intfb.h"
#include "aroini_nsv0.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDIM1',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*       0. DEFINE SOME THRESHOLDS.
!        --------------------------

ZSEUIL=1.E-13_JPRB
ZEPS=1.E-12_JPRB

!     ------------------------------------------------------------------
!*       1. SET DEFAULT VALUES AND MODIFY THEM.
!           -----------------------------------

!*       1.1 SET DEFAULT VALUES.

!        1.1.1 Set implicit default values for items other than GFL attributes

NDGLG =32
NDLON =64
NFLEVG=19
NSMAX =21
NCMAX = 1
NTMAX = 1
NTCMAX= MIN(6,NSMAX)

IF(LOPT_SCALAR) THEN
  NPROMA=50
ELSE
  NPROMA=2047
ENDIF
IPROMA=NPROMA

NPROMB=0
NPROMC=0
NDGUNG= 1
NDGUXG= NDGLG
NDLUNG= 1
NDLUXG= NDLON

NRLEVX=480000

NERA40=0
IF ( LSLDIA ) THEN
  NSLDIA=7
  NSLDIAGP=1
ELSE
  NSLDIA=0
  NSLDIAGP=0
ENDIF
NGHG=0
NTRAC=0
NGRG=0
NGRG_CPLO4=0
NGRG_ASSIM=0
NAERO=0
NACTAERO=0
NAEROUT=0
NUVP=0
LAEROUT=.FALSE.
LUVPOUT=.FALSE.
LAEROSFC=.FALSE.
LAERODIU=.FALSE.
LGRGSFC=.FALSE.
NGFL_FORC=0
NGFL_EZDIAG=0
NGFL_EXT=0
LCO2SFC=.FALSE.
LFIRE=.FALSE.
LSF6SFC=.FALSE.
LCH4SFC=.FALSE.
LTRCMFIX=.FALSE.
NFAUX =0
! the following NFPPYX is not adjustable, for this see NFPPYE, so
!  dont change it, thanks (no big arrays are dimensioned with NFPPYX)
NFPPYX=JPFPPYX
NFPPYE = 0
NPPM=4

NFC2D=1

NMSMAX = NSMAX
NTCMAX= MIN(NTCMAX,NMSMAX)
NSMIN = 21
NXMAX = 1

NVXP=0
NVXP2=0

NCO2SFC=3

NCSI=0
NCSNEC=0
NCXP=0
NTILES=0
NCOM=0
NTVG=0
NLOA=0
NLOE=0
NVEXTR=0
NVEXTRDYN=0
NVXTR2=0
NCEXTR=0
NTSSG=0
NVCLIS=0
NTOZ1D=0
NTOZ2D=0
NTOZ3D=0
NVTEND=0
LDIRCLSMOD=.FALSE.
LTPROF=.FALSE.

LSPT = .TRUE.

! Flexible EC-variables. Defaults
NUSE_ECCI=0

LLCON=LCONMO.OR.LCOIMP.OR.LCOTYGR
LLNMI=LNMIRQ.OR.LNPROJ.OR.LLCON.OR.( NCONF == 131 .AND.(LJCNMI.OR.LJCNMTL) )

! Set default for index to be used for unused/undefined fields
NUNDEFLD=-99999999
!NUNDEFLD=1

!        1.1.2  Set implicit default values for GFL attributes
!               All the GFL attributes defaults computed before reading the
!               namelists NAMDIM and NAMGFL SHOULD be defined in SUDEFO_GFLATTR

CALL SUDEFO_GFLATTR

!        1.1.3 Read NAMFPC.

NFPCLI=0
NFPSURFEX=0
CALL POSNAM(NULNAM,'NAMFPC')
READ(NULNAM,NAMFPC)
IF (LFPART2) NFPCLI=0

!        1.1.4 Modify default values according to LECMWF
!              and to some values read in namelist NAMFPC.

! k.y.: GFL setting-up SHOULD NOT appear in part 1.1.4.!
!       (put it in SUDEFO_GFLATTR).

IF (LECMWF) THEN
  IF(NARGS > 0.OR.LFPART2) THEN
    IF (LFPART2) THEN
      NSMAX=NFPMAX(1)
    ELSE
      NSMAX=NUSMAX
    ENDIF
  ENDIF
  IF(ANY(SPREAD(NCONF,1,4) == (/1,302,601,801/))) THEN
    NCSI=4
    NCSNEC=1
    NTILES=8
    NCOM=35 !KPP
  ELSEIF (NCONF == 501.OR.NCONF == 401) THEN
    NCSI=4
    NCSNEC=1
    NTILES=8
    NCOM=35 !KPP
  ELSEIF (NCONF == 131) THEN
    NCSI=4
    NCSNEC=1
    NTILES=8
    NCOM=35 !KPP
  ELSEIF ((NCONF == 911).OR.(NCONF == 912)) THEN
    NFLEVG = 1
    NUFLEV = 1
  ENDIF
  IF ((NCONF == 1.OR.NCONF == 302).AND.LEO3CH) THEN
    NTOZ3D=0
    NTOZ2D=1
    NTOZ1D=0
    NVCLIS=8
  ENDIF
  NUSE_ECCI=1
ELSE
  IF (NARGS /= 0.OR.LFPART2) THEN
    NDGLG=NUDGL
    NDLON=NUDLON
    NFLEVG = NUFLEV
    NSMAX = NUSMAX
    NTMAX = NUTMAX
    NCMAX = NUCMAX
    IF (ANY(SPREAD(NCONF,1,5) == (/1,302,601,801,940/))) THEN
      IF (ABS(USTRET-1.0_JPRB) <= ZEPS) THEN
        NCMAX = NUSMAX
        NUCMAX = NUSMAX
        NTMAX = NUSMAX
        NUTMAX = NUSMAX
      ENDIF
    ELSEIF (NCONF == 701) THEN
      NCMAX = NUSMAX
      NUCMAX = NUSMAX
    ELSEIF ((NCONF == 911).OR.(NCONF == 912)) THEN
      ! * compute RSTRET:
      IF (NCONF == 911) THEN
        RSTRET=USTRET
      ENDIF
      ! * compute NSMIN:
      NSMIN = NUSMAX
      ! * compute NSMAX:
      IF (NCONF == 911) THEN
        ! * Call SUNCMAX to compute NUCMAX (NUCMAX is unknown in this case!!).
        CALL SUNCMAX(NSMIN,USTRET,ZSEUIL,NUCMAX)
        NSMAX=NUCMAX
      ELSEIF ((NCONF == 912).AND.(USTRET == 1.0_JPRB)) THEN
        ! * No dilatation matrix has been made in this case, and NSMAX=NSMIN.
        NSMAX=NSMIN
      ELSEIF ((NCONF == 912).AND.(USTRET > 1.0_JPRB)) THEN
        ! * A dilatation matrix file has been previously made, which provides
        !   NUCMAX.
        NSMAX=NUCMAX
      ENDIF
        ! * compute NDGLG; for a given couple (NSMIN,NSMAX), the same value of
        !   NDGLG is used for dilatation matrices and rotation matrices.
        !   Save NDGLG in IUNDGLG.
      IF (USTRET > 1.0_JPRB) THEN
        ZNCS=3._JPRB
      ELSE
        ZNCS=1._JPRB
      ENDIF
      ZDLON=3._JPRB*REAL(NSMAX,JPRB)+ZNCS
      ZDGL=0.5_JPRB*ZDLON
      NDGLG=NINT(ZDGL)
      NDGLG=NDGLG+MOD(NDGLG,2)
      IUNDGLG=NDGLG
      ! * compute NFLEVG and NUFLEV.
      NFLEVG = 1
      NUFLEV = 1
    ENDIF
    IF (LELAM) THEN
      NMSMAX=NUSTTYP
      NVARMAX=NMSMAX
      NDLUNG=NULIM(3)
      NDLUXG=NULIM(4)
      NDGUNG=NULIM(5)
      NDGUXG=NULIM(6)
    ENDIF
  ENDIF
  IF(ANY(SPREAD(NCONF,1,8) == (/1,131,302,401,501,601,801,940/))) THEN
    IF (NEINI == 1.OR.NEINI == 3.OR.NEINI == 4) THEN
      IF (NSMAX > 21) THEN
        NXMAX=30
      ELSE
        NXMAX=18
      ENDIF
    ENDIF
  ELSEIF (NCONF == 131) THEN
    IF (NEINI == 3.OR.NEINI == 4) THEN
      IF (NSMAX > 21) THEN
        NXMAX=30
      ELSE
        NXMAX=18
      ENDIF
    ENDIF
  ELSEIF (NCONF == 701) THEN
    NXMAX=1
    NVXP2=4
    IF (LSOLV) THEN
      NVXP2=NVXP2+3
    ENDIF
  ELSEIF (NCONF == 911.OR.NCONF == 912) THEN
    NFLEVG = 1
    NXMAX=1
  ENDIF
ENDIF

IF (.NOT. LELAM) THEN
  NMSMAX =NSMAX
  NVARMAX=NSMAX
ENDIF

!*       1.2 READ NAMELISTS THEN OVERWRITE WITH COMMAND LINE ARGUMENT

! k.y.: All new checkings about GFL attributes SHOULD now be coded in
!       SUCTRL_GFLATTR.

!        1.2.1  Read NAMDIM and NAMGFL

CALL POSNAM(NULNAM,'NAMDIM')
READ(NULNAM,NAMDIM)

IF (NPROMA==0) NPROMA=IPROMA  ! restore default nproma

CALL POSNAM(NULNAM,'NAMGFL')
READ(NULNAM,NAMGFL)

! WE NEED THE ACTUAL NFLEVG AT THIS POINT WHEN GRID POINT NUDGING
NTOTFNUDG=0
IF(LNUDG) THEN
  IF(LNUDST)NTOTFNUDG=NTOTFNUDG+1
  IF(LNUDSM)NTOTFNUDG=NTOTFNUDG+1
  IF(LNUDRM)NTOTFNUDG=NTOTFNUDG+1
  IF(LNUDSD)NTOTFNUDG=NTOTFNUDG+1
  IF(LNUDTG)NTOTFNUDG=NTOTFNUDG+NFLEVG
  IF(LNUDQG)NTOTFNUDG=NTOTFNUDG+NFLEVG
  IF(LNUDUG)NTOTFNUDG=NTOTFNUDG+NFLEVG
  IF(LNUDVG)NTOTFNUDG=NTOTFNUDG+NFLEVG
  IF(LNUDPG)NTOTFNUDG=NTOTFNUDG+1
  NTOTFNUDG=NTOTFNUDG*NFNUDG
ENDIF

!        1.2.2  Some consistency checks for GFL attributes.

CALL SUCTRL_GFLATTR

!        1.2.3  Some consistency checks and resettings for dimensioning variables.

IF (.NOT.LFPART2) THEN
  IPROM2=NPROMA
ELSE
  NPROMA=IPROM2
ENDIF

! Determine whether NPROMA is negative and set LOPTPROMA

LOPTPROMA=(NPROMA > 0) .AND. (.NOT.LREPRO4DVAR)
NPROMA=IABS(NPROMA)

! NMTCMAX not yet defined, put provisionaly value NTCMAX
NMTCMAX=NTCMAX

!        1.2.4  Read NAMGEM, NAMDPHY and NAMMCC.

IF ((NCONF == 911) .AND. (NARGS == 0)) THEN
  ! * one needs RSTRET to compute a correct default value for (NSMAX,NDGLG)
  !   once known NSMIN, if command line is not activated (default=1).
  RSTRET=1._JPRB
  CALL POSNAM(NULNAM,'NAMGEM')
  READ(NULNAM,NAMGEM)
ENDIF

CALL POSNAM(NULNAM,'NAMDPHY')
READ(NULNAM,NAMDPHY)

IF(NUNDEFLD==1)THEN
  IF(NVEXTRDYN==0) NVEXTRDYN=1
  IF(NVEXTR   ==0) NVEXTR   =2
ENDIF

CALL POSNAM(NULNAM,'NAMMCC')
READ(NULNAM,NAMMCC)

!        1.2.5  Some consistency checks and resettings for command line
!               variables.

IF ((NARGS /= 0.OR.LFPART2).AND..NOT.((NCONF == 911).OR.(NCONF == 912))) THEN
  IF (NSMAX /= NUSMAX) THEN
    NSMAX=NUSMAX
    WRITE(NULOUT,*) 'NSMAX   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NDGLG /= NUDGL) THEN
    NDGLG=NUDGL
    WRITE(NULOUT,*) 'NDGLG  OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NDLON /= NUDLON) THEN
    NDLON=NUDLON
    WRITE(NULOUT,*) 'NDLON   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NFLEVG /= NUFLEV) THEN
    NFLEVG=NUFLEV
    WRITE(NULOUT,*) 'NFLEVG   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NCMAX /= NUCMAX) THEN
    NCMAX=NUCMAX
    WRITE(NULOUT,*) 'NCMAX   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NTMAX /= NUTMAX) THEN
    NTMAX=NUTMAX
    WRITE(NULOUT,*) 'NTMAX   OVERWRITTEN BY COMMAND LINE'
  ENDIF

  ! * ALADIN SETTINGS
  IF (LELAM) THEN
    IF (NMSMAX /= NUSTTYP) THEN
      NMSMAX=NUSTTYP
      WRITE(NULOUT,*) 'NMSMAX   OVERWRITTEN BY COMMAND LINE'
    ENDIF
    IF (NDLUNG /= NULIM(3)) THEN
      NDLUNG=NULIM(3)
      WRITE(NULOUT,*) 'NDLUNG  OVERWRITTEN BY COMMAND LINE'
    ENDIF
    IF (NDLUXG /= NULIM(4)) THEN
      NDLUXG=NULIM(4)
      WRITE(NULOUT,*) 'NDLUXG  OVERWRITTEN BY COMMAND LINE'
    ENDIF
    IF (NDGUNG /= NULIM(5)) THEN
      NDGUNG=NULIM(5)
      WRITE(NULOUT,*) 'NDGUNG  OVERWRITTEN BY COMMAND LINE'
    ENDIF
    IF (NDGUXG /= NULIM(6)) THEN
      NDGUXG=NULIM(6)
      WRITE(NULOUT,*) 'NDGUXG  OVERWRITTEN BY COMMAND LINE'
    ENDIF
  ENDIF
ELSEIF ((NARGS /= 0).AND.(NCONF == 911)) THEN
  IF (RSTRET /= USTRET) THEN
    RSTRET=USTRET
    WRITE(NULOUT,*) 'RSTRET   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NSMIN /= NUSMAX) THEN
    NSMIN=NUSMAX
    WRITE(NULOUT,*) 'NSMIN   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NSMAX /= NUCMAX) THEN
    NSMAX=NUCMAX
    WRITE(NULOUT,*) 'NSMAX   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NDGLG /= IUNDGLG) THEN
    NDGLG=IUNDGLG
    WRITE(NULOUT,*) 'NDGLG  OVERWRITTEN BY COMMAND LINE'
  ENDIF
ELSEIF ((NARGS /= 0).AND.(NCONF == 912)) THEN
  IF (NSMIN /= NUSMAX) THEN
    NSMIN=NUSMAX
    WRITE(NULOUT,*) 'NSMIN   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NSMAX /= NUCMAX) THEN
    NSMAX=NUCMAX
    WRITE(NULOUT,*) 'NSMAX   OVERWRITTEN BY COMMAND LINE'
  ENDIF
  IF (NDGLG /= IUNDGLG) THEN
    NDGLG=IUNDGLG
    WRITE(NULOUT,*) 'NDGLG  OVERWRITTEN BY COMMAND LINE'
  ENDIF
ENDIF

!*       1.3 MISCELLANEOUS CHECKINGS AND RESETTINGS.

!        1.3.1  Check and reset dimensions for configurations 911 and 912.

IF ((NARGS == 0) .AND. (NCONF == 911)) THEN
  ! * For configuration 911, if the command line is not activated,
  !   RSTRET and NSMIN are compulsory (if not specified the above
  !   defined default value is taken).
  !   For NSMAX and NDGLG:
  !   - if a negative value is specified in the namelist for NSMAX, but not
  !     for NDGLG, the code recomputes a "good" value of NSMAX but does not
  !     modify NDGLG.
  !   - if a negative value is specified in the namelist for NDGLG, but not
  !     for NSMAX, the code recomputes a "good" value of NDGLG according to 
  !     NSMAX, but does not modify NSMAX.
  !   - if a negative value is specified in the namelist for NSMAX and
  !     NDGLG, the code recomputes a "good" value of NSMAX, then a value of
  !     NDGLG consistent with NSMAX.
  !   - in the other cases, the (positive) default value (computed above)
  !     or the (>=0) namelist value (if present) is taken into account,
  !     even if the value of NDGLG is not consistent with the value of NSMAX;
  !     if NSMAX < NSMIN, then abort.
  IF ((NSMAX >= 0) .AND. (NSMIN > NSMAX)) THEN
    WRITE(NULERR,*) ' SUDIM1: NSMIN HAS TO BE < OR = NSMAX for conf 911'
    CALL ABOR1(' SUDIM1: NSMIN HAS TO BE < OR = NSMAX for conf 911')
  ENDIF
  IF (NSMAX < 0) THEN 
    CALL SUNCMAX(NSMIN,RSTRET,ZSEUIL,NSMAX)
  ENDIF 
  IF (NDGLG < 0) THEN
    IF (RSTRET > 1.0_JPRB) THEN
      ZNCS=3._JPRB
    ELSE
      ZNCS=1._JPRB
    ENDIF
    ZDLON=3._JPRB*REAL(NSMAX,JPRB)+ZNCS
    ZDGL=0.5_JPRB*ZDLON
    NDGLG=NINT(ZDGL)
    NDGLG=NDGLG+MOD(NDGLG,2)
  ENDIF
ENDIF

IF ((NARGS == 0) .AND. (NCONF == 912)) THEN
  ! * For configuration 912, if the command line is not activated:
  !   - if NSMIN <= NSMAX nothing else to do.
  !     if NSMIN > NSMAX, then abort.
  !   - no negative value specified in namelist for NDGLG:
  !     nothing else to do.
  !   - a negative value is given in namelist for NDGLG.
  !     NDGLG is recomputed consistently with NSMIN and NSMAX.
  IF (NSMIN > NSMAX) THEN
    WRITE(NULERR,*) ' SUDIM1: NSMIN HAS TO BE < OR = NSMAX for conf 912'
    CALL ABOR1(' SUDIM1: NSMIN HAS TO BE < OR = NSMAX for conf 912')
  ENDIF
  IF (NDGLG < 0) THEN
    IF (NSMIN < NSMAX) THEN
      ZNCS=3._JPRB
    ELSE
      ZNCS=1._JPRB
    ENDIF
    ZDLON=3._JPRB*REAL(NSMAX,JPRB)+ZNCS
    ZDGL=0.5_JPRB*ZDLON
    NDGLG=NINT(ZDGL)
    NDGLG=NDGLG+MOD(NDGLG,2)
  ENDIF
ENDIF

!        1.3.2  Model error setup, and reading of namelist NAMMODERR.

! Model error setup
NTYPE_MODERR=0
NDIM_MODERR=0
NSERR=0
LBGMODERR=.FALSE.
LFGMODERR=.FALSE.
ALPHAQ=0.0_JPRB
ALPHAQ_T=1.0_JPRB
ALPHAQ_D=1.0_JPRB
ALPHAQ_V=1.0_JPRB
ALPHAQ_Q=0.0_JPRB
ALPHAQ_O=0.0_JPRB
WEAK4D_INTERV=0.0_JPRB
LQEQB=.FALSE.
LJQBAL=.false.
LJQVCOR=.TRUE.
LJQHCOR=.TRUE.
LJQZERO=.FALSE.
TSTEP_ERR=3600.0_JPRB
NPRTMODERR=0
NORDER_MODERR=0
NLEVERR1=NFLEVG+1
NLEVERR0=NFLEVG+1

IF (LMODERR) THEN
  NTYPE_MODERR=2
  NSERR=NSMAX
  ALPHAQ=0.3_JPRB
  NPRTMODERR=1
  NPERIOD_4DVAR=12
  L_WARM_START_MODERR=.TRUE.

! Model error forcing only in stratosphere
  IF (NFLEVG==91) THEN
    NLEVERR1=18  ! 10 hPa
    NLEVERR0=28  ! 40 hPa
  ELSEIF (NFLEVG==60) THEN
    NLEVERR1=11  !  5 hPa
    NLEVERR0=17  ! 20 hPa
  ENDIF

  CALL POSNAM(NULNAM,'NAMMODERR')
  READ(NULNAM,NAMMODERR)

  IF (NTYPE_MODERR/=2) THEN
    NLEVERR1=NFLEVG+1
    NLEVERR0=NFLEVG+1
  ENDIF

  IF (NSERR<0) NSERR=NSMAX

  WRITE(NULOUT,*)'NTYPE_MODERR=',NTYPE_MODERR
  WRITE(NULOUT,*)'WEAK4D_INTERV=',WEAK4D_INTERV
  WRITE(NULOUT,*)'LFGMODERR=',LFGMODERR
  WRITE(NULOUT,*)'LBGMODERR=',LBGMODERR
  WRITE(NULOUT,*)'NSERR=',NSERR
  WRITE(NULOUT,*)'ALPHAQ=',ALPHAQ
  WRITE(NULOUT,*)'ALPHAQ_T,ALPHAQ_D,ALPHAQ_V,ALPHAQ_Q,ALPHAQ_O=', &
                & ALPHAQ_T,ALPHAQ_D,ALPHAQ_V,ALPHAQ_Q,ALPHAQ_O
  WRITE(NULOUT,*)'TSTEP_ERR=',TSTEP_ERR
  WRITE(NULOUT,*)'LQEQB=',LQEQB
  WRITE(NULOUT,*)'LJQBAL=',LJQBAL
  WRITE(NULOUT,*)'LJQVCOR,LJQHCOR=',LJQVCOR,LJQHCOR
  WRITE(NULOUT,*)'NORDER_MODERR=',NORDER_MODERR
  WRITE(NULOUT,*)'NPRTMODERR=',NPRTMODERR
  WRITE(NULOUT,*)'L_WARM_START_MODERR=',L_WARM_START_MODERR
  IF (NLEVERR0<=NFLEVG.OR.NLEVERR1<=NFLEVG) &
    & WRITE(NULOUT,*)'NLEVERR0,NLEVERR1=',NLEVERR0,NLEVERR1

  IF (NTYPE_MODERR<1) CALL ABOR1('SUDIM1: NTYPE_MODERR<1 no model error.')
  IF (NTYPE_MODERR>5) CALL ABOR1('SUDIM1: NTYPE_MODERR invalid.')
  IF (ALPHAQ<=0.0_JPRB) CALL ABOR1('SUDIM1: ALPHAQ negative.')
  IF (NSERR>NSMAX) CALL ABOR1('SUDIM1: NSERR>NSMAX')
  IF (TSTEP_ERR<0.0_JPRB) CALL ABOR1('SUDIM1: TSTEP_ERR<0.')
  IF (WEAK4D_INTERV<0.0_JPRB) CALL ABOR1('SUDIM1: WEAK4D_INTERV<0.')
ENDIF

!        1.3.3  Miscellaneous checkings and resettings.

IF (LELAM) THEN
  INS_ZON=NMSMAX
ELSE
  INS_ZON=NSMAX
ENDIF
IF (INS_ZON == 0) NTCMAX=0

WRITE(UNIT=NULOUT,FMT='('' SUDIM1 PRINTOUTS'',//)')
IF(ANY(SPREAD(NCONF,1,11) == (/1,131,302,401,&
   & 501,601,701,801,901,903,940/))) THEN  
  NFC2D =1
  LVOR=.TRUE.
  IF(LLNMI) THEN
    IF (LRPIMP) THEN
      NXMAX = NLEX+5
    ELSE
      NXMAX=NSMAX
      NLEX=NXMAX
    ENDIF
  ENDIF
  IF(LSLAG) THEN
    IF (LVERCOR.OR.LNHDYN.OR.LNMIRQ.OR.(LSLHD.AND..NOT.LSLHD_STATIC) &
      & .OR.(NCONF == 131.AND.LJC)) THEN
      LUVDER=.TRUE.
    ELSE
      LUVDER=.FALSE.
    ENDIF
  ELSE
    LUVDER=.TRUE.
  ENDIF
  IF (.NOT.LSLAG) NRLEVX=1
  IF(NCONF == 701)THEN
    LADER=.FALSE.
  ELSE
    LADER=.TRUE.
  ENDIF
  IF ((NCONF == 131.AND.NSTOP == 0).AND..NOT.LLNMI) THEN
  ELSEIF ((NCONF == 131.AND.NSTOP == 0).AND.LLNMI) THEN
  ELSEIF (NCONF == 701) THEN
    IF (LSOLV) THEN
      NVXP2=7
    ELSE
      NVXP2=4
    ENDIF
  ELSE
  ENDIF
ELSEIF(ANY(SPREAD(NCONF,1,6) == (/201,202,421,422,521,522/))) THEN  
  NCSI=0
  NCSNEC=0
  NTILES=0
  NCOM=0
  NFLEVG=1
  NRLEVX=1
  LSPT = .FALSE.
  NFC2D = 1
  LVOR=.TRUE.
  LADER=.TRUE.
  LUVDER=.NOT.LSLAG
ELSEIF(ANY(SPREAD(NCONF,1,2) == (/911,912/))) THEN
  NFLEVG=1
  NRLEVX=1
  NDLON =1
  NCMAX=NSMAX
  NTMAX=NSMAX
  LSPT = .FALSE.
  NFC2D =0
  NVCLIS=0
  NTSSG=0
  NTOZ1D=0
  NTOZ2D=0
  NTOZ3D=0
  LVOR=.FALSE.
  LADER=.FALSE.
  LUVDER=.FALSE.
ELSEIF (ANY(SPREAD(NCONF,1,3) == (/923,931,932/))) THEN
  LSPT = .FALSE.
  NFC2D =0
  NTSSG=0
  NVCLIS=0
  NTILES=0
  NCOM=0
  LVOR=.FALSE.
  LADER=.FALSE.
  LUVDER=.FALSE.
ELSE
  WRITE(UNIT=NULERR,FMT='('' SUDIM1: CONFIGURATION NOT EXPECTED'')')
  CALL ABOR1('SUDIM1: part 1.3.3 CONFIGURATION NOT EXPECTED')
ENDIF

IF (NCONF == 923) THEN
  NFAUX=1
ENDIF

IF( LSLPHY ) THEN
  IF(LPC_FULL)THEN
    NVTEND=8
  ELSE
    NVTEND=4
  ENDIF
ENDIF

!        1.3.4  Tests of consistency between YOMPHY and YOMDPHY

IF (NTSSG == 1) THEN
  WRITE(UNIT=NULERR,FMT='('' SUDIM1: NTSSG == 1 IS NOT IMPLEMENTED FOR DM'')')
  CALL ABOR1('SUDIM1: NTSSG == 1 IS NOT IMPLEMENTED FOR MESSAGE PASSING')
ENDIF
IF (LOZONE) THEN
  IF (NVCLIS*(NTOZ1D+NTOZ2D+NTOZ3D) < 4) THEN
    WRITE(UNIT=NULERR,FMT='('' SUDIM1: LOZONE INCONSISTENT WITH YOMDPHY'')')
    CALL ABOR1('SUDIM1: LOZONE INCONSISTENT WITH YOMDPHY ')
  ENDIF
ENDIF
IF (LMPHYS.AND.(LNEBN.AND.(LNEBCO.OR.LGWDC))) THEN
  WRITE(NULERR,&
   & FMT='('' LNEBN INCONSISTENT WITH OTHER CLOUDINESS SCHEMES '')')  
  WRITE(NULOUT,*) 'LNEBN=',LNEBN,' LNEBCO='&
   & ,LNEBCO,' LGWDC=',LGWDC  
  CALL ABOR1('SUDIM1: pb LNEBN')
ENDIF
IF(NVEXTR < NVEXTRDYN) THEN
  WRITE(NULERR,*) ' SUDIM1: NVEXTR=',NVEXTR,' NVEXTRDYN=',NVEXTRDYN
  CALL ABOR1('SUDIM1: NVEXTR can not be < NVEXTRDYN')
ENDIF
IF (NTSSG > 0.AND..NOT.LMCC01)THEN
  WRITE(UNIT=NULERR,FMT='('' SUDIM1: NTSSG MUST BE 0 IF UPDCLISM NOT CALLED'')')
  CALL ABOR1('SUDIM1: NTSSG MUST BE 0 IF UPDCLISM IS NOT CALLED')
ENDIF

!        1.3.5  Tests of consistency in settings for ozone (ECMWF)

IF (LEO3CH.AND..NOT.YO3_NL%LGP) THEN
  WRITE(UNIT=NULERR,FMT='('' SUDIM1: CHEMISTRY CALLED BUT O3 NOT ADVECTED'')')
  CALL ABOR1('SUDIM1: CHEMISTRY CALLED BUT O3 NOT ADVECTED')
ENDIF
IF (LEO3CH) THEN
  IF (NVCLIS*(NTOZ1D+NTOZ2D+NTOZ3D) < 8) THEN
    WRITE(NULERR,FMT='('' LEO3CH=.T. BUT NTOZ2D=0 AND NTOZ3D=0 '',&
     & '' OR NVCLIS /= 8! '')')  
    CALL ABOR1('SUDIM1: pb LEO3CH')
  ENDIF
ENDIF

CALL GSTATS(1934,0)

!*       1.5  Print physic's dimensioning

WRITE(NULOUT,*) '      PHYSICS DIMENSIONING IN SUDIM1 '
WRITE(NULOUT,*) ' NVXP = ',NVXP,' NVXP2 = ',NVXP2
WRITE(NULOUT,*) ' NCXP = ',NCXP,' NTILES = ',NTILES  
WRITE(NULOUT,*) ' NTVG = ',NTVG
WRITE(NULOUT,*) ' NLOA = ',NLOA,' NLOE = ',NLOE
WRITE(NULOUT,*)&
 & ' NVEXTR = ',NVEXTR,' NVXTR2 = ',NVXTR2,&
 & ' NVTEND =',NVTEND,' NVEXTRDYN = ',NVEXTRDYN  
WRITE(NULOUT,*) ' NCEXTR= ',NCEXTR
WRITE(NULOUT,*)' NVCLIS = ',NVCLIS,' NTSSG = ',NTSSG, &
 & ' NTOZ1D = ',NTOZ1D,' NTOZ2D = ',NTOZ2D,' NTOZ3D = ',NTOZ3D, &
 & ' LTPROF = ',LTPROF,' NTOTFNUDG = ',NTOTFNUDG
WRITE(NULOUT,*) ' LDIRCLSMOD = ',LDIRCLSMOD

!*       1.6  Verify values of NCMAX, NTMAX and NXMAX according to YOMNM

! NTMAX will be checked in SUGEM1 too.

NCMAX = MAX(NCMAX,NSMAX)
IF (LELAM) NCMAX = MAX(NCMAX,NMSMAX)
NTMAX = MAX(NTMAX,NSMAX)

IF (LLNMI) THEN

  IF (NVMOD > NFLEVG) THEN
    WRITE(NULERR,*)' SUDIM1: NVMOD=',NVMOD,' NFLEVG=',NFLEVG
    CALL ABOR1(' SUDIM1: NVMOD CANNOT BE GREATER THAN NFLEVG')
  ENDIF

  IF (NXMAX < 1.OR. NCMAX < 1) THEN
    WRITE(NULERR,*)' SUDIM1: NXMAX AND NCMAX CANNOT BE LOWER THAN 1'
    CALL ABOR1(' SUDIM1: NXMAX AND NCMAX CANNOT BE LOWER THAN 1')
  ENDIF

  IF (.NOT.LRPIMP) THEN
    IF (NLEX /= NXMAX .OR. NXMAX /= NCMAX) THEN
      WRITE(NULERR,*)' SUDIM1: NLEX=NXMAX=NCMAX IF PURE EXPLICIT SCHEME'
      CALL ABOR1(' SUDIM1: NLEX=NXMAX=NCMAX IF PURE EXPLICIT SCHEME')  
    ENDIF
  ELSEIF (NLEX > 0) THEN
    IF (NXMAX > NCMAX) THEN
      WRITE(NULERR,*)' SUDIM1: NXMAX CANNOT BE GREATER THAN NCMAX'
      CALL ABOR1(' SUDIM1: NXMAX CANNOT BE GREATER THAN NCMAX')
    ENDIF
    IF (NLEX >= NXMAX-2) THEN
      WRITE(NULERR,*)' SUDIM1: NXMAX MUST BE GREATER THAN NLEX+2'
      CALL ABOR1(' SUDIM1: NXMAX MUST BE GREATER THAN NLEX+2')
    ENDIF
  ENDIF

ENDIF

!*       1.7  Check value of NTCMAX

! * The allowed values for NTCMAX are:
!   - all values NTCMAX <= NSMAX in ARPEGE/IFS.
!   - all values NTCMAX <= NMSMAX in ALADIN.
!   - NTCMAX=NDLON/2 if INS_ZON>0: for informatic validation only,
!     to check that the grid-point fields produced by LRETCFOU=T are identical
!     to the fields written by LWRTCFOU=T in this case.

IF (.NOT.((NTCMAX <= INS_ZON).OR.(NTCMAX == NDLON/2))) THEN
  WRITE (NULERR,*) ' SUDIM1: NTCMAX must be either <= NSMAX '&
   & ,' (or NMSMAX in ALADIN) or equal to NDLON/2;'&
   & ,' other values are not allowed.'  
  CALL ABOR1(' SUDIM1: pb NTCMAX ')
ENDIF

!*       1.8  CHECK THAT PHYSICS DIMENSIONING CONSISTANT WITH PARDIM

IF(NVXP > JPVXP) THEN
  WRITE(NULERR,'(''NVXP MUST BE SMALLER OR EQ. TO JPVXP'')')
  CALL ABOR1(' SUDIM1: PROBLEM WITH NVXP ')
ENDIF
IF(NCXP > JPCXP) THEN
  WRITE(NULERR,'(''NCXP MUST BE SMALLER OR EQ. TO JPCXP'')')
  CALL ABOR1(' SUDIM1: PROBLEM WITH NCXP ')
ENDIF

!*       1.9  COMPUTE THE TOTAL NUMBER OF "GEMS" FIELDS.

NGEMS=NGHG+NGRG+NAERO+NTRAC

!*       1.10 FILL NGRBGRG, NGRBGRGSFC and NGRBTCGRGF from Namelist input
!             one should get rid of NGRBGRG, NGRBGRGSFC and NGRBTCGRGF
!             in the future and always refer to YGRG 
NGRBGRG = -999
NGRBGRGSFC = -999
NGRBTCGRG = -999

IF(NGRG > 0 ) THEN
  DO JGFL = 1, NGRG
    NGRBGRG(JGFL) = YGRG_NL(JGFL)%IGRBCODE
    NGRBGRGSFC(JGFL) = YGRG_NL(JGFL)%IGRIBSFC
    NGRBTCGRG(JGFL) = YGRG_NL(JGFL)%IGRIBTC
    IF (YGRG_NL(JGFL)%NCOUPLO4 > 0 ) THEN 
      NGRG_CPLO4=NGRG_CPLO4 + 1 
      YGRGTEND_NL(NGRG_CPLO4)%CNAME=TRIM(YGRG_NL(JGFL)%CNAME)//'_tend'
    ENDIF
    IF (YGRG_NL(JGFL)%LASSIM) NGRG_ASSIM=NGRG_ASSIM + 1 
  ENDDO 
ENDIF

NGRBTRAC = -999
NGRBTRACSFC = -999
NGRBTCTRAC = -999

IF(NTRAC > 0 ) THEN
  DO JGFL = 1, NTRAC
    NGRBTRAC(JGFL) = YTRAC_NL(JGFL)%IGRBCODE
    NGRBTRACSFC(JGFL) = YTRAC_NL(JGFL)%IGRIBSFC
    NGRBTCTRAC(JGFL) = YTRAC_NL(JGFL)%IGRIBTC
  ENDDO 
ENDIF

CALL GSTATS(1934,1)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDIM1',1,ZHOOK_HANDLE)
END SUBROUTINE SUDIM1
