#ifdef VPP
!OCL SCALAR
#endif
SUBROUTINE SUDYNA(KULOUT)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0 , ONLY : LRUBC    ,LPC_OLD  ,LSLAG    ,LNHDYN   ,LTWOTL   ,LRPLANE
USE YOMCT0B  , ONLY : LECMWF
USE YOMCVER  , ONLY : LVERTFE
USE YOMLUN , ONLY : NULNAM
USE YOMDYNA, ONLY : LGWADV   ,LRDBBC   ,NPDVAR   ,NVDVAR   ,&
 & LSLHD    ,LSLHD_W  ,LSLHD_T  ,LSLHD_SPD,LSLHD_SVD,LSLHD_OLD,&
 & LSLHD_STATIC       ,SLHDKMIN ,SLHDKMAX ,SLHDEPSH ,SLHDEPSV ,&
 & ND4SYS   ,LNHX     ,LNHXDER  ,LNH_PDVD ,LNH_GEOGW,LGWOPT1  ,&
 & LRWSDLW  ,LRWSDLR  ,LRWSDLR2 ,LRWSDLG  ,NGWADVSI, LSLDIA

!**** *SUDYNA*   - Initialize constants and control for the dynamics: part A

!     Purpose.
!     --------
!           Initialize YOMDYNA, a module that controls the
!           dynamics of the model. The variables initialized here are those
!           which are used in some other routines called under SU0YOMA and
!           the initialization of which cannot wait for the call of SUDYN under
!           SU0YOMB.

!           This routine will later collect the initialization of some
!           dynamical variables currently set-up in SUCT0.
!           The variables of YOMCT0/SUCT0/NAMCT0 which would be good candidates
!           to be moved towards YOMDYNA/SUDYNA/NAMDYNA in the future are:
!            LSLAG,LAPRXPK,LTWOTL,LSITRIC,LSPRT,LRUBC,LREGETA,
!            LVERTFE,LPC_OLD,LPC_NESC,LPC_FULL.
!           The moving of LRFRIC,LNHDYN,LVERCOR is also questionable. 

!**   Interface.
!     ----------
!        *CALL* *SUDYNA(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMDYNA

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!     none

!     Called by SU0YOMA.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP)

! Modifications
! -------------
!   Original : September 2002
!   Modified 03-08-20 by F. Vana: option LSLHD
!   Modified August 2003: option LRDBBC.
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   05-Jun-2003 J. Masek   NH cleaning (NPDVAR, NVDVAR, LFULLIMP, LVSLWBC,
!                          LPC_NOTR)
!   15-Feb-2005 F. Vana : Split of the global LSLHD key.
!   20-Feb-2005 J. Vivoda  3TL Eul PC scheme, GWADV scheme for PC_FULL
!   14-Feb-2005 K. Yessad  Update the aborts.
!   N. Wedi and K. Yessad Nov-2007: ND4SYS=2 to stabilize NH model
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   30-Jun-2008 J. Masek   New SLHD interpolators.
! End Modifications
!------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)    :: KULOUT

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IERR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sudyncore.intfb.h"
#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "namdyna.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDYNA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    Set default values.
!              -------------------

LGWADV=.FALSE.
NGWADVSI=1
LGWOPT1=.FALSE.
NPDVAR=2
IF (LNHDYN) THEN
  NVDVAR=4
  LRDBBC=LSLAG
ELSE
  NVDVAR=3
  LRDBBC=.FALSE.
ENDIF
LSLHD_W   =.FALSE.
LSLHD_T   =.FALSE.
LSLHD_SPD =.FALSE.
LSLHD_SVD =.FALSE.
LSLHD_OLD =.TRUE.
LSLHD_STATIC=.FALSE.
SLHDKMIN=0.0_JPRB
SLHDKMAX=1.0_JPRB
SLHDEPSH=0.0_JPRB
SLHDEPSV=0.0_JPRB
! LSLHD defaults for GFLs are set in SUDIM1, before reading NAMGFL!!!

IF (LECMWF) THEN
  IF (LSLAG.AND.LNHDYN.AND.(NVDVAR == 4)) THEN
    ND4SYS=2
  ELSE
    ND4SYS=1
  ENDIF
ELSE
  ND4SYS=1
ENDIF

LRWSDLW=.FALSE.
LRWSDLR=.FALSE.
LRWSDLR2=.FALSE.
LRWSDLG=.FALSE.

LSLDIA=.FALSE.

!     ------------------------------------------------------------------

!*       2.    Modify default values.
!              ----------------------

!        2.1   Read namelist.

CALL POSNAM(NULNAM,'NAMDYNA')
READ(NULNAM,NAMDYNA)

!*       2.2   Reset variables and test.
!              If relevant, is the NH model a pressure departure + vertical
!              divergence one or a Phi-gw one?

IERR=0
IF (LNHDYN) THEN
  IF ((NPDVAR == 3 .OR. NPDVAR == 4).AND. (NVDVAR == 5 .OR. NVDVAR == 6)) THEN
    ! this is a Phi-gw one.
    LNH_GEOGW=.TRUE.
    LNH_PDVD=.FALSE.
    ! and in this case, (LGWADV,NGWADVSI,LRDBBC,ND4SYS) is always (F,1,F,1).
    LGWADV=.FALSE.
    NGWADVSI=1
    LRDBBC=.FALSE.
    ND4SYS=1
  ELSEIF (NPDVAR == 2 .AND. (NVDVAR==3 .OR. NVDVAR==4)) THEN
    ! this is a pressure departure and vertical divergence one.
    LNH_GEOGW=.FALSE.
    LNH_PDVD=.TRUE.
    LGWOPT1=.FALSE.
  ELSE
    ! such configuration does not exist.
    IERR=IERR+1
    WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
    WRITE(KULOUT,*) ' Not existing configuration for (NPDVAR,NVDVAR)!'
  ENDIF
ELSE
  ! hydrostatic model.
  ! and in this case,
  !  * (LNH_PDVD,LNH_GEOGW) is always (F,F).
  !  * (LGWADV,NGWADVSI,LRDBBC,ND4SYS) is always (F,1,F,1).
  !  * (NPDVAR,NVDVAR) is always (2,3) for safety.
  !  * (LSLHD_SPD,LSLHD_SVD) is always (F,F).
  LNH_PDVD=.FALSE.
  LNH_GEOGW=.FALSE.
  LGWADV=.FALSE.
  NGWADVSI=1
  LRDBBC=.FALSE.
  NPDVAR=2
  NVDVAR=3
  LSLHD_SPD = .FALSE.
  LSLHD_SVD = .FALSE.
  ND4SYS=1
  LGWOPT1=.FALSE.
ENDIF

!*       2.3   Provisional set up of model internal key LSLHD

! LSLHD value can be further modified in SUCTRL_GFLATTR, depending on %LSLHD
! attribute of advected GFL fields.

LSLHD=LSLHD_W .OR. LSLHD_T .OR. LSLHD_SPD .OR. LSLHD_SVD

!*       2.4   Adjusting of SLHDKMIN

IF ( LSLHD_OLD.AND.(SLHDKMIN /= 0.0_JPRB) ) THEN
  SLHDKMIN=0.0_JPRB
  WRITE(KULOUT,*) 'SUDYNA: Warning!'
  WRITE(KULOUT,*) '  SLHDKMIN reset to zero for LSLHD_OLD.'
ENDIF

! WARNING: Using non-zero SLHDKMIN redefines weights for high order
! semi-Lagrangian interpolators (subroutines LAIDDI, LAITRI) for
! all advected fields, even if they are not subject to SLHD scheme.
! Computation of modified interpolation weights uses same auxiliary
! quantities as the new SLHD interpolators. Their initialization
! is controlled by internal key LSLHDQUAD.

!*       2.5   Additional settings

LNHX=LNHDYN.AND.(NVDVAR==4).AND.(.NOT.(LGWADV.AND.ND4SYS==1))
! ky: ideally LNHXDER should be T only if the derivatives of NHX are used,
!     but having a different value for LNHX and LNHXDER seems to be problematic
!     for spectral transforms, so for the time being LNHXDER is set = LNHX.
!!neu LNHXDER=LNHDYN.AND.(NVDVAR==4).AND.(.NOT.(LGWADV))
LNHXDER=LNHX

!*       2.6   Some tests

!* --- Tests on LPC_OLD ---

IF( (NVDVAR==4).AND.LPC_OLD ) THEN
  ! * LPC_OLD used with NVDVAR=4?
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LPC_OLD does not make sense with d4 (NVDVAR=4).'
ENDIF

IF( LNH_GEOGW.AND.LPC_OLD ) THEN
  ! * LPC_OLD used with a Phi-gw NH model?
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LPC_OLD does not make sense with a Phi-gw NH model.'
ENDIF

!* --- Tests on LGWADV ---

IF (LGWADV.AND.(.NOT.LSLAG)) THEN
  ! * Use Eulerian scheme when LGWADV=true?
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LGWADV=true is not coded in the Eulerian dynamics.'
ENDIF
IF (LGWADV.AND.LSLAG.AND.(.NOT.LTWOTL)) THEN
  ! * Use SL3TL scheme when LGWADV=true?
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LGWADV=true is not coded in the SL3TL dynamics.'
  ! ky: in this case additional interpolations are required
  !     for the linear terms of "dver" equation, and they are
  !     not coded.
ENDIF
! IF (LGWADV.AND.LSLAG.AND.LTWOTL.AND.(.NOT.LPC_NESC)) THEN
!   ! * Use SL2TL scheme with LPC_NESC=F when LGWADV=true?
!   IERR=IERR+1
!   WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
!   WRITE(KULOUT,*) ' LGWADV=true is not coded for LPC_NESC=F in the SL2TL dyn.'
!   ! ky: in this case additional interpolations are required
!   !     for the linear terms of "dver" equation, and they are
!   !     not coded.
! ENDIF

IF(LGWADV .AND. LRUBC) THEN
  ! * these two options actually can run together on an informatic
  !   point of view, but the current assumptions done with LGWADV=T
  !   are that a particle which is on the top of the atmosphere remains
  !   on the top of the atmosphere;
  !   this is equivalent to assume that "etadot_top" is always zero.
  !   This condition is satisfied when LRUBC=F
  !   but not when LRUBC=T which gives a non-zero value to "etadot_top".
  !   So one forbids combination "LGWADV .AND. LRUBC" which leads
  !   to inconsistencies in the model at the top of the atmosphere.
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LRUBC=true and LGWADV=true are inconsistent.'
ENDIF

!* --- Tests on NGWADVSI ---
IF (NGWADVSI<1 .OR. NGWADVSI>2) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' NGWADVSI should be 1 or 2.'
ENDIF

!* --- Tests on LRDBBC ---
IF ( LRDBBC.AND.LGWADV ) THEN
  ! * Use of lrdbbc=true with lgwadv=true makes no sense!'
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' Options LGWADV=T and LRDBBC=T make no sense together!'
ENDIF
IF (LRDBBC.AND.(.NOT.LSLAG)) THEN
  ! * Use Eulerian scheme when LRDBBC=true?
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LRDBBC=true is the S.-L. discretization of BBC'
ENDIF

!* --- Tests on LSLHD ---

IF ((SLHDKMIN < -2.0_JPRB ).OR.(SLHDKMIN > 12.0_JPRB)) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' SLHDKMIN is restricted to interval <-2,12>.'
ENDIF

IF (LSLHD_OLD) THEN
  IF ((SLHDKMAX < 0.0_JPRB ).OR.(SLHDKMAX > 5.0_JPRB)) THEN
    IERR=IERR+1
    WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
    WRITE(KULOUT,*) ' SLHDKMAX is restricted to interval <0,5>.'
  ENDIF
ELSE
  IF ((SLHDKMAX < -2.0_JPRB ).OR.(SLHDKMAX > 12.0_JPRB)) THEN
    IERR=IERR+1
    WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
    WRITE(KULOUT,*) ' SLHDKMAX is restricted to interval <-2,12>.'
  ENDIF
ENDIF

!* --- Test on SLHDEPSH, SLHDEPSV ---
IF ((SLHDEPSH < 0.0_JPRB ).OR.(SLHDEPSH > 0.5_JPRB )) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) &
   & ' SLHDEPSH must be within interval <0,0.5>.'
ENDIF

IF ((SLHDEPSV < 0.0_JPRB ).OR.(SLHDEPSV > 0.5_JPRB )) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) &
   & ' SLHDEPSV must be within interval <0,0.5>.'
ENDIF

!* --- Tests on ND4SYS ---

IF (.NOT.(LSLAG.AND.LNHDYN.AND.(NVDVAR == 4)) .AND. (ND4SYS == 2)) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) '(LSLAG.AND.LNHDYN.AND.(NVDVAR == 4))=F: use ND4SYS=1.'
ENDIF

!* --- Tests on LVERTFE ---

IF( LNH_GEOGW.AND.(.NOT.LVERTFE) ) THEN
  ! * FD vertical discretisation used in a Phi-gw NH model?
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' Phi-gw NH model is currently coded only for VFE.'
ENDIF

!* --- Tests on Wood & Staniforth deep-layer equations keys ---

IF( LRWSDLR2.AND.(.NOT.LRWSDLR) ) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' If LRWSDLR2=T, LRWSDLR should be T.'
ENDIF
IF( LRWSDLG.AND.(.NOT.LRWSDLR) ) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' If LRWSDLG=T, LRWSDLR should be T.'
ENDIF
IF( LRWSDLW.AND.LNHDYN.AND..NOT.(LNH_PDVD.AND.LGWADV) ) THEN
  ! Not yet coded case.
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LRWSDLW=T in NH currently requires LGWADV=T.'
ENDIF
IF( LRWSDLR.AND..NOT.LNHDYN ) THEN
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LRWSDLR=T is not coded in a hydrostatic model.'
ENDIF
IF( LRWSDLW.AND.LNHDYN.AND..NOT.(LNH_PDVD.AND.LGWADV) ) THEN
  ! Not yet coded case.
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LRWSDLR=T in NH currently requires LGWADV=T.'
ENDIF

!* --- Tests on LSLDIA ---

IF( LSLDIA.AND.LRPLANE ) THEN
  ! Not yet coded case.
  IERR=IERR+1
  WRITE(KULOUT,'(1X,A,I3,A)') ' ! SUDYNA: ERROR NR ',IERR,' !!!'
  WRITE(KULOUT,*) ' LSLDIA is not coded for LRPLANE=T.'
ENDIF

IF (IERR >= 1) THEN
  CALL FLUSH(KULOUT)
  CALL ABOR1(' SUDYNA: ABOR1 CALLED')
ENDIF

!*      2.7   dynamical core setup 

CALL SUDYNCORE

!     ------------------------------------------------------------------

!*       3.    Print final values.
!              -------------------

WRITE(UNIT=KULOUT,FMT='('' YOMDYNA '')')
WRITE(UNIT=KULOUT,FMT='('' LGWADV = '',L2)') LGWADV
WRITE(UNIT=KULOUT,FMT='('' NGWADVSI = '',I2)') NGWADVSI
WRITE(UNIT=KULOUT,FMT='('' LRDBBC = '',L2)') LRDBBC
WRITE(UNIT=KULOUT,FMT='('' NPDVAR = '',I2)') NPDVAR
WRITE(UNIT=KULOUT,FMT='('' NVDVAR = '',I2)') NVDVAR
WRITE(UNIT=KULOUT,FMT='('' LNH_PDVD = '',L2)') LNH_PDVD
WRITE(UNIT=KULOUT,FMT='('' LNH_GEOGW = '',L2)') LNH_GEOGW
WRITE(UNIT=KULOUT,FMT='('' LSLHD_W       = '',L2)') LSLHD_W
WRITE(UNIT=KULOUT,FMT='('' LSLHD_T       = '',L2)') LSLHD_T
WRITE(UNIT=KULOUT,FMT='('' LSLHD_SPD     = '',L2)') LSLHD_SPD
WRITE(UNIT=KULOUT,FMT='('' LSLHD_SVD     = '',L2)') LSLHD_SVD
WRITE(UNIT=KULOUT,FMT='('' LSLHD_OLD     = '',L2)') LSLHD_OLD
WRITE(UNIT=KULOUT,FMT='('' LSLHD_STATIC  = '',L2)') LSLHD_STATIC
WRITE(UNIT=KULOUT,FMT='('' SLHDKMIN      = '',E10.4)') SLHDKMIN
WRITE(UNIT=KULOUT,FMT='('' SLHDKMAX      = '',E10.4)') SLHDKMAX
WRITE(UNIT=KULOUT,FMT='('' SLHDEPSH      = '',E10.4)') SLHDEPSH
WRITE(UNIT=KULOUT,FMT='('' SLHDEPSV      = '',E10.4)') SLHDEPSV
WRITE(UNIT=KULOUT,FMT='('' ND4SYS = '',I1)') ND4SYS
WRITE(UNIT=KULOUT,FMT='('' LNHX = '',L2)') LNHX
WRITE(UNIT=KULOUT,FMT='('' LNHXDER = '',L2)') LNHXDER
WRITE(UNIT=KULOUT,FMT='('' LGWOPT1 = '',L2)') LGWOPT1
WRITE(UNIT=KULOUT,FMT='('' LRWSDLW = '',L2)') LRWSDLW
WRITE(UNIT=KULOUT,FMT='('' LRWSDLR = '',L2)') LRWSDLR
WRITE(UNIT=KULOUT,FMT='('' LRWSDLR2 = '',L2)') LRWSDLR2
WRITE(UNIT=KULOUT,FMT='('' LRWSDLG = '',L2)') LRWSDLG
WRITE(UNIT=KULOUT,FMT='('' LSLDIA = '',L2)') LSLDIA

!     ------------------------------------------------------------------

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUDYNA',1,ZHOOK_HANDLE)
END SUBROUTINE SUDYNA
