SUBROUTINE SUDYNCORE
!**** *SUDYNCORE*  - Initialize LDYNCORE

!     Purpose.
!     --------
!           Initialize LDYNCORE SETUP, ECMWF

!**   Interface.
!     ----------
!        *CALL* *SUDYNCORE

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Nils Wedi *ECMWF*

!     Modifications.
!     --------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMDYNCORE

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IERR
REAL(KIND=JPRB) :: ZDAY
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"
#include "namdyncore.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDYNCORE',0,ZHOOK_HANDLE)

! master switch
LDYNCORE=.FALSE.

! postprocessing in steps (emulating hours for fdb)
LPPSTEPS=.FALSE.

! vertical level distribution from namelist
LVERTINI=.FALSE.

! initial profiles
RT00_DYN=288.15_JPRB  ! T0
RP00_DYN=101315._JPRB ! P0
RU00_DYN=0._JPRB      ! U0
RST0_DYN=0._JPRB      ! DU0/DZ
NOISEVOR=0            ! 1 == initial noise in vorticity

! small planet factor r = a * RPLRADI , ?? wam ??
! (needs unfortunately manual updating in trans/module/tpm_constants.F90 )
RPLRADI=1._JPRB
! RPLRADI= 30000._JPRB/6371229._JPRB
! Coriolis acceleration as RDAY = 86400. * RCORIOI, ?? wam ??
! (needs unfortunately manual updating in surf/module/suscst_mod.F90 )
RCORIOI=1._JPRB

! 3d dry dynamical core tests
NTESTCASE=-1

! aquaplanet defaults
LAQUA=.FALSE.
! default constant SST everywhere
MSSTSCHEME=9

! heldsuarez defaults
LHELDSUAREZ=.FALSE.
! pole - equator T difference
RDELTA_T=60._JPRB
! tropical heating differential
RDELTA_THETA=10._JPRB

RSIGMAB=0.7_JPRB
REFERENCE_LEVEL=1.0_JPRB-RSIGMAB
RPRESSURE_SCALE=1.e-5_JPRB
RHS_KAPPA=2.0_JPRB/7._JPRB

! additional stratosphere
L_HS_WILLIAMSON=.FALSE.
RTHSW_0=200._JPRB
! RPHSW_PL=200._JPRB
RPHSW_PL=10._JPRB
RPHSW_D = 10000._JPRB

! default thresholds for semi-Lagrangian diagnostics
RJACSLDIA=1.5_JPRB
RLIPSLDIA=5.0_JPRB

CALL POSNAM(NULNAM,'NAMDYNCORE')
READ(NULNAM,NAMDYNCORE)

IF( LHELDSUAREZ ) THEN
  ZDAY=86400._JPRB*RCORIOI
  RHS_KF=1.0_JPRB/ZDAY
  RHS_KA=0.025_JPRB*RHS_KF
  RHS_KS=0.25_JPRB*RHS_KF
  RHS_KS_KA=RHS_KS-RHS_KA  

  RPHSW_EQ = RPHSW_D
  RPHSW_EQ_PL_HALF = 0.5_JPRB*(RPHSW_EQ-RPHSW_PL)
  RPHSW_D_REV = 1.0_JPRB/RPHSW_D
ENDIF

!  some tests

IERR=0

IF( LHELDSUAREZ .AND. LAQUA ) THEN
  IERR=IERR+1
  WRITE(NULOUT,'(1X,A,I3,A)') ' ! SUDYNCORE: ERROR NR ',IERR,' !!!'
  WRITE(NULOUT,*) 'LHELDSUAREZ .AND. LAQUA cannot both be true!'
ENDIF

IF( MSSTSCHEME > 9 .OR. MSSTSCHEME < 0 ) THEN
  IERR=IERR+1
  WRITE(NULOUT,'(1X,A,I3,A)') ' ! SUDYNCORE: ERROR NR ',IERR,' !!!'
  WRITE(NULOUT,*) 'INVALID CHOICE FOR MSSTSCHEME: 1..9 !'
ENDIF

IF( NTESTCASE > 12 ) THEN
  IERR=IERR+1
  WRITE(NULOUT,'(1X,A,I3,A)') ' ! SUDYNCORE: ERROR NR ',IERR,' !!!'
  WRITE(NULOUT,*) 'INVALID CHOICE FOR NTESTCASE: -1..12 !'
ENDIF

WRITE(UNIT=NULOUT,FMT='('' LDYNCORE    = '',L2)') LDYNCORE
WRITE(UNIT=NULOUT,FMT='('' RPLRADI     = '',E13.7)') RPLRADI
WRITE(UNIT=NULOUT,FMT='('' RCORIOI     = '',E13.7)') RCORIOI
WRITE(UNIT=NULOUT,FMT='('' LAQUA       = '',L2)') LAQUA
WRITE(UNIT=NULOUT,FMT='('' LHELDSUAREZ = '',L2)') LHELDSUAREZ

IF( LDYNCORE ) THEN
  WRITE(UNIT=NULOUT,FMT='('' RT00_DYN     = '',E13.7)') RT00_DYN
  WRITE(UNIT=NULOUT,FMT='('' RP00_DYN     = '',E13.7)') RP00_DYN
  WRITE(UNIT=NULOUT,FMT='('' RU00_DYN     = '',E13.7)') RU00_DYN
  WRITE(UNIT=NULOUT,FMT='('' RST0_DYN     = '',E13.7)') RST0_DYN
  WRITE(UNIT=NULOUT,FMT='('' NOISEVOR     = '',I1)')    NOISEVOR
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUDYNCORE',1,ZHOOK_HANDLE)

END SUBROUTINE SUDYNCORE
