SUBROUTINE SUECPHYPO
!**** *SUECPHYPO*   - Set up PHYS_GRID, PHYS_SL and DYN_GRID, DYN_SL

!     PURPOSE.
!     --------
!           Set up grid for Coarser/Finer Physics

!**   INTERFACE.
!     ----------
!        CALL *SUECPHYPO* FROM *SUCOAPHY*
!              -------        ------

!        EXPLICIT ARGUMENTS :
!        --------------------
!        NONE

!     METHOD.
!     -------
!        Similar to SUECRAD 

!     EXTERNALS.
!     ----------
!        PHCSET

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        Deborah Salmond  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 22-Nov-2005


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIM   , ONLY : JPMXGL
USE YOMCT0   , ONLY : LOUTPUT  ,NPRINTLEV,LALLOPR,&
 & NPROC     ,N_REGIONS_NS     ,N_REGIONS_EW
USE YOMDIM   , ONLY : NDLON    ,NSMAX    ,NDGENL    ,&
 & NDGSAL   ,NDGLG    ,NDGSAG   ,NDGENG   ,NDSUR1    ,&
 & NDLSUR   ,NDGSUR   ,NGPBLKS  ,NPROMA    ,&
 & NSPEC2, NUMP 
USE YOMLUN   , ONLY : NULNAM   ,NULRAD   ,NULOUT, NULERR
USE YOMCST   , ONLY : RPI
USE YOMMP    , ONLY : MYPROC   ,NPRCIDS  ,LSPLIT ,&
 & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
 & NSTA,NONL,NPTRFRSTLAT,NFRSTLAT,NLSTLAT ,&
 & MY_REGION_NS  ,MY_REGION_EW   ,NGLOBALINDEX
USE YOMGC    , ONLY : GELAT    ,GELAM
USE YOMLEG   , ONLY : RMU      ,RSQM2, RLATIG, RW
USE YOMLAP   , ONLY : NASM0, MYMS
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG   ,NGPTOTMX ,NLOENG
USE YOMTAG   , ONLY : MTAGRAD
USE YOM_PHYS_GRID , ONLY : DYN_SL,PHYS_SL,PHYS_GRID,DYN_GRID,YPHYPOI,YDYNPOI
USE MPL_MODULE   , ONLY : MPL_BROADCAST, MPL_SEND, MPL_RECV
USE YOMCOAPHY   , ONLY : NPHYINT,CPTABLEFIL,CPTABLEDIR,NPHYRES

IMPLICIT NONE

!     LOCAL ARRAYS FOR THE PURPOSE OF READING NAMRGRI (PHYSICS GRID)
INTEGER(KIND=JPIM) :: NRGRI(JPMXGL)

INTEGER(KIND=JPIM) :: IDGL
INTEGER(KIND=JPIM) :: IPIRPTSUR,IPISPTSUR,IPIMAPLEN
INTEGER(KIND=JPIM) :: JLON,JGLAT,JGL,JGLSUR,IDLSUR,IOFF,ILAT,ISTLON,IENDLON
INTEGER(KIND=JPIM) :: IPORPTSUR,IPOSPTSUR,IPOMAPLEN
INTEGER(KIND=JPIM) :: ILBRLATI,IUBRLATI,IGLGLO,IDUM,IROF
INTEGER(KIND=JPIM) :: J,JROC,IGPTOT
INTEGER(KIND=JPIM) :: ILATS_DIFF_F,ILATS_DIFF_C, &
                      & IRISPTSUR,IRIRPTSUR
INTEGER(KIND=JPIM),ALLOCATABLE :: IGLOBALINDEX(:)
INTEGER(KIND=JPIM),PARAMETER :: JP_MIN_HALO=5

LOGICAL :: LLINEAR_GRID
LOGICAL :: LLDEBUG

REAL(KIND=JPRB) :: ZGEMU, ZLON, ZD1, ZD2, ZD3, ZD4, ZD5, ZD6
REAL(KIND=JPRB) :: ZMINPHYSLAT,ZMAXPHYSLAT,ZMINPHYSLON,ZMAXPHYSLON
REAL(KIND=JPRB) :: ZMINMDLLAT,ZMAXMDLLAT,ZMINMDLLON,ZMAXMDLLON
REAL(KIND=JPRB) :: ZDPISNL

CHARACTER (LEN = 300) ::  CLFN
INTEGER(KIND=JPIM), PARAMETER :: JPIOMASTER=1

INTEGER(KIND=JPIM), ALLOCATABLE :: IPISENDPOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPIRECVPOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPISENDPTR(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPIRECVPTR(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPICOMM(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPIMAP(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPOSENDPOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPORECVPOS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPOSENDPTR(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPORECVPTR(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPOCOMM(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IPOMAP(:,:)
INTEGER(KIND=JPIM) :: IPHYRES,IDIR,IFIL,IERR

REAL(KIND=JPRB),ALLOCATABLE :: ZLATX(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZLONX(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE,ZLAT

INTERFACE
#include "setup_trans.h"
#include "trans_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "phcset.intfb.h"
#include "namrgri.h"
#include "naecoaphy.h"

!      ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECPHYPO',0,ZHOOK_HANDLE)

LLDEBUG=.FALSE.
IPHYRES=0

CPTABLEFIL='not set'
CPTABLEDIR='./'


LLINEAR_GRID=NSMAX > (NDLON+3)/3

IF( LLDEBUG )THEN
  WRITE(NULOUT,'("SUECPHYPO: NSMAX=",I6)')NSMAX
  WRITE(NULOUT,'("SUECPHYPO: NDLON=",I6)')NDLON
  WRITE(NULOUT,'("SUECPHYPO: LLINEAR_GRID=",L5)')LLINEAR_GRID
ENDIF


! set the default physics grid resolution for the current model resolution

IF( NPHYINT == 1 ) THEN
  IF( NSMAX == 63 )THEN               
    IF(LLINEAR_GRID) THEN
      IPHYRES=21     
    ELSE
      IPHYRES=42
    ENDIF
  ENDIF
  IF( NSMAX ==   21 ) IPHYRES=   21
  IF( NSMAX ==   95 ) IPHYRES=   63
  IF( NSMAX ==  159 ) IPHYRES=   63
  IF( NSMAX ==  255 ) IPHYRES=   95
  IF( NSMAX ==  319 ) IPHYRES=  159
  IF( NSMAX ==  399 ) IPHYRES=  159
  IF( NSMAX ==  511 ) IPHYRES=  255
  IF( NSMAX ==  639 ) IPHYRES=  319
  IF( NSMAX ==  799 ) IPHYRES=  399
  IF( NSMAX == 1023 ) IPHYRES=  511
  IF( NSMAX ==   42 ) IPHYRES=   21 
  IF( NSMAX ==  106 ) IPHYRES=   63 
  IF( NSMAX ==  170 ) IPHYRES=   63 
  IF( NSMAX ==  213 ) IPHYRES=  106 
  IF( NSMAX ==  266 ) IPHYRES=  106 
  IF( NSMAX ==  341 ) IPHYRES=  170 
  IF( NSMAX ==  426 ) IPHYRES=  213 
  IF( NSMAX ==  533 ) IPHYRES=  266 
  IF( NSMAX ==  682 ) IPHYRES=  341 
  IF( NSMAX == 1279 ) IPHYRES=  639       !
  IF( NSMAX == 2047 ) IPHYRES= 1023       !
ELSEIF(NPHYINT == 2) THEN
  IF( NSMAX == 63 )THEN               
    IF(LLINEAR_GRID) THEN
      IPHYRES=95     
    ELSE
      IPHYRES=106
    ENDIF
  ENDIF
  IF( NSMAX ==   21 ) IPHYRES=  42 
  IF( NSMAX ==   95 ) IPHYRES=  159 
  IF( NSMAX ==  159 ) IPHYRES=  319
  IF( NSMAX ==  255 ) IPHYRES=  511
  IF( NSMAX ==  319 ) IPHYRES=  639
  IF( NSMAX ==  399 ) IPHYRES=  799
  IF( NSMAX ==  511 ) IPHYRES=  799
  IF( NSMAX ==  639 ) IPHYRES= 1239
  IF( NSMAX ==  799 ) IPHYRES= 1599
  IF( NSMAX == 1023 ) IPHYRES= 2047
  IF( NSMAX ==   42 ) IPHYRES=   63 
  IF( NSMAX ==   63 ) IPHYRES=  106
  IF( NSMAX ==  106 ) IPHYRES=  213
  IF( NSMAX ==  170 ) IPHYRES=  319
  IF( NSMAX ==  213 ) IPHYRES=  426
  IF( NSMAX ==  341 ) IPHYRES=  535
  IF( NSMAX ==  426 ) IPHYRES=  682
ELSE
  WRITE(NULERR,*) ' Invalid value of NPHYINT:',NPHYINT
  CALL ABOR1(' INVALID NPHYINT VALUE')
ENDIF

NPHYRES=IPHYRES

CALL POSNAM(NULNAM,'NAECOAPHY')
READ (NULNAM,NAECOAPHY)

WRITE(NULOUT,'("SUECPHYPO: PHYRES=",I4)')NPHYRES

IF( CPTABLEFIL == 'not set' )THEN
  IF( LLINEAR_GRID )THEN
    IF( IPHYRES < 1000 )THEN
      WRITE(CPTABLEFIL,'("rtablel_2",I3.3)')NPHYRES
    ELSE
      WRITE(CPTABLEFIL,'("rtablel_2",I4.4)')NPHYRES
    ENDIF
  ELSE
    IF( IPHYRES < 1000 )THEN
      WRITE(CPTABLEFIL,'("rtable_2" ,I3.3)')NPHYRES
    ELSE
      WRITE(CPTABLEFIL,'("rtable_2" ,I4.4)')NPHYRES
    ENDIF
  ENDIF
ENDIF

PHYS_GRID%NSMAX=NPHYRES
DYN_GRID%NSMAX=NSMAX

IF( MYPROC == JPIOMASTER )THEN

  IDIR=LEN_TRIM(CPTABLEDIR)
  IFIL=LEN_TRIM(CPTABLEFIL)
  CLFN=CPTABLEDIR(1:IDIR)//CPTABLEFIL(1:IFIL)
  WRITE(NULOUT,*) ' Resolution table for physics read from file',CLFN
  OPEN(NULRAD,FILE=CLFN,ACTION="READ",IOSTAT=IERR)
  IF(IERR > 0) THEN
    WRITE(NULOUT,'("SUECPHYPO: UNABLE TO OPEN FILE ",A)')CLFN
    CALL ABOR1('SUECPHYPO: UNABLE TO OPEN PHYSICS GRID RTABLE FILE')
  ENDIF
  NRGRI(:)=0
  CALL POSNAM(NULRAD,'NAMRGRI')
  READ (NULRAD,NAMRGRI)
  IDGL=1
  DO WHILE( NRGRI(IDGL)>0 )
    IF( LLDEBUG )THEN
      WRITE(NULOUT,'("SUECPHYPO: NRGRI(",I4,")=",I4)')IDGL,NRGRI(IDGL)
    ENDIF
    IDGL=IDGL+1
  ENDDO
  IDGL=IDGL-1
  PHYS_GRID%NDGLG=IDGL
  DYN_GRID%NDGLG=NDGLG
  IF( LLDEBUG )THEN
    WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGLG=",I4)')PHYS_GRID%NDGLG
  ENDIF
  CLOSE(NULRAD)
ENDIF
IF( NPROC > 1 )THEN
  CALL MPL_BROADCAST (PHYS_GRID%NDGLG,MTAGRAD,JPIOMASTER,CDSTRING='SUECPHYPO:')
  CALL MPL_BROADCAST (DYN_GRID%NDGLG,MTAGRAD,JPIOMASTER,CDSTRING='SUECPHYPO:')
ENDIF
ALLOCATE(PHYS_GRID%NRGRI(PHYS_GRID%NDGLG))
ALLOCATE(DYN_GRID%NRGRI(DYN_GRID%NDGLG))
IF( MYPROC == JPIOMASTER )THEN
  PHYS_GRID%NRGRI(1:PHYS_GRID%NDGLG)=NRGRI(1:PHYS_GRID%NDGLG)
  DYN_GRID%NRGRI(1:DYN_GRID%NDGLG)=NLOENG(1:DYN_GRID%NDGLG)
ENDIF
IF( NPROC > 1 )THEN
  CALL MPL_BROADCAST (PHYS_GRID%NRGRI(1:PHYS_GRID%NDGLG),MTAGRAD,JPIOMASTER,CDSTRING='SUECPHYPO:')
  CALL MPL_BROADCAST (DYN_GRID%NRGRI(1:DYN_GRID%NDGLG),MTAGRAD,JPIOMASTER,CDSTRING='SUECPHYPO:')
ENDIF

IF( LLDEBUG )THEN
  WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGLG=",I4)')PHYS_GRID%NDGLG
ENDIF

DYN_SL%NSLWIDEN=0
DYN_SL%NSLWIDES=0
DYN_SL%NSLWIDEW=0
DYN_SL%NSLWIDEE=0
PHYS_SL%NSLWIDEN=0
PHYS_SL%NSLWIDES=0
PHYS_SL%NSLWIDEW=0
PHYS_SL%NSLWIDEE=0

PHYS_GRID%NDLON=PHYS_GRID%NRGRI(PHYS_GRID%NDGLG/2)
PHYS_GRID%NDGSUR=2
DYN_GRID%NDLON=DYN_GRID%NRGRI(DYN_GRID%NDGLG/2)
DYN_GRID%NDGSUR=2

WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGSUR       =",I8)')PHYS_GRID%NDGSUR

PHYS_GRID%NDGSAG=1-PHYS_GRID%NDGSUR
PHYS_GRID%NDGENG=PHYS_GRID%NDGLG+PHYS_GRID%NDGSUR
DYN_GRID%NDGSAG=1-DYN_GRID%NDGSUR
DYN_GRID%NDGENG=DYN_GRID%NDGLG+DYN_GRID%NDGSUR

WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGSAG       =",I8)')PHYS_GRID%NDGSAG
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGENG       =",I8)')PHYS_GRID%NDGENG
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGLG        =",I8)')PHYS_GRID%NDGLG
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDLON        =",I8)')PHYS_GRID%NDLON
CALL FLUSH(NULOUT)

ALLOCATE(PHYS_GRID%NLOENG(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG))
PHYS_GRID%NLOENG(1:PHYS_GRID%NDGLG)=PHYS_GRID%NRGRI(1:PHYS_GRID%NDGLG)
ALLOCATE(DYN_GRID%NLOENG(DYN_GRID%NDGSAG:DYN_GRID%NDGENG))
DYN_GRID%NLOENG(1:DYN_GRID%NDGLG)=DYN_GRID%NRGRI(1:DYN_GRID%NDGLG)

IF(PHYS_GRID%NDGSUR >= 1)THEN
  DO JGLSUR=1,PHYS_GRID%NDGSUR
    PHYS_GRID%NLOENG(1-JGLSUR)=PHYS_GRID%NLOENG(JGLSUR)
  ENDDO
  DO JGLSUR=1,PHYS_GRID%NDGSUR
    PHYS_GRID%NLOENG(PHYS_GRID%NDGLG+JGLSUR)= &
     &PHYS_GRID%NLOENG(PHYS_GRID%NDGLG+1-JGLSUR)
  ENDDO
ENDIF
IF(DYN_GRID%NDGSUR >= 1)THEN
  DO JGLSUR=1,DYN_GRID%NDGSUR
    DYN_GRID%NLOENG(1-JGLSUR)=DYN_GRID%NLOENG(JGLSUR)
  ENDDO
  DO JGLSUR=1,DYN_GRID%NDGSUR
    DYN_GRID%NLOENG(DYN_GRID%NDGLG+JGLSUR)= &
     &DYN_GRID%NLOENG(DYN_GRID%NDGLG+1-JGLSUR)
  ENDDO
ENDIF

! Setup the transform package for the physics grid
CALL SETUP_TRANS (KSMAX=PHYS_GRID%NSMAX, &
     & KDGL=PHYS_GRID%NDGLG, &
     & KLOEN=PHYS_GRID%NLOENG(1:PHYS_GRID%NDGLG), &
     & LDSPLIT=LSPLIT, &
     & KRESOL=PHYS_GRID%NRESOL_ID)  

ALLOCATE(PHYS_GRID%NSTA(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
ALLOCATE(PHYS_GRID%NONL(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
ALLOCATE(PHYS_GRID%NPTRFRSTLAT(N_REGIONS_NS))
ALLOCATE(PHYS_GRID%NPTRLSTLAT(N_REGIONS_NS))
ALLOCATE(PHYS_GRID%NFRSTLAT(N_REGIONS_NS))
ALLOCATE(PHYS_GRID%NLSTLAT(N_REGIONS_NS))
ALLOCATE(PHYS_GRID%RMU(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG))
ALLOCATE(PHYS_GRID%RW(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG))
ALLOCATE(PHYS_GRID%RSQM2(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG))
ALLOCATE(PHYS_GRID%RLATIG(PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG))

ALLOCATE(DYN_GRID%NSTA(DYN_GRID%NDGSAG:DYN_GRID%NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
ALLOCATE(DYN_GRID%NONL(DYN_GRID%NDGSAG:DYN_GRID%NDGENG+N_REGIONS_NS-1,N_REGIONS_EW))
ALLOCATE(DYN_GRID%NPTRFRSTLAT(N_REGIONS_NS))
ALLOCATE(DYN_GRID%NFRSTLAT(N_REGIONS_NS))
ALLOCATE(DYN_GRID%NLSTLAT(N_REGIONS_NS))
ALLOCATE(DYN_GRID%RMU(DYN_GRID%NDGSAG:DYN_GRID%NDGENG))
ALLOCATE(DYN_GRID%RW(DYN_GRID%NDGSAG:DYN_GRID%NDGENG))
ALLOCATE(DYN_GRID%RSQM2(DYN_GRID%NDGSAG:DYN_GRID%NDGENG))
ALLOCATE(DYN_GRID%RLATIG(DYN_GRID%NDGSAG:DYN_GRID%NDGENG))

! Interrogate the transform package for the physics grid
CALL TRANS_INQ (KRESOL     =PHYS_GRID%NRESOL_ID, &
     & KSPEC2     =PHYS_GRID%NSPEC2, &
     & KNUMP      =PHYS_GRID%NUMP, &
     & KGPTOT     =PHYS_GRID%NGPTOT, &
     & KGPTOTG    =PHYS_GRID%NGPTOTG, &
     & KGPTOTMX   =PHYS_GRID%NGPTOTMX, &
     & KPTRFRSTLAT=PHYS_GRID%NPTRFRSTLAT, &
     & KPTRLSTLAT=PHYS_GRID%NPTRLSTLAT, &
     & KFRSTLAT   =PHYS_GRID%NFRSTLAT, &
     & KLSTLAT    =PHYS_GRID%NLSTLAT, &
     & KFRSTLOFF  =PHYS_GRID%NFRSTLOFF, &
     & KSTA       =PHYS_GRID%NSTA(1:PHYS_GRID%NDGLG+N_REGIONS_NS-1,:), &
     & KONL       =PHYS_GRID%NONL(1:PHYS_GRID%NDGLG+N_REGIONS_NS-1,:), &
     & KPTRFLOFF  =PHYS_GRID%NPTRFLOFF, &
     & PMU        =PHYS_GRID%RMU(1:), &
     & PGW        =PHYS_GRID%RW(1:)  )

DYN_GRID%NSPEC2=NSPEC2
DYN_GRID%NUMP=NUMP
DYN_GRID%NGPTOT=NGPTOT
DYN_GRID%NGPTOTG=NGPTOTG
DYN_GRID%NGPTOTMX=NGPTOTMX
DYN_GRID%NPTRFRSTLAT=NPTRFRSTLAT
DYN_GRID%NFRSTLAT=NFRSTLAT
DYN_GRID%NLSTLAT=NLSTLAT
DYN_GRID%NFRSTLOFF=NFRSTLOFF
DYN_GRID%NSTA(1:DYN_GRID%NDGLG+N_REGIONS_NS-1,:)=NSTA(1:NDGLG+N_REGIONS_NS-1,:)
DYN_GRID%NONL(1:DYN_GRID%NDGLG+N_REGIONS_NS-1,:)=NONL(1:NDGLG+N_REGIONS_NS-1,:)
DYN_GRID%NPTRFLOFF=NPTRFLOFF
DYN_GRID%RMU(1:)=RMU(1:)
DYN_GRID%RW(1:)=RW(1:)

ALLOCATE(YPHYPOI(PHYS_GRID%NGPTOT))
ALLOCATE(YDYNPOI(NGPTOT))
PHYS_GRID%NPROMA=NPROMA
PHYS_GRID%NGPBLKS=(PHYS_GRID%NGPTOT-1)/PHYS_GRID%NPROMA+1
DYN_GRID%NPROMA=NPROMA
DYN_GRID%NGPBLKS=(DYN_GRID%NGPTOT-1)/DYN_GRID%NPROMA+1

ALLOCATE(PHYS_GRID%GEMU(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GELAM(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GELAT(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GESLO(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GECLO(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GM(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GNORDL(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GNORDM(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GSQM2(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%RCOLON(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%RSILON(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%RINDX(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%RINDY(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%GAW(PHYS_GRID%NGPTOT))
ALLOCATE(PHYS_GRID%NGPLAT(PHYS_GRID%NGPTOT))


DO JGL=1,PHYS_GRID%NDGLG
  PHYS_GRID%RSQM2(JGL) = SQRT(1.0_JPRB - PHYS_GRID%RMU(JGL)*PHYS_GRID%RMU(JGL))
  PHYS_GRID%RLATIG(JGL) = ASIN(PHYS_GRID%RMU(JGL))
ENDDO
DO JGL=1,NDGLG
  DYN_GRID%RSQM2(JGL) = RSQM2(JGL)
  DYN_GRID%RLATIG(JGL) = RLATIG(JGL)
ENDDO
IF(PHYS_GRID%NDGSUR >= 1)THEN
  DO JGLSUR=1,PHYS_GRID%NDGSUR
    PHYS_GRID%RMU(1-JGLSUR)=2.0_JPRB-PHYS_GRID%RMU(JGLSUR)
    PHYS_GRID%RSQM2(1-JGLSUR)=PHYS_GRID%RSQM2(JGLSUR)
    PHYS_GRID%RLATIG(1-JGLSUR)=RPI-PHYS_GRID%RLATIG(JGLSUR)
  ENDDO
  DO JGLSUR=1,PHYS_GRID%NDGSUR
    PHYS_GRID%RMU(PHYS_GRID%NDGLG+JGLSUR)=&
     &-2.0_JPRB-PHYS_GRID%RMU(PHYS_GRID%NDGLG+1-JGLSUR)
    PHYS_GRID%RSQM2(PHYS_GRID%NDGLG+JGLSUR)=&
     &PHYS_GRID%RSQM2(PHYS_GRID%NDGLG+1-JGLSUR)
    PHYS_GRID%RLATIG(PHYS_GRID%NDGLG+JGLSUR)=&
     &-RPI-PHYS_GRID%RLATIG(PHYS_GRID%NDGLG+1-JGLSUR)
  ENDDO
ENDIF
IF(DYN_GRID%NDGSUR >= 1)THEN
  DO JGLSUR=1,DYN_GRID%NDGSUR
    DYN_GRID%RMU(1-JGLSUR)=2.0_JPRB-DYN_GRID%RMU(JGLSUR)
    DYN_GRID%RSQM2(1-JGLSUR)=DYN_GRID%RSQM2(JGLSUR)
    DYN_GRID%RLATIG(1-JGLSUR)=RPI-DYN_GRID%RLATIG(JGLSUR)
  ENDDO
  DO JGLSUR=1,DYN_GRID%NDGSUR
    DYN_GRID%RMU(DYN_GRID%NDGLG+JGLSUR)=&
     &-2.0_JPRB-DYN_GRID%RMU(DYN_GRID%NDGLG+1-JGLSUR)
    DYN_GRID%RSQM2(DYN_GRID%NDGLG+JGLSUR)=&
     &DYN_GRID%RSQM2(DYN_GRID%NDGLG+1-JGLSUR)
    DYN_GRID%RLATIG(DYN_GRID%NDGLG+JGLSUR)=&
     &-RPI-DYN_GRID%RLATIG(DYN_GRID%NDGLG+1-JGLSUR)
  ENDDO
ENDIF

PHYS_GRID%NDGSAL=1
PHYS_GRID%NDGENL=PHYS_GRID%NLSTLAT(MY_REGION_NS)-PHYS_GRID%NFRSTLOFF
PHYS_GRID%NDSUR1=3-MOD(PHYS_GRID%NDLON,2)
IDLSUR=MAX(PHYS_GRID%NDLON,2*PHYS_GRID%NSMAX+1)
PHYS_GRID%NDLSUR=IDLSUR+PHYS_GRID%NDSUR1
PHYS_GRID%MYFRSTACTLAT=PHYS_GRID%NFRSTLAT(MY_REGION_NS)
PHYS_GRID%MYLSTACTLAT=PHYS_GRID%NLSTLAT(MY_REGION_NS)
ALLOCATE(PHYS_GRID%NSTAGP(PHYS_GRID%NDGSAL:PHYS_GRID%NDGENL))
IROF=1
DO JGL=PHYS_GRID%NDGSAL,PHYS_GRID%NDGENL
  IF(PHYS_GRID%NONL(PHYS_GRID%NPTRFLOFF+JGL,MY_REGION_EW) > 0) THEN
    PHYS_GRID%NSTAGP(JGL)=IROF
    IROF=IROF+PHYS_GRID%NONL(PHYS_GRID%NPTRFLOFF+JGL,MY_REGION_EW)
  ELSE
    PHYS_GRID%NSTAGP(JGL)=0
  ENDIF
ENDDO

DYN_GRID%NDGSAL=1
DYN_GRID%NDGENL=DYN_GRID%NLSTLAT(MY_REGION_NS)-DYN_GRID%NFRSTLOFF
DYN_GRID%NDSUR1=3-MOD(DYN_GRID%NDLON,2)
IDLSUR=MAX(DYN_GRID%NDLON,2*DYN_GRID%NSMAX+1)
DYN_GRID%NDLSUR=IDLSUR+DYN_GRID%NDSUR1
DYN_GRID%MYFRSTACTLAT=DYN_GRID%NFRSTLAT(MY_REGION_NS)
DYN_GRID%MYLSTACTLAT=DYN_GRID%NLSTLAT(MY_REGION_NS)

WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NRESOL_ID    =",I8)')PHYS_GRID%NRESOL_ID
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NSMAX        =",I8)')PHYS_GRID%NSMAX
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NSPEC2       =",I8)')PHYS_GRID%NSPEC2
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NGPTOT       =",I8)')PHYS_GRID%NGPTOT
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NGPTOTG      =",I8)')PHYS_GRID%NGPTOTG
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGSAL       =",I8)')PHYS_GRID%NDGSAL
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGENL       =",I8)')PHYS_GRID%NDGENL
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDSUR1       =",I8)')PHYS_GRID%NDSUR1
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDLSUR       =",I8)')PHYS_GRID%NDLSUR
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%MYFRSTACTLAT =",I8)')PHYS_GRID%MYFRSTACTLAT
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%MYLSTACTLAT  =",I8)')PHYS_GRID%MYLSTACTLAT
CALL FLUSH(NULOUT)

ALLOCATE(PHYS_GRID%NASM0(0:PHYS_GRID%NSPEC2))
ALLOCATE(PHYS_GRID%MYMS(PHYS_GRID%NUMP))
CALL TRANS_INQ (KRESOL     =PHYS_GRID%NRESOL_ID, &
     & KASM0      =PHYS_GRID%NASM0, &
     & KMYMS      =PHYS_GRID%MYMS )  

ALLOCATE(DYN_GRID%NASM0(0:DYN_GRID%NSPEC2))
ALLOCATE(DYN_GRID%MYMS(DYN_GRID%NUMP))
DYN_GRID%NASM0(:)=NASM0(:)
DYN_GRID%MYMS(:)=MYMS(:)

IOFF=0
ILAT=PHYS_GRID%NPTRFLOFF
DO JGLAT=PHYS_GRID%NFRSTLAT(MY_REGION_NS), &
   & PHYS_GRID%NLSTLAT(MY_REGION_NS)  
  ZGEMU=PHYS_GRID%RMU(JGLAT)
  ILAT=ILAT+1
  ISTLON  = PHYS_GRID%NSTA(ILAT,MY_REGION_EW)
  IENDLON = ISTLON-1 + PHYS_GRID%NONL(ILAT,MY_REGION_EW)

  ZDPISNL= 2.0_JPRB*RPI/REAL(PHYS_GRID%NLOENG(JGLAT),JPRB)
  DO JLON=ISTLON,IENDLON
    ZLON=  REAL(JLON-1,JPRB)*2.0_JPRB*RPI &
     & /REAL(PHYS_GRID%NLOENG(JGLAT),JPRB)  
    IOFF=IOFF+1
!   PHYS_GRID%GELAM(IOFF) = ZLON
    PHYS_GRID%GELAM(IOFF)=ZDPISNL*REAL(JLON-1,JPRB)
    PHYS_GRID%GELAT(IOFF) = ASIN(ZGEMU)
    PHYS_GRID%GESLO(IOFF) = SIN(ZLON)
    PHYS_GRID%GECLO(IOFF) = COS(ZLON)
    PHYS_GRID%GEMU(IOFF) = ZGEMU
    PHYS_GRID%GM(IOFF) = 1.0_JPRB
    PHYS_GRID%GNORDL(IOFF) = 0.0_JPRB
    PHYS_GRID%GNORDM(IOFF) = 1.0_JPRB
    PHYS_GRID%GSQM2(IOFF) = SQRT(1.0_JPRB-ZGEMU*ZGEMU)
    PHYS_GRID%GAW(IOFF)=PHYS_GRID%RW(JGLAT)/ &
       &              (REAL(PHYS_GRID%NLOENG(JGLAT),JPRB))
    PHYS_GRID%NGPLAT(IOFF) = JGLAT
  ENDDO
ENDDO


!   For grid point interpolations we need to calculate the halo size
!   required by each processor

ALLOCATE(ZLATX(PHYS_GRID%NGPTOTMX))
ALLOCATE(ZLONX(PHYS_GRID%NGPTOTMX))
DO J=1,PHYS_GRID%NGPTOT
  ZLATX(J)=PHYS_GRID%GELAT(J)/RPI*2.0_JPRB*90.0
  ZLONX(J)=(PHYS_GRID%GELAM(J)-RPI)/RPI*180.0
ENDDO
ZMINPHYSLAT=MINVAL(ZLATX(1:PHYS_GRID%NGPTOT))
ZMAXPHYSLAT=MAXVAL(ZLATX(1:PHYS_GRID%NGPTOT))
ZMINPHYSLON=MINVAL(ZLONX(1:PHYS_GRID%NGPTOT))
ZMAXPHYSLON=MAXVAL(ZLONX(1:PHYS_GRID%NGPTOT))
IF( LLDEBUG )THEN
  WRITE(NULOUT,'("PHYGRID,BEGIN")')
  IF( MYPROC /= 1 )THEN
    CALL MPL_SEND(PHYS_GRID%NGPTOT,KDEST=NPRCIDS(1),&
     &KTAG=1,CDSTRING='SUECPHYPO.R')
    CALL MPL_SEND(ZLATX(1:PHYS_GRID%NGPTOT),&
     &KDEST=NPRCIDS(1),KTAG=2,CDSTRING='SUECPHYPO.R')
    CALL MPL_SEND(ZLONX(1:PHYS_GRID%NGPTOT),&
     &KDEST=NPRCIDS(1),KTAG=3,CDSTRING='SUECPHYPO.R')
  ENDIF
  IF( MYPROC == 1 )THEN
    DO JROC=1,NPROC
      IF( JROC == MYPROC )THEN
        DO J=1,PHYS_GRID%NGPTOT
          WRITE(NULOUT,'(F7.2,2X,F7.2,2X,I6)')ZLATX(J),ZLONX(J),MYPROC
        ENDDO
      ELSE
        CALL MPL_RECV(IGPTOT,KSOURCE=NPRCIDS(JROC),KTAG=1,CDSTRING='SUECPHYPO.M')
        CALL MPL_RECV(ZLATX(1:IGPTOT),KSOURCE=NPRCIDS(JROC),&
         &KTAG=2,CDSTRING='SUECPHYPO.M')
        CALL MPL_RECV(ZLONX(1:IGPTOT),KSOURCE=NPRCIDS(JROC),&
         &KTAG=3,CDSTRING='SUECPHYPO.M')
        DO J=1,IGPTOT
          WRITE(NULOUT,'(F7.2,2X,F7.2,2X,I6)')ZLATX(J),ZLONX(J),JROC
        ENDDO
      ENDIF
    ENDDO
  ENDIF
  WRITE(NULOUT,'("PHYGRID,END")')
ENDIF
DEALLOCATE(ZLATX)
DEALLOCATE(ZLONX)
 
ALLOCATE(ZLATX(NGPTOTMX))
ALLOCATE(ZLONX(NGPTOTMX))
DO J=1,NGPTOT
  ZLATX(J)=GELAT(J)/RPI*2.0_JPRB*90.0
  ZLONX(J)=(GELAM(J)-RPI)/RPI*180.0
ENDDO
ZMINMDLLAT=MINVAL(ZLATX(1:NGPTOT))
ZMAXMDLLAT=MAXVAL(ZLATX(1:NGPTOT))
ZMINMDLLON=MINVAL(ZLONX(1:NGPTOT))
ZMAXMDLLON=MAXVAL(ZLONX(1:NGPTOT))
IF( LLDEBUG )THEN
  WRITE(NULOUT,'("MODELGRID,BEGIN")')
  IF( MYPROC /= 1 )THEN
    CALL MPL_SEND(NGPTOT,KDEST=NPRCIDS(1),KTAG=1,CDSTRING='SUECPHYPO')
    CALL MPL_SEND(ZLATX(1:NGPTOT),KDEST=NPRCIDS(1),KTAG=2,CDSTRING='SUECPHYPO')
    CALL MPL_SEND(ZLONX(1:NGPTOT),KDEST=NPRCIDS(1),KTAG=3,CDSTRING='SUECPHYPO')
    CALL MPL_SEND(NGLOBALINDEX(1:NGPTOT),KDEST=NPRCIDS(1),KTAG=4,CDSTRING='SUECPHYPO')
  ENDIF
  IF( MYPROC == 1 )THEN
    DO JROC=1,NPROC
      IF( JROC == MYPROC )THEN
        DO J=1,NGPTOT
          WRITE(NULOUT,'(F7.2,2X,F7.2,2X,I6)')ZLATX(J),ZLONX(J),MYPROC
        ENDDO
      ELSE
        CALL MPL_RECV(IGPTOT,KSOURCE=NPRCIDS(JROC),KTAG=1,CDSTRING='SUECPHYPO')
        CALL MPL_RECV(ZLATX(1:IGPTOT),KSOURCE=NPRCIDS(JROC),KTAG=2,CDSTRING='SUECPHYPO')
        CALL MPL_RECV(ZLONX(1:IGPTOT),KSOURCE=NPRCIDS(JROC),KTAG=3,CDSTRING='SUECPHYPO')
        ALLOCATE(IGLOBALINDEX(1:IGPTOT))
        CALL MPL_RECV(IGLOBALINDEX(1:IGPTOT),KSOURCE=NPRCIDS(JROC),KTAG=4,CDSTRING='SUECPHYPO')
        DO J=1,IGPTOT
          WRITE(NULOUT,'(F7.2,2X,F7.2,2X,I6)')ZLATX(J),ZLONX(J),JROC
        ENDDO
        DEALLOCATE(IGLOBALINDEX)
      ENDIF
    ENDDO
  ENDIF
  WRITE(NULOUT,'("MODELGRID,END")')
ENDIF
DEALLOCATE(ZLATX)
DEALLOCATE(ZLONX)
  
IF( LLDEBUG )THEN
  WRITE(NULOUT,'("ZMINPHYSLAT=",F10.2)')ZMINPHYSLAT
  WRITE(NULOUT,'("ZMINMDLLAT=",F10.2)')ZMINMDLLAT
  WRITE(NULOUT,'("ZMAXPHYSLAT=",F10.2)')ZMAXPHYSLAT
  WRITE(NULOUT,'("ZMAXMDLLAT=",F10.2)')ZMAXMDLLAT
  WRITE(NULOUT,'("ZMINPHYSLON=",F10.2)')ZMINPHYSLON
  WRITE(NULOUT,'("ZMINMDLLON=",F10.2)')ZMINMDLLON
  WRITE(NULOUT,'("ZMAXPHYSLON=",F10.2)')ZMAXPHYSLON
  WRITE(NULOUT,'("ZMAXMDLLON=",F10.2)')ZMAXMDLLON
ENDIF

      ZLAT=NDGLG/180.
      ILATS_DIFF_C=CEILING(ABS(ZMINPHYSLAT-ZMINMDLLAT)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMINPHYSLAT-ZMINMDLLAT)*ZLAT)
      IF( ZMINPHYSLAT < ZMINMDLLAT )THEN
        DYN_SL%NSLWIDES=JP_MIN_HALO+ILATS_DIFF_C
      ELSE
        DYN_SL%NSLWIDES=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ENDIF
      ILATS_DIFF_C=CEILING(ABS(ZMAXPHYSLAT-ZMAXMDLLAT)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMAXPHYSLAT-ZMAXMDLLAT)*ZLAT)
      IF( ZMAXPHYSLAT < ZMAXMDLLAT )THEN
        DYN_SL%NSLWIDEN=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ELSE
        DYN_SL%NSLWIDEN=JP_MIN_HALO+ILATS_DIFF_C
      ENDIF
      ILATS_DIFF_C=CEILING(ABS(ZMINPHYSLON-ZMINMDLLON)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMINPHYSLON-ZMINMDLLON)*ZLAT)
      IF( ZMINPHYSLON < ZMINMDLLON )THEN
        DYN_SL%NSLWIDEW=JP_MIN_HALO+ILATS_DIFF_C
      ELSE
        DYN_SL%NSLWIDEW=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ENDIF
      ILATS_DIFF_C=CEILING(ABS(ZMAXPHYSLON-ZMAXMDLLON)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMAXPHYSLON-ZMAXMDLLON)*ZLAT)
      IF( ZMAXPHYSLON < ZMAXMDLLON )THEN
        DYN_SL%NSLWIDEE=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ELSE
        DYN_SL%NSLWIDEE=JP_MIN_HALO+ILATS_DIFF_C
      ENDIF

      ZLAT=PHYS_GRID%NDGLG/180.
      ILATS_DIFF_C=CEILING(ABS(ZMINPHYSLAT-ZMINMDLLAT)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMINPHYSLAT-ZMINMDLLAT)*ZLAT)
      IF( ZMINMDLLAT < ZMINPHYSLAT )THEN
        PHYS_SL%NSLWIDES=JP_MIN_HALO+ILATS_DIFF_C
      ELSE
        PHYS_SL%NSLWIDES=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ENDIF
      ILATS_DIFF_C=CEILING(ABS(ZMAXPHYSLAT-ZMAXMDLLAT)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMAXPHYSLAT-ZMAXMDLLAT)*ZLAT)
      IF( ZMAXMDLLAT < ZMAXPHYSLAT )THEN
        PHYS_SL%NSLWIDEN=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ELSE
        PHYS_SL%NSLWIDEN=JP_MIN_HALO+ILATS_DIFF_C
      ENDIF
      ILATS_DIFF_C=CEILING(ABS(ZMINPHYSLON-ZMINMDLLON)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMINPHYSLON-ZMINMDLLON)*ZLAT)
      IF( ZMINMDLLON < ZMINPHYSLON )THEN
        PHYS_SL%NSLWIDEW=JP_MIN_HALO+ILATS_DIFF_C
      ELSE
        PHYS_SL%NSLWIDEW=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ENDIF
      ILATS_DIFF_C=CEILING(ABS(ZMAXPHYSLON-ZMAXMDLLON)*ZLAT)
      ILATS_DIFF_F=FLOOR  (ABS(ZMAXPHYSLON-ZMAXMDLLON)*ZLAT)
      IF( ZMAXMDLLON < ZMAXPHYSLON )THEN
        PHYS_SL%NSLWIDEE=MAX(0,JP_MIN_HALO-ILATS_DIFF_F)
      ELSE
        PHYS_SL%NSLWIDEE=JP_MIN_HALO+ILATS_DIFF_C
      ENDIF

PHYS_GRID%NDGSAH=MAX(PHYS_GRID%NDGSAG,&
  & PHYS_GRID%NDGSAL+PHYS_GRID%NFRSTLOFF-PHYS_SL%NSLWIDEN)-PHYS_GRID%NFRSTLOFF  
PHYS_GRID%NDGENH=MIN(PHYS_GRID%NDGENG,&
  & PHYS_GRID%NDGENL+PHYS_GRID%NFRSTLOFF+PHYS_SL%NSLWIDES)-PHYS_GRID%NFRSTLOFF  
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGSAH       =",I8)')PHYS_GRID%NDGSAH
WRITE(NULOUT,'("SUECPHYPO: PHYS_GRID%NDGENH       =",I8)')PHYS_GRID%NDGENH
DYN_GRID%NDGSAH=MAX(DYN_GRID%NDGSAG,&
  & DYN_GRID%NDGSAL+DYN_GRID%NFRSTLOFF-DYN_SL%NSLWIDEN)-DYN_GRID%NFRSTLOFF  
DYN_GRID%NDGENH=MIN(DYN_GRID%NDGENG,&
  & DYN_GRID%NDGENL+DYN_GRID%NFRSTLOFF+DYN_SL%NSLWIDES)-DYN_GRID%NFRSTLOFF  

ILBRLATI = MAX(PHYS_GRID%NDGSAG,&
 & PHYS_GRID%NDGSAL+PHYS_GRID%NFRSTLOFF-PHYS_SL%NSLWIDEN)-PHYS_GRID%NFRSTLOFF  
IUBRLATI = MIN(PHYS_GRID%NDGENG,&
 & PHYS_GRID%NDGENL+PHYS_GRID%NFRSTLOFF+PHYS_SL%NSLWIDES)-PHYS_GRID%NFRSTLOFF  
ALLOCATE(PHYS_GRID%RLATI(ILBRLATI:IUBRLATI))
ALLOCATE(PHYS_GRID%RIPI0(ILBRLATI:IUBRLATI))
ALLOCATE(PHYS_GRID%RIPI1(ILBRLATI:IUBRLATI))
ALLOCATE(PHYS_GRID%RIPI2(ILBRLATI:IUBRLATI))
  
DO JGL= ILBRLATI,IUBRLATI
  IGLGLO=JGL+PHYS_GRID%NFRSTLOFF
  IF(IGLGLO >= 0.AND.IGLGLO <= PHYS_GRID%NDGLG) THEN
    ZD1=PHYS_GRID%RLATIG(IGLGLO-1)-PHYS_GRID%RLATIG(IGLGLO)
    ZD2=PHYS_GRID%RLATIG(IGLGLO-1)-PHYS_GRID%RLATIG(IGLGLO+1)
    ZD3=PHYS_GRID%RLATIG(IGLGLO-1)-PHYS_GRID%RLATIG(IGLGLO+2)
    ZD4=PHYS_GRID%RLATIG(IGLGLO  )-PHYS_GRID%RLATIG(IGLGLO+1)
    ZD5=PHYS_GRID%RLATIG(IGLGLO  )-PHYS_GRID%RLATIG(IGLGLO+2)
    ZD6=PHYS_GRID%RLATIG(IGLGLO+1)-PHYS_GRID%RLATIG(IGLGLO+2)
    PHYS_GRID%RIPI0(JGL)=-1.0_JPRB/(ZD1*ZD4*ZD5)
    PHYS_GRID%RIPI1(JGL)= 1.0_JPRB/(ZD2*ZD4*ZD6)
    PHYS_GRID%RIPI2(JGL)=-1.0_JPRB/(ZD3*ZD5*ZD6)
  ENDIF
  PHYS_GRID%RLATI(JGL)=PHYS_GRID%RLATIG(IGLGLO)
ENDDO

ILBRLATI = MAX(DYN_GRID%NDGSAG,&
 & DYN_GRID%NDGSAL+DYN_GRID%NFRSTLOFF-DYN_SL%NSLWIDEN)-DYN_GRID%NFRSTLOFF  
IUBRLATI = MIN(DYN_GRID%NDGENG,&
 & DYN_GRID%NDGENL+DYN_GRID%NFRSTLOFF+DYN_SL%NSLWIDES)-DYN_GRID%NFRSTLOFF  
ALLOCATE(DYN_GRID%RLATI(ILBRLATI:IUBRLATI))
  
DO JGL= ILBRLATI,IUBRLATI
  IGLGLO=JGL+DYN_GRID%NFRSTLOFF
  DYN_GRID%RLATI(JGL)=DYN_GRID%RLATIG(IGLGLO)
ENDDO

IF( NPROC > 1 )THEN
  IPIRPTSUR=NGPTOTG
  IPISPTSUR=2*NGPTOTG
ELSE
  IPIRPTSUR=0
  IPISPTSUR=0
ENDIF

ALLOCATE(DYN_SL%NSLSTA(NDGSAL-DYN_SL%NSLWIDEN:NDGENL+DYN_SL%NSLWIDES))
ALLOCATE(DYN_SL%NSLONL(NDGSAL-DYN_SL%NSLWIDEN:NDGENL+DYN_SL%NSLWIDES))
ALLOCATE(DYN_SL%NSLOFF(NDGSAL-DYN_SL%NSLWIDEN:NDGENL+DYN_SL%NSLWIDES))
ALLOCATE(DYN_SL%NSLEXT(1-NDLON:NDLON+NDLON,1-DYN_SL%NSLWIDEN:NDGENL+DYN_SL%NSLWIDES))
ALLOCATE(DYN_SL%NSLCORE(NGPTOT))
ALLOCATE(IPISENDPOS(IPISPTSUR))
ALLOCATE(IPIRECVPOS(IPIRPTSUR))
ALLOCATE(IPISENDPTR(NPROC+1))
ALLOCATE(IPIRECVPTR(NPROC+1))
ALLOCATE(IPICOMM(NPROC))
ALLOCATE(IPIMAP(4,NDGLG))
CALL PHCSET('PI',DYN_SL%NSLWIDEN,DYN_SL%NSLWIDES,&
       &         DYN_SL%NSLWIDEW,DYN_SL%NSLWIDEE,&
       & IRIRPTSUR,IRISPTSUR,&
       & NDGLG,NDLON,NDGSAG,NDGENG,IDUM,IDUM,NDGSAL,NDGENL,&
       & NDSUR1,NDLSUR,NDGSUR,NGPTOT,IDUM,&
       & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
       & NSTA,NONL,NLOENG,NPTRFRSTLAT,NFRSTLAT,NLSTLAT,&
       & RMU,RSQM2,&
       & DYN_SL%NSLSTA,DYN_SL%NSLONL,DYN_SL%NSLOFF,DYN_SL%NSLEXT,DYN_SL%NSLCORE,DYN_SL%NASLB1,&
       & DYN_SL%NSLPROCS,DYN_SL%NSLRPT,DYN_SL%NSLSPT,&
       & IPISENDPOS,IPIRECVPOS,IPISENDPTR,IPIRECVPTR,IPICOMM,IPIMAP,IPIMAPLEN)  
WRITE(NULOUT,'("SUECPHYPO: DYN_SL%NASLB1=",I12)')DYN_SL%NASLB1
ALLOCATE(DYN_SL%NSLSENDPOS(DYN_SL%NSLSPT))
ALLOCATE(DYN_SL%NSLRECVPOS(DYN_SL%NSLRPT))
ALLOCATE(DYN_SL%NSLSENDPTR(DYN_SL%NSLPROCS+1))
ALLOCATE(DYN_SL%NSLRECVPTR(DYN_SL%NSLPROCS+1))
ALLOCATE(DYN_SL%NSLCOMM(DYN_SL%NSLPROCS))
DYN_SL%NSLSENDPOS(1:DYN_SL%NSLSPT)=IPISENDPOS(1:DYN_SL%NSLSPT)
DYN_SL%NSLRECVPOS(1:DYN_SL%NSLRPT)=IPIRECVPOS(1:DYN_SL%NSLRPT)
DYN_SL%NSLSENDPTR(1:DYN_SL%NSLPROCS+1)=IPISENDPTR(1:DYN_SL%NSLPROCS+1)
DYN_SL%NSLRECVPTR(1:DYN_SL%NSLPROCS+1)=IPIRECVPTR(1:DYN_SL%NSLPROCS+1)
DYN_SL%NSLCOMM(1:DYN_SL%NSLPROCS)=IPICOMM(1:DYN_SL%NSLPROCS)
DEALLOCATE(IPISENDPOS)
DEALLOCATE(IPIRECVPOS)
DEALLOCATE(IPISENDPTR)
DEALLOCATE(IPIRECVPTR)
DEALLOCATE(IPICOMM)
DEALLOCATE(IPIMAP)

IF( NPROC > 1 )THEN
  IPORPTSUR=PHYS_GRID%NGPTOTG
  IPOSPTSUR=2*PHYS_GRID%NGPTOTG
ELSE
  IPORPTSUR=0
  IPOSPTSUR=0
ENDIF

ALLOCATE(PHYS_SL%NSLSTA(PHYS_GRID%NDGSAL-PHYS_SL%NSLWIDEN:PHYS_GRID%NDGENL+PHYS_SL%NSLWIDES))
ALLOCATE(PHYS_SL%NSLONL(PHYS_GRID%NDGSAL-PHYS_SL%NSLWIDEN:PHYS_GRID%NDGENL+PHYS_SL%NSLWIDES))
ALLOCATE(PHYS_SL%NSLOFF(PHYS_GRID%NDGSAL-PHYS_SL%NSLWIDEN:PHYS_GRID%NDGENL+PHYS_SL%NSLWIDES))
ALLOCATE(PHYS_SL%NSLEXT(1-PHYS_GRID%NDLON:PHYS_GRID%NDLON+PHYS_GRID%NDLON,&
 & 1-PHYS_SL%NSLWIDEN:PHYS_GRID%NDGENL+PHYS_SL%NSLWIDES))  
ALLOCATE(PHYS_SL%NSLCORE(PHYS_GRID%NGPTOT))
ALLOCATE(IPOSENDPOS(IPOSPTSUR))
ALLOCATE(IPORECVPOS(IPORPTSUR))
ALLOCATE(IPOSENDPTR(NPROC+1))
ALLOCATE(IPORECVPTR(NPROC+1))
ALLOCATE(IPOCOMM(NPROC))
ALLOCATE(IPOMAP(4,PHYS_GRID%NDGLG))
CALL PHCSET('PO',PHYS_SL%NSLWIDEN,PHYS_SL%NSLWIDES,&
       &               PHYS_SL%NSLWIDEW,PHYS_SL%NSLWIDEE,&
       & IRIRPTSUR,IRISPTSUR,&
       & PHYS_GRID%NDGLG,PHYS_GRID%NDLON,PHYS_GRID%NDGSAG,&
       & PHYS_GRID%NDGENG,IDUM,IDUM,PHYS_GRID%NDGSAL,PHYS_GRID%NDGENL,&
       & PHYS_GRID%NDSUR1,PHYS_GRID%NDLSUR,PHYS_GRID%NDGSUR,PHYS_GRID%NGPTOT,IDUM,&
       & PHYS_GRID%NPTRFLOFF,PHYS_GRID%NFRSTLOFF,PHYS_GRID%MYFRSTACTLAT,PHYS_GRID%MYLSTACTLAT,&
       & PHYS_GRID%NSTA,PHYS_GRID%NONL,PHYS_GRID%NLOENG,PHYS_GRID%NPTRFRSTLAT,&
       & PHYS_GRID%NFRSTLAT,PHYS_GRID%NLSTLAT,&
       & PHYS_GRID%RMU,PHYS_GRID%RSQM2,&
       & PHYS_SL%NSLSTA,PHYS_SL%NSLONL,PHYS_SL%NSLOFF,PHYS_SL%NSLEXT,PHYS_SL%NSLCORE,PHYS_SL%NASLB1,&
       & PHYS_SL%NSLPROCS,PHYS_SL%NSLRPT,PHYS_SL%NSLSPT,&
       & IPOSENDPOS,IPORECVPOS,IPOSENDPTR,IPORECVPTR,IPOCOMM,IPOMAP,IPOMAPLEN)  
WRITE(NULOUT,'("SUECPHYPO: PHYS_SL%NASLB1=",I12)')PHYS_SL%NASLB1
ALLOCATE(PHYS_SL%NSLSENDPOS(PHYS_SL%NSLSPT))
ALLOCATE(PHYS_SL%NSLRECVPOS(PHYS_SL%NSLRPT))
ALLOCATE(PHYS_SL%NSLSENDPTR(PHYS_SL%NSLPROCS+1))
ALLOCATE(PHYS_SL%NSLRECVPTR(PHYS_SL%NSLPROCS+1))
ALLOCATE(PHYS_SL%NSLCOMM(PHYS_SL%NSLPROCS))
PHYS_SL%NSLSENDPOS(1:PHYS_SL%NSLSPT)=IPOSENDPOS(1:PHYS_SL%NSLSPT)
PHYS_SL%NSLRECVPOS(1:PHYS_SL%NSLRPT)=IPORECVPOS(1:PHYS_SL%NSLRPT)
PHYS_SL%NSLSENDPTR(1:PHYS_SL%NSLPROCS+1)=IPOSENDPTR(1:PHYS_SL%NSLPROCS+1)
PHYS_SL%NSLRECVPTR(1:PHYS_SL%NSLPROCS+1)=IPORECVPTR(1:PHYS_SL%NSLPROCS+1)
PHYS_SL%NSLCOMM(1:PHYS_SL%NSLPROCS)=IPOCOMM(1:PHYS_SL%NSLPROCS)
DEALLOCATE(IPOSENDPOS)
DEALLOCATE(IPORECVPOS)
DEALLOCATE(IPOSENDPTR)
DEALLOCATE(IPORECVPTR)
DEALLOCATE(IPOCOMM)
DEALLOCATE(IPOMAP)

!      ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECPHYPO',1,ZHOOK_HANDLE)
END SUBROUTINE SUECPHYPO
