SUBROUTINE SUEMIS_CONF

!**** *SUEMIS_CONF*  - 

!     PURPOSE.
!     --------
!     Set-up of variables for emissivity calculation (YOMEMIS and NAMEMIS_CONF
!      variables).

!**   INTERFACE.
!     ----------
!       *CALL* *SUEMIS_CONF*

!        EXPLICIT ARGUMENTS
!        --------------------
!         none

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     AUTHOR.
!     -------
!        ???

!     MODIFICATIONS.
!     --------------
!     K. Yessad (Dec 2008): add comments + cleanings
!-----------------------------------------------------------------------------
   
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMCT0B   ,ONLY : LECMWF
USE YOMEMIS  , ONLY : LATLAS_AMSU, LDYN_AMSU, LICE_AMSU, LSKIN_AMSU, &
        &  LATLAS_SSMI, LSKIN_SSMI, LDYN_SSMI, LICE_SSMI, &
        &  LATLAS_SSMIS, LSKIN_SSMIS, LDYN_SSMIS, LICE_SSMIS,&
        &  LATLAS_TMI, LSKIN_TMI, LDYN_TMI, LICE_TMI, &
        &  LATLAS_AMSRE, LSKIN_AMSRE, LDYN_AMSRE, LICE_AMSRE, &
        &  LATLAS_MWTS, LSKIN_MWTS, LDYN_MWTS, &
        &  LATLAS_MWHS, LSKIN_MWHS, LDYN_MWHS, &
        &  LATLAS_MWRI, LSKIN_MWRI, LDYN_MWRI, &
        &  LATLAS_WINDSAT, LSKIN_WINDSAT, LDYN_WINDSAT, LICE_WINDSAT, &
        &  MEMIS_CHAN, MTSKIN_CHAN, LDYN_EMIS, LDYN_TSKIN, &
        &  LATLAS_EMIS, LDYN_ICE
USE YOMTVRAD , ONLY : MSENSOR_AMSUA, MSENSOR_AMSUB, MSENSOR_MHS, &
 & MSENSOR_MWTS, MSENSOR_MWHS, MSENSOR_MWRI, &
 & MSENSOR_SSMI, MSENSOR_SSMIS, MSENSOR_AMSRE, MSENSOR_TMI, MSENSOR_WINDSAT, &
 & CHSENSOR,MSENSOR_CHANCOUNT, MXSENSOR

!-----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM)  :: ISENSOR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "namemis_conf.h"   

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUEMIS_CONF',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------------
  
  ! Default values for switches

  LATLAS_AMSU=.FALSE.
  LDYN_AMSU=.FALSE.
  LSKIN_AMSU=.FALSE.
  LICE_AMSU=.FALSE.

  LATLAS_MWTS=.FALSE.
  LDYN_MWTS=.FALSE.
  LSKIN_MWTS=.FALSE.

  LATLAS_MWHS=.FALSE.
  LDYN_MWHS=.FALSE.
  LSKIN_MWHS=.FALSE.

  LATLAS_MWRI=.FALSE.
  LDYN_MWRI=.FALSE.
  LSKIN_MWRI=.FALSE.

  LATLAS_SSMI=.FALSE.
  LDYN_SSMI=.FALSE.
  LSKIN_SSMI=.FALSE.
  LICE_SSMI=.FALSE.

  LATLAS_SSMIS=.FALSE.
  LDYN_SSMIS=.FALSE.
  LSKIN_SSMIS=.FALSE.
  LICE_SSMIS=.FALSE.

  LATLAS_TMI=.FALSE.
  LDYN_TMI=.FALSE.
  LSKIN_TMI=.FALSE.
  LICE_TMI=.FALSE.

  LATLAS_AMSRE=.FALSE.
  LDYN_AMSRE=.FALSE.
  LSKIN_AMSRE=.FALSE.
  LICE_AMSRE=.FALSE.

  LATLAS_WINDSAT=.FALSE.
  LDYN_WINDSAT=.FALSE.
  LSKIN_WINDSAT=.FALSE.
  LICE_WINDSAT=.FALSE.
  
  ! Also for general switches

  LATLAS_EMIS = .FALSE.
  LDYN_EMIS = .FALSE.
  LDYN_TSKIN = .FALSE.
  LDYN_ICE = .FALSE.

  ! Set channels used for emissivity retrieval

  MEMIS_CHAN = -1
  IF( LECMWF ) THEN
    MEMIS_CHAN(MSENSOR_AMSUA,1:15) = 3
  ELSE
    MEMIS_CHAN(MSENSOR_AMSUA,1:15) = 2
  ENDIF
  MEMIS_CHAN(MSENSOR_AMSUB,1:5)  = 1
  MEMIS_CHAN(MSENSOR_MHS,1:5)    = 1
  MEMIS_CHAN(MSENSOR_MWHS,1:5) = 1
  MEMIS_CHAN(MSENSOR_SSMI,(/1,3,4,6/)) = 1
  MEMIS_CHAN(MSENSOR_SSMI,(/2,5,7/))   = 2
  MEMIS_CHAN(MSENSOR_SSMIS,1:7)   = 1
  MEMIS_CHAN(MSENSOR_SSMIS,8:11) = 18
  MEMIS_CHAN(MSENSOR_SSMIS,(/12,15/))       = 12
  MEMIS_CHAN(MSENSOR_SSMIS,(/13,14,16,17/)) = 13
  MEMIS_CHAN(MSENSOR_SSMIS,18)   = 18
  MEMIS_CHAN(MSENSOR_SSMIS,19:24) = 1
  MEMIS_CHAN(MSENSOR_TMI,(/1,3,5,6,8/))   = 1
  MEMIS_CHAN(MSENSOR_TMI,(/2,4,7,9/))     = 2
  MEMIS_CHAN(MSENSOR_AMSRE,(/1,3,5,7,9,11/))  = 1
  MEMIS_CHAN(MSENSOR_AMSRE,(/2,4,6,8,10,12/)) = 2
  MEMIS_CHAN(MSENSOR_MWTS,1:4) = 1
  MEMIS_CHAN(MSENSOR_MWRI,(/1,3,5,7,9/)) = 1
  MEMIS_CHAN(MSENSOR_MWRI,(/2,4,6,8,10/)) = 2
  MEMIS_CHAN(MSENSOR_WINDSAT,(/1,3,7,11,13/))  = 1
  MEMIS_CHAN(MSENSOR_WINDSAT,(/2,4,8,12,14/)) = 2

  ! Set channels used for skin temperature retrieval

  MTSKIN_CHAN = -1
  MTSKIN_CHAN(MSENSOR_AMSUA) = 3
  MTSKIN_CHAN(MSENSOR_AMSUB) = 1
  MTSKIN_CHAN(MSENSOR_MHS)   = 1
  MTSKIN_CHAN(MSENSOR_MWHS) = 1
  MTSKIN_CHAN(MSENSOR_SSMI)  = 1
  MTSKIN_CHAN(MSENSOR_SSMIS) = 13
  MTSKIN_CHAN(MSENSOR_AMSRE) = 5
  MTSKIN_CHAN(MSENSOR_TMI)   = 3
  MTSKIN_CHAN(MSENSOR_MWTS) = 1 
  MTSKIN_CHAN(MSENSOR_MWRI) = 2
  MTSKIN_CHAN(MSENSOR_WINDSAT)   = 7

  !        1.2 Modify default values 
  CALL POSNAM(NULNAM,'NAMEMIS_CONF')
  READ(NULNAM,NAMEMIS_CONF)

  ! Set general switches if specific ones were used - overwrites settings
  ! in general switches.

  IF( LATLAS_AMSU )THEN
    LATLAS_EMIS(MSENSOR_AMSUA) = .TRUE.
    LATLAS_EMIS(MSENSOR_AMSUB) = .TRUE.
    LATLAS_EMIS(MSENSOR_MHS) = .TRUE.
  ENDIF
  IF( LDYN_AMSU )THEN
    LDYN_EMIS(MSENSOR_AMSUA) = .TRUE.
    LDYN_EMIS(MSENSOR_AMSUB) = .TRUE.
    LDYN_EMIS(MSENSOR_MHS) = .TRUE.
  ENDIF
  IF( LICE_AMSU )THEN
    LDYN_ICE(MSENSOR_AMSUA) = .TRUE.
    LDYN_ICE(MSENSOR_AMSUB) = .TRUE.
    LDYN_ICE(MSENSOR_MHS) = .TRUE.
  ENDIF
  IF( LSKIN_AMSU )THEN
    LDYN_TSKIN(MSENSOR_AMSUA) = .TRUE.
    LDYN_TSKIN(MSENSOR_AMSUB) = .TRUE.
    LDYN_TSKIN(MSENSOR_MHS) = .TRUE.
  ENDIF

  IF( LATLAS_MWTS ) LATLAS_EMIS(MSENSOR_MWTS) = .TRUE.
  IF( LDYN_MWTS )   LDYN_EMIS(MSENSOR_MWTS) = .TRUE.
  IF( LSKIN_MWTS )  LDYN_TSKIN(MSENSOR_MWTS) = .TRUE.

  IF( LATLAS_MWHS ) LATLAS_EMIS(MSENSOR_MWHS) = .TRUE.
  IF( LDYN_MWHS )   LDYN_EMIS(MSENSOR_MWHS) = .TRUE.
  IF( LSKIN_MWHS )  LDYN_TSKIN(MSENSOR_MWHS) = .TRUE.

  IF( LATLAS_MWRI ) LATLAS_EMIS(MSENSOR_MWRI) = .TRUE.
  IF( LDYN_MWRI )   LDYN_EMIS(MSENSOR_MWRI) = .TRUE.
  IF( LSKIN_MWRI )  LDYN_TSKIN(MSENSOR_MWRI) = .TRUE.

  IF( LATLAS_SSMI ) LATLAS_EMIS(MSENSOR_SSMI) = .TRUE.
  IF( LDYN_SSMI )   LDYN_EMIS(MSENSOR_SSMI) = .TRUE.
  IF( LSKIN_SSMI )  LDYN_TSKIN(MSENSOR_SSMI) = .TRUE.
  IF( LICE_SSMI )   LDYN_ICE(MSENSOR_SSMI) = .TRUE.

  IF( LATLAS_SSMIS ) LATLAS_EMIS(MSENSOR_SSMIS) = .TRUE.
  IF( LDYN_SSMIS )   LDYN_EMIS(MSENSOR_SSMIS) = .TRUE.
  IF( LSKIN_SSMIS )  LDYN_TSKIN(MSENSOR_SSMIS) = .TRUE.
  IF( LICE_SSMIS )   LDYN_ICE(MSENSOR_SSMIS) = .TRUE.

  IF( LATLAS_TMI ) LATLAS_EMIS(MSENSOR_TMI) = .TRUE.
  IF( LDYN_TMI )   LDYN_EMIS(MSENSOR_TMI) = .TRUE.
  IF( LSKIN_TMI )  LDYN_TSKIN(MSENSOR_TMI) = .TRUE.
  IF( LICE_TMI )   LDYN_ICE(MSENSOR_TMI) = .TRUE.

  IF( LATLAS_AMSRE ) LATLAS_EMIS(MSENSOR_AMSRE) = .TRUE.
  IF( LDYN_AMSRE )   LDYN_EMIS(MSENSOR_AMSRE) = .TRUE.
  IF( LSKIN_AMSRE )  LDYN_TSKIN(MSENSOR_AMSRE) = .TRUE.
  IF( LICE_AMSRE )   LDYN_ICE(MSENSOR_AMSRE) = .TRUE.

  IF( LATLAS_WINDSAT ) LATLAS_EMIS(MSENSOR_WINDSAT) = .TRUE.
  IF( LDYN_WINDSAT )   LDYN_EMIS(MSENSOR_WINDSAT) = .TRUE.
  IF( LSKIN_WINDSAT )  LDYN_TSKIN(MSENSOR_WINDSAT) = .TRUE.
  IF( LICE_WINDSAT )   LDYN_ICE(MSENSOR_WINDSAT) = .TRUE.

  ! Check that only one switch is used

  DO ISENSOR = 0,MXSENSOR
    IF(COUNT((/LATLAS_EMIS(ISENSOR),LDYN_EMIS(ISENSOR),&
     &         LDYN_TSKIN(ISENSOR)/))>1)THEN
      CALL ABOR1('Conflicting emissivity/skin temperature switches for '//CHSENSOR(ISENSOR))
    ENDIF
  ENDDO

  ! Some output

  IF( ANY(LDYN_EMIS) .OR. ANY(LDYN_TSKIN) .OR. ANY(LATLAS_EMIS) .OR. ANY(LDYN_ICE)) THEN
    WRITE(NULOUT,*) '=============================================================================='
    WRITE(NULOUT,*) 'SUEMIS_CONF: Special configurations for emissivity/skin temperature treatment:'
    WRITE(NULOUT,*) '=============================================================================='
    DO ISENSOR=0,MXSENSOR
      IF( LATLAS_EMIS(ISENSOR) ) THEN
        WRITE(NULOUT,*) CHSENSOR(ISENSOR)//': Atlas emissivities used.'
      ENDIF
      IF( LDYN_EMIS(ISENSOR) ) THEN
        WRITE(NULOUT,*) CHSENSOR(ISENSOR)//': Dynamic emissivities used.'
        WRITE(NULOUT,*) ' Emissivity channel by channel:',&
         &   MEMIS_CHAN(ISENSOR,1:MSENSOR_CHANCOUNT(ISENSOR))
      ENDIF
      IF( LDYN_TSKIN(ISENSOR) ) THEN
        WRITE(NULOUT,*) CHSENSOR(ISENSOR)//': Dynamic skin temperature and atlas emissivities used.'
        WRITE(NULOUT,*) ' Skin temperature channel:',&
         &   MTSKIN_CHAN(ISENSOR)
      ENDIF
      IF( LDYN_ICE(ISENSOR) ) THEN
        WRITE(NULOUT,*) CHSENSOR(ISENSOR)//': Dynamic emissivities used for sea ice'
        WRITE(NULOUT,*) ' Emissivity channel by channel:',&
         &   MEMIS_CHAN(ISENSOR,1:MSENSOR_CHANCOUNT(ISENSOR))
      ENDIF
    ENDDO
    WRITE(NULOUT,*) '=============================================================================='
  ENDIF

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUEMIS_CONF',1,ZHOOK_HANDLE)
END SUBROUTINE SUEMIS_CONF
