SUBROUTINE SUFA

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULNAM   ,NSCRTCH  ,NINISH   ,NPPPSH
USE YOMCT0   , ONLY : LARPEGEF ,NCONF    ,LAROME
USE YOMCT0B  , ONLY : LECMWF
USE YOMFA    , ONLY : NVGRIB,   NBITPG   ,NBITCS   ,NSTRON   ,NPULAP   , &
 & YFAOROG  ,YFASP    ,YFAPSI   ,YFAKHI   ,YFAUGEO  ,YFAVGEO  , &
 & YFAT     ,YFAPD    ,YFAVD    ,YFAQ     ,YFAL     ,YFAI     , &
 & YFAO3    ,YFACLF   ,YFAS     ,YFAR     ,YFAG     ,YFAH     ,YFATKE   , &
 & YFACPF   ,YFASPF   ,YFAFSP1  ,YFAFSP2  ,YFAFSP3  ,YFAFSP4  , &
 & YFAFSP5  ,YFASRC   ,YFASDSAT ,YFACVV   ,YFACVGQ  ,YFAQVA   , &
 & YFAUOM   ,YFAUAL   ,YFADOM   ,YFADAL   ,YFAUEN   ,YFAUNEBH
USE YOMARG   , ONLY : NARGS    ,CUNMEXP
USE YOMFPCT0 , ONLY : NFPCT0
USE YOMOP    , ONLY : CFNPL
USE TYPE_FADS, ONLY : YSUFAD


!**** *SUFA*  - Initialize characteristics for GRIB packing
!               in FA files (to do before FA files opening)
!               Sous-programme de definition de quelques caracteristiques
!               du compactage GRIB dans les fichiers FA (a appeler avant
!               l'ouverture des fichiers FA)

!     PURPOSE.
!     --------
!        Read namelist NAMFA / Lit et verifie la namelist NAMFA.

!**   INTERFACE.
!     ----------
!       *CALL* *SUFA*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS
!        --------------------
!        MODULE YOMLUN
!        MODULE YOMCT0B
!        MODULE YOMFA
!        NAMELIST NAMFA

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!        calls FAVORI and FAGIOT
!        Is called by SU0YOMA.

!     REFERENCE.
!     ----------

!     AUTHOR.
!     -------
!        D. Paradis  *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 99-07-27
!        Modified by D. Paradis 00-08-03: NVGRIB variable introduction
!       R. El Khatib 01-01-25 Cleanings
!       R. El Khatib 03-08-19
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!           M. Tudor 03-10-17 add YFACPF and YFASPF
!       Y. Seity 04-11-16 define AROME fields name
!       P. Termonia 04-09-01 add YFAFSPi, i=1,2,3,4,5
!      D. Paradis & R. El Khatib : 04-07-22 GRIBEX
!      Y. Bouteloup 05-01-20 add YFAR (atmospheric rain)
!      F. Bouyssel 05-11-09 Change second descriptor of YSUFAD
!      B. Sass 01-06-06 Define YFAQVA used with HIRLAM
!      A. Alias 06-04-03 SDSAT and CVV added
!      M. Bellus 29-Sep-2006 add prognostic convection
!           and MAC scheme (coded by L. Gerard 09-Mar-2005)
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IDMOPL, IUNTIN, INBARI, IREP

CHARACTER(LEN=50) :: CLFNISH
CHARACTER(LEN=16) :: CLLEC
CHARACTER(LEN=12) :: CLSCRTCH

LOGICAL :: LLEXIST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "posnam.intfb.h"

!     ------------------------------------------------------------------

#include "namfa.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. SET DEFAULT VALUES
!           -------------------

IF (LARPEGEF) THEN

! FAVORI gives implicit options of GRIB packing for *FA* files
  CALL FAVORI (NVGRIB, NBITPG, NBITCS, NSTRON, NPULAP, IDMOPL)

ELSE

  NBITPG=24
  NBITCS=24
  NVGRIB=2
  NSTRON=10
  NPULAP=1

ENDIF

YFAOROG =YSUFAD('SPECSURFGEOPOTEN',0)
YFASP   =YSUFAD('SURFPRESSION    ',-1)
YFAPSI  =YSUFAD('POT.VITESSE     ',-1)
YFAKHI  =YSUFAD('FONC.COURANT    ',-1)
YFAUGEO =YSUFAD('WIND.U.PHYS     ',-1)
YFAVGEO =YSUFAD('WIND.V.PHYS     ',-1)
YFAT    =YSUFAD('TEMPERATURE     ',-1)
YFAPD   =YSUFAD('PRESS.DEPART    ',-1)
YFAVD   =YSUFAD('VERTIC.DIVER    ',-1)
YFAFSP1 =YSUFAD('CUF1PRESSURE    ',-1)
YFAFSP2 =YSUFAD('CUF2PRESSURE    ',-1)
YFAFSP3 =YSUFAD('CUF3PRESSURE    ',-1)
YFAFSP4 =YSUFAD('CUF4PRESSURE    ',-1)
YFAFSP5 =YSUFAD('CUF5PRESSURE    ',-1)
SELECT CASE(LECMWF)
CASE (.TRUE.)
  YFAQ    =YSUFAD(' HUMIDITY       ',-1)
  YFAL    =YSUFAD(' LIQUID WATER   ',-1)
  YFAI    =YSUFAD(' ICE WATER      ',-1)
  YFAO3   =YSUFAD(' OZONE          ',-1)
  YFACLF  =YSUFAD(' CLOUD FRACTION ',-1)
  YFACPF  =YSUFAD('CV_PREC_FLUX    ',-1)
  YFASPF  =YSUFAD('ST_PREC_FLUX    ',-1)
  YFAS    =YSUFAD('SNOW            ',-1)
  YFAR    =YSUFAD('RAIN            ',-1)
  YFAG    =YSUFAD('GRAUPEL         ',-1)
  YFAH    =YSUFAD('HAIL            ',-1)
  YFATKE  =YSUFAD('TKE             ',-1)
  YFASRC  =YSUFAD('SRC             ',-1)
  YFACVGQ =YSUFAD('CVGQ            ',-1)
  YFAQVA  =YSUFAD('QVA_AMPLITUDE   ',-1)
  YFASDSAT=YSUFAD('                ',-1)
  YFACVV  =YSUFAD('                ',-1)
  YFAUOM  =YSUFAD('                ',-1)
  YFAUAL  =YSUFAD('                ',-1)
  YFADOM  =YSUFAD('                ',-1)
  YFADAL  =YSUFAD('                ',-1)
  YFAUEN  =YSUFAD('                ',-1)
  YFAUNEBH=YSUFAD('                ',-1)
CASE DEFAULT
  YFAQ    =YSUFAD('HUMI.SPECIFI    ',-1)
  YFAL    =YSUFAD('LIQUID_WATER    ',-1)
  YFAI    =YSUFAD('SOLID_WATER     ',-1)
  YFAO3   =YSUFAD('OZONE           ',-1)
  YFACLF  =YSUFAD('CLOUD_FRACTI    ',-1)
  YFACPF  =YSUFAD('CV_PREC_FLUX    ',-1)
  YFASPF  =YSUFAD('ST_PREC_FLUX    ',-1)
  YFAS    =YSUFAD('SNOW            ',-1)
  YFAR    =YSUFAD('RAIN            ',-1)
  YFAG    =YSUFAD('GRAUPEL         ',-1)
  YFAH    =YSUFAD('HAIL            ',-1)
  YFATKE  =YSUFAD('TKE             ',-1)
  YFASRC  =YSUFAD('SRC             ',-1)
  YFACVGQ =YSUFAD('CVGQ            ',-1)
  YFAQVA  =YSUFAD('QVA_AMPLITUDE   ',-1)
  YFASDSAT=YSUFAD('SDSAT           ',-1)
  YFACVV  =YSUFAD('CVV             ',-1)
  YFAUOM  =YSUFAD('UD_OMEGA        ',-1)
  YFAUAL  =YSUFAD('UD_MESH_FRAC    ',-1)
  YFADOM  =YSUFAD('DD_OMEGA        ',-1)
  YFADAL  =YSUFAD('DD_MESH_FRAC    ',-1)
  YFAUEN  =YSUFAD('UD_ENTRAINMENT  ',-1)
  YFAUNEBH=YSUFAD('PSHI_CONV_CLOUD ',-1)
END SELECT
IF (LAROME) THEN
  YFAQ    =YSUFAD('HUMI.SPECIFI    ',-1)
  YFAL    =YSUFAD('CLOUD_WATER     ',-1)
  YFAI    =YSUFAD('ICE_CRYSTAL     ',-1)
  YFAS    =YSUFAD('SNOW            ',-1)
  YFAR    =YSUFAD('RAIN            ',-1)
  YFAG    =YSUFAD('GRAUPEL         ',-1)
  YFAH    =YSUFAD('HAIL            ',-1)
  YFATKE  =YSUFAD('TKE             ',-1)
  YFACLF  =YSUFAD('CLOUD_FRACTI    ',-1)
  YFASRC  =YSUFAD('SRC             ',-1)
ENDIF
!     ------------------------------------------------------------------

!*       2. READ NAMELIST
!           -------------

CALL POSNAM(NULNAM,'NAMFA')
READ(NULNAM,NAMFA)
WRITE(UNIT=NULOUT,FMT='(/,'' NAMELIST NAMFA'')')
WRITE(UNIT=NULOUT,FMT=&
 & '(''   NVGRIB ='',I2,''   NBITPG ='',I3,''   NBITCS ='',I3,&
 & ''   NSTRON ='',I4,''   NPULAP ='',I2,/)')&
 & NVGRIB,NBITPG, NBITCS, NSTRON, NPULAP  

!     ------------------------------------------------------------------

!*       3. Control and set up of GRIB packing parameters for files ARPEGE/ALADIN
!           ---------------------------------------------------------------------

IF (LARPEGEF) THEN
  CALL FAGIOT ( NVGRIB, NBITPG, NBITCS, NSTRON, NPULAP, IDMOPL )
! Try to find out which values are used  : to achieve this : one should 
! create a new file environement
  IF (NCONF /= 901 .AND. (NARGS > 0.OR.NFPCT0==0)) THEN
    IF (NFPCT0 /= 0) THEN
      CLFNISH='ICMSH'//CUNMEXP//'INIT'
      IUNTIN=NINISH
    ELSE
      CLFNISH=CFNPL
      IUNTIN=NPPPSH
    ENDIF
    INQUIRE(FILE=CLFNISH,EXIST=LLEXIST)
    IF (LLEXIST) THEN
      CLLEC='CADRE LECTURE   '
      INBARI=0
      CALL FAITOU(IREP,IUNTIN,.TRUE.,CLFNISH,'OLD',.TRUE.,.TRUE.,1,1,INBARI,CLLEC)
      CALL LFIMST(IREP,IUNTIN,.FALSE.)
      CALL FANOUV(IREP,NSCRTCH,.FALSE.,CLSCRTCH,'NEW',.TRUE.,.TRUE.,1,1,&
       & INBARI,CLLEC)  
      CALL FAVEUR(IREP,NSCRTCH,NVGRIB,NBITPG,NBITCS,NSTRON,NPULAP,IDMOPL)
      CALL FAIRNO(IREP,NSCRTCH,'DELETE')
      CALL FAIRME(IREP,IUNTIN,'UNKNOWN')
    ENDIF
  ENDIF
  IF (NVGRIB == 3) THEN
    WRITE(NULOUT,*) &
     & 'WARNING : GRIBEX TABLES ARE STILL HARD-CODED FOR FA FIELDS'
  ENDIF
ENDIF

WRITE(UNIT=NULOUT,FMT='(/,'' MODULE YOMFA'')')
WRITE(UNIT=NULOUT,FMT=&
 & '(''   NVGRIB ='',I2,''   NBITPG ='',I3,''   NBITCS ='',I3,&
 & ''   NSTRON ='',I4,''   NPULAP ='',I2,/)')&
 & NVGRIB,NBITPG, NBITCS, NSTRON, NPULAP  
WRITE(UNIT=NULOUT,FMT='(3(''( '',A16,1X,I3,'') ''))') &
 & YFAOROG  ,YFASP    ,YFAPSI   ,YFAKHI   ,YFAUGEO  ,YFAVGEO  , &
 & YFAT     ,YFAPD    ,YFAVD    ,YFAQ     ,YFAL     ,YFAI     , &
 & YFAR     ,YFAS     ,YFAG     ,YFAH     ,YFATKE   ,YFASRC   , & 
 & YFAO3    ,YFACLF   ,YFACPF   ,YFASPF   ,YFACVGQ  , &
 & YFASDSAT ,YFACVV   ,YFAQVA   , &
 & YFAUOM   ,YFAUAL   ,YFADOM   ,YFADAL   ,YFAUEN   ,YFAUNEBH
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFA',1,ZHOOK_HANDLE)
END SUBROUTINE SUFA
