SUBROUTINE SUFRAG(KNMAX,KNCMAX,PSTRET,KFACTD,KFACTR)

!**** *SUFRAG - Computation of granularity factors for:
!               - dilatation matrices.
!               - rotation matrices.

!     Purpose.
!     --------

!     Computation of granularity factors for:
!       - dilatation matrices.
!       - rotation matrices.
!     Algorithm assumes that these factors are proportional to the size
!     of the matrices.

!**   Interface.
!     ----------
!        *CALL* *SUFRAG(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KNMAX     - lower truncation.
!          KNCMAX    - upper truncation.
!          PSTRET    - stretching coefficient.

!        OUTPUT:
!          KFACTD    - granularity factor for dilatation matrices.
!          KFACTR    - granularity factor for rotation matrices.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about conf 911 and 912.

!     Externals.
!     ----------
!       None

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP). 

!     Modifications.
!     --------------
!        Original : APRIL 1999
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRET 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFACTD 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFACTR 
REAL(KIND=JPRB) :: ZFACTD,ZFACTDREF,ZFACTR,ZFACTRREF,ZSIZE,ZSIZEREF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

!*  1. CASE NO STRETCHING.
!   ----------------------

IF (LHOOK) CALL DR_HOOK('SUFRAG',0,ZHOOK_HANDLE)
IF (PSTRET == 1.0_JPRB) THEN
  KFACTD=1
  KFACTR=1
ENDIF

! ------------------------------------------------------------------

!*  2. CASE OF STRETCHING.
!   ----------------------

IF (PSTRET > 1.0_JPRB) THEN

  ZSIZEREF=6232525._JPRB

  ZFACTDREF=10._JPRB
  ZSIZE=0.5_JPRB*REAL(KNMAX+1,JPRB)*REAL(KNMAX+2,JPRB)*REAL(KNCMAX+1,JPRB)&
   & -(1.0_JPRB/6._JPRB)*REAL(KNMAX,JPRB)*REAL(KNMAX+1,JPRB)*REAL(KNMAX+2,JPRB)  
  ZFACTD=ZFACTDREF*ZSIZE/ZSIZEREF
  KFACTD=MAX(1,NINT(ZFACTD))

  ZFACTRREF=10._JPRB
  ZSIZE=(1.0_JPRB/6._JPRB)*REAL(KNCMAX+1,JPRB)*REAL(KNCMAX+2,JPRB)*&
   & REAL(2*KNCMAX+3,JPRB)&
   & -(1.0_JPRB/6._JPRB)*REAL(KNCMAX-KNMAX,JPRB)*REAL(KNCMAX-KNMAX+1,JPRB)*&
   & REAL(2*KNCMAX-2*KNMAX+1,JPRB)&
   & -0.5_JPRB*REAL(KNCMAX-KNMAX,JPRB)*REAL(KNCMAX-KNMAX+1,JPRB)  
  ZFACTR=ZFACTRREF*ZSIZE/ZSIZEREF
  KFACTR=MAX(1,NINT(ZFACTR))

ENDIF

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFRAG',1,ZHOOK_HANDLE)
END SUBROUTINE SUFRAG
