!OCL  NOPREEX
SUBROUTINE SUGEM2

!**** *SUGEM2*  - Initialize geometry parameters - part 2

!     Purpose.
!     --------
!           Initialize geometry

!**   Interface.
!     ----------
!        *CALL* *SUGEM2

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        COMMON YOMGEM

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        Radmila Bubnova 92-07-02: remove "set up" of the vertical
!                                  system outside this subroutine.
!        D. Giard : 93-04-30 : computation of R4JP,RC2P1,RC2M1,RCOR0,RCOR1,
!                              (RCOR2) moved to SUGEM1 ; RCOR0,RCOR1,RCOR2
!                              in YOMGEM now.
!        P. Marquet : 96-09-18 : Unsigned modifs (15R2) of G. Mosdzynski
!                            COUPLED DIAG. allocations moved from suallo
!        K. YESSAD: (MARCH 1997): corrections of some bugs.
!          - define NGPLAT for NSTTYP=2.
!          - correct initialisation of geometrical quantities for
!            extra-longitudes and polar and extra-polar latitudes if
!            not distributed memory, correction of a array out of bounds
!            for RW.
!          - replace NLOEN by NLOENG, NDGSA by NDGSAG, NDGEN by NDGENG.
!        V. CASSE: (SPRING 1997): F90 writing, removal of double precision
!          computations.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0 and of the code under LRPOLE.
!        Z. Bencetic Klaic: 98-10-21 ozone coefficients stored in 1d arrays 
!              (NTOZ1D)
!        Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!        Modified 00-08-07 by K. YESSAD:
!          - relaxation of thin layer hypothesis: new geometric
!            arrays RCORIC,GNORDLCL,GNORDMCL,GNORDMCM
!        Modified 03-08-25 by F. Vana: Geographical correction for SLHD
!        Modified 06-06-08 by F. Vana: Consistent setting of SLHDP with LAM
!        Modified 05-09-12 by A. Alias : GMGEC modif. introduced
!                     S. Tyteca  - new diagnostics
!                     P. Marquet - Cloud Forcing diagnostics
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI      ,ROMEGA   ,RA
USE YOMDIM   , ONLY : NDGLG    ,NDGENL,&
 & NDLON    ,NSMAX  
USE YOMGEM   , ONLY : RCOLON   ,RSILON   ,RINDX    ,RINDY    ,&
 & NLOENG   ,NSTAGP   ,NSTTYP   ,NGPTOT   ,RMUCEN   ,&
 & RLOCEN   ,RSTRET   ,RC2P1    ,RC2M1  
USE YOMCT0   , ONLY : NFRCO    ,NPRINTLEV,&
 & LRPLANE  ,LOUTPUT  ,LALLOPR  
USE YOMGC    , ONLY : RCORI    ,GEMU     ,GSQM2    ,GELAM    ,&
 & GELAT    ,GECLO    ,GESLO    ,GM       ,GOMVRL   ,&
 & GOMVRM   ,GNORDL   ,GNORDM   ,OROG     ,OROGL    ,&
 & OROGM    ,OROGLL   ,OROGMM   ,OROGLM   ,GAW      ,&
 & NGPLAT   ,NUNIQUEGP,RCORIC   ,GNORDLCL ,GNORDMCL ,&
 & GNORDMCM  
USE YOMLUN   , ONLY : NULOUT
USE YOMLEG   , ONLY : RW       ,RMU
USE YOMMP    , ONLY : MY_REGION_EW   ,NSTA     ,NONL     ,MYLATS   ,NPTRFLOFF
USE YOMGCO   , ONLY : STRSU    ,STRSV    ,FCHAS    ,FRSOS    ,&
 & FHUMS    ,RUIST    ,FCHLL    ,FCHLN    ,FCHSS    ,&
 & FHUML    ,FHUMN    ,FRLDS    ,QWPRO    ,TMAX2    ,&
 & TMIN2    ,HSNOW    ,TSUR2    ,TSTS     ,TSFL     ,&
 & XPUQ     ,XPVQ     ,FCRFTH   ,FCRFSO
USE YOMDPHY  , ONLY : NTSSG
USE YOMSLD   , ONLY : SLHDP

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IENDLAT, IGLG, IROF, ISTLAT, IU, JGL, JLON

LOGICAL :: LLP

REAL(KIND=JPRB) :: Z2C, Z2OMR, ZA, ZB, ZC2M1, ZC2P1, ZCA1, ZCLABNE, ZCLAC,&
 & ZCLAP, ZCLO, ZCLOC, ZCLOP, ZD, ZDPISNL, ZGEMU, &
 & ZGM, ZGOMVRL, ZGSQM2, ZINCR, ZLON, ZMUDEN, &
 & ZMUNUM, ZQM2, ZRCORI, ZRCORIC, ZSA1, ZSLABNE, ZSLAC, ZSLAP, &
 & ZSLO, ZSLOC, ZSLOP, ZTLABNE, ZUS2C, ZUSA, ZUSCLABNE  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*       1.    ALLOCATE ARRAYS 
!              ---------------

IF (LHOOK) CALL DR_HOOK('SUGEM2',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
ALLOCATE(RCORI  (NGPTOT))
IF(LLP)WRITE(IU,9) 'RCORI    ',SIZE(RCORI    ),SHAPE(RCORI    )
ALLOCATE(RCORIC (NGPTOT))
IF(LLP)WRITE(IU,9) 'RCORIC   ',SIZE(RCORIC   ),SHAPE(RCORIC   )
ALLOCATE(GEMU   (NGPTOT))
IF(LLP)WRITE(IU,9) 'GEMU     ',SIZE(GEMU     ),SHAPE(GEMU     )
ALLOCATE(GSQM2  (NGPTOT))
IF(LLP)WRITE(IU,9) 'GSQM2    ',SIZE(GSQM2    ),SHAPE(GSQM2    )
ALLOCATE(GELAM  (NGPTOT))
IF(LLP)WRITE(IU,9) 'GELAM    ',SIZE(GELAM    ),SHAPE(GELAM    )
ALLOCATE(GELAT  (NGPTOT))
IF(LLP)WRITE(IU,9) 'GELAT    ',SIZE(GELAT    ),SHAPE(GELAT    )
ALLOCATE(GECLO  (NGPTOT))
IF(LLP)WRITE(IU,9) 'GECLO    ',SIZE(GECLO    ),SHAPE(GECLO    )
ALLOCATE(GESLO  (NGPTOT))
IF(LLP)WRITE(IU,9) 'GESLO    ',SIZE(GESLO    ),SHAPE(GESLO    )
ALLOCATE(GM     (NGPTOT))
IF(LLP)WRITE(IU,9) 'GM       ',SIZE(GM       ),SHAPE(GM       )
ALLOCATE(NUNIQUEGP(NGPTOT))
IF(LLP)WRITE(IU,9) 'NUNIQUEGP  ',SIZE(NUNIQUEGP  ),SHAPE(NUNIQUEGP  )
ALLOCATE(GOMVRL (NGPTOT))
IF(LLP)WRITE(IU,9) 'GOMVRL   ',SIZE(GOMVRL   ),SHAPE(GOMVRL   )
ALLOCATE(GOMVRM (NGPTOT))
IF(LLP)WRITE(IU,9) 'GOMVRM   ',SIZE(GOMVRM   ),SHAPE(GOMVRM   )
ALLOCATE(GNORDL (NGPTOT))
IF(LLP)WRITE(IU,9) 'GNORDL   ',SIZE(GNORDL   ),SHAPE(GNORDL   )
ALLOCATE(GNORDM (NGPTOT))
IF(LLP)WRITE(IU,9) 'GNORDM   ',SIZE(GNORDM   ),SHAPE(GNORDM   )
ALLOCATE(GNORDLCL(NGPTOT))
IF(LLP)WRITE(IU,9) 'GNORDLCL ',SIZE(GNORDLCL ),SHAPE(GNORDLCL )
ALLOCATE(GNORDMCL(NGPTOT))
IF(LLP)WRITE(IU,9) 'GNORDMCL ',SIZE(GNORDMCL ),SHAPE(GNORDMCL )
ALLOCATE(GNORDMCM(NGPTOT))
IF(LLP)WRITE(IU,9) 'GNORDMCM ',SIZE(GNORDMCM ),SHAPE(GNORDMCM )
ALLOCATE(GAW (NGPTOT))
IF(LLP)WRITE(IU,9) 'GAW      ',SIZE(GAW      ),SHAPE(GAW      )
ALLOCATE(NGPLAT (NGPTOT))
IF(LLP)WRITE(IU,9) 'NGPLAT   ',SIZE(NGPLAT   ),SHAPE(NGPLAT   )

ALLOCATE(RCOLON (NGPTOT))
IF(LLP)WRITE(IU,9) 'RCOLON   ',SIZE(RCOLON   ),SHAPE(RCOLON   )
ALLOCATE(RSILON (NGPTOT))
IF(LLP)WRITE(IU,9) 'RSILON   ',SIZE(RSILON   ),SHAPE(RSILON   )
ALLOCATE(RINDX  (NGPTOT))
IF(LLP)WRITE(IU,9) 'RINDX    ',SIZE(RINDX    ),SHAPE(RINDX    )
ALLOCATE(RINDY  (NGPTOT))
IF(LLP)WRITE(IU,9) 'RINDY    ',SIZE(RINDY    ),SHAPE(RINDY    )

ALLOCATE(OROG   (NGPTOT))
IF(LLP)WRITE(IU,9) 'OROG     ',SIZE(OROG     ),SHAPE(OROG     )
ALLOCATE(OROGL  (NGPTOT))
IF(LLP)WRITE(IU,9) 'OROGL    ',SIZE(OROGL    ),SHAPE(OROGL    )
ALLOCATE(OROGM  (NGPTOT))
IF(LLP)WRITE(IU,9) 'OROGM    ',SIZE(OROGM    ),SHAPE(OROGM    )
ALLOCATE(OROGLL (NGPTOT))
IF(LLP)WRITE(IU,9) 'OROGLL   ',SIZE(OROGLL   ),SHAPE(OROGLL   )
ALLOCATE(OROGMM (NGPTOT))
IF(LLP)WRITE(IU,9) 'OROGMM   ',SIZE(OROGMM   ),SHAPE(OROGMM   )
ALLOCATE(OROGLM (NGPTOT))
IF(LLP)WRITE(IU,9) 'OROGLM   ',SIZE(OROGLM   ),SHAPE(OROGLM   )

!     COUPLED FIELDS DIAGNOSTICS

IF(NFRCO /= 0) THEN
  ALLOCATE(STRSU(NGPTOT))
  STRSU(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'STRSU    ',SIZE(STRSU),SHAPE(STRSU)
  ALLOCATE(STRSV(NGPTOT))
  STRSV(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'STRSV    ',SIZE(STRSV),SHAPE(STRSV)
  ALLOCATE(FCHAS(NGPTOT,NTSSG+1))
  FCHAS(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FCHAS    ',SIZE(FCHAS),SHAPE(FCHAS)
  ALLOCATE(FRSOS(NGPTOT,NTSSG+1))
  FRSOS(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FRSOS    ',SIZE(FRSOS),SHAPE(FRSOS)
  ALLOCATE(FHUMS(NGPTOT,NTSSG+1))
  FHUMS(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FHUMS    ',SIZE(FHUMS),SHAPE(FHUMS)
  ALLOCATE(RUIST(NGPTOT))
  RUIST(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'RUIST    ',SIZE(RUIST),SHAPE(RUIST)
  ALLOCATE(FCHLL(NGPTOT,NTSSG+1))
  FCHLL(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FCHLL    ',SIZE(FCHLL),SHAPE(FCHLL)
  ALLOCATE(FCHLN(NGPTOT,NTSSG+1))
  FCHLN(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FCHLN    ',SIZE(FCHLN),SHAPE(FCHLN)
  ALLOCATE(FCHSS(NGPTOT,NTSSG+1))
  FCHSS(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FCHSS    ',SIZE(FCHSS),SHAPE(FCHSS)
  ALLOCATE(FHUML(NGPTOT))
  FHUML(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FHUML    ',SIZE(FHUML),SHAPE(FHUML)
  ALLOCATE(FHUMN(NGPTOT))
  FHUMN(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FHUMN    ',SIZE(FHUMN),SHAPE(FHUMN)
  ALLOCATE(FRLDS(NGPTOT))
  FRLDS(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FRLDS    ',SIZE(FRLDS),SHAPE(FRLDS)
  ALLOCATE(QWPRO(NGPTOT))
  QWPRO(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'QWPRO    ',SIZE(QWPRO),SHAPE(QWPRO)
  ALLOCATE(TMAX2(NGPTOT))
  TMAX2(:)=-1000._JPRB
  IF(LLP)WRITE(IU,9) 'TMAX2    ',SIZE(TMAX2),SHAPE(TMAX2)
  ALLOCATE(TMIN2(NGPTOT))
  TMIN2(:)=1000._JPRB
  IF(LLP)WRITE(IU,9) 'TMIN2    ',SIZE(TMIN2),SHAPE(TMIN2)
  ALLOCATE(HSNOW(NGPTOT))
  HSNOW(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'HSNOW    ',SIZE(HSNOW),SHAPE(HSNOW)
  ALLOCATE(TSUR2(NGPTOT))
  TSUR2(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'TSUR2    ',SIZE(TSUR2),SHAPE(TSUR2)
  ALLOCATE(TSTS(NGPTOT))
  TSTS(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'TSTS     ',SIZE(TSTS),SHAPE(TSTS)
  ALLOCATE(TSFL(NGPTOT))
  TSFL(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'TSFL     ',SIZE(TSFL),SHAPE(TSFL)
  ALLOCATE(XPUQ(NGPTOT))
  XPUQ(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'XPUQ     ',SIZE(XPUQ),SHAPE(XPUQ)
  ALLOCATE(XPVQ(NGPTOT))
  XPVQ(:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'XPVQ     ',SIZE(XPVQ),SHAPE(XPVQ)
  ALLOCATE(FCRFTH(NGPTOT,0:1))
  FCRFTH(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FCRFTH   ',SIZE(FCRFTH),SHAPE(FCRFTH)
  ALLOCATE(FCRFSO(NGPTOT,0:1))
  FCRFSO(:,:)=0._JPRB
  IF(LLP)WRITE(IU,9) 'FCRFSO   ',SIZE(FCRFSO),SHAPE(FCRFSO)
ENDIF

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!     ------------------------------------------------------------------

!*       2.    Initialize part of YOMGC.
!              -------------------------

ISTLAT=1
IENDLAT=NDGENL

IF(NSTTYP == 1)THEN

!       2.1  Case NSTTYP=1.

  IF (LOUTPUT) WRITE(UNIT=NULOUT,FMT='('' ******** NSTTYP=1'')')
  ZUS2C = 0.5_JPRB / RSTRET
  ZUSA  = 1.0_JPRB/RA
  CALL GSTATS(1405,0)
!$OMP PARALLEL DO SCHEDULE(STATIC)&
!$OMP&PRIVATE(JGL,IGLG,ZINCR,ZDPISNL,ZMUNUM,ZMUDEN,ZGEMU,&
!$OMP& ZGSQM2,ZGM,ZRCORI,ZRCORIC,ZGOMVRL,IROF,JLON)
  DO JGL = ISTLAT,IENDLAT
    IGLG=MYLATS(JGL)
    IF ((IGLG <= 0).OR.(IGLG >= NDGLG+1)) THEN
      ZINCR=RPI
    ELSE
      ZINCR=0._JPRB
    ENDIF
    ZDPISNL= 2.0_JPRB*RPI/REAL(NLOENG(IGLG),JPRB)
    ZMUNUM = RC2M1 + RC2P1 * RMU(IGLG)
    ZMUDEN = RC2P1 + RC2M1 * RMU(IGLG)
    ZGEMU  = ZMUNUM / ZMUDEN
    ZGSQM2 = SQRT(1.0_JPRB-ZGEMU*ZGEMU)
    ZGM    = ZMUDEN*ZUS2C
    ZRCORI = 2.0_JPRB*ROMEGA*ZGEMU
    ZRCORIC= 2.0_JPRB*ROMEGA*ZGSQM2
    ZGOMVRL= 2.0_JPRB*ROMEGA*RA*ZGSQM2
    IROF=NSTAGP(JGL)-1
    DO JLON = NSTA(NPTRFLOFF+JGL,MY_REGION_EW),&
       & NSTA(NPTRFLOFF+JGL,MY_REGION_EW)+NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1  
      IROF=IROF+1
      RCORI(IROF)=ZRCORI
      RCORIC(IROF)=ZRCORIC
      GEMU (IROF)=ZGEMU
      GSQM2(IROF)=ZGSQM2
      GELAM(IROF)=ZINCR+ZDPISNL*REAL(JLON-1,JPRB)
      GELAT(IROF)=ASIN(ZGEMU)
      GECLO(IROF)=COS(GELAM(IROF))
      GESLO(IROF)=SIN(GELAM(IROF))
      GM   (IROF)=ZGM
      NUNIQUEGP(IROF)=NDLON*(IGLG-1)+JLON
      NGPLAT(IROF)=IGLG
      GOMVRL(IROF)=ZGOMVRL
      GOMVRM(IROF)=0.0_JPRB
      GNORDL(IROF)=0.0_JPRB
      GNORDM(IROF)=1.0_JPRB
      GNORDLCL(IROF)=0.0_JPRB
      GNORDMCL(IROF)=0.0_JPRB
      GNORDMCM(IROF)=-ZUSA*ZGEMU
      GAW(IROF)=RW(IGLG)/(REAL(NLOENG(IGLG),JPRB)*GM(IROF)**2)
    ENDDO
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1405,1)

ELSEIF(NSTTYP == 2)THEN

!       2.2  Case NSTTYP=2.

  IF (LOUTPUT) WRITE(UNIT=NULOUT,FMT='('' ******** NSTTYP=2'')')

  Z2C   = 2.0_JPRB * RSTRET
  ZC2P1 = RSTRET* RSTRET+1.0_JPRB
  ZC2M1 = RSTRET* RSTRET-1.0_JPRB
  ZSLAP = RMUCEN
  ZCLAP = SQRT(1.0_JPRB - ZSLAP*ZSLAP)
  ZSLOP = SIN(RLOCEN)
  ZCLOP = COS(RLOCEN)
  Z2OMR = 2.0_JPRB*ROMEGA*RA
  ZUSA  = 1.0_JPRB/RA

  DO  JGL = ISTLAT,IENDLAT
    IGLG  = MYLATS(JGL)
    IF ((IGLG <= 0).OR.(IGLG >= NDGLG+1)) THEN
      ZINCR=RPI
    ELSE
      ZINCR=0._JPRB
    ENDIF
    ZDPISNL= 2.0_JPRB*RPI/REAL(NLOENG(IGLG),JPRB)
    ZSLAC = RMU(IGLG)
    ZCLAC = SQRT(1.0_JPRB-RMU(IGLG)*RMU(IGLG))
    ZA    = 1.0_JPRB / (ZC2P1 + ZC2M1 * ZSLAC)
    ZGM   = (ZC2P1 + ZC2M1 * ZSLAC) / Z2C
    ZB    = ZC2M1 + ZC2P1 * ZSLAC
!   * "theta_bne" is the latitude on the tilted unstretched sphere associated
!     with the computational sphere; one computes ZCLABNE=cos(theta_bne),
!     ZSLABNE=sin(theta_bne), ZTLABNE=tan(theta_bne),
!     ZUSCLABNE=1/cos(theta_bne).
    ZCLABNE=ZCLAC/ZGM
    ZUSCLABNE=1.0_JPRB/ZCLABNE
    ZSLABNE=ZB*ZA
    ZTLABNE=ZSLABNE/ZCLABNE

    IROF=NSTAGP(JGL)-1
    DO JLON = NSTA(NPTRFLOFF+JGL,MY_REGION_EW),&
       & NSTA(NPTRFLOFF+JGL,MY_REGION_EW)+NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1  
      IROF=IROF+1
      ZSLOC= SIN( ZINCR+ZDPISNL*REAL(JLON-1,JPRB) )
      ZCLOC= COS( ZINCR+ZDPISNL*REAL(JLON-1,JPRB) )
      GEMU(IROF)  = ZA*(Z2C*ZCLAP*ZCLAC*ZCLOC+ZB*ZSLAP)
      GELAT(IROF) = ASIN(GEMU(IROF))
      GSQM2(IROF) = SQRT(1.0_JPRB-GEMU(IROF)*GEMU(IROF))
      ZQM2        = 1.0_JPRB / GSQM2(IROF)
      RCORI(IROF) = 2.0_JPRB * ROMEGA * GEMU(IROF)
      RCORIC(IROF)= 2.0_JPRB * ROMEGA * GSQM2(IROF)
      GM   (IROF) = ZGM
      NUNIQUEGP(IROF)=NDLON*(IGLG-1)+JLON
      NGPLAT(IROF)= IGLG

      ZD = ZA * ZQM2
      ZCLO = ZD *( ZB*ZCLAP*ZCLOP - Z2C*ZCLAC*&
       & (ZSLAP*ZCLOC*ZCLOP-ZSLOP*ZSLOC) )  
      ZSLO = ZD *( ZB*ZCLAP*ZSLOP - Z2C*ZCLAC*&
       & (ZSLAP*ZCLOC*ZSLOP+ZCLOP*ZSLOC) )  
      ZSLO = MIN(1.0_JPRB,MAX(-1.0_JPRB,ZSLO))
      GECLO(IROF) = ZCLO
      GESLO(IROF) = ZSLO
      GELAM(IROF) = SIGN(1.0_JPRB,ZCLO) * ASIN(ZSLO) - RPI *&
       & ( MIN(SIGN(1.0_JPRB,ZCLO),0.0_JPRB) + MIN(SIGN(2.0_JPRB,ZSLO),0.0_JPRB)&
       & +MIN(SIGN(1.0_JPRB,ZCLO),0.0_JPRB) * MIN(SIGN(2.0_JPRB,ZSLO),0.0_JPRB) )  

      ZCA1 = (Z2C*ZSLAP*ZCLAC-ZB*ZCLAP*ZCLOC) * ZA
      ZSA1 = -ZCLAP*ZSLOC
      GOMVRL(IROF) =  Z2OMR * ZCA1
      GOMVRM(IROF) = -Z2OMR * ZSA1
      GNORDL(IROF) = ZSA1 * ZQM2
      GNORDM(IROF) = ZCA1 * ZQM2
      GAW(IROF)=RW(IGLG)/(REAL(NLOENG(IGLG),JPRB)*GM(IROF)**2)
      GNORDLCL(IROF)=-(ZUSA*ZCLAP*ZUSCLABNE)*ZCLOC
      GNORDMCL(IROF)=(ZUSA*ZCLAP*ZTLABNE)*ZSLOC
      GNORDMCM(IROF)=-ZUSA*(ZSLAP*ZSLABNE+ZCLAP*ZCLABNE*ZCLOC)
    ENDDO
  ENDDO

ELSE

!       2.3  Case NSTTYP >2 or <1.

  WRITE(UNIT=NULOUT,FMT='('' ERROR NSTTYP '')')
  CALL ABOR1(' SUGEM2 ')

ENDIF

!     ------------------------------------------------------------------

!*       3.    Initialize trigonometric lines of longitudes.
!              ---------------------------------------------

CALL GSTATS(1406,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JGL,IGLG,ZINCR,ZDPISNL,IROF,JLON,ZLON)
DO JGL = ISTLAT,IENDLAT
  IGLG=MYLATS(JGL)
  IF ((IGLG <= 0).OR.(IGLG >= NDGLG+1)) THEN
    ZINCR=RPI
  ELSE
    ZINCR=0._JPRB
  ENDIF
  ZDPISNL=2.0_JPRB*RPI/REAL(NLOENG(IGLG),JPRB)
  IROF=NSTAGP(JGL)-1
  DO JLON=NSTA(NPTRFLOFF+JGL,MY_REGION_EW),NSTA(NPTRFLOFF+JGL,MY_REGION_EW)&
     & +NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1  
    IROF=IROF+1
    ZLON=ZINCR+ZDPISNL*REAL(JLON-1,JPRB)
    RCOLON(IROF)=COS(ZLON)
    RSILON(IROF)=SIN(ZLON)
    IF (LRPLANE) THEN
      RINDX (IROF)=REAL(JLON,JPRB)
      RINDY (IROF)=REAL(IGLG,JPRB)
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1406,1)

!     ------------------------------------------------------------------

!*       4.    Resolution correction for SLHD.
!              -------------------------------

! * This formula should be 2*RPI*SQRT(RA*RA/(NSMAX*(NSMAX+1)))
! For the consistency reasons with the existing LAM tuning
! it has been multiplied by SQRT(2.)

SLHDP=RPI*SQRT(8.0_JPRB*RA*RA/(NSMAX*(NSMAX+1)))

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGEM2',1,ZHOOK_HANDLE)
END SUBROUTINE SUGEM2
