SUBROUTINE SUGRCFU(KFILE)

!**** *SUGRCFU*  - Initialize the cumulative fluxes from *FA*

!     Purpose.
!     --------
!           Initialize the cumulative fluxes fields of the model from FA.

!**   Interface.
!     ----------
!        *CALL* *SUGRCFU(....)*

!        Explicit arguments : KFILE - indicator for which file is to be read
!        ------------------

!        Implicit arguments :
!        ------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!            OPENFA
!            RDPXFA

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!       R. El Khatib  *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 97-12-05
!        Modified : 98-01-05 D. Giard (soil fluxes)
!        Modified : 99-02-12 E. Bazile (Frozen fluxes)
!        Modified : 99-04-19 J.M. Piriou (Moon radiation)
!        Modified : 00-03-29 R. El Khatib (Unification SM/DM)
!        Modified : 00-03 by R. El Khatib - derived types
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LRPLANE
USE YOMGEM   , ONLY : NGPTOT
USE YOMCFU   , ONLY : NFDCFU   ,NTYPCFU  ,TYPE_CFU
USE YOMGFUB  , ONLY : GFUBUF
USE YOMGC    , ONLY : GM

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
INTEGER(KIND=JPIM) ::  IFLDPT(NFDCFU), ILEV(NFDCFU), IPAR(NFDCFU)
REAL(KIND=JPRB) :: ZFIELD(NGPTOT,NFDCFU), Z1SGM(NGPTOT)
CHARACTER(LEN=4) :: CLPREF(NFDCFU)
CHARACTER(LEN=16) :: CLSUFF(NFDCFU)

INTEGER(KIND=JPIM) :: IOFF, IFILE, JF, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "incgpf.intfb.h"
#include "rdpxfa.intfb.h"
!     ------------------------------------------------------------------

!*    1.    PREPARATIONS.
!           -------------

IF (LHOOK) CALL DR_HOOK('SUGRCFU',0,ZHOOK_HANDLE)
IF (KFILE == 6) THEN
  IFILE=30
ELSE
  IFILE=KFILE
ENDIF
IOFF=0
DO JF=1,NTYPCFU
  DO JL=1,TYPE_CFU(JF)%ILEV
    CLPREF(IOFF+JL)=TYPE_CFU(JF)%CLPREF(JL)
    ILEV(IOFF+JL)=TYPE_CFU(JF)%INUMLEV(JL)
    CLSUFF(IOFF+JL)=TYPE_CFU(JF)%CLSUFF
    IPAR(IOFF+JL)=TYPE_CFU(JF)%IPARITY
    IFLDPT(IOFF+JL)=IOFF+JL
  ENDDO
  IOFF=IOFF+TYPE_CFU(JF)%ILEV
ENDDO

!*    2.    READ FILE AND DISTRIBUTE FIELDS
!           -------------------------------

CALL RDPXFA(ZFIELD,CLPREF,ILEV,CLSUFF,NFDCFU,IFILE)

!     3.    REDUCE VECTOR FIELDS
!           --------------------

IF (LRPLANE) THEN
  Z1SGM(1:NGPTOT)=1.0_JPRB/GM(1:NGPTOT)
  DO JF=1,NFDCFU
    IF (IPAR(JF) == 1) THEN
      ZFIELD(1:NGPTOT,JF)=ZFIELD(1:NGPTOT,JF)*Z1SGM(1:NGPTOT)
    ENDIF
  ENDDO
ENDIF

!     4.    WRITE BUFFER/WORKFILE
!           ---------------------

CALL INCGPF(NFDCFU,GFUBUF,IFLDPT,ZFIELD)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRCFU',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRCFU
