SUBROUTINE SUGRCLIA(LDINT,KLI)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM    ,ONLY : NPROMA
USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG  
USE SURFACE_FIELDS_MIX, ONLY : SD_VX, YSD_VXD, YSD_VX
USE YOMMCC   , ONLY : LMCC01
USE YOMMP    , ONLY : MYPROC   ,NSTRIN
USE YOMCT0   , ONLY : NPROC    ,LOUTPUT
USE YOMTAG   , ONLY : MTAGPART
USE YOMRIP   , ONLY : NINDAT
USE QACLIM   , ONLY : LCLIM
USE MPL_MODULE, ONLY : MPL_BROADCAST

!**** *SUGRCLIA*  - Initialize the clim. gridpoint fields from *FA*

!     Purpose.
!     --------
!           Initialize the clim. gridpoint fields from *FA*

!**   Interface.
!     ----------
!        *CALL* *SUGRCLIA(...)*

!        Explicit arguments :
!        ------------------
!         LDINT   - .TRUE. to make time interpolated fields
!         KLI     - Number of climatology files actually read

!        Implicit arguments :
!        --------------------
!        See 'USE MODULE' above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Calls 'FA' and LFI routines.
!        Is called by SUGRIDF

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        R. EL KHATIB *METEO-FRANCE*
!        after subroutines SUGRCLIADM and SUCACLIA 

!     Modifications.
!     --------------
!        ORIGINAL : 00-02-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        F.Taillefer   20-Jan-2009 Use in CANARI (instead of SUCACLIA)
!     Apr 2008  K. Yessad: use DISGRID instead of DISGRID_C + cleanings
!     ------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDINT 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLI 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JSTGLO,IEND,IBL,JL 
INTEGER(KIND=JPIM) :: INDEX, IREP, JV, J, IDEB, IFIN
INTEGER(KIND=JPIM) :: IDIM                      ! Number of files to read
INTEGER(KIND=JPIM) :: IYEAR, IMONTH, IDAY       ! year, month & day of the model
INTEGER(KIND=JPIM) :: IAGPIO(YSD_VXD%NUMFLDS)            ! I/O control array for distribution
INTEGER(KIND=JPIM) :: ICOUNT                    ! Number of global fields per processor for I/O
LOGICAL   :: LLFILE(2)                 ! .TRUE. if climatology file fits the model date
LOGICAL   :: LLABORT(YSD_VXD%NUMFLDS)           ! .TRUE. to abort if field is missing
REAL(KIND=JPRB)    :: ZALFA                     ! coefficient for time interpolation
REAL(KIND=JPRB)    :: ZFIELDBUF(NGPTOTG)        ! global field buffer

INTEGER(KIND=JPIM), ALLOCATABLE :: IFILE(:)     ! Identificator of files to read
INTEGER(KIND=JPIM), ALLOCATABLE :: IUNTIN(:)    ! unit number of files to read
INTEGER(KIND=JPIM), ALLOCATABLE :: IDATEF(:,:)  ! date of files to read
INTEGER(KIND=JPIM), ALLOCATABLE :: IINDEX(:)    ! start index in files
INTEGER(KIND=JPIM), ALLOCATABLE :: IMMCLI(:)    ! month of the files
REAL(KIND=JPRB),    ALLOCATABLE :: ZFIELDG(:,:) ! global field read from climatology files
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "disgrid.intfb.h"
#include "openfa.intfb.h"
#include "rdgpfa.intfb.h"
#include "sutimincli.intfb.h"

#include "fcttim.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRCLIA',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    OPEN AND CHECK FILES
!              --------------------

LCLIM=.TRUE.

IF (MYPROC <= NSTRIN) THEN
  IF (LDINT) THEN
    IDIM=2
    ALLOCATE(IFILE(IDIM))
    IFILE(1)=17
    IFILE(2)=18
  ELSE
    IDIM=1
    ALLOCATE(IFILE(IDIM))
    IFILE(:)=10
  ENDIF
  ALLOCATE(IUNTIN(IDIM))
  ALLOCATE(IINDEX(IDIM))
  ALLOCATE(IDATEF(11,IDIM))
  ALLOCATE(IMMCLI(IDIM))
  ALLOCATE(ZFIELDG(NGPTOTG,IDIM))
  DO J=1,IDIM
    CALL OPENFA(IFILE(J),IUNTIN(J),IDEB,IFIN,IINDEX(J),IDATEF(1,J))
    IF (IUNTIN(J) > 0) THEN
!     File exists, set the month
      IMMCLI(J)=IDATEF(2,J)
    ELSE
!     Set an impossible month
      IMMCLI(J)=-999
    ENDIF
  ENDDO
  IF (LDINT) THEN 
    IYEAR=NCCAA(NINDAT)
    IMONTH=NMM(NINDAT)
    IDAY=NDD(NINDAT)
    CALL SUTIMINCLI(IDAY,IMONTH,IYEAR,IMMCLI,NULOUT,LLFILE,ZALFA)
  ELSE
    LLFILE(:)=.TRUE.
  ENDIF
  KLI=COUNT(LLFILE)
ENDIF
IF (NSTRIN < NPROC) THEN
  CALL MPL_BROADCAST(KLI,MTAGPART,1,CDSTRING='SUGRCLIA:')
ENDIF

!*     2.    SETUP DISTRIBUTION OF FIELDS
!            ----------------------------

DO JV=1,YSD_VXD%NUMFLDS
  LLABORT(JV) = YSD_VX%YVX(JV)%CNAME /= ' ' .AND. .NOT.LMCC01
ENDDO

ICOUNT=0
DO JV=1,YSD_VXD%NUMFLDS
  IAGPIO(JV)=MOD(JV-1,NSTRIN)+1
  IF (IAGPIO(JV) == MYPROC) THEN
    ICOUNT=ICOUNT+1
  ENDIF
ENDDO
IF (LOUTPUT) THEN
  WRITE(NULOUT,*) 'SUGRCLIA : ICOUNT = ',ICOUNT, ' IAGPIO = '
  WRITE(NULOUT,'(15(1X,I4))') IAGPIO
ENDIF

!*     3.    READ AND DISTRIBUTE FIELDS
!            --------------------------

DO JV=1,YSD_VXD%NUMFLDS
  IF (IAGPIO(JV) == MYPROC) THEN
    DO J=1,IDIM
      IF (LLFILE(J)) THEN
        INDEX=IINDEX(J)
        CALL RDGPFA(IUNTIN(J),YSD_VX%YVX(JV)%CNAME,IDEB,IFIN,INDEX,ZFIELDG(1,J),&
         & JV,0,0,'CLIMATE','VCLIX',LLABORT(JV))  
      ENDIF
    ENDDO
    IF (ALL(LLFILE).AND.LDINT) THEN
      ZFIELDBUF(:)=ZALFA*ZFIELDG(:,1) + (1.0_JPRB-ZALFA)*ZFIELDG(:,2)
    ELSEIF (LLFILE(1)) THEN
      ZFIELDBUF(:)=ZFIELDG(:,1)
    ELSEIF (LLFILE(2)) THEN
      ZFIELDBUF(:)=ZFIELDG(:,2)
    ELSE
      ZFIELDBUF(:)=0.0_JPRB
      LCLIM=.FALSE.
    ENDIF
    IF (NPROC > 1) THEN
      CALL DISGRID(JV,0,NGPTOTG,ZFIELDBUF,LDC=.TRUE.,KIOPROC=IAGPIO(JV))
    ENDIF
  ELSE 
    CALL DISGRID(JV,1,NGPTOT,ZFIELDBUF,LDC=.TRUE.,KIOPROC=IAGPIO(JV))
  ENDIF
  DO JSTGLO=1,NGPTOT,NPROMA
    IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
    IBL=(JSTGLO-1)/NPROMA+1
    DO JL=1,IEND
      SD_VX(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
    ENDDO
  ENDDO
ENDDO

!*     4.  CLOSE FILES
!          ----------

IF (MYPROC <= NSTRIN) THEN
  DO J=1,IDIM
    CALL FAIRME(IREP,IUNTIN(J),'UNKNOWN')
  ENDDO
  DEALLOCATE(IFILE)
  DEALLOCATE(IUNTIN)
  DEALLOCATE(IINDEX)
  DEALLOCATE(IDATEF)
  DEALLOCATE(IMMCLI)
  DEALLOCATE(ZFIELDG)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRCLIA',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRCLIA
