SUBROUTINE SUGRIDA(KFILE)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,&
 & NFLEVG   ,NDGUXG   ,NDLUXG  ,NPROMA   ,NDGENL   ,NDGSAL
USE SURFACE_FIELDS_MIX, ONLY : &
 & SP_SB, YSP_SBD, YSP_SB, SP_SG, YSP_SGD, YSP_SG, &
 & SP_RR, YSP_RRD, YSP_RR, SD_WS, YSD_WSD, YSD_WS, &
 & SD_WW, YSD_WWD, YSD_WW, &
 & SP_EP, YSP_EPD, YSP_EP, SP_X2, YSP_X2D, YSP_X2, &
 & SD_VF, YSD_VFD, YSD_VF, SD_VP, YSD_VPD, YSD_VP, &
 & SP_CI, YSP_CID, YSP_CI, SD_VV, YSD_VVD, YSD_VV, &
 & SD_VA, YSD_VAD, YSD_VA, SD_VC, YSD_VCD, YSD_VC, &
 & SP_CL, YSP_CLD, YSP_CL
USE YOMDPHY  , ONLY : NVCLIS   ,NTOZ2D   ,NTOZ3D
USE YOMGEM   , ONLY : NLOENG   ,NSTTYP   ,NGPTOT   ,NGPTOTG
USE YOMCT0   , ONLY : NCONF    ,LELAM    
USE YOMDAG   , ONLY : NDATGU   ,NRESGU   ,NECHGU   ,NTYPGU
USE YOMOP    , ONLY : NAGDIO   ,MAXFIELD  
USE YOMMCC   , ONLY : LMCC01
USE YOMMP    , ONLY : MYPROC   ,MY_REGION_NS   ,NSTRIN   ,NFRSTLAT ,NLSTLAT, &
 & NONL     ,NPTRFLOFF,MY_REGION_EW
USE YOMOZO   , ONLY : TOZ2DL   ,TOZ3DBL
USE YOMPHY   , ONLY : LOZONE

!**** *SUGRIDA*  - Initialize the gridpoint fields from *FA*

!     Purpose.
!     --------
!           Initialize the gridpoint fields of the model from FA.

!**   Interface.
!     ----------
!        *CALL* *SUGRIDA(...)*

!        Explicit arguments : KFILE - indicator for which file is to be read
!        ------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.     RDGPFA - read ARPEGE field
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        MPP Group *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 96-04-30 (From SUGRIDA)
!                   97-11-15 D. Giard : cleaning
!        Modified : 97-12-02 R. El Khatib (RDGPFA)
!        R. EL Khatib : 98-04-15 Move print out of NFLDPTP to SUPHYDS
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - removal of LRPOLE in YOMCT0 and in 'CALL CHIEN'.
!         - removal of LLPOLE and code under LRPOLE, LLPOLE.
!        D. Giard     : 98-09-07 KFILE=8 for ALADIN
!        D. Giard     : 00-07-11 KFILE=8 as temporary file in finalisation
!        Y. Bouteloup : 02-03-29 xxO3ABC => Climatological ozone profiles
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Marquet   : 02-07-17 add VCLIA for aerosol files
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        A. Alias     : 05-10-10 GMGEC/EAC Modifs
!        P. Marquet   : add *VCLIS for ozone forcing files 
!        M. Deque     : allows NVCLIS=1 with LOZONE=.F.
!        K. Yessad    : read surface trajectory files (LTRAJHR_SURF=T).
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        R. El Khatib : 26-Feb-2008 Bugfix
!        R. ElKhatib   24-Oct-2008 Merge YEMOP and YOMOP, USE OPENFA
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KFILE 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZFIELD(NGPTOT,NVCLIS)
REAL(KIND=JPRB) :: ZFIELDBUF(NGPTOTG)
REAL(KIND=JPRB) :: ZREALG(NDLON*(NDGLG+2))
INTEGER(KIND=JPIM) :: IFLD(MAXFIELD)
INTEGER(KIND=JPIM) :: IDATEF(11)
CHARACTER(LEN = 16) :: CLIDEN
CHARACTER(LEN=20) :: CLSUO3*20

INTEGER(KIND=JPIM) :: IDEB
INTEGER(KIND=JPIM) :: IFIN
INTEGER(KIND=JPIM) :: IINDEX, ILEN, &
 & INDEX, INUM, INUMLAT, IREP, IROF, IUNTIN, &
 & JC, JGL, JLON, JNUM, JV, JFLEV, ICH, JROF, IFIELD, &
 & IGP

INTEGER(KIND=JPIM) :: JSTGLO,IEND,IBL,JL 

REAL(KIND=JPRB) :: ZMIN,ZMOY,ZMAX

LOGICAL :: LLABORT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "openfa.intfb.h"
#include "disgrid.intfb.h"
#include "rdgpfa.intfb.h"
#include "suarpio.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    OPEN AND CHECK FILE AND SEND IT.
!              -------------------------------

IGP=(YSP_SBD%NLEVS*YSP_SBD%NUMFLDS)+YSP_SGD%NUMFLDS+YSP_RRD%NUMFLDS+ &
 &  (YSP_EPD%NUMFLDS*YSP_EPD%NLEVS)+YSP_X2D%NUMFLDS+YSP_CID%NUMFLDS+ &
 &  YSD_VFD%NUMFLDS+YSD_VPD%NUMFLDS+YSD_VVD%NUMFLDS+YSD_VAD%NUMFLDS+ &
 &  YSD_VCD%NUMFLDS+YSD_WSD%NUMFLDS

! If IGP=0, nothing is done in SUGRIDA.
IF (IGP > 0) THEN

CALL SUARPIO(1)
CALL FANMSG(0)
INUMLAT=NLSTLAT(MY_REGION_NS)-NFRSTLAT(MY_REGION_NS)+1

!*       1.1   OPEN FILE (ONLY for NSTRIN PROCS).

IF (MYPROC <= NSTRIN) THEN

  CALL OPENFA(KFILE,IUNTIN,IDEB,IFIN,IINDEX,IDATEF)            

  IF (NCONF == 701) THEN
    CALL FALSIF(IREP,IUNTIN,CLIDEN)
    IF (CLIDEN(1:3) == 'CLI') THEN
      NTYPGU=2
    ELSEIF (CLIDEN(1:3) == 'ARP') THEN
      NTYPGU=3
    ELSEIF (CLIDEN(1:3) == 'EME') THEN
      NTYPGU=4
    ELSEIF (CLIDEN(1:3) == 'CEP') THEN
      NTYPGU=5
    ELSE
      NTYPGU=1
      WRITE(UNIT=NULOUT,FMT='(A45,A16)')&
       & ' TYPE D IDENTIFICATEUR NON PREVU : CLIDEN  : ',CLIDEN  
    ENDIF
    NDATGU=IDATEF(1)*10000+IDATEF(2)*100+IDATEF(3)
    NRESGU=IDATEF(4)*100+IDATEF(5)
    IF (IDATEF(9) == 10.OR.IDATEF(9) == 0) THEN
      IF (IDATEF(7) == 0) THEN
        WRITE(UNIT=NULOUT,FMT='(A45)')&
         & ' LE GUESS EST UNE ANALYSE NON INITIALISEE    '  
        NECHGU=0
      ELSE
        NECHGU=IDATEF(7)
      ENDIF
    ELSEIF (IDATEF(9) == 1) THEN
      IF (IDATEF(7) /= 0) THEN
        WRITE(UNIT=NULOUT,FMT='(A45)')&
         & ' INCOHERENCE ON A UNE ANALYSE                '  
        CALL ABOR1('SUGRIDA: ABOR1 CALLED')
      ENDIF
      WRITE(UNIT=NULOUT,FMT='(A45)')&
       & ' LE GUESS EST UNE ANALYSE INITIALISEE        '  
      NECHGU=0
    ELSE
      CALL ABOR1('SUGRIDA: ABOR1 CALLED')
    ENDIF
  ENDIF

ENDIF   !  MYPROC <= NSTRIN

!     ------------------------------------------------------------------
!*       3.    READ IN PROGNOSTIC FIELDS.
!              -------------------------

INUM=0


!*       3.2   SOILB

DO JC=1,YSP_SBD%NLEVS
  DO JV=1,YSP_SBD%NUMFLDS
    INDEX=IINDEX
    IF (YSP_SB%YSB(JV)%CNAME(JC) /= '                ') THEN
      INUM=INUM+1
      IF (INUM <= MAXFIELD) THEN
        IF (NAGDIO(1,INUM) == MYPROC) THEN
          CALL RDGPFA(IUNTIN,YSP_SB%YSB(JV)%CNAME(JC),IDEB,IFIN,INDEX,&
           & ZFIELDBUF,JC,JV,0,'ARPEGE',&
           & 'SOILB',.TRUE.)  
          CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
        ELSE
          CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
        ENDIF
        DO JSTGLO=1,NGPTOT,NPROMA
          IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
          IBL=(JSTGLO-1)/NPROMA+1
          DO JL=1,IEND
            SP_SB(JL,JC,JV,IBL) =  ZFIELDBUF(JSTGLO+JL-1)
          ENDDO
        ENDDO
      ELSE
        WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
        CALL ABOR1('SUGRIDA : ABOR1 CALLED')
      ENDIF
    ENDIF
  ENDDO
ENDDO


!*       3.5   SNOWG

DO JV=1,YSP_SGD%NUMFLDS
  INDEX=IINDEX
  IF (YSP_SG%YSG(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        IF (JV /= 4) THEN
          LLABORT=.TRUE.
        ELSE
          LLABORT=.FALSE.
        ENDIF
        CALL RDGPFA(IUNTIN,YSP_SG%YSG(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'SNOWG',LLABORT)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SP_SG(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.6   RESVR

DO JV=1,YSP_RRD%NUMFLDS
  INDEX=IINDEX
  IF (YSP_RR%YRR(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSP_RR%YRR(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'RESVR',.TRUE.)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SP_RR(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.7   CLS

DO JV=1,YSP_CLD%NUMFLDS
    INDEX=IINDEX
    IF (YSP_CL%YCL(JV)%CNAME /= '                ') THEN
      INUM=INUM+1
      IF (INUM <= MAXFIELD) THEN
        IF (NAGDIO(1,INUM) == MYPROC) THEN
          CALL RDGPFA(IUNTIN,YSP_CL%YCL(JV)%CNAME,IDEB,IFIN,INDEX,&
           & ZFIELDBUF,JV,0,0,'ARPEGE',&
           & 'RESVR',.TRUE.)
          CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
        ELSE
          CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
        ENDIF
        DO JSTGLO=1,NGPTOT,NPROMA
          IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
          IBL=(JSTGLO-1)/NPROMA+1
           DO JL=1,IEND
            SP_CL(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
          ENDDO
        ENDDO
      ELSE
        WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
        CALL ABOR1('SUGRIDA : ABOR1 CALLED')
      ENDIF
    ENDIF
ENDDO

!*       3.8   EXTRP

DO JV=1,YSP_EPD%NUMFLDS
  DO JC=1,YSP_EPD%NLEVS
    INDEX=IINDEX
    IF (YSP_EP%YEP(JV)%CNAME(JC) /= '                ') THEN
      INUM=INUM+1
      IF (INUM <= MAXFIELD) THEN
        IF (NAGDIO(1,INUM) == MYPROC) THEN
          CALL RDGPFA(IUNTIN,YSP_EP%YEP(JV)%CNAME(JC),IDEB,IFIN,INDEX,&
           & ZFIELDBUF,JC,JV,0,'ARPEGE',&
           & 'EXTRP',.TRUE.)  
          CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
        ELSE
          CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
        ENDIF
        DO JSTGLO=1,NGPTOT,NPROMA
          IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
          IBL=(JSTGLO-1)/NPROMA+1
          DO JL=1,IEND
            SP_EP(JL,JC,JV,IBL) =  ZFIELDBUF(JSTGLO-1+JL)
          ENDDO
        ENDDO
      ELSE
        WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
        CALL ABOR1('SUGRIDA : ABOR1 CALLED')
      ENDIF
    ENDIF
  ENDDO
ENDDO

!*       3.9   XTRP2

DO JV=1,YSP_X2D%NUMFLDS
  INDEX=IINDEX
  IF (YSP_X2%YX2(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        IF (NCONF == 701) THEN
          ILEN=0
          LLABORT=.FALSE.
        ELSE
          LLABORT=.TRUE.
        ENDIF
        CALL RDGPFA(IUNTIN,YSP_X2%YX2(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'XTRP2',LLABORT)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SP_X2(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.10  CANRI

DO JV=1,YSP_CID%NUMFLDS
  INDEX=IINDEX
  INUM=INUM+1
  IF (INUM <= MAXFIELD) THEN
    IF (NAGDIO(1,INUM) == MYPROC) THEN
      LLABORT=.FALSE.
      CALL RDGPFA(IUNTIN,YSP_CI%YCI(JV)%CNAME,IDEB,IFIN,INDEX,&
       & ZFIELDBUF,JV,0,0,'ARPEGE',&
       & 'CANRI',LLABORT)  
      IF (JV == 3) THEN
        ZFIELDBUF(:)=1.0_JPRB
        WRITE(NULOUT,'(''         changed to 1. '')')
        IF (LELAM) THEN
! we won't want to deal with the E-domain ALADIN in CANARI
          IROF=0
          DO JGL=IDEB,NDGUXG
            IROF=IROF+NDLUXG
            DO JLON=NDLUXG+1,NLOENG(JGL)
              IROF=IROF+1
              ZFIELDBUF(IROF)=0.0_JPRB
            ENDDO
          ENDDO
          DO JGL=NDGUXG+1,IFIN
            DO JLON=1,NLOENG(JGL)
              IROF=IROF+1
              ZFIELDBUF(IROF)=0.0_JPRB
            ENDDO
          ENDDO
        ENDIF
      ENDIF
      CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
    ELSE
      CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
    ENDIF
    DO JSTGLO=1,NGPTOT,NPROMA
      IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
      IBL=(JSTGLO-1)/NPROMA+1
      DO JL=1,IEND
        SP_CI(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
      ENDDO
    ENDDO
  ELSE
    WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
    CALL ABOR1('SUGRIDA : ABOR1 CALLED')
  ENDIF
ENDDO

!     ------------------------------------------------------------------
!*       4.    READ IN CLIMATOLOGICAL FIELDS.
!              -----------------------------

!*       4.1   VARSF

DO JV=1,YSD_VFD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_VF%YVF(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_VF%YVF(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'VARSF',.TRUE.)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VF(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       4.2   VCLIP

DO JV=1,YSD_VPD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_VP%YVP(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_VP%YVP(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'VCLIP',.TRUE.)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VP(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       4.3   VCLIV

DO JV=1,YSD_VVD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_VV%YVV(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_VV%YVV(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'VCLIV',.TRUE.)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VV(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       4.4   VCLIA

DO JV=1,YSD_VAD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_VA%YVA(JV)%CNAME /= '                '.AND..NOT.LMCC01) THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_VA%YVA(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'VCLIA',.TRUE.)
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VA(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       4.5   VO3ABC

DO JV=1,YSD_VCD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_VC%YVC(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_VC%YVC(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'VO3ABC',LLABORT)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VC(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       4.6   WAVES

DO JV=1,YSD_WSD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_WS%YWS(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_WS%YWS(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'WAVESS',.TRUE.)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_WS(JL,JV,IBL) =  ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

DO JV=1,YSD_WWD%NUMFLDS
  INDEX=IINDEX
  IF (YSD_WW%YWW(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        CALL RDGPFA(IUNTIN,YSD_WW%YWW(JV)%CNAME,IDEB,IFIN,INDEX,&
         & ZFIELDBUF,JV,0,0,'ARPEGE',&
         & 'WAVESW',.TRUE.)  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_WW(JL,JV,IBL) =  ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO


!     ------------------------------------------------------------------
!*       5.    VCLIS : OZONE COEFFICIENTS  
!              --------------------------

!*             Be carefull : IFIELD is set to zero again, with
!*             the use of ZREALG(NDLON*(NDGLG+2)) and 
!*             ZFIELD(NGPTOT,NVCLIS) as temporary buffers

IF ((LOZONE.OR.(NVCLIS == 1)).AND.(.NOT.LMCC01)) THEN

  !*  There should be 7 coefficients
  IF (NVCLIS /= 7.AND.LOZONE) THEN 
    WRITE(NULOUT,*) ' WRONG NVCLIS (it must be 7) : ', NVCLIS
    CALL ABOR1('SUGRIDA : ABOR1 CALLED (wrong NVCLIS)')
  ENDIF

  !* Open 3D files if they are not allocated 
  IF (NVCLIS*NTOZ3D > 0) THEN
    IF (.NOT.ALLOCATED(TOZ3DBL)) THEN
      ALLOCATE(TOZ3DBL(NGPTOT,NFLEVG*NVCLIS*NTOZ3D))
      WRITE(NULOUT,9990) 'TOZ3DBL  ',SIZE(TOZ3DBL),SHAPE(TOZ3DBL)
    ENDIF
  ENDIF

  !* Open 2D files if they are not allocated 
  IF (NVCLIS*NTOZ2D > 0) THEN
    IF (.NOT.ALLOCATED(TOZ2DL)) THEN
      ALLOCATE(TOZ2DL(NDGSAL:NDGENL,NFLEVG*NVCLIS*NTOZ2D))
      WRITE(NULOUT,9990) 'TOZ2DL   ',SIZE(TOZ2DL),SHAPE(TOZ2DL)
    ENDIF
  ENDIF

  !* Start the vertical loop for the NFLEVG levels 
  DO JFLEV=1,NFLEVG

    INUM=0
    IFIELD=0
    INDEX=IINDEX+1
    DO JV=1,NVCLIS
      WRITE(UNIT=CLSUO3,FMT='(''OZONE.A'',I1)')JV

      !* Send the data
      IFIELD=IFIELD+1
      INUM=INUM+1
      IF (INUM > MAXFIELD) THEN
        WRITE(NULOUT,*) ' IN SUGRIDA : INUM > MAXFIELD ! ', INUM,MAXFIELD 
        CALL ABOR1           ('SUGRIDA : INUM > MAXFIELD')
      ENDIF
      IFLD(INUM)=IFIELD

      IF (MYPROC == NAGDIO(1,INUM)) THEN
        CALL FACILE(IREP,IUNTIN,'S',JFLEV,CLSUO3,ZREALG,.FALSE.)
        ZFIELDBUF=ZREALG(INDEX:NGPTOTG+INDEX-1)
        WRITE(NULOUT,'('' OZONE.A'',I1,'' LEVEL '',I3,&
         & '' RECEIVED FROM ARPEGE FILE'')') IFIELD,JFLEV  
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
        ZFIELD(:,IFIELD)=ZFIELDBUF(:NGPTOT)
      ENDIF ! MYPROC == NAGDIO(1,INUM)
    ENDDO ! JV=1,NVCLIS

    !* Receive the data
    DO JNUM=1,INUM
      IF (MYPROC /= NAGDIO(1,JNUM)) THEN
        IFIELD=IFLD(JNUM)
        CALL DISGRID(JNUM,1,NGPTOT,ZFIELD(1,IFIELD))
      ENDIF
    ENDDO

    !* Fills the common (3D or 2D)
    DO JV=1,NVCLIS
      ICH=(JV-1)*NFLEVG+JFLEV

      ! FULL 3D COMMON (Longitude x Latitude x Level)
      IF (NTOZ3D == 1) THEN
        DO JROF=1,NGPTOT
          TOZ3DBL(JROF,ICH)=ZFIELD(JROF,JV)
        ENDDO
        ZMIN=TOZ3DBL(1,ICH)
        ZMAX=TOZ3DBL(1,ICH)
        ZMOY=0.0_JPRB
        DO JROF=1,NGPTOT
          ZMOY=ZMOY+TOZ3DBL(JROF,ICH)/REAL(NGPTOT)
          ZMIN=MIN(ZMIN,TOZ3DBL(JROF,ICH))
          ZMAX=MAX(ZMAX,TOZ3DBL(JROF,ICH))
        ENDDO
        WRITE(NULOUT,'('' OZONE-3D A'',I1,'' LEVEL '',I3,&
         & '' MIN/MOY/MAX : '', 3(1x,E12.5) )') &
         & JV,JFLEV,ZMIN,ZMOY,ZMAX
      ENDIF

      ! THE 2D COMMON (Latitude x Level)
      IF (NTOZ2D == 1) THEN
        IF (NSTTYP /= 1) THEN
          WRITE(NULOUT,&
           & FMT='('' SUGRIDA ! OZONE INCONSISTENCY : NTOZ2D='',I4 &
           & ,'' NSTTYP='',I4)') NTOZ2D, NSTTYP  
          CALL ABOR1(' SUGRIDA : ABOR1 CALLED')
        ENDIF
        IROF=1
        DO JGL=1,INUMLAT
          TOZ2DL(JGL,ICH)=ZFIELD(IROF,JV)
          IROF=IROF+NONL(NPTRFLOFF+JGL,MY_REGION_EW)
        ENDDO
        ZMIN=TOZ2DL(1,ICH)
        ZMAX=TOZ2DL(1,ICH)
        ZMOY=0.0_JPRB
        DO JROF=1,INUMLAT
          ZMOY=ZMOY+TOZ2DL(JROF,ICH)/real(inumlat)
          ZMIN=MIN(ZMIN,TOZ2DL(JROF,ICH))
          ZMAX=MAX(ZMAX,TOZ2DL(JROF,ICH))
        ENDDO
        WRITE(NULOUT,'('' OZONE-2D A'',I1,'' LEVEL '',I3,&
         & '' MIN/MOY/MAX : '', 3(1x,E12.5) )') &
         & JV,JFLEV, ZMIN,ZMOY,ZMAX  
      ENDIF
    ENDDO   ! JV=1,NVCLIS
  ENDDO   ! JFLEV=1,NFLEVG
ENDIF   ! LOZONE=.T.

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!     ------------------------------------------------------------------
!*       6.    CLOSE FILE.
!              ----------

IF (MYPROC <= NSTRIN) THEN
  CALL FAIRME(IREP,IUNTIN,'UNKNOWN')
ENDIF

ENDIF ! Test on IGP

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRIDA',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRIDA
