SUBROUTINE SUGRIDF(KFILE,PSPOR,LDSPOR,PBLH)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NGPBLKS
USE YOMGEM   , ONLY : NGPTOT 
USE YOMGC    , ONLY : GEMU, GELAM
USE YOMCT0   , ONLY : N3DINI   ,LARPEGEF ,LCANARI
USE SURFACE_FIELDS_MIX, ONLY : GPOPER, &
 & YSP_SBD, SP_SB, YSP_RRD, SP_RR, YSD_WSD, SD_WS, &
 & YSD_WWD, SD_WW, YSP_SGD, SP_SG, &
 & YSD_VVD, SD_VV, YSD_VAD, SD_VA, YSD_VCD, SD_VC, &
 & YSD_VFD, SD_VF, YSD_VXD
USE YOMDYNCORE, ONLY: LAQUA

!**** *SUGRIDF*  - Routine to initialize the gridpoint fields

!     Purpose.
!     --------
!           Initialize the gridpoint fields of the model.

!**   Interface.
!     ----------
!        *CALL* *SUGRIDF(KFILE)*

!        Explicit arguments :
!        --------------------
!        KFILE : an indicator for which file is to be read
!               KFILE = 0 the CFNISH file is read
!               KFILE = 1 the CFNGSH file is read
!               KFILE = 2 the CFNRF  file is read
!               KFILE = 4 the CFNGG  file is read
!               KFILE = 5 the CFNPL  file is read
!               KFILE = 6 a climat. file is read
!               KFILE = 8 the CFANS  file is read
!        PSPOR
!        LDSPOR
!        PBLH

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.     SUGRIDA - read from ARPEGE file
!     ----------     SUGRIDG - read from GRIB file
!                    SUGRCLIA- read climatology from ARPEGE file
!                    SUGRCLIG- read climatology from GRIB file
!                    SUGRIDVA- read climatology from climat. file
!                    SC2WRG  - write gridpoint workfile (buffer)
!                    MIOBUFR - IO setup routine (MIO package)
!                    MIOSREC - IO setup routine (MIO package)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 91-11-15
!                   92-02-29  J.-J. Morcrette (ECMWF physics)
!                   92-03-25   R. El Khatib ( check file with namelist)
!                   92-04-20  R. E.K. and J.-F. G. (reduced grid, etc..)
!                   92-05-05  W. Maurel and P. Le Moigne (vegetation scheme)
!                   93-05-15  M.Hamrud (read from GRIB file)
!                   93-12-10  F.Rabier (write surface fields for trajectory)
!                   94-03-03  R. El Khatib (Full-POS inplementation)
!                   94-10-07  EB and DG (implementation of ISBA)
!                   95-08-10  DG (modify some climatological fields - ISBA)
!                   97-05-02  DG (modify some more fields - ISBA)
!                   97-08-20  DG (call to SUGRIDVA and so on)
!                   97-12-01  R. El Khatib (arg. IFILE to SUGRCLIA)
!                   97-12-01  R. El Khatib (SM/DM to SUCACLIA, SUGRCLIA & SUGRIDVA)
!                   98-08-10  K. YESSAD: removal of LRPOLE option.
!                             - removal of LRPOLE in YOMCT0.
!                             - removal of LLPOLE and code under LRPOLE, LLPOLE.
!                             - one can read ARPEGE files with or without 
!                               poles, but polar values are never copied 
!                               in arrays ZFIELD, ZGPP, GPPBUF from now on.
!      00-02-16 R. El Khatib Unification of SUGRCLIASM-DM with SUCACLIASM-DM 
!                            => new unified subroutine = SUGRCLIA
!                   00-10-26  M.Fisher : added optional argument PBLH
!                   00-10-25  DG (albedo of vegetation)
!                   02-03-29  Y. Bouteloup : Climatological ozone profiles
!                   03-10-01  M.Hamrud : CY28 Cleaning
!                   02-07-17  P. Marquet : add VCLIA for aerosol files
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!      06-01-06 A. Alias Sulfate and volcano aerosols added
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        R. El Khatib  26-Nov-2008  Cleaning
!     ----------------------------------------------------------------------

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPOR(:) 
LOGICAL           ,INTENT(OUT)   :: LDSPOR 
REAL(KIND=JPRB)   ,OPTIONAL,INTENT(OUT)   :: PBLH(NGPTOT) 

!     ----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JV, ICLI  
INTEGER(KIND=JPIM) :: JBL

LOGICAL :: LLINT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ----------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gpnorm2.intfb.h"
#include "gpnorm3.intfb.h"
#include "sugrclia.intfb.h"
#include "sugrida.intfb.h"
#include "sugridg.intfb.h"
#include "sugridva.intfb.h"
#include "gp_sstaqua.intfb.h"

!     ------------------------------------------------------------------
!*       1.    SET DEFAULT VALUES FOR FIELDS.
!              ------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDF',0,ZHOOK_HANDLE)
CALL GSTATS(26,0)

DO JBL=1,NGPBLKS
  CALL GPOPER('SETDEFAULT',KBL=JBL)
ENDDO
!     ------------------------------------------------------------------
!*       3.    READ IN FIELDS FROM ARPEGE FILE.
!              -------------------------------

IF (LARPEGEF) THEN
  IF (KFILE /= 6) THEN
    CALL SUGRIDA(KFILE)
    IF (YSD_VXD%NUMFLDS > 0) THEN
      LLINT=LCANARI 
      CALL SUGRCLIA(LLINT,ICLI)
      IF ((LLINT .AND. ICLI /=2).AND..NOT.LCANARI) THEN
        WRITE(NULOUT,'('' LLINT = '',L2, '' ICLI = '',I2,&
         & '' LCANARI = '',L2)')  LLINT,ICLI,LCANARI  
        CALL ABOR1('SUGRIDF : SUGRCLIA FAILED')
      ENDIF
    ENDIF
  ELSE
    CALL SUGRIDVA
  ENDIF
ENDIF

!     ------------------------------------------------------------------
!*       4.    READ IN FIELDS FROM GRIB FILE.
!              -----------------------------

IF (.NOT.LARPEGEF.AND.(N3DINI == 0 .OR. N3DINI == 3 )) THEN
  IF (PRESENT(PBLH)) THEN
    CALL SUGRIDG(KFILE,LDSPOR,PBLH=PBLH)
  ELSE
    CALL SUGRIDG(KFILE,LDSPOR)
  ENDIF
ENDIF

IF (LAQUA) THEN
  CALL GP_SSTAQUA(GEMU,GELAM,SP_SB,SP_SG,SP_RR,SD_VF)
ENDIF

!*       7.    GRID-POINT NORMS
!              ----------------

WRITE(NULOUT,'(A)') ' SUGRIDF: STATISTICS FOR ALL SURFACE FIELDS'
IF (YSP_SBD%NUMFLDS>0) THEN
  DO JV=1,YSP_SBD%NUMFLDS
    WRITE(NULOUT,'(A,I4)') ' SOILB, FIELD ',JV
    CALL GPNORM3(SP_SB,YSP_SBD%NUMFLDS,JV,KLEVS=YSP_SBD%NLEVS,LDLEVELS=.TRUE.)
  ENDDO
ENDIF
IF (YSP_SGD%NUMFLDS>0) THEN
  WRITE(NULOUT,'(A)') ' SNOWG '
  CALL GPNORM2(YSP_SGD%NUMFLDS,1,SP_SG)
ENDIF
IF (YSP_RRD%NUMFLDS>0) THEN
  WRITE(NULOUT,'(A)') ' RESVR '
  CALL GPNORM2(YSP_RRD%NUMFLDS,1,SP_RR)
ENDIF
IF (YSD_WSD%NUMFLDS>0) THEN
  WRITE(NULOUT,'(A)') ' FROM WAM '
  CALL GPNORM2(YSD_WSD%NUMFLDS,1,SD_WS)
ENDIF
IF (YSD_WWD%NUMFLDS>0) THEN
  WRITE(NULOUT,'(A)') ' TO WAM '
  CALL GPNORM2(YSD_WWD%NUMFLDS,1,SD_WW)
ENDIF

    
IF (YSD_VFD%NUMFLDS > 0) THEN
  WRITE(NULOUT,'(A)') ' VARSF'
  CALL GPNORM2(YSD_VFD%NUMFLDS,1,SD_VF)
ENDIF
IF (YSD_VVD%NUMFLDS > 0) THEN
  WRITE(NULOUT,'(A)') ' VCLIV'
  CALL GPNORM2(YSD_VVD%NUMFLDS,1,SD_VV)
ENDIF
IF (YSD_VAD%NUMFLDS > 0) THEN
  WRITE(NULOUT,'(A)') ' VCLIA'
  CALL GPNORM2(YSD_VAD%NUMFLDS,1,SD_VA)
ENDIF
IF (YSD_VCD%NUMFLDS > 0) THEN
  WRITE(NULOUT,'(A)') ' VO3ABC'
  CALL GPNORM2(YSD_VCD%NUMFLDS,1,SD_VC)
ENDIF

CALL GSTATS(26,1)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRIDF',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRIDF
