SUBROUTINE SUGRIDO(KFILE)

!**** *SUGRIDO*  - Initialize ocean mixed layer model gridpoint fields

!     Purpose.
!     --------
!           Initialize ocean mixed layer model gridpoint fields 
!           from GRIB file.           

!**   Interface.
!     ----------
!        *CALL* *SUGRIDO

!        Explicit arguments :
!        ------------------
!        KFILE : an indicator for which  file is to be read
!                the CFNIGG-OC  file is read

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Y.Takaya *ECMWF*

!     Modifications.
!     --------------
!        Original : 01-Feb-2009  Y.Takaya (modified from SUGRIDUG)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK
USE YOMDIM   , ONLY : NPROMA
USE YOMLUN   , ONLY : NULOUT 
USE YOMGEM   , ONLY : NGPTOT
USE YOMOP    , ONLY : CFNIGG

USE SURFACE_FIELDS_MIX, ONLY : SP_OM    ,YSP_OM, YSP_OMD,&
 & SD_V2    ,YSD_V2   ,YSD_V2D  ,SD_V3    ,YSD_V3   ,YSD_V3D

USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, CLOSE_IOREQUEST

IMPLICIT NONE

! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)       :: KFILE 

INTEGER(KIND=JPIM) :: IGRIB3D(YSP_OMD%NUMFLDS+YSD_V3D%NUMFLDS)
INTEGER(KIND=JPIM) :: ILEVS3D(YSP_OMD%NUMFLDS+YSD_V3D%NUMFLDS)
INTEGER(KIND=JPIM) :: IGRIB2D(YSD_V2D%NUMFLDS)
INTEGER(KIND=JPIM) :: ILEVS2D(YSD_V2D%NUMFLDS)
INTEGER(KIND=JPIM) :: ICL3D, JSTGLO, ICEND, IBL, ICL3D_2
INTEGER(KIND=JPIM) :: ICL2D, ICL2D_2
INTEGER(KIND=JPIM) :: JLEV, J, JFL, JV, JZ
INTEGER(KIND=JPIM) :: ILEVO

REAL(KIND=JPRB),ALLOCATABLE :: ZZ3D(:,:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZ2D(:,:,:)

REAL(KIND=JPRB),ALLOCATABLE :: ZSUMH(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZHSUM(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZDFAC(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSK(:)

REAL(KIND=JPRB) :: ZDSCALE_KPP
REAL(KIND=JPRB) :: ZVC_KPP
REAL(KIND=JPRB) :: ZVD_KPP
REAL(KIND=JPRB) :: ZVE_KPP
REAL(KIND=JPRB) :: ZVDEL_KPP

REAL(KIND=JPRB) :: ZX1
REAL(KIND=JPRB) :: ZX2
REAL(KIND=JPRB) :: ZA
REAL(KIND=JPRB),ALLOCATABLE :: ZSUM(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZB(:)

LOGICAL :: LLVL_NEW_KPP

CHARACTER :: CLFN*16

TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gpnorm2.intfb.h"
#include "gpnorm3.intfb.h"

INTERFACE
#include "surf_inq.h"
END INTERFACE


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDO',0,ZHOOK_HANDLE)

!*       1.    INQUIRE CONSTANTS OF SURF MODULE.
!              ---------------------------------

!USE YOS_OCEAN_ML , ONLY : RDSCALE_KPP, RVC_KPP, RVD_KPP, RVE_KPP, &
! & RVDEL_KPP, LVL_NEW_KPP

!*       2.    READ IN GRID-POINT FIELDS FROM GRIB DATA.
!              -----------------------------------------

CALL SURF_INQ(PRDSCALE_KPP=ZDSCALE_KPP,PRVC_KPP=ZVC_KPP,PRVD_KPP=ZVD_KPP, &
           &  PRVE_KPP=ZVE_KPP,PRVDEL_KPP=ZVDEL_KPP,LDVL_NEW_KPP=LLVL_NEW_KPP)

ICL3D=0
IF( YSP_OMD%NUMFLDS > 0 ) THEN
  DO JFL=1,YSP_OMD%NUMFLDS
    IF(YSP_OM%YOM(JFL)%NREQIN(1) == 2) THEN
      ICL3D = ICL3D + 1
      IGRIB3D(ICL3D) = YSP_OM%YOM(JFL)%IGRBCODE(1)
      ILEVS3D(ICL3D) = YSP_OMD%NLEVS
    ENDIF
  ENDDO
ENDIF
IF( YSD_V3D%NUMFLDS > 0 ) THEN
  DO JFL=1,YSD_V3D%NUMFLDS
    IF(YSD_V3%YV3(JFL)%NREQIN(1) == 2) THEN
      ICL3D = ICL3D + 1
      IGRIB3D(ICL3D) = YSD_V3%YV3(JFL)%IGRBCODE(1)
      ILEVS3D(ICL3D) = YSD_V3D%NLEVS
    ENDIF
  ENDDO
ENDIF

ICL2D=0
IF( YSD_V2D%NUMFLDS > 0 ) THEN
  DO JFL=1, YSD_V2D%NUMFLDS 
    IF(YSD_V2%YV2(JFL)%NREQIN == 2) THEN
      ICL2D = ICL2D + 1 
      IGRIB2D(ICL2D) = YSD_V2%YV2(JFL)%IGRBCODE 
      ILEVS2D(ICL2D) = 1
    ENDIF
  ENDDO
ENDIF

ALLOCATE (ZZ3D(NGPTOT,YSP_OMD%NLEVS,ICL3D,1))
ALLOCATE (ZZ2D(NGPTOT,ICL2D,1))

WRITE(UNIT=NULOUT,FMT='(A)')' READ OCEAN MIXED LAYER MODEL FIELDS FROM GRIB '

CLFN=CFNIGG(1:12)//'OC'
WRITE(UNIT=NULOUT,FMT='(A,A)')&
 & '  READ OCEAN MIXED LAYER MODEL FIELDS FROM GRIB ',CLFN  

CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r',KIOMASTER=1)
CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
 & KGRIB2D=IGRIB2D(1:ICL2D),KLEVS2D=ILEVS2D(1:ICL2D),&
 & KGRIB3D=IGRIB3D(1:ICL3D),KLEVS3D=ILEVS3D(1:ICL3D),KPROMA=NGPTOT)
CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=ZZ2D(:,1:ICL2D,:),PR4=ZZ3D(:,:,1:ICL3D,:))
CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)


!*       3. PUT TO 3D FIELD
!           ---------------

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,ICEND,JLEV,J,&
!$OMP& IBL)
DO JSTGLO=1,NGPTOT,NPROMA
  ICEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
  IBL   = (JSTGLO-1)/NPROMA+1
  ICL3D_2 = 0
  DO JFL=1,YSP_OMD%NUMFLDS
    IF(YSP_OM%YOM(JFL)%NREQIN(1) == 2) THEN 
      ICL3D_2 = ICL3D_2+1
      DO JLEV=1,YSP_OMD%NLEVS
        DO J=1,ICEND
          SP_OM(J,JLEV,YSP_OM%YOM(JFL)%MP,IBL) = ZZ3D(JSTGLO+J-1,JLEV,ICL3D_2,1)
        ENDDO
      ENDDO
    ELSE
      SP_OM(:,:,YSP_OM%YOM(JFL)%MP,IBL) = 0.0_JPRB
    ENDIF
  ENDDO
  DO JFL=1,YSD_V3D%NUMFLDS
    IF(YSD_V3%YV3(JFL)%NREQIN(1) == 2) THEN
      ICL3D_2 = ICL3D_2+1
      DO JLEV=1,YSD_V3D%NLEVS
        DO J=1,ICEND
          SD_V3(J,JLEV,YSD_V3%YV3(JFL)%MP,IBL) = ZZ3D(JSTGLO+J-1,JLEV,ICL3D_2,1)
        ENDDO
      ENDDO
    ELSE
      SD_V3(:,:,YSD_V3%YV3(JFL)%MP,IBL) = 0.0_JPRB
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO


!*       4. PUT TO 2D FIELD
!           ---------------

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,ICEND,J,&
!$OMP& IBL)
DO JSTGLO=1,NGPTOT,NPROMA
  ICEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
  IBL   = (JSTGLO-1)/NPROMA+1
  ICL2D_2 = 0
  DO JFL=1,YSD_V2D%NUMFLDS
    IF(YSD_V2%YV2(JFL)%NREQIN == 2) THEN 
      ICL2D_2 = ICL2D_2+1
      DO J=1,ICEND
        SD_V2(J,YSD_V2%YV2(JFL)%MP,IBL) = ZZ2D(JSTGLO+J-1,ICL2D_2,1)
      ENDDO
    ELSE
      SD_V2(:,YSD_V2%YV2(JFL)%MP,IBL) = 0.0_JPRB
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO


!*       5. SET VERTICAL LEVELS
!           ------------------- 

ILEVO=YSD_V3D%NLEVS-1
ALLOCATE (ZSUMH(NGPTOT)) 
ALLOCATE (ZHSUM(NGPTOT)) 
ALLOCATE (ZDFAC(NGPTOT)) 
ALLOCATE (ZSK(NGPTOT)) 
ALLOCATE (ZSUM(NGPTOT))
ALLOCATE (ZB(NGPTOT))

IF(LLVL_NEW_KPP) THEN

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,ICEND,J,& 
!$OMP& IBL) 
  DO JSTGLO=1,NGPTOT,NPROMA 

    ICEND = MIN(NPROMA,NGPTOT-JSTGLO+1) 
    IBL   = (JSTGLO-1)/NPROMA+1 

    ZA = ZVDEL_KPP * REAL(ILEVO)
    DO J=1,ICEND
      DO JZ=1,YSD_V3D%NLEVS
        SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) = 0.0_JPRB 
        SD_V3(J,JZ,YSD_V3%YZO%MP,IBL) = 0.0_JPRB
        SD_V3(J,JZ,YSD_V3%YDO%MP,IBL) = 0.0_JPRB
      ENDDO

      ZB(JSTGLO+J-1) = SD_V2(J,YSD_V2%YOCDEP%MP,IBL) - ZA - ZVD_KPP
      ZSUM(JSTGLO+J-1) = 0.0_JPRB
    ENDDO

    DO J=1,ICEND         
      DO JZ=1,ILEVO
        ZX1 = REAL(JZ)/REAL(ILEVO)
        ZX2 = REAL(JZ-1)/REAL(ILEVO-1)

        SD_V3(J,JZ+1,YSD_V3%YDO%MP,IBL) = ZA*ZX1 &
               & + ZB(JSTGLO+J-1)*(ZX2**ZVC_KPP) &
               & + ZVD_KPP*(ZX2**ZVE_KPP)
        SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) = SD_V3(J,JZ+1,YSD_V3%YDO%MP,IBL) &
               & - ZSUM(JSTGLO+J-1)
        SD_V3(J,JZ,YSD_V3%YHO_INV%MP,IBL) = 1.0_JPRB / SD_V3(J,JZ,YSD_V3%YHO%MP,IBL)
        SD_V3(J,JZ,YSD_V3%YZO%MP,IBL) = - ( ZSUM(JSTGLO+J-1) &
               & + 0.5_JPRB*SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) )
        ZSUM(JSTGLO+J-1) = ZSUM(JSTGLO+J-1) + SD_V3(J,JZ,YSD_V3%YHO%MP,IBL)
      ENDDO
    ENDDO

    DO J=1,ICEND
      SD_V3(J,ILEVO+1,YSD_V3%YHO%MP,IBL) = 1.E-10_JPRB ! small number
      SD_V3(J,ILEVO+1,YSD_V3%YHO_INV%MP,IBL) = 1.0_JPRB / SD_V3(J,ILEVO+1,YSD_V3%YHO%MP,IBL)
      SD_V3(J,ILEVO+1,YSD_V3%YZO%MP,IBL) = - SD_V2(J,YSD_V2%YOCDEP%MP,IBL)
    ENDDO

  ENDDO
!$OMP END PARALLEL DO

ELSE ! Stretched grid of Large et al. (1994)

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,ICEND,J,& 
!$OMP& IBL) 
  DO JSTGLO=1,NGPTOT,NPROMA 

    ICEND = MIN(NPROMA,NGPTOT-JSTGLO+1) 
    IBL   = (JSTGLO-1)/NPROMA+1 

    DO J=1,ICEND
      DO JZ=1,YSD_V3D%NLEVS
        SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) = 0.0_JPRB 
        SD_V3(J,JZ,YSD_V3%YZO%MP,IBL) = 0.0_JPRB
        SD_V3(J,JZ,YSD_V3%YDO%MP,IBL) = 0.0_JPRB
      ENDDO
      IF ( ZDSCALE_KPP > 0.0_JPRB ) THEN
        ZSUMH(JSTGLO+J-1) = 0.0_JPRB
        ZDFAC(JSTGLO+J-1) = 1.0_JPRB - EXP( -ZDSCALE_KPP )
        DO JZ = 1, ILEVO
          ZSK(JSTGLO+J-1) = - ( REAL( JZ ) - 0.5_JPRB ) / REAL( ILEVO )
          SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) = SD_V2(J,YSD_V2%YOCDEP%MP,IBL) &
               & * ZDFAC(JSTGLO+J-1) / REAL( ILEVO ) &
               & / ZDSCALE_KPP / ( 1.0_JPRB + ZSK(JSTGLO+J-1) * ZDFAC(JSTGLO+J-1) )
          ZSUMH(JSTGLO+J-1) = ZSUMH(JSTGLO+J-1) + SD_V3(J,JZ,YSD_V3%YHO%MP,IBL)
        ENDDO
      ENDIF

      ZHSUM(JSTGLO+J-1) = 0.0_JPRB
      SD_V3(J,1,YSD_V3%YDO%MP,IBL) = 0.0_JPRB
 
      DO JZ = 1, ILEVO
        IF( ZDSCALE_KPP > 0.0_JPRB ) THEN
          SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) = SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) &
                    & * SD_V2(J,YSD_V2%YOCDEP%MP,IBL) / ZSUMH(JSTGLO+J-1)
        ELSE
          SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) = SD_V2(J,YSD_V2%YOCDEP%MP,IBL) / REAL(ILEVO)
        ENDIF
        SD_V3(J,JZ,YSD_V3%YHO_INV%MP,IBL) = 1.0_JPRB / SD_V3(J,JZ,YSD_V3%YHO%MP,IBL)
        SD_V3(J,JZ,YSD_V3%YZO%MP,IBL) =  - ( ZHSUM(JSTGLO+J-1) + 0.5_JPRB * SD_V3(J,JZ,YSD_V3%YHO%MP,IBL) )
        ZHSUM(JSTGLO+J-1) = ZHSUM(JSTGLO+J-1) + SD_V3(J,JZ,YSD_V3%YHO%MP,IBL)
        SD_V3(J,JZ+1,YSD_V3%YDO%MP,IBL) = ZHSUM(JSTGLO+J-1)
      ENDDO

      SD_V3(J,ILEVO+1,YSD_V3%YHO%MP,IBL) = 1.E-10_JPRB ! small number
      SD_V3(J,ILEVO+1,YSD_V3%YHO_INV%MP,IBL) = 1.0_JPRB / SD_V3(J,ILEVO+1,YSD_V3%YHO%MP,IBL)
      SD_V3(J,ILEVO+1,YSD_V3%YZO%MP,IBL) = - SD_V2(J,YSD_V2%YOCDEP%MP,IBL)
    ENDDO

  ENDDO 
!$OMP END PARALLEL DO

ENDIF

!*       6.    GRID-POINT NORMS
!              ----------------

WRITE(NULOUT,'(A)') ' SUGRIDO: STATISTICS FOR ALL OCEAM MIXED LAYER FIELDS'

IF (YSP_OMD%NUMFLDS>0) THEN
  DO JV=1,YSP_OMD%NUMFLDS
    WRITE(NULOUT,'(A,I4)') ' OML PROGNOSTIC FIELDS ',JV
    CALL GPNORM3(SP_OM,YSP_OMD%NUMFLDS,JV,KLEVS=YSP_OMD%NLEVS,LDLEVELS=.TRUE.)
  ENDDO
ENDIF

IF (YSD_V2D%NUMFLDS>0) THEN
  WRITE(NULOUT,'(A)') ' OML 2D DIAG FIELDS '
  CALL GPNORM2(YSD_V2D%NUMFLDS,1,SD_V2)
ENDIF

WRITE(NULOUT,'(A)') ' OML 3D DIAG FIELDS '
DO JV=1,YSP_OMD%NUMFLDS
  CALL GPNORM3(SD_V3,YSD_V3D%NUMFLDS,JV,KLEVS=YSD_V3D%NLEVS,LDLEVELS=.TRUE.)
ENDDO

IF (ALLOCATED(ZZ2D))  DEALLOCATE (ZZ2D)
IF (ALLOCATED(ZZ3D))  DEALLOCATE (ZZ3D)
IF (ALLOCATED(ZSUMH)) DEALLOCATE (ZSUMH) 
IF (ALLOCATED(ZHSUM)) DEALLOCATE (ZHSUM) 
IF (ALLOCATED(ZDFAC)) DEALLOCATE (ZDFAC) 
IF (ALLOCATED(ZSK))   DEALLOCATE (ZSK) 
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDO',1,ZHOOK_HANDLE)

END SUBROUTINE SUGRIDO
