SUBROUTINE SUGRIDU(KFILE)

!**** *SUGRIDU*  - Routine to initialize the upper air grid point
!                  fields of the model.

!     Purpose.
!     --------
!           Initialize the upper air grid point fields of the model.

!**   Interface.
!     ----------
!        *CALL* *SUGRIDU*

!        Explicit arguments :
!        --------------------
!        KFILE : an indicator for which  file is to be read

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  SUGRIDUG - initialize from GRIB file
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 94-06-30
!        Modified 95-03-07 by E. Bazile : CALL SUGRIDUA (liquid water)
!        Modified 98-12-15 by R. El Khatib : Arguments to SUGRIDUA
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : N3DINI   ,LARPEGEF

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sugridua.intfb.h"
#include "sugridug.intfb.h"
#include "sugridug1.intfb.h"
#include "sugridug2.intfb.h"

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDU',0,ZHOOK_HANDLE)
CALL GSTATS(24,0)

!*       1.    INITIALIZE UPPER AIR GRIDPOINT FIELD.
!              -------------------------------------

IF (LARPEGEF) THEN

!*      1.1   INITIALIZE FIELDS FROM *FA* FILE.

  CALL SUGRIDUA(KFILE)

ELSE

!*      1.2   INITIALIZE FIELDS FROM GRIB FILES OR ARTIFICIAL DATA

  IF ( N3DINI == 0 ) THEN
    CALL SUGRIDUG(KFILE)
  ELSEIF( N3DINI == 1 ) THEN
    CALL SUGRIDUG1
  ELSE
    CALL SUGRIDUG2(N3DINI, KFILE)
  ENDIF

ENDIF

!     ------------------------------------------------------------------

CALL GSTATS(24,1)

IF (LHOOK) CALL DR_HOOK('SUGRIDU',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRIDU
