SUBROUTINE SUGRIDUG(KFILE)

!**** *SUGRIDUG*  - Initialize the upper air grid-point fields from GRIB

!     Purpose.
!     --------
!           Initialize the upper air gridpoint fields of the model
!           from GRIB file.

!**   Interface.
!     ----------
!        *CALL* *SUGRIDUG

!        Explicit arguments :
!        ------------------
!        KFILE : an indicator for which  file is to be read
!               KFILE = 1 the CFNGGG-UA file is read
!               KFILE = 6 the CFNFGIG-UA file is read
!               KFILE = 7 the CFNANIG-UA file is read
!               KFILE =12 the CFNBGHRGG-UA file is read
!             else        the CFNIGG-UA  file is read

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 94-06-30
!        Modified : 95-01-25 M.Hamrud (use PBGRIB instead of BUFFER IN)
!        Modified : 96-04    A.Untch  (grid-point prognostic ozone)
!        E. Andersson 960425 : Read AN and FG grid-point q
!        E. Andersson 960726 : Bug reading ANIUA and FGIUA, LMESSP=t
!        M.Hamrud  : 98-08-10 Internal int. from low to high resolution
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of LLPOLE and code under LRPOLE, LLPOLE.
!        E.Holm    : 98-11-06 Ozone included in internal interpolation
!        R. El Khatib : 99-12-28 remove poles rows
!        D.Dent     : 00-09-01  MPE to MPL conversion
!        M.Hortal/M.Hamrud : 03-01-01 GFL
!        M.Janiskova : 04-01-21 initialization of GPCLOUD for new cloud scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        Y.Tremolet    18-Jan-2005 Introduce conserving interpolation option
!        Y.Tremolet    18-Feb-2005 Split pre_grid_biconserv
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        E.Holm        13-Nov-2008 Add high resolution background=12, used
!                                  by rdfpinc for adding increments
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK
USE YOMDIM   , ONLY : NFLEVG, NPROMA
USE YOMLUN   , ONLY : NULOUT 
USE YOMCT0   , ONLY : LFPART2, NINTERPINCR
USE YOMGEM   , ONLY : NGPTOT
USE YOMOP    , ONLY : CFNIGG, CFNPL, CFNFGIG, CFNANIG, CFNGGG, CFNBGHRGG
USE YOMGFL   , ONLY : GFL
USE YOMVFP   , ONLY : LSPINT
USE YOM_YGFL , ONLY : YGFL,YGFLC  ! , YGRG_NL
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, CLOSE_IOREQUEST

IMPLICIT NONE

! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)       :: KFILE 

INTEGER(KIND=JPIM) :: IGRIB3D(YGFL%NUMFLDS)
INTEGER(KIND=JPIM) :: ICL, JGFL, ILEN, JSTGLO, ICEND, ISTC, IBL, ICL2
INTEGER(KIND=JPIM) :: JLEV, J
REAL(KIND=JPRB),ALLOCATABLE :: ZZGFL(:,:,:,:)

CHARACTER :: CLFN*16

LOGICAL :: LLSPINT, LLINCR
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnorm3.intfb.h"
!     ------------------------------------------------------------------

!*       1.    READ IN GRID-POINT FIELDS FROM GRIB DATA.
!              -----------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRIDUG',0,ZHOOK_HANDLE)

LLINCR  = (KFILE == 6.OR.KFILE == 7)
LLSPINT = LSPINT.AND.LLINCR

ICL = 0
IF(YGFL%NUMFLDS > 0 ) THEN
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LGP .AND. YGFLC(JGFL)%NREQIN == 1 .AND. YGFLC(JGFL)%LGPINGP &
       & .AND. .NOT. LLSPINT) THEN  
      ICL = ICL+1
      IGRIB3D(ICL) = YGFLC(JGFL)%IGRBCODE
    ENDIF
  ENDDO
  ALLOCATE (ZZGFL(NGPTOT,NFLEVG,ICL,1))
ENDIF

IF (ICL>0) THEN
  WRITE(UNIT=NULOUT,FMT='(A)')' READ UPPER AIR GRIDPOINT FIELDS FROM GRIB '
  IF (KFILE == 1) THEN
    CLFN=CFNGGG(1:12)//'UA'
    WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' LOW RES GP UA TO BE READ FROM FILE ',CLFN  
  ELSEIF (KFILE == 6) THEN
    CLFN=CFNFGIG(1:12)//'UA'
    WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' LOW RES GP UA FG TO BE READ FROM FILE ',CLFN  
  ELSEIF (KFILE == 7) THEN
    CLFN=CFNANIG(1:12)//'UA'
    WRITE(UNIT=NULOUT,FMT='(A)') 'YIPPIE'  
    WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' LOW RES GP UA AN TO BE READ FROM FILE ',CLFN  
  ELSEIF (KFILE == 12) THEN
    CLFN=CFNBGHRGG(1:13)//'UA'
    WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' HIGH RES GP UA BG TO BE READ FROM FILE ',CLFN  
  ELSE
    IF(.NOT.LFPART2) THEN
      CLFN=CFNIGG(1:12)//'UA'
      WRITE(UNIT=NULOUT,FMT='(A,A)')&
       & ' READ UPPER AIR GRIDPOINT FIELDS FROM GRIB ',CLFN  
    ELSE
      ILEN=INDEX(CFNPL,' ')-1
      CLFN='UA'//CFNPL(3:ILEN)
      WRITE(UNIT=NULOUT,FMT='(A,A)')&
       & ' PRE-PROCESSED UA GRIDPOINT DATA TO BE READ FROM FILE '&
       & ,CLFN  
    ENDIF
  ENDIF

  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r',KIOMASTER=1)
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
   & KGRIB3D=IGRIB3D(1:ICL),KPROMA=NGPTOT,LDINTERP=LLINCR,&
   & KTYPE_INTERP=NINTERPINCR)
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR4=ZZGFL(:,:,1:ICL,:))
  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)
ENDIF

!*       ADD TO GFL GRIDPOINT
!        --------------------

CALL GSTATS(1411,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,ICEND,ISTC,JLEV,J,&
!$OMP& JGFL,IBL,ICL2)
DO JSTGLO=1,NGPTOT,NPROMA
  ICEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
  ISTC  = 1
  IBL   = (JSTGLO-1)/NPROMA+1
  ICL2  = 0
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LGP) THEN
      IF (YGFLC(JGFL)%NREQIN == 1) THEN
        IF (YGFLC(JGFL)%LGPINGP .AND. .NOT. LLSPINT) THEN
          ICL2 = ICL2+1
          DO JLEV=1,NFLEVG
            DO J=ISTC,ICEND
              GFL(J,JLEV,YGFLC(JGFL)%MP,IBL) = ZZGFL(JSTGLO+J-1,JLEV,ICL2,1)
            ENDDO
          ENDDO
        ELSEIF (YGFLC(JGFL)%LTRAJIO .AND. .NOT. LLSPINT) THEN
          GFL(:,:,YGFLC(JGFL)%MP,IBL) = 0.0_JPRB
        ENDIF
      ELSE
        IF (YGFLC(JGFL)%NREQIN == -1) THEN
          GFL(:,:,YGFLC(JGFL)%MP,IBL) = YGFLC(JGFL)%REFVALI
        ELSE
          GFL(:,:,YGFLC(JGFL)%MP,IBL) = 0.0_JPRB
        END IF
      ENDIF
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1411,1)

IF (ALLOCATED(ZZGFL)) DEALLOCATE (ZZGFL)

! check for negative concentrations in grg part of gfl 
!  DO JGFL=1,YGFL%NUMFLDS
!   IF(ANY(YGFLC(JGFL)%IGRBCODE  ==  YGRG_NL(:)%IGRBCODE)  ) THEN 
!      WHERE ( GFL(:,:,YGFLC(JGFL)%MP,:) < 0.0_JPRB)  GFL(:,:,YGFLC(JGFL)%MP,:) = 0.0_JPRB 
!   ENDIF
!  ENDDO


WRITE(NULOUT,'(A)') ' SUGRIDUG : STATISTICS FOR ALL GFL FIELDS'
DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LGP) THEN
    CALL GPNORM3 (GFL,YGFL%NDIM,JGFL,YGFLC(JGFL)%CNAME)
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDUG',1,ZHOOK_HANDLE)

END SUBROUTINE SUGRIDUG
