SUBROUTINE SUGRIDUG1

!**** *SUGRIDUG1*  - Initialize the upper air grid-point fields using
!                    artificial data

!     Purpose.
!     --------
!           Initialize the upper air gridpoint fields of the model
!           using artificial data

!**   Interface.
!     ----------
!        *CALL* *SUGRIDUG1

!        Explicit arguments :
!        ------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        George Mozdzynski *ECMWF*

!     Modifications.
!     --------------
!        Original : 97-09-22 
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of LLPOLE and code under LRPOLE, LLPOLE.
!          - removal of argument .FALSE. in 'CALL SPEREE'.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,&
 & NPROMA   ,NSMAX    ,NSPEC2   ,NUMP  

USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY :    NGPTOT
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL     ,YGFLC     ,YQ
USE YOMLAP   , ONLY : NASM0    ,MYMS

IMPLICIT NONE

REAL(KIND=JPRB),ALLOCATABLE :: ZZQT0(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSPQ(:)

INTEGER(KIND=JPIM) :: ICEND, IENDC, IM, IPCT, IPT, ISP, ISTC, J, JLEV, JMLOC, JN, JSTGLO &
 & , IBL, IFLD,JGFL  


LOGICAL :: LLINIT0
REAL(KIND=JPRB) :: ZQMAX, ZQMIN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnorm3.intfb.h"
#include "speree.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDUG1',0,ZHOOK_HANDLE)
LLINIT0=.TRUE.
IPT=0
! Set Gridpoint part of GFL to ZERO
DO JGFL=1,YGFL%NUMFLDS
  IF(YGFLC(JGFL)%LGP) THEN
    GFL(:,:,JGFL,:) =0.0_JPRB
  ENDIF
ENDDO

IF( YQ%LGP )THEN
  ALLOCATE (ZZQT0(NGPTOT,NFLEVG))
  ALLOCATE (ZSPQ(NSPEC2))
  ZSPQ(:)=0.0_JPRB
  ZZQT0(:,:)=0.0_JPRB
  ZQMIN= 0.25E-5_JPRB
  ZQMAX= 0.01_JPRB
  DO JLEV=1,NFLEVG
    DO JMLOC=1,NUMP
      IM = MYMS(JMLOC)
      DO JN=IM,NSMAX
        ISP = NASM0(IM)+(JN-IM)*2
        ZSPQ(ISP)=1.E-09_JPRB * IM * JN * JLEV &
         & / ( REAL(NSMAX*NSMAX,JPRB) * REAL(NFLEVG,JPRB) )  
        IF (IM  /=  0) THEN
          ZSPQ(ISP+1)=0.0_JPRB
        ENDIF
      ENDDO
    ENDDO
    IF (NUMP > 0.AND. MYMS(1) == 0) THEN
      ZSPQ(NASM0(0))=ZQMIN+ (ZQMAX-ZQMIN)*&
       & MAX(0.0_JPRB,REAL(2*JLEV,JPRB)/REAL(NFLEVG,JPRB)-1.0_JPRB)  
    ENDIF
    CALL SPEREE(1,1,ZSPQ,ZZQT0(1,JLEV))
  ENDDO
  DEALLOCATE(ZSPQ)
ENDIF

!     ------------------------------------------------------------------

!*       ADD TO UPPER AIR GRIDPOINT WORKFILE.
!        ------------------------------------

IF(YQ%LGP) THEN
  DO JSTGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
    IPCT=1
    ISTC=1
    IENDC=ICEND
    IBL=(JSTGLO-1)/NPROMA+1
    IFLD=0
    DO JLEV=1,NFLEVG
      DO J=ISTC,IENDC
        GFL(J,JLEV,YQ%MP,IBL)= ZZQT0(JSTGLO+J-1,JLEV)
      ENDDO
    ENDDO
  ENDDO
ENDIF

IF (ALLOCATED(ZZQT0)) DEALLOCATE (ZZQT0)

WRITE(NULOUT,'(A,A)') ' SUGRIDUG1 : STATISTICS FOR ALL ','GFL FIELDS'
DO JGFL=1,YGFL%NUMFLDS
  WRITE(NULOUT,'(A)') YGFLC(JGFL)%CNAME
  CALL GPNORM3 (GFL,YGFL%NDIM,JGFL)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDUG1',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRIDUG1
