SUBROUTINE SUGRIDUG2(K3DINI,KFILE)

!**** *SUGRIDUG*  - Initialize the upper air grid-point fields for LDYNCORE

!     Purpose.
!     --------
!           Initialize the upper air gridpoint fields of the model for LDYNCORE

!**   Interface.
!     ----------
!        *CALL* *SUGRIDUG2

!        Explicit arguments :
!        ------------------
!          K3DINI == xx specifies the type of initial data to be used

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Nils Wedi *ECMWF*

!     Modifications.
!     --------------
!              adapted from sugridug.F90
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMLUN   , ONLY : NULOUT, NULERR 
USE YOMCT0   , ONLY : LFPART2, N3DINI
USE YOMGEM   , ONLY : NGPTOT
USE YOMOP    , ONLY : CFNIGG   ,CFNPL    ,CFNGGG
USE YOMGFL   , ONLY : GFL
USE YOMVFP   , ONLY : LSPINT
USE YOM_YGFL , ONLY : YGFL     ,YGFLC
!     dynamical core switch
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, CLOSE_IOREQUEST
USE YOMDYNCORE, ONLY : LAQUA
USE YOMGRB   , ONLY  : NGRBQ

IMPLICIT NONE


! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)       :: K3DINI
INTEGER(KIND=JPIM),INTENT(IN)       :: KFILE 

INTEGER(KIND=JPIM) :: IGRIB3D(YGFL%NUMFLDS)
INTEGER(KIND=JPIM) :: ICL, JGFL, ILEN, JSTGLO, ICEND, ISTC, IBL, ICL2
INTEGER(KIND=JPIM) :: JLEV, J, IQ
REAL(KIND=JPRB),ALLOCATABLE :: ZZGFL(:,:,:,:)
REAL(KIND=JPRB)    :: ZQGATE(NFLEVG)

CHARACTER :: CLFN*16

LOGICAL :: LLSPINT, LLINCR
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnorm_gfl.intfb.h"
#include "abor1.intfb.h"
!     ------------------------------------------------------------------

!*       1.    READ IN GRID-POINT FIELDS FROM GRIB DATA.
!              -----------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRIDUG2',0,ZHOOK_HANDLE)

LLINCR  = .FALSE.
LLSPINT = LSPINT.AND.LLINCR

ICL = 0
IF(YGFL%NUMFLDS > 0 ) THEN
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LGP .AND. YGFLC(JGFL)%NREQIN == 1 .AND. YGFLC(JGFL)%LGPINGP &
       & .AND. .NOT. LLSPINT) THEN  
      ICL = ICL+1
      IGRIB3D(ICL) = YGFLC(JGFL)%IGRBCODE
      IF( YGFLC(JGFL)%IGRBCODE == NGRBQ ) THEN
        IQ=ICL
      ENDIF
    ENDIF
  ENDDO
  ALLOCATE (ZZGFL(NGPTOT,NFLEVG,ICL,1))
ENDIF

IF( K3DINI /= 2 ) THEN
  ! set all gfl to 0.
  ZZGFL(:,:,1:ICL,:) = 0._JPRB
  WRITE (NULOUT,*) 'DYNAMICAL CORE, GFL is set to 0.',&
   &' (subroutine sugridug2), '
ELSE
  IF (ICL>0) THEN
    WRITE(UNIT=NULOUT,FMT='(A)')' READ UPPER AIR GRIDPOINT FIELDS FROM GRIB '
    IF (KFILE == 1) THEN
      CLFN=CFNGGG(1:12)//'UA'
      WRITE(UNIT=NULOUT,FMT='(A,A)')&
       & ' LOW RES GP UA TO BE READ FROM FILE ',CLFN  
    ELSE
      IF(.NOT.LFPART2) THEN
        CLFN=CFNIGG(1:12)//'UA'
        WRITE(UNIT=NULOUT,FMT='(A,A)')&
         & ' READ UPPER AIR GRIDPOINT FIELDS FROM GRIB ',CLFN  
      ELSE
        ILEN=INDEX(CFNPL,' ')-1
        CLFN='UA'//CFNPL(3:ILEN)
        WRITE(UNIT=NULOUT,FMT='(A,A)')&
         & ' PRE-PROCESSED UA GRIDPOINT DATA TO BE READ FROM FILE '&
         & ,CLFN  
      ENDIF
    ENDIF
  
    CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r',KIOMASTER=1)
    CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
     & KGRIB3D=IGRIB3D(1:ICL),KPROMA=NGPTOT,LDINTERP=LLINCR)
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR4=ZZGFL(:,:,1:ICL,:))
    CALL CLOSE_IOREQUEST(YL_IOREQUEST)
    CALL CLOSE_IOSTREAM(YL_IOSTREAM)

    IF( .NOT.LAQUA ) THEN
      ! set q profile to 0. (dry run)
      DO JLEV=1,NFLEVG
        ZZGFL(1:NGPTOT,JLEV,IQ,1) = 0.0_JPRB
      ENDDO
      WRITE (NULOUT,*) 'DYNAMICAL CORE, Q set to 0.',&
       &' (subroutine sugridug2), '
    ENDIF
  ENDIF
ENDIF

IF( N3DINI == 3 .AND. LAQUA ) THEN
  
  ! GATE observations
  IF( NFLEVG == 91 ) THEN
    ZQGATE( 1)=   0.0000000006
    ZQGATE( 2)=   0.0000000017
    ZQGATE( 3)=   0.0000000033
    ZQGATE( 4)=   0.0000000059
    ZQGATE( 5)=   0.0000000100
    ZQGATE( 6)=   0.0000000162
    ZQGATE( 7)=   0.0000000251
    ZQGATE( 8)=   0.0000000374
    ZQGATE( 9)=   0.0000000541
    ZQGATE(10)=   0.0000002663
    ZQGATE(11)=   0.0000006001
    ZQGATE(12)=   0.0000010191
    ZQGATE(13)=   0.0000015348
    ZQGATE(14)=   0.0000021590
    ZQGATE(15)=   0.0000029027
    ZQGATE(16)=   0.0000037768
    ZQGATE(17)=   0.0000047911
    ZQGATE(18)=   0.0000058101
    ZQGATE(19)=   0.0000053073
    ZQGATE(20)=   0.0000047417
    ZQGATE(21)=   0.0000041111
    ZQGATE(22)=   0.0000034135
    ZQGATE(23)=   0.0000026474
    ZQGATE(24)=   0.0000023524
    ZQGATE(25)=   0.0000021906
    ZQGATE(26)=   0.0000020162
    ZQGATE(27)=   0.0000018293
    ZQGATE(28)=   0.0000016300
    ZQGATE(29)=   0.0000014183
    ZQGATE(30)=   0.0000011944
    ZQGATE(31)=   0.0000012478
    ZQGATE(32)=   0.0000018785
    ZQGATE(33)=   0.0000025378
    ZQGATE(34)=   0.0000032241
    ZQGATE(35)=   0.0000039362
    ZQGATE(36)=   0.0000046747
    ZQGATE(37)=   0.0000054424
    ZQGATE(38)=   0.0000055431
    ZQGATE(39)=   0.0000047199
    ZQGATE(40)=   0.0000038559
    ZQGATE(41)=   0.0000042603
    ZQGATE(42)=   0.0000050451
    ZQGATE(43)=   0.0000059682
    ZQGATE(44)=   0.0000082236
    ZQGATE(45)=   0.0000106185
    ZQGATE(46)=   0.0000138247
    ZQGATE(47)=   0.0000186030
    ZQGATE(48)=   0.0000236709
    ZQGATE(49)=   0.0000328115
    ZQGATE(50)=   0.0000451185
    ZQGATE(51)=   0.0000581574
    ZQGATE(52)=   0.0000828678
    ZQGATE(53)=   0.0001106892
    ZQGATE(54)=   0.0001490497
    ZQGATE(55)=   0.0002341603
    ZQGATE(56)=   0.0003241798
    ZQGATE(57)=   0.0004511625
    ZQGATE(58)=   0.0006163886
    ZQGATE(59)=   0.0007909446
    ZQGATE(60)=   0.0008678976
    ZQGATE(61)=   0.0009395083
    ZQGATE(62)=   0.0011509743
    ZQGATE(63)=   0.0015663843
    ZQGATE(64)=   0.0019975048
    ZQGATE(65)=   0.0028005708
    ZQGATE(66)=   0.0036158341
    ZQGATE(67)=   0.0041657928
    ZQGATE(68)=   0.0045344373
    ZQGATE(69)=   0.0049318997
    ZQGATE(70)=   0.0055056293
    ZQGATE(71)=   0.0060698730
    ZQGATE(72)=   0.0069188024
    ZQGATE(73)=   0.0079090670
    ZQGATE(74)=   0.0088615203
    ZQGATE(75)=   0.0097082108
    ZQGATE(76)=   0.0105173849
    ZQGATE(77)=   0.0113247298
    ZQGATE(78)=   0.0121413302
    ZQGATE(79)=   0.0129047735
    ZQGATE(80)=   0.0136368591
    ZQGATE(81)=   0.0143264215
    ZQGATE(82)=   0.0149517292
    ZQGATE(83)=   0.0154850231
    ZQGATE(84)=   0.0158915054
    ZQGATE(85)=   0.0162449442
    ZQGATE(86)=   0.0165464040
    ZQGATE(87)=   0.0168046113
    ZQGATE(88)=   0.0170271136
    ZQGATE(89)=   0.0172089636
    ZQGATE(90)=   0.0173474420
    ZQGATE(91)=   0.0174473710
    
  ELSEIF( NFLEVG == 60 ) THEN
    
    ZQGATE( 1)=   0.0000000059
    ZQGATE( 2)=   0.0000000171
    ZQGATE( 3)=   0.0000000299
    ZQGATE( 4)=   0.0000000466
    ZQGATE( 5)=   0.0000001633
    ZQGATE( 6)=   0.0000004590
    ZQGATE( 7)=   0.0000008081
    ZQGATE( 8)=   0.0000012184
    ZQGATE( 9)=   0.0000017029
    ZQGATE(10)=   0.0000022811
    ZQGATE(11)=   0.0000029830
    ZQGATE(12)=   0.0000038483
    ZQGATE(13)=   0.0000049203
    ZQGATE(14)=   0.0000056983
    ZQGATE(15)=   0.0000050721
    ZQGATE(16)=   0.0000042962
    ZQGATE(17)=   0.0000033350
    ZQGATE(18)=   0.0000024118
    ZQGATE(19)=   0.0000021483
    ZQGATE(20)=   0.0000018218
    ZQGATE(21)=   0.0000014172
    ZQGATE(22)=   0.0000013188
    ZQGATE(23)=   0.0000027647
    ZQGATE(24)=   0.0000044243
    ZQGATE(25)=   0.0000054850
    ZQGATE(26)=   0.0000038753
    ZQGATE(27)=   0.0000057417
    ZQGATE(28)=   0.0000108430
    ZQGATE(29)=   0.0000197863
    ZQGATE(30)=   0.0000362868
    ZQGATE(31)=   0.0000628278
    ZQGATE(32)=   0.0001138183
    ZQGATE(33)=   0.0002250295
    ZQGATE(34)=   0.0003884535
    ZQGATE(35)=   0.0006577134
    ZQGATE(36)=   0.0008543505
    ZQGATE(37)=   0.0009613564
    ZQGATE(38)=   0.0014471827
    ZQGATE(39)=   0.0020706374
    ZQGATE(40)=   0.0031572049
    ZQGATE(41)=   0.0040817675
    ZQGATE(42)=   0.0045726113
    ZQGATE(43)=   0.0051821517
    ZQGATE(44)=   0.0059373383
    ZQGATE(45)=   0.0070094881
    ZQGATE(46)=   0.0083121350
    ZQGATE(47)=   0.0094922157
    ZQGATE(48)=   0.0105668437
    ZQGATE(49)=   0.0116440747
    ZQGATE(50)=   0.0126832146
    ZQGATE(51)=   0.0136497095
    ZQGATE(52)=   0.0145409852
    ZQGATE(53)=   0.0153198624
    ZQGATE(54)=   0.0158723202
    ZQGATE(55)=   0.0163260326
    ZQGATE(56)=   0.0166918915
    ZQGATE(57)=   0.0169773102
    ZQGATE(58)=   0.0171925407
    ZQGATE(59)=   0.0173443630
    ZQGATE(60)=   0.0174473766
  ELSE
    WRITE(NULERR,'('' INVALID NUMBER OF LEVELS IN SUGRIDUG2'')')
    CALL ABOR1(' INVALID LEVELS IN SUGRIDUG2 ')
  ENDIF
! set q profile
  DO JLEV=1,NFLEVG
    ZZGFL(1:NGPTOT,JLEV,IQ,1) = ZQGATE(JLEV)
  ENDDO
  WRITE (NULOUT,*) 'DYNAMICAL CORE, Q set to GATE obs',&
   &' (subroutine sugridug2), '
ENDIF

!*       ADD TO GFL GRIDPOINT
!        --------------------

CALL GSTATS(1411,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,ICEND,ISTC,JLEV,J,&
!$OMP& JGFL,IBL,ICL2)
DO JSTGLO=1,NGPTOT,NPROMA
  ICEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
  ISTC  = 1
  IBL   = (JSTGLO-1)/NPROMA+1
  ICL2  = 0
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LGP .AND. YGFLC(JGFL)%NREQIN == 1 .AND. YGFLC(JGFL)%LGPINGP &
     & .AND. .NOT. LLSPINT) THEN  
      ICL2 = ICL2+1
      DO JLEV=1,NFLEVG
        DO J=ISTC,ICEND
          GFL(J,JLEV,YGFLC(JGFL)%MP,IBL) = ZZGFL(JSTGLO+J-1,JLEV,ICL2,1)
        ENDDO
      ENDDO
    ELSEIF(YGFLC(JGFL)%LGP .AND. .NOT. YGFLC(JGFL)%NREQIN == 1) THEN  
      GFL(:,:,YGFLC(JGFL)%MP,IBL) = 0.0_JPRB
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1411,1)

IF (ALLOCATED(ZZGFL)) DEALLOCATE (ZZGFL)

WRITE(NULOUT,'(A)') ' SUGRIDUG : STATISTICS FOR ALL GFL FIELDS'
CALL GPNORM_GFL

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDUG2',1,ZHOOK_HANDLE)

END SUBROUTINE SUGRIDUG2
