SUBROUTINE SUGRIDVA

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPROMA
USE YOMLUN   , ONLY : NULOUT
USE SURFACE_FIELDS_MIX, ONLY : &
 & SD_VF, YSD_VFD, YSD_VF, SD_VP, YSD_VPD, YSD_VP, &
 & SD_VV, YSD_VVD, YSD_VV, &
 & SD_VA, YSD_VAD, YSD_VA, SD_VC, YSD_VCD, YSD_VC

USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG
USE YOMOP    , ONLY : NAGDIO   ,MAXFIELD
USE YOMMP    , ONLY : MYPROC   ,NSTRIN

!**** *SUGRIDVA*  - Initialize the clim. gridpoint fields from *FA*

!     Purpose.
!     --------
!           Initialize the clim. gridpoint fields from *FA*

!**   Interface.
!     ----------
!        *CALL* *SUGRIDVA(...)*

!        Explicit arguments :
!        ------------------

!        Implicit arguments :
!        --------------------
!        See 'USE MODULE' above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        OPENFA
!        RDGPFA
!        DISGRID
!        FAIRME
!        Is called by SUGRIDF

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*
!        from SUGRCLIADM/SUGRIDVA 

!     Modifications.
!     --------------
!        ORIGINAL : 97-12-03
!        Y. Bouteloup : 02-09-11 xxO3ABC => Climatological ozone profiles
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Marquet   : 02-07-17 add VCLIA for aerosol files
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZFIELDBUF(NGPTOTG)
INTEGER(KIND=JPIM) :: IDATEF(11)

INTEGER(KIND=JPIM) :: JSTGLO,IEND,IBL,JL, IDEB, IFIN
INTEGER(KIND=JPIM) :: IFILE, IINDEX,  INDEX, INUM, IREP, IUNTIN, JV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "disgrid.intfb.h"
#include "openfa.intfb.h"
#include "rdgpfa.intfb.h"
#include "suarpio.intfb.h"

!     ------------------------------------------------------------------
!*       1.    PREPARATIONS.
!              ------------

IF (LHOOK) CALL DR_HOOK('SUGRIDVA',0,ZHOOK_HANDLE)
CALL SUARPIO(1)

!     ------------------------------------------------------------------
!*       2.    OPEN AND CHECK FILE.
!              -------------------

IF (MYPROC <= NSTRIN) THEN
  IFILE=19
  CALL OPENFA(IFILE,IUNTIN,IDEB,IFIN,IINDEX,IDATEF)
ENDIF

!     ------------------------------------------------------------------
!*       3.    COUNT, THEN POSSIBLY READ & SEND FIELDS TO OTHERS.
!              -------------------------------------------------

INUM=0

!*       3.1    VARSF

DO JV=1,YSD_VFD%NUMFLDS
  IF (YSD_VF%YVF(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        INDEX=IINDEX
        CALL RDGPFA(IUNTIN,YSD_VF%YVF(JV)%CNAME,IDEB,IFIN,INDEX,ZFIELDBUF,&
         & JV,0,0,'CLIMATE','VARSF',&
         & .FALSE.)             
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VF(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDVA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.2    VCLIP

DO JV=1,YSD_VPD%NUMFLDS
  IF (YSD_VP%YVP(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        INDEX=IINDEX
        CALL RDGPFA(IUNTIN,YSD_VP%YVP(JV)%CNAME,IDEB,IFIN,INDEX,ZFIELDBUF,&
         & JV,0,0,'CLIMATE','VCLIP',&
         & .FALSE.)             
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VP(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDVA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.3    VCLIV

DO JV=1,YSD_VVD%NUMFLDS
  IF (YSD_VV%YVV(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        INDEX=IINDEX
        CALL RDGPFA(IUNTIN,YSD_VV%YVV(JV)%CNAME,IDEB,IFIN,INDEX,ZFIELDBUF,&
         & JV,0,0,'CLIMATE','VCLIV',&
         & .FALSE.)             
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VV(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDVA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.4    VCLIA

DO JV=1,YSD_VAD%NUMFLDS
  IF (YSD_VA%YVA(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        INDEX=IINDEX
        CALL RDGPFA(IUNTIN,YSD_VA%YVA(JV)%CNAME,IDEB,IFIN,INDEX,ZFIELDBUF,&
         & JV,0,0,'CLIMATE','VCLIA',.FALSE.)
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VA(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDVA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!*       3.5    VO3ABC

DO JV=1,YSD_VCD%NUMFLDS
  IF (YSD_VC%YVC(JV)%CNAME /= '                ') THEN
    INUM=INUM+1
    IF (INUM <= MAXFIELD) THEN
      IF (NAGDIO(1,INUM) == MYPROC) THEN
        INDEX=IINDEX
        CALL RDGPFA(IUNTIN,YSD_VC%YVC(JV)%CNAME,IDEB,IFIN,INDEX,ZFIELDBUF,&
         & JV,0,0,'CLIMATE','VO3ABC',&
         & .FALSE.)             
        CALL DISGRID(INUM,0,NGPTOTG,ZFIELDBUF)
      ELSE
        CALL DISGRID(INUM,1,NGPTOT,ZFIELDBUF)
      ENDIF
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL=(JSTGLO-1)/NPROMA+1
        DO JL=1,IEND
          SD_VC(JL,JV,IBL) = ZFIELDBUF(JSTGLO-1+JL)
        ENDDO
      ENDDO
    ELSE
      WRITE(NULOUT,*) ' MAXFIELD TOO SMALL !'
      CALL ABOR1('SUGRIDVA : ABOR1 CALLED')
    ENDIF
  ENDIF
ENDDO

!     ------------------------------------------------------------------
!*       4.    CLOSE FILE.
!              ----------

IF (MYPROC <= NSTRIN) THEN
  CALL FAIRME(IREP,IUNTIN,'UNKNOWN')
ENDIF


!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGRIDVA',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRIDVA
