SUBROUTINE SUGRXFU(KFILE)

!**** *SUGRXFU*  - Initialize the instantaneous fluxes from *FA*

!     Purpose.
!     --------
!           Initialize the instantaneous fluxes fields of the model from FA.

!**   Interface.
!     ----------
!        *CALL* *SUGRXFU(....*

!        Explicit arguments : KFILE - indicator for which file is to be read
!        ------------------   

!        Implicit arguments :
!        ------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!            OPENFA
!            INCGPF

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!       R. El Khatib  *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 97-12-05
!        Modified : 98-01-05 D. Giard (soil fluxes)
!        Modified : 00-03-29 R. El Khatib (Unification SM/DM)
!        Modified : 00-03 by R. El Khatib - derived types
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LRPLANE
USE YOMGEM   , ONLY : NGPTOT
USE YOMXFU   , ONLY : NFDXFU   ,NTYPXFU  ,TYPE_XFU
USE YOMXFUB  , ONLY : XFUBUF
USE YOMGC    , ONLY : GM

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
INTEGER(KIND=JPIM) ::  IFLDPT(NFDXFU), ILEV(NFDXFU), IPAR(NFDXFU)
REAL(KIND=JPRB) :: ZFIELD(NGPTOT,NFDXFU), Z1SGM(NGPTOT)
CHARACTER(LEN=4) :: CLPREF(NFDXFU)
CHARACTER(LEN=16) :: CLSUFF(NFDXFU)

INTEGER(KIND=JPIM) :: IOFF, IFILE, JF, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "incgpf.intfb.h"
#include "rdpxfa.intfb.h"

!     ------------------------------------------------------------------

!*    1.    PREPARATIONS.
!           -------------

IF (LHOOK) CALL DR_HOOK('SUGRXFU',0,ZHOOK_HANDLE)
IF (KFILE == 7) THEN
  IFILE=40
ELSE
  IFILE=KFILE
ENDIF
IOFF=0
DO JF=1,NTYPXFU
  DO JL=1,TYPE_XFU(JF)%ILEV
    CLPREF(IOFF+JL)=TYPE_XFU(JF)%CLPREF(JL)
    ILEV(IOFF+JL)=TYPE_XFU(JF)%INUMLEV(JL)
    CLSUFF(IOFF+JL)=TYPE_XFU(JF)%CLSUFF
    IPAR(IOFF+JL)=TYPE_XFU(JF)%IPARITY
    IFLDPT(IOFF+JL)=IOFF+JL
  ENDDO
  IOFF=IOFF+TYPE_XFU(JF)%ILEV
ENDDO

!*    2.    READ FILE AND DISTRIBUTE FIELDS
!           -------------------------------

CALL RDPXFA(ZFIELD,CLPREF,ILEV,CLSUFF,NFDXFU,IFILE)

!     3.    REDUCE VECTOR FIELDS
!           --------------------

IF (LRPLANE) THEN
  Z1SGM(1:NGPTOT)=1.0_JPRB/GM(1:NGPTOT)
  DO JF=1,NFDXFU
    IF (IPAR(JF) == 1) THEN
      ZFIELD(1:NGPTOT,JF)=ZFIELD(1:NGPTOT,JF)*Z1SGM(1:NGPTOT)
    ENDIF
  ENDDO
ENDIF

!     4.    WRITE BUFFER/WORKFILE
!           ---------------------

CALL INCGPF(NFDXFU,XFUBUF,IFLDPT,ZFIELD)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRXFU',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRXFU
