#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!OPTION! -O nochg
!OCL  NOUNROLL,NOPREEX,NOEVAL
SUBROUTINE SUHDF

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDLON    ,NFLEVL   ,NFLEVG   ,NSMAX   ,LSPT
USE YOMGEM   , ONLY : VBH      ,VAH      ,VP00
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LNHDYN
USE YOMCT0B  , ONLY : LECMWF
USE YOMDYNA  , ONLY : LSLHD_W   ,LSLHD_T  ,LSLHD_SVD ,LSLHD_SPD,&
 & LSLHD_STATIC ,LSLHD
USE YOMDYN   , ONLY : RDIVOR    ,RDIDIV   ,RDITG    ,RDIGFL   ,&
 & RDIPD    ,RDIVD    ,RDISP    ,RDSVOR   ,RDSDIV   ,RDSVD    ,&
 & NSREFDH  ,REXPDH   ,HDIRVOR  ,HDIRDIV  ,HDIRT    ,&
 & HDIRQ    ,HDIRO3   ,HDIRPD   ,HDIRSP   ,&
 & HRDIRVOR ,HRDIRDIV ,HRDIRT   ,& 
 & HRDIRQ   ,HRDIRO3  ,HRDIRPD  ,HRDIRVD  ,HRDIRSP  ,&
 & FRANDH   ,SLEVDH   ,SLEVDH2  ,SLEVDH3  ,&
 & HRDSRVOR ,HRDSRDIV ,HRDSRVD  ,REXPDHS  ,SLEVDHS  ,SLEVDHS2 ,SDRED
USE YOMSTA   , ONLY : STPRE
USE YOMMP    , ONLY : MYLEVS
USE YOM_YGFL , ONLY : YQ        ,YO3      ,YQ_NL    ,YO3_NL

!**** *SUHDF*   - Initialize horizontal diffusion

!     Purpose.
!     --------

!         COMPUTES HORIZONTAL DIFFUSION COEFFICIENTS BY USE OF SHAPE FUNCTION
!          (copied from *SUDYN* without any change)

!         Some remarks:
!          - The code of suhdf.F90 is valid for ARPEGE, the corresponding
!            code for ALADIN is computed in suehdf.F90 (project ALD).
!          - the vertical profile computed in Z_PDILEV and Z_PDILEV_VOR
!            is significantly different between ECMWF and METEO-FRANCE.
!            Z_PDILEV_VOR is not used at ECMWF (because in this case
!            it is always equal to Z_PDILEV, so only Z_PDILEV is used in
!            this case).
!          - There are some differences between the way of computing
!            Z_PDILEV and Z_PDILEV_VOR between ARPEGE and ALADIN: in ALADIN 
!            there is only a subset of options available, which match
!            Z_PDILEV=Z_PDILEV_VOR, so Z_PDILEV is used (in suehdf.F90) but
!            not Z_PDILEV_VOR.
!          - For GFL variables, the old passive scalars are now considered
!            as obsolete features, and there are not any longer diffused;
!            GFL variables other than "q" and "O3" are not diffused.

!**   Interface.
!     ----------
!        *CALL* *SUHDF

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE/ALADIN DOCUMENTATION

!     Author.
!     -------
!        G. Radnoti CNRM/GMAP MICECO

!     Modifications.
!     --------------
!        Original : 92-12-24
!        K. YESSAD: 08/03/1993. Additional horizontal diffusion
!                   on not stretched sphere.
!        JPH.PIEDELIEVRE: 93-03-25 : HORIZONTAL DIFFUSION
!        K. YESSAD: 17/02/1994. Different coefficients for T and q in
!                   horizontal diffusion.
!        K. YESSAD: 18/02/1994. New definition for PDISPE,PDICPE and
!                   new functions PDISPVOR,PDICPVOR.
!                   Change definition of constants HDIR(...), HDCR(...),
!                    now times.
!        K. YESSAD (APRIL 1994): Some cleaning + unified horizontal diffusion.
!        R. BUBNOVA(JANUARY 1995): Set up for non hydrostatic variables.
!        K. YESSAD (NOV 1995): 
!                  - Removal of HDC horizontal diffusion scheme.
!                  - NEXPDH changed into REXPDH.
!        M. Hortal: 96-05-16 Terms for the new thermodynamic equation
!        M. Hortal: 96-10-23 Change PDISPEL (for linear grid)
!        L. Isaksen: 96-12-12 Recoded to local level structure (NFLEVL)
!        A. Untch: 97-11-26 Diffusion for models with top at p<0.1hPa
!        C. Temperton: 98-10-12 Repair shallow-water equation model!
!        M. SZCZECH: 00-04-18 Horizontal diffusion for thermodynamic ozon
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud     : 03-08-01 GFL introduction
!        F. Vana and K. YESSAD: 03-08-25 new vertical profile of diffusion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.YESSAD (Dec 2003): cleaning horizontal diffusion.
!        F. Vana and K. Yessad: modify diffusion when LSLHD=T.
!        K. Yessad: (jul 2004) bound the increase of hor diffusion
!         in the upper stratosphere.
!        F. Vana : (Nov-2004) avoiding double division of RRDXTAU
!        F. Vana : (Feb-2005) split of the global LSLHD key + cleaning
!        K. Yessad 08-Feb-2005: horizontal diffusion for NH variables (case c=1)
!        K. Yessad (Aug 2005): calculation of RCORDI... moved in SURCORDI.
!        K. Yessad (Aug 2005): calculation of RKRF moved in SURAYFRIC.
!        R. El Khatib 28-Jul-2006 Porting to NEC
!        N. Wedi (Nov 2007): adapt HD for NH model at ECMWF
!        F. Vana  13-Jan-2009 : removed argument + recognized LSLHD_STATIC
!     ------------------------------------------------------------------

IMPLICIT NONE

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZPRESH(0:NFLEVG)
REAL(KIND=JPRB) :: Z_PDILEV_VOR(NFLEVG)
REAL(KIND=JPRB) :: Z_PDILEV(NFLEVG)

INTEGER(KIND=JPIM) :: IL, ILEV, INSDH, ISTDV2, JL, JLEV, JLEVG, JN

LOGICAL :: LLGRID, LLMESO, LLSLHD_W  ,LLSLHD_T  ,LLSLHD_Q  ,LLSLHD_O3 ,&
 & LLSLHD_SVD ,LLSLHD_SPD ,LLSLHD

REAL(KIND=JPRB) :: ZBDH, ZBDHVOR,&
 & ZDILEV, ZEFOLD, ZEFOLV, ZENSTDF, ZEPS, ZP1DH, &
 & ZP2DH, ZPKDH, ZPKDH_REF, ZPKDH_SLHD, &
 & ZPKDHVOR, ZPKDHVOR_REF, ZPKDHVOR_SLHD, ZPRES

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

! * FUNCTIONS AND FUNCTION ARGUMENTS:
REAL(KIND=JPRB) :: PDISPE
REAL(KIND=JPRB) :: PDISPEE
REAL(KIND=JPRB) :: PDISPEL
REAL(KIND=JPRB) :: PDISPEX
REAL(KIND=JPRB) :: PDISPEX_L91
REAL(KIND=JPRB) :: PDISPVOR
INTEGER(KIND=JPIM) :: KL, KMAX, KN
REAL(KIND=JPRB) :: PEXPDH

! * HORIZONTAL DIFFUSION SHAPE FUNCTIONS:
PDISPE(KN,KMAX,PEXPDH)=( MAX ( 0.0_JPRB ,&
 & (SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB))-FRANDH)&
 & /(1.0_JPRB-FRANDH) ) )**PEXPDH  
PDISPVOR(KN,KMAX,PEXPDH)=( MAX ( 0.0_JPRB ,&
 & (SQRT(MAX(0.0_JPRB,REAL(KN*(KN+1)-MIN(REAL(KMAX,JPRB),2.0_JPRB),JPRB) &
 & /REAL(KMAX*(KMAX+1)-MIN(REAL(KMAX,JPRB),2.0_JPRB),JPRB))) &
 & -FRANDH)&
 & /(1.0_JPRB-FRANDH) ) )**PEXPDH  
PDISPEL(KN,KMAX,PEXPDH,KL)=&
 & 1.0_JPRB* &
 & (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB)))))&
 & **PEXPDH      *(Z_PDILEV(KL)-1.0_JPRB)+&
 & 0.0_JPRB* &
 & (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB)))))&
 & **(PEXPDH*3/2)*(Z_PDILEV(KL)-1.0_JPRB)+&
 & 1.0_JPRB* &
 & (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB)))))&
 & **(PEXPDH    )  
PDISPEE(KN,KMAX,PEXPDH,KL)=&
 & 1.0_JPRB* &
 & (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB)))))&
 & **PEXPDH      *(Z_PDILEV(KL)-1.0_JPRB)+ 0.0_JPRB*&
 & (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB)))))&
 & **(PEXPDH*3/2)*(Z_PDILEV(KL)-1.0_JPRB)  
PDISPEX(KN,KMAX,PEXPDH,KL)=&
& 5._JPRB* &
& (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB)))))**2&
& *MAX(9._JPRB-KL,0.0_JPRB)  
PDISPEX_L91(KN,KMAX,PEXPDH,KL)=&
 & (MAX(0.0_JPRB,(SQRT(REAL(KN*(KN+1),JPRB)/REAL(KMAX*(KMAX+1),JPRB))))) &
 & *MAX(11._JPRB-KL,0.0_JPRB)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUHDF',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

IF (LECMWF) THEN
  IF(NFLEVG == 19) THEN
    ZENSTDF=2._JPRB
    ISTDV2=6
  ELSE
    ZENSTDF=1.3_JPRB
    ISTDV2=MIN(9,NFLEVG)
    !     ZENSTDF=SQRT(2.)
    !     ISTDV2=MIN(10,NFLEVG)
  ENDIF
  LLMESO=0.5_JPRB*VAH(1) <= 10.5_JPRB
  IF(LLMESO) THEN
    ! * Remark: this piece of code is fully consistent with
    !   lvertfe=F, ndlnpr=0, laprxpk=T
    !   where prehyd(l)=0.5*prehyd(lbar-1)+0.5*prehyd(lbar)
    !   and not completely consistent with the other cases
    !   for lvertfe,ndlnpr,laprxpk.
    WRITE(NULOUT,*) ' SUHDF: ',' Z_PDILEV COMPUTED AS FUNCTION OF PRESSURE: '
    ! FOR MODELS WITH TOP ABOVE 0.105hPa PDILEV INCREASES LINEARLY 
    ! IN LOG10(PRESSURE) FROM 1. AT 10hPa TO 16. AT 0.1hPa.
    ZPRESH(0)=VAH(0)+VBH(0)*VP00
    DO JLEV=1,NFLEVG
      ZPRESH(JLEV)=VAH(JLEV)+VBH(JLEV)*VP00
      ZPRES=0.5_JPRB*(ZPRESH(JLEV)+ZPRESH(JLEV-1))
      ZDILEV=1.0_JPRB+7.5_JPRB*(3._JPRB-LOG10(ZPRES))
      Z_PDILEV(JLEV)=MAX(1.0_JPRB,ZDILEV)
    ENDDO
    WRITE(NULOUT,'(A/(5E24.14))') ' Z_PDILEV: ',(Z_PDILEV(JLEV),JLEV=1,NFLEVG)
  ELSE
    DO JLEV=1,NFLEVG
      Z_PDILEV(JLEV)=MIN(16._JPRB,ZENSTDF**MAX(0,ISTDV2-JLEV))
    ENDDO
  ENDIF
ELSE
  LLMESO=.FALSE.
  ! * provisional value of PDILEV (the final value is computed later).
  DO JLEV=1,NFLEVG
    Z_PDILEV(JLEV)=1.0_JPRB
  ENDDO
ENDIF

! * 1.2: Initialization of SLHD keys

LLSLHD    = (.NOT.LSLHD_STATIC) .AND. LSLHD
LLSLHD_W  = (.NOT.LSLHD_STATIC) .AND. LSLHD_W
LLSLHD_T  = (.NOT.LSLHD_STATIC) .AND. LSLHD_T
LLSLHD_Q  = (.NOT.LSLHD_STATIC) .AND. YQ_NL%LSLHD
LLSLHD_O3 = (.NOT.LSLHD_STATIC) .AND. YO3_NL%LSLHD
LLSLHD_SVD= (.NOT.LSLHD_STATIC) .AND. LSLHD_SVD
LLSLHD_SPD= (.NOT.LSLHD_STATIC) .AND. LSLHD_SPD

!     ------------------------------------------------------------------

!*       2.    MAIN HORIZONTAL DIFFUSION.
!              --------------------------

LLGRID=NSMAX > (NDLON+3)/3
WRITE(NULOUT,*) ' SUHDF, LLGRID=',LLGRID,' NSMAX=',NSMAX,' NDLON=',NDLON
IF(ALLOCATED(RDIGFL)) RDIGFL(:,:,:) = 0.0_JPRB

!*       2.1   Computation of RDI[X] at ECMWF:

IF(LECMWF) THEN

  ! * warning: option LSLHD=T not coded for LECMWF=T.

  WAVE_LOOP_EC : DO JN=0,NSMAX

    !*       2.1.1 3D variables:

    LEVEL_LOOP1_EC : DO JLEV=1,NFLEVL
      ILEV=MYLEVS(JLEV)
      IF (NFLEVG > 1) THEN
        IF(HDIRVOR > 1.0_JPRB) THEN
          RDIVOR(JLEV,JN)=PDISPEL(JN,NSMAX,REXPDH,ILEV)/HDIRVOR
        ELSE
          RDIVOR(JLEV,JN)=0.0_JPRB
        ENDIF
        IF(HDIRDIV > 1.0_JPRB) THEN
          IF(LLMESO) THEN
            IF(NFLEVG == 91) THEN
              RDIDIV(JLEV,JN)=(PDISPEL(JN,NSMAX,REXPDH,ILEV)+&
               & PDISPEX_L91(JN,NSMAX,REXPDH,ILEV))/HDIRDIV  
            ELSE
              IF(NSMAX > 200) THEN
                RDIDIV(JLEV,JN)=(PDISPEL(JN,NSMAX,REXPDH,ILEV)+&
                 & PDISPEX(JN,NSMAX,REXPDH,ILEV))/HDIRDIV
              ELSE
                RDIDIV(JLEV,JN)=PDISPEL(JN,NSMAX,REXPDH,ILEV)/HDIRDIV
              ENDIF  
            ENDIF
          ELSE
            RDIDIV(JLEV,JN)=PDISPEL(JN,NSMAX,REXPDH,ILEV)/HDIRDIV
          ENDIF
        ELSE
          IF(LLMESO) THEN
            IF(NFLEVG == 91) THEN
              RDIDIV(JLEV,JN)=(PDISPEE(JN,NSMAX,REXPDH,ILEV)+&
               & PDISPEX_L91(JN,NSMAX,REXPDH,ILEV))/HDIRVOR  
            ELSE
              IF(NSMAX > 200) THEN
                RDIDIV(JLEV,JN)=(PDISPEE(JN,NSMAX,REXPDH,ILEV)+&
                 & PDISPEX(JN,NSMAX,REXPDH,ILEV))/HDIRVOR
              ELSE
                RDIDIV(JLEV,JN)=PDISPEE(JN,NSMAX,REXPDH,ILEV)/HDIRVOR              
              ENDIF
            ENDIF
          ELSE
            RDIDIV(JLEV,JN)=PDISPEE(JN,NSMAX,REXPDH,ILEV)/HDIRVOR
          ENDIF
        ENDIF
        IF (YQ%LSP) THEN
          IF(HDIRQ  > 1.0_JPRB) THEN
            RDIGFL(JLEV,JN,YQ%MPSP)= &
             & Z_PDILEV(ILEV)*PDISPE(JN,NSMAX,REXPDH)/HDIRQ
          ENDIF
        ENDIF
        IF (YO3%LSP) THEN
          IF(HDIRO3 > 1.0_JPRB) THEN
            RDIGFL(JLEV,JN,YO3%MPSP)= &
             & Z_PDILEV(ILEV)*PDISPE(JN,NSMAX,REXPDH)/HDIRO3
          ENDIF
        ENDIF
        IF(LNHDYN) THEN
          !ori IF(HDIRPD > 1.0_JPRB) THEN
          !ori   RDIPD(JLEV,JN)=Z_PDILEV(ILEV)*PDISPE(JN,NSMAX,REXPDH)/HDIRPD
          !ori ELSE
          !ori   RDIPD(JLEV,JN)=0.0_JPRB
          !ori ENDIF
          IF(HDIRPD > 1.0_JPRB) THEN
            RDIPD(JLEV,JN)=PDISPEL(JN,NSMAX,REXPDH,ILEV)/HDIRPD
          ELSE
            RDIPD(JLEV,JN)=0.0_JPRB
          ENDIF
          !ori IF(HDIRVD > 1.0_JPRB) THEN
          !ori   RDIVD(JLEV,JN)=Z_PDILEV(ILEV)*PDISPE(JN,NSMAX,REXPDH)/HDIRVD
          !ori ELSE
          !ori   RDIVD(JLEV,JN)=0.0_JPRB
          !ori ENDIF
          RDIVD(JLEV,JN)=RDIDIV(JLEV,JN)
        ENDIF
      ELSE
        ! --- Code for shallow-water equations:
        IF(HDIRVOR > 1.0_JPRB) THEN
          RDIVOR(JLEV,JN)=Z_PDILEV(ILEV)*PDISPE(JN,NSMAX,REXPDH)/HDIRVOR
        ELSE
          RDIVOR(JLEV,JN)=0.0_JPRB
        ENDIF
        IF(HDIRDIV > 1.0_JPRB) THEN
          RDIDIV(JLEV,JN)=Z_PDILEV(ILEV)*PDISPE(JN,NSMAX,REXPDH)/HDIRDIV
        ELSE
          RDIDIV(JLEV,JN)=0.0_JPRB
        ENDIF
        ! --- end of code for shallow-water equations
      ENDIF
    ENDDO LEVEL_LOOP1_EC

    DO JLEVG=1,NFLEVG
      IF(HDIRT  > 1.0_JPRB) THEN
        RDITG (JLEVG,JN)=PDISPEL(JN,NSMAX,REXPDH,JLEVG)/HDIRT
      ELSE
        RDITG(JLEVG,JN)=PDISPEE(JN,NSMAX,REXPDH,JLEVG)/HDIRVOR
      ENDIF
    ENDDO

    !*       2.1.2 2D variables:

    IF (HDIRSP > 1.0_JPRB) THEN
      RDISP(JN)=PDISPE(JN,NSMAX,REXPDH)/HDIRSP
    ELSE
      RDISP(JN)=0.0_JPRB
    ENDIF

  ENDDO WAVE_LOOP_EC

ENDIF

!*       2.2   Computation of RDI[X] at METEO-FRANCE:

IF (.NOT.LECMWF) THEN

  WAVE_LOOP_HDI : DO JN=0,NSMAX

    !*       2.2.1 3D variables:

    !   * GMV variables (other than T for numerical diffusion),
    !     and GFL variables.

    LEVEL_LOOP3_HDI : DO JLEV=1,NFLEVL

      !     All "RDI..." arrays dimensioned with NFLEVL.

      ILEV=MYLEVS(JLEV)
      IF (NFLEVG > 1) THEN

        ZEPS=1.E-6_JPRB

        ZP1DH=MAX(2.0_JPRB*ZEPS,SLEVDH*VP00)
        ZP2DH=MIN(MAX(ZEPS,SLEVDH2*VP00),ZP1DH-ZEPS)
        INSDH=MIN(NSREFDH,NSMAX)
        IF(STPRE(ILEV) >= ZP1DH) THEN
          ! * LOW LAYER (BELOW SLEVDH) :
          !   THE DIFFUSION COEFFICIENT IS A(JN)
          !   AND THE TRUNCATION IS NSMAX.
          Z_PDILEV_VOR(ILEV)=1.0_JPRB
          Z_PDILEV(ILEV)=1.0_JPRB
          ZPKDHVOR_REF=PDISPVOR(JN,NSMAX,REXPDH)
          ZPKDH_REF=PDISPE(JN,NSMAX,REXPDH)
          IF (LLSLHD) THEN
            ZPKDHVOR_SLHD=PDISPVOR(JN,NSMAX,REXPDH)*MAX(0.0_JPRB,1.0_JPRB-SDRED)
            ZPKDH_SLHD=PDISPE(JN,NSMAX,REXPDH)*MAX(0.0_JPRB,1.0_JPRB-SDRED)
          ELSE
            ZPKDHVOR_SLHD=ZPKDHVOR_REF
            ZPKDH_SLHD=ZPKDH_REF
          ENDIF
        ELSEIF(STPRE(ILEV) >= ZP2DH) THEN
          ! * MEDIUM LAYER :
          ! THE DIFFUSION COEFFICIENT IS A(JN)/(PRESSURE-B(JN))
          ZBDHVOR=ZP2DH*ZP1DH*&
           & (PDISPVOR(JN,INSDH,REXPDH)-PDISPVOR(JN,NSMAX,REXPDH))&
           & /MAX(ZEPS,ZP1DH*PDISPVOR(JN,INSDH,REXPDH)&
           & -ZP2DH*PDISPVOR(JN,NSMAX,REXPDH))  
          ZBDH=ZP2DH*ZP1DH*&
           & (PDISPE(JN,INSDH,REXPDH)-PDISPE(JN,NSMAX,REXPDH))/&
           & MAX(ZEPS,ZP1DH*PDISPE(JN,INSDH,REXPDH)&
           & -ZP2DH*PDISPE(JN,NSMAX,REXPDH))  
          Z_PDILEV_VOR(ILEV)=(ZP1DH-ZBDHVOR)/(STPRE(ILEV)-ZBDHVOR)
          Z_PDILEV_VOR(ILEV)=MIN(Z_PDILEV_VOR(ILEV),1.0_JPRB/SLEVDH3)
          Z_PDILEV(ILEV)=(ZP1DH-ZBDH)/(STPRE(ILEV)-ZBDH)
          Z_PDILEV(ILEV)=MIN(Z_PDILEV(ILEV),1.0_JPRB/SLEVDH3)
          ZPKDHVOR_REF=PDISPVOR(JN,NSMAX,REXPDH)*Z_PDILEV_VOR(ILEV)
          ZPKDH_REF=PDISPE(JN,NSMAX,REXPDH)*Z_PDILEV(ILEV)
          IF (LLSLHD) THEN
            ZPKDHVOR_SLHD=PDISPVOR(JN,NSMAX,REXPDH) &
             & *MAX(0.0_JPRB,Z_PDILEV_VOR(ILEV)-SDRED)
            ZPKDH_SLHD=PDISPE(JN,NSMAX,REXPDH)*MAX(0.0_JPRB,Z_PDILEV(ILEV)-SDRED)
          ELSE
            ZPKDHVOR_SLHD=ZPKDHVOR_REF
            ZPKDH_SLHD=ZPKDH_REF
          ENDIF
        ELSE
          ! * HIGH LAYER (ABOVE SLEVDH2) :
          ! THE DIFFUSION COEFFICIENT IS A(JN)/PRESSURE
          ! AND THE TRUNCATION IS MIN(NSMAX,NSREFDH)
          Z_PDILEV_VOR(ILEV)=ZP1DH/STPRE(ILEV)
          Z_PDILEV_VOR(ILEV)=MIN(Z_PDILEV_VOR(ILEV),1.0_JPRB/SLEVDH3)
          Z_PDILEV(ILEV)=ZP1DH/STPRE(ILEV)
          Z_PDILEV(ILEV)=MIN(Z_PDILEV(ILEV),1.0_JPRB/SLEVDH3)
          ZPKDHVOR_REF=PDISPVOR(JN,INSDH,REXPDH)*Z_PDILEV_VOR(ILEV)
          ZPKDH_REF=PDISPE(JN,INSDH,REXPDH)*Z_PDILEV(ILEV)
          IF (LLSLHD) THEN
            ZPKDHVOR_SLHD=PDISPVOR(JN,INSDH,REXPDH) &
             & *MAX(0.0_JPRB,Z_PDILEV_VOR(ILEV)-SDRED)
            ZPKDH_SLHD=PDISPE(JN,INSDH,REXPDH)*MAX(0.0_JPRB,Z_PDILEV(ILEV)-SDRED)
          ELSE
            ZPKDHVOR_SLHD=ZPKDHVOR_REF
            ZPKDH_SLHD=ZPKDH_REF
          ENDIF
        ENDIF

      ELSE
        ! * 2D model (SLHD scheme not implemented)
        ZPKDHVOR_REF=PDISPVOR(JN,NSMAX,REXPDH)
        ZPKDH_REF=PDISPE(JN,NSMAX,REXPDH)
        ZPKDHVOR_SLHD=ZPKDHVOR_REF
        ZPKDH_SLHD=ZPKDH_REF
      ENDIF

      ! * vorticity and divergence
      IF (LLSLHD_W) THEN
        ZPKDHVOR=ZPKDHVOR_SLHD
        ZPKDH=ZPKDH_SLHD
      ELSE
        ZPKDHVOR=ZPKDHVOR_REF
        ZPKDH=ZPKDH_REF
      ENDIF

      ! * vorticity:
      RDIVOR(JLEV,JN)=ZPKDHVOR*HRDIRVOR

      ! * divergence:
      RDIDIV(JLEV,JN)=ZPKDH*HRDIRDIV

      ! * NH pressure departure variable:
      IF (LLSLHD_SPD) THEN
        ZPKDH=ZPKDH_SLHD
      ELSE
        ZPKDH=ZPKDH_REF
      ENDIF
      IF(LNHDYN) THEN
        RDIPD (JLEV,JN)=ZPKDH*HRDIRPD
      ENDIF

      ! * NH vertical divergence variable:
      IF (LLSLHD_SVD) THEN
        ZPKDH=ZPKDH_SLHD
      ELSE
        ZPKDH=ZPKDH_REF
      ENDIF
      IF(LNHDYN) THEN
        RDIVD (JLEV,JN)=ZPKDH*HRDIRVD
      ENDIF

      ! -- GFL variables:

      ! * specific humidity:
      IF (LLSLHD_Q) THEN
        ZPKDH=ZPKDH_SLHD
      ELSE
        ZPKDH=ZPKDH_REF
      ENDIF
      IF (YQ%LSP) THEN
        RDIGFL(JLEV,JN,YQ%MPSP)=ZPKDH*HRDIRQ
      ENDIF

      ! * ozone:
      IF (LLSLHD_O3) THEN
        ZPKDH=ZPKDH_SLHD
      ELSE
        ZPKDH=ZPKDH_REF
       ENDIF
      IF (YO3%LSP) THEN
        RDIGFL(JLEV,JN,YO3%MPSP)=ZPKDH*HRDIRO3
      ENDIF

      ! * other GFL variables: they are currently not diffused.

      ! -- end of GFL variables:

    ENDDO LEVEL_LOOP3_HDI

    ! * Temperature for numerical diffusion:

    IF (LSPT) THEN

      LEVEL_LOOP4_HDI : DO JLEVG=1,NFLEVG

        !     All "RDI..." arrays dimensioned with NFLEVG.

        IF (NFLEVG > 1) THEN

          ZEPS=1.E-6_JPRB

          ZP1DH=MAX(2.0_JPRB*ZEPS,SLEVDH*VP00)
          ZP2DH=MIN(MAX(ZEPS,SLEVDH2*VP00),ZP1DH-ZEPS)
          INSDH=MIN(NSREFDH,NSMAX)
          IF(STPRE(JLEVG) >= ZP1DH) THEN
            ! * LOW LAYER (BELOW SLEVDH) :
            !   THE DIFFUSION COEFFICIENT IS A(JN)
            !   AND THE TRUNCATION IS NSMAX
            Z_PDILEV(JLEVG)=1.0_JPRB
            IF (LLSLHD_T) THEN
              ZPKDH=PDISPE(JN,NSMAX,REXPDH)*MAX(0.0_JPRB,1.0_JPRB-SDRED)
            ELSE
              ZPKDH=PDISPE(JN,NSMAX,REXPDH)
            ENDIF
          ELSEIF(STPRE(JLEVG) >= ZP2DH) THEN
            ! * MEDIUM LAYER :
            !   THE DIFFUSION COEFFICIENT IS A(JN)/(PRESSURE-B(JN))
            ZBDH=ZP2DH*ZP1DH*&
             & (PDISPE(JN,INSDH,REXPDH)-PDISPE(JN,NSMAX,REXPDH))/&
             & MAX(ZEPS,ZP1DH*PDISPE(JN,INSDH,REXPDH)&
             & -ZP2DH*PDISPE(JN,NSMAX,REXPDH))  
            Z_PDILEV(JLEVG)=(ZP1DH-ZBDH)/(STPRE(JLEVG)-ZBDH)
            Z_PDILEV(JLEVG)=MIN(Z_PDILEV(JLEVG),1.0_JPRB/SLEVDH3)
            IF (LLSLHD_T) THEN
              ZPKDH=PDISPE(JN,NSMAX,REXPDH)*MAX(0.0_JPRB,Z_PDILEV(JLEVG)-SDRED)
            ELSE
              ZPKDH=PDISPE(JN,NSMAX,REXPDH)*Z_PDILEV(JLEVG)
            ENDIF
          ELSE
            ! * HIGH LAYER (ABOVE SLEVDH2) :
            !   THE DIFFUSION COEFFICIENT IS A(JN)/PRESSURE
            !   AND THE TRUNCATION IS MIN(NSMAX,NSREFDH)
            Z_PDILEV(JLEVG)=ZP1DH/STPRE(JLEVG)
            Z_PDILEV(JLEVG)=MIN(Z_PDILEV(JLEVG),1.0_JPRB/SLEVDH3)
            IF (LLSLHD_T) THEN
              ZPKDH=PDISPE(JN,INSDH,REXPDH)*MAX(0.0_JPRB,Z_PDILEV(JLEVG)-SDRED)
            ELSE
              ZPKDH=PDISPE(JN,INSDH,REXPDH)*Z_PDILEV(JLEVG)
            ENDIF
          ENDIF

        ELSE
          ! * 2D model (SLHD scheme not implemented)
          ZPKDH=PDISPE(JN,NSMAX,REXPDH)
        ENDIF

        RDITG  (JLEVG,JN)=ZPKDH*HRDIRT

      ENDDO LEVEL_LOOP4_HDI

    ENDIF

    !*       2.2.2 2D variables:

    ! Remark: no diffusion is generally done on this variable,
    ! so the SLHD scheme has not been implemented.

    RDISP(JN)=PDISPE(JN,NSMAX,REXPDH)*HRDIRSP

  ENDDO WAVE_LOOP_HDI

ENDIF

!*       2.3   Computation of RDS[X] for VOR,DIV (METEO-FRANCE):

IF ((.NOT.LECMWF).AND.LLSLHD_W.AND.(NFLEVG > 1)) THEN

  WAVE_LOOP_HDS : DO JN=0,NSMAX

    LEVEL_LOOP3_HDS : DO JLEV=1,NFLEVL

      !     All "RDS..." arrays dimensioned with NFLEVL.

      ILEV=MYLEVS(JLEV)
      IF (NFLEVG > 1) THEN

        ZEPS=1.E-6_JPRB

        ZP1DH=MAX(2.0_JPRB*ZEPS,SLEVDHS*VP00)
        ZP2DH=MIN(MAX(ZEPS,SLEVDHS2*VP00),ZP1DH-ZEPS)
        INSDH=MIN(NSREFDH,NSMAX)
        IF(STPRE(ILEV) >= ZP1DH) THEN
          ! * LOW LAYER (BELOW SLEVDH) :
          !   THE DIFFUSION COEFFICIENT IS A(JN)
          !   AND THE TRUNCATION IS NSMAX.
          Z_PDILEV_VOR(ILEV)=1.0_JPRB
          Z_PDILEV(ILEV)=1.0_JPRB
          ZPKDHVOR=PDISPVOR(JN,NSMAX,REXPDH)
          ZPKDH=PDISPE(JN,NSMAX,REXPDH)
        ELSEIF(STPRE(ILEV) >= ZP2DH) THEN
          ! * MEDIUM LAYER :
          !   THE DIFFUSION COEFFICIENT IS A(JN)/(PRESSURE-B(JN))
          ZBDHVOR=ZP2DH*ZP1DH*&
           & (PDISPVOR(JN,INSDH,REXPDHS)-PDISPVOR(JN,NSMAX,REXPDHS))&
           & /MAX(ZEPS,ZP1DH*PDISPVOR(JN,INSDH,REXPDHS)&
           & -ZP2DH*PDISPVOR(JN,NSMAX,REXPDHS))  
          ZBDH=ZP2DH*ZP1DH*&
           & (PDISPE(JN,INSDH,REXPDHS)-PDISPE(JN,NSMAX,REXPDHS))/&
           & MAX(ZEPS,ZP1DH*PDISPE(JN,INSDH,REXPDHS)&
           & -ZP2DH*PDISPE(JN,NSMAX,REXPDHS))  
          Z_PDILEV_VOR(ILEV)=(ZP1DH-ZBDHVOR)/(STPRE(ILEV)-ZBDHVOR)
          Z_PDILEV_VOR(ILEV)=MIN(Z_PDILEV_VOR(ILEV),1.0_JPRB/SLEVDH3)
          Z_PDILEV(ILEV)=(ZP1DH-ZBDH)/(STPRE(ILEV)-ZBDH)
          Z_PDILEV(ILEV)=MIN(Z_PDILEV(ILEV),1.0_JPRB/SLEVDH3)
          ZPKDHVOR=PDISPVOR(JN,NSMAX,REXPDH)*Z_PDILEV_VOR(ILEV)
          ZPKDH=PDISPE(JN,NSMAX,REXPDH)*Z_PDILEV(ILEV)
        ELSE
          ! * HIGH LAYER (ABOVE SLEVDHS2) :
          !   THE DIFFUSION COEFFICIENT IS A(JN)/PRESSURE
          !   AND THE TRUNCATION IS MIN(NSMAX,NSREFDH)
          Z_PDILEV_VOR(ILEV)=ZP1DH/STPRE(ILEV)
          Z_PDILEV_VOR(ILEV)=MIN(Z_PDILEV_VOR(ILEV),1.0_JPRB/SLEVDH3)
          Z_PDILEV(ILEV)=ZP1DH/STPRE(ILEV)
          Z_PDILEV(ILEV)=MIN(Z_PDILEV(ILEV),1.0_JPRB/SLEVDH3)
          ZPKDHVOR=PDISPVOR(JN,INSDH,REXPDH)*Z_PDILEV_VOR(ILEV)
          ZPKDH=PDISPE(JN,INSDH,REXPDH)*Z_PDILEV(ILEV)
        ENDIF

      ENDIF

      ! * vorticity:
      RDSVOR(JLEV,JN)=ZPKDHVOR*HRDSRVOR

      ! * divergence:
      RDSDIV(JLEV,JN)=ZPKDH   *HRDSRDIV

      ! * NH vertical divergence variable:
      IF(LNHDYN) THEN
        RDSVD(JLEV,JN)=ZPKDH*HRDSRVD
      ENDIF

    ENDDO LEVEL_LOOP3_HDS

  ENDDO WAVE_LOOP_HDS

ENDIF

!*       2.4   Diagnostic of e-folding time:

IF(LECMWF) THEN
  DO JL=NFLEVL,1,-1
    IL=MYLEVS(JL)
    IF (IL <= ISTDV2) THEN
      IF(RDIVOR(JL,NSMAX) > 1E-7_JPRB) THEN
        ZEFOLV=1.0_JPRB/RDIVOR(JL,NSMAX)/3600._JPRB
      ELSE
        ZEFOLV=0._JPRB
      ENDIF
      IF(RDIDIV(JL,NSMAX) > 1E-7_JPRB) THEN
        ZEFOLD=1.0_JPRB/RDIDIV(JL,NSMAX)/3600._JPRB
      ELSE
        ZEFOLD=0._JPRB
      ENDIF
      WRITE(NULOUT,*) '  E-FOLDING TIME FOR VORTICITY=',ZEFOLV,&
       & ' HOURS AT LEVEL ',IL  
      WRITE(NULOUT,*) '    FOR DIVERGENCE=',ZEFOLD
    ENDIF
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUHDF',1,ZHOOK_HANDLE)
END SUBROUTINE SUHDF
