! Library:phys $Id: suhlcond.F90 5126 2007-05-11 16:21:54Z towil $

!options xopt(noeval)
subroutine suhlcond(kulout)

!**** *suhlcond*   - Initialize yhlcond 

!     Purpose.
!     --------
!     Initialize "yhlcond" which contains a group of 
!     constants for the HIRLAM condensation processes. 

!**   Interface.
!     ----------
!     *call* *suhlcond(kulout)

!     Explicit arguments :
!     --------------------
!     KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        yhlcond

!     Method.
!     -------
!     See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!     HIRLAM Documentation 

!     Author.
!     -------
!     B.H. Sass, (Danish Meteorological Institute).

!     Modifications.
!     --------------
!     Original : 01/09 2005
!     A.Bogatchev : 26-Oct-2007 removing of severe breaks of coding rools
!     ----------------------------------------------------------------

use parkind1  ,only : jpim,    jprb
use yomhook   ,only : lhook,   dr_hook
use yhloption ,only : hlprt
use yhlconst  ,only : latvap,    rair,     cpair,    epsilo,   & 
     &                gravit,    latice 
use yhlcond   ,only : hdl,       ht273,    he273,    tvirtc,   &
     &                stpevp,    u00max,   hu00,     aecon,    &
     &                conae,     cfreez,   coales,   hccu,     &
     &                hcunrm,    hmrcu,    hmrst,    htaucu,   &
     &                hp0,       cbfeff,   hvterm,   hvsnow,   &
     &                hkmelt,    tanvil,   hcst,     pmoist,   &
     &                prbice,    bfeff,    hdewi,    hmroft,   &
     &                dttabl,    hkevap,   tcir,     hkap,     &
     &                hdldcp,    hecdr,    hldcp,    coalcu,   &
     &                coalst,    sqvsno,   heldr,    elotci,   &
     &                asnow,     bsnow,    snoref

implicit none

 integer(kind=jpim),intent(in)    :: kulout 

 real(kind=jprb)                   :: zhook_handle

!     Declaration of local variables.

 integer(kind=jpim) :: i10,it,i  
 real(kind=jprb)    :: zhedldr,ztci,ztopeq0,ztodpmx,ztscale,zdemax,zapri,ztest, &
 &                     ztphalf,zpodpmx,zfxt,zxti,zx,zx232,zbfmax,zxpt1mp

!  Namelist not implemented for the time being   
!   (include "posnam.intfb.h")

!     ------------------------------------------------------------------

!*       1.    Set default values.
!              -------------------

!        1.1 Set implicit default values

      if (lhook) call dr_hook('suhlcond',0,zhook_handle)

!     ----------------------------------------------------------
!     Initialize variables used in common block 'COMCONDS'
!     used in connection with the STRACO cloud scheme.
!     ----------------------------------------------------------

      hdl    = 0.334e6_JPRB
      heldr  = epsilo*latvap/rair
      zhedldr = epsilo*hdl/rair

      hp0    = 1.e5_JPRB
      ht273  = 273._JPRB
      he273  = 611._JPRB
      tvirtc = 0.61_JPRB

      ztci    = 232._JPRB
      ztopeq0 = 273._JPRB
      ztodpmx = 299._JPRB
      ztscale = (ztodpmx - ztci)*sqrt(2._JPRB)

!     The table look up is done by finding the integer
!     it = 1 + int((t- h273)/dttabl)

      dttabl = 0.1_JPRB

!     Calculate saturation vapor pressure to be used as table.
!     Calculate difference of saturation pressure between
!     water and ice, dewi, for t< t0
!     Calculate a probability function, prbice, for existence of
!     ice crystals.
!     Calculate a temperature function, hmroft,
!     to multiply hmrcu and hmrst.

      zdemax = 0._JPRB
      zapri = 1._JPRB/(1._JPRB -exp(-((ztopeq0-ztci)/ztscale)**2))
      ztest = 2._JPRB *zapri/(2.*zapri-1._JPRB )
      if(ztest <= 0._JPRB ) &  
     &  write(KULOUT,*)  'warning: ztest is < 0 !!!!'
      ztphalf = log(2._JPRB *zapri/(2._JPRB *zapri-1._JPRB ))
      ztphalf = ztci + ztscale*sqrt(ztphalf)
      if(ztphalf < 0.) &
     &  write(KULOUT,*) 'before do 25,  ztphalf=',ztphalf
      zpodpmx = zapri*(exp(-0.5_JPRB)-1._JPRB ) +1._JPRB

      do i = 1,1750
      zfxt = ht273 - (i-1._JPRB) * dttabl

      zxti = 1._JPRB /ht273 - 1._JPRB /zfxt
      hdewi(i)  = he273/zfxt * exp(heldr*zxti) * &
     &            (1._JPRB - exp(zhedldr*zxti))
      zdemax = max(zdemax,hdewi(i))

      prbice(i) = zapri * (exp(-(((zfxt - ztci)/ztscale)**2)) - &
     &            1._JPRB ) + 1._JPRB
      prbice(i) = max( prbice(i) , 0._JPRB)
      if(zfxt < ztci)  prbice(i) = 1._JPRB
      prbice(i) = min(prbice(i) , 1._JPRB)
      if(zfxt > 250._JPRB) &
     & hmroft(i) = 1.33_JPRB *exp(-((zfxt-ht273)*0.066_JPRB )**2)

      if(zfxt <= 250._JPRB) then
      zx = abs (zfxt - 232._JPRB) / 18._JPRB

      zx = zx * (1._JPRB + zx * (1._JPRB + 1.333_JPRB * zx))
      zx232 = 1._JPRB
      if (zfxt < 232._JPRB) zx232 = -1._JPRB

      zx = zx / (1._JPRB + zx) * zx232

      hmroft(i) = 0.5_JPRB*0.15_JPRB*(1.07_JPRB +zx)
      endif

      hmroft(i) = min( hmroft(i) , 1._JPRB)
      hmroft(i) = (1._JPRB -prbice(i))**2 + prbice(i) * hmroft(i)
      hmroft(i)   = max( hmroft(i) , 3.e-2_JPRB )
      enddo

      prbice(1) = 0._JPRB
      hmroft(1) = 1._JPRB

!     Normalize with max difference, zdemax
!     calculate the product of dewi and prbice, called bfeff

      zbfmax = 0._JPRB

      do i = 1,1750

      hdewi(i)  = hdewi(i)/zdemax
      bfeff( i) = prbice(i) * (1._JPRB -prbice(i)) * hdewi(i)
      zbfmax = max(zbfmax,bfeff(i))
      enddo

      do i = 1,1750
      bfeff( i) = bfeff( i) / zbfmax
      enddo

!     ----------------------------------------------------------------
!     Do print-out

!       write(KULOUT,600)
!       write(KULOUT,605) ztopeq0,ztphalf,zpodpmx,ztodpmx
!       write(KULOUT,615)

        do i10 = 1,47,3
          it = 1 - i10
          i = 1 + (i10-1) * 10
          zxpt1mp = prbice(i)*(1._JPRB -prbice(i))
!      write(KULOUT,617) it, hdewi(i),prbice(i),bfeff(i), &
!    &    hmroft(i),zxpt1mp
        enddo

! 600 format(//,' FROM SUHLCOND (condensation initialization) ',/)
!        1         2         3         4         5         6         7
!  23456789012345678901234567890123456789012345678901234567890123456789012

!   605 format('   PROBABILITY TABLES BETWEEN 273 AND 99 K FOR:',//,
!    1 '     DIFFERENCE of saturation pressure over WATER and ICE;',/,
!    2  '     TEMPERATURE FUNCTION to multiply hmrst and hmrcu;',//,
!    3'     ICE CRYSTAL PROBABILITY P = 0 for T = ',f5.1,'  and P = 0.50
!    4  for T = ',f5.1,/,20x,'     and P and T of dP/dT-max are P ='
!    5 ,f5.2,'  and T = ',f5.1,/)

!  615 format(9x,'t',5x,'de',6x,'prob-ice',5x,'bf-f',4x,'hmroft',
!    1  5x,'p*(1-p)')
  617 format(i10,f9.4,4x,f7.4,4x,f7.4,2x,f7.4,4x,f7.4)


!     parameter values in si units
!     ---------------------------------------------------

      conae  = 0.15_JPRB
      aecon  = exp(conae)

      cfreez = 0.12_JPRB
      coales = 100._JPRB
      hccu   = 5.0e-5_JPRB
      hcst   = 5.0e-5_JPRB
      hcunrm = 3.e4_JPRB
      hmrcu  = 3.25e-3_JPRB
      hmrst  = 5.0e-4_JPRB
      htaucu = 3600._JPRB
      tanvil = 253._JPRB

      cbfeff = 4._JPRB
      hvterm = 5._JPRB
      hvsnow = 1._JPRB
      hkmelt = 5.e-5_JPRB

      stpevp = 5.e-5_JPRB
      u00max = 0.975_JPRB
      hu00   = 0.85_JPRB
      pmoist = 3.0_JPRB

      hkevap = 5.0E-05_JPRB
      tcir   = 235._JPRB

!  asnow  - a parameter together with bsnow and snoref to govern the
!           rate of ice precip before the Bergeron-Findeisen mechanism
!           becomes effective, even if the precip from above is pure
!           ice;
!           the factor cbfsno = asnow*(snowrate/snoref)**bsnow
!           and cbfsno = cbfsno/(1+cbfsno)
!           which multiplies the modified ice probability
!  bsnow  - see asnow
!  snoref - see asnow

      asnow = 9._JPRB
      bsnow = 4._JPRB
      snoref = 0.05_JPRB

!  hkmelt - coefficient for melting of ice in precipitation
!  tanvil - temperature below which convective anvil is allowed and
!           treated as stratiform
!  hcst   - conversion rate of cloud to precipitation drops in stratifor
!           clouds
!  pmoist - exponent for computations of hmoist (eq. 3 in OSZ94)

      hkmelt = 2.5e-4_JPRB
      tanvil = 253._JPRB
      pmoist = 3.0_JPRB

!  hkap   - rair/cpair
!  hdldcp - latice/cpair
!  hecdr  - epsilo / hkap
!  hldcp  - latvap/cpair

      hkap   = rair/cpair
      hdldcp = latice/cpair
      hecdr  = epsilo / hkap
      hldcp  = latvap/cpair

!  coalcu - coalescence factor for convection clouds (c0.mr/2g)
!  coalst - coalescence factor for convection clouds (c0.mr/2g)
!  sqvsno - square root of the snow terminal velocity
!  elotci - epsilo*latice/rair/tcir

      coalcu = 0.5_JPRB * hccu* hmrcu / gravit
      coalst = 0.5_JPRB * hcst* hmrst / gravit
      sqvsno = sqrt(hvsnow)
      elotci = epsilo*latice/rair/tcir
                                                                             
!  ----------------------------------------------------------------
!  Print some of the parameters                                       

   if( hlprt > 1 ) then 
!  ----------------------------------------------------------------
     write(unit=kulout,fmt='('' MODULE YHLCOND INITIALIZED '')')

!     write(KULOUT,3) tanvil,hu00,htaucu,hcunrm,hccu,hmrcu,cfreez,
!    1           coales,hkmelt,pmoist
!    3 format(1h ,//,'  with ice phase,'
!    1        ,/,'  convection scheme
!    2 for T(cutop) <=',f5.0,' K',//,' other parameters are',/,
!    3'   u0   taucu    cunrm     cuc0      cumr    cfreez  coales   hkm
!    4elt    pmoist',
!    5  /,f6.2,f7.0,f10.0,e10.2,e10.2,f7.2,f8.0,e11.2,f7.1,//)

!     write(KULOUT,4) hu00,stpevp,cbfeff,hcst,hmrst,
!    1           cfreez,coales,hkmelt,pmoist
!    4 format(1h ,//,'ice phase'
!    1          ,/,'  stratiform cloud of sub-grid scale, i.e., even for
!    2 u .lt. 1.',//,' other parameters are',/,'   u0     kevp    cbfeff
!    3   stc0      stmr    cfreez  coales   hkmelt    pmoist'
!    4    , /,f6.2,e10.2,f6.1,2x,e9.2,e10.2,f7.2,f8.0,e11.2,f7.1,//)

   endif 
!  ---------------------------------------------------------------------
   if (lhook) call dr_hook('suhlcond',1,zhook_handle)
end subroutine suhlcond 
