!options xopt(noeval)
subroutine suhlph(kulout)

!**** *suhlph* 

!     Calls initialization of 
!     HIRLAM PHYSICS in the Meteo-France version.
!     PROVIDED that Meteo-France physics has 
!     already been initialized. 

!     Purpose.
!     --------
!     Organise the setup of physical constants for HIRLAM
!     physics package.

!**   Interface.
!     ----------
!     *call* *suhlph(kulout)

!     Explicit arguments :
!     --------------------
!     kulout : Logical unit for the output

!     Implicit arguments :
!     --------------------
!     None.

!     Method.
!     -------
!     Irrelevant.

!     Externals.
!     ----------
!     suhlconst
!     suhlcond
!     suhlrad
!     suhlturb

!     Reference.
!     ----------

!     Author.
!     -------
!     Bent H. Sass.

!     Modifications.
!     --------------
!       Original          : 04-08-25 (BHSass)
!       Include suhlturb  : 05-09-22 (BHSass) 

!     --------------------------------------------------------
!*    1.    Call routines for specific physics' commons setup.
!     --------------------------------------------------------

use parkind1  ,only : jpim     ,jprb
use yomhook   ,only : lhook    ,dr_hook
use yhloption ,only : lhl

implicit none 

integer (kind=jpim),intent(in)    :: kulout 
real    (kind=jprb)               :: zhook_handle

#include "suhlconst.intfb.h"
#include "suhlcond.intfb.h"
#include "suhlrad.intfb.h"
#include "suhlturb.intfb.h"

if (lhook) call dr_hook('suhlph',0,zhook_handle)
 
! Call the HIRLAM routines to define the constants 
! in modules yhloption, yhlconst, yhlcond, yhlrad and yhlturb
! -------------------------------------------------------------

   call suhlconst(kulout,0)
!  print *, 'suhlconst  was called'

!  ------------------------------------------------------------

   if(lhl) then
     call suhlcond(kulout)
     write(kulout,*) 'suhlcond  was called'
   endif

!  ------------------------------------------------------------

   if(lhl) then
     call suhlrad
     write(kulout,*) 'suhlrad was called'
   endif

!  ------------------------------------------------------------

   if(lhl) then
     call suhlturb(kulout)  
     write(kulout,*) 'suhlturb  was called'
   endif

! -------------------------------------------------------------

if (lhook) call dr_hook('suhlph',1,zhook_handle)
end subroutine suhlph
