! Library:phys $Id: suhlturb.F90 5126 2007-05-11 16:21:54Z towil $

!options xopt(noeval)
subroutine  suhlturb(kulout)

!**** *suhlturb*   - Initialize yhlturb

!     Purpose.
!     --------
!     Initialize "yhlturb" which contains a group of 
!     constants for the HIRLAM VCBR turbulence scheme.

!**   Interface.
!     ----------
!     *CALL* *suhlturb(KULOUT)

!     Explicit arguments :
!     --------------------
!     KULOUT : Logical unit for the output

!     Implicit arguments :
!     --------------------
!     MODULE yhlturb

!     Method.
!     -------
!     See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!     HIRLAM Documentation 

!     Author.
!     -------
!     B.H. Sass, (Danish Meteorological Institute).

!     Modifications.
!     --------------
!     Original : 2005-07-20
!     ----------------------------------------------------------
use parkind1  ,only : jpim,    jprb
use yomhook   ,only : lhook,   dr_hook
use yhloption ,only : hlprt
use yhlturb   ,only : ct, cq, cte, cu, ctvar, &
 &                    cqvar, cht1, cht2, cdis

implicit none 

integer(kind=jpim),intent(in)    :: kulout 
real(kind=jprb) :: zhook_handle

!     ----------------------------------------------------------
!*       1.    Set default values.
!              -------------------

!        1.1 Set implicit default values

      if(lhook) call dr_hook('suhlturb',0,zhook_handle)

!     ----------------------------------------------------------
      cu=1._jprb
      ct=1._jprb 
      cq=1._jprb
      cte=3._jprb
      ctvar=0.139_jprb
      cqvar=0.139_jprb
      cht1=0.069_jprb
      cht2=0.069_jprb
      cdis=1._jprb/(1.0_jprb*(3.75_jprb*3.75_jprb))
!     (consistent with surface boundary condition).
!     ----------------------------------------------------------

! Print diagnostic information if hlprt > 1 
  if( hlprt > 1 ) then 
! --------------------------------------------------------------
    write(unit=kulout,FMT='('' MODULE yhlturb '')')

    write(unit=kulout,FMT='('' ct= '',E10.4,'' cq = '',E10.4 &
 & ,'' cte = '',E10.4,'' cu = '',E10.4,'' ctvar = '',E10.4 &
 & ,'' cqvar = '',E10.4,'' cht1 = '',E10.4,'' cht2 = '',E10.4 &
 & ,'' cdis = '',E10.4 )') &
 & ct, cq, cte, cu, ctvar, cqvar, cht1, cht2, cdis

  endif 
! --------------------------------------------------------------
  if (lhook) call dr_hook('suhlturb',1,zhook_handle)
end subroutine suhlturb
