SUBROUTINE SUINIMODERR

!   Purpose.
!   --------
!     Initialise model error variables from file.

!   Author.
!   -------
!     Y. Tremolet

!   Modifications.
!   --------------
!     Original   09-Jul-2004
! ------------------------------------------------------------------

USE PARKIND1,ONLY: JPRB, JPIM
USE YOMHOOK ,ONLY: LHOOK, DR_HOOK
USE YOMLUN , ONLY: NULOUT
USE YOMCT0 , ONLY: LIFSMIN, LIFSTRAJ
USE YOMVAR , ONLY: NUPTRA
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, NDIM_MODERR, SPMODERR, GPMODERR, &
                     & LFGMODERR, SPFGMODERR, GPFGMODERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
INTEGER(KIND=JPIM) :: jj
CHARACTER(LEN=14) :: clfile
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "rdmoderr.intfb.h"
#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUINIMODERR',0,ZHOOK_HANDLE)

IF (LIFSTRAJ) THEN
  IF (NTYPE_MODERR==5) THEN
    write(nulout,*)'SUINIMODERR: setting MODERR to ZERO'
    DO jj=1,NDIM_MODERR
      SPMODERR(jj)=0.0_JPRB
      GPMODERR(jj)=0.0_JPRB
    ENDDO
  ELSE
    DO jj=1,NDIM_MODERR
      clfile(1:14)='spmoderrfgXXYY'
      IF (NTYPE_MODERR == 1) THEN
        WRITE(clfile(11:12),'(I2.2)')NUPTRA-1
      ELSE
        WRITE(clfile(11:12),'(I2.2)')NUPTRA
      ENDIF
      write(clfile(13:14),'(I2.2)')jj
      write(nulout,*)'SUINIMODERR: Reading model error from ',clfile
      CALL RDMODERR(clfile,SPMODERR(jj),GPMODERR(jj))
    ENDDO
  ENDIF

ELSEIF (LIFSMIN) THEN
  IF (  LFGMODERR .AND. (.NOT.(NTYPE_MODERR == 5))  ) THEN
    write(nulout,*)'SUINIMODERR: setting MODERR from FGMODERR'
    DO jj=1,NDIM_MODERR
      SPMODERR(jj)=SPFGMODERR(jj)
      GPMODERR(jj)=GPFGMODERR(jj)
    ENDDO
  ELSE
    write(nulout,*)'SUINIMODERR: setting MODERR to ZERO'
    DO jj=1,NDIM_MODERR
      SPMODERR(jj)=0.0_JPRB
      GPMODERR(jj)=0.0_JPRB
    ENDDO
  ENDIF

ELSE  ! Forecast
  IF (NTYPE_MODERR/=2) CALL ABOR1('SUINIMODERR: Only forcing can be used')
  IF (NTYPE_MODERR==5) CALL ABOR1('SUINIMODERR: Only pure forcing can be used')
  DO jj=1,NDIM_MODERR
    clfile(1:14)='spmoderror_YYY'
    write(clfile(12:14),'(I3.3)')jj
    write(nulout,*)'SUINIMODERR: Reading model error from ',clfile
    CALL RDMODERR(clfile,SPMODERR(jj),GPMODERR(jj))
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('SUINIMODERR',1,ZHOOK_HANDLE)
END SUBROUTINE SUINIMODERR
