SUBROUTINE SUIOS

!**** *SUIOS* - ROUTINE TO SET UP FOR USING WORK FILES

!     Purpose.   TO SET UP COMMMON BLOCK YOMIOS WHICH CONTAINS   -
!     --------   PARAMETERS FOR USING THE MIO PACKAGE ON WORK
!                FILES. OPENS WORK FILES.

!**   Interface.
!     ----------
!        *CALL* *SUIOS*

!        Explicit arguments :  None.
!        --------------------

!        Implicit arguments : COMMON YOMIOS
!        --------------------

!     Method.
!     -------

!     Externals.      MIOOPEN - open workfile (MIO package)
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentaion of the MIO package

!     Author.
!     -------
!        Mats Hamrud   *ECMWF*

!     Modifications.
!     --------------
!        Original : 91-01-28
!        Modified : 92-09-30 R. El Khatib (restart on accumulated fluxes)
!        Modified by R. EL Khatib : 93-04-02 Setup defaults controled by LECMWF
!        Modified : 93-04-20 jN Thepaut (spectral trajectory file)
!        Modified : 93-05-25 M.Hamrud (FOU1 and FOU2 now with index)
!        Modified : 93-06-10 JN Thepaut(eigen values file)
!        Modified : 93-09-06 R. El Khatib (restart on instantaneous fluxes)
!        Modified : 93-07-05 M.Hamrud (DDH restart)
!        Modified : 93-11-18 M.Hamrud (Use only one Fourier file)
!        Modified : 94-03-03 R. El Khatib (Full-POS inplementation :
!                            configuration 'P' and I/O on GAUXBUF)
!        Modified : 94-03-22 J.-N. Thepaut/M.Hamrud (Work file for
!                   saving additional spectral arrays)
!        Modified : 94-04-07 M.Hamrud (work file for upper air grid-point)
!        Modified : 94-10-27 M.Hamrud (restart file for upper air grid-point)
!        Modified : 95-01-20 M.Fisher (eigenvectors and random vectors)
!        Modified : 95-09-20 M.Fisher (random vectors setup moved to SUOPH)
!        Modified : 96-01-26 J.F. Mahfouf (trajectory at t-dt)
!        Modified : 95-11-20 R. El Khatib (workfiles for safe-keeping gp fields
!                            before post-processing)
!        Modified SEPT 1996 by K. YESSAD:
!          - Spectral transforms for transmission coefficients.
!        Modified : 97-05-07          (work files for trajectory of physics
!                                      adjoint)
!        Modified : 01-06-13 R. El Khatib Restart file name for ISP
!        Modified : 02-06-10 D.Dent  Restart file name for fields
!                                    which are not part of the surface workfile
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad : 04-07-06 Old SUEIOS now in SUIOS.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY :  CNMEXP
USE YOMCT0B  , ONLY :  LECMWF
USE YOMIOS   , ONLY :  CIOSPRF  ,CIOISPRF ,CFRCF    ,&
 & NPCKFGP  ,NPCKFGP5 ,NPCKFCF  ,&
 & NPCKFXF  ,NPCKFGX  ,NPCKFGN
USE YOMLUN   , ONLY : NULOUT   ,NULNAM

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namios.h"
#include "posnam.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUIOS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PARAMETERS UNDER USER CONTROL.
!              ------------------------------

!*       1.1   SET DEFAULT VALUES

!        1.1.1 Set implicit default values

CFRCF    = 'rcf'
CIOSPRF  = 'srf'
CIOISPRF = 'isprf'

NPCKFGP  = 1
NPCKFGP5 = 1
NPCKFCF  = 1
NPCKFXF  = 1
NPCKFGX  = 1
NPCKFGN  = 1

!        1.1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
  CFRCF   = 'RFILE'//CNMEXP(1:4)
  CIOSPRF = 'RSPEC'//CNMEXP(1:4)
  CIOISPRF= 'RFISP'//CNMEXP(1:4)
ENDIF

!*       1.2   READ NAMELIST.

CALL POSNAM(NULNAM,'NAMIOS')
READ(NULNAM,NAMIOS)

WRITE(NULOUT,'('' SUIOS PRINTS '')')
WRITE(NULOUT,'('' NPCKFGP = '',I2,&
 & '' NPCKFGP5= '',I2 )')&
 & NPCKFGP,NPCKFGP5  
WRITE(NULOUT,'('' NPCKFCF = '',I2,&
 & '' NPCKFXF = '',I2,&
 & '' NPCKFGX = '',I2 )')&
 & NPCKFCF,NPCKFXF,NPCKFGX  
WRITE(NULOUT,'('' NPCKFGN = '',I2 )') NPCKFGN

! -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUIOS',1,ZHOOK_HANDLE)
END SUBROUTINE SUIOS
