SUBROUTINE SULAP

!**** *SULAP * - Routine to initialize the Laplace space working common

!     Purpose.
!     --------
!           Initialize COMMON YOMLAP , initialize pointer arrays.

!**   Interface.
!     ----------
!        *CALL* *SULAP *

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------
!        COMMON YOMLAP, corresponding arrays of the area 1 of the memory manager

!     Method.
!     -------
!        See documentation

!     Externals.   NONE.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        P. Marquet : 91-02-15 : NACRN,NACN0,NAXRN,NAXN0,
!                                RCAPIN,RCAPDI,RXAPIN,RXAPDI
!        D. Giard   : 93-03-19 : NATN0,RTAPIN,RTAPDI,NLTN added
!                                RXAPIN,RXAPDI suppressed
!                                Legendre transf. with NTMAX
!        L. Isaksen : 93-06-07 : REPSNM now a 1D triangle
!        K. YESSAD (MARCH 1995): Add new index array NSE0.
!        L. Isaksen : 95-06-06 : Reordering of spectral arrays
!        D. Giard   : 95-08-30 : corrections, suppression of NAXRN,
!                                NAXN0, NATN0, NACRN, NACN0, NSCREL
!        L. Isaksen : 96-02-20 : NASM0G,NSPINDXG,NVAINDX added
!        L. Isaksen : 96-05-05 : Change definitions of NVALUE, NxVALUE
!        Modified SEPT 1996 by K. YESSAD:
!          - Spectral transforms for transmission coefficients.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        R. El Khatib : 04-08-05 use YEMDIM instead of calling ellips
!        K. Yessad 15-May-2006: memory optimisations for stretched geometry
!        0. Spaniel 15-Nov-2007: cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NCONF    ,NPRINTLEV,LOUTPUT  ,LALLOPR  ,LELAM      
USE YOMCST   , ONLY : RA
USE YOMDIM   , ONLY : NSMAX    ,NSEFRE   ,NCMAX    ,&
 & NTMAX    ,NUMP     ,NUMCP    ,NUMTP    ,NSPEC
USE YOMLAP   , ONLY : NLSCM    ,NLACM    ,NLCM     ,NLSCN    ,&
 & NLACN    ,NLCN     ,NASN0    ,NASM0    ,&
 & NASM0G   ,NASRN    ,NATM0    ,NVALUE   ,NCVALUE  ,&
 & NTVALUE  ,NSTREL   ,MYMS     ,NSPINDX  ,NSPINDXG ,&
 & NSPZERO  ,NSE0L    ,NSPNSUM  ,&
 & RLAPDI   ,RLAPIN   ,RCAPDI   ,RCAPIN   ,RTAPDI   ,RTAPIN
USE YOMMP    , ONLY : MYSETW   ,NPROCM    
USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY : NSTTYP   ,RSTRET
USE YOMJG    , ONLY : NGEOBAL  ,NGEOCOR
USE YEMDIM   , ONLY : NISMAX

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), PARAMETER :: JPKS=KIND(RLAPDI)
INTEGER(KIND=JPIM), PARAMETER :: JPKD=KIND(1.0_JPRB)

INTEGER(KIND=JPIM) :: I, IC, ICONF, IM, INSM, ISEFRE,&
 & IU, JIR, JM, JMLOC, JN  

INTEGER(KIND=JPIM) :: ISPINDX(NSEFRE)
LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SULAP',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    Initialize Laplace space positioning.
!              -------------------------------------

IU = NULOUT
ICONF=NCONF/100
LLP = NPRINTLEV >= 1.OR. LALLOPR

DO JM=0,NSMAX
  NLCM(JM)=NSMAX+1-JM
  NLSCM(JM)=(NLCM(JM)+1)/2
  NLACM(JM)=NLCM(JM)/2
ENDDO

DO JN=0,NSMAX
  NLCN(JN)=JN+1
  NLSCN(JN)=(NLCN(JN)+1)/2
  NLACN(JN)=NLCN(JN)/2
ENDDO

DO JN=0,NSMAX
  NASN0(JN)=JN*JN+JN+1
  NASRN(JN)=JN
ENDDO

IF (.NOT.LELAM) THEN
  ! k.y.: not allocated in SUALLO because NUMP is not yet known in SUALLO.
  ALLOCATE(NSE0L(NUMP))
  IF(LLP)WRITE(IU,9) 'NSE0L    ',SIZE(NSE0L    ),SHAPE(NSE0L    )
  IC=0
  DO JMLOC=1,NUMP
    IM=MYMS(JMLOC)
    NSE0L(JMLOC)=IC
    IC=IC+(NSMAX+1-IM)
  ENDDO
  IF (IC /= NSPEC) CALL ABOR1('SULAP: IC /= NSPEC')
ENDIF

!  NATLM0 is initialized in surinc.F. It will be moved to sump.F later.

IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
  WRITE(UNIT=NULOUT,FMT='('' NLCM,NLSCM,NLACM'')')
  WRITE(UNIT=NULOUT,FMT='(30(1X,I3))')NLCM
  WRITE(UNIT=NULOUT,FMT='(30(1X,I3))')NLSCM
  WRITE(UNIT=NULOUT,FMT='(30(1X,I3))')NLACM
  WRITE(UNIT=NULOUT,FMT='('' NLCN,NLSCN,NLACN'')')
  WRITE(UNIT=NULOUT,FMT='(30(1X,I3))')NLCN
  WRITE(UNIT=NULOUT,FMT='(30(1X,I3))')NLSCN
  WRITE(UNIT=NULOUT,FMT='(30(1X,I3))')NLACN
  WRITE(UNIT=NULOUT,FMT='('' NASN0,NASRN'')')
  WRITE(UNIT=NULOUT,FMT='(16(1X,I6))')NASN0
  WRITE(UNIT=NULOUT,FMT='(16(1X,I6))')NASRN
  IF (.NOT.LELAM) THEN
    WRITE(UNIT=NULOUT,FMT='('' NSE0L '')')
    WRITE(UNIT=NULOUT,FMT='(16(1X,I6))')NSE0L
  ENDIF
ENDIF

!      -----------------------------------------------------------------

!*       2.    Initialize Laplacian operator and its inverse.
!              ----------------------------------------------

DO JN=1,NSMAX+2
  RLAPDI(JN)=REAL(-REAL(JN*(JN+1),JPKD)/(REAL(RA,JPKD)*REAL(RA,JPKD)),JPKS)
  RLAPIN(JN)=REAL(-(REAL(RA,JPKD)*REAL(RA,JPKD))/REAL(JN*(JN+1),JPKD),JPKS)
ENDDO
RLAPDI(0)=0._JPRB
RLAPIN(0)=0._JPRB
RLAPDI(-1)=0.0_JPRB
RLAPIN(-1)=0.0_JPRB
DO JN=1,NCMAX
  RCAPDI(JN)=REAL(-REAL(JN*(JN+1),JPKD)/(REAL(RA,JPKD)*REAL(RA,JPKD)),JPKS)
  RCAPIN(JN)=REAL(-(REAL(RA,JPKD)*REAL(RA,JPKD))/REAL(JN*(JN+1),JPKD),JPKS)
ENDDO
RCAPDI(0)=0._JPRB
RCAPIN(0)=0._JPRB
DO JN=1,NTMAX
  RTAPDI(JN)=REAL(-REAL(JN*(JN+1),JPKD)/(REAL(RA,JPKD)*REAL(RA,JPKD)),JPKS)
  RTAPIN(JN)=REAL(-(REAL(RA,JPKD)*REAL(RA,JPKD))/REAL(JN*(JN+1),JPKD),JPKS)
ENDDO
RTAPDI(0)=0._JPRB
RTAPIN(0)=0._JPRB

IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
  WRITE(UNIT=NULOUT,FMT='('' EIGEN-VALUES OF THE LAPLACIAN'')')
  WRITE(UNIT=NULOUT,FMT='('' TRANSFORMED SPHERE'')')
  WRITE(UNIT=NULOUT,FMT='(14(1X,E8.2))')RLAPDI
  WRITE(UNIT=NULOUT,FMT='('' EIGEN-VALUES OF ITS INVERSE'')')
  WRITE(UNIT=NULOUT,FMT='('' TRANSFORMED SPHERE'')')
  WRITE(UNIT=NULOUT,FMT='(14(1X,E8.2))')RLAPIN
ENDIF

!      -----------------------------------------------------------------

!*       3.    Initialize arrays used in the Legendre transforms.
!              --------------------------------------------------

IF (.NOT.LELAM) THEN
  IC=1
  DO JMLOC=1,NUMP
    IM=MYMS(JMLOC)
    DO JN=IM,NSMAX
      NVALUE(IC  )=JN
      NVALUE(IC+1)=JN
      IC=IC+2
    ENDDO
  ENDDO

  IF (NGEOCOR == 2.OR.NSTTYP /= 1.OR.ABS(RSTRET-1.0_JPRB) >= 1.E-14_JPRB) THEN
    IC=1
    DO JMLOC=1,NUMCP
      IM=MYMS(JMLOC)
      DO JN=IM,NCMAX
        NCVALUE(IC  )=JN
        NCVALUE(IC+1)=JN
        IC=IC+2
      ENDDO
    ENDDO

    IC=1
    DO JMLOC=1,NUMTP
      IM=MYMS(JMLOC)
      DO JN=IM,NTMAX
        NTVALUE(IC  )=JN
        NTVALUE(IC+1)=JN
        IC=IC+2
      ENDDO
    ENDDO
  ENDIF

!      -----------------------------------------------------------------

!*       4.    Spectral control variable and state variable relationship
!              ---------------------------------------------------------

  I=0
  DO JN=0,NSMAX
    DO JM=-JN,JN
      IF (NPROCM(ABS(JM)) == MYSETW) THEN
        I=I+1
        IF (JM < 0) THEN
          ISPINDX(I)=NASM0(ABS(JM))+(JN-ABS(JM))*2+1
        ELSE
          ISPINDX(I)=NASM0(JM)+(JN-JM)*2
        ENDIF
      ENDIF
    ENDDO
    NSPNSUM(JN)=I
  ENDDO

! Global version is needed for control space calculations

  INSM=1
  DO JM=0,NSMAX
    NASM0G(JM)=INSM
    INSM=INSM+(NSMAX+1-JM)*2
  ENDDO

  IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
    WRITE(UNIT=NULOUT,FMT='('' NASM0G '')')
    WRITE(UNIT=NULOUT,FMT='(14(1X,I6))')NASM0G
  ENDIF

  IF (ICONF /= 0.AND.ICONF /= 2.AND.NCONF /= 302) THEN
    I=0
    DO JN=0,NSMAX
      DO JM=-JN,JN
        I=I+1
        IF (JM < 0) THEN
          NSPINDXG(I)=NASM0G(ABS(JM))+(JN-ABS(JM))*2+1
        ELSE
          NSPINDXG(I)=NASM0G(JM)+(JN-JM)*2
        ENDIF
      ENDDO
    ENDDO
  ENDIF

!  Define imaginary m=0 positions.

  DO JN=0,NSMAX
    NSPZERO(JN)=1+JN*2+1
  ENDDO

ELSE  ! lam case
  I=0
  DO JN=0,NSMAX
    DO JM=0,NISMAX(JN)
      IF (NPROCM(JM) == MYSETW) I=I+4
    ENDDO
    NSPNSUM(JN)=I
  ENDDO
ENDIF

!  Correspondance between NSMAX- and NTMAX- spectral arrays

IF (NGEOBAL == 2.OR.NGEOCOR == 2.OR.NSMAX < NTMAX) THEN
  I=0
  DO JMLOC=1,NUMP
    IM=MYMS(JMLOC)
    DO JN=IM,NSMAX
      DO JIR=0,1
        I=I+1
        NSTREL(I)=NATM0(IM)+(JN-IM)*2+JIR
      ENDDO
    ENDDO
  ENDDO
ENDIF

IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
  WRITE(NULOUT,'('' NSPNSUM : SPECTRAL COEFFICIENTS UP TO''&
   & ,'' TOTAL WAVE NUMBER N (INCL.)'')')  
  WRITE(UNIT=NULOUT,FMT='(14(1X,I6))')NSPNSUM
ENDIF

! Now we know the proper dimension of NSPINDX : ALLOCATE and FILL

ISEFRE=NSPNSUM(NSMAX)
ALLOCATE(NSPINDX(ISEFRE))
IF(LLP)WRITE(IU,9) 'NSPINDX  ',SIZE(NSPINDX  ),SHAPE(NSPINDX  )
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (.NOT.LELAM) NSPINDX(1:ISEFRE)=ISPINDX(1:ISEFRE)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SULAP',1,ZHOOK_HANDLE)
END SUBROUTINE SULAP
