SUBROUTINE SULEG

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RPI      ,RA
USE YOMDIM   , ONLY : NDGLG    ,NDGSUR
USE YOMLEG   , ONLY : RW       ,RMU      ,R1MU2    ,R1MUI    ,R1MUA    ,&
 & RSQM2    ,R1QM2    ,RACTHE   ,RLATIG  

#ifdef DOC

!**** *SULEG * - initialize the Legendre polynomials

!     Purpose.
!     --------
!           Initialize COMMON YOMLEG

!**   Interface.
!     ----------
!        *CALL* *SULEG*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------
!              COMMON YOMLEG

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by SUGEM.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     -------------- !        Original : 87-10-15
!        MODIFICATION : 91-04  J.M. Piriou:
!                       - Read gaussian latitudes and PNM on LFI
!                       - If file missing, computes
!                       91-04 M.Hamrud:
!                       - IO Scheme introduced
!        MODIFICATION : 91-07-03 P.Courtier suppress derivatives
!        MODIFICATION : 91-07-03 P.Courtier computes RATATH and RACTHE
!        MODIFICATION : 91-07-03 P.Courtier change upper limit (NSMAX+1)
!        MODIFICATION : 91-07-03 P.Courtier change ordering
!     Order of the PNM in the file, as in the model :
!         - increasing wave numbers m
!         - for a given m, from n=NSMAX+1 to m
!        MODIFICATION : 92-07-02 R. Bubnova: shift RATATH calculation
!                                            to SUGEM1
!        MODIFICATION : 92-12-17 P.Courtier multitask computations
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!        MODIFICATION : 93-03-19 D.Giard : n <= NTMAX
!        K. YESSAD    : 93-05-11 : DLMU --> global array DRMU(NDGSAG:NDGENG).
!                       (not stored currently on LFI files).
!        MODIFICATION : 94-02-03 R. El Khatib : subroutine SULEG2 to write out
!                       the Leg. polynomials on workfile or LFI file
!        Modification : 94-08-31 M. Tolstykh: Setup for CUD interpolation
!        Modified by K. YESSAD (MARCH 1995): Extra-latitudes computations
!                 according to value of NDGSUR and LRPOLE only.
!                 + change fancy loop numbering.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of code under LRPOLE.
!        Modified NOV 1999 by K. YESSAD:
!         - in 'lacudw', compute PDLAT and ILA as in 'lascaw' =>
!           'nrlatx', 'nindlat' and 'rdllat' become useless.
!         - 'ndgl' becomes 'ndglg'.
!         - 'ndgsa' becomes 'ndgsag'.
!         - 'ndgen' becomes 'ndgeng'.
!    R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

#endif

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JGL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE 
#include "trans_inq.h" 
END INTERFACE 
!     ------------------------------------------------------------------ 
!*       1.    Gaussian latitudes and weights 
!              ------------------------------ 

IF (LHOOK) CALL DR_HOOK('SULEG',0,ZHOOK_HANDLE)
CALL TRANS_INQ(PMU=RMU(1:),PGW=RW)

!*       2.    Computes related arrays
!              -----------------------

DO JGL=1,NDGLG
  R1MU2(JGL) = 1.0_JPRB - RMU(JGL)*RMU(JGL)
  R1MUI(JGL) = 1.0_JPRB/(1.0_JPRB - RMU(JGL)*RMU(JGL))
  R1MUA(JGL) = 1.0_JPRB/(1.0_JPRB - RMU(JGL)*RMU(JGL))/RA
  RSQM2(JGL) = SQRT(1.0_JPRB - RMU(JGL)*RMU(JGL))
  R1QM2(JGL) = 1.0_JPRB/SQRT(1.0_JPRB - RMU(JGL)*RMU(JGL))
  RACTHE(JGL) = 1.0_JPRB/SQRT(1.0_JPRB - RMU(JGL)*RMU(JGL))/RA
  RLATIG(JGL) = ASIN(RMU(JGL))
ENDDO

!*       3.    Computes poles values.
!              ---------------------

IF(NDGSUR >= 1)THEN

!DIR$ IVDEP
!OCL NOVREC
  DO JGL=1,NDGSUR
    RMU(1-JGL)=RMU(JGL)
    R1MU2(1-JGL)=R1MU2(JGL)
    R1MUI(1-JGL)=R1MUI(JGL)
    R1MUA(1-JGL)=R1MUA(JGL)
    RSQM2(1-JGL)=RSQM2(JGL)
    R1QM2(1-JGL)=R1QM2(JGL)
    RACTHE(1-JGL)=RACTHE(JGL)
    RLATIG(1-JGL)=RPI-RLATIG(JGL)
  ENDDO

!DIR$ IVDEP
!OCL NOVREC
  DO JGL=1,NDGSUR
    RMU(NDGLG+JGL)=RMU(NDGLG+1-JGL)
    R1MU2(NDGLG+JGL)=R1MU2(NDGLG+1-JGL)
    R1MUI(NDGLG+JGL)=R1MUI(NDGLG+1-JGL)
    R1MUA(NDGLG+JGL)=R1MUA(NDGLG+1-JGL)
    RSQM2(NDGLG+JGL)=RSQM2(NDGLG+1-JGL)
    R1QM2(NDGLG+JGL)=R1QM2(NDGLG+1-JGL)
    RACTHE(NDGLG+JGL)=RACTHE(NDGLG+1-JGL)
    RLATIG(NDGLG+JGL)=-RPI-RLATIG(NDGLG+1-JGL)
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SULEG',1,ZHOOK_HANDLE)
END SUBROUTINE SULEG
