SUBROUTINE SULFI(KULOUT,KNMAX,KNCMAX,PSTRET,LDIMST,KNIMES,KFACTD,KFACTR)

!**** *SULFI* - Routine to initialize characteristics for LFI files
!               (designed for rotation and contraction/dilatation matrixes)
!               Sous-programme de definition de quelques caracteristiques
!               des fichiers LFI (prevu pour les matrices de contraction,
!               dilatation ou rotation); a appeler avant l'ouverture

!     Purpose / But
!     -------------
!           Read namelist NAMLFI / Lit et verifie la namelist NAMLFI

!**   Interface
!     ---------
!        *CALL* *SULFI(...)*

!        Explicit arguments / Arguments explicites :
!        -------------------------------------------
! input / entree :
!              KULOUT   :  Logical unit for output
!                          Numero d'unite logique pour les impressions
!              KNMAX    :  Lower truncation
!              KNCMAX   :  Upper truncation
!              PSTRET   :  Stretching coefficient
! output / sortie :
!              LDIMST   :  Option for printing statistics
!                          Option d'impression de statistiques d'utilisation
!                            a la fermeture du fichier
!              KNIMES   :  Level for messages
!                          Niveau de messagerie attribue au fichier
!              KFACTD   :  Factor to enhance I/O on contr./dilat. matrixes
!                          Facteur multiplicatif attribue aux matrices de
!                            contraction et dilatation (pour accelerer les E/S)
!              KFACTR   :  Factor to enhance I/O on rotation matrix
!                          Facteur multiplicatif attribue a la matrice de
!                            rotation (pour accelerer les E/S)

!        Implicit arguments / Arguments implicites :
!        -------------------------------------------
!              Namelist NAMLFI (INPUT)
!              Comdeck YOMLFI (OUTPUT) :

!     Reference
!     ---------
!        Note de travail ARPEGE n12 et additifs

!     Author / Auteur
!     ---------------
!        D. Giard  *METEO-FRANCE*

!     Modifications
!     -------------
!        Original    : 92-12-08
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!        D. Giard    : 94-07-26  separate factors for dilatation and rotation
!                                one or several OPEN/CLOSE of matrix files
!        K. YESSAD   : 99-04-13  adequate defaults for NFACTD and NFACTR
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLFI   , ONLY : NIMES    ,NFACTD   ,NFACTR

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRET 
LOGICAL           ,INTENT(OUT)   :: LDIMST 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNIMES 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFACTD 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFACTR 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sufrag.intfb.h"

!     ------------------------------------------------------------------

! Impressions de quelques lignes a l'ouverture et a la fermeture
! some prints when opening or closing
! (0 : erreurs fatales/fatal errors  ; 2 : bavardages/chatch)

IF (LHOOK) CALL DR_HOOK('SULFI',0,ZHOOK_HANDLE)
NIMES = 1

! Facteur de granularite/Granularity factor

CALL SUFRAG(KNMAX,KNCMAX,PSTRET,NFACTD,NFACTR)
! * upper bound to 120 to be consistent with the limits of the LFI software.
NFACTD=MIN(MAX(NFACTD,1),120)
NFACTR=MIN(MAX(NFACTR,1),120)

! ------------------------------------------------------------------

!*       3.  Verifications/checkings
!        ---------------------------

WRITE(UNIT=KULOUT,FMT='(/,'' NAMELIST NAMLFI'')')
WRITE(UNIT=KULOUT,FMT=&
 & '(''   NFACTD ='',I3,''   NFACTR ='',I3,''   NIMES ='',I3,/)')&
 & NFACTD,NFACTR,NIMES  

! ------------------------------------------------------------------

!*       4.  End / Fin
!        -------------

LDIMST = .FALSE.
KNIMES = NIMES
KFACTD = NFACTD
KFACTR = NFACTR

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SULFI',1,ZHOOK_HANDLE)
END SUBROUTINE SULFI
