SUBROUTINE SULSFORC(KULOUT)

!**** *SULSFORC*   - Initialize forcing parameters

!     Purpose.
!     --------
!           Initialize YOMLSFORC and checkings

!**   Interface.
!     ----------
!        *CALL* *SULSFORC(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Documentation academic case and 1D model (SCUM)

!     Author.
!     -------
!        S. Malardel

!     Modifications.
!     --------------
!        Original : 06-01-04

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN    ,ONLY : NULNAM
USE YOM_YGFL  ,ONLY : NGFL_FORC
USE YOMCT0    ,ONLY : NSTOP
USE YOMDYN   ,ONLY : TSTEP

USE YOMLSFORC

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)   :: KULOUT ! logical unit of output file

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ITOT_FORC, IFORCECH
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "posnam.intfb.h"
#include "abor1.intfb.h"

!     ------------------------------------------------------------------

#include "namlsforc.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SULSFORC',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!         1. Set implicit default values for YOMLSFORC

LGEOST_UV_FRC = .FALSE.
RCORIO_FORC = 0.
NGEOST_UV_TIME = 0
NGEOST_U_DEB = 1
NGEOST_U_NUM = 0
NGEOST_V_DEB = 1
NGEOST_V_NUM = 0

LT_ADV_FRC = .FALSE.
NT_ADV_TIME = 0
NT_ADV_DEB = 1
NT_ADV_NUM = 0

LQV_ADV_FRC = .FALSE.
NQV_ADV_TIME = 0
NQV_ADV_DEB = 1
NQV_ADV_NUM = 0

LSW_FRC = .FALSE.
NLSW_TIME = 0
NLSW_DEB = 1
NLSW_NUM = 0

!         2. Read namelist for forcing

CALL POSNAM(NULNAM,'NAMLSFORC')
READ (NULNAM,NAMLSFORC)

!         3. Check namelist values

ITOT_FORC = NGEOST_U_NUM + NGEOST_V_NUM + NT_ADV_NUM + NLSW_NUM
IF (ITOT_FORC > NGFL_FORC) CALL ABOR1('Size of NGFL_FORC too small')

IFORCECH= NGEOST_UV_TIME*(NGEOST_U_NUM-1)
IF ( (IFORCECH/=0 ) .AND. (IFORCECH/=(NSTOP*INT(TSTEP))) ) THEN
  CALL ABOR1('NGEOST_UV_TIME is not an exact fraction of the the total simulation time')
ENDIF

IFORCECH= NT_ADV_TIME*(NT_ADV_NUM-1)
IF ( (IFORCECH/=0 ) .AND. (IFORCECH/=(NSTOP*INT(TSTEP))) ) THEN
  CALL ABOR1('NT_ADV_TIME is not an exact fraction of the the total simulation time')
ENDIF

IFORCECH= NQV_ADV_TIME*(NQV_ADV_NUM-1)
IF ( (IFORCECH/=0 ) .AND. (IFORCECH/=(NSTOP*INT(TSTEP))) ) THEN
  CALL ABOR1('NQV_ADV_TIME is not an exact fraction of the the total simulation time')
ENDIF

IFORCECH= NLSW_TIME*(NLSW_NUM-1)
IF ( (IFORCECH/=0 ) .AND. (IFORCECH/=(NSTOP*INT(TSTEP))) ) THEN
  CALL ABOR1('NLSW_TIME is not an exact fraction of the the total simulation time')
ENDIF

!         4. Check print

WRITE(UNIT=KULOUT,FMT='('' LGEOST_UV_FRC = '',L5,''RCORIO_FORC = '',E6.4           &
 & '' NGEOST_UV_TIME = '',I5,'' NGEOST_UV_DEB = '',I3,'' NGEOST_UV_NUM = '',I3 )') &
 & LGEOST_UV_FRC, RCORIO_FORC, NGEOST_UV_TIME, NGEOST_U_DEB, NGEOST_U_NUM

WRITE(UNIT=KULOUT,FMT='(''  LT_ADV_FRC = '',L5,                           &
 & '' NT_ADV_TIME = '',I5,'' NT_ADV_DEB = '',I3,'' NT_ADV_NUM = '',I3 )') &
 & LT_ADV_FRC, NT_ADV_TIME, NT_ADV_DEB, NT_ADV_NUM

WRITE(UNIT=KULOUT,FMT='(''  LQV_ADV_FRC = '',L5,                           &
 & '' NQV_ADV_TIME = '',I5,'' NQV_ADV_DEB = '',I3,'' NQV_ADV_NUM = '',I3 )') &
 & LQV_ADV_FRC, NQV_ADV_TIME, NQV_ADV_DEB, NQV_ADV_NUM

WRITE(UNIT=KULOUT,FMT='(''  LSW_FRC = '',L5,                           &
 & '' NLSW_TIME = '',I5,'' NLSW_DEB = '',I3,'' NLSW_NUM = '',I3 )') &
 & LSW_FRC, NLSW_TIME, NLSW_DEB, NLSW_NUM

! -----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SULSFORC',1,ZHOOK_HANDLE)
END SUBROUTINE SULSFORC
