SUBROUTINE SUMETRIC(KSMAX,PMET,PMETDER,PMETKE)

!**** *SUMETRIC* - Compute metric for spectral norms computation.

!     Purpose.
!     --------
!        To compute the metric used for spectral norms computation, 
!        and to initialise the zonal truncation.

!**   Interface.
!     ----------
!        *CALL* *SUMETRIC(KINDPP)

!        Explicit arguments :
!        --------------------
!          KSMAX   : zonal truncation
!          PMET    : metric for basic fields
!          PMETDER : metric for derivatives
!          PMETKE  : metric for kinetic energy

!        Implicit arguments :  See modules above
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------
!      None.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Ryad El Khatib *METEO-FRANCE*

!     Modifications.
!     --------
!        Original : 99-02-08 from SPNORM
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX
USE YOMLAP   , ONLY : RLAPDI   ,RLAPIN

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(OUT)   :: KSMAX 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMET(0:NSMAX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMETDER(0:NSMAX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMETKE(0:NSMAX) 
INTEGER(KIND=JPIM) :: JM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    ZONAL TRUNCATION
!              ----------------

IF (LHOOK) CALL DR_HOOK('SUMETRIC',0,ZHOOK_HANDLE)
KSMAX=NSMAX

!*       2.    METRIC
!              ------

DO JM=0,NSMAX
  PMET   (JM) = 1.0_JPRB
  PMETDER(JM) = -RLAPDI(JM)
  PMETKE (JM) = -RLAPIN(JM)
ENDDO

IF (LHOOK) CALL DR_HOOK('SUMETRIC',1,ZHOOK_HANDLE)
END SUBROUTINE SUMETRIC
