SUBROUTINE SUMP_DILA(KDGLG,KDGNH,KULOUT,&
 & KDGNHX,KDGNHL0,KDGNHL,KDGLL,KPROCGL,KGLG,KGLG0)

!**** *SUMP_DILA * - Routine to initialize parallel environment for conf 911
!                    Distribution for latitudes.

!     Purpose.
!     --------
!      see above

!**   Interface.
!     ----------
!        *CALL* *SUMP_DILA*

!        Explicit arguments :
!        --------------------

!          INPUT:
!           KDGLG     : DM-global number of latitudes.
!           KDGNH     : DM-global number of latitudes (North hem. only).
!           KULOUT    : Logical unit for prints.

!          OUTPUT:
!           KDGNHX    : Max of DM-local number of latitudes (North hem. only).
!           KDGNHL0   : DM-local number of latitudes for all procs
!                       (North hem. only).
!           KDGNHL    : DM-local number of latitudes for the current proc
!                       (North hem. only).
!           KDGLL     : DM-local number of latitudes for the current proc.
!           KPROCGL   : number of the proc. which treats the current latitude.
!           KGLG      : gives the DM-global latitude from the DM-local one
!                       (current processor only).
!           KGLG0     : gives the DM-global latitude from the DM-local one
!                       (all processors).

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.   NONE.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad (CNRM/GMAP)

!     Modifications.
!     --------------
!        Original   : July 2005
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MYSETW
USE YOMCT0   , ONLY : NPRGPNS  ,NPRGPEW  ,NPRTRW   ,NPRTRV   ,NPROC    ,&
 & NPRINTLEV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)   :: KDGLG
INTEGER(KIND=JPIM),INTENT(IN)   :: KDGNH
INTEGER(KIND=JPIM),INTENT(IN)   :: KULOUT
INTEGER(KIND=JPIM),INTENT(OUT)  :: KDGNHX
INTEGER(KIND=JPIM),INTENT(OUT)  :: KDGNHL0(NPRTRW)
INTEGER(KIND=JPIM),INTENT(OUT)  :: KDGNHL
INTEGER(KIND=JPIM),INTENT(OUT)  :: KDGLL
INTEGER(KIND=JPIM),INTENT(OUT)  :: KPROCGL(KDGLG)
INTEGER(KIND=JPIM),INTENT(OUT)  :: KGLG(KDGLG)
INTEGER(KIND=JPIM),INTENT(OUT)  :: KGLG0(KDGLG,NPRTRW)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IERR, JROC, IGLG, JGLLOC, JGLG

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMP_DILA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

! * Check that there is only A-level of parallelisation.
IERR=0
IF (NPRGPNS /= NPRTRW) IERR=IERR+1
IF (NPRGPNS /= NPROC) IERR=IERR+1
IF (NPRGPEW /= 1) IERR=IERR+1
IF (NPRTRV /= 1) IERR=IERR+1
IF (KDGLG /= 2*KDGNH) IERR=IERR+1

IF (IERR > 0) THEN
  WRITE(KULOUT,*) ' SUMP_DILA: in configuration 911, the following'
  WRITE(KULOUT,*) '  conditions must be fulfilled:'
  WRITE(KULOUT,*) '  NPRGPNS=NPRTRW=NPROC'
  WRITE(KULOUT,*) '  NPRGPEW=1; NPRTRV=1'
  WRITE(KULOUT,*) '  KDGLG=2*KDGNH'
  CALL ABOR1(' SUMP_DILA: ABOR1 CALLED')
ENDIF

!     ------------------------------------------------------------------

KDGNHX=KDGNH/NPRTRW+1
IF(NPRTRW*(KDGNH/NPRTRW) == KDGNH) KDGNHX=KDGNHX-1

! * Compute the number of latitudes (Northern hemisphere)
!   used by each processor.
DO JROC=1,NPRTRW
  KDGNHL0(JROC)=KDGNH/NPRTRW
ENDDO
DO JROC=1,MOD(KDGNH,NPRTRW)
  KDGNHL0(JROC)=KDGNHL0(JROC)+1
ENDDO
KDGNHL=KDGNHL0(MYSETW)
KDGLL=2*KDGNHL

! * KPROCGL: number of the processor which treats the current latitude.
!   KGLG(jglloc): gives the DM-global latitude.
KGLG(1:KDGLG)=-999
IGLG=0
DO JROC=1,NPRTRW
  DO JGLLOC=1,KDGNHL0(JROC)
    IGLG=IGLG+1
    KPROCGL(IGLG)=JROC
    KPROCGL(KDGLG+1-IGLG)=JROC
    IF (JROC == MYSETW) THEN
      KGLG(JGLLOC)=IGLG
      KGLG(KDGLL+1-JGLLOC)=KDGLG+1-IGLG
    ENDIF
    KGLG0(JGLLOC,JROC)=IGLG
    KGLG0(2*KDGNHL0(JROC)+1-JGLLOC,JROC)=KDGLG+1-IGLG
  ENDDO
ENDDO

!     ------------------------------------------------------------------

! * Printings:

IF (NPRINTLEV > 0) THEN
  WRITE(KULOUT,'(A)')
  WRITE(KULOUT,'(A)') ' --- SUMP_DILA:'
  WRITE(KULOUT,'(A)') ' KDGNHL0:'
  WRITE(KULOUT,'(16(1X,I4))') (KDGNHL0(JROC), JROC=1,NPRTRW)
  WRITE(KULOUT,'(A,1X,I4)') ' KDGNHL:',KDGNHL
  WRITE(KULOUT,'(A,1X,I4)') ' KDGLL:',KDGLL
  WRITE(KULOUT,'(A)') ' KPROCGL:'
  WRITE(KULOUT,'(16(1X,I4))') (KPROCGL(JGLG), JGLG=1,KDGLG)
  WRITE(KULOUT,'(A)') ' KGLG:'
  WRITE(KULOUT,'(16(1X,I4))') (KGLG(JGLLOC), JGLLOC=1,KDGLL)
  CALL FLUSH(KULOUT)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMP_DILA',1,ZHOOK_HANDLE)
END SUBROUTINE SUMP_DILA
