subroutine sumpini(kulnam)

!**** *sumpini*   - Determine if distributed or shared memory version is used

!     Purpose.
!     --------
!           Read in NAMPAR namelist

!**   Interface.
!     ----------
!        *call* *sumpini

!        explicit arguments : kulnam  - Fortran unit for namelist input
!        --------------------

!        implicit arguments :
!        --------------------

!     method.
!     -------
!        see documentation

!     externals.
!     ----------

!     reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     author.
!     -------
!        MPP Group *ECMWF*

! Modifications
! -------------
!   original : 95-10-01
!   Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!     - removal of LRPOLE in YOMCT0.
!   Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU, LRADVTORO.
!   Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!   Modified 00-04-17 by J. Vivoda: LMSEEP in 1 proc without calls to MPE
!   Modified 00-05-26 by E. Sevault: introduction of LMPOFF
!   Modified 00-09-01 by D.Dent    : MPE to MPL conversion
!   Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!   Modified 02-09-06 by C. Moussy    :LTENC    ,LALLTC   ,RTENC 
!   Modified 02-05-21 by Y.Tremolet: Added MPI groups
!   Modified 03-01-23 by R. El Khatib : Fix for LMPOFF
!   Modified 03-01-03 by J.Hague: Added MASS VF Option (N_VMASS)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!      Dec-2003 K. Yessad  remove NOPGMR
!   09-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LFULLIMP, LVSLWBC)
!   15-Mar-2005 R. El Khatib setup N_VMASS moved to SUJFH
!   Modified 04-11-25 by Y. Seity Add LAROME
!   Modified 05-05-17 by A. Alias Add NFRCORM
!   Modified 06-06-06 M. Ko"hler   Single Column Model option (LSCMEC) 
!   Modified 01-11-07 G.Mozdzynski Use NSPECRESMIN to control max value of NPRTRW
!   R. El Khatin  11-May-2007 LIMP_NOOLAP=.T. as default
!   R. El Khatib 27-Apr-2007   LIMP_NOOLAP=.F. if no message passing
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
!------------------------------------------------------------------
USE PARDIM   , ONLY : JPNULNAM
USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : NPOSTS   ,NHISTS   ,NPISPS   ,NGDITS   ,NSDITS   , &
 &                    NDHFGTS  ,NDHFZTS  ,NDHFDTS  ,NDHPTS   ,NMASSCONS, &
 &                    CNMEXP   ,CFCLASS  ,CTYPE    ,CNPPATH  ,CFDIRLST , &
 &                    CFPNCF , &
 &                    NCONF    ,NSTOP    ,N2DINI   ,NSPPR    ,NFRPOS   , &
 &                    NFRISP   ,NFRHIS   ,N3DINI   ,NFRGDI   ,NFRSDI   , &
 &                    NCNTVAR  ,NFRDHFG  ,NFRDHFZ  ,NFRDHFD  ,NFRDHP   , &
 &                    NFRCO    ,NFRCO_CHEM, NFRCO_VEG, LTM5CO2, NPROC    ,NOUTPUT  ,NPRINTLEV,           &
 &                    NFRMASSCON         ,NPRGPNS  ,NPRGPEW  ,NPRTRW   , &
 &                    NSPECRESMIN        , &
 &                    NPRTRNS  ,NPRTRV   ,NPRTRM   ,NPRTRN   ,LGRBOP   , &
 &                    LARPEGEF ,LSMSSIG  ,LSLAG    ,LAPRXPK  ,LTWOTL   , &
 &                    LARPEGEF_TRAJHR    ,LARPEGEF_TRAJBG,LVFE_REGETA  , &
 &                    LFPSPEC  ,LWROUTLAST , &
 &                    LFDBOP   ,LFPOS    ,LREGETA  ,LSPRT    ,LNHDYN   , &
 &                    LOLDPP   ,LGUESS   ,LOBS     ,LOBSC1   ,LSIMOB   , &
 &                    LELAM    ,LRPLANE  ,LFBDAP   ,LMPOFF   ,LBACKG   , &
 &                    LSCREEN  ,LREFOUT  ,LRUBC    ,LRFRIC   ,LMPDIAG  , &
 &                    L_SCREEN_CALL, L_SPLIT_SCREEN,NFSRHIS  , &
 &                    LALLOPR  ,LSITRIC  ,LNOBGON  ,LREFGEN  ,LRETCFOU , &
 &                    LWRTCFOU ,LRFOUTCNORM        ,LRGPTCNORM         , &
 &                    LVERCOR  ,LPC_OLD  ,LPC_NESC ,LPC_FULL ,LPC_CHEAP, &
 &                    LPC_NESCT,LTENC    ,LALLTC   ,RTENC    ,NINTERPTRAJ, &
 &                    NINTERPINCR,  LSCREEN_OPENMP, &
 &                    LAROME   ,NFRCORM, LSFORC, LSCMEC, LCOUPLO4, LSPBSBAL, &
 &                    NSHISTS, LARPEGEF_RDGP_INIT,LARPEGEF_RDGP_TRAJHR, &
 &                    LARPEGEF_RDGP_TRAJBG
USE YOMCT0B  , ONLY : LECMWF
USE YOMCVER  , ONLY : LVERTFE  ,NVSCH
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMIOPNH , ONLY : LTRAJNH
USE YOMMP    , ONLY : MYPROC   ,NPRCIDS  ,MP_TYPE  ,MBX_SIZE ,LEQ_REGIONS
USE YOMMPI   , ONLY : MINTET   ,MREALT
USE YOMSLPHY , ONLY : LSLPHY
USE YOMSLREP , ONLY : LSLADREP ,LVECADIN
USE YOMOP    , ONLY : CFPATH      !cvj otherwise ca chie dans namct0
USE YOMPLDSW , ONLY : LOPT_SCALAR  ,LOPT_RS6K
USE YOMTAG   , ONLY : MT_DISTRIBUTED_VECTOR
USE MPL_MODULE, ONLY : MPL_BUFFER_METHOD ,MPL_INIT          ,&
 &  MPL_GROUPS_CREATE ,JP_BLOCKING_STANDARD
USE DISTRIBUTED_VECTORS_MIX
USE OML_MOD, ONLY : OML_MAX_THREADS

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(INOUT) :: KULNAM 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDV_CHUNK_SIZE, isqr, ja, ib, iprtrv
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

! - the following ones are normally in different modules.
!   (why not "use" the corresponding module???)
INTEGER(KIND=JPIM) :: NTRACE_STATS
INTEGER(KIND=JPIM) :: NSTATS_MEM
INTEGER(KIND=JPIM) :: NPRNT_STATS
LOGICAL :: LSTATS_MEM,LSTATS_ALLOC
LOGICAL :: LSTATS,LSTATSCPU,LSYNCSTATS,LDETAILED_STATS,LXML_STATS,LLTRACE_STATS
LOGICAL :: LLSTATS_OMP, LLSTATS_COMMS,LBARRIER_STATS,LBARRIER_STATS2


!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gstats_label_ifs.intfb.h"
#include "posnam.intfb.h"
#include "su0dminit.intfb.h"
#include "sualmp0.intfb.h"
#include "sutag.intfb.h"

#include "nampar0.h"
#include "namct0.h"
#include "nam_distributed_vectors.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMPINI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

kulnam = jpnulnam
!     Open kulnam before going parallel for the benifit of the VPP
OPEN(KULNAM,ACTION="READ")
!     read NAMPAR namelist
!     --------------------

#ifdef RS6K
LOPT_SCALAR=.TRUE.
LOPT_RS6K=.TRUE.
#else
LOPT_SCALAR=.FALSE.
LOPT_RS6K=.FALSE.
#endif

nproc=0
nprgpns=0
nprgpew=0
nprtrw =0
nprtrv =0
nspecresmin=0
noutput=1
lmpoff=.false.
lmpdiag=.false.
idv_chunk_size = 1023
! set leq_regions=f to prevent access to n_regions in pe2set
! until nampar1 is read and setup_trans0 called
leq_regions=.false.

lstats = .false.
lstatscpu = .false.
lsyncstats = .false.
ldetailed_stats = .false.
lxml_stats = .false.
lbarrier_stats = .false.
lbarrier_stats2 = .true.
nstats_mem = 0
lstats_mem = .false.
lstats_alloc = .false.
ntrace_stats = 0
nprnt_stats=2
mp_type=JP_BLOCKING_STANDARD
mbx_size=0
nconf=1
LCOUPLO4  =.false.
nprintlev=0
call posnam(kulnam,'NAMPAR0')
read(kulnam,nampar0)
call posnam(kulnam,'NAMCT0')
read(kulnam,namct0)
call posnam(kulnam,'NAM_DISTRIBUTED_VECTORS')
read(kulnam,nam_distributed_vectors)

IF (.not.lstats) ldetailed_stats=.false.
IF (.not.lstats) lxml_stats=.false.
IF (.not.lstats) lbarrier_stats=.false.
IF (.not.lbarrier_stats) lbarrier_stats2=.false.

IF (ldetailed_stats) THEN
  llstats_omp = .true.
  llstats_comms = .true.
  lltrace_stats = .true.
ELSE
  llstats_omp = .false.
  llstats_comms = .false.
  lltrace_stats = .false.
ENDIF
  
!     Determine processor values if only nproc is supplied
!     ----------------------------------------------------

if (nprgpns > 0.and.nprgpew > 0) then
  
  if (nproc > 0.and.nprgpns*nprgpew /= nproc) call abor1(&
   & 'sumpini: inconsistency between nprgpns, nprgpew, and &
   & nproc')  
  nproc=nprgpns*nprgpew
else

! calculate nprgpns and nprgpew from supplied nproc
! this version selects most square-like distribution

  if (nproc == 0) nproc = 1
  isqr=int(sqrt(real(nproc,JPRB)))
  do ja=isqr,nproc
    ib=nproc/ja
    if (ja*ib == nproc) then
      nprgpns=max(ja,ib)
      nprgpew=min(ja,ib)
      exit
    endif
  enddo
endif

if ( nproc > 1 ) lmpoff=.false.

!    init mpl lib according to mp switch
! I don't know why we have to close and re-open KULNAM but it fails
! otherwise on the vpp5000 /MH
CLOSE(KULNAM)
IF (.NOT.LMPOFF) THEN
  CALL MPL_INIT(KOUTPUT=NPRINTLEV)
ENDIF
CALL c_drhook_init_signals(1) ! Get Dr.Hooks signals in case DR_HOOK was OFF
OPEN(KULNAM,ACTION="READ")

! Set defaults for NPRTRW and NPRTRV if necessary (where NPROC=NPRTRW*NPRTRV)
! Note NPRTRW will not be allowed to be greater than NSPECRESMIN

IF( nspecresmin == 0 )THEN
! nspecresmin was not specified in nampar0 so give it a large default value
  nspecresmin = nproc
ENDIF

if(nprtrw > 0) then
! nprtrw must have been set via the namelist read, check it is consistent
! i.e. nproc equals nprtrw * nprtrv
  if (nproc > 0.and.nprtrw*nprtrv /= nproc) call abor1(&
   & 'sumpini: inconsistency between nprtrw, nprtrv, and nproc')  
  if (nprtrw > nspecresmin ) call abor1(&
   & 'sumpini: nprtrw > nspecresmin')
else
  if(LOPT_SCALAR) then
!   set default values of nprtrw and nprtrv for scalar systems
    do iprtrv=4,nproc
      nprtrv=iprtrv
      nprtrw=nproc/nprtrv
      if( nprtrv*nprtrw /= nproc ) cycle
      if( nprtrv > nprtrw ) exit
      if( nprtrw > nspecresmin ) cycle
      if( nprtrw <= nspecresmin/(2*OML_MAX_THREADS()) ) exit
    enddo
!   go for approx square partitioning as backup
    if( nprtrv*nprtrw /= nproc .or. nprtrw > nspecresmin .or. nprtrv > nprtrw )then
      isqr=int(sqrt(real(nproc,JPRB)))
      do ja=isqr,nproc
        ib=nproc/ja
        if (ja*ib == nproc) then
          nprtrw=max(ja,ib)
          nprtrv=min(ja,ib)
          if (nprtrw > nspecresmin ) call abor1(&
           & 'sumpini: nprtrw (approx square value) > nspecresmin')
          exit
        endif
      enddo
    endif
  else
!   set default values of nprtrw and nprtrv for vector systems
    nprtrw=nproc
    nprtrv=1
    do iprtrv=1,nproc
      nprtrv=iprtrv
      nprtrw=nproc/nprtrv
      if( nprtrv*nprtrw /= nproc .or. nprtrw > nspecresmin )then
        cycle
      else
        exit
      endif
    enddo
  endif
endif

nprtrns=nprtrw
nprtrm =nprtrw
nprtrn =nprtrv

! Create communicators for MPI groups
IF (.NOT.LMPOFF) THEN
  CALL MPL_GROUPS_CREATE(nprtrw, nprtrv)
ENDIF

!    Allocate arrays used in initial phase of distributed memory control

call sualmp0

!    Sets up distributed memory environment if required

call su0dminit
if (.not.lmpoff) then
  call MPL_BUFFER_METHOD(kmp_type=mp_type,kmbx_size=mbx_size,KPROCIDS=NPRCIDS)
endif

call sutag

call gstats_setup(nproc,myproc,nprcids,&
 & lstats,lstatscpu,lsyncstats,ldetailed_stats,lbarrier_stats,lbarrier_stats2,&
 & llstats_omp,llstats_comms,lstats_mem,nstats_mem,lstats_alloc,&
 & lltrace_stats,ntrace_stats,nprnt_stats,lxml_stats)  
call gstats_psut
call gstats_label_ifs

call setup_distvec(nproc,myproc,mintet,mrealt,&
 & mt_distributed_vector,idv_chunk_size,nprcids)  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMPINI',1,ZHOOK_HANDLE)
end subroutine sumpini
