SUBROUTINE SUMTS

!**** *SUMTS* - SET UPS FOR TOVS RAD. TRANSFER CODE

!     PURPOSE.
!     -------
!       SET UP CONSTANTS FOR MODEL TO SATELLITE CODE
!       1) READS IN REGRESSION COEFFICIENTS FOR EXTRAPOLATION TO 000.1 mb
!       2) RETURNS F00.M00. PRESSURE LEVEL INFORMATION
!       3) SETS UP DUMMY CLOUD AND EMISSIVITY PARAMETERS

!**   INTERFACE.
!**   ---------
!       *CALL* * SUMTS*  (FROM SUPHY)

!**   EXTERNALS
!     ---------
!        RTTVI   - READ TOVS COEFFICIENTS (RT-CODE)

!     AUTHOR.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 04-08-19
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMTS   , ONLY : NSAT     ,NTYPE    ,LMTS     ,LMTSCL   ,NHLIM
USE YOMLUN   , ONLY : NULNAM   ,NULOUT

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "nammts.h"
#include "sufpphq.intfb.h"
#include "posnam.intfb.h"

IF (LHOOK) CALL DR_HOOK('SUMTS',0,ZHOOK_HANDLE)

! For only one satellite NSAT = 1
! This reflects the order in which coefficients are loaded in RTTVI
NSAT=1
NTYPE=7
NHLIM=-9999

CALL SUFPPHQ

CALL POSNAM(NULNAM,'NAMMTS')
READ(NULNAM,NAMMTS)

WRITE(UNIT=NULOUT,FMT=*) 'LMTS = ',LMTS, 'LMTSCL = ',LMTSCL, 'NSAT = ',NSAT, &
 & 'NTYPE = ',NTYPE, 'NHLIM = ',NHLIM

IF (LMTS) CALL PHRTSETUP

IF (LHOOK) CALL DR_HOOK('SUMTS',1,ZHOOK_HANDLE)

END SUBROUTINE SUMTS
