SUBROUTINE SUNCET13(KNMAXN,KNMAXX,KNCMAX)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!**** *SUNCET13* - Computes NCMAX for a threshold of 1.E-13,
!                  some stretching coefficients and some admissible truncations.

!**   Interface.
!     ----------
!        *CALL* *SUNCET13( ... )

!        Explicit arguments :
!        --------------------
!        INPUT :
!           KNMAXN    - Lower admissible truncation (<=20).
!           KNMAXX    - Upper admissible truncation (>=332).

!        OUTPUT :
!           KNCMAX    - Upper truncation.

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------

!     Externals.
!     ----------
!        none.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (METEO-FRANCE/CNRM/GMAP).

!     Modifications.
!     --------------
!        Original: MAY 1999.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAXN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAXX 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNCMAX(KNMAXN:KNMAXX,6) 
! * OUTPUT:
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

! * c=1.5

IF (LHOOK) CALL DR_HOOK('SUNCET13',0,ZHOOK_HANDLE)
KNCMAX( 20,1)=  47
KNCMAX( 23,1)=  52
KNCMAX( 25,1)=  59
KNCMAX( 29,1)=  63
KNCMAX( 31,1)=  65
KNCMAX( 32,1)=  71
KNCMAX( 35,1)=  79
KNCMAX( 39,1)=  79
KNCMAX( 41,1)=  83
KNCMAX( 47,1)=  95
KNCMAX( 49,1)=  95
KNCMAX( 52,1)=  99
KNCMAX( 53,1)= 105
KNCMAX( 59,1)= 119
KNCMAX( 63,1)= 119
KNCMAX( 65,1)= 127
KNCMAX( 71,1)= 132
KNCMAX( 79,1)= 143
KNCMAX( 83,1)= 149
KNCMAX( 84,1)= 159
KNCMAX( 89,1)= 159
KNCMAX( 95,1)= 169
KNCMAX( 99,1)= 179
KNCMAX(105,1)= 191
KNCMAX(107,1)= 191
KNCMAX(119,1)= 212
KNCMAX(127,1)= 239
KNCMAX(132,1)= 239
KNCMAX(143,1)= 249
KNCMAX(149,1)= 255
KNCMAX(159,1)= 269
KNCMAX(161,1)= 287
KNCMAX(165,1)= 287
KNCMAX(169,1)= 287
KNCMAX(179,1)= 319
KNCMAX(191,1)= 319
KNCMAX(199,1)= 332
KNCMAX(212,1)= 359
KNCMAX(215,1)= 359
KNCMAX(239,1)= 399
KNCMAX(249,1)= 416
KNCMAX(255,1)= 425
KNCMAX(265,1)= 449
KNCMAX(269,1)= 449
KNCMAX(287,1)= 479
KNCMAX(299,1)= 485
KNCMAX(319,1)= 532
KNCMAX(323,1)= 532
KNCMAX(332,1)= 539

! ------------------------------------------------------------------

! * c=2.0

KNCMAX( 20,2)=  65
KNCMAX( 23,2)=  71
KNCMAX( 25,2)=  79
KNCMAX( 29,2)=  89
KNCMAX( 31,2)=  89
KNCMAX( 32,2)=  95
KNCMAX( 35,2)=  99
KNCMAX( 39,2)= 107
KNCMAX( 41,2)= 119
KNCMAX( 47,2)= 127
KNCMAX( 49,2)= 132
KNCMAX( 52,2)= 143
KNCMAX( 53,2)= 143
KNCMAX( 59,2)= 159
KNCMAX( 63,2)= 161
KNCMAX( 65,2)= 165
KNCMAX( 71,2)= 179
KNCMAX( 79,2)= 199
KNCMAX( 83,2)= 212
KNCMAX( 84,2)= 212
KNCMAX( 89,2)= 215
KNCMAX( 95,2)= 239
KNCMAX( 99,2)= 239
KNCMAX(105,2)= 249
KNCMAX(107,2)= 255
KNCMAX(119,2)= 287
KNCMAX(127,2)= 299
KNCMAX(132,2)= 319
KNCMAX(143,2)= 332
KNCMAX(149,2)= 359
KNCMAX(159,2)= 383
KNCMAX(161,2)= 383
KNCMAX(165,2)= 383
KNCMAX(169,2)= 383
KNCMAX(179,2)= 416
KNCMAX(191,2)= 431
KNCMAX(199,2)= 449
KNCMAX(212,2)= 479
KNCMAX(215,2)= 479
KNCMAX(239,2)= 532
KNCMAX(249,2)= 575
KNCMAX(255,2)= 575
KNCMAX(265,2)= 599
KNCMAX(269,2)= 599
KNCMAX(287,2)= 639
KNCMAX(299,2)= 665
KNCMAX(319,2)= 719
KNCMAX(323,2)= 719
KNCMAX(332,2)= 719

! ------------------------------------------------------------------

! * c=2.5

KNCMAX( 20,3)=  83
KNCMAX( 23,3)=  95
KNCMAX( 25,3)=  99
KNCMAX( 29,3)= 119
KNCMAX( 31,3)= 119
KNCMAX( 32,3)= 119
KNCMAX( 35,3)= 127
KNCMAX( 39,3)= 143
KNCMAX( 41,3)= 143
KNCMAX( 47,3)= 159
KNCMAX( 49,3)= 165
KNCMAX( 52,3)= 179
KNCMAX( 53,3)= 179
KNCMAX( 59,3)= 191
KNCMAX( 63,3)= 212
KNCMAX( 65,3)= 212
KNCMAX( 71,3)= 239
KNCMAX( 79,3)= 249
KNCMAX( 83,3)= 255
KNCMAX( 84,3)= 265
KNCMAX( 89,3)= 287
KNCMAX( 95,3)= 287
KNCMAX( 99,3)= 299
KNCMAX(105,3)= 319
KNCMAX(107,3)= 319
KNCMAX(119,3)= 359
KNCMAX(127,3)= 383
KNCMAX(132,3)= 399
KNCMAX(143,3)= 416
KNCMAX(149,3)= 431
KNCMAX(159,3)= 479
KNCMAX(161,3)= 479
KNCMAX(165,3)= 479
KNCMAX(169,3)= 485
KNCMAX(179,3)= 511
KNCMAX(191,3)= 539
KNCMAX(199,3)= 575
KNCMAX(212,3)= 599
KNCMAX(215,3)= 639
KNCMAX(239,3)= 665
KNCMAX(249,3)= 719
KNCMAX(255,3)= 719
KNCMAX(265,3)= 749
KNCMAX(269,3)= 749
KNCMAX(287,3)= 799
KNCMAX(299,3)= 832
KNCMAX(319,3)= 899
KNCMAX(323,3)= 899
KNCMAX(332,3)= 959

! ------------------------------------------------------------------

! * c=3.0

KNCMAX( 20,4)=  99
KNCMAX( 23,4)= 119
KNCMAX( 25,4)= 119
KNCMAX( 29,4)= 132
KNCMAX( 31,4)= 143
KNCMAX( 32,4)= 143
KNCMAX( 35,4)= 159
KNCMAX( 39,4)= 165
KNCMAX( 41,4)= 179
KNCMAX( 47,4)= 191
KNCMAX( 49,4)= 199
KNCMAX( 52,4)= 212
KNCMAX( 53,4)= 212
KNCMAX( 59,4)= 239
KNCMAX( 63,4)= 249
KNCMAX( 65,4)= 249
KNCMAX( 71,4)= 269
KNCMAX( 79,4)= 299
KNCMAX( 83,4)= 319
KNCMAX( 84,4)= 319
KNCMAX( 89,4)= 332
KNCMAX( 95,4)= 359
KNCMAX( 99,4)= 359
KNCMAX(105,4)= 383
KNCMAX(107,4)= 399
KNCMAX(119,4)= 425
KNCMAX(127,4)= 449
KNCMAX(132,4)= 479
KNCMAX(143,4)= 499
KNCMAX(149,4)= 532
KNCMAX(159,4)= 575
KNCMAX(161,4)= 575
KNCMAX(165,4)= 575
KNCMAX(169,4)= 599
KNCMAX(179,4)= 639
KNCMAX(191,4)= 665
KNCMAX(199,4)= 681
KNCMAX(212,4)= 719
KNCMAX(215,4)= 749
KNCMAX(239,4)= 799
KNCMAX(249,4)= 832
KNCMAX(255,4)= 852
KNCMAX(265,4)= 899
KNCMAX(269,4)= 899
KNCMAX(287,4)= 959
KNCMAX(299,4)= 999
KNCMAX(319,4)=1065
KNCMAX(323,4)=1065
KNCMAX(332,4)=1151

! ------------------------------------------------------------------

! * c=3.5

KNCMAX( 20,5)= 119
KNCMAX( 23,5)= 132
KNCMAX( 25,5)= 143
KNCMAX( 29,5)= 159
KNCMAX( 31,5)= 161
KNCMAX( 32,5)= 165
KNCMAX( 35,5)= 179
KNCMAX( 39,5)= 199
KNCMAX( 41,5)= 212
KNCMAX( 47,5)= 239
KNCMAX( 49,5)= 239
KNCMAX( 52,5)= 249
KNCMAX( 53,5)= 249
KNCMAX( 59,5)= 269
KNCMAX( 63,5)= 287
KNCMAX( 65,5)= 299
KNCMAX( 71,5)= 319
KNCMAX( 79,5)= 359
KNCMAX( 83,5)= 359
KNCMAX( 84,5)= 383
KNCMAX( 89,5)= 383
KNCMAX( 95,5)= 416
KNCMAX( 99,5)= 425
KNCMAX(105,5)= 449
KNCMAX(107,5)= 449
KNCMAX(119,5)= 499
KNCMAX(127,5)= 532
KNCMAX(132,5)= 575
KNCMAX(143,5)= 599
KNCMAX(149,5)= 639
KNCMAX(159,5)= 647
KNCMAX(161,5)= 665
KNCMAX(165,5)= 665
KNCMAX(169,5)= 681
KNCMAX(179,5)= 719
KNCMAX(191,5)= 767
KNCMAX(199,5)= 799
KNCMAX(212,5)= 852
KNCMAX(215,5)= 852
KNCMAX(239,5)= 959
KNCMAX(249,5)= 971
KNCMAX(255,5)= 999
KNCMAX(265,5)=1065
KNCMAX(269,5)=1065
KNCMAX(287,5)=1151
KNCMAX(299,5)=1151
KNCMAX(319,5)=1249
KNCMAX(323,5)=1249
KNCMAX(332,5)=1279

! ------------------------------------------------------------------

! * c=4.0

KNCMAX( 20,6)= 143
KNCMAX( 23,6)= 149
KNCMAX( 25,6)= 159
KNCMAX( 29,6)= 179
KNCMAX( 31,6)= 191
KNCMAX( 32,6)= 191
KNCMAX( 35,6)= 212
KNCMAX( 39,6)= 239
KNCMAX( 41,6)= 239
KNCMAX( 47,6)= 255
KNCMAX( 49,6)= 265
KNCMAX( 52,6)= 287
KNCMAX( 53,6)= 287
KNCMAX( 59,6)= 319
KNCMAX( 63,6)= 332
KNCMAX( 65,6)= 340
KNCMAX( 71,6)= 383
KNCMAX( 79,6)= 399
KNCMAX( 83,6)= 416
KNCMAX( 84,6)= 416
KNCMAX( 89,6)= 449
KNCMAX( 95,6)= 479
KNCMAX( 99,6)= 485
KNCMAX(105,6)= 511
KNCMAX(107,6)= 532
KNCMAX(119,6)= 575
KNCMAX(127,6)= 639
KNCMAX(132,6)= 639
KNCMAX(143,6)= 681
KNCMAX(149,6)= 719
KNCMAX(159,6)= 749
KNCMAX(161,6)= 749
KNCMAX(165,6)= 767
KNCMAX(169,6)= 799
KNCMAX(179,6)= 832
KNCMAX(191,6)= 899
KNCMAX(199,6)= 959
KNCMAX(212,6)= 959
KNCMAX(215,6)= 971
KNCMAX(239,6)=1065
KNCMAX(249,6)=1151
KNCMAX(255,6)=1151
KNCMAX(265,6)=1199
KNCMAX(269,6)=1199
KNCMAX(287,6)=1279
KNCMAX(299,6)=1332
KNCMAX(319,6)=1439
KNCMAX(323,6)=1439
KNCMAX(332,6)=1457

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNCET13',1,ZHOOK_HANDLE)
END SUBROUTINE SUNCET13
