SUBROUTINE SUOPH

!**** *SUOPH* - Routine to prepare input/output handling

!     Purpose.   To set up common block YOMOP which contains file-
!     --------   handling parameters.

!**   Interface.
!     ----------
!        *CALL* *SUOPH*

!        Explicit arguments :  None.
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals :
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-03-01
!        F.Rabier and J.N Thepaut : 91-01-15
!        Y.Bouteloup : 92-04-15 Poles in standard frame except
!          for NCONF equal 951 and 952 (CPREP2 and CPREP3)
!        R. El Khatib : 92-06-12 History Witness File
!        R. El Khatib : 92-09-16 parameterization of trajectory file
!                       name in output
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!        D. Giard     : 93-08-24 File for bias : filtered guess - guess
!        Modified by M.Hanmrud    : 93-07-05 Use of ARPEGE file optional
!        Modified by R. EL Khatib : 94-04-29 Subroutine SUFRAME
!        Modified by M.Hamrud     : 94-09-19 New FDB software
!        Modified by M.Fisher     : 95-02-08 added CFNBGV and CFNCGL
!        Modified by J.N Thepaut  : 96-02-14 added CFNFGI and CFNANI
!        E. Andersson : 96-04-25 added CFNFGIG and CFNANIG
!        Modified by M. Janiskova : 96-11-13 added CFNTC for transmission
!                                            coef. for simpl.rad.scheme
!        Modified by M.Fisher     : 97-01-28 added CFNSKFY and CFNSKFZ
!        R. El Khatib : 98-01-30  : "exact" majoration for MAXFIELD
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - removal of LLPOLE, LLPOLE1 and these variables in CALL SUFRAME.
!        R. El Khatib : 00-06-05 : LTIMEP0
!        M. Janousek: 01-11-26 : NCADFORM - format of FA file frame
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R.Buizza      23-May-2005 Added CFNVAREPS (input file with VAREPS accum fields)
!        G. Desroziers and K. Yessad (sept 2005):
!         - split option LTRAJHR into LTRAJHR_ALTI and LTRAJHR_SURF.
!         - adapt option LTRAJHR to METEO-FRANCE configurations.
!        JD Gril : 03-feb-2006 : default NCADFORM=1
!        R. El Khatib : 23-Oct-2008 : CFPATH now a directory ending with a '/'
!          and used also for CFNHWF + merge with sueoph + cleanings + CEFNLSH in namelist
!        E.Holm        13-Nov-2008 Added CFNBGHRSH/GG, background 
!                                  at outer loop resolution.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NFLEVG   ,NMSMAX   ,NSMAX    ,&
 & NDGUNG   ,NDGUXG   ,NDLUNG   ,NDLUXG
USE YOMCT0   , ONLY : CNMEXP   ,CFCLASS  ,CTYPE    ,NCONF    ,&
 & NSTOP    ,LFDBOP   ,LARPEGEF ,NPROC    ,LELAM
USE YOMCT0B  , ONLY : LECMWF
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMOP    , ONLY : CFNSH    ,CFNTLSH  ,CFNGG    ,CFNISH   ,CFNIGG   ,&
 & CFNPL    ,CFNVAREPS,CNMCA    ,CFNRF    ,CFNAN    ,CFNGR    ,CFNINSH  ,&
 & CFNINGG  ,CFNFUN   ,CFNSNC   ,CFNIT0   ,CFNPT0   ,CFNHWF   ,CFNOPA   ,&
 & CFNDEP   ,CFNGSH   ,CFNGGG   ,CFNDDH   ,CFPATH   ,CFNBS    ,CFNBGV   ,&
 & CFNCGL   ,CFNFGI   ,CFNANI   ,CFNFGIG  ,CFNANIG  ,CFNSKFY  ,CFNSKFZ  ,&
 & CFNTC    ,LINC     ,NFDBREF  ,NLPATH   ,MAXFIELD ,CFANS    ,LTIMEP0  ,&
 & NCADFORM ,CEFLS    ,CEFNLSH  ,LBCINC,&
 & CFNTRAJHRGRID,CFNTRAJHRSPEC,CFNTRAJHRSURF,CFNTRAJHR,&
 & CFNTRAJBGGRID,CFNTRAJBGSPEC,CFNTRAJBG,CFNBGHRSH,CFNBGHRGG
USE YOMGEM   , ONLY : NLOENG   ,NMENG    ,VALH     ,&
 & VBH      ,NSTTYP   ,RMUCEN   ,RLOCEN   ,RSTRET   ,VP00  
USE YOMLEG   , ONLY : RMU
USE YOMLUN   , ONLY : NULNAM   ,NULERR   ,NULOUT
USE YOMGRB   , ONLY : NENSFNB ,NLOCGRB ,&
 & NSTREAM, NSYSTEM ,NMETHOD ,NREFERENCE ,NSEC1   ,&
 & NCONSENSUS ,NDWD ,NMFR ,NNCEP ,NUKM, NLEG
USE YOMDYN   , ONLY : TSTEP
USE SURFACE_FIELDS_MIX   , ONLY : NSURFL
USE YOMVAR   , ONLY : LFDBERR
USE YOMMP    , ONLY : NWRTOUT
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, Y_IOSTREAM_FDB
USE FDBSUBS_MOD  , ONLY : IINITFDBSUBS, ISETVALFDBSUBS
USE YEMDIM   , ONLY : NBZONG   ,NBZONL   ,NEDOM
USE YEMGEO   , ONLY : ERPK     ,ELONC    ,ELATC    ,ELON1    ,ELAT1    ,&
 & ELON2    ,ELAT2    ,ELON0    ,ELAT0    ,EDELX    ,EDELY    ,ELX      ,&
 & ELY      ,EXWN     ,EYWN     ,LMAP     ,LMRT

IMPLICIT NONE

CHARACTER(LEN=1) :: CLEG
CHARACTER(LEN=2) :: CLTIME
CHARACTER(LEN=3) :: CLENSFNB
CHARACTER(LEN=4) :: CLFDB
CHARACTER(LEN=5) :: CLSYSTEM, CLMETHOD, CLENSNUM
CHARACTER(LEN=8) :: CLREFERENCE, CLDATE
INTEGER(KIND=JPIM) :: ILMO(12),IOPROCS(NPROC)

INTEGER(KIND=JPIM) :: ID1, IDAYINC, IDD, IDMOPL, IINDAT, &
 & IM1, IMM, INBCSP, INBPDG, INGRIB, IPUILA, IOP,JROC,&
 & ISECRUN, ISSSSS, ISTAT, ISTRON, IYYYY, IYYYY1,ILEN 

LOGICAL :: LLGARD=.TRUE.
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namoph.h"

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "suarpio.intfb.h"
#include "suframe.intfb.h"
#include "sueframe.intfb.h"
#include "updcal.intfb.h"

#include "fcttim.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUOPH',0,ZHOOK_HANDLE)

!*       0.    READ NAMELIST
!              -------------

NLPATH=INDEX(CFPATH,' ')-1

!        0.1 Set implicit default values

LINC=.FALSE.
LBCINC=.FALSE.
IF (NLPATH >=1) THEN
  CFNHWF=CFPATH(1:NLPATH)//'hiwif'
ELSE
  CFNHWF='hiwif'
ENDIF
LTIMEP0 = (NCONF == 1.OR.NCONF == 302)
NCADFORM=1
CEFNLSH='ELSCF'//CNMEXP(1:4)//'ALBC'

!        0.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
ENDIF

CALL POSNAM(NULNAM,'NAMOPH')
READ(NULNAM,NAMOPH)

WRITE(NULOUT,'('' LINC = '',L2,'' CFNHWF = '',A12,'' LTIMEP0 = '',L2,&
 & '' LBCINC = '',L2)') LINC, CFNHWF, LTIMEP0, LBCINC
WRITE(NULOUT,'('' NCADFORM = '',I2)') NCADFORM
ILEN=INDEX(CEFNLSH,' ')-1
WRITE(NULOUT,'('' CEFNLSH = '',A)') CEFNLSH(1:ILEN)

!     ------------------------------------------------------------------

!*       1.    SET FILE NAME FORMATS
!              ---------------------

!*       1.1  WORK FILES AND DIAGNOSTIC OUTPUT

IF (NLPATH >=1) THEN
  CFNSH=CFPATH(1:NLPATH)//'ICMSH'//CNMEXP(1:4)//'0000'//'   '
ELSE
  CFNSH ='ICMSH'//CNMEXP(1:4)//'0000'//'   '
ENDIF
CFNTLSH='ICMTL'//CNMEXP(1:4)//'0000'//'   '
CFNGG  ='ICMGG'//CNMEXP(1:4)//'0000'//'   '
CFNRF  ='ICMRF'//CNMEXP(1:4)//'0000'//'   '
CFNFUN ='ICMCF'//CNMEXP(1:4)//'0000'//'   '
CFNSNC ='ICMSC'//CNMEXP(1:4)//'0000'//'   '
CFNIT0 ='ICMD0'//CNMEXP(1:4)//'0000'//'   '
CFNPT0 ='ICMS0'//CNMEXP(1:4)//'0000'//'   '
CFNOPA ='ECMWF'//CNMEXP(1:4)//'0000'//'   '

!*       1.2  DIAGNOSTIC OUTPUT FILE NAMES
!*            PX: PRESSURE LEVEL, CONTROL VARIABLE
!*            MX: MODEL LEVEL, CONTROL VARIABLE
!*            MG: MODEL LEVEL, GRADIENT
!*            MD: MODEL LEVEL, DEPARTURES (for 151)
!*            MB: MODEL LEVEL, RANDOM BACKGROUND VECTOR
!*            ME: MODEL LEVEL, (SCALED) EIGENVECTOR OF THE HESSIAN
CFNPL ='PX'//CNMEXP(1:4)//'000'//'+'//'0000'
CFNAN ='MX'//CNMEXP(1:4)//'000'//'+'//'0000'
CFNGR ='MG'//CNMEXP(1:4)//'000'//'+'//'0000'
CFNINSH ='MISH'//CNMEXP(1:4)//'000'//'+'//'0000'
CFNINGG ='MIGG'//CNMEXP(1:4)//'000'//'+'//'0000'
CFNDEP='MD'//CNMEXP(1:4)//'000'//'+'//'0000'
CFNBGV='MB'//CNMEXP(1:4)//'000'
CFNCGL='ME'//CNMEXP(1:4)//'000'
CFNSKFY='MY'//CNMEXP(1:4)//'000'
CFNSKFZ='MZ'//CNMEXP(1:4)//'000'

!*       1.3 TRANSMISSION COEFFICIENTS FILE NAME

CFNTC='ICMRC'//CNMEXP(1:4)//'0000'//'   '

!*       1.4 HIGH RESOLUTION TRAJECTORY FILE NAME

IF (LECMWF) THEN
  CFNTRAJHRGRID='TRAJHR00/trajgrid'
  CFNTRAJHRSPEC='TRAJHR00/trajspec'
  CFNTRAJHRSURF='TRAJHR00/trajsurf000'
ELSE
  CFNTRAJHRGRID='TRAJHR00_trajgrid'
  CFNTRAJHRSPEC='TRAJHR00_trajspec'
  CFNTRAJHRSURF='TRAJHR00_trajsurf000'
ENDIF
CFNTRAJHR='TRAJ'//CNMEXP(1:4)//'0000    '

!*       1.5 BACKGROUND FILE NAME

IF (LECMWF) THEN
  CFNTRAJBGGRID='TRAJBG00/bckggrid'
  CFNTRAJBGSPEC='TRAJBG00/bckgspec'
ELSE
  CFNTRAJBGGRID='TRAJBG00_bckggrid'
  CFNTRAJBGSPEC='TRAJBG00_bckgspec'
ENDIF

! ky: I am not yet sure of the name of this file, to be validated
!     and maybe fixed later according to the validations we expect to
!     do with a 4DVAR-LTRAJHR OLIVE script.
CFNTRAJBG=CFNRF

!*       1.6 BACKGROUND AT OUTER LOOP RESOLUTION
!            The high resolution background is needed at all times
!            where the increments are added.

CFNBGHRSH='ICMSH'//CNMEXP(1:4)//'BGHR'//'   '
CFNBGHRGG='ICMGG'//CNMEXP(1:4)//'BGHR'//'   '



!     ------------------------------------------------------------------

!*       2.    PREPARE INITIAL DATA UNITS.
!              ----------------------------

CFNISH='ICMSH'//CNMEXP(1:4)//'INIT'//'   '
CFNIGG='ICMGG'//CNMEXP(1:4)//'INIT'//'   '
CFNGSH='ICMSH'//CNMEXP(1:4)//'IMIN'//'   '
CFNGGG='ICMGG'//CNMEXP(1:4)//'IMIN'//'   '
CFNDDH='DHFTT'//CNMEXP(1:4)//'IMIN'//'   '
CFNBS ='ICMSH'//CNMEXP(1:4)//'BIAS'//'   '
CFANS ='ICMSH'//CNMEXP(1:4)//'SURF'//'   '

! Initial condition file with accum fields for VAREPS
CFNVAREPS ='ICVEP'//CNMEXP(1:4)//'INIT'//'   '

CFNFGI ='ICMSH'//CNMEXP(1:4)//'FGIN'//'   '
CFNANI ='ICMSH'//CNMEXP(1:4)//'ANIN'//'   '
CFNFGIG='ICMGG'//CNMEXP(1:4)//'FGIN'//'   '
CFNANIG='ICMGG'//CNMEXP(1:4)//'ANIN'//'   '

! Lateral Boundary coupling data units

CEFLS='ELSCF'//CNMEXP(1:4)//'COMP'//'    '

!     ------------------------------------------------------------------

!*       3.    INITIALISE STANDARD FRAME OF *FA*
!              ---------------------------------

IF(LARPEGEF) THEN

! Old-style distibuted I/O setup still partly used
  MAXFIELD=MAX(NSURFL,1)
  CALL SUARPIO(1)

  IF (LELAM) THEN

    CNMCA='CADRE.STANDARD.E'
    ISTRON=MAX(0,MIN(11,NMSMAX,NSMAX)-1)

    CALL SUEFRAME(CNMCA,LMAP,NMSMAX,NSMAX,NDGLG,NDLON,LLGARD,LMRT,ELON0,ELAT0,&
     & ELONC,ELATC,EDELX,EDELY,ELON1,ELAT1,ELON2,ELAT2,ERPK,ELX,ELY,EXWN,EYWN,&
     & NDLUNG,NDLUXG,NDGUNG,NDGUXG,NBZONL,NBZONG,NFLEVG,VP00,VALH,VBH,NEDOM,&
     & ISTRON)

  ELSE

!*         3.1  MODIFICATION OF IMPLICITS OPTIONS

    IF (NSMAX <= 11) THEN
      INGRIB=2
      INBPDG=24
      INBCSP=24
      ISTRON=NSMAX-1
      IPUILA=0
      IDMOPL=10
      CALL FAGIOT(INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
    ENDIF

!*       3.2  DEFINE STANDARD FRAME

    CNMCA = 'CADRE.STANDARD  '
    CALL SUFRAME(CNMCA,RMUCEN,RLOCEN,RSTRET,NSTTYP,NSMAX,NDGLG,NDLON,&
     & NLOENG(1),NMENG(1),RMU(1),LLGARD,NFLEVG,VP00,VALH,VBH)  

  ENDIF

ENDIF

! -------------------------------------------------------------------

!*       5.    PREPARE FOR USE OF FIELDS DATA BASE.
!              ------------------------------------

IF (LFDBOP .OR. LFDBERR) THEN

!*       5.1   SET UP FDB
  IF (IABS(NCONF)/100 == 8) THEN
    ISECRUN = NSTOP * TSTEP
    ISSSSS  = ISECRUN+NSSSSS
    IDAYINC = ISSSSS/86400
    IDD     = NDD(NINDAT)
    IMM     = NMM(NINDAT)
    IYYYY   = NCCAA(NINDAT)
    CALL UPDCAL(IDD,IMM,IYYYY,IDAYINC,ID1,IM1,IYYYY1,ILMO,-1)
    IINDAT  = ID1+100*IM1+10000*IYYYY1
    ISSSSS  = MOD(ISSSSS,86400)
  ELSE
    IINDAT  = NINDAT
    ISSSSS  = NSSSSS
  ENDIF

  ISTAT = IINITFDBSUBS()
  CLFDB='fdb '
  IF(NSTREAM == 1090 .OR. NSTREAM == 1220 .OR. &
   & NSTREAM == 1200 .OR. NSTREAM == 1201 ) THEN
    CLFDB='seas'
  ENDIF
  IOP=0
  DO JROC=1,NPROC,NWRTOUT
    IOP = IOP+1
    IOPROCS(IOP) = JROC
  ENDDO
  CALL SETUP_IOSTREAM(Y_IOSTREAM_FDB,CDTYPE='FDB',CDREF=CLFDB,CDMODE='w',&
   & KIOPROCS=IOPROCS(1:IOP))
  NFDBREF = Y_IOSTREAM_FDB%IFILE_HANDLE

  IF(ISTAT /= 0) THEN
    WRITE(NULERR,'(A,I4)') 'ERROR OPENING FDB,ISTAT = ',ISTAT
    CALL ABOR1('SUOPH-ERROR OPENING FDB')
  ENDIF

  WRITE(CLDATE,'(I8.8)') IINDAT
  WRITE(CLTIME,'(I2.2)') NCTH(ISSSSS)
  ISTAT = ISETVALFDBSUBS(NFDBREF,'class',  CFCLASS(1:2))
  IF(NSTREAM == 1026) THEN
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','scda')
  ELSEIF(NSTREAM == 1027) THEN
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','scwv')
  ELSEIF(NSTREAM == 1028) THEN
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','dcda')
  ELSEIF(NSTREAM == 1029) THEN
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','dcwv')
  ELSEIF(NSTREAM == 1030) THEN
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','enda')
  ELSE
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream', 'oper')
  ENDIF
  ISTAT = ISETVALFDBSUBS(NFDBREF,'domain', 'g')
  ISTAT = ISETVALFDBSUBS(NFDBREF,'expver', CNMEXP(1:4))
  ISTAT = ISETVALFDBSUBS(NFDBREF,'date',   CLDATE)
  ISTAT = ISETVALFDBSUBS(NFDBREF,'time',   CLTIME)
  ISTAT = ISETVALFDBSUBS(NFDBREF,'type',   CTYPE(1:2))
  SELECT CASE (NSTREAM)
  CASE(0) ! Atmospheric model
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream', 'oper')
  CASE(1024) ! Daily archive hindcast
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','dahc')
    WRITE(CLREFERENCE,'(I8.8)') NREFERENCE
    ISTAT = ISETVALFDBSUBS(NFDBREF,'refdate',CLREFERENCE)
  CASE(1025) ! Atmospheric model
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','oper')
  CASE(1026) ! Atmospheric model (short cutoff)
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','scda')
  CASE(1028) ! Atmospheric model (delayed cutoff)
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','dcda')
  CASE(1030) ! Ensemble Data Assimilation
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','enda')
  CASE(1035) ! Ensemble prediction system
    WRITE( CLEG, '(i1.1)' ) NLEG
    ISTAT = ISETVALFDBSUBS(NFDBREF,'leg',CLEG)
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','enfo')
    IF( NLOCGRB == 27 .OR. NLOCGRB == 30 ) THEN
      IF(NREFERENCE > 0 ) THEN
        WRITE(CLREFERENCE,'(I8.8)') NREFERENCE
        ISTAT = ISETVALFDBSUBS(NFDBREF,'hdate',CLDATE)
      ENDIF
    ENDIF
  CASE(1036) ! Sensitivity forecast
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','sens')
  CASE(1037) ! Multianalysis ensemble data
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','maed')
    IF(NCONSENSUS == 0) THEN
      IF(NDWD == 1) THEN
        ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','edzw')
      ELSEIF(NMFR == 1) THEN
        ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','lfpw')
      ELSEIF(NNCEP == 1) THEN
        ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','kwbc')
      ELSEIF(NUKM == 1) THEN
        ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','egrr')
      ENDIF
    ELSE
      ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','consensus')
    ENDIF
  CASE(1033) ! Ensemble forecast hindcasts
    WRITE( CLEG, '(i1.1)' ) NLEG
    ISTAT = ISETVALFDBSUBS(NFDBREF,'leg',CLEG)
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','enfh')
    IF( NLOCGRB == 26 .OR. NLOCGRB == 30 ) THEN
      WRITE(CLREFERENCE,'(I8.8)') NREFERENCE
      ISTAT = ISETVALFDBSUBS(NFDBREF,'date',CLREFERENCE)
      ISTAT = ISETVALFDBSUBS(NFDBREF,'hdate',CLDATE)
    ENDIF
  CASE(1039) ! Ensemble forecast hindcasts
    WRITE( CLEG, '(i1.1)' ) NLEG
    ISTAT = ISETVALFDBSUBS(NFDBREF,'leg',CLEG)
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','efhc')
    IF( NLOCGRB == 26 .OR. NLOCGRB == 30 ) THEN
      WRITE(CLREFERENCE,'(I8.8)') NREFERENCE
      ISTAT = ISETVALFDBSUBS(NFDBREF,'refdate',CLREFERENCE)
    ENDIF
  CASE(1090) ! Seasonal forecast
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','seas')
    WRITE(CLSYSTEM,'(I5.5)') NSYSTEM
    IF(NSEC1(38) /= 1) CLSYSTEM='off'
    ISTAT = ISETVALFDBSUBS(NFDBREF,'system',CLSYSTEM)
    WRITE(CLMETHOD,'(I5.5)') NMETHOD
    ISTAT = ISETVALFDBSUBS(NFDBREF,'method',CLMETHOD)
    WRITE(CLENSNUM,'(I5.5)') NENSFNB
    ISTAT = ISETVALFDBSUBS(NFDBREF,'number',CLENSNUM)
  CASE(1200) ! Real-time Monthly forecast
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','mnfc')
    ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','ecmf')
    WRITE(CLSYSTEM,'(I5.5)') NSYSTEM
    IF(NSEC1(38) /= 1) CLSYSTEM='off'
    ISTAT = ISETVALFDBSUBS(NFDBREF,'system',CLSYSTEM)
    WRITE(CLMETHOD,'(I5.5)') NMETHOD
    ISTAT = ISETVALFDBSUBS(NFDBREF,'method',CLMETHOD)
  CASE(1201) ! Hindcasts Monthly forecast
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','mnfh')
    ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','ecmf')
    WRITE(CLSYSTEM,'(I5.5)') NSYSTEM
    IF(NSEC1(38) /= 1) CLSYSTEM='off'
    ISTAT = ISETVALFDBSUBS(NFDBREF,'system',CLSYSTEM)
    WRITE(CLMETHOD,'(I5.5)') NMETHOD
    ISTAT = ISETVALFDBSUBS(NFDBREF,'method',CLMETHOD)
    WRITE(CLREFERENCE,'(I8.8)') NREFERENCE
    ISTAT = ISETVALFDBSUBS(NFDBREF,'refdate',CLREFERENCE)
  CASE(1220) ! Multi-model seasonal forecast
    ISTAT = ISETVALFDBSUBS(NFDBREF,'stream','mmsf')
    ISTAT = ISETVALFDBSUBS(NFDBREF,'origin','ecmf')
    WRITE(CLSYSTEM,'(I5.5)') NSYSTEM
    IF(NSEC1(38) /= 1) CLSYSTEM='off'
    ISTAT = ISETVALFDBSUBS(NFDBREF,'system',CLSYSTEM)
    WRITE(CLMETHOD,'(I5.5)') NMETHOD
    ISTAT = ISETVALFDBSUBS(NFDBREF,'method',CLMETHOD)
    WRITE(CLENSNUM,'(I5.5)') NENSFNB
    ISTAT = ISETVALFDBSUBS(NFDBREF,'number',CLENSNUM)
  CASE DEFAULT
    CALL ABOR1('SUOPH - UNKNOWN STREAM')
  END SELECT
  IF( NENSFNB > 0 .OR. NSTREAM == 1030 ) THEN
    WRITE(CLENSFNB,'(I3.3)') NENSFNB
    ISTAT = ISETVALFDBSUBS(NFDBREF,'number',CLENSFNB)
  ENDIF

ENDIF

! -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUOPH',1,ZHOOK_HANDLE)
END SUBROUTINE SUOPH
