SUBROUTINE SUPTRGPPC(KGPPC,KGPPCO_Y,KGPPCO_RR,KGPPCO_GL,KGPPCO_GM,KGPPCO_WW,&
 & KGPPCF_U ,KGPPCF_V,KGPPCF_T,KGPPCF_PD,KGPPCF_VD,KGPPCF_SP, &
 & KGPPCF_CP,KGPPCF_CPSI,&
 & KGPPCF_NHX,KGPPCF_UP,KGPPCF_VP,KGPPCF_TP,KGPPCF_GFLP,KGPPCF_BBC, &
 & KFGPPC)

! SUPTRGPPC - Pointers of PTRGPPC.
!             See PTRGPPC for comments of dummy arguments.
!             KGPPCO_.. are used for LPC_OLD;
!             KGPPCF_.. are used for LPC_CHEAP.
!             KFGPPC is the number of 2D fields contained in GPPCBUF.
!             (matches with NFGPPC of YOMDIM).

!             These pointers must later to be put in GMV+GMVS (new attributes
!             to create), and also GFL for KGPPCF_GFLP.
!             They are linked to the different PC schemes and they are no
!             longer purely NH pointers, so they have been renamed in Jan 2008.

! Purpose
! -------

! Interface
! ---------

! Externals
! ---------

! Method
! ------
!   Pointers for GPPCBUF buffer

! Reference
! ---------

! Author
! ------
!      Nov-2001 J. VIVODA (with name ald/adiab/ESTR_POINTERS).

! Modifications
! -------------
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR)
!   06-Jul-2004 K. Yessad  Becomes an ARP/ALD setup routine
!                          (old name was ESTR_POINTERS).
!   08-Jan-2008 N. Wedi and K. Yessad: different dev for NH model and PC scheme
!   23-Sep-2008 K. Yessad: remove lpc_xidt
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LNHDYN   ,LPC_OLD  ,LPC_CHEAP
USE YOMSLPHY , ONLY : LSLPHY
USE YOMDYN   , ONLY : LSLINLC1
USE YOMDYNA  , ONLY : LRDBBC   ,LNHX
USE YOMDIM   , ONLY : NFLEVG   ,NUNDEFLD
USE YOM_YGFL , ONLY : YGFL
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!------------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPC
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCO_Y
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCO_RR
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCO_GL
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCO_GM
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCO_WW
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_U
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_V
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_T
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_PD
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_VD
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_SP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_CP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_CPSI
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_NHX
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_UP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_VP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_TP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_GFLP
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGPPCF_BBC
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFGPPC

!------------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDEFAULT, ITMP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPTRGPPC',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

IDEFAULT = NUNDEFLD

! 0. Set default values
KGPPC           = IDEFAULT
KGPPCO_Y        = IDEFAULT
KGPPCO_RR       = IDEFAULT
KGPPCO_GL       = IDEFAULT
KGPPCO_GM       = IDEFAULT
KGPPCO_WW       = IDEFAULT
KGPPCF_U        = IDEFAULT
KGPPCF_V        = IDEFAULT
KGPPCF_T        = IDEFAULT
KGPPCF_PD       = IDEFAULT
KGPPCF_VD       = IDEFAULT
KGPPCF_SP       = IDEFAULT
KGPPCF_CP       = IDEFAULT
KGPPCF_CPSI     = IDEFAULT
KGPPCF_NHX      = IDEFAULT
KGPPCF_UP       = IDEFAULT
KGPPCF_VP       = IDEFAULT
KGPPCF_TP       = IDEFAULT
KGPPCF_GFLP     = IDEFAULT
KGPPCF_BBC      = IDEFAULT

! 1. Set_pointers

KGPPC= 1
ITMP = 1
KFGPPC=0

IF( LPC_OLD.AND.LNHDYN )THEN

  KGPPCO_Y = KGPPC
  KGPPCO_RR = KGPPCO_Y+NFLEVG
  KGPPCO_GL = KGPPCO_RR+NFLEVG
  KGPPCO_GM = KGPPCO_GL+NFLEVG
  KGPPCO_WW = KGPPCO_GM+NFLEVG
  KFGPPC = KFGPPC+5*NFLEVG

ELSEIF( LPC_CHEAP ) THEN

  KGPPCF_U = ITMP;    ITMP = ITMP + NFLEVG
  KGPPCF_V = ITMP;    ITMP = ITMP + NFLEVG
  KGPPCF_T = ITMP;    ITMP = ITMP + NFLEVG
  KGPPCF_SP= ITMP;    ITMP = ITMP + NFLEVG
  KGPPCF_CP= ITMP;    ITMP = ITMP + NFLEVG
  KFGPPC = KFGPPC+5*NFLEVG
  IF (LSLINLC1) THEN
    KGPPCF_CPSI= ITMP;    ITMP = ITMP + NFLEVG
    KFGPPC = KFGPPC+NFLEVG
  ENDIF

  IF( LSLPHY ) THEN
    ! GFL provisionally put here (to be put in GFL(PC) later).
    KGPPCF_UP  = ITMP;    ITMP = ITMP + NFLEVG
    KGPPCF_VP  = ITMP;    ITMP = ITMP + NFLEVG
    KGPPCF_TP  = ITMP;    ITMP = ITMP + NFLEVG
    KGPPCF_GFLP= ITMP;    ITMP = ITMP + YGFL%NUMFLDS*NFLEVG
    KFGPPC = KFGPPC+3*NFLEVG+YGFL%NUMFLDS*NFLEVG
  ENDIF

  IF( LNHDYN )THEN
    KGPPCF_PD = ITMP; ITMP = ITMP + NFLEVG
    KGPPCF_VD = ITMP; ITMP = ITMP + NFLEVG
    KFGPPC = KFGPPC+2*NFLEVG
    IF (LNHX) THEN
      KGPPCF_NHX  = ITMP; ITMP = ITMP + NFLEVG
      KFGPPC = KFGPPC+NFLEVG
    ENDIF
    IF( LRDBBC ) THEN
      KGPPCF_BBC = ITMP; ITMP = ITMP + 3
      KFGPPC = KFGPPC+3      
    ENDIF
  ENDIF

ENDIF

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPTRGPPC',1,ZHOOK_HANDLE)

END SUBROUTINE SUPTRGPPC

