!OCL NOEVAL
SUBROUTINE SURAYFRIC

!     ------------------------------------------------------------------
!**** *SURAYFRIC*   - Initialize Rayleigh friction

!     Purpose.
!     --------

!         Computes the Rayleigh friction coefficient RKRF.

!**   Interface.
!     ----------
!        *CALL* *SURAYFRIC

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE/ALADIN DOCUMENTATION

!     Author.
!     -------
!        K. Yessad (CNRM/GMAP), after some code previously in SUHDF
!         (coded at ECMWF by A. Untch).
!        This is the old part 3 of SUHDF.

!     Modifications.
!     --------------
!        Original : August 2005.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMGEM   , ONLY : VBH      ,VAH      ,VP00
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LRFRIC
USE YOMDYN   , ONLY : RKRF, NMAXLEVRF, RRFPLM, RRFZ1

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JLEV
REAL(KIND=JPRB) :: ZPRESH(0:NFLEVG)
REAL(KIND=JPRB) :: ZPRS, ZRFH, ZRFHH, ZRFP0, ZRFPLM, ZRFTAU, ZRFZ1
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURAYFRIC',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    RAYLEIGH FRICTION
!              -----------------

! * Remark: this piece of code is fully consistent with
!   lvertfe=F, ndlnpr=0, laprxpk=T
!   where prehyd(l)=0.5*prehyd(lbar-1)+0.5*prehyd(lbar)
!   and not completely consistent with the other cases
!   for lvertfe,ndlnpr,laprxpk.

NMAXLEVRF=0
IF(LRFRIC) THEN
  ZRFHH=7._JPRB
  ZRFP0=1.E+5_JPRB
! Reference height for profile now set by namelist input
  ZRFZ1=RRFZ1
  ZRFH =7.7_JPRB
  ZRFTAU=3._JPRB*86400._JPRB

! Rayleigh friction only for p < RRFPLM, set by namelist
  ZRFPLM=RRFPLM

  RKRF(:)=0.0_JPRB
  IF(0.5_JPRB*VAH(1) < ZRFPLM) THEN
    ZPRESH(0)=VAH(0)+VBH(0)*VP00
    DO JLEV=1,NFLEVG
      ZPRESH(JLEV)=VAH(JLEV)+VBH(JLEV)*VP00
      ZPRS=0.5_JPRB*(ZPRESH(JLEV)+ZPRESH(JLEV-1))
      IF(ZPRS < ZRFPLM) then
        NMAXLEVRF=JLEV
        RKRF(JLEV)=(1.0_JPRB-TANH((ZRFZ1-ZRFHH*LOG(ZRFP0/ZPRS))/ZRFH))/ZRFTAU
      ENDIF
    ENDDO
  ELSE
    LRFRIC=.FALSE.
    WRITE(UNIT=NULOUT,FMT='('' NO RAYLEIGH FRICTION APPLIED ''&
     & ,''BECAUSE THE TOP OF THE MODEL IS BELOW '',F4.2 &
     & ,''hPa. LRFRIC SET TO FALSE!'')')  ZRFPLM/100._JPRB  
  ENDIF
  WRITE(NULOUT,'(A)')
  WRITE(NULOUT,'(A)') ' --- SURAYFRIC:'
  WRITE(NULOUT,'(A)') ' RAYLEIGH FRICTION: RKRF(JLEV)'
  WRITE(NULOUT,'(5E24.14)')  (RKRF(JLEV),JLEV=1,NFLEVG)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURAYFRIC',1,ZHOOK_HANDLE)
END SUBROUTINE SURAYFRIC
