!OCL  NOPREEX
SUBROUTINE SURCOF(KNMAX,PBASCO,PBASSI)

!**** *SURCOF - Computation of a basis of sinus-cosinus for independant
!               Fourier transforms.

!     Purpose.
!     --------

!     Computation of a basis of sinus-cosinus for independant
!     Fourier transforms.

!**   Interface.
!     ----------
!        *CALL* *SURCOF(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KNMAX     - truncation.

!        OUTPUT:
!          PBASCO    - basis of cosinus.
!          PBASSI    - basis of sinus.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about spectral transforms.

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP), after some SUGEM2 code. 

!     Modifications.
!     --------------
!        Original : FEB 1999
!        Modified by G> Radnoti 99/11/07  ALADIN geometry
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LELAM
USE YOMCST   , ONLY : RPI
USE YOMDIM   , ONLY : NDGLG    ,NDGSAL   ,NDGENl  
USE YOMGEM   , ONLY : NLOENG   ,NSTAGP   ,NGPTOT
USE YOMMP    , ONLY : MY_REGION_EW   ,NSTA     ,NONL     ,MYLATS   ,NPTRFLOFF

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAX 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBASCO(KNMAX,NGPTOT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBASSI(KNMAX,NGPTOT) 
INTEGER(KIND=JPIM) :: IGLG
INTEGER(KIND=JPIM) :: IROF
INTEGER(KIND=JPIM) :: JGL
INTEGER(KIND=JPIM) :: JLON
INTEGER(KIND=JPIM) :: JM
INTEGER(KIND=JPIM) :: ISTLAT
INTEGER(KIND=JPIM) :: IENDLAT
INTEGER(KIND=JPIM) :: ISTLON
INTEGER(KIND=JPIM) :: IENDLON

REAL(KIND=JPRB) :: ZDPISNL
REAL(KIND=JPRB) :: ZINCR
REAL(KIND=JPRB) :: ZLAM
REAL(KIND=JPRB) :: ZLAMFM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

!*  1.    COMPUTATION.
!         ------------

IF (LHOOK) CALL DR_HOOK('SURCOF',0,ZHOOK_HANDLE)
IF (LELAM) THEN
  DO JM=1,KNMAX
    ISTLAT=1
    IENDLAT=NDGENL

    IROF=1
    DO JGL =ISTLAT,IENDLAT
      IGLG=MYLATS(JGL)
      ISTLON=NSTA(NPTRFLOFF+JGL,MY_REGION_EW)
      IENDLON=NSTA(NPTRFLOFF+JGL,MY_REGION_EW)+NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1
      IF ((IGLG <= 0).OR.(IGLG >= NDGLG+1)) THEN
        ZINCR=RPI
      ELSE
        ZINCR=0._JPRB
      ENDIF
      ZDPISNL=2.0_JPRB*RPI/REAL(NLOENG(IGLG),JPRB)
      DO JLON=ISTLON,IENDLON
        ZLAM=ZINCR+ZDPISNL*REAL(JLON-1,JPRB)
        ZLAMFM=ZLAM*REAL(JM,JPRB)
        PBASCO(JM,IROF)=COS(ZLAMFM)
        PBASSI(JM,IROF)=SIN(ZLAMFM)      
        IROF=IROF+1
      ENDDO
    ENDDO
  ENDDO
ELSE
  DO JM=1,KNMAX
    DO JGL=NDGSAL,NDGENL
      IGLG=MYLATS(JGL)
      IF ((IGLG <= 0).OR.(IGLG >= NDGLG+1)) THEN
        ZINCR=RPI
      ELSE
        ZINCR=0.0_JPRB
      ENDIF
      ZDPISNL= 2.0_JPRB*RPI/REAL(NLOENG(IGLG),JPRB)
      IROF=NSTAGP(JGL)-1
      DO JLON = NSTA(NPTRFLOFF+JGL,MY_REGION_EW), &
         & NSTA(NPTRFLOFF+JGL,MY_REGION_EW)+NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1  
        IROF=IROF+1
        ZLAM=ZINCR+ZDPISNL*REAL(JLON-1,JPRB)
        ZLAMFM=ZLAM*REAL(JM,JPRB)
        PBASCO(JM,IROF)=COS(ZLAMFM)
        PBASSI(JM,IROF)=SIN(ZLAMFM)
      ENDDO
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURCOF',1,ZHOOK_HANDLE)
END SUBROUTINE SURCOF
