SUBROUTINE SURCOFTC

!**** *SURCOFTC - Computation of a basis of sinus-cosinus for independant
!                 Fourier transforms for truncation NTCMAX.

!     Purpose.
!     --------

!     Computation of a basis of sinus-cosinus for independant
!     Fourier transforms for truncation NTCMAX.
!     Is currently used to transform transmission coefficients
!     from Fourier space to grid-point space (or the inverse)
!     inside the grid-point calculations.

!**   Interface.
!     ----------
!        *CALL* *SURCOFTC(...)

!        Explicit arguments : none
!        --------------------

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about spectral transforms.

!     Externals.
!     ----------
!       Calls SURCOF 

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP).

!     Modifications.
!     --------------
!        Original : FEB 1999
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR
USE YOMDIM   , ONLY : NTCMAX
USE YOMGEM   , ONLY : NGPTOT
USE YOMLUN   , ONLY : NULOUT
USE YOMFFTTC , ONLY : RBASCOTC ,RBASSITC

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "surcof.intfb.h"

! ------------------------------------------------------------------

!*  1. ALLOCATIONS.
!   ---------------

IF (LHOOK) CALL DR_HOOK('SURCOFTC',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1 .OR. LALLOPR
IU = NULOUT

IF (ALLOCATED(RBASCOTC)) THEN
  IF(LLP)WRITE(IU,8) 'RBASCOTC  '
  CALL ABOR1(' SURCOFTC: ABOR1 CALLED.')
ELSE
  ALLOCATE(RBASCOTC(NTCMAX,NGPTOT))
  IF(LLP)WRITE(IU,9) 'RBASCOTC ',SIZE(RBASCOTC ),SHAPE(RBASCOTC )
ENDIF

IF (ALLOCATED(RBASSITC)) THEN
  IF(LLP)WRITE(IU,8) 'RBASSITC  '
  CALL ABOR1(' SURCOFTC: ABOR1 CALLED.')
ELSE
  ALLOCATE(RBASSITC(NTCMAX,NGPTOT))
  IF(LLP)WRITE(IU,9) 'RBASSITC ',SIZE(RBASSITC ),SHAPE(RBASSITC )
ENDIF

8 FORMAT(1X,'ARRAY ',A10,' IS ALREADY ALLOCATED ')
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

! ------------------------------------------------------------------

!*  2. COMPUTATION.
!   ---------------

CALL SURCOF(NTCMAX,RBASCOTC,RBASSITC)

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURCOFTC',1,ZHOOK_HANDLE)
END SUBROUTINE SURCOFTC
