SUBROUTINE SURES(KULOUT)

!**** *SURES*   - Routine to initialize restart control

!     Purpose.
!     --------
!           Initialize restart control common
!**   Interface.
!     ----------
!        *CALL* *SURES(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMRES

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        David Dent     *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-06-01
!        Modified by R. El Khatib : 93-04-09 : time stamp controlled by LSDHM
!        Modified by R. El Khatib : 93-04-02 Set-up defaults controlled by LECMWF
!        Modified by D.Dent       : 96-07-29 Initialise signal handler
!        Modified by D.Dent       : 02-06-10 LFASTRES option
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified by R. El Khatib : 04-08-12 : print LFASTRES
!        Modified by R. El Khatib : 04-10-12 : path for cp in movies restart
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULNAM
USE YOMCT0   , ONLY : LSMSSIG
USE YOMCT0B  , ONLY : LECMWF
USE YOMMP    , ONLY : MYPROC
USE YOMDYN   , ONLY : TSTEP
USE YOMRES   , ONLY : JPNWST   ,NRESTS   ,N1RFS    ,N2RFS    ,NFRRES   ,&
          & NFLSTOP  ,NFLREST  ,LSDHM    ,COPY     ,LDELRES

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
INTEGER(KIND=JPIM) :: J

REAL(KIND=JPRB) :: ZUNIT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"

#include "namres.h"
!      ----------------------------------------------------------------

!*       1.    SET DEFAULT VALUES.
!              -------------------

!        1.1 Set implicit default values

!     ZUNIT  : NUMBER OF SECONDS IN TIME UNIT
IF (LHOOK) CALL DR_HOOK('SURES',0,ZHOOK_HANDLE)
ZUNIT=3600._JPRB

NFRRES=-48
N1RFS  =1
N2RFS  =1
DO J=0,JPNWST
  NRESTS(J)=0
ENDDO
LSDHM=.TRUE.
LDELRES=.FALSE.
COPY="cp"

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
  LSDHM=.FALSE.
  LDELRES=.TRUE.
  NFRRES=1
  NRESTS(0)=-11
  NRESTS(1)=-6
  NRESTS(2)=-12
  NRESTS(3)=-18
  NRESTS(4)=-24
  NRESTS(5)=-30
  NRESTS(6)=-36
  NRESTS(7)=-42
  NRESTS(8)=-48
  NRESTS(9)=-54
  NRESTS(10)=-60
  NRESTS(11)=-66
ENDIF
!      ----------------------------------------------------------------

!*       2.    Modifies default values.
!              ------------------------

CALL POSNAM(NULNAM,'NAMRES')
READ(NULNAM,NAMRES)

!  allow for restart frequency to be in hours
IF(NFRRES < 0.AND.TSTEP > 0.0_JPRB) THEN
  NFRRES= (REAL(-NFRRES,JPRB)*ZUNIT+0.5_JPRB)/TSTEP
ENDIF

!      -----------------------------------------------------------

!*       3.    Print final values.
!              -------------------

WRITE(UNIT=KULOUT,FMT='('' NRESTS = '',40(1X,I1,1X))')NRESTS
WRITE(UNIT=KULOUT,FMT='('' NFRRES = '',1X,I6)')NFRRES
WRITE(UNIT=KULOUT,FMT='('' LSDHM = '',L2)') LSDHM
WRITE(UNIT=KULOUT,FMT='('' LDELRES = '',L2)') LDELRES
WRITE(UNIT=KULOUT,FMT='('' COPY = '',A10)') COPY

!      -----------------------------------------------------------

!*       4.    Call signal handler 
!              -------------------

NFLSTOP=0
NFLREST=0
IF(LSMSSIG) CALL IFSSIG(NFLSTOP,NFLREST,MYPROC)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURES',1,ZHOOK_HANDLE)
END SUBROUTINE SURES
