SUBROUTINE SURGRI(KULNAM,KULOUT,KDGL,KDGSA,KDGEN,KLOEN)

!**** *SURGRI*  - Routine to read or compute NLOEN from Namelist

!     Purpose.
!     --------
!        Reads from namelist NRGRI,which corresponds to NLOEN without
!        extra-rows, this interface is necessary because
!     static dimensioning seems necessary for namelists ; compute also
!     the reduced grid according to the allowed rate of aliasing given
!     in namelist

!**   Interface.
!     ----------

!     *CALL* SURGRI(KULNAM,KULOUT,KDGL,KLOEN)

!        Explicit arguments :
!        --------------------
!     * INPUT:
!     KULNAM:  LOGICAL UNIT FOR NAMELIST
!     KULOUT:  LOGICAL UNIT FOR OUTPUT
!     KDGL  :  NUMBER OF GAUSSIAN LATITUDES (as NDGLG of YOMDIM)
!     KDGSA :  as NDGSAG of YOMDIM
!     KDGEN :  as NDGENG of YOMDIM

!     * OUTPUT:
!     KLOEN :  as NLOEN of YOMGEM

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department DOcumentation of the IFS

!     Author.
!     -------
!        Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-03-18
!        Modified : 93-04-28 by R. El Khatib (common block for NAMRGRI)
!        Modified by R. El Khatib  :93-05-06 Set-up through command line
!        Modified : 93-06-2  by P. Courtier imposes symmetric grid
!        Modified : 93-02-07 by R. El Khatib (computation of the reduced grid)
!        Modified : 00-10-25 by M.hamrud (remove dead code)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIM   , ONLY : JPMXGL
USE YOMDIM   , ONLY : NDGNH
USE YOMRGRI  , ONLY : NRGRI

IMPLICIT NONE

! * INPUT:
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGSA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KULNAM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGL 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLOEN(KDGSA:KDGEN) 
! * OUTPUT:
INTEGER(KIND=JPIM) :: ILOTOT, J, JGL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"

#include "namrgri.h"

!     ------------------------------------------------------------------

!*         0. READ NAMELIST
!          ----------------

IF (LHOOK) CALL DR_HOOK('SURGRI',0,ZHOOK_HANDLE)
IF(KDGL >= JPMXGL)THEN
  CALL ABOR1('SURGRI:KDGL >= JPMXGL')
ENDIF

CALL POSNAM(KULNAM,'NAMRGRI')
READ(KULNAM,NAMRGRI)
WRITE(KULOUT,FMT='('' COMMON YOMRGRI'')')
!     ------------------------------------------------------------------

!*         3. MODIFY REDUCED GRID ACCORDING TO NAMELIST
!          --------------------------------------------

CALL POSNAM(KULNAM,'NAMRGRI')
READ(KULNAM,NAMRGRI)
DO J=1,(KDGL+1)/2
  KLOEN(J)=NRGRI(J)
ENDDO
DO J=1,KDGL/2
  KLOEN(KDGL+1-J)=NRGRI(J)
ENDDO

IF(KDGSA <= 0)THEN
  KLOEN(0)=KLOEN(1)
  KLOEN(KDGL+1)=KLOEN(KDGL)
ENDIF

DO JGL=1,NDGNH
  WRITE(KULOUT,FMT='(''   NRGRI('',I3,'')='',I3)')JGL,NRGRI(JGL)
ENDDO
ILOTOT=0
DO JGL=1,KDGL
  ILOTOT=ILOTOT+NRGRI(JGL)
ENDDO
WRITE(KULOUT,FMT='('' NUMBER OF POINTS ON REDUCED GRID = '',I6)') ILOTOT
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURGRI',1,ZHOOK_HANDLE)
END SUBROUTINE SURGRI
