SUBROUTINE SURIP(KULOUT)

!**** *SURIP * - Routine to initialize the common YOMRIP

!     Purpose.
!     --------
!           Initialize and print the common YOMRIP

!**   Interface.
!     ----------
!        *CALL* *SURIP(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit of the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMRIP
!        COMMON YOMARG

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        Modified DEC 1992 by K. YESSAD: changing RDTFLS2 into RTDT (=PTDT).
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!        Modified by R. El Khatib  :93-05-06 Set-up through command line
!        Modified : 95-11-16 Ryad El Khatib (Fullpos LFPART2 without arguments)
!        Modified : 99-04-19 J.M. Piriou (Moon radiation).
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMARG   , ONLY : NARGS    ,NUDATE   ,NUSSSS
USE YOMLUN   , ONLY : NULNAM
USE YOMRIP   , ONLY : NINDAT    ,NSSSSS   ,NSTADD   ,NSTASS   ,&
 & NYMLPPGP, NYMLPPSP, &
 & RTIMST   ,RSTATI   ,RTIMTR   ,RHGMT    ,REQTIM   ,&
 & RSOVR    ,RDEASO   ,RDECLI   ,RWSOVR   ,RIP0     ,&
 & RCODEC   ,RSIDEC   ,RCOVSR   ,RSIVSR   ,RDTSA    ,&
 & RDTSA2   ,RDTS62   ,RDTS22   ,RTDT     ,&
 & RDECLU   ,RTMOLT   ,RIP0LU   ,RCODECLU ,RSIDECLU,RCOVSRLU,RSIVSRLU  
USE YOMCT0   , ONLY : LFPART2
USE YOMCT0B  , ONLY : LECMWF

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"

#include "namrip.h"
#include "fcttim.h"
!     ------------------------------------------------------------------

!*       1.    Initialize YOMRIP.
!              ------------------

!        1.1 Set implicit default values

IF (LHOOK) CALL DR_HOOK('SURIP',0,ZHOOK_HANDLE)
NINDAT=19880826
NSSSSS=43200

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
ENDIF

!*       1.3   READ NAMELIST.
IF (NARGS == 0.AND..NOT.LFPART2) THEN
  CALL POSNAM(NULNAM,'NAMRIP')
  READ(NULNAM,NAMRIP)

!*       1.4   OVERWRITE NAMELIST WITH COMMAND LINE ARGUMENTS
ELSE
  NINDAT=NUDATE
  NSSSSS=NUSSSS
ENDIF

!*       1.5   SET REST OF COMMON TO 0
NSTADD=0
NSTASS=0
NYMLPPGP=0
NYMLPPSP=0
RSTATI=0._JPRB
!* RTIMST AND RTIMTR ARE INITIALIZED IN SUCST
RTIMST=0._JPRB
RTIMTR=0._JPRB
!* ALL THOSE VARIABLES ARE UPDATED BY UPDTIM
RHGMT=0._JPRB
REQTIM=0._JPRB
RSOVR=0._JPRB
RDEASO=0._JPRB
RDECLI=0._JPRB
RWSOVR=0._JPRB
RIP0=0._JPRB
RCODEC=0._JPRB
RSIDEC=0._JPRB
RCOVSR=0._JPRB
RSIVSR=0._JPRB

RDECLU=0._JPRB
RTMOLT=0._JPRB
RIP0LU=0._JPRB
RCODECLU=0._JPRB
RSIDECLU=0._JPRB
RCOVSRLU=0._JPRB
RSIVSRLU=0._JPRB

RDTSA=0._JPRB
RDTSA2=0._JPRB
RDTS62=0._JPRB
RDTS22=0._JPRB

RTDT=0._JPRB

!      ----------------------------------------------------------------

!*       2.    PRINT PART OF YOMRIP.
!              ---------------------

WRITE(UNIT=KULOUT,FMT='('' NINDAT='',I10,'' NSSSSS='',I6)')NINDAT,NSSSSS

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURIP',1,ZHOOK_HANDLE)
END SUBROUTINE SURIP
