SUBROUTINE SUROT(KNSMIN,KNSMAX,KULROT,KULOUT,PLATI,CDNOM,&
 & LDIMST,KNIMES,KFACTM)  

!**** *SUROT* - Computation of rotation (tilting) matrices.

!      Purpose.
!      --------
!               Rotation matrices for ARPEGE/IFS transformation (tilting)
!               are computed.
!**    Interface.
!      ----------
!         *CALL *SUROT*

!      Explicit arguments:
!      -------------------
!            KNSMIN - Lower wave number.                          (input)
!            KNSMAX - Upper wave number.                          (input)
!            KULROT - Logical unit for rotation matrices.         (input)
!            KULOUT - Output logical unit.                        (input)
!            PLATI  - Latitude of the stretching pole.            (input)
!            CDNOM  - Name of the logical unit KULROT.            (input)
!            LDIMST - Option of statistics prints when closing
!                      the file containing rotation matrices.     (input)
!            KNIMES - Mail level for this file.                   (input)
!            KFACTM - Multiplication factor for this file.        (input)

!      Implicit arguments:
!      -------------------

!      Method.
!      -------

!      References.
!      -----------
!        Note de travail ARPEGE/IFS nr 19 (in French).

!      External.
!      ---------
!         Called by SU0YOMB.

!      Author.
!      -------
!         Michel Rochas, DMN.

!      Modifications.
!      --------------
!      Original : 91-02-18.
!      D. Giard : 92-12-08 : options (when opening files).
!      K. YESSAD: 93-05-11 : cleaning, comments put into English.
!      K. YESSAD: 93-07-30 : use call to SUPLIS to compute Leg. polynomials.
!      R. El Khatib: 97-12-03 : portability of LFI
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      K. Yessad (Nov 2008): cleanings
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND2  ,ONLY : JPRH

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNSMIN
INTEGER(KIND=JPIM),INTENT(IN)    :: KNSMAX
INTEGER(KIND=JPIM),INTENT(IN)    :: KULROT
INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATI 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDNOM
LOGICAL           ,INTENT(IN)    :: LDIMST
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIMES
INTEGER(KIND=JPIM),INTENT(IN)    :: KFACTM

!     ------------------------------------------------------------------

REAL(KIND=JPRH) :: Z_DLPNM(0:KNSMAX,0:KNSMAX)
REAL(KIND=JPRH) :: Z_DLA(0:KNSMAX),Z_DLI(0:KNSMAX)
REAL(KIND=JPRH) :: Z_DLSINTA(1),Z_DLCOSTA,Z_DLUCOST

INTEGER(KIND=JPIM), PARAMETER :: JPKD=KIND(Z_DLPNM)

INTEGER(KIND=JPIM) :: INAR, INARI, IRP, JN

REAL(KIND=JPRB) :: ZPNM(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "rotat.intfb.h"
#include "suplis.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUROT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. Initialisations.
!           ----------------

IRP=0
INARI=0

Z_DLSINTA(1)=SIN(REAL(PLATI,JPKD))
Z_DLCOSTA=COS(REAL(PLATI,JPKD))
Z_DLUCOST=1.0_JPRB/Z_DLCOSTA

!*       2. PNM computation (for troncature KNSMAX).
!           ----------------------------------------

CALL SUPLIS(1,KNSMAX,1,Z_DLSINTA,.FALSE.,ZPNM,Z_DLPNM,Z_DLA,Z_DLI)

!        3. LFI file (containing matrices) opening.
!           ---------------------------------------

INAR=2*KNSMAX
CALL LFIAFM(IRP,KULROT,KFACTM)
CALL LFIOUV(IRP,KULROT,.TRUE.,CDNOM,'UNKNOWN',.TRUE.,&
 & LDIMST,KNIMES,INAR,INARI)  

!     Auto-documentation :
CALL LFIINTECR(IRP,KULROT,'NSMIN',KNSMIN,1)
CALL LFIINTECR(IRP,KULROT,'NSMAX',KNSMAX,1)
CALL LFIECR(IRP,KULROT,'RMUCEN',SIN(PLATI),1)

!*       4. Main loop on total wave number.
!           -------------------------------

DO JN=1,KNSMAX
  CALL ROTAT(KNSMIN,KNSMAX,KULROT,KULOUT,JN,Z_DLPNM,Z_DLA,Z_DLI,Z_DLUCOST, &
   & Z_DLSINTA(1))
ENDDO

!*       5. LFI file closing.
!           -----------------

CALL LFIFER(IRP,KULROT,'UNKNOWN')

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUROT',1,ZHOOK_HANDLE)
END SUBROUTINE SUROT

