SUBROUTINE SUSLAD1(KADMAPLEN,KSLMAPLEN,KADMAP,KSLMAP)

!**** *SUSLAD1*  Initialise data structures for SL adjoint

!     Purpose.
!     -------
!           Initialise NGPTOTAD and NADCORE required for SL adjoint.
!           NGPTOTAD represents the number of grid points spanning 
!           a standard width halo around this processors core points.
!           NADCORE are the offsets of the above points in a double
!           standard width semi-langrangian buffer.
!           Note the great care is taken that these data structures 
!           contain no duplicate points (i.e. mirror latitudes or 
!           latitude extension points).
!           It is also important for bit reproducibility that the order 
!           of points in NADCORE is identical to that when executed
!           on a single processor.

!     Method.
!     -------
!        See documentation

!     Author.
!     ------
!        George Mozdzynski  * ECMWF *

!     Modifications.
!     -------------
!        Original  : 00-08-01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NASLB1   ,NSLCORE
USE YOMDIM   , ONLY : NDLON
USE YOMCT0   , ONLY : NPROC    ,NPRINTLEV,LALLOPR
USE YOMGEM   , ONLY : NGPTOT   ,NLOENG
USE YOMSLREP , ONLY : NGPTOTAD ,NADCORE  ,MASK_SLTOT  ,LSLCORE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KADMAPLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLMAPLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KADMAP(4,KADMAPLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLMAP(4,KSLMAPLEN) 
INTEGER(KIND=JPIM) :: JSL, JAD, JOFF, IOFF, ILAT, IU

LOGICAL :: LLERROR, LLFOUND, LLP, LLDEBUG

INTEGER(KIND=JPIM), ALLOCATABLE :: IADCORE(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IADLAT(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: ISLBUF(:,:)

INTEGER(KIND=JPIM) :: IMAP(NDLON)

LOGICAL :: LLUSED(NDLON)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLAD1',0,ZHOOK_HANDLE)
LLDEBUG=.FALSE.

LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT

ALLOCATE(IADCORE(NASLB1))
ALLOCATE(IADLAT(NASLB1))
IADLAT(:)=99999999
IF( NPROC>1 )THEN
  NGPTOTAD=0
  DO JSL=1,KSLMAPLEN
    LLFOUND=.FALSE.
    DO JAD=1,KADMAPLEN
      IF( KSLMAP(1,JSL) == KADMAP(1,JAD) )THEN
        LLFOUND=.TRUE.
        ILAT=KSLMAP(1,JSL)
        IMAP(:)=99999999
        DO JOFF=KADMAP(2,JAD),KADMAP(2,JAD)+KADMAP(3,JAD)-1
          IOFF=MOD(JOFF+NLOENG(ILAT)-1,NLOENG(ILAT))+1
          IMAP(IOFF)=JOFF-KADMAP(2,JAD)+1
        ENDDO
        LLUSED(:)=.FALSE.
        DO JOFF=KSLMAP(2,JSL),KSLMAP(2,JSL)+KSLMAP(3,JSL)-1
          IOFF=MOD(JOFF+NLOENG(ILAT)-1,NLOENG(ILAT))+1
          LLUSED(IOFF)=.TRUE.
        ENDDO
        DO JOFF=1,NLOENG(ILAT)
          IF( LLUSED(JOFF) )THEN
            NGPTOTAD=NGPTOTAD+1
            IF( NGPTOTAD > NASLB1 )THEN
              CALL ABOR1('SUSLAD1: INTERNAL ERROR NGPTOTAD > NASLB1 ')
            ENDIF
            IF( IMAP(JOFF) > NLOENG(ILAT) )THEN
              CALL ABOR1('SUSLAD1: INTERNAL ERRROR IN BUILDING IADCORE)')
            ENDIF
            IADCORE(NGPTOTAD)=KADMAP(4,JAD)+IMAP(JOFF)
            IADLAT(NGPTOTAD)=ILAT
          ENDIF
        ENDDO
      ENDIF
    ENDDO
    IF( .NOT.LLFOUND )THEN
      CALL ABOR1('SUSLAD1: INTERNAL ERROR DURING SEARCH FOR NADCORE POINTS')
    ENDIF
  ENDDO
ELSE
  NGPTOTAD=NGPTOT
ENDIF

WRITE(NULOUT,'("SUSLAD1: NASLB1=",I10," NGPTOTAD=",I10," NGPTOT=",I10)')&
 & NASLB1,NGPTOTAD,NGPTOT  

ALLOCATE(NADCORE(NGPTOTAD))
IF(LLP)WRITE(IU,9) 'NADCORE ',SIZE(NADCORE),SHAPE(NADCORE)
IF( NPROC>1 )THEN
  NADCORE(1:NGPTOTAD)=IADCORE(1:NGPTOTAD)
ELSE
  NADCORE(1:NGPTOTAD)=NSLCORE(1:NGPTOTAD)
ENDIF
IF(ALLOCATED(IADCORE)) DEALLOCATE(IADCORE)

IF( LLDEBUG )THEN
  ! confirm that all points in nslcore are in nadcore
  DO JAD=1,NGPTOTAD
    WRITE(NULOUT,'("SUSLAD1: JAD=",I10," NADCORE(JAD)=",I10)')&
     & JAD,NADCORE(JAD)  
  ENDDO
  DO JSL=1,NGPTOT
    WRITE(NULOUT,'("SUSLAD1: JSL=",I10," NSLCORE(JSL)=",I10)')&
     & JSL,NSLCORE(JSL)  
  ENDDO
  LLERROR=.FALSE.
  DO JSL=1,NGPTOT
    LLFOUND=.FALSE.
    DO JAD=1,NGPTOTAD
      IF( NSLCORE(JSL) == NADCORE(JAD) )THEN
        IF( .NOT.LLFOUND )THEN
          LLFOUND=.TRUE.
        ELSE
          WRITE(NULOUT,'("SUSLAD1: JSL=",I10," NSLCORE(JSL)=",I10,&
           & " OCCURS MORE THAN ONCE IN NADCORE")')JSL,NSLCORE(JSL)  
          LLERROR=.TRUE.
        ENDIF
      ENDIF
    ENDDO
    IF( .NOT.LLFOUND )THEN
      WRITE(NULOUT,'("SUSLAD1: JSL=",I10," NSLCORE(JSL)=",I10,&
       & " DOES NOT OCCUR IN NADCORE")')JSL,NSLCORE(JSL)  
      LLERROR=.TRUE.
    ENDIF
  ENDDO
  IF( LLERROR )THEN
    ALLOCATE (ISLBUF(NASLB1,3))
    ISLBUF(:,:)=99999999
    DO JAD=1,NGPTOTAD
      ISLBUF(NADCORE(JAD),1)=JAD
      ISLBUF(NADCORE(JAD),3)=IADLAT(JAD)
    ENDDO
    DO JSL=1,NGPTOT
      ISLBUF(NSLCORE(JSL),2)=JSL
    ENDDO
    DO JOFF=1,NASLB1
      WRITE(NULOUT,'("SUSLAD1: LAT=",I4," JOFF=",I10," NADCORE POINT ",I10,&
       & "   NSLCORE POINT ",I10)')&
       & ISLBUF(JOFF,3),JOFF,ISLBUF(JOFF,1),ISLBUF(JOFF,2)  
    ENDDO
    IF (ALLOCATED(ISLBUF)) DEALLOCATE(ISLBUF)
    CALL FLUSH(NULOUT)
    CALL ABOR1('SUSLAD1: NSLCORE NOT A SUBSET OF NADCORE')
  ENDIF

ENDIF

IF(ALLOCATED(IADLAT)) DEALLOCATE(IADLAT)

IF (NPROC > 1) THEN
  ALLOCATE(MASK_SLTOT(NASLB1))
  IF(LLP)WRITE(IU,9) 'MASK_SLTOT',SIZE(MASK_SLTOT),SHAPE(MASK_SLTOT)
  MASK_SLTOT(:)=0
  ALLOCATE(LSLCORE(NASLB1))
  IF(LLP)WRITE(IU,9) 'LSLCORE',SIZE(LSLCORE),SHAPE(LSLCORE)
  LSLCORE(:)=.FALSE.
  DO JSL=1,NGPTOT
    LSLCORE(NSLCORE(JSL))=.TRUE.
  ENDDO
ENDIF

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLAD1',1,ZHOOK_HANDLE)
END SUBROUTINE SUSLAD1
