SUBROUTINE SUSLAD2

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NASLB1    ,NSLCORE  ,NSLPROCS,NSLRPT ,&
 & NSLSPT  ,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM  
USE YOMCT0   , ONLY : NPROC     ,NPRINTLEV,LALLOPR
USE YOMGEM   , ONLY : NGPTOT    ,RCOLON   ,RSILON   ,RINDX    ,RINDY
USE YOMGC    , ONLY : GEMU      ,GESLO    ,GECLO    ,GM      ,GNORDL  ,&
 & GNORDM  ,GSQM2  
USE YOMSLREP , ONLY : NGPTOTAD   ,NADCORE  ,&
 & GEMUAD   ,GESLOAD  ,GECLOAD  ,GMAD     ,GNORDLAD  ,&
 & GNORDMAD ,GSQM2AD  ,RCOLONAD ,RSILONAD ,RINDXAD   ,&
 & RINDYAD  

!**** *SUSLAD2*  Initialise constant arrays for SL adjoint interpolation

!     Purpose.
!     -------
!           Initialise constant arrays for SL adjoint interpolation.
!           These arrays cover an area of a standard width halo around 
!           this processors core points.

!     Method.
!     -------
!        See documentation

!     Author.
!     ------
!        George Mozdzynski  * ECMWF *

!     Modifications.
!     -------------
!        Original  : 00-08-01
!        02-10-01  G.Mozdzynski    support for radiation on-demand comms
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Dec 2008: merge SLCOMM+SLCOMM1 -> SLCOMM.
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU, IFLDS, JROF, IDUMARR(2)

LOGICAL :: LLP, LLINC

REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "slcomm.intfb.h"

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUSLAD2',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT

IFLDS=11
ALLOCATE (ZSLBUF(NASLB1,IFLDS))
DO JROF=1,NGPTOT
  ZSLBUF(NSLCORE(JROF), 1)=GEMU  (JROF)
  ZSLBUF(NSLCORE(JROF), 2)=GESLO (JROF)
  ZSLBUF(NSLCORE(JROF), 3)=GECLO (JROF)
  ZSLBUF(NSLCORE(JROF), 4)=GM    (JROF)
  ZSLBUF(NSLCORE(JROF), 5)=GNORDL(JROF)
  ZSLBUF(NSLCORE(JROF), 6)=GNORDM(JROF)
  ZSLBUF(NSLCORE(JROF), 7)=GSQM2 (JROF)
  ZSLBUF(NSLCORE(JROF), 8)=RCOLON(JROF)
  ZSLBUF(NSLCORE(JROF), 9)=RSILON(JROF)
  ZSLBUF(NSLCORE(JROF),10)=RINDX (JROF)
  ZSLBUF(NSLCORE(JROF),11)=RINDY (JROF)
ENDDO
IF (NPROC > 1) THEN
  LLINC=.FALSE.
  CALL SLCOMM(NASLB1,NSLPROCS,NSLRPT,&
   & NSLSPT,NSLSENDPOS,NSLRECVPOS,NSENDPTR,NRECVPTR,NSLCOMM,&
   & IDUMARR,IFLDS,LLINC,0,ZSLBUF)  
ENDIF
ALLOCATE(GEMUAD  (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GEMUAD  ',SIZE(GEMUAD  ),SHAPE(GEMUAD  )
ALLOCATE(GESLOAD (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GESLOAD ',SIZE(GESLOAD ),SHAPE(GESLOAD )
ALLOCATE(GECLOAD (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GECLOAD ',SIZE(GECLOAD ),SHAPE(GECLOAD )
ALLOCATE(GMAD    (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GMAD    ',SIZE(GMAD    ),SHAPE(GMAD    )
ALLOCATE(GNORDLAD(NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GNORDLAD',SIZE(GNORDLAD),SHAPE(GNORDLAD)
ALLOCATE(GNORDMAD(NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GNORDMAD',SIZE(GNORDMAD),SHAPE(GNORDMAD)
ALLOCATE(GSQM2AD (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'GSQM2AD ',SIZE(GSQM2AD ),SHAPE(GSQM2AD )
ALLOCATE(RCOLONAD(NGPTOTAD))
IF (LLP) WRITE(IU,9) 'RCOLONAD',SIZE(RCOLONAD),SHAPE(RCOLONAD)
ALLOCATE(RSILONAD(NGPTOTAD))
IF (LLP) WRITE(IU,9) 'RSILONAD',SIZE(RSILONAD),SHAPE(RSILONAD)
ALLOCATE(RINDXAD (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'RINDXAD ',SIZE(RINDXAD ),SHAPE(RINDXAD )
ALLOCATE(RINDYAD (NGPTOTAD))
IF (LLP) WRITE(IU,9) 'RINDYAD ',SIZE(RINDYAD ),SHAPE(RINDYAD )
DO JROF=1,NGPTOTAD
  GEMUAD  (JROF)=ZSLBUF(NADCORE(JROF), 1)
  GESLOAD (JROF)=ZSLBUF(NADCORE(JROF), 2)
  GECLOAD (JROF)=ZSLBUF(NADCORE(JROF), 3)
  GMAD    (JROF)=ZSLBUF(NADCORE(JROF), 4)
  GNORDLAD(JROF)=ZSLBUF(NADCORE(JROF), 5)
  GNORDMAD(JROF)=ZSLBUF(NADCORE(JROF), 6)
  GSQM2AD (JROF)=ZSLBUF(NADCORE(JROF), 7)
  RCOLONAD(JROF)=ZSLBUF(NADCORE(JROF), 8)
  RSILONAD(JROF)=ZSLBUF(NADCORE(JROF), 9)
  RINDXAD (JROF)=ZSLBUF(NADCORE(JROF),10)
  RINDYAD (JROF)=ZSLBUF(NADCORE(JROF),11)
ENDDO
IF(ALLOCATED(ZSLBUF)) DEALLOCATE(ZSLBUF)

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLAD2',1,ZHOOK_HANDLE)
END SUBROUTINE SUSLAD2
