SUBROUTINE SUSLAD3

!**** *SUSLAD3*  Optimise halo for SL adjoint interpolation

!     Purpose.
!     -------
!           Optimise the size of the standard width halo for SL
!           adjoint interpolation.

!     Method.
!     -------
!        See documentation

!     Author.
!     ------
!        George Mozdzynski  * ECMWF *

!     Modifications.
!     -------------
!        Original  : 00-08-30
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NASLB1     ,NSLCORE
USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR
USE YOMSLREP , ONLY : NGPTOTAD   ,NADCORE  ,NSLMAP   ,MASK_SLTOT,&
 & GEMUAD   ,GESLOAD  ,GECLOAD  ,GMAD     ,GNORDLAD  ,&
 & GNORDMAD ,GSQM2AD  ,RCOLONAD ,RSILONAD ,RINDXAD   ,&
 & RINDYAD  
USE YOMGEM   , ONLY : NGPTOT
USE MPL_MODULE

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU, J, IGPTOTAD

LOGICAL :: LLP, LL_LERROR

INTEGER(KIND=JPIM), ALLOCATABLE :: IADCORE  (:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IMASK    (:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IMASKAD  (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGEMUAD  (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGESLOAD (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGECLOAD (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMAD    (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGNORDLAD(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGNORDMAD(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGSQM2AD (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRCOLONAD(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRSILONAD(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRINDXAD (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZRINDYAD (:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLAD3',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
CALL FLUSH(NULOUT)

ALLOCATE(IADCORE(NASLB1))
ALLOCATE(IMASK(NASLB1))
ALLOCATE(IMASKAD(NASLB1))
ALLOCATE(ZGEMUAD  (NGPTOTAD))
ALLOCATE(ZGESLOAD (NGPTOTAD))
ALLOCATE(ZGECLOAD (NGPTOTAD))
ALLOCATE(ZGMAD    (NGPTOTAD))
ALLOCATE(ZGNORDLAD(NGPTOTAD))
ALLOCATE(ZGNORDMAD(NGPTOTAD))
ALLOCATE(ZGSQM2AD (NGPTOTAD))
ALLOCATE(ZRCOLONAD(NGPTOTAD))
ALLOCATE(ZRSILONAD(NGPTOTAD))
ALLOCATE(ZRINDXAD (NGPTOTAD))
ALLOCATE(ZRINDYAD (NGPTOTAD))
IMASK(:)=-999999
IMASKAD(:)=-999999
! Mark nadcore points
DO J=1,NGPTOTAD
  IMASKAD(NADCORE(J))=1
ENDDO
! Remap points to be unique (i.e. no latitude extension
! and mirror latitude points)
LL_LERROR=.FALSE.
DO J=1,NASLB1
  IF ( MASK_SLTOT(J) > 0 ) THEN
    IMASK(NSLMAP(J))=1
    IF( IMASKAD(NSLMAP(J)) == 0 )THEN
      LL_LERROR=.TRUE.
    ENDIF
  ENDIF
ENDDO
IF( LL_LERROR )THEN
  WRITE(NULOUT,'("SUSLAD3: NASLB1=",I10)')NASLB1
  DO J=1,NASLB1
    IF ( IMASK(NSLMAP(J)) > 0 .AND. IMASKAD(NSLMAP(J)) ==  0 ) THEN
      WRITE(NULOUT,'("SUSLAD3: ERROR, POINT NOT IN NADCORE, J=",I10,&
       & " NSLMAP(J)=",I10)')J,NSLMAP(J)  
    ENDIF
  ENDDO
  DO J=1,NASLB1
    WRITE(NULOUT,'("SUSLAD3: J=",I10," IMASK =",I10," IMASKAD=",I10)')&
     & J,IMASK(J),IMASKAD(J)  
  ENDDO
  CALL FLUSH(NULOUT)
ENDIF
! Make certain the all core points are included
DO J=1,NGPTOT
  IMASK(NSLCORE(J))=1
ENDDO
! Now select from the original nadcore points those that are needed
! Note that these new points have the same order as the original set
! which is required for bit reproducibility
IGPTOTAD=0
DO J=1,NGPTOTAD
  IF ( IMASK(NADCORE(J)) == 1 ) THEN
    IGPTOTAD=IGPTOTAD+1
    IADCORE  (IGPTOTAD)=NADCORE (J)
    ZGEMUAD  (IGPTOTAD)=GEMUAD  (J)
    ZGESLOAD (IGPTOTAD)=GESLOAD (J)
    ZGECLOAD (IGPTOTAD)=GECLOAD (J)
    ZGMAD    (IGPTOTAD)=GMAD    (J)
    ZGNORDLAD(IGPTOTAD)=GNORDLAD(J)
    ZGNORDMAD(IGPTOTAD)=GNORDMAD(J)
    ZGSQM2AD (IGPTOTAD)=GSQM2AD (J)
    ZRCOLONAD(IGPTOTAD)=RCOLONAD(J)
    ZRSILONAD(IGPTOTAD)=RSILONAD(J)
    ZRINDXAD (IGPTOTAD)=RINDXAD (J)
    ZRINDYAD (IGPTOTAD)=RINDYAD (J)
  ENDIF
ENDDO
NGPTOTAD=IGPTOTAD
WRITE(NULOUT,'("SUSLAD3: OPTIMISED NGPTOTAD=",I10)')NGPTOTAD
CALL MPL_ALLREDUCE(IGPTOTAD,'MAX',CDSTRING='SUSLAD3:')
WRITE(NULOUT,'("SUSLAD3: MAXIMUM NGPTOTAD=",I10)')IGPTOTAD
DEALLOCATE(IMASK    )
DEALLOCATE(IMASKAD  )

DEALLOCATE(NADCORE )
DEALLOCATE(GEMUAD  )
DEALLOCATE(GESLOAD )
DEALLOCATE(GECLOAD )
DEALLOCATE(GMAD    )
DEALLOCATE(GNORDLAD)
DEALLOCATE(GNORDMAD)
DEALLOCATE(GSQM2AD )
DEALLOCATE(RCOLONAD)
DEALLOCATE(RSILONAD)
DEALLOCATE(RINDXAD )
DEALLOCATE(RINDYAD )
ALLOCATE(NADCORE (NGPTOTAD))
ALLOCATE(GEMUAD  (NGPTOTAD))
ALLOCATE(GESLOAD (NGPTOTAD))
ALLOCATE(GECLOAD (NGPTOTAD))
ALLOCATE(GMAD    (NGPTOTAD))
ALLOCATE(GNORDLAD(NGPTOTAD))
ALLOCATE(GNORDMAD(NGPTOTAD))
ALLOCATE(GSQM2AD (NGPTOTAD))
ALLOCATE(RCOLONAD(NGPTOTAD))
ALLOCATE(RSILONAD(NGPTOTAD))
ALLOCATE(RINDXAD (NGPTOTAD))
ALLOCATE(RINDYAD (NGPTOTAD))
DO J=1,NGPTOTAD
  NADCORE (J)=IADCORE  (J)
  GEMUAD  (J)=ZGEMUAD  (J)
  GESLOAD (J)=ZGESLOAD (J)
  GECLOAD (J)=ZGECLOAD (J)
  GMAD    (J)=ZGMAD    (J)
  GNORDLAD(J)=ZGNORDLAD(J)
  GNORDMAD(J)=ZGNORDMAD(J)
  GSQM2AD (J)=ZGSQM2AD (J)
  RCOLONAD(J)=ZRCOLONAD(J)
  RSILONAD(J)=ZRSILONAD(J)
  RINDXAD (J)=ZRINDXAD (J)
  RINDYAD (J)=ZRINDYAD (J)
ENDDO
DEALLOCATE(IADCORE  )
DEALLOCATE(ZGEMUAD  )
DEALLOCATE(ZGESLOAD )
DEALLOCATE(ZGECLOAD )
DEALLOCATE(ZGMAD    )
DEALLOCATE(ZGNORDLAD)
DEALLOCATE(ZGNORDMAD)
DEALLOCATE(ZGSQM2AD )
DEALLOCATE(ZRCOLONAD)
DEALLOCATE(ZRSILONAD)
DEALLOCATE(ZRINDXAD )
DEALLOCATE(ZRINDYAD )
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLAD3',1,ZHOOK_HANDLE)
END SUBROUTINE SUSLAD3

