SUBROUTINE SUSLB2

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NUNDEFLD
USE YOMCT0   , ONLY : NPRINTLEV,LSLAG    ,LALLOPR  ,NCONF
USE YOMDYN   , ONLY : NVLAG    ,NWLAG
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2 ,NFLDSLB15
USE PTRSLB1  , ONLY : RPARSL1  ,MSLBUF1  ,MSLB1U9  ,MSLB1V9  ,&
 & MSLB1T9  ,MSLB1GFL9  ,MSLB1PD9 ,MSLB1VD9 ,MSLB1Y9  ,&
 & MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,MSLB1UR9 ,MSLB1VR9 ,&
 & MSLB1U0  ,MSLB1V0  ,MSLB1GFLSP9, &
 & MSLB1T0  ,MSLB1PD0 ,MSLB1VD0 ,MSLB1C9  ,&
 & MSLB1SP9 ,MSLB1SP0 ,MSLB1C0  
USE PTRSLB15  , ONLY : RPARSL15,MSLBUF15,&
 & MSLB1UR05,MSLB1VR05,MSLB1U05 ,MSLB1V05 ,MSLB1SP05,&
 & MSLB1U95 ,MSLB1V95 ,MSLB1SP95  
USE PTRSLB2  , ONLY : MSLBUF2  ,&
 & MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2PDSI,&
 & MSLB2VDSI,MSLB2SPSI,MSLB2VVEL,MSLB2URL ,MSLB2VRL ,&
 & MSLB2WRL ,MSLB2URL5,MSLB2VRL5,&
 & MSLB2USI5,MSLB2VSI5,MSLB2U15 ,MSLB2V15  

!**** *SUSLB2  - Setup relative pointers of PTRSLB1 and PTRSLB2 for 2D model.

!     Purpose.
!     --------
!           Initialize PTRSLB1 and PTRSLB2, modules that contain
!           the relative pointers for the semi-lagrangian buffers.
!           These are used in CPG2 and CPG2LAG to compute the absolute
!           pointers required. Also computes the parities needed for
!           the extension of semi-lagrangian buffer 1 over the poles.
!           Compute total number of fields in semi-lagrangian buffers.

!**   Interface.
!     ----------
!        *CALL* *SUSLB2*

!        Explicit arguments : none
!        --------------------

!        Implicit arguments :
!        --------------------
!        modules PTRSLB1, PTRSLB2

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!      None
!      Is called by SUSC2.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (after SUSLB): move all 2D calculations of SUSLB in SUSLB2.

!     Modifications.
!     --------------
!        Original : 98-08-04
!        Modified 98-12-15 by C.Temperton (new pointers for TL/AD)
!        Modified 99-06-17 by C.Temperton (more new pointers for TL/AD)
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) ::  IFLDSLB2, IPTX, IU

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLB2',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1.   COMPUTE RELATIVE POINTERS FOR SEMI LAGRANGIAN BUFFERS.
!          -----------------------------------------------------

LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
IF (LLP) WRITE (IU,*)
IF (LLP) WRITE (IU,'(A,/)') ' SUSLB2 PRINTOUT : POINTERS,NFIELDS'

!*       1.1  COMPUTE POINTERS FOR SLBUF2

!*       Preliminary set pointers to prevent use of uninitialised variables.
!*       (All the pointers, including those used only in the 3D model).

MSLBUF2  =NUNDEFLD
MSLB2USI =NUNDEFLD
MSLB2VSI =NUNDEFLD
MSLB2TSI =NUNDEFLD
MSLB2PDSI=NUNDEFLD
MSLB2VDSI=NUNDEFLD
MSLB2SPSI=NUNDEFLD
MSLB2VVEL=NUNDEFLD
MSLB2URL =NUNDEFLD
MSLB2VRL =NUNDEFLD
MSLB2WRL =NUNDEFLD
MSLB2URL5=NUNDEFLD
MSLB2VRL5=NUNDEFLD
MSLB2USI5=NUNDEFLD
MSLB2VSI5=NUNDEFLD
MSLB2U15 =NUNDEFLD
MSLB2V15 =NUNDEFLD

!*       Definite computation of useful pointers.

IF (LLP) WRITE (IU,'(A)') ' 1) BUFFER SLBUF2:'
IPTX    = 1
MSLBUF2 = IPTX

IF (LSLAG) THEN
  MSLB2USI  = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2USI  = ',IPTX,1
  IPTX      = IPTX + 1
  MSLB2VSI  = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VSI  = ',IPTX,1
  IPTX      = IPTX + 1
  MSLB2URL  = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2URL  = ',IPTX,1
  IPTX      = IPTX + 1
  MSLB2VRL  = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VRL  = ',IPTX,1
  IPTX      = IPTX + 1
  IF (NCONF == 421.OR.NCONF == 521) THEN
    MSLB2URL5 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2URL5 = ',IPTX,1
    IPTX      = IPTX + 1
    MSLB2VRL5 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VRL5 = ',IPTX,1
    IPTX      = IPTX + 1
    MSLB2USI5 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2USI5 = ',IPTX,1
    IPTX      = IPTX + 1
    MSLB2VSI5 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VSI5 = ',IPTX,1
    IPTX      = IPTX + 1
    MSLB2U15  = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2U15  = ',IPTX,1
    IPTX      = IPTX + 1
    MSLB2V15  = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2V15  = ',IPTX,1
    IPTX      = IPTX + 1
  ENDIF
ENDIF

IFLDSLB2 = IPTX-1
NFLDSLB2 = IFLDSLB2

!*       1.2  COMPUTE POINTERS FOR SLBUF1

!*       Preliminary set pointers to prevent use of uninitialised variables.
!*       (All the pointers, including those used only in the 3D model).

MSLBUF1  =NUNDEFLD
MSLBUF15 =NUNDEFLD
MSLB1U9  =NUNDEFLD
MSLB1V9  =NUNDEFLD
MSLB1T9  =NUNDEFLD
MSLB1GFL9=NUNDEFLD
MSLB1GFLSP9 =NUNDEFLD
MSLB1PD9 =NUNDEFLD
MSLB1VD9 =NUNDEFLD
MSLB1Y9  =NUNDEFLD
MSLB1UR0 =NUNDEFLD
MSLB1VR0 =NUNDEFLD
MSLB1WR0 =NUNDEFLD
MSLB1UR9 =NUNDEFLD
MSLB1VR9 =NUNDEFLD
MSLB1U0  =NUNDEFLD
MSLB1V0  =NUNDEFLD
MSLB1T0  =NUNDEFLD
MSLB1PD0 =NUNDEFLD
MSLB1VD0 =NUNDEFLD
MSLB1C9  =NUNDEFLD
MSLB1SP9 =NUNDEFLD
MSLB1SP0 =NUNDEFLD
MSLB1C0  =NUNDEFLD
MSLB1UR05=NUNDEFLD
MSLB1VR05=NUNDEFLD
MSLB1U05 =NUNDEFLD
MSLB1V05 =NUNDEFLD
MSLB1SP05=NUNDEFLD
MSLB1U95 =NUNDEFLD
MSLB1V95 =NUNDEFLD
MSLB1SP95=NUNDEFLD

!*       Definite computation of useful pointers.

!        Order is:
!         - first: high-order interpolated quantities.
!         - second: linearly interpolated quantities.
!         - then repeat for trajectory variables in case of TL/AD

IF(LSLAG) THEN

  IF (LLP) WRITE (IU,*)
  IF (LLP) WRITE (IU,'(A)') ' 2) BUFFER SLBUF1:'

  IPTX    = 1
  MSLBUF1 = IPTX

!        * Buffers for high order interpolations.

  MSLB1U9  = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1U9     = ',IPTX,1
  IPTX     = IPTX + 1
  MSLB1V9  = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1V9     = ',IPTX,1
  IPTX     = IPTX + 1
  MSLB1SP9 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1SP9    = ',IPTX,1
  IPTX     = IPTX + 1

!        * Buffers for linear interpolations.

  MSLB1UR0 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UR0    = ',IPTX,1
  IPTX     = IPTX + 1
  MSLB1VR0 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VR0    = ',IPTX,1
  IPTX     = IPTX + 1
  IF(NWLAG == 3) THEN
    MSLB1U0  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1U0     = ',IPTX,1
    IPTX     = IPTX + 1
    MSLB1V0  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1V0     = ',IPTX,1
    IPTX     = IPTX + 1
  ENDIF
  IF (NVLAG == 3) THEN
    MSLB1SP0 = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1SP0    = ',IPTX,1
    IPTX     = IPTX + 1
  ENDIF
  NFLDSLB1 = IPTX - 1

  IF (NCONF == 421.OR.NCONF == 521) THEN

    IPTX    = 1
    MSLBUF15 = IPTX
!          * Buffers for high order interpolations.

    MSLB1U95 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1U95    = ',IPTX,1
    IPTX     = IPTX + 1
    MSLB1V95 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1V95    = ',IPTX,1
    IPTX     = IPTX + 1
    MSLB1SP95= IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1SP95   = ',IPTX,1
    IPTX     = IPTX + 1

!          * Buffers for linear interpolations.

    MSLB1UR05= IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UR05   = ',IPTX,1
    IPTX     = IPTX + 1
    MSLB1VR05= IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VR05   = ',IPTX,1
    IPTX     = IPTX + 1
    IF(NWLAG == 3) THEN
      MSLB1U05 = IPTX
      IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1U05    = ',IPTX,1
      IPTX     = IPTX + 1
      MSLB1V05 = IPTX
      IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1V05    = ',IPTX,1
      IPTX     = IPTX + 1
    ENDIF
    IF (NVLAG == 3) THEN
      MSLB1SP05= IPTX
      IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1SP05   = ',IPTX,1
      IPTX     = IPTX + 1
    ENDIF
    NFLDSLB15 = IPTX - 1
  ELSE
    NFLDSLB15 = 1
  ENDIF

ELSE

  NFLDSLB1  = 1
  NFLDSLB15 = 1

ENDIF

IF (LLP) WRITE (IU,*)

!     ------------------------------------------------------------------

!*    2.   COMPUTE PARITIES FOR EXTENSION OVER POLE.
!          -----------------------------------------

IF (LSLAG) THEN
  ALLOCATE(RPARSL1(NFLDSLB1))
  IF(LLP)WRITE(IU,9) 'RPARSL1  ',SIZE(RPARSL1),SHAPE(RPARSL1)
  ALLOCATE(RPARSL15(NFLDSLB15))
  IF(LLP)WRITE(IU,9) 'RPARSL1  ',SIZE(RPARSL1),SHAPE(RPARSL1)
ENDIF
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LSLAG) THEN
  RPARSL1(MSLB1U9)=-1.0_JPRB
  RPARSL1(MSLB1V9)=-1.0_JPRB
  RPARSL1(MSLB1SP9)=+1.0_JPRB
  RPARSL1(MSLB1UR0)=-1.0_JPRB
  RPARSL1(MSLB1VR0)=-1.0_JPRB
  IF(NWLAG == 3) THEN
    RPARSL1(MSLB1U0)=-1.0_JPRB
    RPARSL1(MSLB1V0)=-1.0_JPRB
  ENDIF
  IF (NVLAG == 3) THEN
    RPARSL1(MSLB1SP0)=+1.0_JPRB
  ENDIF
  IF (NCONF == 421.OR.NCONF == 521) THEN
    RPARSL15(MSLB1U95)=-1.0_JPRB
    RPARSL15(MSLB1V95)=-1.0_JPRB
    RPARSL15(MSLB1SP95)=+1.0_JPRB
    RPARSL15(MSLB1UR05)=-1.0_JPRB
    RPARSL15(MSLB1VR05)=-1.0_JPRB
    IF(NWLAG == 3) THEN
      RPARSL15(MSLB1U05)=-1.0_JPRB
      RPARSL15(MSLB1V05)=-1.0_JPRB
    ENDIF
    IF (NVLAG == 3) THEN
      RPARSL15(MSLB1SP05)=+1.0_JPRB
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLB2',1,ZHOOK_HANDLE)
END SUBROUTINE SUSLB2
