SUBROUTINE SUSPE0

!**** *SUSPE0*  - Initialize the spectral fields to 0.

!     Purpose.
!     --------
!           Initialize spectral fields to 0.

!**   Interface.
!     ----------
!        *CALL* *SUSPE0*

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        The spectral fields of the model.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-12-14
!        Modified : 93-04-15 L.Labbe - Init spectral orography if LOROCTV
!        Modified : 95-06-06 L.Isaksen - Reordering of spectral arrays
!        Modified : 96-02-20 L.Isaksen - Message Passing option added
!        Modified : 97-01-17 M.Janiskova - Iniatlize transmission coef.
!                                          for simplified radiation scheme
!        Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!      -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFD2D    
USE YOMSP    , ONLY : SPA3     ,SPA2
USE YOMMP    , ONLY : NPSP
USE YOMCT0   , ONLY : LRETCFOU

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "suspectcfou.intfb.h"
!      -----------------------------------------------------------------

!*       1.    Initialize spectral fields
!              ---------------------------

IF (LHOOK) CALL DR_HOOK('SUSPE0',0,ZHOOK_HANDLE)
SPA3(:,:,:)=0.0_JPRB

IF(NPSP == 1)THEN
  SPA2(:,1:NFD2D)=0.0_JPRB
ENDIF

!*       2.  Initialize transmission coefficients for simlpified
!            radiation scheme

! * storage in Fourier coefficients.
IF (LRETCFOU) THEN
  CALL SUSPECTCFOU
ENDIF

!      -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSPE0',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPE0
