SUBROUTINE SUSPECB(KFILE,PSPVOR,PSPDIV,PSPSP,PSPOR)

!**** *SUSPECB*  - Initialize the spectral fields of barotropic model.

!     Purpose.
!     --------
!           Initialize the spectral fields of the model (barotropic).

!**   Interface.
!     ----------
!        *CALL* *SUSPECB(....)*

!        Explicit arguments :
!        --------------------
!        KFILE : an indicator for which spectral file is to be read
!               KFILE = 0 the CFNISH file is read
!               KFILE = 1 the CFNGSH file is read
!               KFILE = 2 the CFNRF file is read
!               KFILE = 3 the CFNBS file is read
!               PSPVOR etc. - spectral fields

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        FAITOU,FADIES,FACILE,FAIRME,LFIMST,LFILAF,GRIBEX,
!        CHIEN,BALADS,TRAGEO

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 91-11-14
!        Modified : 92-02-14 by R. El Khatib (check file with namelist)
!        Modified : 93-04-13 by L. Labbe (Reads FA files for S.W. + orography)
!        D. Giard : 93-08-24 new option KFILE=3 if N2DINI=2
!        Modified : 93-12-09 by F. Bouttier (cleanup the logic)
!        Modified : 94-03-08 by K. YESSAD (NTMAX instead of NSMAX in TRAGEO).
!        Modified : 94-03-09 by L. Labbe (read ICMRFxxxx+nnnn files, 821)
!        Modified : 94-07-26 by D. Giard (managing matrix files)
!        Modified : 94-09-06 by F. Bouttier (update interface with BALAD)
!        Modified : 95-01-25 M.Hamrud (use PBGRIB instead of BUFFER IN)
!        Modified : 95-06-06 L.Isaksen (Reordering of spectral arrays)
!        Modified 95-10-01 by M.Hamrud (F90 style passing of args.)
!        Modified : 95-10-12 F.Veerse (add an argument to BALADS interface)
!        Modified : 96-04-04 L.Isaksen (added an argument to BALADS interface,
!                                       and adapted for Message Passing)
!        Modified : 98-01-19 L.Isaksen (bugfix)
!        Modified : 98-01-19 C.Temperton (changed amplitude of R-H wave)
!        98-03-17: R. EL KHATIB : SPREORD
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of LRPOLE in 'CALL CHIEN'.
!        Modified : 00-09-01 by D.Dent : MPE to MPL conversion
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        D. Paradis & R. El Khatib : 04-07-22 GRIBEX
!        Modified : 06-02-06 C.Temperton (bugfixes for Williamson et al. tests)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMSC   , ONLY : NINTLEN
USE YOMLUN   , ONLY : NULOUT   ,NINMSH   ,NINISH   ,NFGISH   ,&
 & NULCONT  ,NULROTC  ,NBIAS , NULNAM
USE YOMDIM   , ONLY : NDGLG    ,NDGSAG   ,NDGENG   ,NDLON    ,&
 & NFLEVG   ,NFD2D    ,NFLSUR   ,NSMAX    ,NSPECG   ,&
 & NSPEC2G  ,NSPEC2   ,NS3D     ,NCMAX    ,NCPEC2   ,&
 & NTMAX    ,NTPEC2   ,NUMP     ,NUMCP    ,NUMTP  
USE YOMCST   , ONLY : RDAY     ,RG       ,R, RPI, RA, ROMEGA
USE YOMGEM   , ONLY : NLOEN    ,NMEN     ,VALH     ,VBH      ,&
 & NSTTYP   ,NHTYP    ,RMUCEN   ,RLOCEN   ,RSTRET   ,VP00  , NGPTOT,RC2P1    ,RC2M1
USE YOMLAP   , ONLY : NASM0    ,NACM0    ,NATM0    ,NVALUE   ,&
 & NCVALUE  ,MYMS     ,NSPINDX  ,RLAPDI   ,RCAPIN   ,&
 & RTAPDI  
USE YOMCT0   , ONLY : NCONF    ,NQUAD    ,N2DINI   ,NPROC    ,&
 & NPRTRW   ,LARPEGEF  
USE YOMCT3   , ONLY : NSTEP
USE YOMCT1   , ONLY : LRFILAF
USE YOMRIP   , ONLY : NINDAT
USE YOMOP    , ONLY : CFNISH   ,CNMCA    ,CFNRF    ,CFNGSH   ,CFNBS    ,LINC
USE YOMGRB   , ONLY : NGRBS3   ,NGRBS2
USE YOMDYN   , ONLY : TSTEP    ,REFGEO
USE YOMTAG   , ONLY : MTAGDISTSP
USE YOMMP    , ONLY : NPRCIDS  ,MYPROC   ,MYSETW   ,NPROCM,  NBSETSP, NBSETLEV
USE YOMGC    , ONLY : GEMU, GSQM2, GECLO,  GESLO, GELAM, GELAT, RCORI,&
   & GOMVRL, GOMVRM ,OROGL    ,OROGM 
USE YOMVODCST , ONLY : CSTVO, CSTDIV
USE YOMSWE   , ONLY : ALPHASWE, GMUCENSWE
USE YOMMPG   , ONLY : NPOSSP   ,NDIM0G
USE MPL_MODULE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPVOR(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDIV(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSP(:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPOR(:) 
! === END OF INTERFACE BLOCK ===
REAL(KIND=JPRB) :: ZSPVORT(NFLSUR,NTPEC2), ZSPSPT (NTPEC2)
REAL(KIND=JPRB) :: ZSPVOR(NFLSUR,NCPEC2), ZSPSP (NCPEC2), ZSPEC (NCPEC2)
REAL(KIND=JPRB),ALLOCATABLE :: ZGPFIELDS(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZU(:), ZV(:), ZDIV(:)
REAL(KIND=JPRB) :: ZDIVE(1,NCPEC2)
REAL(KIND=JPRB) :: ZSCALAR(2,NSPEC2)

REAL(KIND=JPRB) :: ZGEMU(NDGSAG:NDGENG)
INTEGER(KIND=JPIM) :: IDATEF(11)

CHARACTER :: CLFNRF*16,CLR1*5,CLR2*4
CHARACTER (LEN = 1) ::  CLMODE, CLDMODE

REAL(KIND=JPRB) :: ZFPDAT(NSPEC2G), ZFLD(NSPEC2G)
INTEGER(KIND=JPIM) :: IGRIB(NSPEC2G+500)
#include "fcttim.h"

INTEGER(KIND=JPIM) :: JPMAXLAT
INTEGER(KIND=JPIM) :: JPMAXLEV

PARAMETER(JPMAXLEV=101,JPMAXLAT=1000)
! IF the number of level in the archived data or the number of latitudes
! are greater than the parameters above, an overwriting will take place
! in the call to GRIBEX
INTEGER(KIND=JPIM) ::   ISEC0(2),ISEC1(125),ISEC2(22+JPMAXLAT),ISEC3(2),ISEC4(42)
REAL(KIND=JPRB) ::      ZSEC2(10+2*(JPMAXLEV+1)),ZSEC3(2)
REAL(KIND=JPRB) :: ZDUMMY(1)
REAL(KIND=JPRB) :: Z12DAY, ZRADI, ZTC, ZSINTC, ZCOSTC, ZLC, ZSINLC, ZCOSLC, ZREFRAD
REAL(KIND=JPRB) :: ZU0, ZGH0, ZGHS0

INTEGER(KIND=JPIM) :: IVSETSC(1)
INTEGER(KIND=JPIM) :: I2SPEC, IDIR, IDIST, IERR, IFIELD, II, IINC,&
 & ILEN, ILENBYT, ILENV, ILEVEL, ILEVTY, ILONS, ILEV,&
 & IM, IN, INBARI, INDAT, INM, IOMASTER, IPARAM, &
 & IRCV, IREP, IREPRM, IRET, ISEFRE, &
 & ISP, ISPORTS, ITAG, ITAGR, ITAILTAG, ITM, &
 & ITYP, IUNITSH, IUNTIN, IVAR, IWORD, JA, JF, &
 & JFLD, JIR, JJ, JM, JMLOC, JN, JNM, JSE, JSP, J  , JINT
INTEGER(KIND=JPIM) :: INGRIB, INBITS, ISTRON, IPUILA

LOGICAL :: LL2D, LL3D, LLFICP, LLMORE
LOGICAL   :: LLEXIST
LOGICAL   :: LLCOSP=.TRUE.

REAL(KIND=JPRB) :: ZEPS, ZRLOREC, ZDSTRET
REAL(KIND=JPRB) :: ZXE, ZTB, ZTE, ZSILAT, ZLAT, ZCOLAT, ZSILON, ZLON, ZX, ZXEMX
REAL(KIND=JPRB) :: ZBX, ZBXE, ZUP, ZINTLAP, ZLATP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!       ---------------------------------------------------------------------
#include "namswe.h"

INTERFACE
#include "inv_trans.h"
#include "dir_trans.h"
END INTERFACE

#include "abor1.intfb.h"
#include "balads.intfb.h"
#include "chien.intfb.h"
#include "spreord.intfb.h"
#include "trageo.intfb.h"
#include "posnam.intfb.h"
#include "lacone.intfb.h"
#include "speuv.intfb.h"

!        0.    RESET ARRAYS
!              ------------

IF (LHOOK) CALL DR_HOOK('SUSPECB',0,ZHOOK_HANDLE)
IF (NFLEVG /= 1) CALL ABOR1('SUSPECB: NFLEVG /= 1')

!  Define the PE responsible for the reading of GRIB data

IOMASTER=1
LLMORE=.TRUE.
IRCV=IOMASTER
ITAILTAG=MTAGDISTSP+999

DO JJ=1,NSPEC2
  PSPVOR(1,JJ)=0.0_JPRB
  PSPDIV(1,JJ)=0.0_JPRB
  PSPSP(JJ)=0.0_JPRB
  PSPOR(JJ)=0.0_JPRB
ENDDO

!      -----------------------------------------------------------

!*       1.    All 2-D MODELS.
!              ---------------

IF((NCONF == 201).OR.(NCONF == 421).OR.(NCONF == 521).OR.&
   & (NCONF == 202).OR.(NCONF == 422).OR.(NCONF == 522)) THEN  

!          1.2. INITIALIZE 2-D FIELDS WITH ROSSBY-HAURWITZ WAVE
!               - - - - - - - - - - - - - - - - - - - - - - -

  IF(MOD(N2DINI,10) == 1)THEN

    WRITE(NULOUT,*) 'SUSPECB : SHALLOW-WATER FIELDS ARE',&
     & ' INITIALIZED WITH ARTIFICIAL INITIAL CONDITIONS'  

    IF(N2DINI == 1) THEN
!      Rossby-Haurwitz wave
      WRITE(NULOUT,*) ' Rossby-Hurwitz wave'
      IF(NCMAX < NSMAX*RSTRET) THEN
        WRITE(NULOUT,*) ' NCMAX HAS TO BE GREATER OR EQUAL'
        WRITE(NULOUT,*) ' TO NSMAX*RSTRET                 '
        WRITE(NULOUT,*) ' IF STRETCHING AND/OR ROTAION    '
        WRITE(NULOUT,*) ' 201 HAS TO BE PRECEEDED BY 911 AND 912'
        CALL ABOR1('NCMAX TOO SMALL')
      ENDIF

!            1.2.1 DEFINE (4,5) WAVE (ON REAL SPHERE)

      IN=5
      IM=4
      DO JJ=1,NCPEC2
        ZSPVOR(1,JJ)=0.0_JPRB
        ZSPEC(JJ)=0.0_JPRB
        ZSPSP(JJ)=0.0_JPRB
      ENDDO
      IF (MYSETW == NPROCM(IM)) THEN
        ZSPVOR(1,NACM0(IM)+(IN-IM)*2)=8.E-6_JPRB
        ZSPEC(NACM0(IM)+(IN-IM)*2)=8.E-6_JPRB
      ENDIF

!            1.2.2 SOLVE LINEAR BALANCE EQUATION ON REAL SPHERE.

      CALL BALADS(.FALSE.,1.0_JPRB/RG,1,1,NUMCP,NCMAX,NACM0,NCPEC2,ZSPEC,ZSPSP)
      DO JJ=1,NCPEC2
        ZSPSP(JJ)=ZSPSP(JJ)*RG
      ENDDO
      IF (MYSETW == NPROCM(0)) THEN
        ZSPSP(NACM0(0))=REFGEO
      ENDIF
      DO JM=0,NCMAX
        INM=NACM0(JM)+(NCMAX-JM)*2
        ZSPSP(INM  )=0.0_JPRB
        ZSPSP(INM+1)=0.0_JPRB
      ENDDO

      IF(NSTTYP /= 1.OR. ABS(RSTRET-1.0_JPRB) >= 1.E-14_JPRB) THEN

!            1.2.3  TRANSFORM TO STRETCHED/ROTATED SPHERE

!           This is done here because 926 does not work for 2D models
        WRITE(NULOUT,*) 'SUSPECB:TRAGEO IS DONE ON SPVOR and SPSP'
        DO JSP=1,NCPEC2
          ZSPVOR(1,JSP)=RCAPIN(NCVALUE(JSP))*ZSPVOR(1,JSP)
        ENDDO
        IDIR=-1
        IFIELD=1
        IF(NSTTYP == 1)THEN
          ISPORTS=0
        ELSE
          ISPORTS=1
        ENDIF
        ITYP=1
        ZRLOREC=0._JPRB
        CALL TRAGEO(NULROTC,NULCONT,NULOUT,&
         & IFIELD,NFLSUR,&
         & NTMAX,NATM0,NTPEC2,NUMTP,NCMAX,NACM0,NCPEC2,&
         & NUMCP,MYMS,ISPORTS,ITYP,IDIR,RLOCEN,ZRLOREC,&
         & ZSPSPT,ZSPSP)  
        CALL TRAGEO(NULROTC,NULCONT,NULOUT,&
         & IFIELD,NFLSUR,&
         & NTMAX,NATM0,NTPEC2,NUMTP,NCMAX,NACM0,NCPEC2,&
         & NUMCP,MYMS,ISPORTS,ITYP,IDIR,RLOCEN,ZRLOREC,&
         & ZSPVORT,ZSPVOR)  
        DO JMLOC=1,NUMP
          IM=MYMS(JMLOC)
          DO JIR=0,MIN(1,IM)
            DO JN=IM,NSMAX
              INM=NASM0(IM)+(JN-IM)*2+JIR
              ITM=NATM0(IM)+(JN-IM)*2+JIR
              PSPVOR(1,INM)=RTAPDI(JN)*ZSPVORT(1,ITM)
              PSPSP (  INM)=ZSPSPT(ITM)
            ENDDO
          ENDDO
        ENDDO

      ELSE

!            1.2.4. DO NOT TRANSFORM THE SPHERE

        DO JSE=1,NSPEC2
          PSPVOR(1,JSE)=ZSPVOR(1,JSE)
          PSPSP(JSE)=ZSPSP(JSE)
        ENDDO

      ENDIF
    ELSEIF(N2DINI == 11) THEN

!     Advection of cosine bell, including direction over the pole
      WRITE(NULOUT,*) ' Advection of a cosine bell'
      ALLOCATE(CSTVO(1,NSPEC2))
      ALLOCATE(CSTDIV(1,NSPEC2))
      ALLOCATE(ZGPFIELDS(NGPTOT,4,1))
      Z12DAY=12._JPRB*RDAY
      ALPHASWE=0.0_JPRB
      CALL POSNAM(NULNAM,'NAMSWE')
      READ(NULNAM,NAMSWE)
      WRITE(NULOUT,*) ' Angle ALPHA=',ALPHASWE,' degrees'
      ALPHASWE=ALPHASWE*RPI/180.0_JPRB
      ZU0=2.0_JPRB*RPI*RA/Z12DAY
      ZLC=3.0_JPRB*RPI*0.5_JPRB
      ZTC=0.0_JPRB
      ZCOSTC=COS(ZTC)
      ZSINTC=SIN(ZTC)
      ZCOSLC=COS(ZLC)
      ZSINLC=SIN(ZLC)
      DO J=1,NGPTOT
        ZGPFIELDS(J,1,1)=ZU0*(GSQM2(J)*COS(ALPHASWE)+GEMU(J)*GECLO(J)*SIN(ALPHASWE))
        ZGPFIELDS(J,2,1)=-ZU0*GESLO(J)*SIN(ALPHASWE)
        ZRADI=RA*ACOS(ZSINTC*GEMU(J)+ZCOSTC*GSQM2(J)*COS(GELAM(J)-ZLC))
        IF(ZRADI < RA/3.0_JPRB) THEN
          ZGPFIELDS(J,3,1)=500.0_JPRB*(1.0_JPRB+COS(3.0_JPRB*RPI*ZRADI/RA))*RG
        ELSE
          ZGPFIELDS(J,3,1)=0.0_JPRB
        ENDIF
      ENDDO
      IVSETSC(1) = NBSETSP
      CALL DIR_TRANS(PSPVOR=ZSPVOR,PSPDIV=ZDIVE, &
        &PSPSCALAR=ZSCALAR,KPROMA=NGPTOT,KVSETUV=NBSETLEV,&
        &PGP=ZGPFIELDS)
      DO JSE=1,NSPEC2
        PSPVOR(1,JSE)=ZSPVOR(1,JSE)
        PSPDIV(1,JSE)=ZDIVE(1,JSE)
        PSPSP(JSE)=ZSCALAR(1,JSE)
        CSTVO(1,JSE)=ZSPVOR(1,JSE)
        CSTDIV(1,JSE)=ZDIVE(1,JSE)
      ENDDO

    ELSEIF(N2DINI == 21) THEN
!     Global steady state non-linear zonal geostrophic flow
      WRITE(NULOUT,*) ' Global steady state non-linear zonal geostrophic flow'
      ALLOCATE(ZGPFIELDS(NGPTOT,4,1))
      Z12DAY=12._JPRB*RDAY
      ALPHASWE=0.0_JPRB
      CALL POSNAM(NULNAM,'NAMSWE')
      READ(NULNAM,NAMSWE)
      WRITE(NULOUT,*) ' Angle ALPHA=',ALPHASWE,' degrees'
      ALPHASWE=ALPHASWE*RPI/180.0_JPRB
      GMUCENSWE=SIN(0.5_JPRB*RPI-ALPHASWE)
      ZU0=2.0_JPRB*RPI*RA/Z12DAY
      ZGH0=2.94E4_JPRB
      DO J=1,NGPTOT
        ZGPFIELDS(J,1,1)=ZU0*(GSQM2(J)*COS(ALPHASWE)+GEMU(J)*GECLO(J)*SIN(ALPHASWE))
        ZGPFIELDS(J,2,1)=-ZU0*GESLO(J)*SIN(ALPHASWE)
        ZGPFIELDS(J,3,1)=ZGH0-(RA*ROMEGA*ZU0+ZU0*ZU0*0.5_JPRB)*&
          &(-GECLO(J)*GSQM2(J)*SIN(ALPHASWE)+GEMU(J)*COS(ALPHASWE))**2
        RCORI(J)=2.0_JPRB*ROMEGA*(-GECLO(J)*GSQM2(J)*SIN(ALPHASWE)+GEMU(J)*COS(ALPHASWE))
      ENDDO
      ZDSTRET=2.0_JPRB*RSTRET
      CALL LACONE(NGPTOT,1,NGPTOT,1,1,ROMEGA,ZDSTRET,RC2M1,RC2P1,GMUCENSWE,&
        &RA,RPI,GELAM,GELAT,GOMVRL,GOMVRM)
      IVSETSC(1) = NBSETSP
      CALL DIR_TRANS(PSPVOR=ZSPVOR,PSPDIV=ZDIVE, &
        &PSPSCALAR=ZSCALAR,KPROMA=NGPTOT,KVSETUV=NBSETLEV,&
        &PGP=ZGPFIELDS)
      DO JSE=1,NSPEC2
        PSPVOR(1,JSE)=ZSPVOR(1,JSE)
        PSPDIV(1,JSE)=ZDIVE(1,JSE)
        PSPSP(JSE)=ZSCALAR(1,JSE)
      ENDDO

    ELSEIF(N2DINI == 31) THEN
      WRITE(NULOUT,*) ' Steady state nonlinear zonal geostrophic flow'
      WRITE(NULOUT,*) '     with compact support'
      ALLOCATE(ZGPFIELDS(NGPTOT,4,1))
      Z12DAY=12._JPRB*RDAY
      ALPHASWE=0.0_JPRB
      CALL POSNAM(NULNAM,'NAMSWE')
      READ(NULNAM,NAMSWE)
      WRITE(NULOUT,*) ' Angle ALPHA=',ALPHASWE,' degrees'
      ALPHASWE=ALPHASWE*RPI/180.0_JPRB
      GMUCENSWE=SIN(0.5_JPRB*RPI-ALPHASWE)
      ZXE=0.3_JPRB
      ZU0=2.0_JPRB*RPI*RA/Z12DAY
      Z12DAY=12._JPRB*RDAY
      ZTB=-RPI/6.0_JPRB
      ZTE=RPI*0.5_JPRB
      ZGH0=2.94E4_JPRB
      DO J=1,NGPTOT
        ZSILAT=GEMU(J)*COS(ALPHASWE)-GSQM2(J)*GECLO(J)*SIN(ALPHASWE)
        ZLAT=ASIN(ZSILAT)
        ZCOLAT=SQRT(1.0_JPRB-ZSILAT*ZSILAT)
        ZSILON=GESLO(J)*GSQM2(J)/ZCOLAT
        ZLON=ASIN(ZSILON)
        IF(COS(ALPHASWE)*GECLO(J)*GSQM2(J)+SIN(ALPHASWE)*GEMU(J) < 0.0_JPRB) THEN
          ZLON=RPI-ZLON
        ENDIF
        ZX=ZXE*(ZLAT-ZTB)/(ZTE-ZTB)
        ZXEMX=ZXE-ZX
        IF(ZX <= 0.0_JPRB) THEN
          ZBX=0.0_JPRB
        ELSE
          ZBX=EXP(-1.0_JPRB/ZX)
        ENDIF
        IF(ZXEMX <= 0.0_JPRB) THEN
          ZBXE=0.0_JPRB
        ELSE
          ZBXE=EXP(-1.0_JPRB/ZXEMX)
        ENDIF
        ZUP=ZU0*ZBX*ZBXE*EXP(4.0_JPRB/ZXE)
        ZGPFIELDS(J,2,1)=-ZUP*SIN(ALPHASWE)*ZSILON/GSQM2(J)
        ZGPFIELDS(J,1,1)=(ZGPFIELDS(J,2,1)*GEMU(J)*GESLO(J)+ZUP*COS(ZLON))/GECLO(J)
        ZGPFIELDS(J,3,1)=ZGH0
        ZINTLAP=RPI/1000.0_JPRB
        DO JINT=1,1000
          ZLATP=-RPI/2+(JINT-1)*ZINTLAP
          IF(ZLATP <= ZLAT) THEN
            ZX=ZXE*(ZLATP-ZTB)/(ZTE-ZTB)
            ZXEMX=ZXE-ZX
            IF(ZX <= 0.0_JPRB) THEN
              ZBX=0.0_JPRB
            ELSE
              ZBX=EXP(-1.0_JPRB/ZX)
            ENDIF
            IF(ZXEMX <= 0.0_JPRB) THEN
              ZBXE=0.0_JPRB
            ELSE
              ZBXE=EXP(-1.0_JPRB/ZXEMX)
            ENDIF
            ZUP=ZU0*ZBX*ZBXE*EXP(4.0_JPRB/ZXE)
            ZGPFIELDS(J,3,1)=ZGPFIELDS(J,3,1)-RA*(2.0_JPRB*ROMEGA*SIN(ZLATP)&
              &+ZUP*TAN(ZLATP)/RA)*ZUP*ZINTLAP
          ENDIF
        ENDDO
        RCORI(J)=2.0_JPRB*ROMEGA*(-GECLO(J)*GSQM2(J)*SIN(ALPHASWE)+GEMU(J)*COS(ALPHASWE))
      ENDDO
      ZDSTRET=2.0_JPRB*RSTRET
      CALL LACONE(NGPTOT,1,NGPTOT,1,1,ROMEGA,ZDSTRET,RC2M1,RC2P1,GMUCENSWE,&
        &RA,RPI,GELAM,GELAT,GOMVRL,GOMVRM)
      IVSETSC(1) = NBSETSP
      CALL DIR_TRANS(PSPVOR=ZSPVOR,PSPDIV=ZDIVE, &
        &PSPSCALAR=ZSCALAR,KPROMA=NGPTOT,KVSETUV=NBSETLEV,&
        &PGP=ZGPFIELDS)
      DO JSE=1,NSPEC2
        PSPVOR(1,JSE)=ZSPVOR(1,JSE)
        PSPDIV(1,JSE)=ZDIVE(1,JSE)
        PSPSP(JSE)=ZSCALAR(1,JSE)
      ENDDO

    ELSEIF(N2DINI == 41) THEN
      WRITE(NULOUT,*) ' Forced non-linear system with a translating low'
      WRITE(NULOUT,*) ' Not yet implemented'
      CALL ABOR1(' Test No 4 not yet implemented')

    ELSEIF(N2DINI == 51) THEN
      WRITE(NULOUT,*) ' Zonal flow over an isolated mountain'
      ALLOCATE(ZGPFIELDS(NGPTOT,4,1))
      Z12DAY=12._JPRB*RDAY
!  ZONAL FLOW => ALPHA=0.0
      ALPHASWE=0.0_JPRB
      ALPHASWE=ALPHASWE*RPI/180.0_JPRB
      GMUCENSWE=SIN(0.5_JPRB*RPI-ALPHASWE)
      ZU0=20.0_JPRB
      ZGH0=5.84476E4_JPRB
      ZGHS0=1.96133E4_JPRB
      ZLC=3.0_JPRB*RPI*0.5_JPRB
      ZTC=RPI/6.0_JPRB
      ZREFRAD=RPI/9.0_JPRB
      DO J=1,NGPTOT
        ZGPFIELDS(J,1,1)=ZU0*(GSQM2(J)*COS(ALPHASWE)+GEMU(J)*GECLO(J)*SIN(ALPHASWE))
        ZGPFIELDS(J,2,1)=-ZU0*GESLO(J)*SIN(ALPHASWE)
        ZGPFIELDS(J,3,1)=ZGH0-(RA*ROMEGA*ZU0+ZU0*ZU0*0.5_JPRB)*&
          &(-GECLO(J)*GSQM2(J)*SIN(ALPHASWE)+GEMU(J)*COS(ALPHASWE))**2
        ZRADI=SQRT(MIN(ZREFRAD*ZREFRAD,(GELAM(J)-ZLC)**2+(GELAT(J)-ZTC)**2))
        ZGPFIELDS(J,4,1)=ZGHS0*(1.0_JPRB-ZRADI/ZREFRAD)
      ENDDO
      IVSETSC(1) = NBSETSP
      CALL DIR_TRANS(PSPVOR=ZSPVOR,PSPDIV=ZDIVE, &
        &PSPSCALAR=ZSCALAR,KPROMA=NGPTOT,KVSETUV=NBSETLEV,&
        &PGP=ZGPFIELDS)
      DO JSE=1,NSPEC2
        PSPVOR(1,JSE)=ZSPVOR(1,JSE)
        PSPDIV(1,JSE)=ZDIVE(1,JSE)
        PSPSP(JSE)=ZSCALAR(1,JSE)
        PSPOR(JSE)=ZSCALAR(2,JSE)
      ENDDO
      ALLOCATE(ZDIV(NSPEC2))
      ALLOCATE(ZU(NGPTOT))
      ALLOCATE(ZV(NGPTOT))
      DO JSE=1,NSPEC2
        ZDIV(JSE)=0.0_JPRB
      ENDDO
      CALL SPEUV(ZDIV,PSPOR,ZU,ZV,1,1,2)
      DO J =1,NGPTOT
        OROGL(J) = ZU(J)
        OROGM(J) = ZV(J)
      ENDDO


    ENDIF

!          1.3. INITIALIZE 2-D FIELDS WITH ARPEGE FILE
!               - - - - - - - - - - - - - - - - - - -

  ELSEIF(N2DINI == 2)THEN
    WRITE(NULOUT,*) 'SUSPECB: SW FIELDS ARE READ ','FROM ARPEGE FILE'
    IF(.NOT.LARPEGEF)THEN
      CALL ABOR1('INCONSISTENCY BETWEEN LARPEGEF AND L2DINI')
    ENDIF

!          1.3.1 OPEN THE FA FILE

    IF (KFILE == 0) THEN
      WRITE(NULOUT,*) 'ARPEGE FILENAME : ',CFNISH
      IUNTIN = NINISH
      INBARI=0
      CALL FAITOU(IREP,IUNTIN,.TRUE.,CFNISH,'OLD',.TRUE.,&
       & .TRUE.,2,1,INBARI,CNMCA)  
    ELSEIF (KFILE == 1) THEN
      WRITE(NULOUT,*) 'ARPEGE FILENAME : ',CFNGSH
      IUNTIN = NINMSH
      INBARI=0
      CALL FAITOU(IREP,IUNTIN,.TRUE.,CFNGSH,'OLD',.TRUE.,&
       & .TRUE.,2,1,INBARI,CNMCA)  
    ELSEIF (KFILE == 2) THEN
      CLR1 = CFNRF(1:5)
      CLR2 = CFNRF(6:9)
      IF (LINC) THEN
        IINC=NINT(REAL(NSTEP,JPRB)*TSTEP/3600._JPRB)
      ELSE
        IINC=NSTEP
      ENDIF
      WRITE(CLFNRF,'(A5,A4,''+'',I4.4)') CLR1,CLR2,IINC
      WRITE(NULOUT,*) 'ARPEGE FILENAME : ',CLFNRF
      IUNTIN = NFGISH
      INBARI=0
      CALL FAITOU(IREP,IUNTIN,.TRUE.,CLFNRF,'OLD',.TRUE.,&
       & .TRUE.,2,1,INBARI,CNMCA)  
    ELSEIF (KFILE == 3) THEN
      WRITE(NULOUT,*) 'ARPEGE FILENAME : ',CFNBS
      IUNTIN = NBIAS
      INBARI=0
      CALL FAITOU(IREP,IUNTIN,.TRUE.,CFNBS,'OLD',.TRUE.,&
       & .TRUE.,2,1,INBARI,CNMCA)  
    ELSE
      WRITE(*,'('' INVALID ARGUMENT TO SUSPECB, KFILE='',I6)') KFILE
      CALL ABOR1('SUSPECB: ABOR1 CALLED')
    ENDIF
    CALL LFIMST(IREP,IUNTIN,.FALSE.)
    IF (LRFILAF) THEN
      CALL LFILAF(IREP,IUNTIN,.FALSE.)
    ENDIF

!           1.3.2  READ AND TEST SOME FIELDS, CHECK DATE

    ZEPS=1.E-10_JPRB
    ILEV=1
    CALL CHIEN(CNMCA,NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,&
     & NDGLG,NDLON,NLOEN,NMEN,ZGEMU,NHTYP,ILEV,VP00,VALH,VBH,&
     & NQUAD,0,NDGSAG,NDGENG,ZEPS,LLFICP,NULOUT)  

    ISEFRE=(NSMAX+1)*(NSMAX+1)

    CALL FADIES(IREP,IUNTIN,IDATEF)
    WRITE(NULOUT,*) 'YOU READ FIELDS OF THE : ',IDATEF(3),' ',&
     & IDATEF(2),' ',IDATEF(1)  
    INDAT = IDATEF(1) * 10000 + IDATEF(2) * 100 + IDATEF(3)
    WRITE(NULOUT,*) ' DATE OF THE MODEL : ',NINDAT
    WRITE(NULOUT,*) ' DATE OF THE FILE  : ',INDAT

    IF (INDAT /= NINDAT) CALL ABOR1('DATE INCOHERENCE')

!           1.3.3  READ FIELDS & CLOSE FILE

    CALL FANION(IREP,IUNTIN,'SURF',1,'PRESSION',LLEXIST,LLCOSP,INGRIB,INBITS,&
     & ISTRON,IPUILA)
    IF (.NOT.(INGRIB==-1 .OR. INGRIB==3)) THEN
      CALL FACILE(IREP,IUNTIN,'SURF',1,'PRESSION',ZSPEC,.TRUE.)
      CALL SPREORD(1,ZSPEC,PSPSP,.TRUE.)
    ELSE
      CALL FACILE(IREP,IUNTIN,'SURF',1,'PRESSION',PSPSP,.TRUE.)
    ENDIF 

    CALL FANION(IREP,IUNTIN,'SPECSURF',1,'GEOPOTENTIEL',LLEXIST,LLCOSP,INGRIB,&
     & INBITS,ISTRON,IPUILA)
    IF (.NOT.(INGRIB==-1 .OR. INGRIB==3)) THEN
      CALL FACILE(IREP,IUNTIN,'SPECSURF',1,'GEOPOTENTIEL',ZSPEC,.TRUE.)
      CALL SPREORD(1,ZSPEC,PSPOR,.TRUE.)
    ELSE
      CALL FACILE(IREP,IUNTIN,'SPECSURF',1,'GEOPOTENTIEL',PSPOR,.TRUE.)
    ENDIF

    CALL FANION(IREP,IUNTIN,'S',1,'FONC.COURANT',LLEXIST,LLCOSP,INGRIB,INBITS,&
     & ISTRON,IPUILA)
    IF (.NOT.(INGRIB==-1 .OR. INGRIB==3)) THEN
      CALL FACILE(IREP,IUNTIN,'S',1,'FONC.COURANT',ZSPEC,.TRUE.)
      CALL SPREORD(1,ZSPEC,PSPVOR,.TRUE.)
    ELSE
      CALL FACILE(IREP,IUNTIN,'S',1,'FONC.COURANT',PSPVOR,.TRUE.)
    ENDIF
    DO JNM=1,NSPEC2
      PSPVOR(1,JNM)=PSPVOR(1,JNM)*RLAPDI(NVALUE(JNM))
    ENDDO

    CALL FANION(IREP,IUNTIN,'S',1,'POT.VITESSE',LLEXIST,LLCOSP,INGRIB,INBITS,&
     & ISTRON,IPUILA)
    IF (.NOT.(INGRIB==-1 .OR. INGRIB==3)) THEN
      CALL FACILE(IREP,IUNTIN,'S',1,'POT.VITESSE',ZSPEC,.TRUE.)
      CALL SPREORD(1,ZSPEC,PSPDIV,.TRUE.)
    ELSE
      CALL FACILE(IREP,IUNTIN,'S',1,'POT.VITESSE',PSPDIV,.TRUE.)
    ENDIF
    DO JNM=1,NSPEC2
      PSPDIV(1,NSPINDX(JNM))=ZSPEC(JNM)*RLAPDI(NVALUE(JNM))
    ENDDO
    CALL FAIRME(IREP,IUNTIN,'UNKNOWN')

!          1.4. INITIALIZE SW FIELDS WITH ECMWF MARS ARCHIVE
!               - - - - - - - - - - - - - - - - - - - - - -

  ELSEIF(N2DINI == 3)THEN

    IUNTIN=NINISH
    IFIELD=NS3D+NFD2D
    I2SPEC=2*NSPECG
    ILENBYT=2*NINTLEN*I2SPEC
    ILENV=2*(1+1)

!  Only the IOMASTER PE reads the GRIB data and distribute it

    IF (MYPROC == IOMASTER) THEN
      CLMODE='R'
      WRITE(NULOUT,*) 'SUSPECB: SW FIELDS ARE READ ',&
       & 'FROM ECMWF MARS RETRIEVE'  
      IF (KFILE == 0) THEN
        WRITE(UNIT=NULOUT,FMT='(A,A)')&
         & ' INITIAL DATA TO BE READ FROM FILE ',CFNISH  
        CALL PBOPEN(IUNITSH,CFNISH,CLMODE,IRET)
      ELSEIF (KFILE == 1) THEN
        WRITE(UNIT=NULOUT,FMT='(A,A)')&
         & ' STARTING POINT FOR MINIMIZATION TO BE READ FROM FILE '&
         & ,CFNGSH  
        CALL PBOPEN(IUNITSH,CFNGSH,CLMODE,IRET)
      ELSEIF (KFILE == 2) THEN
        WRITE(UNIT=NULOUT,FMT='(A,A)')&
         & ' REFERENCE DATA TO BE READ FROM FILE '&
         & ,CFNRF  
        CALL PBOPEN(IUNITSH,CFNRF,CLMODE,IRET)
      ELSE
        WRITE(*,'('' INVALID ARGUMENT TO SUSPECB, KFILE='',I6)') KFILE
        CALL ABOR1('SUSPECB: ABOR1 CALLED')
      ENDIF
      IF(IRET /= 0) THEN
        WRITE(NULOUT,'(A,I2)')'SUSPECB: PROBLEM IN PBOPEN, IRET=',IRET
        CALL ABOR1('SUSPECB: PROBLEM IN PBOPEN')
      ENDIF
    ENDIF

    DO JFLD=1,IFIELD
      IF (MYPROC == IOMASTER) THEN
        CALL PBGRIB(IUNITSH,IGRIB,ILENBYT,ILONS,IRET)
        IF(IRET == 0) THEN
          ILONS=(ILONS+(NINTLEN-1))/NINTLEN
        ELSE
          WRITE(NULOUT,'(A,I2)')'SUSPECB: PROBLEM IN PBGRIB, IRET=',IRET
          CALL ABOR1('SUSPECB: PROBLEM IN PBGRIB')
        ENDIF
        CLDMODE='D'
        CALL GSTATS(1703,0)
        CALL GRIBEX(ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,&
         & ZFPDAT,I2SPEC,IGRIB,ILONS,IWORD,CLDMODE,IERR)  
        CALL GSTATS(1703,1)
        IPARAM=ISEC1(6)
        ILEVTY=ISEC1(7)
        ILEVEL=1
        IREPRM=1
        IF(ISEC2(2) /= NSMAX) THEN
          WRITE(NULOUT,*) ' RESOLUTION OF MODEL ',NSMAX &
           & ,' ,OF INITIAL DATA ',ISEC2(2)  
          CALL ABOR1('SUSPECB: ABOR1 CALLED')
        ENDIF
        IVAR=0
        LL3D=.FALSE.
        LL2D=.FALSE.
        DO JF=1,NS3D
          IF(IPARAM == NGRBS3(JF)) THEN
            WRITE(NULOUT,*) ' PARAM ',IPARAM,' LEVEL ',ILEVEL,' FOUND '
            IVAR=JF
            LL3D=.TRUE.
          ENDIF
        ENDDO
        DO JF=1,NFD2D
          IF(IPARAM == NGRBS2(JF)) THEN
            WRITE(NULOUT,*) ' PARAM ',IPARAM,' LEVEL ',0,' FOUND'
            IVAR=JF
            LL2D=.TRUE.
          ENDIF
        ENDDO
        IF(LL3D) THEN
          WRITE(NULOUT,*) ' 3-D FIELD PARAM=',IPARAM &
           & ,' LEVEL= ',ILEVEL,' INITIALIZED FROM GRIB'  
        ELSEIF(LL2D) THEN
          WRITE(NULOUT,*) ' 2-D FIELD PARAM=',IPARAM,' INITIALIZED FROM GRIB'
        ENDIF
        IREPRM=1
        IF(IREPRM == 1) THEN
          IF (LL3D.OR.LL2D) THEN
            II=0
            DO JM=0,NSMAX
              DO JN=JM,NSMAX
                IDIST=NDIM0G(JM)+(JN-JM)*2
                II=II+1
                ZFLD(IDIST)=ZFPDAT(II)
                IDIST=IDIST+1
                II=II+1
                IF(JM /= 0) THEN
                  ZFLD(IDIST)=ZFPDAT(II)
                ELSE
                  ZFLD(IDIST)=0.0_JPRB
                ENDIF
              ENDDO
            ENDDO
          ENDIF
        ELSEIF(IREPRM == 2) THEN

! Reorder and make sure that imaginary m=0 values are set to zero.

          IF (LL3D.OR.LL2D) THEN
            II = 0
            DO JN=0,NSMAX
              DO JM=-JN,JN
                IM = ABS(JM)
                ISP = NDIM0G(IM)+(JN-IM)*2
                IF(JM < 0) ISP = ISP + 1
                II=II+1
                ZFLD(ISP)=ZFPDAT(II)
                IF(JM == 0) ZFLD(ISP+1)=0.0_JPRB
              ENDDO
            ENDDO
          ENDIF
        ENDIF

! Define tag that identify the field type

        IF(LL3D) THEN
          IF(IVAR == 1) THEN
            ITAG=MTAGDISTSP+333+1
          ELSE
            ITAG=MTAGDISTSP+333+2
          ENDIF
        ELSEIF(LL2D) THEN
          ITAG=MTAGDISTSP+222
        ELSE
          ITAG=MTAGDISTSP+111
        ENDIF

! Send spectral fields to processors

        IF (LL3D.OR.LL2D) THEN
          DO JA=1,NPRTRW
            ILEN = NPOSSP(JA+1)-NPOSSP(JA)
            IF(ILEN > 0) THEN
              IF(JA /= IOMASTER) THEN
                CALL MPL_SEND(ZFLD(NPOSSP(JA):NPOSSP(JA+1)-1), &
                 & KDEST=NPRCIDS(JA),KTAG=ITAG,CDSTRING='SUSPECB:')  
              ENDIF
            ENDIF
          ENDDO
        ENDIF

! Send tag to indicate no more fields

        IF(NPROC > 1.AND.JFLD == IFIELD) THEN
          DO JA=1,NPRTRW
            IF(JA /= IOMASTER) THEN
              CALL MPL_SEND(ZDUMMY(1:1),KDEST=NPRCIDS(JA),KTAG=ITAILTAG, &
               & CDSTRING='SUSPECB:')  
            ENDIF
          ENDDO
        ENDIF
      ENDIF

! All PE's receive spectral fields from IOMASTER

      IF( NSPEC2 > 0 )THEN
        IF( MYPROC /= IOMASTER ) THEN
          CALL MPL_RECV(ZFLD(1:NSPEC2),KSOURCE=NPRCIDS(IRCV), &
           & KOUNT=ILEN,KRECVTAG=ITAGR,CDSTRING='SUSPECB:')  
          IF (ITAGR == ITAILTAG) GOTO 999
          IF( ILEN /= NSPEC2 )THEN
            CALL ABOR1(' SUSPECB: INVALID RECEIVE MESSAGE LENGTH')
          ENDIF
        ELSE
          ITAGR=ITAG
        ENDIF

        ILEVEL=1
        IF (ITAGR == MTAGDISTSP+333+1) THEN
          DO JJ=1,NSPEC2
!                 Relative Vorticity
            PSPVOR(ILEVEL,JJ)=ZFLD(JJ)
          ENDDO
        ELSEIF (ITAGR == MTAGDISTSP+333+2) THEN
          DO JJ=1,NSPEC2
!                 Divergence
            PSPDIV(ILEVEL,JJ)=ZFLD(JJ)
          ENDDO
        ELSEIF (ITAGR == MTAGDISTSP+222) THEN
          DO JJ=1,NSPEC2
!                 Pressure
            PSPSP(JJ)=ZFLD(JJ)
          ENDDO
        ELSEIF (ITAGR == MTAGDISTSP+111) THEN
!   Do Nothing for this field
        ELSE
          CALL ABOR1('SUSPECB: INVALID MESSAGE PASSING TAG')
        ENDIF
      ENDIF
    ENDDO

    IF (MYPROC == IOMASTER) THEN
      CALL PBCLOSE(IUNITSH,IRET)
      IF(IRET /= 0) THEN
        WRITE(NULOUT,'(A,I2)')'SUSPECB: PROBLEM IN PBCLOSE, IRET=',IRET
        CALL ABOR1('SUSPECB: PROBLEM IN PBCLOSE')
      ENDIF
    ENDIF

!          Security if N2DINI>3
  ELSE
    WRITE(NULOUT,*) ' N2DINI=',N2DINI
    CALL ABOR1(' INVALID VALUE FOR N2DINI')
  ENDIF
  999 CONTINUE

!     ------------------------------------------------------------------

!*       2.    BAROTROPIC VORTICITY MODEL (reset D and Phi to 0.)
!              --------------------------------------------------

  IF((NCONF == 202).OR.(NCONF == 422).OR.(NCONF == 522)) THEN
    DO JJ=1,NSPEC2
      PSPDIV(1,JJ)=0.0_JPRB
!           PSPSP(JJ)=0.
    ENDDO
  ENDIF

!     ------------------------------------------------------------------

ELSE
  CALL ABOR1('UNKNOWN 2D CONFIGURATION')
ENDIF

IF (LHOOK) CALL DR_HOOK('SUSPECB',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPECB

