SUBROUTINE SUSPECG2(K3DINI,PSPA3,PSPA2,CDPATH,KFILE,LDINOR,LDSPOR)

!**** *SUSPECG2*  - Initialize the spectral fields for LDYNCORE

!     Purpose.
!     --------
!           Initialize the spectral fields for LDYNCORE
!           For some specified or analytical initial conditions

!**   Interface.
!     ----------
!        *CALL* *SUSPECG2(.....)*

!        Explicit arguments :
!        --------------------
!        K3DINI: type of initial data test case
!        KFILE : an indicator for which spectral file is to be read
!               KFILE = 0 the CFNISH file is read
!               PSPVOR etc. - spectral fields
!               LDINOR - switch for initializing the orography
!               LDSPOR - switch indicating whether spectral orography
!                        field has been read

!        Implicit arguments :
!        --------------------
!        The spectral fields of the model.
!        The boundary condition fields of the model.
!         
!     Method.
!     -------
!        See documentation

!     Externals.  
!     ----------  

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Nils Wedi *ECMWF*

!     Modifications.
!     --------------
!        Original : 07-07-20 Nils Wedi
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMDIM   , ONLY : NFLEVL   ,NSPEC2   , NFLEVG ,&
 & NS3D     ,NS2D     , NFTHER, NSMAX
USE YOMOP    , ONLY : CFNISH
USE YOMGRB   , ONLY : NGRBZ    ,NGRBLNSP ,NGRBSP3 ,&
 & NGRBVO, NGRBD, NGRBT, NGRB118, NGRB119
USE YOMVFP   , ONLY : LSPINT
USE YOM_YGFL , ONLY : YGFL, YGFLC
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, CLOSE_IOREQUEST

USE YOMGC     , ONLY : GEMU,GELAM,GELAT
USE YOMGEM    , ONLY : VAH, VBH, VC, VDELB, NGPTOT
USE YOMCST    , ONLY : RPI, RA, RD, RG, ROMEGA, RCPD
USE YOMDYNCORE, ONLY : RT00_DYN, RP00_DYN, RU00_DYN, RST0_DYN, NTESTCASE, NOISEVOR,&
 & DRAGNORM, RDELTA_T, RPRESSURE_SCALE, RHS_KAPPA

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)             :: K3DINI
REAL(KIND=JPRB),TARGET     ,INTENT(OUT)   :: PSPA3(:,:,:) 
REAL(KIND=JPRB)            ,INTENT(OUT)   :: PSPA2(:,:)
CHARACTER(LEN=*)  ,OPTIONAL,INTENT(IN)    :: CDPATH
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN)    :: KFILE 
LOGICAL           ,OPTIONAL,INTENT(IN)    :: LDINOR 
LOGICAL           ,OPTIONAL,INTENT(INOUT) :: LDSPOR 

CHARACTER (LEN = 16) ::  CLFN

INTEGER(KIND=JPIM) :: IGRIB2D(NS2D),IGRIB3D(NS3D+YGFL%NUMFLDS)
INTEGER(KIND=JPIM) :: IFILE,I2D,JGFL,JK,I3D,ISPTOGP
LOGICAL :: LLINOR, LLSPINT, LLINCR
LOGICAL :: LLMISSING_3D(NFLEVG,NS3D+YGFL%NUMFLDS)
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB),POINTER :: ZSPA3(:,:,:)

INTEGER(KIND=JPIM) :: JWORD, JLEV
REAL(KIND=JPRB)    :: ZPSP(NGPTOT)
REAL(KIND=JPRB)    :: ZU0,ZT0,ZT02,ZVP00,ZRHO0
REAL(KIND=JPRB) ::    ZPRESH(NGPTOT,0:NFLEVG),  ZPRESF (NGPTOT,NFLEVG)
REAL(KIND=JPRB) ::    ZDELP (NGPTOT,NFLEVG), ZRDELP(NGPTOT,NFLEVG)
REAL(KIND=JPRB) ::    ZLNPR (NGPTOT,NFLEVG), ZRPRE (NGPTOT,NFLEVG)
REAL(KIND=JPRB) ::    ZRPP  (NGPTOT,NFLEVG), ZALPH (NGPTOT,NFLEVG)
REAL(KIND=JPRB) ::    ZRTGR (NGPTOT,NFLEVG)
REAL(KIND=JPRB) ::    ZT(NGPTOT,NFLEVG), ZVORG(NGPTOT,NFLEVG)
REAL(KIND=JPRB) ::    ZTEMP(NFLEVL,NSPEC2),ZDIV(NFLEVL,NSPEC2),ZVOR(NFLEVL,NSPEC2),&
 &ZLNSP(NSPEC2),ZOROG(NSPEC2), ZOROGG(NGPTOT), ZPRESHX(0:NFLEVG), ZPRDPD(NFLEVL,NSPEC2)
REAL(KIND=JPRB) ::    ZUSHEAR(NFLEVG)
REAL(KIND=JPRB) ::    ZTVERT(NFLEVG)
REAL(KIND=JPRB) ::    ZRAD,ZR,ZAMP,ZLAMBDA,ZLATC,ZLONC
REAL(KIND=JPRB) ::    ZLATCN, ZLATCS, ZLONCN, ZLONCS, ZLONCX, ZA, ZB, ZRADN, &
 & ZRADS, ZRADX, ZRADY, ZP1, ZP2, ZST, ZZ, ZZ1, ZZ2, ZUPP, ZPRS, ZH
REAL(KIND=JPRB) ::    ZH0, ZZR, ZS, ZH02, ZZR2, ZR2, ZAMP2, ZS2, ZRAD2, ZHAT, ZN
REAL(KIND=JPRB) ::    ZRFPLM,ZUU,ZC,ZRFHH

! noise generator
INTEGER(KIND=JPIM) :: im,ia,ic,iran,itop


#include "abor1.intfb.h"
#include "suspgpg.intfb.h"

#include "reespe.intfb.h"
#include "gpxyb.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"

!      -----------------------------------------------------------

!*       1.    INITIALIZE SPECTRAL FIELDS FROM GRIB FILE.
!              ------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSPECG2',0,ZHOOK_HANDLE)
CALL GSTATS(18,0)

!==================================================================
!       Specific initial data for various testcases
!==================================================================

ZU0   = RU00_DYN
ZT0   = RT00_DYN
ZVP00 = RP00_DYN
ZST   = RST0_DYN
ZN    = sqrt(RG*RG/(RCPD*ZT0))
ZRHO0 = ZVP00/(RD*ZT0)

DRAGNORM = 0.0_JPRB

WRITE(UNIT=NULOUT,FMT='('' INITIALISING FOR: '')')
WRITE(UNIT=NULOUT,FMT='('' NTESTCASE    = '',I2)') NTESTCASE 
WRITE(UNIT=NULOUT,FMT='('' N3DINI       = '',I1)') K3DINI

WRITE(UNIT=NULOUT,FMT='('' ZU0       = '',E13.7)') ZU0
WRITE(UNIT=NULOUT,FMT='('' ZST       = '',E13.7)') ZST
WRITE(UNIT=NULOUT,FMT='('' ZT0       = '',E13.7)') ZT0
WRITE(UNIT=NULOUT,FMT='('' ZVP00     = '',E13.7)') ZVP00/100._JPRB
WRITE(UNIT=NULOUT,FMT='('' ZRHO0     = '',E13.7)') ZRHO0
WRITE(UNIT=NULOUT,FMT='('' ZN        = '',E13.7)') ZN

!=========================================================================
!       Prescribe orography
!=========================================================================

IF( NTESTCASE == 0 ) THEN
  ! Williamson mountain, Bell shaped, case 1; Williamson et al. (1992) JCP, 102, 211--224

! mountain position (centre)
  ZLATC =  RPI/6._JPRB
  ZLONC =  1.5_JPRB*RPI
! mountain height
  ZAMP  =  1500.0_JPRB
! mountain radius
  ZR   = RA/6.0_JPRB

  DO JWORD=1,NGPTOT
    ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
     &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
    ZRAD = RA*ACOS(ZZ)
    IF( ZRAD >= ZR ) THEN
      ZOROGG(JWORD) = 0.0_JPRB
    ELSE
      ZOROGG(JWORD) = RG*ZAMP*0.5_JPRB*(1.0_JPRB+COS(RPI*ZRAD/ZR))
    ENDIF
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZR*ZAMP**2

ELSEIF( NTESTCASE == 1 ) THEN
  ! Williamson mountain, Gaussian, case 5; Williamson et al. (1992) JCP, 102, 211--224

  ! mountain position (centre)
  ZLATC =  RPI/6._JPRB
  ZLONC =  1.5_JPRB*RPI
  ! mountain height
  ZAMP  =  1500.0_JPRB
  ! mountain radius^2
  ZR = (RPI/9.0_JPRB)**2

  DO JWORD=1,NGPTOT
    ZRAD = MIN(ZR,(GELAM(JWORD)-ZLONC)**2&
     &+(GELAT(JWORD)-ZLATC)**2)
    ZOROGG(JWORD) = RG*ZAMP*(1.0_JPRB-sqrt(ZRAD/ZR))
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZR*ZAMP**2

ELSEIF( NTESTCASE == 2 ) THEN
  ! Klemp-Schaer mountain profile; Klemp, Skamarock, Fuhrer (2003) MWR, 131, 1229-1239
  ! set ZU0 = 20.0, RA = 30000.

  ! mountain position (centre)
  ZLATC =  RPI/6._JPRB
  ZLONC =  1.5_JPRB*RPI
  ! mountain height
  ZAMP  =  250.0_JPRB
  ! mountain radius
  ZR   = 5000._JPRB
  ! off centering parameter
  ZLAMBDA=4000._JPRB

  DO JWORD=1,NGPTOT
    ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
     &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
    ZRAD = RA*ACOS(ZZ)
    ZOROGG(JWORD) = RG*ZAMP*EXP(-ZRAD**2/ZR**2)*(COS(RPI*ZRAD/ZLAMBDA))**2    
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZR*ZAMP**2

ELSEIF( NTESTCASE == 3 ) THEN
  ! low froude number flow, RA = RA/100.

  ! mountain position (centre)
  ZLATC =  RPI/6._JPRB
  ZLONC =  1.5_JPRB*RPI
  ! mountain height
  ZAMP = 5000.0_JPRB
  ! mountain radius
  ZR   = RA/6.0_JPRB

  ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
   &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
  ZRAD = RA*ACOS(ZZ)
  IF( ZRAD >= ZR ) THEN
    ZOROGG(JWORD) = 0.0_JPRB
  ELSE
    ZOROGG(JWORD) = RG*ZAMP*(COS(0.5_JPRB*RPI*ZRAD/ZR))**2
  ENDIF

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZR*ZAMP**2
  
ELSEIF( NTESTCASE == 4 .OR. NTESTCASE == 6 ) THEN
  ! elliptic mountain (quasi 2D)
  ! "Witch of Angesi"
  ! profile and wind shear following T. Keller (1994) JAS, 51, 1915--1929

  ! Ri = 16     == N^2/(ZST)^2
  ! set ZU0 = 10.0
  ! RA = 128000./(2*pi),  dx - 250 m
  ! use IFS L91
  ! N  = 0.01  - T0 == 957 K
  ! c  = 2.5E-04
  ! ZST = ZU0 * c
 
  ! mountain height
  ZAMP = 500.0_JPRB

  ! 2 foci points N/S from equator, determine the width of the mountain
  ZLATCN =  RPI/3._JPRB
  ZLONCN =  1.5_JPRB*RPI
  ZLATCS =  -RPI/3._JPRB
  ZLONCS =  1.5_JPRB*RPI
  ! centre of ellipse at equator
  ZLATC =  0._JPRB
  ZLONC =  1.5_JPRB*RPI
  ! long distance of ellipse, x-direction
  ! ZA    = 0.25_JPRB*RPI*RA
  ZA = 2500.0_JPRB
  ! ZR    = 2._JPRB*ZA

  ! distance of a foci point on a great circle to the centre
  ZZ    = SIN(ZLATCN)*SIN(ZLATC)+&
   &COS(ZLATCN)*COS(ZLATC)*COS(ZLONC-ZLONCN)
  ZC    = RA*ACOS(ZZ)
  ZB    = sqrt(abs(ZA**2 - ZC**2))

  DO JWORD=1,NGPTOT
    ! distance of a point on a great circle from foci point 
    ZZ   = SIN(ZLATCN)*SIN(GELAT(JWORD))+&
     &COS(ZLATCN)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONCN)
    ZRADN = RA*ACOS(ZZ)
    ZZ   = SIN(ZLATCS)*SIN(GELAT(JWORD))+&
     &COS(ZLATCS)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONCS)
    ZRADS = RA*ACOS(ZZ)

    !    IF( (ZRADN+ZRADS) > ZR ) THEN
    !      ZOROGG(JWORD) = 0.0_JPRB
    !    ELSE

    ! projection on x, distance to the centre
    ZZ   = SIN(ZLATC)*SIN(ZLATC)+&
     &COS(ZLATC)*COS(ZLATC)*COS(GELAM(JWORD)-ZLONC)
    ZRADX = RA*ACOS(ZZ)      
    ! projection on y, distance to the centre
    ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
     &COS(ZLATC)*COS(GELAT(JWORD))*COS(ZLONC-ZLONC)
    ZRADY = RA*ACOS(ZZ)

    ! quasi 2D setup for NTESTCASE == 6 (witch of Agnesi)    
    ZOROGG(JWORD) = RG*ZAMP/(sqrt(1._JPRB+(ZRADX/ZA)**2+(ZRADY/ZB)**2))

    !    ENDIF
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZA*ZAMP**2

ELSEIF( NTESTCASE == 5 ) THEN
  ! step mountain (like Himalaya, RA = RA, represents marginally resolved mountain) 
  ZAMP = 5000.0_JPRB

  ! centre of mountain
  ZLATC =  0._JPRB
  ZLONC =  1.5_JPRB*RPI

  ZA    = 0.25_JPRB*RPI*RA
  ZP1   = RA*RPI/9.
  ZP2   = ZP1 + RA*RPI*3._JPRB/180._JPRB

  DO JWORD=1,NGPTOT
    ! projection on x, distance to the centre
    ZZ   = SIN(ZLATC)*SIN(ZLATC)+&
     &COS(ZLATC)*COS(ZLATC)*COS(GELAM(JWORD)-ZLONC)
    ZRADX = RA*ACOS(ZZ)      
    ! projection on y, distance to the centre
    ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
     &COS(ZLATC)*COS(GELAT(JWORD))*COS(ZLONC-ZLONC)
    ZRADY = RA*ACOS(ZZ)
    
    IF( ABS(ZRADX) <= ZP1 .AND.  ABS(ZRADY) <= ZA ) THEN
      ! establish plateau
      ZOROGG(JWORD) = RG*ZAMP
    ELSEIF( ABS(ZRADY) <= ZA .AND. &
     & ( ABS(ZRADX) > ZP1 .AND. ABS(ZRADX) <= ZP2 )) THEN
      ! establish linear slope
      ZOROGG(JWORD) = RG*ZAMP * (ZP2 - ZRADX)/(ZP2 - ZP1)
    ELSE
      ! the rest
      ZOROGG(JWORD) = 0._JPRB
    ENDIF
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZA*ZAMP**2

ELSEIF( NTESTCASE == 7 ) THEN
  ! linear critical flow LS2 following Grubisic and Smolarkiewicz (1997) JAS, 1943--1960
  ! ZU0 = 10._JPRB
  ! N  = 0.01 -> T0 = 957.
  ! ZU0/(Na) = 0.2
  ! a  = 5000.
  ! U(z) = ZU0*(1 - z/z_c)
  ! Ri = 1 = (N*z_c/ZU0)**2
  ! h_hut = 0.05 / 0.3 
  ! ZAMP = h_hut*ZU0/N
  ! ZST = ZU0*c = ZU0/z_c
  
  ! position at equator
  ZLATC =  0._JPRB
  ZLONC =  1.5_JPRB*RPI
  
  ! previously tested LS2 linear regime but Ri = 64 ; ZAMP = 83.33_JPRB

  ! LS2
  ZHAT = 0.05_JPRB

  ! mountain height
  ZAMP = ZHAT*ZU0/ZN

  ! half-width
  ZR = 5000._JPRB
  
  DO JWORD=1,NGPTOT
    ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
     &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
    ZRAD = RA*ACOS(ZZ)
    ZOROGG(JWORD) = RG*ZAMP/(sqrt(1._JPRB+(ZRAD/ZR)**2))**3
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZR*ZAMP**2

ELSEIF( NTESTCASE == 8 ) THEN
  ! nonlinear critical flow LS5 following Grubisic and Smolarkiewicz (1997) JAS, 1943--1960

  ! previously LS5 non-linear regime but Ri = 10 , ZAMP = 416.67_JPRB
  ! wind shear profile (in z) was set ZST = 5.625E-03_JPRB

  ! position at equator
  ZLATC =  0._JPRB
  ZLONC =  1.5_JPRB*RPI

  ! LS5
  ZHAT = 0.3_JPRB

  ! mountain height
  ZAMP = ZHAT*ZU0/ZN

  ! half-width
  ZR = 5000._JPRB
  
  DO JWORD=1,NGPTOT
    ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
     &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
    ZRAD = RA*ACOS(ZZ)
    ZOROGG(JWORD) = RG*ZAMP/(sqrt(1._JPRB+(ZRAD/ZR)**2))**3
  ENDDO

  DRAGNORM = 0.25_JPRB*RPI*ZRHO0*ZN*ZU0*ZR*ZAMP**2

ELSE
  ! no orography
  DO JWORD=1,NGPTOT
    ZOROGG(JWORD) = 0.0_JPRB
  ENDDO
  ! constant surface pressure
  ZVP00 = RP00_DYN
  DO JWORD=1,NGPTOT
    ZPSP(JWORD) = LOG ( ZVP00 )
  ENDDO
ENDIF

!=========================================================================
!       Prescribe surface pressure, wind shear, vorticity and NH pressure
!=========================================================================
IF( K3DINI /= 2 ) THEN

  ! set spectral orography
  CALL REESPE(1,1,ZOROG,ZOROGG)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' OROG SET TO ',ZOROG(1),ZOROGG(1),GEMU(1)   
  
  !==========================
  ! prescribe surface pressure
  !==========================
  ZZ1 = 0.5_JPRB*(RA*ROMEGA*2.0_JPRB+ZU0)*ZU0
  ZZ=ZZ1/(RD*ZT0)
  DO JWORD=1,NGPTOT
    ZZ2 = LOG(ZVP00)-ZZ*GEMU(JWORD)**2-ZOROGG(JWORD)/(RD*ZT0)
    ZPRESH(JWORD,NFLEVG) = EXP( ZZ2 )
  ENDDO
  
  CALL GPPREH (NGPTOT,1,NGPTOT,NFLEVG,VAH,VBH,ZPRESH)
  CALL GPXYB (NGPTOT,1,NGPTOT,NFLEVG,VDELB,VC,ZPRESH,&
   &ZDELP,ZRDELP,ZLNPR,ZALPH,ZRTGR,ZRPRE,ZRPP)
  CALL GPPREF (NGPTOT,1,NGPTOT,NFLEVG,ZALPH,ZPRESH,ZPRESF)
 
  DO JWORD=1,NGPTOT
    ZPSP(JWORD) = LOG ( ZPRESH(JWORD,NFLEVG) )
  ENDDO
  ! spectral
  CALL REESPE(1,1,ZLNSP,ZPSP)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' LNSP SET TO ',ZLNSP(1),ZPSP(1),GEMU(1)

  
  !==========================
  ! prescribe zonal wind ZU0, 
  ! and/or linear wind shear, prescribed as ZST = DU0/DZ
  !==========================
  
  ! upper threshold above which U=const (in meter)
  ZUPP = 10500._JPRB
  
  ZPRESHX(0)=VAH(0)+VBH(0)*ZVP00
  DO JLEV=NFLEVG,1,-1
    ZPRESHX(JLEV-1)=VAH(JLEV-1)+VBH(JLEV-1)*ZVP00
    ZPRESHX(JLEV)=VAH(JLEV)+VBH(JLEV)*ZVP00
    ZPRS=0.5_JPRB*(ZPRESHX(JLEV)+ZPRESHX(JLEV-1))
    ZC=-(RD*ZT0/RG)*LOG(ZPRS/ZVP00)
    IF( (ZC >= ZUPP).AND.&
     & ((NTESTCASE==4).OR.(NTESTCASE==6).OR.(NTESTCASE==7).OR.(NTESTCASE==8)) ) THEN
      ZUSHEAR(JLEV) = ZUU
    ELSE
      ZUU = ZU0 - ZST*ZC
      ZUSHEAR(JLEV) = ZUU
    ENDIF
  ENDDO
  
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' ZU0 SET TO ', ZUSHEAR(NFLEVG)

  
  !==========================
  ! zonal flow implies horizontal divergence == 0.
  !==========================
  ! spectral
  DO JWORD=1,NSPEC2
    ZDIV(:,JWORD)=0.0_JPRB
  ENDDO
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' DIVERGENCE SET TO ',ZDIV(1,1),GEMU(1)

  
  !==========================
  ! vorticity for zonal/shear flow
  !==========================
  DO JLEV=NFLEVG,1,-1
    DO JWORD=1,NGPTOT
      ZVORG(JWORD,JLEV) = 2.0_JPRB*ZUSHEAR(JLEV)/RA*GEMU(JWORD)
    ENDDO
  ENDDO

  IF( NOISEVOR == 1 ) THEN
    ! noise generator
    ! add some noise in vorticity
    im=86436
    ia=1093
    ic=18254
    iran=1   
    itop=30
    DO JLEV=1,NFLEVG
      DO JWORD=1,NGPTOT
        iran=mod(iran*ia+ic,im)
        ZVORG(JWORD,JLEV) = ZVORG(JWORD,JLEV) + 1.e-5*float(iran)/float(im)
      ENDDO
    ENDDO
  ENDIF

  ! spectral
  CALL REESPE(NFLEVL,NFLEVG,ZVOR,ZVORG)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' VORTICITY SET TO ',ZVOR(1,1),ZVORG(1,1),GEMU(1)

  !==========================
  ! NH pressure departure
  !==========================

  IF( NTESTCASE == 9 ) THEN
    ! simple acoustic wave (following Landau/Lifschitz + Smolarkiewicz)

    ! position centre of disturbance
    ZLATC =  0._JPRB
    ZLONC =  1.5_JPRB*RPI
    ! radius of disturbance
    ZR   = RA/6.0_JPRB
    ! amplitude disturbance
    ZAMP = 0.082 * ZVP00
    
    ZRFHH=RD*ZT0/RG

    DO JLEV=NFLEVG,1,-1
      DO JWORD=1,NGPTOT
        ZRFPLM=ZRFHH*LOG(ZVP00/ZPRESF(JWORD,JLEV))
        ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
         &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
        ZRAD = RA*ACOS(ZZ)
        IF( ZRAD < ZR ) THEN
          ZH = ZRAD * TAN( ACOS( ZRAD/ZR ) )
          IF( ZRFPLM < ZH ) THEN
            ZT(JWORD,JLEV)= ZAMP
          ELSE
            ZT(JWORD,JLEV)= 0.0_JPRB
          ENDIF
        ELSE
          ZT(JWORD,JLEV)= 0.0_JPRB
        ENDIF
      ENDDO
    ENDDO

    ! spectral
    CALL REESPE(NFLEVL,NFLEVG,ZPRDPD,ZT)
    WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
     &' PRESSURE DEPARTURE SET TO ',ZPRDPD(NFLEVL,1),ZT(1,NFLEVG),GEMU(1)
    
  ELSE
    ! spectral
    DO JWORD=1,NSPEC2
      ZPRDPD(:,JWORD)=0.0_JPRB
    ENDDO
  ENDIF
  
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' PRESSURE DEPARTURE SET TO ',ZPRDPD(NFLEVL,1),GEMU(1)  

  !===========================
  ! initialize other variables
  !===========================

  ! 2d variables  
  I2D = 1
  IGRIB2D(I2D) = NGRBLNSP
  I2D = I2D+1
  IGRIB2D(I2D) = NGRBZ
  LDSPOR = .TRUE.
  ! GMV 
  I3D = 2+NFTHER
  IGRIB3D(1:I3D) = NGRBSP3(1:I3D)

ENDIF

!============================================================

IF( K3DINI == 2 ) THEN

  IF(PRESENT(LDINOR)) THEN
    LLINOR = LDINOR
  ELSE
    LLINOR = .FALSE.
  ENDIF
  
  IF(PRESENT(KFILE)) THEN
    IF(PRESENT(CDPATH)) CALL ABOR1('SUSPECG2:BOTH KFILE AND CDPATH PRESENT')
    IFILE = KFILE
  ELSE
    IF(PRESENT(CDPATH)) THEN
      CLFN = CDPATH
      IFILE = -99
    ELSE
      CALL ABOR1('SUSPECG2:NEITHER KFILE OR CDPATH PRESENT')
    ENDIF
  ENDIF
  
  LLINCR  = (IFILE == 6.OR.IFILE == 7) ! Lower resolution spectral input
  LLSPINT = LSPINT.AND.LLINCR          ! Gridpoint fields from L.R. spec. input
  
  WRITE(UNIT=NULOUT,FMT='(A)')  ' READ SPECTRAL DATA FROM GRIB '
  IF(LLINCR) WRITE(NULOUT,'(A)')' INPUT DATA AT DIFFERENT RESOLUTION'
  
  IF (IFILE == 0) THEN
    CLFN=CFNISH
    WRITE(UNIT=NULOUT,FMT='(A,A)')' INITIAL DATA TO BE READ FROM FILE ',CLFN
  ELSEIF (IFILE == -99) THEN
    WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' DATA TO BE READ FROM FILE ',CLFN  
  ELSE
    WRITE(NULERR,'('' INVALID ARGUMENT TO SUSPECG2, KFILE='',I3)')KFILE
    CALL ABOR1(' INVALID ARGUMENT TO SUSPECG2 ')
  ENDIF
  
  !   2d variables  
  I2D = 0
  
  !     surface pressure
  I2D = I2D+1
  IGRIB2D(I2D) = NGRBLNSP
  !     orography
  IF(PRESENT(LDSPOR)) LDSPOR = .FALSE.
  IF( LLINOR )THEN
    IF(PRESENT(LDSPOR))  LDSPOR = .TRUE.
    I2D = I2D+1
    IGRIB2D(I2D) = NGRBZ
  ENDIF
  
  !   3d variables
  ! GMV 
  I3D = 2+NFTHER
  IGRIB3D(1:I3D) = NGRBSP3(1:I3D)
  !GFL
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LSP) THEN
      IF(YGFLC(JGFL)%NREQIN == 1) THEN
        IGRIB3D(I3D+YGFLC(JGFL)%MPSP) = YGFLC(JGFL)%IGRBCODE
      ELSE
        IGRIB3D(I3D+YGFLC(JGFL)%MPSP) = -99
      ENDIF
    ENDIF
  ENDDO
  I3D = I3D+YGFL%NUMSPFLDS
  !GFL gridpoint fields possibly initialized from spectral
  ISPTOGP=0
  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LGP) THEN
      IF(YGFLC(JGFL)%NREQIN == 1) THEN
        IF(.NOT.YGFLC(JGFL)%LGPINGP .OR. LLSPINT) THEN
          ISPTOGP=ISPTOGP+1
          IGRIB3D(I3D+ISPTOGP) = YGFLC(JGFL)%IGRBCODE
        ENDIF
      ENDIF
    ENDIF
  ENDDO
  
  IF(ISPTOGP == 0) THEN
    ZSPA3 => PSPA3
  ELSE
    ALLOCATE(ZSPA3(NFLEVL,NSPEC2,I3D+ISPTOGP))
  ENDIF
  
  ! initialisation from files
  
  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r',KIOMASTER=1)
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'SPECTRAL_FIELDS',LDGRIB=.TRUE.,&
   & KGRIB3D=IGRIB3D(1:I3D+ISPTOGP),KGRIB2D=IGRIB2D(1:I2D),&
   & LDINTERP=LLINCR)
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=PSPA2(:,1:I2D),PR3=ZSPA3,LDMISSING_3D=LLMISSING_3D)
  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)

!!! To overwrite orog and lnsp needs a full-pos step !!!

ELSEIF( K3DINI == 3 ) THEN

  !==================================================================
  ! prescribe idealized temperature
  !==================================================================

  ! set ZVP00 = 100800.0_JPRB
  ! set ZU0   = 0.0_JPRB
  ! no flow (but set noise in vorticity)

  ! GATE observations
  IF( NFLEVG == 91 ) THEN

    ZTVERT(1)=270.0999755859
    ZTVERT(2)=269.0074768066
    ZTVERT(3)=267.5289916992
    ZTVERT(4)=265.0789184570
    ZTVERT(5)=261.2294311523
    ZTVERT(6)=255.4538726807
    ZTVERT(7)=247.1291503906
    ZTVERT(8)=235.5435333252
    ZTVERT(9)=219.9087219238
    ZTVERT(10)=215.1292724609
    ZTVERT(11)=214.1158599854
    ZTVERT(12)=212.8441619873
    ZTVERT(13)=211.2785797119
    ZTVERT(14)=209.3839721680
    ZTVERT(15)=207.1263122559
    ZTVERT(16)=204.4731750488
    ZTVERT(17)=201.3942718506
    ZTVERT(18)=198.1799926758
    ZTVERT(19)=198.1799926758
    ZTVERT(20)=198.1799926758
    ZTVERT(21)=198.1799926758
    ZTVERT(22)=198.1799926758
    ZTVERT(23)=198.1799926758
    ZTVERT(24)=198.1799926758
    ZTVERT(25)=198.1799926758
    ZTVERT(26)=198.1799926758
    ZTVERT(27)=198.1799926758
    ZTVERT(28)=198.1799926758
    ZTVERT(29)=198.1799926758
    ZTVERT(30)=198.1799926758
    ZTVERT(31)=198.1799926758
    ZTVERT(32)=198.1799926758
    ZTVERT(33)=198.1799926758
    ZTVERT(34)=198.1799926758
    ZTVERT(35)=198.1799926758
    ZTVERT(36)=198.1799926758
    ZTVERT(37)=198.1799926758
    ZTVERT(38)=197.9096984863
    ZTVERT(39)=197.2681884766
    ZTVERT(40)=196.5949249268
    ZTVERT(41)=197.5838775635
    ZTVERT(42)=199.0915679932
    ZTVERT(43)=200.7485656738
    ZTVERT(44)=203.3138122559
    ZTVERT(45)=206.0377349854
    ZTVERT(46)=208.8321685791
    ZTVERT(47)=211.5966339111
    ZTVERT(48)=214.5286407471
    ZTVERT(49)=217.5512542725
    ZTVERT(50)=220.6950378418
    ZTVERT(51)=224.0257720947
    ZTVERT(52)=227.2509613037
    ZTVERT(53)=230.6190795898
    ZTVERT(54)=234.1008300781
    ZTVERT(55)=237.3729095459
    ZTVERT(56)=240.8337097168
    ZTVERT(57)=244.1077575684
    ZTVERT(58)=247.1923217773
    ZTVERT(59)=250.4510498047
    ZTVERT(60)=253.6593322754
    ZTVERT(61)=257.0246276855
    ZTVERT(62)=260.2504272461
    ZTVERT(63)=263.1702575684
    ZTVERT(64)=266.1941833496
    ZTVERT(65)=268.4446411133
    ZTVERT(66)=270.7293090820
    ZTVERT(67)=272.9362182617
    ZTVERT(68)=275.0777587891
    ZTVERT(69)=277.2100219727
    ZTVERT(70)=279.3574218750
    ZTVERT(71)=281.4693298340
    ZTVERT(72)=283.3341674805
    ZTVERT(73)=285.0358581543
    ZTVERT(74)=286.6547546387
    ZTVERT(75)=287.8569030762
    ZTVERT(76)=289.0057983398
    ZTVERT(77)=289.9935302734
    ZTVERT(78)=290.7793273926
    ZTVERT(79)=291.5139770508
    ZTVERT(80)=292.2810058594
    ZTVERT(81)=293.0487976074
    ZTVERT(82)=293.7450256348
    ZTVERT(83)=294.4486999512
    ZTVERT(84)=295.2596740723
    ZTVERT(85)=295.9648132324
    ZTVERT(86)=296.5662536621
    ZTVERT(87)=297.0814208984
    ZTVERT(88)=297.5199890137
    ZTVERT(89)=297.8745727539
    ZTVERT(90)=298.1445922852
    ZTVERT(91)=298.3394165039
    
  ELSEIF( NFLEVG == 60 ) THEN

    ZTVERT(1)=265.1600036621
    ZTVERT(2)=254.6143798828
    ZTVERT(3)=242.6354827881
    ZTVERT(4)=226.9340820312
    ZTVERT(5)=215.4417877197
    ZTVERT(6)=214.5443267822
    ZTVERT(7)=213.4847106934
    ZTVERT(8)=212.2389831543
    ZTVERT(9)=210.7684326172
    ZTVERT(10)=209.0133666992
    ZTVERT(11)=206.8826446533
    ZTVERT(12)=204.2561950684
    ZTVERT(13)=201.0021057129
    ZTVERT(14)=198.1799926758
    ZTVERT(15)=198.1799926758
    ZTVERT(16)=198.1799926758
    ZTVERT(17)=198.1799926758
    ZTVERT(18)=198.1799926758
    ZTVERT(19)=198.1799926758
    ZTVERT(20)=198.1799926758
    ZTVERT(21)=198.1799926758
    ZTVERT(22)=198.1799926758
    ZTVERT(23)=198.1799926758
    ZTVERT(24)=198.1799926758
    ZTVERT(25)=197.8644104004
    ZTVERT(26)=196.8443145752
    ZTVERT(27)=200.4296417236
    ZTVERT(28)=206.2930450439
    ZTVERT(29)=212.2812347412
    ZTVERT(30)=218.4390106201
    ZTVERT(31)=224.8248748779
    ZTVERT(32)=230.9979095459
    ZTVERT(33)=237.0218811035
    ZTVERT(34)=242.9370574951
    ZTVERT(35)=247.9637908936
    ZTVERT(36)=253.0226898193
    ZTVERT(37)=258.0513610840
    ZTVERT(38)=262.3324279785
    ZTVERT(39)=266.3991394043
    ZTVERT(40)=269.4440612793
    ZTVERT(41)=272.4480895996
    ZTVERT(42)=275.2995300293
    ZTVERT(43)=278.1466979980
    ZTVERT(44)=280.9732666016
    ZTVERT(45)=283.4899902344
    ZTVERT(46)=285.7284851074
    ZTVERT(47)=287.5502319336
    ZTVERT(48)=289.0760192871
    ZTVERT(49)=290.3008117676
    ZTVERT(50)=291.3007507324
    ZTVERT(51)=292.2953186035
    ZTVERT(52)=293.2876892090
    ZTVERT(53)=294.1549072266
    ZTVERT(54)=295.2213745117
    ZTVERT(55)=296.1265869141
    ZTVERT(56)=296.8565368652
    ZTVERT(57)=297.4228820801
    ZTVERT(58)=297.8425598145
    ZTVERT(59)=298.1385803223
    ZTVERT(60)=298.3394470215
    
  ELSE
    WRITE(NULERR,'('' INVALID NUMBER OF LEVELS IN SUSPECG2'')')
    CALL ABOR1(' INVALID LEVELS IN SUSPECG2 ')
  ENDIF
  
  ! temperature, provide GATE SOUNDING HERE
  DO JLEV=1,NFLEVG
    DO JWORD=1,NGPTOT
      ZT(JWORD,JLEV)= ZTVERT(JLEV)
    ENDDO
  ENDDO
  ! spectral temperature
  CALL REESPE(NFLEVL,NFLEVG,ZTEMP,ZT)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' TEMPERATURE SET TO ',ZTEMP(1,1),ZT(1,1),GEMU(1)

ELSEIF( K3DINI == 4 ) THEN  
  
  !==================================================================
  ! prescribe idealized temperature, neutral stratification
  !==================================================================
  ! (typically, RT00_DYN==300., RU00_DYN=0.)
  ! there may be issues with RU00_DYN /= 0 and the vertical discretization !!!

  IF( NTESTCASE == 11 ) THEN
    ! two-bubbles test following Robert (1993) JAS, 1865--1873
    ! prescribe potential temperature perturbation and then translate to T

    ZLATC =  0._JPRB
    ZLONC =  1.5_JPRB*RPI
    ! six gridpoint offset
    ZLONCX = 2._JPRB*RPI*6._JPRB/(2.*NSMAX+1)
    ! first bubble
    ZH0   = 15000._JPRB
    ZZR   =  4000._JPRB
    ZR    = 10000._JPRB
    ZAMP  = -0.5_JPRB
    ZS    = 0.33333_JPRB
    ! second bubble
    ZH02   = 6000._JPRB
    ZZR2   =  600._JPRB
    ZR2    =  600._JPRB
    ZAMP2  = 0.15_JPRB
    ZS2    = 0.33333_JPRB
    DO JWORD=1,NGPTOT
      ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
       &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
      ZZ2  = SIN(ZLATC)*SIN(GELAT(JWORD))+&
       &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-(ZLONC+ZLONCX))
      DO JLEV=1,NFLEVG
        ZC=-(RD*ZT0/RG)*LOG(ZPRESF(JWORD,JLEV)/ZPRESH(JWORD,NFLEVG))
        ZRAD = sqrt( (RA*ACOS(ZZ)/ZR)**2 + ((ZC-ZH0)/ZZR)**2 )
        IF( ZRAD <= 1._JPRB ) THEN
          ZOROGG(JWORD) = ZT0 + ZAMP
        ELSE
          ZOROGG(JWORD) = ZT0 + ZAMP*EXP(-ZRAD**2/ZS**2)
        ENDIF
        ! second bubble
        ZRAD2 = sqrt( (RA*ACOS(ZZ2)/ZR2)**2 + ((ZC-ZH02)/ZZR2)**2 )
        IF( ZRAD2 <= 1._JPRB ) THEN
          ZOROGG(JWORD) = ZOROGG(JWORD)  + ZAMP2
        ELSE
          ZOROGG(JWORD) = ZOROGG(JWORD)  + ZAMP2*EXP(-ZRAD2**2/ZS2**2)
        ENDIF
        ZT(JWORD,JLEV) = ZOROGG(JWORD)*(ZPRESF(JWORD,JLEV)/ZVP00)**(RD/RCPD)
        ! ?? issue with LGWADV=F NH code ??
        ZT(JWORD,JLEV) = MAX( 50._JPRB, ZT(JWORD,JLEV))
      ENDDO
    ENDDO

    CALL REESPE(NFLEVL,NFLEVG,ZTEMP,ZT)
    WRITE (0,*) ' ROUTINE SUSPECG, TDYNAMICAL CORE' ,&
     &' TEMPERATURE SET TO ',ZTEMP(1,1),ZT(1,1),GEMU(1)

  ELSEIF( NTESTCASE == 12 ) THEN
    ! single rising bubble, Robert (1993) Fig.3-6, no analytic solution !!!
    ! bubble perturbation in potential temperature

    ! position at equator
    ZLATC =  0._JPRB
    ZLONC =  1.5_JPRB*RPI

    ! bubble
    ! initial height
    ZH0   =  1000._JPRB
    ! vertical dimension
    ZZR   =  4000._JPRB
    ! horizontal dimension
    ZR    = 10000._JPRB

    ZAMP  = +0.5_JPRB
    ZS    = 2._JPRB

    DO JWORD=1,NGPTOT
      ZZ   = SIN(ZLATC)*SIN(GELAT(JWORD))+&
       &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-ZLONC)
      ZZ2  = SIN(ZLATC)*SIN(GELAT(JWORD))+&
       &COS(ZLATC)*COS(GELAT(JWORD))*COS(GELAM(JWORD)-(ZLONC+ZLONCX))
      DO JLEV=1,NFLEVG
        ZC=-(RD*ZT0/RG)*LOG(ZPRESF(JWORD,JLEV)/ZPRESH(JWORD,NFLEVG))
        ZRAD = sqrt( (RA*ACOS(ZZ)/ZR)**2 + ((ZC-ZH0)/ZZR)**2 )
        IF( ZRAD <= 1._JPRB ) THEN
          ZOROGG(JWORD) = ZT0 + ZAMP
        ELSE
          ZOROGG(JWORD) = ZT0 + ZAMP*EXP(-(ZRAD-1.)**2/ZS**2)
        ENDIF
        ! temperature
        ZT(JWORD,JLEV) = ZOROGG(JWORD)*(ZPRESF(JWORD,JLEV)/ZVP00)**(RD/RCPD)
        ! ?? issue with LGWADV=F NH code ??
        ZT(JWORD,JLEV) = MAX( 50._JPRB, ZT(JWORD,JLEV))
      ENDDO
    ENDDO

    CALL REESPE(NFLEVL,NFLEVG,ZTEMP,ZT)
    WRITE (0,*) ' ROUTINE SUSPECG, TDYNAMICAL CORE' ,&
     &' TEMPERATURE SET TO ',ZTEMP(1,1),ZT(1,1),GEMU(1)
    
  ENDIF

ELSEIF( K3DINI == 5 ) THEN  

  !==================================================================
  ! prescribe idealized temperature, isothermal stratification
  !==================================================================
  ! (typically, RT00_DYN==288.15., RU00_DYN=0./10./20.)

  ! temperature
  ! most consistent zonal flow solution is given by
  ! purely isothermal atmosphere, accepting a small contribution
  ! in v-velocity and a non-zero divergence but very small ~1e-12
  ! due to the inconsistency with respect to the given eta levels
  ! and the top of the model p=0

  ! upper threshold above which U=const (in meter) and ZT0 changes
  ZUPP = 10500._JPRB

  ! troposphere - stratosphere model
  IF( NTESTCASE==6 ) THEN
    ZN = 2._JPRB*sqrt(RG*RG/(RCPD*ZT0))
    ZT02 = RG*RG/(RCPD*ZN*ZN)

    ZPRESHX(0)=VAH(0)+VBH(0)*ZVP00
    DO JLEV=NFLEVG,1,-1
      ZPRESHX(JLEV-1)=VAH(JLEV-1)+VBH(JLEV-1)*ZVP00
      ZPRESHX(JLEV)=VAH(JLEV)+VBH(JLEV)*ZVP00
      ZPRS=0.5_JPRB*(ZPRESHX(JLEV)+ZPRESHX(JLEV-1))
      ZC=-(RD*ZT0/RG)*LOG(ZPRS/ZVP00)
      IF( ZC < ZUPP ) THEN
        ZTVERT(JLEV) = ZT0
      ELSE
        ZTVERT(JLEV) = ZT02+(ZT0-ZT02)*(1.0_JPRB-TANH((ZC-ZUPP)/3000._JPRB));
      ENDIF
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      ZTVERT(JLEV) = ZT0
    ENDDO
  ENDIF
  
  DO JLEV=1,NFLEVG
    DO JWORD=1,NGPTOT
      ZT(JWORD,JLEV)= ZTVERT(JLEV)
    ENDDO
  ENDDO

  CALL REESPE(NFLEVL,NFLEVG,ZTEMP,ZT)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' TEMPERATURE SET TO ',ZTEMP(1,1),ZT(1,1),GEMU(1)

ELSEIF( K3DINI == 6 ) THEN  

  !==================================================================
  ! prescribe idealized temperature, linear decrease of T with height
  ! assuming scale height 7 km, set for sea breeze experiment
  !==================================================================
  ! (typically, RT00_DYN==288.15., RU00_DYN=0.)

  ! upper threshold above which ZT0 == constant
  ZUPP = 10500._JPRB

  ZN = 1._JPRB/1000._JPRB

  ZPRESHX(0)=VAH(0)+VBH(0)*ZVP00
  DO JLEV=NFLEVG,1,-1
    ZPRESHX(JLEV-1)=VAH(JLEV-1)+VBH(JLEV-1)*ZVP00
    ZPRESHX(JLEV)=VAH(JLEV)+VBH(JLEV)*ZVP00
    ZPRS=0.5_JPRB*(ZPRESHX(JLEV)+ZPRESHX(JLEV-1))
    ZC=-7000.*LOG(ZPRS/ZVP00)
    IF( ZC < ZUPP ) THEN
      ZTVERT(JLEV) = ZT0 - ZN*ZC
      ZT02=ZTVERT(JLEV)
    ELSE
      ZTVERT(JLEV) = ZT02
    ENDIF
  ENDDO
  
  DO JLEV=1,NFLEVG
    DO JWORD=1,NGPTOT
      ZT(JWORD,JLEV)= ZTVERT(JLEV)
    ENDDO
  ENDDO

  CALL REESPE(NFLEVL,NFLEVG,ZTEMP,ZT)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' TEMPERATURE SET TO ',ZTEMP(1,1),ZT(1,1),GEMU(1)

ELSEIF ( K3DINI == 7 ) THEN

  !==================================================================
  !  HS initial temperature prescribed as if at the pole
  !==================================================================

  ZPRESHX(0)=VAH(0)+VBH(0)*ZVP00
  DO JLEV=NFLEVG,1,-1
    ZPRESHX(JLEV-1)=VAH(JLEV-1)+VBH(JLEV-1)*ZVP00
    ZPRESHX(JLEV)=VAH(JLEV)+VBH(JLEV)*ZVP00
    ZPRS=0.5_JPRB*(ZPRESHX(JLEV)+ZPRESHX(JLEV-1))*RPRESSURE_SCALE
    ZTVERT(JLEV) = MAX ( 200._JPRB,&
     &(315._JPRB- RDELTA_T)* ZPRS**RHS_KAPPA )
  ENDDO

  DO JLEV=1,NFLEVG
    DO JWORD=1,NGPTOT
      ZT(JWORD,JLEV)= ZTVERT(JLEV)
    ENDDO
  ENDDO

  CALL REESPE(NFLEVL,NFLEVG,ZTEMP,ZT)
  WRITE (0,*) ' ROUTINE SUSPECG2, DYNAMICAL CORE' ,&
   &' TEMPERATURE SET TO ',ZTEMP(1,1),ZT(1,1),GEMU(1)

ELSE
  WRITE(NULERR,'('' INVALID SETUP N3DINI IN SUSPECG2'')')
  CALL ABOR1(' INVALID SETUP N3DINI IN SUSPECG2 ')
ENDIF

!==================================================================
! always initialize from computed values above
!==================================================================

DO JGFL=1,I2D
  IF( IGRIB2D(JGFL) == NGRBLNSP ) THEN
    PSPA2(:,JGFL) = ZLNSP(:)
  ENDIF
    
  IF( IGRIB2D(JGFL) == NGRBZ ) THEN
    PSPA2(:,JGFL) = ZOROG(:)
  ENDIF
ENDDO

!==================================================================
! if not from file, initialize also the following from above
!==================================================================

IF( K3DINI /= 2 ) THEN

  DO JGFL=1,I3D
    DO JK=1,NFLEVL
      IF ( IGRIB3D(JGFL) == NGRBVO) THEN 
        PSPA3(JK,:,JGFL) = ZVOR(JK,:)
      ENDIF
      IF ( IGRIB3D(JGFL) == NGRBD) THEN
        PSPA3(JK,:,JGFL) = ZDIV(JK,:)
      ENDIF
      IF ( IGRIB3D(JGFL) == NGRBT) THEN
        PSPA3(JK,:,JGFL) = ZTEMP(JK,:)
      ENDIF
      IF ( IGRIB3D(JGFL) == NGRB118) THEN
        PSPA3(JK,:,JGFL) = ZPRDPD(JK,:)
      ENDIF
      IF ( IGRIB3D(JGFL) == NGRB119) THEN
        PSPA3(JK,:,JGFL) = 0._JPRB
      ENDIF
    ENDDO
  ENDDO

ELSE

  IF(ISPTOGP > 0) THEN
    PSPA3(:,:,:) = ZSPA3(:,:,1:I3D)
    ISPTOGP = 0
    DO JGFL=1,YGFL%NUMFLDS
      IF(YGFLC(JGFL)%LGP) THEN
        IF(YGFLC(JGFL)%NREQIN == 1) THEN
          IF(.NOT.YGFLC(JGFL)%LGPINGP .OR. LLSPINT) THEN
            ISPTOGP=ISPTOGP+1
            CALL SUSPGPG(ZSPA3(:,:,I3D+ISPTOGP),YGFLC(JGFL)%IGRBCODE)
          ENDIF
        ENDIF
      ENDIF
    ENDDO
    DEALLOCATE(ZSPA3)
  ELSE
    NULLIFY(ZSPA3)
  ENDIF
  
ENDIF

CALL GSTATS(18,1)


IF (LHOOK) CALL DR_HOOK('SUSPECG2',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPECG2
