SUBROUTINE SUSPECTCFOU

!**** *SUSPECTCFOU* - READS FOURIER COEFFICIENTS OF TRANSMISSION COEFFICIENTS
!                     ON A LFI FILE. DISTRIBUTED MEMORY CODE.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SUSPECTCFOU(.....)*

!        Explicit arguments :
!        --------------------
!        none

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------
!       Calls some LFI routines.
!       Calls DISFOU.
!       Calls FOUTCNORM.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP). 

!     Modifications.
!     --------------
!        Original : FEB 1999
!        Modified 99-07-21 by K. YESSAD: replace the Curtis matrix by a
!         horizontally constant field to be consistent with the new
!         simplified physics, even if the file read has a horizontally
!         variable one.
!        Modified by G> Radnoti 99/11/07  ALADIN geometry
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LRFOUTCNORM, LELAM
USE YOMDIM   , ONLY : NTCMAX   ,NDGLG    ,NFLEVG   ,NDGUNG   &
 & ,NDGUXG  
USE YOMFOUTC , ONLY : RFORAB3C ,RFORAB4C ,RFORAB6C ,RFORAB3N &
 & ,RFORAB4N ,RFORAB6N ,RFORAT1C ,RFORAT2C ,RFORAT3C &
 & ,RFORAT4C ,RFORAT5C ,RFORAT1N ,RFORAT2N ,RFORAT3N &
 & ,RFORAT4N ,RFORAT5N ,RFOCOR   ,RFOAC  
USE YOMLUN   , ONLY : NTCSR    ,NULOUT
USE YOMMP    , ONLY : MY_REGION_NS ,MYPROC   ,NFRSTLAT ,NLSTLAT ,NGPSET2PE
USE YOMTAG   , ONLY : MTAGDISTFO
USE YOMGEM   , ONLY : RC2P1    ,RC2M1    ,RSTRET
USE YOMLEG   , ONLY : RW       ,RMU

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IDIMG
INTEGER(KIND=JPIM) :: IDIML
INTEGER(KIND=JPIM) :: IERR
INTEGER(KIND=JPIM) :: IFLEVG
INTEGER(KIND=JPIM) :: IFRSTLAT
INTEGER(KIND=JPIM) :: IGLN
INTEGER(KIND=JPIM) :: IGLX
INTEGER(KIND=JPIM) :: IIOPROC
INTEGER(KIND=JPIM) :: ILEN
INTEGER(KIND=JPIM) :: ILENAC
INTEGER(KIND=JPIM) :: ILSTLAT
INTEGER(KIND=JPIM) :: INAR
INTEGER(KIND=JPIM) :: INARFIELDS
INTEGER(KIND=JPIM) :: INARFRAME
INTEGER(KIND=JPIM) :: INARI
INTEGER(KIND=JPIM) :: INDGLG
INTEGER(KIND=JPIM) :: INDGLL
INTEGER(KIND=JPIM) :: INDGUNG
INTEGER(KIND=JPIM) :: INDGUXG
INTEGER(KIND=JPIM) :: INFLEVG
INTEGER(KIND=JPIM) :: INIMES
INTEGER(KIND=JPIM) :: INTCMAX
INTEGER(KIND=JPIM) :: INUMF0
INTEGER(KIND=JPIM) :: INUMF
INTEGER(KIND=JPIM) :: IREP
INTEGER(KIND=JPIM) :: IRP
INTEGER(KIND=JPIM) :: JGLG

LOGICAL :: LLFOUTCNORM
LOGICAL :: LLIMST
LOGICAL :: LLIOPROC

REAL(KIND=JPRB) :: ZDEN
REAL(KIND=JPRB) :: ZFORAB3CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAB4CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAB6CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAB3NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAB4NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAB6NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT1CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT2CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT3CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT4CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT5CG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT1NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT2NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT3NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT4NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFORAT5NG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFOCORG(NFLEVG,0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFOACG((NFLEVG+1)*(NFLEVG+1),0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZFOTC(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1),0:2*NTCMAX,NDGLG)
REAL(KIND=JPRB) :: ZNUM((NFLEVG+1)*(NFLEVG+1))
REAL(KIND=JPRB) :: ZW(NDGLG)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "disfou.intfb.h"
#include "foutcnorm.intfb.h"

! ------------------------------------------------------------------

! 1/ Preliminary initialisations:

IF (LHOOK) CALL DR_HOOK('SUSPECTCFOU',0,ZHOOK_HANDLE)
IRP=0
INARI=0
IF (LELAM) THEN
  INARFRAME=5
ELSE
  INARFRAME=3
ENDIF
INARFIELDS=18
INAR=INARFRAME+INARFIELDS
LLIMST=.FALSE.
INIMES=1
IIOPROC=NGPSET2PE(1,1)
LLIOPROC=(IIOPROC == MYPROC)
IFRSTLAT=MAX(NDGUNG,NFRSTLAT(MY_REGION_NS))
ILSTLAT=MIN(NDGUXG,NLSTLAT(MY_REGION_NS))
INDGLL=ILSTLAT+1-IFRSTLAT
IDIML=(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))*(2*NTCMAX+1)*INDGLL
IDIMG=(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))*(2*NTCMAX+1)*NDGLG
INUMF0=(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))

! ------------------------------------------------------------------

! 2/ Opens file:

IF (LLIOPROC) THEN
  CALL LFIOUV(IRP,NTCSR,.TRUE.,'FOUTC','UNKNOWN',.TRUE.,LLIMST,&
   & INIMES,INAR,INARI)  
ENDIF

! ------------------------------------------------------------------

! 3/ Reads frame and compare to model variables:

IF (LLIOPROC) THEN

  CALL LFIINTLEC(IRP,NTCSR,'NTCMAX',INTCMAX,1)
  CALL LFIINTLEC(IRP,NTCSR,'NDGLG',INDGLG,1)
  CALL LFIINTLEC(IRP,NTCSR,'NFLEVG',INFLEVG,1)
  IF (LELAM) THEN
    CALL LFIINTLEC(IRP,NTCSR,'NDGUNG',INDGUNG,1)
    CALL LFIINTLEC(IRP,NTCSR,'NDGUXG',INDGUXG,1)
  ENDIF

  IERR=0
  IF (INTCMAX /= NTCMAX) THEN
    WRITE (NULOUT,'(A,I4,A,I4,A)') &
     & ' INCONSISTENCY BETWEEN TRUNCATION READ ON FILE (=', &
     & INTCMAX, &
     & ') AND NTCMAX (=',NTCMAX,').'  
    IERR=IERR+1
  ENDIF
  IF (INDGLG /= NDGLG) THEN
    WRITE (NULOUT,'(A,I4,A,I4,A)') &
     & ' INCONSISTENCY BETWEEN NUMBER OF LATITUDES READ ON FILE (=', &
     & INDGLG, &
     & ') AND NDGLG (=',NDGLG,').'  
    IERR=IERR+1
  ENDIF
  IF (INFLEVG /= NFLEVG) THEN
    WRITE (NULOUT,'(A,I4,A,I4,A)') &
     & ' INCONSISTENCY BETWEEN NUMBER OF LAYERS READ ON FILE (=', &
     & INFLEVG, &
     & ') AND NFLEVG (=',NFLEVG,').'  
    IERR=IERR+1
  ENDIF
  IF (LELAM) THEN
    IF (INDGUNG /= NDGUNG) THEN
      WRITE (NULOUT,'(A,I4,A,I4,A)') &
       & ' INCONSISTENCY BETWEEN INDGUNG  (=', &
       & INDGUNG, &
       & ') AND NDGUNG (=',NDGUNG,').'  
      IERR=IERR+1
    ENDIF
    IF (INDGUXG /= NDGUXG) THEN
      WRITE (NULOUT,'(A,I4,A,I4,A)') &
       & ' INCONSISTENCY BETWEEN INDGUXG  (=', &
       & INDGUXG, &
       & ') AND NDGUXG (=',NDGUXG,').'  
      IERR=IERR+1
    ENDIF
  ENDIF
  IF (IERR > 0) THEN
    CALL ABOR1(' SUSPECTCFOU: ABOR1 CALLED')
  ENDIF

ENDIF

! ------------------------------------------------------------------

! 4/ Reads fields:

IF (LLIOPROC) THEN
  IGLN=MAX(1,NDGUNG)
  IGLX=MIN(NDGLG,NDGUXG)
  ILEN=(IGLX-IGLN+1)*(1+2*NTCMAX)*NFLEVG
  ILENAC=(IGLX-IGLN+1)*(1+2*NTCMAX)*(NFLEVG+1)*(NFLEVG+1)
  IREP=0

! * clear-sky parallel/diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAB3C',ZFORAB3CG(1,0,IGLN),ILEN)

! * clear-sky diffuse transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAB4C',ZFORAB4CG(1,0,IGLN),ILEN)

! * clear-sky diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAB6C',ZFORAB6CG(1,0,IGLN),ILEN)

! * cloudy parallel/diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAB3N',ZFORAB3NG(1,0,IGLN),ILEN)

! * cloudy diffuse transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAB4N',ZFORAB4NG(1,0,IGLN),ILEN)

! * cloudy diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAB6N',ZFORAB6NG(1,0,IGLN),ILEN)

! * clear-sky parallel transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAT1C',ZFORAT1CG(1,0,IGLN),ILEN)

! * clear-sky parallel/diffuse transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAT2C',ZFORAT2CG(1,0,IGLN),ILEN)

! * clear-sky parallel/diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAT3C',ZFORAT3CG(1,0,IGLN),ILEN)

! * clear-sky diffuse transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAT4C',ZFORAT4CG(1,0,IGLN),ILEN)

! * clear-sky diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAT5C',ZFORAT5CG(1,0,IGLN),ILEN)

! * cloudy parallel transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAT1N',ZFORAT1NG(1,0,IGLN),ILEN)

! * cloudy parallel/diffuse transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAT2N',ZFORAT2NG(1,0,IGLN),ILEN)

! * cloudy parallel/diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAT3N',ZFORAT3NG(1,0,IGLN),ILEN)

! * cloudy diffuse transmissivity.
  CALL LFILEC(IREP,NTCSR,'RAT4N',ZFORAT4NG(1,0,IGLN),ILEN)

! * cloudy diffuse reflectivity.
  CALL LFILEC(IREP,NTCSR,'RAT5N',ZFORAT5NG(1,0,IGLN),ILEN)

! * correction for cloudiness for thermal radiation.
  CALL LFILEC(IREP,NTCSR,'COR',ZFOCORG(1,0,IGLN),ILEN)

! * Curtis matrix for thermal radiation for clear sky.
  CALL LFILEC(IREP,NTCSR,'AC',ZFOACG(1,0,IGLN),ILENAC)

ENDIF

! ------------------------------------------------------------------

! 4bis/ Particuliar treatment for the Curtis matrix:

! * The file which provides the Curtis matrix can contain a horizontally
!   variable one. Since the new simplified physics now uses a horizontally
!   constant one, one has to compute the horizontal average of this field and
!   reset ZFOACG with this constant field; in practical that means that:
!   - all the non-zero zonal wavenumbers of ZFOACG will be reset to zero.
!   - all the zero zonal wavenumbers of ZFOACG will be reset with a constant
!     equal to the horizontal average.
!   - the weights used to compute the average are consistent with the
!     ones used in the physics (Gaussian weights + mapping factor).

IF (LLIOPROC) THEN

! * reset the non-zero zonal wavenumbers coefficients to zero:
  ZFOACG(1:(NFLEVG+1)*(NFLEVG+1),1:2*NTCMAX,1:NDGLG)=0.0_JPRB

! * computes the average:
  DO JGLG=IGLN,IGLX
    ZW(JGLG)=4._JPRB*RSTRET*RSTRET*RW(JGLG)&
     & /((RC2P1+RMU(JGLG)*RC2M1)*(RC2P1+RMU(JGLG)*RC2M1))  
  ENDDO
  ZNUM(1:(NFLEVG+1)*(NFLEVG+1))=0.0_JPRB
  ZDEN=0._JPRB
  DO JGLG=IGLN,IGLX
    ZNUM(1:(NFLEVG+1)*(NFLEVG+1))=ZNUM(1:(NFLEVG+1)*(NFLEVG+1))&
     & +ZW(JGLG)*ZFOACG(1:(NFLEVG+1)*(NFLEVG+1),0,JGLG)  
    ZDEN=ZDEN+ZW(JGLG)
  ENDDO

! * reset the zero zonal wavenumber coefficient to the average:
  DO JGLG=IGLN,IGLX
    ZFOACG(1:(NFLEVG+1)*(NFLEVG+1),0,JGLG)=ZNUM(1:(NFLEVG+1)*(NFLEVG+1))/ZDEN
  ENDDO

ENDIF

! ------------------------------------------------------------------

! 5/ Memory transfer before processor communications:

IF (LLIOPROC) THEN

  INUMF=0
  ZFOTC=0.0_JPRB
! * clear-sky parallel/diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAB3CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAB4CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAB6CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAB3NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAB4NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAB6NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky parallel transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT1CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT2CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT3CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT4CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT5CG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy parallel transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT1NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT2NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT3NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT4NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFORAT5NG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * correction for cloudiness for thermal radiation.
  ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFOCORG(1:NFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+NFLEVG

! * Curtis matrix for thermal radiation for clear sky.
  IFLEVG=(NFLEVG+1)*(NFLEVG+1)
  ZFOTC(INUMF+1:INUMF+IFLEVG,0:2*NTCMAX,IGLN:IGLX)= &
   & ZFOACG(1:IFLEVG,0:2*NTCMAX,IGLN:IGLX)  
  INUMF=INUMF+(NFLEVG+1)*(NFLEVG+1)

ENDIF

! ------------------------------------------------------------------

! 6/ Processor communications:

! * Proc IIOPROC sends data towards the other procs.
IF (LLIOPROC) THEN
  CALL DISFOU(NTCMAX,INDGLL,NDGLG,NDGLG,INUMF0,0,&
   & MTAGDISTFO,0,IIOPROC,IDIMG,ZFOTC)  
ENDIF

! * All procs other than IIOPROC receive data from IIOPROC.
IF (.NOT.LLIOPROC) THEN
  CALL DISFOU(NTCMAX,INDGLL,NDGLG,NDGLG,INUMF0,1,&
   & MTAGDISTFO,0,IIOPROC,IDIML,ZFOTC)  
ENDIF

! ------------------------------------------------------------------

! 7/ Memory transfer after processor communications:

INUMF=0

! * clear-sky parallel/diffuse reflectivity.
RFORAB3C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
RFORAB4C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
RFORAB6C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
RFORAB3N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
RFORAB4N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
RFORAB6N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky parallel transmissivity.
RFORAT1C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse transmissivity.
RFORAT2C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse reflectivity.
RFORAT3C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
RFORAT4C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
RFORAT5C(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy parallel transmissivity.
RFORAT1N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse transmissivity.
RFORAT2N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
RFORAT3N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
RFORAT4N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
RFORAT5N(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * correction for cloudiness for thermal radiation.
RFOCOR(1:NFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+NFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+NFLEVG

! * Curtis matrix for thermal radiation for clear sky.
IFLEVG=(NFLEVG+1)*(NFLEVG+1)
RFOAC(1:IFLEVG,0:2*NTCMAX,1:INDGLL)= &
 & ZFOTC(INUMF+1:INUMF+IFLEVG,0:2*NTCMAX,1:INDGLL)  
INUMF=INUMF+(NFLEVG+1)*(NFLEVG+1)

! ------------------------------------------------------------------

! 8/ Closes file:

IF (LLIOPROC) CALL LFIFER(IRP,NTCSR,'UNKNOWN')

! ------------------------------------------------------------------

! 9/ Statistics:

LLFOUTCNORM=LRFOUTCNORM(1).OR.LRFOUTCNORM(2).OR. &
 & LRFOUTCNORM(3).OR.LRFOUTCNORM(4)  

IF (LLIOPROC.AND.LLFOUTCNORM) THEN

! * clear-sky parallel/diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAB3C',ZFORAB3CG(1,0,IGLN))

! * clear-sky diffuse transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAB4C',ZFORAB4CG(1,0,IGLN))

! * clear-sky diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAB6C',ZFORAB6CG(1,0,IGLN))

! * cloudy parallel/diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAB3N',ZFORAB3NG(1,0,IGLN))

! * cloudy diffuse transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAB4N',ZFORAB4NG(1,0,IGLN))

! * cloudy diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAB6N',ZFORAB6NG(1,0,IGLN))

! * clear-sky parallel transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT1C',ZFORAT1CG(1,0,IGLN))

! * clear-sky parallel/diffuse transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT2C',ZFORAT2CG(1,0,IGLN))

! * clear-sky parallel/diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT3C',ZFORAT3CG(1,0,IGLN))

! * clear-sky diffuse transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT4C',ZFORAT4CG(1,0,IGLN))

! * clear-sky diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT5C',ZFORAT5CG(1,0,IGLN))

! * cloudy parallel transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT1N',ZFORAT1NG(1,0,IGLN))

! * cloudy parallel/diffuse transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT2N',ZFORAT2NG(1,0,IGLN))

! * cloudy parallel/diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT3N',ZFORAT3NG(1,0,IGLN))

! * cloudy diffuse transmissivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT4N',ZFORAT4NG(1,0,IGLN))

! * cloudy diffuse reflectivity.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'RAT5N',ZFORAT5NG(1,0,IGLN))

! * correction for cloudiness for thermal radiation.
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,NFLEVG,IGLX-IGLN+1,NTCMAX,'COR  ',ZFOCORG(1,0,IGLN))

! * Curtis matrix for thermal radiation for clear sky.
  IFLEVG=(NFLEVG+1)*(NFLEVG+1)
  CALL FOUTCNORM(LRFOUTCNORM,NULOUT,IFLEVG,IGLX-IGLN+1,NTCMAX,'AC   ',ZFOACG(1,0,IGLN))

ENDIF

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSPECTCFOU',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPECTCFOU
