SUBROUTINE SUSPQLIM(PFTDAT,PFQDAT,PSPGP,PDQREF,&
 & PDTREF,PQOLD,KLEVEL,KLEVI,PTMIN)  

!**** *SUSPQLIM*  - Limit gridpoint Q to within 0-1 in relative humidity
!                     after correction above 300 hPa.

!**   Interface.
!     ----------
!        *CALL* *SUSPQLIM(.....)*

!     Externals.  SPEREE - spectral to grid-point transform
!     ----------

!     Modifications.
!     --------------
!        J.N. Thepaut : 96-07-03  original
!        F. Taillefer : 98-03-28  message passing implementation
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of LLPOLE and code under LRPOLE, LLPOLE.
!          - removal of argument LLPOLE in 'CALL SPEREE'.
!        R. El Khatib : 01-08-07 Pruning options
!        K.YESSAD     : 2003-05-16 Allow LSPLIT=T.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NSPEC2   ,NPROMA
USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMGEM   , ONLY : VBH      ,VAH      ,NGPTOT
USE YOMPHY   , ONLY : LNEIGE
USE YOMCT0   , ONLY : NPRGPEW
IMPLICIT NONE

! * DUMMY:
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTDAT(NSPEC2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFQDAT(NSPEC2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPGP(NGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDQREF(NGPTOT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTREF(NGPTOT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQOLD(NSPEC2)
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVEL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVI 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTMIN(NGPTOT) 
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZREET(NGPTOT), ZREEQ(NGPTOT), ZRQOLD(NGPTOT)
REAL(KIND=JPRB) :: ZPRSHO(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZPRSFO(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZFULLP(NPROMA)

INTEGER(KIND=JPIM) :: IIND,IST,IEND,IPROF,JROF,JST

REAL(KIND=JPRB) :: ZDELTA, ZDELTA1, ZDELTA2, ZESLEV, ZPRESS, ZQSAT, ZT, ZTMINI, ZTREF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "gppre.intfb.h"
#include "reespe.intfb.h"
#include "speree.intfb.h"

#include "fcttrm.h"

!     ------------------------------------------------------------------
!*       1.  RE-ORDER AND CONVERT TO GRIDPOINT SPACE.
!            ---------------------------------------
IF (LHOOK) CALL DR_HOOK('SUSPQLIM',0,ZHOOK_HANDLE)
IF(NPRGPEW > 1 ) THEN
  ! NPRGPEW > 1 has to be validated.
  CALL ABOR1 ('SUSPQLIM:(NPRGPEW > 1')
ENDIF

!*      1.1  TRANSFORM T, Q and Qold
CALL SPEREE(1,1,PFTDAT,ZREET)
CALL SPEREE(1,1,PFQDAT,ZREEQ)
IF (KLEVEL  <=  KLEVI) THEN
  CALL SPEREE(1,1,PQOLD,ZRQOLD)
ENDIF

!*     1.2 remplir TMIN
IF (KLEVEL  ==  KLEVI+1) THEN
  PTMIN = ZREET
ELSEIF (KLEVEL  <=  KLEVI) THEN
  PTMIN = MIN(PTMIN, ZREET)
ENDIF
!     ------------------------------------------------------------------
!*       2.  ADD TO UPPER AIR GRIDPOINT WORKFILE.
!            -----------------------------------

DO JST=1,NGPTOT,NPROMA

  IST=JST
  IEND=MIN(IST+NPROMA-1,NGPTOT)
  IPROF=IEND-IST+1

!*       2.1  FIND SURFACE PRESSURE

  DO JROF=IST,IEND
    ZPRSHO(JROF-IST+1,NFLEVG) = EXP(PSPGP(JROF))
  ENDDO

!*       2.2  COMPUTE PRESSURES OF ETA LEVELS

  CALL GPPRE(NPROMA,1,IPROF,NFLEVG,VAH,VBH,ZPRSHO,ZPRSFO)

  ZFULLP(1:IPROF)=ZPRSFO(1:IPROF,KLEVEL)

!*       2.3   TREAT LEVELS SMALLER THAN ILEVI AND LIMIT ALL LEVELS

  DO JROF=1,IPROF
    IIND=JROF+IST-1
    ZT=ZREET(IIND)
    ZPRESS=ZFULLP(JROF)
    IF (LNEIGE) THEN
      ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZT))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF
    ZESLEV=FOEW(ZT,ZDELTA)
    ZQSAT=FOQS(ZESLEV/ZPRESS)

    IF(KLEVEL <= KLEVI) THEN
      ZTMINI = PTMIN(IIND)
      ZTREF = PDTREF(IIND)
      IF (LNEIGE) THEN
        ZDELTA1=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZTMINI))
        ZDELTA2=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZTREF))
      ELSE
        ZDELTA1=0.0_JPRB
        ZDELTA2=0.0_JPRB
      ENDIF

      ZREEQ(IIND)=ZRQOLD(IIND) + FOEW(ZTMINI,ZDELTA1) /&
       & FOEW(ZTREF,ZDELTA2) * PDQREF(IIND)  
    ENDIF

!*            LIMIT VALUES

    ZREEQ(IIND)=MAX(0.0_JPRB,MIN(ZREEQ(IIND),ZQSAT))
  ENDDO

ENDDO

!*       2.4  TRANSFORM BACK Q TO SPECTRAL SPACE.

CALL REESPE(1,1,PFQDAT,ZREEQ)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUSPQLIM',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPQLIM
