SUBROUTINE SUSPSDT

!**** *SUSPSDT*  - Initialize stochastically perturbed parametrization tendency scheme
!                     (SPPT a.k.a. stochastic physics with a spectral pattern generator)
!
!     Purpose.
!     --------
!           Initialize stochastically perturbed parametrization tendency scheme

!**   Interface.
!     ----------
!        *CALL* *SUSPSDT

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        MODULE YOMSPSDT

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M. Leutbecher,  ECMWF

!     Modifications.
!     --------------
!        2009-01-02   *Original*
!        2009-01-03   M Leutbecher:     Refinements
!        2009-11-05   M Leutbecher:     Supersaturation treatment and multi-scale version
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX
USE YOMDYN   , ONLY : TDT
USE YOMGRB   , ONLY : NENSFNB 
USE YOMCST   , ONLY : RA
USE YOMLUN   , ONLY : NULOUT, NULNAM

USE YOMSPSDT , ONLY : LSPSDT, LCLIP_SPEC_SDT, LCLIP_GRID_SDT, &
                     &NSCALES_SDT, NMAXSCALES_SDT, SDEVTOT_SDT, &
                     &LRDPATINIT_SDT, LWRPATTRUN_SDT, CIPATINIT_SDT, COPATTRUN_SDT, &
                     &SDEV_SDT, TAU_SDT, XCLIP_RATIO_SDT, CSPEC_SHAPE_SDT, &
                     &NWAV_TESTSPEC_SDT, NWN_TESTSPEC_SDT, XLCOR_SDT, &
                     &XWC_KAPPA_T, YSPSDT_AR1, YGPSDT, LWRITE_ARP, &
                     &COPATSP_SDT, COPATGP_SDT, &
                     &LQPERTLIMIT2, NQSAT_SDT,  XMEANRED_THICKNESS_SDT, &
                     &LTAPER_BL0, XSIGMATOP, XSIGMABOT, &
                     &XTAPER0, XTAPER1, XTAPER2, XTAPER3, &
                     &LTAPER_ST0, XPRESSTOP_ST0, XPRESSBOT_ST0, &
                     &XTAPER0_ST0, XTAPER1_ST0, XTAPER2_ST0, XTAPER3_ST0, &
                     &NSEED_SDT, LUSESETRAN_SDT, LRESETSEED_SDT, LFIXSPPT
                     
USE SPECTRAL_ARP_MOD,     ONLY : ALLOCATE_ARP, SET_ARP2D, EVOLVE_ARP, SUM_ARPS
USE SPECTRAL_FIELDS_MOD,  ONLY : SPECTRAL_NORM
USE GRIDPOINT_FIELDS_MIX, ONLY : ALLOCATE_GRID

IMPLICIT NONE
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTEGER(KIND=JPIM), DIMENSION(:),  allocatable:: IGRIB
REAL(KIND=JPRB), DIMENSION(:),     allocatable:: zchi
REAL(KIND=JPRB), DIMENSION(:,:),   allocatable:: zsdev2d, zphi2d
INTEGER(KIND=JPIM) :: JN, JSCALE, INWAVE
INTEGER(KIND=JPIM) :: iseed
REAL(KIND=JPRB)    :: ZGAMMAN, ZALPHA, ZPHI,  ZCONSTF0
REAL(KIND=JPRB)    :: ZDSIGMA_M3, ZDPRESS_M3

#include "abor1.intfb.h"
#include "fcttim.h"
#include "namspsdt.h"
#include "posnam.intfb.h"
#include "setran.intfb.h"
#include "read_spec_grib.intfb.h"
#include "write_spec_grib.intfb.h"


IF (LHOOK) CALL DR_HOOK('SUSPSDT',0,ZHOOK_HANDLE)
!     -----------------------------------------------------------------
!*    1. Set default values for spectral stochastic diabatic tendencies
!     -----------------------------------------------------------------
LSPSDT=.FALSE.
LFIXSPPT=.FALSE.
LCLIP_SPEC_SDT=.TRUE.
LCLIP_GRID_SDT=.TRUE.

LWRITE_ARP    =.FALSE.
COPATSP_SDT   ='sppt_pat_sp'
COPATGP_SDT   ='sppt_pat_gp'

LQPERTLIMIT2   =.TRUE.
LUSESETRAN_SDT =.TRUE.
NSEED_SDT= NENSFNB+1000
XMEANRED_THICKNESS_SDT=50.E2_JPRB
NQSAT_SDT = 0
!
!   default setting for pattern
!
LUSESETRAN_SDT =.TRUE.
LRESETSEED_SDT =.FALSE.
NSEED_SDT= NENSFNB+1000
NSCALES_SDT   = 1
CSPEC_SHAPE_SDT='WeaverCourtier'
SDEV_SDT( 1:5)= (/ 0.50_JPRB, 0.25_JPRB, 0.125_JPRB, 0.05_JPRB, 0.025_JPRB /)
!                  6 h,       3d,            30d,           90d,         365d 
TAU_SDT(  1:5)= (/ 2.16E4_JPRB, 2.592E5_JPRB, 2.592E6_JPRB, 7.776E6_JPRB, 3.1536E7_JPRB /)
XLCOR_SDT(1:5)=(/ 500.E3_JPRB, 1000.E3_JPRB, 2000.E3_JPRB, 2000.E3_JPRB, 2000.E3_JPRB /)


XCLIP_RATIO_SDT=2.0_JPRB 

NWAV_TESTSPEC_SDT=1
!ALLOCATE(NWN_TESTSPEC_SDT(NSMAX+1))
NWN_TESTSPEC_SDT(:)=20

LRDPATINIT_SDT=.FALSE.
LWRPATTRUN_SDT=.FALSE.
CIPATINIT_SDT ='sppt_pattern_0'
COPATTRUN_SDT ='sppt_pattern_trun'

LTAPER_BL0=.TRUE.
XSIGMATOP =0.87_JPRB
XSIGMABOT =0.97_JPRB

LTAPER_ST0=.TRUE.
XPRESSTOP_ST0=50.E2_JPRB
XPRESSBOT_ST0=100.E2_JPRB
!     -----------------------------------------------------------------
!*    2. Read Namelist
!     -----------------------------------------------------------------
CALL POSNAM(NULNAM,'NAMSPSDT')
READ(NULNAM,NAMSPSDT)
!     -----------------------------------------------------------------
!*    3. Set derived variables
!     -----------------------------------------------------------------
IF (LSPSDT) THEN
  ALLOCATE(IGRIB(NSCALES_SDT))
  SDEVTOT_SDT=0._JPRB
  DO JSCALE=1, NSCALES_SDT
    XWC_KAPPA_T( JSCALE )= 0.5_JPRB * (XLCOR_SDT( JSCALE )/RA)**2
    IGRIB(JSCALE)= 213000+JSCALE
    SDEVTOT_SDT=SDEVTOT_SDT+ SDEV_SDT(JSCALE)**2
  ENDDO
  SDEVTOT_SDT=SQRT(SDEVTOT_SDT)

  !
  !     tapering in boundary layer
  !
  zdsigma_m3 = 1._jprb/(xsigmabot-xsigmatop)**3

  xtaper3 =  2._jprb                      *zdsigma_m3  
  xtaper2 = -3._jprb*(xsigmatop+xsigmabot)*zdsigma_m3
  xtaper1 =  6._jprb* xsigmatop*xsigmabot *zdsigma_m3
  xtaper0 =  xsigmabot**2 * &
       &  (xsigmabot - 3._jprb*xsigmatop) *zdsigma_m3
  !
  !     tapering above tropopause
  !
  zdpress_m3=1./(xpressbot_st0-xpresstop_st0)**3

  xtaper3_st0 = -2._jprb                              *zdpress_m3
  xtaper2_st0 =  3._jprb*(xpresstop_st0+xpressbot_st0)*zdpress_m3
  xtaper1_st0 = -6._jprb* xpresstop_st0*xpressbot_st0 *zdpress_m3
  xtaper0_st0 =  1._jprb -xpressbot_st0**2 * &
             &  (xpressbot_st0-3._jprb*xpresstop_st0) *zdpress_m3
ENDIF
!     -----------------------------------------------------------------
!*    4. Write settings to NULOUT
!     -----------------------------------------------------------------
WRITE(NULOUT,'('' >> Sub SUSPSDT '')')
WRITE(NULOUT,'(''      spectral stochastic diabatic tendencies LSPSDT= '',L1)') LSPSDT
WRITE(NULOUT,'(''      Fix average changes to tendencies to 0 LFIXSPPT= '',L1)') LFIXSPPT
IF (LSPSDT) THEN
  WRITE(NULOUT,'(''      NSCALES_SDT='',I2)') NSCALES_SDT  
  IF (NSCALES_SDT > NMAXSCALES_SDT) THEN
    WRITE(NULOUT,'(''NMAXSCALES_SDT='',I2)') NMAXSCALES_SDT
    CALL ABOR1('    NSCALES_SDT > NMAXSCALES_SDT')
  ENDIF
  WRITE(NULOUT,'(''      SCALE'',4A14)') 'SDEV_SDT','TAU_SDT','XLCOR_SDT','XWC_KAPPA_T'
  WRITE(NULOUT,'(''      ---------------------------------------------------------------------'')')
  DO JSCALE=1, NSCALES_SDT
    WRITE(NULOUT,'(''      '',I5,4E14.5)') JSCALE, &
         &SDEV_SDT(JSCALE), TAU_SDT(JSCALE),XLCOR_SDT(JSCALE), XWC_KAPPA_T(JSCALE)
  ENDDO
  WRITE(NULOUT,'(''      ---------------------------------------------------------------------'')')
  WRITE(NULOUT,'(''      SDEVTOT_SDT='',E20.14)') SDEVTOT_SDT
  WRITE(NULOUT,'(''      CSPEC_SHAPE_SDT='',A)') trim(CSPEC_SHAPE_SDT)
  IF (CSPEC_SHAPE_SDT=='testSpectrum') THEN
    WRITE(NULOUT,'(''  testSpectrum: NWAV_TESTSPEC_SDT='',I5)') NWAV_TESTSPEC_SDT
    WRITE(NULOUT,'(10X,10I5)') NWN_TESTSPEC_SDT(1:NWAV_TESTSPEC_SDT)
  ENDIF
  WRITE(NULOUT,'(''      LCLIP_SPEC_SDT='',L1,'', LCLIP_GRID_SDT='',L1)') &
       &                 LCLIP_SPEC_SDT,          LCLIP_GRID_SDT
  WRITE(NULOUT,'(''      LRDPATINIT_SDT='',L1,'', CIPATINIT_SDT='',A)') &
       &                 LRDPATINIT_SDT,     TRIM(CIPATINIT_SDT)
  WRITE(NULOUT,'(''      LWRPATTRUN_SDT='',L1,'', COPATTRUN_SDT='',A)') &
       &                 LWRPATTRUN_SDT,     TRIM(COPATTRUN_SDT)
  WRITE(NULOUT,'(''      LQPERTLIMIT2='',L1,'', NQSAT_SDT='',I5)')&
       &  LQPERTLIMIT2, NQSAT_SDT
  WRITE(NULOUT,'(''      XMEANRED_THICKNESS_SDT='',E14.5)') &
       &  XMEANRED_THICKNESS_SDT
  WRITE(NULOUT,'(''      XCLIP_RATIO_SDT='',F12.5)') XCLIP_RATIO_SDT
  WRITE(NULOUT,'(''      LUSESETRAN_SDT='',L1,'', LRESETSEED_SDT='',L1)') &
       & LUSESETRAN_SDT, LRESETSEED_SDT
  WRITE(NULOUT,'(''      NSEED_SDT='',I12)')  NSEED_SDT
  WRITE(NULOUT,'(''      LWRITE_ARP='',L1)')  LWRITE_ARP
  IF (LWRITE_ARP) THEN
    WRITE(NULOUT,'(''      COPATSP_SDT='',A)') TRIM(COPATSP_SDT)
    WRITE(NULOUT,'(''      COPATGP_SDT='',A)') TRIM(COPATGP_SDT)
  ENDIF
  WRITE(NULOUT,'(''      LTAPER_BL0='',L1)')  LTAPER_BL0
  IF (LTAPER_BL0) THEN
    WRITE(NULOUT,'(''XSIGMATOP='',F12.5,'', XSIGMABOT='',F12.5)') &
         & XSIGMATOP, XSIGMABOT
    IF (XSIGMATOP > XSIGMABOT) THEN
      CALL ABOR1('XSIGMATOP > XSIGMABOT is not allowed for tapering.')
    ENDIF
    WRITE(NULOUT,'(''XTAPER3='',E14.5,'', XTAPER2='',E14.5)') &
         & XTAPER3,XTAPER2
    WRITE(NULOUT,'(''XTAPER1='',E14.5,'', XTAPER0='',E14.5)') &
         & XTAPER1,XTAPER0
  ENDIF
  WRITE(NULOUT,'(''      LTAPER_ST0='',L1)')  LTAPER_ST0
  IF (LTAPER_ST0) THEN
    WRITE(NULOUT,'(''XPRESSTOP_ST0='',F12.5,'', XPRESSBOT_ST0='',F12.5)') &
         & XPRESSTOP_ST0, XPRESSBOT_ST0
    IF (XPRESSTOP_ST0 > XPRESSBOT_ST0) THEN
      CALL ABOR1('XPRESSTOP_ST0 > XPRESSBOT_ST0 is not allowed for tapering.')
    ENDIF
    WRITE(NULOUT,'(''XTAPER3_ST0='',E14.5,'', XTAPER2_ST0='',E14.5)') &
         & XTAPER3_ST0,XTAPER2_ST0
    WRITE(NULOUT,'(''XTAPER1_ST0='',E14.5,'', XTAPER0_ST0='',E14.5)') &
         & XTAPER1_ST0,XTAPER0_ST0
  ENDIF
ELSE
  LQPERTLIMIT2   =.FALSE.
  WRITE(NULOUT,'(''      LQPERTLIMIT2='',L1)')  LQPERTLIMIT2
ENDIF
!     -----------------------------------------------------------------
!*    5. Initialize spectral AR(1) process
!     -----------------------------------------------------------------
IF (LSPSDT) THEN
  IF (LUSESETRAN_SDT) THEN
    CALL SETRAN( NSEED_SDT, KOUTSEED=ISEED)
  ELSE
    ISEED=NSEED_SDT
  ENDIF
  WRITE(NULOUT,'('' SUSPSDT: Using seed '',I12)')  ISEED     
  CALL ALLOCATE_ARP(  YSPSDT_AR1,NSMAX,0,0,NSCALES_SDT,0,IGRIB,ISEED,LDCLIP=LCLIP_SPEC_SDT,LDSUM=.TRUE.)
  DEALLOCATE(IGRIB)
  ALLOCATE(IGRIB(1))
  IGRIB=101
  CALL ALLOCATE_GRID( YGPSDT, 0, 1, IGRIB )
  DEALLOCATE(IGRIB)
  allocate(zchi(0:nsmax))
  allocate(zsdev2d(0:nsmax,NSCALES_SDT))
  allocate(zphi2d( 0:nsmax,NSCALES_SDT))
  DO JSCALE=1, NSCALES_SDT
    ZCHI    =  0._JPRB
    SELECT CASE (CSPEC_SHAPE_SDT)
    !CASE ('PowerLaw')
    !  DO IN=1,NSMAX
    !    ZCHI(IN)=IN**REXPONENT ! set mean spectral power as a function of n
    !  ENDDO
    CASE ('testSpectrum')
      IF (NWAV_TESTSPEC_SDT>NSMAX+1) THEN
        CALL ABOR1('suspsdt: NWAV_TESTSPEC_SDT>NSMAX+1')
      ENDIF
      DO JN=1,NWAV_TESTSPEC_SDT
        INWAVE=NWN_TESTSPEC_SDT(JN)
        IF ((INWAVE>=0).AND.(INWAVE<=NSMAX)) THEN
          ZCHI(INWAVE)=SQRT(1._JPRB/REAL(2*INWAVE+1,KIND=JPRB))
        ELSE
          WRITE(NULOUT,'(''  testSpectrum: wavenumber '',I5,'' omitted.'')')
        ENDIF
      ENDDO
    CASE ('WeaverCourtier1')
      DO JN=1,NSMAX
        ZCHI(JN)=exp(-0.5* XWC_KAPPA_T(JSCALE) * REAL(JN*(JN+1),KIND=JPRB))
      ENDDO
    CASE ('WeaverCourtier')
      DO JN=0,NSMAX
        ZCHI(JN)=exp(-0.5* XWC_KAPPA_T(JSCALE) * REAL(JN*(JN+1),KIND=JPRB))
      ENDDO
    CASE DEFAULT
      WRITE(NULOUT,'(''CSPEC_SHAPE_SDT='',A)') trim(CSPEC_SHAPE_SDT)
      CALL ABOR1('SURAND1: unknown shape of variance spectrum.')
    END SELECT

    ZGAMMAN  =  0._JPRB
    DO JN=1,NSMAX
      ZGAMMAN= ZGAMMAN + (2*JN+1)*ZCHI(JN)**2
    ENDDO
    ZPHI    =  EXP(-TDT/TAU_SDT(JSCALE))

    ZCONSTF0= SQRT(0.5_JPRB*(1.0_JPRB - ZPHI*ZPHI))*SDEV_SDT(JSCALE)/SQRT(ZGAMMAN) 
    ZSDEV2D(:,JSCALE) = ZCONSTF0*ZCHI(:)
    ZPHI2D(:, JSCALE) = ZPHI
  ENDDO
  CALL set_arp2d(YSPSDT_AR1, zsdev2d, zphi2d )
  IF (LRDPATINIT_SDT) THEN
    CALL READ_SPEC_GRIB(CIPATINIT_SDT, YSPSDT_AR1%SF)
  ELSE
    CALL evolve_arp(YSPSDT_AR1, ldinit=.true.)
  ENDIF
  CALL sum_arps(YSPSDT_AR1)
  if (lwrite_arp) then
    call write_spec_grib(COPATSP_SDT , yspsdt_ar1%sf)
  endif
  call spectral_norm(YSPSDT_AR1%SF,    'SUSPSDT, YSPSDT_AR1(1,...), ')
  call spectral_norm(YSPSDT_AR1%SFSUM, 'SUSPSDT, YSPSDT_AR1( sum ), ')
  deallocate(zsdev2d, zphi2d, zchi)
ELSE
  ALLOCATE(IGRIB(0))
  CALL ALLOCATE_GRID( YGPSDT, 0, 0, igrib )
  DEALLOCATE(IGRIB)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSPSDT',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPSDT
