SUBROUTINE SUSPSP(PSPSP)

!**** *SUSPSP*  - Setup spectral surface pressure

!     Purpose.
!     --------
!         To possibly change spectral surface pressure in order to be
!         Ln(Ps)

!**   Interface.
!     ----------
!        *CALL* *SUSPSP(.....)*

!        Explicit arguments :
!        --------------------
!        PSPSP : spectral surface pressure

!        Implicit arguments :
!        --------------------
!        See modules above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        SET2PE
!        BROADCREAL
!        SP2LNSP
!        ESP2LNSP

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        Ryad El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 99-04-23 (from suspecasm/dm)
!        R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 B-level distribution
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NSPEC2
USE YOMCT0   , ONLY : LELAM    ,NPROC
USE YOMMP    , ONLY : MYPROC   ,NBSETSP
USE MPL_MODULE, ONLY : MPL_BROADCAST

IMPLICIT NONE

! * INPUT/OUTPUT:
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSP(NSPEC2) 
INTEGER(KIND=JPIM) :: ISETW0   ! Set-W processor owning wavenumber 0
INTEGER(KIND=JPIM) :: IPRSPW0  ! processor owning Surface pressure wavenumber 0

REAL(KIND=JPRB)    :: ZSPW0(1) ! mean value of surface pressure
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "esp2lnsp.intfb.h"
#include "set2pe.intfb.h"
#include "sp2lnsp.intfb.h"

!-----------------------------------------------------------

!*  1. Broadcast mean value of surface pressure

IF (LHOOK) CALL DR_HOOK('SUSPSP',0,ZHOOK_HANDLE)
ISETW0=1
CALL SET2PE(IPRSPW0,0,0,ISETW0,NBSETSP)

IF (MYPROC == IPRSPW0) THEN
  ZSPW0(1)=PSPSP(1)
ENDIF
IF (NPROC > 1) THEN
  CALL MPL_BROADCAST(ZSPW0,KROOT=IPRSPW0,KTAG=1, &
   & CDSTRING='SUSPSP')
ENDIF

!*  2. Change surface pressure if needed

IF (ZSPW0(1) > 8.E+4_JPRB) THEN
  WRITE(NULOUT,*) ' *** SUSPSP PS ----> LN(PS) ***'
  IF (LELAM) THEN
    CALL ESP2LNSP(PSPSP)
  ELSE
    CALL SP2LNSP(PSPSP)
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('SUSPSP',1,ZHOOK_HANDLE)

END SUBROUTINE SUSPSP
