SUBROUTINE SUTRAJP

!**** *SUTRAJP*   - Set-up parameters for adjoint of physics

!     Purpose.
!     --------

!        Set-up physical parametrizations to be activated in
!        the tangent-linear and adjoints versions of the IFS
!        and options related to the management of the
!        trajectory at t-dt

!**   Interface.
!     ----------
!        *CALL* *SUTRAJP*

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        COMMON YOMIOP, YOPHNC, YOMSIMPHL

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        J.-F. Mahfouf

!     Modifications.
!     --------------
!        Original : 96-09-27
!        Modified : 97-05-07 M. Janiskova (set-up for Meteo-France
!                                          adjoint of physics)
!        Modified : 99-02-01 M. Charron (set-up for non-hydrostatic)
!                 2000-01-20 M. Janiskova (set-up for ECMWF phys.
!                                          tendencies) 
!                 2000-12-18 M. Janiskova (packing factor for phys.
!                                          tendencies)
!                 2002-10-30 M. Janiskova (3D fields for cloud parameters)
!                 2003-12-10 P. Lopez     (3D fields for convective parameters)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!                 2008-05-01 M. Janiskova (updated set-up for lw radiation)
!                 2009-09-21 M. Janiskova (3D fields for non-orog.gravity wave)
!                 2010-05-18 M. Janiskova (updated set-up when lemwave)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMIOP   , ONLY :  NPCKFTS5 ,NEXPBTS5 ,&
 & NPCKFPH5 ,NEXPBPH5 ,NG3D95   ,NG3P95   ,NG3DT95 ,&
 & NG3PR95  ,NG3S95   ,NG2D95   ,&
 & NG2T95   ,NG2DSF5  ,NG2DS5  
USE YOMIOPNH , ONLY : NG3NH95
USE YOMSIMPHL, ONLY : LTRAJPS  ,LTRAJPST
USE YOPHNC   , ONLY : LETRAJPT ,LERADSW2 ,LERADN2, LERADLW2, LENCLD2, LEGWWMS2
USE YOEPHY   , ONLY : LEMWAVE

IMPLICIT NONE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    Set-default values for trajectory storage at t-dt
!              --------------------------------------------------

!*       1.2  Packing parameters
!        -----------------------

IF (LHOOK) CALL DR_HOOK('SUTRAJP',0,ZHOOK_HANDLE)
NPCKFTS5 = 1
NEXPBTS5 = 6
NPCKFPH5 = 1
NEXPBPH5 = 6

!     ------------------------------------------------------------------

!*       2.    Set-default values for the number of 3D/2D fields to store
!              ----------------------------------------------------------

! ECMWF set-up

NG3D95 = 9
NG3P95 = 2
NG3S95 = 3
NG2D95 = 11
NG2T95 = 5
NG2DS5 = 0

NG3DT95 = 11
NG3PR95 = 2
NG2DSF5 = 0

IF (LERADN2.OR.LERADSW2.OR.LERADLW2) THEN
  NG3PR95 =  NG3PR95 + 2
ENDIF
IF (LENCLD2) THEN
  NG3DT95 =  NG3DT95 + 5
ENDIF
IF (.NOT.LEMWAVE) THEN
  NG3DT95 =  NG3DT95 + 1
ENDIF
IF (LENCLD2 .AND. LEMWAVE) THEN
  NG3PR95 = NG3PR95+2
  NG2DSF5 = NG2DSF5+4
ENDIF
IF (LEGWWMS2) THEN
  NG3DT95 =  NG3DT95 + 2
ENDIF

IF (.NOT. LETRAJPT) THEN
  NG3DT95 = 0
  NG3PR95 = 0
  NG2DSF5 = 0
  IF (LERADN2.OR.LERADSW2.OR.LERADLW2) THEN
    NG3PR95 =  NG3PR95 + 2
  ENDIF
  IF (LENCLD2) THEN
    NG3DT95 =  NG3DT95 + 5
  ENDIF
  IF (LEGWWMS2) THEN
    NG3DT95 =  NG3DT95 + 2
  ENDIF
ENDIF

! Meteo-France set-up

NG3NH95 = 9

IF (LTRAJPS) THEN
  NG3D95 = 4
  NG3P95 = 0
  NG3S95 = 0
  NG2D95 = 1
  NG2DS5 = 3
ENDIF

IF (LTRAJPST) THEN
  NG3DT95 = 5
  NG3PR95 = 27
  NG2DSF5 = 4
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUTRAJP',1,ZHOOK_HANDLE)
END SUBROUTINE SUTRAJP
