SUBROUTINE SUTRANS(CDMODE)

!**** *SUTRANS * - Initialize the transform package

!     Purpose.  Initialize the transform package
!     --------

!**   Interface.  CALL SUTRANS
!     ---------- 

!     Explicit arguments :  None
!     --------------------
!        Called by SU0YOMB.

!     Externals.
!     ----------
!        SETUP_TRANS0 - basic initialization
!        SETUP_TRANS  - resolution dependent initialization

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!        R. El Khatib 03-01-24 LMPOFF
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib  07-Apr-2005 NPROMATR saved in YOMTRANS
!        D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!        G.Mozdzynski  13-Sep-2006 LEQ_REGION partitioning
!        Y.Seity       31-Aug-2007 LSYNC_TRANS, and RDISTR_E
!        G. Radnoti    16-03-2009  Option to switch to/from mono tasking transforms 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULERR   ,NULNAM
USE YOMCT0   , ONLY : NPROC    ,NPRGPNS  ,NPRGPEW  ,NPRTRW  ,NPRINTLEV ,LMPOFF    ,&
 & N_REGIONS_NS ,N_REGIONS_EW  ,N_REGIONS
USE YOMDIM   , ONLY : NSMAX    ,NDGLG    ,NTMAX
USE YOMGEM   , ONLY : NLOENG
USE YOMMP    , ONLY : LSPLIT   ,NCOMBFLEN  ,LEQ_REGIONS, LSYNC_TRANS
USE YOMTRANS , ONLY : NPROMATR ,NEPROMATR, RDISTR_E, LMONO_TRANS
USE YOEPHY   , ONLY : LERADI
USE YOMWAVELET , ONLY : LJBWAVELET, NMAX_RESOL
USE YOMCOAPHY , ONLY : NPHYINT
USE YOMCST, ONLY     : RA
IMPLICIT NONE

CHARACTER(LEN=5) ,OPTIONAL, INTENT(IN)    :: CDMODE

INTEGER(KIND=JPIM) :: IRESOL
INTEGER(KIND=JPIM) :: I_REGIONS(NPROC+2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
CHARACTER(LEN=5) :: CLMODE
#include "namtrans.h"

INTERFACE
#include "setup_trans0.h"
#include "setup_trans.h"
#include "trans_end.h"
END INTERFACE

#include "abor1.intfb.h"
#include "posnam.intfb.h"

!      -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUTRANS',0,ZHOOK_HANDLE)
CLMODE='SETUP'
IF (PRESENT(CDMODE)) CLMODE=CDMODE
IF (LJBWAVELET) THEN
  NMAX_RESOL = 20
ELSE
  NMAX_RESOL = 1
ENDIF
LMONO_TRANS=(CLMODE == 'MONO1')
!GM Increase max number of grids if ecmwf radiation
!GM I would have prefered to have used LRADGRID but it has not been set yet
!GM Use of LERADI is therefore a compromise here
IF( LERADI ) NMAX_RESOL=NMAX_RESOL+1
IF( NPHYINT > 0) NMAX_RESOL=NMAX_RESOL+1
NPROMATR = 0
RDISTR_E = 0
CALL POSNAM(NULNAM,'NAMTRANS')
READ(NULNAM,NAMTRANS)
WRITE(NULOUT,*)'SUTRANS:NMAX_RESOL=',NMAX_RESOL,' NPROMATR = ',NPROMATR
WRITE(NULOUT,*)'SUTRANS:RDISTR_E=',RDISTR_E
IF (CLMODE == 'MONO1') THEN
  CALL TRANS_END('INTER')
  call SETUP_TRANS0(KOUT=NULOUT,KERR=NULERR,KMAX_RESOL=NMAX_RESOL) 
ELSEIF (CLMODE == 'RESET' ) THEN
  CALL TRANS_END('INTER')
  CALL SETUP_TRANS0(KOUT=NULOUT,KERR=NULERR,KPRGPNS=NPRGPNS,&
   & KPRGPEW=NPRGPEW,KPRTRW=NPRTRW,KPRINTLEV=NPRINTLEV,KMAX_RESOL=NMAX_RESOL,&
   & KPROMATR=NPROMATR,KCOMBFLEN=NCOMBFLEN,LDSYNC_TRANS=LSYNC_TRANS,&
   & LDMPOFF=LMPOFF,LDEQ_REGIONS=LEQ_REGIONS,&
   & K_REGIONS=I_REGIONS(:),K_REGIONS_NS=N_REGIONS_NS,K_REGIONS_EW=N_REGIONS_EW,&
   & PRAD=RA)
! now that we know the actual size of N_REGIONS we can allocate it and copy
! over values from the temporary location
   IF (.NOT. ALLOCATED(N_REGIONS)) ALLOCATE(N_REGIONS(1:N_REGIONS_NS))
   N_REGIONS(1:N_REGIONS_NS)=I_REGIONS(1:N_REGIONS_NS)
ELSEIF (CLMODE == 'SETUP' ) THEN
  CALL SETUP_TRANS0(KOUT=NULOUT,KERR=NULERR,KPRGPNS=NPRGPNS,&
   & KPRGPEW=NPRGPEW,KPRTRW=NPRTRW,KPRINTLEV=NPRINTLEV,KMAX_RESOL=NMAX_RESOL,&
   & KPROMATR=NPROMATR,KCOMBFLEN=NCOMBFLEN,LDSYNC_TRANS=LSYNC_TRANS,&
   & LDMPOFF=LMPOFF,LDEQ_REGIONS=LEQ_REGIONS,&
   & K_REGIONS=I_REGIONS(:),K_REGIONS_NS=N_REGIONS_NS,K_REGIONS_EW=N_REGIONS_EW)
! now that we know the actual size of N_REGIONS we can allocate it and copy
! over values from the temporary location
ALLOCATE(N_REGIONS(1:N_REGIONS_NS))
N_REGIONS(1:N_REGIONS_NS)=I_REGIONS(1:N_REGIONS_NS)

CALL SETUP_TRANS(KSMAX=NSMAX,KDGL=NDGLG,KLOEN=NLOENG(1:NDGLG),&
 & LDSPLIT=LSPLIT,&
 & KTMAX=NTMAX,KRESOL=IRESOL)

!  The main transforms do not specify KRESOL. I.e. they default to KRESOL=1.
   IF (IRESOL/=1) THEN
     CALL ABOR1 ('SUTRANS: MODEL RESOLUTION MUST HAVE KRESOL==1')
   ENDIF
ELSE
   CALL ABOR1('WRONG SUTRANS MODE')
ENDIF
IF (LHOOK) CALL DR_HOOK('SUTRANS',1,ZHOOK_HANDLE)

!      -----------------------------------------------------------------

END SUBROUTINE SUTRANS
