SUBROUTINE SUVAREPS

!**** *SUVAREPS*   - Routine to initialize variables that control 
!                    VARiable Resolution EPS (VAREPS)

!     Purpose.
!     --------
!           Initialize VAREPS control common
!**   Interface.
!     ----------
!        *CALL* *SUVAREPS

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        COMMON YOMVAREPS

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Authors.
!     -------
!        Roberto Buizza  *ECMWF*

!     Modifications.
!     --------------
!        Original : 2004-01-29
!        MODIFIED : 2008-03-11 F. Vitart (add STRD, SSR and STR to the list of possible accumulated parameters)
!     -----------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMVAREPS, ONLY : NRES_MA, NRES_MB, &
                     &NFCHO_TRUNC_INI, NFCLENGTH_INI, NST_FCLENGTH_INI,&
                     &NFCHO_TRUNC, NST_TRUNC, NST_TRUNC_INI, NST_HDIF_INI,&
                     &NFCHO_HDIF_TRANS, NST_HDIF_TRANS,&
                     &NAFVAREPSMAX,NAFVAREPS,NAFVAREPSGC,&
                     &LVAREPS, L_INC_HDIF,&
                     &HDIRVOR_MA, HDIRDIV_MA, HDIRT_MA, HDIRQ_MA, &
                     &HDIRVOR_MB, HDIRDIV_MB, HDIRT_MB, HDIRQ_MB
USE YOMGRB   , ONLY : NGRBES   ,NGRBSMLT ,NGRBLSPF ,NGRBLSP  ,NGRBCP   ,&
                     &NGRBSF   ,NGRBBLD  ,NGRBSSHF ,NGRBSLHF ,&
                     &NGRBSSRD ,NGRBSTRD  ,NGRBSSR  ,NGRBSTR  , NGRBTSR ,&
                     &NGRBTTR  ,NGRBEWSS ,NGRBNSSS ,&
                     &NGRBE    ,NGRBSUND ,NGRBLGWS ,NGRBMGWS ,NGRBGWD  ,&
                     &NGRBSRC  ,NGRBRO   ,NGRBTSRC ,NGRBTTRC ,NGRBSSRC ,&
                     &NGRBSTRC ,NGRBTP   ,NGRBCSF  ,NGRBLSF
USE YOMDYN   , ONLY : LNEWHD, TSTEP, HDIRVOR, HDIRDIV, HDIRT, HDIRQ, RDAMPVOR
USE YOMCT0B  , ONLY : LECMWF
USE YOMFPC   , ONLY : MFPPHY, NFPPHY
USE CONTROL_VECTORS

!     -----------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: ISECS
INTEGER(KIND=JPIM) :: JAF,JSFC,IAFAF,IAFGC(NAFVAREPSMAX)
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZHDIRDIV, ZHDIRVOR

!     -----------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"

#include "namvareps.h"
!     -----------------------------------------------------------------


IF (LHOOK) CALL DR_HOOK('SUVAREPS',0,ZHOOK_HANDLE)

!     -----------------------------------------------------------------

!*    1. Set default values.
!     ----------------------

!     1.1 Set implicit default values

LVAREPS=.FALSE.
L_INC_HDIF=.FALSE.
NRES_MA=NSMAX
NRES_MB=NSMAX
NFCHO_TRUNC_INI=0
NFCLENGTH_INI=0
NFCHO_TRUNC=0
NST_TRUNC_INI=0
NST_TRUNC=0
NFCHO_HDIF_TRANS=0
NST_HDIF_TRANS=0
NST_HDIF_INI=0
NAFVAREPS=0
DO JAF=1,NAFVAREPSMAX
  NAFVAREPSGC(JAF)=0
ENDDO

!     -----------------------------------------------------------------

!*    2. Read Namelist
!     ----------------

CALL POSNAM(NULNAM,'NAMVAREPS')
READ(NULNAM,NAMVAREPS)

ISECS=3600.0
IF( (NFCLENGTH_INI-NFCHO_TRUNC_INI) >= 0 ) THEN
  NST_FCLENGTH_INI=NINT(((NFCLENGTH_INI-NFCHO_TRUNC_INI)*ISECS)/TSTEP)
ELSE
  NST_FCLENGTH_INI=0
ENDIF

NST_TRUNC_INI=NINT((NFCHO_TRUNC_INI*ISECS)/TSTEP)
NST_TRUNC=NINT((NFCHO_TRUNC*ISECS)/TSTEP)
NST_HDIF_TRANS=NINT((NFCHO_HDIF_TRANS*ISECS)/TSTEP)
NST_HDIF_INI=NST_TRUNC-NST_HDIF_TRANS

! Set horizontal diffusion variables

HDIRVOR_MA=HDIRVOR
HDIRDIV_MA=HDIRDIV
HDIRT_MA=HDIRT
HDIRQ_MA=HDIRQ

IF (LECMWF) THEN

  IF (LNEWHD) THEN

    ! NOT YET CODED

    ! * this part (and more particularly the setting of ZHDIRVOR)
    !   is consistent with what is coded in SUHDIR
    !   only in the case (lecmwf=t,lnewhd=f).
    WRITE(NULOUT,*) ' In SUVAREPS, calculation of ZHDIRVOR and HDIR..._MB'
    WRITE(NULOUT,*) ' only works for (LECMWF=T,LNEWHD=F)'
    CALL ABOR1(' SUVAREPS: ABOR1 called ')

  ELSE

    IF(NRES_MB >= 799) THEN
      ZHDIRVOR=1200._JPRB
    ELSEIF(NRES_MB >= 511) THEN
      ZHDIRVOR=1800._JPRB
    ELSEIF(NRES_MB >= 319) THEN
      ZHDIRVOR=2100._JPRB
    ELSEIF(NRES_MB >= 255) THEN
      ZHDIRVOR=2500._JPRB
    ELSEIF(NRES_MB >= 213) THEN
      ZHDIRVOR=2700._JPRB
    ELSEIF(NRES_MB >= 159) THEN
      ZHDIRVOR=5400._JPRB
    ELSEIF(NRES_MB >= 106) THEN
      ZHDIRVOR=12570._JPRB
    ELSEIF(NRES_MB >= 95) THEN
      ZHDIRVOR=15000._JPRB
    ELSEIF(NRES_MB >= 63) THEN
      ZHDIRVOR=27000._JPRB
    ELSEIF(NRES_MB >= 42) THEN
      ZHDIRVOR=54000._JPRB
    ELSEIF(NRES_MB >= 21) THEN
      ZHDIRVOR=108000._JPRB
    ELSE
      CALL ABOR1(' WRONG HORIZONTAL RESOLUTION')
    ENDIF

    IF (RDAMPVOR /= 0._JPRB) THEN
      ZHDIRDIV = ZHDIRVOR / RDAMPVOR
    ELSE
      ZHDIRDIV = 0._JPRB
    ENDIF

    HDIRVOR_MB = REAL(NINT(ZHDIRVOR),JPRB)
    HDIRDIV_MB = REAL(NINT(ZHDIRDIV),JPRB)
    HDIRT_MB   = HDIRVOR_MB
    HDIRQ_MB   = HDIRVOR_MB

  ENDIF

ELSE

  ! NOT YET PROPERLY CODED

  ! * this part (and more particularly the setting of ZHDIRVOR)
  !   is consistent with what is coded in SUHDIR
  !   only in the case (lecmwf=t,lnewhd=f).
  WRITE(NULOUT,*)
  WRITE(NULOUT,*) ' In SUVAREPS, calculation of ZHDIRVOR and HDIR..._MB'
  WRITE(NULOUT,*) ' only works properly for (LECMWF=T,LNEWHD=F).'
  WRITE(NULOUT,*) ' HDIR[X]_MB is provisionally set equal to HDIR[X]_MA.'
  WRITE(NULOUT,*)

  HDIRVOR_MB = HDIRVOR_MA
  HDIRDIV_MB = HDIRDIV_MA
  HDIRT_MB   = HDIRT_MA
  HDIRQ_MB   = HDIRQ_MA
  
ENDIF

IF(LVAREPS) THEN

! List of accumulated fields (NB: IAFGC is dimensioned by NAFVAREPSMAX, set in YOMVAREPS)

  IAFGC(1)=NGRBES
  IAFGC(2)=NGRBSMLT
  IAFGC(3)=NGRBLSPF
  IAFGC(4)=NGRBLSP
  IAFGC(5)=NGRBCP
  IAFGC(6)=NGRBSF
  IAFGC(7)=NGRBBLD
  IAFGC(8)=NGRBSSHF
  IAFGC(9)=NGRBSLHF
  IAFGC(10)=NGRBSSRD
  IAFGC(11)=NGRBSTRD
  IAFGC(12)=NGRBSSR
  IAFGC(13)=NGRBSTR
  IAFGC(14)=NGRBTSR
  IAFGC(15)=NGRBTTR
  IAFGC(16)=NGRBEWSS
  IAFGC(17)=NGRBNSSS
  IAFGC(18)=NGRBE
  IAFGC(19)=NGRBSUND
  IAFGC(20)=NGRBLGWS
  IAFGC(21)=NGRBMGWS
  IAFGC(22)=NGRBGWD
  IAFGC(23)=NGRBSRC
  IAFGC(24)=NGRBRO
  IAFGC(25)=NGRBTSRC
  IAFGC(26)=NGRBTTRC
  IAFGC(27)=NGRBSSRC
  IAFGC(28)=NGRBSTRC
  IAFGC(29)=NGRBTP
  IAFGC(30)=NGRBCSF
  IAFGC(31)=NGRBLSF

! Check whether any of the post-processed surface fields is an accumulated one,
! and add it to the list NAFVAREPSGC

  IAFAF=0
  DO JSFC=1,NFPPHY
    DO JAF=1,NAFVAREPSMAX
      IF(MFPPHY(JSFC) == IAFGC(JAF)) THEN
        IAFAF=IAFAF+1
        IF (IAFAF > NAFVAREPSMAX) THEN
          WRITE(NULOUT,'('' >> Sub SUVAREPS ERROR - IAFAF gt NAFVAREPSMAX'')')
          WRITE(NULOUT,'(''    IAFAF, NAFVAREPSMAX ='',2(I10,1X))') IAFAF, NAFVAREPSMAX
          CALL ABOR1('SUVAREPS: ABOR1 CALLED')
        ENDIF
        NAFVAREPSGC(IAFAF)=MFPPHY(JSFC)
        WRITE(NULOUT,'('' >> Sub SUVAREPS found grid file'')')
        WRITE(NULOUT,'(''    IAFAF, JSFC, MFPPHY(JSFC)='',3(I10,1X))') IAFAF,JSFC,MFPPHY(JSFC)
      ENDIF
    ENDDO
  ENDDO
  NAFVAREPS=IAFAF
ENDIF

!     -----------------------------------------------------------------

!*    3. Print final values.
!     ----------------------

IF (LVAREPS) THEN 
 WRITE(NULOUT,'('' >> Sub SUVAREPS'')')
 WRITE(NULOUT,'('' VARiable Resolution EPS     = '',L1)') LVAREPS
 WRITE(NULOUT,'('' VAREPS variables in common YOMVAREPS'')')
 WRITE(NULOUT,'('' Spectral trunc model_A             NRES_MA= '',I10)') NRES_MA
 WRITE(NULOUT,'('' Spectral trunc model_B             NRES_MB= '',I10)') NRES_MB
 WRITE(NULOUT,'('' Fc-step used to define ICS NFCHO_TRUNC_INI= '',I10)') NFCHO_TRUNC_INI
 WRITE(NULOUT,'('' Fc length previous leg       NFCLENGTH_INI= '',I10)') NFCLENGTH_INI
 WRITE(NULOUT,'('' .. NST Fc len prev leg    NST_FCLENGTH_INI= '',I10)') NST_FCLENGTH_INI
 WRITE(NULOUT,'('' Truncation forecast time       NFCHO_TRUNC= '',I10)') NFCHO_TRUNC
 IF (L_INC_HDIF) THEN
  WRITE(NULOUT,'('' Increase hor diffusion          L_INC_HDIF= '',L1)') L_INC_HDIF
  WRITE(NULOUT,'('' Fc-hours of resol change       NFCHO_TRUNC= '',I10)') NFCHO_TRUNC
  WRITE(NULOUT,'('' .. NST of resol change           NST_TRUNC= '',I10)') NST_TRUNC
  WRITE(NULOUT,'('' Hours of hdif transition  NFCHO_HDIF_TRANS= '',I10)') NFCHO_HDIF_TRANS
  WRITE(NULOUT,'('' .. NST of hdif transition   NST_HDIF_TRANS= '',I10)') NST_HDIF_TRANS
  WRITE(NULOUT,'('' .. Ini NST hdif transit       NST_HDIF_INI= '',I10)') NST_HDIF_INI
  WRITE(NULOUT,'('' Model_A hor-dif             '')') 
  WRITE(NULOUT,'('' Model_A hor-dif HDIRVOR_MA HDIRVOR_MA= '',E26.20)') HDIRVOR_MA
  WRITE(NULOUT,'('' Model_A hor-dif HDIRDIV_MA HDIRDIV_MA= '',E26.20)') HDIRDIV_MA
  WRITE(NULOUT,'('' Model_A hor-dif HDIRT_MA     HDIRT_MA= '',E26.20)') HDIRT_MA
  WRITE(NULOUT,'('' Model_A hor-dif HDIRQ_MA     HDIRQ_MA= '',E26.20)') HDIRQ_MA
  WRITE(NULOUT,'('' Model_B hor-dif             '')') 
  WRITE(NULOUT,'('' Model_B hor-dif HDIRVOR_MB HDIRVOR_MB= '',E26.20)') HDIRVOR_MB
  WRITE(NULOUT,'('' Model_B hor-dif HDIRDIV_MB HDIRDIV_MB= '',E26.20)') HDIRDIV_MB
  WRITE(NULOUT,'('' Model_B hor-dif HDIRT_MB     HDIRT_MB= '',E26.20)') HDIRT_MB
  WRITE(NULOUT,'('' Model_B hor-dif HDIRQ_MB     HDIRQ_MB= '',E26.20)') HDIRQ_MB
 ENDIF
 WRITE(NULOUT,'('' >> SUVAREPS: Accumulated fields to be re-set'')')
 WRITE(NULOUT,'(''    Tot number of Acc Fields  NAFVAREPS= '',I10)') NAFVAREPS
 DO JAF=1,NAFVAREPS
   WRITE(NULOUT,'(''    List of accum fields '',2(1X,I10))') JAF,NAFVAREPSGC(JAF)
 ENDDO
ENDIF

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVAREPS',1,ZHOOK_HANDLE)
END SUBROUTINE SUVAREPS
