#ifdef RS6K
! compiler bug on RS6000
@PROCESS NOOPTIMIZE
#endif
SUBROUTINE SUVERT

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIM   , ONLY : JPMXLE
USE YOMARG   , ONLY : NARGS    ,UVALH    ,UVBH
USE YOMDIM   , ONLY : NFLEVG   ,NRLEVX
USE YOMLUN   , ONLY : NULOUT   ,NULNAM   ,NULERR
USE YOMCT0   , ONLY : LFPART2  ,NPRINTLEV,LSLAG    ,&
 & LREGETA  ,LVFE_REGETA,LOUTPUT  ,LNHDYN  
USE YOMCT0B  , ONLY : LECMWF
USE YOMGEM   , ONLY : VALH     ,VBH      ,VETAH    ,VAH      ,VC       , &
 &                    VDELB    ,VETAF    ,VCUICO   ,VSLD     ,VSLDW    , &
 &                    NVAUTF   ,VCUICOH  ,VSLDH    ,VSLDWH   ,NVAUTH   , &
 &                    TOPPRES  ,VP00     ,VRLEVX   ,VAF      ,VBF      , &
 &                    VDELA    ,VFE_ETAH ,VFE_ETAF ,VFE_RDETAH
USE YOMVV1   , ONLY : VVP00    ,DVALH    ,DVBH
USE YOMCVER  , ONLY : LVERTFE  ,LVSPLIP  ,VRDETAR
USE YOMDYNA  , ONLY : LGWADV   ,LSLHD    ,LSLHDQUAD,SLHDEPSV
USE YOMDYNCORE, ONLY: LDYNCORE, LVERTINI
USE IOSTREAM_MIX , ONLY : INI_IOSTREAM


!**** *SUVERT*  - Routine to initialize vertical coordinate

!     Purpose.
!     --------
!           Initialize the hybrid-cordinate system of the model.

!**   Interface.
!     ----------

!     *CALL* SUVERT
!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        see the modules used above.

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        modified by Radmila Bubnova 92-07-02 (set up vertical system
!           in more complete way)
!        Modified by K. YESSAD and M. HORTAL, NOV and DEC 1992:
!           possible use of an irregularly spaced vertical coordinate ETA,
!           non interpolating vertical semi-lag scheme.
!        modified by Ryad El khatib 93-01-06 (common YOMVV1)
!        Modified by L. Labbe 93-03-13 (Initialize Ai Bi for 2D models)
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!        Modified by R. El Khatib  :93-05-06 Set-up through command line
!        Modified by R. El Khatib  :95-02-06 subroutine SUDEFO_VV1
!        Modified by K. YESSAD (MARCH 1995): add test if correct value for
!                       integer NRLEVX.
!        Modified : 95-11-16 R. El Khatib (Fullpos LFPART2 without arguments)
!        Modified by M. Hortal :2000-05 Setup of vertical finite element scheme
!        Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!         semi-lag advection of vertical divergence in the NH model.
!         => new arrays VCUICOH and NVAUTH.
!        Modified : 04-11-15 by K. YESSAD: add printings for VAF and VBF.
!   15-Feb-2005 K. Yessad  add variable TOPPRES.
!   30-Jun-2008 J. Masek   Auxiliary quantities for SLHD interpolators.
!   15-Sep-2008 K. Yessad  LREGETA -> LREGETA+LVFE_REGETA.
! End Modifications
!-------------------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZVALH(0:NFLEVG), ZVBH(0:NFLEVG), ZVBF(NFLEVG+1)
INTEGER(KIND=JPIM), PARAMETER :: JPKS=KIND(ZVALH)

INTEGER(KIND=JPIM) :: I, IREF, IRLEVXMIN, JL, JLEV, JIT

REAL(KIND=JPRB) :: ZABOT, ZATOP, ZBBOT, ZBTOP, ZD1,&
 & ZD2, ZD3, ZD4, ZD5, ZD6, ZDVETAH, ZEPS, ZFAC  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namvv1.h"

#include "abor1.intfb.h"
#include "sudefo_vv1.intfb.h"
#include "posnam.intfb.h"
#include "suvertfe.intfb.h"
#include "suvsplip.intfb.h"
#include "verint.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVERT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*         1. READ NAMELISTS.
!          ------------------

!        1.1 Set default values and read namelist

ZVALH=0._JPRB
ZVBH=0._JPRB
IF(NFLEVG > JPMXLE) THEN
  WRITE(NULOUT,*) '*** SUVERT: NFLEVG BIGGER THAN EXPECTED LIMIT'
  WRITE(NULOUT,*) JPMXLE,' LEVELS. IF YOUR NFLEVG IS O.K., INCREASE'
  WRITE(NULOUT,*) 'PARAMETER JPMXLE IN PARDIM'
  CALL ABOR1('SUVERT')
ENDIF
CALL SUDEFO_VV1(LECMWF,NFLEVG,ZVALH,ZVBH,VVP00)
DO JLEV=0,NFLEVG
  DVALH(JLEV)=ZVALH(JLEV)
  DVBH(JLEV)=ZVBH(JLEV)
ENDDO
VP00 = VVP00
TOPPRES=0.1_JPRB
IF ((NARGS /= 0.OR.LFPART2).AND..NOT.(LDYNCORE.AND.LVERTINI)) THEN
  DO JLEV=0,NFLEVG
    DVALH(JLEV)=UVALH(JLEV)
    DVBH(JLEV)=UVBH(JLEV)
  ENDDO
ELSE
  IF (NFLEVG > 1) THEN
    CALL POSNAM(NULNAM,'NAMVV1')
    READ(NULNAM,NAMVV1)
  ENDIF
ENDIF

!        1.2 Controls 

ZEPS=EPSILON(1.0_JPRB)*100._JPRB
IF(NFLEVG > 1) THEN
  IF (NARGS /= 0.OR.LFPART2) THEN
    I=0
    DO JLEV=0,NFLEVG
      IF (ABS(REAL(DVALH(JLEV),JPKS)-UVALH(JLEV)) > ZEPS) THEN
        I=1
        DVALH(JLEV)=UVALH(JLEV)
      ENDIF
    ENDDO
    IF(I == 1)WRITE(NULERR,*) 'DVALH OVERWRITTEN BY COMAND LINE'
    I=0
    DO JLEV=0,NFLEVG
      IF (ABS(REAL(DVBH(JLEV),JPKS)-UVBH(JLEV)) > ZEPS) THEN
        I=1
        DVBH(JLEV)=UVBH(JLEV)
      ENDIF
    ENDDO
    IF(I == 1)WRITE(NULERR,*) ' DVBH OVERWRITTEN BY COMAND LINE'
  ENDIF

  ZATOP=REAL(DVALH(0),JPKS)
  ZABOT=REAL(DVALH(NFLEVG),JPKS)
  ZBTOP=REAL(DVBH(0),JPKS)
  ZBBOT=REAL(DVBH(NFLEVG),JPKS)
  IF ((ABS(ZABOT) > ZEPS).OR.&
     & (ABS(ZBTOP) > ZEPS).OR.(ABS(ZBBOT-1.0_JPRB) > ZEPS)) THEN  
    WRITE(NULOUT,*) '*** SUVERT: LIMIT VALUES FOR VERTICAL'
    WRITE(NULOUT,*) 'FUNCTIONS A OR B ARE WRONG. PLEASE CHECK'
    WRITE(NULOUT,*) 'THEM. MAYBE THEN NUMBER OF LEVELS'
    WRITE(NULOUT,*) 'NFLEVG DOES NOT MATCH WITH THE VALUES OF'
    WRITE(NULOUT,*) 'THE FUNCTION A AND B IN NAMELIST'
    CALL ABOR1('SUVERT')
  ENDIF

  WRITE(UNIT=NULOUT,FMT='(1X,''VP00 = '',E13.7)') VP00
  !* The division by VP00 is here just for continuity with ECMWF archives
  DO JLEV=0,NFLEVG
    VAH(JLEV)=REAL(DVALH(JLEV),JPKS)
    VALH(JLEV)=REAL(DVALH(JLEV),JPKS)/VP00
    VBH(JLEV)=REAL(DVBH(JLEV),JPKS)
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*            2. SET UP VERTICAL SYSTEM.
!             --------------------------

!*       2.1   COMPUTE VALUES RELATED TO VERTICAL CO-ORDINATE ETA.
!              IF 3D SEMI-LAGRANGIAN SCHEME.

IF (NFLEVG > 1) THEN
  DO JLEV=0,NFLEVG
    IF (LREGETA) THEN
      VETAH(JLEV)=REAL(JLEV,JPRB)/REAL(NFLEVG,JPRB)
    ELSE
      VETAH(JLEV)=VALH(JLEV)+VBH(JLEV)
    ENDIF
    IF(JLEV > 0) VETAF(JLEV)=(VETAH(JLEV)+VETAH(JLEV-1))*0.5_JPRB
  ENDDO
  VETAF(0)=VETAH(0) ! equal to zero if the top pressure is zero.
  VETAF(NFLEVG+1)=VETAH(NFLEVG) ! equal to one.

  IF(LVERTFE) THEN
    DO JLEV=0,NFLEVG
      IF (LVFE_REGETA) THEN
        VFE_ETAH(JLEV)=REAL(JLEV,JPRB)/REAL(NFLEVG,JPRB)
      ELSE
        VFE_ETAH(JLEV)=VALH(JLEV)+VBH(JLEV)
      ENDIF
      IF(JLEV > 0) VFE_ETAF(JLEV)=(VFE_ETAH(JLEV)+VFE_ETAH(JLEV-1))*0.5_JPRB
    ENDDO
    VFE_ETAF(0)=VFE_ETAH(0) ! equal to zero if the top pressure is zero.
    VFE_ETAF(NFLEVG+1)=VFE_ETAH(NFLEVG) ! equal to one.
    CALL SUVERTFE
  ENDIF
  IF (LSLAG) THEN
    IF(LVSPLIP) THEN
      CALL SUVSPLIP
    ENDIF

    DO JLEV=1,NFLEVG-3
      ZD1=VETAF(JLEV  )-VETAF(JLEV+1)
      ZD2=VETAF(JLEV  )-VETAF(JLEV+2)
      ZD3=VETAF(JLEV  )-VETAF(JLEV+3)
      ZD4=VETAF(JLEV+1)-VETAF(JLEV+2)
      ZD5=VETAF(JLEV+1)-VETAF(JLEV+3)
      ZD6=VETAF(JLEV+2)-VETAF(JLEV+3)
      VCUICO(1,JLEV)= 1.0_JPRB/(ZD1*ZD2*ZD3)
      VCUICO(2,JLEV)=-1.0_JPRB/(ZD1*ZD4*ZD5)
      VCUICO(3,JLEV)= 1.0_JPRB/(ZD2*ZD4*ZD6)
      VCUICO(4,JLEV)=-1.0_JPRB/(ZD3*ZD5*ZD6)
      IF (LSLHDQUAD) THEN
        VSLD(1,JLEV)=(ZD6-ZD2)/ZD4
        VSLD(2,JLEV)=(ZD1-ZD5)/ZD4
        VSLD(3,JLEV)=-ZD4*ZD4/(ZD1*ZD2+ZD5*ZD6)
      ENDIF
      IF (LSLHD) THEN
        VSLDW(1,1,JLEV)=1.0_JPRB-2.0_JPRB*SLHDEPSV*ZD4/ZD1
        VSLDW(1,2,JLEV)=2.0_JPRB*SLHDEPSV*ZD4/ZD5
        VSLDW(1,3,JLEV)=0.0_JPRB
        VSLDW(2,1,JLEV)=2.0_JPRB*SLHDEPSV*ZD4/ZD2
        VSLDW(2,2,JLEV)=1.0_JPRB-2.0_JPRB*SLHDEPSV*ZD4/ZD6
        VSLDW(2,3,JLEV)=0.0_JPRB
        VSLDW(3,1,JLEV)=0.0_JPRB
        VSLDW(3,2,JLEV)=2.0_JPRB*SLHDEPSV*(ZD4*ZD4)/(ZD5*ZD6)
        VSLDW(3,3,JLEV)=1.0_JPRB
      ENDIF
    ENDDO
    IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
      WRITE(NULOUT,*) ''
      WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICO(1,JLEV) '')')
      DO JLEV=1,NFLEVG-3
        WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICO(1,JLEV)
      ENDDO
      WRITE(NULOUT,*) ''
      WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICO(2,JLEV) '')')
      DO JLEV=1,NFLEVG-3
        WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICO(2,JLEV)
      ENDDO
      WRITE(NULOUT,*) ''
      WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICO(3,JLEV) '')')
      DO JLEV=1,NFLEVG-3
        WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICO(3,JLEV)
      ENDDO
      WRITE(NULOUT,*) ''
      WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICO(4,JLEV) '')')
      DO JLEV=1,NFLEVG-3
        WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICO(4,JLEV)
      ENDDO
      IF (LSLHDQUAD) THEN
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VSLD(1,JLEV) '')')
        DO JLEV=1,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VSLD(1,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VSLD(2,JLEV) '')')
        DO JLEV=1,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VSLD(2,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VSLD(3,JLEV) '')')
        DO JLEV=1,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VSLD(3,JLEV)
        ENDDO
      ENDIF
      IF (LSLHD) THEN
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''    VSLDW(1,:,JLEV) '')')
        DO JLEV=1,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))')JLEV,VSLDW(1,:,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''    VSLDW(2,:,JLEV) '')')
        DO JLEV=1,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))')JLEV,VSLDW(2,:,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''    VSLDW(3,:,JLEV) '')')
        DO JLEV=1,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))')JLEV,VSLDW(3,:,JLEV)
        ENDDO
      ENDIF
    ENDIF

    IF (LNHDYN.AND.LGWADV) THEN
      DO JLEV=0,NFLEVG-3
        ZD1=VETAH(JLEV  )-VETAH(JLEV+1)
        ZD2=VETAH(JLEV  )-VETAH(JLEV+2)
        ZD3=VETAH(JLEV  )-VETAH(JLEV+3)
        ZD4=VETAH(JLEV+1)-VETAH(JLEV+2)
        ZD5=VETAH(JLEV+1)-VETAH(JLEV+3)
        ZD6=VETAH(JLEV+2)-VETAH(JLEV+3)
        VCUICOH(1,JLEV)= 1.0_JPRB/(ZD1*ZD2*ZD3)
        VCUICOH(2,JLEV)=-1.0_JPRB/(ZD1*ZD4*ZD5)
        VCUICOH(3,JLEV)= 1.0_JPRB/(ZD2*ZD4*ZD6)
        VCUICOH(4,JLEV)=-1.0_JPRB/(ZD3*ZD5*ZD6)
        IF (LSLHDQUAD) THEN
          VSLDH(1,JLEV)=(ZD6-ZD2)/ZD4
          VSLDH(2,JLEV)=(ZD1-ZD5)/ZD4
          VSLDH(3,JLEV)=-ZD4*ZD4/(ZD1*ZD2+ZD5*ZD6)
        ENDIF
        IF (LSLHD) THEN
          VSLDWH(1,1,JLEV)=1.0_JPRB-2.0_JPRB*SLHDEPSV*ZD4/ZD1
          VSLDWH(1,2,JLEV)=2.0_JPRB*SLHDEPSV*ZD4/ZD5
          VSLDWH(1,3,JLEV)=0.0_JPRB
          VSLDWH(2,1,JLEV)=2.0_JPRB*SLHDEPSV*ZD4/ZD2
          VSLDWH(2,2,JLEV)=1.0_JPRB-2.0_JPRB*SLHDEPSV*ZD4/ZD6
          VSLDWH(2,3,JLEV)=0.0_JPRB
          VSLDWH(3,1,JLEV)=0.0_JPRB
          VSLDWH(3,2,JLEV)=2.0_JPRB*SLHDEPSV*(ZD4*ZD4)/(ZD5*ZD6)
          VSLDWH(3,3,JLEV)=1.0_JPRB
        ENDIF
      ENDDO
      IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICOH(1,JLEV)'')')
        DO JLEV=0,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICOH(1,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICOH(2,JLEV)'')')
        DO JLEV=0,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICOH(2,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICOH(3,JLEV)'')')
        DO JLEV=0,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICOH(3,JLEV)
        ENDDO
        WRITE(NULOUT,*) ''
        WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VCUICOH(4,JLEV)'')')
        DO JLEV=0,NFLEVG-3
          WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VCUICOH(4,JLEV)
        ENDDO
        IF (LSLHDQUAD) THEN
          WRITE(NULOUT,*) ''
          WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VSLDH(1,JLEV)'')')
          DO JLEV=0,NFLEVG-3
            WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VSLDH(1,JLEV)
          ENDDO
          WRITE(NULOUT,*) ''
          WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VSLDH(2,JLEV)'')')
          DO JLEV=0,NFLEVG-3
            WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VSLDH(2,JLEV)
          ENDDO
          WRITE(NULOUT,*) ''
          WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VSLDH(3,JLEV)'')')
          DO JLEV=0,NFLEVG-3
            WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VSLDH(3,JLEV)
          ENDDO
        ENDIF
        IF (LSLHD) THEN
          WRITE(NULOUT,*) ''
          WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''   VSLDWH(1,:,JLEV) '')')
          DO JLEV=0,NFLEVG-3 
            WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))') &
             & JLEV,VSLDWH(1,:,JLEV)
          ENDDO
          WRITE(NULOUT,*) ''
          WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''   VSLDWH(2,:,JLEV) '')')
          DO JLEV=0,NFLEVG-3
            WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))') &
             & JLEV,VSLDWH(2,:,JLEV)
          ENDDO
          WRITE(NULOUT,*) ''
          WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''   VSLDWH(3,:,JLEV) '')')
          DO JLEV=0,NFLEVG-3
            WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))') &
             & JLEV,VSLDWH(3,:,JLEV)
          ENDDO
        ENDIF
      ENDIF
    ENDIF

!           Test if correct value of NRLEVX.

    ZDVETAH=1._JPRB
    DO JLEV=1,NFLEVG
      ZDVETAH=MIN(ZDVETAH,VETAH(JLEV)-VETAH(JLEV-1))
    ENDDO

    IRLEVXMIN=NINT(2.0_JPRB/ZDVETAH)
    IF (NRLEVX < IRLEVXMIN) THEN
      WRITE (NULOUT,*)
      WRITE (NULOUT,*)' NRLEVX (in NAMDIM): INSUFFICIENT VALUE!'
      WRITE (NULOUT,'(A,I10)')' TAKE A VALUE AT LEAST EQUAL TO:',IRLEVXMIN
      WRITE (NULOUT,*)
      CALL ABOR1('SUVERT: ABOR1 CALLED')
    ENDIF

!           Computation of NVAUTF and NVAUTH.

    VRLEVX=REAL(NRLEVX,JPRB)
    ZFAC=(VETAF(NFLEVG+1)-VETAF(0))/VRLEVX

    IREF=1
    DO JL=0,NRLEVX
      IF(JL*ZFAC >= VETAF(IREF+1).AND.IREF < NFLEVG-1) IREF=IREF+1
      NVAUTF(JL)=IREF
    ENDDO

    IF (LNHDYN.AND.LGWADV) THEN
      IREF=0
      DO JL=0,NRLEVX
        IF(JL*ZFAC >= VETAH(IREF+1).AND.IREF < NFLEVG-1) IREF=IREF+1
        NVAUTH(JL)=IREF
      ENDDO
    ENDIF

  ENDIF

  IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN
    WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     ETA            '')')
    DO JLEV=0,NFLEVG
      WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VETAH(JLEV)
    ENDDO
    IF (LVERTFE) THEN
      WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,5X,''     VFE_ETA        '')')
      DO JLEV=0,NFLEVG
        WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,(5X,F20.10))')JLEV,VFE_ETAH(JLEV)
      ENDDO
    ENDIF
  ENDIF

ELSE
!*  Set-up vertical system for 2D models (coherence with SUOPH)
  VAH(0)=0._JPRB
  VAH(1)=0._JPRB
  VALH(0)=0._JPRB
  VALH(1)=0._JPRB
  VBH (0)=0._JPRB
  VBH (1)=1._JPRB
  VETAF(0)=0._JPRB
  VETAF(1)=0.5_JPRB
  VETAF(NFLEVG+1)=1.0_JPRB
  DO JLEV=0,NFLEVG
    VETAH(JLEV)=VALH(JLEV)+VBH(JLEV)
  ENDDO
ENDIF

!*       2.2   COMPUTE VALUES RELATED A AND B.

IF(LVERTFE) THEN
  DO JLEV=1,NFLEVG
    VDELB(JLEV)=VBH(JLEV)-VBH(JLEV-1)
    VFE_RDETAH(JLEV)=1.0_JPRB/(VFE_ETAH(JLEV)-VFE_ETAH(JLEV-1))
    ZVBH(JLEV)=VDELB(JLEV)*VFE_RDETAH(JLEV)
    VDELA(JLEV)=VAH(JLEV)-VAH(JLEV-1)
  ENDDO

  CALL VERINT(1,1,1,NFLEVG,ZVBH(1),ZVBF,0)

! NORMALIZE VDELB
  DO JLEV=1,NFLEVG
    VDELB(JLEV)=VDELB(JLEV)/ZVBF(NFLEVG+1)
    ZVBH(JLEV)=VDELB(JLEV)*VFE_RDETAH(JLEV)
  ENDDO
  CALL VERINT(1,1,1,NFLEVG,ZVBH(1),ZVBF,0)
  DO JLEV=1,NFLEVG
    VBF(JLEV)=ZVBF(JLEV)
  ENDDO
  WRITE(NULOUT,*) ' INTEGRAL OF VDELB=',ZVBF(NFLEVG+1)

! CHECK VDELA
  DO JIT=1,3
    DO JLEV=1,1
      ZVBH(JLEV)=VDELA(JLEV)*VFE_RDETAH(JLEV)
      IF(VDELA(JLEV) < 0.0_JPRB) IREF=JLEV
    ENDDO
    DO JLEV=2,NFLEVG
      ZVBH(JLEV)=VDELA(JLEV)*VFE_RDETAH(JLEV)
      IF(VDELA(JLEV) < 0.0_JPRB .AND. VDELA(JLEV-1) >= 0.0_JPRB) IREF=JLEV
    ENDDO
    CALL VERINT(1,1,1,NFLEVG,ZVBH(1),ZVBF,0)
    WRITE(NULOUT,*) ' ITERATION ',JIT,', INTEGRAL OF VDELA=',ZVBF(NFLEVG+1)
    ZFAC=ZVBF(IREF)/(ZVBF(IREF)-ZVBF(NFLEVG+1))
    DO JLEV=1,IREF
      VDELA(JLEV)=VDELA(JLEV)/ZFAC
      ZVBH(JLEV)=VDELA(JLEV)*VFE_RDETAH(JLEV)
    ENDDO
  ENDDO
  CALL VERINT(1,1,1,NFLEVG,ZVBH(1),ZVBF,0)
  WRITE(NULOUT,*) ' AFTER ADJUSTING, INTEGRAL OF VDELA=',ZVBF(NFLEVG+1)
  DO JLEV=1,NFLEVG
    VAF(JLEV)=ZVBF(JLEV)
  ENDDO

  DO JLEV=1,NFLEVG
    ZVBH(JLEV)=1.0_JPRB
  ENDDO
  CALL VERINT(1,1,1,NFLEVG,ZVBH(1),ZVALH,0)
  WRITE(NULOUT,*) ' INTEGRAL OF 1.=',ZVALH(NFLEVG)

  DO JLEV=1,NFLEVG
    VC(JLEV)=VAH(JLEV)*VBH(JLEV-1)-VAH(JLEV-1)*VBH(JLEV)
  ENDDO
ELSE
  DO JLEV=1,NFLEVG
    VC(JLEV)=VAH(JLEV)*VBH(JLEV-1)-VAH(JLEV-1)*VBH(JLEV)
    VDELB(JLEV)=VBH(JLEV)-VBH(JLEV-1)
  ENDDO
ENDIF

IF(LVSPLIP) THEN
  DO JLEV=NFLEVG,2,-1
    VRDETAR(JLEV)=(VETAH(JLEV)-VETAH(JLEV-1))/(VETAH(JLEV-1)-VETAH(JLEV-2))
  ENDDO
  VRDETAR(1)=1.0_JPRB
ENDIF

IF (LOUTPUT.AND.NPRINTLEV >= 1) THEN

  WRITE(NULOUT,*) ' * A and B on half layers:'
  WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,&
   & 5X,''     ALPHA          '',&
   & 5X,''     B              '',&
   & 5X,''     A              '')')  

  DO JLEV=0,NFLEVG
    WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,3(5X,F20.10))')&
     & JLEV,VALH(JLEV),VBH(JLEV),VAH(JLEV)  
  ENDDO

  IF (LVERTFE) THEN
    WRITE(NULOUT,*)
    WRITE(NULOUT,*) ' * A and B on full layers:'
    WRITE(UNIT=NULOUT,FMT='(1X,''JLEV'',1X,&
     & 5X,''     B              '',&
     & 5X,''     A              '')')  

    DO JLEV=1,NFLEVG
      WRITE(UNIT=NULOUT,FMT='(1X,I3,2X,2(5X,F20.10))')&
       & JLEV,VBF(JLEV),VAF(JLEV)  
    ENDDO
  ENDIF

ENDIF
CALL INI_IOSTREAM(PVAH=VAH,PVBH=VBH)
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVERT',1,ZHOOK_HANDLE)
END SUBROUTINE SUVERT
