SUBROUTINE SUVSPLIP

!**** *SUVSPLIP*  - Setup for vertical cubic spline interpolation

!     Purpose.
!     --------
!           Setup matrix for cubic B-spline interpolation in the vertical

!**   Interface.
!     ----------

!     *CALL* SUVSPLIP
!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        YOMDIM
!        YOMLUN
!        YOMGEM
!        YOMCVER
!        YOMMP

!     Method.
!     -------

!     Externals.
!     ----------
!        MINV

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Agathe Untch ECMWF

!     Modifications.
!     --------------
!        Original : 2000-06
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT   
USE YOMGEM   , ONLY : VETAH    ,VETAF
USE YOMCVER  , ONLY : RVSPTRI   ,RVSPC  , &
 & RFAA     ,RFBB      ,RFCC   ,RFDD  
USE YOMMP    , ONLY : MYPROC

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IDIM,JR,JC,JK,JKC
REAL(KIND=JPRB) :: ZDELTA(-3:NFLEVG+4),ZD1,ZD2,ZD3,ZD4
REAL(KIND=JPRB) :: ZDEL, ZDEL2, ZDEL3
REAL(KIND=JPRB) :: XA,XB,XC,XD, XX
REAL(KIND=JPRB) :: ZAAA2,ZBBB2,ZCCC2,ZDDD2,ZAAA3,ZBBB3,ZCCC3,ZDDD3
REAL(KIND=JPRB) :: ZDDD1,ZDDD4
REAL(KIND=JPRB) :: ZALPHA(4,4), ZBETA(2,2), ZRHS(2), ZBMAX, ZDETB
REAL(KIND=JPRB) :: ZSF(NFLEVG+2,NFLEVG+2),ZSFINV(NFLEVG+2,NFLEVG+2)
REAL(KIND=JPRB) :: ZWORK(2*(NFLEVG+2)), ZEPS, ZDET

LOGICAL :: LLPRINT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVSPLIP',0,ZHOOK_HANDLE)
LLPRINT=.FALSE.

IDIM=NFLEVG+2
ZEPS=100.0_JPRB*TINY(1.0_JPRB)

! compute distance between nodes (full eta levels)

ZDELTA( 0)=2._JPRB*(VETAF(1)-VETAH(0))
ZDELTA(-3)=ZDELTA( 0)
ZDELTA(-2)=ZDELTA( 0)
ZDELTA(-1)=ZDELTA( 0)
DO JK=1,NFLEVG-1
  ZDELTA(JK)=VETAF(JK+1)-VETAF(JK)
ENDDO
ZDELTA(NFLEVG  )=2._JPRB*(VETAH(NFLEVG) -VETAF(NFLEVG))
ZDELTA(NFLEVG+1)=ZDELTA(NFLEVG  )
ZDELTA(NFLEVG+2)=ZDELTA(NFLEVG  )
ZDELTA(NFLEVG+3)=ZDELTA(NFLEVG  )
ZDELTA(NFLEVG+4)=ZDELTA(NFLEVG  )

! compute basis functions    

RFAA(:,:)=0.0_JPRB
RFBB(:,:)=0.0_JPRB
RFCC(:,:)=0.0_JPRB
RFDD(:,:)=0.0_JPRB

ZBMAX=4._JPRB/6._JPRB 
  
DO JK=-1,NFLEVG+2
  ZD1=ZDELTA(JK-2)
  ZD2=ZDELTA(JK-1)
  ZD3=ZDELTA(JK  )
  ZD4=ZDELTA(JK+1)

! matrix alpha
  ZALPHA(1,1)=0.0_JPRB     
  ZALPHA(1,2)=0.0_JPRB      
  ZALPHA(1,3)=ZD3**3      
  ZALPHA(1,4)=ZD4**3+3._JPRB*ZD3*ZD4**2+3._JPRB*ZD3**2*ZD4  
  ZALPHA(2,1)=ZD1**2+2._JPRB*ZD1*ZD2
  ZALPHA(2,2)=ZD2**2
  ZALPHA(2,3)=ZD3**2
  ZALPHA(2,4)=ZD4**2+2._JPRB*ZD3*ZD4
  ZALPHA(3,1)= ZD1
  ZALPHA(3,2)= ZD2
  ZALPHA(3,3)=-ZD3
  ZALPHA(3,4)=-ZD4
  ZALPHA(4,1)=ZD1**3+3._JPRB*ZD1**2*ZD2+3._JPRB*ZD1*ZD2**2
  ZALPHA(4,2)=ZD2**3
  ZALPHA(4,3)=0.0_JPRB
  ZALPHA(4,4)=0.0_JPRB

! matrix beta
  ZBETA(1,1)=ZALPHA(2,2)-ZALPHA(2,1)*ZALPHA(4,2)/ZALPHA(4,1)
  ZBETA(1,2)=ZALPHA(2,3)-ZALPHA(2,4)*ZALPHA(1,3)/ZALPHA(1,4)
  ZBETA(2,1)=ZALPHA(3,2)-ZALPHA(3,1)*ZALPHA(4,2)/ZALPHA(4,1)
  ZBETA(2,2)=ZALPHA(3,3)-ZALPHA(3,4)*ZALPHA(1,3)/ZALPHA(1,4)

! right-hand side vector
  ZRHS(1)=-ZBMAX*(ZALPHA(2,1)/ZALPHA(4,1)+ZALPHA(2,4)/ZALPHA(1,4))
  ZRHS(2)=-ZBMAX*(ZALPHA(3,1)/ZALPHA(4,1)+ZALPHA(3,4)/ZALPHA(1,4))

! determinant of beta 
  ZDETB=ZBETA(1,1)*ZBETA(2,2)-ZBETA(1,2)*ZBETA(2,1)

! coefficients 
  ZDDD2=(ZRHS(1)*ZBETA(2,2)-ZRHS(2)*ZBETA(1,2))/ZDETB
  ZDDD3=(ZRHS(2)*ZBETA(1,1)-ZRHS(1)*ZBETA(2,1))/ZDETB
  ZDDD1=(ZBMAX - ZALPHA(4,2)*ZDDD2)/ZALPHA(4,1)    
  ZDDD4=(ZBMAX - ZALPHA(1,3)*ZDDD3)/ZALPHA(1,4) 

  ZAAA2=        ZD1**3*ZDDD1
  ZBBB2=3._JPRB*ZD1**2*ZDDD1
  ZCCC2=3._JPRB*ZD1   *ZDDD1

  ZAAA3=        ZD4**3*ZDDD4
  ZBBB3=3._JPRB*ZD4**2*ZDDD4
  ZCCC3=3._JPRB*ZD4   *ZDDD4

! coefficients of explicit cubic polynomials in each interval
  
  RFAA(1,JK)=0.0_JPRB
  RFBB(1,JK)=0.0_JPRB
  RFCC(1,JK)=0.0_JPRB
  RFDD(1,JK)=ZDDD1
 
  RFAA(2,JK)=ZAAA2
  RFBB(2,JK)=ZBBB2
  RFCC(2,JK)=ZCCC2
  RFDD(2,JK)=ZDDD2  
  
  ZDEL=ZDELTA(JK)
  ZDEL2=ZDEL*ZDEL
  ZDEL3=ZDEL2*ZDEL
  RFAA(3,JK)=ZAAA3+ZBBB3*ZDEL+        ZCCC3*ZDEL2        +ZDDD3*ZDEL3
  RFBB(3,JK)=     -ZBBB3     -2._JPRB*ZCCC3*ZDEL -3._JPRB*ZDDD3*ZDEL2
  RFCC(3,JK)=                         ZCCC3      +3._JPRB*ZDDD3*ZDEL
  RFDD(3,JK)=                                            -ZDDD3

  ZDEL=ZDELTA(JK+1)
  ZDEL2=ZDEL*ZDEL
  ZDEL3=ZDEL2*ZDEL
  RFAA(4,JK)=         ZDDD4*ZDEL3
  RFBB(4,JK)=-3._JPRB*ZDDD4*ZDEL2
  RFCC(4,JK)= 3._JPRB*ZDDD4*ZDEL
  RFDD(4,JK)=        -ZDDD4 

ENDDO !end loop over nodes

! COMPUTE MATRIX SF ( SF(I,J)=BASISFUNC_J(X_I) )
! ---------------------------------------------

ZSF(:,:)=0.0_JPRB
DO JR=2,IDIM-1
  DO JC=1,IDIM
    JKC=JC-1

    IF(JC == JR-1) ZSF(JR,JC)=RFAA(4,JKC)
    IF(JC == JR  ) ZSF(JR,JC)=RFAA(3,JKC)
    IF(JC == JR+1) ZSF(JR,JC)=RFAA(2,JKC)

  ENDDO
ENDDO

! derivatives

! f' 
!!ZSF(1,1)=RFBB(4,0)
!!ZSF(1,2)=RFBB(3,1)
!!ZSF(1,3)=RFBB(2,2)

! f''
ZSF(1,1)=2.0_JPRB*RFCC(4,0)
ZSF(1,2)=2.0_JPRB*RFCC(3,1)
ZSF(1,3)=2.0_JPRB*RFCC(2,2)

! f'
!!ZSF(IDIM,IDIM-2)=RFBB(4,NFLEVG-1)
!!ZSF(IDIM,IDIM-1)=RFBB(3,NFLEVG  )
!!ZSF(IDIM,IDIM  )=RFBB(2,NFLEVG+1)

! f''
ZSF(IDIM,IDIM-2)=2.0_JPRB*RFCC(4,NFLEVG-1)
ZSF(IDIM,IDIM-1)=2.0_JPRB*RFCC(3,NFLEVG  )
ZSF(IDIM,IDIM  )=2.0_JPRB*RFCC(2,NFLEVG+1)

! invert SF: 
ZDET=0.0_JPRB
ZSFINV(:,:)=ZSF(:,:)
CALL MINV(ZSFINV,IDIM,IDIM,ZWORK,ZDET,ZEPS,0,1)

!  print S and S^-1
IF(MYPROC == 1.AND.LLPRINT) THEN
  WRITE(NULOUT,*) ' DETERMINANT OF SF =',ZDET
  WRITE(NULOUT,*) ' MATRIX SF :'
  DO JR=1,IDIM
    WRITE(NULOUT,'(100F9.3)') (ZSF(JR,JC),JC=1,IDIM)
  ENDDO
  WRITE(NULOUT,*) ' INVERSE OF SF :'
  DO JR=1,IDIM
    WRITE(NULOUT,'(100F9.3)') (ZSFINV(JR,JC),JC=1,IDIM)
  ENDDO
ENDIF

! tridiagonal band of SF for tridiagonal solver

RVSPTRI(:,:)=0.0_JPRB

! diagonal
RVSPTRI(1,0)=ZSF(2,2)-ZSF(2,1)*ZSF(1,2)/ZSF(1,1)
DO JR=2,NFLEVG-1
  RVSPTRI(JR,0)=ZSF(JR+1,JR+1)
ENDDO
RVSPTRI(NFLEVG,0)=ZSF(NFLEVG+1,NFLEVG+1)&
 & -ZSF(NFLEVG+1,NFLEVG+2)*ZSF(NFLEVG+2,NFLEVG+1)/ZSF(NFLEVG+2,NFLEVG+2)  

! lower off-diagonal
DO JR=2,NFLEVG-1
  RVSPTRI(JR,-1)=ZSF(JR+1,JR)
ENDDO
RVSPTRI(NFLEVG,-1)=ZSF(NFLEVG+1,NFLEVG) &
 & -ZSF(NFLEVG+1,NFLEVG+2)*ZSF(NFLEVG+2,NFLEVG)/ZSF(NFLEVG+2,NFLEVG+2)  

! upper off-diagonal
RVSPTRI(1,1)= ZSF(2,3)-ZSF(2,1)*ZSF(1,3)/ZSF(1,1)
DO JR=2,NFLEVG-1
  RVSPTRI(JR,1)=ZSF(JR+1,JR+2)
ENDDO 

IF(MYPROC == 1.AND.LLPRINT) THEN
  WRITE(NULOUT,*) ' tridiagonal band of SF:'
  WRITE(NULOUT,'(100F9.3)') (RVSPTRI(JR,-1),JR=1,NFLEVG)
  WRITE(NULOUT,'(100F9.3)') (RVSPTRI(JR, 0),JR=1,NFLEVG)
  WRITE(NULOUT,'(100F9.3)') (RVSPTRI(JR,+1),JR=1,NFLEVG)
ENDIF

! constants needed in VSPLTRANS

RVSPC(1)=  1.0_JPRB/ZSF(1,1)
RVSPC(2)= -ZSF(1,2)/ZSF(1,1)
RVSPC(3)= -ZSF(1,3)/ZSF(1,1)
RVSPC(4)= -ZSF(2,1)/ZSF(1,1)
RVSPC(5)=  1.0_JPRB/ZSF(NFLEVG+2,NFLEVG+2)
RVSPC(6)= -ZSF(NFLEVG+2,NFLEVG  )/ZSF(NFLEVG+2,NFLEVG+2)
RVSPC(7)= -ZSF(NFLEVG+2,NFLEVG+1)/ZSF(NFLEVG+2,NFLEVG+2)
RVSPC(8)= -ZSF(NFLEVG+1,NFLEVG+2)/ZSF(NFLEVG+2,NFLEVG+2)
RVSPC(9)= 1.0_JPRB/(VETAF(2)-VETAF(1))
RVSPC(10)= 1.0_JPRB/(VETAF(NFLEVG)-VETAF(NFLEVG-1))

IF(MYPROC == 1.AND.LLPRINT) THEN
  WRITE(NULOUT,*) ' constants RVSPC: ',(RVSPC(JR),JR=1,10)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVSPLIP',1,ZHOOK_HANDLE)
END SUBROUTINE SUVSPLIP
