SUBROUTINE CUN1

!**** *CUN1*  - Controls the job to compute unstable modes at level 1

!     Purpose.
!     --------
!           Controls the job at level 1.
!           Compute unstable modes

!**   Interface.
!     ----------
!        *CALL* *CUN1

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.     CUN2   - level 2 control routine for unstable modes
!     ----------     SU1YOM - initialize level 1 commons

!     Reference.
!     ----------
!        ECMWF research department documentation of the
!     "in core model"

!     Author.
!     -------
!        Joseph Tribbia                     *ECMWF*

!     Modifications.
!     --------------
!        Original : 91-10-15
!        Modified by Roberto Buizza 92-02-25
!        Modified by T. Bergot 98-08 : traj full physics for MF
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): prune enhanced diffusion (lfrein).
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT1   , ONLY : N1POS    ,N1GDI    ,N1SDI
USE YOMCT0B  , ONLY : LECMWF
USE YOMPHY   , ONLY : LMPHYS   ,LNEIGE   ,LCVRA    ,LGWD     ,&
 & LHMTO    ,LNEBCO   ,LNEBN    ,LRAY     ,&
 & LSFHYD   ,LSTRA    ,LTHERMO  ,LVDIF  
USE YOMLUN   , ONLY : NULOUT
USE YOMLCZ   , ONLY : NITERL, NWEIGL, XKAPA, L_BALANCED_REDUCTION

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IITERL, IWEIGL
REAL(KIND=JPRB) :: ZKAPA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "balanced_reduction.intfb.h"
#include "cnt2.intfb.h"
#include "cun2.intfb.h"
#include "su1yom.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CUN1',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    Initialize YOMCT1.
!              ------------------

CALL SU1YOM
N1POS=0
N1GDI=0
N1SDI=1
!      ------------------------------------------------------------------

!*       2.    Call level 2 control routine.
!              -----------------------------

!*  2.1 : trajectory computation

IF (.NOT. LECMWF) THEN
  WRITE(NULOUT,'('' >> Sub CUN1 : COMPUTE TRAJECTORY'')')
  CALL CNT2
  LMPHYS=.FALSE.
  LTHERMO=.FALSE.
  LVDIF=.FALSE.
  LRAY=.FALSE.
  LGWD=.FALSE.
  LCVRA=.FALSE.
  LSTRA=.FALSE.
  LSFHYD=.FALSE.
  LNEBCO=.FALSE.
  LNEBN=.FALSE.
  LNEIGE=.FALSE.
  LHMTO=.FALSE.
ENDIF

!*  2.2 : unstable mode

IF (.NOT.L_BALANCED_REDUCTION) THEN
  CALL CUN2
ELSE
  IITERL=NITERL
  IWEIGL=NWEIGL
  ZKAPA=XKAPA
  CALL BALANCED_REDUCTION (XKAPA,IITERL,IWEIGL)
ENDIF
!      ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CUN1',1,ZHOOK_HANDLE)
END SUBROUTINE CUN1
