SUBROUTINE CUN2

!**** *CUN2*  - Level 2 control for unstable mode computation

!     Purpose.
!     --------
!           Compute mean trajectory and call the Lanczos NAG solver

!**   Interface.
!     ----------
!        *CALL* *CUN2*

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.   SU2YOM - initialize level 2 commons
!     ----------   CNT3   - controls direct integration
!                  SUPERT - initialize perturbation
!                  CAIN   - canonical injection
!                  CNT3TL - controls tangent linear integration
!                  CAININ - inverse of the Canonical injection
!                  SCAAS  - scalar product
!                  CNT3AD - controls adjoint integration
!                  SULCZ  - Set up the Lanczos variables
!                  NALAN1 - Connection to Lanczos algorithm
!                  SPNORM -spectral norm computation
!                  RDTLLCZ- read perturbation in grib format
!                  WRTLLCZ- write perturbation in grib format
!                  ERDTLLCZ- read perturbation in grib format (Ald)
!                  EWRTLLCZ- write perturbation in grib format (Ald)

!     Reference.
!     ----------
!        ECMWF research department documentation of the
!     "in core model"

!     Author.
!     -------
!        Roberto Buizza :                   *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-02-25
!        Modification: R Buizza  93-10-27 Covariance matrix constraint
!                      R Buizza  94-01-28 To read trajectory from miotra
!                      R Buizza  94-11-02 To avoid memory overwrite
!                      L Isaksen 96-02-20 Change VATRA to ZVATRA
!                       Buizza   96-06-05 Output in GRIB format 
!                      C Fischer 98-05-05 Add Aladin specific routines
!                      T Bergot  98-08    tests in NALAN1 if not LECMWF
!                                         + traj in cun1
!        R. El Khatib 99-02-05 ESPNORMSM => SPNORM
!                      Y Tremolet 02-08-08 Use control_vectors.
!        C. Fischer 03-09-30 remove call to suellt
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        JJMorcrette 04-04-2005 SRTM
!        M. Janiskova  01-Jan-2006 Set cloud related perturbations to zero
!        M.Hamrud      01-May-2006 Generalized IO scheme
!        M. Leutbecher 01-Nov-2007 SMS event i when initial SVs available
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NFLEVG
USE YOMCT0   , ONLY : LELAM    ,NCONF, LOPDIS
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT1   , ONLY : N1POS    ,N1HIS    ,N1GDI    ,N1SDI
USE YOMVAR   , ONLY : NSIM4D   ,NSIM4DL
USE YOMLCZ   , ONLY : NITERL   ,NEIGEVO  ,XKAPA    ,LANCZOS  ,LEVOLC,&
 & LFORCE   ,LFORCEWR ,LOCNORM  ,LSPTRLC1 ,NWTRMAX1, LSYMCHECK, &
 & NWTRMIN1 ,NLANTYPE, LRAIN    ,LSELSP, L_USE_CONGRAD, &
 & NWEIGL   ,NRITZREAD, NRITZNUMB, NRITZGBH1, NRITZGBH2 , RITZVALS, &
 & LSMSSIG_SVINI
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE YOPHNC   , ONLY : LETRAJP  ,LERADSW2, LENCLD2
USE YOPHLC   , ONLY : LSPHLC
USE YOERAD   , ONLY : NTSW, NSW
USE YOMSP    , ONLY : SPSP
USE YOMVCGL  , ONLY : YVCGLEV, RCGLEV
USE CONTROL_VECTORS_COMM_MOD
USE CONTROL_VECTORS
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM,TYPE_IOSTREAM,CLOSE_IOSTREAM
USE GFL_SUBS_MOD ,ONLY  : DEACT_CLOUD_GFL
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YA, YL, YI, YR, YS
USE YOMMP    , ONLY : MYPROC

IMPLICIT NONE

!     ------------------------------------------------------------------
REAL(KIND=JPRB), ALLOCATABLE :: ZVATRA(:)
TYPE(control_vector) :: YL_ZVEIG
TYPE(control_vector) :: YL_ZVAZX, YL_ZVAZG
TYPE(control_vector) :: YL_ZWORK1, YL_ZWORK2, YL_ZWORK3
TYPE(TYPE_IOSTREAM)  :: YL_IOSTR_SVEVO, YL_IOSTR_SVIFS
REAL(KIND=JPRB) :: ZNORM,ZNORM1,ZNORM2

INTEGER(KIND=JPIM) :: IDIM, IINSV, IITER, IMTRA, IOMASTER,&
 & IOUSV, IRITZREAD, IWEIG, IWRKDIM, JEIG, idum  

REAL(KIND=JPRB) :: ZKAPA, ZPS01, ZPS03, ZRITZ, ZVALU
REAL(KIND=JPRB) :: ZHOOK_HANDLE
CHARACTER (LEN = 20) ::  CLSETEV
CHARACTER (LEN = 256) ::  CLSMSNAME
CHARACTER (LEN = 7) ::  CL_CPENV
PARAMETER ( CL_CPENV = "SMSNAME" )
INTEGER(KIND=JPIM) :: ICPLEN

#include "cain.intfb.h"
#include "cainad.intfb.h"
#include "cainin.intfb.h"
#include "caininad.intfb.h"
#include "cnt3.intfb.h"
#include "cnt3ad.intfb.h"
#include "cnt3tl.intfb.h"
#include "erdtllcz.intfb.h"
#include "ewrtllcz.intfb.h"
#include "lcnorad.intfb.h"
#include "lcnortl.intfb.h"
#include "nalan1.intfb.h"
#include "opk.intfb.h"
#include "rdtllcz.intfb.h"
#include "scaas.intfb.h"
#include "spnorm.intfb.h"
#include "sptrlcz.intfb.h"
#include "su2yom.intfb.h"
#include "suaersn.intfb.h"
#include "suclopn.intfb.h"
#include "supert.intfb.h"
#include "suswn.intfb.h"
#include "wrtllcz.intfb.h"

!     ------------------------------------------------------------------

!*    1. Initializations.
!     -------------------

IF (LHOOK) CALL DR_HOOK('CUN2',0,ZHOOK_HANDLE)
idum=-999

IF(MYPROC==1.AND.LSMSSIG_SVINI)CALL SIGMASTER()
!     1.1 Initialize YOMCT2

CALL SU2YOM


!     1.3 Initialization of the perturbation

CALL ALLOCATE_CTLVEC(YL_ZVAZX)

ZVALU=.000001_JPRB
CALL SUPERT(YL_ZVAZX,ZVALU)

!     1.4 Trajectory computation (if LECMWF else -> cun1 : TB)

IF(LECMWF) THEN
  CALL SPNORM(0)
  N1HIS=1

  WRITE(NULOUT,'('' >> Sub CUN2 : COMPUTE TRAJECTORY'')')
  CALL CNT3

  ! modifications for running TL/AD
  IF (LETRAJP.AND.LSPHLC) THEN
    WRITE(NULOUT,*)&
         & ' LINEAR PHYSICS ACTIVATED ',&
         & ' ==> SIMPLIFIED VERTICAL DIFFUSION SET TO FALSE '  
  ENDIF

  IF (LETRAJP) THEN
    LEPHYS=.TRUE.
    LAGPHY=.TRUE.
    LSPHLC=.FALSE.
        
    IF (LERADSW2 .AND. NSW > 2) THEN
      !       use only 2 spectral intervals for linearized sw
      WRITE(NULOUT,*) 'USING 2 SPEC.INTERVALS FOR LINEARIZED SW'
      NSW=2
      CALL SUSWN   (NTSW, NSW)
      CALL SUCLOPN (NTSW, NSW, NFLEVG)
      CALL SUAERSN (NTSW, NSW)
    ENDIF
  ENDIF

ENDIF
!     -----------------------------------------------------------------

!*    2. Calculation of the modes that maximize the norm defined by SUSCAL
!     --------------------------------------------------------------------

!*    2.1 Define parameters

NSIM4D=0
N1SDI=0
N1GDI=0
N1POS=0
LOPDIS=.false.

!     Re-set some variable used in the prognostic cloud scheme to
!     run TL and AD

WRITE(NULOUT,'('' >> Sub CUN2 '')')

CALL DEACT_CLOUD_GFL

! Put perturbation of cloud variables to zero

IF (LENCLD2) THEN
  GFL(:,:,YA%MP,:) = 0.0_JPRB
  GFL(:,:,YL%MP,:) = 0.0_JPRB
  GFL(:,:,YI%MP,:) = 0.0_JPRB
  GFL(:,:,YR%MP,:) = 0.0_JPRB
  GFL(:,:,YS%MP,:) = 0.0_JPRB
ENDIF

IF (LSYMCHECK) THEN
  WRITE(NULOUT,*)'OPK SYMMETRIC?'

  CALL ALLOCATE_CTLVEC(YL_ZWORK1)
  CALL ALLOCATE_CTLVEC(YL_ZWORK2)
  CALL ALLOCATE_CTLVEC(YL_ZWORK3)

  ZVALU=1.0_JPRB
  CALL SUPERT(YL_ZWORK1,ZVALU)
  ZNORM=DOT_PRODUCT(YL_ZWORK1,YL_ZWORK1)
  YL_ZWORK1%data = YL_ZWORK1%data / SQRT(ZNORM)

  CALL SUPERT(YL_ZWORK2,ZVALU)
  ZNORM=DOT_PRODUCT(YL_ZWORK2,YL_ZWORK2)
  YL_ZWORK2%data = YL_ZWORK2%data / SQRT(ZNORM)

! [OPK w1,w2] = [w1,OPK w2]

  CALL OPK(YL_ZWORK1,YL_ZWORK3)

  ZNORM1=DOT_PRODUCT(YL_ZWORK3,YL_ZWORK2)

  CALL OPK(YL_ZWORK2,YL_ZWORK3)

  ZNORM2=DOT_PRODUCT(YL_ZWORK1,YL_ZWORK3)

  CALL DEALLOCATE_CTLVEC(YL_ZWORK1)
  CALL DEALLOCATE_CTLVEC(YL_ZWORK2)
  CALL DEALLOCATE_CTLVEC(YL_ZWORK3)

  ZNORM=0.5_JPRB*(ZNORM1+ZNORM2)
  IF (ZNORM/=0.0_JPRB) ZNORM=ABS((ZNORM1-ZNORM2)/ZNORM)
  WRITE(NULOUT,*)'SYMMETRIC OPK?: ',ZNORM1,ZNORM2,ZNORM
ENDIF

!*    2.2 Define the initial vector for Lanczos

IF(LANCZOS) THEN

!*    2.3 Lanczos computation of the eigensystem:
!         - define the Lanczos parameters
!           - IDIM    : problem dimension
!           - IWRKDIM : dimension of the working array
!           - IITER   : maximum number of Lanczos iteration
!           - IWEIG   : maximum number of wanted eigenvalues
!           - ZKAPA   : accetable precision for an eigenvalue
!             (see LANNAG documentation for further details)
!         - call routine at level 1 NALAN1

  IDIM=YL_ZVAZX%nsizeg
  IITER=NITERL

!     To avoid the possibility of memory overwriting the maximum 
!     number of wanted eigenvalues has been set equal to the maximum 
!     number of iterations (R Buizza - 02/11/94)

!     Yet another reason to use CONGRAD!!!! (Mike)

  IF (L_USE_CONGRAD) THEN
    IWEIG=NWEIGL
  ELSE
    IWEIG=NITERL
  ENDIF

  IWRKDIM=6*IDIM+10+4*IITER+IITER*IITER
  ZKAPA=XKAPA

  IMTRA=IWRKDIM+2*NITERL+1
  WRITE(NULOUT,*) ' Work array size (ex-NMTRA) = ',IMTRA
  ALLOCATE(ZVATRA(IMTRA))
  CALL ALLGATHER_CTLVEC(YL_ZVAZX, ZVATRA(1:idim))

  CALL NALAN1 (IDIM,IWRKDIM,IITER,IWEIG,ZKAPA,ZVATRA,IMTRA)

  DEALLOCATE(ZVATRA)
  !
  !   inform SMS that initial SVs are available
  ! 
  IF (LSMSSIG_SVINI.AND.(MYPROC == 1)) THEN
    CLSETEV='smsevent i'
    CLSMSNAME="                                             "
    CALL UTIL_CGETENV(CL_CPENV, 'NOSMS', CLSMSNAME, ICPLEN)
    IF (ICPLEN > 0.AND.CLSMSNAME(1:5) /= 'NOSMS') THEN
      CALL SYSTEM(CLSETEV)
      WRITE(UNIT=NULOUT,FMT='('' SMS event i posted '')')
    ELSE
      WRITE(UNIT=NULOUT,&
           & FMT='('' SMS event i '',&
           & '' not posted  because SMSNAME is not defined. LEN='',&
           & I3,A16)') ICPLEN, CLSMSNAME  
    ENDIF
  ENDIF
ENDIF
!     ------------------------------------------------------------------

!*    3. Test the growth rate of some modes
!     -------------------------------------

IF(LEVOLC .AND. LECMWF) THEN

!     Open files: input svifs where SVs(t=0) have been written by NALAN1
!                 output svevo where to write SVs(t=tend)

  IOMASTER=1
  CALL SETUP_IOSTREAM(YL_IOSTR_SVIFS,'CIO','svifs',CDMODE='r',KIOMASTER=IOMASTER,LDNOHELP=.TRUE.)
  CALL SETUP_IOSTREAM(YL_IOSTR_SVEVO,'CIO','svevo',CDMODE='w',KIOMASTER=IOMASTER)

!       Set IRITZREAD equal to the number of computed singular vectors
!       (local sector NSEC1(44)

  IRITZREAD=1
  CALL ALLOCATE_CTLVEC(YL_ZVEIG)
  CALL ALLOCATE_CTLVEC(YL_ZVAZG)

!       ---------------------------------------------------------------
  DO JEIG=1,NEIGEVO
!       ---------------------------------------------------------------
    IF(JEIG <= IRITZREAD) THEN

!     3.2 Read a perturbation from file / memory
      IF (.NOT.LANCZOS) THEN
        IF (LELAM) THEN
          CALL ERDTLLCZ(IINSV,YL_ZVEIG)
        ELSE
          CALL RDTLLCZ(YL_IOSTR_SVIFS,YL_ZVEIG)
        ENDIF
      ELSE
        YL_ZVEIG = YVCGLEV(JEIG)
        ZRITZ = RITZVALS(JEIG)
        NRITZGBH1=NINT( LOG10(ZRITZ)-5 )
        NRITZGBH2=NINT( ZRITZ/(EXP(LOG(10._JPRB)*NRITZGBH1)) )
        NRITZNUMB=JEIG
      ENDIF
!     - control

!     NSEC1(42) = singular vector number
!     NSEC1(54-55) = they are used to store the ritz numbers (set in
!                    NALAN1). Given RITZ, then:
!                    NSEC1(54)=NINT( LOG10(RITZ)-5 )
!                    NSEC1(55)=NINT( RITZ/( EXP(LOG(10.)*NSEC1(54) )
!                    Thus: RITZ=NSEC1(55)*EXP( LOG(10.)*NSEC1(54) )

      WRITE(NULOUT,'('' >> CUN2: JEIG='',I5)') JEIG
      WRITE(NULOUT,'('' SV number -  NSEC1(42)='',I5)')NRITZNUMB
      WRITE(NULOUT,'('' NSEC1(54)='',I10)') NRITZGBH1
      WRITE(NULOUT,'('' NSEC1(55)='',I10)') NRITZGBH2
      ZRITZ=NRITZGBH2*EXP( LOG(10._JPRB)*NRITZGBH1 )
      WRITE(NULOUT,'('' RITZ     ='',E12.5)') ZRITZ

      IRITZREAD=NRITZREAD

!     3.3 Linear model integration

      CALL SCAAS(idum,YL_ZVEIG,YL_ZVEIG,ZPS01)
      CALL CAIN(YL_ZVEIG)
      NSIM4D=NSIM4DL

!     - tendency forcing

      IF (LFORCE) THEN
        SPSP(:)=0.0_JPRB
        LFORCEWR=.TRUE.
        CALL LCNORTL
        LFORCEWR=.FALSE.
        YL_ZVAZX=0.0_JPRB
        CALL CAIN(YL_ZVAZX)
      ENDIF

!     3.3.1 TL integration

      N1HIS=0
      IF (NLANTYPE==10) THEN
        CALL CNT3TL
        CALL CNT3TL
      ELSEIF (NLANTYPE==11) THEN
        CALL CNT3TL
        IF (LOCNORM) CALL LCNORTL
        IF (LSPTRLC1) THEN
          CALL CAININ (YL_ZVAZX)
          YL_ZVAZG=YL_ZVAZX
          CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
          CALL CAIN (YL_ZVAZG)
        ENDIF
        CALL CAININ (YL_ZVAZG)
        CALL CAININAD (YL_ZVAZG)
        IF(LSPTRLC1) THEN
          CALL CAINAD (YL_ZVAZX)
          YL_ZVAZG=YL_ZVAZX
          CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
          CALL CAININAD (YL_ZVAZG)
        ENDIF
        IF(LOCNORM) CALL LCNORAD
        CALL CNT3AD
        YL_ZVAZG%data(:) = 0.0_JPRB
        CALL CAINAD (YL_ZVAZG)
        ZRITZ = RCGLEV(JEIG)
        YL_ZVAZG%data(:) = YL_ZVAZG%data(:)/SQRT(ZRITZ)
        CALL CAIN   (YL_ZVAZG)
      ELSE
        CALL CNT3TL
      ENDIF
      N1HIS=1

!     3.3.2 Local projection operator

!           IF(LOCNORM) THEN
!             CALL LCNORTL
!           ENDIF

      IF(LRAIN) THEN
        WRITE(NULOUT,*) 'PUT RAIN IN SP FIELD'
        SPSP(:)=0.0
        CALL CAININ(YL_ZVAZX)
        LSELSP=.TRUE.
        CALL LCNORTL
        CALL CAININ(YL_ZVAZG)
        YL_ZVAZG%data = YL_ZVAZX%data + YL_ZVAZG%data
        NCONF=131
        CALL CAIN (YL_ZVAZG)
        NCONF=601
      ENDIF

!     3.3.3 Ouput the finale state in unit IEV98

      CALL CAININ(YL_ZVAZG)
      IF (LELAM) THEN
        CALL EWRTLLCZ(IOUSV,YL_ZVAZG)
      ELSE
        CALL WRTLLCZ (YL_IOSTR_SVEVO,YL_ZVAZG)
      ENDIF

      IF (NLANTYPE==11) THEN
        CALL SCAAS(idum,YL_ZVEIG,YL_ZVAZG,ZPS03)
        WRITE(NULOUT,*) 'Inner product of Y and X (should be one)=',ZPS03
      ELSE
        CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZPS03)
        WRITE(NULOUT,'('' Growth rate test '')')
        WRITE(NULOUT,'('' <x;M*x>       = '',E26.19)') ZPS01
        WRITE(NULOUT,'('' <TL*x;M*TL*x> = '',E26.19)') ZPS03
      ENDIF

    ENDIF
!       --------------------------------------------------------------
  ENDDO
!     -----------------------------------------------------------------
  CALL DEALLOCATE_CTLVEC(YL_ZVEIG)
  CALL DEALLOCATE_CTLVEC(YL_ZVAZX)
  CALL DEALLOCATE_CTLVEC(YL_ZVAZG)

!     Close files svifs and svevo

  CALL CLOSE_IOSTREAM(YL_IOSTR_SVIFS)
  CALL CLOSE_IOSTREAM(YL_IOSTR_SVEVO)
  
  IF (ALLOCATED(YVCGLEV) ) THEN
    DO JEIG=1,SIZE(YVCGLEV)
      CALL DEALLOCATE_CTLVEC(YVCGLEV(JEIG))
    ENDDO
    DEALLOCATE (YVCGLEV)
  ENDIF
  IF (ALLOCATED(RCGLEV) )   DEALLOCATE (RCGLEV)
  IF (ALLOCATED(RITZVALS))DEALLOCATE (RITZVALS)
!       --------------------------------------------------------------
ENDIF
!     -----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CUN2',1,ZHOOK_HANDLE)
END SUBROUTINE CUN2
!     -----------------------------------------------------------------

