SUBROUTINE EOF_MATRIX(YD_PVAZIN,YD_PVAZOUT)

!**** *EOF_MATRIX* - Computes resultant control vector for linear operator

!     Purpose.
!     --------
!           For input vector gives the result of applying the adjoint model
!           followed by the tangent linear model, integrated over time.

!**   Interface.
!     ----------
!        *CALL* *EOF_MATRIX(PVAZIN,PVAZOUT)

!        Explicit arguments :
!        --------------------
!        PVAZIN : input variable  (DO NOT MODIFY in the sub)
!        PVAZOUT: output variable

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        CAININ, CMACTL, CAINAD, CMACAD, CAIN, CNT3TL, CNT3AD,

!     Reference.
!     ----------
!        ECMWF research department documentation of the IFS

!     Author.
!     -------
!        Mike Fisher    *ECMWF*  (based on EOF_MATRIX)

!     Modifications.
!     --------------
!        Original : 2003-03-21
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT1   , ONLY : N1HIS
USE YOMCVA   , ONLY :   YSCALPSQRT, YRSCALPSQRT
USE YOMVAR   , ONLY : NSIM4D
USE YOMLCZ   , ONLY : NLANCOUNT,NLANTYPE ,LOCNORM  ,TSTEP_STATE_4D
USE CONTROL_VECTORS

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(IN)    :: YD_PVAZIN 
TYPE(CONTROL_VECTOR),INTENT(OUT)   :: YD_PVAZOUT 
TYPE(control_vector) :: YL_ZVAZX, YL_ZVAZG

INTEGER(KIND=JPIM) ::  idum

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "cain.intfb.h"
#include "cainad.intfb.h"
#include "cainin.intfb.h"
#include "caininad.intfb.h"
#include "cnt3ad.intfb.h"
#include "cnt3tl.intfb.h"
#include "lcnorad.intfb.h"
#include "lcnortl.intfb.h"


!     ------------------------------------------------------------------

!*    1. Define the EOF_MATRIX operator for the Lanczos algorithm
!     -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EOF_MATRIX',0,ZHOOK_HANDLE)
idum=-999

!     Update Lanczos counter

NLANCOUNT=NLANCOUNT+1
WRITE(NULOUT,'('' >> Sub EOF_MATRIX << '')')
WRITE(NULOUT,'('' - Lan counter= '',I5,'' - Int type= '',&
 & I2)') NLANCOUNT,NLANTYPE  

CALL ALLOCATE_CTLVEC(YL_ZVAZX)
CALL ALLOCATE_CTLVEC(YL_ZVAZG)

!     ------------------------------------------------------------------
IF (NLANTYPE==1) THEN
!     ------------------------------------------------------------------

!     NLANTYPE=1: linear and adjoint integration with
!                 - local projection operator
!                 - initial spectral truncation
!                 - final spectral truncation

!     - norm control

  YL_ZVAZX%data =YD_PVAZIN%data  * YRSCALPSQRT%data 

!     - adj inverse canonical inject

  CALL CAININAD (YL_ZVAZX)

!--- Call CNT3TL and throw away the results!
!--- (It should not be necessary to call CNT3TL here. Unfortunately, 
!--- the job fails without it. Presumably, something gets allocated
!--- during CNT3TL and the deallocated during CNT3AD.)

  N1HIS=0
  CALL CNT3TL
  N1HIS=1
  CALL CAININAD (YL_ZVAZX)

!     - adjoint integration (adjoint if calculationg EOFs)

  N1HIS=0
  CALL CNT3AD
  N1HIS=1

!     - call operator to localize the output field into a defined area
!       in gg space

  IF(LOCNORM) CALL LCNORAD

  YL_ZVAZG%data(:) = 0.0_JPRB
  CALL CAINAD (YL_ZVAZG)

!     - Trapezoidal rule integration gives weight 0.5 to end points

  YL_ZVAZG%data(:) = 0.5*YL_ZVAZG%data(:)

!     - adj of inverse of canonical injection

  CALL CAIN (YL_ZVAZG)

!     - adj of localization operator

  IF(LOCNORM) CALL LCNORTL

!     - adj of adj integration (linear integration if calculating EOFs)

  N1HIS=0
  CALL CNT3TL
  N1HIS=1

!     - inverse canonical inject

  CALL CAININ (YL_ZVAZG)

!     - from IFS-variables to pseudo-variables

  YD_PVAZOUT%data = YL_ZVAZG%data * YSCALPSQRT%data

!     - Trapezoidal rule integration. Add end point and multiply by dt

  YL_ZVAZX%data =YD_PVAZIN%data  * YRSCALPSQRT%data 
  CALL CAININAD (YL_ZVAZX)
  IF(LOCNORM) CALL LCNORAD
  YL_ZVAZG%data(:) = 0.0_JPRB
  CALL CAINAD (YL_ZVAZG)
  CALL CAIN (YL_ZVAZG)
  IF(LOCNORM) CALL LCNORTL
  CALL CAININ (YL_ZVAZG)

  YD_PVAZOUT%data = TSTEP_STATE_4D*(YD_PVAZOUT%data + 0.5*YL_ZVAZG%data)

!     ------------------------------------------------------------------
ELSE
  CALL ABOR1 ('EOF_MATRIX CALLED FOR NLANTYPE/=1')
ENDIF

CALL DEALLOCATE_CTLVEC(YL_ZVAZX)
CALL DEALLOCATE_CTLVEC(YL_ZVAZG)

!     ------------------------------------------------------------------
NSIM4D=NSIM4D+1
!     -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('EOF_MATRIX',1,ZHOOK_HANDLE)
END SUBROUTINE EOF_MATRIX
!     -------------------------------------------------------------------

