SUBROUTINE NALAN1 (KDIM,KWRKDIM,KITER,KWEIG,PKAPA,PVATRA,KMTRA)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULUSR1
USE YOMCVA   , ONLY : YRSCALPSQRT,YVAZX, YVAZG
USE YOMLCZ   , ONLY : NLANTYPE ,LANCZOS  ,LNEWNORMT0,LEVOLC, &
                    & L_USE_CONGRAD, L_UNPREDICTABLE_COMPONENTS, &
                    & YV_SUBSPACE, L_SUBSPACE_SVS, N_DIM_SUBSPACE, &
                    & LSPTRLC0,NRITZREAD,NRITZNUMB,NRITZGBH1,NRITZGBH2,RITZVALS
USE YOMMP    , ONLY : MYPROC
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : LELAM    ,NCNTVAR
USE YOMVAR   , ONLY : NSIM4D   ,NSIM4DL, L_INFO_CONTENT
USE YOMCT1   , ONLY : N1HIS
USE YOMCT3   , ONLY : NSTEP
USE YOMVCGL  , ONLY : YVCGLEV   ,RCGLEV   ,NVCGLEV
USE YOMJG    , ONLY : LSUBBG
USE CONTROL_VECTORS_COMM_MOD
USE CONTROL_VECTORS
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM,TYPE_IOSTREAM,CLOSE_IOSTREAM
USE RANDOM_NUMBERS_MIX

!**** *NALAN1* - Level 1 control for unstable modes computation

!     Purpose.
!     --------
!        Make a Lanczos call to compute the eigensystem

!**   Interface.
!     ----------
!        *CALL* *NALAN1 (KDIM,KWRKDIM,KITER,KWEIG,PKAPA)

!        Explicit arguments:
!        -------------------
!        - KDIM    : problem dimension
!        - KWRKDIM : dimension of the Lanczos working array
!        - KITER   : maximum number of Lanczos iterations
!        - KWEIG   : maximum number of wanted eigenvalues
!        - PKAPA   : acceptable ZPRECision for the eigenvalues
!        - PVATRA  : work array used for Lanczos calculations
!        - KMTRA   : dimension of PVATRA

!     Externals:
!     ----------
!        - LANDR
!        - LOADEV

!     Author
!     ------
!        Roberto Buizza *ECMWF*

!     Modification
!     ------------
!        Original : 92-02-25
!        Roberto Buizza : 93-11-12 Introduction of
!                                  the covariance matrix constraint
!        L. Isaksen     : 96-02-20 PVATRA passed as argument
!        R. Buizza      : 96-06-05 Output in GRIB format    
!        T. Bergot      : 98-02    Output in FA format
!        C. Fischer     : 98-05-13 Update for Aladin/LAM model
!        T. Bergot      : 98-08    tests if not LECMWF
!        Y.Tremolet     : 02-08-09 Use control_vectors.
!        C. Fischer     : 03-04-02 echormtl -> echnorm (avec ctl_vec...)
!        M. Leutbecher  : 04-05-07 apply OPK to initial random vector
!        M.Hamrud      01-May-2006 Generalized IO scheme
!        K. Yessad (Sept 2008): adapt to interface of LANDR + cleanings.
!     -----------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KMTRA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM
INTEGER(KIND=JPIM),INTENT(IN)    :: KWRKDIM
INTEGER(KIND=JPIM),INTENT(INOUT) :: KITER
INTEGER(KIND=JPIM),INTENT(IN)    :: KWEIG
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPA
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVATRA(KMTRA)

!     -----------------------------------------------------------------

LOGICAL :: LLOBSC
TYPE(control_vector) :: YL_ZVEIG, YL_ZWRK, YL_ZLANCZOS
TYPE(control_vector) :: YLCGLWK(kiter+1)
TYPE(randomNumberStream) :: YL_RANDOM_STREAM
TYPE(TYPE_IOSTREAM)  ::  YL_IOSTR_SVIFS
REAL(KIND=JPRB) :: ZRITZ(KWEIG), ZBND(KWEIG)
REAL(KIND=JPRB), ALLOCATABLE :: ZRAND(:), ZVECT(:,:)
CHARACTER (LEN = 9) ::  CLFNAME

INTEGER(KIND=JPIM) :: IBND, IEIG, IEIGOK, IERR, IEV, IINSV, ILANSTEP,&
 & IMSGLVL, IOMASTER, IRITZ, IRITZREAD, &
 & ITYPE, JEIG, JN, JS, idum, INEVECS, JVEC, ITER_MIN, jj

REAL(KIND=JPRB) ::  ZCONDM, ZENDL, ZENDR, ZNORM, ZPREC, ZPS01, ZPS03, &
 & ZGOLUBU_INV,ZGOLUBL_INV, ZGOLUBU_LOG,ZGOLUBL_LOG, ZEPS, ZRNORM
REAL(KIND=JPRB) :: zwork(kiter+1,4)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -----------------------------------------------------------------

#include "abor1.intfb.h"
#include "cain.intfb.h"
#include "cainin.intfb.h"
#include "chavar.intfb.h"
#include "chavarad.intfb.h"
#include "chavarin.intfb.h"
#include "chnorm.intfb.h"
#include "cnt3tl.intfb.h"
#include "congrad.intfb.h"
#include "cvaru2i.intfb.h"
#include "echnorm.intfb.h"
#include "ecvaru2i.intfb.h"
#include "ewrtllcz.intfb.h"
#include "ewrtsv.intfb.h"
#include "lcztoald.intfb.h"
#include "lcztoifs.intfb.h"
#include "opk.intfb.h"
#include "scaas.intfb.h"
#include "setran.intfb.h"
#include "wrtllcz.intfb.h"
#include "wrtsv.intfb.h"

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NALAN1',0,ZHOOK_HANDLE)

!     -----------------------------------------------------------------

!*    1. Memory check
!     ---------------

IF(KMTRA<KWRKDIM+2*KITER) CALL ABOR1('NALAN1:ABOR1 CALLED')
!     -----------------------------------------------------------------

idum=-999
!*    2. Compute pointers
!     -------------------

!     Compute pointer for ZRITZ

IRITZ=KWRKDIM+1

!     Compute pointer for ZBND

IBND=KWRKDIM+KITER+1
!     -----------------------------------------------------------------

!*    3. Specify constants for Lanczos algorith (routine LANSO)
!     ---------------------------------------------------------
!        ZCONDM : estimate condition number for M
!        ZENDL  : left end of the interval containing the wanted eigenvalues
!        ZENDR  : right end of the interval containing the wanted eigenvalues
!        IEV    : eigensystem output unit of the Lanczos code (pseudo-variables)
!                 - if IEV.GT.0 both the eigenvalues and the eigenvectors are
!                   wanted, and IEV is the output unit
!                 - if IEV.LE.0 only eigenvalues
!        IMSGLVL:

ZCONDM=1.0_JPRB
ZENDL=-1.E-30_JPRB
ZENDR=1.E-30_JPRB
IEV=NULUSR1
IMSGLVL=5
WRITE(CLFNAME,'(A,I3.3)') 'lczwrk',MYPROC
OPEN(IEV,FORM='UNFORMATTED',FILE=CLFNAME)
!     -----------------------------------------------------------------

!*    4. Make a Lanczos run; see LANDR/CONGRAD for meanings of parameters
!     -----------------------------------------------------------

IF (L_USE_CONGRAD) THEN
 
!--- This particular choice for ZLANCZOS allows CONGRAD to do randomized trace estimation
!--- and estimate the entropy change (information loss) for the forecast.

  CALL ALLOCATE_CTLVEC (YL_ZLANCZOS)
  DO jj=1,kiter+1
    CALL allocate_ctlvec(YLCGLWK(jj))
  ENDDO

  ALLOCATE (ZRAND(KDIM))
  CALL SETRAN (0,YL_RANDOM_STREAM)
  CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
  WHERE (ZRAND(:)>0.5_JPRB)
    ZRAND(:)=1.0_JPRB
  ELSEWHERE
    ZRAND(:)=-1.0_JPRB
  ENDWHERE
  YL_ZLANCZOS = ZRAND(:)
  DEALLOCATE (ZRAND)
  !
  !  apply OPK to start vector to ensure that it is
  !  in the correct subspace
  !  
  IF (.NOT.L_INFO_CONTENT .AND. &
       & (L_SUBSPACE_SVS.OR.LSPTRLC0.OR.LNEWNORMT0)) THEN
    WRITE(nulout,*) 'NALAN1: OPK applied to Lanczos start vector'
    CALL ALLOCATE_CTLVEC(YL_ZWRK)
    WHERE (YRSCALPSQRT%data == 0.0_JPRB)
      YL_ZLANCZOS%data = 0.0_JPRB
    ENDWHERE
    ZRNORM=1.0_JPRB / SQRT(DOT_PRODUCT(YL_ZLANCZOS,YL_ZLANCZOS))
    YL_ZWRK%data=YL_ZLANCZOS%data * ZRNORM
    CALL OPK(YL_ZWRK,YL_ZLANCZOS)  
    CALL DEALLOCATE_CTLVEC(YL_ZWRK)
  ELSE
    WRITE(nulout,*) 'NALAN1: OPK not applied to Lanczos start vector'
  ENDIF

  ! Note, there is again a multiplication with SCALPSQRT_DV
  !  in congrad.

  YL_ZLANCZOS%data = YL_ZLANCZOS%data * YRSCALPSQRT%data

  ZEPS = SQRT(TINY(ZEPS))
  YVAZX%data = 0.0_JPRB
  YVAZG%data = 0.0_JPRB

  INEVECS=KWEIG
  ITER_MIN = KITER
  CALL CONGRAD (YVAZX,YVAZG,YL_ZLANCZOS,ZEPS,PKAPA,KITER,INEVECS, &
   & ITER_MIN, &
   & ZGOLUBU_INV,ZGOLUBL_INV, ZGOLUBU_LOG,ZGOLUBL_LOG, &
   & YLCGLWK,zwork, &
   & LDEVECS=.TRUE., LDSOLVE=.FALSE., LD_LGOLUB=L_INFO_CONTENT, &
   & LD_INFO_CONVERGENCE=.FALSE.)  

  CALL DEALLOCATE_CTLVEC(YL_ZLANCZOS)
  DO jj=1,kiter+1
    CALL deallocate_ctlvec(YLCGLWK(jj))
  ENDDO

  WRITE(NULOUT,'('' >> Sub NALAN1 <<'')')
  WRITE(NULOUT,'('' Lanczos parameters after call CONGRAD'')')
  WRITE(NULOUT,'('' - Lan steps taken      = '',I10)') ILANSTEP
  WRITE(NULOUT,'('' - No. eigenpairs found = '',I10)') NVCGLEV
  WRITE(NULOUT,'('' - accept accuracy      = '',E12.5/)') PKAPA
!!  WRITE (NULOUT,*) 'Entropy increase (upper bound) =',-ZGOLUBU_LOG
!!  WRITE (NULOUT,*) 'Entropy increase (lower bound) =',-ZGOLUBL_LOG

  IRITZREAD = MIN(NVCGLEV,KWEIG)
  ZRITZ(1:IRITZREAD) = RCGLEV(1:IRITZREAD)
ELSE
  LLOBSC=(NLANTYPE>=99)
  CALL LANDR(KDIM,LLOBSC,KITER,KWEIG,ZCONDM,ZENDL,ZENDR,IEV,PKAPA,&
   & ILANSTEP,IEIGOK,PVATRA(IRITZ),PVATRA(IBND),PVATRA(1),&
   & KWRKDIM,IERR,IMSGLVL,MYPROC)  

  WRITE(NULOUT,'('' >> Sub NALAN1 <<'')')
  WRITE(NULOUT,'('' Lanczos parameters after call LANDR'')')
  WRITE(NULOUT,'('' - problem dim          = '',I10)') KDIM
  WRITE(NULOUT,'('' - max num of iter      = '',I10)') KITER
  WRITE(NULOUT,'('' - max num of wanted eig= '',I10)') KWEIG
  WRITE(NULOUT,'('' - Lan steps taken      = '',I10)') ILANSTEP
  WRITE(NULOUT,'('' - OK pairs for machine = '',I10)') IEIGOK
  WRITE(NULOUT,'('' - accept accuracy      = '',E12.5)') PKAPA
  WRITE(NULOUT,'('' - Error status         = '',I10)') IERR
!     -----------------------------------------------------------------

!     Write in the job output all the eigenvalues

  WRITE (NULOUT,'('' J ** RITZ ** EIG=SQRT(ABS(RITZ)) ** BND'')')
  DO JN=1,ILANSTEP
    IF(PVATRA(IBND-1+JN) /= 0) THEN
      ZPREC=PVATRA(IBND-1+JN)/(PKAPA*ABS(PVATRA(IRITZ-1+JN)))
    ELSE
      ZPREC=0.0_JPRB
    ENDIF
    IF(ZPREC <= 1.0_JPRB) THEN
      WRITE(NULOUT,'(2X,I3,3(1X,E20.12),'' ** OK **'',1X,E11.4)')&
       & JN, PVATRA(IRITZ-1+JN),&
       & SQRT(ABS(PVATRA(IRITZ-1+JN))), PVATRA(IBND-1+JN), ZPREC  
    ELSE
      WRITE(NULOUT,'(2X,I3,3(1X,E20.12),''         '',1X,E11.4)')&
       & JN, PVATRA(IRITZ-1+JN),&
       & SQRT(ABS(PVATRA(IRITZ-1+JN))), PVATRA(IBND-1+JN), ZPREC  
    ENDIF
  ENDDO
!     -----------------------------------------------------------------

!*    5. Retrieve the eigensystem (written by NAG RITVEC in unit IEV)
!     ---------------------------------------------------------------
!        KDIM     : dimension of the problem
!        KWEIG    : upper limit to the wanted eigenpairs
!        KITER    : upper limit to the number of Lanczos steps
!        ILANSTEP : number of Lanczos steps actually taken
!        PKAPA    : relative accuracy of the RITZ acceptable as eigenvalue
!        ZRITZ    : eigenvalues
!        ZBND     : error bounds
!        ZVECT    : eigenvectors
!        IERR     : error status

!     5.0 Read using NAG routine LOADEV

  ALLOCATE(ZVECT(KDIM,KWEIG))

  DO JN=1,KDIM
    DO JS=1,KWEIG
      ZVECT(JN,JS)=0.0_JPRB
    ENDDO
  ENDDO
 
  CALL LOADEV(IEV,KDIM,KWEIG,IRITZREAD,ZRITZ,ZBND,ZVECT,IERR,MYPROC,LLOBSC)

  WRITE(NULOUT,'('' >> Sub NALAN1 <<'')')
  WRITE(NULOUT,'('' Lanczos parameters after call LOADEV'')')
  WRITE(NULOUT,'('' - read eigenvectors = '',I10)') IRITZREAD
  WRITE(NULOUT,'('' - error status      = '',I10)') IERR
ENDIF
!     -----------------------------------------------------------------

!     Open svifs file svifs where SVs(t=0) are going to be written

IF (LECMWF) THEN
  IOMASTER=1
  CALL SETUP_IOSTREAM(YL_IOSTR_SVIFS,'CIO','svifs',CDMODE='w',KIOMASTER=IOMASTER)


!     Define the number of SVs computed and set sector parameter

!     NSEC1(44) = number of SVs computed (to be set later in NALAN1
!                 after call to LANCZOS code)

  NRITZREAD=IRITZREAD

!     IRITZREAD: number of acceptable eigenvectors read from the NAG
!                Lanczos working unit IEV

ENDIF

! Deallocate subspace vectors if used

IF (L_SUBSPACE_SVS) THEN
  DO JVEC=1, N_DIM_SUBSPACE
    CALL DEALLOCATE_CTLVEC(YV_SUBSPACE(JVEC))
  ENDDO
  DEALLOCATE(YV_SUBSPACE)
ENDIF

CALL ALLOCATE_CTLVEC(YL_ZVEIG)
CALL ALLOCATE_CTLVEC(YL_ZWRK)

!     Allocate memory to save vectors between nalan2 and cun2.

ALLOCATE (RITZVALS(IRITZREAD))
NVCGLEV = IRITZREAD

IF (.NOT.L_USE_CONGRAD) THEN
  ALLOCATE (YVCGLEV(IRITZREAD))
  DO JEIG =1,IRITZREAD
    CALL ALLOCATE_CTLVEC (YVCGLEV(JEIG))
  ENDDO

  ALLOCATE (RCGLEV(IRITZREAD))
ENDIF

!     -----------------------------------------------------------------
!     -----------------------------------------------------------------
DO JEIG =1,IRITZREAD
!     -----------------------------------------------------------------
!     -----------------------------------------------------------------

!     -----------------------------------------------------------------
  IF (L_USE_CONGRAD) THEN
    IEIG=JEIG
    YL_ZVEIG=YVCGLEV(JEIG)
  ELSE
!     Invert the ordering: input IEV (NAG-Lanczos) from the slowest to the fastest
    IEIG=IRITZREAD-JEIG+1
    YL_ZVEIG=ZVECT(:,IEIG)
  ENDIF

!     5.1 Coordinate transformation

  IF(NLANTYPE==1 .OR. NLANTYPE==10 .OR. NLANTYPE==11) THEN

!     - from pseudo-variables to IFS-variables

    YL_ZVEIG%data = YL_ZVEIG%data * YRSCALPSQRT%data 

!     - from eigenvectors of B^(1/2) M^* B^(-1) M B^(1/2) to (un)predictable components

    IF (L_UNPREDICTABLE_COMPONENTS) THEN
      NCNTVAR=2
      CALL CAININ (YL_ZVEIG)
      LSUBBG=.FALSE.
      CALL CHAVARIN (YL_ZVEIG)
      NCNTVAR=1
      CALL CNT3TL
      NCNTVAR=2
      CALL CHAVAR (YL_ZVEIG)
      CALL CHAVARAD(YL_ZVEIG)
      CALL CAININ (YL_ZVEIG)
      NCNTVAR=1
    ENDIF

!     - redefinition of norm at t0

    IF (LNEWNORMT0) THEN
      YL_ZWRK=YL_ZVEIG
      IF (LELAM) THEN
        CALL ECHNORM(YL_ZWRK,YL_ZVEIG)
      ELSE
        CALL CHNORM(YL_ZWRK,YL_ZVEIG)
      ENDIF
    ENDIF
  ELSEIF(NLANTYPE == 2.OR.NLANTYPE == 3) THEN 

!     - coord transf from real to complex [ R(-1/2) ]

    ITYPE=1
    IF (LELAM) THEN
      CALL LCZTOALD (ITYPE,YL_ZVEIG,YL_ZWRK)
    ELSE
      CALL LCZTOIFS (ITYPE,YL_ZVEIG,YL_ZWRK)
    ENDIF

!     - coordinate transformation from the sub-space where the pdf is
!       isotropic to model variables [ V ]

    CALL CAIN (YL_ZWRK)
    CALL ABOR1('NALAN1: THIS CALL TO CVARU3I IS BROKEN')
!   CALL CVARU3I(YVAZX)

!     - from P to T,lnps variables

    IF (LELAM) THEN
      CALL ECVARU2I
    ELSE
      CALL CVARU2I
    ENDIF

    CALL CAININ (YL_ZVEIG)

!     - renormalize vectors so that to have unitary energy norm

    CALL SCAAS(idum,YL_ZVEIG,YL_ZVEIG,ZNORM)
    ZNORM=SQRT(ZNORM)
    WRITE(NULOUT,'('' >> Sub NALAN1 - Normalization'')')
    WRITE(NULOUT,'('' JEIG, IEIG, SQRT(<V;E*V>)= '',E12.5)')ZNORM
    YL_ZVEIG%data = YL_ZVEIG%data / ZNORM
  ENDIF

  IF (LECMWF) THEN
!     Define the SVs ordering number and the eigenvalue 
!     Set sector parameter

!     NSEC1(42) = singular vector number 
!     NSEC1(54-55) = they are used to store the ritz numbers (set in
!                    NALAN1). Given RITZ, then:
!                    NSEC1(54)=NINT( LOG10(RITZ)-5 )
!                    NSEC1(55)=NINT( RITZ/( EXP(LOG(10.)*NSEC1(54) )
!                    Thus: RITZ=NSEC1(55)*EXP( LOG(10.)*NSEC1(54) )

    NRITZNUMB=JEIG
    NRITZGBH1=NINT( LOG10(ZRITZ(IEIG))-5 )
    NRITZGBH2=NINT( ZRITZ(IEIG)/(EXP(LOG(10._JPRB)*NRITZGBH1)) )

!     For NLANTYPE 11, half-evolve the vector to get the X vectors

    IF (NLANTYPE==11) THEN
      CALL CAIN(YL_ZVEIG)
      N1HIS=0
      CALL CNT3TL
      N1HIS=1
      CALL CAININ(YL_ZVEIG)
      WHERE (YRSCALPSQRT%data(:)==0.0_JPRB) YL_ZVEIG%data(:)=0.0_JPRB
      YL_ZVEIG%data(:) = YL_ZVEIG%data(:) / SQRT(SQRT(ZRITZ(IEIG)))
    ENDIF

!     5.2 Write the results in the output file IEV99 changing the output
!         format (from IFS format to mars or FA format)

    if (LELAM) then
      CALL ABOR1('Can LELAM be true if LECMWF is ?')
      call EWRTLLCZ(IINSV,YL_ZVEIG)
    else
      call WRTLLCZ (YL_IOSTR_SVIFS,YL_ZVEIG)
    endif

!--- save the eigenpair and grib header in memory for communication with the cun2

    YVCGLEV(JEIG) = YL_ZVEIG
    RCGLEV(JEIG) = ZRITZ(IEIG)
    RITZVALS(JEIG) = ZRITZ(IEIG)
  ELSE

! ecriture du SV numero jeig en fichier FA a t=0

    if (LELAM) then
      call EWRTSV(YL_ZVEIG,JEIG)
    else
      call WRTSV(YL_ZVEIG,JEIG)
    endif

! test de croissance et ecriture en fichier FA a t
    IF (LEVOLC) then

!       Linear model integration

      CALL SCAAS(idum,YL_ZVEIG,YL_ZVEIG,ZPS01)
      CALL CAIN(YL_ZVEIG)
      NSIM4D=NSIM4DL

!      TL integration

      N1HIS=0
      CALL CNT3TL
      N1HIS=1

!      Local projection operator

!           IF(LOCNORM) THEN
!             CALL LCNORTL
!           ENDIF

!      Ouput the finale state in FA file

      CALL CAININ(YL_ZVEIG)
      IF (LELAM) THEN
        CALL EWRTSV(YL_ZVEIG,JEIG)
      ELSE
        CALL WRTSV(YL_ZVEIG,JEIG)
      ENDIF
      NSTEP=0

!      test output

      CALL SCAAS(idum,YL_ZVEIG,YL_ZVEIG,ZPS03)
      WRITE(NULOUT,'('' Growth rate test '')')
      WRITE(NULOUT,'('' <x;M*x>       = '',E26.19)') ZPS01
      WRITE(NULOUT,'('' <TL*x;M*TL*x> = '',E26.19)') ZPS03
    ENDIF
  ENDIF

!     ------------------------------------------------------------------
!     ------------------------------------------------------------------
ENDDO
!     ------------------------------------------------------------------
!     ------------------------------------------------------------------
CALL DEALLOCATE_CTLVEC(YL_ZVEIG)
CALL DEALLOCATE_CTLVEC(YL_ZWRK)

!     Close svifs file

IF (LECMWF) THEN
  CALL CLOSE_IOSTREAM(YL_IOSTR_SVIFS)
ENDIF
!     ------------------------------------------------------------------

IF (ALLOCATED(ZVECT)) DEALLOCATE(ZVECT)

!     -----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('NALAN1',1,ZHOOK_HANDLE)
END SUBROUTINE NALAN1
!     -----------------------------------------------------------------

