SUBROUTINE OPK(YD_PVAZIN,YD_PVAZOUT)

!**** *OPK* - Computes resultant control vector for linear operator

!     Purpose.
!     --------
!           For input vector gives the result of tangent linear,adjoint,
!        or combination of these operators acting on the input vector.

!**   Interface.
!     ----------
!        *CALL* *OPK(YD_PVAZIN,YD_PVAZOUT)

!        Explicit arguments :
!        --------------------
!        YD_PVAZIN : input variable  (DO NOT MODIFY in the sub)
!        YD_PVAZOUT: output variable

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        CAININ, CMACTL, CAINAD, CMACAD, CAIN, CNT3TL, CNT3AD,
!        (E)CHNORM,(E)SPTRLCZ

!     Reference.
!     ----------
!        ECMWF research department documentation of the IFS

!     Author.
!     -------
!        Joseph Tribbia & Roberto Buizza    *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-02-25
!        Roberto Buizza : 93-10-25 Covariance matrix constraint when
!                                  computing optimal perturbations
!        Roberto Buizza : 96-02-15 New op to re-define norm at t0
!        Jan Barkmeijer : 96-10-21 option to run in control space
!        Mike Fisher    : 97-12-03 spectral SCALP for NLANTYPE=5,6,8,9
!        Jan Barkmeijer : 00-11-06 option LFORCE to compute forcing SVs
!        Y. Tremolet    : 02-08-08 use control_vectors
!        Claude Fischer : 02-09-11 merge Aladin LAM with global IFS version
!                                  (only for nlantype=1 )
!        Mike Fisher    : 01-05-15 Hankel Singular vectors (NLANTYPE=10,11)
!        Mike Fisher    : 03-03-19 Stochastic Optimals, EOFs and balanced truncation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M Leutbecher  12-May-2004 subspace singular vectors
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NCONF    ,NSTART   ,NSTOP    ,LOBS     ,LELAM ,&
 & LOBSC1   ,LOBSREF  ,LSIMOB  ,NCNTVAR  
USE YOMCT1   , ONLY : N1HIS
USE YOMCVA   , ONLY : YSCALP ,YRSCALP, YSCALPSQRT, YRSCALPSQRT
USE YOMVAR   , ONLY : NSIM4D
USE YOMLCZ   , ONLY : NLANCOUNT,NLANTYPE ,NWTRMIN0 ,NWTRMAX0 ,&
 & NWTRMIN1 ,NWTRMAX1 ,LANCZOS  ,LOCNORM  ,LSPTRLC0 ,&
 & LSPTRLC1 ,LNEWNORMT0,LFORCE  ,LFORCEWR, L_UNPREDICTABLE_COMPONENTS ,&
 & L_SOS, L_EOFS, TSTEP_STATE_4D, &
 & L_SUBSPACE_SVS,N_DIM_SUBSPACE, YV_SUBSPACE

USE YOMCT2   , ONLY : NSTAR2   ,NSTOP2   ,NSTP
USE YOMCT3   , ONLY : NSTEP
USE YOMJG    , ONLY : LSUBBG
USE YOMVRTL  , ONLY : L131TL   ,LOBSTL
USE YOMSKF   , ONLY : LSKF
USE YOMSP    , ONLY : SPQ
USE YEMCT0   , ONLY : NECRIPL  ,NECOTL   ,NECOAD
USE CONTROL_VECTORS

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN)   :: YD_PVAZIN
TYPE(CONTROL_VECTOR), INTENT(INOUT)  :: YD_PVAZOUT
TYPE(control_vector) :: YL_ZWEIGHTS, YL_ZZERO
TYPE(control_vector) :: YL_ZVAZX, YL_ZVAZG, YL_ZVAZY, YL_ZVAZH

INTEGER(KIND=JPIM) :: ILANTYPE, ITYPE, idum, JS

LOGICAL :: LLSKF

REAL(KIND=JPRB) ::  ZNORMV0, ZNORMV0LAN, ZNORMV1, ZNORMV2,&
 & ZNORMV3, ZNORMV4, ZNORMV5, ZNORMV6, ZNORMV6LAN, &
 & ZNORMV7, ZTHETA1, ZTHETA2, ZTHETA3, ZTHETA4, &
 & ZALPHA,ZNORMV8,ZNORMV9,ZNORMV10
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cain.intfb.h"
#include "cainad.intfb.h"
#include "cainin.intfb.h"
#include "caininad.intfb.h"
#include "chavar.intfb.h"
#include "chavarad.intfb.h"
#include "chavarin.intfb.h"
#include "chavarinad.intfb.h"
#include "chnorm.intfb.h"
#include "cnt3ad.intfb.h"
#include "cnt3tl.intfb.h"
#include "cvaru2i.intfb.h"
#include "cvaru2iad.intfb.h"
#include "echnorm.intfb.h"
#include "eof_matrix.intfb.h"
#include "esptrlcz.intfb.h"
#include "lcnorad.intfb.h"
#include "lcnortl.intfb.h"
#include "lcztoifs.intfb.h"
#include "opm.intfb.h"
#include "scaas.intfb.h"
#include "sptrlcz.intfb.h"
#include "su2yom.intfb.h"
#include "suscal.intfb.h"

!     ------------------------------------------------------------------

!*    0. Variable initialization

IF (LHOOK) CALL DR_HOOK('OPK',0,ZHOOK_HANDLE)
ZNORMV0LAN=9999._JPRB
ZNORMV0=9999._JPRB
ZNORMV1=9999._JPRB
ZNORMV7=9999._JPRB
ZNORMV2=9999._JPRB
ZNORMV3=9999._JPRB
ZNORMV4=9999._JPRB
ZNORMV5=9999._JPRB
ZNORMV6=9999._JPRB
ZNORMV6LAN=9999._JPRB
ZNORMV8=9999._JPRB
ZNORMV9=9999._JPRB
ZNORMV10=9999._JPRB
ZTHETA1=9999._JPRB
ZTHETA2=9999._JPRB
ZTHETA3=9999._JPRB
ZTHETA4=9999._JPRB
!     ------------------------------------------------------------------

!*    1. Define the OPK operator for the Lanczos algorithm
!     ----------------------------------------------

idum=-999

!     In case where no specific treatment of the lateral boundaries
!     is foreseen, either in TL or AD model,
!     force coupling at t0 (coupling at t+dt is then inconsistent)

IF (LELAM.AND.(NECOTL == 0.OR.NECOAD == 0)) THEN
  NECRIPL=0
ENDIF

!     Update Lanczos counter

NLANCOUNT=NLANCOUNT+1
WRITE(NULOUT,'('' >> Sub OPK << '')')
WRITE(NULOUT,'('' - Lan counter= '',I5,'' - Int type= '',&
 & I2)') NLANCOUNT,NLANTYPE  

IF (.NOT.L_EOFS) THEN
  CALL ALLOCATE_CTLVEC(YL_ZVAZX)
  CALL ALLOCATE_CTLVEC(YL_ZVAZG)
ENDIF

!     ------------------------------------------------------------------
IF (L_EOFS) THEN

!     EOF MATRIX  (integral over time of M M*)

  CALL EOF_MATRIX (YD_PVAZIN,YD_PVAZOUT)

ELSEIF (NLANTYPE==1 .OR. NLANTYPE==10 .OR. NLANTYPE==11) THEN
!     ------------------------------------------------------------------

!     NLANTYPE=1: linear and adjoint integration with
!                 - local projection operator
!                 - initial spectral truncation
!                 - final spectral truncation
!     NLANTYPE=10: singular vectors of M^2 (pseudo-Hankel singular vectors)
!     NLANTYPE=11: pseudo-Hankel balancing transformation vectors

!     Solve the eigenvalue problem of the operator:
!     E(-1/2).CN0*.S0*.L*.T*.S1*.E.S1.T.L.S0.CN0.E(-1/2)=
!     = E(1/2).E(-1).CN0*.S0*.L*.T*.S1*.E.S1.T.L.S0.CN0.E(-1/2)

!     - coordinate transformation from variables normalized with respect
!       to the canonical Euclidean product (as in the Lanczos code), to
!       variables normalized with respect to the SCALP norm ( as in IFS)
!       [ E(-1/2) ]
!     - canonical injection
!     - re-definition of norm at t0 [ CN0 ]
!     - spectral truncation [ S0 ]
!     - linear integration [ L ]
!     - gg localization [ T ]
!     - spectral truncation [ S1 ]
!     - inv of canonical inj
!     - adjoint of the inverse of the canonical injection [ E ]
!     - adj of spectral truncation [ S1* ]
!     - adj of gg localization [ T* ]
!     - adj of linear integration [ L* ]
!     - adj of spectral truncation [ S0* ]
!     - adj of re-definition of norm at t0 [ CN0* ]
!     - adj of can inj [ E(-1) ]
!     - inverse of coord transformation [ E(1/2) ]

!     - norm control

  ZNORMV0LAN=DOT_PRODUCT(YD_PVAZIN,YD_PVAZIN)

!     - project into subspace
  YL_ZVAZX=YD_PVAZIN
  IF (L_SUBSPACE_SVS) THEN
    WRITE(NULOUT,*) 'OPK: projection in subspace; JS, ZALPHA '
    DO JS=1, N_DIM_SUBSPACE
      ZALPHA=DOT_PRODUCT(YD_PVAZIN,YV_SUBSPACE(JS))
      YL_ZVAZX%data = YL_ZVAZX%data - ZALPHA * YV_SUBSPACE(JS)%data
      write(nulout,'(I3,X,E12.4)') JS, ZALPHA
    ENDDO
    ZNORMV8=DOT_PRODUCT(YL_ZVAZX,YL_ZVAZX)
  ENDIF
!     - from pseudo-variables to IFS-variables in complex space

  YL_ZVAZX%data =YL_ZVAZX%data  * YRSCALPSQRT%data 

!     - norm control

  CALL SCAAS(idum,YL_ZVAZX,YL_ZVAZX,ZNORMV0)

!     - canonical inject

  CALL CAIN (YL_ZVAZX)

!     - re-definition of norm at t0 

  IF (LNEWNORMT0) THEN
    CALL CAININ (YL_ZVAZX)
    IF (.NOT.LELAM) THEN
      CALL CHNORM(YL_ZVAZX,YL_ZVAZG)
    ELSE
      CALL ECHNORM(YL_ZVAZX,YL_ZVAZG)
    ENDIF
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    IF (.NOT.LELAM) THEN
      CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    ELSE
      CALL ESPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    ENDIF
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV7)
  ENDIF

!     - tendency forcing

  IF (LFORCE) THEN
    LFORCEWR=.TRUE.
    CALL LCNORTL
    LFORCEWR=.FALSE.
    CALL ALLOCATE_CTLVEC(YL_ZZERO)
    YL_ZZERO=0.0_JPRB
    CALL CAIN(YL_ZZERO)
    CALL DEALLOCATE_CTLVEC(YL_ZZERO)
  ENDIF

!     - sqrt(background covariance)

  IF (L_UNPREDICTABLE_COMPONENTS) THEN
    CALL CAININ (YL_ZVAZG)
    NCNTVAR=2
    LSUBBG=.FALSE.
    CALL CHAVARIN (YL_ZVAZG)
  ENDIF

!     - linear integration (adjoint if calculating EOFs)

  N1HIS=0
  CALL CNT3TL
  N1HIS=1

!     - inverse(sqrt(background covariance))

  IF (L_UNPREDICTABLE_COMPONENTS) THEN
    NCNTVAR=2
    LSUBBG=.FALSE.
    CALL CHAVAR (YL_ZVAZG)
    CALL CAIN (YL_ZVAZG)
  ENDIF

  IF (NLANTYPE==10 .OR. NLANTYPE==11) THEN
    CALL CAININ (YL_ZVAZX)
    WHERE (YSCALPSQRT%data(:)==0.0_JPRB) YL_ZVAZX%data(:)=0.0_JPRB
    CALL CAIN (YL_ZVAZX)
    CALL CNT3TL
  ENDIF
  N1HIS=1

!     - norm control

  CALL CAININ (YL_ZVAZG)
  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV2)

!     - call operator to localize the output field into a defined area
!       in gg space

  IF(LOCNORM) THEN
    CALL LCNORTL

!     - norm control

    CALL CAININ (YL_ZVAZG)
    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV3)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    IF (.NOT.LELAM) THEN
      CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    ELSE
      CALL ESPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    ENDIF
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV4)
  ENDIF

!     - inv of canonical inj

  CALL CAININ (YL_ZVAZG)

!     - norm control

  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV5)


!     - Trapezoidal rule integration gives weight 0.5 to end points

  IF (L_SOS) THEN
    YL_ZVAZG%data(:) = 0.5*YL_ZVAZG%data(:)
  ENDIF

!     - adj of inverse of canonical injection

  CALL CAININAD (YL_ZVAZG)

!     - adj of spectral truncation

  IF(LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    IF (.NOT.LELAM) THEN
      CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    ELSE
      CALL ESPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    ENDIF
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of localization operator

  IF(LOCNORM) CALL LCNORAD

!     - adj inverse(sqrt(background covariance))

  IF (L_UNPREDICTABLE_COMPONENTS) THEN
    CALL CAININ (YL_ZVAZX)
    NCNTVAR=2
    LSUBBG=.FALSE.
    CALL CHAVARAD (YL_ZVAZX)
  ENDIF

!     - adj of linear integration (linear integration if calculating EOFs)

  N1HIS=0
  CALL CNT3AD
  N1HIS=1

!     - adj sqrt(background covariance)

  IF (L_UNPREDICTABLE_COMPONENTS) THEN
    YL_ZVAZX=0.0_JPRB
    NCNTVAR=2
    LSUBBG=.FALSE.
    CALL CHAVARINAD (YL_ZVAZX)
    CALL CAIN (YL_ZVAZX)
  ENDIF

  IF (NLANTYPE==10 .OR. NLANTYPE==11) THEN
    CALL CAININ (YL_ZVAZG)
    WHERE (YSCALPSQRT%data(:)==0.0_JPRB) YL_ZVAZG%data(:)=0.0_JPRB
    CALL CAIN (YL_ZVAZG)
    CALL CNT3AD
  ENDIF
  N1HIS=1

!     - adj of spectral truncation

  IF(LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    IF (.NOT.LELAM) THEN
      CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    ELSE
      CALL ESPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    ENDIF
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of re-definition of norm at t0 

  IF (LNEWNORMT0) THEN
    CALL CAININ (YL_ZVAZX)
    IF (.NOT.LELAM) THEN
      CALL CHNORM(YL_ZVAZX,YL_ZVAZG)
    ELSE
      CALL ECHNORM(YL_ZVAZX,YL_ZVAZG)
    ENDIF
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of canonical injection

  YL_ZVAZG=0.0_JPRB
  CALL CAINAD (YL_ZVAZG)

!     - norm control

  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV6)

!     - from IFS-variables to pseudo-variables

  YD_PVAZOUT%data = YL_ZVAZG%data * YSCALPSQRT%data


!     - (adjoint of ) project into subspace
  ZNORMV9=DOT_PRODUCT(YD_PVAZOUT,YD_PVAZOUT)
  IF (L_SUBSPACE_SVS) THEN
    YL_ZVAZX=YD_PVAZOUT
    WRITE(NULOUT,*) 'OPK: (adj) projection in subspace; JS, ZALPHA '
    DO JS=1, N_DIM_SUBSPACE
      ZALPHA=DOT_PRODUCT(YD_PVAZOUT,YV_SUBSPACE(JS))
      YL_ZVAZX%data = YL_ZVAZX%data - ZALPHA * YV_SUBSPACE(JS)%data
      write(nulout,'(I3,X,E12.4)') JS, ZALPHA
    ENDDO
    YD_PVAZOUT=YL_ZVAZX
    ZNORMV10=DOT_PRODUCT(YD_PVAZOUT,YD_PVAZOUT)
  ENDIF

!     - Trapezoidal rule integration. Add end point and multiply by dt

  IF (L_SOS) THEN
    YL_ZVAZX%data =YD_PVAZIN%data  * YRSCALPSQRT%data 
    CALL CAIN (YL_ZVAZX)
    IF (LOCNORM) CALL LCNORTL
    CALL CAININ (YL_ZVAZG)
    YL_ZVAZG%data = 0.5 * YL_ZVAZG%data
    CALL CAININAD (YL_ZVAZG)
    IF (LOCNORM) CALL LCNORAD
    CALL CAINAD (YL_ZVAZG)
    YL_ZVAZX%data =YD_PVAZIN%data  * YSCALPSQRT%data
    YD_PVAZOUT%data = TSTEP_STATE_4D*(YD_PVAZOUT%data + 0.5*YL_ZVAZG%data(:))
  ENDIF

!     - norm control

  ZNORMV6LAN=DOT_PRODUCT(YD_PVAZOUT,YD_PVAZOUT)

!     - output the norm control

  WRITE(NULOUT,'('' >> Sub OPK <<'')')
  WRITE(NULOUT,'('' Norm2 V Lanczos: |V|2lan=<V;V> '')')
  WRITE(NULOUT,'('' Norm2 V: |V|2=<V;SCALP*V> '')')
!     WRITE(NULOUT,'('' Angle THETA between V1 and V2: '')')
!     WRITE(NULOUT,'(''   THETA=ACOS( <V1;SCALP*V2>/(|V1|*|V2|) )'')')
  WRITE(NULOUT,'('' V0lan: input vector (pseudo vars.)'')')
  WRITE(NULOUT,'('' V8: input vector projected into subspace (ifs vars.)'')')
  WRITE(NULOUT,'('' V0: input vector    (ifs vars.)'')')
  WRITE(NULOUT,'('' V7: vector after SPTRLCZTL (t=0) '')')
  WRITE(NULOUT,'('' V2: vector after CNT3TL'')')
  WRITE(NULOUT,'('' V3: vector after LOCNORTL '')')
  WRITE(NULOUT,'('' V4: vector after SPTRLCZTL (t=end) '')')
  WRITE(NULOUT,'('' V5: vector before adj int '')')
  WRITE(NULOUT,'('' V6: output vector (ifs vars.)'')')
  WRITE(NULOUT,'('' V9: output vector (pseudo vars.)'')')
  WRITE(NULOUT,'('' V10:proj.output vector (pseudo vars.)'')')
  WRITE(NULOUT,'('' V6lan:   output vector (pseudo vars.)'')')
!     WRITE(NULOUT,'('' THETA1: angle between V1 and V0 '')')
!     WRITE(NULOUT,'('' THETA2: angle between V2 and V0 '')')
!     WRITE(NULOUT,'('' THETA3: angle between V3 and V0 '')')
!     WRITE(NULOUT,'('' THETA4: angle between V3 and V1 '')')
  WRITE(NULOUT,'(''     |V0|2lan= '',E26.20)') ZNORMV0LAN
  WRITE(NULOUT,'(''     |V8|2= '',E26.20)') ZNORMV8
  WRITE(NULOUT,'(''     |V0|2= '',E26.20)') ZNORMV0
  WRITE(NULOUT,'(''     |V7|2= '',E26.20)') ZNORMV7
  WRITE(NULOUT,'(''     |V2|2= '',E26.20)') ZNORMV2
  WRITE(NULOUT,'(''     |V3|2= '',E26.20)') ZNORMV3
  WRITE(NULOUT,'(''     |V4|2= '',E26.20)') ZNORMV4
  WRITE(NULOUT,'(''     |V5|2= '',E26.20)') ZNORMV5
  WRITE(NULOUT,'(''     |V6|2= '',E26.20)') ZNORMV6
  WRITE(NULOUT,'(''     |V9|2= '',E26.20)') ZNORMV9
  WRITE(NULOUT,'(''    |V10|2= '',E26.20)') ZNORMV10
  WRITE(NULOUT,'(''     |V6|2lan= '',E26.20)') ZNORMV6LAN
!     WRITE(NULOUT,'(''         THETA1= '',E26.20)') ZTHETA1
!     WRITE(NULOUT,'(''         THETA2= '',E26.20)') ZTHETA2
!     WRITE(NULOUT,'(''         THETA3= '',E26.20)') ZTHETA3
!     WRITE(NULOUT,'(''         THETA4= '',E26.20)') ZTHETA4

!     ------------------------------------------------------------------
ELSEIF(NLANTYPE == 2) THEN
!     ------------------------------------------------------------------

!     - NLANTYPE=2: linear and adjoint integration with covariance matrix
!                   constraint with
!                 - local projection operator
!                 - initial spectral truncation
!                 - final spectral truncation

!     Solve the eigenvalue problem of the operator:
!     R(-1/2).V*.S0*.L*.T*.S1*.E.S1.T.L.S0.V.R(-1/2)

!     - coord trans from real to complex [ R(-1/2) ]
!     - canonical injection
!     - spectral truncation [ S0 ]
!     - coordinate transformation using covariance matrix [ V ]
!     - linear integration [ L ]
!     - gg localization [ T ]
!     - spectral truncation [ S1 ]
!     - inverse of can inj
!     - adjoint of the inverse of the canonical injection [ E ]
!     - adj of spectral truncation [ S1* ]
!     - adj of gg localization [ T* ]
!     - adj of linear integration [ L* ]
!     - adj of coordinate transformation [ V* ]
!     - adj of spectral truncation [ S0* ]
!     - inverse of canonical injection
!     - adj of coord trans from real to complex [ R(-1/2) ]

!     - norm control

  ZNORMV0LAN=DOT_PRODUCT(YD_PVAZIN,YD_PVAZIN)

!     - coord tranf from real to complex

  ITYPE=1
  CALL LCZTOIFS (ITYPE,YD_PVAZIN,YL_ZVAZX)

!     - canonical inject

  CALL CAIN (YL_ZVAZX)

!     - coordinate transformation form the variables defined in the space
!       where the pdf is isotropic, to model variables

! CALL ABOR1('OPK: ALL THESE CALLS TO CVARU3.. BROKEN/MH')
!!  CALL CVARU3I
  CALL CVARU2I

!     - norm control

  CALL CAININ (YL_ZVAZX)
  CALL SCAAS(idum,YL_ZVAZX,YL_ZVAZX,ZNORMV1)

!     - spectral truncation

  IF (LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV7)
  ENDIF

!     - linear integration

  N1HIS=0
  CALL CNT3TL
  N1HIS=1

!     - norm control

  CALL CAININ (YL_ZVAZX)
  CALL SCAAS(idum,YL_ZVAZX,YL_ZVAZX,ZNORMV2)

!     - call operator to localize the output field into a defined area
!       in gg space

  IF(LOCNORM) THEN
    CALL LCNORTL

!     - norm control

    CALL CAININ (YL_ZVAZG)
    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV3)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV4)
  ENDIF

!     - inv of canonical inj

  CALL CAININ (YL_ZVAZG)

!     - norm control

  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV5)

!     - adj of canonical injection

  CALL CAININAD (YL_ZVAZG)

!     - adj of spectral truncation

  IF(LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of localization operator

  IF(LOCNORM) CALL LCNORAD

!     - adj of linear integration

  N1HIS=0
  CALL CNT3AD
  N1HIS=1

!     - adj of spectral truncation

  IF(LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of coordinate transf

  CALL CVARU2IAD
!  CALL CVARU3IAD

!     - inv of canonical injection

  CALL CAININ (YL_ZVAZX)

!     - adj of coord tranf from real to complex

  ITYPE=1
  CALL LCZTOIFS (ITYPE,YL_ZVAZX,YD_PVAZOUT)

!     - norm control

  ZNORMV6LAN=DOT_PRODUCT(YD_PVAZOUT,YD_PVAZOUT)

!     - Output the norm control

  WRITE(NULOUT,'('' >> Sub OPK <<'')')
  WRITE(NULOUT,'('' Norm2 V Lanczos: |V|2lan=<V;V> '')')
  WRITE(NULOUT,'('' Norm2 V: |V|2=<V;SCALP*V> '')')
!     WRITE(NULOUT,'('' Angle THETA between V1 and V2: '')')
!     WRITE(NULOUT,'(''   THETA=ACOS( <V1;SCALP*V2>/(|V1|*|V2|) )'')')
  WRITE(NULOUT,'('' V0: input vector '')')
  WRITE(NULOUT,'('' V1: vector after coord transf'')')
  WRITE(NULOUT,'('' V7: vector after SPTRLCZTL (t=0) '')')
  WRITE(NULOUT,'('' V2: vector after CNT3TL '')')
  WRITE(NULOUT,'('' V3: vector after LOCNORTL '')')
  WRITE(NULOUT,'('' V4: vector after SPTRLCZTL (t=end) '')')
  WRITE(NULOUT,'('' V5: vector before adj int '')')
  WRITE(NULOUT,'('' V6: output vector '')')
!     WRITE(NULOUT,'('' THETA1: angle between V1 and V0 '')')
!     WRITE(NULOUT,'('' THETA2: angle between V2 and V0 '')')
!     WRITE(NULOUT,'('' THETA3: angle between V3 and V0 '')')
!     WRITE(NULOUT,'('' THETA4: angle between V3 and V1 '')')
  WRITE(NULOUT,'(''     |V0|2lan= '',E26.20)') ZNORMV0LAN
  WRITE(NULOUT,'(''     |V1|2= '',E26.20)') ZNORMV1
  WRITE(NULOUT,'(''     |V7|2= '',E26.20)') ZNORMV7
  WRITE(NULOUT,'(''     |V2|2= '',E26.20)') ZNORMV2
  WRITE(NULOUT,'(''     |V3|2= '',E26.20)') ZNORMV3
  WRITE(NULOUT,'(''     |V4|2= '',E26.20)') ZNORMV4
  WRITE(NULOUT,'(''     |V5|2= '',E26.20)') ZNORMV5
  WRITE(NULOUT,'(''     |V6|2lan= '',E26.20)') ZNORMV6LAN
!     WRITE(NULOUT,'(''         THETA1= '',E26.20)') ZTHETA1
!     WRITE(NULOUT,'(''         THETA2= '',E26.20)') ZTHETA2
!     WRITE(NULOUT,'(''         THETA3= '',E26.20)') ZTHETA3
!     WRITE(NULOUT,'(''         THETA4= '',E26.20)') ZTHETA4

!     ------------------------------------------------------------------
ELSEIF(NLANTYPE == 3) THEN
!     ------------------------------------------------------------------

!     - NLANTYPE=3: covariance matrix eigenvectors with
!                 - local projection operator
!                 - initial spectral truncation
!                 - final spectral truncation

!     Solve the eigenvalue problem of the operator:
!     R(-1/2).V*.S0*.T*.S1*.E.S1.T.S0.V.R(-1/2)

!     - coord transf from real to complex [ R(-1/2) ]
!     - canonical injection
!     - coordinate transformation [ V ]
!     - spectr trunc [ S0 ]
!     - gg localization [ T ]
!     - spectr trunc [ S1 ]
!     - inverse of can inj
!     - adjoint of the inverse of the canonical injection [ E ]
!     - adj of spectr trunc [ S1* ]
!     - adj of gg localization [ T* ]
!     - adj of spectr trunc [ S0* ]
!     - adj of coordinate transformation [ V* ]
!     - inverse of can inj
!     - adj of coord transf from real to complex [ R(-1/2) ]

!     - norm control

  ZNORMV0LAN=DOT_PRODUCT(YD_PVAZIN,YD_PVAZIN)

!     - coord tranf from real to complex

  ITYPE=1
  CALL LCZTOIFS (ITYPE,YD_PVAZIN,YL_ZVAZX)

!     - canonical inject

  CALL CAIN (YL_ZVAZX)

!     - coordinate transformation form the variables defined in the space
!       where the pdf is isotropic, to model variables

! CALL ABOR1('OPK: ALL THESE CALLS TO CVARU3.. BROKEN/MH')
!  CALL CVARU3I
  CALL CVARU2I

!     - norm control

  CALL CAININ(YL_ZVAZX)
  CALL SCAAS(idum,YL_ZVAZX,YL_ZVAZX,ZNORMV1)

!     - spectral truncation

  IF (LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV7)
  ENDIF

!     - call operator to localize the output field into a defined area
!       in gg space

  IF(LOCNORM) THEN
    CALL LCNORTL

!     - norm control

    CALL CAININ (YL_ZVAZG)
    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV3)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV4)
  ENDIF

!     - inv of canonical inj

  CALL CAININ (YL_ZVAZG)

!     - norm control

  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV5)

!     - adj of canonical injection

  CALL CAININAD (YL_ZVAZG)

!     - adj of spectral truncation

  IF(LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of localization operator

  IF(LOCNORM) CALL LCNORAD

!     - adj of spectral truncation

  IF(LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of coordinate transf

  CALL CVARU2IAD
! CALL ABOR1('OPK: ALL THESE CALLS TO CVARU3.. BROKEN/MH')
!  CALL CVARU3IAD

!     - inv of canonical injection

  CALL CAININ (YL_ZVAZX)

!     - adj of coord tranf from real to complex

  ITYPE=1
  CALL LCZTOIFS (ITYPE,YL_ZVAZX,YD_PVAZOUT)

!     - norm control

  ZNORMV6LAN=DOT_PRODUCT(YD_PVAZOUT,YD_PVAZOUT)

!     - Output the norm control

  WRITE(NULOUT,'('' >> Sub OPK <<'')')
  WRITE(NULOUT,'('' Norm2 V Lanczos: |V|2lan=<V;V> '')')
  WRITE(NULOUT,'('' Norm2 V: |V|2=<V;SCALP*V> '')')
!     WRITE(NULOUT,'('' Angle THETA between V1 and V2: '')')
!     WRITE(NULOUT,'(''   THETA=ACOS( <V1;SCALP*V2>/(|V1|*|V2|) )'')')
  WRITE(NULOUT,'('' V0: input vector '')')
  WRITE(NULOUT,'('' V1: vector after coord trans'')')
  WRITE(NULOUT,'('' V7: vector after SPTRLCZTL (t=0) '')')
  WRITE(NULOUT,'('' V3: vector after LOCNORTL '')')
  WRITE(NULOUT,'('' V4: vector after SPTRLCZTL (t=end) '')')
  WRITE(NULOUT,'('' V5: vector before adj int '')')
  WRITE(NULOUT,'('' V6: output vector '')')
!     WRITE(NULOUT,'('' THETA1: angle between V1 and V0 '')')
!     WRITE(NULOUT,'('' THETA2: angle between V2 and V0 '')')
!     WRITE(NULOUT,'('' THETA3: angle between V3 and V0 '')')
!     WRITE(NULOUT,'('' THETA4: angle between V3 and V1 '')')
  WRITE(NULOUT,'(''     |V0|2lan= '',E26.20)') ZNORMV0LAN
  WRITE(NULOUT,'(''     |V1|2= '',E26.20)') ZNORMV1
  WRITE(NULOUT,'(''     |V7|2= '',E26.20)') ZNORMV7
  WRITE(NULOUT,'(''     |V3|2= '',E26.20)') ZNORMV3
  WRITE(NULOUT,'(''     |V4|2= '',E26.20)') ZNORMV4
  WRITE(NULOUT,'(''     |V5|2= '',E26.20)') ZNORMV5
  WRITE(NULOUT,'(''     |V6|2lan= '',E26.20)') ZNORMV6LAN
!     WRITE(NULOUT,'(''         THETA1= '',E26.20)') ZTHETA1
!     WRITE(NULOUT,'(''         THETA2= '',E26.20)') ZTHETA2
!     WRITE(NULOUT,'(''         THETA3= '',E26.20)') ZTHETA3
!     WRITE(NULOUT,'(''         THETA4= '',E26.20)') ZTHETA4

!     ------------------------------------------------------------------

!     ------------------------------------------------------------------
ELSEIF(NLANTYPE == 4) THEN
!     ------------------------------------------------------------------

!     NLANTYPE=4: adaption of nlantype=1 to run in NCONF=131
!                 - linear and adjoint integration with
!                 - local projection operator
!                 - initial spectral truncation
!                 - final spectral truncation

!     Solve the eigenvalue problem of the operator:
!     E(-1/2).S0*.L*.T*.S1*.E.S1.T.L.S0.E(-1/2)=
!     = E(1/2).E(-1).S0*.L*.T*.S1*.E.S1.T.L.S0.E(-1/2)

!     - coordinate transformation from variables normalized with respect
!       to the canonical Euclidean product (as in the Lanczos code), to
!       variables normalized with respect to the SCALP norm ( as in IFS)
!       [ E(-1/2) ]
!     - canonical injection
!     - spectral truncation [ S0 ]
!     - linear integration [ L ]
!     - gg localization [ T ]
!     - spectral truncation [ S1 ]
!     - inv of canonical inj
!     - adjoint of the inverse of the canonical injection [ E ]
!     - adj of spectral truncation [ S1* ]
!     - adj of gg localization [ T* ]
!     - adj of linear integration [ L* ]
!     - adj of spectral truncation [ S0* ]
!     - adj of can inj [ E(-1) ]
!     - inverse of coord transformation [ E(1/2) ]

!     - norm control

! setting in 601

  L131TL=.FALSE.
  LOBSTL=.FALSE.
  LSIMOB=.TRUE.
  LOBS=.FALSE.

! needed for nstar2 & nstop2 & nstp

  CALL SU2YOM

  WRITE(NULOUT,*) 'NSTOP,NSTART,NSTP,NSTEP,NSTAR2,NSTOP2 =',&
   & NSTOP,NSTART,NSTP,NSTEP,NSTAR2,NSTOP2  

  NCONF=601
  CALL SUSCAL

!----------------------------------------------------

  ZNORMV0LAN=DOT_PRODUCT(YD_PVAZIN,YD_PVAZIN)

!     - from pseudo-variables to IFS-variables in complex space

  YL_ZVAZX%data = YD_PVAZIN%data * YRSCALPSQRT%data

!     - norm control

  CALL SCAAS(idum,YL_ZVAZX,YL_ZVAZX,ZNORMV0)

!     - canonical inject

  CALL CAIN (YL_ZVAZX)

!     - re-definition of norm at t0 

  IF (LNEWNORMT0) THEN
    CALL CAININ (YL_ZVAZX)
    CALL CHNORM(YL_ZVAZX,YL_ZVAZG)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV7)
  ENDIF

!     - linear integration

  N1HIS=0
  CALL CNT3TL
  N1HIS=1

!     - norm control

  CALL CAININ (YL_ZVAZG)
  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV2)

!     - call operator to localize the output field into a defined area
!       in gg space

  IF(LOCNORM) THEN
    CALL LCNORTL

!     - norm control

    CALL CAININ (YL_ZVAZG)
    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV3)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)

!     - norm control

    CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV4)
  ENDIF

!     - inv of canonical inj

  CALL CAININ (YL_ZVAZG)

!     - norm control

  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV5)

!     - adj of inverse of canonical injection

  CALL CAININAD (YL_ZVAZG)

!     - adj of spectral truncation

  IF(LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of localization operator

  IF(LOCNORM) CALL LCNORAD

!     - adj of linear integration

  N1HIS=0
  CALL CNT3AD
  N1HIS=1

!     - adj of spectral truncation

  IF(LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of re-definition of norm at t0 

  IF (LNEWNORMT0) THEN
    CALL CAININ (YL_ZVAZX)
    CALL CHNORM(YL_ZVAZX,YL_ZVAZG)
    CALL CAIN (YL_ZVAZG)
  ENDIF

!     - adj of canonical injection

  YL_ZVAZG=0.0_JPRB
  CALL CAINAD (YL_ZVAZG)

!     - norm control

  CALL SCAAS(idum,YL_ZVAZG,YL_ZVAZG,ZNORMV6)

!     - from IFS-variables to pseudo-variables

  YD_PVAZOUT%data=YL_ZVAZG%data*YSCALPSQRT%data

!     - norm control

  ZNORMV6LAN=DOT_PRODUCT(YD_PVAZOUT,YD_PVAZOUT)

!     - output the norm control

  WRITE(NULOUT,'('' >> Sub OPK <<'')')
  WRITE(NULOUT,'('' Norm2 V Lanczos: |V|2lan=<V;V> '')')
  WRITE(NULOUT,'('' Norm2 V: |V|2=<V;SCALP*V> '')')
!     WRITE(NULOUT,'('' Angle THETA between V1 and V2: '')')
!     WRITE(NULOUT,'(''   THETA=ACOS( <V1;SCALP*V2>/(|V1|*|V2|) )'')')
  WRITE(NULOUT,'('' V0: input vector '')')
  WRITE(NULOUT,'('' V7: vector after SPTRLCZ (t=0) '')')
  WRITE(NULOUT,'('' V2: vector after CNT3TL'')')
  WRITE(NULOUT,'('' V3: vector after LOCNORTL '')')
  WRITE(NULOUT,'('' V4: vector after SPTRLCZ (t=end) '')')
  WRITE(NULOUT,'('' V5: vector before adj int '')')
  WRITE(NULOUT,'('' V6: output vector '')')
!     WRITE(NULOUT,'('' THETA1: angle between V1 and V0 '')')
!     WRITE(NULOUT,'('' THETA2: angle between V2 and V0 '')')
!     WRITE(NULOUT,'('' THETA3: angle between V3 and V0 '')')
!     WRITE(NULOUT,'('' THETA4: angle between V3 and V1 '')')
  WRITE(NULOUT,'(''     |V0|2lan= '',E26.20)') ZNORMV0LAN
  WRITE(NULOUT,'(''     |V0|2= '',E26.20)') ZNORMV0
  WRITE(NULOUT,'(''     |V7|2= '',E26.20)') ZNORMV7
  WRITE(NULOUT,'(''     |V2|2= '',E26.20)') ZNORMV2
  WRITE(NULOUT,'(''     |V3|2= '',E26.20)') ZNORMV3
  WRITE(NULOUT,'(''     |V4|2= '',E26.20)') ZNORMV4
  WRITE(NULOUT,'(''     |V5|2= '',E26.20)') ZNORMV5
  WRITE(NULOUT,'(''     |V6|2= '',E26.20)') ZNORMV6
  WRITE(NULOUT,'(''     |V6|2lan= '',E26.20)') ZNORMV6LAN
!     WRITE(NULOUT,'(''         THETA1= '',E26.20)') ZTHETA1
!     WRITE(NULOUT,'(''         THETA2= '',E26.20)') ZTHETA2
!     WRITE(NULOUT,'(''         THETA3= '',E26.20)') ZTHETA3
!     WRITE(NULOUT,'(''         THETA4= '',E26.20)') ZTHETA4

!     ------------------------------------------------------------------
!     ------------------------------------------------------------------
ELSEIF ( (NLANTYPE == 5).or.(NLANTYPE == 6).or.&
   & (NLANTYPE == 8).or.(NLANTYPE == 9)     ) THEN  
!     ------------------------------------------------------------------

!     NLANTYPE=5/6/8/9: adaption of nlantype=1 to run in control space
!                    nlantype=5:  OPM is identity
!                    nlantype=6:  OPM is Hessian
!                    -linear and adjoint integration with
!                    - local projection operator
!                    - initial spectral truncation
!                    - final spectral truncation

!     Solve the eigenvalue problem of the operator:
!     E(-1/2).S0*.L*.T*.S1*.E.S1.T.L.S0.E(-1/2)=
!     = E(1/2).E(-1).S0*.L*.T*.S1*.E.S1.T.L.S0.E(-1/2)

!     - coordinate transformation from variables normalized with respect
!       to the canonical Euclidean product (as in the Lanczos code), to
!       variables normalized with respect to the SCALP norm ( as in IFS)
!       [ E(-1/2) ]
!     - canonical injection
!     - spectral truncation [ S0 ]
!     - linear integration [ L ]
!     - gg localization [ T ]
!     - spectral truncation [ S1 ]
!     - inv of canonical inj
!     - adjoint of the inverse of the canonical injection [ E ]
!     - adj of spectral truncation [ S1* ]
!     - adj of gg localization [ T* ]
!     - adj of linear integration [ L* ]
!     - adj of spectral truncation [ S0* ]
!     - adj of can inj [ E(-1) ]
!     - inverse of coord transformation [ E(1/2) ]

  WRITE(NULOUT,*) 'VARIATIONAL LOGICALS BEFORE OPK'

  WRITE(NULOUT,'('' L131TL='',L1)') L131TL
  WRITE(NULOUT,'('' LOBSTL='',L1)') LOBSTL
  WRITE(NULOUT,'('' LSIMOB='',L1)') LSIMOB
  WRITE(NULOUT,'('' LOBSREF='',L1)') LOBSREF
  WRITE(NULOUT,'('' LOBSC1='',L1)') LOBSC1
  WRITE(NULOUT,'('' LOBS='',L1)') LOBS

  CALL ALLOCATE_CTLVEC(YL_ZVAZY)
  CALL ALLOCATE_CTLVEC(YL_ZVAZH)
  CALL ALLOCATE_CTLVEC(YL_ZWEIGHTS)

! setting in 601

  L131TL=.FALSE.
  LOBSTL=.FALSE.
  LSIMOB=.TRUE.
  LOBS=.FALSE.

! copy the energy inner product weights into ZWEIGHTS

  NCONF=601
  CALL SUSCAL
  YL_ZWEIGHTS = YSCALP

! reset SCALP to the spectral inner product

  NCONF=131
  CALL SUSCAL

! needed for nstar2 & nstop2 & nstp

  CALL SU2YOM

  WRITE(NULOUT,*) 'NSTOP,NSTART,NSTP,NSTEP,NSTAR2,NSTOP2 =',&
   & NSTOP,NSTART,NSTP,NSTEP,NSTAR2,NSTOP2  

!     - norm control

  ZNORMV0LAN = DOT_PRODUCT(YD_PVAZIN,YD_PVAZIN)

! from Euclidean inner product to spectral inner product

  YL_ZVAZX%data = YD_PVAZIN%data * YRSCALPSQRT%data

  ZNORMV0 = DOT_PRODUCT(YL_ZVAZX,YL_ZVAZX,YL_ZWEIGHTS)

! from control to model variables

  LSUBBG=.FALSE.
  YL_ZVAZY = YL_ZVAZX
  CALL CHAVARIN (YL_ZVAZY)
  LSUBBG=.TRUE.

!     - zero specific humidity

  SPQ(:,:) = 0.0_JPRB

!     - spectral truncation

  IF (LSPTRLC0) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAIN (YL_ZVAZG)
    ZNORMV7 = DOT_PRODUCT(YL_ZVAZG,YL_ZVAZG,YL_ZWEIGHTS)
  ENDIF

!     - extra projection

!j         CALL LCNORTL

!     - linear integration

  N1HIS=0
  NCONF=601
  CALL CNT3TL
  NCONF=131
  N1HIS=1

  CALL CAININ (YL_ZVAZG)
  ZNORMV2 = DOT_PRODUCT(YL_ZVAZG,YL_ZVAZG,YL_ZWEIGHTS)

!     - call operator to localize the output field into a defined area
!       in gg space

  IF(LOCNORM) THEN
    CALL LCNORTL
    CALL CAININ (YL_ZVAZG)
    ZNORMV3 = DOT_PRODUCT(YL_ZVAZG,YL_ZVAZG,YL_ZWEIGHTS)
  ENDIF

!     - spectral truncation

  IF (LSPTRLC1) THEN
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAIN (YL_ZVAZG)
    ZNORMV4 = DOT_PRODUCT(YL_ZVAZG,YL_ZVAZG,YL_ZWEIGHTS)
  ENDIF

!     - inv of canonical inj

!     - adj of inverse of canonical injection

  IF(NLANTYPE == 8.OR. NLANTYPE == 9) THEN
    LSUBBG=.FALSE.
    LLSKF=LSKF
    LSKF=.FALSE.
    SPQ(:,:) = 0.0_JPRB
    CALL CHAVAR   (YL_ZVAZH)
    CALL CHAVARAD (YL_ZVAZH)
    SPQ(:,:) = 0.0_JPRB
    LSUBBG=.TRUE.
    LSKF=LLSKF
  ELSE
    CALL CAININ (YL_ZVAZX)
    YL_ZVAZG%data=YL_ZVAZX%data*YL_ZWEIGHTS%data*YRSCALP%data
    CALL CAININAD (YL_ZVAZG)
  ENDIF

!     - adj of spectral truncation

  IF(LSPTRLC1) THEN
    CALL CAINAD (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN1,NWTRMAX1)
    CALL CAININAD (YL_ZVAZG)
  ENDIF

!     - adj of localization operator

  IF(LOCNORM) CALL LCNORAD

!     - adj of linear integration

  N1HIS=0
  NCONF=601
  CALL CNT3AD
  NCONF=131
  N1HIS=1

!     - adjoint of extra projection

!j          CALL LCNORAD

!     - adj of spectral truncation

  IF(LSPTRLC0) THEN
    CALL CAINAD (YL_ZVAZX)
    YL_ZVAZG=YL_ZVAZX
    CALL SPTRLCZ(YL_ZVAZG,NWTRMIN0,NWTRMAX0)
    CALL CAININAD (YL_ZVAZG)
  ENDIF

!     - zero specific humidity

  SPQ(:,:) = 0.0_JPRB

  YL_ZVAZG=0.0_JPRB
  CALL CAINAD (YL_ZVAZG)
  ZNORMV6 = DOT_PRODUCT(YL_ZVAZG,YL_ZVAZG,YL_ZWEIGHTS)

  LSUBBG=.FALSE.
  YL_ZVAZH = 0.0_JPRB
  CALL CHAVARINAD (YL_ZVAZH)
  YD_PVAZOUT = YL_ZVAZH
  LSUBBG=.TRUE.

! reset some variational logicals to 131 setting

  L131TL=.TRUE.
  LOBSTL=.TRUE.
  LSIMOB=.FALSE.
  LOBS=.TRUE.

! back to Euclidean inner product space

  YD_PVAZOUT%data = YD_PVAZOUT%data * YSCALPSQRT%data
  ZNORMV6LAN = DOT_PRODUCT (YD_PVAZOUT,YD_PVAZOUT)

  CALL DEALLOCATE_CTLVEC(YL_ZVAZY)
  CALL DEALLOCATE_CTLVEC(YL_ZVAZH)
  CALL DEALLOCATE_CTLVEC(YL_ZWEIGHTS)

!     - output the norm control

  WRITE(NULOUT,'('' >> Sub OPK <<'')')
  WRITE(NULOUT,'('' Norm2 V Lanczos: |V|2lan=<V;V> '')')
  WRITE(NULOUT,'('' Norm2 V: |V|2=<V;SCALP*V> '')')
!     WRITE(NULOUT,'('' Angle THETA between V1 and V2: '')')
!     WRITE(NULOUT,'(''   THETA=ACOS( <V1;SCALP*V2>/(|V1|*|V2|) )'')')
  WRITE(NULOUT,'('' V0: input vector '')')
  WRITE(NULOUT,'('' V7: vector after SPTRLCZTL (t=0) '')')
  WRITE(NULOUT,'('' V2: vector after CNT3TL'')')
  WRITE(NULOUT,'('' V3: vector after LOCNORTL '')')
  WRITE(NULOUT,'('' V4: vector after SPTRLCZTL (t=end) '')')
  WRITE(NULOUT,'('' V5: vector before adj int '')')
  WRITE(NULOUT,'('' V6: output vector '')')
!     WRITE(NULOUT,'('' THETA1: angle between V1 and V0 '')')
!     WRITE(NULOUT,'('' THETA2: angle between V2 and V0 '')')
!     WRITE(NULOUT,'('' THETA3: angle between V3 and V0 '')')
!     WRITE(NULOUT,'('' THETA4: angle between V3 and V1 '')')
  WRITE(NULOUT,'(''     |V0|2lan= '',E26.20)') ZNORMV0LAN
  WRITE(NULOUT,'(''     |V0|2= '',E26.20)') ZNORMV0
  WRITE(NULOUT,'(''     |V7|2= '',E26.20)') ZNORMV7
  WRITE(NULOUT,'(''     |V2|2= '',E26.20)') ZNORMV2
  WRITE(NULOUT,'(''     |V3|2= '',E26.20)') ZNORMV3
  WRITE(NULOUT,'(''     |V4|2= '',E26.20)') ZNORMV4
  WRITE(NULOUT,'(''     |V6|2= '',E26.20)') ZNORMV6
  WRITE(NULOUT,'(''     |V6|2lan= '',E26.20)') ZNORMV6LAN
!     WRITE(NULOUT,'(''         THETA1= '',E26.20)') ZTHETA1
!     WRITE(NULOUT,'(''         THETA2= '',E26.20)') ZTHETA2
!     WRITE(NULOUT,'(''         THETA3= '',E26.20)') ZTHETA3
!     WRITE(NULOUT,'(''         THETA4= '',E26.20)') ZTHETA4

!     ------------------------------------------------------------------

!     ------------------------------------------------------------------
ELSEIF(NLANTYPE == 7) THEN
!     ------------------------------------------------------------------

!  copy of opm(NLANTYPE=6): Hessian of 3D-Var 

  ILANTYPE=NLANTYPE
  NLANTYPE=6

  CALL OPM(YD_PVAZIN,YD_PVAZOUT)

  NLANTYPE=ILANTYPE

ENDIF

IF (.NOT.L_EOFS) THEN
  CALL DEALLOCATE_CTLVEC(YL_ZVAZX)
  CALL DEALLOCATE_CTLVEC(YL_ZVAZG)
ENDIF

!     ------------------------------------------------------------------
NSIM4D=NSIM4D+1
!     -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('OPK',1,ZHOOK_HANDLE)
END SUBROUTINE OPK
!     -------------------------------------------------------------------

