SUBROUTINE RDTLLCZ(YD_IOSTREAM,YD_PCV)

!**** *RDTLLCZ*

!     Purpose.
!     --------
!           Read from the files of the perturbations computed in conf 601
!     a field, and construct a state vector.

!**   Interface.
!     ----------
!        *CALL* *RDTLLCZ*

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Reference.
!     ----------
!        ECMWF research department documentation of the
!     "in core model"

!     Author.
!     -------
!        Roberto Buizza :                   *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-10-10
!        R Buizza : 93-10-25
!        L Isaksen: 95-06-06 Reordering of spectral arrays
!        L Isaksen: 96-05-05 Adapted for Message Passing
!        R Buizza : 96-06-05 Read file in GRIB format   
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-May-2006 Generalized IO scheme
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NSPEC2  ,NS3D
USE YOMLUN   , ONLY : NULOUT
USE YOMCVA   , ONLY : NVA3D    ,NVA2D
USE YOMMP    , ONLY : NPSP
USE YOMGRB   , ONLY : NGRBLNSP ,NGRBSP3
USE YOMLCZ   , ONLY : NRITZREAD, NRITZNUMB, NRITZGBH1, NRITZGBH2
USE CONTROL_VECTORS
USE CONTROL_VECTORS_COMM_MOD
USE IOSTREAM_MIX , ONLY :  SETUP_IOREQUEST, IO_GET, IO_INQUIRE, &
 &  TYPE_IOSTREAM , TYPE_IOREQUEST, CLOSE_IOREQUEST
USE MPL_MODULE

IMPLICIT NONE

TYPE(TYPE_IOSTREAM) ,INTENT(INOUT) :: YD_IOSTREAM
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YD_PCV 
!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSPA3(NFLEVL,NSPEC2,NS3D), ZSPA2(NSPEC2,NVA2D)
INTEGER(KIND=JPIM) :: IGRIB3D(NS3D),IGRIB2D(1)
INTEGER(KIND=JPIM) :: II, JF,JS,JL,I2D,I3D
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!     1. Read from file KUNIT written by NALAN1
!     -------------------------------------------

IF (LHOOK) CALL DR_HOOK('RDTLLCZ',0,ZHOOK_HANDLE)

!     1.1 Read record with informations

WRITE(NULOUT,'('' >> Sub RDTLLCZ <<'')')

!     1.2 Read a singular vector

!     - vorticity

I2D = 1
IGRIB2D(I2D) = NGRBLNSP
I3D = NS3D
IGRIB3D(1:I3D) = NGRBSP3(1:I3D)

CALL SETUP_IOREQUEST(YL_IOREQUEST,'SPECTRAL_FIELDS',LDGRIB=.TRUE.,&
 & KGRIB3D=IGRIB3D(1:I3D),KGRIB2D=IGRIB2D(1:I2D),CDLEVTYPE='ML',KCHUNKSIZE=1)
CALL IO_GET(YD_IOSTREAM,YL_IOREQUEST,PR2=ZSPA2,PR3=ZSPA3)
CALL IO_INQUIRE(YD_IOSTREAM,YL_IOREQUEST,LDREAD=.FALSE.,&
 & KSVCOMP=NRITZREAD,KNUMB=NRITZNUMB,KRITZ1=NRITZGBH1,KRITZ2=NRITZGBH2)
CALL CLOSE_IOREQUEST(YL_IOREQUEST)

!     -----------------------------------------------------------------

!*    5.    

IF (YD_PCV%LINITCV) THEN
  II=0
  IF (NPSP==1) THEN
    DO JF=1,NVA2D
      DO JS=1,NSPEC2
        II=II+1
        YD_PCV%INITCV(II)=ZSPA2(JS,JF)
      ENDDO
    ENDDO
  ENDIF
  IF (NS3D > NVA3D) THEN
    CALL ABOR1('CAIN: UNEXPECTED INCONSITENCY NS3D > NVA3D ')
  ENDIF
  DO JF=1,NS3D
    DO JS=1,NSPEC2
      DO JL=1,NFLEVL
        II=II+1
        YD_PCV%INITCV(II)=ZSPA3(JL,JS,JF)
      ENDDO
    ENDDO
  ENDDO
ELSE
  CALL ABOR1('RDTLLCZ:LINITCV FALSE')
ENDIF

!     -----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('RDTLLCZ',1,ZHOOK_HANDLE)
END SUBROUTINE RDTLLCZ
