SUBROUTINE SULCZ

!**** *SULCZ*   - Routine to initialize Lanczos common variables

!     Purpose.
!     --------
!           Initialize Lanczos control common
!**   Interface.
!     ----------
!        *CALL* *SULCZ

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        COMMON YOMLCZ

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Authors.
!     -------
!        Roberto Buizza  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-02-26
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!                    R Buizza : 94-11-02 To avoid memory overwriting
!                    F Bouttier 96-02-06 Manage LBACKG
!                    R Buizza : 96-01-30 More norm choices (NLANNORM)
!                    R Buizza : 96-02-15 Add possibility to have different
!                                        norms at t0 and t1 by use of CN0
!                    R Buizza : 96-03-05 Add weight for q-term if total
!                                        energy norm is used
!                    J Barkmeijer: 97-2-14 Add switch LJACDAV to run the
!                                          Jacobi-Davidson scheme
!             Jan Barkmeijer : 00-11-06  LFORCE for computing forcing SVs
!             M. Leutbecher : 2003-06 LRENORMALIZE switch added
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     -----------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NSMAX   ,NPROMA
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMCT0   , ONLY : NCONF    ,LBACKG  ,NSTOP
USE YOMCT0B  , ONLY : LECMWF
USE YOMCST   , ONLY : RPI
USE YOMLCZ   , ONLY : RLANBUF   ,NITERL   ,NWEIGL   ,&
 & NLANCOUNT,NLANTYPE ,NLANNORM ,NEIGEVO  ,NLEVMIN  ,&
 & NLEVMAX  ,NWTRMIN0 ,NWTRMAX0 ,NWTRMIN1 ,NWTRMAX1 ,&
 & NEWNORMT0,NINNER   ,NJDSTOP  ,N_DIM_SUBSPACE,&
 & XKAPA    ,XMIN_RITZ,ALAT1    ,ALON1    ,ALAT3    ,ALON3    ,&
 & COEQTERM ,COENEWQTERM,LANCZOS  ,LEVOLC   ,LSCALC   ,&
 & LOCNORM  ,LSPTRLC0 ,LSPTRLC1 ,LSELU    ,&
 & LSELV    ,LSELT    ,LSELQ    ,LSELSP   ,LNEWNORMT0,&
 & LJACDAV  ,MEMBFGS  ,GREDBFGS ,LCHSYMEIG,&
 & LFORCE   ,LFORCEWR ,LRAIN    ,RLRAIN   ,NRAINSTART,&
 & LSYMCHECK,L_SOS    ,L_EOFS   ,L_BALANCED_REDUCTION,&
 & L_USE_CONGRAD      ,L_UNPREDICTABLE_COMPONENTS,&
 & L_SUBSPACE_SVS, &
 & YSTATE_VECTOR_4D, TSTEP_STATE_4D, NSTEPS_PER_STATE,&
 & LRENORMALIZE, LSMSSIG_SVINI
USE YOMCVA   , ONLY : YVAZX
USE YOMNMIA  , ONLY : LNMIRQ
USE YOMDYN   , ONLY : TSTEP
USE CONTROL_VECTORS
USE GRIB_API_INTERFACE, ONLY : IGRIB_SET_VALUE
USE IOSTREAM_MIX, ONLY : NGRIB_HANDLE_SH, NGRIB_HANDLE_SH_ML, NGRIB_HANDLE_GG,&
 & NGRIB_HANDLE_GG_ML

IMPLICIT NONE

INTEGER(KIND=JPIM) :: J,JTEMP,IMULF,IMDEG,IGRIB_HANDLE,IGRIB_HANDLES(4)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "suforce.intfb.h"

#include "namlcz.h"
!     ------------------------------------------------------------------

!*    1. Set default values.
!     ----------------------

!     1.1 Set implicit default values

IF (LHOOK) CALL DR_HOOK('SULCZ',0,ZHOOK_HANDLE)
LANCZOS=.TRUE.
LJACDAV=.FALSE.
LFORCE=.FALSE.
LFORCEWR=.FALSE.
LSCALC=.FALSE.
LSPTRLC0=.FALSE.
LSPTRLC1=.FALSE.
LCHSYMEIG=.FALSE.
LSYMCHECK=.FALSE.
LRENORMALIZE=.FALSE.
L_UNPREDICTABLE_COMPONENTS=.FALSE.
L_SUBSPACE_SVS=.FALSE.
N_DIM_SUBSPACE=0
L_SOS =.FALSE.
L_EOFS=.FALSE.
L_BALANCED_REDUCTION=.FALSE.
LSMSSIG_SVINI=.FALSE.
TSTEP_STATE_4D=3600.0_JPRB
LRAIN=.FALSE.
NRAINSTART=0
NITERL=100
NWEIGL=50
NLANTYPE=1
NLANCOUNT=0
NEIGEVO=35
XKAPA=0.01_JPRB
XMIN_RITZ=1.0_JPRB
MEMBFGS=200
GREDBFGS=0.1_JPRB

!     Define the norm:
!     NLANNORM = 1 for total-energy norm at t0
!                2 for kin-energy   norm at t0
!                3 for vor-2        norm at t0
!                4 for psi-2        norm at t0
!                5 for rot-kin ener norm at t0

NLANNORM=1

!     Define coefficient in fron of the q-term in the total
!     energy computation

COEQTERM=0._JPRB

!     Define variables to confine the norm conputation.
!     The area is defined :
!     - vertically: by the min (NLEVMIN) and max (NMAXLEV) level number
!       (the top of the atm has lev=1, the surface level lev=NFLEVG);
!     - horizontally: it is defined by the NW (point 1) and the SE
!       (point 3) corners.
!     So we have: - by the south (ALAT3) and the north (ALAT1) latitude
!                 - by the west (ALON1) and the east (ALON3) longitude
!     (Greenwich has lon=0.0, and the long goes from 0.0 to 360.0;
!     - by variable type : Uwind, Vwind, T, Q, Ps.

ALAT1=90.0_JPRB
ALON3=359.9_JPRB
ALON1=0._JPRB
NLEVMIN=1
NLEVMAX=NFLEVG
LSELU=.TRUE.
LSELV=.TRUE.
LSELT=.TRUE.
LSELQ=.TRUE.
LSELSP=.TRUE.

!     Define variables to truncate the state vector in spectral space
!     Spectral components with total wave number outside 
!     the window defined by NWTRMIN and NWTRMAX are set to zero 
!     (<NWTRMIN or >NWTRMAX)

NWTRMIN0=0
NWTRMAX0=NSMAX

NWTRMIN1=0
NWTRMAX1=NSMAX

!     Define whether to change the norm at initial time

LNEWNORMT0=.FALSE.

!     Define the new norm at initial time:
!     NEWNORMT0= 1 for total-energy norm at t0
!                2 for kin-energy   norm at t0
!                3 for vor-2        norm at t0
!                4 for psi-2        norm at t0
!                5 for rot-kin ener norm at t0

NEWNORMT0=1

!     Define coefficient in front of the q-term in the total
!     energy computation

COENEWQTERM=0._JPRB

!*    1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
  LOCNORM=.TRUE.
  ALAT3=30.0_JPRB
  LEVOLC=.TRUE.
  NEIGEVO=50
  L_USE_CONGRAD=.TRUE.
ELSE
  LOCNORM=.FALSE.
  ALAT3=0._JPRB
  LEVOLC=.FALSE.
  NEIGEVO=5
  L_USE_CONGRAD=.FALSE.
ENDIF

!     -----------------------------------------------------------------

!*    2. Read Namelist
!     ----------------

CALL POSNAM(NULNAM,'NAMLCZ')
READ(NULNAM,NAMLCZ)

IF (LECMWF .AND..NOT.(L_USE_CONGRAD.OR.L_BALANCED_REDUCTION)) THEN

!       To avoid the possibility of memory overwriting when computing
!       the most unstable perturbations using the Lanczos code,
!       the number of wanted eigenvectors have been set equal to the 
!       number of iterations (in any case, NWEIGL works in the Lanczos 
!       code only when XKAPA is equal to the accuracy of the machine) 
!       (R Buizza 02-11-94).

  NWEIGL=NITERL
  WRITE(NULOUT,*) 'WARNING: NWEIGL has been reset to ',NWEIGL, &
   & ' to avoid overwriting memory'  
ENDIF

!       Check LBACKG in YOMCT0 (F Bouttier 96-02-06)
IF ((.NOT.LBACKG).AND.(ABS(NCONF)/100 == 6).AND. &
   & (NLANTYPE>1 .AND. NLANTYPE/=10 .AND.NLANTYPE/=11)) THEN  
  WRITE(NULOUT,*) 'WARNING:LBACKG inconsistent, set to .T.'
  LBACKG=.TRUE.
ENDIF

!     rain SVs

IF (LRAIN) THEN
  LSELU=.FALSE.
  LSELV=.FALSE.
  LSELT=.FALSE.
  LSELQ=.FALSE.
  LSELSP=.FALSE.
  NRAINSTART=INT(NRAINSTART*(3600._JPRB/TSTEP))
  ALLOCATE(RLRAIN(NPROMA))
  RLRAIN(:)= 0.0_JPRB
  WRITE(NULOUT,9990) 'RLRAIN  ',SIZE(RLRAIN ),SHAPE(RLRAIN )
ENDIF

!       Check balanced truncation options and allocate 4d state vector

IF (L_BALANCED_REDUCTION .OR. L_EOFS .OR. L_SOS) THEN
  IF (COUNT((/ L_BALANCED_REDUCTION, L_EOFS, L_SOS /))>1) THEN
    WRITE (NULOUT,*) 'L_BALANCED_REDUCTION, L_EOFS, L_SOS are mutually exclusive'
    WRITE (NULOUT,*) 'L_BALANCED_REDUCTION, L_EOFS, L_SOS=', &
     & L_BALANCED_REDUCTION, L_EOFS, L_SOS  
    CALL ABOR1 ('L_BALANCED_REDUCTION, L_EOFS, L_SOS are mutually exclusive')
  ENDIF

  NSTEPS_PER_STATE = TSTEP_STATE_4D/TSTEP 

  IF (ABS(NSTEPS_PER_STATE*TSTEP-TSTEP_STATE_4D)>SQRT(TINY(TSTEP))) THEN
    WRITE (NULOUT,*) 'TSTEP_STATE_4D MUST BE A MULTIPLE OF TSTEP'
    WRITE (NULOUT,*) 'TSTEP_STATE_4D=',TSTEP_STATE_4D,' TSTEP=',TSTEP
    CALL ABOR1 ('TSTEP_STATE_4D MUST BE A MULTIPLE OF TSTEP')
  ENDIF

  IF (NSTEPS_PER_STATE*(NSTOP/NSTEPS_PER_STATE) /= NSTOP) THEN
    WRITE (NULOUT,*) 'NSTEPS_PER_STATE MUST BE A DIVISOR OF NSTOP'
    WRITE (NULOUT,*) 'TSTEP_STATE_4D=',TSTEP_STATE_4D,' TSTEP=',TSTEP
    WRITE (NULOUT,*) 'NSTEPS_PER_STATE=',NSTEPS_PER_STATE
    CALL ABOR1 ('NSTEPS_PER_STATE MUST BE A DIVISOR OF NSTOP')
  ENDIF
 
  WRITE (NULOUT,*) 'ALLOCATING 4D STATE VECTOR'

  ALLOCATE (YSTATE_VECTOR_4D(NSTOP/NSTEPS_PER_STATE-1))
  DO J=1,NSTOP/NSTEPS_PER_STATE-1
    WRITE (NULOUT,*) 'Allocate 4d state: step=',J*NSTEPS_PER_STATE,' J=',J
    CALL ALLOCATE_CTLVEC (YSTATE_VECTOR_4D(J))
  ENDDO
ENDIF

!     -----------------------------------------------------------------

!*    3. Print final values.
!     ----------------------

WRITE(NULOUT,'('' >> Sub SULCZ ,,'')')
WRITE(NULOUT,'('' Lanczos variables in common YOMLCZ'')')
WRITE(NULOUT,'('' Max num iterations        = '',I10)') NITERL
WRITE(NULOUT,'('' Max num wanted eig        = '',I10)') NWEIGL
WRITE(NULOUT,'('' Lanczos iter counter      = '',I10)') NLANCOUNT
WRITE(NULOUT,'('' Accept eigv precision     = '',E26.20)') XKAPA
WRITE(NULOUT,'('' Smallest accepted eig.val.= '',E26.20)') XMIN_RITZ
WRITE(NULOUT,'('' Integr (TL+AD=1, +COVAR=2, COVAR=3)= '',I10)') NLANTYPE
WRITE(NULOUT,'('' Number of TL evolution    = '',I10)') NEIGEVO
WRITE(NULOUT,'('' Norm definition NLANNORM: 1 for Tot energy'')')
WRITE(NULOUT,'(''                           2     Kin energy'')')
WRITE(NULOUT,'(''                           3     vor-Square'')')
WRITE(NULOUT,'(''                           4     psi-Square'')')
WRITE(NULOUT,'(''                           5     rot-kin en'')')
WRITE(NULOUT,'(''                 NLANNORM= '',I10)') NLANNORM
WRITE(NULOUT,'('' Coeff for q-term in tot-en= '',E26.20)') COEQTERM
WRITE(NULOUT,'('' Compute the unstable pertur = '',L1)') LANCZOS
IF (LRAIN) WRITE(NULOUT,*) 'RAIN singular vectors from step',NRAINSTART
WRITE(NULOUT,'('' NOT initial BUT tendency pertur = '',L1)') LFORCE
WRITE(NULOUT,'('' Write basic/TL evolution    = '',L1)') LEVOLC
WRITE(NULOUT,'('' Write the scal prod weight  = '',L1)') LSCALC
WRITE(NULOUT,'('' Localize the norm computat  = '',L1)') LOCNORM
WRITE(NULOUT,'('' Spectr trunc at initial time = '',L1)') LSPTRLC0
WRITE(NULOUT,'('' Spectr trunc at final time   = '',L1)') LSPTRLC1
WRITE(NULOUT,'('' Re-define the norm at t0 = '',L1)') LNEWNORMT0
IF(LOCNORM) THEN
  WRITE(NULOUT,'('' Lat area limit ALAT1,ALAT3='',2(1X,E12.5))')ALAT1,ALAT3
  WRITE(NULOUT,'('' Lon area limit ALON1,ALON3='',2(1X,E12.5))')ALON1,ALON3
  WRITE(NULOUT,'('' Min-max vert lev NLEVMIN,MAX='',2(1X,I5))')&
   & NLEVMIN,NLEVMAX  
  ALAT1=ALAT1*RPI/180._JPRB
  ALAT3=ALAT3*RPI/180._JPRB
  ALON1=ALON1*RPI/180._JPRB
  ALON3=ALON3*RPI/180._JPRB
ENDIF

IF (LRENORMALIZE) THEN
  WRITE(NULOUT,'('' LRENORMALIZE : '',L2)') LRENORMALIZE
ENDIF

IF(LSPTRLC0) THEN
  WRITE(NULOUT,'('' Min-max total wave num t=0 ='',2(1X,I4))')&
   & NWTRMIN0, NWTRMAX0  
ENDIF

IF(LSPTRLC1) THEN
  WRITE(NULOUT,'('' Min-max total wave num tend ='',2(1X,I4))')&
   & NWTRMIN1, NWTRMAX1  
ENDIF

IF(LNEWNORMT0) THEN
  WRITE(NULOUT,'('' New norm at t0 - NEWNORMT0 ='',1(1X,I4))')NEWNORMT0
  WRITE(NULOUT,'('' Coeff for q-term in tot-en= '',E26.20)') COENEWQTERM
ENDIF

IF (L_BALANCED_REDUCTION) WRITE (NULOUT,*) 'Calculating vectors for balanced reduction'
IF (L_SOS) WRITE (NULOUT,*) 'Calculating (finite time) Stochastic Optimals'
IF (L_EOFS) WRITE (NULOUT,*) 'Calculating (finite time) EOFs'
WRITE(NULOUT, '(''LSMSSIG_SVINI='',L1)') LSMSSIG_SVINI
!     -----------------------------------------------------------------

!*    4. Open workfile
!     ----------------

IF(NCONF/100 == 6) THEN
  ALLOCATE(RLANBUF(YVAZX%NSIZEG,NITERL+2))
  WRITE(NULOUT,9990) 'RLANBUF  ',SIZE(RLANBUF  ),SHAPE(RLANBUF  )
ENDIF

! tendency perturbations

IF(LFORCE) THEN
  IF (LNMIRQ.AND.NCONF/100 == 6) CALL ABOR1('ASSYMETRIC OPK')
  CALL SUFORCE
ENDIF

!     5. Redine some parameter of GRIB sector 1 for write SVs in
!        GRIB format

!        NSEC1(42) = singular vector number (to be set later in NALAN1
!                    when writing the SVs)
!        NSEC1(43) = number of iterations (NITERL from NAMLCZ)
!        NSEC1(44) = number of SVs computed (to be set later in NALAN1
!                    after call to LANCZOS code)
!        NSEC1(45) = norm used at initial time (NEWNORMT0 from YOMLCZ)
!        NSEC1(46) = norm used at final time (NLANNORM from YOMLCZ)
!        NSEC1(47) = mutiplication factor to convert lat-lon and xkapa
!                    from real to integer (set to 1000000)
!        NSEC1(48) = lat in deg of NW corner of LPO area multiplied by NSEC1(47)
!                    (NSEC1(47)*ALAT1 from YOMLCZ)
!        NSEC1(49) = lon in deg of NW corner of LPO area multiplied by NSEC1(47)
!                    (NSEC1(47)*ALAT1 from YOMLCZ)
!        NSEC1(50) = lat in deg of SE corner of LPO area multiplied by NSEC1(47)
!                    (NSEC1(47)*ALAT1 from YOMLCZ)
!        NSEC1(51) = lon in deg of SE corner of LPO area multiplied by NSEC1(47)
!                    (NSEC1(47)*ALAT1 from YOMLCZ)
!        NSEC1(52) = accuracy  multiplied by NSEC1(47)
!                    (NSEC1(47)*XKAPA from YOMLCZ)
!        NSEC1(53) = number of SVs evolved (NEIGEVO from YOMLCZ)
!        NSEC1(54-55) = they are used to store the ritz numbers (set in
!                       NALAN1). Given RITZ, then:
!                       NSEC1(54)=NINT( LOG10(RITZ)-5 )
!                       NSEC1(55)=NINT( RITZ/( EXP(LOG(10.)*NSEC1(54) )
!                       Thus: RITZ=NSEC1(55)*EXP( LOG(10.)*NSEC1(54) )

IF (NCONF == 601) THEN
  IGRIB_HANDLES=(/NGRIB_HANDLE_GG,NGRIB_HANDLE_GG_ML,NGRIB_HANDLE_SH_ML,&
   & NGRIB_HANDLE_SH/)
  DO JTEMP=1,4
    IGRIB_HANDLE=IGRIB_HANDLES(JTEMP)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'numberOfIterations',NITERL)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'normAtInitialTime',NEWNORMT0)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'normAtFinalTime',NLANNORM)
    IMULF=10000
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'multiplicationFactorForLatLong',IMULF)
    IMDEG=INT(IMULF*ALAT1*180._JPRB/RPI)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'northWestLatitudeOfLPOArea',IMDEG)
    IMDEG=INT(IMULF*ALON1*180._JPRB/RPI)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'northWestLongitudeOfLPOArea',IMDEG)
    IMDEG=INT(IMULF*ALAT3*180._JPRB/RPI)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'southEastLatitudeOfLPOArea',IMDEG)
    IMDEG=INT(IMULF*ALON3*180._JPRB/RPI)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'southEastLongitudeOfLPOArea',IMDEG)
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'accuracyMultipliedByFactor',INT(IMULF*XKAPA))
    CALL IGRIB_SET_VALUE(IGRIB_HANDLE,'numberOfSingularVectorsEvolved',NEIGEVO)
  ENDDO
ENDIF
!     ------------------------------------------------------------------
9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('SULCZ',1,ZHOOK_HANDLE)
END SUBROUTINE SULCZ
