SUBROUTINE SMOS_GP2OBS(PFIELD_GP,PFIELD,KPOL)

!**** *SMOS_GP2OBS* - 
!
!     J. Munoz Sabater   ECMWF    Oct 2009
!
!     PURPOSE.
!     -------
!     SEND SMOS OBS FROM GRID-POINT SPACE TO OBS PROCESSORS
!
!     Copies input real array (PFIELD_GP) in grid-point
!     space to output real array (PFIELD) in obs space.
!
!**   INTERFACE.
!**   ---------
!**   *CALL* * SMOS_GP2OBS()*  (FROM SMOS_UPDATE)
!
!        Explicit arguments :
!
!
!     Modifications :
!
!          J. Munoz Sabater  23-11-2009 : New array dimension por polarisation 
!     ------------------------------------------------------------------

 USE PARKIND1   ,ONLY : JPIM     ,JPRB
 USE YOMMP     , ONLY : MYPROC
 USE YOMCT0    , ONLY : NPROC
 USE YOMGEM    , ONLY : NGPTOT
 USE YOMSMOS   , ONLY : OB_JOBS, OB_FROM, OB_JROF_GP, GP_COUNT,OB_COUNT
 USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV, MPL_BARRIER

 USE YOMHOOK   , ONLY : LHOOK,   DR_HOOK

 IMPLICIT NONE

 REAL(KIND=JPRB),INTENT(IN)    :: PFIELD_GP(NGPTOT)
 REAL(KIND=JPRB),INTENT(OUT)   :: PFIELD(OB_COUNT(MYPROC,KPOL))
 INTEGER(KIND=JPIM),INTENT(IN) :: KPOL
 
 INTEGER(KIND=JPIM)            :: ITAG,IDIM_GP,IDIM_OB,II,IPROC
 REAL(KIND=JPRB),ALLOCATABLE   :: ZBUF_GP(:,:)
 REAL(KIND=JPRB),ALLOCATABLE   :: ZBUF_OBS(:,:)
 REAL(KIND=JPRB),ALLOCATABLE   :: ZBUF(:)
 REAL(KIND=JPRB)               :: ZHOOK_HANDLE

 IF (LHOOK) CALL DR_HOOK('SMOS_GP2OBS',0,ZHOOK_HANDLE)

 ITAG=1

 IF(MYPROC == 1)THEN
   IDIM_GP=MAXVAL(GP_COUNT(:,KPOL))
   IDIM_OB=MAXVAL(OB_COUNT(:,KPOL))
   ALLOCATE(ZBUF_GP(IDIM_GP,NPROC))
   ALLOCATE(ZBUF_OBS(IDIM_OB,NPROC))
 ENDIF

 ALLOCATE(ZBUF(GP_COUNT(MYPROC,KPOL)))

 IF(MYPROC /= 1)THEN
    
    ! Send current obs space contents to processor 1
    IF(OB_COUNT(MYPROC,KPOL) /= 0)THEN
      CALL GSTATS(903,0)
      CALL MPL_SEND(PFIELD(1:OB_COUNT(MYPROC,KPOL)),&
        & KDEST=1,KTAG=ITAG,CDSTRING='SMOS_GP2OBS' )
      CALL GSTATS(903,1)
    ENDIF
    
 ELSE
    
    ZBUF_GP=0.0_JPRB
    ! Gather current obs space contents from other processors
    CALL GSTATS(903,0)
    DO IPROC=2,NPROC
      IF(OB_COUNT(IPROC,KPOL) /= 0)THEN
        CALL MPL_RECV(ZBUF_OBS(1:OB_COUNT(IPROC,KPOL),IPROC),&
         & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='SMOS_GP2OBS' )
      ENDIF
    ENDDO
    CALL GSTATS(903,1)
    ! And from this processor, 1
    ZBUF_OBS(1:OB_COUNT(1,KPOL),1) = PFIELD(1:OB_COUNT(1,KPOL))
    
 ENDIF
 
 ZBUF=0.0_JPRB
 DO II=1,GP_COUNT(MYPROC,KPOL)
   ZBUF(II)=PFIELD_GP(OB_JROF_GP(II,MYPROC,KPOL))
 ENDDO

  IF(MYPROC /=1)THEN
    
    ! Send grid point space contents to processor 1
    IF(GP_COUNT(MYPROC,KPOL) /= 0)THEN
      CALL GSTATS(903,0)
      CALL MPL_SEND(ZBUF(1:GP_COUNT(MYPROC,KPOL)),&
       & KDEST=1,KTAG=ITAG,CDSTRING='SMOS_GP2OBS' )
      CALL GSTATS(903,1)
    ENDIF
    
  ELSE
    
    ZBUF_GP(1:GP_COUNT(1,KPOL),1)= ZBUF(1:GP_COUNT(1,KPOL))
    
    ! Gather grid point info from other processors   
    CALL GSTATS(903,0)
    DO IPROC=2,NPROC
      IF(GP_COUNT(IPROC,KPOL) /= 0)THEN
        CALL MPL_RECV(ZBUF_GP(1:GP_COUNT(IPROC,KPOL),IPROC),&
         & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='SMOS_GP2OBS' )
      ENDIF
    ENDDO
    CALL GSTATS(903,1)
    
    ! Map back from grid point to obs space   
    DO IPROC=1,NPROC
      DO II=1,GP_COUNT(IPROC,KPOL)
        ZBUF_OBS(OB_JOBS(II,IPROC,KPOL),OB_FROM(II,IPROC,KPOL))= &
                 & ZBUF_GP(II,IPROC)
      ENDDO
    ENDDO
    
  ENDIF


! Send back to PE that owns OB

  IF(MYPROC == 1)THEN
 
    CALL GSTATS(903,0)
    DO IPROC=2,NPROC
      IF(OB_COUNT(IPROC,KPOL) /= 0)THEN
        CALL MPL_SEND(ZBUF_OBS(1:OB_COUNT(IPROC,KPOL),IPROC),&
         & KDEST=IPROC,KTAG=ITAG,CDSTRING='SMOS_GP2OBS' )
      ENDIF
    ENDDO
    CALL GSTATS(903,1)
    
    PFIELD(1:OB_COUNT(1,KPOL)) = ZBUF_OBS(1:OB_COUNT(1,KPOL),1)
    
  ELSE
    
    IF(OB_COUNT(MYPROC,KPOL) /= 0)THEN
      CALL GSTATS(903,0)
      CALL MPL_RECV(PFIELD(1:OB_COUNT(MYPROC,KPOL)),&
       & KSOURCE=1,KTAG=ITAG,CDSTRING='SMOS_GP2OBS' )
      CALL GSTATS(903,1)
    ENDIF
  
  ENDIF


 IF(MYPROC == 1)THEN
   DEALLOCATE(ZBUF_GP)
   DEALLOCATE(ZBUF_OBS)
 ENDIF

 DEALLOCATE(ZBUF)

 CALL GSTATS(909,0)
 CALL MPL_BARRIER(CDSTRING='SMOS_GP2OBS')
 CALL GSTATS(909,1)

 IF (LHOOK) CALL DR_HOOK('SMOS_GP2OBS',1,ZHOOK_HANDLE)

END SUBROUTINE SMOS_GP2OBS
