SUBROUTINE SMOS_IGP2OBS(KFIELD_GP,KFIELD,KPOL)

!**** *SMOS_IGP2OBS* - 
!
!     J. Munoz Sabater       ECMWF   Oct 2009
!
!     PURPOSE.
!     -------
!     SEND SMOS MASK FROM GRID-POINT SPACE TO OBS PROCESSORS
!
!     Copies input integer array (KFIELD_GP) in grid-point
!     space to output integer array (KFIELD) in obs space.
!
!**   INTERFACE.
!**   ---------
!**   *CALL* * 	SMOS_IGP2OBS()*  (FROM SMOS_UPDATE)
!
!        Explicit arguments :
!
!     Modifications :
!     J. Muñoz Sabater  26-11-2009 : Added extra dimension for polarisations
!
!
!     ------------------------------------------------------------------

 USE PARKIND1  , ONLY : JPIM     ,JPRB
 USE YOMMP     , ONLY : MYPROC
 USE YOMCT0    , ONLY : NPROC
 USE YOMGEM    , ONLY : NGPTOT
 USE YOMSMOS   , ONLY : OB_JOBS, OB_FROM, OB_JROF_GP, GP_COUNT, OB_COUNT
 USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV, MPL_BARRIER

 USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

 IMPLICIT NONE

 INTEGER(KIND=JPIM),INTENT(IN)  :: KFIELD_GP(NGPTOT)
 INTEGER(KIND=JPIM),INTENT(OUT) :: KFIELD(OB_COUNT(MYPROC,KPOL))
 INTEGER(KIND=JPIM),INTENT(IN)  :: KPOL

 INTEGER(KIND=JPIM)             :: ITAG,IDIM_GP,IDIM_OB,II,IPROC
 INTEGER(KIND=JPIM),ALLOCATABLE :: IBUF_GP(:,:)
 INTEGER(KIND=JPIM),ALLOCATABLE :: IBUF_OBS(:,:)
 INTEGER(KIND=JPIM),ALLOCATABLE :: IBUF(:)
 REAL(KIND=JPRB)                :: ZHOOK_HANDLE

 IF (LHOOK) CALL DR_HOOK('SMOS_IGP2OBS',0,ZHOOK_HANDLE)

 ITAG=1

 IF(MYPROC /= 1)THEN

  ! Send current contents of obs space to processor 1
  IF(OB_COUNT(MYPROC,KPOL) /= 0)THEN
     CALL GSTATS(905,0)
     CALL MPL_SEND(KFIELD(1:OB_COUNT(MYPROC,KPOL)),&
         & KDEST=1,KTAG=ITAG,CDSTRING='SMOS_IGP2OBS' )
     CALL GSTATS(905,1)
  ENDIF
  
 ELSE
  
  ! A central buffer of all obs and grid space data in processor 1
  IDIM_GP=MAXVAL(GP_COUNT(:,KPOL))
  IDIM_OB=MAXVAL(OB_COUNT(:,KPOL))
  ALLOCATE(IBUF_GP(IDIM_GP,NPROC))
  ALLOCATE(IBUF_OBS(IDIM_OB,NPROC))
  IBUF_GP=0
  IBUF_OBS=0
  
  ! Gather the current obs space contents
  CALL GSTATS(905,0)
  DO IPROC=2,NPROC
    IF(OB_COUNT(IPROC,KPOL) /= 0)THEN
      CALL MPL_RECV(IBUF_OBS(1:OB_COUNT(IPROC,KPOL),IPROC),&
         & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='SMOS_IGP2OBS' )
    ENDIF
  ENDDO
  CALL GSTATS(905,1)
  
  ! And from this processor (1)
  IBUF_OBS(1:OB_COUNT(1,KPOL),1) = KFIELD(:) 
  
 ENDIF

 ALLOCATE(IBUF(GP_COUNT(MYPROC,KPOL)))
 IBUF=0

 DO II=1,GP_COUNT(MYPROC,KPOL)
   IBUF(II)=KFIELD_GP(OB_JROF_GP(II,MYPROC,KPOL))
 ENDDO


! Send back to MYPROC=1 and copy back to OBS addressing

 IF(MYPROC /=1)THEN
   IF(GP_COUNT(MYPROC,KPOL) /= 0)THEN
     CALL GSTATS(905,0)
     CALL MPL_SEND(IBUF(1:GP_COUNT(MYPROC,KPOL)),&
          & KDEST=1,KTAG=ITAG,CDSTRING='SMOS_IGP2OBS' )
     CALL GSTATS(905,1)
  ENDIF

 ELSE

 IBUF_GP(1:GP_COUNT(1,KPOL),1)= IBUF(1:GP_COUNT(1,KPOL))

  CALL GSTATS(905,0)
  DO IPROC=2,NPROC
    IF(GP_COUNT(IPROC,KPOL) /= 0)THEN
      CALL MPL_RECV(IBUF_GP(1:GP_COUNT(IPROC,KPOL),IPROC),&
       & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='SMOS_IGP2OBS' )
    ENDIF
  ENDDO
  CALL GSTATS(905,1)

  DO IPROC=1,NPROC
    DO II=1,GP_COUNT(IPROC,KPOL)
      IBUF_OBS(OB_JOBS(II,IPROC,KPOL),OB_FROM(II,IPROC,KPOL))= &
              & IBUF_GP(II,IPROC)
    ENDDO
  ENDDO

 ENDIF

  
! Send back to PE that owns OB
  
  IF(MYPROC == 1)THEN
  
    CALL GSTATS(905,0)
    DO IPROC=2,NPROC
      IF(OB_COUNT(IPROC,KPOL) /= 0)THEN
        CALL MPL_SEND(IBUF_OBS(1:OB_COUNT(IPROC,KPOL),IPROC),&
         & KDEST=IPROC,KTAG=ITAG,CDSTRING='SMOS_IGP2OBS' )
      ENDIF
    ENDDO
    CALL GSTATS(905,1)
  
    KFIELD(1:OB_COUNT(1,KPOL)) = IBUF_OBS(1:OB_COUNT(1,KPOL),1)
  
  ELSE
  
    IF(OB_COUNT(MYPROC,KPOL) /= 0)THEN
      CALL GSTATS(905,0)
      CALL MPL_RECV(KFIELD(1:OB_COUNT(MYPROC,KPOL)),&
       & KSOURCE=1,KTAG=ITAG,CDSTRING='SMOS_IGP2OBS' )
      CALL GSTATS(905,1)
    ENDIF
  
  
  ENDIF


 IF(MYPROC == 1)THEN
   DEALLOCATE(IBUF_GP)
   DEALLOCATE(IBUF_OBS)
 ENDIF

 DEALLOCATE(IBUF)

 CALL GSTATS(911,0)
 CALL MPL_BARRIER(CDSTRING='SMOS_IGP2OBS')
 CALL GSTATS(911,1)

 IF (LHOOK) CALL DR_HOOK('SMOS_IGP2OBS',1,ZHOOK_HANDLE)

 END SUBROUTINE SMOS_IGP2OBS
