SUBROUTINE smos_obsop (kstart,kend,kproma,klevs,kpol, &     ! in    : dimensions  
                      &  yd_smos_phys_fw, yd_smos_tb) ! in/out: 2d/3d fields, fw TB
!
!**** Observation operator (land emission model) for passive microwave TB modelling 
!     low-frequencies (<20 GHz)
!     
!
!     J. Munoz Sabater   ECMWF    Oct 2009
!
!
!     SMOS_OBSOP is called from CALLPAR (ifs/phys_ec/)
!
!     Modifications:
!
!
!**** MODULES
!
!* KIND     
 use parkind1, only: jpim, jprb
 use yomhook , only: lhook, dr_hook
!
!* IFS
 use yomsmos , only: smos_phys_type, smos_tb_type, smos_active
 use yomcmemtypes  
!
 implicit none
!          
!* Common variables
 integer (kind=jpim)  , intent (in)    :: kstart, kend, kproma, klevs, kpol
!
 type (smos_phys_type), intent (inout) :: yd_smos_phys_fw (kproma)
 type (smos_tb_type)  , intent (inout) :: yd_smos_tb      (kproma)
!
! Local variables
 integer (kind=jpim) :: iprof, inprof, i
 integer (kind=jpim), dimension (kproma) :: jprof

 real (kind=jprb), dimension (kproma,klevs) :: zphys_sm_fw, zphys_st_fw
 real (kind=jprb), dimension (kproma)   :: zphys_skt_fw, zphys_t2m_fw,&
        & zphys_sndt_fw, zphys_sndp_fw, zphys_laih_fw,&
        & zphys_lail_fw, zphys_fch_fw,  zphys_fcl_fw, &
        & zphys_ang_fw, zphys_or_fw,  zphys_lsm_fw
 integer (kind=jpim), dimension (kproma) ::  iphys_vegth_fw, iphys_vegtl_fw, iphys_stp_fw 
 real (kind=jprb), dimension(2)        :: zrad_tb_fw
 REAL(KIND=JPRB),  DIMENSION(kproma,2) :: zoutput

! adjustment variables
 real (kind=jprb) :: zhook_handle

!
!* INTERFACES
!#include "cmem_alloc_types.intfb.h" 
!#include "cmem_main.intfb.h"
!#include "cmem_dealloc_types.intfb.h"
!
 IF (lhook) CALL dr_hook ('SMOS_OBSOP',0,zhook_handle)
!  
!* Extract active profiles
 inprof    = 0
 jprof (:) = 0
  do i = kstart, kend
    if ( yd_smos_tb (i) % screen (kpol) == smos_active ) then 
     inprof = inprof + 1
     jprof (inprof) = i
     !
     zphys_sm_fw    (inprof,1:klevs) = yd_smos_phys_fw (i) % sm (1:klevs)
     zphys_st_fw    (inprof,1:klevs) = yd_smos_phys_fw (i) % st (1:klevs)
     !
     zphys_skt_fw   (inprof) = yd_smos_phys_fw (i) % skt
     zphys_t2m_fw   (inprof) = yd_smos_phys_fw (i) % t2m
     zphys_sndt_fw  (inprof) = yd_smos_phys_fw (i) % sndt
     zphys_sndp_fw  (inprof) = yd_smos_phys_fw (i) % sndp
     zphys_laih_fw  (inprof) = yd_smos_phys_fw (i) % laih
     zphys_lail_fw  (inprof) = yd_smos_phys_fw (i) % lail
     zphys_fch_fw   (inprof) = yd_smos_phys_fw (i) % fch
     zphys_fcl_fw   (inprof) = yd_smos_phys_fw (i) % fcl
     iphys_vegth_fw (inprof) = yd_smos_phys_fw (i) % vegth
     iphys_vegtl_fw (inprof) = yd_smos_phys_fw (i) % vegtl

     zphys_or_fw    (inprof) = yd_smos_phys_fw (i) % or
     zphys_lsm_fw   (inprof) = yd_smos_phys_fw (i) % lsm
     iphys_stp_fw   (inprof) = yd_smos_phys_fw (i) % stp

     zphys_ang_fw   (inprof) = yd_smos_tb      (i) % ang (kpol)
   endif
  enddo  
!
 if (inprof > 0) then
    !* ALLOCATE CMEM variables (enable THREADS saving)
    !  call cmem_alloc_types (inprof, zd_cmem_block)
    !* CALL CMEM
    do i=1,inprof
       call cmem_main(klevs,1       ,& ! in: soil levels, profiles
           ! & zd_cmem_block               ,& ! inout: allocated cmem variables 
            & zphys_sm_fw    (i,:) ,& ! in: soil moisture 
            & zphys_st_fw    (i,:) ,& ! in: soil temperature
            & zphys_skt_fw   (i)   ,& ! in: skin temperature
            & zphys_t2m_fw   (i)   ,& ! in: 2m temperature
            & zphys_sndt_fw  (i)   ,& ! in: snow density
            & zphys_sndp_fw  (i)   ,& ! in: snow depth
            & zphys_laih_fw  (i)   ,& ! in: LAI high vegetation
            & zphys_lail_fw  (i)   ,& ! in: LAI low vegetation
            & zphys_fch_fw   (i)   ,& ! in: fraction cover high vegetation
            & zphys_fcl_fw   (i)   ,& ! in: fraction cover low vegetation
            & iphys_vegth_fw (i)   ,& ! in: high vegetation type
            & iphys_vegtl_fw (i)   ,& ! in: low vegetation type
            & zphys_or_fw    (i)   ,& ! in: geopotential at surface (orography)
            & zphys_lsm_fw   (i)   ,& ! in: land-sea mask
            & iphys_stp_fw   (i)   ,& ! in: soil type
            & zphys_ang_fw   (i)   ,& ! in: incidence angle
            & zrad_tb_fw     (:) )    ! out: TB's

             zoutput(i,:) = zrad_tb_fw(:)
    enddo
    !* DEALLOCATE CMEM variables 
    !   call cmem_dealloc_types (zd_cmem_block)

   !* Restore to kproma-array     
   do iprof = 1, inprof
      i = jprof (iprof)
        if (kpol == 0) yd_smos_tb (i) % fw (kpol) = zoutput (iprof,1) ! H-pol
        if (kpol == 1) yd_smos_tb (i) % fw (kpol) = zoutput (iprof,2) ! V-pol
   enddo
 
 endif
       
 IF (lhook) CALL dr_hook ('SMOS_OBSOP',1,zhook_handle)        
    
END SUBROUTINE smos_obsop
