SUBROUTINE smos_screen (kstart,kend,kproma,klevs, &    ! in    : dimensions  
                       &  yd_smos_phys, yd_smos_tb)    ! in/out: 2d/3d fields, fw TB
!
!**** Observation operator (land emission model) for passive microwave TB modelling 
!     low-frequencies (<20 GHz)
!     
!
!     J. Munoz Sabater   ECMWF    Oct 2009
!
!
!     SMOS_SCREEN is called from CALLPAR (ifs/phys_ec/)
!
!     Modifications:
!
!
!**** MODULES
!
!* KIND     
 USE parkind1, ONLY: jpim, jprb
 USE yomhook , ONLY: lhook, dr_hook

!* IFS
 USE yomsmos , ONLY: npol_max, smos_phys_type, smos_tb_type, &
                   & smos_active, smos_no_obs, smos_obs_tb, smos_use, smos_sea

 IMPLICIT NONE
          
!* Common variables
 INTEGER (KIND=jpim)  , INTENT (IN)    :: kstart, kend, kproma, klevs

 TYPE (smos_phys_type), INTENT (INOUT) :: yd_smos_phys (kproma)
 TYPE (smos_tb_type)  , INTENT (INOUT) :: yd_smos_tb   (kproma)

! Local variables
 INTEGER (KIND=jpim) :: i
 INTEGER (KIND=jpim) :: ipol

! adjustment variables
 real (kind=jprb) :: zhook_handle

!* INTERFACES
#include "smos_obsop.intfb.h"


 IF (lhook) CALL dr_hook ('SMOS_SCREEN',0,zhook_handle)
  

 DO ipol = 0, npol_max-1 
  
  DO i = kstart, kend
     !* Skip grid points that are not related to an observation
     IF ( btest( yd_smos_tb (i) % screen (ipol), smos_no_obs ) ) cycle
     
     !* TB limits
     IF (yd_smos_tb (i) % obs (ipol) < 50.0_JPRB .or. yd_smos_tb (i) % obs (ipol) > 350.0_JPRB) &
       & yd_smos_tb (i) % screen(ipol) = ibset (yd_smos_tb (i) % screen(ipol), smos_obs_tb)
     
     !* Check for sea or lake points (just for information)
     IF (yd_smos_phys (i) % lsm < 0.5_JPRB) &
       & yd_smos_tb (i) % screen (ipol) = ibset (yd_smos_tb (i) % screen(ipol), smos_sea)
     
     !* Set active flag    
     IF (yd_smos_tb (i) % screen (ipol) == 0) &
       & yd_smos_tb (i) % screen (ipol) = ibset (yd_smos_tb (i) % screen(ipol), smos_use)
     
     ! All other possible rejections here
     
  ENDDO ! ipol-loop
   
 ENDDO  ! i-loop
 
 DO ipol = 0, npol_max-1
    
    IF (any ( yd_smos_tb (kstart:kend) % screen (ipol) == smos_active )) THEN
       
       !* Call forward operator
       CALL smos_obsop (kstart, kend, kproma, klevs, ipol, yd_smos_phys, yd_smos_tb)
       
    ENDIF
    
 ENDDO ! ipol-loop          
 
 IF (lhook) CALL dr_hook ('SMOS_SCREEN',1,zhook_handle)        
    
END SUBROUTINE smos_screen
