SUBROUTINE SMOS_UPDATE(PSMOS_TB_BUF,KSMOS_BUF)

!*****SMOS_UPDATE* - 
!
!     J. Muñoz Sabater   ECMWF  Oct-2009
!
!     PURPOSE.
!     -------
!     PUT BRIGHTNESS TEMPERATURES FROM GRID-POINT SPACE TO ODB
!
!**   INTERFACE.
!**   ---------
!**   *CALL* *SMOS_UPDATE()*  (FROM GP_MODEL)
!
!        Explicit arguments :
!        --------------------
!
!     Modifications :
!          J. Munoz Sabater  26-11-2009 : New dimension por polarisation 
!
!
!     ------------------------------------------------------------------

 USE PARKIND1   ,ONLY : JPIM     ,JPRB
 USE YOMHOOK    ,ONLY : LHOOK,   DR_HOOK
 USE YOMDB
 USE YOMMP     , ONLY : MYPROC
 USE YOMCT0    , ONLY : NPROC, LSCREEN
 USE YOMVNMB   , ONLY : NVNUMB
 USE YOMNMEV   , ONLY : NR1EVENT
 USE YOMECTAB  , ONLY : NTSLTOB,NTSLOTNOS,NTSLOTNOE
 USE YOMGEM    , ONLY : NGPTOT
 USE YOMSMOS   , ONLY : OB_COUNT, GP_COUNT, OB_JOBS, OB_FROM, &
                      & OB_JROF_GP, OB_INDEX, SMOS_ACTIVE, NPOL_MAX
 USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV
 USE YOMLUN    , ONLY : NULOUT

 IMPLICIT NONE

 REAL(KIND=JPRB)   ,INTENT(IN)  :: PSMOS_TB_BUF (NGPTOT,0:NPOL_MAX-1)
 INTEGER(KIND=JPIM),INTENT(IN)  :: KSMOS_BUF    (NGPTOT,0:NPOL_MAX-1)

 INTEGER(KIND=JPIM) :: JOBS
 REAL(KIND=JPRB),   ALLOCATABLE :: ZSMOS_TB(:)
 INTEGER(KIND=JPIM),ALLOCATABLE :: ISMOS(:)
 INTEGER(KIND=JPIM)             :: IPOL
 INTEGER(KIND=JPIM)             :: IRET, IVNM, ITS, ILEN, JBODY, IBODY
 INTEGER(KIND=JPIM)             :: IMXBDY, IPROMBMP, INFO(3)
 INTEGER(KIND=JPIM),ALLOCATABLE :: ICMBDY(:)
 INTEGER(KIND=JPIM)             :: IODB_COUNT(NPROC)
 INTEGER(KIND=JPIM)             :: IODBTOTG,ITAG,IPROC
 REAL(KIND=JPRB)                :: ZINFO(0), ZOBSERR
 REAL(KIND=JPRB)                :: ZHOOK_HANDLE


INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE
!
#include "smos_igp2obs.intfb.h"
#include "smos_gp2obs.intfb.h"
#include "fcobs.h"
#include "openmp_obs.h"

 IF (LHOOK) CALL DR_HOOK('SMOS_UPDATE',0,ZHOOK_HANDLE)

DO IPOL = 0, NPOL_MAX-1

 ITS      = NTSLOTNOS
 IPROMBMP = NTSLTOB(ITS) ! IOBTOT

 ALLOCATE(ZSMOS_TB(OB_COUNT(MYPROC,IPOL)))
 ALLOCATE(ISMOS(OB_COUNT(MYPROC,IPOL)))

 ISMOS=0
 ZSMOS_TB=0.0_JPRB


  CALL SMOS_IGP2OBS (KSMOS_BUF(:,IPOL),   ISMOS   ,IPOL)
  CALL SMOS_GP2OBS  (PSMOS_TB_BUF(:,IPOL),ZSMOS_TB,IPOL)

! This variables are not used in GETDB, since they are hardcoded in SQL request
! IOBSTYPE= NSMOS    ! SMOS obstype
! ICDTYP  = NSMOS     ! SMOS codetype
! ISENSOR = 18        ! SMOS sensor identification number
  INFO(1) = NTSLOTNOS
  INFO(2) = NTSLOTNOE
  INFO(3) = IPOL 


 IF(LSCREEN)THEN
    CALL GETDB('MWAVE_UPDATE_SMOS',0, ILEN, INFO, 3, ZINFO, 0, &
         & -1, ITS, -1, -1, -1, -1)  
 ENDIF

 ALLOCATE(ICMBDY(ILEN))

 DO JOBS = 1,ILEN
   ICMBDY(JOBS)=MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
 ENDDO
 
 IMXBDY=MAXVAL(ICMBDY(1:ILEN))

 IODB_COUNT=0
 DO JOBS=1,OB_COUNT(MYPROC,IPOL)
   IF(ISMOS(JOBS) == SMOS_ACTIVE) THEN
      IODB_COUNT(MYPROC)=IODB_COUNT(MYPROC)+1
   ENDIF
 ENDDO

! Send to MYPROC = 1
 ITAG=1

 CALL GSTATS(907,0)
 IF(MYPROC /= 1) THEN
   CALL MPL_SEND(IODB_COUNT(MYPROC),&
        & KDEST=1,KTAG=ITAG,CDSTRING='SMOS_UPDATE' )
 ELSE
   DO IPROC=2,NPROC
     CALL MPL_RECV(IODB_COUNT(IPROC),&
          & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='SMOS_UPDATE' )
   ENDDO
 ENDIF
 CALL GSTATS(907,1)

 IF (MYPROC == 1) THEN
   IODBTOTG=SUM(IODB_COUNT)
   WRITE(NULOUT,*)'ODB_COUNT',IODBTOTG,IODB_COUNT
 ENDIF

 CALL GSTATS(2013,0)
 DO JOBS=1,ILEN
    ROBHDR (JOBS,MDB_TB_FLAG_SMOS)  = ISMOS (JOBS)    ! grid-point flag
    ROBHDR (JOBS,MDB_TBVALUE_SMOS)       = ZSMOS_TB (JOBS) ! Background value     
    IF (ISMOS (JOBS) /= SMOS_ACTIVE) THEN       
      ROBHDR (JOBS,MDBRST)  = ZCHSTAT_REJECT (ROBHDR (JOBS,MDBRST))
      ROBHDR (JOBS,MDBREV1) = ZCHEV1 (ROBHDR (JOBS,MDBREV1),NR1EVENT(16))
    ENDIF
 ENDDO

 DO JBODY=1,IMXBDY
   DO JOBS=1,ILEN
     IF (JBODY > ICMBDY(JOBS))CYCLE
     IBODY = MLNKH2B(JOBS)+(JBODY-1)
     IVNM  = ROBODY(IBODY,MDBVNM)
     IF (IVNM == NVNUMB(97)) THEN          
       IF (ISMOS (JOBS) /= SMOS_ACTIVE) THEN       
             ROBODY(IBODY,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IBODY,MDBDSTA))
          ELSE
             ZOBSERR = 0.04_JPRB
             ROBODY (IBODY,MDBOER) = ZOBSERR                        ! Initial TB Obserr
       ENDIF
    ENDIF
  ENDDO
 ENDDO
 CALL GSTATS(2013,1)
  
 CALL PUTDB('MWAVE_UPDATE_SMOS', 0, IRET, INFO, 0, ZINFO, 0)

 DEALLOCATE(ICMBDY)
 DEALLOCATE(ZSMOS_TB)
 DEALLOCATE(ISMOS)

 IF (IPOL == NPOL_MAX-1) THEN
   DEALLOCATE(OB_COUNT)
   DEALLOCATE(GP_COUNT)
   DEALLOCATE(OB_FROM)
   DEALLOCATE(OB_JOBS)
   DEALLOCATE(OB_INDEX)
   DEALLOCATE(OB_JROF_GP)
 ENDIF

ENDDO ! ipol-loop

 IF (LHOOK) CALL DR_HOOK('SMOS_UPDATE',1,ZHOOK_HANDLE)

END SUBROUTINE SMOS_UPDATE
