SUBROUTINE REESPE(KFLSUR,KFIELD,PSPEC,PREEL)

!**** *REESPE* -Direct transform to Spherical harmonics

!     Purpose.
!     --------
!     Direct transform from grid-point to spectral space.

!**   Interface.
!     ----------
!        *CALL* *REESPE(KFLSUR,KFIELD,PSPEC,PREEL)

!        Explicit arguments :
!        --------------------
!           KFLSUR   : first dimension of PSPEC
!           KFIELD   : number of fields to be transformed
!           PSPEC    : spectral array (output)
!           PREEL    : grid point array (input)

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.  DIR_TRANS - direct transform (TRANS lib)
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        Modified : 95-07-07 G. Mozdzynski - support distributed memory
!        =99-03-29= Deborah Salmond  JUMP in FTINV changed to 1
!        Modified : 99-05-28  D.Salmond - Optimise copies in trltom/trmtol. 
!        Modified : 04/06/99 D.Salmond : change order of AIA and SIA
!        Modified : 25/10/00 M.Hamrud  : TRANS package
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSPEC2 
USE YOMCT0   , ONLY : NPRTRV
USE YOMGEM   , ONLY : NGPTOT
USE YOMMP    , ONLY : MYSETV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLSUR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELD 
REAL(KIND=JPRB)                  :: PSPEC(KFLSUR,NSPEC2) ! UNDETERMINED INTENT
REAL(KIND=JPRB)                  :: PREEL(NGPTOT,KFIELD,1) ! UNDETERMINED INTENT

INTEGER(KIND=JPIM) :: ILF,ISTG,IFLDPP,IREST,IGF
INTEGER(KIND=JPIM) :: INUMLF(NPRTRV),IVSETSC(KFIELD)
INTEGER(KIND=JPIM) :: JROCB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "dir_trans.h"
END INTERFACE

!  Initialisation
IF (LHOOK) CALL DR_HOOK('REESPE',0,ZHOOK_HANDLE)
ILF = 0
IF( KFIELD == 1 )THEN
  IVSETSC(1) = NPRTRV
  IF(NPRTRV == MYSETV) THEN
    ILF = 1
  ENDIF
  ISTG=2
ELSE
  IFLDPP=KFIELD/NPRTRV
  IREST=KFIELD-IFLDPP*NPRTRV
  DO JROCB=1,NPRTRV
    IF( JROCB <= IREST )THEN
      INUMLF(JROCB)=IFLDPP+1
    ELSE
      INUMLF(JROCB)=IFLDPP
    ENDIF
  ENDDO
  ILF = INUMLF(MYSETV)
  ISTG = 1
  DO JROCB=1,NPRTRV
    IVSETSC(ISTG:ISTG+INUMLF(JROCB)-1) = JROCB
    ISTG = ISTG+INUMLF(JROCB)
  ENDDO
ENDIF
IGF = ISTG-1

CALL DIR_TRANS(PSPSCALAR=PSPEC(1:ILF,:),KPROMA=NGPTOT,KVSETSC=IVSETSC(1:IGF),&
 & PGP=PREEL)  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('REESPE',1,ZHOOK_HANDLE)
END SUBROUTINE REESPE
