SUBROUTINE SPEC2GRID(YDSP,YDGP)

!   Purpose
!   -------
!     Spectral transform from spectral fields to grid point fields.

!   Author
!   ------
!     Y. Tremolet

!   Modifications
!   -------------
!     Original   19-Jan-04
!     Y.Tremolet 21-Aug-04 Remove unnecessary copies
!     Y.Tremolet 05-Apr-05 Fix for 2D fields
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMMP   , ONLY: NBSETLEV, NBSETSP
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN)    :: YDSP
TYPE(GRIDPOINT_FIELD),INTENT(INOUT) :: YDGP

LOGICAL :: LLWIND, LL2D
INTEGER(KIND=JPIM) :: ivsetsc2(1)
INTEGER(KIND=JPIM) :: jj,jf
REAL(KIND=JPRB), ALLOCATABLE :: ZSP(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "inv_trans.h"
END INTERFACE
#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPEC2GRID',0,ZHOOK_HANDLE)
IF (YDSP%NS3D/=YDGP%NG3D) CALL ABOR1('SPEC2GRID: Error nb 3D fields')
IF (YDSP%NS2G/=YDGP%NG2D) CALL ABOR1('SPEC2GRID: Error nb 2D fields')

! If spectral vor and div and GP U and V convert winds,
! otherwise treat all fields as scalar fields.
! This could be improved a lot...
llwind=  (ASSOCIATED(YDSP%DIV).AND.ASSOCIATED(YDSP%VOR)) &
 & .AND. (ASSOCIATED(YDGP%W_U).AND.ASSOCIATED(YDGP%W_V))
ll2d = (YDSP%NS2G>0 .AND. YDGP%NG2D>0)

IF (LL2D) THEN
  IVSETSC2(1)=NBSETSP
  ALLOCATE(ZSP(1:YDSP%NS2G,YDSP%NSPEC2))
  DO jf=1,YDSP%NS2D
    DO jj=1,YDSP%NSPEC2
      ZSP(jf,jj)=YDSP%SP2D(jj,jf)
    ENDDO
  ENDDO
  IF (LLWIND) THEN
    CALL INV_TRANS(PSPVOR=YDSP%VOR,  PSPDIV=YDSP%DIV,  &
                 & PGPUV =YDGP%WIND, KVSETUV=NBSETLEV, &
                 & PSPSC3A=YDSP%SCAL , PGP3A  =YDGP%SCAL, &
                 & KVSETSC3A=NBSETLEV, KPROMA=YDGP%NPROMA, &
                 & PSPSC2=ZSP, PGP2=YDGP%GP2D, KVSETSC2=IVSETSC2)
  ELSE
    IF(ASSOCIATED(YDSP%SCAL))THEN
       CALL INV_TRANS(PSPSC3A=YDSP%SCAL , PGP3A  =YDGP%SCAL, &
                 & KVSETSC3A=NBSETLEV, KPROMA=YDGP%NPROMA, &
                 & PSPSC2=ZSP, PGP2=YDGP%GP2D, KVSETSC2=IVSETSC2)
    ELSE
       CALL INV_TRANS( KPROMA=YDGP%NPROMA, &
                 & PSPSC2=ZSP, PGP2=YDGP%GP2D, KVSETSC2=IVSETSC2)
    ENDIF
  ENDIF
  DEALLOCATE(ZSP)
ELSE
  IF (LLWIND) THEN
    CALL INV_TRANS(PSPVOR=YDSP%VOR,  PSPDIV=YDSP%DIV,  &
                 & PGPUV =YDGP%WIND, KVSETUV=NBSETLEV, &
                 & PSPSC3A=YDSP%SCAL , PGP3A  =YDGP%SCAL, &
                 & KVSETSC3A=NBSETLEV, KPROMA=YDGP%NPROMA)
  ELSE
    IF(ASSOCIATED(YDSP%SCAL))THEN
       CALL INV_TRANS(PSPSC3A=YDSP%SCAL , PGP3A  =YDGP%SCAL, &
                 & KVSETSC3A=NBSETLEV, KPROMA=YDGP%NPROMA)
    ENDIF
  ENDIF
ENDIF

CALL HUGIFY_ENDGRID(YDGP)

IF (LHOOK) CALL DR_HOOK('SPEC2GRID',1,ZHOOK_HANDLE)
END SUBROUTINE SPEC2GRID
