SUBROUTINE SPEREE_DER(KFLSUR,KFIELD,PSPEC,PREEL,PREELL,PREELM)

!**** *SPEREE_DER* -Inverse transform to Grid-point space

!     Purpose.
!     --------
!     Transform from Spectral space to grid-point space.
!     Version of SPEREE which also computes the horizontal
!     derivatives.

!**   Interface.
!     ----------
!        *CALL* *SPEREE_DER(...)

!        Explicit arguments :
!        --------------------
!           KFLSUR   : first dimension of PSPEC
!           KFIELD   : number of fields to be transformed
!           PSPEC    : spectral input array
!           PREEL    : grid point output array
!           PREELL   : grid point output array: zonal derivative
!           PREELM   : grid point output array: merid derivative

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.  INV_TRANS - inverse transform (TRANS lib)
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad (CNRM/GMAP), after SPEREE.

!     Modifications.
!     --------------
!        Original : Oct 2005
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSPEC2 
USE YOMCT0   , ONLY : NPRTRV
USE YOMGEM   , ONLY : NGPTOT
USE YOMMP    , ONLY : MYSETV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLSUR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELD 
REAL(KIND=JPRB),INTENT(IN)       :: PSPEC(KFLSUR,NSPEC2)
REAL(KIND=JPRB),INTENT(OUT)      :: PREEL(NGPTOT,KFIELD,1)
REAL(KIND=JPRB),INTENT(OUT)      :: PREELL(NGPTOT,KFIELD,1)
REAL(KIND=JPRB),INTENT(OUT)      :: PREELM(NGPTOT,KFIELD,1)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ILF,ISTG,IFLDPP,IREST,IGF
INTEGER(KIND=JPIM) :: INUMLF(NPRTRV),IVSETSC(KFIELD)
INTEGER(KIND=JPIM) :: JROCB
REAL(KIND=JPRB)    :: ZREEL(NGPTOT,3*KFIELD,1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "inv_trans.h"
END INTERFACE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPEREE_DER',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!  INITIALISATION

ILF = 0
IF( KFIELD == 1 )THEN
  IVSETSC(1) = NPRTRV
  IF(NPRTRV == MYSETV) THEN
    ILF = 1
  ENDIF
  ISTG=2
ELSE
  IFLDPP=KFIELD/NPRTRV
  IREST=KFIELD-IFLDPP*NPRTRV
  DO JROCB=1,NPRTRV
    IF( JROCB <= IREST )THEN
      INUMLF(JROCB)=IFLDPP+1
    ELSE
      INUMLF(JROCB)=IFLDPP
    ENDIF
  ENDDO
  ILF = INUMLF(MYSETV)
  ISTG = 1
  DO JROCB=1,NPRTRV
    IVSETSC(ISTG:ISTG+INUMLF(JROCB)-1) = JROCB
    ISTG = ISTG+INUMLF(JROCB)
  ENDDO
ENDIF
IGF = ISTG-1

CALL INV_TRANS(PSPSCALAR=PSPEC(1:ILF,:),LDSCDERS=.TRUE.,KPROMA=NGPTOT,&
 & KVSETSC=IVSETSC(1:IGF),PGP=ZREEL)  

PREEL (1:NGPTOT,1:KFIELD,1)=ZREEL(1:NGPTOT,1:KFIELD,1)
PREELM(1:NGPTOT,1:KFIELD,1)=ZREEL(1:NGPTOT,KFIELD+1:2*KFIELD,1)
PREELL(1:NGPTOT,1:KFIELD,1)=ZREEL(1:NGPTOT,2*KFIELD+1:3*KFIELD,1)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPEREE_DER',1,ZHOOK_HANDLE)
END SUBROUTINE SPEREE_DER

