SUBROUTINE SPOLTS(KULOUT,KFIELD,KFLDSUR,&
 & KNSMAX,KASM0,KSPEC2,KUMCP,KMYMS,&
 & PRLON,PSPECI,PSPECF)  

!**** *SPOLTS* - Organisation of the longitude rotation of spectral
!                arrays for each wave number JN.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!         *CALL* SPOLTS(KULOUT,KFIELD,KFLDSUR,
!                       KNSMAX,KASM0,KSPEC2,KUMCP,KMYMS,
!                       PRLON,PSPECI,PSPECF)

!        Explicit arguments :
!        --------------------

!      KULOUT      - Output logical unit.                             (INPUT)
!      KFIELD      - Number of fields to be transformed.              (INPUT)
!      KFLDSUR     - Odd number above KFIELD for dimensioning.        (INPUT)
!      KNSMAX      - Troncature.                                      (INPUT)
!      KASM0       - KASM0(JM): Index of the coefficient              (INPUT)
!                    corresponding to the wave number (m=JM,n=JM).
!      KSPEC2      - Dimension for spectral arrays (PSPECI,PSPECF).   (INPUT)
!      KUMCP       - Number of spectral waves handled by this         (INPUT)
!                    processor for KNSMAX
!      KMYMS       - actual wave numbers handled by this processor    (INPUT)
!      PRLON       - Longitude.                                       (INPUT)
!      PSPECI      - Fields before rotation.                          (INPUT)

!      PSPECF      - Fields after rotation.                           (OUTPUT)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals:
!     ----------
!        Calls SPROTLON.
!        Called by TRAGEO and SPIROU.

!     Reference.
!     ----------
!        ARPEGE/IFS note number 19 (in French).

!     Author.
!     -------
!        K. YESSAD. (METEO-FRANCE/CNRM/GMAP)

!     Modifications.
!     --------------
!        Original : FEBRUARY 1991
!        Modification (english comments, cleaning) by K. YESSAD: APRIL 1993.
!        Modified by L. Isaksen 95-06-06 : Reordering of spectral arrays
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     INPUT:

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSUR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNSMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEC2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUMCP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KASM0(0:KNSMAX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMYMS(1:KUMCP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRLON 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPECI(KFLDSUR,KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPECF(KFLDSUR,KSPEC2) 
!     OUTPUT:
REAL(KIND=JPRB) :: ZC(KNSMAX),ZS(KNSMAX)

INTEGER(KIND=JPIM) :: JM, JMLOC, JN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "sprotlon.intfb.h"

!        1.    DUMMY ARGUMENTS CHECKING.
!        -------------------------------

!     * KFLDSUR:
IF (LHOOK) CALL DR_HOOK('SPOLTS',0,ZHOOK_HANDLE)
IF (KFLDSUR < KFIELD) THEN
  WRITE (KULOUT,'(A)')&
   & ' FATAL  : KFLDSUR MUST BE GREATER OR EQUAL THAN KFIELD.'  
  CALL ABOR1('SPOLTS: ABOR1 CALLED')
ELSEIF (MOD(KFLDSUR,2) == 0) THEN
  WRITE (KULOUT,'(A)')&
   & ' WARNING: KFLDSUR=ODD NUMBER WOULD BE BETTER FOR OPTIMIZATION.'  
ENDIF

!        2.    SINES AND COSINES COMPUTATION.
!        ------------------------------------

DO JM=1,KNSMAX
  ZC(JM)=COS(PRLON*REAL(JM,JPRB))
  ZS(JM)=SIN(PRLON*REAL(JM,JPRB))
ENDDO

!        3.    LONGITUDE ROTATION.
!        -------------------------

DO JMLOC=1,KUMCP
  JN=KMYMS(JMLOC)

  CALL SPROTLON(KULOUT,KFIELD,KFLDSUR,&
   & KNSMAX,KASM0,JN,KSPEC2,ZC,ZS,PSPECI,PSPECF)  

ENDDO

IF (LHOOK) CALL DR_HOOK('SPOLTS',1,ZHOOK_HANDLE)
END SUBROUTINE SPOLTS

