!OCL NOEVAL
SUBROUTINE TRANSDIR_NHCONV(LDMODEL_TO_FILE,PSPD,PSVD,PSNHX)

!* TRANSDIR_NHCONV - Conversion of NH variables (model to file and vice-versa)
!                    Direct spectral transforms.

! Purpose
! -------

! Interface
! ---------
!  * INPUT: 
!    LDMODEL_TO_FILE : switch for model to file / file to model conversion
!    PSPD            : NH pressure departure variable
!    PSVD            : NH vertical divergence variable
!    PSNHX           : NH "X" part of the total divergence.     

! Externals
! ---------

! Method
! ------
!   See documentation

! Reference
! ---------

! Author
! ------
!   27-Jan-2005 K. Yessad (after old part 5 of GNHPDVDCONV)

! Modifications
! -------------
!   18-Jul-2005 R. Brozkova: old VDAUX => "X" term         
!   13-Oct-2005 K. Yessad  : B-level parallelisation.
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMDIM   , ONLY : NFLSUR   ,NFLEVG
USE YOMDYNA  , ONLY : LNHX
USE YOMGEM   , ONLY : NGPTOT
USE YOMSP    , ONLY : SPSPD    ,SPSVD    ,SPNHX

!------------------------------------------------------------------------------

IMPLICIT NONE

LOGICAL,INTENT(IN)         :: LDMODEL_TO_FILE 
REAL(KIND=JPRB),INTENT(IN) :: PSPD(NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PSVD(NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(IN) :: PSNHX(NGPTOT,NFLEVG)

!------------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------------------

#include "reespe.intfb.h"

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSDIR_NHCONV',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

!* 1. TRANSFORM NH FIELDS BACK TO SPECTRAL SPACE
!     ------------------------------------------

! * pressure departure variable
CALL REESPE(NFLSUR,NFLEVG,SPSPD,PSPD)

! * pseudo vertical divergence variable
CALL REESPE(NFLSUR,NFLEVG,SPSVD,PSVD)

! * transform only if transformation
!   from file to model (to initialize NHX variable)
IF ( LNHX.AND.(.NOT.LDMODEL_TO_FILE) ) THEN
  ! "X" part of the total divergence
  CALL REESPE(NFLSUR,NFLEVG,SPNHX,PSNHX)
ENDIF

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSDIR_NHCONV',1,ZHOOK_HANDLE)

END SUBROUTINE TRANSDIR_NHCONV

