SUBROUTINE TRANSDIRH(CDCONF)

!**** *TRANSDIRH * - Direct transforms

!     Purpose.  Perform direct transform (gridpoint to spectral)
!     --------

!**   Interface.  CALL TRANSDIRH(CDCONF)
!     ---------- 

!     Explicit arguments : CDCONF - configuration of work
!     --------------------
!        Called by STEPO/STEPOTL.

!     Externals.
!     ----------
!     TRANSDIR_MDL - transforms for thr model   
!     TRANSDIR_FP  - transforms for Full-Pos   

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!        Modified : 03-08-01 M.Hamrud - GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMOMPDIST , ONLY : LOMPDIST, GPP

IMPLICIT NONE
CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "transdir_fp.intfb.h"
#include "transdir_mdl.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSDIRH',0,ZHOOK_HANDLE)
IF(CDCONF(8:8) == 'T') THEN
  CALL ABOR1('TRANSDIRH:CDCONF=T NOT CATERED FOR YET')
ENDIF

IF(CDCONF(8:8) /= 'P') THEN
  CALL TRANSDIR_MDL(CDCONF(8:8))
ELSE
  CALL TRANSDIR_FP(GPP)
ENDIF

IF (ALLOCATED(GPP).AND..NOT.LOMPDIST) DEALLOCATE(GPP)
IF (LHOOK) CALL DR_HOOK('TRANSDIRH',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE TRANSDIRH

