SUBROUTINE TRANSINV_FP(PGP)

!**** *TRANSINV_FP * - Inverse transforms for model

!     Purpose.  Perform inverse transform (spectral to gridpoint)
!     --------

!**   Interface.  CALL TRANSINV_FP(...)
!     ---------- 

!     Explicit arguments : 
!     --------------------
!        PGP    - gridpoint fields

!        Called by TRANSINVH

!     Externals.
!     ----------
!     INV_TRANS - inverse transform (TRANS library)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!    R. El Khatib : 01-03-07 Fix
!    R. El Khatib : 01-03-28 optimisation
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NSPEC2   ,NPROMA
USE YOMFPDIM , ONLY : NFPISCA  ,NFPIVEC  ,NFPISCAG ,NFPIVECG

IMPLICIT NONE

REAL(KIND=JPRB)                  :: PGP(:,:,:) ! UNDETERMINED INTENT
! === END OF INTERFACE BLOCK ===

REAL(KIND=JPRB) :: ZSPSCA(NFPISCA,NSPEC2)
REAL(KIND=JPRB) :: ZSPVOR(NFPIVEC,NSPEC2), ZSPDIV(NFPIVEC,NSPEC2)

INTEGER(KIND=JPIM) :: IVSETSC(NFPISCAG), IVSETVD(NFPIVECG)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "inv_trans.h"
END INTERFACE

#include "fptsa_inv.intfb.h"
#include "sufpvset_inv.intfb.h"

!     ------------------------------------------------------------------

!    1. Setups

IF (LHOOK) CALL DR_HOOK('TRANSINV_FP',0,ZHOOK_HANDLE)
CALL SUFPVSET_INV(IVSETSC,IVSETVD)

!    2. Transfer from the post-processing spectral arrays

CALL FPTSA_INV(PSCA=ZSPSCA(:,:),PVOR=ZSPVOR(:,:),PDIV=ZSPDIV(:,:))

!    3. Spectral transforms

IF (NFPISCAG > 0 .AND. NFPIVECG > 0) THEN
  CALL INV_TRANS(PSPVOR=ZSPVOR(:,:),PSPDIV=ZSPDIV(:,:),PSPSCALAR=ZSPSCA(:,:),&
   & KPROMA=NPROMA,KVSETUV=IVSETVD(:),KVSETSC=IVSETSC(:),PGP=PGP(:,:,:))  
ELSEIF (NFPIVECG > 0 .AND. NFPISCAG == 0) THEN
  CALL INV_TRANS(PSPVOR=ZSPVOR(:,:),PSPDIV=ZSPDIV(:,:),KPROMA=NPROMA,&
   & KVSETUV=IVSETVD(:),PGP=PGP(:,:,:))  
ELSEIF (NFPISCAG > 0 .AND. NFPIVECG == 0) THEN
  CALL INV_TRANS(PSPSCALAR=ZSPSCA(:,:),KPROMA=NPROMA,KVSETSC=IVSETSC(:),&
   & PGP=PGP(:,:,:))  
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINV_FP',1,ZHOOK_HANDLE)

END SUBROUTINE TRANSINV_FP
