SUBROUTINE TRANSINV_JBTOMODELAD (PSPEC,PREEL)

!**** *TRANSINV_JBTOMODELAD * - Inverse transforms for Jb field

!     Purpose.  Perform inverse transforms for Jb field
!     --------

!**   Interface.  CALL TRANSINV_JBTOMODELAD
!     ---------- 

!     Explicit arguments : 
!     --------------------

!        PSPEC    (input)   - Spectral input array
!        PREEL    (ouput)   - Grid point output array   

!     Externals.
!     ----------
!     INV_TRANS - inverse transform (TRANS library)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Pierre BROUSSEAU

!     Modifications.
!     --------------
!        Original : 2001-08-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM     , ONLY : NPROMA, NSPEC2, NFLSUR
USE YOMMP      , ONLY : NBSETLEV

IMPLICIT NONE

!     Arguments

REAL(KIND=JPRB)   ,INTENT(OUT)    :: PSPEC(NFLSUR,NSPEC2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREEL(:,:,:) 
! === END OF INTERFACE BLOCK ===


REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "inv_transad.h"
END INTERFACE


!     ------------------------------------------------------------------

!     Get the grid definition

IF (LHOOK) CALL DR_HOOK('TRANSINV_JBTOMODELAD',0,ZHOOK_HANDLE)

!     Transform to grid space

CALL INV_TRANSAD (PSPSCALAR=PSPEC(:,:), PGP=PREEL(:,:,:),  &
                  & KVSETSC=NBSETLEV, KPROMA=NPROMA)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINV_JBTOMODELAD',1,ZHOOK_HANDLE)
END SUBROUTINE TRANSINV_JBTOMODELAD
