SUBROUTINE TRANSINV_MDLAD(PSPVOR,PSPDIV,PSPSP,PSPHV,PSPGFL,&
 & PGMV,PGMVS,PGFL,LDERR,LDFSCOMP)  

!**** *TRANSINV_MDLAD * - Inverse transforms for model (adjoint)

!     Purpose.  Perform inverse transform (spectral to gridpoint)
!     --------

!**   Interface.  CALL TRANSINV_MDLAD(...)
!     ---------- 

!     Explicit arguments : 
!     --------------------
!        PSPVOR - spectral vorticity
!        PSPDIV - spectral divergence
!        PSPSP  - spectral surface pressure
!        PSPHV  - spectral thermodynamic variables
!        PSPGFL - spectral GFL variables
!        PGMV   - GMV gridpoint
!        PGMVS  - GMV single level gridpoint
!        PGFL   - GFL gridpoint
!        LDERR  - TRUE if derivatives required
!        LDFSCOMP - true if "scientific" computations in fourier space req.

!        Called by TRANSINVHAD

!     Externals.
!     ----------
!     INV_TRANSAD - inverse transform (TRANS library)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!        Modified : 03-08-01 M.Hamrud - GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY :  NSPEC2 , &
 & LVOR   ,LUVDER ,NPROMA  
USE YOMMP    , ONLY : NBSETLEV,NBSETSP,MYSETV
USE YOM_YGFL , ONLY : YGFL
USE YOMGMV   , ONLY : YT0

IMPLICIT NONE

REAL(KIND=JPRB)                  :: PSPVOR(:,:) ! UNDETERMINED INTENT
REAL(KIND=JPRB)                  :: PSPDIV(:,:) ! UNDETERMINED INTENT
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSP(:) 
REAL(KIND=JPRB)                  :: PSPHV(:,:,:) ! UNDETERMINED INTENT
REAL(KIND=JPRB)                  :: PSPGFL(:,:,:) ! UNDETERMINED INTENT
REAL(KIND=JPRB)                  :: PGMV(:,:,:,:) ! UNDETERMINED INTENT
REAL(KIND=JPRB)                  :: PGMVS(:,:,:) ! UNDETERMINED INTENT
REAL(KIND=JPRB)                  :: PGFL(:,:,:,:) ! UNDETERMINED INTENT
LOGICAL           ,INTENT(IN)    :: LDERR 
LOGICAL           ,INTENT(IN)    :: LDFSCOMP 
! === END OF INTERFACE BLOCK ===

REAL(KIND=JPRB) :: ZSP(1,NSPEC2)
INTEGER(KIND=JPIM) :: IST,IOFF,IDIM3,ISTGMV,IENGMV,IDIMGMVS,ISTUV,IENUV
INTEGER(KIND=JPIM) :: IVSETSC(1)
LOGICAL :: LLUVDER,LLVOR,LLSCDERS,LLDIVGP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

EXTERNAL FSPGLH

INTERFACE
#include "inv_transad.h"
END INTERFACE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINV_MDLAD',0,ZHOOK_HANDLE)
LLUVDER  = LUVDER .AND. LDERR
LLVOR    = LVOR   .AND. LDERR
LLSCDERS = LDERR
LLDIVGP  = LDERR

IVSETSC(1) = NBSETSP

CALL GSTATS(1827,0)
IF(NBSETSP == MYSETV) THEN
  IST = 1
  ZSP(IST,:) = PSPSP(:)
ELSE
  IST = 0
ENDIF
CALL GSTATS(1827,1)

ISTGMV=YT0%NDIMUV+1
IENGMV=YT0%NDIM
IDIMGMVS=YT0%NDIMS
ISTUV=1
IENUV=YT0%NDIMUV
IF(.NOT.LLSCDERS) THEN
  IENGMV=ISTGMV-1+(IENGMV-ISTGMV+1)/3
  IDIMGMVS=IDIMGMVS/3
  ISTUV=YT0%MU
  IENUV=YT0%MV
ENDIF

IF(YGFL%NUMSPFLDS>0) THEN
  IOFF=YGFL%NUMFLDS-YGFL%NUMSPFLDS
  IDIM3=YGFL%NUMSPFLDS
  IF(LLSCDERS) IDIM3=IDIM3*3
  IF (LDFSCOMP) THEN
    CALL INV_TRANSAD(&
     & PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:IST,:),PSPSC3A=PSPHV,&
     & PSPSC3B=PSPGFL, &
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,KVSETSC3B=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:),&
     & PGP3B=PGFL(:,:,IOFF+1:IOFF+IDIM3,:),&
     & FSPGL_PROC=FSPGLH)  
  ELSE
    CALL INV_TRANSAD(&
     & PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:IST,:),PSPSC3A=PSPHV,&
     & PSPSC3B=PSPGFL, &
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,KVSETSC3B=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:),&
     & PGP3B=PGFL(:,:,IOFF+1:IOFF+IDIM3,:))  
  ENDIF
ELSE
  IF (LDFSCOMP) THEN
    CALL INV_TRANSAD(PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:1,:),PSPSC3A=PSPHV,&
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:), &
     & FSPGL_PROC=FSPGLH)  
  ELSE
    CALL INV_TRANSAD(PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:1,:),PSPSC3A=PSPHV,&
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:))  
  ENDIF
ENDIF

CALL GSTATS(1828,0)
IF(NBSETSP == MYSETV) THEN
  PSPSP(:) = ZSP(IST,:)
ENDIF
CALL GSTATS(1828,1)
IF (LHOOK) CALL DR_HOOK('TRANSINV_MDLAD',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE TRANSINV_MDLAD
