SUBROUTINE TRANSINVHAD(CDCONF)

!**** *TRANSINVHAD * - Inverse transforms - adjoint

!     Purpose.  Perform inverse transform (spectral to gridpoint)
!     --------

!**   Interface.  CALL TRANSINVHAD(CDCONF)
!     ---------- 

!     Explicit arguments : CDCONF - configuration of work
!     --------------------
!        Called by STEPOAD

!     Externals.
!     ----------
!     TRANSINV_MDLAD - transforms for thr model   

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!        Y.Tremolet: 01-02-28 cleanup
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPHV     ,SPSP   ,SPGFL
USE YOMDIM   , ONLY : LADER
USE YOMGMV   , ONLY : GMV,GMVS
USE YOMGFL   , ONLY : GFL

IMPLICIT NONE

!  CDCONF(2:2) Definition for INVERSE LEGENDRE TRANSFORM 

!                      Spectral data 
!                  SPA3    SPA5    SPA7
!                 ------------------------
!   Derivatives  |Yes No |Yes No |Yes No | 
! ----------------------------------------
!                |       |       |       |
!    F       T0  | A  G  | B  H  | C  I  |
!    o D         |       |       |       |
!    u a         |-----------------------|
!    r t         |       |       |       |
!    i a     T5  | D  J  | E  K  | F  L  |
!    e           |       |       |       |
!    r           |-----------------------|

!     ------------------------------------------------------------------

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 
LOGICAL :: LLDERR,LLADGJ,LLFSCOMP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "transinv_mdlad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINVHAD',0,ZHOOK_HANDLE)
LLDERR = LLE(CDCONF(2:2),'F') .AND. LADER 
LLADGJ = CDCONF(2:2) == 'A'.OR.CDCONF(2:2) == 'D'.OR.CDCONF(2:2) == 'G'.OR.&
 & CDCONF(2:2) == 'J'.OR.CDCONF(2:2) == 'M'  
LLFSCOMP = .FALSE.

IF(LLADGJ) THEN
  IF (CDCONF(3:3) == 'A'.OR.CDCONF(3:3) == 'B') THEN
    CALL TRANSINV_MDLAD(SPVOR,SPDIV,SPSP,SPHV,SPGFL,&
     & GMV,GMVS,GFL,LLDERR,LLFSCOMP)  
  ELSE
    CALL ABOR1(' TRANSINVHAD - CONFIGURATION NOT SUPPORTED 1')
  ENDIF
ELSE
  CALL ABOR1(' TRANSINVHAD - CONFIGURATION NOT SUPPORTED 2')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINVHAD',1,ZHOOK_HANDLE)
END SUBROUTINE TRANSINVHAD
