SUBROUTINE ADD5TO3

!**** *ADD5TO3* - ROUTINE TO ADD SP3 AND SP5 SPECTRAL ARRAYS

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        W. HECKLEY  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-02-20
!        Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!        Modified 95-12-12 by M.Hamrud  : Array syntax for vect. 
!        Modified 97-12-01 by E.Holm    : Ozone included
!        C. Fischer 00-06-20: add mean wind for LAM
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        P.Brousseau   09-May-2007 spectral Jb fields but model state in grid point space
USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFD2D,  NFLEVG, NPROMA, NGPBLKS,NS3D
USE YOMMP    , ONLY : NPSP
USE YOMSP    , ONLY : SPA2     ,SP5A2    ,SPVOR    ,SPDIV    ,&
 & SP5VOR   ,SP5DIV   ,SPT      ,SPGFL   ,&
 & SP5T     ,SP5GFL   ,SPA1     ,SP5A1  
USE YOMCT0   , ONLY : LELAM
USE TRAJECTORY_MOD, ONLY: GET_TRAJ_GRID
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL ,YQ
USE YOMGMV   , ONLY : YT5
USE YOMCVA   , ONLY : NVA3D
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
INTEGER(KIND=JPIM) :: ISTEP0
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL5(:,:,:,:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE



!     ------------------------------------------------------------------

!*    1.    ADD SP5Ax to SPAx.
!           ------------------

IF (LHOOK) CALL DR_HOOK('ADD5TO3',0,ZHOOK_HANDLE)
SPVOR(:,:) = SPVOR(:,:) + SP5VOR(:,:)
SPDIV(:,:) = SPDIV(:,:) + SP5DIV(:,:)
SPT(:,:)   = SPT(:,:)   + SP5T(:,:)
SPGFL(:,:,:)   = SPGFL(:,:,:)   + SP5GFL(:,:,:)


! If humidity Jb field is spectral but model state is in grid point space
! Get initial time grid point trajectory and add to delta values
IF ((NS3D<NVA3D) .AND. (YQ%LGP)) THEN
  ALLOCATE (ZGMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
  ALLOCATE (ZGMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
  ALLOCATE (ZGFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
  ZGMV5=0._JPRB
  ZGMV5S=0._JPRB
  ZGFL5=0._JPRB
  ISTEP0=0
  CALL GET_TRAJ_GRID(ZGMV5,ZGMV5S,ZGFL5,ISTEP0)
  GFL(:,:,YQ%MP,:)=GFL(:,:,YQ%MP,:)+ZGFL5(:,:,YQ%MP,:)
  DEALLOCATE(ZGMV5)
  DEALLOCATE(ZGMV5S)
  DEALLOCATE(ZGFL5)
ENDIF

IF(NPSP == 1) SPA2(:,1:NFD2D) = SPA2(:,1:NFD2D) + SP5A2(:,1:NFD2D)

IF (LELAM) SPA1(:,:) = SPA1(:,:) + SP5A1(:,:)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ADD5TO3',1,ZHOOK_HANDLE)
END SUBROUTINE ADD5TO3
