SUBROUTINE copy_spa2spec(ks3d,ks2d,ks1d,PSP3D,PSP2D,PSP1D,YD_SPEC)

!     Purpose.
!     --------
!       Copy SPA3/SPA2-like arrays into SPECTRAL_FIELD type.
!       Fields are truncated or padded with zeroes if resolution
!       is different.

!     Author.
!     -------
!       Yannick Tremolet *ECMWF*

!     Modifications.
!     --------------
!       Original  : 14-05-02
!     C.Fischer   03-03-31 Aladin spectral treatment added.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
! ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0, ONLY: LELAM
USE YOMDIM, ONLY: NSMAX, NUMP, NFLEVL, NSPEC2, NMSMAX
USE YOMLAP, ONLY: NASM0, MYMS
USE YEMLAP, ONLY: NESM0, NCPL4M
USE YOMMP , ONLY: NBSETSP, MYSETV
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KS3D 
INTEGER(KIND=JPIM),INTENT(IN)    :: KS2D 
INTEGER(KIND=JPIM),INTENT(IN)    :: KS1D 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP3D(NFLEVL,NSPEC2,KS3D) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP2D(NSPEC2,KS2D) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP1D(NFLEVL,KS1D) 
TYPE(SPECTRAL_FIELD),INTENT(INOUT) :: YD_SPEC 
INTEGER(KIND=JPIM) :: jf,jm,im,I_LEN,js1,js2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

IF (LHOOK) CALL DR_HOOK('COPY_SPA2SPEC',0,ZHOOK_HANDLE)
IF (ks3d /= YD_SPEC%ns3d) CALL ABOR1('Wrong number of 3D fields')
IF (ks2d /= YD_SPEC%ns2g) CALL ABOR1('Wrong number of 2D fields')
IF ((ks1d /= YD_SPEC%ns1d).AND.LELAM) CALL ABOR1('Wrong number of 1D fields')

CALL GSTATS(1956,0)
IF ((YD_SPEC%nsmax<NSMAX).AND.(.NOT.LELAM)) THEN      ! Truncate in IFS
  DO jf=1,YD_SPEC%ns3d
    DO jm=1,YD_SPEC%NUMP
      im=MYMS(jm)
      I_LEN=2*(YD_SPEC%NSMAX-im+1)-1
      js1=YD_SPEC%NASM0(im)
      js2=NASM0(im)
      YD_SPEC%sp3d(:,js1:js1+I_LEN,jf) = PSP3D(:,js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
  DO jf=1,YD_SPEC%ns2d
    DO jm=1,YD_SPEC%NUMP
      im=MYMS(jm)
      I_LEN=2*(YD_SPEC%NSMAX-im+1)-1
      js1=YD_SPEC%NASM0(im)
      js2=NASM0(im)
      YD_SPEC%sp2d(js1:js1+I_LEN,jf) = PSP2D(js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
ELSEIF ((YD_SPEC%nsmax>NSMAX).AND.(.NOT.LELAM)) THEN  ! Pad with zeros in IFS
  YD_SPEC%sp3d(:,:,:) = 0.0_JPRB
  DO jf=1,YD_SPEC%ns3d
    DO jm=1,NUMP
      im=MYMS(jm)
      I_LEN=2*(NSMAX-im+1)-1
      js1=YD_SPEC%NASM0(im)
      js2=NASM0(im)
      YD_SPEC%sp3d(:,js1:js1+I_LEN,jf) = PSP3D(:,js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
  YD_SPEC%sp2d(:,:) = 0.0_JPRB
  DO jf=1,YD_SPEC%ns2d
    DO jm=1,NUMP
      im=MYMS(jm)
      I_LEN=2*(NSMAX-im+1)-1
      js1=YD_SPEC%NASM0(im)
      js2=NASM0(im)
      YD_SPEC%sp2d(js1:js1+I_LEN,jf) = PSP2D(js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
ELSEIF ((((YD_SPEC%nsmax<NSMAX).AND.(YD_SPEC%nmsmax<=NMSMAX)).OR. &
   & ((YD_SPEC%nsmax<=NSMAX).AND.(YD_SPEC%nmsmax<NMSMAX))) &
   & .AND.(LELAM)) THEN      ! Truncate in Aladin  
  DO jf=1,YD_SPEC%ns3d
    DO jm=1,YD_SPEC%NUMP
      im=MYMS(jm)
      I_LEN=YD_SPEC%ncpl4m(im)-1
      js1=YD_SPEC%NESM0(im)
      js2=NESM0(im)
      YD_SPEC%sp3d(:,js1:js1+I_LEN,jf) = PSP3D(:,js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
  DO jf=1,YD_SPEC%ns2d
    DO jm=1,YD_SPEC%NUMP
      im=MYMS(jm)
      I_LEN=YD_SPEC%ncpl4m(im)-1
      js1=YD_SPEC%NESM0(im)
      js2=NESM0(im)
      YD_SPEC%sp2d(js1:js1+I_LEN,jf) = PSP2D(js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
ELSEIF ((((YD_SPEC%nsmax>NSMAX).AND.(YD_SPEC%nmsmax>=NMSMAX)).OR. &
   & ((YD_SPEC%nsmax>=NSMAX).AND.(YD_SPEC%nmsmax>NMSMAX))) &
   & .AND.(LELAM)) THEN  ! Pad with zeros in Aladin  
  YD_SPEC%sp3d(:,:,:) = 0.0_JPRB
  DO jf=1,YD_SPEC%ns3d
    DO jm=1,NUMP
      im=MYMS(jm)
      I_LEN=NCPL4M(im)-1
      js1=YD_SPEC%NESM0(im)
      js2=NESM0(im)
      YD_SPEC%sp3d(:,js1:js1+I_LEN,jf) = PSP3D(:,js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
  YD_SPEC%sp2d(:,:) = 0.0_JPRB
  DO jf=1,YD_SPEC%ns2d
    DO jm=1,NUMP
      im=MYMS(jm)
      I_LEN=NCPL4M(im)-1
      js1=YD_SPEC%NESM0(im)
      js2=NESM0(im)
      YD_SPEC%sp2d(js1:js1+I_LEN,jf) = PSP2D(js2:js2+I_LEN,jf)
    ENDDO
  ENDDO
ELSEIF (((YD_SPEC%nsmax/=NSMAX).OR.(YD_SPEC%nmsmax/=NMSMAX)).AND. &
   & (LELAM)) THEN  ! Unfeasible cases left in Aladin  
  CALL ABOR1('COPY_SPA_SPEC error: mismatch in Aladin spectral truncations')
ELSE                                   ! Copy
  YD_SPEC%sp3d(:,:,:)=PSP3D(:,:,:)
  IF(NBSETSP == MYSETV) THEN
    YD_SPEC%sp2d(:,:)  =PSP2D(:,:)
  ENDIF
ENDIF

IF (LELAM) THEN
  DO jf=1,YD_SPEC%ns1d
    YD_SPEC%sp1d(:,jf) = PSP1D(:,jf)
  ENDDO
ENDIF
CALL GSTATS(1956,1)

IF (LHOOK) CALL DR_HOOK('COPY_SPA2SPEC',1,ZHOOK_HANDLE)
END SUBROUTINE copy_spa2spec
