SUBROUTINE DEALCTV

!**** *DEALCTV* - ROUTINE TO DEALLOCATE SPACE FOR CONTROL VARIABLE

!     Purpose.
!     --------
!        Deallocate space for arrays depending on the size of the
!        control variable.

!**   Interface.
!     ----------
!        *CALL* *DEALCTV

!        Explicit arguments : None
!        --------------------

!        Implicit arguments :
!        --------------------
!        COMMON YOMCVA
!        COMMON POMCVA

!     Method.
!     -------
!        See documentation

!     Externals.    DEALLOCATE
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original   95-09-28
!     E.Andersson   96-08-01 : Precalculated SQRT(SCALP)
!     R. El Khatib  01-08-07 Pruning options
!     Y.Tremolet    02-08-06  Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     M. Jidane     09-04-2006 : CY31 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCVA, ONLY: YVAZX     ,YVAZG     , YVAZ0, &
 & YSCALP, YRSCALP, YSCALPSQRT, YRSCALPSQRT  
USE YOMVAR, ONLY: LMODERR
USE YOMMODEL_ERROR, ONLY: SPMODERR, GPMODERR, &
                     & NDIM_MODERR, NGRBMERRSP, NGRBMERRGP, NGRBMERRCTL, &
                     & SPMODERR_HYB, GPMODERR_HYB, NTYPE_MODERR
USE YOMJQ, ONLY: ERRMODMN, ERRHCOR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD
USE CONTROL_VECTORS

IMPLICIT NONE
INTEGER(KIND=JPIM) :: jj
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!------------------------------------------------------

!*        1.    DEALLOCATE
!               ----------

IF (LHOOK) CALL DR_HOOK('DEALCTV',0,ZHOOK_HANDLE)
CALL DEALLOCATE_CTLVEC(YVAZX)
CALL DEALLOCATE_CTLVEC(YVAZG)
CALL DEALLOCATE_CTLVEC(YVAZ0)
CALL DEALLOCATE_CTLVEC(YSCALP)
CALL DEALLOCATE_CTLVEC(YRSCALP)
CALL DEALLOCATE_CTLVEC(YSCALPSQRT)
CALL DEALLOCATE_CTLVEC(YRSCALPSQRT)
IF (LMODERR) THEN
  DO jj=1,NDIM_MODERR
    CALL DEALLOCATE_SPEC(SPMODERR(jj))
    CALL DEALLOCATE_GRID(GPMODERR(jj))
    IF (NTYPE_MODERR == 5) THEN
      CALL DEALLOCATE_SPEC(SPMODERR_HYB(jj))
      CALL DEALLOCATE_GRID(GPMODERR_HYB(jj))
    ENDIF
  ENDDO
  IF (ALLOCATED(SPMODERR)) DEALLOCATE(SPMODERR)
  IF (ALLOCATED(GPMODERR)) DEALLOCATE(GPMODERR)
  IF (ALLOCATED(SPMODERR_HYB)) DEALLOCATE(SPMODERR_HYB)
  IF (ALLOCATED(GPMODERR_HYB)) DEALLOCATE(GPMODERR_HYB)
ENDIF
IF (ALLOCATED(NGRBMERRSP)) DEALLOCATE(NGRBMERRSP)
IF (ALLOCATED(NGRBMERRGP)) DEALLOCATE(NGRBMERRGP)
IF (ALLOCATED(NGRBMERRCTL)) DEALLOCATE(NGRBMERRCTL)
IF (ASSOCIATED(ERRMODMN%V2D)) DEALLOCATE(ERRMODMN%V2D)
IF (ASSOCIATED(ERRMODMN%V3D)) DEALLOCATE(ERRMODMN%V3D)
IF (ASSOCIATED(ERRMODMN%V2DINV)) DEALLOCATE(ERRMODMN%V2DINV)
IF (ASSOCIATED(ERRMODMN%V3DINV)) DEALLOCATE(ERRMODMN%V3DINV)
IF (ASSOCIATED(ERRMODMN%VALPHA)) DEALLOCATE(ERRMODMN%VALPHA)
IF (ASSOCIATED(ERRHCOR%SP2D)) DEALLOCATE(ERRHCOR%SP2D)
IF (ASSOCIATED(ERRHCOR%SP3D)) DEALLOCATE(ERRHCOR%SP3D)

IF (LHOOK) CALL DR_HOOK('DEALCTV',1,ZHOOK_HANDLE)
END SUBROUTINE DEALCTV
