SUBROUTINE DEALDDH

!**** *DEALDDH * - Routine to deallocate space used by DDH

!     Purpose.
!     --------
!           Deallocate space used by DDH.

!**   Interface.
!     ----------
!        *CALL* *DEALDDH*

!     Explicit arguments :  None
!     --------------------
!        Called by FREEMEM

!     Implicit arguments :
!     --------------------
!        Pointers of comdecks

!     Method.
!     -------
!        Extraction from SUALMDH/SUALTDH
!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 95-09-28
!        Modified JUN 1997 by K. YESSAD: add missing deallocations.
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMDDH  , ONLY :  NDDHLA   ,NDDHPU   ,NDDHI    ,NLRDDH   ,NURDDH   ,&
 & NLXDDH   ,NUXDDH   ,HDSFLA   ,HDSFDU   ,HDSF  
USE YOMPADDH , ONLY : NGLALIST ,NGPUMASK
USE YOMTDDH  , ONLY : HDCVB0   ,HDCVB1   ,HDCS0    ,HDCS1

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!------------------------------------------------------

!*        1.    DEALLOCATE
!               ----------

IF (LHOOK) CALL DR_HOOK('DEALDDH',0,ZHOOK_HANDLE)
IF (ALLOCATED(NDDHLA)) DEALLOCATE(NDDHLA)
IF (ALLOCATED(NLRDDH)) DEALLOCATE(NLRDDH)
IF (ALLOCATED(NLXDDH)) DEALLOCATE(NLXDDH)
IF (ALLOCATED(HDSFLA)) DEALLOCATE(HDSFLA)
IF (ALLOCATED(NDDHPU)) DEALLOCATE(NDDHPU)
IF (ALLOCATED(NURDDH)) DEALLOCATE(NURDDH)
IF (ALLOCATED(NUXDDH)) DEALLOCATE(NUXDDH)
IF (ALLOCATED(HDSFDU)) DEALLOCATE(HDSFDU)
IF (ALLOCATED(NDDHI )) DEALLOCATE(NDDHI)
IF (ALLOCATED(HDSF  )) DEALLOCATE(HDSF)

IF (ALLOCATED(HDCVB0)) DEALLOCATE(HDCVB0)
IF (ALLOCATED(HDCVB1)) DEALLOCATE(HDCVB1)
IF (ALLOCATED(HDCS0 )) DEALLOCATE(HDCS0)
IF (ALLOCATED(HDCS1 )) DEALLOCATE(HDCS1)

IF (ALLOCATED(NGLALIST)) DEALLOCATE(NGLALIST)
IF (ALLOCATED(NGPUMASK)) DEALLOCATE(NGPUMASK)

IF (LHOOK) CALL DR_HOOK('DEALDDH',1,ZHOOK_HANDLE)
END SUBROUTINE DEALDDH
