SUBROUTINE DEALFPOS

!**** *DEALFPOS* - Deallocate the working arrays for Full-POS computation.

!     Purpose.
!     -------
!           Deallocation of the working arrays used exclusively by Full-POS

!**   Interface.
!     ---------
!     *CALL* * DEALFPOS*

!     Explicit arguments :  None
!     --------------------
!        Called by FREEMEM

!     Implicit arguments :
!     --------------------
!        Pointers of comdecks

!     Method.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        DEALLOCATE

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified JUN 1997 by K. YESSAD: add missing deallocations.
!        R. El Khatib : 97-06-02  3 NH fields
!        R. El Khatib : 98-04-06  Tunable spectral packing + simplifications
!        Modified JAN 1998 by K. YESSAD: report Lars ISAKSEN modifications
!         relative to DM environnement arrays NFPSTLAT ,NFPSTLON ,NFPRSTLAT,
!         NFPRSTLON which are replaced by others, now deallocated in DEALLO.
!        R. El Khatib : 98-07-21 NFPRGPNUM, NFPRGPIND
!        R. El Khatib : 98-08-27 cleanup adiabatic pp.
!        G. Radnoti   : 98-09-10 EZONE arrays
!        R. El Khatib : 98-10-07 Replace NFPROF by NFPINT
!        R. El Khatib : 98-12-17 Replace NFPLOC for aladin
!        R. El Khatib : 99-03-19 NFPNUMD
!        R. El Khatib : 99-10-06  remove useless variable NOFPLEV
!        D.Dent       : 00-01-10  add missing deallocates
!        R. El Khatib : 00-07-27 GFPAUXBUF
!        R. El Khatib : 00-08-18 derived type
!        D. Dent      : 00-04-12 Support for computation of Montgomery geopot.
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad    : 28-02-2007 DM-features optimisations in FULL-POS
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMFPG   , ONLY : RFPLA    ,RFPLA_DEP,RFPLO    ,RFPLO_DEP,RFPMU    , &
 & RFPGM    ,RFPNORX  ,RFPNORY  ,NFPMEN   ,NFPNUMD_DEP
USE YOM4FPOS , ONLY : NCPHYFP  ,NAPHYFP  ,NIPHYFP  ,NOFPP    ,NOFPH    , &
 & NOFPS    ,NOFPPV   ,NOFPTH   ,NCPFP    ,NCHFP    ,NCTFP    , &
 & NCVFP    ,NCSFP    ,NAFPP    ,NAFPH    ,NAFPT    ,NAFPV    , &
 & NAFPS    ,NIPFP    ,NIHFP    ,NITFP    ,NIVFP    ,NISFP    , &
 & NPFP     ,NHFP     ,NTFP     ,NVFP     ,NSFP     ,NOFP2A   , &
 & NC2FP    ,NI2FP
USE YOMRFPB  , ONLY : RFPBUF   ,NSTARFPB ,REFPBUF
USE YOMVPOS  , ONLY : XFPLEV   ,NFPTRDYN
USE YOMFPF   , ONLY : RFPFIL   ,RFPMAT   ,LFPFIL   ,RFPEXPC
USE YOMFPSC2 , ONLY : NFPSTART ,NFPEND   ,NFPSORT  ,NFPBLOFF , &
 & NFPSTART_DEP,NFPEND_DEP,NFPSORT_DEP,NFPBLOFF_DEP
USE YOMGRB   , ONLY : MSEC2GG
USE YOMMP    , ONLY : NUMPROCFP,NUMPROCFP_DEP      ,NFPSTA   ,NFPONL   , &
 & NFPOFF   ,NFPEXT   ,NFPCORE
USE YOMFPGIND, ONLY : NFPRGPNUM,NFPRGPIND,NFPRGPNUM_DEP,NFPRGPIND_DEP
USE YOMFPEZO , ONLY : NFPSTAE
USE YEMFPG   , ONLY : NFPLOC
USE PTRFPB2  , ONLY : MFPVT0
USE YOMRFPDS , ONLY : MFPVSO
USE YOMWFPB  , ONLY : MLAT     ,MLON     ,MLOS     ,MLONN    ,MLOSS    , &
 & WSTD04   ,WSTD12  ,MWIC     ,MWIC_DEP

!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DEALFPOS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*        1.    DEALLOCATE
!               ----------

IF (ALLOCATED(NFPNUMD_DEP)) DEALLOCATE(NFPNUMD_DEP)
IF (ALLOCATED(RFPLA)) DEALLOCATE(RFPLA)
IF (ALLOCATED(RFPLA_DEP)) DEALLOCATE(RFPLA_DEP)
IF (ALLOCATED(RFPLO)) DEALLOCATE(RFPLO)
IF (ALLOCATED(RFPLO_DEP)) DEALLOCATE(RFPLO_DEP)
IF (ALLOCATED(RFPMU)) DEALLOCATE(RFPMU)
IF (ALLOCATED(NFPMEN)) DEALLOCATE(NFPMEN)
IF (ALLOCATED(RFPGM  )) DEALLOCATE(RFPGM  )
IF (ALLOCATED(RFPNORX)) DEALLOCATE(RFPNORX)
IF (ALLOCATED(RFPNORY)) DEALLOCATE(RFPNORY)
IF (ALLOCATED(NFPSTAE)) DEALLOCATE(NFPSTAE)
IF (ALLOCATED(NOFPP)) DEALLOCATE(NOFPP)
IF (ALLOCATED(NOFPH)) DEALLOCATE(NOFPH)
IF (ALLOCATED(NOFPS)) DEALLOCATE(NOFPS)
IF (ALLOCATED(NOFPPV)) DEALLOCATE(NOFPPV)
IF (ALLOCATED(NOFPTH)) DEALLOCATE(NOFPTH)
IF (ALLOCATED(NCPFP)) DEALLOCATE(NCPFP)
IF (ALLOCATED(NCHFP)) DEALLOCATE(NCHFP)
IF (ALLOCATED(NCTFP)) DEALLOCATE(NCTFP)
IF (ALLOCATED(NCVFP)) DEALLOCATE(NCVFP)
IF (ALLOCATED(NCSFP)) DEALLOCATE(NCSFP)
IF (ALLOCATED(NPFP )) DEALLOCATE(NPFP )
IF (ALLOCATED(NAFPP)) DEALLOCATE(NAFPP)
IF (ALLOCATED(NHFP )) DEALLOCATE(NHFP )
IF (ALLOCATED(NAFPH)) DEALLOCATE(NAFPH)
IF (ALLOCATED(NTFP )) DEALLOCATE(NTFP )
IF (ALLOCATED(NAFPT)) DEALLOCATE(NAFPT)
IF (ALLOCATED(NVFP )) DEALLOCATE(NVFP )
IF (ALLOCATED(NAFPV)) DEALLOCATE(NAFPV)
IF (ALLOCATED(NSFP )) DEALLOCATE(NSFP )
IF (ALLOCATED(NAFPS)) DEALLOCATE(NAFPS)
IF (ALLOCATED(NIPFP)) DEALLOCATE(NIPFP)
IF (ALLOCATED(NIHFP)) DEALLOCATE(NIHFP)
IF (ALLOCATED(NITFP)) DEALLOCATE(NITFP)
IF (ALLOCATED(NIVFP)) DEALLOCATE(NIVFP)
IF (ALLOCATED(NISFP)) DEALLOCATE(NISFP)
IF (ALLOCATED(NC2FP)) DEALLOCATE(NC2FP)
IF (ALLOCATED(NOFP2A)) DEALLOCATE(NOFP2A)
IF (ALLOCATED(NI2FP)) DEALLOCATE(NI2FP)
IF (ALLOCATED(NCPHYFP)) DEALLOCATE(NCPHYFP)
IF (ALLOCATED(NAPHYFP)) DEALLOCATE(NAPHYFP)
IF (ALLOCATED(NIPHYFP)) DEALLOCATE(NIPHYFP)
IF (ALLOCATED(XFPLEV  )) DEALLOCATE(XFPLEV  )
IF (ALLOCATED(NFPTRDYN)) DEALLOCATE(NFPTRDYN)
IF (ALLOCATED(NSTARFPB)) DEALLOCATE(NSTARFPB)
IF (ALLOCATED(RFPFIL)) DEALLOCATE(RFPFIL)
IF (ALLOCATED(LFPFIL)) DEALLOCATE(LFPFIL)

IF (ALLOCATED(RFPBUF)) DEALLOCATE(RFPBUF)
IF (ALLOCATED(REFPBUF)) DEALLOCATE(REFPBUF)

IF (ALLOCATED(NFPSTART)) DEALLOCATE(NFPSTART)
IF (ALLOCATED(NFPSTART_DEP)) DEALLOCATE(NFPSTART_DEP)
IF (ALLOCATED(NFPEND)) DEALLOCATE(NFPEND)
IF (ALLOCATED(NFPEND_DEP)) DEALLOCATE(NFPEND_DEP)
IF (ALLOCATED(NFPSORT)) DEALLOCATE(NFPSORT)
IF (ALLOCATED(NFPSORT_DEP)) DEALLOCATE(NFPSORT_DEP)
IF (ALLOCATED(NFPBLOFF)) DEALLOCATE(NFPBLOFF)
IF (ALLOCATED(NFPBLOFF_DEP)) DEALLOCATE(NFPBLOFF_DEP)

IF (ALLOCATED(RFPMAT)) DEALLOCATE(RFPMAT)
IF (ALLOCATED(RFPEXPC)) DEALLOCATE(RFPEXPC)

IF (ALLOCATED(MSEC2GG)) DEALLOCATE(MSEC2GG)
IF (ALLOCATED(NUMPROCFP)) DEALLOCATE(NUMPROCFP)
IF (ALLOCATED(NUMPROCFP_DEP)) DEALLOCATE(NUMPROCFP_DEP)
IF (ALLOCATED(NFPRGPNUM)) DEALLOCATE(NFPRGPNUM)
IF (ALLOCATED(NFPRGPNUM_DEP)) DEALLOCATE(NFPRGPNUM_DEP)
IF (ALLOCATED(NFPRGPIND)) DEALLOCATE(NFPRGPIND)
IF (ALLOCATED(NFPRGPIND_DEP)) DEALLOCATE(NFPRGPIND_DEP)

IF (ALLOCATED(NFPLOC)) DEALLOCATE(NFPLOC)
IF (ALLOCATED(NFPSTA)) DEALLOCATE(NFPSTA)
IF (ALLOCATED(NFPONL)) DEALLOCATE(NFPONL)
IF (ALLOCATED(NFPOFF)) DEALLOCATE(NFPOFF)
IF (ALLOCATED(NFPEXT)) DEALLOCATE(NFPEXT)
IF (ALLOCATED(NFPCORE)) DEALLOCATE(NFPCORE)

IF (ALLOCATED(MFPVT0)) DEALLOCATE(MFPVT0)
IF (ALLOCATED(MFPVSO)) DEALLOCATE(MFPVSO)

IF (ALLOCATED(MLAT)) DEALLOCATE(MLAT)
IF (ALLOCATED(MLON)) DEALLOCATE(MLON)
IF (ALLOCATED(MLOS)) DEALLOCATE(MLOS)
IF (ALLOCATED(MLONN)) DEALLOCATE(MLONN)
IF (ALLOCATED(MLOSS)) DEALLOCATE(MLOSS)
IF (ALLOCATED(MWIC)) DEALLOCATE(MWIC)
IF (ALLOCATED(MWIC_DEP)) DEALLOCATE(MWIC_DEP)
IF (ALLOCATED(WSTD04)) DEALLOCATE(WSTD04)
IF (ALLOCATED(WSTD12)) DEALLOCATE(WSTD12)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DEALFPOS',1,ZHOOK_HANDLE)
END SUBROUTINE DEALFPOS
