SUBROUTINE DEALSCR(LDLAST)

!**** *DEALSCR* - Deallocate arrays used in screening of observations

!     Purpose.
!     -------
!           Deallocate arrays used in screening of observations

!**   Interface.
!**   ---------
!**         DEALSCR is called from routine SCREEN

!     Author.
!     -------
!        H. JARVINEN *ECMWF*

!     Modifications.
!     --------------
!        Original : 96-05-15
!     Modified JUN 1997 by K. YESSAD: add missing deallocations.
!     H. Jarvinen 971021 : LDLAST to indicate last timeslot in 4d-screening
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning

!------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLEVSE , ONLY : NVCOUN   ,NVPOIN   ,RSTDPL   ,GVDIST
USE YOMSCREE , ONLY : NRDBRR   ,&
 & NRDBRD   ,NREVTOT  ,NDEVTOT  ,NDOBVA   ,NDEPEX   ,&
 & NDEPNE   , &
 & CAMBPL   , &
 & CSUPERSONIC  

IMPLICIT NONE
LOGICAL           ,INTENT(IN)    :: LDLAST 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------

!*        1.    DEALLOCATE
!               ----------

!*           1.1    DECLARED IN YOMLEVSE

IF (LHOOK) CALL DR_HOOK('DEALSCR',0,ZHOOK_HANDLE)
IF(LDLAST) THEN
  IF(ALLOCATED (NVCOUN) )  DEALLOCATE (NVCOUN)
  IF(ALLOCATED (NVPOIN) )  DEALLOCATE (NVPOIN)
  IF(ALLOCATED (RSTDPL) )  DEALLOCATE (RSTDPL)
  IF(ALLOCATED (GVDIST) )  DEALLOCATE (GVDIST)
ENDIF

!*           1.2    DECLARED IN YOMSCREE

IF(LDLAST) THEN
  IF(ALLOCATED (NRDBRR) )  DEALLOCATE (NRDBRR)
  IF(ALLOCATED (NRDBRD) )  DEALLOCATE (NRDBRD)
  IF(ALLOCATED (NREVTOT) ) DEALLOCATE (NREVTOT)
  IF(ALLOCATED (NDEVTOT) ) DEALLOCATE (NDEVTOT)
  IF(ALLOCATED (NDOBVA) )  DEALLOCATE (NDOBVA)
  IF(ALLOCATED (NDEPEX) )  DEALLOCATE (NDEPEX)
  IF(ALLOCATED (NDEPNE) )  DEALLOCATE (NDEPNE)
  IF(ALLOCATED (CAMBPL) )  DEALLOCATE (CAMBPL)
  IF(ALLOCATED (CSUPERSONIC) )  DEALLOCATE (CSUPERSONIC)
ENDIF

!------------------------------------------------------

!*        2.    RETURN
!               ------

IF (LHOOK) CALL DR_HOOK('DEALSCR',1,ZHOOK_HANDLE)
END SUBROUTINE DEALSCR
