SUBROUTINE DEALSPA

!**** *DEALSPA*   - Routine to release spectral arrays

!     Purpose.
!     --------
!           To deallocate spectral arrays
!**   Interface.
!     ----------
!        *CALL* *DEALSPA

!        Explicit arguments :
!        --------------------
!        None.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by FREEMEM.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 95-09-28
!        Modified JUN 1997 by K. YESSAD: add missing deallocations.
!        Modified 97-11-18 by E. Holm  : add ozone deallocations.
!        Modified 02-09-30 by P.Smolikova : add deallocations for d4 in NH
!        Modified 03-09-30 by C. Fischer:deallocate mean wind profiles
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        M. Jidane : 05-04-2006 : add deallocation of GP7A3
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SP5A3    ,&
 & SP5A2    ,SPA3SAVE ,&
 & SPA2SAVE ,SPSPSAVE ,SPVOR    ,SPDIV    ,SP5VOR   ,SP5DIV   ,&
 & SPHV     ,SPT      ,SPQ      ,&
 & SPO3     ,SPL      ,SPI      ,SPSPD    ,SPSVD    ,SPNHX    ,&
 & SP5HV    ,SP5T     ,SP5Q     ,SP5O3    ,SP5L     ,&
 & SP5I     ,SP5SPD   ,SP5SVD   ,&
 & SPD2D    ,&
 & SPSP     ,SP5D2D   ,SP5SP    ,&
 & SPC2D    ,SPOR     ,SPGFL    ,SP5GFL   ,&
 & SPEVELT1 ,SPA1     ,SP5A1    ,SPUB     ,&
 & SPVB     ,SP5UB    ,SP5VB
USE YOMSPJB  , ONLY : SPJB, SP7A3    ,SP7A2, SP7A1, &
 & SP7VOR   ,SP7DIV, SP7HV    ,SP7T     ,&
 & SP7Q     ,SP7O3    ,SP7L     ,SP7I     ,SP7SPD   ,&
 & SP7SVD   ,SP7D2D   ,SP7SP    ,SP7UB    ,SP7VB    ,GP7A3
USE YOMLAP   , ONLY : NSPINDXG ,NSPNSUM
USE YOMSPT7  , ONLY : SPA3T7   ,SPA2T7   ,SPA1T7
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!      ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DEALSPA',0,ZHOOK_HANDLE)

!      ----------------------------------------------------------------

!*       1.    RELEASE SPECTRAL ARRAYS
!              -----------------------

IF (ALLOCATED(SPA3    )) DEALLOCATE(SPA3)
IF (ALLOCATED(SPA2    )) DEALLOCATE(SPA2)
IF (ALLOCATED(SPA1    )) DEALLOCATE(SPA1)
IF (ALLOCATED(SPA3SAVE)) DEALLOCATE(SPA3SAVE)
IF (ALLOCATED(SPA2SAVE)) DEALLOCATE(SPA2SAVE)
IF (ALLOCATED(SPSPSAVE)) DEALLOCATE(SPSPSAVE)

IF (ALLOCATED(SP7A3   )) DEALLOCATE(SP7A3)
IF (ALLOCATED(SP7A2   )) DEALLOCATE(SP7A2)
IF (ALLOCATED(SP7A1   )) DEALLOCATE(SP7A1)
IF (ALLOCATED(GP7A3   )) DEALLOCATE(GP7A3)

IF (ALLOCATED(SP5A3   )) DEALLOCATE(SP5A3)
IF (ALLOCATED(SP5A2   )) DEALLOCATE(SP5A2)
IF (ALLOCATED(SP5A1   )) DEALLOCATE(SP5A1)

IF (ALLOCATED(SPA3T7))   DEALLOCATE(SPA3T7)
IF (ALLOCATED(SPA2T7))   DEALLOCATE(SPA2T7)
IF (ALLOCATED(SPA1T7))   DEALLOCATE(SPA1T7)

CALL DEALLOCATE_SPEC(SPJB)

!*       2.    NULLIFY POINTERS
!              ----------------

IF (ASSOCIATED(SPVOR    )) NULLIFY(SPVOR)
IF (ASSOCIATED(SPDIV    )) NULLIFY(SPDIV)
IF (ASSOCIATED(SPHV     )) NULLIFY(SPHV)
IF (ASSOCIATED(SPT      )) NULLIFY(SPT)
IF (ASSOCIATED(SPQ      )) NULLIFY(SPQ)
IF (ASSOCIATED(SPO3     )) NULLIFY(SPO3)
IF (ASSOCIATED(SPL      )) NULLIFY(SPL)
IF (ASSOCIATED(SPI      )) NULLIFY(SPI)
IF (ASSOCIATED(SPGFL    )) NULLIFY(SPGFL)
IF (ASSOCIATED(SPSPD    )) NULLIFY(SPSPD)
IF (ASSOCIATED(SPSVD    )) NULLIFY(SPSVD)
IF (ASSOCIATED(SPNHX    )) NULLIFY(SPNHX)
IF (ASSOCIATED(SPD2D    )) NULLIFY(SPD2D)
IF (ASSOCIATED(SPSP     )) NULLIFY(SPSP)
IF (ASSOCIATED(SPEVELT1 )) NULLIFY(SPEVELT1)
IF (ASSOCIATED(SPC2D    )) NULLIFY(SPC2D)
IF (ASSOCIATED(SPOR     )) NULLIFY(SPOR)
IF (ASSOCIATED(SPUB     )) NULLIFY(SPUB)
IF (ASSOCIATED(SPVB     )) NULLIFY(SPVB)

IF (ASSOCIATED(SP5VOR   )) NULLIFY(SP5VOR)
IF (ASSOCIATED(SP5DIV   )) NULLIFY(SP5DIV)
IF (ASSOCIATED(SP5HV    )) NULLIFY(SP5HV)
IF (ASSOCIATED(SP5T     )) NULLIFY(SP5T)
IF (ASSOCIATED(SP5Q     )) NULLIFY(SP5Q)
IF (ASSOCIATED(SP5O3    )) NULLIFY(SP5O3)
IF (ASSOCIATED(SP5L     )) NULLIFY(SP5L)
IF (ASSOCIATED(SP5I     )) NULLIFY(SP5I)
IF (ASSOCIATED(SP5GFL   )) NULLIFY(SP5GFL)
IF (ASSOCIATED(SP5SPD   )) NULLIFY(SP5SPD)
IF (ASSOCIATED(SP5SVD   )) NULLIFY(SP5SVD)
IF (ASSOCIATED(SP5D2D   )) NULLIFY(SP5D2D)
IF (ASSOCIATED(SP5SP    )) NULLIFY(SP5SP)
IF (ASSOCIATED(SP5UB    )) NULLIFY(SP5UB)
IF (ASSOCIATED(SP5VB    )) NULLIFY(SP5VB)

IF (ASSOCIATED(SP7VOR   )) NULLIFY(SP7VOR)
IF (ASSOCIATED(SP7DIV   )) NULLIFY(SP7DIV)
IF (ASSOCIATED(SP7HV    )) NULLIFY(SP7HV)
IF (ASSOCIATED(SP7T     )) NULLIFY(SP7T)
IF (ASSOCIATED(SP7Q     )) NULLIFY(SP7Q)
IF (ASSOCIATED(SP7O3    )) NULLIFY(SP7O3)
IF (ASSOCIATED(SP7L     )) NULLIFY(SP7L)
IF (ASSOCIATED(SP7I     )) NULLIFY(SP7I)
IF (ASSOCIATED(SP7SPD   )) NULLIFY(SP7SPD)
IF (ASSOCIATED(SP7SVD   )) NULLIFY(SP7SVD)
IF (ASSOCIATED(SP7D2D   )) NULLIFY(SP7D2D)
IF (ASSOCIATED(SP7SP    )) NULLIFY(SP7SP)
IF (ASSOCIATED(SP7UB    )) NULLIFY(SP7UB)
IF (ASSOCIATED(SP7VB    )) NULLIFY(SP7VB)

IF (ALLOCATED(NSPINDXG)) DEALLOCATE(NSPINDXG)
IF (ALLOCATED(NSPNSUM)) DEALLOCATE(NSPNSUM)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DEALSPA',1,ZHOOK_HANDLE)
END SUBROUTINE DEALSPA
