SUBROUTINE DEALTC

!**** *DEALTC - Desallocations of arrays used for transmission coefficients
!               and spectral transformations of these coefficients.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *DEALTC(...)

!        Explicit arguments : none
!        --------------------

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about spectral transforms.

!     Externals.
!     ----------
!        none 

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP).

!     Modifications.
!     --------------
!        Original : FEB 1999
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR
USE YOMLUN   , ONLY : NULOUT
USE YOMFFTTC , ONLY : RBASCOTC ,RBASSITC
USE YOMFOUTC , ONLY : RFORAB3C ,RFORAB4C ,RFORAB6C ,RFORAB3N &
 & ,RFORAB4N ,RFORAB6N ,RFORAT1C ,RFORAT2C ,RFORAT3C &
 & ,RFORAT4C ,RFORAT5C ,RFORAT1N ,RFORAT2N ,RFORAT3N &
 & ,RFORAT4N ,RFORAT5N ,RFOCOR   ,RFOAC  

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IU

LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

!*  1. DEALLOCATIONS.
!   -----------------

IF (LHOOK) CALL DR_HOOK('DEALTC',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1 .OR. LALLOPR
IU = NULOUT

! * Basis of sinus-cosinus for Fourier transforms.

IF (ALLOCATED(RBASCOTC)) THEN
  DEALLOCATE(RBASCOTC)
  IF(LLP)WRITE(IU,9) 'RBASCOTC  '
ENDIF

IF (ALLOCATED(RBASSITC)) THEN
  DEALLOCATE(RBASSITC)
  IF(LLP)WRITE(IU,9) 'RBASSITC  '
ENDIF

! * Fourier representation of transmission coefficients.

IF (ALLOCATED(RFORAB3C)) THEN
  DEALLOCATE(RFORAB3C)
  IF(LLP)WRITE(IU,9) 'RFORAB3C  '
ENDIF

IF (ALLOCATED(RFORAB4C)) THEN
  DEALLOCATE(RFORAB4C)
  IF(LLP)WRITE(IU,9) 'RFORAB4C  '
ENDIF

IF (ALLOCATED(RFORAB6C)) THEN
  DEALLOCATE(RFORAB6C)
  IF(LLP)WRITE(IU,9) 'RFORAB6C  '
ENDIF

IF (ALLOCATED(RFORAB3N)) THEN
  DEALLOCATE(RFORAB3N)
  IF(LLP)WRITE(IU,9) 'RFORAB3N  '
ENDIF

IF (ALLOCATED(RFORAB4N)) THEN
  DEALLOCATE(RFORAB4N)
  IF(LLP)WRITE(IU,9) 'RFORAB4N  '
ENDIF

IF (ALLOCATED(RFORAB6N)) THEN
  DEALLOCATE(RFORAB6N)
  IF(LLP)WRITE(IU,9) 'RFORAB6N  '
ENDIF

IF (ALLOCATED(RFORAT1C)) THEN
  DEALLOCATE(RFORAT1C)
  IF(LLP)WRITE(IU,9) 'RFORAT1C  '
ENDIF

IF (ALLOCATED(RFORAT2C)) THEN
  DEALLOCATE(RFORAT2C)
  IF(LLP)WRITE(IU,9) 'RFORAT2C  '
ENDIF

IF (ALLOCATED(RFORAT3C)) THEN
  DEALLOCATE(RFORAT3C)
  IF(LLP)WRITE(IU,9) 'RFORAT3C  '
ENDIF

IF (ALLOCATED(RFORAT4C)) THEN
  DEALLOCATE(RFORAT4C)
  IF(LLP)WRITE(IU,9) 'RFORAT4C  '
ENDIF

IF (ALLOCATED(RFORAT5C)) THEN
  DEALLOCATE(RFORAT5C)
  IF(LLP)WRITE(IU,9) 'RFORAT5C  '
ENDIF

IF (ALLOCATED(RFORAT1N)) THEN
  DEALLOCATE(RFORAT1N)
  IF(LLP)WRITE(IU,9) 'RFORAT1N  '
ENDIF

IF (ALLOCATED(RFORAT2N)) THEN
  DEALLOCATE(RFORAT2N)
  IF(LLP)WRITE(IU,9) 'RFORAT2N  '
ENDIF

IF (ALLOCATED(RFORAT3N)) THEN
  DEALLOCATE(RFORAT3N)
  IF(LLP)WRITE(IU,9) 'RFORAT3N  '
ENDIF

IF (ALLOCATED(RFORAT4N)) THEN
  DEALLOCATE(RFORAT4N)
  IF(LLP)WRITE(IU,9) 'RFORAT4N  '
ENDIF

IF (ALLOCATED(RFORAT5N)) THEN
  DEALLOCATE(RFORAT5N)
  IF(LLP)WRITE(IU,9) 'RFORAT5N  '
ENDIF

IF (ALLOCATED(RFOCOR)) THEN
  DEALLOCATE(RFOCOR)
  IF(LLP)WRITE(IU,9) 'RFOCOR    '
ENDIF

IF (ALLOCATED(RFOAC)) THEN
  DEALLOCATE(RFOAC)
  IF(LLP)WRITE(IU,9) 'RFOAC     '
ENDIF

9 FORMAT(1X,'ARRAY ',A10,' DEALLOCATED IN DEALTC ')

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DEALTC',1,ZHOOK_HANDLE)
END SUBROUTINE DEALTC
