SUBROUTINE EMPTB(KWRDS,POUT,PBUF,KBLEN,KSTA,KEXPAND,KEXPB)

!**** *EMPTB  *  - Unpack ot just copy from buffer

!     Purpose.
!     --------
!           Unpack or just copy from user supplied buffer into array.
!           Update buffer pointer.

!**   Interface.
!     ----------
!        *CALL* *EMPTB(...)*

!        Explicit arguments : KWRDS   - number of words to unpack (copy)
!        -------------------- POUT    - array to recieve unpacked data
!                             PBUF    - buffer containing (packed) data
!                             KBLEN   - length of PBUF
!                             KSTA    - start adress in buffer,
!                                       updated by routine
!                             KEXPAND - packing factor (1 to 4)
!                             KEXPB   - number of bits for exponent in
!                                       packed data

!        Implicit arguments : None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  EXPAND21 - unpacks data packed 2 to 1 (Cray routine)
!     ----------  EXPAND31 - unpacks data packed 3 to 1 (Cray routine)
!                 EXPAND41 - unpacks data packed 4 to 1 (Cray routine)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-01-01
!        Modification : 93-06-01 M.Hamrud (comments added)
!        Modification : 94-09-12 M.Hamrud (temp. fix for bug in EXPAND21)
!        Modification : 95-11-01 D.Dent   (remove fix for bug in EXPAND21)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KWRDS 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT(KWRDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBUF(:) 
INTEGER(KIND=JPIM)               :: KBLEN ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEXPAND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEXPB 
! === END OF INTERFACE BLOCK ===

INTEGER(KIND=JPIM) :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    UNPACK DATA.
!              ------------

!*       1.1   NO PACKING
IF (LHOOK) CALL DR_HOOK('EMPTB',0,ZHOOK_HANDLE)
IF(KEXPAND == 1) THEN
  DO J=1,KWRDS
    POUT(J)=PBUF(KSTA+J-1)
  ENDDO
  KSTA=KSTA+KWRDS

!*       1.2   PACKING 2 TO 1
ELSEIF(KEXPAND == 2) THEN
  CALL EXPAND21(KWRDS,PBUF(KSTA),POUT,KEXPB,1)
  KSTA=KSTA+(KWRDS+1)/2

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EMPTB',1,ZHOOK_HANDLE)
END SUBROUTINE EMPTB

