SUBROUTINE FILLB3(KLEVG,KWRDS,PIN,PBUF,KSTA,KPACK,KEXPB)

!**** *FILLB3  *  - Copy into buffer

!     Purpose.
!     --------
!           Copy from array into user supplied buffer .
!           Update buffer pointer.

!**   Interface.
!     ----------
!        *CALL* *FILLB3(...)*

!        Explicit arguments : KLEVG   - Number of fields
!                             KWRDS   - number of words to unpack (copy)
!                             PIN     - array containing unpacked data
!                             PBUF    - buffer containing (packed) data
!                             KSTA    - start adress in buffer,
!                                       updated by routine
!                             KPACK   - packing factor (1 to 4)
!                             KEXPB   - number of bits for exponent in
!                                       packed data

!        Implicit arguments : None
!        --------------------

!     Method.
!     -------
!        3D version of FILLB


!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Deborah Salmond  *ECMWF*

!     Modifications.
!     --------------
!        Original : 09-01-01
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM    ,ONLY : NPROMA

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVG
INTEGER(KIND=JPIM),INTENT(IN)    :: KWRDS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIN(NPROMA,KLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBUF(:) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPACK 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEXPB 
! === END OF INTERFACE BLOCK ===

INTEGER(KIND=JPIM) :: J, JLEV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FILLB3',0,ZHOOK_HANDLE)

DO JLEV=1,KLEVG
  DO J=1,KWRDS
    PBUF(KSTA+J-1)=PIN(J,JLEV)
  ENDDO
  KSTA=KSTA+KWRDS
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FILLB3',1,ZHOOK_HANDLE)
END SUBROUTINE FILLB3

