SUBROUTINE FREEMEM

!**** *FREEMEM*   - Routine to free memory used by pointer allocations

!     Purpose.
!     --------
!           To deallocate arrays

!**   Interface.
!     ----------
!        *CALL* *FREEMEM*

!        Explicit arguments :
!        --------------------
!        None.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by CPREP4.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 95-09-28
!        Modified : 96-01-09 R. El Khatib (deallocate arrays used in Aladin)
!        Modified : 97-06-17 K. YESSAD (add missing deallocations).
!        Modified : 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!        Modified : 00-10-25 M Hamrud (transform package)
!                   01-04-18 G Radnoti (ald transform package)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LELAM

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "dealcan.intfb.h"
#include "dealcos.intfb.h"
#include "dealctv.intfb.h"
#include "dealddh.intfb.h"
#include "dealfpos.intfb.h"
#include "dealges.intfb.h"
#include "deallo.intfb.h"
#include "dealmod.intfb.h"
#include "dealnmi.intfb.h"
#include "dealsc2.intfb.h"
#include "dealscr.intfb.h"
#include "dealspa.intfb.h"
#include "dealtc.intfb.h"
#include "deello.intfb.h"
!     ------------------------------------------------------------------

!*       1.    RELEASE ARRAYS ALLOCATED IN SUALLO
!              -----------------------------------

IF (LHOOK) CALL DR_HOOK('FREEMEM',0,ZHOOK_HANDLE)
CALL DEALLO

!*       2.    RELEASE FULLPOS-RELATED ARRAYS 
!              -------------------------------

CALL DEALFPOS

!*       3.    RELEASE DDH-RELATED ARRAYS 
!              --------------------------

CALL DEALDDH

!*       4.    RELEASE COST FUNCTION RELATED ARRAYS 
!              ------------------------------------

CALL DEALCOS

!*       5.    RELEASE CONTROL VARIABLE RELATED ARRAYS
!              ---------------------------------------

CALL DEALCTV

!*       6.    RELEASE FIRST GUESS CONSTRAINT RELATED ARRAYS
!              ---------------------------------------------

CALL DEALGES

!*       7.    RELEASE CANARI RELATED ARRAYS 
!              -----------------------------

CALL DEALCAN

!*       8.    RELEASE SPECTRAL ARRAYS 
!              -----------------------

CALL DEALSPA

!*       9.    RELEASE GRIDPOINT BUFFERS ETC.
!              ------------------------------

CALL DEALSC2

!*       10.   RELEASE ARRAYS USED IN SCREENING OF OBSERVATIONS.
!              -------------------------------------------------

CALL DEALSCR(.TRUE.)

!*       11.   RELEASE ARRAYS USED IN MODEL INTEGRATION.
!              -----------------------------------------

CALL DEALMOD

!*       12.   RELEASE ARRAYS USED IN NMI.
!              ---------------------------

CALL DEALNMI

!*       13.   RELEASE ARRAYS USED SLOW FOURIER TRANSFORMS OF 'TC' COEFFICIENTS.
!              -----------------------------------------------------------------

CALL DEALTC

!*       14.    RELEASE SPACE ALLOCATED FOR ALADIN
!               ----------------------------------

IF (LELAM) CALL DEELLO
!     ------------------------------------------------------------------

!*       15.    RELEASE SPACE ALLOCATED BY TRANSFORM PACKAGE
!               --------------------------------------------
IF (LELAM) THEN
  CALL ETRANS_END
ELSE
  CALL TRANS_END('FINAL')
ENDIF
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FREEMEM',1,ZHOOK_HANDLE)
END SUBROUTINE FREEMEM
