SUBROUTINE GRID_PSGLOBAL(PSPSP)

!     Purpose.
!     --------
!       Compute global ps grid field from distributed lnps spectral field.
!       High resolution gridpoint surface pressure is required for 
!       conserving interpolation.

!     Arguments.
!     ----------
!       PSPSP   : Local lnPs spectral field

!     Author.
!     -------
!       E.Holm (part of PRE_GRID_BICONSERV by Y. Tremolet)

!     Modifications.
!     --------------
!       Original : 12-Feb-2008
! ----------------------------------------------------------------------

USE PARKIND1,ONLY: JPIM, JPRB
USE YOMHOOK ,ONLY: LHOOK, DR_HOOK
USE YOMGEM  ,ONLY: NGPTOTG, NGPTOT
USE YOMDIM  ,ONLY: NSPEC2
USE YOM_GRID_BICONSERV, ONLY: RGPPRS_HR

IMPLICIT NONE

REAL(KIND=JPRB), INTENT(IN) :: PSPSP(NSPEC2)

REAL(KIND=JPRB), ALLOCATABLE :: ZPRSGLO(:,:),ZPRSLOC(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gathergpf.intfb.h"
#include "speree.intfb.h"
! ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRID_PSGLOBAL',0,ZHOOK_HANDLE)

ALLOCATE(ZPRSGLO(NGPTOTG,1))
ALLOCATE(ZPRSLOC(NGPTOT))

!-----------------------------------------------------------------------
! 1. Transform HR lnPs to local gridpoint space and convert from 
!    ln(ps) to ps.
!-----------------------------------------------------------------------
CALL SPEREE(1,1,PSPSP,ZPRSLOC)
ZPRSLOC(:)=EXP(ZPRSLOC(:))

!-----------------------------------------------------------------------
! 2. Gather local grid point sections to a global field and 
!    write to module
!-----------------------------------------------------------------------
CALL GATHERGPF(ZPRSLOC,ZPRSGLO,1,-1)

RGPPRS_HR(:)=ZPRSGLO(:,1)

DEALLOCATE(ZPRSGLO)
DEALLOCATE(ZPRSLOC)

IF (LHOOK) CALL DR_HOOK('GRID_PSGLOBAL',1,ZHOOK_HANDLE)
END SUBROUTINE GRID_PSGLOBAL
