SUBROUTINE IOPACK(CDCONF)

!**** *IOPACK*  - input/output handling during integration

!     Purpose.
!     --------
!     Write or read the trajectory, write history file ,write out
!     post-processed data or create restart files.

!**   Interface.
!     ----------
!        *CALL* *IOPACK(CDCONF)

!        Explicit arguments :     CDCONF - configuration of call
!        --------------------

!        Implicit arguments :      None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*
!        David Dent                         *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-08-02
!        Version 1: 92-04-06
!        Modified 94-03-09 L.Labbe : CDCONF='V' read verification file (821)
!        Modified 94-04-06 R. El Khatib Full-POS implementation
!        Modified 94-05-10 M.Hamrud ( Move writing of restart files)
!        Modified 94-07-28 J.N. Thepaut (IO on the increments : TL version of
!                                        the incremental)
!        Modified 94-10-27 J.N. Thepaut (Cleaning)
!        Modified 95-07-21 A. Horanyi for ALADIN
!        Modified 95-07-21 R. El Khatib Full-POS ALADIN
!        Modified 95-12-20 J.N. Thepaut (introduction of Full Pos in incremental)
!        Modified 96-09-25 F Ranier : CDCONF='V' read verification file (801)
!        Modified 96-11    M.Janiskova (writing data for simpl.rad.scheme)
!        Modified 97-08-04 G.Hello : read external files or traj (801)
!        Modified 97-10-30 F.Veerse  (No call to WRTTL at DMN)
!        Modified 97-05-23 L. Dragulanescu : ALADIN 3DVAR
!        Modified 98-09-24 C. Fischer+C. Soci: read traj in core if NTRAJE=0
!        Modified 98-10-14 R. El Khatib Full-POS Aladin extension zone
!        Modified 99-02-17 by K. YESSAD: options LRETCFOU, LWRTCFOU.
!        Modified 99-01-11 R. El Khatib Remove wrsfpmio (old-fashionned) ;
!                                       EZOBUF allocated/released here
!                                       FPEZO2H called with CDCONF
!                                       Simplify fullpos control
!        Modified 00-08-09 Y.Tremolet   Cleaning
!        K. YESSAD (10-2002): option lwrtcfou=T: allow to store a temporal
!         average of transmission coefficients.
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-18 Move aladin fullpos computations into stepo.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Seity      11-Jan-2008 add surfex output files (LLS)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LARPEGEF ,NCONF   ,NSTOP   ,LFDBOP  ,LELAM ,LWRTCFOU
USE YOMCT3   , ONLY : NSTEP
USE YOMVRTL  , ONLY : LTLINT
USE YOMFPC   , ONLY : CFPFMT
USE YOMMP    , ONLY : NOUTTYPE
USE YOMSENS  , ONLY : NJROPT
USE YOMVAR   , ONLY : LTRREF

!     ------------------------------------------------------------------

IMPLICIT NONE

! * INPUT:
CHARACTER(LEN=1), INTENT(IN)    :: CDCONF
INTEGER(KIND=JPIM) :: IOUTTYPE

LOGICAL :: LL801, LLA, LLB, LLC, LLFDBOP, LLHS,&
 & LLI, LLJ, LLL, LLR, LLS, LLT, LLV  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "add3to5.intfb.h"
#include "coptra.intfb.h"
#include "ecoptra.intfb.h"
#include "rd801.intfb.h"
#include "rdrinc.intfb.h"
#include "suespe0.intfb.h"
#include "suspe0.intfb.h"
#include "wrhfp.intfb.h"
#include "wrinc.intfb.h"
#include "wrmlfp.intfb.h"
#include "wrmlpp.intfb.h"
#include "wrmlppa.intfb.h"
#include "wrmlppl.intfb.h"
#include "wrplfp.intfb.h"
#include "wrpvlfp.intfb.h"
#include "wrresf.intfb.h"
#include "wrsfp.intfb.h"
#include "wrtcfou.intfb.h"
#include "wrthlfp.intfb.h"
#include "aro_surf_diagh.intfb.h"

!     ------------------------------------------------------------------

!*       1.    WRITE TRAJECTORY.
!              -----------------

IF (LHOOK) CALL DR_HOOK('IOPACK',0,ZHOOK_HANDLE)
CALL GSTATS(11,0)
LLA  = CDCONF == 'A'.OR.CDCONF == 'F'.OR.CDCONF == 'D'.OR.CDCONF == 'S'
LLB  = CDCONF == 'B'
LLC  = CDCONF == 'C'
LLI  = CDCONF == 'I'
LLJ  = CDCONF == 'J'
LLR  = CDCONF == 'F'.OR.CDCONF == 'R'
LLS  = CDCONF == 'S'
LLT  = CDCONF == 'T'
LLL  = CDCONF == 'L'
LLV  = CDCONF == 'V'
LLHS = CDCONF == 'E'.OR.CDCONF == 'U'.OR.CDCONF == 'Y'.OR.&
 & CDCONF == 'M'.OR.CDCONF == 'Z'  
LL801=NSTEP == NSTOP.AND.NCONF == 801.AND..NOT.LTLINT

IF(NCONF/100 == 1) THEN
  LLFDBOP=LFDBOP
  LFDBOP=.FALSE.
  IOUTTYPE=NOUTTYPE
  NOUTTYPE=1
ENDIF

IF (LLT) THEN
  CALL COPTRA(CDCONF)
  IF (LELAM) CALL ECOPTRA(CDCONF)
ENDIF

!     ------------------------------------------------------------------

!*       2.   READ TRAJECTORY.
!             ----------------

IF(LLB) THEN
  IF (LL801.AND. NJROPT == 1) THEN
    IF (.NOT.LTRREF) THEN
      CALL COPTRA(CDCONF)
      IF (LELAM) CALL ECOPTRA(CDCONF)
    ELSE
      CALL RD801
    ENDIF
  ELSE
    CALL COPTRA(CDCONF)
    IF (LELAM) CALL ECOPTRA(CDCONF)
  ENDIF
ENDIF
IF(LLV) THEN
  IF((NCONF == 801).AND.(NJROPT == 1))THEN
    IF(.NOT. LTRREF) THEN
      CALL COPTRA(CDCONF)
      IF (LELAM) THEN
        CALL ECOPTRA(CDCONF)
        CALL ADD3TO5
        CALL SUSPE0
        CALL SUESPE0
      ENDIF
    ELSE
      CALL RD801
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       3.    WRITE RESTART FILES
!              -------------------

IF (LLR) THEN
  CALL WRRESF
ENDIF

!     ------------------------------------------------------------------

!*       4.    WRITE PRESSURE LEVEL DATA.
!              --------------------------

IF (LLC) THEN
  IF (LELAM) THEN
    CALL ABOR1(" IOPACK: CONFIGURATION C IS NOT IN USE ANY MORE")
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       5.    WRITE MODEL LEVEL DATA.
!              -----------------------

IF (LLA) THEN
  IF (LELAM) THEN
    CALL WRMLPPA(CDCONF)
  ELSE
    CALL WRMLPP(CDCONF)
  ENDIF
ENDIF

IF (LLL) THEN
  IF (.NOT.LARPEGEF) THEN
    CALL WRMLPPL(CDCONF)
  ELSE
    CALL ABOR1('IOPACK : CONFIGURATION ''L'' NOT EXPECTED')
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       6.    WRITE MODEL SURFACE DATA.
!              -----------------------

IF (LLS) THEN
   CALL ARO_SURF_DIAGH
ENDIF

!     ------------------------------------------------------------------

!*       7.    WRITE FULL POST-PROCESSING DATA
!              -------------------------------

IF(LLHS) THEN
  IF (.NOT.LARPEGEF) THEN
    IF (CDCONF == 'E') THEN
      CALL WRPLFP(CDCONF)
    ELSEIF (CDCONF == 'Y') THEN
      CALL WRPVLFP(CDCONF)
    ELSEIF (CDCONF == 'M') THEN
      CALL WRTHLFP(CDCONF)
    ELSE
      CALL WRMLFP(CDCONF)
    ENDIF
  ELSE
    IF (CFPFMT == 'MODEL') THEN
      CALL WRSFP(CDCONF)
    ELSE
      CALL WRHFP(CDCONF)
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       8.    WRITE INCREMENTS
!              -------------------

IF (LLI) THEN
  CALL WRINC
ENDIF

!     ------------------------------------------------------------------

!*       9.    READ INCREMENTS.
!              ----------------

IF (LLJ) THEN
  CALL RDRINC
ENDIF

!     ------------------------------------------------------------------

IF(NCONF/100 == 1) THEN
  LFDBOP=LLFDBOP
  NOUTTYPE=IOUTTYPE
ENDIF

!     ------------------------------------------------------------------

!*       10.  WRITE DATA FOR SIMPLIFIED RADIATION SCHEME

! * storage in Fourier coefficients.
IF (LWRTCFOU.AND.NSTEP == NSTOP) THEN
  CALL WRTCFOU
  LWRTCFOU=.FALSE.
ENDIF

CALL GSTATS(11,1)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('IOPACK',1,ZHOOK_HANDLE)
END SUBROUTINE IOPACK
