SUBROUTINE MATRIXIN(CDMETHOD,LDCHECK,PMATRIX,KDIM1,KDIM2,PEPS,KFLAG)

!**** MATRIXIN - A HAT ROUTINE FOR MATIX INVERTION

!      D. VASILJEVIC        ECMWF      20/11/90

!     PURPOSE
!     -------

!        THE AIM OF THIS ROUTINE IS TO COMPUTE THE INVERSE OF THE OBSE-
!     RVATION ERROR CORRELATION MATRIX USING EITHER MINV (CRAY ROUTINE)
!     OR SYMINV (ECMWF ECLIB ROUTINE) OR MAYBE OTHER IN THE FUTURE.

!**   INTERFACE
!     ---------

!        CALL MATRIXIN(PMATRIX,KDIM1,KDIM2,PSCRAT,PEPS)

!     WHERE CDMETHOD = CHOSEN METHOD FOR MATRIX INVERTION
!           KDIM1   = MATRIX ORDER
!           KDIM2   = LEADING DIMENSION
!           PEPS    = TOLERANCE

!**   METHOD
!     ------

!        THE SUBSEQUENT MATRIX INVERTION ROUTINE IS CALLED

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Drusch      16-Jul-2007 introduce KFLAG

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM2
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDMETHOD 
LOGICAL           ,INTENT(IN)    :: LDCHECK 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMATRIX(KDIM2,KDIM2)
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM1
REAL(KIND=JPRB),INTENT(IN)       :: PEPS
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KFLAG
REAL(KIND=JPRB) :: ZMATRIX(KDIM2,KDIM2)
REAL(KIND=JPRB) :: ZREZULT(KDIM2,KDIM2)
REAL(KIND=JPRB) :: ZSCRAT (3*KDIM2)
INTEGER(KIND=JPIM) :: J1, J2, JC1, JC2

REAL(KIND=JPRB) :: ZCOND, ZDET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!        1.       MATRIX INVERTION
!                 ----------------

IF (LHOOK) CALL DR_HOOK('MATRIXIN',0,ZHOOK_HANDLE)

IF (PRESENT(KFLAG)) KFLAG = 0

IF(LDCHECK)         THEN
  DO JC1 = 1 , KDIM2
    DO JC2 = 1 , KDIM2
      ZMATRIX(JC1,JC2) = PMATRIX(JC1,JC2)
    ENDDO
  ENDDO
ENDIF

!*          1.1   GAUSS ELIMINATION (ROUTINE MINV)

IF(CDMETHOD == 'GAUSS')         THEN
  CALL MINV(PMATRIX,KDIM1,KDIM2,ZSCRAT,ZDET,PEPS,0,1)

!*          1.2   CHOLESKI'S METHOD (ROUTINE SYMINV)

ELSEIF(CDMETHOD == 'CHOLESKI')         THEN
  ZCOND=0.0_JPRB
  CALL SYMINV(PMATRIX,KDIM2,KDIM1,ZCOND,ZSCRAT)
  IF(ZCOND < PEPS)       THEN
    WRITE(NULOUT,'('' ILL-CONDITION'',2(E12.6,1X))')ZCOND,PEPS
    IF (PRESENT(KFLAG)) KFLAG = 1
  ENDIF

!*          1.2.1 COPY THE LOWER PART

  DO JC1=1,KDIM1
    DO JC2=1,JC1
      PMATRIX(JC2,JC1)=PMATRIX(JC1,JC2)
    ENDDO
  ENDDO

!*          1.3   NOT KNOWN METHOD

ELSE
  WRITE(NULOUT,'('' *** ERROR IN MATRIXIN: METHOD UNKNOWN'',A)')CDMETHOD
  CALL ABOR1('MATRIXIN: ABOR1 CALLED')
ENDIF

!*          1.4   A*A-1

IF(LDCHECK)        THEN
  CALL MXMAOP(ZMATRIX,KDIM2,1,PMATRIX,KDIM2,1,&
   & ZREZULT,KDIM2,1,KDIM1,KDIM1,KDIM1)  
  WRITE(NULOUT,'('' MATRIXIN: ORIGINAL MATRIX (A)'')')
  DO J1=1,KDIM1
    WRITE(NULOUT,'('' '',15(F7.2,1X))')(ZMATRIX(J1,J2),J2=1,KDIM1)
  ENDDO
  WRITE(NULOUT,'('' MATRIXIN: INVERTED MATRIX (A-1)'')')
  DO J1=1,KDIM1
    WRITE(NULOUT,'('' '',15(F7.2,1X))')(PMATRIX(J1,J2),J2=1,KDIM1)
  ENDDO
  WRITE(NULOUT,'('' MATRIXIN: A*A-1'')')
  DO J1=1,KDIM1
    WRITE(NULOUT,'('' '',15(F7.2,1X))')(ZREZULT(J1,J2),J2=1,KDIM1)
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('MATRIXIN',1,ZHOOK_HANDLE)
END SUBROUTINE MATRIXIN
