SUBROUTINE MODEL2MODERR(KDIR,YDSPEC,YDGRID)
!     Purpose.
!     --------
!      Copy from/to model-state/model-error 

!     Author.
!     -------
!      G. Radnoti 

!     Modifications.
!     --------------
!        Original : 
!        G. Radnoti: 27-Nov-2006
!     ------------------------------------------------------------------
USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMSP   , ONLY: SPA3, SPA2
USE YOMGMV  , ONLY: GMV, GMVS
USE YOMGFL  , ONLY: GFL
USE YOMDIM  , ONLY: NSPEC2, NFLEVL, NPROMA, NGPBLKS, NFLEVG
USE YOMGEM  , ONLY: NGPTOT
USE YOMMODEL_ERROR, ONLY: MERRSP3, MERRSP2, MERRGFL, MERRGMV, MERRGMVS 
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(GRIDPOINT_FIELD), INTENT(INOUT) :: YDGRID
TYPE(SPECTRAL_FIELD),  INTENT(INOUT) :: YDSPEC
INTEGER(KIND=JPIM)  ,  INTENT(IN)    :: KDIR 

INTEGER(KIND=JPIM) :: jj,jf,jl,jb,jg,ilen
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('MODEL2MODERR',0,ZHOOK_HANDLE)

IF (KDIR == 1) THEN
    DO jf=1,YDSPEC%NS3D
      jg=MERRSP3(jf)
      DO jj=1,NSPEC2
        DO jl=1,NFLEVL
          YDSPEC%SP3D(jl,jj,jf) = SPA3(jl,jj,jg)
        ENDDO
      ENDDO
    ENDDO

    DO jf=1,YDSPEC%NS2D
      jg=MERRSP2(jf)
      DO jj=1,NSPEC2
        YDSPEC%SP2D(jj,jf) = SPA2(jj,jg)
      ENDDO
    ENDDO

    DO jb=1,NGPBLKS
      ilen=MIN(NPROMA, NGPTOT-(jb-1)*NPROMA)

      DO jf=1,YDGRID%NG3D
        IF (MERRGMV(jf)>0) THEN
          jg=MERRGMV(jf)
          DO jl=1,NFLEVG
            DO jj=1,ilen
              YDGRID%GP3D(jj,jl,jf,jb) = GMV(jj,jl,jg,jb)
            ENDDO
          ENDDO
        ELSEIF (MERRGFL(jf)>0) THEN
          jg=MERRGFL(jf)
          DO jl=1,NFLEVG
            DO jj=1,ilen
              YDGRID%GP3D(jj,jl,jf,jb) = GFL(jj,jl,jg,jb)
            ENDDO
          ENDDO
        ELSE
          CALL ABOR1('MODEL2MODERR: Field not found in GMV or GFL')
        ENDIF
      ENDDO

      DO jf=1,YDGRID%NG2D
        IF (MERRGMVS(jf)>0) THEN
          jg=MERRGMVS(jf)
          DO jj=1,ilen
            YDGRID%GP2D(jj,jf,jb) = GMVS(jj,jg,jb)
          ENDDO
        ELSE
          CALL ABOR1('MODEL2MODERR: Field not found in GMVS')
        ENDIF
      ENDDO
    ENDDO
ELSEIF (KDIR == -1) THEN
    DO jf=1,YDSPEC%NS3D
      jg=MERRSP3(jf)
      DO jj=1,NSPEC2
        DO jl=1,NFLEVL
          SPA3(jl,jj,jg) = YDSPEC%SP3D(jl,jj,jf)
        ENDDO
      ENDDO
    ENDDO

    DO jf=1,YDSPEC%NS2D
      jg=MERRSP2(jf)
      DO jj=1,NSPEC2
        SPA2(jj,jg) = YDSPEC%SP2D(jj,jf)
      ENDDO
    ENDDO

    DO jb=1,NGPBLKS
      ilen=MIN(NPROMA, NGPTOT-(jb-1)*NPROMA)

      DO jf=1,YDGRID%NG3D
        IF (MERRGMV(jf)>0) THEN
          jg=MERRGMV(jf)
          DO jl=1,NFLEVG
            DO jj=1,ilen
              GMV(jj,jl,jg,jb) = YDGRID%GP3D(jj,jl,jf,jb)
            ENDDO
          ENDDO
        ELSEIF (MERRGFL(jf)>0) THEN
          jg=MERRGFL(jf)
          DO jl=1,NFLEVG
            DO jj=1,ilen
              GFL(jj,jl,jg,jb) = YDGRID%GP3D(jj,jl,jf,jb)
            ENDDO
          ENDDO
        ELSE
          CALL ABOR1('MODEL2MODERR: Field not found in GMV or GFL')
        ENDIF
      ENDDO

      DO jf=1,YDGRID%NG2D
        IF (MERRGMVS(jf)>0) THEN
          jg=MERRGMVS(jf)
          DO jj=1,ilen
            GMVS(jj,jg,jb) = YDGRID%GP2D(jj,jf,jb)
          ENDDO
        ELSE
          CALL ABOR1('MODEL2MODERR: Field not found in GMVS')
        ENDIF
      ENDDO
    ENDDO
ELSE
    CALL ABOR1('MODEL2MODERR: UNKNOWN DIRECTION')
ENDIF 
IF (LHOOK) CALL DR_HOOK('MODEL2MODERR',1,ZHOOK_HANDLE)
END SUBROUTINE MODEL2MODERR       
