SUBROUTINE MODELTOJB(YDSPJB)

!**** *MODELTOJB * - Routine to transfer model state fields to
!                     control variable spectral fields
!     Purpose.
!     --------
!           Transfer for 
!           SPA3, SPA2, SPA1 and possibly GFL
!           to SPA3JB, SPA2JB and SPA1JB
!**   Interface.
!     ----------
!         *CALL* *MODELTOJB*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Gabor Radnoti *ECMWF*

!     Modifications.
!     --------------
!        Original : 2007-01-30

USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NPROMA
USE YOMSP    , ONLY : SPA3,   SPA2,   SPA1, SPGFL
USE YOMGFL   , ONLY : GFL
USE YOMMP    , ONLY : NBSETLEV, NPSP
USE YOMJG    , ONLY : N_SPJB_VARS, SPJB_VARS_INFO
USE YOMCT0   , ONLY : LELAM
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSPJB

INTEGER(KIND=JPIM) :: JFIELD, IPT, IPTJB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "dir_trans.h"
END INTERFACE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('MODELTOJB',0,ZHOOK_HANDLE)

DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB
  IPT   = SPJB_VARS_INFO(JFIELD)%IPT

  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN

    IF (NPSP>0) THEN
      YDSPJB%SP2D(:,IPTJB)= SPA2(:,IPT)
    ENDIF

  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA3) THEN

    YDSPJB%SP3D(:,:,IPTJB)=SPA3(:,:,IPT)

  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPGFL) THEN

    YDSPJB%SP3D(:,:,IPTJB)=SPGFL(:,:,IPT)
  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_GPGFL) THEN

    CALL DIR_TRANS (PSPSCALAR=YDSPJB%SP3D(:,:,IPTJB), PGP=GFL(:,:,IPT,:), &
                  & KVSETSC=NBSETLEV, KPROMA=NPROMA)

  ELSE

    WRITE (NULOUT,*) 'DON''T KNOW WHERE TO GET JB MODEL FIELD : ',JFIELD
    CALL ABOR1 ('DON''T KNOW WHERE TO GET JB MODEL FIELD')

  ENDIF
ENDDO

!-- SPA1 is a special case!

IF (LELAM) YDSPJB%SP1D(:,:)=SPA1(:,:)

IF (LHOOK) CALL DR_HOOK('MODELTOJB',1,ZHOOK_HANDLE)

END SUBROUTINE MODELTOJB
