SUBROUTINE OPENFA(KFILE,KUNTIN,KDEB2,KFIN,KINDEX,KDATE)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULCL1   ,NULCL2   ,NULUSR1  ,&
 & NPPPSH   ,NULUSR2  ,NINISH   ,NINMSH   ,NFGISH   ,&
 & NBIAS    ,NULASE   ,NULASS   ,NTIDE    ,NEGASH   ,NULTRAJHR,NULTRAJBG
USE YOMDIM   , ONLY : NDGLG    ,NDGSAG   ,NDGENG   ,NDLON    ,NFLEVG   ,NSMAX
USE YOMCST   , ONLY : RDAY
USE YOMGEM   , ONLY : NLOENG   ,NMENG    ,VALH     ,&
 & VBH      ,NSTTYP   ,NHTYP    ,RMUCEN   ,RLOCEN   ,&
 & RSTRET   ,VP00  
USE YOMCT0   , ONLY : CNMEXP   ,NQUAD    ,LELAM    ,NCONF
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS
USE YOMCT1   , ONLY : LRFILAF
USE YOMOP    , ONLY : CFNPL    ,CFNISH   ,CFNGSH    ,CFNRF    ,CEFNLSH  ,&
 & CFNBS   ,CFNFGI   ,CFNANI   ,CFANS    ,CFNTRAJHR ,CFNTRAJBG,CETSTAMP
USE YOMFPC   , ONLY : LFPMOIS
USE YOMCT3   , ONLY : NSTEP
USE YEMDYN   , ONLY : TEFRCL
USE YOMDYN   , ONLY : TSTEP
USE YOMINI   , ONLY : LBIAS
USE YEMGEO   , ONLY : LMAP
USE YOMVAR   , ONLY : L3DFGAT

!**** *OPENFA*  - Open a file Arpege

!     Purpose.
!     --------
!           To open a file Arpege/Aladin and test compatibility with model

!**   Interface.
!     ----------
!        *CALL* *OPENFA(...)*

!        Explicit arguments :
!        ------------------
!        INPUT:
!         KFILE   - file identifier
!                 0 = historical file
!                 1 = minimization file
!                 2 = reference file
!                 3 = bias file for incremental DFI
!                 4 = reference file for blending
!                 5 = preprocessing file
!                 6 = background low resolution file
!                 7 = analysis low resolution
!                 8 = Surface analysis
!                 9 = coupling file

!                10 = source climatology file (unique)
!                17 = source climatology file #1
!                18 = source climatology file #2
!                19 = climatology file
!                11 = avarc reference file 

!                30 = cumulated fluxes file

!                40 = instantaneous fluxes file
!               -11 = Jk global analysis in lam geom. file

!        OUTPUT:
!         KUNTIN  - file unit number
!         KDEB2   - first latitude to be read from file
!         KFIN    - last latitude to be read from file
!         KINDEX  - First gridpoint to be read on a field
!         KDATE   - date in file

!        Implicit arguments :
!        --------------------
!        See 'USE MODULE' above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Calls 'FA' and LFI routines.
!        Calls CHIEN and CCHIEN.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*
!        from SUGRCLIA etc 

!     Modifications.
!     --------------
!        ORIGINAL : 97-11-17
!        R. El Khatib : 98-02-26 : NFPMOIS
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0.
!          - removal of LLPOLE and code under LRPOLE, LLPOLE.
!          - removal of LRPOLE in 'CALL CHIEN'.
!        R. El Khatib : 98-12-16 : Coupling files and other missing cases
!        R. El Khatib : 00-02-17 : non-fatal cases(KFILE=4,17 or 18)
!        00-04-05: C. Fischer+C. Soci: bypass the conflict in model versus
!          file date for Aladin conf 801(same old problem as in cy19t1)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        G. Desroziers and K. Yessad (sept 2005):
!         - adapt option LTRAJHR to METEO-FRANCE configurations.
!           => new KFILE=50 for high resolution trajectory files.
!           => new KFILE=51 for background files.
!        R. ElKhatib   24-Oct-2008 Merge YEMOP and YOMOP
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KUNTIN 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KDEB2 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KFIN 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KINDEX 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KDATE(11) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDATES(11), IINC, ILEN
INTEGER(KIND=JPIM) :: IAN, IJOUR, IMOI, INBARI, IREP, ISSS, ISTEP, ITEST, IUNTIN

LOGICAL :: LLFICP, LLEXIST

REAL(KIND=JPRB) :: ZGEMU(NDGSAG:NDGENG)
REAL(KIND=JPRB) :: ZEPS, ZFTIME, ZMTIME, ZMTIME1

CHARACTER :: CLSTTU*7, CLFN*200, CLLEC*16, CLIDEN*16
CHARACTER :: CLLEC2*16, CLINC*4
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "cchien.intfb.h"
#include "chien.intfb.h"

#include "fcttim.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('OPENFA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1.    PREPARATIONS
!           ------------

IF(KFILE == 0) THEN
  CLFN=CFNISH
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NINISH
  WRITE(UNIT=NULOUT,FMT='(A)')' HISTORICAL INPUT DATA TO BE READ FROM FILE  '
ELSEIF(KFILE == 1) THEN
  CLFN=CFNGSH
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NINMSH
  WRITE(UNIT=NULOUT,FMT='(A)')' MINIMIZATION DATA TO BE READ FROM FILE  '
ELSEIF(KFILE == 2) THEN
  CLFN=CFNRF
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NFGISH
  WRITE(UNIT=NULOUT,FMT='(A)')' REFERENCE DATA TO BE READ FROM FILE  '
ELSEIF(KFILE == 3) THEN
  CLFN=CFNBS
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NBIAS
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' BIAS DATA FOR INCREMENTAL DFI TO BE READ FROM FILE  '  
ELSEIF(KFILE == 4) THEN
  CLFN=CFNRF
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NFGISH
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' SPECTRAL GUESS DATA FOR BLENDING TO BE READ FROM FILE  '  
ELSEIF(KFILE == 5) THEN
  CLFN=CFNPL
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NPPPSH
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT PRE-PROCESSED INPUT DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 6) THEN
  CLFN=CFNFGI
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NULASE
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' BACKGROUND LOW RESOLUTION DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 7) THEN
  CLFN=CFNANI
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NULASS
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' ANALYSIS LOW RESOLUTION DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 8) THEN
  CLFN=CFANS
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NTIDE
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' SURFACE ANALYSIS DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 9) THEN
  ILEN=INDEX(CEFNLSH,' ')-1
  CLFN=CEFNLSH(1:ILEN)//CETSTAMP
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NINISH
  WRITE(UNIT=NULOUT,FMT='(A)')' LATERAL BOUNDARY DATA TO BE READ FROM FILE  '
ELSEIF (KFILE == 10) THEN
  CLFN='Const.Clim'
  CLLEC='CADRE CLIMAT    '
  ITEST=-1
  KUNTIN = NULCL1
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT CLIMATOLOGIC INPUT DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 11) THEN
  IINC=NINT(REAL(NSTEP,JPRB)*TSTEP/3600._JPRB)
  WRITE(CLINC,'(I4.4)') IINC
  CLFN=CFNRF(1:5)//CNMEXP(1:4)//'+'//CLINC
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NFGISH
  WRITE(UNIT=NULOUT,FMT='(A)')' AVARC REF DATA TO BE READ FROM FILE  '
ELSEIF(KFILE == 12) THEN
  ILEN=INDEX(CEFNLSH,' ')-1
  CLFN=CEFNLSH(1:ILEN)//'999'
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NINISH
  WRITE(UNIT=NULOUT,FMT='(A)') &
   & ' INCREMENTAL DFI LATERAL BOUNDARY DATA TO BE READ FROM FILE  '
ELSEIF(KFILE == 17) THEN
  CLFN='ICMSH'//CNMEXP(1:4)//'CLIM'
  CLLEC='CADRE LECTURE   '
  ITEST=-1
  KUNTIN = NULCL1
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT CLIMATOLOGIC INPUT DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 18) THEN
  CLFN='ICMSH'//CNMEXP(1:4)//'CLI2'
  CLLEC='CADRE LECTURE   '
  ITEST=-1
  KUNTIN = NULCL2
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT CLIMATOLOGIC INPUT DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 19) THEN
  CLFN='ICMSH'//CNMEXP(1:4)//'CLIM'
  CLLEC='CADRE LECTURE   '
  ITEST=-1
  KUNTIN = NULCL1
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT CLIMATOLOGIC INPUT DATA TO BE READ FROM FILE  '  
ELSEIF(KFILE == 30) THEN
  CLFN='ICMFL'//CNMEXP(1:4)//'INIT'
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NULUSR1
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT CUMULATED INPUT FLUXES TO BE READ FROM FILE  '  
ELSEIF(KFILE == 40) THEN
  CLFN='ICMXF'//CNMEXP(1:4)//'INIT'
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NULUSR2
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GRIDPOINT INSTANTANEOUS INPUT FLUXES TO BE READ FROM FILE '  
ELSEIF(KFILE == 50) THEN
  CLFN=CFNTRAJHR
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NULTRAJHR
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' HIGH RESOLUTION TRAJECTORY TO BE READ FROM FILE '  
ELSEIF(KFILE == 51) THEN
  CLFN=CFNTRAJBG
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NULTRAJBG
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' BACKGROUND TO BE READ FROM FILE '
ELSEIF(KFILE == -11) THEN
  CLFN='ICMJK'//CNMEXP(1:4)//'INIT'
  CLLEC='CADRE LECTURE   '
  ITEST=0
  KUNTIN=NEGASH
  WRITE(UNIT=NULOUT,FMT='(A)')&
   & ' GLOBAL ANALYSIS FOR LAM Jk TO BE READ FROM FILE '
ELSE
  WRITE(NULOUT,'('' INVALID ARGUMENT : KFILE ='',I3)') KFILE
  CALL ABOR1('OPENFA : ABOR1 CALLED')
ENDIF

!     ------------------------------------------------------------------

!*    2.    OPEN AND CHECK FILE.
!           --------------------

!*    2.1  CHECK FILE EXISTENCE
 
INQUIRE(FILE=CLFN,EXIST=LLEXIST)
IF (.NOT.LLEXIST) THEN
  IF (KFILE == 4 .OR. KFILE == 17 .OR. KFILE == 18) THEN
    KUNTIN=-999
    IF (LHOOK) CALL DR_HOOK('OPENFA',1,ZHOOK_HANDLE)
    RETURN
  ENDIF
ENDIF

!*    2.2   OPEN FILE.

INBARI=0
CLSTTU='OLD'
CALL FAITOU(IREP,KUNTIN,.TRUE.,CLFN,CLSTTU,.TRUE.,.TRUE.,1,1,INBARI,CLLEC)
CALL LFIMST(IREP,KUNTIN,.FALSE.)
IF (LRFILAF) THEN
  CALL LFILAF(IREP,KUNTIN,.FALSE.)
ENDIF

!*    2.3   READ AND TEST DATE AND IDENTIFICATOR

CALL FADIES(IREP,KUNTIN,KDATE)
CALL FALSIF(IREP,KUNTIN,CLIDEN)

IMOI=NMM(NINDAT)
IAN=NCCAA(NINDAT)
IJOUR=NDD(NINDAT)

IF (KFILE == 10) THEN
! Tests for climatology files of post-processing
  WRITE(NULOUT,*)' YOU READ CLIMATOLOGY FOR THE MONTH : ',KDATE(2)
  IF (LFPMOIS) THEN
    IUNTIN=NINISH
    CLLEC2='CADRE LECTURE   '
    INBARI=0
    CALL FAITOU(IREP,IUNTIN,.TRUE.,CFNISH,'OLD',.TRUE.,&
     & .TRUE.,1,1,INBARI,CLLEC2)  
    CALL LFIMST(IREP,IUNTIN,.FALSE.)
    CALL FADIES(IREP,IUNTIN,IDATES)
    CALL FAIRME(IREP,IUNTIN,'UNKNOWN')
    WRITE(NULOUT,*) ' MONTH OF THE FILE : ',IDATES(2)
    IF (KDATE(2) /= IDATES(2)) THEN
      WRITE(NULOUT,*)'SOURCE CLIM. AND INITIAL FILE DISAGREE ON MONTH'
      CALL ABOR1('OPENFA : ABOR1 CALLED')
    ENDIF
  ELSE
    WRITE(NULOUT,*) ' MONTH OF THE MODEL : ',IMOI
    IF (KDATE(2) /= IMOI) THEN
      WRITE(NULOUT,*)'SOURCE CLIMATOLOGY AND MODEL DISAGREE ON MONTH'
      CALL ABOR1('OPENFA : ABOR1 CALLED')
    ENDIF
  ENDIF
ELSEIF (KFILE == 19) THEN
! Minimum tests for climatology files
  IF (KDATE(2) /= IMOI) THEN
    WRITE(NULOUT,*)' DIFFERENCE OF MONTH BETWEEN FILE AND MODEL'
    CALL ABOR1('OPENFA: ABOR1 CALLED')
  ENDIF
ELSEIF (KFILE == 17.OR.KFILE == 18) THEN
! "Canari" => no tests for climatology files here
  WRITE(NULOUT,*)' YOU READ FIELDS FOR THE MONTH : ',KDATE(2)
  WRITE(NULOUT,*) ' MONTH OF THE MODEL : ',IMOI
ELSE
! Normal cases
  IF (CLIDEN(1:3) == 'CLI') THEN
!   This is for deadstart
    WRITE(NULOUT,*)' YOU READ DEADSTART FIELDS FOR THE MONTH : ',KDATE(2)
    WRITE(NULOUT,*) ' MONTH OF THE MODEL : ',IMOI
    IF (KDATE(2) /= IMOI) THEN
      WRITE(NULOUT,*)' DIFFERENCE OF MONTH BETWEEN FILE AND MODEL'
      CALL ABOR1('OPENFA : ABOR1 CALLED')
    ENDIF
  ELSE
!   Normal start
    WRITE(NULOUT,*) 'YOU READ FIELDS OF THE : ',KDATE(3),' ',&
     & KDATE(2),' ',KDATE(1)  
    WRITE(NULOUT,*) ' HOUR AND MINUTE : ',KDATE(4),'  ',KDATE(5)
    IF (KDATE(6) == 1) THEN
      WRITE(NULOUT,*) ' FORECAST RANGE IN HOURS : ',KDATE(7)
    ELSEIF (KDATE(6) == 2) THEN
      WRITE(NULOUT,*) ' FORECAST RANGE IN DAYS : ',KDATE(7)
      KDATE(7)=KDATE(7)*24
      WRITE(NULOUT,*) ' EQUIVALENCE IN DAYS : ',KDATE(7)
    ELSE
      WRITE(NULOUT,*) ' ERROR IN KDATE(6) : ',KDATE(6)
      IF (KFILE == 50) THEN
        ! ky: no ABOR1 for the time being but some more severe ABOR1
        !     can be coded in the future when optimising this case.
        WRITE(NULOUT,*)'OPENFA : HIGH RESOLUTION TRAJECTORY FILES'
        WRITE(NULOUT,*)'         NO ABORT FOR THE TIME BEING'
        WRITE(NULOUT,*)'         EXECUTION CONTINUING'
      ELSEIF (KFILE == 51) THEN
        ! ky: no ABOR1 for the time being but some more severe ABOR1
        !     can be coded in the future when optimising this case.
        WRITE(NULOUT,*)'OPENFA : BACKGROUND FILES'
        WRITE(NULOUT,*)'         NO ABORT FOR THE TIME BEING'
        WRITE(NULOUT,*)'         EXECUTION CONTINUING'
      ELSE
        CALL ABOR1('OPENFA : ABOR1 CALLED')
      ENDIF
    ENDIF
!   Test time range
    WRITE(NULOUT,*) ' DATE OF THE MODEL : ',IJOUR,' ',IMOI,' ',IAN
    WRITE(NULOUT,*) ' SECONDS IN THE DAY :',NSSSSS
    WRITE(NULOUT,*) ' IN HOUR         :',NCTH(NSSSSS)
    ISTEP=MAX(NSTEP,0)
    IF (ISTEP > 0) THEN
      WRITE(NULOUT,*) ' + ',ISTEP, ' STEPS'
    ENDIF
    ISSS=KDATE(7)*3600+KDATE(4)*3600+KDATE(5)*60
    ZFTIME=RTIME(KDATE(1),KDATE(2),KDATE(3),ISSS,RDAY)
    IF (KFILE == 9 .AND. .NOT.LBIAS) THEN
      ZMTIME=RTIME(IAN,IMOI,IJOUR,NSSSSS,RDAY)+ISTEP*TSTEP+TEFRCL
    ELSEIF(KFILE == 11) THEN
      ZMTIME=RTIME(IAN,IMOI,IJOUR,NSSSSS,RDAY)+ISTEP*TSTEP
    ELSE
      ZMTIME=RTIME(IAN,IMOI,IJOUR,NSSSSS,RDAY)
    ENDIF 
    IF (KFILE == 9 .AND. NSTEP <= 0) THEN
      ZMTIME1=ZMTIME-TEFRCL
    ELSE
      ZMTIME1=ZMTIME
    ENDIF
    IF ( (ZMTIME > ZFTIME+1800.0_JPRB.OR.ZMTIME <= ZFTIME-1800.0_JPRB) &
       & .AND.(ZMTIME1 > ZFTIME+1800.0_JPRB.OR.ZMTIME1 <= ZFTIME-1800.0_JPRB)) THEN  
      WRITE(NULOUT,*) ' DATE INCOHERENCE BETWEEN FILE AND MODEL'
      IF (L3DFGAT) THEN
        WRITE(NULOUT,*)'OPENFA : L3DFGAT : PAS D ABORT'
      ELSE
        IF (KFILE == 4) THEN
          WRITE(NULOUT,*) ' *** ABORT BLENDING ***'
          CALL FAIRME(IREP,KUNTIN,'UNKNOWN')
          KUNTIN=-999
          IF (LHOOK) CALL DR_HOOK('OPENFA',1,ZHOOK_HANDLE)
          RETURN
        ELSEIF (KFILE == 9) THEN
          IF (LMAP) THEN
            WRITE(NULOUT,*) 'DIFFER. SHOULD BE 1 LBC RANGE!'
            CALL ABOR1('OPENFA : ABOR1 CALLED')
          ELSE
            WRITE(NULOUT,*) ' *** LMAP=.FALSE. => NO ABORT ***'
          ENDIF
        ELSEIF (KFILE == 50) THEN
          ! ky: no ABOR1 for the time being but some more severe ABOR1
          !     can be coded in the future when optimising this case.
          WRITE(NULOUT,*)'OPENFA : HIGH RESOLUTION TRAJECTORY FILES'
          WRITE(NULOUT,*)'         NO ABORT FOR THE TIME BEING'
          WRITE(NULOUT,*)'         EXECUTION CONTINUING'
        ELSEIF (KFILE == 51) THEN
          ! ky: no ABOR1 for the time being but some more severe ABOR1
          !     can be coded in the future when optimising this case.
          WRITE(NULOUT,*)'OPENFA : BACKGROUND FILES'
          WRITE(NULOUT,*)'         NO ABORT FOR THE TIME BEING'
          WRITE(NULOUT,*)'         EXECUTION CONTINUING'
        ELSE
          IF(NCONF==801) THEN
            WRITE(NULOUT,*)' BEWARE: CONF 801 BYPASSES THIS CONFLICT '
          ELSE
            CALL ABOR1('OPENFA : ABOR1 CALLED')
          ENDIF
        ENDIF
      ENDIF
    ENDIF
  ENDIF
ENDIF

!*    2.4   READ AND TEST FRAME

ZEPS=EPSILON(1.0_JPRB)*10000.0_JPRB
IF (.NOT.LELAM) THEN
  CALL CHIEN(CLLEC,NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,NDGLG,&
   & NDLON,NLOENG,NMENG,ZGEMU,NHTYP,NFLEVG,VP00,VALH,&
   & VBH,NQUAD,ITEST,NDGSAG,NDGENG,ZEPS,LLFICP,&
   & NULOUT)  
  IF (LLFICP) THEN
    WRITE(NULOUT,FMT='(A)') ' FILE CONTAINS POLES '
  ELSE
    WRITE(NULOUT,FMT='(A)') ' FILE DOES NOT CONTAIN POLES '
  ENDIF
ELSE
  CALL CCHIEN (CLLEC,KUNTIN,ITEST)
  LLFICP=.FALSE.
ENDIF

KDEB2=1
KFIN=NDGLG

IF (LLFICP) THEN
  KINDEX=NLOENG(1)
ELSE
  KINDEX=0
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('OPENFA',1,ZHOOK_HANDLE)
END SUBROUTINE OPENFA
