SUBROUTINE PKGRIDA (KFIELDG,YD_PGPBUF,KFLDPTP)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NSCRTCH  ,NULOUT
USE YOMOP    , ONLY : CNMCA
USE YOMCT0   , ONLY : NPROC    ,LOUTPUT
USE YOMMP    , ONLY : MYPROC
USE YOMDIM   , ONLY :       NPROMA
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG
USE GRIDPOINT_BUFFERS_MIX , ONLY : gridpoint_buffer

!**** *PKGRIDA*  - PACK GRIDPOINT DATA THROUGH ARPEGE FILE

!     PURPOSE.
!     --------
!        To pack model gridpoint fields by writing to, then reading from a
!         file ARPEGE
!        In SM version : loop on bunches of fields to write & read back, 
!        to limit the cost of memory/extraction.  
!        In DM version : Fields are distributed between the processors ; 
!        Each processor performs its own loop on fields to write & read back
!        on a local file.

!**   INTERFACE.
!     ----------
!       *CALL* *PKGRIDA*

!        EXPLICIT ARGUMENTS
!        --------------------
!        KFIELDG   - number of fields to pack
!        PGPBUF    - memory buffer (used in case of no real IO)
!        KFLDPTP   - field pointers for extraction (initialized up to KFIELDG)

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 95-07-18
!        M.Hamrud : 95-10-20 Interface block for EXTGPF
!        K. YESSAD (APRIL 1997): parallelisation of FULL-POS.
!        Modified by K. YESSAD (OCT 1997): residual bugs corrections in DM code
!        Modified : 98-02-11 R. El Khatib Restructuration & actual DM version
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - ZREALG does not contain polar values any longer, and non polar
!            values are stored from its element 1.
!          - removal of the last argument (.FALSE.) in 'CALL DIWRGRID'.
!        Modified : 98-12-31 R. El Khatib Additive arguments to NEWFA
!        Modified : 99-07-21 R. El Khatib less arguments to NEWFA
!    R. El Khatib : 99-12-28 remove poles rows
!    R. El Khatib : 00-11-03 Optional communications in extgpf
!    R. El Khatib : 01-03-16 No I/O in packing
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!     Apr 2008  K. Yessad: use DISGRID instead of DISGRID_C + cleanings
!     ------------------------------------------------------------------

IMPLICIT NONE

TYPE(GRIDPOINT_BUFFER),INTENT(INOUT) :: YD_PGPBUF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDPTP(YD_PGPBUF%IFIELDS) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM), PARAMETER :: JPFAOVRSZ=2

!     ------------------------------------------------------------------

REAL(KIND=JPRB),ALLOCATABLE :: ZREALG(:,:),ZREALL(:,:), ZGPP(:,:)

REAL(KIND=JPRB) :: ZVALCO(NGPTOTG+JPFAOVRSZ)

INTEGER(KIND=JPIM) :: IASPIO(KFIELDG)

INTEGER(KIND=JPIM) :: IEND, IFIELDS, IREP, IST
INTEGER(KIND=JPIM) :: JF, JKGLO, JROF,INBARI, INOMA, ILONGA

CHARACTER(LEN=16) :: CLSCRTCH, CLNOMA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     IASPIO(j): Processor number which packs fields #j
!     IFIELDS  : Number of fields packed by MYPROC
!     ZREALG   : global array of fields
!     ZREALL   : local  array of fields
!     ZGPP     : local gridpoint buffer

!     ------------------------------------------------------------------

#include "disgrid.intfb.h"
#include "diwrgrid.intfb.h"
#include "incgpf.intfb.h"
#include "sc2rdg.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PKGRIDA',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.1 DISTRIBUTE FIELDS
!            -----------------

DO JF=1,KFIELDG
  IASPIO(JF)=MOD(JF-1,NPROC)+1
ENDDO
IFIELDS=0
DO JF=1,KFIELDG
  IF (IASPIO(JF) == MYPROC) THEN
    IFIELDS=IFIELDS+1
  ENDIF
ENDDO
IF (LOUTPUT) THEN
  WRITE(NULOUT,*) 'PKGRIDA : IFIELDS = ',IFIELDS, ' IASPIO = '
  WRITE(NULOUT,'(15(1X,I4))') IASPIO
ENDIF

!*       1.2 OPEN A SCRATCH FILE ARPEGE
!            --------------------------

IF (IFIELDS > 0) THEN
  CALL FANOUV(IREP,NSCRTCH,.FALSE.,CLSCRTCH,'NEW',.TRUE.,.TRUE.,1,1,INBARI,&
   & CNMCA)  
ENDIF

!*       1.3 PACK DATA THROUGH WRITE AND READ
!            --------------------------------

ALLOCATE(ZGPP(NPROMA,YD_PGPBUF%IFIELDS))
ALLOCATE(ZREALL(NGPTOT,KFIELDG))
ALLOCATE(ZREALG(NGPTOTG,1))

!*       1.3.1 Extract all fields to pack and send those I do not pack

IST=1
DO JKGLO=1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  CALL SC2RDG(YD_PGPBUF,JKGLO,IST,IEND,ZGPP)
  DO JF=1,KFIELDG
    DO JROF=IST,IEND
      ZREALL(JROF+JKGLO-1,JF)=ZGPP(JROF,KFLDPTP(JF))
    ENDDO
  ENDDO
ENDDO

DO JF=1,KFIELDG
  IF (IASPIO(JF) /= MYPROC) THEN
    CALL DIWRGRID(IASPIO(JF),JF,0,NGPTOT,ZREALL(1,JF))
  ENDIF
ENDDO

!*       1.3.2 Receive, pack and send back the fields I pack

DO JF=1,KFIELDG
  IF (IASPIO(JF) == MYPROC) THEN
    ZREALG(1:NGPTOT,1)=ZREALL(1:NGPTOT,JF)
    CALL DIWRGRID(IASPIO(JF),JF,1,NGPTOTG,ZREALG)
    CALL FACOND(IREP,NSCRTCH,'S',0,'FIELD',ZREALG(1,1),.FALSE.,CLNOMA,INOMA,&
     & ZVALCO,ILONGA)  
    CALL FADECO(IREP,NSCRTCH,'S',0,'FIELD',.FALSE.,CLNOMA,INOMA,ZVALCO,&
     & ILONGA,ZREALG(1,1))  
    CALL DISGRID(JF,0,NGPTOTG,ZREALG(1,1),LDC=.TRUE.,KIOPROC=IASPIO(JF))
    ZREALL(1:NGPTOT,JF)=ZREALG(1:NGPTOT,1)
  ENDIF
ENDDO

!*       1.3.3 Receive fields I have not packed and include back all fields

DO JF=1,KFIELDG
  IF (IASPIO(JF) /= MYPROC) THEN
    CALL DISGRID(JF,1,NGPTOT,ZREALL(1,JF),LDC=.TRUE.,KIOPROC=IASPIO(JF))
  ENDIF
ENDDO

CALL INCGPF(KFIELDG,YD_PGPBUF,KFLDPTP,ZREALL)

DEALLOCATE(ZGPP)
DEALLOCATE(ZREALL)
DEALLOCATE(ZREALG)

!*       1.4 CLOSE AND DELETE THE SCRATCH FILE
!            ---------------------------------

IF (IFIELDS > 0) THEN
  CALL FAIRNO(IREP,NSCRTCH,'DELETE')
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PKGRIDA',1,ZHOOK_HANDLE)
END SUBROUTINE PKGRIDA
