SUBROUTINE PRT_CTLVEC_MAX(YDCV,CDGREP)

!     Purpose.
!     --------
!        Print max components of control vector in control space,
!        useful in particular to find where max of eigenvectors
!        are located to diagnose cases of bad convergence.

!     Author.
!     -------
!        Y.Tremolet

!     Modifications.
!     --------------
!        Original : 02-Nov-2004
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCVA  , ONLY: NGRBVAR, NVA3D, NVA2D
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG
USE YOMLUN  , ONLY: NULOUT, NULERR
USE YOMWAVELET, ONLY: N_WAVELET_SCALES, GRID_DEFINITION
USE YOMMODEL_ERROR, ONLY: NGRBMERRCTL, NVE3D
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD
USE MPL_MODULE , ONLY: MPL_ALLREDUCE

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN) :: YDCV 
CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: CDGREP

TYPE(SPECTRAL_FIELD) :: YLSP
INTEGER(KIND=JPIM) :: jj,ii,il,iloc1(1),iloc2(2),iloc3(3),ilev,ifld,icmp, &
                    & jlev,jfld,iend,igptot, &
                    & JKGLO, ICEND, IBL, ISTART, IBUF, JSCALE, INFIELDS
REAL(KIND=JPRB) :: zpar,ztov,zmax(6),zzz
REAL(KIND=JPRB), ALLOCATABLE :: zini2(:),zini3(:,:),zerr2(:,:),zerr3(:,:,:),zbuf(:)
CHARACTER(LEN=24) :: clmax
INTEGER(KIND=JPIM) :: inumax=5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PRT_CTLVEC_MAX',0,ZHOOK_HANDLE)

! Initial condition
! -----------------
IF (YDCV%linitcv) THEN
  IF (YDCV%lwaveletjb) THEN
    INFIELDS=NVA3D*NFLEVG+NVA2D

    ALLOCATE(ZBUF(INFIELDS))
    ZBUF(:)=0.0_JPRB

    DO JSCALE=1,N_WAVELET_SCALES+1
      IGPTOT=GRID_DEFINITION(JSCALE)%NGPTOT
      DO JFLD=1,INFIELDS
        DO JKGLO=1,IGPTOT,NPROMA
          ICEND=MIN(NPROMA,IGPTOT-JKGLO+1)
          IBL=(JKGLO-1)/NPROMA+1
      
          ISTART = YDCV%ibgn_scale(JSCALE) + (JFLD-1)*NPROMA + (IBL-1)*NPROMA*INFIELDS
          IEND  =ISTART-1+ICEND
      
          zzz = MAXVAL(ABS(YDCV%initcv(ISTART:IEND)))
          ZBUF(JFLD)=MAX(ZBUF(JFLD),zzz)
        ENDDO
      ENDDO
    ENDDO

    CALL MPL_ALLREDUCE (PSENDBUF=ZBUF,CDOPER='MAX',CDSTRING='PRT_CTLVEC_MAX')

    ALLOCATE(zini2(NVA2D))
    ALLOCATE(zini3(NFLEVG,NVA3D))

    IBUF=1
    DO JFLD=1,NVA3D
      DO JLEV=1,NFLEVG
        ZINI3(JLEV,JFLD) = ZBUF(IBUF)
        IBUF=IBUF+1
      ENDDO
    ENDDO
    DO JFLD=1,NVA2D
      ZINI2(JFLD) = ZBUF(IBUF)
      IBUF=IBUF+1
    ENDDO

    DEALLOCATE (ZBUF)
  ELSE
    CALL ALLOCATE_SPEC(YLSP,YDCV%nsmax,YDCV%nmsmax, &
                     & YDCV%ns3d,YDCV%ns2d,YDCV%ns1d,NGRBVAR)
    YLSP=YDCV%initcv
    ALLOCATE(zini2(ylsp%ns2g))
    ALLOCATE(zini3(ylsp%nflevg,ylsp%ns3d))
    CALL SPECTRAL_MAX_LEVS(YLSP,zini3(:,:),zini2(:))
    CALL DEALLOCATE_SPEC(YLSP)
  ENDIF
ELSE
  ALLOCATE(zini2(1))
  ALLOCATE(zini3(1,1))
  zini2(1)=0.0_JPRB
  zini3(1,1)=0.0_JPRB
ENDIF

! Model error
! -----------
IF (YDCV%nmoderr>0) THEN
  CALL ALLOCATE_SPEC(YLSP,YDCV%nserr,0,YDCV%nve3d,YDCV%nve2d,0,NGRBMERRCTL)
  ALLOCATE(zerr2(ylsp%ns2g,YDCV%nmoderr))
  ALLOCATE(zerr3(ylsp%nflevg,ylsp%ns3d,YDCV%nmoderr))
  ii=0
  il=YLSP%nspsizel
  DO jj=1,YDCV%nmoderr
    YLSP=YDCV%MODERR(ii+1:ii+il)
    ii=ii+il
    CALL SPECTRAL_MAX_LEVS(YLSP,zerr3(:,:,jj),zerr2(:,jj))
  ENDDO
  CALL DEALLOCATE_SPEC(YLSP)
ELSE
  ALLOCATE(zerr2(1,1))
  ALLOCATE(zerr3(1,1,1))
  zerr2(1,1)=0.0_JPRB
  zerr3(1,1,1)=0.0_JPRB
ENDIF

! Parameter estimation
! --------------------
zpar=0.0_JPRB
IF (YDCV%NPARA>0) THEN
  zpar=MAXVAL(ABS(YDCV%PARAMS(:)))
ENDIF

! TOVS
! ----
ztov=0.0_JPRB
IF (YDCV%ltovscv) THEN
  ztov=MAXVAL(ABS(YDCV%TOVSCV(:)))
  CALL MPL_ALLREDUCE(PSENDBUF=ztov,CDOPER='MAX',CDSTRING='PRT_CTLVEC_MAX')
ENDIF

! Look for max and print diagnostics
! ----------------------------------
IF (PRESENT(CDGREP)) WRITE(NULOUT,*)'PRT_CTLVEC_MAX: Max ',CDGREP

DO jj=1,inumax
  zmax(:)=0.0_JPRB
  IF (YDCV%linitcv) THEN
    zmax(1)=MAXVAL(zini2(:))
    zmax(2)=MAXVAL(zini3(:,:))
  ENDIF
  IF (YDCV%nmoderr>0) THEN
    zmax(3)=MAXVAL(zerr2(:,:))
    zmax(4)=MAXVAL(zerr3(:,:,:))
  ENDIF
  zmax(5)=zpar
  zmax(6)=ztov

  iloc1=MAXLOC(zmax)
  ii=iloc1(1)
  IF (ii>6 .OR. ii<1) THEN
    WRITE(NULERR,*)'PRT_CTLVEC_MAX: Max not found',ii,zmax
    CALL ABOR1('PRT_CTLVEC_MAX: Max not found')
  ENDIF

  WRITE(clmax,'(A8,I3,A2,ES10.4,X)')'Maximum ',jj,'= ',zmax(ii)
  SELECT CASE (ii)
  CASE(1)
    iloc1=MAXLOC(zini2(:))
    ifld=NGRBVAR(NVA3D+iloc1(1))
    WRITE(NULOUT,*)clmax,'in initial condition ',&
                 & 'surface field',ifld
    zini2(iloc1(1))=0.0_JPRB
  CASE(2)
    iloc2=MAXLOC(zini3(:,:))
    ilev=iloc2(1)
    ifld=NGRBVAR(iloc2(2))
    WRITE(NULOUT,*)clmax,'in initial condition ',&
                 & '3D field',ifld,'level=',ilev
    zini3(iloc2(1),iloc2(2))=0.0_JPRB
  CASE(3)
    iloc2=MAXLOC(zerr2(:,:))
    ifld=NGRBMERRCTL(NVE3D+iloc2(1))
    icmp=iloc2(2)
    WRITE(NULOUT,*)clmax,'in model error ',&
                 & 'surface field',ifld,' component=',icmp
    zerr2(iloc2(1),iloc2(2))=0.0_JPRB
  CASE(4)
    iloc3=MAXLOC(zerr3(:,:,:))
    ilev=iloc3(1)
    ifld=NGRBMERRCTL(iloc3(2))
    icmp=iloc3(3)
    WRITE(NULOUT,*)clmax,'in model error ',&
                 & '3D field',ifld,' level=',ilev,' component=',icmp
    zerr3(iloc3(1),iloc3(2),iloc3(3))=0.0_JPRB
  CASE(5)
    WRITE(NULOUT,*)clmax,'in parameter section.'
    zpar=0.0_JPRB
  CASE(6)
    WRITE(NULOUT,*)clmax,'in TOVS control variable.'
    ztov=0.0_JPRB
  END SELECT
ENDDO

! Finish
! ------
IF (ALLOCATED(zini2)) DEALLOCATE(zini2)
IF (ALLOCATED(zini3)) DEALLOCATE(zini3)
IF (ALLOCATED(zerr2)) DEALLOCATE(zerr2)
IF (ALLOCATED(zerr3)) DEALLOCATE(zerr3)

IF (LHOOK) CALL DR_HOOK('PRT_CTLVEC_MAX',1,ZHOOK_HANDLE)
END SUBROUTINE PRT_CTLVEC_MAX
