SUBROUTINE PRT_CTLVEC_NORMS(YDCV,CDGREP)

!     Purpose.
!     --------
!        Print norms of components of control vector in control space.

!     Author.
!     -------
!        Y.Tremolet

!     Modifications.
!     --------------
!        Original : 23-Jul-2004
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCVA  , ONLY: NGRBVAR
USE YOMLUN  , ONLY: NULOUT
USE YOMMODEL_ERROR, ONLY: NGRBMERRCTL
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN) :: YDCV 
CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: CDGREP

TYPE(SPECTRAL_FIELD) :: YLSP
INTEGER(KIND=JPIM) :: jj,ii,il
REAL(KIND=JPRB) :: zz
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PRT_CTLVEC_NORMS',0,ZHOOK_HANDLE)

IF (PRESENT(CDGREP)) WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: Norms ',CDGREP

! Initial condition
! -----------------
IF (YDCV%linitcv) THEN
  IF (YDCV%lwaveletjb) THEN
    WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: No norm yet for wavelet control vector'
  ELSE
    CALL ALLOCATE_SPEC(YLSP,YDCV%nsmax,YDCV%nmsmax, &
                     & YDCV%ns3d,YDCV%ns2d,YDCV%ns1d,NGRBVAR)
    YLSP=YDCV%initcv
    WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: Initial Condition in Control Space'
    CALL SPECTRAL_NORM_LEVS(YLSP,'Init')
    CALL DEALLOCATE_SPEC(YLSP)
  ENDIF
ENDIF

! Model error
! -----------
IF (YDCV%nmoderr>0) THEN
  CALL ALLOCATE_SPEC(YLSP,YDCV%nserr,0,YDCV%nve3d,YDCV%nve2d,0,NGRBMERRCTL)
  ii=0
  il=YLSP%nspsizel
  DO jj=1,YDCV%nmoderr
    YLSP=YDCV%MODERR(ii+1:ii+il)
    ii=ii+il
    WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: Model Error in Control Space',jj
    CALL SPECTRAL_NORM_LEVS(YLSP,'MErr')
  ENDDO
  CALL DEALLOCATE_SPEC(YLSP)
ENDIF

! Parameter estimation
! --------------------
IF (YDCV%NPARA>0) THEN
  zz=0.0_JPRB
  DO ii=1,YDCV%NPARA
    zz=zz+YDCV%PARAMS(ii)*YDCV%PARAMS(ii)
  ENDDO
  zz=SQRT(zz)
  WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: Norm Parameters in Control Space',zz
ENDIF

! TOVS
! ----
IF (YDCV%ltovscv) THEN
  WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: No norm yet for TOVS control variable'
ENDIF

! TS
! --
IF (YDCV%ltscv) THEN
  WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: No norm yet for TS control variable'
ENDIF

! LAM MEAN WIND
! -------------
IF (YDCV%lam1d) THEN
  WRITE(NULOUT,*)'PRT_CTLVEC_NORMS: No norm yet for LAM1D control variable'
ENDIF

IF (LHOOK) CALL DR_HOOK('PRT_CTLVEC_NORMS',1,ZHOOK_HANDLE)
END SUBROUTINE PRT_CTLVEC_NORMS
