SUBROUTINE PRTGOM(KSETS,KSETE)

!****  PRTGOM  - PRINT RMS OF GOM-ARRAYS

!     E. ANDERSSON            07/02/96       ECMWF

!     PURPOSE
!     -------

!        DIAGNOSTIC PRINT OF RMS OF GOM-ARRAYS
!     MODEL VALUES INTERPOLATED TO OBSERVATION POINTS
!     OR (AFTER THE ADJOINT) GRADIENTS

!**   INTERFACE
!     ---------

!        CALL PREINT

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------
!        NONE

!**   MODIFICATIONS
!     -------------
!     Erik Andersson 96:07:02 Reproducible on Fujitsu
!     Mats Hamrud    96:07:19 Message passing
!     Erik Andersson 97:11:14 GOM-variables more flexible
!     Erik Andersson 98:08:18 Print CLW
!     A. Dethof      15-May-05 Add GRG
!     R.Engelen      07-Jul-05 Add GHG
!     Angela Benedetti 15-Aug-05 Add AERO variables
!     Niels Bormann  05:12:08 Exclude limb data
!     C. Payan       07:04:03 I5 to I9 avoid some **** and sometimes abort?!
!     H. Hersbach    12-Feb-10 Add Neutral wind for SCAT
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCT0   , ONLY : NPROC
USE YOMOBSET , ONLY : MTYPOB   ,MLNSET
USE YOMCOCTP , ONLY : NSYNOP   ,NDRIBU   ,NSATEM   ,NPAOB    ,NSCATT, &
                    & NLIMB
USE YOMVAR   , ONLY : NSIM4D, LFCOBS
USE YOMLUN   , ONLY : NULOUT    ,NULERR
USE YOMECTAB , ONLY : NOBSPROFS 
USE YOMDB
USE GOMS_MIX     , ONLY : LGOMUA, YGOMV, YGOMUA, YGOMS
USE YOM_YGFL , ONLY : JPGHG, NGHG,YGHG_NL, JPAERO, NAERO, JPGRG, NGRG, YGRG_NL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSETS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSETE 
REAL(KIND=JPRB), DIMENSION(NFLEVG) ::&
 & ZSURU,ZSURV,ZSURT,ZSURQ,ZSURO3,ZSURPS,ZSURCLW &
 & ,ZCONU,ZCONV,ZCONT,ZCONQ,ZCONO3,ZCONPS,ZCONCLW &
 & ,ZCONSW,ZCONRW,ZCONGW &
 & ,ZTOVU,ZTOVV,ZTOVT,ZTOVQ,ZTOVO3,ZTOVPS,ZTOVCLW  
REAL(KIND=JPRB), DIMENSION(NFLEVG,NAERO)::&
 & ZSURAERO,ZCONAERO,ZTOVAERO
REAL(KIND=JPRB), DIMENSION(NFLEVG,JPGHG) ::&
 & ZSURGHG,ZCONGHG,ZTOVGHG
REAL(KIND=JPRB), DIMENSION(NFLEVG,JPGRG) ::&
 & ZSURGRG,ZCONGRG,ZTOVGRG
REAL(KIND=JPRB) :: ZSCAU,ZSCAV,ZSCAUN,ZSCAVN,ZSCAT,ZSCAQ,ZSCAPS

INTEGER(KIND=JPIM) :: ISUR,ICON,ITOV,ISCA,ITYPOB,IPOSM,IL
INTEGER(KIND=JPIM) :: JSET,JOBS,JLEV,JL,ISET,ILEN,INFO(1)
INTEGER(KIND=JPIM) :: IAERO,JAERO,JLEVAERO
INTEGER(KIND=JPIM) :: IGHG,JGHG,JLEVGHG
INTEGER(KIND=JPIM) :: IGRG,JGRG,JLEVGRG
REAL(KIND=JPRB) :: ZEROO3,ZEROCLW,ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "gathergom.intfb.h"

#include "openmp_obs.h"

!     ------------------------------------------------------------------
!*
!*
!        1.       INITIALIZE
!                 ----------

!*          1.1   INITIALIZE ARRAYS AND COUNTERS

IF (LHOOK) CALL DR_HOOK('PRTGOM',0,ZHOOK_HANDLE)
ZSURU(:)=0.0_JPRB
ZSURV(:)=0.0_JPRB
ZSURT(:)=0.0_JPRB
ZSURQ(:)=0.0_JPRB
ZSURO3(:)=0.0_JPRB
ZSURCLW(:)=0.0_JPRB
ZSURAERO(:,:)=0.0_JPRB
ZSURGHG(:,:)=0.0_JPRB
ZSURGRG(:,:)=0.0_JPRB
ZSURPS(:)=0.0_JPRB

ZCONU(:)=0.0_JPRB
ZCONV(:)=0.0_JPRB
ZCONT(:)=0.0_JPRB
ZCONQ(:)=0.0_JPRB
ZCONO3(:)=0.0_JPRB
ZCONCLW(:)=0.0_JPRB
ZCONAERO(:,:)=0.0_JPRB
ZCONGHG(:,:)=0.0_JPRB
ZCONGRG(:,:)=0.0_JPRB
ZCONSW(:)=0.0_JPRB
ZCONRW(:)=0.0_JPRB
ZCONGW(:)=0.0_JPRB
ZCONPS(:)=0.0_JPRB

ZTOVU(:)=0.0_JPRB
ZTOVV(:)=0.0_JPRB
ZTOVT(:)=0.0_JPRB
ZTOVQ(:)=0.0_JPRB
ZTOVO3(:)=0.0_JPRB
ZTOVCLW(:)=0.0_JPRB
ZTOVAERO(:,:)=0.0_JPRB
ZTOVGHG(:,:)=0.0_JPRB
ZTOVGRG(:,:)=0.0_JPRB
ZTOVPS(:)=0.0_JPRB

ZSCAU=0.0_JPRB
ZSCAV=0.0_JPRB
ZSCAUN=0.0_JPRB
ZSCAVN=0.0_JPRB
ZSCAT=0.0_JPRB
ZSCAQ=0.0_JPRB
ZSCAPS=0.0_JPRB

ISUR=0
ICON=0
ITOV=0
ISCA=0

!*          1.2   LOOP OBSERVATION SETS

SET_LOOP: DO JSET=KSETS,KSETE
  ITYPOB = MOD(MTYPOB(JSET),100)

  ILEN   = MLNSET(JSET)
  ISET = JSET

  CALL GETDB('PRTGOM', 0, ILEN, INFO, 0,ZINFO,0, &
   & ISET, -1, -1, -1, -1, -1)  

  IF (ILEN /= MLNSET(JSET)) THEN
    WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
    WRITE(NULERR,*)'Expecting ',MLNSET(JSET),', but got =',ILEN
    CALL ABORTDB('PRTGOM','INCONSISTENCIES FROM GETDB',ILEN)
  ENDIF

!*          1.3   LOOP OBSERVATIONS WITHIN SETS, SURFACE

  CALL GSTATS(1999,0)
  DO JOBS = 1,MLNSET(JSET)
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

!*          1.3.1 TOVS

    IF(ITYPOB == NSATEM) THEN
      ZTOVPS(NFLEVG) = ZTOVPS(NFLEVG) + YGOMS(IPOSM)%SP**2
      IF(.NOT.LGOMUA(ITYPOB,YGOMV%MU)) &
       & ZTOVU(NFLEVG)  = ZTOVU(NFLEVG)  + YGOMS(IPOSM)%UL**2
      IF(.NOT.LGOMUA(ITYPOB,YGOMV%MV)) &
       & ZTOVV(NFLEVG)  = ZTOVV(NFLEVG)  + YGOMS(IPOSM)%VL**2
      ITOV=ITOV+1

!*          1.3.2 SCATTEROMETER

    ELSEIF(ITYPOB == NSCATT) THEN
      ZSCAPS = ZSCAPS + YGOMS(IPOSM)%SP**2
      ZSCAU  = ZSCAU  + YGOMS(IPOSM)%UL**2
      ZSCAV  = ZSCAV  + YGOMS(IPOSM)%VL**2
      ZSCAUN = ZSCAUN + YGOMS(IPOSM)%U10N**2
      ZSCAVN = ZSCAVN + YGOMS(IPOSM)%V10N**2
      ZSCAT  = ZSCAT  + YGOMS(IPOSM)%TL**2
      ZSCAQ  = ZSCAQ  + YGOMS(IPOSM)%QL**2
      ISCA=ISCA+1

!*          1.3.3 SURFACE DATA

    ELSEIF(ITYPOB == NSYNOP .OR. ITYPOB == NDRIBU .OR.ITYPOB == NPAOB) THEN
      ZSURPS(NFLEVG) = ZSURPS(NFLEVG) + YGOMS(IPOSM)%SP**2
      ISUR=ISUR+1

!*          1.3.4 UPPER AIR DATA

    ELSEIF(NOBSPROFS(ITYPOB) == 1 .AND. ITYPOB /= NLIMB ) THEN
      ZCONPS(NFLEVG) = ZCONPS(NFLEVG) + YGOMS(IPOSM)%SP**2
      ICON=ICON+1
    ENDIF
  ENDDO

!*          1.4   LOOP OBSERVATIONS WITHIN SETS, UPPER AIR

  DO JOBS = 1,MLNSET(JSET)
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)

!*          1.4.1 TOVS

    IF(ITYPOB == NSATEM) THEN
      DO JLEV = 1,NFLEVG
        IF(LGOMUA(ITYPOB,YGOMV%MU)) &
         & ZTOVU(JLEV) = ZTOVU(JLEV) + YGOMUA(IPOSM)%U(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MV)) &
         & ZTOVV(JLEV) = ZTOVV(JLEV) + YGOMUA(IPOSM)%V(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MT)) &
         & ZTOVT(JLEV) = ZTOVT(JLEV) + YGOMUA(IPOSM)%T(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MQ)) &
         & ZTOVQ(JLEV) = ZTOVQ(JLEV) + YGOMUA(IPOSM)%Q(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MO3)) &
         & ZTOVO3(JLEV) = ZTOVO3(JLEV) + YGOMUA(IPOSM)%O3(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MCLW)) &
         & ZTOVCLW(JLEV) = ZTOVCLW(JLEV) + YGOMUA(IPOSM)%CLW(JLEV)**2
        IGHG=0
        DO JGHG=1,JPGHG
          IF(LGOMUA(ITYPOB,YGOMV%MGHG(JGHG))) THEN
            IGHG=IGHG+1
            JLEVGHG = JLEV+(IGHG-1)*NFLEVG
            ZTOVGHG(JLEV,IGHG) = ZTOVGHG(JLEV,IGHG) &
           & + YGOMUA(IPOSM)%GHG(JLEVGHG)**2
          ENDIF
        ENDDO
        IAERO=0
        DO JAERO=1,JPAERO
          IF(LGOMUA(ITYPOB,YGOMV%MAERO(JAERO))) THEN
            IAERO=IAERO+1
            JLEVAERO = JLEV+(IAERO-1)*NFLEVG
            ZTOVAERO(JLEV,IAERO) = ZTOVAERO(JLEV,IAERO) &
           & + YGOMUA(IPOSM)%AERO(JLEVAERO)**2
          ENDIF
        ENDDO
        IGRG=0
        DO JGRG=1,JPGRG
          IF(LGOMUA(ITYPOB,YGOMV%MGRG(JGRG))) THEN
            IGRG=IGRG+1
            JLEVGRG = JLEV+(IGRG-1)*NFLEVG
            ZTOVGRG(JLEV,JGRG) = ZTOVGRG(JLEV,JGRG) &
           & + YGOMUA(IPOSM)%GRG(JLEVGRG)**2
          ENDIF
        ENDDO
      ENDDO

!*          1.4.2 SCATTEROMETER, DO NOTHING

    ELSEIF(ITYPOB == NSCATT) THEN
      CONTINUE

!*          1.4.3 SURFACE DATA, YES THEY HAVE UPPER-AIR COMPONENT

    ELSEIF(ITYPOB == NSYNOP .OR. ITYPOB == NDRIBU .OR.ITYPOB == NPAOB) THEN
      DO JLEV = 1,NFLEVG
        IF(LGOMUA(ITYPOB,YGOMV%MU)) &
         & ZSURU(JLEV) = ZSURU(JLEV) + YGOMUA(IPOSM)%U(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MV)) &
         & ZSURV(JLEV) = ZSURV(JLEV) + YGOMUA(IPOSM)%V(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MT)) &
         & ZSURT(JLEV) = ZSURT(JLEV) + YGOMUA(IPOSM)%T(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MQ)) &
         & ZSURQ(JLEV) = ZSURQ(JLEV) + YGOMUA(IPOSM)%Q(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MO3)) &
         & ZSURO3(JLEV) = ZSURO3(JLEV) + YGOMUA(IPOSM)%O3(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MCLW)) & 
         & ZSURCLW(JLEV) = ZSURCLW(JLEV) + YGOMUA(IPOSM)%CLW(JLEV)**2

        IAERO=0
        DO JAERO=1,JPAERO
          IF(LGOMUA(ITYPOB,YGOMV%MAERO(JAERO))) THEN
            IAERO=IAERO+1
            JLEVAERO = JLEV+(IAERO-1)*NFLEVG
            ZSURAERO(JLEV,IAERO) = ZSURAERO(JLEV,IAERO) &
           & + YGOMUA(IPOSM)%AERO(JLEVAERO)**2
          ENDIF
        ENDDO
        IGHG=0
        DO JGHG=1,JPGHG
          IF(LGOMUA(ITYPOB,YGOMV%MGHG(JGHG))) THEN
            IGHG=IGHG+1
            JLEVGHG = JLEV+(IGHG-1)*NFLEVG
            ZSURGHG(JLEV,JGHG) = ZSURGHG(JLEV,JGHG) &
           & + YGOMUA(IPOSM)%GHG(JLEVGHG)**2
          ENDIF
        ENDDO
        IGRG=0
        DO JGRG=1,JPGRG
          IF(LGOMUA(ITYPOB,YGOMV%MGRG(JGRG))) THEN
            IGRG=IGRG+1
            JLEVGRG = JLEV+(IGRG-1)*NFLEVG
            ZSURGRG(JLEV,JGRG) = ZSURGRG(JLEV,JGRG) &
           & + YGOMUA(IPOSM)%GRG(JLEVGRG)**2
          ENDIF
        ENDDO
      ENDDO

!*          1.4.4 UPPER AIR DATA

    ELSEIF(NOBSPROFS(ITYPOB) == 1 .AND. ITYPOB /= NLIMB ) THEN
      DO JLEV = 1,NFLEVG
        IF(LGOMUA(ITYPOB,YGOMV%MU)) &
         & ZCONU(JLEV) = ZCONU(JLEV) + YGOMUA(IPOSM)%U(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MV)) &
         & ZCONV(JLEV) = ZCONV(JLEV) + YGOMUA(IPOSM)%V(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MT)) &
         & ZCONT(JLEV) = ZCONT(JLEV) + YGOMUA(IPOSM)%T(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MQ)) &
         & ZCONQ(JLEV) = ZCONQ(JLEV) + YGOMUA(IPOSM)%Q(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MO3)) &
         & ZCONO3(JLEV) = ZCONO3(JLEV) + YGOMUA(IPOSM)%O3(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MS)) &
         & ZCONSW(JLEV) = ZCONSW(JLEV) + YGOMUA(IPOSM)%S(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MR)) &
         & ZCONRW(JLEV) = ZCONRW(JLEV) + YGOMUA(IPOSM)%R(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MG)) & 
         & ZCONGW(JLEV) = ZCONGW(JLEV) + YGOMUA(IPOSM)%G(JLEV)**2
        IF(LGOMUA(ITYPOB,YGOMV%MCLW)) &
         & ZCONCLW(JLEV) = ZCONCLW(JLEV) + YGOMUA(IPOSM)%CLW(JLEV)**2

        IAERO=0
        DO JAERO=1,JPAERO
          IF(LGOMUA(ITYPOB,YGOMV%MAERO(JAERO))) THEN
            IAERO=IAERO+1
            JLEVAERO = JLEV+(IAERO-1)*NFLEVG
            ZCONAERO(JLEV,IAERO) = ZCONAERO(JLEV,IAERO) &
           & + YGOMUA(IPOSM)%AERO(JLEVAERO)**2
          ENDIF
        ENDDO
        IGHG=0
        DO JGHG=1,JPGHG
          IF(LGOMUA(ITYPOB,YGOMV%MGHG(JGHG))) THEN
            IGHG=IGHG+1
            JLEVGHG = JLEV+(IGHG-1)*NFLEVG
            ZCONGHG(JLEV,JGHG) = ZCONGHG(JLEV,JGHG) &
           & + YGOMUA(IPOSM)%GHG(JLEVGHG)**2
          ENDIF
        ENDDO
        IGRG=0
        DO JGRG=1,JPGRG
          IF(LGOMUA(ITYPOB,YGOMV%MGRG(JGRG))) THEN
            IGRG=IGRG+1
            JLEVGRG = JLEV+(IGRG-1)*NFLEVG
            ZCONGRG(JLEV,JGRG) = ZCONGRG(JLEV,JGRG) &
           & + YGOMUA(IPOSM)%GRG(JLEVGRG)**2
          ENDIF
        ENDDO
      ENDDO
    ENDIF

  ENDDO
  CALL GSTATS(1999,1)

!*          1.9   CLOSE THE OBS. SET LOOP

  CALL PUTDB('PRTGOM', 0, ILEN, INFO, 0,ZINFO,0)

ENDDO SET_LOOP
!*
!     ------------------------------------------------------------------

!        2.       GATHER INFORMATION FROM OTHER PES.
!                 ----------------------------------

IF(NPROC > 1) THEN
  CALL GATHERGOM(ISUR,ICON,ITOV,ISCA,&
   & ZSURU,ZSURV,ZSURT,ZSURQ,ZSURO3,ZSURCLW,ZSURGHG,ZSURGRG,ZSURAERO,ZSURPS,&
   & ZCONU,ZCONV,ZCONT,ZCONQ,ZCONO3,ZCONCLW,ZCONGHG,ZCONGRG,ZCONAERO,&
   & ZCONSW,ZCONRW,ZCONGW,ZCONPS,&
   & ZTOVU,ZTOVV,ZTOVT,ZTOVQ,ZTOVO3,ZTOVCLW,ZTOVGHG,ZTOVGRG,ZTOVAERO,ZTOVPS,&
   & ZSCAU,ZSCAV,ZSCAUN,ZSCAVN,ZSCAT,ZSCAQ,ZSCAPS)  
ENDIF

!     ------------------------------------------------------------------

!        3.       PRINT
!                 -----

WRITE(NULOUT,'(1X,120(''*''),/)')
WRITE(NULOUT,*) ' RMS OF GOM-ARRAYS, SIMULATION NO: ',NSIM4D
WRITE(NULOUT,*)' MODEL VALUES INTERPOLATED TO OBSERVATION LOCATIONS'
WRITE(NULOUT,*) ' OR, AFTER THE ADJOINT - GRADIENTS'
WRITE(NULOUT,*) ' FOR OBSERVATION SETS ',KSETS,' TO ',KSETE

!*          3.1   UPPER-AIR DATA

WRITE(NULOUT,'(/3X,I9,A,8(A,17X),A)')&
 & ICON,' UPPER-AIR DATA '&
 & ,'U  ','V  ','T  ','Q  ','O3 ','CLW'&
 & ,'S  ','R  ','G  '  
IF(ICON /= 0) THEN
  DO JL=1,NFLEVG
   if(LFCOBS) then
    WRITE(NULOUT,'(1X,I3,2X,9F20.2)')&
     & JL,SQRT(ZCONU(JL) /ICON)&
     & ,SQRT(ZCONV(JL) /ICON)&
     & ,SQRT(ZCONT(JL) /ICON)&
     & ,SQRT(ZCONQ(JL) /ICON)&
     & ,SQRT(ZCONO3(JL)/ICON)&
     & ,SQRT(ZCONCLW(JL)/ICON)&  
     & ,SQRT(ZCONSW(JL)/ICON)&
     & ,SQRT(ZCONRW(JL)/ICON)&
     & ,SQRT(ZCONGW(JL)/ICON)
   else
    WRITE(NULOUT,'(1X,I3,2X,9F20.13)')&
     & JL,SQRT(ZCONU(JL) /ICON)&
     & ,SQRT(ZCONV(JL) /ICON)&
     & ,SQRT(ZCONT(JL) /ICON)&
     & ,SQRT(ZCONQ(JL) /ICON)&
     & ,SQRT(ZCONO3(JL)/ICON)&
     & ,SQRT(ZCONCLW(JL)/ICON)&  
     & ,SQRT(ZCONSW(JL)/ICON)&
     & ,SQRT(ZCONRW(JL)/ICON)&
     & ,SQRT(ZCONGW(JL)/ICON)
   endif
  ENDDO
  if(LFCOBS) then
   WRITE(NULOUT,'(1X,A,6X,F20.2)')'PS: ',SQRT(ZCONPS(NFLEVG)/ICON)
  else
   WRITE(NULOUT,'(1X,A,6X,F20.13)')'PS: ',SQRT(ZCONPS(NFLEVG)/ICON)
  endif
ENDIF


!*          3.1.1   UPPER-AIR AERO DATA

IF(NAERO > 0) THEN
 WRITE(NULOUT,'(/3X,I9,A)')&
   & ICON,' UPPER-AIR AERO DATA '
  IF(ICON /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,3F20.13)')&
       & JL,SQRT(ZCONAERO(JL,1:NAERO) /ICON)
    ENDDO
  ENDIF
ENDIF

!*          3.1.2   UPPER-AIR GHG DATA

IF(NGHG > 0) THEN
  WRITE(NULOUT,'(/3X,I9,A,2(A,17X),A)')&
   & ICON,' UPPER-AIR GHG DATA '&
   & ,YGHG_NL(1:JPGHG)%cname
  IF(ICON /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,3F20.13)')&
       & JL,SQRT(ZCONGHG(JL,1:JPGHG) /ICON)
    ENDDO
  ENDIF
ENDIF

!*          3.1.3   UPPER-AIR GRG DATA

IF(NGRG > 0) THEN
  WRITE(NULOUT,'(/3X,I9,A,4(A,17X),A)')&
   & ICON,' UPPER-AIR GRG DATA '&
   & ,YGRG_NL(1:JPGRG)%cname
  
  IF(ICON /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,5(F24.13,x))')&
       & JL,SQRT(ZCONGRG(JL,1:JPGRG) /ICON)
    ENDDO
  ENDIF
ENDIF
  
!*          3.2   SURFACE DATA

WRITE(NULOUT,'(/3X,I9,A,5(A,17X),A)')&
 & ISUR,' SURFACE OBS    '&
 & ,'U  ','V  ','T  ','Q  ','O3 ','CLW'  
IF(ISUR /= 0) THEN
  DO JL=1,NFLEVG
  if(LFCOBS) then
    WRITE(NULOUT,'(1X,I3,2X,6F20.2)')&
     & JL,SQRT(ZSURU(JL) /ISUR)&
     & ,SQRT(ZSURV(JL) /ISUR)&
     & ,SQRT(ZSURT(JL) /ISUR)&
     & ,SQRT(ZSURQ(JL) /ISUR)&
     & ,SQRT(ZSURO3(JL)/ISUR)&
     & ,SQRT(ZSURCLW(JL)/ISUR)  
   else
    WRITE(NULOUT,'(1X,I3,2X,6F20.13)')&
     & JL,SQRT(ZSURU(JL) /ISUR)&
     & ,SQRT(ZSURV(JL) /ISUR)&
     & ,SQRT(ZSURT(JL) /ISUR)&
     & ,SQRT(ZSURQ(JL) /ISUR)&
     & ,SQRT(ZSURO3(JL)/ISUR)&
     & ,SQRT(ZSURCLW(JL)/ISUR)  
   endif
  ENDDO
  if(LFCOBS) then
   WRITE(NULOUT,'(1X,A,6X,F20.2)')'PS: ',SQRT(ZSURPS(NFLEVG)/ISUR)
  else
   WRITE(NULOUT,'(1X,A,6X,F20.13)')'PS: ',SQRT(ZSURPS(NFLEVG)/ISUR)
  endif
ENDIF


!*          3.2.1   SURFACE AERO DATA

IF(NAERO > 0) THEN
 WRITE(NULOUT,'(/3X,I9,A)')&
  & ISUR,' SURFACE AERO DATA '
  IF(ISUR /= 0) THEN
   DO JL=1,NFLEVG
     WRITE(NULOUT,'(1X,I3,2X,3F20.13)')&
      & JL,SQRT(ZSURAERO(JL,1:NAERO) /ISUR)
   ENDDO
  ENDIF
ENDIF

!*          3.2.2   SURFACE GHG DATA

IF(NGHG > 0) THEN
  WRITE(NULOUT,'(/3X,I9,A,2(A,17X),A)')&
   & ISUR,' SURFACE GHG DATA '&
   & ,YGHG_NL(1:JPGHG)%cname
  IF(ISUR /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,3F20.13)')&
       & JL,SQRT(ZSURGHG(JL,1:JPGHG) /ISUR)
    ENDDO
  ENDIF
ENDIF

!*          3.2.3   SURFACE GRG DATA

IF(NGRG > 0) THEN
  WRITE(NULOUT,'(/3X,I9,A,4(A,17X),A)')&
   & ISUR,' SURFACE GRG DATA '&
   & ,YGRG_NL(1:JPGRG)%cname
  IF(ISUR /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,5(F24.13,x))')&
       & JL,SQRT(ZSURGRG(JL,1:JPGRG) /ISUR)
    ENDDO
  ENDIF
ENDIF

!*          3.3   TOVS

WRITE(NULOUT,'(/3X,I9,A,5(A,17X),A)')&
 & ITOV,' TOVS DATA      '&
 & ,'U  ','V  ','T  ','Q  ','O3 ','CLW'  
IF(ITOV /= 0) THEN
  DO JL=1,NFLEVG
  if(LFCOBS) then
    WRITE(NULOUT,'(1X,I3,2X,6F20.2)')&
     & JL,SQRT(ZTOVU(JL) /ITOV)&
     & ,SQRT(ZTOVV(JL) /ITOV)&
     & ,SQRT(ZTOVT(JL) /ITOV)&
     & ,SQRT(ZTOVQ(JL) /ITOV)&
     & ,SQRT(ZTOVO3(JL)/ITOV)&
     & ,SQRT(ZTOVCLW(JL)/ITOV)  
   else
    WRITE(NULOUT,'(1X,I3,2X,6F20.13)')&
     & JL,SQRT(ZTOVU(JL) /ITOV)&
     & ,SQRT(ZTOVV(JL) /ITOV)&
     & ,SQRT(ZTOVT(JL) /ITOV)&
     & ,SQRT(ZTOVQ(JL) /ITOV)&
     & ,SQRT(ZTOVO3(JL)/ITOV)&
     & ,SQRT(ZTOVCLW(JL)/ITOV)  
   endif
  ENDDO
  if(LFCOBS) then
   WRITE(NULOUT,'(1X,A,6X,F20.2)')'PS: ',SQRT(ZTOVPS(NFLEVG)/ITOV)
  else
   WRITE(NULOUT,'(1X,A,6X,F20.13)')'PS: ',SQRT(ZTOVPS(NFLEVG)/ITOV)
  endif
ENDIF

!*          3.3.1   TOVS AERO DATA

IF(NAERO >0) THEN
 WRITE(NULOUT,'(/3X,I9,A)')&
  & ITOV,' TOVS AER DATA '
 IF(ITOV /= 0) THEN
   DO JL=1,NFLEVG
     WRITE(NULOUT,'(1X,I3,2X,3F20.13)')&
      & JL,SQRT(ZTOVAERO(JL,1:NAERO) /ITOV)
   ENDDO
  ENDIF
ENDIF

!*          3.3.2   TOVS GHG DATA

IF(NGHG >0) THEN
  WRITE(NULOUT,'(/3X,I9,A,2(A,17X),A)')&
   & ITOV,' TOVS GHG DATA '&
   & ,YGHG_NL(1:JPGHG)%cname
  IF(ITOV /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,3F20.13)')&
       & JL,SQRT(ZTOVGHG(JL,1:JPGHG) /ITOV)
    ENDDO
  ENDIF
ENDIF

!*          3.3.3   TOVS GRG DATA

IF(NGRG >0) THEN
  WRITE(NULOUT,'(/3X,I9,A,4(A,17X),A)')&
   & ITOV,' TOVS GRG DATA '&
   & ,YGRG_NL(1:JPGRG)%cname
  IF(ITOV /= 0) THEN
    DO JL=1,NFLEVG
      WRITE(NULOUT,'(1X,I3,2X,5(F24.13,x))')&
       & JL,SQRT(ZTOVGRG(JL,1:JPGRG) /ITOV)
    ENDDO
  ENDIF
ENDIF

!*          3.5   SCATTEROMETER

ZEROO3=0.0_JPRB
ZEROCLW=0.0_JPRB
WRITE(NULOUT,'(/3X,I9,A,7(A,17X),A)')&
 & ISCA,' SCATTEROMETER '&
 & ,'U  ','V  ','UN ','VN ','T  ','Q  ','O3 ','CLW'  
IF(ISCA /= 0) THEN
  IL=NFLEVG
 if(LFCOBS) then
  WRITE(NULOUT,'(1X,I3,5X,8F20.2)')&
   & IL,SQRT(ZSCAU/ISCA)&
   & ,SQRT(ZSCAV/ISCA)&
   & ,SQRT(ZSCAUN/ISCA)&
   & ,SQRT(ZSCAVN/ISCA)&
   & ,SQRT(ZSCAT/ISCA)&
   & ,SQRT(ZSCAQ/ISCA)&
   & ,ZEROO3 &
   & ,ZEROCLW  
  WRITE(NULOUT,'(1X,A,6X,F20.2/)') 'PS: ',SQRT(ZSCAPS/ISCA)
 else
  WRITE(NULOUT,'(1X,I3,5X,8F20.13)')&
   & IL,SQRT(ZSCAU/ISCA)&
   & ,SQRT(ZSCAV/ISCA)&
   & ,SQRT(ZSCAUN/ISCA)&
   & ,SQRT(ZSCAVN/ISCA)&
   & ,SQRT(ZSCAT/ISCA)&
   & ,SQRT(ZSCAQ/ISCA)&
   & ,ZEROO3 &
   & ,ZEROCLW  
  WRITE(NULOUT,'(1X,A,6X,F20.13/)') 'PS: ',SQRT(ZSCAPS/ISCA)
 endif
ENDIF
!*
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PRTGOM',1,ZHOOK_HANDLE)
END SUBROUTINE PRTGOM
