#ifdef VPP
! Fujitsu compiler bug 9606
!OCL SCALAR
#endif
SUBROUTINE PRTJO

!**** *PRTJO* - PRINT DIFFERENT TERMS OF THE COST FUNCTION FROM THE
!****            DIAGNOSTIC JO-TABLE

!     Purpose.
!     -------
!           PRINT DIFFERENT TERMS OF THE COST FUNCTION IN ORDER TO
!     FIND OUT THE RESPECTIVE CONTRIBUTIONS OF EACH OBS TYPE, EACH

!     SUB OBS TYPE, EACH OBSERVED VARIABLE....
!**   Interface.
!**   ---------
!**   *CALL* * PRTJO*
!**         The common YOMCOSJO is an implicit INPUT of SUDIMO  AS IT
!**         CONTAINS THE COST FUNCTION TABLE JOT.

!     Author: Jean Pailleux
!     Date  : 89-12-29

!     Modifications
!     -------------
!     Name changed to PRTJO on 90-4-2: Jean Pailleux
!     Names changed again            :  "     "
!     Erik Andersson - 980818 : Introduced total ozone
!     Erik Andersson - 001003 : JOT Jo table
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Tremolet    29-May-2008 Print Jo table by bins
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIMO
USE YOMDIM   , ONLY : NSMAX
USE YOMANCS  , ONLY : RMDI
USE YOMCMA   , ONLY : NMXOTP
USE YOMLUN   , ONLY : NULOUT
USE YOMCOSJO , ONLY : JOT      ,CVAR_NAME, NPRTBINS
USE YOMCT0   , ONLY : LSCREEN  ,LOBSC1   ,NCONF   ,LGUESS
USE YOMVAR   , ONLY : NSIM4D   ,NUPTRA

IMPLICIT NONE

REAL(KIND=JPRB)    :: ZJOSUM,ZCOST,ZRMS_OBSERR,ZRMS_BGERR,ZJOTOT
INTEGER(KIND=JPIM) :: JOTP, JCTP, JVAR, IJOSUM,ICOUNT,IJOTOT,jbin
CHARACTER(LEN=16) ::  CLJOB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

! Print the JO table
IF (LHOOK) CALL DR_HOOK('PRTJO',0,ZHOOK_HANDLE)
IF(LSCREEN) THEN
  CLJOB='SCREENING JOB   '
ELSEIF(LOBSC1) THEN
  CLJOB='TRAJECTORY JOB  '
ELSE
  CLJOB='MINIMISATION JOB'
ENDIF

IF (NPRTBINS>1) THEN
  DO jbin=1,NPRTBINS
    WRITE(NULOUT,'(/,A,I2,A,A,A,I4.4,3(A,I4))') &
     & ' Jo sub-table (sub-window ',jbin,'),',CLJOB,' T',NSMAX,&
     & ' NCONF= ',NCONF,' NSIM4D= ',NSIM4D,' NUPTRA= ',NUPTRA  
    WRITE(NULOUT,*) &
     & '====================================================================='//&
     & '===================='
    
    ZRMS_BGERR=0.0_JPRB
    ZJOTOT=0.0_JPRB
    IJOTOT=0
    DO JOTP=1,NMXOTP
      ZJOSUM=0.0_JPRB
      IJOSUM=0
      WRITE(NULOUT,*)
      WRITE(NULOUT,'(8X,A,I5,A,A32)') &
       & 'Obstype ',JOTP,' === ',JOT(JOTP)%name  
      WRITE(NULOUT,'(8X,A)') &
       & '--------------------------------------------------'  
    
      DO JCTP=1,JOT(JOTP)%size
        IF(ANY(JOT(JOTP)%jost(JCTP)%count(:,jbin) /= 0)) THEN
          WRITE(NULOUT,'(13X,A,I5,A,A32)') &
           & 'Codetype ',JOT(JOTP)%jost(JCTP)%codetype,' === ',&
           & JOT(JOTP)%jost(JCTP)%sname  
          WRITE(NULOUT,'(16X,2A)') &
           & 'Variable      DataCount          Jo_Costfunction'&
           & ,'       JO/n       ObsErr      BgErr'  
        ENDIF
        DO JVAR=1,JPXVAR
          ICOUNT  =     JOT(JOTP)%jost(JCTP)%count(JVAR,jbin)
          ZCOST   =     JOT(JOTP)%jost(JCTP)%cost(JVAR,jbin)
          ZRMS_OBSERR = JOT(JOTP)%jost(JCTP)%obserr(JVAR)
          IF(LGUESS) ZRMS_BGERR  = JOT(JOTP)%jost(JCTP)%bgerr(JVAR)
          IF(ICOUNT /= 0) THEN
            IF(ZRMS_OBSERR /= 0) &
             & ZRMS_OBSERR = SQRT(ZRMS_OBSERR/ICOUNT)
            IF(LGUESS .AND. ZRMS_BGERR > 0.0_JPRB .AND. ZRMS_BGERR /= RMDI) &
             & ZRMS_BGERR  = SQRT(ZRMS_BGERR /ICOUNT)
            IF(ZRMS_BGERR /= RMDI) THEN
              WRITE(NULOUT,'(19X,A3,3X,I10,3X,G27.13,3X,F8.2,1X,2(2X,E10.3))') &
             &  CVAR_NAME(JVAR),ICOUNT,ZCOST,ZCOST/ICOUNT,ZRMS_OBSERR,ZRMS_BGERR
            ELSE
              WRITE(NULOUT,'(19X,A3,3X,I10,3X,G27.13,3X,F8.2,3X,E10.3,4X,A4)') &
             &  CVAR_NAME(JVAR),ICOUNT,ZCOST,ZCOST/ICOUNT,ZRMS_OBSERR,'RMDI'
            ENDIF
            IJOSUM=IJOSUM + ICOUNT
            ZJOSUM=ZJOSUM + ZCOST
          ENDIF
        ENDDO
      ENDDO
      ZJOTOT=ZJOTOT+ZJOSUM
      IJOTOT=IJOTOT+IJOSUM
      IF(IJOSUM /= 0) THEN
        WRITE(NULOUT,'(25X,A)') &
         & '----------   ---------------------------   --------'  
        WRITE(NULOUT,'(7X,A,I2,A,I10,3X,G27.13,3X,F8.2)') &
         & 'ObsType ',JOTP,' Total: ',IJOSUM,ZJOSUM,ZJOSUM/IJOSUM  
      ENDIF
    ENDDO
    WRITE(NULOUT,*)
    WRITE(NULOUT,*) &
     & ' ---------------------------------------------------------------------------'  
    IF (IJOTOT > 0) THEN
      WRITE(NULOUT,'(13X,A,I10,3X,G27.13,3X,F8.2)') &
       & 'Jo Global : ',IJOTOT,ZJOTOT,ZJOTOT/IJOTOT  
    ELSE
      WRITE(NULOUT,'(13X,A,I10,3X,G27.13,3X,A8)') &
       & 'Jo Global : ',IJOTOT,ZJOTOT,'********'  
    ENDIF
    WRITE(NULOUT,*) &
     & ' ==========================================================================='  
    WRITE(NULOUT,*)
  ENDDO
  WRITE(NULOUT,*)
  WRITE(NULOUT,*) ' End of Jo sub-tables'
ENDIF

WRITE(NULOUT,'(/,A,A,A,I4.4,3(A,I5))') &
 & ' Diagnostic JO-table (JOT) ',CLJOB,' T',NSMAX,&
 & ' NCONF= ',NCONF,' NSIM4D= ',NSIM4D,' NUPTRA= ',NUPTRA  
WRITE(NULOUT,*) &
 & '====================================================================='//&
 & '===================='

ZRMS_BGERR=0.0_JPRB
ZJOTOT=0.0_JPRB
IJOTOT=0
DO JOTP=1,NMXOTP
  ZJOSUM=0.0_JPRB
  IJOSUM=0
  WRITE(NULOUT,*)
  WRITE(NULOUT,'(8X,A,I5,A,A32)') &
   & 'Obstype ',JOTP,' === ',JOT(JOTP)%name  
  WRITE(NULOUT,'(8X,A)') &
   & '--------------------------------------------------'  

  DO JCTP=1,JOT(JOTP)%size
    IF(ANY(JOT(JOTP)%jost(JCTP)%count(:,:) /= 0)) THEN
      WRITE(NULOUT,'(13X,A,I5,A,A32)') &
       & 'Codetype ',JOT(JOTP)%jost(JCTP)%codetype,' === ',&
       & JOT(JOTP)%jost(JCTP)%sname  
      WRITE(NULOUT,'(16X,2A)') &
       & 'Variable      DataCount          Jo_Costfunction'&
       & ,'       JO/n       ObsErr      BgErr'  
    ENDIF
    DO JVAR=1,JPXVAR
      ICOUNT  = SUM(JOT(JOTP)%jost(JCTP)%count(JVAR,:))
      ZCOST   = SUM(JOT(JOTP)%jost(JCTP)%cost(JVAR,:))
      ZRMS_OBSERR = JOT(JOTP)%jost(JCTP)%obserr(JVAR)
      IF(LGUESS) ZRMS_BGERR  = JOT(JOTP)%jost(JCTP)%bgerr(JVAR)
      IF(ICOUNT /= 0) THEN
        IF(ZRMS_OBSERR /= 0) &
         & ZRMS_OBSERR = SQRT(ZRMS_OBSERR/ICOUNT)
        IF(LGUESS .AND. ZRMS_BGERR > 0.0_JPRB .AND. ZRMS_BGERR /= RMDI) &
         & ZRMS_BGERR  = SQRT(ZRMS_BGERR /ICOUNT)
        IF(ZRMS_BGERR /= RMDI) THEN
          WRITE(NULOUT,'(19X,A3,3X,I10,3X,G27.13,3X,F8.2,1X,2(2x,E10.3))') &
         & CVAR_NAME(JVAR),ICOUNT,ZCOST,ZCOST/ICOUNT,ZRMS_OBSERR,ZRMS_BGERR
        ELSE
          WRITE(NULOUT,'(19X,A3,3X,I10,3X,G27.13,3X,F8.2,1X,3X,E10.3,4X,A4)') &
         & CVAR_NAME(JVAR),ICOUNT,ZCOST,ZCOST/ICOUNT,ZRMS_OBSERR,'RMDI'
        ENDIF
        IJOSUM=IJOSUM + ICOUNT
        ZJOSUM=ZJOSUM + ZCOST
      ENDIF
    ENDDO
  ENDDO
  ZJOTOT=ZJOTOT+ZJOSUM
  IJOTOT=IJOTOT+IJOSUM
  IF(IJOSUM /= 0) THEN
    WRITE(NULOUT,'(25X,A)') &
     & '----------   ---------------------------   --------'  
    WRITE(NULOUT,'(7X,A,I2,A,I10,3X,G27.13,3X,F8.2)') &
     & 'ObsType ',JOTP,' Total: ',IJOSUM,ZJOSUM,ZJOSUM/IJOSUM  
  ENDIF
ENDDO
WRITE(NULOUT,*)
WRITE(NULOUT,*) &
 & ' ---------------------------------------------------------------------------'  
IF (IJOTOT > 0) THEN
  WRITE(NULOUT,'(13X,A,I10,3X,G27.13,3X,F8.2)') &
   & 'Jo Global : ',IJOTOT,ZJOTOT,ZJOTOT/IJOTOT  
ELSE
  WRITE(NULOUT,'(13X,A,I10,3X,G27.13,3X,A8)') &
   & 'Jo Global : ',IJOTOT,ZJOTOT,'********'  
ENDIF
WRITE(NULOUT,*) &
 & ' ==========================================================================='  
WRITE(NULOUT,*)
WRITE(NULOUT,*) ' End of JO-table (JOT)'

IF (LHOOK) CALL DR_HOOK('PRTJO',1,ZHOOK_HANDLE)
END SUBROUTINE PRTJO
