SUBROUTINE RDGPFA(KUNTIN,CDFIELD,KDEB2,KFIN,KINDEX,PFIELDBUF,&
 & K1,K2,KPTR,CDFILE,CDGROUP,LDABORT)  

!**** *RDGPFA*  - Read a gridpoint fields from *FA*

!     Purpose.
!     --------
!           To read a gridpoint field from a file Arpege/Aladin
!           Mandatory abort or replace missing field by 0.

!**   Interface.
!     ----------
!        *CALL* *RDGPFA(...)*

!        Explicit arguments : KUNTIN - file unit number
!        ------------------   CDFIELD - field name
!                             KDEB2 - first latitude to be read from file
!                             KFIN  - last latitude to be read from file
!                             KINDEX  - start index to read from fields in file
!                             PFIELDBUF - output array
!                             K1 - first index (for prints)
!                             K2 - second index (for prints)
!                             K2=0 if size(shape(generic group))=1
!                             KPTR - pointer in model buffer (for prints)
!                             CDFILE - file stamp (for prints)
!                             CDGROUP - field group name (for prints)
!                             LDABORT - .TRUE. to abort is file is missing

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.     FACILE - read ARPEGE file
!     ----------    

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        R. El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 97-12-02 (From SUGRIDA)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDLON
USE YOMGEM   , ONLY : NLOENG   ,NGPTOTG
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KUNTIN 
CHARACTER         ,INTENT(IN)    :: CDFIELD*(*) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDEB2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KINDEX 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFIELDBUF(NGPTOTG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: K1 
INTEGER(KIND=JPIM),INTENT(IN)    :: K2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPTR 
CHARACTER         ,INTENT(IN)    :: CDFILE*(*) 
CHARACTER         ,INTENT(IN)    :: CDGROUP*(*) 
LOGICAL           ,INTENT(IN)    :: LDABORT 
REAL(KIND=JPRB) :: ZREALG(NDLON*(NDGLG+2))

INTEGER(KIND=JPIM) :: ILEN, INDEX, IPOS, IREP, IROF, JGL, JLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RDGPFA',0,ZHOOK_HANDLE)
CALL LFINFO(IREP,KUNTIN,CDFIELD,ILEN,IPOS)
IF (ILEN > 0) THEN
  CALL FACILE(IREP,KUNTIN,CDFIELD(1:4),1,CDFIELD(5:),ZREALG,.FALSE.)
  IROF=1
  INDEX=KINDEX
  DO JGL=KDEB2,KFIN
    DO JLON=1,NLOENG(JGL)
      PFIELDBUF(IROF) = ZREALG(INDEX+JLON)
      IROF=IROF+1
    ENDDO
    INDEX=INDEX+NLOENG(JGL)
  ENDDO
  IF (K2 == 0) THEN
!         size(shape(generic group))=1
    WRITE(NULOUT,'(2X,A,'' READ FROM '',A,'' FILE INTO '',&
     & A,1X,I2.2,1X,I3.3)') &
     & CDFIELD,CDFILE,CDGROUP,K1,KPTR  
  ELSE
!         size(shape(generic group))=2
    WRITE(NULOUT,'(2X,A,'' READ FROM '',A,'' FILE INTO '',&
     & A,1X,I2.2,1X,I2.2,1X,I3.3)')&
     & CDFIELD,CDFILE,CDGROUP,K1,K2,KPTR  
  ENDIF
ELSEIF(.NOT.LDABORT) THEN
  PFIELDBUF(:)=0.0_JPRB
  IF (K2 == 0) THEN
!         size(shape(generic group))=1
    WRITE(NULOUT,'(2X,A,'' MISSING FROM '',A,&
     & '' FILE => SET TO 0. INTO '',&
     & A,1X,I2.2,1X,I3.3)') &
     & CDFIELD,CDFILE,CDGROUP,K1,KPTR  
  ELSE
!         size(shape(generic group))=2
    WRITE(NULOUT,'(2X,A,'' MISSING FROM '',A,&
     & '' FILE => SET TO 0. INTO '',&
     & A,1X,I2.2,1X,I2.2,1X,I3.3)') &
     & CDFIELD,CDFILE,CDGROUP,K1,K2,KPTR  
  ENDIF
ELSE
  WRITE(NULOUT,'(2X,A,'' MISSING FROM '',A,'' FILE '')') CDFIELD,CDFILE
  CALL ABOR1('RDGPFA : ABOR1 CALLED')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RDGPFA',1,ZHOOK_HANDLE)
END SUBROUTINE RDGPFA
