SUBROUTINE RDSPEC(LDKEEP)

!**** *RDSPEC*  - read  spectral arrays

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *RDSPEC

!        Explicit arguments : 
!        --------------------
!                  LDKEEP : If .T. then do not deallocate SPAxSAVE

!        Implicit arguments :      None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M.Hamrud   *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-11-18
!        Modified : 95-06-06 L.Isaksen   Reordering of spectral arrays
!        Modified : 98-02-17 R. El Khatib Conditionnal allocation of SPA3 +case aladin
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LELAM
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SPA3SAVE ,SPA2SAVE

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDKEEP 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "euvcopy.intfb.h"
#include "sualspa.intfb.h"


!     ------------------------------------------------------------------

!*       1.    ALLOCATE SPECTRAL ARRAYS
!              ------------------------

IF (LHOOK) CALL DR_HOOK('RDSPEC',0,ZHOOK_HANDLE)
IF (.NOT.ALLOCATED(SPA3)) THEN
  CALL SUALSPA
ENDIF

!*       2.    OPEN FILES
!              ----------

!     ------------------------------------------------------------------

!*       4.    COPY  SPECTRAL DATA
!              -------------------

SPA3(:,:,:)=SPA3SAVE(:,:,:)
SPA2(:,:)=SPA2SAVE(:,:)
IF (.NOT.LDKEEP) DEALLOCATE(SPA3SAVE,SPA2SAVE)

!     ------------------------------------------------------------------

!*       5.    COPY ALADIN MEAN WIND
!              ---------------------

IF (LELAM) THEN
  CALL EUVCOPY(.TRUE.)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RDSPEC',1,ZHOOK_HANDLE)
END SUBROUTINE RDSPEC

