SUBROUTINE READ_GRID_GRIB(CDFILE,YDGRID)

!     Purpose.
!     --------
!     Read grid point fields on one processor and distribute it.

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original : 12-02-01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Tremolet    07-Jun-2004 Use type gridpoint_field
! ----------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN, ONLY: NULERR, nulout
USE YOMMP , ONLY: MYPROC
USE YOMMSC, ONLY: nintlen
USE GRIB_HEADER_MIX
USE GRIDPOINT_FIELDS_MIX

IMPLICIT NONE

CHARACTER(LEN=*),      INTENT(IN)    :: CDFILE 
TYPE(GRIDPOINT_FIELD), INTENT(INOUT) :: YDGRID

INTEGER(KIND=JPIM) :: ii,jb,jj,jf,jl,ioproc,iunit,ierr,iword,ibyte,ilen
REAL(KIND=JPRB), ALLOCATABLE :: zglob(:), zlocal(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: igrib(:)
TYPE(grib_header) :: YLGRIBH
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ircvgpf.intfb.h"
#include "isndgpf.intfb.h"

! ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('READ_GRID_GRIB',0,ZHOOK_HANDLE)
WRITE(nulout,*)'READ_GRID_GRIB: reading file ',CDFILE

ALLOCATE(zlocal(YDGRID%NGPTOT))

ioproc=1
IF (MYPROC==ioproc) THEN
  ALLOCATE(zglob(YDGRID%NGPTOTG))
  ALLOCATE(igrib(YDGRID%NGPTOTG))
  CALL allocate_grib_header(YLGRIBH)
  CALL GSTATS(1754,0)
  CALL PBOPEN(iunit,CDFILE,'r',ierr)
  CALL GSTATS(1754,1)
  IF(ierr /= 0) THEN
    WRITE(nulerr,*)'READ_GRID_GRIB: PBOPEN returned error code',ierr
    CALL ABOR1('READ_GRID_GRIB: error in PBOPEN')
  ENDIF
ELSE
  ALLOCATE(zglob(1))
ENDIF

DO jf=1,YDGRID%NG3D
  DO jl=1,YDGRID%NFLEVG
    IF (MYPROC==ioproc) THEN
      ibyte=nintlen*YDGRID%NGPTOTG
      CALL GSTATS(1754,0)
      CALL PBGRIB(iunit,igrib,ibyte,ilen,ierr)
      CALL GSTATS(1754,1)
      IF (ierr/=0) THEN
        WRITE(nulerr,*)'READ_GRID_GRIB: PBGRIB returned error code',ierr
        WRITE(nulerr,*)'READ_GRID_GRIB: file,jf=',CDFILE,jf
        CALL ABOR1('READ_GRID_GRIB: error in PBGRIB')
      ENDIF

      CALL GSTATS(1703,0)
      ilen = ilen/nintlen
      CALL GRIBEX(YLGRIBH%nsec0,YLGRIBH%nsec1,YLGRIBH%nsec2gg, &
       & YLGRIBH%rsec2,YLGRIBH%nsec3,YLGRIBH%rsec3,YLGRIBH%nsec4gg, &
       & zglob,YDGRID%NGPTOTG,igrib,ilen,iword,'D',ierr)  
      IF (ierr/=0) THEN
        write(nulerr,*)'READ_GRID_GRIB: GRIBEX returned error code ',ierr
        CALL ABOR1('READ_GRID_GRIB: error in GRIBEX')
      ENDIF
      IF (YDGRID%NGRIB(jf)/=YLGRIBH%nsec1(6)) THEN
        write(nulerr,*)'READ_GRID_GRIB: expected ',YDGRID%NGRIB(jf)
        write(nulerr,*)'READ_GRID_GRIB: read     ',YLGRIBH%nsec1(6)
        CALL ABOR1('READ_GRID_GRIB: error grib code')
      ENDIF
      CALL GSTATS(1703,1)

      CALL ISNDGPF(zglob,zlocal)
    ENDIF

    CALL IRCVGPF(zlocal,zglob,ioproc)

    ii=0
    DO jb=1,YDGRID%NGPBLKS
      ilen=MIN(YDGRID%NPROMA, YDGRID%NGPTOT-(jb-1)*YDGRID%NPROMA)
      DO jj=1,ilen
        ii=ii+1
        YDGRID%GP3D(jj,jl,jf,jb)=zlocal(ii)
      ENDDO
    ENDDO
    IF (ii/=YDGRID%NGPTOT) CALL ABOR1('READ_GRID_GRIB: error length 3D')
  ENDDO
ENDDO

DO jf=1,YDGRID%NG2D
  IF (MYPROC==ioproc) THEN
    CALL GSTATS(1754,0)
    ibyte=nintlen*YDGRID%NGPTOTG
    CALL PBGRIB(iunit,igrib,ibyte,ilen,ierr)
    IF (ierr/=0) THEN
      WRITE(nulerr,*)'READ_GRID_GRIB: PBGRIB returned error code',ierr
      WRITE(nulerr,*)'READ_GRID_GRIB: file,jf=',CDFILE,jf
      CALL ABOR1('READ_GRID_GRIB: error in PBGRIB')
    ENDIF
    CALL GSTATS(1754,1)

    CALL GSTATS(1703,0)
    ilen = ilen/nintlen
    CALL GRIBEX(YLGRIBH%nsec0,YLGRIBH%nsec1,YLGRIBH%nsec2gg, &
     & YLGRIBH%rsec2,YLGRIBH%nsec3,YLGRIBH%rsec3,YLGRIBH%nsec4gg, &
     & zglob,YDGRID%NGPTOTG,igrib,ilen,iword,'D',ierr)  
    IF (ierr/=0) THEN
      write(nulerr,*)'READ_GRID_GRIB: GRIBEX returned error code ',ierr
      CALL ABOR1('READ_GRID_GRIB: error in GRIBEX')
    ENDIF
    IF (YDGRID%NGRIB(YDGRID%NG3D+jf)/=YLGRIBH%nsec1(6)) THEN
      write(nulerr,*)'READ_GRID_GRIB: expected ',YDGRID%NGRIB(YDGRID%NG3D+jf)
      write(nulerr,*)'READ_GRID_GRIB: read     ',YLGRIBH%nsec1(6)
      CALL ABOR1('READ_GRID_GRIB: error grib code')
    ENDIF
    CALL GSTATS(1703,1)

    CALL ISNDGPF(zglob,zlocal)
  ENDIF

  CALL IRCVGPF(zlocal,zglob,ioproc)

  ii=0
  DO jb=1,YDGRID%NGPBLKS
    ilen=MIN(YDGRID%NPROMA, YDGRID%NGPTOT-(jb-1)*YDGRID%NPROMA)
    DO jj=1,ilen
      ii=ii+1
      YDGRID%GP2D(jj,jf,jb)=zlocal(ii)
    ENDDO
  ENDDO
  IF (ii/=YDGRID%NGPTOT) CALL ABOR1('READ_GRID_GRIB: error length 2D')
ENDDO

CALL HUGIFY_ENDGRID(YDGRID)

IF (MYPROC==ioproc) THEN
  DEALLOCATE(igrib)
  CALL PBCLOSE(iunit,ierr)
  IF (ierr /= 0) THEN
    write(nulerr,*) 'READ_GRID_GRIB: PBCLOSE returned error code ',ierr
    CALL ABOR1 ('READ_GRID_GRIB: error in PBCLOSE')
  ENDIF
  CALL deallocate_grib_header(YLGRIBH)
ENDIF

DEALLOCATE(zlocal)
DEALLOCATE(zglob)

IF (LHOOK) CALL DR_HOOK('READ_GRID_GRIB',1,ZHOOK_HANDLE)
END SUBROUTINE READ_GRID_GRIB
