SUBROUTINE RESET_ACCFIE_VAREPS(KSTEP)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPROMA
USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : RDAY     ,REA      ,REPSM    ,&
 & RV       ,RCPV     ,RETV     ,&
 & RCW      ,RCS      ,RLVTT    ,RLSTT    ,RTT      ,&
 & RALPW    ,RBETW    ,RGAMW    ,RALPS    ,RBETS    ,&
 & RGAMS    ,RALPD    ,RBETD    ,RGAMD   ,RG
USE SURFACE_FIELDS_MIX , ONLY : SD_VD, YSD_VD, YSD_VDD
USE YOMVAREPS, ONLY : NST_FCLENGTH_INI, NAFVAREPS, NAFVAREPSGC
USE YOMGRB   , ONLY : NLEG,       &
 & NGRBSWVL1 ,NGRBSD   ,NGRBLSP  ,&
 & NGRBCP   ,NGRBSF   ,NGRBSDOR ,NGRBSWVL2 ,NGRBSR   ,&
 & NGRBE    ,NGRBSWVL3 ,NGRBSRC  ,NGRBRO   ,NGRBSWVL4 ,&
 & NGRBCSF  ,NGRBLSF  ,NGRBFSR
USE YOMGEM   , ONLY : NGPTOT

!**** *RESET_ACCFIE_VAREPS*  - UPDATE TIME OF THE MODEL

!     Purpose.
!     --------
!     UPDATE TIME OF THE MODEL

!**   Interface.
!     ----------
!        *CALL* *RESET_ACCFIE_VAREPS(KSTEP)

!        Explicit arguments :
!        --------------------
!        KSTEP   : TIME STEP INDEX

!        Implicit arguments :
!        --------------------
!        YOMRIP

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Roberto Buizza *ECMWF*

!     Modifications.
!     --------------
!        Original : 24 June 2005
!        M.Hamrud      01-Jul-2006 Revised surface fields

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM) :: IFILE, JL, IFIELDS, JSTGLO, IEND, IBLK, IST, IFLDP, JV, IGRBCODE
LOGICAL            :: LLDSPOR
REAL(KIND=JPRB) :: ZFIELD(NGPTOT,YSD_VDD%NUMFLDS)
REAL(KIND=JPRB) :: ZD1, ZD2, ZD3, ZD4, ZSCALE, ZSCALER
REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

!     ------------------------------------------------------------------

!#include "gpnorm2.intfb.h"
#include "sugridg.intfb.h"

#include "fctast.h"
#include "fcttim.h"
#include "fcttrm.h"

IF (LHOOK) CALL DR_HOOK('RESET_ACCFIE_VAREPS',0,ZHOOK_HANDLE)

!        Update accumulated fields for VAREPS

IF( (NLEG >= 2) .AND. (KSTEP == NST_FCLENGTH_INI) ) THEN
  WRITE(NULOUT,'(A,1X,I10)') ' VAREPS accumulated fields - UPDATE KSTEP=',KSTEP
  WRITE(NULOUT,'(A,1X,I10)') ' >> VAREPS accumulated fields - KSTEP=',KSTEP
  WRITE(NULOUT,'(A,1X,I10)') ' >> ......................... - NST_FCLENGTH_INI=',NST_FCLENGTH_INI
  WRITE(NULOUT,'(A,1X,I10)') ' >> ......................... - NLEG=',NLEG
  WRITE(NULOUT,'(A,1X,I10)') ' >> ......................... - NAFVAREPS=',NAFVAREPS
  WRITE(NULOUT,'(A,1X,I10)') ' >> ......................... - NGPTOT=',NGPTOT
  !debug WRITE(NULOUT,'(A)') ' VAREPS accumulated fields - norms BEFORE update'
  !debug CALL FLUSH(NULOUT)
  !debug  CALL GPNORM2(NVDIAG,MVDIAG,GPPBUF)

  IFILE=9
  IFIELDS=NAFVAREPS
  LLDSPOR=.FALSE.
  CALL SUGRIDG(IFILE,LLDSPOR,PFIELD=ZFIELD)

  ZD1 = 0.07_JPRB
  ZD2 = 0.21_JPRB
  ZD3 = 0.72_JPRB
  ZD4 = 1.89_JPRB

  DO JSTGLO=1,NGPTOT,NPROMA

    IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
    IST=1
    IBLK=(JSTGLO-1)/NPROMA+1

    DO JV=1,YSD_VDD%NUMFLDS
      IGRBCODE = YSD_VD%YVD(JV)%IGRBCODE
      IF(IGRBCODE /= -999) THEN
        IFLDP=ISRCHEQ(IFIELDS,NAFVAREPSGC,1,IGRBCODE)
        !debug WRITE(NULOUT,'(">> RESET_ACCFIE_VAREPS: JV,IFLDP=",2(I10,1X))') JV,IFLDP
        IF(IFLDP <= IFIELDS) THEN

! Define re-scaling factor for fields read from MARS (as in wrmlpplg.F90)

          IF (IGRBCODE == NGRBSWVL1 ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBSD  ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBLSP ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBCP  ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBSF  ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBSWVL2) THEN
            ZSCALE = 1.0E-3_JPRB*ZD1/ZD2
          ELSEIF(IGRBCODE == NGRBSR  ) THEN
            ZSCALE = 1.0_JPRB/RG
          ELSEIF(IGRBCODE == NGRBE   ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBSWVL3) THEN
            ZSCALE = 1.0E-3_JPRB*ZD1/ZD3
          ELSEIF(IGRBCODE == NGRBSRC ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBRO  ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBSWVL4) THEN
            ZSCALE = 1.0E-3_JPRB*ZD1/ZD4
          ELSEIF(IGRBCODE == NGRBCSF ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBLSF ) THEN
            ZSCALE = 1.0E-3_JPRB
          ELSEIF(IGRBCODE == NGRBFSR ) THEN
            ZSCALE = 1.0_JPRB/RG
          ELSEIF(IGRBCODE == NGRBSDOR) THEN
            ZSCALE = 1.0_JPRB/RG
          ELSE
            ZSCALE = 1.0_JPRB
          ENDIF
          ZSCALER = 1.0_JPRB / ZSCALE

          DO JL=1,IEND
            SD_VD(JL,JV,IBLK) = ZSCALER*ZFIELD(JSTGLO+JL-1,JV)
          ENDDO
        ENDIF
      ENDIF
    ENDDO

  ENDDO 
  !debug WRITE(NULOUT,'(A)') ' VAREPS accumulated fields - norms AFTER update'
  !debug CALL FLUSH(NULOUT)
  !deubg CALL GPNORM2(NVDIAG,MVDIAG,GPPBUF)
ENDIF

IF (LHOOK) CALL DR_HOOK('RESET_ACCFIE_VAREPS',1,ZHOOK_HANDLE)

END SUBROUTINE RESET_ACCFIE_VAREPS
