SUBROUTINE SAVMODERR

!   Purpose.
!   --------
!     Save model error control variable in GRIB format.

!   Author.
!   -------
!     Y. Tremolet

!   Modifications.
!   --------------
!     Original   22-Aug-2005
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPRB, JPIM
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCT0  , ONLY: NSTART, NSTOP
USE YOMCT3  , ONLY: NSTEP
USE YOMLUN  , ONLY: NULOUT
USE YOMVAR  , ONLY: NUPTRA, LINITCV
USE YOMMODEL_ERROR, ONLY: NDIM_MODERR, NCOMP_MODERR, NSTEP_MODERR, NSERR, &
                     & NVE3GP, NVE2GP, NGRBMERRGP, GPMODERR, GPFGMODERR, &
                     & NVE3SP, NVE2SP, NGRBMERRSP, SPMODERR, SPFGMODERR, &
                     & LFGMODERR, SPMODERR_HYB, GPMODERR_HYB, &
                     & NTYPE_MODERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

CHARACTER(LEN=14) :: clfile, clfile_diag
TYPE(gridpoint_field) :: YLGRID
TYPE(spectral_field)  :: YLSPEC
INTEGER(KIND=JPIM) :: jj, js, isave, istperr, icomp
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "wrmoderr.intfb.h"
#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SAVMODERR',0,ZHOOK_HANDLE)

! Need correct NSTEP for GRIB header
isave=NSTEP

! Allocate work space
CALL ALLOCATE_GRID(YLGRID,NVE3GP,NVE2GP,NGRBMERRGP)
CALL ALLOCATE_SPEC(YLSPEC,NSERR,0,NVE3SP,NVE2SP,0,NGRBMERRSP)

! Book-keeping for model error components
istperr=(NSTOP-NSTART)/NSTEP_MODERR
NSTEP=NSTART
IF (LINITCV .AND. (NTYPE_MODERR == 4 .OR.NTYPE_MODERR == 5)) THEN
!GR: in this cases no model error at window-start, to be consistent with sumoderr!!!!!
  istperr=(NSTOP-NSTART)/(NSTEP_MODERR+1)
  NSTEP=NSTEP+istperr
ENDIF
icomp=0

! Save model error
DO js=1,NSTEP_MODERR
  DO jj=1,NCOMP_MODERR
    icomp=icomp+1

    YLGRID=GPMODERR(icomp)
    YLSPEC=SPMODERR(icomp)

    IF (NTYPE_MODERR == 5) THEN
        clfile_diag(1:14)='spmoderdiaXXYY'
        WRITE(clfile_diag(11:12),'(I2.2)')NUPTRA
        WRITE(clfile_diag(13:14),'(I2.2)') icomp
        CALL WRMODERR(clfile_diag,YLSPEC,YLGRID,jj)
        CALL SELF_ADD(YLGRID,GPMODERR_HYB(icomp))
        CALL SELF_ADD(YLSPEC,SPMODERR_HYB(icomp))
        write(nulout,*) 'nstep of moderr saving:',nstep
        call spectral_norm(ylspec,' savmoderr: before writing reconstructed increments to file')
    ELSE
      IF (LFGMODERR) THEN
        CALL SELF_ADD(YLGRID,GPFGMODERR(icomp))
        CALL SELF_ADD(YLSPEC,SPFGMODERR(icomp))
      ENDIF
    ENDIF
    clfile(1:14)='spmoderrinXXYY'
    WRITE(clfile(11:12),'(I2.2)')NUPTRA
    WRITE(clfile(13:14),'(I2.2)')icomp
    WRITE(NULOUT,*)'SAVMODERR: Saving model error in',clfile
    CALL WRMODERR(clfile,YLSPEC,YLGRID,jj)
  ENDDO
  NSTEP=NSTEP+istperr
ENDDO
IF (NSTEP/=NSTOP) CALL ABOR1('SAVMODERR: Error steps')
IF (icomp/=NDIM_MODERR) CALL ABOR1('SAVMODERR: Error components')
! Done
CALL DEALLOCATE_GRID(YLGRID)
CALL DEALLOCATE_SPEC(YLSPEC)

NSTEP=isave

IF (LHOOK) CALL DR_HOOK('SAVMODERR',1,ZHOOK_HANDLE)
END SUBROUTINE SAVMODERR
