SUBROUTINE SBS5TO3

!**** *SBS5TO3* - ROUTINE TO SUBSTRACT SP5 TO SPA3 SPECTRAL ARRAYS

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        *CALL* *SBS5TO3*

!        IMPLICIT ARGUMENTS :  SP5Ax and SPAx
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        F. Rabier  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-10-10
!        E. Holm  : 97-12-01 Ozone included
!        C. Fischer:00-06-20 add mean wind for LAM
!        P. Brousseau : 07-05-09 spectral Jb fields but model state in grid point space

USE PARKIND1 , ONLY : JPIM ,   JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFD2D, NFLEVG, NPROMA, NGPBLKS,NS3D
USE YOMMP    , ONLY : NPSP
USE YOMSP    , ONLY : SPA1     ,SP5A1    ,SPA2     ,SP5A2    ,&
            &SPDIV    ,SPVOR   ,SP5VOR   ,SP5DIV  ,SPT      ,SPQ      ,&
            &SPO3     ,SP5T    ,SP5Q     ,SP5O3
USE YOMCT0   , ONLY : LELAM
USE TRAJECTORY_MOD, ONLY: GET_TRAJ_GRID
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL, YQ, YO3
USE YOMGMV   , ONLY : YT5
USE YOMCVA   , ONLY : NVA3D
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
INTEGER(KIND=JPIM) :: ISTEP0
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL5(:,:,:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!     ------------------------------------------------------------------

!*    1.    SUB SP5Ax to SPAx.
!           ------------------

IF (LHOOK) CALL DR_HOOK('SBS5TO3',0,ZHOOK_HANDLE)


SPVOR(:,:) = SPVOR(:,:) - SP5VOR(:,:)
SPDIV(:,:) = SPDIV(:,:) - SP5DIV(:,:)
SPT(:,:)   = SPT(:,:)   - SP5T(:,:)

IF ( YQ%LSP) THEN
  SPQ(:,:) = SPQ(:,:) - SP5Q(:,:)
ENDIF

IF (YO3%LSP) THEN
  SPO3(:,:)   = SPO3(:,:)   - SP5O3(:,:)
ENDIF

! If humidity Jb field is spectral but model state is in grid point space
! Get initial time grid point trajectory and subtract them from fields
 
IF ((NS3D<NVA3D) .AND. (YQ%LGP)) THEN
    ALLOCATE (ZGMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
    ALLOCATE (ZGMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
    ALLOCATE (ZGFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
    ZGMV5=0._JPRB
    ZGMV5S=0._JPRB
    ZGFL5=0._JPRB
    ISTEP0=0
    CALL GET_TRAJ_GRID(ZGMV5,ZGMV5S,ZGFL5,ISTEP0)
    GFL(:,:,YQ%MP,:)=GFL(:,:,YQ%MP,:)-ZGFL5(:,:,YQ%MP,:)
    DEALLOCATE(ZGMV5)
    DEALLOCATE(ZGMV5S)
    DEALLOCATE(ZGFL5)
ENDIF




IF(NPSP == 1) THEN
  SPA2(:,1:NFD2D) = SPA2(:,1:NFD2D) - SP5A2(:,1:NFD2D)
ENDIF

IF (LELAM) SPA1(:,:) = SPA1(:,:) - SP5A1(:,:)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SBS5TO3',1,ZHOOK_HANDLE)

END SUBROUTINE SBS5TO3
