SUBROUTINE SBSBGS

!**** *SBSBGS* - Routine to SUBTRACT FIRST GUESS IN SPECTRAL SPACE TO
!****             SP3 ARRAYS

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SBSBGS*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!        SPJB
!        SP7A3
!        SP7A2
!        SP7A1

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        W. HECKLEY  *ECMWF*

!     Modifications.
!     --------------
!        Original : 91-02-20
!        Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!        Modified 95-12-12 by M.Hamrud  : Array syntax for vect. 
!        Modified 98-07-06 E.Andersson  : TOVS control variable
!        Modified 99-06-23 A. Horanyi   : ALADIN - mean wind 
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!-----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK
USE YOMMP    , ONLY : NPSP
USE YOMCT0   , ONLY : LELAM
USE YOMSPJB  , ONLY : SPJB, SP7A3    ,SP7A2    ,SP7A1
USE YOMVAR   , ONLY : LTOVSCV  ,LTSCV
USE YOMTVRAD , ONLY : TOVSCVX  ,TOVSCVBG
USE YOMANCS  , ONLY : RMDI
USE YOMJG    , ONLY : N_SPJB_VARS, SPJB_VARS_INFO
USE YOMTS    , ONLY : TSCVX    ,TSCVBG

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IPTBG, IPTJB, JFIELD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*    1.    SUBTRACT THE BACKGROUND FROM THE SPA*JB ARRAYS
!           ----------------------------------------------

IF (LHOOK) CALL DR_HOOK('SBSBGS',0,ZHOOK_HANDLE)

DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB
  IPTBG = SPJB_VARS_INFO(JFIELD)%IPTBG

  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    IF(NPSP>0) THEN
      SPJB%SP2D(:,IPTJB) = SPJB%SP2D(:,IPTJB) - SP7A2(:,IPTBG)
    ENDIF
  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA3) THEN
    SPJB%SP3D(:,:,IPTJB) = SPJB%SP3D(:,:,IPTJB) - SP7A3(:,:,IPTBG)
  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPGFL) THEN
    SPJB%SP3D(:,:,IPTJB) = SPJB%SP3D(:,:,IPTJB) - SP7A3(:,:,IPTBG)
  ELSE
    CALL ABOR1 ('CANNOT HANDLE GRIDPOINT BG FIELDS')
  ENDIF
ENDDO

!     1-D FIELDS
IF (LELAM) THEN
  SPJB%SP1D(:,:) = SPJB%SP1D(:,:) - SP7A1(:,:)
ENDIF

!     TOVS CONTROL VARIABLE
IF (LTOVSCV) THEN
  WHERE(TOVSCVBG(:,:) /= RMDI)TOVSCVX(:,:)=TOVSCVX(:,:)-TOVSCVBG(:,:)
ENDIF

!     TS CONTROL VARIABLE
IF(LTSCV) THEN
  WHERE(TSCVBG(:) /= RMDI) TSCVX(:)=TSCVX(:)-TSCVBG(:)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SBSBGS',1,ZHOOK_HANDLE)
END SUBROUTINE SBSBGS
