SUBROUTINE SBSFGS

!**** *SBSFGS* - Routine to SUBSTRACT FIRST GUESS IN SPECTRAL SPACE TO
!****             SP3 ARRAYS

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SBSFGS*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        F. RABIER  *ECMWF* (copied from ADDFGS)

!     Modifications.
!     --------------
!        Original : 94-06-24
!        Modified : 95-06-06 L. Isaksen - Reordering of spectral arrays
!        Modified : 96-05-05 L. Isaksen - Adapted for Message Passing
!        Modified : 97-11-13 JN.Thepaut - introducing NFD2D 
!        Modified : 97-05-23 L. Dragulanescu -- 3DVAR ALADIN
!        Modified : 98-07-06 E.Andersson- TOVS control variable
!        Modified : 00-08-04 Y.Tremolet - use module TRAJECTORY
!        Modified : 01-08-20 C. Fischer - get mean wind for LAM/Var
!        G. Desroziers : 02-09-16 also treat Aladin mean wind trajectory
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        G. Radnoti :  03-Dec-2004 Backward propagation in 3DFGAT
!        A.Trojakova   2-Jul-2006  Phasing CY31T1 (fix for ALADIN 3DVAR)
!     ------------------------------------------------------------------

USE PARKIND1 , ONLY : JPRB, JPIM
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LELAM, NPRINTLEV
USE YOMDIM   , ONLY : NFD2D, NFLEVG, NPROMA, NGPBLKS
USE YOMMP    , ONLY : NPSP
USE YOMGEM   , ONLY : NGPTOT
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SP5A3    ,SP5A2    ,&
 & SPA1     ,SP5A1   ,SPA3INI  ,SPA2INI  ,SPA1INI
USE YOMSPJB , ONLY : SP7A3    , SP7A2  ,SP7A1
USE YOMVAR   , ONLY : LTOVSCV, LMODERR, LPROPTL, LTSCV
USE YOMTVRAD , ONLY : TOVSCVX, TOVSCVX5
USE YOMANCS  , ONLY : RMDI
USE TRAJECTORY_MOD, ONLY: GET_TRAJ_SPEC, GET_TRAJ_GRID
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL     ,YGFLC
USE YOMGMV   , ONLY : YT5
USE YOMTS , ONLY : TSCVX  ,TSCVX5
USE YOMTRAJ, ONLY: LTRAJGP

IMPLICIT NONE
INTEGER(KIND=JPIM) :: JGFL, ISTEP0, IBL, ICEND, JKGLO
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL5(:,:,:,:)
LOGICAL :: LLPRINT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpnorm_gfl.intfb.h"
#include "add_moderr_tl.intfb.h"

!     ------------------------------------------------------------------

!*    1.    SUBSTRACT THE TRAJECTORY at initial time FROM THE SP3 ARRAYS
!           ------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SBSFGS',0,ZHOOK_HANDLE)

LLPRINT=NPRINTLEV >= 1

CALL GSTATS(15,0)
IF (.NOT. ALLOCATED(SPA3INI) .OR. .NOT. LPROPTL) CALL GET_TRAJ_SPEC(SP5A3,SP5A2,SP5A1)
CALL GSTATS(15,1)


CALL GSTATS(1863,0)
IF (LPROPTL .AND. ALLOCATED(SPA3INI)) THEN
!     3-D FIELDS 
  SPA3(:,:,:) = SPA3(:,:,:) - SP7A3(:,:,:)-SPA3INI(:,:,:)
!     2-D FIELDS
  IF(NPSP == 1) THEN
    SPA2(:,1:NFD2D) = SPA2(:,1:NFD2D) - SP7A2(:,1:NFD2D)-SPA2INI(:,1:NFD2D)
  ENDIF
ELSE

!     3-D SPECTRAL FIELDS 
  SPA3(:,:,:) = SPA3(:,:,:) - SP5A3(:,:,:)
!     2-D SPECTRAL FIELDS
  IF(NPSP == 1) THEN
    SPA2(:,1:NFD2D) = SPA2(:,1:NFD2D) - SP5A2(:,1:NFD2D)
  ENDIF

!     3-D GRIDPOINT FIELDS

! temporary fix due to abort in GET_TRAJ_GRID for LTRAJGP=F
  IF (YGFL%NUMFLDS5 > 0 .AND. LTRAJGP ) THEN
    IF (LLPRINT) THEN
      WRITE(NULOUT,'(A,I4)') &
       & ' SBSFGS : STATISTICS FOR ALL GFL FIELDS',YGFL%NUMFLDS
       CALL GPNORM_GFL
    ENDIF
    ALLOCATE (ZGMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
    ALLOCATE (ZGMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
    ALLOCATE (ZGFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
    ZGMV5=0._JPRB
    ZGMV5S=0._JPRB
    ZGFL5=0._JPRB

    ISTEP0=0
    CALL GSTATS(1863,2)
    CALL GET_TRAJ_GRID(ZGMV5,ZGMV5S,ZGFL5,KSTEP=ISTEP0)
    CALL GSTATS(1863,3)

    DO JGFL=1,YGFL%NUMFLDS
      IF (YGFLC(JGFL)%LT5 .AND. YGFLC(JGFL)%LGP) THEN
        DO JKGLO=1,NGPTOT,NPROMA
          ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
          IBL=(JKGLO-1)/NPROMA+1
          GFL(1:ICEND,:,YGFLC(JGFL)%MP,IBL) = &
               & GFL(1:ICEND,:,YGFLC(JGFL)%MP,IBL)- &
               & ZGFL5(1:ICEND,:,YGFLC(JGFL)%MP,IBL)
        ENDDO
!       GFL(:,:,YGFLC(JGFL)%MP,:) = &
!              & GFL(:,:,YGFLC(JGFL)%MP,:)-ZGFL5(:,:,YGFLC(JGFL)%MP,:)
      ENDIF
    ENDDO
    DEALLOCATE(ZGMV5)
    DEALLOCATE(ZGMV5S)
    DEALLOCATE(ZGFL5)
    IF (LLPRINT) THEN
      WRITE(NULOUT,'(A,I4)') &
       & ' SBSFGS : STATISTICS FOR ALL GFL FIELDS',YGFL%NUMFLDS
      CALL GPNORM_GFL
    ENDIF
  ENDIF

ENDIF

!     1-D FIELDS (MEAN WIND)

IF (LELAM) THEN
  IF  (LPROPTL .AND. ALLOCATED(SPA1INI)) THEN
    SPA1(:,:) = SPA1(:,:) - SP7A1(:,:)-SPA1INI(:,:)
  ELSE
    SPA1(:,:) = SPA1(:,:) - SP5A1(:,:)
  ENDIF
ENDIF

!     TOVS CONTROL VARIABLE
IF(LTOVSCV) THEN
  WHERE(TOVSCVX5(:,:) /= RMDI) TOVSCVX(:,:)=TOVSCVX(:,:)-TOVSCVX5(:,:)
ENDIF
!     TS CONTROL VARIABLE
IF (LTSCV) THEN
  WHERE(TSCVX5(:) /= RMDI) TSCVX(:)=TSCVX(:)-TSCVX5(:)
ENDIF

CALL GSTATS(1863,1)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SBSFGS',1,ZHOOK_HANDLE)
END SUBROUTINE SBSFGS
