SUBROUTINE SC2WRG(YD_PGPBUF,KJUMP,KSTA,KEND,PGP,KFSTA,KFEND,KPROMA)

!**** *SC2WRG* - Write out gridpoint array to work file

!     Purpose.  Write out the gridpoint array to work file
!     --------  or alternativly copy to grid-point buffer.
!               Optionally the fields are packed before writing out.

!**   Interface.
!     ----------
!        *CALL* *SC2WRG(...)*

!        Explicit arguments :  
!        --------------------  PGPBUF - grid-point buffer     (output)
!                              KJUMP - points to first column 
!                                    processed by this call   (input)
!                              KSTA    - start address        (input)
!                              KEND    - end adress           (input)
!                              PGP     - input array          (input)
!                              KFSTA   - starting field       (input,optional)
!                              KFEND   - ending field         (input,optional)

!        Implicit arguments :  None
!        --------------------

!     Method.
!     -------

!     Externals. 
!     ---------- FILLB    - interface to packing routines

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        MIO documentation

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-08-01
!        Modified : 92-12-20 M.Hamrud - packing and compacting of fields
!        Modified : 02-03-19 G Mozdzynski - support KFSTA, KFEND
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPROMA
USE YOMLUN   , ONLY : NULERR
USE GRIDPOINT_BUFFERS_MIX , ONLY : gridpoint_buffer

IMPLICIT NONE

TYPE(GRIDPOINT_BUFFER),INTENT(INOUT) :: YD_PGPBUF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KJUMP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN),OPTIONAL    :: KPROMA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGP(NPROMA,YD_PGPBUF%IFIELDS) 
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN)    :: KFSTA 
INTEGER(KIND=JPIM),OPTIONAL,INTENT(IN)    :: KFEND 

INTEGER(KIND=JPIM) :: IBLOCKS,IJUMP,IPCKUNIT,ISTA,IWORDS,JF,ISTAGPB,II,J,I1,I2
INTEGER(KIND=JPIM) :: IPROMA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "fillb.intfb.h"

!     ------------------------------------------------------------------

!*       1.    WRITE OUT GRIDPOINT ARRAY.
!              --------------------------

IF (LHOOK) CALL DR_HOOK('SC2WRG',0,ZHOOK_HANDLE)

IF(PRESENT(KPROMA))THEN
  IPROMA=KPROMA
ELSE
  IPROMA=NPROMA
ENDIF

IF( .NOT. ASSOCIATED (YD_PGPBUF%GPBUF)) THEN
  WRITE(NULERR,*)'SC2WRG:GRID-POINT BUFFER NOT ALLOCATED',YD_PGPBUF%CLNAME
  CALL ABOR1('SC2WRG:GRID-POINT BUFFER NOT ALLOCATED')
ENDIF

!*       1.1       CALCULATE LENGTHS

ISTAGPB = 0
IWORDS  = KEND-KSTA+1

!*       1.3       COPY TO GRIDPOINT BUFFER

IJUMP = KJUMP-1
IF (YD_PGPBUF%IPACK == 1) THEN
  ISTA = ISTAGPB+1+IJUMP*YD_PGPBUF%IFIELDS
ELSE

!  It is assumed that GT9BUF data always is read/written in
!  NPROMA chunks (except for the last potentially smaller chunk).
!  Restrictions are necessary because reading/writing of packed
!  data has to start on a word boundary.

  IBLOCKS = IJUMP/IPROMA
  IF (IBLOCKS*IPROMA /= IJUMP)&
   & CALL ABOR1('SC2WRG: INCONSISTENT WRITING OF PACKED DATA')  
  IPCKUNIT = (IPROMA+YD_PGPBUF%IPACK-1)/YD_PGPBUF%IPACK
  ISTA = ISTAGPB+1+IPCKUNIT*IBLOCKS*YD_PGPBUF%IFIELDS
ENDIF

IF(YD_PGPBUF%IPACK == 1) THEN
  IF( PRESENT(KFSTA).AND.PRESENT(KFEND) )THEN
    I1=KFSTA
    I2=KFEND
  ELSE
    I1=1
    I2=YD_PGPBUF%IFIELDS
  ENDIF
  II = ISTA
  DO JF=I1,I2
    II=ISTAGPB+1+IJUMP*YD_PGPBUF%IFIELDS+(JF-1)*IWORDS
    DO J=1,IWORDS
      YD_PGPBUF%GPBUF(II) = PGP(KSTA+J-1,JF)
      II = II+1
    ENDDO
  ENDDO
  ISTA = II
ELSE
  DO JF=1,YD_PGPBUF%IFIELDS
    CALL FILLB(IWORDS,PGP(KSTA,JF),YD_PGPBUF%GPBUF,YD_PGPBUF%IBLEN,ISTA, &
     & YD_PGPBUF%IPACK,6)  
  ENDDO
ENDIF

IF(ISTA > YD_PGPBUF%IBLEN+1) then
  WRITE(NULERR,*)YD_PGPBUF%CLNAME, ISTA,YD_PGPBUF%IBLEN
  CALL ABOR1('SC2WRG:GRID-POINT BUFFER TOO SMALL')
ENDIF

YD_PGPBUF%LFILLED = .TRUE.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SC2WRG',1,ZHOOK_HANDLE)
END SUBROUTINE SC2WRG
