SUBROUTINE SIGCHECK(LDSIGSTOP,LDSIGREST)

!**** *SIGCHECK* - Signal check

!     Purpose.
!     --------
!        Check for external signals requesting creation of restart files

!**   Interface.
!     ----------
!        *CALL* *SIGCHECK(LDSIGSTOP,LDSIGREST)*

!        Explicit arguments : (output)
!        --------------------  
!                LDSIGSTOP: set .TRUE. if stop signal received
!                LDSIGREST: set .TRUE. if restart file signal received

!        Implicit arguments :  MYPROC
!        --------------------

!     Method.
!     -------      Signals are unblocked.
!                  Master process checks flags.
!                  They may be set by signal processor (see IFSSIG)
!                  Values of flags are sent to all other processes

!     Externals.   IFSSIGB, MPL_SEND, MPL_RECV, MPL_BARRIER
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        David Dent  *ECMWF*

!     Modifications.
!     --------------
!        Original : 96-07-29
!        D.Dent     00-09-01 : convert from MPE to MPL
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : NPRCIDS  ,MYPROC
USE YOMCT0   , ONLY : NPROC
USE YOMTAG   , ONLY : MTAGSIG
USE YOMRES   , ONLY : NFLSTOP  ,NFLREST
USE YOMLUN   , ONLY : NULOUT
USE MPL_MODULE

IMPLICIT NONE

LOGICAL           ,INTENT(OUT)   :: LDSIGSTOP 
LOGICAL           ,INTENT(OUT)   :: LDSIGREST 
INTEGER(KIND=JPIM) :: IBUF(2)

INTEGER(KIND=JPIM) ::  IPROC, ITAG, JROC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!         unblock signals

IF (LHOOK) CALL DR_HOOK('SIGCHECK',0,ZHOOK_HANDLE)
CALL IFSSIGB

!         check for arrival of signals

LDSIGSTOP=.FALSE.

CALL GSTATS(658,0)
IF(MYPROC == 1) THEN
!        master process checks flags
  IF(NFLREST == 1) THEN
    LDSIGREST=.TRUE.
    WRITE(NULOUT,'(A)')' SIGNAL RECEIVED: WRITE RESTART FILES'
  ENDIF
  IF(NFLSTOP == 1) THEN
    LDSIGSTOP=.TRUE.
    WRITE(NULOUT,'(A)')' SIGNAL RECEIVED: STOP'
  ENDIF
  DO JROC=2,NPROC
    ITAG=MTAGSIG
    IBUF(1)=NFLSTOP
    IBUF(2)=NFLREST
    CALL MPL_SEND(IBUF,KDEST=NPRCIDS(JROC),KTAG=ITAG, &
     & CDSTRING='SIGCHECK:')  
  ENDDO
ELSE
  ITAG=MTAGSIG
  IPROC=1
  CALL MPL_RECV(IBUF,KSOURCE=NPRCIDS(IPROC),KTAG=ITAG, &
   & CDSTRING='SIGCHECK:')  
  NFLSTOP=IBUF(1)
  NFLREST=IBUF(2)
  IF(NFLREST == 1) THEN
    LDSIGREST=.TRUE.
  ENDIF
  IF(NFLSTOP == 1) THEN
    LDSIGSTOP=.TRUE.
  ENDIF
ENDIF
CALL GSTATS(658,1)
!       reset flag in case execution continues
NFLREST=0
IF(NPROC > 1) THEN
  CALL GSTATS(718,0)
  CALL MPL_BARRIER(CDSTRING='SIGCHECK:')
  CALL GSTATS(718,1)
ENDIF

IF (LHOOK) CALL DR_HOOK('SIGCHECK',1,ZHOOK_HANDLE)
END SUBROUTINE SIGCHECK
