SUBROUTINE SPAREORD(KFLEV,PSPFILE,PSPBUF,LD_FILE_TO_MODEL)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX    ,NSEFRE   ,NSPEC2G


!**** *SPAREORD*  - Reorder spectral data from old to new (grib-like) ordering
!                  or the reverse.

!     Purpose.
!     --------
!           Reorder spectral data structure from file ordering to model ordering 
!           or vice-versa.
!           SM : model ordering is the global spectrum ordering used in the model.

!**   Interface.
!     ----------
!        *CALL* *SPAREORD

!        Explicit arguments :
!        --------------------
!        PSPFILE          : Spectral array ready to be used for file
!        PSPBUF           : Spectral (buffer) array ready to be used in model
!        KFLEV            : Number of fields
!        LD_FILE_TO_MODEL : .TRUE. to order from file to model data structure

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!     None

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-06-10
!        R. El Khatib : 98-02-13 : direct and reverse order ; DM aspect.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib : 04-04-14 : optimization
!        R. El Ouaraini : 03-Oct-2006 : use IDIM0GG instead of NDIM0G !
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM)               :: IDIM0GG(0:NSMAX)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPFILE(KFLEV,NSEFRE) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPBUF(KFLEV,NSPEC2G) 
LOGICAL           ,INTENT(IN)    :: LD_FILE_TO_MODEL 
INTEGER(KIND=JPIM) :: II, IM, ISP, JLEV, JM, JN, IPOS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.   Rearrange spectral data structure 
!             ---------------------------------

IF (LHOOK) CALL DR_HOOK('SPAREORD',0,ZHOOK_HANDLE)

IPOS=1
DO JN=0,NSMAX
   IDIM0GG(JN)=IPOS
   IPOS=IPOS+(NSMAX+1-JN)*2
ENDDO

DO JLEV=1,KFLEV
  II=0
  DO JN=0,NSMAX
    IF (LD_FILE_TO_MODEL) THEN
      DO JM=-JN,-1
        ISP=IDIM0GG(-JM)+(JN+JM)*2 +1
        II = II + 1
        PSPBUF(JLEV,ISP)=PSPFILE(JLEV,II)
      ENDDO
      ISP=IDIM0GG(0)+JN*2
      II = II + 1
      PSPBUF(JLEV,ISP)=PSPFILE(JLEV,II)
      PSPBUF(JLEV,ISP+1)=0.0_JPRB
      DO JM=1,JN
        ISP=IDIM0GG(JM)+(JN-JM)*2
        II = II + 1
        PSPBUF(JLEV,ISP)=PSPFILE(JLEV,II)
      ENDDO
    ELSE
      DO JM=-JN,JN
        IM=ABS(JM)
        IF (JM < 0) THEN
          ISP=IDIM0GG(IM)+(JN-IM)*2 +1
        ELSE
          ISP=IDIM0GG(IM)+(JN-IM)*2
        ENDIF
        II = II + 1
        PSPFILE(JLEV,II)=PSPBUF(JLEV,ISP)
      ENDDO
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPAREORD',1,ZHOOK_HANDLE)
END SUBROUTINE SPAREORD
