SUBROUTINE SPCONVERT(LDMODEL_TO_FILE)

!**** *SPCONVERT*  - Spectral array conversion

!     Purpose.
!     --------
!        To convert model spectral fields into the fields actually written on file, 
!        NH variables are scaled.

!**   Interface.
!     ----------
!        *CALL* *SPCONVERT(LDMODEL_TO_FILE)

!        Explicit arguments :
!        --------------------
!           LDMODEL_TO_FILE : .TRUE. to convert from model to file
!                             .FALSE. to convert from file to model

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.    
!     ----------    

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib  *METEO-FRANCE*

!     Modifications.
!     --------------
!        Original : 98-03-30
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMSP    , ONLY : SPVOR    ,SPDIV
USE YOMLAP   , ONLY : NVALUE   ,RLAPIN   ,RLAPDI
USE YOMDIM   , ONLY : NSPEC2   ,NFLEVL   

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDMODEL_TO_FILE 
INTEGER(KIND=JPIM) :: JJ, JLEV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCONVERT',0,ZHOOK_HANDLE)
IF (LDMODEL_TO_FILE) THEN

!*       1.    FROM MODEL FIELDS TO FILE
!              -------------------------

  DO JLEV=1,NFLEVL
!OCL NOVREC
    DO JJ=1,NSPEC2
      SPVOR(JLEV,JJ)=SPVOR(JLEV,JJ)*RLAPIN(NVALUE(JJ)) ! VOR,DIV overwritten
      SPDIV(JLEV,JJ)=SPDIV(JLEV,JJ)*RLAPIN(NVALUE(JJ)) ! by PSI,KHY
    ENDDO
  ENDDO

ELSE

!*       2.    FROM FILE FIELDS TO MODEL
!              -------------------------

  DO JLEV=1,NFLEVL
!OCL NOVREC
    DO JJ=1,NSPEC2
      SPVOR(JLEV,JJ)=SPVOR(JLEV,JJ)*RLAPDI(NVALUE(JJ))
      SPDIV(JLEV,JJ)=SPDIV(JLEV,JJ)*RLAPDI(NVALUE(JJ))
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCONVERT',1,ZHOOK_HANDLE)
END SUBROUTINE SPCONVERT
