SUBROUTINE SPEC_CONCAT(YDSP1,YDSP2,YDSPOUT)

!   Purpose.
!   --------
!     Concatenate two spectral fields into one.

!   Author.
!   -------
!     Y. Tremolet

!   Modifications.
!   --------------
!     Original   02-Aug-2006
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCT0  , ONLY: LELAM
USE INDEXFIND_MOD, ONLY: INDXFIND
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN)    :: YDSP1
TYPE(SPECTRAL_FIELD), INTENT(IN)    :: YDSP2
TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSPOUT

LOGICAL, ALLOCATABLE :: LL2D(:), LL3D(:)
INTEGER(KIND=JPIM) :: jfi,jfo
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPEC_CONCAT',0,ZHOOK_HANDLE)

IF (YDSP1%ns1d+YDSP2%ns1d/=YDSPOUT%ns1d) CALL ABOR1('SPEC_CONCAT: Error 1D')
IF (YDSP1%ns2d+YDSP2%ns2d/=YDSPOUT%ns2d) CALL ABOR1('SPEC_CONCAT: Error 2D')
IF (YDSP1%ns3d+YDSP2%ns3d/=YDSPOUT%ns3d) CALL ABOR1('SPEC_CONCAT: Error 3D')
IF (YDSP1%nspec2/=YDSPOUT%nspec2 .OR. YDSP2%nspec2/=YDSPOUT%nspec2) &
 & CALL ABOR1('SPEC_CONCAT: Error resolution')

ALLOCATE(LL2D(YDSPOUT%ns2d))
ALLOCATE(LL3D(YDSPOUT%ns3d))
LL2D(:)=.false.
LL3D(:)=.false.

! 2D fields
DO jfi=1,YDSP1%ns2d
  jfo=INDXFIND(YDSPOUT%NGRIB2,YDSP1%NGRIB2(jfi))
  IF (jfo==0) CALL ABOR1('SPEC_CONCAT: 2D field not found')
  IF (LL2D(jfo)) CALL ABOR1('SPEC_CONCAT: duplicate 2D field')
  YDSPOUT%sp2d(:,jfo)=YDSP1%sp2d(:,jfi)
  LL2D(jfo)=.true.
ENDDO

DO jfi=1,YDSP2%ns2d
  jfo=INDXFIND(YDSPOUT%NGRIB2,YDSP2%NGRIB2(jfi))
  IF (jfo==0) CALL ABOR1('SPEC_CONCAT: 2D field not found')
  IF (LL2D(jfo)) CALL ABOR1('SPEC_CONCAT: duplicate 2D field')
  YDSPOUT%sp2d(:,jfo)=YDSP2%sp2d(:,jfi)
  LL2D(jfo)=.true.
ENDDO

! 3D fields
DO jfi=1,YDSP1%ns3d
  jfo=INDXFIND(YDSPOUT%NGRIB3,YDSP1%NGRIB3(jfi))
  IF (jfo==0) CALL ABOR1('SPEC_CONCAT: 3D field not found')
  IF (LL3D(jfo)) CALL ABOR1('SPEC_CONCAT: duplicate 3D field')
  YDSPOUT%sp3d(:,:,jfo)=YDSP1%sp3d(:,:,jfi)
  LL3D(jfo)=.true.
ENDDO 

DO jfi=1,YDSP2%ns3d
  jfo=INDXFIND(YDSPOUT%NGRIB3,YDSP2%NGRIB3(jfi))
  IF (jfo==0) CALL ABOR1('SPEC_CONCAT: 3D field not found')
  IF (LL3D(jfo)) CALL ABOR1('SPEC_CONCAT: duplicate 3D field')
  YDSPOUT%sp3d(:,:,jfo)=YDSP2%sp3d(:,:,jfi)
  LL3D(jfo)=.true.
ENDDO 

! 1D fields
IF (LELAM) THEN
  IF     (YDSP1%ns1d==YDSPOUT%ns1d) THEN
    YDSPOUT%sp1d(:,:)=YDSP1%sp1d(:,:)
  ELSEIF (YDSP2%ns1d==YDSPOUT%ns1d) THEN
    YDSPOUT%sp1d(:,:)=YDSP2%sp1d(:,:)
  ELSE
    CALL ABOR1 ('SPEC_CONCAT: Error LELAM')
  ENDIF
ENDIF

DEALLOCATE(LL2D,LL3D)

IF (LHOOK) CALL DR_HOOK('SPEC_CONCAT',1,ZHOOK_HANDLE)
END SUBROUTINE SPEC_CONCAT
