SUBROUTINE SPEC_SPLIT(YDSPIN,YDSP1,YDSP2)

!   Purpose.
!   --------
!     Split spectral fields into two spectral fields.

!   Author.
!   -------
!     Y. Tremolet

!   Modifications.
!   --------------
!     Original   02-Aug-2006
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCT0  , ONLY: LELAM
USE INDEXFIND_MOD, ONLY: INDXFIND
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN)    :: YDSPIN
TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSP1
TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSP2

LOGICAL, ALLOCATABLE :: LL2D(:), LL3D(:)
INTEGER(KIND=JPIM) :: jfo,jfi
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPEC_SPLIT',0,ZHOOK_HANDLE)

IF (YDSP1%ns1d+YDSP2%ns1d/=YDSPIN%ns1d) CALL ABOR1('SPEC_SPLIT: Error 1D')
IF (YDSP1%ns2d+YDSP2%ns2d/=YDSPIN%ns2d) CALL ABOR1('SPEC_SPLIT: Error 2D')
IF (YDSP1%ns3d+YDSP2%ns3d/=YDSPIN%ns3d) CALL ABOR1('SPEC_SPLIT: Error 3D')
IF (YDSP1%nspec2/=YDSPIN%nspec2 .OR. YDSP2%nspec2/=YDSPIN%nspec2) &
 & CALL ABOR1('SPEC_SPLIT: Error resolution')

ALLOCATE(LL2D(YDSPIN%ns2d))
ALLOCATE(LL3D(YDSPIN%ns3d))
LL2D(:)=.false.
LL3D(:)=.false.

! 2D fields
DO jfo=1,YDSP1%ns2d
  jfi=INDXFIND(YDSPIN%NGRIB2,YDSP1%NGRIB2(jfo))
  IF (jfi==0) CALL ABOR1('SPEC_SPLIT: 2D field not found')
  IF (LL2D(jfi)) CALL ABOR1('SPEC_CONCAT: duplicate 2D field')
  YDSP1%sp2d(:,jfo)=YDSPIN%sp2d(:,jfi)
  LL2D(jfi)=.true.
ENDDO

DO jfo=1,YDSP2%ns2d
  jfi=INDXFIND(YDSPIN%NGRIB2,YDSP2%NGRIB2(jfo))
  IF (jfi==0) CALL ABOR1('SPEC_SPLIT: 2D field not found')
  IF (LL2D(jfi)) CALL ABOR1('SPEC_CONCAT: duplicate 2D field')
  YDSP2%sp2d(:,jfo)=YDSPIN%sp2d(:,jfi)
  LL2D(jfi)=.true.
ENDDO

! 3D fields
DO jfo=1,YDSP1%ns3d
  jfi=INDXFIND(YDSPIN%NGRIB3,YDSP1%NGRIB3(jfo))
  IF (jfi==0) CALL ABOR1('SPEC_SPLIT: 3D field not found')
  IF (LL3D(jfi)) CALL ABOR1('SPEC_CONCAT: duplicate 3D field')
  YDSP1%sp3d(:,:,jfo)=YDSPIN%sp3d(:,:,jfi)
  LL3D(jfi)=.true.
ENDDO 

DO jfo=1,YDSP2%ns3d
  jfi=INDXFIND(YDSPIN%NGRIB3,YDSP2%NGRIB3(jfo))
  IF (jfi==0) CALL ABOR1('SPEC_SPLIT: 3D field not found')
  IF (LL3D(jfi)) CALL ABOR1('SPEC_CONCAT: duplicate 3D field')
  YDSP2%sp3d(:,:,jfo)=YDSPIN%sp3d(:,:,jfi)
  LL3D(jfi)=.true.
ENDDO 

! 1D fields
IF (LELAM) THEN
  IF     (YDSP1%ns1d==YDSPIN%ns1d) THEN
    YDSP1%sp1d(:,:)=YDSPIN%sp1d(:,:)
  ELSEIF (YDSP2%ns1d==YDSPIN%ns1d) THEN
    YDSP2%sp1d(:,:)=YDSPIN%sp1d(:,:)
  ELSE
    CALL ABOR1 ('SPEC_SPLIT: Error LELAM')
  ENDIF
ENDIF

DEALLOCATE(LL2D,LL3D)

IF (LHOOK) CALL DR_HOOK('SPEC_SPLIT',1,ZHOOK_HANDLE)
END SUBROUTINE SPEC_SPLIT
