SUBROUTINE SPECIMZERO

!**** *SPECIMZERO*  - 

!     PURPOSE.
!     --------
!     Set/check imaginary coef M=0 to 0 in SPA3 and SPA2.
!     (note that there is also a routine named SPEC_IMZERO which does the
!     same thing on some other classes of arrays).

!**   INTERFACE.
!     ----------
!       *CALL* *SPECIMZERO*

!        EXPLICIT ARGUMENTS
!        --------------------
!         none

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     AUTHOR.
!     -------
!        ???

!     MODIFICATIONS.
!     --------------
!     Original : ?
!     C.Soci   07-Oct-2004 set Im coef for n=0 to 0 added (LAM case)
!     K. Yessad (Dec 2008): add comments + cleanings
!-----------------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMMP   , ONLY: MYSETW, NPROCM, NPSP
USE YOMSP   , ONLY: SPA3, SPA2
USE YOMDIM  , ONLY: NSMAX, NUMP
USE YOMCT0  , ONLY: LELAM
USE YOMLAP  , ONLY: NSPZERO, MYMS
USE YEMLAP  , ONLY: NESPZERO, NESM0

!-----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: jn, im
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SPECIMZERO',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------------

IF (MYSETW == NPROCM(0)) THEN
  IF (LELAM) THEN
    IF (NPSP==1) THEN
      DO jn=1,2*(NSMAX+1)
        SPA2(NESPZERO(jn),:)=0.0_JPRB
      ENDDO 
    ENDIF
    DO jn=1,2*(NSMAX+1)
      SPA3(:,NESPZERO(jn),:)=0.0_JPRB
    ENDDO 
  ELSE
    IF (NPSP==1) THEN
      DO jn=0,NSMAX
        SPA2(NSPZERO(jn),:)=0.0_JPRB
      ENDDO
    ENDIF
    DO jn=0,NSMAX
      SPA3(:,NSPZERO(jn),:)=0.0_JPRB
    ENDDO
  ENDIF
ENDIF

! Set imaginary coef for n=0 to zero.
IF (LELAM) THEN
  IF (NPSP==1) THEN
    DO jn=1,NUMP
      im=MYMS(jn)
      SPA2(NESM0(im)+1,:)=0.0_JPRB
      SPA2(NESM0(im)+3,:)=0.0_JPRB
    ENDDO
  ENDIF
  DO jn=1,NUMP
    im=MYMS(jn)
    SPA3(:,NESM0(im)+1,:)=0.0_JPRB
    SPA3(:,NESM0(im)+3,:)=0.0_JPRB
  ENDDO
ENDIF

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SPECIMZERO',1,ZHOOK_HANDLE)
END SUBROUTINE SPECIMZERO
