SUBROUTINE SPREORD(KFLEV,PSPFILE,PSPBUF,LD_FILE_TO_MODEL)

!**** *SPREORD*  - Reorder spectral data from old to new (grib-like) ordering
!                  or the reverse.

!     Purpose.
!     --------
!           Reorder spectral data structure from file ordering to model ordering 
!           or vice-versa.
!           SM : model ordering is the global spectrum ordering used in the model.

!**   Interface.
!     ----------
!        *CALL* *SPREORD

!        Explicit arguments :
!        --------------------
!        PSPFILE          : Spectral array ready to be used for file
!        PSPBUF           : Spectral (buffer) array ready to be used in model
!        KFLEV            : Number of fields
!        LD_FILE_TO_MODEL : .TRUE. to order from file to model data structure

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!     None

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-06-10
!        R. El Khatib : 98-02-13 : direct and reverse order ; DM aspect.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSEFRE   ,NSPEC2G
USE YOMCT0   , ONLY : LELAM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPFILE(KFLEV,NSEFRE) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPBUF(KFLEV,NSPEC2G) 
LOGICAL           ,INTENT(IN)    :: LD_FILE_TO_MODEL 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "espareord.intfb.h"
#include "spareord.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPREORD',0,ZHOOK_HANDLE)
IF (LELAM) THEN
  CALL ESPAREORD(KFLEV,PSPFILE,PSPBUF,LD_FILE_TO_MODEL)
ELSE
  CALL SPAREORD(KFLEV,PSPFILE,PSPBUF,LD_FILE_TO_MODEL)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPREORD',1,ZHOOK_HANDLE)
END SUBROUTINE SPREORD
