SUBROUTINE SUALSPAJB(YDSPJB)

!**** *SUALSPAJB * - Routine to allocate space for SPA3JB & SPA2JB

!     Purpose.
!     --------
!           Allocate space for SPA3JB & SPA2JB

!**   Interface.
!     ----------
!        *CALL* *SUALSPAJB*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        L. Isaksen  *ECMWF*

!     Modifications.
!     --------------
!        Original : 04-07-01  Based on sualspa.F90
!        2005-06-08 Mike Fisher : Generalize Jb for GEMS
!        C. Fischer 05-05-23  add pointers to mean wind for Aladin
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LNHDYN
USE YOMDIM   , ONLY : NSMAX, NMSMAX
USE YOMCVA   , ONLY : NVA3D, NVA2D, NVA1D
USE YOMJG    , ONLY : N_SPJB_VARS, SPJB_VARS_INFO
USE YOMGRB   , ONLY : NGRBVO, NGRBD, NGRBT, NGRBQ, NGRBO3, NGRBLNSP,&
                      & NGRBSPD, NGRBSVD, NGRBGHG, NGRBGRG
USE SPECTRAL_FIELDS_MOD
USE YOM_YGFL, ONLY : JPGHG, JPGRG

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSPJB

INTEGER(KIND=JPIM) :: JFIELD, IPTJB, JGHG, JGRG
INTEGER(KIND=JPIM) :: igrb(N_SPJB_VARS)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUALSPAJB',0,ZHOOK_HANDLE)

!!!bc  Arrays SPSPDJB and SPSVDJB may be needed by LAM users
DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB

  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBVO) THEN
    igrb(iptjb)=NGRBVO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBD) THEN
    igrb(iptjb)=NGRBD
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT) THEN
    igrb(iptjb)=NGRBT
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBQ) THEN
    igrb(iptjb)=NGRBQ
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
    igrb(iptjb)=NGRBO3
  ELSEIF (ANY(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBGHG)) THEN
    DO JGHG=1,JPGHG
      IF(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBGHG(JGHG)) THEN
        igrb(iptjb)=NGRBGHG(JGHG)
      ENDIF
    ENDDO
  ELSEIF (ANY(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBGRG)) THEN
    DO JGRG=1,JPGRG
      IF(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBGRG(JGRG)) THEN
        igrb(iptjb)=NGRBGRG(JGRG)
      ENDIF
    ENDDO
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBLNSP) THEN
    igrb(NVA3D+iptjb)=NGRBLNSP
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBSPD .AND. LNHDYN ) THEN    
    igrb(iptjb)=NGRBSPD
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBSVD .AND. LNHDYN ) THEN
    igrb(iptjb)=NGRBSVD
  ENDIF
ENDDO

CALL ALLOCATE_SPEC(YDSPJB,NSMAX,NMSMAX,NVA3D,NVA2D,NVA1D,igrb)
YDSPJB=0.0_JPRB
IF (LHOOK) CALL DR_HOOK('SUALSPAJB',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

END SUBROUTINE SUALSPAJB

