SUBROUTINE VERDDER(KPROMA,KSTART,KPROF,KFLEV,PIN,POUT)

!**** *VERDDER*   Second-order vertical derivative

!     Purpose.
!     --------
!          This subroutine computes the 2nd vertical derivative (with respect
!          to eta) of a function given at full model
!          levels using a general scheme developed by means of finite-elements

!**   Interface.
!     ----------
!        *CALL* *VERDDER(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of work.
!          KPROF   - depth of work.
!          KFLEV   - number of model layers.
!          PIN     - Input field

!        OUTPUT:
!          POUT :    d**2 (PIN)/d(eta**2)   at each model level

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.

!     Author.
!     -------
!         K. Yessad and J. Vivoda (after VERDER).

!     Modifications.
!     --------------
!        Original : AUG 2007
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCVER  , ONLY : LVERTFE  ,RDDERI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIN(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) ::  IDB
! INTEGER(KIND=JPIM) ::  JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VERDDER',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF(.NOT.LVERTFE) CALL ABOR1(' ! LVERTFE=.F. IN VERDDER !!!')

! MATRIX MULTIPLY REPLACED BY BLAS ROUTINE

!DO JLEV=1,KFLEV
!  DO JROF=KSTART,KPROF
!    POUT(JROF,JLEV)=_ZERO_
!  ENDDO
!ENDDO
!DO JLE=1,KFLEV
!  DO JLEV=1,KFLEV
!    DO JROF=KSTART,KPROF
!      POUT(JROF,JLEV)=POUT(JROF,JLEV)+RDDERI(JLEV,JLE)*PIN(JROF,JLE)
!    ENDDO
!  ENDDO
!ENDDO

IDB=SIZE(RDDERI,DIM=1)
IF (KPROF >= KSTART) THEN 
  CALL DGEMM('N','T',KPROF-KSTART+1,KFLEV,KFLEV, &
   & 1.0_JPRB,PIN,KPROMA,RDDERI,IDB,0.0_JPRB,POUT,KPROMA)  
ENDIF
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VERDDER',1,ZHOOK_HANDLE)
END SUBROUTINE VERDDER
