SUBROUTINE WRITE_CTLVEC_GRIB(CDFILE,YDCV)

!     Purpose.
!     --------
!        Save control vector in GRIB file.

!     Author.
!     -------
!        Y.Tremolet

!     Modifications.
!     --------------
!        Original : 04-Nov-2004
!        2005-03-07 M.Fisher : write wavelet jb
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULOUT
USE YOMCVA  , ONLY: NGRBVAR
USE YOMVAR  , ONLY: NUPTRA, MUPTRA
USE YOMMODEL_ERROR, ONLY: NGRBMERRCTL, NTYPE_MODERR
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

CHARACTER(LEN=*)    , INTENT(IN) :: CDFILE
TYPE(CONTROL_VECTOR), INTENT(IN) :: YDCV

TYPE(SPECTRAL_FIELD) :: YLSP
INTEGER(KIND=JPIM) :: jj,ii,il,isecloc(44)
CHARACTER(LEN=1) :: clmode
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "write_spec_grib.intfb.h"
#include "write_wavelet_initcv_grib.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRITE_CTLVEC_GRIB',0,ZHOOK_HANDLE)

clmode="w"

! Initial condition
! -----------------
IF (YDCV%linitcv) THEN
  IF (YDCV%lwaveletjb) THEN
    CALL WRITE_WAVELET_INITCV_GRIB (YDCV,CDFILE)
  ELSE
    isecloc(:)  = -999
    isecloc(37) = 20
    isecloc(39) = 33
    isecloc(42) = NUPTRA
    isecloc(43) = MUPTRA

    CALL ALLOCATE_SPEC(YLSP,YDCV%nsmax,YDCV%nmsmax, &
                     & YDCV%ns3d,YDCV%ns2d,YDCV%ns1d,NGRBVAR)
    YLSP=YDCV%initcv

    CALL WRITE_SPEC_GRIB(CDFILE,YLSP,clmode,isecloc)
    clmode="a"

    CALL DEALLOCATE_SPEC(YLSP)
  ENDIF
ENDIF

! Model error
! -----------
IF (YDCV%nmoderr>0) THEN
  isecloc(:)  = -999
  isecloc(37) = 25
  isecloc(39) = 35
  isecloc(43) = YDCV%nmoderr
  isecloc(44) = NTYPE_MODERR

  CALL ALLOCATE_SPEC(YLSP,YDCV%nserr,0,YDCV%nve3d,YDCV%nve2d,0,NGRBMERRCTL)
  ii=0
  il=YLSP%nspsizel
  DO jj=1,YDCV%nmoderr
    isecloc(42)=jj
    YLSP=YDCV%MODERR(ii+1:ii+il)
    ii=ii+il

    CALL WRITE_SPEC_GRIB(CDFILE,YLSP,clmode,isecloc)
    clmode="a"

  ENDDO
  CALL DEALLOCATE_SPEC(YLSP)
ENDIF

! Parameter estimation
! --------------------
IF (YDCV%NPARA>0) THEN
  WRITE(NULOUT,*)'WRITE_CTLVEC_GRIB: No GRIB for control parameters'
ENDIF

! TOVS
! ----
IF (YDCV%ltovscv) THEN
  WRITE(NULOUT,*)'WRITE_CTLVEC_GRIB: No GRIB for TOVS control variable'
ENDIF

! TS
! --
IF (YDCV%ltscv) THEN
  WRITE(NULOUT,*)'WRITE_CTLVEC_GRIB: No GRIB for TS control variable'
ENDIF

! LAM1D MEAN WIND
! ---------------
IF (YDCV%lam1d) THEN
  WRITE(NULOUT,*)'WRITE_CTLVEC_GRIB: No GRIB for LAM1D control variable'
ENDIF

IF (LHOOK) CALL DR_HOOK('WRITE_CTLVEC_GRIB',1,ZHOOK_HANDLE)
END SUBROUTINE WRITE_CTLVEC_GRIB
