SUBROUTINE WRITE_WAVELET_INITCV_GRIB (YDCV,CDFILE)
!-----------------------------------------------------------------------
!   WRITE_WAVELET_INITCV_GRIB - Write out initcv part of the control vector in GRIB format

!   Purpose. Write out the analysis control vector (initcv part) in GRIB format
!   --------

!   Interface.
!   ----------
!      CALL WRITE_WAVELET_INITCV_GRIB (YDCV,CDFILE)

!      Explicit arguments: 

!      Inputs: YDCV         -- The control vector to be written.
!              CDFILE       -- The filename prefix.
!                              and S=-(planc1)' log_2[(J'')^-1] planc1 /2 ?

!   Externals.   GATHERGPF
!   ----------   

!   Reference.
!   ----------
!       None yet!

!   Author.
!   -------
!       Mike Fisher  *ECMWF*

!   Modifications.
!   --------------
!       Original      2005/03/03
!       2008-02-08 M Fisher - bugfix
!-----------------------------------------------------------------------

USE PARKIND1,ONLY : JPIM, JPRB
USE YOMHOOK ,ONLY : LHOOK, DR_HOOK
USE YOMLUN, ONLY : NULERR
USE YOMMP , ONLY : MYPROC
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG
USE YOMCVA   , ONLY : NVA3D    ,NVA2D
USE YOMMSC, ONLY : NINTLEN
USE YOMWAVELET, ONLY: LJBWAVELET,  GRID_DEFINITION, N_WAVELET_SCALES
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE YOMGRB, ONLY: RSEC2, RSEC3
USE GRIB_HEADER_MIX
USE YOMJG    , ONLY : &
 & N_SPJB_VARS, SPJB_VARS_INFO

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN) :: YDCV
CHARACTER(LEN=*), INTENT(IN) :: CDFILE
TYPE(GRIB_HEADER) :: YLHEADER

REAL(KIND=JPRB), ALLOCATABLE :: ZGLOBAL(:,:),ZLOCAL(:)
INTEGER(KIND=JPIM) :: ICODES_3D(NVA3D), ICODES_2D(NVA2D)
INTEGER(KIND=JPIM), ALLOCATABLE :: IGRIB(:)

INTEGER(KIND=JPIM) :: JSCALE, INFIELDS, IGPTOT, IGPTOTG, JFLD, JKGLO, ICEND, &
                    & IBL, ISTART, IOPROC, IWORD, IUNIT, IERR, JL, II, J
INTEGER(KIND=JPIM) :: I2D, I3D

REAL(KIND=JPRB) :: ZHOOK_HANDLE
CHARACTER(LEN=128) :: CLFILE
CHARACTER(LEN=8) :: CLSUFFIX

INTERFACE
#include "trans_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "gathergpf.intfb.h"
#include "pregrbenc.intfb.h"

IF (LHOOK) CALL DR_HOOK('WRITE_WAVELET_INITCV_GRIB',0,ZHOOK_HANDLE)

IF (.NOT.LJBWAVELET) CALL ABOR1 ('WRITE_WAVELET_INITCV_GRIB ONLY WORKS FOR WAVELET JB')

IOPROC=1
INFIELDS=NFLEVG*NVA3D+NVA2D

!stj
I3D=1
I2D=1
DO J=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(J)%L_IN_SPA2) THEN
    ICODES_2D(I2D)=SPJB_VARS_INFO(J)%IGRIBCODE
    I2D=I2D+1
!  ELSEIF (SPJB_VARS_INFO(J)%L_IN_SPA3) THEN
  ELSE
    ICODES_3D(I3D)=SPJB_VARS_INFO(J)%IGRIBCODE
    I3D=I3D+1
  ENDIF
ENDDO

!IF (NVA3D==5 .AND. NVA2D==1) THEN
!  ICODES_3D=(/NGRBVO,NGRBD,NGRBT,NGRBQ,NGRBO3/)
!  ICODES_2D=(/NGRBLNSP/)
!ELSE
!  CALL ABOR1 ('DON''T KNOW HOW TO ALLOCATE GRIB CODES') !time to do things properly!
!ENDIF
!stj

DO JSCALE=1,N_WAVELET_SCALES+1
  IGPTOT =GRID_DEFINITION(JSCALE)%NGPTOT
  IGPTOTG=GRID_DEFINITION(JSCALE)%NGPTOTG

  ALLOCATE (ZLOCAL(IGPTOT*INFIELDS))
  ALLOCATE (ZGLOBAL(IGPTOTG,INFIELDS))

  II=0
  DO JFLD=1,INFIELDS
    DO JKGLO=1,IGPTOT,NPROMA
      ICEND=MIN(NPROMA,IGPTOT-JKGLO+1)
      IBL=(JKGLO-1)/NPROMA+1

      ISTART = YDCV%ibgn_scale(JSCALE) +(JFLD-1)*NPROMA +(IBL-1)*NPROMA*INFIELDS

      ZLOCAL(II+1:II+ICEND) = YDCV%initcv(ISTART:ISTART-1+ICEND)
      II=II+ICEND
    ENDDO
  ENDDO

  CALL GATHERGPF(ZLOCAL,ZGLOBAL,INFIELDS,IOPROC,JSCALE)
  DEALLOCATE (ZLOCAL)

  IF (MYPROC==IOPROC) THEN
    WRITE (CLSUFFIX,'(''_scale'',I2.2)') JSCALE
    CLFILE=TRIM(CDFILE)//CLSUFFIX
    CALL PBOPEN(iunit,TRIM(CLFILE),'w',ierr)
    IF (ierr /= 0) THEN
      write(NULERR,*)'WRITE_WAVELET_INITCV_GRIB: PBOPEN error code ',ierr
      CALL ABOR1('WRITE_WAVELET_INITCV_GRIB: error opening grib file')
    ENDIF

    DO JFLD=1,NVA3D
      DO JL=1,NFLEVG
        CALL WRITE_IT (ICODES_3D(JFLD),JL,ZGLOBAL(:,JL+(JFLD-1)*NFLEVG))
      ENDDO
    ENDDO

    DO JFLD=1,NVA2D
      CALL WRITE_IT (ICODES_2D(JFLD),1_JPIM,ZGLOBAL(:,JFLD+NFLEVG*NVA3D))
    ENDDO

    CALL PBCLOSE(iunit,ierr)
    IF (ierr /= 0) THEN
      write(NULERR,*)'WRITE_WAVELET_INITCV_GRIB: PBCLOSE error code ',ierr
      CALL ABOR1('WRITE_WAVELET_INITCV_GRIB: error closing grib file')
    ENDIF
  ENDIF

  DEALLOCATE (ZGLOBAL)
ENDDO

IF (LHOOK) CALL DR_HOOK('WRITE_WAVELET_INITCV_GRIB',1,ZHOOK_HANDLE)

CONTAINS
  SUBROUTINE WRITE_IT (KCODE,KLEV,PGRID)
    INTEGER(KIND=JPIM), INTENT(IN) :: KCODE, KLEV
    REAL(KIND=JPRB), INTENT(INOUT) :: PGRID(:)

    REAL(KIND=JPRB) :: ZHOOK_HANDLE
    REAL(KIND=JPRB) :: zzz(1),zmin,zmax,zrat
    REAL(KIND=JPRB), ALLOCATABLE :: ZMU(:),ZGRID(:)
    INTEGER(KIND=JPIM) :: IDGLG

    IF (LHOOK) CALL DR_HOOK('WRITE_WAVELET_INITCV_GRIB:WRITE_IT',0,ZHOOK_HANDLE)

    IDGLG=GRID_DEFINITION(JSCALE)%NDGLG

    CALL ALLOCATE_GRIB_HEADER(YLHEADER)
    ALLOCATE(IGRIB(IGPTOTG*2))
    ALLOCATE(ZMU(IDGLG))

!!!!!!! GRIBEX BUG: copy and overdimension input array:
    ALLOCATE (ZGRID(SIZE(PGRID)+1024))
    ZGRID(1:SIZE(PGRID)) = PGRID(:)
    ZGRID(1+SIZE(PGRID):) = 0.0_JPRB

    CALL TRANS_INQ (KRESOL=GRID_DEFINITION(JSCALE)%NRESOL_ID, PMU=ZMU)

    CALL PREGRBENC(KCODE,KLEV,'GG','ML', &
           & YLHEADER%nsec0,YLHEADER%nsec1,YLHEADER%nsec2gg, &
           & YLHEADER%nsec3,YLHEADER%nsec4gg, &
           & zzz,KPTS=IGPTOTG,KDGLG=IDGLG,&
           & KLOENG=GRID_DEFINITION(JSCALE)%NLOENG, &
           & PFIRSTLAT=ASIN(ZMU(1)),PLASTLAT=ASIN(ZMU(IDGLG)))

    ! Copied from PRESET_GRIB_HEAD /MF
    ! I know the following is horrible/MH
    IF (ALLOCATED(RSEC2)) THEN
      YLHEADER%rsec2(1:SIZE(RSEC2))=RSEC2(:)
      YLHEADER%rsec3(:)=RSEC3(:)
    ELSE
      CALL ABOR1('GRIB_HEADER_MIX:PRESET_GRIB_HEAD: RSEC2 not allocated')
    ENDIF


    !Security for problems in GRIBEX with small range of values
    ZMAX=MAXVAL(ZGRID(1:IGPTOTG))
    ZMIN=MINVAL(ZGRID(1:IGPTOTG))
    IF(ZMAX > ZMIN .AND. ABS(ZMAX) > 1.E-20_JPRB) THEN
      ZRAT=ABS((ZMAX-ZMIN)/ZMAX)
      IF(ZRAT < 1.E-3_JPRB) THEN
        ZGRID(1:IGPTOTG)=ZMIN
      ENDIF
    ENDIF

    CALL GRIBEX(YLHEADER%nsec0,YLHEADER%nsec1,YLHEADER%nsec2gg, &
       & YLHEADER%rsec2,YLHEADER%nsec3,YLHEADER%rsec3, &
       & YLHEADER%nsec4gg,ZGRID,IGPTOTG,IGRIB,SIZE(IGRIB),iword,'C',ierr)  

    CALL PBWRITE(iunit,igrib,iword*NINTLEN,ierr)

    IF (ierr < 0) THEN
      write(NULERR,*)'WRITE_WAVELET_INITCV_GRIB: PBWRITE error code ',ierr
      CALL ABOR1('WRITE_WAVELET_INITCV_GRIB: PBWRITE failed')
    ENDIF

    DEALLOCATE (ZGRID)
    DEALLOCATE(ZMU)
    DEALLOCATE(IGRIB)
    CALL DEALLOCATE_GRIB_HEADER(YLHEADER)

    IF (LHOOK) CALL DR_HOOK('WRITE_WAVELET_INITCV_GRIB:WRITE_IT',1,ZHOOK_HANDLE)
  END SUBROUTINE  WRITE_IT

END SUBROUTINE WRITE_WAVELET_INITCV_GRIB
