SUBROUTINE ADD_MODERR_AD(YDSPERR,YDGPERR,KSTEP,YDSPERR1,YDGPERR1)

!     Purpose.
!     --------
!       Adjoint of adding model error as an increment.

!     Author.
!     -------
!       Y.Tremolet

!     Modifications.
!     --------------
!        Original : 05-Jul-2006
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMSP   , ONLY: SPA3, SPA2
USE YOMGMV  , ONLY: GMV, GMVS
USE YOMGFL  , ONLY: GFL
USE YOMDIM  , ONLY: NSPEC2, NFLEVL, NPROMA, NGPBLKS, NFLEVG
USE YOMGEM  , ONLY: NGPTOT
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, MSTEPERR, &
                        & MERRSP3, MERRSP2, MERRGFL, MERRGMV, MERRGMVS, &
                        & NORDER_MODERR, NSTEP_MODERR, FSTEPERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(SPECTRAL_FIELD),  INTENT(INOUT) :: YDSPERR(:)
TYPE(GRIDPOINT_FIELD), INTENT(INOUT) :: YDGPERR(:)
INTEGER(KIND=JPIM), INTENT(IN) :: KSTEP
TYPE(SPECTRAL_FIELD),  OPTIONAL,  INTENT(INOUT) :: YDSPERR1(:)
TYPE(GRIDPOINT_FIELD), OPTIONAL,  INTENT(INOUT) :: YDGPERR1(:)
INTEGER(KIND=JPIM) :: jj,jf,jl,jb,jg,ilen,ii,jorder
REAL(KIND=JPRB) :: zz
LOGICAL            :: llhyb
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ADD_MODERR_AD',0,ZHOOK_HANDLE)

IF (NTYPE_MODERR/=2.AND.NTYPE_MODERR/=4.AND.NTYPE_MODERR/=5) &
  & CALL ABOR1('ADD_MODERR_AD: Error NTYPE_MODERR')

IF (MSTEPERR(KSTEP)>0) THEN

! Periodic forcing
  IF (NTYPE_MODERR==2.AND.NORDER_MODERR>0) THEN
    DO jorder=2*NORDER_MODERR,1,-1
      ii=MSTEPERR(KSTEP)+jorder*NSTEP_MODERR
      zz=FSTEPERR(kstep,jorder)

      DO jf=1,YDSPERR(ii)%NS3D
        jg=MERRSP3(jf)
        DO jj=1,NSPEC2
          DO jl=1,NFLEVL
            YDSPERR(ii)%SP3D(jl,jj,jf) = YDSPERR(ii)%SP3D(jl,jj,jf) &
                                     & + zz * SPA3(jl,jj,jg)
          ENDDO
        ENDDO
      ENDDO

      DO jf=1,YDSPERR(ii)%NS2D
        jg=MERRSP2(jf)
        DO jj=1,NSPEC2
          YDSPERR(ii)%SP2D(jj,jf) = YDSPERR(ii)%SP2D(jj,jf) + zz * SPA2(jj,jg)
        ENDDO
      ENDDO

      DO jb=1,NGPBLKS
        ilen=MIN(NPROMA, NGPTOT-(jb-1)*NPROMA)

        DO jf=1,YDGPERR(ii)%NG3D
          IF (MERRGMV(jf)>0) THEN
            jg=MERRGMV(jf)
            DO jl=1,NFLEVG
              DO jj=1,ilen
                YDGPERR(ii)%GP3D(jj,jl,jf,jb) = YDGPERR(ii)%GP3D(jj,jl,jf,jb) &
                                            & + zz * GMV(jj,jl,jg,jb)
              ENDDO
            ENDDO
          ELSEIF (MERRGFL(jf)>0) THEN
            jg=MERRGFL(jf)
            DO jl=1,NFLEVG
              DO jj=1,ilen
                YDGPERR(ii)%GP3D(jj,jl,jf,jb) = YDGPERR(ii)%GP3D(jj,jl,jf,jb) &
                                            & + zz * GFL(jj,jl,jg,jb)
              ENDDO
            ENDDO
          ELSE
            CALL ABOR1('ADD_MODERR_TL: Field not found in GMV or GFL')
          ENDIF
        ENDDO

        DO jf=1,YDGPERR(ii)%NG2D
          IF (MERRGMVS(jf)>0) THEN
            jg=MERRGMVS(jf)
            DO jj=1,ilen
              YDGPERR(ii)%GP2D(jj,jf,jb) = YDGPERR(ii)%GP2D(jj,jf,jb)  &
                                       & + zz * GMVS(jj,jg,jb)
            ENDDO
          ELSE
            CALL ABOR1('ADD_MODERR_TL: Field not found in GMVS')
          ENDIF
        ENDDO
      ENDDO
    ENDDO
  ENDIF

  LLHYB= PRESENT(YDSPERR1)
  ii=MSTEPERR(KSTEP)

! Spectral fields
  DO jf=1,YDSPERR(ii)%NS3D
    jg=MERRSP3(jf)
    DO jj=1,NSPEC2
      DO jl=1,NFLEVL
        YDSPERR(ii)%SP3D(jl,jj,jf) = YDSPERR(ii)%SP3D(jl,jj,jf) + SPA3(jl,jj,jg)
        IF (LLHYB) THEN
          SPA3(jl,jj,jg)=SPA3(jl,jj,jg)+YDSPERR1(ii)%SP3D(jl,jj,jf)
          YDSPERR1(ii)%SP3D(jl,jj,jf)=0.0_JPRB
        ENDIF
      ENDDO
    ENDDO
  ENDDO

  DO jf=1,YDSPERR(ii)%NS2D
    jg=MERRSP2(jf)
    DO jj=1,NSPEC2
      YDSPERR(ii)%SP2D(jj,jf) = YDSPERR(ii)%SP2D(jj,jf) + SPA2(jj,jg)
      IF (LLHYB) THEN
        SPA2(jj,jg)=SPA2(jj,jg)+YDSPERR1(ii)%SP2D(jj,jf)
        YDSPERR1(ii)%SP2D(jj,jf)=0.0_JPRB
      ENDIF
    ENDDO
  ENDDO

! Gridpoint fields
  DO jb=1,NGPBLKS
    ilen=MIN(NPROMA, NGPTOT-(jb-1)*NPROMA)

    DO jf=1,YDGPERR(ii)%NG3D
      IF (MERRGMV(jf)>0) THEN
        jg=MERRGMV(jf)
        DO jl=1,NFLEVG
          DO jj=1,ilen
            YDGPERR(ii)%GP3D(jj,jl,jf,jb)=YDGPERR(ii)%GP3D(jj,jl,jf,jb)+GMV(jj,jl,jg,jb)
            IF (LLHYB) THEN
              GMV(jj,jl,jg,jb)=GMV(jj,jl,jg,jb)+YDGPERR1(ii)%GP3D(jj,jl,jf,jb)
              YDGPERR1(ii)%GP3D(jj,jl,jf,jb)=0.0_JPRB
            ENDIF
          ENDDO
        ENDDO
      ELSEIF (MERRGFL(jf)>0) THEN
        jg=MERRGFL(jf)
        DO jl=1,NFLEVG
          DO jj=1,ilen
            YDGPERR(ii)%GP3D(jj,jl,jf,jb)=YDGPERR(ii)%GP3D(jj,jl,jf,jb)+GFL(jj,jl,jg,jb)
            IF (LLHYB) THEN
              GFL(jj,jl,jg,jb)=GFL(jj,jl,jg,jb)+YDGPERR1(ii)%GP3D(jj,jl,jf,jb)
              YDGPERR1(ii)%GP3D(jj,jl,jf,jb)=0.0_JPRB
            ENDIF
          ENDDO
        ENDDO
      ELSE
        CALL ABOR1('ADD_MODERR_AD: Field not found in GMV or GFL')
      ENDIF
    ENDDO

    DO jf=1,YDGPERR(ii)%NG2D
      IF (MERRGMVS(jf)>0) THEN
        jg=MERRGMVS(jf)
        DO jj=1,ilen
          YDGPERR(ii)%GP2D(jj,jf,jb) = YDGPERR(ii)%GP2D(jj,jf,jb) + GMVS(jj,jg,jb)
          IF (LLHYB) THEN
            GMVS(jj,jg,jb)=GMVS(jj,jg,jb)+YDGPERR1(ii)%GP2D(jj,jf,jb)
            YDGPERR1(ii)%GP2D(jj,jf,jb)=0._JPRB
          ENDIF
        ENDDO
      ELSE
        CALL ABOR1('ADD_MODERR_AD: Field not found in GMVS')
      ENDIF
    ENDDO
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('ADD_MODERR_AD',1,ZHOOK_HANDLE)
END SUBROUTINE ADD_MODERR_AD
