SUBROUTINE ADD_MODERR_TL(YDSPERR,YDGPERR,KSTEP,YDSPERR1,YDGPERR1)

!     Purpose.
!     --------
!       Add model error to model state (TL).

!     Author.
!     -------
!       Y.Tremolet

!     Modifications.
!     --------------
!        Original : 05-Jul-2006
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMSP   , ONLY: SPA3, SPA2
USE YOMGMV  , ONLY: GMV, GMVS
USE YOMGFL  , ONLY: GFL
USE YOMDIM  , ONLY: NSPEC2, NFLEVL, NPROMA, NGPBLKS, NFLEVG
USE YOMGEM  , ONLY: NGPTOT
USE YOMLUN  , ONLY: NULOUT
USE YOMCT0  , ONLY: NCONF
USE YOMCT3  , ONLY: NSTEP
USE YOMMODEL_ERROR, ONLY: NPRTMODERR, NTYPE_MODERR, MSTEPERR, &
                        & MERRSP3, MERRSP2, MERRGFL, MERRGMV, MERRGMVS, &
                        & NORDER_MODERR, NSTEP_MODERR, FSTEPERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(SPECTRAL_FIELD),  INTENT(IN) :: YDSPERR(:)
TYPE(GRIDPOINT_FIELD), INTENT(IN) :: YDGPERR(:)
INTEGER(KIND=JPIM), INTENT(IN) :: KSTEP
TYPE(SPECTRAL_FIELD), OPTIONAL,  INTENT(OUT) :: YDSPERR1(:)
TYPE(GRIDPOINT_FIELD), OPTIONAL, INTENT(OUT) :: YDGPERR1(:)
INTEGER(KIND=JPIM) :: jj,jf,jl,jb,jg,ilen,ii,jorder
REAL(KIND=JPRB) :: zz
LOGICAL :: llprt, llevs, llhyb
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "spnorm.intfb.h"
#include "gpnorm_gfl.intfb.h"
#include "gridpoint_norm.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ADD_MODERR_TL',0,ZHOOK_HANDLE)

IF (NTYPE_MODERR/=2.AND.NTYPE_MODERR/=4.AND.NTYPE_MODERR/=5) &
  & CALL ABOR1('ADD_MODERR_TL: Error NTYPE_MODERR')

ii=MSTEPERR(KSTEP)

IF (ii>0) THEN
! Prints equivalents to the ones in RDFPINC
  llprt=(NCONF==1).AND.(NPRTMODERR>=1)
  llevs=(NPRTMODERR>=2)
  llhyb= PRESENT(YDSPERR1)

  IF (llprt) THEN
    WRITE(NULOUT,*)'ADD_MODERR_TL: Norms of high res FG'
    CALL SPNORM(0)
    CALL GPNORM_GFL
    WRITE(NULOUT,*)'ADD_MODERR_TL: Norms of low res INCR'
    CALL  SPECTRAL_NORM(YDSPERR(ii),'MODERR')
    CALL GRIDPOINT_NORM(YDGPERR(ii),'MODERR',llevs)
  ENDIF

! Spectral fields
  DO jf=1,YDSPERR(ii)%NS3D
    jg=MERRSP3(jf)
    DO jj=1,NSPEC2
      DO jl=1,NFLEVL
        IF (LLHYB) YDSPERR1(ii)%SP3D(jl,jj,jf)=SPA3(jl,jj,jg)
        SPA3(jl,jj,jg) = SPA3(jl,jj,jg) + YDSPERR(ii)%SP3D(jl,jj,jf)
      ENDDO
    ENDDO
  ENDDO
  DO jf=1,YDSPERR(ii)%NS2D
    jg=MERRSP2(jf)
    DO jj=1,NSPEC2
      IF (LLHYB) YDSPERR1(ii)%SP2D(jj,jf)=SPA2(jj,jg)
      SPA2(jj,jg) = SPA2(jj,jg) + YDSPERR(ii)%SP2D(jj,jf)
    ENDDO
  ENDDO

  if (llhyb) then
    write(nulout,*)'spectral norms after updating increments in last simul, nstep=',nstep
    call spnorm(0)
  endif

! Gridpoint fields
  DO jb=1,NGPBLKS
    ilen=MIN(NPROMA, NGPTOT-(jb-1)*NPROMA)

    DO jf=1,YDGPERR(ii)%NG3D
      IF (MERRGMV(jf)>0) THEN
        jg=MERRGMV(jf)
        DO jl=1,NFLEVG
          DO jj=1,ilen
            IF (LLHYB) YDGPERR1(ii)%GP3D(jj,jl,jf,jb)=GMV(jj,jl,jg,jb)
            GMV(jj,jl,jg,jb) = GMV(jj,jl,jg,jb) + YDGPERR(ii)%GP3D(jj,jl,jf,jb)
          ENDDO
        ENDDO
      ELSEIF (MERRGFL(jf)>0) THEN
        jg=MERRGFL(jf)
        DO jl=1,NFLEVG
          DO jj=1,ilen
            IF (LLHYB) YDGPERR1(ii)%GP3D(jj,jl,jf,jb)=GFL(jj,jl,jg,jb)
            GFL(jj,jl,jg,jb) = GFL(jj,jl,jg,jb) + YDGPERR(ii)%GP3D(jj,jl,jf,jb)
          ENDDO
        ENDDO
      ELSE
        CALL ABOR1('ADD_MODERR_TL: Field not found in GMV or GFL')
      ENDIF
    ENDDO

    DO jf=1,YDGPERR(ii)%NG2D
      IF (MERRGMVS(jf)>0) THEN
        jg=MERRGMVS(jf)
        DO jj=1,ilen
          IF (LLHYB) YDGPERR1(ii)%GP2D(jj,jf,jb)=GMVS(jj,jg,jb)
          GMVS(jj,jg,jb) = GMVS(jj,jg,jb) + YDGPERR(ii)%GP2D(jj,jf,jb)
        ENDDO
      ELSE
        CALL ABOR1('ADD_MODERR_TL: Field not found in GMVS')
      ENDIF
    ENDDO
  ENDDO

! Periodic forcing
  IF (NTYPE_MODERR==2.AND.NORDER_MODERR>0) THEN
    DO jorder=1,2*NORDER_MODERR
      ii=ii+NSTEP_MODERR
      zz=FSTEPERR(kstep,jorder)

      DO jf=1,YDSPERR(ii)%NS3D
        jg=MERRSP3(jf)
        DO jj=1,NSPEC2
          DO jl=1,NFLEVL
            SPA3(jl,jj,jg) = SPA3(jl,jj,jg) + zz * YDSPERR(ii)%SP3D(jl,jj,jf)
          ENDDO
        ENDDO
      ENDDO

      DO jf=1,YDSPERR(ii)%NS2D
        jg=MERRSP2(jf)
        DO jj=1,NSPEC2
          SPA2(jj,jg) = SPA2(jj,jg) + zz * YDSPERR(ii)%SP2D(jj,jf)
        ENDDO
      ENDDO

      DO jb=1,NGPBLKS
        ilen=MIN(NPROMA, NGPTOT-(jb-1)*NPROMA)

        DO jf=1,YDGPERR(ii)%NG3D
          IF (MERRGMV(jf)>0) THEN
            jg=MERRGMV(jf)
            DO jl=1,NFLEVG
              DO jj=1,ilen
                GMV(jj,jl,jg,jb) = GMV(jj,jl,jg,jb) + zz * YDGPERR(ii)%GP3D(jj,jl,jf,jb)
              ENDDO
            ENDDO
          ELSEIF (MERRGFL(jf)>0) THEN
            jg=MERRGFL(jf)
            DO jl=1,NFLEVG
              DO jj=1,ilen
                GFL(jj,jl,jg,jb) = GFL(jj,jl,jg,jb) + zz * YDGPERR(ii)%GP3D(jj,jl,jf,jb)
              ENDDO
            ENDDO
          ELSE
            CALL ABOR1('ADD_MODERR_TL: Field not found in GMV or GFL')
          ENDIF
        ENDDO

        DO jf=1,YDGPERR(ii)%NG2D
          IF (MERRGMVS(jf)>0) THEN
            jg=MERRGMVS(jf)
            DO jj=1,ilen
              GMVS(jj,jg,jb) = GMVS(jj,jg,jb) + zz * YDGPERR(ii)%GP2D(jj,jf,jb)
            ENDDO
          ELSE
            CALL ABOR1('ADD_MODERR_TL: Field not found in GMVS')
          ENDIF
        ENDDO
      ENDDO
    ENDDO
  ENDIF

! Final prints
  IF (llprt) THEN
    WRITE(NULOUT,*)'ADD_MODERR_TL: Norms of high res AN'
    CALL SPNORM(0)
    CALL GPNORM_GFL
  ENDIF
ENDIF

IF (LHOOK) CALL DR_HOOK('ADD_MODERR_TL',1,ZHOOK_HANDLE)
END SUBROUTINE ADD_MODERR_TL
