SUBROUTINE AVARC(KEVENT)

!**** *AVARC*  - reads in the inputs for AVARC job , computes the
! the cost function and gradient at the time of the avarc event.

!     Purpose.
!     --------
!     variational assimilation of corrections 

!**   Interface.
!     ----------
!        *CALL* *AVARC(KEVENT)
!                KEVENT : the number of the correction event/slot

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Symposium II Project

!     Author.
!     -------
!        Radmila Brozkova  *RC LACE*

!     Modifications.
!     --------------
!        Original : 01-06-01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NSPEC2   ,NFLSUR   ,NFD2D   ,NS3D, NFLEVL,&
 & NUMP     ,NSMAX      
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPT     ,SPQ     ,SPSP
USE YOMCOSJR , ONLY : FALTSEL  ,FSURSEL  ,FSPALT   ,FSPSUR
USE YOMLAP   , ONLY : NASM0, MYMS
USE YOMMP    , ONLY : MYSETB   ,NBSETSP  ,NPSP
USE YOMJR    , ONLY :   AVNORM2  ,AVNORM3  ,AGRASP, AGRAVOR, &
 & AGRADIV, AGRAT    ,AGRAQ    ,ATRAVOR  ,ATRADIV, ATRAT , &
 & ATRAQ  , ATRASP  
USE YOMVAR   , ONLY : ALPHAV
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KEVENT 
! actual departure = trajectory + perturbation - reference
REAL(KIND=JPRB) :: ZSPOR(NSPEC2)      , ZSP (NSPEC2)
REAL(KIND=JPRB) :: ZVOR(NFLSUR,NSPEC2), ZDIV(NFLSUR,NSPEC2)
REAL(KIND=JPRB) :: ZT  (NFLSUR,NSPEC2), ZQ  (NFLSUR,NSPEC2)

! save the perturbation
REAL(KIND=JPRB) :: ZJR2(NFD2D), ZJR1(NFLEVL)
REAL(KIND=JPRB) :: ZJR3(NFLEVL,NS3D)
REAL(KIND=JPRB) :: ZSUMJR(NUMP)

REAL(KIND=JPRB) ::  ZFM, ZTOTALJR

INTEGER(KIND=JPIM) ::  J, JK, JMLOC, IM, JN, INM, JIR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AVARC',0,ZHOOK_HANDLE)
ZVOR = 0.0_JPRB
ZDIV = 0.0_JPRB
ZT   = 0.0_JPRB
ZQ   = 0.0_JPRB
ZSP  = 0.0_JPRB
ZSPOR= 0.0_JPRB

!     ------------------------------------------------------------------

!*       1.    EVALUATE cost function Jr (energy norm) and gradient
!              ----------------------------------------------------

IF(NPSP == 1) THEN
  IF (MYSETB  ==  NBSETSP) THEN
! save surface pressure perturbation
! Z3SP(:) = SPSP(:)
! Z3SPOR(:) = SPOR(:)
! result = trajectory - reference + perturbation
    DO J=1,NSPEC2
      ZSP(J)= ALPHAV * (ATRASP(J,KEVENT) + SPSP(J)) 
    ENDDO
  ENDIF
ENDIF

! save 3D fields perturbations
!Z3VOR(:,:) = SPVOR(:,:)
!Z3DIV(:,:) = SPDIV(:,:)
!Z3T  (:,:) = SPT(:,:)
!Z3Q  (:,:) = SPQ(:,:)
DO J=1,NSPEC2
  DO JK=1,NFLEVL
    ZVOR(JK,J) = ALPHAV * (ATRAVOR(JK,J,KEVENT) + SPVOR(JK,J)) 
    ZDIV(JK,J) = ALPHAV * (ATRADIV(JK,J,KEVENT) + SPDIV(JK,J))
    ZT  (JK,J) = ALPHAV * (ATRAT(JK,J,KEVENT)   + SPT(JK,J)) 
    ZQ  (JK,J) = ALPHAV * (ATRAQ(JK,J,KEVENT)   + SPQ(JK,J)) 
  ENDDO
ENDDO

!remain dry

ZQ= 0.0_JPRB

!                  Mask Operator
!IF(LOCNORM) THEN
!  CALL STEPO  ('0K0000000')
!  CALL STEPOTL('0GB0L0AA0')
!ENDIF

! test: jr=0
!ZVOR = _ZERO_
!ZDIV = _ZERO_
!ZT   = _ZERO_
!ZSP  = _ZERO_

!     3D/2D contributions

ZSUMJR(:) = 0.0_JPRB
DO JMLOC = 1, NUMP
  ZJR3 (:,:) = 0.0_JPRB
  ZJR2 (:)   = 0.0_JPRB
  ZJR1 (:)   = 0.0_JPRB
  IM = MYMS(JMLOC)
  ZFM = 1.0_JPRB + REAL(MIN(1,IM),JPRB)
  DO JN = IM, NSMAX
    DO JIR = 0, MIN(1,IM)
      INM = NASM0(IM) + (JN-IM)*2 + JIR
      DO JK = 1, NFLEVL
! vorticity
        ZJR3(JK,1) = ZJR3(JK,1)&
         & + ZFM*AVNORM3(JK,JN,1)*ZVOR(JK,INM)**2  
        AGRAVOR(JK,INM,KEVENT) = AVNORM3(JK,JN,1)*ZVOR(JK,INM)*2.0_JPRB * &
         & FALTSEL(JK,1)  
! divergence
        ZJR3(JK,2) = ZJR3(JK,2)&
         & + ZFM*AVNORM3(JK,JN,2)*ZDIV(JK,INM)**2  
        AGRADIV(JK,INM,KEVENT) = AVNORM3(JK,JN,2)*ZDIV(JK,INM)*2.0_JPRB * &
         & FALTSEL(JK,2)  
! temperature
        ZJR3(JK,3) = ZJR3(JK,3)&
         & + ZFM*AVNORM3(JK,JN,3)*ZT(JK,INM)**2  
        AGRAT(JK,INM,KEVENT) = AVNORM3(JK,JN,3)*ZT(JK,INM)*2.0_JPRB * &
         & FALTSEL(JK,3)  
! humidity  
        ZJR3(JK,4) = ZJR3(JK,4)&
         & + ZFM*AVNORM3(JK,JN,4)*ZQ(JK,INM)**2  
        AGRAQ(JK,INM,KEVENT) = AVNORM3(JK,JN,4)*ZQ(JK,INM)*2.0_JPRB * &
         & FALTSEL(JK,4)  
      ENDDO
! surface pressure
      IF (MYSETB  ==  NBSETSP) THEN
        ZJR2(1) = ZJR2(1)&
         & + ZFM*AVNORM2(JN,1)*ZSP(INM)**2  
        AGRASP(INM,KEVENT) = AVNORM2(JN,1)*ZSP(INM)*2.0_JPRB * FSURSEL(1)
      ENDIF
    ENDDO
  ENDDO

  DO JK = 1, NFLEVL
    FSPALT(JK,1,JMLOC)= FSPALT(JK,1,JMLOC)+ ZJR3(JK,1)
    FSPALT(JK,2,JMLOC)= FSPALT(JK,2,JMLOC)+ ZJR3(JK,2)
    FSPALT(JK,3,JMLOC)= FSPALT(JK,3,JMLOC)+ ZJR3(JK,3)
    FSPALT(JK,4,JMLOC)= FSPALT(JK,4,JMLOC)+ ZJR3(JK,4)
  ENDDO
  FSPSUR(1,JMLOC)     = FSPSUR(1,JMLOC)   + ZJR2(1)
  ZSUMJR(JMLOC) = SUM(ZJR3(:,:)) + SUM(ZJR2(:))
ENDDO

ZTOTALJR = SUM(ZSUMJR(:))

WRITE(NULOUT,*) 'FOR EVENT = ',KEVENT,' On this CPU: Value of the '&
 & ,'JR cost function = ',ZTOTALJR  

!               Adjoint Mask operator
!IF(LOCNORM)  THEN
!  CALL STEPOAD('0GB0L0AA0')
!ENDIF
! save gradients of 2D fields
!IF(NPSP == 1) THEN
!  IF (MYSETB  ==  NBSETSP) THEN
!    DO J=1,NSPEC2
!      AGRASP(J,KEVENT) = SPSP(J) 
!   ENDDO
! recover surface pressure perturbation
!  SPSP(:) = Z3SP(:)
!  SPOR(:) = Z3SPOR(:)
!  ENDIF
!ENDIF
! save gradients of 3D fields
!DO J=1,NSPEC2
!  DO JK=1,NFLEVL
!    AGRAVOR(JK,J,KEVENT) = SPVOR(JK,J) 
!    AGRADIV(JK,J,KEVENT) = SPDIV(JK,J)
!    AGRAT(JK,J,KEVENT)   = SPT(JK,J) 
!    AGRAQ(JK,J,KEVENT)   = SPQ(JK,J) 
!  ENDDO
!ENDDO
! recover 3D fields perturbations
!SPVOR(:,:) = Z3VOR(:,:)
!SPDIV(:,:) = Z3DIV(:,:)
!SPT(:,:)   = Z3T  (:,:)
!SPQ(:,:)   = Z3Q  (:,:)

IF (LHOOK) CALL DR_HOOK('AVARC',1,ZHOOK_HANDLE)
END SUBROUTINE AVARC
