subroutine bgevecs

!*** *BGEVECS* - Compute variance from ensemble vectors

!   Purpose.
!   --------

!   Interface.
!   ----------

!      CALL BGEVECS

!   Externals.
!   ----------
!              SETRAN
!              RANDOM_NUMBERS_MIX
!              STEPO
!              SC2WRG

!   Reference.
!   ----------
!      None yet!

!   Author.
!   -------
!      Gerald Desroziers

!   Modifications.
!   --------------
!        Original  07-09-01
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPROMA   ,NFD2D    ,NS3D     ,NS2D     ,&
 & NSPEC2   ,NFLEVL
USE YOMCT0   , ONLY : LOBS,  NSPPR
USE YOMCVA   , ONLY : YRSCALPSQRT
USE YOMVAR   , ONLY : NBGVECS  ,NOANEF    ,&
 & LBGTRUNC   ,LBGM     ,NSSBGV_COUNT, LANOBS
USE YOMGEM   , ONLY : NGPTOT
USE YOMANEB  , ONLY : ANEBUF
USE YOMVRTL  , ONLY : L131TL
USE YOMSP    , ONLY : SP5A3    ,SP5A2    ,SPA3     ,SPA2
USE YOMLUN   , ONLY : NULOUT
USE YOMWAVELET, ONLY : CINBGSTATES  ,N_BGMEMBERS, N_BGDATES
USE TRAJECTORY_MOD, ONLY: ltrajgp, get_traj_spec
USE YOMRANDOM_STREAMS, ONLY : YD_RANDOM_STREAM_SCAN2MTL
USE RANDOM_NUMBERS_MIX
USE CONTROL_VECTORS

!-----------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZDUM(1,1)
REAL(KIND=JPRB) :: zane(NPROMA,NOANEF)
REAL(KIND=JPRB), ALLOCATABLE :: ZSP3MEM(:,:,:,:),ZSP2MEM(:,:,:)

type(control_vector) :: YL_ZCHI_DV

INTEGER(KIND=JPIM) :: iendc,  istc, jkglo,  ISEED,  jdate, jmember
CHARACTER (LEN = 16) ::  clfile

LOGICAL :: LL131TL,  LLOBS

REAL(KIND=JPRB) :: zrnbg
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "fltbgerr.intfb.h"
#include "random_ctlvec.intfb.h"
#include "scaleae.intfb.h"
#include "sc2wrg.intfb.h"
#include "setran.intfb.h"
#include "stepo.intfb.h"
#include "stepotl.intfb.h"
#include "readvec.intfb.h"
#include "spnorm.intfb.h"

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('BGEVECS',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------

!              1. Randomization Estimate
!              -------------------------

! Preset anebuf to 0

! Allocate memory to store the vectors

ALLOCATE (ZSP3MEM(NFLEVL,NSPEC2,NS3D,N_BGMEMBERS))
ALLOCATE (ZSP2MEM(NSPEC2,NS2D,N_BGMEMBERS))

do jkglo=1,NGPTOT,NPROMA
  istc=1
  iendc=min(NPROMA,NGPTOT-jkglo+1)
  zane(istc:iendc,:) = 0.0_JPRB

  call SC2WRG (ANEBUF,jkglo,istc,iendc,zane)
enddo

NBGVECS = 0

!--- Take Bg to Fourier space.
IF(.NOT.(LBGM .OR. LTRAJGP)) THEN
  CALL GSTATS(15,0)
  CALL get_traj_spec(SP5A3,SP5A2,ZDUM)
  CALL GSTATS(15,1)
  call STEPO('0E0000000')
ENDIF

call allocate_ctlvec(YL_ZCHI_DV)

LL131TL=L131TL
LLOBS=LOBS
IF(.NOT.LANOBS) THEN
  L131TL=.FALSE.
  LOBS=.FALSE.
ENDIF

!--- generate a date-dependent seed
CALL SETRAN (96753,KOUTSEED=ISEED)

!--- loop over ensemble vectors
DO jdate=1,N_BGDATES
  DO jmember=1,N_BGMEMBERS

!--- read a background

    clfile = CINBGSTATES
    WRITE (clfile(7:16),'(''m'',I3.3,''_d'',I3.3,'' '')') jmember,jdate
    WRITE (NULOUT,*) 'reading member ',jmember,' date ',jdate,' from file ',clfile
    CALL READVEC  (clfile,LDJB=.FALSE.)

    write (nulout,*) 'in bgvecs, before difference: SPA norms are:'
    NSPPR=1
    CALL SPNORM(0)

!--- generate random vector in control space (and update ISEED)

    CALL GSTATS(1958,0)
    CALL RANDOM_CTLVEC (YL_ZCHI_DV,.TRUE.,.FALSE.,.TRUE.,.TRUE.,.FALSE.,.FALSE.,ISEED)
    YL_ZCHI_DV%data = YL_ZCHI_DV%data * YRSCALPSQRT%data
    CALL GSTATS(1958,1)

!--- save current background state

    ZSP3MEM(:,:,:,jmember)     = SPA3(:,:,:)
    ZSP2MEM(:,1:NFD2D,jmember) = SPA2(:,1:NFD2D)

!--- calculate difference from previous state and accumulate variances

    if (jmember>1) then
      SPA3(:,:,:) = ZSP3MEM(:,:,:,jmember) - ZSP3MEM(:,:,:,jmember-1)
      SPA2(:,1:NFD2D) = ZSP2MEM(:,1:NFD2D,jmember) - ZSP2MEM(:,1:NFD2D,jmember-1)
      write (nulout,*) 'in bgvecs, after difference: SPA norms are:'
      NSPPR=1
      CALL SPNORM(0)
      NBGVECS = NBGVECS + 1

!--- transform to grid space and accumulate variances

!--- radiance computation needs trajectory, so use TL routines.

      CALL INITIALIZE_RANDOM_NUMBERS (ISEED,yd_random_stream_scan2mtl)
      ISEED=ISEED+1
      call STEPOTL('0AA00A000')
    endif

  enddo
enddo

call deallocate_ctlvec(YL_ZCHI_DV)
L131TL=LL131TL
LOBS=LLOBS
NSSBGV_COUNT=0

!--- Divide sum of squares by the number of vectors

zrnbg = 1.0_JPRB/real(NBGVECS,JPRB)
call scaleae (zrnbg)

!-----------------------------------------------------------------------

!--- filter to reduce the random noise.

if (LBGTRUNC) call FLTBGERR

IF(ALLOCATED(ZSP3MEM))DEALLOCATE(ZSP3MEM)
IF(ALLOCATED(ZSP2MEM))DEALLOCATE(ZSP2MEM)

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('BGEVECS',1,ZHOOK_HANDLE)
end subroutine bgevecs
