SUBROUTINE CAININ(YDVAZX)

!     Purpose.
!     --------
!       Inverse canonical injection.

!     Explicit arguments :
!     --------------------
!       YDVAZX  : CONTROL VARIABLE (OUTPUT)

!     Author.
!     -------
!       Y.Tremolet (from CAININ_SCALE_HELPER)

!     Modifications.
!     --------------
!        Original : 02-08-05
!     C.Fischer   03-04-02 Aladin spectral treatment added.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher 17-Mar-2004 Abort if wavelet Jb
!        Y.Tremolet    18-Mar-2004 Fix 2D arrays
!        D.Dee    19-Mar-2003  Variational bias correction
!        Y.Tremolet    10-Jun-2004 Initial condition optional in control vector
!        Y.Tremolet    27-Jul-2004 Improve model error
!        P.Burton :  29-09-04 LVARBC cleanup
!        L.Isaksen     05-Sep-2005 Adapt for grid point q
!        A.Dethof      23-Jan-2007 Add GEMS GRG fields
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMDIM  , ONLY: NSPEC2, NFLEVL, NPROMA, NS3D
USE YOMSP   , ONLY: SPA3, SPA2, SPA1
USE YOMGFL  , ONLY : GFL
USE YOM_YGFL, ONLY : YQ, YO3, YGHG, NGHG, YGRG, NGRG
USE YOMMP   , ONLY: NPSP, NBSETLEV
USE YOMTVRAD, ONLY: TOVSCVX,NVATOVLEN
USE YOMDIMO , ONLY: NOBNTV
USE YOMCVA  , ONLY: NVA3D, NVA2D, NVA1D, NVATOVV
USE YOMVAR  , ONLY: LVARBC
USE VARBC_SETUP, ONLY: YVARBC, NOBGROUP, NOBPARAM
USE YOMMODEL_ERROR, ONLY: NDIM_MODERR, SPMODERR, GPMODERR, SPCTLMODERR, SPGPMODERR
USE YOMWAVELET, ONLY: LJBWAVELET
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE SPECTRAL_FIELDS_MOD, ONLY : SPECTRAL_FIELD, ASSIGNMENT(=)
USE YOMTS, ONLY: TSCVX


IMPLICIT NONE
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZX 
INTEGER(KIND=JPIM) :: jl, js, jf, ii, jg, ibeg, iend, il, jj, jghg, jgrg
REAL(KIND=JPRB), ALLOCATABLE :: ZSP(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "dir_trans.h"
END INTERFACE

#include "abor1.intfb.h"
#include "grid2spec.intfb.h"
#include "spec_concat.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CAININ',0,ZHOOK_HANDLE)

IF (LJBWAVELET) CALL ABOR1 ('CAININ CANNOT BE USED WITH WAVELET JB')

! Initial condition
! -----------------
IF (YDVAZX%linitcv) THEN
  ii=0
  IF (NPSP==1) THEN
    DO jf=1,NVA2D
      DO js=1,NSPEC2
        ii=ii+1
        YDVAZX%initcv(ii)=SPA2(js,jf)
      ENDDO
    ENDDO
  ENDIF
  IF (NS3D > NVA3D) THEN
    CALL ABOR1('CAININ: UNEXPECTED INCONSITENCY NS3D > NVA3D ')
  ENDIF
  DO jf=1,NS3D
    DO js=1,NSPEC2
      DO jl=1,NFLEVL
        ii=ii+1
        YDVAZX%initcv(ii)=SPA3(jl,js,jf)
      ENDDO
    ENDDO
  ENDDO
! Transform grid point model state fields to spectral control vector fields where required
  IF (NS3D < NVA3D) THEN
    ALLOCATE(ZSP(NFLEVL,NSPEC2))
    IF (YQ%LGP) THEN
      CALL DIR_TRANS(PSPSCALAR=ZSP,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
       & PGP=GFL(:,:,YQ%MP,:))
      DO js=1,NSPEC2
        DO jl=1,NFLEVL
          ii=ii+1
          YDVAZX%initcv(ii)=ZSP(jl,js)
        ENDDO
      ENDDO
    ENDIF
    IF (YO3%LGP) THEN
      CALL DIR_TRANS(PSPSCALAR=ZSP,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
       & PGP=GFL(:,:,YO3%MP,:))
      DO js=1,NSPEC2
        DO jl=1,NFLEVL
          ii=ii+1
          YDVAZX%initcv(ii)=ZSP(jl,js)
        ENDDO
      ENDDO
    ENDIF
    DO JGHG=1,NGHG
      IF (YGHG(JGHG)%LGP) THEN
        CALL DIR_TRANS(PSPSCALAR=ZSP,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
         & PGP=GFL(:,:,YGHG(JGHG)%MP,:))
        DO js=1,NSPEC2
          DO jl=1,NFLEVL
            ii=ii+1
            YDVAZX%initcv(ii)=ZSP(jl,js)
          ENDDO
        ENDDO
      ENDIF
    ENDDO
    DO JGRG=1,NGRG
      IF (YGRG(JGRG)%LGP) THEN
        CALL DIR_TRANS(PSPSCALAR=ZSP,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
         & PGP=GFL(:,:,YGRG(JGRG)%MP,:))
        DO js=1,NSPEC2
          DO jl=1,NFLEVL
            ii=ii+1
            YDVAZX%initcv(ii)=ZSP(jl,js)
          ENDDO
        ENDDO
      ENDIF
    ENDDO
    DEALLOCATE(ZSP)
  ENDIF
  IF (SIZE(YDVAZX%initcv)/=ii) CALL ABOR1('CAININ: initcv wrong length')
ENDIF

! Model error
! -----------
IF (YDVAZX%nmoderr>0) THEN
  ii=0
  il=SPCTLMODERR%nspsizel
  DO jj=1,NDIM_MODERR
    CALL GRID2SPEC(GPMODERR(jj),SPGPMODERR)
    CALL SPEC_CONCAT(SPMODERR(jj),SPGPMODERR,SPCTLMODERR)
    YDVAZX%MODERR(ii+1:ii+il)=SPCTLMODERR
    ii=ii+il
  ENDDO
  IF (ii/=SIZE(YDVAZX%MODERR)) CALL ABOR1('CAININ: moderr wrong length')
ENDIF

! TOVS control variable
! ---------------------
IF (YDVAZX%ltovscv) THEN
  ii=0
  DO jf=1,NVATOVV
    DO js=1,NVATOVLEN
      ii=ii+1
      YDVAZX%tovscv(ii)=TOVSCVX(js,jf)
    ENDDO
  ENDDO
  IF (ii/=SIZE(YDVAZX%tovscv)) &
   & CALL ABOR1('CAININ: TOVS control variable wrong length')  
ENDIF

! TS control variable
! ---------------------
 IF (YDVAZX%ltscv) THEN
   ii=0
     DO js=1,NOBNTV
       ii=ii+1
       YDVAZX%tscv(ii)=TSCVX(js)
   ENDDO
   IF (ii/=SIZE(YDVAZX%tscv)) CALL ABOR1('CAIN: TS control variable wrong length')
 ENDIF


! MEAN WIND (LAM)
! ---------------
IF (YDVAZX%lam1d) THEN
  ii=0
  DO jf=1,NVA1D
    DO jl=1,NFLEVL
      ii=ii+1
      YDVAZX%lamcv(ii)=SPA1(jl,jf)
    ENDDO
  ENDDO
  IF (ii/=SIZE(YDVAZX%lamcv)) &
   & CALL ABOR1('CAININ: LAM control variable wrong length')  
ENDIF

! Obs bias parameter estimation
! -----------------------------
IF (LVARBC) THEN
  IF (NOBPARAM/=SIZE(YDVAZX%params)) &
   & CALL ABOR1('CAININ: Parameter section of control variable wrong length')  
  ibeg = 1
  DO jg = 1, NOBGROUP
    iend = ibeg + YVARBC(jg)%NPARAM - 1
    YDVAZX%params(ibeg:iend) = YVARBC(jg)%APARAMS
    ibeg = iend + 1
  ENDDO
ENDIF


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CAININ',1,ZHOOK_HANDLE)
END SUBROUTINE CAININ
