SUBROUTINE CHAVARAD (YDVAZG)

!**** *CVARAD*    - Change of Variable

!     PURPOSE.
!     --------

!        Change variable :
!              NCNTVAR = 1 : identity
!              NCNTVAR = 2 : X ==> V((X-Xg)/SIGMAg)

!**   INTERFACE.
!     ----------

!        *CALL* *CHAVARAD*

!        INPUT: YDVAZG

!        OUTPUT: SPA3 and SPA2 of YOMSP

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!           CVAR2AD

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        J.N.THEPAUT and P.COURTIER   *ECMWF*

!     MODIFICATIONS.
!     --------------
!        Bill Heckley 22:09:92 Additional choice of control variable
!        M.Hamrud     15:12:95 Remove NCNTVAR=3
!        Y.Tremolet   05-08-02 Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0 , ONLY: NCNTVAR
USE CONTROL_VECTORS

IMPLICIT NONE
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZG 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "caininad.intfb.h"
#include "cvar2ad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CHAVARAD',0,ZHOOK_HANDLE)
IF (NCNTVAR == 1) THEN
  CALL CAININAD(YDVAZG)
ELSEIF (NCNTVAR == 2) THEN
  CALL CVAR2AD(YDVAZG)
ELSE
  CALL ABOR1(' BAD LUCK, ERROR NCNTVAR IN CHAVARAD')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CHAVARAD',1,ZHOOK_HANDLE)
END SUBROUTINE CHAVARAD
