SUBROUTINE COSENS

!**** *COSENS*  - ENSTROPHY COST FUNCTION 

!     Purpose.
!     --------
!     EVALUATE THE GRADIENT OF THE ENSTROPHY COST-FUNCTION
!     IN SPECTRAL SPACE. 

!**   Interface.
!     ----------
!        *CALL* *COSENS

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation !!!

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!       T. Bergot  *Meteo France - CNRM/GMME* 

!     Modifications.
!     --------------
!       ORIGINAL : 97-07
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NF3D     ,NSMAX    ,NUMP
USE YOMDYN   , ONLY : SIDELP
USE YOMLUN   , ONLY : NULOUT
USE YOMCOSJR , ONLY : FSPALT
USE YOMCST   , ONLY :       RG
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SP5VOR   ,SP5DIV   ,&
 & SPT      ,SPQ      ,SP5T     ,SP5Q     ,SPSP     ,&
 & SP5SP  
USE YOMLAP   , ONLY : NASM0    ,MYMS
USE YOMLCZ   , ONLY :         LOCNORM
USE YOMMP    , ONLY : MYLEVS   ,NPSP
USE MPL_MODULE

IMPLICIT NONE

!     ------------------------------------------------------------------
! -------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSDOM,ZSGLOB,ZVERT
REAL(KIND=JPRB) :: ZFSPALT(NFLEVL,NF3D)

INTEGER(KIND=JPIM) :: ILEV, IM, INM, IWAVE0, JIR, JLEV, JMLOC, JN

REAL(KIND=JPRB) :: Z, ZMET, ZVOR, ZZJ, ZZTOT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cossmq.intfb.h"
#include "stepo.intfb.h"
#include "stepoad.intfb.h"
#include "stepotl.intfb.h"

! -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COSENS',0,ZHOOK_HANDLE)
WRITE(NULOUT,*) 'COSENS : ENTREE DANS COSENS'

!*       1. PRELIMINAIRES
!           --------------   

!         1.1 Mise a zero des tableaux

ZFSPALT(:,:)=0.0_JPRB
ZSDOM=0.0_JPRB
ZSGLOB=0.0_JPRB
ZVERT=0.0_JPRB

!         1.3 Calcul de la surface du domaine et de la surface globale

if (LOCNORM) THEN

  CALL COSSMQ(ZSGLOB,ZSDOM,ZVERT)
  WRITE(NULOUT,*)'COSENS : SURFACE GLOBALE SGLOB= ', ZSGLOB
  WRITE(NULOUT,*)'COSENS : SURFACE DOMAINE SDOM= ', ZSDOM
ELSE
  ZSGLOB=1.0_JPRB
  ZSDOM=1.0_JPRB
ENDIF

!      --------------------------------------------------------------

!*       2. CALCUL DU GRADIENT ET DE LA FONCTION COUT
!           -----------------------------------------

!         2.1 Initialisation des tableaux spectraux par le resultat de  
!             l'integration directe (trajectoire)    

SPVOR(:,:)=SP5VOR(:,:)
SPDIV(:,:)=SP5DIV(:,:)
SPT(:,:)=SP5T(:,:)
SPQ(:,:)=SP5Q(:,:)
IF (NPSP == 1) THEN
  SPSP(:)=SP5SP(:)
ENDIF

!              2.2 Mask        

IF(LOCNORM) THEN
  CALL   STEPO('0K0000000')
  CALL STEPOTL('0GB0L0AA0')
ENDIF

!      2.3 calcul de la fonction cout par niveau 
!       et calcul du gradient de la fonction cout / tourbillon

DO JMLOC=1,NUMP
  IM=MYMS(JMLOC)
  ZMET=1.0_JPRB+REAL(MIN(1,IM),JPRB)
  DO JIR=0,MIN(IM,1)
    DO JN=IM,NSMAX
      INM=NASM0(IM)+(JN-IM)*2+JIR

      DO JLEV=1,NFLEVL
        ILEV=MYLEVS(JLEV)

! traitement VOR

        ZVOR=0.5_JPRB*(ZSGLOB/ZSDOM)*(SIDELP(ILEV)/RG)
        Z=ZMET*ZVOR*(SPVOR(JLEV,INM))**2
        ZFSPALT(JLEV,1)=ZFSPALT(JLEV,1)+Z
! dJ/dVor       
        SPVOR(JLEV,INM)=ZVOR*SPVOR(JLEV,INM)*2.0_JPRB
! J(Vor)
        IWAVE0=1
        FSPALT(JLEV,1,IWAVE0)=ZFSPALT(JLEV,1)

! mise a zero des autres variables 3D (dJ/dX=0)

        SPDIV(JLEV,INM)=0.0_JPRB
        SPT(JLEV,INM)=0.0_JPRB
        SPQ(JLEV,INM)=0
      ENDDO

! mise a zero des autres variables 2D (dJ/dX=0)

      IF (NPSP == 1) THEN
        SPSP(INM)=0.0_JPRB
      ENDIF

    ENDDO
  ENDDO
ENDDO

!           2.5 Adjoint Mask 

IF(LOCNORM) THEN
  CALL STEPOAD('0GB0L0AA0')
ENDIF

!      --------------------------------------------------------------

!*       3. DIAGNOSTIQUES
!           -------------

WRITE(NULOUT,*)'COSENS : DIAGNOSTIQUES FCT COUT ENSTROPHY'
WRITE(NULOUT,*)'COSENS : contribution par niveaux'
ZZJ=0.0_JPRB
DO JLEV=1,NFLEVL ! This is wrong with NPRTRV>1 /MH
  ZZTOT=ZFSPALT(JLEV,1)
  CALL MPL_ALLREDUCE(ZZTOT,'SUM',LDREPROD=.FALSE.,CDSTRING='COSENS:')
  WRITE(NULOUT,301) 'COSENS : ',JLEV,ZZTOT
  301 FORMAT(A9,'LEV',I3,' J=',G13.3)
  ZZJ=ZZJ+ZFSPALT(JLEV,1)
ENDDO
ZZTOT=ZZJ
CALL MPL_ALLREDUCE(ZZTOT,'SUM',LDREPROD=.FALSE.,CDSTRING='COSENS:')
WRITE(NULOUT,*) 'COSENS : fonction cout = ',ZZTOT
WRITE(NULOUT,*)'COSENS : SORTIE DE COSENS'
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COSENS',1,ZHOOK_HANDLE)
END SUBROUTINE COSENS
