SUBROUTINE COSJC

!**** *COSJC*    - Controls JC term computation.

!     PURPOSE.
!     --------

!        Manage data flow and control of JC

!**   INTERFACE.
!     ----------

!        *CALL* *COSJC*

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!           STEPO,SWAP53,JCCOMP,ADD5TO3

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        J.N.THEPAUT and P.COURTIER   *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M Hamrud    : 98-01-10 - Remove use of CAIN.. for temp. storing
!        G. Desroziers : 02-09-16 also treat Aladin mean wind trajectory
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        B.Chapnik     22-sep-2008 introduction of LELAM case
USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0    ,ONLY : LELAM
USE YOMDIM   , ONLY :    NSPEC2   ,&
 & NS3D     ,NS2D     ,NFLSUR ,NFLEVL 
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SP5A3    ,SP5A2   ,&
                      &SPA1    ,SP5A1
USE YOMLUN   , ONLY : NULOUT
USE YOMVRTL  , ONLY : L131TL
USE YOMCT3   , ONLY : NSTEP
USE TRAJECTORY_MOD, ONLY: LTRAJGP, GET_TRAJ_SPEC

IMPLICIT NONE

!    DUMMY REAL
REAL(KIND=JPRB) :: ZDUM(1,1)

REAL(KIND=JPRB) :: ZSPA3(NFLSUR,NSPEC2,NS3D),ZSPA2(NSPEC2,NS2D),ZSPA1(NFLEVL,2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "add3to5.intfb.h"
#include "add5to3.intfb.h"
#include "jccomp.intfb.h"
#include "stepo.intfb.h"
#include "swap53.intfb.h"
!     ------------------------------------------------------------------

!*    1.    PRINT.
!           ------

IF (LHOOK) CALL DR_HOOK('COSJC',0,ZHOOK_HANDLE)
WRITE(UNIT=NULOUT,FMT='('' START JC COMPUTATIONS '')')
!     ------------------------------------------------------------------

!*    2.    Read trajectory.
!           ----------------

CALL GSTATS(15,0)
 IF (LELAM) THEN
  CALL GET_TRAJ_SPEC(SP5A3,SP5A2,SP5A1,KSTEP=NSTEP)
 ELSE
  CALL GET_TRAJ_SPEC(SP5A3,SP5A2,ZDUM,KSTEP=NSTEP)
 ENDIF
CALL GSTATS(15,1)
IF(.NOT.LTRAJGP) THEN
  CALL STEPO('0E0000000')
ENDIF
!     ------------------------------------------------------------------

!*    3.    Swap gradient and trajectory.
!           -----------------------------

IF(L131TL) THEN

!*    building of Xb + dX in SP5 before swaping.

!     save gradient SP3 locally.
  ZSPA3(:,:,:) = SPA3(:,:,:)
  ZSPA2(:,:) = SPA2(:,:)
  IF (LELAM) ZSPA1(:,:) = SPA1(:,:)
!     read dX in SP3
  CALL STEPO('J00000000')

!     add Xb + dX in SP5

  CALL ADD3TO5
!     restore SP3
  SPA3(:,:,:) = ZSPA3(:,:,:)
  SPA2(:,:) = ZSPA2(:,:)
  IF (LELAM) SPA1(:,:) = ZSPA1(:,:)
ENDIF
CALL SWAP53
!     ------------------------------------------------------------------

!*    4.    Evaluate JC and its gradient.
!           -----------------------------

CALL JCCOMP
!     ------------------------------------------------------------------

!*    5.    Combines the gradients.
!           -----------------------

CALL ADD5TO3
!     ------------------------------------------------------------------

!*    6.    PRINT.
!           ------

WRITE(UNIT=NULOUT,FMT='('' END JC COMPUTATIONS '')')

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COSJC',1,ZHOOK_HANDLE)
END SUBROUTINE COSJC

