SUBROUTINE COSJL

!**** *COSJL*  - LINEAR COST FUNCTION IN SPECTRAL SPACE

!     Purpose.
!     --------
!     EVALUATE THE COST-FUNCTION AND THE GRADIENT

!**   Interface.
!     ----------
!        *CALL* *COSJL

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!                  Original : 87-10-15 under the name COSTRA
!       Modified F.Bouttier : 95-04-12 turn quadratic into linear cost
!       Modified L.Isaksen  : 95-06-06 Reordering of spectral arrays
!       Modified M.Hamrud   : 95-12-22 Change of ZGDU etc. layout
!       Modified M.Hamrud   : 95-10-30 Changed CLCONF def.
!       Modified L.Isaksen  : 96-04-04 Adapted for Message Passing
!       Modified G.Hello    : 97-07-22 Set-up. mask definition
!       Modified E.Holm     : 97-12-01 Ozone included
!       Modified G.Hello    : 98-      Surface of masq and cleaning
!       Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of argument .FALSE. in 'CALL SPEREE'.
!        R. El Khatib : 01-08-07 Pruning options
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL, NUMP, NSMAX, NSPEC2
USE YOMDYN   , ONLY : SIDELP ,SIPR

USE YOMLUN   , ONLY : NULOUT
USE YOMCOSJR , ONLY : FSPALT ,FSPSUR
USE YOMSP    , ONLY : SPVOR ,SPDIV ,SP5VOR ,SP5DIV , SPO3,&
 & SPT ,SPQ ,SP5T ,SP5Q ,SPSP ,SP5SP, SP5O3  
USE YOMLAP   , ONLY : NASM0
USE YOMSTA   , ONLY : STTEM
USE YOMLCZ   , ONLY : LOCNORM
USE YOMGEM   , ONLY : NGPTOT ,VP00
USE YOMMP    , ONLY : MYSETW ,NPROCM ,MYLEVS ,NPSP
USE MPL_MODULE
USE YOM_YGFL , ONLY : YO3

IMPLICIT NONE

!     ----------------------------------------

CHARACTER (LEN = 9) :: CLCONF

REAL(KIND=JPRB), DIMENSION (NGPTOT) :: ZGDSP,ZGDSP2
REAL(KIND=JPRB), DIMENSION (6) :: ZZ

INTEGER(KIND=JPIM) :: ILEV, JLEV, JMLOC, JVAL, JVAR

REAL(KIND=JPRB) :: ZO3
REAL(KIND=JPRB) :: ZSGLOB,ZSDOM,ZVERT,ZRAP
REAL(KIND=JPRB) :: ZVOR,ZDIV,ZT,ZQ,ZSP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cossmq.intfb.h"
#include "reespe.intfb.h"
#include "speree.intfb.h"
#include "spnorm.intfb.h"
#include "stepo.intfb.h"
#include "stepoad.intfb.h"
#include "stepotl.intfb.h"

!     ----------------------------------------

!     1. Starts COSJL
!     ---------------

IF (LHOOK) CALL DR_HOOK('COSJL',0,ZHOOK_HANDLE)
ZSGLOB=0.0_JPRB
ZSDOM=0.0_JPRB
ZVERT=0.0_JPRB
ZRAP=0.0_JPRB

ZGDSP(:)=0.0_JPRB
ZGDSP2(:)=0.0_JPRB

CLCONF(1:9)='0GB0L0AA0'

CALL COSSMQ(ZSGLOB,ZSDOM,ZVERT)
ZRAP=ZSGLOB*SIPR/(ZSDOM*ZVERT)
!     --------------------------------------------------------------

!     2. COMPUTATION OF THE COST FUNCTION AND ITS GRADIENT.
!     -----------------------------------------------------

!     2.1 Put trajectory
!     ------------------

SPVOR(:,:) =SP5VOR(:,:)
SPDIV(:,:) =SP5DIV(:,:)
SPT(:,:)   =SP5T(:,:)
SPQ(:,:)   =SP5Q(:,:)
IF (YO3%LSP) THEN
  SPO3(:,:)  =SP5O3(:,:)
ENDIF
IF (NPSP == 1) THEN
  SPSP(:)=SP5SP(:)
ENDIF

CALL SPNORM(0)
!* 

!     2.2 Subtract reference values (standard atmosphere) to traj
!     -----------------------------------------------------------

!*

IF (MYSETW == NPROCM(0)) THEN
  SPT(:,NASM0(0))=SPT(:,NASM0(0))-STTEM(:)
ENDIF

! Log of surface pressure
CALL SPEREE(1,1,SPSP,ZGDSP)
DO JVAL=1,NGPTOT
  ZGDSP(JVAL)=EXP(ZGDSP(JVAL))-VP00
ENDDO
CALL REESPE(1,1,SPSP,ZGDSP)

!*

!     2.3 Mask the difference
!     -----------------------

IF(LOCNORM) THEN
  CALL STEPO('0K0000000')
  CALL STEPOTL(CLCONF)
ENDIF

CALL SPNORM(0)

!     2.4 Cost function & Gradient
!     ----------------------------

DO JLEV=1,NFLEVL
  ILEV=MYLEVS(JLEV)

!      2.4.1 3D cost-function & Gradients
!      ----------------------------------
  ZVOR=SIDELP(ILEV)/VP00/1.0_JPRB
  ZDIV=SIDELP(ILEV)/VP00/1.0_JPRB
  ZT=SIDELP(ILEV)/VP00/1.0_JPRB
  ZQ=SIDELP(ILEV)/VP00/1.E-4_JPRB
  ZO3=0.0_JPRB

  IF(LOCNORM) THEN
    ZVOR=ZVOR*ZRAP
    ZDIV=ZDIV*ZRAP
    ZT=ZT*ZRAP
    ZQ=ZQ*ZRAP
    ZO3=ZO3*ZRAP
  ENDIF

  IF (MYSETW == NPROCM(0)) THEN
    FSPALT(ILEV,1,1)=SPVOR(JLEV,NASM0(0))*ZVOR
    FSPALT(ILEV,2,1)=SPDIV(JLEV,NASM0(0))*ZDIV
    FSPALT(ILEV,3,1)=SPT (JLEV,NASM0(0))*ZT
    FSPALT(ILEV,4,1)=SPQ (JLEV,NASM0(0))*ZQ
    IF (YO3%LSP) THEN
      FSPALT(ILEV,5,1)=SPO3(JLEV,NASM0(0))*ZO3
    ELSE
      FSPALT(ILEV,5,1)=0.0_JPRB
    ENDIF
  ELSE
    DO JMLOC=1,NUMP
      FSPALT(ILEV,1:5,JMLOC)=0.0_JPRB
    ENDDO
  ENDIF
!*3D Gradients*!        
  SPVOR(JLEV,:)=0.0_JPRB
  SPDIV(JLEV,:)=0.0_JPRB
  SPT(JLEV,:)=0.0_JPRB
  SPQ(JLEV,:)=0.0_JPRB
  SPO3(JLEV,:)=0.0_JPRB
  IF(MYSETW == NPROCM(0)) THEN
    SPVOR(JLEV,NASM0(0))=ZVOR
    SPDIV(JLEV,NASM0(0))=ZDIV
    SPT(JLEV,NASM0(0))=ZT
    SPQ(JLEV,NASM0(0))=ZQ
    IF (YO3%LSP) THEN
      SPO3(JLEV,NASM0(0))=ZO3
    ENDIF
  ENDIF

ENDDO

!      2.4.2 Cost and gradient for Surface pressure
!      --------------------------------------------

IF (NPSP == 1) THEN
  ZSP=1/100._JPRB
  IF(LOCNORM) THEN
    ZSP=ZSP*ZRAP
  ENDIF

!* 2DCost function *!

  IF (MYSETW == NPROCM(0)) THEN
    FSPSUR(1,1)=SPSP(NASM0(0))*ZSP
  ELSE
    DO JMLOC=1,NUMP
      FSPSUR(1,JMLOC)=0.0_JPRB
    ENDDO
  ENDIF
!* 2DGradients *!
  SPSP(:)=0.0_JPRB
  ZGDSP(:)=0.0_JPRB
  ZGDSP2(:)=0.0_JPRB
!* Gradient w.r.t. Log(Ps) directly in gridpoint space *!
  CALL SPEREE(1,1,SP5SP,ZGDSP)
  DO JVAL=1,NGPTOT
    ZGDSP2(JVAL)=ZSP*EXP(ZGDSP(JVAL))
  ENDDO
  CALL REESPE(1,1,SPSP,ZGDSP2)

  IF(MYSETW == NPROCM(0)) THEN
    SPSP(2*(NSMAX+1):NSPEC2)=SPSP(2*(NSMAX+1):NSPEC2)*2.0_JPRB
  ELSE
    SPSP(:)=SPSP(:)*2.0_JPRB
  ENDIF

ENDIF
!     2.5 Adjoint Mask
!     ----------------

IF(LOCNORM) THEN
  WRITE(NULOUT,*) 'GRADIENT NORMS BEFORE LOCAL MASKING:'
  CALL SPNORM(0)
  CALL STEPOAD(CLCONF)
  WRITE(NULOUT,*) 'GRADIENT NORMS AFTER LOCAL MASKING:'
  CALL SPNORM(0)
ELSE
  WRITE(NULOUT,*) 'NORMS OF THE ADDED GRADIENT:'
ENDIF

!     ------------------------------------------------------------------

!     3. Writes diagnostics
!     ---------------------

!    Values are communicated here to achieve global values

ZZ(:) = 0.0_JPRB
WRITE(NULOUT,*) 'Contributions to linear cost-function :'
DO JLEV=1,NFLEVL
  ZZ(1:5) = 0.0_JPRB
  DO JMLOC=1,NUMP
    DO JVAR=1,4
      ZZ(JVAR)=ZZ(JVAR)+FSPALT(JLEV,JVAR,JMLOC)
    ENDDO
    IF (YO3%LSP) THEN
      ZZ(5)=ZZ(5)+FSPALT(JLEV,5,JMLOC)
    ENDIF
  ENDDO
  WRITE(NULOUT,311) JLEV,ZZ(1:5)
  311 FORMAT('LEV=',I3,' Cost U=',G13.3,' V=',G13.3,' T=',G13.3,&
   & ' Q=',G13.3,'O3=',G13.3)  
ENDDO

IF (NPSP == 1) THEN
  ZZ(6)=0.0_JPRB
  DO JMLOC=1,NUMP
    ZZ(6)=ZZ(6)+FSPSUR(1,JMLOC)
  ENDDO
  WRITE(NULOUT,312) ZZ(6)
  312 FORMAT(' Cost Ps=',G14.3)
ENDIF

CALL MPL_ALLREDUCE(ZZ,'SUM',LDREPROD=.FALSE.,CDSTRING='COSJL:')

WRITE(NULOUT,*) 'Total  U=',ZZ(1),' V=',ZZ(2),' T=',&
 & ZZ(3),' Q=',ZZ(4),' O3=',ZZ(5),' Ps=',ZZ(6)  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('COSJL',1,ZHOOK_HANDLE)
END SUBROUTINE COSJL
