SUBROUTINE COSSMQ(PGLOB,PDOM,PCOL)
!**** *COSSMQ*  - SURFACE OF LOCAL MASQ

!     Purpose.
!     --------
!     EVALUATE THE SURFACE OF THE LOCAL MASQ

!**   Interface.
!     ----------
!        *CALL* *COSSMQ

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------

!     Modifications.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!                  Original : 98-02-15 
!     ------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0    , ONLY : NPRGPEW
USE YOMDIM    , ONLY : NFLEVL,NDGENL
USE YOMDYN    , ONLY : SIDELP
USE YOMLUN    , ONLY : NULOUT
USE YOMGEM    , ONLY : NSTAGP,NLOENG
USE YOMGC     , ONLY : GELAM,GELAT,GM
USE YOMLEG    , ONLY : RW
USE YOMCST    , ONLY : RPI
USE YOMLCZ    , ONLY : NLEVMIN ,NLEVMAX,ALON1,ALON3,ALAT1,ALAT3
USE YOMMP     , ONLY : MYLEVS,MYLATS,MY_REGION_EW,NSTA,NONL,NPTRFLOFF
USE MPL_MODULE
!     ------------------------------------------------------------------

IMPLICIT NONE
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGLOB 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDOM 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOL 
REAL(KIND=JPRB) :: ZGM,ZGELAM,ZGELAT
REAL(KIND=JPRB) :: ZSGLOBL,ZSDOML,ZDPL

INTEGER(KIND=JPIM) :: IENDLAT, IENDLON, IGLG, ILEV, IROF, ISTLAT,&
 & ISTLON, JGL, JLEV, JLON  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     0. init part
!     ----------------      
IF (LHOOK) CALL DR_HOOK('COSSMQ',0,ZHOOK_HANDLE)
ZGM=0.0_JPRB
ZGELAT=0.0_JPRB
ZGELAM=0.0_JPRB
ZSGLOBL=0.0_JPRB
ZSDOML=0.0_JPRB
ZDPL=0.0_JPRB

WRITE(*,*)ALAT1,ALAT3,ALON1,ALON3

!     1. vertical part
!     ----------------
DO JLEV=1,NFLEVL
  ILEV=MYLEVS(JLEV)
  IF((ILEV >= NLEVMIN).AND.(ILEV <= NLEVMAX)) THEN
    ZDPL=ZDPL+SIDELP(ILEV)
  ENDIF
ENDDO
WRITE(NULOUT,*)"COSSMQ ZDPL",ZDPL
WRITE(*,*)"COSSMQ ZDPL",ZDPL
IF(NPRGPEW > 1) THEN
  CALL MPL_ALLREDUCE(ZDPL,'SUM',LDREPROD=.FALSE.,CDSTRING='COSSMQ:')
ENDIF
PCOL=ZDPL

!     2. horizontal part
!     ------------------
ISTLAT=1
IENDLAT=NDGENL

DO JGL=ISTLAT,IENDLAT
  IGLG=MYLATS(JGL)
  IROF=NSTAGP(JGL)-1
  ISTLON=NSTA(NPTRFLOFF+JGL,MY_REGION_EW)
  IENDLON=NSTA(NPTRFLOFF+JGL,MY_REGION_EW)+NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1
  DO JLON=ISTLON,IENDLON
    IROF=IROF+1
    ZGM=GM(IROF)
    ZGELAT=GELAT(IROF)
    ZGELAM=GELAM(IROF)
    ZSGLOBL=ZSGLOBL+RW(IGLG)/(ZGM**2*NLOENG(IGLG))

    IF(ALON1 < ALON3) THEN
      IF (ZGELAT >= ALAT3.AND.&
         & ZGELAT <= ALAT1.AND.&
         & (ZGELAM >= ALON1.AND.&
         & ZGELAM <= ALON3)    ) THEN  

        ZSDOML=ZSDOML+RW(IGLG)/(ZGM**2*NLOENG(IGLG))

      ENDIF
    ELSE
      IF (ZGELAT >= ALAT3.AND.&
         & ZGELAT <= ALAT1.AND.&
         & ((ZGELAM >= ALON1.AND.&
         & ZGELAM <= 2.0_JPRB*RPI).OR.&
         & (ZGELAM >= 0.0_JPRB  .AND.&
         & ZGELAM <= ALON3))  ) THEN  

        ZSDOML=ZSDOML+RW(IGLG)/(ZGM**2*NLOENG(IGLG))
      ENDIF
    ENDIF
  ENDDO
ENDDO

WRITE(NULOUT,*)"COSSMQ ZSGLOBL",ZSGLOBL
CALL MPL_ALLREDUCE(ZSGLOBL,'SUM',LDREPROD=.FALSE.,CDSTRING='COSSMQ:')
PGLOB = ZSGLOBL
CALL MPL_ALLREDUCE(ZSDOML,'SUM',LDREPROD=.FALSE.,CDSTRING='COSSMQ:')
PDOM = ZSDOML

IF (LHOOK) CALL DR_HOOK('COSSMQ',1,ZHOOK_HANDLE)
END SUBROUTINE COSSMQ
