SUBROUTINE CVAR2 (YDVAZX)

!     CVAR2 : Transformation of model fields

!     Purpose: Transform from model variables
!     --------           to Jb-normalized departures w.r.t. background

!     Externals: SBSBGS CVARU2 BALSTAT BALVERTI CVARU3
!     ----------

!     Author.    W. HECKLEY    *ECMWF*
!     -------

!     Modifications.
!     --------------
!        W. HECKLEY    91:09:30  Change in call to STEPO
!        Bill Heckley  06:02:92 - Provision for use of mass variable P
!        Bill Heckley  04:03:93 - Horizontal Jg correlation computation now a
!                                 part of the control variable.
!        Bill Heckley  07:06:93   separation of code into lower level routines
!        Mike Fisher   95-01-20   LSUBBG and CALL MLTEPS
!        Mike Fisher   95-09-20   LMVCHV switch
!        Mats Hamrud   95-12-15   Remove NFGM .ne. 1, cleaning
!        Mike Fisher   96-05-30   Reverse order of CVARU2 and MLTEPS
!        F. Bouttier   96-07-17   Clean comments
!        F. Bouttier   96-12-13   Implement LSTABAL option
!        Mike Fisher   97-02-21   Simplified Kalman Filter
!        M. Siroka     98-05-13   LELAM for LSTABAL option
!        Mike Fisher 2000-10-26   Jb vertical coordinate
!        Mike Fisher 2001-12-03   Omega-equation balance operator
!        Y.Tremolet  2002-08-05   Use control_vectors.
!        D. Dee      2003-12-12   Variational bias correction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher    17-Mar-2004  Wavelet Jb 
!        Y.Tremolet    18-Mar-2004 Model error change of variable
!        Y.Tremolet    10-Jun-2004 Initial condition optional in control vector
!        Y.Tremolet    27-Jul-2004 Improve model error
!        P.Burton :  29-09-04 LVARBC cleanup
!        C. Fischer    21-May-2005 Really loop on lam1d cv component and 
!                                  scale by its mean wind std dev
!        Elias Holm    08-10-20   Interface cvar3 includes gridpoint 
!                                 fields.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMJG    , ONLY : LSUBBG, TMEANUVER, LSPFCE
USE YOMCVA   , ONLY : YSCALPSQRT, YRSCALPSQRT, NVATOVV, NVA1D, NVA2D, NVA3D
USE YOMGEM   , ONLY : NGPTOT
USE YOMSPJB  , ONLY : SPJB
USE YOMSKF   , ONLY : LSKF, NSKFVECS, YSKFROT, YSKFMAT, SKFSUBMAT
USE YOMANCS  , ONLY : RMDI
USE YOMVAR   , ONLY : LTOVSCV,  LVARBC,  LTSCV
USE YOMTVRAD , ONLY : TOVSCVX, TOVSCVER,NVATOVLEN
USE YOMDIM   , ONLY : NFLEVL, NFLEVG, NPROMA
USE VARBC_SETUP, ONLY : YVARBC, NOBGROUP, NOBPARAM
USE YOMMODEL_ERROR, ONLY : NDIM_MODERR, SPMODERR, GPMODERR, SPCTLMODERR, SPGPMODERR
USE YOMJQ    , ONLY : LQEQB, LJQBAL
USE YOMDIMO , ONLY: NOBNTV
USE CONTROL_VECTORS
USE CONTROL_VECTORS_COMM_MOD, ONLY : ALLGATHER_CVSECTION, SET_CVSECTION
USE SPECTRAL_FIELDS_MOD
USE YOMTS, ONLY : TSCVX, TSCVER
USE YOMMP,  ONLY: MYLEVS

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZX 

TYPE(CONTROL_VECTOR) :: YLCV
INTEGER(KIND=JPIM) :: ii,ji,jj,jf,js,il,jg,ibeg,iend,INGPBLKS
REAL(KIND=JPRB) :: ZWORK1(NSKFVECS), ZWORK2(NSKFVECS)
REAL(KIND=JPRB) :: ZPS
REAL(KIND=JPRB), ALLOCATABLE :: ZGP3(:,:,:), ZGP2(:,:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL :: LLSPFCE
#include "abor1.intfb.h"
#include "cvarbc.intfb.h"
#include "sbsbgs.intfb.h"
#include "grid2spec.intfb.h"
#include "cvar3.intfb.h"
#include "spec_concat.intfb.h"

!     ------------------------------------------------------------------

!     1.    Convert X into X-Xb : subtract background
!           ------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVAR2',0,ZHOOK_HANDLE)
IF (LSUBBG) CALL SBSBGS

!     3.    Apply B matrix
!           --------------

! Parameters to be estimated

IF (LVARBC) THEN ! Obs bias parameters
  IF (NOBPARAM/=SIZE(YDVAZX%params)) &
   & CALL ABOR1('CVAR2: Parameter section of control variable wrong length')
  CALL CVARBC
  ibeg = 1
  DO jg = 1, NOBGROUP
    iend = ibeg + YVARBC(jg)%NPARAM - 1
    YDVAZX%params(ibeg:iend) = YVARBC(jg)%APARAMS
    ibeg = iend + 1
  ENDDO
ENDIF

! MEAN WIND (LAM)

IF (YDVAZX%lam1d) THEN
  ii=0
  DO jf=1,NVA1D
    DO js=1,NFLEVL
      il=MYLEVS(js)
      ii=ii+1
      IF (TMEANUVER(il,jf)/=0.) THEN
        YDVAZX%lamcv(ii) = SPJB%SP1D(js,jf) / TMEANUVER(il,jf)
      ELSE
        YDVAZX%lamcv(ii) = 0.0_JPRB
      ENDIF
    ENDDO
  ENDDO
ENDIF

! TOVS control variable

IF (YDVAZX%ltovscv) THEN
  ii=0
  DO jf=1,NVATOVV
    DO js=1,NVATOVLEN
      ii=ii+1
      IF (TOVSCVER(js,jf)/=RMDI) THEN
        YDVAZX%tovscv(ii) = TOVSCVX(js,jf) / TOVSCVER(js,jf)
      ELSE
        YDVAZX%tovscv(ii) = 0.0_JPRB
      ENDIF
    ENDDO
  ENDDO
ENDIF
! TS control variable

IF (YDVAZX%ltscv) THEN
  ii=0
    DO js=1,NOBNTV
      ii=ii+1
      IF (TSCVER/=RMDI) THEN
        YDVAZX%tscv(ii) = TSCVX(js) / TSCVER
      ELSE
        YDVAZX%tscv(ii) = 0.0_JPRB
      ENDIF
  ENDDO
ENDIF

! Model Error

INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA

ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))
ZGP3(:,:,:)=0.0_JPRB
ZGP2(:,:,:)=0.0_JPRB

IF (YDVAZX%nmoderr>0) THEN
  IF (LQEQB) THEN
    il=SPCTLMODERR%nspsizel
    CALL ALLOCATE_CTLVEC(YLCV,LDINIT=.true.,KMERR=0)
    IF (SIZE(YLCV%INITCV)/=il) CALL ABOR1('CVAR2: Error LQEQB')
    DO jj=1,NDIM_MODERR
      CALL GRID2SPEC(GPMODERR(jj),SPGPMODERR)
      CALL SPEC_CONCAT(SPMODERR(jj),SPGPMODERR,SPCTLMODERR)
      LLSPFCE=LSPFCE
      LSPFCE=.TRUE.
      CALL CVAR3(SPCTLMODERR,YLCV,ZGP3,ZGP2,0)
      LSPFCE=LLSPFCE
      ii=(jj-1)*il
      YDVAZX%MODERR(ii+1:ii+il)=YLCV%INITCV(:)
    ENDDO
    CALL DEALLOCATE_CTLVEC(YLCV)
  ELSE
    DO jj=1,NDIM_MODERR
      CALL GRID2SPEC(GPMODERR(jj),SPGPMODERR)
      CALL SPEC_CONCAT(SPMODERR(jj),SPGPMODERR,SPCTLMODERR)
      CALL CVAR3(SPCTLMODERR,YDVAZX,ZGP3,ZGP2,jj,LJQBAL)
    ENDDO
  ENDIF
ENDIF

! Initial condition
IF (YDVAZX%LINITCV) THEN
  CALL CVAR3(SPJB,YDVAZX,ZGP3,ZGP2,0)
ENDIF

DEALLOCATE (ZGP3)
DEALLOCATE (ZGP2)

!     4.    Simplified Kalman Filter
!           ------------------------
IF (LSKF) THEN
  YDVAZX%data = YDVAZX%data * YSCALPSQRT%data

!*     Apply the rotation matrix

  DO JJ=1,NSKFVECS
    ZPS = DOT_PRODUCT(YDVAZX,YSKFROT(JJ))
    YDVAZX%data = YDVAZX%data - ZPS*YSKFROT(JJ)%data
  ENDDO

!*     Apply the square root inverse covariance matrix

  CALL ALLGATHER_CVSECTION(YDVAZX,ZWORK1,1,NSKFVECS)

  DO jj=1,NSKFVECS
    YDVAZX%data(:) = YDVAZX%data(:) + YSKFMAT(jj)%data(:)*ZWORK1(jj)
  ENDDO

  ZWORK2(:)=0.0_JPRB
  DO ji=1,NSKFVECS
    DO jj=ji,NSKFVECS
      ZWORK2(ji) = ZWORK2(ji) + SKFSUBMAT(ji,jj)*ZWORK1(jj)
    ENDDO
  ENDDO

  CALL SET_CVSECTION(ZWORK2,YDVAZX,1,NSKFVECS)

!*     Apply the inverse of the rotation matrix

  DO JJ=NSKFVECS,1,-1
    ZPS = DOT_PRODUCT(YDVAZX,YSKFROT(JJ))
    YDVAZX%data = YDVAZX%data - ZPS*YSKFROT(JJ)%data
  ENDDO

  YDVAZX%data = YDVAZX%data * YRSCALPSQRT%data
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVAR2',1,ZHOOK_HANDLE)
END SUBROUTINE CVAR2
