SUBROUTINE CVAR2IN(YDVAZX)

!     CVAR2IN : Transformation of model fields, inverse

!     Purpose: Transform from Jb-normalized departures w.r.t. background
!     --------           to model variables

!     Externals: CVARU3I CVARU2I BALSTAT BALVERT ADDBGS
!     ----------

!     Author.    W. HECKLEY    *ECMWF*
!     -------

!     Modifications.
!     --------------
!        W. HECKLEY    91:09:30  Change in call to STEPO
!        Bill Heckley  06:02:92 - Provision for use of mass variable P
!        Bill Heckley  21:02:92 - Use of eigenvectors of vertical correlations
!        Bill Heckley  15:06:92 - Remove the 2 delta eta wave in the T feled
!        Bill Heckley  04:03:93 - Horizontal Jg correlation computation now a
!                                 part of the control variable.
!        Bill Heckley, Erik Andersson, Philippe Courtier 07:06:93 - revised
!                                 approach to 2 delta eta noise problem
!        Mike Fisher   95-01-20   LSUBBG and CALL MLTEPSIN
!        Mike Fisher   95-09-20   LMVCHV switch
!        Mats Hamrud   95-12-15   Remove NFGM .ne. 1,cleaning
!        Mike Fisher   96-05-30   Reverse order of CVARU2I and MLTEPSIN
!        F. Bouttier   96-07-17   Clean comments
!        John Derber   96-07-17   No call to MLTEPS if LSTABAL
!        John Derber   96-08-27   No call to CVARU2I if LSTABAL
!        F. Bouttier   96-12-13   Change STABAL scalar product
!        Mike Fisher   97-02-21   Simplified Kalman Filter
!        M. Siroka     98-05-13   LELAM and STABAL option
!        Mike Fisher 2000-10-26   Jb vertical coordinate
!        Mike Fisher 2001-12-03   Omega-equation balance operator
!        Y.Tremolet  2002-08-05   Use control_vectors.
!        D. Dee      2003-12-12   Variational bias correction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher      17-Mar-2004 Wavelet Jb
!        Y.Tremolet    18-Mar-2004 Model error change of variable
!        Y.Tremolet    10-Jun-2004 Initial condition optional in control vector
!        Y.Tremolet    27-Jul-2004 Improve model error
!        C.Soci        07-Oct-2004 set SPA1 to 0 when no initial condition
!        P.Burton :  29-09-04 LVARBC cleanup
!        C. Fischer    21-May-2005 really loop on lam1d cv component and scale by 
!                      its mean wind std dev
!     ------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK
USE YOMCT0   , ONLY : NCONF, LELAM
USE YOMJG    , ONLY : TMEANUVER, LSPFCE
USE YOMCVA   , ONLY : YSCALPSQRT, YRSCALPSQRT, NVATOVV, NVA1D, NVA2D, NVA3D
USE YOMGEM   , ONLY : NGPTOT
USE YOMSKF   , ONLY : LSKF, NSKFVECS, YSKFROT, YSKFMAT, SKFSUBMAT
USE YOMANCS  , ONLY : RMDI
USE YOMVAR   , ONLY : LTOVSCV,  LVARBC
USE YOMTVRAD , ONLY : TOVSCVX  ,TOVSCVER,NVATOVLEN
USE YOMDIM  ,  ONLY : NFLEVL, NFLEVG, NPROMA
USE YOMSPJB ,  ONLY : SPJB
USE VARBC_SETUP, ONLY : YVARBC, NOBGROUP, NOBPARAM
USE YOMMODEL_ERROR, ONLY : SPMODERR, GPMODERR, SPCTLMODERR, SPGPMODERR, &
                    & NDIM_MODERR
USE YOMJQ   ,  ONLY : LQEQB, LJQBAL
USE CONTROL_VECTORS
USE YOMDIMO , ONLY: NOBNTV
USE CONTROL_VECTORS_COMM_MOD, ONLY : ALLGATHER_CVSECTION, SET_CVSECTION
USE SPECTRAL_FIELDS_MOD
USE YOMTS,  ONLY: TSCVX, TSCVER
USE YOMMP,  ONLY: MYLEVS

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(IN) :: YDVAZX 

TYPE(control_vector) :: YLVAZX, YLCV
INTEGER(KIND=JPIM) :: ii,ji,jj,jf,js,il,jg,ibeg,iend,INGPBLKS
REAL(KIND=JPRB) :: ZWORK1(NSKFVECS)
REAL(KIND=JPRB) :: ZPS
REAL(KIND=JPRB), ALLOCATABLE :: ZGP3(:,:,:), ZGP2(:,:,:)
LOGICAL :: LL2DIMO, LLSPFCE
INTEGER(KIND=JPIM) :: ib(NOBGROUP),ie(NOBGROUP)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "cvar3in.intfb.h"
#include "spec2grid.intfb.h"
#include "cvarbcin.intfb.h"
#include "jbtomodel.intfb.h"
#include "spec_split.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVAR2IN',0,ZHOOK_HANDLE)
CALL ALLOCATE_CTLVEC(YLVAZX)

!     4.    Simplified Kalman Filter
!           ------------------------

IF (LSKF) THEN
  YLVAZX%data = YDVAZX%data * YSCALPSQRT%data

  DO JJ=1,NSKFVECS
    ZPS = DOT_PRODUCT(YLVAZX,YSKFROT(JJ))
    YLVAZX%data = YLVAZX%data - ZPS*YSKFROT(JJ)%data
  ENDDO

  CALL ALLGATHER_CVSECTION(YLVAZX,ZWORK1,1,NSKFVECS)

  DO JI=NSKFVECS,1,-1
    DO JJ=JI+1,NSKFVECS
      ZWORK1(JI) = ZWORK1(JI)-ZWORK1(JJ)*SKFSUBMAT(JI,JJ)
    ENDDO
    ZWORK1(JI) = ZWORK1(JI)/SKFSUBMAT(JI,JI)
  ENDDO

  DO jj=1,NSKFVECS
    YLVAZX%data = YLVAZX%data + YSKFMAT(jj)%data * ZWORK1(jj)
  ENDDO

  CALL SET_CVSECTION(ZWORK1,YLVAZX,1,NSKFVECS)

  DO JJ=NSKFVECS,1,-1
    ZPS = DOT_PRODUCT(YLVAZX,YSKFROT(JJ))
    YLVAZX%data = YLVAZX%data - ZPS*YSKFROT(JJ)%data
  ENDDO

  YLVAZX%data = YLVAZX%data * YRSCALPSQRT%data
ELSE
  YLVAZX = YDVAZX
ENDIF

!     3.    Apply B matrix (inverse). Result returned in SPA3JB and SPA2JB.  
!           -----------------------------------------------------------

! Initial condition

INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA

ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))

IF (YLVAZX%LINITCV) THEN
  CALL CVAR3IN(YLVAZX,SPJB,ZGP3,ZGP2,0)
ELSE
  SPJB=0.0_JPRB
  CALL GSTATS(1445,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jf)
  DO jf=1,NFLEVG*NVA3D
    ZGP3(:,jf,:)=0.0_JPRB
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1445,1)
  ZGP2=0.0_JPRB
ENDIF
CALL JBTOMODEL(SPJB,ZGP3,ZGP2)

! Model Error
IF (YLVAZX%nmoderr>0) THEN
  IF (LQEQB) THEN
    il=SPCTLMODERR%nspsizel
    CALL ALLOCATE_CTLVEC(YLCV,LDINIT=.true.,KMERR=0)
    IF (SIZE(YLCV%INITCV)/=il) CALL ABOR1('CVAR2IN: Error LQEQB')
    DO jj=1,NDIM_MODERR
      ii=(jj-1)*il
      YLCV%INITCV(:)=YLVAZX%MODERR(ii+1:ii+il)
      LLSPFCE=LSPFCE
      LSPFCE=.TRUE.
      CALL CVAR3IN(YLCV,SPCTLMODERR,ZGP3,ZGP2,0)
      LSPFCE=LLSPFCE
      CALL SPEC_SPLIT(SPCTLMODERR,SPMODERR(jj),SPGPMODERR)
      CALL SPEC2GRID(SPGPMODERR,GPMODERR(jj))
    ENDDO
    CALL DEALLOCATE_CTLVEC(YLCV)
  ELSE
    DO jj=1,NDIM_MODERR
      CALL CVAR3IN(YLVAZX,SPCTLMODERR,ZGP3,ZGP2,jj,LJQBAL)
      CALL SPEC_SPLIT(SPCTLMODERR,SPMODERR(jj),SPGPMODERR)
      CALL SPEC2GRID(SPGPMODERR,GPMODERR(jj))
    ENDDO
  ENDIF
ENDIF

DEALLOCATE (ZGP3)
DEALLOCATE (ZGP2)

! TOVS control variable

IF (YLVAZX%ltovscv) THEN
  DO jf=1,NVATOVV
    CALL GSTATS(1445,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(js,ii)    
    DO js=1,NVATOVLEN
      ii=(jf-1)*NVATOVLEN+js
      IF (TOVSCVER(js,jf)/=RMDI) THEN
        TOVSCVX(js,jf)=YLVAZX%tovscv(ii) * TOVSCVER(js,jf)
      ELSE
        TOVSCVX(js,jf)=0.0_JPRB
      ENDIF
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1445,1)
  ENDDO
ENDIF


! TS control variable

 
IF (YLVAZX%ltscv) THEN
  ii=0
  DO js=1,NOBNTV
    ii=ii+1
    IF (TSCVER/=RMDI) THEN
      TSCVX(js)=YLVAZX%tscv(ii) * TSCVER
    ELSE
      TSCVX(js)=0.0_JPRB
    ENDIF
  ENDDO
ENDIF


! MEAN WIND (LAM)

LL2DIMO = (NCONF/100 /= 1) .AND. (NCONF /= 801)

IF (YLVAZX%lam1d) THEN
  ii=0
  DO jf=1,NVA1D
    DO js=1,NFLEVL
      il=MYLEVS(js)
      ii=ii+1
      SPJB%SP1D(js,jf)=YLVAZX%lamcv(ii) * TMEANUVER(il,jf)
    ENDDO
  ENDDO
ELSEIF (LELAM .AND. LL2DIMO) THEN
  SPJB%SP1D(:,:)=0.0_JPRB
ENDIF

! Parameters to be estimated

IF (LVARBC) THEN ! Obs bias parameters
  IF (NOBPARAM/=SIZE(YLVAZX%params)) &
   & CALL ABOR1('CVAR2IN: Parameter section of control variable wrong length')
  ibeg = 1
  DO jg = 1, NOBGROUP
    iend = ibeg + YVARBC(jg)%NPARAM - 1
    ib(jg)=ibeg
    ie(jg)=iend
    ibeg = iend + 1
  ENDDO
  CALL GSTATS(1445,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jg)
  DO jg = 1, NOBGROUP
    YVARBC(jg)%APARAMS = YLVAZX%params(ib(jg):ie(jg))
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1445,1)
  CALL CVARBCIN
ENDIF

CALL DEALLOCATE_CTLVEC(YLVAZX)

IF (LHOOK) CALL DR_HOOK('CVAR2IN',1,ZHOOK_HANDLE)
END SUBROUTINE CVAR2IN
