SUBROUTINE cvarbc
!-----------------------------------------------------------------------
!   CVARBC - Change-of-variable for variational bias parameters

!   Purpose.
!   --------
!   Applies change-of-variable to the control vector section
!   associated with the observation bias parameters

!   Interface.
!   ----------
!   CALL cvarbc ()

!   Implicit input:
!      YVARBC

!   Implicit output:
!      YVARBC

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2004/03/19
!        P.Burton :  29-09-04 LVARBC cleanup
!-----------------------------------------------------------------------

USE parkind1, ONLY : JPIM, JPRB
USE yomhook,  ONLY : lhook, dr_hook
USE YOMJG   , ONLY : LSUBBG
USE varbc_setup, ONLY : YVARBC, NOBGROUP, JPMXNPARAM

IMPLICIT NONE

INTEGER(KIND=JPIM) :: jg, jp, inp
REAL(KIND=JPRB)    :: zp(JPMXNPARAM)
REAL(KIND=JPRB)    :: zhook_handle

IF (lhook) CALL DR_HOOK('CVARBC',0,zhook_handle)
DO jg = 1, NOBGROUP
  inp = YVARBC(jg)%NPARAM
  IF (LSUBBG) THEN
    zp(1:inp) = YVARBC(jg)%APARAMS + YVARBC(jg)%APARAM5 - YVARBC(jg)%APARAM0  
  ELSE
    zp(1:inp) = YVARBC(jg)%APARAMS
  ENDIF
  DO jp = 1, inp
    YVARBC(jg)%APARAMS(jp) = SUM(YVARBC(jg)%ACHGVAR(jp,:) * zp(1:inp))
  ENDDO
ENDDO
IF (lhook) CALL DR_HOOK('CVARBC',1,zhook_handle)

END SUBROUTINE cvarbc
