SUBROUTINE cvarbcad
!-----------------------------------------------------------------------
!   CVARBCAD - Adjoint change-of-variable for variational bias parameters

!   Purpose.
!   --------
!   Applies change-of-variable to the control vector section
!   associated with the observation bias parameters

!   Interface.
!   ----------
!   CALL cvarbcad ()

!   Implicit input:
!      YVARBC

!   Implicit output:
!      YVARBC

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original       : 19-Mar-2004
!   P.Burton       : 29-Sep-2004 LVARBC cleanup and
!                                 changed MPL_ALLREDUCE to LDREPROD=.true.
!   G. Mozdzynski  : 03-Jun-2009 VarBC reproducibility rework
!   T. Wilhelmsson : 26-Feb-2010 More VarBC reproducibility rework
!-----------------------------------------------------------------------

USE parkind1, ONLY : JPIM, JPRB
USE yomhook,  ONLY : lhook, dr_hook
USE varbc_setup, ONLY : YVARBC, NOBGROUP, JPMXNPARAM, NOBPARAM, APARAMS_COMP, NPARAMS_COMP
USE YOMVAR, ONLY : LREPRO4DVAR
USE YOMJG,  ONLY : LSUBBG
USE mpl_module
USE ORDER_INDEPENDENT_SUMMATION_MOD, ONLY : ORDER_INDEP_GLOBAL_SUM2

IMPLICIT NONE

INTEGER(KIND=JPIM) :: ibeg, iend, jg, jp, inp, inparams(JPMXNPARAM)
REAL(KIND=JPRB)    :: zp(JPMXNPARAM)
REAL(KIND=JPRB)    :: zhook_handle

REAL(KIND=JPRB), ALLOCATABLE :: zparams(:)

! Sum across processors

IF (lhook) CALL DR_HOOK('CVARBCAD',0,zhook_handle)

IF (.NOT.LREPRO4DVAR) THEN

  ALLOCATE(zparams(NOBPARAM))
  ibeg = 1
  DO jg = 1, NOBGROUP
    iend = ibeg + YVARBC(jg)%NPARAM - 1
    zparams(ibeg:iend) = YVARBC(jg)%APARAMS
    ibeg = iend + 1
  ENDDO
  CALL GSTATS(517,0)
  CALL MPL_ALLREDUCE(zparams, 'sum', ldreprod=.TRUE.,CDSTRING='CVARBCAD:')
  CALL GSTATS(517,1)
  ibeg = 1
  DO jg = 1, NOBGROUP
    iend = ibeg + YVARBC(jg)%NPARAM - 1
    YVARBC(jg)%APARAMS = zparams(ibeg:iend)
    ibeg = iend + 1
  ENDDO
  DEALLOCATE(zparams)

ELSE
  
  DO jg = 1, NOBGROUP
    inp = YVARBC(jg)%NPARAM
    CALL GSTATS(517,0)
    inparams(1:inp) = nparams_comp(1:inp,jg)
    CALL order_indep_global_sum2(aparams_comp(:,:,jg),zp,inp,1,inparams)
    YVARBC(jg)%APARAMS(1:inp) = YVARBC(jg)%APARAMS(1:inp) + zp(1:inp) 
    CALL GSTATS(517,1)
  ENDDO

  DEALLOCATE(APARAMS_COMP,NPARAMS_COMP) ! Was allocated in taskobad

ENDIF

DO jg = 1, NOBGROUP
  inp = YVARBC(jg)%NPARAM
  IF (LSUBBG) THEN
    zp(1:inp) = YVARBC(jg)%APARAMS + YVARBC(jg)%APARAM5 - YVARBC(jg)%APARAM0  
  ELSE
    zp(1:inp) = YVARBC(jg)%APARAMS
  ENDIF
  DO jp = 1, inp
    YVARBC(jg)%APARAMS(jp) = SUM(YVARBC(jg)%ACHGVAR(jp,:) * zp(1:inp))
  ENDDO
ENDDO
IF (lhook) CALL DR_HOOK('CVARBCAD',1,zhook_handle)

END SUBROUTINE cvarbcad
