SUBROUTINE CVARU2AD

!**** *CVARU2AD*    - TRANSFORMATION OF MODEL FIELDS  *ADJOINT*

!     PURPOSE.
!     --------

!        TRANSFORM FROM MODEL VARIABLES FROM T, ln Ps TO P

!**   INTERFACE.
!     ----------

!        *CALL* *CVARU2AD*

!        INPUT:

!        OUTPUT:

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!             *CVARU2AD* - convert T, ln(Ps) to mass field P

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        W. HECKLEY    *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL 07:06:93
!      L Isaksen  06:06:95  - Reordering of spectral arrays
!      L Isaksen  05:05:96  - Adapted for Message Passing
!      E Holm     20:08:97  - Call to trmtos/trstom includes ozone
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG, NSPEC2
USE YOMSPJB  , ONLY : SPJB
USE YOMMP    , ONLY : NSPEC2V, NPSP

!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZSPTG(NFLEVG,NSPEC2V), ZSPSPG (NSPEC2V)
REAL(KIND=JPRB) :: ZSPG(NFLEVG,NSPEC2V)
REAL(KIND=JPRB) :: ZSP(NSPEC2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sigamad.intfb.h"
#include "trmtos.intfb.h"
#include "trstom.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVARU2AD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1.    TRANSPOSE TEMPERATURE AND PRESSURE TO VERTICAL COLUMNS
!           ------------------------------------------------------

IF (NPSP==1) ZSP=SPJB%SPR
CALL TRMTOS(PSPT =SPJB%TEM,PSPSP =ZSP,&
 &          PSPTG=ZSPTG,PSPSPG=ZSPSPG)

!*    2.    CONVERT FROM TEMPERATURE AND ln(Ps) TO P
!           ----------------------------------------

ZSPG(:,:) = ZSPTG(:,:)
ZSPTG(:,:) = 0.0_JPRB
ZSPSPG(:) = 0.0_JPRB
CALL SIGAMAD(1,NFLEVG,ZSPG,ZSPTG,ZSPSPG,NSPEC2V)

!*    3.    TRANSPOSE TEMPERATURE AND PRESSURE BACK TO LEVEL STRUCTURE
!           ----------------------------------------------------------

CALL TRSTOM(PSPTG=ZSPTG,PSPSPG=ZSPSPG,&
 &          PSPT =SPJB%TEM,PSPSP=ZSP)
IF (NPSP==1) SPJB%SPR=ZSP

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVARU2AD',1,ZHOOK_HANDLE)
END SUBROUTINE CVARU2AD
