SUBROUTINE ECSET_THSAFE(KTSLOTNO,KOBTOT,KOFFTOT)

!**** *ECSET_THSAFE* - PREPARATION OF THE NON-CORRELATED OBSERVATION SETS.
!****           IN THIS VERSION THE OBS SETS ARE PREPARED AND THEIR
!****           IDENTIFICATION STORED IN TABLES.

!---- Please keep this routine always consistent with ecset.F90

!     PURPOSE.
!     -------
!           PREPARE THE OBSERVATIONS SET  BY SET . ONE SET
!     CONTAINS UP TO NMXLEN OBSERVATIONS OF THE SAME TYPE. THE
!     PREPARATION INCLUDES THE EXTRACTION OF THE POSITION NUMBER
!     IN OBA FOR EACH OBSERVATION OF THE SET  AND STORAGE IN AN
!     AD HOC TABLE TO BE PASSED TO THE AD HOC ROUTINE TREATING THE
!     APPROPRIATE OBS TYPE ON THE APPROPRIATE PROCESSOR (DONE BY OBSV).

!**   INTERFACE.
!**   ---------
!**   *CALL* * ECSET_THSAFE*
!**         THE COMMON YOMOBA IS AN IMPLICIT INPUT OF ECSET_THSAFE BECAUSE
!**         OF THE DIFFERENT POINTERS NEEDED TO FETCH THE OBS NUMBERS.
!**         THE COMMON YOMOBSET_THSAFE IS AN IMPLICIT OUTPUT OF ECSET_THSAFE AS IT
!**         CONTAINS THE DESCRIPTION OFTHE OBS SETS.

!     EXTERNALS.  NONE
!     ----------

!     AUTHOR: JEAN PAILLEUX AND PATRICK MOLL
!     DATE  : 90-11-23

!     MODIFICATIONS
!     -------------
!        D. Vasiljevic : 91-02-10
!        M. Hamrud     : 95-09-18 New CMA format
!     E.Andersson 960201 : Incorporating Scat under SURFACEO
!     R. Randriamampianina: 97-04-17 ALADIN 3DVAR
!     E.Andersson 010207 : Fully vectorized version.
!                          Sets defined according to SAT_data_groups
!        S. Saarinen   : 13-Jun-2002 KSETs cannot anymore span over timeslots
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        S. Saarinen   : 15-Dec-2004 initial version hacked from ecset.F90
!        N.Bormann     01-Mar-2005 RARAD becomes LIMB; NLIMBGRP sets for LIMB
!        D.Tan         20-Mar-2007 Increase INGRP(7) for Aeolus
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULERR   ,NULOUT
USE YOMDIMO  , ONLY : NMXSET   ,NMXLEN   
USE YOMCMA   , ONLY : NMXOTP
USE YOMCT0   , ONLY : LELAM    
USE YOMOBSET_THSAFE
USE YOMCOCTP , ONLY : NSYNOP   ,NAIREP   ,NSATOB   ,NDRIBU   ,&
 & NTEMP    ,NPILOT   ,NSATEM   ,NPAOB    ,&
 & NSCATT   ,NRADAR, NLIMB  
USE YOMANCS  , ONLY : RMDI
USE YOMDB
USE YOMTVRAD , ONLY : NSATGRP  ,NSATOBGRP
USE YOMLIMB  , ONLY : NLIMBGRP
USE OML_MOD, ONLY : OML_MY_THREAD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTSLOTNO
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBTOT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOFFTOT 
INTEGER(KIND=JPIM) :: ITYPLIST(NMXOTP),IOTP_GRP(KOBTOT),INGRP(NMXOTP)
LOGICAL   :: LLDONE(KOBTOT)

INTEGER(KIND=JPIM) :: ILISTLEN,ISET,IOTP
INTEGER(KIND=JPIM) :: IBINTAG,ICOUNT,IREST,INSET,ISTSET,IENDSET
INTEGER(KIND=JPIM) :: ISCALE,IGRP,INSETOT,IC
INTEGER(KIND=JPIM) :: JOTP,JGRP,JOBS,JSET
INTEGER(KIND=JPIM) :: INFO(0), IRET
REAL(KIND=JPRB) :: ZINFO(0)

INTEGER(KIND=JPIM) :: INUMTSL, JT, ITSLOFF
INTEGER(KIND=JPIM), ALLOCATABLE :: JSTA(:), JEND(:)
INTEGER(KIND=JPIM) :: ITID

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -------------------------------------------------------------
!*       1. INITIALIZE
!*          ----------

!*          1.1   Initialize ODB

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "abortdb.h"
END INTERFACE

#include "abor1.intfb.h"
!include "settc.intfb.h"

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('ECSET_THSAFE',0,ZHOOK_HANDLE)
ITID = OML_MY_THREAD()
CALL GETDB('ECSET', 0, IRET, INFO, 0,ZINFO,0, &
 & -1, KTSLOTNO, -1, -1, -1, -1)  

IF (IRET /= KOBTOT) THEN
  WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
  WRITE(NULERR,*) 'For timeslot=',KTSLOTNO,' expecting =',KOBTOT, &
   & ', but got =',IRET  
  CALL ABORTDB('ECSET_THSAFE','INCONSISTENCIES FROM GETDB',IRET)
ENDIF

!*          1.1.1 Define parameters for TSLOT-loop to enforce only one TSLOT per KSET
!                 Needed for multipool approach of ODB
!                 Note: We assume that the 'ecset.sql' has been sorted w.r.t. TSLOT-parameter

IF (KTSLOTNO > 0) THEN
  INUMTSL = 1
  ALLOCATE(JSTA(INUMTSL))
  ALLOCATE(JEND(INUMTSL))
  JSTA(1) = 1
  JEND(1) = KOBTOT
  ITSLOFF = KTSLOTNO - 1 ! Used to offset the JT
ELSE
  IF(KOBTOT > 0) THEN
    INUMTSL = MAXVAL(ROBSU(1:KOBTOT,MDB_TSLOT_AT_INDEX))
  ELSE
    INUMTSL = 0
  ENDIF
  ALLOCATE(JSTA(INUMTSL))
  ALLOCATE(JEND(INUMTSL))
  ITSLOFF = 0 ! Used to offset the JT
  JOBS=0
  DO JT=1,INUMTSL
!=== Note: We assume that the 'ecset.sql' has been sorted w.r.t. TSLOT-parameter
    IF (JT == 1) THEN
      IF (KOBTOT>0) THEN
        IC = COUNT(ROBSU(1:KOBTOT,MDB_TSLOT_AT_INDEX) == JT+ITSLOFF)
      ELSE
        IC = 0
      ENDIF
    ELSE ! JT > 1
      IF (KOBTOT>0) THEN
        IC = COUNT(ROBSU(JEND(JT-1)+1:KOBTOT,MDB_TSLOT_AT_INDEX) == JT+ITSLOFF)
      ELSE
        IC = 0
      ENDIF
    ENDIF
    JSTA(JT) = JOBS + 1
    JEND(JT) = JOBS + IC
    JOBS = JOBS + IC
  ENDDO
  IF (JOBS /= KOBTOT) THEN
    WRITE(NULERR,*) '*** ERROR IN ECSET_THSAFE: JOBS /= KOBTOT'
    WRITE(NULERR,*) '    JOBS, KOBTOT=',JOBS, KOBTOT
    WRITE(NULERR,*) '    In ecset.sql, dada possibly not sorted w.r.t. TSLOT-parameter'
    CALL ABORTDB('ECSET_THSAFE','FATAL PROGRAMMING ERROR',JOBS)
  ENDIF
ENDIF

WRITE(NULOUT,'(1x,a,2i6,i12)')'ECSET_THSAFE: KTSLOTNO, INUMTSL, KOBTOT=',KTSLOTNO, INUMTSL, KOBTOT
DO JT=1,INUMTSL
  WRITE(NULOUT,'(1x,a,2i6,2i12)')'>> JT, TIMESLOT#,  JSTA(JT), JEND(JT)=',&
   & JT, JT+ITSLOFF, JSTA(JT), JEND(JT)  
ENDDO
!CALL FLUSH(NULOUT)

!*          1.2   Define the order of processing

!     This list defines the reverse order in which the sets are 
!     processed in TASKOB.
!     We want the cheapest (scat) done last and the dearest (radiances) first.
!     For some observation types the obs_operators require that certain
!     subgroups are treated separately.
!     RTTOV for example, can only do one SatID at a time.

ITYPLIST( 1)=NSCATT   ; INGRP( 1)=1
ITYPLIST( 2)=NAIREP   ; INGRP( 2)=1
ITYPLIST( 3)=NDRIBU   ; INGRP( 3)=1
ITYPLIST( 4)=NPAOB    ; INGRP( 4)=1
ITYPLIST( 5)=NSYNOP   ; INGRP( 5)=1
ITYPLIST( 6)=NSATOB   ; INGRP( 6)=NSATOBGRP
ITYPLIST( 7)=NPILOT   ; INGRP( 7)=1+1 ! Increase for Aeolus
ITYPLIST( 8)=NTEMP    ; INGRP( 8)=1
ITYPLIST( 9)=NLIMB    ; INGRP( 9)=NLIMBGRP
ITYPLIST(10)=NRADAR   ; INGRP(10)=1
ITYPLIST(11)=NSATEM   ; INGRP(11)=NSATGRP
ILISTLEN=11

!*          1.3   The bins are charaterized by their ObsType/Group combination
!                 These will be called BinTags in the following

ISCALE=100
DO JOBS=1,KOBTOT
  IGRP    = ROBSU(JOBS,MDB_AREATYPE_AT_HDR)
  IOTP_GRP(JOBS) = IGRP*ISCALE + ROBSU(JOBS,MDBOTP)
  LLDONE(JOBS)=.FALSE.
ENDDO

!*          1.4   LELAM does not want some data to belong to sets.

IF(LELAM) THEN
  WHERE(ROBSU(1:KOBTOT,MDBTLA) == RMDI)
    IOTP_GRP(1:KOBTOT) = RMDI
    LLDONE(1:KOBTOT)=.TRUE.
  ENDWHERE
ENDIF

!     -------------------------------------------------------------
!*       2. Divide into bins. Divide bins into sets. Vectorized.
!*          ----------------------------------------------------

OBSET(ITID)%MLNSET(:)=NMXLEN
OBSET(ITID)%MTSLSET(:) = -1
INSETOT=0

!*          2.1   Loop ObsTypes and Groups
!                 Count bins and numbe of data in each bin.
!                 If bin contains more than NMXLEN data it is subdivided

DO JOTP=1,ILISTLEN
  IOTP=ITYPLIST(JOTP)
  DO JT=1,INUMTSL
    IF (JEND(JT) - JSTA(JT) + 1 <= 0) CYCLE ! DO JT=1,INUMTSL -loop
    DO JGRP=1,INGRP(JOTP)
      IBINTAG = JGRP*ISCALE + IOTP

      ICOUNT=COUNT(IOTP_GRP(JSTA(JT):JEND(JT)) == IBINTAG)
      IF(ICOUNT == 0) CYCLE ! DO JGRP=1,INGRP(JOTP) -loop

      IF(ICOUNT <= NMXLEN) THEN
        INSET=1
        IREST=ICOUNT
      ELSE
        INSET=ICOUNT/NMXLEN
        IREST=MOD(ICOUNT,NMXLEN)
        IF(IREST /= 0) INSET=INSET+1
      ENDIF

!*          2.2   Start set number and
!                 and end set number

      ISTSET=INSETOT+1
      IENDSET=INSETOT+INSET
      INSETOT=IENDSET
      IF(INSETOT > NMXSET) THEN
        WRITE(NULERR,'(A,I10)')&
         & ' TOO MANY OBSERVATION SETS, INCREASE NMXSET IN YOMDIMO',NMXSET  
        CALL ABOR1('ECSET_THSAFE: NMXSET TOO SMALL')
      ENDIF

!*          2.3   Length of set, Type of observation in set, and a timeslot the set belongs to

      IF(IREST /= 0) OBSET(ITID)%MLNSET(IENDSET) = IREST
      OBSET(ITID)%MTYPOB(ISTSET:IENDSET) = IBINTAG
      OBSET(ITID)%MTSLSET(ISTSET:IENDSET) = JT+ITSLOFF

!*          2.4   Record in ODB which set each obs belongs to.
!                 Create the so_called 'absolute obs number'.

      IC=0
      DO JOBS=JSTA(JT),JEND(JT)
        IF(IOTP_GRP(JOBS) == IBINTAG) THEN
          IC=IC+1
          ISET=ISTSET+(IC-1)/NMXLEN
          ROBSU(JOBS,MDB_KSET_AT_INDEX) = ISET
          LLDONE(JOBS)=.TRUE.
        ENDIF
      ENDDO ! DO JOBS=JSTA(JT),JEND(JT)
    ENDDO ! DO JGRP=1,INGRP(JOTP)
  ENDDO ! DO JT=1,INUMTSL
ENDDO ! DO JOTP=1,ILISTLEN

!     -------------------------------------------------------------
!*       3. ALMOST FINISHED
!*          ---------------

!*          3.1   Total number of sets created.

OBSET(ITID)%NSETOT = INSETOT
OBSET(ITID)%MLNSET(INSETOT+1:NMXSET)=0

!*          3.2   Check that no obs have slipped through the net.
!                 If this happens, go and check the group-definitions in
!                 suobarea.F90, and the INGRP-list above.

ICOUNT = COUNT(LLDONE)
IF(ICOUNT /= KOBTOT) THEN
  WRITE(NULERR,*)'***Error in ECSET_THSAFE: NOT ALL DATA ASSIGNED TO A SET : KTSLOTNO,KOBTOT,KOFFTOT,ICOUNT=',&
   & KTSLOTNO,KOBTOT,KOFFTOT,ICOUNT  
  CALL ABOR1('ECSET_THSAFE: NOT ALL DATA ASSIGNED TO A SET')
ENDIF

!*          3.3   Keep info about max number of data_items per obs, in set.
!                 Used for dimensioning in hop-routines.

DO JSET=1,INSETOT
  OBSET(ITID)%MMXBDY(JSET) = MAXVAL(ROBSU(1:KOBTOT,MLNK_HDR2BODY(2)),   &
   & MASK=ROBSU(1:KOBTOT,MDB_KSET_AT_INDEX) == JSET)  
ENDDO

!*          3.3   Print (in French and FORTRAN) ; why not in Finnish, too ?

!WRITE(NULOUT,*) '  '
!WRITE(NULOUT,*) ' RESULTAT DE LA FABRICATION DES PAQUETS D OBS : OBSET(ITID)%NSETOT=',OBSET(ITID)%NSETOT
!WRITE(NULOUT,*) '  '
!DO JSET = 1 , OBSET(ITID)%NSETOT
!  WRITE(NULOUT,*) ' PAQUET NO ',JSET,',  TYPE NO ',OBSET(ITID)%MTYPOB(JSET),&
!   &',  CONTIENT ',OBSET(ITID)%MLNSET(JSET),' OBS','   OBSET(ITID)%MMXBDY(',JSET,')=',OBSET(ITID)%MMXBDY(JSET)
!ENDDO

DEALLOCATE(JSTA,JEND)

!*          3.4   Close ODB

CALL PUTDB('ECSET', 0, IRET, INFO, 0,ZINFO,0)

!     -------------------------------------------------------------
!*       4. PREPARE FOR TIME CORRELATION
!*          ----------------------------

!-- not applicable to thread safe version; involves communication
!IF (LTC) CALL SETTC

IF (LHOOK) CALL DR_HOOK('ECSET_THSAFE',1,ZHOOK_HANDLE)
END SUBROUTINE ECSET_THSAFE
