SUBROUTINE EVCOST(PZF)

!**** *EVCOST*  - SYNTHESIS OF THE COST FUNCTION

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *EVCOST(PZF)

!        Explicit arguments :
!        --------------------
!        PZF - value of the cost function

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        PRTJO

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-10-15
!        Modified by D. Vasiljevic: 90-11-19
!        Modified by F. Rabier and J.N Thepaut: 91-01-15
!        Modified by E. Andersson: 91-10-08
!        Modified by L.Labbe: 93-03-10 (Add Jb term for NCONF=821)
!        E. Andersson: 94-03-28 Make Jo accumulation reproducible
!        M. Hamrud   : 95-10-09 New CMA format
!        L. Isaksen  : 96-05-05 Adapted for Message Passing
!        F. Bouttier : 96-05-30 Improve GREPCOST printout
!        R. El Khatib: 97-12-05 LFI portability
!        L. Dragulanescu : 97 MAY 3DVAR ALADIN
!        E.Andersson   98-08-18 TOVS control variable
!        C. Fischer : 99-06-21 nvarmax for global cost function (aladin)
!        W.Sadiki+C.Soci: 99-07-09 llamtovs to control obatabs for aladin/TOVS
!        D. Dee      : 03-11-25 Jp for variational parameters corrections
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        V. Guidard    01-Apr-2004 bugfix on FUVB + Add jk event
!        B. Chapnik    22-sep-2008 ad 1D contribution to Jc (lelam case)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NF3D     ,NFD2D    ,&
 & NVARMAX  
USE YOMDIMO  , ONLY : NOBTOTG
USE YOMCMA   , ONLY : NMXOTP
USE YOMCT0   , ONLY :     LGUESS   ,LOBS     ,LOBSC1   ,&
 & LSIMOB   ,NPRINTLEV,LELAM  
USE YOMCOSJC , ONLY : FJCCOST  ,RINCLJC
USE YOMCOSJB , ONLY : FJBCOST
USE YOMCOSJO , ONLY : JOT      ,FJOS
USE YOMCOSJR , ONLY : FALTSEL  ,FSURSEL  ,FSPALT   ,FSPSUR
USE YEMCOSJR , ONLY : FUVB     ,FUVBSEL
USE YOMMP    , ONLY : MYPROC
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMVAR   , ONLY : NSIM4D   ,NDIAG    ,NSIM4DL  ,NITER4D  ,LJC     ,GCVJO    ,LJCDFI   ,LCONGRAD ,LAVCGL ,&
 & LUSEJCDFI, LUSEJCNMI, LMODERR , LAVARC  ,LREPRO4DVAR
USE YOMCVA   , ONLY :   NVAPARAM
USE YOMJCDFI  , ONLY: RSUMJCDFI   ,RSUMJCDFI1D
USE YEMJK    , ONLY : LEJK
USE YOMJQ    , ONLY : FJQCOST
USE VARBC_SETUP, ONLY : AFJPCOST
USE ORDER_INDEPENDENT_SUMMATION_MOD , ONLY : ORDER_INDEP_GLOBAL_SUM
USE MPL_MODULE, ONLY : MPL_ALLREDUCE

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZF 
REAL(KIND=JPRB) :: ZSPALTG(NFLEVG,NF3D,0:NVARMAX)
REAL(KIND=JPRB) :: ZSPSURG(NFD2D,0:NVARMAX)
REAL(KIND=JPRB),ALLOCATABLE :: ZFCCOSTG(:,:)

INTEGER(KIND=JPIM) ::   &
 & JLEV, JM, JOTP, JVAR ,JCTP  

LOGICAL :: LLPRINT

REAL(KIND=JPRB) :: Z, ZDIF, ZFCC, ZJCDFI, ZZCOST, ZZF,&
 & ZZFJCA, ZZFJCS, ZZFJCT, ZZFJGT, ZSUMFJOS, ZSUMJOT
REAL(KIND=JPRB) :: ZHOOK_HANDLE


#include "abor1.intfb.h"
#include "gathercost1.intfb.h"
#include "gathercost2.intfb.h"
#include "gathercosto.intfb.h"
#include "prtjo.intfb.h"
!     ------------------------------------------------------------------

!*       1.       SYNTHESIS
!                 ---------

IF (LHOOK) CALL DR_HOOK('EVCOST',0,ZHOOK_HANDLE)
IF( ALLOCATED(FJCCOST) )THEN
  ALLOCATE (ZFCCOSTG(SIZE(FJCCOST,1),0:NVARMAX))
ELSE
  ALLOCATE (ZFCCOSTG(1,1))
ENDIF

PZF=0.0_JPRB
IF(LOBSC1) THEN
  LJC=.FALSE.
  LUSEJCDFI=.FALSE.
  LGUESS=.FALSE.
  LSIMOB=.FALSE.
ENDIF
LLPRINT=NPRINTLEV >= 1

!*          1.1   GATHER CONTRIBUTIONS TO COST FUNCTIONS FROM OTHER PE'S

IF (LJC.OR.(.NOT.LSIMOB.AND..NOT.LOBSC1).OR.(LGUESS)) THEN
  CALL GATHERCOST1(SIZE(FJCCOST,1),FJCCOST,ZFCCOSTG)
ELSE
  ZFCCOSTG(:,:)=HUGE(Z)
ENDIF

IF(LOBS.AND..NOT.LSIMOB) THEN
  CALL GATHERCOSTO
ENDIF

!*          1.2   JC

ZZFJCT=0.0_JPRB
IF (LJC) THEN

  ZZFJCA = 0.0_JPRB
  DO JVAR=1,NF3D
    DO JM=0,NVARMAX
      ZZFJCA = ZZFJCA +ZFCCOSTG(JVAR,JM)*RINCLJC(JVAR)
    ENDDO
  ENDDO

  ZZFJCS = 0.0_JPRB
  DO JVAR=1,NFD2D
    DO JM=0,NVARMAX
      ZZFJCS = ZZFJCS+ ZFCCOSTG(NF3D+JVAR,JM)*RINCLJC(NF3D+JVAR)
    ENDDO
  ENDDO

  ZZFJCT = ZZFJCA + ZZFJCS
  IF(LUSEJCNMI) THEN
    PZF = PZF + ZZFJCT
  ENDIF

  IF(LLPRINT) THEN
    WRITE(UNIT=NULOUT,FMT='(1X,''CONSTR. FUNC. JC:''&
     & ,E12.6,'' (ALT) '',E12.6,'' (SURF)   ++''&
     & ,'' NSIM4D= '',I4,'' ++'')') ZZFJCA,ZZFJCS,NSIM4D  
    DO JVAR=1,NF3D+NFD2D
      ZFCC=0.0_JPRB
      DO JM=0,NVARMAX
        ZFCC=ZFCC+ZFCCOSTG(JVAR,JM)*RINCLJC(JVAR)
!             WRITE(UNIT=NULOUT,FMT='(1X,''JVAR = '',I3,1X,''JM = '',
!    S             I3,1X,E12.6)')JVAR,JM,ZFCCOSTG(JVAR,JM)*RINCLJC(JVAR)
!    S             ZFCC
      ENDDO
      WRITE(UNIT=NULOUT,FMT='(1X,''JVAR = '',I3,1X,E12.6)')JVAR,ZFCC
    ENDDO
  ENDIF
ENDIF

!     Jc based on DFI

ZJCDFI = 0.0_JPRB
IF (LJCDFI) THEN
   ZJCDFI = 0.5_JPRB*SUM(RSUMJCDFI(:))
  !ZJCDFI = SUM(RSUMJCDFI(:)) ! bc 0.5 factor already in evjcdfi
    !Add the mean wind components
  ZJCDFI=ZJCDFI+RSUMJCDFI1D
  WRITE(NULOUT,*)'Jc-DFI cost function = ',ZJCDFI,'--   NSIM4D = ',NSIM4D
  IF (LUSEJCDFI) PZF = PZF + ZJCDFI
ENDIF
WRITE(NULOUT,FMT='(1X,''EVCOST3  = '',I3,1X,&
 & ''PZF = '',E22.16)') NSIM4D,PZF  

!*          1.3   JB

ZZFJGT=0.0_JPRB
IF ((.NOT.LSIMOB.AND..NOT.LOBSC1).OR.(LGUESS)) THEN
  ZZF=PZF
  PZF=PZF+FJBCOST
  ZZFJGT=PZF-ZZF

  WRITE(UNIT=NULOUT,FMT='(1X,''DIST. FUNC. JB='',E12.6, &
   & '' NSIM4D= '',I4,'' ++'')')  ZZFJGT,NSIM4D  
ENDIF

!*          1.3.5 JQ

IF (LMODERR) THEN
  WRITE(NULOUT,*)'Jq cost function = ',FJQCOST,'--   NSIM4D = ',NSIM4D
  PZF = PZF + FJQCOST
ENDIF

!*          1.3.6 JP

IF (NVAPARAM>0) THEN
  WRITE(NULOUT,*)'Jp cost function = ',AFJPCOST,'--   NSIM4D = ',NSIM4D
  PZF = PZF + AFJPCOST
ENDIF

!*          1.4   JO

IF (LOBS .AND. .NOT.LSIMOB) THEN

!        SUM DIAGNOSTIC JO TABLE
  CALL GSTATS(1961,0)
  IF(MYPROC == 1) THEN
    ZSUMJOT=0.0_JPRB
    DO JOTP=1,NMXOTP
      DO JCTP=1,JOT(JOTP)%size
        ZSUMJOT=ZSUMJOT+SUM(JOT(JOTP)%jost(JCTP)%cost(:,:))
      ENDDO
    ENDDO
  ENDIF
  CALL GSTATS(1961,1)

!        SUM ACTUAL JO TABLE
  CALL GSTATS_BARRIER(725)
  CALL GSTATS(813,0)
  IF (LREPRO4DVAR) THEN
    ZSUMFJOS = ORDER_INDEP_GLOBAL_SUM(FJOS,KNG=NOBTOTG)
  ELSE
    ZSUMFJOS = SUM(FJOS)
    CALL MPL_ALLREDUCE(ZSUMFJOS,'SUM',LDREPROD=.TRUE.,&
     & CDSTRING='EVCOST')
  ENDIF
  CALL GSTATS(813,1)
  CALL GSTATS_BARRIER2(725)
  PZF=PZF+ZSUMFJOS
  GCVJO=ZSUMFJOS

  IF(MYPROC == 1) THEN
    ZDIF = 1.E-8_JPRB
    IF(ABS(ZSUMJOT-ZSUMFJOS)/MAX(1.0_JPRB,ZSUMFJOS) > ZDIF) THEN
      WRITE(NULERR,*) ' TWO ESTIMATES OF JO: ',ZSUMJOT,ZSUMFJOS
      WRITE(NULERR,*) ' THE DIAGNOSTIC JOT TABLE',&
       & ' AND THE ACTUAL JO ACCUMULATION DO NOT AGREE'  
      CALL ABOR1('EVCOST: DIAGNOSTIC JOT TABLE AND ACTUAL '//&
       & 'JO ACCUMULATION DO NOT AGREE')  
    ENDIF
  ENDIF
  WRITE(UNIT=NULOUT,FMT='(1X,''OBS DISTANCE FUNCTION JO=''&
   & ,E22.16,'' NSIM4D= '',I4,'' =='')') ZSUMFJOS,NSIM4D  

!             Summarize all cost functions

  IF (.NOT.(LCONGRAD.OR.LAVCGL) .OR. NITER4D <= 5 .OR. NSIM4D==NSIM4DL) THEN
    WRITE(NULOUT,'('' GREPCOST - ITER,SIM,JO,JB,JC,JQ,JP '',&
     & 2(1X,I3),5(1X,G19.12))')&
     & NITER4D,NSIM4D,ZSUMFJOS,ZZFJGT,ZZFJCT+ZJCDFI,FJQCOST,AFJPCOST  
  ENDIF

  IF(NITER4D <= 5 .OR. NSIM4D==NSIM4DL) THEN
    WRITE(NULOUT,'(3X,A,2(1X,I3),6(1X,Z17))')&
     & 'GREPCOST_hex - ITER,SIM,JO,JB,JC,JC-DFI,JQ,JP'&
     & ,NITER4D,NSIM4D,ZSUMFJOS,ZZFJGT,ZZFJCT,ZJCDFI,FJQCOST,AFJPCOST  
  ENDIF

  IF( MYPROC == 1 .AND. NDIAG /= 0 .AND. (NSIM4D <= 1.OR.NSIM4D == NSIM4DL)) THEN
    CALL PRTJO
  ENDIF

ENDIF

!*          1.5   JR OR JK

IF (LSIMOB.OR.LAVARC .OR. LEJK) THEN

  IF (NDIAG /= 0) THEN
    CALL GATHERCOST2(FSPALT,FSPSUR,ZSPALTG,ZSPSURG)
  ENDIF

! ------   TOTAL DISTANCE FUNCTION --------------------------------

  DO JVAR=1,NF3D
    DO JLEV=1,NFLEVG
      DO JM=0,NVARMAX
        PZF=PZF+ZSPALTG(JLEV,JVAR,JM)*FALTSEL(JLEV,JVAR)
      ENDDO
    ENDDO
  ENDDO
  DO JVAR=1,NFD2D
    DO JM=0,NVARMAX
      PZF=PZF+ZSPSURG(JVAR,JM)*FSURSEL(JVAR)
    ENDDO
  ENDDO
  IF (LELAM) THEN
    DO JVAR=1,2
      DO JLEV=1,NFLEVG
        PZF=PZF+FUVB(JLEV,JVAR)+FUVBSEL(JVAR)
      ENDDO
    ENDDO
  ENDIF

  IF (NDIAG/=0) THEN
    WRITE(NULOUT,FMT='(1X,''COST FUNCTION (SIMUL. = '',I3,1X,&
     & '') = '',E12.6)') NSIM4D,PZF  
    WRITE(NULOUT,FMT='(1X,''== DISTANCE FUNCTION JR=='')')
    DO  JVAR=1,NF3D
      ZZCOST=0.0_JPRB
      DO JM=0,NVARMAX
        DO JLEV=1,NFLEVG
          ZZCOST=ZZCOST+ZSPALTG(JLEV,JVAR,JM)*FALTSEL(JLEV,JVAR)
        ENDDO
      ENDDO
      WRITE(UNIT=NULOUT,FMT='(1X,''ALT JVAR = '',I3,'' COST = '',&
       & E12.6)') JVAR,ZZCOST  
    ENDDO
    DO  JVAR=1,NFD2D
      ZZCOST=0.0_JPRB
      DO JM=0,NVARMAX
        ZZCOST=ZZCOST+ZSPSURG(JVAR,JM)*FSURSEL(JVAR)
      ENDDO
      WRITE(UNIT=NULOUT,FMT='(1X,''SUR JVAR = '',I3,'' COST = '',&
       & E12.6)') JVAR,ZZCOST  
    ENDDO
    IF (LELAM) THEN
      DO JVAR=1,2
        WRITE(UNIT=NULOUT,FMT='(1X,''JVAR = '',I3,I3)') JVAR
        WRITE(UNIT=NULOUT,FMT='(1X,E12.6)')(FUVB(JLEV,JVAR), JLEV=1,NFLEVG)
      ENDDO
    ENDIF

  ENDIF
ENDIF

IF (ALLOCATED(ZFCCOSTG)) DEALLOCATE(ZFCCOSTG)

!     ---------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EVCOST',1,ZHOOK_HANDLE)
END SUBROUTINE EVCOST
