SUBROUTINE EVJQ

!     Purpose.
!     --------
!       Compute Jq (TL and AD).

!     Author.
!     -------
!       Y.Tremolet

!     Modifications.
!     --------------
!        Original : 22-Aug-2005
!        Elias Holm    08-10-20  Interface cvar3/ad includes gridpoint
!                                fields.
!     ------------------------------------------------------------------
!        G. Radnoti: 2006/12/15: Q=B implementation
USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULOUT
USE YOMVAR  , ONLY: NUPTRA
USE YOMJQ   , ONLY: FJQCOST, SPJQMODERR, SPJQMODER5, LQEQB
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, NDIM_MODERR, NPRTMODERR, SPCTLMODERR
USE YOMJG, ONLY : LSPFCE
USE YOMDIM  ,  ONLY : NFLEVG, NPROMA
USE YOMCVA   , ONLY : NVA2D, NVA3D
USE YOMGEM   , ONLY : NGPTOT
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD
IMPLICIT NONE

TYPE(CONTROL_VECTOR) :: YLCV,YLCV_INIT,YLCV_INIT_tmp,YLCV_INIT_tmp1
INTEGER(KIND=JPIM) :: jj,ii,il,INGPBLKS
LOGICAL :: llf=.false.
LOGICAL :: ll_chk_inv=.false., LLSPFCE
REAL(KIND=JPRB) :: zjb, zjt, zjl, zjp, zjs
REAL(KIND=JPRB), ALLOCATABLE :: ZGP3(:,:,:), ZGP2(:,:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "sqrtq.intfb.h"
#include "sqrtqad.intfb.h"
#include "cvar3.intfb.h"
#include "cvar3ad.intfb.h"
#include "cvar3in.intfb.h"
#include "cvar3inad.intfb.h"
#include "abor1.intfb.h"
#include "prt_ctlvec_max.intfb.h"

#include "spec_imzero.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EVJQ',0,ZHOOK_HANDLE)

IF (NTYPE_MODERR/=1) CALL ABOR1('EVJQ: Error NTYPE_MODERR')
CALL ALLOCATE_CTLVEC(YLCV,KPARAM=0,LDINIT=llf,LDTOVS=llf,LDLAM=llf)
IF (LQEQB) CALL ALLOCATE_CTLVEC(YLCV_INIT,LDINIT=.true.,KMERR=0)
IF (ll_chk_inv) CALL ALLOCATE_CTLVEC(YLCV_INIT_tmp,LDINIT=.true.,KMERR=0)
IF (ll_chk_inv) CALL ALLOCATE_CTLVEC(YLCV_INIT_tmp1,LDINIT=.true.,KMERR=0)
ii=0
LLSPFCE=LSPFCE
LSPFCE=.TRUE.
DO jj=1,NDIM_MODERR

! SPJQMODERR contains M(x_{i-1})-x_i
  IF (NPRTMODERR>=1) CALL SPECTRAL_NORM(SPJQMODERR(jj),'EVJQ SPJQ')
! Add nonlinear trajectory if needed
  IF (NUPTRA>0) THEN
    CALL SELF_ADD(SPJQMODERR(jj),SPJQMODER5(jj))
    IF (NPRTMODERR>=1) CALL SPECTRAL_NORM(SPJQMODERR(jj),'EVJQ SPJQ+FG')
  ENDIF
! Compute eta_i = Q^{-1/2}*[M(x_{i-1})-x_i]
  il=SPJQMODERR(jj)%nspsizel
  IF (SPCTLMODERR%nspsizel /= il) CALL ABOR1('EVJQ: Error SPCTLMODERR')
  SPCTLMODERR=SPJQMODERR(jj)
  IF (LQEQB) THEN
    IF (SIZE(YLCV_INIT%INITCV)/=il) CALL ABOR1('EVJQ: Error LQEQB')
    write(nulout,*) 'JQ norm in spectral space before CVAR3:'
    CALL SPECTRAL_NORM(SPCTLMODERR,'EVJQ')
    INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA
    ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
    ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))
    ZGP3(:,:,:)=0.0_JPRB
    ZGP2(:,:,:)=0.0_JPRB
    CALL CVAR3(SPCTLMODERR,YLCV_INIT,ZGP3,ZGP2,0)
    DEALLOCATE (ZGP3)
    DEALLOCATE (ZGP2) 
!           CALL CVAR3(SPJQMODERR(jj),YLCV_INIT,ZGP3,ZGP2,0)
!   check invertibility:
    IF (ll_chk_inv) THEN
      YLCV_INIT_tmp=YLCV_INIT
      SPCTLMODERR=0.0_JPRB
      INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA
      ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
      ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))
      CALL CVAR3IN(YLCV_INIT_tmp,SPCTLMODERR,ZGP3,ZGP2,0)
      DEALLOCATE (ZGP3)
      DEALLOCATE (ZGP2)
      write(nulout,*) 'JQ norm in spectral space after CVAR3,CVAR3IN:'
      CALL SPECTRAL_NORM(SPCTLMODERR,'EVJQ')
    ENDIF
    SPCTLMODERR=YLCV_INIT%INITCV(:)
    CALL SPEC_IMZERO(SPCTLMODERR)
  ELSE
    CALL SQRTQ(SPCTLMODERR)
  ENDIF
! Save for Jq computation
  YLCV%MODERR(ii+1:ii+il)=SPCTLMODERR
  zjb=DOT_PRODUCT(SPCTLMODERR,SPCTLMODERR,.true.)
  write(nulout,*)'EVJQ: Jq',jj,'=',zjb

! Compute dJq/deta_i = 2.0 * Q^{-1} * [M(x_{i-1})-x_i]
  CALL SELF_MUL(SPCTLMODERR,2.0_JPRB)
  write(nulout,*) 'JQ norm in spectral space after CVAR3 + SELF_MUL:'
  CALL SPECTRAL_NORM(SPCTLMODERR,'EVJQ')
  IF (LQEQB) THEN
    IF (SIZE(YLCV_INIT%INITCV)/=il) CALL ABOR1('EVJQ: Error LQEQB')    
    YLCV_INIT%INITCV(:)=SPCTLMODERR
    SPCTLMODERR=0.0_JPRB
    IF (ll_chk_inv) YLCV_INIT_tmp1=YLCV_INIT
    INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA
    ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
    ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))
    CALL CVAR3AD(SPCTLMODERR,YLCV_INIT,ZGP3,ZGP2,0)
    DEALLOCATE (ZGP3)
    DEALLOCATE (ZGP2)
  ELSE
    CALL SQRTQAD(SPCTLMODERR)
  ENDIF
  SPJQMODERR(jj)=SPCTLMODERR
!   check adjoint invertibility:
  IF (ll_chk_inv) THEN
    YLCV_INIT_tmp%initcv(:)=0.0_JPRB
    INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA
    ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
    ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))
    ZGP3(:,:,:)=0.0_JPRB
    ZGP2(:,:,:)=0.0_JPRB
    CALL CVAR3INAD(YLCV_INIT_tmp,spctlmoderr,ZGP3,ZGP2,0)
    DEALLOCATE (ZGP3)
    DEALLOCATE (ZGP2)
    spctlmoderr=YLCV_INIT_tmp%initcv(:)
    CALL SPEC_IMZERO(SPCTLMODERR)
    write(nulout,*) 'JQ norm in spectral space after CVAR3AD,CVAR3INAD:'
    CALL SPECTRAL_NORM(spctlmoderr,'EVJQ INAD')
  ENDIF
  write(nulout,*) 'JQ norm in spectral space after CVAR3AD:'
  CALL SPECTRAL_NORM(SPJQMODERR(jj),'EVJQ AD')
  IF (NPRTMODERR>=1) CALL SPECTRAL_NORM(SPJQMODERR(jj),'EVJQ out')

  ii=ii+il
ENDDO
LSPFCE=LLSPFCE
IF (ii/=SIZE(YLCV%MODERR)) CALL ABOR1('EVJQ: Error size model error')

IF (NPRTMODERR>=1) CALL PRT_CTLVEC_MAX(YLCV,'EVJQ')

! Compute Jq
CALL CTLVEC_SQNORM(YLCV, zjb, FJQCOST, zjt, zjl, zjp, zjs)
write(nulout,*)'EVJQ: Jq=',FJQCOST

CALL DEALLOCATE_CTLVEC(YLCV)
IF (LQEQB) CALL DEALLOCATE_CTLVEC(YLCV_INIT)
IF (ll_chk_inv) CALL DEALLOCATE_CTLVEC(YLCV_INIT_tmp)
IF (ll_chk_inv) CALL DEALLOCATE_CTLVEC(YLCV_INIT_tmp1)
IF (LHOOK) CALL DR_HOOK('EVJQ',1,ZHOOK_HANDLE)
END SUBROUTINE EVJQ
