SUBROUTINE FLTBGERR

!*** *FLTBGERR*

!   Purpose.
!   --------
!      To apply a spectral filter to the standard deviations of
!      background error. This removes much of the random noise produced
!      by the randomization method.

!   Interface.
!   ----------

!      CALL ADVSIGA

!   Externals.
!   ----------

!      MIO routines  SC2RDG SC2WRG

!   Reference.
!   ----------

!   Author.
!   -------
!      Mike Fisher *ECMWF*

!   Modifications.
!   --------------
!      Original   97-05-19
!      M.Hamrud   98-06-02 Changed forced by change of reespe/speree
!      Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of argument .FALSE. in 'CALL SPEREE'.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        G. Desroziers 16-Mar-2006 Enable sigmab filtering for radiances if LECMWF=.F.
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPROMA   ,NSMAX    ,NSPEC2   ,NUMP
USE YOMLAP   , ONLY : NASM0    ,MYMS
USE YOMGEM   , ONLY :    NGPTOT
USE YOMCST   , ONLY : RPI
USE YOMVAR   , ONLY : NOANEF   ,NBGTRUNC
USE YOMANEB  , ONLY : ANEBUF   ,NANEBRAD ,&
 & NANEBRH2 ,NANEBU10 ,NANEBV10 ,NANEBT2  ,NANERADS
USE YOMCT0B  , ONLY : LECMWF

!-----------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: zane(NPROMA,NOANEF),zspec(NOANEF,NSPEC2),zreel(NGPTOT,NOANEF)

INTEGER(KIND=JPIM) :: iendc,im,inm,istc,j,jir,jk,jkglo,jmloc,jn

REAL(KIND=JPRB) :: zfilt
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "reespe.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"
#include "speree.intfb.h"

!-----------------------------------------------------------------------

!--- copy analysis errors into zreel

IF (LHOOK) CALL DR_HOOK('FLTBGERR',0,ZHOOK_HANDLE)
MESSP_LOOP_1 : do jkglo=1,NGPTOT,NPROMA
  istc=1
  iendc=min(NPROMA,NGPTOT-jkglo+1)
  call SC2RDG (ANEBUF,jkglo,istc,iendc,zane)

  do j=1,NOANEF
    do jk=istc,iendc
      zreel(jk+jkglo-1,j) = zane(jk,j)
    enddo
  enddo
enddo MESSP_LOOP_1

!--- filter

zspec(:,:) = 0.0_JPRB   ! Protection for NPRGPEW > 1
call REESPE (NOANEF,NOANEF,zspec,zreel)

DO jmloc=1,NUMP
  im=MYMS(jmloc)
  do jir=0,MIN(1,im)
    do jn=im,NSMAX
      zfilt = cos(0.5_JPRB*RPI*REAL(MIN(jn,NBGTRUNC),JPRB)/REAL(NBGTRUNC, &
       & JPRB))**&
       & 2  
      inm  = NASM0(im)+(jn-im)*2+jir
      zspec(:,inm) = zspec(:,inm) * zfilt
    enddo
  enddo
enddo

call SPEREE (NOANEF,NOANEF,zspec,zreel)

!--- copy errors back into the analysis error buffer

MESSP_LOOP_2 : do jkglo=1,NGPTOT,NPROMA
  istc=1
  iendc=min(NPROMA,NGPTOT-jkglo+1)
  call SC2RDG (ANEBUF,jkglo,istc,iendc,zane)

  DO j=1,NOANEF
    IF(j >= NANEBRAD+1.AND. j <= NANEBRAD+NANERADS .AND.LECMWF) CYCLE
    IF(ANY(j-1 == (/NANEBRH2,NANEBU10,NANEBV10,NANEBT2/))) CYCLE
    DO jk=istc,iendc
      zane(jk,j) = zreel(jk+jkglo-1,j)
    ENDDO
  ENDDO
  call SC2WRG (ANEBUF,jkglo,istc,iendc,zane)
enddo MESSP_LOOP_2

IF (LHOOK) CALL DR_HOOK('FLTBGERR',1,ZHOOK_HANDLE)
END SUBROUTINE FLTBGERR
