SUBROUTINE GETMINI2 (YDVAZX, CDFILE, kvec_start, kvec_end)

!***  *GETMINI2* get starting point of minimisation.

!**   Purpose.
!     --------
!     this routine reads in the control variable.

!**   Interface.
!     *CALL* *GETMINI2* from CVA1

!        Explicit arguments :
!        --------------------
!        YDVAZX  : control variable
!        CDFILE : file name (optional)

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------
!        See documentation : The modules M1QN2,N1QN2,M1QN3,N1QN3
!                            version 1.1.b (February 1989)
!                            J.C Gilbert and C. Lemarechal

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF research department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel Thepaut *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-03-04
!        Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!        Modified 96-10-11 by F.Rabier  : do not use MIO if not on Cray
!        Modified 96-10-15 by F.Rabier  : move read tbmin to suhess
!        Modified 98-01-10 by M.Hamrud  : Minimisation parallelized
!        Modified 00-08-09 by Y.Tremolet: Cleaning
!        Modified 01-02-07 by M.Fisher  : Combine GETMINC and GETMINI
!        Modified 01-02-12 by M.Fisher  : Arguments YDVAZX, CDFILE and KDIM
!        Modified 02-08-06 by Y.Tremolet: Use control_vectors, KDIM from file.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher 17-Mar-2004 Wavelet Jb
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        D.Salmond     18-Apr-2007 Use multiple processors for I/O
!        T.Wilhelmsson 28-Aug-2009 Multiprocessor I/O moved here
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0 , ONLY: NCONF    ,NSTART   ,NPROC
USE YOMNMIA, ONLY: NITNMI   ,LNMIRQ
USE YOMVAR , ONLY: NUPTRA   ,LTRREF
USE YOMLUN , ONLY: NULOUT
USE YOMMP  , ONLY: MYPROC
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST,CLOSE_IOREQUEST
USE YOMWAVELET, ONLY: LJBWAVELET, N_WAVELET_CUTOFFS
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR, ALLOCATE_CTLVEC, DEALLOCATE_CTLVEC, &
                          & ASSIGNMENT(=)
USE CONTROL_VECTORS_COMM_MOD, ONLY : MULTISCATTER_CTLVEC
USE MPL_MODULE, ONLY : MPL_ALLGATHERV
USE YOMGETMINI, ONLY : MIOPROC, MBUF, RZVAZX, NUM_PROCS_PER_IO_PROC

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZX(:)
CHARACTER(LEN=*),INTENT(IN)        :: CDFILE 
INTEGER(KIND=JPIM),INTENT(IN)      :: kvec_start 
INTEGER(KIND=JPIM),INTENT(IN)      :: kvec_end

TYPE(control_vector) :: YL_ZVAZXD(kvec_start:kvec_end)

CHARACTER :: CLTIME*8
CHARACTER :: CLUPTRA*2
CHARACTER :: CLFN*128
CHARACTER :: CLFS*128
CHARACTER :: CLFE*128

INTEGER(KIND=JPIM) :: INSCALES, IDIMS, jvec, JROC
INTEGER(KIND=JPIM), DIMENSION(kvec_start:kvec_end) :: ITLMAX, ITLERR, IVADIM
INTEGER(KIND=JPIM) :: IRECVCOUNTS(NPROC),IRECVBUF(3*(kvec_end-kvec_start+1))
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!     ------------------------------------------------------------------

!*       1   DEFINE FILE NAME.
!           --------------------

IF (LHOOK) CALL DR_HOOK('GETMINI2',0,ZHOOK_HANDLE)

CLFN = CDFILE

jvec = kvec_start

DO JROC = 1,NPROC,NUM_PROCS_PER_IO_PROC
  MIOPROC(jvec) = JROC
  IF (MYPROC == MIOPROC(jvec)) THEN
    WRITE(CLFN(7:9),'(I3.3)') jvec
 ENDIF
 jvec = jvec + 1
 IF (jvec > kvec_end) EXIT
ENDDO

CLFS=CLFN
CLFE=CLFN
WRITE(CLFS(7:9),'(I3.3)') kvec_start
WRITE(CLFE(7:9),'(I3.3)') kvec_end
WRITE(NULOUT,*)'GETMINI2: Read control variable from files ',CLFS(1:9), ' to ', CLFE(1:9)
  

!     ------------------------------------------------------------------

!*    3.  OPEN FILES AND READ IN CONTROL VARIABLE
!     -------------------------------------------

!*       3.1   OPEN FILE
!              ---------

! write(0,*)'GETMINI2 - MIOPROC - READ',kvec_start,kvec_end,MIOPROC(kvec_start:kvec_end)

CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r',KIOPROCS=MIOPROC(kvec_start:kvec_end))
CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')
CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=MBUF)
  

!*       3.2   READ  CONTROL VARIABLE.
!              ----------------------

IRECVCOUNTS(1:NPROC) = 0
DO jvec = kvec_start, kvec_end
  IF(MYPROC == MIOPROC(jvec)) THEN
    IDIMS = MBUF(1)
    DEALLOCATE(RZVAZX)
    ALLOCATE(RZVAZX(IDIMS))
  ENDIF
  IRECVCOUNTS(MIOPROC(jvec)) = 3
ENDDO

CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=RZVAZX)
CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)

! write(0,*)'GETMINI2 - MIOPROC - SCATTER',kvec_start,kvec_end,MIOPROC(kvec_start:kvec_end)

CALL GSTATS(627,0)
CALL MPL_ALLGATHERV(MBUF,IRECVBUF,IRECVCOUNTS,CDSTRING='GETMINI2')
CALL GSTATS(627,1)

DO jvec = kvec_start, kvec_end 
  IVADIM(jvec) = IRECVBUF(3*(jvec-kvec_start) + 1)
  ITLMAX(jvec) = IRECVBUF(3*(jvec-kvec_start) + 2)
  ITLERR(jvec) = IRECVBUF(3*(jvec-kvec_start) + 3)
ENDDO

DO jvec = kvec_start+1, kvec_end 
  IF (IVADIM(kvec_start) /= IVADIM(jvec)) &
 &  CALL ABOR1('GETMINI2: cannot read differently sized control vectors.')
  IF (YDVAZX(kvec_start)%NSIZEG /= YDVAZX(jvec)%NSIZEG) &
 &  CALL ABOR1('GETMINI2: cannot handle differently sized control vectors.')
ENDDO

!*      MESSAGE-PASS THE CONTROL VARIABLE

IF (IVADIM(kvec_start) == YDVAZX(kvec_start)%NSIZEG) THEN
  CALL MULTISCATTER_CTLVEC(YDVAZX,MIOPROC,RZVAZX,kvec_start,kvec_end)
ELSE
  IF (LJBWAVELET) THEN
    DO jvec = kvec_start, kvec_end
      INSCALES=COUNT(N_WAVELET_CUTOFFS<=ITLMAX(jvec))-1
      WRITE (NULOUT,*) 'INPUT CONTROL VECTOR HAS ',INSCALES,' WAVELET SCALES'
      CALL ALLOCATE_CTLVEC(YL_ZVAZXD(jvec),KNSCALES=INSCALES+1,KSERR=ITLERR(jvec))
    ENDDO
  ELSE
    DO jvec = kvec_start, kvec_end
      CALL ALLOCATE_CTLVEC(YL_ZVAZXD(jvec),KSMAX=ITLMAX(jvec),KSERR=ITLERR(jvec))
    ENDDO
  ENDIF

  CALL MULTISCATTER_CTLVEC(YL_ZVAZXD,MIOPROC,RZVAZX,kvec_start,kvec_end)

  CALL GSTATS(1839,0)
  DO jvec = kvec_start, kvec_end
     YDVAZX(jvec) = YL_ZVAZXD(jvec) ! Change of resolution
     CALL DEALLOCATE_CTLVEC(YL_ZVAZXD(jvec))
  ENDDO
  CALL GSTATS(1839,1)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GETMINI2',1,ZHOOK_HANDLE)
END SUBROUTINE GETMINI2
