SUBROUTINE GP_PTR_SSMI

!**** *GP_PTR_SSMI* - RAIN ASSIMILATION - Set pointers in ONEDVAR OUTPUT ARRAYS
!     1D-Var in clouds/precipitation for SSM/I radiances

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     Purpose :  
!     -------
!     Set pointers in ONEDVAR OUTPUT ARRAYS

!     Interface : CALL GP_PTR_SSMI
!     ---------

!       Explicit arguments : None
!       ------------------   

!       Implicit arguments : YOM_PTR_SSMI
!       ------------------

!     Author : Deborah Salmond
!     DATE  : 02-04-2004
!     ------

!     Modifications :
!     A. Geer      4-Apr-2006    Include analysed 10m wind 
!    -------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_PTR_SSMI  ,ONLY : MFLDS_1D, &
 &      M_P0,M_T0,M_U10_FG,M_V10_FG,M_PS_FG,M_PC_FG,M_PS_AN,M_PC_AN, &
 &      M_WV_FG,M_WV_AN,M_ER_FG,M_ER_AN,M_U10_AN,M_V10_AN, &
 &      M_CO_AN, &
 &      MFLDS_1DDIAG, &
 &      M_TCWV_3D,M_RWP_3D,M_SWP_3D,M_CWP_3D,M_IWP_3D,M_RA_3D,M_SN_3D, &
 &      M_TCWV_FG,M_RWP_FG,M_SWP_FG,M_CWP_FG,M_IWP_FG,M_RA_FG,M_SN_FG, &
 &      M_TCWV_AN,M_RWP_AN,M_SWP_AN,M_CWP_AN,M_IWP_AN,M_RA_AN,M_SN_AN, &
 &      MFLDS_2DLEV, &
 &      M_P_FG, &
 &      M_T_FG,M_Q_FG,M_R_FG,M_S_FG,M_W_FG,M_I_FG,M_C_FG, &
 &      M_T_AN,M_Q_AN,M_R_AN,M_S_AN,M_W_AN,M_I_AN,M_C_AN, &
 &      MFLDS_2DCHAN, &
 &      M_TB_OB,M_TB_ER,M_TB_FG,M_TB_AN,M_TB_BI,M_TB_3D

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GP_PTR_SSMI',0,ZHOOK_HANDLE)

MFLDS_1D=15
M_P0=1
M_T0=2
M_U10_FG=3
M_V10_FG=4
M_U10_AN=5
M_V10_AN=6
M_PS_FG=7
M_PC_FG=8
M_PS_AN=9
M_PC_AN=10
M_WV_FG=11
M_WV_AN=12
M_ER_FG=13
M_ER_AN=14
M_CO_AN=15

MFLDS_1DDIAG=21

M_TCWV_3D=1
M_RWP_3D=2
M_SWP_3D=3
M_CWP_3D=4
M_IWP_3D=5
M_RA_3D=6
M_SN_3D=7

M_TCWV_FG=8
M_RWP_FG=9
M_SWP_FG=10
M_CWP_FG=11
M_IWP_FG=12
M_RA_FG=13
M_SN_FG=14

M_TCWV_AN=15
M_RWP_AN=16
M_SWP_AN=17
M_CWP_AN=18
M_IWP_AN=19
M_RA_AN=20
M_SN_AN=21
 
MFLDS_2DLEV=15

M_T_FG=1
M_Q_FG=2
M_R_FG=3
M_S_FG=4
M_W_FG=5
M_I_FG=6
M_C_FG=7
M_P_FG=8

M_T_AN=9
M_Q_AN=10
M_R_AN=11
M_S_AN=12
M_W_AN=13
M_I_AN=14
M_C_AN=15

MFLDS_2DCHAN=6
M_TB_OB=1
M_TB_ER=2
M_TB_FG=3
M_TB_AN=4
M_TB_BI=5
M_TB_3D=6

IF (LHOOK) CALL DR_HOOK('GP_PTR_SSMI',1,ZHOOK_HANDLE)

END SUBROUTINE GP_PTR_SSMI
