SUBROUTINE GRBSPA(KFLEV,KSMAX,KSPEC2,PVALH,PVBH,&
           &PSPVOR,PSPDIV,PSPT,PSPQ,PSPSP,PSPOR,KDATE,KSSSSS)

!**** *GRBSPA*  - INITIALIZE A GRIB FILE

!     Purpose.
!     --------
!           From the spectral fields of forecast differences
!           initialize a GRIB file.

!**   Interface.
!     ----------
!        *CALL* *GRBSPA

!     Author.
!     -------
!        F. Bouttier  ECMWF Oct 1996

!     Modifications.
!     --------------
!        R. Brozkova  LACE  Jan 2001 cleaning
!        S. Stefanescu      27-Sep-2005 adapted for CY29T2
!        S. Stefanescu      06-Nov-2006 adapted for CY30T1
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTEGER(KIND=JPIM), PARAMETER :: JPVERB=2
INTEGER(KIND=JPIM), PARAMETER :: JPDIM=500000

INTEGER(KIND=JPIM),INTENT(INOUT) :: KSSSSS
INTEGER(KIND=JPIM),INTENT(IN) :: KSMAX, KFLEV, KSPEC2

REAL(KIND=JPRB)   ,INTENT(IN) :: PSPVOR(KFLEV,KSPEC2),PSPDIV(KFLEV,KSPEC2),PSPT(KFLEV,KSPEC2)
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPQ(KFLEV,KSPEC2),PSPSP(KSPEC2),PSPOR(KSPEC2)
REAL(KIND=JPRB)   ,INTENT(IN) :: PVALH(0:KFLEV),PVBH(0:KFLEV)

INTEGER(KIND=JPIM) :: JJ, J, JFLD, ILEVEL, I
INTEGER(KIND=JPIM) :: ILUN,IRET
INTEGER(KIND=JPIM),INTENT(IN) :: KDATE
INTEGER(KIND=JPIM) :: INBYT, IWORD
INTEGER(KIND=JPIM) :: ISEC0(2),ISEC1(512),ISEC2(512),ISEC3(2),ISEC4(512)
INTEGER(KIND=JPIM) :: IGRIB(KSPEC2)

REAL(KIND=JPRB) ::    ZSEC2(512),ZSEC3(2),ZSEC4(KSPEC2)
!REAL(KIND=JPRB) ::    ZLO

CHARACTER (LEN = 10) :: CLFILE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('GRBSPA',0,ZHOOK_HANDLE)
WRITE(UNIT=NULOUT,FMT='('' START GRBSPA'')')


!         0. zero sec arrays and define levels

      ISEC0(:)=0
      ISEC1(:)=0
      ISEC2(:)=0
      ISEC3(:)=0
      ISEC4(:)=0

      ZSEC2(:)=0.
      ZSEC3(:)=0.
      ZSEC4(:)=0.

!         1. Open file

      CLFILE='gribdiff'
      IF (JPVERB > 1) PRINT*,' grbspa - opening file ',CLFILE
      CALL PBOPEN(ILUN,CLFILE,'w',IRET)
!     OPEN(UNIT=89,FILE=CLFILE,FORM='UNFORMATTED')
      IF(IRET/=0) PRINT*,' ERROR - pbopen failed, retcode=',IRET
      IF(IRET/=0) CALL ABOR1(' ERROR - pbopen failed')
      WRITE(0,*) 'KSPEC2 KSMAX KFLEV = ',KSPEC2,KSMAX,KFLEV
      WRITE(0,*) 'KDATE = ',KDATE,' KSSSSS = ',KSSSSS
      KSSSSS=KSSSSS/3600
      WRITE(0,*) 'KSSSSS again ',KSSSSS

!         2. encode fields as they come (vor div t q spsp orog)

        ISEC1(1)=128
        ISEC1(2)=98
        ISEC1(3)=40
        ISEC1(4)=255
        ISEC1(5)=128
        ISEC1(7)=109
        ISEC1(9)=0
        ISEC1(10)=mod(KDATE/10000,100)
        ISEC1(11)=mod(KDATE/100,100)
        ISEC1(12)=mod(KDATE,100)
        ISEC1(13)=KSSSSS
        DO  JJ=14,25
          ISEC1(JJ)=0
        ENDDO
        ISEC1(21)=20

        ISEC2(1)=50
        ISEC2(2)=KSMAX
        ISEC2(3)=KSMAX
        ISEC2(4)=KSMAX
        ISEC2(5)=1
        ISEC2(6)=1
        ISEC2(12)=2*KFLEV+2

        ISEC2(13)=INT(ASIN(-1.)*180./3.14159265*1000.)
!        ZLO=ACOS(0.)*180./3.14159265
!       ISEC2(14)=INT(1000.*ZLO)
        ISEC2(14)=0.
        ISEC2(15)=ISEC2(13)
        ISEC2(16)=ISEC2(14)
        ZSEC2(1)=0.
        ZSEC2(2)=1.
        DO J=0,KFLEV
          ZSEC2(J+11)=PVALH(J)
          ZSEC2(J+11+KFLEV+1)=PVBH(J)
        ENDDO
       
        ISEC3(1)=0
        ISEC3(2)=0
        ZSEC3(1)=0.
        ZSEC3(2)=0.

        ISEC4(1)=KSPEC2
        ISEC4(2)=24
        ISEC4(3)=0
        ISEC4(4)=0

        DO JFLD=1,4
        DO ILEVEL=1,KFLEV
!       DO JFLD=1,1
!       DO ILEVEL=1,1
          IF(JFLD == 1) THEN
            ZSEC4(:)=PSPVOR(ILEVEL,:)
            ISEC1(6)=138
          ELSEIF(JFLD == 2) THEN
            ZSEC4(:)=PSPDIV(ILEVEL,:)
            ISEC1(6)=155
          ELSEIF(JFLD == 3) THEN
            ZSEC4(:)=PSPT(ILEVEL,:)
            ISEC1(6)=130
          ELSEIF(JFLD == 4) THEN
            ZSEC4(:)=PSPQ(ILEVEL,:)
            ISEC1(6)=133
          ENDIF
          ISEC1(8)=ILEVEL
          WRITE(0,*) ' ILEVEL JFLD ',ILEVEL,JFLD
!          WRITE(0,*) 'impression des ISEC avant GRIBEX'
!          CALL GRSDBG(1)
!          CALL GRPRS0(ISEC0)
!          CALL GRPRS1(ISEC0,ISEC1)
!          CALL GRPRS2(ISEC0,ISEC2,ZSEC2)
          IF(ILEVEL == 1.AND.JFLD == 1) THEN
            DO I=1,20
            WRITE(0,*) 'ZSEC4(',ILEVEL,JFLD,' ) =',ZSEC4(I)
            ENDDO
          ENDIF
!         CALL GRPRS4(ISEC0,ISEC4,ZSEC4)
          WRITE(0,*) 'on attaque maintenant GRIBEX'
          CALL GRIBEX(ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,ZSEC4,&
        &KSPEC2,IGRIB,KSPEC2,IWORD,'C',IRET)
!         WRITE(0,*) ' Writ ',IWORD,' words of GRIBEX out ISEC0.'&
!       &,ISEC0(1),ISEC0(2)
          IF(IWORD >= JPDIM)THEN
            PRINT*,'Error : too many words, enlarge array IGRIB.'
            STOP
          ENDIF
          INBYT=IWORD*4
          WRITE(0,*) 'IWORD INBYT ',IWORD,INBYT
          CALL PBWRITE(ILUN,IGRIB,INBYT,IRET)
!         WRITE(89) (IGRIB(j),j=1,IWORD)
          IF(IRET==-1) CALL ABOR1(' Error in PBWRITE gribout')
          WRITE(0,*) IRET,' bytes written on output file.'
          WRITE(0,*) ' INBYT = ',INBYT
        ENDDO
        ENDDO

!       surface pressure

        ZSEC4(:)=PSPSP(:)
        ISEC1(6)=152
        ISEC1(8)=1
        WRITE(0,*) ' ILEVEL JFLD ',ISEC1(8),JFLD
        WRITE(0,*) 'on attaque maintenant GRIBEX'
        CALL GRIBEX(ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,ZSEC4,&
       &KSPEC2,IGRIB,KSPEC2,IWORD,'C',IRET)
!       WRITE(0,*) ' Writ ',IWORD,' words of GRIBEX out to disk...'
        IF(IWORD > JPDIM)THEN
          PRINT*,'Error : too many words, enlarge array IGRIB.'
          STOP
        ENDIF
        INBYT=IWORD*4
        CALL PBWRITE(ILUN,IGRIB,INBYT,IRET)
!        WRITE(89) (IGRIB(J),J=1,IWORD)
        IF(IRET==-1) CALL ABOR1(' Error in PBWRITE gribout')
        WRITE(0,*) IRET,' bytes written on output file.'
        WRITE(0,*) ' INBYT = ',INBYT

!       orography

        ZSEC4(:)=PSPOR(:)
        ISEC1(6)=156
        ISEC1(8)=1
        WRITE(0,*) ' ILEVEL JFLD ',ISEC1(8),JFLD
        WRITE(0,*) 'on attaque maintenant GRIBEX'
        CALL GRIBEX(ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,ZSEC4,&
       &KSPEC2,IGRIB,KSPEC2,IWORD,'C',IRET)
!       WRITE(0,*) ' Writ ',IWORD,' words of GRIBEX out to disk...'
        IF(IWORD > JPDIM)THEN
          PRINT*,'Error : too many words, enlarge array IGRIB.'
          STOP
        ENDIF
        INBYT=IWORD*4
        CALL PBWRITE(ILUN,IGRIB,INBYT,IRET)
!        WRITE(89) (IGRIB(J),J=1,IWORD)
        IF(IRET==-1) CALL ABOR1(' Error in PBWRITE gribout')
        WRITE(0,*) IRET,' bytes written on output file.'
        WRITE(0,*) ' INBYT = ',INBYT

        CALL PBCLOSE(ILUN,IRET)
        WRITE(0,*) ' IRET za pbclose ',IRET

        WRITE(UNIT=NULOUT,FMT='('' END GRBSPA'')')
        IF (LHOOK) CALL DR_HOOK('GRBSPA',1,ZHOOK_HANDLE)

        END SUBROUTINE GRBSPA
