SUBROUTINE GTVARBC_GROUPID(kbufrid,ksensor,kchannl,ydkvarbc,knbgroup,kgroupid)
!-----------------------------------------------------------------------
!   GTVARBC_GROUPID

!   Purpose.
!   --------
!   Returning the id of a bias parameter set in a bias parameter table
!   for a given triplet (kbufrid,ksensor,kchannl)

!   Interface.
!   ----------
!   CALL GTVARBC_GROUPID(kbufrid,ksensor,kchannl,ydkvarbc,knbgroup,kgroupid)

!   Input arguments:
!      kbufrid   - BUFR id
!      ksensor   - id of the sensor
!      kchannl   - id of the channel
!      ydkvarbc  - variational bias parameter data table
!      knbgroup  - number of parameter groups

!   Output arguments:
!      kgroupid  -

!   Author.
!   -------
!   Vincent GUIDARD (Meteo-France)

!   Modifications.
!   --------------
!   Original    2008/03/10

!-----------------------------------------------------------------------

USE PARKIND1, ONLY : JPIM, JPRB
USE yomhook , ONLY : lhook, dr_hook
USE YOMVARBC, ONLY : TYPE_VARBC, JPMXNGROUP
USE YOMANCS , ONLY : NMDI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: kbufrid
INTEGER(KIND=JPIM),INTENT(IN) :: ksensor
INTEGER(KIND=JPIM),INTENT(IN) :: kchannl
TYPE(TYPE_VARBC),INTENT(IN) :: ydkvarbc(JPMXNGROUP)
INTEGER(KIND=JPIM),INTENT(IN) :: knbgroup
INTEGER(KIND=JPIM),INTENT(OUT) :: kgroupid

INTEGER(KIND=JPIM) :: jg

REAL(KIND=JPRB)    :: zhook_handle

IF (lhook) CALL dr_hook('GTVARBC_GROUPID',0,zhook_handle)

kgroupid = NMDI

DO jg = 1, knbgroup
   IF ((ydkvarbc(jg)%NBUFRID == kbufrid).AND. &
        & (ydkvarbc(jg)%NSENSOR == ksensor).AND. &
        & (ydkvarbc(jg)%NCHANNL == kchannl)) THEN
      kgroupid = jg
      EXIT
   ENDIF
ENDDO

IF (lhook) CALL dr_hook('GTVARBC_GROUPID',1,zhook_handle)

END SUBROUTINE GTVARBC_GROUPID
