SUBROUTINE JBTOMODELAD(YDSPJB,PGP3,PGP2)


#ifdef DOC

!**** *JBTOMODELAD * - Routine to transfer control variable spectral fields
!                      to model state fields - ADJOINT

!     Purpose.
!     --------
!           Transfer to SPA3JB, SPA2JB and SPA1JB
!           from SPA3, SPA2, SPA1 and possibly GFL
!           (Adjoint of JBTOMODEL)

!**   Interface.
!     ----------
!        *CALL* *JBTOMODELAD*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original : 2004-10-26
!        Mike Fisher : 2005-06-08 Generalize Jb for GEMS
!        C. Fischer 21-May-2005  Aladin spectral transforms added
!        A. Dethof 11-Nov-2008  LOGVAR JB for NOX
!        G. Radnoti 17-Oct-2008  Fix for CVTEST/ADTEST

#endif

USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMSP    , ONLY : SPA3,   SPA2,   SPA1, SPGFL
USE YOMGFL   , ONLY : GFL,GFL5
USE YOMMP    , ONLY : NPSP, NBSETLEV
USE YOMJG    , ONLY : N_SPJB_VARS, SPJB_VARS_INFO, LSPFCE, &
                    & LJB_NONLINEAR_CVHUM, LJB_NONLINEAR_CVO3
USE YOMCT0   , ONLY : LELAM
USE YOMDIM   , ONLY : NFLEVG, NPROMA
USE YOMGRB   , ONLY : NGRBQ, NGRBO3, NGRBNOXLOG
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSPJB
REAL(KIND=JPRB), INTENT(OUT) :: PGP3(:,:,:),PGP2(:,:,:)

INTEGER(KIND=JPIM) :: JFIELD, IPT, IPTJB, J
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "dir_transad.h"
END INTERFACE

#include "abor1.intfb.h"
#include "etransinv_jbtomodelad.intfb.h"
#include "transinv_jbtomodelad.intfb.h"


IF (LHOOK) CALL DR_HOOK('JBTOMODELAD',0,ZHOOK_HANDLE)

!-- SPA1 is a special case!

IF (LELAM) YDSPJB%SP1D(:,:) = SPA1(:,:)

!--- Loop over Jb variables

!CALL GSTATS(1455,0)
DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB
  IPT   = SPJB_VARS_INFO(JFIELD)%IPT

  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN

    IF (NPSP>0) THEN
      YDSPJB%SP2D(:,IPTJB)= SPA2(:,IPT)
      SPA2(:,IPT) = 0.0_JPRB
    ENDIF

  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA3) THEN

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
    DO J=1,SIZE(YDSPJB%SP3D,2)
      YDSPJB%SP3D(:,J,IPTJB)= SPA3(:,J,IPT)
      SPA3(:,J,IPT) = 0.0_JPRB
    ENDDO
!$OMP END PARALLEL DO

  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPGFL) THEN

    IF (      (LJB_NONLINEAR_CVHUM .AND. SPJB_VARS_INFO(JFIELD)%IGRIBCODE == NGRBQ) &
       & .OR. (LJB_NONLINEAR_CVO3 .AND. SPJB_VARS_INFO(JFIELD)%IGRIBCODE == NGRBO3) ) THEN
 !$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
      DO J=1,SIZE(PGP3,3)
        PGP3(:,1+(IPTJB-1)*NFLEVG:IPTJB*NFLEVG,J) = 0.0_JPRB
      ENDDO
!$OMP END PARALLEL DO
      CALL DIR_TRANSAD (PGP=PGP3(:,1+(IPTJB-1)*NFLEVG:IPTJB*NFLEVG,:), &
                       & KVSETSC=NBSETLEV, KPROMA=NPROMA, PSPSCALAR=SPGFL(:,:,IPT))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
      DO J=1,SIZE(YDSPJB%SP3D,2)
        SPGFL(:,J,IPT) = 0.0_JPRB
        YDSPJB%SP3D(:,J,IPTJB) = 0.0_JPRB
      ENDDO
!$OMP END PARALLEL DO
    ELSE
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
      DO J=1,SIZE(YDSPJB%SP3D,2)
        YDSPJB%SP3D(:,J,IPTJB)= SPGFL(:,J,IPT)
        SPGFL(:,J,IPT) = 0.0_JPRB
      ENDDO
!$OMP END PARALLEL DO
    ENDIF

  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_GPGFL.AND. &
   &      SPJB_VARS_INFO(JFIELD)%IGRIBCODE /= NGRBNOXLOG ) THEN

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
    DO J=1,SIZE(YDSPJB%SP3D,2)
      YDSPJB%SP3D(:,J,IPTJB)=0.0_JPRB
    ENDDO
!$OMP END PARALLEL DO
    IF (LSPFCE) THEN
      IF (LELAM) THEN
        CALL ETRANSINV_JBTOMODELAD (YDSPJB%SP3D(:,:,IPTJB),GFL(:,:,IPT,:))
      ELSE
        CALL TRANSINV_JBTOMODELAD (YDSPJB%SP3D(:,:,IPTJB),GFL(:,:,IPT,:))
      ENDIF
      GFL(:,:,IPT,:) = 0.0_JPRB
    ELSE
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
      DO J=1,SIZE(PGP3,3)
        PGP3(:,1+(IPTJB-1)*NFLEVG:IPTJB*NFLEVG,J) = GFL(:,:,IPT,J)
        GFL(:,:,IPT,J) = 0.0_JPRB
      ENDDO
!$OMP END PARALLEL DO
    ENDIF

  ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_GPGFL.AND. &
   &      SPJB_VARS_INFO(JFIELD)%IGRIBCODE == NGRBNOXLOG ) THEN

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
    DO J=1,SIZE(YDSPJB%SP3D,2)
      YDSPJB%SP3D(:,J,IPTJB)=0.0_JPRB
    ENDDO
!$OMP END PARALLEL DO
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
    DO J=1,SIZE(PGP3,3)
      PGP3(:,1+(IPTJB-1)*NFLEVG:IPTJB*NFLEVG,J) = GFL(:,:,IPT,J)* GFL5(:,:,IPT,J)
      GFL(:,:,IPT,J) = 0.0_JPRB
    ENDDO
!$OMP END PARALLEL DO

  ELSE

    WRITE (NULOUT,*) 'DON''T KNOW WHERE TO PUT JB MODEL FIELD : ',JFIELD
    CALL ABOR1 ('DON''T KNOW WHERE TO PUT JB MODEL FIELD')

  ENDIF
ENDDO
!CALL GSTATS(1455,1)

IF (LHOOK) CALL DR_HOOK('JBTOMODELAD',1,ZHOOK_HANDLE)
END SUBROUTINE JBTOMODELAD
