SUBROUTINE JBVCOORD_INTERPOLATE_AD (CDCONF,KNUMB,KSTGLO,PGP3JB,PGP2JB)

!      JBVCOORD_INTERPOLATE - Interpolation to and from Jb vertical
!      levels - ADJOINT

!     Arguments :
!     -----------
!     Input:
!       CDCONF     - M=> FROM Jb vertical levels TO model levels
!                    N=> FROM model levels TO Jb vertical levels
!       KNUMB - number of elements for which computations
!               are performed (used in message passing version)
!       KSTGLO - global offset
!       Input/Output:
!       PGP3JB - 3d gridpoint fields (=transform of SPA3JB)
!       PGP2JB - 2d gridpoint fields (=transform of SPA2JB)

!     Author:  M. Fisher    *ECMWF*
!     -------

!     Modifications:
!     --------------
!     ORIGINAL 2000-03-09.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        E.Holm        20-Oct-2005 Introduce PGP3JB and PGP2JB
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMFCEB  , ONLY : GPBUF_JBVCOORD
USE YOMCVA  , ONLY : NVA3D, NVA2D

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGP3JB(NPROMA,NFLEVG,NVA3D)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGP2JB(NPROMA,NVA2D)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JFLD, IST, ISTC, IEND, IENDC, IDIRECTION
REAL(KIND=JPRB) :: ZINTER(NPROMA,NFLEVG), Z_LEVEL(NPROMA,NFLEVG,2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "sc2rdg.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE_AD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (CDCONF=='M') THEN
  IDIRECTION=1
ELSEIF (CDCONF=='N') THEN
  IDIRECTION=2
ELSE
  CALL ABOR1 ('BAD CDCONF IN JBVCOORD_INTERPOLATE')
ENDIF

IST   = 1
IEND  = KNUMB
ISTC  = 1
IENDC = KNUMB

!       Read the fractional level numbers from the buffer

CALL SC2RDG(GPBUF_JBVCOORD,KSTGLO,ISTC,IENDC,Z_LEVEL)

!       Interpolate all fields in PGP3JB

DO JFLD = 1, NVA3D
  CALL TE_INTERP_AD (ZINTER,PGP3JB(:,:,JFLD))
  PGP3JB(:,:,JFLD)=ZINTER(:,:)
ENDDO

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE_AD',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CONTAINS
SUBROUTINE TE_INTERP_AD (PIN_AD,POUT_AD)

REAL(KIND=JPRB), INTENT(OUT) :: PIN_AD (NPROMA,NFLEVG)
REAL(KIND=JPRB), INTENT(IN)  :: POUT_AD(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZALPHA
INTEGER(KIND=JPIM) :: JROF, JLEV, IABOVE, IBELOW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE_AD:TE_INTERP_AD',0,ZHOOK_HANDLE)
PIN_AD(IST:IEND,:) = 0.0_JPRB

DO JLEV=NFLEVG,1,-1
  DO JROF=IST,IEND
    IABOVE = MAX(INT(Z_LEVEL(JROF,JLEV,IDIRECTION)),1)
    IBELOW = MIN(IABOVE+1,NFLEVG)
    ZALPHA = Z_LEVEL(JROF,JLEV,IDIRECTION) - REAL(IABOVE,JPRB)
    
    IF (IABOVE == 1 .OR. IBELOW == NFLEVG) THEN
!---        adjoint of linear interpolation

      PIN_AD(JROF,IBELOW) = PIN_AD(JROF,IBELOW) &
       & +  ZALPHA    *POUT_AD(JROF,JLEV)  
      PIN_AD(JROF,IABOVE) = PIN_AD(JROF,IABOVE) &
       & +(1.0_JPRB-ZALPHA)*POUT_AD(JROF,JLEV)  
    ELSE
!---        adjoint of cubic fit to the nearest 4 points
      PIN_AD(JROF,IABOVE-1) = PIN_AD(JROF,IABOVE-1) &
       & - ZALPHA * (ZALPHA-1.0_JPRB) * (ZALPHA-2.0_JPRB) &
       & * POUT_AD(JROF,JLEV) / 6.0_JPRB   
   
      PIN_AD(JROF,IABOVE) = PIN_AD(JROF,IABOVE) &
       & + (ZALPHA+1.0_JPRB) * (ZALPHA-1.0_JPRB) * (ZALPHA-2.0_JPRB) &
       & * POUT_AD(JROF,JLEV) * 0.5_JPRB  
   
      PIN_AD(JROF,IBELOW) = PIN_AD(JROF,IBELOW) &
       & - (ZALPHA+1.0_JPRB) * ZALPHA * (ZALPHA-2.0_JPRB) &
       & * POUT_AD(JROF,JLEV) * 0.5_JPRB  
   
      PIN_AD(JROF,IBELOW+1) = PIN_AD(JROF,IBELOW+1) &
       & + (ZALPHA+1.0_JPRB) * ZALPHA * (ZALPHA-1.0_JPRB) &
       & * POUT_AD(JROF,JLEV) / 6.0_JPRB  
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE_AD:TE_INTERP_AD',1,ZHOOK_HANDLE)
END SUBROUTINE TE_INTERP_AD

!     ------------------------------------------------------------------
END SUBROUTINE JBVCOORD_INTERPOLATE_AD
