SUBROUTINE JBVCORG (CDCONF,KSCALE,KNUMB,PLON,PLAT,&
                  & PGP3T0,PGP2T0,PGP3T1,PGP2T1)

!      JBVCOR - Vertical correlations in gridpoint (wavelet) space

!     Arguments :
!     -----------
!     Input:
!       CDCONF - configuration 'P'=> *(V**(-1/2)), 'Q' =>*(V**(1/2))
!                where V is the vertical correlation matrix
!       KSCALE - Wavelet scale
!       KNUMB - number of elements for which computations
!               are performed (used in message passing version)
!       PLON   - longitudes of gridpoints
!       PLAT   - latitudes of gridpoints
!       PGP3T0 - input 3d fields
!       PGP2T0 - input 2d fields
!     Output:
!       PGP3T1 - output 3d fields
!       PGP2T1 - output 2d fields

!     Author:  M. Fisher  *ECMWF* (based on JGNR and JGVCOR)
!     -------

!     Modifications:
!     --------------
!     ORIGINAL 98-06-02.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher      17-Mar-2004 Use nearest matrix rather than interpolating
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA

USE YOMWAVELET,ONLY : WAVELET_VCORS
USE YOMJG , ONLY : SPJB_VARS_INFO, N_SPJB_VARS
USE YOMCVA , ONLY : NVA3D, NVA2D
USE YOMGRB , ONLY : NGRBLNSP, NGRBT

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSCALE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGP3T0(NPROMA,NFLEVG,NVA3D) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGP2T0(NPROMA,NVA2D) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGP3T1(NPROMA,NFLEVG,NVA3D) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGP2T1(NPROMA,NVA2D) 
REAL(KIND=JPRB) :: ZTPS0(NPROMA,NFLEVG+1),ZTPS1(NPROMA,NFLEVG+1)
INTEGER(KIND=JPIM) :: ISP, JFIELD, IPTJB
INTEGER(KIND=JPIM),ALLOCATABLE   :: IINDEX(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "jbmatinterp.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JBVCORG',0,ZHOOK_HANDLE)
IF (CDCONF /= 'P' .AND. CDCONF /= 'Q') THEN
  CALL ABOR1 ('JBVCORG CALLED WITH INVALID CDCONF')
ENDIF

ALLOCATE (IINDEX(KNUMB))

ISP=HUGE(ISP)
DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBLNSP) THEN
    ISP = SPJB_VARS_INFO(JFIELD)%IPTJB
  ENDIF
ENDDO

DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB

!--- find the nearest matrix to each gridpoint
  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE/=NGRBLNSP) THEN
    CALL JBMATINTERP (WAVELET_VCORS(KSCALE,JFIELD)%LON0G, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%NLON_OFFSET, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%DLON, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%LATSG, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%NLAT_OFFSET, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%NLATSG, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%NLATS, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%NLONS, &
                    & PLON, PLAT, KNUMB, &
                    & WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, IINDEX)  
  ENDIF

!--- multiply the column by the matrix
  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE/=NGRBLNSP) THEN

      IF (CDCONF=='P') THEN
        CALL MULTBYMAT (WAVELET_VCORS(KSCALE,JFIELD)%MATS, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATS_SKYLINE_LEFT, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATS_SKYLINE_RIGHT, &
                      & IINDEX,1,WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, &
                      & PGP2T0(1,IPTJB),PGP2T1(1,IPTJB))
      ELSE
        CALL MULTBYMAT (WAVELET_VCORS(KSCALE,JFIELD)%MATINVS, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATINVS_SKYLINE_LEFT, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATINVS_SKYLINE_RIGHT, &
                      & IINDEX,1,WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, &
                      & PGP2T0(1,IPTJB),PGP2T1(1,IPTJB))
      ENDIF
      
    ENDIF
  ELSE
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT) THEN
      IF (ISP==HUGE(ISP)) THEN
        CALL ABOR1 ('T IS IN Jb, BUT LNSP IS MISSING')
      ENDIF
      ZTPS0(1:KNUMB,1:NFLEVG) = PGP3T0 (1:KNUMB,1:NFLEVG,IPTJB)
      ZTPS0(1:KNUMB,NFLEVG+1) = PGP2T0(1:KNUMB,ISP)

      IF (CDCONF=='P') THEN
        CALL MULTBYMAT (WAVELET_VCORS(KSCALE,JFIELD)%MATS, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATS_SKYLINE_LEFT, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATS_SKYLINE_RIGHT, &
                      & IINDEX,NFLEVG+1,WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, &
                      & ZTPS0,ZTPS1)
      ELSE
        CALL MULTBYMAT (WAVELET_VCORS(KSCALE,JFIELD)%MATINVS, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATINVS_SKYLINE_LEFT, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATINVS_SKYLINE_RIGHT, &
                      & IINDEX,NFLEVG+1,WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, &
                      & ZTPS0,ZTPS1)
      ENDIF

      PGP3T1(1:KNUMB,1:NFLEVG,IPTJB) = ZTPS1(1:KNUMB,1:NFLEVG)
      PGP2T1(1:KNUMB,ISP)            = ZTPS1(1:KNUMB,NFLEVG+1)
      
    ELSE

      IF (CDCONF=='P') THEN
        CALL MULTBYMAT (WAVELET_VCORS(KSCALE,JFIELD)%MATS, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATS_SKYLINE_LEFT, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATS_SKYLINE_RIGHT, &
                      & IINDEX,NFLEVG,WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, &
                      & PGP3T0(:,:,IPTJB),PGP3T1(:,:,IPTJB))
      ELSE
        CALL MULTBYMAT (WAVELET_VCORS(KSCALE,JFIELD)%MATINVS, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATINVS_SKYLINE_LEFT, &
                      & WAVELET_VCORS(KSCALE,JFIELD)%MATINVS_SKYLINE_RIGHT, &
                      & IINDEX,NFLEVG,WAVELET_VCORS(KSCALE,JFIELD)%NPTOT, &
                      & PGP3T0(:,:,IPTJB),PGP3T1(:,:,IPTJB))
      ENDIF

    ENDIF
  ENDIF
ENDDO

!     ------------------------------------------------------------------

DEALLOCATE (IINDEX)

IF (LHOOK) CALL DR_HOOK('JBVCORG',1,ZHOOK_HANDLE)

CONTAINS
SUBROUTINE MULTBYMAT (PMAT,KSKY_LEFT,KSKY_RIGHT,KINDEX,KROWS,KNMATS,PVEC,PRESULT)

!         Matrix multiply

INTEGER(KIND=JPIM), DIMENSION(KNUMB), INTENT(IN)           :: KINDEX
INTEGER(KIND=JPIM), INTENT(IN)                             :: KROWS, KNMATS
INTEGER(KIND=JPIM),DIMENSION(KROWS,KNMATS), INTENT(IN)     :: KSKY_LEFT, KSKY_RIGHT
REAL(KIND=JPRB), DIMENSION(KROWS,KROWS,KNMATS), INTENT(IN) :: PMAT
REAL(KIND=JPRB), DIMENSION(NPROMA,KROWS), INTENT(IN)       :: PVEC
REAL(KIND=JPRB), DIMENSION(NPROMA,KROWS), INTENT(OUT)      :: PRESULT

INTEGER(KIND=JPIM) :: JI,JJ,JGP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('JBVCORG:MULTBYMAT',0,ZHOOK_HANDLE)

DO JGP=1,KNUMB
  DO JJ=1,KROWS
    PRESULT(JGP,JJ) = 0.0_JPRB
    DO JI=KSKY_LEFT(JJ,KINDEX(JGP)),KSKY_RIGHT(JJ,KINDEX(JGP))
!mf- This IF block is unnecessary, because the skipped elements have been zeroed in sujbwavelet.
!mf!      IF (JJ>=KSKY_LEFT(JI,KINDEX(JGP)) .AND. JJ<=KSKY_RIGHT(JI,KINDEX(JGP))) THEN
        PRESULT(JGP,JJ) =  PRESULT(JGP,JJ)&
         & + PMAT(JI,JJ,KINDEX(JGP))*PVEC(JGP,JI)  
!mf!      ENDIF
    ENDDO
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('JBVCORG:MULTBYMAT',1,ZHOOK_HANDLE)
END SUBROUTINE MULTBYMAT
END SUBROUTINE JBVCORG
