SUBROUTINE JQVCOR(YDSPEC, YDVCOR)

!     Purpose:
!     --------
!       Apply inverse model error vertical correlations

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original 25-Nov-2003
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULERR
USE YOMDIM  , ONLY: NFLEVG
USE YOMMP   , ONLY: MYSETV
USE YOMJQ   , ONLY: ERRMODVCOR_STRUCT, NVALERR
USE YOMMODEL_ERROR, ONLY: NSERR
USE SPECTRAL_COLUMNS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(SPECTRAL_FIELD),INTENT(INOUT) :: YDSPEC 
TYPE(ERRMODVCOR_STRUCT),INTENT(IN) :: YDVCOR

TYPE(SPECTRAL_COLS) :: YLSPCOLS
REAL(KIND=JPRB) :: zx(NFLEVG+1)
INTEGER(KIND=JPIM) :: inn, ioff, jl, jc
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "trmtos_spec.intfb.h"
#include "trstom_spec.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JQVCOR',0,ZHOOK_HANDLE)

CALL ALLOCATE_SPCOLS(YLSPCOLS,YDSPEC%nspec2, &
                   & YDSPEC%ns3d,YDSPEC%ns2g,YDSPEC%ngrib)

! Transpose to get columns of coefficients
CALL TRMTOS_SPEC(YDSPEC, YLSPCOLS)

ioff=YLSPCOLS%NPTRSV(MYSETV)-1

DO jc=1,YLSPCOLS%NSPEC2V
  inn=NVALERR(jc+ioff)
  IF (inn<0 .OR. inn>NSERR) CALL ABOR1('JQVCOR: Error inn')

  CALL MATVEC(YDVCOR%VOR(:,:,inn),YLSPCOLS%VOR(:,jc))
  CALL MATVEC(YDVCOR%DIV(:,:,inn),YLSPCOLS%DIV(:,jc))
  IF (ASSOCIATED(YLSPCOLS%HUM)) CALL MATVEC(YDVCOR%HUM(:,:,inn),YLSPCOLS%HUM(:,jc))
  IF (ASSOCIATED(YLSPCOLS%OZN)) CALL MATVEC(YDVCOR%OZN(:,:,inn),YLSPCOLS%OZN(:,jc))

  DO jl=1,NFLEVG
    zx(jl)=YLSPCOLS%TEM(jl,jc)
  ENDDO
  zx(NFLEVG+1)=YLSPCOLS%SPR(jc)

  CALL MATVEC(YDVCOR%TPS(:,:,inn),zx)

  DO jl=1,NFLEVG
    YLSPCOLS%TEM(jl,jc)=zx(jl)
  ENDDO
  YLSPCOLS%SPR(jc)=zx(NFLEVG+1)
ENDDO

! Transpose back
CALL TRSTOM_SPEC(YLSPCOLS, YDSPEC)

CALL DEALLOCATE_SPCOLS(YLSPCOLS)

IF (LHOOK) CALL DR_HOOK('JQVCOR',1,ZHOOK_HANDLE)
! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------
SUBROUTINE MATVEC(PM,PX)
! Compute X=MX
IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN)    :: PM(:,:)
REAL(KIND=JPRB), INTENT(INOUT) :: PX(:)
INTEGER(KIND=JPIM) :: ji,jj,ilen
REAL(KIND=JPRB), ALLOCATABLE :: ZZ(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('JQVCOR:MATVEC',0,ZHOOK_HANDLE)
ilen=SIZE(PX)
IF (SIZE(PM,1)/=ilen) THEN
  WRITE(NULERR,*) 'MATVEC SIZE(PM,1)/=ilen ',SIZE(PM,1),ilen
  CALL ABOR1('MATVEC: Error size PM 1')
ENDIF
IF (SIZE(PM,2)/=ilen) THEN
  WRITE(NULERR,*) 'MATVEC SIZE(PM,2)/=ilen ',SIZE(PM,2),ilen
  CALL ABOR1('MATVEC: Error size PM 2')
ENDIF
ALLOCATE(ZZ(ilen))
ZZ(:)=PX(:)
PX(:)=0.0_JPRB
DO jj=1,ilen
  DO ji=1,ilen
    PX(ji)=PX(ji)+PM(ji,jj)*ZZ(jj)
  ENDDO
ENDDO
DEALLOCATE(ZZ)

IF (LHOOK) CALL DR_HOOK('JQVCOR:MATVEC',1,ZHOOK_HANDLE)
END SUBROUTINE MATVEC
! ------------------------------------------------------------------
END SUBROUTINE JQVCOR
