SUBROUTINE LITEST(YDINCR)

!     Purpose:
!     --------
!       Test linearity of tangent linear model inside 4D-Var configuration.

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original 27-Sep-2004
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULOUT
USE YOMVRTL , ONLY: LTLINT
USE YOMJG   , ONLY: LSUBBG
USE YOMDIM  , ONLY: NSMAX, NMSMAX, NS1D
USE YOMCVA  , ONLY: NVA3D, NVA2D, NGRBVAR
USE YOMVAR  , ONLY: NSIM4D, NITER4D, LJCDFI
USE SPECTRAL_FIELDS_MOD
USE CONTROL_VECTORS

! ----------------------------------------------------------------------

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN) :: YDINCR

TYPE(control_vector) :: YLX
TYPE(spectral_field) :: YLSP0, YLSP1
LOGICAL :: llsub, lldfi
INTEGER(KIND=JPIM) :: iter4d,isim4d
REAL(KIND=JPRB) :: za, zeps
REAL(KIND=JPRB) :: zero(NVA3D+NVA2D), znor(NVA3D+NVA2D), zerr(NVA3D+NVA2D)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "chavarin.intfb.h"
#include "cnt3tl.intfb.h"
#include "state2spec.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LITEST',0,ZHOOK_HANDLE)

WRITE(NULOUT,*)'LINTEST -----------------------'
WRITE(NULOUT,*)'LINTEST Starting linearity test'
WRITE(NULOUT,*)'LINTEST -----------------------'

! Initialize
! -----------
iter4d=NITER4D
isim4d=NSIM4D
llsub =LSUBBG
lldfi =LJCDFI
NITER4D=1
NSIM4D =1
LSUBBG =.false.
LJCDFI =.false.
LTLINT=.TRUE.
za=1000.0_JPRB
zeps=EPSILON(zeps)

WRITE(NULOUT,'(A,ES9.3)')' LINTEST Ratio large/small perturbations: ',za
WRITE(NULOUT,*)'LINTEST                Vor        Div        ', &
 & 'Tem        Hum         O3       S.Pres'  

CALL ALLOCATE_CTLVEC(YLX)
CALL ALLOCATE_SPEC(YLSP0,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
CALL ALLOCATE_SPEC(YLSP1,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)

! Run linear model with zero perturbation
! ---------------------------------------
YLX=0.0_JPRB
CALL CHAVARIN(YLX)
CALL CNT3TL

CALL STATE2SPEC(YLSP0)
CALL SPECTRAL_NORM(YLSP0,zero)
WRITE(NULOUT,'(A,6(2X,ES9.3))')' LINTEST Zero Pert :',zero(:)

! Run linear model with perturbation
! ----------------------------------
YLX=YDINCR
CALL CHAVARIN(YLX)
CALL CNT3TL
CALL STATE2SPEC(YLSP0)

CALL SPECTRAL_NORM(YLSP0,znor)
WRITE(NULOUT,'(A,6(2X,ES9.3))')' LINTEST Small Pert:',znor(:)

! Run linear model with za*perturbation
! -------------------------------------
YLX=YDINCR
YLX%data=za*YLX%data
CALL CHAVARIN(YLX)
CALL CNT3TL
CALL STATE2SPEC(YLSP1)

CALL SPECTRAL_NORM(YLSP1,znor)
WRITE(NULOUT,'(A,6(2X,ES9.3))')' LINTEST Large Pert:',znor(:)

! Evaluate error
! --------------
YLSP1%SP3D=YLSP1%SP3D-za*YLSP0%SP3D
YLSP1%SP2D=YLSP1%SP2D-za*YLSP0%SP2D

CALL SPECTRAL_NORM(YLSP1,zerr)
WRITE(NULOUT,'(A,6(2X,ES9.3))')' LINTEST Abs. Error:',zerr(:)

IF (ANY(znor(:)<=zeps)) THEN
  WRITE(NULOUT,*)'LINTEST Meaningless: Perturbation too small'
ELSE
  zerr(:)=zerr(:)/znor(:)
  WRITE(NULOUT,'(A,6(2X,ES9.3))')' LINTEST Rel. Error:',zerr(:)

  zerr(:) = zerr(:) / EPSILON(za)
  WRITE(NULOUT,'(A,6(2X,F9.1))') ' LINTEST /Err Mach.:',zerr(:)
ENDIF

! ------------------------------------------------------------------

CALL DEALLOCATE_CTLVEC(YLX)
CALL DEALLOCATE_SPEC(YLSP0)
CALL DEALLOCATE_SPEC(YLSP1)
NITER4D=iter4d
NSIM4D =isim4d
LSUBBG =llsub
LJCDFI =lldfi
LTLINT=.FALSE.

WRITE(NULOUT,*)'LINTEST -----------------------'
WRITE(NULOUT,*)'LINTEST End Linearity Test'
WRITE(NULOUT,*)'LINTEST -----------------------'

IF (LHOOK) CALL DR_HOOK('LITEST',1,ZHOOK_HANDLE)
END SUBROUTINE LITEST
