SUBROUTINE SAVMINI(YDVAZX,CDFILE)

!***  *SAVMINI* save minimisation algorithm information.

!**   Purpose.
!     --------
!     this routine saves the control variable
!     for a subsequent minimisation.

!**   Interface.
!     *CALL* *SAVMINI (YVAZX)* from CVA1

!        Explicit arguments :
!        --------------------
!        YDVAZX  : control variable
!        CDFILE : file name (optional)

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------
!        none.

!     Externals.
!     ----------
!      none

!     Reference.
!     ----------
!        ECMWF research department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel Thepaut *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-03-04
!        Modified : 95-06-06 L.Isaksen   Reordering of spectral arrays
!        Modified : 96-02-20 L.Isaksen   Adapted for Message Passing
!        Modified : 96-10-11 F.Rabier    Do not use MIO if not on Cray
!        Modified : 98-01-10 M.Hamrud    Minimisation parallelized
!        Modified 97-05    by R. Randriamampianina 3DVAR ALADIN
!        Modified : 99-08-02 M.Fisher    Move M1QN3 stuff to SAVHESS
!        Modified : 00-08-09 Y.Tremolet  Cleaning
!        Modified : 2001-02-12 M. Fisher Optional argument CDFILE
!        Modified : 02-08-06 Y.Tremolet  Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        D.Salmond     18-Apr-2007 Use multiple processors for I/O
!        T.Wilhelmsson 26-Aug-2009 Move multiprocessor I/O to SAVMINI2
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF    ,NSTART   ,NPROC
USE YOMNMIA  , ONLY : NITNMI   ,LNMIRQ
USE YOMVAR   , ONLY : NUPTRA   ,LTRREF
USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : MYPROC
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_PUT, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST,CLOSE_IOREQUEST
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE CONTROL_VECTORS_COMM_MOD, ONLY : GATHER_CTLVEC

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(IN)      :: YDVAZX 
CHARACTER(LEN=*),INTENT(IN),OPTIONAL :: CDFILE 

CHARACTER :: CLTIME*8
CHARACTER :: CLUPTRA*2
CHARACTER :: CLFN*128
REAL(KIND=JPRB), ALLOCATABLE :: ZBUF(:)

INTEGER(KIND=JPIM) :: IDIMR, IOMASTER, ISTEP, IBUF(3)
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.   DEFINE LENGTH OF BUFFER AND FILE NAME
!        ------------------------------------------

IF (LHOOK) CALL DR_HOOK('SAVMINI',0,ZHOOK_HANDLE)
IF (PRESENT(CDFILE)) THEN
  CLFN=CDFILE
ELSE
  IF(.NOT.LNMIRQ) THEN
    ISTEP=NSTART
  ELSE
    ISTEP=NSTART-NITNMI
  ENDIF
  IF(ISTEP >= 0) THEN
    WRITE(CLTIME,'(''+'',I7.7)') ISTEP
  ELSE
    WRITE(CLTIME,'(''-'',I7.7)') -ISTEP
  ENDIF
  WRITE(CLUPTRA,'(I2.2)') NUPTRA

  IF(LTRREF) THEN
    CLFN='tref'//CLTIME
  ELSEIF(NCONF == 131) THEN
    CLFN='traj'//CLUPTRA//CLTIME
  ELSE
    CLFN='traj'//CLTIME
  ENDIF
ENDIF

WRITE(NULOUT,*) 'SAVMINI - I/O not parallel - write to file :',CLFN

!     --------------------------------------------------

!*    2.  WRITE THE CONTROL VARIABLE
!     -------------------------------

IOMASTER = MIN(1,NPROC)
CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='w',KIOMASTER=IOMASTER)
CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')

IF (MYPROC==IOMASTER) THEN
  IBUF(1) = YDVAZX%NSIZEG
  IBUF(2) = YDVAZX%NSMAX
  IBUF(3) = YDVAZX%NSERR
  IDIMR   = YDVAZX%NSIZEG
ELSE
  IDIMR=1
ENDIF
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)

ALLOCATE(ZBUF(IDIMR))

CALL GATHER_CTLVEC(YDVAZX,IOMASTER,ZBUF)

CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=ZBUF)
CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)
DEALLOCATE(ZBUF)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SAVMINI',1,ZHOOK_HANDLE)
END SUBROUTINE SAVMINI
