SUBROUTINE SAVMINI2(YDVAZX,CDFILE,kvec_start,kvec_end)

!***  *SAVMINI2* save minimisation algorithm information.

!**   Purpose.
!     --------
!     this routine saves the control variable
!     for a subsequent minimisation.

!**   Interface.
!     *CALL* *SAVMINI2 (YVAZX)* from CVA1

!        Explicit arguments :
!        --------------------
!        YDVAZX  : control variable
!        CDFILE : file name (optional)

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------
!        none.

!     Externals.
!     ----------
!      none

!     Reference.
!     ----------
!        ECMWF research department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel Thepaut *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-03-04
!        Modified : 95-06-06 L.Isaksen   Reordering of spectral arrays
!        Modified : 96-02-20 L.Isaksen   Adapted for Message Passing
!        Modified : 96-10-11 F.Rabier    Do not use MIO if not on Cray
!        Modified : 98-01-10 M.Hamrud    Minimisation parallelized
!        Modified 97-05    by R. Randriamampianina 3DVAR ALADIN
!        Modified : 99-08-02 M.Fisher    Move M1QN3 stuff to SAVHESS
!        Modified : 00-08-09 Y.Tremolet  Cleaning
!        Modified : 2001-02-12 M. Fisher Optional argument CDFILE
!        Modified : 02-08-06 Y.Tremolet  Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        D.Salmond     18-Apr-2007 Use multiple processors for I/O
!        T.Wilhelmsson 26-Aug-2009 Multi-processor I/O version moved here
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF    ,NSTART   ,NPROC
USE YOMNMIA  , ONLY : NITNMI   ,LNMIRQ
USE YOMVAR   , ONLY : NUPTRA   ,LTRREF
USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : MYPROC
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_PUT, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST,CLOSE_IOREQUEST
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE CONTROL_VECTORS_COMM_MOD, ONLY : MULTIGATHER_CTLVEC
USE YOMGETMINI, ONLY : MIOPROC, MBUF, RZVAZX, NUM_PROCS_PER_IO_PROC

IMPLICIT NONE


TYPE(CONTROL_VECTOR),INTENT(IN) :: YDVAZX(:)
CHARACTER(LEN=*),INTENT(IN)     :: CDFILE 
INTEGER(KIND=JPIM), INTENT(IN)  :: kvec_start 
INTEGER(KIND=JPIM), INTENT(IN)  :: kvec_end

CHARACTER :: CLFN*128
CHARACTER :: CLFS*128
CHARACTER :: CLFE*128

INTEGER(KIND=JPIM) :: IDIMR, jvec, JROC
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       0.1   DEFINE LENGTH OF BUFFER AND FILE NAME.
!              ---------------------------------------

IF (LHOOK) CALL DR_HOOK('SAVMINI2',0,ZHOOK_HANDLE)

CLFN = CDFILE

jvec = kvec_start

DO JROC = 1,NPROC,NUM_PROCS_PER_IO_PROC
  MIOPROC(jvec) = JROC
  IF (MYPROC == MIOPROC(jvec)) THEN
    WRITE(CLFN(7:),'(I3.3)') jvec
  ENDIF
  jvec = jvec + 1
  IF (jvec > kvec_end) EXIT
ENDDO

CLFS = CLFN
CLFE = CLFN
WRITE(CLFS(7:9),'(I3.3)') kvec_start
WRITE(CLFE(7:9),'(I3.3)') kvec_end
WRITE(NULOUT,*) 'SAVMINI2 - I/O parallel - write to files :',CLFS(1:9),' to ',CLFE(1:9)

!     --------------------------------------------------


!*    1.  WRITE THE CONTROL VARIABLE
!     -------------------------------

! write(0,*)'SAVMINI2 - MIOPROC - GATHER',kvec_start,kvec_end,MIOPROC(kvec_start:kvec_end)

DO jvec = kvec_start, kvec_end
  IF(MYPROC == MIOPROC(jvec)) THEN
    MBUF(1) = YDVAZX(jvec)%NSIZEG
    MBUF(2) = YDVAZX(jvec)%NSMAX
    MBUF(3) = YDVAZX(jvec)%NSERR
    IDIMR   = YDVAZX(jvec)%NSIZEG
    DEALLOCATE(RZVAZX)
    ALLOCATE(RZVAZX(IDIMR))
  ENDIF
ENDDO

CALL MULTIGATHER_CTLVEC(YDVAZX,MIOPROC,RZVAZX,kvec_start,kvec_end)


! write(0,*)'SAVMINI2 - MIOPROC - WRITE',kvec_start,kvec_end,MIOPROC(kvec_start:kvec_end)

CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='w',KIOPROCS=MIOPROC(kvec_start:kvec_end))
CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')

CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=MBUF)
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=RZVAZX)

CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SAVMINI2',1,ZHOOK_HANDLE)
END SUBROUTINE SAVMINI2
