SUBROUTINE SCALJGG

!**** *SCALJGS*  - DEFINE INNER PRODUCT FOR JG, GRID POINT COMPUTATIONS

!     PURPOSE.
!     --------

!     DEFINE THE INNER PRODUCT FOR THE MINIMIZATION

!**   INTERFACE.
!     ----------

!        *CALL* *SCALJGG*

!        INPUT:

!        OUTPUT:

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        W. HECKLEY    *ECMWF*


!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-02-01
!        Modified : 91-10-15 Bill Heckley and Mats Hamrud ECMWF
!                            Use of MIOLIB for IO on forecast error files
!        MODIFIED W. HECKLEY 92-09-22 provision for NCNTVAR=3
!        MODIFIED M. Hamrud  95-12-15 remove  NCNTVAR=3,cleaning
!        Modified E. Holm    01-09-14 Move normalization to STEPOTL
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM,    JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMSPJB  , ONLY : SPJB

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE
INTEGER(KIND=JPIM) :: ISCALE

#include "cvargptl.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SCALJGG',0,ZHOOK_HANDLE)
WRITE(NULOUT,'('' SCALJGG'')')

!*    1.    NORMALISATION BY FORECAST ERRORS
!*          AND VERTICAL PROJECTIONS (OPTIONAL)
!*          --------------------------------

!           NORMALISATION IN GRID-POINT SPACE

!*    CONVERT DIFFERENCES FROM VORTICITY, DIVERGENCE TO U,V AND
!*    TRANSFORM FROM SPHERICAL HARMONICS TO MODEL GRID
!*    INPUT TAKEN FROM SP, OUTPUT IN TIME T GRID POINT ARRAYS
!*    : STEPOTL("0AA000000")
!*
!*    NORMALISE THE DIFFERENCES WRT THE FORECAST ERROR VARIANCES
!*    INPUT IN TIME T GRID POINT ARRAYS, OUTPUT IN T + DT ARRAYS
! *** AND IF (LVTPEC) THEN ***
!     PROJECT ONTO THE EIGENVECTORS OF THE VERTICAL PREDICTION
!     ERROR CORRELATION MATRICES
!     AND MULTIPLY BY THE SQUARE ROOT OF THE EIGENVALUES
!*    INPUT IN TIME T + DT GRID POINT ARRAYS, OUTPUT IN T + DT ARRAYS
!*    : STEPOTL("00000G000")
!*
!*    TRANSFORM NORMALISED VALUES FROM THE MODEL GRID TO
!*    SPHERICAL HARMONICS AND CONVERT FROM U,V TO VORTICITY
!*    AND DIVERGENCE
!*    INPUT IN T + DT GRID POINT ARRAYS, OUTPUT IN SP ARRAYS
!*    : STEPOTL("000000AA0")

ISCALE=0
CALL CVARGPTL('G',ISCALE,SPJB%SP3D,SPJB%SP2D,SPJB%SP1D)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SCALJGG',1,ZHOOK_HANDLE)
END SUBROUTINE SCALJGG

