SUBROUTINE SETQCCMA(KDIM1,KDIM2,KDIM3,&
 & LDVARQC,KOBS,KCOUNT,KBODY,&
 & POM1DEPQC,PQCW)  

!**** SETQCCMA - SET QC-RELATED WORDS IN CMA

!     ERIK ANDERSSON         ECMWF           95-12-12

!     PURPOSE
!     -------

!        SETS QC-WORDS IN CMA

!**   INTERFACE
!     ---------

!        CALL SETQCCMA(KDIM1,KDIM2,KDIM3
!                     ,KOBS,KCOUNT,KBODY
!                     ,POM1DEPQC,PQCW)

!        WHERE KDIM1  : FIRST DIMENSION
!              KDIM2  : SECOND DIMENSION
!              KDIM3  : THIRD DIMENSION
!              LDVARQC: VARQC SWITCH
!              KOBS   : A GIVEN OBSERVATION
!              KCOUNT : NO. OF A PARTICULAR OBS. PARAMETER
!              POM1DEPQC:WORK ARRAY TO BE USED IN ADJOINT
!              PQCW   : PROBABILITY OF GROSS ERROR (AFTER QC)

!**   METHOD
!     ------
!        JUST PUT IT IN

!*    EXTERNALS
!     ---------

!        FLGDSE - SET DATUM FLAG
!        CHDSTA - CHANGE DATUM STATUS
!        CHDEVE - CHANGE (ADD) DATUM EVENT

!**   MODIFICATIONS
!     -------------
!     E.Andersson : 96/06/03 - Set new datum event: VARQC done
!     E.Andersson : 96/11/21 - Pass dimensions as arguments
!     E.Andersson : 96/12/04 - VarQC for 4D-Var

!-----------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF    ,LOBSREF  ,LOBSC1
USE YOMVAR   , ONLY : NSIM4D   ,NSIM4DL  ,NUPTRA
USE YOMCMA   , ONLY : NRESUPD
USE YOMANCS  , ONLY : RMDI
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMCOSJO , ONLY : LVARQCG  ,LQVARQC
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM1
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM2
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM3
LOGICAL           ,INTENT(IN)    :: LDVARQC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDIM1) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBODY(KDIM1,KDIM2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POM1DEPQC(KDIM3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCW(KDIM3) 
INTEGER(KIND=JPIM) :: IFL, IFLAG, IBODY, JBODY

REAL(KIND=JPRB) :: ZPGE
LOGICAL :: LLUPDVQC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "nflgdse.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"

!*
!     ------------------------------------------------------------------

!        1.       Set PQCW IN MDB_QC_PGE and POM1DEPQC in MDBIOM0

IF (LHOOK) CALL DR_HOOK('SETQCCMA',0,ZHOOK_HANDLE)
IF(NCONF/100 /= 1.OR. LOBSREF .AND. LHOOK) CALL DR_HOOK('SETQCCMA',1,ZHOOK_HANDLE)
IF(NCONF/100 /= 1.OR. LOBSREF) RETURN

!        1.1      POM1DEPQC is communicated in CMA, for use in adjoint
!                 We therefore avoid re-doing the QC in the adjoint!

DO JBODY=1,KCOUNT(KOBS)
  IBODY=KBODY(KOBS,JBODY)
  ROBODY(IBODY,MDBIOM0(NUPTRA+1))=POM1DEPQC(JBODY)
ENDDO

!        1.2      Record result of QC in CMA, if final simulation,
!                 final update.

IF(LQVARQC) THEN
! Final trajectory - if quadratic VarQC
  LLUPDVQC=LOBSC1 .AND. LVARQCG .AND. NUPTRA == NRESUPD
ELSE
! Final simulation of the last minimisation
  LLUPDVQC=NSIM4D == NSIM4DL .AND. LVARQCG.AND. NUPTRA == NRESUPD-1
ENDIF

IF(LLUPDVQC) THEN
  DO JBODY=1,KCOUNT(KOBS)
    IBODY=KBODY(KOBS,JBODY)

!        1.2.1    Probability of Gross Error
    IF(.NOT.LDVARQC) THEN
      ROBODY(IBODY,MDB_QC_PGE)=RMDI
    ELSE
      ZPGE=1.0_JPRB-PQCW(JBODY)
      ROBODY(IBODY,MDB_QC_PGE)=ZPGE

!        1.2.2    Set VARQC flag in CMA. Reject if > 75% wrong.
      IFL = INT(ZPGE/0.25_JPRB)
      IFLAG = MAX(MIN(IFL,3),0)
      CALL NFLGDSE(IBODY,'VARQC',IFLAG)
      IF(IFLAG == 3) THEN
        ROBODY(IBODY,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IBODY,MDBDSTA))
      ENDIF

!        1.2.3    Set event to record that VARQC has been performed
      ROBODY(IBODY,MDBDEV1) = ZCHEV1( ROBODY(IBODY,MDBDEV1),ND1EVENT(27))
    ENDIF
  ENDDO
ENDIF
!*
!     ------------------------------------------------------------------

!        2.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SETQCCMA',1,ZHOOK_HANDLE)
END SUBROUTINE SETQCCMA
