SUBROUTINE SQRTBAD(YDSPEC,YDCV)

!**** *CVARU3*    - TRANSFORMATION OF MODEL FIELDS *ADJOINT*

!     PURPOSE.
!     --------

!        TRANSFORM FROM MODEL VARIABLES FROM X TO X'

!**   INTERFACE.
!     ----------

!        *CALL* *CVARU3*

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!             *CVARGPAD* - div by fc errors grid-point space-adjoint
!             *JGNRSAD* - div by fc errors spectral space-adjoint
!             *JGCORAD* - multiplication by sqrt of correlations (multitasked)
!             *USER_CLOCK* - timing

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        W. HECKLEY    *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL 93-06-07
!        94-03-03 E. Andersson Multitasking spectral JG comp.
!        94-11-01 M.Hamrud : new configurations for transform steps
!        94-10-07 F. Bouttier  Change multitasking from N to M & interface
!        94-10-xx M. Hamrud    Multitasking diagnostics
!        96-05-05 L. Isaksen   Adapted for Message Passing
!        96-12-18 J.Barkmeijer & F.Bouttier    total energy Jb
!        97-05    G. Gregoric and S. Issara 3DVAR ALADIN
!        97-10-06 D.Dent :  replace SECOND by USER_CLOCK
!        97-11-26 Mike Fisher  Zero m=n=0 component of SPVOR and SPDIV
!        98-07-06 E.Andersson  TOVS control variable
!        00-01-13 C. Fischer   Gridpoint normalization in LAM
!        00-05-15 F.Bouttier   spectral q and o3 normalization
!        02-08-05 Y.Tremolet   Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher      17-Mar-2004 Wavelet Jb
!        L.Isaksen     01-July2004 Replace stepo call by cvargpad call
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NSMAX    ,NSPEC2   ,NFLEVL
USE YOMJG    , ONLY : LSPFCE   ,NGEOCOR  , LJGNRSGP, FCEMN3D, &
                    & SPJB_VARS_INFO, N_SPJB_VARS
USE YOMLUN   , ONLY : NULDILA  ,NULCONT
USE YOMCT0   , ONLY : LELAM   ,NCONF
USE YEMVARGP , ONLY : LEVARGP
USE YOMMP    , ONLY : MYLEVS
USE YOMGRB   , ONLY : NGRBQ, NGRBO3
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(INOUT) :: YDSPEC
TYPE(CONTROL_VECTOR), INTENT(INOUT) :: YDCV 

INTEGER(KIND=JPIM) :: IDUM,IBIDON,IREP,ILV,JLV,ISCALE,IPTJB,JFIELD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "jgcorad.intfb.h"
#include "jgnrs.intfb.h"
#include "cvargpad.intfb.h"
#include "spec_imzero.intfb.h"

!     ------------------------------------------------------------------

!*    2.    SPECTRAL JG CALCULATIONS : CORRELATIONS MULTITASKED OVER M
!           ----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SQRTBAD',0,ZHOOK_HANDLE)
IF(NGEOCOR == 1) THEN
  CALL JGCORAD(.TRUE.,1,NSPEC2,YDCV,YDSPEC)
ELSEIF(NGEOCOR == 2)THEN
  IDUM=-99
  CALL LFIOUV(IREP,NULDILA,.TRUE.,'MATDILA','UNKNOWN',.TRUE.,&
   & .FALSE.,1,NSMAX+1,IBIDON)  
  CALL LFIOUV(IREP,NULCONT,.TRUE.,'MATCONT','UNKNOWN',.TRUE.,&
   & .FALSE.,1,NSMAX+1,IBIDON)  
  CALL JGCORAD(.TRUE.,IDUM,IDUM,YDCV,YDSPEC)
  CALL LFIFER(IREP,NULDILA,'UNKNOWN')
  CALL LFIFER(IREP,NULCONT,'UNKNOWN')
ENDIF

!     ------------------------------------------------------------------

!*    1.    DIVIDE BY FORECAST ERRORS
!           -------------------------

IF (NCONF  ==  131.AND. LELAM) THEN
  IF (LSPFCE.AND.(.NOT.LEVARGP)) THEN
    CALL JGNRS(YDSPEC)
  ELSE
    ISCALE=0
    CALL CVARGPAD('G',ISCALE,YDSPEC%SP3D,YDSPEC%SP2D,YDSPEC%SP1D)
  ENDIF
ELSE
  IF (LSPFCE) THEN
    CALL JGNRS(YDSPEC)
  ELSE
    IF (LJGNRSGP) THEN
      DO JFIELD=1,N_SPJB_VARS
        IPTJB=SPJB_VARS_INFO(JFIELD)%IPTJB
        IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBQ .OR. &
          & SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
          DO JLV=1,NFLEVL
            ILV=MYLEVS(JLV)
            YDSPEC%SP3D(JLV,:,IPTJB)=YDSPEC%SP3D(JLV,:,IPTJB)/FCEMN3D(ILV,IPTJB)
          ENDDO
        ENDIF
      ENDDO
    ENDIF
    ISCALE=0
    CALL CVARGPAD('G',ISCALE,YDSPEC%SP3D,YDSPEC%SP2D,YDSPEC%SP1D)
  ENDIF
ENDIF

CALL SPEC_IMZERO(YDSPEC)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SQRTBAD',1,ZHOOK_HANDLE)
END SUBROUTINE SQRTBAD
