SUBROUTINE SQRTBIN(YDCV,YDSPEC,PGP3,PGP2)

!**** *CVARU3I*    - TRANSFORMATION OF MODEL FIELDS

!     PURPOSE.
!     --------

!        TRANSFORM FROM MODEL VARIABLES FROM X' TO X

!**   INTERFACE.
!     ----------

!        *CALL* *CVARU3I*

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!             *CVARGPTL* - division by forecast errors - grid-point space
!             *JGNRS* - division by forecast errors spectral space
!             *JGCORI*- multiplication by SQRT of correlations (multitasked)

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        W. HECKLEY    *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL 93-07-06
!        94-03-03 E. Andersson Multitasking spectral JG comp.
!        94-11-01 M.Hamrud : new configurations for transform steps
!        94-10-07 F. Bouttier  Change multitasking from N to M & interface
!        94-10-10 M. Hamrud    Multitasking diagnostics
!        96-05-05 L. Isaksen   Adapted for Message Passing
!        96-12-18 J.Barkmeijer & F.Bouttier    total energy Jb
!        97-10-06 D.Dent :  replace SECOND by USER_CLOCK
!        97-11-26 Mike Fisher  Zero m=n=0 component of SPVOR and SPDIV
!        97-05    G. Gregoric and S. Issara    3DVAR ALADIN
!        98-08-18 E.Andersson  TOVS control variable
!        00-05-15 F.Bouttier   spectral q and o3 normalization
!        00-01-13 C. Fischer   Gridpoint normalization in LAM
!        01-09-14 E.Holm       Move forecast error division to STEPOTL
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher      17-03-2004 Wavelet Jb
!        Y.Bouteloup   28-02-2005 Change call to stepotl (see below for explanation)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK
USE YOMDIM   , ONLY : NSMAX, NSPEC2, NFLEVL
USE YOMJG    , ONLY : LSPFCE, NGEOCOR, LJGNRSGP, FCEMN3D, &
                    & SPJB_VARS_INFO, N_SPJB_VARS
USE YOMCT0   , ONLY : LELAM, NCONF
USE YOMLUN   , ONLY : NULDILA, NULCONT
USE YEMVARGP , ONLY : LEVARGP
USE YOMMP    , ONLY : MYLEVS
USE YOMGRB   , ONLY : NGRBQ, NGRBO3
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(IN)    :: YDCV 
TYPE(SPECTRAL_FIELD),INTENT(INOUT) :: YDSPEC
REAL(KIND=JPRB), INTENT(OUT) :: PGP3(:,:,:),PGP2(:,:,:)

INTEGER(KIND=JPIM) :: IDUM,IBIDON,IREP,ILV,JLV,ISCALE,IPTJB,JFIELD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cvargptl.intfb.h"
#include "jgcori.intfb.h"
#include "jgnrsi.intfb.h"
#include "spec_imzero.intfb.h"

!     ------------------------------------------------------------------

!*    2.    SPECTRAL JG CALCULATIONS : CORRELATIONS MULTITASKED OVER 
!           M FOR STRETCHED ROTATED COORDINATES  (NGEOCOR.EQ.2) OR
!           SUBSET OF SPECTRAL COEFFICIENTS FOR ALL OTHER CASES
!           ---------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SQRTBIN',0,ZHOOK_HANDLE)

IF(NGEOCOR == 1) THEN
  CALL JGCORI(.TRUE.,1,NSPEC2,YDCV,YDSPEC)
ELSEIF(NGEOCOR == 2)THEN
  IDUM=-99
  CALL LFIOUV(IREP,NULDILA,.TRUE.,'MATDILA','UNKNOWN',.TRUE.,&
   & .FALSE.,1,NSMAX+1,IBIDON)  
  CALL LFIOUV(IREP,NULCONT,.TRUE.,'MATCONT','UNKNOWN',.TRUE.,&
   & .FALSE.,1,NSMAX+1,IBIDON)  
  CALL JGCORI(.TRUE.,IDUM,IDUM,YDCV,YDSPEC)
  CALL LFIFER(IREP,NULDILA,'UNKNOWN')
  CALL LFIFER(IREP,NULCONT,'UNKNOWN')
ENDIF

!     ------------------------------------------------------------------

!*    3.    DE-NORMALISATION BY FORECAST ERRORS
!*          AND VERTICAL PROJECTIONS (OPTIONAL)
!*          -----------------------------------

IF (NCONF  ==  131.AND. LELAM) THEN
  IF (LSPFCE.AND.(.NOT.LEVARGP)) THEN
    CALL JGNRSI(YDSPEC)
  ELSE
    ISCALE=0
    CALL CVARGPTL('X',ISCALE,YDSPEC%SP3D,YDSPEC%SP2D,YDSPEC%SP1D,PGP3,PGP2)
  ENDIF
ELSE
  IF (LSPFCE) THEN
    CALL JGNRSI(YDSPEC)
  ELSE
    IF (LJGNRSGP) THEN
      DO JFIELD=1,N_SPJB_VARS
        IPTJB=SPJB_VARS_INFO(JFIELD)%IPTJB
        IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBQ .OR. &
          & SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
          DO JLV=1,NFLEVL
            ILV=MYLEVS(JLV)
            YDSPEC%SP3D(JLV,:,IPTJB)=YDSPEC%SP3D(JLV,:,IPTJB)*FCEMN3D(ILV,IPTJB)
          ENDDO
        ENDIF
      ENDDO
    ENDIF
    ISCALE=0
    CALL CVARGPTL('X',ISCALE,YDSPEC%SP3D,YDSPEC%SP2D,YDSPEC%SP1D,PGP3,PGP2)
  ENDIF
ENDIF

CALL SPEC_IMZERO(YDSPEC)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SQRTBIN',1,ZHOOK_HANDLE)
END SUBROUTINE SQRTBIN
