SUBROUTINE SUALCOS

!**** *SUALCOS* - Allocate the arrays for the cost function computation.

!     Purpose.
!     -------
!           Allocation of the arrays needed to store the cost function
 
!**   Interface.
!**   ---------
!**   *CALL* * SUALCOS*
!**         The common YOMCOSJO,YOMCOSJB....are implicit outputs of SUALCOS
!**         SUALCOS is called from routine SUALLO

!     Author: Jean Pailleux
!     Date  : 89-11-27

!     Modifications
!     -------------
!                  Jean Pailleux: 90-06-05
!                  Jean Pailleux: 90-08-12
!                  D. Vasiljevic: 90-11-21
!    F. Rabier and J.N. Thepaut : 91-01-15
!             J. N. Thepaut: 91-02-15 (allocate space for Jc)
!             J. N. Thepaut: 92-11-26 (allocate space in case of NCONF=1)
!             L.Labbe : 93-04-15 (allocate space in case of NCONF=8xx Jr)
!             L.Labbe : 94-03-10 (allocate space in case of NCONF=8xx Jb)
!     E.Andersson     : 94-03-28 (Make Jo reproducible)
!     L.Isaksen       : 96-04-04 (Adapted for Message Passing)
!     L. Dragulanescu : 97-05-24 3DVAR ALADIN
!     E.Andersson     : 94-03-28 Introduce new Jo-table JOT
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     N.Bormann       : 04-12-23 Move JOT allocation for LIMB obs to sulimb
!        V. Guidard    01-Apr-2004 bugfix on FUVB + add Jk event
!     R. El Khatib: 31-Aug-2007 Remove NVATOVV from allocations
!        Y.Tremolet    29-May-2008 Print Jo table by bins
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARDIMO  , ONLY : JPXVAR
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG   ,NF3D     ,NFD2D    ,NUMP
USE YOMLUN   , ONLY : NULOUT
USE YOMCMA   , ONLY : NMXOTP
USE YOMDIMO  , ONLY : NOBTOT
USE YOMCT0   , ONLY : NCONF    ,NPRINTLEV,LOBSC1   ,&
 & LSIMOB   ,LALLOPR  ,LELAM   ,LOBS
USE YOMCOSJO , ONLY : JOT      ,NJOTTSZ  ,NJOTISZ, FJOS, NPRTBINS
USE YOMCOSJC , ONLY : FJCCOST  ,RINCLJC
USE YOMCOSJR , ONLY : FALTSEL  ,FSURSEL  ,FSPALT   ,FSPSUR
USE YEMCOSJR , ONLY : FUVB     ,FUVBSEL
USE YOMMP    , ONLY : NPSP
USE YOMVAR   , ONLY : LAVARC
USE YOMCOCTP , ONLY : CCMOOTP  ,CCMOCTP  ,MNOCTP   ,NSATOB   ,NSATEM, &
 & NLIMB  
USE YEMJK    , ONLY : LEJK
USE YOMMODEL_ERROR, ONLY: N_COUPLED_WINDOWS
IMPLICIT NONE

LOGICAL :: LLP
INTEGER(KIND=JPIM) :: IU,ICDTYP,IOBTYP,INOCTP,IDUM
INTEGER(KIND=JPIM) :: JOTP,JCTP,JOTPNEW,JCTPNEW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cmoctmap.intfb.h"

!     ------------------------------------------------------------------

!*       1.       ALLOCATION OF THE COST FUNCTION
!                 -------------------------------

IF (LHOOK) CALL DR_HOOK('SUALCOS',0,ZHOOK_HANDLE)
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT

!*          1.1   ALLOCATION OF OBS. COST FUNCTION

IF (LLP) WRITE(NULOUT,'('' SUALCOS PRINTOUTS '')')
IF((.NOT.LSIMOB.AND.NCONF/100 == 1).OR.NCONF/100 == 7 &
   & .OR.LOBSC1)                                        THEN  

  ALLOCATE(FJOS(NOBTOT))
  IF(LLP)WRITE(IU,9) 'FJOS     ',SIZE(FJOS     ),SHAPE(FJOS     )
  FJOS(:)=0.0_JPRB

!*          1.1.1 Allocation for code types, for each observation type.

  IF (NPRTBINS==1.AND.N_COUPLED_WINDOWS>1) NPRTBINS=N_COUPLED_WINDOWS
  write(nulout,*)'SUALCOS: NPRTBINS=',NPRTBINS

  NJOTTSZ = 0
  NJOTISZ = 2*JPXVAR+3*JPXVAR*NPRTBINS
  JOT(:)%size=0

  IF(LOBS) THEN
!   DO JOTP=1,NMXOTP
    JOTP=1
    DO WHILE(JOTP<=NMXOTP)
      JOTPNEW=JOTP
      IDUM=1
      CALL CMOCTMAP('SQTOCODE',NULOUT,IOBTYP,ICDTYP,JOTPNEW,IDUM)
      JOT(IOBTYP)%name=CCMOOTP(IOBTYP)
      IF(.NOT.(IOBTYP == NSATEM .OR. IOBTYP == NSATOB .OR. IOBTYP == NLIMB))THEN
        INOCTP=MNOCTP(IOBTYP)
        ALLOCATE(JOT(IOBTYP)%jost(INOCTP))
        JOT(IOBTYP)%size=INOCTP
        NJOTTSZ = NJOTTSZ + JOT(IOBTYP)%size * NJOTISZ

!       DO JCTP=1,INOCTP
        JCTP=1
        DO WHILE(JCTP<=INOCTP)
          JCTPNEW=JCTP
          CALL CMOCTMAP('SQTOCODE',NULOUT,IOBTYP,ICDTYP,JOTPNEW,JCTPNEW)

          write(nulout,*)'SUALCOS: Allocate Jo Table,JPXVAR,NPRTBINS=',JPXVAR,NPRTBINS
          ALLOCATE(JOT(IOBTYP)%jost(JCTP)%cost (JPXVAR,NPRTBINS))
          ALLOCATE(JOT(IOBTYP)%jost(JCTP)%count(JPXVAR,NPRTBINS))
          ALLOCATE(JOT(IOBTYP)%jost(JCTP)%obimpact(JPXVAR,NPRTBINS))

          JOT(IOBTYP)%jost(JCTP)%sname    = CCMOCTP(ICDTYP)
          JOT(IOBTYP)%jost(JCTP)%cost(:,:)= 0.0_JPRB
          JOT(IOBTYP)%jost(JCTP)%obserr(:)= 0.0_JPRB
          JOT(IOBTYP)%jost(JCTP)%bgerr(:) = 0.0_JPRB
          JOT(IOBTYP)%jost(JCTP)%obimpact(:,:) = 0.0_JPRB
          JOT(IOBTYP)%jost(JCTP)%count(:,:) = 0
          JOT(IOBTYP)%jost(JCTP)%codetype = ICDTYP
          IF(JCTPNEW==JCTP)THEN
            JCTP=JCTP+1
          ELSE
            JCTP=JCTPNEW+1
          ENDIF
        ENDDO
      ENDIF
      IF(JOTPNEW==JOTP)THEN
        JOTP=JOTP+1
      ELSE
        JOTP=JOTPNEW+1
      ENDIF
    ENDDO
  ENDIF

ENDIF

!*          1.3   ALLOCATION OF COST FUNCTION FOR SIMULATED DATA

!   --- COST FUNCTION IN GRID-POINT SPACE -----------------------

IF(NCONF/100 == 8.OR.LAVARC.OR.LEJK) THEN
  ALLOCATE(FALTSEL(NFLEVG,NF3D))
  IF(LLP)WRITE(IU,9) 'FALTSEL  ',SIZE(FALTSEL  ),SHAPE(FALTSEL )
  ALLOCATE(FSURSEL(NFD2D))
  IF(LLP)WRITE(IU,9) 'FSURSEL  ',SIZE(FSURSEL  ),SHAPE(FSURSEL)
ENDIF

!   --- COST FUNCTION IN SPECTRAL SPACE -----------------------

IF(NCONF/100 == 8.OR.LAVARC.OR.LEJK) THEN
  IF (LELAM) THEN
    ALLOCATE(FUVB(NFLEVG,2))
    WRITE(IU,9) 'FUVB    ',SIZE(FUVB   ),SHAPE(FUVB   )
    ALLOCATE(FUVBSEL(2))
    WRITE(IU,9) 'FUVBSEL ',SIZE(FUVBSEL),SHAPE(FUVBSEL)
  ENDIF
  ALLOCATE(FSPALT(NFLEVL,NF3D,NUMP))
  IF(LLP)WRITE(IU,9) 'FSPALT   ',SIZE(FSPALT   ),SHAPE(FSPALT )
  IF (NPSP == 1) THEN
    ALLOCATE(FSPSUR(NFD2D,NUMP))
    IF(LLP)WRITE(IU,9) 'FSPSUR   ',SIZE(FSPSUR   ),SHAPE(FSPSUR )
  ENDIF
ENDIF

!*          1.4   ALLOCATION OF CONSTRAINT COST FUNCTION

IF(NCONF/100 == 1)      THEN
  ALLOCATE(FJCCOST(NF3D+NFD2D,NUMP))
  IF(LLP)WRITE(IU,9) 'FJCCOST  ',SIZE(FJCCOST  ),SHAPE(FJCCOST)
  ALLOCATE(RINCLJC(NF3D+NFD2D))
  IF(LLP)WRITE(IU,9) 'RINCLJC  ',SIZE(RINCLJC  ),SHAPE(RINCLJC)
ENDIF

IF (LLP) WRITE(NULOUT,'('' END SUALCOS PRINTOUTS '')')
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('SUALCOS',1,ZHOOK_HANDLE)
END SUBROUTINE SUALCOS
