SUBROUTINE SUALLT7

!**** *SUALLT7* - Allocate additional space for reference.

!     Purpose.
!     --------
!           Allocate space for the grid-point and spectral trajectory.

!**   Interface.
!     ----------
!        *CALL* *SUALLT7*

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------
!        Common  YOMLUN
!        Common  YOMDIM
!        Common  YOMSPT7

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-07-10
!        Modified : 95-06-06 L. Isaksen - Reordering of spectral arrays
!        Modified : 00-08-09 Y. Tremolet - Removed NTRAJE
!        Modified : 03-09-30 C. Fischer - introduce Aladin mean wind
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NSTART   ,NSTOP    ,LELAM
USE YOMDIM   , ONLY : NFLSUR   ,NSPEC2   ,NS3D     ,NS2D     ,NFLEVL
USE YOMLUN   , ONLY : NULOUT
USE YOMNMIA  , ONLY : NITNMI
USE YOMSPT7  , ONLY : SPA3T7   ,SPA2T7   ,SPA1T7

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

!*       1.    ALLOCATE SPACE FOR ARRAYS.
!              --------------------------

IF (LHOOK) CALL DR_HOOK('SUALLT7',0,ZHOOK_HANDLE)
WRITE(NULOUT,'('' SUALLT7 PRINTOUTS '')')

ALLOCATE(SPA3T7(NFLSUR,NSPEC2,NS3D,(NSTART-NITNMI):NSTOP))
ALLOCATE(SPA2T7(NSPEC2,NS2D,(NSTART-NITNMI):NSTOP))
IF(LELAM) THEN
  ALLOCATE(SPA1T7(NFLEVL,2,(NSTART-NITNMI):NSTOP))
ELSE
  ALLOCATE(SPA1T7(1,2,(NSTART-NITNMI):NSTOP))
ENDIF

WRITE(NULOUT,9990) 'SPA3T7   ',SIZE(SPA3T7   ),SHAPE(SPA3T7   )
WRITE(NULOUT,9990) 'SPA2T7   ',SIZE(SPA2T7   ),SHAPE(SPA2T7   )
WRITE(NULOUT,9990) 'SPA1T7   ',SIZE(SPA1T7   ),SHAPE(SPA1T7   )

!     ------------------------------------------------------------------
9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LHOOK) CALL DR_HOOK('SUALLT7',1,ZHOOK_HANDLE)
END SUBROUTINE SUALLT7
