SUBROUTINE SUJBBAL(CDJBTYPE)

!**** *SUJBBAL* - Prepare balance specifications for background constraint Jb

!     Purpose. Prepare the error covariance model.
!     --------

!     Interface. CALL SUJBBAL
!     ----------

!     Explicit arguments : In : CDJBTYPE = config string for Jb
!     -------------------- 

!     Implicit arguments : common YOMJG and related files.
!     --------------------

!     Method.
!     -------

!     Externals: see below.
!     ----------

!     Author : Francois Bouttier *ECMWF*
!     --------

!     Modifications :
!     ---------------
!     96-03-06 F.Bouttier : write original
!     96-01-03 J-N Thepaut (was in sugerco) : constant EPSBAL when stretching
!     96-07-18 F.Bouttier : implement John Derber's statistical balance
!     97-09-25 E.Holm     : include ozone
!     98-01-15 M.Hamrud : changed some reading for efficiency
!     97-11-13 J-N Thepaut IFLEV initialization
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     09-02-10 E.Holm     : zero ozone balance on input
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NSMAX    ,NUMP
USE YOMCT0   , ONLY : NPROC
USE YOMJG    , ONLY : STPS     ,SDIV     ,SO3      ,BFACT1   ,BFACT2, &
                    & SPJB_VARS_INFO
USE YOMGRB   , ONLY : NGRBO3
USE YOMLUN   , ONLY : NULOUT   ,NULTMP
USE YOMLAP   , ONLY : NASM0    ,MYMS
USE YOMMP    , ONLY : MYPROC
USE YOMTAG   , ONLY : MTAGFCE
USE MPL_MODULE

IMPLICIT NONE

! * INPUT:
CHARACTER(LEN=10) ,INTENT(IN)    :: CDJBTYPE 
INTEGER(KIND=JPIM),ALLOCATABLE :: IASM0(:)

REAL(KIND=JPRB),ALLOCATABLE :: ZSTPS(:,:,:),ZSDIV(:,:,:),ZSO3(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZDMAT(:,:),ZQMAT(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZQUMAT(:,:),ZO3MAT(:,:),ZOUMAT(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZBFACT1(:),ZBFACT2(:)
REAL(KIND=JPRB) :: ZWK1(NFLEVG,NFLEVG),ZWK2(NFLEVG,NFLEVG+1)
CHARACTER :: CLID*10,CLCOM*70

INTEGER(KIND=JPIM) :: ICHKWD, ICOR1, ICOR2, IDATE, IDIM1, IDIM2, ILENDEF, &
 & IM, INBMAT, INBSET, ININM, INM, INMAXF, &
 & INSM, IOMASTER, IORIG, IPAR1, IPAR2, IRI, &
 & ISETDIST, ISPECF, ITIME, ITYPDI1, ITYPDI2, &
 & ITYPMAT, IWEIGHT, JJ, JK, JM, JMLOC, JN  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "commjbbal.intfb.h"

!     -----------------------------------------------------------------

!        0. Preliminary printouts

IF (LHOOK) CALL DR_HOOK('SUJBBAL',0,ZHOOK_HANDLE)
WRITE(NULOUT,*) 'SUJBBAL: Jb balance, config=',CDJBTYPE
IF(CDJBTYPE=='STABAL96') THEN
  WRITE(NULOUT,*) ' Statistical linear balance (BALSTAT)'
ELSE
  CALL ABOR1('Unknown Jb balance config')
ENDIF
!-----------------------------------------------------------------------

!         3. Setup of the statistical balance operator (John Derber)

IOMASTER=1
IF(CDJBTYPE=='STABAL96')THEN
  WRITE(NULOUT,*) '3. Read data for statistical balance operator'

!              4.1 Determine resolution of file data
 
  IF (MYPROC==IOMASTER) THEN
    CALL GSTATS(1706,0)
    OPEN(NULTMP,FILE='stabal96.bal',FORM='unformatted')
    READ(NULTMP) CLID
    WRITE(NULOUT,*) 'GSA ID=',CLID
    IF (CDJBTYPE/=CLID) CALL ABOR1('Bad Jb ID in GSA bal file')
    READ(NULTMP) CLCOM
    WRITE(NULOUT,*) ' Description : ',CLCOM
    READ(NULTMP) IORIG,IDATE,ITIME,INBSET
    WRITE(NULOUT,*) ' Center=',IORIG,' Date=',IDATE,' Time=',ITIME
!         GSA set 1 header : horizontal balance definition
    READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
    READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ICOR1,ICOR2
    READ(NULTMP)
    READ(NULTMP)
    ISPECF=IDIM2                     ! number of horiz bal coeffs
    IF (INBMAT/=2) CALL ABOR1('Bad matrix count in GSA set 1')
    IF (ITYPMAT/=4) CALL ABOR1('Not horiz balance in GSA set 1')
!          Skip GSA set 1
    READ(NULTMP)
    READ(NULTMP)
    READ(NULTMP)
    READ(NULTMP)
!         GSA set 2 header : vertical level definition
    READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
    READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ICOR1,ICOR2
    READ(NULTMP)
    READ(NULTMP)
    IF (ITYPMAT/=5) CALL ABOR1('Not vert balance in GSA set 2')
    IF (IDIM1/=IDIM2) CALL ABOR1('Not square balance matrices')
    IF (IDIM1/=NFLEVG) CALL ABOR1('Cannot interpolate balance operators.')
    INMAXF=INBMAT-1                 ! matrices go from n=0 to itruncf
    WRITE(NULOUT,*) ' Found on GSA file : nspec2=',ISPECF,' nsmax=',INMAXF
    IF ( ISPECF/=((INMAXF+1)*(INMAXF+2)) )&
     & CALL ABOR1('Inconsistent dimensions in GSA balance file.')  
    REWIND(NULTMP)
    CALL GSTATS(1706,1)
  ENDIF
!                          Broadcast dimensions if necessary
  CALL GSTATS(637,0)
  CALL MPL_BROADCAST(INMAXF,MTAGFCE+202,IOMASTER,CDSTRING='SUJBBAL:')
  CALL GSTATS(637,1)
  ISPECF=(INMAXF+1)*(INMAXF+2)

!         4.2 Allocate input arrays

  ALLOCATE(IASM0(0:INMAXF))
  ALLOCATE(ZSDIV(NFLEVG,NFLEVG,0:INMAXF),&
   & ZSO3 (NFLEVG,NFLEVG,0:INMAXF),&
   & ZSTPS(NFLEVG+1,2*NFLEVG,0:INMAXF),&
   & ZBFACT1(ISPECF),ZBFACT2(ISPECF)  )  
  WRITE(NULOUT,9990) 'IASM0   ',SIZE(IASM0)
  WRITE(NULOUT,9990) 'ZSTPS   ',SIZE(ZSTPS),SHAPE(ZSTPS)
  WRITE(NULOUT,9990) 'ZSDIV   ',SIZE(ZSDIV),SHAPE(ZSDIV)
  WRITE(NULOUT,9990) 'ZSO3    ',SIZE(ZSO3) ,SHAPE(ZSO3)
  WRITE(NULOUT,9990) 'ZBFACT1 ',SIZE(ZBFACT1),SHAPE(ZBFACT1)
  WRITE(NULOUT,9990) 'ZBFACT2 ',SIZE(ZBFACT2),SHAPE(ZBFACT2)
  9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!           4.3 Read balance operators from GSA file

  IF (MYPROC==IOMASTER) THEN

!             GSA file header
    CALL GSTATS(1706,0)
    READ(NULTMP) CLID
    READ(NULTMP) CLCOM
    READ(NULTMP) IORIG,IDATE,ITIME,INBSET

!             GSA set 1 : header and (fact1,fact2) coefficients
    WRITE(NULOUT,*) ' reading GSA set 1'
    READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
    READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ITYPDI1,ITYPDI2
    READ(NULTMP)
    READ(NULTMP)
    IF (ITYPMAT/=4) CALL ABOR1('Not horiz balance in GSA set 1')
    IF (IDIM2/=ISPECF) CALL ABOR1('Bad resolution in GSA set 1')
    READ(NULTMP)
    READ(NULTMP) (ZBFACT1(JJ),JJ=1,ISPECF),ICHKWD
    IF (ICHKWD/=3141592) CALL ABOR1('Bad GSA control word')
    READ(NULTMP)
    READ(NULTMP) (ZBFACT2(JJ),JJ=1,ISPECF),ICHKWD
    IF (ICHKWD/=3141592) CALL ABOR1('Bad GSA control word')

!             GSA set 2 : SDIV matrices
    WRITE(NULOUT,*) ' reading GSA set 2'
    READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
    READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ITYPDI1,ITYPDI2
    READ(NULTMP)
    READ(NULTMP)
    IF(ITYPMAT/=5) CALL ABOR1('Not vert balance in GSA set 2')
    IF((IDIM1/=NFLEVG).OR.(IDIM2/=NFLEVG))&
     & CALL ABOR1('Bad vertical resolution in GSA set 2')  
    IF((IPAR1/=11).OR.(IPAR2/=15))&
     & CALL ABOR1('Not Pb->divb operator in GSA set 2')  
    DO JN=0,INMAXF
      READ(NULTMP)
!            READ(NULTMP) ((ZSDIV(JJ,JK,JN),JK=1,NFLEVG),JJ=1,NFLEVG),
!     &         ICHKWD
      READ(NULTMP) ZWK1(:,:),ICHKWD
      IF(ICHKWD /= 3141592) CALL ABOR1('Bad GSA control word')
      DO JK=1,NFLEVG
        DO JJ=1,NFLEVG
          ZSDIV(JJ,JK,JN)=ZWK1(JK,JJ)
        ENDDO
      ENDDO
    ENDDO

!             GSA set 3 : STPS/T matrices
    WRITE(NULOUT,*) ' reading GSA set 3'
    READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
    READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ITYPDI1,ITYPDI2
    READ(NULTMP)
    READ(NULTMP)
    IF((IPAR1/=13).OR.(IPAR2/=15))&
     & CALL ABOR1('Not Pb->Tpsb operator in GSA set 3')  
    DO JN=0,INMAXF
      READ(NULTMP)
!            READ(NULTMP) ((ZSTPS(JJ,JK,JN),JK=1,NFLEVG),JJ=1,NFLEVG+1),
!     &       ICHKWD
      READ(NULTMP) ZWK2(:,:),ICHKWD
      IF(ICHKWD /= 3141592) CALL ABOR1('Bad GSA control word')
      DO JK=1,NFLEVG
        DO JJ=1,NFLEVG+1
          ZSTPS(JJ,JK,JN)=ZWK2(JK,JJ)
        ENDDO
      ENDDO
    ENDDO

!             GSA set 4 : STPS/div matrices
    WRITE(NULOUT,*) ' reading GSA set 4'
    READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
    READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ITYPDI1,ITYPDI2
    READ(NULTMP)
    READ(NULTMP)
    IF((IPAR1/=13).OR.(IPAR2/=12))&
     & CALL ABOR1('Not divu->Tpsb operator in GSA set 4')  
    DO JN=0,INMAXF
      READ(NULTMP)
!            READ(NULTMP) ((ZSTPS(JJ,NFLEVG+JK,JN)
!     &                    ,JK=1,NFLEVG),JJ=1,NFLEVG+1),ICHKWD
      READ(NULTMP) ZWK2(:,:),ICHKWD
      IF(ICHKWD /= 3141592) CALL ABOR1('Bad GSA control word')
      DO JK=1,NFLEVG
        DO JJ=1,NFLEVG+1
          ZSTPS(JJ,NFLEVG+JK,JN)=ZWK2(JK,JJ)
        ENDDO
      ENDDO
    ENDDO

!             GSA set 5 : SO3 matrices
    IF(ANY(SPJB_VARS_INFO(:)%IGRIBCODE==NGRBO3))THEN
      WRITE(NULOUT,*) ' reading GSA set 5'
      READ(NULTMP) INBMAT,IWEIGHT,ITYPMAT,ISETDIST,ILENDEF
      READ(NULTMP) IDIM1,IDIM2,IPAR1,IPAR2,ITYPDI1,ITYPDI2
      READ(NULTMP)
      READ(NULTMP)
      IF(ITYPMAT/=5) CALL ABOR1('Not vert balance in GSA set 5')
      IF((IDIM1/=NFLEVG).OR.(IDIM2/=NFLEVG))&
       & CALL ABOR1('Bad vertical resolution in GSA set 5')  
      IF((IPAR1/=17).OR.(IPAR2/=15))&
       & CALL ABOR1('Not Pb->O3b operator in GSA set 5')  
      DO JN=0,INMAXF
        READ(NULTMP)
!              READ(NULTMP) ((ZSO3(JJ,JK,JN),JK=1,NFLEVG),JJ=1,NFLEVG),
!     &           ICHKWD
        READ(NULTMP) ZWK1(:,:),ICHKWD
        IF(ICHKWD /= 3141592) CALL ABOR1('Bad GSA control word')
        DO JK=1,NFLEVG
          DO JJ=1,NFLEVG
            ZSO3(JJ,JK,JN)=0.0_JPRB*ZWK1(JK,JJ)
          ENDDO
        ENDDO
      ENDDO
    ENDIF

    CLOSE(NULTMP)
    CALL GSTATS(1706,1)
  ENDIF

!                 4.3 Distribute to other processors

  IF (NPROC>1) CALL COMMJBBAL(NFLEVG,INMAXF,ISPECF,&
   & ZBFACT1,ZBFACT2,ZSDIV,ZSTPS,ZSO3)  

!                 4.4 Convert to model truncation

  IF (NSMAX>INMAXF) CALL ABOR1(' Stat balance is only up to T106')
  IF (NSMAX<INMAXF) WRITE(NULOUT,*) 'Truncating balance'
  SDIV(:,:,:)=ZSDIV(:,:,0:NSMAX)
  STPS(:,:,:)=ZSTPS(:,:,0:NSMAX)
  IF(ANY(SPJB_VARS_INFO(:)%IGRIBCODE==NGRBO3)) SO3 (:,:,:)=ZSO3 (:,:,0:NSMAX)
  INSM=1
  DO JM=0,INMAXF
    IASM0(JM)=INSM
    INSM=INSM+(INMAXF+1-JM)*2
  ENDDO
  DO JMLOC=1,NUMP
    JM=MYMS(JMLOC)
    DO IRI=0,MIN(1,JM)
      DO IM=JM,NSMAX
        INM=NASM0(JM)+(IM-JM)*2+IRI
        ININM=IASM0(JM)+(IM-JM)*2+IRI
        BFACT1(INM)=ZBFACT1(ININM)
        BFACT2(INM)=ZBFACT2(ININM)
      ENDDO
    ENDDO
  ENDDO
ENDIF

IF (ALLOCATED(IASM0)) DEALLOCATE(IASM0)
IF (ALLOCATED(ZSTPS)) DEALLOCATE(ZSTPS)
IF (ALLOCATED(ZSDIV)) DEALLOCATE(ZSDIV)
IF (ALLOCATED(ZSO3)) DEALLOCATE(ZSO3)
IF (ALLOCATED(ZDMAT)) DEALLOCATE(ZDMAT)
IF (ALLOCATED(ZQMAT)) DEALLOCATE(ZQMAT)
IF (ALLOCATED(ZQUMAT)) DEALLOCATE(ZQUMAT)
IF (ALLOCATED(ZO3MAT)) DEALLOCATE(ZO3MAT)
IF (ALLOCATED(ZOUMAT)) DEALLOCATE(ZOUMAT)
IF (ALLOCATED(ZBFACT1)) DEALLOCATE(ZBFACT1)
IF (ALLOCATED(ZBFACT2)) DEALLOCATE(ZBFACT2)

IF (LHOOK) CALL DR_HOOK('SUJBBAL',1,ZHOOK_HANDLE)
END SUBROUTINE SUJBBAL
