SUBROUTINE SUJBCHVAR

!**** *SUJBCHVAR* - Prepare coefficients of the change of
!                   variable for the background constraint Jb

!     Purpose. Prepare the error covariance model.
!     --------

!     Interface. CALL SUJBCHVAR
!     ----------

!     Implicit arguments : common YOMJG and related files.
!     --------------------

!     Method.
!     -------

!     Externals: see below.
!     ----------

!     Author : Elias Holm *ECMWF*
!     --------

!     Modifications :
!     ---------------
!     2003-12-04 E.Holm : Write original. Here the standard deviation
!                         and the q-T correlation coefficient needed
!                         for the humidity change of variable are 
!                         interpolated linearly from a 60-level coefficient 
!                         set, using constant extrapolation above and
!                         below.
!     2004-01-23 E.Holm : Add ozone coefficients, first version.
!     2007-12-12 E.Holm   Introduce LJB_SUPSAT for humidity
!                         control variable setting including supersaturation
!                         and q-T balance (new 91-L ensemble based coeffs)
!                         Change temperature balance coefficient RQT to
!                         account for tropopause transition and introduce
!                         RQS for tropopause transition of humidity sigmas.`
!     ------------------------------------------------------------------

USE PARKIND1   ,ONLY  : JPIM     ,JPRB
USE YOMHOOK    ,ONLY  : LHOOK,   DR_HOOK
 
USE YOMDIM     , ONLY  : NFLEVG
USE YOMLUN     , ONLY  : NULOUT
USE YOMGEM     , ONLY  : VAH, VBH, VP00
USE YOMJG      , ONLY  : LJB_SUPSAT
USE YOMJBCHVAR , ONLY : RQS, RQT, RQTBAL, RQLMIN, RQLMAX, RQLSTD,&
 & RQHMIN, RQHMAX, RQHSTD, RO3MIN, RO3STD

IMPLICIT NONE

INTEGER(KIND=JPIM), PARAMETER :: ICOEFLEV60=60
INTEGER(KIND=JPIM), PARAMETER :: ICOEFLEV91=91
INTEGER(KIND=JPIM) :: ICOEFLEV, JLEVOUT, JLEVIN, INDX(1:NFLEVG)

REAL(KIND=JPRB) :: ZPRH(1:NFLEVG), ZFRAC(1:NFLEVG)

REAL(KIND=JPRB) :: ZQS(0:6), ZQT(0:6)

REAL(KIND=JPRB) :: ZAH60(0:ICOEFLEV60), ZBH60(0:ICOEFLEV60)
REAL(KIND=JPRB) :: ZQTBAL60 (0:ICOEFLEV60+1,0:5)
REAL(KIND=JPRB) :: ZQLMIN60 (0:ICOEFLEV60+1), ZQLMAX60 (0:ICOEFLEV60+1)
REAL(KIND=JPRB) :: ZQLSTD60 (0:ICOEFLEV60+1,0:5)
REAL(KIND=JPRB) :: ZO3MIN60 (0:ICOEFLEV60+1), ZO3STD60 (0:ICOEFLEV60+1,0:5)

REAL(KIND=JPRB) :: ZAH91(0:ICOEFLEV91), ZBH91(0:ICOEFLEV91)
REAL(KIND=JPRB) :: ZQTBAL91 (0:ICOEFLEV91+1,0:5)
REAL(KIND=JPRB) :: ZQLMIN91(0:ICOEFLEV91+1), ZQLMAX91(0:ICOEFLEV91+1)
REAL(KIND=JPRB) :: ZQLSTD91(0:ICOEFLEV91+1,0:5)
REAL(KIND=JPRB) :: ZQHMIN91(0:ICOEFLEV91+1), ZQHMAX91(0:ICOEFLEV91+1)
REAL(KIND=JPRB) :: ZQHSTD91(0:ICOEFLEV91+1,0:5)
REAL(KIND=JPRB) :: ZO3MIN91 (0:ICOEFLEV91+1), ZO3STD91 (0:ICOEFLEV91+1,0:5)

REAL(KIND=JPRB), ALLOCATABLE :: ZAH(:), ZBH(:), ZPRHIN(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZQTBAL(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZQLMIN(:), ZQLMAX (:)
REAL(KIND=JPRB), ALLOCATABLE :: ZQLSTD(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZQHMIN(:), ZQHMAX(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZQHSTD(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZO3MIN(:), ZO3STD (:,:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUJBCHVAR',0,ZHOOK_HANDLE)

!--------------------------------------------------------------------
! 0.1. Tropopause transition coefficients
!--------------------------------------------------------------------
! Set DATA ZQS/7*1.0/ for stratospheric humidity analysis
DATA ZQS/1.E-9, 0.1, 0.2, 0.5, 0.8, 0.9, 1.0/
DATA ZQT/0., 0.1, 0.2, 0.5, 0.8, 0.9, 1.0/
!--------------------------------------------------------------------
! 0.2. Normalized relative humidity (60-level geometry)
!--------------------------------------------------------------------
!    0.2.1 Vertical geometry of input statistics
!    ----------------------------------------------------------------
DATA ZAH60/                                                0.000000,&
 &    20.000000,   38.425343,   63.647804,   95.636963,  134.483307,&
 &   180.584351,  234.779053,  298.495789,  373.971924,  464.618134,&
 &   575.651001,  713.218079,  883.660522, 1094.834717, 1356.474609,&
 &  1680.640259, 2082.273926, 2579.888672, 3196.421631, 3960.291504,&
 &  4906.708496, 6018.019531, 7306.631348, 8765.053711,10376.126953,&
 & 12077.446289,13775.325195,15379.805664,16819.474609,18045.183594,&
 & 19027.695313,19755.109375,20222.205078,20429.863281,20384.480469,&
 & 20097.402344,19584.330078,18864.750000,17961.357422,16899.468750,&
 & 15706.447266,14411.124023,13043.218750,11632.758789,10209.500977,&
 &  8802.356445, 7438.803223, 6144.314941, 4941.778320, 3850.913330,&
 &  2887.696533, 2063.779785, 1385.912598,  855.361755,  467.333588,&
 &   210.393890,   65.889244,    7.367743,    0.000000,    0.000000/
DATA ZBH60/                                            0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000758235,0.0004613950,&
 & 0.0018151561,0.0050811190,0.0111429105,0.0206778757,0.0341211632,&
 & 0.0516904071,0.0735338330,0.0996746942,0.1300225109,0.1643843204,&
 & 0.2024759352,0.2439331412,0.2883229554,0.3351548910,0.3838921487,&
 & 0.4339629412,0.4847715795,0.5357099175,0.5861684084,0.6355474591,&
 & 0.6832686067,0.7287858129,0.7715966105,0.8112534285,0.8473749161,&
 & 0.8796569109,0.9078838825,0.9319403172,0.9518215060,0.9676452279,&
 & 0.9796627164,0.9882701039,0.9940194488,0.9976301193,1.0000000000/
!    ----------------------------------------------------------------
!    0.2.2 RH humidity-temperature coupling (=1 for RH)
!    ----------------------------------------------------------------
DATA ZQTBAL60(:,0)/62*1.0_JPRB/
DATA ZQTBAL60(:,1)/62*0.0_JPRB/
DATA ZQTBAL60(:,2)/62*0.0_JPRB/
DATA ZQTBAL60(:,3)/62*0.0_JPRB/
DATA ZQTBAL60(:,4)/62*0.0_JPRB/
DATA ZQTBAL60(:,5)/62*0.0_JPRB/
!    ----------------------------------------------------------------
!    0.2.3 RH normalization stdev, min, max
!    ----------------------------------------------------------------
DATA ZQLMIN60/                                  3.9212E-07,&
 &  3.9212E-07,3.1759E-07,3.2222E-07,3.8335E-07,5.6253E-07,&
 &  1.0053E-06,2.0580E-06,4.7136E-06,9.6937E-06,1.5433E-05,&
 &  2.1735E-05,2.9870E-05,3.9825E-05,5.7579E-05,7.5505E-05,&
 &  9.1923E-05,1.1104E-04,1.4026E-04,1.8781E-04,2.4557E-04,&
 &  3.1431E-04,4.0916E-04,5.3904E-04,6.9267E-04,9.8364E-04,&
 &  1.2672E-03,1.5105E-03,1.8801E-03,2.0491E-03,2.1910E-03,&
 &  2.2661E-03,2.0340E-03,2.2823E-03,2.9142E-03,5.0343E-03,&
 &  6.5193E-03,6.2364E-03,6.8736E-03,8.3267E-03,8.5825E-03,&
 &  9.4101E-03,1.0649E-02,1.2381E-02,1.2514E-02,1.4601E-02,&
 &  2.2042E-02,2.9563E-02,4.1602E-02,7.0291E-02,8.4940E-02,&
 &  0.10353   ,0.12355   ,0.12859   ,0.11955   ,0.10573,&
 &  9.6877E-02,9.5986E-02,9.9630E-02,0.10184   ,0.10042,&
 &  0.10042/
DATA ZQLMAX60/62*1.0_JPRB/
DATA ZQLSTD60(:,0)/62*0.0_JPRB/
DATA ZQLSTD60(:,1)/                             0.093108,&
 &  0.093108,  0.066039,  0.082832,  0.101666,  0.097314,&
 &  0.089727,  0.084153,  0.077305,  0.067972,  0.060369,&
 &  0.053035,  0.046499,  0.043645,  0.041177,  0.039750,&
 &  0.038560,  0.037691,  0.036608,  0.035853,  0.037192,&
 &  0.043047,  0.056415,  0.079837,  0.100827,  0.131979,&
 &  0.153796,  0.166886,  0.171332,  0.180675,  0.190080,&
 &  0.197265,  0.202434,  0.210588,  0.225344,  0.227416,&
 &  0.233656,  0.237283,  0.240659,  0.242947,  0.242743,&
 &  0.235225,  0.224957,  0.214648,  0.202170,  0.186833,&
 &  0.172128,  0.160543,  0.145156,  0.132270,  0.121373,&
 &  0.109299,  0.091750,  0.071981,  0.058035,  0.047262,&
 &  0.039424,  0.028500,  0.030258,  0.041486,  0.071113,&
 &  0.071113/
DATA ZQLSTD60(:,2)/                            -0.076952,&
 & -0.076952, -0.322488, -0.553463, -0.668386, -0.569530,&
 & -0.421979, -0.287929, -0.167879, -0.080019, -0.014685,&
 &  0.018358,  0.029218,  0.016141,  0.003976, -0.007530,&
 & -0.016836, -0.021894, -0.023130, -0.026374, -0.039516,&
 & -0.078319, -0.131552, -0.246937, -0.395168, -0.583242,&
 & -0.705326, -0.827205, -0.898073, -0.920236, -0.931204,&
 & -0.937101, -0.946264, -0.939039, -0.938990, -0.855425,&
 & -0.809658, -0.763859, -0.710597, -0.652462, -0.581867,&
 & -0.479024, -0.367996, -0.252548, -0.125389, -0.029402,&
 &  0.001543, -0.022201, -0.069922, -0.140500, -0.209611,&
 & -0.201129, -0.094014,  0.058235,  0.175650,  0.265650,&
 &  0.326462,  0.401328,  0.386794,  0.289533,  0.035672,&
 &  0.035672/
DATA ZQLSTD60(:,3)/                             0.209236,&
 &  0.209236,  1.466958,  2.180061,  2.390545,  1.993113,&
 &  1.437822,  0.916919,  0.468413,  0.186623, -0.029623,&
 & -0.109755, -0.103082, -0.044327,  0.009294,  0.049473,&
 &  0.078637,  0.087949,  0.084256,  0.086169,  0.104839,&
 &  0.187001,  0.341612,  0.671741,  1.005490,  1.438718,&
 &  1.932320,  2.577737,  3.101459,  3.296844,  3.380995,&
 &  3.411815,  3.434324,  3.301300,  3.084531,  2.695908,&
 &  2.437070,  2.214350,  1.935466,  1.631683,  1.299400,&
 &  0.926214,  0.525174,  0.070104, -0.403960, -0.639166,&
 & -0.506962, -0.124761,  0.340188,  0.688376,  0.878450,&
 &  0.743003,  0.317913, -0.203348, -0.598975, -0.890040,&
 & -1.086541, -1.291614, -1.272339, -1.005224, -0.289456,&
 & -0.289456/
DATA ZQLSTD60(:,4)/                            -0.230898,&
 & -0.230898, -2.314230, -3.213177, -3.313629, -2.725058,&
 & -1.929726, -1.167370, -0.532405, -0.165733,  0.112960,&
 &  0.190295,  0.142793,  0.049507, -0.034392, -0.089621,&
 & -0.124492, -0.130589, -0.118632, -0.111804, -0.111164,&
 & -0.172860, -0.381511, -0.726791, -0.870848, -1.227874,&
 & -1.965287, -2.976222, -3.877285, -4.260223, -4.452627,&
 & -4.548254, -4.611901, -4.401036, -3.998429, -3.449380,&
 & -3.074385, -2.770642, -2.367280, -1.922199, -1.448455,&
 & -0.971629, -0.458917,  0.154877,  0.783579,  1.011930,&
 &  0.662536, -0.049538, -0.836119, -1.296979, -1.432511,&
 & -1.127711, -0.512959,  0.158792,  0.649207,  0.995819,&
 &  1.230590,  1.452496,  1.442358,  1.136417,  0.312710,&
 &  0.312710/
DATA ZQLSTD60(:,5)/                             0.089298,&
 &  0.089298,  1.183446,  1.579468,  1.561635,  1.272262,&
 &  0.888726,  0.515481,  0.212726,  0.046443, -0.076408,&
 & -0.101816, -0.067655, -0.019413,  0.023381,  0.049328,&
 &  0.063566,  0.064382,  0.056612,  0.050127,  0.040973,&
 &  0.051923,  0.144430,  0.250297,  0.186763,  0.266569,&
 &  0.609902,  1.083621,  1.526935,  1.726970,  1.836526,&
 &  1.899824,  1.944737,  1.851266,  1.650306,  1.403839,&
 &  1.235168,  1.104101,  0.922263,  0.719722,  0.506974,&
 &  0.307054,  0.093633, -0.171229, -0.441535, -0.516286,&
 & -0.316247,  0.048100,  0.432050,  0.627463,  0.652273,&
 &  0.485921,  0.206163, -0.077281, -0.275962, -0.411119,&
 & -0.502710, -0.583802, -0.580460, -0.455884, -0.123986,&
 & -0.123986/
!    ----------------------------------------------------------------
!    0.2.6 O3 normalization stdev, min
!    ----------------------------------------------------------------
DATA ZO3MIN60/                             9.33E-10,&
 & 9.33E-10, 2.14E-09, 3.26E-09, 4.29E-09, 5.63E-09,&
 & 7.17E-09, 8.85E-09, 1.04E-08, 1.17E-08, 1.26E-08,&
 & 1.32E-08, 1.34E-08, 1.32E-08, 1.27E-08, 1.19E-08,&
 & 1.08E-08, 9.62E-09, 8.31E-09, 6.97E-09, 5.67E-09,&
 & 4.32E-09, 3.06E-09, 2.09E-09, 1.40E-09, 9.30E-10,&
 & 6.58E-10, 4.88E-10, 3.70E-10, 2.96E-10, 2.41E-10,&
 & 1.99E-10, 1.64E-10, 1.37E-10, 1.19E-10, 1.07E-10,&
 & 9.96E-11, 9.36E-11, 8.89E-11, 8.51E-11, 8.20E-11,&
 & 7.93E-11, 7.72E-11, 7.55E-11, 7.43E-11, 7.34E-11,&
 & 7.29E-11, 7.25E-11, 7.23E-11, 7.21E-11, 7.20E-11,&
 & 7.17E-11, 7.14E-11, 7.09E-11, 7.03E-11, 6.95E-11,&
 & 6.86E-11, 6.77E-11, 6.67E-11, 6.58E-11, 6.48E-11,&
 & 6.48E-11/
DATA ZO3STD60(:,0)/62*1./
DATA ZO3STD60(:,1)/62*0./
DATA ZO3STD60(:,2)/62*0./
DATA ZO3STD60(:,3)/62*0./
DATA ZO3STD60(:,4)/62*0./
!--------------------------------------------------------------------
! 0.3. Normalized q/q_s(T^b) humidity (91-level geometry)
!--------------------------------------------------------------------
!    0.3.1 Vertical geometry of input statistics
!    ----------------------------------------------------------------
DATA ZAH91/                                                0.000000,&
 &     2.000040,    3.980832,    7.387186,   12.908319,   21.413612,&
 &    33.952858,   51.746601,   76.167656,  108.715561,  150.986023,&
 &   204.637451,  271.356506,  352.824493,  450.685791,  566.519226,&
 &   701.813354,  857.945801, 1036.166504, 1237.585449, 1463.163940,&
 &  1713.709595, 1989.874390, 2292.155518, 2620.898438, 2976.302246,&
 &  3358.425781, 3767.196045, 4202.416504, 4663.776367, 5150.859863,&
 &  5663.156250, 6199.839355, 6759.727051, 7341.469727, 7942.926270,&
 &  8564.624023, 9208.305664, 9873.560547,10558.881836,11262.484375,&
 & 11982.662109,12713.897461,13453.225586,14192.009766,14922.685547,&
 & 15638.053711,16329.560547,16990.623047,17613.281250,18191.029297,&
 & 18716.968750,19184.544922,19587.513672,19919.796875,20175.394531,&
 & 20348.916016,20434.158203,20426.218750,20319.011719,20107.031250,&
 & 19785.357422,19348.775391,18798.822266,18141.296875,17385.595703,&
 & 16544.585938,15633.566406,14665.645508,13653.219727,12608.383789,&
 & 11543.166992,10471.310547, 9405.222656, 8356.252930, 7335.164551,&
 &  6353.920898, 5422.802734, 4550.215820, 3743.464355, 3010.146973,&
 &  2356.202637, 1784.854614, 1297.656128,  895.193542,  576.314148,&
 &   336.772369,  162.043427,   54.208336,    6.575628,    0.003160,&
 &     0.000000/
DATA ZBH91/                                            0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000000000,0.0000000000,&
 & 0.0000000000,0.0000000000,0.0000000000,0.0000002724,0.0000139116,&
 & 0.0000546672,0.0001313641,0.0002788848,0.0005483841,0.0010001344,&
 & 0.0017010754,0.0027647193,0.0042670486,0.0063221669,0.0090349903,&
 & 0.0125082629,0.0168595780,0.0221886449,0.0286103487,0.0362269096,&
 & 0.0451461338,0.0554742292,0.0673161894,0.0807773024,0.0959640592,&
 & 0.1129790097,0.1319348216,0.1529335231,0.1760910898,0.2015201300,&
 & 0.2293148786,0.2595544755,0.2919934094,0.3263293803,0.3622025847,&
 & 0.3992047608,0.4369063377,0.4750164151,0.5132797360,0.5514584780,&
 & 0.5893171430,0.6265588999,0.6629335880,0.6982235909,0.7322238088,&
 & 0.7646794915,0.7953847647,0.8241854310,0.8509504199,0.8755183816,&
 & 0.8977672458,0.9176509380,0.9351570606,0.9502738118,0.9630070925,&
 & 0.9734660387,0.9822381139,0.9891529679,0.9942041636,0.9976301193,&
 & 1.0000000000/
!    ----------------------------------------------------------------
!    0.3.2 q/q_s(T^b) humidity-temperature coupling
!    ----------------------------------------------------------------
DATA ZQTBAL91(:,0)/93*0.000000/
DATA ZQTBAL91(:,1)/&
 &               38*0.000000,   -0.668375,   -1.282876,   -1.870164,&
 &    -2.453762,   -3.054169,   -3.688984,   -4.373017,   -5.118414,&
 &    -5.934768,   -6.829240,   -7.806678,   -8.869730,  -10.018969,&
 &   -11.253004,  -12.568600,  -13.960798,  -15.423028,  -16.947233,&
 &   -18.523981,  -20.142586,  -21.791225,  -23.457054,  -25.126330,&
 &   -26.784523,  -28.416439,  -30.006335,  -31.538037,  -32.995058,&
 &   -34.360716,  -35.618252,  -36.750947,  -37.742238,  -38.575842,&
 &   -39.235865,  -39.706928,  -39.974278,  -40.023911,  -39.842685,&
 &   -39.418443,  -38.740127,  -37.797896,  -36.583244,  -35.089120,&
 &   -33.310043,  -31.242221,  -28.883668,  -26.234322,  -23.296163,&
 &   -20.073332,  -16.572246,  -12.801717,   -8.773071,   -4.500264,&
 &     0.000000,    0.000000/
DATA ZQTBAL91(:,2)/&
 &               38*0.000000,    3.537106,    6.593120,    9.316330,&
 &    11.839140,   14.278655,   16.737261,   19.303208,   22.051195,&
 &    25.042948,   28.327808,   31.943310,   35.915766,   40.260848,&
 &    44.984171,   50.081874,   55.541205,   61.341101,   67.452773,&
 &    73.840286,   80.461144,   87.266871,   94.203594,  101.212624,&
 &   108.231043,  115.192282,  122.026705,  128.662193,  135.024723,&
 &   141.038956,  146.628815,  151.718068,  156.230912,  160.092557,&
 &   163.229805,  165.571633,  167.049781,  167.599325,  167.159270,&
 &   165.673123,  163.089483,  159.362621,  154.453059,  148.328158,&
 &   140.962698,  132.339462,  122.449814,  111.294288,   98.883167,&
 &    85.237065,   70.387512,   54.377533,   37.262235,   19.109386,&
 &     0.000000,    0.000000/
DATA ZQTBAL91(:,3)/&
 &               38*0.000000,   -7.693179,  -14.285244,  -20.106764,&
 &   -25.451108,  -30.575947,  -35.704748,  -41.028277,  -46.706100,&
 &   -52.868079,  -59.615875,  -67.024447,  -75.143550,  -83.999236,&
 &   -93.595355, -103.915051, -114.922266, -126.563238, -138.767999,&
 &  -151.575421, -164.828839, -178.326079, -191.948855, -205.570233,&
 &  -219.056123, -232.266766, -245.058229, -257.283892, -268.795937,&
 &  -279.446843, -289.090873, -297.585566, -304.793226, -310.582412,&
 &  -314.829431, -317.419826, -318.249867, -317.228042, -314.276546,&
 &  -309.332773, -302.350804, -293.302901, -282.180994, -268.998173,&
 &  -253.790176, -236.616886, -217.563812, -196.743586, -174.297452,&
 &  -150.396755, -125.244432,  -99.076503,  -72.163560,  -44.812261,&
 &   -17.366813,  -17.366813/
DATA ZQTBAL91(:,4)/&
 &               38*0.000000,    6.728332,   12.628449,   18.014795,&
 &    23.162091,   28.307114,   33.650475,   39.358396,   45.564486,&
 &    52.371520,   59.853216,   68.056013,   77.000845,   86.684924,&
 &    97.083512,  108.151703,  119.826195,  132.027073,  144.659582,&
 &   157.801223,  171.244717,  184.722571,  198.100705,  211.239174,&
 &   223.993935,  236.218606,  247.766230,  258.491043,  268.250230,&
 &   276.905693,  284.325816,  290.387222,  294.976543,  297.992180,&
 &   299.346065,  298.965428,  296.794558,  292.796567,  286.955152,&
 &   279.276362,  269.790356,  258.553171,  245.648484,  231.189372,&
 &   215.320082,  198.217788,  180.094358,  161.198116,  141.815606,&
 &   122.273354,  102.939633,   84.226225,   66.590186,   50.535608,&
 &    36.615382,   36.615382/
DATA ZQTBAL91(:,5)/&
 &               38*0.000000,   -1.818214,   -3.485398,   -5.107188,&
 &    -6.773934,   -8.561440,  -10.531707,  -12.733678,  -15.203982,&
 &   -17.967675,  -21.038985,  -24.422056,  -28.111690,  -32.094091,&
 &   -36.347607,  -40.843476,  -45.546569,  -50.416130,  -55.406526,&
 &   -60.542108,  -65.734436,  -70.872138,  -75.898389,  -80.755235,&
 &   -85.384333,  -89.727683,  -93.728371,  -97.331307, -100.483958,&
 &  -103.137091, -105.245505, -106.768777, -107.671992, -107.926483,&
 &  -107.510574, -106.410308, -104.620194, -102.143939,  -98.995191,&
 &   -95.198269,  -90.788908,  -85.814995,  -80.337304,  -74.430237,&
 &   -68.182560,  -61.698143,  -55.096693,  -48.514497,  -42.105158,&
 &   -36.040332,  -30.510467,  -25.725538,  -21.915790,  -19.332470,&
 &   -18.248569,  -18.248569/
!    ----------------------------------------------------------------
!    0.3.3 q/q_s(T^b) normalization stdev, min, max (rh^b<1.x)
!    ----------------------------------------------------------------
DATA ZQLMIN91/93*0.0001_JPRB/
DATA ZQLMAX91/                                             0.815000,&
 &     0.815000,    0.955000,    1.105000,    1.175000,    1.165000,&
 &     1.135000,    1.105000,    1.095000,    1.065000,    1.045000,&
 &     1.035000,    1.015000,    0.995000,    0.975000,    0.955000,&
 &     0.955000,    0.945000,    0.945000,    0.935000,    0.915000,&
 &     0.915000,    0.895000,    0.875000,    0.845000,    0.815000,&
 &     0.795000,    0.775000,    0.745000,    0.705000,    0.665000,&
 &     0.615000,    0.645000,    0.725000,    0.845000,    0.975000,&
 &     1.132000,    1.074021,    1.078088,    1.081855,    1.083594,&
 &     1.083724,    1.082586,    1.080464,    1.077602,    1.074208,&
 &     1.070459,    1.066504,    1.062464,    1.058434,    1.054486,&
 &     1.050673,    1.047032,    1.043585,    1.040347,    1.037324,&
 &     1.034516,    1.031923,    1.029538,    1.027357,    1.025374,&
 &     1.023581,    1.021974,    1.020548,    1.019298,    1.018221,&
 &     1.017315,    1.016578,    1.016010,    1.015610,    1.015378,&
 &     1.015316,    1.015421,    1.015692,    1.016125,    1.016710,&
 &     1.017434,    1.018276,    1.019204,    1.020178,    1.021147,&
 &     1.022049,    1.022814,    1.023371,    1.023653,    1.023606,&
 &     1.023197,    1.022420,    1.021298,    1.019886,    1.018258,&
 &     1.016954,    1.016954/
DATA ZQLSTD91(:,0)/93*0.0_JPRB/
DATA ZQLSTD91(:,1)/                                        0.034618,&
 &     0.034618,    0.031840,    0.028331,    0.025110,    0.022171,&
 &     0.019505,    0.017105,    0.014964,    0.013074,    0.011428,&
 &     0.010019,    0.008839,    0.007880,    0.007136,    0.006599,&
 &     0.006261,    0.006116,    0.006155,    0.006372,    0.006758,&
 &     0.007308,    0.008012,    0.008864,    0.009856,    0.010981,&
 &     0.012232,    0.013601,    0.015080,    0.016663,    0.018342,&
 &     0.020109,    0.021958,    0.023880,    0.025869,    0.027916,&
 &     0.029479,    0.053357,    0.053809,    0.054970,    0.057044,&
 &     0.060135,    0.064320,    0.069646,    0.076137,    0.083792,&
 &     0.092587,    0.102479,    0.113404,    0.125284,    0.138021,&
 &     0.151507,    0.165618,    0.180221,    0.195174,    0.210326,&
 &     0.225522,    0.240601,    0.255401,    0.269756,    0.283505,&
 &     0.296484,    0.308537,    0.319512,    0.329263,    0.337655,&
 &     0.344561,    0.349868,    0.353476,    0.355298,    0.355267,&
 &     0.353334,    0.349468,    0.343663,    0.335932,    0.326317,&
 &     0.314886,    0.301733,    0.286984,    0.270796,    0.253359,&
 &     0.234897,    0.215673,    0.195985,    0.176173,    0.156616,&
 &     0.137738,    0.120008,    0.103938,    0.090091,    0.079079,&
 &     0.072437,    0.072437/
DATA ZQLSTD91(:,2)/&
 &  37*0.000000,   -0.000015,   -0.000142,   -0.000610,   -0.001698,&
 &    -0.003657,   -0.006698,   -0.010991,   -0.016669,   -0.023831,&
 &    -0.032538,   -0.042823,   -0.054685,   -0.068099,   -0.083009,&
 &    -0.099337,   -0.116982,   -0.135822,   -0.155715,   -0.176503,&
 &    -0.198012,   -0.220055,   -0.242433,   -0.264938,   -0.287353,&
 &    -0.309457,   -0.331022,   -0.351820,   -0.371623,   -0.390202,&
 &    -0.407334,   -0.422799,   -0.436386,   -0.447892,   -0.457125,&
 &    -0.463906,   -0.468069,   -0.469466,   -0.467968,   -0.463465,&
 &    -0.455867,   -0.445113,   -0.431162,   -0.414005,   -0.393660,&
 &    -0.370178,   -0.343642,   -0.314171,   -0.281920,   -0.247083,&
 &    -0.209895,   -0.170634,   -0.129621,   -0.087226,   -0.043864,&
 &    -0.010998,   -0.010998/
DATA ZQLSTD91(:,3)/&
 &  37*0.000000,   -0.559597,   -0.490451,   -0.387156,   -0.269905,&
 &    -0.142764,   -0.009396,    0.126923,    0.263288,    0.397146,&
 &     0.526282,    0.648803,    0.763120,    0.867932,    0.962210,&
 &     1.045181,    1.116310,    1.175285,    1.222003,    1.256546,&
 &     1.279174,    1.290301,    1.290484,    1.280403,    1.260846,&
 &     1.232694,    1.196901,    1.154482,    1.106493,    1.054017,&
 &     0.998145,    0.939963,    0.880533,    0.820878,    0.761965,&
 &     0.704688,    0.649852,    0.598160,    0.550189,    0.506382,&
 &     0.467025,    0.432235,    0.401941,    0.375869,    0.353527,&
 &     0.334183,    0.316855,    0.300292,    0.282958,    0.263012,&
 &     0.238299,    0.206327,    0.164255,    0.108872,    0.036585,&
 &    -0.027278,   -0.027278/
DATA ZQLSTD91(:,4)/&
 &  37*0.000000,    1.493179,    1.382958,    1.213328,    1.014777,&
 &     0.794989,    0.560914,    0.318793,    0.074192,   -0.167975,&
 &    -0.403411,   -0.628413,   -0.839837,   -1.035076,   -1.212023,&
 &    -1.369052,   -1.504978,   -1.619037,   -1.710854,   -1.780413,&
 &    -1.828029,   -1.854320,   -1.860176,   -1.846732,   -1.815341,&
 &    -1.767537,   -1.705018,   -1.629607,   -1.543228,   -1.447876,&
 &    -1.345589,   -1.238416,   -1.128394,   -1.017512,   -0.907688,&
 &    -0.800736,   -0.698341,   -0.602027,   -0.513127,   -0.432759,&
 &    -0.361794,   -0.300827,   -0.250147,   -0.209711,   -0.179115,&
 &    -0.157562,   -0.143835,   -0.136269,   -0.132720,   -0.130540,&
 &    -0.126542,   -0.116976,   -0.097500,   -0.063147,   -0.008301,&
 &     0.044960,    0.044960/
DATA ZQLSTD91(:,5)/&
 &  37*0.000000,   -0.939833,   -0.894411,   -0.823497,   -0.739252,&
 &    -0.644998,   -0.543746,   -0.438211,   -0.330819,   -0.223721,&
 &    -0.118809,   -0.017721,    0.078142,    0.167606,    0.249710,&
 &     0.323699,    0.389004,    0.445239,    0.492182,    0.529766,&
 &     0.558067,    0.577291,    0.587764,    0.589918,    0.584278,&
 &     0.571455,    0.552128,    0.527035,    0.496963,    0.462730,&
 &     0.425181,    0.385169,    0.343545,    0.301150,    0.258798,&
 &     0.217264,    0.177278,    0.139506,    0.104540,    0.072890,&
 &     0.044966,    0.021072,    0.001387,   -0.014041,   -0.025307,&
 &    -0.032667,   -0.036544,   -0.037541,   -0.036456,   -0.034291,&
 &    -0.032266,   -0.031827,   -0.034664,   -0.042720,   -0.058201,&
 &    -0.074264,   -0.074264/
!    ----------------------------------------------------------------
!    0.3.4 q/q_s(T^b) normalization stdev, min, max (rh^b>1.x)
!    ----------------------------------------------------------------
DATA ZQHMIN91/&
 &  37*0.000100,    0.885445,    0.880112,    0.876134,    0.876297,&
 &     0.879336,    0.884295,    0.890237,    0.896675,    0.903227,&
 &     0.908511,    0.912495,    0.916544,    0.920979,    0.925515,&
 &     0.930657,    0.936577,    0.942107,    0.946581,    0.950718,&
 &     0.955350,    0.960046,    0.964053,    0.967234,    0.969900,&
 &     0.972299,    0.974228,    0.975936,    0.977826,    0.979469,&
 &     0.980703,    0.981731,    0.982161,    0.981937,    0.981642,&
 &     0.981090,    0.979695,    0.977460,    0.974182,    0.969167,&
 &     0.960679,    0.961000,    0.961300,    0.961600,    0.961885,&
 &     0.970017,    0.975702,    0.980159,    0.983771,    0.986742,&
 &     0.989205,    0.991254,    0.992960,    0.994378,    0.995519,&
 &     0.996198,    0.996198/
DATA ZQHMAX91/                                             0.815000,&
 &     0.815000,    0.955000,    1.105000,    1.175000,    1.165000,&
 &     1.135000,    1.105000,    1.095000,    1.065000,    1.045000,&
 &     1.035000,    1.015000,    0.995000,    0.975000,    0.955000,&
 &     0.955000,    0.945000,    0.945000,    0.935000,    0.915000,&
 &     0.915000,    0.895000,    0.875000,    0.845000,    0.815000,&
 &     0.795000,    0.775000,    0.745000,    0.705000,    0.665000,&
 &     0.615000,    0.645000,    0.725000,    0.845000,    0.975000,&
 &     1.132000,    1.781956,    1.793983,    1.800000,    1.800000,&
 &     1.800000,    1.798122,    1.794144,    1.788349,    1.780390,&
 &     1.779415,    1.789022,    1.797665,    1.798000,    1.798322,&
 &     1.787933,    1.763272,    1.733833,    1.712558,    1.692414,&
 &     1.660254,    1.618858,    1.582022,    1.554182,    1.529706,&
 &     1.505745,    1.486560,    1.465297,    1.435993,    1.409412,&
 &     1.385035,    1.361290,    1.347429,    1.339396,    1.326636,&
 &     1.315486,    1.312000,    1.312000,    1.315608,    1.323558,&
 &     1.331542,    1.339480,    1.344000,    1.339835,    1.327470,&
 &     1.311536,    1.295606,    1.279676,    1.263740,    1.247784,&
 &     1.231776,    1.215667,    1.199377,    1.182835,    1.166040,&
 &     1.155193,    1.155193/
DATA ZQHSTD91(:,0)/&
 &  37*0.000000,   -0.342732,   -0.362103,   -0.420157,   -0.517658,&
 &    -0.643034,   -0.791460,   -0.955774,   -1.134000,   -1.320753,&
 &    -1.473775,   -1.576377,   -1.666136,   -1.758237,   -1.846378,&
 &    -1.964689,   -2.142621,   -2.319001,   -2.429072,   -2.520001,&
 &    -2.687298,   -2.906504,   -3.068638,   -3.109053,   -3.044724,&
 &    -2.905975,   -2.632820,   -2.245553,   -1.783200,   -1.140229,&
 &    -0.209746,    1.021430,    2.363045,    3.752332,    5.445938,&
 &     7.173672,    8.390904,    9.185529,    9.425967,    9.098628,&
 &     8.597799,    7.949048,    7.399437,    7.190437,    7.277026,&
 &     7.410617,    7.314431,    6.927804,    6.184854,    5.021207,&
 &     3.389720,    1.294935,   -1.132950,   -3.478606,   -4.690103,&
 &    -4.527617,   -4.527617/
DATA ZQHSTD91(:,1)/                                        0.034618,&
 &     0.034618,    0.031840,    0.028331,    0.025110,    0.022171,&
 &     0.019505,    0.017105,    0.014964,    0.013074,    0.011428,&
 &     0.010019,    0.008839,    0.007880,    0.007136,    0.006599,&
 &     0.006261,    0.006116,    0.006155,    0.006372,    0.006758,&
 &     0.007308,    0.008012,    0.008864,    0.009856,    0.010981,&
 &     0.012232,    0.013601,    0.015080,    0.016663,    0.018342,&
 &     0.020109,    0.021958,    0.023880,    0.025869,    0.027916,&
 &     0.029479,    0.331875,    0.378546,    0.508366,    0.725481,&
 &     1.007439,    1.341265,    1.710534,    2.111324,    2.531379,&
 &     2.880161,    3.119605,    3.327683,    3.537467,    3.736523,&
 &     4.002079,    4.401710,    4.799068,    5.047051,    5.251522,&
 &     5.630840,    6.128225,    6.489513,    6.559934,    6.372267,&
 &     5.991755,    5.276776,    4.257572,    2.989847,    1.228812,&
 &    -1.302665,   -4.672156,   -8.283015,  -11.963956,  -16.538992,&
 &   -21.224708,  -24.434901,  -26.457369,  -26.941383,  -25.852803,&
 &   -24.327689,  -22.436372,  -20.875532,  -20.333369,  -20.686938,&
 &   -21.237261,  -21.204167,  -20.422443,  -18.711249,  -15.892777,&
 &   -11.836559,   -6.557572,   -0.428397,    5.360709,    7.783330,&
 &     6.464697,    6.464697/
DATA ZQHSTD91(:,2)/&
 &  37*0.000000,    0.201628,    0.170733,    0.086124,   -0.060985,&
 &    -0.258965,   -0.496220,   -0.760415,   -1.049008,   -1.352846,&
 &    -1.610965,   -1.795297,   -1.955478,   -2.114111,   -2.263798,&
 &    -2.460427,   -2.753781,   -3.046842,   -3.232286,   -3.385978,&
 &    -3.668327,   -4.036626,   -4.299055,   -4.334960,   -4.162213,&
 &    -3.824393,   -3.212976,   -2.333039,   -1.189818,    0.400089,&
 &     2.675201,    5.725376,    8.941075,   12.167539,   16.262894,&
 &    20.476127,   23.279953,   24.978585,   25.262451,   24.103310,&
 &    22.584801,   20.770911,   19.311395,   18.855225,   19.291510,&
 &    19.970206,   20.163481,   19.722100,   18.481235,   16.277600,&
 &    12.990931,    8.636838,    3.569253,   -1.077819,   -2.407362,&
 &    -0.352954,   -0.352954/
DATA ZQHSTD91(:,3)/&
 &  37*0.000000,   -0.142512,   -0.135264,   -0.117634,   -0.086171,&
 &    -0.041998,    0.011578,    0.071617,    0.137640,    0.207485,&
 &     0.268568,    0.314295,    0.353995,    0.392469,    0.428519,&
 &     0.475006,    0.543649,    0.612658,    0.657104,    0.694197,&
 &     0.761531,    0.848755,    0.909257,    0.912526,    0.860649,&
 &     0.762297,    0.590593,    0.340668,    0.001086,   -0.472303,&
 &    -1.147782,   -2.060823,   -3.008291,   -3.943957,   -5.158589,&
 &    -6.414417,   -7.225741,   -7.696889,   -7.737451,   -7.339757,&
 &    -6.845687,   -6.274483,   -5.826297,   -5.703388,   -5.872802,&
 &    -6.134899,   -6.265249,   -6.219178,   -5.946820,   -5.398331,&
 &    -4.536771,   -3.367317,   -2.001511,   -0.798422,   -0.680533,&
 &    -1.579158,   -1.579158/
DATA ZQHSTD91(:,4)/93*0.0_JPRB/
DATA ZQHSTD91(:,5)/93*0.0_JPRB/
!    ----------------------------------------------------------------
!    0.3.5 O3 normalization stdev, min
!    ----------------------------------------------------------------
DATA ZO3MIN91/93*1.E-10/
DATA ZO3STD91(:,0)/93*0./
DATA ZO3STD91(:,1)/93*1./
DATA ZO3STD91(:,2)/93*0./
DATA ZO3STD91(:,3)/93*0./
DATA ZO3STD91(:,4)/93*0./
DATA ZO3STD91(:,5)/93*0./

!--------------------------------------------------------------------
! 1. PREPARE INTERPOLATION BETWEEN MODEL LEVELS (ICOEFLEV --> NFLEVG)
!--------------------------------------------------------------------
!    1.1 Allocate output arrays
!    ----------------------------------------------------------------
ALLOCATE(RQS(0:6))
ALLOCATE(RQT(0:6))
ALLOCATE(RQTBAL(NFLEVG,0:5))
ALLOCATE(RQLMIN(NFLEVG))
ALLOCATE(RQLMAX(NFLEVG))
ALLOCATE(RQLSTD(NFLEVG,0:5))
ALLOCATE(RQHMIN(NFLEVG))
ALLOCATE(RQHMAX(NFLEVG))
ALLOCATE(RQHSTD(NFLEVG,0:5))
ALLOCATE(RO3MIN(NFLEVG))
ALLOCATE(RO3STD(NFLEVG,0:5))
!    ----------------------------------------------------------------
!    1.2 Allocate input arrays
!    ----------------------------------------------------------------
IF (LJB_SUPSAT) THEN
  ICOEFLEV = ICOEFLEV91
ELSE
  ICOEFLEV = ICOEFLEV60
ENDIF
ALLOCATE(ZAH   (0:ICOEFLEV))
ALLOCATE(ZBH   (0:ICOEFLEV))
ALLOCATE(ZPRHIN(0:ICOEFLEV+1))
ALLOCATE(ZQTBAL(0:ICOEFLEV+1,0:5))
ALLOCATE(ZQLMIN(0:ICOEFLEV+1))
ALLOCATE(ZQLMAX(0:ICOEFLEV+1))
ALLOCATE(ZQLSTD(0:ICOEFLEV+1,0:5))
ALLOCATE(ZQHMIN(0:ICOEFLEV+1))
ALLOCATE(ZQHMAX(0:ICOEFLEV+1))
ALLOCATE(ZQHSTD(0:ICOEFLEV+1,0:5))
ALLOCATE(ZO3MIN(0:ICOEFLEV+1))
ALLOCATE(ZO3STD(0:ICOEFLEV+1,0:5))
!    ----------------------------------------------------------------
!    1.3 Assign input arrays
!    ----------------------------------------------------------------
IF (LJB_SUPSAT) THEN
  ZAH   (:)  =ZAH91   (:)
  ZBH   (:)  =ZBH91   (:)
  ZQTBAL(:,:)=ZQTBAL91(:,:)
  ZQLMIN(:)  =ZQLMIN91(:)
  ZQLMAX(:)  =ZQLMAX91(:)
  ZQLSTD(:,:)=ZQLSTD91(:,:)
  ZQHMIN(:)  =ZQHMIN91(:)
  ZQHMAX(:)  =ZQHMAX91(:)
  ZQHSTD(:,:)=ZQHSTD91(:,:)
  ZO3MIN(:)  =ZO3MIN91(:)
  ZO3STD(:,:)=ZO3STD91(:,:)
ELSE
  ZAH   (:)  =ZAH60   (:)
  ZBH   (:)  =ZBH60   (:)
  ZQTBAL(:,:)=ZQTBAL60(:,:)
  ZQLMIN(:)  =ZQLMIN60(:)
  ZQLMAX(:)  =ZQLMAX60(:)
  ZQLSTD(:,:)=ZQLSTD60(:,:)
  ZQHMIN(:)  =ZQLMIN60(:)   ! same as L for 60-level coefs
  ZQHMAX(:)  =ZQLMAX60(:)   ! same as L for 60-level coefs
  ZQHSTD(:,:)=ZQLSTD60(:,:) ! same as L for 60-level coefs
  ZO3MIN(:)  =ZO3MIN60(:)
  ZO3STD(:,:)=ZO3STD60(:,:)
ENDIF

!--------------------------------------------------------------------
! 2. INTERPOLATION BETWEEN MODEL LEVELS (ICOEFLEV --> NFLEVG)
!--------------------------------------------------------------------
RQT(:) = ZQT(:)
RQS(:) = ZQS(:)
!    ----------------------------------------------------------------
!    2.1 Same input and output level, no interpolation
!    ----------------------------------------------------------------
IF (ICOEFLEV==NFLEVG) THEN
  RQTBAL(1:NFLEVG,:)=ZQTBAL(1:NFLEVG,:)
  RQLMIN(1:NFLEVG)  =ZQLMIN(1:NFLEVG)
  RQLMAX(1:NFLEVG)  =ZQLMAX(1:NFLEVG)
  RQLSTD(1:NFLEVG,:)=ZQLSTD(1:NFLEVG,:)
  RQHMIN(1:NFLEVG)  =ZQHMIN(1:NFLEVG)
  RQHMAX(1:NFLEVG)  =ZQHMAX(1:NFLEVG)
  RQHSTD(1:NFLEVG,:)=ZQHSTD(1:NFLEVG,:)
  RO3MIN(1:NFLEVG)  =ZO3MIN(1:NFLEVG)
  RO3STD(1:NFLEVG,:)=ZO3STD(1:NFLEVG,:)
ELSE
!    ----------------------------------------------------------------
!    2.2 Different input and output level, interpolation
!    ----------------------------------------------------------------
  WRITE(NULOUT,*) 'SUJBCHVAR - Interpolating humidity/ozone change of variable'
  WRITE(NULOUT,*) '            coefficients from',ICOEFLEV,'to',NFLEVG,'levels.'
!        ------------------------------------------------------------
!        2.2.1 Pressure of ICOEFLEV-level coefficient sets
!        ------------------------------------------------------------
  ZPRHIN(0)  = 0._JPRB
  ZPRHIN(ICOEFLEV+1) = VP00
  DO JLEVIN=1,ICOEFLEV
    ZPRHIN(JLEVIN) = 0.5_JPRB*(  ZAH(JLEVIN-1)+ZAH(JLEVIN)&
     &                         +(ZBH(JLEVIN-1)+ZBH(JLEVIN))*VP00)
  ENDDO
!        ------------------------------------------------------------
!        2.2.2 Pressure of NFLEVG-level coefficient sets
!        ------------------------------------------------------------
  DO JLEVOUT=1,NFLEVG
    ZPRH(JLEVOUT) = 0.5_JPRB*(  VAH(JLEVOUT-1)+VAH(JLEVOUT)&
     &                        +(VBH(JLEVOUT-1)+VBH(JLEVOUT))*VP00)
  ENDDO
!        ------------------------------------------------------------
!        2.2.3 Interpolation coefficients
!        ------------------------------------------------------------
  DO JLEVOUT=1,NFLEVG
    DO JLEVIN = 0, ICOEFLEV
      IF (ZPRH(JLEVOUT) >  ZPRHIN(JLEVIN) .AND.&
       &  ZPRH(JLEVOUT) <= ZPRHIN(JLEVIN+1)) THEN
        INDX(JLEVOUT)  = JLEVIN
        ZFRAC(JLEVOUT) = (ZPRH(JLEVOUT)-ZPRHIN(JLEVIN))&
         &              /(ZPRHIN(JLEVIN+1)-ZPRHIN(JLEVIN))
      ENDIF
    ENDDO
  ENDDO
!        ------------------------------------------------------------
!        2.2.4 Interpolate linearly between model levels 
!              (ICOEFLEV --> NFLEVG) and assign coefficient values
!        ------------------------------------------------------------
  DO JLEVOUT=1,NFLEVG
    JLEVIN = INDX(JLEVOUT)

    RQTBAL(JLEVOUT,:) = (1._JPRB-ZFRAC(JLEVOUT))*ZQTBAL(JLEVIN,:)&
                      &+         ZFRAC(JLEVOUT) *ZQTBAL(JLEVIN+1,:)

    RQLMIN(JLEVOUT)   = (1._JPRB-ZFRAC(JLEVOUT))*ZQLMIN(JLEVIN)&
                      &+         ZFRAC(JLEVOUT) *ZQLMIN(JLEVIN+1)
    RQLMAX(JLEVOUT)   = (1._JPRB-ZFRAC(JLEVOUT))*ZQLMAX(JLEVIN)&
                      &+         ZFRAC(JLEVOUT) *ZQLMAX(JLEVIN+1)
    RQLSTD(JLEVOUT,:) = (1._JPRB-ZFRAC(JLEVOUT))*ZQLSTD(JLEVIN,:)&
                      &+         ZFRAC(JLEVOUT) *ZQLSTD(JLEVIN+1,:)

    RQHMIN(JLEVOUT)   = (1._JPRB-ZFRAC(JLEVOUT))*ZQHMIN(JLEVIN)&
                      &+         ZFRAC(JLEVOUT) *ZQHMIN(JLEVIN+1)
    RQHMAX(JLEVOUT)   = (1._JPRB-ZFRAC(JLEVOUT))*ZQHMAX(JLEVIN)&
                      &+         ZFRAC(JLEVOUT) *ZQHMAX(JLEVIN+1)
    RQHSTD(JLEVOUT,:) = (1._JPRB-ZFRAC(JLEVOUT))*ZQHSTD(JLEVIN,:)&
                      &+         ZFRAC(JLEVOUT) *ZQHSTD(JLEVIN+1,:)

    RO3MIN(JLEVOUT)   = (1._JPRB-ZFRAC(JLEVOUT))*ZO3MIN(JLEVIN)&
                      &+         ZFRAC(JLEVOUT) *ZO3MIN(JLEVIN+1)
    RO3STD(JLEVOUT,:) = (1._JPRB-ZFRAC(JLEVOUT))*ZO3STD(JLEVIN,:)&
                      &+         ZFRAC(JLEVOUT) *ZO3STD(JLEVIN+1,:)
  ENDDO

ENDIF

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJBCHVAR',1,ZHOOK_HANDLE)

END SUBROUTINE SUJBCHVAR
