SUBROUTINE SUJBTEST(KPRT)
!**** *SUJBTEST* - Test technical correctness of background contraint Jb

!     Purpose. Check the change of variable.
!     --------

!     Interface. CALL SUJBTEST(KPRT)
!     ----------

!     Explicit arguments : KPRT (input) = level of verbosity
!     --------------------

!     Implicit arguments : common YOMJG and related files.
!     --------------------

!     Method.
!     -------
!         Test adjoint identity between CVAR2IN/CVAR2INAD
!         Test invertibility between CVAR2/CVAR2IN
!         Test invertibility between CVAR2AD/CVAR2INAD

!     Externals: see below.
!     ----------

!     Author : Francois Bouttier *ECMWF*
!     --------

!     Modifications :
!     ---------------
!     96-11-14 F.Bouttier : write original
!     97-12-15 E.Holm     : ozone included
!     02-09-30 M.FIsher   : RANDOM_NUMBERS_MIX
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NFLSUR   ,NSMAX    ,NSPEC2
USE YOMJG    , ONLY : LSUBBG
USE YOMLUN   , ONLY : NULOUT
USE YOMSPJB  , ONLY : SPJB
USE YOMLAP   , ONLY : NASM0
USE YOMMP    , ONLY : NPSP
USE YOMCVA   , ONLY : YVAZX     ,YVAZG,   NVA3D, NVA2D
USE RANDOM_NUMBERS_MIX
USE SPECTRAL_FIELDS_MOD


IMPLICIT NONE

!        Arguments
INTEGER(KIND=JPIM),INTENT(IN)    :: KPRT 
REAL(KIND=JPRB) :: ZRAND(NSPEC2)
REAL(KIND=JPRB) ::  ZX3(NFLSUR,NSPEC2,NVA3D), ZX2(NSPEC2,NVA2D)
REAL(KIND=JPRB) :: ZFX3(NFLSUR,NSPEC2,NVA3D),ZFX2(NSPEC2,NVA2D)
REAL(KIND=JPRB) ::  ZY3(NFLSUR,NSPEC2,NVA3D), ZY2(NSPEC2,NVA2D)
REAL(KIND=JPRB) :: ZFY3(NFLSUR,NSPEC2,NVA3D),ZFY2(NSPEC2,NVA2D)

INTEGER(KIND=JPIM) :: JL, JXY

LOGICAL :: LLSUBBG

REAL(KIND=JPRB) :: ZS1, ZS1D, ZS1O3, ZS1P, ZS1Q, ZS1T, ZS1V,&
 & ZS2, ZS2D, ZS2O3, ZS2P, ZS2Q, ZS2T, ZS2V, &
 & ZS3D, ZSM  
REAL(KIND=JPRB) :: ZHOOK_HANDLE
TYPE(randomNumberStream) :: YL_RANDOM_STREAM

#include "abor1.intfb.h"
#include "cvar2.intfb.h"
#include "cvar2ad.intfb.h"
#include "cvar2in.intfb.h"
#include "cvar2inad.intfb.h"
#include "setran.intfb.h"

!     -----------------------------------------------------------------

!        1. Preliminary checks

IF (LHOOK) CALL DR_HOOK('SUJBTEST',0,ZHOOK_HANDLE)
WRITE(NULOUT,*) 'SUJBTEST - Testing Jb components'
CALL ABOR1('SUJBTEST cannot do message passing')
LLSUBBG=LSUBBG
LSUBBG=.FALSE.

!        2. Create 2 random spectral perturbations

CALL SETRAN (0,YL_RANDOM_STREAM)

LOOPXY : DO JXY=1,2
  DO JL=1,NFLEVG
    CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
    SPJB%VOR(JL,:) = 1.0_JPRB-2.0_JPRB*ZRAND(:)
    SPJB%VOR(JL,NASM0(0)) = 0.0_JPRB
    CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
    SPJB%DIV(JL,:) = 1.0_JPRB-2.0_JPRB*ZRAND(:)
    SPJB%DIV(JL,NASM0(0)) = 0.0_JPRB
    CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
    SPJB%TEM  (JL,:) = 1.0_JPRB-2.0_JPRB*ZRAND(:)
    CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
    SPJB%HUM  (JL,:) = 1.0_JPRB-2.0_JPRB*ZRAND(:)
    CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
    SPJB%OZN (JL,:) = 1.0_JPRB-2.0_JPRB*ZRAND(:)
  ENDDO

  CALL UNIFORM_DISTRIBUTION (ZRAND,YL_RANDOM_STREAM)
  IF (NPSP==1) SPJB%SPR(:) = 1.0_JPRB-2.0_JPRB*ZRAND(:)
  SPJB%SP3D(:,2:2*NSMAX+2:2,:)=0.0_JPRB
  IF (NPSP==1) SPJB%SPR(2:2*NSMAX+2:2)=0.0_JPRB
  IF (JXY==1) THEN
    ZX3(:,:,:)=SPJB%SP3D(:,:,:)
    IF (NPSP==1) THEN
      ZX2(:,:)  =SPJB%SP2D(:,:)
    ELSE
      ZX2(:,:)  =0.0_JPRB
    ENDIF
    WRITE(NULOUT,*) ' Norm of perturbation X :',&
     & SUM(ZX3(:,:,:)*ZX3(:,:,:))+SUM(ZX2(:,1)*ZX2(:,1))  
  ELSE
    ZY3(:,:,:)=SPJB%SP3D(:,:,:)
    IF (NPSP==1) THEN
      ZY2(:,:)  =SPJB%SP2D(:,:)
    ELSE
      ZY2(:,:)  =0.0_JPRB
    ENDIF
    WRITE(NULOUT,*) ' Norm of perturbation Y :',&
     & SUM(ZY3(:,:,:)*ZY3(:,:,:))+SUM(ZY2(:,1)*ZY2(:,1))  
  ENDIF
ENDDO LOOPXY

!        3. Test of the adjoint between CVAR2IN/CVAR2INAD

WRITE(NULOUT,*) '    Testing adjoint between CVAR2IN/CVAR2INAD'
SPJB%SP3D(:,:,:)=ZX3(:,:,:)
IF (NPSP==1) SPJB%SP2D(:,:)  =ZX2(:,:)
CALL CVAR2IN(YVAZX)
ZFX3(:,:,:)=SPJB%SP3D(:,:,:)
IF (NPSP==1) THEN
  ZFX2(:,:)  =SPJB%SP2D(:,:)
ELSE
  ZFX2(:,:)  =0.0_JPRB
ENDIF
SPJB%SP3D(:,:,:)=ZY3(:,:,:)
IF (NPSP==1) SPJB%SP2D(:,:)  =ZY2(:,:)
CALL CVAR2INAD(YVAZG)
ZFY3(:,:,:)=SPJB%SP3D(:,:,:)
IF (NPSP==1) THEN
  ZFY2(:,:)  =SPJB%SP2D(:,:)
ELSE
  ZFY2(:,:)  =0.0_JPRB
ENDIF
ZS1=SUM(ZFX3(:,:,:)*ZY3(:,:,:))+SUM(ZFX2(:,1)*ZY2(:,1))
ZS2=SUM(ZX3(:,:,:)*ZFY3(:,:,:))+SUM(ZX2(:,1)*ZFY2(:,1))
WRITE(NULOUT,*) ' < CVAR2IN.X , Y >   = ',ZS1
WRITE(NULOUT,*) ' < X , CVAR2INAD Y > = ',ZS2
ZSM=ABS((ZS1+ZS2)/2.0_JPRB)
IF (ZSM >TINY(ZSM)) WRITE(NULOUT,*) ' Relative error =',&
 & ABS(ZS2-ZS1)/ZSM/EPSILON(ZSM),' times the real epsilon'  
!          this details the scalar product per variable (useful if univariate)
ZS1V =SUM(ZFX3(:,:,1)*ZY3(:,:,1))
ZS2V =SUM(ZX3(:,:,1)*ZFY3(:,:,1))
ZS1D =SUM(ZFX3(:,:,2)*ZY3(:,:,2))
ZS2D =SUM(ZX3(:,:,2)*ZFY3(:,:,2))
ZS1T =SUM(ZFX3(:,:,3)*ZY3(:,:,3))
ZS2T =SUM(ZX3(:,:,3)*ZFY3(:,:,3))
ZS1Q =SUM(ZFX3(:,:,4)*ZY3(:,:,4))
ZS2Q =SUM(ZX3(:,:,4)*ZFY3(:,:,4))
ZS1O3=SUM(ZFX3(:,:,5)*ZY3(:,:,5))
ZS2O3=SUM(ZX3(:,:,5)*ZFY3(:,:,5))
ZS1P =SUM(ZFX2(:,1)*ZY2(:,1))
ZS1P =SUM(ZFX2(:,1)*ZY2(:,1))
ZS2P =SUM(ZX2(:,1)*ZFY2(:,1))
IF (KPRT>=1) THEN
  WRITE(NULOUT,*) ' Breakdown per param(useless if multivariate):'
  WRITE(NULOUT,*) '    VOR components : ',ZS1V ,ZS2V
  WRITE(NULOUT,*) '    DIV components : ',ZS1D ,ZS2D
  WRITE(NULOUT,*) '     T  components : ',ZS1T ,ZS2T
  WRITE(NULOUT,*) '     Q  components : ',ZS1Q ,ZS2Q
  WRITE(NULOUT,*) '     O3 components : ',ZS1O3,ZS2O3
  WRITE(NULOUT,*) '     SP components : ',ZS1P ,ZS2P
ENDIF

!         4. Test of the inverse between CVAR2/CVAR2IN
!            (Hopeless with non-uniform background errors or 
!             with T,ps->P transform !)

WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) '    Testing inverse between CVAR2/CVAR2IN'
SPJB%SP3D(:,:,:)=ZX3(:,:,:)
IF (NPSP==1) SPJB%SP2D(:,:)  =ZX2(:,:)
!FB      This tests cvaru3 only
!FB      CALL CVARU3(YVAZX)
!FB      CALL CVARU3I(YVAZX)
CALL CVAR2(YVAZX)
CALL CVAR2IN(YVAZX)
ZFX3(:,:,:)=SPJB%SP3D(:,:,:)-ZX3(:,:,:)
IF (NPSP==1) THEN
  ZFX2(:,:)  =SPJB%SP2D(:,:)-ZX2(:,:)
ELSE
  ZFX2(:,:)  =0.0_JPRB
ENDIF
ZS3D=SUM(ZFX3(:,:,:)*ZFX3(:,:,:))
ZS2D=SUM(ZFX2(:,:)*ZFX2(:,:))
WRITE(NULOUT,*) ' dist( CVAR2IN.CVAR2.X , X ) = ',ZS3D+ZS2D
WRITE(NULOUT,*) '  breakdown 3D/2D fields=',ZS3D,ZS2D
IF (KPRT>=1) THEN
  ZS1V =SUM(ZX3(:,:,1)*ZX3(:,:,1))
  ZS2V =SUM(SPJB%SP3D(:,:,1)*SPJB%SP3D(:,:,1))
  ZS1D =SUM(ZX3(:,:,2)*ZX3(:,:,2))
  ZS2D =SUM(SPJB%SP3D(:,:,2)*SPJB%SP3D(:,:,2))
  ZS1T =SUM(ZX3(:,:,3)*ZX3(:,:,3))
  ZS2T =SUM(SPJB%SP3D(:,:,3)*SPJB%SP3D(:,:,3))
  ZS1Q =SUM(ZX3(:,:,4)*ZX3(:,:,4))
  ZS2Q =SUM(SPJB%SP3D(:,:,4)*SPJB%SP3D(:,:,4))
  ZS1O3=SUM(ZX3(:,:,5)*ZX3(:,:,5))
  ZS2O3=SUM(SPJB%SP3D(:,:,5)*SPJB%SP3D(:,:,5))
  ZS1P =SUM(ZX2(:,1)*ZX2(:,1))
  IF (NPSP==1) THEN
    ZS2P =SUM(SPJB%SP2D(:,1)*SPJB%SP2D(:,1))
  ELSE
    ZS2P =0.0_JPRB
  ENDIF
  WRITE(NULOUT,*) '  VOR components : ',ZS1V ,ZS2V
  WRITE(NULOUT,*) '  DIV components : ',ZS1D ,ZS2D
  WRITE(NULOUT,*) '   T  components : ',ZS1T ,ZS2T
  WRITE(NULOUT,*) '   Q  components : ',ZS1Q ,ZS2Q
  WRITE(NULOUT,*) '   O3 components : ',ZS1O3,ZS2O3
  WRITE(NULOUT,*) '   SP components : ',ZS1P ,ZS2P
ENDIF

WRITE(NULOUT,*) ' '
SPJB%SP3D(:,:,:)=ZY3(:,:,:)
IF (NPSP==1) SPJB%SP2D(:,:)  =ZY2(:,:)
!FB     This tests cvaru3 only
!FB      CALL CVARU3I(YVAZX)
!FB      CALL CVARU3(YVAZX)
CALL CVAR2IN(YVAZX)
CALL CVAR2(YVAZX)
ZFY3(:,:,:)=SPJB%SP3D(:,:,:)-ZY3(:,:,:)
IF (NPSP==1) THEN
  ZFY2(:,:)  =SPJB%SP2D(:,:)-ZY2(:,:)
ELSE
  ZFY2(:,:)  =0.0_JPRB
ENDIF
ZS3D=SUM(ZFY3(:,:,:)*ZFY3(:,:,:))
ZS2D=SUM(ZFY2(:,:)*ZFY2(:,:))
WRITE(NULOUT,*) ' dist( CVAR2.CVAR2IN.Y , Y ) = ',ZS3D+ZS2D
WRITE(NULOUT,*) '  breakdown 3D/2D fields=',ZS3D,ZS2D
IF (KPRT>=1) THEN
  ZS1V =SUM(ZY3(:,:,1)*ZY3(:,:,1))
  ZS2V =SUM(SPJB%SP3D(:,:,1)*SPJB%SP3D(:,:,1))
  ZS1D =SUM(ZY3(:,:,2)*ZY3(:,:,2))
  ZS2D =SUM(SPJB%SP3D(:,:,2)*SPJB%SP3D(:,:,2))
  ZS1T =SUM(ZY3(:,:,3)*ZY3(:,:,3))
  ZS2T =SUM(SPJB%SP3D(:,:,3)*SPJB%SP3D(:,:,3))
  ZS1Q =SUM(ZY3(:,:,4)*ZY3(:,:,4))
  ZS2Q =SUM(SPJB%SP3D(:,:,4)*SPJB%SP3D(:,:,4))
  ZS1O3=SUM(ZY3(:,:,5)*ZY3(:,:,5))
  ZS2O3=SUM(SPJB%SP3D(:,:,5)*SPJB%SP3D(:,:,5))
  ZS1P =SUM(ZY2(:,1)*ZY2(:,1))
  IF (NPSP==1) THEN
    ZS2P =SUM(SPJB%SP2D(:,1)*SPJB%SP2D(:,1))
  ELSE
    ZS2P =0.0_JPRB
  ENDIF
  WRITE(NULOUT,*) '  VOR components : ',ZS1V ,ZS2V
  WRITE(NULOUT,*) '  DIV components : ',ZS1D ,ZS2D
  WRITE(NULOUT,*) '   T  components : ',ZS1T ,ZS2T
  WRITE(NULOUT,*) '   Q  components : ',ZS1Q ,ZS2Q
  WRITE(NULOUT,*) '   O3 components : ',ZS1O3,ZS2O3
  WRITE(NULOUT,*) '   SP components : ',ZS1P ,ZS2P
ENDIF

!         5. Test of the inverse between CVAR2AD/CVAR2INAD
!            (Hopeless with non-uniform background errors or 
!             with T,ps->P transform !)

WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) '    Testing inverse between CVAR2AD/CVAR2INAD'
SPJB%SP3D(:,:,:)=ZX3(:,:,:)
IF (NPSP==1) SPJB%SP2D(:,:)  =ZX2(:,:)
!FB      This tests cvaru3 only
!FB      CALL CVARU3AD(YVAZG)
!FB      CALL CVARU3IAD(YVAZG)
CALL CVAR2AD(YVAZG)
CALL CVAR2INAD(YVAZG)
ZFX3(:,:,:)=SPJB%SP3D(:,:,:)-ZX3(:,:,:)
IF (NPSP==1) THEN
  ZFX2(:,:)  =SPJB%SP2D(:,:)-ZX2(:,:)
ELSE
  ZFX2(:,:)  =0.0_JPRB
ENDIF
ZS3D=SUM(ZFX3(:,:,:)*ZFX3(:,:,:))
ZS2D=SUM(ZFX2(:,:)*ZFX2(:,:))
WRITE(NULOUT,*) ' dist( CVAR2INAD.CVAR2AD.X , X ) = ',ZS3D+ZS2D
WRITE(NULOUT,*) '  breakdown 3D/2D fields=',ZS3D,ZS2D
IF (KPRT>=1) THEN
  ZS1V =SUM(ZX3(:,:,1)*ZX3(:,:,1))
  ZS2V =SUM(SPJB%SP3D(:,:,1)*SPJB%SP3D(:,:,1))
  ZS1D =SUM(ZX3(:,:,2)*ZX3(:,:,2))
  ZS2D =SUM(SPJB%SP3D(:,:,2)*SPJB%SP3D(:,:,2))
  ZS1T =SUM(ZX3(:,:,3)*ZX3(:,:,3))
  ZS2T =SUM(SPJB%SP3D(:,:,3)*SPJB%SP3D(:,:,3))
  ZS1Q =SUM(ZX3(:,:,4)*ZX3(:,:,4))
  ZS2Q =SUM(SPJB%SP3D(:,:,4)*SPJB%SP3D(:,:,4))
  ZS1O3=SUM(ZX3(:,:,5)*ZX3(:,:,5))
  ZS2O3=SUM(SPJB%SP3D(:,:,5)*SPJB%SP3D(:,:,5))
  ZS1P =SUM(ZX2(:,1)*ZX2(:,1))
  IF (NPSP==1) THEN
    ZS2P =SUM(SPJB%SP2D(:,1)*SPJB%SP2D(:,1))
  ELSE
    ZS2P =0.0_JPRB
  ENDIF
  WRITE(NULOUT,*) '  VOR components : ',ZS1V ,ZS2V
  WRITE(NULOUT,*) '  DIV components : ',ZS1D ,ZS2D
  WRITE(NULOUT,*) '   T  components : ',ZS1T ,ZS2T
  WRITE(NULOUT,*) '   Q  components : ',ZS1Q ,ZS2Q
  WRITE(NULOUT,*) '   O3 components : ',ZS1O3,ZS2O3
  WRITE(NULOUT,*) '   SP components : ',ZS1P ,ZS2P
ENDIF

WRITE(NULOUT,*) ' '
SPJB%SP3D(:,:,:)=ZY3(:,:,:)
IF (NPSP==1) SPJB%SP2D(:,:)  =ZY2(:,:)
!FB      This tests cvaru3 only
!FB      CALL CVARU3IAD(YVAZG)
!FB      CALL CVARU3AD(YVAZG)
CALL CVAR2INAD(YVAZG)
CALL CVAR2AD(YVAZG)
ZFY3(:,:,:)=SPJB%SP3D(:,:,:)-ZY3(:,:,:)
IF (NPSP==1) THEN
  ZFY2(:,:)  =SPJB%SP2D(:,:)-ZY2(:,:)
ELSE
  ZFY2(:,:)  =0.0_JPRB
ENDIF
ZS3D=SUM(ZFY3(:,:,:)*ZFY3(:,:,:))
ZS2D=SUM(ZFY2(:,:)*ZFY2(:,:))
WRITE(NULOUT,*) ' dist( CVAR2AD.CVAR2INAD.Y , Y ) = ',ZS3D+ZS2D
WRITE(NULOUT,*) '  breakdown 3D/2D fields=',ZS3D,ZS2D
IF (KPRT>=1) THEN
  ZS1V =SUM(ZY3(:,:,1)*ZY3(:,:,1))
  ZS2V =SUM(SPJB%SP3D(:,:,1)*SPJB%SP3D(:,:,1))
  ZS1D =SUM(ZY3(:,:,2)*ZY3(:,:,2))
  ZS2D =SUM(SPJB%SP3D(:,:,2)*SPJB%SP3D(:,:,2))
  ZS1T =SUM(ZY3(:,:,3)*ZY3(:,:,3))
  ZS2T =SUM(SPJB%SP3D(:,:,3)*SPJB%SP3D(:,:,3))
  ZS1Q =SUM(ZY3(:,:,4)*ZY3(:,:,4))
  ZS2Q =SUM(SPJB%SP3D(:,:,4)*SPJB%SP3D(:,:,4))
  ZS1O3=SUM(ZY3(:,:,5)*ZY3(:,:,5))
  ZS2O3=SUM(SPJB%SP3D(:,:,5)*SPJB%SP3D(:,:,5))
  ZS1P =SUM(ZY2(:,1)*ZY2(:,1))
  IF (NPSP==1) THEN
    ZS2P =SUM(SPJB%SP2D(:,1)*SPJB%SP2D(:,1))
  ELSE
    ZS2P =0.0_JPRB
  ENDIF
  WRITE(NULOUT,*) '  VOR components : ',ZS1V ,ZS2V
  WRITE(NULOUT,*) '  DIV components : ',ZS1D ,ZS2D
  WRITE(NULOUT,*) '   T  components : ',ZS1T ,ZS2T
  WRITE(NULOUT,*) '   Q  components : ',ZS1Q ,ZS2Q
  WRITE(NULOUT,*) '   O3 components : ',ZS1O3,ZS2O3
  WRITE(NULOUT,*) '   SP components : ',ZS1P ,ZS2P
ENDIF

LSUBBG=LLSUBBG
!       Stop here because SPA3JB/2 may now be unsuitable for the subsequent job
WRITE(NULOUT,*) 'End of testing, bye.'
STOP

IF (LHOOK) CALL DR_HOOK('SUJBTEST',1,ZHOOK_HANDLE)
END SUBROUTINE SUJBTEST
