SUBROUTINE SUJBWAVALLS (YD_PVC, KSCALE, KLEVS)

!      SUJBWAVALLS - Allocate and distribute vertical correlation matrices
!                    for the wavelet Jb stats file calculation.

!     Purpose: Allocate and distribute the vertical correlation matrices
!     -------  for the wavelet Jb stats file calculation.

!     Interface: called from SUJBWAVSTATS
!     ---------

!     Author: Mike Fisher 2001-10-12
!     ------

!     Modifications:
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN,     ONLY : NULOUT
USE YOMCST,     ONLY : RPI
USE YOMMP,      ONLY : MYPROC, NPRCIDS
USE YOMCT0,     ONLY : NPROC
USE YOMTAG,     ONLY : MTAGFCE
USE YOMWAVELET, ONLY : N_WAVELET_CUTOFFS, WAVELET_VCOR_STRUCT, &
 & GRID_DEFINITION  
USE MPL_MODULE

IMPLICIT NONE

TYPE(WAVELET_VCOR_STRUCT),INTENT(INOUT) :: YD_PVC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSCALE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVS 
INTEGER(KIND=JPIM) :: INPTOT, J, ILATS, JLAT, JLON, I, JPROC, ILEN, IM1, ILSUM, &
 & ILAT_OFFSET, ITAG, ILATNG
INTEGER(KIND=JPIM), ALLOCATABLE :: INDXL2G(:), INLONS(:), IPELIST(:,:), ILON_OFFSET(:)
INTEGER(KIND=JPIM), PARAMETER :: JPIOMASTER=1
LOGICAL, ALLOCATABLE  :: LLWANTED(:)
REAL(KIND=JPRB)    :: ZDLON, ZEPS, ZLONGP, ZLATGP
REAL(KIND=JPRB), ALLOCATABLE :: ZZDLON(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

9990 FORMAT (' VERTICAL CORRELATION MATRIX ALLOCATED',8(1X,I8))

!         Define the grid for the matrices

IF (LHOOK) CALL DR_HOOK('SUJBWAVALLS',0,ZHOOK_HANDLE)
IF (N_WAVELET_CUTOFFS(KSCALE)<=3) THEN
  ILATS=1
ELSE
  ILATS = 2
  DO WHILE (2*ILATS  <  MIN(63,N_WAVELET_CUTOFFS(MAX(1,KSCALE-1))))
    ILATS = 2*ILATS
  ENDDO
ENDIF

YD_PVC%NLATSG = ILATS
YD_PVC%NLEVS = KLEVS

ALLOCATE (YD_PVC%LATSG(ILATS))
ALLOCATE (YD_PVC%NLONSG(ILATS))
ALLOCATE (YD_PVC%LON0G(ILATS))

DO J=1,ILATS
  YD_PVC%LATSG(J)  = RPI * (0.5_JPRB - (J-0.5_JPRB)/ILATS)
ENDDO
IF (ILATS == 1) THEN
  YD_PVC%NLONSG(:) = 1
ELSE
!OCL NOVREC
  YD_PVC%NLONSG(:) = MAX(2, INT(ILATS*2.0_JPRB*COS(YD_PVC%LATSG(:))))
ENDIF
YD_PVC%LON0G(:)  = 0.0_JPRB

YD_PVC%NPTOTG = SUM(YD_PVC%NLONSG(:))

ALLOCATE (YD_PVC%LPELIST(YD_PVC%NPTOTG,NPROC))
ALLOCATE (YD_PVC%MASTERPE(YD_PVC%NPTOTG))

ALLOCATE (YD_PVC%INDEXG2L(YD_PVC%NPTOTG))
YD_PVC%INDEXG2L(:) = HUGE(YD_PVC%INDEXG2L)

!         Find which matrices are updated by gridpoints on this PE

ALLOCATE(INDXL2G(0:YD_PVC%NPTOTG))
ALLOCATE(INLONS(YD_PVC%NLATSG))
ALLOCATE(ILON_OFFSET(YD_PVC%NLATSG))
ALLOCATE(ZZDLON(YD_PVC%NLATSG))

ALLOCATE(LLWANTED(YD_PVC%NPTOTG))

ZEPS=SQRT(EPSILON(ZEPS))

IF (YD_PVC%NPTOTG == 1) THEN
  LLWANTED(:)=.TRUE.
ELSE
  LLWANTED(:)=.FALSE.

  DO J=1,GRID_DEFINITION(KSCALE)%NGPTOT
    ZLONGP = GRID_DEFINITION(KSCALE)%GELAM(J)
    ZLATGP = GRID_DEFINITION(KSCALE)%GELAT(J)

    ILATNG = 1
    DO JLAT=1,YD_PVC%NLATSG-1
      IF (ZLATGP > 0.5_JPRB*( YD_PVC%LATSG(JLAT) &
       & +YD_PVC%LATSG(JLAT+1))) EXIT  
      ILATNG=ILATNG+1
    ENDDO    

    I=SUM(YD_PVC%NLONSG(1:ILATNG-1))+1
    ZDLON  = 2.0_JPRB * RPI / YD_PVC%NLONSG(ILATNG)
    DO JLON=1,YD_PVC%NLONSG(ILATNG)-1
      IF (ABS(YD_PVC%LON0G(ILATNG)+REAL(JLON-1,JPRB)*ZDLON - ZLONGP) &
       & <=0.5_JPRB*ZDLON  .OR.  &
       & ABS(YD_PVC%LON0G(ILATNG)+REAL(JLON-1,JPRB)*ZDLON - ZLONGP) &
       & >=2.0_JPRB*RPI-0.5*ZDLON ) EXIT  
      I=I+1
    ENDDO

    LLWANTED(I) = .TRUE.
  ENDDO
ENDIF

INPTOT=0
ILATS=0
INLONS(:)=0

ILSUM=0
DO JLAT=1,YD_PVC%NLATSG
  IF (JLAT>1) ILSUM=ILSUM+YD_PVC%NLONSG(JLAT-1)
  ZDLON  = 2.0_JPRB*RPI/YD_PVC%NLONSG(JLAT)

!--- find the start of the contiguous block of wanted matrices

  IF (ALL(LLWANTED(ILSUM+1:ILSUM+YD_PVC%NLONSG(JLAT)))) THEN
    I=1
  ELSEIF (ALL(.NOT.LLWANTED(ILSUM+1:ILSUM+YD_PVC%NLONSG(JLAT)))) THEN
    CYCLE
  ELSE 
    IM1=YD_PVC%NLONSG(JLAT)
    I=1
    DO WHILE (LLWANTED(ILSUM+IM1).OR..NOT.LLWANTED(ILSUM+I))
      IM1=I
      I=I+1
    ENDDO
  ENDIF

  ILON_OFFSET(ILATS+1)=I-1
  ZZDLON(ILATS+1)=ZDLON

!--- update the local tables

  DO JLON=1,YD_PVC%NLONSG(JLAT)
    IF (LLWANTED(I+ILSUM)) THEN
      INPTOT=INPTOT+1
      INLONS(ILATS+1)=INLONS(ILATS+1)+1
      INDXL2G(INPTOT) = I+ILSUM
      YD_PVC%INDEXG2L(I+ILSUM) = INPTOT
    ENDIF
    I=1+MOD(I,YD_PVC%NLONSG(JLAT))
  ENDDO
  IF (INLONS(ILATS+1)>0) THEN
    ILATS=ILATS+1
  ENDIF
  IF (ILATS==1) ILAT_OFFSET = JLAT-1
ENDDO

!         Allocate local arrays

YD_PVC%NPTOT = INPTOT
YD_PVC%NLATS = ILATS
YD_PVC%NLAT_OFFSET    = ILAT_OFFSET

ALLOCATE (YD_PVC%NLONS(ILATS))
ALLOCATE (YD_PVC%NLON_OFFSET(ILATS))
ALLOCATE (YD_PVC%DLON(ILATS))

YD_PVC%NLONS(:) = INLONS(1:ILATS)
YD_PVC%NLON_OFFSET(:) = ILON_OFFSET(1:ILATS)
YD_PVC%DLON(:)  = ZZDLON(1:ILATS)

DEALLOCATE(LLWANTED)
DEALLOCATE(INLONS)
DEALLOCATE(ILON_OFFSET)
DEALLOCATE(ZZDLON)

ALLOCATE (YD_PVC%MATS   (KLEVS,KLEVS,INPTOT))
YD_PVC%MATS  (:,:,:) = 0.0_JPRB
WRITE (NULOUT,9990) SIZE(YD_PVC%MATS),    SHAPE(YD_PVC%MATS)

ALLOCATE (YD_PVC%MATINVS(KLEVS,KLEVS,INPTOT))
YD_PVC%MATINVS(:,:,:) = 0.0_JPRB
WRITE (NULOUT,9990) SIZE(YD_PVC%MATINVS), SHAPE(YD_PVC%MATINVS)

ALLOCATE (YD_PVC%MATS_SKYLINE_LEFT (KLEVS,INPTOT))
ALLOCATE (YD_PVC%MATS_SKYLINE_RIGHT(KLEVS,INPTOT))
ALLOCATE (YD_PVC%MATINVS_SKYLINE_LEFT (KLEVS,INPTOT))
ALLOCATE (YD_PVC%MATINVS_SKYLINE_RIGHT(KLEVS,INPTOT))

ALLOCATE (YD_PVC%EVALS  (KLEVS,INPTOT))
YD_PVC%EVALS (:,:) = 0.0_JPRB

ALLOCATE (YD_PVC%NSAMPS  (INPTOT))
YD_PVC%NSAMPS (:)     = 0

ALLOCATE (YD_PVC%SAMPAV  (KLEVS,INPTOT))
YD_PVC%SAMPAV (:,:)   = 0.0_JPRB

!         Decide which PEs update each matrix, and decide on a master PE.

IF (MYPROC/=JPIOMASTER) THEN
  INDXL2G(0) = YD_PVC%NPTOT
  ITAG=MTAGFCE
  CALL MPL_SEND (INDXL2G(0:YD_PVC%NPTOT), KDEST=NPRCIDS(JPIOMASTER), &
   & KTAG=ITAG, CDSTRING='SUJBWAVALLS:')  
ELSE
  YD_PVC%LPELIST(:,:) = .FALSE.

  DO J=1,YD_PVC%NPTOT
    YD_PVC%LPELIST(INDXL2G(J),MYPROC)=.TRUE.
  ENDDO

  DO JPROC=1,NPROC
    IF (JPROC/=MYPROC) THEN
      ITAG=MTAGFCE
      CALL MPL_RECV (INDXL2G(0:), KSOURCE=NPRCIDS(JPROC), KTAG=ITAG, &
       & KOUNT=ILEN, CDSTRING='SUJBWAVALLS:')  
      IF (ILEN/=INDXL2G(0)+1) CALL ABOR1('SUJBWAVELET: BAD MESSAGE LENGTH')

      DO J=1,ILEN-1
        YD_PVC%LPELIST(INDXL2G(J),JPROC) = .TRUE.
      ENDDO
    ENDIF
  ENDDO

!         Round-robin allocation of master PEs to matrices

  I=1
  DO J=1,YD_PVC%NPTOTG
    DO JPROC=1,NPROC
      I=1+MOD(I,NPROC)
      IF (YD_PVC%LPELIST(J,I)) EXIT
    ENDDO
    IF (.NOT.YD_PVC%LPELIST(J,I)) THEN
      WRITE (NULOUT,*) 'sujbwavalls: no processor owns matrix ',J
      CALL ABOR1 ('sujbwavalls: found a matrix not owned by any processor')
    ENDIF
    YD_PVC%MASTERPE(J) = I
  ENDDO

ENDIF

IF (NPROC>1) THEN
  ITAG=MTAGFCE+1
  CALL MPL_BROADCAST(YD_PVC%MASTERPE,KROOT=JPIOMASTER,KTAG=ITAG, &
   & CDSTRING='SUJBWAVALLS')

  ALLOCATE (IPELIST(YD_PVC%NPTOTG,NPROC))
  WHERE (YD_PVC%LPELIST(:,:))
    IPELIST(:,:)=1
  ELSEWHERE
    IPELIST(:,:)=0
  ENDWHERE

  ITAG=MTAGFCE+2
  I=YD_PVC%NPTOTG*NPROC
  CALL MPL_BROADCAST(IPELIST,KROOT=JPIOMASTER,KTAG=ITAG, &
   & CDSTRING='SUJBWAVALLS')

  YD_PVC%LPELIST(:,:) = IPELIST(:,:)==1
  DEALLOCATE (IPELIST)
ENDIF

DEALLOCATE(INDXL2G)

IF (LHOOK) CALL DR_HOOK('SUJBWAVALLS',1,ZHOOK_HANDLE)
END SUBROUTINE SUJBWAVALLS
