SUBROUTINE SUJBWAVELET0

!      SUJBWAVELET0 - Set up dimensioning for wavelet Jb

!     Purpose: Set up vertical correlation matrices for wavelet Jb
!     -------

!     Interface: called from SU0YOMB
!     ---------

!     Author: Mike Fisher 98-06-17
!     ------

!     Modifications:
!     -------------
!        D.Dent     : 00-09-01  MPE to MPL conversion
!        M.Fisher   : 00-10-26  Distributed the matrices over PEs
!      R. El Khatib : 01-08-07  Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher      06-Aug-2007 use IOSTREAM_MIX
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN,     ONLY : NULOUT
USE YOMDIM,     ONLY : NSMAX
USE YOMMP,      ONLY : MYPROC
USE YOMCT0,     ONLY : NPROC
USE YOMTAG,     ONLY : MTAGFCE
USE YOMWAVELET, ONLY : N_WAVELET_SCALES, N_WAVELET_CUTOFFS,&
 & WAVELET_FILTERS, LJBWSTATS  
USE IOSTREAM_MIX, ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
                   & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, &
                   & CLOSE_IOREQUEST
USE MPL_MODULE, ONLY : MPL_BROADCAST
IMPLICIT NONE

INTEGER(KIND=JPIM), PARAMETER :: JPNSCALES=19
INTEGER(KIND=JPIM) :: I_DEFAULT_CUTOFFS(JPNSCALES)= &
 & (/512, 399, 319, 255, 215, 159, 127, 95, &
 &    63, 47, 31, 21, 15, 10, 7, 5, 3, 2, 1/)

INTEGER(KIND=JPIM) :: JSCALE, INP, INPP1, JN, IORIG, IDATE, ITIME, INBSET,&
 & INSCALES, IOMASTER, ITAG, IARR(1)
INTEGER(KIND=JPIM), ALLOCATABLE :: ICUTOFFS(:),IBUF(:)

CHARACTER :: CLID*10,CLCOM*70,CLFILE*40

TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     -----------------------------------------------------------------

!       Initial setup for wavelet Jb

IF (LHOOK) CALL DR_HOOK('SUJBWAVELET0',0,ZHOOK_HANDLE)

!       Define the wave bands and filters

IF (LJBWSTATS) THEN

  IF (NSMAX>I_DEFAULT_CUTOFFS(1)) THEN
    WRITE (NULOUT) 'Error: NSMAX>I_DEFAULT_CUTOFFS(1) in SUJBWAVELET0'
    CALL ABOR1 ('Error: NSMAX>I_DEFAULT_CUTOFFS(1) in SUJBWAVELET0')
  ENDIF

  N_WAVELET_SCALES=JPNSCALES
  DO WHILE (I_DEFAULT_CUTOFFS(1+JPNSCALES-N_WAVELET_SCALES) > NSMAX)
    N_WAVELET_SCALES = N_WAVELET_SCALES - 1
  ENDDO

  ALLOCATE (N_WAVELET_CUTOFFS(1:N_WAVELET_SCALES+1))

  N_WAVELET_CUTOFFS(1:N_WAVELET_SCALES) =&
   & I_DEFAULT_CUTOFFS(1+JPNSCALES-N_WAVELET_SCALES:JPNSCALES)

  N_WAVELET_CUTOFFS(N_WAVELET_SCALES+1) = 0

  WRITE (NULOUT,*) 'Number of wavelet scales: ',N_WAVELET_SCALES
  WRITE (NULOUT,*) 'Wavelet filter cutoffs: ',  N_WAVELET_CUTOFFS(1:N_WAVELET_SCALES)
  
  ALLOCATE (WAVELET_FILTERS(0:NSMAX,N_WAVELET_SCALES))
  
  DO JSCALE=N_WAVELET_SCALES,1,-1
    INP   = N_WAVELET_CUTOFFS(JSCALE)
    INPP1 = N_WAVELET_CUTOFFS(JSCALE+1)
    WAVELET_FILTERS(0:INPP1    ,JSCALE) = 1.0_JPRB
    WAVELET_FILTERS(INP+1:NSMAX,JSCALE) = 0.0_JPRB
    DO JN=INPP1+1,INP-1
      WAVELET_FILTERS(JN,JSCALE)= REAL(JN-INP,JPRB)/REAL(INPP1-INP,JPRB)
    ENDDO
    WAVELET_FILTERS(INP,JSCALE)= 0._JPRB
  ENDDO
ELSE

!       Open the stats file and check the header

  IOMASTER=1
  ITAG = MTAGFCE+200

  CLFILE='wavelet.cv'
  WRITE (NULOUT,*) '  Opening input file ',CLFILE
  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',CLFILE,CDMODE='r',KIOMASTER=IOMASTER)
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')

  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,CDR=CLID)
  IF (MYPROC==IOMASTER) THEN
    WRITE (NULOUT,*) 'GSA ID=',CLID
    IF (CLID/='WAVELET-07') CALL ABOR1('Bad Jb ID in GSA file')
  ENDIF

  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,CDR=CLCOM)
  IF (MYPROC==IOMASTER) THEN
    WRITE (NULOUT,*) ' Description : ',CLCOM
  ENDIF

  ALLOCATE(IBUF(5))
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
  IF (NPROC>1) THEN
    CALL MPL_BROADCAST(IBUF,KROOT=IOMASTER,KTAG=ITAG, &
      & CDSTRING='SUJBWAVELET0')
    ITAG = ITAG+1
  ENDIF
  IORIG    = IBUF(1)
  IDATE    = IBUF(2)
  ITIME    = IBUF(3)
  INBSET   = IBUF(4)
  INSCALES = IBUF(5)
  WRITE (NULOUT,*) ' Center=',IORIG,' Date=',IDATE,' Time=',ITIME
  DEALLOCATE(IBUF)

!       Read the wavelet filter bands

  ALLOCATE (ICUTOFFS(1:INSCALES+1))
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=ICUTOFFS)

  IF (NPROC>1) THEN
    IF (MYPROC==IOMASTER) IARR(1) = INSCALES
    CALL MPL_BROADCAST(IARR(1:1),KROOT=IOMASTER,KTAG=ITAG, &
      & CDSTRING='SUJBWAVELET0')
    INSCALES = IARR(1)
    ITAG=ITAG+1
    CALL MPL_BROADCAST(ICUTOFFS(1:INSCALES),KROOT=IOMASTER,KTAG=ITAG, &
      & CDSTRING='SUJBWAVELET0')
    ITAG=ITAG+1
  ENDIF

  N_WAVELET_SCALES=INSCALES
  DO WHILE (ICUTOFFS(1+INSCALES-N_WAVELET_SCALES) > NSMAX)
    N_WAVELET_SCALES = N_WAVELET_SCALES - 1
  ENDDO

  WRITE (NULOUT,*) 'Number of wavelet scales: ',N_WAVELET_SCALES

  ALLOCATE(N_WAVELET_CUTOFFS(N_WAVELET_SCALES+1))

  N_WAVELET_CUTOFFS(1:N_WAVELET_SCALES)= &
      & ICUTOFFS(1+INSCALES-N_WAVELET_SCALES:INSCALES)
  N_WAVELET_CUTOFFS(N_WAVELET_SCALES+1) = 0

  WRITE (NULOUT,*) 'Wavelet filter cutoffs: ', &
                 & N_WAVELET_CUTOFFS(1:N_WAVELET_SCALES)

!       Read the wavelet filters

  ALLOCATE (WAVELET_FILTERS(0:NSMAX,N_WAVELET_SCALES))

  DO JSCALE=N_WAVELET_SCALES,1,-1
    INP   = N_WAVELET_CUTOFFS(JSCALE)
    INPP1 = N_WAVELET_CUTOFFS(JSCALE+1)
    WAVELET_FILTERS(0:INPP1    ,JSCALE) = 1.0_JPRB
    WAVELET_FILTERS(INP+1:NSMAX,JSCALE) = 0.0_JPRB
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST, &
            & PR1=WAVELET_FILTERS(INPP1+1:INP,JSCALE))
    IF (NPROC>1) THEN
      CALL MPL_BROADCAST(WAVELET_FILTERS(INPP1+1:INP,JSCALE),KROOT=IOMASTER,KTAG=ITAG, &
       & CDSTRING='SUJBWAVELET0')
      ITAG=ITAG+1
    ENDIF
  ENDDO

  DEALLOCATE(ICUTOFFS)

  !       Close the input file

  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)

ENDIF

IF (LHOOK) CALL DR_HOOK('SUJBWAVELET0',1,ZHOOK_HANDLE)
END SUBROUTINE SUJBWAVELET0
