SUBROUTINE SUJBWAVGEN

!      SUJBWAVGEN - Generate Covariance matrices for wavelet Jb

!     Purpose: Generate statistics for wavelet Jb
!     -------

!     Interface: called from SUJBWAVELET
!     ---------

!     Author: Mike Fisher 98-06-23
!     ------

!     Modifications:
!     -------------
!       M.Hamrud  : 2000-02-02  Removal of MYSENDA etc.
!       M.Fisher  : 2000-08-22  Removal of call to SUJBCOSU. (It spoils
!                               the vertical matrices for large scales.)
!       D.Dent    : 2000-09-01  Convert from MPE to MPL
!       M.Fisher  : 2000-10-26  Distributed the matrices over PEs
!       M.Fisher  : 2001-12-03  Omega-equation balance operator
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!       E.Holm    : 2004-05-24  Add ozone change of variable 
!       K.Yessad  : Nov 2006  Check that the matrix entering EIGENMD is positive
!        definite.
!       E.Holm    : 2007-12-14  Add to FQSIGMA interface (T^b, rh^b) and
!                               change from rh to (q/q_s)_unbal. Simplify
!                               retaining in-memory version only.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN,     ONLY : NULOUT   ,NULERR
USE YOMLAP,     ONLY : NASM0    ,NSPINDXG, MYMS
USE YOMDIM,     ONLY : NFLEVL   ,NSMAX    ,NSPEC2, NFLSUR, &
 & NSPEC2G  ,NFLEVG   ,NPROMA  , NUMP
USE YOMGEM,     ONLY : NGPTOT
USE YOMSPJB,    ONLY : SPJB    , SP7A3, SP7A2
USE YOMJG,      ONLY : BFACT1   ,BFACT2   ,FGSCNM3D  ,FGSCNM2D, FCEIMN ,&
 &NOFCEF   ,FCEMN3D, FCEMN2D, L_JBVCOORD  ,LJB_OMEGA_BALANCE, &
 & LJB_NONLINEAR_BALANCE, LJB_NONLINEAR_CVHUM, SPJB_VARS_INFO, &
 & N_SPJB_VARS, LJB_UNIVARIATE, LJB_TLBALSTAT, RJB_CSLEN
USE YOMCVA,     ONLY : NVA3D    ,NVA2D
USE YOMCT0,     ONLY : NPROC    
USE YOMMPI   , ONLY : MREALT
USE YOMFCEB  , ONLY : FCEBUF   ,BLTOP  
USE YOMWAVELET, ONLY : CINBGSTATES  ,N_BGMEMBERS, N_BGDATES, &
 & N_WAVELET_SCALES, WAVELET_FILTERS, GRID_DEFINITION, WAVELET_VCORS, &
 & WAVELET_VCOR_STRUCT  
USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV, MPL_BARRIER
USE YOMMP,      ONLY : MYPROC, NBSETLEV, NPSP
USE YOMJBCHVAR , ONLY : RQS, RQT, RQTBAL, RQLMIN, RQLMAX,&
 & RQLSTD, RQHMIN, RQHMAX, RQHSTD, RO3MIN, RO3STD
USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT   ,RLSTT    ,RTT      ,RALPW    ,RBETW    ,&
 & RGAMW    ,RALPS   ,RBETS    ,RGAMS    ,RALPD    ,RBETD    ,&
 & RGAMD    ,RA
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,R4IES   ,&
 & R5LES    ,R5IES   ,R5ALVCP  ,R5ALSCP  ,RALVDCP  ,RALSDCP ,&
 & RTWAT    ,RTICE   ,RTICECU  ,RTWAT_RTICE_R, RTWAT_RTICECU_R
USE YOMGRB, ONLY: NGRBVO, NGRBD, NGRBT, NGRBQ, NGRBO3, NGRBLNSP
USE SPECTRAL_FIELDS_MOD

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) ::      j, jlev, inm, jn, jm, jscale, ifl, &
 & jkglo, istc, iendc, jdate, jmember, infcdiffs, &
 & imaxflds, ingpblks, ioff, ibl, icend, igptot, inavg, &
 & ifcebo3, jrof, JFIELD, IPTJB, IPTJBVO, IPTJBT, IPTJBQ, IPTJBO3, &
 & IT, IQ, ISP, IPTFCE, ILEN, &
 & JMLOC, IM, JIR
REAL(KIND=JPRB) :: zndiffs, zpole, zfilt, &
 & ZQP(NPROMA,NFLEVG), ZPRSH(NPROMA,0:NFLEVG), ZPRSF(NPROMA,NFLEVG), &
 & ZQS(NPROMA,NFLEVG), ZRH(NPROMA,NFLEVG), &
 & ZDIF(0:NSMAX), ZGN(0:NSMAX), ZGNSP(0:NSMAX), ZGNGP(0:NSMAX), ZZSP, ZZGP, ZLENSP, ZLENGP
REAL(KIND=JPRB),ALLOCATABLE :: ZZP(:,:),ZGSPA3(:,:,:), ZGSPA2(:,:), &
 & ZSPA3(:,:,:), ZSPA2(:,:), ZFGSCNM(:,:), &
 & ZSPAVG3(:,:,:), ZSPAVG2(:,:), ZPSPEC(:,:),ZFCE(:,:), ZDIV(:,:), &
 & ZSP3MEM(:,:,:,:),ZSP2MEM(:,:,:), ZSP3MEMU(:,:,:,:),ZSP2MEMU(:,:,:),&
 & ZSPMEAN3(:,:,:), ZSPMEAN2(:,:), ZSPDIF3(:,:,:), ZSPDIF2(:,:), &
 & ZGPMEAN3(:,:,:), ZGPMEAN2(:,:,:), ZGPDIF3(:,:,:), ZGPDIF2(:,:,:)
INTEGER(KIND=JPIM),ALLOCATABLE :: ILTROP(:)
CHARACTER (LEN = 16) ::  clfile
LOGICAL :: LLEVI,LLEVN,LLEVZ, LLTLBAL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

                  
!     ------------------------------------------------------------------

INTERFACE
#include "dir_trans.h"
END INTERFACE

#include "abor1.intfb.h"
#include "balomega.intfb.h"
#include "balomegatl.intfb.h"
#include "balnonlin.intfb.h"
#include "balnonlintl.intfb.h"
#include "balstat.intfb.h"
#include "balverti.intfb.h"
#include "copy_spa2spec.intfb.h"
#include "copy_spec2spa.intfb.h"
#include "cvargptl.intfb.h"
#include "eigenmd.intfb.h"
#include "fcttre.h"
#include "fcttrm.h"
#include "fjbchvar.h"
#include "gatherspa.intfb.h"
#include "pregprh.intfb.h"
#include "readvec.intfb.h"
#include "read_spec.intfb.h"
#include "scalefe.intfb.h"
#include "sc2wrg.intfb.h"
#include "sqrtfe.intfb.h"
#include "stepo.intfb.h"
#include "sujbwavvc.intfb.h"
#include "sujbwavwri.intfb.h"
#include "suprffce.intfb.h"
#include "transinv_wavelet.intfb.h"
#include "wavxform.intfb.h"
#include "write_spec.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

9990 FORMAT (' ALLOCATED MEMORY FOR FORECAST DIFFERENCES',8(1X,I8))

IPTJBVO = HUGE(IPTJBVO)
IPTJBT  = HUGE(IPTJBT)
IPTJBQ  = HUGE(IPTJBQ)
ISP     = HUGE(ISP)
DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBVO) THEN
    IPTJBVO = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT) THEN
    IPTJBT  = SPJB_VARS_INFO(JFIELD)%IPTJB
    IT      = 1+NFLEVG*(IPTJBT-1)
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBQ) THEN
    IPTJBQ  = SPJB_VARS_INFO(JFIELD)%IPTJB
    IQ      = 1+NFLEVG*(IPTJBQ-1)
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
    IPTJBO3 = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBLNSP) THEN
    ISP     = SPJB_VARS_INFO(JFIELD)%IPTJB
  ENDIF
ENDDO
  
!       Allocate memory to store the vectors

INFCDIFFS = (N_BGMEMBERS-1)*N_BGDATES

!        spectral memory
ALLOCATE (ZSP3MEM(NFLEVL,NSPEC2,NVA3D,2))
ALLOCATE (ZSP2MEM(NSPEC2,NVA2D,2))
WRITE (NULOUT,9990) SIZE(ZSP3MEM),SHAPE(ZSP3MEM)
WRITE (NULOUT,9990) SIZE(ZSP2MEM),SHAPE(ZSP2MEM)
IF (.NOT. LJB_TLBALSTAT) THEN
  ALLOCATE (ZSP3MEMU(NFLEVL,NSPEC2,NVA3D,2))
  ALLOCATE (ZSP2MEMU(NSPEC2,NVA2D,2))
ENDIF

ALLOCATE (ZSPAVG3(NFLEVL,NSPEC2,NVA3D))
ALLOCATE (ZSPAVG2(NSPEC2,NVA2D))
ZSPAVG3(:,:,:) = 0.0_JPRB
ZSPAVG2  (:,:) = 0.0_JPRB

ALLOCATE (ZSPMEAN3(NFLEVL,NSPEC2,NVA3D))
ALLOCATE (ZSPMEAN2(NSPEC2,NVA2D))
ALLOCATE (ZSPDIF3 (NFLEVL,NSPEC2,NVA3D))
ALLOCATE (ZSPDIF2 (NSPEC2,NVA2D))
IF (L_JBVCOORD) ALLOCATE (BLTOP(NGPTOT))

!        gridpoint memory
IMAXFLDS = NFLEVG*NVA3D+NVA2D
INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA
ALLOCATE (ZGPMEAN3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
ALLOCATE (ZGPMEAN2(NPROMA,NVA2D,INGPBLKS))
ALLOCATE (ZGPDIF3 (NPROMA,NFLEVG*NVA3D,INGPBLKS))
ALLOCATE (ZGPDIF2 (NPROMA,NVA2D,INGPBLKS))
ALLOCATE (ILTROP(NPROMA))

SPJB = 0.0_JPRB

!       Zero FCEBUF
ALLOCATE (ZFCE(NPROMA,NOFCEF))
do jkglo=1,NGPTOT,NPROMA
  istc=1
  iendc=min(NPROMA,NGPTOT-jkglo+1)
  zfce(:,:)=0.0_JPRB
  call SC2WRG (FCEBUF,jkglo,istc,iendc,zfce)
enddo
DEALLOCATE (ZFCE)

!   Low-pass filter coefficients to approximate compact support.
!   For small L, 1-ZGN is equivalent to (1+r/L)exp(-r/L).
ZDIF(:)=1.0_JPRB
ZDIF(NSMAX-10)=0.95_JPRB
ZDIF(NSMAX- 9)=0.90_JPRB
ZDIF(NSMAX- 8)=0.85_JPRB
ZDIF(NSMAX- 7)=0.80_JPRB
ZDIF(NSMAX- 6)=0.70_JPRB
ZDIF(NSMAX- 5)=0.50_JPRB
ZDIF(NSMAX- 4)=0.30_JPRB
ZDIF(NSMAX- 3)=0.20_JPRB
ZDIF(NSMAX- 2)=0.15_JPRB
ZDIF(NSMAX- 1)=0.10_JPRB
ZDIF(NSMAX   )=0.05_JPRB
ZLENGP = RJB_CSLEN
ZLENSP = 5000.E3_JPRB
DO JN=0,NSMAX
  ZZGP = 1.0_JPRB + JN*(JN+1)*(ZLENGP/RA)**2
  ZGNGP(JN) = (1.0_JPRB - 1.0_JPRB / (ZZGP*ZZGP*SQRT(ZZGP)))*ZDIF(JN)
  ZGNGP(JN) = ZDIF(JN)
  ZZSP = 1.0_JPRB + JN*(JN+1)*(ZLENSP/RA)**2
  ZGNSP(JN) = (1.0_JPRB - 1.0_JPRB / (ZZSP*ZZSP*SQRT(ZZSP)))*ZDIF(JN)
  ZGNSP(JN) = ZDIF(JN)
ENDDO


!       Calculate the background differences and the mean background difference

WRITE (NULOUT,*) 'Calculating background differences and their mean'


ifl = 1

! Use full or TL balance operators
LLTLBAL = LJB_TLBALSTAT

DO jdate=1,N_BGDATES
  DO jmember=1,N_BGMEMBERS

!   read a background state

    clfile = CINBGSTATES
    WRITE (clfile(7:16),'(''m'',I3.3,''_d'',I3.3,'' '')') jmember,jdate

    WRITE (NULOUT,*) 'reading member ',jmember,' date ',jdate,' from file ',clfile
    CALL READVEC  (clfile)

!   save current spectral background state

    ZSP3MEM(:,:,:,ifl) = SPJB%SP3D(:,:,:)
    IF (NPSP==1) ZSP2MEM(:,:,ifl)   = SPJB%SP2D(:,:)

!   mean and difference spectral and gridpoint background state

    IF (jmember>1) THEN
      ZSPMEAN3(:,:,:) = 0.5_JPRB*( ZSP3MEM(:,:,:,IFL) + ZSP3MEM(:,:,:,3-IFL) )
      ZSPDIF3 (:,:,:) = ( ZSP3MEM(:,:,:,IFL) - ZSP3MEM(:,:,:,3-IFL) )
      IF (NPSP==1) THEN
        ZSPMEAN2(:,:) = 0.5_JPRB*( ZSP2MEM(:,:,IFL) + ZSP2MEM(:,:,3-IFL) )
        ZSPDIF2 (:,:) = ( ZSP2MEM(:,:,IFL) - ZSP2MEM(:,:,3-IFL) ) 
      ENDIF
!     high-pass filter differences
      DO JFIELD=1,NVA3D
        ZGN(:) = ZGNGP(:)
        IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBVO .OR. &
           &SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBD  .OR. &
           &SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT ) ZGN(:) = ZGNSP(:)
        DO JLEV=1,NFLEVL
          DO JMLOC=1,NUMP
            IM=MYMS(JMLOC)
            DO JIR=0,1
              DO JN=IM,NSMAX
                JM =NASM0 (IM)+(JN-IM)*2+JIR
                ZSPDIF3(JLEV,JM,JFIELD) = ZSPDIF3(JLEV,JM,JFIELD)*ZGN(JN)
                ZSPMEAN3(JLEV,JM,JFIELD) = ZSPMEAN3(JLEV,JM,JFIELD)*ZDIF(JN)
              ENDDO
            ENDDO
          ENDDO
        ENDDO
      ENDDO
      IF (NPSP==1) THEN
        DO JMLOC=1,NUMP
          IM=MYMS(JMLOC)
          DO JIR=0,1
            DO JN=IM,NSMAX
              JM =NASM0 (IM)+(JN-IM)*2+JIR
              ZSPDIF2(JM,:) = ZSPDIF2(JM,:)*ZGNSP(JN)
              ZSPMEAN2(JM,:) = ZSPMEAN2(JM,:)*ZDIF(JN)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
!     transform to gridpoint space
      
      CALL TRANSINV_WAVELET (0, ZSPMEAN3, ZSPMEAN2, ZGPMEAN3, ZGPMEAN2)
      CALL TRANSINV_WAVELET (0, ZSPDIF3 , ZSPDIF2 , ZGPDIF3 , ZGPDIF2)
      DO JKGLO=1,NGPTOT,NPROMA
        ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
        IBL=(JKGLO-1)/NPROMA+1
        DO JLEV=1,NFLEVG
          DO JROF=1,ICEND
            ZGPMEAN3(JROF,IQ+JLEV-1,IBL) = MAX(0._JPRB, ZGPMEAN3(JROF,IQ+JLEV-1,IBL))
            ZGPDIF3 (JROF,IQ+JLEV-1,IBL) = 2.0_JPRB*MIN(    ZGPMEAN3(JROF,IQ+JLEV-1,IBL), &
                                  &     0.5_JPRB*ABS(ZGPDIF3 (JROF,IQ+JLEV-1,IBL)))&
                                  & * (0.5_JPRB+SIGN(0.5_JPRB,ZGPDIF3(JROF,IQ+JLEV-1,IBL)))
          ENDDO
        ENDDO
      ENDDO
    ENDIF

!   calculate pressure on Jb vertical levels

!   EH: Not needed? Can we use the calculation done in pregprh, 
!   maybe including bl-topcalculation there?
    IF (L_JBVCOORD) THEN
      SP7A3(:,:,:) = SPJB%SP3D(:,:,:)
      IF (NPSP==1) SP7A2(:,:) = SPJB%SP2D(:,:)
      CALL ABOR1 ('SUJBWAVGEN: BROKEN: CONVERT JB VERT COORD TO USE SPA3JB/SPA2JB')
      CALL STEPO('0IB00H000')
    ENDIF

!   Apply non-linear change of variable for humidity

    IF (LJB_NONLINEAR_CVHUM) THEN

      IF (jmember>1) THEN

        DO JKGLO=1,NGPTOT,NPROMA
          ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
          IBL=(JKGLO-1)/NPROMA+1

!         Calculate modified rh, tropopause height (and full level pressure)
          CALL PREGPRH (ICEND,ZGPMEAN3(1,IT,IBL), &
                   &          ZGPMEAN3(1,IQ,IBL), &
                   &          ZGPMEAN2(1,ISP,IBL), &
                   &    ZQP,ZPRSH,ZPRSF,ZQS,ZRH,ILTROP)

!         Normalize the difference pair
          DO JLEV=1,NFLEVG
            DO JROF=1,ICEND
              ZGPDIF3(JROF,IQ+JLEV-1,IBL) = ZRH(JROF,JLEV) &
               & *(  ZGPDIF3(JROF,IQ+JLEV-1,IBL)/ZQP(JROF,JLEV) &
               &   - ZGPDIF3(JROF,IT+JLEV-1,IBL) &
               &    *FOETB(ZGPMEAN3(JROF,IT+JLEV-1,IBL)) &
               &    *FQTBAL(JLEV,ILTROP(JROF),ZRH(JROF,JLEV)) )&
               & / FQSIGMA(JLEV,ILTROP(JROF),ZRH(JROF,JLEV),ZRH(JROF,JLEV))
            ENDDO
          ENDDO
        ENDDO

        IF (.NOT.(ASSOCIATED(SPJB%HUM))) THEN
          CALL ABOR1('SUJBWAVGEN: LJB_NONLINEAR_CVHUM REQUIRES Q IN JB')
        ENDIF
        CALL DIR_TRANS (PGP=ZGPDIF3(:,IQ:IQ+NFLEVG-1,:), &
                      & PSPSCALAR=ZSPDIF3(:,:,IPTJBQ), KPROMA=NPROMA, KVSETSC=NBSETLEV)
!       remove wavenumber zero contributions introduced by transform
        DO JLEV=1,NFLEVL
          DO JMLOC=1,NUMP
            IF (MYMS(JMLOC)==0) THEN
              DO JIR=0,1
!               ZSPDIF3(JLEV,JIR,IPTJBQ) = 0.0_JPRB
              ENDDO
            ENDIF
          ENDDO
        ENDDO

      ENDIF

    ENDIF

    IF (.NOT. LLTLBAL) THEN
!   Apply the full balance operator

      IF (.NOT.LJB_UNIVARIATE) THEN
        IF (LJB_OMEGA_BALANCE) THEN
          ALLOCATE (ZDIV(NFLEVL,NSPEC2))
          IF (.NOT.(ASSOCIATED(SPJB%VOR).AND.ASSOCIATED(SPJB%TEM).AND. &
            &       ASSOCIATED(SPJB%DIV))) THEN
            CALL ABOR1('SUJBWAVGEN: LJB_OMEGA_BALANCE REQUIRES VOR, T AND DIV IN JB')
          ENDIF
          CALL BALOMEGA (SPJB%VOR,SPJB%TEM,ZDIV)
          SPJB%DIV(:,:) = SPJB%DIV(:,:) - ZDIV(:,:)
          DEALLOCATE (ZDIV)
        ENDIF

        ALLOCATE (ZZP(NFLEVG,NSPEC2))
        IF (LJB_NONLINEAR_BALANCE) THEN
          IF (.NOT.(ASSOCIATED(SPJB%VOR))) THEN
            CALL ABOR1('SUJBWAVGEN: BALNONLIN REQUIRES VORTICITY IN JB')
          ENDIF
          CALL BALNONLIN(SPJB%VOR,ZZP)
        ELSE
          IF (.NOT.(ASSOCIATED(SPJB%VOR).AND.ASSOCIATED(SPJB%DIV))) THEN
            CALL ABOR1('SUJBWAVGEN: BALSTAT REQUIRES VOR AND DIV IN JB')
          ENDIF
          CALL BALSTAT (NFLEVL,NSMAX,NASM0,NSPEC2,SPJB%VOR,SPJB%DIV, &
                      & ZZP,BFACT1,BFACT2)
        ENDIF

        IF (.NOT.ASSOCIATED(SPJB%TEM).OR. &
          & (NPSP==1.AND..NOT.ASSOCIATED(SPJB%SPR))) THEN
          CALL ABOR1('SUJBWAVGEN: BALVERTI REQUIRES T AND LNSP IN JB')
        ENDIF
        CALL BALVERTI (ZZP,SPJB)

        DEALLOCATE (ZZP)
      ENDIF

!     interpolate onto Jb vertical levels
      IF (L_JBVCOORD) THEN
        CALL ABOR1 ('SUJBWAVGEN: BROKEN: CONVERT TO USE SPA3JB/SPA2JB')
        CALL STEPO ('0AA00NAA0')
      ENDIF

!     save current background state
      ZSP3MEMU(:,:,:,ifl) = SPJB%SP3D(:,:,:)
      IF (NPSP==1) ZSP2MEMU(:,:,ifl)   = SPJB%SP2D(:,:)
      
!     calculate differences
      IF (jmember>1) THEN
        SPJB%SP3D(:,:,:) =   ZSP3MEMU(:,:,:,ifl) - ZSP3MEMU(:,:,:,3-ifl)
        IF (NPSP==1) SPJB%SP2D(:,:) = ZSP2MEMU(:,:,ifl) - ZSP2MEMU(:,:,3-ifl)
        SPJB%SP3D(:,:,IPTJBQ) = ZSPDIF3(:,:,IPTJBQ)
      ENDIF

    ELSEIF (LLTLBAL) THEN
!   Apply the TL balance operator
 
      IF (jmember>1) THEN
        SPJB%SP3D(:,:,:) = ZSPDIF3(:,:,:)
        IF (NPSP==1) SPJB%SP2D(:,:) = ZSPDIF2(:,:)
      ENDIF
      IF (.NOT.LJB_UNIVARIATE) THEN
        IF (LJB_OMEGA_BALANCE) THEN
          ALLOCATE (ZDIV(NFLEVL,NSPEC2))
          IF (.NOT.(ASSOCIATED(SPJB%VOR).AND.ASSOCIATED(SPJB%TEM).AND. &
            &       ASSOCIATED(SPJB%DIV))) THEN
            CALL ABOR1('SUJBWAVGEN: LJB_OMEGA_BALANCE REQUIRES VOR, T AND DIV IN JB')
          ENDIF
          CALL BALOMEGATL (SPJB%VOR,SPJB%TEM,ZSPMEAN3(:,:,IPTJBVO), &
                         & ZSPMEAN3(:,:,IPTJBT),ZDIV)
          SPJB%DIV(:,:) = SPJB%DIV(:,:) - ZDIV(:,:)
          DEALLOCATE (ZDIV)
        ENDIF

        ALLOCATE (ZZP(NFLEVG,NSPEC2))
        IF (LJB_NONLINEAR_BALANCE) THEN
          IF (.NOT.(ASSOCIATED(SPJB%VOR))) THEN
            CALL ABOR1('SUJBWAVGEN: BALNONLINTL REQUIRES VORTICITY IN JB')
          ENDIF
          CALL BALNONLINTL(SPJB%VOR,ZSPMEAN3(:,:,IPTJBVO),ZZP)
        ELSE
          IF (.NOT.(ASSOCIATED(SPJB%VOR).AND.ASSOCIATED(SPJB%DIV))) THEN
            CALL ABOR1('SUJBWAVGEN: BALSTAT REQUIRES VOR AND DIV IN JB')
          ENDIF
          CALL BALSTAT (NFLEVL,NSMAX,NASM0,NSPEC2,SPJB%VOR,SPJB%DIV, &
                      & ZZP,BFACT1,BFACT2)
        ENDIF

        IF (.NOT.ASSOCIATED(SPJB%TEM).OR. &
          & (NPSP==1.AND..NOT.ASSOCIATED(SPJB%SPR))) THEN
          CALL ABOR1('SUJBWAVGEN: BALVERTI REQUIRES T AND LNSP IN JB')
        ENDIF
        CALL BALVERTI (ZZP,SPJB)

        DEALLOCATE (ZZP)
      ENDIF

!     interpolate onto Jb vertical levels
      IF (L_JBVCOORD) THEN
        CALL ABOR1 ('SUJBWAVGEN: BROKEN: CONVERT TO USE SPA3JB/SPA2JB')
        CALL STEPO ('0AA00NAA0')
      ENDIF
    ENDIF
   
!   Update mean difference using the running-mean formula & write differences 

    IF (jmember>1) THEN
      inavg = jmember-1  + (N_BGMEMBERS-1)*(jdate-1)
      ZSPAVG3(:,:,:) = ZSPAVG3(:,:,:) &
                    & +(SPJB%SP3D(:,:,:)-ZSPAVG3(:,:,:))/REAL(inavg,JPRB)
      IF (NPSP==1) ZSPAVG2  (:,:) = ZSPAVG2  (:,:) &
                    & +(SPJB%SP2D(:,:)  -ZSPAVG2(:,:)  )/REAL(inavg,JPRB)
      CALL WRITETMP (inavg)
    ENDIF
    ifl = 3-ifl
  ENDDO
ENDDO

DEALLOCATE (ZSP3MEM)
DEALLOCATE (ZSP2MEM)
IF (ALLOCATED(ZSP3MEMU)) DEALLOCATE (ZSP3MEMU)
IF (ALLOCATED(ZSP2MEMU)) DEALLOCATE (ZSP2MEMU)
DEALLOCATE (ZSPMEAN3)
DEALLOCATE (ZSPMEAN2)
DEALLOCATE (ZSPDIF3)
DEALLOCATE (ZSPDIF2)
DEALLOCATE (ZGPMEAN3)
DEALLOCATE (ZGPMEAN2)
DEALLOCATE (ZGPDIF3)
DEALLOCATE (ZGPDIF2)
DEALLOCATE (ILTROP)
WRITE (NULOUT,'('' DEALLOCATED Z/SP/2/3/MEM, Z/SP/GP/MEAN/DIF/2/3 '')')

!       Subtract the mean difference from the differences,
!       and calculate the gridpoint standard deviations

WRITE (NULOUT,*) 'Calculate Sample Gridpoint Standard Deviations'


DO j=1,INFCDIFFS
  CALL READTMP (j) ! initialise SPJB%SP3D and SPJB%SP2D

  SPJB%SP3D(:,:,:) = SPJB%SP3D(:,:,:) - ZSPAVG3(:,:,:)
  IF (NPSP==1) SPJB%SP2D(:,:)   = SPJB%SP2D(:,:)   - ZSPAVG2(:,:)
  SPJB%SP1D(:,:)   = 0.0_JPRB

  CALL WRITETMP (j)
  
  CALL CVARGPTL ('S',0,SPJB%SP3D,SPJB%SP2D,SPJB%SP1D)
ENDDO

DEALLOCATE (ZSPAVG3)
DEALLOCATE (ZSPAVG2)

zndiffs=1.0_JPRB/REAL(INFCDIFFS-1,JPRB)
CALL SCALEFE (zndiffs)
CALL SQRTFE

!       Calculate the horizontal correlations and vertical covariances

WRITE (NULOUT,*) 'Calculate Horizontal and Vertical Correlations'

ILEN=0
DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    ILEN=ILEN+1
  ELSE
    ILEN=ILEN+NFLEVG
  ENDIF
ENDDO
ALLOCATE (ZFGSCNM(ILEN,0:NSMAX))
ZFGSCNM(:,:) = 0.0_JPRB

DO j=1,INFCDIFFS
  ifl = 1
  CALL READTMP (j)

  SPJB%SP1D(:,:)   = 0.0_JPRB

!   normalize background difference by background error standard deviation

  CALL CVARGPTL ('G',0,SPJB%SP3D,SPJB%SP2D,SPJB%SP1D)

!   accumulate the power spectrum

  ALLOCATE (ZGSPA3(NFLEVG,NSPEC2G,NVA3D))
  ALLOCATE (ZGSPA2(NSPEC2G,NVA2D))

  IF (NPROC > 1) THEN
    CALL GATHERSPA (ZGSPA3,ZGSPA2,SPJB%SP3D,SPJB%SP2D)
  ELSE
    ZGSPA3(:,:,:) = SPJB%SP3D(:,:,:)
    IF (NPSP==1) ZGSPA2(:,:) = SPJB%SP2D(:,:)
  ENDIF

  IOFF=0
  DO JFIELD=1,N_SPJB_VARS
    IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB

    IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
      inm=0
      DO jn=0,NSMAX
        DO jm=-jn,jn
          inm=inm+1
          ZFGSCNM(IOFF+1,jn) = ZFGSCNM(IOFF+1,jn)&
           & + (1.0_JPRB+REAL(MIN(1,ABS(jm)),JPRB) )&
           & *(ZGSPA2(NSPINDXG(inm),IPTJB)**2)  
        ENDDO
      ENDDO
      IOFF=IOFF+1
    ELSE
      DO jlev=1,NFLEVG
        inm=0
        DO jn=0,NSMAX
          DO jm=-jn,jn
            inm=inm+1
            ZFGSCNM(IOFF+jlev,jn) = ZFGSCNM(IOFF+jlev,jn)&
             & + (1.0_JPRB+REAL(MIN(1,ABS(jm)),JPRB) )&
             & *(ZGSPA3(jlev,NSPINDXG(inm),IPTJB)**2)  
          ENDDO
        ENDDO
      ENDDO
      IOFF=IOFF+NFLEVG
    ENDIF
  ENDDO

  DEALLOCATE (ZGSPA3)
  DEALLOCATE (ZGSPA2)

!   calculate the vertical covariance matrices 

  ALLOCATE (ZSPA3(NFLSUR,NSPEC2,NVA3D))
  ALLOCATE (ZSPA2(NSPEC2,NVA2D))

  ZSPA3(:,:,:) = SPJB%SP3D(:,:,:)
  IF (NPSP==1) ZSPA2(:,:) = SPJB%SP2D(:,:)

  DO JSCALE=1,N_WAVELET_SCALES+1
    SPJB%SP3D(:,:,:) = ZSPA3(:,:,:)
    IF (NPSP==1) SPJB%SP2D(:,:) = ZSPA2(:,:)
    CALL WAVXFORM (JSCALE,SPJB%SP3D,SPJB%SP2D)

    zpole = 0._JPRB
    DO jn=0,NSMAX
      IF (JSCALE > N_WAVELET_SCALES) THEN
        zfilt = WAVELET_FILTERS(JN,N_WAVELET_SCALES)
      ELSEIF (JSCALE == 1) THEN
        zfilt = 1.0_JPRB-WAVELET_FILTERS(JN,JSCALE)
      ELSE
        zfilt = (1.0_JPRB-WAVELET_FILTERS(JN,JSCALE))&
         & *WAVELET_FILTERS(JN,JSCALE-1)  
      ENDIF
      zpole = zpole + zfilt*REAL( 2*JN +1, JPRB)
    ENDDO
    zpole = 1.0_JPRB/SQRT(zpole)

    SPJB%SP3D(:,:,:) = SPJB%SP3D(:,:,:) * zpole
    IF (NPSP==1) SPJB%SP2D(:,:) = SPJB%SP2D(:,:) * zpole

!!  This should be faster, but is very slow!
!!    IF (WAVELET_VCORS(JSCALE,1)%NPTOTG == 1) THEN
!!      CALL SUJBWAVVC_SP (JSCALE)
!!    ELSE
      IGPTOT = GRID_DEFINITION(JSCALE)%NGPTOT
      IMAXFLDS = NFLEVG*NVA3D+NVA2D
      INGPBLKS = (IGPTOT+NPROMA-1)/NPROMA

      ALLOCATE (ZGPDIF3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
      ALLOCATE (ZGPDIF2(NPROMA,NVA2D,INGPBLKS))

      CALL TRANSINV_WAVELET (JSCALE, SPJB%SP3D, SPJB%SP2D, ZGPDIF3, ZGPDIF2)

      DO JKGLO=1,IGPTOT,NPROMA
        ICEND=MIN(NPROMA,IGPTOT-JKGLO+1)
        IBL=(JKGLO-1)/NPROMA+1
        IOFF=JKGLO

        CALL SUJBWAVVC (JSCALE,ICEND, &
         & GRID_DEFINITION(JSCALE)%GELAM(IOFF:),&
         & GRID_DEFINITION(JSCALE)%GELAT(IOFF:),&
         & ZGPDIF3(1,1,IBL), ZGPDIF2(1,1,IBL))
      ENDDO

      DEALLOCATE (ZGPDIF2)
      DEALLOCATE (ZGPDIF3)
!!    ENDIF

    CALL MPL_BARRIER (CDSTRING='SUJBWAVGEN')

  ENDDO

  DEALLOCATE (ZSPA3)
  DEALLOCATE (ZSPA2)
ENDDO

!      Normalize to get the correlation power spectrum

DO j=1,ILEN
  zpole = 0._JPRB
  DO jn=0,NSMAX
    zpole = zpole + ZFGSCNM(j,jn)
  ENDDO
  ZFGSCNM(j,:) = ZFGSCNM(j,:) / zpole
ENDDO

!       Calculate the global mean variance

CALL SUPRFFCE

IFCEBO3 = HUGE(IFCEBO3)
DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
    IFCEBO3 = SPJB_VARS_INFO(JFIELD)%IPTFCE
  ENDIF
ENDDO

DO JFIELD=1,N_SPJB_VARS
  IPTJB  = SPJB_VARS_INFO(JFIELD)%IPTJB
  IPTFCE = SPJB_VARS_INFO(JFIELD)%IPTFCE

  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    FCEMN2D(IPTJB)   = FCEIMN(IPTFCE)
  ELSE
    FCEMN3D(:,IPTJB) = FCEIMN(IPTFCE:IPTFCE+NFLEVG-1)
  ENDIF
ENDDO

!       Calculate the covariance power spectrum as the product of the
!       correlation power spectrum and the global mean variance

ALLOCATE (ZPSPEC(SIZE(ZFGSCNM,1),0:NSMAX))

IOFF=0
DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB

  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    ZPSPEC(1+IOFF,:) = ZFGSCNM(1+IOFF,:) * FCEMN2D(IPTJB)**2
    IOFF=IOFF+1
  ELSE
    DO jlev=1,NFLEVG
      ZPSPEC(jlev+IOFF,:) = ZFGSCNM(jlev+IOFF,:) * FCEMN3D(jlev,IPTJB)**2
    ENDDO
    IOFF=IOFF+NFLEVG
  ENDIF
ENDDO

!       Convert the correlation power spectrum to a modal spectrum,
!       and take the inverse square root - as required by JGHCOR.

DO j=1,ILEN
  DO jn=0,NSMAX
    ZFGSCNM(j,jn) = SQRT( REAL(2*jn+1,JPRB) / ZFGSCNM(j,jn) )
  ENDDO
ENDDO

!       Copy ZFGSCNM to their proper places in FGSCNM

IOFF=0
DO JFIELD=1,N_SPJB_VARS
  IPTJB = SPJB_VARS_INFO(JFIELD)%IPTJB

  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    FGSCNM2D(:,IPTJB) = ZFGSCNM(IOFF+1,:)
    IOFF=IOFF+1
  ELSE
    FGSCNM3D(1:NFLEVG,:,IPTJB) = ZFGSCNM(IOFF+1:IOFF+NFLEVG,:)
    IOFF=IOFF+NFLEVG
  ENDIF
ENDDO

DEALLOCATE (ZFGSCNM)


IF (ALLOCATED(BLTOP)) DEALLOCATE (BLTOP)

!       Covariance matrices have been calculated on each PE from the
!       grid columns owned by the PE. Merge the matrices to get the
!       covariance calculated from all contributing grid points.
!       Then calculate eigenvalues and eigenvectors.

DO JFIELD=1,N_SPJB_VARS
  DO JSCALE=1,N_WAVELET_SCALES+1
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE/=NGRBLNSP) THEN
      CALL MERGE_VCOR          (WAVELET_VCORS(JSCALE,JFIELD))
      CALL EIGENDECOMPOSE_VCOR (WAVELET_VCORS(JSCALE,JFIELD))
    ENDIF
  ENDDO
ENDDO

!       Write out the background error statistics

CALL SUJBWAVWRI (ZPSPEC)

DEALLOCATE (ZPSPEC)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN',1,ZHOOK_HANDLE)

CONTAINS

!     ------------------------------------------------------------------

SUBROUTINE MERGE_VCOR (YD_CV)

!  MERGE_VCOR - Merges covariance matrices calculated by different PEs.

USE YOMMP,      ONLY : MYPROC, NPRCIDS
USE YOMMPI,     ONLY : MREALT
USE YOMCT0,     ONLY : NPROC
USE YOMTAG,     ONLY : MTAGFCE

TYPE (WAVELET_VCOR_STRUCT), INTENT(INOUT) :: YD_CV

REAL(KIND=JPRB) :: ZMSG(1+YD_CV%NLEVS*(YD_CV%NLEVS+1))
INTEGER(KIND=JPIM) :: J, JPROC, ITAG, IL, INY, INX, JI, JJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!        Loop over all matrices.

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:MERGE_VCOR',0,ZHOOK_HANDLE)
DO J=1,YD_CV%NPTOTG
  IF (YD_CV%LPELIST(J,MYPROC)) THEN
    IF (YD_CV%MASTERPE(J)/=MYPROC) THEN
!        Send my contribution to the master.
      ITAG=MTAGFCE+J
      DO JJ=1,YD_CV%NLEVS
        DO JI=1,YD_CV%NLEVS
          ZMSG(JI+(JJ-1)*YD_CV%NLEVS) = YD_CV%MATS(JI,JJ,YD_CV%INDEXG2L(J))
        ENDDO
        ZMSG(JJ+YD_CV%NLEVS*YD_CV%NLEVS) = YD_CV%SAMPAV(JJ,YD_CV%INDEXG2L(J))
      ENDDO
      ZMSG(1+YD_CV%NLEVS*(YD_CV%NLEVS+1)) = YD_CV%NSAMPS(YD_CV%INDEXG2L(J))
      CALL MPL_SEND (ZMSG(:),KDEST=NPRCIDS(YD_CV%MASTERPE(J)), &
       & KTAG=ITAG, CDSTRING='SUJBWAVGEN:MERGE_VCOR')  
    ELSE

!        Recieve contributions and update the matrix.
      ITAG=MTAGFCE+J
      DO JPROC=1,NPROC
        IF (JPROC/=MYPROC .AND. YD_CV%LPELIST(J,JPROC)) THEN
          CALL MPL_RECV (ZMSG(:),KSOURCE=NPRCIDS(JPROC), KTAG=ITAG, &
           & CDSTRING='SUJBWAVGEN:MERGE_VCOR')  
          IL = YD_CV%INDEXG2L(J)
          INX = YD_CV%NSAMPS(IL)
          INY = NINT(ZMSG(1+YD_CV%NLEVS*(YD_CV%NLEVS+1)))

          IF (INY > 0) THEN
            DO JJ=1,YD_CV%NLEVS
              DO JI=1,YD_CV%NLEVS
                YD_CV%MATS (JI,JJ,IL) = &
                 & ( REAL(INX,JPRB)*YD_CV%MATS (JI,JJ,IL)       &
                 & +REAL(INY,JPRB)*ZMSG(JI+(JJ-1)*YD_CV%NLEVS) &
                 & ) / REAL(INX+INY,JPRB)                     &
                 & + (REAL(INX*INY,JPRB)/REAL((INX+INY)*(INX+INY),JPRB)) &
                 & *(YD_CV%SAMPAV(JI,IL) - ZMSG(JI+YD_CV%NLEVS*YD_CV%NLEVS)) &
                 & *(YD_CV%SAMPAV(JJ,IL) - ZMSG(JJ+YD_CV%NLEVS*YD_CV%NLEVS))  
              ENDDO
            ENDDO

            DO JJ=1,YD_CV%NLEVS
              YD_CV%SAMPAV(JJ,IL) = &
               & ( REAL(INX,JPRB)*YD_CV%SAMPAV(JJ,IL)             &
               & +REAL(INY,JPRB)*ZMSG(JJ+YD_CV%NLEVS*YD_CV%NLEVS)  &
               & ) / REAL(INX+INY,JPRB)  
            ENDDO

          ENDIF

          YD_CV%NSAMPS(IL) = YD_CV%NSAMPS(IL) + INY
        ENDIF
      ENDDO
    ENDIF
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:MERGE_VCOR',1,ZHOOK_HANDLE)
END SUBROUTINE MERGE_VCOR

!     ------------------------------------------------------------------

SUBROUTINE EIGENDECOMPOSE_VCOR (YD_CV)

!           Calculate the eigendecomposition of the covariance matrices.
!           Only the master PE for each matrix does this.

USE YOMMP,      ONLY : MYPROC, NPRCIDS

TYPE (WAVELET_VCOR_STRUCT), INTENT(INOUT) :: YD_CV

INTEGER(KIND=JPIM) :: JMAT, JJ, JG

REAL(KIND=JPRB) :: ZMAT(YD_CV%NLEVS,YD_CV%NLEVS),ZMODES(YD_CV%NLEVS,YD_CV%NLEVS),&
 & ZEVALS(YD_CV%NLEVS)  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "eigenmd.intfb.h"

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:EIGENDECOMPOSE_VCOR',0,ZHOOK_HANDLE)
DO JG=1,YD_CV%NPTOTG
  IF (YD_CV%MASTERPE(JG)==MYPROC) THEN
    JMAT=YD_CV%INDEXG2L(JG)

    IF (YD_CV%NSAMPS(JMAT)<YD_CV%NLEVS) THEN
      WRITE (NULOUT,*) 'SAMPLE SIZE TOO SMALL. SINGULAR COVARIANCE MATRIX'
      WRITE (NULOUT,*) 'GLOBAL MATRIX NUMBER=',JG, &
       & 'SAMPLE SIZE=',YD_CV%NSAMPS(JMAT)  
      CALL ABOR1 ('SUJBWAVGEN:EIGENDECOMPOSE_VCOR: SAMPLE SIZE TOO SMALL. SINGULAR COVARIANCE MATRIX')
    ENDIF

    ZMAT(:,:) = YD_CV%MATS(:,:,JMAT)
    CALL EIGENMD(ZMAT,ZMODES,ZEVALS,YD_CV%NLEVS,LLEVI,LLEVN,LLEVZ)

    IF (LLEVI) THEN
      WRITE(NULERR,*) ' SUJBWAVGEN:EIGENDECOMPOSE_VCOR: ZMAT has at least one not real eigenvalue.'
      WRITE(NULERR,*) ' for JG =',JG
    ENDIF
    IF (LLEVI) CALL FLUSH(NULERR)
    IF (LLEVI) CALL ABOR1(' SUJBWAVGEN:EIGENDECOMPOSE_VCOR: ABOR1 nr 1a')

    ! * case LLEVN.OR.LLEVZ:
    IF (MINVAL(ZEVALS(:)) <= 0.0_JPRB) THEN
      WRITE (NULOUT,*) 'COVAR. MATRIX NOT POSITIVE DEFINITE'
      WRITE (NULOUT,*) 'SCALE=',JSCALE,' GLOBAL MATRIX NUMBER=',JG, &
       & ' SAMPLE SIZE=',YD_CV%NSAMPS(JMAT), &
       & ' EIGENVALUES=',ZEVALS(:)  
      DO JJ=1,YD_CV%NLEVS
        WRITE (NULOUT,*) YD_CV%MATS(JJ,:,JMAT)
      ENDDO
      CALL ABOR1('SUJBWAVGEN:EIGENDECOMPOSE_VCOR: COVARIANCE MATRIX NOT POSITIVE DEFINITE')
    ENDIF

    YD_CV%MATS(:,:,JMAT)=ZMODES(:,:)
    YD_CV%EVALS(:,JMAT) =ZEVALS(:)
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:EIGENDECOMPOSE_VCOR',1,ZHOOK_HANDLE)

END SUBROUTINE EIGENDECOMPOSE_VCOR

!     ------------------------------------------------------------------

SUBROUTINE WRITETMP (kfl)

! Write SPJB%SP3D(:,:,:) and SPJB%SP2D(:,:) to temporary files

USE SPECTRAL_FIELDS_MOD, ONLY : SPECTRAL_FIELD, ALLOCATE_SPEC, DEALLOCATE_SPEC

INTEGER(KIND=JPIM), INTENT(IN) ::kfl

INTEGER(KIND=JPIM) :: igribsp(NVA3D+NVA2D),jfield,i
REAL(KIND=JPRB)    :: ZSP1D(NFLEVL,0)

TYPE(SPECTRAL_FIELD) :: YL_SPEC

CHARACTER (LEN = 16) ::  CLTMPFILE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:WRITETMP',0,ZHOOK_HANDLE)

i=1
do jfield=1,N_SPJB_VARS
  IF (.NOT.SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    igribsp(i) = SPJB_VARS_INFO(JFIELD)%IGRIBCODE
    i=i+1
  ENDIF
enddo
do jfield=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    igribsp(i) = SPJB_VARS_INFO(JFIELD)%IGRIBCODE
    i=i+1
  ENDIF
enddo

CLTMPFILE = 'sujbwavgen_DNNN_'
WRITE (CLTMPFILE(13:15),'(I3.3)') kfl

CALL ALLOCATE_SPEC (YL_SPEC,NSMAX,NSMAX,NVA3D,NVA2D,0,igribsp)
CALl COPY_SPA2SPEC (NVA3D,NVA2D,0,SPJB%SP3D(:,:,:),SPJB%SP2D(:,:), &
                  & ZSP1D,YL_SPEC)
CALL WRITE_SPEC (CLTMPFILE, YL_SPEC, .TRUE., LDGRIB=.false.)
CALL DEALLOCATE_SPEC(YL_SPEC)

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:WRITETMP',1,ZHOOK_HANDLE)

END SUBROUTINE WRITETMP

!     ------------------------------------------------------------------

SUBROUTINE READTMP (kfl)

! Read SPJB%SP3D(:,:,:) and SPJB%SP2D(:,:) to temporary files

USE YOMGRB, ONLY: NGRBVO, NGRBD, NGRBT, NGRBQ, NGRBO3, NGRBLNSP
USE SPECTRAL_FIELDS_MOD, ONLY : SPECTRAL_FIELD, ALLOCATE_SPEC, DEALLOCATE_SPEC

INTEGER(KIND=JPIM), INTENT(IN) :: kfl

INTEGER(KIND=JPIM) :: igribsp(NVA3D+NVA2D),jfield,i
REAL(KIND=JPRB)    :: ZSP1D(NFLEVL,0)

TYPE(SPECTRAL_FIELD) :: YL_SPEC

CHARACTER (LEN = 16) ::  CLTMPFILE
INTEGER(KIND=JPIM) :: ISTEP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:READTMP',0,ZHOOK_HANDLE)

i=1
do jfield=1,N_SPJB_VARS
  IF (.NOT.SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    igribsp(i) = SPJB_VARS_INFO(JFIELD)%IGRIBCODE
    i=i+1
  ENDIF
enddo
do jfield=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    igribsp(i) = SPJB_VARS_INFO(JFIELD)%IGRIBCODE
    i=i+1
  ENDIF
enddo

CLTMPFILE = 'sujbwavgen_DNNN_'
WRITE (CLTMPFILE(13:15),'(I3.3)') kfl

CALL ALLOCATE_SPEC (YL_SPEC,NSMAX,NSMAX,NVA3D,NVA2D,0,igribsp)
ISTEP=1
CALL READ_SPEC (CLTMPFILE, YL_SPEC, ISTEP, LDGRIB=.false.)
CALl COPY_SPEC2SPA (NVA3D,NVA2D,0,YL_SPEC,SPJB%SP3D(:,:,:), &
                  & SPJB%SP2D(:,:),ZSP1D)
CALL DEALLOCATE_SPEC(YL_SPEC)

IF (LHOOK) CALL DR_HOOK('SUJBWAVGEN:READTMP',1,ZHOOK_HANDLE)
END SUBROUTINE READTMP

!     ------------------------------------------------------------------
END SUBROUTINE SUJBWAVGEN
