SUBROUTINE SUJBWAVWRI (PPSPEC)

!      SUJBWAVWRI - Write Covariance matrices for wavelet Jb

!     Purpose: Write statistics for wavelet Jb
!     -------

!     Interface: called from SUJBWAVELET
!     ---------

!     Author: Mike Fisher 98-06-30
!     ------

!     Modifications:
!     -------------
!     00-10-26 M. Fisher - Distribute the matrices.
!     00-10-26 M. Fisher - Write eigendecomposition to make startup faster
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   2007-07-31 M. Fisher - Write "skylined" matrices to reduce file size
!   2007-08-06 M. Fisher - Use IOSTREAM_MIX

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP,      ONLY : MYPROC
USE YOMDIM,     ONLY : NFLEVG, NSMAX
USE YOMLUN,     ONLY : NULOUT
USE YOMWAVELET, ONLY : WAVELET_VCORS, WAVELET_VCOR_STRUCT, N_WAVELET_SCALES, &
                     & N_WAVELET_CUTOFFS, WAVELET_FILTERS, SKYLINE_TOL
USE YOMJG, ONLY: SPJB_VARS_INFO, N_SPJB_VARS, &
               & LJB_UNIVARIATE
USE YOMGRB , ONLY : NGRBT, NGRBLNSP
USE IOSTREAM_MIX, ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_PUT, &
                   & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, &
                   & CLOSE_IOREQUEST

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPSPEC(:,0:) 
CHARACTER (LEN=40) :: CLFILE
CHARACTER (LEN=10) :: CLID
CHARACTER (LEN=70) :: CLCOM
CHARACTER (LEN=40) :: CLVAR
CHARACTER (LEN=8)  :: CLDATE
INTEGER(KIND=JPIM) :: IORIG,IDATE,ITIME,INBSET,IOMASTER,JSCALE,&
                    & INP,INPP1,ICUTOFF,IOFF,JFIELD,ILEFT,IRIGHT,&
                    & JI,JJ,JK,JMAT
INTEGER(KIND=JPIM), ALLOCATABLE :: IBUF(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZBUF(:), ZBUF2(:,:)
REAL(KIND=JPRB) :: ZSQRT

TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUJBWAVWRI',0,ZHOOK_HANDLE)

IOMASTER=1

!     Write the file header and the wavelet filters


CLFILE='wavelet.cv'
WRITE (NULOUT,*) '  Opening output file for Jb ',CLFILE

CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',CLFILE,CDMODE='w',KIOMASTER=IOMASTER)
CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')

CLID  ='WAVELET-07'
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,CDR=CLID)

CLCOM ='Vertical covariance matrices for wavelet Jb'
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,CDR=CLCOM)

CALL DATE_AND_TIME (DATE=CLDATE)
IF (CLDATE=='        ') THEN
  IDATE=0
ELSE
  READ (CLDATE,'(I8)') IDATE
ENDIF

IORIG =98
ITIME =0000
INBSET=-999

ALLOCATE(IBUF(5))
IBUF(1)=IORIG
IBUF(2)=IDATE
IBUF(3)=ITIME
IBUF(4)=INBSET
IBUF(5)=N_WAVELET_SCALES
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
DEALLOCATE(IBUF)

CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST, &
          & KR1=N_WAVELET_CUTOFFS(1:N_WAVELET_SCALES+1))

DO JSCALE=N_WAVELET_SCALES,1,-1
  INP   = N_WAVELET_CUTOFFS(JSCALE)
  INPP1 = N_WAVELET_CUTOFFS(JSCALE+1)
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST, &
            & PR1=WAVELET_FILTERS(INPP1+1:INP,JSCALE))
ENDDO

!     Write the horizontal power spectra

IOFF=0
DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBLNSP) THEN
      IF (LJB_UNIVARIATE) THEN
        CLVAR='LNSP'
      ELSE
        CLVAR='UNBALANCED LNSP'
      ENDIF
    ELSE
      CLVAR=SPJB_VARS_INFO(JFIELD)%COR_STRING
    ENDIF

    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,CDR=CLVAR)

    ALLOCATE(IBUF(2))
    IBUF(1)=NSMAX
    IBUF(2)=1
    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    DEALLOCATE(IBUF)

    ALLOCATE(ZBUF(0:NSMAX))
    ZBUF(:)=PPSPEC(IOFF+1,:)
    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=ZBUF)
    DEALLOCATE(ZBUF)

    IOFF=IOFF+1
  ELSE
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT) THEN
      IF (LJB_UNIVARIATE) THEN
        CLVAR='TEMPERATURE'
      ELSE
        CLVAR='UNBALANCED TEMPERATURE'
      ENDIF
    ELSE
      CLVAR=SPJB_VARS_INFO(JFIELD)%COR_STRING
    ENDIF

    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,CDR=CLVAR)

    ALLOCATE(IBUF(2))
    IBUF(1)=NSMAX
    IBUF(2)=NFLEVG
    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    DEALLOCATE(IBUF)

    ALLOCATE(ZBUF2(NFLEVG,0:NSMAX))
    ZBUF2(:,:)=PPSPEC(IOFF+1:IOFF+NFLEVG,:)
    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR2=ZBUF2)
    DEALLOCATE(ZBUF2)

    IOFF=IOFF+NFLEVG
  ENDIF
ENDDO

!      Write the vertical correlation matrices

DO JSCALE=N_WAVELET_SCALES+1,1,-1
  ICUTOFF = N_WAVELET_CUTOFFS(JSCALE)
  DO JFIELD=1,N_SPJB_VARS
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE/=NGRBLNSP) THEN
      CALL WRITE_VCOR (WAVELET_VCORS(JSCALE,JFIELD),ICUTOFF, &
                     &SPJB_VARS_INFO(JFIELD)%COR_STRING)
    ENDIF
  ENDDO
ENDDO

CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)

WRITE (NULOUT,*) '  Finished writing file ',CLFILE

IF (LHOOK) CALL DR_HOOK('SUJBWAVWRI',1,ZHOOK_HANDLE)

CONTAINS
SUBROUTINE WRITE_VCOR (YD_PVC, KCUTOFF, CDID)

USE YOMMP,      ONLY : MYPROC, NPRCIDS
USE YOMTAG,     ONLY : MTAGFCE
USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV, MPL_BARRIER

TYPE (WAVELET_VCOR_STRUCT), INTENT(INOUT) :: YD_PVC
INTEGER(KIND=JPIM),                    INTENT(IN) :: KCUTOFF
CHARACTER (LEN=*),          INTENT(IN) :: CDID

REAL(KIND=JPRB)    :: ZMSG(YD_PVC%NLEVS,YD_PVC%NLEVS+2,2), &
                    & ZMSG1D(YD_PVC%NLEVS*(YD_PVC%NLEVS+2)*2)
REAL(KIND=JPRB)    :: ZMODES (YD_PVC%NLEVS,YD_PVC%NLEVS), &
                    & ZMAT   (YD_PVC%NLEVS,YD_PVC%NLEVS), &
                    & ZMATINV(YD_PVC%NLEVS,YD_PVC%NLEVS)

INTEGER(KIND=JPIM) :: I_MAT_SKYLINE_LEFT    (YD_PVC%NLEVS), &
         & I_MAT_SKYLINE_RIGHT   (YD_PVC%NLEVS), &
         & I_MATINV_SKYLINE_LEFT (YD_PVC%NLEVS), &
         & I_MATINV_SKYLINE_RIGHT(YD_PVC%NLEVS)

INTEGER(KIND=JPIM) :: J, I, ITAG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUJBWAVWRI:WRITE_VCOR',0,ZHOOK_HANDLE)

!---- Write header information

CLVAR=CDID
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,CDR=CLVAR)

ALLOCATE(IBUF(3))
IBUF(1)= KCUTOFF
IBUF(2)= YD_PVC%NLATSG
IBUF(3)= YD_PVC%NLEVS
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
DEALLOCATE(IBUF)

CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=YD_PVC%LATSG)
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=YD_PVC%NLONSG)
CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=YD_PVC%LON0G)

DO J=1,YD_PVC%NPTOTG

!---- Generate the matrices and inverse matrices from their eigendecompositions

  IF (YD_PVC%MASTERPE(J)==MYPROC) THEN
    JMAT=YD_PVC%INDEXG2L(J)

    ZMODES(:,:) = YD_PVC%MATS(:,:,JMAT)

    YD_PVC%MATS   (:,:,JMAT) = 0.0_JPRB
    YD_PVC%MATINVS(:,:,JMAT) = 0.0_JPRB

    DO JK=1,YD_PVC%NLEVS
      ZSQRT = SQRT(YD_PVC%EVALS(JK,JMAT))
      DO JJ=1,YD_PVC%NLEVS
!OCL NOVREC
        DO JI=1,YD_PVC%NLEVS
          YD_PVC%MATS(JI,JJ,JMAT)    =  YD_PVC%MATS(JI,JJ,JMAT)&
           & + ZMODES(JI,JK)*ZMODES(JJ,JK) * (1.0_JPRB/ZSQRT)
          YD_PVC%MATINVS(JI,JJ,JMAT) =  YD_PVC%MATINVS(JI,JJ,JMAT)&
           & + ZMODES(JI,JK)*ZMODES(JJ,JK) * ZSQRT  
        ENDDO
      ENDDO
    ENDDO

!---- Construct matrix skylines

    DO JK=1,YD_PVC%NLEVS
      YD_PVC%MATS_SKYLINE_LEFT(JK,JMAT)=1
      DO JJ=1,JK-1
        IF (ABS(YD_PVC%MATS(JJ,JK,JMAT)) &
          & >SKYLINE_TOL*YD_PVC%MATS(JK,JK,JMAT)) EXIT
        YD_PVC%MATS_SKYLINE_LEFT(JK,JMAT)=YD_PVC%MATS_SKYLINE_LEFT(JK,JMAT)+1
      ENDDO

      YD_PVC%MATS_SKYLINE_RIGHT(JK,JMAT)=YD_PVC%NLEVS
      DO JJ=YD_PVC%NLEVS,JK+1,-1
        IF (ABS(YD_PVC%MATS(JJ,JK,JMAT)) &
          & >SKYLINE_TOL*YD_PVC%MATS(JK,JK,JMAT)) EXIT
        YD_PVC%MATS_SKYLINE_RIGHT(JK,JMAT)=YD_PVC%MATS_SKYLINE_RIGHT(JK,JMAT)-1
      ENDDO

      YD_PVC%MATINVS_SKYLINE_LEFT(JK,JMAT)=1
      DO JJ=1,JK-1
        IF (ABS(YD_PVC%MATINVS(JJ,JK,JMAT)) &
          & >SKYLINE_TOL*YD_PVC%MATINVS(JK,JK,JMAT)) EXIT
        YD_PVC%MATINVS_SKYLINE_LEFT(JK,JMAT)= &
          & YD_PVC%MATINVS_SKYLINE_LEFT(JK,JMAT)+1
      ENDDO

      YD_PVC%MATINVS_SKYLINE_RIGHT(JK,JMAT)=YD_PVC%NLEVS
      DO JJ=YD_PVC%NLEVS,JK+1,-1
        IF (ABS(YD_PVC%MATINVS(JJ,JK,JMAT)) &
          & >SKYLINE_TOL*YD_PVC%MATINVS(JK,JK,JMAT)) EXIT
        YD_PVC%MATINVS_SKYLINE_RIGHT(JK,JMAT)= &
          & YD_PVC%MATINVS_SKYLINE_RIGHT(JK,JMAT)-1
      ENDDO
    ENDDO

!---- Zero elements outside the skylines.

    DO JK=1,YD_PVC%NLEVS
      DO JJ=1,YD_PVC%MATS_SKYLINE_LEFT (JK,JMAT)-1
        YD_PVC%MATS(JJ,JK,JMAT) = 0.0_JPRB
        YD_PVC%MATS(JK,JJ,JMAT) = 0.0_JPRB
      ENDDO
      DO JJ=YD_PVC%MATS_SKYLINE_RIGHT(JK,JMAT)+1,YD_PVC%NLEVS
        YD_PVC%MATS(JJ,JK,JMAT) = 0.0_JPRB
        YD_PVC%MATS(JK,JJ,JMAT) = 0.0_JPRB
      ENDDO
      DO JJ=1,YD_PVC%MATINVS_SKYLINE_LEFT (JK,JMAT)-1
        YD_PVC%MATINVS(JJ,JK,JMAT) = 0.0_JPRB
        YD_PVC%MATINVS(JK,JJ,JMAT) = 0.0_JPRB
      ENDDO
      DO JJ=YD_PVC%MATINVS_SKYLINE_RIGHT(JK,JMAT)+1,YD_PVC%NLEVS
        YD_PVC%MATINVS(JJ,JK,JMAT) = 0.0_JPRB
        YD_PVC%MATINVS(JK,JJ,JMAT) = 0.0_JPRB
      ENDDO
    ENDDO

!--- Send the matrix to the IOMASTER

    ZMSG(:,1:YD_PVC%NLEVS,1) = YD_PVC%MATS   (:,:,JMAT)
    ZMSG(:,1:YD_PVC%NLEVS,2) = YD_PVC%MATINVS(:,:,JMAT)
    ZMSG(:,YD_PVC%NLEVS+1,1) = YD_PVC%MATS_SKYLINE_LEFT    (:,JMAT)
    ZMSG(:,YD_PVC%NLEVS+2,1) = YD_PVC%MATS_SKYLINE_RIGHT   (:,JMAT)
    ZMSG(:,YD_PVC%NLEVS+1,2) = YD_PVC%MATINVS_SKYLINE_LEFT (:,JMAT)
    ZMSG(:,YD_PVC%NLEVS+2,2) = YD_PVC%MATINVS_SKYLINE_RIGHT(:,JMAT)

    IF (MYPROC/=IOMASTER) THEN
      I=1
      DO JK=1,2
        DO JJ=1,YD_PVC%NLEVS+2
          DO JI=1,YD_PVC%NLEVS
            ZMSG1D(I)=ZMSG(JI,JJ,JK)
            I=I+1
          ENDDO
        ENDDO
      ENDDO
      ITAG=MTAGFCE+J
      CALL MPL_SEND (ZMSG1D(:), KDEST=NPRCIDS(IOMASTER), KTAG=ITAG, &
       & CDSTRING='SUJBWAVWRI:WRITE_VCOR')
    ENDIF
  ELSE
    IF (MYPROC==IOMASTER) THEN
      ITAG=MTAGFCE+J
      CALL MPL_RECV (ZMSG1D(:), KSOURCE=NPRCIDS(YD_PVC%MASTERPE(J)), &
       & KTAG=ITAG, CDSTRING='SUJBWAVWRI:WRITE_VCOR')

      I=1
      DO JK=1,2
        DO JJ=1,YD_PVC%NLEVS+2
          DO JI=1,YD_PVC%NLEVS
            ZMSG(JI,JJ,JK)=ZMSG1D(I)
            I=I+1
          ENDDO
        ENDDO
      ENDDO
    ENDIF
  ENDIF

!--- Write the skylined matrix to file. NB: Only IOMASTER has
!--- the correct data, but all processors call the IO routines.

  IF (MYPROC==IOMASTER) THEN
    ZMAT   (:,:)           = ZMSG(:,1:YD_PVC%NLEVS,1)
    ZMATINV(:,:)           = ZMSG(:,1:YD_PVC%NLEVS,2)
    I_MAT_SKYLINE_LEFT     = NINT(ZMSG(:,YD_PVC%NLEVS+1,1))
    I_MAT_SKYLINE_RIGHT    = NINT(ZMSG(:,YD_PVC%NLEVS+2,1))
    I_MATINV_SKYLINE_LEFT  = NINT(ZMSG(:,YD_PVC%NLEVS+1,2))
    I_MATINV_SKYLINE_RIGHT = NINT(ZMSG(:,YD_PVC%NLEVS+2,2))
  ENDIF

  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=I_MAT_SKYLINE_LEFT)
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=I_MAT_SKYLINE_RIGHT)

  DO JK=1,YD_PVC%NLEVS
    ILEFT  = I_MAT_SKYLINE_LEFT (JK)
    IRIGHT = I_MAT_SKYLINE_RIGHT(JK)
    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=ZMAT(ILEFT:IRIGHT,JK))
  ENDDO
    
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=I_MATINV_SKYLINE_LEFT)
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,KR1=I_MATINV_SKYLINE_RIGHT)

  DO JK=1,YD_PVC%NLEVS
    ILEFT  = I_MATINV_SKYLINE_LEFT (JK)
    IRIGHT = I_MATINV_SKYLINE_RIGHT(JK)
    CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR1=ZMATINV(ILEFT:IRIGHT,JK))
  ENDDO
 
  CALL MPL_BARRIER (CDSTRING='SUJBWAVWRI:WRITE_VCOR')
ENDDO

IF (LHOOK) CALL DR_HOOK('SUJBWAVWRI:WRITE_VCOR',1,ZHOOK_HANDLE)
END SUBROUTINE WRITE_VCOR
END SUBROUTINE SUJBWAVWRI
