SUBROUTINE SUJQSTD(PCOVT,PCOVV,PCOVD,PCOVQ,PCOVO,YDERRMN)

!     Purpose.
!     --------
!       Compute model error standard deviations profiles
!       from input data.
!       Adapted from SUJBSTD.

!     Author.
!     -------
!       Yannick Tremolet

!     Modifications.
!     --------------
!       Original  : 12-May-2004
!        Y.Tremolet    21-Jul-2004 Use NSERR
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMDIM  , ONLY: NFLEVG, NFLEVL
USE YOMMP   , ONLY: MYLEVS
USE YOMLUN  , ONLY: NULOUT
USE YOMDYN  , ONLY: TSTEP
USE YOMGEM  , ONLY: VP00
USE YOMJQ   , ONLY: ERRMODMN_STRUCT, NLEVERR1, NLEVERR0
USE YOMMODEL_ERROR, ONLY: ALPHAQ, NVE3D, NSERR, TSTEP_ERR, LSCALERR, &
                        & ALPHAQ_T,ALPHAQ_D,ALPHAQ_V,ALPHAQ_Q,ALPHAQ_O 

IMPLICIT NONE
REAL(KIND=JPRB), INTENT(IN) :: PCOVT(NFLEVG+1,NFLEVG+1,0:NSERR)
REAL(KIND=JPRB), INTENT(IN) :: PCOVV(NFLEVG,NFLEVG,0:NSERR)
REAL(KIND=JPRB), INTENT(IN) :: PCOVD(NFLEVG,NFLEVG,0:NSERR)
REAL(KIND=JPRB), INTENT(IN) :: PCOVQ(NFLEVG,NFLEVG,0:NSERR)
REAL(KIND=JPRB), INTENT(IN) :: PCOVO(NFLEVG,NFLEVG,0:NSERR)
TYPE(ERRMODMN_STRUCT), INTENT(INOUT) :: YDERRMN

REAL(KIND=JPRB) :: ZVAR(NFLEVG,5),ZSTD(NFLEVG,5),ZLEV(NFLEVG)
REAL(KIND=JPRB) :: ZVARSP,ZSTDSP,za
INTEGER(KIND=JPIM) :: jl,jf,il
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJQSTD',0,ZHOOK_HANDLE)

! Total standard deviations
DO jl=1,NFLEVG
  ZVAR(jl,1) = ALPHAQ_V * SQRT( SUM(PCOVV(jl,jl,:)) )
  ZVAR(jl,2) = ALPHAQ_D * SQRT( SUM(PCOVD(jl,jl,:)) )
  ZVAR(jl,3) = ALPHAQ_T * SQRT( SUM(PCOVT(jl,jl,:)) )
  ZVAR(jl,4) = ALPHAQ_Q * SQRT( SUM(PCOVQ(jl,jl,:)) )
  ZVAR(jl,5) = ALPHAQ_O * SQRT( SUM(PCOVO(jl,jl,:)) )
ENDDO
ZVARSP = ALPHAQ_T * SQRT( SUM(PCOVT(NFLEVG+1,NFLEVG+1,:)) )

! Scaling Jq standard deviations
DO jl=1,NFLEVG
  IF (jl<=NLEVERR1) THEN
    za=ALPHAQ
  ELSEIF (jl<NLEVERR0) THEN
    za=(REAL(NLEVERR0-jl,JPRB)/REAL(NLEVERR0-NLEVERR1,JPRB)) * ALPHAQ
  ELSE
    za=0.0_JPRB
  ENDIF
  WRITE(NULOUT,*)'SUJQSTD: jl,za=',jl,za
  DO jf=1,NVE3D
    ZSTD(jl,jf)=za*ZVAR(jl,jf)
  ENDDO
  ZLEV(jl)=za
ENDDO
! za is the same as for last level (ie surface)
ZSTDSP=za*ZVARSP

! Forcing in control space is per hour
IF (LSCALERR) THEN
  IF (TSTEP_ERR/=0.0_JPRB) THEN
    za = TSTEP_ERR/3600.0_JPRB
  ELSE
    za = TSTEP/3600.0_JPRB
  ENDIF
  ZSTD(:,:)=za*ZSTD(:,:)
  ZSTDSP=za*ZSTDSP
ENDIF

! Copy to proper arrays
za=EPSILON(za)*EPSILON(za)
WRITE(NULOUT,*)'SUJQSTD: Minimum acceptable standard deviation=',za
DO jl=1,NFLEVL
  il=MYLEVS(jl)
  DO jf=1,NVE3D
    IF (ZSTD(il,jf)>za) THEN
      YDERRMN%V3D(jl,jf)   =ZSTD(il,jf)
      YDERRMN%V3DINV(jl,jf)=1.0_JPRB/ZSTD(il,jf)
    ELSE
      YDERRMN%V3D(jl,jf)   =0.0_JPRB
      YDERRMN%V3DINV(jl,jf)=0.0_JPRB
    ENDIF
  ENDDO
  YDERRMN%VALPHA(jl)=ZLEV(il)
ENDDO
IF (ZSTDSP>za) THEN
  YDERRMN%V2D(1)   =ZSTDSP
  YDERRMN%V2DINV(1)=1.0_JPRB/ZSTDSP
ELSE
  YDERRMN%V2D(1)   =0.0_JPRB
  YDERRMN%V2DINV(1)=0.0_JPRB
ENDIF
YDERRMN%VSURF=ZLEV(NFLEVG)

! Diagnostics
WRITE(NULOUT,*)'Jq vertical standard deviation profiles:'
WRITE(NULOUT,*)' Surf pressure : ',ZSTDSP*VP00
WRITE(NULOUT,*)'        Vor         Div          T           Q          O3'
DO jl=1,NFLEVG
  WRITE(NULOUT,'(I3,6(1X,G11.4))')jl,(ZSTD(jl,jf),jf=1,NVE3D),ZLEV(jl)
ENDDO

IF (LHOOK) CALL DR_HOOK('SUJQSTD',1,ZHOOK_HANDLE)
END SUBROUTINE SUJQSTD
