SUBROUTINE SUJR

!**** *SUJR* - Routine to initialize cost function common - YOMCOSJR

!     Purpose.
!     --------
!           Initialize YOMCOSJR

!**   Interface.
!     ----------

!        *CALL* *SUJR

!        Explicit arguments :
!        --------------------
!           NONE

!        Implicit arguments :
!        --------------------
!        COMMON YOMCOSJR

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------

!        DRASKO VASILJEVIC *ECMWF*

!     Modifications.
!     --------------
!        Original : 90-11-15
!        Modified by F.Rabier and J.N Thepaut : 91-01-15
!        Modified by L.Isaksen : 96-04-04  Adapted for Message Passing
!        Modified by E.Holm    : 97-12-03  Ozone included
!        Modified by L.Dragulanescu: 97-05-24 ALADIN 3DVAR
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        V. Guidard    01-Apr-2004 bugfix on FUVB
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LELAM
USE YOMCOSJR , ONLY : FALTSEL  ,FSURSEL  ,FSPALT   ,FSPSUR
USE YEMCOSJR , ONLY : FUVB     ,FUVBSEL
USE YOMMP    , ONLY : NPSP
!!USE YOM_YGFL , ONLY : YO3

IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE


!      -----------------------------------------------------------

!*       1.    RESET REFERENCE COST FUNCTION TO 0.
!              -----------------------------------

IF (LHOOK) CALL DR_HOOK('SUJR',0,ZHOOK_HANDLE)
FSPALT(:,:,:)=0.0_JPRB
IF (NPSP == 1) FSPSUR(:,:)=0.0_JPRB

IF (LELAM) FUVB(:,:)=0.0_JPRB

!      -----------------------------------------------------------

!*       2.    CONTROL OF COST-FUNCTION.
!              -------------------------

!*          2.35  U AND V

FALTSEL(:,1:2)=1.0_JPRB

!*          2.37  T

FALTSEL(:,3)=1.0_JPRB

!*          2.38  Q

FALTSEL(:,4)=1.0_JPRB

!*          2.39  O3

!!IF (YO3%LSPJB) FALTSEL(:,5)=1.0_JPRB

!*          2.40 Mean wind in case of ALADIN

IF (LELAM) FUVBSEL(1:2)=1.0_JPRB

!*          2.41  PS

FSURSEL(:)=1.0_JPRB

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJR',1,ZHOOK_HANDLE)
END SUBROUTINE SUJR

