SUBROUTINE SUNNE

!**** *SUNNE*  - 

!     PURPOSE.
!     --------
!     Set-up of NAMNN and YOMNNE variables (neural network bias correction)

!**   INTERFACE.
!     ----------
!       *CALL* *SUNNE*

!        EXPLICIT ARGUMENTS
!        --------------------
!        none

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     AUTHOR.
!     -------
!        ???

!     MODIFICATIONS.
!     --------------
!     K. Yessad (Dec 2008): add comments + cleanings
!-----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIMO  , ONLY : NLSAT    ,NMXTCHU
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMNNE, ONLY              : NINPUTS, NINPPB, NHIDDEN, NDIM, &
 & LNNWRI, LNNMSKRS, LNNBC, LNNHC, LNNLEARN, LNNREAD, &
 & LNNPCA, LNNSELECT, LNNJAC, LNNOUT, LNNM1QN3, &
 & NO_OF_EPOCHS, SCAL, SELECTINP, NU, NU2, LISS, &
 & EIG_VEC, EIG_VAL, &
 & JAC_MOY, JAC_STD, &
 & W, INP_MOY, INP_STD, OUT_MOY, OUT_STD, &
 & BETA 

!-----------------------------------------------------------------------------

IMPLICIT NONE

LOGICAL     :: LLEXIST
INTEGER(KIND=JPIM)   :: I, J
CHARACTER             :: CLFILE_WEIGHTS*30, CLFILE_PCA*30, CLFILE_JAC*30
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------------

#include "posnam.intfb.h"
#include "namnn.h"

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUNNE',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------------

WRITE(NULOUT,*)'----> SUNNE'
! Set default values
LNNWRI   = .FALSE. !! .T. = DUMP NEURAL NETWORK LEARNING FILES
LNNMSKRS = .FALSE. !! .T. = DUMP ONLY NEAR RADIOSONDES
LNNBC    = .FALSE. !! .T. = RUN NEURAL NETWORK BIAS CORRECTION
LNNHC    = .FALSE. !! .T. = RUN NEURAL NETWORK OBSERVATION OPERATOR CORRECTION
LNNLEARN = .FALSE. !! learning/testing mode
LNNREAD  = .TRUE.  !! read/calculate weights, normalization, PCA, Jacobian
LNNPCA   = .TRUE.  !! PCA pre-processing
LNNSELECT= .FALSE. !! select useful input PCs (in pre-processing)
LNNJAC   = .FALSE. !! write sensitivity files
LNNOUT   = .FALSE. !! test performance
LNNM1QN3 = .FALSE. !! use M1QN3 minimisation program
NINPUTS  =2*NLSAT+6!! number of inputs
NHIDDEN  = 3       !! number of hidden neurons
NO_OF_EPOCHS=100   !! maximum number of epochs (in learning process)
SCAL     = 0.83    !! scale for input/output normalisation
SELECTINP= 0.9     !! threshold for input selection
NU       = 1.      !! regularization factor (for performance function)
NU2      = 1000.   !! regularization factor (for performance function)
ALLOCATE(LISS(3))
LISS(1)  = 7       !! Bounds for regularization smoothing
LISS(2)  = 50      !! Bounds for regularization smoothing
LISS(3)  = 92      !! Bounds for regularization smoothing
BETA     = 0.5     !! Coefficient for analysis correction before learning process
                   !! (estimation of analysis bias from beta*(Guess-Ana)

WRITE(NULOUT,*)'Reading NN namelist'
CALL POSNAM(NULNAM,'NAMNN')
READ(NULNAM,NAMNN)
WRITE(NULOUT,NAMNN)
NINPPB   =NINPUTS+1!! INPut Plus Bias : accounts for bias to input
NDIM=NINPPB*NHIDDEN+(NHIDDEN+1) ! Dimension of 1D 'W' and 'GW' arrays

ALLOCATE(INP_MOY(NMXTCHU,NINPUTS))                        !mean of input  vector
ALLOCATE(INP_STD(NMXTCHU,NINPUTS))                        !std  of input  vector
ALLOCATE(OUT_MOY(NMXTCHU))                                !mean of output vector
ALLOCATE(OUT_STD(NMXTCHU))                                !std  of output vector
ALLOCATE(W(NMXTCHU,(NINPUTS+1)*NHIDDEN+(NHIDDEN+1)))      !weights read from file (all channels)
ALLOCATE(JAC_MOY(NMXTCHU,NINPUTS))
ALLOCATE(JAC_STD(NMXTCHU,NINPUTS))
ALLOCATE(EIG_VEC(NINPUTS,NINPUTS))
ALLOCATE(EIG_VAL(NINPUTS))

CLFILE_WEIGHTS = 'nn_weights.dat'
CLFILE_PCA     = 'pca_transf_matrices.dat'
CLFILE_JAC     = 'nn_sensitivity.dat'

!! Read weights and normalization parameters !!
INQUIRE(file=TRIM(CLFILE_WEIGHTS), exist=LLEXIST)
LNNREAD=(LNNREAD .AND. LLEXIST)
IF (LNNREAD) THEN                                  !! Read network weights & normalization params
  OPEN (7,status='old',file=CLFILE_WEIGHTS)
  READ(7,*) W
  READ(7,*) INP_MOY,INP_STD,OUT_MOY,OUT_STD
  READ(7,*) SCAL
  CLOSE(7)
ELSE                                               !! generate initial random weights !!
  CALL RANDOM_NUMBER(W)
  W=((W-0.5)*2)/10
  INP_MOY=0. ; INP_STD=0. ; OUT_MOY=0. ; OUT_STD=0.
ENDIF

!! Read PCA file !!
INQUIRE(file=TRIM(CLFILE_PCA),exist=LLEXIST)
LNNPCA=(LNNPCA .AND. LLEXIST)
IF (LNNPCA) THEN
  OPEN( 25, FILE = CLFILE_PCA, STATUS = "old" )
  DO I=1,NINPUTS
    READ(25,*) (EIG_VEC(J,I), J=1,NINPUTS)          !----- Eigenvectors
  ENDDO
  READ(25,*)    (EIG_VAL(J), J=1,NINPUTS)           !----- Eigenvalues
  CLOSE(25)
ENDIF

!! Read sensitivity (jacobian) file !!
INQUIRE(file=TRIM(CLFILE_JAC), exist=LLEXIST)
LNNSELECT=(LNNSELECT .AND. LLEXIST)
IF (LNNSELECT) THEN
  OPEN(22,FILE = CLFILE_JAC, status="old")
  READ(22,*) JAC_MOY,JAC_STD
  CLOSE(22)
ELSE
  JAC_MOY(:,:)=0.
  JAC_STD(:,:)=0.
ENDIF

!-----------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUNNE',1,ZHOOK_HANDLE)
END SUBROUTINE SUNNE
