SUBROUTINE SUPRFFCE

!**** *SUPRFFCE*  - CALCULATE GLOBAL MEAN PROFILE OF BACKGROUND ERROR

!     PURPOSE.
!     --------

!        CALCULATE THE GLOBAL MEAN PROFILE OF BACKGROUND ERROR

!**   INTERFACE.
!     ----------

!        *CALL* *SUPRFFCE

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!     AUTHOR.
!     -------

!        M. Fisher    *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NPROC
USE YOMDIM   , ONLY : NPROMA
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG
USE YOMLUN   , ONLY : NULOUT
USE YOMJG    , ONLY : FCEIMN   ,NOFCEF
USE YOMFCEB  , ONLY : FCEBUF
USE YOMGC    , ONLY : GAW
USE MPL_MODULE

!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) ::  ZFCE(NPROMA,NOFCEF)
REAL(KIND=JPRB), ALLOCATABLE :: ZGLOBAL(:),ZLOCAL(:,:)

INTEGER(KIND=JPIM) :: IENDC, IN, ISTC, JGP, JK, JKGLO, JVAR

REAL(KIND=JPRB) :: ZDA, ZSUMA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "gatherbdy.intfb.h"
#include "sc2rdg.intfb.h"

!     ------------------------------------------------------------------

!       1. CALCULATE HORIZONTAL INTEGRAL OF BACKGROUND ERRORS

IF (LHOOK) CALL DR_HOOK('SUPRFFCE',0,ZHOOK_HANDLE)
ZSUMA=0._JPRB
IN=0

ALLOCATE (ZLOCAL(NGPTOT,NOFCEF))
ZLOCAL(:,:) = 0.0_JPRB
FCEIMN(:)=0.0_JPRB

MESSP_LOOP : DO JKGLO=1,NGPTOT,NPROMA
  ISTC=1
  IENDC=MIN(NPROMA,NGPTOT-JKGLO+1)
  CALL SC2RDG(FCEBUF,JKGLO,ISTC,IENDC,ZFCE)

  DO JK=ISTC,IENDC
    ZDA    = GAW(JKGLO-1+JK)
    ZSUMA  = ZSUMA + ZDA
    ZLOCAL(JKGLO-1+JK,:) = ZLOCAL(JKGLO-1+JK,:)+ZDA*ZFCE(JK,:)*ZFCE(JK,:)
  ENDDO
ENDDO  MESSP_LOOP

!--- GATHER THE GLOBAL SUM OF AREA OVER ALL GRIDBOXES (SAFETY CHECK)

CALL GSTATS(669,0)
IF (NPROC > 1) THEN
  CALL MPL_ALLREDUCE(ZSUMA,'SUM',LDREPROD=.FALSE.,CDSTRING='SUPRFFCE:')
ENDIF
CALL GSTATS(669,1)

IF (ABS(ZSUMA-1.0_JPRB) > 1E-5_JPRB) THEN
  WRITE (NULOUT,*) 'ERROR IN SUPRFFCE: GLOBAL SUM OF GAW<>1'
  CALL ABOR1 ('ERROR IN SUPRFFCE: GLOBAL SUM OF GAW<>1')
ENDIF

!--- GATHER THE GLOBAL ARRAY FOR EACH VARIABLE AND CALCULATE THE MEAN

IF (NPROC > 1) THEN
  ALLOCATE(ZGLOBAL(NGPTOTG))
  DO JVAR=1,NOFCEF
    CALL GATHERBDY(ZLOCAL(:,JVAR),ZGLOBAL)
    DO JGP=1,NGPTOTG
      FCEIMN(JVAR) = FCEIMN(JVAR) + ZGLOBAL(JGP)
    ENDDO
  ENDDO

  DEALLOCATE(ZGLOBAL)
ELSE
  DO JVAR=1,NOFCEF
    DO JGP=1,NGPTOTG
      FCEIMN(JVAR) = FCEIMN(JVAR) + ZLOCAL(JGP,JVAR)
    ENDDO
  ENDDO
ENDIF

DEALLOCATE (ZLOCAL)

!--- CONVERT VARIANCE TO STANDARD DEVIATION

FCEIMN(:) = SQRT(FCEIMN(:))

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPRFFCE',1,ZHOOK_HANDLE)
END SUBROUTINE SUPRFFCE
