SUBROUTINE SURINC

!**** *SURINC*   - Routine to initialize incremental variational jobsn

!     Purpose.
!     --------
!           Initialize resolution of internal loop
!**   Interface.
!     ----------
!        *CALL* *SURINC

!        Explicit arguments : none
!        --------------------

!        Implicit arguments :
!        --------------------
!        COMMON YOMRINC

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel Thepaut *ECMWF*

!     Modifications.
!     --------------
!        Original : 93-01-29
!        Modified by R. EL Khatib : 93-04-02 Set-up defaults controled by LECMWF
!        Modified by F. Rabier: 93-12-10 Possibility of writing trajectory
!                               at lower resolution
!        J.N. Thepaut: 94-08-04 Introduction of a switch to hard constrain the
!                               humidity field to remain reasonable in the
!                               minimisation at lower resolution
!        L. Isaksen:   95-06-06 Reordering of spectral arrays
!        M. Fisher:    96-08-01 Make I/O optional
!        A. Horanyi:   92-11-03 3DVAR ALADIN
!        M. Torki  :   99-04-16 Option to use Multi-incremental
!        A. Peuch  :   00-08-21 Introduction of LO3FILT,O3NLMINO3 and O3NLCURV
!        Y. Tremolet:  03-01-13 Clean-up
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        E. Holm   :   09-04-01 Changed to QNLMINQ=1E-8, QNLCURV=2E+7 which
!                               better reflects the model forecast limit 1E-8
!                               set in qnegat.F90.
!     --------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMCT0B  , ONLY : LECMWF
USE YOMRINC  , ONLY : QNLMINQ  ,QNLCURV

IMPLICIT NONE

#include "namrinc.h"
!      ----------------------------------------------------------------

!*       1.    SET DEFAULT VALUES.
!              -------------------

!        1.1 Set implicit default values

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"
IF (LHOOK) CALL DR_HOOK('SURINC',0,ZHOOK_HANDLE)
QNLMINQ=1.0E-8_JPRB
QNLCURV=2.0E+7_JPRB

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
ENDIF

!      ----------------------------------------------------------------

!*       2.    Modifies default values.
!              ------------------------

CALL POSNAM(NULNAM,'NAMRINC')
READ(NULNAM,NAMRINC)

!      -----------------------------------------------------------

!*       3.    Print final values.
!              -------------------

WRITE(NULOUT,'('' NAMELIST NAMRINC '')')
WRITE(NULOUT,NAMRINC)

IF (LHOOK) CALL DR_HOOK('SURINC',1,ZHOOK_HANDLE)
END SUBROUTINE SURINC
