SUBROUTINE SUSCALMERR(YDSCALP)

!     Purpose:
!     --------
!       Define the inner product for model error control variable.

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original 17-Mar-2004
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMMODEL_ERROR, ONLY: NSERR, NVE3D, NVE2D
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDSCALP 

TYPE(SPECTRAL_FIELD) :: YLSPEC
REAL(KIND=JPRB), ALLOCATABLE :: ZMFACTA(:)
INTEGER(KIND=JPIM) :: jn, js, jl, jf, ii, jj
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!    -------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSCALMERR',0,ZHOOK_HANDLE)

CALL INQ_SPEC(YLSPEC,NSERR,0,NVE3D,NVE2D,0)

ALLOCATE(ZMFACTA(YLSPEC%nspec2))
ZMFACTA(:)=2.0_JPRB
IF (YLSPEC%nasm0(0)>0) THEN
  DO jn=0,NSERR
    ZMFACTA(2*jn+1)=1.0_JPRB  ! Real part
    ZMFACTA(2*jn+2)=0.0_JPRB  ! Imaginary part
  ENDDO
ENDIF

ii=0
DO jj=1,YDSCALP%NMODERR
  DO jf=1,YLSPEC%ns2d
    DO js=1,YLSPEC%nspec2
      ii=ii+1
      YDSCALP%MODERR(ii) = ZMFACTA(js)
    ENDDO
  ENDDO
  DO jf=1,YLSPEC%ns3d
    DO js=1,YLSPEC%nspec2
      DO jl=1,YLSPEC%nflevl
        ii=ii+1
        YDSCALP%MODERR(ii) = ZMFACTA(js)
      ENDDO
    ENDDO
  ENDDO
ENDDO

IF (ii/=SIZE(YDSCALP%MODERR)) CALL ABOR1('SUSCALMERR: error size')

CALL DEALLOCATE_SPEC(YLSPEC)
DEALLOCATE(ZMFACTA)

IF (LHOOK) CALL DR_HOOK('SUSCALMERR',1,ZHOOK_HANDLE)
END SUBROUTINE SUSCALMERR
