subroutine suskf

!-----------------------------------------------------------------------
!   SUSKF - Setup for the Simplified Kalman Filter

!   Purpose.
!   -------- 

!   Interface.
!   ----------
!      CALL SUSKF

!   Externals.
!   ----------

!   Reference.
!   ----------
!       None yet!

!   Author.
!   -------
!       Mike Fisher  *ECMWF*

!   Modifications.
!   --------------
!       Original   97-01-16
!     Y.Tremolet   02-08-06  Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMSP    , ONLY : SPVOR    ,SPDIV   ,SPGFL
USE YOMLAP   , ONLY : NASM0    ,MYMS
USE YOMCVA   , ONLY : YSCALP , YSCALPSQRT
USE YOMJG    , ONLY : LSUBBG   ,LSPFCE
USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMSKF   , ONLY : LSKF     ,LWRISKF  ,NSKFVECS ,NWRISKF  ,&
 & YSKFROT  ,YSKFMAT   ,SKFSUBMAT,CINSKFY  ,CINSKFZ  ,LROKF, &
 & CINCOVARX, CINCOVARY, COUTCOVARZ, NCOVARVECS, L_SCALE_Y  
USE YOMDYN   , ONLY : TSTEP
USE YOMOP    , ONLY : CFNSKFY  ,CFNSKFZ
USE YOMMSC   , ONLY : N_DEFAULT_REAL_KIND, N_DOUBLE_KIND
USE CONTROL_VECTORS_COMM_MOD, ONLY: SET_CVSECTION, ALLGATHER_CVSECTION, &
                              & GET_CVINDEX
USE CONTROL_VECTORS

IMPLICIT NONE

#include "namskf.h"

REAL(KIND=JPRB), ALLOCATABLE, DIMENSION(:,:) :: zU,zmat
REAL(KIND=JPRB), ALLOCATABLE, DIMENSION(:)   :: zlam,zwork1,zwork2
TYPE(control_vector), ALLOCATABLE :: YL_ZZVECS(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: jndx(:)
CHARACTER (LEN = 16) ::  clfile
INTEGER(KIND=JPIM) :: ji,jj,jvec,itries,jtry,info
REAL(KIND=JPRB) :: zritz,zps,zalpha,zzps
LOGICAL :: llspfce
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "cain.intfb.h"
#include "cainin.intfb.h"
#include "chavar.intfb.h"
#include "chavarinad.intfb.h"
#include "posnam.intfb.h"
#include "readvec.intfb.h"

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!       1. Set default values

IF (LHOOK) CALL DR_HOOK('SUSKF',0,ZHOOK_HANDLE)
LSKF     =.false.
LWRISKF  =.false.
L_SCALE_Y = .false.
NSKFVECS =10
NWRISKF  = 6*3600/TSTEP
CINSKFY  = CFNSKFY
CINSKFZ  = CFNSKFZ
LROKF    =.false.
NCOVARVECS=0
CINCOVARX = 'inputx'
CINCOVARY = 'inputy'
COUTCOVARZ = 'outptz'

!       2. Read the Namelist

CALL POSNAM(NULNAM,'NAMSKF')
READ (NULNAM,NAMSKF)
WRITE(NULOUT,NAMSKF)

IF (.not.LSKF .AND. LHOOK) CALL DR_HOOK('SUSKF',1,ZHOOK_HANDLE)
IF (.not.LSKF) RETURN

WRITE (NULOUT,'(//'' Simplified Kalman Filter''/' //&
 & ''' ------------------------''/)')  

!       3. Allocate Arrays

ALLOCATE (YSKFMAT(NSKFVECS))
ALLOCATE (YSKFROT(NSKFVECS))
DO JJ=1,NSKFVECS
  CALL ALLOCATE_CTLVEC(YSKFMAT(JJ))
  CALL ALLOCATE_CTLVEC(YSKFROT(JJ))
ENDDO
ALLOCATE (SKFSUBMAT(NSKFVECS,NSKFVECS))

ALLOCATE (YL_ZZVECS(NSKFVECS))
DO jvec=1,NSKFVECS
  CALL ALLOCATE_CTLVEC(YL_ZZVECS(jvec))
ENDDO
ALLOCATE (zU     (NSKFVECS,NSKFVECS))
ALLOCATE (zmat   (NSKFVECS,NSKFVECS))
ALLOCATE (zlam   (NSKFVECS))
ALLOCATE (zwork1 (NSKFVECS))
ALLOCATE (zwork2 (NSKFVECS))

LSUBBG = .false.
LLSPFCE= LSPFCE
LSPFCE = .true.

ALLOCATE(JNDX(YSKFROT(1)%NSIZEL))
CALL GET_CVINDEX(YSKFROT(1), jndx)

!       4. read vectors and eigenvalues and apply the change of variable
!          (to Euclidean control space)

do jvec=1,NSKFVECS
  LSKF   = .true.
  clfile = CINSKFY
  write(clfile(7:9),'(I3.3)') jvec
  call readvec (clfile,zritz)

  if (L_SCALE_Y) then
    zlam(jvec) = zritz
  else
    zlam(jvec) = 1.0_JPRB
  endif

  SPGFL(:,:,:) = 0.0_JPRB

  LSKF   = .false.
  call CHAVAR (YSKFROT(jvec))

  YSKFROT(jvec)%data = YSKFROT(jvec)%data * YSCALPSQRT%data
  call CAIN (YSKFROT(jvec))
  if (ANY(MYMS(:)==0)) then
    SPVOR(:,NASM0(0)) = 0.0_JPRB
    SPDIV(:,NASM0(0)) = 0.0_JPRB
  endif
  call CAININ (YSKFROT(jvec))
  IF (YSKFROT(jvec)%nmoderr>0) YSKFROT(jvec)%moderr(:) = 0.0_JPRB
  IF (YSKFROT(jvec)%ltovscv  ) YSKFROT(jvec)%tovscv(:) = 0.0_JPRB
  IF (YSKFROT(jvec)%lam1d    ) YSKFROT(jvec)%lamcv(:)  = 0.0_JPRB
  IF (YSKFROT(jvec)%npara>0  ) YSKFROT(jvec)%params(:) = 0.0_JPRB

  LSKF   = .true.
  clfile = CINSKFZ
  write(clfile(7:9),'(I3.3)') jvec
  call readvec (clfile)
  SPGFL(:,:,:) = 0.0_JPRB

  LSKF   = .false.
  call CAININ (YL_ZZVECS(jvec))
  WHERE (YSCALP%data(:) /= 0.0_JPRB )
    YL_ZZVECS(jvec)%data(:) = YL_ZZVECS(jvec)%data(:) /YSCALP%data(:) /zlam(jvec)
  ELSEWHERE
    YL_ZZVECS(jvec)%data(:) = 0.0_JPRB
  ENDWHERE
  call CAIN (YL_ZZVECS(jvec))
  YL_ZZVECS(jvec) = 0.0_JPRB
  call CHAVARINAD (YL_ZZVECS(jvec))
  IF (YL_ZZVECS(jvec)%linitcv) YL_ZZVECS(jvec)%initcv(:) = &
                       & YL_ZZVECS(jvec)%initcv(:) * YSCALPSQRT%data(:)
  IF (YL_ZZVECS(jvec)%nmoderr>0) YL_ZZVECS(jvec)%moderr(:) = 0.0_JPRB
  IF (YL_ZZVECS(jvec)%ltovscv  ) YL_ZZVECS(jvec)%tovscv(:) = 0.0_JPRB
  IF (YL_ZZVECS(jvec)%lam1d    ) YL_ZZVECS(jvec)%lamcv(:)  = 0.0_JPRB
  IF (YL_ZZVECS(jvec)%npara>0  ) YL_ZZVECS(jvec)%params(:) = 0.0_JPRB
enddo

WRITE (NULOUT,*) 'Eigenvalues are: ',zlam(1:NSKFVECS)

LSUBBG = .true.
LSKF   = .true.
LSPFCE = LLSPFCE

!       6. Generate the Housholder transformations which rotate
!          coordinates so that YSKFROT is upper triangular. Save
!          this upper triangle in zU and replace YSKFROT with the
!          vectors which define the Householder transformations.

do jj=1,NSKFVECS
  CALL ALLGATHER_CVSECTION(YSKFROT(jj),zwork1(1:jj),1,jj)
  zps = DOT_PRODUCT(YSKFROT(jj),YSKFROT(jj)) &
   & - DOT_PRODUCT(zwork1(1:jj),zwork1(1:jj))  

  if (zwork1(jj) < 0.0_JPRB) then
    zU(jj,jj) = -sqrt(zps+zwork1(jj)*zwork1(jj))
  else
    zU(jj,jj) =  sqrt(zps+zwork1(jj)*zwork1(jj))
  endif

  zwork1(jj) = zwork1(jj) - zU(jj,jj)

  zps = zps + zwork1(jj)*zwork1(jj)

  CALL SET_CVSECTION(zwork1(jj:jj),YSKFROT(jj),jj,jj)

  zzps=sqrt(2.0_JPRB/zps)
  WHERE (jndx(:)>=jj) YSKFROT(jj)%data(:) = zzps * YSKFROT(jj)%data(:)

  zU(1:jj-1,jj) = zwork1(1:jj-1)

  do ji=jj+1,NSKFVECS
    zps = DOT_PRODUCT (YSKFROT(ji),YSKFROT(jj))
    if (jj > 1) then
      CALL ALLGATHER_CVSECTION(YSKFROT(ji),zwork2(1:jj-1),1,jj-1)
      zps = zps - DOT_PRODUCT (zwork1(1:jj-1),zwork2(1:jj-1))
    endif
    WHERE (jndx(:)>=jj) YSKFROT(ji)%data(:) = YSKFROT(ji)%data(:) - zps * YSKFROT(jj)%data(:)
  enddo
enddo

!          explicitly zero the low elements

do jj=2,NSKFVECS
  WHERE (jndx(:)<jj) YSKFROT(jj)%data(:) = 0.0_JPRB
enddo

!       7. Rotate the 'z' vectors

do jj=1,NSKFVECS
  do ji=1,NSKFVECS
    zps = DOT_PRODUCT (YL_ZZVECS(ji),YSKFROT(jj))
    YL_ZZVECS(ji)%data = YL_ZZVECS(ji)%data - zps*YSKFROT(jj)%data
  enddo
enddo

!       8. Solve for the elements of the inverse covariance matrix

do jj=1,NSKFVECS
  YSKFMAT(jj) = YL_ZZVECS(jj)
  do ji=1,jj-1
    YSKFMAT(jj)%data = YSKFMAT(jj)%data - YSKFMAT(ji)%data * zU(ji,jj)
  enddo
  YSKFMAT(jj)%data = YSKFMAT(jj)%data / zU(jj,jj)
enddo

!       9. Calculate Cholesky decomposition of E - alpha*transpose(F)*F
!          where alpha is chosen to ensure positive definiteness

WRITE (NULOUT,*) 'Inverse covariance matrix for the subspace:'
do jj=1,NSKFVECS
  CALL ALLGATHER_CVSECTION(YSKFMAT(jj),zU(:,jj),1,NSKFVECS)
  WRITE (NULOUT,*) 'column ',jj,':'
  WRITE (NULOUT,*) zU(:,jj)
enddo

do jj=1,NSKFVECS
  if (zU(jj,jj) <= 0.0_JPRB) then
    write (NULOUT,*) 'Error: non-positive diagonal element. Element ',jj,' is ',zU(jj,jj)
    call ABOR1 ('SUSKF: non-positive diagonal element')
  endif
enddo

do jj=1,NSKFVECS
  do ji=1,jj-1
    zU(ji,jj) = 0.5_JPRB*(zU(ji,jj) + zU(jj,ji))
    zU(jj,ji) = zU(ji,jj)
  enddo
enddo

itries=1
zalpha = 0.0_JPRB
DO jtry=1,itries+1

  do jj=1,NSKFVECS
    CALL SET_CVSECTION(zU(1:jj,jj),YSKFMAT(jj),1,jj)
    CALL ALLGATHER_CVSECTION(YSKFMAT(jj),zwork1(:),1,NSKFVECS)
    do ji=1,jj
      CALL ALLGATHER_CVSECTION(YSKFMAT(ji),zwork2,1,NSKFVECS)
      zwork1(ji) = zwork1(ji) - DOT_PRODUCT(YSKFMAT(jj),YSKFMAT(ji)) &
       & + DOT_PRODUCT(zwork1,zwork2)  
      CALL SET_CVSECTION(zwork1(ji:ji),YSKFMAT(jj),ji,ji)
    enddo
  enddo

  do jj=1,NSKFVECS
    CALL ALLGATHER_CVSECTION(YSKFMAT(jj),zmat(1:jj,jj),1,jj)
  enddo

  if (JPRB == N_DEFAULT_REAL_KIND) then
    call SPOTRF ('U',NSKFVECS,zmat,NSKFVECS,info)
  ELSEIF (JPRB == N_DOUBLE_KIND) then
    call DPOTRF ('U',NSKFVECS,zmat,NSKFVECS,info)
  else
    call ABOR1 ('REAL(KIND=JPRB) is neither default real nor double precision')
  endif

  do jj=1,NSKFVECS
    CALL SET_CVSECTION(zmat(1:jj,jj),YSKFMAT(jj),1,jj)
  enddo

  if (info == 0) then
    EXIT
  ELSEIF (info > 0 .and. jtry <= itries) then
    zalpha = zalpha*0.5_JPRB
    if (jtry == itries) zalpha=0.0_JPRB
    write (NULOUT,*) 'Warning: reducing x-correlations by ',zalpha
    do jj=1,NSKFVECS
      WHERE(jndx(:)>NSKFVECS) YSKFMAT(jj)%data(:) = zalpha * YSKFMAT(jj)%data(:)
    enddo
  else
    write (NULOUT,*) 'Error: Cholesky decomp. failed in suskf'
    call ABOR1 ('Cholesky decomposition failed in suskf')
  endif
ENDDO

!       9.5 Copy elements of YSKFMAT into SKFSUBMAT

do jj=1,NSKFVECS
  CALL ALLGATHER_CVSECTION(YSKFMAT(jj),SKFSUBMAT(:,jj),1,NSKFVECS)
enddo

!      10. Dealocate arrays

DO jvec=1,NSKFVECS
  CALL DEALLOCATE_CTLVEC(YL_ZZVECS(jvec))
ENDDO
DEALLOCATE (YL_ZZVECS)
DEALLOCATE (zU    )
DEALLOCATE (zmat  )
DEALLOCATE (zlam  )
DEALLOCATE (zwork1)
DEALLOCATE (zwork2)
DEALLOCATE(JNDX)

IF (LHOOK) CALL DR_HOOK('SUSKF',1,ZHOOK_HANDLE)
end subroutine suskf
