SUBROUTINE TLPROP(YDINCR)

!     Purpose:
!     --------
!       Propagate increment by tangent linear model
!       backward in time  inside 4D-Var configuration.
!       It is  called in 3dfgat, at the end of
!       minimization  to propagate 
!       analysis increment back to beginning of
!       analysis window. The aim is to enable outer looping
!       just like in real 4D-Var. 
!     Author.
!     -------
!       G. Radnoti,  on the analogy of TLTEST

!     Modifications.
!     --------------
!       Original 31-Oct-2003
!        G. Radnoti : 20-July-2004 adaptation of TLTEST to TLPROP
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULOUT
USE YOMDIM  , ONLY: NSMAX, NMSMAX, NS1D 
USE YOMCVA  , ONLY: NVA3D, NVA2D, NGRBVAR
USE YOMCT0  , ONLY: NCONF, NSTART, NSTOP,LOBS
USE YOMCT2  , ONLY: NSTOP2
USE YOMVRTL , ONLY: LTLINT
USE YOMJG   , ONLY: LSUBBG
USE YOMVAR  , ONLY: NSIM4D, NSIM4DL, NITER4D, LJCDFI,L3DFGAT
USE YOMTNEWT, ONLY: LIDMODEL
USE YOEPHY  , ONLY: LEPHYS
USE YOPHNC  , ONLY: LETRAJP
USE YOEPHLI , ONLY: LPHYLIN
USE YOPHLC  , ONLY: LSPHLC
USE TRAJECTORY_MOD, ONLY: LTRAJHR, LTRAJSAVE, LPRTTRAJ, LTRAJRESET
USE SPECTRAL_FIELDS_MOD, ONLY: SPECTRAL_FIELD, DEALLOCATE_SPEC, ALLOCATE_SPEC, SPECTRAL_NORM_LEVS
USE CONTROL_VECTORS, ONLY: CONTROL_VECTOR, ALLOCATE_CTLVEC, DEALLOCATE_CTLVEC, ASSIGNMENT(=)
USE YOMTRAJ, ONLY: MSTEPTRAJW,MSTEPTRAJR,TSTEP_TRAJ
USE YOMDYN,    ONLY: TSTEP
USE YOMNMIA,   ONLY: NITNMI
USE YOMPPC ,   ONLY: LRSUP
USE YOMOP   ,  ONLY: CFNINSH
! ----------------------------------------------------------------------

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN) :: YDINCR

TYPE(control_vector) :: YLX
TYPE(spectral_field) :: ylspincrini,ylspincrfin,ylspincrini1,ylspincrdif,ylspincrevol
LOGICAL :: llsub, lldfi, lltra, llphy, llpli, lltrap, llidmodel
LOGICAL :: ll3dfgat,llsphlc,llrsup,llobs,lldiag
INTEGER(KIND=JPIM) :: iter4d,isim4d,istop,jiter,itermax
INTEGER(KIND=JPIM) :: isteptraj,i,itime,itsteplr,itstephr
REAL(KIND=JPRB) :: ZHOOK_HANDLE,ztstep
character :: clfinc*16
INTEGER(KIND=JPIM), ALLOCATABLE :: isteptrajw(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: isteptrajr(:)
#include "abor1.intfb.h"
#include "chavarin.intfb.h"
#include "cnt3tl.intfb.h"
#include "state2spec.intfb.h"
#include "restart_cnt3.intfb.h"
#include "sbsfgs.intfb.h"
! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TLPROP',0,ZHOOK_HANDLE)
IF (NCONF/=131)  CALL ABOR1('TLPROP not tested with NCONF/=131')
! Initialize
! ----------
lldiag=.false.
itermax=1
IF (lldiag) itermax=2
iter4d=NITER4D
isim4d=NSIM4D
llsub =LSUBBG
lldfi =LJCDFI
lltra =LTRAJHR
llphy =LEPHYS
lltrap=LETRAJP
llpli =LPHYLIN
llsphlc=LSPHLC
llrsup=LRSUP
llobs=LOBS
NITER4D=1
ztstep=TSTEP
istop=NSTOP
ALLOCATE(isteptrajr(nstart-nitnmi:istop))
ALLOCATE(isteptrajw(nstart-nitnmi:istop))
isteptrajr(:)=MSTEPTRAJR(:)
isteptrajw(:)=MSTEPTRAJW(:)
IF (lldiag) THEN
     CALL ALLOCATE_SPEC(ylspincrini,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
     CALL ALLOCATE_SPEC(ylspincrini1,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
     CALL ALLOCATE_SPEC(ylspincrfin,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
     CALL ALLOCATE_SPEC(ylspincrdif,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
     CALL ALLOCATE_SPEC(ylspincrevol,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
ENDIF
do jiter=1,itermax
  WRITE(NULOUT,*)'TLPROP ----------------------------------'
  WRITE(NULOUT,*)'TLPROP Starting propagating tangent linear model jiter=',jiter
  WRITE(NULOUT,*)'TLPROP ----------------------------------'
! avoid obs treatment
  LOBS=.false.
! force output during tl run:
  NSIM4D =NSIM4DL
  LRSUP=.FALSE.
!   
  LSUBBG =.true.
  LJCDFI =.false.
  IF (jiter == 1) CALL ALLOCATE_CTLVEC(YLX)

! instead of rerunning nonlinear model (CNT3)
! we use trajectories that are still in the memory 
! from the minimization. Keeping LTRAJHR=.TRUE.
! we make sure that the same trajectory is used
! as during minimization.
! TL run will be adiabatic:
  LEPHYS =.false.
  LETRAJP=.false.
! LSPHLC can be set true or false 
  LSPHLC=.true.
! Run linear model from increment of minimization
! -----------------------------------------------
  WRITE(NULOUT,*)'TLPROP Calling linear model'
  LTLINT=.TRUE.
  IF (jiter == 1) THEN
    CALL RESTART_CNT3
    YLX=YDINCR
    CALL CHAVARIN(YLX)
    CALL SBSFGS
    IF (lldiag) CALL STATE2SPEC(ylspincrini)
  ENDIF

! We have to take care that time goes backwards and that trajectory
! is taken also backwards starting from T/2 (T: 4dvar window length) and
! make sure abort if trajectory is intended to be modified
! or if intended to be read from outside range [0,T/2]
  MSTEPTRAJW(:)=HUGE(1)
  MSTEPTRAJR(:)=HUGE(1)
! set MSTEPTRAJR (based on code in init_trajectory), just reversing
  itstephr=nint(ztstep)
  itsteplr=nint(tstep_traj)
  isteptraj=0
  DO i=nstart-nitnmi,nstart-1
    isteptraj=isteptraj+1
    MSTEPTRAJR(i)=isteptraj
  ENDDO
  IF (jiter == 1) THEN
    DO i=istop/2,nstart,-1
      itime=(i-nstart)*itstephr
      IF (mod(itime,MAX(itsteplr,1))==0) THEN
          isteptraj=isteptraj+1
      ENDIF
      MSTEPTRAJR(i)=isteptraj
    ENDDO
  ELSE
    DO i=nstart,istop/2
      itime=(i-nstart)*itstephr
      IF (mod(itime,MAX(itsteplr,1))==0) THEN
          isteptraj=isteptraj+1
      ENDIF
      MSTEPTRAJR(i)=isteptraj
    ENDDO
  ENDIF
  DO i=nstart-nitnmi,istop
    write(NULOUT,*)'GREPTRAJ: MSTEPTRAJ(',i,')=',MSTEPTRAJW(i),MSTEPTRAJR(i)
  ENDDO
! 3DFGAT increment is valid at T/2, we have to propagate back to time 0
  NSTOP = istop/2
  NSTOP2=NSTOP
  IF (jiter == 1) THEN
    TSTEP=-ZTSTEP 
  ELSE
    TSTEP=ZTSTEP
  ENDIF
  WRITE(NULOUT,*)'TLPROP Running for',NSTOP-NSTART+1,'time-steps.'
  WRITE(NULOUT,*)'TLPROP ----------------------------------'

  WRITE(NULOUT,*)'TLPROP LETRAJP,LEPHYS,LPHYLIN=',LETRAJP,LEPHYS,LPHYLIN
! Backward TL should in no case be just an imitation with lidmodel=.t.
  llidmodel=LIDMODEL
  ll3dfgat=L3DFGAT
  LIDMODEL=.FALSE.
  L3DFGAT=.FALSE.
  clfinc=cfninsh
  write(cfninsh(1:2),'(I2.2)') jiter 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  CALL CNT3TL
!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  IF (jiter == 1 .and. lldiag) CALL STATE2SPEC(ylspincrfin)
  IF (jiter == 2 .and. lldiag) THEN 
     CALL STATE2SPEC(ylspincrini1)
     ylspincrdif%SP3D=ylspincrini1%SP3D - ylspincrini%SP3D
     ylspincrdif%SP2D=ylspincrini1%SP2D - ylspincrini%SP2D
     ylspincrevol%SP3D=ylspincrfin%SP3D - ylspincrini%SP3D
     ylspincrevol%SP2D=ylspincrfin%SP2D - ylspincrini%SP2D
     write(nulout,*) ' initial increment norms:'
     CALL SPECTRAL_NORM_LEVS(ylspincrini,'TLPROP')
     write(nulout,*) ' irreversibility difference norms:'
     CALL SPECTRAL_NORM_LEVS(ylspincrdif,'TLPROP')
     write(nulout,*) ' tendency norms:'
     CALL SPECTRAL_NORM_LEVS(ylspincrevol,'TLPROP')
  ENDIF
  WRITE(NULOUT,*)'TLPROP ----------------------------------'
  WRITE(NULOUT,*)'TLPROP End Tangent Linear Propagation'
  WRITE(NULOUT,*)'TLPROP ----------------------------------'
enddo
! some restoration
MSTEPTRAJR(:)=isteptrajr(:)
MSTEPTRAJW(:)=isteptrajw(:)
! some restoration
cfninsh=clfinc
LRSUP=llrsup
LOBS=llobs
LIDMODEL=llidmodel
L3DFGAT=ll3dfgat
TSTEP=ZTSTEP
DEALLOCATE(isteptrajr)
DEALLOCATE(isteptrajw)
LTLINT=.FALSE.
LTRAJHR=lltra
LEPHYS =llphy
LPHYLIN=llpli
LSPHLC=llsphlc
LETRAJP=lltrap
LPRTTRAJ  =.false.
LTRAJSAVE =.false.
LTRAJRESET=.false.
NITER4D=iter4d
NSIM4D =isim4d
LSUBBG =llsub
LJCDFI =lldfi
NSTOP=istop
NSTOP2=NSTOP
CALL DEALLOCATE_CTLVEC(YLX)
IF (lldiag) THEN
  CALL DEALLOCATE_SPEC(ylspincrini)
  CALL DEALLOCATE_SPEC(ylspincrfin)
  CALL DEALLOCATE_SPEC(ylspincrini1)
  CALL DEALLOCATE_SPEC(ylspincrdif)
  CALL DEALLOCATE_SPEC(ylspincrevol)
ENDIF
IF (LHOOK) CALL DR_HOOK('TLPROP',1,ZHOOK_HANDLE)
END SUBROUTINE TLPROP
