SUBROUTINE TLTEST(YDINCR)

!     Purpose:
!     --------
!       Test Tangent linear model inside 4D-Var configuration.

!     Author.
!     -------
!       Y. Tremolet

!     Modifications.
!     --------------
!       Original 31-Oct-2003
!     G Radnoti: 03/12/2004 reproducibility: reset rad. intervals after tltest 
!     JJMorcrette 04-04-2005 SRTM
!     G. Desroziers and K. Yessad (sept 2005):
!      - split option LTRAJHR into LTRAJHR_ALTI and LTRAJHR_SURF.
!      - adapt option LTRAJHR to METEO-FRANCE configurations.
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLUN  , ONLY: NULOUT, NULERR
USE YOMCT0  , ONLY: NCONF, NSTART, NSTOP, LOBS
USE YOMCT1  , ONLY: N1SDI
USE YOMVRTL , ONLY: LTLINT
USE YOMJG   , ONLY: LSUBBG
USE YOMDIM  , ONLY: NSMAX, NMSMAX, NS1D,NFLEVG
USE YOMCVA  , ONLY: NVA3D, NVA2D, NGRBVAR
USE YOMVAR  , ONLY: NSIM4D, NITER4D, LJCDFI
USE YOEPHY  , ONLY: LEPHYS
USE YOPHNC  , ONLY: LETRAJP, LERADSW2
USE YOEPHLI , ONLY: LPHYLIN
USE TESTVAR_MIX , ONLY: NSTOP_TLTEST, NPHYS_TLTEST
USE TRAJECTORY_MOD, ONLY: LTRAJHR,LTRAJHR_ALTI,LTRAJHR_SURF, LTRAJSAVE, LPRTTRAJ, LTRAJRESET
USE SPECTRAL_FIELDS_MOD
USE CONTROL_VECTORS
USE YOERAD  , ONLY: NTSW, NSW

! ----------------------------------------------------------------------

IMPLICIT NONE

TYPE(CONTROL_VECTOR), INTENT(IN) :: YDINCR

! ----------------------------------------------------------------------

INTEGER(KIND=JPIM), PARAMETER :: idia=5
INTEGER(KIND=JPIM), PARAMETER :: itermax=8
TYPE(control_vector) :: YLX
TYPE(spectral_field) :: YLSPTLM, YLSPNL0, YLSPNL1, YLSPDIF, YLSPERR
LOGICAL :: llsub, lldfi, lltra,lltra_alti,lltra_surf, llphy, llpli, lltrap, llobs
INTEGER(KIND=JPIM) :: iter4d,isim4d,istop,iter,jter,jf,js,jl,jdia
INTEGER(KIND=JPIM) :: incs,incl,il,is
REAL(KIND=JPRB) :: za,zd,zeps
REAL(KIND=JPRB) :: ztlm(NVA3D+NVA2D), zerr(NVA3D+NVA2D,itermax)
REAL(KIND=JPRB) :: zdia(NVA3D,idia,itermax)
INTEGER(KIND=JPIM) :: indx(idia),ilev(idia)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ----------------------------------------------------------------------

#include "abor1.intfb.h"
#include "chavarin.intfb.h"
#include "cnt3.intfb.h"
#include "cnt3tl.intfb.h"
#include "state2spec.intfb.h"
#include "addfgs.intfb.h"
#include "restart_cnt3.intfb.h"
#include "suaersn.intfb.h"
#include "suclopn.intfb.h"
#include "suswn.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TLTEST',0,ZHOOK_HANDLE)

! ----------------------------------------------------------------------

IF (NCONF/=131)  CALL ABOR1('TLTEST not tested with NCONF/=131')

WRITE(NULOUT,*)'TLTEST ----------------------------------'
WRITE(NULOUT,*)'TLTEST Starting test tangent linear model'
WRITE(NULOUT,*)'TLTEST ----------------------------------'

! Initialize
! -----------
zeps=EPSILON(zeps)
iter4d=NITER4D
isim4d=NSIM4D
llsub =LSUBBG
lldfi =LJCDFI
lltra =LTRAJHR
lltra_alti =LTRAJHR_ALTI
lltra_surf =LTRAJHR_SURF
llphy =LEPHYS
lltrap=LETRAJP
llpli =LPHYLIN
llobs=LOBS
NITER4D=1
NSIM4D =1
LSUBBG =.false.
LJCDFI =.false.

CALL ALLOCATE_CTLVEC(YLX)
CALL ALLOCATE_SPEC(YLSPTLM,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
CALL ALLOCATE_SPEC(YLSPNL0,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
CALL ALLOCATE_SPEC(YLSPNL1,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
CALL ALLOCATE_SPEC(YLSPDIF,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)
CALL ALLOCATE_SPEC(YLSPERR,NSMAX,NMSMAX,NVA3D,NVA2D,NS1D,NGRBVAR)

incs=INT((YLSPTLM%NSPEC2*(idia-1))/idia)
incl=INT(YLSPTLM%NFLEVL/idia)
IF (incs<1) incs=1
IF (incl<1) incl=1
is=1
il=1
DO jdia=1,idia
  is=is+incs
  IF (is>YLSPTLM%NSPEC2) is=is-YLSPTLM%NSPEC2
  indx(jdia)=is
  il=il+incl
  IF (il>YLSPTLM%NFLEVL) il=il-YLSPTLM%NFLEVL
  ilev(jdia)=il
ENDDO
SELECT CASE (NPHYS_TLTEST)
CASE(0)
  LEPHYS =.false.
  LETRAJP=.false.
  WRITE(NULOUT,*)'TLTEST Running without physics'
CASE(1)
  LEPHYS =LETRAJP
  LPHYLIN=LETRAJP
  WRITE(NULOUT,*)'TLTEST Running with available linear physics'
CASE(2)
  WRITE(NULOUT,*)'TLTEST Running with all available physics'
CASE DEFAULT
  WRITE(NULERR,*)'TLTEST NPHYS_TLTEST=',NPHYS_TLTEST
  CALL ABOR1('TLTEST: Error NPHYS_TLTEST')
END SELECT
IF (NSTOP_TLTEST>=0) THEN
  istop = NSTOP
  NSTOP = NSTART + NSTOP_TLTEST - 1
  IF (NSTOP>istop) CALL ABOR1('TLTEST: NSTOP_TLTEST is too large')
ENDIF
WRITE(NULOUT,*)'TLTEST Running for',NSTOP-NSTART+1,'time-steps.'
WRITE(NULOUT,*)'TLTEST ----------------------------------'

WRITE(NULOUT,*)'TLTEST LETRAJP,LEPHYS,LPHYLIN=',LETRAJP,LEPHYS,LPHYLIN

! Run nonlinear model and get trajectory
! --------------------------------------
LTRAJHR   =.false.
LTRAJHR_ALTI   =.false.
LTRAJHR_SURF   =.false.
LTRAJSAVE =.true.
LTRAJRESET=.true.
LOBS=.false.
N1SDI=1

WRITE(NULOUT,*)'TLTEST Calling nonlinear model 0'
CALL RESTART_CNT3

YLX=0.0_JPRB
CALL CHAVARIN(YLX)
CALL ADDFGS

CALL CNT3

LTRAJSAVE =.false.
LTRAJRESET=.false.

CALL STATE2SPEC(YLSPNL0)
CALL SPECTRAL_NORM(YLSPNL0,'TLTEST NL 0')
WRITE(NULOUT,*)'TLTEST ----------------------------------'

! Run linear model from increment
! -------------------------------
WRITE(NULOUT,*)'TLTEST Calling linear model'

LTLINT=.TRUE.
CALL RESTART_CNT3

YLX=YDINCR
CALL CHAVARIN(YLX)

CALL CNT3TL
LTLINT=.FALSE.

CALL STATE2SPEC(YLSPTLM)
CALL SPECTRAL_NORM(YLSPTLM,'TLTEST TL  ',ztlm)
WRITE(NULOUT,*)'TLTEST ----------------------------------'

! ---------------------------
! Loop over perturbation size
! ---------------------------
IF (ANY(ztlm(:)<=zeps)) THEN
  iter=1  ! Do only once if perturbation is zero.
ELSE
  iter=itermax
ENDIF
N1SDI=0

za=1.0_JPRB
DO jter=1,iter

! Run nonlinear model from trajectory+increment
! ---------------------------------------------
  WRITE(NULOUT,'(A,ES8.2)')' TLTEST Calling nonlinear model za=',za
  CALL RESTART_CNT3

  YLX%data(:)=za*YDINCR%data(:)
  CALL CHAVARIN(YLX)
  CALL ADDFGS

  CALL CNT3

  CALL STATE2SPEC(YLSPNL1)

! Calculate diagnostics
! ---------------------
! Finite difference derivative
  YLSPDIF%SP3D=YLSPNL1%SP3D-YLSPNL0%SP3D
  YLSPDIF%SP2D=YLSPNL1%SP2D-YLSPNL0%SP2D

  zd=1.0_JPRB/za
  YLSPDIF%SP3D=zd*YLSPDIF%SP3D
  YLSPDIF%SP2D=zd*YLSPDIF%SP2D

! Error norm
  YLSPERR%SP3D=YLSPDIF%SP3D-YLSPTLM%SP3D
  YLSPERR%SP2D=YLSPDIF%SP2D-YLSPTLM%SP2D
  CALL SPECTRAL_NORM(YLSPERR,zerr(:,jter))

! Error coef
  DO jdia=1,idia
    jl=ilev(jdia)
    js=indx(jdia)
    DO jf=1,YLSPERR%NS3D
      IF (YLSPTLM%SP3D(jl,js,jf)/=0.0_JPRB) THEN
        zdia(jf,jdia,jter) = YLSPDIF%SP3D(jl,js,jf) / YLSPTLM%SP3D(jl,js,jf)
      ELSE
        zdia(jf,jdia,jter) = 123456789.0_JPRB
      ENDIF
    ENDDO
  ENDDO

  za=za/10.0_JPRB
ENDDO
WRITE(NULOUT,*)'TLTEST ----------------------------------'
WRITE(NULOUT,*)'TLTEST End Loop'
WRITE(NULOUT,*)'TLTEST ----------------------------------'

IF (ANY(ztlm(:)<=zeps)) THEN
  WRITE(NULOUT,*)'TLTEST Perturbation too small'
ELSE
  WRITE(NULOUT,*)'TLTEST               Vor        Div        ', &
   & 'Tem        Hum         O3       S.Pres'  
  DO jter=1,iter
    zerr(:,jter)=zerr(:,jter)/ztlm(:)
    WRITE(NULOUT,'(A,6(2X,ES9.3))')' TLTEST Rel Error:',zerr(:,jter)
  ENDDO
ENDIF

DO jdia=1,idia
  WRITE(NULOUT,*)'TLTEST -----------------------------------------------'
  WRITE(NULOUT,*)'TLTEST Convergence test, level=',ilev(jdia), &
   & ' spectral index=',indx(jdia)  
  WRITE(NULOUT,*)'TLTEST       Vorticity    Divergence    Temperature  ', &
   & '  Humidity        Ozone'  
  DO jter=1,iter
    WRITE(NULOUT,'(A,5(2X,F12.10))')' TLTEST T1',zdia(:,jdia,jter)  
  ENDDO
ENDDO

! ------------------------------------------------------------------
! Run nonlinear model to restore trajectory
! -----------------------------------------
LTRAJHR=lltra
LTRAJHR_ALTI=lltra_alti
LTRAJHR_SURF=lltra_surf
LEPHYS =llphy
LPHYLIN=llpli
LETRAJP=lltrap
!rg uncomment following lines
WRITE(NULOUT,*)'TLTEST ----------------------------------'
WRITE(NULOUT,*)'TLTEST Restoring trajectory'

N1SDI=1
NITER4D=0
NSIM4D =0
LPRTTRAJ  =.true.
LTRAJSAVE =.true.
LTRAJRESET=.true.
CALL RESTART_CNT3

CALL CNT3
!rg uncomment finished
LPRTTRAJ  =.false.
LTRAJSAVE =.false.
LTRAJRESET=.false.
LOBS=llobs
!reset  radiation setting like in cva1
IF (LEPHYS.AND.LERADSW2) THEN
    IF (NSW > 2) NSW=2
    CALL SUSWN  (NTSW, NSW)
    CALL SUCLOPN(NTSW, NSW, NFLEVG)
    CALL SUAERSN(NTSW, NSW)
ENDIF
! ------------------------------------------------------------------

CALL DEALLOCATE_CTLVEC(YLX)
CALL DEALLOCATE_SPEC(YLSPTLM)
CALL DEALLOCATE_SPEC(YLSPNL0)
CALL DEALLOCATE_SPEC(YLSPNL1)
CALL DEALLOCATE_SPEC(YLSPDIF)
CALL DEALLOCATE_SPEC(YLSPERR)
NITER4D=iter4d
NSIM4D =isim4d
LSUBBG =llsub
LJCDFI =lldfi
IF (NSTOP_TLTEST>0) NSTOP=istop
WRITE(NULOUT,*)'TLTEST ----------------------------------'
WRITE(NULOUT,*)'TLTEST End Tangent Linear Test'
WRITE(NULOUT,*)'TLTEST ----------------------------------'

! ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TLTEST',1,ZHOOK_HANDLE)
END SUBROUTINE TLTEST
