SUBROUTINE UPSPEC

!**** *UPSPEC*  - Updates spectral arrays (NCONF=131).

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *UPSPEC

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel Thepaut *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-12-25 (Merry Christmas !!)
!        Modified : 95-06-06 L. Isaksen - Reordering of spectral arrays
!        Modified : 95-10-30 M.Hamrud - Removed CLCONF
!        Modified : 96-01-29 M.Fisher - Allow NTLPEC2G.ne.NSPEC2
!     J-N Thepaut : 95-08-29 (update spectral array via fullpos)
!     E Andersson : 96-04-29 LARCHFP for ECMWF use
!     E. Holm     : 97-11-28 Call to SUSPEC changed (incl. ozone) 
!     L.Dragulanescu : 97-05-05 ALADIN 3DVAR
!     R. El Khatib: 98-04-02 Cleanup ALADIN 3DVAR
!     Y.Tremolet  : 02-05-21 Cleanup
!     Y.Tremolet  : 02-08-21 Cleanup more
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!        G. Radnoti    21-Jun-2007 Allow late4dstart for overlapping long windows
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NSTART   ,LOBSC1
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT3   , ONLY : NSTEP
USE YOMNMIA  , ONLY : NITNMI
USE YOMVAR   , ONLY : NUPTRA, LMODERR, LREABVEC
USE YOMMODEL_ERROR, ONLY : NTYPE_MODERR, L_WARM_START_MODERR, &
  & NVE3GP, NVE2GP, NGRBMERRGP, NSERR, NVE3SP,NVE2SP, NGRBMERRSP
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD
IMPLICIT NONE

REAL(KIND=JPRB) :: ZHOOK_HANDLE
TYPE(GRIDPOINT_FIELD) :: YLGRID
TYPE(SPECTRAL_FIELD) :: YLSPEC
#include "abor1.intfb.h"
#include "rdfpinc.intfb.h"
#include "stepo.intfb.h"
#include "rdmoderr.intfb.h"
#include "model2moderr.intfb.h"
!      -----------------------------------------------------------

!*       1.    Read/write spectral arrays from previous variational file.
!              ----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('UPSPEC',0,ZHOOK_HANDLE)
IF (.NOT.LOBSC1) CALL ABOR1('UPSPEC: ONLY LOBSC1=TRUE SUPPORTED')

NSTEP=NSTART-NITNMI

IF (LECMWF) THEN
  IF (NUPTRA <= 1 .AND. LMODERR .AND. NTYPE_MODERR==5 .AND. L_WARM_START_MODERR) THEN
!   replace initial field by the model error field from previous cycle valid at same time
!   for overlapping long windows; already first traj may differ from bckgr
    CALL ALLOCATE_GRID(YLGRID,NVE3GP,NVE2GP,NGRBMERRGP)
    CALL ALLOCATE_SPEC(YLSPEC,NSERR,0,NVE3SP,NVE2SP,0,NGRBMERRSP)
    CALL RDMODERR('spmoderr_bg_ini_01',YLSPEC,YLGRID)
    CALL MODEL2MODERR(-1,YLSPEC,YLGRID)
    CALL DEALLOCATE_SPEC(YLSPEC)
    CALL DEALLOCATE_GRID(YLGRID)
  ENDIF
  IF (NUPTRA>0) CALL rdfpinc
ELSE
  IF (NUPTRA>0) THEN
    call rdfpinc
  ELSE
    CALL STEPO('T00000000')
  ENDIF
ENDIF
IF (NUPTRA == 0 .AND. LREABVEC) THEN
  call rdfpinc
ENDIF

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('UPSPEC',1,ZHOOK_HANDLE)
END SUBROUTINE UPSPEC
