SUBROUTINE WEAK_CONSTRAINT

!     Purpose.
!     --------
!       Apply weak constraint.

!     Author.
!     -------
!       Y.Tremolet

!     Modifications.
!     --------------
!        Original : 19-Aug-2005
!        G. Radnoti: 27-Nov-2006: super-saturation check, etc with RDFPINC 
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCT0  , ONLY: LIFSTRAJ, LIFSMIN
USE YOMVAR  , ONLY: NUPTRA, MUPTRA
USE YOMCT3  , ONLY: NSTEP
USE YOMLUN  , ONLY: NULOUT
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, MSTEPERR, SPMODERR, GPMODERR, &
                     & LFGMODERR, SPFGMODERR, GPFGMODERR, &
                     & NVE3GP, NVE2GP, NGRBMERRGP, LOVERLAPPING4D, &
                     & NVE3SP, NVE2SP, NGRBMERRSP, NSERR, NPRTMODERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(GRIDPOINT_FIELD) :: YLGRID, YLGRID_INCR
TYPE(SPECTRAL_FIELD) :: YLSPEC, YLSPEC_INCR
INTEGER(KIND=JPIM) :: ic
LOGICAL :: LLFG,LLFPINC
CHARACTER(LEN=14) :: clfile
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "rdmoderr.intfb.h"
#include "wrmoderr.intfb.h"
#include "rdfpinc.intfb.h"
#include "model2moderr.intfb.h" 
#include "abor1.intfb.h"
#include "spnorm.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WEAK_CONSTRAINT',0,ZHOOK_HANDLE)

IF (NTYPE_MODERR/=1 .AND. NTYPE_MODERR/=5) CALL ABOR1('WEAK_CONSTRAINT: Error NTYPE_MODERR')

ic=MSTEPERR(NSTEP)
LLFPINC=.FALSE.
IF (ic>0) THEN

! In ifstraj and forecast, GPMODERR(ic) could be allocated
! and deallocated here if memory is an issue for long windows.

! Most of the I/O could be done through normal post-processing
! but one would have to get through the CDCONF/STEPO firewall...

  IF (LIFSTRAJ) THEN
    CALL ALLOCATE_GRID(YLGRID,NVE3GP,NVE2GP,NGRBMERRGP)
    CALL ALLOCATE_SPEC(YLSPEC,NSERR,0,NVE3SP,NVE2SP,0,NGRBMERRSP)

!   Save M(x_{i-1})

!      SPA3...==> YLSPEC...
    CALL MODEL2MODERR(1,YLSPEC,YLGRID)

    IF (NUPTRA>0.OR.LFGMODERR) THEN
      LLFG=.true.
!     In trajectory job, x_k is only needed once,
!     compute it here, by reading the FG (if any)...
      clfile(1:14)='spmoderrfgXXYY'
      IF (NUPTRA == 0) THEN
        write(clfile(11:12),'(I2.2)') 99
      ELSE
        write(clfile(11:12),'(I2.2)')NUPTRA-1
      ENDIF
      write(clfile(13:14),'(I2.2)')ic
      WRITE(NULOUT,*)'WEAK_CONSTRAINT: Reading model error from ',clfile
      IF ( NUPTRA > 0 .OR. (NUPTRA == 0 .AND. LOVERLAPPING4D(ic)) ) THEN
        WRITE(NULOUT,*) 'Reading model error from file ',clfile
        CALL RDMODERR(clfile,SPMODERR(ic),GPMODERR(ic))
      ELSE
        WRITE(NULOUT,*) 'Setting model error field equal to actual state'
        SPMODERR(ic)=YLSPEC
        GPMODERR(ic)=YLGRID        
      ENDIF
!     ...and adding the increment if needed.
      IF (NUPTRA>0) THEN
        clfile(1:14)='spmoderrinXXYY'
        write(clfile(11:12),'(I2.2)')NUPTRA-1
        write(clfile(13:14),'(I2.2)')ic
        WRITE(NULOUT,*)'WEAK_CONSTRAINT: Reading model error from ',clfile
        CALL ALLOCATE_GRID(YLGRID_INCR,NVE3GP,NVE2GP,NGRBMERRGP)
        CALL ALLOCATE_SPEC(YLSPEC_INCR,NSERR,0,NVE3SP,NVE2SP,0,NGRBMERRSP)       
        CALL RDMODERR(clfile,YLSPEC_INCR,YLGRID_INCR)
!rg        CALL SELF_ADD(SPMODERR(ic),YLSPEC_INCR)
!rg        CALL SELF_ADD(GPMODERR(ic),YLGRID_INCR)
!       CALL something like in RDFPINC for supersaturation check, etc...
!                SPMODERR...==> SPA3...
        CALL MODEL2MODERR(-1,SPMODERR(ic),GPMODERR(ic))
        CALL RDFPINC(YLSPEC_INCR,YLGRID_INCR)
        LLFPINC=.TRUE.
!       LLFPINC: rdfpinc called==> satur-checked sum FG+INCR already in SPA3,SPA2,GMV,GFL...
      ENDIF
    ELSE
      LLFG=.false.
    ENDIF

  ELSEIF (LIFSMIN) THEN
    LLFG=.true.
    IF (LFGMODERR) THEN
      IF (NTYPE_MODERR == 1) THEN
        SPMODERR(ic)=SPFGMODERR(ic)
        GPMODERR(ic)=GPFGMODERR(ic)
      ELSE
!       ntype_moderr =5:
        clfile(1:14)='spmoderrfgXXYY'
        write(clfile(11:12),'(I2.2)')NUPTRA
        write(clfile(13:14),'(I2.2)')ic
        WRITE(NULOUT,*)'WEAK_CONSTRAINT: Reading model error from ',clfile
        CALL RDMODERR(clfile,SPMODERR(ic),GPMODERR(ic))
      ENDIF
    ELSE
!rg:
      LLFG=.FALSE.
      SPMODERR(ic)=0.0_JPRB
      GPMODERR(ic)=0.0_JPRB
    ENDIF

  ELSE ! forecast
    CALL ABOR1('weak_constraint is not ready for forecast mode')
    LLFG=.true.
    clfile(1:14)='spmoderror_YYY'
    write(clfile(12:14),'(I3.3)')ic
    write(nulout,*)'WEAK_CONSTRAINT: Reading model error from ',clfile
    CALL RDMODERR(clfile,SPMODERR(ic),GPMODERR(ic))
  ENDIF

! Apply weak constraint
! ---------------------
  IF (LLFG) THEN
!              SPMODERR...==>SPA3...         
    IF (LLFPINC) THEN
!     SPA3==>SPMODERR
      CALL MODEL2MODERR(1,SPMODERR(ic),GPMODERR(ic))
    ELSE
!     SPMODERR==>SPA3
      CALL MODEL2MODERR(-1,SPMODERR(ic),GPMODERR(ic))
    ENDIF
    IF (NPRTMODERR>=1) &
     & WRITE(NULOUT,*)'WEAK_CONSTRAINT applied, NSTEP, ic=',NSTEP,ic
    IF (NPRTMODERR>=1) call spnorm(0)
  ELSE
    IF (NPRTMODERR>=1) &
     & WRITE(NULOUT,*)'WEAK_CONSTRAINT not applied, NSTEP, ic=',NSTEP,ic
  ENDIF

  IF (LIFSTRAJ) THEN
!     Save FG for next min and outer loop (or AN if last outer loop)
      clfile(1:14)='spmoderrfgXXYY'
      write(clfile(11:12),'(I2.2)')NUPTRA
      write(clfile(13:14),'(I2.2)')ic
      IF (NPRTMODERR>=1) WRITE(NULOUT,*)'WEAK_CONSTRAINT: Saving ',clfile
      CALL WRMODERR(clfile,SPMODERR(ic),GPMODERR(ic))

!     Compute and save M(x_{i-1})-x_i
    IF (NUPTRA<MUPTRA) THEN
      CALL SELF_SUB(YLSPEC,SPMODERR(ic))
      CALL SELF_SUB(YLGRID,GPMODERR(ic))
      IF (NTYPE_MODERR == 5) THEN
!       in case of NTYPE_MODERR = 5 we need x_i - M(x_{i-1})
        CALL SELF_MUL(YLSPEC,-1.0_JPRB)
        CALL SELF_MUL(YLGRID,-1.0_JPRB)
      ENDIF
      clfile(1:14)='spmoderrjqXXYY'
      write(clfile(11:12),'(I2.2)')NUPTRA
      write(clfile(13:14),'(I2.2)')ic
      IF (NPRTMODERR>=1) WRITE(NULOUT,*)'WEAK_CONSTRAINT: Saving ',clfile
      CALL WRMODERR(clfile,YLSPEC,YLGRID)
    ENDIF
    CALL DEALLOCATE_SPEC(YLSPEC)
    CALL DEALLOCATE_GRID(YLGRID)
    IF (NUPTRA > 0) THEN
      CALL DEALLOCATE_SPEC(YLSPEC_INCR)
      CALL DEALLOCATE_GRID(YLGRID_INCR)
    ENDIF
  ENDIF

ENDIF

IF (LHOOK) CALL DR_HOOK('WEAK_CONSTRAINT',1,ZHOOK_HANDLE)
END SUBROUTINE WEAK_CONSTRAINT
