SUBROUTINE WEAK_CONSTRAINT_TL

!     Purpose.
!     --------
!       Apply weak constraint.

!     Author.
!     -------
!       Y.Tremolet

!     Modifications.
!     --------------
!        Original : 19-Aug-2005
!     ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMCT3  , ONLY: NSTEP
USE YOMLUN  , ONLY: NULOUT
USE YOMVAR  , ONLY: NSIM4D, NSIM4DL, NUPTRA
USE YOMJQ   , ONLY: SPJQMODERR
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, MSTEPERR, SPMODERR, GPMODERR, &
                     & NSERR, NVE3SP, NVE2SP, NVE3GP, NVE2GP, &
                     & NGRBMERRSP, NGRBMERRGP, SPGPMODERR, NPRTMODERR
USE GRIDPOINT_FIELDS_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

TYPE(GRIDPOINT_FIELD) :: YLGRID
TYPE(SPECTRAL_FIELD) :: YLSPEC
INTEGER(KIND=JPIM) :: ic
CHARACTER(LEN=14) :: clfile
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "wrmoderr.intfb.h"
#include "grid2spec.intfb.h"
#include "spec_concat.intfb.h"
#include "abor1.intfb.h"
#include "model2moderr.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WEAK_CONSTRAINT_TL',0,ZHOOK_HANDLE)

IF (NTYPE_MODERR/=1) CALL ABOR1('WEAK_CONSTRAINT_TL: Error NTYPE_MODERR')

ic=MSTEPERR(NSTEP)

!  SPJQMODERR<--SPA3-SPMODERR
!  SPA3<--SPMODERR


IF (ic>0) THEN
!rg  CALL ABOR1('WEAK_CONSTRAINT_TL: Not ready for SPMODERR')
  CALL ALLOCATE_GRID(YLGRID,NVE3GP,NVE2GP,NGRBMERRGP)
  CALL ALLOCATE_SPEC(YLSPEC,NSERR,0,NVE3SP,NVE2SP,0,NGRBMERRSP)
!          SPA3...==>YLSPEC...
  CALL MODEL2MODERR(1,YLSPEC,YLGRID)
  CALL SELF_SUB(YLSPEC,SPMODERR(ic))
  CALL SELF_SUB(YLGRID,GPMODERR(ic))
!          SPMODERR...===>SPA3...
  CALL MODEL2MODERR(-1,SPMODERR(ic),GPMODERR(ic))
  CALL GRID2SPEC(YLGRID,SPGPMODERR)
  CALL SPEC_CONCAT(YLSPEC,SPGPMODERR,SPJQMODERR(ic))
  IF (NPRTMODERR>=1) &
    & WRITE(NULOUT,*)'WEAK_CONSTRAINT_TL applied, NSTEP, ic=',NSTEP,ic

! Save increment for next outer loop.
  IF (NSIM4D==NSIM4DL) THEN
    clfile(1:14)='spmoderrinXXYY'
    write(clfile(11:12),'(I2.2)')NUPTRA
    write(clfile(13:14),'(I2.2)')ic
    CALL WRMODERR(clfile,SPMODERR(ic),GPMODERR(ic))
  ENDIF

  CALL DEALLOCATE_SPEC(YLSPEC)
  CALL DEALLOCATE_GRID(YLGRID)
ENDIF

IF (LHOOK) CALL DR_HOOK('WEAK_CONSTRAINT_TL',1,ZHOOK_HANDLE)
END SUBROUTINE WEAK_CONSTRAINT_TL
