SUBROUTINE WRNHTRAJM (KSTEP,KSTA,KEND,KJUMP,KFIELDS,&
 & PSPD95,PSVD95,PDIVT95,PUT95,PVT95,PTT95,PSP95)  

!**** *WRNHTRAJM* - Write out the trajectory at t-dt to work file
!                   for some nonhydrostatic fields

! Purpose
! -------
!   Write out the trajectory at t-dt to work file
!   for some nonhydrostatic fields

! Interface
! ---------

!   KSTEP   - time step                (input)
!   KSTA    - start adress             (input)
!   KEND    - end adress               (input)
!   KJUMP   - offset into gp array     (input)
!   KFIELDS - number of fields         (input)
! Trajectory to be written in a buffer (input):
!   PSPD95
!   PSVD95
!   PDIVT95
!   PUT95
!   PVT95
!   PTT95
!   PSP95

! Externals 
! ---------
!   FILLB    - interface to packing routines
!   MIOWRITE - write routine from MIO package

! Method
! ------

! Reference
! ---------

! Author
! ------
!      Sep-1998 M. Charron (CNRM/GMAP - visitor)

! Modifications
! -------------
!   23-Apr-2002 C. Fischer  transpose trajnh for cy25 
!   01-Oct-2003 M. Hamrud   CY28 Cleaning
!   10-Jan-2004 M. Hamrud   CY28R1 Cleaning
!   09-Jun-2004 J. Masek    NH cleaning (LVSLWBC)
! End Modifications
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMIOP   , ONLY : NPCKFT95 ,NEXPBT95
USE YOMTNH   , ONLY : NLENNH95B,TRAJNH

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KJUMP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPD95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSVD95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIVT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP95(NPROMA) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IBLOCKS, IJUMP, ILENB, IPCKUNIT, ISTAGPB, IWORDS, JF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "fillb.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRNHTRAJM',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    WRITE OUT GRIDPOINT ARRAY.
!              --------------------------

!*       1.1       CALCULATE LENGTHS

IWORDS = KEND-KSTA+1

!*       1.2       WRITE OUT BUFFER

ILENB   = NLENNH95B
IJUMP=KJUMP-1
IF (NPCKFT95 == 1) THEN
  ISTAGPB = 1+IJUMP*KFIELDS
ELSE

!  It is assumed that TRAJNH data always is read/written in
!  NPROMA chunks (except for the last potentially smaller chunk).
!  Restrictions are necessary because reading/writing of packed
!  data has to start on a word boundary.

  IBLOCKS=IJUMP/NPROMA
  IF (IBLOCKS*NPROMA /= IJUMP)&
   & CALL ABOR1('WRNHTRAJ: INCONSISTENT WRITING OF PACKED DATA')  
  IPCKUNIT = (NPROMA+NPCKFT95-1)/NPCKFT95
  ISTAGPB = 1+IPCKUNIT*IBLOCKS*KFIELDS
ENDIF

!        1.3      UPPER AIR 3D FIELDS

DO JF=1,NFLEVG
  CALL FILLB(IWORDS,PSPD95 (KSTA,JF),TRAJNH(:,KSTEP),&
   & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  
  CALL FILLB(IWORDS,PSVD95 (KSTA,JF),TRAJNH(:,KSTEP),&
   & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  
  CALL FILLB(IWORDS,PDIVT95(KSTA,JF),TRAJNH(:,KSTEP),&
   & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  
  CALL FILLB(IWORDS,PUT95  (KSTA,JF),TRAJNH(:,KSTEP),&
   & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  
  CALL FILLB(IWORDS,PVT95  (KSTA,JF),TRAJNH(:,KSTEP),&
   & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  
  CALL FILLB(IWORDS,PTT95  (KSTA,JF),TRAJNH(:,KSTEP),&
   & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  
ENDDO
CALL FILLB(IWORDS,PSP95(KSTA),TRAJNH(:,KSTEP),&
 & ILENB,ISTAGPB,NPCKFT95,NEXPBT95)  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRNHTRAJM',1,ZHOOK_HANDLE)
END SUBROUTINE WRNHTRAJM
