      SUBROUTINE oldBUFR_READ(kunit,
     $                        kbuf, kbufsize, 
     $                        kbuflen, kbuflensize,
     $                        kwhat, kret)

      implicit none

c
c***  SUBROUTINE BUFR_READ(kunit,
c    $                     kbuf, kbufsize, 
c    $                     kbuflen, kbuflensize,
c    $                     kwhat, kret)
c
c     Sami Saarinen, ECMWF, 19/10/95
c
c***  Purpose:
c
c     To read data from the BUFR file
c
c***  Externals:
c
c     PBBUFR    - Reads the next BUFR-report (message) from BUFR-file
c
c***  Notes:
c
c     Coded in Fortran-90
c
c***  Input variables
      integer, intent(in) :: kunit ! PBIO file pointer
      integer, intent(in) :: kwhat ! Type of read:
                                   ! 1 = Read the next report
                                   ! n = Read the 'n' next reports
                                   !-1 = Read all the remaining reports
      integer, intent(in) :: kbufsize    ! Length of the input kbuf-array
      integer, intent(in) :: kbuflensize ! Length of the BUFR-length array
c***  Output variables
      integer, intent(out):: kbuf(kbufsize)       ! Input array for BUFR-data
      integer, intent(out):: kbuflen(kbuflensize) ! BUFR-length array
      integer, intent(out):: kret  ! Return code:
                                   !   >= 0 , # of BUFR-reports read
                                   ! -1 = Error in reading data
                                   ! -2 = kbufsize was not large
                                   !      enough to hold all the data
                                   ! -3 = kbuflensize was not large
                                   !      enough to hold the BUFR-lengths
c-----------------------------------------------------------------------
! === END OF INTERFACE BLOCK ===

      integer,  parameter :: jp_bignum = 2147483647
      integer iret_pbio
      integer i_reports, i_reports_read, i_report_len
      integer j, i_ptrbufr
      integer nobytes

c-----------------------------------------------------------------------
c
c     1. Read BUFR-reports (messages) from BUFR-file
c        -------------------------------------------
c

      inquire(iolength=nobytes) kbufsize

      if ( kwhat < 0 ) then
         i_reports = jp_bignum
      else
         i_reports = kwhat
      endif

      i_reports_read = 0
      i_ptrbufr = 0

      kbuf(:) = 0

      do j=1,i_reports
         call PBBUFR(kunit, 
     $        kbuf(i_ptrbufr + 1), nobytes*(kbufsize - i_ptrbufr),
     $        i_report_len, iret_pbio)

         if (iret_pbio == -1) then
            if (kwhat < 0) exit ! do j=1,i_reports
         endif

         if (iret_pbio < 0) then
            kret = -1
            if (iret_pbio == -3) kret = -2
            return
         endif

         i_reports_read = i_reports_read + 1
         if (i_reports_read > kbuflensize) then
            kret = -3
            return
         endif

c--         i_report_len = i_report_len/nobytes + 1
         i_report_len = (i_report_len+nobytes-1)/nobytes
         kbuflen(j) = i_report_len
         i_ptrbufr = i_ptrbufr + i_report_len

         if (i_ptrbufr > kbufsize) then
            kret = -2
            return
         endif
      enddo

      kret = i_reports_read

      return
      END SUBROUTINE oldBUFR_READ
