#!/bin/ksh
# --------------------------------------------------------------
# Transformation eventuelle d'un fichier de DDH de sa forme LFI a LFA.
# --------------------------------------------------------------
# Methode:
# Si le fichier est un fichier LFI, il est transforme en LFA.
# Le fichier LFA resultant ecrase alors le fichier d'entree.
# Si le fichier est un fichier LFA, rien n'est effectue.
# Externes: lecteur de fichier lfa (ddhr) et convertisseur en fichier lfa (lfi2lfa)
# Auteur:   97-01, J.M. Piriou.
# Modifications: 98-06, J.Barckicke
# --------------------------------------------------------------
# En entree: liste de nom de fichiers
# En sortie: fichiers (transformes en ) lfa
# --------------------------------------------------------------
: ${ddhr=ddhr} ${lfi2lfa=lfi2lfa}
if [ $# -eq 0 ]; then
	echo "Conversion eventuelle de fichiers DDH LFI vers leur forme LFA."
	echo "Utilisation: lfi2lfa.sh f1 [f2] ... [fn]"
	echo "Les fichiers fi sont en entree/sortie."
	exit 1
fi
for fent in $*
do
	res=$($ddhr -type $fent -Wl,-d200,-g250) # ddhr -type: donne le type du fichier: LFA ou non.
	if [ $? -ne 0 ] ; then
		exit 1
	fi
	if [ "$res" = "NON_lfa" ]; then
		echo "Le fichier "$fent" n'est pas LFA; conversion LFI > LFA."
		$lfi2lfa $fent $fent.lfa -Wl,-d200,-g250
		if [ $? -ne 0 ]; then exit 1; fi
		mv $fent.lfa $fent        # le resultat ecrase le fichier d'entree.
	else
		echo "Le fichier "$fent" est deja LFA; pas de conversion."
	fi
done
