      MODULE FAMODU

      IMPLICIT NONE

#include "precision.h"
!
#include "facomp.h"

      SAVE
!*
!      FAMODU - MODULE POUR LE LOGICIEL FA

!        D. PARADIS       METEO FRANCE     21/7/00

!  ----------------------------------------------------------------------
!  1 - COEFFICIENTS MULTIPLICATIFS DES COEFF SPECTRAUX
!
!     XLAP1D  = Coefficients elementaires pour "laplacien" [n*(n+1)],
!              ainsi que leurs inverses.
!     XLAP2D  = Cf. XLAP1D, mais etale dans un champ complet, et calcules
!              pour les puissances 1 a JPUILA, ainsi que les inverses.
!     FLAP1D  = Tableau des "puissances de laplacien" pour l'ecriture
!                     (n*(n+1))**NPUFLA()
!              ( Ce dernier tableau n'est calcule que pour une
!                puissance de laplacien non nulle )
!     XLAP1DA = ALADIN version de XLAP1D
!     XLAP2DA = ALADIN version de XLAP2D
!     FLAP1DA = ALADIN version de FLAP1D
!     LIXLAP  = .T. s'il faut initialiser XLAPxDx
!     LIFLAP() = .T. s'il faut initialiser FLAP1Dx()
!
!  2 - EN-TETE POUR GRIBEX (SECTIONS 1 ET 2)
!
!     JPSEC1   = taille du tableau contenant les elements de la section 1
!     JPSEC2   = taille du tableau contenant les elements de la section 2
!     JPSEC4   = taille du tableau contenant les elements de la section 4
!     NSEC1    = tableau contenant les elts 2:21 de la section 1 de GRIBEX
!     LISEC1() = .T. s'il faut initialiser le tableau NSEC1 ci-dessus
!
!      Tableaux contenant des elements de la section 2 de GRIBEX:
!     NSEC2SP  = cas de la representation spectrale ARPEGE avec niveaux modele
!     NSEC2GG  = cas de la grille de Gauss
!     NSEC2LL  = cas de la grille latitude-longitude
!     NSEC2LA  = cas de la grille Lambert conforme (type general de Aladin)
!     NSEC2AL  = cas de la representation spectrale Aladin deguisee lat-lon
!     NSC2ALF  = supplement a NSEC2AL dependant du fichier
!     XSEC2    = coordonnee verticale
!     LISEC2() = .T. s'il faut initialiser les tableaux ci-dessus sauf NSC2ALF
!     LISC2F() = .T. s'il faut initialiser NSC2ALF
!
!  3 - PARAMETRES DEFINISSANT LE CODAGE GRIBEX
!
!     NCODGRI  = tableau contenant les parametres de codage implicites
!     NCOGRIF  = tableau contenant les parametres de codage pour chaque fichier
!     CIPREF   = tableau contenant les prefixes des noms des champs connus
!     CISUFF   = tableau contenant les suffixes des noms des champs connus
!     NCODPA   = tableau contenant les 6 descripteurs GRIB des champs connus
!                NCODPA(i,1)= numero de version de la table de code parametre,
!                             KSEC1(1)
!                NCODPA(i,2)= indicateur de parametre, KSEC1(6)
!                NCODPA(i,3)= indicateur de type de niveau, KSEC1(7)
!                NCODPA(i,4)= niveau (premier niveau de la couche), KSEC1(8)
!                NCODPA(i,5)= deuxieme niveau de la couche, KSEC1(9)
!                NCODPA(i,6)= indicateur de type de champ, KSEC1(18)
!                             (0 sauf si min/max dans le temps => 2,
!                                  ou si cumul dans le temps   => 4 )
!     JPXPAR   = nombre maximal de champs connus pour CISUFF, CIPREF et NCODPA
!     NBPARC   = nombre de champs connus pour CISUFF, CIPREF et NCODPA
!
!  4 - TABLEAU POUR CODER LES COEF SPECTRAUX ALADIN AVEC GRIBEX
!
!     NOMPAR   = tableau decrivant la position des coeff spectraux (CSP) ALADIN
!                en fonction du nombre d'onde zonal dans un tableau de donnees
!                stocke dans FA avec la methode de codage 3 ou -1 (rangt modele).
!                NOMPAR est l'equivalent de NOZPAR mais pour un rangement vertical
!                des CSP: NOMPAR(1)=NSMAX, NOMPAR(2)=NMSMAX (ces 2 valeurs de
!                NOMPAR sont speciales commme pour NOZPAR) et pour un JM donne,
!                compris entre 0 et NMSMAX, NOMPAR(2*JM+3) donne l'indice du
!                premier CSP associe a JM qui est contenu dans un champ spectral
!                et NOMPAR(2*JM+4) donne l'indice du dernier CSP associe a JM
!                qui est contenu dans un champ spectral.
!                
!  ----------------------------------------------------------------------

!  1 - COEFFICIENTS MULTIPLICATIFS DES COEFF SPECTRAUX

      REAL (KIND=JPDBLR), ALLOCATABLE:: XLAP1D(:,:)  ,XLAP1DA(:,:)
      REAL (KIND=JPDBLR), ALLOCATABLE:: XLAP2D(:,:,:),XLAP2DA(:,:,:)
      REAL (KIND=JPDBLR), ALLOCATABLE:: FLAP1D(:,:)  ,FLAP1DA(:,:)

      LOGICAL LIXLAP, LIFLAP(JPNXFA)
 
!  2 - EN-TETE POUR GRIBEX (SECTIONS 1 ET 2)

      INTEGER, PARAMETER :: JPSEC1=37, JPSEC2=22+MAX(JPXTRO-1,JPXLAT)
      INTEGER, PARAMETER :: JPSEC4=42
      INTEGER  NSEC1(2:21,JPNXFA), NSEC2SP(22,JPNXCA)
      INTEGER  NSEC2LL(22,JPNXCA)
      INTEGER  NSEC2GG(22+JPXLAT,JPNXCA)
      INTEGER  NSEC2LA(22,JPNXCA)
      INTEGER  NSEC2AL(22,JPNXCA), NSC2ALF(JPXTRO-1,JPNXFA)

      REAL (KIND=JPDBLR) XSEC2(10+2*(JPXNIV+1),JPNXCA)

      LOGICAL LISEC1(JPNXFA), LISEC2(JPNXCA), LISC2F(JPNXFA)
 
!  3 - PARAMETRES DEFINISSANT LE CODAGE GRIBEX

      INTEGER NCODGRI(10), NCOGRIF(10,JPNXFA)
      INTEGER, PARAMETER :: JPXPAR=500
      INTEGER NBPARC
      CHARACTER(LEN=JPXPRF), DIMENSION(JPXPAR) :: CIPREF
      CHARACTER(LEN=JPXSUF), DIMENSION(JPXPAR) :: CISUFF
      INTEGER NCODPA(JPXPAR,6)

!  4 - TABLEAU POUR CODER LES COEF SPECTRAUX ALADIN AVEC GRIBEX

      INTEGER NOMPAR(2*JPXTRO+4,JPNXCA)

      END MODULE FAMODU
