MODULE GRIB_API_INTERFACE
!**** Interface to GRIB_API

!     Purpose.
!     --------
!     Fortran 90 Interface to calling GRIB API

!     Author.
!     -------
!        M.Hamrud     ECMWF

!     Modifications.
!     --------------
!        Original: 2005-11-15

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB     ,JPRM
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK
USE GRIB_API
!USE GRIB_TYPESIZES
USE MPL_ABORT_MOD
IMPLICIT NONE

PRIVATE
INTERFACE IGRIB_GET_VALUE
MODULE PROCEDURE &
 & IGRIB_GET_INT, IGRIB_GET_REAL4, IGRIB_GET_REAL8,IGRIB_GET_CHAR,&
 & IGRIB_GET_INT_ARRAY  , IGRIB_GET_REAL4_ARRAY, IGRIB_GET_REAL8_ARRAY
END INTERFACE
INTERFACE IGRIB_SET_VALUE
MODULE PROCEDURE &
 & IGRIB_SET_INT, IGRIB_SET_REAL4, IGRIB_SET_REAL8,IGRIB_SET_CHAR,&
 & IGRIB_SET_INT_ARRAY  , IGRIB_SET_REAL4_ARRAY, IGRIB_SET_REAL8_ARRAY
END INTERFACE
INTERFACE IGRIB_READ_BYTES
MODULE PROCEDURE &
 & IGRIB_READ_BYTES_INT,IGRIB_READ_BYTES_REAL8,IGRIB_READ_BYTES_CHAR
END INTERFACE
INTERFACE IGRIB_WRITE_BYTES
MODULE PROCEDURE &
 & IGRIB_WRITE_BYTES_INT,IGRIB_WRITE_BYTES_REAL8,IGRIB_WRITE_BYTES_CHAR
END INTERFACE

PUBLIC IGRIB_GET_VALUE,IGRIB_SET_VALUE,IGRIB_OPEN_FILE,IGRIB_CLOSE_FILE,&
 & IGRIB_RELEASE,JPGRIB_END_OF_FILE,&
 & IGRIB_GET_MESSAGE_SIZE,IGRIB_NEW_FROM_MESSAGE,IGRIB_NEW_FROM_TEMPLATE,&
 & IGRIB_CLONE,IGRIB_GET_MESSAGE,IGRIB_READ_FROM_FILE, &
 & IGRIB_READ_BYTES,IGRIB_WRITE_BYTES

INTEGER,PARAMETER :: JPGRIB_SUCCESS=0
INTEGER,PARAMETER :: JPGRIB_END_OF_FILE=-1
INTEGER,PARAMETER :: JPINTLEN=4

CONTAINS

SUBROUTINE IGRIB_GET_INT(KHANDLE,CDKEY,KVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
INTEGER(KIND=JPIM),INTENT(OUT) :: KVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_INT',0,ZHOOK_HANDLE)
CALL GRIB_GET_INT(KHANDLE,CDKEY,KVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_INT',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_INT',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_INT

SUBROUTINE IGRIB_GET_REAL4(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRM)   ,INTENT(OUT) :: PVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL4',0,ZHOOK_HANDLE)
CALL GRIB_GET_REAL4(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_REAL4',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL4',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_REAL4

SUBROUTINE IGRIB_GET_REAL8(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRB)   ,INTENT(OUT) :: PVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL8',0,ZHOOK_HANDLE)
CALL GRIB_GET_REAL8(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_REAL8',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL8',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_REAL8

SUBROUTINE IGRIB_GET_CHAR(KHANDLE,CDKEY,CDVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
CHARACTER(LEN=*)  ,INTENT(OUT) :: CDVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_CHAR',0,ZHOOK_HANDLE)
CALL GRIB_GET_STRING(KHANDLE,CDKEY,CDVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_STRING',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_CHAR',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_CHAR

SUBROUTINE IGRIB_GET_INT_ARRAY(KHANDLE,CDKEY,KVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
INTEGER(KIND=JPIM),INTENT(OUT) :: KVAL(:)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET,ISIZE
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_INT_ARRAY',0,ZHOOK_HANDLE)

CALL GRIB_GET_INT_ARRAY(KHANDLE,CDKEY,KVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_INT',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_INT_ARRAY',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_INT_ARRAY

SUBROUTINE IGRIB_GET_REAL4_ARRAY(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRM)   ,INTENT(OUT) :: PVAL(:)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL4_ARRAY',0,ZHOOK_HANDLE)

CALL GRIB_GET_REAL4_ARRAY(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_REAL4_ARRAY',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL4_ARRAY',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_REAL4_ARRAY

SUBROUTINE IGRIB_GET_REAL8_ARRAY(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRB)   ,INTENT(OUT) :: PVAL(:)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL8_ARRAY',0,ZHOOK_HANDLE)

CALL GRIB_GET_REAL8_ARRAY(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_REAL8_ARRAY',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_GET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_REAL8_ARRAY',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_REAL8_ARRAY
!====================================================================
SUBROUTINE IGRIB_SET_INT(KHANDLE,CDKEY,KVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
INTEGER(KIND=JPIM),INTENT(IN) :: KVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_INT',0,ZHOOK_HANDLE)
CALL GRIB_SET_INT(KHANDLE,CDKEY,KVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_INT',KHANDLE,CDKEY,KVAL,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_INT',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_INT

SUBROUTINE IGRIB_SET_REAL4(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRM)   ,INTENT(IN) :: PVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL4',0,ZHOOK_HANDLE)
CALL GRIB_SET_REAL4(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_REAL4',KHANDLE,CDKEY,PVAL,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL4',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_REAL4

SUBROUTINE IGRIB_SET_REAL8(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRB)   ,INTENT(IN) :: PVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL8',0,ZHOOK_HANDLE)
CALL GRIB_SET_REAL8(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_REAL8',KHANDLE,CDKEY,PVAL,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL8',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_REAL8

SUBROUTINE IGRIB_SET_CHAR(KHANDLE,CDKEY,CDVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
CHARACTER(LEN=*)  ,INTENT(IN) :: CDVAL
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_CHAR',0,ZHOOK_HANDLE)
CALL GRIB_SET_STRING(KHANDLE,CDKEY,CDVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_STRING',KHANDLE,CDKEY,CDVAL,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_CHAR',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_CHAR

SUBROUTINE IGRIB_SET_INT_ARRAY(KHANDLE,CDKEY,KVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
INTEGER(KIND=JPIM),INTENT(IN) :: KVAL(:)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_INT_ARRAY',0,ZHOOK_HANDLE)
CALL GRIB_SET_INT_ARRAY(KHANDLE,CDKEY,KVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_INT_ARRAY',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_INT_ARRAY',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_INT_ARRAY

SUBROUTINE IGRIB_SET_REAL4_ARRAY(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRM)   ,INTENT(IN) :: PVAL(:)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL4_ARRAY',0,ZHOOK_HANDLE)
CALL GRIB_SET_REAL4_ARRAY(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_REAL4_ARRAY',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL4_ARRAY',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_REAL4_ARRAY

SUBROUTINE IGRIB_SET_REAL8_ARRAY(KHANDLE,CDKEY,PVAL,KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
CHARACTER(LEN=*)  ,INTENT(IN) :: CDKEY
REAL(KIND=JPRB)   ,INTENT(IN) :: PVAL(:)
INTEGER(KIND=JPIM),OPTIONAL,INTENT(OUT) :: KRET

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL8_ARRAY',0,ZHOOK_HANDLE)
CALL GRIB_SET_REAL8_ARRAY(KHANDLE,CDKEY,PVAL,STATUS=IRET)
IF(PRESENT(KRET)) THEN
  KRET = IRET
ELSEIF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_SET_REAL8_ARRAY',KHANDLE,CDKEY,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_SET_VALUE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_SET_REAL8_ARRAY',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_SET_REAL8_ARRAY

SUBROUTINE IGRIB_OPEN_FILE(KFILE,CDPATH,CDMODE)
INTEGER(KIND=JPIM),INTENT(OUT)  :: KFILE
CHARACTER(LEN=*)  ,INTENT(IN)   :: CDPATH
CHARACTER(LEN=1)  ,INTENT(IN)   :: CDMODE

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_OPEN_FILE',0,ZHOOK_HANDLE)
CALL GRIB_OPEN_FILE(KFILE,CDPATH,CDMODE,STATUS=IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_OPEN_FILE ',TRIM(CDPATH),' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_OPEN_FILE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_OPEN_FILE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_OPEN_FILE

SUBROUTINE IGRIB_CLOSE_FILE(KFILE)
INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_CLOSE_FILE',0,ZHOOK_HANDLE)
CALL GRIB_CLOSE_FILE(KFILE,STATUS=IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_CLOSE_FILE ',KFILE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_CLOSE_FILE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_CLOSE_FILE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_CLOSE_FILE

SUBROUTINE IGRIB_READ_FROM_FILE(KFILE,KBUF,kbytes,kret)
INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE
INTEGER(KIND=JPIM),INTENT(OUT) :: KBUF(:)
INTEGER(KIND=JPIM),INTENT(INOUT) :: kbytes
INTEGER(KIND=JPIM),INTENT(OUT) :: KRET
INTEGER(KIND=JPIM) :: IRET,ILEN
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_FROM_FILE',0,ZHOOK_HANDLE)

CALL GRIB_READ_FROM_FILE(KFILE,KBUF,KBYTES,IRET)
IF(IRET /= JPGRIB_SUCCESS .AND. IRET /= GRIB_END_OF_FILE ) THEN
  WRITE(0,*) 'GRIB_READ_FROM_FILE ',KFILE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_READ_FROM_FILE FAILED')
ENDIF
KRET=IRET
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_FROM_FILE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_READ_FROM_FILE

SUBROUTINE IGRIB_READ_BYTES_INT(KFILE,KBUF,KBYTES,KRET)
INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE
INTEGER(KIND=JPIM),INTENT(OUT) :: KBUF(:)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBYTES
INTEGER(KIND=JPIM),INTENT(OUT) :: KRET
INTEGER(KIND=JPIM) :: IRET,ILEN
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_BYTES_INT',0,ZHOOK_HANDLE)

CALL GRIB_READ_BYTES(KFILE,KBUF,KBYTES,IRET)
IF(IRET /= JPGRIB_SUCCESS .AND. IRET /= GRIB_END_OF_FILE ) THEN
  WRITE(0,*) 'GRIB_READ_BYTES_INT ',KFILE,KBYTES,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_READ_BYTES_INT FAILED')
ENDIF
KRET=IRET
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_BYTES_INT',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_READ_BYTES_INT

SUBROUTINE IGRIB_READ_BYTES_REAL8(KFILE,PBUF,KBYTES,KRET)
INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE
REAL(KIND=JPRB)   ,INTENT(OUT) :: PBUF(:)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBYTES
INTEGER(KIND=JPIM),INTENT(OUT) :: KRET
INTEGER(KIND=JPIM) :: IRET,ILEN
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_BYTES_INT',0,ZHOOK_HANDLE)

CALL GRIB_READ_BYTES(KFILE,pbuf,kbytes,IRET)
IF(IRET /= JPGRIB_SUCCESS .AND. IRET /= GRIB_END_OF_FILE ) THEN
  WRITE(0,*) 'GRIB_READ_BYTES_INT ',KFILE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_READ_BYTES_INT FAILED')
ENDIF
KRET=IRET
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_BYTES_INT',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_READ_BYTES_REAL8

SUBROUTINE IGRIB_READ_BYTES_CHAR(KFILE,CDBUF,KBYTES,KRET)
INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE
CHARACTER(LEN=1)  ,INTENT(OUT) :: CDBUF(:)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBYTES
INTEGER(KIND=JPIM),INTENT(OUT) :: KRET
INTEGER(KIND=JPIM) :: IRET,ILEN
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_BYTES_CHAR',0,ZHOOK_HANDLE)

CALL GRIB_READ_BYTES(KFILE,CDBUF,KBYTES,IRET)
IF(IRET /= JPGRIB_SUCCESS .AND. IRET /= GRIB_END_OF_FILE ) THEN
  WRITE(0,*) 'GRIB_READ_BYTES_CHAR ',KFILE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_READ_BYTES_CHAR FAILED')
ENDIF
KRET=IRET
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_READ_BYTES_CHAR',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_READ_BYTES_CHAR


SUBROUTINE IGRIB_WRITE_BYTES_INT(KFILE,KBUF,KBYTES)
INTEGER(KIND=JPIM),INTENT(IN) :: KFILE
INTEGER(KIND=JPIM),INTENT(IN) :: KBUF(:)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBYTES
INTEGER(KIND=JPIM) :: IRET,ILEN
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_WRITE_BYTES_INT',0,ZHOOK_HANDLE)

CALL GRIB_WRITE_BYTES(KFILE,KBUF,KBYTES,IRET)
IF(IRET /= JPGRIB_SUCCESS ) THEN
  WRITE(0,*) 'GRIB_WRITE_BYTES_INT ',KFILE,KBYTES,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_WRITE_BYTES_INT FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_WRITE_BYTES_INT',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_WRITE_BYTES_INT

SUBROUTINE IGRIB_WRITE_BYTES_REAL8(KFILE,PBUF,KBYTES)
INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE
REAL(KIND=JPRB)   ,INTENT(IN) :: PBUF(:)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBYTES
INTEGER(KIND=JPIM) :: IRET,ILEN
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_WRITE_BYTES_INT',0,ZHOOK_HANDLE)

CALL GRIB_WRITE_BYTES(KFILE,PBUF,KBYTES,IRET)
IF(IRET /= JPGRIB_SUCCESS ) THEN
  WRITE(0,*) 'GRIB_WRITE_BYTES_INT ',KFILE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_WRITE_BYTES_REAL8 FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_WRITE_BYTES_REAL8',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_WRITE_BYTES_REAL8

SUBROUTINE IGRIB_WRITE_BYTES_CHAR(KFILE,CDBUF,KBYTES)
INTEGER(KIND=JPIM),INTENT(IN) :: KFILE
CHARACTER(LEN=1)  ,INTENT(IN) :: CDBUF(:)
INTEGER(KIND=JPIM),INTENT(IN) :: KBYTES
INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_WRITE_BYTES_CHAR',0,ZHOOK_HANDLE)

CALL GRIB_WRITE_BYTES(KFILE,CDBUF,KBYTES,IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_WRITE_BYTES_CHAR ',KFILE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_WRITE_BYTES_CHAR FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_WRITE_BYTES_CHAR',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_WRITE_BYTES_CHAR




!!$SUBROUTINE IGRIB_NEW_FROM_FILE(KFILE,KHANDLE)
!!$INTEGER(KIND=JPIM),INTENT(IN)  :: KFILE
!!$INTEGER(KIND=JPIM),INTENT(OUT) :: KHANDLE
!!$INTEGER(KIND=JPIM),INTENT(OUT) :: KRET
!!$
!!$INTEGER(KIND=JPIM) :: IRET
!!$INTEGER(KIND=JPIM) :: GRIB_NEW_FROM_FILE
!!$REAL(KIND=JPRB)    :: ZHOOK_HANDLE
!!$
!!$IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_NEW_FROM_FILE',0,ZHOOK_HANDLE)
!!$CALL GRIB_NEW_FROM_FILE(KFILE,KHANDLE)
!!$IF(KRET /= JPGRIB_SUCCESS) THEN
!!$  IF(KRET /= JPGRIB_END_OF_FILE) THEN
!!$    WRITE(0,*) 'GRIB_NEW_FROM_FILE',KFILE,' FAILED',KRET
!!$   CALL ERR_MSG(IRET)
!!$   CALL MPL_ABORT('GRIB_NEW_FROM_FILE FAILED')
!!$  ENDIF
!!$ENDIF
!!$IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_NEW_FROM_FILE',1,ZHOOK_HANDLE)
!!$
!!$END SUBROUTINE IGRIB_NEW_FROM_FILE

SUBROUTINE IGRIB_NEW_FROM_MESSAGE(KHANDLE,KBUF)
INTEGER(KIND=JPIM),INTENT(OUT) :: KHANDLE
INTEGER(KIND=JPIM),INTENT(IN)  :: KBUF(:)

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_NEW_FROM_MESSAGE',0,ZHOOK_HANDLE)

CALL GRIB_NEW_FROM_MESSAGE(KHANDLE,KBUF,STATUS=IRET)

IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'CALL TO GRIB_NEW_FROM_MESSAGE FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('IGRIB_NEW_FROM_MESSAGE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_NEW_FROM_MESSAGE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_NEW_FROM_MESSAGE

SUBROUTINE IGRIB_RELEASE(KHANDLE)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_RELEASE',0,ZHOOK_HANDLE)
CALL GRIB_RELEASE(KHANDLE,STATUS=IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_RELEASE ',KHANDLE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT('GRIB_RELEASE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_RELEASE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_RELEASE

SUBROUTINE IGRIB_GET_MESSAGE_SIZE(KHANDLE,KBYTES)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
INTEGER(KIND=JPIM),INTENT(OUT) :: KBYTES

INTEGER(KIND=JPIM) :: IRET
INTEGER(KIND=kindofsize_t) :: IBYTES
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_MESSAGE_SIZE',0,ZHOOK_HANDLE)
CALL GRIB_GET_MESSAGE_SIZE(KHANDLE,IBYTES,STATUS=IRET)
KBYTES = IBYTES
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_GET_MESSAGE_SIZE ',KHANDLE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT(' GRIB_GET_MESSAGE_SIZE FAIL')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_MESSAGE_SIZE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_MESSAGE_SIZE

SUBROUTINE IGRIB_GET_MESSAGE(KHANDLE,KGRIB)
INTEGER(KIND=JPIM),INTENT(IN) :: KHANDLE
INTEGER(KIND=JPIM),INTENT(OUT) :: KGRIB(:)

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE
CHARACTER(LEN=1),ALLOCATABLE   :: CLS(:)
INTEGER(KIND=JPIM) :: ilenint
INTEGER(KIND=JPIM) :: ILEN

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_MESSAGE',0,ZHOOK_HANDLE)

!TEMP CODING
CALL IGRIB_GET_MESSAGE_SIZE(KHANDLE,ILEN)
ilenint=(ilen+3)/4
allocate(cls(ilenint*4))
CALL GRIB_COPY_MESSAGE(KHANDLE,cls,STATUS=IRET)
KGRIB(1:ilenint)=0
KGRIB(1:ilenint)=TRANSFER(cls,KGRIB)
deallocate(cls)
!CALL GRIB_COPY_MESSAGE(KHANDLE,KGRIB,STATUS=IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_COPY_MESSAGE ',KHANDLE,' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT(' GRIB_COPY_MESSAGE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_GET_MESSAGE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_GET_MESSAGE

SUBROUTINE IGRIB_NEW_FROM_TEMPLATE(KHANDLE,CDNAME)
INTEGER(KIND=JPIM),INTENT(OUT) :: KHANDLE
CHARACTER(LEN=*),INTENT(IN) :: CDNAME

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_NEW_FROM_TEMPLATE',0,ZHOOK_HANDLE)
CALL GRIB_NEW_FROM_TEMPLATE(KHANDLE,TRIM(CDNAME),STATUS=IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_NEW_FROM_TEMPLATE ',TRIM(CDNAME),' FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT(' GRIB_NEW_FROM_TEMPLATE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_NEW_FROM_TEMPLATE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_NEW_FROM_TEMPLATE

SUBROUTINE IGRIB_CLONE(KHANDLE1,KHANDLE2)
INTEGER(KIND=JPIM),INTENT(IN)  :: KHANDLE1
INTEGER(KIND=JPIM),INTENT(OUT) :: KHANDLE2

INTEGER(KIND=JPIM) :: IRET
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_CLONE',0,ZHOOK_HANDLE)
CALL GRIB_CLONE(KHANDLE1,KHANDLE2,STATUS=IRET)
IF(IRET /= JPGRIB_SUCCESS) THEN
  WRITE(0,*) 'GRIB_CLONE FAILED',IRET
  CALL ERR_MSG(IRET)
  CALL MPL_ABORT(' GRIB_CLONE FAILED')
ENDIF
IF (LHOOK) CALL DR_HOOK('GRIB_API:IGRIB_CLONE',1,ZHOOK_HANDLE)

END SUBROUTINE IGRIB_CLONE

SUBROUTINE ERR_MSG(KRET)
INTEGER(KIND=JPIM),INTENT(IN) :: KRET
INTEGER(KIND=JPIM) :: IRET
CHARACTER(LEN=256) :: CLERRMSG

CLERRMSG = ''
CALL GRIB_GET_ERROR_STRING(IRET,CLERRMSG,STATUS=IRET)
WRITE(0,*) TRIM(CLERRMSG)
END SUBROUTINE ERR_MSG

END MODULE GRIB_API_INTERFACE



