#if defined(NECSX) && defined(REALHUGE)

MODULE quad_emu_mod

!        NEC 2007. Emulation of vectorized quadruple precision

USE PARKIND2  ,ONLY : JPRH

IMPLICIT NONE

EXTERNAL qvmulqv, ismulqv, qvaddqv, qvsubqv
EXTERNAL r16vcpqv, qvcpr16v

TYPE :: quads
  REAL(8) :: h, l
END TYPE quads
TYPE :: quadv
  REAL(8), pointer, dimension(:) :: h, l
END TYPE quadv

CONTAINS

SUBROUTINE QXMY (V1,V2,N,V3)
USE PARKIND1  ,ONLY : JPIM
USE PARKIND2  ,ONLY : JPRH
  ! ***************************
  ! V3(1:N) = V1(1:N) * V2(1:N)
  ! ***************************
  INTEGER(KIND=JPIM), INTENT(IN) :: N
  REAL(KIND=JPRH), DIMENSION(N), INTENT(IN)  :: V1, V2
  REAL(KIND=JPRH), DIMENSION(N), INTENT(OUT) :: V3
  TYPE (quadv),save :: QV1, QV2, QV3
  logical first
  data first / .true. /
  integer, save :: NMAX = -1

  if (first  .or.  n>nmax) then
    if(.not.first)DEALLOCATE (QV1%h,QV1%l,QV2%h,QV2%l,QV3%h,QV3%l)
    first = .false. 
    nmax = n
    ALLOCATE ( QV1%h(nmax) , QV1%l(nmax) )
    ALLOCATE ( QV2%h(nmax) , QV2%l(nmax) )
    ALLOCATE ( QV3%h(nmax) , QV3%l(nmax) )
  endif

  ! QV1(:) = V1(:)
  CALL R16VCPQV(V1,QV1%h,QV1%l,N)

  ! QV2(:) = V2(:)
  CALL R16VCPQV(V2,QV2%h,QV2%l,N)

  ! QV3(:) = QV1(:) * QV2(:)

  CALL QVMULQV(QV1%h(1),QV1%l(1),QV2%h(1),QV2%l(1),QV3%h(1),QV3%l(1),N)

  ! V3(:) = QV3(:)
  CALL QVCPR16V(QV3%h,QV3%l,V3,N)

RETURN
END SUBROUTINE QXMY

SUBROUTINE QAXPY (ALPHA,V1,V2,N)
USE PARKIND1  ,ONLY : JPIM
USE PARKIND2  ,ONLY : JPRH
  ! ***********************************
  ! V2(1:N) = ALPHA * V1(1:N) + V2(1:N)
  ! ***********************************
  INTEGER(KIND=JPIM), INTENT(IN) :: N
  REAL(KIND=JPRH),               INTENT(IN)    :: ALPHA
  REAL(KIND=JPRH), DIMENSION(N), INTENT(IN)    :: V1
  REAL(KIND=JPRH), DIMENSION(N), INTENT(INOUT) :: V2
  TYPE (quadv),save :: QALPHA
  TYPE (quadv),save :: QV1, QV2, QVtmp
  logical first
  data first / .true. /
  integer, save :: NMAX = -1

  if (first  .or.  n>nmax) then
    if(.not.first)then
      DEALLOCATE (QALPHA%h,QALPHA%l,QVtmp%h,QVtmp%l,QV1%h,QV1%l,QV2%h,QV2%l)
    endif
    first = .false.
    nmax = n
    ALLOCATE ( QALPHA%h(nmax) , QALPHA%l(nmax) )
    ALLOCATE ( QV1%h(nmax) , QV1%l(nmax) )
    ALLOCATE ( QV2%h(nmax) , QV2%l(nmax) )
    ALLOCATE ( QVtmp%h(nmax) , QVtmp%l(nmax) )
  endif

  ! QALPHA(:) = ALPHA
  CALL R16SCPQV(ALPHA,QALPHA%h,QALPHA%l,N)

  ! QV1(:) = V1(:)
  CALL R16VCPQV(V1,QV1%h,QV1%l,N)

  ! QV2(:) = V2(:)
  CALL R16VCPQV(V2,QV2%h,QV2%l,N)

  ! QVtmp(:) = ALPHA*QV1(:)
  call qvmulqv(QALPHA%h,QALPHA%l,QV1%h,QV1%l,QVtmp%h,QVtmp%l,N)

  ! QVtmp(:) = QVtmp(:) + QV2(:)
  call qvaddqv(QVtmp%h,QVtmp%l,QV2%h,QV2%l,QVtmp%h,QVtmp%l,n)

  ! V2(:) = QVtmp(:)
  call qvcpr16v(QVtmp%h,QVtmp%l,V2,N)

RETURN
END SUBROUTINE QAXPY

SUBROUTINE SQRTIVDV (I1, I2, N, X)
USE PARKIND1  ,ONLY : JPIM, JPIB
USE PARKIND2  ,ONLY : JPRH
  ! *********************************************
  ! X(1:N) = SQRT ( REAL(I1(1:N) / REAL(I2(1:N) )
  ! *********************************************
  INTEGER(KIND=JPIM), INTENT(IN) :: N
  INTEGER(KIND=JPIB), INTENT(IN), DIMENSION(N) :: I1
  INTEGER(KIND=JPIB), INTENT(IN), DIMENSION(N) :: I2
  REAL(KIND=JPRH), INTENT(INOUT), DIMENSION(N) :: X
  TYPE (quadv),save :: QV1, QV2, QV3, QVx
  logical first
  data first / .true. /
  integer, save :: NMAX = -1

  if (first  .or.  n>nmax) then
    if(.not.first)then
      DEALLOCATE (QV1%h,QV1%l,QV2%h,QV2%l,QV3%h,QV3%l)
      DEALLOCATE (QVx%h,QVx%l)
    endif
    first = .false.
    nmax = n
    ALLOCATE ( QV1%h(nmax) , QV1%l(nmax) )
    ALLOCATE ( QV2%h(nmax) , QV2%l(nmax) )
    ALLOCATE ( QV3%h(nmax) , QV3%l(nmax) )
    ALLOCATE ( QVx%h(nmax) , QVx%l(nmax) )
  endif
  ! QV1(:) = I1(:)
  CALL I4VCPQV(I1,QV1%h,QV1%l,N)
  ! QV2(:) = I2(:)
  CALL I4VCPQV(I2,QV2%h,QV2%l,N)
  ! QV3(:) = I1(:) / I2(:)
  CALL qvdivqv(QV1%h,QV1%l,QV2%h,QV2%l,QV3%h,QV3%l,n)
  ! QVx(:) = sqrt ( QV3(:) )
  CALL qvsqrt (QV3%h,QV3%l,QVx%h,QVx%l,n)
  ! X(:) = QVx(:)
  call qvcpr16v(QVx%h,QVx%l,x,n)

RETURN
END SUBROUTINE SQRTIVDV

END MODULE quad_emu_mod



! New routines ------------------------------------------------
      subroutine I4VCPQV(i4,ahi,alo,n)
! For SX vectorized version
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      INTEGER(KIND=JPIM) :: n
      INTEGER(KIND=JPIB) :: i4(n)
      REAL(KIND=JPRB) :: ahi(n),alo(n)
      REAL(KIND=JPRH) :: r16(n)

      r16(1:n) = i4(1:n)
      call r16vcpqv(r16,ahi,alo,n)

      end

      subroutine r16vcpqv(r16,ahi,alo,n)
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
#ifdef SCALAR
! Fortran equivalent code
       INTEGER(KIND=JPIM) :: n
       REAL(KIND=JPRH) :: r16(n)
       REAL(KIND=JPRB) :: ahi(n),alo(n)
       integer i
       do i=1,n
         ahi(i)=r16(i)
         alo(i)=r16(i)-ahi(i)
       enddo
#else
! For SX vectorized version
      INTEGER(KIND=JPIM) :: n
      REAL(KIND=JPRB) :: r16(2,n)
      REAL(KIND=JPRB) :: ahi(n),alo(n)

      REAL(KIND=JPRB) :: aa,bb
      INTEGER(KIND=JPIB) :: ii,jj,msk1,msk2,msk3
      equivalence(aa,ii)
      equivalence(bb,jj)
      parameter(msk1=Z'7FF0000000000000')
      parameter(msk2=Z'FFF0000000000000')
      parameter(msk3=Z'0340000000000000')

      integer i

      do i=1,n
        ahi(i)=r16(1,i)
        bb=r16(1,i)
        jj=iand(jj,msk1)
        if(jj.lt.msk3) then
          alo(i)=0_JPRB
        else
          aa=r16(2,i)
          ii=iand(ii,msk2)
          alo(i)=r16(2,i)-aa
        endif
      enddo
#endif
      end

      subroutine qvcpr16v(ahi,alo,r16,n)
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
#ifdef SCALAR
! Fortran equivalent code
       integer n
       REAL(KIND=JPRB) :: ahi(n),alo(n)
       REAL(KIND=JPRH) :: r16(n)
       integer i
       do i=1,n
         r16(i)=ahi(i)
         r16(i)=r16(i)+alo(i)
       enddo
#else
! For SX vectorized version
      REAL(KIND=JPRB) :: ahi(n),alo(n)
      REAL(KIND=JPRB) :: r16(2,n)
      integer n
!
      REAL(KIND=JPRB) :: a1,a2,aa,zhi,zlo,agomi,r
      INTEGER(KIND=JPIB) :: ii,msk
      equivalence(aa,ii)
      parameter(msk=Z'FFF0000000000000')
      integer i
!
      do i=1,n
         a1=ahi(i)
         a2=alo(i)
         aa=a1
         ii=iand(ii,msk)
         agomi=aa*2_JPRB**(-52)
         zhi=a1
         zlo=a2+agomi
         r=a1*a2
         if (r.lt.0) zhi=a1-agomi
         r16(1,i)=zhi
         r16(2,i)=(a1-zhi)+zlo
      enddo
#endif
      end
! New routines ------------------------------------------------

      subroutine r16scpqv(r16,ahi,alo,n)
! ... Quadruple precision native to Quadruple emulation pair
!     QV(1:N) = R16
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: r16(2)
      REAL(KIND=JPRB) :: ahi(n),alo(n)

      REAL(KIND=JPRB) :: aa
      INTEGER(KIND=JPIB) :: ii,msk
      equivalence(aa,ii)
      parameter(msk=Z'FFF0000000000000')

      integer i

      do i=1,n
        ahi(i)=r16(1)
        aa=r16(2)
        ii=iand(ii,msk)
        alo(i)=r16(2)-aa
      enddo

      end

      subroutine qvmulqv(ahi,alo,chi,clo,achi,aclo,n)
! ... multiplication of Quadruple emulation pair
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: ahi(n),alo(n),chi(n),clo(n),achi(n),aclo(n)

      integer, parameter :: nbits = DIGITS(1.0d0)
      REAL(KIND=JPRB), parameter :: constant = 2.d0**(nbits-nbits/2)+1.d0

      REAL(KIND=JPRB) :: zhi,zlo,zz
      REAL(KIND=JPRB) :: a1,a2,c1,c2,t
      integer i

!     longmul(a, c) RESULT(ac)
!     z = exactmul2(a%hi, c%hi)
!     zz = ((a%hi + a%lo) * c%lo + a%lo * c%hi) + z%lo
!     ac%hi = z%hi + zz
!     ac%lo = (z%hi - ac%hi) + zz

!     exactmul2(a, c) RESULT(ac)
!     t = constant * a
!     a1 = (a - t) + t
!     a2 = a - a1
!     t = constant * c
!     c1 = (c - t) + t
!     c2 = c - c1
!     ac%hi = a * c
!     ac%lo = (((a1 * c1 - ac%hi) + a1 * c2) + c1 * a2) + c2 * a2

      do i=1,n
        t = constant * ahi(i)
        a1 = (ahi(i) - t) + t
        a2 = ahi(i) - a1
        t = constant * chi(i)
        c1 = (chi(i) - t) + t
        c2 = chi(i) - c1
        zhi = ahi(i) * chi(i)
        zlo = (((a1 * c1 - zhi) + a1 * c2) + c1 * a2) + c2 * a2
        zz = ((ahi(i) + alo(i)) * clo(i) + alo(i) * chi(i)) + zlo
        achi(i) = zhi + zz
        aclo(i) = (zhi - achi(i)) + zz
      enddo

      end
      subroutine qvdivqv(ahi,alo,chi,clo,achi,aclo,n)
! ... division of Quadruple emulation pair 
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: ahi(n),alo(n),chi(n),clo(n),achi(n),aclo(n)

      integer, parameter :: nbits = DIGITS(1.0d0)
      REAL(KIND=JPRB), parameter :: constant = 2.d0**(nbits-nbits/2)+1.d0

      REAL(KIND=JPRB) :: z,zz,qhi,qlo
      REAL(KIND=JPRB) :: a1,a2,c1,c2,t
      integer i

!     longdiv(a, c) RESULT(ac)
!     z = a%hi / c%hi
!     q = exactmul2(c%hi, z)
!     zz = ((((a%hi - q%hi) - q%lo) + a%lo) - z*c%lo) / (c%hi + c%lo)
!     ac%hi = z + zz
!     ac%lo = (z - ac%hi) + zz

!     exactmul2(a, c) RESULT(ac)
!     t = constant * a
!     a1 = (a - t) + t
!     a2 = a - a1
!     t = constant * c
!     c1 = (c - t) + t
!     c2 = c - c1
!     ac%hi = a * c
!     ac%lo = (((a1 * c1 - ac%hi) + a1 * c2) + c1 * a2) + c2 * a2

      do i=1,n
        z = ahi(i)/chi(i)
        t = constant * chi(i)
        a1 = (chi(i) - t) + t
        a2 = chi(i) - a1
        t = constant * z
        c1 = (z - t) + t
        c2 = z - c1
        qhi = chi(i) * z
        qlo = (((a1 * c1 - qhi) + a1 * c2) + c1 * a2) + c2 * a2
        zz = ((((ahi(i) - qhi) - qlo) + alo(i)) - z*clo(i)) / (chi(i) + clo(i))
        achi(i) = z + zz
        aclo(i) = (z - achi(i)) + zz
      enddo

      end
      subroutine qvaddqv(ahi,alo,chi,clo,achi,aclo,n)
! ... addition of Quadruple emulation pair
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: ahi(n),alo(n),chi(n),clo(n),achi(n),aclo(n)

      REAL(KIND=JPRB) :: z,zz,q
      integer i

!     longadd(a, c) RESULT(ac)
!     z = a%hi + c%hi
!     q = a%hi - z
!     zz = (((q + c%hi) + (a%hi - (q + z))) + a%lo) + c%lo
!     ac%hi = z + zz
!     ac%lo = (z - ac%hi) + zz

      do i=1,n
        z = ahi(i) + chi(i)
        q = ahi(i) - z
        zz = (((q + chi(i)) + (ahi(i) - (q + z))) + alo(i)) + clo(i)
        achi(i) = z + zz
        aclo(i) = (z - achi(i)) + zz
      enddo

      end
      subroutine qvsubqv(ahi,alo,chi,clo,achi,aclo,n)
! ... subtraction of Quadruple emulation pair
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: ahi(n),alo(n),chi(n),clo(n),achi(n),aclo(n)

      REAL(KIND=JPRB) :: z,zz,q
      integer i

!     longsub(a, c) RESULT(ac)
!     z = a%hi - c%hi
!     q = a%hi - z
!     zz = (((q - c%hi) + (a%hi - (q + z))) + a%lo) - c%lo
!     ac%hi = z + zz
!     ac%lo = (z - ac%hi) + zz

      do i=1,n
        z = ahi(i) - chi(i)
        q = ahi(i) - z
        zz = (((q - chi(i)) + (ahi(i) - (q + z))) + alo(i)) - clo(i)
        achi(i) = z + zz
        aclo(i) = (z - achi(i)) + zz
      enddo

      end
      subroutine dvmuldv(a,c,achi,aclo,n)
! ... multiplication of double precision to Quadruple emulation pair
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: a(n),c(n),achi(n),aclo(n)

      integer, parameter :: nbits = DIGITS(1.0d0)
      REAL(KIND=JPRB), parameter :: constant = 2.d0**(nbits-nbits/2)+1.d0

      REAL(KIND=JPRB) :: a1,a2,c1,c2,t
      integer i

!     exactmul2(a, c) RESULT(ac)
!     t = constant * a
!     a1 = (a - t) + t
!     a2 = a - a1
!     t = constant * c
!     c1 = (c - t) + t
!     c2 = c - c1
!     ac%hi = a * c
!     ac%lo = (((a1 * c1 - ac%hi) + a1 * c2) + c1 * a2) + c2 * a2

      do i=1,n
        t = constant * a(i)
        a1 = (a(i) - t) + t
        a2 = a(i) - a1
        t = constant * c(i)
        c1 = (c(i) - t) + t
        c2 = c(i) - c1
        achi(i) = a(i) * c(i)
        aclo(i) = (((a1 * c1 - achi(i)) + a1 * c2) + c1 * a2) + c2 * a2
      enddo

      end
      subroutine ismulqv(is,qhi,qlo,qihi,qilo,n)
! ... integer scale of Quadruple emulation pair
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer is,n
      REAL(KIND=JPRB) :: qhi(n),qlo(n),qihi(n),qilo(n)

      integer, parameter :: nbits = DIGITS(1.0d0)
      REAL(KIND=JPRB), parameter :: constant = 2.d0**(nbits-nbits/2)+1.d0

      REAL(KIND=JPRB) :: a1,a2,q1,q2,t,zhi,zlo,zz
      integer i

      t = constant * is
      a1 = (dble(is) - t) + t
      a2 = dble(is) - a1
      do i=1,n
        t = constant * qhi(i)
        q1 = (qhi(i) - t) + t
        q2 = qhi(i) - q1
        zhi = is * qhi(i)
        zlo = (((a1 * q1 - zhi) + a1 * q2) + q1 * a2) + q2 * a2
        zz = is * qlo(i) + zlo
        qihi(i) = zhi + zz
        qilo(i) = (zhi - qihi(i)) + zz
      enddo

      end
      subroutine qvdivis(ahi,alo,is,aihi,ailo,n)
! ... integer division of Quadruple emulation pair
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer is,n
      REAL(KIND=JPRB) :: ahi(n),alo(n),aihi(n),ailo(n)

      integer, parameter :: nbits = DIGITS(1.0d0)
      REAL(KIND=JPRB), parameter :: constant = 2.d0**(nbits-nbits/2)+1.d0

      REAL(KIND=JPRB) :: z,zz,qhi,qlo
      REAL(KIND=JPRB) :: a1,a2,c1,c2,t
      integer i

!     div_quad_int(a, b) RESULT(c)
!     Divide a quadruple-precision number (a) by an integer (b)
!     z = a%hi / b
!     q = exactmul2(DBLE(b), z)
!     zz = (((a%hi - q%hi) - q%lo) + a%lo) / b
!     c%hi = z + zz
!     c%lo = (z - c%hi) + zz

!     exactmul2(a, c) RESULT(ac)
!     t = constant * a
!     a1 = (a - t) + t
!     a2 = a - a1
!     t = constant * c
!     c1 = (c - t) + t
!     c2 = c - c1
!     ac%hi = a * c
!     ac%lo = (((a1 * c1 - ac%hi) + a1 * c2) + c1 * a2) + c2 * a2

      t = constant * is
      a1 = (dble(is) - t) + t
      a2 = dble(is) - a1
      do i=1,n
        z = ahi(i)/is
        t = constant * z
        c1 = (z - t) + t
        c2 = z - c1
        qhi = dble(is) * z
        qlo = (((a1 * c1 - qhi) + a1 * c2) + c1 * a2) + c2 * a2
        zz = (((ahi(i) - qhi) - qlo) + alo(i)) / is
        aihi(i) = z + zz
        ailo(i) = (z - aihi(i)) + zz
      enddo

      end

      subroutine qvsqrt(ahi,alo,bhi,blo,n)
      USE PARKIND1  ,ONLY : JPIM, JPIB, JPRB
      USE PARKIND2  ,ONLY : JPRH
      integer n
      REAL(KIND=JPRB) :: ahi(n),alo(n),bhi(n),blo(n)

      integer, parameter :: nbits = DIGITS(1.0d0)
      REAL(KIND=JPRB), parameter :: constant = 2.d0**(nbits-nbits/2)+1.d0

      REAL(KIND=JPRB) :: a1,a2,c1,c2,t
      REAL(KIND=JPRB) :: tthi,ttlo,tt,res
      integer i

!     longsqrt(a) RESULT(b)
!     t = SQRT(a%hi)
!     tt = exactmul2(t, t)
!     res = (((a%hi - tt%hi) - tt%lo) + a%lo) * 0.5_dp / t
!     b%hi = t + res
!     b%lo = (t - b%hi) + res

!     exactmul2(a, c) RESULT(ac)
!     t = constant * a
!     a1 = (a - t) + t
!     a2 = a - a1
!     t = constant * c
!     c1 = (c - t) + t
!     c2 = c - c1
!     ac%hi = a * c
!     ac%lo = (((a1 * c1 - ac%hi) + a1 * c2) + c1 * a2) + c2 * a2

      do i=1,n
        tt = SQRT(ahi(i))
        t = constant * tt
        a1 = (tt - t) + t
        a2 = tt - a1
        t = constant * tt
        c1 = (tt - t) + t
        c2 = tt - c1
        tthi = tt * tt
        ttlo = (((a1 * c1 - tthi) + a1 * c2) + c1 * a2) + c2 * a2
        res = (((ahi(i) - tthi) - ttlo) + alo(i)) * 0.5d0 / tt
        bhi(i) = tt + res
        blo(i) = (tt - bhi(i)) + res
      enddo

      end
#else
MODULE quad_emu_mod
END MODULE quad_emu_mod
#endif
