      program gribdiff
      implicit none

      integer iargc
      integer, parameter :: info_offset = 255
      integer info(info_offset + 1024)
      character(len=80) file(2)
      integer fp(2), k, j, i, iret, idebug, icount
      integer, allocatable :: NLOEN(:)
      integer NDGL, NDLON, i_regular_grid
      integer nlen_one_field, num_fields
      REAL, allocatable :: fields(:,:,:)
      integer, allocatable :: nfield_len(:,:)
      integer, allocatable :: nfield_code(:,:)
      integer, allocatable :: nfield_date(:,:)
      integer, allocatable :: nfield_time(:,:)
      REAL, allocatable :: diff(:)
      integer, allocatable :: index(:)
      
      if (iargc() /= 2) then
         print *,'Usage: gribdiff file_1 file_2'
         stop '*Error*'
      endif

      idebug = 0
      num_fields = 0

      do j=1,2
         call GETARG(j, file(j))

         print '(/,a)',
     $        ' Opening GRIB-file "'//trim(file(j))//'" ...'
      
         call GRSDBG(idebug)

         call GRIB_OPEN(fp(j), file(j), 'rB',
     $        info, size(info), iret)

         print *,'GRIB_OPEN : rc=',iret

         print '(a,a4,a,/,a,a2,a,i4)',
     $        ' Experiment id="',info(22),'"',
     $        ' GRID-type=',info(3),
     $        ', VERSION CODE TABLE=',info(10)

         print '(a,i8.8,a,i4.4)',
     $        ' Date=',info(11),
     $        ' Time=',info(12)

         print '(a,i10)',
     $        ' No. of fields: ',info(9)

         if (j == 2 .and. num_fields /= info(9)) then
            print *,'Cannot compare; diff. no. of fields'
            stop '*Error*'
         endif

         num_fields = info(9)

         NDGL  = info(6)
         NDLON = info(7)

         write(*,'(1x,a,i4," x",i4)')
     $        'Grid size (NDLON x NDGL) : ',
     $        NDLON, NDGL
 
         if (j == 1) then
            allocate(NLOEN(0:NDGL+1))
            
            i_regular_grid = info(4)
            
            if (i_regular_grid == 1) then
c             print *,' (reduced grid)'
               NLOEN(1:NDGL) = 
     $              info(info_offset + 1:info_offset + NDGL)
               NLOEN(0)      = NLOEN(2)
               NLOEN(NDGL+1) = NLOEN(NDGL-1)
            else
c             print *,' (full grid)'
               NLOEN(0:NDGL+1) = NDLON
            endif

            nlen_one_field = sum(NLOEN(1:NDGL))

            deallocate(NLOEN)

            allocate(fields(nlen_one_field, num_fields,2))
            allocate(nfield_len(num_fields,2))
            allocate(nfield_code(num_fields,2))
            allocate(nfield_date(num_fields,2))
            allocate(nfield_time(num_fields,2))
         endif

         call GRSDBG(0) 

         call GRIB_READ(fp(j),
     $        fields(1,1,j), 
     $        nlen_one_field * num_fields,
     $        nfield_len(1,j),
     $        num_fields,
     $        nfield_code(1,j),
     $        nfield_date(1,j),
     $        nfield_time(1,j),
     $        'D', -1, iret)

         print *,'GRIB_READ : rc=',iret

         do k=1,num_fields
            print '(a,i4,a,i12,a,i4,a,i8.8,a,i4.4)',
     $           ' Field#',k,' : Length=',nfield_len(k,j),
     $           ' Code=',nfield_code(k,j),
     $           ' Date=',nfield_date(k,j),
     $           ' Time=',nfield_time(k,j)
         enddo

         call GRIB_CLOSE(fp(j), iret)
         print *,'GRIB_CLOSE : rc=',iret
      enddo

c--   diff

      allocate(diff(nlen_one_field))

c      do k=1,num_fields
      do k=1,1
         print *,'Field#',k,' : Code=',nfield_code(k,1)

         do i=1,nlen_one_field
            diff(i) = fields(i, k, 1) - fields(i, k, 2)
         enddo

         icount = count(diff(:) /= 0)

         print *, 'No. of diffs : ',icount

         if (icount > 0) then
            allocate(index(icount))
            j = 0
            do i=1,nlen_one_field
               if (diff(i) /= 0) then
                  j = j + 1
                  index(j) = i
               endif
            enddo

 1000       format(1p,(i8," : ",
     $           "Value#1",1x,g15.7,2x,b64.64,2x,z16.16,/,11x,
     $           "Value#2",1x,g15.7,2x,b64.64,2x,z16.16,/,11x,
     $           "Diff   ",1x,g15.7,2x,b64.64,2x,z16.16))
            write(*,1000)
     $           (
     $           index(j),
     $           fields(index(j), k, 1), fields(index(j), k, 1),
     $           fields(index(j), k, 1),
     $           fields(index(j), k, 2), fields(index(j), k, 2),
     $           fields(index(j), k, 2),
     $           diff(index(j)), diff(index(j)), diff(index(j)),
     $           j=1,icount)

            deallocate(index)
         endif
      enddo

      END PROGRAM GRIBDIFF
