SUBROUTINE SURFDEALLO

!**** *SURFDEALLO * - Routine to deallocate space for global variables
!                     from surface subroutine
!     Purpose.
!     --------
!           Deallocate space for the global fields.

!**   Interface.
!     ----------
!        *CALL* *SURFDEALLO*

!     Explicit arguments :  None
!     --------------------
!        Called by DEALLO

!     Implicit arguments :
!     --------------------
!        Pointers of comdecks

!     Method.
!     -------
!        Extraction from DEALLO

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        J.F. Estrade *ECMWF* 03-10-01
!     Modifications.
!     --------------
!        Original : 03-10-01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

!ifndef INTERFACE

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOS_SW    , ONLY : RALBICE_AR, RALBICE_AN, RWEIGHT, RSUN
USE YOS_SOIL  , ONLY : RDAT     ,RDAW     ,RDAI
USE YOS_VEG   , ONLY : RVCOV, RVLAI, RVROOTSA, RVLAMSK, RVLAMSKS, RVRSMIN &
                   & , RVHSTR, RVTRSR, RVZ0M, RVZ0H

!endif INTERFACE

IMPLICIT NONE

! Declaration of arguments

!ifndef INTERFACE

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SURFDEALLO',0,ZHOOK_HANDLE)
IF (ALLOCATED(RALBICE_AR)) DEALLOCATE(RALBICE_AR)
IF (ALLOCATED(RALBICE_AN)) DEALLOCATE(RALBICE_AN)
IF (ALLOCATED(RWEIGHT)) DEALLOCATE(RWEIGHT)
IF (ALLOCATED(RSUN)) DEALLOCATE(RSUN)
IF (ALLOCATED(RDAT)) DEALLOCATE(RDAT)
IF (ALLOCATED(RDAW)) DEALLOCATE(RDAW)
IF (ALLOCATED(RDAI)) DEALLOCATE(RDAI)
IF (ALLOCATED(RVCOV)) DEALLOCATE(RVCOV)
IF (ALLOCATED(RVLAI)) DEALLOCATE(RVLAI)
IF (ALLOCATED(RVLAMSK)) DEALLOCATE(RVLAMSK)
IF (ALLOCATED(RVLAMSKS)) DEALLOCATE(RVLAMSKS)
IF (ALLOCATED(RVTRSR)) DEALLOCATE(RVTRSR)
IF (ALLOCATED(RVROOTSA)) DEALLOCATE(RVROOTSA)
IF (ALLOCATED(RVRSMIN)) DEALLOCATE(RVRSMIN)
IF (ALLOCATED(RVHSTR)) DEALLOCATE(RVHSTR)
IF (ALLOCATED(RVZ0M)) DEALLOCATE(RVZ0M)
IF (ALLOCATED(RVZ0H)) DEALLOCATE(RVZ0H)
IF (LHOOK) CALL DR_HOOK('SURFDEALLO',1,ZHOOK_HANDLE)

!endif INTERFACE

!     ------------------------------------------------------------------

END SUBROUTINE SURFDEALLO
