MODULE SRFSN_RSN_MOD
CONTAINS
SUBROUTINE SRFSN_RSN(KIDIA  , KFDIA  , KLON   ,PTMST, &
 & PASNM1M, PRSNM1M, PSSNM1M, PTSNM1M,PFRTI, &
 & PSSFC, PSSFL, PTSRF, PVSRF, PUSRF, &
 & ZLW, ZPMSNINT,PMSN,&
 & PASN, PRSN,PSSN, &
 & PDHSSS )
 
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOS_CST   , ONLY : RTT, RG, RLMLT, RDAY
USE YOS_SOIL  , ONLY : RFRTINY, RFRSMALL, RALFMAXSN, RHOMINSN,           &
 & RHOMAXSN,   RSFRESH,    RSNPER,     RALFMINPSN,                         &
 & RTAUF,      RTAUA,      RALFMINSN,                                      &
 & RHOMINSNA,  RHOMINSNB,  RHOMINSNC,  RHOMINSND,                          &
 & RSNDTOVERA, RSNDTOVERB, RSNDTOVERC,                                     &
 & RSNDTDESTA, RSNDTDESTB, RSNDTDESTC, RSNDTDESTROI, RHOMAXSN_NEW,         &
 & LESNRSN,LESNAS

#ifdef DOC
!**** *SRFSN* - CONTAINS SNOW ALBEDO AND DENSITY PARAMETRIZATION 
!     PURPOSE.
!     --------
!          COMPUTES CHANGES IN SNOW DENSITY AND ALBEDO

!**   INTERFACE.
!     ----------
!          *SRFSN_RSN* IS CALLED FROM *SURFTSTP*.

!     PARAMETER   DESCRIPTION                                    UNITS
!     ---------   -----------                                    -----

!     INPUT PARAMETERS (INTEGER):
!    *KIDIA*      START POINT
!    *KFDIA*      END POINT
!    *KLON*       NUMBER OF GRID POINTS PER PACKET

!     INPUT PARAMETERS (REAL):
!    *PTMST*      TIME STEP                                      S

!     INPUT PARAMETERS AT T-1 OR CONSTANT IN TIME (REAL):
!    *PSSNM1M*    SNOW MASS (per unit area)                    kg/m**2/s
!    *PTSNM1M*    SNOW TEMPERATURE                               K
!    *PASNM1M*    SNOW ALBEDO                                    -
!    *PRSNM1M*    SNOW DENSITY                                 KG/M3
!    *PFRTI*      TILE FRACTIONS                                 -
!    *PUSRF*      X-COMPONENT OF WIND (LAST LEVEL)             M/S
!    *PVSRF*      Y-COMPONENT OF WIND.(LAST LEVEL)             M/S
!    *PTSRF*      TEMPERATURE (LAST LEVEL)                     K
!    *ZLW*        SNOW LIQUID WATER CONTENT                   kg/m**2
!    *ZPMSNINT*   INTERNAL PHASE CHANGE                        w.m-2 

!     PARAMETERS AT T+1 :
!    *PASN*       SNOW ALBEDO                                    -
!    *PRSN*       SNOW DENSITY                               KG/M**3

!    FLUXES FROM SNOW SCHEME:
!    *PMSN*       FLUX OF MELT WATER FROM SNOW TO SOIL       KG/M**2/S (#)

!     OUTPUT PARAMETERS (DIAGNOSTIC):
!    *PDHSSS*     Diagnostic array for snow mass (see module yomcdh)

!     METHOD.
!     -------
!
!     EXTERNALS.
!     ----------
!
!     REFERENCE.
!     ----------
!
!     ORIGINAL :
!     E. Dutra       12/11/2008     SPLIT CODE FROM SRFSN
!                                   New snow density parameterization 
!     ------------------------------------------------------------------
#endif

IMPLICIT NONE

! Declaration of arguments

INTEGER(KIND=JPIM), INTENT(IN)   :: KIDIA
INTEGER(KIND=JPIM), INTENT(IN)   :: KFDIA
INTEGER(KIND=JPIM), INTENT(IN)   :: KLON

REAL(KIND=JPRB),    INTENT(IN)   :: PTMST
REAL(KIND=JPRB),    INTENT(IN)   :: PASNM1M(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PRSNM1M(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PSSNM1M(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PTSNM1M(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PFRTI(:,:)
REAL(KIND=JPRB),    INTENT(IN)   :: PSSFC(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PSSFL(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PTSRF(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PVSRF(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PUSRF(:)
REAL(KIND=JPRB),    INTENT(IN)   :: ZLW(:)
REAL(KIND=JPRB),    INTENT(IN)   :: ZPMSNINT(:)
REAL(KIND=JPRB),    INTENT(IN)   :: PMSN(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PASN(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PRSN(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PSSN(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PDHSSS(:,:,:)



!      LOCAL VARIABLES

LOGICAL            :: LLNOSNOW(KLON)  ! SNOW DO NOT EXISTS ZFRSN < RFRSMALL 

REAL(KIND=JPRB)    :: ZFRSN(KLON),&   ! SNOW FRACTION (TILES 5+7)
                      ZSSFC(KLON),&   ! TOTAL CONVECTIVE SNOW FLUX  
                      ZSSFL(KLON)     ! TOTAL LARGE SCALE SNOW FLUX

REAL(KIND=JPRB)    :: ZRHOMINSN,&     ! FRESH SNOW DENSITY (KG/M**3)
                      ZRSTAR,&        ! SNOW DENSITY AFTER SNOWFALL
                      ZRSNDT,&        ! FACTOR FOR SNOW DEPTH CHANGE 
                      ZRSNDTOVER,&    ! FACTOR FOR SNOW DEPTH CHANGE DUE TO OVERBURNING
                      ZRSNDTDEST,&    ! FACTOR FOR SNOW DEPTH CHANGE DUE TO DESTRUCTIVE METAMORPHISM
                      ZRSNDTMELT,&    ! FACTOR FOR SNOW DEPTH CHANGE DUE TO MELTING
                      ZLINA,&         ! ALBEDO DECREASE IN NON MELTING-CONDITIONS
                      ZEXPF,&         ! EXPRESSION USED IN THE CALCULATION OF SNOW DENSITY ABLATION (SIMPLE FORM) 
                      ZTMST,&         ! 1/PTMST
                      ZALFMINSN,&     ! MINIMUM SNOW ALBEDO
                      ZSNPERT         ! THRESHOLD FOR PERMANENT SNOW (9 m) 

REAL(KIND=JPRB)    :: ZHOOK_HANDLE
INTEGER(KIND=JPIM) :: JL  ! CYCLE IN POINT

IF (LHOOK) CALL DR_HOOK('SRFSN_RSN_MOD:SRFSN_RSN',0,ZHOOK_HANDLE)

ZEXPF=EXP(-RTAUF*PTMST/RDAY)
ZLINA=RTAUA*PTMST/RDAY
ZTMST=1.0_JPRB/PTMST

!*           1. SNOW FRACTION

DO JL=KIDIA,KFDIA
  ZFRSN(JL)=MAX(PFRTI(JL,5)+PFRTI(JL,7),RFRTINY)
  IF (ZFRSN(JL) < RFRSMALL) THEN
    LLNOSNOW(JL)=.TRUE.
  ELSE
    LLNOSNOW(JL)=.FALSE.
  ENDIF
  ZSSFC(JL)=(PFRTI(JL,3)+PFRTI(JL,4)+PFRTI(JL,5)&
   & +PFRTI(JL,6)+PFRTI(JL,7)+PFRTI(JL,8))*PSSFC(JL)  
  ZSSFL(JL)=(PFRTI(JL,3)+PFRTI(JL,4)+PFRTI(JL,5)&
   & +PFRTI(JL,6)+PFRTI(JL,7)+PFRTI(JL,8))*PSSFL(JL)  
ENDDO

!*         2. NEW SNOW ALBEDO AND DENSITY.
!             ----------------------------

DO JL=KIDIA,KFDIA
  IF (LLNOSNOW(JL)) THEN
    PASN(JL)=RALFMAXSN
    PRSN(JL)=RHOMINSN
    !PDHSSS(JL,1,9)=0.0_JPRB
    !PDHSSS(JL,1,10)=0.0_JPRB
    !PDHSSS(JL,1,11)=0.0_JPRB
  ELSE

! NEW SNOW DENSITY DUE TO FRESH SNOW 
    IF(LESNRSN) THEN 
    ! NEW SNOW DENSITY (INCLUDE FRESH SNOW)
      ZRHOMINSN=RHOMINSNA + RHOMINSNB*(PTSRF(JL)-RTT) + &
        & RHOMINSNC*(PVSRF(JL)**2+PUSRF(JL)**2)**0.25_JPRB
      ZRHOMINSN=MAX(ZRHOMINSN,RHOMINSND)
      ZRSTAR=(PSSNM1M(JL)*PRSNM1M(JL)&
        & +PTMST*(ZSSFC(JL)+ZSSFL(JL))*ZRHOMINSN)&
        & /(PSSNM1M(JL)+PTMST*(ZSSFC(JL)+ZSSFL(JL)))
    ELSE
      ZRSTAR=(PSSNM1M(JL)*PRSNM1M(JL)&
        & +PTMST*(ZSSFC(JL)+ZSSFL(JL))*RHOMINSN)&
        & /(PSSNM1M(JL)+PTMST*(ZSSFC(JL)+ZSSFL(JL)))
    ENDIF

! SNOW COMPACTATION 

    IF(LESNRSN) THEN 
! NEW PARAMETRIZATION 
      ! COMPACTATION DUE TO OVERBURNING 

      ZRSNDTOVER=0.5*PSSNM1M(JL)*RG/(RSNDTOVERA*&
       & EXP(RSNDTOVERB*(RTT-PTSNM1M(JL))+RSNDTOVERC*PRSNM1M(JL)))

      ! COMPACTATION DUE TO DESTRUCTIVE METAMORPHISM
      ZRSNDTDEST=RSNDTDESTA*EXP(-RSNDTDESTB*(RTT-PTSNM1M(JL))-&
        & RSNDTDESTC*MAX(0.0_JPRB,PRSNM1M(JL)-RSNDTDESTROI))

      ! COMPACTATION DUE TO MELTING 
      IF (ZPMSNINT(JL) > 0.0_JPRB .AND. PSSNM1M(JL) >= 15.0_JPRB) THEN 
          ZRSNDTMELT=ZTMST*(ZPMSNINT(JL)*PTMST/RLMLT)/(PSSNM1M(JL)-ZLW(JL))
      ELSE
        ZRSNDTMELT=0.0_JPRB
      ENDIF

      ! TOTAL COMPACTATION 
      ZRSNDT=ZRSNDTOVER+ZRSNDTDEST+ZRSNDTMELT
      ! NEW DENSITY 
      PRSN(JL)=ZRSTAR+ZRSTAR*ZRSNDT*PTMST
      PRSN(JL)=MIN(RHOMAXSN_NEW,PRSN(JL))
      PRSN(JL)=MAX(RHOMINSN,PRSN(JL))

      !* DDH DIAGNOSTICS
      !PDHSSS(JL,1,9)=ZRSNDTOVER
      !PDHSSS(JL,1,10)=ZRSNDTDEST
      !PDHSSS(JL,1,11)=ZRSNDTMELT 
    ELSE
! OLD PARAMETRIZATION 
      PRSN(JL)=(ZRSTAR-RHOMAXSN)*ZEXPF+RHOMAXSN

      ! SET DIAGNOSTICS TO ZERO
      !PDHSSS(JL,1,9)=0.0_JPRB
      !PDHSSS(JL,1,10)=0.0_JPRB
      !PDHSSS(JL,1,11)=0.0_JPRB
    ENDIF 

! SNOW ALBEDO UPDATE 
    IF (LESNAS) THEN
      ZALFMINSN=RALFMINSN
      IF (PMSN(JL) > 0.0_JPRB .OR. ZPMSNINT(JL) > 0.0_JPRB .OR. PTSNM1M(JL) > RTT-2._JPRB ) THEN
        PASN(JL)=(PASNM1M(JL)-ZALFMINSN)*ZEXPF+ZALFMINSN
      ELSE
        PASN(JL)=MAX(ZALFMINSN,PASNM1M(JL)-ZLINA)
      ENDIF
      PASN(JL)=PASN(JL)+ MIN(MAX((ZSSFC(JL)+ZSSFL(JL))*PTMST , 0._JPRB)/(10.) , 1._JPRB) *(RALFMAXSN-PASN(JL))
      PASN(JL)=MIN(RALFMAXSN,MAX(PASN(JL),RALFMINSN))
    ELSE
      ZALFMINSN=RALFMINSN
      IF (PMSN(JL) > 0.0_JPRB .OR. ZPMSNINT(JL) > 0.0_JPRB ) THEN
        PASN(JL)=(PASNM1M(JL)-ZALFMINSN)*ZEXPF+ZALFMINSN
      ELSE
        PASN(JL)=MAX(ZALFMINSN,PASNM1M(JL)-ZLINA)
      ENDIF
      IF (ZSSFC(JL)+ZSSFL(JL) > RSFRESH) PASN(JL)=RALFMAXSN 
    ENDIF    
    
!Permanent snow reset was potentially dangerous at 1m threshold and 9m is used instead.
    ZSNPERT=9000.0_JPRB ! permanent snow threshold
    IF (PSSNM1M(JL) >= ZSNPERT) THEN
      PASN(JL)=RALFMINPSN
      PRSN(JL)=RHOMAXSN
      PSSN(JL)=10000.0_JPRB !reset to glaciers value of 10000 kg/m2 (SWE=10m)
    ENDIF
  ENDIF
ENDDO


IF (LHOOK) CALL DR_HOOK('SRFSN_LWINP_MOD:SRFSN',1,ZHOOK_HANDLE)
END SUBROUTINE SRFSN_RSN

END MODULE SRFSN_RSN_MOD
