MODULE SURFRAD_CTL_MOD
CONTAINS
SUBROUTINE SURFRAD_CTL(KDDN,KMMN,KMON,KSECO,&
 & KIDIA,KFDIA,KLON,KTILES,KSW,&
 & LDNH,&
 & PALBF,PALBICEF,PTVH,&
 & PALUVP,PALUVD,PALNIP,PALNID,&
 & PASN,PMU0,PTS,PWND,&
 & PWS1,KSOTY,PLIT,PFRTI,PHLICE,PTLICE,&  
 & PALBD,PALBP,PALB,&
 & PEMIR,PEMIW,PEMIT,&
 & PALBTI,PCCNL,PCCNO)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOS_SOIL  , ONLY : RWCAP,RWPWP,RWCAPM,RWPWPM,LEVGEN,LESN09
USE YOS_FLAKE , ONLY : LEFLAKE,RH_ICE_MIN_FLK
USE YOS_RDI   , ONLY : RALBSFO,RALBSEAD,REMISD,REMISL,&
 & REMISN,REMISS,REPALB  
USE YOS_LW    , ONLY : NSIL,TSTAND,XP
USE YOS_SW    , ONLY : RSUN,RALBICE_AR,RALBICE_AN,RWEIGHT,ICEALB_SPECWEIGHT
USE YOS_RAD   , ONLY : LCCNL,LCCNO,RCCNSEA,RCCNLND,LE4ALB
USE YOS_CST   , ONLY : RDAY,RTT
USE YOMECEARTH, ONLY : LECEARTH, LAMIP, LLANDICE !CLN LECEARTH_UNCOUP

#ifdef DOC
!**** *SURFRAD  - COMPUTES RADIATIVE PROPERTIES OF SURFACE

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SURFRAD* FROM *CALLPAR*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! LDNH   : LOGICAL  : .TRUE. FOR Northern Hemisphere
! PALBF  : REAL     : FIXED BACKGROUND SURFACE SHORTWAVE ALBEDO
! PALBICEF REAL     : FIXED SEA-ICE ALBEDO (FROM COUPLER)
! PTVH   : REAL     : DOMINANT HIGH VEGETATION TYPE
! PALUVP : REAL     : MODIS UV-VIS PARALLEL (DIRECT) ALBEDO
! PALUVD : REAL     : MODIS UV-VIS DIFFUSE ALBEDO
! PALNIP : REAL     : MODIS N-IR PARALLEL (DIFFUSE) ALBEDO
! PALNID : REAL     : MODIS N-IR DIFFUSE ALBEDO
! PASN   : REAL     : ALBEDO OF EXPOSED SNOW (TYPE 5)
! PMU0   : REAL     : COSINE OF SOLAR ZENITH ANGLE
! PTS    : REAL     : SURFACE TEMPERATURE
! PWND   : REAL     : WIND INTENSITY AT LOWEST LEVEL
! PWS1   : REAL     : TOP LAYER SOIL MOISTURE CONTENT
! KSOTY  : INTEGER  : SOIL TYPE                           (1-7)
! PFRTI  : REAL     : TILE FRACTIONS                      (0-1)
!            1 : WATER                  5 : SNOW ON LOW-VEG+BARE-SOIL
!            2 : ICE                    6 : DRY SNOW-FREE HIGH-VEG
!            3 : WET SKIN               7 : SNOW UNDER HIGH-VEG
!            4 : DRY SNOW-FREE LOW-VEG  8 : BARE SOIL
! PHLICE : REAL     : LAKE ICE THICKNESS (m) 
! PTLICE : REAL     : LAKE ICE TEMPERATURE (K) 

!     ==== OUTPUTS ===
! PALBD  : REAL     : SURFACE ALBEDO FOR DIFFUSE RADIATION
! PALBP  : REAL     : SURFACE ALBEDO FOR PARALLEL RADIATION
! PALB   : REAL     : AVERAGE SW ALBEDO (DIAGNOSTIC ONLY)
! PEMIR  : REAL     : SURFACE LONGWAVE EMISSIVITY OUTSIDE WINDOW
! PEMIW  : REAL     : SURFACE LONGWAVE WINDOW EMISSIVITY
! PEMIT  : REAL     : SURFACE LONGWAVE EMISSIVITY
! PALBTI : REAL     : BROADBAND ALBEDO FOR TILE FRACTIONS
! PCCNL  : REAL     : CCN CONCENTRATION OVER LAND
! PCCNO  : REAL     : CCN CONCENTRATION OVER OCEAN

!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!          NONE

!     REFERENCE.
!     ----------
!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    91/03/15

!     MODIFICATIONS.
!     --------------
!     J.-J. MORCRETTE  ECMWF 94/11/15  DIRECT/DIFFUSE ALBEDOS
!     J.-J. MORCRETTE 96/06/07  moisture dep. emissiv. / spectral alb.  
!     PJANSSEN/JJMORCRETTE ECMWF     96/11/07  WIND DEPENDENT SEA ALBEDO
!     PViterbo         ECMWF 99/03/03  Albedo for tile fractions
!     J.-J. Morcrette ECMWF 00/10/24 Spectral albedo for all surfaces
!     JJMorcrette     01-10-08  CCNs concentration over ocean
!     J.F. Estrade *ECMWF* 03-10-01 move in surf vob
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     JJMorcrette 20060511 MODIS albedo
!     G. Balsamo    ECMWF   08-01-2006 Include Van Genuchten Hydro.
!     G. Balsamo    ECMWF   03-07-2006 Add soil type
!     E. Dutra/G. Balsamo   01-05-2008 Add lake tile
!     E. Dutra              16-11-2009 snow 2099 cleaning / changes albedo of shaded snow 
!-----------------------------------------------------------------------
#endif

IMPLICIT NONE

! Declaration of arguments

INTEGER(KIND=JPIM), INTENT(IN)  :: KDDN
INTEGER(KIND=JPIM), INTENT(IN)  :: KMMN
INTEGER(KIND=JPIM), INTENT(IN)  :: KMON(:)
INTEGER(KIND=JPIM), INTENT(IN)  :: KSECO
INTEGER(KIND=JPIM), INTENT(IN)  :: KIDIA
INTEGER(KIND=JPIM), INTENT(IN)  :: KFDIA
INTEGER(KIND=JPIM), INTENT(IN)  :: KSW
INTEGER(KIND=JPIM), INTENT(IN)  :: KLON
INTEGER(KIND=JPIM), INTENT(IN)  :: KTILES
INTEGER(KIND=JPIM), INTENT(IN)  :: KSOTY(:)

REAL(KIND=JPRB),    INTENT(IN)  :: PALBF(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PALBICEF(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PTVH(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PALUVP(KLON),PALUVD(KLON)
REAL(KIND=JPRB),    INTENT(IN)  :: PALNIP(KLON),PALNID(KLON)
REAL(KIND=JPRB),    INTENT(IN)  :: PASN(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PMU0(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PTS(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PWND(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PWS1(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PFRTI(:,:)
REAL(KIND=JPRB),    INTENT(IN)  :: PHLICE(:) 
REAL(KIND=JPRB),    INTENT(IN)  :: PTLICE(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PLIT(:)

REAL(KIND=JPRB),    INTENT(OUT) :: PALBD(:,:)
REAL(KIND=JPRB),    INTENT(OUT) :: PALBP(:,:)
REAL(KIND=JPRB),    INTENT(OUT) :: PALBTI(:,:)
REAL(KIND=JPRB),    INTENT(OUT) :: PALB(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PEMIR(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PEMIW(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PEMIT(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PCCNL(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PCCNO(:)

LOGICAL,   INTENT(IN)  :: LDNH(:)

!     -----------------------------------------------------------------
!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZBSUR(KLON),ZTS(KLON),ZEXPL(KLON),ZEXPO(KLON),ZQ(KLON)

REAL(KIND=JPRB) :: ZEMIWTI1,ZEMIWTI2,ZEMIWTI3,ZEMIWTI4,ZEMIWTI5,ZEMIWTI6,&
 & ZEMIWTI7,ZEMIWTI8  
REAL(KIND=JPRB) :: ZADTI1,ZADTI2,ZADTI3,ZADTI4,ZADTI5,ZADTI6,ZADTI7,ZADTI8
REAL(KIND=JPRB) :: ZAPTI1,ZAPTI2,ZAPTI3,ZAPTI4,ZAPTI5,ZAPTI6,ZAPTI7,ZAPTI8
REAL(KIND=JPRB) :: ZEMIWTI9,ZADTI9,ZAPTI9 

INTEGER(KIND=JPIM) :: JL, JS, JNU, JSW, IM1, IM2

REAL(KIND=JPRB) :: ZBSPE, ZDELEM, ZPROP, ZALBICE_AR, ZALBICE_AN &
 & , ZALBD, ZALBP, ZLEN, ZW1, ZW2,ZSTAND,ZWCP  
 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZLICE(KLON),ZLWAT(KLON)
REAL(KIND=JPRB) :: ZALICE

REAL(KIND=JPRB) :: RALBSFO_NEW(0:20),ZZALBF(KLON)

!* see Moody et al. 2007 RSE
RALBSFO_NEW(0:20)=RALBSFO
RALBSFO_NEW(3)=0.27_JPRB !* Evergreen needle forest
RALBSFO_NEW(4)=0.33_JPRB !* Deciduous needle forest 
RALBSFO_NEW(5)=0.31_JPRB !* Deciduous broad forest
RALBSFO_NEW(6)=0.38_JPRB !* Evergreen broad forest
RALBSFO_NEW(18)=0.29_JPRB !* Mixed forest
RALBSFO_NEW(19)=0.29_JPRB !* == Mixed forest (Interrupted forest !)

!     ------------------------------------------------------------------
!*         1.     INITIAL CALCULATIONS
!                 --------------------

IF (LHOOK) CALL DR_HOOK('SURFRAD_CTL_MOD:SURFRAD_CTL',0,ZHOOK_HANDLE)
ZSTAND=1.0_JPRB/TSTAND

DO JL=KIDIA,KFDIA
  PALB(JL)=0.0_JPRB
  ZBSUR(JL)=0.0_JPRB
  PEMIT(JL)=0.0_JPRB
  ZTS(JL)=(PTS(JL)-TSTAND)*ZSTAND
  ZZALBF(JL)=PALBF(JL)
!MSM Ice surface albedo   
 IF (LLANDICE .AND. PLIT(JL) .GT. 0.5_JPRB) THEN
   ZZALBF(JL)=0.6_JPRB
 ENDIF
ENDDO
DO JNU=1,KTILES
  DO JL=KIDIA,KFDIA
    PALBTI(JL,JNU)=0.0_JPRB
  ENDDO
ENDDO

!* FIND LAKE POINTS WITH ICE COVER 
DO JL=KIDIA,KFDIA
  IF(PHLICE(JL) > RH_ICE_MIN_FLK ) THEN
    ZLICE(JL)=1._JPRB
    ZLWAT(JL)=0._JPRB
  ELSE
    ZLICE(JL)=0._JPRB
    ZLWAT(JL)=1._JPRB
  ENDIF
ENDDO 

!     ------------------------------------------------------------------       
!*         2.     SURFACE LONGWAVE WINDOW EMISSIVITY
!                 ----------------------------------

DO JL=KIDIA,KFDIA
  PEMIR(JL)=REMISS
ENDDO

!*         2.1    DEPENDENCE OF WINDOW EMISSIVITY ON SOIL MOISTURE
!                 ------------------------------------------------

DO JL=KIDIA,KFDIA
!            1 : WATER                  5 : SNOW ON LOW-VEG+BARE-SOIL
!            2 : ICE                    6 : DRY SNOW-FREE HIGH-VEG
!            3 : WET SKIN               7 : SNOW UNDER HIGH-VEG
!            4 : DRY SNOW-FREE LOW-VEG  8 : BARE SOIL
!            9 : LAKES 
! WATER
  ZEMIWTI1=REMISS
! SEA-ICE
  ZEMIWTI2=REMISN
! WET SKIN
  ZEMIWTI3=REMISL
! LAKES  
  ZEMIWTI9=ZEMIWTI1*ZLWAT(JL)+ZEMIWTI2*ZLICE(JL)  
  
! DRY SNOW-FREE LOW-VEG
  IF(LEVGEN)THEN
    IF (KSOTY(JL)> 0_JPIM) THEN
      JS=KSOTY(JL)
      ZWCP=1.0_JPRB/(RWCAPM(JS)-RWPWPM(JS))
      ZPROP = MAX(0.0_JPRB,MIN(1.0_JPRB,(PWS1(JL)-RWPWPM(JS))*ZWCP ) )
    ELSE
      ZPROP = 1.0_JPRB
    ENDIF
  ELSE
    ZWCP=1.0_JPRB/(RWCAP-RWPWP)
    ZPROP = MAX(0.0_JPRB, MIN(1.0_JPRB, (PWS1(JL)-RWPWP)*ZWCP ) )
  ENDIF
  ZDELEM= (REMISL-REMISD)*ZPROP
  ZEMIWTI4=REMISD+ZDELEM
! SNOW ON LOW-VEG+BARE-SOIL
  ZEMIWTI5=REMISN
! DRY SNOW-FREE HIGH-VEG
  ZEMIWTI6=ZEMIWTI4
! SNOW UNDER HIGH-VEG
  ZEMIWTI7=ZEMIWTI4
! BARE SOIL
  IF (PALBF(JL) > 0.30_JPRB) THEN
!   desert emissivity
    ZEMIWTI8=REMISD
  ELSE
    ZEMIWTI8=ZEMIWTI4
  ENDIF
! SUM OVER TILES

  IF  (LLANDICE .AND. PLIT(JL) .GT. 0.5_JPRB) THEN
  PEMIW(JL)=PFRTI(JL,1)*ZEMIWTI1+PFRTI(JL,2)*ZEMIWTI2&
   & +PFRTI(JL,3)*REMISN+PFRTI(JL,4)*REMISN&
   & +PFRTI(JL,5)*ZEMIWTI5+PFRTI(JL,6)*REMISN&
   & +PFRTI(JL,7)*ZEMIWTI7+PFRTI(JL,8)*REMISN
     ELSE
  PEMIW(JL)=PFRTI(JL,1)*ZEMIWTI1+PFRTI(JL,2)*ZEMIWTI2&
   & +PFRTI(JL,3)*ZEMIWTI3+PFRTI(JL,4)*ZEMIWTI4&
   & +PFRTI(JL,5)*ZEMIWTI5+PFRTI(JL,6)*ZEMIWTI6&
   & +PFRTI(JL,7)*ZEMIWTI7+PFRTI(JL,8)*ZEMIWTI8
  ENDIF 
  IF (LEFLAKE) THEN
    PEMIW(JL)=PEMIW(JL)+PFRTI(JL,9)*ZEMIWTI9    
  ENDIF                       
ENDDO

!*         2.2    AVERAGE EMISSIVITY OVER LONGWAVE SPECTRUM
!                 -----------------------------------------

DO JNU=1,NSIL
  DO JL=KIDIA,KFDIA
    ZBSPE=XP(1,JNU)+ZTS(JL)*(XP(2,JNU)+ZTS(JL)*(XP(3,JNU)&
     & +ZTS(JL)*(XP(4,JNU)+ZTS(JL)*(XP(5,JNU)+ZTS(JL)*(XP(6,JNU)&
     & )))))  
    ZBSUR(JL)=ZBSUR(JL)+ZBSPE
    IF (JNU == 3.OR. JNU == 4) THEN
      PEMIT(JL)=PEMIT(JL)+ZBSPE*PEMIR(JL)
    ELSE
      PEMIT(JL)=PEMIT(JL)+ZBSPE*PEMIW(JL)
    ENDIF
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  PEMIT(JL)=PEMIT(JL)/ZBSUR(JL)
ENDDO

!     ----------------------------------------------------------------          

!*         3.     SURFACE SHORTWAVE ALBEDO
!                 ------------------------

! Time interpolate the monthly values

IF (KDDN >= 15) THEN
  IM1=KMMN
  IM2=1+MOD(KMMN,12)
  ZLEN=KMON(IM1)*RDAY
  ZW1=REAL((KMON(IM1)-KDDN+15)*RDAY-KSECO,JPRB)/ZLEN
  ZW2=1.-ZW1
ELSE
  IM1=1+MOD(KMMN+10,12)
  IM2=KMMN
  ZLEN=KMON(IM1)*RDAY
  ZW1=REAL((15-KDDN)*RDAY-KSECO,JPRB)/ZLEN
  ZW2=1.-ZW1
ENDIF

DO JSW=1,KSW
  DO JL=KIDIA,KFDIA

!-- MODIS ALBEDO, SPECTRAL AND PARALLEL VS. DIFFUSE

    IF (LE4ALB) THEN
      IF (JSW <= 3) THEN
!- using MODIS UV-Vis albedo
        ZALBP=(MIN(1._JPRB-REPALB, MAX( PALUVP(JL), REPALB) ) )
        ZALBD=(MIN(1._JPRB-REPALB, MAX( PALUVD(JL), REPALB) ) )
      ELSE
!- using MODIS Near IR albedo
        ZALBP=(MIN(1._JPRB-REPALB, MAX( PALNIP(JL), REPALB) ) )
        ZALBD=(MIN(1._JPRB-REPALB, MAX( PALNID(JL), REPALB) ) )
      ENDIF

! WET SKIN
      ZADTI3=ZALBD
      ZAPTI3=ZALBP
! DRY SNOW-FREE LOW-VEG
      ZADTI4=ZALBD
      ZAPTI4=ZALBP
! DRY SNOW-FREE HIGH-VEG
      ZADTI6=ZALBD
      ZAPTI6=ZALBP
! BARE SOIL
      ZADTI8=ZALBD
      ZAPTI8=ZALBP

    ELSE

!-- ERBE ALBEDO, FLAT, ISOTROPIC
! WET SKIN
      ZADTI3=ZZALBF(JL)
      ZAPTI3=ZZALBF(JL)
! DRY SNOW-FREE LOW-VEG
      ZADTI4=ZZALBF(JL)
      ZAPTI4=ZZALBF(JL)
! DRY SNOW-FREE HIGH-VEG
      ZADTI6=ZZALBF(JL)
      ZAPTI6=ZZALBF(JL)
! BARE SOIL
      ZADTI8=ZZALBF(JL)
      ZAPTI8=ZZALBF(JL)
    ENDIF
    
! WATER
! Taylor et al. for sea
    ZADTI1=RALBSEAD
!    ZADTI1 = 0.07_JPRB !FOR DEBUGGING PURPOSES ONLY
    ZAPTI1=MAX(0.037_JPRB/(1.1_JPRB*PMU0(JL)**1.4_JPRB+0.15_JPRB),REPALB)
!    ZAPTI1=0.08_JPRB    ! DEBUGGING PURPOSES ONLY
    
! SEA-ICE
! Treat sea-ice albedo differently if atm-ocean coupled runs
! (i.e. albedo coming from NEMO)
    IF (.NOT. LAMIP) THEN
    ! ocean-atmosphere coupled
    ! multiply the NEMO broadband albedo with weights for each spectral band
      ZADTI2=PALBICEF(JL)*ICEALB_SPECWEIGHT(JSW)
      ZAPTI2=ZADTI2
    ELSE
    ! default IFS
      ZALBICE_AR=ZW1*RALBICE_AR(IM1,JSW)+ZW2*RALBICE_AR(IM2,JSW)
      ZALBICE_AN=ZW1*RALBICE_AN(IM1,JSW)+ZW2*RALBICE_AN(IM2,JSW)
      IF (LDNH(JL)) THEN
        ZADTI2=ZALBICE_AR
        ZAPTI2=ZALBICE_AR
      ELSE
        ZADTI2=ZALBICE_AN
        ZAPTI2=ZALBICE_AN
      ENDIF
    ENDIF

! SNOW ON LOW-VEG+BARE-SOIL
    ZADTI5=PASN(JL)
    ZAPTI5=PASN(JL)
   
! SNOW UNDER HIGH-VEG
    IF ( LESN09 ) THEN 
      ZADTI7=RALBSFO_NEW(NINT(PTVH(JL)))
      ZAPTI7=RALBSFO_NEW(NINT(PTVH(JL)))
    ELSE
      ZADTI7=RALBSFO
      ZAPTI7=RALBSFO
    ENDIF
    
    !* use new formulation ofr lake ice albedo calculation 
    ZALICE=EXP(-95.6_JPRB*(RTT-PTLICE(JL))/RTT)
    ZALICE=0.7_JPRB*(1.0_JPRB-ZALICE)+0.4_JPRB*ZALICE
    
    !* ZALICE=ZADTI2 !* use montlhy sea-ice albedo 
    
    ZADTI9=ZADTI1*ZLWAT(JL)+ZALICE*ZLICE(JL)
    ZAPTI9=ZAPTI1*ZLWAT(JL)+ZALICE*ZLICE(JL)


    
!* AVERAGE (DIFFUSE+PARALLEL) ALBEDO OVER SHORTWAVE SPECTRUM, FOR 
!   EACH TILE (USED IN SURFACE AND VERT.DIFFUSION SCHEMES)
    PALBTI(JL,1)=PALBTI(JL,1)+RSUN(JSW)*(ZADTI1+ZAPTI1)*0.5_JPRB
    PALBTI(JL,2)=PALBTI(JL,2)+RSUN(JSW)*(ZADTI2+ZAPTI2)*0.5_JPRB
    PALBTI(JL,3)=PALBTI(JL,3)+RSUN(JSW)*(ZADTI3+ZAPTI3)*0.5_JPRB
    PALBTI(JL,4)=PALBTI(JL,4)+RSUN(JSW)*(ZADTI4+ZAPTI4)*0.5_JPRB
    PALBTI(JL,5)=PALBTI(JL,5)+RSUN(JSW)*(ZADTI5+ZAPTI5)*0.5_JPRB
    PALBTI(JL,6)=PALBTI(JL,6)+RSUN(JSW)*(ZADTI6+ZAPTI6)*0.5_JPRB
    PALBTI(JL,7)=PALBTI(JL,7)+RSUN(JSW)*(ZADTI7+ZAPTI7)*0.5_JPRB
    PALBTI(JL,8)=PALBTI(JL,8)+RSUN(JSW)*(ZADTI8+ZAPTI8)*0.5_JPRB
    IF (LEFLAKE) THEN 
      PALBTI(JL,9)=PALBTI(JL,9)+RSUN(JSW)*(ZADTI9+ZAPTI9)*0.5_JPRB   
    ENDIF
    
! SUM OVER TILES SPECTRALLY DEFINED FOR SHORTWAVE RADIATION SCHEME
    PALBD(JL,JSW)=PFRTI(JL,1)*ZADTI1*RWEIGHT(JSW,1) &
     & +PFRTI(JL,2)*ZADTI2*RWEIGHT(JSW,2) &
     & +PFRTI(JL,3)*ZADTI3*RWEIGHT(JSW,3) &
     & +PFRTI(JL,4)*ZADTI4*RWEIGHT(JSW,4) &
     & +PFRTI(JL,5)*ZADTI5*RWEIGHT(JSW,5) &
     & +PFRTI(JL,6)*ZADTI6*RWEIGHT(JSW,6) &
     & +PFRTI(JL,7)*ZADTI7*RWEIGHT(JSW,7) &
     & +PFRTI(JL,8)*ZADTI8*RWEIGHT(JSW,8) 

    IF (LEFLAKE) THEN
      PALBD(JL,JSW)=PALBD(JL,JSW) &
     & +PFRTI(JL,9)*ZADTI9*RWEIGHT(JSW,9)    
    ENDIF

    PALBP(JL,JSW)=PFRTI(JL,1)*ZAPTI1*RWEIGHT(JSW,1) &
     & +PFRTI(JL,2)*ZAPTI2*RWEIGHT(JSW,2) &
     & +PFRTI(JL,3)*ZAPTI3*RWEIGHT(JSW,3) &
     & +PFRTI(JL,4)*ZAPTI4*RWEIGHT(JSW,4) &
     & +PFRTI(JL,5)*ZAPTI5*RWEIGHT(JSW,5) &
     & +PFRTI(JL,6)*ZAPTI6*RWEIGHT(JSW,6) &
     & +PFRTI(JL,7)*ZAPTI7*RWEIGHT(JSW,7) &
     & +PFRTI(JL,8)*ZAPTI8*RWEIGHT(JSW,8)

    IF (LEFLAKE) THEN
      PALBP(JL,JSW)=PALBP(JL,JSW) &
     & +PFRTI(JL,9)*ZAPTI9*RWEIGHT(JSW,9)    
    ENDIF
     
!* AVERAGE (DIFFUSE) ALBEDO OVER SHORTWAVE SPECTRUM (DIAGNOSTICS ONLY)
    PALB(JL)=PALB(JL)+RSUN(JSW)*PALBD(JL,JSW)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         4.     CCN EMISSION OVER OCEAN
!                 -----------------------

IF (LCCNO) THEN      
  DO JL=KIDIA,KFDIA
    IF (PWND(JL) > 30._JPRB) THEN
      ZQ(JL)=327._JPRB  
    ELSEIF (PWND(JL) > 15._JPRB) THEN  
      ZQ(JL)=EXP(0.13_JPRB*PWND(JL)+1.89_JPRB)
    ELSE
      ZQ(JL)=EXP(0.16_JPRB*PWND(JL)+1.44_JPRB)
    ENDIF  
    ZEXPO(JL)=1.2_JPRB+0.5_JPRB*LOG10(ZQ(JL))
    PCCNO(JL)=10._JPRB**ZEXPO(JL)
  ENDDO
ELSE
  DO JL=KIDIA,KFDIA
    PCCNO(JL)=RCCNSEA
  ENDDO
ENDIF    

IF (LCCNL) THEN      
  DO JL=KIDIA,KFDIA
    IF (PWND(JL) <= 15._JPRB) THEN
      ZQ(JL)=EXP(0.16_JPRB*PWND(JL)+1.45_JPRB)
    ELSE
      ZQ(JL)=EXP(0.13_JPRB*PWND(JL)+1.89_JPRB)
    ENDIF
    ZEXPL(JL)=2.21_JPRB+0.3_JPRB*LOG10(ZQ(JL))
    PCCNL(JL)=10._JPRB**ZEXPL(JL)
  ENDDO
ELSE
  DO JL=KIDIA,KFDIA
    PCCNL(JL)=RCCNLND
  ENDDO
ENDIF  

DO JL=KIDIA,KFDIA
  9401 FORMAT(1x,'CCN ',I6,3E13.5)  
ENDDO
IF (LHOOK) CALL DR_HOOK('SURFRAD_CTL_MOD:SURFRAD_CTL',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE SURFRAD_CTL
END MODULE SURFRAD_CTL_MOD
