MODULE SUSSOIL_MOD
CONTAINS
SUBROUTINE SUSSOIL(PTHRFRTI,LD_LEVGEN,LD_LESSRO,LD_LESN09)
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOS_DIM   , ONLY : NCSS,JPTEXT
USE YOS_SOIL  , ONLY : NSOTY    ,RRCSOIL  ,&
 & RLAMBDADRY         ,RLAMSAT1 ,RLAMBDAICE,&
 & RLAMBDAWAT         ,RKERST1  ,RKERST2  ,RKERST3,&
 & RWB      ,RCWPSIS  ,RWCONS   ,RWSAT    ,&
 & RWCAP    ,RWPWP    ,RSIMP    ,RLICE    ,RGH2O   ,&
 & RQWEVAP  ,RQWSBCR  ,RQSNCR   ,RWLMAX   ,RPSFR   ,&
 & RTF1     ,RTF2     ,RTF3     ,RTF4     ,&
 & RTFREEZSICE        ,RTMELTSICE         ,RDARSICE,&
 & RDANSICE ,RRCSICE  ,RCONDSICE,RDFSICE  ,RCIMIN  ,&
 & RHOICE   ,RLAMICE  ,&
 & RHOCI    ,RALFMINSN,RALFMINPSN,RALFMAXSN,RSNPER ,&
 & RHOMINSN ,RHOMAXSN ,&
 & RTAUF    ,RTAUA    ,RSFRESH  ,RFRSMALL ,RFRTINY ,&
 & RALAMSN  ,RDSNMAX  ,RDAT     ,RDAW     ,RDAI    ,&
 & RTHRFRTI,&
 & RCWPSISM, RMVGALPHA, &
 & RWCONSM, RMFACM, RNFACM, RLAMBDAM, RWSATM,  &
 & RWCAPM, RWPWPM, RWRESTM, RDMAXM, RDMINM, &
 & RQWEVAPM,RQWSBCRM,RLAMBDADRYM,RLAMSAT1M,RRCSOILM,&
 & LEVGEN, LESSRO,LESN09,&
 & RSRDEP, RSIGORMIN, RSIGORMAX,&
 & RLWCSWEA,RLWCSWEB,RLWCSWEC,RTEMPAMP,RHOMINSNA,&
 & RHOMINSNB,RHOMINSNC,RHOMINSND,RSNDTOVERA,RSNDTOVERB,&
 & RSNDTOVERC,RSNDTDESTA,RSNDTDESTB,RSNDTDESTC,RSNDTDESTROI,RHOMAXSN_NEW

USE YOS_CST   , ONLY : RTT, RPI

USE CPTAVE_MOD

#ifdef DOC
!**   *SUSSSOIL* IS THE SET-UP ROUTINE FOR COMMON BLOCK *YOESOIL*

!     PURPOSE
!     -------
!          THIS ROUTINE INITIALIZES THE CONSTANTS IN COMMON BLOCK
!     *YOESOIL*

!     INTERFACE.
!     ----------
!     CALL *SUSURF* FROM *SUPHEC*

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!        *CPTAVE* IS CALLED TO GET SOIL VALUES FOR EACH OF 3 BROAD
!                                     TEXTURAL CLASSES

!     REFERENCE.
!     ----------

!     Original    A.C.M. BELJAARS         E.C.M.W.F.      89/11/02

!     MODIFICATIONS
!     -------------
!     J.-J. MORCRETTE         E.C.M.W.F.      91/07/14
!     P. VITERBO              E.C.M.W.F.       8/10/93
!     P. Viterbo     99-03-26    Tiling of the land surface
!     C. Fischer 00-12-20 Meteo-France recode initialization of rdat to avoid
!                         memory overflow on SUN workstation
!     J.F. Estrade *ECMWF* 03-10-01 move in surf vob
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     P. Viterbo    24-05-2004      Change surface units
!     P. Viterbo   ECMWF   03-12-2004  Include user-defined RTHRFRTI
!     G. Balsamo   ECMWF   08-01-2006  Include Van Genuchten Hydro.
!     G. Balsamo   ECMWF   11-01-2006  Include sub-grid scale runoff
!     E. Dutra             12-11.2008  Include new snow parameterization
!     E. Dutra             16-11-2009  snow 2009 cleaning
!     F. Catalano A. Alessandri ENEA    10/2017     Four components soil albedo estimated from MODIS
!     ------------------------------------------------------------------

#endif

IMPLICIT NONE

! Declaration of arguments

REAL(KIND=JPRB)   ,INTENT(IN)    :: PTHRFRTI
LOGICAL           ,INTENT(IN)    :: LD_LEVGEN
LOGICAL           ,INTENT(IN)    :: LD_LESSRO
LOGICAL           ,INTENT(IN)    :: LD_LESN09

REAL(KIND=JPRB) :: ZTHECAP(JPTEXT),ZTHEPWP(JPTEXT),ZTHESAT(JPTEXT),&
 & ZWB(JPTEXT),ZWCONS(JPTEXT),ZWPSIS(JPTEXT)  
REAL(KIND=JPRB), ALLOCATABLE :: ZMVGALPHA(:),ZWCONSM(:),ZNFAC(:),&
 & ZLAMBDA(:),ZWSATM(:),ZWCAPM(:),ZWRES(:),ZWPWPM(:)

REAL(KIND=JPRB) :: ZRCGDRY, ZRHOSM, ZRHOD, ZLAMBDAQ, ZLAMBDAO, ZQ, ZLAMBDASM
REAL(KIND=JPRB) :: ZDMIN, ZDMAX, ZWFAC, ZSEMAX, ZSEMIN, ZWMAX, ZWMIN,&
 & ZFAC, ZPSIPWP, ZPSICAP
INTEGER(KIND=JPIM) :: I, JS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1.     SET CONSTANTS
!                 -------------

IF (LHOOK) CALL DR_HOOK('SUSSOIL_MOD:SUSSOIL',0,ZHOOK_HANDLE)
RLICE  =0.3_JPRB
RGH2O  =4.18E6_JPRB
RQSNCR =1.0_JPRB/15._JPRB
RWLMAX =0.2_JPRB
RPSFR=2._JPRB

!     LAND SURFACE CONSTANTS

LEVGEN=LD_LEVGEN
LESSRO=LD_LESSRO

!    SNOW LOGICALS

LESN09=LD_LESN09
IF (LESN09) RQSNCR =1.0_JPRB/10._JPRB

!GPB IF(.NOT.LEVGEN) &

     CALL CPTAVE(ZTHESAT,ZTHECAP,ZTHEPWP,ZWCONS,ZWPSIS,ZWB)

!     CONSTANTS DEFINING SOIL DISCRETIZATION

IF(.NOT.ALLOCATED(RDAT)) ALLOCATE(RDAT(NCSS))
IF(.NOT.ALLOCATED(RDAW)) ALLOCATE(RDAW(NCSS))
IF(.NOT.ALLOCATED(RDAI)) ALLOCATE(RDAI(NCSS))

IF (NCSS >= 1) RDAT(1)=0.07_JPRB
IF (NCSS >= 2) RDAT(2)=0.21_JPRB
IF (NCSS >= 3) RDAT(3)=0.72_JPRB
IF (NCSS >= 4) RDAT(4)=1.89_JPRB
DO I=1,NCSS
  RDAW(I)=RDAT(I)
  RDAI(I)=RDAT(I)
ENDDO

!     CONSTANTS FOR HYDRAULIC DIFFUSIVITY AND HYDRAULIC CONDUCTIVITY
!GPB IF (LEVGEN) THEN
! VAN GENUCHTEN (MV) HYDROLOGY
!
!     COMPUTE AVERAGE SOIL PROPERTIES FOR SOIL TYPES
!
!     SOIL TYPE 1 (COARSE)
!     SOIL TYPE 2 (MEDIUM)
!     SOIL TYPE 3 (MEDIUM FINE)
!     SOIL TYPE 4 (FINE)
!     SOIL TYPE 5 (VERY FINE)
!     SOIL TYPE 6 (EXT.TROP. ORGANIC SOIL)
!     SOIL TYPE 7 (TROP. ORGANIC SOIL)

  NSOTY=7
  IF(.NOT.ALLOCATED(ZMVGALPHA)) ALLOCATE (ZMVGALPHA(0:NSOTY))
  IF(.NOT.ALLOCATED(ZWCONSM)) ALLOCATE (ZWCONSM(0:NSOTY))
  IF(.NOT.ALLOCATED(ZNFAC)) ALLOCATE (ZNFAC(0:NSOTY))
  IF(.NOT.ALLOCATED(ZLAMBDA)) ALLOCATE (ZLAMBDA(0:NSOTY))
  IF(.NOT.ALLOCATED(ZWSATM)) ALLOCATE (ZWSATM(0:NSOTY))
  IF(.NOT.ALLOCATED(ZWCAPM)) ALLOCATE (ZWCAPM(0:NSOTY))
  IF(.NOT.ALLOCATED(ZWRES)) ALLOCATE (ZWRES(0:NSOTY))
  IF(.NOT.ALLOCATED(ZWPWPM)) ALLOCATE (ZWPWPM(0:NSOTY))
  IF(.NOT.ALLOCATED(RCWPSISM)) ALLOCATE (RCWPSISM(0:NSOTY))
  IF(.NOT.ALLOCATED(RMVGALPHA)) ALLOCATE (RMVGALPHA(0:NSOTY))
  IF(.NOT.ALLOCATED(RWCONSM)) ALLOCATE (RWCONSM(0:NSOTY))
  IF(.NOT.ALLOCATED(RMFACM)) ALLOCATE (RMFACM(0:NSOTY))
  IF(.NOT.ALLOCATED(RNFACM)) ALLOCATE (RNFACM(0:NSOTY))
  IF(.NOT.ALLOCATED(RLAMBDAM)) ALLOCATE (RLAMBDAM(0:NSOTY))
  IF(.NOT.ALLOCATED(RWSATM)) ALLOCATE (RWSATM(0:NSOTY))
  IF(.NOT.ALLOCATED(RWCAPM)) ALLOCATE (RWCAPM(0:NSOTY))
  IF(.NOT.ALLOCATED(RWPWPM)) ALLOCATE (RWPWPM(0:NSOTY))
  IF(.NOT.ALLOCATED(RWRESTM)) ALLOCATE (RWRESTM(0:NSOTY))
  IF(.NOT.ALLOCATED(RDMAXM)) ALLOCATE (RDMAXM(0:NSOTY))
  IF(.NOT.ALLOCATED(RDMINM)) ALLOCATE (RDMINM(0:NSOTY))
  IF(.NOT.ALLOCATED(RQWEVAPM)) ALLOCATE (RQWEVAPM(0:NSOTY))
  IF(.NOT.ALLOCATED(RQWSBCRM)) ALLOCATE (RQWSBCRM(0:NSOTY))

  ZMVGALPHA(1:NSOTY)=(/3.83_JPRB,3.14_JPRB,0.83_JPRB,3.67_JPRB,2.65_JPRB,1.300_JPRB,3.14_JPRB/)
  ZWCONSM(1:NSOTY)=(/6.94E-6_JPRB,1.16E-6_JPRB,2.63E-7_JPRB,2.87E-6_JPRB,&
 &             1.740E-6_JPRB, 0.93E-6_JPRB,1.16E-6_JPRB/)
  ZNFAC(1:NSOTY)=(/1.3774_JPRB,1.1804_JPRB,1.2539_JPRB,1.1012_JPRB,1.1033_JPRB,1.2039_JPRB,1.1804_JPRB/)
  ZLAMBDA(1:NSOTY)=(/1.250_JPRB,-2.3421_JPRB,-0.5884_JPRB,-1.9772_JPRB,2.5000_JPRB,&
 &             0.40000_JPRB,-2.3421_JPRB/)
  ZWSATM(1:NSOTY)=(/0.403_JPRB,0.439_JPRB,0.430_JPRB,0.520_JPRB,0.614_JPRB,0.766_JPRB,0.439_JPRB/)
  ZWRES(1:NSOTY)=(/0.025_JPRB,0.010_JPRB,0.010_JPRB,0.010_JPRB,0.010_JPRB,0.010_JPRB,0.010_JPRB/)

! Set 0 SOILTYPE (WATER) VALUES TO 0
  ZMVGALPHA(0)=0.0_JPRB
  ZWCONSM(0)=0.0_JPRB
  ZNFAC(0)=0.0_JPRB
  ZLAMBDA(0)=0.0_JPRB
  ZWSATM(0)=0.0_JPRB
  ZWCAPM(0)=0.0_JPRB
  ZWRES(0)=0.0_JPRB
  ZWPWPM(0)=0.0_JPRB
  RCWPSISM(0)=0.0_JPRB
  RMVGALPHA(0)=0.0_JPRB
  RWCONSM(0)=0.0_JPRB
  RMFACM(0)=0.0_JPRB
  RNFACM(0)=0.0_JPRB
  RLAMBDAM(0)=0.0_JPRB
  RWSATM(0)=0.0_JPRB
  RWCAPM(0)=0.0_JPRB
  RWPWPM(0)=0.0_JPRB
  RWRESTM(0)=0.0_JPRB
  RDMAXM(0)=0.0_JPRB
  RDMINM(0)=0.0_JPRB
  RQWEVAPM(0)=0.0_JPRB
  RQWSBCRM(0)=0.0_JPRB

  ZFAC=1000._JPRB*100._JPRB/(1000._JPRB*9.8_JPRB)
  ZPSIPWP=-15._JPRB*ZFAC   ! Classic permanent wilting point
  IF (LEVGEN) THEN
    ZPSICAP=-0.10_JPRB*ZFAC  ! Value valid for sandy soil in the literature---> larger AWC 
                             ! favourably compared to FC obs for medium soils (Ukraine/Russia)
                             ! produce invariant equilibrium after rescaling from TESSEL
                             ! enlarge the range of action for SM analysis (active if PWP<W<CAP)
  ELSE
    ZPSICAP=-0.33_JPRB*ZFAC  ! Value mostly used in literature (Hillel, 1998)
  ENDIF
! ZPSICAP=-0.05_JPRB*ZFAC  ! Value used in HYPRES framework (only!)
! WRITE(NULOUT,'(/a)')'SOTYPE  RCWPSISM    RMVGALPHA     RWCONSM' &
! &     //'       RMFACM' &
! &     //'       RNFACM     RLAMBDAM       RWSATM       RWCAPM ' &
! &     //'      RWPWPM      RWRESTM       RDMAXM       RDMINM ' &
! &     //'    RQWEVAPM     RQWSBCRM' 

  DO JS=1,NSOTY
    RCWPSISM(JS)=(1._JPRB/ZMVGALPHA(JS))
    RMVGALPHA(JS)=ZMVGALPHA(JS)
    RWCONSM(JS)=ZWCONSM(JS)
    RMFACM(JS)=1._JPRB-(1._JPRB/ZNFAC(JS))
    RLAMBDAM(JS)=ZLAMBDA(JS)
    RNFACM(JS)=ZNFAC(JS)
    RWSATM(JS)=ZWSATM(JS)
    ZWCAPM(JS)=ZWRES(JS)+(ZWSATM(JS)-ZWRES(JS)) &
 &           *(1._JPRB/(1._JPRB+((ABS(RMVGALPHA(JS)*ZPSICAP)) &
 &           **RNFACM(JS))))**RMFACM(JS)
    RWCAPM(JS)=ZWCAPM(JS)
    ZWPWPM(JS)=ZWRES(JS)+(ZWSATM(JS)-ZWRES(JS)) &
 &           *(1._JPRB/(1._JPRB+((ABS(RMVGALPHA(JS)*ZPSIPWP)) &
 &           **RNFACM(JS))))**RMFACM(JS)
    RWPWPM(JS)=ZWPWPM(JS)
    RWRESTM(JS)=ZWRES(JS)
    ZWMAX=.999_JPRB*RWSATM(JS)
    ZSEMAX=(ZWMAX-RWRESTM(JS))/(RWSATM(JS)-RWRESTM(JS))
    ZWFAC=(RWSATM(JS)-RWRESTM(JS))
    ZDMAX=(((1._JPRB-RMFACM(JS))*RWCONSM(JS))/ &
 &        (RMVGALPHA(JS)*RMFACM(JS)*ZWFAC)) &
 &         *(ZSEMAX**(RLAMBDAM(JS)-(1._JPRB/RMFACM(JS)))) &
 &         *(((1._JPRB-(ZSEMAX**(1._JPRB/RMFACM(JS))))**(RMFACM(JS))) &
 &         + ((1._JPRB-(ZSEMAX**(1._JPRB/RMFACM(JS))))**(-RMFACM(JS)))-2._JPRB)
    RDMAXM(JS)=ZDMAX
    ZWMIN=1.001_JPRB*RWRESTM(JS)
    ZSEMIN=(ZWMIN-RWRESTM(JS))/(RWSATM(JS)-RWRESTM(JS))
    ZDMIN=(((1._JPRB-RMFACM(JS))*RWCONSM(JS))/ &
 &        (RMVGALPHA(JS)*RMFACM(JS)*ZWFAC)) &
 &         *(ZSEMIN**(RLAMBDAM(JS)-(1._JPRB/RMFACM(JS)))) &
 &         *(((1._JPRB-(ZSEMIN**(1._JPRB/RMFACM(JS))))**(RMFACM(JS))) &
 &         + ((1._JPRB-(ZSEMIN**(1._JPRB/RMFACM(JS))))**(-RMFACM(JS)))-2._JPRB)
    RDMINM(JS)=ZDMIN
 
    RQWEVAPM(JS)=1._JPRB/(RWCAPM(JS)-RWPWPM(JS))
    RQWSBCRM(JS)=RPI/(1.6_JPRB*RWCAPM(JS))
!          WRITE(NULOUT,'(1X,I2,20(1X,E12.5))')JS,RCWPSISM(JS), &
! &        RMVGALPHA(JS), &
! &        RWCONSM(JS),RMFACM(JS),RNFACM(JS),RLAMBDAM(JS), &
! &        RWSATM(JS), &
! &        RWCAPM(JS),RWPWPM(JS),RWRESTM(JS), &
! &        RDMAXM(JS),RDMINM(JS), &
! &        RQWEVAPM(JS),RQWSBCRM(JS)
  ENDDO
 
!GPB ELSE
! CLAPP AND HORNBERGER (CH) HYDROLOGY ON A LOAMY SOIL
!        MEDIUM SOIL TYPE VALUES ARE CHOSEN EVERYWHERE

  RWB=ZWB(2)
  RWCONS=ZWCONS(2)
  RCWPSIS=ZWPSIS(2)

!     CONSTANTS FOR DEFINING THE SOIL WATER HOLDING CHARACTERISTICS

!        MEDIUM SOIL TYPE VALUES ARE CHOSEN EVERYWHERE

  RWSAT=ZTHESAT(2)
  RWCAP=ZTHECAP(2)
  RWPWP=ZTHEPWP(2)

  RQWEVAP=1.0_JPRB/(RWCAP-RWPWP)
  RQWSBCR=RPI/(RWCAP*1.6_JPRB)
!GPB ENDIF
!     CONSTANTS FOR THERMAL DIFFUSIVITY

!     VALUES TAKEN FROM Peters-Liddard et al. 1998
!     COMMON TO CH AND MV
ZRHOSM=2700._JPRB
ZLAMBDAQ=7.7_JPRB
ZLAMBDAO=2._JPRB
ZQ=0.4_JPRB
ZLAMBDASM=(ZLAMBDAQ**ZQ)*(ZLAMBDAO**(1.0_JPRB-ZQ))
ZRCGDRY=1.6E6_JPRB

RLAMBDAICE=2.2_JPRB
RLAMBDAWAT=0.57_JPRB
RKERST1=0.1_JPRB
RKERST2=1.0_JPRB
RKERST3=1.0_JPRB

!GPB IF(LEVGEN)THEN

!     VAN GENUCHTEN 
  
  IF(.NOT.ALLOCATED(RLAMBDADRYM)) ALLOCATE (RLAMBDADRYM(NSOTY))
  IF(.NOT.ALLOCATED(RLAMSAT1M)) ALLOCATE (RLAMSAT1M(NSOTY))
  IF(.NOT.ALLOCATED(RRCSOILM)) ALLOCATE (RRCSOILM(NSOTY))
  DO JS=1,NSOTY
    ZRHOD=(1.0_JPRB-RWSATM(JS))*ZRHOSM
    RLAMBDADRYM(JS)=(0.135_JPRB*ZRHOD+64.7_JPRB)/(ZRHOSM-0.947*ZRHOD)
    RLAMSAT1M(JS)=ZLAMBDASM**(1.0_JPRB-RWSATM(JS))
!     SOIL HEAT CAPACITY

    RRCSOILM(JS)=(1.0_JPRB-RWSATM(JS))*ZRCGDRY+RWCAPM(JS)*RGH2O
  ENDDO
!GPB ELSE

!     CLAPP-HORNBERGER

  ZRHOD=(1.0_JPRB-RWSAT)*ZRHOSM
  RLAMBDADRY=(0.135_JPRB*ZRHOD+64.7_JPRB)/(ZRHOSM-0.947_JPRB*ZRHOD)
  RLAMSAT1=ZLAMBDASM**(1.0_JPRB-RWSAT)

!     SOIL HEAT CAPACITY

!     SET TO A CONSTANT REPRESENTATIVE OF THE VALUE AT FIELD CAPACITY

  RRCSOIL=(1-RWSAT)*ZRCGDRY+RWCAP*RGH2O

!GPB ENDIF

!     FORWARD TIMESTEP WEIGHTING FACTOR FOR TIME INTEGRATION

RSIMP=1._JPRB

!     CONSTANTS FOR SOIL WATER FREEZING

RTF1=RTT+1.0_JPRB
RTF2=RTT-3._JPRB
RTF3=0.5_JPRB*(RTF1+RTF2)
RTF4=RPI/(RTF1-RTF2)

!     SEA ICE RELATED CONSTANTS

RTFREEZSICE=RTT - 1.7_JPRB
RTMELTSICE=RTT
RDARSICE=1.5_JPRB
RDANSICE=1.5_JPRB
RRCSICE=1.884E06_JPRB
RCONDSICE=2.03_JPRB
RDFSICE=RCONDSICE/RRCSICE
RCIMIN=0.0_JPRB

!     SURFACE RUNOFF RELATED CONSTANTS

RSRDEP=0.5_JPRB
RSIGORMIN=100.0_JPRB
RSIGORMAX=1000.0_JPRB
    
!     SNOW RELATED CONSTANTS

RHOICE=920._JPRB
RLAMICE=2.22_JPRB
RHOCI=2.05E6_JPRB
RALFMINSN=0.50_JPRB
RALFMINPSN=0.80_JPRB ! Pirazzini (2004) 
RALFMAXSN=0.85_JPRB
RSNPER=10000.0_JPRB
RHOMINSN=100._JPRB
RHOMAXSN=300._JPRB
RTAUF=0.24_JPRB
RTAUA=0.008_JPRB
RSFRESH=1.0_JPRB/3600._JPRB
RFRTINY =1.E-7_JPRB
RFRSMALL=10._JPRB*RFRTINY
RALAMSN=1.88_JPRB
RDSNMAX=1._JPRB

!    SNOW RELATED - NEW DENSITY PARAMETERIZATION
RLWCSWEA=200._JPRB
RLWCSWEB=0.03_JPRB
RLWCSWEC=0.1_JPRB
RTEMPAMP=4.0_JPRB

RHOMINSNA=109._JPRB
RHOMINSNB=6._JPRB
RHOMINSNC=26._JPRB
RHOMINSND=50._JPRB

RSNDTOVERA=3.7E7_JPRB
RSNDTOVERB=8.1E-2_JPRB
RSNDTOVERC=1.8E-2_JPRB

RSNDTDESTA=2.8E-6_JPRB
RSNDTDESTB=4.2E-2_JPRB
RSNDTDESTC=460.0_JPRB
RSNDTDESTROI=150.0_JPRB

RHOMAXSN_NEW=450._JPRB

! Tiles constants

RTHRFRTI=PTHRFRTI

IF (LHOOK) CALL DR_HOOK('SUSSOIL_MOD:SUSSOIL',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE SUSSOIL
END MODULE SUSSOIL_MOD
