MODULE SUSURF_CTL_MOD
CONTAINS
SUBROUTINE SUSURF_CTL(KSW,KCSS,KSIL,KTILES,KTSW,&
 & LD_LEFLAKE, LD_LEOCML,&       
 & LD_LLCCNL,LD_LLCCNO,LD_LEVGEN,LD_LESSRO,LD_LELAIV,LD_LESN09,&
 & LD_LEOCWA,LD_LEOCCO,LD_LEOCSA,LD_LEOCLA,LD_LLE4ALB,&
 & LD_LSCMEC,LD_LROUGH,PEXTZ0M,PEXTZ0H,&
 & PTHRFRTI,PTSTAND,PXP,PRCCNSEA,PRCCNLND,&
 & PRSUN)


USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!ifndef INTERFACE

USE YOS_DIM   , ONLY : NCSS, NTILES, NMONTH

USE SUSCST_MOD
USE SUSTHF_MOD
USE SUSRAD_MOD
USE SUSSOIL_MOD
USE SUSVEG_MOD
USE SUVEXC_MOD
USE SUVEXCS_MOD
USE SUSFLAKE_MOD 
USE SUSOCEAN_ML_MOD 

#ifdef DOC
!**   *SUSURF* IS THE SET-UP ROUTINE FOR COMMON BLOCK *YOESOIL*

!     PURPOSE
!     -------
!          THIS ROUTINE INITIALIZES THE CONSTANTS IN COMMON BLOCK
!     *YOESOIL*

!     INTERFACE.
!     ----------
!     CALL *SUSURF* FROM *SUPHEC*

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!     SUSCST     Setup general constants
!     SUSTHF     Setup thermodynamic function constants
!     SUSRAD     Setup radiation constants
!     SUSSOIL    Setup soil constants
!     SUSVEG     Setup vegetation constants
!     SUVEXC     Setup surface exchange coefficients constants
!     SUVEXCS    Setup static stability constants

!     REFERENCE.
!     ----------

!     Original    A.C.M. BELJAARS         E.C.M.W.F.      89/11/02

!     MODIFICATIONS
!     -------------
!     J.-J. MORCRETTE         E.C.M.W.F.      91/07/14
!     P. VITERBO              E.C.M.W.F.       8/10/93
!     P. Viterbo     99-03-26    Tiling of the land surface
!     C. Fischer 00-12-20 Meteo-France recode initialization of rdat to avoid
!                         memory overflow on SUN workstation
!     J.F. Estrade *ECMWF* 03-10-01 move in surf vob
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     P. Viterbo    24-05-2004      Change surface units
!     P. Viterbo   ECMWF   03-12-2004  Include user-defined RTHRFRTI
!     P. Viterbo   ECMWF   May 2005    Externalise surf
!     JJMorcrette 20060511 MODIS albedo
!     G. Balsamo   ECMWF   10-01-2006  Include Vangenucthen Hydro.
!     G. Balsamo   ECMWF   11-01-2006  Include sub-grid surface runoff
!     V. Stepanenko/G. Balsamo 01-05-2008 add lake tile
!     Y. Takaya    ECMWF   07-10-2008  Include setup for ocean mixed layer model 
!     G. Balsamo   ECMWF   13-10-2008  Include switch for liquid water in snow
!     E. Dutra             12-11.2008  Include new snow parameterization
!     Y. Takaya    ECMWF   21-08-2009  Include Langmuir effect to skin layer model


!     S. Boussetta/G.Balsamo May 2009 Add lai
!     E. Dutra             16-11-2009  snow 2009 cleaning
!  INTERFACE: 

!    Integers (In):

!      KSW       : NUMBER OF SHORTWAVE SPECTRAL INTERVALS
!      KCSS      : Number of soil levels
!      KSIL      : NUMBER OF (infrared) SPECTRAL INTERVALS
!      KTILES    : Number of surface tiles
!      KTSW      : Maximum possible number of sw spectral intervals

!    Logicals (In):

!      LD_LLCCNL : .T. IF CCN CONCENTRATION OVER LAND IS DIAGNOSED
!      LD_LLCCNO : .T. IF CCN CONCENTRATION OVER OCEAN IS DIAGNOSED
!      LD_LEVGEN : .T. IF VAN GENUCHTEN HYDRO IS ACTIVATED
!      LD_LESSRO : .T. IF SUB-GRID SURFACE RUNOFF IS ACTIVATED
!      LD_LELAIV : .T. IF LAI FROM CLIMATE FIELDS IS USED
!      LD_LESN09 : .T. IF SNOW 2009 is ACTIVATED
!      LD_LEOCWA : .T. if WARM OCEAN LAYER PARAMETRIZATION active
!      LD_LEOCCO : .T. if COOL OCEAN SKIN PARAMETRIZATION active
!      LD_LEOCSA : .T. if SALINTY EFFECT ON SATURATION AT OCEAN SURFACE active
!      LD_LEOCLA : .T. if LANGMUIR CIRCULATION EFFECT IN VOSKIN active
!      LD_LLE4ALB: .T. IF MODIS ALBEDO IS USED
!      LD_LEFLAKE: .T. IF FLAKE PARAMETRIZATION USED FOR LAKES 
!      LD_LEOCML : .T. IF OCEAN MIXED LAYER MODEL (KPP) ACTIVATE

!    Reals (In):

!      PTHRFRTI  : ! MINIMUM THRESHOLD FOR TILE FRACTION
!      PTSTAND   : ! REFERENCE TEMPERATURE FOR TEMPERATURE DEPENDENCE
!      PXP       : ! POLYNOMIAL COEFFICIENTS OF PLANCK FUNCTION
!      PRCCNSEA  : ! NUMBER CONCENTRATION (CM-3) OF CCNs OVER SEA
!      PRCCNLND  : ! NUMBER CONCENTRATION (CM-3) OF CCNs OVER LAND
!      PRSUN     : ! SOLAR FRACTION IN SPECTRAL INTERVALS

!     ------------------------------------------------------------------

#endif

IMPLICIT NONE

! Declaration of arguments

INTEGER(KIND=JPIM),INTENT(IN)    :: KSW 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTHRFRTI
INTEGER(KIND=JPIM),INTENT(IN)    :: KCSS  
INTEGER(KIND=JPIM),INTENT(IN)    :: KSIL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTSW 
LOGICAL           ,INTENT(IN)    :: LD_LLCCNL 
LOGICAL           ,INTENT(IN)    :: LD_LLCCNO
LOGICAL           ,INTENT(IN)    :: LD_LEVGEN  
LOGICAL           ,INTENT(IN)    :: LD_LESSRO
LOGICAL           ,INTENT(IN)    :: LD_LELAIV 
LOGICAL           ,INTENT(IN)    :: LD_LESN09
LOGICAL           ,INTENT(IN)    :: LD_LLE4ALB  
LOGICAL           ,INTENT(IN)    :: LD_LEOCWA
LOGICAL           ,INTENT(IN)    :: LD_LEOCCO
LOGICAL           ,INTENT(IN)    :: LD_LEOCSA
LOGICAL           ,INTENT(IN)    :: LD_LEOCLA
LOGICAL           ,INTENT(IN)    :: LD_LSCMEC
LOGICAL           ,INTENT(IN)    :: LD_LROUGH
LOGICAL           ,INTENT(IN)    :: LD_LEFLAKE
LOGICAL           ,INTENT(IN)    :: LD_LEOCML 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXTZ0M
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXTZ0H
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP(6,6) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCCNSEA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCCNLND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSUN(:) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSURF_CTL_MOD:SUSURF_CTL',0,ZHOOK_HANDLE)

! Initialise dimensions

NCSS=KCSS
NTILES=KTILES
NMONTH=12

! Initialise fundamental constants

CALL SUSCST

! Initialise constants defined from fundamental constants

CALL SUSTHF

! Initialise radiation constants

CALL SUSRAD(KSW,KSIL,KTSW,&
          & LD_LLCCNL,LD_LLCCNO,LD_LLE4ALB,&
          & PTSTAND,PXP,PRCCNSEA,PRCCNLND,PRSUN)

! Initialise soil constants

CALL SUSSOIL(PTHRFRTI,LD_LEVGEN,LD_LESSRO,LD_LESN09)

! Initialise vegetation constants

CALL SUSVEG(LD_LELAIV)

! Initialise surface exchange coefficient constants

CALL SUVEXC(LD_LEOCWA,LD_LEOCCO,LD_LEOCSA,LD_LEOCLA,&
          & LD_LSCMEC,LD_LROUGH,PEXTZ0M,PEXTZ0H)

! Initialise static stability functions constant

CALL SUVEXCS

CALL SUSFLAKE(LD_LEFLAKE) 

! Initialize the ocean mixed layer model.

CALL SUSOCEAN_ML(LD_LEOCML)

IF (LHOOK) CALL DR_HOOK('SUSURF_CTL_MOD:SUSURF_CTL',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE SUSURF_CTL
END MODULE SUSURF_CTL_MOD
