SUBROUTINE RDFVAR(CFILE,CVAR,POUTPUT)

USE YOMFORC1S, ONLY : JPSTPFC  , DTIMFC   ,RTSTFC   ,NSTPFC,DIMFORC
USE YOMLUN1S , ONLY : NULOUT   ,NULNAM
USE YOMDYN1S , ONLY : TSTEP , NSTEP
USE YOMCT01S , ONLY : NSTOP    ,NSTART
USE YOMGC1S  , ONLY : LMASK
USE YOMLOG1S , ONLY : NDIMCDF,LREADDAY
USE YOMCST   , ONLY : RDAY 
USE YOMRIP   , ONLY : RTIMTR,RTIMST
USE YOMDPHY  , ONLY : NPOI,    NLON      ,NLAT     ,NLALO
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE NETCDF
USE NETCDF_UTILS, ONLY: NCERROR
USE MPL_MODULE
#ifdef DOC

!**** *RDFVAR  * - ROUTINE TO READ A SINGLE FIELD FROM A NETCDF FILE
!
! INTERFACE
! ---------
! CFILE		C	INPUT FILE
! CVAR		C	NETCDF VARIABLE NAME
! POUTPUT	R	ARRAY WITH OUTPUT VALUES
!
#endif
IMPLICIT NONE

#include "minmax.intfb.h"

CHARACTER(LEN=*),INTENT(IN) :: CFILE,CVAR
REAL(KIND=JPRB),DIMENSION(NPOI,JPSTPFC),INTENT(OUT) :: POUTPUT


!* LOCAL
CHARACTER(LEN=100) :: CNAME,CTUNITS
INTEGER(KIND=JPIM) ::  NCID,NIDTIM,NIDLON,NIDLAT,NITIM,NILAT,NILON,TVARID
INTEGER(KIND=JPIM) ::  VARID,VNDIMS,VDIMIDS(NF90_MAX_VAR_DIMS)
INTEGER(KIND=JPIM) ::  IYEAR,IMON,IDAY,IHOU,IMIN,ISEC,IFSS
INTEGER(KIND=JPIM) ::  IRDST,INSTFC,JT,NPTOT,NMX,NMY

REAL(KIND=JPRB)    :: ZFOR_START,ZFOR_REF,ZFOR_END,ZTSTRT,&
                      ZTIMEN,ZFCEN,ZZV,ZDELS

INTEGER ISTART4(4),ICOUNT4(4)
INTEGER ISTART3(3),ICOUNT3(3)
REAL(KIND=JPRB),ALLOCATABLE :: ZREAL3(:,:)

#include "fcttim.h"

POUTPUT(:,:)=0._JPRB
!* Open forcing file 
CALl NCERROR( NF90_OPEN(TRIM(CFILE),NF90_NOWRITE,NCID),'Opening '//TRIM(CFILE))
WRITE(NULOUT,*)'NETCDF-FILE ',CFILE(1:len_trim(CFILE)),' OPENED ON UNIT ',NCID

!! get variable ID
CALL NCERROR( NF90_INQ_VARID(NCID, TRIM(CVAR), VARID),'getting varid '//TRIM(CVAR) )

!* Find VARIABLE DIMENSIONS 
CALL NCERROR( NF90_INQUIRE_VARIABLE(NCID,VARID,ndims=VNDIMS,dimids=VDIMIDS),'getting var info '//TRIM(CVAR) )
NIDTIM=VDIMIDS(VNDIMS)
NIDLAT=-1
IF( VNDIMS == 2 ) THEN
  NIDLON=VDIMIDS(1)
ELSEIF (VNDIMS == 3 .OR. VNDIMS == 4 ) THEN
  NIDLON=VDIMIDS(1)
  NIDLAT=VDIMIDS(2)
ELSE
  WRITE(NULOUT,*) 'WRONG NUMBER OF VARIABLE DIMENSIONS',TRIM(CFILE),TRIM(CVAR),VNDIMS
  CALL ABOR1('RDFVAR')
ENDIF
IF ( NIDLAT == -1 .AND. DIMFORC == 2 ) THEN
  WRITE(NULOUT,*) 'LAT NOT FOUND IN DIMENSION, BUT NAMELIST SAYS FORCING IS 2D'
  CALL ABOR1('RDFVAR')
ENDIF 

!* get dimension lengths 
CALL NCERROR( NF90_INQUIRE_DIMENSION(NCID,NIDTIM,len=NITIM),'getting time len '//TRIM(CVAR) )
CALL NCERROR( NF90_INQUIRE_DIMENSION(NCID,NIDLON,len=NILON),'getting lon len '//TRIM(CVAR) )
IF (NIDLAT /= -1 ) THEN
  CALL NCERROR( NF90_INQUIRE_DIMENSION(NCID,NIDLAT,len=NILAT),'getting lat len '//TRIM(CVAR) )
ELSE
  NILAT = 0
ENDIF
IF(NILON.NE.NLON .OR. NILAT.NE.NLAT) THEN
  WRITE(*,*)'NLON OR NLAT NOT SPECIFIED CORRECTLY:'
  WRITE(*,*)'NLON IN NAMELIST AND FORCING: ',NLON,NILON
  WRITE(*,*)'NLAT IN NAMELIST AND FORCING: ',NLAT,NILAT
  CALL ABOR1('RDFVAR')
ENDIF

!* Read time 
CALL NCERROR( NF90_INQ_VARID(NCID, 'time', TVARID),'getting varid '//'TIME')
CALL NCERROR( NF90_GET_ATT(NCID, TVARID, 'units', CTUNITS),'getting time units')
print*,CTUNITS
IF ( CTUNITS(1:14) == "seconds since ") THEN
  READ(CTUNITS,'(14x,1i4,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2)')IYEAR,IMON,IDAY,IHOU,IMIN,ISEC
  ZDELS=1._JPRB
ELSEIF ( CTUNITS(1:14) == "minutes since ") THEN
  READ(CTUNITS,'(14x,1i4,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2)')IYEAR,IMON,IDAY,IHOU,IMIN,ISEC
  ZDELS=60._JPRB
ELSEIF ( CTUNITS(1:12) == "hours since ") THEN
  READ(CTUNITS,'(12x,1i4,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2)')IYEAR,IMON,IDAY,IHOU,IMIN,ISEC
  ZDELS=3600._JPRB
ELSEIF ( CTUNITS(1:11) == "days since ") THEN
  READ(CTUNITS,'(11x,1i4,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2,1x,1i2.2)')IYEAR,IMON,IDAY,IHOU,IMIN,ISEC
  ZDELS=86400._JPRB
ELSE
  PRINT*, "Time units in netcdf not supported"
  PRINT*,CTUNITS
  CALL ABOR1('in rdfvar')
ENDIF

CALL NCERROR( NF90_GET_VAR(NCID,TVARID,ZFOR_START,start=(/1/)),'reading time start')
CALL NCERROR( NF90_GET_VAR(NCID,TVARID,ZFOR_END,start=(/NITIM/)),'reading time end')

!* forcing start/end 
IFSS=3600*IHOU+60*IMIN+ISEC
ZFOR_REF=RTIME(IYEAR,IMON,IDAY,IFSS)  !! forcing reference time 
ZFOR_START=ZFOR_REF+ZFOR_START*ZDELS  !! forcing start 
ZFOR_END=ZFOR_REF+ZFOR_END*ZDELS       !! forcing end 

!* check if forcing is available since model run start 
ZTSTRT=RTIMTR-0.5*TSTEP  ! RTIMTR == ABSOLUTE TIME OF THE MODEL / Midpoint time of current time step (Julian seconds)
IF (ZFOR_START.GT.ZTSTRT) THEN
  WRITE(NULOUT,*) " STOP IN ROUTINE SUFCDF"
  WRITE(NULOUT,*) " RUN STARTS EARLIER THAN FORCING DATA"
  WRITE(NULOUT,*) " RUN STARTS AT ",ZTSTRT
  WRITE(NULOUT,*) " FORCING DATA STARTS ",ZFOR_START-ZTSTRT,&
     &     ' SECS LATER'
  WRITE(NULOUT,*) CTUNITS
  CALL ABOR1('in rdfvar')
ENDIF

!* find first forcing step needed to load:
! IRDST=NINT((ZTSTRT-ZFOR_START)/DTIMFC)+1
IRDST=FLOOR((ZTSTRT-ZFOR_START)/DTIMFC)+1 ! new
!RTSTFC=ZFOR_REF+DTIMFC*(IRDST-1)
RTSTFC=ZFOR_START+DTIMFC*(IRDST-1)
ZZV=MOD(NINT(RTSTFC-RTIME(IYEAR,IMON,IDAY,0)),NINT(RDAY))
IF ( ZZV  .NE. 0 ) THEN
   WRITE(*,*) '1st Forcing read not start at 00:00 UTC - this is problematic'
   WRITE(*,*) ' Please check this carefully to avoid interpolation problems!!' 
!    CALL ABOR1('in rdfvar')
ENDIF
!* find the number of steps to read 
! ZTIMEN=ZTSTRT+(NSTOP-NSTART+1)*TSTEP
! ZTIMEN=ZTSTRT+(NSTOP-NSTART+1)*TSTEP 
ZTIMEN=ZTSTRT+(NSTOP-NSTEP+1)*TSTEP ! Fixed for daily chunks read 

! print*,'NSTOP,NSTART,IRDST,ZTIMEN',NSTOP,NSTART,IRDST,ZTIMEN
DO JT=IRDST,NITIM
  INSTFC=JT-IRDST+1
  ZFCEN=RTSTFC+DTIMFC*INSTFC
!   print*,'zz',JT,INSTFC,ZFCEN,(ZFCEN-ZTSTRT)/REAL(86400.)
  IF(ZFCEN.GT.ZTIMEN) EXIT
ENDDO
NSTPFC=INSTFC

!* check if forcing is available since model run start 
! ZFCEN=RTSTFC+DTIMFC*(NSTPFC-1) ! removed for daily chunks read 
IF (ZFCEN+TSTEP.LT.ZTIMEN) THEN
  WRITE(NULOUT,*) " STOP IN ROUTINE SUFCDF"
  WRITE(NULOUT,*) " RUN ENDS LATER THAN FORCING DATA"
  WRITE(NULOUT,*) ' RUN BETWEEN ',ZTSTRT-ZTSTRT,' AND ',(ZTIMEN-ZTSTRT)/REAL(86400.)
  WRITE(NULOUT,*) ' FORCING DATA BETWEEN ',(RTSTFC-ZTSTRT)/REAL(86400.),' AND ',(ZFCEN-ZTSTRT)/REAL(86400.),' Difference ',ABS(ZFCEN+TSTEP-ZTIMEN),' Time step model ',TSTEP
  CALL ABOR1('RDFVAR')
ENDIF

IF(NSTPFC.GT.JPSTPFC)THEN
  WRITE(*,*)'INCREASE JPSTPFC TO CONTAIN FORCING FIELDS'
  WRITE(*,*)'NSTPFC=',NSTPFC,'JPSTPFC=',JPSTPFC
  IF (LREADDAY) THEN
    WRITE(*,*) "Reading daily chunks, this should not be a problem...."
    NSTPFC=JPSTPFC
  ELSE
    CALL  ABOR1('RDFVAR')
  ENDIF
ENDIF


WRITE(NULOUT,*) 'Forcing start-model start,Forcing end - model start,model end - model start (seconds since..):'
WRITE(NULOUT,*)RTSTFC-ZTSTRT,ZFCEN-ZTSTRT,ZTIMEN-ZTSTRT

WRITE(NULOUT,*)' FORCING READ BETWEEN FORCING TIME STEPS ',&
     &  IRDST,' AND ',IRDST+NSTPFC-1



!** ----- Read the data 
IF (DIMFORC == 2 ) THEN
  ISTART4(1:4) =(/1,1,1,IRDST/)
  ICOUNT4(1:4) =(/NILON,NILAT,1,NSTPFC/)
  ISTART3(1:3) =(/1,1,IRDST/)
  ICOUNT3(1:3) =(/NILON,NILAT,NSTPFC/)
  NPTOT=NILON*NILAT
  NMX=NLON
  NMY=NLAT
ELSE
  ISTART3(1:2) =(/1,IRDST/)
  ICOUNT3(1:2) =(/NILON,NSTPFC/)
  NPTOT=NILON
  NMX=NILON
  NMY=1
ENDIF

ALLOCATE (ZREAL3(NPTOT,NSTPFC))

IF ( VNDIMS == 2 ) THEN
  CALL NCERROR( NF90_GET_VAR(NCID,VARID,ZREAL3,ISTART3(1:2),ICOUNT3(1:2)),'READING2 '//CVAR)
ELSEIF ( VNDIMS == 3 ) THEN
  CALL NCERROR( NF90_GET_VAR(NCID,VARID,ZREAL3,ISTART3(1:3),ICOUNT3(1:3)),'READING3 '//CVAR)
ELSEIF ( VNDIMS == 4 ) THEN
  CALL NCERROR( NF90_GET_VAR(NCID,VARID,ZREAL3,ISTART4(1:4),ICOUNT4(1:4)),'READING4 '//CVAR)
ENDIF

CALL NCERROR( NF90_CLOSE(NCID),'CLOSING FILE')

DO JT=1,NSTPFC
  POUTPUT(1:NPOI,JT) = PACK(ZREAL3(1:NPTOT,JT),LMASK)
ENDDO
CALL MINMAX(CVAR,ZREAL3(:,1),NMX,NMY,LMASK,NULOUT)

WRITE(NULOUT,*) ' FORCING DATA ',TRIM(CVAR),' READ FOR ',NSTPFC,' FORCING STEPS'
IF (ALLOCATED(ZREAL3)) DEALLOCATE(ZREAL3)

RETURN

END SUBROUTINE RDFVAR


