SUBROUTINE RDRES(LOPLEFT)
USE PARKIND1  ,ONLY : JPIM     ,JPRB

#ifdef DOC

!**** *RDRES*   - Routine to read restart file

!     Purpose.
!     --------
!           Initialize model from restart file

!***  Interface.
!     ----------
!        *CALL* *RDRES

!     Explicit arguments :
!     --------------------
!         LOPLEFT: FLAG INDICATING WHETHER MORE POINTS NEED TO BE DONE

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        Bart vd Hurk KNMI

!     Modifications.
!     --------------
!        Original : 2000-07-18

#endif

USE YOMLUN1S , ONLY : NULOUT
USE YOMCT01S , ONLY : NSTART, LNF
USE YOMGC1S  , ONLY : LMASK
USE YOMDYN1S , ONLY : TSTEP
USE YOMRIP   , ONLY : RTIMTR    ,RTIMST
USE YOMDPHY  , ONLY : NLON      ,NLAT     ,NTILES    ,NLALO ,NVHILO
USE YOMGPD1S , ONLY : VDIEWSSTL ,VDINSSSTL,VDISSHFTL,VDIETL    ,VDTSKTL, &
                    & VDANDAYVT,VDANFMVT, &
		    & VDRESPBSTR,VDRESPBSTR2,VDBIOMASS_LAST

USE YOMLOG1S , ONLY : NDIMCDF

IMPLICIT NONE
LOGICAL LOPLEFT		! FLAG FOR RUNNING MORE GRID POINTS

!* Netcdf interface
INTEGER ISTART2(2),ICOUNT2(2)
INTEGER ISTART3(3),ICOUNT3(3)
INTEGER NCID,IERR,NVARID,NSTEPI,NVARTYP
INTEGER NDIMS(100),NVDIMS,NVATTS
INTEGER NDIM, NVARS, NGATTS, IRECDIM
CHARACTER*100 CNAME

REAL*8, ALLOCATABLE :: ZINPUT(:),ZINPUTT(:,:),ZINPUTV(:,:)

!* Local variables
INTEGER(KIND=JPIM) :: JTILE, JVT, IVAR
LOGICAL LOPRES


#include "netcdf.inc"

WRITE(NULOUT,*)'READING RESTART FILE'

!*         0.  Open surface climate field

NCID = NCOPN('restartin.nc', NCNOWRIT, IERR)
WRITE(NULOUT,*)'NETCDF-FILE restartin.nc OPENED ON UNIT ',NCID

!          0.5 Check dimensions and timing

CALL NCINQ(NCID, NDIM, NVARS, NGATTS, IRECDIM, IERR)


NVARID = NCVID(NCID, 'timestp', IERR)
CALL NCVGT1(NCID, NVARID, 1, NSTEPI, IERR)
IF(NSTEPI /= NSTART .AND. LNF )THEN
!IF(NSTEPI /= NSTART )THEN
  WRITE(NULOUT,*)'INITIAL STEP IN RESTART FILE INCORRECT: ',NSTART,NSTEPI
  CALL ABORT
ENDIF
NSTEPI=NSTART


RTIMTR=RTIMST+(NSTEPI+0.5)*TSTEP


!*         1.  Read coordinates

CALL SUGC1S(NCID,LOPLEFT)

!*         1.  Read surface characteristics from NetCDF file(s)

CALL SUGPD1S(NCID)

!*         2.  Read surface prognostics

CALL SUGP1S(NCID)

!*         3.  Read tile fluxes, tile Z0, assimilation per vegtype and
!              accumulated quantities (for respiration calibration)

IF(NDIMCDF == 2)THEN
  ISTART2(1)=1
  ISTART2(2)=1
  ICOUNT2(1)=NLON
  ICOUNT2(2)=NLAT
ELSE
  ISTART2(1)=1
  ICOUNT2(1)=NLON
ENDIF
ALLOCATE (ZINPUT(NLALO))


IF(NDIMCDF == 2)THEN
  ISTART3(1)=1
  ISTART3(2)=1
  ISTART3(3)=1
  ICOUNT3(1)=NLON
  ICOUNT3(2)=NLAT
  ICOUNT3(3)=NTILES
ELSE
  ISTART3(1)=1
  ISTART3(2)=1
  ICOUNT3(1)=NLON
  ICOUNT3(2)=NTILES
ENDIF
ALLOCATE (ZINPUTT(NLALO,NTILES))

!     sensible heat flux
NVARID = NCVID(NCID, 'Qh', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTT, IERR)
DO JTILE=1,NTILES
  VDISSHFTL(:,JTILE)=PACK(ZINPUTT(:,JTILE),LMASK)
ENDDO

!     latent heat flux
NVARID = NCVID(NCID, 'Evap', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTT, IERR)
DO JTILE=1,NTILES
  VDIETL(:,JTILE)=PACK(ZINPUTT(:,JTILE),LMASK)
ENDDO

!     momentum flux u
NVARID = NCVID(NCID, 'Momu', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTT, IERR)
DO JTILE=1,NTILES
  VDIEWSSTL(:,JTILE)=PACK(ZINPUTT(:,JTILE),LMASK)
ENDDO

!     momentum flux v
NVARID = NCVID(NCID, 'Momv', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTT, IERR)
DO JTILE=1,NTILES
  VDINSSSTL(:,JTILE)=PACK(ZINPUTT(:,JTILE),LMASK)
ENDDO

!     skin temperature
NVARID = NCVID(NCID, 'SkinT', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTT, IERR)
DO JTILE=1,NTILES
  VDTSKTL(:,JTILE)=PACK(ZINPUTT(:,JTILE),LMASK)
ENDDO

IF(NDIMCDF == 2)THEN
  ISTART3(1)=1
  ISTART3(2)=1
  ISTART3(3)=1
  ICOUNT3(1)=NLON
  ICOUNT3(2)=NLAT
  ICOUNT3(3)=NVHILO
ELSE
  ISTART3(1)=1
  ISTART3(2)=1
  ICOUNT3(1)=NLON
  ICOUNT3(2)=NVHILO
ENDIF
ALLOCATE (ZINPUTV(NLALO,NVHILO))

!     daily net CO2 assimilation
NVARID = NCVID(NCID, 'Anday', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTV, IERR)
DO JVT=1,NVHILO
  VDANDAYVT(:,JVT)=PACK(ZINPUTV(:,JVT),LMASK)
ENDDO

!     maximum leaf assimilation
NVARID = NCVID(NCID, 'Anfm', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTV, IERR)
DO JVT=1,NVHILO
  VDANFMVT(:,JVT)=PACK(ZINPUTV(:,JVT),LMASK)
ENDDO

!     respiration of above ground structural biomass
NVARID = NCVID(NCID, 'Rstr', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTV, IERR)
DO JVT=1,NVHILO
  VDRESPBSTR(:,JVT)=PACK(ZINPUTV(:,JVT),LMASK)
ENDDO

!     respiration of below ground structural biomass
NVARID = NCVID(NCID, 'Rstr2', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTV, IERR)
DO JVT=1,NVHILO
  VDRESPBSTR2(:,JVT)=PACK(ZINPUTV(:,JVT),LMASK)
ENDDO

!     biomass of previous day
NVARID = NCVID(NCID, 'Blast', IERR)
CALL NCVGT(NCID, NVARID, ISTART3, ICOUNT3, ZINPUTV, IERR)
DO JVT=1,NVHILO
  VDBIOMASS_LAST(:,JVT)=PACK(ZINPUTV(:,JVT),LMASK)
ENDDO

!* -- close datafile
CALL NCCLOS(NCID,IERR)

RETURN

END SUBROUTINE RDRES
