SUBROUTINE RDSUPR(NCID)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMGP1S0 , ONLY : GP0      ,TSLNU0   ,TILNU0   ,QLINU0 &
                     &,FSNNU0   ,TSNNU0   ,ASNNU0   ,RSNNU0,WSNNU0 &
                     &,TRENU0   ,WRENU0 &
                     &,TLICENU0,TLMNWNU0,TLWMLNU0,TLBOTNU0,TLSFNU0 & ! FLAKE
                     &,HLICENU0,HLMLNU0 &                             ! FLAKE 
                     &,LAINU0 , BSTRNU0, BSTR2NU0,UONU0,VONU0,TONU0,SONU0 

USE YOMDPHY  , ONLY : NCSS,NLON, NLAT, NPOI, NGPP, NLALO,NVHILO,NCSNEC

USE YOETHF   , ONLY : RHOH2O
USE YOMGC1S  , ONLY : LMASK
USE YOESOIL1S, ONLY : RDAW
USE YOMLUN1S , ONLY : NULOUT
USE YOMLOG1S , ONLY : NDIMCDF
USE YOEPHY   , ONLY : LECTESSEL
USE NETCDF
USE NETCDF_UTILS, ONLY: NCERROR
USE MPL_MODULE
#ifdef DOC

!**** *RDSUPR * - Reading netCDF file containing surface prognostic fields

!     Purpose.
!     --------
!            initialization surface prognostics

!**   Interface.
!     ----------
!        *CALL* *RDSUPR(NCID)

!        Explicit arguments :
!        --------------------
!     NCID      INT     NetCDF file ID


!        Implicit arguments :
!        --------------------


!     Method.
!     -------
!       Opens a file called 'soilinit' to read relevant fields
!       In the file, fields are assumed to be stored as 
!           FIELD(LAT,LON)
!       or as
!           FIELD(NLEVS,LAT,LON)


!     Externals.
!     ----------
!       NETCDF-utilities

!     Reference.
!     ----------

!     Author.
!     -------
!        Bart vd Hurk, KNMI

!     Modifications.
!     --------------
!        Original : 2000-07-13
!        E. Dutra 07/2014 : netcdf4 

!     ------------------------------------------------------------------

#endif
IMPLICIT NONE
#include "minmax.intfb.h"

INTEGER(KIND=JPIM),INTENT(IN) :: NCID



!* LOCAL VARIABLES

INTEGER ISTART2(2),ICOUNT2(2)
INTEGER ISTART3(3),ICOUNT3(3)
INTEGER NDIMS(100)
REAL(KIND=JPRB),ALLOCATABLE :: ZREALD(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZREAL3D(:,:)
INTEGER NIDLON,NIDLAT,IERR,NILON,NILAT,NIDLEVS,NILEVS,NVARID,NVARTYP, &
    &    NVDIMS,NVATTS,NIDVTYPES,NIVTYPES,JK,NISNLEVS
INTEGER(KIND=JPIM) :: NMX,NMY,ILEVS,JL,J,IVTYPES,JD,DIMLEN,NIMON
CHARACTER*100 CNAME
CHARACTER*20 CDUM

LOGICAL LOPRES,LRSNML,LEXTENDSOIL
INTEGER IVAR,NDIM,NGATTS,IRECDIM,NVARS,STATUS
INTEGER(KIND=JPIM)           :: NVARS2D,NVARS3D
CHARACTER(LEN=20)            :: CVARS2D(40),CVARS3D(40),CVAR

DATA ISTART2/1,1/
DATA ISTART3/1,1,1/

!* check the dimensions
CALL NCERROR (NF90_INQUIRE(NCID, nDimensions=NDIM,nVariables=NVARS),'FINDING N DIMS')

NILON=-999
NILAT=-999
NILEVS=-999
NIVTYPES=-999
NISNLEVS=-999
DO JD=1,NDIM
  CALL NCERROR(NF90_INQUIRE_DIMENSION(NCID, JD, name=CNAME, len=DIMLEN),'INQ DIMENSION')
  SELECT CASE (TRIM(CNAME))
    CASE('x','lon')
      NILON=DIMLEN
      WRITE(NULOUT,*)'DIMENSION x ',TRIM(CNAME),' FOUND WITH LEN=',NILON
   CASE('y','lat')
      NILAT=DIMLEN
      WRITE(NULOUT,*)'DIMENSION y ',TRIM(CNAME),' FOUND WITH LEN=',NILAT
   CASE('nlevs')
      NILEVS=DIMLEN
      WRITE(NULOUT,*)'DIMENSION nlevs ',TRIM(CNAME),' FOUND WITH LEN=',NILEVS
   CASE('vtype')
      NIVTYPES=DIMLEN
      WRITE(NULOUT,*)'DIMENSION vtype ',TRIM(CNAME),' FOUND WITH LEN=',NIVTYPES
   CASE('nlevsn')
      NISNLEVS=DIMLEN
      WRITE(NULOUT,*)'DIMENSION vtype ',TRIM(CNAME),' FOUND WITH LEN=',NISNLEVS
  END SELECT
END DO
IF (NDIMCDF == 1 ) THEN
  NILAT = 0
ENDIF
LRSNML=.TRUE.
LEXTENDSOIL=.FALSE.
IF(NILON /= NLON .OR. NILAT /= NLAT) THEN
  WRITE(NULOUT,*)'NLON OR NLAT NOT SPECIFIED CORRECTLY:'
  WRITE(NULOUT,*)'NLON IN NAMELIST AND SOILINIT: ',NLON,NILON
  WRITE(NULOUT,*)'NLAT IN NAMELIST AND SOILINIT: ',NLAT,NILAT
  CALL ABOR1('RDSUPR:')
ENDIF
IF(NILEVS /= NCSS) THEN
  WRITE(NULOUT,*)'NLEVS NOT SPECIFIED CORRECTLY:'
  WRITE(NULOUT,*)'NLEVS IN CODE AND SOILINIT: ',NCSS,NILEVS
  IF(NILEVS == 4 .AND. NCSS > 4) THEN
     WRITE(NULOUT,*)'Starting from the 4 layer scheme'
     WRITE(NULOUT,*)'All levels below layer 4 set equal to layer 4, this should be just a temporary fix'
     LEXTENDSOIL=.TRUE.
  ELSE
    CALL ABOR1('RDSUPR:')
  ENDIF
ENDIF
IF (NIVTYPES == -999 ) THEN
  NIVTYPES=NVHILO
  WRITE(NULOUT,*)'vtype DIMENSION NOT FOUND IN NETCDF FILE!!! SET==NVHILO'
ENDIF
IF(NIVTYPES /= NVHILO) THEN
  WRITE(NULOUT,*)'NVHILO NOT SPECIFIED CORRECTLY:'
  WRITE(NULOUT,*)'NVHILO IN CODE AND PROGINIT: ',NVHILO,NIVTYPES
  CALL ABOR1('RDSUPR:')
ENDIF
IF (NISNLEVS == -999  ) THEN
  IF  (NCSNEC>1) THEN
    WRITE(NULOUT,*)'Input does not contain snow multi-layer'
    WRITE(NULOUT,*) 'Input initialized from total fields'
    LRSNML=.FALSE.
  ELSE
    LRSNML=.FALSE.
  ENDIF
ENDIF
IF(NISNLEVS /= -999 .AND. (NCSNEC /= NISNLEVS) ) THEN
  WRITE(NULOUT,*)'Input contains:',NISNLEVS,' snow layer, while running with:',NCSNEC,' layers'
  WRITE(NULOUT,*) 'Input initialized from total fields'
  LRSNML=.FALSE.
ENDIF




!* Note the reversal of the order of array storage

IF(NDIMCDF == 2)THEN
  ICOUNT2(1)=NILON
  ICOUNT2(2)=NILAT

  ICOUNT3(1)=NILON
  ICOUNT3(2)=NILAT
  ICOUNT3(3)=NILEVS
  NMX=NLON
  NMY=NLAT
ELSE
  ICOUNT2(1)=NILON

  ICOUNT3(1)=NILON
  ICOUNT3(2)=NILEVS
  NMX=NLALO
  NMY=1
ENDIF


!* Read surface prognostic values

! 3D FIELDS
ALLOCATE (ZREAL3D(NLALO,NCSS))
NVARS3D=3
 CVARS3D(1:NVARS3D)=(/'SoilTemp ','icetemp  ','SoilMoist'   /)
DO IVAR=1,NVARS3D
  CVAR=TRIM(CVARS3D(IVAR))
  STATUS = NF90_INQ_VARID(NCID, CVAR, NVARID)
  IF ( STATUS /= 0 ) THEN
    WRITE(NULOUT,'(A)') CVAR//' NOT PRESENT IN INPUT FILE'
  ELSE
    CALL NCERROR( NF90_GET_VAR(NCID,NVARID,ZREAL3D,ISTART3,ICOUNT3),'READING '//CVAR)
  ENDIF
  IF (LEXTENDSOIL) THEN
    DO ILEVS=NILEVS+1,NCSS
      ZREAL3D(:,ILEVS) = ZREAL3D(:,NILEVS)
    ENDDO
  ENDIF
  DO ILEVS=1,NCSS
    SELECT CASE(CVAR)
      CASE('SoilTemp')
        IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
        TSLNU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
      CASE('SoilMoist')
        IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
        CALL NCERROR( NF90_GET_ATT(NCID, NVARID, 'units', CNAME),'Getting SM units')
        IF(CNAME(1:6) /= 'm3 m-3'.AND. CNAME(1:5) /= 'm3/m3') THEN
          ZREAL3D(:,ILEVS)=ZREAL3D(:,ILEVS)/(RHOH2O*RDAW(ILEVS))
          WRITE(NULOUT,*)'RDSUPR: INPUT SOIL MOISTURE CONVERTED TO M3 M-3'
        ENDIF
        QLINU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
      CASE('icetemp')
        IF ( STATUS /= 0 ) ZREAL3D(:,ILEVS) = 273.
        TILNU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
      CASE DEFAULT
        WRITE(NULOUT,*) CVAR, ' Not defined in RDCLIM'
        CALL ABOR1('RDSUPR:')
    END SELECT
    WRITE(CDUM,'(A8,I2.2)')TRIM(CVAR),ILEVS
    CALL MINMAX(CDUM,ZREAL3D(:,ILEVS),NMX,NMY,LMASK,NULOUT)
  ENDDO
  
ENDDO


ALLOCATE (ZREALD(NLALO))
NVARS2D=14
 CVARS2D(1:NVARS2D)=(/'SWE      ','snowdens ','SAlbedo  ','SnowT    ',&
                      'CanopInt ','AvgSurfT ','TLICE    ','TLMNW    ',&
                      'TLWML    ','TLBOT    ','TLSF     ','HLICE    ',&
                      'HLML     ','slw      '/)

DO IVAR=1,NVARS2D
  CVAR=TRIM(CVARS2D(IVAR))
  STATUS = NF90_INQ_VARID(NCID, CVAR, NVARID)
  IF ( STATUS /= 0 ) THEN
    WRITE(NULOUT,'(A)') CVAR//' NOT PRESENT IN INPUT FILE'
  ELSE
    CALL NCERROR( NF90_GET_VAR(NCID,NVARID,ZREALD,ISTART2,ICOUNT2),'READING '//CVAR)
  ENDIF
  SELECT CASE(CVAR)
    CASE('SWE')
      IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
      FSNNU0(:,1)=PACK(ZREALD,LMASK)
    CASE('snowdens')
      IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
      RSNNU0(:,1)=PACK(ZREALD,LMASK)
    CASE('SAlbedo')
      IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
      ASNNU0=PACK(ZREALD,LMASK)
    CASE('SnowT')
      IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
      TSNNU0(:,1)=PACK(ZREALD,LMASK)
    CASE('CanopInt')
      IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
      WRENU0=PACK(ZREALD,LMASK)
    CASE('AvgSurfT')
      IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
      TRENU0=PACK(ZREALD,LMASK)
    CASE('TLICE')
      IF ( STATUS /= 0 ) ZREALD(:) = 273._JPRB
      TLICENU0=PACK(ZREALD,LMASK)
    CASE('TLMNW')
      IF ( STATUS /= 0 ) ZREALD(:) = 288._JPRB
      TLMNWNU0=PACK(ZREALD,LMASK)
    CASE('TLWML')
      IF ( STATUS /= 0 ) ZREALD(:) = 288._JPRB
      TLWMLNU0=PACK(ZREALD,LMASK)
    CASE('TLBOT')
      IF ( STATUS /= 0 ) ZREALD(:) = 288._JPRB
      TLBOTNU0=PACK(ZREALD,LMASK)
    CASE('TLSF')
      IF ( STATUS /= 0 ) ZREALD(:) = 0.65_JPRB
      TLSFNU0=PACK(ZREALD,LMASK) 
    CASE('HLICE')
      IF ( STATUS /= 0 ) ZREALD(:) = 0._JPRB
      HLICENU0=PACK(ZREALD,LMASK)
    CASE('HLML')
      IF ( STATUS /= 0 ) ZREALD(:) = 10._JPRB
      HLMLNU0=PACK(ZREALD,LMASK)
    CASE('slw')
      IF ( STATUS /= 0 ) ZREALD(:) = 0._JPRB
      WRITE(NULOUT,'(A)') CVAR//' Set to Zero'
      WSNNU0(:,1)=PACK(ZREALD,LMASK)
    CASE DEFAULT
      WRITE(NULOUT,*) CVAR, ' Not defined in RDCLIM'
      CALL ABOR1('RDSUPR:')
  END SELECT
  WRITE(CDUM,*)TRIM(CVAR)
  CALL MINMAX(CDUM,ZREALD,NMX,NMY,LMASK,NULOUT)
ENDDO

!! Read snow ML fields
IF ( LRSNML ) THEN
  IF(NDIMCDF == 2)THEN
    ICOUNT2(1)=NILON
    ICOUNT2(2)=NILAT

    ICOUNT3(1)=NILON
    ICOUNT3(2)=NILAT
    ICOUNT3(3)=NCSNEC
    NMX=NLON
    NMY=NLAT
  ELSE
    ICOUNT2(1)=NILON

    ICOUNT3(1)=NILON
    ICOUNT3(2)=NCSNEC
    NMX=NLALO
    NMY=1
  ENDIF
  DEALLOCATE (ZREAL3D)
  ALLOCATE (ZREAL3D(NLALO,NCSNEC))
  
  NVARS3D=4
  CVARS3D(1:NVARS3D)=(/'SWEML     ','snowdensML','SnowTML   ','slwML     '/)
  DO IVAR=1,NVARS3D
    CVAR=TRIM(CVARS3D(IVAR))
    STATUS = NF90_INQ_VARID(NCID, CVAR, NVARID)
    IF ( STATUS /= 0 ) THEN
      WRITE(NULOUT,'(A)') CVAR//' NOT PRESENT IN INPUT FILE'
    ELSE
      CALL NCERROR( NF90_GET_VAR(NCID,NVARID,ZREAL3D,ISTART3,ICOUNT3),'READING '//CVAR)
    ENDIF
    DO ILEVS=1,NCSNEC
      SELECT CASE(CVAR)
        CASE('SWEML')
          IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
          FSNNU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
        CASE('snowdensML')
          IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
          RSNNU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
        CASE('SnowTML')
          IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
          TSNNU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
        CASE('slwML')
          IF ( STATUS /= 0 ) CALL ABOR1('RDSUPR:')
          WSNNU0(:,ILEVS)=PACK(ZREAL3D(:,ILEVS),LMASK)
        CASE DEFAULT
          WRITE(NULOUT,*) CVAR, ' Not defined in RDCLIM'
          CALL ABOR1('RDSUPR:')
      END SELECT
      WRITE(CDUM,'(A8,I2.2)')TRIM(CVAR),ILEVS
      CALL MINMAX(CDUM,ZREAL3D(:,ILEVS),NMX,NMY,LMASK,NULOUT)
    ENDDO
  ENDDO
  
ELSE
  ! Fill up snow 3D fields 
  WRITE(NULOUT,'(A)') 'Filling Snow 3D fields with 2D information'
  DO JK=2,NCSNEC
    FSNNU0(:,JK) = 0.0_JPRB
    WSNNU0(:,JK) = 0.0_JPRB
    RSNNU0(:,JK) = RSNNU0(:,1)
    TSNNU0(:,JK) = TSNNU0(:,1)
  ENDDO
ENDIF 
    



!* < CTESSEL
!    IF (LECTESSEL) THEN 
DEALLOCATE (ZREAL3D)
ALLOCATE (ZREAL3D(NLALO,NVHILO))

IF(NDIMCDF == 2)THEN
  ICOUNT3(1)=NILON
  ICOUNT3(2)=NILAT
  ICOUNT3(3)=NIVTYPES
ELSE
  ICOUNT3(1)=NILON
  ICOUNT3(2)=NIVTYPES
ENDIF

NVARS3D=3
 CVARS3D(1:NVARS3D)=(/'laivt    ','Biomstr  ','Biomstr2 '   /)
DO IVAR=1,NVARS3D
  CVAR=TRIM(CVARS3D(IVAR))
  STATUS = NF90_INQ_VARID(NCID, CVAR, NVARID)
  IF ( STATUS /= 0 ) THEN
    WRITE(NULOUT,'(A)') CVAR//' NOT PRESENT IN INPUT FILE'
  ELSE
    CALL NCERROR( NF90_GET_VAR(NCID,NVARID,ZREAL3D,ISTART3,ICOUNT3),'READING '//CVAR)
  ENDIF
  DO IVTYPES=1,NVHILO
    SELECT CASE(CVAR)
      CASE('laivt')
        IF ( STATUS /= 0 ) ZREAL3D(:,IVTYPES) = 0.3_JPRB
        LAINU0(:,IVTYPES)=PACK(ZREAL3D(:,IVTYPES),LMASK)
      CASE('Biomstr')
        IF ( STATUS /= 0 ) ZREAL3D(:,IVTYPES) = 0._JPRB
        BSTRNU0(:,IVTYPES)=PACK(ZREAL3D(:,IVTYPES),LMASK)
      CASE('Biomstr2')
        IF ( STATUS /= 0 ) ZREAL3D(:,IVTYPES) = 0._JPRB
        BSTR2NU0(:,IVTYPES)=PACK(ZREAL3D(:,IVTYPES),LMASK)
      CASE DEFAULT
        WRITE(NULOUT,*) CVAR, ' Not defined in RDCLIM'
        CALL ABOR1('RDSUPR:')
    END SELECT
    WRITE(CDUM,'(A8,I2.2)')TRIM(CVAR),ILEVS
    CALL MINMAX(CDUM,ZREAL3D(:,IVTYPES),NMX,NMY,LMASK,NULOUT)
  ENDDO
ENDDO

!* Set ocean data to zero - this should be removed at some point ! 
UONU0(:,:) = 0._JPRB
VONU0(:,:) = 0._JPRB
TONU0(:,:) = 0._JPRB
SONU0(:,:) = 0._JPRB

!* Check final values

DO JL=1,NPOI
  DO J=1,NGPP
    IF (GP0(JL,J) == -999.) THEN
      WRITE(NULOUT,*) 'ERROR SOIL INITIALISATION', JL,J
      CALL ABOR1('RDSUPR:')
    ENDIF
  ENDDO
ENDDO
DEALLOCATE (ZREAL3D)
DEALLOCATE (ZREALD)
END SUBROUTINE RDSUPR
