SUBROUTINE SUGPD1S(NCID)
USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMLUN1S , ONLY : NULNAM   ,NULOUT
USE YOMGPD1S , ONLY : GPD       ,&
            &VFZ0F    ,VFALBF   ,VFITM    ,VFZ0H    ,&
            &VFALUVP  ,VFALUVD  ,VFALNIP  ,VFALNID  ,&
            &VFCVL    ,VFCVH    ,VFTVL    ,VFTVH    ,&
            &VFCOVVT  ,VFR0VT,VFRSML   ,VFRSMH    ,&
	    &VFLAIL   ,VFLAIH   ,VFSST    ,&
            &VFCI     ,VFSOTY   ,VFSDOR   ,VFGEO    ,&
            &VDLSP    ,VDCP     ,VDSF     ,VDSSHF   ,VDSLHF   ,&
            &VDSSR    ,VDSTR    ,VDEWSS   ,VDNSSS   ,VDE      ,&
            &VDRO     ,VDALB    ,VDIEWSS  ,VDINSSS  ,VDISSHF  ,&
            &VDIE     ,VDCSF    ,VDLSSF   ,VDZ0F    ,VDZ0H    ,&
            &VDSSRD   ,VDSTRD   ,VDIEWSSTL,VDINSSSTL,VDISSHFTL,&
            &VDIETL   ,VDTSKTL  ,VDROS    ,VDMLT,&
            &VFLDEPTH ,VFCLAKE  ,&!FLAKE 
            &VFZO     ,VFHO     ,VFHO_INV ,VFDO     ,VFOCDEPTH,&   !KPP
            &VFADVT   ,VFADVS   ,VFTRI0   ,VFTRI1   ,VFSWDK_SAVE,& !KPP
            &VDANDAYVT,VDANFMVT ,&
	    &VDRESPBSTR,VDRESPBSTR2,VDBIOMASS_LAST,VDBLOSSVT,VDBGAINVT,&
	    &VFSOALUVP,VFSOALUVD,VFSOALNIP,VFSOALNID

USE PTRGPD1S , ONLY :  MVFZ0F   ,MVFALBF  ,MVFITM   ,&
            &MVFALUVP ,MVFALUVD ,MVFALNIP ,MVFALNID ,&
            &MVFZ0H   ,MVFCVL   ,MVFCVH   ,MVFTVL   ,MVFTVH   ,&
	    &MVFLAIL  ,MVFLAIH  ,&
            &MVFCOVVT ,MVFR0VT,MVFRSML  ,MVFRSMH,&

            &MVFSST   ,MVFCI    ,MVFSOTY  ,MVFSDOR  ,MVFGEO   ,&
            &MVDLSP   ,MVDCP    ,MVDSF    ,MVDSSHF  ,MVDSLHF  ,&
            &MVDSSR   ,MVDSTR   ,MVDEWSS  ,MVDNSSS  ,MVDE     ,&
            &MVDRO    ,MVDALB   ,MVDIEWSS ,MVDINSSS ,MVDISSHF ,&
            &MVDIE    ,MVDCSF   ,MVDLSSF  ,MVDZ0F   ,MVDZ0H   ,&
            &MVDSSRD  ,MVDSTRD  ,MVDIEWSSTL,MVDINSSSTL,MVDISSHFTL,&
            &MVDIETL  ,MVDTSKTL ,MVDROS   ,MVDMLT,&
            &MVFLDEPTH,MVFCLAKE ,&!FLAKE
            &MVFZO    ,MVFHO    ,MVFHO_INV,MVFDO    ,MVFOCDEPTH,& !KPP
            &MVFADVT  ,MVFADVS  ,MVFTRI0  ,MVFTRI1  ,MVFSWDK_SAVE,& !KPP
            &MVDANDAYVT,MVDANFMVT,&
            &MVDRESPBSTR,MVDRESPBSTR2,MVDBIOMASS_LAST,MVDBLOSSVT,MVDBGAINVT,&
            &MVFSOALUVP,MVFSOALUVD,MVFSOALNIP,MVFSOALNID   


USE YOMCC1S  , ONLY : GPCC,VCALB,VCLAIL,VCLAIH
USE YOMLOG1S , ONLY : CFSURF
USE YOMDPHY  , ONLY : NVSF    ,NVPD     ,NGCC     ,NPOI     ,&
                     &NTILES  ,NVHILO  ,NVDIAG   ,ISLAB    ,&
                     &NCOM    ,NVELO    ,NSCLRO    ,&    !KPP
                     &NCLIMDAT
USE YOEPHY   , ONLY : LEOCML, LECLIM10D

#ifdef DOC

!**** *SUGPD1S*   - Routine to initialize soil and vegetation properties

!     Purpose.
!     --------
!           Initialize surface properties

!***  Interface.
!     ----------
!        *CALL* *SUGPD1S(NCID)

!     Explicit arguments :
!     --------------------
!        NCID : NETCDF INPUT FILE UNIT

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by SUINIF1S.

!     Reference.
!     ----------
!        ECMWF Research Department documentation 
!        of the one column surface model

!     Author.
!     -------
!        Jean-Francois Mahfouf and Pedro Viterbo  *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-03-21
!        Bart vd HURK (KNMI) Multi-column input from netCDF
!        Gianpaolo Balsamo (ECMWF) add grib interface
!        Y. Takaya (ECMWF) add ocean mixed layer variables
!        E. Dutra (U.Lisbon) add Flake variables
!        G. Balsamo (ECMWF) add LAI

#endif
IMPLICIT NONE
#include "namgpd1s.h"
#include "rdclim.intfb.h"

INTEGER NCID

INTEGER(KIND=JPIM) :: IVSF,IVCC,IVDIAG,MVCALB,MVCLAIL,MVCLAIH
INTEGER(KIND=JPIM) :: IST,IEND,J,JMO,KULOUT

!*        ALLOCATE TOTAL SIZE and pointer stuff.

KULOUT=NULOUT
   WRITE(UNIT=KULOUT,FMT=*) 'NVHILO=',NVHILO
IF(ISLAB > 1)THEN
  DEALLOCATE (GPD)
ENDIF
ALLOCATE (GPD(NPOI,NVPD))
MVFZ0F=1
MVFALBF=MVFZ0F+1
MVFALUVP=MVFALBF+1
MVFALUVD=MVFALUVP+1
MVFALNIP=MVFALUVD+1
MVFALNID=MVFALNIP+1
MVFSOALUVP=MVFALNID+1
MVFSOALUVD=MVFSOALUVP+1
MVFSOALNIP=MVFSOALUVD+1
MVFSOALNID=MVFSOALNIP+1 
MVFITM=MVFSOALNID+1
MVFGEO=MVFITM+1
MVFZ0H=MVFGEO+1
MVFCVL=MVFZ0H+1
MVFCVH=MVFCVL+1
MVFTVL=MVFCVH+1
MVFTVH=MVFTVL+1
MVFLAIL=MVFTVH+1
MVFLAIH=MVFLAIL+1
MVFRSML=MVFLAIH+1
MVFRSMH=MVFRSML+1

MVFSST=MVFRSMH+1
MVFCI=MVFSST+1
MVFSOTY=MVFCI+1
MVFSDOR=MVFSOTY+1
MVFR0VT=MVFSDOR+1


MVFCLAKE=MVFR0VT+NVHILO    ! FLAKE
!MVFLDEPTH=MVFR0VT+NVHILO  ! FLAKE 
MVFLDEPTH=MVFCLAKE+1  ! FLAKE (same as above)


MVFZO = MVFLDEPTH+1              !KPP
MVFHO = MVFZO+NCOM+1             !KPP
MVFHO_INV = MVFHO+NCOM+1         !KPP
MVFDO = MVFHO_INV+NCOM+1         !KPP
MVFADVT = MVFDO+NCOM+1           !KPP
MVFADVS = MVFADVT+NCOM           !KPP
MVFTRI0 = MVFADVS+NCOM           !KPP
MVFTRI1 = MVFTRI0 + NCOM+1       !KPP 
MVFSWDK_SAVE = MVFTRI1 + NCOM+1  !KPP
MVFOCDEPTH = MVFSWDK_SAVE + NCOM+1 !KPP
IVSF=MVFOCDEPTH                    ! KPP

MVDLSP=MVFOCDEPTH+1   
MVDCP=MVDLSP+1
MVDSF=MVDCP+1
MVDSSHF=MVDSF+1
MVDSLHF=MVDSSHF+1
MVDSSR=MVDSLHF+1
MVDSTR=MVDSSR+1
MVDEWSS=MVDSTR+1
MVDNSSS=MVDEWSS+1
MVDE=MVDNSSS+1
MVDRO=MVDE+1
MVDROS=MVDRO+1
MVDMLT=MVDROS+1
MVDALB=MVDMLT+1
MVDIEWSS=MVDALB+1
MVDINSSS=MVDIEWSS+1
MVDISSHF=MVDINSSS+1
MVDIE=MVDISSHF+1
MVDCSF=MVDIE+1
MVDLSSF=MVDCSF+1
MVDZ0F=MVDLSSF+1
MVDZ0H=MVDZ0F+1
MVDSSRD=MVDZ0H+1
MVDSTRD=MVDSSRD+1
MVDIEWSSTL=MVDSTRD+1
MVDINSSSTL=MVDIEWSSTL+NTILES
MVDISSHFTL=MVDINSSSTL+NTILES
MVDIETL=MVDISSHFTL+NTILES
MVDTSKTL=MVDIETL+NTILES

MVDANDAYVT=MVDTSKTL+NTILES
MVDANFMVT=MVDANDAYVT+NVHILO
MVDRESPBSTR=MVDANFMVT+NVHILO
MVDRESPBSTR2=MVDRESPBSTR+NVHILO
MVDBIOMASS_LAST=MVDRESPBSTR2+NVHILO
MVDBLOSSVT=MVDBIOMASS_LAST+NVHILO
MVDBGAINVT=MVDBLOSSVT+NVHILO
IVDIAG=MVDBGAINVT+NVHILO-(IVSF)

   WRITE(UNIT=KULOUT,FMT=*) 'pointer dimension finish'

VFZ0F => GPD(:,MVFZ0F)
VFALBF => GPD(:,MVFALBF)
VFALUVP => GPD(:,MVFALUVP)
VFALUVD => GPD(:,MVFALUVD)
VFALNIP => GPD(:,MVFALNIP)
VFALNID => GPD(:,MVFALNID)
VFSOALUVP => GPD(:,MVFSOALUVP)
VFSOALUVD => GPD(:,MVFSOALUVD)
VFSOALNIP => GPD(:,MVFSOALNIP)
VFSOALNID => GPD(:,MVFSOALNID)
VFITM => GPD(:,MVFITM)
VFGEO => GPD(:,MVFGEO)
VFZ0H => GPD(:,MVFZ0H)
VFCVL => GPD(:,MVFCVL)
VFCVH => GPD(:,MVFCVH)
VFTVL => GPD(:,MVFTVL)
VFTVH => GPD(:,MVFTVH)
VFLAIL => GPD(:,MVFLAIL)
VFLAIH => GPD(:,MVFLAIH)

VFRSML=> GPD(:,MVFRSML)
VFRSMH=> GPD(:,MVFRSMH)


VFSST => GPD(:,MVFSST)
VFCI => GPD(:,MVFCI)
VFSOTY => GPD(:,MVFSOTY)
VFSDOR => GPD(:,MVFSDOR)

VFR0VT => GPD(:,MVFR0VT:MVFR0VT+NVHILO-1)


VDLSP => GPD(:,MVDLSP)
VDCP => GPD(:,MVDCP)
VDSF => GPD(:,MVDSF)
VDSSHF => GPD(:,MVDSSHF)
VDSLHF => GPD(:,MVDSLHF)
VDSSR => GPD(:,MVDSSR)
VDSTR => GPD(:,MVDSTR)
VDEWSS => GPD(:,MVDEWSS)
VDNSSS => GPD(:,MVDNSSS)
VDE => GPD(:,MVDE)
VDRO => GPD(:,MVDRO)
VDROS => GPD(:,MVDROS)
VDMLT => GPD(:,MVDMLT)
VDALB => GPD(:,MVDALB)
VDIEWSS => GPD(:,MVDIEWSS)
VDINSSS => GPD(:,MVDINSSS)
VDISSHF => GPD(:,MVDISSHF)
VDIE => GPD(:,MVDIE)
VDCSF => GPD(:,MVDCSF)
VDLSSF => GPD(:,MVDLSSF)
VDZ0F => GPD(:,MVDZ0F)
VDZ0H => GPD(:,MVDZ0H)
VDSSRD => GPD(:,MVDSSRD)
VDSTRD => GPD(:,MVDSTRD)
VDIEWSSTL => GPD(:,MVDIEWSSTL:MVDIEWSSTL+NTILES-1)
VDINSSSTL => GPD(:,MVDINSSSTL:MVDINSSSTL+NTILES-1)
VDISSHFTL => GPD(:,MVDISSHFTL:MVDISSHFTL+NTILES-1)
VDIETL => GPD(:,MVDIETL:MVDIETL+NTILES-1)
VDTSKTL => GPD(:,MVDTSKTL:MVDTSKTL+NTILES-1)

VFLDEPTH => GPD(:,MVFLDEPTH) !FLAKE  
VFCLAKE => GPD(:,MVFCLAKE)   !FLAKE 

VFZO => GPD(:,MVFZO:MVFZO+(NCOM+1)-1)   !KPP
VFHO => GPD(:,MVFHO:MVFHO+(NCOM+1)-1)   !KPP
VFHO_INV => GPD(:,MVFHO_INV:MVFHO_INV+(NCOM+1)-1) !KPP
VFDO => GPD(:,MVFDO:MVFDO+(NCOM+1)-1)   !KPP
VFADVT => GPD(:,MVFADVT:MVFADVT+NCOM-1) !KPP
VFADVS => GPD(:,MVFADVS:MVFADVS+NCOM-1) !KPP
VFTRI0 => GPD(:,MVFTRI0:MVFTRI0+(NCOM+1)-1) !KPP
VFTRI1 => GPD(:,MVFTRI1:MVFTRI1+(NCOM+1)-1) !KPP
VFSWDK_SAVE => GPD(:,MVFSWDK_SAVE:MVFSWDK_SAVE+(NCOM+1)-1) !KPP
VFOCDEPTH => GPD(:,MVFOCDEPTH) !KPP

VDANDAYVT => GPD(:,MVDANDAYVT:MVDANDAYVT+NVHILO-1)
VDANFMVT => GPD(:,MVDANFMVT:MVDANFMVT+NVHILO-1)
VDRESPBSTR => GPD(:,MVDRESPBSTR:MVDRESPBSTR+NVHILO-1)
VDRESPBSTR2 => GPD(:,MVDRESPBSTR2:MVDRESPBSTR2+NVHILO-1)
VDBIOMASS_LAST => GPD(:,MVDBIOMASS_LAST:MVDBIOMASS_LAST+NVHILO-1)
VDBLOSSVT => GPD(:,MVDBLOSSVT:MVDBLOSSVT+NVHILO-1)
VDBGAINVT => GPD(:,MVDBGAINVT:MVDBGAINVT+NVHILO-1)

  WRITE(*,*) 'pointer allocation finish'

!*        ALLOCATE TOTAL SIZE and pointer stuff for seasonal array.

IF(ISLAB > 1)THEN
  DEALLOCATE (GPCC)
ENDIF

ALLOCATE (GPCC(NPOI,NCLIMDAT,NGCC))


MVCALB=1
MVCLAIL=MVCALB+1
MVCLAIH=MVCLAIL+1
IVCC=MVCLAIH
VCALB => GPCC(:,:,MVCALB)
VCLAIL => GPCC(:,:,MVCLAIL)
VCLAIH => GPCC(:,:,MVCLAIH)

  WRITE(*,*) 'climate pointer allocation finish'
! final checking

IF (IVSF /= NVSF) THEN
  WRITE(*,*) ' SUGPD1S: ARRAY GPD'
  WRITE(*,*) ' INCONSISTENCY IN THE SPACE FOR THE CLIMATE'
  WRITE(*,*) ' PART OF GENERAL ARRAY'
  WRITE(*,*) ' NVSF=',NVSF,'IVSF=',IVSF
  WRITE(*,*) ' CHANGE THE SIZE OF NVSF'
  CALL ABORT
ENDIF

IF (IVDIAG /= NVDIAG) THEN
  WRITE(*,*) ' SUGPD1S: ARRAY GPD'
  WRITE(*,*) ' INCONSISTENCY IN THE SPACE FOR THE DIAGNOSTICS'
  WRITE(*,*) ' PART OF GENERAL ARRAY'
  WRITE(*,*) ' CHANGE THE SIZE OF N VDIAG'
  WRITE(*,*) ' NVDIAG=',NVDIAG,'IVDIAG=',IVDIAG
  WRITE(*,*) ' IVDIAG,NVDIAG=',IVDIAG,NVDIAG
  CALL ABORT
ENDIF

IF (IVCC /= NGCC) THEN
  WRITE(*,*) ' SUGPD1S: ARRAY GPCC'
  WRITE(*,*) ' INCONSISTENCY IN THE SPACE FOR THE SEASONALLY'
  WRITE(*,*) ' VARYING ARRAY'
  WRITE(*,*) ' NGCC=',NGCC,'IVCC=',IVCC
  WRITE(*,*) ' CHANGE THE SIZE OF NGCC'
  CALL ABORT
ENDIF


!*         1.  Initialisation to absurd values

IST=1
IEND=NPOI
GPD(IST:IEND,1:NVSF)=-999.


GPCC(IST:IEND,NCLIMDAT,1:NGCC)=-999.


IF ( CFSURF == 'netcdf' ) THEN

!*         1.  Read surface characteristics from NetCDF file(s)
  WRITE(*,*) 'Entring RDCLIM'
  CALL RDCLIM(NCID)
  WRITE(*,*) 'RDCLIM is over'

ELSE

!*       2.    Modifies default values.

  VGEO=0. ! Default value | VGEO is not in namelist namgpd1s 
  

!        2.1   Read namelist

  REWIND(NULNAM)
  READ(NULNAM,NAMGPD1S)

  VCALB(1,:)=VALBF(:)
  VCLAIL(1,:)=VLAIL(:)
  VCLAIH(1,:)=VLAIH(:)
  VFRSML=VRSML
  VFRSMH=VRSMH

  VGEO=0.
  VFZ0F=VZ0F
  VFITM=VITM
  VFGEO=VGEO
  VFZ0H=VZ0H
  VFCVL=VCVL
  VFCVH=VCVH
  VFTVL=VTVL
  VFTVH=VTVH
  VFSOTY=VSOTY
  VFSDOR=VSDOR
  VFSST=VSST
  VFCI=VCI
  VFLDEPTH=VLDEPTH  !FLAKE 
  VFCLAKE=VCLAKE    !FLAKE 

  VFR0VT(:,1)=VR0L   
  VFR0VT(:,2)=VR0H


! Mock value of albedo for checking
  VFALBF=SUM(VCALB,DIM=2)/NCLIMDAT
  VFALUVP=VFALBF
  VFALUVD=VFALBF
  VFALNIP=VFALBF
  VFALNID=VFALBF
  VFSOALUVP=VFALBF
  VFSOALUVD=VFALBF
  VFSOALNIP=VFALBF
  VFSOALNID=VFALBF
  VFLAIL=SUM(VCLAIL,DIM=2)/NCLIMDAT
  VFLAIH=SUM(VCLAIH,DIM=2)/NCLIMDAT

  IF (.NOT. LEOCML) THEN
    VFZO = 0.0_JPRB
    VFHO = 0.0_JPRB
    VFHO_INV = 0.0_JPRB
    VFDO = 0.0_JPRB
    VFADVT = 0.0_JPRB
    VFADVS = 0.0_JPRB
    VFTRI0 = 0.0_JPRB
    VFTRI1 = 0.0_JPRB
    VFSWDK_SAVE = 0.0_JPRB
    VFOCDEPTH = 0.0_JPRB
    PRINT *,"OCEAN DIAGNOSTIC INIT to 0"
  ENDIF

!*       3.    Print final values.

  DO J=1,NVSF
    IF (GPD(1,J) == -999.) THEN
      WRITE(*,*) 'PB WITH SURFACE PROPERTIES ', J,"VARIABLE"
      CALL ABORT
    ENDIF
  ENDDO
  DO J=1,NGCC
    do jmo=1,NCLIMDAT
      IF (GPCC(1,jmo,J) == -999.) THEN
        WRITE(*,*) 'PB WITH SURFACE SEASONAL PROPERTIES '
        CALL ABORT
      ENDIF
    enddo
  ENDDO
  WRITE(UNIT=KULOUT,FMT='('' MODULE YOMGPD1S '')')
  WRITE(UNIT=KULOUT,FMT=*) ' VFZ0F   =  ',VFZ0F
  WRITE(UNIT=KULOUT,FMT=*) ' VFITM   =  ',VFITM
  WRITE(UNIT=KULOUT,FMT=*) ' VFGEO   =  ',VFGEO
  WRITE(UNIT=KULOUT,FMT=*) ' VFZ0H  =  ',VFZ0H
  WRITE(UNIT=KULOUT,FMT=*) ' VFCVL  =  ',VFCVL
  WRITE(UNIT=KULOUT,FMT=*) ' VFCVH  =  ',VFCVH
  WRITE(UNIT=KULOUT,FMT=*) ' VFTVL  =  ',VFTVL
  WRITE(UNIT=KULOUT,FMT=*) ' VFTVH  =  ',VFTVH
  WRITE(UNIT=KULOUT,FMT=*) ' VFSST  =  ',VFSST
  WRITE(UNIT=KULOUT,FMT=*) ' VFCI   =  ',VFCI
  WRITE(UNIT=KULOUT,FMT=*) ' VFSOTY =  ',VFSOTY
  WRITE(UNIT=KULOUT,FMT=*) ' VFSDOR =  ',VFSDOR
  WRITE(UNIT=KULOUT,FMT=*) 'VFLDEPTH=  ',VFLDEPTH    !FLAKE 
  WRITE(UNIT=KULOUT,FMT=*) 'VFCLAKE=  ',VFCLAKE      !FLAKE 
  WRITE(UNIT=KULOUT,FMT=*) ' VFRSML =  ',VFRSML
  WRITE(UNIT=KULOUT,FMT=*) ' VFRSMH=   ',VFRSMH
  WRITE(UNIT=KULOUT,FMT=*) ' VFR0H=   ',VR0H
  WRITE(UNIT=KULOUT,FMT=*) ' VFR0L=   ',VR0L

  WRITE(UNIT=KULOUT,FMT='('' MODULE YOMGCC1S '')')
  WRITE(UNIT=KULOUT,FMT=*) ' VCALB  =  ',VCALB
  WRITE(UNIT=KULOUT,FMT=*) ' VCLAIL =  ',VCLAIL
  WRITE(UNIT=KULOUT,FMT=*) ' VCLAIH =  ',VCLAIH

ENDIF

RETURN
END SUBROUTINE SUGPD1S
