SUBROUTINE SUINIF1S(LOPLEFT)
USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMCT01S , ONLY : NSTART
USE YOMDPHY  , ONLY : NPOI     ,ISLAB    ,NTRAC
USE YOMFORC1S, ONLY : GFOR     ,JPSTPFC  ,JPTYFC   ,UFI     ,&
                      &VFI      ,TFI      ,QFI      ,PSFI    ,&
                      &SRFFI    ,TRFFI    ,R30FI    ,S30FI   ,&
                      &R30FI_C  ,S30FI_C
USE YOMLUN1S , ONLY : NULOUT
USE YOMLOG1S , ONLY : CFFORC   ,CFOUT,CFINIT,CFSURF
USE YOMGF1S  , ONLY : UNLEV0   ,VNLEV0   ,TNLEV0   ,QNLEV0   ,CNLEV0 ,&
                      &PNLP0    ,UNLEV1   ,VNLEV1   ,TNLEV1   ,&
                      &QNLEV1   ,CNLEV1   ,FSSRD    ,FSTRD    ,FLSRF    ,&
                      &FCRF     ,FLSSF    ,FCSF 

!**** *SUINIF1S*  - Initialize the fields of the one column surface model.

!     Purpose.
!     --------
!           Initialize the fields of the one column surface model.

!***  Interface.
!     ----------
!        *CALL* *SUINIF1S*

!        Explicit arguments :
!        --------------------
!         LOPLEFT: FLAG INDICATING WHETHER MORE POINTS NEED TO BE DONE

!        Implicit arguments :
!        --------------------
!         NONE

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by CNT31S

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the one column model


!     Author.
!     -------
!        Jean-Francois Mahfouf and Pedro Viterbo   *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-03-21
!        Bart vd HURK (KNMI) Multi-column use of netCDF and restart utility

IMPLICIT NONE

#include "sudyn1s.intfb.h"
#include "suct01s.intfb.h"
#include "rdres.intfb.h"
#include "sugc1s.intfb.h"
#include "sugpd1s.intfb.h"
#include "sugp1s.intfb.h"
#include "sufcdf.intfb.h"
#include "sugdi1s.intfb.h"
#include "sucdh1s.intfb.h"

INTEGER :: NCID,IERR		! for netcdf interface

INTEGER(KIND=JPIM) :: MUFI,MVFI,MTFI,MQFI,MPSFI,MSRFFI,MTRFFI,MR30FI,MS30FI, &
   &   MR30CFI,MS30CFI

LOGICAL LOPLEFT		! FLAG FOR RUNNING MORE GRID POINTS
      
#include "netcdf.inc"

!*       SETUP OF TIME VARIABLES.

CALL SUDYN1S (NULOUT)

!*       RUN AND OUTPUT CONTROLS.

CALL SUCT01S (NULOUT)

!*       READ RESTART

IF(NSTART /= 0)THEN

  IF(CFOUT == 'netcdf')THEN
    CALL RDRES(LOPLEFT)
  ELSE
    WRITE(NULOUT,*)'RESTART FILE REQUIRES NETCDF CONFIGURATION'
    CALL ABORT
  ENDIF

ELSE

!*       SETUP OF GEOGRAPHICAL CONSTANTS.

!* Open surface climate field

  IF (CFSURF == 'netcdf')THEN
    NCID = NCOPN('surfclim', NCNOWRIT, IERR)
    WRITE(NULOUT,*)'NETCDF-FILE surfclim OPENED ON UNIT ',NCID
  ENDIF
  CALL SUGC1S (NCID,LOPLEFT)

!*       SETUP OF SOIL/VEGETATION CONSTANTS.

  CALL SUGPD1S (NCID)
  IF (CFSURF == 'netcdf')THEN
    CALL NCCLOS(NCID,IERR)
  ENDIF

!*       INITIALISATION OF SOIL PROGNOSTIC VARIABLES.

!* Open surface prognostics field

  IF (CFINIT == 'netcdf')THEN
    NCID = NCOPN('soilinit', NCNOWRIT, IERR)
    WRITE(NULOUT,*)'NETCDF-FILE soilinit OPENED ON UNIT ',NCID
  ENDIF
  CALL SUGP1S (NCID)
  IF (CFINIT == 'netcdf')THEN
    CALL NCCLOS(NCID,IERR)
  ENDIF

ENDIF

!*       INITIALISATION OF THE ATMOSPHERIC FORCING.

!*       ALLOCATION OF INPUT FORCING
IF(ISLAB > 1)THEN
  DEALLOCATE (GFOR)
ENDIF
ALLOCATE (GFOR(NPOI,JPSTPFC,JPTYFC))

MUFI=1
MVFI=MUFI+1
MTFI=MVFI+1
MQFI=MTFI+1
MPSFI=MQFI+1
MSRFFI=MPSFI+1
MTRFFI=MSRFFI+1
MR30FI=MTRFFI+1
MS30FI=MR30FI+1
MR30CFI=MS30FI+1
MS30CFI=MR30CFI+1

IF(MS30CFI /= JPTYFC)THEN
  WRITE(*,*)'NUMBER OF FORCING VARIABLES NOT COMPATIBLE: ',MS30CFI,JPTYFC
  CALL ABORT
ENDIF

UFI => GFOR(:,:,MUFI)
VFI => GFOR(:,:,MVFI)
TFI => GFOR(:,:,MTFI)
QFI => GFOR(:,:,MQFI)
PSFI => GFOR(:,:,MPSFI)
SRFFI => GFOR(:,:,MSRFFI)
TRFFI => GFOR(:,:,MTRFFI)
R30FI => GFOR(:,:,MR30FI)
S30FI => GFOR(:,:,MS30FI)
R30FI_C => GFOR(:,:,MR30CFI)
S30FI_C => GFOR(:,:,MS30CFI)

IF (CFFORC == 'netcdf')THEN
  CALL SUFCDF
ELSE
  WRITE (NULOUT,"(' Forcing data set ',a,' not known ')")&
   &CFFORC(1:len_trim(CFFORC))
  CALL ABORT
ENDIF

IF(ISLAB > 1)THEN
  DEALLOCATE (UNLEV0)
  DEALLOCATE (VNLEV0)
  DEALLOCATE (TNLEV0)
  DEALLOCATE (QNLEV0)
  DEALLOCATE (CNLEV0)
  DEALLOCATE (PNLP0)
  DEALLOCATE (UNLEV1)
  DEALLOCATE (VNLEV1)
  DEALLOCATE (TNLEV1)
  DEALLOCATE (QNLEV1)
  DEALLOCATE (CNLEV1)
  DEALLOCATE (FSSRD)
  DEALLOCATE (FSTRD)
  DEALLOCATE (FLSRF)
  DEALLOCATE (FCRF)
  DEALLOCATE (FLSSF)
  DEALLOCATE (FCSF)
ENDIF
ALLOCATE (UNLEV0(NPOI))
ALLOCATE (VNLEV0(NPOI))
ALLOCATE (TNLEV0(NPOI))
ALLOCATE (QNLEV0(NPOI))
ALLOCATE (CNLEV0(NPOI,NTRAC))	! check what happens if ntrac = 0
ALLOCATE (PNLP0(NPOI))
ALLOCATE (UNLEV1(NPOI))
ALLOCATE (VNLEV1(NPOI))
ALLOCATE (TNLEV1(NPOI))
ALLOCATE (QNLEV1(NPOI))
ALLOCATE (CNLEV1(NPOI,NTRAC))
ALLOCATE (FSSRD(NPOI))
ALLOCATE (FSTRD(NPOI))
ALLOCATE (FLSRF(NPOI))
ALLOCATE (FCRF(NPOI))
ALLOCATE (FLSSF(NPOI))
ALLOCATE (FCSF(NPOI))

CNLEV0(:,:) = 0._JPRB
CNLEV1(:,:) = 0._JPRB 
!*       ALLOCATES THE DIAGNOSTIC VARIABLES.

CALL SUGDI1S
CALL SUCDH1S

RETURN
END SUBROUTINE SUINIF1S
