SUBROUTINE WRTRES
USE PARKIND1  ,ONLY : JPIM     ,JPRB,JPRM,JPRD

!**** *WRTRES*  - writes NetCDF restart file

!     Purpose.
!     --------
!        writes out restart file

!**   Interface.
!     ----------
!        *CALL* *WRTRES

!     Explicit arguments :
!     --------------------

!     Implicit arguments :
!     --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------

!     Author.
!     -------
!        Bart vd Hurk (KNMI)

!     Modifications.
!     --------------
!        original: 17/7/2000
!        E. DUTRA     02/2009 - ADD LAKES RELATED
!        E. Dutra     11/2009 - Add LAI restart

USE YOMLUN1S , ONLY : NPOSRES  ,NULOUT ,RMISS
USE YOMLOG1S , ONLY : NDIMCDF,LNCSNC
USE YOETHF   , ONLY : RHOH2O
USE YOMDYN1S , ONLY : NSTEP    ,TSTEP
USE YOMCC1S  , ONLY : VCALB,VCLAIL,VCLAIH
USE YOMGP1S0 , ONLY : TSLNU0   ,QLINU0   ,TILNU0   ,FSNNU0   ,&
            &TSNNU0   ,ASNNU0   ,RSNNU0   ,WRENU0   ,TRENU0,&
            &TLICENU0,TLMNWNU0,TLWMLNU0,TLBOTNU0,TLSFNU0,& !FLAKE
            &HLICENU0,HLMLNU0, &                           !FLAKE 
            &LAINU0,BSTRNU0, BSTR2NU0,WSNNU0

USE YOMGPD1S , ONLY : VFZ0F    ,VFZ0H     ,VFITM    ,VFCVL &
                     &,VFCVH    ,VFTVL     ,VFTVH    ,VFSST &
                     &,VFCI     ,VFSOTY    ,VFSDOR  ,VFGEO &
                     &,VDIEWSSTL ,VDINSSSTL,VDISSHFTL &
                     &,VDIETL   ,VDTSKTL,VFLDEPTH,VFCLAKE &
                     &,VFR0VT   ,VDANDAYVT,VDANFMVT &
		     &,VDRESPBSTR,VDRESPBSTR2,VDBIOMASS_LAST&
		     &,VFSOALUVD,VFSOALUVP,VFSOALNID,VFSOALNIP

USE YOMDPHY  , ONLY : NLAT     ,NLON      ,NPOI     ,NTILES &
                     &,NLALO     ,NSLAB    ,ISLAB   ,NCSS &
		     &,NVHILO, NCLIMDAT,NCSNEC

USE YOESOIL1S, ONLY : RDAW
USE YOMRIP   , ONLY : RSTATI
USE YOMGC1S  , ONLY : LMASK
USE YOEPHY   , ONLY : LEVGEN   ,LESSRO, LEFLAKE, LECTESSEL,LESNML
USE YOMECEARTH, ONLY : LRDALB


IMPLICIT NONE

!* Netcdf Interface
INTEGER NPOS,IERR,NSTPP,NVARID
REAL(KIND=JPRD) ZTIME
INTEGER ISTART1(1),ISTART2(2),ISTART3(3),ISTART4(4)
INTEGER ICOUNT1(1),ICOUNT2(2),ICOUNT3(3),ICOUNT4(4)
REAL(KIND=JPRD),ALLOCATABLE :: ZMASK(:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUT(:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUTA(:,:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUTT(:,:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUTS(:,:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUTSN(:,:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUTV(:,:)
REAL(KIND=JPRD),ALLOCATABLE :: ZOUTPUTVM(:,:,:)

!* Local declarations
REAL(KIND=JPRB),ALLOCATABLE :: ZQLINU0(:,:)
INTEGER(KIND=JPIM) :: IX1,IX2,IY1,IY2,NLEN,IST,IEN,ILEVS,JM,JTILE,JK,JVT

#include "netcdf.inc"

!     ------------------------------------------------------------------

!*       1.   SCALING SOME VARIABLES.
!             -----------------------

!     NOTE: UNITS OF FSNNU,WRENU IN THE SURF1D MODEL:            KG/M2
!           UNITS OF SNOW AND INTERCEP RESER WATER IN THE
!                                     PHYSICS:                   KG/M2
!           UNITS OF SNOW AND INTERCEP RESER WATER IN THE
!                                     INPUT AND OUTPUT FILES:    KG/M2=MM

IF(NDIMCDF == 2)THEN
  IX1=1
  IX2=NLON
  IY1=(ISLAB-1)*NSLAB+1
  IY2=MIN(NLAT,ISLAB*NSLAB)
  NLEN=(IX2-IX1+1)*(IY2-IY1+1)
  IST=IX1+(IY1-1)*NLON
  IEN=IST+NLEN-1
ELSE
  IST=(ISLAB-1)*NSLAB+1
  IEN=MIN(NLALO,ISLAB*NSLAB)
  NLEN=IEN-IST+1
ENDIF
ALLOCATE (ZOUTPUT(NLEN))		!INFLATED OUTPUT ARRAY
ALLOCATE (ZOUTPUTA(NLEN,NCLIMDAT))		!INFLATED OUTPUT ARRAY
ALLOCATE (ZOUTPUTT(NLEN,NTILES))
ALLOCATE (ZOUTPUTS(NLEN,NCSS))
ALLOCATE (ZOUTPUTSN(NLEN,NCSNEC))

ALLOCATE (ZOUTPUTV(NLEN,NVHILO))
ALLOCATE (ZOUTPUTVM(NLEN,NVHILO,12))


ALLOCATE (ZQLINU0(NPOI,NCSS))
DO ILEVS=1,NCSS
  ZQLINU0(:,ILEVS)=QLINU0(:,ILEVS)*RDAW(ILEVS)*RHOH2O
ENDDO

NPOS = NPOSRES

!* -- write spatial and time coordinates

!     time
ISTART1(1)=1
ZTIME=RSTATI-TSTEP*0.5
NVARID = NCVID(NPOS,'time',IERR)
CALL NCVPT1 (NPOS,NVARID,ISTART1,ZTIME,IERR)
NVARID = NCVID(NPOS,'timestp',IERR)
NSTPP=NSTEP
CALL NCVPT1 (NPOS,NVARID,ISTART1,NSTPP,IERR)

!* -- Surface characteristics
!***********************************************************************
IF(NDIMCDF == 2)THEN
  ISTART2(1)=IX1
  ISTART2(2)=IY1
  ICOUNT2(1)=IX2-IX1+1
  ICOUNT2(2)=IY2-IY1+1
ELSE
  ISTART2(1)=IST
  ICOUNT2(1)=NLEN
ENDIF

!     catchment mask
ALLOCATE (ZMASK(NLEN))
ZMASK(:)=RMISS
WHERE (LMASK(IST:IEN))
  ZMASK=1.
END WHERE
NVARID = NCVID(NPOS,'Mask',IERR)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZMASK,IERR)

!     momentum roughness
NVARID = NCVID(NPOS,'z0m',IERR)
ZOUTPUT=UNPACK(VFZ0F,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     heat roughness
NVARID = NCVID(NPOS,'lz0h',IERR)
ZOUTPUT=UNPACK(LOG(VFZ0H),LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     monthly albedo
IF(NDIMCDF == 2)THEN
  ISTART3(1)=IX1
  ISTART3(2)=IY1
  ISTART3(3)=1
  ICOUNT3(1)=IX2-IX1+1
  ICOUNT3(2)=IY2-IY1+1
  ICOUNT3(3)=NCLIMDAT
ELSE
  ISTART3(1)=IST
  ISTART3(2)=1
  ICOUNT3(1)=NLEN
  ICOUNT3(2)=NCLIMDAT
ENDIF
DO JM=1,NCLIMDAT
  ZOUTPUTA(:,JM)=UNPACK(VCALB(:,JM),LMASK(IST:IEN),RMISS)
ENDDO
NVARID = NCVID(NPOS,'Malbedo',IERR)
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTA,IERR)

! LAI low veg
DO JM=1,NCLIMDAT
  ZOUTPUTA(:,JM)=UNPACK(VCLAIL(:,JM),LMASK(IST:IEN),RMISS)
ENDDO
NVARID = NCVID(NPOS,'Mlail',IERR)
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTA,IERR)

! LAI high veg
DO JM=1,NCLIMDAT
  ZOUTPUTA(:,JM)=UNPACK(VCLAIH(:,JM),LMASK(IST:IEN),RMISS)
ENDDO
NVARID = NCVID(NPOS,'Mlaih',IERR)
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTA,IERR)

!     land sea mask
NVARID = NCVID(NPOS,'landsea',IERR)
ZOUTPUT=UNPACK(VFITM,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     geopotential
NVARID = NCVID(NPOS,'geopot',IERR)
ZOUTPUT=UNPACK(VFGEO,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     low vegetation coverage
NVARID = NCVID(NPOS,'cvl',IERR)
ZOUTPUT=UNPACK(VFCVL,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
!     high vegetation coverage
NVARID = NCVID(NPOS,'cvh',IERR)
ZOUTPUT=UNPACK(VFCVH,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     low vegetation type
NVARID = NCVID(NPOS,'tvl',IERR)
ZOUTPUT=UNPACK(VFTVL,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
!     high vegetation type
NVARID = NCVID(NPOS,'tvh',IERR)
ZOUTPUT=UNPACK(VFTVH,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     soil type
NVARID = NCVID(NPOS,'sotype',IERR)
ZOUTPUT=UNPACK(VFSOTY,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     orographic variance
NVARID = NCVID(NPOS,'sdor',IERR)
ZOUTPUT=UNPACK(VFSDOR,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     sst
NVARID = NCVID(NPOS,'sst',IERR)
ZOUTPUT=UNPACK(VFSST,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     sea ice fraction
NVARID = NCVID(NPOS,'seaice',IERR)
ZOUTPUT=UNPACK(VFCI,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

IF (.NOT. LRDALB) THEN
  ! Soil albedo
  NVARID = NCVID(NPOS,'soaluvd',IERR)
  ZOUTPUT=UNPACK(VFSOALUVD,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  NVARID = NCVID(NPOS,'soaluvp',IERR)
  ZOUTPUT=UNPACK(VFSOALUVP,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  NVARID = NCVID(NPOS,'soalnid',IERR)
  ZOUTPUT=UNPACK(VFSOALNID,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  NVARID = NCVID(NPOS,'soalnip',IERR)
  ZOUTPUT=UNPACK(VFSOALNIP,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
ENDIF 


!* -- Tile specific quantities
!***********************************************************************
!     sensible heat flux
IF(NDIMCDF == 2)THEN
  ISTART3(1)=IX1
  ISTART3(2)=IY1
  ISTART3(3)=1
  ICOUNT3(1)=IX2-IX1+1
  ICOUNT3(2)=IY2-IY1+1
  ICOUNT3(3)=NTILES
ELSE
  ISTART3(1)=IST
  ISTART3(2)=1
  ICOUNT3(1)=NLEN
  ICOUNT3(2)=NTILES
ENDIF
NVARID = NCVID(NPOS,'Qh',IERR)
DO JTILE=1,NTILES
  ZOUTPUTT(:,JTILE)=UNPACK(VDISSHFTL(:,JTILE),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTT,IERR)

!     latent heat flux
NVARID = NCVID(NPOS,'Evap',IERR)
DO JTILE=1,NTILES
  ZOUTPUTT(:,JTILE)=UNPACK(VDIETL(:,JTILE),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTT,IERR)

!     momentum flux u
NVARID = NCVID(NPOS,'Momu',IERR)
DO JTILE=1,NTILES
  ZOUTPUTT(:,JTILE)=UNPACK(VDIEWSSTL(:,JTILE),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTT,IERR)

!     momentum flux v
NVARID = NCVID(NPOS,'Momv',IERR)
DO JTILE=1,NTILES
  ZOUTPUTT(:,JTILE)=UNPACK(VDINSSSTL(:,JTILE),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTT,IERR)

!     skin temperature
NVARID = NCVID(NPOS,'SkinT',IERR)
DO JTILE=1,NTILES
  ZOUTPUTT(:,JTILE)=UNPACK(VDTSKTL(:,JTILE),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTT,IERR)

!*-- Prognostic quantities
!***********************************************************************
!     skin layer content
NVARID = NCVID(NPOS,'CanopInt',IERR)
ZOUTPUT=UNPACK(WRENU0,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     snow mass
NVARID = NCVID(NPOS,'SWE',IERR)
ZOUTPUT=UNPACK(FSNNU0(:,1),LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     snow temp
NVARID = NCVID(NPOS,'SnowT',IERR)
ZOUTPUT=UNPACK(TSNNU0(:,1),LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     snow density
NVARID = NCVID(NPOS,'snowdens',IERR)
ZOUTPUT=UNPACK(RSNNU0(:,1),LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     snow albedo
NVARID = NCVID(NPOS,'SAlbedo',IERR)
ZOUTPUT=UNPACK(ASNNU0,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)

!     skin temperature
NVARID = NCVID(NPOS,'AvgSurfT',IERR)
ZOUTPUT=UNPACK(TRENU0,LMASK(IST:IEN),RMISS)
CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)


IF (LEFLAKE) THEN
  !     Lake ice temperature
  NVARID = NCVID(NPOS,'TLICE',IERR)
  ZOUTPUT=UNPACK(TLICENU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake mean water temperature
  NVARID = NCVID(NPOS,'TLMNW',IERR)
  ZOUTPUT=UNPACK(TLMNWNU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake mixed layer temperature
  NVARID = NCVID(NPOS,'TLWML',IERR)
  ZOUTPUT=UNPACK(TLWMLNU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake bottom temperature 
  NVARID = NCVID(NPOS,'TLBOT',IERR)
  ZOUTPUT=UNPACK(TLBOTNU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !    lake temperature shape factor 
  NVARID = NCVID(NPOS,'TLSF',IERR)
  ZOUTPUT=UNPACK(TLSFNU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake ice thickness 
  NVARID = NCVID(NPOS,'HLICE',IERR)
  ZOUTPUT=UNPACK(HLICENU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake mixed layer thickness
  NVARID = NCVID(NPOS,'HLML',IERR)
  ZOUTPUT=UNPACK(HLMLNU0,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake depth
  NVARID = NCVID(NPOS,'LDEPTH',IERR)
  ZOUTPUT=UNPACK(VFLDEPTH,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
  
  !     lake cover - lake mask
  NVARID = NCVID(NPOS,'CLAKE',IERR)
  ZOUTPUT=UNPACK(VFCLAKE,LMASK(IST:IEN),RMISS)
  CALL NCVPT (NPOS,NVARID,ISTART2,ICOUNT2,ZOUTPUT,IERR)
ENDIF

IF(NDIMCDF == 2)THEN
  ISTART3(1)=IX1
  ISTART3(2)=IY1
  ISTART3(3)=1
  ICOUNT3(1)=IX2-IX1+1
  ICOUNT3(2)=IY2-IY1+1
  ICOUNT3(3)=NCSS
ELSE
  ISTART3(1)=IST
  ISTART3(2)=1
  ICOUNT3(1)=NLEN
  ICOUNT3(2)=NCSS
ENDIF

!     soil temperature
NVARID = NCVID(NPOS,'SoilTemp',IERR)
DO JK=1,NCSS
  ZOUTPUTS(:,JK)=UNPACK(TSLNU0(:,JK),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTS,IERR)

!     soil moisture
NVARID = NCVID(NPOS,'SoilMoist',IERR)
DO JK=1,NCSS
  ZOUTPUTS(:,JK)=UNPACK(ZQLINU0(:,JK),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTS,IERR)

!     sea ice temperature
NVARID = NCVID(NPOS,'icetemp',IERR)
DO JK=1,NCSS
  ZOUTPUTS(:,JK)=UNPACK(TILNU0(:,JK),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTS,IERR)

IF (LESNML) THEN
  
  IF(NDIMCDF == 2)THEN
    ICOUNT3(3)=NCSNEC
  ELSE
    ICOUNT3(2)=NCSNEC
  ENDIF
  
  !     SWE
  NVARID = NCVID(NPOS,'SWEML',IERR)
  DO JK=1,NCSNEC
    ZOUTPUTSN(:,JK)=UNPACK(FSNNU0(:,JK),LMASK(IST:IEN),RMISS)
  ENDDO
  CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTSN,IERR)
  !     SwnoT
  NVARID = NCVID(NPOS,'SnowTML',IERR)
  DO JK=1,NCSNEC
    ZOUTPUTSN(:,JK)=UNPACK(TSNNU0(:,JK),LMASK(IST:IEN),RMISS)
  ENDDO
  CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTSN,IERR)
  !     Snowdensity 
  NVARID = NCVID(NPOS,'snowdensML',IERR)
  DO JK=1,NCSNEC
    ZOUTPUTSN(:,JK)=UNPACK(RSNNU0(:,JK),LMASK(IST:IEN),RMISS)
  ENDDO
  CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTSN,IERR)
  !     Snow liquid water 
  NVARID = NCVID(NPOS,'slwML',IERR)
  DO JK=1,NCSNEC
    ZOUTPUTSN(:,JK)=UNPACK(WSNNU0(:,JK),LMASK(IST:IEN),RMISS)
  ENDDO
  CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTSN,IERR) 
 

ENDIF 



!IF (LECTESSEL) THEN

IF(NDIMCDF == 2)THEN
  ICOUNT3(3)=NVHILO
ELSE
  ICOUNT3(2)=NVHILO
ENDIF
!     reference respiration

DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(VFR0VT(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
NVARID = NCVID(NPOS,'r0vt',IERR)
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)


!     lai per vegetation type
NVARID = NCVID(NPOS,'laivt',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(LAINU0(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)


!     Bstr per vegetation type
NVARID = NCVID(NPOS,'Biomstr',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(BSTRNU0(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)

!     Bstr2 per vegetation type
NVARID = NCVID(NPOS,'Biomstr2',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(BSTR2NU0(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)

!* -- Vegetation type specific quantities
!***********************************************************************
IF(NDIMCDF == 2)THEN
  ISTART3(1)=IX1
  ISTART3(2)=IY1
  ISTART3(3)=1
  ICOUNT3(1)=IX2-IX1+1
  ICOUNT3(2)=IY2-IY1+1
  ICOUNT3(3)=NVHILO
ELSE
  ISTART3(1)=IST
  ISTART3(2)=1
  ICOUNT3(1)=NLEN
  ICOUNT3(2)=NVHILO
ENDIF

!     daily net CO2 assimilation
NVARID = NCVID(NPOS,'Anday',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(VDANDAYVT(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)
!     daily maximum leaf assimilation
NVARID = NCVID(NPOS,'Anfm',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(VDANFMVT(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)

!     respiration of above ground structural biomass
NVARID = NCVID(NPOS,'Rstr',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(VDRESPBSTR(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)

!     respiration of below ground structural biomass
NVARID = NCVID(NPOS,'Rstr2',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(VDRESPBSTR2(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)

!     biomass of previous day
NVARID = NCVID(NPOS,'Blast',IERR)
DO JVT=1,NVHILO
  ZOUTPUTV(:,JVT)=UNPACK(VDBIOMASS_LAST(:,JVT),LMASK(IST:IEN),RMISS)
ENDDO
CALL NCVPT (NPOS,NVARID,ISTART3,ICOUNT3,ZOUTPUTV,IERR)

!     synchronize to disk
IF (LNCSNC) CALL NCSNC(NPOS,IERR)
!***********************************************************************

WRITE(NULOUT,*)'RESTART FILE WRITTEN FOR TIMESTEP: ',NSTEP
DEALLOCATE (ZOUTPUT)		!INFLATED OUTPUT ARRAY
DEALLOCATE (ZOUTPUTA)		!INFLATED OUTPUT ARRAY
DEALLOCATE (ZOUTPUTT)
DEALLOCATE (ZOUTPUTS)
DEALLOCATE (ZOUTPUTSN)
DEALLOCATE (ZQLINU0)
DEALLOCATE (ZMASK)
DEALLOCATE (ZOUTPUTV)
DEALLOCATE (ZOUTPUTVM)

END SUBROUTINE WRTRES
