#!/bin/ksh
#
#   Job to create meteorological forcing 
# Emanuel Dutra, November 2017

set -exu

. config.ksh 


CVARS="Wind Tair Qair PSurf SWdown LWdown Rainf Snowf Ctpf"
CVARS="PSurf" 

snameGG="sp/2t/2d/10u/10v/ssrd/strd/lsp/cp/sf"
snameGGML="q"
snameSH="u/v/t"

## check and create DATADIR
mkdir -p $DATADIR
rm -rfv $DATADIR/*

WDIR=$TMPDIR/create_forcing  # Work directory 
mkdir -p $WDIR 
cd $WDIR 
rm -rfv *

## cat/organize files 
if [[ -r $FORCRAW/forcing_${FTYPE}_${YMD}.tar.gz ]]; then
  # netcdf already done 
  tar -zxvf $FORCRAW/forcing_${FTYPE}_${YMD}.tar.gz
  
else
  if [[ -r $FORCRAW/forcing_${YMD}.tar.gz ]]; then
    
    # processed grib already done 
    tar -zxvf $FORCRAW/forcing_${YMD}.tar.gz
    
  else
    # processed grib not read, process GG and SH 
    if [[ $YMD == $DSTART ]]; then
      grib_copy -w shortName=$snameGG $FORCRAW/ICMGG${FEXP}+000000  tmp_gg.grb 
      grib_copy -w shortName=$snameGG $FORCRAW/ICMGG${FEXP}+${YMD:0:6}  tmp1_gg.grb 
      cat tmp1_gg.grb >> tmp_gg.grb 
      rm -f tmp1_gg.grb
      if $HAS_ML; then
      grib_copy -w shortName=$snameGGML,level=$FLEV $FORCRAW/ICMGG${FEXP}+000000  tmp1_gg.grb 
      grib_copy -w shortName=$snameGGML,level=$FLEV $FORCRAW/ICMGG${FEXP}+${YMD:0:6}  tmp2_gg.grb 
      cat tmp1_gg.grb >> tmp_gg.grb 
      cat tmp2_gg.grb >> tmp_gg.grb 
      rm -f tmp1_gg.grb tmp1_gg.grb
      
      grib_copy -w shortName=$snameSH,level=$FLEV $FORCRAW/ICMSH${FEXP}+000000  tmp_sh.grb 
      grib_copy -w shortName=$snameSH,level=$FLEV $FORCRAW/ICMSH${FEXP}+${YMD:0:6}  tmp1_sh.grb 
      cat tmp1_sh.grb >> tmp_sh.grb 
      rm -f tmp1_sh.grb
      fi
      
    else
      YMP=$( date -d "$YMD - 1 month" +"%Y%m" )
      grib_copy -w shortName=$snameGG,date=${YMD} $FORCRAW/ICMGG${FEXP}+${YMP}  tmp_gg.grb 
      grib_copy -w shortName=$snameGG $FORCRAW/ICMGG${FEXP}+${YMD:0:6}  tmp1_gg.grb 
      cat tmp1_gg.grb >> tmp_gg.grb 
      rm -f tmp1_gg.grb
      if $HAS_ML; then
      grib_copy -w shortName=$snameGGML,date=${YMD},level=$FLEV $FORCRAW/ICMGG${FEXP}+${YMP}  tmp1_gg.grb 
      grib_copy -w shortName=$snameGGML,level=$FLEV $FORCRAW/ICMGG${FEXP}+${YMD:0:6}  tmp2_gg.grb 
      cat tmp1_gg.grb >> tmp_gg.grb 
      cat tmp2_gg.grb >> tmp_gg.grb 
      rm -f tmp1_gg.grb tmp1_gg.grb
      
      grib_copy -w shortName=$snameSH,date=${YMD} $FORCRAW/ICMSH${FEXP}+${YMP}  tmp_sh.grb 
      grib_copy -w shortName=$snameSH $FORCRAW/ICMSH${FEXP}+${YMD:0:6}  tmp1_sh.grb 
      cat tmp1_sh.grb >> tmp_sh.grb 
      rm -f tmp1_sh.grb
      fi
    fi # end of special case for start-date in processing grib files   
    
    tar -czvf forcing_${YMD}.tar.gz tmp_??.grb
    mv forcing_${YMD}.tar.gz $FORCRAW/forcing_${YMD}.tar.gz
  fi 
  # TODO put this before the .tar.gz creation
  if [[ $FTYPE == "ML" ]]; then
    # convert SH to GG 
    mars << EOF
      read,source="tmp_sh.grb",grid=N128,target="tmp1_gg.grb"
EOF
    cat tmp1_gg.grb >> tmp_gg.grb 
    rm -f tmp1_gg.grb 
  fi 
    
   
  
  python << EOF
from create_forcing import *
SURFCLIM="$FIXDIR/surfclim"
INPUTGRB="tmp_gg.grb"
FTYPE="$FTYPE"
tunits="seconds since ${YMD:0:4}-${YMD:4:2}-${YMD:6:2} 00:00:00"
process(SURFCLIM,INPUTGRB,FTYPE,tunits)
EOF
  ncks -A -v Ctpf Ctpf.nc Rainf.nc 
  rm -f Ctpf.nc

  tar -czvf forcing_${FTYPE}_${YMD}.tar.gz *.nc
  mv forcing_${FTYPE}_${YMD}.tar.gz $FORCRAW
fi # end of processing 

mv *.nc $DATADIR

rm -rfv $WDIR 

exit 0
