#include "tm5.inc"
!
! Levels.
!

module dims_levels

  use const_ec_v, only : lme, a_ec, b_ec
  use dims_grid, only : nregions_all
  
  implicit none
  
  ! --- in/out ------------------------------
  
  public
  
  
  ! --- const -------------------------------
  
  
  integer, parameter ::  lm(0:nregions_all) = 34

  ! Only lowest layers are used for convec calculations (sub files)
  ! to avoid memory problems.
#ifdef without_lmax_conv
  ! don't bother about computation time and storage ...
  integer,parameter :: lmax_conv = 34
#else
  ! Quote Michiel van Weele:
  !     "..., convectie kan wel tot ~70 hPa (19km) gaan in de tropen."
  integer,parameter :: lmax_conv = 23
#endif

  ! select ECMWF half levels;
  ! TM levels are number bottom-up:
  
  ! These levels are the same as the "34 levels out of 91 levels". To get that, the correspondence between
  ! 91 and 137 comes from this web page:
  ! http://www.ecmwf.int/en/forecasts/documentation-and-suppor/correspondence-between-l91-and-l137-model-levels
  
  integer, parameter   ::  echlevs(0:34) = (/ &
       137, 134, 129, 124, 119, 114, 110, 105, 101,  97,&
        93,  88,  84,  81,  78,  76,  73,  70,  67,  65,&
        62,  59,  57,  54,  51,  46,  42,  37,  32,  27,&
        22,  17,  12,   7,   0 /)

  ! values filled in module geometry ...
  real  :: at(1:34+1)
  real  :: bt(1:34+1)


end module dims_levels

