#include "tm5.inc"
! ----------------------------------------------------------------------
!
! NAME
!   const_ec_v - vertical level definition of the driving model
!
! USAGE
!   use const_ec_v
!
! ----------------------------------------------------------------------

module const_ec_v

  implicit none
  
  ! --- in/out -----------------------------
  
  private 

  public :: lme, a_ec, b_ec
  
  
  ! --- const --------------------------------
  
  ! number of levels:
  integer, parameter :: lme = 34
      
  ! Hybrid coordinate coefficients at half level interfaces,
  ! specifying 34 vertical ECMWF levels.
  ! Coefficient a is given in [Pa] .
  ! Pressure at a certain hybrid level is defined as: p = a + b.SP

  real, parameter  :: a_ec(0:lme) = (/   &
           0.000000,                             &
          21.413612,    76.167656,               &
         204.637451,   450.685791,               &
         857.945801,                             &
        1463.163940,  2292.155518,               &
        3358.425781,  4663.776367,               &
        6199.839355,  7341.469727,               &
        8564.624023,  9873.560547,               &
       11262.484375, 12713.897461, 14192.009766, &
       15638.053711, 16990.623047,               &
       18191.029297, 19184.544922, 19919.796875, &
       20348.916016, 20319.011719,               &
       19348.775391,                             &
       17385.595703, 14665.645508,               &
       11543.166992,  8356.252930,               &
        5422.802734,                             &
        3010.146973,  1297.656128,               &
         336.772369,     6.575628,               &
           0.000000 /)

  real, parameter  :: b_ec(0:lme) = (/   &
       0.000000,                     &
       0.000000, 0.000000,           &
       0.000000, 0.000000,           &
       0.000000,                     &
       0.000000, 0.000000,           &
       0.000000, 0.000000,           &
       0.000000, 0.000000,           &
       0.000055, 0.000279,           &
       0.001000, 0.002765, 0.006322, &
       0.012508, 0.022189,           &
       0.036227, 0.055474, 0.080777, &
       0.112979, 0.176091,           &
       0.259554,                     &
       0.362203, 0.475016,           &
       0.589317, 0.698224,           &
       0.795385,                     &
       0.875518, 0.935157,           &
       0.973466, 0.994204,           &
       1.000000 /)

end module const_ec_v
