! First include the set of model-wide compiler flags 
#include "tm5.inc"

Module MTM5Data
  !
  ! This module defines the structure to store the instantaneous TM5 fields needed by DOMINO
  !

  implicit none

  private

  public :: TTM5Data
  
  ! ------------------------------------------------------------------
  ! structure containing TM5 fields at the time of the observations
  !    note: fields are 3D and not 4D like in the TM5 output files
  ! ------------------------------------------------------------------

  ! im,jm,lm  : dimensions of the 3D TM5 fields (lon,lat,lev)
  ! lon       : longitude, degree east, dimension "im"
  ! lat       : latitude, degree north, dimension "jm"
  ! hyai      : hybrid A coefficient at layer interfaces, unit Pa, dimension "lm+1"
  ! hybi      : hybrid B coefficient at layer interfaces, unit 1, dimension "lm+1"
  ! hyam      : hybrid A coefficient at layer midpoints, unit Pa, dimension "lm"
  ! hybm      : hybrid B coefficient at layer midpoints, unit 1, dimension "lm"
  ! date      : date-time of the fields: year, month, day, hour, minute, second
  ! ps        : surface pressure, unit "Pa", dimension ("im","jm")
  ! oro       : surface altitude of TM5 grid, unit "m", dimension ("im","jm")
  !             based on ECMWF interpolated orography field
  ! t         : temperature, unit "K", dimension ("im","jm","lm")
  ! no2       : volume mixing ratio of NO2 in humid air, unit "1", dimension ("im","jm","lm") 

  type TTM5Data
     integer                             :: im, jm, lm
     real,dimension(:),allocatable       :: lon       
     real,dimension(:),allocatable       :: lat
     real,dimension(:),allocatable       :: hyai
     real,dimension(:),allocatable       :: hybi
     real,dimension(:),allocatable       :: hyam
     real,dimension(:),allocatable       :: hybm
     integer,dimension(6)                :: date
     real,dimension(:,:),allocatable     :: ps
     real,dimension(:,:),allocatable     :: oro
     real,dimension(:,:,:),allocatable   :: t
     integer,dimension(:,:),allocatable  :: ltropo
     real,dimension(:,:,:),allocatable   :: no2
  end type TTM5Data


end Module MTM5Data
