#ifndef  __MAPPER_HPP__
#define __MAPPER_HPP__
#include "parallel_tree.hpp"

enum verbosity
{
	SILENT = 0,
	PROGRESS = 1,
	DETAILS = 2
};

void cptOffsetsFromLengths(const int *lengths, int *offsets, int sz);

class Mapper : CParallelTree
{
public:
	Mapper(int verbose = 2) : CParallelTree(), verbose(verbose) { }
	~Mapper();
	
	double buildSSTree(vector<Node>& srcMsh, vector<Node>& trgMsh)
	{
		buildParallelTree(srcMsh, trgMsh);
	}

	/** @param trgElts are the elements of the unstructured target grid
	    Returns the timings for substeps: */
	vector<double> computeWeights(vector<Elt>& trgElts, vector<Elt>& srcElts, int interpOrder);

	/* where weights are returned after call to `computeWeights` */
	double *remapMatrix;
	int *srcAddress;
	int *dstAddress;
	int nWeights;

private:
	/** @return number of weights (local to cpu) */
	int remap(Elt* elements, int nbElements, int order);

	void computeGrads();

	void computeIntersection(Elt* elements, int nbElements);

	/* For collecting the weights
	   FIXME put here as quasi global variables for the moment */
	int *srcOffset; /* for MPI */
	int *dstOffset; 
	int *srcLength; /* for MPI */
	int *dstLength; 

	bool write_weights;
	int verbose;
};

#endif
