SUBROUTINE TRACARE (PSLAC,PSLOC,PCLOC,PSLAR,PSLOR,PCLOR,KLEN,&
 & PSLAP,PCLAP,PSLOP,PCLOP,PFACDI,KTYP)  

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!                                                                      C
!     ROUTINE DE PASSAGE DE LA SPHERE DE CALCUL A LA SPHERE REELLE     C
!     ------- -- ------- -- -- ------ -- ------ - -- ------ ------     C
!                                                                      C
!           VERSION 1.0   J.-F. GELEYN   18/01/88  (CATORE)            C
!           MODIF 07/02/91 Y. BOUTELOUP ---->  SUPPRESSION DU COMMON   C
!           Modif 05/06/91 Y. Bouteloup ---->  Suppression de la       C
!                                              derniere rotation       C
!                                                                      C
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

!     FONCTION
!     --------

!     TRANSFORME UN VECTEUR DE LONGUEUR KLEN DE POINTS DE LA SPHERE
!  ASSOCIEE A LA GRILLE DE COLLOCATION DU MODELE EN UN VECTEUR DE MEME
!  LONGUEUR DE POINTS DE LA SPHERE REELLE. LES POINTS SONT CARACTERISES
!  SUR CHAQUE SPHERE PAR LE SINUS DE LA LATITUDE, LE SINUS DE LA
!  LONGITUDE ET LE COSINUS DE LA LONGITUDE.
!  LA SPHERE DE CALCUL EST DEFINIE DE LA MANIERE SUIVANTE : LE POLE DE
!  DILATATION EST SITUE A LA LONGITUDE PI SUR UN EQUATEUR TANGENT A LA
!  COURBE TRANSFORMEE DU PARALLELE REEL EN CE POINT, LES DEUX DIRECTIONS
!  POSITIVES (OUEST-EST) COINCIDANT.
!  LA TRANSFORMATION CONFORME DE SPHERE A SPHERE EST DEFINIE PAR LES
!  SINUS ET COSINUS DE LA LATITUDE ET DE LA LONGITUDE DU POLE DE
!  DILATATION ET PAR LE FACTEUR DE DILATATION (SUPERIEUR OU EGAL A UN).

!     VARIABLES D'ENTREE
!     --------- --------

!  PSLAC = SINUS DE LA PSEUDO-LATITUDE SUR LA SPHERE DE CALCUL
!  PSLOC = SINUS DE LA PSEUDO-LONGITUDE SUR LA SPHERE DE CALCUL
!  PCLOC = COSINUS DE LA PSEUDO-LONGITUDE SUR LA SPHERE DE CALCUL

!     VARIABLE DE CONTROLE
!     -------- -- --------

!  KLEN = LONGUEUR DES VECTEURS D'ENTREE ET DE SORTIE

!     VARIABLES DE SORTIE
!     --------- -- ------

!  PSLAR = SINUS DE LA LATITUDE REELLE
!  PSLOR = SINUS DE LA LONGITUDE REELLE
!  PCLOR = COSINUS DE LA LONGITUDE REELLE

!     PARAMETRES DE LA TRANSFORMATION
!     ---------- -- -- --------------

!  PSLAP  = SINUS DE LA LATITUDE REELLE DU POLE DE DILATATION
!  PCLAP  = COSINUS DE LA LATITUDE REELLE DU POLE DE DILATATION
!  PSLOP  = SINUS DE LA LONGITUDE REELLE DU POLE DE DILATATION
!  PCLOP  = COSINUS DE LA LONGITUDE REELLE DU POLE DE DILATATION
!  PFACDI = FACTEUR DE DILATATION
!  KTYP   = TYPE DE LA TRANSFORMATION   KTYP=1 -->  SCHMIDT SEULE
!                                       KTYP=2 -->  ARPEGE COMPLETE
!     CONSTANTE DE CALCUL
!     --------- -- ------

!  ZEPS = VALEUR DE SECURITE NUMERIQUE

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLAC(KLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLOC(KLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOC(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSLAR(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSLOR(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLOR(KLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLAP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLAP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLOP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFACDI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYP 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZA, ZB, ZC, ZC2M1, ZC2P1, ZCLAC, ZD, ZDBLC, ZE, ZEPS, ZF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('TRACARE',0,ZHOOK_HANDLE)
ZDBLC=2.0_JPRB*PFACDI
ZC2P1=PFACDI*PFACDI+1.0_JPRB
ZC2M1=PFACDI*PFACDI-1.0_JPRB
ZEPS=1.E-12_JPRB

!  -------------------------------------------------------
!       TRANSFORMATION ARPEGE COMPLETE

IF (KTYP == 2) THEN
  DO JL=1,KLEN
    ZCLAC=SQRT(1.0_JPRB-PSLAC(JL)*PSLAC(JL))
    ZA=1.0_JPRB/(ZC2P1+ZC2M1*PSLAC(JL))
    ZB=ZC2P1*PSLAC(JL)+ZC2M1
    ZC=ZDBLC*PCLAP*ZCLAC*PCLOC(JL)+ZB*PSLAP
    PSLAR(JL)=ZC*ZA
    ZD=ZA/SQRT(MAX(ZEPS,1.0_JPRB-PSLAR(JL)*PSLAR(JL)))
    ZE=ZB*PCLAP*PCLOP-ZDBLC*ZCLAC*(PSLAP*PCLOC(JL)*PCLOP-PSLOP*PSLOC(JL))
    ZF=ZB*PCLAP*PSLOP-ZDBLC*ZCLAC*(PSLAP*PCLOC(JL)*PSLOP+PCLOP*PSLOC(JL))
    PCLOR(JL)=ZE*ZD
    PSLOR(JL)=ZF*ZD
    PSLAR(JL)=MAX(-1.0_JPRB, MIN(1.0_JPRB,PSLAR(JL)))
    PCLOR(JL)=MAX(-1.0_JPRB, MIN(1.0_JPRB,PCLOR(JL)))
    PSLOR(JL)=MAX(-1.0_JPRB, MIN(1.0_JPRB,PSLOR(JL)))
  ENDDO

!     ------------------------------------------------------------------
!    TRANSFORMATION DE SCHMIDT SEULE

ELSEIF (KTYP == 1) THEN
  DO JL=1,KLEN
    PSLOR(JL)=PSLOC(JL)
    PCLOR(JL)=PCLOC(JL)
    PSLAR(JL)=(ZC2P1*PSLAC(JL)+ZC2M1)/(ZC2P1+ZC2M1*PSLAC(JL))
    PSLAR(JL)=MAX(-1.0_JPRB, MIN(1.0_JPRB,PSLAR(JL)))
  ENDDO
ELSE
  CALL ABOR1('TRACARE: ABOR1 CALLED')
ENDIF

IF (LHOOK) CALL DR_HOOK('TRACARE',1,ZHOOK_HANDLE)
END SUBROUTINE TRACARE

