SUBROUTINE UPDRGAS

!**** *UPDRGAS*

!     PURPOSE.
!     --------

!     Updates the concentrations of the uniformly mixed gases for the
!     radiative calculations, and also the solar forcing

!**   INTERFACE.
!     ----------

!     CALL UPDRGAS     from *UPDTIM*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===

! KINDAT

!     AUTHORS.
!     --------

!       JJMorcrette, ECMWF, 00/01/18

!       JJMorcrette 050119 revised time variations of uniformly mixed gases
!       T Stockdale 2010-04-27  CMIP5 solar forcing data

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM,    JPIB,     JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RDAY,  RI0
USE YOMCT0   , ONLY :     LTWOTL ,LNF 
USE YOMCT2   , ONLY : NSTAR2
USE YOMDYN   , ONLY : TSTEP
USE YOMLUN   , ONLY : NULOUT 
USE YOMRIP   , ONLY : NINDAT, NSTADD
USE YOERAD   , ONLY : LHGHG , NHINCSOL, NSCEN, &
 & LCMIP5, NCMIPFIXYR, NRCP, CMIP5DATADIR, &
 & LCMIP6, L1PCTCO2, LA4xCO2, CMIP6DATADIR, SSPNAME 
USE YOERDI   , ONLY : RCARDI, RCH4  , RN2O  , RNO2  , RCFC11, RCFC12, RSOLINC
USE YOEMETH  , ONLY : RQLIM

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IDY0, IDY, IMN0, IMN, INDSC &
 &, IYR0, IYR, IYR1, IYR2, IRFY1, IRFY2
INTEGER(KIND=JPIM) :: ISTADD, ITIME
INTEGER(KIND=JPIB) :: IZT
INTEGER(KIND=JPIM) :: ILMONTH(12)

INTEGER(KIND=JPIM), SAVE :: IYR2OLD=-1

!REAL_B :: ZAIRMWG, ZCO2MWG, ZCH4MWG , ZN2OMWG , ZNO2MWG , ZC11MWG , ZC12MWG
REAL(KIND=JPRB) :: ZCO2    , ZCH4    , ZN2O    , ZNO2    , ZC11    , ZC12
REAL(KIND=JPRB) :: ZCO2RMWG, ZCH4RMWG, ZN2ORMWG, ZNO2RMWG, ZC11RMWG, ZC12RMWG
REAL(KIND=JPRB) :: ZCO2REF , ZCH4REF , ZN2OREF , ZNO2REF , ZC11REF , ZC12REF
REAL(KIND=JPRB) :: ZCO2CHG , ZCH4CHG , ZN2OCHG , ZNO2CHG , ZC11CHG , ZC12CHG
REAL(KIND=JPRB) :: ZCO2MCHG, ZCH4MCHG, ZN2OMCHG, ZNO2MCHG, ZC11MCHG, ZC12MCHG

INTEGER(KIND=JPIM) :: JGAS
REAL(KIND=JPRB) :: ZHVAR(151,5), ZFVAR( 101,3,5), ZINCSOL(154), ZINSOLM(1848)
REAL(KIND=JPRB) :: ZCMIP5SOL(1851:2008)
REAL(KIND=JPRB) :: ZFIXNO2, ZCONC(6), ZGRADC, ZSOLINM, ZSOLINY

!EC-EARTH: For CMIP5 and CMIP6 runs
REAL(KIND=JPRB), SAVE :: ZSOLYEAR(399), ZSOLMONTH(5400)
REAL(KIND=JPRB), SAVE :: ZFCONC(2,5)=0._JPRB
LOGICAL, SAVE         :: FIRST_CALL = .TRUE.
LOGICAL               :: LREAD 
INTEGER(KIND=JPIM)    :: IYR_CMIP5

! For CMIP6 DECK
INTEGER(KIND=JPIM) :: IYR_4CO2, IYR1_4CO2, IYR2_4CO2
! Increase rate of co2 per year in 1pctCO2 experiment
REAL(KIND=JPRB),PARAMETER :: RCO2INC=0.01_JPRB 
! co2 level of co2 the 1pctCO2/Abrupt4xCO2 experiment
REAL(KIND=JPRB),PARAMETER :: R4xCO2=4.0_JPRB 
! Reference volume concentrations in D.Cariolle's climatology taken from su_ghgclim.F90
REAL(KIND=JPRB) :: ZGCO2Y1, ZGCO2Y2
! reference volume concentrations for CH4 at the surface (taken from SU_GHGCLIM.F90)
! RCH4_CL=1.78 ppmv
REAL(KIND=JPRB),PARAMETER :: RCH4_CL = 1.78_JPRB 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "netcdf.inc"
#include "updcal.intfb.h"

#include "fcttim.h"

IF (LHOOK) CALL DR_HOOK('UPDRGAS',0,ZHOOK_HANDLE)

!-- concentrations for CO2, CH4, N2O, CFC11, CFC12 for 1850-2000

ZHVAR( :, 1) = (/ &
  &  284.70_JPRB, 284.90_JPRB, 285.00_JPRB, 285.10_JPRB, 285.30_JPRB&
  &, 285.40_JPRB, 285.60_JPRB, 285.70_JPRB, 285.90_JPRB, 286.10_JPRB&
  &, 286.20_JPRB, 286.40_JPRB, 286.50_JPRB, 286.60_JPRB, 286.80_JPRB&
  &, 286.90_JPRB, 287.00_JPRB, 287.10_JPRB, 287.20_JPRB, 287.40_JPRB&
  &, 287.50_JPRB, 287.70_JPRB, 287.90_JPRB, 288.10_JPRB, 288.40_JPRB&
  &, 288.70_JPRB, 289.00_JPRB, 289.40_JPRB, 289.80_JPRB, 290.20_JPRB&
  &, 290.70_JPRB, 291.20_JPRB, 291.70_JPRB, 292.10_JPRB, 292.60_JPRB&
  &, 293.00_JPRB, 293.30_JPRB, 293.60_JPRB, 293.80_JPRB, 294.00_JPRB&
  &, 294.20_JPRB, 294.30_JPRB, 294.50_JPRB, 294.60_JPRB, 294.70_JPRB&
  &, 294.80_JPRB, 294.90_JPRB, 295.00_JPRB, 295.20_JPRB, 295.50_JPRB&
  &, 295.80_JPRB, 296.10_JPRB, 296.50_JPRB, 296.80_JPRB, 297.20_JPRB&
  &, 297.60_JPRB, 298.10_JPRB, 298.50_JPRB, 298.90_JPRB, 299.30_JPRB&
  &, 299.70_JPRB, 300.10_JPRB, 300.40_JPRB, 300.80_JPRB, 301.10_JPRB&
  &, 301.40_JPRB, 301.70_JPRB, 302.10_JPRB, 302.40_JPRB, 302.70_JPRB&
  &, 303.00_JPRB, 303.40_JPRB, 303.80_JPRB, 304.10_JPRB, 304.50_JPRB&
  &, 305.00_JPRB, 305.40_JPRB, 305.80_JPRB, 306.30_JPRB, 306.80_JPRB&
  &, 307.20_JPRB, 307.70_JPRB, 308.20_JPRB, 308.60_JPRB, 309.00_JPRB&
  &, 309.40_JPRB, 309.80_JPRB, 310.00_JPRB, 310.20_JPRB, 310.30_JPRB&
  &, 310.40_JPRB, 310.40_JPRB, 310.30_JPRB, 310.20_JPRB, 310.10_JPRB&
  &, 310.10_JPRB, 310.10_JPRB, 310.20_JPRB, 310.30_JPRB, 310.50_JPRB&
  &, 310.70_JPRB, 311.10_JPRB, 311.50_JPRB, 311.90_JPRB, 312.40_JPRB&
  &, 313.00_JPRB, 313.60_JPRB, 314.20_JPRB, 314.90_JPRB, 315.60_JPRB&
  &, 316.30_JPRB, 317.00_JPRB, 317.70_JPRB, 318.40_JPRB, 319.20_JPRB&
  &, 320.00_JPRB, 320.80_JPRB, 321.80_JPRB, 322.80_JPRB, 323.80_JPRB&
  &, 324.80_JPRB, 325.84_JPRB, 326.92_JPRB, 328.04_JPRB, 329.21_JPRB&
  &, 330.42_JPRB, 331.66_JPRB, 332.95_JPRB, 334.26_JPRB, 335.62_JPRB&
  &, 337.00_JPRB, 338.41_JPRB, 339.86_JPRB, 341.32_JPRB, 342.81_JPRB&
  &, 344.31_JPRB, 345.83_JPRB, 347.36_JPRB, 348.91_JPRB, 350.45_JPRB&
  &, 352.00_JPRB, 353.55_JPRB, 355.09_JPRB, 356.63_JPRB, 358.16_JPRB&
  &, 359.68_JPRB, 361.18_JPRB, 362.67_JPRB, 364.14_JPRB, 365.58_JPRB&
  &, 367.00_JPRB /)
ZHVAR( :, 2) = (/ &
  &  791.60_JPRB, 792.93_JPRB, 794.27_JPRB, 795.63_JPRB, 797.00_JPRB&
  &, 798.39_JPRB, 799.79_JPRB, 801.22_JPRB, 802.66_JPRB, 804.12_JPRB&
  &, 805.60_JPRB, 807.10_JPRB, 808.62_JPRB, 810.16_JPRB, 811.71_JPRB&
  &, 813.26_JPRB, 814.82_JPRB, 816.37_JPRB, 817.92_JPRB, 819.47_JPRB&
  &, 821.00_JPRB, 822.52_JPRB, 824.03_JPRB, 825.55_JPRB, 827.08_JPRB&
  &, 828.63_JPRB, 830.22_JPRB, 831.85_JPRB, 833.54_JPRB, 835.28_JPRB&
  &, 837.10_JPRB, 838.99_JPRB, 840.95_JPRB, 842.95_JPRB, 844.98_JPRB&
  &, 847.01_JPRB, 849.03_JPRB, 851.02_JPRB, 852.96_JPRB, 854.82_JPRB&
  &, 856.60_JPRB, 858.28_JPRB, 859.93_JPRB, 861.59_JPRB, 863.34_JPRB&
  &, 865.25_JPRB, 867.38_JPRB, 869.79_JPRB, 872.56_JPRB, 875.74_JPRB&
  &, 879.40_JPRB, 883.54_JPRB, 887.90_JPRB, 892.21_JPRB, 896.50_JPRB&
  &, 900.86_JPRB, 905.30_JPRB, 909.80_JPRB, 914.40_JPRB, 919.15_JPRB&
  &, 924.00_JPRB, 928.90_JPRB, 933.90_JPRB, 939.04_JPRB, 944.30_JPRB&
  &, 949.65_JPRB, 955.10_JPRB, 960.69_JPRB, 966.40_JPRB, 972.22_JPRB&
  &, 978.10_JPRB, 983.99_JPRB, 989.90_JPRB, 995.84_JPRB,1001.80_JPRB&
  &,1007.74_JPRB,1013.60_JPRB,1019.34_JPRB,1025.00_JPRB,1030.62_JPRB&
  &,1036.20_JPRB,1041.73_JPRB,1047.20_JPRB,1052.61_JPRB,1058.00_JPRB&
  &,1063.39_JPRB,1068.70_JPRB,1073.86_JPRB,1078.90_JPRB,1083.90_JPRB&
  &,1088.90_JPRB,1093.93_JPRB,1099.00_JPRB,1104.14_JPRB,1109.40_JPRB&
  &,1114.86_JPRB,1120.60_JPRB,1126.70_JPRB,1133.20_JPRB,1140.12_JPRB&
  &,1147.50_JPRB,1155.40_JPRB,1163.80_JPRB,1172.69_JPRB,1182.10_JPRB&
  &,1192.03_JPRB,1202.40_JPRB,1213.12_JPRB,1224.20_JPRB,1235.67_JPRB&
  &,1247.50_JPRB,1259.67_JPRB,1272.20_JPRB,1285.12_JPRB,1298.40_JPRB&
  &,1312.01_JPRB,1326.00_JPRB,1340.44_JPRB,1355.30_JPRB,1370.51_JPRB&
  &,1386.00_JPRB,1401.69_JPRB,1417.50_JPRB,1433.36_JPRB,1449.30_JPRB&
  &,1465.36_JPRB,1481.50_JPRB,1497.70_JPRB,1514.00_JPRB,1530.45_JPRB&
  &,1547.10_JPRB,1563.96_JPRB,1580.90_JPRB,1597.74_JPRB,1614.20_JPRB&
  &,1629.99_JPRB,1644.90_JPRB,1658.77_JPRB,1671.60_JPRB,1683.43_JPRB&
  &,1694.30_JPRB,1704.26_JPRB,1713.35_JPRB,1721.61_JPRB,1729.09_JPRB&
  &,1735.84_JPRB,1741.88_JPRB,1747.28_JPRB,1752.07_JPRB,1756.30_JPRB&
  &,1760.00_JPRB /)
ZHVAR( :, 3) = (/ &
  &  275.68_JPRB, 275.79_JPRB, 275.90_JPRB, 276.01_JPRB, 276.11_JPRB&
  &, 276.22_JPRB, 276.32_JPRB, 276.41_JPRB, 276.51_JPRB, 276.60_JPRB&
  &, 276.69_JPRB, 276.77_JPRB, 276.86_JPRB, 276.94_JPRB, 277.02_JPRB&
  &, 277.09_JPRB, 277.17_JPRB, 277.24_JPRB, 277.31_JPRB, 277.38_JPRB&
  &, 277.44_JPRB, 277.51_JPRB, 277.58_JPRB, 277.64_JPRB, 277.71_JPRB&
  &, 277.78_JPRB, 277.85_JPRB, 277.92_JPRB, 277.99_JPRB, 278.07_JPRB&
  &, 278.14_JPRB, 278.22_JPRB, 278.29_JPRB, 278.36_JPRB, 278.43_JPRB&
  &, 278.50_JPRB, 278.57_JPRB, 278.63_JPRB, 278.70_JPRB, 278.76_JPRB&
  &, 278.83_JPRB, 278.90_JPRB, 278.97_JPRB, 279.04_JPRB, 279.12_JPRB&
  &, 279.20_JPRB, 279.28_JPRB, 279.36_JPRB, 279.45_JPRB, 279.55_JPRB&
  &, 279.64_JPRB, 279.75_JPRB, 279.86_JPRB, 279.98_JPRB, 280.11_JPRB&
  &, 280.25_JPRB, 280.41_JPRB, 280.58_JPRB, 280.76_JPRB, 280.96_JPRB&
  &, 281.16_JPRB, 281.38_JPRB, 281.60_JPRB, 281.82_JPRB, 282.05_JPRB&
  &, 282.28_JPRB, 282.50_JPRB, 282.73_JPRB, 282.96_JPRB, 283.19_JPRB&
  &, 283.42_JPRB, 283.65_JPRB, 283.89_JPRB, 284.13_JPRB, 284.37_JPRB&
  &, 284.60_JPRB, 284.83_JPRB, 285.05_JPRB, 285.26_JPRB, 285.46_JPRB&
  &, 285.66_JPRB, 285.85_JPRB, 286.04_JPRB, 286.23_JPRB, 286.42_JPRB&
  &, 286.62_JPRB, 286.83_JPRB, 287.04_JPRB, 287.26_JPRB, 287.49_JPRB&
  &, 287.72_JPRB, 287.95_JPRB, 288.17_JPRB, 288.40_JPRB, 288.62_JPRB&
  &, 288.84_JPRB, 289.06_JPRB, 289.28_JPRB, 289.51_JPRB, 289.75_JPRB&
  &, 289.99_JPRB, 290.24_JPRB, 290.49_JPRB, 290.75_JPRB, 291.01_JPRB&
  &, 291.27_JPRB, 291.53_JPRB, 291.80_JPRB, 292.06_JPRB, 292.34_JPRB&
  &, 292.62_JPRB, 292.91_JPRB, 293.21_JPRB, 293.53_JPRB, 293.86_JPRB&
  &, 294.21_JPRB, 294.58_JPRB, 294.97_JPRB, 295.38_JPRB, 295.80_JPRB&
  &, 296.25_JPRB, 296.71_JPRB, 297.18_JPRB, 297.68_JPRB, 298.19_JPRB&
  &, 298.72_JPRB, 299.26_JPRB, 299.83_JPRB, 300.41_JPRB, 301.01_JPRB&
  &, 301.63_JPRB, 302.26_JPRB, 302.91_JPRB, 303.57_JPRB, 304.24_JPRB&
  &, 304.92_JPRB, 305.61_JPRB, 306.32_JPRB, 307.03_JPRB, 307.74_JPRB&
  &, 308.47_JPRB, 309.20_JPRB, 309.94_JPRB, 310.68_JPRB, 311.43_JPRB&
  &, 312.19_JPRB, 312.95_JPRB, 313.71_JPRB, 314.47_JPRB, 315.23_JPRB&
  &, 316.00_JPRB /)
ZHVAR( :, 4) = (/ &
  &   12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB,  12.48_JPRB&
  &,  12.49_JPRB,  12.49_JPRB,  12.49_JPRB,  12.49_JPRB,  12.49_JPRB&
  &,  12.49_JPRB,  12.52_JPRB,  12.55_JPRB,  12.55_JPRB,  12.58_JPRB&
  &,  12.65_JPRB,  12.73_JPRB,  12.86_JPRB,  12.97_JPRB,  13.05_JPRB&
  &,  13.13_JPRB,  13.19_JPRB,  13.22_JPRB,  13.35_JPRB,  13.46_JPRB&
  &,  13.60_JPRB,  13.77_JPRB,  13.94_JPRB,  14.10_JPRB,  14.32_JPRB&
  &,  14.61_JPRB,  14.94_JPRB,  15.24_JPRB,  15.56_JPRB,  15.94_JPRB&
  &,  16.43_JPRB,  16.96_JPRB,  17.63_JPRB,  18.33_JPRB,  19.00_JPRB&
  &,  19.75_JPRB,  20.68_JPRB,  21.79_JPRB,  23.14_JPRB,  24.68_JPRB&
  &,  26.21_JPRB,  27.38_JPRB,  28.56_JPRB,  30.03_JPRB,  31.45_JPRB&
  &,  32.77_JPRB,  34.23_JPRB,  35.69_JPRB,  37.15_JPRB,  38.62_JPRB&
  &,  40.20_JPRB,  41.94_JPRB,  43.77_JPRB,  45.86_JPRB,  48.04_JPRB&
  &,  50.58_JPRB,  53.61_JPRB,  57.29_JPRB,  61.88_JPRB,  67.44_JPRB&
  &,  74.28_JPRB,  82.87_JPRB,  93.23_JPRB, 105.01_JPRB, 118.58_JPRB&
  &, 134.03_JPRB, 150.48_JPRB, 167.23_JPRB, 185.12_JPRB, 205.26_JPRB&
  &, 225.68_JPRB, 246.66_JPRB, 268.29_JPRB, 289.50_JPRB, 309.65_JPRB&
  &, 329.77_JPRB, 349.14_JPRB, 368.27_JPRB, 388.14_JPRB, 409.55_JPRB&
  &, 434.11_JPRB, 458.86_JPRB, 487.47_JPRB, 515.98_JPRB, 539.82_JPRB&
  &, 563.29_JPRB, 583.20_JPRB, 599.03_JPRB, 611.20_JPRB, 620.16_JPRB&
  &, 626.82_JPRB, 631.77_JPRB, 635.83_JPRB, 640.05_JPRB, 645.56_JPRB&
  &, 653.45_JPRB /)
ZHVAR( :, 5) = (/ &
  &    0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB,   0.00_JPRB&
  &,   0.00_JPRB,   0.05_JPRB,   0.10_JPRB,   0.15_JPRB,   0.25_JPRB&
  &,   0.35_JPRB,   0.45_JPRB,   0.60_JPRB,   0.80_JPRB,   1.05_JPRB&
  &,   1.45_JPRB,   2.00_JPRB,   2.90_JPRB,   4.10_JPRB,   5.40_JPRB&
  &,   6.80_JPRB,   8.30_JPRB,  10.00_JPRB,  11.75_JPRB,  13.80_JPRB&
  &,  16.05_JPRB,  18.65_JPRB,  21.65_JPRB,  24.90_JPRB,  28.40_JPRB&
  &,  32.55_JPRB,  37.20_JPRB,  42.60_JPRB,  48.80_JPRB,  56.10_JPRB&
  &,  64.40_JPRB,  73.65_JPRB,  84.10_JPRB,  95.80_JPRB, 108.85_JPRB&
  &, 123.35_JPRB, 138.95_JPRB, 155.85_JPRB, 174.45_JPRB, 194.80_JPRB&
  &, 215.50_JPRB, 235.55_JPRB, 254.75_JPRB, 272.65_JPRB, 289.70_JPRB&
  &, 307.65_JPRB, 324.20_JPRB, 339.40_JPRB, 355.75_JPRB, 372.55_JPRB&
  &, 390.80_JPRB, 410.40_JPRB, 430.15_JPRB, 450.60_JPRB, 469.00_JPRB&
  &, 485.20_JPRB, 498.64_JPRB, 509.37_JPRB, 517.69_JPRB, 523.94_JPRB&
  &, 528.40_JPRB, 531.40_JPRB, 533.25_JPRB, 534.26_JPRB, 534.74_JPRB&
  &, 535.00_JPRB /)

!-- scenario A1B 2000-2100
ZFVAR( :, 1, 1) = (/ &
!  &  352.00_JPRB, 353.55_JPRB, 355.09_JPRB, 356.63_JPRB, 358.16_JPRB&
!  &, 359.68_JPRB, 361.18_JPRB, 362.67_JPRB, 364.14_JPRB, 365.58_JPRB&
  &  367.00_JPRB, 368.72_JPRB, 370.50_JPRB, 372.35_JPRB, 374.28_JPRB&
  &, 376.30_JPRB, 378.41_JPRB, 380.62_JPRB, 382.95_JPRB, 385.41_JPRB&
  &, 388.00_JPRB, 390.73_JPRB, 393.58_JPRB, 396.52_JPRB, 399.54_JPRB&
  &, 402.62_JPRB, 405.72_JPRB, 408.84_JPRB, 411.94_JPRB, 415.00_JPRB&
  &, 418.00_JPRB, 420.93_JPRB, 423.79_JPRB, 426.62_JPRB, 429.43_JPRB&
  &, 432.23_JPRB, 435.06_JPRB, 437.93_JPRB, 440.86_JPRB, 443.88_JPRB&
  &, 447.00_JPRB, 450.24_JPRB, 453.59_JPRB, 457.05_JPRB, 460.59_JPRB&
  &, 464.20_JPRB, 467.88_JPRB, 471.62_JPRB, 475.39_JPRB, 479.19_JPRB&
  &, 483.00_JPRB, 486.82_JPRB, 490.64_JPRB, 494.48_JPRB, 498.33_JPRB&
  &, 502.20_JPRB, 506.10_JPRB, 510.02_JPRB, 513.98_JPRB, 517.97_JPRB&
  &, 522.00_JPRB, 526.07_JPRB, 530.18_JPRB, 534.32_JPRB, 538.46_JPRB&
  &, 542.61_JPRB, 546.75_JPRB, 550.87_JPRB, 554.96_JPRB, 559.00_JPRB&
  &, 563.00_JPRB, 566.94_JPRB, 570.82_JPRB, 574.66_JPRB, 578.46_JPRB&
  &, 582.24_JPRB, 585.99_JPRB, 589.74_JPRB, 593.48_JPRB, 597.23_JPRB&
  &, 601.00_JPRB, 604.79_JPRB, 608.60_JPRB, 612.42_JPRB, 616.25_JPRB&
  &, 620.07_JPRB, 623.90_JPRB, 627.70_JPRB, 631.50_JPRB, 635.26_JPRB&
  &, 639.00_JPRB, 642.70_JPRB, 646.36_JPRB, 649.98_JPRB, 653.56_JPRB&
  &, 657.10_JPRB, 660.58_JPRB, 664.02_JPRB, 667.40_JPRB, 670.73_JPRB&
  &, 674.00_JPRB, 677.21_JPRB, 680.37_JPRB, 683.45_JPRB, 686.47_JPRB&
  &, 689.42_JPRB, 692.30_JPRB, 695.09_JPRB, 697.81_JPRB, 700.45_JPRB&
  &, 703.00_JPRB /)
ZFVAR( :, 1, 2) = (/ &
!  & 1694.30_JPRB,1704.26_JPRB,1713.35_JPRB,1721.61_JPRB,1729.09_JPRB&
!  &,1735.84_JPRB,1741.88_JPRB,1747.28_JPRB,1752.07_JPRB,1756.30_JPRB&
  & 1760.00_JPRB,1768.91_JPRB,1778.32_JPRB,1788.23_JPRB,1798.63_JPRB&
  &,1809.51_JPRB,1820.87_JPRB,1832.71_JPRB,1845.01_JPRB,1857.78_JPRB&
  &,1871.00_JPRB,1884.67_JPRB,1898.78_JPRB,1913.32_JPRB,1928.27_JPRB&
  &,1943.63_JPRB,1959.37_JPRB,1975.49_JPRB,1991.98_JPRB,2008.82_JPRB&
  &,2026.00_JPRB,2043.50_JPRB,2061.24_JPRB,2079.15_JPRB,2097.13_JPRB&
  &,2115.11_JPRB,2133.00_JPRB,2150.71_JPRB,2168.18_JPRB,2185.30_JPRB&
  &,2202.00_JPRB,2218.20_JPRB,2233.88_JPRB,2248.99_JPRB,2263.52_JPRB&
  &,2277.44_JPRB,2290.73_JPRB,2303.35_JPRB,2315.29_JPRB,2326.51_JPRB&
  &,2337.00_JPRB,2346.72_JPRB,2355.69_JPRB,2363.88_JPRB,2371.32_JPRB&
  &,2378.00_JPRB,2383.91_JPRB,2389.07_JPRB,2393.47_JPRB,2397.11_JPRB&
  &,2400.00_JPRB,2402.13_JPRB,2403.50_JPRB,2404.10_JPRB,2403.91_JPRB&
  &,2402.95_JPRB,2401.18_JPRB,2398.61_JPRB,2395.23_JPRB,2391.03_JPRB&
  &,2386.00_JPRB,2380.15_JPRB,2373.52_JPRB,2366.20_JPRB,2358.24_JPRB&
  &,2349.72_JPRB,2340.70_JPRB,2331.25_JPRB,2321.44_JPRB,2311.33_JPRB&
  &,2301.00_JPRB,2290.50_JPRB,2279.85_JPRB,2269.07_JPRB,2258.17_JPRB&
  &,2247.17_JPRB,2236.07_JPRB,2224.89_JPRB,2213.64_JPRB,2202.34_JPRB&
  &,2191.00_JPRB,2179.63_JPRB,2168.25_JPRB,2156.86_JPRB,2145.48_JPRB&
  &,2134.11_JPRB,2122.78_JPRB,2111.49_JPRB,2100.26_JPRB,2089.09_JPRB&
  &,2078.00_JPRB,2067.00_JPRB,2056.10_JPRB,2045.31_JPRB,2034.65_JPRB&
  &,2024.13_JPRB,2013.76_JPRB,2003.55_JPRB,1993.51_JPRB,1983.66_JPRB&
  &,1974.00_JPRB /)
ZFVAR( :, 1, 3) = (/ &
!  &  308.47_JPRB, 309.20_JPRB, 309.94_JPRB, 310.68_JPRB, 311.43_JPRB&
!  &, 312.19_JPRB, 312.95_JPRB, 313.71_JPRB, 314.47_JPRB, 315.23_JPRB&
  &  316.00_JPRB, 316.82_JPRB, 317.65_JPRB, 318.47_JPRB, 319.29_JPRB&
  &, 320.10_JPRB, 320.90_JPRB, 321.70_JPRB, 322.48_JPRB, 323.25_JPRB&
  &, 324.00_JPRB, 324.74_JPRB, 325.46_JPRB, 326.16_JPRB, 326.86_JPRB&
  &, 327.55_JPRB, 328.24_JPRB, 328.93_JPRB, 329.61_JPRB, 330.30_JPRB&
  &, 331.00_JPRB, 331.70_JPRB, 332.41_JPRB, 333.13_JPRB, 333.84_JPRB&
  &, 334.56_JPRB, 335.26_JPRB, 335.97_JPRB, 336.66_JPRB, 337.34_JPRB&
  &, 338.00_JPRB, 338.65_JPRB, 339.28_JPRB, 339.90_JPRB, 340.50_JPRB&
  &, 341.10_JPRB, 341.69_JPRB, 342.27_JPRB, 342.85_JPRB, 343.42_JPRB&
  &, 344.00_JPRB, 344.58_JPRB, 345.16_JPRB, 345.74_JPRB, 346.33_JPRB&
  &, 346.93_JPRB, 347.53_JPRB, 348.13_JPRB, 348.75_JPRB, 349.37_JPRB&
  &, 350.00_JPRB, 350.64_JPRB, 351.29_JPRB, 351.93_JPRB, 352.57_JPRB&
  &, 353.20_JPRB, 353.81_JPRB, 354.40_JPRB, 354.97_JPRB, 355.50_JPRB&
  &, 356.00_JPRB, 356.46_JPRB, 356.88_JPRB, 357.28_JPRB, 357.66_JPRB&
  &, 358.03_JPRB, 358.40_JPRB, 358.77_JPRB, 359.16_JPRB, 359.57_JPRB&
  &, 360.00_JPRB, 360.47_JPRB, 360.96_JPRB, 361.48_JPRB, 362.01_JPRB&
  &, 362.54_JPRB, 363.07_JPRB, 363.59_JPRB, 364.09_JPRB, 364.56_JPRB&
  &, 365.00_JPRB, 365.40_JPRB, 365.76_JPRB, 366.09_JPRB, 366.39_JPRB&
  &, 366.68_JPRB, 366.95_JPRB, 367.21_JPRB, 367.47_JPRB, 367.73_JPRB&
  &, 368.00_JPRB, 368.29_JPRB, 368.59_JPRB, 368.91_JPRB, 369.26_JPRB&
  &, 369.63_JPRB, 370.03_JPRB, 370.47_JPRB, 370.94_JPRB, 371.45_JPRB&
  &, 372.00_JPRB /)
ZFVAR( :, 1, 4) = (/ &
!  &  563.29_JPRB, 583.20_JPRB, 599.03_JPRB, 611.20_JPRB, 620.16_JPRB&
!  &, 626.82_JPRB, 631.77_JPRB, 635.83_JPRB, 640.05_JPRB, 645.56_JPRB&
  &  653.45_JPRB, 664.78_JPRB, 678.74_JPRB, 694.18_JPRB, 710.01_JPRB&
  &, 725.16_JPRB, 738.77_JPRB, 750.83_JPRB, 761.34_JPRB, 770.48_JPRB&
  &, 778.32_JPRB, 784.85_JPRB, 790.16_JPRB, 794.13_JPRB, 796.72_JPRB&
  &, 797.79_JPRB, 797.36_JPRB, 795.67_JPRB, 792.98_JPRB, 789.58_JPRB&
  &, 785.76_JPRB, 781.83_JPRB, 777.92_JPRB, 774.31_JPRB, 771.15_JPRB&
  &, 768.66_JPRB, 767.04_JPRB, 766.25_JPRB, 766.34_JPRB, 767.26_JPRB&
  &, 769.05_JPRB, 771.68_JPRB, 775.18_JPRB, 779.58_JPRB, 784.86_JPRB&
  &, 791.11_JPRB, 798.27_JPRB, 806.26_JPRB, 814.96_JPRB, 824.24_JPRB&
  &, 834.00_JPRB, 844.22_JPRB, 854.72_JPRB, 865.54_JPRB, 876.62_JPRB&
  &, 887.99_JPRB, 899.57_JPRB, 911.44_JPRB, 923.57_JPRB, 936.02_JPRB&
  &, 948.81_JPRB, 961.69_JPRB, 974.90_JPRB, 988.34_JPRB,1001.93_JPRB&
  &,1015.62_JPRB,1029.31_JPRB,1042.96_JPRB,1056.54_JPRB,1069.99_JPRB&
  &,1083.27_JPRB,1096.57_JPRB,1109.65_JPRB,1122.39_JPRB,1134.78_JPRB&
  &,1146.79_JPRB,1158.31_JPRB,1169.35_JPRB,1179.87_JPRB,1189.81_JPRB&
  &,1199.12_JPRB,1207.81_JPRB,1215.87_JPRB,1223.39_JPRB,1230.40_JPRB&
  &,1236.95_JPRB,1243.10_JPRB,1248.90_JPRB,1254.39_JPRB,1259.62_JPRB&
  &,1264.61_JPRB,1269.41_JPRB,1274.05_JPRB,1278.45_JPRB,1282.68_JPRB&
  &,1286.68_JPRB,1290.50_JPRB,1294.08_JPRB,1297.43_JPRB,1300.56_JPRB&
  &,1303.42_JPRB,1305.92_JPRB,1308.12_JPRB,1310.05_JPRB,1311.77_JPRB&
  &,1313.14_JPRB,1314.25_JPRB,1315.08_JPRB,1315.65_JPRB,1315.92_JPRB&
  &,1315.93_JPRB /)
ZFVAR( :, 1, 5) = (/ &
!  &  485.20_JPRB, 498.64_JPRB, 509.37_JPRB, 517.69_JPRB, 523.94_JPRB&
!  &, 528.40_JPRB, 531.40_JPRB, 533.25_JPRB, 534.26_JPRB, 534.74_JPRB&
  &  535.00_JPRB, 535.29_JPRB, 535.58_JPRB, 535.76_JPRB, 535.73_JPRB&
  &, 535.40_JPRB, 534.68_JPRB, 533.53_JPRB, 531.93_JPRB, 529.86_JPRB&
  &, 527.30_JPRB, 524.25_JPRB, 520.76_JPRB, 516.92_JPRB, 512.81_JPRB&
  &, 508.50_JPRB, 504.07_JPRB, 499.55_JPRB, 494.96_JPRB, 490.34_JPRB&
  &, 485.70_JPRB, 481.07_JPRB, 476.46_JPRB, 471.87_JPRB, 467.32_JPRB&
  &, 462.80_JPRB, 458.32_JPRB, 453.89_JPRB, 449.49_JPRB, 445.13_JPRB&
  &, 440.80_JPRB, 436.51_JPRB, 432.25_JPRB, 428.03_JPRB, 423.84_JPRB&
  &, 419.70_JPRB, 415.60_JPRB, 411.54_JPRB, 407.52_JPRB, 403.54_JPRB&
  &, 399.60_JPRB, 395.70_JPRB, 391.85_JPRB, 388.03_JPRB, 384.25_JPRB&
  &, 380.50_JPRB, 376.79_JPRB, 373.11_JPRB, 369.47_JPRB, 365.87_JPRB&
  &, 362.30_JPRB, 358.77_JPRB, 355.28_JPRB, 351.82_JPRB, 348.39_JPRB&
  &, 345.00_JPRB, 341.64_JPRB, 338.30_JPRB, 335.00_JPRB, 331.73_JPRB&
  &, 328.50_JPRB, 325.30_JPRB, 322.13_JPRB, 318.99_JPRB, 315.88_JPRB&
  &, 312.80_JPRB, 309.74_JPRB, 306.71_JPRB, 303.71_JPRB, 300.74_JPRB&
  &, 297.80_JPRB, 294.90_JPRB, 292.03_JPRB, 289.20_JPRB, 286.39_JPRB&
  &, 283.60_JPRB, 280.83_JPRB, 278.09_JPRB, 275.36_JPRB, 272.67_JPRB&
  &, 270.00_JPRB, 267.36_JPRB, 264.76_JPRB, 262.18_JPRB, 259.63_JPRB&
  &, 257.10_JPRB, 254.59_JPRB, 252.11_JPRB, 249.65_JPRB, 247.21_JPRB&
  &, 244.80_JPRB, 242.41_JPRB, 240.05_JPRB, 237.71_JPRB, 235.39_JPRB&
  &, 233.10_JPRB, 230.83_JPRB, 228.59_JPRB, 226.37_JPRB, 224.17_JPRB&
  &, 222.00_JPRB /)

!-- scenario A2
ZFVAR( :, 2, 1) = (/ &
!  &  352.00_JPRB, 353.55_JPRB, 355.09_JPRB, 356.63_JPRB, 358.16_JPRB&
!  &, 359.68_JPRB, 361.18_JPRB, 362.67_JPRB, 364.14_JPRB, 365.58_JPRB&
  &  367.00_JPRB, 368.59_JPRB, 370.22_JPRB, 371.90_JPRB, 373.65_JPRB&
  &, 375.46_JPRB, 377.36_JPRB, 379.36_JPRB, 381.45_JPRB, 383.66_JPRB&
  &, 386.00_JPRB, 388.47_JPRB, 391.05_JPRB, 393.74_JPRB, 396.51_JPRB&
  &, 399.35_JPRB, 402.24_JPRB, 405.17_JPRB, 408.12_JPRB, 411.07_JPRB&
  &, 414.00_JPRB, 416.91_JPRB, 419.80_JPRB, 422.69_JPRB, 425.59_JPRB&
  &, 428.52_JPRB, 431.48_JPRB, 434.50_JPRB, 437.58_JPRB, 440.74_JPRB&
  &, 444.00_JPRB, 447.36_JPRB, 450.82_JPRB, 454.37_JPRB, 458.01_JPRB&
  &, 461.71_JPRB, 465.48_JPRB, 469.30_JPRB, 473.17_JPRB, 477.07_JPRB&
  &, 481.00_JPRB, 484.95_JPRB, 488.92_JPRB, 492.92_JPRB, 496.95_JPRB&
  &, 501.02_JPRB, 505.12_JPRB, 509.27_JPRB, 513.46_JPRB, 517.70_JPRB&
  &, 522.00_JPRB, 526.36_JPRB, 530.77_JPRB, 535.24_JPRB, 539.76_JPRB&
  &, 544.34_JPRB, 548.97_JPRB, 553.65_JPRB, 558.39_JPRB, 563.17_JPRB&
  &, 568.00_JPRB, 572.88_JPRB, 577.81_JPRB, 582.80_JPRB, 587.86_JPRB&
  &, 592.99_JPRB, 598.21_JPRB, 603.51_JPRB, 608.90_JPRB, 614.40_JPRB&
  &, 620.00_JPRB, 625.72_JPRB, 631.54_JPRB, 637.48_JPRB, 643.53_JPRB&
  &, 649.68_JPRB, 655.94_JPRB, 662.31_JPRB, 668.77_JPRB, 675.34_JPRB&
  &, 682.00_JPRB, 688.76_JPRB, 695.62_JPRB, 702.57_JPRB, 709.62_JPRB&
  &, 716.77_JPRB, 724.02_JPRB, 731.36_JPRB, 738.81_JPRB, 746.35_JPRB&
  &, 754.00_JPRB, 761.75_JPRB, 769.59_JPRB, 777.54_JPRB, 785.59_JPRB&
  &, 793.74_JPRB, 801.99_JPRB, 810.34_JPRB, 818.79_JPRB, 827.35_JPRB&
  &, 836.00_JPRB /)
ZFVAR( :, 2, 2) = (/ &
!  & 1694.30_JPRB,1704.26_JPRB,1713.35_JPRB,1721.61_JPRB,1729.09_JPRB&
!  &,1735.84_JPRB,1741.88_JPRB,1747.28_JPRB,1752.07_JPRB,1756.30_JPRB&
  & 1760.00_JPRB,1768.35_JPRB,1777.10_JPRB,1786.26_JPRB,1795.80_JPRB&
  &,1805.73_JPRB,1816.05_JPRB,1826.73_JPRB,1837.79_JPRB,1849.22_JPRB&
  &,1861.00_JPRB,1873.14_JPRB,1885.62_JPRB,1898.44_JPRB,1911.59_JPRB&
  &,1925.06_JPRB,1938.85_JPRB,1952.94_JPRB,1967.34_JPRB,1982.03_JPRB&
  &,1997.00_JPRB,2012.25_JPRB,2027.79_JPRB,2043.61_JPRB,2059.73_JPRB&
  &,2076.15_JPRB,2092.88_JPRB,2109.93_JPRB,2127.29_JPRB,2144.98_JPRB&
  &,2163.00_JPRB,2181.35_JPRB,2200.02_JPRB,2218.96_JPRB,2238.16_JPRB&
  &,2257.58_JPRB,2277.19_JPRB,2296.97_JPRB,2316.88_JPRB,2336.90_JPRB&
  &,2357.00_JPRB,2377.15_JPRB,2397.36_JPRB,2417.64_JPRB,2437.98_JPRB&
  &,2458.41_JPRB,2478.92_JPRB,2499.53_JPRB,2520.24_JPRB,2541.06_JPRB&
  &,2562.00_JPRB,2583.06_JPRB,2604.26_JPRB,2625.58_JPRB,2647.05_JPRB&
  &,2668.66_JPRB,2690.41_JPRB,2712.32_JPRB,2734.39_JPRB,2756.61_JPRB&
  &,2779.00_JPRB,2801.56_JPRB,2824.28_JPRB,2847.15_JPRB,2870.17_JPRB&
  &,2893.33_JPRB,2916.63_JPRB,2940.05_JPRB,2963.59_JPRB,2987.24_JPRB&
  &,3011.00_JPRB,3034.85_JPRB,3058.79_JPRB,3082.80_JPRB,3106.88_JPRB&
  &,3131.00_JPRB,3155.17_JPRB,3179.36_JPRB,3203.57_JPRB,3227.79_JPRB&
  &,3252.00_JPRB,3276.19_JPRB,3300.37_JPRB,3324.53_JPRB,3348.66_JPRB&
  &,3372.78_JPRB,3396.87_JPRB,3420.94_JPRB,3444.98_JPRB,3469.00_JPRB&
  &,3493.00_JPRB,3516.97_JPRB,3540.91_JPRB,3564.81_JPRB,3588.69_JPRB&
  &,3612.52_JPRB,3636.31_JPRB,3660.06_JPRB,3683.76_JPRB,3707.40_JPRB&
  &,3731.00_JPRB /)
ZFVAR( :, 2, 3) = (/ &
!  &  308.47_JPRB, 309.20_JPRB, 309.94_JPRB, 310.68_JPRB, 311.43_JPRB&
!  &, 312.19_JPRB, 312.95_JPRB, 313.71_JPRB, 314.47_JPRB, 315.23_JPRB&
  &  316.00_JPRB, 316.86_JPRB, 317.73_JPRB, 318.61_JPRB, 319.50_JPRB&
  &, 320.40_JPRB, 321.30_JPRB, 322.22_JPRB, 323.14_JPRB, 324.07_JPRB&
  &, 325.00_JPRB, 325.94_JPRB, 326.89_JPRB, 327.85_JPRB, 328.82_JPRB&
  &, 329.80_JPRB, 330.80_JPRB, 331.82_JPRB, 332.86_JPRB, 333.92_JPRB&
  &, 335.00_JPRB, 336.11_JPRB, 337.24_JPRB, 338.40_JPRB, 339.58_JPRB&
  &, 340.77_JPRB, 341.99_JPRB, 343.22_JPRB, 344.47_JPRB, 345.73_JPRB&
  &, 347.00_JPRB, 348.28_JPRB, 349.57_JPRB, 350.87_JPRB, 352.17_JPRB&
  &, 353.48_JPRB, 354.79_JPRB, 356.10_JPRB, 357.40_JPRB, 358.70_JPRB&
  &, 360.00_JPRB, 361.29_JPRB, 362.57_JPRB, 363.86_JPRB, 365.14_JPRB&
  &, 366.43_JPRB, 367.72_JPRB, 369.02_JPRB, 370.33_JPRB, 371.66_JPRB&
  &, 373.00_JPRB, 374.36_JPRB, 375.74_JPRB, 377.12_JPRB, 378.52_JPRB&
  &, 379.93_JPRB, 381.35_JPRB, 382.76_JPRB, 384.18_JPRB, 385.59_JPRB&
  &, 387.00_JPRB, 388.40_JPRB, 389.80_JPRB, 391.19_JPRB, 392.58_JPRB&
  &, 393.97_JPRB, 395.36_JPRB, 396.76_JPRB, 398.16_JPRB, 399.58_JPRB&
  &, 401.00_JPRB, 402.44_JPRB, 403.89_JPRB, 405.35_JPRB, 406.83_JPRB&
  &, 408.33_JPRB, 409.83_JPRB, 411.35_JPRB, 412.89_JPRB, 414.44_JPRB&
  &, 416.00_JPRB, 417.58_JPRB, 419.17_JPRB, 420.76_JPRB, 422.37_JPRB&
  &, 423.98_JPRB, 425.59_JPRB, 427.20_JPRB, 428.81_JPRB, 430.41_JPRB&
  &, 432.00_JPRB, 433.58_JPRB, 435.15_JPRB, 436.71_JPRB, 438.24_JPRB&
  &, 439.76_JPRB, 441.26_JPRB, 442.73_JPRB, 444.18_JPRB, 445.61_JPRB&
  &, 447.00_JPRB /)
ZFVAR( :, 2, 4) = (/ &
!  &  563.29_JPRB, 583.20_JPRB, 599.03_JPRB, 611.20_JPRB, 620.16_JPRB&
!  &, 626.82_JPRB, 631.77_JPRB, 635.83_JPRB, 640.05_JPRB, 645.56_JPRB&
  &  653.45_JPRB, 664.82_JPRB, 678.71_JPRB, 694.10_JPRB, 709.86_JPRB&
  &, 724.89_JPRB, 738.38_JPRB, 750.21_JPRB, 760.47_JPRB, 769.32_JPRB&
  &, 776.84_JPRB, 782.97_JPRB, 787.81_JPRB, 791.21_JPRB, 793.02_JPRB&
  &, 793.17_JPRB, 791.58_JPRB, 788.45_JPRB, 784.02_JPRB, 778.56_JPRB&
  &, 772.32_JPRB, 765.53_JPRB, 758.46_JPRB, 751.24_JPRB, 744.14_JPRB&
  &, 737.37_JPRB, 731.07_JPRB, 725.35_JPRB, 720.12_JPRB, 715.46_JPRB&
  &, 711.41_JPRB, 707.90_JPRB, 705.00_JPRB, 702.67_JPRB, 700.93_JPRB&
  &, 699.79_JPRB, 699.26_JPRB, 699.20_JPRB, 699.45_JPRB, 699.92_JPRB&
  &, 700.44_JPRB, 701.04_JPRB, 701.56_JPRB, 702.13_JPRB, 702.76_JPRB&
  &, 703.52_JPRB, 704.42_JPRB, 705.61_JPRB, 707.13_JPRB, 709.14_JPRB&
  &, 711.69_JPRB, 714.61_JPRB, 718.17_JPRB, 722.26_JPRB, 726.91_JPRB&
  &, 732.01_JPRB, 737.56_JPRB, 743.54_JPRB, 749.89_JPRB, 756.66_JPRB&
  &, 763.79_JPRB, 771.50_JPRB, 779.61_JPRB, 788.03_JPRB, 796.76_JPRB&
  &, 805.85_JPRB, 815.27_JPRB, 824.95_JPRB, 835.01_JPRB, 845.36_JPRB&
  &, 856.00_JPRB, 866.97_JPRB, 878.27_JPRB, 889.86_JPRB, 901.80_JPRB&
  &, 914.04_JPRB, 926.58_JPRB, 939.46_JPRB, 952.63_JPRB, 966.12_JPRB&
  &, 979.89_JPRB, 993.95_JPRB,1008.28_JPRB,1022.90_JPRB,1037.80_JPRB&
  &,1052.94_JPRB,1068.35_JPRB,1083.98_JPRB,1099.86_JPRB,1116.02_JPRB&
  &,1132.34_JPRB,1148.77_JPRB,1165.39_JPRB,1182.18_JPRB,1199.14_JPRB&
  &,1216.22_JPRB,1233.51_JPRB,1250.82_JPRB,1268.33_JPRB,1285.95_JPRB&
  &,1303.65_JPRB /)
ZFVAR( :, 2, 5) = (/ &
!  &  485.20_JPRB, 498.64_JPRB, 509.37_JPRB, 517.69_JPRB, 523.94_JPRB&
!  &, 528.40_JPRB, 531.40_JPRB, 533.25_JPRB, 534.26_JPRB, 534.74_JPRB&
  &  535.00_JPRB, 535.29_JPRB, 535.58_JPRB, 535.76_JPRB, 535.73_JPRB&
  &, 535.40_JPRB, 534.68_JPRB, 533.53_JPRB, 531.93_JPRB, 529.86_JPRB&
  &, 527.30_JPRB, 524.25_JPRB, 520.76_JPRB, 516.92_JPRB, 512.81_JPRB&
  &, 508.50_JPRB, 504.07_JPRB, 499.55_JPRB, 494.96_JPRB, 490.34_JPRB&
  &, 485.70_JPRB, 481.07_JPRB, 476.46_JPRB, 471.87_JPRB, 467.32_JPRB&
  &, 462.80_JPRB, 458.32_JPRB, 453.89_JPRB, 449.49_JPRB, 445.13_JPRB&
  &, 440.80_JPRB, 436.51_JPRB, 432.25_JPRB, 428.03_JPRB, 423.84_JPRB&
  &, 419.70_JPRB, 415.60_JPRB, 411.54_JPRB, 407.52_JPRB, 403.54_JPRB&
  &, 399.60_JPRB, 395.70_JPRB, 391.85_JPRB, 388.03_JPRB, 384.25_JPRB&
  &, 380.50_JPRB, 376.79_JPRB, 373.11_JPRB, 369.47_JPRB, 365.87_JPRB&
  &, 362.30_JPRB, 358.77_JPRB, 355.28_JPRB, 351.82_JPRB, 348.39_JPRB&
  &, 345.00_JPRB, 341.64_JPRB, 338.30_JPRB, 335.00_JPRB, 331.73_JPRB&
  &, 328.50_JPRB, 325.30_JPRB, 322.13_JPRB, 318.99_JPRB, 315.88_JPRB&
  &, 312.80_JPRB, 309.74_JPRB, 306.71_JPRB, 303.71_JPRB, 300.74_JPRB&
  &, 297.80_JPRB, 294.90_JPRB, 292.03_JPRB, 289.20_JPRB, 286.39_JPRB&
  &, 283.60_JPRB, 280.83_JPRB, 278.09_JPRB, 275.36_JPRB, 272.67_JPRB&
  &, 270.00_JPRB, 267.36_JPRB, 264.76_JPRB, 262.18_JPRB, 259.63_JPRB&
  &, 257.10_JPRB, 254.59_JPRB, 252.11_JPRB, 249.65_JPRB, 247.21_JPRB&
  &, 244.80_JPRB, 242.41_JPRB, 240.05_JPRB, 237.71_JPRB, 235.39_JPRB&
  &, 233.10_JPRB, 230.83_JPRB, 228.59_JPRB, 226.37_JPRB, 224.17_JPRB&
  &, 222.00_JPRB /)

!-- scenario B1
ZFVAR( :, 3, 1) = (/ &
!  &  352.00_JPRB, 353.55_JPRB, 355.09_JPRB, 356.63_JPRB, 358.16_JPRB&
!  &, 359.68_JPRB, 361.18_JPRB, 362.67_JPRB, 364.14_JPRB, 365.58_JPRB&
  &  367.00_JPRB, 368.67_JPRB, 370.37_JPRB, 372.12_JPRB, 373.92_JPRB&
  &, 375.77_JPRB, 377.68_JPRB, 379.65_JPRB, 381.69_JPRB, 383.81_JPRB&
  &, 386.00_JPRB, 388.27_JPRB, 390.61_JPRB, 393.01_JPRB, 395.44_JPRB&
  &, 397.90_JPRB, 400.36_JPRB, 402.82_JPRB, 405.25_JPRB, 407.65_JPRB&
  &, 410.00_JPRB, 412.29_JPRB, 414.52_JPRB, 416.71_JPRB, 418.87_JPRB&
  &, 421.02_JPRB, 423.17_JPRB, 425.33_JPRB, 427.51_JPRB, 429.73_JPRB&
  &, 432.00_JPRB, 434.33_JPRB, 436.72_JPRB, 439.15_JPRB, 441.63_JPRB&
  &, 444.14_JPRB, 446.68_JPRB, 449.25_JPRB, 451.83_JPRB, 454.41_JPRB&
  &, 457.00_JPRB, 459.58_JPRB, 462.15_JPRB, 464.71_JPRB, 467.26_JPRB&
  &, 469.78_JPRB, 472.29_JPRB, 474.76_JPRB, 477.21_JPRB, 479.62_JPRB&
  &, 482.00_JPRB, 484.34_JPRB, 486.63_JPRB, 488.88_JPRB, 491.08_JPRB&
  &, 493.22_JPRB, 495.31_JPRB, 497.33_JPRB, 499.29_JPRB, 501.18_JPRB&
  &, 503.00_JPRB, 504.74_JPRB, 506.42_JPRB, 508.02_JPRB, 509.58_JPRB&
  &, 511.07_JPRB, 512.53_JPRB, 513.94_JPRB, 515.32_JPRB, 516.67_JPRB&
  &, 518.00_JPRB, 519.31_JPRB, 520.61_JPRB, 521.88_JPRB, 523.13_JPRB&
  &, 524.36_JPRB, 525.55_JPRB, 526.72_JPRB, 527.85_JPRB, 528.94_JPRB&
  &, 530.00_JPRB, 531.02_JPRB, 531.99_JPRB, 532.92_JPRB, 533.80_JPRB&
  &, 534.63_JPRB, 535.41_JPRB, 536.14_JPRB, 536.82_JPRB, 537.44_JPRB&
  &, 538.00_JPRB, 538.50_JPRB, 538.94_JPRB, 539.32_JPRB, 539.63_JPRB&
  &, 539.87_JPRB, 540.05_JPRB, 540.15_JPRB, 540.17_JPRB, 540.13_JPRB&
  &, 540.00_JPRB /)
ZFVAR( :, 3, 2) = (/ &
!  & 1694.30_JPRB,1704.26_JPRB,1713.35_JPRB,1721.61_JPRB,1729.09_JPRB&
! &,1735.84_JPRB,1741.88_JPRB,1747.28_JPRB,1752.07_JPRB,1756.30_JPRB&
  & 1760.00_JPRB,1766.50_JPRB,1773.06_JPRB,1779.67_JPRB,1786.33_JPRB&
  &,1793.03_JPRB,1799.77_JPRB,1806.54_JPRB,1813.34_JPRB,1820.16_JPRB&
  &,1827.00_JPRB,1833.85_JPRB,1840.68_JPRB,1847.47_JPRB,1854.17_JPRB&
  &,1860.76_JPRB,1867.22_JPRB,1873.50_JPRB,1879.58_JPRB,1885.42_JPRB&
  &,1891.00_JPRB,1896.28_JPRB,1901.25_JPRB,1905.88_JPRB,1910.15_JPRB&
  &,1914.04_JPRB,1917.52_JPRB,1920.58_JPRB,1923.19_JPRB,1925.34_JPRB&
  &,1927.00_JPRB,1928.16_JPRB,1928.82_JPRB,1929.02_JPRB,1928.77_JPRB&
  &,1928.09_JPRB,1927.01_JPRB,1925.53_JPRB,1923.70_JPRB,1921.51_JPRB&
  &,1919.00_JPRB,1916.19_JPRB,1913.09_JPRB,1909.73_JPRB,1906.15_JPRB&
  &,1902.35_JPRB,1898.36_JPRB,1894.21_JPRB,1889.91_JPRB,1885.50_JPRB&
  &,1881.00_JPRB,1876.43_JPRB,1871.81_JPRB,1867.16_JPRB,1862.52_JPRB&
  &,1857.90_JPRB,1853.33_JPRB,1848.83_JPRB,1844.43_JPRB,1840.14_JPRB&
  &,1836.00_JPRB,1832.01_JPRB,1828.15_JPRB,1824.38_JPRB,1820.65_JPRB&
  &,1816.93_JPRB,1813.17_JPRB,1809.34_JPRB,1805.40_JPRB,1801.30_JPRB&
  &,1797.00_JPRB,1792.47_JPRB,1787.71_JPRB,1782.71_JPRB,1777.48_JPRB&
  &,1772.00_JPRB,1766.29_JPRB,1760.33_JPRB,1754.13_JPRB,1747.69_JPRB&
  &,1741.00_JPRB,1734.07_JPRB,1726.91_JPRB,1719.53_JPRB,1711.96_JPRB&
  &,1704.19_JPRB,1696.24_JPRB,1688.14_JPRB,1679.89_JPRB,1671.50_JPRB&
  &,1663.00_JPRB,1654.39_JPRB,1645.69_JPRB,1636.90_JPRB,1628.04_JPRB&
  &,1619.13_JPRB,1610.16_JPRB,1601.15_JPRB,1592.11_JPRB,1583.06_JPRB&
  &,1574.00_JPRB /)
ZFVAR( :, 3, 3) = (/ &
!  &  308.47_JPRB, 309.20_JPRB, 309.94_JPRB, 310.68_JPRB, 311.43_JPRB&
!  &, 312.19_JPRB, 312.95_JPRB, 313.71_JPRB, 314.47_JPRB, 315.23_JPRB&
  &  316.00_JPRB, 316.77_JPRB, 317.54_JPRB, 318.32_JPRB, 319.10_JPRB&
  &, 319.88_JPRB, 320.68_JPRB, 321.49_JPRB, 322.31_JPRB, 323.14_JPRB&
  &, 324.00_JPRB, 324.88_JPRB, 325.77_JPRB, 326.67_JPRB, 327.58_JPRB&
  &, 328.50_JPRB, 329.42_JPRB, 330.33_JPRB, 331.23_JPRB, 332.13_JPRB&
  &, 333.00_JPRB, 333.85_JPRB, 334.69_JPRB, 335.51_JPRB, 336.32_JPRB&
  &, 337.11_JPRB, 337.90_JPRB, 338.68_JPRB, 339.45_JPRB, 340.22_JPRB&
  &, 341.00_JPRB, 341.78_JPRB, 342.56_JPRB, 343.35_JPRB, 344.14_JPRB&
  &, 344.93_JPRB, 345.73_JPRB, 346.54_JPRB, 347.35_JPRB, 348.17_JPRB&
  &, 349.00_JPRB, 349.83_JPRB, 350.67_JPRB, 351.50_JPRB, 352.33_JPRB&
  &, 353.16_JPRB, 353.97_JPRB, 354.76_JPRB, 355.53_JPRB, 356.28_JPRB&
  &, 357.00_JPRB, 357.69_JPRB, 358.35_JPRB, 358.98_JPRB, 359.60_JPRB&
  &, 360.19_JPRB, 360.77_JPRB, 361.34_JPRB, 361.90_JPRB, 362.45_JPRB&
  &, 363.00_JPRB, 363.55_JPRB, 364.10_JPRB, 364.65_JPRB, 365.18_JPRB&
  &, 365.70_JPRB, 366.21_JPRB, 366.69_JPRB, 367.16_JPRB, 367.59_JPRB&
  &, 368.00_JPRB, 368.37_JPRB, 368.72_JPRB, 369.04_JPRB, 369.34_JPRB&
  &, 369.62_JPRB, 369.90_JPRB, 370.17_JPRB, 370.44_JPRB, 370.72_JPRB&
  &, 371.00_JPRB, 371.30_JPRB, 371.60_JPRB, 371.92_JPRB, 372.24_JPRB&
  &, 372.55_JPRB, 372.87_JPRB, 373.17_JPRB, 373.46_JPRB, 373.74_JPRB&
  &, 374.00_JPRB, 374.24_JPRB, 374.45_JPRB, 374.63_JPRB, 374.79_JPRB&
  &, 374.91_JPRB, 375.00_JPRB, 375.06_JPRB, 375.08_JPRB, 375.06_JPRB&
  &, 375.00_JPRB /)                                                    
ZFVAR( :, 3, 4) = (/ &
!  &  563.29_JPRB, 583.20_JPRB, 599.03_JPRB, 611.20_JPRB, 620.16_JPRB&
!  &, 626.82_JPRB, 631.77_JPRB, 635.83_JPRB, 640.05_JPRB, 645.56_JPRB&
  &  653.45_JPRB, 664.97_JPRB, 679.02_JPRB, 694.53_JPRB, 710.35_JPRB&
  &, 725.41_JPRB, 738.84_JPRB, 750.50_JPRB, 760.52_JPRB, 769.01_JPRB&
  &, 776.00_JPRB, 781.47_JPRB, 785.50_JPRB, 788.00_JPRB, 788.85_JPRB&
  &, 787.99_JPRB, 785.39_JPRB, 781.23_JPRB, 775.85_JPRB, 769.52_JPRB&
  &, 762.48_JPRB, 755.06_JPRB, 747.37_JPRB, 739.67_JPRB, 732.08_JPRB&
  &, 724.86_JPRB, 718.09_JPRB, 711.82_JPRB, 705.99_JPRB, 700.59_JPRB&
  &, 695.61_JPRB, 691.05_JPRB, 686.91_JPRB, 683.26_JPRB, 680.20_JPRB&
  &, 677.76_JPRB, 675.95_JPRB, 674.72_JPRB, 673.96_JPRB, 673.61_JPRB&
  &, 673.56_JPRB, 673.84_JPRB, 674.32_JPRB, 675.00_JPRB, 675.74_JPRB&
  &, 676.65_JPRB, 677.63_JPRB, 678.77_JPRB, 680.02_JPRB, 681.43_JPRB&
  &, 683.01_JPRB, 684.55_JPRB, 686.22_JPRB, 688.04_JPRB, 689.93_JPRB&
  &, 691.77_JPRB, 693.61_JPRB, 695.38_JPRB, 697.04_JPRB, 698.62_JPRB&
  &, 700.07_JPRB, 701.60_JPRB, 703.00_JPRB, 704.25_JPRB, 705.34_JPRB&
  &, 706.32_JPRB, 707.15_JPRB, 707.86_JPRB, 708.49_JPRB, 708.99_JPRB&
  &, 709.40_JPRB, 709.75_JPRB, 709.99_JPRB, 710.22_JPRB, 710.38_JPRB&
  &, 710.45_JPRB, 710.47_JPRB, 710.49_JPRB, 710.43_JPRB, 710.35_JPRB&
  &, 710.21_JPRB, 710.02_JPRB, 709.77_JPRB, 709.51_JPRB, 709.17_JPRB&
  &, 708.78_JPRB, 708.32_JPRB, 707.80_JPRB, 707.24_JPRB, 706.58_JPRB&
  &, 705.86_JPRB, 704.94_JPRB, 703.90_JPRB, 702.76_JPRB, 701.50_JPRB&
  &, 700.15_JPRB, 698.63_JPRB, 696.98_JPRB, 695.21_JPRB, 693.29_JPRB&
  &, 691.21_JPRB /)
ZFVAR( :, 3, 5) = (/ &
!  &  485.20_JPRB, 498.64_JPRB, 509.37_JPRB, 517.69_JPRB, 523.94_JPRB&
!  &, 528.40_JPRB, 531.40_JPRB, 533.25_JPRB, 534.26_JPRB, 534.74_JPRB&
  &  535.00_JPRB, 535.29_JPRB, 535.58_JPRB, 535.76_JPRB, 535.73_JPRB&
  &, 535.40_JPRB, 534.68_JPRB, 533.53_JPRB, 531.93_JPRB, 529.86_JPRB&
  &, 527.30_JPRB, 524.25_JPRB, 520.76_JPRB, 516.92_JPRB, 512.81_JPRB&
  &, 508.50_JPRB, 504.07_JPRB, 499.55_JPRB, 494.96_JPRB, 490.34_JPRB&
  &, 485.70_JPRB, 481.07_JPRB, 476.46_JPRB, 471.87_JPRB, 467.32_JPRB&
  &, 462.80_JPRB, 458.32_JPRB, 453.89_JPRB, 449.49_JPRB, 445.13_JPRB&
  &, 440.80_JPRB, 436.51_JPRB, 432.25_JPRB, 428.03_JPRB, 423.84_JPRB&
  &, 419.70_JPRB, 415.60_JPRB, 411.54_JPRB, 407.52_JPRB, 403.54_JPRB&
  &, 399.60_JPRB, 395.70_JPRB, 391.85_JPRB, 388.03_JPRB, 384.25_JPRB&
  &, 380.50_JPRB, 376.79_JPRB, 373.11_JPRB, 369.47_JPRB, 365.87_JPRB&
  &, 362.30_JPRB, 358.77_JPRB, 355.28_JPRB, 351.82_JPRB, 348.39_JPRB&
  &, 345.00_JPRB, 341.64_JPRB, 338.30_JPRB, 335.00_JPRB, 331.73_JPRB&
  &, 328.50_JPRB, 325.30_JPRB, 322.13_JPRB, 318.99_JPRB, 315.88_JPRB&
  &, 312.80_JPRB, 309.74_JPRB, 306.71_JPRB, 303.71_JPRB, 300.74_JPRB&
  &, 297.80_JPRB, 294.90_JPRB, 292.03_JPRB, 289.20_JPRB, 286.39_JPRB&
  &, 283.60_JPRB, 280.83_JPRB, 278.09_JPRB, 275.36_JPRB, 272.67_JPRB&
  &, 270.00_JPRB, 267.36_JPRB, 264.76_JPRB, 262.18_JPRB, 259.63_JPRB&
  &, 257.10_JPRB, 254.59_JPRB, 252.11_JPRB, 249.65_JPRB, 247.21_JPRB&
  &, 244.80_JPRB, 242.41_JPRB, 240.05_JPRB, 237.71_JPRB, 235.39_JPRB&
  &, 233.10_JPRB, 230.83_JPRB, 228.59_JPRB, 226.37_JPRB, 224.17_JPRB&
  &, 222.00_JPRB /)

!-- annual mean solar insolation at TOA from Krivova and Solanki 2002

  ZINCSOL( :) = (/ &
  &  1364.9547_JPRB, 1364.9337_JPRB, 1364.8495_JPRB, 1364.7183_JPRB, 1364.5078_JPRB&
  &, 1364.3409_JPRB, 1364.3134_JPRB, 1364.5446_JPRB, 1364.8776_JPRB, 1365.1927_JPRB&
  &, 1365.1946_JPRB, 1365.0619_JPRB, 1364.9293_JPRB, 1364.7836_JPRB, 1364.8109_JPRB&
  &, 1364.6390_JPRB, 1364.4718_JPRB, 1364.3588_JPRB, 1364.7078_JPRB, 1365.0214_JPRB&
  &, 1365.3192_JPRB, 1365.2324_JPRB, 1365.1753_JPRB, 1364.8832_JPRB, 1364.6561_JPRB&
  &, 1364.2898_JPRB, 1364.2396_JPRB, 1364.2617_JPRB, 1364.1466_JPRB, 1364.2170_JPRB&
  &, 1364.3368_JPRB, 1364.5590_JPRB, 1364.4569_JPRB, 1364.3893_JPRB, 1364.4696_JPRB&
  &, 1364.4153_JPRB, 1364.1873_JPRB, 1364.1057_JPRB, 1364.0573_JPRB, 1364.0934_JPRB&
  &, 1364.1005_JPRB, 1364.3910_JPRB, 1364.6971_JPRB, 1364.7747_JPRB, 1364.7816_JPRB&
  &, 1364.7076_JPRB, 1364.5402_JPRB, 1364.2621_JPRB, 1364.3280_JPRB, 1364.2018_JPRB&
  &, 1364.1810_JPRB, 1364.1093_JPRB, 1364.0638_JPRB, 1364.2029_JPRB, 1364.4266_JPRB&
  &, 1364.3743_JPRB, 1364.4276_JPRB, 1364.4055_JPRB, 1364.4335_JPRB, 1364.3944_JPRB&
  &, 1364.2128_JPRB, 1364.1459_JPRB, 1364.1454_JPRB, 1364.2623_JPRB, 1364.2972_JPRB&
  &, 1364.7023_JPRB, 1364.9022_JPRB, 1365.2134_JPRB, 1365.1145_JPRB, 1364.8568_JPRB&
  &, 1364.6387_JPRB, 1364.5365_JPRB, 1364.3905_JPRB, 1364.3478_JPRB, 1364.4185_JPRB&
  &, 1364.5974_JPRB, 1364.6935_JPRB, 1364.8872_JPRB, 1364.8397_JPRB, 1364.7524_JPRB&
  &, 1364.6827_JPRB, 1364.5896_JPRB, 1364.4991_JPRB, 1364.5112_JPRB, 1364.5651_JPRB&
  &, 1364.8190_JPRB, 1365.3113_JPRB, 1365.3152_JPRB, 1365.3331_JPRB, 1365.3092_JPRB&
  &, 1365.3125_JPRB, 1365.2059_JPRB, 1365.0557_JPRB, 1364.8947_JPRB, 1364.9139_JPRB&
  &, 1365.2025_JPRB, 1365.4434_JPRB, 1365.7584_JPRB, 1366.0043_JPRB, 1365.9574_JPRB&
  &, 1365.7809_JPRB, 1365.6195_JPRB, 1365.4186_JPRB, 1365.2979_JPRB, 1365.2026_JPRB&
  &, 1365.5472_JPRB, 1366.0938_JPRB, 1366.2767_JPRB, 1366.2490_JPRB, 1365.9980_JPRB&
  &, 1366.0659_JPRB, 1365.6847_JPRB, 1365.4200_JPRB, 1365.2816_JPRB, 1365.0278_JPRB&
  &, 1365.0356_JPRB, 1365.2876_JPRB, 1365.4667_JPRB, 1365.5751_JPRB, 1365.6283_JPRB&
  &, 1365.5963_JPRB, 1365.3969_JPRB, 1365.5189_JPRB, 1365.2910_JPRB, 1365.3031_JPRB&
  &, 1365.1409_JPRB, 1365.1628_JPRB, 1365.4098_JPRB, 1365.8561_JPRB, 1366.5015_JPRB&
  &, 1366.4833_JPRB, 1366.5184_JPRB, 1366.0620_JPRB, 1366.0229_JPRB, 1365.6791_JPRB&
  &, 1365.4935_JPRB, 1365.4829_JPRB, 1365.5917_JPRB, 1365.9048_JPRB, 1366.3468_JPRB&
  &, 1366.3116_JPRB, 1366.2269_JPRB, 1366.0371_JPRB, 1365.7560_JPRB, 1365.5923_JPRB&
  &, 1365.5034_JPRB, 1365.4052_JPRB, 1365.5460_JPRB, 1365.8844_JPRB, 1366.0829_JPRB&
  &, 1366.3046_JPRB, 1366.2340_JPRB, 1366.2845_JPRB, 1365.8322_JPRB /)

!-- CMIP5 recommended values, with varying background term (Lean et al 2005), prior
!   to adjustment to TIM scale with factor 0.9965.

  ZCMIP5SOL(1851:1900) = (/ &
  &  1365.7744_JPRB, 1365.6947_JPRB, 1365.5848_JPRB, 1365.4614_JPRB, 1365.3828_JPRB&
  &, 1365.3853_JPRB, 1365.4946_JPRB, 1365.6875_JPRB, 1365.9054_JPRB, 1365.9718_JPRB&
  &, 1365.8623_JPRB, 1365.7120_JPRB, 1365.6283_JPRB, 1365.5660_JPRB, 1365.4755_JPRB&
  &, 1365.4119_JPRB, 1365.3597_JPRB, 1365.5194_JPRB, 1365.7557_JPRB, 1365.9944_JPRB&
  &, 1365.9343_JPRB, 1365.8876_JPRB, 1365.6866_JPRB, 1365.5582_JPRB, 1365.4095_JPRB&
  &, 1365.3593_JPRB, 1365.3596_JPRB, 1365.3309_JPRB, 1365.3533_JPRB, 1365.5000_JPRB&
  &, 1365.6443_JPRB, 1365.6676_JPRB, 1365.7147_JPRB, 1365.6166_JPRB, 1365.5070_JPRB&
  &, 1365.3417_JPRB, 1365.2982_JPRB, 1365.2628_JPRB, 1365.2344_JPRB, 1365.2690_JPRB&
  &, 1365.5204_JPRB, 1365.6190_JPRB, 1365.7600_JPRB, 1365.8553_JPRB, 1365.7390_JPRB&
  &, 1365.5581_JPRB, 1365.4126_JPRB, 1365.3899_JPRB, 1365.3381_JPRB, 1365.3074_JPRB /)

  ZCMIP5SOL(1901:1950) = (/ &
  &  1365.2292_JPRB, 1365.2378_JPRB, 1365.4479_JPRB, 1365.7180_JPRB, 1365.5291_JPRB&
  &, 1365.7255_JPRB, 1365.6097_JPRB, 1365.6748_JPRB, 1365.5642_JPRB, 1365.4309_JPRB&
  &, 1365.3473_JPRB, 1365.3010_JPRB, 1365.3175_JPRB, 1365.3844_JPRB, 1365.6890_JPRB&
  &, 1365.8990_JPRB, 1366.0480_JPRB, 1366.0096_JPRB, 1365.7802_JPRB, 1365.6178_JPRB&
  &, 1365.5127_JPRB, 1365.3948_JPRB, 1365.4265_JPRB, 1365.4581_JPRB, 1365.6622_JPRB&
  &, 1365.7633_JPRB, 1365.9468_JPRB, 1365.8245_JPRB, 1365.7833_JPRB, 1365.7655_JPRB&
  &, 1365.6436_JPRB, 1365.5364_JPRB, 1365.4156_JPRB, 1365.5275_JPRB, 1365.7439_JPRB&
  &, 1366.1333_JPRB, 1366.0676_JPRB, 1366.0031_JPRB, 1365.9868_JPRB, 1365.9242_JPRB&
  &, 1365.8451_JPRB, 1365.7419_JPRB, 1365.5841_JPRB, 1365.6140_JPRB, 1365.8810_JPRB&
  &, 1365.9791_JPRB, 1366.2185_JPRB, 1366.3490_JPRB, 1366.2555_JPRB, 1366.0131_JPRB /)

  ZCMIP5SOL(1951:2000) = (/ &
  &  1365.7765_JPRB, 1365.7676_JPRB, 1365.6284_JPRB, 1365.6564_JPRB, 1365.7773_JPRB&
  &, 1366.3109_JPRB, 1366.6681_JPRB, 1366.6328_JPRB, 1366.3828_JPRB, 1366.2767_JPRB&
  &, 1365.9199_JPRB, 1365.7484_JPRB, 1365.6963_JPRB, 1365.6976_JPRB, 1365.7341_JPRB&
  &, 1365.9178_JPRB, 1366.1143_JPRB, 1366.1644_JPRB, 1366.2476_JPRB, 1366.2426_JPRB&
  &, 1365.9580_JPRB, 1366.0525_JPRB, 1365.7991_JPRB, 1365.7271_JPRB, 1365.5345_JPRB&
  &, 1365.6453_JPRB, 1365.8331_JPRB, 1366.2747_JPRB, 1366.6348_JPRB, 1366.6482_JPRB&
  &, 1366.6951_JPRB, 1366.2859_JPRB, 1366.1992_JPRB, 1365.8103_JPRB, 1365.6416_JPRB&
  &, 1365.6379_JPRB, 1365.7899_JPRB, 1366.0826_JPRB, 1366.6479_JPRB, 1366.5533_JPRB&
  &, 1366.4457_JPRB, 1366.3021_JPRB, 1366.0286_JPRB, 1365.7971_JPRB, 1365.6996_JPRB&
  &, 1365.6121_JPRB, 1365.7399_JPRB, 1366.1021_JPRB, 1366.3851_JPRB, 1366.6836_JPRB /)

  ZCMIP5SOL(2001:2008) = (/ &
  &  1366.6022_JPRB, 1366.6807_JPRB, 1366.2300_JPRB, 1366.0480_JPRB, 1365.8545_JPRB&
  &, 1365.8107_JPRB, 1365.7240_JPRB, 1365.6918_JPRB /)


!-- monthly mean solar insolation at TOA from Krivova and Solanki 2002 (diff to 1365. W/m2)

ZINSOLM(  1:120) = (/ &
  &  0.0662842_JPRB, 0.1324463_JPRB, 0.0378418_JPRB,-0.1724854_JPRB,-0.0842285_JPRB,-0.0701904_JPRB&
  &,-0.2603760_JPRB,-0.0954590_JPRB, 0.0804443_JPRB,-0.0205078_JPRB,-0.1368408_JPRB,-0.0183105_JPRB&
  &, 0.0129395_JPRB, 0.1480713_JPRB,-0.0131836_JPRB,-0.0695801_JPRB,-0.0979004_JPRB,-0.1221924_JPRB&
  &,-0.2313232_JPRB,-0.1201172_JPRB,-0.0426025_JPRB,-0.0949707_JPRB,-0.1579590_JPRB,-0.0069580_JPRB&
  &,-0.0065918_JPRB,-0.0704346_JPRB,-0.0272217_JPRB,-0.0504150_JPRB,-0.1562500_JPRB,-0.2033691_JPRB&
  &,-0.2622070_JPRB,-0.2749023_JPRB,-0.2913818_JPRB,-0.0418701_JPRB,-0.1517334_JPRB,-0.2683105_JPRB&
  &,-0.2711182_JPRB,-0.2451172_JPRB,-0.2733154_JPRB,-0.1975098_JPRB,-0.3212891_JPRB,-0.2443848_JPRB&
  &,-0.2292480_JPRB,-0.1911621_JPRB,-0.3085938_JPRB,-0.2718506_JPRB,-0.3873291_JPRB,-0.4390869_JPRB&
  &,-0.5186768_JPRB,-0.5256348_JPRB,-0.4727783_JPRB,-0.4079590_JPRB,-0.4605713_JPRB,-0.5108643_JPRB&
  &,-0.5135498_JPRB,-0.5334473_JPRB,-0.4597168_JPRB,-0.5979004_JPRB,-0.4428711_JPRB,-0.4631348_JPRB&
  &,-0.5693359_JPRB,-0.5842285_JPRB,-0.5550537_JPRB,-0.6512451_JPRB,-0.6519775_JPRB,-0.6870117_JPRB&
  &,-0.7298584_JPRB,-0.7187500_JPRB,-0.7327881_JPRB,-0.6464844_JPRB,-0.6704102_JPRB,-0.7122803_JPRB&
  &,-0.7331543_JPRB,-0.6953125_JPRB,-0.7271729_JPRB,-0.6685791_JPRB,-0.7200928_JPRB,-0.6936035_JPRB&
  &,-0.7026367_JPRB,-0.6466064_JPRB,-0.6829834_JPRB,-0.6877441_JPRB,-0.6578369_JPRB,-0.6239014_JPRB&
  &,-0.5656738_JPRB,-0.6396484_JPRB,-0.6513672_JPRB,-0.5926514_JPRB,-0.4049072_JPRB,-0.5112305_JPRB&
  &,-0.5041504_JPRB,-0.4786377_JPRB,-0.2490234_JPRB,-0.2531738_JPRB,-0.3449707_JPRB,-0.2701416_JPRB&
  &,-0.2957764_JPRB,-0.2799072_JPRB,-0.1486816_JPRB,-0.2749023_JPRB,-0.2462158_JPRB,-0.1748047_JPRB&
  &,-0.1177979_JPRB,-0.0856934_JPRB, 0.1237793_JPRB, 0.0999756_JPRB,-0.0672607_JPRB,-0.0023193_JPRB&
  &, 0.1190186_JPRB, 0.1474609_JPRB, 0.1621094_JPRB, 0.1805420_JPRB, 0.1671143_JPRB, 0.1331787_JPRB&
  &, 0.1795654_JPRB, 0.2698975_JPRB, 0.2893066_JPRB, 0.3175049_JPRB, 0.2325439_JPRB, 0.1148682_JPRB /)

ZINSOLM(121:240) = (/ &
  &  0.1370850_JPRB, 0.1658936_JPRB, 0.1779785_JPRB, 0.1077881_JPRB, 0.2259521_JPRB, 0.2147217_JPRB&
  &, 0.2785645_JPRB, 0.2580566_JPRB, 0.1834717_JPRB, 0.2080078_JPRB, 0.2271729_JPRB, 0.1492920_JPRB&
  &,-0.0472412_JPRB, 0.0979004_JPRB, 0.2166748_JPRB, 0.1230469_JPRB,-0.0649414_JPRB, 0.1293945_JPRB&
  &, 0.0882568_JPRB, 0.1003418_JPRB, 0.0695801_JPRB, 0.0028076_JPRB,-0.0423584_JPRB, 0.0686035_JPRB&
  &,-0.0015869_JPRB,-0.0145264_JPRB,-0.1917725_JPRB,-0.1502686_JPRB, 0.0051270_JPRB, 0.1177979_JPRB&
  &, 0.0874023_JPRB,-0.0500488_JPRB,-0.0769043_JPRB,-0.1860352_JPRB,-0.1718750_JPRB,-0.2145996_JPRB&
  &,-0.1782227_JPRB,-0.0640869_JPRB,-0.0361328_JPRB,-0.1787109_JPRB,-0.1137695_JPRB,-0.2530518_JPRB&
  &,-0.3010254_JPRB,-0.2266846_JPRB,-0.4071045_JPRB,-0.2973633_JPRB,-0.2727051_JPRB,-0.2689209_JPRB&
  &,-0.0773926_JPRB,-0.1243896_JPRB,-0.0455322_JPRB,-0.2441406_JPRB,-0.2080078_JPRB,-0.1251221_JPRB&
  &,-0.1187744_JPRB,-0.1745605_JPRB,-0.3798828_JPRB,-0.3009033_JPRB,-0.1319580_JPRB,-0.3376465_JPRB&
  &,-0.2292480_JPRB,-0.2060547_JPRB,-0.2863770_JPRB,-0.3452148_JPRB,-0.3073730_JPRB,-0.3417969_JPRB&
  &,-0.3779297_JPRB,-0.3244629_JPRB,-0.4467773_JPRB,-0.4714355_JPRB,-0.4582520_JPRB,-0.5358887_JPRB&
  &,-0.3679199_JPRB,-0.3029785_JPRB,-0.4121094_JPRB,-0.4969482_JPRB,-0.5460205_JPRB,-0.5290527_JPRB&
  &,-0.5897217_JPRB,-0.5889893_JPRB,-0.6311035_JPRB,-0.5594482_JPRB,-0.6206055_JPRB,-0.6936035_JPRB&
  &,-0.7268066_JPRB,-0.7185059_JPRB,-0.6166992_JPRB,-0.6676025_JPRB,-0.7009277_JPRB,-0.6909180_JPRB&
  &,-0.6832275_JPRB,-0.6702881_JPRB,-0.5703125_JPRB,-0.5806885_JPRB,-0.6051025_JPRB,-0.4625244_JPRB&
  &,-0.5059814_JPRB,-0.5408936_JPRB,-0.3833008_JPRB,-0.2990723_JPRB,-0.3825684_JPRB,-0.4011230_JPRB&
  &,-0.3790283_JPRB,-0.2749023_JPRB,-0.1838379_JPRB,-0.0932617_JPRB,-0.0483398_JPRB,-0.0124512_JPRB&
  &,-0.0332031_JPRB,-0.0969238_JPRB,-0.1674805_JPRB,-0.1744385_JPRB, 0.1862793_JPRB, 0.1823730_JPRB&
  &,-0.0546875_JPRB, 0.0902100_JPRB, 0.0747070_JPRB,-0.0528564_JPRB, 0.0717773_JPRB, 0.2318115_JPRB /)

ZINSOLM(241:360) = (/ &
  &  0.1271973_JPRB, 0.2750244_JPRB, 0.4030762_JPRB, 0.3906250_JPRB, 0.4028320_JPRB, 0.3591309_JPRB&
  &, 0.3784180_JPRB, 0.2330322_JPRB, 0.2341309_JPRB, 0.4144287_JPRB, 0.3591309_JPRB, 0.2542725_JPRB&
  &, 0.1461182_JPRB, 0.2905273_JPRB, 0.3996582_JPRB, 0.4222412_JPRB, 0.3812256_JPRB, 0.2009277_JPRB&
  &, 0.1816406_JPRB, 0.2149658_JPRB, 0.0911865_JPRB, 0.1290283_JPRB, 0.1979980_JPRB, 0.1335449_JPRB&
  &, 0.1157227_JPRB, 0.1984863_JPRB, 0.0823975_JPRB, 0.2269287_JPRB, 0.1912842_JPRB, 0.2523193_JPRB&
  &, 0.2099609_JPRB, 0.1179199_JPRB, 0.2193604_JPRB, 0.1837158_JPRB, 0.2274170_JPRB, 0.0771484_JPRB&
  &, 0.0549316_JPRB, 0.2086182_JPRB, 0.1064453_JPRB,-0.0064697_JPRB,-0.2506104_JPRB,-0.2978516_JPRB&
  &,-0.1383057_JPRB,-0.1054688_JPRB,-0.2032471_JPRB,-0.2700195_JPRB,-0.2106934_JPRB,-0.2879639_JPRB&
  &,-0.2189941_JPRB,-0.2182617_JPRB,-0.2174072_JPRB,-0.4309082_JPRB,-0.3000488_JPRB,-0.3120117_JPRB&
  &,-0.2115479_JPRB,-0.2762451_JPRB,-0.5382080_JPRB,-0.4279785_JPRB,-0.5653076_JPRB,-0.4096680_JPRB&
  &,-0.6779785_JPRB,-0.7060547_JPRB,-0.4849854_JPRB,-0.5631104_JPRB,-0.7392578_JPRB,-0.6826172_JPRB&
  &,-0.7255859_JPRB,-0.6654053_JPRB,-0.8985596_JPRB,-0.8005371_JPRB,-0.7084961_JPRB,-0.8702393_JPRB&
  &,-0.7445068_JPRB,-0.7875977_JPRB,-0.4757080_JPRB,-0.8808594_JPRB,-0.8061523_JPRB,-0.9505615_JPRB&
  &,-0.6462402_JPRB,-0.8220215_JPRB,-0.7911377_JPRB,-0.7214355_JPRB,-0.7489014_JPRB,-0.7479248_JPRB&
  &,-0.5738525_JPRB,-0.7789307_JPRB,-0.7429199_JPRB,-0.6861572_JPRB,-0.5605469_JPRB,-0.6264648_JPRB&
  &,-0.7340088_JPRB,-0.7843018_JPRB,-0.6613770_JPRB,-0.9169922_JPRB,-0.8886719_JPRB,-0.9050293_JPRB&
  &,-0.8020020_JPRB,-0.7050781_JPRB,-0.7911377_JPRB,-0.9573975_JPRB,-0.7777100_JPRB,-0.8267822_JPRB&
  &,-0.9183350_JPRB,-0.9720459_JPRB,-0.8414307_JPRB,-0.8253174_JPRB,-0.9268799_JPRB,-0.8974609_JPRB&
  &,-0.9652100_JPRB,-0.7484131_JPRB,-0.7580566_JPRB,-0.8221436_JPRB,-0.7686768_JPRB,-0.6652832_JPRB&
  &,-0.7890625_JPRB,-0.6339111_JPRB,-0.8562012_JPRB,-0.7532959_JPRB,-0.7846680_JPRB,-0.8504639_JPRB /)

ZINSOLM(361:480) = (/ &
  & -0.7349854_JPRB,-0.6092529_JPRB,-0.6665039_JPRB,-0.8565674_JPRB,-0.7935791_JPRB,-0.6649170_JPRB&
  &,-0.7257080_JPRB,-0.5419922_JPRB,-0.3508301_JPRB,-0.5456543_JPRB,-0.7265625_JPRB,-0.7423096_JPRB&
  &,-0.7237549_JPRB,-0.4320068_JPRB,-0.4973145_JPRB,-0.3864746_JPRB,-0.4487305_JPRB,-0.2786865_JPRB&
  &,-0.1945801_JPRB,-0.4055176_JPRB,-0.5634766_JPRB,-0.2757568_JPRB,-0.5374756_JPRB,-0.5498047_JPRB&
  &,-0.4001465_JPRB,-0.1802979_JPRB,-0.3736572_JPRB,-0.4357910_JPRB,-0.8004150_JPRB,-0.5523682_JPRB&
  &,-0.4973145_JPRB,-0.6785889_JPRB,-0.6044922_JPRB,-0.7998047_JPRB,-0.6778564_JPRB,-0.5179443_JPRB&
  &,-0.4602051_JPRB,-0.6394043_JPRB,-0.6541748_JPRB,-0.2630615_JPRB,-0.6981201_JPRB,-0.5529785_JPRB&
  &,-0.8303223_JPRB,-0.6068115_JPRB,-0.8537598_JPRB,-0.6676025_JPRB,-0.5948486_JPRB,-0.5076904_JPRB&
  &,-0.4398193_JPRB,-0.4260254_JPRB,-0.2415771_JPRB,-0.6202393_JPRB,-0.5751953_JPRB,-0.4293213_JPRB&
  &,-0.4365234_JPRB,-0.4776611_JPRB,-0.5941162_JPRB,-0.7740479_JPRB,-0.6525879_JPRB,-0.6995850_JPRB&
  &,-0.5974121_JPRB,-0.3939209_JPRB,-0.5909424_JPRB,-0.5493164_JPRB,-0.3540039_JPRB,-0.5340576_JPRB&
  &,-0.5085449_JPRB,-0.4938965_JPRB,-0.7033691_JPRB,-0.7645264_JPRB,-0.7248535_JPRB,-0.8021240_JPRB&
  &,-0.8510742_JPRB,-0.7570801_JPRB,-0.6030273_JPRB,-0.6069336_JPRB,-0.8846436_JPRB,-0.7919922_JPRB&
  &,-0.7181396_JPRB,-0.8200684_JPRB,-0.7680664_JPRB,-0.9447021_JPRB,-1.0926514_JPRB,-0.9154053_JPRB&
  &,-0.9125977_JPRB,-0.7803955_JPRB,-1.0061035_JPRB,-0.9713135_JPRB,-0.8089600_JPRB,-0.9272461_JPRB&
  &,-0.7934570_JPRB,-0.7114258_JPRB,-0.9985352_JPRB,-0.9337158_JPRB,-0.9130859_JPRB,-0.9747314_JPRB&
  &,-0.8684082_JPRB,-0.9738770_JPRB,-0.9560547_JPRB,-0.9267578_JPRB,-0.9927979_JPRB,-0.8609619_JPRB&
  &,-0.9853516_JPRB,-1.0290527_JPRB,-0.9201660_JPRB,-0.9055176_JPRB,-0.9571533_JPRB,-0.9366455_JPRB&
  &,-0.8653564_JPRB,-0.7238770_JPRB,-0.8830566_JPRB,-0.9348145_JPRB,-0.9085693_JPRB,-1.0866699_JPRB&
  &,-0.9207764_JPRB,-0.7880859_JPRB,-0.9163818_JPRB,-0.9744873_JPRB,-0.9864502_JPRB,-0.8905029_JPRB /)

ZINSOLM(481:600) = (/ &
  & -0.9193115_JPRB,-0.8728027_JPRB,-0.9417725_JPRB,-0.9788818_JPRB,-0.9096680_JPRB,-0.9575195_JPRB&
  &,-0.8504639_JPRB,-0.9149170_JPRB,-0.7678223_JPRB,-0.9381104_JPRB,-0.8610840_JPRB,-0.8811035_JPRB&
  &,-0.8822021_JPRB,-0.6895752_JPRB,-0.8730469_JPRB,-0.7725830_JPRB,-0.4891357_JPRB,-0.3823242_JPRB&
  &,-0.4462891_JPRB,-0.6389160_JPRB,-0.6879883_JPRB,-0.3065186_JPRB,-0.5089111_JPRB,-0.6320801_JPRB&
  &,-0.3885498_JPRB,-0.7004395_JPRB,-0.4373779_JPRB,-0.2464600_JPRB,-0.3656006_JPRB,-0.1768799_JPRB&
  &,-0.6342773_JPRB, 0.0434570_JPRB,-0.3239746_JPRB,-0.1523438_JPRB,-0.2486572_JPRB,-0.0028076_JPRB&
  &,-0.2246094_JPRB,-0.3000488_JPRB,-0.2342529_JPRB,-0.0035400_JPRB,-0.0487061_JPRB,-0.0858154_JPRB&
  &,-0.2608643_JPRB,-0.1145020_JPRB,-0.4091797_JPRB,-0.2974854_JPRB,-0.2872314_JPRB,-0.4371338_JPRB&
  &,-0.2928467_JPRB,-0.2053223_JPRB,-0.2691650_JPRB,-0.3328857_JPRB,-0.1306152_JPRB,-0.1970215_JPRB&
  &,-0.0477295_JPRB,-0.2757568_JPRB,-0.0704346_JPRB,-0.2479248_JPRB,-0.1718750_JPRB,-0.3792725_JPRB&
  &,-0.2755127_JPRB,-0.2025146_JPRB,-0.3469238_JPRB,-0.1142578_JPRB,-0.2077637_JPRB,-0.2426758_JPRB&
  &,-0.2845459_JPRB,-0.4465332_JPRB,-0.3170166_JPRB,-0.3994141_JPRB,-0.4550781_JPRB,-0.2149658_JPRB&
  &,-0.5391846_JPRB,-0.3686523_JPRB,-0.2689209_JPRB,-0.3431396_JPRB,-0.5010986_JPRB,-0.4799805_JPRB&
  &,-0.3778076_JPRB,-0.5670166_JPRB,-0.4967041_JPRB,-0.5648193_JPRB,-0.5738525_JPRB,-0.4350586_JPRB&
  &,-0.9333496_JPRB,-0.9387207_JPRB,-0.6890869_JPRB,-0.5881348_JPRB,-0.7595215_JPRB,-0.8840332_JPRB&
  &,-0.4946289_JPRB,-0.8260498_JPRB,-0.4520264_JPRB,-0.6978760_JPRB,-0.8312988_JPRB,-0.7611084_JPRB&
  &,-0.6260986_JPRB,-0.5644531_JPRB,-0.5688477_JPRB,-0.7802734_JPRB,-0.6147461_JPRB,-0.5679932_JPRB&
  &,-0.8364258_JPRB,-0.6292725_JPRB,-0.8082275_JPRB,-0.6724854_JPRB,-0.5811768_JPRB,-0.8133545_JPRB&
  &,-0.7414551_JPRB,-0.8165283_JPRB,-0.8098145_JPRB,-0.8084717_JPRB,-0.8668213_JPRB,-0.7108154_JPRB&
  &,-0.8054199_JPRB,-0.7917480_JPRB,-0.7990723_JPRB,-0.7548828_JPRB,-0.8395996_JPRB,-0.8328857_JPRB /)

ZINSOLM(601:720) = (/ &
  & -0.8157959_JPRB,-0.7510986_JPRB,-0.9298096_JPRB,-0.7591553_JPRB,-0.7667236_JPRB,-0.7907715_JPRB&
  &,-0.8156738_JPRB,-0.7498779_JPRB,-0.8214111_JPRB,-0.8194580_JPRB,-0.8929443_JPRB,-0.9156494_JPRB&
  &,-0.9204102_JPRB,-0.8756104_JPRB,-0.8007812_JPRB,-0.8813477_JPRB,-0.8966064_JPRB,-0.9044189_JPRB&
  &,-0.9693604_JPRB,-0.9040527_JPRB,-0.8957520_JPRB,-0.8353271_JPRB,-0.9460449_JPRB,-0.8603516_JPRB&
  &,-0.9393311_JPRB,-1.0191650_JPRB,-0.8990479_JPRB,-0.9718018_JPRB,-0.9429932_JPRB,-0.9313965_JPRB&
  &,-0.9418945_JPRB,-0.9454346_JPRB,-0.9454346_JPRB,-0.8137207_JPRB,-0.8745117_JPRB,-1.0109863_JPRB&
  &,-0.9027100_JPRB,-0.8165283_JPRB,-0.9007568_JPRB,-0.7575684_JPRB,-0.8190918_JPRB,-0.8266602_JPRB&
  &,-0.6923828_JPRB,-0.5964355_JPRB,-0.8726807_JPRB,-1.0396729_JPRB,-0.7779541_JPRB,-0.5601807_JPRB&
  &,-0.7104492_JPRB,-0.8114014_JPRB,-0.5849609_JPRB,-0.7811279_JPRB,-0.4830322_JPRB,-0.4759521_JPRB&
  &,-0.4241943_JPRB,-0.3406982_JPRB,-0.6634521_JPRB,-0.4321289_JPRB,-0.6379395_JPRB,-0.5355225_JPRB&
  &,-0.6944580_JPRB,-0.6964111_JPRB,-0.7979736_JPRB,-0.5819092_JPRB,-0.5775146_JPRB,-0.5438232_JPRB&
  &,-0.7282715_JPRB,-0.6728516_JPRB,-0.4761963_JPRB,-0.8577881_JPRB,-0.3636475_JPRB,-0.5175781_JPRB&
  &,-0.6838379_JPRB,-0.7462158_JPRB,-0.5526123_JPRB,-0.4805908_JPRB,-0.6750488_JPRB,-0.4062500_JPRB&
  &,-0.1207275_JPRB,-0.6536865_JPRB,-0.3632812_JPRB,-0.8489990_JPRB,-0.7191162_JPRB,-0.6193848_JPRB&
  &,-0.4140625_JPRB,-0.6412354_JPRB,-0.4733887_JPRB,-0.5354004_JPRB,-0.6566162_JPRB,-0.9028320_JPRB&
  &,-0.6765137_JPRB,-0.5500488_JPRB,-0.3621826_JPRB,-0.5772705_JPRB,-0.5686035_JPRB,-0.7761230_JPRB&
  &,-0.6340332_JPRB,-0.5903320_JPRB,-0.6542969_JPRB,-0.4222412_JPRB,-0.5970459_JPRB,-0.5364990_JPRB&
  &,-0.5258789_JPRB,-0.4487305_JPRB,-0.4655762_JPRB,-0.6501465_JPRB,-0.6093750_JPRB,-0.6622314_JPRB&
  &,-0.6822510_JPRB,-0.6162109_JPRB,-0.4342041_JPRB,-0.6966553_JPRB,-0.6350098_JPRB,-0.7552490_JPRB&
  &,-0.6835938_JPRB,-0.6506348_JPRB,-0.7397461_JPRB,-0.4317627_JPRB,-0.4234619_JPRB,-0.5170898_JPRB /)

ZINSOLM(721:840) = (/ &
  & -0.7226562_JPRB,-0.7896729_JPRB,-0.7572021_JPRB,-0.8781738_JPRB,-0.7597656_JPRB,-0.8244629_JPRB&
  &,-0.8273926_JPRB,-0.8576660_JPRB,-0.7354736_JPRB,-0.5383301_JPRB,-0.9112549_JPRB,-0.8464355_JPRB&
  &,-0.9139404_JPRB,-0.8424072_JPRB,-0.8505859_JPRB,-0.7882080_JPRB,-0.7811279_JPRB,-0.9139404_JPRB&
  &,-0.8361816_JPRB,-0.8500977_JPRB,-0.8786621_JPRB,-0.7906494_JPRB,-0.9107666_JPRB,-0.8916016_JPRB&
  &,-0.9298096_JPRB,-0.8518066_JPRB,-0.8258057_JPRB,-0.9105225_JPRB,-0.8652344_JPRB,-0.9074707_JPRB&
  &,-0.8815918_JPRB,-0.9338379_JPRB,-0.7214355_JPRB,-0.7945557_JPRB,-0.8072510_JPRB,-0.8247070_JPRB&
  &,-0.8598633_JPRB,-0.7774658_JPRB,-0.7547607_JPRB,-0.7518311_JPRB,-0.7495117_JPRB,-0.7379150_JPRB&
  &,-0.7297363_JPRB,-0.6798096_JPRB,-0.6802979_JPRB,-0.7286377_JPRB,-0.7445068_JPRB,-0.6580811_JPRB&
  &,-0.6516113_JPRB,-0.7530518_JPRB,-0.7587891_JPRB,-0.6619873_JPRB,-0.7181396_JPRB,-0.6761475_JPRB&
  &,-0.7529297_JPRB,-0.8496094_JPRB,-0.7833252_JPRB,-0.6667480_JPRB,-0.6137695_JPRB,-0.5466309_JPRB&
  &,-0.5306396_JPRB,-0.3851318_JPRB,-0.3293457_JPRB,-0.5604248_JPRB,-0.4742432_JPRB,-0.1429443_JPRB&
  &,-0.2194824_JPRB,-0.0308838_JPRB,-0.2615967_JPRB,-0.1334229_JPRB,-0.2022705_JPRB,-0.3018799_JPRB&
  &,-0.1882324_JPRB,-0.1220703_JPRB,-0.0487061_JPRB, 0.1018066_JPRB, 0.0225830_JPRB,-0.2054443_JPRB&
  &,-0.0866699_JPRB,-0.2521973_JPRB,-0.1474609_JPRB,-0.1983643_JPRB, 0.1267090_JPRB,-0.1763916_JPRB&
  &, 0.0656738_JPRB,-0.2148438_JPRB, 0.3374023_JPRB, 0.0963135_JPRB, 0.4454346_JPRB, 0.3107910_JPRB&
  &, 0.4656982_JPRB, 0.0140381_JPRB, 0.3841553_JPRB, 0.0753174_JPRB, 0.4449463_JPRB, 0.1356201_JPRB&
  &,-0.0040283_JPRB, 0.0556641_JPRB, 0.1440430_JPRB, 0.2170410_JPRB, 0.1488037_JPRB, 0.0333252_JPRB&
  &, 0.4532471_JPRB, 0.0677490_JPRB, 0.2198486_JPRB, 0.1369629_JPRB, 0.1341553_JPRB,-0.2325439_JPRB&
  &,-0.1572266_JPRB,-0.0615234_JPRB,-0.1522217_JPRB,-0.1346436_JPRB,-0.0767822_JPRB, 0.1593018_JPRB&
  &,-0.0164795_JPRB,-0.2293701_JPRB,-0.2957764_JPRB,-0.1697998_JPRB,-0.3557129_JPRB,-0.2286377_JPRB /)

ZINSOLM(841:960) = (/ &
  & -0.4288330_JPRB,-0.0192871_JPRB,-0.2742920_JPRB,-0.6218262_JPRB,-0.2487793_JPRB,-0.2768555_JPRB&
  &,-0.4190674_JPRB,-0.4583740_JPRB,-0.5220947_JPRB,-0.1228027_JPRB,-0.5440674_JPRB,-0.3980713_JPRB&
  &,-0.3977051_JPRB,-0.3560791_JPRB,-0.4362793_JPRB,-0.3541260_JPRB,-0.6657715_JPRB,-0.3060303_JPRB&
  &,-0.2702637_JPRB,-0.4400635_JPRB,-0.5548096_JPRB,-0.5380859_JPRB,-0.6688232_JPRB,-0.5742188_JPRB&
  &,-0.5361328_JPRB,-0.5159912_JPRB,-0.4458008_JPRB,-0.5766602_JPRB,-0.6533203_JPRB,-0.5948486_JPRB&
  &,-0.5810547_JPRB,-0.6390381_JPRB,-0.6667480_JPRB,-0.6842041_JPRB,-0.7076416_JPRB,-0.7109375_JPRB&
  &,-0.6246338_JPRB,-0.7480469_JPRB,-0.6910400_JPRB,-0.6497803_JPRB,-0.6956787_JPRB,-0.5733643_JPRB&
  &,-0.6744385_JPRB,-0.6988525_JPRB,-0.5561523_JPRB,-0.5983887_JPRB,-0.5886230_JPRB,-0.7264404_JPRB&
  &,-0.7634277_JPRB,-0.6491699_JPRB,-0.7283936_JPRB,-0.5758057_JPRB,-0.4899902_JPRB,-0.5333252_JPRB&
  &,-0.4140625_JPRB,-0.5886230_JPRB,-0.5089111_JPRB,-0.5249023_JPRB,-0.6513672_JPRB,-0.5495605_JPRB&
  &,-0.6710205_JPRB,-0.4683838_JPRB,-0.5372314_JPRB,-0.3249512_JPRB,-0.3386230_JPRB,-0.2608643_JPRB&
  &,-0.2576904_JPRB,-0.3044434_JPRB,-0.0577393_JPRB,-0.2346191_JPRB,-0.4998779_JPRB,-0.8752441_JPRB&
  &,-0.9790039_JPRB,-0.5982666_JPRB,-0.3319092_JPRB,-0.2939453_JPRB,-0.1837158_JPRB,-0.1356201_JPRB&
  &,-0.2891846_JPRB,-0.1036377_JPRB,-0.3328857_JPRB,-0.2813721_JPRB,-0.0141602_JPRB,-0.1351318_JPRB&
  &,-0.3201904_JPRB, 0.0913086_JPRB,-0.0190430_JPRB, 0.1022949_JPRB, 0.1243896_JPRB,-0.1667480_JPRB&
  &,-0.2082520_JPRB,-0.3430176_JPRB,-0.1851807_JPRB,-0.0660400_JPRB,-0.0843506_JPRB,-0.2795410_JPRB&
  &,-0.1169434_JPRB,-0.1019287_JPRB,-0.1966553_JPRB,-0.1136475_JPRB,-0.1428223_JPRB, 0.0533447_JPRB&
  &,-0.1442871_JPRB,-0.0014648_JPRB,-0.4302979_JPRB,-0.1784668_JPRB,-0.2657471_JPRB,-0.2838135_JPRB&
  &,-0.0366211_JPRB,-0.0479736_JPRB,-0.4125977_JPRB,-0.3321533_JPRB,-0.1311035_JPRB,-0.0921631_JPRB&
  &,-0.0135498_JPRB, 0.0550537_JPRB,-0.2274170_JPRB,-0.4588623_JPRB,-0.6669922_JPRB,-0.6063232_JPRB /)

ZINSOLM(961:1080) = (/ &
  & -0.1557617_JPRB,-0.1890869_JPRB,-0.4127197_JPRB,-0.2952881_JPRB,-0.2758789_JPRB,-0.3117676_JPRB&
  &,-0.3658447_JPRB,-0.3798828_JPRB,-0.2915039_JPRB,-0.4199219_JPRB,-0.3842773_JPRB,-0.3253174_JPRB&
  &,-0.5008545_JPRB,-0.3273926_JPRB,-0.4122314_JPRB,-0.3144531_JPRB,-0.3613281_JPRB,-0.4277344_JPRB&
  &,-0.3867188_JPRB,-0.4527588_JPRB,-0.3780518_JPRB,-0.5223389_JPRB,-0.4230957_JPRB,-0.4177246_JPRB&
  &,-0.4898682_JPRB,-0.5020752_JPRB,-0.5213623_JPRB,-0.5433350_JPRB,-0.4769287_JPRB,-0.3972168_JPRB&
  &,-0.4985352_JPRB,-0.5378418_JPRB,-0.4805908_JPRB,-0.4731445_JPRB,-0.5400391_JPRB,-0.5474854_JPRB&
  &,-0.4311523_JPRB,-0.5195312_JPRB,-0.4978027_JPRB,-0.4880371_JPRB,-0.4913330_JPRB,-0.4930420_JPRB&
  &,-0.5699463_JPRB,-0.5192871_JPRB,-0.4515381_JPRB,-0.4932861_JPRB,-0.4624023_JPRB,-0.4475098_JPRB&
  &,-0.4848633_JPRB,-0.4195557_JPRB,-0.4804688_JPRB,-0.5394287_JPRB,-0.3753662_JPRB,-0.4810791_JPRB&
  &,-0.4309082_JPRB,-0.4223633_JPRB,-0.4614258_JPRB,-0.4233398_JPRB,-0.4172363_JPRB,-0.2823486_JPRB&
  &,-0.3548584_JPRB,-0.2830811_JPRB,-0.1901855_JPRB,-0.3458252_JPRB,-0.1752930_JPRB,-0.0335693_JPRB&
  &,-0.1860352_JPRB,-0.1818848_JPRB,-0.0122070_JPRB, 0.0754395_JPRB,-0.1951904_JPRB,-0.2897949_JPRB&
  &,-0.2631836_JPRB, 0.1815186_JPRB, 0.2895508_JPRB, 0.3023682_JPRB, 0.2325439_JPRB, 0.3582764_JPRB&
  &, 0.1595459_JPRB, 0.5678711_JPRB, 0.4036865_JPRB, 0.5175781_JPRB, 0.3771973_JPRB, 0.6094971_JPRB&
  &, 0.4388428_JPRB, 0.3422852_JPRB, 0.3891602_JPRB, 0.1424561_JPRB, 0.1778564_JPRB, 0.1684570_JPRB&
  &,-0.0081787_JPRB, 0.4097900_JPRB, 0.5019531_JPRB, 0.3120117_JPRB, 0.4598389_JPRB, 0.4470215_JPRB&
  &, 0.0368652_JPRB, 0.6047363_JPRB, 0.2846680_JPRB, 0.0302734_JPRB, 0.4268799_JPRB, 0.5803223_JPRB&
  &, 0.4007568_JPRB, 0.6380615_JPRB, 0.2796631_JPRB, 0.2413330_JPRB, 0.3046875_JPRB, 0.1701660_JPRB&
  &, 0.4182129_JPRB, 0.4508057_JPRB, 0.3807373_JPRB, 0.3762207_JPRB, 0.6567383_JPRB, 0.7720947_JPRB&
  &, 0.2304688_JPRB, 0.1431885_JPRB,-0.2259521_JPRB, 0.1319580_JPRB, 0.2362061_JPRB, 0.1384277_JPRB /)

ZINSOLM(1081:1200) = (/ &
  & -0.0592041_JPRB, 0.2169189_JPRB, 0.3232422_JPRB, 0.4288330_JPRB, 0.2927246_JPRB, 0.4320068_JPRB&
  &, 0.3985596_JPRB, 0.4311523_JPRB, 0.3457031_JPRB, 0.1037598_JPRB, 0.4263916_JPRB, 0.4095459_JPRB&
  &, 0.2641602_JPRB, 0.2497559_JPRB, 0.2709961_JPRB, 0.1141357_JPRB, 0.1335449_JPRB, 0.3590088_JPRB&
  &, 0.3403320_JPRB, 0.2868652_JPRB, 0.1342773_JPRB, 0.1112061_JPRB, 0.0621338_JPRB, 0.1440430_JPRB&
  &, 0.1805420_JPRB, 0.1387939_JPRB, 0.1849365_JPRB, 0.2861328_JPRB, 0.0302734_JPRB,-0.1131592_JPRB&
  &,-0.0438232_JPRB,-0.0322266_JPRB,-0.0102539_JPRB,-0.0118408_JPRB, 0.0402832_JPRB, 0.0180664_JPRB&
  &,-0.0676270_JPRB,-0.0268555_JPRB,-0.0915527_JPRB,-0.1844482_JPRB,-0.1340332_JPRB,-0.1522217_JPRB&
  &,-0.1461182_JPRB, 0.0329590_JPRB,-0.1252441_JPRB,-0.2209473_JPRB,-0.1049805_JPRB,-0.0426025_JPRB&
  &,-0.2084961_JPRB,-0.1883545_JPRB,-0.0643311_JPRB,-0.2121582_JPRB,-0.0787354_JPRB,-0.1635742_JPRB&
  &,-0.1054688_JPRB, 0.0806885_JPRB,-0.0753174_JPRB, 0.0021973_JPRB,-0.0692139_JPRB, 0.0496826_JPRB&
  &, 0.0552979_JPRB, 0.0111084_JPRB,-0.0823975_JPRB, 0.0937500_JPRB, 0.2358398_JPRB, 0.2745361_JPRB&
  &, 0.3255615_JPRB, 0.1582031_JPRB, 0.2353516_JPRB, 0.4702148_JPRB, 0.4813232_JPRB, 0.1712646_JPRB&
  &, 0.0175781_JPRB,-0.3732910_JPRB, 0.3618164_JPRB, 0.4129639_JPRB, 0.7343750_JPRB, 0.7866211_JPRB&
  &, 0.1811523_JPRB, 0.4284668_JPRB, 0.4675293_JPRB, 0.8894043_JPRB, 0.9052734_JPRB, 0.5083008_JPRB&
  &, 0.4920654_JPRB, 0.5234375_JPRB,-0.0551758_JPRB,-0.0120850_JPRB, 0.6179199_JPRB, 1.3234863_JPRB&
  &, 1.0087891_JPRB, 0.7631836_JPRB, 1.3588867_JPRB, 1.0570068_JPRB, 1.0461426_JPRB, 0.9766846_JPRB&
  &, 1.1719971_JPRB, 0.8450928_JPRB, 0.8631592_JPRB, 0.8123779_JPRB, 0.6070557_JPRB, 1.0974121_JPRB&
  &, 1.2554932_JPRB, 1.2551270_JPRB, 1.1466064_JPRB, 1.1632080_JPRB, 1.0657959_JPRB, 0.7666016_JPRB&
  &, 0.6927490_JPRB, 0.6347656_JPRB, 0.5861816_JPRB, 1.2812500_JPRB, 1.1176758_JPRB, 1.0812988_JPRB&
  &, 1.0477295_JPRB, 0.7189941_JPRB, 1.0739746_JPRB, 1.0877686_JPRB, 1.0162354_JPRB, 1.1491699_JPRB /)

ZINSOLM(1201:1320) = (/ &
  &  0.9489746_JPRB, 0.4852295_JPRB, 0.9218750_JPRB, 0.5921631_JPRB, 0.9302979_JPRB, 0.9060059_JPRB&
  &, 0.9058838_JPRB, 0.9271240_JPRB, 0.6804199_JPRB, 0.8066406_JPRB, 0.5991211_JPRB, 0.6682129_JPRB&
  &, 0.6740723_JPRB, 0.7751465_JPRB, 0.5711670_JPRB, 0.7305908_JPRB, 0.1385498_JPRB, 0.3216553_JPRB&
  &, 0.7117920_JPRB, 0.7711182_JPRB, 0.9783936_JPRB, 0.5638428_JPRB, 0.6458740_JPRB, 0.5510254_JPRB&
  &, 0.5439453_JPRB, 0.2918701_JPRB, 0.3787842_JPRB, 0.3989258_JPRB, 0.3438721_JPRB, 0.5125732_JPRB&
  &, 0.4158936_JPRB, 0.6956787_JPRB, 0.4344482_JPRB, 0.3293457_JPRB, 0.2324219_JPRB, 0.4468994_JPRB&
  &, 0.4322510_JPRB, 0.2795410_JPRB, 0.2474365_JPRB, 0.3872070_JPRB, 0.2469482_JPRB, 0.3715820_JPRB&
  &, 0.2662354_JPRB, 0.3823242_JPRB, 0.3704834_JPRB, 0.2518311_JPRB, 0.1695557_JPRB, 0.1685791_JPRB&
  &, 0.1137695_JPRB, 0.1555176_JPRB, 0.2541504_JPRB, 0.2061768_JPRB, 0.1520996_JPRB, 0.0975342_JPRB&
  &, 0.2352295_JPRB, 0.2797852_JPRB, 0.1308594_JPRB, 0.2829590_JPRB, 0.2940674_JPRB, 0.2299805_JPRB&
  &, 0.2656250_JPRB, 0.3750000_JPRB, 0.2153320_JPRB, 0.3217773_JPRB, 0.5570068_JPRB, 0.4792480_JPRB&
  &, 0.5529785_JPRB, 0.5922852_JPRB, 0.7136230_JPRB, 0.7135010_JPRB, 0.8074951_JPRB, 0.9711914_JPRB&
  &, 0.6040039_JPRB, 0.4721680_JPRB, 1.1652832_JPRB, 1.2606201_JPRB, 1.1916504_JPRB, 1.2398682_JPRB&
  &, 1.2750244_JPRB, 1.1202393_JPRB, 1.3785400_JPRB, 1.3441162_JPRB, 0.9256592_JPRB, 1.1466064_JPRB&
  &, 1.4591064_JPRB, 1.5904541_JPRB, 1.7065430_JPRB, 1.7301025_JPRB, 1.3474121_JPRB, 0.5584717_JPRB&
  &, 1.2154541_JPRB, 1.3790283_JPRB, 0.8187256_JPRB, 1.0050049_JPRB, 1.5640869_JPRB, 0.9462891_JPRB&
  &, 1.4453125_JPRB, 1.3940430_JPRB, 0.4709473_JPRB, 1.0205078_JPRB, 1.3361816_JPRB, 1.3857422_JPRB&
  &, 1.1728516_JPRB, 1.4821777_JPRB, 1.2412109_JPRB, 1.4549561_JPRB, 1.4777832_JPRB, 1.1058350_JPRB&
  &, 0.4310303_JPRB, 1.1363525_JPRB, 0.8310547_JPRB, 1.1892090_JPRB, 1.1716309_JPRB, 1.0545654_JPRB&
  &, 1.0385742_JPRB, 1.0341797_JPRB, 1.0959473_JPRB, 1.2344971_JPRB, 0.8588867_JPRB, 0.9011230_JPRB /)

ZINSOLM(1321:1440) = (/ &
  &  1.0487061_JPRB, 0.9053955_JPRB, 1.2207031_JPRB, 1.1662598_JPRB, 0.9656982_JPRB, 1.1899414_JPRB&
  &, 1.0948486_JPRB, 1.0052490_JPRB, 1.3889160_JPRB, 0.8916016_JPRB, 0.8220215_JPRB, 1.0922852_JPRB&
  &, 0.6856689_JPRB, 0.7237549_JPRB, 0.7481689_JPRB, 0.8258057_JPRB, 0.7369385_JPRB, 0.8820801_JPRB&
  &, 0.6627197_JPRB, 0.7700195_JPRB, 0.6516113_JPRB, 0.5393066_JPRB, 0.4589844_JPRB, 0.5325928_JPRB&
  &, 0.3867188_JPRB, 0.4437256_JPRB, 0.4454346_JPRB, 0.5378418_JPRB, 0.4504395_JPRB, 0.5700684_JPRB&
  &, 0.2348633_JPRB, 0.3142090_JPRB, 0.5546875_JPRB, 0.4957275_JPRB, 0.2944336_JPRB, 0.3127441_JPRB&
  &, 0.2121582_JPRB, 0.2960205_JPRB, 0.1812744_JPRB, 0.2983398_JPRB, 0.4471436_JPRB, 0.3927002_JPRB&
  &, 0.1940918_JPRB, 0.3522949_JPRB, 0.2811279_JPRB, 0.3320312_JPRB, 0.2381592_JPRB, 0.1546631_JPRB&
  &, 0.1136475_JPRB, 0.2016602_JPRB, 0.1033936_JPRB, 0.0084229_JPRB, 0.0296631_JPRB, 0.0297852_JPRB&
  &,-0.0538330_JPRB, 0.0272217_JPRB,-0.0798340_JPRB,-0.0554199_JPRB,-0.0306396_JPRB, 0.0404053_JPRB&
  &, 0.0699463_JPRB, 0.0432129_JPRB, 0.0212402_JPRB,-0.0319824_JPRB, 0.1666260_JPRB, 0.0690918_JPRB&
  &,-0.0078125_JPRB,-0.0280762_JPRB, 0.0262451_JPRB, 0.0689697_JPRB, 0.0168457_JPRB, 0.0140381_JPRB&
  &, 0.1661377_JPRB, 0.0885010_JPRB,-0.0858154_JPRB, 0.2280273_JPRB, 0.3715820_JPRB, 0.4677734_JPRB&
  &, 0.3774414_JPRB, 0.4027100_JPRB, 0.1389160_JPRB, 0.4376221_JPRB, 0.3917236_JPRB, 0.4682617_JPRB&
  &, 0.5843506_JPRB, 0.5163574_JPRB, 0.4248047_JPRB, 0.4809570_JPRB, 0.1820068_JPRB, 0.3309326_JPRB&
  &, 0.4665527_JPRB, 0.3326416_JPRB, 0.5640869_JPRB, 0.6116943_JPRB, 0.4459229_JPRB, 0.6590576_JPRB&
  &, 0.2672119_JPRB, 0.4830322_JPRB, 0.6427002_JPRB, 0.5480957_JPRB, 0.7100830_JPRB, 0.5145264_JPRB&
  &, 0.4682617_JPRB, 0.5754395_JPRB, 0.9019775_JPRB, 0.6966553_JPRB, 0.5266113_JPRB, 0.5650635_JPRB&
  &, 0.5474854_JPRB, 0.9017334_JPRB, 0.6206055_JPRB, 0.6237793_JPRB, 0.8833008_JPRB, 0.3803711_JPRB&
  &, 0.6745605_JPRB, 0.6387939_JPRB, 0.8021240_JPRB, 0.4411621_JPRB, 0.3598633_JPRB, 0.6660156_JPRB /)

ZINSOLM(1441:1560) = (/ &
  &  0.6595459_JPRB, 0.5330811_JPRB, 0.6850586_JPRB, 0.4650879_JPRB, 0.6247559_JPRB, 0.6630859_JPRB&
  &, 0.6810303_JPRB, 0.6103516_JPRB, 0.7430420_JPRB, 0.5325928_JPRB, 0.5054932_JPRB, 0.4525146_JPRB&
  &, 0.2371826_JPRB, 0.3419189_JPRB, 0.4650879_JPRB, 0.5333252_JPRB, 0.4073486_JPRB, 0.3793945_JPRB&
  &, 0.5729980_JPRB, 0.2193604_JPRB, 0.3450928_JPRB, 0.1750488_JPRB, 0.4035645_JPRB, 0.6828613_JPRB&
  &, 0.5989990_JPRB, 0.5557861_JPRB, 0.5322266_JPRB, 0.6267090_JPRB, 0.4696045_JPRB, 0.6156006_JPRB&
  &, 0.6600342_JPRB, 0.5816650_JPRB, 0.6372070_JPRB, 0.3409424_JPRB, 0.2668457_JPRB, 0.3416748_JPRB&
  &, 0.3986816_JPRB, 0.3809814_JPRB, 0.2724609_JPRB, 0.4624023_JPRB, 0.3930664_JPRB, 0.3314209_JPRB&
  &, 0.1113281_JPRB, 0.1746826_JPRB, 0.4160156_JPRB, 0.2150879_JPRB, 0.1800537_JPRB, 0.1563721_JPRB&
  &, 0.2801514_JPRB, 0.2163086_JPRB, 0.2005615_JPRB, 0.3310547_JPRB, 0.4011230_JPRB, 0.3913574_JPRB&
  &, 0.4519043_JPRB, 0.2745361_JPRB, 0.3061523_JPRB, 0.4105225_JPRB, 0.2011719_JPRB, 0.1732178_JPRB&
  &, 0.1828613_JPRB, 0.0948486_JPRB, 0.0979004_JPRB, 0.0163574_JPRB, 0.0959473_JPRB, 0.1066895_JPRB&
  &, 0.2673340_JPRB, 0.2678223_JPRB, 0.1475830_JPRB, 0.0671387_JPRB, 0.2128906_JPRB, 0.1324463_JPRB&
  &, 0.1304932_JPRB, 0.0888672_JPRB, 0.2083740_JPRB, 0.0778809_JPRB, 0.2542725_JPRB, 0.2795410_JPRB&
  &,-0.0139160_JPRB, 0.1307373_JPRB, 0.1225586_JPRB, 0.2938232_JPRB, 0.1368408_JPRB, 0.2445068_JPRB&
  &, 0.2519531_JPRB, 0.3564453_JPRB, 0.1270752_JPRB, 0.2242432_JPRB, 0.4479980_JPRB, 0.4515381_JPRB&
  &, 0.4117432_JPRB, 0.4567871_JPRB, 0.5001221_JPRB, 0.5278320_JPRB, 0.5220947_JPRB, 0.6387939_JPRB&
  &, 0.8074951_JPRB, 0.4716797_JPRB, 0.4466553_JPRB, 0.6413574_JPRB, 0.8314209_JPRB, 1.0579834_JPRB&
  &, 0.8826904_JPRB, 0.8641357_JPRB, 1.2717285_JPRB, 1.2926025_JPRB, 1.0029297_JPRB, 0.7014160_JPRB&
  &, 1.3457031_JPRB, 1.4536133_JPRB, 2.1147461_JPRB, 1.7207031_JPRB, 1.8306885_JPRB, 1.0101318_JPRB&
  &, 1.6041260_JPRB, 1.0673828_JPRB, 1.1827393_JPRB, 1.4176025_JPRB, 1.4420166_JPRB, 1.8271484_JPRB /)

ZINSOLM(1561:1680) = (/ &
  &  1.5811768_JPRB, 1.2242432_JPRB, 1.8155518_JPRB, 1.2635498_JPRB, 1.3132324_JPRB, 1.6065674_JPRB&
  &, 1.5599365_JPRB, 1.6564941_JPRB, 1.4934082_JPRB, 1.4903564_JPRB, 1.1126709_JPRB, 1.6823730_JPRB&
  &, 2.1666260_JPRB, 1.4066162_JPRB, 1.6798096_JPRB, 1.6832275_JPRB, 1.6606445_JPRB, 1.6551514_JPRB&
  &, 0.6584473_JPRB, 1.4431152_JPRB, 1.6376953_JPRB, 1.2836914_JPRB, 1.5460205_JPRB, 1.3992920_JPRB&
  &, 1.5163574_JPRB, 0.7548828_JPRB, 0.7503662_JPRB, 1.3254395_JPRB, 1.3002930_JPRB, 0.5585938_JPRB&
  &, 1.0417480_JPRB, 0.9897461_JPRB, 1.3747559_JPRB, 1.3411865_JPRB, 1.0443115_JPRB, 0.7481689_JPRB&
  &, 1.2258301_JPRB, 1.1234131_JPRB, 1.0500488_JPRB, 1.0566406_JPRB, 0.9154053_JPRB, 0.8989258_JPRB&
  &, 1.0715332_JPRB, 1.2479248_JPRB, 1.1208496_JPRB, 0.8917236_JPRB, 0.8768311_JPRB, 0.7956543_JPRB&
  &, 0.1981201_JPRB, 0.4016113_JPRB, 0.8878174_JPRB, 0.3176270_JPRB, 0.6619873_JPRB, 0.9838867_JPRB&
  &, 0.9804688_JPRB, 0.8571777_JPRB, 0.8297119_JPRB, 0.7169189_JPRB, 0.6291504_JPRB, 0.6832275_JPRB&
  &, 0.5820312_JPRB, 0.6140137_JPRB, 0.5168457_JPRB, 0.3620605_JPRB, 0.4283447_JPRB, 0.4918213_JPRB&
  &, 0.4588623_JPRB, 0.5371094_JPRB, 0.4996338_JPRB, 0.4056396_JPRB, 0.5095215_JPRB, 0.5166016_JPRB&
  &, 0.5057373_JPRB, 0.4138184_JPRB, 0.4968262_JPRB, 0.4677734_JPRB, 0.5380859_JPRB, 0.4932861_JPRB&
  &, 0.4492188_JPRB, 0.4127197_JPRB, 0.4442139_JPRB, 0.4832764_JPRB, 0.5744629_JPRB, 0.5162354_JPRB&
  &, 0.4705811_JPRB, 0.4321289_JPRB, 0.3834229_JPRB, 0.4256592_JPRB, 0.5178223_JPRB, 0.6036377_JPRB&
  &, 0.5770264_JPRB, 0.5625000_JPRB, 0.6389160_JPRB, 0.6750488_JPRB, 0.8542480_JPRB, 0.9582520_JPRB&
  &, 0.9393311_JPRB, 0.9161377_JPRB, 0.7030029_JPRB, 0.8255615_JPRB, 1.0462646_JPRB, 0.6923828_JPRB&
  &, 0.5441895_JPRB, 0.8251953_JPRB, 1.2447510_JPRB, 1.0587158_JPRB, 1.0828857_JPRB, 0.9786377_JPRB&
  &, 1.2026367_JPRB, 1.6140137_JPRB, 1.3376465_JPRB, 1.5478516_JPRB, 1.5554199_JPRB, 0.9255371_JPRB&
  &, 1.5750732_JPRB, 1.2227783_JPRB, 0.8590088_JPRB, 1.5617676_JPRB, 1.3713379_JPRB, 1.3878174_JPRB /)

ZINSOLM(1681:1800) = (/ &
  &  1.4655762_JPRB, 1.3358154_JPRB, 1.2139893_JPRB, 1.5332031_JPRB, 1.3658447_JPRB, 1.6545410_JPRB&
  &, 1.2442627_JPRB, 0.9606934_JPRB, 1.6166992_JPRB, 1.2329102_JPRB, 0.7941895_JPRB, 1.3223877_JPRB&
  &, 0.8269043_JPRB, 1.2515869_JPRB, 1.4791260_JPRB, 1.9494629_JPRB, 1.3112793_JPRB, 1.0718994_JPRB&
  &, 1.3924561_JPRB, 1.4024658_JPRB, 1.5172119_JPRB, 0.7301025_JPRB, 1.3500977_JPRB, 0.4396973_JPRB&
  &, 1.2391357_JPRB, 1.2890625_JPRB, 1.5253906_JPRB, 1.2296143_JPRB, 1.0875244_JPRB, 1.0590820_JPRB&
  &, 0.8309326_JPRB, 0.8905029_JPRB, 0.9705811_JPRB, 0.5184326_JPRB, 0.8751221_JPRB, 0.9306641_JPRB&
  &, 1.0527344_JPRB, 0.5184326_JPRB, 0.8621826_JPRB, 1.0252686_JPRB, 0.8581543_JPRB, 0.8320312_JPRB&
  &, 0.8638916_JPRB, 0.8715820_JPRB, 0.7069092_JPRB, 0.4321289_JPRB, 0.5329590_JPRB, 0.5161133_JPRB&
  &, 0.4595947_JPRB, 0.7368164_JPRB, 0.8117676_JPRB, 0.7264404_JPRB, 0.5742188_JPRB, 0.6384277_JPRB&
  &, 0.5611572_JPRB, 0.5925293_JPRB, 0.4766846_JPRB, 0.4913330_JPRB, 0.6788330_JPRB, 0.3597412_JPRB&
  &, 0.6038818_JPRB, 0.4847412_JPRB, 0.5517578_JPRB, 0.6197510_JPRB, 0.4848633_JPRB, 0.4766846_JPRB&
  &, 0.4573975_JPRB, 0.5213623_JPRB, 0.5310059_JPRB, 0.4274902_JPRB, 0.5622559_JPRB, 0.3199463_JPRB&
  &, 0.4888916_JPRB, 0.5069580_JPRB, 0.4224854_JPRB, 0.4323730_JPRB, 0.4195557_JPRB, 0.3601074_JPRB&
  &, 0.3068848_JPRB, 0.3236084_JPRB, 0.4495850_JPRB, 0.3848877_JPRB, 0.2767334_JPRB, 0.4916992_JPRB&
  &, 0.4899902_JPRB, 0.4851074_JPRB, 0.5103760_JPRB, 0.5258789_JPRB, 0.5561523_JPRB, 0.4952393_JPRB&
  &, 0.4831543_JPRB, 0.5074463_JPRB, 0.4514160_JPRB, 0.7419434_JPRB, 0.6231689_JPRB, 0.6828613_JPRB&
  &, 0.8248291_JPRB, 0.7181396_JPRB, 0.5894775_JPRB, 0.8336182_JPRB, 0.7250977_JPRB, 0.8503418_JPRB&
  &, 1.1383057_JPRB, 0.8526611_JPRB, 0.9094238_JPRB, 1.1306152_JPRB, 0.9317627_JPRB, 1.1096191_JPRB&
  &, 1.2613525_JPRB, 1.0196533_JPRB, 1.1168213_JPRB, 1.0205078_JPRB, 1.0311279_JPRB, 0.9674072_JPRB&
  &, 1.0782471_JPRB, 1.1671143_JPRB, 1.3685303_JPRB, 0.9180908_JPRB, 0.8964844_JPRB, 1.1486816_JPRB /)

ZINSOLM(1801:1848) = (/ &
  &  1.4390869_JPRB, 1.3453369_JPRB, 0.9357910_JPRB, 1.2257080_JPRB, 0.9549561_JPRB, 1.0341797_JPRB&
  &, 1.1075439_JPRB, 1.5349121_JPRB, 1.0858154_JPRB, 1.7623291_JPRB, 1.5391846_JPRB, 1.6878662_JPRB&
  &, 1.6337891_JPRB, 1.7200928_JPRB, 1.1361084_JPRB, 0.9962158_JPRB, 1.1579590_JPRB, 0.9003906_JPRB&
  &, 1.2360840_JPRB, 0.9765625_JPRB, 0.8106689_JPRB, 1.4261475_JPRB, 1.2322998_JPRB, 1.5828857_JPRB&
  &, 2.0045166_JPRB, 1.9329834_JPRB, 1.7397461_JPRB, 1.2086182_JPRB, 1.1928711_JPRB, 1.2495117_JPRB&
  &, 0.6547852_JPRB, 0.8068848_JPRB, 1.0555420_JPRB, 1.0684814_JPRB, 1.0422363_JPRB, 1.4594727_JPRB&
  &, 1.3198242_JPRB, 1.2974854_JPRB, 0.8085938_JPRB, 0.8233643_JPRB, 0.8262939_JPRB, 0.6127930_JPRB&
  &, 0.7316895_JPRB, 0.8566895_JPRB, 1.0091553_JPRB, 0.0020752_JPRB, 0.7579346_JPRB, 0.9428711_JPRB /)

                                                                                     
!*         1. SETTING CONSTANT VALUES
!             -----------------------

!!! IF (LHOOK) CALL DR_HOOK('UPDRGAS',0,ZHOOK_HANDLE)
IF(.NOT.LNF.AND.NSTADD == 0) THEN
! IN CASE OF RESTART:
  ITIME=NINT(TSTEP,KIND(ITIME))
  IF (LTWOTL) THEN
    IZT=NINT(TSTEP*(REAL(NSTAR2,JPRB)+0.5_JPRB),KIND(IZT))
  ELSE
    IZT=INT(ITIME,KIND(IZT))*INT(NSTAR2,KIND(IZT))
  ENDIF
  ISTADD=INT(IZT/NINT(RDAY,KIND(IZT)),KIND(ISTADD))
ELSE
  ISTADD=NSTADD
ENDIF

!*         1.1   TIME INDEX RELATED TO PRESENT CONDITIONS
!                ----------------------------------------

IYR0=NCCAA(NINDAT)
IMN0=NMM(NINDAT)
IDY0=NDD(NINDAT)

CALL UPDCAL(IDY0,IMN0,IYR0, ISTADD, IDY,IMN,IYR, ILMONTH, NULOUT)

IF (IMN <= 6) THEN
  IYR1=IYR-1
  IYR2=IYR
  IMN0=IMN+6
ELSEIF (IMN > 6) THEN
  IYR1=IYR
  IYR2=IYR+1
  IMN0=IMN-6
ENDIF
INDSC=MIN( (IYR-1850)*12+IMN, 1848 )
ZSOLINM=ZINSOLM(INDSC)+1365.0_JPRB

!*         2.    CONCENTRATIONS
!                --------------

!*         2.1   CONCENTRATIONS AS DEFINED IN ERA-40
!                -----------------------------------

!ZAIRMWG = 28.970_JPRB
!ZCO2MWG = 44.011_JPRB
!ZCH4MWG = 16.043_JPRB
!ZN2OMWG = 44.013_JPRB
!ZNO2MWG = 46.006_JPRB
!ZC11MWG = 137.3686_JPRB
!ZC12MWG = 120.9140_JPRB

!ZGASRMWG = ZGASMWG / ZAIRMWG

ZCO2RMWG = 1.5191923_JPRB
ZCH4RMWG = 0.5537798_JPRB
ZN2ORMWG = 1.5192613_JPRB
ZNO2RMWG = 1.5880566_JPRB
ZC11RMWG = 4.7417535_JPRB
ZC12RMWG = 4.1737660_JPRB

ZFIXNO2 = 500._JPRB

IF (.NOT.LHGHG) THEN
! to arrive here, LECO2VAR must be TRUE in naerad/yoerad

!*  Concentration of the various trace gases (IPCC/SACC values for 1990)
!        CO2         CH4        N2O        CFC11       CFC12
!      353ppmv     1.72ppmv   310ppbv     280pptv     484pptv

  ZCO2REF = 353.E-06_JPRB
  ZCH4REF = 1.72E-06_JPRB
  ZN2OREF = 310.E-09_JPRB
  ZNO2REF = 500.E-13_JPRB
  ZC11REF = 280.E-12_JPRB
  ZC12REF = 484.E-12_JPRB

  ZCO2CHG = 1.5E-06_JPRB
  ZCH4CHG = 12.E-09_JPRB
  ZN2OCHG = 0.8E-09_JPRB
  ZNO2CHG = 0.0E+00_JPRB
  ZC11CHG = 9.5E-12_JPRB
  ZC12CHG = 17.E-12_JPRB

!ZGASMCHG = ZGASCHG / 12.

  ZCO2MCHG = 0.125E-06_JPRB
  ZCH4MCHG = 1.000E-09_JPRB
  ZN2OMCHG = 6.667E-11_JPRB
  ZNO2MCHG = 0.000E+00_JPRB
  ZC11MCHG = 7.917E-13_JPRB
  ZC12MCHG = 1.417E-12_JPRB

  IRFY1 = IYR1 - 1990
  IRFY2 = IYR2 - 1990

  ZCO2 = ZCO2REF + IRFY1*ZCO2CHG + IMN0*ZCO2MCHG
  ZCH4 = ZCH4REF + IRFY1*ZCH4CHG + IMN0*ZCH4MCHG
  ZN2O = ZN2OREF + IRFY1*ZN2OCHG + IMN0*ZN2OMCHG
  ZNO2 = ZNO2REF + IRFY1*ZNO2CHG + IMN0*ZNO2MCHG
  ZC11 = ZC11REF + IRFY1*ZC11CHG + IMN0*ZC11MCHG
  ZC12 = ZC12REF + IRFY1*ZC12CHG + IMN0*ZC12MCHG
  ZC11 = MAX(0._JPRB, ZC11)
  ZC12 = MAX(0._JPRB, ZC12)

  RCARDI  = ZCO2 * ZCO2RMWG
  RCH4    = ZCH4 * ZCH4RMWG
  RN2O    = ZN2O * ZN2ORMWG
  RNO2    = ZNO2 * ZNO2RMWG
  RCFC11  = ZC11 * ZC11RMWG
  RCFC12  = ZC12 * ZC12RMWG


!print 9001, ZCO2, ZCH4, ZN2O, ZNO2, ZC11, ZC12
!9001 format(1x,'Conc.UMG: ',6E13.5)

ELSE

!*         2.2   CONCENTRATIONS AS DEFINED IN "ENSEMBLES" PROJECT
!                ------------------------------------------------

  CMIP: IF (.NOT.LCMIP5 .AND. .NOT.LCMIP6) THEN
  IF (IYR1 <  1850) IYR1=1850
  IF (IYR2 <= 1850) IYR2=1850

  IF (IYR1 >= 2100) IYR1=2100
  IF (IYR2 >  2100) IYR2=2100

!-- observed concentration for the period 1850-2000
  IF (IYR2 <= 2000) THEN
    IRFY1= IYR1-1849
    IRFY2= IYR2-1849
    DO JGAS=1,5
      ZGRADC= (ZHVAR(IRFY2,JGAS)-ZHVAR(IRFY1,JGAS))/12._JPRB
      ZCONC(JGAS) = ZHVAR(IRFY1,JGAS) + IMN0 * ZGRADC
    ENDDO
    ZCONC(6)=ZFIXNO2
    ZSOLINY=(ZINCSOL(IRFY2)+ZINCSOL(IRFY1))/2._JPRB

!-- choice of scenario (A1B, A2, B1) for 2000-2100
  ELSEIF (IYR1 >= 2000) THEN
    IRFY1= IYR1-1999
    IRFY2= IYR2-1999
    DO JGAS=1,5
      ZGRADC= (ZFVAR(IRFY2,NSCEN,JGAS)-ZFVAR(IRFY1,NSCEN,JGAS)) / 12._JPRB
      ZCONC(JGAS) = ZFVAR(IRFY1,NSCEN,JGAS) + IMN0 * ZGRADC        
    ENDDO
    ZCONC(6)=ZFIXNO2
    ZSOLINY=ZINCSOL(151)

  ENDIF

  IF(NHINCSOL == 3) THEN
!-- Solar constant for NHINCSOL=3. After 2008, use a 13 year repeat cycle
!   as specified in CMIP5. Accurate enough for the next few years.
!   Also apply factor 0.9965, to correct to TIM scale
    IRFY1=IYR
    IF(IYR < 1851) THEN
      CALL ABOR1(' UPDRGAS: solar data not available')
    ENDIF
    IF(IYR > 2008) THEN
      IRFY1=MOD((IYR-2009),13)+1996
    ENDIF
    ZSOLINY=ZCMIP5SOL(IRFY1)*0.9965
  ENDIF

  RCARDI  = ZCONC(1) * 1.E-06_JPRB * ZCO2RMWG
  RCH4    = ZCONC(2) * 1.E-09_JPRB * ZCH4RMWG
  RN2O    = ZCONC(3) * 1.E-09_JPRB * ZN2ORMWG
  RCFC11  = ZCONC(4) * 1.E-12_JPRB * ZC11RMWG
  RCFC12  = ZCONC(5) * 1.E-12_JPRB * ZC12RMWG
  RNO2    = ZCONC(6) * 1.E-13_JPRB * ZNO2RMWG

  IF (NHINCSOL == 0) THEN
    RSOLINC=RI0
  ELSEIF (NHINCSOL == 1) THEN
    RSOLINC=ZSOLINY
  ELSEIF (NHINCSOL == 2) THEN
    RSOLINC=ZSOLINM
  ELSEIF (NHINCSOL == 3) THEN
    RSOLINC=ZSOLINY
  ELSE
    CALL ABOR1(' UPDRGAS: invalid value for NHINCSOL')
  ENDIF

  ELSE CMIP

    ! -- GHG FORCING

    IF ( LCMIP6 .AND. IYR2.NE.IYR2OLD ) THEN
      ! call READCMIP6GHGDATA twice at the begin of a leg
      IF (IYR2OLD<0) THEN
        CALL READCMIP6GHGDATA(IYR1)
        WRITE(NULOUT,*) 'UPDRGAS Read CMIP6 GHG concentrations. IYR=',IYR1
      ENDIF
      CALL READCMIP6GHGDATA(IYR2)
      WRITE(NULOUT,*) 'UPDRGAS Read CMIP6 GHG concentrations. IYR=',IYR2
      IYR2OLD=IYR2
    ELSE IF ( LCMIP5 .AND. IYR2.NE.IYR2OLD ) THEN
      CALL READCMIP5GHGDATA
      WRITE(NULOUT,FMT='(''UPDRGAS Use CMIP5 GHG concentrations. NRCP= '',I1)') NRCP
      IYR2OLD=IYR2
    ENDIF

    DO JGAS=1,5
      ZGRADC= (ZFCONC(2,JGAS)-ZFCONC(1,JGAS)) / 12._JPRB
      ZCONC(JGAS) = ZFCONC(1,JGAS) + IMN0 * ZGRADC 
    ENDDO
    ZCONC(6)=ZFIXNO2

    WRITE(NULOUT,*)'UPDRGAS GHG'
    WRITE(NULOUT,FMT='('' IYR ='',I4,'' IMN ='',I4,'' IMN0 ='',I4 &
      & ,'' IYR1='',I4,'' IYR2='',I4)') &
      & IYR,IMN,IMN0,IYR1,IYR2
    DO JGAS=1,6
      WRITE(NULOUT,*) 'JGAS=',JGAS,' ZCONC(JGAS)=',ZCONC(JGAS)
    ENDDO 

    RCARDI  = ZCONC(1) * 1.E-06_JPRB * ZCO2RMWG
    RCH4    = ZCONC(2) * 1.E-09_JPRB * ZCH4RMWG
    RN2O    = ZCONC(3) * 1.E-09_JPRB * ZN2ORMWG
    RCFC11  = ZCONC(4) * 1.E-12_JPRB * ZC11RMWG
    RCFC12  = ZCONC(5) * 1.E-12_JPRB * ZC12RMWG
    RNO2    = ZCONC(6) * 1.E-13_JPRB * ZNO2RMWG

    ! MODIFY RQLIM SO TO SCALE THE PRODUCTION OF WATER BAPOUR BY CH4 OXIDATION
    !   RQLIM = RQLIM * [1.0 + (2.0/6.8)*Δ[CH4]_gm(t)].
    RQLIM = 4.25E-6_JPRB * (1.0_JPRB + (2.0_JPRB/6.8_JPRB)* (ZCONC(2) * 1.E-3_JPRB - RCH4_CL ))
    IF(IDY == 1) THEN
      WRITE(NULOUT,'(''IYR='',I4,'' IMN='',I2,'' IDY='',I2,'' CMIP6_GHGs='',6F8.2,'' Updated RQLIM='',E10.4)')  &
                    &  IYR,IMN,IDY,(ZCONC(JGAS),JGAS=1,6),RQLIM
      WRITE(NULOUT,*) 'Updating RQLIM=4.25e-6 * (1.+2./6.8 * Delt[CH4] )=',RQLIM
    ENDIF

    ! -- SOLAR FORCING 
    IF( LCMIP6 ) THEN
      IF (FIRST_CALL) THEN 
        CALL READCMIP6SOLARDATA
        FIRST_CALL = .FALSE.
      ENDIF

      IF (IYR < 1850 .OR. NCMIPFIXYR==1850) THEN ! Use the mean of the period 1850-01-01 to 1873-01-28 
        RSOLINC = 1360.7470703125 ! Based on solarforcing_picontrol_fx_3.2.nc from http://solarisheppa.geomar.de/cmip6 
      ELSE ! Use monthly values
        IF (IYR > 2299) THEN
          IYR_CMIP5 = 2299 - 10 + Modulo(IYR-2300,11) ! Repeat last solar cycle data after 2299
        ELSE
          IYR_CMIP5 = IYR
        END IF
        INDSC   = (IYR_CMIP5-1850)*12+IMN
        RSOLINC = ZSOLMONTH(INDSC)
      ENDIF
      WRITE(NULOUT,*) 'UPDRGAS SOLAR DATA'
      WRITE(NULOUT,*) 'RSOLINC=',RSOLINC
    ELSE
      IF (FIRST_CALL) THEN 
        CALL READCMIP5SOLARDATA
        FIRST_CALL = .FALSE.
      ENDIF

      IF (IYR < 1850 .OR. NCMIPFIXYR==1850) THEN ! Use the mean of the period 1844-1856
        RSOLINC = SUM(ZSOLYEAR(1844-1609:1856-1609))/13
      ELSE IF (IYR < 1882) THEN ! Use yearly mean values up to 1882
        ZGRADC  =(ZSOLYEAR(IYR2-1609)-ZSOLYEAR(IYR1-1609))/12._JPRB
        RSOLINC = ZSOLYEAR(IYR1-1609) + IMN0 * ZGRADC
      ELSE ! Use monthly values
        IF (IYR > 2008) THEN
          IYR_CMIP5 = 2008 - 12 + Modulo(IYR-2009,13) ! Repeat last solar cycle data after 2008
        ELSE
          IYR_CMIP5 = IYR
        END IF
        INDSC   = (IYR_CMIP5-1882)*12+IMN
        RSOLINC = ZSOLMONTH(INDSC)
      ENDIF
      WRITE(NULOUT,*) 'UPDRGAS SOLAR DATA'
      WRITE(NULOUT,*) 'RSOLINC=',RSOLINC
    ENDIF 

  ENDIF CMIP

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('UPDRGAS',1,ZHOOK_HANDLE)

CONTAINS

SUBROUTINE READCMIP5SOLARDATA

  USE YOMMP      ,ONLY : MYPROC
  USE MPL_MODULE ,ONLY : MPL_BROADCAST
  INTEGER ,PARAMETER :: RPRC=1,ITAG=12345

  INTEGER(KIND=JPIM) :: I, IUNIT
  LOGICAL            :: IS_OPEN
  REAL(KIND=JPRB)    :: SKIP
  CHARACTER(LEN=132) :: SKIP_LINE

  IF (MYPROC==RPRC) THEN
    !Find a free unit number
    DO I=10,1000
      INQUIRE(UNIT=I,OPENED=IS_OPEN)
      IF (.NOT.IS_OPEN) THEN
        IUNIT = I
        EXIT
      END IF
    ENDDO
    OPEN(UNIT=IUNIT,FILE=TRIM(CMIP5DATADIR)//'/CMIP5_TSI_WLS_ann_1610_2008.txt', &
      &  STATUS='OLD',FORM='FORMATTED')
    !Skip the first three lines
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    DO I=1,SIZE(ZSOLYEAR)
      READ(UNIT=IUNIT,FMT=*) SKIP,SKIP,ZSOLYEAR(I)
    ENDDO
    CLOSE(UNIT=IUNIT)
    OPEN(UNIT=IUNIT,FILE=TRIM(CMIP5DATADIR)//'/CMIP5_TSI_WLS_mon_1882_2008.txt', &
      &  STATUS='OLD',FORM='FORMATTED')
    !Skip the first three lines
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    DO I=1,1524
      READ(UNIT=IUNIT,FMT=*) SKIP,SKIP,SKIP,ZSOLMONTH(I)
    ENDDO
    CLOSE(UNIT=IUNIT)

!--- scaling suggested by CMIP5
    ZSOLYEAR(:)=0.9965*ZSOLYEAR(:)
    ZSOLMONTH(:)=0.9965*ZSOLMONTH(:)
  ENDIF
  ! distribute to all PROCS
  CALL MPL_BROADCAST(ZSOLYEAR,KTAG=ITAG+1,KROOT=RPRC,CDSTRING='CMIP5SOLAR-1: ')
  CALL MPL_BROADCAST(ZSOLMONTH,KTAG=ITAG+2,KROOT=RPRC,CDSTRING='CMIP5SOLAR-2: ')

  RETURN
END SUBROUTINE READCMIP5SOLARDATA

SUBROUTINE READCMIP6SOLARDATA

  USE YOMMP      ,ONLY : MYPROC
  USE MPL_MODULE ,ONLY : MPL_BROADCAST
  INTEGER ,PARAMETER :: RPRC=1,ITAG=12345

  INTEGER(KIND=JPIM) :: I, IUNIT
  LOGICAL            :: IS_OPEN
  REAL(KIND=JPRB)    :: SKIP
  CHARACTER(LEN=132) :: SKIP_LINE

  IF (MYPROC==RPRC) THEN
    !Find a free unit number
    DO I=10,1000
      INQUIRE(UNIT=I,OPENED=IS_OPEN)
      IF (.NOT.IS_OPEN) THEN
        IUNIT = I
        EXIT
      END IF
    ENDDO
    OPEN(UNIT=IUNIT,FILE=TRIM(CMIP6DATADIR)//'/solarforcing_ref_monthly.txt', &
      &  STATUS='OLD',FORM='FORMATTED')
    !Skip the first three lines
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    DO I=1,SIZE(ZSOLMONTH)
      READ(UNIT=IUNIT,FMT=*) SKIP,SKIP,ZSOLMONTH(I)
    ENDDO
    CLOSE(UNIT=IUNIT)

  ENDIF
  ! distribute to all PROCS
  CALL MPL_BROADCAST(ZSOLMONTH,KTAG=ITAG+2,KROOT=RPRC,CDSTRING='CMIP6SOLAR-1: ')

  RETURN
END SUBROUTINE READCMIP6SOLARDATA

SUBROUTINE READCMIP5GHGDATA

  USE YOMMP      ,ONLY : MYPROC
  USE MPL_MODULE ,ONLY : MPL_BROADCAST
  INTEGER ,PARAMETER :: RPRC=1, ITAG=12345

  INTEGER(KIND=JPIM) :: I,IUNIT,IOS,IYEAR,IYEAR1,IYEAR2
  LOGICAL            :: IS_OPEN, LFOUND_YR1, LFOUND_YR2
  REAL(KIND=JPRB)    :: ZFCONC1(5)
  CHARACTER(LEN=132) :: SKIP_LINE,FILENAME
  REAL ,ALLOCATABLE  :: ZFCONC_BUF(:)

  ALLOCATE(ZFCONC_BUF(SIZE(ZFCONC)))

  IF (MYPROC==RPRC) THEN
    !Find a free unit number
    DO I=10,1000
      INQUIRE(UNIT=I,OPENED=IS_OPEN)
      IF (.NOT.IS_OPEN) THEN
        IUNIT = I
        EXIT
      END IF
    ENDDO

    SELECT CASE (NRCP)
    CASE (0)
      FILENAME='ghg_histo.txt'
    CASE (1)
      FILENAME='ghg_rcp3PD.txt'
    CASE (2)
      FILENAME='ghg_rcp45.txt'
    CASE (3)
      FILENAME='ghg_rcp60.txt'
    CASE (4)
      FILENAME='ghg_rcp85.txt'
    CASE DEFAULT
      WRITE(NULOUT,*) 'ERROR: NRCP=',NRCP,' IS NOT PROPERLY DEFINED'
    END SELECT

    IF (NCMIPFIXYR>0) THEN
      IYEAR1=NCMIPFIXYR
      IYEAR2=NCMIPFIXYR
    ELSE
      IYEAR1=IYR1
      IYEAR2=IYR2
    ENDIF

    LFOUND_YR1=.FALSE.
    LFOUND_YR2=.FALSE.

    IF (LA4xCO2.OR.L1PCTCO2) CALL ABOR1('4xCO2 or 1pctCO2 not implemented for CMIP5 GHG forcing')

    OPEN(UNIT=IUNIT,FILE=TRIM(CMIP5DATADIR)//'/'//TRIM(FILENAME),STATUS='OLD',FORM='FORMATTED')
    !Skip the first two lines
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    READ(UNIT=IUNIT,FMT=*) SKIP_LINE
    DO
      READ(UNIT=IUNIT,FMT=*,IOSTAT=IOS) IYEAR, ZFCONC1
      IF (IYEAR==IYEAR1) THEN
        ZFCONC(1,:)=ZFCONC1
        LFOUND_YR1=.TRUE.
      ENDIF
      IF (IYEAR==IYEAR2) THEN
        ZFCONC(2,:)=ZFCONC1
        LFOUND_YR2=.TRUE.
      ENDIF
      IF ((LFOUND_YR1.AND.LFOUND_YR2).OR.IOS/=0) EXIT
    ENDDO
    CLOSE(UNIT=IUNIT)

    IF (IOS>0) THEN
      CALL ABOR1('Problem occurred when reading '//TRIM(CMIP5DATADIR)//'/'//TRIM(FILENAME))
    ELSE IF (IOS<0) THEN
      WRITE(NULOUT, *) 'End of file reached in '//TRIM(CMIP5DATADIR)//'/'//TRIM(FILENAME)
      WRITE(NULOUT, *) 'Last year found :',IYEAR
      CALL ABOR1('End of file reached in '//TRIM(CMIP5DATADIR)//'/'//TRIM(FILENAME))
    ENDIF
    
    ZFCONC_BUF=RESHAPE(ZFCONC,SHAPE(ZFCONC_BUF))
  ENDIF
  ! distribute to all PROCS
  CALL MPL_BROADCAST(ZFCONC_BUF,KTAG=ITAG+3,KROOT=RPRC,CDSTRING='CMIP5GHG-1: ')
  ZFCONC=RESHAPE(ZFCONC_BUF,SHAPE(ZFCONC))
  DEALLOCATE(ZFCONC_BUF)

  RETURN

END SUBROUTINE READCMIP5GHGDATA

SUBROUTINE READCMIP6GHGDATA(IYEAR)

  USE YOMMP      ,ONLY : MYPROC
  USE MPL_MODULE ,ONLY : MPL_BROADCAST
  INTEGER ,PARAMETER :: RPRC=1, ITAG=12345

  INTEGER(KIND=JPIM), INTENT(IN)  :: IYEAR

  REAL ,ALLOCATABLE  :: ZFCONC_BUF(:)

  CHARACTER(LEN=255) :: FILENAME, FILEID
  CHARACTER(LEN=10)  :: NC_NAME
  INTEGER(KIND=JPIM) :: I, IUNIT, IFIRSTYR, IDX

  CHARACTER (LEN=*), PARAMETER :: NY_NAME='time'
  INTEGER(KIND=JPIM) :: NY_VARID
  REAL(KIND=JPRB) :: ZZ_YEARS

  CHARACTER (LEN=*), PARAMETER :: CO2_FILE='mole-fraction-of-carbon-dioxide-in-air'
  CHARACTER (LEN=*), PARAMETER :: CO2_NAME='mole_fraction_of_carbon_dioxide_in_air'
  INTEGER(KIND=JPIM) :: CO2_VARID
  REAL(KIND=JPRB) :: ZZCO2

  CHARACTER (LEN=*), PARAMETER :: CH4_FILE='mole-fraction-of-methane-in-air'
  CHARACTER (LEN=*), PARAMETER :: CH4_NAME='mole_fraction_of_methane_in_air'
  INTEGER(KIND=JPIM) :: CH4_VARID
  REAL(KIND=JPRB) :: ZZCH4

  CHARACTER (LEN=*), PARAMETER :: N2O_FILE='mole-fraction-of-nitrous-oxide-in-air'
  CHARACTER (LEN=*), PARAMETER :: N2O_NAME='mole_fraction_of_nitrous_oxide_in_air'
  INTEGER(KIND=JPIM) :: N2O_VARID
  REAL(KIND=JPRB) :: ZZN2O

  CHARACTER (LEN=*), PARAMETER :: CFC11_FILE='mole-fraction-of-cfc11eq-in-air'
  CHARACTER (LEN=*), PARAMETER :: CFC11_NAME='mole_fraction_of_cfc11eq_in_air'
  INTEGER(KIND=JPIM) :: CFC11_VARID
  REAL(KIND=JPRB) :: ZZCFC11

  CHARACTER (LEN=*), PARAMETER :: CFC12_FILE='mole-fraction-of-cfc12-in-air'
  CHARACTER (LEN=*), PARAMETER :: CFC12_NAME='mole_fraction_of_cfc12_in_air'
  INTEGER(KIND=JPIM) :: CFC12_VARID
  REAL(KIND=JPRB) :: ZZCFC12


  ALLOCATE(ZFCONC_BUF(SIZE(ZFCONC)))

  IF (MYPROC==RPRC) THEN
    IF (IYEAR < 2015) THEN
      FILEID='_input4MIPs_GHGConcentrations_CMIP_UoM-CMIP-1-2-0_gr1-GMNHSH_0000-2014'
      ! The first year in the data set is 0
      IFIRSTYR=0
    ELSE 
      SELECT CASE (TRIM(SSPNAME))
        CASE ("SSP1-1.9")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-IMAGE-ssp119-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP1-2.6")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-IMAGE-ssp126-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP2-4.5")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-MESSAGE-GLOBIOM-ssp245-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP3-7.0")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-AIM-ssp370-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP3-LowNTCF")
          FILEID='_input4MIPs_GHGConcentrations_AerChemMIP_UoM-AIM-ssp370-lowNTCF-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP4-3.4")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-GCAM4-ssp434-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP4-6.0")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-GCAM4-ssp460-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP5-3.4-OS")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-REMIND-MAGPIE-ssp534-over-1-2-1_gr1-GMNHSH_2015-2500'
        CASE ("SSP5-8.5")
          FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-REMIND-MAGPIE-ssp585-1-2-1_gr1-GMNHSH_2015-2500'
        CASE DEFAULT
          ! default to SSP3-7.0 but only to finish historical runs (last step of historical)
          IF (.NOT.FIRST_CALL) THEN
            FILEID='_input4MIPs_GHGConcentrations_ScenarioMIP_UoM-AIM-ssp370-1-2-1_gr1-GMNHSH_2015-2500'
          ELSE
            CALL ABOR1('READCMIP6GHGDATA : Unknown scenario')
          ENDIF
      END SELECT
      ! The first year in the data set is 2015
      IFIRSTYR=2015
    END IF

    IF ( NCMIPFIXYR.GT.0 ) THEN
      IDX=NCMIPFIXYR-IFIRSTYR+1
    ELSE
      IDX=MIN(IYEAR,2500)-IFIRSTYR+1
    ENDIF

    ! Read in CO2 data
    WRITE(FILENAME,'(A)') TRIM(CMIP6DATADIR)//'/'//TRIM(CO2_FILE)//TRIM(FILEID)//'.nc'
    WRITE(NULOUT,*) TRIM(FILENAME)
    
    CALL CHECKGHG( NF_OPEN(FILENAME, NF_NOWRITE, IUNIT) )

    ! Check timestamp only from CO2 file
    CALL CHECKGHG( NF_INQ_VARID(IUNIT, NY_NAME, NY_VARID) )  
    CALL CHECKGHG( NF_GET_VARA_DOUBLE(IUNIT, NY_VARID, IDX, 1, ZZ_YEARS) )
    WRITE(NULOUT,*) 'ZZ_YEARS=',ZZ_YEARS

    CALL CHECKGHG( NF_INQ_VARID(IUNIT, CO2_NAME, CO2_VARID) )  
    CALL CHECKGHG( NF_GET_VARA_DOUBLE(IUNIT, CO2_VARID, (/1,IDX/), (/1,1/), ZZCO2) ) 
    CALL CHECKGHG( NF_CLOSE(IUNIT) )
    WRITE(NULOUT,*) 'ZZCO2=',ZZCO2

    ! Read in CH4 data
    WRITE(FILENAME,'(A)') TRIM(CMIP6DATADIR)//'/'//TRIM(CH4_FILE)//TRIM(FILEID)//'.nc'
    WRITE(NULOUT,*) TRIM(FILENAME)
  
    CALL CHECKGHG( NF_OPEN(FILENAME, NF_NOWRITE, IUNIT) )
    CALL CHECKGHG( NF_INQ_VARID(IUNIT, CH4_NAME, CH4_VARID) )  
    CALL CHECKGHG( NF_GET_VARA_DOUBLE(IUNIT, CH4_VARID, (/1,IDX/), (/1,1/), ZZCH4) ) 
    CALL CHECKGHG( NF_CLOSE(IUNIT) )
    WRITE(NULOUT,*) 'ZZCH4=',ZZCH4

    ! Read in N2O data
    WRITE(FILENAME,'(A)') TRIM(CMIP6DATADIR)//'/'//TRIM(N2O_FILE)//TRIM(FILEID)//'.nc'
    WRITE(NULOUT,*) TRIM(FILENAME)
  
    CALL CHECKGHG( NF_OPEN(FILENAME, NF_NOWRITE, IUNIT) )
    CALL CHECKGHG( NF_INQ_VARID(IUNIT, N2O_NAME, N2O_VARID) )  
    CALL CHECKGHG( NF_GET_VARA_DOUBLE(IUNIT, N2O_VARID, (/1,IDX/), (/1,1/), ZZN2O) ) 
    CALL CHECKGHG( NF_CLOSE(IUNIT) )
    WRITE(NULOUT,*) 'ZZN2O=',ZZN2O

    ! Read in CFC11 data
    WRITE(FILENAME,'(A)') TRIM(CMIP6DATADIR)//'/'//TRIM(CFC11_FILE)//TRIM(FILEID)//'.nc'
    WRITE(NULOUT,*) TRIM(FILENAME)
  
    CALL CHECKGHG( NF_OPEN(FILENAME, NF_NOWRITE, IUNIT) )
    CALL CHECKGHG( NF_INQ_VARID(IUNIT, CFC11_NAME, CFC11_VARID) )  
    CALL CHECKGHG( NF_GET_VARA_DOUBLE(IUNIT, CFC11_VARID, (/1,IDX/), (/1,1/), ZZCFC11) ) 
    CALL CHECKGHG( NF_CLOSE(IUNIT) )
    WRITE(NULOUT,*) 'ZZCFC11=',ZZCFC11

    ! Read in CFC12 data
    WRITE(FILENAME,'(A)') TRIM(CMIP6DATADIR)//'/'//TRIM(CFC12_FILE)//TRIM(FILEID)//'.nc'
    WRITE(NULOUT,*) TRIM(FILENAME)
  
    CALL CHECKGHG( NF_OPEN(FILENAME, NF_NOWRITE, IUNIT) )
    CALL CHECKGHG( NF_INQ_VARID(IUNIT, CFC12_NAME, CFC12_VARID) )  
    CALL CHECKGHG( NF_GET_VARA_DOUBLE(IUNIT, CFC12_VARID, (/1,IDX/), (/1,1/), ZZCFC12) ) 
    CALL CHECKGHG( NF_CLOSE(IUNIT) )
    WRITE(NULOUT,*) 'ZZCFC12=',ZZCFC12

    ! replace IYR1 with IYR2 and fill IYR2 with new values
    ZFCONC(1,:)=ZFCONC(2,:)

    ZFCONC(2,2)=ZZCH4
    ZFCONC(2,3)=ZZN2O
    ZFCONC(2,4)=ZZCFC11
    ZFCONC(2,5)=ZZCFC12

    IF (LA4xCO2) THEN
      ! Abrupt4xCO2 - The CO2 abruptly quadrupled of the NCMIPFIXYR level and held constant
      ZFCONC(2,1)=ZZCO2 * R4xCO2     
      WRITE(NULOUT,'(''LA4xCO2='',L1,'' CO2 concentration prescribed as '',F8.2)') &
        &   LA4xCO2,ZFCONC(2,1) 

    ELSE IF (L1PCTCO2) THEN
      ! 1pctCO2 - the CO2 increase at the rate of 1%/year until it reaches 4-times the NCMIPFIXYR level (at year 140)
      
      I=MIN(IYEAR-NCMIPFIXYR,140)
      ZFCONC(2,1)=ZZCO2 * EXP(I*LOG(1._JPRB+RCO2INC))

      WRITE(NULOUT,'(''L1PCTCO2='',L1,'' CO2 concentration prescribed as '',F8.2)') &
        &   L1PCTCO2,ZFCONC(2,1) 

    ELSE
      ! default, use the CO2 value read from the file
      ZFCONC(2,1)=ZZCO2
    ENDIF

    ZFCONC_BUF=RESHAPE(ZFCONC,SHAPE(ZFCONC_BUF))
  ENDIF
  ! distribute to all PROCS
  CALL MPL_BROADCAST(ZFCONC_BUF,KTAG=ITAG+3,KROOT=RPRC,CDSTRING='CMIP6GHG-1: ')
  ZFCONC=RESHAPE(ZFCONC_BUF,SHAPE(ZFCONC))
  DEALLOCATE(ZFCONC_BUF)

  RETURN
END SUBROUTINE READCMIP6GHGDATA

SUBROUTINE CHECKGHG(STATUS)
  INTEGER, INTENT ( IN) :: STATUS

  IF(STATUS /= NF_NOERR) THEN
    CALL ABOR1('READCMIP6GHGDATA: '//TRIM(NF_STRERROR(STATUS)))
  END IF
  RETURN
END SUBROUTINE CHECKGHG
    
END SUBROUTINE UPDRGAS
