#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SUECAEC ( KINDAT, KMINUT )

!**** *SUECAEC* - DEFINES NEW CLIMATOLOGICAL DISTRIBUTION OF AEROSOLS

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SUECAEC* FROM *UPDTIM*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===
!     AEROSOL TOTAL OPTICAL THICKNESS (0.55 MICRON)

!-- TEGEN ET AL. MONTHLY CLIMATOLOGY OF TROPOSPHERIC AEROSOLS

! RTAEBC : BLACK CARBON AEROSOL
! RTAEOR : ORGANIC AEROSOL
! RTAESD : SAND DUST AEROSOL
! RTAESS : SEA SALT AEROSOL
! RTAESU : SULFATE AEROSOL

!-- GISS HISTORY OF VOLCANIC AEROSOLS

! RTAEVO : VOLCANIC AEROSOL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    98/12/21

!     MODIFICATIONS.
!     --------------
!     990115: JJMorcrette, Volcanic aerosols
!     990525: JJMorcrette, Monthly Climatology of Tropospheric Aerosols
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.
!     JJ.Morcrette 20080206 revision handling GISS volcanic aerosols
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOEAERC   ,ONLY : RAERBC, RAEROR, RAERSD, RAERSS, RAERSU,&
 & REPAER, RTAEBC, RTAEOR, RTAESD, RTAESS, RTAESU, RTAEVO, RVOLCDEC
USE YOERAD    ,ONLY : LCMIP5, LCMIP6, NRCP, NCMIPFIXYR, NVOLCSTOP,&
 & CCMIP6_STRAT_SIMP, LCMIP6_STRATAER_SIMP,LCMIP6_STRATAER_BCKGD 
USE YOMLUN   , ONLY : NULOUT, NULERR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KINDAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMINUT 
!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!*       0.2   LOCAL ARRAYS.
!              -------------

! The data set is available from 1850, but the average over 1850-2014 is stored in the year 1849
!(same convention as for the GISS dataset)
INTEGER(KIND=JPIM),PARAMETER    :: ISHIST=1849, IEHIST=2100, INLAT=24,IU=999
REAL(KIND=JPRB),SAVE :: ZLATV(INLAT) , ZAERV(INLAT,ISHIST:IEHIST,12)
REAL(KIND=JPRB),SAVE :: ZYTIME(12), ZMDAY(12) 

INTEGER(KIND=JPIM) :: IDY, IM, IM1, IM2, IMN, IYR, IYR1, IYR2, JI, JL
INTEGER(KIND=JPIM) :: IDYADD, IDAYMIN, IT1, IT2, INYR, INMN, INDY, IMV1, IMV2
INTEGER(KIND=JPIM) :: ILMO(12)

INTEGER(KIND=JPIM) :: INLEV,IYEAR,ILAT,IDUM, IOS
REAL(KIND=JPRB) :: Z_GRID, ZINT, ZLAT, ZRES, ZRESM, ZRESP, ZTIMI, ZXTIME
REAL(KIND=JPRB) :: ZHOOK_HANDLE, ZW1, ZW2
REAL(KIND=JPRB)    :: ZAERV_D(12), ZLATV_D(INLAT)
CHARACTER(LEN=1000):: CLINE

INTEGER(KIND=JPIM),SAVE :: ISYEAR=IEHIST, IEYEAR=ISHIST
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

LOGICAL :: LLOPEN,LLSTRAT


#include "abor1.intfb.h"
#include "updcal.intfb.h"

#include "fcttim.h"      
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECAEC',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  IF (LCMIP6_STRATAER_SIMP) THEN
    ! CMIP6 forcing, vertically integrated (implemented by M. Menegoz, provided by Chris Roberts, ECMWF)
    WRITE(NULOUT,*)"SUECAEC: READ SIMPLIFIED CMIP6 STRATOSPHERIC AEROSOL FORCING FILE: "//TRIM(CCMIP6_STRAT_SIMP)
    
    LLOPEN=.FALSE.
    IOS= 0
    INQUIRE(FILE=CCMIP6_STRAT_SIMP,IOSTAT=IOS,EXIST=LLSTRAT,OPENED=LLOPEN)
    LLSTRAT = LLSTRAT .AND. (IOS == 0) .AND. .NOT.LLOPEN
    IF (.NOT.LLSTRAT) THEN
      WRITE(NULERR,'('' ERROR IN SUECAEC.F90 :'',&
       & '' THE FORCING FILE IS NOT AVAILABLE !'')')
      CALL ABOR1('SUECAEC')
    ENDIF
    
    INLEV=0
    ZAERV(:,:,:)=0.0_JPRB

!- Read Volcanic GISS data from file, in ascii format as originally provided:
!
!    Optical Thickness in <xx1>to<yy1> km Level
!    Year/Lat Jan   Feb   Mar   Apr   May   Jun   Jul   Aug   Sep   Oct   Nov   Dec 
!    <year1> <lat1>  <optical thickness, Jan> ... <optical thickness, Dec> 
!            <lat2>  <optical thickness, Jan> ... <optical thickness, Dec>
!             .....
!            <lat24> <optical thickness, Jan> ... <optical thickness, Dec>
!    <year2> <lat1>  <optical thickness, Jan> ... <optical thickness, Dec> 
!             .....
!    Optical Thickness in <xx2>to<yy2> km Level
!             .....
    
     OPEN(UNIT=IU,FILE=TRIM(CCMIP6_STRAT_SIMP),FORM='FORMATTED')

     ILAT=1
     DO IDUM=1,100*INLAT*(IEHIST-ISHIST+1)  ! Dummy counter up to 100 levels to prevent endless loop
       READ(IU,'(A1000)',END=999)CLINE
       IF (INDEX(CLINE,"Optical")/=0) THEN
          INLEV=INLEV+1
          WRITE(NULOUT,'(A,I2,":",X,A)')"SUECAEC: INLEV=",INLEV,TRIM(CLINE)
          CYCLE
       ENDIF
       IF (INDEX(CLINE,"Year")   /=0) CYCLE

       IF (ILAT==1) THEN
          READ(CLINE,*)IYEAR,ZLATV_D(ILAT),ZAERV_D(1:12)
          ISYEAR=MIN(ISYEAR,IYEAR)
          IEYEAR=MAX(IEYEAR,IYEAR)
       ELSE
          READ(CLINE,*)      ZLATV_D(ILAT),ZAERV_D(1:12)
       ENDIF
       ZLATV(ILAT) = REAL(ZLATV_D(ILAT),JPRB)

!    - Here, we use the total optical thickness vertically integrated
!    - Activating the flag LCMIP6_STRATAER_FULL skip this forcing, to use a level-dependent forcing
       ZAERV(ILAT,IYEAR,1:12)=ZAERV(ILAT,IYEAR,1:12)+REAL(ZAERV_D(1:12),JPRB)

       ILAT=MOD(ILAT,INLAT)+1
     ENDDO
     CALL ABOR1('SUECAEC: CMIP6 HISTORY FILE TOO LONG')
999  CONTINUE
     CLOSE (IU)
     
     ! sane
     IF (ISYEAR /= 1849 .or. IEYEAR /= 2014) THEN
        CALL ABOR1('SUECAEC: READ WRONG FILE')
     ENDIF

!  - Write out some statistics
     DO IYEAR=ISYEAR,IEYEAR
       WRITE(NULOUT,*)"SUECAEC: AVERAGE VOLCANIC OPTICAL DEPTH FOR YEAR ",&
       &               IYEAR,":, ",SUM(ZAERV(1:INLAT,IYEAR,1:12))/(INLAT*12)
     ENDDO

   ELSE

     ! Use hard-coded Volcanic GISS data
     ISYEAR=1849
     IEYEAR =2011
     CALL PART1
     CALL PART2
     CALL PART3
     CALL PART4

     ! Rescale to optical depth
     ZAERV(:,:,:)=1.E-04_JPRB*ZAERV(:,:,:)

   ENDIF

ENDIF
!     ------------------------------------------------------------------

!*         1.     TIME INDEX WITHIN AEROSOL CLIMATOLOGY
!                 -------------------------------------

!*         1.1    SEASONAL TIME INDEX WITHIN TEGEN'S AEROSOL CLIMATOLOGY
!                 ------------------------------------------------------

IDY=NDD(KINDAT)-1
IMN=NMM(KINDAT)
IF (IMN == 1) THEN
  ZXTIME=REAL(IDY)*1440._JPRB + KMINUT
ELSEIF (IMN == 2) THEN
  IF(IDY == 28) IDY=IDY-1
! A DAY IN FEB. IS 28.25*24*60/28=1452.8571min LONG.
  ZXTIME=44640._JPRB+REAL(IDY)*1452.8571_JPRB+KMINUT
ELSE
  ZXTIME=(ZMDAY(IMN-1)+REAL(IDY))*1440._JPRB+KMINUT
ENDIF
! 525960=MINUTES IN A SIDERAL YEAR (365.25d)
ZXTIME=MOD(ZXTIME,525960._JPRB)

IM1=0
IM2=0
IF (ZXTIME <= ZYTIME(1)) THEN
  IM1=12
  IM2=1
  ZTIMI=(ZYTIME(1)-ZXTIME)/44640._JPRB
ELSEIF(ZXTIME > ZYTIME(12)) THEN
  IM1=12
  IM2=1
  ZTIMI=(548280._JPRB-ZXTIME)/44640._JPRB
! 548280.=(365.25d + 15.5d)*24*60
ELSE
  DO IM=1,11
    IF (ZXTIME > ZYTIME(IM) .AND. ZXTIME <= ZYTIME(IM+1)) THEN
      IM1=IM
      IM2=IM+1
      ZTIMI=(ZXTIME-ZYTIME(IM2))/(ZYTIME(IM1)-ZYTIME(IM2))
    ENDIF
  ENDDO
  IF (IM1 == 0.OR. IM2 == 0 ) THEN
    CALL ABOR1('Problem with time interpolation in suecaec!')
  ENDIF
ENDIF

!*         1.2    TIME INDEX WITHIN GISS VOLCANIC AEROSOL CLIMATOLOGY
!                 ---------------------------------------------------

IYR=NCCAA(KINDAT)
IMN=NMM(KINDAT)
IDY=NDD(KINDAT)

IDYADD=KMINUT/1440
IDAYMIN=KMINUT-1440*IDYADD

CALL UPDCAL(IDY,IMN,IYR,IDYADD,INDY,INMN,INYR,ILMO,-1)

! exponential decay of volcanic aerosols after NVOLCSTOP=YYYYMM
IF ((100*INYR+INMN).GE.NVOLCSTOP) THEN
  IYR1=INT(NVOLCSTOP/100)
  IMV1=NVOLCSTOP-100*IYR1
  RVOLCDEC=EXP(-(REAL(INYR-IYR1)+REAL(INMN-IMV1)/12.))
  INYR=IYR1
  INMN=IMV1
  INDY=16
ELSE
  RVOLCDEC=1.
ENDIF

! Assume monthly means valid at 0Z on the 16th of each month and interpolate linearly
IF(INDY >= 16) THEN
  IMV1=INMN
  IMV2=1+MOD(INMN,12)
  IYR1=INYR
  IF(IMV1 /= 12) THEN
    IYR2=INYR
  ELSE
    IYR2=INYR+1
  ENDIF
  IT1=16
  IT2=16+ILMO(IMV1)
ELSE
  IMV1=1+MOD(INMN+10,12)
  IMV2=INMN
  IF(IMV1 /= 12) THEN
    IYR1=INYR
  ELSE
    IYR1=INYR-1
  ENDIF
  IYR2=INYR
  IT1=16-ILMO(IMV1)
  IT2=16
ENDIF
IF (.NOT.(LCMIP5.OR.LCMIP6)) THEN
  ! Ensure date within 1957-2010 range
  IYR1=MAX(IYR1,1957)
  IYR2=MAX(IYR2,1957)
  IYR1=MIN(IYR1,2010)
  IYR2=MIN(IYR2,2010)
ELSE IF (NCMIPFIXYR>0 .OR. LCMIP6_STRATAER_BCKGD) THEN
  ! use mean value for fixed-year runs
  IYR1=1849
  IYR2=1849
ELSE IF ( (LCMIP5 .AND. NRCP/=0 )&
     .OR. (LCMIP6_STRATAER_SIMP .AND. IYR2>IEYEAR)) THEN
  ! use mean value for scenario 
  IF (IYR1>IEYEAR) IYR1=1849
  IYR2=1849
ELSE
  ! Ensure date within 1850-2011 range
  IYR1=MAX(IYR1,1850)
  IYR2=MAX(IYR2,1850)
  IYR1=MIN(IYR1,IEYEAR)
  IYR2=MIN(IYR2,IEYEAR)
ENDIF

ZW1=REAL(IT2*1440-(INDY*1440+IDAYMIN),JPRB)/REAL((IT2-IT1)*1440,JPRB)
ZW2=1.0_JPRB-ZW1


!*         2.0    TOTAL OPTICAL THICKNESS
!                 -----------------------

!*         2.1    TROPOSPHERIC AEROSOL MONTHLY CLIMATOLOGY
!                 ----------------------------------------

!-- VECTOR OF LATITUDES FOR TEGEN ET AL. AEROSOL CLIMATOLOGY:

DO JI=1,46
  DO JL=1,72
    RTAEBC(JL,JI)=RAERBC(JL,JI,IM2)&
     & +ZTIMI*(RAERBC(JL,JI,IM1)-RAERBC(JL,JI,IM2))       
    RTAEOR(JL,JI)=RAEROR(JL,JI,IM2)&
     & +ZTIMI*(RAEROR(JL,JI,IM1)-RAEROR(JL,JI,IM2))       
    RTAESD(JL,JI)=RAERSD(JL,JI,IM2)&
     & +ZTIMI*(RAERSD(JL,JI,IM1)-RAERSD(JL,JI,IM2))       
    RTAESS(JL,JI)=RAERSS(JL,JI,IM2)&
     & +ZTIMI*(RAERSS(JL,JI,IM1)-RAERSS(JL,JI,IM2))       
    RTAESU(JL,JI)=RAERSU(JL,JI,IM2)&
     & +ZTIMI*(RAERSU(JL,JI,IM1)-RAERSU(JL,JI,IM2))  
  ENDDO
ENDDO

!*         2.2    VOLCANIC AEROSOL CLIMATOLOGY
!                 ----------------------------

! print *, 'suecaec: iyr1,imv1,zw1,iyr2,imv2,zw2',iyr1,imv1,zw1,iyr2,imv2,zw2

Z_GRID=4._JPRB
DO JI=1,46
  ZLAT=90._JPRB-(JI-1)*Z_GRID
  DO JL=1,23
    IF (ZLAT == ZLATV(JL)) THEN
      ZRES=ZW1*ZAERV(JL,IYR1,IMV1)+ZW2*ZAERV(JL,IYR2,IMV2)
    ELSEIF (ZLAT < ZLATV(JL).AND.ZLAT >= ZLATV(JL+1))THEN
      ZINT=(ZLAT-ZLATV(JL+1))/(ZLATV(JL)-ZLATV(JL+1))
      ZRESM=ZW1*ZAERV(JL  ,IYR1,IMV1)+ZW2*ZAERV(JL  ,IYR2,IMV2)
      ZRESP=ZW1*ZAERV(JL+1,IYR1,IMV1)+ZW2*ZAERV(JL+1,IYR2,IMV2)
      ZRES=ZRESP-ZINT*(ZRESP-ZRESM)
    ENDIF
  ENDDO
!  RTAEVO(JI) is now total optical thickness
!  RTAEVO(JI)=ZRES*1.E-04_JPRB
  RTAEVO(JI)=ZRES ! The factor has already been applied before
ENDDO

!-- security for aerosol optical thickness
REPAER=1.E-12_JPRB

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SUECAEC',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS


SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SUECAEC:PART1',0,ZHOOK_HANDLE)


ZMDAY = (/&
 & 31._JPRB,    59.25_JPRB,  90.25_JPRB, 120.25_JPRB, 151.25_JPRB, 181.25_JPRB &
 & ,  212.25_JPRB, 243.25_JPRB, 273.25_JPRB, 304.25_JPRB, 334.25_JPRB, 365.25_JPRB &
 & /)  
ZYTIME= (/&
 & 22320._JPRB,  64980._JPRB, 107640._JPRB, 151560._JPRB, 195480._JPRB, 239400._JPRB &
 & , 283320._JPRB, 327960._JPRB, 371880._JPRB, 415800._JPRB, 459720._JPRB, 503640._JPRB &
 & /)  

!     ------------------------------------------------------------------

!     VOLCANIC AEROSOL OPTICAL THICKNESS (GISS)     

!- reference latitudes

ZLATV(:) = (/&
 & 90.0_JPRB, 82.2_JPRB, 74.3_JPRB, 66.5_JPRB, 58.7_JPRB, 50.9_JPRB, 43.0_JPRB, 35.2_JPRB, 27.4_JPRB, 19.6_JPRB, 11.7_JPRB, &
 & 3.9_JPRB, -3.9_JPRB,-11.7_JPRB,-19.6_JPRB,-27.4_JPRB,-35.2_JPRB,-43.0_JPRB,-50.9_JPRB,-58.7_JPRB,-66.5_JPRB,-74.3_JPRB,&
 & -82.2_JPRB,-90.0_JPRB /)   

!- total optical thickness * 10000

! K.Wyser: The GISS dataset covers 1850-2011. The mean value is
! stored here as year 1849

ZAERV(: ,1849, 1) = (/&
 &  147,  148,  154,  153,  159,  158,  152,  143,  118,  128,  141,  150,&
 &  144,  138,  120,  106,  111,  117,  124,  129,  130,  133,  131,  132/)
ZAERV(: ,1849, 2) = (/&
 &  156,  156,  161,  159,  163,  159,  151,  141,  114,  122,  133,  142,&
 &  134,  126,  112,  103,  111,  118,  126,  130,  130,  133,  131,  132/)
ZAERV(: ,1849, 3) = (/&
 &  158,  158,  162,  161,  161,  153,  144,  135,  110,  119,  129,  138,&
 &  130,  123,  109,  103,  111,  117,  124,  129,  130,  132,  129,  130/)
ZAERV(: ,1849, 4) = (/&
 &  154,  155,  159,  159,  156,  147,  138,  130,  107,  132,  124,  133,&
 &  126,  120,  108,  102,  112,  117,  123,  126,  129,  131,  127,  127/)
ZAERV(: ,1849, 5) = (/&
 &  152,  153,  156,  155,  150,  144,  135,  126,  111,  132,  128,  130,&
 &  125,  118,  107,  104,  115,  119,  123,  125,  127,  129,  125,  125/)
ZAERV(: ,1849, 6) = (/&
 &  150,  150,  153,  151,  146,  140,  131,  123,  108,  128,  130,  132,&
 &  126,  117,  108,  105,  116,  122,  124,  125,  127,  128,  124,  123/)
ZAERV(: ,1849, 7) = (/&
 &  148,  148,  149,  147,  143,  138,  130,  124,  109,  128,  134,  140,&
 &  132,  122,  109,  107,  118,  123,  125,  127,  128,  128,  125,  124/)
ZAERV(: ,1849, 8) = (/&
 &  147,  147,  147,  146,  142,  137,  130,  125,  110,  127,  133,  145,&
 &  138,  127,  110,  105,  115,  121,  122,  126,  126,  125,  124,  123/)
ZAERV(: ,1849, 9) = (/&
 &  143,  143,  144,  143,  140,  136,  130,  125,  113,  130,  139,  151,&
 &  142,  130,  112,  104,  113,  118,  120,  118,  116,  116,  116,  115/)
ZAERV(: ,1849,10) = (/&
 &  143,  143,  144,  142,  141,  138,  134,  130,  119,  136,  147,  160,&
 &  150,  139,  121,  111,  118,  123,  124,  118,  117,  115,  114,  113/)
ZAERV(: ,1849,11) = (/&
 &  142,  142,  143,  140,  140,  141,  137,  133,  118,  133,  144,  158,&
 &  152,  145,  126,  111,  115,  121,  124,  122,  120,  119,  117,  117/)
ZAERV(: ,1849,12) = (/&
 &  143,  143,  148,  146,  148,  151,  147,  139,  117,  129,  138,  151,&
 &  146,  139,  121,  107,  111,  117,  123,  125,  124,  126,  125,  126/)

! GISS dataset 1850-2011

ZAERV(: ,1850, 1) = (/&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1850, 2) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1850, 3) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1850, 4) = (/&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1850, 5) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1850, 6) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1850, 7) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1850, 8) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1850, 9) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1850,10) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1850,11) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1850,12) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1851, 1) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1851, 2) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1851, 3) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1851, 4) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1851, 5) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1851, 6) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1851, 7) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1851, 8) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1851, 9) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1851,10) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1851,11) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1851,12) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1852, 1) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1852, 2) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1852, 3) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1852, 4) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1852, 5) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1852, 6) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1852, 7) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1852, 8) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1852, 9) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1852,10) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1852,11) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1852,12) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1853, 1) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1853, 2) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1853, 3) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1853, 4) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1853, 5) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1853, 6) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1853, 7) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1853, 8) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1853, 9) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1853,10) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1853,11) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1853,12) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1854, 1) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1854, 2) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1854, 3) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1854, 4) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1854, 5) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1854, 6) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1854, 7) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1854, 8) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1854, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1854,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1854,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1854,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1855,10) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1855,11) = (/&
 &   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,&
 &   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75/)
ZAERV(: ,1855,12) = (/&
 &  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,&
 &  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122/)
ZAERV(: ,1856, 1) = (/&
 &  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,&
 &  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,  168,  168/)
ZAERV(: ,1856, 2) = (/&
 &  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,&
 &  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,  214/)
ZAERV(: ,1856, 3) = (/&
 &  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,&
 &  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,  260/)
ZAERV(: ,1856, 4) = (/&
 &  306,  306,  306,  306,  306,  306,  306,  306,  306,  306,  306,  306,&
 &  306,  306,  306,  306,  306,  306,  306,  306,  306,  306,  306,  306/)
ZAERV(: ,1856, 5) = (/&
 &  352,  352,  352,  352,  352,  352,  352,  352,  352,  352,  352,  352,&
 &  352,  352,  352,  352,  352,  352,  352,  352,  352,  352,  352,  352/)
ZAERV(: ,1856, 6) = (/&
 &  399,  399,  399,  399,  399,  399,  399,  399,  399,  399,  399,  399,&
 &  399,  399,  399,  399,  399,  399,  399,  399,  399,  399,  399,  399/)
ZAERV(: ,1856, 7) = (/&
 &  433,  433,  433,  433,  433,  433,  433,  433,  433,  433,  433,  433,&
 &  433,  433,  433,  433,  433,  433,  433,  433,  433,  433,  433,  433/)
ZAERV(: ,1856, 8) = (/&
 &  456,  456,  456,  456,  456,  456,  456,  456,  456,  456,  456,  456,&
 &  456,  456,  456,  456,  456,  456,  456,  456,  456,  456,  456,  456/)
ZAERV(: ,1856, 9) = (/&
 &  479,  479,  479,  479,  479,  479,  479,  479,  479,  479,  479,  479,&
 &  479,  479,  479,  479,  479,  479,  479,  479,  479,  479,  479,  479/)
ZAERV(: ,1856,10) = (/&
 &  501,  501,  502,  500,  502,  502,  500,  501,  500,  502,  501,  501,&
 &  501,  501,  501,  501,  500,  501,  501,  501,  500,  501,  502,  501/)
ZAERV(: ,1856,11) = (/&
 &  524,  524,  525,  523,  525,  525,  523,  524,  523,  525,  524,  524,&
 &  524,  524,  524,  524,  523,  524,  524,  524,  523,  524,  525,  524/)
ZAERV(: ,1856,12) = (/&
 &  547,  547,  548,  546,  548,  548,  546,  547,  546,  548,  547,  547,&
 &  547,  547,  547,  547,  546,  547,  547,  547,  546,  547,  548,  547/)
ZAERV(: ,1857, 1) = (/&
 &  570,  570,  571,  569,  571,  571,  569,  570,  569,  571,  570,  570,&
 &  570,  570,  570,  570,  569,  570,  570,  570,  569,  570,  571,  570/)
ZAERV(: ,1857, 2) = (/&
 &  593,  593,  594,  592,  594,  594,  592,  593,  592,  594,  593,  593,&
 &  593,  593,  593,  593,  592,  593,  593,  593,  592,  593,  594,  593/)
ZAERV(: ,1857, 3) = (/&
 &  615,  615,  616,  614,  616,  616,  614,  615,  614,  616,  615,  615,&
 &  615,  615,  615,  615,  614,  615,  615,  615,  614,  615,  616,  615/)
ZAERV(: ,1857, 4) = (/&
 &  638,  638,  639,  637,  639,  639,  637,  638,  637,  639,  638,  638,&
 &  638,  638,  638,  638,  637,  638,  638,  638,  637,  638,  639,  638/)
ZAERV(: ,1857, 5) = (/&
 &  661,  661,  662,  660,  662,  662,  660,  661,  660,  662,  661,  661,&
 &  661,  661,  661,  661,  660,  661,  661,  661,  660,  661,  662,  661/)
ZAERV(: ,1857, 6) = (/&
 &  684,  684,  685,  683,  685,  685,  683,  684,  683,  685,  684,  684,&
 &  684,  684,  684,  684,  683,  684,  684,  684,  683,  684,  685,  684/)
ZAERV(: ,1857, 7) = (/&
 &  675,  675,  676,  674,  676,  676,  674,  675,  674,  676,  675,  675,&
 &  675,  675,  675,  675,  674,  675,  675,  675,  674,  675,  676,  675/)
ZAERV(: ,1857, 8) = (/&
 &  636,  636,  637,  635,  637,  637,  635,  636,  635,  637,  636,  636,&
 &  636,  636,  636,  636,  635,  636,  636,  636,  635,  636,  637,  636/)
ZAERV(: ,1857, 9) = (/&
 &  596,  596,  597,  595,  597,  597,  595,  596,  595,  597,  596,  596,&
 &  596,  596,  596,  596,  595,  596,  596,  596,  595,  596,  597,  596/)
ZAERV(: ,1857,10) = (/&
 &  557,  557,  558,  556,  558,  558,  556,  557,  556,  558,  557,  557,&
 &  557,  557,  557,  557,  556,  557,  557,  557,  556,  557,  558,  557/)
ZAERV(: ,1857,11) = (/&
 &  517,  517,  518,  516,  518,  518,  516,  517,  516,  518,  517,  517,&
 &  517,  517,  517,  517,  516,  517,  517,  517,  516,  517,  518,  517/)
ZAERV(: ,1857,12) = (/&
 &  478,  478,  478,  478,  478,  478,  478,  478,  478,  478,  478,  478,&
 &  478,  478,  478,  478,  478,  478,  478,  478,  478,  478,  478,  478/)
ZAERV(: ,1858, 1) = (/&
 &  440,  440,  440,  440,  440,  440,  440,  440,  440,  440,  440,  440,&
 &  440,  440,  440,  440,  440,  440,  440,  440,  440,  440,  440,  440/)
ZAERV(: ,1858, 2) = (/&
 &  405,  405,  405,  405,  405,  405,  405,  405,  405,  405,  405,  405,&
 &  405,  405,  405,  405,  405,  405,  405,  405,  405,  405,  405,  405/)
ZAERV(: ,1858, 3) = (/&
 &  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,&
 &  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,  372/)
ZAERV(: ,1858, 4) = (/&
 &  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,&
 &  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,  343/)
ZAERV(: ,1858, 5) = (/&
 &  315,  315,  315,  315,  315,  315,  315,  315,  315,  315,  315,  315,&
 &  315,  315,  315,  315,  315,  315,  315,  315,  315,  315,  315,  315/)
ZAERV(: ,1858, 6) = (/&
 &  290,  290,  290,  290,  290,  290,  290,  290,  290,  290,  290,  290,&
 &  290,  290,  290,  290,  290,  290,  290,  290,  290,  290,  290,  290/)
ZAERV(: ,1858, 7) = (/&
 &  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,&
 &  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,  267/)
ZAERV(: ,1858, 8) = (/&
 &  246,  246,  246,  246,  246,  246,  246,  246,  246,  246,  246,  246,&
 &  246,  246,  246,  246,  246,  246,  246,  246,  246,  246,  246,  246/)
ZAERV(: ,1858, 9) = (/&
 &  226,  226,  226,  226,  226,  226,  226,  226,  226,  226,  226,  226,&
 &  226,  226,  226,  226,  226,  226,  226,  226,  226,  226,  226,  226/)
ZAERV(: ,1858,10) = (/&
 &  208,  208,  208,  208,  208,  208,  208,  208,  208,  208,  208,  208,&
 &  208,  208,  208,  208,  208,  208,  208,  208,  208,  208,  208,  208/)
ZAERV(: ,1858,11) = (/&
 &  192,  192,  192,  192,  192,  192,  192,  192,  192,  192,  192,  192,&
 &  192,  192,  192,  192,  192,  192,  192,  192,  192,  192,  192,  192/)
ZAERV(: ,1858,12) = (/&
 &  176,  176,  176,  176,  176,  176,  176,  176,  176,  176,  176,  176,&
 &  176,  176,  176,  176,  176,  176,  176,  176,  176,  176,  176,  176/)
ZAERV(: ,1859, 1) = (/&
 &  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,&
 &  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162/)
ZAERV(: ,1859, 2) = (/&
 &  150,  150,  150,  150,  150,  150,  150,  150,  150,  150,  150,  150,&
 &  150,  150,  150,  150,  150,  150,  150,  150,  150,  150,  150,  150/)
ZAERV(: ,1859, 3) = (/&
 &  139,  139,  139,  139,  139,  139,  139,  139,  139,  139,  139,  139,&
 &  139,  139,  139,  139,  139,  139,  139,  139,  139,  139,  139,  139/)
ZAERV(: ,1859, 4) = (/&
 &  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,&
 &  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130/)
ZAERV(: ,1859, 5) = (/&
 &  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,&
 &  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120/)
ZAERV(: ,1859, 6) = (/&
 &  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,&
 &  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110/)
ZAERV(: ,1859, 7) = (/&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103/)
ZAERV(: ,1859, 8) = (/&
 &   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,&
 &   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97/)
ZAERV(: ,1859, 9) = (/&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91/)
ZAERV(: ,1859,10) = (/&
 &   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,&
 &   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,   86/)
ZAERV(: ,1859,11) = (/&
 &   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,&
 &   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80/)
ZAERV(: ,1859,12) = (/&
 &   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,&
 &   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74/)
ZAERV(: ,1860, 1) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69/)
ZAERV(: ,1860, 2) = (/&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64/)
ZAERV(: ,1860, 3) = (/&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59/)
ZAERV(: ,1860, 4) = (/&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1860, 5) = (/&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1860, 6) = (/&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1860, 7) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1860, 8) = (/&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1860, 9) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1860,10) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1860,11) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1860,12) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1861, 1) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1861, 2) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1861, 3) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1861, 4) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1861, 5) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1861, 6) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1861, 7) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1861, 8) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1861, 9) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1861,10) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1861,11) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66/)
ZAERV(: ,1861,12) = (/&
 &   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,&
 &   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79/)
ZAERV(: ,1862, 1) = (/&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92/)
ZAERV(: ,1862, 2) = (/&
 &  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,&
 &  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105/)
ZAERV(: ,1862, 3) = (/&
 &  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,&
 &  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118/)
ZAERV(: ,1862, 4) = (/&
 &  132,  132,  132,  132,  132,  132,  132,  132,  132,  132,  132,  132,&
 &  132,  132,  132,  132,  132,  132,  132,  132,  132,  132,  132,  132/)
ZAERV(: ,1862, 5) = (/&
 &  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,&
 &  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145/)
ZAERV(: ,1862, 6) = (/&
 &  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,&
 &  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,  158/)
ZAERV(: ,1862, 7) = (/&
 &  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,&
 &  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162,  162/)
ZAERV(: ,1862, 8) = (/&
 &  156,  156,  156,  156,  156,  156,  156,  156,  156,  156,  156,  156,&
 &  156,  156,  156,  156,  156,  156,  156,  156,  156,  156,  156,  156/)
ZAERV(: ,1862, 9) = (/&
 &  151,  151,  151,  151,  151,  151,  151,  151,  151,  151,  151,  151,&
 &  151,  151,  151,  151,  151,  151,  151,  151,  151,  151,  151,  151/)
ZAERV(: ,1862,10) = (/&
 &  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,&
 &  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145,  145/)
ZAERV(: ,1862,11) = (/&
 &  140,  140,  140,  140,  140,  140,  140,  140,  140,  140,  140,  140,&
 &  140,  140,  140,  140,  140,  140,  140,  140,  140,  140,  140,  140/)
ZAERV(: ,1862,12) = (/&
 &  135,  135,  135,  135,  135,  135,  135,  135,  135,  135,  135,  135,&
 &  135,  135,  135,  135,  135,  135,  135,  135,  135,  135,  135,  135/)
ZAERV(: ,1863, 1) = (/&
 &  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,&
 &  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129/)
ZAERV(: ,1863, 2) = (/&
 &  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,&
 &  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124/)
ZAERV(: ,1863, 3) = (/&
 &  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,&
 &  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118/)
ZAERV(: ,1863, 4) = (/&
 &  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,&
 &  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,  113/)
ZAERV(: ,1863, 5) = (/&
 &  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,&
 &  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107/)
ZAERV(: ,1863, 6) = (/&
 &  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,&
 &  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102/)
ZAERV(: ,1863, 7) = (/&
 &   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,&
 &   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97/)
ZAERV(: ,1863, 8) = (/&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92/)
ZAERV(: ,1863, 9) = (/&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87/)
ZAERV(: ,1863,10) = (/&
 &   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,&
 &   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82/)
ZAERV(: ,1863,11) = (/&
 &   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,&
 &   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78/)
ZAERV(: ,1863,12) = (/&
 &   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,&
 &   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73/)
ZAERV(: ,1864, 1) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68/)
ZAERV(: ,1864, 2) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63/)
ZAERV(: ,1864, 3) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58/)
ZAERV(: ,1864, 4) = (/&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1864, 5) = (/&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1864, 6) = (/&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1864, 7) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1864, 8) = (/&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1864, 9) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1864,10) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1864,11) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1864,12) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1865, 1) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1865, 2) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1865, 3) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1865, 4) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1865, 5) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1865, 6) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1865, 7) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1865, 8) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1865, 9) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1865,10) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1865,11) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1865,12) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1866, 1) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1866, 2) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1866, 3) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1866, 4) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1866, 5) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1866, 6) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1866, 7) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1866, 8) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1866, 9) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1866,10) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1866,11) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1866,12) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1867, 1) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1867, 2) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1867, 3) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1867, 4) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1867, 5) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1867, 6) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1867, 7) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1867, 8) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1867, 9) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1867,10) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1867,11) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1867,12) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1868, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1868,12) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1869, 1) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1869, 2) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1869, 3) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1869, 4) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1869, 5) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1869, 6) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1869, 7) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1869, 8) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1869, 9) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1869,10) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1869,11) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1869,12) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1870, 1) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1870, 2) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 3) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 4) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 5) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 6) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 7) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 8) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870, 9) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870,10) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870,11) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1870,12) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1871, 1) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1871, 2) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1871, 3) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1871, 4) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1871, 5) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1871, 6) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1871, 7) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1871, 8) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1871, 9) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1871,10) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1871,11) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1871,12) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1872, 1) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1872, 2) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1872, 3) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1872, 4) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1872, 5) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1872, 6) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1872, 7) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1872, 8) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1872, 9) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1872,10) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1872,11) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1872,12) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1873, 1) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1873, 2) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1873, 3) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1873, 4) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1873, 5) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1873, 6) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1873, 7) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1873, 8) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1873, 9) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1873,10) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1873,11) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1873,12) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1874, 1) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1874, 2) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1874, 3) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1874, 4) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1874, 5) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1874, 6) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1874, 7) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1874, 8) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1874, 9) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1874,10) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1874,11) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1874,12) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1875, 1) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1875, 2) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1875, 3) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1875, 4) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1875, 5) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1875, 6) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1875, 7) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1875, 8) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1875, 9) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1875,10) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1875,11) = (/&
 &   80,   80,   80,   80,   80,   80,   80,   80,    6,    6,    6,    6,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1875,12) = (/&
 &  110,  110,  110,  110,  110,  110,  110,  110,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1876, 1) = (/&
 &  140,  140,  140,  140,  140,  140,  140,  140,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1876, 2) = (/&
 &  169,  169,  169,  169,  169,  169,  169,  169,    5,    5,    5,    5,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1876, 3) = (/&
 &  199,  199,  199,  199,  199,  199,  199,  199,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1876, 4) = (/&
 &  229,  229,  229,  229,  229,  229,  229,  229,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1876, 5) = (/&
 &  258,  258,  258,  258,  258,  258,  258,  258,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1876, 6) = (/&
 &  288,  288,  288,  288,  288,  288,  288,  288,    4,    4,    4,    4,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1876, 7) = (/&
 &  299,  299,  299,  299,  299,  299,  299,  299,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1876, 8) = (/&
 &  292,  292,  292,  292,  292,  292,  292,  292,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1876, 9) = (/&
 &  284,  284,  284,  284,  284,  284,  284,  284,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1876,10) = (/&
 &  277,  277,  277,  277,  277,  277,  277,  277,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1876,11) = (/&
 &  270,  270,  270,  270,  270,  270,  270,  270,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1876,12) = (/&
 &  262,  262,  262,  262,  262,  262,  262,  262,    3,    3,    3,    3,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1877, 1) = (/&
 &  255,  255,  255,  255,  255,  255,  255,  255,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 2) = (/&
 &  248,  248,  248,  248,  248,  248,  248,  248,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 3) = (/&
 &  240,  240,  240,  240,  240,  240,  240,  240,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 4) = (/&
 &  233,  233,  233,  233,  233,  233,  233,  233,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 5) = (/&
 &  226,  226,  226,  226,  226,  226,  226,  226,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 6) = (/&
 &  218,  218,  218,  218,  218,  218,  218,  218,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 7) = (/&
 &  211,  211,  211,  211,  211,  211,  211,  211,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 8) = (/&
 &  203,  203,  203,  203,  203,  203,  203,  203,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877, 9) = (/&
 &  195,  195,  195,  195,  195,  195,  195,  195,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877,10) = (/&
 &  187,  187,  187,  187,  187,  187,  187,  187,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877,11) = (/&
 &  178,  178,  178,  178,  178,  178,  178,  178,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1877,12) = (/&
 &  170,  170,  170,  170,  170,  170,  170,  170,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 1) = (/&
 &  162,  162,  162,  162,  162,  162,  162,  162,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 2) = (/&
 &  154,  154,  154,  154,  154,  154,  154,  154,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 3) = (/&
 &  146,  146,  146,  146,  146,  146,  146,  146,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 4) = (/&
 &  138,  138,  138,  138,  138,  138,  138,  138,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 5) = (/&
 &  130,  130,  130,  130,  130,  130,  130,  130,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 6) = (/&
 &  122,  122,  122,  122,  122,  122,  122,  122,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 7) = (/&
 &  116,  116,  116,  116,  116,  116,  116,  116,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 8) = (/&
 &  113,  113,  113,  113,  113,  113,  113,  113,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878, 9) = (/&
 &  109,  109,  109,  109,  109,  109,  109,  109,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878,10) = (/&
 &  106,  106,  106,  106,  106,  106,  106,  106,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878,11) = (/&
 &  102,  102,  102,  102,  102,  102,  102,  102,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1878,12) = (/&
 &   99,   99,   99,   99,   99,   99,   99,   99,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 1) = (/&
 &   95,   95,   95,   95,   95,   95,   95,   95,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 2) = (/&
 &   92,   92,   92,   92,   92,   92,   92,   92,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 3) = (/&
 &   89,   89,   89,   89,   89,   89,   89,   89,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 4) = (/&
 &   85,   85,   85,   85,   85,   85,   85,   85,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 5) = (/&
 &   82,   82,   82,   82,   82,   82,   82,   82,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 6) = (/&
 &   78,   78,   78,   78,   78,   78,   78,   78,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 7) = (/&
 &   75,   75,   75,   75,   75,   75,   75,   75,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 8) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879, 9) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879,10) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879,11) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1879,12) = (/&
 &   59,   59,   59,   59,   59,   59,   59,   59,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 1) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 2) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 3) = (/&
 &   50,   50,   50,   50,   50,   50,   50,   50,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 4) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 5) = (/&
 &   44,   44,   44,   44,   44,   44,   44,   44,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 6) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 7) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 8) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880, 9) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880,10) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880,11) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1880,12) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 1) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 2) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 3) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 4) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 5) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 6) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 7) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 8) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881, 9) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881,10) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881,11) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1881,12) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 1) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 2) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 3) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 4) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 5) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 6) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 7) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 8) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882, 9) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882,10) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882,11) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1882,12) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1883, 1) = (/&
 &  291,  291,  291,  291,  291,  291,  291,  291,  291,  291,  291,  291,&
 &  291,  291,  291,  291,  291,  291,  291,  291,  291,  291,  291,  291/)
ZAERV(: ,1883, 2) = (/&
 &  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,&
 &  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328/)
ZAERV(: ,1883, 3) = (/&
 &  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,&
 &  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,  302/)
ZAERV(: ,1883, 4) = (/&
 &  278,  278,  278,  278,  278,  278,  278,  278,  278,  278,  278,  278,&
 &  278,  278,  278,  278,  278,  278,  278,  278,  278,  278,  278,  278/)
ZAERV(: ,1883, 5) = (/&
 &  255,  255,  255,  255,  255,  255,  255,  255,  255,  255,  255,  255,&
 &  255,  255,  255,  255,  255,  255,  255,  255,  255,  255,  255,  255/)
ZAERV(: ,1883, 6) = (/&
 &  235,  235,  235,  235,  235,  235,  235,  235,  235,  235,  235,  235,&
 &  235,  235,  235,  235,  235,  235,  235,  235,  235,  235,  235,  235/)
ZAERV(: ,1883, 7) = (/&
 &  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,&
 &  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,  216/)
ZAERV(: ,1883, 8) = (/&
 &  105,  108,  122,  110,  109,   82,   78,   70,   57,   67,  194,  615,&
 &  793,  417,  181,   66,   50,   57,   58,   62,   64,   74,  177,  186/)
ZAERV(: ,1883, 9) = (/&
 &  117,  123,  189,  167,  149,   99,  102,  103,   87,  162,  859, 1341,&
 & 1323,  741,  549,  235,   98,  114,  131,  140,  153,  178,  247,  252/)
ZAERV(: ,1883,10) = (/&
 &  127,  134,  202,  186,  168,  211,  300,  363,  366,  794, 2010, 2133,&
 & 1903, 1192, 1052,  530,  221,  213,  239,  249,  259,  302,  373,  377/)
ZAERV(: ,1883,11) = (/&
 &  165,  174,  283,  206,  240,  496,  623,  629,  598, 1090, 2229, 2490,&
 & 2210, 1798, 1639,  972,  439,  345,  359,  370,  364,  406,  404,  404/)
ZAERV(: ,1883,12) = (/&
 &  209,  228,  487,  602,  671,  959, 1122,  991,  859, 1287, 2302, 2522,&
 & 2383, 2143, 1953, 1176,  675,  608,  583,  568,  585,  823,  616,  603/)
ZAERV(: ,1884, 1) = (/&
 &  448,  492, 1040, 1164, 1364, 1396, 1367, 1123,  911, 1604, 2818, 2532,&
 & 2337, 2156, 1889, 1239,  856,  886,  847,  755,  763, 1111,  805,  788/)
ZAERV(: ,1884, 2) = (/&
 & 1362, 1361, 1332, 1343, 1555, 1291, 1017,  865,  793, 1277, 2284, 2176,&
 & 1926, 1774, 1768, 1350, 1494, 1737, 1800, 1365, 1196, 1323,  939,  916/)
ZAERV(: ,1884, 3) = (/&
 & 1836, 1836, 1763, 1611, 1566, 1275,  940,  790,  779, 1397, 2275, 2208,&
 & 2061, 2022, 1694, 1464, 1834, 1940, 1907, 1721, 1534, 1496, 1045, 1017/)
ZAERV(: ,1884, 4) = (/&
 & 1639, 1644, 1718, 1711, 1562, 1279,  959,  802,  814, 1335, 2103, 2050,&
 & 2071, 2030, 1673, 1589, 1970, 2022, 1984, 1848, 1731, 1602, 1135, 1104/)
ZAERV(: ,1884, 5) = (/&
 & 1606, 1622, 1801, 1763, 1487, 1248,  976,  835,  910, 1347, 1872, 1941,&
 & 1939, 1876, 1636, 1508, 1742, 1865, 1846, 1729, 1679, 1596, 1199, 1169/)
ZAERV(: ,1884, 6) = (/&
 & 1565, 1587, 1882, 1761, 1500, 1369, 1129,  975, 1003, 1442, 1840, 1852,&
 & 1802, 1713, 1571, 1415, 1462, 1613, 1631, 1610, 1648, 1617, 1261, 1234/)
ZAERV(: ,1884, 7) = (/&
 & 1504, 1517, 1681, 1622, 1507, 1426, 1364, 1230, 1140, 1336, 1722, 1777,&
 & 1787, 1717, 1578, 1384, 1429, 1502, 1547, 1573, 1567, 1534, 1315, 1299/)
ZAERV(: ,1884, 8) = (/&
 & 1449, 1456, 1600, 1628, 1536, 1582, 1613, 1427, 1190, 1265, 1522, 1556,&
 & 1536, 1424, 1277, 1096, 1069, 1223, 1362, 1437, 1470, 1470, 1366, 1357/)
ZAERV(: ,1884, 9) = (/&
 & 1398, 1402, 1483, 1571, 1507, 1689, 1686, 1451, 1176, 1211, 1385, 1438,&
 & 1558, 1505, 1294, 1037,  921, 1055, 1231, 1300, 1431, 1424, 1401, 1398/)
ZAERV(: ,1884,10) = (/&
 & 1340, 1334, 1296, 1383, 1390, 1561, 1686, 1507, 1188, 1210, 1209, 1135,&
 & 1205, 1353, 1166, 1004,  911, 1054, 1178, 1327, 1388, 1378, 1399, 1400/)
ZAERV(: ,1884,11) = (/&
 & 1285, 1273, 1127, 1144, 1295, 1412, 1474, 1306, 1028, 1064, 1079,  979,&
 & 1067, 1298, 1174,  951,  858, 1005, 1147, 1286, 1355, 1359, 1381, 1382/)
ZAERV(: ,1884,12) = (/&
 & 1232, 1223, 1115, 1122, 1225, 1352, 1330, 1179,  960,  917,  953,  907,&
 &  979, 1115, 1095,  913,  877, 1038, 1195, 1263, 1278, 1304, 1313, 1313/)
ZAERV(: ,1885, 1) = (/&
 & 1175, 1167, 1090, 1145, 1252, 1254, 1171, 1030,  884,  878,  905,  865,&
 &  950,  973,  862,  794,  903, 1079, 1212, 1247, 1233, 1229, 1231, 1231/)
ZAERV(: ,1885, 2) = (/&
 & 1120, 1119, 1096, 1101, 1202, 1131,  954,  756,  635,  639,  731,  713,&
 &  727,  735,  682,  731,  918, 1083, 1191, 1260, 1163, 1157, 1149, 1149/)
ZAERV(: ,1885, 3) = (/&
 & 1095, 1097, 1116, 1147, 1130, 1008,  827,  636,  587,  589,  660,  691,&
 &  672,  608,  573,  662,  907, 1105, 1238, 1284, 1098, 1096, 1081, 1081/)
ZAERV(: ,1885, 4) = (/&
 & 1146, 1147, 1160, 1108, 1000,  870,  704,  572,  526,  509,  575,  620,&
 &  631,  547,  474,  531,  760, 1022, 1103, 1085, 1020, 1021,  998,  996/)
ZAERV(: ,1885, 5) = (/&
 & 1158, 1160, 1125, 1005,  939,  836,  669,  532,  489,  482,  534,  575,&
 &  570,  494,  428,  471,  673,  879,  977,  993,  978,  972,  915,  911/)
ZAERV(: ,1885, 6) = (/&
 & 1108, 1109, 1029,  925,  891,  795,  649,  514,  454,  452,  475,  513,&
 &  522,  449,  383,  422,  584,  799,  893,  918,  909,  901,  830,  825/)
ZAERV(: ,1885, 7) = (/&
 & 1041, 1041,  970,  894,  819,  785,  715,  581,  461,  439,  467,  492,&
 &  504,  433,  356,  370,  480,  665,  774,  790,  775,  740,  732,  732/)
ZAERV(: ,1885, 8) = (/&
 &  973,  972,  911,  860,  828,  803,  742,  580,  429,  392,  450,  443,&
 &  427,  384,  331,  315,  372,  527,  647,  710,  723,  706,  644,  639/)
ZAERV(: ,1885, 9) = (/&
 &  910,  909,  850,  824,  835,  830,  776,  576,  414,  372,  410,  425,&
 &  407,  366,  320,  299,  303,  387,  519,  558,  565,  574,  561,  560/)
ZAERV(: ,1885,10) = (/&
 &  835,  833,  784,  767,  793,  793,  720,  510,  368,  355,  419,  417,&
 &  398,  394,  327,  289,  298,  365,  459,  514,  548,  558,  557,  557/)
ZAERV(: ,1885,11) = (/&
 &  748,  749,  735,  726,  727,  704,  611,  465,  352,  327,  356,  384,&
 &  393,  381,  330,  287,  285,  337,  410,  466,  506,  558,  561,  561/)
ZAERV(: ,1885,12) = (/&
 &  653,  656,  688,  680,  676,  660,  574,  451,  342,  322,  355,  379,&
 &  379,  334,  290,  269,  280,  348,  411,  449,  477,  538,  539,  538/)
ZAERV(: ,1886, 1) = (/&
 &  594,  594,  590,  586,  593,  592,  496,  411,  326,  313,  352,  377,&
 &  332,  285,  262,  254,  287,  358,  420,  443,  466,  518,  515,  515/)
ZAERV(: ,1886, 2) = (/&
 &  562,  564,  592,  592,  554,  502,  385,  290,  256,  261,  288,  301,&
 &  292,  264,  238,  234,  281,  377,  421,  436,  447,  499,  492,  492/)
ZAERV(: ,1886, 3) = (/&
 &  499,  500,  509,  496,  480,  414,  317,  246,  228,  242,  276,  287,&
 &  267,  239,  210,  214,  280,  376,  423,  423,  424,  475,  465,  465/)
ZAERV(: ,1886, 4) = (/&
 &  520,  519,  501,  466,  430,  358,  278,  225,  208,  224,  258,  274,&
 &  243,  230,  190,  185,  266,  362,  406,  407,  414,  447,  432,  433/)
ZAERV(: ,1886, 5) = (/&
 &  525,  524,  495,  420,  378,  316,  250,  199,  184,  201,  228,  238,&
 &  225,  193,  167,  169,  223,  295,  357,  401,  422,  418,  398,  397/)
ZAERV(: ,1886, 6) = (/&
 &  494,  493,  462,  373,  339,  295,  242,  191,  166,  174,  194,  199,&
 &  197,  177,  160,  165,  216,  285,  334,  367,  386,  376,  364,  364/)
ZAERV(: ,1886, 7) = (/&
 &  456,  456,  426,  357,  312,  298,  275,  216,  173,  175,  192,  188,&
 &  186,  172,  153,  150,  185,  233,  270,  291,  330,  349,  355,  356/)
ZAERV(: ,1886, 8) = (/&
 &  422,  422,  390,  330,  310,  309,  279,  205,  157,  159,  176,  160,&
 &  161,  169,  152,  135,  142,  171,  215,  250,  280,  284,  297,  298/)
ZAERV(: ,1886, 9) = (/&
 &  385,  385,  352,  304,  309,  321,  293,  217,  166,  164,  189,  162,&
 &  166,  176,  152,  129,  121,  140,  186,  216,  228,  230,  226,  226/)
ZAERV(: ,1886,10) = (/&
 &  691,  692,  692,  692,  693,  692,  692,  692,  693,  692,  692,  692,&
 &  691,  692,  693,  692,  692,  692,  692,  692,  692,  692,  693,  693/)
ZAERV(: ,1886,11) = (/&
 &  637,  637,  637,  637,  637,  637,  637,  637,  637,  637,  637,  637,&
 &  638,  636,  637,  637,  637,  637,  637,  637,  636,  636,  636,  636/)
ZAERV(: ,1886,12) = (/&
 &  586,  586,  586,  586,  586,  586,  586,  586,  586,  586,  586,  586,&
 &  586,  586,  586,  587,  586,  586,  586,  586,  585,  586,  586,  586/)
ZAERV(: ,1887, 1) = (/&
 &  540,  539,  539,  539,  540,  540,  538,  540,  539,  539,  539,  539,&
 &  540,  539,  539,  538,  539,  539,  539,  539,  539,  538,  539,  539/)
ZAERV(: ,1887, 2) = (/&
 &  496,  496,  496,  496,  497,  496,  496,  496,  496,  496,  496,  496,&
 &  496,  496,  496,  496,  496,  496,  496,  496,  496,  495,  496,  496/)
ZAERV(: ,1887, 3) = (/&
 &  457,  457,  457,  457,  458,  457,  456,  457,  457,  457,  457,  457,&
 &  457,  457,  457,  456,  457,  457,  457,  457,  457,  457,  457,  457/)
ZAERV(: ,1887, 4) = (/&
 &  420,  420,  420,  420,  421,  420,  420,  420,  420,  420,  420,  420,&
 &  420,  420,  420,  420,  420,  420,  420,  420,  420,  419,  420,  420/)
ZAERV(: ,1887, 5) = (/&
 &  387,  387,  387,  387,  387,  387,  387,  387,  387,  387,  387,  387,&
 &  387,  387,  387,  387,  387,  387,  387,  387,  387,  387,  387,  387/)
ZAERV(: ,1887, 6) = (/&
 &  356,  356,  356,  356,  356,  356,  356,  356,  356,  356,  356,  356,&
 &  356,  356,  356,  356,  356,  356,  356,  356,  356,  356,  356,  356/)
ZAERV(: ,1887, 7) = (/&
 &  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,&
 &  328,  328,  328,  327,  328,  328,  328,  328,  328,  328,  328,  328/)
ZAERV(: ,1887, 8) = (/&
 &  345,  345,  345,  345,  345,  345,  345,  345,  345,  345,  345,  345,&
 &  345,  345,  345,  345,  345,  345,  345,  345,  345,  345,  345,  345/)
ZAERV(: ,1887, 9) = (/&
 &  317,  317,  317,  317,  317,  317,  317,  317,  317,  317,  317,  317,&
 &  317,  317,  317,  317,  317,  317,  317,  317,  317,  317,  317,  317/)
ZAERV(: ,1887,10) = (/&
 &  292,  292,  292,  292,  292,  292,  292,  292,  292,  292,  292,  292,&
 &  292,  292,  292,  292,  292,  292,  292,  292,  292,  292,  292,  292/)
ZAERV(: ,1887,11) = (/&
 &  269,  269,  269,  269,  269,  269,  269,  269,  269,  269,  269,  269,&
 &  269,  269,  269,  269,  269,  269,  269,  269,  269,  269,  269,  269/)
ZAERV(: ,1887,12) = (/&
 &  247,  247,  247,  247,  247,  247,  247,  247,  247,  247,  247,  247,&
 &  247,  247,  247,  247,  247,  247,  247,  247,  247,  247,  247,  247/)
ZAERV(: ,1888, 1) = (/&
 &  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,&
 &  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228/)
ZAERV(: ,1888, 2) = (/&
 &  210,  210,  210,  210,  210,  210,  210,  210,  210,  210,  210,  210,&
 &  210,  210,  210,  210,  210,  210,  210,  210,  210,  210,  210,  210/)
ZAERV(: ,1888, 3) = (/&
 &  193,  193,  193,  193,  193,  193,  193,  193,  193,  193,  193,  193,&
 &  193,  193,  193,  193,  193,  193,  193,  193,  193,  193,  193,  193/)
ZAERV(: ,1888, 4) = (/&
 &  178,  178,  178,  178,  178,  178,  178,  178,  178,  178,  178,  178,&
 &  178,  178,  178,  178,  178,  178,  178,  178,  178,  178,  178,  178/)
ZAERV(: ,1888, 5) = (/&
 &  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,&
 &  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,  163,  163/)
ZAERV(: ,1888, 6) = (/&
 &  119,  119,  119,  119,  119,  119,  119,  119,  124,  124,  124,  124,&
 &   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98/)
ZAERV(: ,1888, 7) = (/&
 &  141,  141,  141,  141,  141,  141,  141,  141,  146,  146,  146,  146,&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116/)
ZAERV(: ,1888, 8) = (/&
 &  162,  162,  162,  162,  162,  162,  162,  162,  169,  169,  169,  169,&
 &  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,  133/)
ZAERV(: ,1888, 9) = (/&
 &  242,  242,  242,  242,  242,  242,  242,  242,  251,  251,  251,  251,&
 &  198,  198,  198,  198,  198,  198,  198,  198,  198,  198,  198,  198/)
ZAERV(: ,1888,10) = (/&
 &  322,  322,  322,  322,  322,  322,  322,  322,  334,  334,  334,  334,&
 &  263,  263,  263,  263,  263,  263,  263,  263,  263,  263,  263,  263/)
ZAERV(: ,1888,11) = (/&
 &  401,  401,  401,  401,  401,  401,  401,  401,  416,  416,  416,  416,&
 &  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328,  328/)
ZAERV(: ,1888,12) = (/&
 &  410,  410,  410,  410,  410,  410,  410,  410,  425,  425,  425,  425,&
 &  335,  335,  335,  335,  335,  335,  335,  335,  335,  335,  335,  335/)
ZAERV(: ,1889, 1) = (/&
 &  419,  419,  419,  419,  419,  419,  419,  419,  435,  435,  435,  435,&
 &  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,  343,  343/)
ZAERV(: ,1889, 2) = (/&
 &  427,  427,  427,  427,  427,  427,  427,  427,  443,  443,  443,  443,&
 &  350,  350,  350,  350,  350,  350,  350,  350,  350,  350,  350,  350/)
ZAERV(: ,1889, 3) = (/&
 &  393,  393,  393,  393,  393,  393,  393,  393,  408,  408,  407,  408,&
 &  322,  322,  322,  322,  322,  322,  322,  322,  322,  322,  322,  322/)
ZAERV(: ,1889, 4) = (/&
 &  362,  362,  362,  362,  362,  362,  362,  362,  375,  375,  375,  375,&
 &  296,  296,  296,  296,  296,  296,  296,  296,  296,  296,  296,  296/)
ZAERV(: ,1889, 5) = (/&
 &  333,  333,  333,  333,  333,  333,  333,  333,  345,  345,  345,  345,&
 &  272,  272,  272,  272,  272,  272,  272,  272,  272,  272,  272,  272/)
ZAERV(: ,1889, 6) = (/&
 &  306,  306,  306,  306,  306,  306,  306,  306,  318,  318,  318,  318,&
 &  251,  251,  251,  251,  251,  251,  251,  251,  251,  251,  251,  251/)
ZAERV(: ,1889, 7) = (/&
 &  282,  282,  282,  282,  282,  282,  282,  282,  293,  293,  293,  293,&
 &  231,  231,  231,  231,  231,  231,  231,  231,  231,  231,  231,  231/)
ZAERV(: ,1889, 8) = (/&
 &  260,  260,  260,  260,  260,  260,  260,  260,  269,  269,  269,  269,&
 &  212,  212,  212,  212,  212,  212,  212,  212,  212,  212,  212,  212/)
ZAERV(: ,1889, 9) = (/&
 &  239,  239,  239,  239,  239,  239,  239,  239,  248,  248,  248,  248,&
 &  195,  195,  195,  195,  195,  195,  195,  195,  195,  195,  195,  195/)
ZAERV(: ,1889,10) = (/&
 &  220,  220,  220,  220,  220,  220,  220,  220,  228,  228,  228,  228,&
 &  180,  180,  180,  180,  180,  180,  180,  180,  180,  180,  180,  180/)
ZAERV(: ,1889,11) = (/&
 &  208,  208,  208,  208,  208,  208,  208,  208,  215,  215,  215,  215,&
 &  170,  170,  170,  170,  170,  170,  170,  170,  170,  170,  170,  170/)
ZAERV(: ,1889,12) = (/&
 &  279,  279,  279,  279,  279,  279,  279,  279,  289,  289,  289,  289,&
 &  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228/)
ZAERV(: ,1890, 1) = (/&
 &  350,  350,  350,  350,  350,  350,  350,  350,  363,  363,  363,  363,&
 &  286,  286,  286,  286,  286,  286,  286,  286,  286,  286,  286,  286/)
ZAERV(: ,1890, 2) = (/&
 &  421,  421,  421,  421,  421,  421,  421,  421,  437,  437,  437,  437,&
 &  344,  344,  344,  344,  344,  344,  344,  344,  344,  344,  344,  344/)
ZAERV(: ,1890, 3) = (/&
 &  455,  455,  455,  455,  455,  455,  455,  455,  472,  472,  472,  472,&
 &  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,  372,  372/)
ZAERV(: ,1890, 4) = (/&
 &  490,  490,  490,  490,  490,  490,  490,  490,  507,  509,  508,  508,&
 &  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,  401/)
ZAERV(: ,1890, 5) = (/&
 &  524,  524,  525,  523,  525,  525,  523,  524,  543,  545,  544,  544,&
 &  429,  429,  429,  429,  429,  429,  429,  429,  429,  429,  429,  429/)
ZAERV(: ,1890, 6) = (/&
 &  482,  482,  482,  482,  482,  482,  482,  482,  500,  500,  500,  500,&
 &  395,  395,  395,  395,  395,  395,  395,  395,  395,  395,  395,  395/)
ZAERV(: ,1890, 7) = (/&
 &  444,  444,  444,  444,  444,  444,  444,  444,  460,  460,  460,  460,&
 &  363,  363,  363,  363,  363,  363,  363,  363,  363,  363,  363,  363/)
ZAERV(: ,1890, 8) = (/&
 &  408,  408,  408,  408,  408,  408,  408,  408,  424,  424,  424,  424,&
 &  334,  334,  334,  334,  334,  334,  334,  334,  334,  334,  334,  334/)
ZAERV(: ,1890, 9) = (/&
 &  376,  376,  376,  376,  376,  376,  376,  376,  390,  390,  390,  390,&
 &  307,  307,  307,  307,  307,  307,  307,  307,  307,  307,  307,  307/)
ZAERV(: ,1890,10) = (/&
 &  401,  401,  401,  401,  401,  401,  401,  401,  359,  359,  359,  359,&
 &  283,  283,  283,  283,  283,  283,  283,  283,  283,  283,  283,  283/)
ZAERV(: ,1890,11) = (/&
 &  595,  595,  596,  594,  596,  596,  594,  595,  330,  330,  330,  330,&
 &  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,  260,  260/)
ZAERV(: ,1890,12) = (/&
 &  681,  681,  682,  680,  682,  682,  680,  681,  354,  354,  354,  354,&
 &  240,  240,  240,  240,  240,  240,  240,  240,  240,  240,  240,  240/)
ZAERV(: ,1891, 1) = (/&
 &  768,  768,  769,  767,  769,  769,  767,  768,  398,  398,  398,  398,&
 &  221,  221,  221,  221,  221,  221,  221,  221,  221,  221,  221,  221/)
ZAERV(: ,1891, 2) = (/&
 &  854,  854,  855,  853,  855,  855,  853,  854,  443,  443,  443,  443,&
 &  203,  203,  203,  203,  203,  203,  203,  203,  203,  203,  203,  203/)
ZAERV(: ,1891, 3) = (/&
 &  785,  785,  786,  784,  786,  786,  784,  785,  408,  408,  408,  408,&
 &  187,  187,  187,  187,  187,  187,  187,  187,  187,  187,  187,  187/)
ZAERV(: ,1891, 4) = (/&
 &  723,  723,  724,  722,  724,  724,  722,  723,  375,  375,  375,  375,&
 &  172,  172,  172,  172,  172,  172,  172,  172,  172,  172,  172,  172/)
ZAERV(: ,1891, 5) = (/&
 &  665,  665,  666,  664,  666,  666,  664,  665,  345,  345,  345,  345,&
 &  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,  158,  158/)
ZAERV(: ,1891, 6) = (/&
 &  612,  612,  613,  611,  613,  613,  611,  612,  318,  318,  318,  318,&
 &  146,  146,  146,  146,  146,  146,  146,  146,  146,  146,  146,  146/)
ZAERV(: ,1891, 7) = (/&
 &  563,  563,  564,  562,  564,  564,  562,  563,  293,  293,  293,  293,&
 &  134,  134,  134,  134,  134,  134,  134,  134,  134,  134,  134,  134/)
ZAERV(: ,1891, 8) = (/&
 &  518,  518,  519,  517,  519,  519,  517,  518,  269,  269,  269,  269,&
 &  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124/)
ZAERV(: ,1891, 9) = (/&
 &  477,  477,  477,  477,  477,  477,  477,  477,  248,  248,  248,  248,&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114/)
ZAERV(: ,1891,10) = (/&
 &  439,  439,  439,  439,  439,  439,  439,  439,  228,  228,  228,  228,&
 &  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105/)
ZAERV(: ,1891,11) = (/&
 &  421,  421,  421,  421,  421,  421,  421,  421,  219,  219,  219,  219,&
 &   96,   96,   96,   96,   96,   96,   96,   96,   96,   96,   96,   96/)
ZAERV(: ,1891,12) = (/&
 &  436,  436,  436,  436,  436,  436,  436,  436,  227,  227,  227,  227,&
 &   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89/)
ZAERV(: ,1892, 1) = (/&
 &  451,  451,  451,  451,  451,  451,  451,  451,  234,  234,  234,  234,&
 &   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82/)
ZAERV(: ,1892, 2) = (/&
 &  466,  466,  466,  466,  466,  466,  466,  466,  242,  242,  242,  242,&
 &   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75/)
ZAERV(: ,1892, 3) = (/&
 &  531,  531,  532,  530,  532,  532,  530,  531,  276,  276,  276,  276,&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69/)
ZAERV(: ,1892, 4) = (/&
 &  595,  595,  596,  594,  596,  596,  594,  595,  309,  309,  309,  309,&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64/)
ZAERV(: ,1892, 5) = (/&
 &  660,  660,  661,  659,  661,  661,  659,  660,  343,  343,  343,  343,&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59/)
ZAERV(: ,1892, 6) = (/&
 &  607,  607,  608,  606,  608,  608,  606,  607,  315,  315,  315,  315,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1892, 7) = (/&
 &  559,  559,  560,  558,  560,  560,  558,  559,  290,  290,  290,  290,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1892, 8) = (/&
 &  514,  514,  515,  513,  515,  515,  513,  514,  267,  267,  267,  267,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1892, 9) = (/&
 &  473,  473,  473,  473,  473,  473,  473,  473,  246,  246,  246,  246,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1892,10) = (/&
 &  435,  435,  435,  435,  435,  435,  435,  435,  226,  226,  226,  226,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1892,11) = (/&
 &  401,  401,  401,  401,  401,  401,  401,  401,  208,  208,  208,  208,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1892,12) = (/&
 &  369,  369,  369,  369,  369,  369,  369,  369,  192,  192,  192,  192,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1893, 1) = (/&
 &  339,  339,  339,  339,  339,  339,  339,  339,  176,  176,  176,  176,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1893, 2) = (/&
 &  312,  312,  312,  312,  312,  312,  312,  312,  162,  162,  162,  162,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1893, 3) = (/&
 &  287,  287,  287,  287,  287,  287,  287,  287,  150,  150,  150,  150,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1893, 4) = (/&
 &  264,  264,  264,  264,  264,  264,  264,  264,  138,  138,  138,  138,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1893, 5) = (/&
 &  243,  243,  243,  243,  243,  243,  243,  243,  127,  127,  127,  127,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1893, 6) = (/&
 &  224,  224,  224,  224,  224,  224,  224,  224,  117,  117,  117,  117,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1893, 7) = (/&
 &  206,  206,  206,  206,  206,  206,  206,  206,  107,  107,  107,  107,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1893, 8) = (/&
 &  190,  190,  190,  190,  190,  190,  190,  190,   99,   99,   99,   99,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1893, 9) = (/&
 &  175,  175,  175,  175,  175,  175,  175,  175,   91,   91,   91,   91,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1893,10) = (/&
 &  161,  161,  161,  161,  161,  161,  161,  161,   84,   84,   84,   84,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1893,11) = (/&
 &  148,  148,  148,  148,  148,  148,  148,  148,   77,   77,   77,   77,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1893,12) = (/&
 &  136,  136,  136,  136,  136,  136,  136,  136,   71,   71,   71,   71,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1894, 1) = (/&
 &  125,  125,  125,  125,  125,  125,  125,  125,   66,   66,   66,   66,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1894, 2) = (/&
 &  115,  115,  115,  115,  115,  115,  115,  115,   60,   60,   60,   60,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1894, 3) = (/&
 &  106,  106,  106,  106,  106,  106,  106,  106,   56,   56,   56,   56,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1894, 4) = (/&
 &   98,   98,   98,   98,   98,   98,   98,   98,   51,   51,   51,   51,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1894, 5) = (/&
 &   90,   90,   90,   90,   90,   90,   90,   90,   47,   47,   47,   47,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1894, 6) = (/&
 &   83,   83,   83,   83,   83,   83,   83,   83,   44,   44,   44,   44,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1894, 7) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   40,   40,   40,   40,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1894, 8) = (/&
 &   70,   70,   70,   70,   70,   70,   70,   70,   37,   37,   37,   37,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1894, 9) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   34,   34,   34,   34,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1894,10) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,   31,   31,   31,   31,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1894,11) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   29,   29,   29,   29,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1894,12) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   27,   27,   27,   27,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1895, 1) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   25,   25,   25,   25,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1895, 2) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   23,   23,   23,   23,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1895, 3) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   21,   21,   21,   21,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1895, 4) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   19,   19,   19,   19,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1895, 5) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   18,   18,   18,   18,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1895, 6) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   17,   17,   17,   17,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1895, 7) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   15,   15,   15,   15,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1895, 8) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   14,   14,   14,   14,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1895, 9) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   13,   13,   13,   13,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1895,10) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   12,   12,   12,   12,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1895,11) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   11,   11,   11,   11,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1895,12) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   10,   10,   10,   10,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1896, 1) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &   82,   82,   82,   82,  162,  162,  162,  162,  162,  162,  162,  162/)
ZAERV(: ,1896, 2) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &   75,   75,   75,   75,  149,  149,  149,  149,  149,  149,  149,  149/)
ZAERV(: ,1896, 3) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    5,    5,    5,    5,&
 &   92,   92,   92,   92,  183,  183,  183,  183,  183,  183,  183,  183/)
ZAERV(: ,1896, 4) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &  178,  178,  178,  178,  353,  353,  353,  353,  353,  353,  353,  353/)
ZAERV(: ,1896, 5) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &  263,  263,  263,  263,  523,  524,  524,  524,  523,  524,  525,  524/)
ZAERV(: ,1896, 6) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    4,    4,    4,    4,&
 &  301,  301,  301,  301,  599,  600,  600,  600,  599,  600,  601,  600/)
ZAERV(: ,1896, 7) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &  339,  339,  339,  339,  675,  676,  676,  676,  675,  676,  677,  676/)
ZAERV(: ,1896, 8) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &  377,  377,  377,  377,  750,  751,  751,  751,  750,  751,  752,  751/)
ZAERV(: ,1896, 9) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &  347,  347,  347,  347,  690,  691,  691,  691,  690,  691,  692,  691/)
ZAERV(: ,1896,10) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    3,    3,    3,    3,&
 &  319,  319,  319,  319,  635,  636,  636,  636,  635,  636,  637,  636/)
ZAERV(: ,1896,11) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &  294,  294,  294,  294,  584,  585,  585,  585,  584,  585,  586,  585/)
ZAERV(: ,1896,12) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &  271,  271,  271,  271,  538,  539,  539,  539,  538,  539,  540,  539/)
ZAERV(: ,1897, 1) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &  249,  249,  249,  249,  496,  496,  496,  496,  496,  496,  496,  496/)
ZAERV(: ,1897, 2) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &  229,  229,  229,  229,  456,  456,  456,  456,  456,  456,  456,  456/)
ZAERV(: ,1897, 3) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &  211,  211,  211,  211,  420,  420,  420,  420,  420,  420,  420,  420/)
ZAERV(: ,1897, 4) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    2,    2,    2,    2,&
 &  194,  194,  194,  194,  386,  386,  386,  386,  386,  386,  386,  386/)
ZAERV(: ,1897, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  186,  186,  186,  186,  371,  371,  371,  371,  371,  371,  371,  371/)
ZAERV(: ,1897, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  193,  193,  193,  193,  384,  384,  384,  384,  384,  384,  384,  384/)
ZAERV(: ,1897, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  199,  199,  199,  199,  397,  397,  397,  397,  397,  397,  397,  397/)
ZAERV(: ,1897, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  206,  206,  206,  206,  410,  410,  410,  410,  410,  410,  410,  410/)
ZAERV(: ,1897, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  235,  235,  235,  235,  467,  467,  467,  467,  467,  467,  467,  467/)
ZAERV(: ,1897,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  263,  263,  263,  263,  523,  524,  524,  524,  523,  524,  525,  524/)
ZAERV(: ,1897,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  292,  292,  292,  292,  580,  581,  581,  581,  580,  581,  582,  581/)
ZAERV(: ,1897,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  268,  268,  268,  268,  534,  535,  535,  535,  534,  535,  536,  535/)
ZAERV(: ,1898, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  247,  247,  247,  247,  492,  492,  492,  492,  492,  492,  492,  492/)
ZAERV(: ,1898, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  227,  227,  227,  227,  453,  453,  453,  453,  453,  453,  453,  453/)
ZAERV(: ,1898, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  209,  209,  209,  209,  417,  417,  417,  417,  417,  417,  417,  417/)
ZAERV(: ,1898, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  193,  193,  193,  193,  383,  383,  383,  383,  383,  383,  383,  383/)
ZAERV(: ,1898, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  177,  177,  177,  177,  353,  353,  353,  353,  353,  353,  353,  353/)
ZAERV(: ,1898, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  163,  163,  163,  163,  325,  325,  325,  325,  325,  325,  325,  325/)
ZAERV(: ,1898, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  150,  150,  150,  150,  299,  299,  299,  299,  299,  299,  299,  299/)
ZAERV(: ,1898, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  138,  138,  138,  138,  275,  275,  275,  275,  275,  275,  275,  275/)
ZAERV(: ,1898, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  127,  127,  127,  127,  253,  253,  253,  253,  253,  253,  253,  253/)
ZAERV(: ,1898,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  117,  117,  117,  117,  233,  233,  233,  233,  233,  233,  233,  233/)
ZAERV(: ,1898,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &  108,  108,  108,  108,  214,  214,  214,  214,  214,  214,  214,  214/)
ZAERV(: ,1898,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   99,   99,   99,   99,  197,  197,  197,  197,  197,  197,  197,  197/)
ZAERV(: ,1899, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   92,   92,   92,   92,  182,  182,  182,  182,  182,  182,  182,  182/)
ZAERV(: ,1899, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   84,   84,   84,   84,  167,  167,  167,  167,  167,  167,  167,  167/)
ZAERV(: ,1899, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   78,   78,   78,   78,  154,  154,  154,  154,  154,  154,  154,  154/)
ZAERV(: ,1899, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   72,   72,   72,   72,  142,  142,  142,  142,  142,  142,  142,  142/)
ZAERV(: ,1899, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   66,   66,   66,   66,  130,  130,  130,  130,  130,  130,  130,  130/)
ZAERV(: ,1899, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   61,   61,   61,   61,  120,  120,  120,  120,  120,  120,  120,  120/)
ZAERV(: ,1899, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   56,   56,   56,   56,  111,  111,  111,  111,  111,  111,  111,  111/)
ZAERV(: ,1899, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   52,   52,   52,   52,  102,  102,  102,  102,  102,  102,  102,  102/)
ZAERV(: ,1899, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   47,   47,   47,   47,   94,   94,   94,   94,   94,   94,   94,   94/)
ZAERV(: ,1899,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   44,   44,   44,   44,   86,   86,   86,   86,   86,   86,   86,   86/)
ZAERV(: ,1899,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   40,   40,   40,   40,   79,   79,   79,   79,   79,   79,   79,   79/)
ZAERV(: ,1899,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   37,   37,   37,   37,   73,   73,   73,   73,   73,   73,   73,   73/)

IF (LHOOK) CALL DR_HOOK('SUECAEC:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1


SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SUECAEC:PART2',0,ZHOOK_HANDLE)

ZAERV(: ,1900, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   34,   34,   34,   34,   67,   67,   67,   67,   67,   67,   67,   67/)
ZAERV(: ,1900, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   32,   32,   32,   32,   62,   62,   62,   62,   62,   62,   62,   62/)
ZAERV(: ,1900, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   29,   29,   29,   29,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1900, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   27,   27,   27,   27,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1900, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   25,   25,   25,   25,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1900, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   23,   23,   23,   23,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1900, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   21,   21,   21,   21,   41,   41,   41,   41,   41,   41,   41,   41/)
ZAERV(: ,1900, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   20,   20,   20,   20,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1900, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   18,   18,   18,   18,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1900,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   17,   17,   17,   17,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1900,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   15,   15,   15,   15,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1900,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   14,   14,   14,   14,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1901, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   13,   13,   13,   13,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1901, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   12,   12,   12,   12,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1901, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   11,   11,   11,   11,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1901, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   11,   11,   11,   11,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1901, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   10,   10,   10,   10,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1901, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    9,    9,    9,    9,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1901, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    8,    8,    8,    8,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1901, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    8,    8,    8,    8,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1901, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    7,    7,    7,    7,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1901,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    7,    7,    7,    7,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1901,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    6,    6,    6,    6,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1901,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    6,    6,    6,    6,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1902, 1) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   39,   39,   39,   39,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1902, 2) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   58,   58,   58,   58,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1902, 3) = (/&
 &   85,   85,   85,   85,   85,   85,   85,   85,   88,   88,   88,   88,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1902, 4) = (/&
 &  116,  116,  116,  116,  116,  116,  116,  116,  119,  119,  119,  119,&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60/)
ZAERV(: ,1902, 5) = (/&
 &  145,  145,  145,  145,  145,  145,  145,  145,  150,  150,  150,  150,&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76/)
ZAERV(: ,1902, 6) = (/&
 &  134,  134,  134,  134,  134,  134,  134,  134,  138,  138,  138,  138,&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70/)
ZAERV(: ,1902, 7) = (/&
 &  123,  123,  123,  123,  123,  123,  123,  123,  127,  127,  127,  127,&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64/)
ZAERV(: ,1902, 8) = (/&
 &   93,   89,   37,   32,   31,   30,   29,   26,   33,   91,  209,  397,&
 &  308,   97,   34,   29,   36,   39,   42,   55,   55,   61,   54,   53/)
ZAERV(: ,1902, 9) = (/&
 &  127,  124,   90,   77,   70,   66,   58,   49,  118,  275,  371,  662,&
 &  671,  430,   81,   44,   52,   52,   50,   75,   84,   95,   62,   59/)
ZAERV(: ,1902,10) = (/&
 &  189,  187,  152,  130,  125,  120,  107,  111,  265,  526,  597,  952,&
 & 1067, 1005,  398,  184,  182,  150,  106,   85,   93,  102,   68,   64/)
ZAERV(: ,1902,11) = (/&
 &  202,  202,  204,  183,  186,  180,  173,  220,  487,  820,  900, 1105,&
 & 1246, 1115,  546,  299,  315,  312,  249,  120,  103,  142,   87,   83/)
ZAERV(: ,1902,12) = (/&
 &  302,  308,  412,  293,  284,  292,  305,  338,  588,  977, 1072, 1192,&
 & 1262, 1152,  644,  430,  496,  561,  480,  336,  302,  244,  114,  105/)
ZAERV(: ,1903, 1) = (/&
 &  394,  403,  556,  382,  378,  424,  444,  428,  620,  945, 1078, 1169,&
 & 1267, 1409,  802,  456,  562,  684,  698,  682,  582,  521,  247,  224/)
ZAERV(: ,1903, 2) = (/&
 &  459,  470,  662,  598,  683,  900,  869,  748,  675,  884,  888,  963,&
 & 1088, 1142,  639,  397,  433,  509,  646,  778,  672,  667,  681,  681/)
ZAERV(: ,1903, 3) = (/&
 &  509,  523,  749,  767,  861,  954,  971,  918,  732,  847, 1011, 1031,&
 & 1104, 1138,  699,  390,  396,  471,  638,  784,  806,  882,  918,  918/)
ZAERV(: ,1903, 4) = (/&
 &  553,  568,  802,  866,  924,  993, 1011,  985,  795,  837, 1015, 1036,&
 & 1026, 1052,  668,  408,  402,  480,  640,  782,  856,  859,  822,  820/)
ZAERV(: ,1903, 5) = (/&
 &  585,  600,  799,  840,  865,  924,  933,  871,  755,  819,  939,  970,&
 &  971,  936,  674,  455,  418,  488,  625,  744,  882,  901,  811,  803/)
ZAERV(: ,1903, 6) = (/&
 &  618,  631,  809,  825,  805,  816,  807,  732,  708,  786,  857,  901,&
 &  926,  921,  722,  502,  488,  565,  685,  750,  881,  942,  794,  783/)
ZAERV(: ,1903, 7) = (/&
 &  650,  658,  767,  784,  787,  774,  752,  715,  692,  789,  859,  894,&
 &  889,  861,  669,  570,  615,  683,  713,  754,  811,  841,  759,  752/)
ZAERV(: ,1903, 8) = (/&
 &  679,  684,  735,  735,  719,  681,  612,  535,  548,  639,  713,  768,&
 &  779,  762,  633,  596,  714,  807,  792,  768,  814,  801,  729,  725/)
ZAERV(: ,1903, 9) = (/&
 &  700,  701,  712,  716,  651,  616,  528,  461,  519,  647,  753,  779,&
 &  719,  693,  606,  588,  726,  844,  845,  754,  786,  742,  701,  700/)
ZAERV(: ,1903,10) = (/&
 &  701,  700,  690,  695,  664,  590,  527,  456,  503,  583,  677,  603,&
 &  568,  605,  606,  594,  754,  844,  781,  696,  692,  648,  668,  670/)
ZAERV(: ,1903,11) = (/&
 &  691,  691,  680,  678,  643,  574,  503,  429,  476,  587,  649,  534,&
 &  490,  540,  532,  515,  653,  738,  707,  648,  572,  564,  637,  643/)
ZAERV(: ,1903,12) = (/&
 &  657,  657,  652,  640,  632,  598,  520,  439,  457,  548,  558,  490,&
 &  454,  477,  459,  481,  590,  665,  676,  613,  561,  558,  612,  616/)
ZAERV(: ,1904, 1) = (/&
 &  616,  616,  615,  617,  624,  607,  540,  452,  398,  432,  487,  476,&
 &  433,  453,  439,  443,  515,  586,  628,  626,  573,  545,  584,  588/)
ZAERV(: ,1904, 2) = (/&
 &  575,  575,  579,  582,  630,  596,  542,  460,  366,  342,  368,  364,&
 &  357,  366,  320,  318,  378,  478,  566,  602,  551,  548,  560,  560/)
ZAERV(: ,1904, 3) = (/&
 &  541,  541,  548,  549,  642,  619,  553,  454,  332,  287,  304,  337,&
 &  346,  330,  295,  294,  318,  414,  504,  565,  574,  559,  549,  548/)
ZAERV(: ,1904, 4) = (/&
 &  499,  499,  511,  510,  543,  552,  511,  381,  266,  238,  274,  316,&
 &  310,  288,  255,  264,  286,  352,  435,  500,  554,  581,  574,  574/)
ZAERV(: ,1904, 5) = (/&
 &  456,  458,  487,  489,  497,  489,  440,  337,  236,  214,  247,  286,&
 &  288,  268,  241,  245,  266,  335,  418,  470,  503,  563,  581,  580/)
ZAERV(: ,1904, 6) = (/&
 &  413,  416,  451,  455,  460,  447,  400,  292,  212,  192,  225,  262,&
 &  257,  238,  227,  227,  257,  325,  398,  446,  463,  515,  555,  554/)
ZAERV(: ,1904, 7) = (/&
 &  367,  367,  371,  388,  396,  387,  333,  241,  185,  179,  217,  252,&
 &  247,  234,  220,  231,  291,  358,  393,  410,  447,  486,  521,  521/)
ZAERV(: ,1904, 8) = (/&
 &  320,  323,  354,  362,  356,  324,  264,  186,  158,  166,  192,  214,&
 &  222,  226,  197,  215,  291,  372,  402,  415,  430,  456,  487,  487/)
ZAERV(: ,1904, 9) = (/&
 &  281,  281,  288,  283,  279,  260,  194,  152,  150,  161,  184,  204,&
 &  213,  206,  187,  207,  289,  389,  416,  418,  412,  425,  455,  455/)
ZAERV(: ,1904,10) = (/&
 &  279,  279,  279,  274,  258,  230,  183,  150,  145,  164,  197,  200,&
 &  209,  210,  178,  184,  256,  361,  397,  397,  384,  392,  417,  418/)
ZAERV(: ,1904,11) = (/&
 &  281,  281,  279,  253,  234,  206,  169,  143,  144,  165,  191,  197,&
 &  193,  179,  164,  176,  233,  306,  352,  364,  363,  368,  375,  374/)
ZAERV(: ,1904,12) = (/&
 &  269,  270,  269,  239,  225,  206,  174,  141,  135,  146,  168,  190,&
 &  190,  178,  162,  172,  226,  288,  330,  339,  341,  345,  328,  327/)
ZAERV(: ,1905, 1) = (/&
 &  258,  258,  260,  233,  222,  211,  180,  144,  128,  131,  143,  167,&
 &  189,  176,  157,  163,  206,  249,  296,  297,  294,  296,  297,  297/)
ZAERV(: ,1905, 2) = (/&
 &  247,  247,  250,  224,  218,  211,  189,  141,  118,  119,  132,  147,&
 &  151,  145,  131,  129,  146,  193,  251,  278,  296,  297,  283,  282/)
ZAERV(: ,1905, 3) = (/&
 &  233,  233,  238,  212,  212,  212,  189,  141,  107,  106,  120,  134,&
 &  144,  138,  121,  114,  124,  159,  208,  240,  249,  255,  251,  250/)
ZAERV(: ,1905, 4) = (/&
 &  217,  217,  224,  207,  204,  203,  181,  134,   93,   96,  115,  122,&
 &  137,  130,  113,  104,  113,  140,  180,  216,  233,  251,  260,  260/)
ZAERV(: ,1905, 5) = (/&
 &  199,  200,  209,  211,  201,  179,  148,  112,   85,   84,   97,  113,&
 &  119,  114,  101,   92,  100,  125,  158,  190,  211,  248,  263,  263/)
ZAERV(: ,1905, 6) = (/&
 &  183,  183,  189,  194,  184,  167,  143,  108,   83,   81,   89,   99,&
 &  100,   97,   87,   83,   96,  121,  148,  170,  187,  232,  247,  248/)
ZAERV(: ,1905, 7) = (/&
 &  179,  178,  175,  165,  146,  135,  117,   93,   75,   77,   86,   93,&
 &   95,   97,   88,   87,  108,  138,  150,  157,  179,  214,  229,  229/)
ZAERV(: ,1905, 8) = (/&
 &  150,  149,  142,  140,  126,  108,   86,   71,   68,   76,   85,   81,&
 &   81,   88,   80,   79,  103,  140,  155,  156,  166,  196,  212,  212/)
ZAERV(: ,1905, 9) = (/&
 &  114,  114,  115,  114,  109,   93,   70,   61,   65,   76,   89,   83,&
 &   81,   95,   82,   84,  109,  147,  161,  155,  152,  176,  193,  193/)
ZAERV(: ,1905,10) = (/&
 &  117,  117,  114,  108,  102,   90,   70,   61,   66,   79,  102,   86,&
 &   79,   84,   79,   76,   99,  133,  148,  147,  141,  156,  173,  173/)
ZAERV(: ,1905,11) = (/&
 &  121,  121,  116,  103,   97,   83,   69,   60,   63,   75,   88,   76,&
 &   74,   73,   70,   73,   97,  124,  139,  138,  132,  142,  151,  151/)
ZAERV(: ,1905,12) = (/&
 &  139,  139,  133,  101,   96,   88,   72,   62,   61,   70,   86,   86,&
 &   86,   81,   70,   75,  103,  132,  142,  136,  132,  132,  128,  128/)
ZAERV(: ,1906, 1) = (/&
 &  163,  162,  158,  110,  104,  105,   89,   68,   59,   61,   69,   75,&
 &   83,   78,   66,   66,   85,  110,  129,  131,  132,  126,  121,  122/)
ZAERV(: ,1906, 2) = (/&
 &  203,  203,  200,  171,  144,  111,   98,   70,   53,   50,   56,   69,&
 &   82,   74,   64,   60,   71,   91,  117,  129,  134,  136,  148,  149/)
ZAERV(: ,1906, 3) = (/&
 &   90,   90,   91,   95,   87,   88,   81,   67,   41,   41,   47,   48,&
 &   55,   57,   55,   54,   54,   66,   89,  108,  119,  123,  124,  124/)
ZAERV(: ,1906, 4) = (/&
 &   91,   91,   92,   95,   87,   87,   81,   69,   39,   38,   46,   48,&
 &   55,   58,   55,   51,   54,   62,   81,  102,  107,  116,  120,  120/)
ZAERV(: ,1906, 5) = (/&
 &   93,   93,   93,   94,   87,   87,   74,   55,   41,   37,   43,   50,&
 &   52,   52,   51,   48,   48,   53,   70,   86,  101,  115,  121,  121/)
ZAERV(: ,1906, 6) = (/&
 &   95,   95,   94,   92,   87,   84,   66,   48,   33,   32,   38,   44,&
 &   46,   43,   42,   43,   46,   50,   63,   79,  100,  119,  129,  129/)
ZAERV(: ,1906, 7) = (/&
 &   96,   96,   92,   87,   79,   71,   53,   41,   33,   31,   38,   42,&
 &   46,   48,   44,   46,   60,   70,   71,   77,   93,  100,  103,  103/)
ZAERV(: ,1906, 8) = (/&
 &  119,  119,  119,  119,  119,  119,  119,  119,  122,  122,  122,  122,&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62/)
ZAERV(: ,1906, 9) = (/&
 &  109,  109,  109,  109,  109,  109,  109,  109,  113,  113,  113,  113,&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1906,10) = (/&
 &  101,  101,  101,  101,  101,  101,  101,  101,  104,  104,  104,  104,&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1906,11) = (/&
 &   93,   93,   93,   93,   93,   93,   93,   93,   96,   96,   96,   96,&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48/)
ZAERV(: ,1906,12) = (/&
 &   85,   85,   85,   85,   85,   85,   85,   85,   88,   88,   88,   88,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1907, 1) = (/&
 &   79,   79,   79,   79,   79,   79,   79,   79,   81,   81,   81,   81,&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41/)
ZAERV(: ,1907, 2) = (/&
 &   83,   83,   83,   83,   83,   83,   83,   83,   75,   75,   75,   75,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1907, 3) = (/&
 &  164,  164,  164,  164,  164,  164,  164,  164,   69,   69,   69,   69,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1907, 4) = (/&
 &  243,  243,  243,  243,  243,  243,  243,  243,   63,   63,   63,   63,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1907, 5) = (/&
 &  324,  324,  324,  324,  324,  324,  324,  324,   58,   58,   58,   58,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1907, 6) = (/&
 &  335,  335,  335,  335,  335,  335,  335,  335,   54,   54,   54,   54,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1907, 7) = (/&
 &  346,  346,  346,  346,  346,  346,  346,  346,   50,   50,   50,   50,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1907, 8) = (/&
 &  358,  358,  358,  358,  358,  358,  358,  358,   46,   46,   46,   46,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1907, 9) = (/&
 &  331,  331,  331,  331,  331,  331,  331,  331,   42,   42,   42,   42,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1907,10) = (/&
 &  304,  304,  304,  304,  304,  304,  304,  304,   39,   39,   39,   39,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1907,11) = (/&
 &  280,  280,  280,  280,  280,  280,  280,  280,   36,   36,   36,   36,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1907,12) = (/&
 &  367,  367,  367,  367,  367,  367,  367,  367,   33,   33,   33,   33,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1908, 1) = (/&
 &  459,  459,  459,  459,  459,  459,  459,  459,   30,   30,   30,   30,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1908, 2) = (/&
 &  549,  549,  549,  549,  550,  549,  550,  549,   28,   28,   28,   28,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1908, 3) = (/&
 &  505,  505,  507,  506,  506,  506,  506,  505,   26,   26,   26,   26,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1908, 4) = (/&
 &  466,  466,  466,  466,  466,  466,  466,  466,   24,   24,   24,   24,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1908, 5) = (/&
 &  429,  429,  429,  429,  429,  429,  429,  429,   22,   22,   22,   22,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1908, 6) = (/&
 &  395,  395,  395,  395,  395,  395,  395,  395,   20,   20,   20,   20,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1908, 7) = (/&
 &  363,  363,  363,  363,  363,  363,  363,  363,   19,   19,   19,   19,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1908, 8) = (/&
 &  334,  334,  334,  334,  334,  334,  334,  334,   17,   17,   17,   17,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1908, 9) = (/&
 &  307,  307,  307,  307,  307,  307,  307,  307,   16,   16,   16,   16,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1908,10) = (/&
 &  283,  283,  283,  283,  283,  283,  283,  283,   15,   15,   15,   15,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1908,11) = (/&
 &  260,  260,  260,  260,  260,  260,  260,  260,   14,   14,   14,   14,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1908,12) = (/&
 &  240,  240,  240,  240,  240,  240,  240,  240,   13,   13,   13,   13,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1909, 1) = (/&
 &  221,  221,  221,  221,  221,  221,  221,  221,   12,   12,   12,   12,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1909, 2) = (/&
 &  207,  207,  207,  207,  207,  207,  207,  207,   11,   11,   11,   11,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1909, 3) = (/&
 &  190,  190,  190,  190,  190,  190,  190,  190,   10,   10,   10,   10,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1909, 4) = (/&
 &  175,  175,  175,  175,  175,  175,  175,  175,    9,    9,    9,    9,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1909, 5) = (/&
 &  161,  161,  161,  161,  161,  161,  161,  161,    9,    9,    9,    9,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1909, 6) = (/&
 &  149,  149,  149,  149,  149,  149,  149,  149,    8,    8,    8,    8,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1909, 7) = (/&
 &  137,  137,  137,  137,  137,  137,  137,  137,    8,    8,    8,    8,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1909, 8) = (/&
 &  126,  126,  126,  126,  126,  126,  126,  126,    7,    7,    7,    7,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1909, 9) = (/&
 &  116,  116,  116,  116,  116,  116,  116,  116,    7,    7,    7,    7,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1909,10) = (/&
 &  107,  107,  107,  107,  107,  107,  107,  107,    6,    6,    6,    6,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1909,11) = (/&
 &   98,   98,   98,   98,   98,   98,   98,   98,    6,    6,    6,    6,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1909,12) = (/&
 &   91,   91,   91,   91,   91,   91,   91,   91,    5,    5,    5,    5,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1910, 1) = (/&
 &  106,  106,  106,  106,  106,  106,  106,  106,    5,    5,    5,    5,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1910, 2) = (/&
 &  159,  159,  159,  159,  159,  159,  159,  159,    5,    5,    5,    5,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1910, 3) = (/&
 &  154,  154,  154,  154,  154,  154,  154,  154,    4,    4,    4,    4,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1910, 4) = (/&
 &  150,  150,  150,  150,  150,  150,  150,  150,    4,    4,    4,    4,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1910, 5) = (/&
 &  145,  145,  145,  145,  145,  145,  145,  145,    4,    4,    4,    4,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910, 6) = (/&
 &  134,  134,  134,  134,  134,  134,  134,  134,    4,    4,    4,    4,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910, 7) = (/&
 &  123,  123,  123,  123,  123,  123,  123,  123,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910, 8) = (/&
 &  113,  113,  113,  113,  113,  113,  113,  113,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910, 9) = (/&
 &  104,  104,  104,  104,  104,  104,  104,  104,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910,10) = (/&
 &   96,   96,   96,   96,   96,   96,   96,   96,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910,11) = (/&
 &   88,   88,   88,   88,   88,   88,   88,   88,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1910,12) = (/&
 &   81,   81,   81,   81,   81,   81,   81,   81,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 1) = (/&
 &   75,   75,   75,   75,   75,   75,   75,   75,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 2) = (/&
 &   90,   90,   90,   90,   90,   90,   90,   90,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 3) = (/&
 &   83,   83,   83,   83,   83,   83,   83,   83,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 4) = (/&
 &   77,   77,   77,   77,   77,   77,   77,   77,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 5) = (/&
 &   71,   71,   71,   71,   71,   71,   71,   71,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 6) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 7) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 8) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911, 9) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911,10) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911,11) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1911,12) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 1) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 2) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 3) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,    8,    8,    8,    8,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 4) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   15,   15,   15,   15,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 5) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   22,   22,   22,   22,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 6) = (/&
 &  453,  453,  453,  453,  453,  453,  453,  453,  149,  149,  149,  149,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 7) = (/&
 &  840,  840,  841,  839,  841,  841,  839,  840,  277,  277,  277,  277,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 8) = (/&
 & 1228, 1228, 1229, 1227, 1229, 1229, 1227, 1228,  404,  404,  404,  404,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912, 9) = (/&
 & 1130, 1130, 1131, 1129, 1131, 1131, 1129, 1130,  406,  406,  406,  406,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912,10) = (/&
 & 1040, 1040, 1041, 1039, 1041, 1041, 1039, 1040,  409,  409,  409,  409,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912,11) = (/&
 &  957,  957,  958,  956,  958,  958,  956,  957,  411,  411,  411,  411,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1912,12) = (/&
 &  880,  880,  881,  879,  881,  881,  879,  880,  453,  453,  453,  453,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 1) = (/&
 &  810,  810,  811,  809,  811,  811,  809,  810,  495,  495,  495,  495,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 2) = (/&
 &  745,  745,  746,  744,  746,  746,  744,  745,  535,  537,  536,  536,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 3) = (/&
 &  686,  686,  687,  685,  687,  687,  685,  686,  494,  494,  494,  494,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 4) = (/&
 &  631,  631,  632,  630,  632,  632,  630,  631,  454,  454,  454,  454,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 5) = (/&
 &  693,  693,  694,  692,  694,  694,  692,  693,  418,  418,  418,  418,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 6) = (/&
 &  638,  638,  639,  637,  639,  639,  637,  638,  385,  385,  385,  385,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 7) = (/&
 &  587,  587,  588,  586,  588,  588,  586,  587,  354,  354,  354,  354,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 8) = (/&
 &  540,  540,  541,  539,  541,  541,  539,  540,  326,  326,  326,  326,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913, 9) = (/&
 &  497,  497,  497,  497,  497,  497,  497,  497,  300,  300,  300,  300,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913,10) = (/&
 &  457,  457,  457,  457,  457,  457,  457,  457,  276,  276,  276,  276,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913,11) = (/&
 &  421,  421,  421,  421,  421,  421,  421,  421,  254,  254,  254,  254,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1913,12) = (/&
 &  387,  387,  387,  387,  387,  387,  387,  387,  234,  234,  234,  234,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 1) = (/&
 &  356,  356,  356,  356,  356,  356,  356,  356,  215,  215,  215,  215,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 2) = (/&
 &  328,  328,  328,  328,  328,  328,  328,  328,  210,  210,  210,  210,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 3) = (/&
 &  302,  302,  302,  302,  302,  302,  302,  302,  201,  201,  201,  201,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 4) = (/&
 &  278,  278,  278,  278,  278,  278,  278,  278,  191,  191,  191,  191,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 5) = (/&
 &  256,  256,  256,  256,  256,  256,  256,  256,  182,  182,  182,  182,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 6) = (/&
 &  235,  235,  235,  235,  235,  235,  235,  235,  167,  167,  167,  167,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 7) = (/&
 &  216,  216,  216,  216,  216,  216,  216,  216,  154,  154,  154,  154,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 8) = (/&
 &  199,  199,  199,  199,  199,  199,  199,  199,  142,  142,  142,  142,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914, 9) = (/&
 &  183,  183,  183,  183,  183,  183,  183,  183,  131,  131,  131,  131,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914,10) = (/&
 &  169,  169,  169,  169,  169,  169,  169,  169,  120,  120,  120,  120,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914,11) = (/&
 &  155,  155,  155,  155,  155,  155,  155,  155,  111,  111,  111,  111,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1914,12) = (/&
 &  143,  143,  143,  143,  143,  143,  143,  143,  102,  102,  102,  102,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 1) = (/&
 &  132,  132,  132,  132,  132,  132,  132,  132,   94,   94,   94,   94,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 2) = (/&
 &  121,  121,  121,  121,  121,  121,  121,  121,   86,   86,   86,   86,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 3) = (/&
 &  112,  112,  112,  112,  112,  112,  112,  112,   80,   80,   80,   80,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 4) = (/&
 &  103,  103,  103,  103,  103,  103,  103,  103,   73,   73,   73,   73,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 5) = (/&
 &   95,   95,   95,   95,   95,   95,   95,   95,   68,   68,   68,   68,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 6) = (/&
 &   87,   87,   87,   87,   87,   87,   87,   87,   62,   62,   62,   62,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 7) = (/&
 &   80,   80,   80,   80,   80,   80,   80,   80,   57,   57,   57,   57,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 8) = (/&
 &   74,   74,   74,   74,   74,   74,   74,   74,   53,   53,   53,   53,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915, 9) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   49,   49,   49,   49,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915,10) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   45,   45,   45,   45,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915,11) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   71,   71,   71,   71,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1915,12) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   71,   71,   71,   71,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1916, 1) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   65,   65,   65,   65,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1916, 2) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   60,   60,   60,   60,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1916, 3) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   55,   55,   55,   55,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1916, 4) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   51,   51,   51,   51,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1916, 5) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   47,   47,   47,   47,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1916, 6) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   43,   43,   43,   43,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1916, 7) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   40,   40,   40,   40,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1916, 8) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   37,   37,   37,   37,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1916, 9) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   34,   34,   34,   34,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1916,10) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   31,   31,   31,   31,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1916,11) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   29,   29,   29,   29,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1916,12) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   29,   29,   29,   29,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1917, 1) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   36,   36,   36,   36,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1917, 2) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   33,   33,   33,   33,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1917, 3) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   30,   30,   30,   30,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1917, 4) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   28,   28,   28,   28,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1917, 5) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   26,   26,   26,   26,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1917, 6) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   24,   24,   24,   24,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1917, 7) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   22,   22,   22,   22,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1917, 8) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1917, 9) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   19,   19,   19,   19,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1917,10) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1917,11) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1917,12) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   15,   15,   15,   15,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1918, 1) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   22,   22,   22,   22,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1918, 2) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   22,   22,   22,   22,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1918, 3) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   29,   29,   29,   29,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1918, 4) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   27,   27,   27,   27,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1918, 5) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   25,   25,   25,   25,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1918, 6) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   23,   23,   23,   23,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1918, 7) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   21,   21,   21,   21,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1918, 8) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1918, 9) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   18,   18,   18,   18,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1918,10) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   17,   17,   17,   17,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1918,11) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1918,12) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   15,   15,   15,   15,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1919, 1) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   15,   15,   15,   15,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1919, 2) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   22,   22,   22,   22,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1919, 3) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   29,   29,   29,   29,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1919, 4) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   29,   29,   29,   29,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1919, 5) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   27,   27,   27,   27,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1919, 6) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   25,   25,   25,   25,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1919, 7) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   23,   23,   23,   23,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1919, 8) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   21,   21,   21,   21,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1919, 9) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1919,10) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   18,   18,   18,   18,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1919,11) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   17,   17,   17,   17,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1919,12) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1920, 1) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1920, 2) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   13,   13,   13,   13,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1920, 3) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1920, 4) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1920, 5) = (/&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71/)
ZAERV(: ,1920, 6) = (/&
 &  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,&
 &  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101/)
ZAERV(: ,1920, 7) = (/&
 &  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,&
 &  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141/)
ZAERV(: ,1920, 8) = (/&
 &  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,&
 &  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,  171/)
ZAERV(: ,1920, 9) = (/&
 &  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,&
 &  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161/)
ZAERV(: ,1920,10) = (/&
 &  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,&
 &  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148/)
ZAERV(: ,1920,11) = (/&
 &  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,&
 &  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136/)
ZAERV(: ,1920,12) = (/&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126/)
ZAERV(: ,1921, 1) = (/&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116/)
ZAERV(: ,1921, 2) = (/&
 &  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,&
 &  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106/)
ZAERV(: ,1921, 3) = (/&
 &   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,&
 &   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98/)
ZAERV(: ,1921, 4) = (/&
 &   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,&
 &   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90/)
ZAERV(: ,1921, 5) = (/&
 &   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,&
 &   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83/)
ZAERV(: ,1921, 6) = (/&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77/)
ZAERV(: ,1921, 7) = (/&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71/)
ZAERV(: ,1921, 8) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65/)
ZAERV(: ,1921, 9) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60/)
ZAERV(: ,1921,10) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55/)
ZAERV(: ,1921,11) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51/)
ZAERV(: ,1921,12) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1922, 1) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1922, 2) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1922, 3) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1922, 4) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1922, 5) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1922, 6) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1922, 7) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1922, 8) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1922, 9) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1922,10) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1922,11) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1922,12) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1923, 1) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1923, 2) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1923, 3) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1923, 4) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1923, 5) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1923, 6) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1923, 7) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1923, 8) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1923, 9) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1923,10) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1923,11) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1923,12) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1924, 1) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1924, 2) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48/)
ZAERV(: ,1924, 3) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56/)
ZAERV(: ,1924, 4) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1924, 5) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1924, 6) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65/)
ZAERV(: ,1924, 7) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81/)
ZAERV(: ,1924, 8) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76/)
ZAERV(: ,1924, 9) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70/)
ZAERV(: ,1924,10) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70/)
ZAERV(: ,1924,11) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67/)
ZAERV(: ,1924,12) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64/)
ZAERV(: ,1925, 1) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59/)
ZAERV(: ,1925, 2) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1925, 3) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62/)
ZAERV(: ,1925, 4) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61/)
ZAERV(: ,1925, 5) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69/)
ZAERV(: ,1925, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64/)
ZAERV(: ,1925, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59/)
ZAERV(: ,1925, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1925, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1925,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1925,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1925,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1926, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1926, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1926, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1926, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1926, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1926, 6) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1926, 7) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1926, 8) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1926, 9) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1926,10) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1926,11) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1926,12) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1927, 1) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1927, 2) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1927, 3) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1927, 4) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1927, 5) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1927, 6) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1927, 7) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1927, 8) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1927, 9) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1927,10) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1927,11) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1927,12) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1928, 1) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1928, 2) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1928, 3) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1928, 4) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1928, 5) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70/)
ZAERV(: ,1928, 6) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85/)
ZAERV(: ,1928, 7) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98/)
ZAERV(: ,1928, 8) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &  112,  112,  112,  112,  112,  112,  112,  112,  112,  112,  112,  112/)
ZAERV(: ,1928, 9) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103/)
ZAERV(: ,1928,10) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95/)
ZAERV(: ,1928,11) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95/)
ZAERV(: ,1928,12) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120/)
ZAERV(: ,1929, 1) = (/&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,&
 &  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122/)
ZAERV(: ,1929, 2) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,  137/)
ZAERV(: ,1929, 3) = (/&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,&
 &  131,  131,  131,  131,  131,  131,  131,  131,  131,  131,  131,  131/)
ZAERV(: ,1929, 4) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,&
 &  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123/)
ZAERV(: ,1929, 5) = (/&
 &   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114/)
ZAERV(: ,1929, 6) = (/&
 &   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114/)
ZAERV(: ,1929, 7) = (/&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116/)
ZAERV(: ,1929, 8) = (/&
 &  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114/)
ZAERV(: ,1929, 9) = (/&
 &  104,  104,  104,  104,  104,  104,  104,  104,  104,  104,  104,  104,&
 &  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105,  105/)
ZAERV(: ,1929,10) = (/&
 &  100,  100,  100,  100,  100,  100,  100,  100,  100,  100,  100,  100,&
 &   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97/)
ZAERV(: ,1929,11) = (/&
 &   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91/)
ZAERV(: ,1929,12) = (/&
 &   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,&
 &   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84/)
ZAERV(: ,1930, 1) = (/&
 &   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77/)
ZAERV(: ,1930, 2) = (/&
 &   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71/)
ZAERV(: ,1930, 3) = (/&
 &   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65/)
ZAERV(: ,1930, 4) = (/&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61/)
ZAERV(: ,1930, 5) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69/)
ZAERV(: ,1930, 6) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71/)
ZAERV(: ,1930, 7) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66/)
ZAERV(: ,1930, 8) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61/)
ZAERV(: ,1930, 9) = (/&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56/)
ZAERV(: ,1930,10) = (/&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1930,11) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48/)
ZAERV(: ,1930,12) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1931, 1) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1931, 2) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1931, 3) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1931, 4) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1931, 5) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1931, 6) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1931, 7) = (/&
 &   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1931, 8) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1931, 9) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1931,10) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1931,11) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1931,12) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1932, 1) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1932, 2) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67/)
ZAERV(: ,1932, 3) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76/)
ZAERV(: ,1932, 4) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89/)
ZAERV(: ,1932, 5) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106/)
ZAERV(: ,1932, 6) = (/&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,&
 &  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123/)
ZAERV(: ,1932, 7) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &  127,  127,  127,  127,  127,  127,  127,  127,  127,  127,  127,  127/)
ZAERV(: ,1932, 8) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,&
 &  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129/)
ZAERV(: ,1932, 9) = (/&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,&
 &  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120,  120/)
ZAERV(: ,1932,10) = (/&
 &   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114/)
ZAERV(: ,1932,11) = (/&
 &   88,   88,   88,   88,   88,   88,   88,   88,   88,   88,   88,   88,&
 &  108,  108,  108,  108,  108,  108,  108,  108,  108,  108,  108,  108/)
ZAERV(: ,1932,12) = (/&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,&
 &  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,  102/)
ZAERV(: ,1933, 1) = (/&
 &   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,&
 &   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97,   97/)
ZAERV(: ,1933, 2) = (/&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,&
 &   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89/)
ZAERV(: ,1933, 3) = (/&
 &   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,&
 &   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,   86,   86/)
ZAERV(: ,1933, 4) = (/&
 &   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,&
 &   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85/)
ZAERV(: ,1933, 5) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82,   82/)
ZAERV(: ,1933, 6) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76/)
ZAERV(: ,1933, 7) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70/)
ZAERV(: ,1933, 8) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64/)
ZAERV(: ,1933, 9) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59/)
ZAERV(: ,1933,10) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1933,11) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1933,12) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1934, 1) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1934, 2) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1934, 3) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1934, 4) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1934, 5) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1934, 6) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1934, 7) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1934, 8) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1934, 9) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41/)
ZAERV(: ,1934,10) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1934,11) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1934,12) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1935, 1) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1935, 2) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1935, 3) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1935, 4) = (/&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1935, 5) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1935, 6) = (/&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1935, 7) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1935, 8) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1935, 9) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41/)
ZAERV(: ,1935,10) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1935,11) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1935,12) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1936, 1) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1936, 2) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1936, 3) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1936, 4) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1936, 5) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1936, 6) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1936, 7) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1936, 8) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1936, 9) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1936,10) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1936,11) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1936,12) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1937, 1) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1937, 2) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1937, 3) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1937, 4) = (/&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1937, 5) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1937, 6) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1937, 7) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1937, 8) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1937, 9) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1937,10) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1937,11) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1937,12) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1938, 1) = (/&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1938, 2) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1938, 3) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1938, 4) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1938, 5) = (/&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1938, 6) = (/&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1938, 7) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1938, 8) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1938, 9) = (/&
 &   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,   74,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1938,10) = (/&
 &   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1938,11) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1938,12) = (/&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1939, 1) = (/&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1939, 2) = (/&
 &   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1939, 3) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1939, 4) = (/&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1939, 5) = (/&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1939, 6) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1939, 7) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1939, 8) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1939, 9) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1939,10) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1939,11) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1939,12) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1940, 1) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1940, 2) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50,   50/)
ZAERV(: ,1940, 3) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1940, 4) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1940, 5) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1940, 6) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1940, 7) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1940, 8) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41/)
ZAERV(: ,1940, 9) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1940,10) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1940,11) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1940,12) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1941, 1) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1941, 2) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1941, 3) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23/)
ZAERV(: ,1941, 4) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1941, 5) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1941, 6) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1941, 7) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1941, 8) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1941, 9) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1941,10) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1941,11) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1941,12) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1942, 1) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1942, 2) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1942, 3) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1942, 4) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1942, 5) = (/&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1942, 6) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1942, 7) = (/&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1942, 8) = (/&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1942, 9) = (/&
 &  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1942,10) = (/&
 &  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1942,11) = (/&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1942,12) = (/&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1943, 1) = (/&
 &  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,  117,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1943, 2) = (/&
 &  108,  108,  108,  108,  108,  108,  108,  108,  108,  108,  108,  108,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1943, 3) = (/&
 &   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1943, 4) = (/&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1943, 5) = (/&
 &   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1943, 6) = (/&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1943, 7) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1943, 8) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1943, 9) = (/&
 &   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1943,10) = (/&
 &   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,   73,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1943,11) = (/&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1943,12) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 1) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 2) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 3) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 4) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 5) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 6) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 7) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 8) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944, 9) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944,10) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944,11) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1944,12) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 1) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 2) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 3) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 4) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 5) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 6) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 7) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 8) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945, 9) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945,10) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945,11) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1945,12) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1946, 1) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1946, 2) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1946, 3) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1946, 4) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1946, 5) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1946, 6) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1946, 7) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1946, 8) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1946, 9) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20/)
ZAERV(: ,1946,10) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1946,11) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17/)
ZAERV(: ,1946,12) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1947, 1) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1947, 2) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1947, 3) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1947, 4) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1947, 5) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1947, 6) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1947, 7) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1947, 8) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1947, 9) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1947,10) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1947,11) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1947,12) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1948, 1) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1948, 2) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1948, 3) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1948, 4) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1948, 5) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1948, 6) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1948, 7) = (/&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1948, 8) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1948, 9) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1948,10) = (/&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1948,11) = (/&
 &   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,   20,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1948,12) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1949, 1) = (/&
 &   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,   17,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1949, 2) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1949, 3) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1949, 4) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1949, 5) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1949, 6) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1949, 7) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1949, 8) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48/)
ZAERV(: ,1949, 9) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1949,10) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1949,11) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1949,12) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)

IF (LHOOK) CALL DR_HOOK('SUECAEC:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2


SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SUECAEC:PART3',0,ZHOOK_HANDLE)

ZAERV(: ,1950, 1) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1950, 2) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1950, 3) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1950, 4) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1950, 5) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1950, 6) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1950, 7) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1950, 8) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1950, 9) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1950,10) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1950,11) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1950,12) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28,   28/)
ZAERV(: ,1951, 1) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26/)
ZAERV(: ,1951, 2) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24/)
ZAERV(: ,1951, 3) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,   22/)
ZAERV(: ,1951, 4) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21/)
ZAERV(: ,1951, 5) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19/)
ZAERV(: ,1951, 6) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18/)
ZAERV(: ,1951, 7) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16/)
ZAERV(: ,1951, 8) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15/)
ZAERV(: ,1951, 9) = (/&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14/)
ZAERV(: ,1951,10) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13/)
ZAERV(: ,1951,11) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12/)
ZAERV(: ,1951,12) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11/)
ZAERV(: ,1952, 1) = (/&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10/)
ZAERV(: ,1952, 2) = (/&
 &   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,   75,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1952, 3) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9/)
ZAERV(: ,1952, 4) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1952, 5) = (/&
 &   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,   70,&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8/)
ZAERV(: ,1952, 6) = (/&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1952, 7) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7/)
ZAERV(: ,1952, 8) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1952, 9) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6/)
ZAERV(: ,1952,10) = (/&
 &   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1952,11) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1952,12) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5/)
ZAERV(: ,1953, 1) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1953, 2) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1953, 3) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1953, 4) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4/)
ZAERV(: ,1953, 5) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1953, 6) = (/&
 &   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,   64,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1953, 7) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1953, 8) = (/&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1953, 9) = (/&
 &   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1953,10) = (/&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3/)
ZAERV(: ,1953,11) = (/&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1953,12) = (/&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 1) = (/&
 &   96,   96,   96,   96,   96,   96,   96,   96,   96,   96,   96,   96,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 2) = (/&
 &   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 3) = (/&
 &   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 4) = (/&
 &   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 5) = (/&
 &   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 6) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 7) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 8) = (/&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954, 9) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954,10) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954,11) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1954,12) = (/&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 1) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 2) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 3) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 4) = (/&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 5) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 6) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 7) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 8) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955, 9) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955,10) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955,11) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1955,12) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 1) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 2) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 3) = (/&
 &   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,   26,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 4) = (/&
 &   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 5) = (/&
 &   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,   23,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 6) = (/&
 &   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 7) = (/&
 &   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,   19,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 8) = (/&
 &   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,   18,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956, 9) = (/&
 &   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,   16,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956,10) = (/&
 &   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,   15,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956,11) = (/&
 &   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,   14,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1956,12) = (/&
 &   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,   13,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 1) = (/&
 &   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 2) = (/&
 &   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 3) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 4) = (/&
 &   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 5) = (/&
 &    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 6) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 7) = (/&
 &    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 8) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957, 9) = (/&
 &    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957,10) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957,11) = (/&
 &    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1957,12) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 1) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 2) = (/&
 &    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 3) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 4) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 5) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 6) = (/&
 &    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 7) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 8) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958, 9) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958,10) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958,11) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1958,12) = (/&
 &    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 4) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 5) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 6) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 7) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 8) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959, 9) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959,10) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959,11) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1959,12) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1960, 1) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1960, 2) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1960, 3) = (/&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,&
 &    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2/)
ZAERV(: ,1960, 4) = (/&
 &   15,   15,   15,   16,   16,   16,   15,   16,   17,   16,   16,   14,&
 &   13,   13,   14,   15,   15,   15,   16,   16,   17,   16,   16,   16/)
ZAERV(: ,1960, 5) = (/&
 &   29,   29,   29,   30,   30,   30,   28,   29,   32,   29,   29,   27,&
 &   24,   25,   26,   27,   28,   29,   30,   31,   31,   31,   31,   30/)
ZAERV(: ,1960, 6) = (/&
 &   44,   44,   44,   46,   45,   45,   42,   44,   47,   44,   44,   40,&
 &   36,   37,   39,   41,   41,   43,   45,   46,   47,   47,   46,   45/)
ZAERV(: ,1960, 7) = (/&
 &   58,   58,   58,   61,   60,   60,   57,   59,   63,   59,   59,   53,&
 &   48,   50,   52,   55,   55,   58,   60,   62,   62,   62,   61,   60/)
ZAERV(: ,1960, 8) = (/&
 &   72,   72,   72,   75,   74,   74,   70,   72,   78,   73,   73,   66,&
 &   59,   61,   65,   67,   68,   71,   74,   76,   77,   77,   76,   74/)
ZAERV(: ,1960, 9) = (/&
 &   86,   86,   86,   90,   90,   90,   84,   87,   94,   87,   87,   79,&
 &   71,   74,   78,   81,   82,   85,   89,   92,   93,   92,   91,   89/)
ZAERV(: ,1960,10) = (/&
 &   86,   86,   86,   90,   90,   90,   84,   87,   94,   87,   87,   79,&
 &   71,   74,   78,   81,   82,   85,   89,   92,   93,   92,   91,   89/)
ZAERV(: ,1960,11) = (/&
 &   86,   86,   86,   90,   90,   90,   84,   87,   94,   87,   87,   79,&
 &   71,   74,   78,   81,   82,   85,   89,   92,   93,   92,   91,   89/)
ZAERV(: ,1960,12) = (/&
 &   86,   86,   86,   90,   90,   90,   84,   87,   94,   87,   87,   79,&
 &   71,   74,   78,   81,   82,   85,   89,   92,   93,   92,   91,   89/)
ZAERV(: ,1961, 1) = (/&
 &   79,   79,   79,   83,   82,   82,   77,   80,   86,   80,   80,   73,&
 &   65,   68,   72,   75,   75,   79,   82,   85,   85,   85,   84,   82/)
ZAERV(: ,1961, 2) = (/&
 &   73,   73,   73,   77,   76,   76,   71,   74,   80,  165,  165,   80,&
 &   60,   62,   66,   73,   90,   80,   75,   78,   78,   78,   77,   75/)
ZAERV(: ,1961, 3) = (/&
 &   67,   67,   67,   71,   70,   70,   66,   68,  136,  329,  329,  180,&
 &   77,   63,   61,   67,   83,   74,   69,   72,   72,   72,   71,   69/)
ZAERV(: ,1961, 4) = (/&
 &   62,   62,   62,   65,   64,   64,   60,   63,  163,  398,  398,  213,&
 &   84,   69,   56,   62,   76,   68,   64,   66,   67,   66,   65,   64/)
ZAERV(: ,1961, 5) = (/&
 &   69,   69,   69,   60,   59,   59,   56,   58,  150,  366,  366,  196,&
 &   78,   63,   52,   57,   70,   63,   59,   61,   61,   61,   60,   59/)
ZAERV(: ,1961, 6) = (/&
 &   78,   78,   78,   55,   55,   55,   51,   53,  139,  337,  337,  180,&
 &   72,   58,   48,   67,   90,   80,   69,   58,   57,   56,   56,   54/)
ZAERV(: ,1961, 7) = (/&
 &   71,   71,   71,   51,   63,   63,   47,   49,  128,  310,  310,  166,&
 &   66,   54,   44,   61,   83,   74,   64,   54,   52,   52,   51,   50/)
ZAERV(: ,1961, 8) = (/&
 &   66,   66,   66,   47,   58,   58,   44,   70,  117,  285,  285,  153,&
 &  126,  104,   75,   57,   76,   68,   59,   49,   48,   48,   47,   46/)
ZAERV(: ,1961, 9) = (/&
 &   61,   61,   61,   43,   53,   53,   40,   64,  108,  263,  263,  168,&
 &  203,  172,  134,   90,   70,   63,   54,   46,   44,   44,   43,   42/)
ZAERV(: ,1961,10) = (/&
 &   56,   56,   56,   40,   49,   49,   37,   59,  100,  242,  242,  179,&
 &  230,  188,  135,   83,   65,   58,   50,   42,   41,   41,   40,   39/)
ZAERV(: ,1961,11) = (/&
 &   52,   52,   52,   37,   45,   45,   67,   55,   92,  222,  222,  164,&
 &  237,  193,  135,   76,   60,   53,   46,   39,   38,   37,   37,   36/)
ZAERV(: ,1961,12) = (/&
 &   47,   47,   47,   34,   42,   42,   67,   50,   84,  205,  205,  151,&
 &  219,  177,  124,   70,   55,   49,   43,   36,   35,   34,   34,   33/)
ZAERV(: ,1962, 1) = (/&
 &   44,   44,   44,   31,   39,   39,   62,   46,   78,  188,  188,  139,&
 &  201,  163,  115,   65,   51,   45,   39,   33,   32,   32,   31,   31/)
ZAERV(: ,1962, 2) = (/&
 &   40,   40,   40,   29,   45,   45,   57,   43,   72,  173,  173,  128,&
 &  185,  150,  106,   60,   58,   52,   45,   37,   30,   29,   29,   28/)
ZAERV(: ,1962, 3) = (/&
 &   37,   37,   37,   27,   63,   63,   53,   39,   66,  160,  160,  119,&
 &  175,  189,  207,  225,  235,  210,  182,  151,  115,   77,   38,   26/)
ZAERV(: ,1962, 4) = (/&
 &   34,   34,   34,   25,   58,   58,   49,   36,   61,  147,  147,  169,&
 &  272,  265,  256,  241,  219,  196,  170,  141,  106,   71,   35,   24/)
ZAERV(: ,1962, 5) = (/&
 &   32,   32,   32,   23,   53,   53,   84,   34,   56,  135,  135,  156,&
 &  251,  243,  235,  222,  202,  180,  156,  130,   98,   65,   32,   22/)
ZAERV(: ,1962, 6) = (/&
 &   29,   29,   29,   21,   49,   49,  125,   31,   52,  125,  125,  143,&
 &  231,  224,  217,  204,  186,  166,  144,  119,   90,   60,   30,   21/)
ZAERV(: ,1962, 7) = (/&
 &   27,   27,   27,   19,   45,   45,  134,   29,   48,  115,  115,  211,&
 &  272,  227,  199,  188,  171,  153,  132,  110,   83,   56,   28,   19/)
ZAERV(: ,1962, 8) = (/&
 &   25,   25,   25,   19,   42,   42,  123,   26,   54,  130,  130,  317,&
 &  425,  344,  242,  173,  157,  141,  122,  101,   76,   51,   25,   18/)
ZAERV(: ,1962, 9) = (/&
 &   27,   27,   27,   17,   39,   39,  113,   24,   50,  120,  120,  291,&
 &  391,  317,  223,  159,  145,  130,  112,   93,   70,   47,   24,   16/)
ZAERV(: ,1962,10) = (/&
 &   69,   69,   69,   16,   36,   36,  104,  104,   73,  111,  111,  268,&
 &  360,  292,  205,  147,  133,  119,  103,   86,   65,   44,   22,   15/)
ZAERV(: ,1962,11) = (/&
 &  112,  112,  112,   46,   33,   33,   96,   96,   68,  102,  102,  247,&
 &  331,  268,  189,  135,  123,  110,   95,   79,   60,   40,   20,   14/)
ZAERV(: ,1962,12) = (/&
 &  154,  154,  154,   55,   30,   30,   88,   88,   62,   94,   94,  227,&
 &  305,  247,  174,  124,  113,  101,   88,   73,   55,   37,   19,   13/)
ZAERV(: ,1963, 1) = (/&
 &  197,  197,  197,   50,   28,   28,   84,   81,   57,  122,  122,  209,&
 &  281,  227,  160,  114,  104,   93,   81,   67,   51,   34,   17,   12/)
ZAERV(: ,1963, 2) = (/&
 &  239,  239,  239,   64,   26,   26,  109,   75,   58,  139,  139,  192,&
 &  258,  209,  147,  105,   96,   86,   74,   62,   47,   31,   16,   11/)
ZAERV(: ,1963, 3) = (/&
 &  248,  248,  248,  135,   24,   24,  100,  130,  144,  139,  139,  177,&
 &  238,  193,  135,   97,   88,   79,   68,   57,   43,   29,   15,   10/)
ZAERV(: ,1963, 4) = (/&
 &  228,  228,  228,  135,   22,   22,   92,  120,  133,  130,  130,  230,&
 &  286,  276,  263,  246,  219,  196,  170,  141,  106,   71,   35,   10/)
ZAERV(: ,1963, 5) = (/&
 &  210,  210,  210,  124,   36,   36,  109,  110,  122,  122,  122,  451,&
 &  648,  635,  621,  596,  550,  491,  426,  352,  267,  178,   88,    9/)
ZAERV(: ,1963, 6) = (/&
 &  193,  193,  193,  114,   34,   34,  100,  102,  112,  174,  174,  485,&
 &  669,  745,  845,  942, 1011,  902,  782,  646,  490,  325,  161,    8/)
ZAERV(: ,1963, 7) = (/&
 &  178,  178,  178,  105,   31,   31,   93,  121,  194,  277,  277,  475,&
 &  616,  709,  869, 1031, 1164, 1038,  902,  746,  564,  374,  185,    8/)
ZAERV(: ,1963, 8) = (/&
 &  164,  164,  164,   97,   29,   29,   85,  242,  310,  363,  363,  475,&
 &  566,  688,  893, 1104, 1285, 1147,  995,  824,  623,  412,  204,    7/)
ZAERV(: ,1963, 9) = (/&
 &  151,  151,  151,   89,   26,   26,   78,  223,  285,  398,  398,  545,&
 &  621,  779,  985, 1197, 1375, 1225, 1063,  881,  667,  442,  219,    7/)
ZAERV(: ,1963,10) = (/&
 &  139,  139,  139,   82,   81,   81,  100,  205,  263,  366,  366,  885,&
 & 1190, 1245, 1320, 1381, 1399, 1288, 1164, 1021,  846,  658,  470,  282/)
ZAERV(: ,1963,11) = (/&
 &  137,  137,  137,   99,  134,  134,  175,  362,  328,  337,  337, 1061,&
 & 1575, 1535, 1496, 1426, 1311, 1317, 1321, 1312, 1269, 1210, 1141, 1063/)
ZAERV(: ,1963,12) = (/&
 &  188,  188,  188,  162,  187,  187,  274,  423,  339,  310,  310,  975,&
 & 1448, 1414, 1377, 1312, 1206, 1213, 1216, 1244, 1304, 1345, 1374, 1387/)
ZAERV(: ,1964, 1) = (/&
 &  239,  239,  239,  206,  284,  284,  308,  397,  322,  285,  285,  899,&
 & 1331, 1301, 1266, 1208, 1110, 1115, 1118, 1146, 1200, 1237, 1303, 1353/)
ZAERV(: ,1964, 2) = (/&
 &  290,  290,  290,  269,  346,  346,  291,  365,  297,  263,  263,  827,&
 & 1225, 1197, 1164, 1112, 1020, 1026, 1029, 1054, 1104, 1153, 1218, 1265/)
ZAERV(: ,1964, 3) = (/&
 &  333,  333,  333,  323,  337,  337,  268,  336,  273,  277,  277,  760,&
 & 1128, 1102, 1072, 1022,  939,  945,  946,  970, 1016, 1063, 1120, 1163/)
ZAERV(: ,1964, 4) = (/&
 &  367,  367,  367,  349,  311,  310,  247,  309,  251,  346,  346,  818,&
 & 1093, 1013,  987,  941,  864,  870,  871,  893,  935,  978, 1032, 1072/)
ZAERV(: ,1964, 5) = (/&
 &  384,  384,  384,  322,  286,  286,  227,  285,  231,  381,  381,  927,&
 & 1245, 1114,  950,  865,  794,  800,  802,  823,  859,  900,  948,  985/)
ZAERV(: ,1964, 6) = (/&
 &  367,  367,  367,  297,  263,  263,  209,  262,  215,  398,  398, 1090,&
 & 1496, 1327, 1114,  861,  731,  736,  738,  760,  803,  837,  873,  907/)
ZAERV(: ,1964, 7) = (/&
 &  342,  342,  342,  272,  242,  242,  192,  241,  258,  389,  389, 1323,&
 & 1879, 1670, 1403, 1086,  728,  761,  789,  815,  819,  815,  803,  834/)
ZAERV(: ,1964, 8) = (/&
 &  314,  314,  314,  260,  223,  223,  177,  222,  287,  363,  363, 1353,&
 & 1943, 1716, 1434, 1098,  720,  740,  758,  771,  764,  750,  740,  768/)
ZAERV(: ,1964, 9) = (/&
 &  289,  289,  289,  323,  205,  205,  163,  276,  319,  334,  334, 1244,&
 & 1786, 1579, 1318, 1012,  663,  680,  698,  710,  703,  690,  680,  706/)
ZAERV(: ,1964,10) = (/&
 &  266,  266,  266,  297,  205,  205,  150,  371,  358,  308,  308, 1143,&
 & 1643, 1452, 1214,  930,  611,  627,  642,  653,  648,  634,  625,  651/)
ZAERV(: ,1964,11) = (/&
 &  245,  245,  245,  273,  284,  284,  192,  449,  388,  283,  283, 1052,&
 & 1514, 1338, 1118,  856,  561,  577,  590,  600,  595,  631,  712,  783/)
ZAERV(: ,1964,12) = (/&
 &  226,  226,  226,  251,  364,  364,  357,  457,  370,  261,  261,  970,&
 & 1390, 1230, 1028,  787,  517,  530,  544,  552,  580,  685,  783,  870/)
ZAERV(: ,1965, 1) = (/&
 &  273,  273,  273,  260,  435,  435,  407,  423,  340,  240,  240,  892,&
 & 1280, 1133,  946,  724,  475,  488,  499,  509,  581,  684,  779,  861/)
ZAERV(: ,1965, 2) = (/&
 &  486,  486,  486,  385,  444,  444,  375,  389,  313,  221,  221,  819,&
 & 1178, 1041,  870,  668,  437,  449,  460,  467,  538,  628,  718,  793/)
ZAERV(: ,1965, 3) = (/&
 &  563,  563,  564,  574,  408,  408,  345,  358,  288,  203,  203,  754,&
 & 1083,  958,  800,  613,  402,  413,  423,  430,  495,  578,  661,  730/)
ZAERV(: ,1965, 4) = (/&
 &  518,  519,  519,  592,  376,  376,  317,  329,  276,  208,  208,  696,&
 &  997,  882,  737,  564,  370,  380,  389,  396,  456,  532,  608,  672/)
ZAERV(: ,1965, 5) = (/&
 &  477,  477,  477,  565,  346,  346,  292,  303,  254,  277,  277,  639,&
 &  917,  811,  679,  520,  341,  350,  358,  364,  420,  490,  558,  617/)
ZAERV(: ,1965, 6) = (/&
 &  439,  439,  439,  520,  318,  318,  269,  279,  234,  303,  303,  589,&
 &  843,  746,  624,  478,  324,  322,  330,  335,  386,  450,  514,  568/)
ZAERV(: ,1965, 7) = (/&
 &  404,  404,  404,  477,  293,  293,  247,  257,  226,  312,  312,  541,&
 &  778,  687,  574,  440,  298,  296,  303,  309,  355,  415,  473,  523/)
ZAERV(: ,1965, 8) = (/&
 &  371,  371,  371,  439,  269,  269,  228,  236,  224,  287,  287,  498,&
 &  715,  632,  528,  405,  275,  273,  279,  284,  327,  381,  435,  481/)
ZAERV(: ,1965, 9) = (/&
 &  342,  342,  342,  404,  258,  258,  209,  218,  231,  264,  264,  458,&
 &  657,  582,  486,  372,  253,  251,  257,  261,  301,  351,  400,  442/)
ZAERV(: ,1965,10) = (/&
 &  315,  315,  315,  372,  258,  258,  193,  233,  244,  243,  243,  422,&
 &  604,  535,  447,  343,  233,  231,  236,  241,  277,  323,  368,  407/)
ZAERV(: ,1965,11) = (/&
 &  289,  289,  289,  342,  249,  249,  250,  251,  256,  224,  224,  388,&
 &  557,  492,  411,  315,  243,  243,  242,  238,  255,  297,  339,  375/)
ZAERV(: ,1965,12) = (/&
 &  266,  266,  266,  315,  240,  240,  291,  259,  276,  251,  251,  357,&
 &  512,  453,  379,  343,  381,  353,  322,  287,  243,  274,  312,  345/)
ZAERV(: ,1966, 1) = (/&
 &  286,  286,  286,  300,  298,  298,  279,  289,  312,  290,  290,  329,&
 &  472,  417,  348,  315,  350,  325,  297,  305,  308,  306,  302,  317/)
ZAERV(: ,1966, 2) = (/&
 &  273,  273,  273,  287,  284,  284,  266,  276,  298,  277,  277,  302,&
 &  434,  384,  321,  290,  322,  299,  282,  292,  294,  292,  289,  292/)
ZAERV(: ,1966, 3) = (/&
 &  261,  261,  261,  273,  271,  271,  254,  263,  284,  264,  264,  278,&
 &  399,  353,  295,  267,  297,  275,  269,  278,  280,  279,  275,  269/)
ZAERV(: ,1966, 4) = (/&
 &  248,  248,  248,  260,  258,  258,  241,  251,  270,  251,  251,  256,&
 &  367,  325,  272,  246,  273,  253,  256,  264,  267,  265,  262,  255/)
ZAERV(: ,1966, 5) = (/&
 &  235,  235,  235,  247,  244,  244,  229,  238,  256,  238,  238,  236,&
 &  338,  299,  250,  226,  251,  233,  242,  251,  253,  251,  248,  242/)
ZAERV(: ,1966, 6) = (/&
 &  222,  222,  222,  233,  231,  231,  216,  225,  242,  225,  225,  217,&
 &  311,  275,  230,  209,  231,  220,  229,  237,  239,  238,  235,  229/)
ZAERV(: ,1966, 7) = (/&
 &  210,  210,  210,  220,  218,  218,  204,  212,  228,  212,  212,  200,&
 &  286,  253,  212,  197,  213,  208,  216,  223,  225,  224,  221,  216/)
ZAERV(: ,1966, 8) = (/&
 &  197,  197,  197,  206,  205,  205,  192,  199,  215,  199,  199,  184,&
 &  264,  233,  195,  185,  196,  195,  203,  210,  212,  210,  208,  203/)
ZAERV(: ,1966, 9) = (/&
 &  184,  184,  184,  193,  191,  191,  179,  186,  201,  187,  187,  169,&
 &  243,  215,  179,  173,  180,  182,  190,  196,  198,  197,  194,  190/)
ZAERV(: ,1966,10) = (/&
 &  171,  171,  171,  180,  178,  178,  167,  173,  187,  174,  174,  157,&
 &  223,  198,  165,  161,  166,  170,  177,  183,  184,  183,  181,  176/)
ZAERV(: ,1966,11) = (/&
 &  159,  159,  159,  166,  165,  165,  154,  160,  173,  161,  161,  145,&
 &  205,  182,  152,  149,  153,  157,  163,  169,  170,  169,  167,  163/)
ZAERV(: ,1966,12) = (/&
 &  146,  146,  146,  153,  152,  152,  142,  147,  159,  148,  148,  134,&
 &  189,  167,  140,  137,  141,  145,  150,  156,  157,  156,  154,  150/)
ZAERV(: ,1967, 1) = (/&
 &  134,  134,  134,  141,  140,  140,  131,  136,  146,  136,  136,  123,&
 &  174,  154,  129,  126,  130,  133,  139,  143,  144,  144,  142,  138/)
ZAERV(: ,1967, 2) = (/&
 &  124,  124,  124,  130,  129,  129,  120,  125,  135,  125,  125,  113,&
 &  160,  142,  119,  116,  119,  123,  128,  132,  133,  132,  131,  127/)
ZAERV(: ,1967, 3) = (/&
 &  114,  114,  114,  119,  118,  118,  111,  115,  124,  115,  115,  104,&
 &  148,  131,  109,  107,  110,  113,  117,  121,  122,  122,  120,  117/)
ZAERV(: ,1967, 4) = (/&
 &  105,  105,  105,  110,  109,  109,  102,  106,  114,  106,  106,   96,&
 &  136,  120,  101,   99,  101,  104,  108,  112,  113,  112,  111,  108/)
ZAERV(: ,1967, 5) = (/&
 &  112,  112,  112,  117,  116,  116,  109,  113,  122,  113,  113,  102,&
 &  125,  111,  101,  105,  106,  111,  115,  119,  120,  119,  118,  115/)
ZAERV(: ,1967, 6) = (/&
 &  129,  129,  129,  135,  134,  134,  125,  130,  140,  130,  130,  118,&
 &  115,  110,  116,  121,  122,  128,  133,  137,  138,  138,  136,  133/)
ZAERV(: ,1967, 7) = (/&
 &  146,  146,  146,  153,  151,  151,  142,  147,  159,  148,  148,  133,&
 &  119,  124,  131,  137,  138,  144,  150,  155,  157,  156,  154,  150/)
ZAERV(: ,1967, 8) = (/&
 &  163,  163,  163,  171,  169,  169,  158,  165,  177,  165,  165,  149,&
 &  133,  139,  147,  153,  154,  161,  168,  174,  175,  174,  172,  168/)
ZAERV(: ,1967, 9) = (/&
 &  180,  180,  180,  189,  187,  187,  175,  182,  196,  182,  182,  165,&
 &  147,  153,  162,  169,  171,  178,  185,  192,  193,  192,  190,  185/)
ZAERV(: ,1967,10) = (/&
 &  197,  197,  197,  206,  205,  205,  192,  199,  215,  199,  199,  180,&
 &  161,  168,  177,  185,  187,  195,  203,  210,  212,  210,  208,  203/)
ZAERV(: ,1967,11) = (/&
 &  181,  181,  181,  190,  188,  188,  176,  183,  197,  184,  184,  166,&
 &  148,  155,  163,  170,  172,  180,  187,  193,  195,  194,  191,  187/)
ZAERV(: ,1967,12) = (/&
 &  167,  167,  167,  175,  173,  173,  162,  169,  182,  169,  169,  153,&
 &  136,  142,  150,  157,  158,  165,  172,  178,  179,  178,  176,  172/)
ZAERV(: ,1968, 1) = (/&
 &  154,  154,  154,  161,  160,  160,  149,  155,  167,  156,  156,  141,&
 &  126,  131,  138,  144,  146,  152,  158,  164,  165,  164,  162,  158/)
ZAERV(: ,1968, 2) = (/&
 &  141,  141,  141,  148,  147,  147,  138,  143,  154,  143,  143,  129,&
 &  116,  121,  127,  133,  134,  140,  146,  151,  152,  151,  149,  146/)
ZAERV(: ,1968, 3) = (/&
 &  130,  130,  130,  136,  135,  135,  127,  131,  142,  132,  132,  119,&
 &  106,  111,  117,  122,  123,  129,  134,  139,  140,  139,  137,  134/)
ZAERV(: ,1968, 4) = (/&
 &  120,  120,  120,  126,  124,  124,  117,  121,  131,  121,  121,  110,&
 &   98,  102,  108,  113,  114,  119,  123,  128,  129,  128,  126,  123/)
ZAERV(: ,1968, 5) = (/&
 &  146,  146,  146,  153,  151,  151,  142,  147,  159,  148,  148,  133,&
 &  119,  124,  131,  137,  138,  144,  150,  155,  157,  156,  154,  150/)
ZAERV(: ,1968, 6) = (/&
 &  214,  214,  214,  224,  222,  222,  208,  216,  233,  217,  217,  196,&
 &  175,  182,  193,  201,  203,  212,  221,  228,  230,  229,  226,  220/)
ZAERV(: ,1968, 7) = (/&
 &  282,  282,  282,  296,  293,  293,  274,  285,  307,  286,  286,  258,&
 &  230,  241,  254,  265,  268,  279,  291,  301,  303,  301,  298,  291/)
ZAERV(: ,1968, 8) = (/&
 &  350,  350,  350,  367,  364,  364,  341,  354,  382,  355,  355,  320,&
 &  286,  299,  315,  329,  332,  347,  361,  373,  376,  374,  370,  361/)
ZAERV(: ,1968, 9) = (/&
 &  418,  418,  418,  439,  435,  435,  407,  423,  456,  424,  424,  383,&
 &  342,  357,  377,  393,  397,  414,  431,  446,  450,  447,  442,  431/)
ZAERV(: ,1968,10) = (/&
 &  486,  486,  486,  510,  506,  506,  473,  492,  531,  493,  492,  445,&
 &  397,  415,  438,  457,  461,  482,  502,  519,  523,  521,  513,  500/)
ZAERV(: ,1968,11) = (/&
 &  470,  470,  470,  493,  489,  489,  458,  475,  513,  476,  476,  430,&
 &  384,  401,  424,  442,  446,  466,  485,  502,  504,  503,  497,  484/)
ZAERV(: ,1968,12) = (/&
 &  455,  455,  455,  477,  473,  473,  443,  460,  496,  461,  461,  416,&
 &  372,  388,  410,  427,  432,  451,  469,  485,  489,  486,  480,  469/)
ZAERV(: ,1969, 1) = (/&
 &  439,  439,  439,  460,  456,  456,  427,  443,  478,  445,  445,  401,&
 &  358,  374,  395,  412,  416,  435,  452,  468,  472,  469,  463,  452/)
ZAERV(: ,1969, 2) = (/&
 &  423,  423,  423,  444,  440,  440,  412,  428,  462,  429,  429,  387,&
 &  346,  361,  381,  398,  402,  420,  437,  451,  455,  452,  447,  436/)
ZAERV(: ,1969, 3) = (/&
 &  407,  407,  407,  427,  423,  423,  396,  411,  444,  413,  413,  373,&
 &  333,  347,  367,  383,  386,  404,  420,  434,  438,  435,  430,  420/)
ZAERV(: ,1969, 4) = (/&
 &  392,  392,  392,  411,  407,  407,  381,  396,  427,  397,  397,  359,&
 &  320,  334,  353,  368,  372,  388,  404,  418,  421,  419,  414,  404/)
ZAERV(: ,1969, 5) = (/&
 &  376,  376,  376,  394,  391,  391,  366,  380,  410,  381,  381,  344,&
 &  307,  320,  338,  353,  356,  372,  387,  401,  404,  402,  397,  387/)
ZAERV(: ,1969, 6) = (/&
 &  359,  359,  359,  377,  374,  374,  350,  363,  392,  364,  364,  329,&
 &  294,  307,  324,  338,  341,  356,  371,  383,  386,  384,  380,  370/)
ZAERV(: ,1969, 7) = (/&
 &  344,  344,  344,  361,  358,  358,  335,  348,  375,  349,  349,  315,&
 &  281,  294,  310,  323,  327,  341,  355,  367,  370,  368,  363,  355/)
ZAERV(: ,1969, 8) = (/&
 &  328,  328,  328,  344,  341,  341,  319,  331,  358,  332,  332,  300,&
 &  268,  280,  296,  308,  311,  325,  338,  350,  353,  351,  346,  338/)
ZAERV(: ,1969, 9) = (/&
 &  313,  313,  313,  328,  325,  325,  304,  316,  341,  317,  317,  286,&
 &  256,  267,  282,  294,  297,  310,  322,  333,  336,  334,  330,  322/)
ZAERV(: ,1969,10) = (/&
 &  296,  296,  296,  311,  308,  308,  289,  300,  323,  300,  300,  271,&
 &  242,  253,  267,  279,  281,  294,  306,  316,  319,  317,  313,  305/)
ZAERV(: ,1969,11) = (/&
 &  281,  281,  281,  295,  292,  292,  274,  284,  306,  285,  285,  257,&
 &  230,  240,  253,  264,  267,  279,  290,  300,  302,  300,  297,  290/)
ZAERV(: ,1969,12) = (/&
 &  265,  265,  265,  278,  275,  275,  258,  268,  289,  268,  268,  243,&
 &  217,  226,  239,  249,  251,  263,  273,  282,  285,  283,  280,  273/)
ZAERV(: ,1970, 1) = (/&
 &  249,  249,  249,  261,  259,  259,  242,  251,  271,  252,  252,  228,&
 &  203,  212,  224,  234,  236,  247,  257,  265,  267,  266,  263,  256/)
ZAERV(: ,1970, 2) = (/&
 &  233,  233,  233,  245,  243,  243,  227,  236,  254,  237,  237,  214,&
 &  191,  199,  210,  219,  222,  231,  241,  249,  251,  249,  247,  241/)
ZAERV(: ,1970, 3) = (/&
 &  217,  217,  217,  228,  226,  226,  212,  220,  237,  220,  220,  199,&
 &  178,  185,  196,  204,  206,  215,  224,  232,  234,  232,  229,  224/)
ZAERV(: ,1970, 4) = (/&
 &  202,  202,  202,  212,  210,  210,  197,  204,  220,  205,  205,  185,&
 &  165,  172,  182,  190,  192,  200,  208,  215,  217,  216,  213,  208/)
ZAERV(: ,1970, 5) = (/&
 &  186,  186,  186,  195,  193,  193,  181,  188,  203,  188,  188,  170,&
 &  152,  159,  168,  175,  176,  184,  192,  198,  200,  199,  196,  192/)
ZAERV(: ,1970, 6) = (/&
 &  171,  171,  171,  179,  178,  178,  167,  173,  186,  173,  173,  157,&
 &  140,  146,  154,  161,  162,  170,  176,  182,  184,  183,  181,  176/)
ZAERV(: ,1970, 7) = (/&
 &  157,  157,  157,  165,  164,  164,  153,  159,  172,  160,  160,  144,&
 &  129,  134,  142,  148,  149,  156,  162,  168,  169,  168,  166,  162/)
ZAERV(: ,1970, 8) = (/&
 &  145,  145,  145,  152,  151,  151,  141,  147,  158,  147,  147,  133,&
 &  119,  124,  131,  136,  138,  144,  149,  155,  156,  155,  153,  149/)
ZAERV(: ,1970, 9) = (/&
 &  133,  133,  133,  140,  139,  139,  130,  135,  145,  135,  135,  122,&
 &  109,  114,  120,  125,  127,  132,  138,  142,  143,  143,  141,  138/)
ZAERV(: ,1970,10) = (/&
 &  123,  123,  123,  129,  128,  128,  120,  124,  134,  125,  125,  113,&
 &  101,  105,  111,  115,  117,  122,  127,  131,  132,  131,  130,  127/)
ZAERV(: ,1970,11) = (/&
 &  113,  113,  113,  119,  118,  118,  110,  114,  123,  115,  115,  104,&
 &   93,   97,  102,  106,  107,  112,  117,  121,  122,  121,  119,  117/)
ZAERV(: ,1970,12) = (/&
 &  104,  104,  104,  109,  108,  108,  101,  105,  113,  106,  106,   95,&
 &   85,   89,   94,   98,   99,  103,  107,  111,  112,  111,  110,  107/)
ZAERV(: ,1971, 1) = (/&
 &   96,   96,   96,  101,  100,  100,   93,   97,  105,   97,   97,   88,&
 &   79,   82,   87,   90,   91,   95,   99,  102,  103,  102,  101,   99/)
ZAERV(: ,1971, 2) = (/&
 &   88,   88,   88,   93,   92,   92,   86,   89,   96,   89,   89,   81,&
 &   72,   75,   80,   83,   84,   88,   91,   94,   95,   94,   93,   91/)
ZAERV(: ,1971, 3) = (/&
 &   81,   81,   81,   85,   85,   85,   79,   82,   89,   82,   82,   75,&
 &   67,   69,   73,   76,   77,   81,   84,   87,   87,   87,   86,   84/)
ZAERV(: ,1971, 4) = (/&
 &   75,   75,   75,   79,   78,   78,   73,   76,   82,   76,   76,   69,&
 &   61,   64,   68,   70,   71,   74,   77,   80,   80,   80,   79,   77/)
ZAERV(: ,1971, 5) = (/&
 &   69,   69,   69,   72,   72,   72,   67,   70,   75,   70,   70,   63,&
 &   57,   59,   62,   65,   66,   68,   71,   74,   74,   74,   73,   71/)
ZAERV(: ,1971, 6) = (/&
 &   64,   64,   64,   67,   66,   66,   62,   64,   69,   64,   64,   58,&
 &   52,   54,   57,   60,   60,   63,   66,   68,   68,   68,   67,   65/)
ZAERV(: ,1971, 7) = (/&
 &   59,   59,   59,   61,   61,   61,   57,   59,   64,   59,   59,   54,&
 &   48,   50,   53,   55,   56,   58,   60,   62,   63,   63,   62,   60/)
ZAERV(: ,1971, 8) = (/&
 &   54,   54,   54,   57,   56,   56,   53,   55,   59,   55,   55,   49,&
 &   44,   46,   49,   51,   51,   53,   56,   57,   58,   58,   57,   56/)
ZAERV(: ,1971, 9) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58/)
ZAERV(: ,1971,10) = (/&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54/)
ZAERV(: ,1971,11) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1971,12) = (/&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,&
 &   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46,   46/)
ZAERV(: ,1972, 1) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1972, 2) = (/&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,&
 &   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,   39/)
ZAERV(: ,1972, 3) = (/&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,&
 &   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36/)
ZAERV(: ,1972, 4) = (/&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1972, 5) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1972, 6) = (/&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,&
 &   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,   30/)
ZAERV(: ,1972, 7) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1972, 8) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1972, 9) = (/&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1972,10) = (/&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1972,11) = (/&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1972,12) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53/)
ZAERV(: ,1973, 1) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1973, 2) = (/&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61/)
ZAERV(: ,1973, 3) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65/)
ZAERV(: ,1973, 4) = (/&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,&
 &   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,   68/)
ZAERV(: ,1973, 5) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72/)
ZAERV(: ,1973, 6) = (/&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,&
 &   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76,   76/)
ZAERV(: ,1973, 7) = (/&
 &   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,&
 &   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80,   80/)
ZAERV(: ,1973, 8) = (/&
 &   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,&
 &   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84/)
ZAERV(: ,1973, 9) = (/&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,&
 &   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87,   87/)
ZAERV(: ,1973,10) = (/&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,&
 &   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,   91/)
ZAERV(: ,1973,11) = (/&
 &   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,&
 &   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,   95/)
ZAERV(: ,1973,12) = (/&
 &   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,&
 &   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,   99/)
ZAERV(: ,1974, 1) = (/&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103/)
ZAERV(: ,1974, 2) = (/&
 &  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,&
 &  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107,  107/)
ZAERV(: ,1974, 3) = (/&
 &  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,&
 &  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,  110/)
ZAERV(: ,1974, 4) = (/&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,&
 &  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114,  114/)
ZAERV(: ,1974, 5) = (/&
 &  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,&
 &  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118,  118/)
ZAERV(: ,1974, 6) = (/&
 &  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,&
 &  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122,  122/)
ZAERV(: ,1974, 7) = (/&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126/)
ZAERV(: ,1974, 8) = (/&
 &  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,&
 &  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,  130/)
ZAERV(: ,1974, 9) = (/&
 &  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,&
 &  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,  133/)
ZAERV(: ,1974,10) = (/&
 &  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,&
 &  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,  137,  137/)
ZAERV(: ,1974,11) = (/&
 &  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,&
 &  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141,  141/)
ZAERV(: ,1974,12) = (/&
 &  184,  184,  184,  184,  184,  184,  184,  184,  184,  184,  184,  184,&
 &  184,  184,  184,  184,  184,  184,  184,  184,  184,  184,  184,  184/)
ZAERV(: ,1975, 1) = (/&
 &  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,&
 &  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,  228/)
ZAERV(: ,1975, 2) = (/&
 &  271,  271,  271,  271,  271,  271,  271,  271,  271,  271,  271,  271,&
 &  271,  271,  271,  271,  271,  271,  271,  271,  271,  271,  271,  271/)
ZAERV(: ,1975, 3) = (/&
 &  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,&
 &  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,  314/)
ZAERV(: ,1975, 4) = (/&
 &  358,  358,  358,  358,  358,  358,  358,  358,  358,  358,  358,  358,&
 &  358,  358,  358,  358,  358,  358,  358,  358,  358,  358,  358,  358/)
ZAERV(: ,1975, 5) = (/&
 &  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,&
 &  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,  401,  401/)
ZAERV(: ,1975, 6) = (/&
 &  369,  369,  369,  369,  369,  369,  369,  369,  369,  369,  369,  369,&
 &  369,  369,  369,  369,  369,  369,  369,  369,  369,  369,  369,  369/)
ZAERV(: ,1975, 7) = (/&
 &  340,  340,  340,  340,  340,  340,  340,  340,  340,  340,  340,  340,&
 &  340,  340,  340,  340,  340,  340,  340,  340,  340,  340,  340,  340/)
ZAERV(: ,1975, 8) = (/&
 &  313,  313,  313,  313,  313,  313,  313,  313,  313,  313,  313,  313,&
 &  313,  313,  313,  313,  313,  313,  313,  313,  313,  313,  313,  313/)
ZAERV(: ,1975, 9) = (/&
 &  288,  288,  288,  288,  288,  288,  288,  288,  288,  288,  288,  288,&
 &  288,  288,  288,  288,  288,  288,  288,  288,  288,  288,  288,  288/)
ZAERV(: ,1975,10) = (/&
 &  265,  265,  265,  265,  265,  265,  265,  265,  265,  265,  265,  265,&
 &  265,  265,  265,  265,  265,  265,  265,  265,  265,  265,  265,  265/)
ZAERV(: ,1975,11) = (/&
 &  244,  244,  244,  244,  244,  244,  244,  244,  244,  244,  244,  244,&
 &  244,  244,  244,  244,  244,  244,  244,  244,  244,  244,  244,  244/)
ZAERV(: ,1975,12) = (/&
 &  224,  224,  224,  224,  224,  224,  224,  224,  224,  224,  224,  224,&
 &  224,  224,  224,  224,  224,  224,  224,  224,  224,  224,  224,  224/)
ZAERV(: ,1976, 1) = (/&
 &  206,  206,  206,  206,  206,  206,  206,  206,  206,  206,  206,  206,&
 &  206,  206,  206,  206,  206,  206,  206,  206,  206,  206,  206,  206/)
ZAERV(: ,1976, 2) = (/&
 &  190,  190,  190,  190,  190,  190,  190,  190,  190,  190,  190,  190,&
 &  190,  190,  190,  190,  190,  190,  190,  190,  190,  190,  190,  190/)
ZAERV(: ,1976, 3) = (/&
 &  175,  175,  175,  175,  175,  175,  175,  175,  175,  175,  175,  175,&
 &  175,  175,  175,  175,  175,  175,  175,  175,  175,  175,  175,  175/)
ZAERV(: ,1976, 4) = (/&
 &  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,&
 &  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161,  161/)
ZAERV(: ,1976, 5) = (/&
 &  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,&
 &  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148,  148/)
ZAERV(: ,1976, 6) = (/&
 &  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,&
 &  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136,  136/)
ZAERV(: ,1976, 7) = (/&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126/)
ZAERV(: ,1976, 8) = (/&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,&
 &  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116,  116/)
ZAERV(: ,1976, 9) = (/&
 &  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,&
 &  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106,  106/)
ZAERV(: ,1976,10) = (/&
 &   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,&
 &   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98,   98/)
ZAERV(: ,1976,11) = (/&
 &   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,&
 &   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90,   90/)
ZAERV(: ,1976,12) = (/&
 &   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,&
 &   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83/)
ZAERV(: ,1977, 1) = (/&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,&
 &   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77,   77/)
ZAERV(: ,1977, 2) = (/&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,&
 &   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71,   71/)
ZAERV(: ,1977, 3) = (/&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,&
 &   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,   65/)
ZAERV(: ,1977, 4) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60/)
ZAERV(: ,1977, 5) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55/)
ZAERV(: ,1977, 6) = (/&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,&
 &   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,   51/)
ZAERV(: ,1977, 7) = (/&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1977, 8) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1977, 9) = (/&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,&
 &   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,   40/)
ZAERV(: ,1977,10) = (/&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,&
 &   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37,   37/)
ZAERV(: ,1977,11) = (/&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,&
 &   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,   34/)
ZAERV(: ,1977,12) = (/&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,&
 &   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31,   31/)
ZAERV(: ,1978, 1) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1978, 2) = (/&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,&
 &   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27/)
ZAERV(: ,1978, 3) = (/&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,&
 &   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25,   25/)
ZAERV(: ,1978, 4) = (/&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,&
 &   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,   29/)
ZAERV(: ,1978, 5) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48/)
ZAERV(: ,1978, 6) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66/)
ZAERV(: ,1978, 7) = (/&
 &   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,&
 &   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84/)
ZAERV(: ,1978, 8) = (/&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,&
 &  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103,  103/)
ZAERV(: ,1978, 9) = (/&
 &  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,&
 &  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,  121/)
ZAERV(: ,1978,10) = (/&
 &  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,&
 &  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123,  123/)
ZAERV(: ,1978,11) = (/&
 &  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,&
 &  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124,  124/)
ZAERV(: ,1978,12) = (/&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,&
 &  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126,  126/)
ZAERV(: ,1979, 1) = (/&
 &  128,  128,  128,  128,  128,  128,  128,  128,  128,  128,  128,  128,&
 &  128,  128,  128,  128,  128,  128,  128,  128,  128,  128,  128,  128/)
ZAERV(: ,1979, 2) = (/&
 &  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,&
 &  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129,  129/)
ZAERV(: ,1979, 3) = (/&
 &  119,  119,  119,  119,  119,  119,  119,  119,  119,  119,  119,  119,&
 &  119,  119,  119,  119,  119,  119,  119,  119,  119,  119,  119,  119/)
ZAERV(: ,1979, 4) = (/&
 &  109,  109,  109,  109,  109,  109,  109,  109,  109,  109,  109,  109,&
 &  109,  109,  109,  109,  109,  109,  109,  109,  109,  109,  109,  109/)
ZAERV(: ,1979, 5) = (/&
 &  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,&
 &  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101,  101/)
ZAERV(: ,1979, 6) = (/&
 &   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,&
 &   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,   93,   93/)
ZAERV(: ,1979, 7) = (/&
 &   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,&
 &   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,   85/)
ZAERV(: ,1979, 8) = (/&
 &   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,&
 &   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79,   79/)
ZAERV(: ,1979, 9) = (/&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,&
 &   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72,   72/)
ZAERV(: ,1979,10) = (/&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,&
 &   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,   67/)
ZAERV(: ,1979,11) = (/&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,&
 &   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61/)
ZAERV(: ,1979,12) = (/&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,&
 &   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57/)
ZAERV(: ,1980, 1) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1980, 2) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48/)
ZAERV(: ,1980, 3) = (/&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44/)
ZAERV(: ,1980, 4) = (/&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,&
 &   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,   41/)
ZAERV(: ,1980, 5) = (/&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,&
 &   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,   38/)
ZAERV(: ,1980, 6) = (/&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1980, 7) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1980, 8) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32/)
ZAERV(: ,1980, 9) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,&
 &   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,   33/)
ZAERV(: ,1980,10) = (/&
 &   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,&
 &   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,   35/)
ZAERV(: ,1980,11) = (/&
 &   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1980,12) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,   42/)
ZAERV(: ,1981, 1) = (/&
 &   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1981, 2) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1981, 3) = (/&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1981, 4) = (/&
 &   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1981, 5) = (/&
 &   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,&
 &   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,   43/)
ZAERV(: ,1981, 6) = (/&
 &   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,&
 &   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45/)
ZAERV(: ,1981, 7) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1981, 8) = (/&
 &   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,   58,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1981, 9) = (/&
 &   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,&
 &   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,   49/)
ZAERV(: ,1981,10) = (/&
 &   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,   55,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1981,11) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1981,12) = (/&
 &   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,   56,&
 &   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47,   47/)
ZAERV(: ,1982, 1) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1982, 2) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1982, 3) = (/&
 &   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,   62,&
 &   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,   52/)
ZAERV(: ,1982, 4) = (/&
 &  102,  102,  102,  102,  102,   57,   57,   57,  124, 2821,  125,   57,&
 &   48,   48,   48,   48,   48,   48,   48,   67,   67,   67,   67,   67/)
ZAERV(: ,1982, 5) = (/&
 &  186,  186,  186,  186,  186,  225,  195,  147, 1159, 3002, 1272,  103,&
 &   78,   59,   45,   44,   44,   44,   44,   62,   62,   62,   62,   62/)
ZAERV(: ,1982, 6) = (/&
 &  254,  254,  254,  254,  254,  275,  239,  280, 1268, 2759, 1939,  444,&
 &  268,  164,  100,   79,   74,   69,   63,   67,   67,   67,   67,   67/)
ZAERV(: ,1982, 7) = (/&
 &  285,  285,  285,  285,  285,  300,  264,  438, 1185, 2541, 2387, 1286,&
 &  665,  355,  188,  145,  152,  161,  164,   97,   97,   97,   97,   97/)
ZAERV(: ,1982, 8) = (/&
 &  348,  348,  348,  348,  348,  378,  353,  505, 1243, 2340, 2271, 1578,&
 &  830,  469,  261,  198,  210,  228,  222,  144,  144,  144,  144,  144/)
ZAERV(: ,1982, 9) = (/&
 &  388,  388,  388,  388,  388,  488,  486,  558, 1320, 2151, 2090, 1720,&
 &  939,  580,  346,  257,  274,  302,  276,  133,  133,  133,  133,  133/)
ZAERV(: ,1982,10) = (/&
 &  477,  477,  477,  477,  477,  631,  671,  620, 1401, 1979, 1923, 1876,&
 & 1061,  717,  460,  338,  360,  403,  343,  122,  122,  122,  122,  122/)
ZAERV(: ,1982,11) = (/&
 &  690,  690,  691,  691,  690,  884,  902,  775, 1289, 1821, 1769, 1835,&
 & 1189,  827,  532,  344,  403,  474,  431,  196,  196,  196,  196,  196/)
ZAERV(: ,1982,12) = (/&
 & 1086, 1086, 1087, 1086, 1087, 1242, 1127, 1018, 1186, 1676, 1626, 1690,&
 & 1256,  866,  533,  317,  398,  498,  527,  462,  462,  462,  462,  462/)
ZAERV(: ,1983, 1) = (/&
 & 1215, 1215, 1216, 1216, 1215, 1304, 1145, 1020, 1091, 1544, 1497, 1555,&
 & 1155,  798,  495,  311,  450,  520,  546,  551,  550,  551,  551,  550/)
ZAERV(: ,1983, 2) = (/&
 & 1379, 1378, 1379, 1379, 1379, 1317, 1133,  978, 1004, 1419, 1378, 1429,&
 & 1064,  733,  456,  353,  516,  535,  547,  577,  577,  576,  576,  577/)
ZAERV(: ,1983, 3) = (/&
 & 2016, 2016, 2016, 2016, 2016, 1343, 1129,  933,  925, 1304, 1268, 1315,&
 &  979,  674,  419,  401,  590,  547,  542,  530,  531,  530,  530,  530/)
ZAERV(: ,1983, 4) = (/&
 & 1853, 1853, 1855, 1855, 1855, 1366, 1126,  893,  850, 1201, 1166, 1211,&
 &  900,  621,  386,  459,  677,  555,  538,  488,  488,  488,  488,  488/)
ZAERV(: ,1983, 5) = (/&
 & 1707, 1708, 1707, 1707, 1707, 1361, 1095,  847,  782, 1105, 1072, 1113,&
 &  828,  572,  355,  504,  742,  553,  528,  450,  449,  449,  449,  449/)
ZAERV(: ,1983, 6) = (/&
 & 1573, 1573, 1575, 1573, 1576, 1252, 1008,  778,  720, 1017,  987, 1023,&
 &  762,  526,  327,  463,  683,  509,  486,  413,  413,  413,  413,  413/)
ZAERV(: ,1983, 7) = (/&
 & 1448, 1448, 1448, 1447, 1448, 1152,  926,  717,  662,  936,  909,  943,&
 &  702,  484,  301,  426,  628,  468,  447,  380,  381,  380,  380,  380/)
ZAERV(: ,1983, 8) = (/&
 & 1332, 1332, 1333, 1332, 1332, 1059,  853,  660,  609,  862,  837,  867,&
 &  645,  445,  277,  392,  578,  431,  412,  350,  350,  350,  350,  350/)
ZAERV(: ,1983, 9) = (/&
 & 1226, 1226, 1226, 1225, 1226,  976,  785,  607,  561,  792,  769,  799,&
 &  595,  410,  255,  361,  531,  397,  379,  322,  322,  322,  322,  322/)
ZAERV(: ,1983,10) = (/&
 & 1127, 1127, 1128, 1127, 1128,  898,  722,  560,  517,  728,  708,  734,&
 &  546,  377,  234,  332,  489,  366,  349,  296,  296,  296,  296,  296/)
ZAERV(: ,1983,11) = (/&
 & 1038, 1037, 1037, 1038, 1038,  826,  665,  515,  475,  671,  650,  676,&
 &  502,  347,  216,  305,  450,  336,  321,  273,  273,  273,  273,  273/)
ZAERV(: ,1983,12) = (/&
 &  955,  954,  955,  954,  954,  759,  612,  473,  437,  618,  600,  623,&
 &  463,  319,  199,  281,  414,  309,  295,  295,  295,  295,  295,  295/)
ZAERV(: ,1984, 1) = (/&
 &  878,  878,  878,  879,  878,  700,  562,  435,  402,  568,  552,  571,&
 &  426,  294,  183,  259,  381,  284,  272,  311,  311,  311,  311,  311/)
ZAERV(: ,1984, 2) = (/&
 &  808,  808,  809,  808,  809,  643,  518,  401,  370,  522,  509,  526,&
 &  392,  270,  168,  238,  351,  262,  250,  294,  294,  294,  294,  294/)
ZAERV(: ,1984, 3) = (/&
 &  745,  745,  744,  744,  745,  593,  476,  369,  341,  481,  467,  484,&
 &  361,  249,  155,  219,  323,  241,  230,  271,  271,  271,  271,  271/)
ZAERV(: ,1984, 4) = (/&
 &  685,  685,  684,  684,  685,  544,  438,  339,  313,  442,  430,  446,&
 &  332,  229,  143,  202,  297,  222,  212,  249,  249,  249,  249,  249/)
ZAERV(: ,1984, 5) = (/&
 &  631,  631,  630,  631,  630,  502,  403,  312,  288,  407,  395,  410,&
 &  305,  211,  131,  186,  273,  204,  195,  229,  229,  229,  229,  229/)
ZAERV(: ,1984, 6) = (/&
 &  579,  579,  579,  579,  580,  461,  371,  287,  265,  375,  364,  378,&
 &  281,  194,  121,  171,  251,  188,  179,  250,  250,  250,  250,  250/)
ZAERV(: ,1984, 7) = (/&
 &  533,  533,  533,  533,  533,  424,  341,  264,  244,  345,  335,  347,&
 &  259,  179,  111,  157,  231,  173,  165,  277,  277,  277,  277,  277/)
ZAERV(: ,1984, 8) = (/&
 &  491,  491,  491,  491,  491,  390,  314,  243,  225,  317,  308,  320,&
 &  238,  164,  102,  145,  213,  159,  152,  255,  255,  255,  255,  255/)
ZAERV(: ,1984, 9) = (/&
 &  451,  451,  451,  451,  451,  359,  289,  224,  207,  292,  284,  294,&
 &  219,  151,   94,  133,  196,  147,  140,  235,  235,  235,  235,  235/)
ZAERV(: ,1984,10) = (/&
 &  415,  415,  415,  415,  415,  331,  266,  206,  190,  269,  261,  271,&
 &  202,  139,   87,  123,  180,  135,  129,  216,  216,  216,  216,  216/)
ZAERV(: ,1984,11) = (/&
 &  382,  382,  382,  382,  382,  304,  245,  190,  175,  247,  240,  249,&
 &  186,  128,   80,  113,  166,  124,  119,  199,  199,  199,  199,  199/)
ZAERV(: ,1984,12) = (/&
 &  352,  352,  352,  352,  352,  280,  225,  175,  161,  228,  221,  229,&
 &  171,  118,   74,  104,  153,  114,  109,  200,  200,  200,  200,  200/)
ZAERV(: ,1985, 1) = (/&
 &  230,  230,  230,  230,  230,  257,  252,  187,  135,  121,  134,  135,&
 &  141,  146,  129,  105,  113,  146,  183,  214,  214,  214,  214,  214/)
ZAERV(: ,1985, 2) = (/&
 &  238,  238,  238,  238,  238,  269,  263,  179,  128,  116,  120,  118,&
 &  126,  128,  103,  108,  112,  134,  213,  214,  212,  214,  216,  216/)
ZAERV(: ,1985, 3) = (/&
 &  223,  223,  223,  223,  224,  221,  203,  123,   89,   91,  104,  116,&
 &  117,  117,  113,  110,  111,  122,  158,  209,  206,  203,  203,  203/)
ZAERV(: ,1985, 4) = (/&
 &  235,  235,  235,  241,  239,  192,  156,  116,   94,   93,   98,  118,&
 &  124,  116,  106,  108,  112,  135,  166,  193,  193,  193,  193,  193/)
ZAERV(: ,1985, 5) = (/&
 &  211,  211,  211,  211,  211,  156,  139,  109,   89,   85,   99,  110,&
 &  113,  106,   97,  101,  116,  150,  162,  177,  177,  177,  177,  177/)
ZAERV(: ,1985, 6) = (/&
 &  173,  173,  173,  173,  174,  152,  119,   95,   84,   87,  104,  113,&
 &  117,  106,   93,   96,  122,  172,  179,  183,  183,  183,  183,  183/)
ZAERV(: ,1985, 7) = (/&
 &  151,  151,  151,  151,  151,  123,   99,   80,   80,   89,  110,  116,&
 &  123,   99,   80,   87,  121,  162,  177,  206,  206,  206,  206,  206/)
ZAERV(: ,1985, 8) = (/&
 &  153,  153,  153,  153,  153,  120,   98,   83,   79,   87,  104,  114,&
 &  119,   94,   74,   79,  108,  145,  166,  192,  192,  192,  192,  192/)
ZAERV(: ,1985, 9) = (/&
 &  156,  156,  156,  156,  157,  117,   97,   87,   78,   85,   98,  111,&
 &  116,   89,   68,   70,   94,  128,  155,   86,   90,   90,   90,   90/)
ZAERV(: ,1985,10) = (/&
 &  158,  158,  158,  158,  158,  126,   98,   78,   77,   85,   94,  108,&
 &  109,   87,   74,   81,  106,  127,  131,   76,   77,   74,   74,   74/)
ZAERV(: ,1985,11) = (/&
 &  147,  147,  147,  147,  147,  134,  112,   84,   78,   79,   95,  106,&
 &  105,   82,   71,   64,   77,  108,  133,   88,   86,   86,   86,   86/)
ZAERV(: ,1985,12) = (/&
 &  151,  151,  151,  151,  151,  152,  135,  103,   78,   80,  140,  255,&
 &  189,   99,   72,   63,   72,   94,  114,  117,  117,  117,  117,  117/)
ZAERV(: ,1986, 1) = (/&
 &  179,  179,  179,  179,  179,  169,  158,  122,   77,   81,  186,  405,&
 &  272,  117,   72,   61,   67,   80,   95,  121,  121,  121,  121,  121/)
ZAERV(: ,1986, 2) = (/&
 &  225,  225,  225,  225,  225,  185,  183,  139,  105,  112,  240,  390,&
 &  285,  116,   33,   65,   68,   84,   99,  115,  115,  116,  119,  119/)
ZAERV(: ,1986, 3) = (/&
 &  153,  153,  153,  153,  151,  161,  141,   87,   82,   95,  198,  365,&
 &  291,  141,   82,   68,   68,   87,   98,  122,  119,  118,  118,  118/)
ZAERV(: ,1986, 4) = (/&
 &  174,  174,  174,  182,  180,  150,  141,  111,   95,  121,  179,  355,&
 &  274,  139,   82,   69,   70,   90,  108,  117,  117,  117,  117,  117/)
ZAERV(: ,1986, 5) = (/&
 &  173,  173,  173,  173,  177,  131,  114,   95,   92,  111,  187,  302,&
 &  272,  137,   80,   68,   77,   97,  118,  117,  117,  117,  117,  117/)
ZAERV(: ,1986, 6) = (/&
 &  150,  150,  150,  150,  151,  131,  100,   80,   77,   97,  164,  252,&
 &  241,  126,   71,   72,   93,  120,  132,  127,  127,  127,  127,  127/)
ZAERV(: ,1986, 7) = (/&
 &  136,  136,  136,  136,  136,  117,   86,   64,   63,   84,  141,  203,&
 &  209,  139,   88,   79,  108,  123,  135,  202,  202,  202,  202,  202/)
ZAERV(: ,1986, 8) = (/&
 &  142,  142,  142,  141,  140,  102,   65,   72,   79,   96,  118,  189,&
 &  195,  146,  111,   87,  123,  145,  151,  304,  304,  304,  304,  304/)
ZAERV(: ,1986, 9) = (/&
 &  150,  150,  150,  152,  152,  113,   95,   81,   94,  110,  145,  182,&
 &  179,  156,   80,   71,  105,  130,  141,  140,  145,  145,  145,  145/)
ZAERV(: ,1986,10) = (/&
 &  149,  149,  149,  149,  149,  119,  101,   83,   88,  115,  170,  208,&
 &  187,  154,  106,   87,  101,  137,  139,   60,   64,   63,   63,   63/)
ZAERV(: ,1986,11) = (/&
 &  145,  145,  145,  145,  145,  142,  119,   98,   85,  114,  160,  185,&
 &  182,  152,   96,   74,   86,   98,  112,   93,   95,   95,   95,   95/)
ZAERV(: ,1986,12) = (/&
 &  162,  162,  162,  162,  162,  153,  137,  108,   88,  107,  151,  163,&
 &  161,  136,   86,   52,   61,   87,  106,  120,  121,  121,  121,  121/)
ZAERV(: ,1987, 1) = (/&
 &  248,  248,  248,  248,  248,  160,  106,  114,   92,  113,  137,  142,&
 &  139,  120,   75,   29,   36,   77,   94,  119,  119,  119,  119,  119/)
ZAERV(: ,1987, 2) = (/&
 &  188,  188,  188,  188,  188,  158,  156,  120,  105,  105,  120,  122,&
 &  117,   83,   77,   50,   52,   78,  102,  113,  111,  114,  113,  113/)
ZAERV(: ,1987, 3) = (/&
 &  177,  177,  177,  177,  177,  163,  146,   99,   84,   88,  111,  114,&
 &  109,  101,   78,   70,   69,   79,   83,  129,  129,  129,  129,  129/)
ZAERV(: ,1987, 4) = (/&
 &  165,  165,  165,  168,  171,  136,  125,  105,   87,   90,   97,  106,&
 &  106,   98,   77,   71,   70,   84,   94,  116,  116,  116,  116,  116/)
ZAERV(: ,1987, 5) = (/&
 &  159,  159,  159,  159,  153,  149,  119,  101,   93,   91,  103,   96,&
 &   96,   92,   81,   66,   74,   83,   94,  106,  106,  106,  106,  106/)
ZAERV(: ,1987, 6) = (/&
 &  152,  152,  152,  152,  154,  120,  105,   91,   86,   87,   96,   92,&
 &   90,   88,   92,   84,   96,  116,  116,  143,  143,  143,  143,  143/)
ZAERV(: ,1987, 7) = (/&
 &  125,  125,  125,  125,  127,  109,   90,   81,   80,   83,   91,   87,&
 &   84,   88,   84,   89,  103,  115,  119,  231,  231,  231,  231,  231/)
ZAERV(: ,1987, 8) = (/&
 &  133,  133,  130,  130,  131,  103,   92,   82,   81,   84,   81,   86,&
 &   88,   85,   86,   87,  110,  126,  120,  231,  231,  231,  231,  231/)
ZAERV(: ,1987, 9) = (/&
 &  150,  150,  150,  150,  150,  111,   94,   83,   83,   84,   83,   89,&
 &   86,   80,   72,   81,  101,  122,  115,  181,  180,  180,  180,  180/)
ZAERV(: ,1987,10) = (/&
 &  141,  141,  141,  141,  141,  120,  100,   82,   83,   77,   78,   87,&
 &   87,   74,   69,   73,  103,  123,  121,   87,   92,   83,   83,   83/)
ZAERV(: ,1987,11) = (/&
 &  145,  145,  145,  145,  145,  126,  112,   84,   71,   68,   75,   84,&
 &   84,   69,   67,   75,   90,  105,  110,   71,   74,   74,   74,   74/)
ZAERV(: ,1987,12) = (/&
 &  140,  140,  140,  140,  140,  126,  116,   83,   64,   60,   69,   83,&
 &   80,   73,   64,   66,   77,  103,  100,  111,  115,  115,  115,  115/)
ZAERV(: ,1988, 1) = (/&
 &  140,  140,  140,  140,  140,  137,  114,   81,   55,   58,   68,   81,&
 &   76,   76,   62,   57,   64,   75,   88,  115,  116,  116,  116,  116/)
ZAERV(: ,1988, 2) = (/&
 &  178,  178,  178,  178,  178,  136,  124,   89,   53,   48,   67,   77,&
 &   74,   65,   64,   61,   65,   73,   82,  114,  112,  113,  113,  113/)
ZAERV(: ,1988, 3) = (/&
 &  128,  128,  128,  128,  127,  113,   95,   67,   46,   42,   58,   75,&
 &   74,   69,   65,   64,   67,   72,   76,  114,  114,  114,  114,  114/)
ZAERV(: ,1988, 4) = (/&
 &  123,  123,  123,  125,  130,   92,   83,   62,   43,   45,   59,   67,&
 &   69,   55,   61,   59,   63,   74,   89,  109,  109,  109,  109,  109/)
ZAERV(: ,1988, 5) = (/&
 &  104,  104,  104,  104,  103,   85,   75,   55,   42,   45,   56,   67,&
 &   68,   64,   58,   57,   68,   83,   95,   98,   98,   98,   98,   98/)
ZAERV(: ,1988, 6) = (/&
 &   82,   82,   82,   82,   84,   71,   50,   47,   41,   45,   57,   66,&
 &   68,   71,   62,   68,   80,   97,  102,  111,  111,  111,  111,  111/)
ZAERV(: ,1988, 7) = (/&
 &   69,   69,   69,   69,   70,   55,   47,   40,   41,   44,   58,   65,&
 &   68,   66,   59,   59,   80,   99,  105,  244,  244,  244,  244,  244/)
ZAERV(: ,1988, 8) = (/&
 &   78,   78,   75,   73,   74,   63,   51,   43,   42,   46,   54,   66,&
 &   69,   67,   59,   66,   91,  119,  114,  379,  379,  379,  379,  379/)
ZAERV(: ,1988, 9) = (/&
 &   95,   95,   95,   95,   97,   66,   55,   46,   44,   48,   58,   67,&
 &   71,   62,   52,   57,   80,   95,  105,   96,   96,   96,   96,   96/)
ZAERV(: ,1988,10) = (/&
 &   91,   91,   91,   91,   91,   68,   60,   46,   42,   45,   54,   61,&
 &   64,   63,   57,   58,   74,  101,  106,   68,   64,   63,   63,   63/)
ZAERV(: ,1988,11) = (/&
 &   89,   89,   89,   89,   89,   77,   68,   51,   44,   46,   54,   60,&
 &   64,   64,   57,   55,   66,   81,   92,  101,  101,  101,  101,  101/)
ZAERV(: ,1988,12) = (/&
 &   85,   85,   85,   85,   85,   83,   80,   58,   47,   50,   51,   61,&
 &   61,   66,   55,   52,   60,   71,   87,  115,  113,  113,  113,  113/)
ZAERV(: ,1989, 1) = (/&
 &  161,  161,  161,  161,  161,   89,   78,   58,   42,   45,   50,   62,&
 &   59,   68,   52,   50,   54,   64,   77,  101,  101,  101,  101,  101/)
ZAERV(: ,1989, 2) = (/&
 &   84,   84,   84,   84,   84,   92,   84,   62,   44,   43,   50,   54,&
 &   55,   47,   52,   52,   55,   64,   77,   99,   97,   97,   95,   95/)
ZAERV(: ,1989, 3) = (/&
 &   75,   75,   75,   74,   77,   74,   64,   54,   38,   41,   48,   56,&
 &   61,   58,   53,   54,   56,   65,   76,  101,   99,   96,   96,   96/)
ZAERV(: ,1989, 4) = (/&
 &   80,   80,   80,   82,   83,   73,   68,   50,   45,   43,   47,   52,&
 &   55,   52,   49,   48,   50,   62,   72,   94,   94,   94,   94,   94/)
ZAERV(: ,1989, 5) = (/&
 &   68,   68,   68,   68,   67,   62,   58,   43,   39,   38,   45,   50,&
 &    3,   13,    2,   23,   50,   69,   80,  103,  103,  103,  103,  103/)
ZAERV(: ,1989, 6) = (/&
 &   81,   81,   81,   81,   82,   57,   49,   39,   37,   38,   44,   47,&
 &   25,   56,   52,   50,   64,   77,   80,   98,   98,   98,   98,   98/)
ZAERV(: ,1989, 7) = (/&
 &   81,   81,   81,   81,   79,   44,   41,   35,   35,   38,   44,   45,&
 &   47,   51,   50,   53,   66,   81,   87,  204,  204,  204,  204,  204/)
ZAERV(: ,1989, 8) = (/&
 &   70,   70,   68,   66,   66,   43,   43,   37,   39,   43,   38,   41,&
 &   48,   51,   53,   58,   74,   90,   91,  370,  370,  370,  370,  370/)
ZAERV(: ,1989, 9) = (/&
 &   72,   72,   72,   72,   72,   41,   46,   40,   43,   47,   46,   44,&
 &   43,   47,   47,   43,   54,   80,   80,  162,  163,  163,  163,  163/)
ZAERV(: ,1989,10) = (/&
 &   72,   72,   72,   72,   72,   56,   49,   41,   43,   44,   43,   42,&
 &   42,   47,   49,   51,   66,   77,   82,   64,   61,   59,   59,   59/)
ZAERV(: ,1989,11) = (/&
 &   74,   74,   74,   74,   74,   65,   58,   48,   43,   43,   44,   46,&
 &   46,   48,   48,   49,   59,   64,   76,   64,   67,   67,   67,   67/)
ZAERV(: ,1989,12) = (/&
 &   75,   75,   75,   75,   75,   74,   72,   48,   43,   37,   37,   41,&
 &   41,   45,   45,   44,   49,   57,   69,   78,   78,   78,   78,   78/)
ZAERV(: ,1990, 1) = (/&
 &  158,  158,  158,  158,  158,   82,   73,   58,   39,   36,   36,   35,&
 &   37,   42,   42,   40,   41,   46,   60,   75,   78,   78,   78,   78/)
ZAERV(: ,1990, 2) = (/&
 &  314,  314,  314,  314,  314,   77,   70,   57,   42,   34,   32,   34,&
 &   50,   29,   59,   45,   43,   50,   60,   76,   78,   78,   78,   78/)
ZAERV(: ,1990, 3) = (/&
 &   78,   78,   78,   68,   69,   69,   60,   49,   38,   34,   42,   59,&
 &   71,  108,   75,   49,   46,   53,   60,   85,   83,   83,   83,   83/)
ZAERV(: ,1990, 4) = (/&
 &   78,   78,   78,   76,   76,   72,   65,   51,   40,    8,    2,    2,&
 &    2,   62,   79,   57,   26,   29,   50,   74,   74,   74,   74,   74/)
ZAERV(: ,1990, 5) = (/&
 &   75,   75,   75,   75,   73,   56,   51,   40,   37,   38,   40,   40,&
 &    2,   14,   23,   29,   33,   76,   85,   66,   66,   66,   66,   66/)
ZAERV(: ,1990, 6) = (/&
 &   75,   75,   75,   75,   75,   52,   41,   36,   35,   38,   42,   47,&
 &   33,   73,   62,   57,   77,   92,   95,   94,   94,   94,   94,   94/)
ZAERV(: ,1990, 7) = (/&
 &   69,   69,   69,   69,   66,   63,   37,   31,   34,   39,   44,   54,&
 &   65,   65,   56,   63,   83,   96,  106,  210,  210,  210,  210,  210/)
ZAERV(: ,1990, 8) = (/&
 &   64,   64,   64,   63,   62,   61,   41,   35,   36,   41,   48,   57,&
 &   70,   67,   61,   63,   85,  102,  108,  204,  204,  204,  204,  204/)
ZAERV(: ,1990, 9) = (/&
 &   65,   65,   65,   65,   67,   59,   45,   38,   39,   43,   52,   58,&
 &   67,   59,   48,   52,   65,   88,   93,  163,  172,  172,  172,  172/)
ZAERV(: ,1990,10) = (/&
 &   74,   74,   74,   74,   74,   58,   48,   41,   39,   41,   48,   58,&
 &   64,   58,   50,   56,   69,   87,   95,   53,   55,   51,   51,   51/)
ZAERV(: ,1990,11) = (/&
 &   74,   74,   74,   74,   74,   61,   56,   43,   40,   44,   49,   60,&
 &   66,   59,   52,   55,   66,   79,   88,   62,   67,   67,   67,   67/)
ZAERV(: ,1990,12) = (/&
 &   80,   80,   80,   80,   80,   68,   63,   49,   36,   38,   40,   60,&
 &   66,   59,   52,   49,   52,   67,   82,   82,   83,   83,   83,   83/)
ZAERV(: ,1991, 1) = (/&
 &  103,  104,  111,  116,   88,   67,   65,   53,   39,   37,   44,   52,&
 &   54,   51,   45,   48,   50,   57,   72,   99,   92,   84,   90,   90/)
ZAERV(: ,1991, 2) = (/&
 &   86,   87,   84,   76,   70,   69,   66,   55,   40,   37,   44,   50,&
 &   49,   46,   45,   45,   50,   57,   68,   92,  102,   91,   86,   85/)
ZAERV(: ,1991, 3) = (/&
 &   74,   74,   73,   69,   66,   60,   52,   43,   35,   35,   42,   50,&
 &   52,   50,   48,   49,   52,   59,   69,   84,  112,  110,   93,   91/)
ZAERV(: ,1991, 4) = (/&
 &   78,   77,   72,   71,   67,   60,   53,   44,   37,   35,   39,   44,&
 &   46,   45,   44,   44,   48,   56,   66,   79,  106,  106,   91,   89/)
ZAERV(: ,1991, 5) = (/&
 &  119,  115,   64,   60,   58,   53,   46,   39,   34,   34,   41,   48,&
 &   48,   48,   45,   49,   57,   65,   69,   88,  101,  102,   87,   85/)
ZAERV(: ,1991, 6) = (/&
 &  169,  161,   67,   58,   56,   53,   52,   46,   60,  165,  379,  721,&
 &  559,  176,   61,   52,   64,   71,   75,   99,  100,  111,   98,   96/)
ZAERV(: ,1991, 7) = (/&
 &  229,  225,  162,  139,  127,  119,  104,   89,  214,  499,  674, 1203,&
 & 1219,  781,  147,   79,   94,   93,   90,  135,  152,  172,  112,  106/)
ZAERV(: ,1991, 8) = (/&
 &  343,  339,  275,  236,  226,  217,  194,  201,  481,  956, 1084, 1730,&
 & 1939, 1827,  722,  333,  330,  273,  192,  153,  169,  184,  122,  116/)
ZAERV(: ,1991, 9) = (/&
 &  367,  367,  369,  331,  337,  326,  313,  399,  884, 1490, 1635, 2009,&
 & 2264, 2026,  991,  543,  572,  567,  451,  218,  187,  257,  158,  150/)
ZAERV(: ,1991,10) = (/&
 &  548,  560,  748,  532,  516,  531,  553,  614, 1069, 1776, 1948, 2166,&
 & 2293, 2093, 1170,  781,  901, 1020,  872,  610,  547,  443,  207,  190/)
ZAERV(: ,1991,11) = (/&
 &  716,  732, 1010,  694,  687,  770,  806,  778, 1126, 1717, 1960, 2125,&
 & 2302, 2562, 1458,  828, 1021, 1243, 1269, 1240, 1058,  946,  447,  407/)
ZAERV(: ,1991,12) = (/&
 &  833,  854, 1203, 1087, 1241, 1636, 1579, 1358, 1227, 1607, 1613, 1751,&
 & 1978, 2076, 1161,  721,  786,  925, 1174, 1414, 1221, 1211, 1237, 1238/)
ZAERV(: ,1992, 1) = (/&
 &  925,  950, 1360, 1394, 1565, 1734, 1764, 1668, 1331, 1540, 1838, 1874,&
 & 2007, 2068, 1270,  708,  719,  855, 1159, 1424, 1464, 1603, 1669, 1669/)
ZAERV(: ,1992, 2) = (/&
 & 1004, 1032, 1456, 1574, 1680, 1804, 1838, 1791, 1445, 1521, 1845, 1883,&
 & 1864, 1912, 1214,  740,  729,  872, 1163, 1420, 1556, 1562, 1494, 1490/)
ZAERV(: ,1992, 3) = (/&
 & 1063, 1090, 1451, 1527, 1572, 1678, 1696, 1584, 1371, 1487, 1706, 1763,&
 & 1764, 1702, 1225,  827,  759,  887, 1135, 1352, 1603, 1638, 1474, 1460/)
ZAERV(: ,1992, 4) = (/&
 & 1122, 1146, 1470, 1498, 1463, 1482, 1466, 1329, 1286, 1428, 1557, 1638,&
 & 1684, 1673, 1311,  912,  887, 1026, 1244, 1363, 1601, 1711, 1443, 1422/)
ZAERV(: ,1992, 5) = (/&
 & 1181, 1195, 1394, 1424, 1430, 1406, 1366, 1299, 1258, 1435, 1561, 1625,&
 & 1616, 1565, 1215, 1036, 1118, 1240, 1296, 1370, 1474, 1528, 1379, 1367/)
ZAERV(: ,1992, 6) = (/&
 & 1234, 1242, 1336, 1336, 1306, 1238, 1112,  972,  996, 1161, 1295, 1396,&
 & 1415, 1384, 1150, 1082, 1297, 1466, 1438, 1396, 1480, 1455, 1324, 1317/)
ZAERV(: ,1992, 7) = (/&
 & 1271, 1274, 1294, 1301, 1182, 1119,  959,  837,  943, 1176, 1369, 1416,&
 & 1307, 1259, 1101, 1069, 1319, 1533, 1536, 1370, 1428, 1348, 1274, 1271/)
ZAERV(: ,1992, 8) = (/&
 & 1273, 1272, 1253, 1262, 1206, 1071,  958,  828,  913, 1060, 1230, 1096,&
 & 1032, 1099, 1100, 1080, 1370, 1533, 1419, 1264, 1257, 1178, 1213, 1218/)
ZAERV(: ,1992, 9) = (/&
 & 1256, 1255, 1235, 1232, 1169, 1043,  914,  780,  865, 1067, 1180,  970,&
 &  890,  981,  967,  935, 1187, 1340, 1284, 1177, 1040, 1025, 1157, 1168/)
ZAERV(: ,1992,10) = (/&
 & 1194, 1194, 1185, 1162, 1148, 1086,  944,  797,  830,  996, 1014,  890,&
 &  825,  866,  834,  873, 1072, 1209, 1229, 1114, 1020, 1014, 1112, 1120/)
ZAERV(: ,1992,11) = (/&
 & 1119, 1119, 1117, 1121, 1134, 1102,  981,  821,  722,  784,  885,  864,&
 &  787,  823,  798,  804,  936, 1065, 1140, 1138, 1041,  991, 1061, 1068/)
ZAERV(: ,1992,12) = (/&
 & 1045, 1045, 1052, 1057, 1145, 1083,  985,  835,  665,  620,  668,  661,&
 &  648,  665,  581,  577,  687,  868, 1028, 1093, 1001,  996, 1017, 1018/)
ZAERV(: ,1993, 1) = (/&
 &  983,  983,  996,  998, 1167, 1125, 1005,  825,  602,  521,  552,  611,&
 &  628,  600,  535,  534,  578,  752,  916, 1027, 1043, 1015,  997,  996/)
ZAERV(: ,1993, 2) = (/&
 &  906,  907,  928,  927,  986, 1003,  929,  691,  483,  431,  497,  574,&
 &  563,  523,  463,  479,  520,  640,  791,  909, 1007, 1055, 1043, 1042/)
ZAERV(: ,1993, 3) = (/&
 &  828,  832,  884,  889,  903,  888,  799,  612,  428,  389,  449,  518,&
 &  523,  486,  438,  444,  484,  608,  760,  854,  914, 1023, 1055, 1053/)
ZAERV(: ,1993, 4) = (/&
 &  750,  755,  819,  826,  835,  812,  727,  531,  384,  348,  408,  475,&
 &  466,  432,  411,  413,  467,  590,  723,  810,  841,  936, 1008, 1007/)
ZAERV(: ,1993, 5) = (/&
 &  666,  666,  673,  705,  719,  704,  605,  437,  336,  324,  394,  458,&
 &  448,  425,  399,  419,  528,  650,  714,  745,  813,  882,  946,  946/)
ZAERV(: ,1993, 6) = (/&
 &  581,  586,  642,  657,  646,  588,  479,  338,  286,  301,  349,  389,&
 &  403,  410,  357,  390,  527,  675,  730,  753,  782,  828,  884,  885/)
ZAERV(: ,1993, 7) = (/&
 &  510,  511,  522,  514,  507,  471,  352,  276,  272,  291,  333,  370,&
 &  386,  373,  339,  376,  524,  706,  755,  759,  749,  773,  827,  827/)
ZAERV(: ,1993, 8) = (/&
 &  506,  506,  507,  498,  467,  417,  332,  271,  263,  297,  358,  362,&
 &  379,  381,  323,  334,  464,  655,  721,  721,  698,  713,  758,  759/)
ZAERV(: ,1993, 9) = (/&
 &  510,  510,  507,  460,  424,  373,  306,  259,  261,  300,  346,  358,&
 &  349,  324,  297,  320,  422,  556,  640,  661,  660,  668,  681,  680/)
ZAERV(: ,1993,10) = (/&
 &  489,  490,  489,  433,  408,  374,  316,  255,  245,  264,  304,  344,&
 &  345,  323,  293,  311,  410,  522,  600,  615,  619,  626,  596,  594/)
ZAERV(: ,1993,11) = (/&
 &  468,  468,  471,  423,  403,  382,  326,  261,  231,  238,  259,  302,&
 &  343,  320,  285,  296,  374,  451,  538,  540,  533,  537,  540,  540/)
ZAERV(: ,1993,12) = (/&
 &  447,  447,  453,  406,  396,  383,  343,  256,  213,  216,  240,  266,&
 &  274,  262,  237,  233,  264,  350,  456,  504,  538,  539,  513,  511/)
ZAERV(: ,1994, 1) = (/&
 &  423,  423,  432,  385,  385,  385,  342,  255,  194,  191,  217,  243,&
 &  261,  251,  220,  207,  224,  288,  377,  436,  451,  462,  455,  454/)
ZAERV(: ,1994, 2) = (/&
 &  393,  393,  406,  376,  370,  369,  329,  242,  168,  173,  209,  221,&
 &  249,  235,  204,  189,  205,  253,  326,  391,  423,  455,  471,  472/)
ZAERV(: ,1994, 3) = (/&
 &  361,  362,  380,  383,  364,  324,  268,  203,  154,  152,  176,  205,&
 &  216,  207,  183,  167,  181,  227,  287,  344,  382,  450,  476,  477/)
ZAERV(: ,1994, 4) = (/&
 &  331,  331,  342,  351,  333,  304,  259,  196,  150,  146,  161,  179,&
 &  181,  176,  158,  151,  174,  220,  268,  308,  339,  420,  448,  449/)
ZAERV(: ,1994, 5) = (/&
 &  324,  323,  317,  300,  265,  245,  212,  168,  136,  139,  156,  169,&
 &  171,  175,  159,  157,  196,  250,  271,  284,  324,  387,  415,  415/)
ZAERV(: ,1994, 6) = (/&
 &  271,  270,  258,  254,  228,  196,  156,  129,  123,  138,  154,  146,&
 &  146,  160,  145,  143,  187,  254,  281,  282,  300,  355,  384,  384/)
ZAERV(: ,1994, 7) = (/&
 &  206,  206,  209,  207,  197,  169,  127,  110,  117,  138,  160,  151,&
 &  147,  172,  149,  151,  197,  266,  292,  281,  276,  320,  350,  350/)
ZAERV(: ,1994, 8) = (/&
 &  212,  212,  206,  196,  185,  163,  126,  111,  120,  142,  185,  155,&
 &  143,  152,  143,  137,  180,  241,  268,  267,  255,  283,  314,  314/)
ZAERV(: ,1994, 9) = (/&
 &  219,  219,  210,  187,  176,  150,  125,  108,  114,  136,  160,  138,&
 &  134,  132,  126,  132,  175,  224,  251,  250,  240,  257,  274,  274/)
ZAERV(: ,1994,10) = (/&
 &  251,  251,  241,  183,  174,  160,  130,  112,  110,  126,  155,  155,&
 &  155,  147,  126,  135,  186,  240,  257,  247,  239,  239,  232,  232/)
ZAERV(: ,1994,11) = (/&
 &  295,  294,  286,  199,  189,  190,  161,  123,  107,  111,  124,  135,&
 &  150,  141,  119,  119,  153,  199,  233,  238,  239,  229,  220,  220/)
ZAERV(: ,1994,12) = (/&
 &  368,  368,  363,  310,  262,  202,  178,  127,   96,   91,  101,  125,&
 &  149,  133,  115,  108,  128,  165,  212,  234,  242,  247,  268,  269/)
ZAERV(: ,1995, 1) = (/&
 &  162,  162,  165,  171,  158,  160,  147,  121,   74,   73,   85,   87,&
 &   99,  103,   99,   98,   97,  120,  160,  195,  216,  222,  224,  224/)
ZAERV(: ,1995, 2) = (/&
 &  164,  164,  166,  171,  157,  157,  146,  124,   71,   68,   83,   86,&
 &   99,  104,   99,   91,   98,  112,  147,  184,  194,  210,  217,  217/)
ZAERV(: ,1995, 3) = (/&
 &  168,  168,  169,  170,  158,  158,  133,   99,   73,   66,   78,   90,&
 &   94,   94,   91,   86,   86,   96,  126,  155,  183,  208,  219,  219/)
ZAERV(: ,1995, 4) = (/&
 &  172,  172,  170,  167,  158,  151,  119,   87,   59,   58,   69,   80,&
 &   82,   77,   75,   77,   82,   90,  113,  142,  180,  216,  234,  234/)
ZAERV(: ,1995, 5) = (/&
 &  173,  173,  167,  157,  143,  129,   96,   73,   59,   56,   68,   76,&
 &   83,   87,   79,   83,  108,  126,  128,  140,  168,  181,  187,  187/)
ZAERV(: ,1995, 6) = (/&
 &  150,  150,  148,  143,  120,  109,   76,   62,   62,   58,   67,   77,&
 &   82,   75,   64,   70,   92,  128,  147,  138,  163,  178,  184,  184/)
ZAERV(: ,1995, 7) = (/&
 &  124,  124,  126,  133,  112,   95,   64,   59,   58,   60,   62,   75,&
 &   81,   86,   80,   85,  116,  142,  156,  137,  157,  170,  175,  175/)
ZAERV(: ,1995, 8) = (/&
 &  108,  108,  102,   93,   88,   75,   63,   60,   61,   64,   68,   72,&
 &   77,   78,   69,   73,  100,  135,  156,  144,  148,  151,  153,  153/)
ZAERV(: ,1995, 9) = (/&
 &   74,   74,   79,   91,   89,   74,   59,   52,   50,   54,   62,   73,&
 &   72,   67,   57,   59,   85,  129,  150,  156,  162,  258,  314,  314/)
ZAERV(: ,1995,10) = (/&
 &   69,   69,   75,   92,   93,   82,   62,   52,   53,   60,   70,   78,&
 &   80,   74,   70,   72,  104,  129,  145,  150,  146,  215,  253,  253/)
ZAERV(: ,1995,11) = (/&
 &   89,   89,   90,   92,   99,   98,   87,   68,   57,   60,   66,   73,&
 &   78,   73,   72,   75,   92,  113,  134,  157,  131,  131,  121,  121/)
ZAERV(: ,1995,12) = (/&
 &   78,   78,   80,   86,  103,  100,   88,   69,   49,   51,   61,   68,&
 &   70,   67,   64,   64,   68,   87,  116,  135,  121,   92,   82,   82/)
ZAERV(: ,1996, 1) = (/&
 &   72,   72,   75,   82,  105,  110,  101,   79,   52,   51,   55,   63,&
 &   66,   67,   60,   55,   55,   63,   92,  115,  123,  124,  125,  125/)
ZAERV(: ,1996, 2) = (/&
 &   76,   76,   78,   84,  102,  108,   98,   75,   48,   49,   54,   60,&
 &   65,   67,   61,   58,   54,   61,   84,  110,  117,  119,  128,  128/)
ZAERV(: ,1996, 3) = (/&
 &  158,  158,  141,  107,   96,   95,   84,   62,   43,   46,   55,   60,&
 &   61,   60,   54,   51,   52,   60,   73,   93,  104,   99,   97,   97/)
ZAERV(: ,1996, 4) = (/&
 &  144,  144,  137,  121,   99,   92,   74,   59,   46,   46,   50,   52,&
 &   49,   48,   45,   43,   44,   50,   70,   85,   97,   95,   95,   95/)
ZAERV(: ,1996, 5) = (/&
 &  102,  102,  112,  109,   92,   83,   62,   59,   48,   44,   47,   50,&
 &   49,   61,   59,   59,   75,   84,   81,   81,   96,  103,  106,  106/)
ZAERV(: ,1996, 6) = (/&
 &  112,  112,  104,   87,   80,   71,   52,   52,   48,   44,   49,   53,&
 &   48,   49,   46,   46,   68,   84,   88,   78,   75,   74,   74,   74/)
ZAERV(: ,1996, 7) = (/&
 &   83,   83,   80,   73,   72,   62,   46,   40,   49,   49,   57,   58,&
 &   54,   59,   61,   62,   85,  101,   94,   80,   99,  109,  114,  114/)
ZAERV(: ,1996, 8) = (/&
 &   63,   63,   65,   70,   71,   59,   46,   45,   46,   50,   56,   53,&
 &   53,   54,   51,   54,   75,   88,   96,   93,   99,  103,  104,  104/)
ZAERV(: ,1996, 9) = (/&
 &   54,   54,   58,   69,   72,   66,   50,   40,   40,   42,   42,   41,&
 &   43,   45,   47,   50,   69,   91,  109,  114,  101,  101,  100,  100/)
ZAERV(: ,1996,10) = (/&
 &   70,   70,   70,   70,   70,   66,   47,   41,   44,   50,   56,   55,&
 &   57,   60,   58,   55,   67,   90,  101,   99,   99,   94,   92,   92/)
ZAERV(: ,1996,11) = (/&
 &   67,   67,   68,   70,   73,   74,   67,   53,   50,   51,   46,   51,&
 &   59,   61,   58,   53,   59,   89,  100,   97,   87,   90,   90,   90/)
ZAERV(: ,1996,12) = (/&
 &   67,   67,   69,   72,   80,   76,   70,   59,   47,   47,   38,   44,&
 &   52,   53,   52,   49,   50,   72,   92,   92,   88,   99,  104,  104/)
ZAERV(: ,1997, 1) = (/&
 &   69,   69,   71,   76,   87,   88,   81,   75,   49,   42,   38,   46,&
 &   52,   53,   52,   48,   44,   46,   70,   81,   93,  103,  108,  108/)
ZAERV(: ,1997, 2) = (/&
 &   57,   57,   62,   76,   91,   86,   71,   51,   36,   38,   42,   48,&
 &   51,   53,   53,   48,   43,   49,   67,   80,   89,   91,   92,   92/)
ZAERV(: ,1997, 3) = (/&
 &   64,   64,   68,   79,   88,   81,   68,   52,   38,   35,   38,   42,&
 &   46,   45,   44,   42,   42,   50,   61,   74,   84,   78,   75,   75/)
ZAERV(: ,1997, 4) = (/&
 &   74,   74,   77,   79,   83,   82,   70,   55,   37,   38,   41,   42,&
 &   43,   41,   37,   35,   38,   52,   64,   71,   81,   71,   67,   67/)
ZAERV(: ,1997, 5) = (/&
 &   79,   79,   82,   75,   66,   68,   57,   45,   33,   35,   37,   39,&
 &   44,   53,   48,   48,   65,   74,   71,   74,   88,   93,   95,   95/)
ZAERV(: ,1997, 6) = (/&
 &   85,   85,   80,   69,   60,   57,   45,   37,   32,   34,   35,   38,&
 &   40,   37,   34,   35,   53,   66,   70,   80,  102,  109,  112,  112/)
ZAERV(: ,1997, 7) = (/&
 &   66,   66,   65,   61,   53,   50,   44,   38,   37,   41,   44,   46,&
 &   44,   43,   44,   45,   67,   83,   81,   89,   90,   87,   85,   85/)
ZAERV(: ,1997, 8) = (/&
 &   68,   68,   62,   55,   52,   50,   42,   39,   40,   45,   51,   51,&
 &   47,   39,   33,   35,   51,   66,   76,   97,   84,   63,   56,   56/)
ZAERV(: ,1997, 9) = (/&
 &   59,   59,   59,   59,   53,   46,   35,   34,   34,   35,   39,   41,&
 &   41,   48,   47,   47,   75,   93,   95,   98,   83,   66,   60,   60/)
ZAERV(: ,1997,10) = (/&
 &   57,   57,   57,   57,   57,   55,   43,   37,   35,   38,   45,   48,&
 &   48,   43,   39,   41,   57,   69,   77,   78,   80,   77,   80,   80/)
ZAERV(: ,1997,11) = (/&
 &   52,   52,   52,   52,   53,   50,   40,   34,   33,   35,   41,   44,&
 &   44,   40,   36,   38,   53,   63,   71,   72,   74,   71,   73,   73/)
ZAERV(: ,1997,12) = (/&
 &   48,   48,   48,   48,   49,   46,   37,   31,   30,   32,   38,   40,&
 &   41,   37,   34,   35,   49,   58,   65,   66,   68,   65,   68,   68/)
ZAERV(: ,1998, 1) = (/&
 &   44,   44,   44,   44,   45,   43,   34,   29,   28,   30,   35,   37,&
 &   38,   34,   31,   32,   45,   54,   60,   61,   63,   60,   62,   62/)
ZAERV(: ,1998, 2) = (/&
 &   41,   41,   41,   41,   41,   39,   31,   26,   26,   27,   32,   34,&
 &   35,   31,   29,   30,   41,   50,   56,   56,   58,   55,   57,   57/)
ZAERV(: ,1998, 3) = (/&
 &   38,   38,   38,   38,   38,   36,   29,   24,   24,   25,   30,   32,&
 &   32,   29,   26,   27,   38,   46,   51,   52,   53,   51,   53,   53/)
ZAERV(: ,1998, 4) = (/&
 &   35,   35,   35,   35,   35,   34,   27,   23,   22,   23,   28,   29,&
 &   30,   27,   24,   25,   35,   42,   47,   48,   49,   47,   49,   49/)
ZAERV(: ,1998, 5) = (/&
 &   32,   32,   32,   32,   32,   31,   25,   21,   20,   21,   26,   27,&
 &   27,   25,   22,   23,   32,   39,   43,   44,   45,   43,   45,   45/)
ZAERV(: ,1998, 6) = (/&
 &   30,   30,   30,   30,   30,   29,   23,   20,   19,   20,   24,   25,&
 &   25,   23,   21,   22,   30,   36,   40,   40,   42,   40,   41,   41/)
ZAERV(: ,1998, 7) = (/&
 &   27,   27,   27,   27,   28,   26,   23,   20,   19,   18,   22,   23,&
 &   23,   21,   19,   22,   28,   33,   37,   37,   39,   37,   38,   38/)
ZAERV(: ,1998, 8) = (/&
 &   25,   25,   25,   25,   26,   25,   23,   20,   19,   18,   20,   21,&
 &   22,   19,   19,   22,   27,   30,   34,   34,   36,   34,   35,   35/)
ZAERV(: ,1998, 9) = (/&
 &   23,   23,   23,   23,   26,   25,   23,   20,   19,   18,   19,   20,&
 &   20,   18,   19,   22,   27,   29,   31,   32,   33,   31,   32,   33/)
ZAERV(: ,1998,10) = (/&
 &   21,   21,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   18,   17,   19,   22,   27,   29,   29,   29,   30,   29,   30,   33/)
ZAERV(: ,1998,11) = (/&
 &   20,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   28,   27,   28,   33/)
ZAERV(: ,1998,12) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   26,   25,   25,   33/)
ZAERV(: ,1999, 1) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   24,   23,   24,   33/)
ZAERV(: ,1999, 2) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   21,   22,   33/)
ZAERV(: ,1999, 3) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999, 4) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999, 5) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999, 6) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999, 7) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999, 8) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999, 9) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999,10) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999,11) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,1999,12) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)

IF (LHOOK) CALL DR_HOOK('SUECAEC:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3


SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SUECAEC:PART4',0,ZHOOK_HANDLE)

ZAERV(: ,2000, 1) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 2) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 3) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 4) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 5) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 6) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 7) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 8) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000, 9) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000,10) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000,11) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2000,12) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 1) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 2) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 3) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 4) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 5) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 6) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 7) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 8) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001, 9) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001,10) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   27,   29,   27,   28,   23,   20,   21,   33/)
ZAERV(: ,2001,11) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   19,   18,   18,   18,&
 &   17,   17,   19,   22,   24,   27,   27,   26,   22,   19,   20,   33/)
ZAERV(: ,2001,12) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   18,   18,   18,   19,&
 &   19,   18,   19,   21,   24,   25,   27,   28,   27,   27,   27,   33/)
ZAERV(: ,2002, 1) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   18,   17,   17,   18,&
 &   19,   18,   19,   20,   22,   24,   26,   28,   29,   30,   31,   33/)
ZAERV(: ,2002, 2) = (/&
 &   19,   20,   23,   23,   26,   25,   23,   20,   17,   16,   18,   19,&
 &   21,   19,   19,   20,   21,   23,   25,   26,   28,   30,   31,   33/)
ZAERV(: ,2002, 3) = (/&
 &   19,   19,   21,   22,   23,   23,   22,   20,   17,   16,   17,   18,&
 &   21,   19,   19,   19,   21,   22,   23,   24,   25,   26,   30,   32/)
ZAERV(: ,2002, 4) = (/&
 &   21,   21,   22,   22,   22,   22,   21,   19,   17,   16,   17,   19,&
 &   21,   19,   19,   20,   22,   23,   24,   25,   25,   26,   29,   32/)
ZAERV(: ,2002, 5) = (/&
 &   15,   15,   17,   18,   19,   18,   17,   15,   15,   15,   16,   17,&
 &   21,   19,   19,   21,   23,   25,   25,   26,   25,   26,   29,   31/)
ZAERV(: ,2002, 6) = (/&
 &   11,   12,   15,   17,   18,   17,   15,   14,   14,   14,   16,   17,&
 &   21,   19,   20,   21,   24,   26,   26,   27,   25,   26,   29,   30/)
ZAERV(: ,2002, 7) = (/&
 &    1,    1,    2,    3,    4,    4,    4,    6,    8,   13,   15,   17,&
 &   20,   20,   20,   22,   25,   27,   27,   27,   25,   27,   28,   29/)
ZAERV(: ,2002, 8) = (/&
 &   21,   21,   21,   21,   22,   21,   19,   18,   17,   17,   19,   19,&
 &   20,   20,   21,   22,   26,   28,   28,   28,   26,   27,   28,   29/)
ZAERV(: ,2002, 9) = (/&
 &   21,   20,   21,   20,   21,   19,   19,   18,   17,   18,   19,   19,&
 &   20,   20,   21,   23,   27,   29,   29,   29,   26,   27,   27,   28/)
ZAERV(: ,2002,10) = (/&
 &   20,   20,   21,   21,   22,   21,   20,   19,   18,   19,   22,   25,&
 &   23,   22,   22,   22,   25,   27,   27,   27,   25,   24,   22,   24/)
ZAERV(: ,2002,11) = (/&
 &   21,   21,   20,   23,   25,   25,   24,   19,   19,   21,   26,   30,&
 &   27,   24,   23,   23,   24,   26,   26,   25,   25,   26,   26,   27/)
ZAERV(: ,2002,12) = (/&
 &   21,   21,   20,   24,   28,   28,   29,   24,   22,   24,   30,   30,&
 &   29,   26,   24,   24,   25,   26,   26,   26,   25,   26,   26,   30/)
ZAERV(: ,2003, 1) = (/&
 &   22,   22,   20,   26,   30,   32,   33,   29,   26,   27,   29,   30,&
 &   27,   25,   23,   23,   25,   28,   29,   30,   30,   30,   31,   33/)
ZAERV(: ,2003, 2) = (/&
 &   23,   23,   25,   27,   30,   32,   31,   28,   25,   24,   27,   29,&
 &   30,   27,   24,   26,   29,   31,   32,   33,   33,   33,   34,   36/)
ZAERV(: ,2003, 3) = (/&
 &   23,   24,   25,   27,   30,   30,   29,   26,   23,   22,   24,   26,&
 &   26,   25,   23,   23,   26,   28,   28,   30,   32,   34,   36,   39/)
ZAERV(: ,2003, 4) = (/&
 &   27,   27,   28,   29,   30,   29,   28,   25,   24,   23,   24,   26,&
 &   23,   24,   23,   24,   27,   29,   29,   30,   32,   34,   38,   41/)
ZAERV(: ,2003, 5) = (/&
 &   21,   22,   25,   26,   27,   26,   24,   22,   22,   22,   22,   23,&
 &   19,   24,   24,   25,   28,   30,   30,   30,   31,   34,   40,   43/)
ZAERV(: ,2003, 6) = (/&
 &   16,   17,   22,   25,   26,   25,   23,   20,   21,   22,   21,   21,&
 &   20,   24,   24,   26,   29,   31,   31,   31,   30,   35,   42,   46/)
ZAERV(: ,2003, 7) = (/&
 &   25,   25,   26,   27,   27,   26,   23,   22,   23,   23,   23,   22,&
 &   21,   24,   24,   27,   30,   32,   32,   31,   30,   35,   44,   48/)
ZAERV(: ,2003, 8) = (/&
 &   30,   30,   31,   31,   30,   31,   30,   28,   28,   28,   27,   25,&
 &   22,   24,   25,   28,   31,   33,   33,   31,   29,   35,   46,   50/)
ZAERV(: ,2003, 9) = (/&
 &   32,   32,   33,   33,   34,   33,   31,   30,   29,   30,   28,   26,&
 &   23,   24,   25,   29,   32,   34,   34,   32,   29,   36,   48,   53/)
ZAERV(: ,2003,10) = (/&
 &   31,   32,   33,   33,   33,   33,   32,   28,   27,   28,   27,   26,&
 &   22,   23,   25,   28,   32,   33,   34,   31,   27,   26,   27,   33/)
ZAERV(: ,2003,11) = (/&
 &   30,   31,   32,   32,   33,   32,   33,   30,   27,   26,   26,   27,&
 &   25,   24,   24,   27,   30,   31,   33,   33,   30,   27,   26,   33/)
ZAERV(: ,2003,12) = (/&
 &   29,   30,   31,   31,   32,   30,   31,   28,   24,   25,   23,   23,&
 &   26,   24,   23,   24,   27,   29,   31,   32,   32,   32,   33,   33/)
ZAERV(: ,2004, 1) = (/&
 &   28,   29,   30,   31,   31,   28,   29,   26,   21,   21,   22,   23,&
 &   26,   23,   21,   22,   24,   26,   29,   31,   33,   33,   34,   33/)
ZAERV(: ,2004, 2) = (/&
 &   27,   28,   30,   30,   31,   27,   27,   23,   18,   17,   20,   22,&
 &   23,   22,   21,   22,   23,   25,   28,   29,   30,   31,   33,   33/)
ZAERV(: ,2004, 3) = (/&
 &   26,   27,   29,   30,   30,   28,   26,   22,   20,   19,   20,   22,&
 &   24,   21,   21,   22,   26,   28,   26,   30,   30,   31,   34,   34/)
ZAERV(: ,2004, 4) = (/&
 &   25,   26,   27,   28,   29,   27,   25,   22,   20,   18,   19,   22,&
 &   25,   21,   21,   22,   26,   29,   27,   30,   30,   31,   35,   34/)
ZAERV(: ,2004, 5) = (/&
 &   23,   23,   24,   25,   25,   24,   22,   20,   18,   18,   18,   19,&
 &   19,   17,   21,   23,   27,   29,   28,   31,   30,   31,   36,   34/)
ZAERV(: ,2004, 6) = (/&
 &   20,   20,   22,   23,   24,   23,   21,   19,   18,   19,   20,   19,&
 &   18,   19,   21,   23,   27,   29,   29,   31,   30,   32,   36,   34/)
ZAERV(: ,2004, 7) = (/&
 &   24,   23,   23,   24,   25,   23,   20,   19,   19,   19,   21,   20,&
 &   17,   21,   21,   24,   28,   30,   30,   31,   30,   32,   37,   34/)
ZAERV(: ,2004, 8) = (/&
 &   25,   25,   25,   25,   26,   25,   23,   21,   21,   21,   23,   22,&
 &   23,   24,   22,   24,   28,   30,   31,   32,   30,   32,   38,   34/)
ZAERV(: ,2004, 9) = (/&
 &   25,   25,   26,   26,   26,   25,   23,   22,   20,   21,   23,   24,&
 &   25,   23,   22,   24,   29,   31,   32,   32,   30,   32,   39,   35/)
ZAERV(: ,2004,10) = (/&
 &   26,   26,   25,   25,   26,   25,   24,   23,   22,   23,   25,   26,&
 &   26,   25,   22,   24,   27,   29,   30,   29,   28,   26,   27,   26/)
ZAERV(: ,2004,11) = (/&
 &   26,   26,   26,   25,   26,   25,   24,   24,   23,   23,   25,   28,&
 &   26,   25,   23,   23,   26,   28,   28,   28,   26,   25,   25,   28/)
ZAERV(: ,2004,12) = (/&
 &   27,   26,   26,   26,   26,   26,   26,   26,   25,   24,   27,   28,&
 &   32,   26,   26,   24,   25,   27,   29,   29,   30,   29,   31,   30/)
ZAERV(: ,2005, 1) = (/&
 &   28,   26,   27,   26,   26,   28,   29,   27,   26,   25,   30,   38,&
 &   37,   27,   25,   24,   27,   28,   31,   32,   32,   32,   34,   32/)
ZAERV(: ,2005, 2) = (/&
 &   29,   27,   28,   26,   26,   29,   31,   29,   27,   26,   33,   48,&
 &   43,   28,   24,   25,   26,   28,   30,   33,   33,   34,   34,   34/)
ZAERV(: ,2005, 3) = (/&
 &   30,   30,   30,   29,   30,   32,   33,   31,   29,   30,   37,   51,&
 &   49,   36,   27,   26,   29,   31,   33,   33,   31,   35,   35,   33/)
ZAERV(: ,2005, 4) = (/&
 &   37,   36,   34,   33,   33,   34,   34,   32,   31,   30,   39,   51,&
 &   44,   36,   28,   25,   31,   33,   35,   34,   32,   35,   36,   32/)
ZAERV(: ,2005, 5) = (/&
 &   34,   33,   33,   34,   34,   34,   33,   31,   30,   30,   36,   44,&
 &   39,   31,   28,   28,   33,   35,   36,   35,   33,   35,   36,   32/)
ZAERV(: ,2005, 6) = (/&
 &   30,   31,   31,   36,   36,   33,   32,   29,   30,   30,   34,   34,&
 &   35,   28,   29,   32,   35,   37,   38,   36,   33,   35,   37,   31/)
ZAERV(: ,2005, 7) = (/&
 &   36,   36,   36,   36,   37,   34,   32,   30,   29,   29,   31,   32,&
 &   34,   30,   30,   35,   37,   39,   39,   37,   34,   35,   37,   31/)
ZAERV(: ,2005, 8) = (/&
 &   39,   40,   38,   40,   40,   37,   36,   34,   33,   31,   33,   34,&
 &   36,   34,   32,   38,   39,   41,   41,   38,   34,   35,   38,   30/)
ZAERV(: ,2005, 9) = (/&
 &   38,   38,   38,   37,   37,   36,   35,   33,   33,   36,   37,   37,&
 &   37,   38,   36,   37,   41,   42,   42,   39,   35,   35,   39,   30/)
ZAERV(: ,2005,10) = (/&
 &   38,   38,   38,   39,   40,   39,   37,   35,   34,   35,   38,   39,&
 &   39,   39,   33,   35,   39,   41,   41,   36,   30,   24,   24,   29/)
ZAERV(: ,2005,11) = (/&
 &   38,   38,   38,   39,   40,   41,   39,   36,   35,   35,   37,   40,&
 &   41,   38,   34,   34,   38,   39,   39,   36,   32,   31,   30,   28/)
ZAERV(: ,2005,12) = (/&
 &   37,   38,   38,   39,   41,   41,   39,   35,   34,   31,   34,   37,&
 &   39,   37,   31,   31,   34,   37,   40,   38,   36,   35,   34,   30/)
ZAERV(: ,2006, 1) = (/&
 &   37,   37,   38,   39,   41,   40,   39,   34,   32,   28,   31,   33,&
 &   38,   32,   28,   27,   29,   32,   36,   38,   39,   41,   42,   42/)
ZAERV(: ,2006, 2) = (/&
 &   37,   37,   38,   40,   42,   40,   39,   33,   31,   27,   29,   29,&
 &   37,   30,   28,   27,   29,   32,   37,   38,   40,   41,   43,   44/)
ZAERV(: ,2006, 3) = (/&
 &   37,   37,   38,   39,   40,   38,   36,   32,   28,   26,   27,   30,&
 &   36,   30,   27,   26,   28,   31,   34,   36,   37,   40,   45,   47/)
ZAERV(: ,2006, 4) = (/&
 &   39,   39,   41,   42,   42,   39,   36,   32,   28,   26,   27,   31,&
 &   32,   29,   28,   29,   30,   33,   36,   36,   37,   42,   49,   49/)
ZAERV(: ,2006, 5) = (/&
 &   35,   36,   36,   38,   37,   36,   32,   29,   27,   24,   25,   27,&
 &   27,   25,   24,   22,   33,   36,   38,   36,   36,   44,   52,   52/)
ZAERV(: ,2006, 6) = (/&
 &   30,   31,   31,   37,   36,   34,   32,   28,   28,   27,   36,   37,&
 &   39,   27,   25,   26,   35,   38,   40,   36,   36,   47,   55,   55/)
ZAERV(: ,2006, 7) = (/&
 &   36,   35,   33,   37,   38,   35,   33,   30,   33,   38,   48,   52,&
 &   45,   31,   28,   31,   37,   40,   42,   36,   36,   49,   58,   58/)
ZAERV(: ,2006, 8) = (/&
 &   41,   41,   40,   41,   42,   39,   37,   35,   38,   43,   52,   55,&
 &   47,   35,   31,   35,   40,   42,   44,   36,   36,   51,   62,   60/)
ZAERV(: ,2006, 9) = (/&
 &   42,   42,   41,   41,   42,   39,   37,   37,   38,   47,   55,   59,&
 &   51,   41,   32,   35,   41,   44,   44,   38,   36,   53,   65,   63/)
ZAERV(: ,2006,10) = (/&
 &   39,   40,   40,   40,   43,   42,   40,   38,   40,   48,   57,   67,&
 &   57,   43,   32,   36,   40,   43,   42,   38,   31,   29,   33,   44/)
ZAERV(: ,2006,11) = (/&
 &   38,   40,   41,   44,   44,   44,   44,   44,   45,   52,   65,   75,&
 &   74,   53,   43,   41,   45,   45,   43,   38,   30,   24,   25,   26/)
ZAERV(: ,2006,12) = (/&
 &   36,   40,   43,   48,   48,   47,   50,   46,   50,   54,   65,   68,&
 &   68,   61,   51,   52,   54,   56,   52,   48,   40,   37,   38,   31/)
ZAERV(: ,2007, 1) = (/&
 &   35,   41,   45,   51,   53,   51,   55,   47,   47,   49,   58,   61,&
 &   62,   57,   51,   49,   53,   55,   59,   56,   54,   50,   49,   48/)
ZAERV(: ,2007, 2) = (/&
 &   33,   41,   46,   55,   57,   55,   61,   49,   44,   43,   49,   54,&
 &   56,   52,   45,   42,   45,   49,   55,   56,   60,   58,   61,   58/)
ZAERV(: ,2007, 3) = (/&
 &   32,   34,   45,   52,   56,   55,   51,   44,   38,   39,   45,   51,&
 &   50,   46,   39,   37,   39,   42,   47,   49,   49,   50,   51,   57/)
ZAERV(: ,2007, 4) = (/&
 &   46,   47,   52,   55,   56,   57,   54,   45,   38,   36,   42,   49,&
 &   45,   41,   37,   37,   39,   44,   48,   48,   48,   49,   50,   55/)
ZAERV(: ,2007, 5) = (/&
 &   47,   47,   46,   50,   48,   48,   44,   40,   36,   33,   38,   43,&
 &   42,   38,   31,   28,   41,   46,   50,   46,   46,   48,   50,   54/)
ZAERV(: ,2007, 6) = (/&
 &   41,   42,   43,   51,   49,   48,   42,   38,   37,   34,   38,   38,&
 &   38,   37,   32,   31,   42,   49,   51,   45,   45,   47,   49,   52/)
ZAERV(: ,2007, 7) = (/&
 &   47,   47,   46,   50,   50,   48,   41,   37,   38,   34,   35,   36,&
 &   37,   36,   33,   34,   44,   51,   53,   44,   43,   46,   49,   51/)
ZAERV(: ,2007, 8) = (/&
 &   49,   49,   49,   49,   49,   46,   40,   38,   37,   36,   37,   36,&
 &   37,   37,   34,   37,   46,   53,   54,   43,   41,   45,   49,   49/)
ZAERV(: ,2007, 9) = (/&
 &   49,   49,   49,   49,   48,   43,   40,   37,   37,   38,   39,   39,&
 &   38,   40,   39,   42,   49,   53,   54,   47,   37,   42,   48,   48/)
ZAERV(: ,2007,10) = (/&
 &   45,   45,   45,   46,   46,   44,   41,   38,   39,   41,   40,   39,&
 &   38,   40,   38,   41,   47,   50,   50,   42,   34,   26,   26,   25/)
ZAERV(: ,2007,11) = (/&
 &   43,   43,   44,   46,   49,   49,   46,   42,   41,   42,   42,   41,&
 &   38,   40,   38,   41,   44,   45,   49,   44,   34,   26,   26,   26/)
ZAERV(: ,2007,12) = (/&
 &   41,   41,   44,   45,   48,   48,   44,   37,   40,   38,   37,   35,&
 &   37,   38,   38,   38,   39,   41,   46,   45,   41,   35,   32,   30/)
ZAERV(: ,2008, 1) = (/&
 &   39,   39,   43,   44,   47,   46,   42,   35,   34,   32,   34,   36,&
 &   36,   35,   34,   33,   34,   36,   42,   43,   46,   47,   49,   49/)
ZAERV(: ,2008, 2) = (/&
 &   37,   37,   42,   44,   46,   45,   39,   32,   30,   29,   32,   33,&
 &   35,   33,   32,   32,   33,   35,   39,   42,   45,   46,   47,   49/)
ZAERV(: ,2008, 3) = (/&
 &   44,   44,   43,   42,   43,   42,   40,   35,   29,   28,   29,   31,&
 &   32,   31,   29,   30,   31,   32,   34,   36,   39,   39,   47,   49/)
ZAERV(: ,2008, 4) = (/&
 &   46,   46,   46,   44,   42,   40,   37,   32,   29,   26,   28,   33,&
 &   30,   28,   28,   28,   30,   30,   36,   38,   40,   42,   48,   49/)
ZAERV(: ,2008, 5) = (/&
 &   39,   40,   40,   42,   41,   40,   37,   31,   27,   24,   25,   27,&
 &   27,   26,   26,   24,   31,   33,   39,   40,   41,   45,   49,   49/)
ZAERV(: ,2008, 6) = (/&
 &   31,   32,   36,   40,   41,   39,   35,   31,   28,   25,   25,   26,&
 &   26,   25,   21,   25,   33,   37,   41,   41,   43,   48,   50,   50/)
ZAERV(: ,2008, 7) = (/&
 &   34,   35,   40,   40,   41,   37,   35,   32,   29,   26,   27,   27,&
 &   28,   26,   24,   27,   35,   40,   43,   43,   44,   51,   51,   50/)
ZAERV(: ,2008, 8) = (/&
 &   46,   46,   48,   48,   48,   45,   40,   35,   31,   28,   30,   31,&
 &   29,   27,   27,   29,   36,   43,   46,   44,   45,   54,   52,   50/)
ZAERV(: ,2008, 9) = (/&
 &   69,   68,   68,   68,   68,   63,   57,   47,   43,   38,   36,   34,&
 &   32,   30,   30,   33,   39,   43,   46,   44,   41,   48,   53,   50/)
ZAERV(: ,2008,10) = (/&
 &   74,   75,   77,   78,   80,   74,   67,   57,   51,   46,   40,   39,&
 &   35,   33,   30,   33,   38,   41,   42,   40,   34,   29,   29,   29/)
ZAERV(: ,2008,11) = (/&
 &   69,   69,   71,   70,   73,   74,   69,   58,   52,   50,   48,   51,&
 &   41,   37,   32,   34,   37,   38,   40,   40,   33,   28,   28,   27/)
ZAERV(: ,2008,12) = (/&
 &   63,   63,   65,   64,   66,   64,   59,   56,   46,   47,   50,   54,&
 &   48,   41,   33,   33,   34,   37,   39,   39,   38,   32,   30,   35/)
ZAERV(: ,2009, 1) = (/&
 &   58,   58,   59,   58,   60,   54,   49,   48,   37,   39,   44,   45,&
 &   46,   41,   30,   29,   30,   33,   37,   39,   40,   39,   42,   45/)
ZAERV(: ,2009, 2) = (/&
 &   52,   52,   53,   53,   53,   44,   40,   39,   33,   35,   38,   39,&
 &   42,   37,   34,   33,   34,   35,   38,   39,   41,   41,   42,   43/)
ZAERV(: ,2009, 3) = (/&
 &   46,   47,   47,   47,   47,   45,   43,   39,   35,   34,   36,   37,&
 &   42,   41,   37,   37,   40,   42,   41,   41,   38,   37,   35,   45/)
ZAERV(: ,2009, 4) = (/&
 &   47,   47,   48,   47,   47,   45,   43,   37,   33,   30,   33,   35,&
 &   41,   40,   35,   32,   32,   43,   42,   41,   38,   38,   38,   48/)
ZAERV(: ,2009, 5) = (/&
 &   44,   44,   47,   45,   48,   45,   43,   37,   31,   29,   32,   33,&
 &   35,   35,   30,   27,   34,   44,   43,   41,   38,   38,   41,   50/)
ZAERV(: ,2009, 6) = (/&
 &   35,   38,   47,   51,   52,   47,   44,   37,   30,   30,   30,   31,&
 &   31,   30,   25,   29,   37,   46,   44,   41,   39,   38,   44,   52/)
ZAERV(: ,2009, 7) = (/&
 &  135,  134,  131,  121,  114,  104,   75,   53,   51,   44,   39,   37,&
 &   31,   31,   26,   31,   40,   47,   45,   41,   39,   39,   47,   54/)
ZAERV(: ,2009, 8) = (/&
 &  150,  151,  155,  149,  146,  131,   97,   76,   70,   62,   49,   43,&
 &   35,   33,   32,   34,   43,   49,   46,   41,   39,   39,   50,   57/)
ZAERV(: ,2009, 9) = (/&
 &  124,  124,  124,  121,  121,  116,   99,   81,   72,   65,   55,   48,&
 &   38,   37,   36,   40,   47,   49,   48,   41,   34,   45,   55,   59/)
ZAERV(: ,2009,10) = (/&
 &   89,   89,   93,   94,   98,   99,   92,   72,   63,   60,   55,   49,&
 &   38,   36,   37,   40,   45,   47,   46,   40,   31,   25,   25,   25/)
ZAERV(: ,2009,11) = (/&
 &   80,   80,   82,   80,   81,   82,   79,   68,   61,   56,   52,   49,&
 &   40,   38,   37,   39,   43,   44,   44,   43,   35,   28,   25,   23/)
ZAERV(: ,2009,12) = (/&
 &   70,   70,   72,   70,   72,   70,   66,   57,   44,   42,   42,   43,&
 &   39,   38,   35,   34,   35,   39,   43,   45,   43,   37,   36,   39/)
ZAERV(: ,2010, 1) = (/&
 &   61,   61,   62,   61,   63,   58,   53,   47,   37,   35,   35,   36,&
 &   35,   37,   32,   29,   32,   36,   40,   43,   45,   45,   47,   51/)
ZAERV(: ,2010, 2) = (/&
 &   51,   51,   52,   52,   55,   46,   39,   38,   30,   30,   33,   37,&
 &   40,   34,   29,   28,   30,   33,   38,   40,   42,   44,   45,   47/)
ZAERV(: ,2010, 3) = (/&
 &   42,   42,   42,   43,   46,   46,   44,   37,   32,   30,   32,   33,&
 &   29,   28,   27,   27,   29,   30,   31,   32,   33,   35,   35,   45/)
ZAERV(: ,2010, 4) = (/&
 &   45,   45,   45,   47,   47,   46,   43,   36,   32,   29,   32,   35,&
 &   30,   28,   27,   27,   29,   29,   33,   32,   33,   36,   36,   42/)
ZAERV(: ,2010, 5) = (/&
 &   43,   43,   45,   45,   45,   44,   41,   36,   30,   28,   29,   31,&
 &   26,   26,   26,   24,   31,   32,   34,   33,   33,   38,   38,   40/)
ZAERV(: ,2010, 6) = (/&
 &   36,   38,   43,   45,   45,   43,   40,   36,   32,   30,   30,   29,&
 &   26,   25,   24,   26,   33,   35,   36,   33,   32,   39,   40,   37/)
ZAERV(: ,2010, 7) = (/&
 &   43,   43,   43,   44,   44,   42,   38,   35,   33,   31,   30,   31,&
 &   27,   27,   24,   28,   36,   38,   37,   33,   32,   41,   42,   34/)
ZAERV(: ,2010, 8) = (/&
 &   45,   44,   44,   44,   44,   41,   37,   35,   33,   32,   32,   31,&
 &   28,   28,   29,   31,   38,   42,   39,   33,   32,   42,   44,   32/)
ZAERV(: ,2010, 9) = (/&
 &   43,   43,   42,   41,   42,   39,   37,   35,   33,   33,   32,   33,&
 &   30,   29,   29,   34,   39,   42,   42,   38,   36,   42,   46,   29/)
ZAERV(: ,2010,10) = (/&
 &   42,   42,   41,   41,   43,   40,   36,   34,   32,   31,   31,   32,&
 &   30,   30,   30,   34,   39,   40,   39,   34,   29,   26,   25,   27/)
ZAERV(: ,2010,11) = (/&
 &   40,   39,   38,   39,   41,   40,   37,   34,   31,   31,   35,   39,&
 &   46,   42,   34,   36,   37,   38,   38,   36,   30,   23,   24,   23/)
ZAERV(: ,2010,12) = (/&
 &   37,   37,   36,   38,   40,   40,   38,   34,   35,   40,   43,   47,&
 &   58,   59,   56,   54,   50,   47,   44,   41,   36,   30,   29,   30/)
ZAERV(: ,2011, 1) = (/&
 &   34,   34,   34,   36,   39,   39,   38,   35,   35,   40,   42,   45,&
 &   52,   54,   49,   48,   50,   50,   51,   50,   48,   45,   46,   50/)
ZAERV(: ,2011, 2) = (/&
 &   31,   31,   32,   35,   38,   39,   38,   36,   33,   35,   38,   40,&
 &   45,   46,   44,   43,   44,   45,   49,   52,   54,   53,   53,   53/)
ZAERV(: ,2011, 3) = (/&
 &   28,   28,   29,   33,   37,   38,   38,   34,   31,   32,   35,   38,&
 &   40,   39,   37,   37,   39,   40,   42,   44,   46,   46,   51,   53/)
ZAERV(: ,2011, 4) = (/&
 &   37,   37,   36,   34,   34,   34,   34,   31,   29,   29,   33,   38,&
 &   36,   35,   33,   32,   34,   41,   43,   43,   44,   44,   50,   53/)
ZAERV(: ,2011, 5) = (/&
 &   27,   29,   32,   34,   36,   35,   33,   30,   29,   28,   31,   35,&
 &   34,   33,   30,   26,   36,   43,   43,   42,   42,   43,   50,   53/)
ZAERV(: ,2011, 6) = (/&
 &   22,   24,   32,   37,   38,   37,   33,   30,   30,   31,   31,   31,&
 &   32,   30,   29,   30,   38,   44,   44,   41,   40,   42,   49,   53/)
ZAERV(: ,2011, 7) = (/&
 &   49,   49,   58,   62,   70,   77,   64,   72,   89,   76,   60,   47,&
 &   35,   32,   30,   33,   41,   46,   45,   40,   38,   41,   49,   53/)
ZAERV(: ,2011, 8) = (/&
 &   99,   99,  106,  107,  120,  115,  106,   98,  104,   97,   75,   64,&
 &   40,   36,   33,   37,   43,   47,   45,   39,   36,   40,   48,   53/)
ZAERV(: ,2011, 9) = (/&
 &  116,  117,  115,  116,  115,  106,  102,   85,   78,   83,   79,   72,&
 &   48,   42,   40,   41,   46,   49,   46,   38,   34,   39,   47,   53/)
ZAERV(: ,2011,10) = (/&
 &   95,   97,  103,  107,  110,  104,   94,   82,   75,   76,   75,   70,&
 &   51,   42,   40,   43,   48,   48,   46,   39,   28,   29,   33,   36/)
ZAERV(: ,2011,11) = (/&
 &   81,   83,   89,   93,   94,   89,   88,   80,   68,   64,   69,   69,&
 &   50,   44,   41,   42,   45,   47,   45,   40,   33,   24,   24,   25/)
ZAERV(: ,2011,12) = (/&
 &   66,   68,   75,   80,   79,   77,   78,   67,   58,   59,   60,   62,&
 &   51,   47,   41,   40,   43,   46,   46,   45,   36,   27,   22,   18/)

IF (LHOOK) CALL DR_HOOK('SUECAEC:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4

END SUBROUTINE SUECAEC
