!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2000 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_A1B2000* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF



!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB, 0.68053E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.70686E+00_JPRB, 0.70507E+00_JPRB, 0.70349E+00_JPRB, 0.70206E+00_JPRB, 0.70071E+00_JPRB, 0.69970E+00_JPRB, 0.69881E+00_JPRB ,&
&  0.69750E+00_JPRB, 0.69618E+00_JPRB, 0.69488E+00_JPRB, 0.69367E+00_JPRB, 0.69240E+00_JPRB, 0.69103E+00_JPRB, 0.68949E+00_JPRB ,&
&  0.68784E+00_JPRB, 0.68633E+00_JPRB, 0.68504E+00_JPRB, 0.68338E+00_JPRB, 0.68155E+00_JPRB, 0.67949E+00_JPRB, 0.67731E+00_JPRB ,&
&  0.67483E+00_JPRB, 0.67224E+00_JPRB, 0.66945E+00_JPRB, 0.66666E+00_JPRB, 0.66364E+00_JPRB, 0.65987E+00_JPRB, 0.65656E+00_JPRB ,&
&  0.65417E+00_JPRB, 0.65197E+00_JPRB, 0.64992E+00_JPRB, 0.64774E+00_JPRB, 0.64595E+00_JPRB, 0.64483E+00_JPRB, 0.64373E+00_JPRB ,&
&  0.64291E+00_JPRB, 0.64237E+00_JPRB, 0.64276E+00_JPRB, 0.64345E+00_JPRB, 0.64440E+00_JPRB, 0.64622E+00_JPRB, 0.64904E+00_JPRB ,&
&  0.65162E+00_JPRB, 0.65402E+00_JPRB, 0.65680E+00_JPRB, 0.65974E+00_JPRB, 0.66280E+00_JPRB, 0.66583E+00_JPRB, 0.66874E+00_JPRB ,&
&  0.67094E+00_JPRB, 0.67285E+00_JPRB, 0.67482E+00_JPRB, 0.67653E+00_JPRB, 0.67741E+00_JPRB, 0.67821E+00_JPRB, 0.67900E+00_JPRB ,&
&  0.67964E+00_JPRB, 0.68007E+00_JPRB, 0.68077E+00_JPRB, 0.68195E+00_JPRB, 0.68314E+00_JPRB, 0.68439E+00_JPRB, 0.68605E+00_JPRB ,&
&  0.68821E+00_JPRB, 0.68976E+00_JPRB, 0.69163E+00_JPRB, 0.69398E+00_JPRB, 0.69693E+00_JPRB, 0.70040E+00_JPRB, 0.70333E+00_JPRB ,&
&  0.70643E+00_JPRB, 0.70974E+00_JPRB, 0.71319E+00_JPRB, 0.71565E+00_JPRB, 0.71838E+00_JPRB, 0.72097E+00_JPRB, 0.72343E+00_JPRB ,&
&  0.72416E+00_JPRB, 0.72544E+00_JPRB, 0.72720E+00_JPRB, 0.72909E+00_JPRB, 0.72838E+00_JPRB, 0.72787E+00_JPRB, 0.72741E+00_JPRB ,&
&  0.72827E+00_JPRB, 0.72608E+00_JPRB, 0.72482E+00_JPRB, 0.72307E+00_JPRB, 0.72209E+00_JPRB, 0.71977E+00_JPRB, 0.71791E+00_JPRB ,&
&  0.71587E+00_JPRB, 0.71422E+00_JPRB, 0.71226E+00_JPRB, 0.71025E+00_JPRB, 0.70840E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.72588E+00_JPRB, 0.72240E+00_JPRB, 0.71876E+00_JPRB, 0.71512E+00_JPRB, 0.71151E+00_JPRB, 0.70873E+00_JPRB, 0.70596E+00_JPRB ,&
&  0.70299E+00_JPRB, 0.70060E+00_JPRB, 0.69849E+00_JPRB, 0.69671E+00_JPRB, 0.69495E+00_JPRB, 0.69440E+00_JPRB, 0.69341E+00_JPRB ,&
&  0.69187E+00_JPRB, 0.68950E+00_JPRB, 0.68765E+00_JPRB, 0.68466E+00_JPRB, 0.68122E+00_JPRB, 0.67650E+00_JPRB, 0.67208E+00_JPRB ,&
&  0.66690E+00_JPRB, 0.66048E+00_JPRB, 0.65487E+00_JPRB, 0.65088E+00_JPRB, 0.64681E+00_JPRB, 0.64147E+00_JPRB, 0.63768E+00_JPRB ,&
&  0.63551E+00_JPRB, 0.63295E+00_JPRB, 0.63068E+00_JPRB, 0.62859E+00_JPRB, 0.62516E+00_JPRB, 0.62291E+00_JPRB, 0.62080E+00_JPRB ,&
&  0.61762E+00_JPRB, 0.61529E+00_JPRB, 0.61634E+00_JPRB, 0.61785E+00_JPRB, 0.62040E+00_JPRB, 0.62480E+00_JPRB, 0.63309E+00_JPRB ,&
&  0.64363E+00_JPRB, 0.65363E+00_JPRB, 0.66401E+00_JPRB, 0.67359E+00_JPRB, 0.68139E+00_JPRB, 0.68785E+00_JPRB, 0.69340E+00_JPRB ,&
&  0.69722E+00_JPRB, 0.70011E+00_JPRB, 0.70164E+00_JPRB, 0.70172E+00_JPRB, 0.70045E+00_JPRB, 0.69991E+00_JPRB, 0.69982E+00_JPRB ,&
&  0.69993E+00_JPRB, 0.69676E+00_JPRB, 0.69550E+00_JPRB, 0.69601E+00_JPRB, 0.69792E+00_JPRB, 0.70003E+00_JPRB, 0.70401E+00_JPRB ,&
&  0.70957E+00_JPRB, 0.71618E+00_JPRB, 0.72280E+00_JPRB, 0.73149E+00_JPRB, 0.74148E+00_JPRB, 0.75252E+00_JPRB, 0.75969E+00_JPRB ,&
&  0.76859E+00_JPRB, 0.77827E+00_JPRB, 0.78851E+00_JPRB, 0.79130E+00_JPRB, 0.79492E+00_JPRB, 0.79814E+00_JPRB, 0.80228E+00_JPRB ,&
&  0.79952E+00_JPRB, 0.79937E+00_JPRB, 0.79955E+00_JPRB, 0.80210E+00_JPRB, 0.79805E+00_JPRB, 0.79634E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.79590E+00_JPRB, 0.78806E+00_JPRB, 0.78313E+00_JPRB, 0.77666E+00_JPRB, 0.77213E+00_JPRB, 0.76290E+00_JPRB, 0.75536E+00_JPRB ,&
&  0.74865E+00_JPRB, 0.74401E+00_JPRB, 0.73857E+00_JPRB, 0.73382E+00_JPRB, 0.72980E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.74721E+00_JPRB, 0.74028E+00_JPRB, 0.73041E+00_JPRB, 0.71829E+00_JPRB, 0.70707E+00_JPRB, 0.69893E+00_JPRB, 0.69045E+00_JPRB ,&
&  0.68364E+00_JPRB, 0.67834E+00_JPRB, 0.67477E+00_JPRB, 0.67231E+00_JPRB, 0.66992E+00_JPRB, 0.67090E+00_JPRB, 0.67200E+00_JPRB ,&
&  0.67356E+00_JPRB, 0.67387E+00_JPRB, 0.67687E+00_JPRB, 0.67729E+00_JPRB, 0.67600E+00_JPRB, 0.67167E+00_JPRB, 0.66605E+00_JPRB ,&
&  0.65805E+00_JPRB, 0.64852E+00_JPRB, 0.63751E+00_JPRB, 0.62464E+00_JPRB, 0.61272E+00_JPRB, 0.56738E+00_JPRB, 0.55455E+00_JPRB ,&
&  0.54982E+00_JPRB, 0.54391E+00_JPRB, 0.54117E+00_JPRB, 0.54124E+00_JPRB, 0.54780E+00_JPRB, 0.54322E+00_JPRB, 0.53664E+00_JPRB ,&
&  0.50327E+00_JPRB, 0.49629E+00_JPRB, 0.49152E+00_JPRB, 0.48930E+00_JPRB, 0.49514E+00_JPRB, 0.51666E+00_JPRB, 0.54714E+00_JPRB ,&
&  0.60097E+00_JPRB, 0.62465E+00_JPRB, 0.64419E+00_JPRB, 0.66689E+00_JPRB, 0.68580E+00_JPRB, 0.70069E+00_JPRB, 0.71502E+00_JPRB ,&
&  0.72740E+00_JPRB, 0.73969E+00_JPRB, 0.74920E+00_JPRB, 0.75476E+00_JPRB, 0.75653E+00_JPRB, 0.75745E+00_JPRB, 0.75760E+00_JPRB ,&
&  0.75726E+00_JPRB, 0.75578E+00_JPRB, 0.75502E+00_JPRB, 0.75374E+00_JPRB, 0.75126E+00_JPRB, 0.75119E+00_JPRB, 0.75593E+00_JPRB ,&
&  0.76415E+00_JPRB, 0.77686E+00_JPRB, 0.78911E+00_JPRB, 0.80724E+00_JPRB, 0.82500E+00_JPRB, 0.84807E+00_JPRB, 0.85850E+00_JPRB ,&
&  0.87140E+00_JPRB, 0.87998E+00_JPRB, 0.89117E+00_JPRB, 0.89220E+00_JPRB, 0.89339E+00_JPRB, 0.89264E+00_JPRB, 0.89433E+00_JPRB ,&
&  0.88928E+00_JPRB, 0.88580E+00_JPRB, 0.88085E+00_JPRB, 0.88025E+00_JPRB, 0.87337E+00_JPRB, 0.87185E+00_JPRB, 0.86760E+00_JPRB ,&
&  0.86950E+00_JPRB, 0.86002E+00_JPRB, 0.85510E+00_JPRB, 0.84321E+00_JPRB, 0.83284E+00_JPRB, 0.81538E+00_JPRB, 0.79978E+00_JPRB ,&
&  0.78615E+00_JPRB, 0.77596E+00_JPRB, 0.76732E+00_JPRB, 0.76006E+00_JPRB, 0.75318E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.75746E+00_JPRB, 0.73950E+00_JPRB, 0.72028E+00_JPRB, 0.70441E+00_JPRB, 0.68974E+00_JPRB, 0.67906E+00_JPRB, 0.66880E+00_JPRB ,&
&  0.66126E+00_JPRB, 0.65430E+00_JPRB, 0.64956E+00_JPRB, 0.64577E+00_JPRB, 0.64337E+00_JPRB, 0.64287E+00_JPRB, 0.64262E+00_JPRB ,&
&  0.64387E+00_JPRB, 0.64506E+00_JPRB, 0.64784E+00_JPRB, 0.65004E+00_JPRB, 0.65249E+00_JPRB, 0.65061E+00_JPRB, 0.64624E+00_JPRB ,&
&  0.63839E+00_JPRB, 0.62804E+00_JPRB, 0.59049E+00_JPRB, 0.57124E+00_JPRB, 0.53922E+00_JPRB, 0.51483E+00_JPRB, 0.50206E+00_JPRB ,&
&  0.48533E+00_JPRB, 0.47635E+00_JPRB, 0.45941E+00_JPRB, 0.44517E+00_JPRB, 0.43570E+00_JPRB, 0.41979E+00_JPRB, 0.40989E+00_JPRB ,&
&  0.39823E+00_JPRB, 0.38342E+00_JPRB, 0.37681E+00_JPRB, 0.37576E+00_JPRB, 0.38219E+00_JPRB, 0.40147E+00_JPRB, 0.43275E+00_JPRB ,&
&  0.49058E+00_JPRB, 0.55532E+00_JPRB, 0.64429E+00_JPRB, 0.67150E+00_JPRB, 0.67531E+00_JPRB, 0.69648E+00_JPRB, 0.71412E+00_JPRB ,&
&  0.72556E+00_JPRB, 0.75141E+00_JPRB, 0.77835E+00_JPRB, 0.79218E+00_JPRB, 0.80178E+00_JPRB, 0.80666E+00_JPRB, 0.81109E+00_JPRB ,&
&  0.81507E+00_JPRB, 0.82560E+00_JPRB, 0.83104E+00_JPRB, 0.83388E+00_JPRB, 0.83625E+00_JPRB, 0.83852E+00_JPRB, 0.84818E+00_JPRB ,&
&  0.85677E+00_JPRB, 0.88041E+00_JPRB, 0.90201E+00_JPRB, 0.93814E+00_JPRB, 0.95503E+00_JPRB, 0.97598E+00_JPRB, 0.98296E+00_JPRB ,&
&  0.99305E+00_JPRB, 0.99945E+00_JPRB, 0.10038E+01_JPRB, 0.99505E+00_JPRB, 0.99439E+00_JPRB, 0.10025E+01_JPRB, 0.10011E+01_JPRB ,&
&  0.99728E+00_JPRB, 0.99358E+00_JPRB, 0.98877E+00_JPRB, 0.98376E+00_JPRB, 0.97971E+00_JPRB, 0.97984E+00_JPRB, 0.97713E+00_JPRB ,&
&  0.98103E+00_JPRB, 0.97339E+00_JPRB, 0.96585E+00_JPRB, 0.95173E+00_JPRB, 0.93465E+00_JPRB, 0.91061E+00_JPRB, 0.87976E+00_JPRB ,&
&  0.85253E+00_JPRB, 0.83128E+00_JPRB, 0.80944E+00_JPRB, 0.79236E+00_JPRB, 0.77459E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.70698E+00_JPRB, 0.68455E+00_JPRB, 0.66532E+00_JPRB, 0.65105E+00_JPRB, 0.63830E+00_JPRB, 0.62980E+00_JPRB, 0.62207E+00_JPRB ,&
&  0.61799E+00_JPRB, 0.61384E+00_JPRB, 0.61258E+00_JPRB, 0.61120E+00_JPRB, 0.61205E+00_JPRB, 0.61424E+00_JPRB, 0.61790E+00_JPRB ,&
&  0.62283E+00_JPRB, 0.62671E+00_JPRB, 0.63118E+00_JPRB, 0.63174E+00_JPRB, 0.63285E+00_JPRB, 0.63141E+00_JPRB, 0.63040E+00_JPRB ,&
&  0.62193E+00_JPRB, 0.61119E+00_JPRB, 0.58453E+00_JPRB, 0.56395E+00_JPRB, 0.53244E+00_JPRB, 0.51019E+00_JPRB, 0.49303E+00_JPRB ,&
&  0.49039E+00_JPRB, 0.45872E+00_JPRB, 0.46028E+00_JPRB, 0.43698E+00_JPRB, 0.41471E+00_JPRB, 0.39453E+00_JPRB, 0.37427E+00_JPRB ,&
&  0.36212E+00_JPRB, 0.34163E+00_JPRB, 0.33284E+00_JPRB, 0.33227E+00_JPRB, 0.33547E+00_JPRB, 0.35276E+00_JPRB, 0.37926E+00_JPRB ,&
&  0.43734E+00_JPRB, 0.57750E+00_JPRB, 0.62598E+00_JPRB, 0.64742E+00_JPRB, 0.64141E+00_JPRB, 0.64433E+00_JPRB, 0.63898E+00_JPRB ,&
&  0.66668E+00_JPRB, 0.70382E+00_JPRB, 0.74187E+00_JPRB, 0.76857E+00_JPRB, 0.80060E+00_JPRB, 0.82171E+00_JPRB, 0.84078E+00_JPRB ,&
&  0.86158E+00_JPRB, 0.89070E+00_JPRB, 0.91538E+00_JPRB, 0.93527E+00_JPRB, 0.95734E+00_JPRB, 0.97819E+00_JPRB, 0.10049E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.10634E+01_JPRB, 0.10789E+01_JPRB, 0.11007E+01_JPRB, 0.11061E+01_JPRB, 0.11055E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.11270E+01_JPRB, 0.11365E+01_JPRB, 0.11365E+01_JPRB, 0.11366E+01_JPRB, 0.11200E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.11127E+01_JPRB, 0.11092E+01_JPRB, 0.11034E+01_JPRB, 0.10921E+01_JPRB, 0.10773E+01_JPRB, 0.10611E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.10203E+01_JPRB, 0.10006E+01_JPRB, 0.98024E+00_JPRB, 0.95991E+00_JPRB, 0.93817E+00_JPRB, 0.90301E+00_JPRB ,&
&  0.86013E+00_JPRB, 0.82207E+00_JPRB, 0.78724E+00_JPRB, 0.75993E+00_JPRB, 0.73186E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.66482E+00_JPRB, 0.64125E+00_JPRB, 0.62040E+00_JPRB, 0.60722E+00_JPRB, 0.59505E+00_JPRB, 0.58885E+00_JPRB, 0.58222E+00_JPRB ,&
&  0.57888E+00_JPRB, 0.57626E+00_JPRB, 0.57543E+00_JPRB, 0.57545E+00_JPRB, 0.57700E+00_JPRB, 0.57949E+00_JPRB, 0.58377E+00_JPRB ,&
&  0.58948E+00_JPRB, 0.59706E+00_JPRB, 0.60642E+00_JPRB, 0.61163E+00_JPRB, 0.61657E+00_JPRB, 0.61861E+00_JPRB, 0.61968E+00_JPRB ,&
&  0.61392E+00_JPRB, 0.60608E+00_JPRB, 0.58264E+00_JPRB, 0.56135E+00_JPRB, 0.54210E+00_JPRB, 0.52063E+00_JPRB, 0.51649E+00_JPRB ,&
&  0.51296E+00_JPRB, 0.50315E+00_JPRB, 0.48778E+00_JPRB, 0.47893E+00_JPRB, 0.46490E+00_JPRB, 0.40708E+00_JPRB, 0.39730E+00_JPRB ,&
&  0.37296E+00_JPRB, 0.34075E+00_JPRB, 0.32963E+00_JPRB, 0.32287E+00_JPRB, 0.32449E+00_JPRB, 0.33746E+00_JPRB, 0.35983E+00_JPRB ,&
&  0.41421E+00_JPRB, 0.57241E+00_JPRB, 0.61037E+00_JPRB, 0.62244E+00_JPRB, 0.60268E+00_JPRB, 0.60677E+00_JPRB, 0.60969E+00_JPRB ,&
&  0.62961E+00_JPRB, 0.64965E+00_JPRB, 0.67457E+00_JPRB, 0.69667E+00_JPRB, 0.71973E+00_JPRB, 0.73487E+00_JPRB, 0.76218E+00_JPRB ,&
&  0.80894E+00_JPRB, 0.86779E+00_JPRB, 0.91961E+00_JPRB, 0.97595E+00_JPRB, 0.10374E+01_JPRB, 0.11454E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12661E+01_JPRB, 0.12490E+01_JPRB, 0.12445E+01_JPRB, 0.12357E+01_JPRB, 0.12339E+01_JPRB, 0.12312E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.12566E+01_JPRB, 0.12747E+01_JPRB, 0.12532E+01_JPRB, 0.12121E+01_JPRB, 0.11940E+01_JPRB, 0.11617E+01_JPRB, 0.11587E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.11725E+01_JPRB, 0.11630E+01_JPRB, 0.11415E+01_JPRB, 0.11204E+01_JPRB, 0.10970E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.10360E+01_JPRB, 0.10087E+01_JPRB, 0.98617E+00_JPRB, 0.96563E+00_JPRB, 0.94698E+00_JPRB, 0.91933E+00_JPRB ,&
&  0.87225E+00_JPRB, 0.81383E+00_JPRB, 0.76832E+00_JPRB, 0.72654E+00_JPRB, 0.69184E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.63190E+00_JPRB, 0.61258E+00_JPRB, 0.59931E+00_JPRB, 0.59261E+00_JPRB, 0.58670E+00_JPRB, 0.58334E+00_JPRB, 0.57703E+00_JPRB ,&
&  0.57116E+00_JPRB, 0.56570E+00_JPRB, 0.56119E+00_JPRB, 0.55762E+00_JPRB, 0.55580E+00_JPRB, 0.55486E+00_JPRB, 0.55545E+00_JPRB ,&
&  0.55727E+00_JPRB, 0.55381E+00_JPRB, 0.55620E+00_JPRB, 0.56505E+00_JPRB, 0.57779E+00_JPRB, 0.58929E+00_JPRB, 0.60210E+00_JPRB ,&
&  0.60302E+00_JPRB, 0.59532E+00_JPRB, 0.58356E+00_JPRB, 0.56631E+00_JPRB, 0.55000E+00_JPRB, 0.53864E+00_JPRB, 0.52860E+00_JPRB ,&
&  0.51881E+00_JPRB, 0.50774E+00_JPRB, 0.51394E+00_JPRB, 0.51184E+00_JPRB, 0.48916E+00_JPRB, 0.47703E+00_JPRB, 0.42592E+00_JPRB ,&
&  0.38294E+00_JPRB, 0.34342E+00_JPRB, 0.33220E+00_JPRB, 0.32691E+00_JPRB, 0.33349E+00_JPRB, 0.34759E+00_JPRB, 0.38090E+00_JPRB ,&
&  0.44516E+00_JPRB, 0.56999E+00_JPRB, 0.59745E+00_JPRB, 0.60982E+00_JPRB, 0.61578E+00_JPRB, 0.61324E+00_JPRB, 0.61306E+00_JPRB ,&
&  0.62705E+00_JPRB, 0.64189E+00_JPRB, 0.66531E+00_JPRB, 0.69024E+00_JPRB, 0.72090E+00_JPRB, 0.74758E+00_JPRB, 0.76997E+00_JPRB ,&
&  0.82554E+00_JPRB, 0.87004E+00_JPRB, 0.91817E+00_JPRB, 0.10249E+01_JPRB, 0.12069E+01_JPRB, 0.13830E+01_JPRB, 0.14388E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.14015E+01_JPRB, 0.13451E+01_JPRB, 0.13064E+01_JPRB, 0.12893E+01_JPRB, 0.12862E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.13076E+01_JPRB, 0.13389E+01_JPRB, 0.14231E+01_JPRB, 0.13606E+01_JPRB, 0.13227E+01_JPRB, 0.12833E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11726E+01_JPRB, 0.11577E+01_JPRB, 0.11364E+01_JPRB, 0.11159E+01_JPRB, 0.10989E+01_JPRB, 0.10938E+01_JPRB ,&
&  0.10802E+01_JPRB, 0.10580E+01_JPRB, 0.10197E+01_JPRB, 0.99648E+00_JPRB, 0.97388E+00_JPRB, 0.94986E+00_JPRB, 0.92309E+00_JPRB ,&
&  0.87299E+00_JPRB, 0.80647E+00_JPRB, 0.75243E+00_JPRB, 0.70437E+00_JPRB, 0.66313E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.62622E+00_JPRB, 0.61167E+00_JPRB, 0.60716E+00_JPRB, 0.60951E+00_JPRB, 0.61098E+00_JPRB, 0.59879E+00_JPRB, 0.58578E+00_JPRB ,&
&  0.57663E+00_JPRB, 0.57403E+00_JPRB, 0.57111E+00_JPRB, 0.56552E+00_JPRB, 0.56132E+00_JPRB, 0.55748E+00_JPRB, 0.55402E+00_JPRB ,&
&  0.55082E+00_JPRB, 0.54854E+00_JPRB, 0.54736E+00_JPRB, 0.54427E+00_JPRB, 0.54614E+00_JPRB, 0.55612E+00_JPRB, 0.57285E+00_JPRB ,&
&  0.58938E+00_JPRB, 0.59394E+00_JPRB, 0.59094E+00_JPRB, 0.58040E+00_JPRB, 0.56297E+00_JPRB, 0.54768E+00_JPRB, 0.53761E+00_JPRB ,&
&  0.52334E+00_JPRB, 0.51341E+00_JPRB, 0.51284E+00_JPRB, 0.52422E+00_JPRB, 0.53396E+00_JPRB, 0.51018E+00_JPRB, 0.45322E+00_JPRB ,&
&  0.40430E+00_JPRB, 0.36732E+00_JPRB, 0.34941E+00_JPRB, 0.34536E+00_JPRB, 0.35676E+00_JPRB, 0.38214E+00_JPRB, 0.42117E+00_JPRB ,&
&  0.48806E+00_JPRB, 0.54236E+00_JPRB, 0.56460E+00_JPRB, 0.58576E+00_JPRB, 0.58750E+00_JPRB, 0.58267E+00_JPRB, 0.57950E+00_JPRB ,&
&  0.58559E+00_JPRB, 0.59892E+00_JPRB, 0.61873E+00_JPRB, 0.63964E+00_JPRB, 0.67167E+00_JPRB, 0.72094E+00_JPRB, 0.81889E+00_JPRB ,&
&  0.10461E+01_JPRB, 0.10929E+01_JPRB, 0.10247E+01_JPRB, 0.11534E+01_JPRB, 0.13438E+01_JPRB, 0.14927E+01_JPRB, 0.15838E+01_JPRB ,&
&  0.15730E+01_JPRB, 0.14853E+01_JPRB, 0.14155E+01_JPRB, 0.13719E+01_JPRB, 0.13453E+01_JPRB, 0.13300E+01_JPRB, 0.13311E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.13791E+01_JPRB, 0.14734E+01_JPRB, 0.12911E+01_JPRB, 0.12096E+01_JPRB, 0.11901E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.11108E+01_JPRB, 0.11152E+01_JPRB, 0.11298E+01_JPRB, 0.10781E+01_JPRB, 0.10307E+01_JPRB, 0.99477E+00_JPRB ,&
&  0.98899E+00_JPRB, 0.97533E+00_JPRB, 0.96402E+00_JPRB, 0.96805E+00_JPRB, 0.95814E+00_JPRB, 0.91858E+00_JPRB, 0.89255E+00_JPRB ,&
&  0.85637E+00_JPRB, 0.79538E+00_JPRB, 0.75128E+00_JPRB, 0.70797E+00_JPRB, 0.66050E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.60657E+00_JPRB, 0.61676E+00_JPRB, 0.63358E+00_JPRB, 0.64106E+00_JPRB, 0.64069E+00_JPRB, 0.60673E+00_JPRB, 0.57299E+00_JPRB ,&
&  0.55305E+00_JPRB, 0.54618E+00_JPRB, 0.54794E+00_JPRB, 0.54943E+00_JPRB, 0.54978E+00_JPRB, 0.55089E+00_JPRB, 0.55088E+00_JPRB ,&
&  0.54542E+00_JPRB, 0.53737E+00_JPRB, 0.53070E+00_JPRB, 0.52656E+00_JPRB, 0.52668E+00_JPRB, 0.53332E+00_JPRB, 0.54571E+00_JPRB ,&
&  0.56262E+00_JPRB, 0.57497E+00_JPRB, 0.57960E+00_JPRB, 0.57581E+00_JPRB, 0.56995E+00_JPRB, 0.56426E+00_JPRB, 0.55085E+00_JPRB ,&
&  0.53050E+00_JPRB, 0.52684E+00_JPRB, 0.52907E+00_JPRB, 0.53833E+00_JPRB, 0.55473E+00_JPRB, 0.54195E+00_JPRB, 0.48278E+00_JPRB ,&
&  0.42658E+00_JPRB, 0.39085E+00_JPRB, 0.37759E+00_JPRB, 0.38647E+00_JPRB, 0.41603E+00_JPRB, 0.46154E+00_JPRB, 0.48016E+00_JPRB ,&
&  0.49791E+00_JPRB, 0.51457E+00_JPRB, 0.52715E+00_JPRB, 0.53894E+00_JPRB, 0.54761E+00_JPRB, 0.55185E+00_JPRB, 0.55345E+00_JPRB ,&
&  0.55411E+00_JPRB, 0.55813E+00_JPRB, 0.56814E+00_JPRB, 0.56534E+00_JPRB, 0.59710E+00_JPRB, 0.73734E+00_JPRB, 0.89308E+00_JPRB ,&
&  0.10434E+01_JPRB, 0.11000E+01_JPRB, 0.11742E+01_JPRB, 0.13302E+01_JPRB, 0.14461E+01_JPRB, 0.15258E+01_JPRB, 0.15683E+01_JPRB ,&
&  0.15637E+01_JPRB, 0.14774E+01_JPRB, 0.14043E+01_JPRB, 0.13816E+01_JPRB, 0.13760E+01_JPRB, 0.13743E+01_JPRB, 0.13317E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.12435E+01_JPRB, 0.11542E+01_JPRB, 0.10717E+01_JPRB, 0.10570E+01_JPRB, 0.10874E+01_JPRB, 0.11383E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.11401E+01_JPRB, 0.11363E+01_JPRB, 0.11567E+01_JPRB, 0.10409E+01_JPRB, 0.96462E+00_JPRB, 0.91160E+00_JPRB ,&
&  0.89988E+00_JPRB, 0.89600E+00_JPRB, 0.88596E+00_JPRB, 0.91146E+00_JPRB, 0.90916E+00_JPRB, 0.88738E+00_JPRB, 0.83679E+00_JPRB ,&
&  0.76934E+00_JPRB, 0.71379E+00_JPRB, 0.67293E+00_JPRB, 0.63820E+00_JPRB, 0.61884E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.68093E+00_JPRB, 0.68269E+00_JPRB, 0.67907E+00_JPRB, 0.67637E+00_JPRB, 0.66295E+00_JPRB, 0.64412E+00_JPRB, 0.62497E+00_JPRB ,&
&  0.61661E+00_JPRB, 0.60142E+00_JPRB, 0.58501E+00_JPRB, 0.56391E+00_JPRB, 0.54674E+00_JPRB, 0.53224E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.53544E+00_JPRB, 0.54018E+00_JPRB, 0.53755E+00_JPRB, 0.52867E+00_JPRB, 0.52202E+00_JPRB, 0.52270E+00_JPRB, 0.53145E+00_JPRB ,&
&  0.54549E+00_JPRB, 0.56213E+00_JPRB, 0.57284E+00_JPRB, 0.57638E+00_JPRB, 0.57631E+00_JPRB, 0.57452E+00_JPRB, 0.56802E+00_JPRB ,&
&  0.55297E+00_JPRB, 0.54682E+00_JPRB, 0.55067E+00_JPRB, 0.56133E+00_JPRB, 0.58112E+00_JPRB, 0.56948E+00_JPRB, 0.51299E+00_JPRB ,&
&  0.44829E+00_JPRB, 0.42678E+00_JPRB, 0.43447E+00_JPRB, 0.48178E+00_JPRB, 0.50359E+00_JPRB, 0.52050E+00_JPRB, 0.53273E+00_JPRB ,&
&  0.52764E+00_JPRB, 0.51838E+00_JPRB, 0.52333E+00_JPRB, 0.54426E+00_JPRB, 0.54806E+00_JPRB, 0.54976E+00_JPRB, 0.54853E+00_JPRB ,&
&  0.54684E+00_JPRB, 0.55179E+00_JPRB, 0.56182E+00_JPRB, 0.56312E+00_JPRB, 0.68672E+00_JPRB, 0.91809E+00_JPRB, 0.10208E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.12603E+01_JPRB, 0.14166E+01_JPRB, 0.14623E+01_JPRB, 0.14492E+01_JPRB, 0.14744E+01_JPRB, 0.15520E+01_JPRB ,&
&  0.15440E+01_JPRB, 0.15111E+01_JPRB, 0.14702E+01_JPRB, 0.14657E+01_JPRB, 0.14175E+01_JPRB, 0.13608E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12722E+01_JPRB, 0.12105E+01_JPRB, 0.11329E+01_JPRB, 0.11040E+01_JPRB, 0.10962E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.11410E+01_JPRB, 0.11495E+01_JPRB, 0.11653E+01_JPRB, 0.10680E+01_JPRB, 0.94605E+00_JPRB, 0.86879E+00_JPRB ,&
&  0.84097E+00_JPRB, 0.82578E+00_JPRB, 0.79783E+00_JPRB, 0.81561E+00_JPRB, 0.83363E+00_JPRB, 0.82292E+00_JPRB, 0.76000E+00_JPRB ,&
&  0.70484E+00_JPRB, 0.67526E+00_JPRB, 0.65861E+00_JPRB, 0.66955E+00_JPRB, 0.67970E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.72594E+00_JPRB, 0.72998E+00_JPRB, 0.71918E+00_JPRB, 0.70722E+00_JPRB, 0.70192E+00_JPRB, 0.68510E+00_JPRB, 0.65324E+00_JPRB ,&
&  0.63449E+00_JPRB, 0.60229E+00_JPRB, 0.58167E+00_JPRB, 0.56865E+00_JPRB, 0.56054E+00_JPRB, 0.54796E+00_JPRB, 0.52618E+00_JPRB ,&
&  0.51895E+00_JPRB, 0.52477E+00_JPRB, 0.53776E+00_JPRB, 0.53978E+00_JPRB, 0.53653E+00_JPRB, 0.53302E+00_JPRB, 0.53448E+00_JPRB ,&
&  0.54300E+00_JPRB, 0.55815E+00_JPRB, 0.57532E+00_JPRB, 0.58180E+00_JPRB, 0.57931E+00_JPRB, 0.57653E+00_JPRB, 0.57811E+00_JPRB ,&
&  0.57240E+00_JPRB, 0.56820E+00_JPRB, 0.57634E+00_JPRB, 0.58292E+00_JPRB, 0.60600E+00_JPRB, 0.62005E+00_JPRB, 0.60042E+00_JPRB ,&
&  0.50973E+00_JPRB, 0.50588E+00_JPRB, 0.56552E+00_JPRB, 0.58319E+00_JPRB, 0.59692E+00_JPRB, 0.59191E+00_JPRB, 0.58761E+00_JPRB ,&
&  0.58415E+00_JPRB, 0.58314E+00_JPRB, 0.58033E+00_JPRB, 0.57968E+00_JPRB, 0.57840E+00_JPRB, 0.57781E+00_JPRB, 0.57671E+00_JPRB ,&
&  0.58338E+00_JPRB, 0.55101E+00_JPRB, 0.55796E+00_JPRB, 0.60139E+00_JPRB, 0.80877E+00_JPRB, 0.94749E+00_JPRB, 0.10572E+01_JPRB ,&
&  0.12025E+01_JPRB, 0.13112E+01_JPRB, 0.14446E+01_JPRB, 0.15300E+01_JPRB, 0.15452E+01_JPRB, 0.15466E+01_JPRB, 0.16117E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.16284E+01_JPRB, 0.16055E+01_JPRB, 0.15564E+01_JPRB, 0.14569E+01_JPRB, 0.14172E+01_JPRB, 0.13960E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.14194E+01_JPRB, 0.13552E+01_JPRB, 0.12734E+01_JPRB, 0.12133E+01_JPRB, 0.12152E+01_JPRB, 0.12287E+01_JPRB ,&
&  0.12288E+01_JPRB, 0.12443E+01_JPRB, 0.12290E+01_JPRB, 0.11972E+01_JPRB, 0.11236E+01_JPRB, 0.10550E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.96940E+00_JPRB, 0.89840E+00_JPRB, 0.80956E+00_JPRB, 0.79465E+00_JPRB, 0.80397E+00_JPRB, 0.78404E+00_JPRB, 0.74199E+00_JPRB ,&
&  0.72176E+00_JPRB, 0.70504E+00_JPRB, 0.69015E+00_JPRB, 0.70363E+00_JPRB, 0.71240E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.72759E+00_JPRB, 0.73089E+00_JPRB, 0.71997E+00_JPRB, 0.70769E+00_JPRB, 0.71408E+00_JPRB, 0.69976E+00_JPRB, 0.69553E+00_JPRB ,&
&  0.70206E+00_JPRB, 0.68760E+00_JPRB, 0.70811E+00_JPRB, 0.62817E+00_JPRB, 0.57396E+00_JPRB, 0.55691E+00_JPRB, 0.53591E+00_JPRB ,&
&  0.53072E+00_JPRB, 0.54096E+00_JPRB, 0.54890E+00_JPRB, 0.55574E+00_JPRB, 0.56013E+00_JPRB, 0.54837E+00_JPRB, 0.54363E+00_JPRB ,&
&  0.54837E+00_JPRB, 0.56198E+00_JPRB, 0.58229E+00_JPRB, 0.58919E+00_JPRB, 0.59127E+00_JPRB, 0.59130E+00_JPRB, 0.59154E+00_JPRB ,&
&  0.58175E+00_JPRB, 0.59529E+00_JPRB, 0.62410E+00_JPRB, 0.61705E+00_JPRB, 0.62852E+00_JPRB, 0.66350E+00_JPRB, 0.66497E+00_JPRB ,&
&  0.68184E+00_JPRB, 0.63730E+00_JPRB, 0.66956E+00_JPRB, 0.68083E+00_JPRB, 0.67886E+00_JPRB, 0.66148E+00_JPRB, 0.64428E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63785E+00_JPRB, 0.64223E+00_JPRB, 0.64776E+00_JPRB, 0.64652E+00_JPRB, 0.65941E+00_JPRB, 0.67525E+00_JPRB ,&
&  0.69941E+00_JPRB, 0.64982E+00_JPRB, 0.75470E+00_JPRB, 0.81208E+00_JPRB, 0.95855E+00_JPRB, 0.11323E+01_JPRB, 0.12378E+01_JPRB ,&
&  0.15183E+01_JPRB, 0.15154E+01_JPRB, 0.15993E+01_JPRB, 0.16222E+01_JPRB, 0.16405E+01_JPRB, 0.16875E+01_JPRB, 0.17595E+01_JPRB ,&
&  0.18691E+01_JPRB, 0.17655E+01_JPRB, 0.18122E+01_JPRB, 0.16863E+01_JPRB, 0.15959E+01_JPRB, 0.15876E+01_JPRB, 0.15790E+01_JPRB ,&
&  0.15950E+01_JPRB, 0.16029E+01_JPRB, 0.15795E+01_JPRB, 0.14064E+01_JPRB, 0.13732E+01_JPRB, 0.13659E+01_JPRB, 0.12850E+01_JPRB ,&
&  0.13100E+01_JPRB, 0.12981E+01_JPRB, 0.12266E+01_JPRB, 0.12895E+01_JPRB, 0.11653E+01_JPRB, 0.10895E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.11154E+01_JPRB, 0.97519E+00_JPRB, 0.90402E+00_JPRB, 0.85907E+00_JPRB, 0.81432E+00_JPRB, 0.77190E+00_JPRB, 0.77600E+00_JPRB ,&
&  0.73265E+00_JPRB, 0.73040E+00_JPRB, 0.75007E+00_JPRB, 0.73719E+00_JPRB, 0.72475E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.73515E+00_JPRB, 0.72452E+00_JPRB, 0.71267E+00_JPRB, 0.70497E+00_JPRB, 0.72814E+00_JPRB, 0.73824E+00_JPRB, 0.73746E+00_JPRB ,&
&  0.75553E+00_JPRB, 0.74738E+00_JPRB, 0.74267E+00_JPRB, 0.72491E+00_JPRB, 0.70557E+00_JPRB, 0.65020E+00_JPRB, 0.56136E+00_JPRB ,&
&  0.53549E+00_JPRB, 0.54844E+00_JPRB, 0.57742E+00_JPRB, 0.59190E+00_JPRB, 0.58199E+00_JPRB, 0.56479E+00_JPRB, 0.55651E+00_JPRB ,&
&  0.56271E+00_JPRB, 0.57066E+00_JPRB, 0.58769E+00_JPRB, 0.60120E+00_JPRB, 0.60749E+00_JPRB, 0.61311E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.61815E+00_JPRB, 0.64504E+00_JPRB, 0.69062E+00_JPRB, 0.64843E+00_JPRB, 0.69974E+00_JPRB, 0.76232E+00_JPRB, 0.77904E+00_JPRB ,&
&  0.81719E+00_JPRB, 0.81848E+00_JPRB, 0.79392E+00_JPRB, 0.78162E+00_JPRB, 0.74956E+00_JPRB, 0.72077E+00_JPRB, 0.70846E+00_JPRB ,&
&  0.70742E+00_JPRB, 0.70961E+00_JPRB, 0.71433E+00_JPRB, 0.71878E+00_JPRB, 0.72580E+00_JPRB, 0.78965E+00_JPRB, 0.87122E+00_JPRB ,&
&  0.92112E+00_JPRB, 0.99451E+00_JPRB, 0.11250E+01_JPRB, 0.11402E+01_JPRB, 0.12400E+01_JPRB, 0.13581E+01_JPRB, 0.13942E+01_JPRB ,&
&  0.15472E+01_JPRB, 0.15528E+01_JPRB, 0.17383E+01_JPRB, 0.18769E+01_JPRB, 0.20037E+01_JPRB, 0.20686E+01_JPRB, 0.21250E+01_JPRB ,&
&  0.22704E+01_JPRB, 0.23412E+01_JPRB, 0.21147E+01_JPRB, 0.19781E+01_JPRB, 0.18604E+01_JPRB, 0.18458E+01_JPRB, 0.17793E+01_JPRB ,&
&  0.17890E+01_JPRB, 0.18342E+01_JPRB, 0.18217E+01_JPRB, 0.17798E+01_JPRB, 0.16853E+01_JPRB, 0.15647E+01_JPRB, 0.14473E+01_JPRB ,&
&  0.13804E+01_JPRB, 0.12414E+01_JPRB, 0.11676E+01_JPRB, 0.11897E+01_JPRB, 0.11928E+01_JPRB, 0.10626E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.11865E+01_JPRB, 0.10939E+01_JPRB, 0.10076E+01_JPRB, 0.91156E+00_JPRB, 0.86839E+00_JPRB, 0.83079E+00_JPRB, 0.76541E+00_JPRB ,&
&  0.71305E+00_JPRB, 0.75500E+00_JPRB, 0.76363E+00_JPRB, 0.75064E+00_JPRB, 0.74016E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.74704E+00_JPRB, 0.72573E+00_JPRB, 0.70577E+00_JPRB, 0.70675E+00_JPRB, 0.73465E+00_JPRB, 0.74767E+00_JPRB, 0.76773E+00_JPRB ,&
&  0.77882E+00_JPRB, 0.78170E+00_JPRB, 0.77236E+00_JPRB, 0.73912E+00_JPRB, 0.72722E+00_JPRB, 0.73541E+00_JPRB, 0.70664E+00_JPRB ,&
&  0.64045E+00_JPRB, 0.64185E+00_JPRB, 0.62861E+00_JPRB, 0.63672E+00_JPRB, 0.64067E+00_JPRB, 0.61495E+00_JPRB, 0.60146E+00_JPRB ,&
&  0.61358E+00_JPRB, 0.60429E+00_JPRB, 0.60422E+00_JPRB, 0.61060E+00_JPRB, 0.62181E+00_JPRB, 0.64010E+00_JPRB, 0.66040E+00_JPRB ,&
&  0.66423E+00_JPRB, 0.72113E+00_JPRB, 0.78067E+00_JPRB, 0.75996E+00_JPRB, 0.84689E+00_JPRB, 0.94709E+00_JPRB, 0.99013E+00_JPRB ,&
&  0.10070E+01_JPRB, 0.96571E+00_JPRB, 0.90835E+00_JPRB, 0.86029E+00_JPRB, 0.81472E+00_JPRB, 0.79523E+00_JPRB, 0.79260E+00_JPRB ,&
&  0.79526E+00_JPRB, 0.79496E+00_JPRB, 0.79414E+00_JPRB, 0.80528E+00_JPRB, 0.81842E+00_JPRB, 0.95958E+00_JPRB, 0.11532E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.14594E+01_JPRB, 0.15921E+01_JPRB, 0.16790E+01_JPRB, 0.17673E+01_JPRB, 0.17879E+01_JPRB, 0.17213E+01_JPRB ,&
&  0.17878E+01_JPRB, 0.17548E+01_JPRB, 0.22232E+01_JPRB, 0.23079E+01_JPRB, 0.23252E+01_JPRB, 0.24365E+01_JPRB, 0.24100E+01_JPRB ,&
&  0.24101E+01_JPRB, 0.22697E+01_JPRB, 0.21970E+01_JPRB, 0.21163E+01_JPRB, 0.20516E+01_JPRB, 0.21060E+01_JPRB, 0.20148E+01_JPRB ,&
&  0.20309E+01_JPRB, 0.20277E+01_JPRB, 0.17992E+01_JPRB, 0.16137E+01_JPRB, 0.13925E+01_JPRB, 0.14662E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.13126E+01_JPRB, 0.11135E+01_JPRB, 0.11804E+01_JPRB, 0.12373E+01_JPRB, 0.12658E+01_JPRB, 0.13106E+01_JPRB, 0.13191E+01_JPRB ,&
&  0.13467E+01_JPRB, 0.13618E+01_JPRB, 0.11876E+01_JPRB, 0.10463E+01_JPRB, 0.96600E+00_JPRB, 0.90183E+00_JPRB, 0.78957E+00_JPRB ,&
&  0.76295E+00_JPRB, 0.79675E+00_JPRB, 0.80651E+00_JPRB, 0.79437E+00_JPRB, 0.76939E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.77684E+00_JPRB, 0.75152E+00_JPRB, 0.72255E+00_JPRB, 0.72700E+00_JPRB, 0.74814E+00_JPRB, 0.75548E+00_JPRB, 0.77802E+00_JPRB ,&
&  0.80360E+00_JPRB, 0.80580E+00_JPRB, 0.77011E+00_JPRB, 0.74267E+00_JPRB, 0.73544E+00_JPRB, 0.75817E+00_JPRB, 0.80742E+00_JPRB ,&
&  0.73793E+00_JPRB, 0.72763E+00_JPRB, 0.66693E+00_JPRB, 0.68191E+00_JPRB, 0.77514E+00_JPRB, 0.71755E+00_JPRB, 0.71053E+00_JPRB ,&
&  0.69030E+00_JPRB, 0.68157E+00_JPRB, 0.65911E+00_JPRB, 0.64408E+00_JPRB, 0.64325E+00_JPRB, 0.66715E+00_JPRB, 0.71938E+00_JPRB ,&
&  0.76202E+00_JPRB, 0.88675E+00_JPRB, 0.10085E+01_JPRB, 0.99891E+00_JPRB, 0.10913E+01_JPRB, 0.11586E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.10737E+01_JPRB, 0.97813E+00_JPRB, 0.92534E+00_JPRB, 0.89670E+00_JPRB, 0.89100E+00_JPRB, 0.89009E+00_JPRB ,&
&  0.89330E+00_JPRB, 0.90184E+00_JPRB, 0.90673E+00_JPRB, 0.93056E+00_JPRB, 0.95768E+00_JPRB, 0.11336E+01_JPRB, 0.14591E+01_JPRB ,&
&  0.18737E+01_JPRB, 0.20588E+01_JPRB, 0.22750E+01_JPRB, 0.26262E+01_JPRB, 0.26970E+01_JPRB, 0.25899E+01_JPRB, 0.23404E+01_JPRB ,&
&  0.21854E+01_JPRB, 0.20562E+01_JPRB, 0.22048E+01_JPRB, 0.23827E+01_JPRB, 0.23742E+01_JPRB, 0.25875E+01_JPRB, 0.25337E+01_JPRB ,&
&  0.23082E+01_JPRB, 0.21561E+01_JPRB, 0.20511E+01_JPRB, 0.20498E+01_JPRB, 0.19923E+01_JPRB, 0.20725E+01_JPRB, 0.20569E+01_JPRB ,&
&  0.20587E+01_JPRB, 0.18188E+01_JPRB, 0.13959E+01_JPRB, 0.13029E+01_JPRB, 0.10298E+01_JPRB, 0.10523E+01_JPRB, 0.13073E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.13018E+01_JPRB, 0.13148E+01_JPRB, 0.13886E+01_JPRB, 0.14204E+01_JPRB, 0.16992E+01_JPRB, 0.18039E+01_JPRB ,&
&  0.15949E+01_JPRB, 0.15792E+01_JPRB, 0.14945E+01_JPRB, 0.13636E+01_JPRB, 0.12359E+01_JPRB, 0.10696E+01_JPRB, 0.94662E+00_JPRB ,&
&  0.90990E+00_JPRB, 0.88058E+00_JPRB, 0.87056E+00_JPRB, 0.83922E+00_JPRB, 0.79899E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.82931E+00_JPRB, 0.80606E+00_JPRB, 0.76328E+00_JPRB, 0.76120E+00_JPRB, 0.76493E+00_JPRB, 0.77228E+00_JPRB, 0.79901E+00_JPRB ,&
&  0.82669E+00_JPRB, 0.80717E+00_JPRB, 0.76326E+00_JPRB, 0.74805E+00_JPRB, 0.75516E+00_JPRB, 0.78031E+00_JPRB, 0.83745E+00_JPRB ,&
&  0.93057E+00_JPRB, 0.92869E+00_JPRB, 0.81479E+00_JPRB, 0.70035E+00_JPRB, 0.76167E+00_JPRB, 0.75431E+00_JPRB, 0.80988E+00_JPRB ,&
&  0.78822E+00_JPRB, 0.84041E+00_JPRB, 0.80358E+00_JPRB, 0.78581E+00_JPRB, 0.77634E+00_JPRB, 0.81731E+00_JPRB, 0.91352E+00_JPRB ,&
&  0.10501E+01_JPRB, 0.12777E+01_JPRB, 0.14853E+01_JPRB, 0.14850E+01_JPRB, 0.15432E+01_JPRB, 0.14084E+01_JPRB, 0.13914E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.11833E+01_JPRB, 0.10752E+01_JPRB, 0.10177E+01_JPRB, 0.99040E+00_JPRB, 0.97745E+00_JPRB, 0.98856E+00_JPRB ,&
&  0.10184E+01_JPRB, 0.10302E+01_JPRB, 0.10079E+01_JPRB, 0.99915E+00_JPRB, 0.10500E+01_JPRB, 0.11815E+01_JPRB, 0.13947E+01_JPRB ,&
&  0.18225E+01_JPRB, 0.20997E+01_JPRB, 0.22667E+01_JPRB, 0.27106E+01_JPRB, 0.29312E+01_JPRB, 0.29598E+01_JPRB, 0.28420E+01_JPRB ,&
&  0.26754E+01_JPRB, 0.25107E+01_JPRB, 0.25736E+01_JPRB, 0.26178E+01_JPRB, 0.26594E+01_JPRB, 0.27013E+01_JPRB, 0.25808E+01_JPRB ,&
&  0.22795E+01_JPRB, 0.21003E+01_JPRB, 0.20330E+01_JPRB, 0.19208E+01_JPRB, 0.18743E+01_JPRB, 0.17665E+01_JPRB, 0.18107E+01_JPRB ,&
&  0.18891E+01_JPRB, 0.11450E+01_JPRB, 0.93889E+00_JPRB, 0.10979E+01_JPRB, 0.94173E+00_JPRB, 0.97411E+00_JPRB, 0.11244E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.11383E+01_JPRB, 0.13147E+01_JPRB, 0.14962E+01_JPRB, 0.17110E+01_JPRB, 0.21212E+01_JPRB, 0.21467E+01_JPRB ,&
&  0.19919E+01_JPRB, 0.18008E+01_JPRB, 0.18166E+01_JPRB, 0.17379E+01_JPRB, 0.15760E+01_JPRB, 0.13332E+01_JPRB, 0.11475E+01_JPRB ,&
&  0.10400E+01_JPRB, 0.98943E+00_JPRB, 0.95557E+00_JPRB, 0.90824E+00_JPRB, 0.85249E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.90756E+00_JPRB, 0.87402E+00_JPRB, 0.80217E+00_JPRB, 0.78347E+00_JPRB, 0.79177E+00_JPRB, 0.81676E+00_JPRB, 0.84498E+00_JPRB ,&
&  0.85094E+00_JPRB, 0.80919E+00_JPRB, 0.77870E+00_JPRB, 0.77473E+00_JPRB, 0.78293E+00_JPRB, 0.79246E+00_JPRB, 0.81097E+00_JPRB ,&
&  0.84518E+00_JPRB, 0.91727E+00_JPRB, 0.79069E+00_JPRB, 0.75585E+00_JPRB, 0.68953E+00_JPRB, 0.74008E+00_JPRB, 0.81564E+00_JPRB ,&
&  0.91078E+00_JPRB, 0.91573E+00_JPRB, 0.89127E+00_JPRB, 0.91325E+00_JPRB, 0.93192E+00_JPRB, 0.99941E+00_JPRB, 0.12091E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.18004E+01_JPRB, 0.18887E+01_JPRB, 0.20124E+01_JPRB, 0.18417E+01_JPRB, 0.15807E+01_JPRB, 0.15448E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.12554E+01_JPRB, 0.11289E+01_JPRB, 0.10658E+01_JPRB, 0.10328E+01_JPRB, 0.10071E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.10501E+01_JPRB, 0.10438E+01_JPRB, 0.10397E+01_JPRB, 0.11217E+01_JPRB, 0.12496E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.16371E+01_JPRB, 0.16722E+01_JPRB, 0.14993E+01_JPRB, 0.21429E+01_JPRB, 0.31233E+01_JPRB, 0.31198E+01_JPRB, 0.27223E+01_JPRB ,&
&  0.28306E+01_JPRB, 0.27917E+01_JPRB, 0.28800E+01_JPRB, 0.29855E+01_JPRB, 0.29613E+01_JPRB, 0.28158E+01_JPRB, 0.26023E+01_JPRB ,&
&  0.23340E+01_JPRB, 0.21668E+01_JPRB, 0.20695E+01_JPRB, 0.21492E+01_JPRB, 0.20795E+01_JPRB, 0.20098E+01_JPRB, 0.19286E+01_JPRB ,&
&  0.16471E+01_JPRB, 0.14462E+01_JPRB, 0.10489E+01_JPRB, 0.10760E+01_JPRB, 0.86211E+00_JPRB, 0.87578E+00_JPRB, 0.10223E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.11049E+01_JPRB, 0.13014E+01_JPRB, 0.15228E+01_JPRB, 0.21466E+01_JPRB, 0.25853E+01_JPRB, 0.27259E+01_JPRB ,&
&  0.25904E+01_JPRB, 0.22151E+01_JPRB, 0.22332E+01_JPRB, 0.20387E+01_JPRB, 0.18921E+01_JPRB, 0.16657E+01_JPRB, 0.14138E+01_JPRB ,&
&  0.12408E+01_JPRB, 0.11521E+01_JPRB, 0.10908E+01_JPRB, 0.10169E+01_JPRB, 0.94119E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.95094E+00_JPRB, 0.88930E+00_JPRB, 0.79392E+00_JPRB, 0.77731E+00_JPRB, 0.79597E+00_JPRB, 0.85142E+00_JPRB, 0.88433E+00_JPRB ,&
&  0.86493E+00_JPRB, 0.81797E+00_JPRB, 0.79253E+00_JPRB, 0.78680E+00_JPRB, 0.78397E+00_JPRB, 0.77381E+00_JPRB, 0.77514E+00_JPRB ,&
&  0.78046E+00_JPRB, 0.79581E+00_JPRB, 0.71886E+00_JPRB, 0.67257E+00_JPRB, 0.63741E+00_JPRB, 0.67964E+00_JPRB, 0.76709E+00_JPRB ,&
&  0.84706E+00_JPRB, 0.90945E+00_JPRB, 0.98854E+00_JPRB, 0.10521E+01_JPRB, 0.10765E+01_JPRB, 0.12252E+01_JPRB, 0.15569E+01_JPRB ,&
&  0.21281E+01_JPRB, 0.21049E+01_JPRB, 0.22226E+01_JPRB, 0.20984E+01_JPRB, 0.18965E+01_JPRB, 0.16859E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.11655E+01_JPRB, 0.10558E+01_JPRB, 0.99250E+00_JPRB, 0.99148E+00_JPRB, 0.10141E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.11074E+01_JPRB, 0.10796E+01_JPRB, 0.11263E+01_JPRB, 0.12552E+01_JPRB, 0.13422E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.13561E+01_JPRB, 0.13520E+01_JPRB, 0.18570E+01_JPRB, 0.21823E+01_JPRB, 0.27716E+01_JPRB, 0.31257E+01_JPRB, 0.30407E+01_JPRB ,&
&  0.30843E+01_JPRB, 0.29978E+01_JPRB, 0.30033E+01_JPRB, 0.24954E+01_JPRB, 0.28172E+01_JPRB, 0.27077E+01_JPRB, 0.24869E+01_JPRB ,&
&  0.22845E+01_JPRB, 0.21624E+01_JPRB, 0.21272E+01_JPRB, 0.21632E+01_JPRB, 0.22287E+01_JPRB, 0.22876E+01_JPRB, 0.17593E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.15509E+01_JPRB, 0.14145E+01_JPRB, 0.12004E+01_JPRB, 0.10261E+01_JPRB, 0.10542E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.11535E+01_JPRB, 0.13831E+01_JPRB, 0.18801E+01_JPRB, 0.27514E+01_JPRB, 0.33002E+01_JPRB, 0.32807E+01_JPRB ,&
&  0.28693E+01_JPRB, 0.29088E+01_JPRB, 0.26693E+01_JPRB, 0.23359E+01_JPRB, 0.20856E+01_JPRB, 0.18788E+01_JPRB, 0.16743E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.13489E+01_JPRB, 0.12440E+01_JPRB, 0.11298E+01_JPRB, 0.10139E+01_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.98343E+00_JPRB, 0.91598E+00_JPRB, 0.82564E+00_JPRB, 0.81156E+00_JPRB, 0.85274E+00_JPRB, 0.90806E+00_JPRB, 0.91745E+00_JPRB ,&
&  0.85211E+00_JPRB, 0.80967E+00_JPRB, 0.78325E+00_JPRB, 0.78426E+00_JPRB, 0.78278E+00_JPRB, 0.76552E+00_JPRB, 0.74981E+00_JPRB ,&
&  0.77272E+00_JPRB, 0.73592E+00_JPRB, 0.66925E+00_JPRB, 0.70361E+00_JPRB, 0.72444E+00_JPRB, 0.67099E+00_JPRB, 0.72932E+00_JPRB ,&
&  0.81377E+00_JPRB, 0.91393E+00_JPRB, 0.99701E+00_JPRB, 0.11447E+01_JPRB, 0.13393E+01_JPRB, 0.16435E+01_JPRB, 0.20200E+01_JPRB ,&
&  0.21616E+01_JPRB, 0.22754E+01_JPRB, 0.23247E+01_JPRB, 0.21482E+01_JPRB, 0.17935E+01_JPRB, 0.15131E+01_JPRB, 0.13549E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.10557E+01_JPRB, 0.99319E+00_JPRB, 0.99873E+00_JPRB, 0.10546E+01_JPRB, 0.11072E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.11516E+01_JPRB, 0.11865E+01_JPRB, 0.13384E+01_JPRB, 0.11280E+01_JPRB, 0.11419E+01_JPRB, 0.11499E+01_JPRB ,&
&  0.15675E+01_JPRB, 0.16805E+01_JPRB, 0.17930E+01_JPRB, 0.22764E+01_JPRB, 0.20963E+01_JPRB, 0.24040E+01_JPRB, 0.29565E+01_JPRB ,&
&  0.32317E+01_JPRB, 0.32563E+01_JPRB, 0.32780E+01_JPRB, 0.21203E+01_JPRB, 0.18141E+01_JPRB, 0.23998E+01_JPRB, 0.23500E+01_JPRB ,&
&  0.21980E+01_JPRB, 0.22243E+01_JPRB, 0.21730E+01_JPRB, 0.22942E+01_JPRB, 0.17780E+01_JPRB, 0.15293E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.10948E+01_JPRB, 0.12908E+01_JPRB, 0.11976E+01_JPRB, 0.12612E+01_JPRB, 0.12276E+01_JPRB, 0.11714E+01_JPRB ,&
&  0.12663E+01_JPRB, 0.14586E+01_JPRB, 0.18685E+01_JPRB, 0.27106E+01_JPRB, 0.37301E+01_JPRB, 0.34675E+01_JPRB, 0.37733E+01_JPRB ,&
&  0.36879E+01_JPRB, 0.33231E+01_JPRB, 0.27517E+01_JPRB, 0.25024E+01_JPRB, 0.22791E+01_JPRB, 0.20866E+01_JPRB, 0.18859E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.15156E+01_JPRB, 0.13597E+01_JPRB, 0.12022E+01_JPRB, 0.10663E+01_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.10602E+01_JPRB, 0.99000E+00_JPRB, 0.88354E+00_JPRB, 0.87805E+00_JPRB, 0.92625E+00_JPRB, 0.94681E+00_JPRB, 0.89755E+00_JPRB ,&
&  0.84308E+00_JPRB, 0.79750E+00_JPRB, 0.78220E+00_JPRB, 0.77480E+00_JPRB, 0.76595E+00_JPRB, 0.76373E+00_JPRB, 0.75623E+00_JPRB ,&
&  0.80114E+00_JPRB, 0.80156E+00_JPRB, 0.72236E+00_JPRB, 0.67569E+00_JPRB, 0.73629E+00_JPRB, 0.68998E+00_JPRB, 0.67397E+00_JPRB ,&
&  0.82147E+00_JPRB, 0.96812E+00_JPRB, 0.11256E+01_JPRB, 0.13855E+01_JPRB, 0.16893E+01_JPRB, 0.22066E+01_JPRB, 0.24662E+01_JPRB ,&
&  0.24712E+01_JPRB, 0.24311E+01_JPRB, 0.21583E+01_JPRB, 0.18202E+01_JPRB, 0.15026E+01_JPRB, 0.13091E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.96769E+00_JPRB, 0.96699E+00_JPRB, 0.10117E+01_JPRB, 0.10764E+01_JPRB, 0.11097E+01_JPRB, 0.11264E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.12850E+01_JPRB, 0.13377E+01_JPRB, 0.13280E+01_JPRB, 0.12875E+01_JPRB, 0.11602E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.15362E+01_JPRB, 0.15356E+01_JPRB, 0.17051E+01_JPRB, 0.20641E+01_JPRB, 0.24212E+01_JPRB, 0.24215E+01_JPRB, 0.29565E+01_JPRB ,&
&  0.24651E+01_JPRB, 0.20602E+01_JPRB, 0.17515E+01_JPRB, 0.13291E+01_JPRB, 0.15988E+01_JPRB, 0.22781E+01_JPRB, 0.23371E+01_JPRB ,&
&  0.22552E+01_JPRB, 0.22490E+01_JPRB, 0.22483E+01_JPRB, 0.19420E+01_JPRB, 0.14991E+01_JPRB, 0.12371E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.16196E+01_JPRB, 0.16890E+01_JPRB, 0.16265E+01_JPRB, 0.14234E+01_JPRB, 0.13124E+01_JPRB, 0.15198E+01_JPRB, 0.15743E+01_JPRB ,&
&  0.18104E+01_JPRB, 0.20746E+01_JPRB, 0.33383E+01_JPRB, 0.44558E+01_JPRB, 0.46400E+01_JPRB, 0.44699E+01_JPRB, 0.46745E+01_JPRB ,&
&  0.40821E+01_JPRB, 0.36950E+01_JPRB, 0.31761E+01_JPRB, 0.27282E+01_JPRB, 0.24218E+01_JPRB, 0.22059E+01_JPRB, 0.20336E+01_JPRB ,&
&  0.18352E+01_JPRB, 0.16835E+01_JPRB, 0.15149E+01_JPRB, 0.13091E+01_JPRB, 0.11505E+01_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.11578E+01_JPRB, 0.10608E+01_JPRB, 0.94061E+00_JPRB, 0.93245E+00_JPRB, 0.97541E+00_JPRB, 0.94747E+00_JPRB, 0.87099E+00_JPRB ,&
&  0.84301E+00_JPRB, 0.80201E+00_JPRB, 0.78831E+00_JPRB, 0.77272E+00_JPRB, 0.75685E+00_JPRB, 0.75124E+00_JPRB, 0.77422E+00_JPRB ,&
&  0.85226E+00_JPRB, 0.95876E+00_JPRB, 0.86107E+00_JPRB, 0.77134E+00_JPRB, 0.74490E+00_JPRB, 0.72349E+00_JPRB, 0.64403E+00_JPRB ,&
&  0.86310E+00_JPRB, 0.10858E+01_JPRB, 0.12859E+01_JPRB, 0.16332E+01_JPRB, 0.22070E+01_JPRB, 0.23282E+01_JPRB, 0.23932E+01_JPRB ,&
&  0.23587E+01_JPRB, 0.21854E+01_JPRB, 0.18467E+01_JPRB, 0.15346E+01_JPRB, 0.13166E+01_JPRB, 0.11844E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.97559E+00_JPRB, 0.96910E+00_JPRB, 0.99235E+00_JPRB, 0.10319E+01_JPRB, 0.11146E+01_JPRB, 0.11880E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.13366E+01_JPRB, 0.14192E+01_JPRB, 0.14530E+01_JPRB, 0.14576E+01_JPRB, 0.14901E+01_JPRB, 0.13659E+01_JPRB, 0.18075E+01_JPRB ,&
&  0.17277E+01_JPRB, 0.17543E+01_JPRB, 0.17463E+01_JPRB, 0.22094E+01_JPRB, 0.23329E+01_JPRB, 0.23243E+01_JPRB, 0.26295E+01_JPRB ,&
&  0.20135E+01_JPRB, 0.18246E+01_JPRB, 0.19129E+01_JPRB, 0.18974E+01_JPRB, 0.15963E+01_JPRB, 0.17022E+01_JPRB, 0.23860E+01_JPRB ,&
&  0.19028E+01_JPRB, 0.20777E+01_JPRB, 0.21801E+01_JPRB, 0.21541E+01_JPRB, 0.10444E+01_JPRB, 0.81581E+00_JPRB, 0.13288E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.10406E+01_JPRB, 0.81304E+00_JPRB, 0.10272E+01_JPRB, 0.85793E+00_JPRB, 0.11099E+01_JPRB, 0.18037E+01_JPRB ,&
&  0.23952E+01_JPRB, 0.32173E+01_JPRB, 0.54919E+01_JPRB, 0.56738E+01_JPRB, 0.50218E+01_JPRB, 0.50517E+01_JPRB, 0.45412E+01_JPRB ,&
&  0.41019E+01_JPRB, 0.32352E+01_JPRB, 0.30594E+01_JPRB, 0.27198E+01_JPRB, 0.24578E+01_JPRB, 0.22893E+01_JPRB, 0.21321E+01_JPRB ,&
&  0.19529E+01_JPRB, 0.17848E+01_JPRB, 0.16070E+01_JPRB, 0.14231E+01_JPRB, 0.12750E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.12150E+01_JPRB, 0.11072E+01_JPRB, 0.99867E+00_JPRB, 0.97360E+00_JPRB, 0.99357E+00_JPRB, 0.91617E+00_JPRB, 0.86265E+00_JPRB ,&
&  0.82161E+00_JPRB, 0.79981E+00_JPRB, 0.78065E+00_JPRB, 0.75721E+00_JPRB, 0.74032E+00_JPRB, 0.75409E+00_JPRB, 0.79184E+00_JPRB ,&
&  0.85203E+00_JPRB, 0.95029E+00_JPRB, 0.10904E+01_JPRB, 0.97112E+00_JPRB, 0.88873E+00_JPRB, 0.80537E+00_JPRB, 0.74682E+00_JPRB ,&
&  0.97782E+00_JPRB, 0.12855E+01_JPRB, 0.15100E+01_JPRB, 0.17335E+01_JPRB, 0.19802E+01_JPRB, 0.20092E+01_JPRB, 0.21510E+01_JPRB ,&
&  0.21097E+01_JPRB, 0.18149E+01_JPRB, 0.15683E+01_JPRB, 0.13604E+01_JPRB, 0.12202E+01_JPRB, 0.10967E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.98696E+00_JPRB, 0.98483E+00_JPRB, 0.98621E+00_JPRB, 0.10713E+01_JPRB, 0.12048E+01_JPRB, 0.12918E+01_JPRB, 0.13731E+01_JPRB ,&
&  0.14403E+01_JPRB, 0.14685E+01_JPRB, 0.14773E+01_JPRB, 0.15042E+01_JPRB, 0.15934E+01_JPRB, 0.15172E+01_JPRB, 0.13970E+01_JPRB ,&
&  0.13613E+01_JPRB, 0.15106E+01_JPRB, 0.20878E+01_JPRB, 0.22868E+01_JPRB, 0.22061E+01_JPRB, 0.23323E+01_JPRB, 0.23323E+01_JPRB ,&
&  0.22712E+01_JPRB, 0.23643E+01_JPRB, 0.20774E+01_JPRB, 0.21564E+01_JPRB, 0.19691E+01_JPRB, 0.14812E+01_JPRB, 0.18348E+01_JPRB ,&
&  0.19687E+01_JPRB, 0.18919E+01_JPRB, 0.14823E+01_JPRB, 0.10939E+01_JPRB, 0.11282E+01_JPRB, 0.92331E+00_JPRB, 0.70306E+00_JPRB ,&
&  0.67570E+00_JPRB, 0.65398E+00_JPRB, 0.61070E+00_JPRB, 0.64628E+00_JPRB, 0.67773E+00_JPRB, 0.10871E+01_JPRB, 0.18945E+01_JPRB ,&
&  0.34949E+01_JPRB, 0.52369E+01_JPRB, 0.63505E+01_JPRB, 0.60343E+01_JPRB, 0.51612E+01_JPRB, 0.49320E+01_JPRB, 0.44825E+01_JPRB ,&
&  0.33759E+01_JPRB, 0.28911E+01_JPRB, 0.27028E+01_JPRB, 0.25826E+01_JPRB, 0.24864E+01_JPRB, 0.23867E+01_JPRB, 0.22236E+01_JPRB ,&
&  0.20199E+01_JPRB, 0.18563E+01_JPRB, 0.16953E+01_JPRB, 0.15005E+01_JPRB, 0.13481E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.12533E+01_JPRB, 0.11458E+01_JPRB, 0.10792E+01_JPRB, 0.10312E+01_JPRB, 0.99956E+00_JPRB, 0.89528E+00_JPRB, 0.85466E+00_JPRB ,&
&  0.80002E+00_JPRB, 0.78524E+00_JPRB, 0.75204E+00_JPRB, 0.73575E+00_JPRB, 0.72914E+00_JPRB, 0.76210E+00_JPRB, 0.82165E+00_JPRB ,&
&  0.88409E+00_JPRB, 0.10004E+01_JPRB, 0.12163E+01_JPRB, 0.12089E+01_JPRB, 0.12203E+01_JPRB, 0.11346E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.16149E+01_JPRB, 0.17753E+01_JPRB, 0.18437E+01_JPRB, 0.20025E+01_JPRB, 0.20215E+01_JPRB, 0.19316E+01_JPRB ,&
&  0.18105E+01_JPRB, 0.15743E+01_JPRB, 0.13842E+01_JPRB, 0.12457E+01_JPRB, 0.11313E+01_JPRB, 0.10392E+01_JPRB, 0.99115E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.97606E+00_JPRB, 0.10060E+01_JPRB, 0.11350E+01_JPRB, 0.12629E+01_JPRB, 0.13420E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.15065E+01_JPRB, 0.15176E+01_JPRB, 0.15150E+01_JPRB, 0.14996E+01_JPRB, 0.12842E+01_JPRB, 0.11445E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.16330E+01_JPRB, 0.19565E+01_JPRB, 0.21706E+01_JPRB, 0.25294E+01_JPRB, 0.22543E+01_JPRB, 0.21892E+01_JPRB, 0.22018E+01_JPRB ,&
&  0.21629E+01_JPRB, 0.25221E+01_JPRB, 0.20180E+01_JPRB, 0.19092E+01_JPRB, 0.20928E+01_JPRB, 0.16271E+01_JPRB, 0.15690E+01_JPRB ,&
&  0.18355E+01_JPRB, 0.17531E+01_JPRB, 0.14581E+01_JPRB, 0.11842E+01_JPRB, 0.15541E+01_JPRB, 0.16536E+01_JPRB, 0.87073E+00_JPRB ,&
&  0.73348E+00_JPRB, 0.69344E+00_JPRB, 0.66256E+00_JPRB, 0.64456E+00_JPRB, 0.70415E+00_JPRB, 0.12207E+01_JPRB, 0.27232E+01_JPRB ,&
&  0.42396E+01_JPRB, 0.64709E+01_JPRB, 0.67431E+01_JPRB, 0.62603E+01_JPRB, 0.51972E+01_JPRB, 0.46588E+01_JPRB, 0.38155E+01_JPRB ,&
&  0.32584E+01_JPRB, 0.28721E+01_JPRB, 0.26528E+01_JPRB, 0.25423E+01_JPRB, 0.24421E+01_JPRB, 0.23440E+01_JPRB, 0.22117E+01_JPRB ,&
&  0.20402E+01_JPRB, 0.18857E+01_JPRB, 0.17287E+01_JPRB, 0.15330E+01_JPRB, 0.13760E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.12606E+01_JPRB, 0.11810E+01_JPRB, 0.11442E+01_JPRB, 0.11045E+01_JPRB, 0.10170E+01_JPRB, 0.91089E+00_JPRB, 0.82818E+00_JPRB ,&
&  0.78476E+00_JPRB, 0.76119E+00_JPRB, 0.74141E+00_JPRB, 0.73784E+00_JPRB, 0.74848E+00_JPRB, 0.78781E+00_JPRB, 0.83778E+00_JPRB ,&
&  0.91824E+00_JPRB, 0.10448E+01_JPRB, 0.11844E+01_JPRB, 0.13934E+01_JPRB, 0.13912E+01_JPRB, 0.12585E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.15947E+01_JPRB, 0.19567E+01_JPRB, 0.20740E+01_JPRB, 0.20183E+01_JPRB, 0.19617E+01_JPRB, 0.19267E+01_JPRB, 0.17970E+01_JPRB ,&
&  0.16067E+01_JPRB, 0.14077E+01_JPRB, 0.12478E+01_JPRB, 0.11526E+01_JPRB, 0.10671E+01_JPRB, 0.10121E+01_JPRB, 0.10113E+01_JPRB ,&
&  0.98277E+00_JPRB, 0.94979E+00_JPRB, 0.10364E+01_JPRB, 0.11741E+01_JPRB, 0.12833E+01_JPRB, 0.13944E+01_JPRB, 0.15072E+01_JPRB ,&
&  0.15499E+01_JPRB, 0.15570E+01_JPRB, 0.15479E+01_JPRB, 0.15338E+01_JPRB, 0.12727E+01_JPRB, 0.14799E+01_JPRB, 0.15621E+01_JPRB ,&
&  0.17272E+01_JPRB, 0.19164E+01_JPRB, 0.19861E+01_JPRB, 0.22872E+01_JPRB, 0.22021E+01_JPRB, 0.21034E+01_JPRB, 0.20126E+01_JPRB ,&
&  0.21279E+01_JPRB, 0.20044E+01_JPRB, 0.17147E+01_JPRB, 0.17461E+01_JPRB, 0.19435E+01_JPRB, 0.21163E+01_JPRB, 0.14358E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.16463E+01_JPRB, 0.14958E+01_JPRB, 0.13665E+01_JPRB, 0.17165E+01_JPRB, 0.22796E+01_JPRB, 0.17150E+01_JPRB ,&
&  0.89290E+00_JPRB, 0.74165E+00_JPRB, 0.77379E+00_JPRB, 0.83961E+00_JPRB, 0.86632E+00_JPRB, 0.15295E+01_JPRB, 0.56850E+01_JPRB ,&
&  0.60686E+01_JPRB, 0.74032E+01_JPRB, 0.66707E+01_JPRB, 0.56686E+01_JPRB, 0.49647E+01_JPRB, 0.43158E+01_JPRB, 0.36608E+01_JPRB ,&
&  0.31534E+01_JPRB, 0.28417E+01_JPRB, 0.25944E+01_JPRB, 0.24196E+01_JPRB, 0.22500E+01_JPRB, 0.20838E+01_JPRB, 0.19455E+01_JPRB ,&
&  0.18554E+01_JPRB, 0.17502E+01_JPRB, 0.16445E+01_JPRB, 0.14949E+01_JPRB, 0.13604E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.12353E+01_JPRB, 0.11786E+01_JPRB, 0.11516E+01_JPRB, 0.11062E+01_JPRB, 0.10281E+01_JPRB, 0.95706E+00_JPRB, 0.85758E+00_JPRB ,&
&  0.79062E+00_JPRB, 0.75530E+00_JPRB, 0.74155E+00_JPRB, 0.75041E+00_JPRB, 0.77857E+00_JPRB, 0.81988E+00_JPRB, 0.85564E+00_JPRB ,&
&  0.96556E+00_JPRB, 0.10677E+01_JPRB, 0.12470E+01_JPRB, 0.14779E+01_JPRB, 0.15435E+01_JPRB, 0.13849E+01_JPRB, 0.14726E+01_JPRB ,&
&  0.19594E+01_JPRB, 0.22342E+01_JPRB, 0.20577E+01_JPRB, 0.19797E+01_JPRB, 0.19706E+01_JPRB, 0.19948E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.14876E+01_JPRB, 0.13604E+01_JPRB, 0.12108E+01_JPRB, 0.11163E+01_JPRB, 0.10471E+01_JPRB, 0.10432E+01_JPRB, 0.99618E+00_JPRB ,&
&  0.93989E+00_JPRB, 0.97054E+00_JPRB, 0.10763E+01_JPRB, 0.11903E+01_JPRB, 0.13036E+01_JPRB, 0.14211E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.15956E+01_JPRB, 0.15912E+01_JPRB, 0.15578E+01_JPRB, 0.14750E+01_JPRB, 0.14861E+01_JPRB, 0.15344E+01_JPRB, 0.16685E+01_JPRB ,&
&  0.17343E+01_JPRB, 0.18663E+01_JPRB, 0.19449E+01_JPRB, 0.20372E+01_JPRB, 0.20155E+01_JPRB, 0.19832E+01_JPRB, 0.18766E+01_JPRB ,&
&  0.18859E+01_JPRB, 0.17529E+01_JPRB, 0.14741E+01_JPRB, 0.16065E+01_JPRB, 0.18839E+01_JPRB, 0.20586E+01_JPRB, 0.18884E+01_JPRB ,&
&  0.16695E+01_JPRB, 0.14749E+01_JPRB, 0.14784E+01_JPRB, 0.17144E+01_JPRB, 0.19814E+01_JPRB, 0.23836E+01_JPRB, 0.30477E+01_JPRB ,&
&  0.32263E+01_JPRB, 0.19129E+01_JPRB, 0.16044E+01_JPRB, 0.22069E+01_JPRB, 0.20149E+01_JPRB, 0.22857E+01_JPRB, 0.41784E+01_JPRB ,&
&  0.58760E+01_JPRB, 0.68263E+01_JPRB, 0.62987E+01_JPRB, 0.52050E+01_JPRB, 0.46421E+01_JPRB, 0.39165E+01_JPRB, 0.34065E+01_JPRB ,&
&  0.29085E+01_JPRB, 0.25086E+01_JPRB, 0.22286E+01_JPRB, 0.20226E+01_JPRB, 0.18406E+01_JPRB, 0.17059E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.16168E+01_JPRB, 0.15648E+01_JPRB, 0.15069E+01_JPRB, 0.13971E+01_JPRB, 0.12996E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.11834E+01_JPRB, 0.11479E+01_JPRB, 0.11180E+01_JPRB, 0.10723E+01_JPRB, 0.10603E+01_JPRB, 0.10393E+01_JPRB, 0.95037E+00_JPRB ,&
&  0.87566E+00_JPRB, 0.76921E+00_JPRB, 0.76174E+00_JPRB, 0.78381E+00_JPRB, 0.81837E+00_JPRB, 0.85979E+00_JPRB, 0.92900E+00_JPRB ,&
&  0.10304E+01_JPRB, 0.11308E+01_JPRB, 0.13655E+01_JPRB, 0.16086E+01_JPRB, 0.18342E+01_JPRB, 0.21615E+01_JPRB, 0.15111E+01_JPRB ,&
&  0.15920E+01_JPRB, 0.20404E+01_JPRB, 0.19532E+01_JPRB, 0.19275E+01_JPRB, 0.18880E+01_JPRB, 0.18609E+01_JPRB, 0.15597E+01_JPRB ,&
&  0.13879E+01_JPRB, 0.12960E+01_JPRB, 0.11737E+01_JPRB, 0.10771E+01_JPRB, 0.10536E+01_JPRB, 0.10296E+01_JPRB, 0.95416E+00_JPRB ,&
&  0.95056E+00_JPRB, 0.10088E+01_JPRB, 0.11022E+01_JPRB, 0.12008E+01_JPRB, 0.13144E+01_JPRB, 0.14426E+01_JPRB, 0.15238E+01_JPRB ,&
&  0.15984E+01_JPRB, 0.16131E+01_JPRB, 0.15855E+01_JPRB, 0.14815E+01_JPRB, 0.15189E+01_JPRB, 0.15572E+01_JPRB, 0.16012E+01_JPRB ,&
&  0.15258E+01_JPRB, 0.14766E+01_JPRB, 0.17064E+01_JPRB, 0.18858E+01_JPRB, 0.18682E+01_JPRB, 0.17358E+01_JPRB, 0.16133E+01_JPRB ,&
&  0.16993E+01_JPRB, 0.17214E+01_JPRB, 0.17195E+01_JPRB, 0.16853E+01_JPRB, 0.18735E+01_JPRB, 0.20320E+01_JPRB, 0.21193E+01_JPRB ,&
&  0.21521E+01_JPRB, 0.20400E+01_JPRB, 0.20736E+01_JPRB, 0.22341E+01_JPRB, 0.23453E+01_JPRB, 0.26632E+01_JPRB, 0.30406E+01_JPRB ,&
&  0.36517E+01_JPRB, 0.43810E+01_JPRB, 0.45965E+01_JPRB, 0.38591E+01_JPRB, 0.37101E+01_JPRB, 0.35232E+01_JPRB, 0.43670E+01_JPRB ,&
&  0.56575E+01_JPRB, 0.54930E+01_JPRB, 0.50873E+01_JPRB, 0.49434E+01_JPRB, 0.37739E+01_JPRB, 0.31811E+01_JPRB, 0.27769E+01_JPRB ,&
&  0.24824E+01_JPRB, 0.21472E+01_JPRB, 0.18886E+01_JPRB, 0.16955E+01_JPRB, 0.15156E+01_JPRB, 0.14352E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.14225E+01_JPRB, 0.14034E+01_JPRB, 0.13670E+01_JPRB, 0.12951E+01_JPRB, 0.12345E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.11272E+01_JPRB, 0.10860E+01_JPRB, 0.10606E+01_JPRB, 0.10399E+01_JPRB, 0.10843E+01_JPRB, 0.10946E+01_JPRB, 0.10827E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.81815E+00_JPRB, 0.79923E+00_JPRB, 0.82109E+00_JPRB, 0.86327E+00_JPRB, 0.91628E+00_JPRB, 0.99982E+00_JPRB ,&
&  0.10827E+01_JPRB, 0.12196E+01_JPRB, 0.14584E+01_JPRB, 0.17353E+01_JPRB, 0.19605E+01_JPRB, 0.21021E+01_JPRB, 0.20144E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.20532E+01_JPRB, 0.18943E+01_JPRB, 0.19004E+01_JPRB, 0.18590E+01_JPRB, 0.17482E+01_JPRB, 0.14692E+01_JPRB ,&
&  0.13003E+01_JPRB, 0.12309E+01_JPRB, 0.11767E+01_JPRB, 0.11505E+01_JPRB, 0.10993E+01_JPRB, 0.10082E+01_JPRB, 0.98169E+00_JPRB ,&
&  0.10132E+01_JPRB, 0.10920E+01_JPRB, 0.11825E+01_JPRB, 0.12702E+01_JPRB, 0.13802E+01_JPRB, 0.14830E+01_JPRB, 0.15439E+01_JPRB ,&
&  0.16089E+01_JPRB, 0.16201E+01_JPRB, 0.15480E+01_JPRB, 0.15307E+01_JPRB, 0.15325E+01_JPRB, 0.15699E+01_JPRB, 0.15836E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.14539E+01_JPRB, 0.15980E+01_JPRB, 0.16215E+01_JPRB, 0.15438E+01_JPRB, 0.15801E+01_JPRB, 0.15127E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.17064E+01_JPRB, 0.20327E+01_JPRB, 0.17959E+01_JPRB, 0.18435E+01_JPRB, 0.20336E+01_JPRB, 0.20297E+01_JPRB ,&
&  0.19143E+01_JPRB, 0.21137E+01_JPRB, 0.22822E+01_JPRB, 0.26528E+01_JPRB, 0.29594E+01_JPRB, 0.32747E+01_JPRB, 0.33458E+01_JPRB ,&
&  0.36400E+01_JPRB, 0.40294E+01_JPRB, 0.45836E+01_JPRB, 0.38264E+01_JPRB, 0.37096E+01_JPRB, 0.38274E+01_JPRB, 0.45695E+01_JPRB ,&
&  0.51736E+01_JPRB, 0.48282E+01_JPRB, 0.46732E+01_JPRB, 0.39806E+01_JPRB, 0.30533E+01_JPRB, 0.26365E+01_JPRB, 0.23036E+01_JPRB ,&
&  0.20505E+01_JPRB, 0.17940E+01_JPRB, 0.15978E+01_JPRB, 0.14305E+01_JPRB, 0.13239E+01_JPRB, 0.13054E+01_JPRB, 0.13187E+01_JPRB ,&
&  0.13165E+01_JPRB, 0.13044E+01_JPRB, 0.12823E+01_JPRB, 0.12253E+01_JPRB, 0.11719E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.10913E+01_JPRB, 0.10714E+01_JPRB, 0.10604E+01_JPRB, 0.10297E+01_JPRB, 0.10783E+01_JPRB, 0.11845E+01_JPRB, 0.12603E+01_JPRB ,&
&  0.12147E+01_JPRB, 0.84755E+00_JPRB, 0.83719E+00_JPRB, 0.86032E+00_JPRB, 0.89888E+00_JPRB, 0.96938E+00_JPRB, 0.10546E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.13238E+01_JPRB, 0.15682E+01_JPRB, 0.18625E+01_JPRB, 0.20749E+01_JPRB, 0.23143E+01_JPRB, 0.22985E+01_JPRB ,&
&  0.16564E+01_JPRB, 0.15098E+01_JPRB, 0.18362E+01_JPRB, 0.18079E+01_JPRB, 0.16991E+01_JPRB, 0.16464E+01_JPRB, 0.15590E+01_JPRB ,&
&  0.14229E+01_JPRB, 0.13053E+01_JPRB, 0.12922E+01_JPRB, 0.12512E+01_JPRB, 0.10862E+01_JPRB, 0.10554E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11894E+01_JPRB, 0.12713E+01_JPRB, 0.13518E+01_JPRB, 0.14523E+01_JPRB, 0.15184E+01_JPRB, 0.15819E+01_JPRB ,&
&  0.16160E+01_JPRB, 0.16363E+01_JPRB, 0.15941E+01_JPRB, 0.15282E+01_JPRB, 0.15328E+01_JPRB, 0.15343E+01_JPRB, 0.15127E+01_JPRB ,&
&  0.14988E+01_JPRB, 0.15568E+01_JPRB, 0.15851E+01_JPRB, 0.15148E+01_JPRB, 0.14178E+01_JPRB, 0.15772E+01_JPRB, 0.16239E+01_JPRB ,&
&  0.17525E+01_JPRB, 0.18676E+01_JPRB, 0.22737E+01_JPRB, 0.18141E+01_JPRB, 0.18210E+01_JPRB, 0.19619E+01_JPRB, 0.20165E+01_JPRB ,&
&  0.20442E+01_JPRB, 0.23087E+01_JPRB, 0.26398E+01_JPRB, 0.29986E+01_JPRB, 0.33478E+01_JPRB, 0.33358E+01_JPRB, 0.40028E+01_JPRB ,&
&  0.37917E+01_JPRB, 0.43841E+01_JPRB, 0.40939E+01_JPRB, 0.40443E+01_JPRB, 0.35577E+01_JPRB, 0.37123E+01_JPRB, 0.39317E+01_JPRB ,&
&  0.42212E+01_JPRB, 0.38883E+01_JPRB, 0.34331E+01_JPRB, 0.29666E+01_JPRB, 0.23965E+01_JPRB, 0.21235E+01_JPRB, 0.18594E+01_JPRB ,&
&  0.16234E+01_JPRB, 0.14632E+01_JPRB, 0.13903E+01_JPRB, 0.13315E+01_JPRB, 0.13059E+01_JPRB, 0.12973E+01_JPRB, 0.12937E+01_JPRB ,&
&  0.12763E+01_JPRB, 0.12552E+01_JPRB, 0.12278E+01_JPRB, 0.11725E+01_JPRB, 0.11253E+01_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.10633E+01_JPRB, 0.10412E+01_JPRB, 0.10136E+01_JPRB, 0.94573E+00_JPRB, 0.90598E+00_JPRB, 0.90950E+00_JPRB, 0.89835E+00_JPRB ,&
&  0.88339E+00_JPRB, 0.85780E+00_JPRB, 0.87132E+00_JPRB, 0.90759E+00_JPRB, 0.95278E+00_JPRB, 0.10296E+01_JPRB, 0.11316E+01_JPRB ,&
&  0.12644E+01_JPRB, 0.13945E+01_JPRB, 0.16071E+01_JPRB, 0.18234E+01_JPRB, 0.20002E+01_JPRB, 0.21432E+01_JPRB, 0.21989E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.13613E+01_JPRB, 0.17397E+01_JPRB, 0.18322E+01_JPRB, 0.17266E+01_JPRB, 0.17162E+01_JPRB, 0.16381E+01_JPRB ,&
&  0.14498E+01_JPRB, 0.13913E+01_JPRB, 0.13941E+01_JPRB, 0.12954E+01_JPRB, 0.11313E+01_JPRB, 0.11394E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.12369E+01_JPRB, 0.12883E+01_JPRB, 0.13530E+01_JPRB, 0.14301E+01_JPRB, 0.14977E+01_JPRB, 0.15825E+01_JPRB, 0.16421E+01_JPRB ,&
&  0.16607E+01_JPRB, 0.16641E+01_JPRB, 0.16541E+01_JPRB, 0.15867E+01_JPRB, 0.15414E+01_JPRB, 0.15158E+01_JPRB, 0.14839E+01_JPRB ,&
&  0.14865E+01_JPRB, 0.14783E+01_JPRB, 0.15346E+01_JPRB, 0.16128E+01_JPRB, 0.17388E+01_JPRB, 0.16520E+01_JPRB, 0.17951E+01_JPRB ,&
&  0.18668E+01_JPRB, 0.20600E+01_JPRB, 0.21110E+01_JPRB, 0.23205E+01_JPRB, 0.15811E+01_JPRB, 0.17916E+01_JPRB, 0.18000E+01_JPRB ,&
&  0.23211E+01_JPRB, 0.25650E+01_JPRB, 0.29364E+01_JPRB, 0.33265E+01_JPRB, 0.38091E+01_JPRB, 0.37811E+01_JPRB, 0.40301E+01_JPRB ,&
&  0.39934E+01_JPRB, 0.38722E+01_JPRB, 0.36060E+01_JPRB, 0.36581E+01_JPRB, 0.34978E+01_JPRB, 0.38360E+01_JPRB, 0.35013E+01_JPRB ,&
&  0.34726E+01_JPRB, 0.30853E+01_JPRB, 0.27314E+01_JPRB, 0.24677E+01_JPRB, 0.20120E+01_JPRB, 0.16780E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.13708E+01_JPRB, 0.13278E+01_JPRB, 0.12928E+01_JPRB, 0.12730E+01_JPRB, 0.12730E+01_JPRB, 0.12744E+01_JPRB, 0.12705E+01_JPRB ,&
&  0.12535E+01_JPRB, 0.12255E+01_JPRB, 0.11903E+01_JPRB, 0.11406E+01_JPRB, 0.10997E+01_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.10260E+01_JPRB, 0.99712E+00_JPRB, 0.95649E+00_JPRB, 0.90591E+00_JPRB, 0.88154E+00_JPRB, 0.86442E+00_JPRB, 0.86285E+00_JPRB ,&
&  0.86614E+00_JPRB, 0.88596E+00_JPRB, 0.89915E+00_JPRB, 0.92557E+00_JPRB, 0.95972E+00_JPRB, 0.10257E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.12792E+01_JPRB, 0.14924E+01_JPRB, 0.17597E+01_JPRB, 0.19719E+01_JPRB, 0.20874E+01_JPRB, 0.20001E+01_JPRB ,&
&  0.18998E+01_JPRB, 0.18734E+01_JPRB, 0.17755E+01_JPRB, 0.14144E+01_JPRB, 0.13509E+01_JPRB, 0.16244E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.13420E+01_JPRB, 0.12957E+01_JPRB, 0.12482E+01_JPRB, 0.12090E+01_JPRB, 0.12215E+01_JPRB, 0.12573E+01_JPRB ,&
&  0.13056E+01_JPRB, 0.13558E+01_JPRB, 0.14217E+01_JPRB, 0.14902E+01_JPRB, 0.15615E+01_JPRB, 0.16635E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.17267E+01_JPRB, 0.17169E+01_JPRB, 0.16912E+01_JPRB, 0.16428E+01_JPRB, 0.15598E+01_JPRB, 0.15481E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.15899E+01_JPRB, 0.16142E+01_JPRB, 0.16515E+01_JPRB, 0.17622E+01_JPRB, 0.18503E+01_JPRB, 0.17124E+01_JPRB, 0.17377E+01_JPRB ,&
&  0.19313E+01_JPRB, 0.21686E+01_JPRB, 0.19929E+01_JPRB, 0.23308E+01_JPRB, 0.14591E+01_JPRB, 0.17579E+01_JPRB, 0.22072E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.27792E+01_JPRB, 0.31658E+01_JPRB, 0.35981E+01_JPRB, 0.40181E+01_JPRB, 0.37702E+01_JPRB, 0.38379E+01_JPRB ,&
&  0.35813E+01_JPRB, 0.31700E+01_JPRB, 0.31091E+01_JPRB, 0.30646E+01_JPRB, 0.32220E+01_JPRB, 0.32608E+01_JPRB, 0.26723E+01_JPRB ,&
&  0.22698E+01_JPRB, 0.24627E+01_JPRB, 0.23903E+01_JPRB, 0.22425E+01_JPRB, 0.17454E+01_JPRB, 0.13687E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.12777E+01_JPRB, 0.12486E+01_JPRB, 0.12396E+01_JPRB, 0.12422E+01_JPRB, 0.12527E+01_JPRB, 0.12535E+01_JPRB, 0.12476E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.11983E+01_JPRB, 0.11583E+01_JPRB, 0.11107E+01_JPRB, 0.10658E+01_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.98016E+00_JPRB, 0.94981E+00_JPRB, 0.90611E+00_JPRB, 0.86681E+00_JPRB, 0.81857E+00_JPRB, 0.79468E+00_JPRB, 0.79227E+00_JPRB ,&
&  0.81081E+00_JPRB, 0.83730E+00_JPRB, 0.85355E+00_JPRB, 0.85785E+00_JPRB, 0.88176E+00_JPRB, 0.89462E+00_JPRB, 0.92029E+00_JPRB ,&
&  0.97765E+00_JPRB, 0.10769E+01_JPRB, 0.12719E+01_JPRB, 0.15809E+01_JPRB, 0.18689E+01_JPRB, 0.21349E+01_JPRB, 0.20144E+01_JPRB ,&
&  0.18933E+01_JPRB, 0.18152E+01_JPRB, 0.17683E+01_JPRB, 0.17167E+01_JPRB, 0.15240E+01_JPRB, 0.16155E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.16396E+01_JPRB, 0.15238E+01_JPRB, 0.13739E+01_JPRB, 0.13235E+01_JPRB, 0.12895E+01_JPRB, 0.12970E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.14144E+01_JPRB, 0.14836E+01_JPRB, 0.15714E+01_JPRB, 0.16763E+01_JPRB, 0.17753E+01_JPRB, 0.18200E+01_JPRB ,&
&  0.18085E+01_JPRB, 0.17948E+01_JPRB, 0.17721E+01_JPRB, 0.16986E+01_JPRB, 0.16754E+01_JPRB, 0.17062E+01_JPRB, 0.17507E+01_JPRB ,&
&  0.18094E+01_JPRB, 0.17667E+01_JPRB, 0.18392E+01_JPRB, 0.19664E+01_JPRB, 0.19602E+01_JPRB, 0.18892E+01_JPRB, 0.19100E+01_JPRB ,&
&  0.20492E+01_JPRB, 0.22726E+01_JPRB, 0.17053E+01_JPRB, 0.20629E+01_JPRB, 0.24878E+01_JPRB, 0.22425E+01_JPRB, 0.23323E+01_JPRB ,&
&  0.26720E+01_JPRB, 0.29519E+01_JPRB, 0.32906E+01_JPRB, 0.36367E+01_JPRB, 0.40296E+01_JPRB, 0.43599E+01_JPRB, 0.32185E+01_JPRB ,&
&  0.29988E+01_JPRB, 0.26373E+01_JPRB, 0.25676E+01_JPRB, 0.26004E+01_JPRB, 0.26816E+01_JPRB, 0.25623E+01_JPRB, 0.20624E+01_JPRB ,&
&  0.18581E+01_JPRB, 0.20223E+01_JPRB, 0.19461E+01_JPRB, 0.19826E+01_JPRB, 0.18489E+01_JPRB, 0.12078E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.12066E+01_JPRB, 0.12047E+01_JPRB, 0.12081E+01_JPRB, 0.12144E+01_JPRB, 0.12129E+01_JPRB, 0.12070E+01_JPRB ,&
&  0.11839E+01_JPRB, 0.11448E+01_JPRB, 0.11011E+01_JPRB, 0.10561E+01_JPRB, 0.10152E+01_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.94281E+00_JPRB, 0.90552E+00_JPRB, 0.85949E+00_JPRB, 0.80921E+00_JPRB, 0.76152E+00_JPRB, 0.74209E+00_JPRB, 0.73323E+00_JPRB ,&
&  0.73940E+00_JPRB, 0.76266E+00_JPRB, 0.77519E+00_JPRB, 0.77033E+00_JPRB, 0.76380E+00_JPRB, 0.77509E+00_JPRB, 0.81837E+00_JPRB ,&
&  0.87458E+00_JPRB, 0.94807E+00_JPRB, 0.10921E+01_JPRB, 0.13950E+01_JPRB, 0.17697E+01_JPRB, 0.20698E+01_JPRB, 0.20254E+01_JPRB ,&
&  0.18958E+01_JPRB, 0.18888E+01_JPRB, 0.18761E+01_JPRB, 0.18870E+01_JPRB, 0.19015E+01_JPRB, 0.19505E+01_JPRB, 0.19994E+01_JPRB ,&
&  0.17748E+01_JPRB, 0.14560E+01_JPRB, 0.13708E+01_JPRB, 0.13283E+01_JPRB, 0.13336E+01_JPRB, 0.13493E+01_JPRB, 0.13795E+01_JPRB ,&
&  0.14165E+01_JPRB, 0.14752E+01_JPRB, 0.15582E+01_JPRB, 0.16841E+01_JPRB, 0.18165E+01_JPRB, 0.18948E+01_JPRB, 0.19254E+01_JPRB ,&
&  0.19363E+01_JPRB, 0.19394E+01_JPRB, 0.19739E+01_JPRB, 0.18119E+01_JPRB, 0.18798E+01_JPRB, 0.19474E+01_JPRB, 0.20021E+01_JPRB ,&
&  0.19947E+01_JPRB, 0.19329E+01_JPRB, 0.20643E+01_JPRB, 0.21238E+01_JPRB, 0.21839E+01_JPRB, 0.21747E+01_JPRB, 0.21536E+01_JPRB ,&
&  0.22347E+01_JPRB, 0.20911E+01_JPRB, 0.15543E+01_JPRB, 0.16068E+01_JPRB, 0.17979E+01_JPRB, 0.17921E+01_JPRB, 0.24372E+01_JPRB ,&
&  0.26384E+01_JPRB, 0.28023E+01_JPRB, 0.29731E+01_JPRB, 0.31452E+01_JPRB, 0.33370E+01_JPRB, 0.34597E+01_JPRB, 0.28081E+01_JPRB ,&
&  0.24134E+01_JPRB, 0.21857E+01_JPRB, 0.21176E+01_JPRB, 0.20647E+01_JPRB, 0.20076E+01_JPRB, 0.18629E+01_JPRB, 0.17796E+01_JPRB ,&
&  0.17105E+01_JPRB, 0.16562E+01_JPRB, 0.15533E+01_JPRB, 0.14384E+01_JPRB, 0.12753E+01_JPRB, 0.11623E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.11844E+01_JPRB, 0.11605E+01_JPRB, 0.11592E+01_JPRB, 0.11553E+01_JPRB, 0.11551E+01_JPRB, 0.11477E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.10714E+01_JPRB, 0.10389E+01_JPRB, 0.10026E+01_JPRB, 0.96860E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.91051E+00_JPRB, 0.86725E+00_JPRB, 0.81553E+00_JPRB, 0.77189E+00_JPRB, 0.76531E+00_JPRB, 0.76778E+00_JPRB, 0.75183E+00_JPRB ,&
&  0.74333E+00_JPRB, 0.73751E+00_JPRB, 0.75353E+00_JPRB, 0.75455E+00_JPRB, 0.75740E+00_JPRB, 0.77146E+00_JPRB, 0.79080E+00_JPRB ,&
&  0.80778E+00_JPRB, 0.82276E+00_JPRB, 0.89421E+00_JPRB, 0.10467E+01_JPRB, 0.13308E+01_JPRB, 0.17031E+01_JPRB, 0.19705E+01_JPRB ,&
&  0.21162E+01_JPRB, 0.22388E+01_JPRB, 0.23438E+01_JPRB, 0.24348E+01_JPRB, 0.22854E+01_JPRB, 0.23297E+01_JPRB, 0.25283E+01_JPRB ,&
&  0.18675E+01_JPRB, 0.13495E+01_JPRB, 0.14061E+01_JPRB, 0.13028E+01_JPRB, 0.13157E+01_JPRB, 0.13637E+01_JPRB, 0.14027E+01_JPRB ,&
&  0.14545E+01_JPRB, 0.15241E+01_JPRB, 0.16142E+01_JPRB, 0.17381E+01_JPRB, 0.18552E+01_JPRB, 0.19325E+01_JPRB, 0.19835E+01_JPRB ,&
&  0.20216E+01_JPRB, 0.20738E+01_JPRB, 0.21609E+01_JPRB, 0.21376E+01_JPRB, 0.21130E+01_JPRB, 0.22132E+01_JPRB, 0.22643E+01_JPRB ,&
&  0.23155E+01_JPRB, 0.23273E+01_JPRB, 0.20974E+01_JPRB, 0.21286E+01_JPRB, 0.23330E+01_JPRB, 0.22250E+01_JPRB, 0.22304E+01_JPRB ,&
&  0.23670E+01_JPRB, 0.20891E+01_JPRB, 0.14743E+01_JPRB, 0.14923E+01_JPRB, 0.17185E+01_JPRB, 0.20389E+01_JPRB, 0.23954E+01_JPRB ,&
&  0.25817E+01_JPRB, 0.26709E+01_JPRB, 0.26565E+01_JPRB, 0.26647E+01_JPRB, 0.25511E+01_JPRB, 0.24216E+01_JPRB, 0.22685E+01_JPRB ,&
&  0.19723E+01_JPRB, 0.18592E+01_JPRB, 0.17459E+01_JPRB, 0.16767E+01_JPRB, 0.16581E+01_JPRB, 0.15908E+01_JPRB, 0.15360E+01_JPRB ,&
&  0.16224E+01_JPRB, 0.16476E+01_JPRB, 0.17737E+01_JPRB, 0.14630E+01_JPRB, 0.11809E+01_JPRB, 0.11516E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.11734E+01_JPRB, 0.11567E+01_JPRB, 0.11427E+01_JPRB, 0.11285E+01_JPRB, 0.11180E+01_JPRB, 0.10950E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.10267E+01_JPRB, 0.10006E+01_JPRB, 0.98423E+00_JPRB, 0.96241E+00_JPRB, 0.93273E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.86087E+00_JPRB, 0.79847E+00_JPRB, 0.77017E+00_JPRB, 0.80843E+00_JPRB, 0.78015E+00_JPRB, 0.80139E+00_JPRB, 0.79183E+00_JPRB ,&
&  0.79141E+00_JPRB, 0.79056E+00_JPRB, 0.80669E+00_JPRB, 0.82822E+00_JPRB, 0.85448E+00_JPRB, 0.87615E+00_JPRB, 0.88529E+00_JPRB ,&
&  0.89662E+00_JPRB, 0.90852E+00_JPRB, 0.92795E+00_JPRB, 0.96416E+00_JPRB, 0.10318E+01_JPRB, 0.11473E+01_JPRB, 0.13689E+01_JPRB ,&
&  0.15928E+01_JPRB, 0.17068E+01_JPRB, 0.17448E+01_JPRB, 0.17446E+01_JPRB, 0.19317E+01_JPRB, 0.23993E+01_JPRB, 0.27494E+01_JPRB ,&
&  0.16610E+01_JPRB, 0.14163E+01_JPRB, 0.12788E+01_JPRB, 0.11680E+01_JPRB, 0.12008E+01_JPRB, 0.13120E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.13924E+01_JPRB, 0.14548E+01_JPRB, 0.15388E+01_JPRB, 0.16380E+01_JPRB, 0.17180E+01_JPRB, 0.17869E+01_JPRB, 0.18642E+01_JPRB ,&
&  0.19505E+01_JPRB, 0.20724E+01_JPRB, 0.22065E+01_JPRB, 0.23536E+01_JPRB, 0.23563E+01_JPRB, 0.26068E+01_JPRB, 0.26518E+01_JPRB ,&
&  0.26732E+01_JPRB, 0.25526E+01_JPRB, 0.22728E+01_JPRB, 0.22931E+01_JPRB, 0.24036E+01_JPRB, 0.23890E+01_JPRB, 0.22741E+01_JPRB ,&
&  0.23135E+01_JPRB, 0.20388E+01_JPRB, 0.16567E+01_JPRB, 0.17352E+01_JPRB, 0.18636E+01_JPRB, 0.20874E+01_JPRB, 0.21109E+01_JPRB ,&
&  0.22413E+01_JPRB, 0.22317E+01_JPRB, 0.21677E+01_JPRB, 0.20572E+01_JPRB, 0.19443E+01_JPRB, 0.19037E+01_JPRB, 0.18859E+01_JPRB ,&
&  0.17190E+01_JPRB, 0.16357E+01_JPRB, 0.15324E+01_JPRB, 0.15242E+01_JPRB, 0.15748E+01_JPRB, 0.13470E+01_JPRB, 0.14188E+01_JPRB ,&
&  0.14808E+01_JPRB, 0.15396E+01_JPRB, 0.13590E+01_JPRB, 0.15562E+01_JPRB, 0.11518E+01_JPRB, 0.10782E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.11309E+01_JPRB, 0.10968E+01_JPRB, 0.10873E+01_JPRB, 0.10733E+01_JPRB, 0.10695E+01_JPRB, 0.10498E+01_JPRB, 0.99843E+00_JPRB ,&
&  0.95787E+00_JPRB, 0.93582E+00_JPRB, 0.93132E+00_JPRB, 0.91900E+00_JPRB, 0.90242E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.79654E+00_JPRB, 0.76192E+00_JPRB, 0.81414E+00_JPRB, 0.77842E+00_JPRB, 0.77643E+00_JPRB, 0.79126E+00_JPRB, 0.80758E+00_JPRB ,&
&  0.81241E+00_JPRB, 0.82776E+00_JPRB, 0.85623E+00_JPRB, 0.89995E+00_JPRB, 0.94064E+00_JPRB, 0.97110E+00_JPRB, 0.10056E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.10489E+01_JPRB, 0.10592E+01_JPRB, 0.10700E+01_JPRB, 0.11003E+01_JPRB, 0.11360E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.12794E+01_JPRB, 0.12802E+01_JPRB, 0.13156E+01_JPRB, 0.13536E+01_JPRB, 0.14811E+01_JPRB, 0.17464E+01_JPRB ,&
&  0.13508E+01_JPRB, 0.13698E+01_JPRB, 0.12757E+01_JPRB, 0.11968E+01_JPRB, 0.12184E+01_JPRB, 0.11874E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.12978E+01_JPRB, 0.13380E+01_JPRB, 0.13693E+01_JPRB, 0.14086E+01_JPRB, 0.14690E+01_JPRB, 0.15320E+01_JPRB ,&
&  0.16220E+01_JPRB, 0.17420E+01_JPRB, 0.18430E+01_JPRB, 0.20028E+01_JPRB, 0.21571E+01_JPRB, 0.23079E+01_JPRB, 0.23730E+01_JPRB ,&
&  0.24113E+01_JPRB, 0.24198E+01_JPRB, 0.22583E+01_JPRB, 0.22610E+01_JPRB, 0.22494E+01_JPRB, 0.21504E+01_JPRB, 0.19427E+01_JPRB ,&
&  0.17609E+01_JPRB, 0.18058E+01_JPRB, 0.18907E+01_JPRB, 0.18617E+01_JPRB, 0.18939E+01_JPRB, 0.18933E+01_JPRB, 0.18835E+01_JPRB ,&
&  0.19046E+01_JPRB, 0.18042E+01_JPRB, 0.17022E+01_JPRB, 0.15703E+01_JPRB, 0.15182E+01_JPRB, 0.15164E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.14655E+01_JPRB, 0.13967E+01_JPRB, 0.13768E+01_JPRB, 0.14713E+01_JPRB, 0.16235E+01_JPRB, 0.20973E+01_JPRB, 0.16800E+01_JPRB ,&
&  0.12767E+01_JPRB, 0.11612E+01_JPRB, 0.87605E+00_JPRB, 0.11466E+01_JPRB, 0.99527E+00_JPRB, 0.96788E+00_JPRB, 0.10056E+01_JPRB ,&
&  0.10245E+01_JPRB, 0.10204E+01_JPRB, 0.10597E+01_JPRB, 0.10231E+01_JPRB, 0.99055E+00_JPRB, 0.97357E+00_JPRB, 0.97205E+00_JPRB ,&
&  0.92107E+00_JPRB, 0.86687E+00_JPRB, 0.81125E+00_JPRB, 0.81873E+00_JPRB, 0.77260E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.68653E+00_JPRB, 0.71419E+00_JPRB, 0.80934E+00_JPRB, 0.74532E+00_JPRB, 0.74429E+00_JPRB, 0.76169E+00_JPRB, 0.78874E+00_JPRB ,&
&  0.81777E+00_JPRB, 0.85493E+00_JPRB, 0.89483E+00_JPRB, 0.94911E+00_JPRB, 0.99726E+00_JPRB, 0.10386E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.11449E+01_JPRB, 0.11754E+01_JPRB, 0.11915E+01_JPRB, 0.12039E+01_JPRB, 0.12308E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.13606E+01_JPRB, 0.13988E+01_JPRB, 0.14611E+01_JPRB, 0.15075E+01_JPRB, 0.15666E+01_JPRB, 0.13824E+01_JPRB ,&
&  0.15053E+01_JPRB, 0.13268E+01_JPRB, 0.12476E+01_JPRB, 0.12140E+01_JPRB, 0.11742E+01_JPRB, 0.11456E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.11370E+01_JPRB, 0.11452E+01_JPRB, 0.11245E+01_JPRB, 0.11418E+01_JPRB, 0.12015E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.13194E+01_JPRB, 0.14295E+01_JPRB, 0.15445E+01_JPRB, 0.16282E+01_JPRB, 0.17065E+01_JPRB, 0.17032E+01_JPRB, 0.16950E+01_JPRB ,&
&  0.17255E+01_JPRB, 0.17980E+01_JPRB, 0.17556E+01_JPRB, 0.17838E+01_JPRB, 0.17649E+01_JPRB, 0.16283E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.14684E+01_JPRB, 0.16087E+01_JPRB, 0.19940E+01_JPRB, 0.18518E+01_JPRB, 0.16913E+01_JPRB, 0.16734E+01_JPRB ,&
&  0.15908E+01_JPRB, 0.14578E+01_JPRB, 0.13578E+01_JPRB, 0.12082E+01_JPRB, 0.11928E+01_JPRB, 0.12007E+01_JPRB, 0.12201E+01_JPRB ,&
&  0.11715E+01_JPRB, 0.11227E+01_JPRB, 0.11090E+01_JPRB, 0.12273E+01_JPRB, 0.16772E+01_JPRB, 0.17556E+01_JPRB, 0.19510E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.71637E+00_JPRB, 0.80885E+00_JPRB, 0.85464E+00_JPRB, 0.82276E+00_JPRB, 0.81044E+00_JPRB, 0.87139E+00_JPRB ,&
&  0.94897E+00_JPRB, 0.95026E+00_JPRB, 0.92693E+00_JPRB, 0.93163E+00_JPRB, 0.92751E+00_JPRB, 0.93041E+00_JPRB, 0.85196E+00_JPRB ,&
&  0.82852E+00_JPRB, 0.76999E+00_JPRB, 0.73435E+00_JPRB, 0.71569E+00_JPRB, 0.67622E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.64158E+00_JPRB, 0.71235E+00_JPRB, 0.78253E+00_JPRB, 0.71767E+00_JPRB, 0.72988E+00_JPRB, 0.75517E+00_JPRB, 0.79132E+00_JPRB ,&
&  0.83181E+00_JPRB, 0.87921E+00_JPRB, 0.92830E+00_JPRB, 0.98563E+00_JPRB, 0.10323E+01_JPRB, 0.10756E+01_JPRB, 0.11219E+01_JPRB ,&
&  0.11687E+01_JPRB, 0.11890E+01_JPRB, 0.12285E+01_JPRB, 0.12410E+01_JPRB, 0.12685E+01_JPRB, 0.12760E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.13576E+01_JPRB, 0.13968E+01_JPRB, 0.14407E+01_JPRB, 0.14801E+01_JPRB, 0.14900E+01_JPRB, 0.15868E+01_JPRB, 0.12784E+01_JPRB ,&
&  0.14675E+01_JPRB, 0.12991E+01_JPRB, 0.12077E+01_JPRB, 0.12046E+01_JPRB, 0.11710E+01_JPRB, 0.11596E+01_JPRB, 0.11503E+01_JPRB ,&
&  0.11418E+01_JPRB, 0.11453E+01_JPRB, 0.11431E+01_JPRB, 0.11145E+01_JPRB, 0.10322E+01_JPRB, 0.10616E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.12128E+01_JPRB, 0.12673E+01_JPRB, 0.13041E+01_JPRB, 0.13255E+01_JPRB, 0.13056E+01_JPRB, 0.12792E+01_JPRB ,&
&  0.12523E+01_JPRB, 0.12606E+01_JPRB, 0.11875E+01_JPRB, 0.12191E+01_JPRB, 0.13059E+01_JPRB, 0.11306E+01_JPRB, 0.89765E+00_JPRB ,&
&  0.79607E+00_JPRB, 0.11051E+01_JPRB, 0.15692E+01_JPRB, 0.19494E+01_JPRB, 0.17153E+01_JPRB, 0.15705E+01_JPRB, 0.14823E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.11966E+01_JPRB, 0.10860E+01_JPRB, 0.96640E+00_JPRB, 0.92293E+00_JPRB, 0.90953E+00_JPRB, 0.91845E+00_JPRB ,&
&  0.89965E+00_JPRB, 0.88538E+00_JPRB, 0.90049E+00_JPRB, 0.94551E+00_JPRB, 0.11323E+01_JPRB, 0.12810E+01_JPRB, 0.22306E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.83785E+00_JPRB, 0.12686E+01_JPRB, 0.53959E+00_JPRB, 0.63890E+00_JPRB, 0.68107E+00_JPRB, 0.72116E+00_JPRB ,&
&  0.67567E+00_JPRB, 0.85334E+00_JPRB, 0.12062E+01_JPRB, 0.12323E+01_JPRB, 0.12884E+01_JPRB, 0.13632E+01_JPRB, 0.84436E+00_JPRB ,&
&  0.70178E+00_JPRB, 0.69578E+00_JPRB, 0.70081E+00_JPRB, 0.64753E+00_JPRB, 0.63559E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.60353E+00_JPRB, 0.65964E+00_JPRB, 0.73039E+00_JPRB, 0.78094E+00_JPRB, 0.76557E+00_JPRB, 0.76959E+00_JPRB, 0.79115E+00_JPRB ,&
&  0.81920E+00_JPRB, 0.85506E+00_JPRB, 0.89196E+00_JPRB, 0.93983E+00_JPRB, 0.97425E+00_JPRB, 0.10176E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.11495E+01_JPRB, 0.11728E+01_JPRB, 0.12030E+01_JPRB, 0.12367E+01_JPRB, 0.12662E+01_JPRB, 0.12964E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.13416E+01_JPRB, 0.13715E+01_JPRB, 0.13931E+01_JPRB, 0.14602E+01_JPRB, 0.15104E+01_JPRB, 0.17055E+01_JPRB, 0.12466E+01_JPRB ,&
&  0.13919E+01_JPRB, 0.12581E+01_JPRB, 0.11744E+01_JPRB, 0.11861E+01_JPRB, 0.11698E+01_JPRB, 0.11469E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.11770E+01_JPRB, 0.11516E+01_JPRB, 0.11824E+01_JPRB, 0.11270E+01_JPRB, 0.10998E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.12567E+01_JPRB, 0.13199E+01_JPRB, 0.13407E+01_JPRB, 0.13767E+01_JPRB, 0.13815E+01_JPRB, 0.13593E+01_JPRB, 0.13256E+01_JPRB ,&
&  0.12852E+01_JPRB, 0.12711E+01_JPRB, 0.12782E+01_JPRB, 0.13011E+01_JPRB, 0.13109E+01_JPRB, 0.11592E+01_JPRB, 0.95664E+00_JPRB ,&
&  0.75925E+00_JPRB, 0.78680E+00_JPRB, 0.13002E+01_JPRB, 0.17268E+01_JPRB, 0.15418E+01_JPRB, 0.14360E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.11788E+01_JPRB, 0.10415E+01_JPRB, 0.94830E+00_JPRB, 0.84737E+00_JPRB, 0.80370E+00_JPRB, 0.78843E+00_JPRB, 0.82471E+00_JPRB ,&
&  0.84460E+00_JPRB, 0.82458E+00_JPRB, 0.82375E+00_JPRB, 0.83649E+00_JPRB, 0.10179E+01_JPRB, 0.13395E+01_JPRB, 0.13877E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.97637E+00_JPRB, 0.83457E+00_JPRB, 0.84553E+00_JPRB, 0.61523E+00_JPRB, 0.64396E+00_JPRB, 0.69294E+00_JPRB ,&
&  0.87763E+00_JPRB, 0.95555E+00_JPRB, 0.12920E+01_JPRB, 0.35083E+01_JPRB, 0.40106E+01_JPRB, 0.50030E+01_JPRB, 0.20260E+01_JPRB ,&
&  0.97271E+00_JPRB, 0.66691E+00_JPRB, 0.60371E+00_JPRB, 0.60639E+00_JPRB, 0.59531E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.56947E+00_JPRB, 0.61410E+00_JPRB, 0.65206E+00_JPRB, 0.70964E+00_JPRB, 0.73242E+00_JPRB, 0.74123E+00_JPRB, 0.74385E+00_JPRB ,&
&  0.73586E+00_JPRB, 0.74420E+00_JPRB, 0.76888E+00_JPRB, 0.82774E+00_JPRB, 0.89103E+00_JPRB, 0.95454E+00_JPRB, 0.10241E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.11769E+01_JPRB, 0.11743E+01_JPRB, 0.11919E+01_JPRB, 0.12334E+01_JPRB, 0.12748E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.13039E+01_JPRB, 0.13404E+01_JPRB, 0.13857E+01_JPRB, 0.14574E+01_JPRB, 0.15505E+01_JPRB, 0.16406E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.12325E+01_JPRB, 0.12052E+01_JPRB, 0.12169E+01_JPRB, 0.11636E+01_JPRB, 0.10981E+01_JPRB, 0.10948E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.12170E+01_JPRB, 0.12467E+01_JPRB, 0.12317E+01_JPRB, 0.13137E+01_JPRB, 0.12687E+01_JPRB, 0.14141E+01_JPRB ,&
&  0.15342E+01_JPRB, 0.15951E+01_JPRB, 0.16249E+01_JPRB, 0.16541E+01_JPRB, 0.16885E+01_JPRB, 0.17440E+01_JPRB, 0.18080E+01_JPRB ,&
&  0.18412E+01_JPRB, 0.18499E+01_JPRB, 0.18469E+01_JPRB, 0.18308E+01_JPRB, 0.16255E+01_JPRB, 0.16473E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.94046E+00_JPRB, 0.82266E+00_JPRB, 0.11569E+01_JPRB, 0.14971E+01_JPRB, 0.14045E+01_JPRB, 0.12929E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.94538E+00_JPRB, 0.86762E+00_JPRB, 0.79526E+00_JPRB, 0.73752E+00_JPRB, 0.71539E+00_JPRB, 0.72102E+00_JPRB ,&
&  0.72825E+00_JPRB, 0.72577E+00_JPRB, 0.72995E+00_JPRB, 0.77361E+00_JPRB, 0.91159E+00_JPRB, 0.12652E+01_JPRB, 0.18104E+01_JPRB ,&
&  0.21249E+01_JPRB, 0.10196E+01_JPRB, 0.80592E+00_JPRB, 0.74376E+00_JPRB, 0.72410E+00_JPRB, 0.75106E+00_JPRB, 0.79716E+00_JPRB ,&
&  0.94589E+00_JPRB, 0.12208E+01_JPRB, 0.11321E+01_JPRB, 0.13142E+01_JPRB, 0.25835E+01_JPRB, 0.27968E+01_JPRB, 0.16277E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.74652E+00_JPRB, 0.61225E+00_JPRB, 0.57310E+00_JPRB, 0.56444E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.56890E+00_JPRB, 0.58230E+00_JPRB, 0.60225E+00_JPRB, 0.62761E+00_JPRB, 0.62602E+00_JPRB, 0.62852E+00_JPRB, 0.61690E+00_JPRB ,&
&  0.61592E+00_JPRB, 0.63642E+00_JPRB, 0.68660E+00_JPRB, 0.77140E+00_JPRB, 0.83536E+00_JPRB, 0.92333E+00_JPRB, 0.10430E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10852E+01_JPRB, 0.10854E+01_JPRB, 0.11009E+01_JPRB, 0.11314E+01_JPRB, 0.11709E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.12482E+01_JPRB, 0.13341E+01_JPRB, 0.14097E+01_JPRB, 0.15092E+01_JPRB, 0.16059E+01_JPRB, 0.16685E+01_JPRB, 0.17939E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.13095E+01_JPRB, 0.12974E+01_JPRB, 0.12666E+01_JPRB, 0.10909E+01_JPRB, 0.99789E+00_JPRB, 0.10385E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.13501E+01_JPRB, 0.15218E+01_JPRB, 0.13287E+01_JPRB, 0.14207E+01_JPRB, 0.14691E+01_JPRB, 0.16325E+01_JPRB ,&
&  0.16954E+01_JPRB, 0.17535E+01_JPRB, 0.18295E+01_JPRB, 0.19136E+01_JPRB, 0.19970E+01_JPRB, 0.21087E+01_JPRB, 0.22141E+01_JPRB ,&
&  0.22890E+01_JPRB, 0.23030E+01_JPRB, 0.23415E+01_JPRB, 0.20904E+01_JPRB, 0.18127E+01_JPRB, 0.18400E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.14905E+01_JPRB, 0.10802E+01_JPRB, 0.11305E+01_JPRB, 0.13835E+01_JPRB, 0.13458E+01_JPRB, 0.12548E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.99242E+00_JPRB, 0.85230E+00_JPRB, 0.78951E+00_JPRB, 0.72970E+00_JPRB, 0.68943E+00_JPRB, 0.69213E+00_JPRB, 0.71939E+00_JPRB ,&
&  0.72478E+00_JPRB, 0.71872E+00_JPRB, 0.71328E+00_JPRB, 0.73011E+00_JPRB, 0.79019E+00_JPRB, 0.85004E+00_JPRB, 0.90160E+00_JPRB ,&
&  0.87477E+00_JPRB, 0.84412E+00_JPRB, 0.81839E+00_JPRB, 0.82992E+00_JPRB, 0.83860E+00_JPRB, 0.83690E+00_JPRB, 0.86273E+00_JPRB ,&
&  0.87172E+00_JPRB, 0.86833E+00_JPRB, 0.96239E+00_JPRB, 0.11678E+01_JPRB, 0.12394E+01_JPRB, 0.13090E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.87482E+00_JPRB, 0.74284E+00_JPRB, 0.65701E+00_JPRB, 0.60096E+00_JPRB, 0.55815E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.55529E+00_JPRB, 0.55527E+00_JPRB, 0.55311E+00_JPRB, 0.56474E+00_JPRB, 0.56493E+00_JPRB, 0.56655E+00_JPRB, 0.56339E+00_JPRB ,&
&  0.58953E+00_JPRB, 0.64080E+00_JPRB, 0.71976E+00_JPRB, 0.80966E+00_JPRB, 0.87751E+00_JPRB, 0.96080E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.99882E+00_JPRB, 0.10024E+01_JPRB, 0.10350E+01_JPRB, 0.10648E+01_JPRB, 0.11102E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.12872E+01_JPRB, 0.13841E+01_JPRB, 0.14983E+01_JPRB, 0.16280E+01_JPRB, 0.17271E+01_JPRB, 0.18194E+01_JPRB ,&
&  0.72103E+00_JPRB, 0.94929E+00_JPRB, 0.13000E+01_JPRB, 0.13649E+01_JPRB, 0.10606E+01_JPRB, 0.94668E+00_JPRB, 0.87511E+00_JPRB ,&
&  0.10317E+01_JPRB, 0.12493E+01_JPRB, 0.13662E+01_JPRB, 0.14870E+01_JPRB, 0.14754E+01_JPRB, 0.15941E+01_JPRB, 0.16984E+01_JPRB ,&
&  0.17332E+01_JPRB, 0.17749E+01_JPRB, 0.18594E+01_JPRB, 0.19640E+01_JPRB, 0.20577E+01_JPRB, 0.21572E+01_JPRB, 0.22599E+01_JPRB ,&
&  0.23686E+01_JPRB, 0.24081E+01_JPRB, 0.24682E+01_JPRB, 0.20879E+01_JPRB, 0.21268E+01_JPRB, 0.22085E+01_JPRB, 0.24765E+01_JPRB ,&
&  0.21244E+01_JPRB, 0.13620E+01_JPRB, 0.11308E+01_JPRB, 0.13208E+01_JPRB, 0.13017E+01_JPRB, 0.12400E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.92334E+00_JPRB, 0.81547E+00_JPRB, 0.76864E+00_JPRB, 0.75155E+00_JPRB, 0.73955E+00_JPRB, 0.75689E+00_JPRB, 0.78099E+00_JPRB ,&
&  0.77880E+00_JPRB, 0.77169E+00_JPRB, 0.75884E+00_JPRB, 0.75830E+00_JPRB, 0.80310E+00_JPRB, 0.85098E+00_JPRB, 0.88274E+00_JPRB ,&
&  0.89289E+00_JPRB, 0.90103E+00_JPRB, 0.92504E+00_JPRB, 0.95468E+00_JPRB, 0.98681E+00_JPRB, 0.99949E+00_JPRB, 0.10063E+01_JPRB ,&
&  0.89679E+00_JPRB, 0.83680E+00_JPRB, 0.81605E+00_JPRB, 0.80670E+00_JPRB, 0.80508E+00_JPRB, 0.83278E+00_JPRB, 0.78329E+00_JPRB ,&
&  0.75252E+00_JPRB, 0.73908E+00_JPRB, 0.70505E+00_JPRB, 0.63430E+00_JPRB, 0.56813E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.57123E+00_JPRB, 0.57000E+00_JPRB, 0.56132E+00_JPRB, 0.56171E+00_JPRB, 0.55445E+00_JPRB, 0.56149E+00_JPRB, 0.57655E+00_JPRB ,&
&  0.60851E+00_JPRB, 0.66548E+00_JPRB, 0.73717E+00_JPRB, 0.80211E+00_JPRB, 0.87823E+00_JPRB, 0.93887E+00_JPRB, 0.94922E+00_JPRB ,&
&  0.93869E+00_JPRB, 0.94103E+00_JPRB, 0.93938E+00_JPRB, 0.97269E+00_JPRB, 0.10069E+01_JPRB, 0.10560E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.12837E+01_JPRB, 0.14014E+01_JPRB, 0.15417E+01_JPRB, 0.17323E+01_JPRB, 0.19509E+01_JPRB, 0.21814E+01_JPRB ,&
&  0.15502E+01_JPRB, 0.83045E+00_JPRB, 0.14039E+01_JPRB, 0.15315E+01_JPRB, 0.11845E+01_JPRB, 0.10521E+01_JPRB, 0.89327E+00_JPRB ,&
&  0.86036E+00_JPRB, 0.11377E+01_JPRB, 0.12715E+01_JPRB, 0.13987E+01_JPRB, 0.14478E+01_JPRB, 0.15649E+01_JPRB, 0.16554E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.17460E+01_JPRB, 0.17963E+01_JPRB, 0.18523E+01_JPRB, 0.19324E+01_JPRB, 0.20196E+01_JPRB, 0.21315E+01_JPRB ,&
&  0.22678E+01_JPRB, 0.23388E+01_JPRB, 0.23862E+01_JPRB, 0.20416E+01_JPRB, 0.21468E+01_JPRB, 0.20366E+01_JPRB, 0.19541E+01_JPRB ,&
&  0.18232E+01_JPRB, 0.11992E+01_JPRB, 0.10050E+01_JPRB, 0.12612E+01_JPRB, 0.12324E+01_JPRB, 0.97442E+00_JPRB, 0.10369E+01_JPRB ,&
&  0.92765E+00_JPRB, 0.82771E+00_JPRB, 0.81993E+00_JPRB, 0.83527E+00_JPRB, 0.80288E+00_JPRB, 0.79786E+00_JPRB, 0.80065E+00_JPRB ,&
&  0.83363E+00_JPRB, 0.82269E+00_JPRB, 0.80085E+00_JPRB, 0.81696E+00_JPRB, 0.84848E+00_JPRB, 0.89370E+00_JPRB, 0.93059E+00_JPRB ,&
&  0.97227E+00_JPRB, 0.10060E+01_JPRB, 0.10391E+01_JPRB, 0.10814E+01_JPRB, 0.11159E+01_JPRB, 0.10686E+01_JPRB, 0.10475E+01_JPRB ,&
&  0.95875E+00_JPRB, 0.92428E+00_JPRB, 0.88306E+00_JPRB, 0.85453E+00_JPRB, 0.81603E+00_JPRB, 0.82161E+00_JPRB, 0.84779E+00_JPRB ,&
&  0.87401E+00_JPRB, 0.87511E+00_JPRB, 0.79982E+00_JPRB, 0.66870E+00_JPRB, 0.59412E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.57498E+00_JPRB, 0.55962E+00_JPRB, 0.55105E+00_JPRB, 0.56549E+00_JPRB, 0.56400E+00_JPRB, 0.57221E+00_JPRB, 0.59187E+00_JPRB ,&
&  0.62333E+00_JPRB, 0.66494E+00_JPRB, 0.69891E+00_JPRB, 0.74741E+00_JPRB, 0.81346E+00_JPRB, 0.87614E+00_JPRB, 0.88817E+00_JPRB ,&
&  0.87976E+00_JPRB, 0.87790E+00_JPRB, 0.88824E+00_JPRB, 0.90514E+00_JPRB, 0.94343E+00_JPRB, 0.99068E+00_JPRB, 0.10421E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.12180E+01_JPRB, 0.13279E+01_JPRB, 0.14537E+01_JPRB, 0.16192E+01_JPRB, 0.18836E+01_JPRB, 0.22684E+01_JPRB ,&
&  0.33294E+01_JPRB, 0.34097E+01_JPRB, 0.12616E+01_JPRB, 0.15934E+01_JPRB, 0.14177E+01_JPRB, 0.12179E+01_JPRB, 0.97685E+00_JPRB ,&
&  0.90716E+00_JPRB, 0.10541E+01_JPRB, 0.11961E+01_JPRB, 0.13342E+01_JPRB, 0.13247E+01_JPRB, 0.14235E+01_JPRB, 0.15122E+01_JPRB ,&
&  0.16110E+01_JPRB, 0.16659E+01_JPRB, 0.16843E+01_JPRB, 0.16880E+01_JPRB, 0.17195E+01_JPRB, 0.17830E+01_JPRB, 0.18990E+01_JPRB ,&
&  0.20656E+01_JPRB, 0.22276E+01_JPRB, 0.23350E+01_JPRB, 0.21188E+01_JPRB, 0.21784E+01_JPRB, 0.19223E+01_JPRB, 0.17340E+01_JPRB ,&
&  0.14052E+01_JPRB, 0.12527E+01_JPRB, 0.12541E+01_JPRB, 0.12811E+01_JPRB, 0.11374E+01_JPRB, 0.10178E+01_JPRB, 0.10934E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.92002E+00_JPRB, 0.87810E+00_JPRB, 0.86023E+00_JPRB, 0.84909E+00_JPRB, 0.83497E+00_JPRB, 0.84558E+00_JPRB ,&
&  0.87474E+00_JPRB, 0.87867E+00_JPRB, 0.85645E+00_JPRB, 0.88303E+00_JPRB, 0.91389E+00_JPRB, 0.95107E+00_JPRB, 0.98564E+00_JPRB ,&
&  0.10294E+01_JPRB, 0.10805E+01_JPRB, 0.11201E+01_JPRB, 0.11445E+01_JPRB, 0.11054E+01_JPRB, 0.10612E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10183E+01_JPRB, 0.99613E+00_JPRB, 0.97123E+00_JPRB, 0.92340E+00_JPRB, 0.93672E+00_JPRB, 0.94036E+00_JPRB ,&
&  0.97351E+00_JPRB, 0.10004E+01_JPRB, 0.90821E+00_JPRB, 0.69575E+00_JPRB, 0.61074E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.61307E+00_JPRB, 0.59792E+00_JPRB, 0.59324E+00_JPRB, 0.59969E+00_JPRB, 0.60326E+00_JPRB, 0.59386E+00_JPRB, 0.59593E+00_JPRB ,&
&  0.60691E+00_JPRB, 0.62262E+00_JPRB, 0.62258E+00_JPRB, 0.65631E+00_JPRB, 0.73305E+00_JPRB, 0.81363E+00_JPRB, 0.82402E+00_JPRB ,&
&  0.81599E+00_JPRB, 0.80511E+00_JPRB, 0.80084E+00_JPRB, 0.81778E+00_JPRB, 0.83892E+00_JPRB, 0.89391E+00_JPRB, 0.95378E+00_JPRB ,&
&  0.10168E+01_JPRB, 0.10868E+01_JPRB, 0.11463E+01_JPRB, 0.11985E+01_JPRB, 0.13000E+01_JPRB, 0.14749E+01_JPRB, 0.19383E+01_JPRB ,&
&  0.27938E+01_JPRB, 0.52897E+01_JPRB, 0.15442E+01_JPRB, 0.15856E+01_JPRB, 0.15453E+01_JPRB, 0.13859E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.11776E+01_JPRB, 0.10771E+01_JPRB, 0.11774E+01_JPRB, 0.12698E+01_JPRB, 0.12004E+01_JPRB, 0.12342E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.14312E+01_JPRB, 0.15146E+01_JPRB, 0.15577E+01_JPRB, 0.15383E+01_JPRB, 0.15146E+01_JPRB, 0.15078E+01_JPRB, 0.15478E+01_JPRB ,&
&  0.16779E+01_JPRB, 0.19182E+01_JPRB, 0.21576E+01_JPRB, 0.21284E+01_JPRB, 0.22366E+01_JPRB, 0.20018E+01_JPRB, 0.18475E+01_JPRB ,&
&  0.15793E+01_JPRB, 0.15082E+01_JPRB, 0.15474E+01_JPRB, 0.14678E+01_JPRB, 0.12408E+01_JPRB, 0.11289E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.10216E+01_JPRB, 0.93547E+00_JPRB, 0.87470E+00_JPRB, 0.87316E+00_JPRB, 0.89146E+00_JPRB, 0.87656E+00_JPRB ,&
&  0.91168E+00_JPRB, 0.95460E+00_JPRB, 0.93560E+00_JPRB, 0.97110E+00_JPRB, 0.99635E+00_JPRB, 0.10156E+01_JPRB, 0.10319E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.11075E+01_JPRB, 0.11449E+01_JPRB, 0.11220E+01_JPRB, 0.10576E+01_JPRB, 0.10293E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11850E+01_JPRB, 0.10687E+01_JPRB, 0.99869E+00_JPRB, 0.99688E+00_JPRB, 0.97702E+00_JPRB, 0.97029E+00_JPRB ,&
&  0.99385E+00_JPRB, 0.10567E+01_JPRB, 0.10857E+01_JPRB, 0.78436E+00_JPRB, 0.67034E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.68679E+00_JPRB, 0.66348E+00_JPRB, 0.64212E+00_JPRB, 0.62368E+00_JPRB, 0.61256E+00_JPRB, 0.61749E+00_JPRB, 0.62209E+00_JPRB ,&
&  0.61750E+00_JPRB, 0.61897E+00_JPRB, 0.61411E+00_JPRB, 0.61899E+00_JPRB, 0.64265E+00_JPRB, 0.69139E+00_JPRB, 0.74351E+00_JPRB ,&
&  0.74668E+00_JPRB, 0.71755E+00_JPRB, 0.71147E+00_JPRB, 0.70618E+00_JPRB, 0.72588E+00_JPRB, 0.75600E+00_JPRB, 0.81503E+00_JPRB ,&
&  0.89768E+00_JPRB, 0.96798E+00_JPRB, 0.10004E+01_JPRB, 0.10103E+01_JPRB, 0.10420E+01_JPRB, 0.11058E+01_JPRB, 0.15135E+01_JPRB ,&
&  0.23910E+01_JPRB, 0.50358E+01_JPRB, 0.35488E+01_JPRB, 0.24529E+01_JPRB, 0.16985E+01_JPRB, 0.14438E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.15505E+01_JPRB, 0.14857E+01_JPRB, 0.14367E+01_JPRB, 0.12389E+01_JPRB, 0.11147E+01_JPRB, 0.10610E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.12125E+01_JPRB, 0.13126E+01_JPRB, 0.13773E+01_JPRB, 0.13865E+01_JPRB, 0.13531E+01_JPRB, 0.13302E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.14111E+01_JPRB, 0.17699E+01_JPRB, 0.21028E+01_JPRB, 0.22885E+01_JPRB, 0.21500E+01_JPRB, 0.20559E+01_JPRB ,&
&  0.20360E+01_JPRB, 0.18845E+01_JPRB, 0.17995E+01_JPRB, 0.16072E+01_JPRB, 0.13320E+01_JPRB, 0.13583E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.10863E+01_JPRB, 0.10095E+01_JPRB, 0.92066E+00_JPRB, 0.92780E+00_JPRB, 0.92927E+00_JPRB, 0.90596E+00_JPRB ,&
&  0.94101E+00_JPRB, 0.98381E+00_JPRB, 0.97083E+00_JPRB, 0.98377E+00_JPRB, 0.99560E+00_JPRB, 0.10199E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10588E+01_JPRB, 0.11101E+01_JPRB, 0.10591E+01_JPRB, 0.10025E+01_JPRB, 0.99211E+00_JPRB, 0.98015E+00_JPRB, 0.99217E+00_JPRB ,&
&  0.10461E+01_JPRB, 0.11337E+01_JPRB, 0.10905E+01_JPRB, 0.10807E+01_JPRB, 0.10294E+01_JPRB, 0.10154E+01_JPRB, 0.95548E+00_JPRB ,&
&  0.94264E+00_JPRB, 0.91905E+00_JPRB, 0.92681E+00_JPRB, 0.85017E+00_JPRB, 0.74995E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.74028E+00_JPRB, 0.71387E+00_JPRB, 0.66046E+00_JPRB, 0.61697E+00_JPRB, 0.61052E+00_JPRB, 0.61679E+00_JPRB, 0.62783E+00_JPRB ,&
&  0.63224E+00_JPRB, 0.63316E+00_JPRB, 0.63015E+00_JPRB, 0.62342E+00_JPRB, 0.62740E+00_JPRB, 0.61413E+00_JPRB, 0.63772E+00_JPRB ,&
&  0.66507E+00_JPRB, 0.66766E+00_JPRB, 0.67723E+00_JPRB, 0.65283E+00_JPRB, 0.64977E+00_JPRB, 0.64688E+00_JPRB, 0.67823E+00_JPRB ,&
&  0.75087E+00_JPRB, 0.84307E+00_JPRB, 0.89947E+00_JPRB, 0.91646E+00_JPRB, 0.93347E+00_JPRB, 0.99712E+00_JPRB, 0.12634E+01_JPRB ,&
&  0.19952E+01_JPRB, 0.43194E+01_JPRB, 0.20451E+01_JPRB, 0.30058E+01_JPRB, 0.20338E+01_JPRB, 0.15614E+01_JPRB, 0.12789E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.16446E+01_JPRB, 0.13766E+01_JPRB, 0.11918E+01_JPRB, 0.10549E+01_JPRB, 0.10024E+01_JPRB, 0.98916E+00_JPRB ,&
&  0.10198E+01_JPRB, 0.11081E+01_JPRB, 0.11887E+01_JPRB, 0.12262E+01_JPRB, 0.12049E+01_JPRB, 0.11614E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.11508E+01_JPRB, 0.11552E+01_JPRB, 0.13089E+01_JPRB, 0.15094E+01_JPRB, 0.20651E+01_JPRB, 0.22809E+01_JPRB, 0.23487E+01_JPRB ,&
&  0.24140E+01_JPRB, 0.23219E+01_JPRB, 0.20335E+01_JPRB, 0.17700E+01_JPRB, 0.15304E+01_JPRB, 0.14563E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.11881E+01_JPRB, 0.11154E+01_JPRB, 0.10464E+01_JPRB, 0.97382E+00_JPRB, 0.94142E+00_JPRB, 0.93248E+00_JPRB, 0.91101E+00_JPRB ,&
&  0.94293E+00_JPRB, 0.98321E+00_JPRB, 0.97651E+00_JPRB, 0.96863E+00_JPRB, 0.97294E+00_JPRB, 0.99167E+00_JPRB, 0.10123E+01_JPRB ,&
&  0.10454E+01_JPRB, 0.11003E+01_JPRB, 0.10442E+01_JPRB, 0.92756E+00_JPRB, 0.92684E+00_JPRB, 0.96425E+00_JPRB, 0.98723E+00_JPRB ,&
&  0.10698E+01_JPRB, 0.11385E+01_JPRB, 0.11217E+01_JPRB, 0.10760E+01_JPRB, 0.10713E+01_JPRB, 0.11055E+01_JPRB, 0.99041E+00_JPRB ,&
&  0.93546E+00_JPRB, 0.90133E+00_JPRB, 0.87472E+00_JPRB, 0.84558E+00_JPRB, 0.79929E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.74574E+00_JPRB, 0.69871E+00_JPRB, 0.64384E+00_JPRB, 0.61870E+00_JPRB, 0.62321E+00_JPRB, 0.61094E+00_JPRB, 0.61930E+00_JPRB ,&
&  0.64073E+00_JPRB, 0.64264E+00_JPRB, 0.64215E+00_JPRB, 0.63194E+00_JPRB, 0.62200E+00_JPRB, 0.60428E+00_JPRB, 0.58908E+00_JPRB ,&
&  0.60522E+00_JPRB, 0.65267E+00_JPRB, 0.67125E+00_JPRB, 0.65868E+00_JPRB, 0.64577E+00_JPRB, 0.62759E+00_JPRB, 0.62492E+00_JPRB ,&
&  0.65479E+00_JPRB, 0.71668E+00_JPRB, 0.78384E+00_JPRB, 0.82083E+00_JPRB, 0.84277E+00_JPRB, 0.87413E+00_JPRB, 0.10439E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.42685E+01_JPRB, 0.18003E+01_JPRB, 0.31716E+01_JPRB, 0.23043E+01_JPRB, 0.17853E+01_JPRB, 0.13493E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.14092E+01_JPRB, 0.12810E+01_JPRB, 0.11722E+01_JPRB, 0.10715E+01_JPRB, 0.10085E+01_JPRB, 0.99058E+00_JPRB ,&
&  0.97699E+00_JPRB, 0.97629E+00_JPRB, 0.10029E+01_JPRB, 0.10299E+01_JPRB, 0.10335E+01_JPRB, 0.10146E+01_JPRB, 0.99944E+00_JPRB ,&
&  0.99934E+00_JPRB, 0.10066E+01_JPRB, 0.10908E+01_JPRB, 0.12728E+01_JPRB, 0.16200E+01_JPRB, 0.21434E+01_JPRB, 0.27355E+01_JPRB ,&
&  0.29959E+01_JPRB, 0.28560E+01_JPRB, 0.23754E+01_JPRB, 0.20323E+01_JPRB, 0.17887E+01_JPRB, 0.16656E+01_JPRB, 0.14764E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.12179E+01_JPRB, 0.11302E+01_JPRB, 0.10686E+01_JPRB, 0.10075E+01_JPRB, 0.98604E+00_JPRB, 0.95926E+00_JPRB ,&
&  0.96326E+00_JPRB, 0.10028E+01_JPRB, 0.99648E+00_JPRB, 0.97626E+00_JPRB, 0.94765E+00_JPRB, 0.95045E+00_JPRB, 0.95904E+00_JPRB ,&
&  0.98822E+00_JPRB, 0.10533E+01_JPRB, 0.10463E+01_JPRB, 0.94059E+00_JPRB, 0.90899E+00_JPRB, 0.94048E+00_JPRB, 0.97996E+00_JPRB ,&
&  0.10847E+01_JPRB, 0.11723E+01_JPRB, 0.11759E+01_JPRB, 0.11732E+01_JPRB, 0.11637E+01_JPRB, 0.12290E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.96736E+00_JPRB, 0.92162E+00_JPRB, 0.88303E+00_JPRB, 0.82975E+00_JPRB, 0.80353E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.76828E+00_JPRB, 0.70248E+00_JPRB, 0.66221E+00_JPRB, 0.63328E+00_JPRB, 0.63182E+00_JPRB, 0.62430E+00_JPRB, 0.62124E+00_JPRB ,&
&  0.64000E+00_JPRB, 0.65230E+00_JPRB, 0.66562E+00_JPRB, 0.65371E+00_JPRB, 0.63038E+00_JPRB, 0.62290E+00_JPRB, 0.60888E+00_JPRB ,&
&  0.59789E+00_JPRB, 0.62963E+00_JPRB, 0.66291E+00_JPRB, 0.67361E+00_JPRB, 0.66133E+00_JPRB, 0.63108E+00_JPRB, 0.61674E+00_JPRB ,&
&  0.62498E+00_JPRB, 0.64640E+00_JPRB, 0.68226E+00_JPRB, 0.72188E+00_JPRB, 0.75308E+00_JPRB, 0.77986E+00_JPRB, 0.83364E+00_JPRB ,&
&  0.10606E+01_JPRB, 0.15077E+01_JPRB, 0.24484E+01_JPRB, 0.30474E+01_JPRB, 0.24405E+01_JPRB, 0.18881E+01_JPRB, 0.15089E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.13008E+01_JPRB, 0.12190E+01_JPRB, 0.11287E+01_JPRB, 0.10486E+01_JPRB, 0.98781E+00_JPRB, 0.96535E+00_JPRB ,&
&  0.93314E+00_JPRB, 0.90709E+00_JPRB, 0.88537E+00_JPRB, 0.87687E+00_JPRB, 0.87249E+00_JPRB, 0.86927E+00_JPRB, 0.86712E+00_JPRB ,&
&  0.87676E+00_JPRB, 0.89022E+00_JPRB, 0.93001E+00_JPRB, 0.10685E+01_JPRB, 0.14347E+01_JPRB, 0.19544E+01_JPRB, 0.27795E+01_JPRB ,&
&  0.34059E+01_JPRB, 0.31345E+01_JPRB, 0.26602E+01_JPRB, 0.22837E+01_JPRB, 0.20141E+01_JPRB, 0.18632E+01_JPRB, 0.17054E+01_JPRB ,&
&  0.15637E+01_JPRB, 0.14137E+01_JPRB, 0.12880E+01_JPRB, 0.12039E+01_JPRB, 0.11049E+01_JPRB, 0.10546E+01_JPRB, 0.10482E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.10156E+01_JPRB, 0.10112E+01_JPRB, 0.10333E+01_JPRB, 0.99907E+00_JPRB, 0.96461E+00_JPRB, 0.94017E+00_JPRB ,&
&  0.93736E+00_JPRB, 0.99162E+00_JPRB, 0.10228E+01_JPRB, 0.96026E+00_JPRB, 0.94255E+00_JPRB, 0.96033E+00_JPRB, 0.99764E+00_JPRB ,&
&  0.10679E+01_JPRB, 0.11562E+01_JPRB, 0.12063E+01_JPRB, 0.12535E+01_JPRB, 0.11949E+01_JPRB, 0.12655E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.96030E+00_JPRB, 0.92391E+00_JPRB, 0.88204E+00_JPRB, 0.84647E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.88122E+00_JPRB, 0.79524E+00_JPRB, 0.72576E+00_JPRB, 0.68525E+00_JPRB, 0.65281E+00_JPRB, 0.63772E+00_JPRB, 0.63042E+00_JPRB ,&
&  0.64018E+00_JPRB, 0.65045E+00_JPRB, 0.66613E+00_JPRB, 0.66319E+00_JPRB, 0.64237E+00_JPRB, 0.63884E+00_JPRB, 0.64941E+00_JPRB ,&
&  0.64145E+00_JPRB, 0.64578E+00_JPRB, 0.65806E+00_JPRB, 0.65064E+00_JPRB, 0.63795E+00_JPRB, 0.62281E+00_JPRB, 0.60832E+00_JPRB ,&
&  0.61190E+00_JPRB, 0.61854E+00_JPRB, 0.63322E+00_JPRB, 0.65828E+00_JPRB, 0.69213E+00_JPRB, 0.72352E+00_JPRB, 0.74925E+00_JPRB ,&
&  0.81808E+00_JPRB, 0.15402E+01_JPRB, 0.22183E+01_JPRB, 0.26265E+01_JPRB, 0.23965E+01_JPRB, 0.18879E+01_JPRB, 0.16099E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.12396E+01_JPRB, 0.11570E+01_JPRB, 0.10878E+01_JPRB, 0.10090E+01_JPRB, 0.96194E+00_JPRB, 0.94182E+00_JPRB ,&
&  0.91384E+00_JPRB, 0.87142E+00_JPRB, 0.82564E+00_JPRB, 0.78883E+00_JPRB, 0.77315E+00_JPRB, 0.77250E+00_JPRB, 0.77710E+00_JPRB ,&
&  0.78512E+00_JPRB, 0.79779E+00_JPRB, 0.82569E+00_JPRB, 0.88436E+00_JPRB, 0.11886E+01_JPRB, 0.13687E+01_JPRB, 0.22596E+01_JPRB ,&
&  0.27636E+01_JPRB, 0.24280E+01_JPRB, 0.22101E+01_JPRB, 0.20595E+01_JPRB, 0.19038E+01_JPRB, 0.17607E+01_JPRB, 0.16772E+01_JPRB ,&
&  0.15893E+01_JPRB, 0.14814E+01_JPRB, 0.13890E+01_JPRB, 0.13228E+01_JPRB, 0.12534E+01_JPRB, 0.11996E+01_JPRB, 0.11277E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.10574E+01_JPRB, 0.10388E+01_JPRB, 0.10492E+01_JPRB, 0.10574E+01_JPRB, 0.10195E+01_JPRB, 0.98062E+00_JPRB ,&
&  0.93979E+00_JPRB, 0.94325E+00_JPRB, 0.98832E+00_JPRB, 0.95263E+00_JPRB, 0.95548E+00_JPRB, 0.98719E+00_JPRB, 0.10127E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.11109E+01_JPRB, 0.11972E+01_JPRB, 0.12676E+01_JPRB, 0.13035E+01_JPRB, 0.13552E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.10083E+01_JPRB, 0.94460E+00_JPRB, 0.90001E+00_JPRB, 0.92264E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.94381E+00_JPRB, 0.88504E+00_JPRB, 0.81903E+00_JPRB, 0.77243E+00_JPRB, 0.73362E+00_JPRB, 0.70462E+00_JPRB, 0.69624E+00_JPRB ,&
&  0.69027E+00_JPRB, 0.67990E+00_JPRB, 0.68444E+00_JPRB, 0.68156E+00_JPRB, 0.67592E+00_JPRB, 0.66588E+00_JPRB, 0.67357E+00_JPRB ,&
&  0.68092E+00_JPRB, 0.67776E+00_JPRB, 0.67118E+00_JPRB, 0.66091E+00_JPRB, 0.64926E+00_JPRB, 0.63421E+00_JPRB, 0.62424E+00_JPRB ,&
&  0.61756E+00_JPRB, 0.61810E+00_JPRB, 0.62506E+00_JPRB, 0.63885E+00_JPRB, 0.66462E+00_JPRB, 0.69799E+00_JPRB, 0.73050E+00_JPRB ,&
&  0.74031E+00_JPRB, 0.10237E+01_JPRB, 0.14043E+01_JPRB, 0.20520E+01_JPRB, 0.22207E+01_JPRB, 0.18906E+01_JPRB, 0.16029E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.12326E+01_JPRB, 0.11299E+01_JPRB, 0.10577E+01_JPRB, 0.98418E+00_JPRB, 0.94180E+00_JPRB, 0.90836E+00_JPRB ,&
&  0.87611E+00_JPRB, 0.83812E+00_JPRB, 0.79326E+00_JPRB, 0.76080E+00_JPRB, 0.74751E+00_JPRB, 0.74725E+00_JPRB, 0.74250E+00_JPRB ,&
&  0.73750E+00_JPRB, 0.74584E+00_JPRB, 0.75384E+00_JPRB, 0.77093E+00_JPRB, 0.93087E+00_JPRB, 0.11127E+01_JPRB, 0.14855E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.14205E+01_JPRB, 0.14080E+01_JPRB, 0.13956E+01_JPRB, 0.14127E+01_JPRB, 0.14216E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14651E+01_JPRB, 0.14495E+01_JPRB, 0.14104E+01_JPRB, 0.13792E+01_JPRB, 0.13441E+01_JPRB, 0.13141E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.11948E+01_JPRB, 0.11497E+01_JPRB, 0.11203E+01_JPRB, 0.10841E+01_JPRB, 0.10707E+01_JPRB, 0.10467E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.96109E+00_JPRB, 0.92213E+00_JPRB, 0.94363E+00_JPRB, 0.95551E+00_JPRB, 0.95424E+00_JPRB, 0.95099E+00_JPRB, 0.96978E+00_JPRB ,&
&  0.10082E+01_JPRB, 0.11008E+01_JPRB, 0.11549E+01_JPRB, 0.12783E+01_JPRB, 0.13159E+01_JPRB, 0.13359E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.11624E+01_JPRB, 0.10651E+01_JPRB, 0.97393E+00_JPRB, 0.92561E+00_JPRB, 0.96899E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.10544E+01_JPRB, 0.99060E+00_JPRB, 0.93086E+00_JPRB, 0.85925E+00_JPRB, 0.80277E+00_JPRB, 0.77510E+00_JPRB, 0.75364E+00_JPRB ,&
&  0.72861E+00_JPRB, 0.70623E+00_JPRB, 0.70111E+00_JPRB, 0.70379E+00_JPRB, 0.70470E+00_JPRB, 0.70192E+00_JPRB, 0.69197E+00_JPRB ,&
&  0.68875E+00_JPRB, 0.69278E+00_JPRB, 0.68633E+00_JPRB, 0.67480E+00_JPRB, 0.66201E+00_JPRB, 0.65112E+00_JPRB, 0.64273E+00_JPRB ,&
&  0.63789E+00_JPRB, 0.64044E+00_JPRB, 0.64218E+00_JPRB, 0.65282E+00_JPRB, 0.67257E+00_JPRB, 0.69710E+00_JPRB, 0.72942E+00_JPRB ,&
&  0.74241E+00_JPRB, 0.72431E+00_JPRB, 0.97819E+00_JPRB, 0.13328E+01_JPRB, 0.16235E+01_JPRB, 0.16512E+01_JPRB, 0.15119E+01_JPRB ,&
&  0.13881E+01_JPRB, 0.12533E+01_JPRB, 0.11474E+01_JPRB, 0.10606E+01_JPRB, 0.98607E+00_JPRB, 0.92716E+00_JPRB, 0.88431E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.80962E+00_JPRB, 0.77835E+00_JPRB, 0.74997E+00_JPRB, 0.73979E+00_JPRB, 0.73098E+00_JPRB, 0.72114E+00_JPRB ,&
&  0.71375E+00_JPRB, 0.71212E+00_JPRB, 0.71322E+00_JPRB, 0.71850E+00_JPRB, 0.74120E+00_JPRB, 0.80558E+00_JPRB, 0.91305E+00_JPRB ,&
&  0.96672E+00_JPRB, 0.99962E+00_JPRB, 0.10056E+01_JPRB, 0.10026E+01_JPRB, 0.10182E+01_JPRB, 0.10540E+01_JPRB, 0.11147E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.12467E+01_JPRB, 0.12871E+01_JPRB, 0.13133E+01_JPRB, 0.13099E+01_JPRB, 0.12884E+01_JPRB, 0.12536E+01_JPRB ,&
&  0.12165E+01_JPRB, 0.11627E+01_JPRB, 0.11134E+01_JPRB, 0.10752E+01_JPRB, 0.10480E+01_JPRB, 0.10282E+01_JPRB, 0.10014E+01_JPRB ,&
&  0.96650E+00_JPRB, 0.92948E+00_JPRB, 0.92129E+00_JPRB, 0.93016E+00_JPRB, 0.93752E+00_JPRB, 0.93338E+00_JPRB, 0.94210E+00_JPRB ,&
&  0.95285E+00_JPRB, 0.99376E+00_JPRB, 0.10243E+01_JPRB, 0.12314E+01_JPRB, 0.12736E+01_JPRB, 0.12307E+01_JPRB, 0.11861E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.10647E+01_JPRB, 0.10047E+01_JPRB, 0.96206E+00_JPRB, 0.10585E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.97766E+00_JPRB, 0.95996E+00_JPRB, 0.91334E+00_JPRB, 0.85670E+00_JPRB, 0.80410E+00_JPRB, 0.76035E+00_JPRB, 0.73446E+00_JPRB ,&
&  0.71408E+00_JPRB, 0.69997E+00_JPRB, 0.70198E+00_JPRB, 0.71230E+00_JPRB, 0.72483E+00_JPRB, 0.72734E+00_JPRB, 0.72165E+00_JPRB ,&
&  0.71073E+00_JPRB, 0.69440E+00_JPRB, 0.68392E+00_JPRB, 0.67724E+00_JPRB, 0.66726E+00_JPRB, 0.66258E+00_JPRB, 0.66020E+00_JPRB ,&
&  0.66277E+00_JPRB, 0.66730E+00_JPRB, 0.66972E+00_JPRB, 0.67847E+00_JPRB, 0.69369E+00_JPRB, 0.71742E+00_JPRB, 0.74304E+00_JPRB ,&
&  0.75019E+00_JPRB, 0.65776E+00_JPRB, 0.75648E+00_JPRB, 0.97518E+00_JPRB, 0.12111E+01_JPRB, 0.13359E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.12185E+01_JPRB, 0.11427E+01_JPRB, 0.10572E+01_JPRB, 0.97775E+00_JPRB, 0.91103E+00_JPRB, 0.85144E+00_JPRB ,&
&  0.80546E+00_JPRB, 0.77300E+00_JPRB, 0.75409E+00_JPRB, 0.74052E+00_JPRB, 0.72900E+00_JPRB, 0.70862E+00_JPRB, 0.69543E+00_JPRB ,&
&  0.69237E+00_JPRB, 0.69185E+00_JPRB, 0.69395E+00_JPRB, 0.69726E+00_JPRB, 0.71206E+00_JPRB, 0.73548E+00_JPRB, 0.75611E+00_JPRB ,&
&  0.77429E+00_JPRB, 0.79575E+00_JPRB, 0.80779E+00_JPRB, 0.80763E+00_JPRB, 0.80785E+00_JPRB, 0.84287E+00_JPRB, 0.87529E+00_JPRB ,&
&  0.91814E+00_JPRB, 0.98207E+00_JPRB, 0.10578E+01_JPRB, 0.11029E+01_JPRB, 0.11089E+01_JPRB, 0.10862E+01_JPRB, 0.10453E+01_JPRB ,&
&  0.10184E+01_JPRB, 0.98756E+00_JPRB, 0.95769E+00_JPRB, 0.93540E+00_JPRB, 0.92172E+00_JPRB, 0.91653E+00_JPRB, 0.91183E+00_JPRB ,&
&  0.90661E+00_JPRB, 0.89546E+00_JPRB, 0.89127E+00_JPRB, 0.90863E+00_JPRB, 0.92928E+00_JPRB, 0.93666E+00_JPRB, 0.93405E+00_JPRB ,&
&  0.92627E+00_JPRB, 0.92811E+00_JPRB, 0.94357E+00_JPRB, 0.10520E+01_JPRB, 0.10513E+01_JPRB, 0.10588E+01_JPRB, 0.10420E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.98949E+00_JPRB, 0.94661E+00_JPRB, 0.91166E+00_JPRB, 0.90952E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.84765E+00_JPRB, 0.87829E+00_JPRB, 0.87458E+00_JPRB, 0.83716E+00_JPRB, 0.79718E+00_JPRB, 0.76976E+00_JPRB, 0.75132E+00_JPRB ,&
&  0.74371E+00_JPRB, 0.73478E+00_JPRB, 0.73523E+00_JPRB, 0.74509E+00_JPRB, 0.75642E+00_JPRB, 0.75917E+00_JPRB, 0.74293E+00_JPRB ,&
&  0.72142E+00_JPRB, 0.70417E+00_JPRB, 0.69532E+00_JPRB, 0.69148E+00_JPRB, 0.68638E+00_JPRB, 0.68376E+00_JPRB, 0.69193E+00_JPRB ,&
&  0.70172E+00_JPRB, 0.71370E+00_JPRB, 0.72669E+00_JPRB, 0.73752E+00_JPRB, 0.74592E+00_JPRB, 0.75405E+00_JPRB, 0.77417E+00_JPRB ,&
&  0.78631E+00_JPRB, 0.61827E+00_JPRB, 0.65403E+00_JPRB, 0.79529E+00_JPRB, 0.94139E+00_JPRB, 0.11108E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.11368E+01_JPRB, 0.10794E+01_JPRB, 0.10154E+01_JPRB, 0.94784E+00_JPRB, 0.88846E+00_JPRB, 0.83368E+00_JPRB ,&
&  0.79525E+00_JPRB, 0.76597E+00_JPRB, 0.74749E+00_JPRB, 0.72773E+00_JPRB, 0.71476E+00_JPRB, 0.70514E+00_JPRB, 0.69668E+00_JPRB ,&
&  0.68992E+00_JPRB, 0.68222E+00_JPRB, 0.68221E+00_JPRB, 0.68682E+00_JPRB, 0.68620E+00_JPRB, 0.69240E+00_JPRB, 0.69254E+00_JPRB ,&
&  0.70589E+00_JPRB, 0.72508E+00_JPRB, 0.73230E+00_JPRB, 0.73174E+00_JPRB, 0.73489E+00_JPRB, 0.74706E+00_JPRB, 0.76433E+00_JPRB ,&
&  0.78080E+00_JPRB, 0.81677E+00_JPRB, 0.85283E+00_JPRB, 0.88904E+00_JPRB, 0.91145E+00_JPRB, 0.91317E+00_JPRB, 0.90109E+00_JPRB ,&
&  0.90185E+00_JPRB, 0.89392E+00_JPRB, 0.88198E+00_JPRB, 0.86599E+00_JPRB, 0.84765E+00_JPRB, 0.83561E+00_JPRB, 0.83345E+00_JPRB ,&
&  0.83912E+00_JPRB, 0.85323E+00_JPRB, 0.86396E+00_JPRB, 0.88621E+00_JPRB, 0.89884E+00_JPRB, 0.89698E+00_JPRB, 0.89525E+00_JPRB ,&
&  0.89327E+00_JPRB, 0.88872E+00_JPRB, 0.88383E+00_JPRB, 0.92916E+00_JPRB, 0.10004E+01_JPRB, 0.10056E+01_JPRB, 0.99062E+00_JPRB ,&
&  0.96570E+00_JPRB, 0.92023E+00_JPRB, 0.89497E+00_JPRB, 0.72360E+00_JPRB, 0.80755E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.79756E+00_JPRB, 0.82602E+00_JPRB, 0.83090E+00_JPRB, 0.81489E+00_JPRB, 0.78828E+00_JPRB, 0.77294E+00_JPRB, 0.76352E+00_JPRB ,&
&  0.75753E+00_JPRB, 0.75306E+00_JPRB, 0.76177E+00_JPRB, 0.76773E+00_JPRB, 0.76885E+00_JPRB, 0.76005E+00_JPRB, 0.75094E+00_JPRB ,&
&  0.73683E+00_JPRB, 0.71949E+00_JPRB, 0.70663E+00_JPRB, 0.69545E+00_JPRB, 0.69401E+00_JPRB, 0.70190E+00_JPRB, 0.71807E+00_JPRB ,&
&  0.73895E+00_JPRB, 0.75365E+00_JPRB, 0.76566E+00_JPRB, 0.77833E+00_JPRB, 0.78098E+00_JPRB, 0.78161E+00_JPRB, 0.78546E+00_JPRB ,&
&  0.79259E+00_JPRB, 0.60842E+00_JPRB, 0.65324E+00_JPRB, 0.71797E+00_JPRB, 0.78562E+00_JPRB, 0.88878E+00_JPRB, 0.97524E+00_JPRB ,&
&  0.10063E+01_JPRB, 0.97209E+00_JPRB, 0.94205E+00_JPRB, 0.90630E+00_JPRB, 0.86720E+00_JPRB, 0.83264E+00_JPRB, 0.79507E+00_JPRB ,&
&  0.76627E+00_JPRB, 0.74643E+00_JPRB, 0.73698E+00_JPRB, 0.72826E+00_JPRB, 0.71856E+00_JPRB, 0.70624E+00_JPRB, 0.69206E+00_JPRB ,&
&  0.67772E+00_JPRB, 0.66976E+00_JPRB, 0.66460E+00_JPRB, 0.66050E+00_JPRB, 0.66195E+00_JPRB, 0.66632E+00_JPRB, 0.66998E+00_JPRB ,&
&  0.67878E+00_JPRB, 0.68610E+00_JPRB, 0.68869E+00_JPRB, 0.68741E+00_JPRB, 0.68537E+00_JPRB, 0.68641E+00_JPRB, 0.70102E+00_JPRB ,&
&  0.70961E+00_JPRB, 0.72544E+00_JPRB, 0.74540E+00_JPRB, 0.77886E+00_JPRB, 0.80778E+00_JPRB, 0.81826E+00_JPRB, 0.82092E+00_JPRB ,&
&  0.82145E+00_JPRB, 0.81286E+00_JPRB, 0.79699E+00_JPRB, 0.77994E+00_JPRB, 0.76757E+00_JPRB, 0.76371E+00_JPRB, 0.75782E+00_JPRB ,&
&  0.75938E+00_JPRB, 0.76640E+00_JPRB, 0.77167E+00_JPRB, 0.78534E+00_JPRB, 0.80521E+00_JPRB, 0.81610E+00_JPRB, 0.80826E+00_JPRB ,&
&  0.80320E+00_JPRB, 0.80531E+00_JPRB, 0.80548E+00_JPRB, 0.85932E+00_JPRB, 0.88372E+00_JPRB, 0.89272E+00_JPRB, 0.88424E+00_JPRB ,&
&  0.87305E+00_JPRB, 0.86050E+00_JPRB, 0.64455E+00_JPRB, 0.74325E+00_JPRB, 0.76545E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.78619E+00_JPRB, 0.78032E+00_JPRB, 0.77140E+00_JPRB, 0.77157E+00_JPRB, 0.77204E+00_JPRB, 0.76810E+00_JPRB, 0.76525E+00_JPRB ,&
&  0.76171E+00_JPRB, 0.75755E+00_JPRB, 0.76552E+00_JPRB, 0.77322E+00_JPRB, 0.77461E+00_JPRB, 0.76876E+00_JPRB, 0.76248E+00_JPRB ,&
&  0.75770E+00_JPRB, 0.74924E+00_JPRB, 0.74041E+00_JPRB, 0.73665E+00_JPRB, 0.73898E+00_JPRB, 0.75282E+00_JPRB, 0.77406E+00_JPRB ,&
&  0.78996E+00_JPRB, 0.78929E+00_JPRB, 0.79291E+00_JPRB, 0.80402E+00_JPRB, 0.80663E+00_JPRB, 0.80839E+00_JPRB, 0.81558E+00_JPRB ,&
&  0.69704E+00_JPRB, 0.57280E+00_JPRB, 0.63753E+00_JPRB, 0.69665E+00_JPRB, 0.74385E+00_JPRB, 0.79942E+00_JPRB, 0.86642E+00_JPRB ,&
&  0.88987E+00_JPRB, 0.87674E+00_JPRB, 0.86169E+00_JPRB, 0.84102E+00_JPRB, 0.81584E+00_JPRB, 0.79856E+00_JPRB, 0.78479E+00_JPRB ,&
&  0.78144E+00_JPRB, 0.77040E+00_JPRB, 0.75322E+00_JPRB, 0.73916E+00_JPRB, 0.72718E+00_JPRB, 0.71162E+00_JPRB, 0.69649E+00_JPRB ,&
&  0.68083E+00_JPRB, 0.66738E+00_JPRB, 0.66309E+00_JPRB, 0.66090E+00_JPRB, 0.66317E+00_JPRB, 0.66314E+00_JPRB, 0.65830E+00_JPRB ,&
&  0.65470E+00_JPRB, 0.65605E+00_JPRB, 0.66483E+00_JPRB, 0.67252E+00_JPRB, 0.67439E+00_JPRB, 0.67198E+00_JPRB, 0.67969E+00_JPRB ,&
&  0.68466E+00_JPRB, 0.69544E+00_JPRB, 0.70963E+00_JPRB, 0.73412E+00_JPRB, 0.75303E+00_JPRB, 0.75830E+00_JPRB, 0.76073E+00_JPRB ,&
&  0.76599E+00_JPRB, 0.76041E+00_JPRB, 0.74486E+00_JPRB, 0.73381E+00_JPRB, 0.72825E+00_JPRB, 0.72067E+00_JPRB, 0.71783E+00_JPRB ,&
&  0.72270E+00_JPRB, 0.72553E+00_JPRB, 0.72193E+00_JPRB, 0.72566E+00_JPRB, 0.73601E+00_JPRB, 0.74987E+00_JPRB, 0.76219E+00_JPRB ,&
&  0.75293E+00_JPRB, 0.76087E+00_JPRB, 0.76205E+00_JPRB, 0.78365E+00_JPRB, 0.81040E+00_JPRB, 0.81020E+00_JPRB, 0.80607E+00_JPRB ,&
&  0.79941E+00_JPRB, 0.77462E+00_JPRB, 0.77452E+00_JPRB, 0.77414E+00_JPRB, 0.77674E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.73445E+00_JPRB, 0.72624E+00_JPRB, 0.72044E+00_JPRB, 0.72880E+00_JPRB, 0.74356E+00_JPRB, 0.75397E+00_JPRB, 0.75323E+00_JPRB ,&
&  0.75232E+00_JPRB, 0.75630E+00_JPRB, 0.76418E+00_JPRB, 0.76659E+00_JPRB, 0.76008E+00_JPRB, 0.74396E+00_JPRB, 0.73571E+00_JPRB ,&
&  0.73362E+00_JPRB, 0.73454E+00_JPRB, 0.73446E+00_JPRB, 0.73881E+00_JPRB, 0.75843E+00_JPRB, 0.78900E+00_JPRB, 0.81798E+00_JPRB ,&
&  0.83229E+00_JPRB, 0.82686E+00_JPRB, 0.82063E+00_JPRB, 0.81552E+00_JPRB, 0.81428E+00_JPRB, 0.81913E+00_JPRB, 0.82007E+00_JPRB ,&
&  0.81216E+00_JPRB, 0.66654E+00_JPRB, 0.72270E+00_JPRB, 0.74402E+00_JPRB, 0.76893E+00_JPRB, 0.79864E+00_JPRB, 0.83817E+00_JPRB ,&
&  0.84699E+00_JPRB, 0.83693E+00_JPRB, 0.81617E+00_JPRB, 0.80144E+00_JPRB, 0.78574E+00_JPRB, 0.78034E+00_JPRB, 0.77693E+00_JPRB ,&
&  0.77491E+00_JPRB, 0.77176E+00_JPRB, 0.74976E+00_JPRB, 0.71438E+00_JPRB, 0.69471E+00_JPRB, 0.68243E+00_JPRB, 0.67209E+00_JPRB ,&
&  0.66247E+00_JPRB, 0.64970E+00_JPRB, 0.63726E+00_JPRB, 0.62693E+00_JPRB, 0.62823E+00_JPRB, 0.62636E+00_JPRB, 0.61377E+00_JPRB ,&
&  0.60854E+00_JPRB, 0.60631E+00_JPRB, 0.61368E+00_JPRB, 0.62364E+00_JPRB, 0.62731E+00_JPRB, 0.62197E+00_JPRB, 0.63050E+00_JPRB ,&
&  0.65058E+00_JPRB, 0.66332E+00_JPRB, 0.67470E+00_JPRB, 0.68960E+00_JPRB, 0.69725E+00_JPRB, 0.69693E+00_JPRB, 0.69815E+00_JPRB ,&
&  0.70444E+00_JPRB, 0.70299E+00_JPRB, 0.69535E+00_JPRB, 0.68921E+00_JPRB, 0.69041E+00_JPRB, 0.69111E+00_JPRB, 0.70093E+00_JPRB ,&
&  0.71017E+00_JPRB, 0.71323E+00_JPRB, 0.70922E+00_JPRB, 0.71184E+00_JPRB, 0.71757E+00_JPRB, 0.72734E+00_JPRB, 0.73406E+00_JPRB ,&
&  0.74111E+00_JPRB, 0.73881E+00_JPRB, 0.72723E+00_JPRB, 0.73273E+00_JPRB, 0.74573E+00_JPRB, 0.74748E+00_JPRB, 0.73741E+00_JPRB ,&
&  0.72746E+00_JPRB, 0.71862E+00_JPRB, 0.72135E+00_JPRB, 0.72289E+00_JPRB, 0.72926E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.72155E+00_JPRB, 0.71796E+00_JPRB, 0.71715E+00_JPRB, 0.72859E+00_JPRB, 0.74632E+00_JPRB, 0.74996E+00_JPRB, 0.75096E+00_JPRB ,&
&  0.76273E+00_JPRB, 0.77145E+00_JPRB, 0.77067E+00_JPRB, 0.76895E+00_JPRB, 0.76298E+00_JPRB, 0.74921E+00_JPRB, 0.73862E+00_JPRB ,&
&  0.74385E+00_JPRB, 0.74941E+00_JPRB, 0.75171E+00_JPRB, 0.77576E+00_JPRB, 0.80751E+00_JPRB, 0.84510E+00_JPRB, 0.87056E+00_JPRB ,&
&  0.87952E+00_JPRB, 0.87209E+00_JPRB, 0.86831E+00_JPRB, 0.86299E+00_JPRB, 0.85763E+00_JPRB, 0.85935E+00_JPRB, 0.85383E+00_JPRB ,&
&  0.84386E+00_JPRB, 0.75307E+00_JPRB, 0.78307E+00_JPRB, 0.79254E+00_JPRB, 0.80829E+00_JPRB, 0.83264E+00_JPRB, 0.86217E+00_JPRB ,&
&  0.86353E+00_JPRB, 0.85821E+00_JPRB, 0.84717E+00_JPRB, 0.82565E+00_JPRB, 0.80812E+00_JPRB, 0.79809E+00_JPRB, 0.78109E+00_JPRB ,&
&  0.75372E+00_JPRB, 0.71679E+00_JPRB, 0.68406E+00_JPRB, 0.66177E+00_JPRB, 0.65164E+00_JPRB, 0.64897E+00_JPRB, 0.64434E+00_JPRB ,&
&  0.64225E+00_JPRB, 0.64058E+00_JPRB, 0.62985E+00_JPRB, 0.62018E+00_JPRB, 0.61020E+00_JPRB, 0.60355E+00_JPRB, 0.59046E+00_JPRB ,&
&  0.58214E+00_JPRB, 0.57113E+00_JPRB, 0.57027E+00_JPRB, 0.58978E+00_JPRB, 0.61004E+00_JPRB, 0.61402E+00_JPRB, 0.62647E+00_JPRB ,&
&  0.64315E+00_JPRB, 0.65000E+00_JPRB, 0.65138E+00_JPRB, 0.64864E+00_JPRB, 0.64157E+00_JPRB, 0.64135E+00_JPRB, 0.64927E+00_JPRB ,&
&  0.65616E+00_JPRB, 0.66599E+00_JPRB, 0.66923E+00_JPRB, 0.67369E+00_JPRB, 0.68020E+00_JPRB, 0.68513E+00_JPRB, 0.69604E+00_JPRB ,&
&  0.70164E+00_JPRB, 0.70579E+00_JPRB, 0.71735E+00_JPRB, 0.72881E+00_JPRB, 0.73357E+00_JPRB, 0.73591E+00_JPRB, 0.74495E+00_JPRB ,&
&  0.74838E+00_JPRB, 0.74645E+00_JPRB, 0.73842E+00_JPRB, 0.72757E+00_JPRB, 0.72154E+00_JPRB, 0.71055E+00_JPRB, 0.69242E+00_JPRB ,&
&  0.67559E+00_JPRB, 0.67383E+00_JPRB, 0.68362E+00_JPRB, 0.69418E+00_JPRB, 0.71188E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.70902E+00_JPRB, 0.72067E+00_JPRB, 0.73388E+00_JPRB, 0.74671E+00_JPRB, 0.76020E+00_JPRB, 0.76004E+00_JPRB, 0.76164E+00_JPRB ,&
&  0.78103E+00_JPRB, 0.78403E+00_JPRB, 0.78322E+00_JPRB, 0.77433E+00_JPRB, 0.76781E+00_JPRB, 0.76757E+00_JPRB, 0.75860E+00_JPRB ,&
&  0.75413E+00_JPRB, 0.76049E+00_JPRB, 0.77877E+00_JPRB, 0.80676E+00_JPRB, 0.83408E+00_JPRB, 0.87001E+00_JPRB, 0.89652E+00_JPRB ,&
&  0.90062E+00_JPRB, 0.88976E+00_JPRB, 0.88990E+00_JPRB, 0.89316E+00_JPRB, 0.89336E+00_JPRB, 0.87645E+00_JPRB, 0.87201E+00_JPRB ,&
&  0.85181E+00_JPRB, 0.86003E+00_JPRB, 0.73007E+00_JPRB, 0.79984E+00_JPRB, 0.84374E+00_JPRB, 0.87835E+00_JPRB, 0.88980E+00_JPRB ,&
&  0.88229E+00_JPRB, 0.86941E+00_JPRB, 0.84531E+00_JPRB, 0.81822E+00_JPRB, 0.80859E+00_JPRB, 0.78837E+00_JPRB, 0.76519E+00_JPRB ,&
&  0.71648E+00_JPRB, 0.65434E+00_JPRB, 0.62269E+00_JPRB, 0.61121E+00_JPRB, 0.60560E+00_JPRB, 0.60684E+00_JPRB, 0.61208E+00_JPRB ,&
&  0.61565E+00_JPRB, 0.61610E+00_JPRB, 0.60494E+00_JPRB, 0.58833E+00_JPRB, 0.56856E+00_JPRB, 0.55797E+00_JPRB, 0.55505E+00_JPRB ,&
&  0.55134E+00_JPRB, 0.53846E+00_JPRB, 0.54164E+00_JPRB, 0.57124E+00_JPRB, 0.59584E+00_JPRB, 0.61303E+00_JPRB, 0.62038E+00_JPRB ,&
&  0.62573E+00_JPRB, 0.61968E+00_JPRB, 0.61603E+00_JPRB, 0.60741E+00_JPRB, 0.58469E+00_JPRB, 0.57890E+00_JPRB, 0.60006E+00_JPRB ,&
&  0.62371E+00_JPRB, 0.63718E+00_JPRB, 0.64519E+00_JPRB, 0.65719E+00_JPRB, 0.66983E+00_JPRB, 0.67702E+00_JPRB, 0.68336E+00_JPRB ,&
&  0.68811E+00_JPRB, 0.69279E+00_JPRB, 0.70629E+00_JPRB, 0.72115E+00_JPRB, 0.72824E+00_JPRB, 0.72957E+00_JPRB, 0.72952E+00_JPRB ,&
&  0.72654E+00_JPRB, 0.71889E+00_JPRB, 0.70995E+00_JPRB, 0.70153E+00_JPRB, 0.68268E+00_JPRB, 0.65955E+00_JPRB, 0.64255E+00_JPRB ,&
&  0.63390E+00_JPRB, 0.64402E+00_JPRB, 0.65196E+00_JPRB, 0.66467E+00_JPRB, 0.69166E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.74849E+00_JPRB, 0.76262E+00_JPRB, 0.77413E+00_JPRB, 0.79520E+00_JPRB, 0.81434E+00_JPRB, 0.81163E+00_JPRB, 0.80480E+00_JPRB ,&
&  0.80006E+00_JPRB, 0.79559E+00_JPRB, 0.78836E+00_JPRB, 0.77867E+00_JPRB, 0.77211E+00_JPRB, 0.77149E+00_JPRB, 0.77212E+00_JPRB ,&
&  0.77373E+00_JPRB, 0.79376E+00_JPRB, 0.82658E+00_JPRB, 0.85219E+00_JPRB, 0.88016E+00_JPRB, 0.92325E+00_JPRB, 0.94353E+00_JPRB ,&
&  0.92684E+00_JPRB, 0.92584E+00_JPRB, 0.91502E+00_JPRB, 0.90053E+00_JPRB, 0.90244E+00_JPRB, 0.89323E+00_JPRB, 0.86519E+00_JPRB ,&
&  0.85899E+00_JPRB, 0.85272E+00_JPRB, 0.84861E+00_JPRB, 0.87680E+00_JPRB, 0.89907E+00_JPRB, 0.89938E+00_JPRB, 0.89659E+00_JPRB ,&
&  0.87455E+00_JPRB, 0.83748E+00_JPRB, 0.80191E+00_JPRB, 0.78451E+00_JPRB, 0.78248E+00_JPRB, 0.76244E+00_JPRB, 0.73436E+00_JPRB ,&
&  0.67686E+00_JPRB, 0.62966E+00_JPRB, 0.60354E+00_JPRB, 0.58799E+00_JPRB, 0.58297E+00_JPRB, 0.58029E+00_JPRB, 0.57581E+00_JPRB ,&
&  0.58108E+00_JPRB, 0.57583E+00_JPRB, 0.54797E+00_JPRB, 0.51876E+00_JPRB, 0.51377E+00_JPRB, 0.52244E+00_JPRB, 0.54188E+00_JPRB ,&
&  0.55685E+00_JPRB, 0.55225E+00_JPRB, 0.55896E+00_JPRB, 0.56641E+00_JPRB, 0.58337E+00_JPRB, 0.60023E+00_JPRB, 0.60169E+00_JPRB ,&
&  0.60207E+00_JPRB, 0.59725E+00_JPRB, 0.58448E+00_JPRB, 0.57619E+00_JPRB, 0.56524E+00_JPRB, 0.56112E+00_JPRB, 0.57281E+00_JPRB ,&
&  0.59022E+00_JPRB, 0.59979E+00_JPRB, 0.61057E+00_JPRB, 0.62402E+00_JPRB, 0.63490E+00_JPRB, 0.64336E+00_JPRB, 0.65019E+00_JPRB ,&
&  0.66051E+00_JPRB, 0.67526E+00_JPRB, 0.69011E+00_JPRB, 0.70308E+00_JPRB, 0.70321E+00_JPRB, 0.68926E+00_JPRB, 0.66404E+00_JPRB ,&
&  0.64709E+00_JPRB, 0.63374E+00_JPRB, 0.63048E+00_JPRB, 0.63583E+00_JPRB, 0.63126E+00_JPRB, 0.61220E+00_JPRB, 0.60285E+00_JPRB ,&
&  0.61385E+00_JPRB, 0.63774E+00_JPRB, 0.66054E+00_JPRB, 0.69171E+00_JPRB, 0.72453E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.78620E+00_JPRB, 0.81781E+00_JPRB, 0.84196E+00_JPRB, 0.85066E+00_JPRB, 0.84857E+00_JPRB, 0.83990E+00_JPRB, 0.81748E+00_JPRB ,&
&  0.79311E+00_JPRB, 0.77588E+00_JPRB, 0.75751E+00_JPRB, 0.73103E+00_JPRB, 0.70902E+00_JPRB, 0.70438E+00_JPRB, 0.71694E+00_JPRB ,&
&  0.76318E+00_JPRB, 0.80416E+00_JPRB, 0.82054E+00_JPRB, 0.83391E+00_JPRB, 0.88342E+00_JPRB, 0.92502E+00_JPRB, 0.93393E+00_JPRB ,&
&  0.92877E+00_JPRB, 0.93034E+00_JPRB, 0.89384E+00_JPRB, 0.86060E+00_JPRB, 0.86073E+00_JPRB, 0.86623E+00_JPRB, 0.84833E+00_JPRB ,&
&  0.87427E+00_JPRB, 0.89172E+00_JPRB, 0.88576E+00_JPRB, 0.86140E+00_JPRB, 0.85427E+00_JPRB, 0.84398E+00_JPRB, 0.83191E+00_JPRB ,&
&  0.80986E+00_JPRB, 0.76164E+00_JPRB, 0.73088E+00_JPRB, 0.72863E+00_JPRB, 0.72140E+00_JPRB, 0.71413E+00_JPRB, 0.70059E+00_JPRB ,&
&  0.66056E+00_JPRB, 0.62469E+00_JPRB, 0.59763E+00_JPRB, 0.57900E+00_JPRB, 0.56727E+00_JPRB, 0.55812E+00_JPRB, 0.54844E+00_JPRB ,&
&  0.54001E+00_JPRB, 0.51251E+00_JPRB, 0.48426E+00_JPRB, 0.47802E+00_JPRB, 0.48367E+00_JPRB, 0.49055E+00_JPRB, 0.51748E+00_JPRB ,&
&  0.54244E+00_JPRB, 0.54081E+00_JPRB, 0.53709E+00_JPRB, 0.53957E+00_JPRB, 0.54856E+00_JPRB, 0.54918E+00_JPRB, 0.53949E+00_JPRB ,&
&  0.53489E+00_JPRB, 0.53288E+00_JPRB, 0.52379E+00_JPRB, 0.51712E+00_JPRB, 0.51671E+00_JPRB, 0.52094E+00_JPRB, 0.53547E+00_JPRB ,&
&  0.54433E+00_JPRB, 0.54548E+00_JPRB, 0.55887E+00_JPRB, 0.57601E+00_JPRB, 0.58647E+00_JPRB, 0.59478E+00_JPRB, 0.60076E+00_JPRB ,&
&  0.61441E+00_JPRB, 0.63412E+00_JPRB, 0.64926E+00_JPRB, 0.65141E+00_JPRB, 0.62856E+00_JPRB, 0.60440E+00_JPRB, 0.58546E+00_JPRB ,&
&  0.57732E+00_JPRB, 0.58194E+00_JPRB, 0.58958E+00_JPRB, 0.59699E+00_JPRB, 0.60377E+00_JPRB, 0.60834E+00_JPRB, 0.60774E+00_JPRB ,&
&  0.62741E+00_JPRB, 0.65868E+00_JPRB, 0.68395E+00_JPRB, 0.71422E+00_JPRB, 0.74582E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.85278E+00_JPRB, 0.87037E+00_JPRB, 0.88211E+00_JPRB, 0.88274E+00_JPRB, 0.86715E+00_JPRB, 0.84040E+00_JPRB, 0.81754E+00_JPRB ,&
&  0.79141E+00_JPRB, 0.76606E+00_JPRB, 0.74197E+00_JPRB, 0.70916E+00_JPRB, 0.69144E+00_JPRB, 0.69470E+00_JPRB, 0.74347E+00_JPRB ,&
&  0.80866E+00_JPRB, 0.81343E+00_JPRB, 0.80718E+00_JPRB, 0.83218E+00_JPRB, 0.88431E+00_JPRB, 0.91587E+00_JPRB, 0.93659E+00_JPRB ,&
&  0.93913E+00_JPRB, 0.90591E+00_JPRB, 0.87272E+00_JPRB, 0.85935E+00_JPRB, 0.88287E+00_JPRB, 0.90085E+00_JPRB, 0.90612E+00_JPRB ,&
&  0.90180E+00_JPRB, 0.88076E+00_JPRB, 0.86195E+00_JPRB, 0.84606E+00_JPRB, 0.85027E+00_JPRB, 0.82517E+00_JPRB, 0.77258E+00_JPRB ,&
&  0.70625E+00_JPRB, 0.67968E+00_JPRB, 0.66779E+00_JPRB, 0.65306E+00_JPRB, 0.64209E+00_JPRB, 0.64223E+00_JPRB, 0.63312E+00_JPRB ,&
&  0.61966E+00_JPRB, 0.60388E+00_JPRB, 0.58866E+00_JPRB, 0.57072E+00_JPRB, 0.55659E+00_JPRB, 0.53666E+00_JPRB, 0.51799E+00_JPRB ,&
&  0.50069E+00_JPRB, 0.47934E+00_JPRB, 0.45963E+00_JPRB, 0.44571E+00_JPRB, 0.43942E+00_JPRB, 0.44957E+00_JPRB, 0.48938E+00_JPRB ,&
&  0.52079E+00_JPRB, 0.51797E+00_JPRB, 0.51581E+00_JPRB, 0.52015E+00_JPRB, 0.51350E+00_JPRB, 0.50364E+00_JPRB, 0.49849E+00_JPRB ,&
&  0.49389E+00_JPRB, 0.49031E+00_JPRB, 0.48903E+00_JPRB, 0.49131E+00_JPRB, 0.49376E+00_JPRB, 0.49200E+00_JPRB, 0.49426E+00_JPRB ,&
&  0.49862E+00_JPRB, 0.50364E+00_JPRB, 0.52056E+00_JPRB, 0.54498E+00_JPRB, 0.56576E+00_JPRB, 0.58066E+00_JPRB, 0.59452E+00_JPRB ,&
&  0.61576E+00_JPRB, 0.62879E+00_JPRB, 0.61625E+00_JPRB, 0.60431E+00_JPRB, 0.59820E+00_JPRB, 0.60347E+00_JPRB, 0.61409E+00_JPRB ,&
&  0.63479E+00_JPRB, 0.64775E+00_JPRB, 0.65398E+00_JPRB, 0.65558E+00_JPRB, 0.65889E+00_JPRB, 0.65986E+00_JPRB, 0.67260E+00_JPRB ,&
&  0.70658E+00_JPRB, 0.73583E+00_JPRB, 0.74320E+00_JPRB, 0.77586E+00_JPRB, 0.81774E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.89195E+00_JPRB, 0.89619E+00_JPRB, 0.88631E+00_JPRB, 0.88184E+00_JPRB, 0.87114E+00_JPRB, 0.85742E+00_JPRB, 0.83498E+00_JPRB ,&
&  0.80860E+00_JPRB, 0.78039E+00_JPRB, 0.75367E+00_JPRB, 0.73176E+00_JPRB, 0.73552E+00_JPRB, 0.76784E+00_JPRB, 0.79512E+00_JPRB ,&
&  0.78398E+00_JPRB, 0.77983E+00_JPRB, 0.79663E+00_JPRB, 0.82694E+00_JPRB, 0.85996E+00_JPRB, 0.89044E+00_JPRB, 0.91224E+00_JPRB ,&
&  0.92985E+00_JPRB, 0.91148E+00_JPRB, 0.91522E+00_JPRB, 0.91738E+00_JPRB, 0.91141E+00_JPRB, 0.89358E+00_JPRB, 0.86490E+00_JPRB ,&
&  0.83636E+00_JPRB, 0.80901E+00_JPRB, 0.77528E+00_JPRB, 0.71670E+00_JPRB, 0.78993E+00_JPRB, 0.71373E+00_JPRB, 0.68529E+00_JPRB ,&
&  0.65810E+00_JPRB, 0.63981E+00_JPRB, 0.61908E+00_JPRB, 0.59674E+00_JPRB, 0.58899E+00_JPRB, 0.58429E+00_JPRB, 0.57627E+00_JPRB ,&
&  0.56964E+00_JPRB, 0.56233E+00_JPRB, 0.55127E+00_JPRB, 0.53732E+00_JPRB, 0.52424E+00_JPRB, 0.51031E+00_JPRB, 0.49753E+00_JPRB ,&
&  0.47895E+00_JPRB, 0.45179E+00_JPRB, 0.42319E+00_JPRB, 0.40211E+00_JPRB, 0.39108E+00_JPRB, 0.40028E+00_JPRB, 0.43310E+00_JPRB ,&
&  0.46693E+00_JPRB, 0.47293E+00_JPRB, 0.48299E+00_JPRB, 0.48626E+00_JPRB, 0.47823E+00_JPRB, 0.46854E+00_JPRB, 0.45131E+00_JPRB ,&
&  0.43834E+00_JPRB, 0.43662E+00_JPRB, 0.43817E+00_JPRB, 0.44237E+00_JPRB, 0.44753E+00_JPRB, 0.45469E+00_JPRB, 0.46461E+00_JPRB ,&
&  0.47367E+00_JPRB, 0.47668E+00_JPRB, 0.48884E+00_JPRB, 0.51556E+00_JPRB, 0.54664E+00_JPRB, 0.57827E+00_JPRB, 0.60541E+00_JPRB ,&
&  0.62385E+00_JPRB, 0.63680E+00_JPRB, 0.64519E+00_JPRB, 0.66040E+00_JPRB, 0.67613E+00_JPRB, 0.69190E+00_JPRB, 0.70005E+00_JPRB ,&
&  0.69761E+00_JPRB, 0.70128E+00_JPRB, 0.71859E+00_JPRB, 0.74705E+00_JPRB, 0.76689E+00_JPRB, 0.80079E+00_JPRB, 0.82202E+00_JPRB ,&
&  0.84451E+00_JPRB, 0.85028E+00_JPRB, 0.84492E+00_JPRB, 0.85751E+00_JPRB, 0.87437E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.87715E+00_JPRB, 0.88766E+00_JPRB, 0.88537E+00_JPRB, 0.88414E+00_JPRB, 0.88566E+00_JPRB, 0.87837E+00_JPRB, 0.86411E+00_JPRB ,&
&  0.84982E+00_JPRB, 0.82314E+00_JPRB, 0.79250E+00_JPRB, 0.77861E+00_JPRB, 0.78580E+00_JPRB, 0.78536E+00_JPRB, 0.76652E+00_JPRB ,&
&  0.76460E+00_JPRB, 0.77815E+00_JPRB, 0.80087E+00_JPRB, 0.82622E+00_JPRB, 0.86043E+00_JPRB, 0.89594E+00_JPRB, 0.92239E+00_JPRB ,&
&  0.93277E+00_JPRB, 0.93074E+00_JPRB, 0.92309E+00_JPRB, 0.89632E+00_JPRB, 0.86056E+00_JPRB, 0.83324E+00_JPRB, 0.81242E+00_JPRB ,&
&  0.78658E+00_JPRB, 0.73845E+00_JPRB, 0.66924E+00_JPRB, 0.66374E+00_JPRB, 0.60036E+00_JPRB, 0.58054E+00_JPRB, 0.60280E+00_JPRB ,&
&  0.61825E+00_JPRB, 0.60035E+00_JPRB, 0.57633E+00_JPRB, 0.55635E+00_JPRB, 0.55230E+00_JPRB, 0.55073E+00_JPRB, 0.54544E+00_JPRB ,&
&  0.54139E+00_JPRB, 0.53449E+00_JPRB, 0.52025E+00_JPRB, 0.50716E+00_JPRB, 0.49477E+00_JPRB, 0.48240E+00_JPRB, 0.47384E+00_JPRB ,&
&  0.44549E+00_JPRB, 0.40227E+00_JPRB, 0.37011E+00_JPRB, 0.35604E+00_JPRB, 0.34831E+00_JPRB, 0.35221E+00_JPRB, 0.36534E+00_JPRB ,&
&  0.37608E+00_JPRB, 0.37713E+00_JPRB, 0.37191E+00_JPRB, 0.35956E+00_JPRB, 0.34018E+00_JPRB, 0.29602E+00_JPRB, 0.27069E+00_JPRB ,&
&  0.28717E+00_JPRB, 0.32045E+00_JPRB, 0.35507E+00_JPRB, 0.36544E+00_JPRB, 0.37487E+00_JPRB, 0.37684E+00_JPRB, 0.36182E+00_JPRB ,&
&  0.33308E+00_JPRB, 0.30361E+00_JPRB, 0.29808E+00_JPRB, 0.30018E+00_JPRB, 0.30512E+00_JPRB, 0.31428E+00_JPRB, 0.33049E+00_JPRB ,&
&  0.35477E+00_JPRB, 0.39305E+00_JPRB, 0.41530E+00_JPRB, 0.41797E+00_JPRB, 0.41061E+00_JPRB, 0.39929E+00_JPRB, 0.39718E+00_JPRB ,&
&  0.41137E+00_JPRB, 0.47205E+00_JPRB, 0.59364E+00_JPRB, 0.74006E+00_JPRB, 0.74028E+00_JPRB, 0.78450E+00_JPRB, 0.81923E+00_JPRB ,&
&  0.84908E+00_JPRB, 0.86256E+00_JPRB, 0.84937E+00_JPRB, 0.85058E+00_JPRB, 0.85714E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.83657E+00_JPRB, 0.85636E+00_JPRB, 0.86713E+00_JPRB, 0.88093E+00_JPRB, 0.88092E+00_JPRB, 0.85053E+00_JPRB, 0.83577E+00_JPRB ,&
&  0.84023E+00_JPRB, 0.83508E+00_JPRB, 0.82394E+00_JPRB, 0.81691E+00_JPRB, 0.80775E+00_JPRB, 0.79778E+00_JPRB, 0.78932E+00_JPRB ,&
&  0.78086E+00_JPRB, 0.77563E+00_JPRB, 0.78239E+00_JPRB, 0.79612E+00_JPRB, 0.80735E+00_JPRB, 0.81345E+00_JPRB, 0.81829E+00_JPRB ,&
&  0.81769E+00_JPRB, 0.78458E+00_JPRB, 0.73932E+00_JPRB, 0.70626E+00_JPRB, 0.68972E+00_JPRB, 0.67591E+00_JPRB, 0.64268E+00_JPRB ,&
&  0.57460E+00_JPRB, 0.45315E+00_JPRB, 0.39717E+00_JPRB, 0.43867E+00_JPRB, 0.55628E+00_JPRB, 0.52165E+00_JPRB, 0.54236E+00_JPRB ,&
&  0.57292E+00_JPRB, 0.58876E+00_JPRB, 0.58450E+00_JPRB, 0.57391E+00_JPRB, 0.57026E+00_JPRB, 0.57374E+00_JPRB, 0.57034E+00_JPRB ,&
&  0.56685E+00_JPRB, 0.55085E+00_JPRB, 0.54123E+00_JPRB, 0.53017E+00_JPRB, 0.51239E+00_JPRB, 0.48652E+00_JPRB, 0.47076E+00_JPRB ,&
&  0.43232E+00_JPRB, 0.39452E+00_JPRB, 0.38351E+00_JPRB, 0.38416E+00_JPRB, 0.35347E+00_JPRB, 0.34122E+00_JPRB, 0.32659E+00_JPRB ,&
&  0.28747E+00_JPRB, 0.26645E+00_JPRB, 0.23481E+00_JPRB, 0.21511E+00_JPRB, 0.20105E+00_JPRB, 0.18725E+00_JPRB, 0.18544E+00_JPRB ,&
&  0.19063E+00_JPRB, 0.20359E+00_JPRB, 0.22502E+00_JPRB, 0.25826E+00_JPRB, 0.28826E+00_JPRB, 0.24678E+00_JPRB, 0.22937E+00_JPRB ,&
&  0.21516E+00_JPRB, 0.21193E+00_JPRB, 0.21775E+00_JPRB, 0.22452E+00_JPRB, 0.23141E+00_JPRB, 0.23954E+00_JPRB, 0.24749E+00_JPRB ,&
&  0.25333E+00_JPRB, 0.25651E+00_JPRB, 0.25737E+00_JPRB, 0.25670E+00_JPRB, 0.25561E+00_JPRB, 0.25640E+00_JPRB, 0.25988E+00_JPRB ,&
&  0.26627E+00_JPRB, 0.27873E+00_JPRB, 0.30263E+00_JPRB, 0.33795E+00_JPRB, 0.38393E+00_JPRB, 0.44430E+00_JPRB, 0.53646E+00_JPRB ,&
&  0.66124E+00_JPRB, 0.82513E+00_JPRB, 0.87011E+00_JPRB, 0.87925E+00_JPRB, 0.84828E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.84134E+00_JPRB, 0.83875E+00_JPRB, 0.84623E+00_JPRB, 0.83573E+00_JPRB, 0.81651E+00_JPRB, 0.80735E+00_JPRB, 0.82418E+00_JPRB ,&
&  0.84379E+00_JPRB, 0.81412E+00_JPRB, 0.79226E+00_JPRB, 0.77290E+00_JPRB, 0.75903E+00_JPRB, 0.74502E+00_JPRB, 0.72280E+00_JPRB ,&
&  0.70259E+00_JPRB, 0.69508E+00_JPRB, 0.69298E+00_JPRB, 0.68778E+00_JPRB, 0.67741E+00_JPRB, 0.66722E+00_JPRB, 0.64942E+00_JPRB ,&
&  0.61649E+00_JPRB, 0.58186E+00_JPRB, 0.56373E+00_JPRB, 0.54002E+00_JPRB, 0.52312E+00_JPRB, 0.50362E+00_JPRB, 0.47592E+00_JPRB ,&
&  0.43075E+00_JPRB, 0.37273E+00_JPRB, 0.33729E+00_JPRB, 0.37412E+00_JPRB, 0.46859E+00_JPRB, 0.46219E+00_JPRB, 0.48169E+00_JPRB ,&
&  0.51649E+00_JPRB, 0.53774E+00_JPRB, 0.54819E+00_JPRB, 0.55145E+00_JPRB, 0.56056E+00_JPRB, 0.56619E+00_JPRB, 0.56262E+00_JPRB ,&
&  0.54781E+00_JPRB, 0.49550E+00_JPRB, 0.45269E+00_JPRB, 0.34956E+00_JPRB, 0.27848E+00_JPRB, 0.24096E+00_JPRB, 0.21164E+00_JPRB ,&
&  0.19260E+00_JPRB, 0.18150E+00_JPRB, 0.17909E+00_JPRB, 0.17868E+00_JPRB, 0.18418E+00_JPRB, 0.19033E+00_JPRB, 0.19064E+00_JPRB ,&
&  0.18695E+00_JPRB, 0.17922E+00_JPRB, 0.16822E+00_JPRB, 0.16184E+00_JPRB, 0.15860E+00_JPRB, 0.15900E+00_JPRB, 0.16128E+00_JPRB ,&
&  0.16300E+00_JPRB, 0.16705E+00_JPRB, 0.17958E+00_JPRB, 0.19041E+00_JPRB, 0.20397E+00_JPRB, 0.18669E+00_JPRB, 0.17236E+00_JPRB ,&
&  0.16705E+00_JPRB, 0.17161E+00_JPRB, 0.17863E+00_JPRB, 0.18389E+00_JPRB, 0.18938E+00_JPRB, 0.19402E+00_JPRB, 0.19873E+00_JPRB ,&
&  0.20346E+00_JPRB, 0.20688E+00_JPRB, 0.20858E+00_JPRB, 0.21038E+00_JPRB, 0.21201E+00_JPRB, 0.21445E+00_JPRB, 0.21742E+00_JPRB ,&
&  0.22244E+00_JPRB, 0.23188E+00_JPRB, 0.24411E+00_JPRB, 0.26432E+00_JPRB, 0.28326E+00_JPRB, 0.30216E+00_JPRB, 0.32403E+00_JPRB ,&
&  0.34488E+00_JPRB, 0.42534E+00_JPRB, 0.61873E+00_JPRB, 0.93891E+00_JPRB, 0.93223E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.78920E+00_JPRB, 0.80144E+00_JPRB, 0.81824E+00_JPRB, 0.79461E+00_JPRB, 0.77922E+00_JPRB, 0.78242E+00_JPRB, 0.78306E+00_JPRB ,&
&  0.75973E+00_JPRB, 0.72429E+00_JPRB, 0.67488E+00_JPRB, 0.63403E+00_JPRB, 0.56874E+00_JPRB, 0.48487E+00_JPRB, 0.44708E+00_JPRB ,&
&  0.41254E+00_JPRB, 0.40879E+00_JPRB, 0.41084E+00_JPRB, 0.42677E+00_JPRB, 0.43543E+00_JPRB, 0.43190E+00_JPRB, 0.40994E+00_JPRB ,&
&  0.38272E+00_JPRB, 0.35557E+00_JPRB, 0.33799E+00_JPRB, 0.32410E+00_JPRB, 0.32388E+00_JPRB, 0.32197E+00_JPRB, 0.31542E+00_JPRB ,&
&  0.30699E+00_JPRB, 0.31148E+00_JPRB, 0.32895E+00_JPRB, 0.35706E+00_JPRB, 0.40604E+00_JPRB, 0.38591E+00_JPRB, 0.39262E+00_JPRB ,&
&  0.42272E+00_JPRB, 0.44444E+00_JPRB, 0.45304E+00_JPRB, 0.45615E+00_JPRB, 0.44921E+00_JPRB, 0.47350E+00_JPRB, 0.41690E+00_JPRB ,&
&  0.31001E+00_JPRB, 0.25109E+00_JPRB, 0.20904E+00_JPRB, 0.18189E+00_JPRB, 0.16368E+00_JPRB, 0.15305E+00_JPRB, 0.14135E+00_JPRB ,&
&  0.13596E+00_JPRB, 0.13198E+00_JPRB, 0.13115E+00_JPRB, 0.13291E+00_JPRB, 0.13567E+00_JPRB, 0.13917E+00_JPRB, 0.14051E+00_JPRB ,&
&  0.14257E+00_JPRB, 0.14279E+00_JPRB, 0.14214E+00_JPRB, 0.14216E+00_JPRB, 0.14127E+00_JPRB, 0.14214E+00_JPRB, 0.14230E+00_JPRB ,&
&  0.14240E+00_JPRB, 0.14236E+00_JPRB, 0.14405E+00_JPRB, 0.14486E+00_JPRB, 0.14586E+00_JPRB, 0.14299E+00_JPRB, 0.14146E+00_JPRB ,&
&  0.14180E+00_JPRB, 0.14399E+00_JPRB, 0.14746E+00_JPRB, 0.15140E+00_JPRB, 0.15680E+00_JPRB, 0.16107E+00_JPRB, 0.16604E+00_JPRB ,&
&  0.17038E+00_JPRB, 0.17466E+00_JPRB, 0.17884E+00_JPRB, 0.18310E+00_JPRB, 0.18767E+00_JPRB, 0.19223E+00_JPRB, 0.19695E+00_JPRB ,&
&  0.20186E+00_JPRB, 0.21019E+00_JPRB, 0.22035E+00_JPRB, 0.23808E+00_JPRB, 0.26202E+00_JPRB, 0.28879E+00_JPRB, 0.33572E+00_JPRB ,&
&  0.39524E+00_JPRB, 0.62378E+00_JPRB, 0.92406E+00_JPRB, 0.75354E+00_JPRB, 0.77115E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.77935E+00_JPRB, 0.79754E+00_JPRB, 0.81358E+00_JPRB, 0.81081E+00_JPRB, 0.79339E+00_JPRB, 0.73563E+00_JPRB, 0.67211E+00_JPRB ,&
&  0.59335E+00_JPRB, 0.52722E+00_JPRB, 0.45779E+00_JPRB, 0.39772E+00_JPRB, 0.34728E+00_JPRB, 0.30846E+00_JPRB, 0.28503E+00_JPRB ,&
&  0.26842E+00_JPRB, 0.26113E+00_JPRB, 0.25824E+00_JPRB, 0.26273E+00_JPRB, 0.26503E+00_JPRB, 0.27038E+00_JPRB, 0.26469E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.24966E+00_JPRB, 0.24762E+00_JPRB, 0.25628E+00_JPRB, 0.26946E+00_JPRB, 0.28227E+00_JPRB, 0.29893E+00_JPRB ,&
&  0.31717E+00_JPRB, 0.32298E+00_JPRB, 0.33705E+00_JPRB, 0.34065E+00_JPRB, 0.32598E+00_JPRB, 0.32769E+00_JPRB, 0.33351E+00_JPRB ,&
&  0.34666E+00_JPRB, 0.36520E+00_JPRB, 0.37579E+00_JPRB, 0.39576E+00_JPRB, 0.34280E+00_JPRB, 0.28271E+00_JPRB, 0.23094E+00_JPRB ,&
&  0.19242E+00_JPRB, 0.17050E+00_JPRB, 0.15511E+00_JPRB, 0.14729E+00_JPRB, 0.13973E+00_JPRB, 0.13561E+00_JPRB, 0.13129E+00_JPRB ,&
&  0.12941E+00_JPRB, 0.12941E+00_JPRB, 0.12903E+00_JPRB, 0.12883E+00_JPRB, 0.12939E+00_JPRB, 0.13025E+00_JPRB, 0.12986E+00_JPRB ,&
&  0.12929E+00_JPRB, 0.12914E+00_JPRB, 0.12878E+00_JPRB, 0.12853E+00_JPRB, 0.12780E+00_JPRB, 0.12781E+00_JPRB, 0.12638E+00_JPRB ,&
&  0.12572E+00_JPRB, 0.12444E+00_JPRB, 0.12375E+00_JPRB, 0.12376E+00_JPRB, 0.12424E+00_JPRB, 0.12470E+00_JPRB, 0.12542E+00_JPRB ,&
&  0.12670E+00_JPRB, 0.12787E+00_JPRB, 0.12980E+00_JPRB, 0.13164E+00_JPRB, 0.13459E+00_JPRB, 0.13722E+00_JPRB, 0.14091E+00_JPRB ,&
&  0.14483E+00_JPRB, 0.14956E+00_JPRB, 0.15501E+00_JPRB, 0.16114E+00_JPRB, 0.16638E+00_JPRB, 0.17149E+00_JPRB, 0.17703E+00_JPRB ,&
&  0.18138E+00_JPRB, 0.18858E+00_JPRB, 0.19665E+00_JPRB, 0.21007E+00_JPRB, 0.23813E+00_JPRB, 0.26181E+00_JPRB, 0.31048E+00_JPRB ,&
&  0.39240E+00_JPRB, 0.64577E+00_JPRB, 0.87759E+00_JPRB, 0.72996E+00_JPRB, 0.76106E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.65757E+00_JPRB, 0.68857E+00_JPRB, 0.71267E+00_JPRB, 0.71462E+00_JPRB, 0.71326E+00_JPRB, 0.67583E+00_JPRB, 0.61774E+00_JPRB ,&
&  0.51107E+00_JPRB, 0.44145E+00_JPRB, 0.39743E+00_JPRB, 0.35587E+00_JPRB, 0.31821E+00_JPRB, 0.29277E+00_JPRB, 0.27539E+00_JPRB ,&
&  0.26222E+00_JPRB, 0.25495E+00_JPRB, 0.24942E+00_JPRB, 0.24477E+00_JPRB, 0.23974E+00_JPRB, 0.23488E+00_JPRB, 0.22614E+00_JPRB ,&
&  0.22083E+00_JPRB, 0.21736E+00_JPRB, 0.21591E+00_JPRB, 0.21832E+00_JPRB, 0.22517E+00_JPRB, 0.23210E+00_JPRB, 0.24269E+00_JPRB ,&
&  0.25568E+00_JPRB, 0.26954E+00_JPRB, 0.27710E+00_JPRB, 0.28044E+00_JPRB, 0.28338E+00_JPRB, 0.28156E+00_JPRB, 0.25909E+00_JPRB ,&
&  0.26409E+00_JPRB, 0.28671E+00_JPRB, 0.30497E+00_JPRB, 0.31493E+00_JPRB, 0.27047E+00_JPRB, 0.22923E+00_JPRB, 0.18865E+00_JPRB ,&
&  0.16928E+00_JPRB, 0.15396E+00_JPRB, 0.14599E+00_JPRB, 0.13996E+00_JPRB, 0.13551E+00_JPRB, 0.13298E+00_JPRB, 0.13077E+00_JPRB ,&
&  0.12880E+00_JPRB, 0.12805E+00_JPRB, 0.12678E+00_JPRB, 0.12629E+00_JPRB, 0.12546E+00_JPRB, 0.12543E+00_JPRB, 0.12433E+00_JPRB ,&
&  0.12321E+00_JPRB, 0.12231E+00_JPRB, 0.12177E+00_JPRB, 0.12152E+00_JPRB, 0.12078E+00_JPRB, 0.12005E+00_JPRB, 0.11828E+00_JPRB ,&
&  0.11707E+00_JPRB, 0.11588E+00_JPRB, 0.11501E+00_JPRB, 0.11515E+00_JPRB, 0.11575E+00_JPRB, 0.11762E+00_JPRB, 0.11869E+00_JPRB ,&
&  0.11895E+00_JPRB, 0.11908E+00_JPRB, 0.11915E+00_JPRB, 0.11990E+00_JPRB, 0.12102E+00_JPRB, 0.12292E+00_JPRB, 0.12558E+00_JPRB ,&
&  0.12878E+00_JPRB, 0.13368E+00_JPRB, 0.13861E+00_JPRB, 0.14410E+00_JPRB, 0.14923E+00_JPRB, 0.15424E+00_JPRB, 0.16086E+00_JPRB ,&
&  0.16762E+00_JPRB, 0.17555E+00_JPRB, 0.18723E+00_JPRB, 0.19967E+00_JPRB, 0.22207E+00_JPRB, 0.24152E+00_JPRB, 0.27282E+00_JPRB ,&
&  0.34304E+00_JPRB, 0.48959E+00_JPRB, 0.57525E+00_JPRB, 0.58686E+00_JPRB, 0.61989E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.47712E+00_JPRB, 0.48859E+00_JPRB, 0.50361E+00_JPRB, 0.51267E+00_JPRB, 0.52267E+00_JPRB, 0.49000E+00_JPRB, 0.46423E+00_JPRB ,&
&  0.41160E+00_JPRB, 0.38148E+00_JPRB, 0.35641E+00_JPRB, 0.33339E+00_JPRB, 0.31290E+00_JPRB, 0.29477E+00_JPRB, 0.27745E+00_JPRB ,&
&  0.26634E+00_JPRB, 0.25960E+00_JPRB, 0.25049E+00_JPRB, 0.24034E+00_JPRB, 0.22951E+00_JPRB, 0.22004E+00_JPRB, 0.21504E+00_JPRB ,&
&  0.21041E+00_JPRB, 0.20707E+00_JPRB, 0.20569E+00_JPRB, 0.20584E+00_JPRB, 0.20770E+00_JPRB, 0.20793E+00_JPRB, 0.20862E+00_JPRB ,&
&  0.20563E+00_JPRB, 0.21273E+00_JPRB, 0.22209E+00_JPRB, 0.22634E+00_JPRB, 0.23420E+00_JPRB, 0.23279E+00_JPRB, 0.22805E+00_JPRB ,&
&  0.22103E+00_JPRB, 0.21270E+00_JPRB, 0.20331E+00_JPRB, 0.19171E+00_JPRB, 0.18126E+00_JPRB, 0.17351E+00_JPRB, 0.16591E+00_JPRB ,&
&  0.16132E+00_JPRB, 0.15583E+00_JPRB, 0.14942E+00_JPRB, 0.14391E+00_JPRB, 0.13915E+00_JPRB, 0.13522E+00_JPRB, 0.13166E+00_JPRB ,&
&  0.12906E+00_JPRB, 0.12752E+00_JPRB, 0.12606E+00_JPRB, 0.12568E+00_JPRB, 0.12462E+00_JPRB, 0.12358E+00_JPRB, 0.12255E+00_JPRB ,&
&  0.12200E+00_JPRB, 0.12085E+00_JPRB, 0.11995E+00_JPRB, 0.11916E+00_JPRB, 0.11790E+00_JPRB, 0.11682E+00_JPRB, 0.11557E+00_JPRB ,&
&  0.11450E+00_JPRB, 0.11396E+00_JPRB, 0.11369E+00_JPRB, 0.11367E+00_JPRB, 0.11401E+00_JPRB, 0.11402E+00_JPRB, 0.11432E+00_JPRB ,&
&  0.11467E+00_JPRB, 0.11516E+00_JPRB, 0.11519E+00_JPRB, 0.11555E+00_JPRB, 0.11643E+00_JPRB, 0.11764E+00_JPRB, 0.12052E+00_JPRB ,&
&  0.12381E+00_JPRB, 0.12805E+00_JPRB, 0.13266E+00_JPRB, 0.13792E+00_JPRB, 0.14320E+00_JPRB, 0.14843E+00_JPRB, 0.15423E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16854E+00_JPRB, 0.17641E+00_JPRB, 0.18834E+00_JPRB, 0.21398E+00_JPRB, 0.22816E+00_JPRB, 0.24318E+00_JPRB ,&
&  0.26976E+00_JPRB, 0.30605E+00_JPRB, 0.40928E+00_JPRB, 0.44193E+00_JPRB, 0.45918E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.22040E+00_JPRB, 0.22047E+00_JPRB, 0.22127E+00_JPRB, 0.22780E+00_JPRB, 0.23997E+00_JPRB, 0.31703E+00_JPRB, 0.30814E+00_JPRB ,&
&  0.30090E+00_JPRB, 0.27264E+00_JPRB, 0.26899E+00_JPRB, 0.26183E+00_JPRB, 0.25224E+00_JPRB, 0.24438E+00_JPRB, 0.23969E+00_JPRB ,&
&  0.23460E+00_JPRB, 0.22904E+00_JPRB, 0.22581E+00_JPRB, 0.22253E+00_JPRB, 0.21933E+00_JPRB, 0.21573E+00_JPRB, 0.21295E+00_JPRB ,&
&  0.21046E+00_JPRB, 0.20872E+00_JPRB, 0.20680E+00_JPRB, 0.20308E+00_JPRB, 0.20143E+00_JPRB, 0.19819E+00_JPRB, 0.19337E+00_JPRB ,&
&  0.18710E+00_JPRB, 0.18370E+00_JPRB, 0.17992E+00_JPRB, 0.17655E+00_JPRB, 0.17927E+00_JPRB, 0.17825E+00_JPRB, 0.17604E+00_JPRB ,&
&  0.17252E+00_JPRB, 0.16888E+00_JPRB, 0.16416E+00_JPRB, 0.15965E+00_JPRB, 0.15588E+00_JPRB, 0.15278E+00_JPRB, 0.15011E+00_JPRB ,&
&  0.14798E+00_JPRB, 0.14618E+00_JPRB, 0.14376E+00_JPRB, 0.14156E+00_JPRB, 0.13923E+00_JPRB, 0.13719E+00_JPRB, 0.13511E+00_JPRB ,&
&  0.13348E+00_JPRB, 0.13195E+00_JPRB, 0.13035E+00_JPRB, 0.12871E+00_JPRB, 0.12743E+00_JPRB, 0.12573E+00_JPRB, 0.12447E+00_JPRB ,&
&  0.12337E+00_JPRB, 0.12197E+00_JPRB, 0.12095E+00_JPRB, 0.11970E+00_JPRB, 0.11877E+00_JPRB, 0.11802E+00_JPRB, 0.11728E+00_JPRB ,&
&  0.11628E+00_JPRB, 0.11562E+00_JPRB, 0.11537E+00_JPRB, 0.11519E+00_JPRB, 0.11490E+00_JPRB, 0.11496E+00_JPRB, 0.11530E+00_JPRB ,&
&  0.11577E+00_JPRB, 0.11633E+00_JPRB, 0.11722E+00_JPRB, 0.11828E+00_JPRB, 0.11984E+00_JPRB, 0.12190E+00_JPRB, 0.12516E+00_JPRB ,&
&  0.12869E+00_JPRB, 0.13261E+00_JPRB, 0.13664E+00_JPRB, 0.14113E+00_JPRB, 0.14566E+00_JPRB, 0.15028E+00_JPRB, 0.15560E+00_JPRB ,&
&  0.16134E+00_JPRB, 0.16576E+00_JPRB, 0.16983E+00_JPRB, 0.17417E+00_JPRB, 0.18316E+00_JPRB, 0.18991E+00_JPRB, 0.19514E+00_JPRB ,&
&  0.20098E+00_JPRB, 0.20892E+00_JPRB, 0.21373E+00_JPRB, 0.21703E+00_JPRB, 0.21899E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.19248E+00_JPRB, 0.19506E+00_JPRB, 0.19678E+00_JPRB, 0.19787E+00_JPRB, 0.19762E+00_JPRB, 0.19853E+00_JPRB, 0.19948E+00_JPRB ,&
&  0.20075E+00_JPRB, 0.20231E+00_JPRB, 0.20341E+00_JPRB, 0.20441E+00_JPRB, 0.20488E+00_JPRB, 0.20486E+00_JPRB, 0.20483E+00_JPRB ,&
&  0.20462E+00_JPRB, 0.20436E+00_JPRB, 0.20106E+00_JPRB, 0.19809E+00_JPRB, 0.19597E+00_JPRB, 0.19477E+00_JPRB, 0.19369E+00_JPRB ,&
&  0.19280E+00_JPRB, 0.19159E+00_JPRB, 0.19015E+00_JPRB, 0.18799E+00_JPRB, 0.18474E+00_JPRB, 0.18158E+00_JPRB, 0.17833E+00_JPRB ,&
&  0.17540E+00_JPRB, 0.17302E+00_JPRB, 0.17081E+00_JPRB, 0.16859E+00_JPRB, 0.16723E+00_JPRB, 0.16568E+00_JPRB, 0.16409E+00_JPRB ,&
&  0.16209E+00_JPRB, 0.15994E+00_JPRB, 0.15789E+00_JPRB, 0.15579E+00_JPRB, 0.15377E+00_JPRB, 0.15178E+00_JPRB, 0.15033E+00_JPRB ,&
&  0.14881E+00_JPRB, 0.14765E+00_JPRB, 0.14638E+00_JPRB, 0.14522E+00_JPRB, 0.14400E+00_JPRB, 0.14285E+00_JPRB, 0.14154E+00_JPRB ,&
&  0.14065E+00_JPRB, 0.13972E+00_JPRB, 0.13878E+00_JPRB, 0.13790E+00_JPRB, 0.13703E+00_JPRB, 0.13620E+00_JPRB, 0.13524E+00_JPRB ,&
&  0.13437E+00_JPRB, 0.13381E+00_JPRB, 0.13323E+00_JPRB, 0.13257E+00_JPRB, 0.13197E+00_JPRB, 0.13156E+00_JPRB, 0.13121E+00_JPRB ,&
&  0.13088E+00_JPRB, 0.13087E+00_JPRB, 0.13104E+00_JPRB, 0.13122E+00_JPRB, 0.13140E+00_JPRB, 0.13180E+00_JPRB, 0.13239E+00_JPRB ,&
&  0.13308E+00_JPRB, 0.13383E+00_JPRB, 0.13479E+00_JPRB, 0.13615E+00_JPRB, 0.13765E+00_JPRB, 0.13938E+00_JPRB, 0.14126E+00_JPRB ,&
&  0.14329E+00_JPRB, 0.14546E+00_JPRB, 0.14775E+00_JPRB, 0.14999E+00_JPRB, 0.15240E+00_JPRB, 0.15481E+00_JPRB, 0.15713E+00_JPRB ,&
&  0.15955E+00_JPRB, 0.16193E+00_JPRB, 0.16437E+00_JPRB, 0.16659E+00_JPRB, 0.16998E+00_JPRB, 0.17219E+00_JPRB, 0.17451E+00_JPRB ,&
&  0.17669E+00_JPRB, 0.17920E+00_JPRB, 0.18174E+00_JPRB, 0.18488E+00_JPRB, 0.18800E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB, 0.15796E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB, 0.69863E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.76385E+00_JPRB, 0.76062E+00_JPRB, 0.75709E+00_JPRB, 0.75388E+00_JPRB, 0.74999E+00_JPRB, 0.74651E+00_JPRB, 0.74301E+00_JPRB ,&
&  0.74000E+00_JPRB, 0.73685E+00_JPRB, 0.73339E+00_JPRB, 0.72963E+00_JPRB, 0.72593E+00_JPRB, 0.72235E+00_JPRB, 0.71879E+00_JPRB ,&
&  0.71521E+00_JPRB, 0.71186E+00_JPRB, 0.70834E+00_JPRB, 0.70431E+00_JPRB, 0.70099E+00_JPRB, 0.69717E+00_JPRB, 0.69205E+00_JPRB ,&
&  0.68460E+00_JPRB, 0.67677E+00_JPRB, 0.66947E+00_JPRB, 0.66410E+00_JPRB, 0.66016E+00_JPRB, 0.65697E+00_JPRB, 0.65423E+00_JPRB ,&
&  0.65261E+00_JPRB, 0.65173E+00_JPRB, 0.65116E+00_JPRB, 0.64990E+00_JPRB, 0.64908E+00_JPRB, 0.65001E+00_JPRB, 0.65042E+00_JPRB ,&
&  0.65021E+00_JPRB, 0.65022E+00_JPRB, 0.65128E+00_JPRB, 0.65223E+00_JPRB, 0.65292E+00_JPRB, 0.65436E+00_JPRB, 0.65716E+00_JPRB ,&
&  0.66007E+00_JPRB, 0.66257E+00_JPRB, 0.66572E+00_JPRB, 0.66918E+00_JPRB, 0.67273E+00_JPRB, 0.67620E+00_JPRB, 0.68103E+00_JPRB ,&
&  0.68635E+00_JPRB, 0.69110E+00_JPRB, 0.69554E+00_JPRB, 0.69891E+00_JPRB, 0.70171E+00_JPRB, 0.70309E+00_JPRB, 0.70392E+00_JPRB ,&
&  0.70364E+00_JPRB, 0.70465E+00_JPRB, 0.70461E+00_JPRB, 0.70468E+00_JPRB, 0.70379E+00_JPRB, 0.70465E+00_JPRB, 0.70513E+00_JPRB ,&
&  0.70719E+00_JPRB, 0.70850E+00_JPRB, 0.71111E+00_JPRB, 0.71367E+00_JPRB, 0.71769E+00_JPRB, 0.72208E+00_JPRB, 0.72680E+00_JPRB ,&
&  0.73209E+00_JPRB, 0.73835E+00_JPRB, 0.74582E+00_JPRB, 0.75054E+00_JPRB, 0.75517E+00_JPRB, 0.75966E+00_JPRB, 0.76455E+00_JPRB ,&
&  0.76836E+00_JPRB, 0.77183E+00_JPRB, 0.77530E+00_JPRB, 0.77798E+00_JPRB, 0.77967E+00_JPRB, 0.78109E+00_JPRB, 0.78247E+00_JPRB ,&
&  0.78317E+00_JPRB, 0.78364E+00_JPRB, 0.78388E+00_JPRB, 0.78423E+00_JPRB, 0.78321E+00_JPRB, 0.78142E+00_JPRB, 0.77937E+00_JPRB ,&
&  0.77765E+00_JPRB, 0.77511E+00_JPRB, 0.77256E+00_JPRB, 0.76982E+00_JPRB, 0.76718E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.88026E+00_JPRB, 0.87424E+00_JPRB, 0.86794E+00_JPRB, 0.86223E+00_JPRB, 0.85423E+00_JPRB, 0.84609E+00_JPRB, 0.83758E+00_JPRB ,&
&  0.82874E+00_JPRB, 0.81803E+00_JPRB, 0.80641E+00_JPRB, 0.79415E+00_JPRB, 0.78204E+00_JPRB, 0.77192E+00_JPRB, 0.76414E+00_JPRB ,&
&  0.75673E+00_JPRB, 0.74843E+00_JPRB, 0.74004E+00_JPRB, 0.72923E+00_JPRB, 0.71826E+00_JPRB, 0.70806E+00_JPRB, 0.69982E+00_JPRB ,&
&  0.68889E+00_JPRB, 0.67713E+00_JPRB, 0.66497E+00_JPRB, 0.65312E+00_JPRB, 0.64173E+00_JPRB, 0.63041E+00_JPRB, 0.61927E+00_JPRB ,&
&  0.60916E+00_JPRB, 0.60175E+00_JPRB, 0.59470E+00_JPRB, 0.58712E+00_JPRB, 0.57882E+00_JPRB, 0.57613E+00_JPRB, 0.57321E+00_JPRB ,&
&  0.57051E+00_JPRB, 0.56974E+00_JPRB, 0.57420E+00_JPRB, 0.57812E+00_JPRB, 0.58226E+00_JPRB, 0.58678E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.60253E+00_JPRB, 0.61139E+00_JPRB, 0.62270E+00_JPRB, 0.63437E+00_JPRB, 0.64410E+00_JPRB, 0.65146E+00_JPRB, 0.66159E+00_JPRB ,&
&  0.66919E+00_JPRB, 0.67923E+00_JPRB, 0.68647E+00_JPRB, 0.69023E+00_JPRB, 0.69030E+00_JPRB, 0.69077E+00_JPRB, 0.69277E+00_JPRB ,&
&  0.69320E+00_JPRB, 0.69856E+00_JPRB, 0.70368E+00_JPRB, 0.70900E+00_JPRB, 0.71102E+00_JPRB, 0.71699E+00_JPRB, 0.72088E+00_JPRB ,&
&  0.72894E+00_JPRB, 0.74129E+00_JPRB, 0.75428E+00_JPRB, 0.76888E+00_JPRB, 0.78676E+00_JPRB, 0.80831E+00_JPRB, 0.82591E+00_JPRB ,&
&  0.84257E+00_JPRB, 0.85772E+00_JPRB, 0.87136E+00_JPRB, 0.87986E+00_JPRB, 0.88766E+00_JPRB, 0.89486E+00_JPRB, 0.90189E+00_JPRB ,&
&  0.90481E+00_JPRB, 0.90795E+00_JPRB, 0.91128E+00_JPRB, 0.91403E+00_JPRB, 0.91330E+00_JPRB, 0.91201E+00_JPRB, 0.91118E+00_JPRB ,&
&  0.91061E+00_JPRB, 0.90929E+00_JPRB, 0.90864E+00_JPRB, 0.90917E+00_JPRB, 0.90884E+00_JPRB, 0.90710E+00_JPRB, 0.90516E+00_JPRB ,&
&  0.90343E+00_JPRB, 0.89985E+00_JPRB, 0.89520E+00_JPRB, 0.89049E+00_JPRB, 0.88629E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.92076E+00_JPRB, 0.91834E+00_JPRB, 0.91587E+00_JPRB, 0.91329E+00_JPRB, 0.90833E+00_JPRB, 0.90276E+00_JPRB, 0.89572E+00_JPRB ,&
&  0.88741E+00_JPRB, 0.87611E+00_JPRB, 0.86381E+00_JPRB, 0.85019E+00_JPRB, 0.83692E+00_JPRB, 0.82519E+00_JPRB, 0.81413E+00_JPRB ,&
&  0.80351E+00_JPRB, 0.78977E+00_JPRB, 0.77055E+00_JPRB, 0.74650E+00_JPRB, 0.72551E+00_JPRB, 0.70807E+00_JPRB, 0.69074E+00_JPRB ,&
&  0.66371E+00_JPRB, 0.63511E+00_JPRB, 0.60931E+00_JPRB, 0.58259E+00_JPRB, 0.56283E+00_JPRB, 0.51750E+00_JPRB, 0.50201E+00_JPRB ,&
&  0.48904E+00_JPRB, 0.47920E+00_JPRB, 0.47468E+00_JPRB, 0.47158E+00_JPRB, 0.47541E+00_JPRB, 0.46790E+00_JPRB, 0.46329E+00_JPRB ,&
&  0.44433E+00_JPRB, 0.44614E+00_JPRB, 0.45080E+00_JPRB, 0.45728E+00_JPRB, 0.47022E+00_JPRB, 0.48555E+00_JPRB, 0.50893E+00_JPRB ,&
&  0.55056E+00_JPRB, 0.56947E+00_JPRB, 0.59784E+00_JPRB, 0.61978E+00_JPRB, 0.63365E+00_JPRB, 0.64130E+00_JPRB, 0.65084E+00_JPRB ,&
&  0.66106E+00_JPRB, 0.67721E+00_JPRB, 0.68651E+00_JPRB, 0.69258E+00_JPRB, 0.69242E+00_JPRB, 0.69068E+00_JPRB, 0.69129E+00_JPRB ,&
&  0.69113E+00_JPRB, 0.69745E+00_JPRB, 0.70834E+00_JPRB, 0.72938E+00_JPRB, 0.75908E+00_JPRB, 0.79484E+00_JPRB, 0.82989E+00_JPRB ,&
&  0.86136E+00_JPRB, 0.89365E+00_JPRB, 0.91960E+00_JPRB, 0.94462E+00_JPRB, 0.97150E+00_JPRB, 0.99455E+00_JPRB, 0.10143E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.10355E+01_JPRB, 0.10349E+01_JPRB, 0.10320E+01_JPRB, 0.10250E+01_JPRB, 0.10188E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.10139E+01_JPRB, 0.10147E+01_JPRB, 0.10105E+01_JPRB, 0.10044E+01_JPRB, 0.99775E+00_JPRB, 0.99146E+00_JPRB ,&
&  0.98444E+00_JPRB, 0.97800E+00_JPRB, 0.97208E+00_JPRB, 0.96612E+00_JPRB, 0.96077E+00_JPRB, 0.95304E+00_JPRB, 0.94638E+00_JPRB ,&
&  0.93950E+00_JPRB, 0.93608E+00_JPRB, 0.93116E+00_JPRB, 0.92720E+00_JPRB, 0.92334E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.83989E+00_JPRB, 0.83873E+00_JPRB, 0.84001E+00_JPRB, 0.84018E+00_JPRB, 0.84283E+00_JPRB, 0.84460E+00_JPRB, 0.84768E+00_JPRB ,&
&  0.84954E+00_JPRB, 0.85182E+00_JPRB, 0.85175E+00_JPRB, 0.85050E+00_JPRB, 0.84677E+00_JPRB, 0.84297E+00_JPRB, 0.83790E+00_JPRB ,&
&  0.83011E+00_JPRB, 0.81429E+00_JPRB, 0.78927E+00_JPRB, 0.76142E+00_JPRB, 0.73008E+00_JPRB, 0.69518E+00_JPRB, 0.65832E+00_JPRB ,&
&  0.61916E+00_JPRB, 0.58521E+00_JPRB, 0.54029E+00_JPRB, 0.51571E+00_JPRB, 0.48840E+00_JPRB, 0.46859E+00_JPRB, 0.45530E+00_JPRB ,&
&  0.44122E+00_JPRB, 0.42929E+00_JPRB, 0.41629E+00_JPRB, 0.40271E+00_JPRB, 0.39373E+00_JPRB, 0.38075E+00_JPRB, 0.37418E+00_JPRB ,&
&  0.36939E+00_JPRB, 0.36748E+00_JPRB, 0.36807E+00_JPRB, 0.37221E+00_JPRB, 0.38052E+00_JPRB, 0.39244E+00_JPRB, 0.41324E+00_JPRB ,&
&  0.45873E+00_JPRB, 0.52220E+00_JPRB, 0.62010E+00_JPRB, 0.64066E+00_JPRB, 0.64022E+00_JPRB, 0.65500E+00_JPRB, 0.67223E+00_JPRB ,&
&  0.68951E+00_JPRB, 0.70854E+00_JPRB, 0.72324E+00_JPRB, 0.72867E+00_JPRB, 0.73616E+00_JPRB, 0.74298E+00_JPRB, 0.75111E+00_JPRB ,&
&  0.76081E+00_JPRB, 0.78099E+00_JPRB, 0.81110E+00_JPRB, 0.84708E+00_JPRB, 0.88782E+00_JPRB, 0.92731E+00_JPRB, 0.95795E+00_JPRB ,&
&  0.97959E+00_JPRB, 0.99827E+00_JPRB, 0.10257E+01_JPRB, 0.10550E+01_JPRB, 0.10853E+01_JPRB, 0.11034E+01_JPRB, 0.11194E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.11338E+01_JPRB, 0.11251E+01_JPRB, 0.11125E+01_JPRB, 0.11025E+01_JPRB, 0.11062E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.10975E+01_JPRB, 0.10855E+01_JPRB, 0.10650E+01_JPRB, 0.10425E+01_JPRB, 0.10259E+01_JPRB, 0.10106E+01_JPRB, 0.99959E+00_JPRB ,&
&  0.98809E+00_JPRB, 0.97554E+00_JPRB, 0.96084E+00_JPRB, 0.94189E+00_JPRB, 0.92267E+00_JPRB, 0.90189E+00_JPRB, 0.88546E+00_JPRB ,&
&  0.87006E+00_JPRB, 0.86007E+00_JPRB, 0.85114E+00_JPRB, 0.84603E+00_JPRB, 0.84126E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.78037E+00_JPRB, 0.78743E+00_JPRB, 0.79574E+00_JPRB, 0.80132E+00_JPRB, 0.80588E+00_JPRB, 0.80953E+00_JPRB, 0.81138E+00_JPRB ,&
&  0.81266E+00_JPRB, 0.81400E+00_JPRB, 0.81631E+00_JPRB, 0.81875E+00_JPRB, 0.82131E+00_JPRB, 0.82455E+00_JPRB, 0.82665E+00_JPRB ,&
&  0.82881E+00_JPRB, 0.81967E+00_JPRB, 0.79677E+00_JPRB, 0.76942E+00_JPRB, 0.73863E+00_JPRB, 0.69840E+00_JPRB, 0.65219E+00_JPRB ,&
&  0.61282E+00_JPRB, 0.58419E+00_JPRB, 0.54907E+00_JPRB, 0.52295E+00_JPRB, 0.49976E+00_JPRB, 0.48830E+00_JPRB, 0.47266E+00_JPRB ,&
&  0.47100E+00_JPRB, 0.42939E+00_JPRB, 0.41922E+00_JPRB, 0.39148E+00_JPRB, 0.37271E+00_JPRB, 0.35838E+00_JPRB, 0.34895E+00_JPRB ,&
&  0.34631E+00_JPRB, 0.34304E+00_JPRB, 0.34352E+00_JPRB, 0.34849E+00_JPRB, 0.35529E+00_JPRB, 0.36965E+00_JPRB, 0.38694E+00_JPRB ,&
&  0.42971E+00_JPRB, 0.56411E+00_JPRB, 0.61501E+00_JPRB, 0.62854E+00_JPRB, 0.63485E+00_JPRB, 0.65599E+00_JPRB, 0.68040E+00_JPRB ,&
&  0.71655E+00_JPRB, 0.74426E+00_JPRB, 0.77239E+00_JPRB, 0.78883E+00_JPRB, 0.82315E+00_JPRB, 0.85451E+00_JPRB, 0.88154E+00_JPRB ,&
&  0.90895E+00_JPRB, 0.93337E+00_JPRB, 0.95885E+00_JPRB, 0.97751E+00_JPRB, 0.98221E+00_JPRB, 0.99063E+00_JPRB, 0.99800E+00_JPRB ,&
&  0.10113E+01_JPRB, 0.10235E+01_JPRB, 0.10604E+01_JPRB, 0.11093E+01_JPRB, 0.11620E+01_JPRB, 0.11963E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.12389E+01_JPRB, 0.12514E+01_JPRB, 0.12636E+01_JPRB, 0.12481E+01_JPRB, 0.12188E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.11451E+01_JPRB, 0.11111E+01_JPRB, 0.10842E+01_JPRB, 0.10606E+01_JPRB, 0.10406E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.99200E+00_JPRB, 0.96133E+00_JPRB, 0.92857E+00_JPRB, 0.88997E+00_JPRB, 0.85582E+00_JPRB, 0.82154E+00_JPRB, 0.80035E+00_JPRB ,&
&  0.78179E+00_JPRB, 0.77502E+00_JPRB, 0.76829E+00_JPRB, 0.77063E+00_JPRB, 0.77202E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.73048E+00_JPRB, 0.74721E+00_JPRB, 0.75954E+00_JPRB, 0.76508E+00_JPRB, 0.76998E+00_JPRB, 0.77654E+00_JPRB, 0.77926E+00_JPRB ,&
&  0.78229E+00_JPRB, 0.78290E+00_JPRB, 0.78599E+00_JPRB, 0.78806E+00_JPRB, 0.79249E+00_JPRB, 0.79538E+00_JPRB, 0.79811E+00_JPRB ,&
&  0.80120E+00_JPRB, 0.80127E+00_JPRB, 0.79285E+00_JPRB, 0.77217E+00_JPRB, 0.74240E+00_JPRB, 0.70798E+00_JPRB, 0.66256E+00_JPRB ,&
&  0.61975E+00_JPRB, 0.59074E+00_JPRB, 0.56064E+00_JPRB, 0.53649E+00_JPRB, 0.52289E+00_JPRB, 0.50998E+00_JPRB, 0.50758E+00_JPRB ,&
&  0.50430E+00_JPRB, 0.49047E+00_JPRB, 0.47127E+00_JPRB, 0.44543E+00_JPRB, 0.42227E+00_JPRB, 0.37115E+00_JPRB, 0.36354E+00_JPRB ,&
&  0.35076E+00_JPRB, 0.33985E+00_JPRB, 0.33688E+00_JPRB, 0.33840E+00_JPRB, 0.34384E+00_JPRB, 0.35907E+00_JPRB, 0.38177E+00_JPRB ,&
&  0.41572E+00_JPRB, 0.57379E+00_JPRB, 0.61503E+00_JPRB, 0.63195E+00_JPRB, 0.66163E+00_JPRB, 0.69213E+00_JPRB, 0.71769E+00_JPRB ,&
&  0.74585E+00_JPRB, 0.77346E+00_JPRB, 0.81506E+00_JPRB, 0.85579E+00_JPRB, 0.89767E+00_JPRB, 0.93405E+00_JPRB, 0.95777E+00_JPRB ,&
&  0.98161E+00_JPRB, 0.99336E+00_JPRB, 0.10402E+01_JPRB, 0.10562E+01_JPRB, 0.10504E+01_JPRB, 0.10942E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.11101E+01_JPRB, 0.11383E+01_JPRB, 0.12059E+01_JPRB, 0.13002E+01_JPRB, 0.13752E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.14035E+01_JPRB, 0.13871E+01_JPRB, 0.13614E+01_JPRB, 0.13463E+01_JPRB, 0.12999E+01_JPRB, 0.12450E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.11752E+01_JPRB, 0.11558E+01_JPRB, 0.11362E+01_JPRB, 0.11060E+01_JPRB, 0.10766E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.97611E+00_JPRB, 0.91011E+00_JPRB, 0.85646E+00_JPRB, 0.81162E+00_JPRB, 0.78210E+00_JPRB, 0.75125E+00_JPRB, 0.73400E+00_JPRB ,&
&  0.71660E+00_JPRB, 0.71004E+00_JPRB, 0.70343E+00_JPRB, 0.70730E+00_JPRB, 0.71437E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.70335E+00_JPRB, 0.72496E+00_JPRB, 0.74451E+00_JPRB, 0.75786E+00_JPRB, 0.76219E+00_JPRB, 0.76623E+00_JPRB, 0.76800E+00_JPRB ,&
&  0.77093E+00_JPRB, 0.77344E+00_JPRB, 0.77512E+00_JPRB, 0.77240E+00_JPRB, 0.77007E+00_JPRB, 0.76617E+00_JPRB, 0.76575E+00_JPRB ,&
&  0.76462E+00_JPRB, 0.75517E+00_JPRB, 0.74858E+00_JPRB, 0.74069E+00_JPRB, 0.72492E+00_JPRB, 0.70031E+00_JPRB, 0.66828E+00_JPRB ,&
&  0.62788E+00_JPRB, 0.59345E+00_JPRB, 0.56799E+00_JPRB, 0.54673E+00_JPRB, 0.53089E+00_JPRB, 0.52034E+00_JPRB, 0.51657E+00_JPRB ,&
&  0.52158E+00_JPRB, 0.51319E+00_JPRB, 0.50488E+00_JPRB, 0.47216E+00_JPRB, 0.43554E+00_JPRB, 0.42004E+00_JPRB, 0.38762E+00_JPRB ,&
&  0.36056E+00_JPRB, 0.34189E+00_JPRB, 0.33704E+00_JPRB, 0.33712E+00_JPRB, 0.34551E+00_JPRB, 0.36598E+00_JPRB, 0.41121E+00_JPRB ,&
&  0.48973E+00_JPRB, 0.62403E+00_JPRB, 0.65585E+00_JPRB, 0.68754E+00_JPRB, 0.72155E+00_JPRB, 0.75532E+00_JPRB, 0.78062E+00_JPRB ,&
&  0.80586E+00_JPRB, 0.83257E+00_JPRB, 0.88199E+00_JPRB, 0.93329E+00_JPRB, 0.10016E+01_JPRB, 0.10611E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11191E+01_JPRB, 0.11394E+01_JPRB, 0.11592E+01_JPRB, 0.12183E+01_JPRB, 0.12752E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.13210E+01_JPRB, 0.13299E+01_JPRB, 0.13659E+01_JPRB, 0.14470E+01_JPRB, 0.15393E+01_JPRB, 0.15735E+01_JPRB ,&
&  0.15316E+01_JPRB, 0.15132E+01_JPRB, 0.15620E+01_JPRB, 0.14852E+01_JPRB, 0.14314E+01_JPRB, 0.13581E+01_JPRB, 0.12716E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.11972E+01_JPRB, 0.11865E+01_JPRB, 0.11748E+01_JPRB, 0.11414E+01_JPRB, 0.10973E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.92742E+00_JPRB, 0.85326E+00_JPRB, 0.79877E+00_JPRB, 0.76670E+00_JPRB, 0.74058E+00_JPRB, 0.71191E+00_JPRB, 0.68744E+00_JPRB ,&
&  0.66584E+00_JPRB, 0.65641E+00_JPRB, 0.65869E+00_JPRB, 0.67379E+00_JPRB, 0.68969E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.71525E+00_JPRB, 0.73493E+00_JPRB, 0.74728E+00_JPRB, 0.74547E+00_JPRB, 0.73090E+00_JPRB, 0.70835E+00_JPRB, 0.69526E+00_JPRB ,&
&  0.69382E+00_JPRB, 0.69994E+00_JPRB, 0.70707E+00_JPRB, 0.70639E+00_JPRB, 0.70795E+00_JPRB, 0.71045E+00_JPRB, 0.71516E+00_JPRB ,&
&  0.71931E+00_JPRB, 0.72504E+00_JPRB, 0.73094E+00_JPRB, 0.72237E+00_JPRB, 0.70647E+00_JPRB, 0.68508E+00_JPRB, 0.66093E+00_JPRB ,&
&  0.63253E+00_JPRB, 0.60612E+00_JPRB, 0.58125E+00_JPRB, 0.56511E+00_JPRB, 0.55066E+00_JPRB, 0.54483E+00_JPRB, 0.53979E+00_JPRB ,&
&  0.53439E+00_JPRB, 0.53329E+00_JPRB, 0.53178E+00_JPRB, 0.50841E+00_JPRB, 0.47147E+00_JPRB, 0.43802E+00_JPRB, 0.40320E+00_JPRB ,&
&  0.37701E+00_JPRB, 0.36144E+00_JPRB, 0.35265E+00_JPRB, 0.35181E+00_JPRB, 0.36429E+00_JPRB, 0.39428E+00_JPRB, 0.46540E+00_JPRB ,&
&  0.61665E+00_JPRB, 0.65663E+00_JPRB, 0.67798E+00_JPRB, 0.70129E+00_JPRB, 0.72726E+00_JPRB, 0.75551E+00_JPRB, 0.78802E+00_JPRB ,&
&  0.82542E+00_JPRB, 0.86081E+00_JPRB, 0.90995E+00_JPRB, 0.94966E+00_JPRB, 0.10068E+01_JPRB, 0.10295E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.13683E+01_JPRB, 0.13968E+01_JPRB, 0.13057E+01_JPRB, 0.13340E+01_JPRB, 0.14049E+01_JPRB, 0.14734E+01_JPRB, 0.15342E+01_JPRB ,&
&  0.15705E+01_JPRB, 0.15749E+01_JPRB, 0.15791E+01_JPRB, 0.15829E+01_JPRB, 0.16108E+01_JPRB, 0.16384E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.16174E+01_JPRB, 0.16238E+01_JPRB, 0.16609E+01_JPRB, 0.14006E+01_JPRB, 0.12859E+01_JPRB, 0.12559E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.11711E+01_JPRB, 0.11917E+01_JPRB, 0.12013E+01_JPRB, 0.11215E+01_JPRB, 0.10178E+01_JPRB, 0.90974E+00_JPRB ,&
&  0.84829E+00_JPRB, 0.80231E+00_JPRB, 0.75824E+00_JPRB, 0.73987E+00_JPRB, 0.72008E+00_JPRB, 0.69052E+00_JPRB, 0.65254E+00_JPRB ,&
&  0.64110E+00_JPRB, 0.64603E+00_JPRB, 0.65651E+00_JPRB, 0.68591E+00_JPRB, 0.70406E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.69268E+00_JPRB, 0.73110E+00_JPRB, 0.73692E+00_JPRB, 0.70776E+00_JPRB, 0.67865E+00_JPRB, 0.63051E+00_JPRB, 0.59283E+00_JPRB ,&
&  0.57740E+00_JPRB, 0.57298E+00_JPRB, 0.57625E+00_JPRB, 0.58164E+00_JPRB, 0.59136E+00_JPRB, 0.60690E+00_JPRB, 0.62438E+00_JPRB ,&
&  0.63470E+00_JPRB, 0.64119E+00_JPRB, 0.64986E+00_JPRB, 0.66194E+00_JPRB, 0.66521E+00_JPRB, 0.65317E+00_JPRB, 0.63568E+00_JPRB ,&
&  0.61088E+00_JPRB, 0.59158E+00_JPRB, 0.57582E+00_JPRB, 0.56629E+00_JPRB, 0.56537E+00_JPRB, 0.57106E+00_JPRB, 0.57771E+00_JPRB ,&
&  0.57646E+00_JPRB, 0.56817E+00_JPRB, 0.56233E+00_JPRB, 0.54866E+00_JPRB, 0.51039E+00_JPRB, 0.47094E+00_JPRB, 0.42532E+00_JPRB ,&
&  0.39523E+00_JPRB, 0.38408E+00_JPRB, 0.38664E+00_JPRB, 0.40693E+00_JPRB, 0.46245E+00_JPRB, 0.57205E+00_JPRB, 0.60895E+00_JPRB ,&
&  0.64340E+00_JPRB, 0.66668E+00_JPRB, 0.68709E+00_JPRB, 0.71171E+00_JPRB, 0.73350E+00_JPRB, 0.76594E+00_JPRB, 0.80410E+00_JPRB ,&
&  0.85978E+00_JPRB, 0.91539E+00_JPRB, 0.99015E+00_JPRB, 0.91729E+00_JPRB, 0.95279E+00_JPRB, 0.11336E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.14198E+01_JPRB, 0.14601E+01_JPRB, 0.14940E+01_JPRB, 0.16304E+01_JPRB, 0.16997E+01_JPRB, 0.17884E+01_JPRB, 0.17889E+01_JPRB ,&
&  0.17486E+01_JPRB, 0.16902E+01_JPRB, 0.16371E+01_JPRB, 0.16202E+01_JPRB, 0.16553E+01_JPRB, 0.17059E+01_JPRB, 0.16726E+01_JPRB ,&
&  0.16355E+01_JPRB, 0.15867E+01_JPRB, 0.14426E+01_JPRB, 0.12563E+01_JPRB, 0.11823E+01_JPRB, 0.11632E+01_JPRB, 0.11716E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.11663E+01_JPRB, 0.11770E+01_JPRB, 0.11873E+01_JPRB, 0.10602E+01_JPRB, 0.92675E+00_JPRB, 0.81720E+00_JPRB ,&
&  0.76154E+00_JPRB, 0.73646E+00_JPRB, 0.70885E+00_JPRB, 0.71398E+00_JPRB, 0.69644E+00_JPRB, 0.66902E+00_JPRB, 0.63726E+00_JPRB ,&
&  0.62296E+00_JPRB, 0.61905E+00_JPRB, 0.63775E+00_JPRB, 0.66201E+00_JPRB, 0.67599E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.74247E+00_JPRB, 0.74475E+00_JPRB, 0.70789E+00_JPRB, 0.67919E+00_JPRB, 0.64859E+00_JPRB, 0.62153E+00_JPRB, 0.59398E+00_JPRB ,&
&  0.58633E+00_JPRB, 0.56087E+00_JPRB, 0.53891E+00_JPRB, 0.52016E+00_JPRB, 0.51222E+00_JPRB, 0.51034E+00_JPRB, 0.52466E+00_JPRB ,&
&  0.55501E+00_JPRB, 0.59398E+00_JPRB, 0.61994E+00_JPRB, 0.62243E+00_JPRB, 0.62063E+00_JPRB, 0.60979E+00_JPRB, 0.60188E+00_JPRB ,&
&  0.58639E+00_JPRB, 0.57871E+00_JPRB, 0.58420E+00_JPRB, 0.60270E+00_JPRB, 0.62408E+00_JPRB, 0.63035E+00_JPRB, 0.61735E+00_JPRB ,&
&  0.59685E+00_JPRB, 0.58424E+00_JPRB, 0.58210E+00_JPRB, 0.57769E+00_JPRB, 0.54629E+00_JPRB, 0.49727E+00_JPRB, 0.44753E+00_JPRB ,&
&  0.41371E+00_JPRB, 0.41016E+00_JPRB, 0.45037E+00_JPRB, 0.55194E+00_JPRB, 0.58752E+00_JPRB, 0.61323E+00_JPRB, 0.63470E+00_JPRB ,&
&  0.62839E+00_JPRB, 0.63854E+00_JPRB, 0.68754E+00_JPRB, 0.77733E+00_JPRB, 0.79838E+00_JPRB, 0.83735E+00_JPRB, 0.88327E+00_JPRB ,&
&  0.94604E+00_JPRB, 0.96083E+00_JPRB, 0.10201E+01_JPRB, 0.10142E+01_JPRB, 0.12017E+01_JPRB, 0.16504E+01_JPRB, 0.16203E+01_JPRB ,&
&  0.16764E+01_JPRB, 0.17270E+01_JPRB, 0.18718E+01_JPRB, 0.19289E+01_JPRB, 0.19323E+01_JPRB, 0.19433E+01_JPRB, 0.18951E+01_JPRB ,&
&  0.17828E+01_JPRB, 0.17178E+01_JPRB, 0.16586E+01_JPRB, 0.16785E+01_JPRB, 0.16846E+01_JPRB, 0.16885E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.16499E+01_JPRB, 0.16013E+01_JPRB, 0.14837E+01_JPRB, 0.13318E+01_JPRB, 0.12488E+01_JPRB, 0.11762E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11145E+01_JPRB, 0.11537E+01_JPRB, 0.11846E+01_JPRB, 0.10868E+01_JPRB, 0.92423E+00_JPRB, 0.78996E+00_JPRB ,&
&  0.71504E+00_JPRB, 0.68349E+00_JPRB, 0.64782E+00_JPRB, 0.63857E+00_JPRB, 0.63986E+00_JPRB, 0.63801E+00_JPRB, 0.62840E+00_JPRB ,&
&  0.62185E+00_JPRB, 0.62994E+00_JPRB, 0.66795E+00_JPRB, 0.70929E+00_JPRB, 0.72871E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.73633E+00_JPRB, 0.73438E+00_JPRB, 0.70932E+00_JPRB, 0.69218E+00_JPRB, 0.67631E+00_JPRB, 0.64501E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.59374E+00_JPRB, 0.54349E+00_JPRB, 0.51853E+00_JPRB, 0.50936E+00_JPRB, 0.51169E+00_JPRB, 0.50853E+00_JPRB, 0.49996E+00_JPRB ,&
&  0.51002E+00_JPRB, 0.53970E+00_JPRB, 0.59045E+00_JPRB, 0.61542E+00_JPRB, 0.62319E+00_JPRB, 0.61507E+00_JPRB, 0.59619E+00_JPRB ,&
&  0.57771E+00_JPRB, 0.57727E+00_JPRB, 0.60367E+00_JPRB, 0.65472E+00_JPRB, 0.67833E+00_JPRB, 0.67417E+00_JPRB, 0.65724E+00_JPRB ,&
&  0.63437E+00_JPRB, 0.61807E+00_JPRB, 0.60589E+00_JPRB, 0.59552E+00_JPRB, 0.57232E+00_JPRB, 0.55044E+00_JPRB, 0.50205E+00_JPRB ,&
&  0.45836E+00_JPRB, 0.46448E+00_JPRB, 0.55421E+00_JPRB, 0.59884E+00_JPRB, 0.62428E+00_JPRB, 0.63504E+00_JPRB, 0.65095E+00_JPRB ,&
&  0.68183E+00_JPRB, 0.72308E+00_JPRB, 0.77441E+00_JPRB, 0.83540E+00_JPRB, 0.88624E+00_JPRB, 0.94113E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.94965E+00_JPRB, 0.93668E+00_JPRB, 0.10435E+01_JPRB, 0.14834E+01_JPRB, 0.17165E+01_JPRB, 0.18213E+01_JPRB ,&
&  0.19235E+01_JPRB, 0.18923E+01_JPRB, 0.19537E+01_JPRB, 0.20718E+01_JPRB, 0.21331E+01_JPRB, 0.21361E+01_JPRB, 0.20821E+01_JPRB ,&
&  0.19990E+01_JPRB, 0.19219E+01_JPRB, 0.18257E+01_JPRB, 0.17840E+01_JPRB, 0.17287E+01_JPRB, 0.17286E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.16493E+01_JPRB, 0.16130E+01_JPRB, 0.15027E+01_JPRB, 0.13794E+01_JPRB, 0.12804E+01_JPRB, 0.12155E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.11308E+01_JPRB, 0.11566E+01_JPRB, 0.12079E+01_JPRB, 0.11682E+01_JPRB, 0.10340E+01_JPRB, 0.91115E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.72032E+00_JPRB, 0.65797E+00_JPRB, 0.64418E+00_JPRB, 0.65338E+00_JPRB, 0.66288E+00_JPRB, 0.67558E+00_JPRB ,&
&  0.69504E+00_JPRB, 0.70501E+00_JPRB, 0.71452E+00_JPRB, 0.73568E+00_JPRB, 0.73518E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.72885E+00_JPRB, 0.73782E+00_JPRB, 0.72241E+00_JPRB, 0.71424E+00_JPRB, 0.70390E+00_JPRB, 0.66084E+00_JPRB, 0.63424E+00_JPRB ,&
&  0.65941E+00_JPRB, 0.61567E+00_JPRB, 0.64415E+00_JPRB, 0.56688E+00_JPRB, 0.53539E+00_JPRB, 0.52968E+00_JPRB, 0.51615E+00_JPRB ,&
&  0.52176E+00_JPRB, 0.54865E+00_JPRB, 0.58299E+00_JPRB, 0.61160E+00_JPRB, 0.63764E+00_JPRB, 0.62005E+00_JPRB, 0.60251E+00_JPRB ,&
&  0.58450E+00_JPRB, 0.58767E+00_JPRB, 0.62535E+00_JPRB, 0.65792E+00_JPRB, 0.68653E+00_JPRB, 0.69988E+00_JPRB, 0.69585E+00_JPRB ,&
&  0.65361E+00_JPRB, 0.64339E+00_JPRB, 0.64272E+00_JPRB, 0.62480E+00_JPRB, 0.60196E+00_JPRB, 0.58752E+00_JPRB, 0.56576E+00_JPRB ,&
&  0.54440E+00_JPRB, 0.54320E+00_JPRB, 0.59542E+00_JPRB, 0.62962E+00_JPRB, 0.64583E+00_JPRB, 0.64824E+00_JPRB, 0.66235E+00_JPRB ,&
&  0.68471E+00_JPRB, 0.72073E+00_JPRB, 0.78200E+00_JPRB, 0.87150E+00_JPRB, 0.95723E+00_JPRB, 0.10579E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.13069E+01_JPRB, 0.10563E+01_JPRB, 0.12486E+01_JPRB, 0.13202E+01_JPRB, 0.15670E+01_JPRB, 0.18411E+01_JPRB, 0.19106E+01_JPRB ,&
&  0.21691E+01_JPRB, 0.21479E+01_JPRB, 0.22135E+01_JPRB, 0.22601E+01_JPRB, 0.23223E+01_JPRB, 0.23499E+01_JPRB, 0.23243E+01_JPRB ,&
&  0.23338E+01_JPRB, 0.20697E+01_JPRB, 0.20675E+01_JPRB, 0.19586E+01_JPRB, 0.18609E+01_JPRB, 0.18282E+01_JPRB, 0.17475E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.16360E+01_JPRB, 0.15681E+01_JPRB, 0.13797E+01_JPRB, 0.13200E+01_JPRB, 0.12702E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11193E+01_JPRB, 0.10776E+01_JPRB, 0.10416E+01_JPRB, 0.11673E+01_JPRB, 0.11666E+01_JPRB, 0.10438E+01_JPRB, 0.90928E+00_JPRB ,&
&  0.81278E+00_JPRB, 0.73611E+00_JPRB, 0.75226E+00_JPRB, 0.74839E+00_JPRB, 0.72352E+00_JPRB, 0.71815E+00_JPRB, 0.77249E+00_JPRB ,&
&  0.74820E+00_JPRB, 0.75260E+00_JPRB, 0.78026E+00_JPRB, 0.77116E+00_JPRB, 0.74463E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.74358E+00_JPRB, 0.76123E+00_JPRB, 0.73948E+00_JPRB, 0.73597E+00_JPRB, 0.73412E+00_JPRB, 0.69739E+00_JPRB, 0.65890E+00_JPRB ,&
&  0.66685E+00_JPRB, 0.66763E+00_JPRB, 0.67222E+00_JPRB, 0.67478E+00_JPRB, 0.67822E+00_JPRB, 0.62352E+00_JPRB, 0.52853E+00_JPRB ,&
&  0.50823E+00_JPRB, 0.53906E+00_JPRB, 0.59544E+00_JPRB, 0.63533E+00_JPRB, 0.63963E+00_JPRB, 0.62552E+00_JPRB, 0.61927E+00_JPRB ,&
&  0.62264E+00_JPRB, 0.62904E+00_JPRB, 0.65953E+00_JPRB, 0.68162E+00_JPRB, 0.69237E+00_JPRB, 0.71100E+00_JPRB, 0.72633E+00_JPRB ,&
&  0.69988E+00_JPRB, 0.70263E+00_JPRB, 0.69545E+00_JPRB, 0.64839E+00_JPRB, 0.65311E+00_JPRB, 0.62962E+00_JPRB, 0.60097E+00_JPRB ,&
&  0.59437E+00_JPRB, 0.61284E+00_JPRB, 0.63390E+00_JPRB, 0.66189E+00_JPRB, 0.67945E+00_JPRB, 0.68550E+00_JPRB, 0.69472E+00_JPRB ,&
&  0.71943E+00_JPRB, 0.76772E+00_JPRB, 0.82832E+00_JPRB, 0.91717E+00_JPRB, 0.10345E+01_JPRB, 0.11513E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.14772E+01_JPRB, 0.15515E+01_JPRB, 0.14758E+01_JPRB, 0.16868E+01_JPRB, 0.19627E+01_JPRB, 0.20326E+01_JPRB ,&
&  0.21981E+01_JPRB, 0.21501E+01_JPRB, 0.23314E+01_JPRB, 0.25403E+01_JPRB, 0.27107E+01_JPRB, 0.26875E+01_JPRB, 0.26025E+01_JPRB ,&
&  0.26203E+01_JPRB, 0.25183E+01_JPRB, 0.22585E+01_JPRB, 0.21379E+01_JPRB, 0.20124E+01_JPRB, 0.19591E+01_JPRB, 0.18304E+01_JPRB ,&
&  0.17516E+01_JPRB, 0.17098E+01_JPRB, 0.16428E+01_JPRB, 0.15585E+01_JPRB, 0.14566E+01_JPRB, 0.13566E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.11548E+01_JPRB, 0.10167E+01_JPRB, 0.97114E+00_JPRB, 0.10489E+01_JPRB, 0.11305E+01_JPRB, 0.98939E+00_JPRB, 0.84477E+00_JPRB ,&
&  0.78705E+00_JPRB, 0.78504E+00_JPRB, 0.85592E+00_JPRB, 0.84907E+00_JPRB, 0.82790E+00_JPRB, 0.81155E+00_JPRB, 0.77216E+00_JPRB ,&
&  0.72610E+00_JPRB, 0.78558E+00_JPRB, 0.79268E+00_JPRB, 0.77325E+00_JPRB, 0.74692E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.78053E+00_JPRB, 0.81219E+00_JPRB, 0.78556E+00_JPRB, 0.77548E+00_JPRB, 0.77571E+00_JPRB, 0.72634E+00_JPRB, 0.70814E+00_JPRB ,&
&  0.70705E+00_JPRB, 0.70482E+00_JPRB, 0.69819E+00_JPRB, 0.69775E+00_JPRB, 0.69900E+00_JPRB, 0.69063E+00_JPRB, 0.63289E+00_JPRB ,&
&  0.57179E+00_JPRB, 0.62173E+00_JPRB, 0.62371E+00_JPRB, 0.66890E+00_JPRB, 0.71339E+00_JPRB, 0.68180E+00_JPRB, 0.65986E+00_JPRB ,&
&  0.67281E+00_JPRB, 0.66799E+00_JPRB, 0.67412E+00_JPRB, 0.69645E+00_JPRB, 0.72998E+00_JPRB, 0.78028E+00_JPRB, 0.80880E+00_JPRB ,&
&  0.77450E+00_JPRB, 0.76621E+00_JPRB, 0.73858E+00_JPRB, 0.72081E+00_JPRB, 0.71178E+00_JPRB, 0.69563E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.66040E+00_JPRB, 0.67132E+00_JPRB, 0.68534E+00_JPRB, 0.71371E+00_JPRB, 0.73980E+00_JPRB, 0.75720E+00_JPRB, 0.77358E+00_JPRB ,&
&  0.82160E+00_JPRB, 0.88413E+00_JPRB, 0.93727E+00_JPRB, 0.10200E+01_JPRB, 0.11335E+01_JPRB, 0.13787E+01_JPRB, 0.15817E+01_JPRB ,&
&  0.16777E+01_JPRB, 0.17068E+01_JPRB, 0.17545E+01_JPRB, 0.18184E+01_JPRB, 0.20119E+01_JPRB, 0.21640E+01_JPRB, 0.22832E+01_JPRB ,&
&  0.24598E+01_JPRB, 0.24115E+01_JPRB, 0.28526E+01_JPRB, 0.30182E+01_JPRB, 0.29916E+01_JPRB, 0.30088E+01_JPRB, 0.28330E+01_JPRB ,&
&  0.27361E+01_JPRB, 0.25133E+01_JPRB, 0.24032E+01_JPRB, 0.22869E+01_JPRB, 0.21643E+01_JPRB, 0.21297E+01_JPRB, 0.19836E+01_JPRB ,&
&  0.19287E+01_JPRB, 0.18542E+01_JPRB, 0.16097E+01_JPRB, 0.14323E+01_JPRB, 0.12285E+01_JPRB, 0.12986E+01_JPRB, 0.12455E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.95581E+00_JPRB, 0.10286E+01_JPRB, 0.11388E+01_JPRB, 0.12496E+01_JPRB, 0.12340E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.89020E+00_JPRB, 0.90722E+00_JPRB, 0.90666E+00_JPRB, 0.91602E+00_JPRB, 0.92090E+00_JPRB, 0.92249E+00_JPRB, 0.82323E+00_JPRB ,&
&  0.76274E+00_JPRB, 0.80079E+00_JPRB, 0.80781E+00_JPRB, 0.79061E+00_JPRB, 0.77614E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.82713E+00_JPRB, 0.87095E+00_JPRB, 0.86533E+00_JPRB, 0.83250E+00_JPRB, 0.81852E+00_JPRB, 0.76280E+00_JPRB, 0.74374E+00_JPRB ,&
&  0.74919E+00_JPRB, 0.74613E+00_JPRB, 0.72395E+00_JPRB, 0.72157E+00_JPRB, 0.72228E+00_JPRB, 0.71594E+00_JPRB, 0.70814E+00_JPRB ,&
&  0.61743E+00_JPRB, 0.69117E+00_JPRB, 0.64207E+00_JPRB, 0.68088E+00_JPRB, 0.83604E+00_JPRB, 0.79374E+00_JPRB, 0.76776E+00_JPRB ,&
&  0.74414E+00_JPRB, 0.73406E+00_JPRB, 0.71265E+00_JPRB, 0.72490E+00_JPRB, 0.76430E+00_JPRB, 0.85113E+00_JPRB, 0.93753E+00_JPRB ,&
&  0.92846E+00_JPRB, 0.89663E+00_JPRB, 0.85898E+00_JPRB, 0.83384E+00_JPRB, 0.79198E+00_JPRB, 0.77427E+00_JPRB, 0.78126E+00_JPRB ,&
&  0.78031E+00_JPRB, 0.77233E+00_JPRB, 0.78490E+00_JPRB, 0.81917E+00_JPRB, 0.84794E+00_JPRB, 0.86590E+00_JPRB, 0.90806E+00_JPRB ,&
&  0.97719E+00_JPRB, 0.10336E+01_JPRB, 0.10691E+01_JPRB, 0.11471E+01_JPRB, 0.12474E+01_JPRB, 0.14836E+01_JPRB, 0.17287E+01_JPRB ,&
&  0.20558E+01_JPRB, 0.20817E+01_JPRB, 0.22571E+01_JPRB, 0.25602E+01_JPRB, 0.26811E+01_JPRB, 0.26939E+01_JPRB, 0.26860E+01_JPRB ,&
&  0.27677E+01_JPRB, 0.27401E+01_JPRB, 0.29097E+01_JPRB, 0.31156E+01_JPRB, 0.30505E+01_JPRB, 0.31646E+01_JPRB, 0.29590E+01_JPRB ,&
&  0.26155E+01_JPRB, 0.24266E+01_JPRB, 0.23385E+01_JPRB, 0.22532E+01_JPRB, 0.21147E+01_JPRB, 0.21466E+01_JPRB, 0.20507E+01_JPRB ,&
&  0.20166E+01_JPRB, 0.17816E+01_JPRB, 0.13343E+01_JPRB, 0.12253E+01_JPRB, 0.95881E+00_JPRB, 0.98828E+00_JPRB, 0.12538E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.11447E+01_JPRB, 0.11943E+01_JPRB, 0.13250E+01_JPRB, 0.14831E+01_JPRB, 0.17057E+01_JPRB, 0.14602E+01_JPRB ,&
&  0.11670E+01_JPRB, 0.11079E+01_JPRB, 0.10870E+01_JPRB, 0.10709E+01_JPRB, 0.10614E+01_JPRB, 0.10781E+01_JPRB, 0.10283E+01_JPRB ,&
&  0.96053E+00_JPRB, 0.90223E+00_JPRB, 0.87246E+00_JPRB, 0.84342E+00_JPRB, 0.82982E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.87577E+00_JPRB, 0.91382E+00_JPRB, 0.92527E+00_JPRB, 0.88784E+00_JPRB, 0.85938E+00_JPRB, 0.81152E+00_JPRB, 0.78240E+00_JPRB ,&
&  0.78490E+00_JPRB, 0.77808E+00_JPRB, 0.75106E+00_JPRB, 0.74771E+00_JPRB, 0.74420E+00_JPRB, 0.73194E+00_JPRB, 0.73766E+00_JPRB ,&
&  0.80337E+00_JPRB, 0.86713E+00_JPRB, 0.76554E+00_JPRB, 0.69472E+00_JPRB, 0.82460E+00_JPRB, 0.81042E+00_JPRB, 0.84306E+00_JPRB ,&
&  0.81008E+00_JPRB, 0.84634E+00_JPRB, 0.84370E+00_JPRB, 0.86794E+00_JPRB, 0.95306E+00_JPRB, 0.10624E+01_JPRB, 0.11599E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.12230E+01_JPRB, 0.11803E+01_JPRB, 0.11375E+01_JPRB, 0.11191E+01_JPRB, 0.10368E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.95870E+00_JPRB, 0.92454E+00_JPRB, 0.93531E+00_JPRB, 0.95246E+00_JPRB, 0.97720E+00_JPRB, 0.10337E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.11126E+01_JPRB, 0.11132E+01_JPRB, 0.11322E+01_JPRB, 0.12094E+01_JPRB, 0.13571E+01_JPRB, 0.15894E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.20464E+01_JPRB, 0.21825E+01_JPRB, 0.25836E+01_JPRB, 0.27681E+01_JPRB, 0.27464E+01_JPRB, 0.27603E+01_JPRB ,&
&  0.28730E+01_JPRB, 0.29518E+01_JPRB, 0.31264E+01_JPRB, 0.32238E+01_JPRB, 0.32716E+01_JPRB, 0.32517E+01_JPRB, 0.29633E+01_JPRB ,&
&  0.25511E+01_JPRB, 0.23012E+01_JPRB, 0.22146E+01_JPRB, 0.20696E+01_JPRB, 0.19651E+01_JPRB, 0.17986E+01_JPRB, 0.18621E+01_JPRB ,&
&  0.19835E+01_JPRB, 0.11146E+01_JPRB, 0.87781E+00_JPRB, 0.10484E+01_JPRB, 0.90332E+00_JPRB, 0.93922E+00_JPRB, 0.11038E+01_JPRB ,&
&  0.10770E+01_JPRB, 0.10740E+01_JPRB, 0.12844E+01_JPRB, 0.14463E+01_JPRB, 0.16675E+01_JPRB, 0.19878E+01_JPRB, 0.17881E+01_JPRB ,&
&  0.15471E+01_JPRB, 0.14226E+01_JPRB, 0.14194E+01_JPRB, 0.13581E+01_JPRB, 0.13625E+01_JPRB, 0.13222E+01_JPRB, 0.12581E+01_JPRB ,&
&  0.11635E+01_JPRB, 0.10903E+01_JPRB, 0.10139E+01_JPRB, 0.94994E+00_JPRB, 0.90100E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.97441E+00_JPRB, 0.99112E+00_JPRB, 0.98290E+00_JPRB, 0.94701E+00_JPRB, 0.90412E+00_JPRB, 0.87500E+00_JPRB, 0.83750E+00_JPRB ,&
&  0.82144E+00_JPRB, 0.80655E+00_JPRB, 0.78015E+00_JPRB, 0.77258E+00_JPRB, 0.75426E+00_JPRB, 0.73761E+00_JPRB, 0.74610E+00_JPRB ,&
&  0.79540E+00_JPRB, 0.87528E+00_JPRB, 0.72677E+00_JPRB, 0.69415E+00_JPRB, 0.67335E+00_JPRB, 0.74822E+00_JPRB, 0.81844E+00_JPRB ,&
&  0.90031E+00_JPRB, 0.92463E+00_JPRB, 0.96818E+00_JPRB, 0.10605E+01_JPRB, 0.12157E+01_JPRB, 0.13165E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.15195E+01_JPRB, 0.16051E+01_JPRB, 0.14926E+01_JPRB, 0.15049E+01_JPRB, 0.15110E+01_JPRB, 0.14018E+01_JPRB, 0.13987E+01_JPRB ,&
&  0.12553E+01_JPRB, 0.11655E+01_JPRB, 0.11021E+01_JPRB, 0.10584E+01_JPRB, 0.10426E+01_JPRB, 0.10336E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.10937E+01_JPRB, 0.10896E+01_JPRB, 0.10939E+01_JPRB, 0.11461E+01_JPRB, 0.12602E+01_JPRB, 0.13550E+01_JPRB ,&
&  0.16472E+01_JPRB, 0.16501E+01_JPRB, 0.14416E+01_JPRB, 0.21012E+01_JPRB, 0.28328E+01_JPRB, 0.26557E+01_JPRB, 0.23918E+01_JPRB ,&
&  0.26479E+01_JPRB, 0.28637E+01_JPRB, 0.31193E+01_JPRB, 0.32233E+01_JPRB, 0.32475E+01_JPRB, 0.31370E+01_JPRB, 0.28731E+01_JPRB ,&
&  0.25309E+01_JPRB, 0.23650E+01_JPRB, 0.23141E+01_JPRB, 0.23774E+01_JPRB, 0.22370E+01_JPRB, 0.20973E+01_JPRB, 0.19857E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.14615E+01_JPRB, 0.10253E+01_JPRB, 0.10742E+01_JPRB, 0.86159E+00_JPRB, 0.87986E+00_JPRB, 0.10306E+01_JPRB ,&
&  0.10106E+01_JPRB, 0.11069E+01_JPRB, 0.13385E+01_JPRB, 0.15602E+01_JPRB, 0.20698E+01_JPRB, 0.22489E+01_JPRB, 0.21968E+01_JPRB ,&
&  0.19314E+01_JPRB, 0.16674E+01_JPRB, 0.17139E+01_JPRB, 0.16830E+01_JPRB, 0.16812E+01_JPRB, 0.15965E+01_JPRB, 0.14919E+01_JPRB ,&
&  0.13696E+01_JPRB, 0.12620E+01_JPRB, 0.11863E+01_JPRB, 0.10915E+01_JPRB, 0.10030E+01_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.10984E+01_JPRB, 0.10769E+01_JPRB, 0.10304E+01_JPRB, 0.96988E+00_JPRB, 0.91470E+00_JPRB, 0.88982E+00_JPRB, 0.86708E+00_JPRB ,&
&  0.85545E+00_JPRB, 0.83236E+00_JPRB, 0.81010E+00_JPRB, 0.78927E+00_JPRB, 0.77035E+00_JPRB, 0.75504E+00_JPRB, 0.76758E+00_JPRB ,&
&  0.80660E+00_JPRB, 0.76943E+00_JPRB, 0.63108E+00_JPRB, 0.58122E+00_JPRB, 0.56163E+00_JPRB, 0.63920E+00_JPRB, 0.73964E+00_JPRB ,&
&  0.86255E+00_JPRB, 0.94219E+00_JPRB, 0.10834E+01_JPRB, 0.12668E+01_JPRB, 0.14726E+01_JPRB, 0.16434E+01_JPRB, 0.17710E+01_JPRB ,&
&  0.21383E+01_JPRB, 0.19624E+01_JPRB, 0.19986E+01_JPRB, 0.18969E+01_JPRB, 0.18426E+01_JPRB, 0.16639E+01_JPRB, 0.14805E+01_JPRB ,&
&  0.13428E+01_JPRB, 0.12268E+01_JPRB, 0.11272E+01_JPRB, 0.10486E+01_JPRB, 0.10099E+01_JPRB, 0.10130E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.11115E+01_JPRB, 0.11014E+01_JPRB, 0.10716E+01_JPRB, 0.10965E+01_JPRB, 0.11968E+01_JPRB, 0.13017E+01_JPRB, 0.13851E+01_JPRB ,&
&  0.14254E+01_JPRB, 0.15369E+01_JPRB, 0.22373E+01_JPRB, 0.24281E+01_JPRB, 0.26889E+01_JPRB, 0.26934E+01_JPRB, 0.25971E+01_JPRB ,&
&  0.27845E+01_JPRB, 0.28763E+01_JPRB, 0.30202E+01_JPRB, 0.25032E+01_JPRB, 0.28902E+01_JPRB, 0.28514E+01_JPRB, 0.26333E+01_JPRB ,&
&  0.24335E+01_JPRB, 0.24256E+01_JPRB, 0.23314E+01_JPRB, 0.23446E+01_JPRB, 0.23942E+01_JPRB, 0.24758E+01_JPRB, 0.19275E+01_JPRB ,&
&  0.15266E+01_JPRB, 0.16459E+01_JPRB, 0.14697E+01_JPRB, 0.12297E+01_JPRB, 0.10474E+01_JPRB, 0.10849E+01_JPRB, 0.11199E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.12984E+01_JPRB, 0.15482E+01_JPRB, 0.20497E+01_JPRB, 0.26318E+01_JPRB, 0.28325E+01_JPRB, 0.26297E+01_JPRB ,&
&  0.21998E+01_JPRB, 0.22520E+01_JPRB, 0.21625E+01_JPRB, 0.20589E+01_JPRB, 0.20040E+01_JPRB, 0.18871E+01_JPRB, 0.17204E+01_JPRB ,&
&  0.15461E+01_JPRB, 0.14162E+01_JPRB, 0.13103E+01_JPRB, 0.12057E+01_JPRB, 0.11292E+01_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.11759E+01_JPRB, 0.11400E+01_JPRB, 0.10814E+01_JPRB, 0.10174E+01_JPRB, 0.95677E+00_JPRB, 0.92468E+00_JPRB, 0.90149E+00_JPRB ,&
&  0.89647E+00_JPRB, 0.84884E+00_JPRB, 0.82633E+00_JPRB, 0.80302E+00_JPRB, 0.78618E+00_JPRB, 0.78040E+00_JPRB, 0.79707E+00_JPRB ,&
&  0.82364E+00_JPRB, 0.72024E+00_JPRB, 0.58020E+00_JPRB, 0.59245E+00_JPRB, 0.64029E+00_JPRB, 0.64483E+00_JPRB, 0.73462E+00_JPRB ,&
&  0.84866E+00_JPRB, 0.98093E+00_JPRB, 0.11522E+01_JPRB, 0.14671E+01_JPRB, 0.17801E+01_JPRB, 0.20634E+01_JPRB, 0.23108E+01_JPRB ,&
&  0.23648E+01_JPRB, 0.24440E+01_JPRB, 0.23732E+01_JPRB, 0.21396E+01_JPRB, 0.18281E+01_JPRB, 0.15749E+01_JPRB, 0.14233E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.11881E+01_JPRB, 0.11071E+01_JPRB, 0.10635E+01_JPRB, 0.10517E+01_JPRB, 0.10758E+01_JPRB, 0.11189E+01_JPRB ,&
&  0.11396E+01_JPRB, 0.11162E+01_JPRB, 0.10976E+01_JPRB, 0.11588E+01_JPRB, 0.11681E+01_JPRB, 0.12052E+01_JPRB, 0.12005E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.19382E+01_JPRB, 0.21242E+01_JPRB, 0.23965E+01_JPRB, 0.21127E+01_JPRB, 0.22320E+01_JPRB, 0.26533E+01_JPRB ,&
&  0.28949E+01_JPRB, 0.29278E+01_JPRB, 0.30584E+01_JPRB, 0.21833E+01_JPRB, 0.19117E+01_JPRB, 0.26064E+01_JPRB, 0.25345E+01_JPRB ,&
&  0.23393E+01_JPRB, 0.24213E+01_JPRB, 0.24203E+01_JPRB, 0.26070E+01_JPRB, 0.20112E+01_JPRB, 0.17801E+01_JPRB, 0.13356E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.11688E+01_JPRB, 0.13368E+01_JPRB, 0.12555E+01_JPRB, 0.12944E+01_JPRB, 0.13016E+01_JPRB, 0.12686E+01_JPRB ,&
&  0.13422E+01_JPRB, 0.15242E+01_JPRB, 0.20159E+01_JPRB, 0.28578E+01_JPRB, 0.36331E+01_JPRB, 0.32473E+01_JPRB, 0.32262E+01_JPRB ,&
&  0.30787E+01_JPRB, 0.28871E+01_JPRB, 0.25640E+01_JPRB, 0.24975E+01_JPRB, 0.23404E+01_JPRB, 0.21089E+01_JPRB, 0.18410E+01_JPRB ,&
&  0.16230E+01_JPRB, 0.14909E+01_JPRB, 0.13717E+01_JPRB, 0.12755E+01_JPRB, 0.12122E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.12479E+01_JPRB, 0.11966E+01_JPRB, 0.11386E+01_JPRB, 0.10725E+01_JPRB, 0.10140E+01_JPRB, 0.97284E+00_JPRB, 0.94601E+00_JPRB ,&
&  0.94269E+00_JPRB, 0.88390E+00_JPRB, 0.83261E+00_JPRB, 0.80908E+00_JPRB, 0.80152E+00_JPRB, 0.80520E+00_JPRB, 0.82402E+00_JPRB ,&
&  0.86568E+00_JPRB, 0.84612E+00_JPRB, 0.65054E+00_JPRB, 0.60008E+00_JPRB, 0.71314E+00_JPRB, 0.70139E+00_JPRB, 0.73732E+00_JPRB ,&
&  0.88433E+00_JPRB, 0.10564E+01_JPRB, 0.13228E+01_JPRB, 0.17910E+01_JPRB, 0.21936E+01_JPRB, 0.26021E+01_JPRB, 0.28287E+01_JPRB ,&
&  0.29320E+01_JPRB, 0.26769E+01_JPRB, 0.22839E+01_JPRB, 0.18913E+01_JPRB, 0.15956E+01_JPRB, 0.14041E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.11340E+01_JPRB, 0.10947E+01_JPRB, 0.10614E+01_JPRB, 0.10826E+01_JPRB, 0.10912E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.11381E+01_JPRB, 0.11285E+01_JPRB, 0.11493E+01_JPRB, 0.11842E+01_JPRB, 0.12081E+01_JPRB, 0.11945E+01_JPRB, 0.12705E+01_JPRB ,&
&  0.14252E+01_JPRB, 0.16781E+01_JPRB, 0.19550E+01_JPRB, 0.21972E+01_JPRB, 0.24151E+01_JPRB, 0.24626E+01_JPRB, 0.28021E+01_JPRB ,&
&  0.23863E+01_JPRB, 0.19731E+01_JPRB, 0.16832E+01_JPRB, 0.13746E+01_JPRB, 0.16760E+01_JPRB, 0.24288E+01_JPRB, 0.25044E+01_JPRB ,&
&  0.24669E+01_JPRB, 0.25590E+01_JPRB, 0.26002E+01_JPRB, 0.22378E+01_JPRB, 0.18395E+01_JPRB, 0.14381E+01_JPRB, 0.17084E+01_JPRB ,&
&  0.17695E+01_JPRB, 0.17548E+01_JPRB, 0.16318E+01_JPRB, 0.14466E+01_JPRB, 0.12841E+01_JPRB, 0.15334E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.16565E+01_JPRB, 0.18893E+01_JPRB, 0.32323E+01_JPRB, 0.43138E+01_JPRB, 0.44055E+01_JPRB, 0.39816E+01_JPRB, 0.40368E+01_JPRB ,&
&  0.36505E+01_JPRB, 0.34310E+01_JPRB, 0.30729E+01_JPRB, 0.26973E+01_JPRB, 0.23546E+01_JPRB, 0.20609E+01_JPRB, 0.18216E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.15361E+01_JPRB, 0.14129E+01_JPRB, 0.13438E+01_JPRB, 0.12909E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.13259E+01_JPRB, 0.12658E+01_JPRB, 0.12085E+01_JPRB, 0.11467E+01_JPRB, 0.10906E+01_JPRB, 0.10282E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.98277E+00_JPRB, 0.91819E+00_JPRB, 0.85664E+00_JPRB, 0.83251E+00_JPRB, 0.84166E+00_JPRB, 0.85475E+00_JPRB, 0.87028E+00_JPRB ,&
&  0.90793E+00_JPRB, 0.10728E+01_JPRB, 0.82230E+00_JPRB, 0.69868E+00_JPRB, 0.73967E+00_JPRB, 0.75105E+00_JPRB, 0.71002E+00_JPRB ,&
&  0.91947E+00_JPRB, 0.11704E+01_JPRB, 0.14705E+01_JPRB, 0.19851E+01_JPRB, 0.26847E+01_JPRB, 0.27067E+01_JPRB, 0.28242E+01_JPRB ,&
&  0.28241E+01_JPRB, 0.24417E+01_JPRB, 0.20471E+01_JPRB, 0.17313E+01_JPRB, 0.15111E+01_JPRB, 0.13606E+01_JPRB, 0.12729E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11714E+01_JPRB, 0.11334E+01_JPRB, 0.11252E+01_JPRB, 0.11144E+01_JPRB, 0.11133E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.11245E+01_JPRB, 0.11371E+01_JPRB, 0.11960E+01_JPRB, 0.13168E+01_JPRB, 0.12071E+01_JPRB, 0.15496E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.16102E+01_JPRB, 0.18319E+01_JPRB, 0.23445E+01_JPRB, 0.25029E+01_JPRB, 0.24754E+01_JPRB, 0.26477E+01_JPRB ,&
&  0.21204E+01_JPRB, 0.18942E+01_JPRB, 0.19999E+01_JPRB, 0.20035E+01_JPRB, 0.16833E+01_JPRB, 0.17537E+01_JPRB, 0.26211E+01_JPRB ,&
&  0.21214E+01_JPRB, 0.23390E+01_JPRB, 0.24520E+01_JPRB, 0.24188E+01_JPRB, 0.12203E+01_JPRB, 0.93165E+00_JPRB, 0.13959E+01_JPRB ,&
&  0.15479E+01_JPRB, 0.10808E+01_JPRB, 0.81778E+00_JPRB, 0.10007E+01_JPRB, 0.82155E+00_JPRB, 0.10827E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.20070E+01_JPRB, 0.25318E+01_JPRB, 0.43559E+01_JPRB, 0.50126E+01_JPRB, 0.48871E+01_JPRB, 0.47331E+01_JPRB, 0.42469E+01_JPRB ,&
&  0.39203E+01_JPRB, 0.32807E+01_JPRB, 0.31623E+01_JPRB, 0.27572E+01_JPRB, 0.23952E+01_JPRB, 0.21328E+01_JPRB, 0.19200E+01_JPRB ,&
&  0.17557E+01_JPRB, 0.16149E+01_JPRB, 0.15070E+01_JPRB, 0.14394E+01_JPRB, 0.13845E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.14386E+01_JPRB, 0.13613E+01_JPRB, 0.12879E+01_JPRB, 0.12182E+01_JPRB, 0.11478E+01_JPRB, 0.10922E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.94665E+00_JPRB, 0.89479E+00_JPRB, 0.85491E+00_JPRB, 0.85457E+00_JPRB, 0.86824E+00_JPRB, 0.86435E+00_JPRB ,&
&  0.86929E+00_JPRB, 0.91994E+00_JPRB, 0.97820E+00_JPRB, 0.88872E+00_JPRB, 0.82696E+00_JPRB, 0.78370E+00_JPRB, 0.79443E+00_JPRB ,&
&  0.10629E+01_JPRB, 0.13906E+01_JPRB, 0.17250E+01_JPRB, 0.21418E+01_JPRB, 0.25157E+01_JPRB, 0.25214E+01_JPRB, 0.27271E+01_JPRB ,&
&  0.26010E+01_JPRB, 0.23079E+01_JPRB, 0.20093E+01_JPRB, 0.17218E+01_JPRB, 0.15228E+01_JPRB, 0.13740E+01_JPRB, 0.12656E+01_JPRB ,&
&  0.12041E+01_JPRB, 0.11709E+01_JPRB, 0.11405E+01_JPRB, 0.11092E+01_JPRB, 0.10911E+01_JPRB, 0.10935E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.10958E+01_JPRB, 0.11111E+01_JPRB, 0.11379E+01_JPRB, 0.12251E+01_JPRB, 0.12362E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.12327E+01_JPRB, 0.17586E+01_JPRB, 0.21426E+01_JPRB, 0.22447E+01_JPRB, 0.23401E+01_JPRB, 0.22887E+01_JPRB ,&
&  0.22313E+01_JPRB, 0.23322E+01_JPRB, 0.21397E+01_JPRB, 0.22638E+01_JPRB, 0.20853E+01_JPRB, 0.15565E+01_JPRB, 0.19432E+01_JPRB ,&
&  0.21080E+01_JPRB, 0.20744E+01_JPRB, 0.16983E+01_JPRB, 0.12404E+01_JPRB, 0.12983E+01_JPRB, 0.10500E+01_JPRB, 0.76933E+00_JPRB ,&
&  0.73144E+00_JPRB, 0.70046E+00_JPRB, 0.64356E+00_JPRB, 0.65408E+00_JPRB, 0.66937E+00_JPRB, 0.10612E+01_JPRB, 0.17074E+01_JPRB ,&
&  0.27260E+01_JPRB, 0.39598E+01_JPRB, 0.51724E+01_JPRB, 0.56005E+01_JPRB, 0.50408E+01_JPRB, 0.47621E+01_JPRB, 0.43748E+01_JPRB ,&
&  0.35421E+01_JPRB, 0.31590E+01_JPRB, 0.29690E+01_JPRB, 0.26487E+01_JPRB, 0.23785E+01_JPRB, 0.21857E+01_JPRB, 0.20261E+01_JPRB ,&
&  0.18686E+01_JPRB, 0.17291E+01_JPRB, 0.16307E+01_JPRB, 0.15665E+01_JPRB, 0.15088E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.15014E+01_JPRB, 0.14220E+01_JPRB, 0.13490E+01_JPRB, 0.12851E+01_JPRB, 0.12227E+01_JPRB, 0.11685E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.10508E+01_JPRB, 0.96613E+00_JPRB, 0.91042E+00_JPRB, 0.87565E+00_JPRB, 0.86625E+00_JPRB, 0.87304E+00_JPRB, 0.87192E+00_JPRB ,&
&  0.87321E+00_JPRB, 0.89119E+00_JPRB, 0.99213E+00_JPRB, 0.98519E+00_JPRB, 0.97886E+00_JPRB, 0.95493E+00_JPRB, 0.10224E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.17480E+01_JPRB, 0.21242E+01_JPRB, 0.24176E+01_JPRB, 0.27202E+01_JPRB, 0.27971E+01_JPRB, 0.27061E+01_JPRB ,&
&  0.24705E+01_JPRB, 0.22422E+01_JPRB, 0.19787E+01_JPRB, 0.16877E+01_JPRB, 0.14772E+01_JPRB, 0.13533E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.11849E+01_JPRB, 0.11486E+01_JPRB, 0.10997E+01_JPRB, 0.10624E+01_JPRB, 0.10696E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.11043E+01_JPRB, 0.11389E+01_JPRB, 0.11462E+01_JPRB, 0.10490E+01_JPRB, 0.93238E+00_JPRB, 0.10534E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.15092E+01_JPRB, 0.16414E+01_JPRB, 0.21360E+01_JPRB, 0.22485E+01_JPRB, 0.21726E+01_JPRB, 0.22201E+01_JPRB ,&
&  0.21992E+01_JPRB, 0.24168E+01_JPRB, 0.19412E+01_JPRB, 0.19568E+01_JPRB, 0.21930E+01_JPRB, 0.17225E+01_JPRB, 0.16842E+01_JPRB ,&
&  0.19503E+01_JPRB, 0.18344E+01_JPRB, 0.15842E+01_JPRB, 0.12424E+01_JPRB, 0.16118E+01_JPRB, 0.16639E+01_JPRB, 0.91772E+00_JPRB ,&
&  0.77809E+00_JPRB, 0.72782E+00_JPRB, 0.69166E+00_JPRB, 0.67235E+00_JPRB, 0.73561E+00_JPRB, 0.12929E+01_JPRB, 0.26693E+01_JPRB ,&
&  0.33643E+01_JPRB, 0.51911E+01_JPRB, 0.60712E+01_JPRB, 0.61999E+01_JPRB, 0.53038E+01_JPRB, 0.44129E+01_JPRB, 0.36225E+01_JPRB ,&
&  0.33612E+01_JPRB, 0.32642E+01_JPRB, 0.29760E+01_JPRB, 0.27565E+01_JPRB, 0.25288E+01_JPRB, 0.23210E+01_JPRB, 0.21335E+01_JPRB ,&
&  0.19478E+01_JPRB, 0.18123E+01_JPRB, 0.17112E+01_JPRB, 0.16612E+01_JPRB, 0.15840E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.15399E+01_JPRB, 0.14760E+01_JPRB, 0.13846E+01_JPRB, 0.13153E+01_JPRB, 0.12500E+01_JPRB, 0.11817E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.10785E+01_JPRB, 0.99534E+00_JPRB, 0.93410E+00_JPRB, 0.90817E+00_JPRB, 0.89303E+00_JPRB, 0.87861E+00_JPRB, 0.87190E+00_JPRB ,&
&  0.86429E+00_JPRB, 0.87231E+00_JPRB, 0.95070E+00_JPRB, 0.10701E+01_JPRB, 0.10761E+01_JPRB, 0.10423E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.15988E+01_JPRB, 0.20022E+01_JPRB, 0.22259E+01_JPRB, 0.24236E+01_JPRB, 0.24348E+01_JPRB, 0.24238E+01_JPRB, 0.23085E+01_JPRB ,&
&  0.21207E+01_JPRB, 0.19308E+01_JPRB, 0.17446E+01_JPRB, 0.15424E+01_JPRB, 0.13842E+01_JPRB, 0.12985E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.11899E+01_JPRB, 0.11435E+01_JPRB, 0.10729E+01_JPRB, 0.10355E+01_JPRB, 0.10520E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.11451E+01_JPRB, 0.11807E+01_JPRB, 0.11679E+01_JPRB, 0.98717E+00_JPRB, 0.11170E+01_JPRB, 0.11826E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.13911E+01_JPRB, 0.14219E+01_JPRB, 0.17863E+01_JPRB, 0.20609E+01_JPRB, 0.20431E+01_JPRB, 0.19995E+01_JPRB ,&
&  0.21325E+01_JPRB, 0.19795E+01_JPRB, 0.17202E+01_JPRB, 0.17797E+01_JPRB, 0.19044E+01_JPRB, 0.20937E+01_JPRB, 0.14369E+01_JPRB ,&
&  0.13316E+01_JPRB, 0.15887E+01_JPRB, 0.14464E+01_JPRB, 0.12037E+01_JPRB, 0.15239E+01_JPRB, 0.20534E+01_JPRB, 0.17004E+01_JPRB ,&
&  0.89082E+00_JPRB, 0.73922E+00_JPRB, 0.79936E+00_JPRB, 0.88590E+00_JPRB, 0.85058E+00_JPRB, 0.15348E+01_JPRB, 0.52661E+01_JPRB ,&
&  0.49612E+01_JPRB, 0.63740E+01_JPRB, 0.63347E+01_JPRB, 0.58279E+01_JPRB, 0.50163E+01_JPRB, 0.40744E+01_JPRB, 0.36281E+01_JPRB ,&
&  0.33360E+01_JPRB, 0.31877E+01_JPRB, 0.30452E+01_JPRB, 0.28895E+01_JPRB, 0.26817E+01_JPRB, 0.24026E+01_JPRB, 0.21576E+01_JPRB ,&
&  0.19698E+01_JPRB, 0.18378E+01_JPRB, 0.17315E+01_JPRB, 0.16577E+01_JPRB, 0.15891E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.15917E+01_JPRB, 0.14874E+01_JPRB, 0.13810E+01_JPRB, 0.13115E+01_JPRB, 0.12369E+01_JPRB, 0.11693E+01_JPRB, 0.11612E+01_JPRB ,&
&  0.11178E+01_JPRB, 0.10380E+01_JPRB, 0.96553E+00_JPRB, 0.93298E+00_JPRB, 0.90899E+00_JPRB, 0.88776E+00_JPRB, 0.88199E+00_JPRB ,&
&  0.87454E+00_JPRB, 0.88714E+00_JPRB, 0.93497E+00_JPRB, 0.10364E+01_JPRB, 0.10695E+01_JPRB, 0.10007E+01_JPRB, 0.11414E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.20510E+01_JPRB, 0.20934E+01_JPRB, 0.21777E+01_JPRB, 0.21879E+01_JPRB, 0.22080E+01_JPRB, 0.20279E+01_JPRB ,&
&  0.17626E+01_JPRB, 0.15845E+01_JPRB, 0.14459E+01_JPRB, 0.13752E+01_JPRB, 0.13137E+01_JPRB, 0.12611E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.11433E+01_JPRB, 0.10866E+01_JPRB, 0.10221E+01_JPRB, 0.10242E+01_JPRB, 0.10578E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.11980E+01_JPRB, 0.12144E+01_JPRB, 0.11442E+01_JPRB, 0.11338E+01_JPRB, 0.11516E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.12471E+01_JPRB, 0.13435E+01_JPRB, 0.14802E+01_JPRB, 0.16402E+01_JPRB, 0.18365E+01_JPRB, 0.19046E+01_JPRB, 0.18477E+01_JPRB ,&
&  0.18736E+01_JPRB, 0.17404E+01_JPRB, 0.15042E+01_JPRB, 0.15674E+01_JPRB, 0.17017E+01_JPRB, 0.19227E+01_JPRB, 0.18074E+01_JPRB ,&
&  0.15476E+01_JPRB, 0.13130E+01_JPRB, 0.12472E+01_JPRB, 0.13023E+01_JPRB, 0.15021E+01_JPRB, 0.19343E+01_JPRB, 0.27969E+01_JPRB ,&
&  0.28435E+01_JPRB, 0.19499E+01_JPRB, 0.16538E+01_JPRB, 0.21937E+01_JPRB, 0.20188E+01_JPRB, 0.21952E+01_JPRB, 0.36907E+01_JPRB ,&
&  0.51153E+01_JPRB, 0.62391E+01_JPRB, 0.60290E+01_JPRB, 0.53064E+01_JPRB, 0.46931E+01_JPRB, 0.38833E+01_JPRB, 0.35780E+01_JPRB ,&
&  0.33619E+01_JPRB, 0.31640E+01_JPRB, 0.31023E+01_JPRB, 0.29611E+01_JPRB, 0.26993E+01_JPRB, 0.23751E+01_JPRB, 0.20886E+01_JPRB ,&
&  0.19151E+01_JPRB, 0.18124E+01_JPRB, 0.17409E+01_JPRB, 0.16932E+01_JPRB, 0.16493E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.15684E+01_JPRB, 0.14365E+01_JPRB, 0.13428E+01_JPRB, 0.12678E+01_JPRB, 0.11904E+01_JPRB, 0.11502E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.10614E+01_JPRB, 0.99361E+00_JPRB, 0.93957E+00_JPRB, 0.90320E+00_JPRB, 0.89399E+00_JPRB, 0.89965E+00_JPRB ,&
&  0.89423E+00_JPRB, 0.90672E+00_JPRB, 0.92487E+00_JPRB, 0.10165E+01_JPRB, 0.11575E+01_JPRB, 0.14159E+01_JPRB, 0.11215E+01_JPRB ,&
&  0.15738E+01_JPRB, 0.21239E+01_JPRB, 0.21156E+01_JPRB, 0.20317E+01_JPRB, 0.19185E+01_JPRB, 0.18761E+01_JPRB, 0.17006E+01_JPRB ,&
&  0.14682E+01_JPRB, 0.13452E+01_JPRB, 0.12638E+01_JPRB, 0.12304E+01_JPRB, 0.12234E+01_JPRB, 0.11911E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.10691E+01_JPRB, 0.10298E+01_JPRB, 0.10137E+01_JPRB, 0.10292E+01_JPRB, 0.10665E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.12392E+01_JPRB, 0.12386E+01_JPRB, 0.11621E+01_JPRB, 0.11780E+01_JPRB, 0.11923E+01_JPRB, 0.12198E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.11473E+01_JPRB, 0.13566E+01_JPRB, 0.15857E+01_JPRB, 0.17054E+01_JPRB, 0.16478E+01_JPRB, 0.15545E+01_JPRB ,&
&  0.16631E+01_JPRB, 0.16968E+01_JPRB, 0.16793E+01_JPRB, 0.15554E+01_JPRB, 0.16870E+01_JPRB, 0.18365E+01_JPRB, 0.19333E+01_JPRB ,&
&  0.18724E+01_JPRB, 0.15939E+01_JPRB, 0.14678E+01_JPRB, 0.14635E+01_JPRB, 0.15045E+01_JPRB, 0.18807E+01_JPRB, 0.25890E+01_JPRB ,&
&  0.33633E+01_JPRB, 0.45017E+01_JPRB, 0.45363E+01_JPRB, 0.35933E+01_JPRB, 0.32407E+01_JPRB, 0.33568E+01_JPRB, 0.36438E+01_JPRB ,&
&  0.53411E+01_JPRB, 0.58172E+01_JPRB, 0.54072E+01_JPRB, 0.56555E+01_JPRB, 0.45029E+01_JPRB, 0.37567E+01_JPRB, 0.33946E+01_JPRB ,&
&  0.32828E+01_JPRB, 0.31232E+01_JPRB, 0.29582E+01_JPRB, 0.27619E+01_JPRB, 0.24636E+01_JPRB, 0.21791E+01_JPRB, 0.19822E+01_JPRB ,&
&  0.18592E+01_JPRB, 0.17868E+01_JPRB, 0.17322E+01_JPRB, 0.16962E+01_JPRB, 0.16574E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.14655E+01_JPRB, 0.13466E+01_JPRB, 0.12317E+01_JPRB, 0.11757E+01_JPRB, 0.11322E+01_JPRB, 0.11291E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.10774E+01_JPRB, 0.98227E+00_JPRB, 0.91473E+00_JPRB, 0.89300E+00_JPRB, 0.90119E+00_JPRB, 0.90781E+00_JPRB ,&
&  0.91130E+00_JPRB, 0.92852E+00_JPRB, 0.97701E+00_JPRB, 0.10243E+01_JPRB, 0.11934E+01_JPRB, 0.14378E+01_JPRB, 0.14643E+01_JPRB ,&
&  0.17015E+01_JPRB, 0.25745E+01_JPRB, 0.21317E+01_JPRB, 0.18927E+01_JPRB, 0.17553E+01_JPRB, 0.18454E+01_JPRB, 0.15999E+01_JPRB ,&
&  0.13555E+01_JPRB, 0.12888E+01_JPRB, 0.12700E+01_JPRB, 0.11988E+01_JPRB, 0.11699E+01_JPRB, 0.11200E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10480E+01_JPRB, 0.10205E+01_JPRB, 0.10256E+01_JPRB, 0.10565E+01_JPRB, 0.11041E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.12601E+01_JPRB, 0.12030E+01_JPRB, 0.11909E+01_JPRB, 0.11987E+01_JPRB, 0.12256E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.11794E+01_JPRB, 0.13043E+01_JPRB, 0.13625E+01_JPRB, 0.13801E+01_JPRB, 0.14592E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.15630E+01_JPRB, 0.15151E+01_JPRB, 0.17397E+01_JPRB, 0.15360E+01_JPRB, 0.16960E+01_JPRB, 0.18312E+01_JPRB, 0.17539E+01_JPRB ,&
&  0.15457E+01_JPRB, 0.14997E+01_JPRB, 0.14089E+01_JPRB, 0.14688E+01_JPRB, 0.17178E+01_JPRB, 0.22300E+01_JPRB, 0.27999E+01_JPRB ,&
&  0.34140E+01_JPRB, 0.40591E+01_JPRB, 0.45550E+01_JPRB, 0.36542E+01_JPRB, 0.33821E+01_JPRB, 0.34731E+01_JPRB, 0.40281E+01_JPRB ,&
&  0.53358E+01_JPRB, 0.55838E+01_JPRB, 0.56060E+01_JPRB, 0.49856E+01_JPRB, 0.39660E+01_JPRB, 0.33721E+01_JPRB, 0.30912E+01_JPRB ,&
&  0.29281E+01_JPRB, 0.27822E+01_JPRB, 0.25049E+01_JPRB, 0.22813E+01_JPRB, 0.21222E+01_JPRB, 0.19836E+01_JPRB, 0.18360E+01_JPRB ,&
&  0.17441E+01_JPRB, 0.16698E+01_JPRB, 0.16347E+01_JPRB, 0.16160E+01_JPRB, 0.15622E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.12984E+01_JPRB, 0.12147E+01_JPRB, 0.11636E+01_JPRB, 0.11493E+01_JPRB, 0.11269E+01_JPRB, 0.11562E+01_JPRB, 0.12098E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.10387E+01_JPRB, 0.94047E+00_JPRB, 0.90788E+00_JPRB, 0.90201E+00_JPRB, 0.90004E+00_JPRB, 0.90750E+00_JPRB ,&
&  0.92049E+00_JPRB, 0.96571E+00_JPRB, 0.10347E+01_JPRB, 0.11220E+01_JPRB, 0.13501E+01_JPRB, 0.16719E+01_JPRB, 0.19113E+01_JPRB ,&
&  0.20944E+01_JPRB, 0.21330E+01_JPRB, 0.20862E+01_JPRB, 0.17941E+01_JPRB, 0.16429E+01_JPRB, 0.16958E+01_JPRB, 0.16256E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.13126E+01_JPRB, 0.12466E+01_JPRB, 0.11662E+01_JPRB, 0.11198E+01_JPRB, 0.10944E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10125E+01_JPRB, 0.10067E+01_JPRB, 0.10382E+01_JPRB, 0.10868E+01_JPRB, 0.11579E+01_JPRB, 0.12686E+01_JPRB ,&
&  0.13137E+01_JPRB, 0.12700E+01_JPRB, 0.12325E+01_JPRB, 0.11824E+01_JPRB, 0.12049E+01_JPRB, 0.12260E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12595E+01_JPRB, 0.12911E+01_JPRB, 0.12804E+01_JPRB, 0.12225E+01_JPRB, 0.13564E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.14662E+01_JPRB, 0.15247E+01_JPRB, 0.18321E+01_JPRB, 0.15072E+01_JPRB, 0.17067E+01_JPRB, 0.17612E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.14885E+01_JPRB, 0.14481E+01_JPRB, 0.14779E+01_JPRB, 0.16138E+01_JPRB, 0.20715E+01_JPRB, 0.26197E+01_JPRB, 0.36974E+01_JPRB ,&
&  0.37549E+01_JPRB, 0.42450E+01_JPRB, 0.39948E+01_JPRB, 0.39360E+01_JPRB, 0.36218E+01_JPRB, 0.36944E+01_JPRB, 0.40623E+01_JPRB ,&
&  0.46897E+01_JPRB, 0.47234E+01_JPRB, 0.44412E+01_JPRB, 0.39457E+01_JPRB, 0.33033E+01_JPRB, 0.28373E+01_JPRB, 0.25850E+01_JPRB ,&
&  0.23544E+01_JPRB, 0.21543E+01_JPRB, 0.19943E+01_JPRB, 0.19273E+01_JPRB, 0.18464E+01_JPRB, 0.17490E+01_JPRB, 0.16618E+01_JPRB ,&
&  0.16084E+01_JPRB, 0.15610E+01_JPRB, 0.15203E+01_JPRB, 0.14693E+01_JPRB, 0.14001E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.11639E+01_JPRB, 0.11210E+01_JPRB, 0.10996E+01_JPRB, 0.10681E+01_JPRB, 0.10391E+01_JPRB, 0.10369E+01_JPRB, 0.10324E+01_JPRB ,&
&  0.98106E+00_JPRB, 0.93254E+00_JPRB, 0.91841E+00_JPRB, 0.91815E+00_JPRB, 0.90780E+00_JPRB, 0.89484E+00_JPRB, 0.90045E+00_JPRB ,&
&  0.93572E+00_JPRB, 0.10029E+01_JPRB, 0.10946E+01_JPRB, 0.12703E+01_JPRB, 0.15103E+01_JPRB, 0.18128E+01_JPRB, 0.21046E+01_JPRB ,&
&  0.21063E+01_JPRB, 0.15444E+01_JPRB, 0.17894E+01_JPRB, 0.17666E+01_JPRB, 0.15595E+01_JPRB, 0.15512E+01_JPRB, 0.15174E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.12581E+01_JPRB, 0.12408E+01_JPRB, 0.11967E+01_JPRB, 0.10936E+01_JPRB, 0.10688E+01_JPRB, 0.10351E+01_JPRB ,&
&  0.10151E+01_JPRB, 0.10047E+01_JPRB, 0.10201E+01_JPRB, 0.10843E+01_JPRB, 0.11659E+01_JPRB, 0.12698E+01_JPRB, 0.13460E+01_JPRB ,&
&  0.13399E+01_JPRB, 0.13100E+01_JPRB, 0.13008E+01_JPRB, 0.12495E+01_JPRB, 0.12559E+01_JPRB, 0.12724E+01_JPRB, 0.12689E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.12452E+01_JPRB, 0.12829E+01_JPRB, 0.13430E+01_JPRB, 0.13854E+01_JPRB, 0.12941E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.14698E+01_JPRB, 0.16390E+01_JPRB, 0.17787E+01_JPRB, 0.19986E+01_JPRB, 0.14850E+01_JPRB, 0.16023E+01_JPRB, 0.14289E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.15994E+01_JPRB, 0.18156E+01_JPRB, 0.21148E+01_JPRB, 0.27532E+01_JPRB, 0.33275E+01_JPRB, 0.43284E+01_JPRB ,&
&  0.39146E+01_JPRB, 0.37788E+01_JPRB, 0.36877E+01_JPRB, 0.40242E+01_JPRB, 0.39157E+01_JPRB, 0.41591E+01_JPRB, 0.40267E+01_JPRB ,&
&  0.41575E+01_JPRB, 0.37952E+01_JPRB, 0.34255E+01_JPRB, 0.31429E+01_JPRB, 0.26577E+01_JPRB, 0.21520E+01_JPRB, 0.18791E+01_JPRB ,&
&  0.17208E+01_JPRB, 0.16743E+01_JPRB, 0.16421E+01_JPRB, 0.16148E+01_JPRB, 0.15684E+01_JPRB, 0.15226E+01_JPRB, 0.14830E+01_JPRB ,&
&  0.14452E+01_JPRB, 0.13960E+01_JPRB, 0.13474E+01_JPRB, 0.12972E+01_JPRB, 0.12357E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.10755E+01_JPRB, 0.10428E+01_JPRB, 0.10239E+01_JPRB, 0.99790E+00_JPRB, 0.98084E+00_JPRB, 0.97475E+00_JPRB, 0.96714E+00_JPRB ,&
&  0.94419E+00_JPRB, 0.93173E+00_JPRB, 0.92839E+00_JPRB, 0.91820E+00_JPRB, 0.89488E+00_JPRB, 0.88833E+00_JPRB, 0.91693E+00_JPRB ,&
&  0.98651E+00_JPRB, 0.11001E+01_JPRB, 0.12402E+01_JPRB, 0.13996E+01_JPRB, 0.16323E+01_JPRB, 0.18099E+01_JPRB, 0.18785E+01_JPRB ,&
&  0.20108E+01_JPRB, 0.20576E+01_JPRB, 0.18189E+01_JPRB, 0.13262E+01_JPRB, 0.11994E+01_JPRB, 0.14040E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.11974E+01_JPRB, 0.11626E+01_JPRB, 0.11410E+01_JPRB, 0.11155E+01_JPRB, 0.10741E+01_JPRB, 0.10470E+01_JPRB, 0.10254E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.10378E+01_JPRB, 0.10932E+01_JPRB, 0.12039E+01_JPRB, 0.13127E+01_JPRB, 0.13814E+01_JPRB, 0.14002E+01_JPRB ,&
&  0.14064E+01_JPRB, 0.14135E+01_JPRB, 0.14105E+01_JPRB, 0.13762E+01_JPRB, 0.13408E+01_JPRB, 0.13540E+01_JPRB, 0.13579E+01_JPRB ,&
&  0.13571E+01_JPRB, 0.13493E+01_JPRB, 0.13432E+01_JPRB, 0.13800E+01_JPRB, 0.13941E+01_JPRB, 0.12862E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.15320E+01_JPRB, 0.18156E+01_JPRB, 0.18135E+01_JPRB, 0.21417E+01_JPRB, 0.13470E+01_JPRB, 0.14987E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.16883E+01_JPRB, 0.19772E+01_JPRB, 0.22733E+01_JPRB, 0.26761E+01_JPRB, 0.32833E+01_JPRB, 0.36368E+01_JPRB, 0.40709E+01_JPRB ,&
&  0.35474E+01_JPRB, 0.32770E+01_JPRB, 0.33574E+01_JPRB, 0.36615E+01_JPRB, 0.38419E+01_JPRB, 0.38796E+01_JPRB, 0.34178E+01_JPRB ,&
&  0.29255E+01_JPRB, 0.30390E+01_JPRB, 0.26828E+01_JPRB, 0.25443E+01_JPRB, 0.20055E+01_JPRB, 0.15908E+01_JPRB, 0.15241E+01_JPRB ,&
&  0.14557E+01_JPRB, 0.14060E+01_JPRB, 0.13962E+01_JPRB, 0.13772E+01_JPRB, 0.13549E+01_JPRB, 0.13358E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.12398E+01_JPRB, 0.12042E+01_JPRB, 0.11672E+01_JPRB, 0.11239E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.10098E+01_JPRB, 0.98756E+00_JPRB, 0.97954E+00_JPRB, 0.94699E+00_JPRB, 0.92129E+00_JPRB, 0.91844E+00_JPRB, 0.92649E+00_JPRB ,&
&  0.91966E+00_JPRB, 0.91165E+00_JPRB, 0.89713E+00_JPRB, 0.88558E+00_JPRB, 0.89104E+00_JPRB, 0.92644E+00_JPRB, 0.99218E+00_JPRB ,&
&  0.10793E+01_JPRB, 0.12056E+01_JPRB, 0.13661E+01_JPRB, 0.15176E+01_JPRB, 0.17119E+01_JPRB, 0.17990E+01_JPRB, 0.17789E+01_JPRB ,&
&  0.18384E+01_JPRB, 0.19111E+01_JPRB, 0.18815E+01_JPRB, 0.18316E+01_JPRB, 0.14995E+01_JPRB, 0.15675E+01_JPRB, 0.15960E+01_JPRB ,&
&  0.15237E+01_JPRB, 0.12827E+01_JPRB, 0.11406E+01_JPRB, 0.11032E+01_JPRB, 0.10622E+01_JPRB, 0.10480E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.11549E+01_JPRB, 0.12456E+01_JPRB, 0.13600E+01_JPRB, 0.14388E+01_JPRB, 0.14695E+01_JPRB, 0.14871E+01_JPRB ,&
&  0.15102E+01_JPRB, 0.15251E+01_JPRB, 0.15385E+01_JPRB, 0.15094E+01_JPRB, 0.15022E+01_JPRB, 0.15083E+01_JPRB, 0.15361E+01_JPRB ,&
&  0.15634E+01_JPRB, 0.14911E+01_JPRB, 0.15011E+01_JPRB, 0.15575E+01_JPRB, 0.15264E+01_JPRB, 0.14796E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.17273E+01_JPRB, 0.20676E+01_JPRB, 0.16929E+01_JPRB, 0.18404E+01_JPRB, 0.21452E+01_JPRB, 0.18470E+01_JPRB, 0.17555E+01_JPRB ,&
&  0.19679E+01_JPRB, 0.22188E+01_JPRB, 0.24938E+01_JPRB, 0.28687E+01_JPRB, 0.34268E+01_JPRB, 0.41185E+01_JPRB, 0.34931E+01_JPRB ,&
&  0.30799E+01_JPRB, 0.28719E+01_JPRB, 0.30068E+01_JPRB, 0.31885E+01_JPRB, 0.32720E+01_JPRB, 0.32353E+01_JPRB, 0.28434E+01_JPRB ,&
&  0.24819E+01_JPRB, 0.25452E+01_JPRB, 0.23290E+01_JPRB, 0.21215E+01_JPRB, 0.19986E+01_JPRB, 0.14419E+01_JPRB, 0.13620E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.12516E+01_JPRB, 0.12498E+01_JPRB, 0.12392E+01_JPRB, 0.12236E+01_JPRB, 0.11980E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11201E+01_JPRB, 0.10993E+01_JPRB, 0.10800E+01_JPRB, 0.10432E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.96594E+00_JPRB, 0.95345E+00_JPRB, 0.94205E+00_JPRB, 0.91713E+00_JPRB, 0.89151E+00_JPRB, 0.88877E+00_JPRB, 0.88406E+00_JPRB ,&
&  0.86399E+00_JPRB, 0.84299E+00_JPRB, 0.81822E+00_JPRB, 0.82936E+00_JPRB, 0.85712E+00_JPRB, 0.91515E+00_JPRB, 0.98369E+00_JPRB ,&
&  0.10687E+01_JPRB, 0.11914E+01_JPRB, 0.13594E+01_JPRB, 0.14946E+01_JPRB, 0.16836E+01_JPRB, 0.17761E+01_JPRB, 0.18216E+01_JPRB ,&
&  0.18911E+01_JPRB, 0.19689E+01_JPRB, 0.20168E+01_JPRB, 0.20307E+01_JPRB, 0.19908E+01_JPRB, 0.19822E+01_JPRB, 0.20347E+01_JPRB ,&
&  0.16952E+01_JPRB, 0.12875E+01_JPRB, 0.11839E+01_JPRB, 0.11276E+01_JPRB, 0.11113E+01_JPRB, 0.11138E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.12904E+01_JPRB, 0.13843E+01_JPRB, 0.14596E+01_JPRB, 0.15284E+01_JPRB, 0.15758E+01_JPRB, 0.16021E+01_JPRB ,&
&  0.16489E+01_JPRB, 0.16946E+01_JPRB, 0.17654E+01_JPRB, 0.16741E+01_JPRB, 0.17806E+01_JPRB, 0.18017E+01_JPRB, 0.18236E+01_JPRB ,&
&  0.17883E+01_JPRB, 0.16831E+01_JPRB, 0.17394E+01_JPRB, 0.17812E+01_JPRB, 0.18245E+01_JPRB, 0.18287E+01_JPRB, 0.18365E+01_JPRB ,&
&  0.20309E+01_JPRB, 0.20732E+01_JPRB, 0.16233E+01_JPRB, 0.15406E+01_JPRB, 0.16383E+01_JPRB, 0.15001E+01_JPRB, 0.18666E+01_JPRB ,&
&  0.20134E+01_JPRB, 0.22089E+01_JPRB, 0.24178E+01_JPRB, 0.27495E+01_JPRB, 0.31317E+01_JPRB, 0.34352E+01_JPRB, 0.30537E+01_JPRB ,&
&  0.25083E+01_JPRB, 0.23745E+01_JPRB, 0.24316E+01_JPRB, 0.24754E+01_JPRB, 0.25214E+01_JPRB, 0.25420E+01_JPRB, 0.24240E+01_JPRB ,&
&  0.22353E+01_JPRB, 0.20245E+01_JPRB, 0.18239E+01_JPRB, 0.14933E+01_JPRB, 0.13165E+01_JPRB, 0.12522E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.11391E+01_JPRB, 0.11593E+01_JPRB, 0.11627E+01_JPRB, 0.11522E+01_JPRB, 0.11394E+01_JPRB, 0.11085E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10536E+01_JPRB, 0.10344E+01_JPRB, 0.10202E+01_JPRB, 0.98973E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.90746E+00_JPRB, 0.88549E+00_JPRB, 0.87061E+00_JPRB, 0.85751E+00_JPRB, 0.83321E+00_JPRB, 0.81598E+00_JPRB, 0.80754E+00_JPRB ,&
&  0.77098E+00_JPRB, 0.76770E+00_JPRB, 0.75103E+00_JPRB, 0.75956E+00_JPRB, 0.79180E+00_JPRB, 0.82377E+00_JPRB, 0.87277E+00_JPRB ,&
&  0.91537E+00_JPRB, 0.97289E+00_JPRB, 0.10679E+01_JPRB, 0.12753E+01_JPRB, 0.15103E+01_JPRB, 0.16838E+01_JPRB, 0.18840E+01_JPRB ,&
&  0.19630E+01_JPRB, 0.20318E+01_JPRB, 0.21677E+01_JPRB, 0.23544E+01_JPRB, 0.24576E+01_JPRB, 0.25241E+01_JPRB, 0.24532E+01_JPRB ,&
&  0.18147E+01_JPRB, 0.12230E+01_JPRB, 0.12184E+01_JPRB, 0.11358E+01_JPRB, 0.11550E+01_JPRB, 0.11871E+01_JPRB, 0.12254E+01_JPRB ,&
&  0.12926E+01_JPRB, 0.13568E+01_JPRB, 0.14239E+01_JPRB, 0.14821E+01_JPRB, 0.15642E+01_JPRB, 0.16553E+01_JPRB, 0.16912E+01_JPRB ,&
&  0.17172E+01_JPRB, 0.17510E+01_JPRB, 0.17841E+01_JPRB, 0.17424E+01_JPRB, 0.18039E+01_JPRB, 0.19203E+01_JPRB, 0.20121E+01_JPRB ,&
&  0.20982E+01_JPRB, 0.21021E+01_JPRB, 0.18232E+01_JPRB, 0.18462E+01_JPRB, 0.20517E+01_JPRB, 0.19482E+01_JPRB, 0.20260E+01_JPRB ,&
&  0.23146E+01_JPRB, 0.23573E+01_JPRB, 0.15471E+01_JPRB, 0.14447E+01_JPRB, 0.15471E+01_JPRB, 0.16520E+01_JPRB, 0.18102E+01_JPRB ,&
&  0.19083E+01_JPRB, 0.20807E+01_JPRB, 0.22773E+01_JPRB, 0.24427E+01_JPRB, 0.25456E+01_JPRB, 0.24581E+01_JPRB, 0.23466E+01_JPRB ,&
&  0.20090E+01_JPRB, 0.19101E+01_JPRB, 0.19482E+01_JPRB, 0.19996E+01_JPRB, 0.20662E+01_JPRB, 0.21548E+01_JPRB, 0.19793E+01_JPRB ,&
&  0.18676E+01_JPRB, 0.17692E+01_JPRB, 0.18573E+01_JPRB, 0.15548E+01_JPRB, 0.11795E+01_JPRB, 0.11066E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.11096E+01_JPRB, 0.11177E+01_JPRB, 0.11185E+01_JPRB, 0.11102E+01_JPRB, 0.10920E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10468E+01_JPRB, 0.10189E+01_JPRB, 0.98935E+00_JPRB, 0.96028E+00_JPRB, 0.93440E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.79953E+00_JPRB, 0.77767E+00_JPRB, 0.77082E+00_JPRB, 0.73338E+00_JPRB, 0.74881E+00_JPRB, 0.72292E+00_JPRB, 0.75626E+00_JPRB ,&
&  0.72980E+00_JPRB, 0.72544E+00_JPRB, 0.72514E+00_JPRB, 0.73927E+00_JPRB, 0.75370E+00_JPRB, 0.77278E+00_JPRB, 0.78832E+00_JPRB ,&
&  0.80976E+00_JPRB, 0.85501E+00_JPRB, 0.92157E+00_JPRB, 0.10011E+01_JPRB, 0.11056E+01_JPRB, 0.12088E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.14179E+01_JPRB, 0.15200E+01_JPRB, 0.15779E+01_JPRB, 0.17352E+01_JPRB, 0.21727E+01_JPRB, 0.27096E+01_JPRB, 0.27603E+01_JPRB ,&
&  0.17615E+01_JPRB, 0.12525E+01_JPRB, 0.11276E+01_JPRB, 0.10451E+01_JPRB, 0.10859E+01_JPRB, 0.11898E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.12840E+01_JPRB, 0.13383E+01_JPRB, 0.13980E+01_JPRB, 0.14787E+01_JPRB, 0.15734E+01_JPRB, 0.16084E+01_JPRB ,&
&  0.15943E+01_JPRB, 0.16201E+01_JPRB, 0.16239E+01_JPRB, 0.17248E+01_JPRB, 0.19059E+01_JPRB, 0.21301E+01_JPRB, 0.22694E+01_JPRB ,&
&  0.23497E+01_JPRB, 0.23128E+01_JPRB, 0.20056E+01_JPRB, 0.19998E+01_JPRB, 0.20862E+01_JPRB, 0.20884E+01_JPRB, 0.20591E+01_JPRB ,&
&  0.22440E+01_JPRB, 0.21875E+01_JPRB, 0.16735E+01_JPRB, 0.16297E+01_JPRB, 0.16431E+01_JPRB, 0.16926E+01_JPRB, 0.16549E+01_JPRB ,&
&  0.17836E+01_JPRB, 0.19150E+01_JPRB, 0.20140E+01_JPRB, 0.20353E+01_JPRB, 0.19867E+01_JPRB, 0.18912E+01_JPRB, 0.18991E+01_JPRB ,&
&  0.17798E+01_JPRB, 0.17711E+01_JPRB, 0.17853E+01_JPRB, 0.17997E+01_JPRB, 0.20322E+01_JPRB, 0.18222E+01_JPRB, 0.18097E+01_JPRB ,&
&  0.15561E+01_JPRB, 0.14799E+01_JPRB, 0.12349E+01_JPRB, 0.14659E+01_JPRB, 0.11034E+01_JPRB, 0.94473E+00_JPRB, 0.95551E+00_JPRB ,&
&  0.97712E+00_JPRB, 0.10110E+01_JPRB, 0.10071E+01_JPRB, 0.10139E+01_JPRB, 0.10165E+01_JPRB, 0.10009E+01_JPRB, 0.99161E+00_JPRB ,&
&  0.97338E+00_JPRB, 0.94885E+00_JPRB, 0.90851E+00_JPRB, 0.87389E+00_JPRB, 0.83862E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.76808E+00_JPRB, 0.76552E+00_JPRB, 0.74167E+00_JPRB, 0.70694E+00_JPRB, 0.68378E+00_JPRB, 0.66755E+00_JPRB, 0.67493E+00_JPRB ,&
&  0.68347E+00_JPRB, 0.70497E+00_JPRB, 0.72366E+00_JPRB, 0.74346E+00_JPRB, 0.75219E+00_JPRB, 0.75776E+00_JPRB, 0.77325E+00_JPRB ,&
&  0.79416E+00_JPRB, 0.81443E+00_JPRB, 0.84225E+00_JPRB, 0.88151E+00_JPRB, 0.91997E+00_JPRB, 0.96833E+00_JPRB, 0.98900E+00_JPRB ,&
&  0.98818E+00_JPRB, 0.10162E+01_JPRB, 0.10384E+01_JPRB, 0.10877E+01_JPRB, 0.13111E+01_JPRB, 0.15486E+01_JPRB, 0.18327E+01_JPRB ,&
&  0.13938E+01_JPRB, 0.13314E+01_JPRB, 0.11661E+01_JPRB, 0.10682E+01_JPRB, 0.10856E+01_JPRB, 0.10653E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10901E+01_JPRB, 0.10904E+01_JPRB, 0.10959E+01_JPRB, 0.11094E+01_JPRB, 0.11294E+01_JPRB, 0.11826E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.11204E+01_JPRB, 0.11618E+01_JPRB, 0.12170E+01_JPRB, 0.13801E+01_JPRB, 0.16109E+01_JPRB, 0.18160E+01_JPRB, 0.18311E+01_JPRB ,&
&  0.18422E+01_JPRB, 0.19431E+01_JPRB, 0.18985E+01_JPRB, 0.18899E+01_JPRB, 0.18628E+01_JPRB, 0.18353E+01_JPRB, 0.18764E+01_JPRB ,&
&  0.18040E+01_JPRB, 0.17788E+01_JPRB, 0.18258E+01_JPRB, 0.16873E+01_JPRB, 0.16029E+01_JPRB, 0.15348E+01_JPRB, 0.15279E+01_JPRB ,&
&  0.15663E+01_JPRB, 0.16424E+01_JPRB, 0.16905E+01_JPRB, 0.16436E+01_JPRB, 0.15965E+01_JPRB, 0.15351E+01_JPRB, 0.15720E+01_JPRB ,&
&  0.15910E+01_JPRB, 0.15145E+01_JPRB, 0.14915E+01_JPRB, 0.15701E+01_JPRB, 0.16554E+01_JPRB, 0.26093E+01_JPRB, 0.23367E+01_JPRB ,&
&  0.13753E+01_JPRB, 0.11058E+01_JPRB, 0.80214E+00_JPRB, 0.11655E+01_JPRB, 0.87601E+00_JPRB, 0.83127E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.87286E+00_JPRB, 0.88890E+00_JPRB, 0.89835E+00_JPRB, 0.89345E+00_JPRB, 0.88999E+00_JPRB, 0.88856E+00_JPRB, 0.87151E+00_JPRB ,&
&  0.85757E+00_JPRB, 0.83808E+00_JPRB, 0.83174E+00_JPRB, 0.79568E+00_JPRB, 0.77707E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.73054E+00_JPRB, 0.72602E+00_JPRB, 0.69403E+00_JPRB, 0.65214E+00_JPRB, 0.63238E+00_JPRB, 0.63664E+00_JPRB, 0.65178E+00_JPRB ,&
&  0.66659E+00_JPRB, 0.68685E+00_JPRB, 0.71103E+00_JPRB, 0.73485E+00_JPRB, 0.75625E+00_JPRB, 0.77904E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.83146E+00_JPRB, 0.86231E+00_JPRB, 0.89430E+00_JPRB, 0.92782E+00_JPRB, 0.95754E+00_JPRB, 0.10091E+01_JPRB, 0.10451E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.11551E+01_JPRB, 0.12235E+01_JPRB, 0.12914E+01_JPRB, 0.13788E+01_JPRB, 0.14492E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.14562E+01_JPRB, 0.13237E+01_JPRB, 0.11743E+01_JPRB, 0.10730E+01_JPRB, 0.10100E+01_JPRB, 0.96177E+00_JPRB, 0.91628E+00_JPRB ,&
&  0.89280E+00_JPRB, 0.85650E+00_JPRB, 0.81833E+00_JPRB, 0.78973E+00_JPRB, 0.77807E+00_JPRB, 0.82547E+00_JPRB, 0.81214E+00_JPRB ,&
&  0.86916E+00_JPRB, 0.97627E+00_JPRB, 0.10638E+01_JPRB, 0.12047E+01_JPRB, 0.13275E+01_JPRB, 0.14048E+01_JPRB, 0.13588E+01_JPRB ,&
&  0.13667E+01_JPRB, 0.14599E+01_JPRB, 0.14260E+01_JPRB, 0.14387E+01_JPRB, 0.14288E+01_JPRB, 0.13911E+01_JPRB, 0.13796E+01_JPRB ,&
&  0.12473E+01_JPRB, 0.13413E+01_JPRB, 0.15596E+01_JPRB, 0.17586E+01_JPRB, 0.15659E+01_JPRB, 0.14428E+01_JPRB, 0.14441E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.14150E+01_JPRB, 0.14334E+01_JPRB, 0.13720E+01_JPRB, 0.13211E+01_JPRB, 0.12763E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.12532E+01_JPRB, 0.12794E+01_JPRB, 0.14296E+01_JPRB, 0.16672E+01_JPRB, 0.17840E+01_JPRB, 0.21438E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.75378E+00_JPRB, 0.78600E+00_JPRB, 0.86454E+00_JPRB, 0.74687E+00_JPRB, 0.70441E+00_JPRB, 0.75180E+00_JPRB ,&
&  0.79326E+00_JPRB, 0.81772E+00_JPRB, 0.80398E+00_JPRB, 0.82510E+00_JPRB, 0.86703E+00_JPRB, 0.85803E+00_JPRB, 0.80484E+00_JPRB ,&
&  0.81233E+00_JPRB, 0.77482E+00_JPRB, 0.75626E+00_JPRB, 0.74909E+00_JPRB, 0.74212E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.68725E+00_JPRB, 0.67303E+00_JPRB, 0.63996E+00_JPRB, 0.59981E+00_JPRB, 0.58304E+00_JPRB, 0.59785E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.64268E+00_JPRB, 0.66758E+00_JPRB, 0.69372E+00_JPRB, 0.72274E+00_JPRB, 0.75371E+00_JPRB, 0.78263E+00_JPRB, 0.80491E+00_JPRB ,&
&  0.82079E+00_JPRB, 0.84270E+00_JPRB, 0.86278E+00_JPRB, 0.88080E+00_JPRB, 0.88966E+00_JPRB, 0.89457E+00_JPRB, 0.89934E+00_JPRB ,&
&  0.93846E+00_JPRB, 0.97885E+00_JPRB, 0.10316E+01_JPRB, 0.10868E+01_JPRB, 0.11269E+01_JPRB, 0.11500E+01_JPRB, 0.96095E+00_JPRB ,&
&  0.12913E+01_JPRB, 0.12578E+01_JPRB, 0.11502E+01_JPRB, 0.10495E+01_JPRB, 0.96698E+00_JPRB, 0.90598E+00_JPRB, 0.85384E+00_JPRB ,&
&  0.81547E+00_JPRB, 0.83884E+00_JPRB, 0.80624E+00_JPRB, 0.81097E+00_JPRB, 0.74348E+00_JPRB, 0.78074E+00_JPRB, 0.81929E+00_JPRB ,&
&  0.90747E+00_JPRB, 0.10058E+01_JPRB, 0.10921E+01_JPRB, 0.11641E+01_JPRB, 0.11992E+01_JPRB, 0.12240E+01_JPRB, 0.11740E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11661E+01_JPRB, 0.10772E+01_JPRB, 0.10685E+01_JPRB, 0.10894E+01_JPRB, 0.97777E+00_JPRB, 0.90583E+00_JPRB ,&
&  0.83890E+00_JPRB, 0.10375E+01_JPRB, 0.14881E+01_JPRB, 0.17580E+01_JPRB, 0.15318E+01_JPRB, 0.14613E+01_JPRB, 0.13775E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.12641E+01_JPRB, 0.12511E+01_JPRB, 0.12041E+01_JPRB, 0.11114E+01_JPRB, 0.10541E+01_JPRB, 0.10802E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.10472E+01_JPRB, 0.10283E+01_JPRB, 0.11782E+01_JPRB, 0.12843E+01_JPRB, 0.12092E+01_JPRB, 0.20047E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.89452E+00_JPRB, 0.12133E+01_JPRB, 0.55795E+00_JPRB, 0.62931E+00_JPRB, 0.59310E+00_JPRB, 0.61306E+00_JPRB ,&
&  0.52572E+00_JPRB, 0.61470E+00_JPRB, 0.98397E+00_JPRB, 0.12924E+01_JPRB, 0.14788E+01_JPRB, 0.14736E+01_JPRB, 0.87361E+00_JPRB ,&
&  0.77783E+00_JPRB, 0.75380E+00_JPRB, 0.73639E+00_JPRB, 0.71418E+00_JPRB, 0.69061E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.64024E+00_JPRB, 0.60255E+00_JPRB, 0.58341E+00_JPRB, 0.56991E+00_JPRB, 0.55746E+00_JPRB, 0.57544E+00_JPRB, 0.59078E+00_JPRB ,&
&  0.61429E+00_JPRB, 0.64369E+00_JPRB, 0.67940E+00_JPRB, 0.71074E+00_JPRB, 0.75379E+00_JPRB, 0.79564E+00_JPRB, 0.82959E+00_JPRB ,&
&  0.85779E+00_JPRB, 0.88102E+00_JPRB, 0.90071E+00_JPRB, 0.91137E+00_JPRB, 0.91821E+00_JPRB, 0.92316E+00_JPRB, 0.92582E+00_JPRB ,&
&  0.92713E+00_JPRB, 0.95391E+00_JPRB, 0.10044E+01_JPRB, 0.10617E+01_JPRB, 0.10997E+01_JPRB, 0.11570E+01_JPRB, 0.74773E+00_JPRB ,&
&  0.11128E+01_JPRB, 0.11368E+01_JPRB, 0.11046E+01_JPRB, 0.10251E+01_JPRB, 0.94960E+00_JPRB, 0.89819E+00_JPRB, 0.86393E+00_JPRB ,&
&  0.84895E+00_JPRB, 0.87135E+00_JPRB, 0.84764E+00_JPRB, 0.90309E+00_JPRB, 0.92077E+00_JPRB, 0.88939E+00_JPRB, 0.96510E+00_JPRB ,&
&  0.10249E+01_JPRB, 0.11102E+01_JPRB, 0.11665E+01_JPRB, 0.12100E+01_JPRB, 0.12392E+01_JPRB, 0.12693E+01_JPRB, 0.12769E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.12358E+01_JPRB, 0.11783E+01_JPRB, 0.11528E+01_JPRB, 0.11418E+01_JPRB, 0.99099E+00_JPRB, 0.91698E+00_JPRB ,&
&  0.72919E+00_JPRB, 0.77159E+00_JPRB, 0.12590E+01_JPRB, 0.16730E+01_JPRB, 0.15438E+01_JPRB, 0.14754E+01_JPRB, 0.13284E+01_JPRB ,&
&  0.11944E+01_JPRB, 0.11270E+01_JPRB, 0.10908E+01_JPRB, 0.10426E+01_JPRB, 0.96869E+00_JPRB, 0.91936E+00_JPRB, 0.91554E+00_JPRB ,&
&  0.91283E+00_JPRB, 0.83779E+00_JPRB, 0.81666E+00_JPRB, 0.89071E+00_JPRB, 0.93764E+00_JPRB, 0.10952E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.11906E+01_JPRB, 0.91410E+00_JPRB, 0.83340E+00_JPRB, 0.60696E+00_JPRB, 0.54238E+00_JPRB, 0.52673E+00_JPRB ,&
&  0.54661E+00_JPRB, 0.49475E+00_JPRB, 0.99488E+00_JPRB, 0.37957E+01_JPRB, 0.37349E+01_JPRB, 0.47242E+01_JPRB, 0.24877E+01_JPRB ,&
&  0.12029E+01_JPRB, 0.76797E+00_JPRB, 0.67933E+00_JPRB, 0.67223E+00_JPRB, 0.65443E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.56826E+00_JPRB, 0.54567E+00_JPRB, 0.52148E+00_JPRB, 0.52793E+00_JPRB, 0.53345E+00_JPRB, 0.53957E+00_JPRB, 0.56709E+00_JPRB ,&
&  0.62513E+00_JPRB, 0.64018E+00_JPRB, 0.66823E+00_JPRB, 0.71412E+00_JPRB, 0.78072E+00_JPRB, 0.83477E+00_JPRB, 0.88538E+00_JPRB ,&
&  0.93766E+00_JPRB, 0.97287E+00_JPRB, 0.97819E+00_JPRB, 0.97362E+00_JPRB, 0.98109E+00_JPRB, 0.99463E+00_JPRB, 0.10167E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.10549E+01_JPRB, 0.11084E+01_JPRB, 0.11690E+01_JPRB, 0.12226E+01_JPRB, 0.12212E+01_JPRB, 0.77923E+00_JPRB ,&
&  0.93439E+00_JPRB, 0.98711E+00_JPRB, 0.10122E+01_JPRB, 0.98765E+00_JPRB, 0.92275E+00_JPRB, 0.86307E+00_JPRB, 0.86813E+00_JPRB ,&
&  0.92030E+00_JPRB, 0.97463E+00_JPRB, 0.98460E+00_JPRB, 0.97807E+00_JPRB, 0.10978E+01_JPRB, 0.10685E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.12482E+01_JPRB, 0.12997E+01_JPRB, 0.13594E+01_JPRB, 0.13949E+01_JPRB, 0.14429E+01_JPRB, 0.15020E+01_JPRB, 0.15556E+01_JPRB ,&
&  0.15723E+01_JPRB, 0.15530E+01_JPRB, 0.15725E+01_JPRB, 0.16432E+01_JPRB, 0.14995E+01_JPRB, 0.16077E+01_JPRB, 0.13250E+01_JPRB ,&
&  0.94227E+00_JPRB, 0.84599E+00_JPRB, 0.11357E+01_JPRB, 0.15037E+01_JPRB, 0.14839E+01_JPRB, 0.14513E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.11588E+01_JPRB, 0.10471E+01_JPRB, 0.10021E+01_JPRB, 0.95862E+00_JPRB, 0.90258E+00_JPRB, 0.85036E+00_JPRB, 0.81791E+00_JPRB ,&
&  0.77306E+00_JPRB, 0.73125E+00_JPRB, 0.75875E+00_JPRB, 0.76567E+00_JPRB, 0.80427E+00_JPRB, 0.10143E+01_JPRB, 0.14799E+01_JPRB ,&
&  0.23284E+01_JPRB, 0.12510E+01_JPRB, 0.95204E+00_JPRB, 0.80163E+00_JPRB, 0.65151E+00_JPRB, 0.57574E+00_JPRB, 0.53524E+00_JPRB ,&
&  0.53037E+00_JPRB, 0.62196E+00_JPRB, 0.75538E+00_JPRB, 0.14546E+01_JPRB, 0.25904E+01_JPRB, 0.26428E+01_JPRB, 0.17889E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.76960E+00_JPRB, 0.62334E+00_JPRB, 0.59412E+00_JPRB, 0.57243E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.52415E+00_JPRB, 0.51253E+00_JPRB, 0.51078E+00_JPRB, 0.50700E+00_JPRB, 0.54120E+00_JPRB, 0.56407E+00_JPRB, 0.58591E+00_JPRB ,&
&  0.62055E+00_JPRB, 0.64318E+00_JPRB, 0.67668E+00_JPRB, 0.73921E+00_JPRB, 0.80783E+00_JPRB, 0.86545E+00_JPRB, 0.89633E+00_JPRB ,&
&  0.91703E+00_JPRB, 0.92130E+00_JPRB, 0.91934E+00_JPRB, 0.91403E+00_JPRB, 0.93893E+00_JPRB, 0.95615E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.11242E+01_JPRB, 0.12003E+01_JPRB, 0.12699E+01_JPRB, 0.13442E+01_JPRB, 0.14007E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.65275E+00_JPRB, 0.87583E+00_JPRB, 0.91436E+00_JPRB, 0.90455E+00_JPRB, 0.85219E+00_JPRB, 0.84206E+00_JPRB, 0.92767E+00_JPRB ,&
&  0.10414E+01_JPRB, 0.10769E+01_JPRB, 0.11863E+01_JPRB, 0.10865E+01_JPRB, 0.12278E+01_JPRB, 0.12648E+01_JPRB, 0.13651E+01_JPRB ,&
&  0.14347E+01_JPRB, 0.14831E+01_JPRB, 0.15531E+01_JPRB, 0.16264E+01_JPRB, 0.17097E+01_JPRB, 0.17999E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.20185E+01_JPRB, 0.21480E+01_JPRB, 0.23276E+01_JPRB, 0.22751E+01_JPRB, 0.20976E+01_JPRB, 0.22062E+01_JPRB, 0.20907E+01_JPRB ,&
&  0.17813E+01_JPRB, 0.13316E+01_JPRB, 0.13316E+01_JPRB, 0.14432E+01_JPRB, 0.14287E+01_JPRB, 0.14058E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.10619E+01_JPRB, 0.98505E+00_JPRB, 0.93096E+00_JPRB, 0.86789E+00_JPRB, 0.82497E+00_JPRB, 0.79774E+00_JPRB ,&
&  0.77748E+00_JPRB, 0.79262E+00_JPRB, 0.81027E+00_JPRB, 0.76141E+00_JPRB, 0.73354E+00_JPRB, 0.79929E+00_JPRB, 0.93677E+00_JPRB ,&
&  0.11031E+01_JPRB, 0.96299E+00_JPRB, 0.84004E+00_JPRB, 0.82861E+00_JPRB, 0.77453E+00_JPRB, 0.71646E+00_JPRB, 0.69004E+00_JPRB ,&
&  0.68995E+00_JPRB, 0.67835E+00_JPRB, 0.78157E+00_JPRB, 0.11791E+01_JPRB, 0.15311E+01_JPRB, 0.17618E+01_JPRB, 0.14563E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.77909E+00_JPRB, 0.62594E+00_JPRB, 0.58649E+00_JPRB, 0.56586E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.56001E+00_JPRB, 0.55069E+00_JPRB, 0.55158E+00_JPRB, 0.54839E+00_JPRB, 0.58083E+00_JPRB, 0.61724E+00_JPRB, 0.62758E+00_JPRB ,&
&  0.62325E+00_JPRB, 0.64239E+00_JPRB, 0.66094E+00_JPRB, 0.69493E+00_JPRB, 0.76372E+00_JPRB, 0.79939E+00_JPRB, 0.84121E+00_JPRB ,&
&  0.88273E+00_JPRB, 0.88673E+00_JPRB, 0.86582E+00_JPRB, 0.86347E+00_JPRB, 0.89981E+00_JPRB, 0.93552E+00_JPRB, 0.99156E+00_JPRB ,&
&  0.10675E+01_JPRB, 0.11595E+01_JPRB, 0.12729E+01_JPRB, 0.14051E+01_JPRB, 0.15551E+01_JPRB, 0.16891E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.73752E+00_JPRB, 0.70911E+00_JPRB, 0.84294E+00_JPRB, 0.83959E+00_JPRB, 0.80251E+00_JPRB, 0.84200E+00_JPRB, 0.92213E+00_JPRB ,&
&  0.97929E+00_JPRB, 0.93720E+00_JPRB, 0.10227E+01_JPRB, 0.12964E+01_JPRB, 0.13106E+01_JPRB, 0.13949E+01_JPRB, 0.14700E+01_JPRB ,&
&  0.15378E+01_JPRB, 0.16106E+01_JPRB, 0.16708E+01_JPRB, 0.17237E+01_JPRB, 0.17938E+01_JPRB, 0.18841E+01_JPRB, 0.20087E+01_JPRB ,&
&  0.21688E+01_JPRB, 0.24123E+01_JPRB, 0.26507E+01_JPRB, 0.24105E+01_JPRB, 0.24654E+01_JPRB, 0.25968E+01_JPRB, 0.29294E+01_JPRB ,&
&  0.26602E+01_JPRB, 0.19305E+01_JPRB, 0.15211E+01_JPRB, 0.14233E+01_JPRB, 0.12951E+01_JPRB, 0.12720E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.11315E+01_JPRB, 0.10615E+01_JPRB, 0.99291E+00_JPRB, 0.92765E+00_JPRB, 0.84797E+00_JPRB, 0.82359E+00_JPRB, 0.81379E+00_JPRB ,&
&  0.85096E+00_JPRB, 0.87728E+00_JPRB, 0.85864E+00_JPRB, 0.82538E+00_JPRB, 0.80049E+00_JPRB, 0.83540E+00_JPRB, 0.90545E+00_JPRB ,&
&  0.95231E+00_JPRB, 0.95863E+00_JPRB, 0.96403E+00_JPRB, 0.97227E+00_JPRB, 0.97545E+00_JPRB, 0.96951E+00_JPRB, 0.93444E+00_JPRB ,&
&  0.87620E+00_JPRB, 0.82987E+00_JPRB, 0.82954E+00_JPRB, 0.81657E+00_JPRB, 0.97293E+00_JPRB, 0.12679E+01_JPRB, 0.13040E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.86003E+00_JPRB, 0.69578E+00_JPRB, 0.64441E+00_JPRB, 0.62009E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.66626E+00_JPRB, 0.63216E+00_JPRB, 0.62427E+00_JPRB, 0.60828E+00_JPRB, 0.62171E+00_JPRB, 0.64766E+00_JPRB, 0.66681E+00_JPRB ,&
&  0.67142E+00_JPRB, 0.67038E+00_JPRB, 0.67171E+00_JPRB, 0.69239E+00_JPRB, 0.72105E+00_JPRB, 0.75203E+00_JPRB, 0.79091E+00_JPRB ,&
&  0.81725E+00_JPRB, 0.83213E+00_JPRB, 0.81039E+00_JPRB, 0.78093E+00_JPRB, 0.80035E+00_JPRB, 0.83480E+00_JPRB, 0.88591E+00_JPRB ,&
&  0.95373E+00_JPRB, 0.10337E+01_JPRB, 0.11488E+01_JPRB, 0.13366E+01_JPRB, 0.16217E+01_JPRB, 0.19656E+01_JPRB, 0.23222E+01_JPRB ,&
&  0.18524E+01_JPRB, 0.87175E+00_JPRB, 0.90304E+00_JPRB, 0.96101E+00_JPRB, 0.87484E+00_JPRB, 0.91689E+00_JPRB, 0.93955E+00_JPRB ,&
&  0.94070E+00_JPRB, 0.95893E+00_JPRB, 0.95035E+00_JPRB, 0.10770E+01_JPRB, 0.12481E+01_JPRB, 0.13545E+01_JPRB, 0.14532E+01_JPRB ,&
&  0.15279E+01_JPRB, 0.15828E+01_JPRB, 0.16094E+01_JPRB, 0.16365E+01_JPRB, 0.16937E+01_JPRB, 0.17820E+01_JPRB, 0.19050E+01_JPRB ,&
&  0.20817E+01_JPRB, 0.22983E+01_JPRB, 0.25749E+01_JPRB, 0.23338E+01_JPRB, 0.26036E+01_JPRB, 0.26135E+01_JPRB, 0.26491E+01_JPRB ,&
&  0.24764E+01_JPRB, 0.17864E+01_JPRB, 0.14298E+01_JPRB, 0.13905E+01_JPRB, 0.11857E+01_JPRB, 0.93854E+00_JPRB, 0.10418E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.10364E+01_JPRB, 0.10106E+01_JPRB, 0.92842E+00_JPRB, 0.85588E+00_JPRB, 0.83916E+00_JPRB, 0.89269E+00_JPRB ,&
&  0.94647E+00_JPRB, 0.97818E+00_JPRB, 0.94011E+00_JPRB, 0.93388E+00_JPRB, 0.91882E+00_JPRB, 0.94979E+00_JPRB, 0.10018E+01_JPRB ,&
&  0.10459E+01_JPRB, 0.10877E+01_JPRB, 0.11112E+01_JPRB, 0.11445E+01_JPRB, 0.11821E+01_JPRB, 0.11128E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.98871E+00_JPRB, 0.96999E+00_JPRB, 0.91614E+00_JPRB, 0.87935E+00_JPRB, 0.85553E+00_JPRB, 0.88812E+00_JPRB, 0.99153E+00_JPRB ,&
&  0.97575E+00_JPRB, 0.86280E+00_JPRB, 0.74679E+00_JPRB, 0.71607E+00_JPRB, 0.72155E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.76543E+00_JPRB, 0.68662E+00_JPRB, 0.64309E+00_JPRB, 0.63775E+00_JPRB, 0.62393E+00_JPRB, 0.62923E+00_JPRB, 0.63816E+00_JPRB ,&
&  0.63518E+00_JPRB, 0.65422E+00_JPRB, 0.64834E+00_JPRB, 0.66238E+00_JPRB, 0.68327E+00_JPRB, 0.69098E+00_JPRB, 0.71665E+00_JPRB ,&
&  0.74218E+00_JPRB, 0.77117E+00_JPRB, 0.79662E+00_JPRB, 0.76506E+00_JPRB, 0.76393E+00_JPRB, 0.77393E+00_JPRB, 0.80152E+00_JPRB ,&
&  0.85440E+00_JPRB, 0.92649E+00_JPRB, 0.10100E+01_JPRB, 0.11143E+01_JPRB, 0.12674E+01_JPRB, 0.15433E+01_JPRB, 0.20572E+01_JPRB ,&
&  0.33319E+01_JPRB, 0.27557E+01_JPRB, 0.89231E+00_JPRB, 0.11162E+01_JPRB, 0.10847E+01_JPRB, 0.10769E+01_JPRB, 0.94226E+00_JPRB ,&
&  0.99633E+00_JPRB, 0.10727E+01_JPRB, 0.98762E+00_JPRB, 0.95911E+00_JPRB, 0.10234E+01_JPRB, 0.11807E+01_JPRB, 0.13197E+01_JPRB ,&
&  0.14018E+01_JPRB, 0.14567E+01_JPRB, 0.14707E+01_JPRB, 0.14869E+01_JPRB, 0.15155E+01_JPRB, 0.15755E+01_JPRB, 0.16745E+01_JPRB ,&
&  0.18296E+01_JPRB, 0.20509E+01_JPRB, 0.23373E+01_JPRB, 0.22618E+01_JPRB, 0.27173E+01_JPRB, 0.26276E+01_JPRB, 0.24741E+01_JPRB ,&
&  0.21008E+01_JPRB, 0.17832E+01_JPRB, 0.15875E+01_JPRB, 0.13735E+01_JPRB, 0.10400E+01_JPRB, 0.87925E+00_JPRB, 0.94763E+00_JPRB ,&
&  0.98069E+00_JPRB, 0.91272E+00_JPRB, 0.88898E+00_JPRB, 0.88280E+00_JPRB, 0.84600E+00_JPRB, 0.90954E+00_JPRB, 0.99286E+00_JPRB ,&
&  0.10299E+01_JPRB, 0.10307E+01_JPRB, 0.10211E+01_JPRB, 0.10146E+01_JPRB, 0.10150E+01_JPRB, 0.10352E+01_JPRB, 0.10794E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11465E+01_JPRB, 0.11646E+01_JPRB, 0.12045E+01_JPRB, 0.12423E+01_JPRB, 0.11664E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.10397E+01_JPRB, 0.10014E+01_JPRB, 0.96669E+00_JPRB, 0.91129E+00_JPRB, 0.88118E+00_JPRB, 0.87605E+00_JPRB ,&
&  0.95768E+00_JPRB, 0.89554E+00_JPRB, 0.83918E+00_JPRB, 0.77209E+00_JPRB, 0.79366E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.80708E+00_JPRB, 0.75985E+00_JPRB, 0.68250E+00_JPRB, 0.65240E+00_JPRB, 0.64481E+00_JPRB, 0.63184E+00_JPRB, 0.61389E+00_JPRB ,&
&  0.59042E+00_JPRB, 0.58201E+00_JPRB, 0.60793E+00_JPRB, 0.63895E+00_JPRB, 0.66640E+00_JPRB, 0.66961E+00_JPRB, 0.67841E+00_JPRB ,&
&  0.68471E+00_JPRB, 0.70793E+00_JPRB, 0.75231E+00_JPRB, 0.76573E+00_JPRB, 0.75764E+00_JPRB, 0.75542E+00_JPRB, 0.76244E+00_JPRB ,&
&  0.77851E+00_JPRB, 0.80586E+00_JPRB, 0.85910E+00_JPRB, 0.92068E+00_JPRB, 0.10106E+01_JPRB, 0.11657E+01_JPRB, 0.14965E+01_JPRB ,&
&  0.22010E+01_JPRB, 0.38998E+01_JPRB, 0.10266E+01_JPRB, 0.13653E+01_JPRB, 0.13177E+01_JPRB, 0.12071E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.11185E+01_JPRB, 0.11002E+01_JPRB, 0.99415E+00_JPRB, 0.84937E+00_JPRB, 0.93421E+00_JPRB, 0.10959E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.12897E+01_JPRB, 0.13028E+01_JPRB, 0.12894E+01_JPRB, 0.12965E+01_JPRB, 0.13147E+01_JPRB, 0.13663E+01_JPRB ,&
&  0.14748E+01_JPRB, 0.16609E+01_JPRB, 0.19115E+01_JPRB, 0.19673E+01_JPRB, 0.25058E+01_JPRB, 0.26690E+01_JPRB, 0.25748E+01_JPRB ,&
&  0.21362E+01_JPRB, 0.19917E+01_JPRB, 0.18251E+01_JPRB, 0.14875E+01_JPRB, 0.11343E+01_JPRB, 0.94924E+00_JPRB, 0.97719E+00_JPRB ,&
&  0.95169E+00_JPRB, 0.90017E+00_JPRB, 0.88328E+00_JPRB, 0.87203E+00_JPRB, 0.88499E+00_JPRB, 0.95274E+00_JPRB, 0.10089E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.10392E+01_JPRB, 0.10273E+01_JPRB, 0.10233E+01_JPRB, 0.10327E+01_JPRB, 0.10478E+01_JPRB, 0.10908E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.11526E+01_JPRB, 0.11725E+01_JPRB, 0.11564E+01_JPRB, 0.11820E+01_JPRB, 0.11510E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.11727E+01_JPRB, 0.10654E+01_JPRB, 0.10035E+01_JPRB, 0.10066E+01_JPRB, 0.98041E+00_JPRB, 0.95349E+00_JPRB ,&
&  0.99112E+00_JPRB, 0.10059E+01_JPRB, 0.97880E+00_JPRB, 0.85163E+00_JPRB, 0.85525E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.85506E+00_JPRB, 0.80662E+00_JPRB, 0.75275E+00_JPRB, 0.68892E+00_JPRB, 0.68623E+00_JPRB, 0.69147E+00_JPRB, 0.66067E+00_JPRB ,&
&  0.60882E+00_JPRB, 0.60165E+00_JPRB, 0.61052E+00_JPRB, 0.63344E+00_JPRB, 0.65301E+00_JPRB, 0.65651E+00_JPRB, 0.67803E+00_JPRB ,&
&  0.67098E+00_JPRB, 0.66757E+00_JPRB, 0.69056E+00_JPRB, 0.72718E+00_JPRB, 0.73545E+00_JPRB, 0.73818E+00_JPRB, 0.75442E+00_JPRB ,&
&  0.76195E+00_JPRB, 0.75342E+00_JPRB, 0.75764E+00_JPRB, 0.78268E+00_JPRB, 0.84296E+00_JPRB, 0.94901E+00_JPRB, 0.11463E+01_JPRB ,&
&  0.18105E+01_JPRB, 0.42393E+01_JPRB, 0.28497E+01_JPRB, 0.24156E+01_JPRB, 0.17351E+01_JPRB, 0.13902E+01_JPRB, 0.12566E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.13055E+01_JPRB, 0.12865E+01_JPRB, 0.10426E+01_JPRB, 0.88082E+00_JPRB, 0.81736E+00_JPRB, 0.90511E+00_JPRB ,&
&  0.99575E+00_JPRB, 0.10560E+01_JPRB, 0.11039E+01_JPRB, 0.11098E+01_JPRB, 0.11135E+01_JPRB, 0.11274E+01_JPRB, 0.11439E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.13173E+01_JPRB, 0.15159E+01_JPRB, 0.17408E+01_JPRB, 0.20324E+01_JPRB, 0.23893E+01_JPRB, 0.27535E+01_JPRB ,&
&  0.26543E+01_JPRB, 0.24061E+01_JPRB, 0.20428E+01_JPRB, 0.15719E+01_JPRB, 0.11850E+01_JPRB, 0.11431E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.95879E+00_JPRB, 0.90732E+00_JPRB, 0.90414E+00_JPRB, 0.92606E+00_JPRB, 0.95597E+00_JPRB, 0.99571E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10336E+01_JPRB, 0.10299E+01_JPRB, 0.10244E+01_JPRB, 0.10262E+01_JPRB, 0.10310E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.11046E+01_JPRB, 0.10799E+01_JPRB, 0.10789E+01_JPRB, 0.10966E+01_JPRB, 0.11032E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.11496E+01_JPRB, 0.11319E+01_JPRB, 0.11025E+01_JPRB, 0.10761E+01_JPRB, 0.10790E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10436E+01_JPRB, 0.97024E+00_JPRB, 0.92546E+00_JPRB, 0.91477E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.90931E+00_JPRB, 0.86909E+00_JPRB, 0.82493E+00_JPRB, 0.77200E+00_JPRB, 0.73876E+00_JPRB, 0.73806E+00_JPRB, 0.74766E+00_JPRB ,&
&  0.71824E+00_JPRB, 0.69022E+00_JPRB, 0.67834E+00_JPRB, 0.69184E+00_JPRB, 0.71417E+00_JPRB, 0.71817E+00_JPRB, 0.71955E+00_JPRB ,&
&  0.70722E+00_JPRB, 0.70173E+00_JPRB, 0.68942E+00_JPRB, 0.69708E+00_JPRB, 0.74855E+00_JPRB, 0.75150E+00_JPRB, 0.74393E+00_JPRB ,&
&  0.76292E+00_JPRB, 0.75840E+00_JPRB, 0.73921E+00_JPRB, 0.74142E+00_JPRB, 0.76284E+00_JPRB, 0.83080E+00_JPRB, 0.98036E+00_JPRB ,&
&  0.16763E+01_JPRB, 0.39263E+01_JPRB, 0.19608E+01_JPRB, 0.27538E+01_JPRB, 0.19094E+01_JPRB, 0.15455E+01_JPRB, 0.12463E+01_JPRB ,&
&  0.12684E+01_JPRB, 0.14958E+01_JPRB, 0.12548E+01_JPRB, 0.10887E+01_JPRB, 0.93515E+00_JPRB, 0.86054E+00_JPRB, 0.89277E+00_JPRB ,&
&  0.93829E+00_JPRB, 0.94442E+00_JPRB, 0.95570E+00_JPRB, 0.97237E+00_JPRB, 0.96373E+00_JPRB, 0.96319E+00_JPRB, 0.98248E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.10643E+01_JPRB, 0.11900E+01_JPRB, 0.13589E+01_JPRB, 0.17922E+01_JPRB, 0.23199E+01_JPRB, 0.28997E+01_JPRB ,&
&  0.29322E+01_JPRB, 0.29685E+01_JPRB, 0.23121E+01_JPRB, 0.16877E+01_JPRB, 0.13039E+01_JPRB, 0.11813E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.10068E+01_JPRB, 0.97774E+00_JPRB, 0.94356E+00_JPRB, 0.94738E+00_JPRB, 0.10045E+01_JPRB, 0.10423E+01_JPRB, 0.10289E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.10302E+01_JPRB, 0.10248E+01_JPRB, 0.10100E+01_JPRB, 0.10193E+01_JPRB, 0.10188E+01_JPRB, 0.10105E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.10361E+01_JPRB, 0.10364E+01_JPRB, 0.10455E+01_JPRB, 0.10588E+01_JPRB, 0.10690E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.11118E+01_JPRB, 0.11245E+01_JPRB, 0.11229E+01_JPRB, 0.11067E+01_JPRB, 0.11487E+01_JPRB, 0.11064E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.10828E+01_JPRB, 0.10189E+01_JPRB, 0.98787E+00_JPRB, 0.96871E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.97939E+00_JPRB, 0.92358E+00_JPRB, 0.88324E+00_JPRB, 0.84184E+00_JPRB, 0.80804E+00_JPRB, 0.78563E+00_JPRB, 0.78395E+00_JPRB ,&
&  0.77168E+00_JPRB, 0.73898E+00_JPRB, 0.71566E+00_JPRB, 0.71527E+00_JPRB, 0.71434E+00_JPRB, 0.71942E+00_JPRB, 0.72312E+00_JPRB ,&
&  0.71808E+00_JPRB, 0.70699E+00_JPRB, 0.70319E+00_JPRB, 0.70737E+00_JPRB, 0.73617E+00_JPRB, 0.77679E+00_JPRB, 0.78863E+00_JPRB ,&
&  0.77923E+00_JPRB, 0.77137E+00_JPRB, 0.75089E+00_JPRB, 0.73325E+00_JPRB, 0.73495E+00_JPRB, 0.77409E+00_JPRB, 0.89649E+00_JPRB ,&
&  0.15765E+01_JPRB, 0.42834E+01_JPRB, 0.18751E+01_JPRB, 0.29125E+01_JPRB, 0.20273E+01_JPRB, 0.16246E+01_JPRB, 0.12980E+01_JPRB ,&
&  0.12963E+01_JPRB, 0.14262E+01_JPRB, 0.12813E+01_JPRB, 0.11459E+01_JPRB, 0.10228E+01_JPRB, 0.94523E+00_JPRB, 0.91374E+00_JPRB ,&
&  0.94768E+00_JPRB, 0.93472E+00_JPRB, 0.91875E+00_JPRB, 0.91402E+00_JPRB, 0.89465E+00_JPRB, 0.88796E+00_JPRB, 0.89744E+00_JPRB ,&
&  0.89527E+00_JPRB, 0.90207E+00_JPRB, 0.95977E+00_JPRB, 0.12054E+01_JPRB, 0.15055E+01_JPRB, 0.22151E+01_JPRB, 0.32762E+01_JPRB ,&
&  0.33013E+01_JPRB, 0.34424E+01_JPRB, 0.25521E+01_JPRB, 0.18327E+01_JPRB, 0.13889E+01_JPRB, 0.12174E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.10381E+01_JPRB, 0.10027E+01_JPRB, 0.99826E+00_JPRB, 0.10262E+01_JPRB, 0.10240E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.99740E+00_JPRB, 0.97930E+00_JPRB, 0.97096E+00_JPRB, 0.98328E+00_JPRB, 0.98990E+00_JPRB, 0.98519E+00_JPRB ,&
&  0.98130E+00_JPRB, 0.98756E+00_JPRB, 0.10096E+01_JPRB, 0.10261E+01_JPRB, 0.10479E+01_JPRB, 0.10565E+01_JPRB, 0.10352E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.11010E+01_JPRB, 0.11055E+01_JPRB, 0.11572E+01_JPRB, 0.11809E+01_JPRB, 0.12807E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11020E+01_JPRB, 0.10549E+01_JPRB, 0.10350E+01_JPRB, 0.10262E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.10402E+01_JPRB, 0.99884E+00_JPRB, 0.96323E+00_JPRB, 0.92311E+00_JPRB, 0.87452E+00_JPRB, 0.84436E+00_JPRB, 0.82085E+00_JPRB ,&
&  0.80525E+00_JPRB, 0.77174E+00_JPRB, 0.74651E+00_JPRB, 0.74088E+00_JPRB, 0.74381E+00_JPRB, 0.73649E+00_JPRB, 0.73450E+00_JPRB ,&
&  0.74039E+00_JPRB, 0.74313E+00_JPRB, 0.73386E+00_JPRB, 0.72462E+00_JPRB, 0.72592E+00_JPRB, 0.76165E+00_JPRB, 0.77948E+00_JPRB ,&
&  0.78650E+00_JPRB, 0.77612E+00_JPRB, 0.75322E+00_JPRB, 0.72446E+00_JPRB, 0.71765E+00_JPRB, 0.73541E+00_JPRB, 0.82651E+00_JPRB ,&
&  0.13641E+01_JPRB, 0.17431E+01_JPRB, 0.30098E+01_JPRB, 0.30209E+01_JPRB, 0.21282E+01_JPRB, 0.17119E+01_JPRB, 0.14921E+01_JPRB ,&
&  0.14739E+01_JPRB, 0.14105E+01_JPRB, 0.12836E+01_JPRB, 0.11787E+01_JPRB, 0.10893E+01_JPRB, 0.10148E+01_JPRB, 0.95955E+00_JPRB ,&
&  0.93548E+00_JPRB, 0.91843E+00_JPRB, 0.90677E+00_JPRB, 0.90252E+00_JPRB, 0.89252E+00_JPRB, 0.88786E+00_JPRB, 0.88729E+00_JPRB ,&
&  0.88980E+00_JPRB, 0.88650E+00_JPRB, 0.89559E+00_JPRB, 0.98824E+00_JPRB, 0.13599E+01_JPRB, 0.19965E+01_JPRB, 0.29834E+01_JPRB ,&
&  0.35839E+01_JPRB, 0.33641E+01_JPRB, 0.24511E+01_JPRB, 0.18695E+01_JPRB, 0.14406E+01_JPRB, 0.12126E+01_JPRB, 0.11510E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11032E+01_JPRB, 0.10577E+01_JPRB, 0.10419E+01_JPRB, 0.10382E+01_JPRB, 0.10186E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.10002E+01_JPRB, 0.98766E+00_JPRB, 0.97332E+00_JPRB, 0.95731E+00_JPRB, 0.94629E+00_JPRB, 0.94292E+00_JPRB ,&
&  0.94357E+00_JPRB, 0.94268E+00_JPRB, 0.98476E+00_JPRB, 0.10052E+01_JPRB, 0.10618E+01_JPRB, 0.10861E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.11056E+01_JPRB, 0.11036E+01_JPRB, 0.11041E+01_JPRB, 0.11826E+01_JPRB, 0.11978E+01_JPRB, 0.13377E+01_JPRB, 0.12087E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11382E+01_JPRB, 0.10985E+01_JPRB, 0.10627E+01_JPRB, 0.10494E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.10539E+01_JPRB, 0.10076E+01_JPRB, 0.97557E+00_JPRB, 0.95069E+00_JPRB, 0.91643E+00_JPRB, 0.88158E+00_JPRB, 0.86673E+00_JPRB ,&
&  0.85548E+00_JPRB, 0.82915E+00_JPRB, 0.79950E+00_JPRB, 0.76405E+00_JPRB, 0.75240E+00_JPRB, 0.75600E+00_JPRB, 0.76135E+00_JPRB ,&
&  0.76000E+00_JPRB, 0.76093E+00_JPRB, 0.75962E+00_JPRB, 0.75768E+00_JPRB, 0.75576E+00_JPRB, 0.77784E+00_JPRB, 0.78773E+00_JPRB ,&
&  0.78038E+00_JPRB, 0.78146E+00_JPRB, 0.76048E+00_JPRB, 0.73816E+00_JPRB, 0.72723E+00_JPRB, 0.71552E+00_JPRB, 0.72609E+00_JPRB ,&
&  0.10817E+01_JPRB, 0.19954E+01_JPRB, 0.28831E+01_JPRB, 0.26586E+01_JPRB, 0.20866E+01_JPRB, 0.16680E+01_JPRB, 0.15540E+01_JPRB ,&
&  0.14753E+01_JPRB, 0.13948E+01_JPRB, 0.12834E+01_JPRB, 0.11929E+01_JPRB, 0.11196E+01_JPRB, 0.10626E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.96755E+00_JPRB, 0.94955E+00_JPRB, 0.92256E+00_JPRB, 0.91002E+00_JPRB, 0.90256E+00_JPRB, 0.90410E+00_JPRB, 0.90472E+00_JPRB ,&
&  0.90917E+00_JPRB, 0.91128E+00_JPRB, 0.91350E+00_JPRB, 0.94310E+00_JPRB, 0.11650E+01_JPRB, 0.13123E+01_JPRB, 0.21490E+01_JPRB ,&
&  0.30136E+01_JPRB, 0.24861E+01_JPRB, 0.19666E+01_JPRB, 0.16774E+01_JPRB, 0.14406E+01_JPRB, 0.12264E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.10731E+01_JPRB, 0.10353E+01_JPRB, 0.10205E+01_JPRB, 0.10043E+01_JPRB, 0.99285E+00_JPRB, 0.98261E+00_JPRB ,&
&  0.96243E+00_JPRB, 0.96124E+00_JPRB, 0.96058E+00_JPRB, 0.95942E+00_JPRB, 0.94448E+00_JPRB, 0.93731E+00_JPRB, 0.92966E+00_JPRB ,&
&  0.92390E+00_JPRB, 0.91034E+00_JPRB, 0.95349E+00_JPRB, 0.96546E+00_JPRB, 0.10458E+01_JPRB, 0.11140E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11124E+01_JPRB, 0.11339E+01_JPRB, 0.11354E+01_JPRB, 0.12148E+01_JPRB, 0.13599E+01_JPRB, 0.15019E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.11981E+01_JPRB, 0.11553E+01_JPRB, 0.11076E+01_JPRB, 0.10900E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.11063E+01_JPRB, 0.10481E+01_JPRB, 0.10014E+01_JPRB, 0.95849E+00_JPRB, 0.93893E+00_JPRB, 0.91198E+00_JPRB, 0.88177E+00_JPRB ,&
&  0.86332E+00_JPRB, 0.85750E+00_JPRB, 0.84592E+00_JPRB, 0.82390E+00_JPRB, 0.80463E+00_JPRB, 0.79688E+00_JPRB, 0.79538E+00_JPRB ,&
&  0.79301E+00_JPRB, 0.79174E+00_JPRB, 0.78931E+00_JPRB, 0.78112E+00_JPRB, 0.78323E+00_JPRB, 0.79177E+00_JPRB, 0.80076E+00_JPRB ,&
&  0.78340E+00_JPRB, 0.76942E+00_JPRB, 0.75830E+00_JPRB, 0.74686E+00_JPRB, 0.74290E+00_JPRB, 0.72379E+00_JPRB, 0.70247E+00_JPRB ,&
&  0.84853E+00_JPRB, 0.13713E+01_JPRB, 0.19261E+01_JPRB, 0.23673E+01_JPRB, 0.20661E+01_JPRB, 0.17371E+01_JPRB, 0.15902E+01_JPRB ,&
&  0.14811E+01_JPRB, 0.13547E+01_JPRB, 0.12520E+01_JPRB, 0.11808E+01_JPRB, 0.11401E+01_JPRB, 0.10929E+01_JPRB, 0.10230E+01_JPRB ,&
&  0.97145E+00_JPRB, 0.93974E+00_JPRB, 0.92675E+00_JPRB, 0.92620E+00_JPRB, 0.91971E+00_JPRB, 0.92872E+00_JPRB, 0.93332E+00_JPRB ,&
&  0.93382E+00_JPRB, 0.92804E+00_JPRB, 0.91854E+00_JPRB, 0.91592E+00_JPRB, 0.10279E+01_JPRB, 0.11316E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.15248E+01_JPRB, 0.15691E+01_JPRB, 0.15055E+01_JPRB, 0.14504E+01_JPRB, 0.13689E+01_JPRB, 0.12238E+01_JPRB, 0.11172E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10288E+01_JPRB, 0.99402E+00_JPRB, 0.98113E+00_JPRB, 0.96878E+00_JPRB, 0.95489E+00_JPRB, 0.94939E+00_JPRB ,&
&  0.94197E+00_JPRB, 0.93608E+00_JPRB, 0.93015E+00_JPRB, 0.93646E+00_JPRB, 0.92993E+00_JPRB, 0.91093E+00_JPRB, 0.89677E+00_JPRB ,&
&  0.89214E+00_JPRB, 0.87825E+00_JPRB, 0.89403E+00_JPRB, 0.93769E+00_JPRB, 0.98333E+00_JPRB, 0.10155E+01_JPRB, 0.10529E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.11790E+01_JPRB, 0.11618E+01_JPRB, 0.12777E+01_JPRB, 0.14554E+01_JPRB, 0.15836E+01_JPRB, 0.14158E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.12554E+01_JPRB, 0.12015E+01_JPRB, 0.11341E+01_JPRB, 0.11533E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.11132E+01_JPRB, 0.10758E+01_JPRB, 0.10328E+01_JPRB, 0.98058E+00_JPRB, 0.95395E+00_JPRB, 0.93108E+00_JPRB, 0.90925E+00_JPRB ,&
&  0.89743E+00_JPRB, 0.88564E+00_JPRB, 0.87690E+00_JPRB, 0.86001E+00_JPRB, 0.84318E+00_JPRB, 0.83270E+00_JPRB, 0.82071E+00_JPRB ,&
&  0.82045E+00_JPRB, 0.82012E+00_JPRB, 0.81180E+00_JPRB, 0.80289E+00_JPRB, 0.79832E+00_JPRB, 0.79430E+00_JPRB, 0.79749E+00_JPRB ,&
&  0.78897E+00_JPRB, 0.77863E+00_JPRB, 0.77833E+00_JPRB, 0.77073E+00_JPRB, 0.75725E+00_JPRB, 0.74514E+00_JPRB, 0.71376E+00_JPRB ,&
&  0.73456E+00_JPRB, 0.88137E+00_JPRB, 0.12858E+01_JPRB, 0.17433E+01_JPRB, 0.17211E+01_JPRB, 0.16603E+01_JPRB, 0.15303E+01_JPRB ,&
&  0.14121E+01_JPRB, 0.13080E+01_JPRB, 0.11916E+01_JPRB, 0.11130E+01_JPRB, 0.10869E+01_JPRB, 0.10591E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.99065E+00_JPRB, 0.95742E+00_JPRB, 0.93638E+00_JPRB, 0.93961E+00_JPRB, 0.93379E+00_JPRB, 0.93325E+00_JPRB, 0.93497E+00_JPRB ,&
&  0.92824E+00_JPRB, 0.92051E+00_JPRB, 0.90268E+00_JPRB, 0.88978E+00_JPRB, 0.88213E+00_JPRB, 0.89413E+00_JPRB, 0.96608E+00_JPRB ,&
&  0.10683E+01_JPRB, 0.11083E+01_JPRB, 0.11327E+01_JPRB, 0.11614E+01_JPRB, 0.11801E+01_JPRB, 0.11595E+01_JPRB, 0.11016E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.10234E+01_JPRB, 0.98309E+00_JPRB, 0.98627E+00_JPRB, 0.97032E+00_JPRB, 0.94862E+00_JPRB, 0.93078E+00_JPRB ,&
&  0.91102E+00_JPRB, 0.90882E+00_JPRB, 0.89581E+00_JPRB, 0.89059E+00_JPRB, 0.88662E+00_JPRB, 0.87680E+00_JPRB, 0.86399E+00_JPRB ,&
&  0.86201E+00_JPRB, 0.85613E+00_JPRB, 0.86406E+00_JPRB, 0.89333E+00_JPRB, 0.93322E+00_JPRB, 0.96183E+00_JPRB, 0.99290E+00_JPRB ,&
&  0.10243E+01_JPRB, 0.10839E+01_JPRB, 0.10903E+01_JPRB, 0.12660E+01_JPRB, 0.14630E+01_JPRB, 0.14872E+01_JPRB, 0.14091E+01_JPRB ,&
&  0.13299E+01_JPRB, 0.12592E+01_JPRB, 0.12104E+01_JPRB, 0.11364E+01_JPRB, 0.11082E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.98313E+00_JPRB, 0.99291E+00_JPRB, 0.96911E+00_JPRB, 0.92886E+00_JPRB, 0.90623E+00_JPRB, 0.89594E+00_JPRB, 0.89264E+00_JPRB ,&
&  0.88711E+00_JPRB, 0.87522E+00_JPRB, 0.87898E+00_JPRB, 0.88425E+00_JPRB, 0.88016E+00_JPRB, 0.86693E+00_JPRB, 0.85731E+00_JPRB ,&
&  0.85399E+00_JPRB, 0.85688E+00_JPRB, 0.86118E+00_JPRB, 0.85027E+00_JPRB, 0.82800E+00_JPRB, 0.81253E+00_JPRB, 0.79938E+00_JPRB ,&
&  0.79772E+00_JPRB, 0.79782E+00_JPRB, 0.79585E+00_JPRB, 0.79737E+00_JPRB, 0.77582E+00_JPRB, 0.75125E+00_JPRB, 0.73066E+00_JPRB ,&
&  0.71818E+00_JPRB, 0.67469E+00_JPRB, 0.87200E+00_JPRB, 0.11702E+01_JPRB, 0.13154E+01_JPRB, 0.13097E+01_JPRB, 0.12205E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.11162E+01_JPRB, 0.10704E+01_JPRB, 0.10199E+01_JPRB, 0.98911E+00_JPRB, 0.96970E+00_JPRB, 0.95497E+00_JPRB ,&
&  0.94158E+00_JPRB, 0.92628E+00_JPRB, 0.91469E+00_JPRB, 0.89387E+00_JPRB, 0.87654E+00_JPRB, 0.87203E+00_JPRB, 0.87059E+00_JPRB ,&
&  0.87176E+00_JPRB, 0.86428E+00_JPRB, 0.86147E+00_JPRB, 0.85919E+00_JPRB, 0.83723E+00_JPRB, 0.83297E+00_JPRB, 0.83988E+00_JPRB ,&
&  0.86794E+00_JPRB, 0.89637E+00_JPRB, 0.91080E+00_JPRB, 0.95938E+00_JPRB, 0.99309E+00_JPRB, 0.10224E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.99340E+00_JPRB, 0.97732E+00_JPRB, 0.97104E+00_JPRB, 0.95711E+00_JPRB, 0.92685E+00_JPRB, 0.89684E+00_JPRB ,&
&  0.88451E+00_JPRB, 0.87784E+00_JPRB, 0.86189E+00_JPRB, 0.85306E+00_JPRB, 0.84463E+00_JPRB, 0.82944E+00_JPRB, 0.81682E+00_JPRB ,&
&  0.80839E+00_JPRB, 0.80775E+00_JPRB, 0.81092E+00_JPRB, 0.82712E+00_JPRB, 0.85362E+00_JPRB, 0.88717E+00_JPRB, 0.92183E+00_JPRB ,&
&  0.95574E+00_JPRB, 0.99021E+00_JPRB, 0.10352E+01_JPRB, 0.11698E+01_JPRB, 0.12232E+01_JPRB, 0.12520E+01_JPRB, 0.12814E+01_JPRB ,&
&  0.12656E+01_JPRB, 0.12056E+01_JPRB, 0.11415E+01_JPRB, 0.10497E+01_JPRB, 0.94020E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.92875E+00_JPRB, 0.92818E+00_JPRB, 0.91427E+00_JPRB, 0.89505E+00_JPRB, 0.88169E+00_JPRB, 0.87274E+00_JPRB, 0.87381E+00_JPRB ,&
&  0.87072E+00_JPRB, 0.86407E+00_JPRB, 0.86613E+00_JPRB, 0.86777E+00_JPRB, 0.86932E+00_JPRB, 0.86593E+00_JPRB, 0.87004E+00_JPRB ,&
&  0.87029E+00_JPRB, 0.87126E+00_JPRB, 0.87387E+00_JPRB, 0.86677E+00_JPRB, 0.84604E+00_JPRB, 0.83106E+00_JPRB, 0.81623E+00_JPRB ,&
&  0.81486E+00_JPRB, 0.81449E+00_JPRB, 0.81509E+00_JPRB, 0.80874E+00_JPRB, 0.78838E+00_JPRB, 0.77038E+00_JPRB, 0.75872E+00_JPRB ,&
&  0.75772E+00_JPRB, 0.62420E+00_JPRB, 0.71274E+00_JPRB, 0.95104E+00_JPRB, 0.10654E+01_JPRB, 0.10262E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.10073E+01_JPRB, 0.98817E+00_JPRB, 0.95524E+00_JPRB, 0.90745E+00_JPRB, 0.88994E+00_JPRB, 0.86691E+00_JPRB ,&
&  0.84709E+00_JPRB, 0.83927E+00_JPRB, 0.83176E+00_JPRB, 0.82958E+00_JPRB, 0.81756E+00_JPRB, 0.80433E+00_JPRB, 0.80415E+00_JPRB ,&
&  0.81136E+00_JPRB, 0.81055E+00_JPRB, 0.81401E+00_JPRB, 0.81753E+00_JPRB, 0.81116E+00_JPRB, 0.82541E+00_JPRB, 0.83328E+00_JPRB ,&
&  0.84150E+00_JPRB, 0.84642E+00_JPRB, 0.84957E+00_JPRB, 0.85488E+00_JPRB, 0.87223E+00_JPRB, 0.91210E+00_JPRB, 0.93866E+00_JPRB ,&
&  0.93489E+00_JPRB, 0.92416E+00_JPRB, 0.92841E+00_JPRB, 0.92973E+00_JPRB, 0.92056E+00_JPRB, 0.90030E+00_JPRB, 0.87689E+00_JPRB ,&
&  0.86345E+00_JPRB, 0.85178E+00_JPRB, 0.84169E+00_JPRB, 0.82848E+00_JPRB, 0.81338E+00_JPRB, 0.80004E+00_JPRB, 0.79023E+00_JPRB ,&
&  0.78426E+00_JPRB, 0.78213E+00_JPRB, 0.78894E+00_JPRB, 0.80618E+00_JPRB, 0.82934E+00_JPRB, 0.85713E+00_JPRB, 0.88734E+00_JPRB ,&
&  0.92334E+00_JPRB, 0.96484E+00_JPRB, 0.10036E+01_JPRB, 0.10570E+01_JPRB, 0.11558E+01_JPRB, 0.11863E+01_JPRB, 0.12002E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.11391E+01_JPRB, 0.10586E+01_JPRB, 0.82188E+00_JPRB, 0.90131E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.89542E+00_JPRB, 0.89116E+00_JPRB, 0.88623E+00_JPRB, 0.86917E+00_JPRB, 0.85017E+00_JPRB, 0.84195E+00_JPRB, 0.84683E+00_JPRB ,&
&  0.84105E+00_JPRB, 0.82662E+00_JPRB, 0.83062E+00_JPRB, 0.83332E+00_JPRB, 0.83988E+00_JPRB, 0.84269E+00_JPRB, 0.84607E+00_JPRB ,&
&  0.84964E+00_JPRB, 0.85293E+00_JPRB, 0.85036E+00_JPRB, 0.85783E+00_JPRB, 0.84972E+00_JPRB, 0.84353E+00_JPRB, 0.84013E+00_JPRB ,&
&  0.83136E+00_JPRB, 0.82238E+00_JPRB, 0.81493E+00_JPRB, 0.80458E+00_JPRB, 0.79658E+00_JPRB, 0.78700E+00_JPRB, 0.77104E+00_JPRB ,&
&  0.78173E+00_JPRB, 0.63364E+00_JPRB, 0.72394E+00_JPRB, 0.88379E+00_JPRB, 0.88652E+00_JPRB, 0.87427E+00_JPRB, 0.88774E+00_JPRB ,&
&  0.90867E+00_JPRB, 0.91851E+00_JPRB, 0.91069E+00_JPRB, 0.88569E+00_JPRB, 0.83549E+00_JPRB, 0.81091E+00_JPRB, 0.79671E+00_JPRB ,&
&  0.78097E+00_JPRB, 0.76943E+00_JPRB, 0.76251E+00_JPRB, 0.76399E+00_JPRB, 0.76404E+00_JPRB, 0.76097E+00_JPRB, 0.75731E+00_JPRB ,&
&  0.76056E+00_JPRB, 0.76998E+00_JPRB, 0.77542E+00_JPRB, 0.78068E+00_JPRB, 0.78573E+00_JPRB, 0.78123E+00_JPRB, 0.77981E+00_JPRB ,&
&  0.77895E+00_JPRB, 0.78055E+00_JPRB, 0.77983E+00_JPRB, 0.78273E+00_JPRB, 0.78917E+00_JPRB, 0.81592E+00_JPRB, 0.83918E+00_JPRB ,&
&  0.84560E+00_JPRB, 0.84647E+00_JPRB, 0.85041E+00_JPRB, 0.85546E+00_JPRB, 0.86045E+00_JPRB, 0.86109E+00_JPRB, 0.85205E+00_JPRB ,&
&  0.84149E+00_JPRB, 0.82726E+00_JPRB, 0.81946E+00_JPRB, 0.81331E+00_JPRB, 0.80386E+00_JPRB, 0.79786E+00_JPRB, 0.79753E+00_JPRB ,&
&  0.79012E+00_JPRB, 0.77797E+00_JPRB, 0.78005E+00_JPRB, 0.79430E+00_JPRB, 0.80749E+00_JPRB, 0.81712E+00_JPRB, 0.83693E+00_JPRB ,&
&  0.87268E+00_JPRB, 0.91303E+00_JPRB, 0.94433E+00_JPRB, 0.10094E+01_JPRB, 0.10507E+01_JPRB, 0.10774E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10496E+01_JPRB, 0.75935E+00_JPRB, 0.87194E+00_JPRB, 0.89294E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.88762E+00_JPRB, 0.87385E+00_JPRB, 0.85736E+00_JPRB, 0.83666E+00_JPRB, 0.81733E+00_JPRB, 0.80542E+00_JPRB, 0.80980E+00_JPRB ,&
&  0.80784E+00_JPRB, 0.79496E+00_JPRB, 0.80440E+00_JPRB, 0.81698E+00_JPRB, 0.82703E+00_JPRB, 0.83638E+00_JPRB, 0.83948E+00_JPRB ,&
&  0.84284E+00_JPRB, 0.83949E+00_JPRB, 0.84224E+00_JPRB, 0.85386E+00_JPRB, 0.87202E+00_JPRB, 0.87342E+00_JPRB, 0.87024E+00_JPRB ,&
&  0.84762E+00_JPRB, 0.83614E+00_JPRB, 0.82557E+00_JPRB, 0.81270E+00_JPRB, 0.80101E+00_JPRB, 0.79664E+00_JPRB, 0.79880E+00_JPRB ,&
&  0.68473E+00_JPRB, 0.59210E+00_JPRB, 0.71153E+00_JPRB, 0.80862E+00_JPRB, 0.80772E+00_JPRB, 0.80404E+00_JPRB, 0.82129E+00_JPRB ,&
&  0.82795E+00_JPRB, 0.82739E+00_JPRB, 0.82628E+00_JPRB, 0.81716E+00_JPRB, 0.78672E+00_JPRB, 0.75279E+00_JPRB, 0.73514E+00_JPRB ,&
&  0.72474E+00_JPRB, 0.72980E+00_JPRB, 0.73210E+00_JPRB, 0.73967E+00_JPRB, 0.73988E+00_JPRB, 0.73277E+00_JPRB, 0.73099E+00_JPRB ,&
&  0.73140E+00_JPRB, 0.74108E+00_JPRB, 0.75004E+00_JPRB, 0.75643E+00_JPRB, 0.75851E+00_JPRB, 0.75871E+00_JPRB, 0.75517E+00_JPRB ,&
&  0.75257E+00_JPRB, 0.75073E+00_JPRB, 0.74744E+00_JPRB, 0.74999E+00_JPRB, 0.75546E+00_JPRB, 0.76815E+00_JPRB, 0.78791E+00_JPRB ,&
&  0.79940E+00_JPRB, 0.80301E+00_JPRB, 0.80553E+00_JPRB, 0.80663E+00_JPRB, 0.81699E+00_JPRB, 0.82416E+00_JPRB, 0.81854E+00_JPRB ,&
&  0.80954E+00_JPRB, 0.80268E+00_JPRB, 0.80512E+00_JPRB, 0.80624E+00_JPRB, 0.80893E+00_JPRB, 0.80857E+00_JPRB, 0.80207E+00_JPRB ,&
&  0.79249E+00_JPRB, 0.77813E+00_JPRB, 0.76752E+00_JPRB, 0.76712E+00_JPRB, 0.77762E+00_JPRB, 0.78633E+00_JPRB, 0.80570E+00_JPRB ,&
&  0.83956E+00_JPRB, 0.88098E+00_JPRB, 0.91554E+00_JPRB, 0.96775E+00_JPRB, 0.10038E+01_JPRB, 0.10254E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.97181E+00_JPRB, 0.95591E+00_JPRB, 0.92485E+00_JPRB, 0.90435E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.83974E+00_JPRB, 0.83100E+00_JPRB, 0.81433E+00_JPRB, 0.78512E+00_JPRB, 0.74862E+00_JPRB, 0.75523E+00_JPRB, 0.76600E+00_JPRB ,&
&  0.75932E+00_JPRB, 0.74930E+00_JPRB, 0.75951E+00_JPRB, 0.78430E+00_JPRB, 0.80855E+00_JPRB, 0.82228E+00_JPRB, 0.82420E+00_JPRB ,&
&  0.81881E+00_JPRB, 0.82048E+00_JPRB, 0.82855E+00_JPRB, 0.84551E+00_JPRB, 0.84454E+00_JPRB, 0.82655E+00_JPRB, 0.81551E+00_JPRB ,&
&  0.81239E+00_JPRB, 0.81047E+00_JPRB, 0.80767E+00_JPRB, 0.79066E+00_JPRB, 0.77929E+00_JPRB, 0.77610E+00_JPRB, 0.77645E+00_JPRB ,&
&  0.77285E+00_JPRB, 0.63523E+00_JPRB, 0.74569E+00_JPRB, 0.75417E+00_JPRB, 0.76185E+00_JPRB, 0.75286E+00_JPRB, 0.75252E+00_JPRB ,&
&  0.75705E+00_JPRB, 0.77295E+00_JPRB, 0.76663E+00_JPRB, 0.74394E+00_JPRB, 0.71981E+00_JPRB, 0.69305E+00_JPRB, 0.67512E+00_JPRB ,&
&  0.66631E+00_JPRB, 0.66970E+00_JPRB, 0.68014E+00_JPRB, 0.69117E+00_JPRB, 0.70277E+00_JPRB, 0.70463E+00_JPRB, 0.70851E+00_JPRB ,&
&  0.71086E+00_JPRB, 0.71551E+00_JPRB, 0.71756E+00_JPRB, 0.72265E+00_JPRB, 0.72426E+00_JPRB, 0.72073E+00_JPRB, 0.71303E+00_JPRB ,&
&  0.70224E+00_JPRB, 0.69581E+00_JPRB, 0.69518E+00_JPRB, 0.69798E+00_JPRB, 0.70654E+00_JPRB, 0.72202E+00_JPRB, 0.73847E+00_JPRB ,&
&  0.75639E+00_JPRB, 0.76836E+00_JPRB, 0.77509E+00_JPRB, 0.77797E+00_JPRB, 0.78712E+00_JPRB, 0.79657E+00_JPRB, 0.79536E+00_JPRB ,&
&  0.78700E+00_JPRB, 0.77757E+00_JPRB, 0.77325E+00_JPRB, 0.76786E+00_JPRB, 0.76638E+00_JPRB, 0.76540E+00_JPRB, 0.76009E+00_JPRB ,&
&  0.75151E+00_JPRB, 0.74966E+00_JPRB, 0.75097E+00_JPRB, 0.74055E+00_JPRB, 0.74297E+00_JPRB, 0.75395E+00_JPRB, 0.77422E+00_JPRB ,&
&  0.79972E+00_JPRB, 0.83300E+00_JPRB, 0.86570E+00_JPRB, 0.89746E+00_JPRB, 0.91984E+00_JPRB, 0.93248E+00_JPRB, 0.93291E+00_JPRB ,&
&  0.91911E+00_JPRB, 0.89628E+00_JPRB, 0.87568E+00_JPRB, 0.86173E+00_JPRB, 0.85771E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.79249E+00_JPRB, 0.79156E+00_JPRB, 0.77977E+00_JPRB, 0.73917E+00_JPRB, 0.70810E+00_JPRB, 0.71636E+00_JPRB, 0.72446E+00_JPRB ,&
&  0.72362E+00_JPRB, 0.73837E+00_JPRB, 0.76331E+00_JPRB, 0.78853E+00_JPRB, 0.80538E+00_JPRB, 0.80512E+00_JPRB, 0.80894E+00_JPRB ,&
&  0.80611E+00_JPRB, 0.80375E+00_JPRB, 0.81352E+00_JPRB, 0.81835E+00_JPRB, 0.80763E+00_JPRB, 0.78663E+00_JPRB, 0.78424E+00_JPRB ,&
&  0.79030E+00_JPRB, 0.78602E+00_JPRB, 0.76772E+00_JPRB, 0.75519E+00_JPRB, 0.74496E+00_JPRB, 0.75012E+00_JPRB, 0.75710E+00_JPRB ,&
&  0.75181E+00_JPRB, 0.70603E+00_JPRB, 0.74451E+00_JPRB, 0.75403E+00_JPRB, 0.75847E+00_JPRB, 0.74388E+00_JPRB, 0.74785E+00_JPRB ,&
&  0.75865E+00_JPRB, 0.76539E+00_JPRB, 0.74610E+00_JPRB, 0.72406E+00_JPRB, 0.70452E+00_JPRB, 0.68811E+00_JPRB, 0.66118E+00_JPRB ,&
&  0.65137E+00_JPRB, 0.64880E+00_JPRB, 0.66250E+00_JPRB, 0.67697E+00_JPRB, 0.69365E+00_JPRB, 0.70054E+00_JPRB, 0.71102E+00_JPRB ,&
&  0.71301E+00_JPRB, 0.70709E+00_JPRB, 0.70153E+00_JPRB, 0.69696E+00_JPRB, 0.69567E+00_JPRB, 0.68954E+00_JPRB, 0.68056E+00_JPRB ,&
&  0.67331E+00_JPRB, 0.67144E+00_JPRB, 0.67591E+00_JPRB, 0.68524E+00_JPRB, 0.69731E+00_JPRB, 0.71683E+00_JPRB, 0.73266E+00_JPRB ,&
&  0.75183E+00_JPRB, 0.75875E+00_JPRB, 0.76477E+00_JPRB, 0.77820E+00_JPRB, 0.78682E+00_JPRB, 0.78552E+00_JPRB, 0.78242E+00_JPRB ,&
&  0.77851E+00_JPRB, 0.77142E+00_JPRB, 0.77456E+00_JPRB, 0.77221E+00_JPRB, 0.77145E+00_JPRB, 0.76206E+00_JPRB, 0.74631E+00_JPRB ,&
&  0.74020E+00_JPRB, 0.73571E+00_JPRB, 0.72720E+00_JPRB, 0.72441E+00_JPRB, 0.72232E+00_JPRB, 0.74062E+00_JPRB, 0.77273E+00_JPRB ,&
&  0.79095E+00_JPRB, 0.81553E+00_JPRB, 0.84986E+00_JPRB, 0.87315E+00_JPRB, 0.87550E+00_JPRB, 0.87826E+00_JPRB, 0.86997E+00_JPRB ,&
&  0.85356E+00_JPRB, 0.84176E+00_JPRB, 0.82816E+00_JPRB, 0.82155E+00_JPRB, 0.81513E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.74479E+00_JPRB, 0.76574E+00_JPRB, 0.73200E+00_JPRB, 0.71233E+00_JPRB, 0.69060E+00_JPRB, 0.68471E+00_JPRB, 0.67496E+00_JPRB ,&
&  0.68549E+00_JPRB, 0.70250E+00_JPRB, 0.72631E+00_JPRB, 0.73430E+00_JPRB, 0.74116E+00_JPRB, 0.74390E+00_JPRB, 0.76069E+00_JPRB ,&
&  0.75586E+00_JPRB, 0.74746E+00_JPRB, 0.75051E+00_JPRB, 0.74012E+00_JPRB, 0.73134E+00_JPRB, 0.73626E+00_JPRB, 0.74525E+00_JPRB ,&
&  0.75533E+00_JPRB, 0.74658E+00_JPRB, 0.72915E+00_JPRB, 0.70240E+00_JPRB, 0.68967E+00_JPRB, 0.68687E+00_JPRB, 0.69450E+00_JPRB ,&
&  0.70016E+00_JPRB, 0.72729E+00_JPRB, 0.66316E+00_JPRB, 0.71974E+00_JPRB, 0.72633E+00_JPRB, 0.72281E+00_JPRB, 0.72058E+00_JPRB ,&
&  0.72490E+00_JPRB, 0.71448E+00_JPRB, 0.70613E+00_JPRB, 0.69836E+00_JPRB, 0.67612E+00_JPRB, 0.65832E+00_JPRB, 0.63951E+00_JPRB ,&
&  0.62295E+00_JPRB, 0.61283E+00_JPRB, 0.61834E+00_JPRB, 0.63622E+00_JPRB, 0.65441E+00_JPRB, 0.66450E+00_JPRB, 0.67062E+00_JPRB ,&
&  0.67388E+00_JPRB, 0.66644E+00_JPRB, 0.65612E+00_JPRB, 0.65575E+00_JPRB, 0.66170E+00_JPRB, 0.66618E+00_JPRB, 0.66890E+00_JPRB ,&
&  0.66891E+00_JPRB, 0.64570E+00_JPRB, 0.63250E+00_JPRB, 0.64866E+00_JPRB, 0.66660E+00_JPRB, 0.68663E+00_JPRB, 0.71283E+00_JPRB ,&
&  0.73301E+00_JPRB, 0.74114E+00_JPRB, 0.74100E+00_JPRB, 0.74799E+00_JPRB, 0.75491E+00_JPRB, 0.76360E+00_JPRB, 0.77027E+00_JPRB ,&
&  0.75933E+00_JPRB, 0.75639E+00_JPRB, 0.75684E+00_JPRB, 0.75634E+00_JPRB, 0.74685E+00_JPRB, 0.72348E+00_JPRB, 0.69101E+00_JPRB ,&
&  0.67597E+00_JPRB, 0.66079E+00_JPRB, 0.64965E+00_JPRB, 0.64638E+00_JPRB, 0.64822E+00_JPRB, 0.67473E+00_JPRB, 0.71269E+00_JPRB ,&
&  0.75060E+00_JPRB, 0.78189E+00_JPRB, 0.80909E+00_JPRB, 0.83389E+00_JPRB, 0.83336E+00_JPRB, 0.81355E+00_JPRB, 0.80614E+00_JPRB ,&
&  0.79285E+00_JPRB, 0.77766E+00_JPRB, 0.76613E+00_JPRB, 0.75624E+00_JPRB, 0.74511E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.74488E+00_JPRB, 0.74569E+00_JPRB, 0.72966E+00_JPRB, 0.71765E+00_JPRB, 0.70015E+00_JPRB, 0.67798E+00_JPRB, 0.65859E+00_JPRB ,&
&  0.66015E+00_JPRB, 0.67555E+00_JPRB, 0.69041E+00_JPRB, 0.70110E+00_JPRB, 0.71661E+00_JPRB, 0.71976E+00_JPRB, 0.72611E+00_JPRB ,&
&  0.72176E+00_JPRB, 0.68462E+00_JPRB, 0.67871E+00_JPRB, 0.68626E+00_JPRB, 0.68806E+00_JPRB, 0.69409E+00_JPRB, 0.70499E+00_JPRB ,&
&  0.71495E+00_JPRB, 0.71025E+00_JPRB, 0.70200E+00_JPRB, 0.67221E+00_JPRB, 0.66533E+00_JPRB, 0.67011E+00_JPRB, 0.67506E+00_JPRB ,&
&  0.68531E+00_JPRB, 0.70283E+00_JPRB, 0.70584E+00_JPRB, 0.71449E+00_JPRB, 0.72242E+00_JPRB, 0.72577E+00_JPRB, 0.72193E+00_JPRB ,&
&  0.69590E+00_JPRB, 0.66999E+00_JPRB, 0.65945E+00_JPRB, 0.64269E+00_JPRB, 0.62080E+00_JPRB, 0.60541E+00_JPRB, 0.59799E+00_JPRB ,&
&  0.59425E+00_JPRB, 0.59959E+00_JPRB, 0.60599E+00_JPRB, 0.62008E+00_JPRB, 0.62533E+00_JPRB, 0.62951E+00_JPRB, 0.63491E+00_JPRB ,&
&  0.63458E+00_JPRB, 0.63050E+00_JPRB, 0.63047E+00_JPRB, 0.64012E+00_JPRB, 0.64577E+00_JPRB, 0.64313E+00_JPRB, 0.63836E+00_JPRB ,&
&  0.62496E+00_JPRB, 0.61075E+00_JPRB, 0.61784E+00_JPRB, 0.63303E+00_JPRB, 0.64051E+00_JPRB, 0.66664E+00_JPRB, 0.70018E+00_JPRB ,&
&  0.72866E+00_JPRB, 0.74720E+00_JPRB, 0.74714E+00_JPRB, 0.74349E+00_JPRB, 0.74183E+00_JPRB, 0.74911E+00_JPRB, 0.76118E+00_JPRB ,&
&  0.74515E+00_JPRB, 0.73575E+00_JPRB, 0.73351E+00_JPRB, 0.71399E+00_JPRB, 0.67740E+00_JPRB, 0.64595E+00_JPRB, 0.62276E+00_JPRB ,&
&  0.60423E+00_JPRB, 0.59380E+00_JPRB, 0.59759E+00_JPRB, 0.60091E+00_JPRB, 0.61355E+00_JPRB, 0.64935E+00_JPRB, 0.68701E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.74965E+00_JPRB, 0.75700E+00_JPRB, 0.76590E+00_JPRB, 0.77283E+00_JPRB, 0.76321E+00_JPRB, 0.75518E+00_JPRB ,&
&  0.75319E+00_JPRB, 0.74480E+00_JPRB, 0.73544E+00_JPRB, 0.72547E+00_JPRB, 0.73241E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.73814E+00_JPRB, 0.72992E+00_JPRB, 0.71481E+00_JPRB, 0.68718E+00_JPRB, 0.66912E+00_JPRB, 0.64187E+00_JPRB, 0.62666E+00_JPRB ,&
&  0.62909E+00_JPRB, 0.63065E+00_JPRB, 0.63601E+00_JPRB, 0.64533E+00_JPRB, 0.64331E+00_JPRB, 0.63863E+00_JPRB, 0.63384E+00_JPRB ,&
&  0.63663E+00_JPRB, 0.63695E+00_JPRB, 0.64540E+00_JPRB, 0.66086E+00_JPRB, 0.67713E+00_JPRB, 0.68642E+00_JPRB, 0.69633E+00_JPRB ,&
&  0.70394E+00_JPRB, 0.69778E+00_JPRB, 0.69329E+00_JPRB, 0.66259E+00_JPRB, 0.64470E+00_JPRB, 0.64969E+00_JPRB, 0.65707E+00_JPRB ,&
&  0.66712E+00_JPRB, 0.67783E+00_JPRB, 0.68732E+00_JPRB, 0.69341E+00_JPRB, 0.67941E+00_JPRB, 0.67535E+00_JPRB, 0.66440E+00_JPRB ,&
&  0.62147E+00_JPRB, 0.57954E+00_JPRB, 0.55698E+00_JPRB, 0.54942E+00_JPRB, 0.54413E+00_JPRB, 0.55009E+00_JPRB, 0.55784E+00_JPRB ,&
&  0.57136E+00_JPRB, 0.58364E+00_JPRB, 0.58677E+00_JPRB, 0.58988E+00_JPRB, 0.59145E+00_JPRB, 0.59068E+00_JPRB, 0.59134E+00_JPRB ,&
&  0.58886E+00_JPRB, 0.58750E+00_JPRB, 0.59476E+00_JPRB, 0.59520E+00_JPRB, 0.58609E+00_JPRB, 0.58313E+00_JPRB, 0.58064E+00_JPRB ,&
&  0.57770E+00_JPRB, 0.57493E+00_JPRB, 0.57170E+00_JPRB, 0.58195E+00_JPRB, 0.60796E+00_JPRB, 0.63773E+00_JPRB, 0.66333E+00_JPRB ,&
&  0.67804E+00_JPRB, 0.69007E+00_JPRB, 0.69326E+00_JPRB, 0.69517E+00_JPRB, 0.69462E+00_JPRB, 0.68584E+00_JPRB, 0.68246E+00_JPRB ,&
&  0.67458E+00_JPRB, 0.65528E+00_JPRB, 0.62953E+00_JPRB, 0.58960E+00_JPRB, 0.55091E+00_JPRB, 0.52883E+00_JPRB, 0.52841E+00_JPRB ,&
&  0.54060E+00_JPRB, 0.55311E+00_JPRB, 0.55953E+00_JPRB, 0.56758E+00_JPRB, 0.58229E+00_JPRB, 0.59999E+00_JPRB, 0.61873E+00_JPRB ,&
&  0.63067E+00_JPRB, 0.63108E+00_JPRB, 0.62908E+00_JPRB, 0.63772E+00_JPRB, 0.64937E+00_JPRB, 0.66802E+00_JPRB, 0.68827E+00_JPRB ,&
&  0.70594E+00_JPRB, 0.70579E+00_JPRB, 0.70641E+00_JPRB, 0.70794E+00_JPRB, 0.72778E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.73066E+00_JPRB, 0.72835E+00_JPRB, 0.69858E+00_JPRB, 0.66620E+00_JPRB, 0.64712E+00_JPRB, 0.62629E+00_JPRB, 0.60969E+00_JPRB ,&
&  0.60833E+00_JPRB, 0.60570E+00_JPRB, 0.60358E+00_JPRB, 0.60090E+00_JPRB, 0.59908E+00_JPRB, 0.59593E+00_JPRB, 0.58377E+00_JPRB ,&
&  0.57919E+00_JPRB, 0.58559E+00_JPRB, 0.60790E+00_JPRB, 0.63300E+00_JPRB, 0.65319E+00_JPRB, 0.67949E+00_JPRB, 0.69075E+00_JPRB ,&
&  0.70266E+00_JPRB, 0.71519E+00_JPRB, 0.71730E+00_JPRB, 0.68419E+00_JPRB, 0.65693E+00_JPRB, 0.65788E+00_JPRB, 0.66896E+00_JPRB ,&
&  0.66930E+00_JPRB, 0.67248E+00_JPRB, 0.67554E+00_JPRB, 0.67217E+00_JPRB, 0.65145E+00_JPRB, 0.61322E+00_JPRB, 0.58300E+00_JPRB ,&
&  0.55797E+00_JPRB, 0.53855E+00_JPRB, 0.53247E+00_JPRB, 0.52079E+00_JPRB, 0.51881E+00_JPRB, 0.52888E+00_JPRB, 0.54248E+00_JPRB ,&
&  0.55666E+00_JPRB, 0.56322E+00_JPRB, 0.57030E+00_JPRB, 0.56670E+00_JPRB, 0.56240E+00_JPRB, 0.56528E+00_JPRB, 0.56527E+00_JPRB ,&
&  0.56484E+00_JPRB, 0.55579E+00_JPRB, 0.55291E+00_JPRB, 0.54574E+00_JPRB, 0.54154E+00_JPRB, 0.54157E+00_JPRB, 0.54405E+00_JPRB ,&
&  0.54492E+00_JPRB, 0.55022E+00_JPRB, 0.55181E+00_JPRB, 0.55408E+00_JPRB, 0.56372E+00_JPRB, 0.58054E+00_JPRB, 0.59734E+00_JPRB ,&
&  0.60986E+00_JPRB, 0.61964E+00_JPRB, 0.62730E+00_JPRB, 0.63310E+00_JPRB, 0.63294E+00_JPRB, 0.62818E+00_JPRB, 0.61422E+00_JPRB ,&
&  0.58361E+00_JPRB, 0.55083E+00_JPRB, 0.52596E+00_JPRB, 0.50613E+00_JPRB, 0.50256E+00_JPRB, 0.51371E+00_JPRB, 0.53170E+00_JPRB ,&
&  0.54526E+00_JPRB, 0.54921E+00_JPRB, 0.54310E+00_JPRB, 0.54119E+00_JPRB, 0.54594E+00_JPRB, 0.55535E+00_JPRB, 0.56265E+00_JPRB ,&
&  0.56187E+00_JPRB, 0.55887E+00_JPRB, 0.56303E+00_JPRB, 0.57050E+00_JPRB, 0.58513E+00_JPRB, 0.60354E+00_JPRB, 0.62326E+00_JPRB ,&
&  0.64286E+00_JPRB, 0.65807E+00_JPRB, 0.67778E+00_JPRB, 0.69765E+00_JPRB, 0.72292E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.71108E+00_JPRB, 0.68273E+00_JPRB, 0.65295E+00_JPRB, 0.62773E+00_JPRB, 0.61564E+00_JPRB, 0.59810E+00_JPRB, 0.58347E+00_JPRB ,&
&  0.58266E+00_JPRB, 0.59051E+00_JPRB, 0.58582E+00_JPRB, 0.57982E+00_JPRB, 0.57049E+00_JPRB, 0.56283E+00_JPRB, 0.55373E+00_JPRB ,&
&  0.54974E+00_JPRB, 0.56117E+00_JPRB, 0.58676E+00_JPRB, 0.59953E+00_JPRB, 0.60571E+00_JPRB, 0.63621E+00_JPRB, 0.66761E+00_JPRB ,&
&  0.67692E+00_JPRB, 0.69092E+00_JPRB, 0.69498E+00_JPRB, 0.66949E+00_JPRB, 0.65276E+00_JPRB, 0.64618E+00_JPRB, 0.64989E+00_JPRB ,&
&  0.64516E+00_JPRB, 0.64933E+00_JPRB, 0.64096E+00_JPRB, 0.57141E+00_JPRB, 0.57043E+00_JPRB, 0.53985E+00_JPRB, 0.52926E+00_JPRB ,&
&  0.52394E+00_JPRB, 0.52920E+00_JPRB, 0.52524E+00_JPRB, 0.51587E+00_JPRB, 0.51212E+00_JPRB, 0.51540E+00_JPRB, 0.51702E+00_JPRB ,&
&  0.51296E+00_JPRB, 0.50261E+00_JPRB, 0.50721E+00_JPRB, 0.51363E+00_JPRB, 0.51735E+00_JPRB, 0.52456E+00_JPRB, 0.53519E+00_JPRB ,&
&  0.52509E+00_JPRB, 0.51581E+00_JPRB, 0.50709E+00_JPRB, 0.50739E+00_JPRB, 0.51382E+00_JPRB, 0.52784E+00_JPRB, 0.53231E+00_JPRB ,&
&  0.52996E+00_JPRB, 0.52859E+00_JPRB, 0.52962E+00_JPRB, 0.53609E+00_JPRB, 0.55101E+00_JPRB, 0.56569E+00_JPRB, 0.56688E+00_JPRB ,&
&  0.56796E+00_JPRB, 0.57248E+00_JPRB, 0.57752E+00_JPRB, 0.57922E+00_JPRB, 0.57512E+00_JPRB, 0.55992E+00_JPRB, 0.54244E+00_JPRB ,&
&  0.52592E+00_JPRB, 0.50884E+00_JPRB, 0.49462E+00_JPRB, 0.49032E+00_JPRB, 0.50383E+00_JPRB, 0.53344E+00_JPRB, 0.55011E+00_JPRB ,&
&  0.55421E+00_JPRB, 0.55635E+00_JPRB, 0.55969E+00_JPRB, 0.56064E+00_JPRB, 0.56257E+00_JPRB, 0.56048E+00_JPRB, 0.55375E+00_JPRB ,&
&  0.54233E+00_JPRB, 0.53129E+00_JPRB, 0.52901E+00_JPRB, 0.53398E+00_JPRB, 0.53978E+00_JPRB, 0.55248E+00_JPRB, 0.56956E+00_JPRB ,&
&  0.59194E+00_JPRB, 0.63047E+00_JPRB, 0.66297E+00_JPRB, 0.68805E+00_JPRB, 0.70583E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.68686E+00_JPRB, 0.66032E+00_JPRB, 0.64259E+00_JPRB, 0.62886E+00_JPRB, 0.61047E+00_JPRB, 0.59651E+00_JPRB, 0.59089E+00_JPRB ,&
&  0.59907E+00_JPRB, 0.59770E+00_JPRB, 0.58797E+00_JPRB, 0.57984E+00_JPRB, 0.56400E+00_JPRB, 0.54983E+00_JPRB, 0.54405E+00_JPRB ,&
&  0.55258E+00_JPRB, 0.56877E+00_JPRB, 0.58863E+00_JPRB, 0.59671E+00_JPRB, 0.59596E+00_JPRB, 0.60437E+00_JPRB, 0.62364E+00_JPRB ,&
&  0.65224E+00_JPRB, 0.67219E+00_JPRB, 0.67909E+00_JPRB, 0.66567E+00_JPRB, 0.64785E+00_JPRB, 0.63948E+00_JPRB, 0.62507E+00_JPRB ,&
&  0.60833E+00_JPRB, 0.61149E+00_JPRB, 0.54407E+00_JPRB, 0.53642E+00_JPRB, 0.51320E+00_JPRB, 0.49864E+00_JPRB, 0.49485E+00_JPRB ,&
&  0.50804E+00_JPRB, 0.51203E+00_JPRB, 0.50358E+00_JPRB, 0.49133E+00_JPRB, 0.48303E+00_JPRB, 0.47197E+00_JPRB, 0.46176E+00_JPRB ,&
&  0.45484E+00_JPRB, 0.45756E+00_JPRB, 0.46661E+00_JPRB, 0.47599E+00_JPRB, 0.48701E+00_JPRB, 0.49759E+00_JPRB, 0.49415E+00_JPRB ,&
&  0.48245E+00_JPRB, 0.48040E+00_JPRB, 0.48750E+00_JPRB, 0.50222E+00_JPRB, 0.51819E+00_JPRB, 0.52553E+00_JPRB, 0.52895E+00_JPRB ,&
&  0.52067E+00_JPRB, 0.50356E+00_JPRB, 0.48897E+00_JPRB, 0.47701E+00_JPRB, 0.45071E+00_JPRB, 0.38776E+00_JPRB, 0.36521E+00_JPRB ,&
&  0.37635E+00_JPRB, 0.43473E+00_JPRB, 0.50400E+00_JPRB, 0.49569E+00_JPRB, 0.49192E+00_JPRB, 0.47774E+00_JPRB, 0.44124E+00_JPRB ,&
&  0.39874E+00_JPRB, 0.33764E+00_JPRB, 0.31683E+00_JPRB, 0.31348E+00_JPRB, 0.31287E+00_JPRB, 0.31912E+00_JPRB, 0.33162E+00_JPRB ,&
&  0.34798E+00_JPRB, 0.37447E+00_JPRB, 0.39161E+00_JPRB, 0.39810E+00_JPRB, 0.39641E+00_JPRB, 0.38035E+00_JPRB, 0.36641E+00_JPRB ,&
&  0.35732E+00_JPRB, 0.38255E+00_JPRB, 0.44172E+00_JPRB, 0.52430E+00_JPRB, 0.52424E+00_JPRB, 0.54975E+00_JPRB, 0.57505E+00_JPRB ,&
&  0.59920E+00_JPRB, 0.63936E+00_JPRB, 0.66547E+00_JPRB, 0.67103E+00_JPRB, 0.68909E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.64653E+00_JPRB, 0.63400E+00_JPRB, 0.62551E+00_JPRB, 0.61956E+00_JPRB, 0.61258E+00_JPRB, 0.61121E+00_JPRB, 0.61047E+00_JPRB ,&
&  0.60468E+00_JPRB, 0.59632E+00_JPRB, 0.59734E+00_JPRB, 0.58027E+00_JPRB, 0.55583E+00_JPRB, 0.53948E+00_JPRB, 0.53733E+00_JPRB ,&
&  0.54132E+00_JPRB, 0.54854E+00_JPRB, 0.55765E+00_JPRB, 0.56628E+00_JPRB, 0.57116E+00_JPRB, 0.58285E+00_JPRB, 0.60372E+00_JPRB ,&
&  0.62803E+00_JPRB, 0.64170E+00_JPRB, 0.64582E+00_JPRB, 0.63762E+00_JPRB, 0.62508E+00_JPRB, 0.61217E+00_JPRB, 0.60108E+00_JPRB ,&
&  0.59187E+00_JPRB, 0.48986E+00_JPRB, 0.42206E+00_JPRB, 0.42885E+00_JPRB, 0.49739E+00_JPRB, 0.48114E+00_JPRB, 0.47946E+00_JPRB ,&
&  0.49207E+00_JPRB, 0.49202E+00_JPRB, 0.47873E+00_JPRB, 0.46168E+00_JPRB, 0.44579E+00_JPRB, 0.43429E+00_JPRB, 0.42601E+00_JPRB ,&
&  0.42232E+00_JPRB, 0.42921E+00_JPRB, 0.43733E+00_JPRB, 0.44980E+00_JPRB, 0.45287E+00_JPRB, 0.44007E+00_JPRB, 0.42464E+00_JPRB ,&
&  0.40983E+00_JPRB, 0.40447E+00_JPRB, 0.40687E+00_JPRB, 0.41350E+00_JPRB, 0.42225E+00_JPRB, 0.42902E+00_JPRB, 0.42670E+00_JPRB ,&
&  0.39817E+00_JPRB, 0.37508E+00_JPRB, 0.34692E+00_JPRB, 0.32041E+00_JPRB, 0.29745E+00_JPRB, 0.28209E+00_JPRB, 0.27408E+00_JPRB ,&
&  0.27210E+00_JPRB, 0.27959E+00_JPRB, 0.30109E+00_JPRB, 0.34990E+00_JPRB, 0.41578E+00_JPRB, 0.33690E+00_JPRB, 0.29812E+00_JPRB ,&
&  0.27689E+00_JPRB, 0.26392E+00_JPRB, 0.25902E+00_JPRB, 0.25994E+00_JPRB, 0.26424E+00_JPRB, 0.27044E+00_JPRB, 0.27532E+00_JPRB ,&
&  0.27924E+00_JPRB, 0.28266E+00_JPRB, 0.28441E+00_JPRB, 0.28521E+00_JPRB, 0.28672E+00_JPRB, 0.28789E+00_JPRB, 0.28682E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.29193E+00_JPRB, 0.30979E+00_JPRB, 0.33784E+00_JPRB, 0.37059E+00_JPRB, 0.40627E+00_JPRB, 0.45865E+00_JPRB ,&
&  0.52426E+00_JPRB, 0.59868E+00_JPRB, 0.61710E+00_JPRB, 0.63024E+00_JPRB, 0.64055E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.58818E+00_JPRB, 0.59160E+00_JPRB, 0.59547E+00_JPRB, 0.59757E+00_JPRB, 0.59198E+00_JPRB, 0.58457E+00_JPRB, 0.57990E+00_JPRB ,&
&  0.58510E+00_JPRB, 0.58522E+00_JPRB, 0.56202E+00_JPRB, 0.54395E+00_JPRB, 0.53362E+00_JPRB, 0.53008E+00_JPRB, 0.52716E+00_JPRB ,&
&  0.52107E+00_JPRB, 0.51896E+00_JPRB, 0.52015E+00_JPRB, 0.53135E+00_JPRB, 0.54059E+00_JPRB, 0.54866E+00_JPRB, 0.55599E+00_JPRB ,&
&  0.55130E+00_JPRB, 0.56381E+00_JPRB, 0.58531E+00_JPRB, 0.58437E+00_JPRB, 0.58148E+00_JPRB, 0.57413E+00_JPRB, 0.55951E+00_JPRB ,&
&  0.51104E+00_JPRB, 0.44671E+00_JPRB, 0.38562E+00_JPRB, 0.39754E+00_JPRB, 0.47417E+00_JPRB, 0.48287E+00_JPRB, 0.46784E+00_JPRB ,&
&  0.47842E+00_JPRB, 0.47398E+00_JPRB, 0.45603E+00_JPRB, 0.44277E+00_JPRB, 0.42730E+00_JPRB, 0.41728E+00_JPRB, 0.41020E+00_JPRB ,&
&  0.40736E+00_JPRB, 0.40644E+00_JPRB, 0.39137E+00_JPRB, 0.34260E+00_JPRB, 0.31458E+00_JPRB, 0.29738E+00_JPRB, 0.27958E+00_JPRB ,&
&  0.26551E+00_JPRB, 0.25689E+00_JPRB, 0.25560E+00_JPRB, 0.25587E+00_JPRB, 0.26072E+00_JPRB, 0.26304E+00_JPRB, 0.26516E+00_JPRB ,&
&  0.26510E+00_JPRB, 0.26395E+00_JPRB, 0.25994E+00_JPRB, 0.25545E+00_JPRB, 0.25232E+00_JPRB, 0.25186E+00_JPRB, 0.25157E+00_JPRB ,&
&  0.25421E+00_JPRB, 0.25808E+00_JPRB, 0.27452E+00_JPRB, 0.29201E+00_JPRB, 0.29036E+00_JPRB, 0.27531E+00_JPRB, 0.26089E+00_JPRB ,&
&  0.25099E+00_JPRB, 0.24595E+00_JPRB, 0.24280E+00_JPRB, 0.24230E+00_JPRB, 0.24528E+00_JPRB, 0.24952E+00_JPRB, 0.25060E+00_JPRB ,&
&  0.25041E+00_JPRB, 0.25108E+00_JPRB, 0.25116E+00_JPRB, 0.25161E+00_JPRB, 0.25229E+00_JPRB, 0.25494E+00_JPRB, 0.25987E+00_JPRB ,&
&  0.26571E+00_JPRB, 0.27619E+00_JPRB, 0.28805E+00_JPRB, 0.30384E+00_JPRB, 0.31533E+00_JPRB, 0.32392E+00_JPRB, 0.33534E+00_JPRB ,&
&  0.35316E+00_JPRB, 0.39197E+00_JPRB, 0.46109E+00_JPRB, 0.58627E+00_JPRB, 0.58045E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.55607E+00_JPRB, 0.56719E+00_JPRB, 0.57941E+00_JPRB, 0.57110E+00_JPRB, 0.55748E+00_JPRB, 0.54981E+00_JPRB, 0.54965E+00_JPRB ,&
&  0.54945E+00_JPRB, 0.54676E+00_JPRB, 0.53479E+00_JPRB, 0.52204E+00_JPRB, 0.50404E+00_JPRB, 0.45611E+00_JPRB, 0.43821E+00_JPRB ,&
&  0.42080E+00_JPRB, 0.42341E+00_JPRB, 0.42786E+00_JPRB, 0.43433E+00_JPRB, 0.44821E+00_JPRB, 0.45901E+00_JPRB, 0.46093E+00_JPRB ,&
&  0.45087E+00_JPRB, 0.43252E+00_JPRB, 0.41486E+00_JPRB, 0.40112E+00_JPRB, 0.40066E+00_JPRB, 0.40054E+00_JPRB, 0.39339E+00_JPRB ,&
&  0.37972E+00_JPRB, 0.37357E+00_JPRB, 0.37323E+00_JPRB, 0.39315E+00_JPRB, 0.44523E+00_JPRB, 0.46454E+00_JPRB, 0.43964E+00_JPRB ,&
&  0.44400E+00_JPRB, 0.43895E+00_JPRB, 0.43487E+00_JPRB, 0.43614E+00_JPRB, 0.42596E+00_JPRB, 0.40627E+00_JPRB, 0.37241E+00_JPRB ,&
&  0.32927E+00_JPRB, 0.30696E+00_JPRB, 0.28607E+00_JPRB, 0.26626E+00_JPRB, 0.25085E+00_JPRB, 0.24190E+00_JPRB, 0.23436E+00_JPRB ,&
&  0.22946E+00_JPRB, 0.22528E+00_JPRB, 0.22257E+00_JPRB, 0.22255E+00_JPRB, 0.22415E+00_JPRB, 0.22491E+00_JPRB, 0.22585E+00_JPRB ,&
&  0.22729E+00_JPRB, 0.22807E+00_JPRB, 0.22850E+00_JPRB, 0.22976E+00_JPRB, 0.23123E+00_JPRB, 0.23390E+00_JPRB, 0.23758E+00_JPRB ,&
&  0.24212E+00_JPRB, 0.24581E+00_JPRB, 0.24936E+00_JPRB, 0.25556E+00_JPRB, 0.25618E+00_JPRB, 0.25498E+00_JPRB, 0.24714E+00_JPRB ,&
&  0.24144E+00_JPRB, 0.23892E+00_JPRB, 0.23631E+00_JPRB, 0.23560E+00_JPRB, 0.23672E+00_JPRB, 0.23865E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24194E+00_JPRB, 0.24198E+00_JPRB, 0.24116E+00_JPRB, 0.24107E+00_JPRB, 0.24168E+00_JPRB, 0.24183E+00_JPRB, 0.24543E+00_JPRB ,&
&  0.25340E+00_JPRB, 0.26510E+00_JPRB, 0.27800E+00_JPRB, 0.29566E+00_JPRB, 0.31227E+00_JPRB, 0.32901E+00_JPRB, 0.35680E+00_JPRB ,&
&  0.39622E+00_JPRB, 0.50349E+00_JPRB, 0.57233E+00_JPRB, 0.51152E+00_JPRB, 0.53580E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.52851E+00_JPRB, 0.53594E+00_JPRB, 0.54683E+00_JPRB, 0.55135E+00_JPRB, 0.55943E+00_JPRB, 0.54774E+00_JPRB, 0.54068E+00_JPRB ,&
&  0.51050E+00_JPRB, 0.48857E+00_JPRB, 0.45770E+00_JPRB, 0.41918E+00_JPRB, 0.39335E+00_JPRB, 0.37346E+00_JPRB, 0.36652E+00_JPRB ,&
&  0.35828E+00_JPRB, 0.35313E+00_JPRB, 0.34784E+00_JPRB, 0.34564E+00_JPRB, 0.34412E+00_JPRB, 0.34503E+00_JPRB, 0.34464E+00_JPRB ,&
&  0.34093E+00_JPRB, 0.33526E+00_JPRB, 0.32856E+00_JPRB, 0.33130E+00_JPRB, 0.34405E+00_JPRB, 0.35879E+00_JPRB, 0.37619E+00_JPRB ,&
&  0.39632E+00_JPRB, 0.40691E+00_JPRB, 0.41705E+00_JPRB, 0.42113E+00_JPRB, 0.40569E+00_JPRB, 0.40829E+00_JPRB, 0.41134E+00_JPRB ,&
&  0.41929E+00_JPRB, 0.42581E+00_JPRB, 0.42271E+00_JPRB, 0.41805E+00_JPRB, 0.36709E+00_JPRB, 0.33429E+00_JPRB, 0.30576E+00_JPRB ,&
&  0.28446E+00_JPRB, 0.27089E+00_JPRB, 0.25940E+00_JPRB, 0.24962E+00_JPRB, 0.23951E+00_JPRB, 0.23305E+00_JPRB, 0.22823E+00_JPRB ,&
&  0.22437E+00_JPRB, 0.22274E+00_JPRB, 0.22121E+00_JPRB, 0.22076E+00_JPRB, 0.22119E+00_JPRB, 0.22056E+00_JPRB, 0.22049E+00_JPRB ,&
&  0.22028E+00_JPRB, 0.21960E+00_JPRB, 0.21969E+00_JPRB, 0.22095E+00_JPRB, 0.22298E+00_JPRB, 0.22627E+00_JPRB, 0.22733E+00_JPRB ,&
&  0.22828E+00_JPRB, 0.22895E+00_JPRB, 0.22904E+00_JPRB, 0.23104E+00_JPRB, 0.23214E+00_JPRB, 0.23232E+00_JPRB, 0.23076E+00_JPRB ,&
&  0.22910E+00_JPRB, 0.22784E+00_JPRB, 0.22778E+00_JPRB, 0.22832E+00_JPRB, 0.22984E+00_JPRB, 0.23214E+00_JPRB, 0.23261E+00_JPRB ,&
&  0.23346E+00_JPRB, 0.23300E+00_JPRB, 0.23256E+00_JPRB, 0.23452E+00_JPRB, 0.23679E+00_JPRB, 0.23927E+00_JPRB, 0.24258E+00_JPRB ,&
&  0.24428E+00_JPRB, 0.25142E+00_JPRB, 0.26444E+00_JPRB, 0.28464E+00_JPRB, 0.31644E+00_JPRB, 0.33322E+00_JPRB, 0.37290E+00_JPRB ,&
&  0.43395E+00_JPRB, 0.57520E+00_JPRB, 0.63905E+00_JPRB, 0.49760E+00_JPRB, 0.51310E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.47096E+00_JPRB, 0.48319E+00_JPRB, 0.50015E+00_JPRB, 0.51066E+00_JPRB, 0.52461E+00_JPRB, 0.52714E+00_JPRB, 0.52129E+00_JPRB ,&
&  0.50273E+00_JPRB, 0.47372E+00_JPRB, 0.45727E+00_JPRB, 0.43654E+00_JPRB, 0.41530E+00_JPRB, 0.39378E+00_JPRB, 0.38422E+00_JPRB ,&
&  0.37276E+00_JPRB, 0.35964E+00_JPRB, 0.34626E+00_JPRB, 0.33465E+00_JPRB, 0.32345E+00_JPRB, 0.31313E+00_JPRB, 0.30715E+00_JPRB ,&
&  0.30103E+00_JPRB, 0.30051E+00_JPRB, 0.29985E+00_JPRB, 0.30390E+00_JPRB, 0.31510E+00_JPRB, 0.33379E+00_JPRB, 0.35955E+00_JPRB ,&
&  0.39817E+00_JPRB, 0.40336E+00_JPRB, 0.38860E+00_JPRB, 0.38229E+00_JPRB, 0.37486E+00_JPRB, 0.36850E+00_JPRB, 0.35576E+00_JPRB ,&
&  0.36316E+00_JPRB, 0.37464E+00_JPRB, 0.39005E+00_JPRB, 0.37841E+00_JPRB, 0.33278E+00_JPRB, 0.31280E+00_JPRB, 0.28928E+00_JPRB ,&
&  0.27747E+00_JPRB, 0.26751E+00_JPRB, 0.25965E+00_JPRB, 0.25198E+00_JPRB, 0.24362E+00_JPRB, 0.23862E+00_JPRB, 0.23308E+00_JPRB ,&
&  0.22929E+00_JPRB, 0.22786E+00_JPRB, 0.22553E+00_JPRB, 0.22567E+00_JPRB, 0.22561E+00_JPRB, 0.22488E+00_JPRB, 0.22424E+00_JPRB ,&
&  0.22307E+00_JPRB, 0.22183E+00_JPRB, 0.22133E+00_JPRB, 0.22195E+00_JPRB, 0.22286E+00_JPRB, 0.22372E+00_JPRB, 0.22164E+00_JPRB ,&
&  0.22037E+00_JPRB, 0.21912E+00_JPRB, 0.21903E+00_JPRB, 0.22042E+00_JPRB, 0.22092E+00_JPRB, 0.22186E+00_JPRB, 0.22209E+00_JPRB ,&
&  0.22226E+00_JPRB, 0.22287E+00_JPRB, 0.22251E+00_JPRB, 0.22323E+00_JPRB, 0.22349E+00_JPRB, 0.22462E+00_JPRB, 0.22539E+00_JPRB ,&
&  0.22623E+00_JPRB, 0.22834E+00_JPRB, 0.23054E+00_JPRB, 0.23372E+00_JPRB, 0.23801E+00_JPRB, 0.24073E+00_JPRB, 0.24521E+00_JPRB ,&
&  0.25051E+00_JPRB, 0.25948E+00_JPRB, 0.27670E+00_JPRB, 0.29599E+00_JPRB, 0.32004E+00_JPRB, 0.33147E+00_JPRB, 0.33953E+00_JPRB ,&
&  0.35203E+00_JPRB, 0.38445E+00_JPRB, 0.43481E+00_JPRB, 0.45743E+00_JPRB, 0.46342E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.46743E+00_JPRB, 0.46908E+00_JPRB, 0.47013E+00_JPRB, 0.46700E+00_JPRB, 0.46615E+00_JPRB, 0.46620E+00_JPRB, 0.46349E+00_JPRB ,&
&  0.45551E+00_JPRB, 0.45325E+00_JPRB, 0.44831E+00_JPRB, 0.44096E+00_JPRB, 0.43368E+00_JPRB, 0.42171E+00_JPRB, 0.40526E+00_JPRB ,&
&  0.38332E+00_JPRB, 0.36389E+00_JPRB, 0.34936E+00_JPRB, 0.33579E+00_JPRB, 0.32252E+00_JPRB, 0.30957E+00_JPRB, 0.30456E+00_JPRB ,&
&  0.30010E+00_JPRB, 0.29824E+00_JPRB, 0.29773E+00_JPRB, 0.30110E+00_JPRB, 0.30857E+00_JPRB, 0.31794E+00_JPRB, 0.32823E+00_JPRB ,&
&  0.33979E+00_JPRB, 0.35061E+00_JPRB, 0.35182E+00_JPRB, 0.34439E+00_JPRB, 0.33740E+00_JPRB, 0.33306E+00_JPRB, 0.32972E+00_JPRB ,&
&  0.32578E+00_JPRB, 0.31758E+00_JPRB, 0.30811E+00_JPRB, 0.29810E+00_JPRB, 0.28800E+00_JPRB, 0.28064E+00_JPRB, 0.27205E+00_JPRB ,&
&  0.26576E+00_JPRB, 0.26004E+00_JPRB, 0.25386E+00_JPRB, 0.24858E+00_JPRB, 0.24498E+00_JPRB, 0.24200E+00_JPRB, 0.23894E+00_JPRB ,&
&  0.23749E+00_JPRB, 0.23604E+00_JPRB, 0.23443E+00_JPRB, 0.23290E+00_JPRB, 0.23094E+00_JPRB, 0.22948E+00_JPRB, 0.22817E+00_JPRB ,&
&  0.22719E+00_JPRB, 0.22632E+00_JPRB, 0.22580E+00_JPRB, 0.22525E+00_JPRB, 0.22480E+00_JPRB, 0.22380E+00_JPRB, 0.22263E+00_JPRB ,&
&  0.22194E+00_JPRB, 0.22177E+00_JPRB, 0.22101E+00_JPRB, 0.22063E+00_JPRB, 0.22032E+00_JPRB, 0.21954E+00_JPRB, 0.22004E+00_JPRB ,&
&  0.22127E+00_JPRB, 0.22264E+00_JPRB, 0.22287E+00_JPRB, 0.22392E+00_JPRB, 0.22518E+00_JPRB, 0.22678E+00_JPRB, 0.22940E+00_JPRB ,&
&  0.23138E+00_JPRB, 0.23345E+00_JPRB, 0.23606E+00_JPRB, 0.23827E+00_JPRB, 0.24176E+00_JPRB, 0.24607E+00_JPRB, 0.25141E+00_JPRB ,&
&  0.25871E+00_JPRB, 0.26756E+00_JPRB, 0.28004E+00_JPRB, 0.29530E+00_JPRB, 0.31750E+00_JPRB, 0.32573E+00_JPRB, 0.33354E+00_JPRB ,&
&  0.33973E+00_JPRB, 0.35157E+00_JPRB, 0.43372E+00_JPRB, 0.44836E+00_JPRB, 0.46320E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.34300E+00_JPRB, 0.35309E+00_JPRB, 0.36145E+00_JPRB, 0.36938E+00_JPRB, 0.36959E+00_JPRB, 0.43407E+00_JPRB, 0.41388E+00_JPRB ,&
&  0.40411E+00_JPRB, 0.37677E+00_JPRB, 0.37828E+00_JPRB, 0.37426E+00_JPRB, 0.36797E+00_JPRB, 0.36102E+00_JPRB, 0.35507E+00_JPRB ,&
&  0.34909E+00_JPRB, 0.34145E+00_JPRB, 0.33677E+00_JPRB, 0.33093E+00_JPRB, 0.32531E+00_JPRB, 0.31966E+00_JPRB, 0.31625E+00_JPRB ,&
&  0.31371E+00_JPRB, 0.31222E+00_JPRB, 0.31110E+00_JPRB, 0.30969E+00_JPRB, 0.31190E+00_JPRB, 0.31295E+00_JPRB, 0.31092E+00_JPRB ,&
&  0.30699E+00_JPRB, 0.30335E+00_JPRB, 0.29890E+00_JPRB, 0.29435E+00_JPRB, 0.29156E+00_JPRB, 0.28900E+00_JPRB, 0.28634E+00_JPRB ,&
&  0.28271E+00_JPRB, 0.27866E+00_JPRB, 0.27450E+00_JPRB, 0.27117E+00_JPRB, 0.26796E+00_JPRB, 0.26464E+00_JPRB, 0.26153E+00_JPRB ,&
&  0.25825E+00_JPRB, 0.25499E+00_JPRB, 0.25172E+00_JPRB, 0.24964E+00_JPRB, 0.24788E+00_JPRB, 0.24626E+00_JPRB, 0.24478E+00_JPRB ,&
&  0.24420E+00_JPRB, 0.24349E+00_JPRB, 0.24259E+00_JPRB, 0.24147E+00_JPRB, 0.24043E+00_JPRB, 0.23930E+00_JPRB, 0.23841E+00_JPRB ,&
&  0.23832E+00_JPRB, 0.23775E+00_JPRB, 0.23716E+00_JPRB, 0.23637E+00_JPRB, 0.23602E+00_JPRB, 0.23520E+00_JPRB, 0.23418E+00_JPRB ,&
&  0.23325E+00_JPRB, 0.23240E+00_JPRB, 0.23223E+00_JPRB, 0.23201E+00_JPRB, 0.23176E+00_JPRB, 0.23157E+00_JPRB, 0.23200E+00_JPRB ,&
&  0.23253E+00_JPRB, 0.23319E+00_JPRB, 0.23427E+00_JPRB, 0.23552E+00_JPRB, 0.23663E+00_JPRB, 0.23772E+00_JPRB, 0.23927E+00_JPRB ,&
&  0.24127E+00_JPRB, 0.24328E+00_JPRB, 0.24553E+00_JPRB, 0.24752E+00_JPRB, 0.25101E+00_JPRB, 0.25477E+00_JPRB, 0.25892E+00_JPRB ,&
&  0.26392E+00_JPRB, 0.27084E+00_JPRB, 0.27887E+00_JPRB, 0.28612E+00_JPRB, 0.29365E+00_JPRB, 0.30151E+00_JPRB, 0.30798E+00_JPRB ,&
&  0.31359E+00_JPRB, 0.31928E+00_JPRB, 0.32463E+00_JPRB, 0.33054E+00_JPRB, 0.33692E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.30077E+00_JPRB, 0.30364E+00_JPRB, 0.30602E+00_JPRB, 0.30769E+00_JPRB, 0.30778E+00_JPRB, 0.30935E+00_JPRB, 0.30959E+00_JPRB ,&
&  0.30961E+00_JPRB, 0.30934E+00_JPRB, 0.31007E+00_JPRB, 0.31086E+00_JPRB, 0.31137E+00_JPRB, 0.31164E+00_JPRB, 0.31167E+00_JPRB ,&
&  0.31114E+00_JPRB, 0.31012E+00_JPRB, 0.30601E+00_JPRB, 0.30372E+00_JPRB, 0.30233E+00_JPRB, 0.30163E+00_JPRB, 0.30124E+00_JPRB ,&
&  0.30038E+00_JPRB, 0.29937E+00_JPRB, 0.29777E+00_JPRB, 0.29758E+00_JPRB, 0.29678E+00_JPRB, 0.29573E+00_JPRB, 0.29422E+00_JPRB ,&
&  0.29272E+00_JPRB, 0.29153E+00_JPRB, 0.29007E+00_JPRB, 0.28858E+00_JPRB, 0.28678E+00_JPRB, 0.28458E+00_JPRB, 0.28179E+00_JPRB ,&
&  0.27892E+00_JPRB, 0.27600E+00_JPRB, 0.27359E+00_JPRB, 0.27136E+00_JPRB, 0.26941E+00_JPRB, 0.26785E+00_JPRB, 0.26671E+00_JPRB ,&
&  0.26544E+00_JPRB, 0.26399E+00_JPRB, 0.26252E+00_JPRB, 0.26129E+00_JPRB, 0.26007E+00_JPRB, 0.25900E+00_JPRB, 0.25846E+00_JPRB ,&
&  0.25815E+00_JPRB, 0.25777E+00_JPRB, 0.25731E+00_JPRB, 0.25689E+00_JPRB, 0.25636E+00_JPRB, 0.25588E+00_JPRB, 0.25539E+00_JPRB ,&
&  0.25497E+00_JPRB, 0.25474E+00_JPRB, 0.25438E+00_JPRB, 0.25400E+00_JPRB, 0.25376E+00_JPRB, 0.25336E+00_JPRB, 0.25291E+00_JPRB ,&
&  0.25240E+00_JPRB, 0.25214E+00_JPRB, 0.25182E+00_JPRB, 0.25143E+00_JPRB, 0.25106E+00_JPRB, 0.25080E+00_JPRB, 0.25082E+00_JPRB ,&
&  0.25079E+00_JPRB, 0.25070E+00_JPRB, 0.25076E+00_JPRB, 0.25153E+00_JPRB, 0.25234E+00_JPRB, 0.25332E+00_JPRB, 0.25441E+00_JPRB ,&
&  0.25583E+00_JPRB, 0.25732E+00_JPRB, 0.25900E+00_JPRB, 0.26060E+00_JPRB, 0.26274E+00_JPRB, 0.26487E+00_JPRB, 0.26699E+00_JPRB ,&
&  0.26902E+00_JPRB, 0.27112E+00_JPRB, 0.27324E+00_JPRB, 0.27540E+00_JPRB, 0.27826E+00_JPRB, 0.28065E+00_JPRB, 0.28334E+00_JPRB ,&
&  0.28591E+00_JPRB, 0.28879E+00_JPRB, 0.29171E+00_JPRB, 0.29429E+00_JPRB, 0.29689E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB, 0.27405E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB, 0.70319E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.71830E+00_JPRB, 0.71758E+00_JPRB, 0.71699E+00_JPRB, 0.71673E+00_JPRB, 0.71586E+00_JPRB, 0.71453E+00_JPRB, 0.71273E+00_JPRB ,&
&  0.71068E+00_JPRB, 0.70849E+00_JPRB, 0.70552E+00_JPRB, 0.70252E+00_JPRB, 0.69959E+00_JPRB, 0.69673E+00_JPRB, 0.69331E+00_JPRB ,&
&  0.68972E+00_JPRB, 0.68567E+00_JPRB, 0.68152E+00_JPRB, 0.67723E+00_JPRB, 0.67382E+00_JPRB, 0.67095E+00_JPRB, 0.66895E+00_JPRB ,&
&  0.66714E+00_JPRB, 0.66539E+00_JPRB, 0.66363E+00_JPRB, 0.66240E+00_JPRB, 0.66198E+00_JPRB, 0.66136E+00_JPRB, 0.66086E+00_JPRB ,&
&  0.66051E+00_JPRB, 0.66062E+00_JPRB, 0.66045E+00_JPRB, 0.66019E+00_JPRB, 0.66004E+00_JPRB, 0.66087E+00_JPRB, 0.66150E+00_JPRB ,&
&  0.66233E+00_JPRB, 0.66293E+00_JPRB, 0.66337E+00_JPRB, 0.66369E+00_JPRB, 0.66429E+00_JPRB, 0.66510E+00_JPRB, 0.66596E+00_JPRB ,&
&  0.66694E+00_JPRB, 0.66834E+00_JPRB, 0.67071E+00_JPRB, 0.67356E+00_JPRB, 0.67636E+00_JPRB, 0.67929E+00_JPRB, 0.68265E+00_JPRB ,&
&  0.68607E+00_JPRB, 0.68925E+00_JPRB, 0.69265E+00_JPRB, 0.69619E+00_JPRB, 0.70002E+00_JPRB, 0.70372E+00_JPRB, 0.70758E+00_JPRB ,&
&  0.71127E+00_JPRB, 0.71514E+00_JPRB, 0.71943E+00_JPRB, 0.72383E+00_JPRB, 0.72794E+00_JPRB, 0.73149E+00_JPRB, 0.73533E+00_JPRB ,&
&  0.73939E+00_JPRB, 0.74342E+00_JPRB, 0.74651E+00_JPRB, 0.74988E+00_JPRB, 0.75332E+00_JPRB, 0.75666E+00_JPRB, 0.75907E+00_JPRB ,&
&  0.76159E+00_JPRB, 0.76439E+00_JPRB, 0.76732E+00_JPRB, 0.76832E+00_JPRB, 0.76902E+00_JPRB, 0.76940E+00_JPRB, 0.76908E+00_JPRB ,&
&  0.76765E+00_JPRB, 0.76600E+00_JPRB, 0.76429E+00_JPRB, 0.76213E+00_JPRB, 0.75911E+00_JPRB, 0.75573E+00_JPRB, 0.75188E+00_JPRB ,&
&  0.74781E+00_JPRB, 0.74293E+00_JPRB, 0.73808E+00_JPRB, 0.73416E+00_JPRB, 0.73137E+00_JPRB, 0.72844E+00_JPRB, 0.72540E+00_JPRB ,&
&  0.72235E+00_JPRB, 0.72093E+00_JPRB, 0.72045E+00_JPRB, 0.72007E+00_JPRB, 0.71939E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.71640E+00_JPRB, 0.71628E+00_JPRB, 0.71627E+00_JPRB, 0.71554E+00_JPRB, 0.71266E+00_JPRB, 0.70995E+00_JPRB, 0.70698E+00_JPRB ,&
&  0.70456E+00_JPRB, 0.70249E+00_JPRB, 0.69889E+00_JPRB, 0.69438E+00_JPRB, 0.68982E+00_JPRB, 0.68588E+00_JPRB, 0.68177E+00_JPRB ,&
&  0.67785E+00_JPRB, 0.67373E+00_JPRB, 0.66967E+00_JPRB, 0.66286E+00_JPRB, 0.65600E+00_JPRB, 0.64974E+00_JPRB, 0.64597E+00_JPRB ,&
&  0.64249E+00_JPRB, 0.63939E+00_JPRB, 0.63576E+00_JPRB, 0.63213E+00_JPRB, 0.62876E+00_JPRB, 0.62576E+00_JPRB, 0.62266E+00_JPRB ,&
&  0.62065E+00_JPRB, 0.61935E+00_JPRB, 0.61830E+00_JPRB, 0.61695E+00_JPRB, 0.61782E+00_JPRB, 0.61924E+00_JPRB, 0.62059E+00_JPRB ,&
&  0.62179E+00_JPRB, 0.62322E+00_JPRB, 0.62632E+00_JPRB, 0.62924E+00_JPRB, 0.63190E+00_JPRB, 0.63570E+00_JPRB, 0.63835E+00_JPRB ,&
&  0.63975E+00_JPRB, 0.64161E+00_JPRB, 0.64633E+00_JPRB, 0.65073E+00_JPRB, 0.65507E+00_JPRB, 0.65938E+00_JPRB, 0.66506E+00_JPRB ,&
&  0.67046E+00_JPRB, 0.67607E+00_JPRB, 0.68220E+00_JPRB, 0.68925E+00_JPRB, 0.69680E+00_JPRB, 0.70487E+00_JPRB, 0.71299E+00_JPRB ,&
&  0.72063E+00_JPRB, 0.72838E+00_JPRB, 0.73617E+00_JPRB, 0.74396E+00_JPRB, 0.75136E+00_JPRB, 0.75763E+00_JPRB, 0.76464E+00_JPRB ,&
&  0.77210E+00_JPRB, 0.78113E+00_JPRB, 0.78713E+00_JPRB, 0.79368E+00_JPRB, 0.80070E+00_JPRB, 0.81068E+00_JPRB, 0.81473E+00_JPRB ,&
&  0.81823E+00_JPRB, 0.82131E+00_JPRB, 0.82468E+00_JPRB, 0.82617E+00_JPRB, 0.82758E+00_JPRB, 0.82880E+00_JPRB, 0.82908E+00_JPRB ,&
&  0.82836E+00_JPRB, 0.82636E+00_JPRB, 0.82307E+00_JPRB, 0.81779E+00_JPRB, 0.81173E+00_JPRB, 0.80535E+00_JPRB, 0.79753E+00_JPRB ,&
&  0.79009E+00_JPRB, 0.78074E+00_JPRB, 0.77103E+00_JPRB, 0.76298E+00_JPRB, 0.75682E+00_JPRB, 0.74889E+00_JPRB, 0.74076E+00_JPRB ,&
&  0.73315E+00_JPRB, 0.72701E+00_JPRB, 0.72341E+00_JPRB, 0.72024E+00_JPRB, 0.71830E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.71617E+00_JPRB, 0.71501E+00_JPRB, 0.71312E+00_JPRB, 0.71021E+00_JPRB, 0.70512E+00_JPRB, 0.70045E+00_JPRB, 0.69571E+00_JPRB ,&
&  0.69172E+00_JPRB, 0.68790E+00_JPRB, 0.68463E+00_JPRB, 0.68163E+00_JPRB, 0.67879E+00_JPRB, 0.67665E+00_JPRB, 0.67444E+00_JPRB ,&
&  0.67268E+00_JPRB, 0.67038E+00_JPRB, 0.66812E+00_JPRB, 0.66429E+00_JPRB, 0.65790E+00_JPRB, 0.64829E+00_JPRB, 0.63835E+00_JPRB ,&
&  0.62883E+00_JPRB, 0.62085E+00_JPRB, 0.61296E+00_JPRB, 0.60547E+00_JPRB, 0.59962E+00_JPRB, 0.56111E+00_JPRB, 0.55010E+00_JPRB ,&
&  0.54976E+00_JPRB, 0.55100E+00_JPRB, 0.55623E+00_JPRB, 0.56330E+00_JPRB, 0.57257E+00_JPRB, 0.56571E+00_JPRB, 0.56253E+00_JPRB ,&
&  0.53629E+00_JPRB, 0.53752E+00_JPRB, 0.53992E+00_JPRB, 0.54377E+00_JPRB, 0.55222E+00_JPRB, 0.56172E+00_JPRB, 0.57632E+00_JPRB ,&
&  0.60627E+00_JPRB, 0.61337E+00_JPRB, 0.62159E+00_JPRB, 0.63133E+00_JPRB, 0.64023E+00_JPRB, 0.64812E+00_JPRB, 0.65693E+00_JPRB ,&
&  0.66489E+00_JPRB, 0.67310E+00_JPRB, 0.68122E+00_JPRB, 0.69028E+00_JPRB, 0.70003E+00_JPRB, 0.71024E+00_JPRB, 0.72091E+00_JPRB ,&
&  0.73128E+00_JPRB, 0.74257E+00_JPRB, 0.75394E+00_JPRB, 0.76572E+00_JPRB, 0.77751E+00_JPRB, 0.78994E+00_JPRB, 0.80445E+00_JPRB ,&
&  0.81869E+00_JPRB, 0.83395E+00_JPRB, 0.84406E+00_JPRB, 0.85461E+00_JPRB, 0.86169E+00_JPRB, 0.87056E+00_JPRB, 0.87300E+00_JPRB ,&
&  0.87586E+00_JPRB, 0.87295E+00_JPRB, 0.87230E+00_JPRB, 0.86937E+00_JPRB, 0.86739E+00_JPRB, 0.86627E+00_JPRB, 0.86348E+00_JPRB ,&
&  0.86185E+00_JPRB, 0.85842E+00_JPRB, 0.85569E+00_JPRB, 0.85023E+00_JPRB, 0.84283E+00_JPRB, 0.83326E+00_JPRB, 0.82316E+00_JPRB ,&
&  0.81489E+00_JPRB, 0.80784E+00_JPRB, 0.80002E+00_JPRB, 0.79246E+00_JPRB, 0.78266E+00_JPRB, 0.77116E+00_JPRB, 0.75892E+00_JPRB ,&
&  0.74819E+00_JPRB, 0.73856E+00_JPRB, 0.73034E+00_JPRB, 0.72348E+00_JPRB, 0.71927E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.71538E+00_JPRB, 0.71410E+00_JPRB, 0.71178E+00_JPRB, 0.70936E+00_JPRB, 0.70401E+00_JPRB, 0.69937E+00_JPRB, 0.69463E+00_JPRB ,&
&  0.69000E+00_JPRB, 0.68578E+00_JPRB, 0.68130E+00_JPRB, 0.67796E+00_JPRB, 0.67402E+00_JPRB, 0.67134E+00_JPRB, 0.66738E+00_JPRB ,&
&  0.66552E+00_JPRB, 0.66239E+00_JPRB, 0.65911E+00_JPRB, 0.65290E+00_JPRB, 0.64651E+00_JPRB, 0.63903E+00_JPRB, 0.62615E+00_JPRB ,&
&  0.61085E+00_JPRB, 0.59715E+00_JPRB, 0.56525E+00_JPRB, 0.55561E+00_JPRB, 0.53939E+00_JPRB, 0.53031E+00_JPRB, 0.52761E+00_JPRB ,&
&  0.52298E+00_JPRB, 0.51989E+00_JPRB, 0.51078E+00_JPRB, 0.50101E+00_JPRB, 0.49133E+00_JPRB, 0.47616E+00_JPRB, 0.46700E+00_JPRB ,&
&  0.45720E+00_JPRB, 0.45306E+00_JPRB, 0.45474E+00_JPRB, 0.46121E+00_JPRB, 0.47567E+00_JPRB, 0.49443E+00_JPRB, 0.51830E+00_JPRB ,&
&  0.55135E+00_JPRB, 0.58173E+00_JPRB, 0.61365E+00_JPRB, 0.62220E+00_JPRB, 0.63316E+00_JPRB, 0.64430E+00_JPRB, 0.65373E+00_JPRB ,&
&  0.65291E+00_JPRB, 0.65712E+00_JPRB, 0.67752E+00_JPRB, 0.68734E+00_JPRB, 0.69717E+00_JPRB, 0.70710E+00_JPRB, 0.71736E+00_JPRB ,&
&  0.72763E+00_JPRB, 0.73894E+00_JPRB, 0.75078E+00_JPRB, 0.76563E+00_JPRB, 0.78710E+00_JPRB, 0.81129E+00_JPRB, 0.83274E+00_JPRB ,&
&  0.85170E+00_JPRB, 0.87351E+00_JPRB, 0.88926E+00_JPRB, 0.90369E+00_JPRB, 0.91686E+00_JPRB, 0.93429E+00_JPRB, 0.94744E+00_JPRB ,&
&  0.95513E+00_JPRB, 0.95056E+00_JPRB, 0.93824E+00_JPRB, 0.91511E+00_JPRB, 0.90347E+00_JPRB, 0.91118E+00_JPRB, 0.90671E+00_JPRB ,&
&  0.90423E+00_JPRB, 0.89710E+00_JPRB, 0.89177E+00_JPRB, 0.88338E+00_JPRB, 0.87470E+00_JPRB, 0.86353E+00_JPRB, 0.84867E+00_JPRB ,&
&  0.83586E+00_JPRB, 0.82694E+00_JPRB, 0.81717E+00_JPRB, 0.80693E+00_JPRB, 0.79242E+00_JPRB, 0.77742E+00_JPRB, 0.76341E+00_JPRB ,&
&  0.75104E+00_JPRB, 0.74036E+00_JPRB, 0.73043E+00_JPRB, 0.72348E+00_JPRB, 0.71901E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.72048E+00_JPRB, 0.71844E+00_JPRB, 0.71324E+00_JPRB, 0.70872E+00_JPRB, 0.70247E+00_JPRB, 0.69693E+00_JPRB, 0.69044E+00_JPRB ,&
&  0.68446E+00_JPRB, 0.67853E+00_JPRB, 0.67308E+00_JPRB, 0.66841E+00_JPRB, 0.66377E+00_JPRB, 0.65987E+00_JPRB, 0.65674E+00_JPRB ,&
&  0.65430E+00_JPRB, 0.65075E+00_JPRB, 0.64565E+00_JPRB, 0.63954E+00_JPRB, 0.63422E+00_JPRB, 0.62861E+00_JPRB, 0.62123E+00_JPRB ,&
&  0.61342E+00_JPRB, 0.60128E+00_JPRB, 0.58085E+00_JPRB, 0.56959E+00_JPRB, 0.55653E+00_JPRB, 0.55273E+00_JPRB, 0.54649E+00_JPRB ,&
&  0.54974E+00_JPRB, 0.50764E+00_JPRB, 0.50221E+00_JPRB, 0.48362E+00_JPRB, 0.46911E+00_JPRB, 0.45796E+00_JPRB, 0.44154E+00_JPRB ,&
&  0.43403E+00_JPRB, 0.42537E+00_JPRB, 0.42268E+00_JPRB, 0.42797E+00_JPRB, 0.43753E+00_JPRB, 0.45816E+00_JPRB, 0.48445E+00_JPRB ,&
&  0.51824E+00_JPRB, 0.58502E+00_JPRB, 0.60302E+00_JPRB, 0.61786E+00_JPRB, 0.63049E+00_JPRB, 0.64062E+00_JPRB, 0.63000E+00_JPRB ,&
&  0.63120E+00_JPRB, 0.63399E+00_JPRB, 0.64655E+00_JPRB, 0.66431E+00_JPRB, 0.68187E+00_JPRB, 0.69001E+00_JPRB, 0.69924E+00_JPRB ,&
&  0.70675E+00_JPRB, 0.71839E+00_JPRB, 0.73065E+00_JPRB, 0.75149E+00_JPRB, 0.78686E+00_JPRB, 0.82475E+00_JPRB, 0.85755E+00_JPRB ,&
&  0.88583E+00_JPRB, 0.90428E+00_JPRB, 0.91603E+00_JPRB, 0.93393E+00_JPRB, 0.95783E+00_JPRB, 0.98935E+00_JPRB, 0.10077E+01_JPRB ,&
&  0.10186E+01_JPRB, 0.10259E+01_JPRB, 0.10223E+01_JPRB, 0.10129E+01_JPRB, 0.98954E+00_JPRB, 0.96689E+00_JPRB, 0.95461E+00_JPRB ,&
&  0.95819E+00_JPRB, 0.94995E+00_JPRB, 0.93630E+00_JPRB, 0.91491E+00_JPRB, 0.89907E+00_JPRB, 0.88113E+00_JPRB, 0.86220E+00_JPRB ,&
&  0.84606E+00_JPRB, 0.83455E+00_JPRB, 0.81987E+00_JPRB, 0.80655E+00_JPRB, 0.78910E+00_JPRB, 0.77332E+00_JPRB, 0.76139E+00_JPRB ,&
&  0.74980E+00_JPRB, 0.74100E+00_JPRB, 0.73192E+00_JPRB, 0.72674E+00_JPRB, 0.72369E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.72203E+00_JPRB, 0.71711E+00_JPRB, 0.71117E+00_JPRB, 0.70621E+00_JPRB, 0.70136E+00_JPRB, 0.69786E+00_JPRB, 0.69498E+00_JPRB ,&
&  0.69139E+00_JPRB, 0.68767E+00_JPRB, 0.68378E+00_JPRB, 0.67952E+00_JPRB, 0.67519E+00_JPRB, 0.67133E+00_JPRB, 0.66802E+00_JPRB ,&
&  0.66300E+00_JPRB, 0.65720E+00_JPRB, 0.64972E+00_JPRB, 0.64319E+00_JPRB, 0.63734E+00_JPRB, 0.63492E+00_JPRB, 0.63055E+00_JPRB ,&
&  0.62609E+00_JPRB, 0.61775E+00_JPRB, 0.60340E+00_JPRB, 0.59216E+00_JPRB, 0.58879E+00_JPRB, 0.58027E+00_JPRB, 0.58108E+00_JPRB ,&
&  0.57633E+00_JPRB, 0.56750E+00_JPRB, 0.55149E+00_JPRB, 0.54714E+00_JPRB, 0.53946E+00_JPRB, 0.47757E+00_JPRB, 0.46124E+00_JPRB ,&
&  0.44040E+00_JPRB, 0.41940E+00_JPRB, 0.41170E+00_JPRB, 0.41006E+00_JPRB, 0.41524E+00_JPRB, 0.43511E+00_JPRB, 0.46234E+00_JPRB ,&
&  0.49253E+00_JPRB, 0.57159E+00_JPRB, 0.59174E+00_JPRB, 0.60894E+00_JPRB, 0.61900E+00_JPRB, 0.62362E+00_JPRB, 0.61720E+00_JPRB ,&
&  0.61526E+00_JPRB, 0.61724E+00_JPRB, 0.63174E+00_JPRB, 0.65373E+00_JPRB, 0.67196E+00_JPRB, 0.67510E+00_JPRB, 0.67527E+00_JPRB ,&
&  0.67169E+00_JPRB, 0.68255E+00_JPRB, 0.70278E+00_JPRB, 0.73169E+00_JPRB, 0.77502E+00_JPRB, 0.80828E+00_JPRB, 0.85131E+00_JPRB ,&
&  0.88960E+00_JPRB, 0.91662E+00_JPRB, 0.95845E+00_JPRB, 0.10038E+01_JPRB, 0.10359E+01_JPRB, 0.10576E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10755E+01_JPRB, 0.10821E+01_JPRB, 0.10806E+01_JPRB, 0.10685E+01_JPRB, 0.10421E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.99507E+00_JPRB, 0.10011E+01_JPRB, 0.98886E+00_JPRB, 0.95060E+00_JPRB, 0.91660E+00_JPRB, 0.88977E+00_JPRB, 0.86882E+00_JPRB ,&
&  0.85451E+00_JPRB, 0.84417E+00_JPRB, 0.82795E+00_JPRB, 0.81231E+00_JPRB, 0.79551E+00_JPRB, 0.77863E+00_JPRB, 0.76721E+00_JPRB ,&
&  0.75707E+00_JPRB, 0.75108E+00_JPRB, 0.74328E+00_JPRB, 0.73587E+00_JPRB, 0.73029E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.72266E+00_JPRB, 0.71791E+00_JPRB, 0.71161E+00_JPRB, 0.70585E+00_JPRB, 0.70227E+00_JPRB, 0.70098E+00_JPRB, 0.69745E+00_JPRB ,&
&  0.69235E+00_JPRB, 0.68840E+00_JPRB, 0.68221E+00_JPRB, 0.67908E+00_JPRB, 0.67499E+00_JPRB, 0.67039E+00_JPRB, 0.66655E+00_JPRB ,&
&  0.66152E+00_JPRB, 0.64781E+00_JPRB, 0.63992E+00_JPRB, 0.63865E+00_JPRB, 0.63901E+00_JPRB, 0.64069E+00_JPRB, 0.64241E+00_JPRB ,&
&  0.63874E+00_JPRB, 0.63029E+00_JPRB, 0.62182E+00_JPRB, 0.60740E+00_JPRB, 0.59699E+00_JPRB, 0.59626E+00_JPRB, 0.60118E+00_JPRB ,&
&  0.59864E+00_JPRB, 0.58120E+00_JPRB, 0.58291E+00_JPRB, 0.58163E+00_JPRB, 0.56353E+00_JPRB, 0.55627E+00_JPRB, 0.49715E+00_JPRB ,&
&  0.45742E+00_JPRB, 0.42705E+00_JPRB, 0.41534E+00_JPRB, 0.40838E+00_JPRB, 0.41585E+00_JPRB, 0.43517E+00_JPRB, 0.47611E+00_JPRB ,&
&  0.51889E+00_JPRB, 0.56270E+00_JPRB, 0.57541E+00_JPRB, 0.58951E+00_JPRB, 0.59580E+00_JPRB, 0.60943E+00_JPRB, 0.61966E+00_JPRB ,&
&  0.62781E+00_JPRB, 0.63622E+00_JPRB, 0.65310E+00_JPRB, 0.67056E+00_JPRB, 0.70386E+00_JPRB, 0.73407E+00_JPRB, 0.75354E+00_JPRB ,&
&  0.77488E+00_JPRB, 0.78473E+00_JPRB, 0.80622E+00_JPRB, 0.83125E+00_JPRB, 0.86648E+00_JPRB, 0.88630E+00_JPRB, 0.90507E+00_JPRB ,&
&  0.92627E+00_JPRB, 0.98095E+00_JPRB, 0.10576E+01_JPRB, 0.11458E+01_JPRB, 0.11729E+01_JPRB, 0.11715E+01_JPRB, 0.11342E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.11075E+01_JPRB, 0.12425E+01_JPRB, 0.12302E+01_JPRB, 0.12200E+01_JPRB, 0.12018E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10407E+01_JPRB, 0.10087E+01_JPRB, 0.95221E+00_JPRB, 0.90339E+00_JPRB, 0.87301E+00_JPRB, 0.85789E+00_JPRB ,&
&  0.85068E+00_JPRB, 0.84778E+00_JPRB, 0.83445E+00_JPRB, 0.82263E+00_JPRB, 0.80606E+00_JPRB, 0.78733E+00_JPRB, 0.77472E+00_JPRB ,&
&  0.76200E+00_JPRB, 0.75304E+00_JPRB, 0.74593E+00_JPRB, 0.73838E+00_JPRB, 0.73102E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.72512E+00_JPRB, 0.72365E+00_JPRB, 0.72164E+00_JPRB, 0.72497E+00_JPRB, 0.73001E+00_JPRB, 0.72686E+00_JPRB, 0.71864E+00_JPRB ,&
&  0.70261E+00_JPRB, 0.68507E+00_JPRB, 0.66946E+00_JPRB, 0.65846E+00_JPRB, 0.65160E+00_JPRB, 0.64758E+00_JPRB, 0.64442E+00_JPRB ,&
&  0.64275E+00_JPRB, 0.64182E+00_JPRB, 0.63821E+00_JPRB, 0.63197E+00_JPRB, 0.63107E+00_JPRB, 0.63450E+00_JPRB, 0.63911E+00_JPRB ,&
&  0.64216E+00_JPRB, 0.63901E+00_JPRB, 0.63231E+00_JPRB, 0.62140E+00_JPRB, 0.60844E+00_JPRB, 0.61430E+00_JPRB, 0.60610E+00_JPRB ,&
&  0.59698E+00_JPRB, 0.59522E+00_JPRB, 0.60091E+00_JPRB, 0.60155E+00_JPRB, 0.59787E+00_JPRB, 0.57939E+00_JPRB, 0.53072E+00_JPRB ,&
&  0.48163E+00_JPRB, 0.44258E+00_JPRB, 0.42064E+00_JPRB, 0.41456E+00_JPRB, 0.42523E+00_JPRB, 0.45383E+00_JPRB, 0.49713E+00_JPRB ,&
&  0.55045E+00_JPRB, 0.56792E+00_JPRB, 0.58227E+00_JPRB, 0.59891E+00_JPRB, 0.61473E+00_JPRB, 0.63840E+00_JPRB, 0.66445E+00_JPRB ,&
&  0.69524E+00_JPRB, 0.73681E+00_JPRB, 0.79298E+00_JPRB, 0.86735E+00_JPRB, 0.94924E+00_JPRB, 0.94850E+00_JPRB, 0.10197E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.12050E+01_JPRB, 0.11737E+01_JPRB, 0.11187E+01_JPRB, 0.10873E+01_JPRB, 0.10799E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.11509E+01_JPRB, 0.12077E+01_JPRB, 0.12683E+01_JPRB, 0.12923E+01_JPRB, 0.12898E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.12028E+01_JPRB, 0.14533E+01_JPRB, 0.13264E+01_JPRB, 0.12245E+01_JPRB, 0.11814E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.10411E+01_JPRB, 0.99062E+00_JPRB, 0.94610E+00_JPRB, 0.89465E+00_JPRB, 0.84830E+00_JPRB, 0.80850E+00_JPRB ,&
&  0.79673E+00_JPRB, 0.79181E+00_JPRB, 0.78645E+00_JPRB, 0.78266E+00_JPRB, 0.77183E+00_JPRB, 0.76533E+00_JPRB, 0.75951E+00_JPRB ,&
&  0.74434E+00_JPRB, 0.73346E+00_JPRB, 0.73057E+00_JPRB, 0.72899E+00_JPRB, 0.72696E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.68144E+00_JPRB, 0.69921E+00_JPRB, 0.73119E+00_JPRB, 0.74956E+00_JPRB, 0.75436E+00_JPRB, 0.73022E+00_JPRB, 0.70199E+00_JPRB ,&
&  0.67364E+00_JPRB, 0.64280E+00_JPRB, 0.62339E+00_JPRB, 0.61282E+00_JPRB, 0.60724E+00_JPRB, 0.60376E+00_JPRB, 0.60200E+00_JPRB ,&
&  0.60197E+00_JPRB, 0.60181E+00_JPRB, 0.60573E+00_JPRB, 0.61662E+00_JPRB, 0.62637E+00_JPRB, 0.62872E+00_JPRB, 0.63039E+00_JPRB ,&
&  0.63441E+00_JPRB, 0.63727E+00_JPRB, 0.63288E+00_JPRB, 0.62085E+00_JPRB, 0.61627E+00_JPRB, 0.63104E+00_JPRB, 0.63486E+00_JPRB ,&
&  0.63072E+00_JPRB, 0.61751E+00_JPRB, 0.61805E+00_JPRB, 0.63079E+00_JPRB, 0.63150E+00_JPRB, 0.61866E+00_JPRB, 0.57138E+00_JPRB ,&
&  0.51195E+00_JPRB, 0.46390E+00_JPRB, 0.44703E+00_JPRB, 0.46305E+00_JPRB, 0.49979E+00_JPRB, 0.55535E+00_JPRB, 0.57440E+00_JPRB ,&
&  0.58771E+00_JPRB, 0.59725E+00_JPRB, 0.60926E+00_JPRB, 0.63476E+00_JPRB, 0.67115E+00_JPRB, 0.70799E+00_JPRB, 0.74750E+00_JPRB ,&
&  0.80473E+00_JPRB, 0.86438E+00_JPRB, 0.93957E+00_JPRB, 0.91672E+00_JPRB, 0.96060E+00_JPRB, 0.10621E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.12144E+01_JPRB, 0.12629E+01_JPRB, 0.13252E+01_JPRB, 0.13171E+01_JPRB, 0.12971E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.12763E+01_JPRB, 0.12885E+01_JPRB, 0.13497E+01_JPRB, 0.14198E+01_JPRB, 0.14263E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.13110E+01_JPRB, 0.12025E+01_JPRB, 0.10953E+01_JPRB, 0.10210E+01_JPRB, 0.10122E+01_JPRB, 0.10377E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10454E+01_JPRB, 0.10101E+01_JPRB, 0.97934E+00_JPRB, 0.89187E+00_JPRB, 0.82790E+00_JPRB, 0.77822E+00_JPRB ,&
&  0.75938E+00_JPRB, 0.74449E+00_JPRB, 0.73303E+00_JPRB, 0.74128E+00_JPRB, 0.72983E+00_JPRB, 0.72525E+00_JPRB, 0.72068E+00_JPRB ,&
&  0.70987E+00_JPRB, 0.69341E+00_JPRB, 0.68468E+00_JPRB, 0.67831E+00_JPRB, 0.67710E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.75600E+00_JPRB, 0.76789E+00_JPRB, 0.77673E+00_JPRB, 0.77898E+00_JPRB, 0.76048E+00_JPRB, 0.74468E+00_JPRB, 0.73096E+00_JPRB ,&
&  0.72270E+00_JPRB, 0.66919E+00_JPRB, 0.62906E+00_JPRB, 0.60139E+00_JPRB, 0.58163E+00_JPRB, 0.56313E+00_JPRB, 0.55795E+00_JPRB ,&
&  0.56121E+00_JPRB, 0.56869E+00_JPRB, 0.58104E+00_JPRB, 0.59558E+00_JPRB, 0.60827E+00_JPRB, 0.61356E+00_JPRB, 0.62192E+00_JPRB ,&
&  0.63619E+00_JPRB, 0.64904E+00_JPRB, 0.65473E+00_JPRB, 0.66197E+00_JPRB, 0.66788E+00_JPRB, 0.68298E+00_JPRB, 0.68702E+00_JPRB ,&
&  0.68528E+00_JPRB, 0.67646E+00_JPRB, 0.67158E+00_JPRB, 0.68708E+00_JPRB, 0.69723E+00_JPRB, 0.69515E+00_JPRB, 0.63326E+00_JPRB ,&
&  0.53673E+00_JPRB, 0.49806E+00_JPRB, 0.51098E+00_JPRB, 0.57215E+00_JPRB, 0.59303E+00_JPRB, 0.60830E+00_JPRB, 0.62407E+00_JPRB ,&
&  0.61449E+00_JPRB, 0.60698E+00_JPRB, 0.62670E+00_JPRB, 0.68291E+00_JPRB, 0.71349E+00_JPRB, 0.74760E+00_JPRB, 0.79436E+00_JPRB ,&
&  0.87305E+00_JPRB, 0.93658E+00_JPRB, 0.10101E+01_JPRB, 0.97659E+00_JPRB, 0.11082E+01_JPRB, 0.13759E+01_JPRB, 0.13453E+01_JPRB ,&
&  0.13447E+01_JPRB, 0.13615E+01_JPRB, 0.14397E+01_JPRB, 0.14911E+01_JPRB, 0.14995E+01_JPRB, 0.14862E+01_JPRB, 0.14816E+01_JPRB ,&
&  0.14623E+01_JPRB, 0.14569E+01_JPRB, 0.14716E+01_JPRB, 0.15559E+01_JPRB, 0.15605E+01_JPRB, 0.15294E+01_JPRB, 0.14879E+01_JPRB ,&
&  0.14123E+01_JPRB, 0.12994E+01_JPRB, 0.11927E+01_JPRB, 0.11112E+01_JPRB, 0.10862E+01_JPRB, 0.10892E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10576E+01_JPRB, 0.10490E+01_JPRB, 0.10441E+01_JPRB, 0.10335E+01_JPRB, 0.94602E+00_JPRB, 0.84538E+00_JPRB, 0.77018E+00_JPRB ,&
&  0.71907E+00_JPRB, 0.68535E+00_JPRB, 0.65711E+00_JPRB, 0.65992E+00_JPRB, 0.67418E+00_JPRB, 0.69273E+00_JPRB, 0.68364E+00_JPRB ,&
&  0.67905E+00_JPRB, 0.67258E+00_JPRB, 0.67865E+00_JPRB, 0.70726E+00_JPRB, 0.73455E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.79505E+00_JPRB, 0.80546E+00_JPRB, 0.81165E+00_JPRB, 0.79620E+00_JPRB, 0.77346E+00_JPRB, 0.75594E+00_JPRB, 0.73819E+00_JPRB ,&
&  0.72785E+00_JPRB, 0.64937E+00_JPRB, 0.60912E+00_JPRB, 0.58964E+00_JPRB, 0.57681E+00_JPRB, 0.55205E+00_JPRB, 0.53281E+00_JPRB ,&
&  0.53346E+00_JPRB, 0.55167E+00_JPRB, 0.58462E+00_JPRB, 0.61044E+00_JPRB, 0.63484E+00_JPRB, 0.65418E+00_JPRB, 0.66211E+00_JPRB ,&
&  0.67006E+00_JPRB, 0.68130E+00_JPRB, 0.69229E+00_JPRB, 0.70675E+00_JPRB, 0.72460E+00_JPRB, 0.75774E+00_JPRB, 0.80233E+00_JPRB ,&
&  0.83650E+00_JPRB, 0.85045E+00_JPRB, 0.82026E+00_JPRB, 0.82715E+00_JPRB, 0.84140E+00_JPRB, 0.84939E+00_JPRB, 0.84941E+00_JPRB ,&
&  0.61812E+00_JPRB, 0.58808E+00_JPRB, 0.65270E+00_JPRB, 0.64538E+00_JPRB, 0.65135E+00_JPRB, 0.64673E+00_JPRB, 0.64002E+00_JPRB ,&
&  0.64033E+00_JPRB, 0.65163E+00_JPRB, 0.67800E+00_JPRB, 0.71412E+00_JPRB, 0.74342E+00_JPRB, 0.77450E+00_JPRB, 0.82229E+00_JPRB ,&
&  0.93265E+00_JPRB, 0.89346E+00_JPRB, 0.90779E+00_JPRB, 0.10129E+01_JPRB, 0.12345E+01_JPRB, 0.14190E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.16118E+01_JPRB, 0.16141E+01_JPRB, 0.16204E+01_JPRB, 0.16650E+01_JPRB, 0.16852E+01_JPRB, 0.16856E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.17568E+01_JPRB, 0.17901E+01_JPRB, 0.18228E+01_JPRB, 0.17574E+01_JPRB, 0.17021E+01_JPRB, 0.16382E+01_JPRB ,&
&  0.15590E+01_JPRB, 0.14621E+01_JPRB, 0.13451E+01_JPRB, 0.12506E+01_JPRB, 0.11817E+01_JPRB, 0.11717E+01_JPRB, 0.11605E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.11103E+01_JPRB, 0.10807E+01_JPRB, 0.10494E+01_JPRB, 0.98695E+00_JPRB, 0.92328E+00_JPRB, 0.89048E+00_JPRB ,&
&  0.80481E+00_JPRB, 0.73956E+00_JPRB, 0.66519E+00_JPRB, 0.66169E+00_JPRB, 0.69904E+00_JPRB, 0.71408E+00_JPRB, 0.71396E+00_JPRB ,&
&  0.74000E+00_JPRB, 0.75622E+00_JPRB, 0.75674E+00_JPRB, 0.76855E+00_JPRB, 0.78297E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.81274E+00_JPRB, 0.80966E+00_JPRB, 0.80230E+00_JPRB, 0.78869E+00_JPRB, 0.78510E+00_JPRB, 0.77445E+00_JPRB, 0.77604E+00_JPRB ,&
&  0.79686E+00_JPRB, 0.75108E+00_JPRB, 0.79347E+00_JPRB, 0.66767E+00_JPRB, 0.60025E+00_JPRB, 0.56713E+00_JPRB, 0.54728E+00_JPRB ,&
&  0.55478E+00_JPRB, 0.58752E+00_JPRB, 0.61741E+00_JPRB, 0.63938E+00_JPRB, 0.68324E+00_JPRB, 0.69278E+00_JPRB, 0.69293E+00_JPRB ,&
&  0.69926E+00_JPRB, 0.70741E+00_JPRB, 0.74310E+00_JPRB, 0.76005E+00_JPRB, 0.80855E+00_JPRB, 0.87544E+00_JPRB, 0.98353E+00_JPRB ,&
&  0.94544E+00_JPRB, 0.95403E+00_JPRB, 0.10254E+01_JPRB, 0.10547E+01_JPRB, 0.10732E+01_JPRB, 0.10529E+01_JPRB, 0.10089E+01_JPRB ,&
&  0.97203E+00_JPRB, 0.81910E+00_JPRB, 0.82623E+00_JPRB, 0.80213E+00_JPRB, 0.77994E+00_JPRB, 0.75742E+00_JPRB, 0.72354E+00_JPRB ,&
&  0.70203E+00_JPRB, 0.70245E+00_JPRB, 0.72032E+00_JPRB, 0.74833E+00_JPRB, 0.78985E+00_JPRB, 0.84065E+00_JPRB, 0.89292E+00_JPRB ,&
&  0.10192E+01_JPRB, 0.96098E+00_JPRB, 0.11710E+01_JPRB, 0.12802E+01_JPRB, 0.14481E+01_JPRB, 0.16777E+01_JPRB, 0.18486E+01_JPRB ,&
&  0.20875E+01_JPRB, 0.20501E+01_JPRB, 0.20495E+01_JPRB, 0.20236E+01_JPRB, 0.19790E+01_JPRB, 0.19626E+01_JPRB, 0.19875E+01_JPRB ,&
&  0.20587E+01_JPRB, 0.20562E+01_JPRB, 0.22069E+01_JPRB, 0.21600E+01_JPRB, 0.20170E+01_JPRB, 0.19194E+01_JPRB, 0.18092E+01_JPRB ,&
&  0.17150E+01_JPRB, 0.16495E+01_JPRB, 0.15784E+01_JPRB, 0.14095E+01_JPRB, 0.13633E+01_JPRB, 0.13328E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.11103E+01_JPRB, 0.10188E+01_JPRB, 0.10244E+01_JPRB, 0.95518E+00_JPRB, 0.90101E+00_JPRB, 0.88992E+00_JPRB ,&
&  0.85200E+00_JPRB, 0.76126E+00_JPRB, 0.75716E+00_JPRB, 0.78044E+00_JPRB, 0.76086E+00_JPRB, 0.74159E+00_JPRB, 0.77673E+00_JPRB ,&
&  0.77982E+00_JPRB, 0.79386E+00_JPRB, 0.83412E+00_JPRB, 0.82914E+00_JPRB, 0.82835E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.82104E+00_JPRB, 0.79807E+00_JPRB, 0.78314E+00_JPRB, 0.77959E+00_JPRB, 0.79332E+00_JPRB, 0.82224E+00_JPRB, 0.81098E+00_JPRB ,&
&  0.83309E+00_JPRB, 0.84115E+00_JPRB, 0.82894E+00_JPRB, 0.80555E+00_JPRB, 0.76832E+00_JPRB, 0.67487E+00_JPRB, 0.56924E+00_JPRB ,&
&  0.55548E+00_JPRB, 0.60619E+00_JPRB, 0.65333E+00_JPRB, 0.67245E+00_JPRB, 0.70983E+00_JPRB, 0.71232E+00_JPRB, 0.71283E+00_JPRB ,&
&  0.71721E+00_JPRB, 0.73152E+00_JPRB, 0.76959E+00_JPRB, 0.83520E+00_JPRB, 0.89798E+00_JPRB, 0.10029E+01_JPRB, 0.11039E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.97595E+00_JPRB, 0.99014E+00_JPRB, 0.96760E+00_JPRB, 0.10688E+01_JPRB, 0.10954E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10024E+01_JPRB, 0.94127E+00_JPRB, 0.90052E+00_JPRB, 0.86163E+00_JPRB, 0.83009E+00_JPRB, 0.81413E+00_JPRB ,&
&  0.82354E+00_JPRB, 0.84092E+00_JPRB, 0.87307E+00_JPRB, 0.92757E+00_JPRB, 0.99020E+00_JPRB, 0.10615E+01_JPRB, 0.11739E+01_JPRB ,&
&  0.12760E+01_JPRB, 0.14610E+01_JPRB, 0.16878E+01_JPRB, 0.16627E+01_JPRB, 0.18363E+01_JPRB, 0.20131E+01_JPRB, 0.20742E+01_JPRB ,&
&  0.22335E+01_JPRB, 0.22251E+01_JPRB, 0.23255E+01_JPRB, 0.23891E+01_JPRB, 0.24389E+01_JPRB, 0.24393E+01_JPRB, 0.23811E+01_JPRB ,&
&  0.23644E+01_JPRB, 0.25316E+01_JPRB, 0.25773E+01_JPRB, 0.24750E+01_JPRB, 0.22763E+01_JPRB, 0.21379E+01_JPRB, 0.19810E+01_JPRB ,&
&  0.19148E+01_JPRB, 0.19045E+01_JPRB, 0.18332E+01_JPRB, 0.17541E+01_JPRB, 0.16239E+01_JPRB, 0.14743E+01_JPRB, 0.13524E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.10456E+01_JPRB, 0.95343E+00_JPRB, 0.92837E+00_JPRB, 0.91511E+00_JPRB, 0.85265E+00_JPRB, 0.84952E+00_JPRB ,&
&  0.85627E+00_JPRB, 0.83071E+00_JPRB, 0.87289E+00_JPRB, 0.88322E+00_JPRB, 0.85611E+00_JPRB, 0.82396E+00_JPRB, 0.78073E+00_JPRB ,&
&  0.74789E+00_JPRB, 0.79528E+00_JPRB, 0.83512E+00_JPRB, 0.84981E+00_JPRB, 0.84276E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.83570E+00_JPRB, 0.81056E+00_JPRB, 0.77921E+00_JPRB, 0.77661E+00_JPRB, 0.80879E+00_JPRB, 0.84092E+00_JPRB, 0.86069E+00_JPRB ,&
&  0.85427E+00_JPRB, 0.85398E+00_JPRB, 0.84226E+00_JPRB, 0.82703E+00_JPRB, 0.79948E+00_JPRB, 0.76626E+00_JPRB, 0.69702E+00_JPRB ,&
&  0.62427E+00_JPRB, 0.72125E+00_JPRB, 0.68665E+00_JPRB, 0.70298E+00_JPRB, 0.78126E+00_JPRB, 0.76139E+00_JPRB, 0.75361E+00_JPRB ,&
&  0.76722E+00_JPRB, 0.78407E+00_JPRB, 0.83464E+00_JPRB, 0.88771E+00_JPRB, 0.97467E+00_JPRB, 0.11461E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.10058E+01_JPRB, 0.93335E+00_JPRB, 0.95272E+00_JPRB, 0.10074E+01_JPRB, 0.10642E+01_JPRB, 0.11164E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10005E+01_JPRB, 0.91539E+00_JPRB, 0.85962E+00_JPRB, 0.83033E+00_JPRB, 0.82053E+00_JPRB, 0.82078E+00_JPRB ,&
&  0.82649E+00_JPRB, 0.85028E+00_JPRB, 0.90216E+00_JPRB, 0.98730E+00_JPRB, 0.10905E+01_JPRB, 0.13249E+01_JPRB, 0.15465E+01_JPRB ,&
&  0.16479E+01_JPRB, 0.17712E+01_JPRB, 0.19651E+01_JPRB, 0.21584E+01_JPRB, 0.23778E+01_JPRB, 0.24275E+01_JPRB, 0.24648E+01_JPRB ,&
&  0.25424E+01_JPRB, 0.25087E+01_JPRB, 0.28199E+01_JPRB, 0.28636E+01_JPRB, 0.28481E+01_JPRB, 0.28823E+01_JPRB, 0.27482E+01_JPRB ,&
&  0.26292E+01_JPRB, 0.25234E+01_JPRB, 0.25066E+01_JPRB, 0.24333E+01_JPRB, 0.23473E+01_JPRB, 0.23392E+01_JPRB, 0.21936E+01_JPRB ,&
&  0.21596E+01_JPRB, 0.20892E+01_JPRB, 0.17940E+01_JPRB, 0.15677E+01_JPRB, 0.13351E+01_JPRB, 0.13572E+01_JPRB, 0.13105E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.98061E+00_JPRB, 0.96129E+00_JPRB, 0.93332E+00_JPRB, 0.93997E+00_JPRB, 0.97342E+00_JPRB, 0.97096E+00_JPRB ,&
&  0.98084E+00_JPRB, 0.10144E+01_JPRB, 0.10093E+01_JPRB, 0.10493E+01_JPRB, 0.10056E+01_JPRB, 0.93984E+00_JPRB, 0.79913E+00_JPRB ,&
&  0.75602E+00_JPRB, 0.79020E+00_JPRB, 0.82513E+00_JPRB, 0.84686E+00_JPRB, 0.85031E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.87306E+00_JPRB, 0.84949E+00_JPRB, 0.81833E+00_JPRB, 0.81329E+00_JPRB, 0.84284E+00_JPRB, 0.87556E+00_JPRB, 0.89337E+00_JPRB ,&
&  0.88375E+00_JPRB, 0.88133E+00_JPRB, 0.85949E+00_JPRB, 0.84274E+00_JPRB, 0.82303E+00_JPRB, 0.78877E+00_JPRB, 0.79265E+00_JPRB ,&
&  0.68294E+00_JPRB, 0.81817E+00_JPRB, 0.70071E+00_JPRB, 0.70103E+00_JPRB, 0.88486E+00_JPRB, 0.84830E+00_JPRB, 0.85295E+00_JPRB ,&
&  0.83122E+00_JPRB, 0.86503E+00_JPRB, 0.88530E+00_JPRB, 0.94206E+00_JPRB, 0.11151E+01_JPRB, 0.12219E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10138E+01_JPRB, 0.10423E+01_JPRB, 0.10632E+01_JPRB, 0.10645E+01_JPRB, 0.10878E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.98281E+00_JPRB, 0.93096E+00_JPRB, 0.91015E+00_JPRB, 0.90591E+00_JPRB, 0.91239E+00_JPRB, 0.91667E+00_JPRB ,&
&  0.93253E+00_JPRB, 0.96723E+00_JPRB, 0.10142E+01_JPRB, 0.11194E+01_JPRB, 0.12323E+01_JPRB, 0.15177E+01_JPRB, 0.19154E+01_JPRB ,&
&  0.23735E+01_JPRB, 0.24971E+01_JPRB, 0.27560E+01_JPRB, 0.31997E+01_JPRB, 0.32414E+01_JPRB, 0.31275E+01_JPRB, 0.29247E+01_JPRB ,&
&  0.28630E+01_JPRB, 0.28029E+01_JPRB, 0.28404E+01_JPRB, 0.29478E+01_JPRB, 0.29484E+01_JPRB, 0.30576E+01_JPRB, 0.29403E+01_JPRB ,&
&  0.26322E+01_JPRB, 0.24882E+01_JPRB, 0.24279E+01_JPRB, 0.23909E+01_JPRB, 0.22734E+01_JPRB, 0.22762E+01_JPRB, 0.21488E+01_JPRB ,&
&  0.21140E+01_JPRB, 0.18399E+01_JPRB, 0.13894E+01_JPRB, 0.12704E+01_JPRB, 0.10259E+01_JPRB, 0.10580E+01_JPRB, 0.13273E+01_JPRB ,&
&  0.12913E+01_JPRB, 0.11510E+01_JPRB, 0.10466E+01_JPRB, 0.99461E+00_JPRB, 0.10068E+01_JPRB, 0.12737E+01_JPRB, 0.13588E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.12498E+01_JPRB, 0.13003E+01_JPRB, 0.13318E+01_JPRB, 0.12694E+01_JPRB, 0.11795E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.97867E+00_JPRB, 0.94989E+00_JPRB, 0.93941E+00_JPRB, 0.92012E+00_JPRB, 0.89689E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.96642E+00_JPRB, 0.92958E+00_JPRB, 0.89501E+00_JPRB, 0.88743E+00_JPRB, 0.90481E+00_JPRB, 0.93220E+00_JPRB, 0.94667E+00_JPRB ,&
&  0.93147E+00_JPRB, 0.91998E+00_JPRB, 0.89144E+00_JPRB, 0.86753E+00_JPRB, 0.84585E+00_JPRB, 0.82430E+00_JPRB, 0.81425E+00_JPRB ,&
&  0.86699E+00_JPRB, 0.10057E+01_JPRB, 0.82455E+00_JPRB, 0.70652E+00_JPRB, 0.80399E+00_JPRB, 0.82783E+00_JPRB, 0.89633E+00_JPRB ,&
&  0.87159E+00_JPRB, 0.95789E+00_JPRB, 0.97469E+00_JPRB, 0.10508E+01_JPRB, 0.12456E+01_JPRB, 0.13059E+01_JPRB, 0.13067E+01_JPRB ,&
&  0.13052E+01_JPRB, 0.12416E+01_JPRB, 0.12360E+01_JPRB, 0.12591E+01_JPRB, 0.12685E+01_JPRB, 0.11335E+01_JPRB, 0.11061E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.10339E+01_JPRB, 0.99898E+00_JPRB, 0.99385E+00_JPRB, 0.10056E+01_JPRB, 0.10239E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.11232E+01_JPRB, 0.11482E+01_JPRB, 0.12216E+01_JPRB, 0.13349E+01_JPRB, 0.15381E+01_JPRB, 0.17479E+01_JPRB ,&
&  0.21295E+01_JPRB, 0.24491E+01_JPRB, 0.27266E+01_JPRB, 0.32552E+01_JPRB, 0.34206E+01_JPRB, 0.33434E+01_JPRB, 0.32502E+01_JPRB ,&
&  0.31982E+01_JPRB, 0.31191E+01_JPRB, 0.31861E+01_JPRB, 0.32268E+01_JPRB, 0.32753E+01_JPRB, 0.32691E+01_JPRB, 0.30480E+01_JPRB ,&
&  0.27164E+01_JPRB, 0.24915E+01_JPRB, 0.23736E+01_JPRB, 0.21941E+01_JPRB, 0.20394E+01_JPRB, 0.18780E+01_JPRB, 0.18646E+01_JPRB ,&
&  0.19853E+01_JPRB, 0.11767E+01_JPRB, 0.94801E+00_JPRB, 0.10959E+01_JPRB, 0.94149E+00_JPRB, 0.98778E+00_JPRB, 0.11382E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11151E+01_JPRB, 0.11618E+01_JPRB, 0.11765E+01_JPRB, 0.12922E+01_JPRB, 0.17087E+01_JPRB, 0.18174E+01_JPRB ,&
&  0.17023E+01_JPRB, 0.15731E+01_JPRB, 0.16388E+01_JPRB, 0.15981E+01_JPRB, 0.15424E+01_JPRB, 0.14787E+01_JPRB, 0.13653E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.11980E+01_JPRB, 0.11243E+01_JPRB, 0.10567E+01_JPRB, 0.10071E+01_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.11137E+01_JPRB, 0.10710E+01_JPRB, 0.10379E+01_JPRB, 0.10168E+01_JPRB, 0.10134E+01_JPRB, 0.10300E+01_JPRB, 0.10313E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.98326E+00_JPRB, 0.94206E+00_JPRB, 0.90756E+00_JPRB, 0.88150E+00_JPRB, 0.87573E+00_JPRB, 0.86471E+00_JPRB ,&
&  0.85027E+00_JPRB, 0.97929E+00_JPRB, 0.77033E+00_JPRB, 0.70931E+00_JPRB, 0.67000E+00_JPRB, 0.77486E+00_JPRB, 0.87408E+00_JPRB ,&
&  0.91617E+00_JPRB, 0.97455E+00_JPRB, 0.10560E+01_JPRB, 0.13239E+01_JPRB, 0.15762E+01_JPRB, 0.15753E+01_JPRB, 0.17140E+01_JPRB ,&
&  0.17391E+01_JPRB, 0.16966E+01_JPRB, 0.15101E+01_JPRB, 0.15229E+01_JPRB, 0.13549E+01_JPRB, 0.11949E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.11745E+01_JPRB, 0.11385E+01_JPRB, 0.10959E+01_JPRB, 0.10886E+01_JPRB, 0.10816E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.11402E+01_JPRB, 0.11894E+01_JPRB, 0.12582E+01_JPRB, 0.13227E+01_JPRB, 0.14322E+01_JPRB, 0.15829E+01_JPRB, 0.16469E+01_JPRB ,&
&  0.19119E+01_JPRB, 0.19740E+01_JPRB, 0.18004E+01_JPRB, 0.24138E+01_JPRB, 0.34338E+01_JPRB, 0.34224E+01_JPRB, 0.31730E+01_JPRB ,&
&  0.33966E+01_JPRB, 0.34284E+01_JPRB, 0.35494E+01_JPRB, 0.36894E+01_JPRB, 0.36533E+01_JPRB, 0.35004E+01_JPRB, 0.32298E+01_JPRB ,&
&  0.28561E+01_JPRB, 0.25958E+01_JPRB, 0.24102E+01_JPRB, 0.24030E+01_JPRB, 0.21925E+01_JPRB, 0.20688E+01_JPRB, 0.19767E+01_JPRB ,&
&  0.17352E+01_JPRB, 0.14989E+01_JPRB, 0.10861E+01_JPRB, 0.11233E+01_JPRB, 0.91255E+00_JPRB, 0.94617E+00_JPRB, 0.10965E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.12156E+01_JPRB, 0.13400E+01_JPRB, 0.14023E+01_JPRB, 0.17882E+01_JPRB, 0.21488E+01_JPRB, 0.24673E+01_JPRB ,&
&  0.24245E+01_JPRB, 0.20209E+01_JPRB, 0.20807E+01_JPRB, 0.19818E+01_JPRB, 0.19449E+01_JPRB, 0.18684E+01_JPRB, 0.17623E+01_JPRB ,&
&  0.16261E+01_JPRB, 0.14755E+01_JPRB, 0.13437E+01_JPRB, 0.12387E+01_JPRB, 0.11662E+01_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.12161E+01_JPRB, 0.11602E+01_JPRB, 0.11243E+01_JPRB, 0.11035E+01_JPRB, 0.11000E+01_JPRB, 0.11122E+01_JPRB, 0.11008E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.10307E+01_JPRB, 0.98226E+00_JPRB, 0.94050E+00_JPRB, 0.91340E+00_JPRB, 0.90798E+00_JPRB, 0.89842E+00_JPRB ,&
&  0.86046E+00_JPRB, 0.83344E+00_JPRB, 0.68005E+00_JPRB, 0.61295E+00_JPRB, 0.59729E+00_JPRB, 0.67982E+00_JPRB, 0.83233E+00_JPRB ,&
&  0.94569E+00_JPRB, 0.10523E+01_JPRB, 0.13009E+01_JPRB, 0.16654E+01_JPRB, 0.19348E+01_JPRB, 0.20449E+01_JPRB, 0.21636E+01_JPRB ,&
&  0.25188E+01_JPRB, 0.19955E+01_JPRB, 0.19075E+01_JPRB, 0.17162E+01_JPRB, 0.15735E+01_JPRB, 0.14773E+01_JPRB, 0.13849E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.12408E+01_JPRB, 0.11626E+01_JPRB, 0.11274E+01_JPRB, 0.11236E+01_JPRB, 0.11487E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.13262E+01_JPRB, 0.13886E+01_JPRB, 0.13746E+01_JPRB, 0.13931E+01_JPRB, 0.15461E+01_JPRB, 0.16674E+01_JPRB, 0.17087E+01_JPRB ,&
&  0.16478E+01_JPRB, 0.17019E+01_JPRB, 0.23110E+01_JPRB, 0.27450E+01_JPRB, 0.31685E+01_JPRB, 0.34324E+01_JPRB, 0.35364E+01_JPRB ,&
&  0.36860E+01_JPRB, 0.36017E+01_JPRB, 0.36472E+01_JPRB, 0.32712E+01_JPRB, 0.36137E+01_JPRB, 0.35221E+01_JPRB, 0.32818E+01_JPRB ,&
&  0.30105E+01_JPRB, 0.28631E+01_JPRB, 0.25896E+01_JPRB, 0.24855E+01_JPRB, 0.24952E+01_JPRB, 0.25183E+01_JPRB, 0.19616E+01_JPRB ,&
&  0.15697E+01_JPRB, 0.17117E+01_JPRB, 0.14173E+01_JPRB, 0.12348E+01_JPRB, 0.11021E+01_JPRB, 0.11564E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.14346E+01_JPRB, 0.14865E+01_JPRB, 0.17264E+01_JPRB, 0.21759E+01_JPRB, 0.25661E+01_JPRB, 0.27934E+01_JPRB ,&
&  0.25300E+01_JPRB, 0.26223E+01_JPRB, 0.25350E+01_JPRB, 0.24238E+01_JPRB, 0.23981E+01_JPRB, 0.23082E+01_JPRB, 0.21509E+01_JPRB ,&
&  0.19362E+01_JPRB, 0.17328E+01_JPRB, 0.15646E+01_JPRB, 0.14201E+01_JPRB, 0.13015E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.14263E+01_JPRB, 0.13521E+01_JPRB, 0.12988E+01_JPRB, 0.12649E+01_JPRB, 0.12561E+01_JPRB, 0.12581E+01_JPRB, 0.12227E+01_JPRB ,&
&  0.11551E+01_JPRB, 0.10962E+01_JPRB, 0.10423E+01_JPRB, 0.99722E+00_JPRB, 0.95528E+00_JPRB, 0.93298E+00_JPRB, 0.91464E+00_JPRB ,&
&  0.89616E+00_JPRB, 0.79301E+00_JPRB, 0.63240E+00_JPRB, 0.64738E+00_JPRB, 0.70133E+00_JPRB, 0.67869E+00_JPRB, 0.79218E+00_JPRB ,&
&  0.93167E+00_JPRB, 0.11716E+01_JPRB, 0.14737E+01_JPRB, 0.20084E+01_JPRB, 0.23503E+01_JPRB, 0.25737E+01_JPRB, 0.26537E+01_JPRB ,&
&  0.25547E+01_JPRB, 0.23930E+01_JPRB, 0.21663E+01_JPRB, 0.19049E+01_JPRB, 0.16021E+01_JPRB, 0.14838E+01_JPRB, 0.13954E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.12222E+01_JPRB, 0.11749E+01_JPRB, 0.11934E+01_JPRB, 0.12396E+01_JPRB, 0.13114E+01_JPRB, 0.14290E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.14623E+01_JPRB, 0.13662E+01_JPRB, 0.14298E+01_JPRB, 0.14228E+01_JPRB, 0.14908E+01_JPRB, 0.15151E+01_JPRB ,&
&  0.19670E+01_JPRB, 0.21964E+01_JPRB, 0.22738E+01_JPRB, 0.27776E+01_JPRB, 0.25813E+01_JPRB, 0.27864E+01_JPRB, 0.35653E+01_JPRB ,&
&  0.37521E+01_JPRB, 0.37332E+01_JPRB, 0.38586E+01_JPRB, 0.30174E+01_JPRB, 0.24890E+01_JPRB, 0.34045E+01_JPRB, 0.32865E+01_JPRB ,&
&  0.31096E+01_JPRB, 0.30679E+01_JPRB, 0.28695E+01_JPRB, 0.29496E+01_JPRB, 0.23697E+01_JPRB, 0.19825E+01_JPRB, 0.14720E+01_JPRB ,&
&  0.14074E+01_JPRB, 0.13337E+01_JPRB, 0.14767E+01_JPRB, 0.13754E+01_JPRB, 0.14562E+01_JPRB, 0.13560E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.16523E+01_JPRB, 0.18572E+01_JPRB, 0.22580E+01_JPRB, 0.27760E+01_JPRB, 0.26551E+01_JPRB, 0.29838E+01_JPRB ,&
&  0.30579E+01_JPRB, 0.29775E+01_JPRB, 0.26323E+01_JPRB, 0.26646E+01_JPRB, 0.25578E+01_JPRB, 0.24384E+01_JPRB, 0.22858E+01_JPRB ,&
&  0.20993E+01_JPRB, 0.19244E+01_JPRB, 0.17753E+01_JPRB, 0.16402E+01_JPRB, 0.15236E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.16031E+01_JPRB, 0.15356E+01_JPRB, 0.14809E+01_JPRB, 0.14464E+01_JPRB, 0.14244E+01_JPRB, 0.13849E+01_JPRB, 0.12993E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.11350E+01_JPRB, 0.10679E+01_JPRB, 0.10137E+01_JPRB, 0.95868E+00_JPRB, 0.93877E+00_JPRB, 0.93316E+00_JPRB ,&
&  0.92390E+00_JPRB, 0.91344E+00_JPRB, 0.71390E+00_JPRB, 0.66695E+00_JPRB, 0.76898E+00_JPRB, 0.73372E+00_JPRB, 0.75024E+00_JPRB ,&
&  0.98051E+00_JPRB, 0.13080E+01_JPRB, 0.17099E+01_JPRB, 0.23953E+01_JPRB, 0.27181E+01_JPRB, 0.30445E+01_JPRB, 0.30909E+01_JPRB ,&
&  0.31781E+01_JPRB, 0.26950E+01_JPRB, 0.22470E+01_JPRB, 0.18889E+01_JPRB, 0.16084E+01_JPRB, 0.14352E+01_JPRB, 0.13280E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.11929E+01_JPRB, 0.12261E+01_JPRB, 0.12400E+01_JPRB, 0.13001E+01_JPRB, 0.13553E+01_JPRB, 0.14418E+01_JPRB ,&
&  0.14784E+01_JPRB, 0.14481E+01_JPRB, 0.14313E+01_JPRB, 0.15393E+01_JPRB, 0.15784E+01_JPRB, 0.15578E+01_JPRB, 0.16564E+01_JPRB ,&
&  0.18492E+01_JPRB, 0.19753E+01_JPRB, 0.21502E+01_JPRB, 0.24852E+01_JPRB, 0.29073E+01_JPRB, 0.29321E+01_JPRB, 0.34340E+01_JPRB ,&
&  0.28084E+01_JPRB, 0.24917E+01_JPRB, 0.21817E+01_JPRB, 0.18441E+01_JPRB, 0.21390E+01_JPRB, 0.33505E+01_JPRB, 0.33076E+01_JPRB ,&
&  0.31030E+01_JPRB, 0.31737E+01_JPRB, 0.31416E+01_JPRB, 0.27011E+01_JPRB, 0.22021E+01_JPRB, 0.17566E+01_JPRB, 0.21396E+01_JPRB ,&
&  0.21742E+01_JPRB, 0.23497E+01_JPRB, 0.21073E+01_JPRB, 0.17074E+01_JPRB, 0.15761E+01_JPRB, 0.19603E+01_JPRB, 0.20334E+01_JPRB ,&
&  0.21444E+01_JPRB, 0.23085E+01_JPRB, 0.32691E+01_JPRB, 0.33996E+01_JPRB, 0.33037E+01_JPRB, 0.32376E+01_JPRB, 0.34948E+01_JPRB ,&
&  0.32204E+01_JPRB, 0.31442E+01_JPRB, 0.30027E+01_JPRB, 0.27847E+01_JPRB, 0.25951E+01_JPRB, 0.24135E+01_JPRB, 0.22515E+01_JPRB ,&
&  0.21234E+01_JPRB, 0.20015E+01_JPRB, 0.18680E+01_JPRB, 0.17646E+01_JPRB, 0.16760E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.17077E+01_JPRB, 0.16495E+01_JPRB, 0.16050E+01_JPRB, 0.15557E+01_JPRB, 0.14964E+01_JPRB, 0.14120E+01_JPRB, 0.13107E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.11492E+01_JPRB, 0.10876E+01_JPRB, 0.10352E+01_JPRB, 0.98311E+00_JPRB, 0.95754E+00_JPRB, 0.94493E+00_JPRB ,&
&  0.95144E+00_JPRB, 0.11557E+01_JPRB, 0.88272E+00_JPRB, 0.73818E+00_JPRB, 0.77819E+00_JPRB, 0.77899E+00_JPRB, 0.73909E+00_JPRB ,&
&  0.10327E+01_JPRB, 0.14617E+01_JPRB, 0.18024E+01_JPRB, 0.25226E+01_JPRB, 0.32951E+01_JPRB, 0.31533E+01_JPRB, 0.31250E+01_JPRB ,&
&  0.30587E+01_JPRB, 0.25519E+01_JPRB, 0.21809E+01_JPRB, 0.18972E+01_JPRB, 0.16374E+01_JPRB, 0.14664E+01_JPRB, 0.13414E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.13032E+01_JPRB, 0.12876E+01_JPRB, 0.13003E+01_JPRB, 0.13255E+01_JPRB, 0.14132E+01_JPRB, 0.15069E+01_JPRB ,&
&  0.14819E+01_JPRB, 0.14597E+01_JPRB, 0.15036E+01_JPRB, 0.16477E+01_JPRB, 0.17417E+01_JPRB, 0.16583E+01_JPRB, 0.19257E+01_JPRB ,&
&  0.18750E+01_JPRB, 0.20313E+01_JPRB, 0.20919E+01_JPRB, 0.24780E+01_JPRB, 0.27539E+01_JPRB, 0.28296E+01_JPRB, 0.30985E+01_JPRB ,&
&  0.23867E+01_JPRB, 0.21756E+01_JPRB, 0.22522E+01_JPRB, 0.22233E+01_JPRB, 0.19657E+01_JPRB, 0.22489E+01_JPRB, 0.33103E+01_JPRB ,&
&  0.26139E+01_JPRB, 0.28403E+01_JPRB, 0.28932E+01_JPRB, 0.28738E+01_JPRB, 0.15415E+01_JPRB, 0.12108E+01_JPRB, 0.20310E+01_JPRB ,&
&  0.22647E+01_JPRB, 0.14623E+01_JPRB, 0.10451E+01_JPRB, 0.12601E+01_JPRB, 0.10441E+01_JPRB, 0.15349E+01_JPRB, 0.24633E+01_JPRB ,&
&  0.29491E+01_JPRB, 0.33200E+01_JPRB, 0.46881E+01_JPRB, 0.41941E+01_JPRB, 0.38121E+01_JPRB, 0.40464E+01_JPRB, 0.36886E+01_JPRB ,&
&  0.34302E+01_JPRB, 0.29730E+01_JPRB, 0.29477E+01_JPRB, 0.26158E+01_JPRB, 0.23867E+01_JPRB, 0.22421E+01_JPRB, 0.21644E+01_JPRB ,&
&  0.21188E+01_JPRB, 0.20605E+01_JPRB, 0.19779E+01_JPRB, 0.18796E+01_JPRB, 0.17799E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.17314E+01_JPRB, 0.16880E+01_JPRB, 0.16503E+01_JPRB, 0.15898E+01_JPRB, 0.15062E+01_JPRB, 0.14042E+01_JPRB, 0.13065E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.11703E+01_JPRB, 0.11120E+01_JPRB, 0.10624E+01_JPRB, 0.10135E+01_JPRB, 0.97807E+00_JPRB, 0.95669E+00_JPRB ,&
&  0.94227E+00_JPRB, 0.10076E+01_JPRB, 0.11295E+01_JPRB, 0.98496E+00_JPRB, 0.92437E+00_JPRB, 0.88166E+00_JPRB, 0.88390E+00_JPRB ,&
&  0.12082E+01_JPRB, 0.16033E+01_JPRB, 0.20004E+01_JPRB, 0.25421E+01_JPRB, 0.30007E+01_JPRB, 0.29202E+01_JPRB, 0.30491E+01_JPRB ,&
&  0.27978E+01_JPRB, 0.24172E+01_JPRB, 0.20937E+01_JPRB, 0.18438E+01_JPRB, 0.16459E+01_JPRB, 0.15036E+01_JPRB, 0.14065E+01_JPRB ,&
&  0.13657E+01_JPRB, 0.13476E+01_JPRB, 0.13011E+01_JPRB, 0.12666E+01_JPRB, 0.13288E+01_JPRB, 0.14562E+01_JPRB, 0.15251E+01_JPRB ,&
&  0.14919E+01_JPRB, 0.15052E+01_JPRB, 0.15414E+01_JPRB, 0.16439E+01_JPRB, 0.17740E+01_JPRB, 0.17621E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.15157E+01_JPRB, 0.16557E+01_JPRB, 0.21487E+01_JPRB, 0.23061E+01_JPRB, 0.24582E+01_JPRB, 0.26886E+01_JPRB, 0.28315E+01_JPRB ,&
&  0.27568E+01_JPRB, 0.27290E+01_JPRB, 0.23431E+01_JPRB, 0.24453E+01_JPRB, 0.22791E+01_JPRB, 0.20193E+01_JPRB, 0.25845E+01_JPRB ,&
&  0.26659E+01_JPRB, 0.24997E+01_JPRB, 0.20446E+01_JPRB, 0.16243E+01_JPRB, 0.16316E+01_JPRB, 0.13228E+01_JPRB, 0.95388E+00_JPRB ,&
&  0.93233E+00_JPRB, 0.89408E+00_JPRB, 0.79907E+00_JPRB, 0.79834E+00_JPRB, 0.79297E+00_JPRB, 0.13759E+01_JPRB, 0.25556E+01_JPRB ,&
&  0.40653E+01_JPRB, 0.49306E+01_JPRB, 0.54328E+01_JPRB, 0.49096E+01_JPRB, 0.41794E+01_JPRB, 0.42355E+01_JPRB, 0.38912E+01_JPRB ,&
&  0.30557E+01_JPRB, 0.27774E+01_JPRB, 0.26733E+01_JPRB, 0.24478E+01_JPRB, 0.23231E+01_JPRB, 0.22150E+01_JPRB, 0.21247E+01_JPRB ,&
&  0.20426E+01_JPRB, 0.19833E+01_JPRB, 0.19219E+01_JPRB, 0.18505E+01_JPRB, 0.17881E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.17091E+01_JPRB, 0.16699E+01_JPRB, 0.16182E+01_JPRB, 0.15453E+01_JPRB, 0.14555E+01_JPRB, 0.13695E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.12162E+01_JPRB, 0.11561E+01_JPRB, 0.11005E+01_JPRB, 0.10456E+01_JPRB, 0.99914E+00_JPRB, 0.97320E+00_JPRB ,&
&  0.96050E+00_JPRB, 0.96828E+00_JPRB, 0.11531E+01_JPRB, 0.11126E+01_JPRB, 0.11475E+01_JPRB, 0.11429E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.14634E+01_JPRB, 0.18636E+01_JPRB, 0.23340E+01_JPRB, 0.26265E+01_JPRB, 0.29924E+01_JPRB, 0.31004E+01_JPRB, 0.29825E+01_JPRB ,&
&  0.26517E+01_JPRB, 0.23518E+01_JPRB, 0.20731E+01_JPRB, 0.18606E+01_JPRB, 0.16554E+01_JPRB, 0.15121E+01_JPRB, 0.14276E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.13609E+01_JPRB, 0.12799E+01_JPRB, 0.12677E+01_JPRB, 0.13495E+01_JPRB, 0.14526E+01_JPRB, 0.15226E+01_JPRB ,&
&  0.15519E+01_JPRB, 0.15731E+01_JPRB, 0.16274E+01_JPRB, 0.17278E+01_JPRB, 0.15580E+01_JPRB, 0.13806E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.17586E+01_JPRB, 0.20374E+01_JPRB, 0.20626E+01_JPRB, 0.24488E+01_JPRB, 0.24999E+01_JPRB, 0.25001E+01_JPRB, 0.27542E+01_JPRB ,&
&  0.27493E+01_JPRB, 0.28494E+01_JPRB, 0.21813E+01_JPRB, 0.22110E+01_JPRB, 0.26382E+01_JPRB, 0.22271E+01_JPRB, 0.22255E+01_JPRB ,&
&  0.24903E+01_JPRB, 0.22138E+01_JPRB, 0.19841E+01_JPRB, 0.18030E+01_JPRB, 0.25272E+01_JPRB, 0.24819E+01_JPRB, 0.12343E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.92162E+00_JPRB, 0.84306E+00_JPRB, 0.76582E+00_JPRB, 0.80730E+00_JPRB, 0.14968E+01_JPRB, 0.37711E+01_JPRB ,&
&  0.43999E+01_JPRB, 0.64441E+01_JPRB, 0.65102E+01_JPRB, 0.57868E+01_JPRB, 0.45945E+01_JPRB, 0.42894E+01_JPRB, 0.34623E+01_JPRB ,&
&  0.30240E+01_JPRB, 0.28092E+01_JPRB, 0.25535E+01_JPRB, 0.23431E+01_JPRB, 0.21397E+01_JPRB, 0.20133E+01_JPRB, 0.19490E+01_JPRB ,&
&  0.18949E+01_JPRB, 0.18513E+01_JPRB, 0.18193E+01_JPRB, 0.17844E+01_JPRB, 0.17510E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.15921E+01_JPRB, 0.15410E+01_JPRB, 0.14873E+01_JPRB, 0.14217E+01_JPRB, 0.13641E+01_JPRB, 0.13268E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.12471E+01_JPRB, 0.11885E+01_JPRB, 0.11224E+01_JPRB, 0.10590E+01_JPRB, 0.10078E+01_JPRB, 0.98203E+00_JPRB ,&
&  0.96530E+00_JPRB, 0.97086E+00_JPRB, 0.10733E+01_JPRB, 0.11840E+01_JPRB, 0.12509E+01_JPRB, 0.12145E+01_JPRB, 0.14172E+01_JPRB ,&
&  0.18131E+01_JPRB, 0.22794E+01_JPRB, 0.27182E+01_JPRB, 0.28708E+01_JPRB, 0.29912E+01_JPRB, 0.29466E+01_JPRB, 0.27341E+01_JPRB ,&
&  0.24797E+01_JPRB, 0.22951E+01_JPRB, 0.20844E+01_JPRB, 0.18786E+01_JPRB, 0.16892E+01_JPRB, 0.15640E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.13419E+01_JPRB, 0.12699E+01_JPRB, 0.12779E+01_JPRB, 0.13593E+01_JPRB, 0.14748E+01_JPRB, 0.15614E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.16164E+01_JPRB, 0.16679E+01_JPRB, 0.17555E+01_JPRB, 0.14644E+01_JPRB, 0.16829E+01_JPRB, 0.17265E+01_JPRB ,&
&  0.18118E+01_JPRB, 0.19343E+01_JPRB, 0.19395E+01_JPRB, 0.22673E+01_JPRB, 0.24158E+01_JPRB, 0.24220E+01_JPRB, 0.24690E+01_JPRB ,&
&  0.25805E+01_JPRB, 0.22513E+01_JPRB, 0.19816E+01_JPRB, 0.21706E+01_JPRB, 0.24525E+01_JPRB, 0.29406E+01_JPRB, 0.20236E+01_JPRB ,&
&  0.18360E+01_JPRB, 0.21660E+01_JPRB, 0.21084E+01_JPRB, 0.19869E+01_JPRB, 0.25321E+01_JPRB, 0.30093E+01_JPRB, 0.25172E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.88571E+00_JPRB, 0.93701E+00_JPRB, 0.96529E+00_JPRB, 0.95073E+00_JPRB, 0.16248E+01_JPRB, 0.60841E+01_JPRB ,&
&  0.54148E+01_JPRB, 0.71322E+01_JPRB, 0.66495E+01_JPRB, 0.55790E+01_JPRB, 0.45140E+01_JPRB, 0.39425E+01_JPRB, 0.34319E+01_JPRB ,&
&  0.28564E+01_JPRB, 0.25087E+01_JPRB, 0.22074E+01_JPRB, 0.20013E+01_JPRB, 0.18872E+01_JPRB, 0.18058E+01_JPRB, 0.17548E+01_JPRB ,&
&  0.17106E+01_JPRB, 0.16786E+01_JPRB, 0.16634E+01_JPRB, 0.16551E+01_JPRB, 0.16353E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.14764E+01_JPRB, 0.14175E+01_JPRB, 0.13597E+01_JPRB, 0.13293E+01_JPRB, 0.13133E+01_JPRB, 0.12928E+01_JPRB, 0.12796E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.12177E+01_JPRB, 0.11670E+01_JPRB, 0.11045E+01_JPRB, 0.10450E+01_JPRB, 0.10014E+01_JPRB, 0.97751E+00_JPRB ,&
&  0.97860E+00_JPRB, 0.98489E+00_JPRB, 0.10437E+01_JPRB, 0.11686E+01_JPRB, 0.13203E+01_JPRB, 0.12855E+01_JPRB, 0.14584E+01_JPRB ,&
&  0.23082E+01_JPRB, 0.29025E+01_JPRB, 0.28025E+01_JPRB, 0.28690E+01_JPRB, 0.29827E+01_JPRB, 0.31061E+01_JPRB, 0.27408E+01_JPRB ,&
&  0.24156E+01_JPRB, 0.21531E+01_JPRB, 0.18982E+01_JPRB, 0.17189E+01_JPRB, 0.15885E+01_JPRB, 0.15059E+01_JPRB, 0.14473E+01_JPRB ,&
&  0.13768E+01_JPRB, 0.13080E+01_JPRB, 0.12685E+01_JPRB, 0.12939E+01_JPRB, 0.13931E+01_JPRB, 0.14972E+01_JPRB, 0.15865E+01_JPRB ,&
&  0.16127E+01_JPRB, 0.16218E+01_JPRB, 0.16472E+01_JPRB, 0.16244E+01_JPRB, 0.16632E+01_JPRB, 0.17101E+01_JPRB, 0.17944E+01_JPRB ,&
&  0.18174E+01_JPRB, 0.19382E+01_JPRB, 0.20756E+01_JPRB, 0.21422E+01_JPRB, 0.22462E+01_JPRB, 0.23379E+01_JPRB, 0.22964E+01_JPRB ,&
&  0.22793E+01_JPRB, 0.20633E+01_JPRB, 0.19159E+01_JPRB, 0.21850E+01_JPRB, 0.24054E+01_JPRB, 0.26521E+01_JPRB, 0.25437E+01_JPRB ,&
&  0.22095E+01_JPRB, 0.18407E+01_JPRB, 0.18630E+01_JPRB, 0.21347E+01_JPRB, 0.24084E+01_JPRB, 0.26201E+01_JPRB, 0.37074E+01_JPRB ,&
&  0.36681E+01_JPRB, 0.24942E+01_JPRB, 0.21225E+01_JPRB, 0.27478E+01_JPRB, 0.23802E+01_JPRB, 0.24344E+01_JPRB, 0.38971E+01_JPRB ,&
&  0.53767E+01_JPRB, 0.68128E+01_JPRB, 0.62175E+01_JPRB, 0.48355E+01_JPRB, 0.42100E+01_JPRB, 0.35846E+01_JPRB, 0.29854E+01_JPRB ,&
&  0.24689E+01_JPRB, 0.22032E+01_JPRB, 0.20086E+01_JPRB, 0.18236E+01_JPRB, 0.16798E+01_JPRB, 0.16085E+01_JPRB, 0.15804E+01_JPRB ,&
&  0.15543E+01_JPRB, 0.15445E+01_JPRB, 0.15468E+01_JPRB, 0.15478E+01_JPRB, 0.15210E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.13438E+01_JPRB, 0.13037E+01_JPRB, 0.12877E+01_JPRB, 0.12574E+01_JPRB, 0.12243E+01_JPRB, 0.11988E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.11778E+01_JPRB, 0.11409E+01_JPRB, 0.11003E+01_JPRB, 0.10542E+01_JPRB, 0.10034E+01_JPRB, 0.97468E+00_JPRB, 0.96411E+00_JPRB ,&
&  0.97149E+00_JPRB, 0.99141E+00_JPRB, 0.10708E+01_JPRB, 0.12268E+01_JPRB, 0.14144E+01_JPRB, 0.17043E+01_JPRB, 0.14973E+01_JPRB ,&
&  0.21114E+01_JPRB, 0.31308E+01_JPRB, 0.31075E+01_JPRB, 0.30669E+01_JPRB, 0.29958E+01_JPRB, 0.29092E+01_JPRB, 0.25518E+01_JPRB ,&
&  0.22238E+01_JPRB, 0.19868E+01_JPRB, 0.17524E+01_JPRB, 0.16016E+01_JPRB, 0.15230E+01_JPRB, 0.14615E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.13148E+01_JPRB, 0.12663E+01_JPRB, 0.12507E+01_JPRB, 0.12896E+01_JPRB, 0.13619E+01_JPRB, 0.14632E+01_JPRB, 0.15389E+01_JPRB ,&
&  0.15766E+01_JPRB, 0.15905E+01_JPRB, 0.16058E+01_JPRB, 0.15376E+01_JPRB, 0.16132E+01_JPRB, 0.16797E+01_JPRB, 0.17427E+01_JPRB ,&
&  0.16817E+01_JPRB, 0.16222E+01_JPRB, 0.18708E+01_JPRB, 0.20911E+01_JPRB, 0.21770E+01_JPRB, 0.21011E+01_JPRB, 0.19940E+01_JPRB ,&
&  0.20903E+01_JPRB, 0.21196E+01_JPRB, 0.22969E+01_JPRB, 0.22781E+01_JPRB, 0.23938E+01_JPRB, 0.25240E+01_JPRB, 0.26257E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.23438E+01_JPRB, 0.22414E+01_JPRB, 0.22496E+01_JPRB, 0.22332E+01_JPRB, 0.24240E+01_JPRB, 0.32621E+01_JPRB ,&
&  0.41427E+01_JPRB, 0.47501E+01_JPRB, 0.49379E+01_JPRB, 0.42044E+01_JPRB, 0.36508E+01_JPRB, 0.34460E+01_JPRB, 0.38350E+01_JPRB ,&
&  0.55366E+01_JPRB, 0.59612E+01_JPRB, 0.51516E+01_JPRB, 0.48362E+01_JPRB, 0.36495E+01_JPRB, 0.32117E+01_JPRB, 0.27340E+01_JPRB ,&
&  0.23729E+01_JPRB, 0.20906E+01_JPRB, 0.18588E+01_JPRB, 0.16898E+01_JPRB, 0.15757E+01_JPRB, 0.14835E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.14101E+01_JPRB, 0.14216E+01_JPRB, 0.14358E+01_JPRB, 0.14279E+01_JPRB, 0.13885E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.12650E+01_JPRB, 0.12492E+01_JPRB, 0.12309E+01_JPRB, 0.11903E+01_JPRB, 0.11600E+01_JPRB, 0.11404E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.10648E+01_JPRB, 0.10305E+01_JPRB, 0.99212E+00_JPRB, 0.96243E+00_JPRB, 0.94165E+00_JPRB, 0.94665E+00_JPRB ,&
&  0.98909E+00_JPRB, 0.10707E+01_JPRB, 0.12172E+01_JPRB, 0.13643E+01_JPRB, 0.15694E+01_JPRB, 0.18885E+01_JPRB, 0.20519E+01_JPRB ,&
&  0.21902E+01_JPRB, 0.35257E+01_JPRB, 0.31062E+01_JPRB, 0.29452E+01_JPRB, 0.28017E+01_JPRB, 0.25968E+01_JPRB, 0.22588E+01_JPRB ,&
&  0.18992E+01_JPRB, 0.17371E+01_JPRB, 0.16255E+01_JPRB, 0.15203E+01_JPRB, 0.14611E+01_JPRB, 0.13521E+01_JPRB, 0.13065E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.12603E+01_JPRB, 0.12477E+01_JPRB, 0.12912E+01_JPRB, 0.13698E+01_JPRB, 0.14384E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.15420E+01_JPRB, 0.15432E+01_JPRB, 0.15035E+01_JPRB, 0.14915E+01_JPRB, 0.15155E+01_JPRB, 0.15840E+01_JPRB, 0.16539E+01_JPRB ,&
&  0.16496E+01_JPRB, 0.16241E+01_JPRB, 0.17784E+01_JPRB, 0.18268E+01_JPRB, 0.18171E+01_JPRB, 0.19082E+01_JPRB, 0.18003E+01_JPRB ,&
&  0.19909E+01_JPRB, 0.18917E+01_JPRB, 0.23771E+01_JPRB, 0.21057E+01_JPRB, 0.21662E+01_JPRB, 0.23351E+01_JPRB, 0.23005E+01_JPRB ,&
&  0.20862E+01_JPRB, 0.21097E+01_JPRB, 0.20569E+01_JPRB, 0.21168E+01_JPRB, 0.22939E+01_JPRB, 0.26477E+01_JPRB, 0.33829E+01_JPRB ,&
&  0.37060E+01_JPRB, 0.36925E+01_JPRB, 0.38077E+01_JPRB, 0.32267E+01_JPRB, 0.31348E+01_JPRB, 0.30995E+01_JPRB, 0.35575E+01_JPRB ,&
&  0.50838E+01_JPRB, 0.51777E+01_JPRB, 0.46518E+01_JPRB, 0.39298E+01_JPRB, 0.30678E+01_JPRB, 0.26916E+01_JPRB, 0.22946E+01_JPRB ,&
&  0.20214E+01_JPRB, 0.17832E+01_JPRB, 0.16413E+01_JPRB, 0.15307E+01_JPRB, 0.14483E+01_JPRB, 0.13698E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.13256E+01_JPRB, 0.13284E+01_JPRB, 0.13112E+01_JPRB, 0.12862E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.12509E+01_JPRB, 0.12579E+01_JPRB, 0.12604E+01_JPRB, 0.12853E+01_JPRB, 0.13319E+01_JPRB, 0.13987E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.10003E+01_JPRB, 0.97936E+00_JPRB, 0.95678E+00_JPRB, 0.93293E+00_JPRB, 0.93601E+00_JPRB, 0.98555E+00_JPRB ,&
&  0.10659E+01_JPRB, 0.11759E+01_JPRB, 0.13044E+01_JPRB, 0.14927E+01_JPRB, 0.17466E+01_JPRB, 0.21887E+01_JPRB, 0.26086E+01_JPRB ,&
&  0.25667E+01_JPRB, 0.28345E+01_JPRB, 0.30311E+01_JPRB, 0.26652E+01_JPRB, 0.24145E+01_JPRB, 0.22575E+01_JPRB, 0.20635E+01_JPRB ,&
&  0.17733E+01_JPRB, 0.16269E+01_JPRB, 0.15824E+01_JPRB, 0.15487E+01_JPRB, 0.13192E+01_JPRB, 0.12808E+01_JPRB, 0.12703E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.12779E+01_JPRB, 0.12894E+01_JPRB, 0.13477E+01_JPRB, 0.14108E+01_JPRB, 0.14644E+01_JPRB, 0.15085E+01_JPRB ,&
&  0.15329E+01_JPRB, 0.15270E+01_JPRB, 0.15092E+01_JPRB, 0.14236E+01_JPRB, 0.14423E+01_JPRB, 0.14844E+01_JPRB, 0.15286E+01_JPRB ,&
&  0.15674E+01_JPRB, 0.16529E+01_JPRB, 0.17028E+01_JPRB, 0.16742E+01_JPRB, 0.15853E+01_JPRB, 0.17710E+01_JPRB, 0.17659E+01_JPRB ,&
&  0.18840E+01_JPRB, 0.19806E+01_JPRB, 0.25067E+01_JPRB, 0.18877E+01_JPRB, 0.19845E+01_JPRB, 0.21171E+01_JPRB, 0.21002E+01_JPRB ,&
&  0.19528E+01_JPRB, 0.19532E+01_JPRB, 0.19020E+01_JPRB, 0.19709E+01_JPRB, 0.23545E+01_JPRB, 0.28093E+01_JPRB, 0.37197E+01_JPRB ,&
&  0.32161E+01_JPRB, 0.33347E+01_JPRB, 0.29291E+01_JPRB, 0.29669E+01_JPRB, 0.29182E+01_JPRB, 0.28724E+01_JPRB, 0.30186E+01_JPRB ,&
&  0.39712E+01_JPRB, 0.36636E+01_JPRB, 0.31967E+01_JPRB, 0.27082E+01_JPRB, 0.21779E+01_JPRB, 0.18908E+01_JPRB, 0.16239E+01_JPRB ,&
&  0.14851E+01_JPRB, 0.14142E+01_JPRB, 0.14015E+01_JPRB, 0.13600E+01_JPRB, 0.13329E+01_JPRB, 0.13036E+01_JPRB, 0.12923E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12697E+01_JPRB, 0.12587E+01_JPRB, 0.12497E+01_JPRB, 0.12436E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.12793E+01_JPRB, 0.12671E+01_JPRB, 0.12581E+01_JPRB, 0.12532E+01_JPRB, 0.12458E+01_JPRB, 0.12146E+01_JPRB, 0.11491E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.98962E+00_JPRB, 0.97170E+00_JPRB, 0.95916E+00_JPRB, 0.96015E+00_JPRB, 0.99021E+00_JPRB, 0.10517E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.12475E+01_JPRB, 0.14121E+01_JPRB, 0.16252E+01_JPRB, 0.18952E+01_JPRB, 0.23256E+01_JPRB, 0.28281E+01_JPRB ,&
&  0.28353E+01_JPRB, 0.21228E+01_JPRB, 0.24415E+01_JPRB, 0.23682E+01_JPRB, 0.21912E+01_JPRB, 0.20160E+01_JPRB, 0.17673E+01_JPRB ,&
&  0.15435E+01_JPRB, 0.14746E+01_JPRB, 0.14399E+01_JPRB, 0.13718E+01_JPRB, 0.12658E+01_JPRB, 0.12811E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.13535E+01_JPRB, 0.13603E+01_JPRB, 0.14132E+01_JPRB, 0.14513E+01_JPRB, 0.15036E+01_JPRB, 0.15689E+01_JPRB ,&
&  0.15973E+01_JPRB, 0.16006E+01_JPRB, 0.15884E+01_JPRB, 0.15018E+01_JPRB, 0.14900E+01_JPRB, 0.15218E+01_JPRB, 0.15406E+01_JPRB ,&
&  0.15631E+01_JPRB, 0.15546E+01_JPRB, 0.16139E+01_JPRB, 0.17171E+01_JPRB, 0.18311E+01_JPRB, 0.16933E+01_JPRB, 0.18654E+01_JPRB ,&
&  0.19537E+01_JPRB, 0.21290E+01_JPRB, 0.20599E+01_JPRB, 0.23046E+01_JPRB, 0.16201E+01_JPRB, 0.18002E+01_JPRB, 0.17300E+01_JPRB ,&
&  0.18817E+01_JPRB, 0.18133E+01_JPRB, 0.18782E+01_JPRB, 0.21341E+01_JPRB, 0.26560E+01_JPRB, 0.32610E+01_JPRB, 0.34488E+01_JPRB ,&
&  0.26091E+01_JPRB, 0.24329E+01_JPRB, 0.26028E+01_JPRB, 0.29384E+01_JPRB, 0.29990E+01_JPRB, 0.29609E+01_JPRB, 0.26919E+01_JPRB ,&
&  0.28841E+01_JPRB, 0.25182E+01_JPRB, 0.22262E+01_JPRB, 0.19121E+01_JPRB, 0.14964E+01_JPRB, 0.13193E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.13002E+01_JPRB, 0.13066E+01_JPRB, 0.13086E+01_JPRB, 0.12908E+01_JPRB, 0.12720E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.12557E+01_JPRB, 0.12565E+01_JPRB, 0.12574E+01_JPRB, 0.12684E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.12286E+01_JPRB, 0.12022E+01_JPRB, 0.11780E+01_JPRB, 0.11555E+01_JPRB, 0.11387E+01_JPRB, 0.10953E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10060E+01_JPRB, 0.98643E+00_JPRB, 0.98257E+00_JPRB, 0.99000E+00_JPRB, 0.10172E+01_JPRB, 0.10477E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.13313E+01_JPRB, 0.15065E+01_JPRB, 0.17169E+01_JPRB, 0.19802E+01_JPRB, 0.23012E+01_JPRB, 0.25393E+01_JPRB ,&
&  0.27723E+01_JPRB, 0.28102E+01_JPRB, 0.24094E+01_JPRB, 0.17455E+01_JPRB, 0.16219E+01_JPRB, 0.16461E+01_JPRB, 0.14686E+01_JPRB ,&
&  0.14271E+01_JPRB, 0.13834E+01_JPRB, 0.13433E+01_JPRB, 0.13252E+01_JPRB, 0.13265E+01_JPRB, 0.13547E+01_JPRB, 0.13871E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.14327E+01_JPRB, 0.14597E+01_JPRB, 0.15213E+01_JPRB, 0.15711E+01_JPRB, 0.16292E+01_JPRB, 0.16792E+01_JPRB ,&
&  0.17063E+01_JPRB, 0.17347E+01_JPRB, 0.17131E+01_JPRB, 0.16741E+01_JPRB, 0.16249E+01_JPRB, 0.16475E+01_JPRB, 0.16789E+01_JPRB ,&
&  0.17101E+01_JPRB, 0.17102E+01_JPRB, 0.17237E+01_JPRB, 0.18380E+01_JPRB, 0.19247E+01_JPRB, 0.17611E+01_JPRB, 0.18244E+01_JPRB ,&
&  0.20544E+01_JPRB, 0.21977E+01_JPRB, 0.17671E+01_JPRB, 0.20546E+01_JPRB, 0.13247E+01_JPRB, 0.16497E+01_JPRB, 0.18497E+01_JPRB ,&
&  0.18208E+01_JPRB, 0.18973E+01_JPRB, 0.20188E+01_JPRB, 0.23593E+01_JPRB, 0.28525E+01_JPRB, 0.31866E+01_JPRB, 0.28678E+01_JPRB ,&
&  0.21885E+01_JPRB, 0.21559E+01_JPRB, 0.23450E+01_JPRB, 0.25379E+01_JPRB, 0.25954E+01_JPRB, 0.24349E+01_JPRB, 0.19319E+01_JPRB ,&
&  0.15842E+01_JPRB, 0.18196E+01_JPRB, 0.17615E+01_JPRB, 0.17081E+01_JPRB, 0.13223E+01_JPRB, 0.11467E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.12615E+01_JPRB, 0.13038E+01_JPRB, 0.13195E+01_JPRB, 0.13108E+01_JPRB, 0.12883E+01_JPRB, 0.12690E+01_JPRB, 0.12645E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.12535E+01_JPRB, 0.12515E+01_JPRB, 0.12547E+01_JPRB, 0.12492E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.11207E+01_JPRB, 0.10853E+01_JPRB, 0.10628E+01_JPRB, 0.10465E+01_JPRB, 0.10403E+01_JPRB, 0.10141E+01_JPRB, 0.97443E+00_JPRB ,&
&  0.95031E+00_JPRB, 0.94050E+00_JPRB, 0.95177E+00_JPRB, 0.98496E+00_JPRB, 0.10315E+01_JPRB, 0.10755E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.14028E+01_JPRB, 0.15878E+01_JPRB, 0.18587E+01_JPRB, 0.20830E+01_JPRB, 0.22970E+01_JPRB, 0.23541E+01_JPRB ,&
&  0.24588E+01_JPRB, 0.24711E+01_JPRB, 0.23179E+01_JPRB, 0.21325E+01_JPRB, 0.16882E+01_JPRB, 0.16614E+01_JPRB, 0.17701E+01_JPRB ,&
&  0.18296E+01_JPRB, 0.15722E+01_JPRB, 0.14072E+01_JPRB, 0.14034E+01_JPRB, 0.14023E+01_JPRB, 0.14444E+01_JPRB, 0.14858E+01_JPRB ,&
&  0.15245E+01_JPRB, 0.15601E+01_JPRB, 0.15939E+01_JPRB, 0.16490E+01_JPRB, 0.17101E+01_JPRB, 0.17382E+01_JPRB, 0.17749E+01_JPRB ,&
&  0.17828E+01_JPRB, 0.17978E+01_JPRB, 0.18009E+01_JPRB, 0.17787E+01_JPRB, 0.17939E+01_JPRB, 0.18403E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.19964E+01_JPRB, 0.19389E+01_JPRB, 0.20402E+01_JPRB, 0.21815E+01_JPRB, 0.21682E+01_JPRB, 0.20932E+01_JPRB, 0.21035E+01_JPRB ,&
&  0.22703E+01_JPRB, 0.24235E+01_JPRB, 0.15249E+01_JPRB, 0.17214E+01_JPRB, 0.20300E+01_JPRB, 0.18356E+01_JPRB, 0.17747E+01_JPRB ,&
&  0.18730E+01_JPRB, 0.19983E+01_JPRB, 0.22000E+01_JPRB, 0.24455E+01_JPRB, 0.28628E+01_JPRB, 0.36108E+01_JPRB, 0.25618E+01_JPRB ,&
&  0.18589E+01_JPRB, 0.18055E+01_JPRB, 0.19155E+01_JPRB, 0.20022E+01_JPRB, 0.20085E+01_JPRB, 0.17906E+01_JPRB, 0.13886E+01_JPRB ,&
&  0.12381E+01_JPRB, 0.16494E+01_JPRB, 0.17262E+01_JPRB, 0.18213E+01_JPRB, 0.16138E+01_JPRB, 0.10947E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.12548E+01_JPRB, 0.12834E+01_JPRB, 0.12651E+01_JPRB, 0.12602E+01_JPRB, 0.12563E+01_JPRB, 0.12479E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.12433E+01_JPRB, 0.12415E+01_JPRB, 0.12415E+01_JPRB, 0.12119E+01_JPRB, 0.11609E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.10024E+01_JPRB, 0.97438E+00_JPRB, 0.95941E+00_JPRB, 0.93807E+00_JPRB, 0.92436E+00_JPRB, 0.90583E+00_JPRB, 0.89484E+00_JPRB ,&
&  0.89128E+00_JPRB, 0.89415E+00_JPRB, 0.91431E+00_JPRB, 0.94811E+00_JPRB, 0.99468E+00_JPRB, 0.10566E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.14457E+01_JPRB, 0.16545E+01_JPRB, 0.19010E+01_JPRB, 0.20822E+01_JPRB, 0.21405E+01_JPRB, 0.22509E+01_JPRB ,&
&  0.23222E+01_JPRB, 0.23387E+01_JPRB, 0.22870E+01_JPRB, 0.22078E+01_JPRB, 0.20974E+01_JPRB, 0.20877E+01_JPRB, 0.21830E+01_JPRB ,&
&  0.19849E+01_JPRB, 0.15609E+01_JPRB, 0.14779E+01_JPRB, 0.14674E+01_JPRB, 0.14928E+01_JPRB, 0.15278E+01_JPRB, 0.15772E+01_JPRB ,&
&  0.16169E+01_JPRB, 0.16456E+01_JPRB, 0.16712E+01_JPRB, 0.17035E+01_JPRB, 0.17790E+01_JPRB, 0.18143E+01_JPRB, 0.18572E+01_JPRB ,&
&  0.18907E+01_JPRB, 0.19249E+01_JPRB, 0.19895E+01_JPRB, 0.18781E+01_JPRB, 0.20056E+01_JPRB, 0.21068E+01_JPRB, 0.22010E+01_JPRB ,&
&  0.22156E+01_JPRB, 0.21737E+01_JPRB, 0.23159E+01_JPRB, 0.23933E+01_JPRB, 0.24612E+01_JPRB, 0.24877E+01_JPRB, 0.24498E+01_JPRB ,&
&  0.25355E+01_JPRB, 0.23624E+01_JPRB, 0.14841E+01_JPRB, 0.14185E+01_JPRB, 0.15711E+01_JPRB, 0.14894E+01_JPRB, 0.17980E+01_JPRB ,&
&  0.18996E+01_JPRB, 0.20460E+01_JPRB, 0.22017E+01_JPRB, 0.24261E+01_JPRB, 0.27077E+01_JPRB, 0.33297E+01_JPRB, 0.23211E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.13965E+01_JPRB, 0.14277E+01_JPRB, 0.14667E+01_JPRB, 0.15243E+01_JPRB, 0.14467E+01_JPRB, 0.13187E+01_JPRB ,&
&  0.13005E+01_JPRB, 0.13624E+01_JPRB, 0.13925E+01_JPRB, 0.12976E+01_JPRB, 0.10862E+01_JPRB, 0.10299E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11600E+01_JPRB, 0.11606E+01_JPRB, 0.11714E+01_JPRB, 0.11673E+01_JPRB, 0.11628E+01_JPRB, 0.11579E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11364E+01_JPRB, 0.11144E+01_JPRB, 0.10676E+01_JPRB, 0.10295E+01_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.85746E+00_JPRB, 0.85788E+00_JPRB, 0.84562E+00_JPRB, 0.83021E+00_JPRB, 0.82186E+00_JPRB, 0.80220E+00_JPRB, 0.82321E+00_JPRB ,&
&  0.81974E+00_JPRB, 0.84137E+00_JPRB, 0.86461E+00_JPRB, 0.88847E+00_JPRB, 0.93363E+00_JPRB, 0.97993E+00_JPRB, 0.10469E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.13541E+01_JPRB, 0.16003E+01_JPRB, 0.19089E+01_JPRB, 0.20121E+01_JPRB, 0.20767E+01_JPRB, 0.21890E+01_JPRB ,&
&  0.23283E+01_JPRB, 0.24379E+01_JPRB, 0.24921E+01_JPRB, 0.25645E+01_JPRB, 0.26296E+01_JPRB, 0.25568E+01_JPRB, 0.24529E+01_JPRB ,&
&  0.19996E+01_JPRB, 0.14768E+01_JPRB, 0.15488E+01_JPRB, 0.14578E+01_JPRB, 0.15171E+01_JPRB, 0.15778E+01_JPRB, 0.16213E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.16852E+01_JPRB, 0.17172E+01_JPRB, 0.17623E+01_JPRB, 0.18301E+01_JPRB, 0.18921E+01_JPRB, 0.19553E+01_JPRB ,&
&  0.19992E+01_JPRB, 0.20405E+01_JPRB, 0.20712E+01_JPRB, 0.20167E+01_JPRB, 0.20924E+01_JPRB, 0.22467E+01_JPRB, 0.23754E+01_JPRB ,&
&  0.24763E+01_JPRB, 0.24649E+01_JPRB, 0.22321E+01_JPRB, 0.22725E+01_JPRB, 0.24242E+01_JPRB, 0.23458E+01_JPRB, 0.24538E+01_JPRB ,&
&  0.26868E+01_JPRB, 0.24246E+01_JPRB, 0.14222E+01_JPRB, 0.13411E+01_JPRB, 0.14916E+01_JPRB, 0.16164E+01_JPRB, 0.17285E+01_JPRB ,&
&  0.18133E+01_JPRB, 0.18890E+01_JPRB, 0.20008E+01_JPRB, 0.21335E+01_JPRB, 0.21872E+01_JPRB, 0.22694E+01_JPRB, 0.18940E+01_JPRB ,&
&  0.12174E+01_JPRB, 0.11907E+01_JPRB, 0.12623E+01_JPRB, 0.13327E+01_JPRB, 0.14897E+01_JPRB, 0.14547E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.12987E+01_JPRB, 0.14789E+01_JPRB, 0.17332E+01_JPRB, 0.14949E+01_JPRB, 0.10134E+01_JPRB, 0.93871E+00_JPRB, 0.94145E+00_JPRB ,&
&  0.97900E+00_JPRB, 0.98417E+00_JPRB, 0.10050E+01_JPRB, 0.10198E+01_JPRB, 0.10393E+01_JPRB, 0.10253E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.98303E+00_JPRB, 0.96100E+00_JPRB, 0.93565E+00_JPRB, 0.90017E+00_JPRB, 0.87284E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.73416E+00_JPRB, 0.69849E+00_JPRB, 0.66202E+00_JPRB, 0.65804E+00_JPRB, 0.66086E+00_JPRB, 0.64602E+00_JPRB, 0.67980E+00_JPRB ,&
&  0.65440E+00_JPRB, 0.68123E+00_JPRB, 0.69448E+00_JPRB, 0.69436E+00_JPRB, 0.73675E+00_JPRB, 0.77748E+00_JPRB, 0.83069E+00_JPRB ,&
&  0.90529E+00_JPRB, 0.10253E+01_JPRB, 0.11774E+01_JPRB, 0.13668E+01_JPRB, 0.15962E+01_JPRB, 0.17056E+01_JPRB, 0.18160E+01_JPRB ,&
&  0.19713E+01_JPRB, 0.21782E+01_JPRB, 0.24734E+01_JPRB, 0.28107E+01_JPRB, 0.32043E+01_JPRB, 0.30278E+01_JPRB, 0.25766E+01_JPRB ,&
&  0.18963E+01_JPRB, 0.14575E+01_JPRB, 0.13754E+01_JPRB, 0.12833E+01_JPRB, 0.13547E+01_JPRB, 0.15055E+01_JPRB, 0.15681E+01_JPRB ,&
&  0.16104E+01_JPRB, 0.16542E+01_JPRB, 0.16991E+01_JPRB, 0.17534E+01_JPRB, 0.18199E+01_JPRB, 0.18946E+01_JPRB, 0.19322E+01_JPRB ,&
&  0.19542E+01_JPRB, 0.19793E+01_JPRB, 0.20026E+01_JPRB, 0.21021E+01_JPRB, 0.22439E+01_JPRB, 0.24190E+01_JPRB, 0.24368E+01_JPRB ,&
&  0.24336E+01_JPRB, 0.23459E+01_JPRB, 0.20649E+01_JPRB, 0.20266E+01_JPRB, 0.20897E+01_JPRB, 0.21348E+01_JPRB, 0.22062E+01_JPRB ,&
&  0.24078E+01_JPRB, 0.20536E+01_JPRB, 0.14677E+01_JPRB, 0.15365E+01_JPRB, 0.16166E+01_JPRB, 0.17173E+01_JPRB, 0.15934E+01_JPRB ,&
&  0.16117E+01_JPRB, 0.16312E+01_JPRB, 0.16892E+01_JPRB, 0.16914E+01_JPRB, 0.17077E+01_JPRB, 0.16776E+01_JPRB, 0.15228E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.12008E+01_JPRB, 0.11663E+01_JPRB, 0.11768E+01_JPRB, 0.14348E+01_JPRB, 0.13983E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.13322E+01_JPRB, 0.11789E+01_JPRB, 0.14229E+01_JPRB, 0.91908E+00_JPRB, 0.81936E+00_JPRB, 0.80363E+00_JPRB ,&
&  0.82914E+00_JPRB, 0.83842E+00_JPRB, 0.85499E+00_JPRB, 0.84462E+00_JPRB, 0.85982E+00_JPRB, 0.89165E+00_JPRB, 0.89626E+00_JPRB ,&
&  0.84887E+00_JPRB, 0.83223E+00_JPRB, 0.79791E+00_JPRB, 0.75917E+00_JPRB, 0.74816E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.61446E+00_JPRB, 0.60349E+00_JPRB, 0.59818E+00_JPRB, 0.59261E+00_JPRB, 0.60108E+00_JPRB, 0.60393E+00_JPRB, 0.60631E+00_JPRB ,&
&  0.60689E+00_JPRB, 0.61913E+00_JPRB, 0.62826E+00_JPRB, 0.64126E+00_JPRB, 0.65558E+00_JPRB, 0.68795E+00_JPRB, 0.72568E+00_JPRB ,&
&  0.77125E+00_JPRB, 0.82353E+00_JPRB, 0.88159E+00_JPRB, 0.96515E+00_JPRB, 0.10831E+01_JPRB, 0.11662E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.15549E+01_JPRB, 0.17535E+01_JPRB, 0.20201E+01_JPRB, 0.23247E+01_JPRB, 0.23830E+01_JPRB, 0.22295E+01_JPRB ,&
&  0.16360E+01_JPRB, 0.15234E+01_JPRB, 0.14073E+01_JPRB, 0.12834E+01_JPRB, 0.12990E+01_JPRB, 0.12663E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.13268E+01_JPRB, 0.13141E+01_JPRB, 0.13244E+01_JPRB, 0.13336E+01_JPRB, 0.13433E+01_JPRB, 0.13279E+01_JPRB ,&
&  0.13220E+01_JPRB, 0.13676E+01_JPRB, 0.15695E+01_JPRB, 0.18328E+01_JPRB, 0.20187E+01_JPRB, 0.20901E+01_JPRB, 0.19759E+01_JPRB ,&
&  0.18265E+01_JPRB, 0.17121E+01_JPRB, 0.16057E+01_JPRB, 0.15778E+01_JPRB, 0.16389E+01_JPRB, 0.16855E+01_JPRB, 0.18265E+01_JPRB ,&
&  0.17615E+01_JPRB, 0.15937E+01_JPRB, 0.16073E+01_JPRB, 0.16386E+01_JPRB, 0.16530E+01_JPRB, 0.16207E+01_JPRB, 0.15262E+01_JPRB ,&
&  0.15334E+01_JPRB, 0.14724E+01_JPRB, 0.14555E+01_JPRB, 0.13842E+01_JPRB, 0.12929E+01_JPRB, 0.12088E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.10867E+01_JPRB, 0.10313E+01_JPRB, 0.10336E+01_JPRB, 0.11413E+01_JPRB, 0.13589E+01_JPRB, 0.21293E+01_JPRB, 0.19163E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.98113E+00_JPRB, 0.72303E+00_JPRB, 0.94956E+00_JPRB, 0.75608E+00_JPRB, 0.71432E+00_JPRB, 0.71225E+00_JPRB ,&
&  0.71751E+00_JPRB, 0.71319E+00_JPRB, 0.69284E+00_JPRB, 0.68887E+00_JPRB, 0.69888E+00_JPRB, 0.70832E+00_JPRB, 0.69895E+00_JPRB ,&
&  0.66128E+00_JPRB, 0.64426E+00_JPRB, 0.67167E+00_JPRB, 0.62819E+00_JPRB, 0.61755E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.55476E+00_JPRB, 0.55270E+00_JPRB, 0.56565E+00_JPRB, 0.57813E+00_JPRB, 0.58534E+00_JPRB, 0.59177E+00_JPRB, 0.60241E+00_JPRB ,&
&  0.60924E+00_JPRB, 0.61846E+00_JPRB, 0.62838E+00_JPRB, 0.64631E+00_JPRB, 0.67250E+00_JPRB, 0.71242E+00_JPRB, 0.75814E+00_JPRB ,&
&  0.81433E+00_JPRB, 0.87475E+00_JPRB, 0.94240E+00_JPRB, 0.10155E+01_JPRB, 0.11062E+01_JPRB, 0.12321E+01_JPRB, 0.14048E+01_JPRB ,&
&  0.15015E+01_JPRB, 0.16232E+01_JPRB, 0.17718E+01_JPRB, 0.18998E+01_JPRB, 0.19203E+01_JPRB, 0.18733E+01_JPRB, 0.15333E+01_JPRB ,&
&  0.17006E+01_JPRB, 0.15178E+01_JPRB, 0.13614E+01_JPRB, 0.12314E+01_JPRB, 0.11424E+01_JPRB, 0.10662E+01_JPRB, 0.10088E+01_JPRB ,&
&  0.98918E+00_JPRB, 0.94429E+00_JPRB, 0.88936E+00_JPRB, 0.85061E+00_JPRB, 0.82482E+00_JPRB, 0.84001E+00_JPRB, 0.82013E+00_JPRB ,&
&  0.88986E+00_JPRB, 0.10476E+01_JPRB, 0.12481E+01_JPRB, 0.14965E+01_JPRB, 0.16337E+01_JPRB, 0.16016E+01_JPRB, 0.13550E+01_JPRB ,&
&  0.12074E+01_JPRB, 0.11286E+01_JPRB, 0.11043E+01_JPRB, 0.11594E+01_JPRB, 0.11971E+01_JPRB, 0.12718E+01_JPRB, 0.13133E+01_JPRB ,&
&  0.13065E+01_JPRB, 0.12646E+01_JPRB, 0.13672E+01_JPRB, 0.17309E+01_JPRB, 0.16987E+01_JPRB, 0.15569E+01_JPRB, 0.15191E+01_JPRB ,&
&  0.14292E+01_JPRB, 0.13692E+01_JPRB, 0.13393E+01_JPRB, 0.12628E+01_JPRB, 0.11602E+01_JPRB, 0.10617E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.97505E+00_JPRB, 0.91263E+00_JPRB, 0.88695E+00_JPRB, 0.95915E+00_JPRB, 0.11966E+01_JPRB, 0.14657E+01_JPRB, 0.20643E+01_JPRB ,&
&  0.11572E+01_JPRB, 0.71412E+00_JPRB, 0.72590E+00_JPRB, 0.74113E+00_JPRB, 0.67456E+00_JPRB, 0.64080E+00_JPRB, 0.69404E+00_JPRB ,&
&  0.76018E+00_JPRB, 0.82735E+00_JPRB, 0.82094E+00_JPRB, 0.71099E+00_JPRB, 0.66772E+00_JPRB, 0.64793E+00_JPRB, 0.62333E+00_JPRB ,&
&  0.60282E+00_JPRB, 0.56503E+00_JPRB, 0.57857E+00_JPRB, 0.56114E+00_JPRB, 0.55872E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.51555E+00_JPRB, 0.51283E+00_JPRB, 0.52026E+00_JPRB, 0.52801E+00_JPRB, 0.52982E+00_JPRB, 0.53183E+00_JPRB, 0.53374E+00_JPRB ,&
&  0.53711E+00_JPRB, 0.53763E+00_JPRB, 0.54039E+00_JPRB, 0.54823E+00_JPRB, 0.56495E+00_JPRB, 0.60092E+00_JPRB, 0.64293E+00_JPRB ,&
&  0.68030E+00_JPRB, 0.71549E+00_JPRB, 0.75014E+00_JPRB, 0.77981E+00_JPRB, 0.81073E+00_JPRB, 0.83112E+00_JPRB, 0.88786E+00_JPRB ,&
&  0.96040E+00_JPRB, 0.10473E+01_JPRB, 0.11357E+01_JPRB, 0.12002E+01_JPRB, 0.12573E+01_JPRB, 0.12309E+01_JPRB, 0.10183E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.13817E+01_JPRB, 0.12461E+01_JPRB, 0.11203E+01_JPRB, 0.10332E+01_JPRB, 0.95465E+00_JPRB, 0.88177E+00_JPRB ,&
&  0.81806E+00_JPRB, 0.85305E+00_JPRB, 0.81346E+00_JPRB, 0.85531E+00_JPRB, 0.75085E+00_JPRB, 0.78220E+00_JPRB, 0.90092E+00_JPRB ,&
&  0.99764E+00_JPRB, 0.10869E+01_JPRB, 0.11901E+01_JPRB, 0.13045E+01_JPRB, 0.13659E+01_JPRB, 0.13329E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.10787E+01_JPRB, 0.10297E+01_JPRB, 0.10945E+01_JPRB, 0.10941E+01_JPRB, 0.10717E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.98843E+00_JPRB, 0.10515E+01_JPRB, 0.12493E+01_JPRB, 0.16567E+01_JPRB, 0.16256E+01_JPRB, 0.15548E+01_JPRB, 0.14971E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.13297E+01_JPRB, 0.12740E+01_JPRB, 0.11908E+01_JPRB, 0.10972E+01_JPRB, 0.10094E+01_JPRB, 0.94889E+00_JPRB ,&
&  0.90572E+00_JPRB, 0.84920E+00_JPRB, 0.81157E+00_JPRB, 0.81485E+00_JPRB, 0.89644E+00_JPRB, 0.10177E+01_JPRB, 0.17153E+01_JPRB ,&
&  0.11440E+01_JPRB, 0.90783E+00_JPRB, 0.98044E+00_JPRB, 0.47326E+00_JPRB, 0.55567E+00_JPRB, 0.58100E+00_JPRB, 0.69065E+00_JPRB ,&
&  0.84934E+00_JPRB, 0.13444E+01_JPRB, 0.20576E+01_JPRB, 0.18472E+01_JPRB, 0.14137E+01_JPRB, 0.91703E+00_JPRB, 0.56033E+00_JPRB ,&
&  0.53931E+00_JPRB, 0.53351E+00_JPRB, 0.53722E+00_JPRB, 0.52841E+00_JPRB, 0.52448E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.46537E+00_JPRB, 0.45574E+00_JPRB, 0.46380E+00_JPRB, 0.47356E+00_JPRB, 0.46982E+00_JPRB, 0.46960E+00_JPRB, 0.46469E+00_JPRB ,&
&  0.45541E+00_JPRB, 0.44915E+00_JPRB, 0.47140E+00_JPRB, 0.51379E+00_JPRB, 0.57290E+00_JPRB, 0.64997E+00_JPRB, 0.72635E+00_JPRB ,&
&  0.77747E+00_JPRB, 0.83661E+00_JPRB, 0.88824E+00_JPRB, 0.92987E+00_JPRB, 0.93511E+00_JPRB, 0.93753E+00_JPRB, 0.92280E+00_JPRB ,&
&  0.91387E+00_JPRB, 0.93021E+00_JPRB, 0.98069E+00_JPRB, 0.10291E+01_JPRB, 0.11156E+01_JPRB, 0.12449E+01_JPRB, 0.81008E+00_JPRB ,&
&  0.11907E+01_JPRB, 0.11796E+01_JPRB, 0.11202E+01_JPRB, 0.10229E+01_JPRB, 0.97303E+00_JPRB, 0.89726E+00_JPRB, 0.83771E+00_JPRB ,&
&  0.84306E+00_JPRB, 0.88660E+00_JPRB, 0.89110E+00_JPRB, 0.97108E+00_JPRB, 0.10496E+01_JPRB, 0.10685E+01_JPRB, 0.11950E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.13949E+01_JPRB, 0.15039E+01_JPRB, 0.15825E+01_JPRB, 0.16492E+01_JPRB, 0.16695E+01_JPRB, 0.16454E+01_JPRB ,&
&  0.15405E+01_JPRB, 0.14677E+01_JPRB, 0.14835E+01_JPRB, 0.15587E+01_JPRB, 0.14159E+01_JPRB, 0.13495E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.96989E+00_JPRB, 0.95025E+00_JPRB, 0.11085E+01_JPRB, 0.14611E+01_JPRB, 0.15336E+01_JPRB, 0.14776E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.12762E+01_JPRB, 0.11916E+01_JPRB, 0.11244E+01_JPRB, 0.10547E+01_JPRB, 0.98736E+00_JPRB, 0.92639E+00_JPRB, 0.86807E+00_JPRB ,&
&  0.83683E+00_JPRB, 0.79785E+00_JPRB, 0.75445E+00_JPRB, 0.76525E+00_JPRB, 0.82989E+00_JPRB, 0.10228E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.12470E+01_JPRB, 0.89785E+00_JPRB, 0.75985E+00_JPRB, 0.71391E+00_JPRB, 0.55498E+00_JPRB, 0.57473E+00_JPRB, 0.73735E+00_JPRB ,&
&  0.11492E+01_JPRB, 0.16826E+01_JPRB, 0.28220E+01_JPRB, 0.50487E+01_JPRB, 0.27977E+01_JPRB, 0.21621E+01_JPRB, 0.53850E+00_JPRB ,&
&  0.51620E+00_JPRB, 0.51621E+00_JPRB, 0.51216E+00_JPRB, 0.49495E+00_JPRB, 0.48957E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.45020E+00_JPRB, 0.44169E+00_JPRB, 0.45014E+00_JPRB, 0.44843E+00_JPRB, 0.45216E+00_JPRB, 0.45952E+00_JPRB, 0.46401E+00_JPRB ,&
&  0.46781E+00_JPRB, 0.48441E+00_JPRB, 0.57496E+00_JPRB, 0.70020E+00_JPRB, 0.81978E+00_JPRB, 0.91144E+00_JPRB, 0.95386E+00_JPRB ,&
&  0.99655E+00_JPRB, 0.10182E+01_JPRB, 0.10135E+01_JPRB, 0.10027E+01_JPRB, 0.10193E+01_JPRB, 0.10700E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.11853E+01_JPRB, 0.12261E+01_JPRB, 0.12898E+01_JPRB, 0.13505E+01_JPRB, 0.13000E+01_JPRB, 0.80452E+00_JPRB ,&
&  0.10409E+01_JPRB, 0.10835E+01_JPRB, 0.10877E+01_JPRB, 0.10102E+01_JPRB, 0.93877E+00_JPRB, 0.90127E+00_JPRB, 0.95774E+00_JPRB ,&
&  0.10261E+01_JPRB, 0.10602E+01_JPRB, 0.10556E+01_JPRB, 0.10919E+01_JPRB, 0.14271E+01_JPRB, 0.14512E+01_JPRB, 0.16339E+01_JPRB ,&
&  0.17629E+01_JPRB, 0.19025E+01_JPRB, 0.19937E+01_JPRB, 0.21050E+01_JPRB, 0.21981E+01_JPRB, 0.22575E+01_JPRB, 0.23431E+01_JPRB ,&
&  0.23646E+01_JPRB, 0.24044E+01_JPRB, 0.23873E+01_JPRB, 0.22982E+01_JPRB, 0.19222E+01_JPRB, 0.20306E+01_JPRB, 0.14992E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.10698E+01_JPRB, 0.12069E+01_JPRB, 0.13274E+01_JPRB, 0.13333E+01_JPRB, 0.12399E+01_JPRB, 0.11008E+01_JPRB ,&
&  0.10439E+01_JPRB, 0.10159E+01_JPRB, 0.98816E+00_JPRB, 0.94208E+00_JPRB, 0.92097E+00_JPRB, 0.88397E+00_JPRB, 0.85290E+00_JPRB ,&
&  0.83537E+00_JPRB, 0.79761E+00_JPRB, 0.75652E+00_JPRB, 0.77805E+00_JPRB, 0.87420E+00_JPRB, 0.11522E+01_JPRB, 0.16424E+01_JPRB ,&
&  0.17914E+01_JPRB, 0.97980E+00_JPRB, 0.83651E+00_JPRB, 0.73043E+00_JPRB, 0.67934E+00_JPRB, 0.69184E+00_JPRB, 0.76598E+00_JPRB ,&
&  0.98205E+00_JPRB, 0.17322E+01_JPRB, 0.26070E+01_JPRB, 0.19599E+01_JPRB, 0.22238E+01_JPRB, 0.15330E+01_JPRB, 0.63465E+00_JPRB ,&
&  0.49966E+00_JPRB, 0.48516E+00_JPRB, 0.48903E+00_JPRB, 0.47909E+00_JPRB, 0.46606E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.49946E+00_JPRB, 0.48784E+00_JPRB, 0.48669E+00_JPRB, 0.49710E+00_JPRB, 0.50368E+00_JPRB, 0.52857E+00_JPRB, 0.55960E+00_JPRB ,&
&  0.57584E+00_JPRB, 0.64678E+00_JPRB, 0.80548E+00_JPRB, 0.94173E+00_JPRB, 0.10003E+01_JPRB, 0.10431E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.99649E+00_JPRB, 0.97391E+00_JPRB, 0.93468E+00_JPRB, 0.93877E+00_JPRB, 0.96675E+00_JPRB, 0.10168E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.12324E+01_JPRB, 0.12976E+01_JPRB, 0.13579E+01_JPRB, 0.14407E+01_JPRB, 0.14335E+01_JPRB, 0.13683E+01_JPRB ,&
&  0.67499E+00_JPRB, 0.10520E+01_JPRB, 0.10531E+01_JPRB, 0.99803E+00_JPRB, 0.93755E+00_JPRB, 0.94548E+00_JPRB, 0.10800E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.12215E+01_JPRB, 0.13924E+01_JPRB, 0.13457E+01_JPRB, 0.16218E+01_JPRB, 0.17676E+01_JPRB, 0.19196E+01_JPRB ,&
&  0.20551E+01_JPRB, 0.21529E+01_JPRB, 0.22591E+01_JPRB, 0.23648E+01_JPRB, 0.24709E+01_JPRB, 0.25572E+01_JPRB, 0.26725E+01_JPRB ,&
&  0.28025E+01_JPRB, 0.29872E+01_JPRB, 0.31614E+01_JPRB, 0.30144E+01_JPRB, 0.25850E+01_JPRB, 0.24460E+01_JPRB, 0.20303E+01_JPRB ,&
&  0.14923E+01_JPRB, 0.10886E+01_JPRB, 0.11541E+01_JPRB, 0.12300E+01_JPRB, 0.11868E+01_JPRB, 0.11108E+01_JPRB, 0.10313E+01_JPRB ,&
&  0.97629E+00_JPRB, 0.93572E+00_JPRB, 0.88267E+00_JPRB, 0.90066E+00_JPRB, 0.92345E+00_JPRB, 0.91356E+00_JPRB, 0.88946E+00_JPRB ,&
&  0.88375E+00_JPRB, 0.85750E+00_JPRB, 0.82960E+00_JPRB, 0.81725E+00_JPRB, 0.85503E+00_JPRB, 0.95490E+00_JPRB, 0.10418E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.94963E+00_JPRB, 0.85863E+00_JPRB, 0.84435E+00_JPRB, 0.82234E+00_JPRB, 0.80634E+00_JPRB, 0.78822E+00_JPRB ,&
&  0.82991E+00_JPRB, 0.99451E+00_JPRB, 0.13913E+01_JPRB, 0.17788E+01_JPRB, 0.14215E+01_JPRB, 0.11227E+01_JPRB, 0.64792E+00_JPRB ,&
&  0.52163E+00_JPRB, 0.50357E+00_JPRB, 0.52066E+00_JPRB, 0.52367E+00_JPRB, 0.52274E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.60740E+00_JPRB, 0.59364E+00_JPRB, 0.57607E+00_JPRB, 0.59225E+00_JPRB, 0.62589E+00_JPRB, 0.66842E+00_JPRB, 0.68719E+00_JPRB ,&
&  0.71182E+00_JPRB, 0.81408E+00_JPRB, 0.94888E+00_JPRB, 0.10684E+01_JPRB, 0.10904E+01_JPRB, 0.10608E+01_JPRB, 0.96612E+00_JPRB ,&
&  0.94437E+00_JPRB, 0.92334E+00_JPRB, 0.91484E+00_JPRB, 0.90711E+00_JPRB, 0.92193E+00_JPRB, 0.97143E+00_JPRB, 0.10507E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.12661E+01_JPRB, 0.13938E+01_JPRB, 0.15317E+01_JPRB, 0.16582E+01_JPRB, 0.17139E+01_JPRB, 0.16030E+01_JPRB ,&
&  0.67071E+00_JPRB, 0.77950E+00_JPRB, 0.10691E+01_JPRB, 0.10654E+01_JPRB, 0.10228E+01_JPRB, 0.10237E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.11981E+01_JPRB, 0.12204E+01_JPRB, 0.13589E+01_JPRB, 0.15879E+01_JPRB, 0.16759E+01_JPRB, 0.18624E+01_JPRB, 0.19559E+01_JPRB ,&
&  0.20465E+01_JPRB, 0.21473E+01_JPRB, 0.22644E+01_JPRB, 0.23795E+01_JPRB, 0.24946E+01_JPRB, 0.25999E+01_JPRB, 0.27487E+01_JPRB ,&
&  0.29556E+01_JPRB, 0.31679E+01_JPRB, 0.33440E+01_JPRB, 0.30062E+01_JPRB, 0.31680E+01_JPRB, 0.28179E+01_JPRB, 0.24667E+01_JPRB ,&
&  0.16080E+01_JPRB, 0.11449E+01_JPRB, 0.11727E+01_JPRB, 0.12675E+01_JPRB, 0.11583E+01_JPRB, 0.10479E+01_JPRB, 0.94848E+00_JPRB ,&
&  0.89043E+00_JPRB, 0.83060E+00_JPRB, 0.86298E+00_JPRB, 0.90215E+00_JPRB, 0.89852E+00_JPRB, 0.94266E+00_JPRB, 0.90863E+00_JPRB ,&
&  0.91951E+00_JPRB, 0.92917E+00_JPRB, 0.91839E+00_JPRB, 0.92540E+00_JPRB, 0.94153E+00_JPRB, 0.96890E+00_JPRB, 0.97073E+00_JPRB ,&
&  0.97418E+00_JPRB, 0.95764E+00_JPRB, 0.95591E+00_JPRB, 0.95887E+00_JPRB, 0.94366E+00_JPRB, 0.91848E+00_JPRB, 0.86401E+00_JPRB ,&
&  0.80387E+00_JPRB, 0.81000E+00_JPRB, 0.90755E+00_JPRB, 0.10343E+01_JPRB, 0.98282E+00_JPRB, 0.84879E+00_JPRB, 0.67599E+00_JPRB ,&
&  0.66181E+00_JPRB, 0.68254E+00_JPRB, 0.68344E+00_JPRB, 0.66268E+00_JPRB, 0.63794E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.72264E+00_JPRB, 0.69409E+00_JPRB, 0.65495E+00_JPRB, 0.64744E+00_JPRB, 0.70214E+00_JPRB, 0.75051E+00_JPRB, 0.75285E+00_JPRB ,&
&  0.76161E+00_JPRB, 0.82705E+00_JPRB, 0.94528E+00_JPRB, 0.10387E+01_JPRB, 0.10694E+01_JPRB, 0.10620E+01_JPRB, 0.97131E+00_JPRB ,&
&  0.93843E+00_JPRB, 0.91896E+00_JPRB, 0.89634E+00_JPRB, 0.88906E+00_JPRB, 0.89310E+00_JPRB, 0.91016E+00_JPRB, 0.94887E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.11115E+01_JPRB, 0.12712E+01_JPRB, 0.14987E+01_JPRB, 0.18290E+01_JPRB, 0.22173E+01_JPRB, 0.25549E+01_JPRB ,&
&  0.19264E+01_JPRB, 0.80598E+00_JPRB, 0.91413E+00_JPRB, 0.12026E+01_JPRB, 0.11337E+01_JPRB, 0.11530E+01_JPRB, 0.11296E+01_JPRB ,&
&  0.11788E+01_JPRB, 0.12518E+01_JPRB, 0.13325E+01_JPRB, 0.15271E+01_JPRB, 0.16664E+01_JPRB, 0.18170E+01_JPRB, 0.18756E+01_JPRB ,&
&  0.19361E+01_JPRB, 0.20141E+01_JPRB, 0.20927E+01_JPRB, 0.21739E+01_JPRB, 0.22677E+01_JPRB, 0.23743E+01_JPRB, 0.25456E+01_JPRB ,&
&  0.27627E+01_JPRB, 0.29600E+01_JPRB, 0.31028E+01_JPRB, 0.26684E+01_JPRB, 0.28009E+01_JPRB, 0.22972E+01_JPRB, 0.18305E+01_JPRB ,&
&  0.15303E+01_JPRB, 0.13137E+01_JPRB, 0.12453E+01_JPRB, 0.13088E+01_JPRB, 0.11623E+01_JPRB, 0.93426E+00_JPRB, 0.94574E+00_JPRB ,&
&  0.87352E+00_JPRB, 0.85533E+00_JPRB, 0.86507E+00_JPRB, 0.82215E+00_JPRB, 0.88403E+00_JPRB, 0.91757E+00_JPRB, 0.94742E+00_JPRB ,&
&  0.94205E+00_JPRB, 0.96571E+00_JPRB, 0.99008E+00_JPRB, 0.10321E+01_JPRB, 0.10322E+01_JPRB, 0.10454E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10578E+01_JPRB, 0.10736E+01_JPRB, 0.10914E+01_JPRB, 0.10882E+01_JPRB, 0.99978E+00_JPRB, 0.93551E+00_JPRB ,&
&  0.89639E+00_JPRB, 0.87228E+00_JPRB, 0.82573E+00_JPRB, 0.86187E+00_JPRB, 0.86369E+00_JPRB, 0.86951E+00_JPRB, 0.89948E+00_JPRB ,&
&  0.91215E+00_JPRB, 0.91348E+00_JPRB, 0.86170E+00_JPRB, 0.80420E+00_JPRB, 0.76676E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.79139E+00_JPRB, 0.73885E+00_JPRB, 0.70551E+00_JPRB, 0.71511E+00_JPRB, 0.76511E+00_JPRB, 0.81659E+00_JPRB, 0.81405E+00_JPRB ,&
&  0.79902E+00_JPRB, 0.81963E+00_JPRB, 0.88962E+00_JPRB, 0.95011E+00_JPRB, 0.95909E+00_JPRB, 0.96798E+00_JPRB, 0.98935E+00_JPRB ,&
&  0.94137E+00_JPRB, 0.91174E+00_JPRB, 0.89847E+00_JPRB, 0.88035E+00_JPRB, 0.88161E+00_JPRB, 0.87873E+00_JPRB, 0.88733E+00_JPRB ,&
&  0.91855E+00_JPRB, 0.96801E+00_JPRB, 0.10676E+01_JPRB, 0.12219E+01_JPRB, 0.14620E+01_JPRB, 0.19351E+01_JPRB, 0.26638E+01_JPRB ,&
&  0.38279E+01_JPRB, 0.25237E+01_JPRB, 0.91594E+00_JPRB, 0.11958E+01_JPRB, 0.12765E+01_JPRB, 0.12984E+01_JPRB, 0.11523E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.12629E+01_JPRB, 0.12527E+01_JPRB, 0.14215E+01_JPRB, 0.15513E+01_JPRB, 0.16639E+01_JPRB, 0.17266E+01_JPRB ,&
&  0.17530E+01_JPRB, 0.18061E+01_JPRB, 0.18453E+01_JPRB, 0.18759E+01_JPRB, 0.19146E+01_JPRB, 0.19791E+01_JPRB, 0.20964E+01_JPRB ,&
&  0.23263E+01_JPRB, 0.26023E+01_JPRB, 0.28422E+01_JPRB, 0.24693E+01_JPRB, 0.24837E+01_JPRB, 0.20114E+01_JPRB, 0.17526E+01_JPRB ,&
&  0.15784E+01_JPRB, 0.15975E+01_JPRB, 0.15815E+01_JPRB, 0.13836E+01_JPRB, 0.11638E+01_JPRB, 0.10221E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.95201E+00_JPRB, 0.88261E+00_JPRB, 0.85610E+00_JPRB, 0.88052E+00_JPRB, 0.90981E+00_JPRB, 0.93002E+00_JPRB, 0.94823E+00_JPRB ,&
&  0.98214E+00_JPRB, 0.99535E+00_JPRB, 0.10425E+01_JPRB, 0.10740E+01_JPRB, 0.11041E+01_JPRB, 0.11590E+01_JPRB, 0.11731E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.11791E+01_JPRB, 0.11930E+01_JPRB, 0.12078E+01_JPRB, 0.11472E+01_JPRB, 0.10535E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.99916E+00_JPRB, 0.93313E+00_JPRB, 0.93116E+00_JPRB, 0.92795E+00_JPRB, 0.98381E+00_JPRB, 0.10545E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.10566E+01_JPRB, 0.98519E+00_JPRB, 0.85664E+00_JPRB, 0.84193E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.85012E+00_JPRB, 0.83287E+00_JPRB, 0.80455E+00_JPRB, 0.78513E+00_JPRB, 0.80816E+00_JPRB, 0.84267E+00_JPRB, 0.84131E+00_JPRB ,&
&  0.81490E+00_JPRB, 0.79913E+00_JPRB, 0.83406E+00_JPRB, 0.87098E+00_JPRB, 0.86812E+00_JPRB, 0.88635E+00_JPRB, 0.89678E+00_JPRB ,&
&  0.93090E+00_JPRB, 0.92605E+00_JPRB, 0.90727E+00_JPRB, 0.88128E+00_JPRB, 0.87874E+00_JPRB, 0.86579E+00_JPRB, 0.86609E+00_JPRB ,&
&  0.88017E+00_JPRB, 0.90640E+00_JPRB, 0.96029E+00_JPRB, 0.10455E+01_JPRB, 0.11488E+01_JPRB, 0.13581E+01_JPRB, 0.17856E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.29775E+01_JPRB, 0.10443E+01_JPRB, 0.13653E+01_JPRB, 0.16298E+01_JPRB, 0.15479E+01_JPRB, 0.13881E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.13123E+01_JPRB, 0.12879E+01_JPRB, 0.12863E+01_JPRB, 0.13429E+01_JPRB, 0.14260E+01_JPRB, 0.14369E+01_JPRB ,&
&  0.14772E+01_JPRB, 0.15387E+01_JPRB, 0.16347E+01_JPRB, 0.16562E+01_JPRB, 0.16514E+01_JPRB, 0.16394E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.18328E+01_JPRB, 0.21014E+01_JPRB, 0.25028E+01_JPRB, 0.23155E+01_JPRB, 0.23143E+01_JPRB, 0.20855E+01_JPRB, 0.18921E+01_JPRB ,&
&  0.17420E+01_JPRB, 0.17923E+01_JPRB, 0.17590E+01_JPRB, 0.15298E+01_JPRB, 0.12655E+01_JPRB, 0.11366E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.94357E+00_JPRB, 0.92690E+00_JPRB, 0.93801E+00_JPRB, 0.99146E+00_JPRB, 0.99197E+00_JPRB, 0.98111E+00_JPRB ,&
&  0.10114E+01_JPRB, 0.10079E+01_JPRB, 0.10370E+01_JPRB, 0.10782E+01_JPRB, 0.11239E+01_JPRB, 0.11952E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.12771E+01_JPRB, 0.12842E+01_JPRB, 0.12382E+01_JPRB, 0.11703E+01_JPRB, 0.11241E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.11395E+01_JPRB, 0.10189E+01_JPRB, 0.95998E+00_JPRB, 0.98134E+00_JPRB, 0.98514E+00_JPRB, 0.10733E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.12466E+01_JPRB, 0.11961E+01_JPRB, 0.92289E+00_JPRB, 0.90835E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.94532E+00_JPRB, 0.90323E+00_JPRB, 0.88109E+00_JPRB, 0.86070E+00_JPRB, 0.84253E+00_JPRB, 0.87413E+00_JPRB, 0.86504E+00_JPRB ,&
&  0.84016E+00_JPRB, 0.80824E+00_JPRB, 0.82368E+00_JPRB, 0.86965E+00_JPRB, 0.87139E+00_JPRB, 0.85203E+00_JPRB, 0.83907E+00_JPRB ,&
&  0.83889E+00_JPRB, 0.85590E+00_JPRB, 0.85958E+00_JPRB, 0.87292E+00_JPRB, 0.87458E+00_JPRB, 0.86462E+00_JPRB, 0.85834E+00_JPRB ,&
&  0.87512E+00_JPRB, 0.89131E+00_JPRB, 0.91535E+00_JPRB, 0.94113E+00_JPRB, 0.98100E+00_JPRB, 0.10295E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.24495E+01_JPRB, 0.19997E+01_JPRB, 0.23669E+01_JPRB, 0.22020E+01_JPRB, 0.19878E+01_JPRB, 0.17288E+01_JPRB ,&
&  0.17891E+01_JPRB, 0.16660E+01_JPRB, 0.16241E+01_JPRB, 0.12969E+01_JPRB, 0.11643E+01_JPRB, 0.11648E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.11503E+01_JPRB, 0.12892E+01_JPRB, 0.13501E+01_JPRB, 0.13726E+01_JPRB, 0.13740E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.14584E+01_JPRB, 0.15935E+01_JPRB, 0.19670E+01_JPRB, 0.22226E+01_JPRB, 0.21604E+01_JPRB, 0.21597E+01_JPRB, 0.21345E+01_JPRB ,&
&  0.21754E+01_JPRB, 0.20441E+01_JPRB, 0.18866E+01_JPRB, 0.16413E+01_JPRB, 0.13647E+01_JPRB, 0.13422E+01_JPRB, 0.11945E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.99329E+00_JPRB, 0.98754E+00_JPRB, 0.10112E+01_JPRB, 0.10513E+01_JPRB, 0.10460E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10297E+01_JPRB, 0.10213E+01_JPRB, 0.10488E+01_JPRB, 0.10756E+01_JPRB, 0.11187E+01_JPRB, 0.11599E+01_JPRB ,&
&  0.12176E+01_JPRB, 0.12716E+01_JPRB, 0.12490E+01_JPRB, 0.12129E+01_JPRB, 0.11523E+01_JPRB, 0.11113E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10937E+01_JPRB, 0.10720E+01_JPRB, 0.10313E+01_JPRB, 0.99857E+00_JPRB, 0.97368E+00_JPRB, 0.97052E+00_JPRB ,&
&  0.99160E+00_JPRB, 0.10471E+01_JPRB, 0.10382E+01_JPRB, 0.98704E+00_JPRB, 0.97333E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.10041E+01_JPRB, 0.98091E+00_JPRB, 0.93551E+00_JPRB, 0.92853E+00_JPRB, 0.89945E+00_JPRB, 0.92465E+00_JPRB, 0.92870E+00_JPRB ,&
&  0.89393E+00_JPRB, 0.85936E+00_JPRB, 0.83485E+00_JPRB, 0.85294E+00_JPRB, 0.87398E+00_JPRB, 0.86171E+00_JPRB, 0.82369E+00_JPRB ,&
&  0.81086E+00_JPRB, 0.80796E+00_JPRB, 0.80709E+00_JPRB, 0.81136E+00_JPRB, 0.82773E+00_JPRB, 0.85166E+00_JPRB, 0.85012E+00_JPRB ,&
&  0.84760E+00_JPRB, 0.88179E+00_JPRB, 0.91686E+00_JPRB, 0.92309E+00_JPRB, 0.92521E+00_JPRB, 0.93345E+00_JPRB, 0.10279E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.25146E+01_JPRB, 0.17687E+01_JPRB, 0.30517E+01_JPRB, 0.24163E+01_JPRB, 0.21317E+01_JPRB, 0.17399E+01_JPRB ,&
&  0.16093E+01_JPRB, 0.17097E+01_JPRB, 0.14537E+01_JPRB, 0.12453E+01_JPRB, 0.10869E+01_JPRB, 0.10156E+01_JPRB, 0.95705E+00_JPRB ,&
&  0.90219E+00_JPRB, 0.92853E+00_JPRB, 0.99075E+00_JPRB, 0.10717E+01_JPRB, 0.11223E+01_JPRB, 0.11364E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.12036E+01_JPRB, 0.12620E+01_JPRB, 0.15045E+01_JPRB, 0.18026E+01_JPRB, 0.20815E+01_JPRB, 0.22178E+01_JPRB, 0.23850E+01_JPRB ,&
&  0.24475E+01_JPRB, 0.23692E+01_JPRB, 0.20427E+01_JPRB, 0.17398E+01_JPRB, 0.15135E+01_JPRB, 0.14286E+01_JPRB, 0.13275E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.11051E+01_JPRB, 0.10837E+01_JPRB, 0.10777E+01_JPRB, 0.11227E+01_JPRB, 0.11237E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.10139E+01_JPRB, 0.10258E+01_JPRB, 0.10485E+01_JPRB, 0.10755E+01_JPRB, 0.10798E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.12050E+01_JPRB, 0.12111E+01_JPRB, 0.12166E+01_JPRB, 0.11666E+01_JPRB, 0.11301E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.11353E+01_JPRB, 0.11250E+01_JPRB, 0.10806E+01_JPRB, 0.10378E+01_JPRB, 0.10263E+01_JPRB, 0.10259E+01_JPRB, 0.95687E+00_JPRB ,&
&  0.95150E+00_JPRB, 0.98898E+00_JPRB, 0.10393E+01_JPRB, 0.10073E+01_JPRB, 0.10037E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.10284E+01_JPRB, 0.10196E+01_JPRB, 0.96053E+00_JPRB, 0.94671E+00_JPRB, 0.92325E+00_JPRB, 0.91710E+00_JPRB, 0.92724E+00_JPRB ,&
&  0.91785E+00_JPRB, 0.89124E+00_JPRB, 0.85751E+00_JPRB, 0.84118E+00_JPRB, 0.85202E+00_JPRB, 0.86511E+00_JPRB, 0.83330E+00_JPRB ,&
&  0.79568E+00_JPRB, 0.78181E+00_JPRB, 0.77378E+00_JPRB, 0.77623E+00_JPRB, 0.78741E+00_JPRB, 0.80389E+00_JPRB, 0.81400E+00_JPRB ,&
&  0.81044E+00_JPRB, 0.85156E+00_JPRB, 0.90048E+00_JPRB, 0.92246E+00_JPRB, 0.91383E+00_JPRB, 0.92188E+00_JPRB, 0.98663E+00_JPRB ,&
&  0.11877E+01_JPRB, 0.32464E+01_JPRB, 0.19292E+01_JPRB, 0.31553E+01_JPRB, 0.24896E+01_JPRB, 0.21263E+01_JPRB, 0.17063E+01_JPRB ,&
&  0.15756E+01_JPRB, 0.15241E+01_JPRB, 0.13664E+01_JPRB, 0.12207E+01_JPRB, 0.11085E+01_JPRB, 0.10193E+01_JPRB, 0.94855E+00_JPRB ,&
&  0.90833E+00_JPRB, 0.89850E+00_JPRB, 0.91058E+00_JPRB, 0.96607E+00_JPRB, 0.10048E+01_JPRB, 0.10145E+01_JPRB, 0.10139E+01_JPRB ,&
&  0.10342E+01_JPRB, 0.10756E+01_JPRB, 0.12185E+01_JPRB, 0.16195E+01_JPRB, 0.18887E+01_JPRB, 0.21922E+01_JPRB, 0.30344E+01_JPRB ,&
&  0.29674E+01_JPRB, 0.28417E+01_JPRB, 0.22228E+01_JPRB, 0.18641E+01_JPRB, 0.16316E+01_JPRB, 0.14883E+01_JPRB, 0.14250E+01_JPRB ,&
&  0.13454E+01_JPRB, 0.12539E+01_JPRB, 0.12140E+01_JPRB, 0.11862E+01_JPRB, 0.11753E+01_JPRB, 0.11658E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10401E+01_JPRB, 0.10477E+01_JPRB, 0.10636E+01_JPRB, 0.10839E+01_JPRB, 0.10741E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.10806E+01_JPRB, 0.11377E+01_JPRB, 0.11610E+01_JPRB, 0.11813E+01_JPRB, 0.11766E+01_JPRB, 0.11535E+01_JPRB, 0.11340E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.11889E+01_JPRB, 0.11207E+01_JPRB, 0.11118E+01_JPRB, 0.11054E+01_JPRB, 0.11534E+01_JPRB, 0.99890E+00_JPRB ,&
&  0.96046E+00_JPRB, 0.94874E+00_JPRB, 0.10125E+01_JPRB, 0.10233E+01_JPRB, 0.10386E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.10516E+01_JPRB, 0.10267E+01_JPRB, 0.99392E+00_JPRB, 0.94975E+00_JPRB, 0.93532E+00_JPRB, 0.91818E+00_JPRB, 0.90723E+00_JPRB ,&
&  0.90242E+00_JPRB, 0.88656E+00_JPRB, 0.86959E+00_JPRB, 0.84629E+00_JPRB, 0.83649E+00_JPRB, 0.84732E+00_JPRB, 0.84213E+00_JPRB ,&
&  0.82371E+00_JPRB, 0.79924E+00_JPRB, 0.79008E+00_JPRB, 0.78042E+00_JPRB, 0.76994E+00_JPRB, 0.76968E+00_JPRB, 0.77959E+00_JPRB ,&
&  0.78539E+00_JPRB, 0.80561E+00_JPRB, 0.83905E+00_JPRB, 0.86968E+00_JPRB, 0.87162E+00_JPRB, 0.87681E+00_JPRB, 0.95802E+00_JPRB ,&
&  0.12339E+01_JPRB, 0.12721E+01_JPRB, 0.26552E+01_JPRB, 0.30380E+01_JPRB, 0.25258E+01_JPRB, 0.21871E+01_JPRB, 0.18502E+01_JPRB ,&
&  0.16717E+01_JPRB, 0.14485E+01_JPRB, 0.12676E+01_JPRB, 0.11403E+01_JPRB, 0.10662E+01_JPRB, 0.98252E+00_JPRB, 0.94269E+00_JPRB ,&
&  0.92691E+00_JPRB, 0.91042E+00_JPRB, 0.91010E+00_JPRB, 0.92790E+00_JPRB, 0.93307E+00_JPRB, 0.93669E+00_JPRB, 0.93123E+00_JPRB ,&
&  0.94293E+00_JPRB, 0.97121E+00_JPRB, 0.10564E+01_JPRB, 0.13065E+01_JPRB, 0.17536E+01_JPRB, 0.22437E+01_JPRB, 0.30118E+01_JPRB ,&
&  0.34004E+01_JPRB, 0.32141E+01_JPRB, 0.24127E+01_JPRB, 0.19774E+01_JPRB, 0.17390E+01_JPRB, 0.15709E+01_JPRB, 0.14919E+01_JPRB ,&
&  0.14201E+01_JPRB, 0.13480E+01_JPRB, 0.12945E+01_JPRB, 0.13049E+01_JPRB, 0.12466E+01_JPRB, 0.12043E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.10811E+01_JPRB, 0.10473E+01_JPRB, 0.10283E+01_JPRB, 0.10457E+01_JPRB, 0.10543E+01_JPRB, 0.10495E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.10788E+01_JPRB, 0.11213E+01_JPRB, 0.11253E+01_JPRB, 0.11596E+01_JPRB, 0.11707E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.11942E+01_JPRB, 0.12116E+01_JPRB, 0.11652E+01_JPRB, 0.11899E+01_JPRB, 0.11642E+01_JPRB, 0.12178E+01_JPRB, 0.10846E+01_JPRB ,&
&  0.10074E+01_JPRB, 0.96096E+00_JPRB, 0.96647E+00_JPRB, 0.10189E+01_JPRB, 0.10583E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.10527E+01_JPRB, 0.10288E+01_JPRB, 0.10044E+01_JPRB, 0.98011E+00_JPRB, 0.95831E+00_JPRB, 0.94492E+00_JPRB, 0.92990E+00_JPRB ,&
&  0.92820E+00_JPRB, 0.92539E+00_JPRB, 0.91108E+00_JPRB, 0.88032E+00_JPRB, 0.85489E+00_JPRB, 0.84871E+00_JPRB, 0.85256E+00_JPRB ,&
&  0.84556E+00_JPRB, 0.83347E+00_JPRB, 0.83549E+00_JPRB, 0.81754E+00_JPRB, 0.79531E+00_JPRB, 0.78092E+00_JPRB, 0.77548E+00_JPRB ,&
&  0.78042E+00_JPRB, 0.79261E+00_JPRB, 0.82137E+00_JPRB, 0.84841E+00_JPRB, 0.85212E+00_JPRB, 0.84656E+00_JPRB, 0.85005E+00_JPRB ,&
&  0.11067E+01_JPRB, 0.18351E+01_JPRB, 0.29073E+01_JPRB, 0.27523E+01_JPRB, 0.23885E+01_JPRB, 0.20261E+01_JPRB, 0.18451E+01_JPRB ,&
&  0.15848E+01_JPRB, 0.14252E+01_JPRB, 0.12657E+01_JPRB, 0.11300E+01_JPRB, 0.10334E+01_JPRB, 0.97524E+00_JPRB, 0.94187E+00_JPRB ,&
&  0.91004E+00_JPRB, 0.89263E+00_JPRB, 0.89358E+00_JPRB, 0.89744E+00_JPRB, 0.89787E+00_JPRB, 0.89134E+00_JPRB, 0.88337E+00_JPRB ,&
&  0.89531E+00_JPRB, 0.92166E+00_JPRB, 0.97218E+00_JPRB, 0.11115E+01_JPRB, 0.15758E+01_JPRB, 0.18250E+01_JPRB, 0.26736E+01_JPRB ,&
&  0.35566E+01_JPRB, 0.30398E+01_JPRB, 0.24421E+01_JPRB, 0.20557E+01_JPRB, 0.18151E+01_JPRB, 0.16421E+01_JPRB, 0.15379E+01_JPRB ,&
&  0.14370E+01_JPRB, 0.13731E+01_JPRB, 0.13267E+01_JPRB, 0.12780E+01_JPRB, 0.12481E+01_JPRB, 0.11939E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.11011E+01_JPRB, 0.10632E+01_JPRB, 0.10216E+01_JPRB, 0.10171E+01_JPRB, 0.10247E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.10424E+01_JPRB, 0.10946E+01_JPRB, 0.10891E+01_JPRB, 0.11348E+01_JPRB, 0.11776E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.11770E+01_JPRB, 0.11978E+01_JPRB, 0.12155E+01_JPRB, 0.13073E+01_JPRB, 0.13869E+01_JPRB, 0.12128E+01_JPRB ,&
&  0.10867E+01_JPRB, 0.10220E+01_JPRB, 0.98559E+00_JPRB, 0.99056E+00_JPRB, 0.10561E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.10912E+01_JPRB, 0.10713E+01_JPRB, 0.10379E+01_JPRB, 0.10112E+01_JPRB, 0.99931E+00_JPRB, 0.98106E+00_JPRB, 0.96606E+00_JPRB ,&
&  0.95637E+00_JPRB, 0.94072E+00_JPRB, 0.92781E+00_JPRB, 0.91033E+00_JPRB, 0.88891E+00_JPRB, 0.87063E+00_JPRB, 0.86292E+00_JPRB ,&
&  0.86070E+00_JPRB, 0.84956E+00_JPRB, 0.85228E+00_JPRB, 0.85272E+00_JPRB, 0.84843E+00_JPRB, 0.83538E+00_JPRB, 0.82243E+00_JPRB ,&
&  0.81508E+00_JPRB, 0.80230E+00_JPRB, 0.80089E+00_JPRB, 0.82346E+00_JPRB, 0.84236E+00_JPRB, 0.84582E+00_JPRB, 0.83795E+00_JPRB ,&
&  0.94963E+00_JPRB, 0.14974E+01_JPRB, 0.21603E+01_JPRB, 0.23213E+01_JPRB, 0.21786E+01_JPRB, 0.19402E+01_JPRB, 0.17009E+01_JPRB ,&
&  0.14943E+01_JPRB, 0.13300E+01_JPRB, 0.12290E+01_JPRB, 0.11482E+01_JPRB, 0.10713E+01_JPRB, 0.10079E+01_JPRB, 0.95905E+00_JPRB ,&
&  0.91179E+00_JPRB, 0.88981E+00_JPRB, 0.87801E+00_JPRB, 0.87408E+00_JPRB, 0.87034E+00_JPRB, 0.86680E+00_JPRB, 0.87164E+00_JPRB ,&
&  0.88260E+00_JPRB, 0.89719E+00_JPRB, 0.92676E+00_JPRB, 0.98309E+00_JPRB, 0.12292E+01_JPRB, 0.16261E+01_JPRB, 0.21703E+01_JPRB ,&
&  0.22980E+01_JPRB, 0.23263E+01_JPRB, 0.21489E+01_JPRB, 0.19098E+01_JPRB, 0.17157E+01_JPRB, 0.15662E+01_JPRB, 0.14664E+01_JPRB ,&
&  0.13689E+01_JPRB, 0.12983E+01_JPRB, 0.12469E+01_JPRB, 0.11913E+01_JPRB, 0.11315E+01_JPRB, 0.11033E+01_JPRB, 0.11083E+01_JPRB ,&
&  0.11229E+01_JPRB, 0.11144E+01_JPRB, 0.10779E+01_JPRB, 0.10347E+01_JPRB, 0.98726E+00_JPRB, 0.97018E+00_JPRB, 0.96641E+00_JPRB ,&
&  0.96871E+00_JPRB, 0.98560E+00_JPRB, 0.10289E+01_JPRB, 0.10718E+01_JPRB, 0.11039E+01_JPRB, 0.11201E+01_JPRB, 0.11215E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.11900E+01_JPRB, 0.11698E+01_JPRB, 0.12586E+01_JPRB, 0.13551E+01_JPRB, 0.14236E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.11931E+01_JPRB, 0.11001E+01_JPRB, 0.10539E+01_JPRB, 0.10341E+01_JPRB, 0.10822E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11239E+01_JPRB, 0.11036E+01_JPRB, 0.10697E+01_JPRB, 0.10392E+01_JPRB, 0.10040E+01_JPRB, 0.97752E+00_JPRB, 0.95596E+00_JPRB ,&
&  0.94287E+00_JPRB, 0.92163E+00_JPRB, 0.91022E+00_JPRB, 0.90065E+00_JPRB, 0.89577E+00_JPRB, 0.88526E+00_JPRB, 0.86989E+00_JPRB ,&
&  0.85336E+00_JPRB, 0.84278E+00_JPRB, 0.83854E+00_JPRB, 0.84151E+00_JPRB, 0.85032E+00_JPRB, 0.85596E+00_JPRB, 0.85040E+00_JPRB ,&
&  0.84110E+00_JPRB, 0.82677E+00_JPRB, 0.81000E+00_JPRB, 0.82565E+00_JPRB, 0.84249E+00_JPRB, 0.84807E+00_JPRB, 0.83613E+00_JPRB ,&
&  0.89317E+00_JPRB, 0.11078E+01_JPRB, 0.15178E+01_JPRB, 0.16504E+01_JPRB, 0.16568E+01_JPRB, 0.16771E+01_JPRB, 0.15540E+01_JPRB ,&
&  0.14266E+01_JPRB, 0.12803E+01_JPRB, 0.11574E+01_JPRB, 0.10930E+01_JPRB, 0.10520E+01_JPRB, 0.10009E+01_JPRB, 0.95872E+00_JPRB ,&
&  0.90668E+00_JPRB, 0.87896E+00_JPRB, 0.86103E+00_JPRB, 0.85397E+00_JPRB, 0.85081E+00_JPRB, 0.85159E+00_JPRB, 0.85749E+00_JPRB ,&
&  0.86388E+00_JPRB, 0.86847E+00_JPRB, 0.87964E+00_JPRB, 0.91133E+00_JPRB, 0.97386E+00_JPRB, 0.11542E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.15810E+01_JPRB, 0.16993E+01_JPRB, 0.17859E+01_JPRB, 0.17750E+01_JPRB, 0.16580E+01_JPRB, 0.15173E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.13250E+01_JPRB, 0.12369E+01_JPRB, 0.11576E+01_JPRB, 0.10966E+01_JPRB, 0.10425E+01_JPRB, 0.10195E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.10514E+01_JPRB, 0.10261E+01_JPRB, 0.99372E+00_JPRB, 0.95009E+00_JPRB, 0.92170E+00_JPRB, 0.91740E+00_JPRB ,&
&  0.92625E+00_JPRB, 0.94761E+00_JPRB, 0.98094E+00_JPRB, 0.10190E+01_JPRB, 0.10524E+01_JPRB, 0.10723E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10877E+01_JPRB, 0.11066E+01_JPRB, 0.10674E+01_JPRB, 0.12004E+01_JPRB, 0.13537E+01_JPRB, 0.13382E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.11461E+01_JPRB, 0.10781E+01_JPRB, 0.10419E+01_JPRB, 0.10859E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.96861E+00_JPRB, 0.98738E+00_JPRB, 0.98669E+00_JPRB, 0.97802E+00_JPRB, 0.95559E+00_JPRB, 0.92467E+00_JPRB, 0.90138E+00_JPRB ,&
&  0.88903E+00_JPRB, 0.86997E+00_JPRB, 0.86092E+00_JPRB, 0.85723E+00_JPRB, 0.86014E+00_JPRB, 0.86691E+00_JPRB, 0.86664E+00_JPRB ,&
&  0.85661E+00_JPRB, 0.84157E+00_JPRB, 0.83095E+00_JPRB, 0.82957E+00_JPRB, 0.83880E+00_JPRB, 0.84906E+00_JPRB, 0.85574E+00_JPRB ,&
&  0.85842E+00_JPRB, 0.85164E+00_JPRB, 0.84850E+00_JPRB, 0.85898E+00_JPRB, 0.86774E+00_JPRB, 0.86045E+00_JPRB, 0.84129E+00_JPRB ,&
&  0.85684E+00_JPRB, 0.86412E+00_JPRB, 0.10199E+01_JPRB, 0.11714E+01_JPRB, 0.12971E+01_JPRB, 0.13454E+01_JPRB, 0.13282E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.11905E+01_JPRB, 0.11069E+01_JPRB, 0.10367E+01_JPRB, 0.98498E+00_JPRB, 0.95175E+00_JPRB, 0.93023E+00_JPRB ,&
&  0.89945E+00_JPRB, 0.86934E+00_JPRB, 0.84311E+00_JPRB, 0.82838E+00_JPRB, 0.82302E+00_JPRB, 0.81837E+00_JPRB, 0.81379E+00_JPRB ,&
&  0.80853E+00_JPRB, 0.80532E+00_JPRB, 0.81162E+00_JPRB, 0.83076E+00_JPRB, 0.86388E+00_JPRB, 0.94249E+00_JPRB, 0.10690E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.12787E+01_JPRB, 0.13831E+01_JPRB, 0.14675E+01_JPRB, 0.14652E+01_JPRB, 0.14124E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.11384E+01_JPRB, 0.10440E+01_JPRB, 0.98593E+00_JPRB, 0.96499E+00_JPRB, 0.96290E+00_JPRB, 0.98140E+00_JPRB ,&
&  0.99032E+00_JPRB, 0.98673E+00_JPRB, 0.96026E+00_JPRB, 0.93274E+00_JPRB, 0.91590E+00_JPRB, 0.89366E+00_JPRB, 0.88742E+00_JPRB ,&
&  0.88828E+00_JPRB, 0.90059E+00_JPRB, 0.92411E+00_JPRB, 0.94779E+00_JPRB, 0.98069E+00_JPRB, 0.10056E+01_JPRB, 0.10160E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.10219E+01_JPRB, 0.10241E+01_JPRB, 0.11165E+01_JPRB, 0.11547E+01_JPRB, 0.11797E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11474E+01_JPRB, 0.10925E+01_JPRB, 0.10269E+01_JPRB, 0.99321E+00_JPRB, 0.91208E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.89986E+00_JPRB, 0.91567E+00_JPRB, 0.91636E+00_JPRB, 0.91033E+00_JPRB, 0.90083E+00_JPRB, 0.87941E+00_JPRB, 0.85489E+00_JPRB ,&
&  0.83006E+00_JPRB, 0.81851E+00_JPRB, 0.81323E+00_JPRB, 0.81622E+00_JPRB, 0.82128E+00_JPRB, 0.82962E+00_JPRB, 0.84185E+00_JPRB ,&
&  0.85348E+00_JPRB, 0.85894E+00_JPRB, 0.86312E+00_JPRB, 0.86100E+00_JPRB, 0.85904E+00_JPRB, 0.86113E+00_JPRB, 0.86529E+00_JPRB ,&
&  0.87398E+00_JPRB, 0.87682E+00_JPRB, 0.87333E+00_JPRB, 0.88217E+00_JPRB, 0.89589E+00_JPRB, 0.88571E+00_JPRB, 0.86702E+00_JPRB ,&
&  0.86147E+00_JPRB, 0.75413E+00_JPRB, 0.78979E+00_JPRB, 0.92792E+00_JPRB, 0.10470E+01_JPRB, 0.11556E+01_JPRB, 0.11816E+01_JPRB ,&
&  0.11510E+01_JPRB, 0.10941E+01_JPRB, 0.10377E+01_JPRB, 0.98428E+00_JPRB, 0.94646E+00_JPRB, 0.90840E+00_JPRB, 0.88235E+00_JPRB ,&
&  0.86526E+00_JPRB, 0.84657E+00_JPRB, 0.81995E+00_JPRB, 0.79757E+00_JPRB, 0.77908E+00_JPRB, 0.76106E+00_JPRB, 0.75083E+00_JPRB ,&
&  0.75049E+00_JPRB, 0.76106E+00_JPRB, 0.77440E+00_JPRB, 0.79220E+00_JPRB, 0.82526E+00_JPRB, 0.86583E+00_JPRB, 0.92479E+00_JPRB ,&
&  0.98282E+00_JPRB, 0.10451E+01_JPRB, 0.11074E+01_JPRB, 0.11604E+01_JPRB, 0.12060E+01_JPRB, 0.12123E+01_JPRB, 0.11659E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.10182E+01_JPRB, 0.97256E+00_JPRB, 0.94195E+00_JPRB, 0.93679E+00_JPRB, 0.94720E+00_JPRB, 0.95253E+00_JPRB ,&
&  0.94266E+00_JPRB, 0.92152E+00_JPRB, 0.89384E+00_JPRB, 0.86846E+00_JPRB, 0.85704E+00_JPRB, 0.85103E+00_JPRB, 0.84148E+00_JPRB ,&
&  0.83309E+00_JPRB, 0.83316E+00_JPRB, 0.84264E+00_JPRB, 0.86404E+00_JPRB, 0.89208E+00_JPRB, 0.92426E+00_JPRB, 0.94460E+00_JPRB ,&
&  0.95783E+00_JPRB, 0.97023E+00_JPRB, 0.98380E+00_JPRB, 0.10077E+01_JPRB, 0.10728E+01_JPRB, 0.11012E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.10610E+01_JPRB, 0.10160E+01_JPRB, 0.82478E+00_JPRB, 0.87976E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.85748E+00_JPRB, 0.85534E+00_JPRB, 0.85411E+00_JPRB, 0.84519E+00_JPRB, 0.83735E+00_JPRB, 0.82485E+00_JPRB, 0.80266E+00_JPRB ,&
&  0.78221E+00_JPRB, 0.77322E+00_JPRB, 0.77136E+00_JPRB, 0.77139E+00_JPRB, 0.77333E+00_JPRB, 0.77892E+00_JPRB, 0.78999E+00_JPRB ,&
&  0.80353E+00_JPRB, 0.81786E+00_JPRB, 0.83463E+00_JPRB, 0.84997E+00_JPRB, 0.85314E+00_JPRB, 0.84968E+00_JPRB, 0.84965E+00_JPRB ,&
&  0.85554E+00_JPRB, 0.85427E+00_JPRB, 0.84729E+00_JPRB, 0.84947E+00_JPRB, 0.85843E+00_JPRB, 0.85015E+00_JPRB, 0.83220E+00_JPRB ,&
&  0.82320E+00_JPRB, 0.68743E+00_JPRB, 0.72949E+00_JPRB, 0.80488E+00_JPRB, 0.89650E+00_JPRB, 0.98047E+00_JPRB, 0.10158E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.96480E+00_JPRB, 0.92799E+00_JPRB, 0.88443E+00_JPRB, 0.86221E+00_JPRB, 0.84678E+00_JPRB, 0.82907E+00_JPRB ,&
&  0.80312E+00_JPRB, 0.77607E+00_JPRB, 0.74633E+00_JPRB, 0.71982E+00_JPRB, 0.69884E+00_JPRB, 0.68843E+00_JPRB, 0.68480E+00_JPRB ,&
&  0.68457E+00_JPRB, 0.69535E+00_JPRB, 0.70179E+00_JPRB, 0.71270E+00_JPRB, 0.73048E+00_JPRB, 0.75708E+00_JPRB, 0.79069E+00_JPRB ,&
&  0.83174E+00_JPRB, 0.87706E+00_JPRB, 0.92130E+00_JPRB, 0.95188E+00_JPRB, 0.97343E+00_JPRB, 0.98924E+00_JPRB, 0.98179E+00_JPRB ,&
&  0.94824E+00_JPRB, 0.91198E+00_JPRB, 0.90570E+00_JPRB, 0.90617E+00_JPRB, 0.89738E+00_JPRB, 0.88930E+00_JPRB, 0.87687E+00_JPRB ,&
&  0.85590E+00_JPRB, 0.83112E+00_JPRB, 0.81163E+00_JPRB, 0.79206E+00_JPRB, 0.77959E+00_JPRB, 0.77413E+00_JPRB, 0.75867E+00_JPRB ,&
&  0.74848E+00_JPRB, 0.74768E+00_JPRB, 0.75656E+00_JPRB, 0.77779E+00_JPRB, 0.80674E+00_JPRB, 0.83525E+00_JPRB, 0.85473E+00_JPRB ,&
&  0.86615E+00_JPRB, 0.87736E+00_JPRB, 0.88383E+00_JPRB, 0.93017E+00_JPRB, 0.95460E+00_JPRB, 0.97441E+00_JPRB, 0.99117E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.10058E+01_JPRB, 0.76460E+00_JPRB, 0.83678E+00_JPRB, 0.84707E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.86019E+00_JPRB, 0.83309E+00_JPRB, 0.81636E+00_JPRB, 0.80545E+00_JPRB, 0.79703E+00_JPRB, 0.78775E+00_JPRB, 0.77145E+00_JPRB ,&
&  0.75418E+00_JPRB, 0.74465E+00_JPRB, 0.73672E+00_JPRB, 0.72731E+00_JPRB, 0.72790E+00_JPRB, 0.73336E+00_JPRB, 0.74424E+00_JPRB ,&
&  0.75634E+00_JPRB, 0.77185E+00_JPRB, 0.78673E+00_JPRB, 0.80545E+00_JPRB, 0.81758E+00_JPRB, 0.82588E+00_JPRB, 0.83013E+00_JPRB ,&
&  0.83299E+00_JPRB, 0.83324E+00_JPRB, 0.82546E+00_JPRB, 0.81547E+00_JPRB, 0.81313E+00_JPRB, 0.81512E+00_JPRB, 0.81034E+00_JPRB ,&
&  0.71143E+00_JPRB, 0.62817E+00_JPRB, 0.68633E+00_JPRB, 0.75399E+00_JPRB, 0.82210E+00_JPRB, 0.88004E+00_JPRB, 0.90637E+00_JPRB ,&
&  0.89349E+00_JPRB, 0.86107E+00_JPRB, 0.83115E+00_JPRB, 0.79598E+00_JPRB, 0.78062E+00_JPRB, 0.77385E+00_JPRB, 0.76420E+00_JPRB ,&
&  0.75389E+00_JPRB, 0.74145E+00_JPRB, 0.72517E+00_JPRB, 0.69982E+00_JPRB, 0.68158E+00_JPRB, 0.66834E+00_JPRB, 0.66540E+00_JPRB ,&
&  0.66705E+00_JPRB, 0.67063E+00_JPRB, 0.67414E+00_JPRB, 0.67707E+00_JPRB, 0.68432E+00_JPRB, 0.69542E+00_JPRB, 0.70893E+00_JPRB ,&
&  0.72902E+00_JPRB, 0.75088E+00_JPRB, 0.78093E+00_JPRB, 0.80174E+00_JPRB, 0.81555E+00_JPRB, 0.82699E+00_JPRB, 0.84048E+00_JPRB ,&
&  0.84568E+00_JPRB, 0.84048E+00_JPRB, 0.82011E+00_JPRB, 0.80974E+00_JPRB, 0.80845E+00_JPRB, 0.80658E+00_JPRB, 0.79753E+00_JPRB ,&
&  0.77980E+00_JPRB, 0.75818E+00_JPRB, 0.73611E+00_JPRB, 0.72348E+00_JPRB, 0.72198E+00_JPRB, 0.72023E+00_JPRB, 0.71491E+00_JPRB ,&
&  0.71049E+00_JPRB, 0.71017E+00_JPRB, 0.71596E+00_JPRB, 0.72759E+00_JPRB, 0.74348E+00_JPRB, 0.76320E+00_JPRB, 0.77983E+00_JPRB ,&
&  0.79694E+00_JPRB, 0.81420E+00_JPRB, 0.82617E+00_JPRB, 0.85491E+00_JPRB, 0.88245E+00_JPRB, 0.89962E+00_JPRB, 0.90545E+00_JPRB ,&
&  0.91613E+00_JPRB, 0.91322E+00_JPRB, 0.90520E+00_JPRB, 0.89838E+00_JPRB, 0.88352E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.80751E+00_JPRB, 0.78386E+00_JPRB, 0.76881E+00_JPRB, 0.76091E+00_JPRB, 0.74976E+00_JPRB, 0.72920E+00_JPRB, 0.71246E+00_JPRB ,&
&  0.69654E+00_JPRB, 0.68708E+00_JPRB, 0.68237E+00_JPRB, 0.68069E+00_JPRB, 0.68609E+00_JPRB, 0.69134E+00_JPRB, 0.69801E+00_JPRB ,&
&  0.70926E+00_JPRB, 0.72749E+00_JPRB, 0.74303E+00_JPRB, 0.75721E+00_JPRB, 0.76987E+00_JPRB, 0.78237E+00_JPRB, 0.78546E+00_JPRB ,&
&  0.78786E+00_JPRB, 0.79510E+00_JPRB, 0.79634E+00_JPRB, 0.78362E+00_JPRB, 0.77046E+00_JPRB, 0.77376E+00_JPRB, 0.77113E+00_JPRB ,&
&  0.76178E+00_JPRB, 0.64779E+00_JPRB, 0.70343E+00_JPRB, 0.73077E+00_JPRB, 0.76231E+00_JPRB, 0.79279E+00_JPRB, 0.80917E+00_JPRB ,&
&  0.80474E+00_JPRB, 0.78546E+00_JPRB, 0.76382E+00_JPRB, 0.73582E+00_JPRB, 0.72736E+00_JPRB, 0.72000E+00_JPRB, 0.71050E+00_JPRB ,&
&  0.70424E+00_JPRB, 0.69808E+00_JPRB, 0.68925E+00_JPRB, 0.67720E+00_JPRB, 0.66184E+00_JPRB, 0.65046E+00_JPRB, 0.64037E+00_JPRB ,&
&  0.63742E+00_JPRB, 0.62968E+00_JPRB, 0.62271E+00_JPRB, 0.62346E+00_JPRB, 0.63535E+00_JPRB, 0.64130E+00_JPRB, 0.64236E+00_JPRB ,&
&  0.64406E+00_JPRB, 0.64809E+00_JPRB, 0.65970E+00_JPRB, 0.67674E+00_JPRB, 0.69084E+00_JPRB, 0.70095E+00_JPRB, 0.71497E+00_JPRB ,&
&  0.73691E+00_JPRB, 0.75256E+00_JPRB, 0.75289E+00_JPRB, 0.74032E+00_JPRB, 0.73205E+00_JPRB, 0.73082E+00_JPRB, 0.71959E+00_JPRB ,&
&  0.70117E+00_JPRB, 0.68898E+00_JPRB, 0.68545E+00_JPRB, 0.68414E+00_JPRB, 0.68022E+00_JPRB, 0.67424E+00_JPRB, 0.67334E+00_JPRB ,&
&  0.67457E+00_JPRB, 0.67494E+00_JPRB, 0.67411E+00_JPRB, 0.67684E+00_JPRB, 0.68312E+00_JPRB, 0.69153E+00_JPRB, 0.70314E+00_JPRB ,&
&  0.72000E+00_JPRB, 0.73614E+00_JPRB, 0.74533E+00_JPRB, 0.76139E+00_JPRB, 0.78009E+00_JPRB, 0.79051E+00_JPRB, 0.80927E+00_JPRB ,&
&  0.81502E+00_JPRB, 0.81657E+00_JPRB, 0.81763E+00_JPRB, 0.82297E+00_JPRB, 0.82328E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.75862E+00_JPRB, 0.73685E+00_JPRB, 0.72277E+00_JPRB, 0.70982E+00_JPRB, 0.70051E+00_JPRB, 0.68164E+00_JPRB, 0.67076E+00_JPRB ,&
&  0.66370E+00_JPRB, 0.65904E+00_JPRB, 0.65736E+00_JPRB, 0.65688E+00_JPRB, 0.66687E+00_JPRB, 0.67338E+00_JPRB, 0.67585E+00_JPRB ,&
&  0.68344E+00_JPRB, 0.70099E+00_JPRB, 0.71718E+00_JPRB, 0.73067E+00_JPRB, 0.74116E+00_JPRB, 0.74864E+00_JPRB, 0.74825E+00_JPRB ,&
&  0.75172E+00_JPRB, 0.75947E+00_JPRB, 0.76315E+00_JPRB, 0.75283E+00_JPRB, 0.73991E+00_JPRB, 0.73321E+00_JPRB, 0.73407E+00_JPRB ,&
&  0.72348E+00_JPRB, 0.68186E+00_JPRB, 0.69738E+00_JPRB, 0.70954E+00_JPRB, 0.72405E+00_JPRB, 0.73321E+00_JPRB, 0.74278E+00_JPRB ,&
&  0.74555E+00_JPRB, 0.72825E+00_JPRB, 0.70583E+00_JPRB, 0.68975E+00_JPRB, 0.69316E+00_JPRB, 0.69022E+00_JPRB, 0.68029E+00_JPRB ,&
&  0.67550E+00_JPRB, 0.67141E+00_JPRB, 0.66356E+00_JPRB, 0.65034E+00_JPRB, 0.63849E+00_JPRB, 0.63034E+00_JPRB, 0.62481E+00_JPRB ,&
&  0.61324E+00_JPRB, 0.60366E+00_JPRB, 0.59849E+00_JPRB, 0.59284E+00_JPRB, 0.58823E+00_JPRB, 0.59265E+00_JPRB, 0.59652E+00_JPRB ,&
&  0.59657E+00_JPRB, 0.59832E+00_JPRB, 0.60721E+00_JPRB, 0.62266E+00_JPRB, 0.63636E+00_JPRB, 0.65168E+00_JPRB, 0.66926E+00_JPRB ,&
&  0.68673E+00_JPRB, 0.70240E+00_JPRB, 0.70795E+00_JPRB, 0.71189E+00_JPRB, 0.70604E+00_JPRB, 0.69670E+00_JPRB, 0.67795E+00_JPRB ,&
&  0.65728E+00_JPRB, 0.65302E+00_JPRB, 0.65385E+00_JPRB, 0.65972E+00_JPRB, 0.65819E+00_JPRB, 0.65536E+00_JPRB, 0.65361E+00_JPRB ,&
&  0.64949E+00_JPRB, 0.64588E+00_JPRB, 0.64089E+00_JPRB, 0.63624E+00_JPRB, 0.63947E+00_JPRB, 0.64540E+00_JPRB, 0.65896E+00_JPRB ,&
&  0.67718E+00_JPRB, 0.69676E+00_JPRB, 0.71691E+00_JPRB, 0.73226E+00_JPRB, 0.73918E+00_JPRB, 0.74381E+00_JPRB, 0.74899E+00_JPRB ,&
&  0.75156E+00_JPRB, 0.75196E+00_JPRB, 0.75991E+00_JPRB, 0.76513E+00_JPRB, 0.76969E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.69699E+00_JPRB, 0.67186E+00_JPRB, 0.65278E+00_JPRB, 0.64545E+00_JPRB, 0.64145E+00_JPRB, 0.62883E+00_JPRB, 0.62287E+00_JPRB ,&
&  0.62853E+00_JPRB, 0.63012E+00_JPRB, 0.63677E+00_JPRB, 0.63594E+00_JPRB, 0.63502E+00_JPRB, 0.63762E+00_JPRB, 0.62998E+00_JPRB ,&
&  0.63003E+00_JPRB, 0.63913E+00_JPRB, 0.64984E+00_JPRB, 0.66428E+00_JPRB, 0.67635E+00_JPRB, 0.68542E+00_JPRB, 0.69303E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.70233E+00_JPRB, 0.70577E+00_JPRB, 0.70139E+00_JPRB, 0.69018E+00_JPRB, 0.68139E+00_JPRB, 0.67986E+00_JPRB ,&
&  0.67837E+00_JPRB, 0.67434E+00_JPRB, 0.63014E+00_JPRB, 0.65454E+00_JPRB, 0.66190E+00_JPRB, 0.66643E+00_JPRB, 0.66998E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.65168E+00_JPRB, 0.64446E+00_JPRB, 0.63168E+00_JPRB, 0.63018E+00_JPRB, 0.63596E+00_JPRB, 0.63375E+00_JPRB ,&
&  0.62862E+00_JPRB, 0.61750E+00_JPRB, 0.60309E+00_JPRB, 0.59212E+00_JPRB, 0.58937E+00_JPRB, 0.58720E+00_JPRB, 0.57921E+00_JPRB ,&
&  0.57381E+00_JPRB, 0.57296E+00_JPRB, 0.57125E+00_JPRB, 0.56535E+00_JPRB, 0.55050E+00_JPRB, 0.53700E+00_JPRB, 0.53457E+00_JPRB ,&
&  0.53820E+00_JPRB, 0.54558E+00_JPRB, 0.56001E+00_JPRB, 0.57874E+00_JPRB, 0.59892E+00_JPRB, 0.61782E+00_JPRB, 0.63492E+00_JPRB ,&
&  0.64188E+00_JPRB, 0.64571E+00_JPRB, 0.64962E+00_JPRB, 0.65473E+00_JPRB, 0.65573E+00_JPRB, 0.64768E+00_JPRB, 0.62749E+00_JPRB ,&
&  0.60607E+00_JPRB, 0.60025E+00_JPRB, 0.60407E+00_JPRB, 0.61304E+00_JPRB, 0.61751E+00_JPRB, 0.61684E+00_JPRB, 0.61012E+00_JPRB ,&
&  0.60575E+00_JPRB, 0.59722E+00_JPRB, 0.58829E+00_JPRB, 0.58393E+00_JPRB, 0.58366E+00_JPRB, 0.58983E+00_JPRB, 0.60741E+00_JPRB ,&
&  0.62852E+00_JPRB, 0.65207E+00_JPRB, 0.67305E+00_JPRB, 0.68127E+00_JPRB, 0.68626E+00_JPRB, 0.68604E+00_JPRB, 0.67749E+00_JPRB ,&
&  0.68217E+00_JPRB, 0.70165E+00_JPRB, 0.71222E+00_JPRB, 0.71435E+00_JPRB, 0.71235E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.64322E+00_JPRB, 0.61465E+00_JPRB, 0.60269E+00_JPRB, 0.60420E+00_JPRB, 0.60743E+00_JPRB, 0.60287E+00_JPRB, 0.60976E+00_JPRB ,&
&  0.62073E+00_JPRB, 0.62907E+00_JPRB, 0.63344E+00_JPRB, 0.62936E+00_JPRB, 0.62666E+00_JPRB, 0.62363E+00_JPRB, 0.60882E+00_JPRB ,&
&  0.60392E+00_JPRB, 0.60220E+00_JPRB, 0.60639E+00_JPRB, 0.61359E+00_JPRB, 0.62146E+00_JPRB, 0.63376E+00_JPRB, 0.64285E+00_JPRB ,&
&  0.64876E+00_JPRB, 0.65240E+00_JPRB, 0.65648E+00_JPRB, 0.65660E+00_JPRB, 0.65291E+00_JPRB, 0.64358E+00_JPRB, 0.63643E+00_JPRB ,&
&  0.63217E+00_JPRB, 0.62108E+00_JPRB, 0.61066E+00_JPRB, 0.60938E+00_JPRB, 0.61260E+00_JPRB, 0.61581E+00_JPRB, 0.61584E+00_JPRB ,&
&  0.60396E+00_JPRB, 0.58790E+00_JPRB, 0.58655E+00_JPRB, 0.58620E+00_JPRB, 0.59107E+00_JPRB, 0.60304E+00_JPRB, 0.60617E+00_JPRB ,&
&  0.60241E+00_JPRB, 0.58775E+00_JPRB, 0.56820E+00_JPRB, 0.56440E+00_JPRB, 0.56664E+00_JPRB, 0.55962E+00_JPRB, 0.55281E+00_JPRB ,&
&  0.55162E+00_JPRB, 0.54870E+00_JPRB, 0.54275E+00_JPRB, 0.53458E+00_JPRB, 0.52172E+00_JPRB, 0.50503E+00_JPRB, 0.49649E+00_JPRB ,&
&  0.49594E+00_JPRB, 0.51126E+00_JPRB, 0.53873E+00_JPRB, 0.56969E+00_JPRB, 0.59908E+00_JPRB, 0.61339E+00_JPRB, 0.61279E+00_JPRB ,&
&  0.60068E+00_JPRB, 0.59765E+00_JPRB, 0.60280E+00_JPRB, 0.60730E+00_JPRB, 0.60845E+00_JPRB, 0.59776E+00_JPRB, 0.58038E+00_JPRB ,&
&  0.56769E+00_JPRB, 0.55828E+00_JPRB, 0.55720E+00_JPRB, 0.56218E+00_JPRB, 0.56704E+00_JPRB, 0.56818E+00_JPRB, 0.56211E+00_JPRB ,&
&  0.55578E+00_JPRB, 0.54911E+00_JPRB, 0.54510E+00_JPRB, 0.54767E+00_JPRB, 0.55273E+00_JPRB, 0.55791E+00_JPRB, 0.57095E+00_JPRB ,&
&  0.58978E+00_JPRB, 0.60864E+00_JPRB, 0.61308E+00_JPRB, 0.60923E+00_JPRB, 0.60906E+00_JPRB, 0.60637E+00_JPRB, 0.60380E+00_JPRB ,&
&  0.61400E+00_JPRB, 0.63680E+00_JPRB, 0.65351E+00_JPRB, 0.66312E+00_JPRB, 0.66364E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.56673E+00_JPRB, 0.54968E+00_JPRB, 0.54995E+00_JPRB, 0.55716E+00_JPRB, 0.56124E+00_JPRB, 0.56891E+00_JPRB, 0.58815E+00_JPRB ,&
&  0.59926E+00_JPRB, 0.60633E+00_JPRB, 0.60900E+00_JPRB, 0.60351E+00_JPRB, 0.60239E+00_JPRB, 0.59508E+00_JPRB, 0.58225E+00_JPRB ,&
&  0.57690E+00_JPRB, 0.57587E+00_JPRB, 0.57756E+00_JPRB, 0.58143E+00_JPRB, 0.58375E+00_JPRB, 0.58670E+00_JPRB, 0.59005E+00_JPRB ,&
&  0.59184E+00_JPRB, 0.59180E+00_JPRB, 0.59088E+00_JPRB, 0.59104E+00_JPRB, 0.59000E+00_JPRB, 0.58227E+00_JPRB, 0.57350E+00_JPRB ,&
&  0.56406E+00_JPRB, 0.55497E+00_JPRB, 0.55228E+00_JPRB, 0.55651E+00_JPRB, 0.56138E+00_JPRB, 0.56432E+00_JPRB, 0.56202E+00_JPRB ,&
&  0.55763E+00_JPRB, 0.55571E+00_JPRB, 0.55863E+00_JPRB, 0.56557E+00_JPRB, 0.57401E+00_JPRB, 0.58353E+00_JPRB, 0.58854E+00_JPRB ,&
&  0.58273E+00_JPRB, 0.56480E+00_JPRB, 0.55380E+00_JPRB, 0.55316E+00_JPRB, 0.54920E+00_JPRB, 0.53418E+00_JPRB, 0.52375E+00_JPRB ,&
&  0.51895E+00_JPRB, 0.51008E+00_JPRB, 0.50071E+00_JPRB, 0.48803E+00_JPRB, 0.47646E+00_JPRB, 0.47211E+00_JPRB, 0.47081E+00_JPRB ,&
&  0.47382E+00_JPRB, 0.49213E+00_JPRB, 0.52595E+00_JPRB, 0.55878E+00_JPRB, 0.57312E+00_JPRB, 0.56667E+00_JPRB, 0.54967E+00_JPRB ,&
&  0.54172E+00_JPRB, 0.53822E+00_JPRB, 0.53952E+00_JPRB, 0.53931E+00_JPRB, 0.53605E+00_JPRB, 0.52727E+00_JPRB, 0.51998E+00_JPRB ,&
&  0.50958E+00_JPRB, 0.49811E+00_JPRB, 0.49349E+00_JPRB, 0.49077E+00_JPRB, 0.48787E+00_JPRB, 0.48803E+00_JPRB, 0.48728E+00_JPRB ,&
&  0.48796E+00_JPRB, 0.49185E+00_JPRB, 0.50330E+00_JPRB, 0.50943E+00_JPRB, 0.51003E+00_JPRB, 0.49964E+00_JPRB, 0.50071E+00_JPRB ,&
&  0.50367E+00_JPRB, 0.50157E+00_JPRB, 0.49708E+00_JPRB, 0.49699E+00_JPRB, 0.49704E+00_JPRB, 0.49914E+00_JPRB, 0.51109E+00_JPRB ,&
&  0.53820E+00_JPRB, 0.56184E+00_JPRB, 0.58133E+00_JPRB, 0.58727E+00_JPRB, 0.58370E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.50951E+00_JPRB, 0.51484E+00_JPRB, 0.52809E+00_JPRB, 0.53903E+00_JPRB, 0.54250E+00_JPRB, 0.55197E+00_JPRB, 0.56298E+00_JPRB ,&
&  0.57185E+00_JPRB, 0.57342E+00_JPRB, 0.56954E+00_JPRB, 0.55925E+00_JPRB, 0.54714E+00_JPRB, 0.53716E+00_JPRB, 0.53322E+00_JPRB ,&
&  0.53544E+00_JPRB, 0.53854E+00_JPRB, 0.54413E+00_JPRB, 0.54722E+00_JPRB, 0.55097E+00_JPRB, 0.54985E+00_JPRB, 0.55040E+00_JPRB ,&
&  0.55232E+00_JPRB, 0.55340E+00_JPRB, 0.55392E+00_JPRB, 0.55018E+00_JPRB, 0.54717E+00_JPRB, 0.54454E+00_JPRB, 0.53580E+00_JPRB ,&
&  0.52611E+00_JPRB, 0.51970E+00_JPRB, 0.52359E+00_JPRB, 0.53485E+00_JPRB, 0.54195E+00_JPRB, 0.53889E+00_JPRB, 0.53734E+00_JPRB ,&
&  0.53756E+00_JPRB, 0.54274E+00_JPRB, 0.55108E+00_JPRB, 0.56325E+00_JPRB, 0.57152E+00_JPRB, 0.58240E+00_JPRB, 0.58299E+00_JPRB ,&
&  0.57651E+00_JPRB, 0.56713E+00_JPRB, 0.56406E+00_JPRB, 0.56272E+00_JPRB, 0.53332E+00_JPRB, 0.50105E+00_JPRB, 0.48526E+00_JPRB ,&
&  0.47679E+00_JPRB, 0.46630E+00_JPRB, 0.45050E+00_JPRB, 0.43948E+00_JPRB, 0.43840E+00_JPRB, 0.44338E+00_JPRB, 0.45487E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.47563E+00_JPRB, 0.49086E+00_JPRB, 0.50080E+00_JPRB, 0.50099E+00_JPRB, 0.48873E+00_JPRB, 0.47733E+00_JPRB ,&
&  0.47052E+00_JPRB, 0.46850E+00_JPRB, 0.46691E+00_JPRB, 0.46486E+00_JPRB, 0.46296E+00_JPRB, 0.46244E+00_JPRB, 0.46144E+00_JPRB ,&
&  0.45721E+00_JPRB, 0.44990E+00_JPRB, 0.44102E+00_JPRB, 0.43613E+00_JPRB, 0.43538E+00_JPRB, 0.43563E+00_JPRB, 0.43746E+00_JPRB ,&
&  0.44645E+00_JPRB, 0.45772E+00_JPRB, 0.46668E+00_JPRB, 0.47047E+00_JPRB, 0.46715E+00_JPRB, 0.45894E+00_JPRB, 0.45396E+00_JPRB ,&
&  0.45023E+00_JPRB, 0.44762E+00_JPRB, 0.44423E+00_JPRB, 0.44271E+00_JPRB, 0.44264E+00_JPRB, 0.45028E+00_JPRB, 0.46647E+00_JPRB ,&
&  0.48640E+00_JPRB, 0.51212E+00_JPRB, 0.52377E+00_JPRB, 0.52237E+00_JPRB, 0.51550E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.47890E+00_JPRB, 0.49194E+00_JPRB, 0.50832E+00_JPRB, 0.51983E+00_JPRB, 0.52465E+00_JPRB, 0.52992E+00_JPRB, 0.53220E+00_JPRB ,&
&  0.52683E+00_JPRB, 0.52452E+00_JPRB, 0.51891E+00_JPRB, 0.51088E+00_JPRB, 0.50395E+00_JPRB, 0.50050E+00_JPRB, 0.50093E+00_JPRB ,&
&  0.50312E+00_JPRB, 0.50502E+00_JPRB, 0.50163E+00_JPRB, 0.49993E+00_JPRB, 0.50394E+00_JPRB, 0.50939E+00_JPRB, 0.51299E+00_JPRB ,&
&  0.51454E+00_JPRB, 0.51613E+00_JPRB, 0.52097E+00_JPRB, 0.52404E+00_JPRB, 0.52415E+00_JPRB, 0.51751E+00_JPRB, 0.51250E+00_JPRB ,&
&  0.50743E+00_JPRB, 0.50562E+00_JPRB, 0.51189E+00_JPRB, 0.50403E+00_JPRB, 0.51858E+00_JPRB, 0.52027E+00_JPRB, 0.52556E+00_JPRB ,&
&  0.53356E+00_JPRB, 0.54717E+00_JPRB, 0.55791E+00_JPRB, 0.56621E+00_JPRB, 0.56869E+00_JPRB, 0.57178E+00_JPRB, 0.56184E+00_JPRB ,&
&  0.54804E+00_JPRB, 0.52936E+00_JPRB, 0.51057E+00_JPRB, 0.49005E+00_JPRB, 0.47004E+00_JPRB, 0.45454E+00_JPRB, 0.44711E+00_JPRB ,&
&  0.43807E+00_JPRB, 0.42412E+00_JPRB, 0.41079E+00_JPRB, 0.40565E+00_JPRB, 0.41034E+00_JPRB, 0.42052E+00_JPRB, 0.43600E+00_JPRB ,&
&  0.44546E+00_JPRB, 0.44797E+00_JPRB, 0.44472E+00_JPRB, 0.44054E+00_JPRB, 0.43622E+00_JPRB, 0.42885E+00_JPRB, 0.42451E+00_JPRB ,&
&  0.42102E+00_JPRB, 0.41540E+00_JPRB, 0.41259E+00_JPRB, 0.41423E+00_JPRB, 0.41729E+00_JPRB, 0.41934E+00_JPRB, 0.41803E+00_JPRB ,&
&  0.41531E+00_JPRB, 0.41076E+00_JPRB, 0.40274E+00_JPRB, 0.39854E+00_JPRB, 0.39914E+00_JPRB, 0.40378E+00_JPRB, 0.41142E+00_JPRB ,&
&  0.41389E+00_JPRB, 0.41998E+00_JPRB, 0.42581E+00_JPRB, 0.42779E+00_JPRB, 0.43021E+00_JPRB, 0.43207E+00_JPRB, 0.43202E+00_JPRB ,&
&  0.43012E+00_JPRB, 0.42500E+00_JPRB, 0.41807E+00_JPRB, 0.41537E+00_JPRB, 0.41677E+00_JPRB, 0.42389E+00_JPRB, 0.43338E+00_JPRB ,&
&  0.44271E+00_JPRB, 0.45692E+00_JPRB, 0.46781E+00_JPRB, 0.47030E+00_JPRB, 0.47259E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.45834E+00_JPRB, 0.46998E+00_JPRB, 0.48354E+00_JPRB, 0.49287E+00_JPRB, 0.49176E+00_JPRB, 0.49158E+00_JPRB, 0.49757E+00_JPRB ,&
&  0.49119E+00_JPRB, 0.48377E+00_JPRB, 0.47780E+00_JPRB, 0.47175E+00_JPRB, 0.46973E+00_JPRB, 0.46855E+00_JPRB, 0.47167E+00_JPRB ,&
&  0.47397E+00_JPRB, 0.47300E+00_JPRB, 0.47150E+00_JPRB, 0.47240E+00_JPRB, 0.47916E+00_JPRB, 0.48759E+00_JPRB, 0.49464E+00_JPRB ,&
&  0.49611E+00_JPRB, 0.49980E+00_JPRB, 0.50568E+00_JPRB, 0.51070E+00_JPRB, 0.51320E+00_JPRB, 0.51468E+00_JPRB, 0.51339E+00_JPRB ,&
&  0.51073E+00_JPRB, 0.50806E+00_JPRB, 0.49102E+00_JPRB, 0.50410E+00_JPRB, 0.50856E+00_JPRB, 0.50926E+00_JPRB, 0.51433E+00_JPRB ,&
&  0.52196E+00_JPRB, 0.53085E+00_JPRB, 0.53507E+00_JPRB, 0.53643E+00_JPRB, 0.53166E+00_JPRB, 0.52217E+00_JPRB, 0.49769E+00_JPRB ,&
&  0.47711E+00_JPRB, 0.45581E+00_JPRB, 0.44248E+00_JPRB, 0.43429E+00_JPRB, 0.42776E+00_JPRB, 0.42221E+00_JPRB, 0.41711E+00_JPRB ,&
&  0.41005E+00_JPRB, 0.40376E+00_JPRB, 0.39991E+00_JPRB, 0.39987E+00_JPRB, 0.40390E+00_JPRB, 0.41470E+00_JPRB, 0.42800E+00_JPRB ,&
&  0.43373E+00_JPRB, 0.42583E+00_JPRB, 0.41753E+00_JPRB, 0.40850E+00_JPRB, 0.39353E+00_JPRB, 0.36645E+00_JPRB, 0.34983E+00_JPRB ,&
&  0.34597E+00_JPRB, 0.35852E+00_JPRB, 0.37930E+00_JPRB, 0.38540E+00_JPRB, 0.39099E+00_JPRB, 0.38867E+00_JPRB, 0.38327E+00_JPRB ,&
&  0.37397E+00_JPRB, 0.35105E+00_JPRB, 0.33911E+00_JPRB, 0.33580E+00_JPRB, 0.33380E+00_JPRB, 0.33448E+00_JPRB, 0.33728E+00_JPRB ,&
&  0.34174E+00_JPRB, 0.34886E+00_JPRB, 0.35379E+00_JPRB, 0.35572E+00_JPRB, 0.35861E+00_JPRB, 0.35877E+00_JPRB, 0.35762E+00_JPRB ,&
&  0.35411E+00_JPRB, 0.36105E+00_JPRB, 0.37975E+00_JPRB, 0.40784E+00_JPRB, 0.41438E+00_JPRB, 0.42229E+00_JPRB, 0.43127E+00_JPRB ,&
&  0.43678E+00_JPRB, 0.44222E+00_JPRB, 0.44295E+00_JPRB, 0.44388E+00_JPRB, 0.44922E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.43402E+00_JPRB, 0.44110E+00_JPRB, 0.44960E+00_JPRB, 0.46064E+00_JPRB, 0.46297E+00_JPRB, 0.46451E+00_JPRB, 0.47353E+00_JPRB ,&
&  0.47374E+00_JPRB, 0.47065E+00_JPRB, 0.46743E+00_JPRB, 0.46780E+00_JPRB, 0.47006E+00_JPRB, 0.47261E+00_JPRB, 0.47234E+00_JPRB ,&
&  0.46333E+00_JPRB, 0.45822E+00_JPRB, 0.46113E+00_JPRB, 0.46430E+00_JPRB, 0.46899E+00_JPRB, 0.47550E+00_JPRB, 0.48124E+00_JPRB ,&
&  0.48442E+00_JPRB, 0.48818E+00_JPRB, 0.49087E+00_JPRB, 0.49724E+00_JPRB, 0.50691E+00_JPRB, 0.51363E+00_JPRB, 0.51973E+00_JPRB ,&
&  0.50469E+00_JPRB, 0.45684E+00_JPRB, 0.42661E+00_JPRB, 0.43270E+00_JPRB, 0.48087E+00_JPRB, 0.49136E+00_JPRB, 0.49220E+00_JPRB ,&
&  0.49509E+00_JPRB, 0.50255E+00_JPRB, 0.50203E+00_JPRB, 0.49622E+00_JPRB, 0.48426E+00_JPRB, 0.46507E+00_JPRB, 0.44170E+00_JPRB ,&
&  0.42611E+00_JPRB, 0.41589E+00_JPRB, 0.40909E+00_JPRB, 0.40222E+00_JPRB, 0.39867E+00_JPRB, 0.39379E+00_JPRB, 0.38612E+00_JPRB ,&
&  0.38306E+00_JPRB, 0.38185E+00_JPRB, 0.38303E+00_JPRB, 0.38449E+00_JPRB, 0.38626E+00_JPRB, 0.38733E+00_JPRB, 0.38347E+00_JPRB ,&
&  0.37136E+00_JPRB, 0.35550E+00_JPRB, 0.34039E+00_JPRB, 0.32394E+00_JPRB, 0.31003E+00_JPRB, 0.30143E+00_JPRB, 0.29445E+00_JPRB ,&
&  0.29176E+00_JPRB, 0.29787E+00_JPRB, 0.31555E+00_JPRB, 0.33558E+00_JPRB, 0.36612E+00_JPRB, 0.34116E+00_JPRB, 0.31781E+00_JPRB ,&
&  0.30680E+00_JPRB, 0.30142E+00_JPRB, 0.29845E+00_JPRB, 0.29863E+00_JPRB, 0.30039E+00_JPRB, 0.30092E+00_JPRB, 0.30106E+00_JPRB ,&
&  0.30251E+00_JPRB, 0.30512E+00_JPRB, 0.30703E+00_JPRB, 0.30869E+00_JPRB, 0.31125E+00_JPRB, 0.31418E+00_JPRB, 0.31572E+00_JPRB ,&
&  0.31654E+00_JPRB, 0.32152E+00_JPRB, 0.33137E+00_JPRB, 0.34282E+00_JPRB, 0.35411E+00_JPRB, 0.36587E+00_JPRB, 0.38337E+00_JPRB ,&
&  0.40493E+00_JPRB, 0.43034E+00_JPRB, 0.43012E+00_JPRB, 0.43075E+00_JPRB, 0.43106E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.42116E+00_JPRB, 0.43184E+00_JPRB, 0.43723E+00_JPRB, 0.44259E+00_JPRB, 0.44485E+00_JPRB, 0.44625E+00_JPRB, 0.45419E+00_JPRB ,&
&  0.45902E+00_JPRB, 0.46286E+00_JPRB, 0.46528E+00_JPRB, 0.46655E+00_JPRB, 0.46265E+00_JPRB, 0.45620E+00_JPRB, 0.45439E+00_JPRB ,&
&  0.45332E+00_JPRB, 0.45388E+00_JPRB, 0.45294E+00_JPRB, 0.45298E+00_JPRB, 0.45349E+00_JPRB, 0.45825E+00_JPRB, 0.46519E+00_JPRB ,&
&  0.46688E+00_JPRB, 0.47128E+00_JPRB, 0.48420E+00_JPRB, 0.49196E+00_JPRB, 0.49686E+00_JPRB, 0.49625E+00_JPRB, 0.49028E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.43376E+00_JPRB, 0.40694E+00_JPRB, 0.40834E+00_JPRB, 0.46050E+00_JPRB, 0.46644E+00_JPRB, 0.46601E+00_JPRB ,&
&  0.46450E+00_JPRB, 0.46866E+00_JPRB, 0.46880E+00_JPRB, 0.45409E+00_JPRB, 0.44338E+00_JPRB, 0.42879E+00_JPRB, 0.41606E+00_JPRB ,&
&  0.40487E+00_JPRB, 0.39719E+00_JPRB, 0.38374E+00_JPRB, 0.35614E+00_JPRB, 0.33904E+00_JPRB, 0.33033E+00_JPRB, 0.31661E+00_JPRB ,&
&  0.30620E+00_JPRB, 0.30262E+00_JPRB, 0.30179E+00_JPRB, 0.30089E+00_JPRB, 0.30486E+00_JPRB, 0.30691E+00_JPRB, 0.30539E+00_JPRB ,&
&  0.30168E+00_JPRB, 0.29861E+00_JPRB, 0.29471E+00_JPRB, 0.28942E+00_JPRB, 0.28662E+00_JPRB, 0.28581E+00_JPRB, 0.28640E+00_JPRB ,&
&  0.29058E+00_JPRB, 0.29684E+00_JPRB, 0.31060E+00_JPRB, 0.32022E+00_JPRB, 0.31593E+00_JPRB, 0.30439E+00_JPRB, 0.29420E+00_JPRB ,&
&  0.28738E+00_JPRB, 0.28463E+00_JPRB, 0.28257E+00_JPRB, 0.28039E+00_JPRB, 0.28105E+00_JPRB, 0.28320E+00_JPRB, 0.28465E+00_JPRB ,&
&  0.28490E+00_JPRB, 0.28676E+00_JPRB, 0.28790E+00_JPRB, 0.28920E+00_JPRB, 0.29032E+00_JPRB, 0.29214E+00_JPRB, 0.29464E+00_JPRB ,&
&  0.29695E+00_JPRB, 0.30232E+00_JPRB, 0.30823E+00_JPRB, 0.31493E+00_JPRB, 0.31913E+00_JPRB, 0.32229E+00_JPRB, 0.32878E+00_JPRB ,&
&  0.33506E+00_JPRB, 0.35078E+00_JPRB, 0.38214E+00_JPRB, 0.41821E+00_JPRB, 0.41145E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.40860E+00_JPRB, 0.41738E+00_JPRB, 0.42508E+00_JPRB, 0.43104E+00_JPRB, 0.43488E+00_JPRB, 0.43569E+00_JPRB, 0.43850E+00_JPRB ,&
&  0.44610E+00_JPRB, 0.45057E+00_JPRB, 0.45401E+00_JPRB, 0.45318E+00_JPRB, 0.45263E+00_JPRB, 0.43430E+00_JPRB, 0.43003E+00_JPRB ,&
&  0.42401E+00_JPRB, 0.41980E+00_JPRB, 0.41383E+00_JPRB, 0.41367E+00_JPRB, 0.41338E+00_JPRB, 0.41545E+00_JPRB, 0.41624E+00_JPRB ,&
&  0.41665E+00_JPRB, 0.41512E+00_JPRB, 0.41212E+00_JPRB, 0.41031E+00_JPRB, 0.41155E+00_JPRB, 0.41279E+00_JPRB, 0.40972E+00_JPRB ,&
&  0.40530E+00_JPRB, 0.40418E+00_JPRB, 0.40727E+00_JPRB, 0.41418E+00_JPRB, 0.45396E+00_JPRB, 0.44838E+00_JPRB, 0.44626E+00_JPRB ,&
&  0.44293E+00_JPRB, 0.43869E+00_JPRB, 0.43799E+00_JPRB, 0.43129E+00_JPRB, 0.42160E+00_JPRB, 0.39953E+00_JPRB, 0.38723E+00_JPRB ,&
&  0.35299E+00_JPRB, 0.33422E+00_JPRB, 0.31777E+00_JPRB, 0.30713E+00_JPRB, 0.29669E+00_JPRB, 0.29063E+00_JPRB, 0.28527E+00_JPRB ,&
&  0.28120E+00_JPRB, 0.27794E+00_JPRB, 0.27503E+00_JPRB, 0.27566E+00_JPRB, 0.27752E+00_JPRB, 0.27848E+00_JPRB, 0.27948E+00_JPRB ,&
&  0.28005E+00_JPRB, 0.28057E+00_JPRB, 0.28119E+00_JPRB, 0.28259E+00_JPRB, 0.28447E+00_JPRB, 0.28695E+00_JPRB, 0.29011E+00_JPRB ,&
&  0.29523E+00_JPRB, 0.29918E+00_JPRB, 0.30063E+00_JPRB, 0.30338E+00_JPRB, 0.30309E+00_JPRB, 0.29714E+00_JPRB, 0.29041E+00_JPRB ,&
&  0.28337E+00_JPRB, 0.27969E+00_JPRB, 0.27606E+00_JPRB, 0.27350E+00_JPRB, 0.27287E+00_JPRB, 0.27282E+00_JPRB, 0.27480E+00_JPRB ,&
&  0.27699E+00_JPRB, 0.27897E+00_JPRB, 0.28037E+00_JPRB, 0.28187E+00_JPRB, 0.28380E+00_JPRB, 0.28524E+00_JPRB, 0.28713E+00_JPRB ,&
&  0.28956E+00_JPRB, 0.29286E+00_JPRB, 0.29774E+00_JPRB, 0.30342E+00_JPRB, 0.31076E+00_JPRB, 0.32156E+00_JPRB, 0.34038E+00_JPRB ,&
&  0.36499E+00_JPRB, 0.42005E+00_JPRB, 0.42470E+00_JPRB, 0.39536E+00_JPRB, 0.40165E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.38510E+00_JPRB, 0.39207E+00_JPRB, 0.39911E+00_JPRB, 0.40666E+00_JPRB, 0.41614E+00_JPRB, 0.42026E+00_JPRB, 0.42224E+00_JPRB ,&
&  0.41811E+00_JPRB, 0.41367E+00_JPRB, 0.40297E+00_JPRB, 0.39345E+00_JPRB, 0.38416E+00_JPRB, 0.37680E+00_JPRB, 0.37098E+00_JPRB ,&
&  0.36744E+00_JPRB, 0.36471E+00_JPRB, 0.36038E+00_JPRB, 0.35693E+00_JPRB, 0.35385E+00_JPRB, 0.35395E+00_JPRB, 0.35433E+00_JPRB ,&
&  0.35385E+00_JPRB, 0.35425E+00_JPRB, 0.35288E+00_JPRB, 0.35805E+00_JPRB, 0.36729E+00_JPRB, 0.37845E+00_JPRB, 0.39302E+00_JPRB ,&
&  0.41341E+00_JPRB, 0.42440E+00_JPRB, 0.43985E+00_JPRB, 0.44005E+00_JPRB, 0.43818E+00_JPRB, 0.43437E+00_JPRB, 0.43124E+00_JPRB ,&
&  0.42651E+00_JPRB, 0.41973E+00_JPRB, 0.41468E+00_JPRB, 0.40827E+00_JPRB, 0.37857E+00_JPRB, 0.35550E+00_JPRB, 0.33774E+00_JPRB ,&
&  0.32257E+00_JPRB, 0.30826E+00_JPRB, 0.29559E+00_JPRB, 0.28926E+00_JPRB, 0.28405E+00_JPRB, 0.28097E+00_JPRB, 0.27774E+00_JPRB ,&
&  0.27427E+00_JPRB, 0.27315E+00_JPRB, 0.27128E+00_JPRB, 0.27061E+00_JPRB, 0.27173E+00_JPRB, 0.27166E+00_JPRB, 0.27247E+00_JPRB ,&
&  0.27321E+00_JPRB, 0.27440E+00_JPRB, 0.27711E+00_JPRB, 0.27917E+00_JPRB, 0.28126E+00_JPRB, 0.28323E+00_JPRB, 0.28414E+00_JPRB ,&
&  0.28604E+00_JPRB, 0.28767E+00_JPRB, 0.28762E+00_JPRB, 0.28684E+00_JPRB, 0.28499E+00_JPRB, 0.28078E+00_JPRB, 0.27661E+00_JPRB ,&
&  0.27210E+00_JPRB, 0.26884E+00_JPRB, 0.26763E+00_JPRB, 0.26659E+00_JPRB, 0.26748E+00_JPRB, 0.26827E+00_JPRB, 0.26911E+00_JPRB ,&
&  0.27101E+00_JPRB, 0.27281E+00_JPRB, 0.27494E+00_JPRB, 0.27806E+00_JPRB, 0.28039E+00_JPRB, 0.28239E+00_JPRB, 0.28490E+00_JPRB ,&
&  0.28500E+00_JPRB, 0.28935E+00_JPRB, 0.29481E+00_JPRB, 0.30213E+00_JPRB, 0.31529E+00_JPRB, 0.32555E+00_JPRB, 0.34864E+00_JPRB ,&
&  0.38364E+00_JPRB, 0.44896E+00_JPRB, 0.44196E+00_JPRB, 0.37486E+00_JPRB, 0.37879E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.37330E+00_JPRB, 0.37640E+00_JPRB, 0.37970E+00_JPRB, 0.38377E+00_JPRB, 0.38877E+00_JPRB, 0.39331E+00_JPRB, 0.39788E+00_JPRB ,&
&  0.40070E+00_JPRB, 0.39786E+00_JPRB, 0.39878E+00_JPRB, 0.39427E+00_JPRB, 0.38968E+00_JPRB, 0.38366E+00_JPRB, 0.37747E+00_JPRB ,&
&  0.37112E+00_JPRB, 0.36208E+00_JPRB, 0.35245E+00_JPRB, 0.34395E+00_JPRB, 0.33672E+00_JPRB, 0.33118E+00_JPRB, 0.32679E+00_JPRB ,&
&  0.32377E+00_JPRB, 0.32431E+00_JPRB, 0.32641E+00_JPRB, 0.33248E+00_JPRB, 0.34260E+00_JPRB, 0.35504E+00_JPRB, 0.37402E+00_JPRB ,&
&  0.40068E+00_JPRB, 0.40279E+00_JPRB, 0.40825E+00_JPRB, 0.40734E+00_JPRB, 0.40312E+00_JPRB, 0.40119E+00_JPRB, 0.39758E+00_JPRB ,&
&  0.39823E+00_JPRB, 0.39017E+00_JPRB, 0.39987E+00_JPRB, 0.38284E+00_JPRB, 0.35543E+00_JPRB, 0.34004E+00_JPRB, 0.32165E+00_JPRB ,&
&  0.30845E+00_JPRB, 0.29721E+00_JPRB, 0.28884E+00_JPRB, 0.28545E+00_JPRB, 0.28155E+00_JPRB, 0.27974E+00_JPRB, 0.27590E+00_JPRB ,&
&  0.27196E+00_JPRB, 0.27013E+00_JPRB, 0.26699E+00_JPRB, 0.26687E+00_JPRB, 0.26705E+00_JPRB, 0.26647E+00_JPRB, 0.26660E+00_JPRB ,&
&  0.26705E+00_JPRB, 0.26820E+00_JPRB, 0.27025E+00_JPRB, 0.27210E+00_JPRB, 0.27321E+00_JPRB, 0.27385E+00_JPRB, 0.27367E+00_JPRB ,&
&  0.27415E+00_JPRB, 0.27381E+00_JPRB, 0.27294E+00_JPRB, 0.27158E+00_JPRB, 0.26949E+00_JPRB, 0.26731E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26346E+00_JPRB, 0.26278E+00_JPRB, 0.26216E+00_JPRB, 0.26231E+00_JPRB, 0.26284E+00_JPRB, 0.26461E+00_JPRB, 0.26628E+00_JPRB ,&
&  0.26819E+00_JPRB, 0.27094E+00_JPRB, 0.27333E+00_JPRB, 0.27627E+00_JPRB, 0.27893E+00_JPRB, 0.28038E+00_JPRB, 0.28451E+00_JPRB ,&
&  0.28985E+00_JPRB, 0.29717E+00_JPRB, 0.30581E+00_JPRB, 0.31401E+00_JPRB, 0.32448E+00_JPRB, 0.34006E+00_JPRB, 0.36098E+00_JPRB ,&
&  0.38218E+00_JPRB, 0.40423E+00_JPRB, 0.39765E+00_JPRB, 0.37073E+00_JPRB, 0.36992E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.37333E+00_JPRB, 0.37177E+00_JPRB, 0.37254E+00_JPRB, 0.37379E+00_JPRB, 0.37782E+00_JPRB, 0.38342E+00_JPRB, 0.38946E+00_JPRB ,&
&  0.38914E+00_JPRB, 0.39241E+00_JPRB, 0.39146E+00_JPRB, 0.38861E+00_JPRB, 0.38392E+00_JPRB, 0.37916E+00_JPRB, 0.37431E+00_JPRB ,&
&  0.36752E+00_JPRB, 0.35882E+00_JPRB, 0.35038E+00_JPRB, 0.34174E+00_JPRB, 0.33357E+00_JPRB, 0.32512E+00_JPRB, 0.32029E+00_JPRB ,&
&  0.31697E+00_JPRB, 0.31612E+00_JPRB, 0.31813E+00_JPRB, 0.32077E+00_JPRB, 0.32609E+00_JPRB, 0.33338E+00_JPRB, 0.34307E+00_JPRB ,&
&  0.35246E+00_JPRB, 0.36323E+00_JPRB, 0.36889E+00_JPRB, 0.36877E+00_JPRB, 0.36856E+00_JPRB, 0.36649E+00_JPRB, 0.36305E+00_JPRB ,&
&  0.35642E+00_JPRB, 0.34898E+00_JPRB, 0.34031E+00_JPRB, 0.33114E+00_JPRB, 0.32178E+00_JPRB, 0.31306E+00_JPRB, 0.30497E+00_JPRB ,&
&  0.29841E+00_JPRB, 0.29283E+00_JPRB, 0.28769E+00_JPRB, 0.28337E+00_JPRB, 0.28004E+00_JPRB, 0.27758E+00_JPRB, 0.27452E+00_JPRB ,&
&  0.27199E+00_JPRB, 0.26976E+00_JPRB, 0.26708E+00_JPRB, 0.26526E+00_JPRB, 0.26363E+00_JPRB, 0.26258E+00_JPRB, 0.26172E+00_JPRB ,&
&  0.26171E+00_JPRB, 0.26170E+00_JPRB, 0.26242E+00_JPRB, 0.26312E+00_JPRB, 0.26393E+00_JPRB, 0.26391E+00_JPRB, 0.26371E+00_JPRB ,&
&  0.26329E+00_JPRB, 0.26288E+00_JPRB, 0.26142E+00_JPRB, 0.26028E+00_JPRB, 0.25947E+00_JPRB, 0.25858E+00_JPRB, 0.25850E+00_JPRB ,&
&  0.25932E+00_JPRB, 0.26016E+00_JPRB, 0.26081E+00_JPRB, 0.26244E+00_JPRB, 0.26406E+00_JPRB, 0.26643E+00_JPRB, 0.26932E+00_JPRB ,&
&  0.27140E+00_JPRB, 0.27387E+00_JPRB, 0.27660E+00_JPRB, 0.27977E+00_JPRB, 0.28361E+00_JPRB, 0.28757E+00_JPRB, 0.29225E+00_JPRB ,&
&  0.29754E+00_JPRB, 0.30296E+00_JPRB, 0.30907E+00_JPRB, 0.31583E+00_JPRB, 0.32408E+00_JPRB, 0.33051E+00_JPRB, 0.34022E+00_JPRB ,&
&  0.34768E+00_JPRB, 0.34537E+00_JPRB, 0.38546E+00_JPRB, 0.37905E+00_JPRB, 0.37499E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.31638E+00_JPRB, 0.32287E+00_JPRB, 0.32739E+00_JPRB, 0.33189E+00_JPRB, 0.32599E+00_JPRB, 0.36296E+00_JPRB, 0.36121E+00_JPRB ,&
&  0.36195E+00_JPRB, 0.34944E+00_JPRB, 0.35054E+00_JPRB, 0.34956E+00_JPRB, 0.34692E+00_JPRB, 0.34296E+00_JPRB, 0.33923E+00_JPRB ,&
&  0.33593E+00_JPRB, 0.33136E+00_JPRB, 0.32594E+00_JPRB, 0.32129E+00_JPRB, 0.31760E+00_JPRB, 0.31447E+00_JPRB, 0.31174E+00_JPRB ,&
&  0.31110E+00_JPRB, 0.31236E+00_JPRB, 0.31479E+00_JPRB, 0.31338E+00_JPRB, 0.31421E+00_JPRB, 0.31599E+00_JPRB, 0.31816E+00_JPRB ,&
&  0.32019E+00_JPRB, 0.32197E+00_JPRB, 0.32199E+00_JPRB, 0.32025E+00_JPRB, 0.31925E+00_JPRB, 0.31701E+00_JPRB, 0.31491E+00_JPRB ,&
&  0.31232E+00_JPRB, 0.30945E+00_JPRB, 0.30606E+00_JPRB, 0.30335E+00_JPRB, 0.30037E+00_JPRB, 0.29674E+00_JPRB, 0.29324E+00_JPRB ,&
&  0.29024E+00_JPRB, 0.28754E+00_JPRB, 0.28455E+00_JPRB, 0.28178E+00_JPRB, 0.27913E+00_JPRB, 0.27643E+00_JPRB, 0.27403E+00_JPRB ,&
&  0.27221E+00_JPRB, 0.27068E+00_JPRB, 0.26915E+00_JPRB, 0.26755E+00_JPRB, 0.26600E+00_JPRB, 0.26451E+00_JPRB, 0.26330E+00_JPRB ,&
&  0.26318E+00_JPRB, 0.26250E+00_JPRB, 0.26184E+00_JPRB, 0.26150E+00_JPRB, 0.26159E+00_JPRB, 0.26110E+00_JPRB, 0.26089E+00_JPRB ,&
&  0.26084E+00_JPRB, 0.26087E+00_JPRB, 0.26111E+00_JPRB, 0.26141E+00_JPRB, 0.26185E+00_JPRB, 0.26241E+00_JPRB, 0.26332E+00_JPRB ,&
&  0.26440E+00_JPRB, 0.26571E+00_JPRB, 0.26711E+00_JPRB, 0.26855E+00_JPRB, 0.27001E+00_JPRB, 0.27156E+00_JPRB, 0.27330E+00_JPRB ,&
&  0.27504E+00_JPRB, 0.27672E+00_JPRB, 0.27836E+00_JPRB, 0.27998E+00_JPRB, 0.28269E+00_JPRB, 0.28529E+00_JPRB, 0.28807E+00_JPRB ,&
&  0.29108E+00_JPRB, 0.29464E+00_JPRB, 0.29840E+00_JPRB, 0.30247E+00_JPRB, 0.30705E+00_JPRB, 0.30842E+00_JPRB, 0.30949E+00_JPRB ,&
&  0.31010E+00_JPRB, 0.31309E+00_JPRB, 0.31484E+00_JPRB, 0.31437E+00_JPRB, 0.31387E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.28907E+00_JPRB, 0.28917E+00_JPRB, 0.28921E+00_JPRB, 0.28919E+00_JPRB, 0.28777E+00_JPRB, 0.28815E+00_JPRB, 0.28851E+00_JPRB ,&
&  0.28920E+00_JPRB, 0.29063E+00_JPRB, 0.29230E+00_JPRB, 0.29357E+00_JPRB, 0.29449E+00_JPRB, 0.29508E+00_JPRB, 0.29553E+00_JPRB ,&
&  0.29564E+00_JPRB, 0.29593E+00_JPRB, 0.29254E+00_JPRB, 0.29131E+00_JPRB, 0.29089E+00_JPRB, 0.29089E+00_JPRB, 0.29107E+00_JPRB ,&
&  0.29119E+00_JPRB, 0.29117E+00_JPRB, 0.29062E+00_JPRB, 0.29093E+00_JPRB, 0.29126E+00_JPRB, 0.29148E+00_JPRB, 0.29173E+00_JPRB ,&
&  0.29197E+00_JPRB, 0.29217E+00_JPRB, 0.29210E+00_JPRB, 0.29188E+00_JPRB, 0.29175E+00_JPRB, 0.29126E+00_JPRB, 0.29041E+00_JPRB ,&
&  0.28937E+00_JPRB, 0.28806E+00_JPRB, 0.28676E+00_JPRB, 0.28556E+00_JPRB, 0.28453E+00_JPRB, 0.28365E+00_JPRB, 0.28280E+00_JPRB ,&
&  0.28178E+00_JPRB, 0.28043E+00_JPRB, 0.27901E+00_JPRB, 0.27762E+00_JPRB, 0.27629E+00_JPRB, 0.27508E+00_JPRB, 0.27380E+00_JPRB ,&
&  0.27283E+00_JPRB, 0.27201E+00_JPRB, 0.27120E+00_JPRB, 0.27050E+00_JPRB, 0.26982E+00_JPRB, 0.26917E+00_JPRB, 0.26853E+00_JPRB ,&
&  0.26767E+00_JPRB, 0.26720E+00_JPRB, 0.26669E+00_JPRB, 0.26646E+00_JPRB, 0.26653E+00_JPRB, 0.26658E+00_JPRB, 0.26671E+00_JPRB ,&
&  0.26691E+00_JPRB, 0.26716E+00_JPRB, 0.26729E+00_JPRB, 0.26750E+00_JPRB, 0.26783E+00_JPRB, 0.26825E+00_JPRB, 0.26876E+00_JPRB ,&
&  0.26932E+00_JPRB, 0.26992E+00_JPRB, 0.27059E+00_JPRB, 0.27133E+00_JPRB, 0.27211E+00_JPRB, 0.27296E+00_JPRB, 0.27377E+00_JPRB ,&
&  0.27459E+00_JPRB, 0.27536E+00_JPRB, 0.27612E+00_JPRB, 0.27671E+00_JPRB, 0.27758E+00_JPRB, 0.27841E+00_JPRB, 0.27940E+00_JPRB ,&
&  0.28041E+00_JPRB, 0.28148E+00_JPRB, 0.28242E+00_JPRB, 0.28342E+00_JPRB, 0.28453E+00_JPRB, 0.28432E+00_JPRB, 0.28420E+00_JPRB ,&
&  0.28416E+00_JPRB, 0.28443E+00_JPRB, 0.28437E+00_JPRB, 0.28470E+00_JPRB, 0.28555E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB, 0.27360E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB ,&
&  0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB, 0.74793E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.76427E+00_JPRB, 0.76427E+00_JPRB, 0.76421E+00_JPRB, 0.76395E+00_JPRB, 0.76357E+00_JPRB, 0.76340E+00_JPRB, 0.76317E+00_JPRB ,&
&  0.76294E+00_JPRB, 0.76265E+00_JPRB, 0.76272E+00_JPRB, 0.76236E+00_JPRB, 0.76204E+00_JPRB, 0.76149E+00_JPRB, 0.76102E+00_JPRB ,&
&  0.76048E+00_JPRB, 0.76006E+00_JPRB, 0.75961E+00_JPRB, 0.75938E+00_JPRB, 0.75922E+00_JPRB, 0.75933E+00_JPRB, 0.75935E+00_JPRB ,&
&  0.75928E+00_JPRB, 0.75878E+00_JPRB, 0.75851E+00_JPRB, 0.75768E+00_JPRB, 0.75674E+00_JPRB, 0.75517E+00_JPRB, 0.75338E+00_JPRB ,&
&  0.75097E+00_JPRB, 0.74880E+00_JPRB, 0.74700E+00_JPRB, 0.74552E+00_JPRB, 0.74433E+00_JPRB, 0.74353E+00_JPRB, 0.74284E+00_JPRB ,&
&  0.74204E+00_JPRB, 0.74144E+00_JPRB, 0.74094E+00_JPRB, 0.74036E+00_JPRB, 0.73939E+00_JPRB, 0.73829E+00_JPRB, 0.73722E+00_JPRB ,&
&  0.73598E+00_JPRB, 0.73455E+00_JPRB, 0.73355E+00_JPRB, 0.73280E+00_JPRB, 0.73178E+00_JPRB, 0.72998E+00_JPRB, 0.72812E+00_JPRB ,&
&  0.72643E+00_JPRB, 0.72454E+00_JPRB, 0.72252E+00_JPRB, 0.72060E+00_JPRB, 0.71926E+00_JPRB, 0.71803E+00_JPRB, 0.71685E+00_JPRB ,&
&  0.71669E+00_JPRB, 0.71764E+00_JPRB, 0.71858E+00_JPRB, 0.71934E+00_JPRB, 0.72057E+00_JPRB, 0.72193E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.72496E+00_JPRB, 0.72676E+00_JPRB, 0.72911E+00_JPRB, 0.73126E+00_JPRB, 0.73369E+00_JPRB, 0.73617E+00_JPRB, 0.73899E+00_JPRB ,&
&  0.74165E+00_JPRB, 0.74444E+00_JPRB, 0.74687E+00_JPRB, 0.74942E+00_JPRB, 0.75100E+00_JPRB, 0.75255E+00_JPRB, 0.75356E+00_JPRB ,&
&  0.75464E+00_JPRB, 0.75546E+00_JPRB, 0.75649E+00_JPRB, 0.75724E+00_JPRB, 0.75818E+00_JPRB, 0.75890E+00_JPRB, 0.75968E+00_JPRB ,&
&  0.76036E+00_JPRB, 0.76113E+00_JPRB, 0.76164E+00_JPRB, 0.76211E+00_JPRB, 0.76230E+00_JPRB, 0.76279E+00_JPRB, 0.76311E+00_JPRB ,&
&  0.76343E+00_JPRB, 0.76376E+00_JPRB, 0.76411E+00_JPRB, 0.76442E+00_JPRB, 0.76450E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.76524E+00_JPRB, 0.76715E+00_JPRB, 0.76785E+00_JPRB, 0.76860E+00_JPRB, 0.76890E+00_JPRB, 0.76999E+00_JPRB, 0.77070E+00_JPRB ,&
&  0.77137E+00_JPRB, 0.77240E+00_JPRB, 0.77433E+00_JPRB, 0.77496E+00_JPRB, 0.77527E+00_JPRB, 0.77494E+00_JPRB, 0.77459E+00_JPRB ,&
&  0.77380E+00_JPRB, 0.77260E+00_JPRB, 0.77134E+00_JPRB, 0.77045E+00_JPRB, 0.76961E+00_JPRB, 0.76910E+00_JPRB, 0.76823E+00_JPRB ,&
&  0.76642E+00_JPRB, 0.76417E+00_JPRB, 0.76162E+00_JPRB, 0.75732E+00_JPRB, 0.75238E+00_JPRB, 0.74712E+00_JPRB, 0.74140E+00_JPRB ,&
&  0.73515E+00_JPRB, 0.73041E+00_JPRB, 0.72599E+00_JPRB, 0.72193E+00_JPRB, 0.72061E+00_JPRB, 0.71942E+00_JPRB, 0.71826E+00_JPRB ,&
&  0.71721E+00_JPRB, 0.71509E+00_JPRB, 0.71475E+00_JPRB, 0.71473E+00_JPRB, 0.71514E+00_JPRB, 0.71663E+00_JPRB, 0.71765E+00_JPRB ,&
&  0.71831E+00_JPRB, 0.71795E+00_JPRB, 0.71712E+00_JPRB, 0.71765E+00_JPRB, 0.71919E+00_JPRB, 0.71964E+00_JPRB, 0.71841E+00_JPRB ,&
&  0.71733E+00_JPRB, 0.71540E+00_JPRB, 0.71307E+00_JPRB, 0.71041E+00_JPRB, 0.70924E+00_JPRB, 0.70748E+00_JPRB, 0.70554E+00_JPRB ,&
&  0.70439E+00_JPRB, 0.70440E+00_JPRB, 0.70479E+00_JPRB, 0.70549E+00_JPRB, 0.70715E+00_JPRB, 0.70903E+00_JPRB, 0.71096E+00_JPRB ,&
&  0.71308E+00_JPRB, 0.71659E+00_JPRB, 0.72022E+00_JPRB, 0.72367E+00_JPRB, 0.72750E+00_JPRB, 0.73173E+00_JPRB, 0.73596E+00_JPRB ,&
&  0.73962E+00_JPRB, 0.74262E+00_JPRB, 0.74430E+00_JPRB, 0.74686E+00_JPRB, 0.74933E+00_JPRB, 0.75250E+00_JPRB, 0.75507E+00_JPRB ,&
&  0.75794E+00_JPRB, 0.76000E+00_JPRB, 0.76133E+00_JPRB, 0.76262E+00_JPRB, 0.76353E+00_JPRB, 0.76406E+00_JPRB, 0.76410E+00_JPRB ,&
&  0.76382E+00_JPRB, 0.76347E+00_JPRB, 0.76300E+00_JPRB, 0.76239E+00_JPRB, 0.76176E+00_JPRB, 0.76107E+00_JPRB, 0.76050E+00_JPRB ,&
&  0.76041E+00_JPRB, 0.76118E+00_JPRB, 0.76245E+00_JPRB, 0.76322E+00_JPRB, 0.76413E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.75774E+00_JPRB, 0.75919E+00_JPRB, 0.76021E+00_JPRB, 0.76212E+00_JPRB, 0.76578E+00_JPRB, 0.76940E+00_JPRB, 0.77102E+00_JPRB ,&
&  0.77366E+00_JPRB, 0.77689E+00_JPRB, 0.78100E+00_JPRB, 0.78301E+00_JPRB, 0.78448E+00_JPRB, 0.78462E+00_JPRB, 0.78375E+00_JPRB ,&
&  0.78177E+00_JPRB, 0.77929E+00_JPRB, 0.77744E+00_JPRB, 0.77577E+00_JPRB, 0.77384E+00_JPRB, 0.77178E+00_JPRB, 0.76643E+00_JPRB ,&
&  0.76154E+00_JPRB, 0.75856E+00_JPRB, 0.75140E+00_JPRB, 0.74229E+00_JPRB, 0.73510E+00_JPRB, 0.68346E+00_JPRB, 0.66670E+00_JPRB ,&
&  0.66463E+00_JPRB, 0.66029E+00_JPRB, 0.66186E+00_JPRB, 0.66465E+00_JPRB, 0.67349E+00_JPRB, 0.66612E+00_JPRB, 0.66123E+00_JPRB ,&
&  0.63028E+00_JPRB, 0.63069E+00_JPRB, 0.62600E+00_JPRB, 0.62396E+00_JPRB, 0.62803E+00_JPRB, 0.63128E+00_JPRB, 0.64261E+00_JPRB ,&
&  0.67254E+00_JPRB, 0.68169E+00_JPRB, 0.68537E+00_JPRB, 0.68867E+00_JPRB, 0.69450E+00_JPRB, 0.69876E+00_JPRB, 0.70174E+00_JPRB ,&
&  0.70323E+00_JPRB, 0.70359E+00_JPRB, 0.70434E+00_JPRB, 0.70451E+00_JPRB, 0.70456E+00_JPRB, 0.70326E+00_JPRB, 0.70272E+00_JPRB ,&
&  0.70251E+00_JPRB, 0.70412E+00_JPRB, 0.70645E+00_JPRB, 0.70900E+00_JPRB, 0.71228E+00_JPRB, 0.71452E+00_JPRB, 0.71672E+00_JPRB ,&
&  0.71904E+00_JPRB, 0.72106E+00_JPRB, 0.72291E+00_JPRB, 0.72505E+00_JPRB, 0.72776E+00_JPRB, 0.72957E+00_JPRB, 0.73191E+00_JPRB ,&
&  0.73410E+00_JPRB, 0.73694E+00_JPRB, 0.73944E+00_JPRB, 0.74497E+00_JPRB, 0.74992E+00_JPRB, 0.75627E+00_JPRB, 0.76199E+00_JPRB ,&
&  0.76770E+00_JPRB, 0.77157E+00_JPRB, 0.77453E+00_JPRB, 0.77656E+00_JPRB, 0.77843E+00_JPRB, 0.77976E+00_JPRB, 0.77995E+00_JPRB ,&
&  0.77938E+00_JPRB, 0.77734E+00_JPRB, 0.77486E+00_JPRB, 0.77164E+00_JPRB, 0.76861E+00_JPRB, 0.76467E+00_JPRB, 0.76110E+00_JPRB ,&
&  0.75713E+00_JPRB, 0.75540E+00_JPRB, 0.75483E+00_JPRB, 0.75578E+00_JPRB, 0.75674E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.76485E+00_JPRB, 0.76379E+00_JPRB, 0.76268E+00_JPRB, 0.76493E+00_JPRB, 0.76860E+00_JPRB, 0.77471E+00_JPRB, 0.77590E+00_JPRB ,&
&  0.78074E+00_JPRB, 0.78566E+00_JPRB, 0.79130E+00_JPRB, 0.79343E+00_JPRB, 0.79531E+00_JPRB, 0.79390E+00_JPRB, 0.79067E+00_JPRB ,&
&  0.78746E+00_JPRB, 0.78270E+00_JPRB, 0.77966E+00_JPRB, 0.77574E+00_JPRB, 0.77021E+00_JPRB, 0.76616E+00_JPRB, 0.75910E+00_JPRB ,&
&  0.75482E+00_JPRB, 0.74954E+00_JPRB, 0.70080E+00_JPRB, 0.68564E+00_JPRB, 0.66188E+00_JPRB, 0.64863E+00_JPRB, 0.64272E+00_JPRB ,&
&  0.63420E+00_JPRB, 0.62600E+00_JPRB, 0.61428E+00_JPRB, 0.59876E+00_JPRB, 0.58104E+00_JPRB, 0.55557E+00_JPRB, 0.54332E+00_JPRB ,&
&  0.53189E+00_JPRB, 0.52904E+00_JPRB, 0.53092E+00_JPRB, 0.53802E+00_JPRB, 0.55277E+00_JPRB, 0.56883E+00_JPRB, 0.58850E+00_JPRB ,&
&  0.61524E+00_JPRB, 0.64259E+00_JPRB, 0.65676E+00_JPRB, 0.65725E+00_JPRB, 0.67911E+00_JPRB, 0.69107E+00_JPRB, 0.69819E+00_JPRB ,&
&  0.70133E+00_JPRB, 0.70038E+00_JPRB, 0.70963E+00_JPRB, 0.71205E+00_JPRB, 0.71325E+00_JPRB, 0.71423E+00_JPRB, 0.71667E+00_JPRB ,&
&  0.72110E+00_JPRB, 0.72595E+00_JPRB, 0.72944E+00_JPRB, 0.73269E+00_JPRB, 0.73472E+00_JPRB, 0.73549E+00_JPRB, 0.73621E+00_JPRB ,&
&  0.73727E+00_JPRB, 0.73804E+00_JPRB, 0.73869E+00_JPRB, 0.73880E+00_JPRB, 0.73983E+00_JPRB, 0.74033E+00_JPRB, 0.74024E+00_JPRB ,&
&  0.74198E+00_JPRB, 0.74486E+00_JPRB, 0.74831E+00_JPRB, 0.74756E+00_JPRB, 0.75269E+00_JPRB, 0.76535E+00_JPRB, 0.77275E+00_JPRB ,&
&  0.78246E+00_JPRB, 0.78946E+00_JPRB, 0.79683E+00_JPRB, 0.80273E+00_JPRB, 0.80747E+00_JPRB, 0.80919E+00_JPRB, 0.80824E+00_JPRB ,&
&  0.80669E+00_JPRB, 0.80360E+00_JPRB, 0.80128E+00_JPRB, 0.79672E+00_JPRB, 0.79264E+00_JPRB, 0.78657E+00_JPRB, 0.78140E+00_JPRB ,&
&  0.77453E+00_JPRB, 0.77069E+00_JPRB, 0.76441E+00_JPRB, 0.76413E+00_JPRB, 0.76374E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.77834E+00_JPRB, 0.77492E+00_JPRB, 0.77461E+00_JPRB, 0.77941E+00_JPRB, 0.79018E+00_JPRB, 0.80283E+00_JPRB, 0.80351E+00_JPRB ,&
&  0.80443E+00_JPRB, 0.80429E+00_JPRB, 0.80731E+00_JPRB, 0.80650E+00_JPRB, 0.80524E+00_JPRB, 0.80078E+00_JPRB, 0.79702E+00_JPRB ,&
&  0.79342E+00_JPRB, 0.78515E+00_JPRB, 0.77476E+00_JPRB, 0.76853E+00_JPRB, 0.76569E+00_JPRB, 0.76588E+00_JPRB, 0.76177E+00_JPRB ,&
&  0.75479E+00_JPRB, 0.73833E+00_JPRB, 0.70290E+00_JPRB, 0.68078E+00_JPRB, 0.65634E+00_JPRB, 0.64884E+00_JPRB, 0.63563E+00_JPRB ,&
&  0.64305E+00_JPRB, 0.58758E+00_JPRB, 0.58655E+00_JPRB, 0.56183E+00_JPRB, 0.54377E+00_JPRB, 0.52377E+00_JPRB, 0.50403E+00_JPRB ,&
&  0.49767E+00_JPRB, 0.49023E+00_JPRB, 0.48869E+00_JPRB, 0.49715E+00_JPRB, 0.50942E+00_JPRB, 0.53259E+00_JPRB, 0.55805E+00_JPRB ,&
&  0.57622E+00_JPRB, 0.63971E+00_JPRB, 0.65689E+00_JPRB, 0.66561E+00_JPRB, 0.68555E+00_JPRB, 0.69911E+00_JPRB, 0.70207E+00_JPRB ,&
&  0.70480E+00_JPRB, 0.70686E+00_JPRB, 0.71883E+00_JPRB, 0.72699E+00_JPRB, 0.73896E+00_JPRB, 0.74307E+00_JPRB, 0.74669E+00_JPRB ,&
&  0.75333E+00_JPRB, 0.76132E+00_JPRB, 0.76855E+00_JPRB, 0.77582E+00_JPRB, 0.78115E+00_JPRB, 0.78465E+00_JPRB, 0.78607E+00_JPRB ,&
&  0.78626E+00_JPRB, 0.78503E+00_JPRB, 0.78432E+00_JPRB, 0.78381E+00_JPRB, 0.78268E+00_JPRB, 0.78425E+00_JPRB, 0.78468E+00_JPRB ,&
&  0.78575E+00_JPRB, 0.78449E+00_JPRB, 0.78185E+00_JPRB, 0.78124E+00_JPRB, 0.78584E+00_JPRB, 0.79000E+00_JPRB, 0.79962E+00_JPRB ,&
&  0.81298E+00_JPRB, 0.82054E+00_JPRB, 0.82766E+00_JPRB, 0.83415E+00_JPRB, 0.83977E+00_JPRB, 0.83999E+00_JPRB, 0.83758E+00_JPRB ,&
&  0.83478E+00_JPRB, 0.83138E+00_JPRB, 0.82810E+00_JPRB, 0.82229E+00_JPRB, 0.81682E+00_JPRB, 0.80835E+00_JPRB, 0.80076E+00_JPRB ,&
&  0.79162E+00_JPRB, 0.78719E+00_JPRB, 0.77901E+00_JPRB, 0.77796E+00_JPRB, 0.77753E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.80323E+00_JPRB, 0.80027E+00_JPRB, 0.80206E+00_JPRB, 0.80713E+00_JPRB, 0.81426E+00_JPRB, 0.82402E+00_JPRB, 0.82820E+00_JPRB ,&
&  0.83184E+00_JPRB, 0.83277E+00_JPRB, 0.83100E+00_JPRB, 0.81902E+00_JPRB, 0.80978E+00_JPRB, 0.80419E+00_JPRB, 0.79864E+00_JPRB ,&
&  0.78977E+00_JPRB, 0.78214E+00_JPRB, 0.77693E+00_JPRB, 0.77416E+00_JPRB, 0.77088E+00_JPRB, 0.77369E+00_JPRB, 0.76597E+00_JPRB ,&
&  0.74384E+00_JPRB, 0.72416E+00_JPRB, 0.69315E+00_JPRB, 0.66900E+00_JPRB, 0.65232E+00_JPRB, 0.63825E+00_JPRB, 0.63881E+00_JPRB ,&
&  0.63382E+00_JPRB, 0.62970E+00_JPRB, 0.62574E+00_JPRB, 0.62306E+00_JPRB, 0.62470E+00_JPRB, 0.53019E+00_JPRB, 0.50532E+00_JPRB ,&
&  0.48812E+00_JPRB, 0.47477E+00_JPRB, 0.47078E+00_JPRB, 0.47040E+00_JPRB, 0.47851E+00_JPRB, 0.50599E+00_JPRB, 0.54564E+00_JPRB ,&
&  0.57029E+00_JPRB, 0.65303E+00_JPRB, 0.66793E+00_JPRB, 0.68217E+00_JPRB, 0.70662E+00_JPRB, 0.72253E+00_JPRB, 0.72744E+00_JPRB ,&
&  0.73951E+00_JPRB, 0.76249E+00_JPRB, 0.79153E+00_JPRB, 0.82166E+00_JPRB, 0.83092E+00_JPRB, 0.84775E+00_JPRB, 0.85600E+00_JPRB ,&
&  0.85947E+00_JPRB, 0.85357E+00_JPRB, 0.85171E+00_JPRB, 0.85196E+00_JPRB, 0.84800E+00_JPRB, 0.84888E+00_JPRB, 0.84248E+00_JPRB ,&
&  0.84380E+00_JPRB, 0.83836E+00_JPRB, 0.84741E+00_JPRB, 0.85219E+00_JPRB, 0.85838E+00_JPRB, 0.86446E+00_JPRB, 0.86537E+00_JPRB ,&
&  0.86334E+00_JPRB, 0.86052E+00_JPRB, 0.85052E+00_JPRB, 0.83467E+00_JPRB, 0.82751E+00_JPRB, 0.81907E+00_JPRB, 0.82124E+00_JPRB ,&
&  0.82708E+00_JPRB, 0.84099E+00_JPRB, 0.84598E+00_JPRB, 0.85092E+00_JPRB, 0.85567E+00_JPRB, 0.85422E+00_JPRB, 0.85590E+00_JPRB ,&
&  0.85583E+00_JPRB, 0.85637E+00_JPRB, 0.85340E+00_JPRB, 0.84733E+00_JPRB, 0.83879E+00_JPRB, 0.82662E+00_JPRB, 0.81384E+00_JPRB ,&
&  0.80618E+00_JPRB, 0.80443E+00_JPRB, 0.80011E+00_JPRB, 0.80055E+00_JPRB, 0.80188E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.82771E+00_JPRB, 0.82251E+00_JPRB, 0.82632E+00_JPRB, 0.83176E+00_JPRB, 0.84110E+00_JPRB, 0.85070E+00_JPRB, 0.85340E+00_JPRB ,&
&  0.86376E+00_JPRB, 0.86231E+00_JPRB, 0.84831E+00_JPRB, 0.83154E+00_JPRB, 0.81176E+00_JPRB, 0.79657E+00_JPRB, 0.78917E+00_JPRB ,&
&  0.78739E+00_JPRB, 0.77625E+00_JPRB, 0.76936E+00_JPRB, 0.76472E+00_JPRB, 0.76327E+00_JPRB, 0.76567E+00_JPRB, 0.75401E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.71256E+00_JPRB, 0.69457E+00_JPRB, 0.67021E+00_JPRB, 0.64711E+00_JPRB, 0.63241E+00_JPRB, 0.63489E+00_JPRB ,&
&  0.63948E+00_JPRB, 0.63117E+00_JPRB, 0.63188E+00_JPRB, 0.62856E+00_JPRB, 0.61738E+00_JPRB, 0.60874E+00_JPRB, 0.53138E+00_JPRB ,&
&  0.50229E+00_JPRB, 0.47576E+00_JPRB, 0.46546E+00_JPRB, 0.46375E+00_JPRB, 0.48076E+00_JPRB, 0.51155E+00_JPRB, 0.56687E+00_JPRB ,&
&  0.61523E+00_JPRB, 0.66025E+00_JPRB, 0.69171E+00_JPRB, 0.71501E+00_JPRB, 0.74008E+00_JPRB, 0.76525E+00_JPRB, 0.78473E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.86784E+00_JPRB, 0.91938E+00_JPRB, 0.96899E+00_JPRB, 0.10036E+01_JPRB, 0.10201E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.94610E+00_JPRB, 0.91314E+00_JPRB, 0.89295E+00_JPRB, 0.87885E+00_JPRB, 0.87984E+00_JPRB, 0.86487E+00_JPRB ,&
&  0.86216E+00_JPRB, 0.87005E+00_JPRB, 0.87400E+00_JPRB, 0.87874E+00_JPRB, 0.88569E+00_JPRB, 0.88829E+00_JPRB, 0.89377E+00_JPRB ,&
&  0.90556E+00_JPRB, 0.92497E+00_JPRB, 0.96426E+00_JPRB, 0.93387E+00_JPRB, 0.89481E+00_JPRB, 0.85562E+00_JPRB, 0.83353E+00_JPRB ,&
&  0.82932E+00_JPRB, 0.83383E+00_JPRB, 0.84309E+00_JPRB, 0.84460E+00_JPRB, 0.84768E+00_JPRB, 0.85083E+00_JPRB, 0.86636E+00_JPRB ,&
&  0.87128E+00_JPRB, 0.87558E+00_JPRB, 0.87511E+00_JPRB, 0.87021E+00_JPRB, 0.85927E+00_JPRB, 0.84421E+00_JPRB, 0.83043E+00_JPRB ,&
&  0.82559E+00_JPRB, 0.82687E+00_JPRB, 0.82681E+00_JPRB, 0.82932E+00_JPRB, 0.83019E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.83782E+00_JPRB, 0.84107E+00_JPRB, 0.85428E+00_JPRB, 0.86886E+00_JPRB, 0.88083E+00_JPRB, 0.86725E+00_JPRB, 0.85084E+00_JPRB ,&
&  0.84686E+00_JPRB, 0.84833E+00_JPRB, 0.83454E+00_JPRB, 0.82123E+00_JPRB, 0.80004E+00_JPRB, 0.78233E+00_JPRB, 0.77608E+00_JPRB ,&
&  0.77329E+00_JPRB, 0.76471E+00_JPRB, 0.75297E+00_JPRB, 0.73805E+00_JPRB, 0.73549E+00_JPRB, 0.73562E+00_JPRB, 0.72515E+00_JPRB ,&
&  0.71488E+00_JPRB, 0.70836E+00_JPRB, 0.69789E+00_JPRB, 0.68049E+00_JPRB, 0.66009E+00_JPRB, 0.64977E+00_JPRB, 0.64345E+00_JPRB ,&
&  0.63859E+00_JPRB, 0.63937E+00_JPRB, 0.64916E+00_JPRB, 0.64557E+00_JPRB, 0.64393E+00_JPRB, 0.63750E+00_JPRB, 0.58410E+00_JPRB ,&
&  0.52657E+00_JPRB, 0.48737E+00_JPRB, 0.46502E+00_JPRB, 0.46339E+00_JPRB, 0.48646E+00_JPRB, 0.53588E+00_JPRB, 0.59424E+00_JPRB ,&
&  0.65177E+00_JPRB, 0.70297E+00_JPRB, 0.74370E+00_JPRB, 0.77158E+00_JPRB, 0.79758E+00_JPRB, 0.83214E+00_JPRB, 0.87584E+00_JPRB ,&
&  0.93756E+00_JPRB, 0.10148E+01_JPRB, 0.10863E+01_JPRB, 0.11554E+01_JPRB, 0.12041E+01_JPRB, 0.11602E+01_JPRB, 0.12439E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.12780E+01_JPRB, 0.10525E+01_JPRB, 0.96661E+00_JPRB, 0.94831E+00_JPRB, 0.95377E+00_JPRB, 0.96036E+00_JPRB ,&
&  0.97337E+00_JPRB, 0.97826E+00_JPRB, 0.97196E+00_JPRB, 0.96484E+00_JPRB, 0.96723E+00_JPRB, 0.97332E+00_JPRB, 0.99458E+00_JPRB ,&
&  0.10371E+01_JPRB, 0.12441E+01_JPRB, 0.15434E+01_JPRB, 0.11685E+01_JPRB, 0.10386E+01_JPRB, 0.94214E+00_JPRB, 0.84867E+00_JPRB ,&
&  0.81051E+00_JPRB, 0.79991E+00_JPRB, 0.80610E+00_JPRB, 0.81329E+00_JPRB, 0.81340E+00_JPRB, 0.81857E+00_JPRB, 0.82891E+00_JPRB ,&
&  0.84612E+00_JPRB, 0.86293E+00_JPRB, 0.87339E+00_JPRB, 0.87872E+00_JPRB, 0.87334E+00_JPRB, 0.85635E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83150E+00_JPRB, 0.84068E+00_JPRB, 0.84717E+00_JPRB, 0.85022E+00_JPRB, 0.84527E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.83276E+00_JPRB, 0.86608E+00_JPRB, 0.90945E+00_JPRB, 0.91622E+00_JPRB, 0.90692E+00_JPRB, 0.85637E+00_JPRB, 0.80553E+00_JPRB ,&
&  0.77704E+00_JPRB, 0.76843E+00_JPRB, 0.76480E+00_JPRB, 0.75335E+00_JPRB, 0.74105E+00_JPRB, 0.73277E+00_JPRB, 0.72560E+00_JPRB ,&
&  0.71929E+00_JPRB, 0.71418E+00_JPRB, 0.70845E+00_JPRB, 0.70102E+00_JPRB, 0.70403E+00_JPRB, 0.70720E+00_JPRB, 0.70528E+00_JPRB ,&
&  0.70297E+00_JPRB, 0.70328E+00_JPRB, 0.69156E+00_JPRB, 0.67151E+00_JPRB, 0.66268E+00_JPRB, 0.66422E+00_JPRB, 0.67037E+00_JPRB ,&
&  0.66054E+00_JPRB, 0.64670E+00_JPRB, 0.65226E+00_JPRB, 0.66560E+00_JPRB, 0.65893E+00_JPRB, 0.65399E+00_JPRB, 0.60334E+00_JPRB ,&
&  0.54034E+00_JPRB, 0.49869E+00_JPRB, 0.48427E+00_JPRB, 0.51427E+00_JPRB, 0.57403E+00_JPRB, 0.65722E+00_JPRB, 0.70266E+00_JPRB ,&
&  0.73335E+00_JPRB, 0.77722E+00_JPRB, 0.82187E+00_JPRB, 0.87508E+00_JPRB, 0.94570E+00_JPRB, 0.10303E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.12526E+01_JPRB, 0.12647E+01_JPRB, 0.11326E+01_JPRB, 0.10994E+01_JPRB, 0.11045E+01_JPRB, 0.12198E+01_JPRB ,&
&  0.13470E+01_JPRB, 0.12506E+01_JPRB, 0.11729E+01_JPRB, 0.11779E+01_JPRB, 0.11670E+01_JPRB, 0.11877E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.11408E+01_JPRB, 0.10902E+01_JPRB, 0.10490E+01_JPRB, 0.10409E+01_JPRB, 0.10617E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.12757E+01_JPRB, 0.12167E+01_JPRB, 0.93428E+00_JPRB, 0.88927E+00_JPRB, 0.87197E+00_JPRB, 0.84968E+00_JPRB ,&
&  0.81015E+00_JPRB, 0.78620E+00_JPRB, 0.78237E+00_JPRB, 0.78532E+00_JPRB, 0.77542E+00_JPRB, 0.76843E+00_JPRB, 0.76886E+00_JPRB ,&
&  0.77591E+00_JPRB, 0.79099E+00_JPRB, 0.81090E+00_JPRB, 0.84504E+00_JPRB, 0.85758E+00_JPRB, 0.85216E+00_JPRB, 0.83028E+00_JPRB ,&
&  0.81831E+00_JPRB, 0.82073E+00_JPRB, 0.82269E+00_JPRB, 0.82118E+00_JPRB, 0.82304E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.95546E+00_JPRB, 0.96199E+00_JPRB, 0.95786E+00_JPRB, 0.95495E+00_JPRB, 0.90435E+00_JPRB, 0.87512E+00_JPRB, 0.83015E+00_JPRB ,&
&  0.81739E+00_JPRB, 0.79349E+00_JPRB, 0.75133E+00_JPRB, 0.70215E+00_JPRB, 0.67388E+00_JPRB, 0.66019E+00_JPRB, 0.66045E+00_JPRB ,&
&  0.67008E+00_JPRB, 0.68036E+00_JPRB, 0.68475E+00_JPRB, 0.68062E+00_JPRB, 0.68700E+00_JPRB, 0.69063E+00_JPRB, 0.68768E+00_JPRB ,&
&  0.68990E+00_JPRB, 0.69800E+00_JPRB, 0.69950E+00_JPRB, 0.69442E+00_JPRB, 0.69304E+00_JPRB, 0.69536E+00_JPRB, 0.69144E+00_JPRB ,&
&  0.67459E+00_JPRB, 0.65828E+00_JPRB, 0.66171E+00_JPRB, 0.68736E+00_JPRB, 0.66833E+00_JPRB, 0.66430E+00_JPRB, 0.60275E+00_JPRB ,&
&  0.54220E+00_JPRB, 0.52784E+00_JPRB, 0.56198E+00_JPRB, 0.65136E+00_JPRB, 0.68396E+00_JPRB, 0.71999E+00_JPRB, 0.74762E+00_JPRB ,&
&  0.74937E+00_JPRB, 0.77076E+00_JPRB, 0.84284E+00_JPRB, 0.98545E+00_JPRB, 0.10656E+01_JPRB, 0.11751E+01_JPRB, 0.12859E+01_JPRB ,&
&  0.13746E+01_JPRB, 0.14432E+01_JPRB, 0.14944E+01_JPRB, 0.14074E+01_JPRB, 0.14318E+01_JPRB, 0.15728E+01_JPRB, 0.14071E+01_JPRB ,&
&  0.13760E+01_JPRB, 0.13243E+01_JPRB, 0.13528E+01_JPRB, 0.14485E+01_JPRB, 0.13917E+01_JPRB, 0.14349E+01_JPRB, 0.14123E+01_JPRB ,&
&  0.12995E+01_JPRB, 0.12274E+01_JPRB, 0.11701E+01_JPRB, 0.11445E+01_JPRB, 0.11266E+01_JPRB, 0.11272E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.12080E+01_JPRB, 0.12232E+01_JPRB, 0.11143E+01_JPRB, 0.93119E+00_JPRB, 0.86084E+00_JPRB, 0.83190E+00_JPRB, 0.81379E+00_JPRB ,&
&  0.79229E+00_JPRB, 0.77879E+00_JPRB, 0.78157E+00_JPRB, 0.78544E+00_JPRB, 0.78011E+00_JPRB, 0.76428E+00_JPRB, 0.74874E+00_JPRB ,&
&  0.73994E+00_JPRB, 0.74273E+00_JPRB, 0.74211E+00_JPRB, 0.77450E+00_JPRB, 0.80858E+00_JPRB, 0.83342E+00_JPRB, 0.82779E+00_JPRB ,&
&  0.82190E+00_JPRB, 0.83548E+00_JPRB, 0.85731E+00_JPRB, 0.90074E+00_JPRB, 0.92626E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.99362E+00_JPRB, 0.10055E+01_JPRB, 0.10087E+01_JPRB, 0.99309E+00_JPRB, 0.94957E+00_JPRB, 0.89767E+00_JPRB, 0.79984E+00_JPRB ,&
&  0.79064E+00_JPRB, 0.74688E+00_JPRB, 0.70051E+00_JPRB, 0.66845E+00_JPRB, 0.65789E+00_JPRB, 0.64168E+00_JPRB, 0.63460E+00_JPRB ,&
&  0.63584E+00_JPRB, 0.65729E+00_JPRB, 0.68066E+00_JPRB, 0.68157E+00_JPRB, 0.68855E+00_JPRB, 0.69187E+00_JPRB, 0.68502E+00_JPRB ,&
&  0.69352E+00_JPRB, 0.71270E+00_JPRB, 0.73389E+00_JPRB, 0.75038E+00_JPRB, 0.73659E+00_JPRB, 0.72655E+00_JPRB, 0.71793E+00_JPRB ,&
&  0.71114E+00_JPRB, 0.70562E+00_JPRB, 0.70292E+00_JPRB, 0.71403E+00_JPRB, 0.69397E+00_JPRB, 0.67236E+00_JPRB, 0.65855E+00_JPRB ,&
&  0.57465E+00_JPRB, 0.58402E+00_JPRB, 0.66123E+00_JPRB, 0.69420E+00_JPRB, 0.72723E+00_JPRB, 0.76301E+00_JPRB, 0.81941E+00_JPRB ,&
&  0.87047E+00_JPRB, 0.92531E+00_JPRB, 0.99890E+00_JPRB, 0.11002E+01_JPRB, 0.12382E+01_JPRB, 0.14004E+01_JPRB, 0.15528E+01_JPRB ,&
&  0.17059E+01_JPRB, 0.14113E+01_JPRB, 0.13770E+01_JPRB, 0.16012E+01_JPRB, 0.19951E+01_JPRB, 0.17961E+01_JPRB, 0.16657E+01_JPRB ,&
&  0.15589E+01_JPRB, 0.14699E+01_JPRB, 0.14431E+01_JPRB, 0.15514E+01_JPRB, 0.15900E+01_JPRB, 0.16820E+01_JPRB, 0.17405E+01_JPRB ,&
&  0.15782E+01_JPRB, 0.14625E+01_JPRB, 0.13469E+01_JPRB, 0.12947E+01_JPRB, 0.12653E+01_JPRB, 0.12478E+01_JPRB, 0.12317E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.11771E+01_JPRB, 0.10962E+01_JPRB, 0.99248E+00_JPRB, 0.91714E+00_JPRB, 0.87358E+00_JPRB, 0.84539E+00_JPRB ,&
&  0.82342E+00_JPRB, 0.80181E+00_JPRB, 0.78669E+00_JPRB, 0.78163E+00_JPRB, 0.78543E+00_JPRB, 0.80205E+00_JPRB, 0.84391E+00_JPRB ,&
&  0.81844E+00_JPRB, 0.80232E+00_JPRB, 0.77505E+00_JPRB, 0.79297E+00_JPRB, 0.83581E+00_JPRB, 0.87497E+00_JPRB, 0.88376E+00_JPRB ,&
&  0.90744E+00_JPRB, 0.91299E+00_JPRB, 0.90223E+00_JPRB, 0.92976E+00_JPRB, 0.95211E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.10266E+01_JPRB, 0.10452E+01_JPRB, 0.10468E+01_JPRB, 0.10464E+01_JPRB, 0.99539E+00_JPRB, 0.91221E+00_JPRB, 0.83455E+00_JPRB ,&
&  0.89244E+00_JPRB, 0.82240E+00_JPRB, 0.81193E+00_JPRB, 0.69828E+00_JPRB, 0.66082E+00_JPRB, 0.64645E+00_JPRB, 0.62756E+00_JPRB ,&
&  0.60911E+00_JPRB, 0.63826E+00_JPRB, 0.67464E+00_JPRB, 0.68857E+00_JPRB, 0.70353E+00_JPRB, 0.71147E+00_JPRB, 0.70246E+00_JPRB ,&
&  0.71679E+00_JPRB, 0.74713E+00_JPRB, 0.80081E+00_JPRB, 0.81027E+00_JPRB, 0.78245E+00_JPRB, 0.78007E+00_JPRB, 0.78353E+00_JPRB ,&
&  0.75692E+00_JPRB, 0.74635E+00_JPRB, 0.76835E+00_JPRB, 0.77147E+00_JPRB, 0.75730E+00_JPRB, 0.70978E+00_JPRB, 0.68098E+00_JPRB ,&
&  0.68757E+00_JPRB, 0.67430E+00_JPRB, 0.72825E+00_JPRB, 0.75729E+00_JPRB, 0.79287E+00_JPRB, 0.83503E+00_JPRB, 0.88121E+00_JPRB ,&
&  0.94714E+00_JPRB, 0.10390E+01_JPRB, 0.11624E+01_JPRB, 0.13342E+01_JPRB, 0.15233E+01_JPRB, 0.17508E+01_JPRB, 0.19218E+01_JPRB ,&
&  0.21405E+01_JPRB, 0.16434E+01_JPRB, 0.20075E+01_JPRB, 0.21525E+01_JPRB, 0.24280E+01_JPRB, 0.24132E+01_JPRB, 0.23190E+01_JPRB ,&
&  0.22807E+01_JPRB, 0.19927E+01_JPRB, 0.18133E+01_JPRB, 0.17040E+01_JPRB, 0.17239E+01_JPRB, 0.18661E+01_JPRB, 0.19780E+01_JPRB ,&
&  0.20866E+01_JPRB, 0.19847E+01_JPRB, 0.18193E+01_JPRB, 0.16845E+01_JPRB, 0.15827E+01_JPRB, 0.15125E+01_JPRB, 0.14327E+01_JPRB ,&
&  0.13558E+01_JPRB, 0.12729E+01_JPRB, 0.11900E+01_JPRB, 0.10766E+01_JPRB, 0.10163E+01_JPRB, 0.96262E+00_JPRB, 0.90676E+00_JPRB ,&
&  0.87922E+00_JPRB, 0.83299E+00_JPRB, 0.78565E+00_JPRB, 0.76367E+00_JPRB, 0.74629E+00_JPRB, 0.75075E+00_JPRB, 0.78561E+00_JPRB ,&
&  0.80545E+00_JPRB, 0.79965E+00_JPRB, 0.85817E+00_JPRB, 0.90747E+00_JPRB, 0.93540E+00_JPRB, 0.96103E+00_JPRB, 0.10108E+01_JPRB ,&
&  0.94574E+00_JPRB, 0.95017E+00_JPRB, 0.10023E+01_JPRB, 0.10094E+01_JPRB, 0.10156E+01_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.10890E+01_JPRB, 0.11159E+01_JPRB, 0.11048E+01_JPRB, 0.11011E+01_JPRB, 0.10198E+01_JPRB, 0.98093E+00_JPRB, 0.89190E+00_JPRB ,&
&  0.91381E+00_JPRB, 0.88011E+00_JPRB, 0.80049E+00_JPRB, 0.77554E+00_JPRB, 0.78208E+00_JPRB, 0.69959E+00_JPRB, 0.63136E+00_JPRB ,&
&  0.60356E+00_JPRB, 0.62184E+00_JPRB, 0.69264E+00_JPRB, 0.72784E+00_JPRB, 0.73516E+00_JPRB, 0.74416E+00_JPRB, 0.73625E+00_JPRB ,&
&  0.80457E+00_JPRB, 0.82206E+00_JPRB, 0.87518E+00_JPRB, 0.90263E+00_JPRB, 0.88812E+00_JPRB, 0.89277E+00_JPRB, 0.90723E+00_JPRB ,&
&  0.85307E+00_JPRB, 0.83423E+00_JPRB, 0.82689E+00_JPRB, 0.77858E+00_JPRB, 0.83954E+00_JPRB, 0.80489E+00_JPRB, 0.75765E+00_JPRB ,&
&  0.74899E+00_JPRB, 0.77549E+00_JPRB, 0.81212E+00_JPRB, 0.84928E+00_JPRB, 0.87141E+00_JPRB, 0.90500E+00_JPRB, 0.96167E+00_JPRB ,&
&  0.10437E+01_JPRB, 0.11711E+01_JPRB, 0.13141E+01_JPRB, 0.15423E+01_JPRB, 0.17643E+01_JPRB, 0.18216E+01_JPRB, 0.21526E+01_JPRB ,&
&  0.23175E+01_JPRB, 0.25460E+01_JPRB, 0.28201E+01_JPRB, 0.26269E+01_JPRB, 0.29033E+01_JPRB, 0.28656E+01_JPRB, 0.26217E+01_JPRB ,&
&  0.26279E+01_JPRB, 0.23849E+01_JPRB, 0.23003E+01_JPRB, 0.23029E+01_JPRB, 0.24312E+01_JPRB, 0.23736E+01_JPRB, 0.23439E+01_JPRB ,&
&  0.24102E+01_JPRB, 0.26340E+01_JPRB, 0.24474E+01_JPRB, 0.23733E+01_JPRB, 0.21935E+01_JPRB, 0.20453E+01_JPRB, 0.18666E+01_JPRB ,&
&  0.17465E+01_JPRB, 0.16091E+01_JPRB, 0.14794E+01_JPRB, 0.13748E+01_JPRB, 0.12320E+01_JPRB, 0.11166E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.92580E+00_JPRB, 0.81710E+00_JPRB, 0.77969E+00_JPRB, 0.75589E+00_JPRB, 0.74909E+00_JPRB, 0.71948E+00_JPRB, 0.73471E+00_JPRB ,&
&  0.78434E+00_JPRB, 0.85929E+00_JPRB, 0.97084E+00_JPRB, 0.10364E+01_JPRB, 0.11212E+01_JPRB, 0.12016E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.10031E+01_JPRB, 0.10786E+01_JPRB, 0.10914E+01_JPRB, 0.10889E+01_JPRB, 0.10858E+01_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.12383E+01_JPRB, 0.12474E+01_JPRB, 0.12187E+01_JPRB, 0.11735E+01_JPRB, 0.10628E+01_JPRB, 0.10060E+01_JPRB, 0.93509E+00_JPRB ,&
&  0.93795E+00_JPRB, 0.89731E+00_JPRB, 0.82235E+00_JPRB, 0.79322E+00_JPRB, 0.79365E+00_JPRB, 0.77818E+00_JPRB, 0.71026E+00_JPRB ,&
&  0.66992E+00_JPRB, 0.72178E+00_JPRB, 0.72629E+00_JPRB, 0.82058E+00_JPRB, 0.89511E+00_JPRB, 0.85239E+00_JPRB, 0.87616E+00_JPRB ,&
&  0.95614E+00_JPRB, 0.96984E+00_JPRB, 0.97673E+00_JPRB, 0.97854E+00_JPRB, 0.98135E+00_JPRB, 0.98730E+00_JPRB, 0.97243E+00_JPRB ,&
&  0.90627E+00_JPRB, 0.86320E+00_JPRB, 0.82779E+00_JPRB, 0.82293E+00_JPRB, 0.89110E+00_JPRB, 0.92050E+00_JPRB, 0.88602E+00_JPRB ,&
&  0.84927E+00_JPRB, 0.83508E+00_JPRB, 0.84492E+00_JPRB, 0.90391E+00_JPRB, 0.94348E+00_JPRB, 0.96617E+00_JPRB, 0.10281E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.12479E+01_JPRB, 0.13718E+01_JPRB, 0.15318E+01_JPRB, 0.16134E+01_JPRB, 0.19065E+01_JPRB, 0.23676E+01_JPRB ,&
&  0.25523E+01_JPRB, 0.27769E+01_JPRB, 0.30423E+01_JPRB, 0.33368E+01_JPRB, 0.35511E+01_JPRB, 0.33264E+01_JPRB, 0.30965E+01_JPRB ,&
&  0.30212E+01_JPRB, 0.28938E+01_JPRB, 0.34554E+01_JPRB, 0.32639E+01_JPRB, 0.30405E+01_JPRB, 0.29677E+01_JPRB, 0.28469E+01_JPRB ,&
&  0.28214E+01_JPRB, 0.28214E+01_JPRB, 0.27786E+01_JPRB, 0.27226E+01_JPRB, 0.26514E+01_JPRB, 0.26259E+01_JPRB, 0.24589E+01_JPRB ,&
&  0.23541E+01_JPRB, 0.21590E+01_JPRB, 0.17635E+01_JPRB, 0.14521E+01_JPRB, 0.11850E+01_JPRB, 0.11900E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.95547E+00_JPRB, 0.81659E+00_JPRB, 0.79143E+00_JPRB, 0.76489E+00_JPRB, 0.76879E+00_JPRB, 0.75748E+00_JPRB, 0.74798E+00_JPRB ,&
&  0.79821E+00_JPRB, 0.92743E+00_JPRB, 0.10126E+01_JPRB, 0.11531E+01_JPRB, 0.13074E+01_JPRB, 0.14247E+01_JPRB, 0.12103E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.13072E+01_JPRB, 0.13262E+01_JPRB, 0.13125E+01_JPRB, 0.12728E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.15149E+01_JPRB, 0.14527E+01_JPRB, 0.13730E+01_JPRB, 0.12607E+01_JPRB, 0.11117E+01_JPRB, 0.10183E+01_JPRB, 0.94398E+00_JPRB ,&
&  0.93580E+00_JPRB, 0.89931E+00_JPRB, 0.83765E+00_JPRB, 0.80931E+00_JPRB, 0.80165E+00_JPRB, 0.79545E+00_JPRB, 0.80466E+00_JPRB ,&
&  0.73512E+00_JPRB, 0.86915E+00_JPRB, 0.83514E+00_JPRB, 0.96511E+00_JPRB, 0.12533E+01_JPRB, 0.10517E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.11090E+01_JPRB, 0.10832E+01_JPRB, 0.10680E+01_JPRB, 0.11092E+01_JPRB, 0.11305E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.94315E+00_JPRB, 0.90998E+00_JPRB, 0.90523E+00_JPRB, 0.97154E+00_JPRB, 0.10226E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.99846E+00_JPRB, 0.96579E+00_JPRB, 0.93796E+00_JPRB, 0.95893E+00_JPRB, 0.99907E+00_JPRB, 0.10375E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.13005E+01_JPRB, 0.14070E+01_JPRB, 0.15455E+01_JPRB, 0.15856E+01_JPRB, 0.19507E+01_JPRB, 0.25432E+01_JPRB ,&
&  0.30192E+01_JPRB, 0.30260E+01_JPRB, 0.32991E+01_JPRB, 0.39845E+01_JPRB, 0.42619E+01_JPRB, 0.41136E+01_JPRB, 0.38384E+01_JPRB ,&
&  0.35595E+01_JPRB, 0.33410E+01_JPRB, 0.34467E+01_JPRB, 0.34308E+01_JPRB, 0.32057E+01_JPRB, 0.32847E+01_JPRB, 0.32059E+01_JPRB ,&
&  0.30431E+01_JPRB, 0.29686E+01_JPRB, 0.30002E+01_JPRB, 0.30121E+01_JPRB, 0.28493E+01_JPRB, 0.28823E+01_JPRB, 0.27686E+01_JPRB ,&
&  0.26741E+01_JPRB, 0.22903E+01_JPRB, 0.16592E+01_JPRB, 0.13517E+01_JPRB, 0.10281E+01_JPRB, 0.10188E+01_JPRB, 0.12188E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.10945E+01_JPRB, 0.98039E+00_JPRB, 0.90969E+00_JPRB, 0.86980E+00_JPRB, 0.90988E+00_JPRB, 0.94194E+00_JPRB ,&
&  0.96691E+00_JPRB, 0.11198E+01_JPRB, 0.13456E+01_JPRB, 0.15827E+01_JPRB, 0.17258E+01_JPRB, 0.17709E+01_JPRB, 0.16145E+01_JPRB ,&
&  0.17048E+01_JPRB, 0.16709E+01_JPRB, 0.16629E+01_JPRB, 0.16575E+01_JPRB, 0.16057E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.17935E+01_JPRB, 0.16867E+01_JPRB, 0.15301E+01_JPRB, 0.13453E+01_JPRB, 0.11393E+01_JPRB, 0.10107E+01_JPRB, 0.95879E+00_JPRB ,&
&  0.94768E+00_JPRB, 0.91529E+00_JPRB, 0.86006E+00_JPRB, 0.82770E+00_JPRB, 0.81565E+00_JPRB, 0.81510E+00_JPRB, 0.85708E+00_JPRB ,&
&  0.97785E+00_JPRB, 0.12032E+01_JPRB, 0.10574E+01_JPRB, 0.92842E+00_JPRB, 0.10551E+01_JPRB, 0.10337E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.12123E+01_JPRB, 0.11474E+01_JPRB, 0.11615E+01_JPRB, 0.12993E+01_JPRB, 0.13630E+01_JPRB, 0.13855E+01_JPRB ,&
&  0.13592E+01_JPRB, 0.12959E+01_JPRB, 0.12216E+01_JPRB, 0.12227E+01_JPRB, 0.12813E+01_JPRB, 0.12176E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.11202E+01_JPRB, 0.10601E+01_JPRB, 0.10580E+01_JPRB, 0.11036E+01_JPRB, 0.11820E+01_JPRB, 0.12312E+01_JPRB ,&
&  0.12987E+01_JPRB, 0.13579E+01_JPRB, 0.13878E+01_JPRB, 0.14876E+01_JPRB, 0.16447E+01_JPRB, 0.18793E+01_JPRB, 0.20740E+01_JPRB ,&
&  0.24522E+01_JPRB, 0.26424E+01_JPRB, 0.28619E+01_JPRB, 0.35117E+01_JPRB, 0.39454E+01_JPRB, 0.40258E+01_JPRB, 0.40185E+01_JPRB ,&
&  0.39414E+01_JPRB, 0.38488E+01_JPRB, 0.39141E+01_JPRB, 0.37577E+01_JPRB, 0.36390E+01_JPRB, 0.36495E+01_JPRB, 0.35836E+01_JPRB ,&
&  0.32817E+01_JPRB, 0.31483E+01_JPRB, 0.31675E+01_JPRB, 0.30467E+01_JPRB, 0.29226E+01_JPRB, 0.27581E+01_JPRB, 0.27573E+01_JPRB ,&
&  0.27729E+01_JPRB, 0.16590E+01_JPRB, 0.12682E+01_JPRB, 0.13725E+01_JPRB, 0.10783E+01_JPRB, 0.97965E+00_JPRB, 0.11132E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10762E+01_JPRB, 0.11131E+01_JPRB, 0.11269E+01_JPRB, 0.11337E+01_JPRB, 0.12813E+01_JPRB, 0.14048E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.15751E+01_JPRB, 0.19242E+01_JPRB, 0.21471E+01_JPRB, 0.22081E+01_JPRB, 0.21979E+01_JPRB, 0.21457E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.20108E+01_JPRB, 0.19604E+01_JPRB, 0.19314E+01_JPRB, 0.18820E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.20549E+01_JPRB, 0.19332E+01_JPRB, 0.17251E+01_JPRB, 0.14769E+01_JPRB, 0.12111E+01_JPRB, 0.10511E+01_JPRB, 0.10254E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.98452E+00_JPRB, 0.93324E+00_JPRB, 0.89286E+00_JPRB, 0.87677E+00_JPRB, 0.86929E+00_JPRB, 0.88781E+00_JPRB ,&
&  0.91245E+00_JPRB, 0.11749E+01_JPRB, 0.98468E+00_JPRB, 0.83908E+00_JPRB, 0.75923E+00_JPRB, 0.85663E+00_JPRB, 0.94342E+00_JPRB ,&
&  0.10606E+01_JPRB, 0.12018E+01_JPRB, 0.12565E+01_JPRB, 0.14716E+01_JPRB, 0.16542E+01_JPRB, 0.17230E+01_JPRB, 0.18476E+01_JPRB ,&
&  0.18534E+01_JPRB, 0.18623E+01_JPRB, 0.16830E+01_JPRB, 0.16722E+01_JPRB, 0.15974E+01_JPRB, 0.14500E+01_JPRB, 0.14911E+01_JPRB ,&
&  0.13967E+01_JPRB, 0.13285E+01_JPRB, 0.12661E+01_JPRB, 0.12552E+01_JPRB, 0.12845E+01_JPRB, 0.12959E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.13171E+01_JPRB, 0.13964E+01_JPRB, 0.14743E+01_JPRB, 0.16012E+01_JPRB, 0.17951E+01_JPRB, 0.18694E+01_JPRB ,&
&  0.20734E+01_JPRB, 0.20144E+01_JPRB, 0.18997E+01_JPRB, 0.25359E+01_JPRB, 0.36413E+01_JPRB, 0.40000E+01_JPRB, 0.38339E+01_JPRB ,&
&  0.41927E+01_JPRB, 0.43153E+01_JPRB, 0.43448E+01_JPRB, 0.41977E+01_JPRB, 0.40444E+01_JPRB, 0.39661E+01_JPRB, 0.38912E+01_JPRB ,&
&  0.36233E+01_JPRB, 0.35034E+01_JPRB, 0.34143E+01_JPRB, 0.33555E+01_JPRB, 0.31870E+01_JPRB, 0.31276E+01_JPRB, 0.31199E+01_JPRB ,&
&  0.28501E+01_JPRB, 0.23651E+01_JPRB, 0.16187E+01_JPRB, 0.15846E+01_JPRB, 0.11673E+01_JPRB, 0.10103E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.11159E+01_JPRB, 0.12391E+01_JPRB, 0.13282E+01_JPRB, 0.16495E+01_JPRB, 0.19566E+01_JPRB, 0.22446E+01_JPRB ,&
&  0.23684E+01_JPRB, 0.22587E+01_JPRB, 0.26347E+01_JPRB, 0.27034E+01_JPRB, 0.26959E+01_JPRB, 0.26980E+01_JPRB, 0.26787E+01_JPRB ,&
&  0.25912E+01_JPRB, 0.24684E+01_JPRB, 0.23619E+01_JPRB, 0.22541E+01_JPRB, 0.21570E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.21908E+01_JPRB, 0.20380E+01_JPRB, 0.17973E+01_JPRB, 0.15298E+01_JPRB, 0.12634E+01_JPRB, 0.10809E+01_JPRB, 0.10610E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.11022E+01_JPRB, 0.10677E+01_JPRB, 0.10034E+01_JPRB, 0.96395E+00_JPRB, 0.94551E+00_JPRB, 0.92666E+00_JPRB ,&
&  0.90244E+00_JPRB, 0.93857E+00_JPRB, 0.78264E+00_JPRB, 0.66725E+00_JPRB, 0.64666E+00_JPRB, 0.69499E+00_JPRB, 0.83664E+00_JPRB ,&
&  0.99687E+00_JPRB, 0.12874E+01_JPRB, 0.15800E+01_JPRB, 0.18500E+01_JPRB, 0.21794E+01_JPRB, 0.24626E+01_JPRB, 0.25824E+01_JPRB ,&
&  0.27227E+01_JPRB, 0.23301E+01_JPRB, 0.23228E+01_JPRB, 0.21591E+01_JPRB, 0.20039E+01_JPRB, 0.18743E+01_JPRB, 0.17768E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.15488E+01_JPRB, 0.14462E+01_JPRB, 0.13824E+01_JPRB, 0.13517E+01_JPRB, 0.13261E+01_JPRB, 0.13094E+01_JPRB ,&
&  0.13508E+01_JPRB, 0.14166E+01_JPRB, 0.14713E+01_JPRB, 0.15115E+01_JPRB, 0.16592E+01_JPRB, 0.18689E+01_JPRB, 0.19494E+01_JPRB ,&
&  0.18572E+01_JPRB, 0.19031E+01_JPRB, 0.24639E+01_JPRB, 0.27833E+01_JPRB, 0.32079E+01_JPRB, 0.37314E+01_JPRB, 0.41026E+01_JPRB ,&
&  0.43619E+01_JPRB, 0.42399E+01_JPRB, 0.42279E+01_JPRB, 0.38611E+01_JPRB, 0.43024E+01_JPRB, 0.42078E+01_JPRB, 0.39958E+01_JPRB ,&
&  0.37446E+01_JPRB, 0.37288E+01_JPRB, 0.34801E+01_JPRB, 0.33217E+01_JPRB, 0.32978E+01_JPRB, 0.34603E+01_JPRB, 0.30828E+01_JPRB ,&
&  0.26019E+01_JPRB, 0.28847E+01_JPRB, 0.25302E+01_JPRB, 0.20937E+01_JPRB, 0.16424E+01_JPRB, 0.14622E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.12760E+01_JPRB, 0.12679E+01_JPRB, 0.14112E+01_JPRB, 0.18126E+01_JPRB, 0.24748E+01_JPRB, 0.29528E+01_JPRB, 0.31434E+01_JPRB ,&
&  0.31142E+01_JPRB, 0.36740E+01_JPRB, 0.36160E+01_JPRB, 0.34487E+01_JPRB, 0.33604E+01_JPRB, 0.33178E+01_JPRB, 0.32055E+01_JPRB ,&
&  0.29726E+01_JPRB, 0.27812E+01_JPRB, 0.26193E+01_JPRB, 0.24708E+01_JPRB, 0.23182E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.22369E+01_JPRB, 0.20978E+01_JPRB, 0.18647E+01_JPRB, 0.15810E+01_JPRB, 0.13243E+01_JPRB, 0.11502E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.11697E+01_JPRB, 0.11796E+01_JPRB, 0.11689E+01_JPRB, 0.11376E+01_JPRB, 0.10852E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.98603E+00_JPRB, 0.85133E+00_JPRB, 0.68719E+00_JPRB, 0.64991E+00_JPRB, 0.67015E+00_JPRB, 0.69225E+00_JPRB, 0.79255E+00_JPRB ,&
&  0.98645E+00_JPRB, 0.14513E+01_JPRB, 0.19082E+01_JPRB, 0.24384E+01_JPRB, 0.29497E+01_JPRB, 0.33285E+01_JPRB, 0.33496E+01_JPRB ,&
&  0.31535E+01_JPRB, 0.31429E+01_JPRB, 0.29423E+01_JPRB, 0.26551E+01_JPRB, 0.24033E+01_JPRB, 0.21918E+01_JPRB, 0.19999E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.16365E+01_JPRB, 0.15473E+01_JPRB, 0.14994E+01_JPRB, 0.14750E+01_JPRB, 0.14608E+01_JPRB, 0.14590E+01_JPRB ,&
&  0.14730E+01_JPRB, 0.15077E+01_JPRB, 0.15319E+01_JPRB, 0.16017E+01_JPRB, 0.15639E+01_JPRB, 0.17134E+01_JPRB, 0.18398E+01_JPRB ,&
&  0.20939E+01_JPRB, 0.20964E+01_JPRB, 0.21569E+01_JPRB, 0.24950E+01_JPRB, 0.24584E+01_JPRB, 0.28420E+01_JPRB, 0.37458E+01_JPRB ,&
&  0.40725E+01_JPRB, 0.40574E+01_JPRB, 0.42232E+01_JPRB, 0.35041E+01_JPRB, 0.31109E+01_JPRB, 0.43114E+01_JPRB, 0.41396E+01_JPRB ,&
&  0.38622E+01_JPRB, 0.37215E+01_JPRB, 0.34492E+01_JPRB, 0.34594E+01_JPRB, 0.29412E+01_JPRB, 0.26221E+01_JPRB, 0.19853E+01_JPRB ,&
&  0.21620E+01_JPRB, 0.21286E+01_JPRB, 0.23962E+01_JPRB, 0.22285E+01_JPRB, 0.21589E+01_JPRB, 0.19701E+01_JPRB, 0.16874E+01_JPRB ,&
&  0.15522E+01_JPRB, 0.15530E+01_JPRB, 0.18698E+01_JPRB, 0.28447E+01_JPRB, 0.40009E+01_JPRB, 0.38354E+01_JPRB, 0.44221E+01_JPRB ,&
&  0.47264E+01_JPRB, 0.47637E+01_JPRB, 0.41425E+01_JPRB, 0.40283E+01_JPRB, 0.37760E+01_JPRB, 0.35251E+01_JPRB, 0.32356E+01_JPRB ,&
&  0.30016E+01_JPRB, 0.28337E+01_JPRB, 0.26833E+01_JPRB, 0.25224E+01_JPRB, 0.23609E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.22762E+01_JPRB, 0.21441E+01_JPRB, 0.19163E+01_JPRB, 0.16357E+01_JPRB, 0.13842E+01_JPRB, 0.12237E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.12379E+01_JPRB, 0.12471E+01_JPRB, 0.12361E+01_JPRB, 0.12146E+01_JPRB, 0.11888E+01_JPRB, 0.11502E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.95512E+00_JPRB, 0.74060E+00_JPRB, 0.65128E+00_JPRB, 0.74612E+00_JPRB, 0.74064E+00_JPRB, 0.78636E+00_JPRB ,&
&  0.10930E+01_JPRB, 0.16266E+01_JPRB, 0.22555E+01_JPRB, 0.29201E+01_JPRB, 0.35628E+01_JPRB, 0.38355E+01_JPRB, 0.37776E+01_JPRB ,&
&  0.38333E+01_JPRB, 0.35197E+01_JPRB, 0.30577E+01_JPRB, 0.26564E+01_JPRB, 0.23730E+01_JPRB, 0.21810E+01_JPRB, 0.19692E+01_JPRB ,&
&  0.17544E+01_JPRB, 0.16085E+01_JPRB, 0.15300E+01_JPRB, 0.14830E+01_JPRB, 0.14681E+01_JPRB, 0.14438E+01_JPRB, 0.14226E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.15000E+01_JPRB, 0.15595E+01_JPRB, 0.16394E+01_JPRB, 0.15890E+01_JPRB, 0.16477E+01_JPRB, 0.18537E+01_JPRB ,&
&  0.19523E+01_JPRB, 0.19465E+01_JPRB, 0.20143E+01_JPRB, 0.21929E+01_JPRB, 0.25366E+01_JPRB, 0.27313E+01_JPRB, 0.34046E+01_JPRB ,&
&  0.32555E+01_JPRB, 0.30423E+01_JPRB, 0.28713E+01_JPRB, 0.25135E+01_JPRB, 0.28816E+01_JPRB, 0.40949E+01_JPRB, 0.41884E+01_JPRB ,&
&  0.39138E+01_JPRB, 0.37170E+01_JPRB, 0.35329E+01_JPRB, 0.31405E+01_JPRB, 0.25875E+01_JPRB, 0.20008E+01_JPRB, 0.24307E+01_JPRB ,&
&  0.26584E+01_JPRB, 0.27746E+01_JPRB, 0.26406E+01_JPRB, 0.22346E+01_JPRB, 0.20867E+01_JPRB, 0.25042E+01_JPRB, 0.23700E+01_JPRB ,&
&  0.20828E+01_JPRB, 0.21164E+01_JPRB, 0.33299E+01_JPRB, 0.48241E+01_JPRB, 0.54142E+01_JPRB, 0.50672E+01_JPRB, 0.55696E+01_JPRB ,&
&  0.52836E+01_JPRB, 0.50402E+01_JPRB, 0.46326E+01_JPRB, 0.41279E+01_JPRB, 0.36915E+01_JPRB, 0.33322E+01_JPRB, 0.30435E+01_JPRB ,&
&  0.28838E+01_JPRB, 0.27940E+01_JPRB, 0.26849E+01_JPRB, 0.25697E+01_JPRB, 0.24122E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.23019E+01_JPRB, 0.21531E+01_JPRB, 0.19151E+01_JPRB, 0.16331E+01_JPRB, 0.14287E+01_JPRB, 0.12950E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.12447E+01_JPRB, 0.12438E+01_JPRB, 0.12301E+01_JPRB, 0.12147E+01_JPRB, 0.11992E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11966E+01_JPRB, 0.91316E+00_JPRB, 0.76365E+00_JPRB, 0.79091E+00_JPRB, 0.83694E+00_JPRB, 0.83861E+00_JPRB ,&
&  0.12630E+01_JPRB, 0.19522E+01_JPRB, 0.24925E+01_JPRB, 0.30869E+01_JPRB, 0.38538E+01_JPRB, 0.36200E+01_JPRB, 0.36000E+01_JPRB ,&
&  0.34949E+01_JPRB, 0.32142E+01_JPRB, 0.28889E+01_JPRB, 0.25443E+01_JPRB, 0.22150E+01_JPRB, 0.20111E+01_JPRB, 0.18732E+01_JPRB ,&
&  0.17257E+01_JPRB, 0.16026E+01_JPRB, 0.15338E+01_JPRB, 0.15123E+01_JPRB, 0.14883E+01_JPRB, 0.14698E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.14802E+01_JPRB, 0.14965E+01_JPRB, 0.15225E+01_JPRB, 0.16389E+01_JPRB, 0.17260E+01_JPRB, 0.17061E+01_JPRB, 0.20037E+01_JPRB ,&
&  0.19535E+01_JPRB, 0.19657E+01_JPRB, 0.19740E+01_JPRB, 0.21643E+01_JPRB, 0.23536E+01_JPRB, 0.25894E+01_JPRB, 0.31063E+01_JPRB ,&
&  0.30017E+01_JPRB, 0.29931E+01_JPRB, 0.31449E+01_JPRB, 0.31392E+01_JPRB, 0.28871E+01_JPRB, 0.31830E+01_JPRB, 0.42186E+01_JPRB ,&
&  0.34160E+01_JPRB, 0.34168E+01_JPRB, 0.34199E+01_JPRB, 0.33269E+01_JPRB, 0.17890E+01_JPRB, 0.13664E+01_JPRB, 0.21501E+01_JPRB ,&
&  0.23166E+01_JPRB, 0.14656E+01_JPRB, 0.11166E+01_JPRB, 0.15376E+01_JPRB, 0.12841E+01_JPRB, 0.16970E+01_JPRB, 0.26671E+01_JPRB ,&
&  0.30224E+01_JPRB, 0.29172E+01_JPRB, 0.44479E+01_JPRB, 0.53825E+01_JPRB, 0.55986E+01_JPRB, 0.58043E+01_JPRB, 0.58448E+01_JPRB ,&
&  0.55867E+01_JPRB, 0.46092E+01_JPRB, 0.45079E+01_JPRB, 0.39605E+01_JPRB, 0.35173E+01_JPRB, 0.32379E+01_JPRB, 0.30177E+01_JPRB ,&
&  0.27881E+01_JPRB, 0.26000E+01_JPRB, 0.25123E+01_JPRB, 0.24975E+01_JPRB, 0.24226E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.22645E+01_JPRB, 0.21137E+01_JPRB, 0.18687E+01_JPRB, 0.15973E+01_JPRB, 0.14331E+01_JPRB, 0.13158E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.12315E+01_JPRB, 0.12237E+01_JPRB, 0.12098E+01_JPRB, 0.11963E+01_JPRB, 0.11957E+01_JPRB, 0.11957E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11348E+01_JPRB, 0.10970E+01_JPRB, 0.97240E+00_JPRB, 0.94812E+00_JPRB, 0.93634E+00_JPRB, 0.10251E+01_JPRB ,&
&  0.14255E+01_JPRB, 0.21460E+01_JPRB, 0.27231E+01_JPRB, 0.29920E+01_JPRB, 0.30990E+01_JPRB, 0.29426E+01_JPRB, 0.31658E+01_JPRB ,&
&  0.29493E+01_JPRB, 0.27058E+01_JPRB, 0.24684E+01_JPRB, 0.22773E+01_JPRB, 0.20917E+01_JPRB, 0.19623E+01_JPRB, 0.18696E+01_JPRB ,&
&  0.17325E+01_JPRB, 0.16202E+01_JPRB, 0.15708E+01_JPRB, 0.15344E+01_JPRB, 0.15033E+01_JPRB, 0.14844E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.14653E+01_JPRB, 0.14772E+01_JPRB, 0.15126E+01_JPRB, 0.15963E+01_JPRB, 0.18089E+01_JPRB, 0.18261E+01_JPRB, 0.17463E+01_JPRB ,&
&  0.17005E+01_JPRB, 0.17568E+01_JPRB, 0.20264E+01_JPRB, 0.20713E+01_JPRB, 0.21685E+01_JPRB, 0.24084E+01_JPRB, 0.27983E+01_JPRB ,&
&  0.32119E+01_JPRB, 0.35079E+01_JPRB, 0.31516E+01_JPRB, 0.32366E+01_JPRB, 0.30171E+01_JPRB, 0.25148E+01_JPRB, 0.29723E+01_JPRB ,&
&  0.32521E+01_JPRB, 0.28658E+01_JPRB, 0.23384E+01_JPRB, 0.18944E+01_JPRB, 0.19582E+01_JPRB, 0.15666E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.91125E+00_JPRB, 0.85634E+00_JPRB, 0.78899E+00_JPRB, 0.88481E+00_JPRB, 0.92489E+00_JPRB, 0.14904E+01_JPRB, 0.26230E+01_JPRB ,&
&  0.38433E+01_JPRB, 0.42517E+01_JPRB, 0.51900E+01_JPRB, 0.55492E+01_JPRB, 0.54572E+01_JPRB, 0.56936E+01_JPRB, 0.58675E+01_JPRB ,&
&  0.47567E+01_JPRB, 0.42427E+01_JPRB, 0.40935E+01_JPRB, 0.35837E+01_JPRB, 0.32969E+01_JPRB, 0.30910E+01_JPRB, 0.28442E+01_JPRB ,&
&  0.25467E+01_JPRB, 0.22980E+01_JPRB, 0.22284E+01_JPRB, 0.22935E+01_JPRB, 0.23534E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.22090E+01_JPRB, 0.20683E+01_JPRB, 0.18517E+01_JPRB, 0.16135E+01_JPRB, 0.14752E+01_JPRB, 0.13784E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.12543E+01_JPRB, 0.12355E+01_JPRB, 0.12227E+01_JPRB, 0.12196E+01_JPRB, 0.12257E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.11928E+01_JPRB, 0.11352E+01_JPRB, 0.11803E+01_JPRB, 0.11296E+01_JPRB, 0.11329E+01_JPRB, 0.10753E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.16186E+01_JPRB, 0.23638E+01_JPRB, 0.29020E+01_JPRB, 0.29424E+01_JPRB, 0.29205E+01_JPRB, 0.28220E+01_JPRB, 0.25809E+01_JPRB ,&
&  0.23284E+01_JPRB, 0.22116E+01_JPRB, 0.20932E+01_JPRB, 0.20255E+01_JPRB, 0.19685E+01_JPRB, 0.19251E+01_JPRB, 0.18366E+01_JPRB ,&
&  0.17354E+01_JPRB, 0.16728E+01_JPRB, 0.16023E+01_JPRB, 0.15507E+01_JPRB, 0.15011E+01_JPRB, 0.14600E+01_JPRB, 0.14303E+01_JPRB ,&
&  0.14361E+01_JPRB, 0.14758E+01_JPRB, 0.15487E+01_JPRB, 0.17034E+01_JPRB, 0.16409E+01_JPRB, 0.14980E+01_JPRB, 0.16258E+01_JPRB ,&
&  0.17395E+01_JPRB, 0.18945E+01_JPRB, 0.18984E+01_JPRB, 0.20760E+01_JPRB, 0.21177E+01_JPRB, 0.22133E+01_JPRB, 0.26022E+01_JPRB ,&
&  0.30130E+01_JPRB, 0.34313E+01_JPRB, 0.28789E+01_JPRB, 0.28550E+01_JPRB, 0.31583E+01_JPRB, 0.26108E+01_JPRB, 0.25603E+01_JPRB ,&
&  0.28945E+01_JPRB, 0.25401E+01_JPRB, 0.22799E+01_JPRB, 0.20465E+01_JPRB, 0.27611E+01_JPRB, 0.28506E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.90819E+00_JPRB, 0.82128E+00_JPRB, 0.79510E+00_JPRB, 0.89705E+00_JPRB, 0.15902E+01_JPRB, 0.33345E+01_JPRB ,&
&  0.40455E+01_JPRB, 0.52475E+01_JPRB, 0.59696E+01_JPRB, 0.60552E+01_JPRB, 0.56840E+01_JPRB, 0.54119E+01_JPRB, 0.47451E+01_JPRB ,&
&  0.46137E+01_JPRB, 0.41652E+01_JPRB, 0.36240E+01_JPRB, 0.31714E+01_JPRB, 0.28171E+01_JPRB, 0.24627E+01_JPRB, 0.21818E+01_JPRB ,&
&  0.19972E+01_JPRB, 0.19433E+01_JPRB, 0.19639E+01_JPRB, 0.20834E+01_JPRB, 0.22577E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.21536E+01_JPRB, 0.20768E+01_JPRB, 0.18871E+01_JPRB, 0.16740E+01_JPRB, 0.15403E+01_JPRB, 0.14466E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.13321E+01_JPRB, 0.12765E+01_JPRB, 0.12346E+01_JPRB, 0.12138E+01_JPRB, 0.12202E+01_JPRB, 0.12228E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.11397E+01_JPRB, 0.11488E+01_JPRB, 0.11858E+01_JPRB, 0.11739E+01_JPRB, 0.11228E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.18872E+01_JPRB, 0.26732E+01_JPRB, 0.29731E+01_JPRB, 0.29534E+01_JPRB, 0.26901E+01_JPRB, 0.25374E+01_JPRB, 0.20180E+01_JPRB ,&
&  0.18040E+01_JPRB, 0.18372E+01_JPRB, 0.18390E+01_JPRB, 0.19067E+01_JPRB, 0.19506E+01_JPRB, 0.19206E+01_JPRB, 0.18383E+01_JPRB ,&
&  0.17615E+01_JPRB, 0.16573E+01_JPRB, 0.15769E+01_JPRB, 0.15088E+01_JPRB, 0.14540E+01_JPRB, 0.14091E+01_JPRB, 0.14125E+01_JPRB ,&
&  0.14631E+01_JPRB, 0.15203E+01_JPRB, 0.16351E+01_JPRB, 0.17555E+01_JPRB, 0.14640E+01_JPRB, 0.15856E+01_JPRB, 0.16395E+01_JPRB ,&
&  0.16838E+01_JPRB, 0.17475E+01_JPRB, 0.17136E+01_JPRB, 0.18352E+01_JPRB, 0.20123E+01_JPRB, 0.21527E+01_JPRB, 0.24093E+01_JPRB ,&
&  0.28849E+01_JPRB, 0.27564E+01_JPRB, 0.25160E+01_JPRB, 0.25568E+01_JPRB, 0.28097E+01_JPRB, 0.33494E+01_JPRB, 0.23324E+01_JPRB ,&
&  0.21411E+01_JPRB, 0.24812E+01_JPRB, 0.23668E+01_JPRB, 0.23667E+01_JPRB, 0.30536E+01_JPRB, 0.38307E+01_JPRB, 0.35564E+01_JPRB ,&
&  0.15173E+01_JPRB, 0.10254E+01_JPRB, 0.10246E+01_JPRB, 0.10119E+01_JPRB, 0.10091E+01_JPRB, 0.16153E+01_JPRB, 0.60016E+01_JPRB ,&
&  0.51533E+01_JPRB, 0.62673E+01_JPRB, 0.63848E+01_JPRB, 0.58176E+01_JPRB, 0.54795E+01_JPRB, 0.49776E+01_JPRB, 0.44576E+01_JPRB ,&
&  0.38752E+01_JPRB, 0.33484E+01_JPRB, 0.28293E+01_JPRB, 0.23356E+01_JPRB, 0.19602E+01_JPRB, 0.17267E+01_JPRB, 0.16412E+01_JPRB ,&
&  0.16428E+01_JPRB, 0.16974E+01_JPRB, 0.17854E+01_JPRB, 0.19243E+01_JPRB, 0.21111E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.21035E+01_JPRB, 0.20731E+01_JPRB, 0.18923E+01_JPRB, 0.17037E+01_JPRB, 0.15744E+01_JPRB, 0.14973E+01_JPRB, 0.14081E+01_JPRB ,&
&  0.13715E+01_JPRB, 0.12899E+01_JPRB, 0.12152E+01_JPRB, 0.11751E+01_JPRB, 0.11772E+01_JPRB, 0.11736E+01_JPRB, 0.11751E+01_JPRB ,&
&  0.11591E+01_JPRB, 0.11359E+01_JPRB, 0.11408E+01_JPRB, 0.11456E+01_JPRB, 0.11910E+01_JPRB, 0.10816E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.29947E+01_JPRB, 0.27366E+01_JPRB, 0.26623E+01_JPRB, 0.25019E+01_JPRB, 0.24599E+01_JPRB, 0.17878E+01_JPRB ,&
&  0.16595E+01_JPRB, 0.17324E+01_JPRB, 0.17777E+01_JPRB, 0.18666E+01_JPRB, 0.18857E+01_JPRB, 0.18524E+01_JPRB, 0.17973E+01_JPRB ,&
&  0.16949E+01_JPRB, 0.15989E+01_JPRB, 0.15438E+01_JPRB, 0.14866E+01_JPRB, 0.14315E+01_JPRB, 0.14176E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.15095E+01_JPRB, 0.15728E+01_JPRB, 0.17084E+01_JPRB, 0.16754E+01_JPRB, 0.16082E+01_JPRB, 0.16103E+01_JPRB, 0.16626E+01_JPRB ,&
&  0.16581E+01_JPRB, 0.16955E+01_JPRB, 0.17236E+01_JPRB, 0.17468E+01_JPRB, 0.19061E+01_JPRB, 0.21375E+01_JPRB, 0.23263E+01_JPRB ,&
&  0.25516E+01_JPRB, 0.24443E+01_JPRB, 0.21870E+01_JPRB, 0.22529E+01_JPRB, 0.25585E+01_JPRB, 0.30598E+01_JPRB, 0.29368E+01_JPRB ,&
&  0.26130E+01_JPRB, 0.21381E+01_JPRB, 0.22278E+01_JPRB, 0.28527E+01_JPRB, 0.33404E+01_JPRB, 0.37673E+01_JPRB, 0.51308E+01_JPRB ,&
&  0.51046E+01_JPRB, 0.28350E+01_JPRB, 0.22827E+01_JPRB, 0.29642E+01_JPRB, 0.21480E+01_JPRB, 0.19034E+01_JPRB, 0.37211E+01_JPRB ,&
&  0.49088E+01_JPRB, 0.58926E+01_JPRB, 0.60790E+01_JPRB, 0.54498E+01_JPRB, 0.51700E+01_JPRB, 0.45486E+01_JPRB, 0.36629E+01_JPRB ,&
&  0.29316E+01_JPRB, 0.23137E+01_JPRB, 0.18984E+01_JPRB, 0.16829E+01_JPRB, 0.15964E+01_JPRB, 0.15526E+01_JPRB, 0.15389E+01_JPRB ,&
&  0.15616E+01_JPRB, 0.16230E+01_JPRB, 0.17284E+01_JPRB, 0.18939E+01_JPRB, 0.20527E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.20420E+01_JPRB, 0.19911E+01_JPRB, 0.18328E+01_JPRB, 0.16840E+01_JPRB, 0.15816E+01_JPRB, 0.14972E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.14819E+01_JPRB, 0.13058E+01_JPRB, 0.11837E+01_JPRB, 0.11335E+01_JPRB, 0.11123E+01_JPRB, 0.11156E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.11265E+01_JPRB, 0.11444E+01_JPRB, 0.11612E+01_JPRB, 0.12242E+01_JPRB, 0.14507E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.19718E+01_JPRB, 0.32281E+01_JPRB, 0.27569E+01_JPRB, 0.25304E+01_JPRB, 0.22693E+01_JPRB, 0.21817E+01_JPRB, 0.17544E+01_JPRB ,&
&  0.16426E+01_JPRB, 0.16722E+01_JPRB, 0.17120E+01_JPRB, 0.17603E+01_JPRB, 0.17858E+01_JPRB, 0.17664E+01_JPRB, 0.16687E+01_JPRB ,&
&  0.15843E+01_JPRB, 0.15242E+01_JPRB, 0.14994E+01_JPRB, 0.14647E+01_JPRB, 0.14445E+01_JPRB, 0.14624E+01_JPRB, 0.14898E+01_JPRB ,&
&  0.15320E+01_JPRB, 0.16559E+01_JPRB, 0.17687E+01_JPRB, 0.16450E+01_JPRB, 0.16358E+01_JPRB, 0.16672E+01_JPRB, 0.16994E+01_JPRB ,&
&  0.16133E+01_JPRB, 0.15235E+01_JPRB, 0.16439E+01_JPRB, 0.17799E+01_JPRB, 0.19027E+01_JPRB, 0.20062E+01_JPRB, 0.20702E+01_JPRB ,&
&  0.22789E+01_JPRB, 0.23316E+01_JPRB, 0.23611E+01_JPRB, 0.22093E+01_JPRB, 0.23957E+01_JPRB, 0.27870E+01_JPRB, 0.29453E+01_JPRB ,&
&  0.30613E+01_JPRB, 0.28330E+01_JPRB, 0.30693E+01_JPRB, 0.34332E+01_JPRB, 0.35623E+01_JPRB, 0.35999E+01_JPRB, 0.41038E+01_JPRB ,&
&  0.47305E+01_JPRB, 0.56677E+01_JPRB, 0.51210E+01_JPRB, 0.40608E+01_JPRB, 0.33632E+01_JPRB, 0.28108E+01_JPRB, 0.36921E+01_JPRB ,&
&  0.50271E+01_JPRB, 0.52120E+01_JPRB, 0.51567E+01_JPRB, 0.59260E+01_JPRB, 0.43664E+01_JPRB, 0.32177E+01_JPRB, 0.23425E+01_JPRB ,&
&  0.18286E+01_JPRB, 0.16544E+01_JPRB, 0.16221E+01_JPRB, 0.15724E+01_JPRB, 0.15460E+01_JPRB, 0.15244E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.16951E+01_JPRB, 0.18077E+01_JPRB, 0.19332E+01_JPRB, 0.20288E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.19133E+01_JPRB, 0.18319E+01_JPRB, 0.17197E+01_JPRB, 0.16087E+01_JPRB, 0.15083E+01_JPRB, 0.14186E+01_JPRB, 0.13645E+01_JPRB ,&
&  0.15481E+01_JPRB, 0.12400E+01_JPRB, 0.11178E+01_JPRB, 0.10823E+01_JPRB, 0.10750E+01_JPRB, 0.10736E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.11279E+01_JPRB, 0.11689E+01_JPRB, 0.12315E+01_JPRB, 0.13837E+01_JPRB, 0.17505E+01_JPRB, 0.19919E+01_JPRB ,&
&  0.21429E+01_JPRB, 0.36497E+01_JPRB, 0.27522E+01_JPRB, 0.24308E+01_JPRB, 0.21865E+01_JPRB, 0.21263E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.16634E+01_JPRB, 0.16344E+01_JPRB, 0.16512E+01_JPRB, 0.16492E+01_JPRB, 0.16443E+01_JPRB, 0.15988E+01_JPRB, 0.15536E+01_JPRB ,&
&  0.15071E+01_JPRB, 0.14758E+01_JPRB, 0.14546E+01_JPRB, 0.14396E+01_JPRB, 0.14352E+01_JPRB, 0.14739E+01_JPRB, 0.15235E+01_JPRB ,&
&  0.16216E+01_JPRB, 0.17615E+01_JPRB, 0.17215E+01_JPRB, 0.16515E+01_JPRB, 0.16255E+01_JPRB, 0.16734E+01_JPRB, 0.17054E+01_JPRB ,&
&  0.16410E+01_JPRB, 0.15663E+01_JPRB, 0.16626E+01_JPRB, 0.17105E+01_JPRB, 0.17407E+01_JPRB, 0.18995E+01_JPRB, 0.19099E+01_JPRB ,&
&  0.21646E+01_JPRB, 0.21826E+01_JPRB, 0.26080E+01_JPRB, 0.23581E+01_JPRB, 0.23013E+01_JPRB, 0.26484E+01_JPRB, 0.26537E+01_JPRB ,&
&  0.24431E+01_JPRB, 0.27636E+01_JPRB, 0.31925E+01_JPRB, 0.36716E+01_JPRB, 0.38160E+01_JPRB, 0.38684E+01_JPRB, 0.37890E+01_JPRB ,&
&  0.35066E+01_JPRB, 0.37072E+01_JPRB, 0.32239E+01_JPRB, 0.27524E+01_JPRB, 0.30460E+01_JPRB, 0.29824E+01_JPRB, 0.40472E+01_JPRB ,&
&  0.52034E+01_JPRB, 0.52276E+01_JPRB, 0.52440E+01_JPRB, 0.44949E+01_JPRB, 0.28819E+01_JPRB, 0.20226E+01_JPRB, 0.16097E+01_JPRB ,&
&  0.15333E+01_JPRB, 0.15668E+01_JPRB, 0.15848E+01_JPRB, 0.15586E+01_JPRB, 0.15525E+01_JPRB, 0.15662E+01_JPRB, 0.16157E+01_JPRB ,&
&  0.17086E+01_JPRB, 0.18168E+01_JPRB, 0.18844E+01_JPRB, 0.19241E+01_JPRB, 0.19540E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.17753E+01_JPRB, 0.17017E+01_JPRB, 0.16181E+01_JPRB, 0.15479E+01_JPRB, 0.15090E+01_JPRB, 0.14289E+01_JPRB, 0.14454E+01_JPRB ,&
&  0.15973E+01_JPRB, 0.11520E+01_JPRB, 0.10484E+01_JPRB, 0.10366E+01_JPRB, 0.10449E+01_JPRB, 0.10629E+01_JPRB, 0.11045E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.11869E+01_JPRB, 0.12625E+01_JPRB, 0.14311E+01_JPRB, 0.17595E+01_JPRB, 0.23183E+01_JPRB, 0.26874E+01_JPRB ,&
&  0.29279E+01_JPRB, 0.29556E+01_JPRB, 0.27880E+01_JPRB, 0.22483E+01_JPRB, 0.20934E+01_JPRB, 0.20453E+01_JPRB, 0.20250E+01_JPRB ,&
&  0.17535E+01_JPRB, 0.16454E+01_JPRB, 0.16121E+01_JPRB, 0.15623E+01_JPRB, 0.15037E+01_JPRB, 0.14905E+01_JPRB, 0.14863E+01_JPRB ,&
&  0.14745E+01_JPRB, 0.14535E+01_JPRB, 0.14408E+01_JPRB, 0.14437E+01_JPRB, 0.14780E+01_JPRB, 0.15622E+01_JPRB, 0.16493E+01_JPRB ,&
&  0.17499E+01_JPRB, 0.18093E+01_JPRB, 0.17407E+01_JPRB, 0.16449E+01_JPRB, 0.16253E+01_JPRB, 0.16732E+01_JPRB, 0.16908E+01_JPRB ,&
&  0.16733E+01_JPRB, 0.16866E+01_JPRB, 0.17194E+01_JPRB, 0.17224E+01_JPRB, 0.16608E+01_JPRB, 0.18490E+01_JPRB, 0.19012E+01_JPRB ,&
&  0.20948E+01_JPRB, 0.22227E+01_JPRB, 0.27128E+01_JPRB, 0.22301E+01_JPRB, 0.24039E+01_JPRB, 0.25152E+01_JPRB, 0.25469E+01_JPRB ,&
&  0.24996E+01_JPRB, 0.29520E+01_JPRB, 0.34491E+01_JPRB, 0.37029E+01_JPRB, 0.40569E+01_JPRB, 0.38510E+01_JPRB, 0.40167E+01_JPRB ,&
&  0.30697E+01_JPRB, 0.30023E+01_JPRB, 0.24007E+01_JPRB, 0.26470E+01_JPRB, 0.28176E+01_JPRB, 0.29051E+01_JPRB, 0.34962E+01_JPRB ,&
&  0.43919E+01_JPRB, 0.41721E+01_JPRB, 0.35454E+01_JPRB, 0.26102E+01_JPRB, 0.17271E+01_JPRB, 0.14635E+01_JPRB, 0.14042E+01_JPRB ,&
&  0.14407E+01_JPRB, 0.15519E+01_JPRB, 0.15805E+01_JPRB, 0.15988E+01_JPRB, 0.16077E+01_JPRB, 0.16337E+01_JPRB, 0.16849E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.18296E+01_JPRB, 0.18608E+01_JPRB, 0.18526E+01_JPRB, 0.18308E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.16189E+01_JPRB, 0.15540E+01_JPRB, 0.15011E+01_JPRB, 0.14309E+01_JPRB, 0.13018E+01_JPRB, 0.12041E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.10407E+01_JPRB, 0.10488E+01_JPRB, 0.10636E+01_JPRB, 0.10877E+01_JPRB, 0.11215E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.12330E+01_JPRB, 0.13252E+01_JPRB, 0.15107E+01_JPRB, 0.18054E+01_JPRB, 0.21642E+01_JPRB, 0.26187E+01_JPRB, 0.31925E+01_JPRB ,&
&  0.31731E+01_JPRB, 0.21369E+01_JPRB, 0.23914E+01_JPRB, 0.22143E+01_JPRB, 0.20497E+01_JPRB, 0.18878E+01_JPRB, 0.17931E+01_JPRB ,&
&  0.17158E+01_JPRB, 0.16689E+01_JPRB, 0.16451E+01_JPRB, 0.15918E+01_JPRB, 0.15069E+01_JPRB, 0.15168E+01_JPRB, 0.15231E+01_JPRB ,&
&  0.15162E+01_JPRB, 0.15210E+01_JPRB, 0.15389E+01_JPRB, 0.15774E+01_JPRB, 0.16525E+01_JPRB, 0.17463E+01_JPRB, 0.18196E+01_JPRB ,&
&  0.18634E+01_JPRB, 0.18986E+01_JPRB, 0.18488E+01_JPRB, 0.17811E+01_JPRB, 0.17353E+01_JPRB, 0.17459E+01_JPRB, 0.17237E+01_JPRB ,&
&  0.17067E+01_JPRB, 0.16813E+01_JPRB, 0.17370E+01_JPRB, 0.18409E+01_JPRB, 0.19096E+01_JPRB, 0.18133E+01_JPRB, 0.19774E+01_JPRB ,&
&  0.21147E+01_JPRB, 0.22972E+01_JPRB, 0.23921E+01_JPRB, 0.28860E+01_JPRB, 0.20917E+01_JPRB, 0.22837E+01_JPRB, 0.22327E+01_JPRB ,&
&  0.28310E+01_JPRB, 0.30980E+01_JPRB, 0.33838E+01_JPRB, 0.36446E+01_JPRB, 0.41544E+01_JPRB, 0.37965E+01_JPRB, 0.35400E+01_JPRB ,&
&  0.28607E+01_JPRB, 0.23625E+01_JPRB, 0.21046E+01_JPRB, 0.23904E+01_JPRB, 0.26627E+01_JPRB, 0.30650E+01_JPRB, 0.31836E+01_JPRB ,&
&  0.33988E+01_JPRB, 0.27891E+01_JPRB, 0.21405E+01_JPRB, 0.18694E+01_JPRB, 0.14937E+01_JPRB, 0.13853E+01_JPRB, 0.14158E+01_JPRB ,&
&  0.14742E+01_JPRB, 0.15639E+01_JPRB, 0.15875E+01_JPRB, 0.15924E+01_JPRB, 0.16014E+01_JPRB, 0.16185E+01_JPRB, 0.16513E+01_JPRB ,&
&  0.16847E+01_JPRB, 0.17216E+01_JPRB, 0.17336E+01_JPRB, 0.17107E+01_JPRB, 0.16688E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.14774E+01_JPRB, 0.14151E+01_JPRB, 0.13171E+01_JPRB, 0.11930E+01_JPRB, 0.10950E+01_JPRB, 0.10366E+01_JPRB, 0.10063E+01_JPRB ,&
&  0.10184E+01_JPRB, 0.10435E+01_JPRB, 0.10740E+01_JPRB, 0.11147E+01_JPRB, 0.11563E+01_JPRB, 0.11926E+01_JPRB, 0.12735E+01_JPRB ,&
&  0.13657E+01_JPRB, 0.15230E+01_JPRB, 0.17618E+01_JPRB, 0.20224E+01_JPRB, 0.22579E+01_JPRB, 0.25133E+01_JPRB, 0.27577E+01_JPRB ,&
&  0.30306E+01_JPRB, 0.30456E+01_JPRB, 0.26887E+01_JPRB, 0.18470E+01_JPRB, 0.16262E+01_JPRB, 0.17347E+01_JPRB, 0.17008E+01_JPRB ,&
&  0.16825E+01_JPRB, 0.16501E+01_JPRB, 0.16244E+01_JPRB, 0.16113E+01_JPRB, 0.16102E+01_JPRB, 0.16255E+01_JPRB, 0.16284E+01_JPRB ,&
&  0.16453E+01_JPRB, 0.16762E+01_JPRB, 0.17153E+01_JPRB, 0.17626E+01_JPRB, 0.18088E+01_JPRB, 0.18695E+01_JPRB, 0.19405E+01_JPRB ,&
&  0.19873E+01_JPRB, 0.20533E+01_JPRB, 0.20708E+01_JPRB, 0.20414E+01_JPRB, 0.19633E+01_JPRB, 0.19361E+01_JPRB, 0.18948E+01_JPRB ,&
&  0.18782E+01_JPRB, 0.18518E+01_JPRB, 0.18639E+01_JPRB, 0.19608E+01_JPRB, 0.20172E+01_JPRB, 0.18480E+01_JPRB, 0.19015E+01_JPRB ,&
&  0.21809E+01_JPRB, 0.24503E+01_JPRB, 0.22455E+01_JPRB, 0.27861E+01_JPRB, 0.17674E+01_JPRB, 0.21793E+01_JPRB, 0.25948E+01_JPRB ,&
&  0.26724E+01_JPRB, 0.29027E+01_JPRB, 0.32023E+01_JPRB, 0.34587E+01_JPRB, 0.35499E+01_JPRB, 0.32766E+01_JPRB, 0.30622E+01_JPRB ,&
&  0.25945E+01_JPRB, 0.20954E+01_JPRB, 0.18916E+01_JPRB, 0.20083E+01_JPRB, 0.23262E+01_JPRB, 0.26606E+01_JPRB, 0.22799E+01_JPRB ,&
&  0.18191E+01_JPRB, 0.19092E+01_JPRB, 0.17625E+01_JPRB, 0.18783E+01_JPRB, 0.16081E+01_JPRB, 0.13634E+01_JPRB, 0.14363E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.15367E+01_JPRB, 0.15589E+01_JPRB, 0.15487E+01_JPRB, 0.15566E+01_JPRB, 0.15613E+01_JPRB, 0.15665E+01_JPRB ,&
&  0.15823E+01_JPRB, 0.16125E+01_JPRB, 0.16192E+01_JPRB, 0.15781E+01_JPRB, 0.15238E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.13071E+01_JPRB, 0.11965E+01_JPRB, 0.10743E+01_JPRB, 0.98355E+00_JPRB, 0.94013E+00_JPRB, 0.92909E+00_JPRB, 0.94307E+00_JPRB ,&
&  0.97510E+00_JPRB, 0.10147E+01_JPRB, 0.10619E+01_JPRB, 0.11213E+01_JPRB, 0.11774E+01_JPRB, 0.12367E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.14204E+01_JPRB, 0.15572E+01_JPRB, 0.17470E+01_JPRB, 0.19580E+01_JPRB, 0.21366E+01_JPRB, 0.23179E+01_JPRB, 0.24871E+01_JPRB ,&
&  0.26617E+01_JPRB, 0.27141E+01_JPRB, 0.26111E+01_JPRB, 0.24287E+01_JPRB, 0.19333E+01_JPRB, 0.19753E+01_JPRB, 0.20962E+01_JPRB ,&
&  0.21460E+01_JPRB, 0.18283E+01_JPRB, 0.17275E+01_JPRB, 0.17045E+01_JPRB, 0.17111E+01_JPRB, 0.17326E+01_JPRB, 0.17468E+01_JPRB ,&
&  0.17748E+01_JPRB, 0.18046E+01_JPRB, 0.18306E+01_JPRB, 0.18525E+01_JPRB, 0.18822E+01_JPRB, 0.19495E+01_JPRB, 0.20785E+01_JPRB ,&
&  0.21237E+01_JPRB, 0.21639E+01_JPRB, 0.21870E+01_JPRB, 0.21522E+01_JPRB, 0.22217E+01_JPRB, 0.22940E+01_JPRB, 0.22712E+01_JPRB ,&
&  0.22393E+01_JPRB, 0.21132E+01_JPRB, 0.21641E+01_JPRB, 0.22807E+01_JPRB, 0.22601E+01_JPRB, 0.21386E+01_JPRB, 0.21655E+01_JPRB ,&
&  0.24176E+01_JPRB, 0.26984E+01_JPRB, 0.20003E+01_JPRB, 0.23226E+01_JPRB, 0.27554E+01_JPRB, 0.24126E+01_JPRB, 0.22290E+01_JPRB ,&
&  0.21070E+01_JPRB, 0.22544E+01_JPRB, 0.26900E+01_JPRB, 0.28869E+01_JPRB, 0.28741E+01_JPRB, 0.33191E+01_JPRB, 0.27159E+01_JPRB ,&
&  0.22822E+01_JPRB, 0.18139E+01_JPRB, 0.16439E+01_JPRB, 0.16507E+01_JPRB, 0.18081E+01_JPRB, 0.18715E+01_JPRB, 0.14340E+01_JPRB ,&
&  0.12587E+01_JPRB, 0.15998E+01_JPRB, 0.16665E+01_JPRB, 0.19218E+01_JPRB, 0.18892E+01_JPRB, 0.13111E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.14336E+01_JPRB, 0.14724E+01_JPRB, 0.14820E+01_JPRB, 0.14812E+01_JPRB, 0.14793E+01_JPRB, 0.14774E+01_JPRB, 0.14829E+01_JPRB ,&
&  0.14924E+01_JPRB, 0.15013E+01_JPRB, 0.14963E+01_JPRB, 0.14518E+01_JPRB, 0.13865E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.10498E+01_JPRB, 0.95082E+00_JPRB, 0.88016E+00_JPRB, 0.85608E+00_JPRB, 0.86460E+00_JPRB, 0.87942E+00_JPRB, 0.88892E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.94536E+00_JPRB, 0.99434E+00_JPRB, 0.10377E+01_JPRB, 0.10888E+01_JPRB, 0.11357E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.12863E+01_JPRB, 0.13729E+01_JPRB, 0.15173E+01_JPRB, 0.17295E+01_JPRB, 0.18937E+01_JPRB, 0.20391E+01_JPRB, 0.22482E+01_JPRB ,&
&  0.24582E+01_JPRB, 0.25759E+01_JPRB, 0.27090E+01_JPRB, 0.27604E+01_JPRB, 0.26447E+01_JPRB, 0.24962E+01_JPRB, 0.25562E+01_JPRB ,&
&  0.21215E+01_JPRB, 0.17282E+01_JPRB, 0.17211E+01_JPRB, 0.17442E+01_JPRB, 0.17861E+01_JPRB, 0.17931E+01_JPRB, 0.18054E+01_JPRB ,&
&  0.18293E+01_JPRB, 0.18427E+01_JPRB, 0.18532E+01_JPRB, 0.18743E+01_JPRB, 0.19232E+01_JPRB, 0.20179E+01_JPRB, 0.21508E+01_JPRB ,&
&  0.21876E+01_JPRB, 0.22392E+01_JPRB, 0.23312E+01_JPRB, 0.21486E+01_JPRB, 0.22741E+01_JPRB, 0.24713E+01_JPRB, 0.25939E+01_JPRB ,&
&  0.25840E+01_JPRB, 0.24384E+01_JPRB, 0.24971E+01_JPRB, 0.25381E+01_JPRB, 0.25529E+01_JPRB, 0.25251E+01_JPRB, 0.25378E+01_JPRB ,&
&  0.26671E+01_JPRB, 0.25374E+01_JPRB, 0.19151E+01_JPRB, 0.18900E+01_JPRB, 0.20177E+01_JPRB, 0.17229E+01_JPRB, 0.18019E+01_JPRB ,&
&  0.16209E+01_JPRB, 0.17207E+01_JPRB, 0.21068E+01_JPRB, 0.24130E+01_JPRB, 0.25873E+01_JPRB, 0.29815E+01_JPRB, 0.25258E+01_JPRB ,&
&  0.18379E+01_JPRB, 0.14451E+01_JPRB, 0.12426E+01_JPRB, 0.11665E+01_JPRB, 0.12420E+01_JPRB, 0.12928E+01_JPRB, 0.12080E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.12324E+01_JPRB, 0.12791E+01_JPRB, 0.12813E+01_JPRB, 0.11911E+01_JPRB, 0.12035E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.12767E+01_JPRB, 0.13078E+01_JPRB, 0.13248E+01_JPRB, 0.13251E+01_JPRB, 0.13300E+01_JPRB, 0.13292E+01_JPRB, 0.13349E+01_JPRB ,&
&  0.13407E+01_JPRB, 0.13349E+01_JPRB, 0.13045E+01_JPRB, 0.12545E+01_JPRB, 0.11627E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.83947E+00_JPRB, 0.78155E+00_JPRB, 0.76605E+00_JPRB, 0.77559E+00_JPRB, 0.78650E+00_JPRB, 0.79901E+00_JPRB, 0.81283E+00_JPRB ,&
&  0.82081E+00_JPRB, 0.82622E+00_JPRB, 0.83690E+00_JPRB, 0.84244E+00_JPRB, 0.88143E+00_JPRB, 0.92641E+00_JPRB, 0.98485E+00_JPRB ,&
&  0.10732E+01_JPRB, 0.11530E+01_JPRB, 0.12226E+01_JPRB, 0.13422E+01_JPRB, 0.14233E+01_JPRB, 0.15021E+01_JPRB, 0.16334E+01_JPRB ,&
&  0.18035E+01_JPRB, 0.19962E+01_JPRB, 0.21828E+01_JPRB, 0.23780E+01_JPRB, 0.26330E+01_JPRB, 0.28738E+01_JPRB, 0.29097E+01_JPRB ,&
&  0.21954E+01_JPRB, 0.16275E+01_JPRB, 0.17454E+01_JPRB, 0.16523E+01_JPRB, 0.17075E+01_JPRB, 0.17474E+01_JPRB, 0.17551E+01_JPRB ,&
&  0.17792E+01_JPRB, 0.17909E+01_JPRB, 0.18010E+01_JPRB, 0.18155E+01_JPRB, 0.18737E+01_JPRB, 0.19552E+01_JPRB, 0.20630E+01_JPRB ,&
&  0.21146E+01_JPRB, 0.21573E+01_JPRB, 0.21690E+01_JPRB, 0.21519E+01_JPRB, 0.22482E+01_JPRB, 0.24610E+01_JPRB, 0.26186E+01_JPRB ,&
&  0.26980E+01_JPRB, 0.27550E+01_JPRB, 0.25214E+01_JPRB, 0.25266E+01_JPRB, 0.25299E+01_JPRB, 0.23531E+01_JPRB, 0.24325E+01_JPRB ,&
&  0.25483E+01_JPRB, 0.23124E+01_JPRB, 0.15859E+01_JPRB, 0.16119E+01_JPRB, 0.16630E+01_JPRB, 0.15977E+01_JPRB, 0.15579E+01_JPRB ,&
&  0.13048E+01_JPRB, 0.13343E+01_JPRB, 0.15246E+01_JPRB, 0.19477E+01_JPRB, 0.21802E+01_JPRB, 0.23079E+01_JPRB, 0.20619E+01_JPRB ,&
&  0.14791E+01_JPRB, 0.11895E+01_JPRB, 0.10293E+01_JPRB, 0.10273E+01_JPRB, 0.11598E+01_JPRB, 0.13385E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.11857E+01_JPRB, 0.13627E+01_JPRB, 0.16881E+01_JPRB, 0.14002E+01_JPRB, 0.10527E+01_JPRB, 0.10634E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.11070E+01_JPRB, 0.11155E+01_JPRB, 0.11197E+01_JPRB, 0.11237E+01_JPRB, 0.11376E+01_JPRB, 0.11400E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.10992E+01_JPRB, 0.10699E+01_JPRB, 0.99323E+00_JPRB, 0.91041E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.70674E+00_JPRB, 0.69739E+00_JPRB, 0.69997E+00_JPRB, 0.71531E+00_JPRB, 0.71446E+00_JPRB, 0.71433E+00_JPRB, 0.74452E+00_JPRB ,&
&  0.73067E+00_JPRB, 0.71930E+00_JPRB, 0.72103E+00_JPRB, 0.69673E+00_JPRB, 0.71938E+00_JPRB, 0.74224E+00_JPRB, 0.78644E+00_JPRB ,&
&  0.83227E+00_JPRB, 0.86116E+00_JPRB, 0.88497E+00_JPRB, 0.91923E+00_JPRB, 0.98597E+00_JPRB, 0.10363E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.12769E+01_JPRB, 0.14101E+01_JPRB, 0.17151E+01_JPRB, 0.24177E+01_JPRB, 0.29790E+01_JPRB, 0.29722E+01_JPRB ,&
&  0.22227E+01_JPRB, 0.15898E+01_JPRB, 0.15019E+01_JPRB, 0.13695E+01_JPRB, 0.14250E+01_JPRB, 0.15594E+01_JPRB, 0.15904E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.15724E+01_JPRB, 0.15693E+01_JPRB, 0.15718E+01_JPRB, 0.15852E+01_JPRB, 0.15923E+01_JPRB, 0.15717E+01_JPRB ,&
&  0.15119E+01_JPRB, 0.14375E+01_JPRB, 0.14679E+01_JPRB, 0.17336E+01_JPRB, 0.19551E+01_JPRB, 0.23261E+01_JPRB, 0.22789E+01_JPRB ,&
&  0.21762E+01_JPRB, 0.20773E+01_JPRB, 0.20551E+01_JPRB, 0.20346E+01_JPRB, 0.19896E+01_JPRB, 0.19368E+01_JPRB, 0.19872E+01_JPRB ,&
&  0.20624E+01_JPRB, 0.18983E+01_JPRB, 0.13470E+01_JPRB, 0.14591E+01_JPRB, 0.14896E+01_JPRB, 0.15335E+01_JPRB, 0.13437E+01_JPRB ,&
&  0.11103E+01_JPRB, 0.10694E+01_JPRB, 0.11384E+01_JPRB, 0.13106E+01_JPRB, 0.16036E+01_JPRB, 0.17318E+01_JPRB, 0.15659E+01_JPRB ,&
&  0.12561E+01_JPRB, 0.10377E+01_JPRB, 0.92105E+00_JPRB, 0.92342E+00_JPRB, 0.11544E+01_JPRB, 0.13198E+01_JPRB, 0.14766E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.11991E+01_JPRB, 0.10855E+01_JPRB, 0.13678E+01_JPRB, 0.91600E+00_JPRB, 0.84816E+00_JPRB, 0.84171E+00_JPRB ,&
&  0.85114E+00_JPRB, 0.84206E+00_JPRB, 0.83195E+00_JPRB, 0.81198E+00_JPRB, 0.79181E+00_JPRB, 0.84558E+00_JPRB, 0.89896E+00_JPRB ,&
&  0.85111E+00_JPRB, 0.87935E+00_JPRB, 0.84046E+00_JPRB, 0.77844E+00_JPRB, 0.74806E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.66487E+00_JPRB, 0.67255E+00_JPRB, 0.67308E+00_JPRB, 0.68080E+00_JPRB, 0.69800E+00_JPRB, 0.71101E+00_JPRB, 0.72890E+00_JPRB ,&
&  0.74814E+00_JPRB, 0.75724E+00_JPRB, 0.76640E+00_JPRB, 0.77861E+00_JPRB, 0.78763E+00_JPRB, 0.81129E+00_JPRB, 0.83414E+00_JPRB ,&
&  0.86378E+00_JPRB, 0.88728E+00_JPRB, 0.91208E+00_JPRB, 0.95039E+00_JPRB, 0.99859E+00_JPRB, 0.10710E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.12494E+01_JPRB, 0.13701E+01_JPRB, 0.15145E+01_JPRB, 0.16328E+01_JPRB, 0.16946E+01_JPRB, 0.17299E+01_JPRB, 0.19251E+01_JPRB ,&
&  0.16498E+01_JPRB, 0.16532E+01_JPRB, 0.15572E+01_JPRB, 0.13926E+01_JPRB, 0.13754E+01_JPRB, 0.12799E+01_JPRB, 0.12395E+01_JPRB ,&
&  0.12318E+01_JPRB, 0.11758E+01_JPRB, 0.11445E+01_JPRB, 0.11169E+01_JPRB, 0.10869E+01_JPRB, 0.10298E+01_JPRB, 0.96466E+00_JPRB ,&
&  0.94428E+00_JPRB, 0.98466E+00_JPRB, 0.10949E+01_JPRB, 0.13359E+01_JPRB, 0.15710E+01_JPRB, 0.16467E+01_JPRB, 0.15379E+01_JPRB ,&
&  0.14660E+01_JPRB, 0.14453E+01_JPRB, 0.13741E+01_JPRB, 0.13238E+01_JPRB, 0.13848E+01_JPRB, 0.14346E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.13781E+01_JPRB, 0.12541E+01_JPRB, 0.12249E+01_JPRB, 0.12796E+01_JPRB, 0.13133E+01_JPRB, 0.13127E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.93485E+00_JPRB, 0.89991E+00_JPRB, 0.95214E+00_JPRB, 0.11268E+01_JPRB, 0.11608E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.93121E+00_JPRB, 0.84252E+00_JPRB, 0.81751E+00_JPRB, 0.85257E+00_JPRB, 0.10171E+01_JPRB, 0.20969E+01_JPRB, 0.28354E+01_JPRB ,&
&  0.10976E+01_JPRB, 0.90695E+00_JPRB, 0.63422E+00_JPRB, 0.88565E+00_JPRB, 0.71359E+00_JPRB, 0.69458E+00_JPRB, 0.69462E+00_JPRB ,&
&  0.68811E+00_JPRB, 0.68681E+00_JPRB, 0.68583E+00_JPRB, 0.67156E+00_JPRB, 0.66316E+00_JPRB, 0.67079E+00_JPRB, 0.71067E+00_JPRB ,&
&  0.70839E+00_JPRB, 0.71205E+00_JPRB, 0.69218E+00_JPRB, 0.68973E+00_JPRB, 0.67521E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.65377E+00_JPRB, 0.66687E+00_JPRB, 0.68537E+00_JPRB, 0.70906E+00_JPRB, 0.72602E+00_JPRB, 0.74825E+00_JPRB, 0.76678E+00_JPRB ,&
&  0.79089E+00_JPRB, 0.81013E+00_JPRB, 0.82679E+00_JPRB, 0.84481E+00_JPRB, 0.86742E+00_JPRB, 0.89437E+00_JPRB, 0.92857E+00_JPRB ,&
&  0.96599E+00_JPRB, 0.10033E+01_JPRB, 0.10440E+01_JPRB, 0.10902E+01_JPRB, 0.11437E+01_JPRB, 0.12088E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.13291E+01_JPRB, 0.13892E+01_JPRB, 0.14514E+01_JPRB, 0.15006E+01_JPRB, 0.15183E+01_JPRB, 0.15566E+01_JPRB, 0.14022E+01_JPRB ,&
&  0.16338E+01_JPRB, 0.15498E+01_JPRB, 0.14406E+01_JPRB, 0.13134E+01_JPRB, 0.12046E+01_JPRB, 0.11053E+01_JPRB, 0.10157E+01_JPRB ,&
&  0.10166E+01_JPRB, 0.96034E+00_JPRB, 0.93744E+00_JPRB, 0.92593E+00_JPRB, 0.85762E+00_JPRB, 0.85349E+00_JPRB, 0.87718E+00_JPRB ,&
&  0.92954E+00_JPRB, 0.10277E+01_JPRB, 0.10845E+01_JPRB, 0.11528E+01_JPRB, 0.12478E+01_JPRB, 0.11943E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.99383E+00_JPRB, 0.81639E+00_JPRB, 0.89910E+00_JPRB, 0.94919E+00_JPRB, 0.98754E+00_JPRB, 0.10295E+01_JPRB ,&
&  0.95266E+00_JPRB, 0.84369E+00_JPRB, 0.91411E+00_JPRB, 0.11667E+01_JPRB, 0.11649E+01_JPRB, 0.11153E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.98544E+00_JPRB, 0.91594E+00_JPRB, 0.85107E+00_JPRB, 0.82478E+00_JPRB, 0.87107E+00_JPRB, 0.87105E+00_JPRB, 0.82265E+00_JPRB ,&
&  0.75991E+00_JPRB, 0.74468E+00_JPRB, 0.76167E+00_JPRB, 0.80633E+00_JPRB, 0.99216E+00_JPRB, 0.12070E+01_JPRB, 0.19086E+01_JPRB ,&
&  0.11245E+01_JPRB, 0.63902E+00_JPRB, 0.69856E+00_JPRB, 0.73135E+00_JPRB, 0.66332E+00_JPRB, 0.64643E+00_JPRB, 0.67456E+00_JPRB ,&
&  0.70436E+00_JPRB, 0.71538E+00_JPRB, 0.67270E+00_JPRB, 0.67147E+00_JPRB, 0.66720E+00_JPRB, 0.66397E+00_JPRB, 0.66309E+00_JPRB ,&
&  0.65702E+00_JPRB, 0.65441E+00_JPRB, 0.66083E+00_JPRB, 0.65390E+00_JPRB, 0.65003E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.57325E+00_JPRB, 0.58253E+00_JPRB, 0.59591E+00_JPRB, 0.60792E+00_JPRB, 0.61609E+00_JPRB, 0.62996E+00_JPRB, 0.64032E+00_JPRB ,&
&  0.65211E+00_JPRB, 0.65752E+00_JPRB, 0.66683E+00_JPRB, 0.67528E+00_JPRB, 0.68615E+00_JPRB, 0.70184E+00_JPRB, 0.71789E+00_JPRB ,&
&  0.73072E+00_JPRB, 0.75137E+00_JPRB, 0.77481E+00_JPRB, 0.79114E+00_JPRB, 0.82857E+00_JPRB, 0.89868E+00_JPRB, 0.97900E+00_JPRB ,&
&  0.10438E+01_JPRB, 0.11289E+01_JPRB, 0.12083E+01_JPRB, 0.12465E+01_JPRB, 0.12493E+01_JPRB, 0.13016E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.13986E+01_JPRB, 0.12688E+01_JPRB, 0.11915E+01_JPRB, 0.11204E+01_JPRB, 0.10517E+01_JPRB, 0.10082E+01_JPRB ,&
&  0.10128E+01_JPRB, 0.11666E+01_JPRB, 0.11429E+01_JPRB, 0.11683E+01_JPRB, 0.10347E+01_JPRB, 0.10127E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.12276E+01_JPRB, 0.12025E+01_JPRB, 0.12459E+01_JPRB, 0.12982E+01_JPRB, 0.13135E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.12128E+01_JPRB, 0.11233E+01_JPRB, 0.12752E+01_JPRB, 0.11856E+01_JPRB, 0.10865E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.82843E+00_JPRB, 0.81278E+00_JPRB, 0.87815E+00_JPRB, 0.99197E+00_JPRB, 0.98736E+00_JPRB, 0.99024E+00_JPRB, 0.96671E+00_JPRB ,&
&  0.93133E+00_JPRB, 0.90628E+00_JPRB, 0.85965E+00_JPRB, 0.84341E+00_JPRB, 0.84527E+00_JPRB, 0.81607E+00_JPRB, 0.75731E+00_JPRB ,&
&  0.72556E+00_JPRB, 0.74074E+00_JPRB, 0.74540E+00_JPRB, 0.76797E+00_JPRB, 0.86182E+00_JPRB, 0.88291E+00_JPRB, 0.12990E+01_JPRB ,&
&  0.96394E+00_JPRB, 0.82745E+00_JPRB, 0.10811E+01_JPRB, 0.49440E+00_JPRB, 0.59057E+00_JPRB, 0.69667E+00_JPRB, 0.88850E+00_JPRB ,&
&  0.93410E+00_JPRB, 0.11971E+01_JPRB, 0.15351E+01_JPRB, 0.12203E+01_JPRB, 0.93800E+00_JPRB, 0.64862E+00_JPRB, 0.61513E+00_JPRB ,&
&  0.60439E+00_JPRB, 0.59222E+00_JPRB, 0.59251E+00_JPRB, 0.58204E+00_JPRB, 0.57703E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.49664E+00_JPRB, 0.49945E+00_JPRB, 0.50883E+00_JPRB, 0.51539E+00_JPRB, 0.52228E+00_JPRB, 0.52790E+00_JPRB, 0.53298E+00_JPRB ,&
&  0.54808E+00_JPRB, 0.55727E+00_JPRB, 0.58040E+00_JPRB, 0.59935E+00_JPRB, 0.61661E+00_JPRB, 0.65186E+00_JPRB, 0.68191E+00_JPRB ,&
&  0.72936E+00_JPRB, 0.79120E+00_JPRB, 0.81836E+00_JPRB, 0.85495E+00_JPRB, 0.87654E+00_JPRB, 0.90073E+00_JPRB, 0.91881E+00_JPRB ,&
&  0.99730E+00_JPRB, 0.11489E+01_JPRB, 0.12523E+01_JPRB, 0.13614E+01_JPRB, 0.13657E+01_JPRB, 0.14465E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.13817E+01_JPRB, 0.13925E+01_JPRB, 0.13110E+01_JPRB, 0.12674E+01_JPRB, 0.12409E+01_JPRB, 0.12206E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.12389E+01_JPRB, 0.12968E+01_JPRB, 0.12647E+01_JPRB, 0.13047E+01_JPRB, 0.13465E+01_JPRB, 0.13122E+01_JPRB, 0.14325E+01_JPRB ,&
&  0.15209E+01_JPRB, 0.15896E+01_JPRB, 0.16809E+01_JPRB, 0.18008E+01_JPRB, 0.18854E+01_JPRB, 0.19370E+01_JPRB, 0.19488E+01_JPRB ,&
&  0.19450E+01_JPRB, 0.19883E+01_JPRB, 0.21252E+01_JPRB, 0.21004E+01_JPRB, 0.17500E+01_JPRB, 0.15010E+01_JPRB, 0.12362E+01_JPRB ,&
&  0.93055E+00_JPRB, 0.81112E+00_JPRB, 0.81838E+00_JPRB, 0.88614E+00_JPRB, 0.86928E+00_JPRB, 0.85821E+00_JPRB, 0.84662E+00_JPRB ,&
&  0.84102E+00_JPRB, 0.85598E+00_JPRB, 0.86020E+00_JPRB, 0.85812E+00_JPRB, 0.83330E+00_JPRB, 0.77649E+00_JPRB, 0.72051E+00_JPRB ,&
&  0.71865E+00_JPRB, 0.72161E+00_JPRB, 0.73603E+00_JPRB, 0.77688E+00_JPRB, 0.85613E+00_JPRB, 0.10140E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.73726E+00_JPRB, 0.70299E+00_JPRB, 0.73141E+00_JPRB, 0.65310E+00_JPRB, 0.89527E+00_JPRB, 0.13008E+01_JPRB ,&
&  0.19101E+01_JPRB, 0.21556E+01_JPRB, 0.28269E+01_JPRB, 0.41304E+01_JPRB, 0.26076E+01_JPRB, 0.16809E+01_JPRB, 0.57514E+00_JPRB ,&
&  0.56395E+00_JPRB, 0.53805E+00_JPRB, 0.52981E+00_JPRB, 0.51248E+00_JPRB, 0.50456E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.40953E+00_JPRB, 0.40308E+00_JPRB, 0.41379E+00_JPRB, 0.43150E+00_JPRB, 0.44530E+00_JPRB, 0.47615E+00_JPRB, 0.51315E+00_JPRB ,&
&  0.55357E+00_JPRB, 0.60413E+00_JPRB, 0.64790E+00_JPRB, 0.68005E+00_JPRB, 0.73348E+00_JPRB, 0.80802E+00_JPRB, 0.84355E+00_JPRB ,&
&  0.90997E+00_JPRB, 0.95471E+00_JPRB, 0.97960E+00_JPRB, 0.99594E+00_JPRB, 0.10295E+01_JPRB, 0.10827E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12809E+01_JPRB, 0.13839E+01_JPRB, 0.14498E+01_JPRB, 0.14804E+01_JPRB, 0.14832E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.14000E+01_JPRB, 0.14785E+01_JPRB, 0.14981E+01_JPRB, 0.14587E+01_JPRB, 0.14127E+01_JPRB, 0.13833E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.14234E+01_JPRB, 0.13864E+01_JPRB, 0.13739E+01_JPRB, 0.16391E+01_JPRB, 0.16636E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.19112E+01_JPRB, 0.20080E+01_JPRB, 0.21426E+01_JPRB, 0.22849E+01_JPRB, 0.24318E+01_JPRB, 0.25585E+01_JPRB, 0.26678E+01_JPRB ,&
&  0.27675E+01_JPRB, 0.28800E+01_JPRB, 0.29920E+01_JPRB, 0.25991E+01_JPRB, 0.21263E+01_JPRB, 0.18050E+01_JPRB, 0.12674E+01_JPRB ,&
&  0.91491E+00_JPRB, 0.82806E+00_JPRB, 0.85723E+00_JPRB, 0.87423E+00_JPRB, 0.81568E+00_JPRB, 0.77852E+00_JPRB, 0.75176E+00_JPRB ,&
&  0.76306E+00_JPRB, 0.79267E+00_JPRB, 0.81476E+00_JPRB, 0.81688E+00_JPRB, 0.78294E+00_JPRB, 0.73686E+00_JPRB, 0.70097E+00_JPRB ,&
&  0.72650E+00_JPRB, 0.73589E+00_JPRB, 0.74662E+00_JPRB, 0.81707E+00_JPRB, 0.91672E+00_JPRB, 0.11486E+01_JPRB, 0.14268E+01_JPRB ,&
&  0.13220E+01_JPRB, 0.75692E+00_JPRB, 0.69191E+00_JPRB, 0.67428E+00_JPRB, 0.67281E+00_JPRB, 0.74318E+00_JPRB, 0.90252E+00_JPRB ,&
&  0.12668E+01_JPRB, 0.20551E+01_JPRB, 0.22972E+01_JPRB, 0.15158E+01_JPRB, 0.20262E+01_JPRB, 0.13317E+01_JPRB, 0.55590E+00_JPRB ,&
&  0.52875E+00_JPRB, 0.51019E+00_JPRB, 0.48155E+00_JPRB, 0.44949E+00_JPRB, 0.42614E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.41336E+00_JPRB, 0.39496E+00_JPRB, 0.40563E+00_JPRB, 0.43018E+00_JPRB, 0.45951E+00_JPRB, 0.50441E+00_JPRB, 0.55923E+00_JPRB ,&
&  0.60786E+00_JPRB, 0.65643E+00_JPRB, 0.74005E+00_JPRB, 0.76263E+00_JPRB, 0.78372E+00_JPRB, 0.83133E+00_JPRB, 0.86624E+00_JPRB ,&
&  0.90779E+00_JPRB, 0.94362E+00_JPRB, 0.98289E+00_JPRB, 0.10210E+01_JPRB, 0.10757E+01_JPRB, 0.11357E+01_JPRB, 0.11861E+01_JPRB ,&
&  0.12545E+01_JPRB, 0.13279E+01_JPRB, 0.14317E+01_JPRB, 0.15399E+01_JPRB, 0.16395E+01_JPRB, 0.16527E+01_JPRB, 0.16843E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.15405E+01_JPRB, 0.15642E+01_JPRB, 0.15462E+01_JPRB, 0.14907E+01_JPRB, 0.14448E+01_JPRB, 0.14628E+01_JPRB ,&
&  0.14934E+01_JPRB, 0.14434E+01_JPRB, 0.14913E+01_JPRB, 0.15375E+01_JPRB, 0.18125E+01_JPRB, 0.18779E+01_JPRB, 0.19744E+01_JPRB ,&
&  0.20849E+01_JPRB, 0.21863E+01_JPRB, 0.22997E+01_JPRB, 0.24380E+01_JPRB, 0.25805E+01_JPRB, 0.27153E+01_JPRB, 0.28501E+01_JPRB ,&
&  0.29600E+01_JPRB, 0.30617E+01_JPRB, 0.31936E+01_JPRB, 0.26846E+01_JPRB, 0.22669E+01_JPRB, 0.18755E+01_JPRB, 0.13366E+01_JPRB ,&
&  0.97201E+00_JPRB, 0.82483E+00_JPRB, 0.85736E+00_JPRB, 0.90035E+00_JPRB, 0.82712E+00_JPRB, 0.77502E+00_JPRB, 0.71004E+00_JPRB ,&
&  0.70659E+00_JPRB, 0.72018E+00_JPRB, 0.72820E+00_JPRB, 0.72289E+00_JPRB, 0.72736E+00_JPRB, 0.69097E+00_JPRB, 0.67975E+00_JPRB ,&
&  0.70473E+00_JPRB, 0.70546E+00_JPRB, 0.72022E+00_JPRB, 0.75879E+00_JPRB, 0.82290E+00_JPRB, 0.87611E+00_JPRB, 0.87456E+00_JPRB ,&
&  0.86933E+00_JPRB, 0.77148E+00_JPRB, 0.72695E+00_JPRB, 0.73574E+00_JPRB, 0.71439E+00_JPRB, 0.69581E+00_JPRB, 0.71477E+00_JPRB ,&
&  0.79972E+00_JPRB, 0.90972E+00_JPRB, 0.99995E+00_JPRB, 0.97994E+00_JPRB, 0.10124E+01_JPRB, 0.80437E+00_JPRB, 0.60827E+00_JPRB ,&
&  0.57008E+00_JPRB, 0.55861E+00_JPRB, 0.51310E+00_JPRB, 0.48086E+00_JPRB, 0.44483E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.50984E+00_JPRB, 0.45073E+00_JPRB, 0.44617E+00_JPRB, 0.48853E+00_JPRB, 0.53188E+00_JPRB, 0.57687E+00_JPRB, 0.61843E+00_JPRB ,&
&  0.66241E+00_JPRB, 0.68778E+00_JPRB, 0.73699E+00_JPRB, 0.76311E+00_JPRB, 0.78366E+00_JPRB, 0.79339E+00_JPRB, 0.80692E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.88330E+00_JPRB, 0.92193E+00_JPRB, 0.97912E+00_JPRB, 0.10405E+01_JPRB, 0.11005E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.13209E+01_JPRB, 0.14028E+01_JPRB, 0.15002E+01_JPRB, 0.15802E+01_JPRB, 0.16105E+01_JPRB, 0.15600E+01_JPRB ,&
&  0.82227E+00_JPRB, 0.12812E+01_JPRB, 0.16073E+01_JPRB, 0.15711E+01_JPRB, 0.15172E+01_JPRB, 0.14538E+01_JPRB, 0.14585E+01_JPRB ,&
&  0.13946E+01_JPRB, 0.12964E+01_JPRB, 0.13602E+01_JPRB, 0.17388E+01_JPRB, 0.18509E+01_JPRB, 0.19385E+01_JPRB, 0.19980E+01_JPRB ,&
&  0.20736E+01_JPRB, 0.21305E+01_JPRB, 0.21970E+01_JPRB, 0.22689E+01_JPRB, 0.23583E+01_JPRB, 0.24430E+01_JPRB, 0.25158E+01_JPRB ,&
&  0.25679E+01_JPRB, 0.25904E+01_JPRB, 0.25793E+01_JPRB, 0.20527E+01_JPRB, 0.20089E+01_JPRB, 0.17431E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.86214E+00_JPRB, 0.86202E+00_JPRB, 0.90450E+00_JPRB, 0.82959E+00_JPRB, 0.77412E+00_JPRB, 0.69601E+00_JPRB ,&
&  0.69566E+00_JPRB, 0.69525E+00_JPRB, 0.69841E+00_JPRB, 0.68707E+00_JPRB, 0.68861E+00_JPRB, 0.65098E+00_JPRB, 0.67959E+00_JPRB ,&
&  0.70430E+00_JPRB, 0.70636E+00_JPRB, 0.72437E+00_JPRB, 0.76239E+00_JPRB, 0.81047E+00_JPRB, 0.85351E+00_JPRB, 0.86345E+00_JPRB ,&
&  0.85610E+00_JPRB, 0.83346E+00_JPRB, 0.82573E+00_JPRB, 0.81843E+00_JPRB, 0.80618E+00_JPRB, 0.79418E+00_JPRB, 0.77366E+00_JPRB ,&
&  0.75171E+00_JPRB, 0.76109E+00_JPRB, 0.79419E+00_JPRB, 0.73532E+00_JPRB, 0.77494E+00_JPRB, 0.70486E+00_JPRB, 0.68950E+00_JPRB ,&
&  0.67156E+00_JPRB, 0.65326E+00_JPRB, 0.60668E+00_JPRB, 0.58305E+00_JPRB, 0.55053E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.61502E+00_JPRB, 0.54427E+00_JPRB, 0.48868E+00_JPRB, 0.50383E+00_JPRB, 0.56828E+00_JPRB, 0.62267E+00_JPRB, 0.65148E+00_JPRB ,&
&  0.68569E+00_JPRB, 0.69582E+00_JPRB, 0.70357E+00_JPRB, 0.71468E+00_JPRB, 0.73983E+00_JPRB, 0.75045E+00_JPRB, 0.77896E+00_JPRB ,&
&  0.79810E+00_JPRB, 0.81285E+00_JPRB, 0.83266E+00_JPRB, 0.87954E+00_JPRB, 0.93884E+00_JPRB, 0.99961E+00_JPRB, 0.10590E+01_JPRB ,&
&  0.11288E+01_JPRB, 0.12119E+01_JPRB, 0.13282E+01_JPRB, 0.14679E+01_JPRB, 0.16599E+01_JPRB, 0.18724E+01_JPRB, 0.20333E+01_JPRB ,&
&  0.14474E+01_JPRB, 0.72535E+00_JPRB, 0.13679E+01_JPRB, 0.16321E+01_JPRB, 0.15236E+01_JPRB, 0.14602E+01_JPRB, 0.13827E+01_JPRB ,&
&  0.13105E+01_JPRB, 0.12537E+01_JPRB, 0.12490E+01_JPRB, 0.15235E+01_JPRB, 0.17824E+01_JPRB, 0.18801E+01_JPRB, 0.19179E+01_JPRB ,&
&  0.19517E+01_JPRB, 0.19660E+01_JPRB, 0.19914E+01_JPRB, 0.20323E+01_JPRB, 0.20831E+01_JPRB, 0.21194E+01_JPRB, 0.21398E+01_JPRB ,&
&  0.21535E+01_JPRB, 0.21332E+01_JPRB, 0.20544E+01_JPRB, 0.15322E+01_JPRB, 0.14148E+01_JPRB, 0.12843E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.92906E+00_JPRB, 0.88431E+00_JPRB, 0.89609E+00_JPRB, 0.83685E+00_JPRB, 0.74249E+00_JPRB, 0.71550E+00_JPRB ,&
&  0.71698E+00_JPRB, 0.69949E+00_JPRB, 0.67404E+00_JPRB, 0.65600E+00_JPRB, 0.68690E+00_JPRB, 0.69011E+00_JPRB, 0.69495E+00_JPRB ,&
&  0.69716E+00_JPRB, 0.73648E+00_JPRB, 0.77574E+00_JPRB, 0.81080E+00_JPRB, 0.84736E+00_JPRB, 0.88939E+00_JPRB, 0.89749E+00_JPRB ,&
&  0.91279E+00_JPRB, 0.90824E+00_JPRB, 0.90830E+00_JPRB, 0.88885E+00_JPRB, 0.86861E+00_JPRB, 0.83183E+00_JPRB, 0.82070E+00_JPRB ,&
&  0.81628E+00_JPRB, 0.81616E+00_JPRB, 0.80469E+00_JPRB, 0.79138E+00_JPRB, 0.80925E+00_JPRB, 0.80021E+00_JPRB, 0.80615E+00_JPRB ,&
&  0.80123E+00_JPRB, 0.77929E+00_JPRB, 0.72220E+00_JPRB, 0.67634E+00_JPRB, 0.65579E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.67248E+00_JPRB, 0.62744E+00_JPRB, 0.56563E+00_JPRB, 0.52480E+00_JPRB, 0.54563E+00_JPRB, 0.61181E+00_JPRB, 0.66487E+00_JPRB ,&
&  0.68485E+00_JPRB, 0.68423E+00_JPRB, 0.67496E+00_JPRB, 0.67381E+00_JPRB, 0.68760E+00_JPRB, 0.68497E+00_JPRB, 0.69943E+00_JPRB ,&
&  0.72320E+00_JPRB, 0.73891E+00_JPRB, 0.75572E+00_JPRB, 0.78194E+00_JPRB, 0.81288E+00_JPRB, 0.86668E+00_JPRB, 0.91929E+00_JPRB ,&
&  0.96182E+00_JPRB, 0.10057E+01_JPRB, 0.10766E+01_JPRB, 0.11786E+01_JPRB, 0.13269E+01_JPRB, 0.15896E+01_JPRB, 0.20488E+01_JPRB ,&
&  0.28862E+01_JPRB, 0.19568E+01_JPRB, 0.93585E+00_JPRB, 0.15745E+01_JPRB, 0.16545E+01_JPRB, 0.15581E+01_JPRB, 0.13595E+01_JPRB ,&
&  0.13393E+01_JPRB, 0.12294E+01_JPRB, 0.11608E+01_JPRB, 0.12701E+01_JPRB, 0.14139E+01_JPRB, 0.16108E+01_JPRB, 0.17318E+01_JPRB ,&
&  0.17576E+01_JPRB, 0.17682E+01_JPRB, 0.17788E+01_JPRB, 0.18050E+01_JPRB, 0.18498E+01_JPRB, 0.18988E+01_JPRB, 0.19107E+01_JPRB ,&
&  0.19231E+01_JPRB, 0.19449E+01_JPRB, 0.18879E+01_JPRB, 0.14551E+01_JPRB, 0.13736E+01_JPRB, 0.13387E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.11635E+01_JPRB, 0.10994E+01_JPRB, 0.10437E+01_JPRB, 0.93789E+00_JPRB, 0.84785E+00_JPRB, 0.79372E+00_JPRB, 0.80364E+00_JPRB ,&
&  0.78954E+00_JPRB, 0.74375E+00_JPRB, 0.71554E+00_JPRB, 0.69497E+00_JPRB, 0.71611E+00_JPRB, 0.74427E+00_JPRB, 0.74945E+00_JPRB ,&
&  0.77256E+00_JPRB, 0.80534E+00_JPRB, 0.82904E+00_JPRB, 0.85150E+00_JPRB, 0.88431E+00_JPRB, 0.91856E+00_JPRB, 0.94078E+00_JPRB ,&
&  0.96625E+00_JPRB, 0.97345E+00_JPRB, 0.97683E+00_JPRB, 0.96821E+00_JPRB, 0.92639E+00_JPRB, 0.88459E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.87691E+00_JPRB, 0.86837E+00_JPRB, 0.84001E+00_JPRB, 0.84944E+00_JPRB, 0.84569E+00_JPRB, 0.86227E+00_JPRB, 0.88883E+00_JPRB ,&
&  0.91367E+00_JPRB, 0.89402E+00_JPRB, 0.80955E+00_JPRB, 0.71634E+00_JPRB, 0.69252E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.70161E+00_JPRB, 0.68643E+00_JPRB, 0.65087E+00_JPRB, 0.58006E+00_JPRB, 0.55563E+00_JPRB, 0.57016E+00_JPRB, 0.63271E+00_JPRB ,&
&  0.65996E+00_JPRB, 0.66774E+00_JPRB, 0.65301E+00_JPRB, 0.63771E+00_JPRB, 0.63019E+00_JPRB, 0.63311E+00_JPRB, 0.65197E+00_JPRB ,&
&  0.67237E+00_JPRB, 0.66821E+00_JPRB, 0.66534E+00_JPRB, 0.67331E+00_JPRB, 0.69756E+00_JPRB, 0.73490E+00_JPRB, 0.78925E+00_JPRB ,&
&  0.83458E+00_JPRB, 0.85889E+00_JPRB, 0.88167E+00_JPRB, 0.94586E+00_JPRB, 0.10268E+01_JPRB, 0.10677E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.17552E+01_JPRB, 0.25929E+01_JPRB, 0.11046E+01_JPRB, 0.16602E+01_JPRB, 0.18997E+01_JPRB, 0.16934E+01_JPRB, 0.15375E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.12508E+01_JPRB, 0.10763E+01_JPRB, 0.10966E+01_JPRB, 0.11552E+01_JPRB, 0.12291E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.14456E+01_JPRB, 0.15123E+01_JPRB, 0.15673E+01_JPRB, 0.16095E+01_JPRB, 0.16656E+01_JPRB, 0.17500E+01_JPRB, 0.18249E+01_JPRB ,&
&  0.18767E+01_JPRB, 0.19554E+01_JPRB, 0.19833E+01_JPRB, 0.16144E+01_JPRB, 0.15493E+01_JPRB, 0.16412E+01_JPRB, 0.15703E+01_JPRB ,&
&  0.14038E+01_JPRB, 0.13139E+01_JPRB, 0.11890E+01_JPRB, 0.10278E+01_JPRB, 0.90611E+00_JPRB, 0.85905E+00_JPRB, 0.86044E+00_JPRB ,&
&  0.82533E+00_JPRB, 0.79417E+00_JPRB, 0.78556E+00_JPRB, 0.78660E+00_JPRB, 0.78774E+00_JPRB, 0.79407E+00_JPRB, 0.81129E+00_JPRB ,&
&  0.84437E+00_JPRB, 0.87439E+00_JPRB, 0.89477E+00_JPRB, 0.90762E+00_JPRB, 0.92470E+00_JPRB, 0.94621E+00_JPRB, 0.95401E+00_JPRB ,&
&  0.97585E+00_JPRB, 0.99127E+00_JPRB, 0.10067E+01_JPRB, 0.10044E+01_JPRB, 0.98158E+00_JPRB, 0.96571E+00_JPRB, 0.96218E+00_JPRB ,&
&  0.95883E+00_JPRB, 0.94326E+00_JPRB, 0.85294E+00_JPRB, 0.83508E+00_JPRB, 0.86434E+00_JPRB, 0.83832E+00_JPRB, 0.87567E+00_JPRB ,&
&  0.94008E+00_JPRB, 0.98380E+00_JPRB, 0.93132E+00_JPRB, 0.73543E+00_JPRB, 0.71062E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.72712E+00_JPRB, 0.71794E+00_JPRB, 0.70586E+00_JPRB, 0.65711E+00_JPRB, 0.61182E+00_JPRB, 0.59093E+00_JPRB, 0.60550E+00_JPRB ,&
&  0.64065E+00_JPRB, 0.66501E+00_JPRB, 0.66421E+00_JPRB, 0.65322E+00_JPRB, 0.65019E+00_JPRB, 0.65729E+00_JPRB, 0.66913E+00_JPRB ,&
&  0.67316E+00_JPRB, 0.66511E+00_JPRB, 0.65480E+00_JPRB, 0.65943E+00_JPRB, 0.66236E+00_JPRB, 0.67360E+00_JPRB, 0.70289E+00_JPRB ,&
&  0.74372E+00_JPRB, 0.76586E+00_JPRB, 0.77087E+00_JPRB, 0.78949E+00_JPRB, 0.84918E+00_JPRB, 0.86736E+00_JPRB, 0.89718E+00_JPRB ,&
&  0.11043E+01_JPRB, 0.20682E+01_JPRB, 0.22888E+01_JPRB, 0.26745E+01_JPRB, 0.23675E+01_JPRB, 0.19463E+01_JPRB, 0.18256E+01_JPRB ,&
&  0.17981E+01_JPRB, 0.15185E+01_JPRB, 0.13024E+01_JPRB, 0.10900E+01_JPRB, 0.10732E+01_JPRB, 0.10893E+01_JPRB, 0.10980E+01_JPRB ,&
&  0.11488E+01_JPRB, 0.12606E+01_JPRB, 0.13288E+01_JPRB, 0.13907E+01_JPRB, 0.14501E+01_JPRB, 0.15202E+01_JPRB, 0.16262E+01_JPRB ,&
&  0.17493E+01_JPRB, 0.19079E+01_JPRB, 0.20887E+01_JPRB, 0.19411E+01_JPRB, 0.17254E+01_JPRB, 0.19445E+01_JPRB, 0.20117E+01_JPRB ,&
&  0.19475E+01_JPRB, 0.16269E+01_JPRB, 0.13615E+01_JPRB, 0.11547E+01_JPRB, 0.99746E+00_JPRB, 0.10201E+01_JPRB, 0.94006E+00_JPRB ,&
&  0.87074E+00_JPRB, 0.85875E+00_JPRB, 0.84766E+00_JPRB, 0.85392E+00_JPRB, 0.85561E+00_JPRB, 0.84726E+00_JPRB, 0.85490E+00_JPRB ,&
&  0.87962E+00_JPRB, 0.90257E+00_JPRB, 0.91914E+00_JPRB, 0.93145E+00_JPRB, 0.94838E+00_JPRB, 0.96115E+00_JPRB, 0.95982E+00_JPRB ,&
&  0.95631E+00_JPRB, 0.97090E+00_JPRB, 0.94702E+00_JPRB, 0.96395E+00_JPRB, 0.96783E+00_JPRB, 0.96169E+00_JPRB, 0.92694E+00_JPRB ,&
&  0.92891E+00_JPRB, 0.94474E+00_JPRB, 0.90023E+00_JPRB, 0.87611E+00_JPRB, 0.88060E+00_JPRB, 0.86096E+00_JPRB, 0.80809E+00_JPRB ,&
&  0.81032E+00_JPRB, 0.79671E+00_JPRB, 0.77926E+00_JPRB, 0.75645E+00_JPRB, 0.73966E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.76818E+00_JPRB, 0.75821E+00_JPRB, 0.75325E+00_JPRB, 0.72866E+00_JPRB, 0.69884E+00_JPRB, 0.67447E+00_JPRB, 0.66324E+00_JPRB ,&
&  0.66482E+00_JPRB, 0.67722E+00_JPRB, 0.68767E+00_JPRB, 0.67672E+00_JPRB, 0.67186E+00_JPRB, 0.68275E+00_JPRB, 0.68814E+00_JPRB ,&
&  0.69077E+00_JPRB, 0.68834E+00_JPRB, 0.68518E+00_JPRB, 0.68423E+00_JPRB, 0.68023E+00_JPRB, 0.66836E+00_JPRB, 0.66844E+00_JPRB ,&
&  0.68615E+00_JPRB, 0.70309E+00_JPRB, 0.71080E+00_JPRB, 0.71396E+00_JPRB, 0.74627E+00_JPRB, 0.76674E+00_JPRB, 0.79631E+00_JPRB ,&
&  0.89588E+00_JPRB, 0.18160E+01_JPRB, 0.16896E+01_JPRB, 0.30680E+01_JPRB, 0.25872E+01_JPRB, 0.21567E+01_JPRB, 0.18287E+01_JPRB ,&
&  0.17504E+01_JPRB, 0.18248E+01_JPRB, 0.14902E+01_JPRB, 0.12452E+01_JPRB, 0.11131E+01_JPRB, 0.10759E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.10334E+01_JPRB, 0.10747E+01_JPRB, 0.11351E+01_JPRB, 0.12079E+01_JPRB, 0.12694E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.14453E+01_JPRB, 0.16036E+01_JPRB, 0.18489E+01_JPRB, 0.18647E+01_JPRB, 0.18688E+01_JPRB, 0.21276E+01_JPRB, 0.23965E+01_JPRB ,&
&  0.23863E+01_JPRB, 0.20557E+01_JPRB, 0.15896E+01_JPRB, 0.12879E+01_JPRB, 0.11360E+01_JPRB, 0.10947E+01_JPRB, 0.10133E+01_JPRB ,&
&  0.94405E+00_JPRB, 0.91830E+00_JPRB, 0.90839E+00_JPRB, 0.89627E+00_JPRB, 0.89112E+00_JPRB, 0.88613E+00_JPRB, 0.88209E+00_JPRB ,&
&  0.89093E+00_JPRB, 0.92279E+00_JPRB, 0.93291E+00_JPRB, 0.93617E+00_JPRB, 0.94219E+00_JPRB, 0.94535E+00_JPRB, 0.93636E+00_JPRB ,&
&  0.94096E+00_JPRB, 0.95508E+00_JPRB, 0.92409E+00_JPRB, 0.91804E+00_JPRB, 0.93126E+00_JPRB, 0.93184E+00_JPRB, 0.93612E+00_JPRB ,&
&  0.97967E+00_JPRB, 0.99429E+00_JPRB, 0.95768E+00_JPRB, 0.90497E+00_JPRB, 0.91506E+00_JPRB, 0.92690E+00_JPRB, 0.81804E+00_JPRB ,&
&  0.80385E+00_JPRB, 0.79488E+00_JPRB, 0.79401E+00_JPRB, 0.78951E+00_JPRB, 0.77908E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.78287E+00_JPRB, 0.76967E+00_JPRB, 0.76560E+00_JPRB, 0.75258E+00_JPRB, 0.73004E+00_JPRB, 0.70868E+00_JPRB, 0.69978E+00_JPRB ,&
&  0.69367E+00_JPRB, 0.69658E+00_JPRB, 0.70507E+00_JPRB, 0.69694E+00_JPRB, 0.69823E+00_JPRB, 0.70557E+00_JPRB, 0.70822E+00_JPRB ,&
&  0.70209E+00_JPRB, 0.69866E+00_JPRB, 0.69249E+00_JPRB, 0.68817E+00_JPRB, 0.68577E+00_JPRB, 0.67470E+00_JPRB, 0.66835E+00_JPRB ,&
&  0.66835E+00_JPRB, 0.67588E+00_JPRB, 0.68291E+00_JPRB, 0.68673E+00_JPRB, 0.70474E+00_JPRB, 0.71870E+00_JPRB, 0.73905E+00_JPRB ,&
&  0.83931E+00_JPRB, 0.19365E+01_JPRB, 0.15602E+01_JPRB, 0.28042E+01_JPRB, 0.24564E+01_JPRB, 0.20206E+01_JPRB, 0.16774E+01_JPRB ,&
&  0.16460E+01_JPRB, 0.16592E+01_JPRB, 0.15032E+01_JPRB, 0.13703E+01_JPRB, 0.12524E+01_JPRB, 0.11408E+01_JPRB, 0.10565E+01_JPRB ,&
&  0.99102E+00_JPRB, 0.95989E+00_JPRB, 0.96014E+00_JPRB, 0.97751E+00_JPRB, 0.10197E+01_JPRB, 0.10709E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.12652E+01_JPRB, 0.14374E+01_JPRB, 0.17159E+01_JPRB, 0.17473E+01_JPRB, 0.19547E+01_JPRB, 0.27149E+01_JPRB ,&
&  0.28207E+01_JPRB, 0.25679E+01_JPRB, 0.18964E+01_JPRB, 0.14244E+01_JPRB, 0.12395E+01_JPRB, 0.11620E+01_JPRB, 0.10882E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.97547E+00_JPRB, 0.97266E+00_JPRB, 0.94074E+00_JPRB, 0.91098E+00_JPRB, 0.91150E+00_JPRB, 0.92481E+00_JPRB ,&
&  0.93522E+00_JPRB, 0.92561E+00_JPRB, 0.93966E+00_JPRB, 0.93599E+00_JPRB, 0.92482E+00_JPRB, 0.90767E+00_JPRB, 0.89871E+00_JPRB ,&
&  0.90467E+00_JPRB, 0.92982E+00_JPRB, 0.92488E+00_JPRB, 0.90957E+00_JPRB, 0.91539E+00_JPRB, 0.92821E+00_JPRB, 0.93845E+00_JPRB ,&
&  0.10167E+01_JPRB, 0.10345E+01_JPRB, 0.10077E+01_JPRB, 0.99269E+00_JPRB, 0.97285E+00_JPRB, 0.99123E+00_JPRB, 0.83013E+00_JPRB ,&
&  0.80329E+00_JPRB, 0.79519E+00_JPRB, 0.80037E+00_JPRB, 0.80856E+00_JPRB, 0.80362E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.81060E+00_JPRB, 0.78856E+00_JPRB, 0.78072E+00_JPRB, 0.77244E+00_JPRB, 0.75826E+00_JPRB, 0.74278E+00_JPRB, 0.74110E+00_JPRB ,&
&  0.74198E+00_JPRB, 0.74249E+00_JPRB, 0.73321E+00_JPRB, 0.71714E+00_JPRB, 0.71508E+00_JPRB, 0.71305E+00_JPRB, 0.70910E+00_JPRB ,&
&  0.70445E+00_JPRB, 0.70302E+00_JPRB, 0.69286E+00_JPRB, 0.68696E+00_JPRB, 0.68814E+00_JPRB, 0.68418E+00_JPRB, 0.67423E+00_JPRB ,&
&  0.66396E+00_JPRB, 0.65969E+00_JPRB, 0.66699E+00_JPRB, 0.67488E+00_JPRB, 0.69299E+00_JPRB, 0.69969E+00_JPRB, 0.71540E+00_JPRB ,&
&  0.84035E+00_JPRB, 0.80897E+00_JPRB, 0.18468E+01_JPRB, 0.24116E+01_JPRB, 0.23329E+01_JPRB, 0.19914E+01_JPRB, 0.17552E+01_JPRB ,&
&  0.16731E+01_JPRB, 0.15606E+01_JPRB, 0.14767E+01_JPRB, 0.13592E+01_JPRB, 0.12369E+01_JPRB, 0.11057E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.99572E+00_JPRB, 0.95755E+00_JPRB, 0.94508E+00_JPRB, 0.94414E+00_JPRB, 0.95238E+00_JPRB, 0.96081E+00_JPRB, 0.98545E+00_JPRB ,&
&  0.10104E+01_JPRB, 0.10432E+01_JPRB, 0.11146E+01_JPRB, 0.13249E+01_JPRB, 0.16204E+01_JPRB, 0.18771E+01_JPRB, 0.24403E+01_JPRB ,&
&  0.30214E+01_JPRB, 0.29096E+01_JPRB, 0.21767E+01_JPRB, 0.15936E+01_JPRB, 0.13297E+01_JPRB, 0.12256E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.10205E+01_JPRB, 0.99264E+00_JPRB, 0.96600E+00_JPRB, 0.93705E+00_JPRB, 0.91539E+00_JPRB, 0.93277E+00_JPRB ,&
&  0.95338E+00_JPRB, 0.95423E+00_JPRB, 0.93410E+00_JPRB, 0.91890E+00_JPRB, 0.90501E+00_JPRB, 0.88643E+00_JPRB, 0.88029E+00_JPRB ,&
&  0.88314E+00_JPRB, 0.90744E+00_JPRB, 0.92307E+00_JPRB, 0.91307E+00_JPRB, 0.91912E+00_JPRB, 0.94604E+00_JPRB, 0.98053E+00_JPRB ,&
&  0.10315E+01_JPRB, 0.10494E+01_JPRB, 0.10218E+01_JPRB, 0.10306E+01_JPRB, 0.98693E+00_JPRB, 0.99158E+00_JPRB, 0.86049E+00_JPRB ,&
&  0.81587E+00_JPRB, 0.80751E+00_JPRB, 0.82013E+00_JPRB, 0.82957E+00_JPRB, 0.84077E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.83597E+00_JPRB, 0.80277E+00_JPRB, 0.78963E+00_JPRB, 0.78799E+00_JPRB, 0.78077E+00_JPRB, 0.78222E+00_JPRB, 0.78484E+00_JPRB ,&
&  0.78003E+00_JPRB, 0.76256E+00_JPRB, 0.74718E+00_JPRB, 0.73054E+00_JPRB, 0.72452E+00_JPRB, 0.71665E+00_JPRB, 0.71450E+00_JPRB ,&
&  0.70638E+00_JPRB, 0.70157E+00_JPRB, 0.69323E+00_JPRB, 0.69004E+00_JPRB, 0.68900E+00_JPRB, 0.68263E+00_JPRB, 0.66105E+00_JPRB ,&
&  0.65445E+00_JPRB, 0.65384E+00_JPRB, 0.66373E+00_JPRB, 0.67667E+00_JPRB, 0.69303E+00_JPRB, 0.70014E+00_JPRB, 0.71004E+00_JPRB ,&
&  0.82285E+00_JPRB, 0.11662E+01_JPRB, 0.18355E+01_JPRB, 0.19592E+01_JPRB, 0.19593E+01_JPRB, 0.17867E+01_JPRB, 0.16423E+01_JPRB ,&
&  0.15141E+01_JPRB, 0.14578E+01_JPRB, 0.14475E+01_JPRB, 0.13263E+01_JPRB, 0.11512E+01_JPRB, 0.10138E+01_JPRB, 0.96320E+00_JPRB ,&
&  0.93475E+00_JPRB, 0.91131E+00_JPRB, 0.89986E+00_JPRB, 0.89916E+00_JPRB, 0.89700E+00_JPRB, 0.89694E+00_JPRB, 0.90688E+00_JPRB ,&
&  0.91642E+00_JPRB, 0.93150E+00_JPRB, 0.95666E+00_JPRB, 0.10621E+01_JPRB, 0.14416E+01_JPRB, 0.15516E+01_JPRB, 0.22002E+01_JPRB ,&
&  0.31245E+01_JPRB, 0.28382E+01_JPRB, 0.22739E+01_JPRB, 0.17635E+01_JPRB, 0.14696E+01_JPRB, 0.13261E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.11352E+01_JPRB, 0.10646E+01_JPRB, 0.10092E+01_JPRB, 0.98484E+00_JPRB, 0.95922E+00_JPRB, 0.93386E+00_JPRB, 0.92619E+00_JPRB ,&
&  0.93871E+00_JPRB, 0.94723E+00_JPRB, 0.93489E+00_JPRB, 0.90950E+00_JPRB, 0.89048E+00_JPRB, 0.88767E+00_JPRB, 0.88530E+00_JPRB ,&
&  0.88932E+00_JPRB, 0.90122E+00_JPRB, 0.93902E+00_JPRB, 0.89946E+00_JPRB, 0.90843E+00_JPRB, 0.94561E+00_JPRB, 0.96820E+00_JPRB ,&
&  0.97350E+00_JPRB, 0.10024E+01_JPRB, 0.10333E+01_JPRB, 0.10330E+01_JPRB, 0.11147E+01_JPRB, 0.10802E+01_JPRB, 0.92303E+00_JPRB ,&
&  0.83702E+00_JPRB, 0.81253E+00_JPRB, 0.83132E+00_JPRB, 0.85822E+00_JPRB, 0.88995E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.87427E+00_JPRB, 0.82096E+00_JPRB, 0.79202E+00_JPRB, 0.78801E+00_JPRB, 0.79283E+00_JPRB, 0.78832E+00_JPRB, 0.79072E+00_JPRB ,&
&  0.78255E+00_JPRB, 0.76551E+00_JPRB, 0.74668E+00_JPRB, 0.73036E+00_JPRB, 0.72222E+00_JPRB, 0.71579E+00_JPRB, 0.71070E+00_JPRB ,&
&  0.70659E+00_JPRB, 0.70253E+00_JPRB, 0.69535E+00_JPRB, 0.69733E+00_JPRB, 0.69827E+00_JPRB, 0.69517E+00_JPRB, 0.67139E+00_JPRB ,&
&  0.65880E+00_JPRB, 0.66000E+00_JPRB, 0.66692E+00_JPRB, 0.67311E+00_JPRB, 0.68320E+00_JPRB, 0.69562E+00_JPRB, 0.71077E+00_JPRB ,&
&  0.76945E+00_JPRB, 0.10882E+01_JPRB, 0.14589E+01_JPRB, 0.17002E+01_JPRB, 0.17722E+01_JPRB, 0.17043E+01_JPRB, 0.15657E+01_JPRB ,&
&  0.14560E+01_JPRB, 0.13918E+01_JPRB, 0.13315E+01_JPRB, 0.11842E+01_JPRB, 0.10319E+01_JPRB, 0.94941E+00_JPRB, 0.90796E+00_JPRB ,&
&  0.88916E+00_JPRB, 0.88267E+00_JPRB, 0.87444E+00_JPRB, 0.86771E+00_JPRB, 0.86112E+00_JPRB, 0.86068E+00_JPRB, 0.86554E+00_JPRB ,&
&  0.87216E+00_JPRB, 0.87440E+00_JPRB, 0.88618E+00_JPRB, 0.90007E+00_JPRB, 0.11211E+01_JPRB, 0.13940E+01_JPRB, 0.18261E+01_JPRB ,&
&  0.20194E+01_JPRB, 0.20624E+01_JPRB, 0.19150E+01_JPRB, 0.17149E+01_JPRB, 0.15353E+01_JPRB, 0.13894E+01_JPRB, 0.12660E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.10697E+01_JPRB, 0.99384E+00_JPRB, 0.95036E+00_JPRB, 0.92828E+00_JPRB, 0.90352E+00_JPRB, 0.89091E+00_JPRB ,&
&  0.89258E+00_JPRB, 0.89908E+00_JPRB, 0.89925E+00_JPRB, 0.89151E+00_JPRB, 0.87635E+00_JPRB, 0.86502E+00_JPRB, 0.85981E+00_JPRB ,&
&  0.86721E+00_JPRB, 0.87038E+00_JPRB, 0.89997E+00_JPRB, 0.89795E+00_JPRB, 0.89365E+00_JPRB, 0.89544E+00_JPRB, 0.91111E+00_JPRB ,&
&  0.94387E+00_JPRB, 0.99701E+00_JPRB, 0.99262E+00_JPRB, 0.10338E+01_JPRB, 0.10896E+01_JPRB, 0.10784E+01_JPRB, 0.95710E+00_JPRB ,&
&  0.87072E+00_JPRB, 0.83076E+00_JPRB, 0.82465E+00_JPRB, 0.84438E+00_JPRB, 0.88843E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.90398E+00_JPRB, 0.86252E+00_JPRB, 0.82273E+00_JPRB, 0.79862E+00_JPRB, 0.78831E+00_JPRB, 0.78391E+00_JPRB, 0.78068E+00_JPRB ,&
&  0.77530E+00_JPRB, 0.76710E+00_JPRB, 0.74680E+00_JPRB, 0.73126E+00_JPRB, 0.72042E+00_JPRB, 0.71625E+00_JPRB, 0.71278E+00_JPRB ,&
&  0.71152E+00_JPRB, 0.70898E+00_JPRB, 0.70246E+00_JPRB, 0.69995E+00_JPRB, 0.70039E+00_JPRB, 0.69413E+00_JPRB, 0.67562E+00_JPRB ,&
&  0.65649E+00_JPRB, 0.65695E+00_JPRB, 0.66318E+00_JPRB, 0.66834E+00_JPRB, 0.67752E+00_JPRB, 0.68802E+00_JPRB, 0.70661E+00_JPRB ,&
&  0.76164E+00_JPRB, 0.90943E+00_JPRB, 0.12496E+01_JPRB, 0.14455E+01_JPRB, 0.15345E+01_JPRB, 0.15546E+01_JPRB, 0.14707E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.12427E+01_JPRB, 0.11450E+01_JPRB, 0.10411E+01_JPRB, 0.96932E+00_JPRB, 0.91656E+00_JPRB, 0.88140E+00_JPRB ,&
&  0.86349E+00_JPRB, 0.85665E+00_JPRB, 0.85001E+00_JPRB, 0.84098E+00_JPRB, 0.83455E+00_JPRB, 0.83116E+00_JPRB, 0.83074E+00_JPRB ,&
&  0.83396E+00_JPRB, 0.84198E+00_JPRB, 0.85070E+00_JPRB, 0.86014E+00_JPRB, 0.88795E+00_JPRB, 0.10014E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.14485E+01_JPRB, 0.15708E+01_JPRB, 0.15461E+01_JPRB, 0.14990E+01_JPRB, 0.14540E+01_JPRB, 0.13621E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.10830E+01_JPRB, 0.99127E+00_JPRB, 0.91864E+00_JPRB, 0.87323E+00_JPRB, 0.84798E+00_JPRB, 0.83506E+00_JPRB ,&
&  0.83495E+00_JPRB, 0.84392E+00_JPRB, 0.84610E+00_JPRB, 0.85040E+00_JPRB, 0.84571E+00_JPRB, 0.83618E+00_JPRB, 0.82742E+00_JPRB ,&
&  0.81936E+00_JPRB, 0.81259E+00_JPRB, 0.82399E+00_JPRB, 0.82917E+00_JPRB, 0.83095E+00_JPRB, 0.83927E+00_JPRB, 0.85193E+00_JPRB ,&
&  0.86950E+00_JPRB, 0.91376E+00_JPRB, 0.90842E+00_JPRB, 0.10153E+01_JPRB, 0.11029E+01_JPRB, 0.10342E+01_JPRB, 0.96310E+00_JPRB ,&
&  0.90104E+00_JPRB, 0.85720E+00_JPRB, 0.83834E+00_JPRB, 0.85561E+00_JPRB, 0.89860E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.80799E+00_JPRB, 0.79989E+00_JPRB, 0.79098E+00_JPRB, 0.78719E+00_JPRB, 0.78304E+00_JPRB, 0.77114E+00_JPRB, 0.76059E+00_JPRB ,&
&  0.75618E+00_JPRB, 0.74976E+00_JPRB, 0.74249E+00_JPRB, 0.73254E+00_JPRB, 0.71991E+00_JPRB, 0.71059E+00_JPRB, 0.70855E+00_JPRB ,&
&  0.70408E+00_JPRB, 0.70157E+00_JPRB, 0.69988E+00_JPRB, 0.69421E+00_JPRB, 0.68742E+00_JPRB, 0.67760E+00_JPRB, 0.66152E+00_JPRB ,&
&  0.64122E+00_JPRB, 0.63423E+00_JPRB, 0.64298E+00_JPRB, 0.65125E+00_JPRB, 0.65742E+00_JPRB, 0.66071E+00_JPRB, 0.67492E+00_JPRB ,&
&  0.73965E+00_JPRB, 0.81732E+00_JPRB, 0.95857E+00_JPRB, 0.11473E+01_JPRB, 0.12588E+01_JPRB, 0.12367E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.11108E+01_JPRB, 0.10524E+01_JPRB, 0.98551E+00_JPRB, 0.92767E+00_JPRB, 0.88550E+00_JPRB, 0.86299E+00_JPRB ,&
&  0.84814E+00_JPRB, 0.83509E+00_JPRB, 0.82670E+00_JPRB, 0.81355E+00_JPRB, 0.80260E+00_JPRB, 0.79435E+00_JPRB, 0.79444E+00_JPRB ,&
&  0.79733E+00_JPRB, 0.80251E+00_JPRB, 0.80820E+00_JPRB, 0.81567E+00_JPRB, 0.82919E+00_JPRB, 0.86638E+00_JPRB, 0.97354E+00_JPRB ,&
&  0.11112E+01_JPRB, 0.12422E+01_JPRB, 0.12993E+01_JPRB, 0.12781E+01_JPRB, 0.12517E+01_JPRB, 0.12191E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.10532E+01_JPRB, 0.99037E+00_JPRB, 0.94069E+00_JPRB, 0.89769E+00_JPRB, 0.86681E+00_JPRB, 0.84423E+00_JPRB, 0.82744E+00_JPRB ,&
&  0.81725E+00_JPRB, 0.81300E+00_JPRB, 0.80954E+00_JPRB, 0.80699E+00_JPRB, 0.80224E+00_JPRB, 0.78842E+00_JPRB, 0.76881E+00_JPRB ,&
&  0.74758E+00_JPRB, 0.74304E+00_JPRB, 0.75339E+00_JPRB, 0.75754E+00_JPRB, 0.76110E+00_JPRB, 0.77381E+00_JPRB, 0.79545E+00_JPRB ,&
&  0.81209E+00_JPRB, 0.84150E+00_JPRB, 0.85980E+00_JPRB, 0.95575E+00_JPRB, 0.97586E+00_JPRB, 0.95359E+00_JPRB, 0.94118E+00_JPRB ,&
&  0.91244E+00_JPRB, 0.87069E+00_JPRB, 0.83364E+00_JPRB, 0.82856E+00_JPRB, 0.77425E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.78879E+00_JPRB, 0.78123E+00_JPRB, 0.77087E+00_JPRB, 0.76682E+00_JPRB, 0.76558E+00_JPRB, 0.75106E+00_JPRB, 0.73795E+00_JPRB ,&
&  0.72885E+00_JPRB, 0.72660E+00_JPRB, 0.72755E+00_JPRB, 0.72650E+00_JPRB, 0.71928E+00_JPRB, 0.70695E+00_JPRB, 0.69961E+00_JPRB ,&
&  0.68923E+00_JPRB, 0.68015E+00_JPRB, 0.67727E+00_JPRB, 0.67590E+00_JPRB, 0.66963E+00_JPRB, 0.66147E+00_JPRB, 0.65421E+00_JPRB ,&
&  0.63899E+00_JPRB, 0.62833E+00_JPRB, 0.62582E+00_JPRB, 0.62919E+00_JPRB, 0.63345E+00_JPRB, 0.63294E+00_JPRB, 0.65432E+00_JPRB ,&
&  0.73141E+00_JPRB, 0.71002E+00_JPRB, 0.71765E+00_JPRB, 0.86409E+00_JPRB, 0.97440E+00_JPRB, 0.99442E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10375E+01_JPRB, 0.10078E+01_JPRB, 0.94511E+00_JPRB, 0.89450E+00_JPRB, 0.85685E+00_JPRB, 0.82334E+00_JPRB ,&
&  0.79641E+00_JPRB, 0.77355E+00_JPRB, 0.76078E+00_JPRB, 0.74762E+00_JPRB, 0.74112E+00_JPRB, 0.72928E+00_JPRB, 0.72478E+00_JPRB ,&
&  0.72558E+00_JPRB, 0.73315E+00_JPRB, 0.74218E+00_JPRB, 0.74993E+00_JPRB, 0.76552E+00_JPRB, 0.79236E+00_JPRB, 0.84582E+00_JPRB ,&
&  0.92724E+00_JPRB, 0.10341E+01_JPRB, 0.11039E+01_JPRB, 0.11207E+01_JPRB, 0.11020E+01_JPRB, 0.10648E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.94749E+00_JPRB, 0.89308E+00_JPRB, 0.85405E+00_JPRB, 0.83226E+00_JPRB, 0.81687E+00_JPRB, 0.80489E+00_JPRB, 0.79321E+00_JPRB ,&
&  0.78296E+00_JPRB, 0.77741E+00_JPRB, 0.77424E+00_JPRB, 0.76919E+00_JPRB, 0.76193E+00_JPRB, 0.74402E+00_JPRB, 0.71779E+00_JPRB ,&
&  0.69814E+00_JPRB, 0.68881E+00_JPRB, 0.69284E+00_JPRB, 0.69710E+00_JPRB, 0.70404E+00_JPRB, 0.72357E+00_JPRB, 0.74676E+00_JPRB ,&
&  0.76522E+00_JPRB, 0.78998E+00_JPRB, 0.81656E+00_JPRB, 0.87029E+00_JPRB, 0.92215E+00_JPRB, 0.91517E+00_JPRB, 0.91462E+00_JPRB ,&
&  0.90348E+00_JPRB, 0.87715E+00_JPRB, 0.84925E+00_JPRB, 0.71376E+00_JPRB, 0.77508E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.77577E+00_JPRB, 0.76847E+00_JPRB, 0.75161E+00_JPRB, 0.74019E+00_JPRB, 0.73160E+00_JPRB, 0.72167E+00_JPRB, 0.70786E+00_JPRB ,&
&  0.69770E+00_JPRB, 0.69301E+00_JPRB, 0.68962E+00_JPRB, 0.68866E+00_JPRB, 0.68806E+00_JPRB, 0.68166E+00_JPRB, 0.67210E+00_JPRB ,&
&  0.66248E+00_JPRB, 0.65380E+00_JPRB, 0.65456E+00_JPRB, 0.65335E+00_JPRB, 0.64776E+00_JPRB, 0.64309E+00_JPRB, 0.63699E+00_JPRB ,&
&  0.62192E+00_JPRB, 0.60612E+00_JPRB, 0.59381E+00_JPRB, 0.59642E+00_JPRB, 0.60295E+00_JPRB, 0.59829E+00_JPRB, 0.61705E+00_JPRB ,&
&  0.69418E+00_JPRB, 0.63518E+00_JPRB, 0.66464E+00_JPRB, 0.74672E+00_JPRB, 0.79264E+00_JPRB, 0.83709E+00_JPRB, 0.86953E+00_JPRB ,&
&  0.89469E+00_JPRB, 0.91080E+00_JPRB, 0.86758E+00_JPRB, 0.81528E+00_JPRB, 0.76643E+00_JPRB, 0.72578E+00_JPRB, 0.69679E+00_JPRB ,&
&  0.68007E+00_JPRB, 0.67529E+00_JPRB, 0.67259E+00_JPRB, 0.66545E+00_JPRB, 0.65963E+00_JPRB, 0.65278E+00_JPRB, 0.64885E+00_JPRB ,&
&  0.64728E+00_JPRB, 0.64992E+00_JPRB, 0.65608E+00_JPRB, 0.66987E+00_JPRB, 0.68508E+00_JPRB, 0.70275E+00_JPRB, 0.72868E+00_JPRB ,&
&  0.76853E+00_JPRB, 0.82732E+00_JPRB, 0.88812E+00_JPRB, 0.91807E+00_JPRB, 0.91508E+00_JPRB, 0.89170E+00_JPRB, 0.85517E+00_JPRB ,&
&  0.81599E+00_JPRB, 0.78277E+00_JPRB, 0.76112E+00_JPRB, 0.74712E+00_JPRB, 0.73323E+00_JPRB, 0.72102E+00_JPRB, 0.71122E+00_JPRB ,&
&  0.70318E+00_JPRB, 0.69418E+00_JPRB, 0.69110E+00_JPRB, 0.68697E+00_JPRB, 0.67983E+00_JPRB, 0.66685E+00_JPRB, 0.65382E+00_JPRB ,&
&  0.64215E+00_JPRB, 0.63355E+00_JPRB, 0.63141E+00_JPRB, 0.63682E+00_JPRB, 0.64574E+00_JPRB, 0.66325E+00_JPRB, 0.68651E+00_JPRB ,&
&  0.70555E+00_JPRB, 0.72576E+00_JPRB, 0.74307E+00_JPRB, 0.80183E+00_JPRB, 0.83971E+00_JPRB, 0.86121E+00_JPRB, 0.87146E+00_JPRB ,&
&  0.87319E+00_JPRB, 0.87432E+00_JPRB, 0.68265E+00_JPRB, 0.75345E+00_JPRB, 0.76938E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.77837E+00_JPRB, 0.75623E+00_JPRB, 0.72914E+00_JPRB, 0.70769E+00_JPRB, 0.69328E+00_JPRB, 0.68484E+00_JPRB, 0.67692E+00_JPRB ,&
&  0.66839E+00_JPRB, 0.66629E+00_JPRB, 0.66578E+00_JPRB, 0.66547E+00_JPRB, 0.66493E+00_JPRB, 0.66055E+00_JPRB, 0.65162E+00_JPRB ,&
&  0.64165E+00_JPRB, 0.62937E+00_JPRB, 0.62700E+00_JPRB, 0.62592E+00_JPRB, 0.61935E+00_JPRB, 0.61471E+00_JPRB, 0.61396E+00_JPRB ,&
&  0.60722E+00_JPRB, 0.59655E+00_JPRB, 0.57751E+00_JPRB, 0.57126E+00_JPRB, 0.57379E+00_JPRB, 0.57379E+00_JPRB, 0.58193E+00_JPRB ,&
&  0.57497E+00_JPRB, 0.56237E+00_JPRB, 0.62066E+00_JPRB, 0.65965E+00_JPRB, 0.68005E+00_JPRB, 0.69989E+00_JPRB, 0.72184E+00_JPRB ,&
&  0.71708E+00_JPRB, 0.71853E+00_JPRB, 0.69640E+00_JPRB, 0.66144E+00_JPRB, 0.63703E+00_JPRB, 0.62310E+00_JPRB, 0.61612E+00_JPRB ,&
&  0.61763E+00_JPRB, 0.61885E+00_JPRB, 0.61765E+00_JPRB, 0.61579E+00_JPRB, 0.61566E+00_JPRB, 0.61170E+00_JPRB, 0.60935E+00_JPRB ,&
&  0.61093E+00_JPRB, 0.61479E+00_JPRB, 0.61712E+00_JPRB, 0.62203E+00_JPRB, 0.63222E+00_JPRB, 0.64774E+00_JPRB, 0.66456E+00_JPRB ,&
&  0.68161E+00_JPRB, 0.70221E+00_JPRB, 0.73332E+00_JPRB, 0.75778E+00_JPRB, 0.76417E+00_JPRB, 0.75073E+00_JPRB, 0.72896E+00_JPRB ,&
&  0.70787E+00_JPRB, 0.69237E+00_JPRB, 0.68076E+00_JPRB, 0.67132E+00_JPRB, 0.65901E+00_JPRB, 0.64750E+00_JPRB, 0.63578E+00_JPRB ,&
&  0.62769E+00_JPRB, 0.61823E+00_JPRB, 0.61139E+00_JPRB, 0.60551E+00_JPRB, 0.60249E+00_JPRB, 0.60219E+00_JPRB, 0.60366E+00_JPRB ,&
&  0.60416E+00_JPRB, 0.60032E+00_JPRB, 0.59877E+00_JPRB, 0.60329E+00_JPRB, 0.60908E+00_JPRB, 0.61964E+00_JPRB, 0.63502E+00_JPRB ,&
&  0.65408E+00_JPRB, 0.67344E+00_JPRB, 0.69240E+00_JPRB, 0.73608E+00_JPRB, 0.78108E+00_JPRB, 0.81560E+00_JPRB, 0.83299E+00_JPRB ,&
&  0.84240E+00_JPRB, 0.83378E+00_JPRB, 0.83047E+00_JPRB, 0.81556E+00_JPRB, 0.79566E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.71583E+00_JPRB, 0.68791E+00_JPRB, 0.66362E+00_JPRB, 0.64513E+00_JPRB, 0.63300E+00_JPRB, 0.62758E+00_JPRB, 0.62192E+00_JPRB ,&
&  0.61827E+00_JPRB, 0.61395E+00_JPRB, 0.61115E+00_JPRB, 0.61312E+00_JPRB, 0.61391E+00_JPRB, 0.61391E+00_JPRB, 0.61457E+00_JPRB ,&
&  0.60838E+00_JPRB, 0.59495E+00_JPRB, 0.58422E+00_JPRB, 0.58110E+00_JPRB, 0.58185E+00_JPRB, 0.57777E+00_JPRB, 0.57626E+00_JPRB ,&
&  0.57927E+00_JPRB, 0.57117E+00_JPRB, 0.55733E+00_JPRB, 0.54813E+00_JPRB, 0.54839E+00_JPRB, 0.54579E+00_JPRB, 0.54376E+00_JPRB ,&
&  0.57124E+00_JPRB, 0.52454E+00_JPRB, 0.57570E+00_JPRB, 0.58425E+00_JPRB, 0.59182E+00_JPRB, 0.59716E+00_JPRB, 0.59846E+00_JPRB ,&
&  0.59479E+00_JPRB, 0.58775E+00_JPRB, 0.58034E+00_JPRB, 0.57246E+00_JPRB, 0.56642E+00_JPRB, 0.55957E+00_JPRB, 0.55397E+00_JPRB ,&
&  0.55368E+00_JPRB, 0.55389E+00_JPRB, 0.55773E+00_JPRB, 0.56297E+00_JPRB, 0.57044E+00_JPRB, 0.57305E+00_JPRB, 0.57537E+00_JPRB ,&
&  0.57820E+00_JPRB, 0.58003E+00_JPRB, 0.58007E+00_JPRB, 0.57908E+00_JPRB, 0.57780E+00_JPRB, 0.58265E+00_JPRB, 0.58961E+00_JPRB ,&
&  0.59632E+00_JPRB, 0.60629E+00_JPRB, 0.62091E+00_JPRB, 0.63219E+00_JPRB, 0.63716E+00_JPRB, 0.63409E+00_JPRB, 0.62565E+00_JPRB ,&
&  0.61978E+00_JPRB, 0.61882E+00_JPRB, 0.61872E+00_JPRB, 0.61331E+00_JPRB, 0.60524E+00_JPRB, 0.59773E+00_JPRB, 0.58767E+00_JPRB ,&
&  0.57488E+00_JPRB, 0.56421E+00_JPRB, 0.55784E+00_JPRB, 0.55494E+00_JPRB, 0.55471E+00_JPRB, 0.55406E+00_JPRB, 0.55344E+00_JPRB ,&
&  0.55514E+00_JPRB, 0.55540E+00_JPRB, 0.55705E+00_JPRB, 0.56220E+00_JPRB, 0.56711E+00_JPRB, 0.57421E+00_JPRB, 0.58429E+00_JPRB ,&
&  0.59849E+00_JPRB, 0.61558E+00_JPRB, 0.63133E+00_JPRB, 0.65826E+00_JPRB, 0.69621E+00_JPRB, 0.72678E+00_JPRB, 0.74342E+00_JPRB ,&
&  0.75772E+00_JPRB, 0.76244E+00_JPRB, 0.75806E+00_JPRB, 0.75141E+00_JPRB, 0.73626E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.65140E+00_JPRB, 0.63376E+00_JPRB, 0.61972E+00_JPRB, 0.60876E+00_JPRB, 0.60086E+00_JPRB, 0.59724E+00_JPRB, 0.59320E+00_JPRB ,&
&  0.58973E+00_JPRB, 0.58695E+00_JPRB, 0.57973E+00_JPRB, 0.57251E+00_JPRB, 0.57157E+00_JPRB, 0.57695E+00_JPRB, 0.58119E+00_JPRB ,&
&  0.57801E+00_JPRB, 0.56719E+00_JPRB, 0.55701E+00_JPRB, 0.55473E+00_JPRB, 0.55753E+00_JPRB, 0.55992E+00_JPRB, 0.55959E+00_JPRB ,&
&  0.56131E+00_JPRB, 0.56239E+00_JPRB, 0.55401E+00_JPRB, 0.53674E+00_JPRB, 0.52861E+00_JPRB, 0.52392E+00_JPRB, 0.52888E+00_JPRB ,&
&  0.53945E+00_JPRB, 0.53981E+00_JPRB, 0.55045E+00_JPRB, 0.54699E+00_JPRB, 0.54595E+00_JPRB, 0.54274E+00_JPRB, 0.53632E+00_JPRB ,&
&  0.53097E+00_JPRB, 0.52518E+00_JPRB, 0.52121E+00_JPRB, 0.51892E+00_JPRB, 0.51843E+00_JPRB, 0.51892E+00_JPRB, 0.52249E+00_JPRB ,&
&  0.52269E+00_JPRB, 0.52027E+00_JPRB, 0.52253E+00_JPRB, 0.52945E+00_JPRB, 0.53826E+00_JPRB, 0.54343E+00_JPRB, 0.54817E+00_JPRB ,&
&  0.55208E+00_JPRB, 0.55432E+00_JPRB, 0.55728E+00_JPRB, 0.55809E+00_JPRB, 0.55438E+00_JPRB, 0.55361E+00_JPRB, 0.55358E+00_JPRB ,&
&  0.55596E+00_JPRB, 0.56024E+00_JPRB, 0.56716E+00_JPRB, 0.57520E+00_JPRB, 0.57896E+00_JPRB, 0.57621E+00_JPRB, 0.57270E+00_JPRB ,&
&  0.57329E+00_JPRB, 0.57756E+00_JPRB, 0.58234E+00_JPRB, 0.58502E+00_JPRB, 0.58408E+00_JPRB, 0.57664E+00_JPRB, 0.56759E+00_JPRB ,&
&  0.55799E+00_JPRB, 0.54810E+00_JPRB, 0.53922E+00_JPRB, 0.53366E+00_JPRB, 0.52932E+00_JPRB, 0.52640E+00_JPRB, 0.52464E+00_JPRB ,&
&  0.52410E+00_JPRB, 0.52436E+00_JPRB, 0.52776E+00_JPRB, 0.53425E+00_JPRB, 0.53812E+00_JPRB, 0.54679E+00_JPRB, 0.55798E+00_JPRB ,&
&  0.57033E+00_JPRB, 0.58363E+00_JPRB, 0.59967E+00_JPRB, 0.62159E+00_JPRB, 0.64718E+00_JPRB, 0.66930E+00_JPRB, 0.68642E+00_JPRB ,&
&  0.69759E+00_JPRB, 0.69833E+00_JPRB, 0.69118E+00_JPRB, 0.68177E+00_JPRB, 0.66853E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.59227E+00_JPRB, 0.58318E+00_JPRB, 0.57586E+00_JPRB, 0.56956E+00_JPRB, 0.56394E+00_JPRB, 0.56106E+00_JPRB, 0.55815E+00_JPRB ,&
&  0.55542E+00_JPRB, 0.55207E+00_JPRB, 0.55000E+00_JPRB, 0.54599E+00_JPRB, 0.54263E+00_JPRB, 0.54527E+00_JPRB, 0.55070E+00_JPRB ,&
&  0.54921E+00_JPRB, 0.54231E+00_JPRB, 0.53683E+00_JPRB, 0.53666E+00_JPRB, 0.54006E+00_JPRB, 0.54260E+00_JPRB, 0.54558E+00_JPRB ,&
&  0.54540E+00_JPRB, 0.54471E+00_JPRB, 0.54305E+00_JPRB, 0.52719E+00_JPRB, 0.51196E+00_JPRB, 0.50439E+00_JPRB, 0.50017E+00_JPRB ,&
&  0.50040E+00_JPRB, 0.50876E+00_JPRB, 0.49140E+00_JPRB, 0.50082E+00_JPRB, 0.49670E+00_JPRB, 0.49045E+00_JPRB, 0.48682E+00_JPRB ,&
&  0.48471E+00_JPRB, 0.48244E+00_JPRB, 0.48068E+00_JPRB, 0.48245E+00_JPRB, 0.48489E+00_JPRB, 0.48546E+00_JPRB, 0.48703E+00_JPRB ,&
&  0.48669E+00_JPRB, 0.48913E+00_JPRB, 0.49140E+00_JPRB, 0.49662E+00_JPRB, 0.50061E+00_JPRB, 0.50317E+00_JPRB, 0.50648E+00_JPRB ,&
&  0.51107E+00_JPRB, 0.51534E+00_JPRB, 0.51941E+00_JPRB, 0.52283E+00_JPRB, 0.52424E+00_JPRB, 0.52400E+00_JPRB, 0.51987E+00_JPRB ,&
&  0.51656E+00_JPRB, 0.51641E+00_JPRB, 0.51957E+00_JPRB, 0.52645E+00_JPRB, 0.53298E+00_JPRB, 0.53505E+00_JPRB, 0.53571E+00_JPRB ,&
&  0.53725E+00_JPRB, 0.54114E+00_JPRB, 0.54204E+00_JPRB, 0.54084E+00_JPRB, 0.53885E+00_JPRB, 0.53617E+00_JPRB, 0.53153E+00_JPRB ,&
&  0.52539E+00_JPRB, 0.52037E+00_JPRB, 0.51519E+00_JPRB, 0.50392E+00_JPRB, 0.49534E+00_JPRB, 0.49449E+00_JPRB, 0.49609E+00_JPRB ,&
&  0.49565E+00_JPRB, 0.49413E+00_JPRB, 0.49533E+00_JPRB, 0.50104E+00_JPRB, 0.50117E+00_JPRB, 0.50816E+00_JPRB, 0.52223E+00_JPRB ,&
&  0.53683E+00_JPRB, 0.55027E+00_JPRB, 0.56251E+00_JPRB, 0.57577E+00_JPRB, 0.58945E+00_JPRB, 0.60100E+00_JPRB, 0.61101E+00_JPRB ,&
&  0.61598E+00_JPRB, 0.61447E+00_JPRB, 0.60837E+00_JPRB, 0.60442E+00_JPRB, 0.60090E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.55766E+00_JPRB, 0.55620E+00_JPRB, 0.55446E+00_JPRB, 0.55220E+00_JPRB, 0.54879E+00_JPRB, 0.54551E+00_JPRB, 0.54034E+00_JPRB ,&
&  0.53491E+00_JPRB, 0.53041E+00_JPRB, 0.52830E+00_JPRB, 0.52747E+00_JPRB, 0.52861E+00_JPRB, 0.53261E+00_JPRB, 0.53939E+00_JPRB ,&
&  0.53942E+00_JPRB, 0.53260E+00_JPRB, 0.52940E+00_JPRB, 0.52938E+00_JPRB, 0.53178E+00_JPRB, 0.53351E+00_JPRB, 0.53623E+00_JPRB ,&
&  0.53797E+00_JPRB, 0.53848E+00_JPRB, 0.53757E+00_JPRB, 0.53021E+00_JPRB, 0.51309E+00_JPRB, 0.50271E+00_JPRB, 0.49369E+00_JPRB ,&
&  0.48843E+00_JPRB, 0.48460E+00_JPRB, 0.48139E+00_JPRB, 0.48032E+00_JPRB, 0.47924E+00_JPRB, 0.47740E+00_JPRB, 0.47739E+00_JPRB ,&
&  0.47552E+00_JPRB, 0.47307E+00_JPRB, 0.47178E+00_JPRB, 0.47237E+00_JPRB, 0.47414E+00_JPRB, 0.47576E+00_JPRB, 0.47569E+00_JPRB ,&
&  0.47669E+00_JPRB, 0.48043E+00_JPRB, 0.48150E+00_JPRB, 0.48171E+00_JPRB, 0.48154E+00_JPRB, 0.48109E+00_JPRB, 0.48119E+00_JPRB ,&
&  0.48356E+00_JPRB, 0.48872E+00_JPRB, 0.49317E+00_JPRB, 0.49770E+00_JPRB, 0.49973E+00_JPRB, 0.50073E+00_JPRB, 0.50077E+00_JPRB ,&
&  0.49767E+00_JPRB, 0.49362E+00_JPRB, 0.49224E+00_JPRB, 0.49483E+00_JPRB, 0.50018E+00_JPRB, 0.50634E+00_JPRB, 0.51008E+00_JPRB ,&
&  0.51294E+00_JPRB, 0.51405E+00_JPRB, 0.51191E+00_JPRB, 0.51071E+00_JPRB, 0.50801E+00_JPRB, 0.50346E+00_JPRB, 0.50085E+00_JPRB ,&
&  0.49834E+00_JPRB, 0.49610E+00_JPRB, 0.49401E+00_JPRB, 0.48499E+00_JPRB, 0.47727E+00_JPRB, 0.47261E+00_JPRB, 0.46961E+00_JPRB ,&
&  0.46845E+00_JPRB, 0.46749E+00_JPRB, 0.46717E+00_JPRB, 0.46838E+00_JPRB, 0.47055E+00_JPRB, 0.47733E+00_JPRB, 0.49073E+00_JPRB ,&
&  0.51053E+00_JPRB, 0.52898E+00_JPRB, 0.53938E+00_JPRB, 0.54723E+00_JPRB, 0.55301E+00_JPRB, 0.55640E+00_JPRB, 0.55952E+00_JPRB ,&
&  0.56363E+00_JPRB, 0.56516E+00_JPRB, 0.56320E+00_JPRB, 0.56071E+00_JPRB, 0.55961E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.52572E+00_JPRB, 0.52651E+00_JPRB, 0.52551E+00_JPRB, 0.52449E+00_JPRB, 0.52317E+00_JPRB, 0.52180E+00_JPRB, 0.51958E+00_JPRB ,&
&  0.51436E+00_JPRB, 0.50767E+00_JPRB, 0.50371E+00_JPRB, 0.50192E+00_JPRB, 0.50753E+00_JPRB, 0.51585E+00_JPRB, 0.52330E+00_JPRB ,&
&  0.52237E+00_JPRB, 0.51313E+00_JPRB, 0.51025E+00_JPRB, 0.51216E+00_JPRB, 0.51468E+00_JPRB, 0.51748E+00_JPRB, 0.52127E+00_JPRB ,&
&  0.52539E+00_JPRB, 0.52957E+00_JPRB, 0.53136E+00_JPRB, 0.52860E+00_JPRB, 0.52175E+00_JPRB, 0.50772E+00_JPRB, 0.49557E+00_JPRB ,&
&  0.48751E+00_JPRB, 0.48136E+00_JPRB, 0.47756E+00_JPRB, 0.47653E+00_JPRB, 0.47431E+00_JPRB, 0.47056E+00_JPRB, 0.46534E+00_JPRB ,&
&  0.46086E+00_JPRB, 0.45888E+00_JPRB, 0.46127E+00_JPRB, 0.46316E+00_JPRB, 0.46413E+00_JPRB, 0.46386E+00_JPRB, 0.46089E+00_JPRB ,&
&  0.45920E+00_JPRB, 0.45863E+00_JPRB, 0.45825E+00_JPRB, 0.45972E+00_JPRB, 0.45731E+00_JPRB, 0.45532E+00_JPRB, 0.45735E+00_JPRB ,&
&  0.46187E+00_JPRB, 0.46588E+00_JPRB, 0.46959E+00_JPRB, 0.47255E+00_JPRB, 0.47390E+00_JPRB, 0.47618E+00_JPRB, 0.47758E+00_JPRB ,&
&  0.47663E+00_JPRB, 0.47296E+00_JPRB, 0.46955E+00_JPRB, 0.47003E+00_JPRB, 0.47241E+00_JPRB, 0.47515E+00_JPRB, 0.47876E+00_JPRB ,&
&  0.47942E+00_JPRB, 0.47436E+00_JPRB, 0.46927E+00_JPRB, 0.46473E+00_JPRB, 0.46051E+00_JPRB, 0.45724E+00_JPRB, 0.45561E+00_JPRB ,&
&  0.45514E+00_JPRB, 0.45205E+00_JPRB, 0.44607E+00_JPRB, 0.44115E+00_JPRB, 0.44049E+00_JPRB, 0.43958E+00_JPRB, 0.43567E+00_JPRB ,&
&  0.43603E+00_JPRB, 0.43850E+00_JPRB, 0.43734E+00_JPRB, 0.43542E+00_JPRB, 0.43744E+00_JPRB, 0.44297E+00_JPRB, 0.45431E+00_JPRB ,&
&  0.46946E+00_JPRB, 0.48680E+00_JPRB, 0.50284E+00_JPRB, 0.51677E+00_JPRB, 0.52522E+00_JPRB, 0.52745E+00_JPRB, 0.52593E+00_JPRB ,&
&  0.52545E+00_JPRB, 0.52631E+00_JPRB, 0.52546E+00_JPRB, 0.52386E+00_JPRB, 0.52434E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.49632E+00_JPRB, 0.49666E+00_JPRB, 0.49729E+00_JPRB, 0.49872E+00_JPRB, 0.49970E+00_JPRB, 0.49878E+00_JPRB, 0.49841E+00_JPRB ,&
&  0.49510E+00_JPRB, 0.49032E+00_JPRB, 0.48739E+00_JPRB, 0.48888E+00_JPRB, 0.49805E+00_JPRB, 0.50971E+00_JPRB, 0.51666E+00_JPRB ,&
&  0.51215E+00_JPRB, 0.49957E+00_JPRB, 0.49642E+00_JPRB, 0.49866E+00_JPRB, 0.50085E+00_JPRB, 0.50497E+00_JPRB, 0.51108E+00_JPRB ,&
&  0.51665E+00_JPRB, 0.52043E+00_JPRB, 0.52456E+00_JPRB, 0.52619E+00_JPRB, 0.52626E+00_JPRB, 0.51895E+00_JPRB, 0.50763E+00_JPRB ,&
&  0.49673E+00_JPRB, 0.48887E+00_JPRB, 0.48382E+00_JPRB, 0.48021E+00_JPRB, 0.47518E+00_JPRB, 0.47018E+00_JPRB, 0.46369E+00_JPRB ,&
&  0.46064E+00_JPRB, 0.46051E+00_JPRB, 0.46155E+00_JPRB, 0.45975E+00_JPRB, 0.45719E+00_JPRB, 0.45197E+00_JPRB, 0.44755E+00_JPRB ,&
&  0.44575E+00_JPRB, 0.44683E+00_JPRB, 0.44792E+00_JPRB, 0.44685E+00_JPRB, 0.44326E+00_JPRB, 0.44070E+00_JPRB, 0.44197E+00_JPRB ,&
&  0.44777E+00_JPRB, 0.44757E+00_JPRB, 0.44660E+00_JPRB, 0.44752E+00_JPRB, 0.44987E+00_JPRB, 0.45192E+00_JPRB, 0.45136E+00_JPRB ,&
&  0.45065E+00_JPRB, 0.45051E+00_JPRB, 0.45070E+00_JPRB, 0.45250E+00_JPRB, 0.45261E+00_JPRB, 0.45052E+00_JPRB, 0.44500E+00_JPRB ,&
&  0.43659E+00_JPRB, 0.42599E+00_JPRB, 0.42164E+00_JPRB, 0.41852E+00_JPRB, 0.41547E+00_JPRB, 0.41480E+00_JPRB, 0.41618E+00_JPRB ,&
&  0.41479E+00_JPRB, 0.40864E+00_JPRB, 0.40588E+00_JPRB, 0.40643E+00_JPRB, 0.40550E+00_JPRB, 0.40772E+00_JPRB, 0.41158E+00_JPRB ,&
&  0.42191E+00_JPRB, 0.42769E+00_JPRB, 0.42939E+00_JPRB, 0.42718E+00_JPRB, 0.42737E+00_JPRB, 0.42907E+00_JPRB, 0.43343E+00_JPRB ,&
&  0.44009E+00_JPRB, 0.44734E+00_JPRB, 0.45483E+00_JPRB, 0.46311E+00_JPRB, 0.47344E+00_JPRB, 0.48334E+00_JPRB, 0.48722E+00_JPRB ,&
&  0.48758E+00_JPRB, 0.48815E+00_JPRB, 0.48964E+00_JPRB, 0.49135E+00_JPRB, 0.49468E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.46143E+00_JPRB, 0.46404E+00_JPRB, 0.46660E+00_JPRB, 0.46837E+00_JPRB, 0.47121E+00_JPRB, 0.47616E+00_JPRB, 0.47815E+00_JPRB ,&
&  0.47620E+00_JPRB, 0.47398E+00_JPRB, 0.47379E+00_JPRB, 0.47729E+00_JPRB, 0.48410E+00_JPRB, 0.49114E+00_JPRB, 0.49883E+00_JPRB ,&
&  0.49458E+00_JPRB, 0.48184E+00_JPRB, 0.47717E+00_JPRB, 0.47914E+00_JPRB, 0.48398E+00_JPRB, 0.48937E+00_JPRB, 0.49603E+00_JPRB ,&
&  0.50236E+00_JPRB, 0.50650E+00_JPRB, 0.51081E+00_JPRB, 0.51471E+00_JPRB, 0.51719E+00_JPRB, 0.51534E+00_JPRB, 0.51083E+00_JPRB ,&
&  0.50545E+00_JPRB, 0.49994E+00_JPRB, 0.49493E+00_JPRB, 0.47927E+00_JPRB, 0.47675E+00_JPRB, 0.46785E+00_JPRB, 0.46050E+00_JPRB ,&
&  0.45546E+00_JPRB, 0.45129E+00_JPRB, 0.44862E+00_JPRB, 0.44553E+00_JPRB, 0.44114E+00_JPRB, 0.43371E+00_JPRB, 0.43146E+00_JPRB ,&
&  0.43032E+00_JPRB, 0.43119E+00_JPRB, 0.42637E+00_JPRB, 0.42371E+00_JPRB, 0.42078E+00_JPRB, 0.41847E+00_JPRB, 0.42254E+00_JPRB ,&
&  0.42760E+00_JPRB, 0.42502E+00_JPRB, 0.42102E+00_JPRB, 0.42075E+00_JPRB, 0.42507E+00_JPRB, 0.42672E+00_JPRB, 0.42765E+00_JPRB ,&
&  0.42935E+00_JPRB, 0.42978E+00_JPRB, 0.42505E+00_JPRB, 0.41984E+00_JPRB, 0.41128E+00_JPRB, 0.40034E+00_JPRB, 0.39726E+00_JPRB ,&
&  0.39421E+00_JPRB, 0.38851E+00_JPRB, 0.38526E+00_JPRB, 0.38421E+00_JPRB, 0.38386E+00_JPRB, 0.38475E+00_JPRB, 0.38494E+00_JPRB ,&
&  0.38504E+00_JPRB, 0.38152E+00_JPRB, 0.37633E+00_JPRB, 0.37190E+00_JPRB, 0.36585E+00_JPRB, 0.36857E+00_JPRB, 0.37850E+00_JPRB ,&
&  0.39082E+00_JPRB, 0.39839E+00_JPRB, 0.40331E+00_JPRB, 0.40674E+00_JPRB, 0.40979E+00_JPRB, 0.41113E+00_JPRB, 0.41341E+00_JPRB ,&
&  0.41648E+00_JPRB, 0.41787E+00_JPRB, 0.41653E+00_JPRB, 0.41679E+00_JPRB, 0.42011E+00_JPRB, 0.42540E+00_JPRB, 0.43179E+00_JPRB ,&
&  0.43816E+00_JPRB, 0.44359E+00_JPRB, 0.44820E+00_JPRB, 0.45392E+00_JPRB, 0.45779E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.43325E+00_JPRB, 0.43779E+00_JPRB, 0.44210E+00_JPRB, 0.44676E+00_JPRB, 0.45327E+00_JPRB, 0.45661E+00_JPRB, 0.45768E+00_JPRB ,&
&  0.45900E+00_JPRB, 0.45954E+00_JPRB, 0.46100E+00_JPRB, 0.46535E+00_JPRB, 0.47154E+00_JPRB, 0.47603E+00_JPRB, 0.48183E+00_JPRB ,&
&  0.48184E+00_JPRB, 0.47289E+00_JPRB, 0.46484E+00_JPRB, 0.46450E+00_JPRB, 0.47002E+00_JPRB, 0.47579E+00_JPRB, 0.48303E+00_JPRB ,&
&  0.49018E+00_JPRB, 0.49534E+00_JPRB, 0.49821E+00_JPRB, 0.49963E+00_JPRB, 0.50151E+00_JPRB, 0.50102E+00_JPRB, 0.49796E+00_JPRB ,&
&  0.49369E+00_JPRB, 0.48447E+00_JPRB, 0.46496E+00_JPRB, 0.46225E+00_JPRB, 0.45352E+00_JPRB, 0.44597E+00_JPRB, 0.44076E+00_JPRB ,&
&  0.43655E+00_JPRB, 0.43307E+00_JPRB, 0.42908E+00_JPRB, 0.42554E+00_JPRB, 0.42147E+00_JPRB, 0.41426E+00_JPRB, 0.41046E+00_JPRB ,&
&  0.40696E+00_JPRB, 0.40410E+00_JPRB, 0.40123E+00_JPRB, 0.40122E+00_JPRB, 0.40137E+00_JPRB, 0.40399E+00_JPRB, 0.40525E+00_JPRB ,&
&  0.40199E+00_JPRB, 0.39819E+00_JPRB, 0.39527E+00_JPRB, 0.39688E+00_JPRB, 0.39915E+00_JPRB, 0.40071E+00_JPRB, 0.40127E+00_JPRB ,&
&  0.39864E+00_JPRB, 0.39382E+00_JPRB, 0.38883E+00_JPRB, 0.38314E+00_JPRB, 0.37716E+00_JPRB, 0.35152E+00_JPRB, 0.34046E+00_JPRB ,&
&  0.33756E+00_JPRB, 0.34673E+00_JPRB, 0.35874E+00_JPRB, 0.35778E+00_JPRB, 0.36046E+00_JPRB, 0.36077E+00_JPRB, 0.35822E+00_JPRB ,&
&  0.34889E+00_JPRB, 0.33548E+00_JPRB, 0.32679E+00_JPRB, 0.32295E+00_JPRB, 0.32014E+00_JPRB, 0.32174E+00_JPRB, 0.32803E+00_JPRB ,&
&  0.33847E+00_JPRB, 0.35020E+00_JPRB, 0.35905E+00_JPRB, 0.36304E+00_JPRB, 0.36183E+00_JPRB, 0.35874E+00_JPRB, 0.35459E+00_JPRB ,&
&  0.35022E+00_JPRB, 0.35615E+00_JPRB, 0.37063E+00_JPRB, 0.39036E+00_JPRB, 0.39303E+00_JPRB, 0.39833E+00_JPRB, 0.40198E+00_JPRB ,&
&  0.40745E+00_JPRB, 0.41433E+00_JPRB, 0.42026E+00_JPRB, 0.42643E+00_JPRB, 0.43023E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.41110E+00_JPRB, 0.41624E+00_JPRB, 0.42319E+00_JPRB, 0.43088E+00_JPRB, 0.43638E+00_JPRB, 0.43891E+00_JPRB, 0.44160E+00_JPRB ,&
&  0.44504E+00_JPRB, 0.44649E+00_JPRB, 0.44756E+00_JPRB, 0.45032E+00_JPRB, 0.45594E+00_JPRB, 0.45905E+00_JPRB, 0.46394E+00_JPRB ,&
&  0.46589E+00_JPRB, 0.46359E+00_JPRB, 0.45662E+00_JPRB, 0.45457E+00_JPRB, 0.45817E+00_JPRB, 0.46298E+00_JPRB, 0.46821E+00_JPRB ,&
&  0.47359E+00_JPRB, 0.47850E+00_JPRB, 0.48080E+00_JPRB, 0.48137E+00_JPRB, 0.48147E+00_JPRB, 0.48127E+00_JPRB, 0.47860E+00_JPRB ,&
&  0.47305E+00_JPRB, 0.43318E+00_JPRB, 0.40673E+00_JPRB, 0.41073E+00_JPRB, 0.42205E+00_JPRB, 0.41611E+00_JPRB, 0.41459E+00_JPRB ,&
&  0.41307E+00_JPRB, 0.40945E+00_JPRB, 0.40406E+00_JPRB, 0.39963E+00_JPRB, 0.39418E+00_JPRB, 0.38710E+00_JPRB, 0.38140E+00_JPRB ,&
&  0.37993E+00_JPRB, 0.38488E+00_JPRB, 0.38632E+00_JPRB, 0.38818E+00_JPRB, 0.38652E+00_JPRB, 0.38812E+00_JPRB, 0.38659E+00_JPRB ,&
&  0.38085E+00_JPRB, 0.37432E+00_JPRB, 0.37201E+00_JPRB, 0.37149E+00_JPRB, 0.37258E+00_JPRB, 0.37524E+00_JPRB, 0.37479E+00_JPRB ,&
&  0.36181E+00_JPRB, 0.35022E+00_JPRB, 0.33159E+00_JPRB, 0.31351E+00_JPRB, 0.30041E+00_JPRB, 0.29452E+00_JPRB, 0.29024E+00_JPRB ,&
&  0.28951E+00_JPRB, 0.29573E+00_JPRB, 0.31318E+00_JPRB, 0.33414E+00_JPRB, 0.34696E+00_JPRB, 0.32860E+00_JPRB, 0.31333E+00_JPRB ,&
&  0.30412E+00_JPRB, 0.29916E+00_JPRB, 0.29492E+00_JPRB, 0.29353E+00_JPRB, 0.29408E+00_JPRB, 0.29640E+00_JPRB, 0.29971E+00_JPRB ,&
&  0.30454E+00_JPRB, 0.30911E+00_JPRB, 0.31188E+00_JPRB, 0.31321E+00_JPRB, 0.31396E+00_JPRB, 0.31476E+00_JPRB, 0.31415E+00_JPRB ,&
&  0.31403E+00_JPRB, 0.31763E+00_JPRB, 0.32469E+00_JPRB, 0.33369E+00_JPRB, 0.34196E+00_JPRB, 0.35129E+00_JPRB, 0.36441E+00_JPRB ,&
&  0.37882E+00_JPRB, 0.39033E+00_JPRB, 0.39458E+00_JPRB, 0.40089E+00_JPRB, 0.40631E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.40159E+00_JPRB, 0.40861E+00_JPRB, 0.41361E+00_JPRB, 0.42360E+00_JPRB, 0.42652E+00_JPRB, 0.42952E+00_JPRB, 0.43211E+00_JPRB ,&
&  0.43851E+00_JPRB, 0.44218E+00_JPRB, 0.44004E+00_JPRB, 0.43948E+00_JPRB, 0.43860E+00_JPRB, 0.43718E+00_JPRB, 0.43817E+00_JPRB ,&
&  0.43828E+00_JPRB, 0.44041E+00_JPRB, 0.43796E+00_JPRB, 0.43517E+00_JPRB, 0.43709E+00_JPRB, 0.44192E+00_JPRB, 0.44516E+00_JPRB ,&
&  0.43989E+00_JPRB, 0.43832E+00_JPRB, 0.44677E+00_JPRB, 0.44873E+00_JPRB, 0.44981E+00_JPRB, 0.45169E+00_JPRB, 0.45135E+00_JPRB ,&
&  0.43729E+00_JPRB, 0.41154E+00_JPRB, 0.38404E+00_JPRB, 0.38618E+00_JPRB, 0.40087E+00_JPRB, 0.39466E+00_JPRB, 0.39297E+00_JPRB ,&
&  0.39013E+00_JPRB, 0.38558E+00_JPRB, 0.38128E+00_JPRB, 0.37916E+00_JPRB, 0.37638E+00_JPRB, 0.37158E+00_JPRB, 0.36996E+00_JPRB ,&
&  0.36873E+00_JPRB, 0.37764E+00_JPRB, 0.37248E+00_JPRB, 0.34993E+00_JPRB, 0.33330E+00_JPRB, 0.32420E+00_JPRB, 0.31237E+00_JPRB ,&
&  0.29854E+00_JPRB, 0.29076E+00_JPRB, 0.28969E+00_JPRB, 0.28767E+00_JPRB, 0.28899E+00_JPRB, 0.29011E+00_JPRB, 0.29122E+00_JPRB ,&
&  0.29195E+00_JPRB, 0.29160E+00_JPRB, 0.28689E+00_JPRB, 0.28082E+00_JPRB, 0.27752E+00_JPRB, 0.27752E+00_JPRB, 0.28039E+00_JPRB ,&
&  0.28657E+00_JPRB, 0.29299E+00_JPRB, 0.30767E+00_JPRB, 0.31545E+00_JPRB, 0.30651E+00_JPRB, 0.30149E+00_JPRB, 0.29589E+00_JPRB ,&
&  0.28511E+00_JPRB, 0.28103E+00_JPRB, 0.27767E+00_JPRB, 0.27584E+00_JPRB, 0.27675E+00_JPRB, 0.28018E+00_JPRB, 0.28177E+00_JPRB ,&
&  0.28424E+00_JPRB, 0.28704E+00_JPRB, 0.28817E+00_JPRB, 0.28908E+00_JPRB, 0.28986E+00_JPRB, 0.29190E+00_JPRB, 0.29398E+00_JPRB ,&
&  0.29567E+00_JPRB, 0.29869E+00_JPRB, 0.30235E+00_JPRB, 0.30720E+00_JPRB, 0.31146E+00_JPRB, 0.31398E+00_JPRB, 0.32157E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.34829E+00_JPRB, 0.37740E+00_JPRB, 0.40167E+00_JPRB, 0.39661E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.40308E+00_JPRB, 0.40744E+00_JPRB, 0.40809E+00_JPRB, 0.41138E+00_JPRB, 0.41641E+00_JPRB, 0.42220E+00_JPRB, 0.42709E+00_JPRB ,&
&  0.43238E+00_JPRB, 0.43208E+00_JPRB, 0.42700E+00_JPRB, 0.41950E+00_JPRB, 0.41323E+00_JPRB, 0.39643E+00_JPRB, 0.38769E+00_JPRB ,&
&  0.38046E+00_JPRB, 0.38022E+00_JPRB, 0.38068E+00_JPRB, 0.38237E+00_JPRB, 0.38334E+00_JPRB, 0.38835E+00_JPRB, 0.38975E+00_JPRB ,&
&  0.38701E+00_JPRB, 0.38172E+00_JPRB, 0.37631E+00_JPRB, 0.37531E+00_JPRB, 0.37792E+00_JPRB, 0.37979E+00_JPRB, 0.37674E+00_JPRB ,&
&  0.37308E+00_JPRB, 0.37288E+00_JPRB, 0.37346E+00_JPRB, 0.37490E+00_JPRB, 0.38444E+00_JPRB, 0.37914E+00_JPRB, 0.37460E+00_JPRB ,&
&  0.36753E+00_JPRB, 0.36148E+00_JPRB, 0.35735E+00_JPRB, 0.35773E+00_JPRB, 0.36090E+00_JPRB, 0.35999E+00_JPRB, 0.36261E+00_JPRB ,&
&  0.35145E+00_JPRB, 0.34219E+00_JPRB, 0.32723E+00_JPRB, 0.31524E+00_JPRB, 0.30273E+00_JPRB, 0.29418E+00_JPRB, 0.28630E+00_JPRB ,&
&  0.27959E+00_JPRB, 0.27388E+00_JPRB, 0.26853E+00_JPRB, 0.26687E+00_JPRB, 0.26782E+00_JPRB, 0.26780E+00_JPRB, 0.26724E+00_JPRB ,&
&  0.26771E+00_JPRB, 0.26864E+00_JPRB, 0.26935E+00_JPRB, 0.27035E+00_JPRB, 0.27205E+00_JPRB, 0.27480E+00_JPRB, 0.27856E+00_JPRB ,&
&  0.28440E+00_JPRB, 0.28899E+00_JPRB, 0.29147E+00_JPRB, 0.29284E+00_JPRB, 0.28974E+00_JPRB, 0.28428E+00_JPRB, 0.27884E+00_JPRB ,&
&  0.27327E+00_JPRB, 0.27068E+00_JPRB, 0.26829E+00_JPRB, 0.26716E+00_JPRB, 0.26689E+00_JPRB, 0.26775E+00_JPRB, 0.26959E+00_JPRB ,&
&  0.27141E+00_JPRB, 0.27379E+00_JPRB, 0.27536E+00_JPRB, 0.27726E+00_JPRB, 0.27956E+00_JPRB, 0.28155E+00_JPRB, 0.28364E+00_JPRB ,&
&  0.28600E+00_JPRB, 0.28784E+00_JPRB, 0.29100E+00_JPRB, 0.29489E+00_JPRB, 0.30238E+00_JPRB, 0.31101E+00_JPRB, 0.32630E+00_JPRB ,&
&  0.34404E+00_JPRB, 0.37586E+00_JPRB, 0.38543E+00_JPRB, 0.39368E+00_JPRB, 0.40139E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.39891E+00_JPRB, 0.40154E+00_JPRB, 0.40544E+00_JPRB, 0.41097E+00_JPRB, 0.41795E+00_JPRB, 0.42437E+00_JPRB, 0.42594E+00_JPRB ,&
&  0.41639E+00_JPRB, 0.40349E+00_JPRB, 0.38515E+00_JPRB, 0.37072E+00_JPRB, 0.35659E+00_JPRB, 0.34608E+00_JPRB, 0.33759E+00_JPRB ,&
&  0.33203E+00_JPRB, 0.33163E+00_JPRB, 0.33191E+00_JPRB, 0.33309E+00_JPRB, 0.33190E+00_JPRB, 0.33285E+00_JPRB, 0.33304E+00_JPRB ,&
&  0.33166E+00_JPRB, 0.33149E+00_JPRB, 0.33139E+00_JPRB, 0.33483E+00_JPRB, 0.34166E+00_JPRB, 0.34794E+00_JPRB, 0.35414E+00_JPRB ,&
&  0.36146E+00_JPRB, 0.36340E+00_JPRB, 0.36754E+00_JPRB, 0.36614E+00_JPRB, 0.36267E+00_JPRB, 0.35822E+00_JPRB, 0.35192E+00_JPRB ,&
&  0.34761E+00_JPRB, 0.34375E+00_JPRB, 0.34255E+00_JPRB, 0.34292E+00_JPRB, 0.34291E+00_JPRB, 0.34549E+00_JPRB, 0.33989E+00_JPRB ,&
&  0.33287E+00_JPRB, 0.32239E+00_JPRB, 0.30998E+00_JPRB, 0.30072E+00_JPRB, 0.29169E+00_JPRB, 0.28521E+00_JPRB, 0.27942E+00_JPRB ,&
&  0.27408E+00_JPRB, 0.27053E+00_JPRB, 0.26680E+00_JPRB, 0.26403E+00_JPRB, 0.26315E+00_JPRB, 0.26121E+00_JPRB, 0.26012E+00_JPRB ,&
&  0.25967E+00_JPRB, 0.26009E+00_JPRB, 0.26185E+00_JPRB, 0.26342E+00_JPRB, 0.26562E+00_JPRB, 0.26806E+00_JPRB, 0.27012E+00_JPRB ,&
&  0.27288E+00_JPRB, 0.27514E+00_JPRB, 0.27543E+00_JPRB, 0.27485E+00_JPRB, 0.27269E+00_JPRB, 0.26980E+00_JPRB, 0.26729E+00_JPRB ,&
&  0.26468E+00_JPRB, 0.26285E+00_JPRB, 0.26220E+00_JPRB, 0.26129E+00_JPRB, 0.26175E+00_JPRB, 0.26294E+00_JPRB, 0.26378E+00_JPRB ,&
&  0.26554E+00_JPRB, 0.26697E+00_JPRB, 0.26867E+00_JPRB, 0.27167E+00_JPRB, 0.27443E+00_JPRB, 0.27688E+00_JPRB, 0.27935E+00_JPRB ,&
&  0.27945E+00_JPRB, 0.28120E+00_JPRB, 0.28361E+00_JPRB, 0.28805E+00_JPRB, 0.30122E+00_JPRB, 0.31022E+00_JPRB, 0.33094E+00_JPRB ,&
&  0.34825E+00_JPRB, 0.38243E+00_JPRB, 0.38169E+00_JPRB, 0.38825E+00_JPRB, 0.39507E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.39331E+00_JPRB, 0.40017E+00_JPRB, 0.40691E+00_JPRB, 0.41210E+00_JPRB, 0.41570E+00_JPRB, 0.41886E+00_JPRB, 0.41696E+00_JPRB ,&
&  0.41087E+00_JPRB, 0.39610E+00_JPRB, 0.38480E+00_JPRB, 0.37352E+00_JPRB, 0.36406E+00_JPRB, 0.35497E+00_JPRB, 0.34653E+00_JPRB ,&
&  0.33840E+00_JPRB, 0.33161E+00_JPRB, 0.32592E+00_JPRB, 0.32093E+00_JPRB, 0.31414E+00_JPRB, 0.30814E+00_JPRB, 0.30332E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29918E+00_JPRB, 0.30156E+00_JPRB, 0.30603E+00_JPRB, 0.31441E+00_JPRB, 0.32274E+00_JPRB, 0.33203E+00_JPRB ,&
&  0.34620E+00_JPRB, 0.34940E+00_JPRB, 0.34830E+00_JPRB, 0.34767E+00_JPRB, 0.34270E+00_JPRB, 0.33648E+00_JPRB, 0.32737E+00_JPRB ,&
&  0.32885E+00_JPRB, 0.32688E+00_JPRB, 0.33443E+00_JPRB, 0.33445E+00_JPRB, 0.33311E+00_JPRB, 0.33365E+00_JPRB, 0.32638E+00_JPRB ,&
&  0.32139E+00_JPRB, 0.31219E+00_JPRB, 0.30507E+00_JPRB, 0.29741E+00_JPRB, 0.28960E+00_JPRB, 0.28500E+00_JPRB, 0.27894E+00_JPRB ,&
&  0.27452E+00_JPRB, 0.27129E+00_JPRB, 0.26705E+00_JPRB, 0.26442E+00_JPRB, 0.26210E+00_JPRB, 0.25966E+00_JPRB, 0.25840E+00_JPRB ,&
&  0.25744E+00_JPRB, 0.25710E+00_JPRB, 0.25726E+00_JPRB, 0.25748E+00_JPRB, 0.25802E+00_JPRB, 0.25899E+00_JPRB, 0.25970E+00_JPRB ,&
&  0.26051E+00_JPRB, 0.26081E+00_JPRB, 0.26044E+00_JPRB, 0.26005E+00_JPRB, 0.25887E+00_JPRB, 0.25810E+00_JPRB, 0.25705E+00_JPRB ,&
&  0.25641E+00_JPRB, 0.25673E+00_JPRB, 0.25700E+00_JPRB, 0.25770E+00_JPRB, 0.25839E+00_JPRB, 0.25999E+00_JPRB, 0.26148E+00_JPRB ,&
&  0.26307E+00_JPRB, 0.26510E+00_JPRB, 0.26673E+00_JPRB, 0.26885E+00_JPRB, 0.27137E+00_JPRB, 0.27300E+00_JPRB, 0.27594E+00_JPRB ,&
&  0.27881E+00_JPRB, 0.28242E+00_JPRB, 0.28874E+00_JPRB, 0.29565E+00_JPRB, 0.30496E+00_JPRB, 0.31611E+00_JPRB, 0.33005E+00_JPRB ,&
&  0.34784E+00_JPRB, 0.37240E+00_JPRB, 0.37381E+00_JPRB, 0.38218E+00_JPRB, 0.38776E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.38209E+00_JPRB, 0.38722E+00_JPRB, 0.39156E+00_JPRB, 0.39544E+00_JPRB, 0.39895E+00_JPRB, 0.39998E+00_JPRB, 0.39908E+00_JPRB ,&
&  0.38991E+00_JPRB, 0.38321E+00_JPRB, 0.37494E+00_JPRB, 0.36748E+00_JPRB, 0.36057E+00_JPRB, 0.35208E+00_JPRB, 0.34496E+00_JPRB ,&
&  0.33770E+00_JPRB, 0.33042E+00_JPRB, 0.32359E+00_JPRB, 0.31555E+00_JPRB, 0.30719E+00_JPRB, 0.29916E+00_JPRB, 0.29434E+00_JPRB ,&
&  0.29160E+00_JPRB, 0.29153E+00_JPRB, 0.29464E+00_JPRB, 0.29854E+00_JPRB, 0.30357E+00_JPRB, 0.31002E+00_JPRB, 0.31695E+00_JPRB ,&
&  0.32131E+00_JPRB, 0.32365E+00_JPRB, 0.32310E+00_JPRB, 0.32139E+00_JPRB, 0.31924E+00_JPRB, 0.31750E+00_JPRB, 0.31479E+00_JPRB ,&
&  0.31133E+00_JPRB, 0.30909E+00_JPRB, 0.30863E+00_JPRB, 0.31008E+00_JPRB, 0.31019E+00_JPRB, 0.30857E+00_JPRB, 0.30467E+00_JPRB ,&
&  0.30019E+00_JPRB, 0.29564E+00_JPRB, 0.29087E+00_JPRB, 0.28641E+00_JPRB, 0.28277E+00_JPRB, 0.27889E+00_JPRB, 0.27467E+00_JPRB ,&
&  0.27203E+00_JPRB, 0.26925E+00_JPRB, 0.26619E+00_JPRB, 0.26330E+00_JPRB, 0.26090E+00_JPRB, 0.25904E+00_JPRB, 0.25748E+00_JPRB ,&
&  0.25630E+00_JPRB, 0.25540E+00_JPRB, 0.25469E+00_JPRB, 0.25417E+00_JPRB, 0.25367E+00_JPRB, 0.25286E+00_JPRB, 0.25206E+00_JPRB ,&
&  0.25126E+00_JPRB, 0.25100E+00_JPRB, 0.25050E+00_JPRB, 0.25053E+00_JPRB, 0.25080E+00_JPRB, 0.25096E+00_JPRB, 0.25149E+00_JPRB ,&
&  0.25264E+00_JPRB, 0.25385E+00_JPRB, 0.25443E+00_JPRB, 0.25556E+00_JPRB, 0.25667E+00_JPRB, 0.25811E+00_JPRB, 0.26002E+00_JPRB ,&
&  0.26120E+00_JPRB, 0.26251E+00_JPRB, 0.26403E+00_JPRB, 0.26566E+00_JPRB, 0.26846E+00_JPRB, 0.27150E+00_JPRB, 0.27488E+00_JPRB ,&
&  0.27925E+00_JPRB, 0.28348E+00_JPRB, 0.28833E+00_JPRB, 0.29408E+00_JPRB, 0.30453E+00_JPRB, 0.31130E+00_JPRB, 0.31977E+00_JPRB ,&
&  0.32803E+00_JPRB, 0.33177E+00_JPRB, 0.36654E+00_JPRB, 0.36934E+00_JPRB, 0.37498E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.30892E+00_JPRB, 0.31555E+00_JPRB, 0.31981E+00_JPRB, 0.32428E+00_JPRB, 0.32118E+00_JPRB, 0.35985E+00_JPRB, 0.35887E+00_JPRB ,&
&  0.35789E+00_JPRB, 0.34425E+00_JPRB, 0.34237E+00_JPRB, 0.33782E+00_JPRB, 0.33232E+00_JPRB, 0.32622E+00_JPRB, 0.32094E+00_JPRB ,&
&  0.31608E+00_JPRB, 0.31062E+00_JPRB, 0.30620E+00_JPRB, 0.30211E+00_JPRB, 0.29875E+00_JPRB, 0.29578E+00_JPRB, 0.29369E+00_JPRB ,&
&  0.29320E+00_JPRB, 0.29475E+00_JPRB, 0.29802E+00_JPRB, 0.29971E+00_JPRB, 0.30033E+00_JPRB, 0.30084E+00_JPRB, 0.30136E+00_JPRB ,&
&  0.30128E+00_JPRB, 0.30111E+00_JPRB, 0.29980E+00_JPRB, 0.29741E+00_JPRB, 0.29544E+00_JPRB, 0.29400E+00_JPRB, 0.29290E+00_JPRB ,&
&  0.29189E+00_JPRB, 0.29092E+00_JPRB, 0.28988E+00_JPRB, 0.28929E+00_JPRB, 0.28834E+00_JPRB, 0.28693E+00_JPRB, 0.28483E+00_JPRB ,&
&  0.28270E+00_JPRB, 0.28049E+00_JPRB, 0.27779E+00_JPRB, 0.27563E+00_JPRB, 0.27353E+00_JPRB, 0.27140E+00_JPRB, 0.26926E+00_JPRB ,&
&  0.26806E+00_JPRB, 0.26688E+00_JPRB, 0.26539E+00_JPRB, 0.26351E+00_JPRB, 0.26209E+00_JPRB, 0.26059E+00_JPRB, 0.25929E+00_JPRB ,&
&  0.25845E+00_JPRB, 0.25757E+00_JPRB, 0.25666E+00_JPRB, 0.25573E+00_JPRB, 0.25505E+00_JPRB, 0.25426E+00_JPRB, 0.25353E+00_JPRB ,&
&  0.25287E+00_JPRB, 0.25211E+00_JPRB, 0.25193E+00_JPRB, 0.25185E+00_JPRB, 0.25193E+00_JPRB, 0.25198E+00_JPRB, 0.25230E+00_JPRB ,&
&  0.25279E+00_JPRB, 0.25352E+00_JPRB, 0.25441E+00_JPRB, 0.25523E+00_JPRB, 0.25603E+00_JPRB, 0.25686E+00_JPRB, 0.25808E+00_JPRB ,&
&  0.25940E+00_JPRB, 0.26079E+00_JPRB, 0.26221E+00_JPRB, 0.26331E+00_JPRB, 0.26577E+00_JPRB, 0.26809E+00_JPRB, 0.27054E+00_JPRB ,&
&  0.27336E+00_JPRB, 0.27674E+00_JPRB, 0.28038E+00_JPRB, 0.28429E+00_JPRB, 0.28951E+00_JPRB, 0.29268E+00_JPRB, 0.29494E+00_JPRB ,&
&  0.29745E+00_JPRB, 0.30175E+00_JPRB, 0.30316E+00_JPRB, 0.30443E+00_JPRB, 0.30636E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.28190E+00_JPRB, 0.28218E+00_JPRB, 0.28224E+00_JPRB, 0.28213E+00_JPRB, 0.27984E+00_JPRB, 0.28001E+00_JPRB, 0.27993E+00_JPRB ,&
&  0.28007E+00_JPRB, 0.28027E+00_JPRB, 0.28091E+00_JPRB, 0.28169E+00_JPRB, 0.28246E+00_JPRB, 0.28319E+00_JPRB, 0.28385E+00_JPRB ,&
&  0.28426E+00_JPRB, 0.28478E+00_JPRB, 0.28341E+00_JPRB, 0.28255E+00_JPRB, 0.28243E+00_JPRB, 0.28251E+00_JPRB, 0.28247E+00_JPRB ,&
&  0.28237E+00_JPRB, 0.28222E+00_JPRB, 0.28170E+00_JPRB, 0.28135E+00_JPRB, 0.28119E+00_JPRB, 0.28097E+00_JPRB, 0.28064E+00_JPRB ,&
&  0.28024E+00_JPRB, 0.27992E+00_JPRB, 0.27954E+00_JPRB, 0.27910E+00_JPRB, 0.27899E+00_JPRB, 0.27885E+00_JPRB, 0.27827E+00_JPRB ,&
&  0.27759E+00_JPRB, 0.27658E+00_JPRB, 0.27565E+00_JPRB, 0.27480E+00_JPRB, 0.27412E+00_JPRB, 0.27379E+00_JPRB, 0.27344E+00_JPRB ,&
&  0.27296E+00_JPRB, 0.27230E+00_JPRB, 0.27168E+00_JPRB, 0.27089E+00_JPRB, 0.27019E+00_JPRB, 0.26955E+00_JPRB, 0.26880E+00_JPRB ,&
&  0.26798E+00_JPRB, 0.26713E+00_JPRB, 0.26626E+00_JPRB, 0.26528E+00_JPRB, 0.26433E+00_JPRB, 0.26336E+00_JPRB, 0.26230E+00_JPRB ,&
&  0.26126E+00_JPRB, 0.26082E+00_JPRB, 0.26031E+00_JPRB, 0.25988E+00_JPRB, 0.25955E+00_JPRB, 0.25930E+00_JPRB, 0.25905E+00_JPRB ,&
&  0.25886E+00_JPRB, 0.25865E+00_JPRB, 0.25855E+00_JPRB, 0.25845E+00_JPRB, 0.25845E+00_JPRB, 0.25852E+00_JPRB, 0.25870E+00_JPRB ,&
&  0.25891E+00_JPRB, 0.25918E+00_JPRB, 0.25954E+00_JPRB, 0.26015E+00_JPRB, 0.26075E+00_JPRB, 0.26144E+00_JPRB, 0.26213E+00_JPRB ,&
&  0.26289E+00_JPRB, 0.26364E+00_JPRB, 0.26440E+00_JPRB, 0.26513E+00_JPRB, 0.26612E+00_JPRB, 0.26707E+00_JPRB, 0.26812E+00_JPRB ,&
&  0.26916E+00_JPRB, 0.27029E+00_JPRB, 0.27141E+00_JPRB, 0.27260E+00_JPRB, 0.27397E+00_JPRB, 0.27436E+00_JPRB, 0.27478E+00_JPRB ,&
&  0.27530E+00_JPRB, 0.27606E+00_JPRB, 0.27673E+00_JPRB, 0.27764E+00_JPRB, 0.27917E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB, 0.26827E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB, 0.78440E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.77659E+00_JPRB, 0.77757E+00_JPRB, 0.77838E+00_JPRB, 0.77882E+00_JPRB, 0.77922E+00_JPRB, 0.77952E+00_JPRB, 0.77966E+00_JPRB ,&
&  0.77948E+00_JPRB, 0.77982E+00_JPRB, 0.78046E+00_JPRB, 0.78106E+00_JPRB, 0.78157E+00_JPRB, 0.78199E+00_JPRB, 0.78208E+00_JPRB ,&
&  0.78272E+00_JPRB, 0.78625E+00_JPRB, 0.78868E+00_JPRB, 0.78965E+00_JPRB, 0.79037E+00_JPRB, 0.79093E+00_JPRB, 0.79117E+00_JPRB ,&
&  0.79153E+00_JPRB, 0.79189E+00_JPRB, 0.79215E+00_JPRB, 0.79225E+00_JPRB, 0.79241E+00_JPRB, 0.79268E+00_JPRB, 0.79305E+00_JPRB ,&
&  0.79369E+00_JPRB, 0.79509E+00_JPRB, 0.79698E+00_JPRB, 0.79902E+00_JPRB, 0.80140E+00_JPRB, 0.80462E+00_JPRB, 0.80745E+00_JPRB ,&
&  0.80958E+00_JPRB, 0.81084E+00_JPRB, 0.81006E+00_JPRB, 0.80761E+00_JPRB, 0.80423E+00_JPRB, 0.80082E+00_JPRB, 0.79745E+00_JPRB ,&
&  0.79349E+00_JPRB, 0.78987E+00_JPRB, 0.78671E+00_JPRB, 0.78397E+00_JPRB, 0.78140E+00_JPRB, 0.77927E+00_JPRB, 0.77737E+00_JPRB ,&
&  0.77598E+00_JPRB, 0.77451E+00_JPRB, 0.77319E+00_JPRB, 0.77192E+00_JPRB, 0.77053E+00_JPRB, 0.76860E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.76504E+00_JPRB, 0.76404E+00_JPRB, 0.76331E+00_JPRB, 0.76245E+00_JPRB, 0.76176E+00_JPRB, 0.76189E+00_JPRB, 0.76242E+00_JPRB ,&
&  0.76288E+00_JPRB, 0.76370E+00_JPRB, 0.76481E+00_JPRB, 0.76538E+00_JPRB, 0.76555E+00_JPRB, 0.76528E+00_JPRB, 0.76485E+00_JPRB ,&
&  0.76431E+00_JPRB, 0.76442E+00_JPRB, 0.76461E+00_JPRB, 0.76553E+00_JPRB, 0.76647E+00_JPRB, 0.76762E+00_JPRB, 0.76891E+00_JPRB ,&
&  0.76987E+00_JPRB, 0.77089E+00_JPRB, 0.77170E+00_JPRB, 0.77279E+00_JPRB, 0.77373E+00_JPRB, 0.77431E+00_JPRB, 0.77455E+00_JPRB ,&
&  0.77469E+00_JPRB, 0.77459E+00_JPRB, 0.77446E+00_JPRB, 0.77413E+00_JPRB, 0.77404E+00_JPRB, 0.77409E+00_JPRB, 0.77410E+00_JPRB ,&
&  0.77407E+00_JPRB, 0.77434E+00_JPRB, 0.77455E+00_JPRB, 0.77520E+00_JPRB, 0.77578E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.77671E+00_JPRB, 0.77783E+00_JPRB, 0.77877E+00_JPRB, 0.77901E+00_JPRB, 0.77928E+00_JPRB, 0.77897E+00_JPRB, 0.77805E+00_JPRB ,&
&  0.77610E+00_JPRB, 0.77490E+00_JPRB, 0.77448E+00_JPRB, 0.77379E+00_JPRB, 0.77234E+00_JPRB, 0.77434E+00_JPRB, 0.77828E+00_JPRB ,&
&  0.77748E+00_JPRB, 0.77586E+00_JPRB, 0.77337E+00_JPRB, 0.77105E+00_JPRB, 0.76834E+00_JPRB, 0.76523E+00_JPRB, 0.76277E+00_JPRB ,&
&  0.76172E+00_JPRB, 0.75991E+00_JPRB, 0.75724E+00_JPRB, 0.75368E+00_JPRB, 0.75265E+00_JPRB, 0.75161E+00_JPRB, 0.75027E+00_JPRB ,&
&  0.74884E+00_JPRB, 0.74554E+00_JPRB, 0.74246E+00_JPRB, 0.74078E+00_JPRB, 0.74515E+00_JPRB, 0.74933E+00_JPRB, 0.75402E+00_JPRB ,&
&  0.75870E+00_JPRB, 0.76339E+00_JPRB, 0.76926E+00_JPRB, 0.77256E+00_JPRB, 0.77261E+00_JPRB, 0.77582E+00_JPRB, 0.77824E+00_JPRB ,&
&  0.77790E+00_JPRB, 0.77483E+00_JPRB, 0.77266E+00_JPRB, 0.77277E+00_JPRB, 0.77313E+00_JPRB, 0.77308E+00_JPRB, 0.77363E+00_JPRB ,&
&  0.77488E+00_JPRB, 0.77571E+00_JPRB, 0.77552E+00_JPRB, 0.77522E+00_JPRB, 0.77382E+00_JPRB, 0.77147E+00_JPRB, 0.76828E+00_JPRB ,&
&  0.76562E+00_JPRB, 0.76228E+00_JPRB, 0.75876E+00_JPRB, 0.75519E+00_JPRB, 0.75229E+00_JPRB, 0.74861E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.74172E+00_JPRB, 0.74185E+00_JPRB, 0.74350E+00_JPRB, 0.74314E+00_JPRB, 0.74302E+00_JPRB, 0.74213E+00_JPRB, 0.74362E+00_JPRB ,&
&  0.74525E+00_JPRB, 0.74912E+00_JPRB, 0.75317E+00_JPRB, 0.75927E+00_JPRB, 0.76480E+00_JPRB, 0.77023E+00_JPRB, 0.77408E+00_JPRB ,&
&  0.77796E+00_JPRB, 0.78030E+00_JPRB, 0.78144E+00_JPRB, 0.78155E+00_JPRB, 0.78220E+00_JPRB, 0.78218E+00_JPRB, 0.78129E+00_JPRB ,&
&  0.77985E+00_JPRB, 0.77908E+00_JPRB, 0.77786E+00_JPRB, 0.77619E+00_JPRB, 0.77500E+00_JPRB, 0.77365E+00_JPRB, 0.77268E+00_JPRB ,&
&  0.77174E+00_JPRB, 0.77182E+00_JPRB, 0.77214E+00_JPRB, 0.77341E+00_JPRB, 0.77502E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.76047E+00_JPRB, 0.76247E+00_JPRB, 0.76387E+00_JPRB, 0.76324E+00_JPRB, 0.76318E+00_JPRB, 0.76347E+00_JPRB, 0.76402E+00_JPRB ,&
&  0.76489E+00_JPRB, 0.76589E+00_JPRB, 0.76733E+00_JPRB, 0.76798E+00_JPRB, 0.76825E+00_JPRB, 0.77452E+00_JPRB, 0.77632E+00_JPRB ,&
&  0.77473E+00_JPRB, 0.77101E+00_JPRB, 0.76671E+00_JPRB, 0.76160E+00_JPRB, 0.75514E+00_JPRB, 0.74710E+00_JPRB, 0.73392E+00_JPRB ,&
&  0.72587E+00_JPRB, 0.71773E+00_JPRB, 0.70861E+00_JPRB, 0.70634E+00_JPRB, 0.70367E+00_JPRB, 0.67295E+00_JPRB, 0.67163E+00_JPRB ,&
&  0.69753E+00_JPRB, 0.70867E+00_JPRB, 0.71174E+00_JPRB, 0.71591E+00_JPRB, 0.68935E+00_JPRB, 0.65371E+00_JPRB, 0.62937E+00_JPRB ,&
&  0.58894E+00_JPRB, 0.61877E+00_JPRB, 0.63769E+00_JPRB, 0.64274E+00_JPRB, 0.63985E+00_JPRB, 0.63530E+00_JPRB, 0.65970E+00_JPRB ,&
&  0.71898E+00_JPRB, 0.73026E+00_JPRB, 0.74530E+00_JPRB, 0.76162E+00_JPRB, 0.77852E+00_JPRB, 0.79083E+00_JPRB, 0.80511E+00_JPRB ,&
&  0.81364E+00_JPRB, 0.82004E+00_JPRB, 0.82018E+00_JPRB, 0.81822E+00_JPRB, 0.80960E+00_JPRB, 0.80257E+00_JPRB, 0.79548E+00_JPRB ,&
&  0.79255E+00_JPRB, 0.78978E+00_JPRB, 0.78702E+00_JPRB, 0.78143E+00_JPRB, 0.77346E+00_JPRB, 0.76488E+00_JPRB, 0.75571E+00_JPRB ,&
&  0.74694E+00_JPRB, 0.73660E+00_JPRB, 0.72990E+00_JPRB, 0.72450E+00_JPRB, 0.72567E+00_JPRB, 0.72481E+00_JPRB, 0.72869E+00_JPRB ,&
&  0.73217E+00_JPRB, 0.73809E+00_JPRB, 0.74430E+00_JPRB, 0.75367E+00_JPRB, 0.76097E+00_JPRB, 0.76710E+00_JPRB, 0.77030E+00_JPRB ,&
&  0.77302E+00_JPRB, 0.77373E+00_JPRB, 0.77348E+00_JPRB, 0.77167E+00_JPRB, 0.77062E+00_JPRB, 0.76975E+00_JPRB, 0.76824E+00_JPRB ,&
&  0.76711E+00_JPRB, 0.76602E+00_JPRB, 0.76527E+00_JPRB, 0.76416E+00_JPRB, 0.76275E+00_JPRB, 0.75981E+00_JPRB, 0.75657E+00_JPRB ,&
&  0.75307E+00_JPRB, 0.75204E+00_JPRB, 0.75201E+00_JPRB, 0.75425E+00_JPRB, 0.75655E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.76665E+00_JPRB, 0.76360E+00_JPRB, 0.76225E+00_JPRB, 0.76123E+00_JPRB, 0.76104E+00_JPRB, 0.76110E+00_JPRB, 0.76152E+00_JPRB ,&
&  0.76310E+00_JPRB, 0.76550E+00_JPRB, 0.76717E+00_JPRB, 0.76743E+00_JPRB, 0.76913E+00_JPRB, 0.77196E+00_JPRB, 0.77274E+00_JPRB ,&
&  0.76859E+00_JPRB, 0.76273E+00_JPRB, 0.75673E+00_JPRB, 0.74966E+00_JPRB, 0.74293E+00_JPRB, 0.73709E+00_JPRB, 0.72596E+00_JPRB ,&
&  0.71885E+00_JPRB, 0.70823E+00_JPRB, 0.66843E+00_JPRB, 0.67247E+00_JPRB, 0.66456E+00_JPRB, 0.66317E+00_JPRB, 0.66750E+00_JPRB ,&
&  0.65624E+00_JPRB, 0.65484E+00_JPRB, 0.64029E+00_JPRB, 0.61715E+00_JPRB, 0.60263E+00_JPRB, 0.57993E+00_JPRB, 0.57870E+00_JPRB ,&
&  0.57414E+00_JPRB, 0.57982E+00_JPRB, 0.59311E+00_JPRB, 0.59767E+00_JPRB, 0.59444E+00_JPRB, 0.59361E+00_JPRB, 0.60676E+00_JPRB ,&
&  0.63933E+00_JPRB, 0.68665E+00_JPRB, 0.75026E+00_JPRB, 0.78482E+00_JPRB, 0.81835E+00_JPRB, 0.84726E+00_JPRB, 0.87559E+00_JPRB ,&
&  0.89757E+00_JPRB, 0.91627E+00_JPRB, 0.92229E+00_JPRB, 0.92473E+00_JPRB, 0.91593E+00_JPRB, 0.89732E+00_JPRB, 0.87204E+00_JPRB ,&
&  0.84618E+00_JPRB, 0.82746E+00_JPRB, 0.81407E+00_JPRB, 0.80873E+00_JPRB, 0.80726E+00_JPRB, 0.80638E+00_JPRB, 0.80677E+00_JPRB ,&
&  0.80403E+00_JPRB, 0.78808E+00_JPRB, 0.77487E+00_JPRB, 0.75815E+00_JPRB, 0.75251E+00_JPRB, 0.74889E+00_JPRB, 0.75287E+00_JPRB ,&
&  0.75837E+00_JPRB, 0.77049E+00_JPRB, 0.78467E+00_JPRB, 0.78920E+00_JPRB, 0.79705E+00_JPRB, 0.80715E+00_JPRB, 0.80590E+00_JPRB ,&
&  0.80446E+00_JPRB, 0.80036E+00_JPRB, 0.79365E+00_JPRB, 0.78612E+00_JPRB, 0.77784E+00_JPRB, 0.77005E+00_JPRB, 0.76333E+00_JPRB ,&
&  0.76001E+00_JPRB, 0.75954E+00_JPRB, 0.76408E+00_JPRB, 0.76940E+00_JPRB, 0.77605E+00_JPRB, 0.77948E+00_JPRB, 0.77855E+00_JPRB ,&
&  0.77312E+00_JPRB, 0.76941E+00_JPRB, 0.76639E+00_JPRB, 0.76752E+00_JPRB, 0.76801E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.78279E+00_JPRB, 0.78107E+00_JPRB, 0.78421E+00_JPRB, 0.78915E+00_JPRB, 0.79368E+00_JPRB, 0.79527E+00_JPRB, 0.78787E+00_JPRB ,&
&  0.78166E+00_JPRB, 0.77612E+00_JPRB, 0.77190E+00_JPRB, 0.76927E+00_JPRB, 0.76900E+00_JPRB, 0.76555E+00_JPRB, 0.76059E+00_JPRB ,&
&  0.75594E+00_JPRB, 0.75368E+00_JPRB, 0.75303E+00_JPRB, 0.75234E+00_JPRB, 0.75410E+00_JPRB, 0.75316E+00_JPRB, 0.74959E+00_JPRB ,&
&  0.74610E+00_JPRB, 0.73087E+00_JPRB, 0.69991E+00_JPRB, 0.69042E+00_JPRB, 0.67906E+00_JPRB, 0.68051E+00_JPRB, 0.68104E+00_JPRB ,&
&  0.67903E+00_JPRB, 0.60637E+00_JPRB, 0.61591E+00_JPRB, 0.59607E+00_JPRB, 0.58290E+00_JPRB, 0.58075E+00_JPRB, 0.56513E+00_JPRB ,&
&  0.55502E+00_JPRB, 0.54136E+00_JPRB, 0.53993E+00_JPRB, 0.54615E+00_JPRB, 0.54492E+00_JPRB, 0.55750E+00_JPRB, 0.57480E+00_JPRB ,&
&  0.60292E+00_JPRB, 0.73036E+00_JPRB, 0.78816E+00_JPRB, 0.82402E+00_JPRB, 0.85493E+00_JPRB, 0.89070E+00_JPRB, 0.92905E+00_JPRB ,&
&  0.96118E+00_JPRB, 0.99960E+00_JPRB, 0.10219E+01_JPRB, 0.10240E+01_JPRB, 0.10451E+01_JPRB, 0.10427E+01_JPRB, 0.10223E+01_JPRB ,&
&  0.98348E+00_JPRB, 0.92650E+00_JPRB, 0.88374E+00_JPRB, 0.85763E+00_JPRB, 0.83850E+00_JPRB, 0.82643E+00_JPRB, 0.82122E+00_JPRB ,&
&  0.83030E+00_JPRB, 0.82854E+00_JPRB, 0.82227E+00_JPRB, 0.80344E+00_JPRB, 0.79265E+00_JPRB, 0.78533E+00_JPRB, 0.78593E+00_JPRB ,&
&  0.79074E+00_JPRB, 0.80363E+00_JPRB, 0.82231E+00_JPRB, 0.83614E+00_JPRB, 0.84379E+00_JPRB, 0.84330E+00_JPRB, 0.84762E+00_JPRB ,&
&  0.84441E+00_JPRB, 0.83618E+00_JPRB, 0.82756E+00_JPRB, 0.81909E+00_JPRB, 0.81054E+00_JPRB, 0.80477E+00_JPRB, 0.80516E+00_JPRB ,&
&  0.80731E+00_JPRB, 0.80737E+00_JPRB, 0.80587E+00_JPRB, 0.80646E+00_JPRB, 0.80975E+00_JPRB, 0.81238E+00_JPRB, 0.81258E+00_JPRB ,&
&  0.80511E+00_JPRB, 0.79713E+00_JPRB, 0.79043E+00_JPRB, 0.78943E+00_JPRB, 0.78735E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.77207E+00_JPRB, 0.77923E+00_JPRB, 0.79868E+00_JPRB, 0.81542E+00_JPRB, 0.82518E+00_JPRB, 0.82419E+00_JPRB, 0.81179E+00_JPRB ,&
&  0.80093E+00_JPRB, 0.79300E+00_JPRB, 0.78392E+00_JPRB, 0.77533E+00_JPRB, 0.76544E+00_JPRB, 0.75733E+00_JPRB, 0.75175E+00_JPRB ,&
&  0.74849E+00_JPRB, 0.74910E+00_JPRB, 0.75031E+00_JPRB, 0.75130E+00_JPRB, 0.75367E+00_JPRB, 0.75697E+00_JPRB, 0.75696E+00_JPRB ,&
&  0.75627E+00_JPRB, 0.74402E+00_JPRB, 0.71796E+00_JPRB, 0.69953E+00_JPRB, 0.69403E+00_JPRB, 0.68313E+00_JPRB, 0.69094E+00_JPRB ,&
&  0.69139E+00_JPRB, 0.68997E+00_JPRB, 0.68398E+00_JPRB, 0.68034E+00_JPRB, 0.65086E+00_JPRB, 0.54791E+00_JPRB, 0.55097E+00_JPRB ,&
&  0.53523E+00_JPRB, 0.51459E+00_JPRB, 0.50978E+00_JPRB, 0.50836E+00_JPRB, 0.50975E+00_JPRB, 0.53551E+00_JPRB, 0.56558E+00_JPRB ,&
&  0.59370E+00_JPRB, 0.73692E+00_JPRB, 0.77815E+00_JPRB, 0.82313E+00_JPRB, 0.86415E+00_JPRB, 0.91388E+00_JPRB, 0.96831E+00_JPRB ,&
&  0.10072E+01_JPRB, 0.10500E+01_JPRB, 0.10894E+01_JPRB, 0.11235E+01_JPRB, 0.11484E+01_JPRB, 0.11688E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.10795E+01_JPRB, 0.10114E+01_JPRB, 0.97017E+00_JPRB, 0.92789E+00_JPRB, 0.89934E+00_JPRB, 0.87998E+00_JPRB ,&
&  0.87944E+00_JPRB, 0.86325E+00_JPRB, 0.86444E+00_JPRB, 0.85271E+00_JPRB, 0.84585E+00_JPRB, 0.83968E+00_JPRB, 0.83144E+00_JPRB ,&
&  0.82063E+00_JPRB, 0.82464E+00_JPRB, 0.84713E+00_JPRB, 0.85712E+00_JPRB, 0.86808E+00_JPRB, 0.86212E+00_JPRB, 0.84791E+00_JPRB ,&
&  0.82493E+00_JPRB, 0.81979E+00_JPRB, 0.80866E+00_JPRB, 0.80385E+00_JPRB, 0.80112E+00_JPRB, 0.80968E+00_JPRB, 0.82172E+00_JPRB ,&
&  0.82840E+00_JPRB, 0.82740E+00_JPRB, 0.82378E+00_JPRB, 0.82209E+00_JPRB, 0.81912E+00_JPRB, 0.81349E+00_JPRB, 0.80519E+00_JPRB ,&
&  0.79672E+00_JPRB, 0.79235E+00_JPRB, 0.78349E+00_JPRB, 0.77738E+00_JPRB, 0.77433E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.75945E+00_JPRB, 0.78289E+00_JPRB, 0.81361E+00_JPRB, 0.83413E+00_JPRB, 0.84216E+00_JPRB, 0.82898E+00_JPRB, 0.80985E+00_JPRB ,&
&  0.79457E+00_JPRB, 0.78363E+00_JPRB, 0.77388E+00_JPRB, 0.76689E+00_JPRB, 0.75943E+00_JPRB, 0.75212E+00_JPRB, 0.74664E+00_JPRB ,&
&  0.74328E+00_JPRB, 0.73088E+00_JPRB, 0.72778E+00_JPRB, 0.73280E+00_JPRB, 0.73646E+00_JPRB, 0.74382E+00_JPRB, 0.75120E+00_JPRB ,&
&  0.75062E+00_JPRB, 0.74339E+00_JPRB, 0.73682E+00_JPRB, 0.71772E+00_JPRB, 0.69848E+00_JPRB, 0.69292E+00_JPRB, 0.70215E+00_JPRB ,&
&  0.68826E+00_JPRB, 0.66093E+00_JPRB, 0.68701E+00_JPRB, 0.70310E+00_JPRB, 0.66139E+00_JPRB, 0.61856E+00_JPRB, 0.56032E+00_JPRB ,&
&  0.54050E+00_JPRB, 0.50907E+00_JPRB, 0.49985E+00_JPRB, 0.49668E+00_JPRB, 0.50613E+00_JPRB, 0.53104E+00_JPRB, 0.58025E+00_JPRB ,&
&  0.63816E+00_JPRB, 0.74419E+00_JPRB, 0.78292E+00_JPRB, 0.83832E+00_JPRB, 0.88733E+00_JPRB, 0.93046E+00_JPRB, 0.96419E+00_JPRB ,&
&  0.10178E+01_JPRB, 0.10953E+01_JPRB, 0.11608E+01_JPRB, 0.12032E+01_JPRB, 0.12531E+01_JPRB, 0.13089E+01_JPRB, 0.13967E+01_JPRB ,&
&  0.15121E+01_JPRB, 0.13077E+01_JPRB, 0.11897E+01_JPRB, 0.11121E+01_JPRB, 0.10580E+01_JPRB, 0.10403E+01_JPRB, 0.98064E+00_JPRB ,&
&  0.96095E+00_JPRB, 0.95422E+00_JPRB, 0.93873E+00_JPRB, 0.93315E+00_JPRB, 0.92668E+00_JPRB, 0.91865E+00_JPRB, 0.90363E+00_JPRB ,&
&  0.88434E+00_JPRB, 0.89345E+00_JPRB, 0.99685E+00_JPRB, 0.10194E+01_JPRB, 0.10022E+01_JPRB, 0.93843E+00_JPRB, 0.86530E+00_JPRB ,&
&  0.81411E+00_JPRB, 0.79292E+00_JPRB, 0.78367E+00_JPRB, 0.77881E+00_JPRB, 0.78272E+00_JPRB, 0.79083E+00_JPRB, 0.80659E+00_JPRB ,&
&  0.80096E+00_JPRB, 0.80771E+00_JPRB, 0.81567E+00_JPRB, 0.81921E+00_JPRB, 0.81620E+00_JPRB, 0.80630E+00_JPRB, 0.79902E+00_JPRB ,&
&  0.79260E+00_JPRB, 0.78554E+00_JPRB, 0.77462E+00_JPRB, 0.76173E+00_JPRB, 0.75614E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.73568E+00_JPRB, 0.77632E+00_JPRB, 0.82610E+00_JPRB, 0.85808E+00_JPRB, 0.85873E+00_JPRB, 0.83109E+00_JPRB, 0.80030E+00_JPRB ,&
&  0.77972E+00_JPRB, 0.77076E+00_JPRB, 0.76076E+00_JPRB, 0.75434E+00_JPRB, 0.74923E+00_JPRB, 0.74391E+00_JPRB, 0.73865E+00_JPRB ,&
&  0.73465E+00_JPRB, 0.73094E+00_JPRB, 0.72465E+00_JPRB, 0.71572E+00_JPRB, 0.71214E+00_JPRB, 0.71841E+00_JPRB, 0.73047E+00_JPRB ,&
&  0.74325E+00_JPRB, 0.74664E+00_JPRB, 0.74784E+00_JPRB, 0.73799E+00_JPRB, 0.72180E+00_JPRB, 0.70336E+00_JPRB, 0.68904E+00_JPRB ,&
&  0.68181E+00_JPRB, 0.68546E+00_JPRB, 0.68131E+00_JPRB, 0.70337E+00_JPRB, 0.69051E+00_JPRB, 0.64086E+00_JPRB, 0.58747E+00_JPRB ,&
&  0.55511E+00_JPRB, 0.51847E+00_JPRB, 0.50267E+00_JPRB, 0.50739E+00_JPRB, 0.54109E+00_JPRB, 0.58351E+00_JPRB, 0.65233E+00_JPRB ,&
&  0.77654E+00_JPRB, 0.84941E+00_JPRB, 0.90335E+00_JPRB, 0.94978E+00_JPRB, 0.98688E+00_JPRB, 0.10299E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.12318E+01_JPRB, 0.13027E+01_JPRB, 0.13183E+01_JPRB, 0.13602E+01_JPRB, 0.13229E+01_JPRB, 0.15272E+01_JPRB ,&
&  0.20111E+01_JPRB, 0.17310E+01_JPRB, 0.15017E+01_JPRB, 0.13727E+01_JPRB, 0.12985E+01_JPRB, 0.12274E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.10626E+01_JPRB, 0.10442E+01_JPRB, 0.98856E+00_JPRB, 0.93197E+00_JPRB, 0.92526E+00_JPRB, 0.98577E+00_JPRB ,&
&  0.11420E+01_JPRB, 0.13616E+01_JPRB, 0.16048E+01_JPRB, 0.11306E+01_JPRB, 0.98628E+00_JPRB, 0.88254E+00_JPRB, 0.82804E+00_JPRB ,&
&  0.80947E+00_JPRB, 0.80610E+00_JPRB, 0.80746E+00_JPRB, 0.80171E+00_JPRB, 0.79334E+00_JPRB, 0.78748E+00_JPRB, 0.77984E+00_JPRB ,&
&  0.77154E+00_JPRB, 0.77776E+00_JPRB, 0.78934E+00_JPRB, 0.79927E+00_JPRB, 0.80968E+00_JPRB, 0.80924E+00_JPRB, 0.80550E+00_JPRB ,&
&  0.80080E+00_JPRB, 0.78689E+00_JPRB, 0.76991E+00_JPRB, 0.74253E+00_JPRB, 0.72756E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.72708E+00_JPRB, 0.77455E+00_JPRB, 0.84067E+00_JPRB, 0.87876E+00_JPRB, 0.87649E+00_JPRB, 0.83193E+00_JPRB, 0.77368E+00_JPRB ,&
&  0.73263E+00_JPRB, 0.70814E+00_JPRB, 0.69953E+00_JPRB, 0.69634E+00_JPRB, 0.69671E+00_JPRB, 0.69793E+00_JPRB, 0.69755E+00_JPRB ,&
&  0.69569E+00_JPRB, 0.69426E+00_JPRB, 0.69247E+00_JPRB, 0.69335E+00_JPRB, 0.69492E+00_JPRB, 0.69912E+00_JPRB, 0.70729E+00_JPRB ,&
&  0.71819E+00_JPRB, 0.72488E+00_JPRB, 0.72681E+00_JPRB, 0.72446E+00_JPRB, 0.72474E+00_JPRB, 0.73072E+00_JPRB, 0.73173E+00_JPRB ,&
&  0.71289E+00_JPRB, 0.69131E+00_JPRB, 0.69515E+00_JPRB, 0.71240E+00_JPRB, 0.70465E+00_JPRB, 0.66328E+00_JPRB, 0.60436E+00_JPRB ,&
&  0.56579E+00_JPRB, 0.53880E+00_JPRB, 0.55498E+00_JPRB, 0.60407E+00_JPRB, 0.67504E+00_JPRB, 0.82361E+00_JPRB, 0.91335E+00_JPRB ,&
&  0.99039E+00_JPRB, 0.10350E+01_JPRB, 0.10635E+01_JPRB, 0.10965E+01_JPRB, 0.11616E+01_JPRB, 0.12360E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.13924E+01_JPRB, 0.14904E+01_JPRB, 0.16125E+01_JPRB, 0.13183E+01_JPRB, 0.13316E+01_JPRB, 0.15225E+01_JPRB, 0.17319E+01_JPRB ,&
&  0.18312E+01_JPRB, 0.17147E+01_JPRB, 0.16470E+01_JPRB, 0.16696E+01_JPRB, 0.15460E+01_JPRB, 0.14384E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.12979E+01_JPRB, 0.12759E+01_JPRB, 0.11692E+01_JPRB, 0.10516E+01_JPRB, 0.10296E+01_JPRB, 0.10330E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.11173E+01_JPRB, 0.11572E+01_JPRB, 0.10476E+01_JPRB, 0.82517E+00_JPRB, 0.76776E+00_JPRB, 0.74532E+00_JPRB, 0.74904E+00_JPRB ,&
&  0.75884E+00_JPRB, 0.77018E+00_JPRB, 0.78286E+00_JPRB, 0.78746E+00_JPRB, 0.78412E+00_JPRB, 0.76863E+00_JPRB, 0.75065E+00_JPRB ,&
&  0.73443E+00_JPRB, 0.73024E+00_JPRB, 0.73669E+00_JPRB, 0.73606E+00_JPRB, 0.75303E+00_JPRB, 0.76951E+00_JPRB, 0.78659E+00_JPRB ,&
&  0.78290E+00_JPRB, 0.77050E+00_JPRB, 0.75721E+00_JPRB, 0.72884E+00_JPRB, 0.70934E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.77305E+00_JPRB, 0.81895E+00_JPRB, 0.86049E+00_JPRB, 0.88272E+00_JPRB, 0.87505E+00_JPRB, 0.84235E+00_JPRB, 0.76868E+00_JPRB ,&
&  0.73903E+00_JPRB, 0.70014E+00_JPRB, 0.68222E+00_JPRB, 0.66864E+00_JPRB, 0.65879E+00_JPRB, 0.65099E+00_JPRB, 0.65038E+00_JPRB ,&
&  0.65668E+00_JPRB, 0.66354E+00_JPRB, 0.66961E+00_JPRB, 0.67741E+00_JPRB, 0.68316E+00_JPRB, 0.69333E+00_JPRB, 0.70530E+00_JPRB ,&
&  0.71327E+00_JPRB, 0.71850E+00_JPRB, 0.72443E+00_JPRB, 0.72854E+00_JPRB, 0.73304E+00_JPRB, 0.73852E+00_JPRB, 0.73536E+00_JPRB ,&
&  0.71866E+00_JPRB, 0.69842E+00_JPRB, 0.70347E+00_JPRB, 0.71920E+00_JPRB, 0.71259E+00_JPRB, 0.68439E+00_JPRB, 0.62253E+00_JPRB ,&
&  0.58290E+00_JPRB, 0.59481E+00_JPRB, 0.68969E+00_JPRB, 0.85714E+00_JPRB, 0.91951E+00_JPRB, 0.98224E+00_JPRB, 0.10167E+01_JPRB ,&
&  0.96413E+00_JPRB, 0.96574E+00_JPRB, 0.10343E+01_JPRB, 0.12317E+01_JPRB, 0.12826E+01_JPRB, 0.13677E+01_JPRB, 0.14950E+01_JPRB ,&
&  0.17141E+01_JPRB, 0.18464E+01_JPRB, 0.19097E+01_JPRB, 0.16880E+01_JPRB, 0.19967E+01_JPRB, 0.25087E+01_JPRB, 0.22752E+01_JPRB ,&
&  0.21934E+01_JPRB, 0.21219E+01_JPRB, 0.21587E+01_JPRB, 0.19967E+01_JPRB, 0.17670E+01_JPRB, 0.16180E+01_JPRB, 0.15673E+01_JPRB ,&
&  0.14962E+01_JPRB, 0.14378E+01_JPRB, 0.12475E+01_JPRB, 0.11738E+01_JPRB, 0.11754E+01_JPRB, 0.11424E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.98967E+00_JPRB, 0.91633E+00_JPRB, 0.84063E+00_JPRB, 0.79113E+00_JPRB, 0.77846E+00_JPRB, 0.78097E+00_JPRB ,&
&  0.78448E+00_JPRB, 0.78746E+00_JPRB, 0.79544E+00_JPRB, 0.80580E+00_JPRB, 0.80192E+00_JPRB, 0.77319E+00_JPRB, 0.76323E+00_JPRB ,&
&  0.75061E+00_JPRB, 0.72952E+00_JPRB, 0.71088E+00_JPRB, 0.71555E+00_JPRB, 0.72165E+00_JPRB, 0.73901E+00_JPRB, 0.75847E+00_JPRB ,&
&  0.77504E+00_JPRB, 0.78095E+00_JPRB, 0.77431E+00_JPRB, 0.77698E+00_JPRB, 0.76399E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.83042E+00_JPRB, 0.84312E+00_JPRB, 0.86944E+00_JPRB, 0.88930E+00_JPRB, 0.89988E+00_JPRB, 0.84044E+00_JPRB, 0.78135E+00_JPRB ,&
&  0.79510E+00_JPRB, 0.72375E+00_JPRB, 0.68070E+00_JPRB, 0.66977E+00_JPRB, 0.66778E+00_JPRB, 0.64903E+00_JPRB, 0.63649E+00_JPRB ,&
&  0.63448E+00_JPRB, 0.64913E+00_JPRB, 0.67290E+00_JPRB, 0.69123E+00_JPRB, 0.71055E+00_JPRB, 0.72745E+00_JPRB, 0.73565E+00_JPRB ,&
&  0.74176E+00_JPRB, 0.74744E+00_JPRB, 0.75115E+00_JPRB, 0.75204E+00_JPRB, 0.74837E+00_JPRB, 0.74795E+00_JPRB, 0.74122E+00_JPRB ,&
&  0.72327E+00_JPRB, 0.70842E+00_JPRB, 0.71102E+00_JPRB, 0.72808E+00_JPRB, 0.73244E+00_JPRB, 0.72549E+00_JPRB, 0.69629E+00_JPRB ,&
&  0.62701E+00_JPRB, 0.68506E+00_JPRB, 0.89991E+00_JPRB, 0.96445E+00_JPRB, 0.10082E+01_JPRB, 0.10655E+01_JPRB, 0.11148E+01_JPRB ,&
&  0.11575E+01_JPRB, 0.11964E+01_JPRB, 0.12337E+01_JPRB, 0.12868E+01_JPRB, 0.13612E+01_JPRB, 0.14819E+01_JPRB, 0.17220E+01_JPRB ,&
&  0.21851E+01_JPRB, 0.18191E+01_JPRB, 0.17245E+01_JPRB, 0.18622E+01_JPRB, 0.26566E+01_JPRB, 0.28826E+01_JPRB, 0.26218E+01_JPRB ,&
&  0.24922E+01_JPRB, 0.23615E+01_JPRB, 0.22413E+01_JPRB, 0.20382E+01_JPRB, 0.18901E+01_JPRB, 0.18464E+01_JPRB, 0.17965E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.15818E+01_JPRB, 0.13766E+01_JPRB, 0.13264E+01_JPRB, 0.13193E+01_JPRB, 0.12503E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.98981E+00_JPRB, 0.93368E+00_JPRB, 0.89372E+00_JPRB, 0.86196E+00_JPRB, 0.84289E+00_JPRB, 0.82913E+00_JPRB ,&
&  0.82126E+00_JPRB, 0.81928E+00_JPRB, 0.80750E+00_JPRB, 0.81659E+00_JPRB, 0.81788E+00_JPRB, 0.80018E+00_JPRB, 0.81265E+00_JPRB ,&
&  0.80825E+00_JPRB, 0.79484E+00_JPRB, 0.76409E+00_JPRB, 0.76768E+00_JPRB, 0.78793E+00_JPRB, 0.82733E+00_JPRB, 0.85908E+00_JPRB ,&
&  0.90856E+00_JPRB, 0.89357E+00_JPRB, 0.84837E+00_JPRB, 0.83541E+00_JPRB, 0.82440E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.86832E+00_JPRB, 0.87880E+00_JPRB, 0.88694E+00_JPRB, 0.90375E+00_JPRB, 0.91896E+00_JPRB, 0.86912E+00_JPRB, 0.82897E+00_JPRB ,&
&  0.88462E+00_JPRB, 0.79553E+00_JPRB, 0.84242E+00_JPRB, 0.74847E+00_JPRB, 0.70118E+00_JPRB, 0.67604E+00_JPRB, 0.65970E+00_JPRB ,&
&  0.65002E+00_JPRB, 0.67729E+00_JPRB, 0.71253E+00_JPRB, 0.74799E+00_JPRB, 0.80504E+00_JPRB, 0.79486E+00_JPRB, 0.78259E+00_JPRB ,&
&  0.77194E+00_JPRB, 0.76562E+00_JPRB, 0.77799E+00_JPRB, 0.77288E+00_JPRB, 0.76722E+00_JPRB, 0.76001E+00_JPRB, 0.74082E+00_JPRB ,&
&  0.69913E+00_JPRB, 0.68872E+00_JPRB, 0.70825E+00_JPRB, 0.72737E+00_JPRB, 0.74625E+00_JPRB, 0.75213E+00_JPRB, 0.76077E+00_JPRB ,&
&  0.77868E+00_JPRB, 0.80186E+00_JPRB, 0.93778E+00_JPRB, 0.97623E+00_JPRB, 0.10214E+01_JPRB, 0.10537E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.11892E+01_JPRB, 0.12211E+01_JPRB, 0.12810E+01_JPRB, 0.13911E+01_JPRB, 0.16227E+01_JPRB, 0.20340E+01_JPRB ,&
&  0.25512E+01_JPRB, 0.19598E+01_JPRB, 0.23836E+01_JPRB, 0.24178E+01_JPRB, 0.27640E+01_JPRB, 0.30366E+01_JPRB, 0.30778E+01_JPRB ,&
&  0.33113E+01_JPRB, 0.29044E+01_JPRB, 0.25931E+01_JPRB, 0.22758E+01_JPRB, 0.20273E+01_JPRB, 0.19839E+01_JPRB, 0.19369E+01_JPRB ,&
&  0.19652E+01_JPRB, 0.18821E+01_JPRB, 0.16074E+01_JPRB, 0.14606E+01_JPRB, 0.14089E+01_JPRB, 0.13795E+01_JPRB, 0.12783E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.11727E+01_JPRB, 0.11060E+01_JPRB, 0.10308E+01_JPRB, 0.97863E+00_JPRB, 0.95117E+00_JPRB, 0.91110E+00_JPRB ,&
&  0.90220E+00_JPRB, 0.88254E+00_JPRB, 0.82160E+00_JPRB, 0.81651E+00_JPRB, 0.80918E+00_JPRB, 0.77040E+00_JPRB, 0.78263E+00_JPRB ,&
&  0.81278E+00_JPRB, 0.81031E+00_JPRB, 0.86308E+00_JPRB, 0.93174E+00_JPRB, 0.95179E+00_JPRB, 0.95415E+00_JPRB, 0.10475E+01_JPRB ,&
&  0.96663E+00_JPRB, 0.96119E+00_JPRB, 0.95533E+00_JPRB, 0.90557E+00_JPRB, 0.87649E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.92745E+00_JPRB, 0.91514E+00_JPRB, 0.90764E+00_JPRB, 0.91894E+00_JPRB, 0.93476E+00_JPRB, 0.94012E+00_JPRB, 0.90621E+00_JPRB ,&
&  0.95910E+00_JPRB, 0.92625E+00_JPRB, 0.90797E+00_JPRB, 0.89791E+00_JPRB, 0.87992E+00_JPRB, 0.78122E+00_JPRB, 0.67953E+00_JPRB ,&
&  0.66531E+00_JPRB, 0.69283E+00_JPRB, 0.75955E+00_JPRB, 0.80155E+00_JPRB, 0.84899E+00_JPRB, 0.84446E+00_JPRB, 0.83155E+00_JPRB ,&
&  0.82060E+00_JPRB, 0.79879E+00_JPRB, 0.80081E+00_JPRB, 0.79704E+00_JPRB, 0.78444E+00_JPRB, 0.78712E+00_JPRB, 0.79109E+00_JPRB ,&
&  0.73598E+00_JPRB, 0.70076E+00_JPRB, 0.69655E+00_JPRB, 0.68784E+00_JPRB, 0.75148E+00_JPRB, 0.77655E+00_JPRB, 0.79573E+00_JPRB ,&
&  0.82074E+00_JPRB, 0.85787E+00_JPRB, 0.91956E+00_JPRB, 0.96868E+00_JPRB, 0.10016E+01_JPRB, 0.10245E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.11427E+01_JPRB, 0.12065E+01_JPRB, 0.13182E+01_JPRB, 0.14945E+01_JPRB, 0.16755E+01_JPRB, 0.23027E+01_JPRB ,&
&  0.25225E+01_JPRB, 0.28296E+01_JPRB, 0.32042E+01_JPRB, 0.29913E+01_JPRB, 0.32652E+01_JPRB, 0.34326E+01_JPRB, 0.33060E+01_JPRB ,&
&  0.32378E+01_JPRB, 0.28604E+01_JPRB, 0.27700E+01_JPRB, 0.27107E+01_JPRB, 0.27038E+01_JPRB, 0.25198E+01_JPRB, 0.23271E+01_JPRB ,&
&  0.24360E+01_JPRB, 0.27082E+01_JPRB, 0.21598E+01_JPRB, 0.19331E+01_JPRB, 0.17512E+01_JPRB, 0.16952E+01_JPRB, 0.16003E+01_JPRB ,&
&  0.15770E+01_JPRB, 0.15736E+01_JPRB, 0.15021E+01_JPRB, 0.14026E+01_JPRB, 0.12260E+01_JPRB, 0.11458E+01_JPRB, 0.10562E+01_JPRB ,&
&  0.10107E+01_JPRB, 0.94642E+00_JPRB, 0.85766E+00_JPRB, 0.82878E+00_JPRB, 0.79475E+00_JPRB, 0.76282E+00_JPRB, 0.78814E+00_JPRB ,&
&  0.87948E+00_JPRB, 0.96014E+00_JPRB, 0.11429E+01_JPRB, 0.11952E+01_JPRB, 0.12268E+01_JPRB, 0.12626E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.98916E+00_JPRB, 0.10705E+01_JPRB, 0.10321E+01_JPRB, 0.98530E+00_JPRB, 0.94752E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.10300E+01_JPRB, 0.10000E+01_JPRB, 0.97000E+00_JPRB, 0.96303E+00_JPRB, 0.97231E+00_JPRB, 0.97018E+00_JPRB, 0.97568E+00_JPRB ,&
&  0.99585E+00_JPRB, 0.98605E+00_JPRB, 0.95475E+00_JPRB, 0.93686E+00_JPRB, 0.91680E+00_JPRB, 0.91528E+00_JPRB, 0.81508E+00_JPRB ,&
&  0.75231E+00_JPRB, 0.81114E+00_JPRB, 0.79759E+00_JPRB, 0.86020E+00_JPRB, 0.95831E+00_JPRB, 0.92717E+00_JPRB, 0.92244E+00_JPRB ,&
&  0.89451E+00_JPRB, 0.84814E+00_JPRB, 0.83660E+00_JPRB, 0.84697E+00_JPRB, 0.86868E+00_JPRB, 0.92633E+00_JPRB, 0.96257E+00_JPRB ,&
&  0.86839E+00_JPRB, 0.74628E+00_JPRB, 0.68875E+00_JPRB, 0.69101E+00_JPRB, 0.75116E+00_JPRB, 0.80788E+00_JPRB, 0.83926E+00_JPRB ,&
&  0.86284E+00_JPRB, 0.88831E+00_JPRB, 0.92643E+00_JPRB, 0.97825E+00_JPRB, 0.10027E+01_JPRB, 0.10269E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.11652E+01_JPRB, 0.12532E+01_JPRB, 0.13997E+01_JPRB, 0.14557E+01_JPRB, 0.17987E+01_JPRB, 0.23849E+01_JPRB ,&
&  0.27138E+01_JPRB, 0.29985E+01_JPRB, 0.34315E+01_JPRB, 0.39766E+01_JPRB, 0.42099E+01_JPRB, 0.39473E+01_JPRB, 0.36341E+01_JPRB ,&
&  0.34397E+01_JPRB, 0.30556E+01_JPRB, 0.33846E+01_JPRB, 0.31398E+01_JPRB, 0.29459E+01_JPRB, 0.29448E+01_JPRB, 0.28088E+01_JPRB ,&
&  0.27691E+01_JPRB, 0.28316E+01_JPRB, 0.27247E+01_JPRB, 0.26296E+01_JPRB, 0.24116E+01_JPRB, 0.23595E+01_JPRB, 0.21913E+01_JPRB ,&
&  0.22097E+01_JPRB, 0.21738E+01_JPRB, 0.18496E+01_JPRB, 0.15594E+01_JPRB, 0.12914E+01_JPRB, 0.12421E+01_JPRB, 0.11375E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.97317E+00_JPRB, 0.94131E+00_JPRB, 0.88972E+00_JPRB, 0.86235E+00_JPRB, 0.89399E+00_JPRB, 0.92031E+00_JPRB ,&
&  0.10355E+01_JPRB, 0.12407E+01_JPRB, 0.13306E+01_JPRB, 0.13737E+01_JPRB, 0.14200E+01_JPRB, 0.14663E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.11432E+01_JPRB, 0.11210E+01_JPRB, 0.10841E+01_JPRB, 0.10525E+01_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.11777E+01_JPRB, 0.11231E+01_JPRB, 0.10729E+01_JPRB, 0.10410E+01_JPRB, 0.10221E+01_JPRB, 0.10116E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10282E+01_JPRB, 0.10023E+01_JPRB, 0.98167E+00_JPRB, 0.96101E+00_JPRB, 0.96420E+00_JPRB, 0.98831E+00_JPRB ,&
&  0.86481E+00_JPRB, 0.96944E+00_JPRB, 0.84097E+00_JPRB, 0.89299E+00_JPRB, 0.12130E+01_JPRB, 0.10612E+01_JPRB, 0.10175E+01_JPRB ,&
&  0.94112E+00_JPRB, 0.93840E+00_JPRB, 0.93252E+00_JPRB, 0.10142E+01_JPRB, 0.10803E+01_JPRB, 0.11718E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.10532E+01_JPRB, 0.86862E+00_JPRB, 0.78164E+00_JPRB, 0.75546E+00_JPRB, 0.79631E+00_JPRB, 0.83429E+00_JPRB, 0.91377E+00_JPRB ,&
&  0.95117E+00_JPRB, 0.97482E+00_JPRB, 0.99697E+00_JPRB, 0.10356E+01_JPRB, 0.10594E+01_JPRB, 0.10928E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.12099E+01_JPRB, 0.12952E+01_JPRB, 0.13822E+01_JPRB, 0.15260E+01_JPRB, 0.16089E+01_JPRB, 0.19169E+01_JPRB, 0.25538E+01_JPRB ,&
&  0.29990E+01_JPRB, 0.32492E+01_JPRB, 0.39517E+01_JPRB, 0.52054E+01_JPRB, 0.56070E+01_JPRB, 0.53921E+01_JPRB, 0.47886E+01_JPRB ,&
&  0.41774E+01_JPRB, 0.35866E+01_JPRB, 0.34780E+01_JPRB, 0.32824E+01_JPRB, 0.30389E+01_JPRB, 0.31502E+01_JPRB, 0.30583E+01_JPRB ,&
&  0.28993E+01_JPRB, 0.28675E+01_JPRB, 0.28175E+01_JPRB, 0.28475E+01_JPRB, 0.26945E+01_JPRB, 0.26987E+01_JPRB, 0.26079E+01_JPRB ,&
&  0.26332E+01_JPRB, 0.23987E+01_JPRB, 0.17896E+01_JPRB, 0.14936E+01_JPRB, 0.12220E+01_JPRB, 0.11521E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.12366E+01_JPRB, 0.11946E+01_JPRB, 0.11977E+01_JPRB, 0.11646E+01_JPRB, 0.12037E+01_JPRB, 0.14230E+01_JPRB, 0.13017E+01_JPRB ,&
&  0.12422E+01_JPRB, 0.14006E+01_JPRB, 0.16202E+01_JPRB, 0.17771E+01_JPRB, 0.18547E+01_JPRB, 0.17976E+01_JPRB, 0.14655E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.13389E+01_JPRB, 0.12995E+01_JPRB, 0.12473E+01_JPRB, 0.12103E+01_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.14109E+01_JPRB, 0.13443E+01_JPRB, 0.12683E+01_JPRB, 0.11804E+01_JPRB, 0.11290E+01_JPRB, 0.11146E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.11194E+01_JPRB, 0.10808E+01_JPRB, 0.10544E+01_JPRB, 0.10354E+01_JPRB, 0.10306E+01_JPRB, 0.10421E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.13022E+01_JPRB, 0.10825E+01_JPRB, 0.92368E+00_JPRB, 0.10466E+01_JPRB, 0.98797E+00_JPRB, 0.10247E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.11493E+01_JPRB, 0.11616E+01_JPRB, 0.12678E+01_JPRB, 0.14035E+01_JPRB, 0.16122E+01_JPRB, 0.16569E+01_JPRB ,&
&  0.15044E+01_JPRB, 0.12504E+01_JPRB, 0.10478E+01_JPRB, 0.99425E+00_JPRB, 0.10364E+01_JPRB, 0.96586E+00_JPRB, 0.10290E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.10768E+01_JPRB, 0.11029E+01_JPRB, 0.11222E+01_JPRB, 0.11636E+01_JPRB, 0.12263E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.14550E+01_JPRB, 0.14890E+01_JPRB, 0.15470E+01_JPRB, 0.17011E+01_JPRB, 0.19376E+01_JPRB, 0.21343E+01_JPRB ,&
&  0.26202E+01_JPRB, 0.30524E+01_JPRB, 0.37941E+01_JPRB, 0.50118E+01_JPRB, 0.56001E+01_JPRB, 0.57558E+01_JPRB, 0.55379E+01_JPRB ,&
&  0.50672E+01_JPRB, 0.45730E+01_JPRB, 0.44264E+01_JPRB, 0.39916E+01_JPRB, 0.37219E+01_JPRB, 0.36485E+01_JPRB, 0.35210E+01_JPRB ,&
&  0.32803E+01_JPRB, 0.31089E+01_JPRB, 0.30724E+01_JPRB, 0.29601E+01_JPRB, 0.28930E+01_JPRB, 0.27862E+01_JPRB, 0.28192E+01_JPRB ,&
&  0.31051E+01_JPRB, 0.20552E+01_JPRB, 0.15904E+01_JPRB, 0.16715E+01_JPRB, 0.13658E+01_JPRB, 0.12134E+01_JPRB, 0.13062E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.12923E+01_JPRB, 0.15485E+01_JPRB, 0.15619E+01_JPRB, 0.18239E+01_JPRB, 0.21852E+01_JPRB, 0.19528E+01_JPRB ,&
&  0.18979E+01_JPRB, 0.18630E+01_JPRB, 0.22209E+01_JPRB, 0.24858E+01_JPRB, 0.25033E+01_JPRB, 0.22857E+01_JPRB, 0.19855E+01_JPRB ,&
&  0.17485E+01_JPRB, 0.16239E+01_JPRB, 0.15644E+01_JPRB, 0.15133E+01_JPRB, 0.14742E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.16669E+01_JPRB, 0.15843E+01_JPRB, 0.15103E+01_JPRB, 0.14286E+01_JPRB, 0.13665E+01_JPRB, 0.13187E+01_JPRB, 0.12657E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.11414E+01_JPRB, 0.11109E+01_JPRB, 0.11004E+01_JPRB, 0.11006E+01_JPRB, 0.11144E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.13309E+01_JPRB, 0.10830E+01_JPRB, 0.92707E+00_JPRB, 0.82111E+00_JPRB, 0.87656E+00_JPRB, 0.95907E+00_JPRB ,&
&  0.11055E+01_JPRB, 0.12192E+01_JPRB, 0.14246E+01_JPRB, 0.17528E+01_JPRB, 0.20202E+01_JPRB, 0.22470E+01_JPRB, 0.23668E+01_JPRB ,&
&  0.21967E+01_JPRB, 0.20332E+01_JPRB, 0.16042E+01_JPRB, 0.14970E+01_JPRB, 0.13577E+01_JPRB, 0.12030E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.12281E+01_JPRB, 0.12359E+01_JPRB, 0.12751E+01_JPRB, 0.13160E+01_JPRB, 0.13543E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.14925E+01_JPRB, 0.15390E+01_JPRB, 0.15877E+01_JPRB, 0.17182E+01_JPRB, 0.19658E+01_JPRB, 0.21285E+01_JPRB ,&
&  0.25837E+01_JPRB, 0.27261E+01_JPRB, 0.29438E+01_JPRB, 0.44510E+01_JPRB, 0.63796E+01_JPRB, 0.64515E+01_JPRB, 0.59342E+01_JPRB ,&
&  0.58553E+01_JPRB, 0.58093E+01_JPRB, 0.59206E+01_JPRB, 0.57177E+01_JPRB, 0.51782E+01_JPRB, 0.48772E+01_JPRB, 0.45432E+01_JPRB ,&
&  0.40969E+01_JPRB, 0.37928E+01_JPRB, 0.36277E+01_JPRB, 0.35973E+01_JPRB, 0.34558E+01_JPRB, 0.34205E+01_JPRB, 0.34501E+01_JPRB ,&
&  0.33775E+01_JPRB, 0.29226E+01_JPRB, 0.20064E+01_JPRB, 0.19471E+01_JPRB, 0.15471E+01_JPRB, 0.13881E+01_JPRB, 0.14497E+01_JPRB ,&
&  0.13319E+01_JPRB, 0.15725E+01_JPRB, 0.18786E+01_JPRB, 0.19341E+01_JPRB, 0.27241E+01_JPRB, 0.30567E+01_JPRB, 0.29960E+01_JPRB ,&
&  0.28260E+01_JPRB, 0.26261E+01_JPRB, 0.30656E+01_JPRB, 0.32239E+01_JPRB, 0.31768E+01_JPRB, 0.29391E+01_JPRB, 0.26000E+01_JPRB ,&
&  0.22478E+01_JPRB, 0.20324E+01_JPRB, 0.19238E+01_JPRB, 0.18488E+01_JPRB, 0.17552E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.19255E+01_JPRB, 0.18179E+01_JPRB, 0.17136E+01_JPRB, 0.16319E+01_JPRB, 0.15604E+01_JPRB, 0.14809E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.12800E+01_JPRB, 0.12305E+01_JPRB, 0.12099E+01_JPRB, 0.11865E+01_JPRB, 0.11707E+01_JPRB, 0.11707E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.12003E+01_JPRB, 0.12293E+01_JPRB, 0.99108E+00_JPRB, 0.81951E+00_JPRB, 0.72043E+00_JPRB, 0.75625E+00_JPRB, 0.88075E+00_JPRB ,&
&  0.10612E+01_JPRB, 0.13827E+01_JPRB, 0.18965E+01_JPRB, 0.23137E+01_JPRB, 0.27465E+01_JPRB, 0.31675E+01_JPRB, 0.32978E+01_JPRB ,&
&  0.35390E+01_JPRB, 0.28314E+01_JPRB, 0.25574E+01_JPRB, 0.21277E+01_JPRB, 0.18533E+01_JPRB, 0.16865E+01_JPRB, 0.15378E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.13460E+01_JPRB, 0.13201E+01_JPRB, 0.13445E+01_JPRB, 0.13718E+01_JPRB, 0.14226E+01_JPRB, 0.15088E+01_JPRB ,&
&  0.16100E+01_JPRB, 0.16670E+01_JPRB, 0.16426E+01_JPRB, 0.16507E+01_JPRB, 0.19778E+01_JPRB, 0.24205E+01_JPRB, 0.26751E+01_JPRB ,&
&  0.27063E+01_JPRB, 0.30445E+01_JPRB, 0.51792E+01_JPRB, 0.60379E+01_JPRB, 0.64936E+01_JPRB, 0.71912E+01_JPRB, 0.72810E+01_JPRB ,&
&  0.67188E+01_JPRB, 0.61574E+01_JPRB, 0.58932E+01_JPRB, 0.52056E+01_JPRB, 0.57884E+01_JPRB, 0.57217E+01_JPRB, 0.53889E+01_JPRB ,&
&  0.48829E+01_JPRB, 0.45689E+01_JPRB, 0.42480E+01_JPRB, 0.40646E+01_JPRB, 0.40102E+01_JPRB, 0.40776E+01_JPRB, 0.38261E+01_JPRB ,&
&  0.33556E+01_JPRB, 0.35746E+01_JPRB, 0.30312E+01_JPRB, 0.24542E+01_JPRB, 0.20187E+01_JPRB, 0.18916E+01_JPRB, 0.17968E+01_JPRB ,&
&  0.16917E+01_JPRB, 0.18146E+01_JPRB, 0.23269E+01_JPRB, 0.29502E+01_JPRB, 0.38298E+01_JPRB, 0.42915E+01_JPRB, 0.41069E+01_JPRB ,&
&  0.36965E+01_JPRB, 0.39535E+01_JPRB, 0.38555E+01_JPRB, 0.36215E+01_JPRB, 0.34423E+01_JPRB, 0.31522E+01_JPRB, 0.28864E+01_JPRB ,&
&  0.26061E+01_JPRB, 0.24192E+01_JPRB, 0.22877E+01_JPRB, 0.21711E+01_JPRB, 0.20477E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.20636E+01_JPRB, 0.19488E+01_JPRB, 0.18323E+01_JPRB, 0.17276E+01_JPRB, 0.16198E+01_JPRB, 0.15261E+01_JPRB, 0.14261E+01_JPRB ,&
&  0.13546E+01_JPRB, 0.13168E+01_JPRB, 0.12722E+01_JPRB, 0.12327E+01_JPRB, 0.12080E+01_JPRB, 0.11892E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.11099E+01_JPRB, 0.90728E+00_JPRB, 0.84444E+00_JPRB, 0.82398E+00_JPRB, 0.78347E+00_JPRB, 0.86759E+00_JPRB ,&
&  0.11049E+01_JPRB, 0.16022E+01_JPRB, 0.22128E+01_JPRB, 0.29471E+01_JPRB, 0.37366E+01_JPRB, 0.43586E+01_JPRB, 0.44623E+01_JPRB ,&
&  0.39691E+01_JPRB, 0.39014E+01_JPRB, 0.34273E+01_JPRB, 0.29968E+01_JPRB, 0.25489E+01_JPRB, 0.22177E+01_JPRB, 0.19610E+01_JPRB ,&
&  0.17449E+01_JPRB, 0.16199E+01_JPRB, 0.15737E+01_JPRB, 0.15744E+01_JPRB, 0.16154E+01_JPRB, 0.16819E+01_JPRB, 0.17765E+01_JPRB ,&
&  0.18159E+01_JPRB, 0.18247E+01_JPRB, 0.17630E+01_JPRB, 0.18325E+01_JPRB, 0.20839E+01_JPRB, 0.27554E+01_JPRB, 0.29480E+01_JPRB ,&
&  0.35865E+01_JPRB, 0.41721E+01_JPRB, 0.44169E+01_JPRB, 0.55946E+01_JPRB, 0.49729E+01_JPRB, 0.61221E+01_JPRB, 0.74837E+01_JPRB ,&
&  0.68518E+01_JPRB, 0.61303E+01_JPRB, 0.59488E+01_JPRB, 0.49652E+01_JPRB, 0.43742E+01_JPRB, 0.60205E+01_JPRB, 0.56621E+01_JPRB ,&
&  0.51881E+01_JPRB, 0.50931E+01_JPRB, 0.46844E+01_JPRB, 0.44718E+01_JPRB, 0.40305E+01_JPRB, 0.36612E+01_JPRB, 0.27923E+01_JPRB ,&
&  0.28911E+01_JPRB, 0.26921E+01_JPRB, 0.28482E+01_JPRB, 0.25432E+01_JPRB, 0.23878E+01_JPRB, 0.22049E+01_JPRB, 0.20207E+01_JPRB ,&
&  0.19432E+01_JPRB, 0.20919E+01_JPRB, 0.34900E+01_JPRB, 0.53149E+01_JPRB, 0.61640E+01_JPRB, 0.52063E+01_JPRB, 0.55841E+01_JPRB ,&
&  0.52212E+01_JPRB, 0.44835E+01_JPRB, 0.36838E+01_JPRB, 0.33567E+01_JPRB, 0.30747E+01_JPRB, 0.28388E+01_JPRB, 0.26642E+01_JPRB ,&
&  0.25081E+01_JPRB, 0.23700E+01_JPRB, 0.22696E+01_JPRB, 0.22151E+01_JPRB, 0.21496E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.21068E+01_JPRB, 0.19887E+01_JPRB, 0.18618E+01_JPRB, 0.17548E+01_JPRB, 0.16523E+01_JPRB, 0.15594E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.14336E+01_JPRB, 0.13747E+01_JPRB, 0.12902E+01_JPRB, 0.12308E+01_JPRB, 0.12021E+01_JPRB, 0.12004E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11344E+01_JPRB, 0.95343E+00_JPRB, 0.88207E+00_JPRB, 0.96271E+00_JPRB, 0.91968E+00_JPRB, 0.94676E+00_JPRB ,&
&  0.13207E+01_JPRB, 0.20412E+01_JPRB, 0.27341E+01_JPRB, 0.35279E+01_JPRB, 0.46479E+01_JPRB, 0.52807E+01_JPRB, 0.51038E+01_JPRB ,&
&  0.51304E+01_JPRB, 0.44316E+01_JPRB, 0.39118E+01_JPRB, 0.34884E+01_JPRB, 0.30647E+01_JPRB, 0.26241E+01_JPRB, 0.22264E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.18597E+01_JPRB, 0.17887E+01_JPRB, 0.17237E+01_JPRB, 0.17600E+01_JPRB, 0.18221E+01_JPRB, 0.18865E+01_JPRB ,&
&  0.19047E+01_JPRB, 0.18871E+01_JPRB, 0.18338E+01_JPRB, 0.18640E+01_JPRB, 0.22940E+01_JPRB, 0.28471E+01_JPRB, 0.32483E+01_JPRB ,&
&  0.35041E+01_JPRB, 0.38030E+01_JPRB, 0.41682E+01_JPRB, 0.49152E+01_JPRB, 0.61323E+01_JPRB, 0.64785E+01_JPRB, 0.73817E+01_JPRB ,&
&  0.59328E+01_JPRB, 0.49761E+01_JPRB, 0.41335E+01_JPRB, 0.32337E+01_JPRB, 0.40923E+01_JPRB, 0.63821E+01_JPRB, 0.63512E+01_JPRB ,&
&  0.59106E+01_JPRB, 0.58240E+01_JPRB, 0.54602E+01_JPRB, 0.44771E+01_JPRB, 0.37553E+01_JPRB, 0.30510E+01_JPRB, 0.36225E+01_JPRB ,&
&  0.36043E+01_JPRB, 0.34875E+01_JPRB, 0.31148E+01_JPRB, 0.25736E+01_JPRB, 0.22640E+01_JPRB, 0.24997E+01_JPRB, 0.23564E+01_JPRB ,&
&  0.23076E+01_JPRB, 0.28707E+01_JPRB, 0.62663E+01_JPRB, 0.79946E+01_JPRB, 0.74215E+01_JPRB, 0.60694E+01_JPRB, 0.63970E+01_JPRB ,&
&  0.51831E+01_JPRB, 0.42284E+01_JPRB, 0.35254E+01_JPRB, 0.29991E+01_JPRB, 0.27142E+01_JPRB, 0.25561E+01_JPRB, 0.24842E+01_JPRB ,&
&  0.24430E+01_JPRB, 0.24128E+01_JPRB, 0.23366E+01_JPRB, 0.22514E+01_JPRB, 0.21782E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.21439E+01_JPRB, 0.20236E+01_JPRB, 0.19173E+01_JPRB, 0.18042E+01_JPRB, 0.16918E+01_JPRB, 0.16090E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.14722E+01_JPRB, 0.13950E+01_JPRB, 0.13205E+01_JPRB, 0.12742E+01_JPRB, 0.12543E+01_JPRB, 0.12428E+01_JPRB, 0.12244E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.12584E+01_JPRB, 0.10760E+01_JPRB, 0.10052E+01_JPRB, 0.10501E+01_JPRB, 0.10713E+01_JPRB, 0.11355E+01_JPRB ,&
&  0.17396E+01_JPRB, 0.25084E+01_JPRB, 0.28773E+01_JPRB, 0.35730E+01_JPRB, 0.50952E+01_JPRB, 0.48400E+01_JPRB, 0.48764E+01_JPRB ,&
&  0.49208E+01_JPRB, 0.42096E+01_JPRB, 0.37771E+01_JPRB, 0.33795E+01_JPRB, 0.30433E+01_JPRB, 0.27490E+01_JPRB, 0.24475E+01_JPRB ,&
&  0.21981E+01_JPRB, 0.20392E+01_JPRB, 0.19651E+01_JPRB, 0.19654E+01_JPRB, 0.19549E+01_JPRB, 0.19484E+01_JPRB, 0.19375E+01_JPRB ,&
&  0.19148E+01_JPRB, 0.18685E+01_JPRB, 0.17943E+01_JPRB, 0.19563E+01_JPRB, 0.24782E+01_JPRB, 0.29075E+01_JPRB, 0.38367E+01_JPRB ,&
&  0.37647E+01_JPRB, 0.40664E+01_JPRB, 0.41386E+01_JPRB, 0.56272E+01_JPRB, 0.65154E+01_JPRB, 0.67327E+01_JPRB, 0.71280E+01_JPRB ,&
&  0.55544E+01_JPRB, 0.47521E+01_JPRB, 0.44050E+01_JPRB, 0.41043E+01_JPRB, 0.40241E+01_JPRB, 0.46646E+01_JPRB, 0.62907E+01_JPRB ,&
&  0.50081E+01_JPRB, 0.50964E+01_JPRB, 0.50957E+01_JPRB, 0.46831E+01_JPRB, 0.26218E+01_JPRB, 0.19874E+01_JPRB, 0.30717E+01_JPRB ,&
&  0.31824E+01_JPRB, 0.19184E+01_JPRB, 0.13987E+01_JPRB, 0.17742E+01_JPRB, 0.14108E+01_JPRB, 0.17713E+01_JPRB, 0.27092E+01_JPRB ,&
&  0.32162E+01_JPRB, 0.44847E+01_JPRB, 0.79438E+01_JPRB, 0.73447E+01_JPRB, 0.66421E+01_JPRB, 0.64300E+01_JPRB, 0.56829E+01_JPRB ,&
&  0.46525E+01_JPRB, 0.36365E+01_JPRB, 0.34112E+01_JPRB, 0.29322E+01_JPRB, 0.26732E+01_JPRB, 0.25384E+01_JPRB, 0.25463E+01_JPRB ,&
&  0.26336E+01_JPRB, 0.26464E+01_JPRB, 0.25686E+01_JPRB, 0.24166E+01_JPRB, 0.22629E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.21995E+01_JPRB, 0.20898E+01_JPRB, 0.19825E+01_JPRB, 0.18761E+01_JPRB, 0.17789E+01_JPRB, 0.16967E+01_JPRB, 0.15897E+01_JPRB ,&
&  0.14571E+01_JPRB, 0.13756E+01_JPRB, 0.13434E+01_JPRB, 0.13126E+01_JPRB, 0.12948E+01_JPRB, 0.12947E+01_JPRB, 0.12770E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.12752E+01_JPRB, 0.13179E+01_JPRB, 0.12478E+01_JPRB, 0.12088E+01_JPRB, 0.11512E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.22285E+01_JPRB, 0.29486E+01_JPRB, 0.31845E+01_JPRB, 0.34422E+01_JPRB, 0.41566E+01_JPRB, 0.42164E+01_JPRB, 0.47642E+01_JPRB ,&
&  0.43934E+01_JPRB, 0.38780E+01_JPRB, 0.34372E+01_JPRB, 0.30776E+01_JPRB, 0.28093E+01_JPRB, 0.25880E+01_JPRB, 0.24396E+01_JPRB ,&
&  0.22749E+01_JPRB, 0.21121E+01_JPRB, 0.20471E+01_JPRB, 0.19459E+01_JPRB, 0.18758E+01_JPRB, 0.18354E+01_JPRB, 0.18760E+01_JPRB ,&
&  0.18626E+01_JPRB, 0.18264E+01_JPRB, 0.18075E+01_JPRB, 0.20588E+01_JPRB, 0.28046E+01_JPRB, 0.33294E+01_JPRB, 0.34237E+01_JPRB ,&
&  0.35017E+01_JPRB, 0.37221E+01_JPRB, 0.46172E+01_JPRB, 0.54769E+01_JPRB, 0.61110E+01_JPRB, 0.66136E+01_JPRB, 0.64903E+01_JPRB ,&
&  0.62004E+01_JPRB, 0.57450E+01_JPRB, 0.46044E+01_JPRB, 0.43679E+01_JPRB, 0.42431E+01_JPRB, 0.38098E+01_JPRB, 0.44811E+01_JPRB ,&
&  0.47091E+01_JPRB, 0.41979E+01_JPRB, 0.33606E+01_JPRB, 0.27152E+01_JPRB, 0.27799E+01_JPRB, 0.20383E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.12225E+01_JPRB, 0.11080E+01_JPRB, 0.99706E+00_JPRB, 0.10600E+01_JPRB, 0.10854E+01_JPRB, 0.16663E+01_JPRB, 0.29374E+01_JPRB ,&
&  0.46120E+01_JPRB, 0.60697E+01_JPRB, 0.72809E+01_JPRB, 0.63735E+01_JPRB, 0.54434E+01_JPRB, 0.55439E+01_JPRB, 0.53666E+01_JPRB ,&
&  0.37107E+01_JPRB, 0.32077E+01_JPRB, 0.31144E+01_JPRB, 0.27969E+01_JPRB, 0.25978E+01_JPRB, 0.25816E+01_JPRB, 0.27245E+01_JPRB ,&
&  0.28699E+01_JPRB, 0.28656E+01_JPRB, 0.27439E+01_JPRB, 0.25266E+01_JPRB, 0.23327E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.21644E+01_JPRB, 0.20286E+01_JPRB, 0.19369E+01_JPRB, 0.18770E+01_JPRB, 0.18162E+01_JPRB, 0.17133E+01_JPRB, 0.15333E+01_JPRB ,&
&  0.13996E+01_JPRB, 0.13488E+01_JPRB, 0.13400E+01_JPRB, 0.13206E+01_JPRB, 0.13003E+01_JPRB, 0.12922E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.12527E+01_JPRB, 0.13769E+01_JPRB, 0.13783E+01_JPRB, 0.13738E+01_JPRB, 0.13049E+01_JPRB, 0.16782E+01_JPRB ,&
&  0.26068E+01_JPRB, 0.31953E+01_JPRB, 0.33516E+01_JPRB, 0.32664E+01_JPRB, 0.39286E+01_JPRB, 0.43478E+01_JPRB, 0.43480E+01_JPRB ,&
&  0.37035E+01_JPRB, 0.31411E+01_JPRB, 0.26093E+01_JPRB, 0.23226E+01_JPRB, 0.22889E+01_JPRB, 0.22789E+01_JPRB, 0.22310E+01_JPRB ,&
&  0.20968E+01_JPRB, 0.19983E+01_JPRB, 0.18914E+01_JPRB, 0.17996E+01_JPRB, 0.17632E+01_JPRB, 0.17828E+01_JPRB, 0.18368E+01_JPRB ,&
&  0.18536E+01_JPRB, 0.18532E+01_JPRB, 0.19690E+01_JPRB, 0.23911E+01_JPRB, 0.28445E+01_JPRB, 0.29397E+01_JPRB, 0.34437E+01_JPRB ,&
&  0.40113E+01_JPRB, 0.44986E+01_JPRB, 0.44971E+01_JPRB, 0.56558E+01_JPRB, 0.59684E+01_JPRB, 0.59713E+01_JPRB, 0.60018E+01_JPRB ,&
&  0.57914E+01_JPRB, 0.58138E+01_JPRB, 0.44092E+01_JPRB, 0.39626E+01_JPRB, 0.44639E+01_JPRB, 0.39932E+01_JPRB, 0.39734E+01_JPRB ,&
&  0.44678E+01_JPRB, 0.38935E+01_JPRB, 0.33108E+01_JPRB, 0.30180E+01_JPRB, 0.40938E+01_JPRB, 0.38238E+01_JPRB, 0.16045E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.10836E+01_JPRB, 0.94269E+00_JPRB, 0.90992E+00_JPRB, 0.10640E+01_JPRB, 0.17817E+01_JPRB, 0.39404E+01_JPRB ,&
&  0.47164E+01_JPRB, 0.67454E+01_JPRB, 0.68866E+01_JPRB, 0.61917E+01_JPRB, 0.50618E+01_JPRB, 0.46673E+01_JPRB, 0.41359E+01_JPRB ,&
&  0.36466E+01_JPRB, 0.31729E+01_JPRB, 0.27059E+01_JPRB, 0.24178E+01_JPRB, 0.23242E+01_JPRB, 0.24258E+01_JPRB, 0.26680E+01_JPRB ,&
&  0.27954E+01_JPRB, 0.28080E+01_JPRB, 0.27076E+01_JPRB, 0.25233E+01_JPRB, 0.23188E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.20453E+01_JPRB, 0.19828E+01_JPRB, 0.19436E+01_JPRB, 0.18694E+01_JPRB, 0.17717E+01_JPRB, 0.16422E+01_JPRB, 0.14765E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.13602E+01_JPRB, 0.13541E+01_JPRB, 0.13169E+01_JPRB, 0.12732E+01_JPRB, 0.12509E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.12255E+01_JPRB, 0.12089E+01_JPRB, 0.12722E+01_JPRB, 0.13692E+01_JPRB, 0.14272E+01_JPRB, 0.14820E+01_JPRB, 0.19289E+01_JPRB ,&
&  0.28499E+01_JPRB, 0.32600E+01_JPRB, 0.33409E+01_JPRB, 0.33285E+01_JPRB, 0.33857E+01_JPRB, 0.35672E+01_JPRB, 0.31075E+01_JPRB ,&
&  0.26118E+01_JPRB, 0.22496E+01_JPRB, 0.19785E+01_JPRB, 0.19731E+01_JPRB, 0.20776E+01_JPRB, 0.21461E+01_JPRB, 0.20946E+01_JPRB ,&
&  0.20277E+01_JPRB, 0.19086E+01_JPRB, 0.18335E+01_JPRB, 0.17648E+01_JPRB, 0.17613E+01_JPRB, 0.18113E+01_JPRB, 0.18661E+01_JPRB ,&
&  0.19161E+01_JPRB, 0.19934E+01_JPRB, 0.22321E+01_JPRB, 0.26192E+01_JPRB, 0.25414E+01_JPRB, 0.31671E+01_JPRB, 0.36017E+01_JPRB ,&
&  0.39928E+01_JPRB, 0.42311E+01_JPRB, 0.42607E+01_JPRB, 0.49990E+01_JPRB, 0.56077E+01_JPRB, 0.56855E+01_JPRB, 0.53555E+01_JPRB ,&
&  0.53653E+01_JPRB, 0.46269E+01_JPRB, 0.40140E+01_JPRB, 0.36850E+01_JPRB, 0.38807E+01_JPRB, 0.47862E+01_JPRB, 0.35805E+01_JPRB ,&
&  0.34727E+01_JPRB, 0.40508E+01_JPRB, 0.36921E+01_JPRB, 0.35876E+01_JPRB, 0.42171E+01_JPRB, 0.49042E+01_JPRB, 0.37710E+01_JPRB ,&
&  0.14871E+01_JPRB, 0.10718E+01_JPRB, 0.11068E+01_JPRB, 0.10701E+01_JPRB, 0.10184E+01_JPRB, 0.18618E+01_JPRB, 0.73915E+01_JPRB ,&
&  0.60058E+01_JPRB, 0.79049E+01_JPRB, 0.73884E+01_JPRB, 0.61804E+01_JPRB, 0.52570E+01_JPRB, 0.42246E+01_JPRB, 0.36731E+01_JPRB ,&
&  0.29677E+01_JPRB, 0.25407E+01_JPRB, 0.23199E+01_JPRB, 0.22075E+01_JPRB, 0.22125E+01_JPRB, 0.22831E+01_JPRB, 0.24579E+01_JPRB ,&
&  0.25682E+01_JPRB, 0.25851E+01_JPRB, 0.24803E+01_JPRB, 0.23170E+01_JPRB, 0.21598E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.19634E+01_JPRB, 0.19330E+01_JPRB, 0.18797E+01_JPRB, 0.17744E+01_JPRB, 0.16696E+01_JPRB, 0.15289E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.13646E+01_JPRB, 0.13382E+01_JPRB, 0.12963E+01_JPRB, 0.12454E+01_JPRB, 0.12118E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.11743E+01_JPRB, 0.11931E+01_JPRB, 0.13010E+01_JPRB, 0.15384E+01_JPRB, 0.17245E+01_JPRB, 0.19743E+01_JPRB ,&
&  0.30546E+01_JPRB, 0.32570E+01_JPRB, 0.25545E+01_JPRB, 0.25632E+01_JPRB, 0.27159E+01_JPRB, 0.28985E+01_JPRB, 0.22307E+01_JPRB ,&
&  0.19852E+01_JPRB, 0.18725E+01_JPRB, 0.17621E+01_JPRB, 0.18236E+01_JPRB, 0.19180E+01_JPRB, 0.20005E+01_JPRB, 0.20135E+01_JPRB ,&
&  0.19284E+01_JPRB, 0.18775E+01_JPRB, 0.18673E+01_JPRB, 0.18421E+01_JPRB, 0.18311E+01_JPRB, 0.18881E+01_JPRB, 0.19541E+01_JPRB ,&
&  0.20415E+01_JPRB, 0.21870E+01_JPRB, 0.24719E+01_JPRB, 0.26276E+01_JPRB, 0.28820E+01_JPRB, 0.32366E+01_JPRB, 0.36598E+01_JPRB ,&
&  0.37782E+01_JPRB, 0.40323E+01_JPRB, 0.43334E+01_JPRB, 0.46211E+01_JPRB, 0.49557E+01_JPRB, 0.52211E+01_JPRB, 0.49291E+01_JPRB ,&
&  0.47882E+01_JPRB, 0.41320E+01_JPRB, 0.35675E+01_JPRB, 0.33882E+01_JPRB, 0.35777E+01_JPRB, 0.44357E+01_JPRB, 0.44094E+01_JPRB ,&
&  0.42369E+01_JPRB, 0.33710E+01_JPRB, 0.33799E+01_JPRB, 0.40508E+01_JPRB, 0.40632E+01_JPRB, 0.43852E+01_JPRB, 0.57083E+01_JPRB ,&
&  0.50739E+01_JPRB, 0.26422E+01_JPRB, 0.21525E+01_JPRB, 0.27884E+01_JPRB, 0.18606E+01_JPRB, 0.19631E+01_JPRB, 0.42318E+01_JPRB ,&
&  0.58186E+01_JPRB, 0.70942E+01_JPRB, 0.67268E+01_JPRB, 0.55792E+01_JPRB, 0.49258E+01_JPRB, 0.36364E+01_JPRB, 0.28476E+01_JPRB ,&
&  0.22852E+01_JPRB, 0.20940E+01_JPRB, 0.19997E+01_JPRB, 0.19033E+01_JPRB, 0.18827E+01_JPRB, 0.19893E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.22529E+01_JPRB, 0.22839E+01_JPRB, 0.22429E+01_JPRB, 0.21432E+01_JPRB, 0.20306E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.18578E+01_JPRB, 0.18362E+01_JPRB, 0.17959E+01_JPRB, 0.16875E+01_JPRB, 0.15746E+01_JPRB, 0.14556E+01_JPRB, 0.14163E+01_JPRB ,&
&  0.13806E+01_JPRB, 0.13482E+01_JPRB, 0.13130E+01_JPRB, 0.12548E+01_JPRB, 0.12219E+01_JPRB, 0.11925E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.12121E+01_JPRB, 0.11851E+01_JPRB, 0.12332E+01_JPRB, 0.14715E+01_JPRB, 0.18606E+01_JPRB, 0.26248E+01_JPRB, 0.21615E+01_JPRB ,&
&  0.28912E+01_JPRB, 0.33411E+01_JPRB, 0.24515E+01_JPRB, 0.23493E+01_JPRB, 0.23677E+01_JPRB, 0.23727E+01_JPRB, 0.19995E+01_JPRB ,&
&  0.18335E+01_JPRB, 0.17910E+01_JPRB, 0.17787E+01_JPRB, 0.17892E+01_JPRB, 0.18483E+01_JPRB, 0.18987E+01_JPRB, 0.18561E+01_JPRB ,&
&  0.18558E+01_JPRB, 0.18806E+01_JPRB, 0.18858E+01_JPRB, 0.18804E+01_JPRB, 0.19078E+01_JPRB, 0.19715E+01_JPRB, 0.20529E+01_JPRB ,&
&  0.21910E+01_JPRB, 0.23877E+01_JPRB, 0.26758E+01_JPRB, 0.27271E+01_JPRB, 0.29443E+01_JPRB, 0.33431E+01_JPRB, 0.35728E+01_JPRB ,&
&  0.35230E+01_JPRB, 0.34581E+01_JPRB, 0.39356E+01_JPRB, 0.43664E+01_JPRB, 0.45301E+01_JPRB, 0.45592E+01_JPRB, 0.43410E+01_JPRB ,&
&  0.43895E+01_JPRB, 0.41422E+01_JPRB, 0.40828E+01_JPRB, 0.35168E+01_JPRB, 0.36672E+01_JPRB, 0.42400E+01_JPRB, 0.41704E+01_JPRB ,&
&  0.47036E+01_JPRB, 0.44077E+01_JPRB, 0.44283E+01_JPRB, 0.42550E+01_JPRB, 0.39691E+01_JPRB, 0.40448E+01_JPRB, 0.44202E+01_JPRB ,&
&  0.46739E+01_JPRB, 0.51756E+01_JPRB, 0.49123E+01_JPRB, 0.30357E+01_JPRB, 0.23466E+01_JPRB, 0.24929E+01_JPRB, 0.33309E+01_JPRB ,&
&  0.50115E+01_JPRB, 0.53392E+01_JPRB, 0.49271E+01_JPRB, 0.56865E+01_JPRB, 0.36918E+01_JPRB, 0.25802E+01_JPRB, 0.20292E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.17344E+01_JPRB, 0.16758E+01_JPRB, 0.15751E+01_JPRB, 0.15664E+01_JPRB, 0.16507E+01_JPRB, 0.17771E+01_JPRB ,&
&  0.19083E+01_JPRB, 0.19932E+01_JPRB, 0.20293E+01_JPRB, 0.19728E+01_JPRB, 0.19015E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.16962E+01_JPRB, 0.16951E+01_JPRB, 0.16359E+01_JPRB, 0.15503E+01_JPRB, 0.14777E+01_JPRB, 0.14173E+01_JPRB, 0.14089E+01_JPRB ,&
&  0.13244E+01_JPRB, 0.12827E+01_JPRB, 0.12496E+01_JPRB, 0.12119E+01_JPRB, 0.12160E+01_JPRB, 0.12306E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.13059E+01_JPRB, 0.13712E+01_JPRB, 0.15634E+01_JPRB, 0.19733E+01_JPRB, 0.24966E+01_JPRB, 0.31962E+01_JPRB, 0.31482E+01_JPRB ,&
&  0.27683E+01_JPRB, 0.35703E+01_JPRB, 0.23637E+01_JPRB, 0.23821E+01_JPRB, 0.23911E+01_JPRB, 0.25617E+01_JPRB, 0.22721E+01_JPRB ,&
&  0.19780E+01_JPRB, 0.19072E+01_JPRB, 0.18953E+01_JPRB, 0.18151E+01_JPRB, 0.17980E+01_JPRB, 0.17737E+01_JPRB, 0.17773E+01_JPRB ,&
&  0.18205E+01_JPRB, 0.18682E+01_JPRB, 0.18981E+01_JPRB, 0.19393E+01_JPRB, 0.20095E+01_JPRB, 0.20860E+01_JPRB, 0.22188E+01_JPRB ,&
&  0.24230E+01_JPRB, 0.26441E+01_JPRB, 0.27398E+01_JPRB, 0.28286E+01_JPRB, 0.29487E+01_JPRB, 0.32363E+01_JPRB, 0.34587E+01_JPRB ,&
&  0.34905E+01_JPRB, 0.34362E+01_JPRB, 0.37529E+01_JPRB, 0.38560E+01_JPRB, 0.38718E+01_JPRB, 0.41596E+01_JPRB, 0.38974E+01_JPRB ,&
&  0.41311E+01_JPRB, 0.38030E+01_JPRB, 0.42224E+01_JPRB, 0.36549E+01_JPRB, 0.36940E+01_JPRB, 0.39295E+01_JPRB, 0.34921E+01_JPRB ,&
&  0.34777E+01_JPRB, 0.40054E+01_JPRB, 0.41557E+01_JPRB, 0.41689E+01_JPRB, 0.40910E+01_JPRB, 0.39777E+01_JPRB, 0.37631E+01_JPRB ,&
&  0.33286E+01_JPRB, 0.32716E+01_JPRB, 0.29227E+01_JPRB, 0.19425E+01_JPRB, 0.16710E+01_JPRB, 0.21095E+01_JPRB, 0.25943E+01_JPRB ,&
&  0.36018E+01_JPRB, 0.39674E+01_JPRB, 0.43337E+01_JPRB, 0.35305E+01_JPRB, 0.22578E+01_JPRB, 0.17169E+01_JPRB, 0.15926E+01_JPRB ,&
&  0.15406E+01_JPRB, 0.14860E+01_JPRB, 0.13975E+01_JPRB, 0.13177E+01_JPRB, 0.13299E+01_JPRB, 0.14217E+01_JPRB, 0.15327E+01_JPRB ,&
&  0.16487E+01_JPRB, 0.17230E+01_JPRB, 0.17296E+01_JPRB, 0.17004E+01_JPRB, 0.16986E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.16267E+01_JPRB, 0.16274E+01_JPRB, 0.16176E+01_JPRB, 0.16260E+01_JPRB, 0.16466E+01_JPRB, 0.16874E+01_JPRB, 0.16595E+01_JPRB ,&
&  0.14800E+01_JPRB, 0.12117E+01_JPRB, 0.12148E+01_JPRB, 0.12402E+01_JPRB, 0.13163E+01_JPRB, 0.13885E+01_JPRB, 0.14856E+01_JPRB ,&
&  0.16333E+01_JPRB, 0.18308E+01_JPRB, 0.21542E+01_JPRB, 0.26031E+01_JPRB, 0.31332E+01_JPRB, 0.37834E+01_JPRB, 0.38464E+01_JPRB ,&
&  0.34079E+01_JPRB, 0.26304E+01_JPRB, 0.25167E+01_JPRB, 0.24439E+01_JPRB, 0.23034E+01_JPRB, 0.24866E+01_JPRB, 0.24363E+01_JPRB ,&
&  0.21292E+01_JPRB, 0.20109E+01_JPRB, 0.19772E+01_JPRB, 0.18877E+01_JPRB, 0.18009E+01_JPRB, 0.18152E+01_JPRB, 0.18654E+01_JPRB ,&
&  0.19268E+01_JPRB, 0.19727E+01_JPRB, 0.19988E+01_JPRB, 0.20463E+01_JPRB, 0.21450E+01_JPRB, 0.22421E+01_JPRB, 0.24229E+01_JPRB ,&
&  0.26292E+01_JPRB, 0.28024E+01_JPRB, 0.28612E+01_JPRB, 0.28003E+01_JPRB, 0.28836E+01_JPRB, 0.30586E+01_JPRB, 0.32282E+01_JPRB ,&
&  0.33601E+01_JPRB, 0.35266E+01_JPRB, 0.36267E+01_JPRB, 0.36647E+01_JPRB, 0.34574E+01_JPRB, 0.38301E+01_JPRB, 0.36508E+01_JPRB ,&
&  0.37406E+01_JPRB, 0.35616E+01_JPRB, 0.39822E+01_JPRB, 0.33199E+01_JPRB, 0.35403E+01_JPRB, 0.34685E+01_JPRB, 0.32920E+01_JPRB ,&
&  0.33460E+01_JPRB, 0.38848E+01_JPRB, 0.40663E+01_JPRB, 0.40187E+01_JPRB, 0.40397E+01_JPRB, 0.35317E+01_JPRB, 0.36433E+01_JPRB ,&
&  0.28644E+01_JPRB, 0.28870E+01_JPRB, 0.19901E+01_JPRB, 0.14944E+01_JPRB, 0.12528E+01_JPRB, 0.15813E+01_JPRB, 0.21873E+01_JPRB ,&
&  0.29721E+01_JPRB, 0.29458E+01_JPRB, 0.24311E+01_JPRB, 0.21182E+01_JPRB, 0.17444E+01_JPRB, 0.14929E+01_JPRB, 0.14462E+01_JPRB ,&
&  0.14234E+01_JPRB, 0.13465E+01_JPRB, 0.12708E+01_JPRB, 0.12521E+01_JPRB, 0.12740E+01_JPRB, 0.13799E+01_JPRB, 0.14603E+01_JPRB ,&
&  0.15430E+01_JPRB, 0.15692E+01_JPRB, 0.15742E+01_JPRB, 0.15872E+01_JPRB, 0.16010E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.15658E+01_JPRB, 0.15358E+01_JPRB, 0.15236E+01_JPRB, 0.15062E+01_JPRB, 0.14787E+01_JPRB, 0.13988E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.12600E+01_JPRB, 0.12062E+01_JPRB, 0.12829E+01_JPRB, 0.13900E+01_JPRB, 0.15238E+01_JPRB, 0.16682E+01_JPRB, 0.18328E+01_JPRB ,&
&  0.20367E+01_JPRB, 0.22855E+01_JPRB, 0.26013E+01_JPRB, 0.28743E+01_JPRB, 0.31502E+01_JPRB, 0.35216E+01_JPRB, 0.37814E+01_JPRB ,&
&  0.32665E+01_JPRB, 0.21470E+01_JPRB, 0.24378E+01_JPRB, 0.25374E+01_JPRB, 0.23650E+01_JPRB, 0.23825E+01_JPRB, 0.22966E+01_JPRB ,&
&  0.21301E+01_JPRB, 0.20728E+01_JPRB, 0.20544E+01_JPRB, 0.20163E+01_JPRB, 0.19212E+01_JPRB, 0.19534E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.20226E+01_JPRB, 0.20173E+01_JPRB, 0.20317E+01_JPRB, 0.21435E+01_JPRB, 0.23371E+01_JPRB, 0.24340E+01_JPRB, 0.25883E+01_JPRB ,&
&  0.27241E+01_JPRB, 0.29027E+01_JPRB, 0.29553E+01_JPRB, 0.28947E+01_JPRB, 0.29413E+01_JPRB, 0.30134E+01_JPRB, 0.31114E+01_JPRB ,&
&  0.32400E+01_JPRB, 0.33039E+01_JPRB, 0.34295E+01_JPRB, 0.36962E+01_JPRB, 0.38397E+01_JPRB, 0.34412E+01_JPRB, 0.35066E+01_JPRB ,&
&  0.35006E+01_JPRB, 0.34444E+01_JPRB, 0.33669E+01_JPRB, 0.41119E+01_JPRB, 0.31004E+01_JPRB, 0.31205E+01_JPRB, 0.28583E+01_JPRB ,&
&  0.32715E+01_JPRB, 0.35620E+01_JPRB, 0.37883E+01_JPRB, 0.38449E+01_JPRB, 0.37295E+01_JPRB, 0.33450E+01_JPRB, 0.34955E+01_JPRB ,&
&  0.30300E+01_JPRB, 0.26027E+01_JPRB, 0.18715E+01_JPRB, 0.13979E+01_JPRB, 0.14067E+01_JPRB, 0.23034E+01_JPRB, 0.25837E+01_JPRB ,&
&  0.27145E+01_JPRB, 0.21184E+01_JPRB, 0.19149E+01_JPRB, 0.20933E+01_JPRB, 0.17566E+01_JPRB, 0.14133E+01_JPRB, 0.13205E+01_JPRB ,&
&  0.13340E+01_JPRB, 0.13129E+01_JPRB, 0.12589E+01_JPRB, 0.12583E+01_JPRB, 0.13180E+01_JPRB, 0.14126E+01_JPRB, 0.14809E+01_JPRB ,&
&  0.15127E+01_JPRB, 0.15018E+01_JPRB, 0.15118E+01_JPRB, 0.15290E+01_JPRB, 0.15738E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.14262E+01_JPRB, 0.13684E+01_JPRB, 0.13215E+01_JPRB, 0.12733E+01_JPRB, 0.12204E+01_JPRB, 0.11811E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.12492E+01_JPRB, 0.13426E+01_JPRB, 0.14586E+01_JPRB, 0.16065E+01_JPRB, 0.17806E+01_JPRB, 0.19590E+01_JPRB ,&
&  0.21866E+01_JPRB, 0.24126E+01_JPRB, 0.25423E+01_JPRB, 0.26350E+01_JPRB, 0.27509E+01_JPRB, 0.29392E+01_JPRB, 0.31333E+01_JPRB ,&
&  0.32875E+01_JPRB, 0.32011E+01_JPRB, 0.28963E+01_JPRB, 0.21401E+01_JPRB, 0.18762E+01_JPRB, 0.21673E+01_JPRB, 0.21598E+01_JPRB ,&
&  0.21358E+01_JPRB, 0.20727E+01_JPRB, 0.20315E+01_JPRB, 0.19888E+01_JPRB, 0.19848E+01_JPRB, 0.20264E+01_JPRB, 0.20606E+01_JPRB ,&
&  0.20604E+01_JPRB, 0.20771E+01_JPRB, 0.21425E+01_JPRB, 0.22820E+01_JPRB, 0.24814E+01_JPRB, 0.25940E+01_JPRB, 0.27057E+01_JPRB ,&
&  0.27927E+01_JPRB, 0.29362E+01_JPRB, 0.29863E+01_JPRB, 0.29947E+01_JPRB, 0.29293E+01_JPRB, 0.29453E+01_JPRB, 0.30862E+01_JPRB ,&
&  0.32489E+01_JPRB, 0.33975E+01_JPRB, 0.35757E+01_JPRB, 0.38162E+01_JPRB, 0.39148E+01_JPRB, 0.34422E+01_JPRB, 0.32719E+01_JPRB ,&
&  0.33623E+01_JPRB, 0.34235E+01_JPRB, 0.31973E+01_JPRB, 0.41225E+01_JPRB, 0.26661E+01_JPRB, 0.29329E+01_JPRB, 0.29918E+01_JPRB ,&
&  0.29367E+01_JPRB, 0.30981E+01_JPRB, 0.33079E+01_JPRB, 0.34470E+01_JPRB, 0.32794E+01_JPRB, 0.29417E+01_JPRB, 0.33857E+01_JPRB ,&
&  0.31219E+01_JPRB, 0.23271E+01_JPRB, 0.17434E+01_JPRB, 0.12583E+01_JPRB, 0.15905E+01_JPRB, 0.26580E+01_JPRB, 0.22714E+01_JPRB ,&
&  0.17525E+01_JPRB, 0.17437E+01_JPRB, 0.19183E+01_JPRB, 0.21188E+01_JPRB, 0.18092E+01_JPRB, 0.12874E+01_JPRB, 0.12682E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.13123E+01_JPRB, 0.13343E+01_JPRB, 0.13577E+01_JPRB, 0.13986E+01_JPRB, 0.14550E+01_JPRB, 0.15071E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.15053E+01_JPRB, 0.15313E+01_JPRB, 0.15260E+01_JPRB, 0.14869E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.11733E+01_JPRB, 0.11043E+01_JPRB, 0.10533E+01_JPRB, 0.10327E+01_JPRB, 0.10301E+01_JPRB, 0.10407E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.12064E+01_JPRB, 0.12877E+01_JPRB, 0.13652E+01_JPRB, 0.14640E+01_JPRB, 0.15726E+01_JPRB, 0.17018E+01_JPRB ,&
&  0.18400E+01_JPRB, 0.19851E+01_JPRB, 0.20489E+01_JPRB, 0.20503E+01_JPRB, 0.20894E+01_JPRB, 0.21513E+01_JPRB, 0.23648E+01_JPRB ,&
&  0.27236E+01_JPRB, 0.30268E+01_JPRB, 0.31697E+01_JPRB, 0.29237E+01_JPRB, 0.22823E+01_JPRB, 0.22382E+01_JPRB, 0.24361E+01_JPRB ,&
&  0.24945E+01_JPRB, 0.22350E+01_JPRB, 0.20470E+01_JPRB, 0.20320E+01_JPRB, 0.20427E+01_JPRB, 0.20896E+01_JPRB, 0.21214E+01_JPRB ,&
&  0.21337E+01_JPRB, 0.21647E+01_JPRB, 0.22458E+01_JPRB, 0.23386E+01_JPRB, 0.24749E+01_JPRB, 0.26132E+01_JPRB, 0.27560E+01_JPRB ,&
&  0.28176E+01_JPRB, 0.28840E+01_JPRB, 0.28369E+01_JPRB, 0.27239E+01_JPRB, 0.27039E+01_JPRB, 0.28300E+01_JPRB, 0.30578E+01_JPRB ,&
&  0.32739E+01_JPRB, 0.33468E+01_JPRB, 0.35950E+01_JPRB, 0.37949E+01_JPRB, 0.36990E+01_JPRB, 0.34804E+01_JPRB, 0.34235E+01_JPRB ,&
&  0.34461E+01_JPRB, 0.37023E+01_JPRB, 0.27588E+01_JPRB, 0.33837E+01_JPRB, 0.34801E+01_JPRB, 0.30686E+01_JPRB, 0.26974E+01_JPRB ,&
&  0.25012E+01_JPRB, 0.23121E+01_JPRB, 0.20820E+01_JPRB, 0.22195E+01_JPRB, 0.23341E+01_JPRB, 0.25287E+01_JPRB, 0.29020E+01_JPRB ,&
&  0.27528E+01_JPRB, 0.20664E+01_JPRB, 0.14850E+01_JPRB, 0.12160E+01_JPRB, 0.14600E+01_JPRB, 0.20134E+01_JPRB, 0.17141E+01_JPRB ,&
&  0.14695E+01_JPRB, 0.16143E+01_JPRB, 0.17148E+01_JPRB, 0.20056E+01_JPRB, 0.20021E+01_JPRB, 0.11971E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.13055E+01_JPRB, 0.13208E+01_JPRB, 0.13364E+01_JPRB, 0.13485E+01_JPRB, 0.13822E+01_JPRB, 0.14148E+01_JPRB ,&
&  0.14207E+01_JPRB, 0.14130E+01_JPRB, 0.13831E+01_JPRB, 0.13262E+01_JPRB, 0.12438E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.91993E+00_JPRB, 0.90723E+00_JPRB, 0.89239E+00_JPRB, 0.90914E+00_JPRB, 0.92695E+00_JPRB, 0.92903E+00_JPRB, 0.94763E+00_JPRB ,&
&  0.97263E+00_JPRB, 0.99926E+00_JPRB, 0.10245E+01_JPRB, 0.10480E+01_JPRB, 0.11247E+01_JPRB, 0.12065E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.13423E+01_JPRB, 0.13753E+01_JPRB, 0.13682E+01_JPRB, 0.13266E+01_JPRB, 0.12891E+01_JPRB, 0.12089E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.11878E+01_JPRB, 0.15107E+01_JPRB, 0.20346E+01_JPRB, 0.25332E+01_JPRB, 0.29883E+01_JPRB, 0.25916E+01_JPRB, 0.27806E+01_JPRB ,&
&  0.24073E+01_JPRB, 0.19018E+01_JPRB, 0.18876E+01_JPRB, 0.19799E+01_JPRB, 0.20808E+01_JPRB, 0.20866E+01_JPRB, 0.21264E+01_JPRB ,&
&  0.21516E+01_JPRB, 0.21797E+01_JPRB, 0.22289E+01_JPRB, 0.23338E+01_JPRB, 0.24118E+01_JPRB, 0.24657E+01_JPRB, 0.25362E+01_JPRB ,&
&  0.25879E+01_JPRB, 0.25851E+01_JPRB, 0.25813E+01_JPRB, 0.23914E+01_JPRB, 0.25054E+01_JPRB, 0.27550E+01_JPRB, 0.29802E+01_JPRB ,&
&  0.31183E+01_JPRB, 0.31863E+01_JPRB, 0.34648E+01_JPRB, 0.35596E+01_JPRB, 0.35601E+01_JPRB, 0.35409E+01_JPRB, 0.36196E+01_JPRB ,&
&  0.36043E+01_JPRB, 0.32742E+01_JPRB, 0.24204E+01_JPRB, 0.27353E+01_JPRB, 0.25409E+01_JPRB, 0.21495E+01_JPRB, 0.22323E+01_JPRB ,&
&  0.20395E+01_JPRB, 0.18205E+01_JPRB, 0.15447E+01_JPRB, 0.15472E+01_JPRB, 0.17099E+01_JPRB, 0.19171E+01_JPRB, 0.23782E+01_JPRB ,&
&  0.20122E+01_JPRB, 0.15946E+01_JPRB, 0.12298E+01_JPRB, 0.11768E+01_JPRB, 0.12639E+01_JPRB, 0.14401E+01_JPRB, 0.15279E+01_JPRB ,&
&  0.14910E+01_JPRB, 0.14066E+01_JPRB, 0.14898E+01_JPRB, 0.16774E+01_JPRB, 0.12201E+01_JPRB, 0.10465E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.11191E+01_JPRB, 0.11236E+01_JPRB, 0.11166E+01_JPRB, 0.11290E+01_JPRB, 0.11552E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.10721E+01_JPRB, 0.10121E+01_JPRB, 0.96962E+00_JPRB, 0.93650E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.71015E+00_JPRB, 0.71044E+00_JPRB, 0.71352E+00_JPRB, 0.72408E+00_JPRB, 0.72693E+00_JPRB, 0.72518E+00_JPRB, 0.71161E+00_JPRB ,&
&  0.70176E+00_JPRB, 0.68610E+00_JPRB, 0.67868E+00_JPRB, 0.67705E+00_JPRB, 0.69983E+00_JPRB, 0.74387E+00_JPRB, 0.80246E+00_JPRB ,&
&  0.81560E+00_JPRB, 0.80897E+00_JPRB, 0.77080E+00_JPRB, 0.74126E+00_JPRB, 0.75513E+00_JPRB, 0.74877E+00_JPRB, 0.71764E+00_JPRB ,&
&  0.69536E+00_JPRB, 0.76468E+00_JPRB, 0.95339E+00_JPRB, 0.12378E+01_JPRB, 0.17461E+01_JPRB, 0.20419E+01_JPRB, 0.26252E+01_JPRB ,&
&  0.24936E+01_JPRB, 0.16983E+01_JPRB, 0.17549E+01_JPRB, 0.16770E+01_JPRB, 0.17799E+01_JPRB, 0.18352E+01_JPRB, 0.18529E+01_JPRB ,&
&  0.19011E+01_JPRB, 0.19444E+01_JPRB, 0.19872E+01_JPRB, 0.20363E+01_JPRB, 0.20603E+01_JPRB, 0.20412E+01_JPRB, 0.20242E+01_JPRB ,&
&  0.20016E+01_JPRB, 0.19526E+01_JPRB, 0.19706E+01_JPRB, 0.20660E+01_JPRB, 0.23257E+01_JPRB, 0.26190E+01_JPRB, 0.26573E+01_JPRB ,&
&  0.26917E+01_JPRB, 0.28569E+01_JPRB, 0.28041E+01_JPRB, 0.30010E+01_JPRB, 0.30619E+01_JPRB, 0.29898E+01_JPRB, 0.31959E+01_JPRB ,&
&  0.32849E+01_JPRB, 0.27765E+01_JPRB, 0.19123E+01_JPRB, 0.21295E+01_JPRB, 0.20677E+01_JPRB, 0.19555E+01_JPRB, 0.20225E+01_JPRB ,&
&  0.18796E+01_JPRB, 0.17232E+01_JPRB, 0.15634E+01_JPRB, 0.14789E+01_JPRB, 0.14993E+01_JPRB, 0.14461E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.13778E+01_JPRB, 0.13313E+01_JPRB, 0.12613E+01_JPRB, 0.12343E+01_JPRB, 0.13848E+01_JPRB, 0.16298E+01_JPRB, 0.14314E+01_JPRB ,&
&  0.14148E+01_JPRB, 0.14067E+01_JPRB, 0.16267E+01_JPRB, 0.17536E+01_JPRB, 0.10391E+01_JPRB, 0.84540E+00_JPRB, 0.80540E+00_JPRB ,&
&  0.81319E+00_JPRB, 0.80618E+00_JPRB, 0.79688E+00_JPRB, 0.78136E+00_JPRB, 0.81460E+00_JPRB, 0.87299E+00_JPRB, 0.85086E+00_JPRB ,&
&  0.84026E+00_JPRB, 0.78003E+00_JPRB, 0.75419E+00_JPRB, 0.74465E+00_JPRB, 0.73591E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.64575E+00_JPRB, 0.63621E+00_JPRB, 0.62692E+00_JPRB, 0.61714E+00_JPRB, 0.62560E+00_JPRB, 0.62926E+00_JPRB, 0.64171E+00_JPRB ,&
&  0.65569E+00_JPRB, 0.65841E+00_JPRB, 0.66734E+00_JPRB, 0.67112E+00_JPRB, 0.68964E+00_JPRB, 0.70530E+00_JPRB, 0.73648E+00_JPRB ,&
&  0.76674E+00_JPRB, 0.78318E+00_JPRB, 0.80312E+00_JPRB, 0.82312E+00_JPRB, 0.83502E+00_JPRB, 0.84850E+00_JPRB, 0.83248E+00_JPRB ,&
&  0.81418E+00_JPRB, 0.81521E+00_JPRB, 0.85134E+00_JPRB, 0.98922E+00_JPRB, 0.12515E+01_JPRB, 0.15910E+01_JPRB, 0.22692E+01_JPRB ,&
&  0.22775E+01_JPRB, 0.15382E+01_JPRB, 0.13467E+01_JPRB, 0.11892E+01_JPRB, 0.12368E+01_JPRB, 0.13164E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.12883E+01_JPRB, 0.12712E+01_JPRB, 0.12407E+01_JPRB, 0.11694E+01_JPRB, 0.10817E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.99102E+00_JPRB, 0.95973E+00_JPRB, 0.10837E+01_JPRB, 0.14131E+01_JPRB, 0.16727E+01_JPRB, 0.20986E+01_JPRB, 0.19806E+01_JPRB ,&
&  0.19972E+01_JPRB, 0.20529E+01_JPRB, 0.20305E+01_JPRB, 0.21796E+01_JPRB, 0.22460E+01_JPRB, 0.23306E+01_JPRB, 0.23442E+01_JPRB ,&
&  0.24761E+01_JPRB, 0.22162E+01_JPRB, 0.16448E+01_JPRB, 0.18965E+01_JPRB, 0.18807E+01_JPRB, 0.19221E+01_JPRB, 0.18398E+01_JPRB ,&
&  0.17356E+01_JPRB, 0.16153E+01_JPRB, 0.15143E+01_JPRB, 0.14402E+01_JPRB, 0.14403E+01_JPRB, 0.14132E+01_JPRB, 0.13043E+01_JPRB ,&
&  0.12859E+01_JPRB, 0.12881E+01_JPRB, 0.12663E+01_JPRB, 0.12295E+01_JPRB, 0.14681E+01_JPRB, 0.15617E+01_JPRB, 0.19913E+01_JPRB ,&
&  0.13821E+01_JPRB, 0.12676E+01_JPRB, 0.10056E+01_JPRB, 0.12578E+01_JPRB, 0.84216E+00_JPRB, 0.67776E+00_JPRB, 0.67180E+00_JPRB ,&
&  0.65022E+00_JPRB, 0.62591E+00_JPRB, 0.60547E+00_JPRB, 0.60077E+00_JPRB, 0.61333E+00_JPRB, 0.64638E+00_JPRB, 0.64175E+00_JPRB ,&
&  0.64286E+00_JPRB, 0.64540E+00_JPRB, 0.65643E+00_JPRB, 0.67344E+00_JPRB, 0.66529E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.66450E+00_JPRB, 0.66562E+00_JPRB, 0.67531E+00_JPRB, 0.69298E+00_JPRB, 0.70282E+00_JPRB, 0.71107E+00_JPRB, 0.71808E+00_JPRB ,&
&  0.73964E+00_JPRB, 0.76461E+00_JPRB, 0.78716E+00_JPRB, 0.80857E+00_JPRB, 0.82865E+00_JPRB, 0.84544E+00_JPRB, 0.88360E+00_JPRB ,&
&  0.90421E+00_JPRB, 0.93326E+00_JPRB, 0.95174E+00_JPRB, 0.97190E+00_JPRB, 0.98932E+00_JPRB, 0.98028E+00_JPRB, 0.98644E+00_JPRB ,&
&  0.99845E+00_JPRB, 0.10315E+01_JPRB, 0.10637E+01_JPRB, 0.11221E+01_JPRB, 0.11990E+01_JPRB, 0.12245E+01_JPRB, 0.14065E+01_JPRB ,&
&  0.13906E+01_JPRB, 0.14891E+01_JPRB, 0.12779E+01_JPRB, 0.10947E+01_JPRB, 0.10444E+01_JPRB, 0.93602E+00_JPRB, 0.87309E+00_JPRB ,&
&  0.87050E+00_JPRB, 0.85392E+00_JPRB, 0.81975E+00_JPRB, 0.77754E+00_JPRB, 0.73161E+00_JPRB, 0.76467E+00_JPRB, 0.80743E+00_JPRB ,&
&  0.93742E+00_JPRB, 0.10142E+01_JPRB, 0.11301E+01_JPRB, 0.12450E+01_JPRB, 0.13733E+01_JPRB, 0.14375E+01_JPRB, 0.14105E+01_JPRB ,&
&  0.14106E+01_JPRB, 0.13933E+01_JPRB, 0.13366E+01_JPRB, 0.15240E+01_JPRB, 0.16679E+01_JPRB, 0.17418E+01_JPRB, 0.17698E+01_JPRB ,&
&  0.17771E+01_JPRB, 0.16596E+01_JPRB, 0.16616E+01_JPRB, 0.16976E+01_JPRB, 0.16681E+01_JPRB, 0.17043E+01_JPRB, 0.16298E+01_JPRB ,&
&  0.15301E+01_JPRB, 0.14576E+01_JPRB, 0.14341E+01_JPRB, 0.14426E+01_JPRB, 0.14760E+01_JPRB, 0.14868E+01_JPRB, 0.13843E+01_JPRB ,&
&  0.13085E+01_JPRB, 0.12624E+01_JPRB, 0.12287E+01_JPRB, 0.12165E+01_JPRB, 0.12998E+01_JPRB, 0.25632E+01_JPRB, 0.33795E+01_JPRB ,&
&  0.13004E+01_JPRB, 0.93983E+00_JPRB, 0.60555E+00_JPRB, 0.74553E+00_JPRB, 0.66663E+00_JPRB, 0.65738E+00_JPRB, 0.72668E+00_JPRB ,&
&  0.73744E+00_JPRB, 0.72054E+00_JPRB, 0.65230E+00_JPRB, 0.58725E+00_JPRB, 0.60663E+00_JPRB, 0.61620E+00_JPRB, 0.61457E+00_JPRB ,&
&  0.62916E+00_JPRB, 0.63395E+00_JPRB, 0.64343E+00_JPRB, 0.66423E+00_JPRB, 0.65898E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.69404E+00_JPRB, 0.70901E+00_JPRB, 0.72883E+00_JPRB, 0.75540E+00_JPRB, 0.78099E+00_JPRB, 0.80119E+00_JPRB, 0.81514E+00_JPRB ,&
&  0.83819E+00_JPRB, 0.87008E+00_JPRB, 0.89075E+00_JPRB, 0.90083E+00_JPRB, 0.92158E+00_JPRB, 0.94004E+00_JPRB, 0.97218E+00_JPRB ,&
&  0.99219E+00_JPRB, 0.10374E+01_JPRB, 0.10735E+01_JPRB, 0.11087E+01_JPRB, 0.11373E+01_JPRB, 0.11706E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.12275E+01_JPRB, 0.12800E+01_JPRB, 0.13236E+01_JPRB, 0.13518E+01_JPRB, 0.14046E+01_JPRB, 0.14770E+01_JPRB, 0.12569E+01_JPRB ,&
&  0.14973E+01_JPRB, 0.13634E+01_JPRB, 0.11901E+01_JPRB, 0.10639E+01_JPRB, 0.97938E+00_JPRB, 0.91911E+00_JPRB, 0.88755E+00_JPRB ,&
&  0.89270E+00_JPRB, 0.93124E+00_JPRB, 0.96314E+00_JPRB, 0.94046E+00_JPRB, 0.89222E+00_JPRB, 0.96906E+00_JPRB, 0.11352E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.12883E+01_JPRB, 0.13320E+01_JPRB, 0.13371E+01_JPRB, 0.13896E+01_JPRB, 0.14599E+01_JPRB, 0.14868E+01_JPRB ,&
&  0.14814E+01_JPRB, 0.14322E+01_JPRB, 0.12982E+01_JPRB, 0.15384E+01_JPRB, 0.15534E+01_JPRB, 0.15123E+01_JPRB, 0.14065E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.12271E+01_JPRB, 0.13127E+01_JPRB, 0.14706E+01_JPRB, 0.14540E+01_JPRB, 0.14340E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.12473E+01_JPRB, 0.12767E+01_JPRB, 0.13391E+01_JPRB, 0.13909E+01_JPRB, 0.13959E+01_JPRB, 0.13089E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.11352E+01_JPRB, 0.10813E+01_JPRB, 0.10600E+01_JPRB, 0.11417E+01_JPRB, 0.13532E+01_JPRB, 0.19750E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.65704E+00_JPRB, 0.66872E+00_JPRB, 0.64183E+00_JPRB, 0.66430E+00_JPRB, 0.66385E+00_JPRB, 0.81997E+00_JPRB ,&
&  0.10281E+01_JPRB, 0.11736E+01_JPRB, 0.11472E+01_JPRB, 0.88235E+00_JPRB, 0.71489E+00_JPRB, 0.66452E+00_JPRB, 0.65685E+00_JPRB ,&
&  0.66661E+00_JPRB, 0.66455E+00_JPRB, 0.66771E+00_JPRB, 0.67397E+00_JPRB, 0.68461E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.67405E+00_JPRB, 0.68473E+00_JPRB, 0.69878E+00_JPRB, 0.71384E+00_JPRB, 0.73271E+00_JPRB, 0.75219E+00_JPRB, 0.76697E+00_JPRB ,&
&  0.78353E+00_JPRB, 0.80449E+00_JPRB, 0.82502E+00_JPRB, 0.83929E+00_JPRB, 0.85393E+00_JPRB, 0.88096E+00_JPRB, 0.91235E+00_JPRB ,&
&  0.92694E+00_JPRB, 0.97643E+00_JPRB, 0.10025E+01_JPRB, 0.10518E+01_JPRB, 0.10989E+01_JPRB, 0.11543E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.12992E+01_JPRB, 0.13254E+01_JPRB, 0.13177E+01_JPRB, 0.13409E+01_JPRB, 0.14617E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.14274E+01_JPRB, 0.13511E+01_JPRB, 0.12097E+01_JPRB, 0.11446E+01_JPRB, 0.10933E+01_JPRB, 0.10794E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.11256E+01_JPRB, 0.12139E+01_JPRB, 0.11793E+01_JPRB, 0.11950E+01_JPRB, 0.12276E+01_JPRB, 0.13560E+01_JPRB, 0.14880E+01_JPRB ,&
&  0.15254E+01_JPRB, 0.16195E+01_JPRB, 0.16569E+01_JPRB, 0.16987E+01_JPRB, 0.17773E+01_JPRB, 0.18548E+01_JPRB, 0.19058E+01_JPRB ,&
&  0.18847E+01_JPRB, 0.18486E+01_JPRB, 0.17203E+01_JPRB, 0.18201E+01_JPRB, 0.17476E+01_JPRB, 0.14989E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.10038E+01_JPRB, 0.10627E+01_JPRB, 0.11419E+01_JPRB, 0.12520E+01_JPRB, 0.12006E+01_JPRB, 0.11467E+01_JPRB, 0.10123E+01_JPRB ,&
&  0.96124E+00_JPRB, 0.95785E+00_JPRB, 0.10164E+01_JPRB, 0.10974E+01_JPRB, 0.11480E+01_JPRB, 0.11422E+01_JPRB, 0.10743E+01_JPRB ,&
&  0.10085E+01_JPRB, 0.92994E+00_JPRB, 0.89984E+00_JPRB, 0.91274E+00_JPRB, 0.93020E+00_JPRB, 0.91730E+00_JPRB, 0.12241E+01_JPRB ,&
&  0.90078E+00_JPRB, 0.79590E+00_JPRB, 0.92112E+00_JPRB, 0.49228E+00_JPRB, 0.66502E+00_JPRB, 0.83613E+00_JPRB, 0.11031E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.18193E+01_JPRB, 0.22561E+01_JPRB, 0.17168E+01_JPRB, 0.12000E+01_JPRB, 0.71398E+00_JPRB, 0.62131E+00_JPRB ,&
&  0.63106E+00_JPRB, 0.63613E+00_JPRB, 0.64547E+00_JPRB, 0.64640E+00_JPRB, 0.65991E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.62038E+00_JPRB, 0.61484E+00_JPRB, 0.61425E+00_JPRB, 0.60474E+00_JPRB, 0.60061E+00_JPRB, 0.60191E+00_JPRB, 0.59095E+00_JPRB ,&
&  0.59960E+00_JPRB, 0.61768E+00_JPRB, 0.62508E+00_JPRB, 0.64666E+00_JPRB, 0.66017E+00_JPRB, 0.70221E+00_JPRB, 0.76451E+00_JPRB ,&
&  0.80983E+00_JPRB, 0.90647E+00_JPRB, 0.95958E+00_JPRB, 0.10231E+01_JPRB, 0.10779E+01_JPRB, 0.11363E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.13379E+01_JPRB, 0.14259E+01_JPRB, 0.14884E+01_JPRB, 0.15029E+01_JPRB, 0.16812E+01_JPRB, 0.13189E+01_JPRB ,&
&  0.14477E+01_JPRB, 0.14313E+01_JPRB, 0.13377E+01_JPRB, 0.12868E+01_JPRB, 0.12362E+01_JPRB, 0.12073E+01_JPRB, 0.11908E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.12256E+01_JPRB, 0.12352E+01_JPRB, 0.13361E+01_JPRB, 0.14516E+01_JPRB, 0.15336E+01_JPRB, 0.16351E+01_JPRB ,&
&  0.17291E+01_JPRB, 0.18167E+01_JPRB, 0.19338E+01_JPRB, 0.20567E+01_JPRB, 0.21857E+01_JPRB, 0.22711E+01_JPRB, 0.23647E+01_JPRB ,&
&  0.24045E+01_JPRB, 0.24361E+01_JPRB, 0.22901E+01_JPRB, 0.20358E+01_JPRB, 0.17682E+01_JPRB, 0.14849E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.98480E+00_JPRB, 0.95017E+00_JPRB, 0.96629E+00_JPRB, 0.99970E+00_JPRB, 0.91706E+00_JPRB, 0.85628E+00_JPRB, 0.82101E+00_JPRB ,&
&  0.78298E+00_JPRB, 0.73057E+00_JPRB, 0.76569E+00_JPRB, 0.82687E+00_JPRB, 0.85040E+00_JPRB, 0.84293E+00_JPRB, 0.82510E+00_JPRB ,&
&  0.79454E+00_JPRB, 0.75258E+00_JPRB, 0.76332E+00_JPRB, 0.79223E+00_JPRB, 0.85724E+00_JPRB, 0.96623E+00_JPRB, 0.87092E+00_JPRB ,&
&  0.94487E+00_JPRB, 0.70025E+00_JPRB, 0.67978E+00_JPRB, 0.68686E+00_JPRB, 0.69983E+00_JPRB, 0.89174E+00_JPRB, 0.12289E+01_JPRB ,&
&  0.18342E+01_JPRB, 0.20655E+01_JPRB, 0.22964E+01_JPRB, 0.28412E+01_JPRB, 0.24730E+01_JPRB, 0.18381E+01_JPRB, 0.57060E+00_JPRB ,&
&  0.56879E+00_JPRB, 0.57361E+00_JPRB, 0.59941E+00_JPRB, 0.60835E+00_JPRB, 0.61309E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.52935E+00_JPRB, 0.51323E+00_JPRB, 0.50490E+00_JPRB, 0.49971E+00_JPRB, 0.48447E+00_JPRB, 0.48574E+00_JPRB, 0.48364E+00_JPRB ,&
&  0.50156E+00_JPRB, 0.52495E+00_JPRB, 0.56568E+00_JPRB, 0.61535E+00_JPRB, 0.68121E+00_JPRB, 0.73086E+00_JPRB, 0.79238E+00_JPRB ,&
&  0.83266E+00_JPRB, 0.88433E+00_JPRB, 0.94041E+00_JPRB, 0.99135E+00_JPRB, 0.10511E+01_JPRB, 0.10975E+01_JPRB, 0.11694E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.13128E+01_JPRB, 0.14136E+01_JPRB, 0.15028E+01_JPRB, 0.15741E+01_JPRB, 0.16129E+01_JPRB, 0.12627E+01_JPRB ,&
&  0.14828E+01_JPRB, 0.15062E+01_JPRB, 0.14993E+01_JPRB, 0.14374E+01_JPRB, 0.13509E+01_JPRB, 0.12731E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12503E+01_JPRB, 0.12408E+01_JPRB, 0.12644E+01_JPRB, 0.14891E+01_JPRB, 0.15665E+01_JPRB, 0.16635E+01_JPRB ,&
&  0.17680E+01_JPRB, 0.18846E+01_JPRB, 0.20049E+01_JPRB, 0.21313E+01_JPRB, 0.22221E+01_JPRB, 0.23256E+01_JPRB, 0.24371E+01_JPRB ,&
&  0.25122E+01_JPRB, 0.25761E+01_JPRB, 0.24822E+01_JPRB, 0.19860E+01_JPRB, 0.16899E+01_JPRB, 0.14456E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.90085E+00_JPRB, 0.85197E+00_JPRB, 0.86853E+00_JPRB, 0.87044E+00_JPRB, 0.80271E+00_JPRB, 0.73813E+00_JPRB, 0.69400E+00_JPRB ,&
&  0.62808E+00_JPRB, 0.61527E+00_JPRB, 0.60934E+00_JPRB, 0.65028E+00_JPRB, 0.67153E+00_JPRB, 0.64409E+00_JPRB, 0.63454E+00_JPRB ,&
&  0.61102E+00_JPRB, 0.59143E+00_JPRB, 0.63744E+00_JPRB, 0.71589E+00_JPRB, 0.88495E+00_JPRB, 0.10702E+01_JPRB, 0.11982E+01_JPRB ,&
&  0.12193E+01_JPRB, 0.71265E+00_JPRB, 0.66688E+00_JPRB, 0.65376E+00_JPRB, 0.67040E+00_JPRB, 0.72005E+00_JPRB, 0.81288E+00_JPRB ,&
&  0.10198E+01_JPRB, 0.13839E+01_JPRB, 0.12137E+01_JPRB, 0.92792E+00_JPRB, 0.98577E+00_JPRB, 0.90899E+00_JPRB, 0.51746E+00_JPRB ,&
&  0.51933E+00_JPRB, 0.51699E+00_JPRB, 0.53329E+00_JPRB, 0.53829E+00_JPRB, 0.54093E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.46921E+00_JPRB, 0.45139E+00_JPRB, 0.43248E+00_JPRB, 0.42313E+00_JPRB, 0.41615E+00_JPRB, 0.40801E+00_JPRB, 0.41963E+00_JPRB ,&
&  0.44971E+00_JPRB, 0.47710E+00_JPRB, 0.52282E+00_JPRB, 0.59096E+00_JPRB, 0.65807E+00_JPRB, 0.69902E+00_JPRB, 0.74414E+00_JPRB ,&
&  0.78371E+00_JPRB, 0.82662E+00_JPRB, 0.86173E+00_JPRB, 0.89536E+00_JPRB, 0.93944E+00_JPRB, 0.99639E+00_JPRB, 0.10741E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.12614E+01_JPRB, 0.13774E+01_JPRB, 0.15193E+01_JPRB, 0.16551E+01_JPRB, 0.17103E+01_JPRB, 0.18022E+01_JPRB ,&
&  0.12745E+01_JPRB, 0.15529E+01_JPRB, 0.15458E+01_JPRB, 0.14958E+01_JPRB, 0.14325E+01_JPRB, 0.13751E+01_JPRB, 0.13427E+01_JPRB ,&
&  0.13228E+01_JPRB, 0.12352E+01_JPRB, 0.12175E+01_JPRB, 0.12325E+01_JPRB, 0.13617E+01_JPRB, 0.14558E+01_JPRB, 0.15521E+01_JPRB ,&
&  0.16516E+01_JPRB, 0.17651E+01_JPRB, 0.18834E+01_JPRB, 0.19989E+01_JPRB, 0.20933E+01_JPRB, 0.22046E+01_JPRB, 0.22841E+01_JPRB ,&
&  0.23786E+01_JPRB, 0.24550E+01_JPRB, 0.24817E+01_JPRB, 0.20594E+01_JPRB, 0.16964E+01_JPRB, 0.14912E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.89090E+00_JPRB, 0.77691E+00_JPRB, 0.79720E+00_JPRB, 0.82294E+00_JPRB, 0.77853E+00_JPRB, 0.72887E+00_JPRB, 0.66826E+00_JPRB ,&
&  0.62785E+00_JPRB, 0.56943E+00_JPRB, 0.54774E+00_JPRB, 0.56168E+00_JPRB, 0.59788E+00_JPRB, 0.57074E+00_JPRB, 0.53703E+00_JPRB ,&
&  0.51545E+00_JPRB, 0.50972E+00_JPRB, 0.56666E+00_JPRB, 0.63451E+00_JPRB, 0.72465E+00_JPRB, 0.77585E+00_JPRB, 0.80258E+00_JPRB ,&
&  0.84743E+00_JPRB, 0.71225E+00_JPRB, 0.68610E+00_JPRB, 0.68785E+00_JPRB, 0.67971E+00_JPRB, 0.68179E+00_JPRB, 0.69571E+00_JPRB ,&
&  0.72963E+00_JPRB, 0.79379E+00_JPRB, 0.81064E+00_JPRB, 0.75271E+00_JPRB, 0.68362E+00_JPRB, 0.63278E+00_JPRB, 0.58615E+00_JPRB ,&
&  0.58810E+00_JPRB, 0.57442E+00_JPRB, 0.55948E+00_JPRB, 0.53488E+00_JPRB, 0.50353E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.48174E+00_JPRB, 0.45121E+00_JPRB, 0.40949E+00_JPRB, 0.37389E+00_JPRB, 0.39797E+00_JPRB, 0.39968E+00_JPRB, 0.41667E+00_JPRB ,&
&  0.42401E+00_JPRB, 0.45111E+00_JPRB, 0.47869E+00_JPRB, 0.51918E+00_JPRB, 0.57291E+00_JPRB, 0.62701E+00_JPRB, 0.67296E+00_JPRB ,&
&  0.71436E+00_JPRB, 0.75458E+00_JPRB, 0.78509E+00_JPRB, 0.79806E+00_JPRB, 0.81983E+00_JPRB, 0.85610E+00_JPRB, 0.90116E+00_JPRB ,&
&  0.96295E+00_JPRB, 0.10444E+01_JPRB, 0.11557E+01_JPRB, 0.12966E+01_JPRB, 0.14604E+01_JPRB, 0.15513E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.91487E+00_JPRB, 0.13732E+01_JPRB, 0.16157E+01_JPRB, 0.15201E+01_JPRB, 0.14758E+01_JPRB, 0.14399E+01_JPRB, 0.14046E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.11950E+01_JPRB, 0.11534E+01_JPRB, 0.12595E+01_JPRB, 0.12063E+01_JPRB, 0.13002E+01_JPRB, 0.13731E+01_JPRB ,&
&  0.14491E+01_JPRB, 0.15282E+01_JPRB, 0.16208E+01_JPRB, 0.17387E+01_JPRB, 0.18599E+01_JPRB, 0.19645E+01_JPRB, 0.20590E+01_JPRB ,&
&  0.21812E+01_JPRB, 0.22845E+01_JPRB, 0.23119E+01_JPRB, 0.18253E+01_JPRB, 0.16762E+01_JPRB, 0.15839E+01_JPRB, 0.14347E+01_JPRB ,&
&  0.98656E+00_JPRB, 0.78508E+00_JPRB, 0.78896E+00_JPRB, 0.81969E+00_JPRB, 0.78792E+00_JPRB, 0.75193E+00_JPRB, 0.70911E+00_JPRB ,&
&  0.65206E+00_JPRB, 0.60357E+00_JPRB, 0.57122E+00_JPRB, 0.55869E+00_JPRB, 0.57852E+00_JPRB, 0.57880E+00_JPRB, 0.55880E+00_JPRB ,&
&  0.53618E+00_JPRB, 0.54263E+00_JPRB, 0.58604E+00_JPRB, 0.62346E+00_JPRB, 0.67720E+00_JPRB, 0.71050E+00_JPRB, 0.74860E+00_JPRB ,&
&  0.76250E+00_JPRB, 0.71048E+00_JPRB, 0.71090E+00_JPRB, 0.72441E+00_JPRB, 0.72638E+00_JPRB, 0.72536E+00_JPRB, 0.71702E+00_JPRB ,&
&  0.70635E+00_JPRB, 0.71980E+00_JPRB, 0.72594E+00_JPRB, 0.73099E+00_JPRB, 0.71271E+00_JPRB, 0.69063E+00_JPRB, 0.67919E+00_JPRB ,&
&  0.65931E+00_JPRB, 0.63109E+00_JPRB, 0.60430E+00_JPRB, 0.56106E+00_JPRB, 0.51890E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.50611E+00_JPRB, 0.47081E+00_JPRB, 0.42914E+00_JPRB, 0.41313E+00_JPRB, 0.43846E+00_JPRB, 0.46063E+00_JPRB, 0.45679E+00_JPRB ,&
&  0.45109E+00_JPRB, 0.44135E+00_JPRB, 0.43694E+00_JPRB, 0.44739E+00_JPRB, 0.48663E+00_JPRB, 0.54374E+00_JPRB, 0.58906E+00_JPRB ,&
&  0.63128E+00_JPRB, 0.66451E+00_JPRB, 0.69075E+00_JPRB, 0.69932E+00_JPRB, 0.69418E+00_JPRB, 0.70719E+00_JPRB, 0.72950E+00_JPRB ,&
&  0.76920E+00_JPRB, 0.82600E+00_JPRB, 0.90222E+00_JPRB, 0.10154E+01_JPRB, 0.11773E+01_JPRB, 0.13758E+01_JPRB, 0.15790E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.56115E+00_JPRB, 0.14504E+01_JPRB, 0.17190E+01_JPRB, 0.16203E+01_JPRB, 0.16012E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.13305E+01_JPRB, 0.12341E+01_JPRB, 0.12663E+01_JPRB, 0.11660E+01_JPRB, 0.11719E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.12598E+01_JPRB, 0.13250E+01_JPRB, 0.14060E+01_JPRB, 0.15007E+01_JPRB, 0.16478E+01_JPRB, 0.17548E+01_JPRB ,&
&  0.18578E+01_JPRB, 0.19425E+01_JPRB, 0.19576E+01_JPRB, 0.14819E+01_JPRB, 0.12653E+01_JPRB, 0.11587E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.96635E+00_JPRB, 0.83295E+00_JPRB, 0.81198E+00_JPRB, 0.83364E+00_JPRB, 0.81233E+00_JPRB, 0.76188E+00_JPRB, 0.76222E+00_JPRB ,&
&  0.69947E+00_JPRB, 0.64996E+00_JPRB, 0.61231E+00_JPRB, 0.58357E+00_JPRB, 0.58999E+00_JPRB, 0.59839E+00_JPRB, 0.60356E+00_JPRB ,&
&  0.59298E+00_JPRB, 0.59533E+00_JPRB, 0.61717E+00_JPRB, 0.62516E+00_JPRB, 0.63635E+00_JPRB, 0.65092E+00_JPRB, 0.66471E+00_JPRB ,&
&  0.70999E+00_JPRB, 0.71597E+00_JPRB, 0.73143E+00_JPRB, 0.75620E+00_JPRB, 0.76919E+00_JPRB, 0.75094E+00_JPRB, 0.74923E+00_JPRB ,&
&  0.75302E+00_JPRB, 0.75755E+00_JPRB, 0.74369E+00_JPRB, 0.75221E+00_JPRB, 0.73777E+00_JPRB, 0.72730E+00_JPRB, 0.72911E+00_JPRB ,&
&  0.73814E+00_JPRB, 0.72966E+00_JPRB, 0.68841E+00_JPRB, 0.60110E+00_JPRB, 0.55090E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.54283E+00_JPRB, 0.48817E+00_JPRB, 0.44998E+00_JPRB, 0.45124E+00_JPRB, 0.47407E+00_JPRB, 0.48188E+00_JPRB, 0.48216E+00_JPRB ,&
&  0.47254E+00_JPRB, 0.46027E+00_JPRB, 0.44177E+00_JPRB, 0.43328E+00_JPRB, 0.43870E+00_JPRB, 0.45303E+00_JPRB, 0.49326E+00_JPRB ,&
&  0.53338E+00_JPRB, 0.57093E+00_JPRB, 0.60129E+00_JPRB, 0.60322E+00_JPRB, 0.58437E+00_JPRB, 0.58235E+00_JPRB, 0.59365E+00_JPRB ,&
&  0.60820E+00_JPRB, 0.64140E+00_JPRB, 0.68782E+00_JPRB, 0.75354E+00_JPRB, 0.85651E+00_JPRB, 0.10325E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.18070E+01_JPRB, 0.12440E+01_JPRB, 0.82071E+00_JPRB, 0.15672E+01_JPRB, 0.18845E+01_JPRB, 0.19568E+01_JPRB, 0.16431E+01_JPRB ,&
&  0.15834E+01_JPRB, 0.14834E+01_JPRB, 0.13791E+01_JPRB, 0.13616E+01_JPRB, 0.12066E+01_JPRB, 0.11515E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10836E+01_JPRB, 0.11266E+01_JPRB, 0.11674E+01_JPRB, 0.12077E+01_JPRB, 0.13076E+01_JPRB, 0.14349E+01_JPRB ,&
&  0.15658E+01_JPRB, 0.16406E+01_JPRB, 0.16600E+01_JPRB, 0.13039E+01_JPRB, 0.11468E+01_JPRB, 0.10757E+01_JPRB, 0.10374E+01_JPRB ,&
&  0.95468E+00_JPRB, 0.91567E+00_JPRB, 0.90531E+00_JPRB, 0.87074E+00_JPRB, 0.83163E+00_JPRB, 0.79727E+00_JPRB, 0.80910E+00_JPRB ,&
&  0.75916E+00_JPRB, 0.71067E+00_JPRB, 0.67644E+00_JPRB, 0.65600E+00_JPRB, 0.65115E+00_JPRB, 0.65289E+00_JPRB, 0.64738E+00_JPRB ,&
&  0.65415E+00_JPRB, 0.65849E+00_JPRB, 0.66652E+00_JPRB, 0.65807E+00_JPRB, 0.64898E+00_JPRB, 0.64750E+00_JPRB, 0.64003E+00_JPRB ,&
&  0.67494E+00_JPRB, 0.71806E+00_JPRB, 0.74347E+00_JPRB, 0.77611E+00_JPRB, 0.78130E+00_JPRB, 0.77561E+00_JPRB, 0.79611E+00_JPRB ,&
&  0.79367E+00_JPRB, 0.80325E+00_JPRB, 0.77416E+00_JPRB, 0.78379E+00_JPRB, 0.76046E+00_JPRB, 0.73869E+00_JPRB, 0.73233E+00_JPRB ,&
&  0.76575E+00_JPRB, 0.80947E+00_JPRB, 0.79167E+00_JPRB, 0.63233E+00_JPRB, 0.57465E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.52878E+00_JPRB, 0.50736E+00_JPRB, 0.46710E+00_JPRB, 0.47160E+00_JPRB, 0.47516E+00_JPRB, 0.48338E+00_JPRB, 0.48392E+00_JPRB ,&
&  0.48381E+00_JPRB, 0.48511E+00_JPRB, 0.48735E+00_JPRB, 0.47822E+00_JPRB, 0.45323E+00_JPRB, 0.44233E+00_JPRB, 0.44485E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.48532E+00_JPRB, 0.51843E+00_JPRB, 0.52402E+00_JPRB, 0.52284E+00_JPRB, 0.51733E+00_JPRB, 0.51372E+00_JPRB ,&
&  0.51823E+00_JPRB, 0.52817E+00_JPRB, 0.54351E+00_JPRB, 0.57771E+00_JPRB, 0.63897E+00_JPRB, 0.73559E+00_JPRB, 0.90337E+00_JPRB ,&
&  0.13285E+01_JPRB, 0.14218E+01_JPRB, 0.67444E+00_JPRB, 0.12914E+01_JPRB, 0.19410E+01_JPRB, 0.20720E+01_JPRB, 0.19855E+01_JPRB ,&
&  0.18834E+01_JPRB, 0.16847E+01_JPRB, 0.15457E+01_JPRB, 0.14026E+01_JPRB, 0.12144E+01_JPRB, 0.11138E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.96115E+00_JPRB, 0.95672E+00_JPRB, 0.99582E+00_JPRB, 0.10291E+01_JPRB, 0.10344E+01_JPRB, 0.10615E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.12955E+01_JPRB, 0.14425E+01_JPRB, 0.15043E+01_JPRB, 0.12403E+01_JPRB, 0.11262E+01_JPRB, 0.11379E+01_JPRB, 0.11107E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.99886E+00_JPRB, 0.95213E+00_JPRB, 0.90526E+00_JPRB, 0.85551E+00_JPRB, 0.83933E+00_JPRB, 0.85464E+00_JPRB ,&
&  0.81788E+00_JPRB, 0.76602E+00_JPRB, 0.73106E+00_JPRB, 0.71052E+00_JPRB, 0.69694E+00_JPRB, 0.69095E+00_JPRB, 0.68371E+00_JPRB ,&
&  0.68210E+00_JPRB, 0.69226E+00_JPRB, 0.70468E+00_JPRB, 0.70186E+00_JPRB, 0.68717E+00_JPRB, 0.66756E+00_JPRB, 0.65208E+00_JPRB ,&
&  0.66427E+00_JPRB, 0.69525E+00_JPRB, 0.74327E+00_JPRB, 0.76295E+00_JPRB, 0.78014E+00_JPRB, 0.80362E+00_JPRB, 0.80938E+00_JPRB ,&
&  0.82050E+00_JPRB, 0.84473E+00_JPRB, 0.78667E+00_JPRB, 0.77739E+00_JPRB, 0.79368E+00_JPRB, 0.75507E+00_JPRB, 0.71607E+00_JPRB ,&
&  0.73840E+00_JPRB, 0.80257E+00_JPRB, 0.90743E+00_JPRB, 0.63713E+00_JPRB, 0.57161E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.52267E+00_JPRB, 0.49270E+00_JPRB, 0.48505E+00_JPRB, 0.48290E+00_JPRB, 0.48583E+00_JPRB, 0.49451E+00_JPRB, 0.50400E+00_JPRB ,&
&  0.50560E+00_JPRB, 0.50566E+00_JPRB, 0.50588E+00_JPRB, 0.50187E+00_JPRB, 0.48936E+00_JPRB, 0.47643E+00_JPRB, 0.46369E+00_JPRB ,&
&  0.46001E+00_JPRB, 0.46197E+00_JPRB, 0.47582E+00_JPRB, 0.48358E+00_JPRB, 0.48441E+00_JPRB, 0.48177E+00_JPRB, 0.48271E+00_JPRB ,&
&  0.48669E+00_JPRB, 0.48671E+00_JPRB, 0.48599E+00_JPRB, 0.50047E+00_JPRB, 0.53048E+00_JPRB, 0.58978E+00_JPRB, 0.69547E+00_JPRB ,&
&  0.99340E+00_JPRB, 0.12557E+01_JPRB, 0.12879E+01_JPRB, 0.19917E+01_JPRB, 0.24166E+01_JPRB, 0.23551E+01_JPRB, 0.22839E+01_JPRB ,&
&  0.21373E+01_JPRB, 0.19165E+01_JPRB, 0.18123E+01_JPRB, 0.14880E+01_JPRB, 0.12592E+01_JPRB, 0.10918E+01_JPRB, 0.96760E+00_JPRB ,&
&  0.87018E+00_JPRB, 0.82791E+00_JPRB, 0.84598E+00_JPRB, 0.89077E+00_JPRB, 0.92033E+00_JPRB, 0.92951E+00_JPRB, 0.96665E+00_JPRB ,&
&  0.10657E+01_JPRB, 0.12503E+01_JPRB, 0.13976E+01_JPRB, 0.13005E+01_JPRB, 0.11096E+01_JPRB, 0.11986E+01_JPRB, 0.12937E+01_JPRB ,&
&  0.14302E+01_JPRB, 0.11655E+01_JPRB, 0.10034E+01_JPRB, 0.93472E+00_JPRB, 0.87559E+00_JPRB, 0.92629E+00_JPRB, 0.89827E+00_JPRB ,&
&  0.85987E+00_JPRB, 0.81572E+00_JPRB, 0.77483E+00_JPRB, 0.74894E+00_JPRB, 0.72908E+00_JPRB, 0.71600E+00_JPRB, 0.70655E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.70281E+00_JPRB, 0.71463E+00_JPRB, 0.72073E+00_JPRB, 0.71878E+00_JPRB, 0.70392E+00_JPRB, 0.68190E+00_JPRB ,&
&  0.66756E+00_JPRB, 0.67905E+00_JPRB, 0.69020E+00_JPRB, 0.71709E+00_JPRB, 0.74632E+00_JPRB, 0.76093E+00_JPRB, 0.74619E+00_JPRB ,&
&  0.74617E+00_JPRB, 0.77608E+00_JPRB, 0.79097E+00_JPRB, 0.80713E+00_JPRB, 0.80810E+00_JPRB, 0.79321E+00_JPRB, 0.71447E+00_JPRB ,&
&  0.67502E+00_JPRB, 0.64946E+00_JPRB, 0.62264E+00_JPRB, 0.59716E+00_JPRB, 0.55522E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.52491E+00_JPRB, 0.50571E+00_JPRB, 0.49940E+00_JPRB, 0.50174E+00_JPRB, 0.50430E+00_JPRB, 0.50823E+00_JPRB, 0.51834E+00_JPRB ,&
&  0.52314E+00_JPRB, 0.52409E+00_JPRB, 0.52261E+00_JPRB, 0.52006E+00_JPRB, 0.51157E+00_JPRB, 0.50141E+00_JPRB, 0.48973E+00_JPRB ,&
&  0.48038E+00_JPRB, 0.45949E+00_JPRB, 0.44850E+00_JPRB, 0.45550E+00_JPRB, 0.45915E+00_JPRB, 0.45925E+00_JPRB, 0.46060E+00_JPRB ,&
&  0.46095E+00_JPRB, 0.46526E+00_JPRB, 0.46552E+00_JPRB, 0.47298E+00_JPRB, 0.48833E+00_JPRB, 0.52904E+00_JPRB, 0.59514E+00_JPRB ,&
&  0.79969E+00_JPRB, 0.11695E+01_JPRB, 0.11576E+01_JPRB, 0.23191E+01_JPRB, 0.25673E+01_JPRB, 0.24983E+01_JPRB, 0.22247E+01_JPRB ,&
&  0.20524E+01_JPRB, 0.20988E+01_JPRB, 0.16774E+01_JPRB, 0.13928E+01_JPRB, 0.12307E+01_JPRB, 0.10856E+01_JPRB, 0.93052E+00_JPRB ,&
&  0.81064E+00_JPRB, 0.74866E+00_JPRB, 0.72814E+00_JPRB, 0.74584E+00_JPRB, 0.78836E+00_JPRB, 0.82673E+00_JPRB, 0.85922E+00_JPRB ,&
&  0.91669E+00_JPRB, 0.10530E+01_JPRB, 0.12466E+01_JPRB, 0.12157E+01_JPRB, 0.11503E+01_JPRB, 0.12710E+01_JPRB, 0.15054E+01_JPRB ,&
&  0.17648E+01_JPRB, 0.15103E+01_JPRB, 0.11296E+01_JPRB, 0.98005E+00_JPRB, 0.93812E+00_JPRB, 0.96483E+00_JPRB, 0.95172E+00_JPRB ,&
&  0.90707E+00_JPRB, 0.86720E+00_JPRB, 0.81749E+00_JPRB, 0.78369E+00_JPRB, 0.75973E+00_JPRB, 0.73031E+00_JPRB, 0.71142E+00_JPRB ,&
&  0.70421E+00_JPRB, 0.70206E+00_JPRB, 0.70504E+00_JPRB, 0.70412E+00_JPRB, 0.70689E+00_JPRB, 0.70979E+00_JPRB, 0.70860E+00_JPRB ,&
&  0.69831E+00_JPRB, 0.67647E+00_JPRB, 0.65738E+00_JPRB, 0.66545E+00_JPRB, 0.69357E+00_JPRB, 0.70298E+00_JPRB, 0.70238E+00_JPRB ,&
&  0.73263E+00_JPRB, 0.75574E+00_JPRB, 0.76472E+00_JPRB, 0.76345E+00_JPRB, 0.79817E+00_JPRB, 0.85606E+00_JPRB, 0.74364E+00_JPRB ,&
&  0.69772E+00_JPRB, 0.66055E+00_JPRB, 0.63094E+00_JPRB, 0.60640E+00_JPRB, 0.56624E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.55359E+00_JPRB, 0.53859E+00_JPRB, 0.52809E+00_JPRB, 0.52398E+00_JPRB, 0.51711E+00_JPRB, 0.51605E+00_JPRB, 0.52344E+00_JPRB ,&
&  0.53663E+00_JPRB, 0.54118E+00_JPRB, 0.53995E+00_JPRB, 0.53758E+00_JPRB, 0.53134E+00_JPRB, 0.52197E+00_JPRB, 0.51005E+00_JPRB ,&
&  0.49952E+00_JPRB, 0.48382E+00_JPRB, 0.46310E+00_JPRB, 0.44760E+00_JPRB, 0.45046E+00_JPRB, 0.45566E+00_JPRB, 0.45668E+00_JPRB ,&
&  0.45617E+00_JPRB, 0.46242E+00_JPRB, 0.46492E+00_JPRB, 0.47492E+00_JPRB, 0.48289E+00_JPRB, 0.50716E+00_JPRB, 0.56891E+00_JPRB ,&
&  0.73412E+00_JPRB, 0.13334E+01_JPRB, 0.12219E+01_JPRB, 0.21395E+01_JPRB, 0.24130E+01_JPRB, 0.23253E+01_JPRB, 0.20382E+01_JPRB ,&
&  0.18862E+01_JPRB, 0.17735E+01_JPRB, 0.14927E+01_JPRB, 0.12580E+01_JPRB, 0.11070E+01_JPRB, 0.99158E+00_JPRB, 0.89135E+00_JPRB ,&
&  0.81368E+00_JPRB, 0.76443E+00_JPRB, 0.73379E+00_JPRB, 0.71346E+00_JPRB, 0.71792E+00_JPRB, 0.74252E+00_JPRB, 0.77853E+00_JPRB ,&
&  0.82172E+00_JPRB, 0.89296E+00_JPRB, 0.10619E+01_JPRB, 0.12220E+01_JPRB, 0.11405E+01_JPRB, 0.12647E+01_JPRB, 0.17143E+01_JPRB ,&
&  0.19800E+01_JPRB, 0.19805E+01_JPRB, 0.14094E+01_JPRB, 0.10977E+01_JPRB, 0.10013E+01_JPRB, 0.10109E+01_JPRB, 0.10074E+01_JPRB ,&
&  0.96709E+00_JPRB, 0.93472E+00_JPRB, 0.88145E+00_JPRB, 0.82406E+00_JPRB, 0.78096E+00_JPRB, 0.74893E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.68999E+00_JPRB, 0.68105E+00_JPRB, 0.67572E+00_JPRB, 0.66968E+00_JPRB, 0.66838E+00_JPRB, 0.67459E+00_JPRB, 0.68395E+00_JPRB ,&
&  0.69342E+00_JPRB, 0.68810E+00_JPRB, 0.65918E+00_JPRB, 0.63978E+00_JPRB, 0.64292E+00_JPRB, 0.65397E+00_JPRB, 0.65766E+00_JPRB ,&
&  0.70329E+00_JPRB, 0.72934E+00_JPRB, 0.73511E+00_JPRB, 0.76627E+00_JPRB, 0.79808E+00_JPRB, 0.93265E+00_JPRB, 0.78314E+00_JPRB ,&
&  0.71165E+00_JPRB, 0.66897E+00_JPRB, 0.64663E+00_JPRB, 0.62589E+00_JPRB, 0.59012E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.58146E+00_JPRB, 0.56255E+00_JPRB, 0.55197E+00_JPRB, 0.53770E+00_JPRB, 0.52537E+00_JPRB, 0.52027E+00_JPRB, 0.52505E+00_JPRB ,&
&  0.54031E+00_JPRB, 0.55369E+00_JPRB, 0.54886E+00_JPRB, 0.54632E+00_JPRB, 0.54434E+00_JPRB, 0.53618E+00_JPRB, 0.52438E+00_JPRB ,&
&  0.51258E+00_JPRB, 0.50142E+00_JPRB, 0.48743E+00_JPRB, 0.47172E+00_JPRB, 0.46084E+00_JPRB, 0.46091E+00_JPRB, 0.45921E+00_JPRB ,&
&  0.45770E+00_JPRB, 0.46178E+00_JPRB, 0.46611E+00_JPRB, 0.47382E+00_JPRB, 0.48179E+00_JPRB, 0.49838E+00_JPRB, 0.55472E+00_JPRB ,&
&  0.75105E+00_JPRB, 0.59000E+00_JPRB, 0.14601E+01_JPRB, 0.18918E+01_JPRB, 0.19568E+01_JPRB, 0.18244E+01_JPRB, 0.17444E+01_JPRB ,&
&  0.16596E+01_JPRB, 0.14900E+01_JPRB, 0.13378E+01_JPRB, 0.12042E+01_JPRB, 0.10411E+01_JPRB, 0.93404E+00_JPRB, 0.86942E+00_JPRB ,&
&  0.82086E+00_JPRB, 0.77840E+00_JPRB, 0.74247E+00_JPRB, 0.70885E+00_JPRB, 0.68493E+00_JPRB, 0.68232E+00_JPRB, 0.70320E+00_JPRB ,&
&  0.73275E+00_JPRB, 0.77573E+00_JPRB, 0.89504E+00_JPRB, 0.10558E+01_JPRB, 0.11284E+01_JPRB, 0.12309E+01_JPRB, 0.14612E+01_JPRB ,&
&  0.18421E+01_JPRB, 0.20243E+01_JPRB, 0.16231E+01_JPRB, 0.12948E+01_JPRB, 0.11146E+01_JPRB, 0.10704E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.98844E+00_JPRB, 0.92715E+00_JPRB, 0.85793E+00_JPRB, 0.79846E+00_JPRB, 0.75948E+00_JPRB, 0.73110E+00_JPRB ,&
&  0.70210E+00_JPRB, 0.67435E+00_JPRB, 0.65673E+00_JPRB, 0.64625E+00_JPRB, 0.64088E+00_JPRB, 0.64018E+00_JPRB, 0.64760E+00_JPRB ,&
&  0.66429E+00_JPRB, 0.67554E+00_JPRB, 0.67484E+00_JPRB, 0.64928E+00_JPRB, 0.64073E+00_JPRB, 0.64720E+00_JPRB, 0.65810E+00_JPRB ,&
&  0.70209E+00_JPRB, 0.71811E+00_JPRB, 0.72994E+00_JPRB, 0.77006E+00_JPRB, 0.76623E+00_JPRB, 0.82588E+00_JPRB, 0.76330E+00_JPRB ,&
&  0.70237E+00_JPRB, 0.67096E+00_JPRB, 0.65863E+00_JPRB, 0.64103E+00_JPRB, 0.61209E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.60471E+00_JPRB, 0.58543E+00_JPRB, 0.57162E+00_JPRB, 0.54927E+00_JPRB, 0.53612E+00_JPRB, 0.53060E+00_JPRB, 0.53455E+00_JPRB ,&
&  0.54106E+00_JPRB, 0.54806E+00_JPRB, 0.54459E+00_JPRB, 0.54058E+00_JPRB, 0.53391E+00_JPRB, 0.52879E+00_JPRB, 0.52040E+00_JPRB ,&
&  0.51552E+00_JPRB, 0.51087E+00_JPRB, 0.50340E+00_JPRB, 0.49379E+00_JPRB, 0.48105E+00_JPRB, 0.47512E+00_JPRB, 0.46850E+00_JPRB ,&
&  0.46247E+00_JPRB, 0.45995E+00_JPRB, 0.46140E+00_JPRB, 0.46989E+00_JPRB, 0.48128E+00_JPRB, 0.49694E+00_JPRB, 0.53580E+00_JPRB ,&
&  0.76455E+00_JPRB, 0.10000E+01_JPRB, 0.16082E+01_JPRB, 0.16453E+01_JPRB, 0.14825E+01_JPRB, 0.13046E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.12483E+01_JPRB, 0.11811E+01_JPRB, 0.11033E+01_JPRB, 0.10497E+01_JPRB, 0.99214E+00_JPRB, 0.95255E+00_JPRB, 0.89329E+00_JPRB ,&
&  0.83862E+00_JPRB, 0.78118E+00_JPRB, 0.72006E+00_JPRB, 0.67410E+00_JPRB, 0.64436E+00_JPRB, 0.63233E+00_JPRB, 0.63887E+00_JPRB ,&
&  0.65282E+00_JPRB, 0.67252E+00_JPRB, 0.74189E+00_JPRB, 0.88616E+00_JPRB, 0.10917E+01_JPRB, 0.11262E+01_JPRB, 0.14269E+01_JPRB ,&
&  0.18461E+01_JPRB, 0.18143E+01_JPRB, 0.16465E+01_JPRB, 0.15076E+01_JPRB, 0.13148E+01_JPRB, 0.12348E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.11454E+01_JPRB, 0.10431E+01_JPRB, 0.93703E+00_JPRB, 0.85650E+00_JPRB, 0.80045E+00_JPRB, 0.76649E+00_JPRB, 0.75138E+00_JPRB ,&
&  0.73501E+00_JPRB, 0.71022E+00_JPRB, 0.68071E+00_JPRB, 0.65678E+00_JPRB, 0.64338E+00_JPRB, 0.63932E+00_JPRB, 0.64050E+00_JPRB ,&
&  0.64542E+00_JPRB, 0.65361E+00_JPRB, 0.68464E+00_JPRB, 0.66006E+00_JPRB, 0.66610E+00_JPRB, 0.68848E+00_JPRB, 0.69940E+00_JPRB ,&
&  0.69722E+00_JPRB, 0.72171E+00_JPRB, 0.73854E+00_JPRB, 0.77245E+00_JPRB, 0.85291E+00_JPRB, 0.87200E+00_JPRB, 0.78747E+00_JPRB ,&
&  0.70369E+00_JPRB, 0.66872E+00_JPRB, 0.66619E+00_JPRB, 0.65656E+00_JPRB, 0.64073E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.63191E+00_JPRB, 0.60784E+00_JPRB, 0.58697E+00_JPRB, 0.56560E+00_JPRB, 0.54739E+00_JPRB, 0.53904E+00_JPRB, 0.53756E+00_JPRB ,&
&  0.53953E+00_JPRB, 0.53899E+00_JPRB, 0.53531E+00_JPRB, 0.52813E+00_JPRB, 0.52282E+00_JPRB, 0.51789E+00_JPRB, 0.51358E+00_JPRB ,&
&  0.50617E+00_JPRB, 0.50664E+00_JPRB, 0.50266E+00_JPRB, 0.49550E+00_JPRB, 0.48745E+00_JPRB, 0.47803E+00_JPRB, 0.46900E+00_JPRB ,&
&  0.46316E+00_JPRB, 0.46104E+00_JPRB, 0.46132E+00_JPRB, 0.46551E+00_JPRB, 0.47792E+00_JPRB, 0.49168E+00_JPRB, 0.51387E+00_JPRB ,&
&  0.68582E+00_JPRB, 0.10123E+01_JPRB, 0.12344E+01_JPRB, 0.13686E+01_JPRB, 0.12164E+01_JPRB, 0.10610E+01_JPRB, 0.98122E+00_JPRB ,&
&  0.94720E+00_JPRB, 0.93680E+00_JPRB, 0.89890E+00_JPRB, 0.85351E+00_JPRB, 0.80046E+00_JPRB, 0.77433E+00_JPRB, 0.74466E+00_JPRB ,&
&  0.71151E+00_JPRB, 0.68109E+00_JPRB, 0.65239E+00_JPRB, 0.63788E+00_JPRB, 0.62586E+00_JPRB, 0.61185E+00_JPRB, 0.60253E+00_JPRB ,&
&  0.60282E+00_JPRB, 0.60838E+00_JPRB, 0.63317E+00_JPRB, 0.72442E+00_JPRB, 0.91877E+00_JPRB, 0.10738E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.13209E+01_JPRB, 0.13659E+01_JPRB, 0.14014E+01_JPRB, 0.13963E+01_JPRB, 0.13227E+01_JPRB, 0.12690E+01_JPRB, 0.12583E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.10556E+01_JPRB, 0.91761E+00_JPRB, 0.83264E+00_JPRB, 0.78942E+00_JPRB, 0.76978E+00_JPRB, 0.75468E+00_JPRB ,&
&  0.74670E+00_JPRB, 0.73658E+00_JPRB, 0.71613E+00_JPRB, 0.68968E+00_JPRB, 0.66496E+00_JPRB, 0.64686E+00_JPRB, 0.63702E+00_JPRB ,&
&  0.62975E+00_JPRB, 0.62923E+00_JPRB, 0.64911E+00_JPRB, 0.66245E+00_JPRB, 0.67166E+00_JPRB, 0.68319E+00_JPRB, 0.69703E+00_JPRB ,&
&  0.71517E+00_JPRB, 0.76465E+00_JPRB, 0.74901E+00_JPRB, 0.78796E+00_JPRB, 0.84420E+00_JPRB, 0.83577E+00_JPRB, 0.76247E+00_JPRB ,&
&  0.68999E+00_JPRB, 0.65621E+00_JPRB, 0.65467E+00_JPRB, 0.65698E+00_JPRB, 0.65609E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.72470E+00_JPRB, 0.66802E+00_JPRB, 0.62087E+00_JPRB, 0.58247E+00_JPRB, 0.55907E+00_JPRB, 0.54509E+00_JPRB, 0.53543E+00_JPRB ,&
&  0.53033E+00_JPRB, 0.52599E+00_JPRB, 0.51725E+00_JPRB, 0.51081E+00_JPRB, 0.50598E+00_JPRB, 0.50351E+00_JPRB, 0.50111E+00_JPRB ,&
&  0.49174E+00_JPRB, 0.48996E+00_JPRB, 0.48996E+00_JPRB, 0.48533E+00_JPRB, 0.47892E+00_JPRB, 0.47202E+00_JPRB, 0.46313E+00_JPRB ,&
&  0.45697E+00_JPRB, 0.45485E+00_JPRB, 0.45650E+00_JPRB, 0.46031E+00_JPRB, 0.46454E+00_JPRB, 0.47656E+00_JPRB, 0.50418E+00_JPRB ,&
&  0.64933E+00_JPRB, 0.82108E+00_JPRB, 0.95229E+00_JPRB, 0.10116E+01_JPRB, 0.88940E+00_JPRB, 0.82725E+00_JPRB, 0.79939E+00_JPRB ,&
&  0.80027E+00_JPRB, 0.80786E+00_JPRB, 0.77116E+00_JPRB, 0.71129E+00_JPRB, 0.65713E+00_JPRB, 0.62757E+00_JPRB, 0.60843E+00_JPRB ,&
&  0.59107E+00_JPRB, 0.57766E+00_JPRB, 0.56557E+00_JPRB, 0.56531E+00_JPRB, 0.56565E+00_JPRB, 0.56191E+00_JPRB, 0.55831E+00_JPRB ,&
&  0.55702E+00_JPRB, 0.55923E+00_JPRB, 0.56695E+00_JPRB, 0.60488E+00_JPRB, 0.70455E+00_JPRB, 0.81431E+00_JPRB, 0.92227E+00_JPRB ,&
&  0.97376E+00_JPRB, 0.10260E+01_JPRB, 0.10909E+01_JPRB, 0.11418E+01_JPRB, 0.11447E+01_JPRB, 0.11354E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.10082E+01_JPRB, 0.89503E+00_JPRB, 0.81202E+00_JPRB, 0.76718E+00_JPRB, 0.74463E+00_JPRB, 0.73200E+00_JPRB ,&
&  0.72129E+00_JPRB, 0.71073E+00_JPRB, 0.69428E+00_JPRB, 0.67116E+00_JPRB, 0.64547E+00_JPRB, 0.62756E+00_JPRB, 0.61426E+00_JPRB ,&
&  0.60701E+00_JPRB, 0.60993E+00_JPRB, 0.62453E+00_JPRB, 0.64689E+00_JPRB, 0.66362E+00_JPRB, 0.67553E+00_JPRB, 0.68161E+00_JPRB ,&
&  0.68987E+00_JPRB, 0.73147E+00_JPRB, 0.71837E+00_JPRB, 0.80384E+00_JPRB, 0.88122E+00_JPRB, 0.82538E+00_JPRB, 0.75370E+00_JPRB ,&
&  0.68717E+00_JPRB, 0.64267E+00_JPRB, 0.62714E+00_JPRB, 0.64692E+00_JPRB, 0.75794E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.67689E+00_JPRB, 0.65251E+00_JPRB, 0.61695E+00_JPRB, 0.58314E+00_JPRB, 0.56043E+00_JPRB, 0.54628E+00_JPRB, 0.53769E+00_JPRB ,&
&  0.52997E+00_JPRB, 0.52145E+00_JPRB, 0.51181E+00_JPRB, 0.49951E+00_JPRB, 0.49178E+00_JPRB, 0.48799E+00_JPRB, 0.48517E+00_JPRB ,&
&  0.48042E+00_JPRB, 0.47095E+00_JPRB, 0.46806E+00_JPRB, 0.46536E+00_JPRB, 0.46398E+00_JPRB, 0.46212E+00_JPRB, 0.45690E+00_JPRB ,&
&  0.45037E+00_JPRB, 0.44715E+00_JPRB, 0.45023E+00_JPRB, 0.45190E+00_JPRB, 0.45397E+00_JPRB, 0.45982E+00_JPRB, 0.47840E+00_JPRB ,&
&  0.57534E+00_JPRB, 0.64997E+00_JPRB, 0.74825E+00_JPRB, 0.82574E+00_JPRB, 0.78508E+00_JPRB, 0.74142E+00_JPRB, 0.71851E+00_JPRB ,&
&  0.68882E+00_JPRB, 0.66841E+00_JPRB, 0.63274E+00_JPRB, 0.58902E+00_JPRB, 0.55281E+00_JPRB, 0.54121E+00_JPRB, 0.53334E+00_JPRB ,&
&  0.52665E+00_JPRB, 0.52246E+00_JPRB, 0.51977E+00_JPRB, 0.51879E+00_JPRB, 0.51926E+00_JPRB, 0.51830E+00_JPRB, 0.51721E+00_JPRB ,&
&  0.51824E+00_JPRB, 0.52219E+00_JPRB, 0.52672E+00_JPRB, 0.53839E+00_JPRB, 0.57878E+00_JPRB, 0.65049E+00_JPRB, 0.72776E+00_JPRB ,&
&  0.77747E+00_JPRB, 0.82155E+00_JPRB, 0.88164E+00_JPRB, 0.92945E+00_JPRB, 0.95584E+00_JPRB, 0.96006E+00_JPRB, 0.94641E+00_JPRB ,&
&  0.92756E+00_JPRB, 0.89618E+00_JPRB, 0.83122E+00_JPRB, 0.76219E+00_JPRB, 0.71730E+00_JPRB, 0.68917E+00_JPRB, 0.66860E+00_JPRB ,&
&  0.65341E+00_JPRB, 0.63494E+00_JPRB, 0.61581E+00_JPRB, 0.59974E+00_JPRB, 0.58990E+00_JPRB, 0.58074E+00_JPRB, 0.57266E+00_JPRB ,&
&  0.57455E+00_JPRB, 0.58673E+00_JPRB, 0.60519E+00_JPRB, 0.61824E+00_JPRB, 0.62717E+00_JPRB, 0.63625E+00_JPRB, 0.64069E+00_JPRB ,&
&  0.64704E+00_JPRB, 0.67333E+00_JPRB, 0.69165E+00_JPRB, 0.76844E+00_JPRB, 0.78065E+00_JPRB, 0.76368E+00_JPRB, 0.72272E+00_JPRB ,&
&  0.67566E+00_JPRB, 0.63757E+00_JPRB, 0.60910E+00_JPRB, 0.62035E+00_JPRB, 0.67191E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.61494E+00_JPRB, 0.60539E+00_JPRB, 0.59285E+00_JPRB, 0.57546E+00_JPRB, 0.56321E+00_JPRB, 0.55286E+00_JPRB, 0.54083E+00_JPRB ,&
&  0.52871E+00_JPRB, 0.51836E+00_JPRB, 0.50922E+00_JPRB, 0.49858E+00_JPRB, 0.48693E+00_JPRB, 0.48292E+00_JPRB, 0.48156E+00_JPRB ,&
&  0.47741E+00_JPRB, 0.46921E+00_JPRB, 0.46178E+00_JPRB, 0.46004E+00_JPRB, 0.45929E+00_JPRB, 0.45877E+00_JPRB, 0.45542E+00_JPRB ,&
&  0.45007E+00_JPRB, 0.44921E+00_JPRB, 0.45234E+00_JPRB, 0.45278E+00_JPRB, 0.44853E+00_JPRB, 0.45000E+00_JPRB, 0.46452E+00_JPRB ,&
&  0.53898E+00_JPRB, 0.51068E+00_JPRB, 0.57487E+00_JPRB, 0.68348E+00_JPRB, 0.65495E+00_JPRB, 0.63997E+00_JPRB, 0.60616E+00_JPRB ,&
&  0.57188E+00_JPRB, 0.55382E+00_JPRB, 0.54019E+00_JPRB, 0.52171E+00_JPRB, 0.50290E+00_JPRB, 0.49254E+00_JPRB, 0.49183E+00_JPRB ,&
&  0.49644E+00_JPRB, 0.49889E+00_JPRB, 0.49959E+00_JPRB, 0.49458E+00_JPRB, 0.49148E+00_JPRB, 0.48923E+00_JPRB, 0.48814E+00_JPRB ,&
&  0.48763E+00_JPRB, 0.49063E+00_JPRB, 0.49514E+00_JPRB, 0.49925E+00_JPRB, 0.51267E+00_JPRB, 0.55208E+00_JPRB, 0.61315E+00_JPRB ,&
&  0.67186E+00_JPRB, 0.72479E+00_JPRB, 0.76566E+00_JPRB, 0.80214E+00_JPRB, 0.82472E+00_JPRB, 0.84266E+00_JPRB, 0.84626E+00_JPRB ,&
&  0.82539E+00_JPRB, 0.78913E+00_JPRB, 0.74960E+00_JPRB, 0.70617E+00_JPRB, 0.66670E+00_JPRB, 0.63554E+00_JPRB, 0.61110E+00_JPRB ,&
&  0.59262E+00_JPRB, 0.57999E+00_JPRB, 0.57020E+00_JPRB, 0.55910E+00_JPRB, 0.55260E+00_JPRB, 0.55266E+00_JPRB, 0.55642E+00_JPRB ,&
&  0.56308E+00_JPRB, 0.57115E+00_JPRB, 0.57680E+00_JPRB, 0.57934E+00_JPRB, 0.57995E+00_JPRB, 0.58245E+00_JPRB, 0.58615E+00_JPRB ,&
&  0.59813E+00_JPRB, 0.62244E+00_JPRB, 0.64419E+00_JPRB, 0.72900E+00_JPRB, 0.77601E+00_JPRB, 0.76445E+00_JPRB, 0.72938E+00_JPRB ,&
&  0.68667E+00_JPRB, 0.64369E+00_JPRB, 0.61923E+00_JPRB, 0.54750E+00_JPRB, 0.62281E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.57377E+00_JPRB, 0.57010E+00_JPRB, 0.56388E+00_JPRB, 0.55694E+00_JPRB, 0.55010E+00_JPRB, 0.54041E+00_JPRB, 0.52756E+00_JPRB ,&
&  0.51549E+00_JPRB, 0.50459E+00_JPRB, 0.49735E+00_JPRB, 0.49021E+00_JPRB, 0.47972E+00_JPRB, 0.47307E+00_JPRB, 0.47183E+00_JPRB ,&
&  0.46987E+00_JPRB, 0.46414E+00_JPRB, 0.45749E+00_JPRB, 0.45616E+00_JPRB, 0.45394E+00_JPRB, 0.45105E+00_JPRB, 0.44696E+00_JPRB ,&
&  0.44276E+00_JPRB, 0.44147E+00_JPRB, 0.44190E+00_JPRB, 0.44274E+00_JPRB, 0.43940E+00_JPRB, 0.44083E+00_JPRB, 0.45560E+00_JPRB ,&
&  0.51427E+00_JPRB, 0.45455E+00_JPRB, 0.51198E+00_JPRB, 0.57312E+00_JPRB, 0.54258E+00_JPRB, 0.51810E+00_JPRB, 0.49749E+00_JPRB ,&
&  0.48243E+00_JPRB, 0.47555E+00_JPRB, 0.47176E+00_JPRB, 0.46655E+00_JPRB, 0.46155E+00_JPRB, 0.45604E+00_JPRB, 0.45474E+00_JPRB ,&
&  0.45614E+00_JPRB, 0.46245E+00_JPRB, 0.46626E+00_JPRB, 0.46663E+00_JPRB, 0.46317E+00_JPRB, 0.45937E+00_JPRB, 0.45611E+00_JPRB ,&
&  0.45464E+00_JPRB, 0.45587E+00_JPRB, 0.45844E+00_JPRB, 0.46297E+00_JPRB, 0.47204E+00_JPRB, 0.49182E+00_JPRB, 0.52756E+00_JPRB ,&
&  0.57498E+00_JPRB, 0.61742E+00_JPRB, 0.65347E+00_JPRB, 0.69059E+00_JPRB, 0.72349E+00_JPRB, 0.74318E+00_JPRB, 0.74348E+00_JPRB ,&
&  0.72590E+00_JPRB, 0.69293E+00_JPRB, 0.66597E+00_JPRB, 0.64064E+00_JPRB, 0.61624E+00_JPRB, 0.59641E+00_JPRB, 0.58295E+00_JPRB ,&
&  0.57437E+00_JPRB, 0.56864E+00_JPRB, 0.56474E+00_JPRB, 0.56096E+00_JPRB, 0.56036E+00_JPRB, 0.56067E+00_JPRB, 0.56274E+00_JPRB ,&
&  0.56138E+00_JPRB, 0.55777E+00_JPRB, 0.54783E+00_JPRB, 0.53451E+00_JPRB, 0.52745E+00_JPRB, 0.52583E+00_JPRB, 0.52874E+00_JPRB ,&
&  0.53794E+00_JPRB, 0.55597E+00_JPRB, 0.57498E+00_JPRB, 0.63979E+00_JPRB, 0.67431E+00_JPRB, 0.69038E+00_JPRB, 0.68950E+00_JPRB ,&
&  0.66890E+00_JPRB, 0.63583E+00_JPRB, 0.50798E+00_JPRB, 0.56924E+00_JPRB, 0.58149E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.56857E+00_JPRB, 0.55849E+00_JPRB, 0.54916E+00_JPRB, 0.53975E+00_JPRB, 0.52940E+00_JPRB, 0.51812E+00_JPRB, 0.50651E+00_JPRB ,&
&  0.49650E+00_JPRB, 0.48800E+00_JPRB, 0.47996E+00_JPRB, 0.47301E+00_JPRB, 0.46642E+00_JPRB, 0.46320E+00_JPRB, 0.46219E+00_JPRB ,&
&  0.46244E+00_JPRB, 0.46161E+00_JPRB, 0.45974E+00_JPRB, 0.45944E+00_JPRB, 0.45880E+00_JPRB, 0.45611E+00_JPRB, 0.45233E+00_JPRB ,&
&  0.44637E+00_JPRB, 0.44120E+00_JPRB, 0.43918E+00_JPRB, 0.44133E+00_JPRB, 0.44021E+00_JPRB, 0.44255E+00_JPRB, 0.45216E+00_JPRB ,&
&  0.44083E+00_JPRB, 0.41918E+00_JPRB, 0.45883E+00_JPRB, 0.50469E+00_JPRB, 0.48286E+00_JPRB, 0.46266E+00_JPRB, 0.45463E+00_JPRB ,&
&  0.44708E+00_JPRB, 0.44280E+00_JPRB, 0.43954E+00_JPRB, 0.43799E+00_JPRB, 0.43878E+00_JPRB, 0.43834E+00_JPRB, 0.43714E+00_JPRB ,&
&  0.43684E+00_JPRB, 0.44027E+00_JPRB, 0.44392E+00_JPRB, 0.44616E+00_JPRB, 0.44603E+00_JPRB, 0.44443E+00_JPRB, 0.44038E+00_JPRB ,&
&  0.43661E+00_JPRB, 0.43439E+00_JPRB, 0.43453E+00_JPRB, 0.43839E+00_JPRB, 0.44544E+00_JPRB, 0.45754E+00_JPRB, 0.47827E+00_JPRB ,&
&  0.50785E+00_JPRB, 0.53891E+00_JPRB, 0.56661E+00_JPRB, 0.59410E+00_JPRB, 0.62487E+00_JPRB, 0.65226E+00_JPRB, 0.66717E+00_JPRB ,&
&  0.65896E+00_JPRB, 0.63543E+00_JPRB, 0.60789E+00_JPRB, 0.58560E+00_JPRB, 0.56914E+00_JPRB, 0.55842E+00_JPRB, 0.55194E+00_JPRB ,&
&  0.54950E+00_JPRB, 0.54786E+00_JPRB, 0.54635E+00_JPRB, 0.54563E+00_JPRB, 0.54496E+00_JPRB, 0.54205E+00_JPRB, 0.53778E+00_JPRB ,&
&  0.53092E+00_JPRB, 0.52031E+00_JPRB, 0.50806E+00_JPRB, 0.49786E+00_JPRB, 0.49107E+00_JPRB, 0.48926E+00_JPRB, 0.49159E+00_JPRB ,&
&  0.50002E+00_JPRB, 0.51779E+00_JPRB, 0.53824E+00_JPRB, 0.58477E+00_JPRB, 0.61777E+00_JPRB, 0.64073E+00_JPRB, 0.65116E+00_JPRB ,&
&  0.64581E+00_JPRB, 0.62395E+00_JPRB, 0.60215E+00_JPRB, 0.58894E+00_JPRB, 0.58185E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.54776E+00_JPRB, 0.53250E+00_JPRB, 0.51681E+00_JPRB, 0.50571E+00_JPRB, 0.49728E+00_JPRB, 0.48943E+00_JPRB, 0.48144E+00_JPRB ,&
&  0.47261E+00_JPRB, 0.46440E+00_JPRB, 0.45658E+00_JPRB, 0.44905E+00_JPRB, 0.44499E+00_JPRB, 0.44283E+00_JPRB, 0.44300E+00_JPRB ,&
&  0.44543E+00_JPRB, 0.45001E+00_JPRB, 0.45182E+00_JPRB, 0.45125E+00_JPRB, 0.45172E+00_JPRB, 0.45131E+00_JPRB, 0.44929E+00_JPRB ,&
&  0.44478E+00_JPRB, 0.43938E+00_JPRB, 0.43632E+00_JPRB, 0.43681E+00_JPRB, 0.43686E+00_JPRB, 0.43787E+00_JPRB, 0.43884E+00_JPRB ,&
&  0.44432E+00_JPRB, 0.41065E+00_JPRB, 0.44025E+00_JPRB, 0.46610E+00_JPRB, 0.45846E+00_JPRB, 0.45011E+00_JPRB, 0.44209E+00_JPRB ,&
&  0.43335E+00_JPRB, 0.42852E+00_JPRB, 0.42580E+00_JPRB, 0.42681E+00_JPRB, 0.42976E+00_JPRB, 0.43133E+00_JPRB, 0.43181E+00_JPRB ,&
&  0.42935E+00_JPRB, 0.42963E+00_JPRB, 0.42910E+00_JPRB, 0.42913E+00_JPRB, 0.42997E+00_JPRB, 0.43107E+00_JPRB, 0.43117E+00_JPRB ,&
&  0.42958E+00_JPRB, 0.42634E+00_JPRB, 0.42314E+00_JPRB, 0.42284E+00_JPRB, 0.42457E+00_JPRB, 0.42896E+00_JPRB, 0.43739E+00_JPRB ,&
&  0.45077E+00_JPRB, 0.46730E+00_JPRB, 0.48611E+00_JPRB, 0.50754E+00_JPRB, 0.53486E+00_JPRB, 0.56142E+00_JPRB, 0.58177E+00_JPRB ,&
&  0.58659E+00_JPRB, 0.58030E+00_JPRB, 0.56585E+00_JPRB, 0.54840E+00_JPRB, 0.53313E+00_JPRB, 0.51840E+00_JPRB, 0.50497E+00_JPRB ,&
&  0.49494E+00_JPRB, 0.48670E+00_JPRB, 0.48113E+00_JPRB, 0.47870E+00_JPRB, 0.47685E+00_JPRB, 0.47355E+00_JPRB, 0.47053E+00_JPRB ,&
&  0.46760E+00_JPRB, 0.46497E+00_JPRB, 0.46306E+00_JPRB, 0.46108E+00_JPRB, 0.45838E+00_JPRB, 0.45614E+00_JPRB, 0.45686E+00_JPRB ,&
&  0.46195E+00_JPRB, 0.47321E+00_JPRB, 0.49247E+00_JPRB, 0.52192E+00_JPRB, 0.54995E+00_JPRB, 0.57199E+00_JPRB, 0.58483E+00_JPRB ,&
&  0.58754E+00_JPRB, 0.58503E+00_JPRB, 0.57635E+00_JPRB, 0.56783E+00_JPRB, 0.56035E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.51461E+00_JPRB, 0.50383E+00_JPRB, 0.49257E+00_JPRB, 0.48362E+00_JPRB, 0.47680E+00_JPRB, 0.46982E+00_JPRB, 0.46215E+00_JPRB ,&
&  0.45615E+00_JPRB, 0.44915E+00_JPRB, 0.44362E+00_JPRB, 0.43854E+00_JPRB, 0.43454E+00_JPRB, 0.43306E+00_JPRB, 0.43358E+00_JPRB ,&
&  0.43599E+00_JPRB, 0.44137E+00_JPRB, 0.44421E+00_JPRB, 0.44257E+00_JPRB, 0.44418E+00_JPRB, 0.44630E+00_JPRB, 0.44661E+00_JPRB ,&
&  0.44403E+00_JPRB, 0.44066E+00_JPRB, 0.43696E+00_JPRB, 0.43464E+00_JPRB, 0.43446E+00_JPRB, 0.43571E+00_JPRB, 0.43306E+00_JPRB ,&
&  0.42875E+00_JPRB, 0.42479E+00_JPRB, 0.43781E+00_JPRB, 0.44712E+00_JPRB, 0.44701E+00_JPRB, 0.44162E+00_JPRB, 0.43497E+00_JPRB ,&
&  0.42671E+00_JPRB, 0.42209E+00_JPRB, 0.42225E+00_JPRB, 0.42426E+00_JPRB, 0.42682E+00_JPRB, 0.42818E+00_JPRB, 0.42971E+00_JPRB ,&
&  0.42682E+00_JPRB, 0.42531E+00_JPRB, 0.42332E+00_JPRB, 0.42192E+00_JPRB, 0.42267E+00_JPRB, 0.42392E+00_JPRB, 0.42484E+00_JPRB ,&
&  0.42416E+00_JPRB, 0.42498E+00_JPRB, 0.42586E+00_JPRB, 0.42507E+00_JPRB, 0.42657E+00_JPRB, 0.42896E+00_JPRB, 0.43246E+00_JPRB ,&
&  0.43762E+00_JPRB, 0.44496E+00_JPRB, 0.45336E+00_JPRB, 0.46506E+00_JPRB, 0.48251E+00_JPRB, 0.50096E+00_JPRB, 0.51522E+00_JPRB ,&
&  0.52233E+00_JPRB, 0.52209E+00_JPRB, 0.51675E+00_JPRB, 0.50706E+00_JPRB, 0.49749E+00_JPRB, 0.48882E+00_JPRB, 0.47845E+00_JPRB ,&
&  0.46665E+00_JPRB, 0.45841E+00_JPRB, 0.45260E+00_JPRB, 0.44875E+00_JPRB, 0.44502E+00_JPRB, 0.44250E+00_JPRB, 0.44205E+00_JPRB ,&
&  0.44357E+00_JPRB, 0.44531E+00_JPRB, 0.44569E+00_JPRB, 0.44539E+00_JPRB, 0.44572E+00_JPRB, 0.44407E+00_JPRB, 0.44324E+00_JPRB ,&
&  0.44553E+00_JPRB, 0.45152E+00_JPRB, 0.46333E+00_JPRB, 0.48572E+00_JPRB, 0.50373E+00_JPRB, 0.51775E+00_JPRB, 0.52767E+00_JPRB ,&
&  0.53348E+00_JPRB, 0.53619E+00_JPRB, 0.53483E+00_JPRB, 0.53096E+00_JPRB, 0.52444E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.47147E+00_JPRB, 0.46646E+00_JPRB, 0.46070E+00_JPRB, 0.45518E+00_JPRB, 0.45009E+00_JPRB, 0.44494E+00_JPRB, 0.44016E+00_JPRB ,&
&  0.43560E+00_JPRB, 0.43251E+00_JPRB, 0.43027E+00_JPRB, 0.42843E+00_JPRB, 0.42480E+00_JPRB, 0.42309E+00_JPRB, 0.42360E+00_JPRB ,&
&  0.42584E+00_JPRB, 0.42816E+00_JPRB, 0.42980E+00_JPRB, 0.42916E+00_JPRB, 0.43134E+00_JPRB, 0.43346E+00_JPRB, 0.43357E+00_JPRB ,&
&  0.43263E+00_JPRB, 0.43164E+00_JPRB, 0.42934E+00_JPRB, 0.42648E+00_JPRB, 0.42637E+00_JPRB, 0.42681E+00_JPRB, 0.42326E+00_JPRB ,&
&  0.41643E+00_JPRB, 0.42022E+00_JPRB, 0.41269E+00_JPRB, 0.42808E+00_JPRB, 0.43207E+00_JPRB, 0.42656E+00_JPRB, 0.41941E+00_JPRB ,&
&  0.41631E+00_JPRB, 0.41653E+00_JPRB, 0.41949E+00_JPRB, 0.42168E+00_JPRB, 0.42259E+00_JPRB, 0.42294E+00_JPRB, 0.42291E+00_JPRB ,&
&  0.41537E+00_JPRB, 0.41079E+00_JPRB, 0.41019E+00_JPRB, 0.41096E+00_JPRB, 0.41109E+00_JPRB, 0.41187E+00_JPRB, 0.41392E+00_JPRB ,&
&  0.41560E+00_JPRB, 0.41677E+00_JPRB, 0.41801E+00_JPRB, 0.42064E+00_JPRB, 0.42245E+00_JPRB, 0.42337E+00_JPRB, 0.42530E+00_JPRB ,&
&  0.42841E+00_JPRB, 0.43245E+00_JPRB, 0.43629E+00_JPRB, 0.43984E+00_JPRB, 0.44710E+00_JPRB, 0.45730E+00_JPRB, 0.46570E+00_JPRB ,&
&  0.46947E+00_JPRB, 0.46955E+00_JPRB, 0.46733E+00_JPRB, 0.46167E+00_JPRB, 0.45366E+00_JPRB, 0.44710E+00_JPRB, 0.44168E+00_JPRB ,&
&  0.43422E+00_JPRB, 0.43007E+00_JPRB, 0.42711E+00_JPRB, 0.42329E+00_JPRB, 0.42004E+00_JPRB, 0.41828E+00_JPRB, 0.41780E+00_JPRB ,&
&  0.41908E+00_JPRB, 0.42211E+00_JPRB, 0.42394E+00_JPRB, 0.42631E+00_JPRB, 0.42984E+00_JPRB, 0.43340E+00_JPRB, 0.43342E+00_JPRB ,&
&  0.43199E+00_JPRB, 0.43162E+00_JPRB, 0.43384E+00_JPRB, 0.44142E+00_JPRB, 0.45151E+00_JPRB, 0.46048E+00_JPRB, 0.46880E+00_JPRB ,&
&  0.47521E+00_JPRB, 0.47890E+00_JPRB, 0.47939E+00_JPRB, 0.47882E+00_JPRB, 0.47591E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.43772E+00_JPRB, 0.43618E+00_JPRB, 0.43430E+00_JPRB, 0.43121E+00_JPRB, 0.42774E+00_JPRB, 0.42441E+00_JPRB, 0.42134E+00_JPRB ,&
&  0.41903E+00_JPRB, 0.41809E+00_JPRB, 0.41820E+00_JPRB, 0.41805E+00_JPRB, 0.41759E+00_JPRB, 0.41813E+00_JPRB, 0.41825E+00_JPRB ,&
&  0.41958E+00_JPRB, 0.41966E+00_JPRB, 0.41722E+00_JPRB, 0.41666E+00_JPRB, 0.41915E+00_JPRB, 0.42391E+00_JPRB, 0.42573E+00_JPRB ,&
&  0.42345E+00_JPRB, 0.42314E+00_JPRB, 0.42279E+00_JPRB, 0.42054E+00_JPRB, 0.41896E+00_JPRB, 0.41803E+00_JPRB, 0.41421E+00_JPRB ,&
&  0.40923E+00_JPRB, 0.41043E+00_JPRB, 0.41216E+00_JPRB, 0.41661E+00_JPRB, 0.41744E+00_JPRB, 0.41633E+00_JPRB, 0.41672E+00_JPRB ,&
&  0.41472E+00_JPRB, 0.41413E+00_JPRB, 0.41362E+00_JPRB, 0.41397E+00_JPRB, 0.41212E+00_JPRB, 0.40937E+00_JPRB, 0.40426E+00_JPRB ,&
&  0.39459E+00_JPRB, 0.39123E+00_JPRB, 0.39322E+00_JPRB, 0.39763E+00_JPRB, 0.39724E+00_JPRB, 0.39801E+00_JPRB, 0.40116E+00_JPRB ,&
&  0.40515E+00_JPRB, 0.40812E+00_JPRB, 0.40997E+00_JPRB, 0.41119E+00_JPRB, 0.41283E+00_JPRB, 0.41438E+00_JPRB, 0.41526E+00_JPRB ,&
&  0.41641E+00_JPRB, 0.41847E+00_JPRB, 0.42105E+00_JPRB, 0.42392E+00_JPRB, 0.42631E+00_JPRB, 0.42952E+00_JPRB, 0.43232E+00_JPRB ,&
&  0.43302E+00_JPRB, 0.43077E+00_JPRB, 0.42755E+00_JPRB, 0.42228E+00_JPRB, 0.41632E+00_JPRB, 0.41226E+00_JPRB, 0.40956E+00_JPRB ,&
&  0.40486E+00_JPRB, 0.40086E+00_JPRB, 0.39791E+00_JPRB, 0.39699E+00_JPRB, 0.39697E+00_JPRB, 0.39797E+00_JPRB, 0.39996E+00_JPRB ,&
&  0.40166E+00_JPRB, 0.40459E+00_JPRB, 0.41101E+00_JPRB, 0.41536E+00_JPRB, 0.41750E+00_JPRB, 0.42057E+00_JPRB, 0.42191E+00_JPRB ,&
&  0.42007E+00_JPRB, 0.41841E+00_JPRB, 0.41725E+00_JPRB, 0.41943E+00_JPRB, 0.42298E+00_JPRB, 0.42580E+00_JPRB, 0.42821E+00_JPRB ,&
&  0.43234E+00_JPRB, 0.43646E+00_JPRB, 0.43785E+00_JPRB, 0.43838E+00_JPRB, 0.43882E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.40333E+00_JPRB, 0.40374E+00_JPRB, 0.40322E+00_JPRB, 0.40207E+00_JPRB, 0.40080E+00_JPRB, 0.39972E+00_JPRB, 0.39963E+00_JPRB ,&
&  0.39965E+00_JPRB, 0.40012E+00_JPRB, 0.40085E+00_JPRB, 0.40143E+00_JPRB, 0.40297E+00_JPRB, 0.40482E+00_JPRB, 0.40635E+00_JPRB ,&
&  0.40795E+00_JPRB, 0.40710E+00_JPRB, 0.40419E+00_JPRB, 0.40150E+00_JPRB, 0.40221E+00_JPRB, 0.40766E+00_JPRB, 0.41369E+00_JPRB ,&
&  0.41521E+00_JPRB, 0.41625E+00_JPRB, 0.41662E+00_JPRB, 0.41535E+00_JPRB, 0.41378E+00_JPRB, 0.41335E+00_JPRB, 0.40978E+00_JPRB ,&
&  0.40519E+00_JPRB, 0.40414E+00_JPRB, 0.40367E+00_JPRB, 0.40170E+00_JPRB, 0.40209E+00_JPRB, 0.40529E+00_JPRB, 0.40504E+00_JPRB ,&
&  0.40442E+00_JPRB, 0.40262E+00_JPRB, 0.39934E+00_JPRB, 0.39625E+00_JPRB, 0.39254E+00_JPRB, 0.38580E+00_JPRB, 0.37802E+00_JPRB ,&
&  0.37181E+00_JPRB, 0.37180E+00_JPRB, 0.37723E+00_JPRB, 0.38297E+00_JPRB, 0.38204E+00_JPRB, 0.38102E+00_JPRB, 0.38258E+00_JPRB ,&
&  0.38623E+00_JPRB, 0.39081E+00_JPRB, 0.39374E+00_JPRB, 0.39297E+00_JPRB, 0.39184E+00_JPRB, 0.39372E+00_JPRB, 0.39640E+00_JPRB ,&
&  0.39769E+00_JPRB, 0.39913E+00_JPRB, 0.40019E+00_JPRB, 0.40171E+00_JPRB, 0.40241E+00_JPRB, 0.40145E+00_JPRB, 0.40122E+00_JPRB ,&
&  0.39934E+00_JPRB, 0.39615E+00_JPRB, 0.39155E+00_JPRB, 0.38595E+00_JPRB, 0.38167E+00_JPRB, 0.37893E+00_JPRB, 0.37871E+00_JPRB ,&
&  0.37787E+00_JPRB, 0.37914E+00_JPRB, 0.37937E+00_JPRB, 0.37714E+00_JPRB, 0.37416E+00_JPRB, 0.37420E+00_JPRB, 0.37777E+00_JPRB ,&
&  0.38260E+00_JPRB, 0.38618E+00_JPRB, 0.39202E+00_JPRB, 0.39496E+00_JPRB, 0.39702E+00_JPRB, 0.39952E+00_JPRB, 0.40015E+00_JPRB ,&
&  0.39748E+00_JPRB, 0.39528E+00_JPRB, 0.39473E+00_JPRB, 0.39739E+00_JPRB, 0.39844E+00_JPRB, 0.39828E+00_JPRB, 0.39644E+00_JPRB ,&
&  0.39633E+00_JPRB, 0.39875E+00_JPRB, 0.40001E+00_JPRB, 0.40035E+00_JPRB, 0.40152E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.37695E+00_JPRB, 0.37938E+00_JPRB, 0.38181E+00_JPRB, 0.38394E+00_JPRB, 0.38531E+00_JPRB, 0.38633E+00_JPRB, 0.38549E+00_JPRB ,&
&  0.38530E+00_JPRB, 0.38687E+00_JPRB, 0.38893E+00_JPRB, 0.39060E+00_JPRB, 0.39157E+00_JPRB, 0.39348E+00_JPRB, 0.39540E+00_JPRB ,&
&  0.39665E+00_JPRB, 0.39561E+00_JPRB, 0.39193E+00_JPRB, 0.39022E+00_JPRB, 0.38911E+00_JPRB, 0.39420E+00_JPRB, 0.40283E+00_JPRB ,&
&  0.40669E+00_JPRB, 0.40904E+00_JPRB, 0.41188E+00_JPRB, 0.41242E+00_JPRB, 0.41266E+00_JPRB, 0.41247E+00_JPRB, 0.40802E+00_JPRB ,&
&  0.40303E+00_JPRB, 0.39857E+00_JPRB, 0.39560E+00_JPRB, 0.39237E+00_JPRB, 0.39251E+00_JPRB, 0.39059E+00_JPRB, 0.38934E+00_JPRB ,&
&  0.38840E+00_JPRB, 0.38592E+00_JPRB, 0.38251E+00_JPRB, 0.37689E+00_JPRB, 0.36997E+00_JPRB, 0.36106E+00_JPRB, 0.35488E+00_JPRB ,&
&  0.35240E+00_JPRB, 0.35605E+00_JPRB, 0.36464E+00_JPRB, 0.36858E+00_JPRB, 0.36534E+00_JPRB, 0.36455E+00_JPRB, 0.36778E+00_JPRB ,&
&  0.37262E+00_JPRB, 0.37798E+00_JPRB, 0.37991E+00_JPRB, 0.37905E+00_JPRB, 0.37708E+00_JPRB, 0.37866E+00_JPRB, 0.38294E+00_JPRB ,&
&  0.38669E+00_JPRB, 0.38865E+00_JPRB, 0.38790E+00_JPRB, 0.38694E+00_JPRB, 0.38436E+00_JPRB, 0.37865E+00_JPRB, 0.37101E+00_JPRB ,&
&  0.36404E+00_JPRB, 0.35969E+00_JPRB, 0.35636E+00_JPRB, 0.35315E+00_JPRB, 0.35198E+00_JPRB, 0.35242E+00_JPRB, 0.35470E+00_JPRB ,&
&  0.35563E+00_JPRB, 0.35752E+00_JPRB, 0.35769E+00_JPRB, 0.35358E+00_JPRB, 0.35029E+00_JPRB, 0.35169E+00_JPRB, 0.35330E+00_JPRB ,&
&  0.35592E+00_JPRB, 0.35857E+00_JPRB, 0.36123E+00_JPRB, 0.36234E+00_JPRB, 0.36397E+00_JPRB, 0.36670E+00_JPRB, 0.36834E+00_JPRB ,&
&  0.36917E+00_JPRB, 0.36979E+00_JPRB, 0.37162E+00_JPRB, 0.37458E+00_JPRB, 0.37730E+00_JPRB, 0.37949E+00_JPRB, 0.37894E+00_JPRB ,&
&  0.37794E+00_JPRB, 0.37674E+00_JPRB, 0.37737E+00_JPRB, 0.37789E+00_JPRB, 0.37666E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.36992E+00_JPRB, 0.36996E+00_JPRB, 0.37340E+00_JPRB, 0.37709E+00_JPRB, 0.37800E+00_JPRB, 0.37784E+00_JPRB, 0.37537E+00_JPRB ,&
&  0.37370E+00_JPRB, 0.37485E+00_JPRB, 0.37662E+00_JPRB, 0.37787E+00_JPRB, 0.37997E+00_JPRB, 0.38276E+00_JPRB, 0.38496E+00_JPRB ,&
&  0.38501E+00_JPRB, 0.38307E+00_JPRB, 0.37974E+00_JPRB, 0.38012E+00_JPRB, 0.38001E+00_JPRB, 0.38213E+00_JPRB, 0.38971E+00_JPRB ,&
&  0.39518E+00_JPRB, 0.39892E+00_JPRB, 0.40205E+00_JPRB, 0.40245E+00_JPRB, 0.40193E+00_JPRB, 0.40350E+00_JPRB, 0.40015E+00_JPRB ,&
&  0.39392E+00_JPRB, 0.38825E+00_JPRB, 0.38288E+00_JPRB, 0.37582E+00_JPRB, 0.37844E+00_JPRB, 0.37597E+00_JPRB, 0.37297E+00_JPRB ,&
&  0.36881E+00_JPRB, 0.36337E+00_JPRB, 0.35699E+00_JPRB, 0.35100E+00_JPRB, 0.34655E+00_JPRB, 0.34241E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33535E+00_JPRB, 0.33750E+00_JPRB, 0.34333E+00_JPRB, 0.34589E+00_JPRB, 0.34328E+00_JPRB, 0.34414E+00_JPRB, 0.35082E+00_JPRB ,&
&  0.35772E+00_JPRB, 0.36100E+00_JPRB, 0.36085E+00_JPRB, 0.36133E+00_JPRB, 0.36193E+00_JPRB, 0.36333E+00_JPRB, 0.36402E+00_JPRB ,&
&  0.36298E+00_JPRB, 0.36128E+00_JPRB, 0.35868E+00_JPRB, 0.35266E+00_JPRB, 0.34551E+00_JPRB, 0.33933E+00_JPRB, 0.33684E+00_JPRB ,&
&  0.33940E+00_JPRB, 0.33937E+00_JPRB, 0.33980E+00_JPRB, 0.34156E+00_JPRB, 0.34217E+00_JPRB, 0.34149E+00_JPRB, 0.34057E+00_JPRB ,&
&  0.34196E+00_JPRB, 0.34100E+00_JPRB, 0.34228E+00_JPRB, 0.33945E+00_JPRB, 0.33491E+00_JPRB, 0.33036E+00_JPRB, 0.32603E+00_JPRB ,&
&  0.32354E+00_JPRB, 0.32277E+00_JPRB, 0.32479E+00_JPRB, 0.32552E+00_JPRB, 0.32749E+00_JPRB, 0.32982E+00_JPRB, 0.33181E+00_JPRB ,&
&  0.33297E+00_JPRB, 0.33446E+00_JPRB, 0.33726E+00_JPRB, 0.34278E+00_JPRB, 0.34856E+00_JPRB, 0.35356E+00_JPRB, 0.35866E+00_JPRB ,&
&  0.36336E+00_JPRB, 0.36666E+00_JPRB, 0.37251E+00_JPRB, 0.37372E+00_JPRB, 0.37299E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.36837E+00_JPRB, 0.36593E+00_JPRB, 0.36603E+00_JPRB, 0.36694E+00_JPRB, 0.36648E+00_JPRB, 0.36568E+00_JPRB, 0.36307E+00_JPRB ,&
&  0.36077E+00_JPRB, 0.36118E+00_JPRB, 0.36254E+00_JPRB, 0.36507E+00_JPRB, 0.36948E+00_JPRB, 0.37417E+00_JPRB, 0.37359E+00_JPRB ,&
&  0.37046E+00_JPRB, 0.36905E+00_JPRB, 0.37111E+00_JPRB, 0.37309E+00_JPRB, 0.37338E+00_JPRB, 0.37407E+00_JPRB, 0.37778E+00_JPRB ,&
&  0.38247E+00_JPRB, 0.38459E+00_JPRB, 0.38755E+00_JPRB, 0.38715E+00_JPRB, 0.38346E+00_JPRB, 0.38372E+00_JPRB, 0.38191E+00_JPRB ,&
&  0.37699E+00_JPRB, 0.37097E+00_JPRB, 0.36179E+00_JPRB, 0.36257E+00_JPRB, 0.35892E+00_JPRB, 0.35590E+00_JPRB, 0.35135E+00_JPRB ,&
&  0.34634E+00_JPRB, 0.34214E+00_JPRB, 0.33746E+00_JPRB, 0.33369E+00_JPRB, 0.33180E+00_JPRB, 0.33169E+00_JPRB, 0.32952E+00_JPRB ,&
&  0.32819E+00_JPRB, 0.33002E+00_JPRB, 0.33144E+00_JPRB, 0.32847E+00_JPRB, 0.32822E+00_JPRB, 0.33457E+00_JPRB, 0.33974E+00_JPRB ,&
&  0.34180E+00_JPRB, 0.34178E+00_JPRB, 0.33909E+00_JPRB, 0.33834E+00_JPRB, 0.33729E+00_JPRB, 0.33532E+00_JPRB, 0.33190E+00_JPRB ,&
&  0.32671E+00_JPRB, 0.32413E+00_JPRB, 0.32236E+00_JPRB, 0.31881E+00_JPRB, 0.31280E+00_JPRB, 0.29616E+00_JPRB, 0.28700E+00_JPRB ,&
&  0.28721E+00_JPRB, 0.29618E+00_JPRB, 0.31073E+00_JPRB, 0.31767E+00_JPRB, 0.32388E+00_JPRB, 0.32907E+00_JPRB, 0.32973E+00_JPRB ,&
&  0.33055E+00_JPRB, 0.31057E+00_JPRB, 0.29961E+00_JPRB, 0.29068E+00_JPRB, 0.28261E+00_JPRB, 0.27752E+00_JPRB, 0.27585E+00_JPRB ,&
&  0.27860E+00_JPRB, 0.28435E+00_JPRB, 0.28959E+00_JPRB, 0.29227E+00_JPRB, 0.29211E+00_JPRB, 0.28926E+00_JPRB, 0.28615E+00_JPRB ,&
&  0.28617E+00_JPRB, 0.29318E+00_JPRB, 0.30659E+00_JPRB, 0.31950E+00_JPRB, 0.32882E+00_JPRB, 0.33878E+00_JPRB, 0.34709E+00_JPRB ,&
&  0.35283E+00_JPRB, 0.35822E+00_JPRB, 0.36474E+00_JPRB, 0.36709E+00_JPRB, 0.36811E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.35701E+00_JPRB, 0.35339E+00_JPRB, 0.35342E+00_JPRB, 0.35370E+00_JPRB, 0.35248E+00_JPRB, 0.35229E+00_JPRB, 0.35152E+00_JPRB ,&
&  0.35123E+00_JPRB, 0.34934E+00_JPRB, 0.34745E+00_JPRB, 0.34786E+00_JPRB, 0.34995E+00_JPRB, 0.35231E+00_JPRB, 0.35290E+00_JPRB ,&
&  0.35268E+00_JPRB, 0.35399E+00_JPRB, 0.35730E+00_JPRB, 0.36079E+00_JPRB, 0.36146E+00_JPRB, 0.36178E+00_JPRB, 0.36362E+00_JPRB ,&
&  0.36596E+00_JPRB, 0.36680E+00_JPRB, 0.36960E+00_JPRB, 0.36740E+00_JPRB, 0.36293E+00_JPRB, 0.36118E+00_JPRB, 0.35963E+00_JPRB ,&
&  0.35309E+00_JPRB, 0.32964E+00_JPRB, 0.31446E+00_JPRB, 0.31903E+00_JPRB, 0.33459E+00_JPRB, 0.33390E+00_JPRB, 0.33171E+00_JPRB ,&
&  0.32945E+00_JPRB, 0.32642E+00_JPRB, 0.32383E+00_JPRB, 0.32287E+00_JPRB, 0.32302E+00_JPRB, 0.32302E+00_JPRB, 0.32164E+00_JPRB ,&
&  0.32210E+00_JPRB, 0.32702E+00_JPRB, 0.33312E+00_JPRB, 0.33025E+00_JPRB, 0.32576E+00_JPRB, 0.31872E+00_JPRB, 0.31422E+00_JPRB ,&
&  0.31001E+00_JPRB, 0.30566E+00_JPRB, 0.30292E+00_JPRB, 0.30203E+00_JPRB, 0.30175E+00_JPRB, 0.29966E+00_JPRB, 0.29607E+00_JPRB ,&
&  0.28886E+00_JPRB, 0.27966E+00_JPRB, 0.27023E+00_JPRB, 0.26096E+00_JPRB, 0.25428E+00_JPRB, 0.24954E+00_JPRB, 0.24584E+00_JPRB ,&
&  0.24765E+00_JPRB, 0.25933E+00_JPRB, 0.27836E+00_JPRB, 0.29833E+00_JPRB, 0.31697E+00_JPRB, 0.29507E+00_JPRB, 0.28463E+00_JPRB ,&
&  0.27331E+00_JPRB, 0.26345E+00_JPRB, 0.25654E+00_JPRB, 0.25175E+00_JPRB, 0.24825E+00_JPRB, 0.24621E+00_JPRB, 0.24601E+00_JPRB ,&
&  0.24896E+00_JPRB, 0.25308E+00_JPRB, 0.25682E+00_JPRB, 0.25784E+00_JPRB, 0.25729E+00_JPRB, 0.25559E+00_JPRB, 0.25466E+00_JPRB ,&
&  0.25598E+00_JPRB, 0.26166E+00_JPRB, 0.27101E+00_JPRB, 0.28454E+00_JPRB, 0.29667E+00_JPRB, 0.30919E+00_JPRB, 0.32363E+00_JPRB ,&
&  0.33890E+00_JPRB, 0.35286E+00_JPRB, 0.35650E+00_JPRB, 0.35976E+00_JPRB, 0.35991E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.35128E+00_JPRB, 0.34820E+00_JPRB, 0.34621E+00_JPRB, 0.34648E+00_JPRB, 0.34658E+00_JPRB, 0.34907E+00_JPRB, 0.34823E+00_JPRB ,&
&  0.34384E+00_JPRB, 0.33922E+00_JPRB, 0.33787E+00_JPRB, 0.33783E+00_JPRB, 0.33676E+00_JPRB, 0.33541E+00_JPRB, 0.33612E+00_JPRB ,&
&  0.33455E+00_JPRB, 0.33644E+00_JPRB, 0.33932E+00_JPRB, 0.34083E+00_JPRB, 0.34154E+00_JPRB, 0.34175E+00_JPRB, 0.34199E+00_JPRB ,&
&  0.33588E+00_JPRB, 0.33407E+00_JPRB, 0.33971E+00_JPRB, 0.33695E+00_JPRB, 0.33300E+00_JPRB, 0.32961E+00_JPRB, 0.32845E+00_JPRB ,&
&  0.31988E+00_JPRB, 0.30799E+00_JPRB, 0.29535E+00_JPRB, 0.30124E+00_JPRB, 0.31809E+00_JPRB, 0.31532E+00_JPRB, 0.31303E+00_JPRB ,&
&  0.31278E+00_JPRB, 0.31212E+00_JPRB, 0.31302E+00_JPRB, 0.31359E+00_JPRB, 0.31467E+00_JPRB, 0.31494E+00_JPRB, 0.31629E+00_JPRB ,&
&  0.31782E+00_JPRB, 0.32398E+00_JPRB, 0.32745E+00_JPRB, 0.30774E+00_JPRB, 0.29088E+00_JPRB, 0.27801E+00_JPRB, 0.26564E+00_JPRB ,&
&  0.25460E+00_JPRB, 0.25227E+00_JPRB, 0.24867E+00_JPRB, 0.24204E+00_JPRB, 0.24246E+00_JPRB, 0.24314E+00_JPRB, 0.24229E+00_JPRB ,&
&  0.24017E+00_JPRB, 0.23886E+00_JPRB, 0.23744E+00_JPRB, 0.23506E+00_JPRB, 0.23419E+00_JPRB, 0.23551E+00_JPRB, 0.23924E+00_JPRB ,&
&  0.24505E+00_JPRB, 0.25146E+00_JPRB, 0.26605E+00_JPRB, 0.27841E+00_JPRB, 0.27448E+00_JPRB, 0.26730E+00_JPRB, 0.26020E+00_JPRB ,&
&  0.24930E+00_JPRB, 0.24393E+00_JPRB, 0.23859E+00_JPRB, 0.23426E+00_JPRB, 0.23225E+00_JPRB, 0.23242E+00_JPRB, 0.23318E+00_JPRB ,&
&  0.23438E+00_JPRB, 0.23645E+00_JPRB, 0.23728E+00_JPRB, 0.23692E+00_JPRB, 0.23616E+00_JPRB, 0.23678E+00_JPRB, 0.23854E+00_JPRB ,&
&  0.24170E+00_JPRB, 0.24714E+00_JPRB, 0.25371E+00_JPRB, 0.26183E+00_JPRB, 0.26888E+00_JPRB, 0.27546E+00_JPRB, 0.28569E+00_JPRB ,&
&  0.29379E+00_JPRB, 0.31251E+00_JPRB, 0.33869E+00_JPRB, 0.36317E+00_JPRB, 0.35727E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.33234E+00_JPRB, 0.32975E+00_JPRB, 0.32712E+00_JPRB, 0.32674E+00_JPRB, 0.32745E+00_JPRB, 0.33017E+00_JPRB, 0.32777E+00_JPRB ,&
&  0.32318E+00_JPRB, 0.32147E+00_JPRB, 0.32281E+00_JPRB, 0.32267E+00_JPRB, 0.32216E+00_JPRB, 0.31201E+00_JPRB, 0.30624E+00_JPRB ,&
&  0.29938E+00_JPRB, 0.29715E+00_JPRB, 0.29535E+00_JPRB, 0.29619E+00_JPRB, 0.29840E+00_JPRB, 0.29977E+00_JPRB, 0.29559E+00_JPRB ,&
&  0.29116E+00_JPRB, 0.28698E+00_JPRB, 0.28259E+00_JPRB, 0.28073E+00_JPRB, 0.28207E+00_JPRB, 0.28341E+00_JPRB, 0.28354E+00_JPRB ,&
&  0.28426E+00_JPRB, 0.28710E+00_JPRB, 0.28977E+00_JPRB, 0.29341E+00_JPRB, 0.30613E+00_JPRB, 0.30409E+00_JPRB, 0.30280E+00_JPRB ,&
&  0.30320E+00_JPRB, 0.30372E+00_JPRB, 0.30491E+00_JPRB, 0.30572E+00_JPRB, 0.31002E+00_JPRB, 0.31290E+00_JPRB, 0.31498E+00_JPRB ,&
&  0.30144E+00_JPRB, 0.29392E+00_JPRB, 0.28635E+00_JPRB, 0.27721E+00_JPRB, 0.26606E+00_JPRB, 0.25709E+00_JPRB, 0.24887E+00_JPRB ,&
&  0.24191E+00_JPRB, 0.23531E+00_JPRB, 0.22928E+00_JPRB, 0.22694E+00_JPRB, 0.22607E+00_JPRB, 0.22473E+00_JPRB, 0.22458E+00_JPRB ,&
&  0.22512E+00_JPRB, 0.22588E+00_JPRB, 0.22636E+00_JPRB, 0.22792E+00_JPRB, 0.23039E+00_JPRB, 0.23427E+00_JPRB, 0.23843E+00_JPRB ,&
&  0.24461E+00_JPRB, 0.24907E+00_JPRB, 0.25435E+00_JPRB, 0.25892E+00_JPRB, 0.25749E+00_JPRB, 0.25234E+00_JPRB, 0.24688E+00_JPRB ,&
&  0.24012E+00_JPRB, 0.23621E+00_JPRB, 0.23222E+00_JPRB, 0.22932E+00_JPRB, 0.22764E+00_JPRB, 0.22663E+00_JPRB, 0.22685E+00_JPRB ,&
&  0.22688E+00_JPRB, 0.22732E+00_JPRB, 0.22680E+00_JPRB, 0.22706E+00_JPRB, 0.22793E+00_JPRB, 0.22986E+00_JPRB, 0.23238E+00_JPRB ,&
&  0.23630E+00_JPRB, 0.24084E+00_JPRB, 0.24618E+00_JPRB, 0.25201E+00_JPRB, 0.26054E+00_JPRB, 0.27169E+00_JPRB, 0.28748E+00_JPRB ,&
&  0.30416E+00_JPRB, 0.33440E+00_JPRB, 0.34639E+00_JPRB, 0.33856E+00_JPRB, 0.33590E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.31643E+00_JPRB, 0.31769E+00_JPRB, 0.31888E+00_JPRB, 0.32114E+00_JPRB, 0.32392E+00_JPRB, 0.32491E+00_JPRB, 0.32514E+00_JPRB ,&
&  0.31590E+00_JPRB, 0.30843E+00_JPRB, 0.30164E+00_JPRB, 0.29141E+00_JPRB, 0.28330E+00_JPRB, 0.27607E+00_JPRB, 0.27086E+00_JPRB ,&
&  0.26588E+00_JPRB, 0.26337E+00_JPRB, 0.26054E+00_JPRB, 0.25968E+00_JPRB, 0.25943E+00_JPRB, 0.26003E+00_JPRB, 0.25853E+00_JPRB ,&
&  0.25682E+00_JPRB, 0.25800E+00_JPRB, 0.25873E+00_JPRB, 0.26319E+00_JPRB, 0.26978E+00_JPRB, 0.27723E+00_JPRB, 0.28451E+00_JPRB ,&
&  0.29380E+00_JPRB, 0.29773E+00_JPRB, 0.30087E+00_JPRB, 0.30163E+00_JPRB, 0.30038E+00_JPRB, 0.29989E+00_JPRB, 0.29790E+00_JPRB ,&
&  0.29767E+00_JPRB, 0.29734E+00_JPRB, 0.29881E+00_JPRB, 0.29997E+00_JPRB, 0.29851E+00_JPRB, 0.29626E+00_JPRB, 0.29092E+00_JPRB ,&
&  0.28288E+00_JPRB, 0.27642E+00_JPRB, 0.27013E+00_JPRB, 0.26520E+00_JPRB, 0.25815E+00_JPRB, 0.25198E+00_JPRB, 0.24592E+00_JPRB ,&
&  0.24068E+00_JPRB, 0.23676E+00_JPRB, 0.23205E+00_JPRB, 0.22841E+00_JPRB, 0.22670E+00_JPRB, 0.22463E+00_JPRB, 0.22373E+00_JPRB ,&
&  0.22368E+00_JPRB, 0.22407E+00_JPRB, 0.22625E+00_JPRB, 0.22804E+00_JPRB, 0.23084E+00_JPRB, 0.23410E+00_JPRB, 0.23604E+00_JPRB ,&
&  0.23949E+00_JPRB, 0.24246E+00_JPRB, 0.24409E+00_JPRB, 0.24531E+00_JPRB, 0.24439E+00_JPRB, 0.24141E+00_JPRB, 0.23840E+00_JPRB ,&
&  0.23440E+00_JPRB, 0.23159E+00_JPRB, 0.22977E+00_JPRB, 0.22777E+00_JPRB, 0.22672E+00_JPRB, 0.22579E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22398E+00_JPRB, 0.22361E+00_JPRB, 0.22324E+00_JPRB, 0.22432E+00_JPRB, 0.22547E+00_JPRB, 0.22761E+00_JPRB, 0.23020E+00_JPRB ,&
&  0.23257E+00_JPRB, 0.23623E+00_JPRB, 0.24020E+00_JPRB, 0.24566E+00_JPRB, 0.25558E+00_JPRB, 0.26361E+00_JPRB, 0.28109E+00_JPRB ,&
&  0.29365E+00_JPRB, 0.31847E+00_JPRB, 0.32875E+00_JPRB, 0.31771E+00_JPRB, 0.31659E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.30586E+00_JPRB, 0.30800E+00_JPRB, 0.30887E+00_JPRB, 0.31085E+00_JPRB, 0.31280E+00_JPRB, 0.31349E+00_JPRB, 0.31373E+00_JPRB ,&
&  0.31133E+00_JPRB, 0.30529E+00_JPRB, 0.30060E+00_JPRB, 0.29409E+00_JPRB, 0.28784E+00_JPRB, 0.28119E+00_JPRB, 0.27523E+00_JPRB ,&
&  0.26896E+00_JPRB, 0.26326E+00_JPRB, 0.25745E+00_JPRB, 0.25255E+00_JPRB, 0.24903E+00_JPRB, 0.24613E+00_JPRB, 0.24308E+00_JPRB ,&
&  0.24047E+00_JPRB, 0.24164E+00_JPRB, 0.24430E+00_JPRB, 0.24939E+00_JPRB, 0.25731E+00_JPRB, 0.26456E+00_JPRB, 0.27251E+00_JPRB ,&
&  0.28623E+00_JPRB, 0.28962E+00_JPRB, 0.28960E+00_JPRB, 0.29172E+00_JPRB, 0.29121E+00_JPRB, 0.29178E+00_JPRB, 0.28197E+00_JPRB ,&
&  0.28463E+00_JPRB, 0.28680E+00_JPRB, 0.29463E+00_JPRB, 0.29770E+00_JPRB, 0.29237E+00_JPRB, 0.28811E+00_JPRB, 0.28027E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.26910E+00_JPRB, 0.26581E+00_JPRB, 0.26387E+00_JPRB, 0.25903E+00_JPRB, 0.25546E+00_JPRB, 0.24948E+00_JPRB ,&
&  0.24455E+00_JPRB, 0.24133E+00_JPRB, 0.23727E+00_JPRB, 0.23475E+00_JPRB, 0.23270E+00_JPRB, 0.23067E+00_JPRB, 0.22914E+00_JPRB ,&
&  0.22829E+00_JPRB, 0.22759E+00_JPRB, 0.22828E+00_JPRB, 0.22908E+00_JPRB, 0.23063E+00_JPRB, 0.23187E+00_JPRB, 0.23211E+00_JPRB ,&
&  0.23238E+00_JPRB, 0.23283E+00_JPRB, 0.23300E+00_JPRB, 0.23370E+00_JPRB, 0.23319E+00_JPRB, 0.23228E+00_JPRB, 0.23114E+00_JPRB ,&
&  0.22988E+00_JPRB, 0.22937E+00_JPRB, 0.22848E+00_JPRB, 0.22791E+00_JPRB, 0.22606E+00_JPRB, 0.22478E+00_JPRB, 0.22407E+00_JPRB ,&
&  0.22341E+00_JPRB, 0.22384E+00_JPRB, 0.22395E+00_JPRB, 0.22480E+00_JPRB, 0.22614E+00_JPRB, 0.22733E+00_JPRB, 0.22946E+00_JPRB ,&
&  0.23215E+00_JPRB, 0.23528E+00_JPRB, 0.24067E+00_JPRB, 0.24631E+00_JPRB, 0.25253E+00_JPRB, 0.26026E+00_JPRB, 0.26708E+00_JPRB ,&
&  0.27673E+00_JPRB, 0.29152E+00_JPRB, 0.29356E+00_JPRB, 0.29789E+00_JPRB, 0.30284E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.29037E+00_JPRB, 0.29420E+00_JPRB, 0.29676E+00_JPRB, 0.29845E+00_JPRB, 0.30014E+00_JPRB, 0.29946E+00_JPRB, 0.29789E+00_JPRB ,&
&  0.29367E+00_JPRB, 0.29222E+00_JPRB, 0.28944E+00_JPRB, 0.28643E+00_JPRB, 0.28292E+00_JPRB, 0.27878E+00_JPRB, 0.27474E+00_JPRB ,&
&  0.27021E+00_JPRB, 0.26526E+00_JPRB, 0.25942E+00_JPRB, 0.25300E+00_JPRB, 0.24712E+00_JPRB, 0.24161E+00_JPRB, 0.23850E+00_JPRB ,&
&  0.23710E+00_JPRB, 0.23797E+00_JPRB, 0.24095E+00_JPRB, 0.24457E+00_JPRB, 0.24895E+00_JPRB, 0.25388E+00_JPRB, 0.26003E+00_JPRB ,&
&  0.26525E+00_JPRB, 0.27149E+00_JPRB, 0.27528E+00_JPRB, 0.27688E+00_JPRB, 0.27550E+00_JPRB, 0.27546E+00_JPRB, 0.27454E+00_JPRB ,&
&  0.27331E+00_JPRB, 0.27141E+00_JPRB, 0.27075E+00_JPRB, 0.27101E+00_JPRB, 0.27032E+00_JPRB, 0.26856E+00_JPRB, 0.26600E+00_JPRB ,&
&  0.26356E+00_JPRB, 0.26147E+00_JPRB, 0.25875E+00_JPRB, 0.25643E+00_JPRB, 0.25469E+00_JPRB, 0.25249E+00_JPRB, 0.24953E+00_JPRB ,&
&  0.24688E+00_JPRB, 0.24426E+00_JPRB, 0.24116E+00_JPRB, 0.23856E+00_JPRB, 0.23598E+00_JPRB, 0.23410E+00_JPRB, 0.23241E+00_JPRB ,&
&  0.23170E+00_JPRB, 0.23094E+00_JPRB, 0.23072E+00_JPRB, 0.23061E+00_JPRB, 0.23104E+00_JPRB, 0.23073E+00_JPRB, 0.23028E+00_JPRB ,&
&  0.22974E+00_JPRB, 0.22960E+00_JPRB, 0.22905E+00_JPRB, 0.22885E+00_JPRB, 0.22864E+00_JPRB, 0.22790E+00_JPRB, 0.22753E+00_JPRB ,&
&  0.22756E+00_JPRB, 0.22741E+00_JPRB, 0.22678E+00_JPRB, 0.22668E+00_JPRB, 0.22627E+00_JPRB, 0.22570E+00_JPRB, 0.22596E+00_JPRB ,&
&  0.22574E+00_JPRB, 0.22593E+00_JPRB, 0.22624E+00_JPRB, 0.22672E+00_JPRB, 0.22783E+00_JPRB, 0.22924E+00_JPRB, 0.23114E+00_JPRB ,&
&  0.23409E+00_JPRB, 0.23677E+00_JPRB, 0.23945E+00_JPRB, 0.24256E+00_JPRB, 0.24856E+00_JPRB, 0.25131E+00_JPRB, 0.25619E+00_JPRB ,&
&  0.26063E+00_JPRB, 0.25858E+00_JPRB, 0.28157E+00_JPRB, 0.28272E+00_JPRB, 0.28508E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.24704E+00_JPRB, 0.25209E+00_JPRB, 0.25574E+00_JPRB, 0.25892E+00_JPRB, 0.25566E+00_JPRB, 0.27552E+00_JPRB, 0.27360E+00_JPRB ,&
&  0.27368E+00_JPRB, 0.26803E+00_JPRB, 0.26953E+00_JPRB, 0.26844E+00_JPRB, 0.26618E+00_JPRB, 0.26321E+00_JPRB, 0.26006E+00_JPRB ,&
&  0.25678E+00_JPRB, 0.25285E+00_JPRB, 0.24926E+00_JPRB, 0.24579E+00_JPRB, 0.24293E+00_JPRB, 0.24041E+00_JPRB, 0.23905E+00_JPRB ,&
&  0.23914E+00_JPRB, 0.24069E+00_JPRB, 0.24336E+00_JPRB, 0.24476E+00_JPRB, 0.24627E+00_JPRB, 0.24787E+00_JPRB, 0.24959E+00_JPRB ,&
&  0.25099E+00_JPRB, 0.25291E+00_JPRB, 0.25392E+00_JPRB, 0.25353E+00_JPRB, 0.25256E+00_JPRB, 0.25223E+00_JPRB, 0.25221E+00_JPRB ,&
&  0.25284E+00_JPRB, 0.25321E+00_JPRB, 0.25342E+00_JPRB, 0.25377E+00_JPRB, 0.25410E+00_JPRB, 0.25366E+00_JPRB, 0.25249E+00_JPRB ,&
&  0.25149E+00_JPRB, 0.25068E+00_JPRB, 0.24900E+00_JPRB, 0.24740E+00_JPRB, 0.24590E+00_JPRB, 0.24432E+00_JPRB, 0.24268E+00_JPRB ,&
&  0.24149E+00_JPRB, 0.24030E+00_JPRB, 0.23882E+00_JPRB, 0.23711E+00_JPRB, 0.23551E+00_JPRB, 0.23410E+00_JPRB, 0.23299E+00_JPRB ,&
&  0.23276E+00_JPRB, 0.23216E+00_JPRB, 0.23163E+00_JPRB, 0.23110E+00_JPRB, 0.23096E+00_JPRB, 0.23049E+00_JPRB, 0.23015E+00_JPRB ,&
&  0.22988E+00_JPRB, 0.22955E+00_JPRB, 0.22918E+00_JPRB, 0.22882E+00_JPRB, 0.22858E+00_JPRB, 0.22824E+00_JPRB, 0.22791E+00_JPRB ,&
&  0.22769E+00_JPRB, 0.22752E+00_JPRB, 0.22733E+00_JPRB, 0.22691E+00_JPRB, 0.22648E+00_JPRB, 0.22597E+00_JPRB, 0.22594E+00_JPRB ,&
&  0.22607E+00_JPRB, 0.22632E+00_JPRB, 0.22681E+00_JPRB, 0.22697E+00_JPRB, 0.22782E+00_JPRB, 0.22863E+00_JPRB, 0.22961E+00_JPRB ,&
&  0.23062E+00_JPRB, 0.23222E+00_JPRB, 0.23397E+00_JPRB, 0.23590E+00_JPRB, 0.23900E+00_JPRB, 0.23999E+00_JPRB, 0.24083E+00_JPRB ,&
&  0.24186E+00_JPRB, 0.24559E+00_JPRB, 0.24660E+00_JPRB, 0.24644E+00_JPRB, 0.24703E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.23568E+00_JPRB, 0.23596E+00_JPRB, 0.23597E+00_JPRB, 0.23583E+00_JPRB, 0.23387E+00_JPRB, 0.23308E+00_JPRB, 0.23237E+00_JPRB ,&
&  0.23184E+00_JPRB, 0.23158E+00_JPRB, 0.23200E+00_JPRB, 0.23256E+00_JPRB, 0.23314E+00_JPRB, 0.23372E+00_JPRB, 0.23423E+00_JPRB ,&
&  0.23447E+00_JPRB, 0.23475E+00_JPRB, 0.23366E+00_JPRB, 0.23313E+00_JPRB, 0.23297E+00_JPRB, 0.23303E+00_JPRB, 0.23301E+00_JPRB ,&
&  0.23312E+00_JPRB, 0.23321E+00_JPRB, 0.23319E+00_JPRB, 0.23356E+00_JPRB, 0.23419E+00_JPRB, 0.23470E+00_JPRB, 0.23524E+00_JPRB ,&
&  0.23576E+00_JPRB, 0.23651E+00_JPRB, 0.23720E+00_JPRB, 0.23785E+00_JPRB, 0.23869E+00_JPRB, 0.23953E+00_JPRB, 0.24003E+00_JPRB ,&
&  0.24041E+00_JPRB, 0.24054E+00_JPRB, 0.24072E+00_JPRB, 0.24088E+00_JPRB, 0.24115E+00_JPRB, 0.24148E+00_JPRB, 0.24155E+00_JPRB ,&
&  0.24149E+00_JPRB, 0.24134E+00_JPRB, 0.24111E+00_JPRB, 0.24056E+00_JPRB, 0.23997E+00_JPRB, 0.23939E+00_JPRB, 0.23881E+00_JPRB ,&
&  0.23832E+00_JPRB, 0.23776E+00_JPRB, 0.23714E+00_JPRB, 0.23648E+00_JPRB, 0.23579E+00_JPRB, 0.23512E+00_JPRB, 0.23444E+00_JPRB ,&
&  0.23375E+00_JPRB, 0.23330E+00_JPRB, 0.23281E+00_JPRB, 0.23239E+00_JPRB, 0.23205E+00_JPRB, 0.23172E+00_JPRB, 0.23139E+00_JPRB ,&
&  0.23107E+00_JPRB, 0.23068E+00_JPRB, 0.23033E+00_JPRB, 0.22994E+00_JPRB, 0.22959E+00_JPRB, 0.22919E+00_JPRB, 0.22887E+00_JPRB ,&
&  0.22856E+00_JPRB, 0.22829E+00_JPRB, 0.22806E+00_JPRB, 0.22795E+00_JPRB, 0.22788E+00_JPRB, 0.22790E+00_JPRB, 0.22793E+00_JPRB ,&
&  0.22798E+00_JPRB, 0.22800E+00_JPRB, 0.22801E+00_JPRB, 0.22786E+00_JPRB, 0.22806E+00_JPRB, 0.22827E+00_JPRB, 0.22857E+00_JPRB ,&
&  0.22886E+00_JPRB, 0.22933E+00_JPRB, 0.22982E+00_JPRB, 0.23036E+00_JPRB, 0.23121E+00_JPRB, 0.23152E+00_JPRB, 0.23179E+00_JPRB ,&
&  0.23208E+00_JPRB, 0.23271E+00_JPRB, 0.23324E+00_JPRB, 0.23352E+00_JPRB, 0.23411E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB, 0.23163E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB, 0.10462E+01_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.10143E+01_JPRB, 0.10089E+01_JPRB, 0.10029E+01_JPRB, 0.99660E+00_JPRB, 0.98956E+00_JPRB, 0.98187E+00_JPRB, 0.97289E+00_JPRB ,&
&  0.96415E+00_JPRB, 0.95569E+00_JPRB, 0.94825E+00_JPRB, 0.94161E+00_JPRB, 0.93563E+00_JPRB, 0.93100E+00_JPRB, 0.92913E+00_JPRB ,&
&  0.92888E+00_JPRB, 0.92861E+00_JPRB, 0.92894E+00_JPRB, 0.92914E+00_JPRB, 0.92881E+00_JPRB, 0.92810E+00_JPRB, 0.92727E+00_JPRB ,&
&  0.92685E+00_JPRB, 0.92677E+00_JPRB, 0.92659E+00_JPRB, 0.92715E+00_JPRB, 0.92805E+00_JPRB, 0.92913E+00_JPRB, 0.93009E+00_JPRB ,&
&  0.93197E+00_JPRB, 0.93425E+00_JPRB, 0.93723E+00_JPRB, 0.94057E+00_JPRB, 0.94688E+00_JPRB, 0.95366E+00_JPRB, 0.96078E+00_JPRB ,&
&  0.96767E+00_JPRB, 0.97483E+00_JPRB, 0.98136E+00_JPRB, 0.98725E+00_JPRB, 0.99268E+00_JPRB, 0.99894E+00_JPRB, 0.10064E+01_JPRB ,&
&  0.10147E+01_JPRB, 0.10237E+01_JPRB, 0.10333E+01_JPRB, 0.10431E+01_JPRB, 0.10523E+01_JPRB, 0.10617E+01_JPRB, 0.10715E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10909E+01_JPRB, 0.11006E+01_JPRB, 0.11101E+01_JPRB, 0.11196E+01_JPRB, 0.11304E+01_JPRB, 0.11427E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.11665E+01_JPRB, 0.11783E+01_JPRB, 0.11910E+01_JPRB, 0.12075E+01_JPRB, 0.12157E+01_JPRB, 0.12240E+01_JPRB ,&
&  0.12319E+01_JPRB, 0.12429E+01_JPRB, 0.12496E+01_JPRB, 0.12567E+01_JPRB, 0.12634E+01_JPRB, 0.12689E+01_JPRB, 0.12719E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.12766E+01_JPRB, 0.12762E+01_JPRB, 0.12736E+01_JPRB, 0.12696E+01_JPRB, 0.12649E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.12480E+01_JPRB, 0.12403E+01_JPRB, 0.12333E+01_JPRB, 0.12236E+01_JPRB, 0.12139E+01_JPRB, 0.12041E+01_JPRB, 0.11941E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.11690E+01_JPRB, 0.11556E+01_JPRB, 0.11414E+01_JPRB, 0.11274E+01_JPRB, 0.11130E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10652E+01_JPRB, 0.10482E+01_JPRB, 0.10316E+01_JPRB, 0.10193E+01_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.98430E+00_JPRB, 0.97179E+00_JPRB, 0.96593E+00_JPRB, 0.96145E+00_JPRB, 0.95664E+00_JPRB, 0.95024E+00_JPRB, 0.94328E+00_JPRB ,&
&  0.93777E+00_JPRB, 0.93038E+00_JPRB, 0.92412E+00_JPRB, 0.92175E+00_JPRB, 0.91781E+00_JPRB, 0.91553E+00_JPRB, 0.91351E+00_JPRB ,&
&  0.91203E+00_JPRB, 0.91015E+00_JPRB, 0.90870E+00_JPRB, 0.90566E+00_JPRB, 0.90109E+00_JPRB, 0.89518E+00_JPRB, 0.89037E+00_JPRB ,&
&  0.88705E+00_JPRB, 0.88466E+00_JPRB, 0.88136E+00_JPRB, 0.87853E+00_JPRB, 0.87909E+00_JPRB, 0.87898E+00_JPRB, 0.87798E+00_JPRB ,&
&  0.88175E+00_JPRB, 0.88516E+00_JPRB, 0.88582E+00_JPRB, 0.88406E+00_JPRB, 0.88137E+00_JPRB, 0.88037E+00_JPRB, 0.88397E+00_JPRB ,&
&  0.88921E+00_JPRB, 0.90503E+00_JPRB, 0.91894E+00_JPRB, 0.93217E+00_JPRB, 0.94457E+00_JPRB, 0.95958E+00_JPRB, 0.97724E+00_JPRB ,&
&  0.99616E+00_JPRB, 0.10129E+01_JPRB, 0.10312E+01_JPRB, 0.10527E+01_JPRB, 0.10752E+01_JPRB, 0.10985E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11693E+01_JPRB, 0.11959E+01_JPRB, 0.12242E+01_JPRB, 0.12492E+01_JPRB, 0.12793E+01_JPRB, 0.13134E+01_JPRB ,&
&  0.13537E+01_JPRB, 0.13770E+01_JPRB, 0.14029E+01_JPRB, 0.14295E+01_JPRB, 0.14608E+01_JPRB, 0.14811E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.15142E+01_JPRB, 0.15243E+01_JPRB, 0.15214E+01_JPRB, 0.15157E+01_JPRB, 0.15123E+01_JPRB, 0.15057E+01_JPRB, 0.14978E+01_JPRB ,&
&  0.14911E+01_JPRB, 0.14876E+01_JPRB, 0.14777E+01_JPRB, 0.14650E+01_JPRB, 0.14481E+01_JPRB, 0.14304E+01_JPRB, 0.14008E+01_JPRB ,&
&  0.13730E+01_JPRB, 0.13512E+01_JPRB, 0.13356E+01_JPRB, 0.13176E+01_JPRB, 0.13001E+01_JPRB, 0.12839E+01_JPRB, 0.12680E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.12350E+01_JPRB, 0.12164E+01_JPRB, 0.11966E+01_JPRB, 0.11802E+01_JPRB, 0.11578E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.10835E+01_JPRB, 0.10578E+01_JPRB, 0.10286E+01_JPRB, 0.10010E+01_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.10001E+01_JPRB, 0.97129E+00_JPRB, 0.95468E+00_JPRB, 0.94833E+00_JPRB, 0.94329E+00_JPRB, 0.93864E+00_JPRB, 0.93406E+00_JPRB ,&
&  0.92620E+00_JPRB, 0.91889E+00_JPRB, 0.91803E+00_JPRB, 0.91832E+00_JPRB, 0.91647E+00_JPRB, 0.91368E+00_JPRB, 0.90921E+00_JPRB ,&
&  0.90374E+00_JPRB, 0.89614E+00_JPRB, 0.88766E+00_JPRB, 0.87654E+00_JPRB, 0.86483E+00_JPRB, 0.85136E+00_JPRB, 0.83295E+00_JPRB ,&
&  0.81828E+00_JPRB, 0.80505E+00_JPRB, 0.78834E+00_JPRB, 0.78234E+00_JPRB, 0.77988E+00_JPRB, 0.74277E+00_JPRB, 0.73490E+00_JPRB ,&
&  0.74562E+00_JPRB, 0.75013E+00_JPRB, 0.75372E+00_JPRB, 0.75745E+00_JPRB, 0.75545E+00_JPRB, 0.73857E+00_JPRB, 0.73239E+00_JPRB ,&
&  0.70360E+00_JPRB, 0.72501E+00_JPRB, 0.74067E+00_JPRB, 0.75560E+00_JPRB, 0.77091E+00_JPRB, 0.79217E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.91769E+00_JPRB, 0.95245E+00_JPRB, 0.99348E+00_JPRB, 0.10345E+01_JPRB, 0.10847E+01_JPRB, 0.11361E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.12760E+01_JPRB, 0.13319E+01_JPRB, 0.13992E+01_JPRB, 0.14582E+01_JPRB, 0.15115E+01_JPRB, 0.15522E+01_JPRB ,&
&  0.15935E+01_JPRB, 0.16235E+01_JPRB, 0.16462E+01_JPRB, 0.16599E+01_JPRB, 0.16756E+01_JPRB, 0.16852E+01_JPRB, 0.16961E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.17061E+01_JPRB, 0.16949E+01_JPRB, 0.16718E+01_JPRB, 0.16575E+01_JPRB, 0.16529E+01_JPRB, 0.16430E+01_JPRB ,&
&  0.16299E+01_JPRB, 0.16148E+01_JPRB, 0.15942E+01_JPRB, 0.15687E+01_JPRB, 0.15337E+01_JPRB, 0.15026E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.14047E+01_JPRB, 0.13821E+01_JPRB, 0.13565E+01_JPRB, 0.13296E+01_JPRB, 0.13027E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.12471E+01_JPRB, 0.12233E+01_JPRB, 0.12047E+01_JPRB, 0.11855E+01_JPRB, 0.11714E+01_JPRB, 0.11532E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11062E+01_JPRB, 0.10843E+01_JPRB, 0.10577E+01_JPRB, 0.10256E+01_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.10340E+01_JPRB, 0.10057E+01_JPRB, 0.97165E+00_JPRB, 0.95070E+00_JPRB, 0.94181E+00_JPRB, 0.93209E+00_JPRB, 0.92285E+00_JPRB ,&
&  0.91625E+00_JPRB, 0.91613E+00_JPRB, 0.91759E+00_JPRB, 0.91608E+00_JPRB, 0.91163E+00_JPRB, 0.90437E+00_JPRB, 0.89475E+00_JPRB ,&
&  0.88222E+00_JPRB, 0.86764E+00_JPRB, 0.85500E+00_JPRB, 0.84285E+00_JPRB, 0.82991E+00_JPRB, 0.81743E+00_JPRB, 0.79825E+00_JPRB ,&
&  0.78138E+00_JPRB, 0.76610E+00_JPRB, 0.71464E+00_JPRB, 0.71316E+00_JPRB, 0.70349E+00_JPRB, 0.70172E+00_JPRB, 0.70667E+00_JPRB ,&
&  0.70227E+00_JPRB, 0.68674E+00_JPRB, 0.67007E+00_JPRB, 0.64935E+00_JPRB, 0.63374E+00_JPRB, 0.61629E+00_JPRB, 0.60935E+00_JPRB ,&
&  0.60764E+00_JPRB, 0.61328E+00_JPRB, 0.62627E+00_JPRB, 0.64199E+00_JPRB, 0.65938E+00_JPRB, 0.67513E+00_JPRB, 0.70046E+00_JPRB ,&
&  0.76256E+00_JPRB, 0.84605E+00_JPRB, 0.95213E+00_JPRB, 0.10123E+01_JPRB, 0.10867E+01_JPRB, 0.11563E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.13323E+01_JPRB, 0.14017E+01_JPRB, 0.14750E+01_JPRB, 0.15522E+01_JPRB, 0.16219E+01_JPRB, 0.17033E+01_JPRB ,&
&  0.17725E+01_JPRB, 0.18352E+01_JPRB, 0.18720E+01_JPRB, 0.18934E+01_JPRB, 0.18998E+01_JPRB, 0.18869E+01_JPRB, 0.18713E+01_JPRB ,&
&  0.18612E+01_JPRB, 0.18462E+01_JPRB, 0.18259E+01_JPRB, 0.17773E+01_JPRB, 0.17563E+01_JPRB, 0.17502E+01_JPRB, 0.17243E+01_JPRB ,&
&  0.16879E+01_JPRB, 0.16460E+01_JPRB, 0.16028E+01_JPRB, 0.15428E+01_JPRB, 0.14878E+01_JPRB, 0.14664E+01_JPRB, 0.14459E+01_JPRB ,&
&  0.14277E+01_JPRB, 0.14062E+01_JPRB, 0.13849E+01_JPRB, 0.13527E+01_JPRB, 0.13059E+01_JPRB, 0.12617E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11652E+01_JPRB, 0.11401E+01_JPRB, 0.11145E+01_JPRB, 0.10980E+01_JPRB, 0.10840E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.10754E+01_JPRB, 0.10755E+01_JPRB, 0.10731E+01_JPRB, 0.10683E+01_JPRB, 0.10541E+01_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.11252E+01_JPRB, 0.11125E+01_JPRB, 0.10822E+01_JPRB, 0.10410E+01_JPRB, 0.10057E+01_JPRB, 0.97720E+00_JPRB, 0.96066E+00_JPRB ,&
&  0.94921E+00_JPRB, 0.94300E+00_JPRB, 0.93535E+00_JPRB, 0.92759E+00_JPRB, 0.91636E+00_JPRB, 0.90425E+00_JPRB, 0.89080E+00_JPRB ,&
&  0.87148E+00_JPRB, 0.85158E+00_JPRB, 0.83314E+00_JPRB, 0.82070E+00_JPRB, 0.81040E+00_JPRB, 0.80299E+00_JPRB, 0.79597E+00_JPRB ,&
&  0.78874E+00_JPRB, 0.78003E+00_JPRB, 0.75219E+00_JPRB, 0.74283E+00_JPRB, 0.73143E+00_JPRB, 0.73100E+00_JPRB, 0.72422E+00_JPRB ,&
&  0.73672E+00_JPRB, 0.65693E+00_JPRB, 0.65226E+00_JPRB, 0.63381E+00_JPRB, 0.61912E+00_JPRB, 0.60177E+00_JPRB, 0.58498E+00_JPRB ,&
&  0.57731E+00_JPRB, 0.56653E+00_JPRB, 0.56284E+00_JPRB, 0.57451E+00_JPRB, 0.58634E+00_JPRB, 0.61049E+00_JPRB, 0.63556E+00_JPRB ,&
&  0.67440E+00_JPRB, 0.84351E+00_JPRB, 0.92977E+00_JPRB, 0.99378E+00_JPRB, 0.10623E+01_JPRB, 0.11208E+01_JPRB, 0.11553E+01_JPRB ,&
&  0.11986E+01_JPRB, 0.12513E+01_JPRB, 0.13054E+01_JPRB, 0.13411E+01_JPRB, 0.14752E+01_JPRB, 0.15742E+01_JPRB, 0.16884E+01_JPRB ,&
&  0.17969E+01_JPRB, 0.19119E+01_JPRB, 0.20157E+01_JPRB, 0.20994E+01_JPRB, 0.21594E+01_JPRB, 0.21726E+01_JPRB, 0.21492E+01_JPRB ,&
&  0.21364E+01_JPRB, 0.21202E+01_JPRB, 0.21134E+01_JPRB, 0.20485E+01_JPRB, 0.20111E+01_JPRB, 0.19804E+01_JPRB, 0.19058E+01_JPRB ,&
&  0.18197E+01_JPRB, 0.17421E+01_JPRB, 0.16636E+01_JPRB, 0.15794E+01_JPRB, 0.14990E+01_JPRB, 0.14509E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.13854E+01_JPRB, 0.13488E+01_JPRB, 0.12962E+01_JPRB, 0.12443E+01_JPRB, 0.12084E+01_JPRB, 0.11727E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11026E+01_JPRB, 0.10792E+01_JPRB, 0.10627E+01_JPRB, 0.10561E+01_JPRB, 0.10559E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10803E+01_JPRB, 0.10936E+01_JPRB, 0.11089E+01_JPRB, 0.11212E+01_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.11269E+01_JPRB, 0.11330E+01_JPRB, 0.11296E+01_JPRB, 0.11166E+01_JPRB, 0.10942E+01_JPRB, 0.10670E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10165E+01_JPRB, 0.99657E+00_JPRB, 0.97723E+00_JPRB, 0.96087E+00_JPRB, 0.94213E+00_JPRB, 0.92798E+00_JPRB, 0.91248E+00_JPRB ,&
&  0.90164E+00_JPRB, 0.89514E+00_JPRB, 0.88260E+00_JPRB, 0.86845E+00_JPRB, 0.84681E+00_JPRB, 0.82851E+00_JPRB, 0.81417E+00_JPRB ,&
&  0.80068E+00_JPRB, 0.78958E+00_JPRB, 0.76284E+00_JPRB, 0.74457E+00_JPRB, 0.73438E+00_JPRB, 0.72083E+00_JPRB, 0.72592E+00_JPRB ,&
&  0.71794E+00_JPRB, 0.71981E+00_JPRB, 0.72284E+00_JPRB, 0.71542E+00_JPRB, 0.70479E+00_JPRB, 0.57586E+00_JPRB, 0.54525E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.52886E+00_JPRB, 0.52791E+00_JPRB, 0.53617E+00_JPRB, 0.55077E+00_JPRB, 0.59176E+00_JPRB, 0.62463E+00_JPRB ,&
&  0.64343E+00_JPRB, 0.81974E+00_JPRB, 0.89298E+00_JPRB, 0.95760E+00_JPRB, 0.10100E+01_JPRB, 0.10465E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.10926E+01_JPRB, 0.11282E+01_JPRB, 0.11556E+01_JPRB, 0.12111E+01_JPRB, 0.13125E+01_JPRB, 0.14701E+01_JPRB, 0.16325E+01_JPRB ,&
&  0.18120E+01_JPRB, 0.19490E+01_JPRB, 0.20838E+01_JPRB, 0.21904E+01_JPRB, 0.23019E+01_JPRB, 0.23756E+01_JPRB, 0.24083E+01_JPRB ,&
&  0.24456E+01_JPRB, 0.24208E+01_JPRB, 0.24458E+01_JPRB, 0.24024E+01_JPRB, 0.23611E+01_JPRB, 0.23059E+01_JPRB, 0.21608E+01_JPRB ,&
&  0.19887E+01_JPRB, 0.18720E+01_JPRB, 0.17626E+01_JPRB, 0.16235E+01_JPRB, 0.15197E+01_JPRB, 0.14448E+01_JPRB, 0.14112E+01_JPRB ,&
&  0.13843E+01_JPRB, 0.13663E+01_JPRB, 0.13144E+01_JPRB, 0.12578E+01_JPRB, 0.12195E+01_JPRB, 0.11922E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.10903E+01_JPRB, 0.10688E+01_JPRB, 0.10502E+01_JPRB, 0.10361E+01_JPRB, 0.10297E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.10472E+01_JPRB, 0.10653E+01_JPRB, 0.10900E+01_JPRB, 0.11175E+01_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.11082E+01_JPRB, 0.11194E+01_JPRB, 0.11207E+01_JPRB, 0.11119E+01_JPRB, 0.10907E+01_JPRB, 0.10657E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10025E+01_JPRB, 0.98348E+00_JPRB, 0.96829E+00_JPRB, 0.95419E+00_JPRB, 0.94297E+00_JPRB, 0.93397E+00_JPRB ,&
&  0.92693E+00_JPRB, 0.89608E+00_JPRB, 0.88274E+00_JPRB, 0.87995E+00_JPRB, 0.88016E+00_JPRB, 0.87606E+00_JPRB, 0.86947E+00_JPRB ,&
&  0.84288E+00_JPRB, 0.80590E+00_JPRB, 0.77101E+00_JPRB, 0.74127E+00_JPRB, 0.71592E+00_JPRB, 0.70092E+00_JPRB, 0.70097E+00_JPRB ,&
&  0.70132E+00_JPRB, 0.68934E+00_JPRB, 0.68788E+00_JPRB, 0.70481E+00_JPRB, 0.70294E+00_JPRB, 0.63187E+00_JPRB, 0.54095E+00_JPRB ,&
&  0.52827E+00_JPRB, 0.50711E+00_JPRB, 0.50738E+00_JPRB, 0.51479E+00_JPRB, 0.53357E+00_JPRB, 0.56493E+00_JPRB, 0.61515E+00_JPRB ,&
&  0.66993E+00_JPRB, 0.79069E+00_JPRB, 0.86576E+00_JPRB, 0.91999E+00_JPRB, 0.96816E+00_JPRB, 0.10049E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.10603E+01_JPRB, 0.11090E+01_JPRB, 0.11703E+01_JPRB, 0.12732E+01_JPRB, 0.14768E+01_JPRB, 0.16807E+01_JPRB ,&
&  0.19592E+01_JPRB, 0.21588E+01_JPRB, 0.22285E+01_JPRB, 0.22386E+01_JPRB, 0.23583E+01_JPRB, 0.25270E+01_JPRB, 0.25981E+01_JPRB ,&
&  0.27276E+01_JPRB, 0.28685E+01_JPRB, 0.28343E+01_JPRB, 0.26872E+01_JPRB, 0.25932E+01_JPRB, 0.25326E+01_JPRB, 0.23297E+01_JPRB ,&
&  0.21002E+01_JPRB, 0.20070E+01_JPRB, 0.20046E+01_JPRB, 0.17852E+01_JPRB, 0.16302E+01_JPRB, 0.14863E+01_JPRB, 0.13978E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.12876E+01_JPRB, 0.12246E+01_JPRB, 0.11763E+01_JPRB, 0.11492E+01_JPRB, 0.11297E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.10877E+01_JPRB, 0.10666E+01_JPRB, 0.10454E+01_JPRB, 0.10302E+01_JPRB, 0.10144E+01_JPRB, 0.10043E+01_JPRB, 0.99975E+00_JPRB ,&
&  0.10019E+01_JPRB, 0.10132E+01_JPRB, 0.10324E+01_JPRB, 0.10586E+01_JPRB, 0.10891E+01_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.11099E+01_JPRB, 0.11157E+01_JPRB, 0.10978E+01_JPRB, 0.10656E+01_JPRB, 0.10237E+01_JPRB, 0.97171E+00_JPRB, 0.93411E+00_JPRB ,&
&  0.91171E+00_JPRB, 0.90160E+00_JPRB, 0.89059E+00_JPRB, 0.87997E+00_JPRB, 0.87440E+00_JPRB, 0.87202E+00_JPRB, 0.87240E+00_JPRB ,&
&  0.87479E+00_JPRB, 0.87698E+00_JPRB, 0.87508E+00_JPRB, 0.86063E+00_JPRB, 0.85668E+00_JPRB, 0.85786E+00_JPRB, 0.86269E+00_JPRB ,&
&  0.85524E+00_JPRB, 0.82342E+00_JPRB, 0.78418E+00_JPRB, 0.75501E+00_JPRB, 0.72444E+00_JPRB, 0.70787E+00_JPRB, 0.68812E+00_JPRB ,&
&  0.66277E+00_JPRB, 0.65419E+00_JPRB, 0.67102E+00_JPRB, 0.69923E+00_JPRB, 0.68913E+00_JPRB, 0.70052E+00_JPRB, 0.57679E+00_JPRB ,&
&  0.53528E+00_JPRB, 0.51107E+00_JPRB, 0.50594E+00_JPRB, 0.52085E+00_JPRB, 0.56123E+00_JPRB, 0.60918E+00_JPRB, 0.65378E+00_JPRB ,&
&  0.69896E+00_JPRB, 0.77671E+00_JPRB, 0.82289E+00_JPRB, 0.85977E+00_JPRB, 0.90306E+00_JPRB, 0.94563E+00_JPRB, 0.97727E+00_JPRB ,&
&  0.10017E+01_JPRB, 0.10495E+01_JPRB, 0.11226E+01_JPRB, 0.12366E+01_JPRB, 0.14656E+01_JPRB, 0.14952E+01_JPRB, 0.16128E+01_JPRB ,&
&  0.21846E+01_JPRB, 0.24946E+01_JPRB, 0.24316E+01_JPRB, 0.23825E+01_JPRB, 0.24062E+01_JPRB, 0.25361E+01_JPRB, 0.27334E+01_JPRB ,&
&  0.30295E+01_JPRB, 0.32604E+01_JPRB, 0.32372E+01_JPRB, 0.29105E+01_JPRB, 0.27460E+01_JPRB, 0.26479E+01_JPRB, 0.23733E+01_JPRB ,&
&  0.20638E+01_JPRB, 0.20841E+01_JPRB, 0.21537E+01_JPRB, 0.18044E+01_JPRB, 0.15667E+01_JPRB, 0.14159E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.12492E+01_JPRB, 0.12043E+01_JPRB, 0.11649E+01_JPRB, 0.11203E+01_JPRB, 0.10760E+01_JPRB, 0.10365E+01_JPRB ,&
&  0.10060E+01_JPRB, 0.98281E+00_JPRB, 0.97390E+00_JPRB, 0.97258E+00_JPRB, 0.96880E+00_JPRB, 0.96495E+00_JPRB, 0.97024E+00_JPRB ,&
&  0.98287E+00_JPRB, 0.98881E+00_JPRB, 0.10080E+01_JPRB, 0.10369E+01_JPRB, 0.10764E+01_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.10634E+01_JPRB, 0.10566E+01_JPRB, 0.10312E+01_JPRB, 0.99178E+00_JPRB, 0.94493E+00_JPRB, 0.88196E+00_JPRB, 0.83199E+00_JPRB ,&
&  0.80315E+00_JPRB, 0.78683E+00_JPRB, 0.77609E+00_JPRB, 0.77014E+00_JPRB, 0.76952E+00_JPRB, 0.77456E+00_JPRB, 0.77471E+00_JPRB ,&
&  0.77720E+00_JPRB, 0.78041E+00_JPRB, 0.78638E+00_JPRB, 0.79605E+00_JPRB, 0.80284E+00_JPRB, 0.80868E+00_JPRB, 0.81572E+00_JPRB ,&
&  0.82267E+00_JPRB, 0.80887E+00_JPRB, 0.77593E+00_JPRB, 0.75093E+00_JPRB, 0.74767E+00_JPRB, 0.75626E+00_JPRB, 0.72865E+00_JPRB ,&
&  0.67964E+00_JPRB, 0.65560E+00_JPRB, 0.65117E+00_JPRB, 0.70669E+00_JPRB, 0.74470E+00_JPRB, 0.75190E+00_JPRB, 0.62859E+00_JPRB ,&
&  0.56002E+00_JPRB, 0.53130E+00_JPRB, 0.53502E+00_JPRB, 0.57099E+00_JPRB, 0.63433E+00_JPRB, 0.70174E+00_JPRB, 0.78188E+00_JPRB ,&
&  0.82853E+00_JPRB, 0.87214E+00_JPRB, 0.88638E+00_JPRB, 0.91349E+00_JPRB, 0.95956E+00_JPRB, 0.10050E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.11512E+01_JPRB, 0.12943E+01_JPRB, 0.12234E+01_JPRB, 0.13006E+01_JPRB, 0.14186E+01_JPRB, 0.16225E+01_JPRB ,&
&  0.20221E+01_JPRB, 0.22214E+01_JPRB, 0.23338E+01_JPRB, 0.25661E+01_JPRB, 0.25222E+01_JPRB, 0.25146E+01_JPRB, 0.26428E+01_JPRB ,&
&  0.30901E+01_JPRB, 0.34605E+01_JPRB, 0.33682E+01_JPRB, 0.30395E+01_JPRB, 0.29128E+01_JPRB, 0.28311E+01_JPRB, 0.24927E+01_JPRB ,&
&  0.20684E+01_JPRB, 0.18255E+01_JPRB, 0.16171E+01_JPRB, 0.13864E+01_JPRB, 0.12903E+01_JPRB, 0.12335E+01_JPRB, 0.11976E+01_JPRB ,&
&  0.11724E+01_JPRB, 0.11472E+01_JPRB, 0.11284E+01_JPRB, 0.10528E+01_JPRB, 0.98165E+00_JPRB, 0.89971E+00_JPRB, 0.84185E+00_JPRB ,&
&  0.80484E+00_JPRB, 0.79717E+00_JPRB, 0.81063E+00_JPRB, 0.83831E+00_JPRB, 0.89566E+00_JPRB, 0.94558E+00_JPRB, 0.97786E+00_JPRB ,&
&  0.99222E+00_JPRB, 0.97464E+00_JPRB, 0.97223E+00_JPRB, 0.98884E+00_JPRB, 0.10372E+01_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.99137E+00_JPRB, 0.94102E+00_JPRB, 0.90866E+00_JPRB, 0.90686E+00_JPRB, 0.85147E+00_JPRB, 0.79658E+00_JPRB, 0.76274E+00_JPRB ,&
&  0.76128E+00_JPRB, 0.75835E+00_JPRB, 0.75615E+00_JPRB, 0.72443E+00_JPRB, 0.69690E+00_JPRB, 0.68273E+00_JPRB, 0.67793E+00_JPRB ,&
&  0.69214E+00_JPRB, 0.71023E+00_JPRB, 0.72593E+00_JPRB, 0.72618E+00_JPRB, 0.72525E+00_JPRB, 0.73134E+00_JPRB, 0.74892E+00_JPRB ,&
&  0.77240E+00_JPRB, 0.78084E+00_JPRB, 0.77091E+00_JPRB, 0.76244E+00_JPRB, 0.77758E+00_JPRB, 0.79270E+00_JPRB, 0.75514E+00_JPRB ,&
&  0.71671E+00_JPRB, 0.68345E+00_JPRB, 0.67412E+00_JPRB, 0.71300E+00_JPRB, 0.78205E+00_JPRB, 0.80185E+00_JPRB, 0.67943E+00_JPRB ,&
&  0.59362E+00_JPRB, 0.57706E+00_JPRB, 0.62581E+00_JPRB, 0.75243E+00_JPRB, 0.81227E+00_JPRB, 0.86895E+00_JPRB, 0.93909E+00_JPRB ,&
&  0.89660E+00_JPRB, 0.89579E+00_JPRB, 0.92785E+00_JPRB, 0.10751E+01_JPRB, 0.11254E+01_JPRB, 0.11556E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.12800E+01_JPRB, 0.14719E+01_JPRB, 0.13423E+01_JPRB, 0.15267E+01_JPRB, 0.18615E+01_JPRB, 0.19175E+01_JPRB ,&
&  0.21191E+01_JPRB, 0.22481E+01_JPRB, 0.25930E+01_JPRB, 0.26226E+01_JPRB, 0.25175E+01_JPRB, 0.25387E+01_JPRB, 0.27893E+01_JPRB ,&
&  0.33435E+01_JPRB, 0.37660E+01_JPRB, 0.36378E+01_JPRB, 0.34423E+01_JPRB, 0.32977E+01_JPRB, 0.31051E+01_JPRB, 0.27268E+01_JPRB ,&
&  0.22115E+01_JPRB, 0.18241E+01_JPRB, 0.15308E+01_JPRB, 0.12836E+01_JPRB, 0.11665E+01_JPRB, 0.11213E+01_JPRB, 0.10723E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.10460E+01_JPRB, 0.10309E+01_JPRB, 0.94975E+00_JPRB, 0.87577E+00_JPRB, 0.80922E+00_JPRB, 0.76138E+00_JPRB ,&
&  0.74237E+00_JPRB, 0.72251E+00_JPRB, 0.71266E+00_JPRB, 0.73994E+00_JPRB, 0.82175E+00_JPRB, 0.91933E+00_JPRB, 0.96878E+00_JPRB ,&
&  0.95521E+00_JPRB, 0.91924E+00_JPRB, 0.94273E+00_JPRB, 0.99227E+00_JPRB, 0.10159E+01_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.99058E+00_JPRB, 0.97617E+00_JPRB, 0.96518E+00_JPRB, 0.94972E+00_JPRB, 0.91940E+00_JPRB, 0.85266E+00_JPRB, 0.80987E+00_JPRB ,&
&  0.81750E+00_JPRB, 0.79341E+00_JPRB, 0.77616E+00_JPRB, 0.73702E+00_JPRB, 0.69971E+00_JPRB, 0.66170E+00_JPRB, 0.64363E+00_JPRB ,&
&  0.62848E+00_JPRB, 0.64914E+00_JPRB, 0.68949E+00_JPRB, 0.70211E+00_JPRB, 0.70417E+00_JPRB, 0.70592E+00_JPRB, 0.72703E+00_JPRB ,&
&  0.75290E+00_JPRB, 0.77811E+00_JPRB, 0.79131E+00_JPRB, 0.80639E+00_JPRB, 0.82217E+00_JPRB, 0.83572E+00_JPRB, 0.82014E+00_JPRB ,&
&  0.79913E+00_JPRB, 0.76313E+00_JPRB, 0.73452E+00_JPRB, 0.75583E+00_JPRB, 0.81681E+00_JPRB, 0.87229E+00_JPRB, 0.83036E+00_JPRB ,&
&  0.66785E+00_JPRB, 0.67512E+00_JPRB, 0.85234E+00_JPRB, 0.92378E+00_JPRB, 0.98941E+00_JPRB, 0.10292E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.11267E+01_JPRB, 0.11719E+01_JPRB, 0.12073E+01_JPRB, 0.12345E+01_JPRB, 0.12613E+01_JPRB, 0.13003E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.12126E+01_JPRB, 0.11992E+01_JPRB, 0.13617E+01_JPRB, 0.18611E+01_JPRB, 0.21103E+01_JPRB, 0.21471E+01_JPRB ,&
&  0.22927E+01_JPRB, 0.23492E+01_JPRB, 0.25327E+01_JPRB, 0.27903E+01_JPRB, 0.29115E+01_JPRB, 0.31242E+01_JPRB, 0.34163E+01_JPRB ,&
&  0.38208E+01_JPRB, 0.42130E+01_JPRB, 0.42077E+01_JPRB, 0.40427E+01_JPRB, 0.37641E+01_JPRB, 0.34150E+01_JPRB, 0.28873E+01_JPRB ,&
&  0.22696E+01_JPRB, 0.18317E+01_JPRB, 0.14695E+01_JPRB, 0.12233E+01_JPRB, 0.10831E+01_JPRB, 0.10206E+01_JPRB, 0.97597E+00_JPRB ,&
&  0.97442E+00_JPRB, 0.96751E+00_JPRB, 0.94712E+00_JPRB, 0.88358E+00_JPRB, 0.83222E+00_JPRB, 0.82615E+00_JPRB, 0.82916E+00_JPRB ,&
&  0.82917E+00_JPRB, 0.82070E+00_JPRB, 0.77023E+00_JPRB, 0.79126E+00_JPRB, 0.87038E+00_JPRB, 0.95494E+00_JPRB, 0.98625E+00_JPRB ,&
&  0.98696E+00_JPRB, 0.95080E+00_JPRB, 0.96054E+00_JPRB, 0.99009E+00_JPRB, 0.99181E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.10041E+01_JPRB, 0.10074E+01_JPRB, 0.99287E+00_JPRB, 0.98170E+00_JPRB, 0.95945E+00_JPRB, 0.87692E+00_JPRB, 0.84832E+00_JPRB ,&
&  0.91913E+00_JPRB, 0.86180E+00_JPRB, 0.91362E+00_JPRB, 0.80024E+00_JPRB, 0.76946E+00_JPRB, 0.72897E+00_JPRB, 0.70108E+00_JPRB ,&
&  0.66640E+00_JPRB, 0.66576E+00_JPRB, 0.69585E+00_JPRB, 0.72522E+00_JPRB, 0.72984E+00_JPRB, 0.71704E+00_JPRB, 0.72341E+00_JPRB ,&
&  0.73979E+00_JPRB, 0.77494E+00_JPRB, 0.82524E+00_JPRB, 0.85236E+00_JPRB, 0.86880E+00_JPRB, 0.88813E+00_JPRB, 0.89327E+00_JPRB ,&
&  0.84412E+00_JPRB, 0.82726E+00_JPRB, 0.81503E+00_JPRB, 0.82215E+00_JPRB, 0.85878E+00_JPRB, 0.91968E+00_JPRB, 0.95588E+00_JPRB ,&
&  0.96340E+00_JPRB, 0.84884E+00_JPRB, 0.96782E+00_JPRB, 0.10082E+01_JPRB, 0.10786E+01_JPRB, 0.11213E+01_JPRB, 0.11523E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.12341E+01_JPRB, 0.12814E+01_JPRB, 0.13275E+01_JPRB, 0.13802E+01_JPRB, 0.14587E+01_JPRB, 0.15096E+01_JPRB ,&
&  0.16823E+01_JPRB, 0.13015E+01_JPRB, 0.14500E+01_JPRB, 0.16413E+01_JPRB, 0.22195E+01_JPRB, 0.25059E+01_JPRB, 0.25792E+01_JPRB ,&
&  0.30138E+01_JPRB, 0.29406E+01_JPRB, 0.32098E+01_JPRB, 0.34478E+01_JPRB, 0.37714E+01_JPRB, 0.41912E+01_JPRB, 0.44417E+01_JPRB ,&
&  0.44873E+01_JPRB, 0.45651E+01_JPRB, 0.47633E+01_JPRB, 0.43385E+01_JPRB, 0.37954E+01_JPRB, 0.32553E+01_JPRB, 0.26109E+01_JPRB ,&
&  0.21822E+01_JPRB, 0.18023E+01_JPRB, 0.14793E+01_JPRB, 0.12515E+01_JPRB, 0.10859E+01_JPRB, 0.98322E+00_JPRB, 0.92970E+00_JPRB ,&
&  0.90618E+00_JPRB, 0.88020E+00_JPRB, 0.86526E+00_JPRB, 0.83351E+00_JPRB, 0.80343E+00_JPRB, 0.80629E+00_JPRB, 0.84094E+00_JPRB ,&
&  0.84606E+00_JPRB, 0.85677E+00_JPRB, 0.90987E+00_JPRB, 0.10277E+01_JPRB, 0.10550E+01_JPRB, 0.10866E+01_JPRB, 0.12033E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10727E+01_JPRB, 0.11164E+01_JPRB, 0.10711E+01_JPRB, 0.10171E+01_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.10759E+01_JPRB, 0.10646E+01_JPRB, 0.10318E+01_JPRB, 0.10145E+01_JPRB, 0.10100E+01_JPRB, 0.96385E+00_JPRB, 0.92079E+00_JPRB ,&
&  0.98490E+00_JPRB, 0.97474E+00_JPRB, 0.97506E+00_JPRB, 0.95593E+00_JPRB, 0.93858E+00_JPRB, 0.86412E+00_JPRB, 0.81229E+00_JPRB ,&
&  0.76944E+00_JPRB, 0.76850E+00_JPRB, 0.81626E+00_JPRB, 0.83406E+00_JPRB, 0.83167E+00_JPRB, 0.80516E+00_JPRB, 0.79191E+00_JPRB ,&
&  0.77982E+00_JPRB, 0.78960E+00_JPRB, 0.83612E+00_JPRB, 0.88037E+00_JPRB, 0.88239E+00_JPRB, 0.88483E+00_JPRB, 0.91087E+00_JPRB ,&
&  0.86968E+00_JPRB, 0.86546E+00_JPRB, 0.86317E+00_JPRB, 0.80984E+00_JPRB, 0.91111E+00_JPRB, 0.95591E+00_JPRB, 0.98323E+00_JPRB ,&
&  0.10284E+01_JPRB, 0.10684E+01_JPRB, 0.10940E+01_JPRB, 0.11097E+01_JPRB, 0.11422E+01_JPRB, 0.11653E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.12735E+01_JPRB, 0.13243E+01_JPRB, 0.13909E+01_JPRB, 0.15109E+01_JPRB, 0.15685E+01_JPRB, 0.18161E+01_JPRB ,&
&  0.19740E+01_JPRB, 0.20483E+01_JPRB, 0.22664E+01_JPRB, 0.21599E+01_JPRB, 0.27684E+01_JPRB, 0.31249E+01_JPRB, 0.29783E+01_JPRB ,&
&  0.31490E+01_JPRB, 0.31475E+01_JPRB, 0.37889E+01_JPRB, 0.43415E+01_JPRB, 0.48522E+01_JPRB, 0.53094E+01_JPRB, 0.54624E+01_JPRB ,&
&  0.53421E+01_JPRB, 0.54966E+01_JPRB, 0.53321E+01_JPRB, 0.47294E+01_JPRB, 0.39889E+01_JPRB, 0.34644E+01_JPRB, 0.27749E+01_JPRB ,&
&  0.22050E+01_JPRB, 0.17486E+01_JPRB, 0.14691E+01_JPRB, 0.13288E+01_JPRB, 0.10880E+01_JPRB, 0.10101E+01_JPRB, 0.93123E+00_JPRB ,&
&  0.85754E+00_JPRB, 0.81233E+00_JPRB, 0.81758E+00_JPRB, 0.82570E+00_JPRB, 0.80420E+00_JPRB, 0.81077E+00_JPRB, 0.81861E+00_JPRB ,&
&  0.89029E+00_JPRB, 0.10485E+01_JPRB, 0.13451E+01_JPRB, 0.14135E+01_JPRB, 0.13942E+01_JPRB, 0.13827E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.12834E+01_JPRB, 0.12309E+01_JPRB, 0.11452E+01_JPRB, 0.10810E+01_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.11359E+01_JPRB, 0.11136E+01_JPRB, 0.10906E+01_JPRB, 0.10712E+01_JPRB, 0.10592E+01_JPRB, 0.10337E+01_JPRB, 0.10102E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.10055E+01_JPRB, 0.99973E+00_JPRB, 0.98915E+00_JPRB, 0.96822E+00_JPRB, 0.98079E+00_JPRB, 0.90771E+00_JPRB ,&
&  0.85398E+00_JPRB, 0.88863E+00_JPRB, 0.88356E+00_JPRB, 0.94673E+00_JPRB, 0.10214E+01_JPRB, 0.98839E+00_JPRB, 0.96651E+00_JPRB ,&
&  0.93724E+00_JPRB, 0.88773E+00_JPRB, 0.87861E+00_JPRB, 0.89277E+00_JPRB, 0.90059E+00_JPRB, 0.91559E+00_JPRB, 0.94341E+00_JPRB ,&
&  0.87827E+00_JPRB, 0.81891E+00_JPRB, 0.78638E+00_JPRB, 0.79260E+00_JPRB, 0.86612E+00_JPRB, 0.98067E+00_JPRB, 0.10497E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.10900E+01_JPRB, 0.11426E+01_JPRB, 0.11787E+01_JPRB, 0.12077E+01_JPRB, 0.12502E+01_JPRB, 0.12965E+01_JPRB ,&
&  0.13356E+01_JPRB, 0.13801E+01_JPRB, 0.14330E+01_JPRB, 0.15208E+01_JPRB, 0.15446E+01_JPRB, 0.17630E+01_JPRB, 0.22485E+01_JPRB ,&
&  0.24951E+01_JPRB, 0.25791E+01_JPRB, 0.27316E+01_JPRB, 0.30185E+01_JPRB, 0.34238E+01_JPRB, 0.34168E+01_JPRB, 0.34450E+01_JPRB ,&
&  0.37584E+01_JPRB, 0.39256E+01_JPRB, 0.48927E+01_JPRB, 0.53891E+01_JPRB, 0.56846E+01_JPRB, 0.60984E+01_JPRB, 0.60192E+01_JPRB ,&
&  0.59024E+01_JPRB, 0.56700E+01_JPRB, 0.55023E+01_JPRB, 0.49315E+01_JPRB, 0.41316E+01_JPRB, 0.35937E+01_JPRB, 0.27905E+01_JPRB ,&
&  0.22540E+01_JPRB, 0.17802E+01_JPRB, 0.13297E+01_JPRB, 0.11469E+01_JPRB, 0.97519E+00_JPRB, 0.94688E+00_JPRB, 0.89230E+00_JPRB ,&
&  0.82672E+00_JPRB, 0.82959E+00_JPRB, 0.86561E+00_JPRB, 0.95806E+00_JPRB, 0.95762E+00_JPRB, 0.87035E+00_JPRB, 0.92710E+00_JPRB ,&
&  0.10449E+01_JPRB, 0.14076E+01_JPRB, 0.15624E+01_JPRB, 0.17079E+01_JPRB, 0.16275E+01_JPRB, 0.16028E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.14622E+01_JPRB, 0.13517E+01_JPRB, 0.12494E+01_JPRB, 0.11802E+01_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.11891E+01_JPRB, 0.11541E+01_JPRB, 0.11418E+01_JPRB, 0.11467E+01_JPRB, 0.11487E+01_JPRB, 0.11108E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.10570E+01_JPRB, 0.10408E+01_JPRB, 0.10313E+01_JPRB, 0.10120E+01_JPRB, 0.10254E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.99873E+00_JPRB, 0.10901E+01_JPRB, 0.96810E+00_JPRB, 0.10025E+01_JPRB, 0.13066E+01_JPRB, 0.11962E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.10616E+01_JPRB, 0.97569E+00_JPRB, 0.94670E+00_JPRB, 0.95260E+00_JPRB, 0.99381E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.92635E+00_JPRB, 0.83834E+00_JPRB, 0.80996E+00_JPRB, 0.81456E+00_JPRB, 0.87145E+00_JPRB, 0.96408E+00_JPRB, 0.10908E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.11636E+01_JPRB, 0.12177E+01_JPRB, 0.12841E+01_JPRB, 0.13525E+01_JPRB, 0.14321E+01_JPRB, 0.14812E+01_JPRB ,&
&  0.15121E+01_JPRB, 0.15393E+01_JPRB, 0.15813E+01_JPRB, 0.16929E+01_JPRB, 0.16792E+01_JPRB, 0.20032E+01_JPRB, 0.30287E+01_JPRB ,&
&  0.36416E+01_JPRB, 0.33552E+01_JPRB, 0.37153E+01_JPRB, 0.47884E+01_JPRB, 0.51177E+01_JPRB, 0.50144E+01_JPRB, 0.48751E+01_JPRB ,&
&  0.50683E+01_JPRB, 0.53328E+01_JPRB, 0.56704E+01_JPRB, 0.61565E+01_JPRB, 0.64492E+01_JPRB, 0.66389E+01_JPRB, 0.63612E+01_JPRB ,&
&  0.59506E+01_JPRB, 0.56950E+01_JPRB, 0.53719E+01_JPRB, 0.48534E+01_JPRB, 0.39553E+01_JPRB, 0.33685E+01_JPRB, 0.26263E+01_JPRB ,&
&  0.21685E+01_JPRB, 0.16245E+01_JPRB, 0.10892E+01_JPRB, 0.89999E+00_JPRB, 0.82382E+00_JPRB, 0.91438E+00_JPRB, 0.12442E+01_JPRB ,&
&  0.92579E+00_JPRB, 0.10214E+01_JPRB, 0.11382E+01_JPRB, 0.12988E+01_JPRB, 0.13984E+01_JPRB, 0.13882E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.11734E+01_JPRB, 0.14444E+01_JPRB, 0.19051E+01_JPRB, 0.20605E+01_JPRB, 0.18749E+01_JPRB, 0.18417E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.17918E+01_JPRB, 0.16349E+01_JPRB, 0.14775E+01_JPRB, 0.13341E+01_JPRB, 0.12644E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.12082E+01_JPRB, 0.12009E+01_JPRB, 0.12156E+01_JPRB, 0.12461E+01_JPRB, 0.12671E+01_JPRB, 0.12086E+01_JPRB, 0.11885E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11230E+01_JPRB, 0.10923E+01_JPRB, 0.10804E+01_JPRB, 0.10655E+01_JPRB, 0.10605E+01_JPRB, 0.10859E+01_JPRB ,&
&  0.11796E+01_JPRB, 0.13863E+01_JPRB, 0.12377E+01_JPRB, 0.10123E+01_JPRB, 0.11282E+01_JPRB, 0.11127E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.11978E+01_JPRB, 0.11514E+01_JPRB, 0.11023E+01_JPRB, 0.11533E+01_JPRB, 0.12343E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.10842E+01_JPRB, 0.10166E+01_JPRB, 0.10268E+01_JPRB, 0.11280E+01_JPRB, 0.10665E+01_JPRB, 0.12014E+01_JPRB ,&
&  0.12747E+01_JPRB, 0.13408E+01_JPRB, 0.14103E+01_JPRB, 0.14983E+01_JPRB, 0.15744E+01_JPRB, 0.16499E+01_JPRB, 0.16901E+01_JPRB ,&
&  0.17134E+01_JPRB, 0.17076E+01_JPRB, 0.16671E+01_JPRB, 0.17439E+01_JPRB, 0.18798E+01_JPRB, 0.21698E+01_JPRB, 0.29366E+01_JPRB ,&
&  0.35495E+01_JPRB, 0.36982E+01_JPRB, 0.45380E+01_JPRB, 0.57437E+01_JPRB, 0.59609E+01_JPRB, 0.58391E+01_JPRB, 0.60908E+01_JPRB ,&
&  0.64872E+01_JPRB, 0.68857E+01_JPRB, 0.73072E+01_JPRB, 0.74307E+01_JPRB, 0.73632E+01_JPRB, 0.68178E+01_JPRB, 0.62311E+01_JPRB ,&
&  0.56580E+01_JPRB, 0.54549E+01_JPRB, 0.52605E+01_JPRB, 0.45050E+01_JPRB, 0.36417E+01_JPRB, 0.29849E+01_JPRB, 0.26453E+01_JPRB ,&
&  0.25390E+01_JPRB, 0.12467E+01_JPRB, 0.84371E+00_JPRB, 0.69942E+00_JPRB, 0.67847E+00_JPRB, 0.74677E+00_JPRB, 0.97440E+00_JPRB ,&
&  0.88997E+00_JPRB, 0.11593E+01_JPRB, 0.16563E+01_JPRB, 0.18834E+01_JPRB, 0.22377E+01_JPRB, 0.26856E+01_JPRB, 0.20708E+01_JPRB ,&
&  0.16859E+01_JPRB, 0.17997E+01_JPRB, 0.23732E+01_JPRB, 0.24676E+01_JPRB, 0.21897E+01_JPRB, 0.21203E+01_JPRB, 0.21289E+01_JPRB ,&
&  0.19656E+01_JPRB, 0.18557E+01_JPRB, 0.16183E+01_JPRB, 0.13733E+01_JPRB, 0.12633E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.13253E+01_JPRB, 0.13156E+01_JPRB, 0.13499E+01_JPRB, 0.13839E+01_JPRB, 0.13742E+01_JPRB, 0.13147E+01_JPRB, 0.12867E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.11865E+01_JPRB, 0.11441E+01_JPRB, 0.11238E+01_JPRB, 0.11076E+01_JPRB, 0.11029E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.13193E+01_JPRB, 0.11172E+01_JPRB, 0.98024E+00_JPRB, 0.85490E+00_JPRB, 0.92483E+00_JPRB, 0.10296E+01_JPRB ,&
&  0.11964E+01_JPRB, 0.12645E+01_JPRB, 0.13812E+01_JPRB, 0.15382E+01_JPRB, 0.17002E+01_JPRB, 0.16644E+01_JPRB, 0.17820E+01_JPRB ,&
&  0.17138E+01_JPRB, 0.16058E+01_JPRB, 0.13758E+01_JPRB, 0.14974E+01_JPRB, 0.14762E+01_JPRB, 0.13492E+01_JPRB, 0.15686E+01_JPRB ,&
&  0.16346E+01_JPRB, 0.16869E+01_JPRB, 0.17303E+01_JPRB, 0.17670E+01_JPRB, 0.17888E+01_JPRB, 0.17554E+01_JPRB, 0.17004E+01_JPRB ,&
&  0.16283E+01_JPRB, 0.16127E+01_JPRB, 0.16562E+01_JPRB, 0.17391E+01_JPRB, 0.20592E+01_JPRB, 0.24135E+01_JPRB, 0.31014E+01_JPRB ,&
&  0.38485E+01_JPRB, 0.38499E+01_JPRB, 0.41085E+01_JPRB, 0.56843E+01_JPRB, 0.73165E+01_JPRB, 0.72965E+01_JPRB, 0.73957E+01_JPRB ,&
&  0.78494E+01_JPRB, 0.79485E+01_JPRB, 0.77833E+01_JPRB, 0.73455E+01_JPRB, 0.69360E+01_JPRB, 0.64605E+01_JPRB, 0.59077E+01_JPRB ,&
&  0.53693E+01_JPRB, 0.51870E+01_JPRB, 0.50096E+01_JPRB, 0.44616E+01_JPRB, 0.36327E+01_JPRB, 0.32073E+01_JPRB, 0.28558E+01_JPRB ,&
&  0.25248E+01_JPRB, 0.15965E+01_JPRB, 0.98801E+00_JPRB, 0.80013E+00_JPRB, 0.75815E+00_JPRB, 0.79905E+00_JPRB, 0.10226E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.15114E+01_JPRB, 0.22181E+01_JPRB, 0.25497E+01_JPRB, 0.38333E+01_JPRB, 0.45068E+01_JPRB, 0.40750E+01_JPRB ,&
&  0.31694E+01_JPRB, 0.25701E+01_JPRB, 0.29712E+01_JPRB, 0.29063E+01_JPRB, 0.26324E+01_JPRB, 0.23863E+01_JPRB, 0.23211E+01_JPRB ,&
&  0.21558E+01_JPRB, 0.20326E+01_JPRB, 0.18560E+01_JPRB, 0.15653E+01_JPRB, 0.13904E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.15703E+01_JPRB, 0.15063E+01_JPRB, 0.14791E+01_JPRB, 0.14649E+01_JPRB, 0.13839E+01_JPRB, 0.13247E+01_JPRB, 0.13226E+01_JPRB ,&
&  0.13060E+01_JPRB, 0.12448E+01_JPRB, 0.12002E+01_JPRB, 0.11852E+01_JPRB, 0.11742E+01_JPRB, 0.11775E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.10846E+01_JPRB, 0.11581E+01_JPRB, 0.96684E+00_JPRB, 0.82724E+00_JPRB, 0.78122E+00_JPRB, 0.89630E+00_JPRB, 0.10802E+01_JPRB ,&
&  0.13342E+01_JPRB, 0.15899E+01_JPRB, 0.19661E+01_JPRB, 0.22434E+01_JPRB, 0.24616E+01_JPRB, 0.25623E+01_JPRB, 0.26977E+01_JPRB ,&
&  0.29147E+01_JPRB, 0.23110E+01_JPRB, 0.22152E+01_JPRB, 0.21393E+01_JPRB, 0.20952E+01_JPRB, 0.20273E+01_JPRB, 0.19995E+01_JPRB ,&
&  0.19296E+01_JPRB, 0.18828E+01_JPRB, 0.18409E+01_JPRB, 0.17921E+01_JPRB, 0.17432E+01_JPRB, 0.16964E+01_JPRB, 0.16578E+01_JPRB ,&
&  0.16717E+01_JPRB, 0.16347E+01_JPRB, 0.16586E+01_JPRB, 0.20743E+01_JPRB, 0.29614E+01_JPRB, 0.35835E+01_JPRB, 0.39417E+01_JPRB ,&
&  0.38177E+01_JPRB, 0.41046E+01_JPRB, 0.65553E+01_JPRB, 0.72940E+01_JPRB, 0.78575E+01_JPRB, 0.89110E+01_JPRB, 0.92481E+01_JPRB ,&
&  0.88332E+01_JPRB, 0.79053E+01_JPRB, 0.70189E+01_JPRB, 0.61289E+01_JPRB, 0.66124E+01_JPRB, 0.61765E+01_JPRB, 0.54821E+01_JPRB ,&
&  0.49450E+01_JPRB, 0.49092E+01_JPRB, 0.44784E+01_JPRB, 0.40126E+01_JPRB, 0.36737E+01_JPRB, 0.36779E+01_JPRB, 0.29429E+01_JPRB ,&
&  0.23482E+01_JPRB, 0.22523E+01_JPRB, 0.16456E+01_JPRB, 0.13319E+01_JPRB, 0.12323E+01_JPRB, 0.12563E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.16797E+01_JPRB, 0.21889E+01_JPRB, 0.29385E+01_JPRB, 0.36914E+01_JPRB, 0.51643E+01_JPRB, 0.59793E+01_JPRB, 0.56945E+01_JPRB ,&
&  0.44419E+01_JPRB, 0.40805E+01_JPRB, 0.36234E+01_JPRB, 0.33523E+01_JPRB, 0.30499E+01_JPRB, 0.27035E+01_JPRB, 0.25131E+01_JPRB ,&
&  0.22939E+01_JPRB, 0.21403E+01_JPRB, 0.20187E+01_JPRB, 0.18181E+01_JPRB, 0.16867E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.18807E+01_JPRB, 0.17721E+01_JPRB, 0.16185E+01_JPRB, 0.15071E+01_JPRB, 0.13949E+01_JPRB, 0.13225E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.12379E+01_JPRB, 0.12386E+01_JPRB, 0.12502E+01_JPRB, 0.12433E+01_JPRB, 0.12254E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.10076E+01_JPRB, 0.87094E+00_JPRB, 0.86932E+00_JPRB, 0.97841E+00_JPRB, 0.11043E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.20629E+01_JPRB, 0.25864E+01_JPRB, 0.34029E+01_JPRB, 0.38971E+01_JPRB, 0.40844E+01_JPRB, 0.40336E+01_JPRB ,&
&  0.37027E+01_JPRB, 0.36625E+01_JPRB, 0.34488E+01_JPRB, 0.31001E+01_JPRB, 0.27555E+01_JPRB, 0.24879E+01_JPRB, 0.22602E+01_JPRB ,&
&  0.20649E+01_JPRB, 0.19085E+01_JPRB, 0.18070E+01_JPRB, 0.17321E+01_JPRB, 0.17282E+01_JPRB, 0.17252E+01_JPRB, 0.17484E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.16900E+01_JPRB, 0.20125E+01_JPRB, 0.30495E+01_JPRB, 0.38752E+01_JPRB, 0.44514E+01_JPRB, 0.41924E+01_JPRB ,&
&  0.47595E+01_JPRB, 0.54148E+01_JPRB, 0.58030E+01_JPRB, 0.68260E+01_JPRB, 0.60723E+01_JPRB, 0.71894E+01_JPRB, 0.85665E+01_JPRB ,&
&  0.83626E+01_JPRB, 0.74845E+01_JPRB, 0.69270E+01_JPRB, 0.62751E+01_JPRB, 0.55821E+01_JPRB, 0.65185E+01_JPRB, 0.55681E+01_JPRB ,&
&  0.48575E+01_JPRB, 0.46217E+01_JPRB, 0.41917E+01_JPRB, 0.41549E+01_JPRB, 0.34864E+01_JPRB, 0.30857E+01_JPRB, 0.22041E+01_JPRB ,&
&  0.22682E+01_JPRB, 0.20382E+01_JPRB, 0.20647E+01_JPRB, 0.18722E+01_JPRB, 0.19351E+01_JPRB, 0.18898E+01_JPRB, 0.20050E+01_JPRB ,&
&  0.25161E+01_JPRB, 0.31383E+01_JPRB, 0.46502E+01_JPRB, 0.56889E+01_JPRB, 0.77646E+01_JPRB, 0.69640E+01_JPRB, 0.67520E+01_JPRB ,&
&  0.57638E+01_JPRB, 0.51791E+01_JPRB, 0.42277E+01_JPRB, 0.40130E+01_JPRB, 0.35294E+01_JPRB, 0.31186E+01_JPRB, 0.27618E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.22160E+01_JPRB, 0.20614E+01_JPRB, 0.19442E+01_JPRB, 0.19261E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.18077E+01_JPRB, 0.17045E+01_JPRB, 0.15936E+01_JPRB, 0.14779E+01_JPRB, 0.13455E+01_JPRB, 0.12722E+01_JPRB, 0.12279E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.12079E+01_JPRB, 0.12224E+01_JPRB, 0.12306E+01_JPRB, 0.12336E+01_JPRB, 0.12295E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.99611E+00_JPRB, 0.87287E+00_JPRB, 0.85885E+00_JPRB, 0.10964E+01_JPRB, 0.13756E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.19433E+01_JPRB, 0.25899E+01_JPRB, 0.32196E+01_JPRB, 0.43855E+01_JPRB, 0.55381E+01_JPRB, 0.62151E+01_JPRB, 0.57502E+01_JPRB ,&
&  0.55692E+01_JPRB, 0.49941E+01_JPRB, 0.41802E+01_JPRB, 0.35115E+01_JPRB, 0.29105E+01_JPRB, 0.24385E+01_JPRB, 0.20849E+01_JPRB ,&
&  0.18630E+01_JPRB, 0.17201E+01_JPRB, 0.16151E+01_JPRB, 0.15743E+01_JPRB, 0.15996E+01_JPRB, 0.15668E+01_JPRB, 0.16387E+01_JPRB ,&
&  0.17330E+01_JPRB, 0.18836E+01_JPRB, 0.25029E+01_JPRB, 0.36100E+01_JPRB, 0.46299E+01_JPRB, 0.49764E+01_JPRB, 0.48792E+01_JPRB ,&
&  0.48350E+01_JPRB, 0.50111E+01_JPRB, 0.54678E+01_JPRB, 0.57991E+01_JPRB, 0.63917E+01_JPRB, 0.71325E+01_JPRB, 0.82418E+01_JPRB ,&
&  0.65667E+01_JPRB, 0.53996E+01_JPRB, 0.45018E+01_JPRB, 0.44183E+01_JPRB, 0.52221E+01_JPRB, 0.64360E+01_JPRB, 0.61358E+01_JPRB ,&
&  0.53942E+01_JPRB, 0.49241E+01_JPRB, 0.44369E+01_JPRB, 0.39390E+01_JPRB, 0.40359E+01_JPRB, 0.26315E+01_JPRB, 0.28934E+01_JPRB ,&
&  0.29782E+01_JPRB, 0.29592E+01_JPRB, 0.26873E+01_JPRB, 0.23037E+01_JPRB, 0.22678E+01_JPRB, 0.27152E+01_JPRB, 0.32655E+01_JPRB ,&
&  0.41727E+01_JPRB, 0.50508E+01_JPRB, 0.84302E+01_JPRB, 0.90318E+01_JPRB, 0.90361E+01_JPRB, 0.75050E+01_JPRB, 0.68866E+01_JPRB ,&
&  0.55843E+01_JPRB, 0.51547E+01_JPRB, 0.49125E+01_JPRB, 0.44398E+01_JPRB, 0.38874E+01_JPRB, 0.34647E+01_JPRB, 0.30032E+01_JPRB ,&
&  0.26290E+01_JPRB, 0.23183E+01_JPRB, 0.20796E+01_JPRB, 0.19241E+01_JPRB, 0.18704E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.17387E+01_JPRB, 0.16548E+01_JPRB, 0.15344E+01_JPRB, 0.13604E+01_JPRB, 0.12581E+01_JPRB, 0.12201E+01_JPRB, 0.12142E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.12258E+01_JPRB, 0.12241E+01_JPRB, 0.12215E+01_JPRB, 0.12360E+01_JPRB, 0.12198E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.10792E+01_JPRB, 0.94514E+00_JPRB, 0.98483E+00_JPRB, 0.12366E+01_JPRB, 0.17038E+01_JPRB, 0.16772E+01_JPRB ,&
&  0.24171E+01_JPRB, 0.29912E+01_JPRB, 0.33645E+01_JPRB, 0.44158E+01_JPRB, 0.70670E+01_JPRB, 0.70275E+01_JPRB, 0.65816E+01_JPRB ,&
&  0.60344E+01_JPRB, 0.49417E+01_JPRB, 0.39295E+01_JPRB, 0.31500E+01_JPRB, 0.25722E+01_JPRB, 0.21962E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.17578E+01_JPRB, 0.16315E+01_JPRB, 0.15633E+01_JPRB, 0.15745E+01_JPRB, 0.15446E+01_JPRB, 0.15295E+01_JPRB, 0.16577E+01_JPRB ,&
&  0.18205E+01_JPRB, 0.20401E+01_JPRB, 0.26174E+01_JPRB, 0.34858E+01_JPRB, 0.44580E+01_JPRB, 0.49591E+01_JPRB, 0.59492E+01_JPRB ,&
&  0.54043E+01_JPRB, 0.55671E+01_JPRB, 0.55026E+01_JPRB, 0.66583E+01_JPRB, 0.66279E+01_JPRB, 0.73306E+01_JPRB, 0.79033E+01_JPRB ,&
&  0.57764E+01_JPRB, 0.47898E+01_JPRB, 0.48048E+01_JPRB, 0.54437E+01_JPRB, 0.46293E+01_JPRB, 0.49467E+01_JPRB, 0.63136E+01_JPRB ,&
&  0.50281E+01_JPRB, 0.47442E+01_JPRB, 0.46526E+01_JPRB, 0.48415E+01_JPRB, 0.29899E+01_JPRB, 0.21458E+01_JPRB, 0.33007E+01_JPRB ,&
&  0.31694E+01_JPRB, 0.18519E+01_JPRB, 0.13721E+01_JPRB, 0.17422E+01_JPRB, 0.14470E+01_JPRB, 0.20567E+01_JPRB, 0.39711E+01_JPRB ,&
&  0.52525E+01_JPRB, 0.65280E+01_JPRB, 0.10419E+02_JPRB, 0.86151E+01_JPRB, 0.78538E+01_JPRB, 0.71168E+01_JPRB, 0.61341E+01_JPRB ,&
&  0.54976E+01_JPRB, 0.44384E+01_JPRB, 0.44639E+01_JPRB, 0.40099E+01_JPRB, 0.36928E+01_JPRB, 0.34523E+01_JPRB, 0.30766E+01_JPRB ,&
&  0.26839E+01_JPRB, 0.23266E+01_JPRB, 0.20441E+01_JPRB, 0.18751E+01_JPRB, 0.18024E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.15862E+01_JPRB, 0.14874E+01_JPRB, 0.14289E+01_JPRB, 0.13314E+01_JPRB, 0.12532E+01_JPRB, 0.12225E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.12087E+01_JPRB, 0.11865E+01_JPRB, 0.11914E+01_JPRB, 0.12051E+01_JPRB, 0.11946E+01_JPRB, 0.11373E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.10674E+01_JPRB, 0.11903E+01_JPRB, 0.12900E+01_JPRB, 0.16187E+01_JPRB, 0.19630E+01_JPRB, 0.22971E+01_JPRB ,&
&  0.27639E+01_JPRB, 0.30930E+01_JPRB, 0.34958E+01_JPRB, 0.43224E+01_JPRB, 0.52723E+01_JPRB, 0.56358E+01_JPRB, 0.58280E+01_JPRB ,&
&  0.48560E+01_JPRB, 0.39248E+01_JPRB, 0.33636E+01_JPRB, 0.29511E+01_JPRB, 0.24886E+01_JPRB, 0.21245E+01_JPRB, 0.18627E+01_JPRB ,&
&  0.17187E+01_JPRB, 0.16198E+01_JPRB, 0.16215E+01_JPRB, 0.15386E+01_JPRB, 0.15226E+01_JPRB, 0.15529E+01_JPRB, 0.17034E+01_JPRB ,&
&  0.18988E+01_JPRB, 0.21519E+01_JPRB, 0.25911E+01_JPRB, 0.33878E+01_JPRB, 0.48453E+01_JPRB, 0.56326E+01_JPRB, 0.54907E+01_JPRB ,&
&  0.51646E+01_JPRB, 0.51379E+01_JPRB, 0.59523E+01_JPRB, 0.66173E+01_JPRB, 0.63015E+01_JPRB, 0.71560E+01_JPRB, 0.72763E+01_JPRB ,&
&  0.63953E+01_JPRB, 0.56912E+01_JPRB, 0.49146E+01_JPRB, 0.55055E+01_JPRB, 0.51221E+01_JPRB, 0.45366E+01_JPRB, 0.50859E+01_JPRB ,&
&  0.50977E+01_JPRB, 0.44709E+01_JPRB, 0.37187E+01_JPRB, 0.31968E+01_JPRB, 0.32004E+01_JPRB, 0.27135E+01_JPRB, 0.19807E+01_JPRB ,&
&  0.16783E+01_JPRB, 0.13941E+01_JPRB, 0.11835E+01_JPRB, 0.12339E+01_JPRB, 0.11426E+01_JPRB, 0.19200E+01_JPRB, 0.35656E+01_JPRB ,&
&  0.61327E+01_JPRB, 0.74263E+01_JPRB, 0.90359E+01_JPRB, 0.79095E+01_JPRB, 0.71572E+01_JPRB, 0.71769E+01_JPRB, 0.69146E+01_JPRB ,&
&  0.47149E+01_JPRB, 0.41256E+01_JPRB, 0.41112E+01_JPRB, 0.38253E+01_JPRB, 0.35564E+01_JPRB, 0.32806E+01_JPRB, 0.28963E+01_JPRB ,&
&  0.24774E+01_JPRB, 0.21064E+01_JPRB, 0.19048E+01_JPRB, 0.18240E+01_JPRB, 0.17295E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.13027E+01_JPRB, 0.12722E+01_JPRB, 0.12833E+01_JPRB, 0.12571E+01_JPRB, 0.12186E+01_JPRB, 0.12168E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.11810E+01_JPRB, 0.11421E+01_JPRB, 0.11436E+01_JPRB, 0.11682E+01_JPRB, 0.11683E+01_JPRB, 0.11613E+01_JPRB, 0.11181E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10722E+01_JPRB, 0.12394E+01_JPRB, 0.14085E+01_JPRB, 0.19900E+01_JPRB, 0.23046E+01_JPRB, 0.27719E+01_JPRB ,&
&  0.27765E+01_JPRB, 0.29310E+01_JPRB, 0.33813E+01_JPRB, 0.35138E+01_JPRB, 0.43165E+01_JPRB, 0.41121E+01_JPRB, 0.35346E+01_JPRB ,&
&  0.27275E+01_JPRB, 0.26377E+01_JPRB, 0.25468E+01_JPRB, 0.23619E+01_JPRB, 0.21348E+01_JPRB, 0.19356E+01_JPRB, 0.18050E+01_JPRB ,&
&  0.17441E+01_JPRB, 0.17082E+01_JPRB, 0.15911E+01_JPRB, 0.15368E+01_JPRB, 0.15684E+01_JPRB, 0.16216E+01_JPRB, 0.17677E+01_JPRB ,&
&  0.19866E+01_JPRB, 0.22577E+01_JPRB, 0.26613E+01_JPRB, 0.38315E+01_JPRB, 0.44642E+01_JPRB, 0.46572E+01_JPRB, 0.52789E+01_JPRB ,&
&  0.56726E+01_JPRB, 0.60321E+01_JPRB, 0.56409E+01_JPRB, 0.66724E+01_JPRB, 0.61385E+01_JPRB, 0.64513E+01_JPRB, 0.66288E+01_JPRB ,&
&  0.59481E+01_JPRB, 0.56882E+01_JPRB, 0.47180E+01_JPRB, 0.47187E+01_JPRB, 0.52489E+01_JPRB, 0.49929E+01_JPRB, 0.47167E+01_JPRB ,&
&  0.49184E+01_JPRB, 0.43500E+01_JPRB, 0.38703E+01_JPRB, 0.36379E+01_JPRB, 0.47126E+01_JPRB, 0.49614E+01_JPRB, 0.23672E+01_JPRB ,&
&  0.16382E+01_JPRB, 0.15079E+01_JPRB, 0.11772E+01_JPRB, 0.10326E+01_JPRB, 0.11844E+01_JPRB, 0.19518E+01_JPRB, 0.44908E+01_JPRB ,&
&  0.52891E+01_JPRB, 0.87180E+01_JPRB, 0.91963E+01_JPRB, 0.92555E+01_JPRB, 0.81912E+01_JPRB, 0.70583E+01_JPRB, 0.54801E+01_JPRB ,&
&  0.53056E+01_JPRB, 0.46967E+01_JPRB, 0.40166E+01_JPRB, 0.36253E+01_JPRB, 0.32241E+01_JPRB, 0.28244E+01_JPRB, 0.25010E+01_JPRB ,&
&  0.22505E+01_JPRB, 0.19708E+01_JPRB, 0.17957E+01_JPRB, 0.16623E+01_JPRB, 0.14569E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.10616E+01_JPRB, 0.10880E+01_JPRB, 0.11457E+01_JPRB, 0.11574E+01_JPRB, 0.11402E+01_JPRB, 0.11550E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.11125E+01_JPRB, 0.11113E+01_JPRB, 0.11280E+01_JPRB, 0.11523E+01_JPRB, 0.11799E+01_JPRB, 0.11934E+01_JPRB, 0.11573E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.11243E+01_JPRB, 0.12258E+01_JPRB, 0.16631E+01_JPRB, 0.22068E+01_JPRB, 0.27541E+01_JPRB, 0.29537E+01_JPRB ,&
&  0.26556E+01_JPRB, 0.27000E+01_JPRB, 0.31213E+01_JPRB, 0.32644E+01_JPRB, 0.30363E+01_JPRB, 0.30609E+01_JPRB, 0.22352E+01_JPRB ,&
&  0.18987E+01_JPRB, 0.19088E+01_JPRB, 0.18842E+01_JPRB, 0.18863E+01_JPRB, 0.18503E+01_JPRB, 0.17770E+01_JPRB, 0.17649E+01_JPRB ,&
&  0.17583E+01_JPRB, 0.16609E+01_JPRB, 0.16211E+01_JPRB, 0.16261E+01_JPRB, 0.16761E+01_JPRB, 0.17300E+01_JPRB, 0.18690E+01_JPRB ,&
&  0.21180E+01_JPRB, 0.24592E+01_JPRB, 0.30025E+01_JPRB, 0.42105E+01_JPRB, 0.39331E+01_JPRB, 0.51068E+01_JPRB, 0.53927E+01_JPRB ,&
&  0.55496E+01_JPRB, 0.56705E+01_JPRB, 0.53481E+01_JPRB, 0.58168E+01_JPRB, 0.58263E+01_JPRB, 0.60863E+01_JPRB, 0.58964E+01_JPRB ,&
&  0.55987E+01_JPRB, 0.48596E+01_JPRB, 0.48728E+01_JPRB, 0.47415E+01_JPRB, 0.48787E+01_JPRB, 0.61922E+01_JPRB, 0.47166E+01_JPRB ,&
&  0.41743E+01_JPRB, 0.46140E+01_JPRB, 0.42784E+01_JPRB, 0.41321E+01_JPRB, 0.49688E+01_JPRB, 0.59522E+01_JPRB, 0.49783E+01_JPRB ,&
&  0.20078E+01_JPRB, 0.13982E+01_JPRB, 0.12778E+01_JPRB, 0.11207E+01_JPRB, 0.98068E+00_JPRB, 0.18068E+01_JPRB, 0.93460E+01_JPRB ,&
&  0.65112E+01_JPRB, 0.90671E+01_JPRB, 0.91795E+01_JPRB, 0.87003E+01_JPRB, 0.79916E+01_JPRB, 0.64588E+01_JPRB, 0.54580E+01_JPRB ,&
&  0.44091E+01_JPRB, 0.35888E+01_JPRB, 0.29924E+01_JPRB, 0.25484E+01_JPRB, 0.22745E+01_JPRB, 0.21341E+01_JPRB, 0.20724E+01_JPRB ,&
&  0.19754E+01_JPRB, 0.17729E+01_JPRB, 0.15759E+01_JPRB, 0.13209E+01_JPRB, 0.11097E+01_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.10471E+01_JPRB, 0.10563E+01_JPRB, 0.10939E+01_JPRB, 0.11521E+01_JPRB, 0.11840E+01_JPRB, 0.11804E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.12084E+01_JPRB, 0.12542E+01_JPRB, 0.12730E+01_JPRB, 0.12991E+01_JPRB, 0.12808E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.12238E+01_JPRB, 0.14516E+01_JPRB, 0.19582E+01_JPRB, 0.23961E+01_JPRB, 0.27176E+01_JPRB, 0.29308E+01_JPRB ,&
&  0.27470E+01_JPRB, 0.22254E+01_JPRB, 0.20359E+01_JPRB, 0.22851E+01_JPRB, 0.24186E+01_JPRB, 0.26924E+01_JPRB, 0.18744E+01_JPRB ,&
&  0.17216E+01_JPRB, 0.17011E+01_JPRB, 0.16782E+01_JPRB, 0.16935E+01_JPRB, 0.17161E+01_JPRB, 0.17089E+01_JPRB, 0.17177E+01_JPRB ,&
&  0.16952E+01_JPRB, 0.17073E+01_JPRB, 0.17433E+01_JPRB, 0.17848E+01_JPRB, 0.18255E+01_JPRB, 0.18722E+01_JPRB, 0.20386E+01_JPRB ,&
&  0.23372E+01_JPRB, 0.27356E+01_JPRB, 0.32838E+01_JPRB, 0.39423E+01_JPRB, 0.44505E+01_JPRB, 0.47770E+01_JPRB, 0.52250E+01_JPRB ,&
&  0.53742E+01_JPRB, 0.55188E+01_JPRB, 0.54356E+01_JPRB, 0.52816E+01_JPRB, 0.52889E+01_JPRB, 0.57578E+01_JPRB, 0.54642E+01_JPRB ,&
&  0.52132E+01_JPRB, 0.49349E+01_JPRB, 0.48786E+01_JPRB, 0.47850E+01_JPRB, 0.49279E+01_JPRB, 0.60016E+01_JPRB, 0.59058E+01_JPRB ,&
&  0.50082E+01_JPRB, 0.41924E+01_JPRB, 0.40780E+01_JPRB, 0.43976E+01_JPRB, 0.47425E+01_JPRB, 0.52696E+01_JPRB, 0.73390E+01_JPRB ,&
&  0.59360E+01_JPRB, 0.27034E+01_JPRB, 0.15615E+01_JPRB, 0.15689E+01_JPRB, 0.99726E+00_JPRB, 0.12299E+01_JPRB, 0.35445E+01_JPRB ,&
&  0.49620E+01_JPRB, 0.61577E+01_JPRB, 0.64918E+01_JPRB, 0.59793E+01_JPRB, 0.54282E+01_JPRB, 0.42120E+01_JPRB, 0.31128E+01_JPRB ,&
&  0.22835E+01_JPRB, 0.18289E+01_JPRB, 0.16008E+01_JPRB, 0.15241E+01_JPRB, 0.16058E+01_JPRB, 0.17251E+01_JPRB, 0.17992E+01_JPRB ,&
&  0.17590E+01_JPRB, 0.15643E+01_JPRB, 0.12823E+01_JPRB, 0.10745E+01_JPRB, 0.10221E+01_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.11847E+01_JPRB, 0.11742E+01_JPRB, 0.12033E+01_JPRB, 0.12624E+01_JPRB, 0.12892E+01_JPRB, 0.13088E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.14484E+01_JPRB, 0.14615E+01_JPRB, 0.14836E+01_JPRB, 0.14998E+01_JPRB, 0.14777E+01_JPRB, 0.14134E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.13677E+01_JPRB, 0.15231E+01_JPRB, 0.18900E+01_JPRB, 0.24347E+01_JPRB, 0.28134E+01_JPRB, 0.35452E+01_JPRB, 0.26664E+01_JPRB ,&
&  0.24486E+01_JPRB, 0.23653E+01_JPRB, 0.18607E+01_JPRB, 0.19820E+01_JPRB, 0.20507E+01_JPRB, 0.21646E+01_JPRB, 0.18234E+01_JPRB ,&
&  0.17273E+01_JPRB, 0.17110E+01_JPRB, 0.16643E+01_JPRB, 0.16687E+01_JPRB, 0.16823E+01_JPRB, 0.17076E+01_JPRB, 0.17225E+01_JPRB ,&
&  0.17824E+01_JPRB, 0.18734E+01_JPRB, 0.19430E+01_JPRB, 0.19697E+01_JPRB, 0.19983E+01_JPRB, 0.20618E+01_JPRB, 0.23041E+01_JPRB ,&
&  0.26164E+01_JPRB, 0.29962E+01_JPRB, 0.35323E+01_JPRB, 0.40127E+01_JPRB, 0.43799E+01_JPRB, 0.44821E+01_JPRB, 0.48617E+01_JPRB ,&
&  0.48082E+01_JPRB, 0.46419E+01_JPRB, 0.48733E+01_JPRB, 0.50151E+01_JPRB, 0.50873E+01_JPRB, 0.51944E+01_JPRB, 0.48050E+01_JPRB ,&
&  0.49756E+01_JPRB, 0.53492E+01_JPRB, 0.58781E+01_JPRB, 0.52806E+01_JPRB, 0.52236E+01_JPRB, 0.58591E+01_JPRB, 0.60367E+01_JPRB ,&
&  0.55640E+01_JPRB, 0.48375E+01_JPRB, 0.46007E+01_JPRB, 0.41523E+01_JPRB, 0.41215E+01_JPRB, 0.44145E+01_JPRB, 0.54362E+01_JPRB ,&
&  0.57355E+01_JPRB, 0.57330E+01_JPRB, 0.29811E+01_JPRB, 0.13714E+01_JPRB, 0.11452E+01_JPRB, 0.16178E+01_JPRB, 0.23788E+01_JPRB ,&
&  0.30991E+01_JPRB, 0.34154E+01_JPRB, 0.36670E+01_JPRB, 0.40599E+01_JPRB, 0.34557E+01_JPRB, 0.26241E+01_JPRB, 0.20170E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.13503E+01_JPRB, 0.12210E+01_JPRB, 0.12453E+01_JPRB, 0.14116E+01_JPRB, 0.15698E+01_JPRB, 0.16313E+01_JPRB ,&
&  0.14892E+01_JPRB, 0.12552E+01_JPRB, 0.11438E+01_JPRB, 0.11203E+01_JPRB, 0.11497E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.13380E+01_JPRB, 0.13658E+01_JPRB, 0.14269E+01_JPRB, 0.15117E+01_JPRB, 0.15854E+01_JPRB, 0.16665E+01_JPRB, 0.17327E+01_JPRB ,&
&  0.17532E+01_JPRB, 0.17399E+01_JPRB, 0.17332E+01_JPRB, 0.16924E+01_JPRB, 0.16695E+01_JPRB, 0.16562E+01_JPRB, 0.17224E+01_JPRB ,&
&  0.19352E+01_JPRB, 0.22724E+01_JPRB, 0.26371E+01_JPRB, 0.29233E+01_JPRB, 0.31052E+01_JPRB, 0.34661E+01_JPRB, 0.32247E+01_JPRB ,&
&  0.22580E+01_JPRB, 0.28940E+01_JPRB, 0.19604E+01_JPRB, 0.20566E+01_JPRB, 0.21593E+01_JPRB, 0.23836E+01_JPRB, 0.20129E+01_JPRB ,&
&  0.17808E+01_JPRB, 0.17695E+01_JPRB, 0.17676E+01_JPRB, 0.17800E+01_JPRB, 0.17535E+01_JPRB, 0.17971E+01_JPRB, 0.18862E+01_JPRB ,&
&  0.19955E+01_JPRB, 0.20656E+01_JPRB, 0.21067E+01_JPRB, 0.21358E+01_JPRB, 0.22039E+01_JPRB, 0.23219E+01_JPRB, 0.26364E+01_JPRB ,&
&  0.29513E+01_JPRB, 0.33587E+01_JPRB, 0.37898E+01_JPRB, 0.42086E+01_JPRB, 0.41651E+01_JPRB, 0.42079E+01_JPRB, 0.44656E+01_JPRB ,&
&  0.44937E+01_JPRB, 0.43743E+01_JPRB, 0.45737E+01_JPRB, 0.45611E+01_JPRB, 0.45004E+01_JPRB, 0.48018E+01_JPRB, 0.44375E+01_JPRB ,&
&  0.51437E+01_JPRB, 0.52418E+01_JPRB, 0.63122E+01_JPRB, 0.56258E+01_JPRB, 0.52075E+01_JPRB, 0.54077E+01_JPRB, 0.50399E+01_JPRB ,&
&  0.44123E+01_JPRB, 0.43201E+01_JPRB, 0.40253E+01_JPRB, 0.36544E+01_JPRB, 0.35903E+01_JPRB, 0.37437E+01_JPRB, 0.42331E+01_JPRB ,&
&  0.44129E+01_JPRB, 0.41832E+01_JPRB, 0.29370E+01_JPRB, 0.15196E+01_JPRB, 0.12571E+01_JPRB, 0.14606E+01_JPRB, 0.19047E+01_JPRB ,&
&  0.24150E+01_JPRB, 0.25950E+01_JPRB, 0.26351E+01_JPRB, 0.23378E+01_JPRB, 0.18640E+01_JPRB, 0.15209E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.12418E+01_JPRB, 0.12035E+01_JPRB, 0.11969E+01_JPRB, 0.13163E+01_JPRB, 0.14436E+01_JPRB, 0.14820E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.11706E+01_JPRB, 0.12225E+01_JPRB, 0.12693E+01_JPRB, 0.13112E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.15222E+01_JPRB, 0.15989E+01_JPRB, 0.17031E+01_JPRB, 0.18475E+01_JPRB, 0.20395E+01_JPRB, 0.22460E+01_JPRB, 0.21979E+01_JPRB ,&
&  0.21128E+01_JPRB, 0.19945E+01_JPRB, 0.20549E+01_JPRB, 0.21188E+01_JPRB, 0.21842E+01_JPRB, 0.22864E+01_JPRB, 0.24420E+01_JPRB ,&
&  0.26405E+01_JPRB, 0.28380E+01_JPRB, 0.30561E+01_JPRB, 0.32836E+01_JPRB, 0.35106E+01_JPRB, 0.39377E+01_JPRB, 0.39352E+01_JPRB ,&
&  0.33206E+01_JPRB, 0.22487E+01_JPRB, 0.23247E+01_JPRB, 0.20866E+01_JPRB, 0.21576E+01_JPRB, 0.23733E+01_JPRB, 0.23490E+01_JPRB ,&
&  0.20677E+01_JPRB, 0.19797E+01_JPRB, 0.19415E+01_JPRB, 0.19217E+01_JPRB, 0.18725E+01_JPRB, 0.19547E+01_JPRB, 0.20543E+01_JPRB ,&
&  0.21189E+01_JPRB, 0.21643E+01_JPRB, 0.22226E+01_JPRB, 0.22838E+01_JPRB, 0.23864E+01_JPRB, 0.25758E+01_JPRB, 0.29311E+01_JPRB ,&
&  0.32245E+01_JPRB, 0.35441E+01_JPRB, 0.38638E+01_JPRB, 0.39748E+01_JPRB, 0.38956E+01_JPRB, 0.38801E+01_JPRB, 0.39926E+01_JPRB ,&
&  0.41363E+01_JPRB, 0.43739E+01_JPRB, 0.44348E+01_JPRB, 0.43930E+01_JPRB, 0.41593E+01_JPRB, 0.46193E+01_JPRB, 0.46183E+01_JPRB ,&
&  0.51798E+01_JPRB, 0.53265E+01_JPRB, 0.63047E+01_JPRB, 0.53328E+01_JPRB, 0.48620E+01_JPRB, 0.45716E+01_JPRB, 0.42198E+01_JPRB ,&
&  0.37037E+01_JPRB, 0.36867E+01_JPRB, 0.34078E+01_JPRB, 0.29006E+01_JPRB, 0.26542E+01_JPRB, 0.25431E+01_JPRB, 0.31467E+01_JPRB ,&
&  0.34194E+01_JPRB, 0.34033E+01_JPRB, 0.21588E+01_JPRB, 0.15408E+01_JPRB, 0.13353E+01_JPRB, 0.13482E+01_JPRB, 0.16717E+01_JPRB ,&
&  0.21059E+01_JPRB, 0.20755E+01_JPRB, 0.18249E+01_JPRB, 0.16860E+01_JPRB, 0.14473E+01_JPRB, 0.12337E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11111E+01_JPRB, 0.11450E+01_JPRB, 0.12340E+01_JPRB, 0.12658E+01_JPRB, 0.13028E+01_JPRB, 0.12610E+01_JPRB ,&
&  0.12969E+01_JPRB, 0.13378E+01_JPRB, 0.13744E+01_JPRB, 0.14174E+01_JPRB, 0.14557E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.15709E+01_JPRB, 0.16525E+01_JPRB, 0.17473E+01_JPRB, 0.18572E+01_JPRB, 0.19716E+01_JPRB, 0.20463E+01_JPRB, 0.20815E+01_JPRB ,&
&  0.21617E+01_JPRB, 0.22675E+01_JPRB, 0.23844E+01_JPRB, 0.24770E+01_JPRB, 0.25264E+01_JPRB, 0.26004E+01_JPRB, 0.26911E+01_JPRB ,&
&  0.27795E+01_JPRB, 0.28903E+01_JPRB, 0.30479E+01_JPRB, 0.32445E+01_JPRB, 0.34860E+01_JPRB, 0.37851E+01_JPRB, 0.39180E+01_JPRB ,&
&  0.31104E+01_JPRB, 0.20352E+01_JPRB, 0.24027E+01_JPRB, 0.22989E+01_JPRB, 0.22226E+01_JPRB, 0.22713E+01_JPRB, 0.22377E+01_JPRB ,&
&  0.21271E+01_JPRB, 0.21093E+01_JPRB, 0.21183E+01_JPRB, 0.20912E+01_JPRB, 0.20296E+01_JPRB, 0.20853E+01_JPRB, 0.21410E+01_JPRB ,&
&  0.21953E+01_JPRB, 0.22535E+01_JPRB, 0.23111E+01_JPRB, 0.23664E+01_JPRB, 0.24821E+01_JPRB, 0.26784E+01_JPRB, 0.29920E+01_JPRB ,&
&  0.32997E+01_JPRB, 0.35063E+01_JPRB, 0.36874E+01_JPRB, 0.37921E+01_JPRB, 0.36984E+01_JPRB, 0.36088E+01_JPRB, 0.36415E+01_JPRB ,&
&  0.37842E+01_JPRB, 0.39635E+01_JPRB, 0.42281E+01_JPRB, 0.44667E+01_JPRB, 0.47157E+01_JPRB, 0.44945E+01_JPRB, 0.48660E+01_JPRB ,&
&  0.51934E+01_JPRB, 0.51332E+01_JPRB, 0.54275E+01_JPRB, 0.61466E+01_JPRB, 0.41448E+01_JPRB, 0.38241E+01_JPRB, 0.33689E+01_JPRB ,&
&  0.35189E+01_JPRB, 0.33362E+01_JPRB, 0.28536E+01_JPRB, 0.22582E+01_JPRB, 0.19701E+01_JPRB, 0.17537E+01_JPRB, 0.21481E+01_JPRB ,&
&  0.26706E+01_JPRB, 0.26459E+01_JPRB, 0.19293E+01_JPRB, 0.16388E+01_JPRB, 0.14914E+01_JPRB, 0.16388E+01_JPRB, 0.18710E+01_JPRB ,&
&  0.19651E+01_JPRB, 0.16999E+01_JPRB, 0.15671E+01_JPRB, 0.16744E+01_JPRB, 0.13770E+01_JPRB, 0.10119E+01_JPRB, 0.86143E+00_JPRB ,&
&  0.86673E+00_JPRB, 0.91290E+00_JPRB, 0.99155E+00_JPRB, 0.10954E+01_JPRB, 0.11362E+01_JPRB, 0.11839E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13737E+01_JPRB, 0.14288E+01_JPRB, 0.14529E+01_JPRB, 0.15115E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.14506E+01_JPRB, 0.15297E+01_JPRB, 0.15903E+01_JPRB, 0.16578E+01_JPRB, 0.17172E+01_JPRB, 0.17755E+01_JPRB, 0.18570E+01_JPRB ,&
&  0.19634E+01_JPRB, 0.20822E+01_JPRB, 0.21950E+01_JPRB, 0.22809E+01_JPRB, 0.23644E+01_JPRB, 0.24191E+01_JPRB, 0.24649E+01_JPRB ,&
&  0.24876E+01_JPRB, 0.25501E+01_JPRB, 0.26141E+01_JPRB, 0.27074E+01_JPRB, 0.28365E+01_JPRB, 0.29848E+01_JPRB, 0.31301E+01_JPRB ,&
&  0.32164E+01_JPRB, 0.30540E+01_JPRB, 0.26121E+01_JPRB, 0.19078E+01_JPRB, 0.17048E+01_JPRB, 0.19809E+01_JPRB, 0.20351E+01_JPRB ,&
&  0.20648E+01_JPRB, 0.20528E+01_JPRB, 0.20176E+01_JPRB, 0.20628E+01_JPRB, 0.20895E+01_JPRB, 0.21204E+01_JPRB, 0.21393E+01_JPRB ,&
&  0.21771E+01_JPRB, 0.22317E+01_JPRB, 0.22698E+01_JPRB, 0.23140E+01_JPRB, 0.24050E+01_JPRB, 0.25735E+01_JPRB, 0.28240E+01_JPRB ,&
&  0.31351E+01_JPRB, 0.33242E+01_JPRB, 0.34455E+01_JPRB, 0.35429E+01_JPRB, 0.34474E+01_JPRB, 0.33884E+01_JPRB, 0.34350E+01_JPRB ,&
&  0.35861E+01_JPRB, 0.37667E+01_JPRB, 0.40984E+01_JPRB, 0.44592E+01_JPRB, 0.46340E+01_JPRB, 0.44460E+01_JPRB, 0.45551E+01_JPRB ,&
&  0.47250E+01_JPRB, 0.46741E+01_JPRB, 0.46910E+01_JPRB, 0.58858E+01_JPRB, 0.38365E+01_JPRB, 0.38027E+01_JPRB, 0.36179E+01_JPRB ,&
&  0.34277E+01_JPRB, 0.33295E+01_JPRB, 0.25032E+01_JPRB, 0.18195E+01_JPRB, 0.15955E+01_JPRB, 0.14470E+01_JPRB, 0.17929E+01_JPRB ,&
&  0.22315E+01_JPRB, 0.19852E+01_JPRB, 0.16183E+01_JPRB, 0.14939E+01_JPRB, 0.15107E+01_JPRB, 0.18398E+01_JPRB, 0.20099E+01_JPRB ,&
&  0.16875E+01_JPRB, 0.14997E+01_JPRB, 0.14619E+01_JPRB, 0.18135E+01_JPRB, 0.16131E+01_JPRB, 0.89171E+00_JPRB, 0.74308E+00_JPRB ,&
&  0.75742E+00_JPRB, 0.81355E+00_JPRB, 0.87840E+00_JPRB, 0.93433E+00_JPRB, 0.97508E+00_JPRB, 0.10228E+01_JPRB, 0.10761E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.12359E+01_JPRB, 0.12971E+01_JPRB, 0.13518E+01_JPRB, 0.13940E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.10987E+01_JPRB, 0.11693E+01_JPRB, 0.12350E+01_JPRB, 0.12784E+01_JPRB, 0.12926E+01_JPRB, 0.13047E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.13024E+01_JPRB, 0.13357E+01_JPRB, 0.14227E+01_JPRB, 0.15886E+01_JPRB, 0.16806E+01_JPRB, 0.17895E+01_JPRB ,&
&  0.17931E+01_JPRB, 0.18051E+01_JPRB, 0.18207E+01_JPRB, 0.18584E+01_JPRB, 0.19322E+01_JPRB, 0.20415E+01_JPRB, 0.22956E+01_JPRB ,&
&  0.27920E+01_JPRB, 0.31072E+01_JPRB, 0.30548E+01_JPRB, 0.26909E+01_JPRB, 0.21482E+01_JPRB, 0.19603E+01_JPRB, 0.22193E+01_JPRB ,&
&  0.22753E+01_JPRB, 0.21156E+01_JPRB, 0.20124E+01_JPRB, 0.19989E+01_JPRB, 0.20064E+01_JPRB, 0.20254E+01_JPRB, 0.20420E+01_JPRB ,&
&  0.20741E+01_JPRB, 0.21137E+01_JPRB, 0.21410E+01_JPRB, 0.21772E+01_JPRB, 0.22533E+01_JPRB, 0.23656E+01_JPRB, 0.25095E+01_JPRB ,&
&  0.27173E+01_JPRB, 0.28567E+01_JPRB, 0.29179E+01_JPRB, 0.29404E+01_JPRB, 0.29666E+01_JPRB, 0.30672E+01_JPRB, 0.32541E+01_JPRB ,&
&  0.34507E+01_JPRB, 0.35548E+01_JPRB, 0.38959E+01_JPRB, 0.41686E+01_JPRB, 0.41368E+01_JPRB, 0.40748E+01_JPRB, 0.40889E+01_JPRB ,&
&  0.40792E+01_JPRB, 0.40193E+01_JPRB, 0.35877E+01_JPRB, 0.50149E+01_JPRB, 0.51192E+01_JPRB, 0.43603E+01_JPRB, 0.36425E+01_JPRB ,&
&  0.33157E+01_JPRB, 0.30462E+01_JPRB, 0.25103E+01_JPRB, 0.19327E+01_JPRB, 0.16716E+01_JPRB, 0.17866E+01_JPRB, 0.19518E+01_JPRB ,&
&  0.20226E+01_JPRB, 0.16420E+01_JPRB, 0.14781E+01_JPRB, 0.14509E+01_JPRB, 0.13999E+01_JPRB, 0.17633E+01_JPRB, 0.16983E+01_JPRB ,&
&  0.13443E+01_JPRB, 0.12536E+01_JPRB, 0.13224E+01_JPRB, 0.17857E+01_JPRB, 0.24882E+01_JPRB, 0.80520E+00_JPRB, 0.64282E+00_JPRB ,&
&  0.66201E+00_JPRB, 0.69536E+00_JPRB, 0.73732E+00_JPRB, 0.75491E+00_JPRB, 0.75350E+00_JPRB, 0.78889E+00_JPRB, 0.84579E+00_JPRB ,&
&  0.87272E+00_JPRB, 0.89326E+00_JPRB, 0.91772E+00_JPRB, 0.99435E+00_JPRB, 0.10395E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.71309E+00_JPRB, 0.75113E+00_JPRB, 0.77642E+00_JPRB, 0.80210E+00_JPRB, 0.81102E+00_JPRB, 0.80026E+00_JPRB, 0.76114E+00_JPRB ,&
&  0.73213E+00_JPRB, 0.70547E+00_JPRB, 0.69898E+00_JPRB, 0.68544E+00_JPRB, 0.71445E+00_JPRB, 0.77039E+00_JPRB, 0.80926E+00_JPRB ,&
&  0.91343E+00_JPRB, 0.85268E+00_JPRB, 0.84682E+00_JPRB, 0.87388E+00_JPRB, 0.87614E+00_JPRB, 0.91480E+00_JPRB, 0.10632E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.15562E+01_JPRB, 0.19429E+01_JPRB, 0.25651E+01_JPRB, 0.27516E+01_JPRB, 0.22183E+01_JPRB, 0.23453E+01_JPRB ,&
&  0.21006E+01_JPRB, 0.17834E+01_JPRB, 0.17720E+01_JPRB, 0.17527E+01_JPRB, 0.18008E+01_JPRB, 0.18174E+01_JPRB, 0.18641E+01_JPRB ,&
&  0.19154E+01_JPRB, 0.19704E+01_JPRB, 0.20127E+01_JPRB, 0.20447E+01_JPRB, 0.20871E+01_JPRB, 0.21393E+01_JPRB, 0.22133E+01_JPRB ,&
&  0.22653E+01_JPRB, 0.22797E+01_JPRB, 0.23260E+01_JPRB, 0.23265E+01_JPRB, 0.25135E+01_JPRB, 0.27349E+01_JPRB, 0.29533E+01_JPRB ,&
&  0.30999E+01_JPRB, 0.32204E+01_JPRB, 0.34970E+01_JPRB, 0.35787E+01_JPRB, 0.35899E+01_JPRB, 0.35615E+01_JPRB, 0.34115E+01_JPRB ,&
&  0.31641E+01_JPRB, 0.25850E+01_JPRB, 0.24505E+01_JPRB, 0.38638E+01_JPRB, 0.38170E+01_JPRB, 0.31314E+01_JPRB, 0.32277E+01_JPRB ,&
&  0.29981E+01_JPRB, 0.28373E+01_JPRB, 0.24915E+01_JPRB, 0.20745E+01_JPRB, 0.17786E+01_JPRB, 0.16809E+01_JPRB, 0.18421E+01_JPRB ,&
&  0.16304E+01_JPRB, 0.13656E+01_JPRB, 0.13753E+01_JPRB, 0.14310E+01_JPRB, 0.14025E+01_JPRB, 0.15280E+01_JPRB, 0.14845E+01_JPRB ,&
&  0.12909E+01_JPRB, 0.10707E+01_JPRB, 0.12122E+01_JPRB, 0.15849E+01_JPRB, 0.11666E+01_JPRB, 0.65172E+00_JPRB, 0.55777E+00_JPRB ,&
&  0.56743E+00_JPRB, 0.57925E+00_JPRB, 0.58954E+00_JPRB, 0.59485E+00_JPRB, 0.59917E+00_JPRB, 0.61646E+00_JPRB, 0.62640E+00_JPRB ,&
&  0.63506E+00_JPRB, 0.66036E+00_JPRB, 0.66792E+00_JPRB, 0.68143E+00_JPRB, 0.70343E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.61599E+00_JPRB, 0.61554E+00_JPRB, 0.60811E+00_JPRB, 0.60894E+00_JPRB, 0.61399E+00_JPRB, 0.62910E+00_JPRB, 0.63870E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.64455E+00_JPRB, 0.62785E+00_JPRB, 0.63096E+00_JPRB, 0.65213E+00_JPRB, 0.66841E+00_JPRB, 0.68441E+00_JPRB ,&
&  0.73066E+00_JPRB, 0.74443E+00_JPRB, 0.74101E+00_JPRB, 0.74634E+00_JPRB, 0.73896E+00_JPRB, 0.78939E+00_JPRB, 0.79315E+00_JPRB ,&
&  0.77229E+00_JPRB, 0.79787E+00_JPRB, 0.93813E+00_JPRB, 0.12987E+01_JPRB, 0.16417E+01_JPRB, 0.16649E+01_JPRB, 0.19370E+01_JPRB ,&
&  0.18571E+01_JPRB, 0.14219E+01_JPRB, 0.14867E+01_JPRB, 0.13084E+01_JPRB, 0.13337E+01_JPRB, 0.13950E+01_JPRB, 0.14095E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.14195E+01_JPRB, 0.14238E+01_JPRB, 0.14393E+01_JPRB, 0.14341E+01_JPRB, 0.14260E+01_JPRB, 0.14080E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.13243E+01_JPRB, 0.14358E+01_JPRB, 0.16886E+01_JPRB, 0.19765E+01_JPRB, 0.22873E+01_JPRB, 0.24074E+01_JPRB ,&
&  0.24983E+01_JPRB, 0.26046E+01_JPRB, 0.25111E+01_JPRB, 0.26073E+01_JPRB, 0.27782E+01_JPRB, 0.25718E+01_JPRB, 0.23974E+01_JPRB ,&
&  0.22090E+01_JPRB, 0.17461E+01_JPRB, 0.18058E+01_JPRB, 0.26851E+01_JPRB, 0.28928E+01_JPRB, 0.27829E+01_JPRB, 0.28184E+01_JPRB ,&
&  0.26791E+01_JPRB, 0.24824E+01_JPRB, 0.21398E+01_JPRB, 0.18898E+01_JPRB, 0.16806E+01_JPRB, 0.14825E+01_JPRB, 0.15009E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12426E+01_JPRB, 0.13961E+01_JPRB, 0.15044E+01_JPRB, 0.15930E+01_JPRB, 0.18157E+01_JPRB, 0.14250E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.10427E+01_JPRB, 0.14731E+01_JPRB, 0.19799E+01_JPRB, 0.95896E+00_JPRB, 0.62932E+00_JPRB, 0.48743E+00_JPRB ,&
&  0.50279E+00_JPRB, 0.52071E+00_JPRB, 0.52505E+00_JPRB, 0.53008E+00_JPRB, 0.54370E+00_JPRB, 0.55423E+00_JPRB, 0.57389E+00_JPRB ,&
&  0.58376E+00_JPRB, 0.59153E+00_JPRB, 0.60011E+00_JPRB, 0.61501E+00_JPRB, 0.61190E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.65987E+00_JPRB, 0.66944E+00_JPRB, 0.67639E+00_JPRB, 0.69488E+00_JPRB, 0.72268E+00_JPRB, 0.73104E+00_JPRB, 0.74904E+00_JPRB ,&
&  0.76039E+00_JPRB, 0.78057E+00_JPRB, 0.80188E+00_JPRB, 0.82719E+00_JPRB, 0.84515E+00_JPRB, 0.85442E+00_JPRB, 0.87975E+00_JPRB ,&
&  0.90198E+00_JPRB, 0.89862E+00_JPRB, 0.91087E+00_JPRB, 0.93617E+00_JPRB, 0.94116E+00_JPRB, 0.97496E+00_JPRB, 0.96857E+00_JPRB ,&
&  0.99625E+00_JPRB, 0.99358E+00_JPRB, 0.10206E+01_JPRB, 0.11114E+01_JPRB, 0.12167E+01_JPRB, 0.14225E+01_JPRB, 0.18186E+01_JPRB ,&
&  0.16012E+01_JPRB, 0.12118E+01_JPRB, 0.10956E+01_JPRB, 0.93755E+00_JPRB, 0.93273E+00_JPRB, 0.95629E+00_JPRB, 0.94420E+00_JPRB ,&
&  0.91838E+00_JPRB, 0.86603E+00_JPRB, 0.83790E+00_JPRB, 0.81865E+00_JPRB, 0.83477E+00_JPRB, 0.90365E+00_JPRB, 0.94331E+00_JPRB ,&
&  0.92824E+00_JPRB, 0.10390E+01_JPRB, 0.11924E+01_JPRB, 0.14108E+01_JPRB, 0.15862E+01_JPRB, 0.19558E+01_JPRB, 0.19499E+01_JPRB ,&
&  0.18613E+01_JPRB, 0.18144E+01_JPRB, 0.16301E+01_JPRB, 0.17176E+01_JPRB, 0.18520E+01_JPRB, 0.17641E+01_JPRB, 0.15597E+01_JPRB ,&
&  0.15246E+01_JPRB, 0.14634E+01_JPRB, 0.16229E+01_JPRB, 0.22400E+01_JPRB, 0.24838E+01_JPRB, 0.25507E+01_JPRB, 0.24324E+01_JPRB ,&
&  0.22569E+01_JPRB, 0.20174E+01_JPRB, 0.18010E+01_JPRB, 0.16915E+01_JPRB, 0.16345E+01_JPRB, 0.14893E+01_JPRB, 0.13354E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.12830E+01_JPRB, 0.14214E+01_JPRB, 0.15193E+01_JPRB, 0.18514E+01_JPRB, 0.16527E+01_JPRB, 0.18490E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.99924E+00_JPRB, 0.81250E+00_JPRB, 0.10705E+01_JPRB, 0.71876E+00_JPRB, 0.53855E+00_JPRB, 0.52258E+00_JPRB ,&
&  0.53556E+00_JPRB, 0.54392E+00_JPRB, 0.55678E+00_JPRB, 0.56616E+00_JPRB, 0.57818E+00_JPRB, 0.60119E+00_JPRB, 0.62510E+00_JPRB ,&
&  0.64693E+00_JPRB, 0.63575E+00_JPRB, 0.63566E+00_JPRB, 0.64115E+00_JPRB, 0.65630E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.70037E+00_JPRB, 0.72203E+00_JPRB, 0.74398E+00_JPRB, 0.76601E+00_JPRB, 0.78522E+00_JPRB, 0.80354E+00_JPRB, 0.82265E+00_JPRB ,&
&  0.84160E+00_JPRB, 0.86368E+00_JPRB, 0.88714E+00_JPRB, 0.91252E+00_JPRB, 0.93473E+00_JPRB, 0.94671E+00_JPRB, 0.96381E+00_JPRB ,&
&  0.97856E+00_JPRB, 0.98622E+00_JPRB, 0.98209E+00_JPRB, 0.10115E+01_JPRB, 0.10407E+01_JPRB, 0.10585E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.10975E+01_JPRB, 0.10915E+01_JPRB, 0.11208E+01_JPRB, 0.11421E+01_JPRB, 0.10954E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.97136E+00_JPRB, 0.10652E+01_JPRB, 0.97393E+00_JPRB, 0.85513E+00_JPRB, 0.82359E+00_JPRB, 0.73073E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.71744E+00_JPRB, 0.71372E+00_JPRB, 0.70828E+00_JPRB, 0.72437E+00_JPRB, 0.79346E+00_JPRB, 0.92355E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.12692E+01_JPRB, 0.14233E+01_JPRB, 0.15912E+01_JPRB, 0.17650E+01_JPRB, 0.18544E+01_JPRB, 0.17804E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.15685E+01_JPRB, 0.12486E+01_JPRB, 0.13911E+01_JPRB, 0.15952E+01_JPRB, 0.15608E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.12476E+01_JPRB, 0.12956E+01_JPRB, 0.15308E+01_JPRB, 0.18259E+01_JPRB, 0.20022E+01_JPRB, 0.20633E+01_JPRB, 0.19625E+01_JPRB ,&
&  0.18378E+01_JPRB, 0.16222E+01_JPRB, 0.15204E+01_JPRB, 0.15515E+01_JPRB, 0.15401E+01_JPRB, 0.14541E+01_JPRB, 0.12666E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.12721E+01_JPRB, 0.13682E+01_JPRB, 0.14669E+01_JPRB, 0.16265E+01_JPRB, 0.23995E+01_JPRB, 0.24713E+01_JPRB ,&
&  0.10430E+01_JPRB, 0.84564E+00_JPRB, 0.54328E+00_JPRB, 0.63822E+00_JPRB, 0.66657E+00_JPRB, 0.63035E+00_JPRB, 0.67749E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.67890E+00_JPRB, 0.66863E+00_JPRB, 0.64450E+00_JPRB, 0.64449E+00_JPRB, 0.63976E+00_JPRB, 0.65217E+00_JPRB ,&
&  0.65861E+00_JPRB, 0.65819E+00_JPRB, 0.66584E+00_JPRB, 0.67698E+00_JPRB, 0.68570E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.72555E+00_JPRB, 0.74945E+00_JPRB, 0.77346E+00_JPRB, 0.79505E+00_JPRB, 0.81613E+00_JPRB, 0.83574E+00_JPRB, 0.85648E+00_JPRB ,&
&  0.87766E+00_JPRB, 0.90042E+00_JPRB, 0.92248E+00_JPRB, 0.94968E+00_JPRB, 0.97590E+00_JPRB, 0.99126E+00_JPRB, 0.10166E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10703E+01_JPRB, 0.10912E+01_JPRB, 0.11320E+01_JPRB, 0.11673E+01_JPRB, 0.11904E+01_JPRB, 0.12311E+01_JPRB ,&
&  0.12490E+01_JPRB, 0.12569E+01_JPRB, 0.12677E+01_JPRB, 0.13005E+01_JPRB, 0.13175E+01_JPRB, 0.14174E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.10644E+01_JPRB, 0.98151E+00_JPRB, 0.91617E+00_JPRB, 0.86211E+00_JPRB, 0.80866E+00_JPRB, 0.77882E+00_JPRB ,&
&  0.78988E+00_JPRB, 0.78897E+00_JPRB, 0.80836E+00_JPRB, 0.84097E+00_JPRB, 0.91328E+00_JPRB, 0.10479E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.14152E+01_JPRB, 0.15416E+01_JPRB, 0.16670E+01_JPRB, 0.18039E+01_JPRB, 0.19313E+01_JPRB, 0.20146E+01_JPRB ,&
&  0.20025E+01_JPRB, 0.19508E+01_JPRB, 0.16875E+01_JPRB, 0.18356E+01_JPRB, 0.19559E+01_JPRB, 0.18019E+01_JPRB, 0.14469E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.10935E+01_JPRB, 0.11842E+01_JPRB, 0.14453E+01_JPRB, 0.15533E+01_JPRB, 0.15711E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.14101E+01_JPRB, 0.12843E+01_JPRB, 0.12386E+01_JPRB, 0.12686E+01_JPRB, 0.13364E+01_JPRB, 0.13751E+01_JPRB, 0.12965E+01_JPRB ,&
&  0.12642E+01_JPRB, 0.12414E+01_JPRB, 0.12217E+01_JPRB, 0.12031E+01_JPRB, 0.12952E+01_JPRB, 0.12589E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.84369E+00_JPRB, 0.67361E+00_JPRB, 0.67424E+00_JPRB, 0.64529E+00_JPRB, 0.71653E+00_JPRB, 0.75203E+00_JPRB, 0.92012E+00_JPRB ,&
&  0.10566E+01_JPRB, 0.11082E+01_JPRB, 0.10388E+01_JPRB, 0.85932E+00_JPRB, 0.72023E+00_JPRB, 0.66564E+00_JPRB, 0.66493E+00_JPRB ,&
&  0.66635E+00_JPRB, 0.66408E+00_JPRB, 0.67316E+00_JPRB, 0.69006E+00_JPRB, 0.70538E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.69261E+00_JPRB, 0.71418E+00_JPRB, 0.73779E+00_JPRB, 0.75698E+00_JPRB, 0.78183E+00_JPRB, 0.81810E+00_JPRB, 0.85438E+00_JPRB ,&
&  0.88531E+00_JPRB, 0.90823E+00_JPRB, 0.92549E+00_JPRB, 0.95035E+00_JPRB, 0.98251E+00_JPRB, 0.10269E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.11621E+01_JPRB, 0.11929E+01_JPRB, 0.12477E+01_JPRB, 0.12762E+01_JPRB, 0.13415E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14282E+01_JPRB, 0.14430E+01_JPRB, 0.14753E+01_JPRB, 0.14626E+01_JPRB, 0.15207E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.12136E+01_JPRB, 0.11156E+01_JPRB, 0.10701E+01_JPRB, 0.10299E+01_JPRB, 0.10001E+01_JPRB, 0.98651E+00_JPRB ,&
&  0.97698E+00_JPRB, 0.96014E+00_JPRB, 0.91594E+00_JPRB, 0.92152E+00_JPRB, 0.97835E+00_JPRB, 0.10922E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.13279E+01_JPRB, 0.14494E+01_JPRB, 0.15657E+01_JPRB, 0.17141E+01_JPRB, 0.19095E+01_JPRB, 0.21129E+01_JPRB, 0.23073E+01_JPRB ,&
&  0.23669E+01_JPRB, 0.23949E+01_JPRB, 0.22071E+01_JPRB, 0.22162E+01_JPRB, 0.22815E+01_JPRB, 0.19473E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.99418E+00_JPRB, 0.10993E+01_JPRB, 0.12344E+01_JPRB, 0.11885E+01_JPRB, 0.11327E+01_JPRB, 0.10470E+01_JPRB ,&
&  0.97202E+00_JPRB, 0.90926E+00_JPRB, 0.91884E+00_JPRB, 0.98514E+00_JPRB, 0.10748E+01_JPRB, 0.11820E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.11981E+01_JPRB, 0.11290E+01_JPRB, 0.10678E+01_JPRB, 0.10417E+01_JPRB, 0.10214E+01_JPRB, 0.94727E+00_JPRB, 0.10537E+01_JPRB ,&
&  0.81714E+00_JPRB, 0.77343E+00_JPRB, 0.78057E+00_JPRB, 0.57952E+00_JPRB, 0.75435E+00_JPRB, 0.93116E+00_JPRB, 0.11866E+01_JPRB ,&
&  0.13109E+01_JPRB, 0.17528E+01_JPRB, 0.21563E+01_JPRB, 0.15440E+01_JPRB, 0.11510E+01_JPRB, 0.72793E+00_JPRB, 0.65694E+00_JPRB ,&
&  0.65814E+00_JPRB, 0.64694E+00_JPRB, 0.64879E+00_JPRB, 0.66114E+00_JPRB, 0.67464E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.64508E+00_JPRB, 0.66025E+00_JPRB, 0.68229E+00_JPRB, 0.68665E+00_JPRB, 0.69657E+00_JPRB, 0.71642E+00_JPRB, 0.73686E+00_JPRB ,&
&  0.75672E+00_JPRB, 0.77053E+00_JPRB, 0.78582E+00_JPRB, 0.80504E+00_JPRB, 0.82886E+00_JPRB, 0.88577E+00_JPRB, 0.96872E+00_JPRB ,&
&  0.10400E+01_JPRB, 0.11082E+01_JPRB, 0.11931E+01_JPRB, 0.12319E+01_JPRB, 0.12950E+01_JPRB, 0.13732E+01_JPRB, 0.14262E+01_JPRB ,&
&  0.14968E+01_JPRB, 0.15320E+01_JPRB, 0.15670E+01_JPRB, 0.16504E+01_JPRB, 0.16106E+01_JPRB, 0.16476E+01_JPRB, 0.12678E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.13008E+01_JPRB, 0.12405E+01_JPRB, 0.12111E+01_JPRB, 0.11717E+01_JPRB, 0.11117E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.97349E+00_JPRB, 0.89983E+00_JPRB, 0.91904E+00_JPRB, 0.95065E+00_JPRB, 0.10156E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.12795E+01_JPRB, 0.13992E+01_JPRB, 0.15144E+01_JPRB, 0.16820E+01_JPRB, 0.19074E+01_JPRB, 0.21463E+01_JPRB, 0.23942E+01_JPRB ,&
&  0.25554E+01_JPRB, 0.26760E+01_JPRB, 0.27068E+01_JPRB, 0.23330E+01_JPRB, 0.22393E+01_JPRB, 0.18089E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.98166E+00_JPRB, 0.10122E+01_JPRB, 0.10588E+01_JPRB, 0.96332E+00_JPRB, 0.88783E+00_JPRB, 0.80197E+00_JPRB ,&
&  0.74256E+00_JPRB, 0.72977E+00_JPRB, 0.75397E+00_JPRB, 0.82731E+00_JPRB, 0.91055E+00_JPRB, 0.10125E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.10293E+01_JPRB, 0.96296E+00_JPRB, 0.92568E+00_JPRB, 0.91875E+00_JPRB, 0.94922E+00_JPRB, 0.10042E+01_JPRB, 0.90243E+00_JPRB ,&
&  0.83065E+00_JPRB, 0.70665E+00_JPRB, 0.68689E+00_JPRB, 0.74210E+00_JPRB, 0.80040E+00_JPRB, 0.10335E+01_JPRB, 0.14489E+01_JPRB ,&
&  0.20599E+01_JPRB, 0.21805E+01_JPRB, 0.26407E+01_JPRB, 0.34991E+01_JPRB, 0.25725E+01_JPRB, 0.18244E+01_JPRB, 0.59213E+00_JPRB ,&
&  0.60819E+00_JPRB, 0.60620E+00_JPRB, 0.60795E+00_JPRB, 0.61014E+00_JPRB, 0.62179E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.54200E+00_JPRB, 0.56729E+00_JPRB, 0.55827E+00_JPRB, 0.56172E+00_JPRB, 0.56876E+00_JPRB, 0.58068E+00_JPRB, 0.60606E+00_JPRB ,&
&  0.61290E+00_JPRB, 0.64827E+00_JPRB, 0.68714E+00_JPRB, 0.71888E+00_JPRB, 0.76194E+00_JPRB, 0.82241E+00_JPRB, 0.89994E+00_JPRB ,&
&  0.98407E+00_JPRB, 0.10349E+01_JPRB, 0.10819E+01_JPRB, 0.11187E+01_JPRB, 0.11811E+01_JPRB, 0.12617E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.14073E+01_JPRB, 0.14763E+01_JPRB, 0.15562E+01_JPRB, 0.16697E+01_JPRB, 0.15989E+01_JPRB, 0.15844E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.12942E+01_JPRB, 0.13156E+01_JPRB, 0.12980E+01_JPRB, 0.12729E+01_JPRB, 0.12190E+01_JPRB, 0.11305E+01_JPRB, 0.10726E+01_JPRB ,&
&  0.10209E+01_JPRB, 0.96507E+00_JPRB, 0.89840E+00_JPRB, 0.86069E+00_JPRB, 0.92816E+00_JPRB, 0.93441E+00_JPRB, 0.10438E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.13209E+01_JPRB, 0.14577E+01_JPRB, 0.16210E+01_JPRB, 0.18357E+01_JPRB, 0.20813E+01_JPRB, 0.23306E+01_JPRB ,&
&  0.25152E+01_JPRB, 0.26689E+01_JPRB, 0.26395E+01_JPRB, 0.22250E+01_JPRB, 0.20072E+01_JPRB, 0.16064E+01_JPRB, 0.11939E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.96942E+00_JPRB, 0.99429E+00_JPRB, 0.98142E+00_JPRB, 0.87148E+00_JPRB, 0.77436E+00_JPRB, 0.69256E+00_JPRB ,&
&  0.62839E+00_JPRB, 0.62550E+00_JPRB, 0.68094E+00_JPRB, 0.75453E+00_JPRB, 0.82366E+00_JPRB, 0.89287E+00_JPRB, 0.89689E+00_JPRB ,&
&  0.86481E+00_JPRB, 0.80730E+00_JPRB, 0.79287E+00_JPRB, 0.84898E+00_JPRB, 0.98982E+00_JPRB, 0.11342E+01_JPRB, 0.12290E+01_JPRB ,&
&  0.89530E+00_JPRB, 0.70639E+00_JPRB, 0.69583E+00_JPRB, 0.71744E+00_JPRB, 0.75801E+00_JPRB, 0.83090E+00_JPRB, 0.94761E+00_JPRB ,&
&  0.11243E+01_JPRB, 0.15003E+01_JPRB, 0.14466E+01_JPRB, 0.11807E+01_JPRB, 0.16756E+01_JPRB, 0.13986E+01_JPRB, 0.53907E+00_JPRB ,&
&  0.51934E+00_JPRB, 0.50889E+00_JPRB, 0.51634E+00_JPRB, 0.52171E+00_JPRB, 0.53651E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.50831E+00_JPRB, 0.49757E+00_JPRB, 0.47989E+00_JPRB, 0.48907E+00_JPRB, 0.49017E+00_JPRB, 0.50862E+00_JPRB, 0.52983E+00_JPRB ,&
&  0.55864E+00_JPRB, 0.61372E+00_JPRB, 0.66877E+00_JPRB, 0.71443E+00_JPRB, 0.74447E+00_JPRB, 0.78179E+00_JPRB, 0.81753E+00_JPRB ,&
&  0.84453E+00_JPRB, 0.86991E+00_JPRB, 0.90569E+00_JPRB, 0.95537E+00_JPRB, 0.10200E+01_JPRB, 0.10941E+01_JPRB, 0.11843E+01_JPRB ,&
&  0.12851E+01_JPRB, 0.13938E+01_JPRB, 0.15307E+01_JPRB, 0.15867E+01_JPRB, 0.16305E+01_JPRB, 0.16764E+01_JPRB, 0.16586E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.13775E+01_JPRB, 0.13380E+01_JPRB, 0.12811E+01_JPRB, 0.12356E+01_JPRB, 0.11614E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.96506E+00_JPRB, 0.92540E+00_JPRB, 0.85216E+00_JPRB, 0.87252E+00_JPRB, 0.88353E+00_JPRB, 0.94273E+00_JPRB ,&
&  0.10395E+01_JPRB, 0.11877E+01_JPRB, 0.13714E+01_JPRB, 0.15471E+01_JPRB, 0.17192E+01_JPRB, 0.19081E+01_JPRB, 0.21719E+01_JPRB ,&
&  0.24526E+01_JPRB, 0.26385E+01_JPRB, 0.27473E+01_JPRB, 0.23920E+01_JPRB, 0.19620E+01_JPRB, 0.17425E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.10342E+01_JPRB, 0.91271E+00_JPRB, 0.94309E+00_JPRB, 0.94753E+00_JPRB, 0.85303E+00_JPRB, 0.76799E+00_JPRB, 0.68765E+00_JPRB ,&
&  0.61258E+00_JPRB, 0.56945E+00_JPRB, 0.59135E+00_JPRB, 0.67894E+00_JPRB, 0.72505E+00_JPRB, 0.75303E+00_JPRB, 0.74513E+00_JPRB ,&
&  0.73088E+00_JPRB, 0.69136E+00_JPRB, 0.66073E+00_JPRB, 0.67834E+00_JPRB, 0.72986E+00_JPRB, 0.77611E+00_JPRB, 0.77347E+00_JPRB ,&
&  0.74742E+00_JPRB, 0.73482E+00_JPRB, 0.73271E+00_JPRB, 0.74628E+00_JPRB, 0.75413E+00_JPRB, 0.76927E+00_JPRB, 0.78547E+00_JPRB ,&
&  0.80759E+00_JPRB, 0.84521E+00_JPRB, 0.82446E+00_JPRB, 0.79794E+00_JPRB, 0.86223E+00_JPRB, 0.90932E+00_JPRB, 0.59758E+00_JPRB ,&
&  0.51386E+00_JPRB, 0.48056E+00_JPRB, 0.48305E+00_JPRB, 0.47821E+00_JPRB, 0.51657E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.51612E+00_JPRB, 0.49823E+00_JPRB, 0.48840E+00_JPRB, 0.47959E+00_JPRB, 0.46477E+00_JPRB, 0.48892E+00_JPRB, 0.49511E+00_JPRB ,&
&  0.52042E+00_JPRB, 0.56169E+00_JPRB, 0.61321E+00_JPRB, 0.64479E+00_JPRB, 0.65645E+00_JPRB, 0.68660E+00_JPRB, 0.71225E+00_JPRB ,&
&  0.72890E+00_JPRB, 0.74138E+00_JPRB, 0.75966E+00_JPRB, 0.78344E+00_JPRB, 0.82525E+00_JPRB, 0.88901E+00_JPRB, 0.98414E+00_JPRB ,&
&  0.10942E+01_JPRB, 0.12135E+01_JPRB, 0.13425E+01_JPRB, 0.14643E+01_JPRB, 0.15496E+01_JPRB, 0.16175E+01_JPRB, 0.15822E+01_JPRB ,&
&  0.93271E+00_JPRB, 0.12693E+01_JPRB, 0.14540E+01_JPRB, 0.13536E+01_JPRB, 0.12663E+01_JPRB, 0.11860E+01_JPRB, 0.11079E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.95439E+00_JPRB, 0.92212E+00_JPRB, 0.92399E+00_JPRB, 0.83643E+00_JPRB, 0.83656E+00_JPRB, 0.86455E+00_JPRB ,&
&  0.92118E+00_JPRB, 0.10252E+01_JPRB, 0.12214E+01_JPRB, 0.14374E+01_JPRB, 0.15583E+01_JPRB, 0.16782E+01_JPRB, 0.19015E+01_JPRB ,&
&  0.22296E+01_JPRB, 0.24746E+01_JPRB, 0.26465E+01_JPRB, 0.20470E+01_JPRB, 0.17873E+01_JPRB, 0.17429E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.11283E+01_JPRB, 0.92275E+00_JPRB, 0.93908E+00_JPRB, 0.95202E+00_JPRB, 0.86379E+00_JPRB, 0.80596E+00_JPRB, 0.73842E+00_JPRB ,&
&  0.65380E+00_JPRB, 0.59081E+00_JPRB, 0.54538E+00_JPRB, 0.57653E+00_JPRB, 0.60855E+00_JPRB, 0.62805E+00_JPRB, 0.60612E+00_JPRB ,&
&  0.63992E+00_JPRB, 0.63155E+00_JPRB, 0.63209E+00_JPRB, 0.64656E+00_JPRB, 0.71732E+00_JPRB, 0.75086E+00_JPRB, 0.76724E+00_JPRB ,&
&  0.78471E+00_JPRB, 0.79778E+00_JPRB, 0.79759E+00_JPRB, 0.80214E+00_JPRB, 0.80339E+00_JPRB, 0.79678E+00_JPRB, 0.78196E+00_JPRB ,&
&  0.75836E+00_JPRB, 0.74598E+00_JPRB, 0.71980E+00_JPRB, 0.69090E+00_JPRB, 0.64625E+00_JPRB, 0.65143E+00_JPRB, 0.58462E+00_JPRB ,&
&  0.53891E+00_JPRB, 0.50742E+00_JPRB, 0.48672E+00_JPRB, 0.48264E+00_JPRB, 0.49270E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.53002E+00_JPRB, 0.52610E+00_JPRB, 0.52064E+00_JPRB, 0.50942E+00_JPRB, 0.49522E+00_JPRB, 0.49371E+00_JPRB, 0.48053E+00_JPRB ,&
&  0.47758E+00_JPRB, 0.49873E+00_JPRB, 0.53307E+00_JPRB, 0.56171E+00_JPRB, 0.58759E+00_JPRB, 0.60368E+00_JPRB, 0.62632E+00_JPRB ,&
&  0.63138E+00_JPRB, 0.63092E+00_JPRB, 0.62864E+00_JPRB, 0.62833E+00_JPRB, 0.63868E+00_JPRB, 0.67370E+00_JPRB, 0.73479E+00_JPRB ,&
&  0.82168E+00_JPRB, 0.93715E+00_JPRB, 0.10800E+01_JPRB, 0.12372E+01_JPRB, 0.13948E+01_JPRB, 0.15497E+01_JPRB, 0.16238E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.60803E+00_JPRB, 0.13361E+01_JPRB, 0.14867E+01_JPRB, 0.13703E+01_JPRB, 0.12999E+01_JPRB, 0.11694E+01_JPRB ,&
&  0.10473E+01_JPRB, 0.10048E+01_JPRB, 0.94329E+00_JPRB, 0.90108E+00_JPRB, 0.82102E+00_JPRB, 0.80805E+00_JPRB, 0.80644E+00_JPRB ,&
&  0.83066E+00_JPRB, 0.89557E+00_JPRB, 0.10343E+01_JPRB, 0.12666E+01_JPRB, 0.14223E+01_JPRB, 0.14855E+01_JPRB, 0.16340E+01_JPRB ,&
&  0.19201E+01_JPRB, 0.21546E+01_JPRB, 0.22428E+01_JPRB, 0.15983E+01_JPRB, 0.12250E+01_JPRB, 0.11371E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.94704E+00_JPRB, 0.94182E+00_JPRB, 0.95965E+00_JPRB, 0.90247E+00_JPRB, 0.82543E+00_JPRB, 0.81683E+00_JPRB ,&
&  0.75683E+00_JPRB, 0.69177E+00_JPRB, 0.62920E+00_JPRB, 0.59776E+00_JPRB, 0.58031E+00_JPRB, 0.59023E+00_JPRB, 0.58983E+00_JPRB ,&
&  0.59431E+00_JPRB, 0.61560E+00_JPRB, 0.62750E+00_JPRB, 0.64551E+00_JPRB, 0.69217E+00_JPRB, 0.74029E+00_JPRB, 0.77497E+00_JPRB ,&
&  0.79832E+00_JPRB, 0.82585E+00_JPRB, 0.84311E+00_JPRB, 0.85061E+00_JPRB, 0.85031E+00_JPRB, 0.82659E+00_JPRB, 0.82212E+00_JPRB ,&
&  0.81824E+00_JPRB, 0.80398E+00_JPRB, 0.76936E+00_JPRB, 0.74903E+00_JPRB, 0.69284E+00_JPRB, 0.65171E+00_JPRB, 0.58486E+00_JPRB ,&
&  0.55786E+00_JPRB, 0.51769E+00_JPRB, 0.49062E+00_JPRB, 0.48642E+00_JPRB, 0.49919E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.53729E+00_JPRB, 0.53688E+00_JPRB, 0.53798E+00_JPRB, 0.54053E+00_JPRB, 0.53530E+00_JPRB, 0.53648E+00_JPRB, 0.51898E+00_JPRB ,&
&  0.49100E+00_JPRB, 0.47992E+00_JPRB, 0.49439E+00_JPRB, 0.49878E+00_JPRB, 0.50081E+00_JPRB, 0.52081E+00_JPRB, 0.54237E+00_JPRB ,&
&  0.55201E+00_JPRB, 0.55493E+00_JPRB, 0.55326E+00_JPRB, 0.55232E+00_JPRB, 0.54517E+00_JPRB, 0.54815E+00_JPRB, 0.57092E+00_JPRB ,&
&  0.60984E+00_JPRB, 0.68069E+00_JPRB, 0.78639E+00_JPRB, 0.91947E+00_JPRB, 0.10719E+01_JPRB, 0.12852E+01_JPRB, 0.14822E+01_JPRB ,&
&  0.18472E+01_JPRB, 0.12677E+01_JPRB, 0.78394E+00_JPRB, 0.13443E+01_JPRB, 0.15125E+01_JPRB, 0.15019E+01_JPRB, 0.12745E+01_JPRB ,&
&  0.11995E+01_JPRB, 0.11471E+01_JPRB, 0.10947E+01_JPRB, 0.10097E+01_JPRB, 0.87659E+00_JPRB, 0.82162E+00_JPRB, 0.77274E+00_JPRB ,&
&  0.75117E+00_JPRB, 0.77586E+00_JPRB, 0.85574E+00_JPRB, 0.10217E+01_JPRB, 0.12307E+01_JPRB, 0.13382E+01_JPRB, 0.14066E+01_JPRB ,&
&  0.16062E+01_JPRB, 0.18483E+01_JPRB, 0.18969E+01_JPRB, 0.13570E+01_JPRB, 0.10954E+01_JPRB, 0.10333E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.99541E+00_JPRB, 0.10150E+01_JPRB, 0.10385E+01_JPRB, 0.98230E+00_JPRB, 0.91075E+00_JPRB, 0.85682E+00_JPRB, 0.86615E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.77028E+00_JPRB, 0.72371E+00_JPRB, 0.66534E+00_JPRB, 0.60919E+00_JPRB, 0.59989E+00_JPRB, 0.59451E+00_JPRB ,&
&  0.60219E+00_JPRB, 0.62347E+00_JPRB, 0.62947E+00_JPRB, 0.65198E+00_JPRB, 0.67827E+00_JPRB, 0.70997E+00_JPRB, 0.74298E+00_JPRB ,&
&  0.77011E+00_JPRB, 0.80099E+00_JPRB, 0.82958E+00_JPRB, 0.83775E+00_JPRB, 0.82926E+00_JPRB, 0.81377E+00_JPRB, 0.83464E+00_JPRB ,&
&  0.84032E+00_JPRB, 0.84670E+00_JPRB, 0.81441E+00_JPRB, 0.81655E+00_JPRB, 0.76406E+00_JPRB, 0.71647E+00_JPRB, 0.65884E+00_JPRB ,&
&  0.64617E+00_JPRB, 0.60564E+00_JPRB, 0.57120E+00_JPRB, 0.52770E+00_JPRB, 0.52962E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.55805E+00_JPRB, 0.55021E+00_JPRB, 0.54935E+00_JPRB, 0.55066E+00_JPRB, 0.54853E+00_JPRB, 0.55000E+00_JPRB, 0.55147E+00_JPRB ,&
&  0.52499E+00_JPRB, 0.49875E+00_JPRB, 0.48001E+00_JPRB, 0.47053E+00_JPRB, 0.46691E+00_JPRB, 0.48043E+00_JPRB, 0.48695E+00_JPRB ,&
&  0.48471E+00_JPRB, 0.48437E+00_JPRB, 0.50168E+00_JPRB, 0.50635E+00_JPRB, 0.50570E+00_JPRB, 0.50121E+00_JPRB, 0.50131E+00_JPRB ,&
&  0.51396E+00_JPRB, 0.54196E+00_JPRB, 0.59728E+00_JPRB, 0.67497E+00_JPRB, 0.75996E+00_JPRB, 0.90499E+00_JPRB, 0.10842E+01_JPRB ,&
&  0.13913E+01_JPRB, 0.14240E+01_JPRB, 0.70366E+00_JPRB, 0.11771E+01_JPRB, 0.15739E+01_JPRB, 0.16179E+01_JPRB, 0.15040E+01_JPRB ,&
&  0.14007E+01_JPRB, 0.13096E+01_JPRB, 0.13570E+01_JPRB, 0.12593E+01_JPRB, 0.10188E+01_JPRB, 0.87171E+00_JPRB, 0.76895E+00_JPRB ,&
&  0.71067E+00_JPRB, 0.69408E+00_JPRB, 0.71961E+00_JPRB, 0.80519E+00_JPRB, 0.97444E+00_JPRB, 0.11333E+01_JPRB, 0.12134E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.15473E+01_JPRB, 0.17004E+01_JPRB, 0.12647E+01_JPRB, 0.10821E+01_JPRB, 0.11048E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.10772E+01_JPRB, 0.10724E+01_JPRB, 0.10062E+01_JPRB, 0.92714E+00_JPRB, 0.89482E+00_JPRB, 0.90467E+00_JPRB ,&
&  0.85686E+00_JPRB, 0.81592E+00_JPRB, 0.77837E+00_JPRB, 0.73211E+00_JPRB, 0.67268E+00_JPRB, 0.62289E+00_JPRB, 0.62254E+00_JPRB ,&
&  0.62676E+00_JPRB, 0.64357E+00_JPRB, 0.64729E+00_JPRB, 0.66332E+00_JPRB, 0.67863E+00_JPRB, 0.69699E+00_JPRB, 0.71551E+00_JPRB ,&
&  0.74018E+00_JPRB, 0.76564E+00_JPRB, 0.78880E+00_JPRB, 0.78517E+00_JPRB, 0.78260E+00_JPRB, 0.78788E+00_JPRB, 0.79947E+00_JPRB ,&
&  0.82300E+00_JPRB, 0.84506E+00_JPRB, 0.80786E+00_JPRB, 0.81281E+00_JPRB, 0.82818E+00_JPRB, 0.79501E+00_JPRB, 0.75308E+00_JPRB ,&
&  0.78699E+00_JPRB, 0.81999E+00_JPRB, 0.75645E+00_JPRB, 0.60749E+00_JPRB, 0.57589E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.59372E+00_JPRB, 0.58079E+00_JPRB, 0.57116E+00_JPRB, 0.56845E+00_JPRB, 0.56358E+00_JPRB, 0.56050E+00_JPRB, 0.56689E+00_JPRB ,&
&  0.55738E+00_JPRB, 0.54020E+00_JPRB, 0.51697E+00_JPRB, 0.49304E+00_JPRB, 0.47428E+00_JPRB, 0.47560E+00_JPRB, 0.48145E+00_JPRB ,&
&  0.48226E+00_JPRB, 0.47911E+00_JPRB, 0.47847E+00_JPRB, 0.48208E+00_JPRB, 0.48053E+00_JPRB, 0.47736E+00_JPRB, 0.47397E+00_JPRB ,&
&  0.47355E+00_JPRB, 0.48024E+00_JPRB, 0.50243E+00_JPRB, 0.54085E+00_JPRB, 0.58769E+00_JPRB, 0.66322E+00_JPRB, 0.77148E+00_JPRB ,&
&  0.10138E+01_JPRB, 0.11686E+01_JPRB, 0.12653E+01_JPRB, 0.18900E+01_JPRB, 0.19855E+01_JPRB, 0.18535E+01_JPRB, 0.18007E+01_JPRB ,&
&  0.17301E+01_JPRB, 0.16338E+01_JPRB, 0.17451E+01_JPRB, 0.14297E+01_JPRB, 0.11639E+01_JPRB, 0.94376E+00_JPRB, 0.79983E+00_JPRB ,&
&  0.71980E+00_JPRB, 0.66879E+00_JPRB, 0.65684E+00_JPRB, 0.67752E+00_JPRB, 0.76273E+00_JPRB, 0.91092E+00_JPRB, 0.10303E+01_JPRB ,&
&  0.11193E+01_JPRB, 0.12823E+01_JPRB, 0.15149E+01_JPRB, 0.13130E+01_JPRB, 0.11177E+01_JPRB, 0.12390E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.14923E+01_JPRB, 0.12312E+01_JPRB, 0.11268E+01_JPRB, 0.10461E+01_JPRB, 0.94867E+00_JPRB, 0.10039E+01_JPRB, 0.93477E+00_JPRB ,&
&  0.88630E+00_JPRB, 0.84878E+00_JPRB, 0.81129E+00_JPRB, 0.77646E+00_JPRB, 0.73523E+00_JPRB, 0.68853E+00_JPRB, 0.66376E+00_JPRB ,&
&  0.66424E+00_JPRB, 0.67228E+00_JPRB, 0.67277E+00_JPRB, 0.67902E+00_JPRB, 0.68020E+00_JPRB, 0.69257E+00_JPRB, 0.70718E+00_JPRB ,&
&  0.73081E+00_JPRB, 0.74520E+00_JPRB, 0.72333E+00_JPRB, 0.71609E+00_JPRB, 0.71540E+00_JPRB, 0.71606E+00_JPRB, 0.71388E+00_JPRB ,&
&  0.73205E+00_JPRB, 0.77328E+00_JPRB, 0.79560E+00_JPRB, 0.81523E+00_JPRB, 0.82835E+00_JPRB, 0.80473E+00_JPRB, 0.74544E+00_JPRB ,&
&  0.72539E+00_JPRB, 0.72737E+00_JPRB, 0.71702E+00_JPRB, 0.64913E+00_JPRB, 0.61719E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.60842E+00_JPRB, 0.59595E+00_JPRB, 0.58784E+00_JPRB, 0.57900E+00_JPRB, 0.56887E+00_JPRB, 0.56533E+00_JPRB, 0.56908E+00_JPRB ,&
&  0.56709E+00_JPRB, 0.55626E+00_JPRB, 0.54064E+00_JPRB, 0.52490E+00_JPRB, 0.50334E+00_JPRB, 0.49108E+00_JPRB, 0.48380E+00_JPRB ,&
&  0.47854E+00_JPRB, 0.47720E+00_JPRB, 0.47203E+00_JPRB, 0.46915E+00_JPRB, 0.46629E+00_JPRB, 0.45762E+00_JPRB, 0.45289E+00_JPRB ,&
&  0.44981E+00_JPRB, 0.45362E+00_JPRB, 0.46373E+00_JPRB, 0.48430E+00_JPRB, 0.51510E+00_JPRB, 0.56475E+00_JPRB, 0.62759E+00_JPRB ,&
&  0.78444E+00_JPRB, 0.10267E+01_JPRB, 0.99834E+00_JPRB, 0.20133E+01_JPRB, 0.19852E+01_JPRB, 0.19290E+01_JPRB, 0.18056E+01_JPRB ,&
&  0.17908E+01_JPRB, 0.19691E+01_JPRB, 0.17669E+01_JPRB, 0.14927E+01_JPRB, 0.12034E+01_JPRB, 0.99381E+00_JPRB, 0.83719E+00_JPRB ,&
&  0.74049E+00_JPRB, 0.68263E+00_JPRB, 0.65519E+00_JPRB, 0.64341E+00_JPRB, 0.64781E+00_JPRB, 0.72722E+00_JPRB, 0.85820E+00_JPRB ,&
&  0.95989E+00_JPRB, 0.10824E+01_JPRB, 0.13013E+01_JPRB, 0.12461E+01_JPRB, 0.11748E+01_JPRB, 0.13324E+01_JPRB, 0.16276E+01_JPRB ,&
&  0.19093E+01_JPRB, 0.15687E+01_JPRB, 0.12610E+01_JPRB, 0.11244E+01_JPRB, 0.10434E+01_JPRB, 0.10297E+01_JPRB, 0.96707E+00_JPRB ,&
&  0.89994E+00_JPRB, 0.86277E+00_JPRB, 0.83211E+00_JPRB, 0.80433E+00_JPRB, 0.77129E+00_JPRB, 0.73934E+00_JPRB, 0.70483E+00_JPRB ,&
&  0.69320E+00_JPRB, 0.69136E+00_JPRB, 0.69122E+00_JPRB, 0.69878E+00_JPRB, 0.69856E+00_JPRB, 0.70554E+00_JPRB, 0.71411E+00_JPRB ,&
&  0.73114E+00_JPRB, 0.73768E+00_JPRB, 0.69858E+00_JPRB, 0.67409E+00_JPRB, 0.66895E+00_JPRB, 0.66416E+00_JPRB, 0.66415E+00_JPRB ,&
&  0.69282E+00_JPRB, 0.73024E+00_JPRB, 0.75658E+00_JPRB, 0.77090E+00_JPRB, 0.82671E+00_JPRB, 0.84073E+00_JPRB, 0.75902E+00_JPRB ,&
&  0.72227E+00_JPRB, 0.69458E+00_JPRB, 0.67042E+00_JPRB, 0.63956E+00_JPRB, 0.62474E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.61196E+00_JPRB, 0.60217E+00_JPRB, 0.59344E+00_JPRB, 0.58217E+00_JPRB, 0.56717E+00_JPRB, 0.55835E+00_JPRB, 0.55960E+00_JPRB ,&
&  0.56367E+00_JPRB, 0.55707E+00_JPRB, 0.54729E+00_JPRB, 0.53291E+00_JPRB, 0.51241E+00_JPRB, 0.50106E+00_JPRB, 0.49281E+00_JPRB ,&
&  0.48062E+00_JPRB, 0.47076E+00_JPRB, 0.45705E+00_JPRB, 0.45062E+00_JPRB, 0.44903E+00_JPRB, 0.44312E+00_JPRB, 0.43189E+00_JPRB ,&
&  0.43381E+00_JPRB, 0.44185E+00_JPRB, 0.45207E+00_JPRB, 0.46351E+00_JPRB, 0.48647E+00_JPRB, 0.52503E+00_JPRB, 0.58338E+00_JPRB ,&
&  0.69985E+00_JPRB, 0.11566E+01_JPRB, 0.92606E+00_JPRB, 0.15714E+01_JPRB, 0.16085E+01_JPRB, 0.17145E+01_JPRB, 0.17424E+01_JPRB ,&
&  0.17809E+01_JPRB, 0.18172E+01_JPRB, 0.17004E+01_JPRB, 0.14138E+01_JPRB, 0.11351E+01_JPRB, 0.95821E+00_JPRB, 0.84679E+00_JPRB ,&
&  0.77342E+00_JPRB, 0.71765E+00_JPRB, 0.67771E+00_JPRB, 0.65079E+00_JPRB, 0.63182E+00_JPRB, 0.63577E+00_JPRB, 0.70064E+00_JPRB ,&
&  0.81385E+00_JPRB, 0.91991E+00_JPRB, 0.10883E+01_JPRB, 0.12605E+01_JPRB, 0.11691E+01_JPRB, 0.12761E+01_JPRB, 0.17047E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.20508E+01_JPRB, 0.15027E+01_JPRB, 0.12600E+01_JPRB, 0.11584E+01_JPRB, 0.10904E+01_JPRB, 0.98962E+00_JPRB ,&
&  0.89810E+00_JPRB, 0.85371E+00_JPRB, 0.82814E+00_JPRB, 0.80904E+00_JPRB, 0.78584E+00_JPRB, 0.75291E+00_JPRB, 0.71932E+00_JPRB ,&
&  0.69137E+00_JPRB, 0.68771E+00_JPRB, 0.69043E+00_JPRB, 0.69471E+00_JPRB, 0.70191E+00_JPRB, 0.70815E+00_JPRB, 0.71529E+00_JPRB ,&
&  0.72331E+00_JPRB, 0.73158E+00_JPRB, 0.70110E+00_JPRB, 0.67250E+00_JPRB, 0.65891E+00_JPRB, 0.64939E+00_JPRB, 0.64368E+00_JPRB ,&
&  0.66523E+00_JPRB, 0.69197E+00_JPRB, 0.72128E+00_JPRB, 0.76985E+00_JPRB, 0.83078E+00_JPRB, 0.89023E+00_JPRB, 0.74691E+00_JPRB ,&
&  0.73068E+00_JPRB, 0.70052E+00_JPRB, 0.67289E+00_JPRB, 0.64285E+00_JPRB, 0.62732E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.61085E+00_JPRB, 0.60176E+00_JPRB, 0.59276E+00_JPRB, 0.58297E+00_JPRB, 0.57101E+00_JPRB, 0.55997E+00_JPRB, 0.55358E+00_JPRB ,&
&  0.55140E+00_JPRB, 0.55228E+00_JPRB, 0.54678E+00_JPRB, 0.53172E+00_JPRB, 0.51439E+00_JPRB, 0.49834E+00_JPRB, 0.48906E+00_JPRB ,&
&  0.48051E+00_JPRB, 0.46899E+00_JPRB, 0.45491E+00_JPRB, 0.44187E+00_JPRB, 0.43786E+00_JPRB, 0.43883E+00_JPRB, 0.43285E+00_JPRB ,&
&  0.43842E+00_JPRB, 0.44623E+00_JPRB, 0.45364E+00_JPRB, 0.46104E+00_JPRB, 0.48019E+00_JPRB, 0.51407E+00_JPRB, 0.56857E+00_JPRB ,&
&  0.67340E+00_JPRB, 0.58551E+00_JPRB, 0.10214E+01_JPRB, 0.12095E+01_JPRB, 0.13111E+01_JPRB, 0.14046E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.17568E+01_JPRB, 0.17003E+01_JPRB, 0.15369E+01_JPRB, 0.13051E+01_JPRB, 0.10957E+01_JPRB, 0.94302E+00_JPRB, 0.84351E+00_JPRB ,&
&  0.78143E+00_JPRB, 0.73414E+00_JPRB, 0.69232E+00_JPRB, 0.65665E+00_JPRB, 0.63264E+00_JPRB, 0.61947E+00_JPRB, 0.62469E+00_JPRB ,&
&  0.66795E+00_JPRB, 0.75143E+00_JPRB, 0.86537E+00_JPRB, 0.10658E+01_JPRB, 0.11524E+01_JPRB, 0.12157E+01_JPRB, 0.13778E+01_JPRB ,&
&  0.17639E+01_JPRB, 0.20067E+01_JPRB, 0.16787E+01_JPRB, 0.14032E+01_JPRB, 0.12606E+01_JPRB, 0.11189E+01_JPRB, 0.99290E+00_JPRB ,&
&  0.89666E+00_JPRB, 0.83947E+00_JPRB, 0.80294E+00_JPRB, 0.78138E+00_JPRB, 0.77021E+00_JPRB, 0.75430E+00_JPRB, 0.73016E+00_JPRB ,&
&  0.70923E+00_JPRB, 0.69636E+00_JPRB, 0.69268E+00_JPRB, 0.69454E+00_JPRB, 0.69698E+00_JPRB, 0.70306E+00_JPRB, 0.70507E+00_JPRB ,&
&  0.70318E+00_JPRB, 0.70417E+00_JPRB, 0.70189E+00_JPRB, 0.67780E+00_JPRB, 0.66571E+00_JPRB, 0.65128E+00_JPRB, 0.64058E+00_JPRB ,&
&  0.65601E+00_JPRB, 0.66732E+00_JPRB, 0.68950E+00_JPRB, 0.74983E+00_JPRB, 0.79230E+00_JPRB, 0.82888E+00_JPRB, 0.73494E+00_JPRB ,&
&  0.71224E+00_JPRB, 0.68442E+00_JPRB, 0.66007E+00_JPRB, 0.63619E+00_JPRB, 0.62566E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.61374E+00_JPRB, 0.60439E+00_JPRB, 0.59294E+00_JPRB, 0.58195E+00_JPRB, 0.56915E+00_JPRB, 0.55633E+00_JPRB, 0.54733E+00_JPRB ,&
&  0.54189E+00_JPRB, 0.54070E+00_JPRB, 0.54092E+00_JPRB, 0.52970E+00_JPRB, 0.51465E+00_JPRB, 0.50068E+00_JPRB, 0.49196E+00_JPRB ,&
&  0.48482E+00_JPRB, 0.47631E+00_JPRB, 0.46606E+00_JPRB, 0.45413E+00_JPRB, 0.44830E+00_JPRB, 0.44518E+00_JPRB, 0.44314E+00_JPRB ,&
&  0.44822E+00_JPRB, 0.45284E+00_JPRB, 0.45908E+00_JPRB, 0.46793E+00_JPRB, 0.48521E+00_JPRB, 0.51621E+00_JPRB, 0.56735E+00_JPRB ,&
&  0.67325E+00_JPRB, 0.91301E+00_JPRB, 0.11450E+01_JPRB, 0.10654E+01_JPRB, 0.10725E+01_JPRB, 0.11113E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.14290E+01_JPRB, 0.14709E+01_JPRB, 0.13479E+01_JPRB, 0.11835E+01_JPRB, 0.10383E+01_JPRB, 0.92465E+00_JPRB, 0.84300E+00_JPRB ,&
&  0.76877E+00_JPRB, 0.70385E+00_JPRB, 0.65741E+00_JPRB, 0.62504E+00_JPRB, 0.60398E+00_JPRB, 0.59113E+00_JPRB, 0.58640E+00_JPRB ,&
&  0.59457E+00_JPRB, 0.62495E+00_JPRB, 0.69387E+00_JPRB, 0.83175E+00_JPRB, 0.11088E+01_JPRB, 0.11029E+01_JPRB, 0.13051E+01_JPRB ,&
&  0.16950E+01_JPRB, 0.16722E+01_JPRB, 0.15502E+01_JPRB, 0.13918E+01_JPRB, 0.12436E+01_JPRB, 0.11086E+01_JPRB, 0.99141E+00_JPRB ,&
&  0.91201E+00_JPRB, 0.85342E+00_JPRB, 0.80081E+00_JPRB, 0.76289E+00_JPRB, 0.73990E+00_JPRB, 0.73158E+00_JPRB, 0.72537E+00_JPRB ,&
&  0.71606E+00_JPRB, 0.70498E+00_JPRB, 0.69720E+00_JPRB, 0.69434E+00_JPRB, 0.69197E+00_JPRB, 0.69115E+00_JPRB, 0.67900E+00_JPRB ,&
&  0.66747E+00_JPRB, 0.66218E+00_JPRB, 0.69608E+00_JPRB, 0.67092E+00_JPRB, 0.66091E+00_JPRB, 0.65079E+00_JPRB, 0.63548E+00_JPRB ,&
&  0.62578E+00_JPRB, 0.65043E+00_JPRB, 0.67349E+00_JPRB, 0.72543E+00_JPRB, 0.83543E+00_JPRB, 0.85730E+00_JPRB, 0.74948E+00_JPRB ,&
&  0.70969E+00_JPRB, 0.67096E+00_JPRB, 0.63817E+00_JPRB, 0.62264E+00_JPRB, 0.62568E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.61416E+00_JPRB, 0.60100E+00_JPRB, 0.58795E+00_JPRB, 0.57629E+00_JPRB, 0.56178E+00_JPRB, 0.54888E+00_JPRB, 0.53690E+00_JPRB ,&
&  0.52815E+00_JPRB, 0.52348E+00_JPRB, 0.52382E+00_JPRB, 0.52460E+00_JPRB, 0.52206E+00_JPRB, 0.51377E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.48925E+00_JPRB, 0.48047E+00_JPRB, 0.47009E+00_JPRB, 0.45772E+00_JPRB, 0.44597E+00_JPRB, 0.44075E+00_JPRB, 0.44039E+00_JPRB ,&
&  0.44461E+00_JPRB, 0.45010E+00_JPRB, 0.45405E+00_JPRB, 0.46572E+00_JPRB, 0.48397E+00_JPRB, 0.51445E+00_JPRB, 0.55831E+00_JPRB ,&
&  0.63560E+00_JPRB, 0.88826E+00_JPRB, 0.94781E+00_JPRB, 0.93367E+00_JPRB, 0.94307E+00_JPRB, 0.96048E+00_JPRB, 0.10555E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.12072E+01_JPRB, 0.11722E+01_JPRB, 0.10503E+01_JPRB, 0.94785E+00_JPRB, 0.87586E+00_JPRB, 0.80567E+00_JPRB ,&
&  0.73844E+00_JPRB, 0.68532E+00_JPRB, 0.64333E+00_JPRB, 0.61480E+00_JPRB, 0.59330E+00_JPRB, 0.57733E+00_JPRB, 0.56860E+00_JPRB ,&
&  0.56744E+00_JPRB, 0.57300E+00_JPRB, 0.59497E+00_JPRB, 0.64608E+00_JPRB, 0.85004E+00_JPRB, 0.10264E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.12410E+01_JPRB, 0.12786E+01_JPRB, 0.12933E+01_JPRB, 0.12359E+01_JPRB, 0.11436E+01_JPRB, 0.10322E+01_JPRB, 0.94741E+00_JPRB ,&
&  0.89373E+00_JPRB, 0.83896E+00_JPRB, 0.78594E+00_JPRB, 0.73739E+00_JPRB, 0.70387E+00_JPRB, 0.68691E+00_JPRB, 0.67830E+00_JPRB ,&
&  0.66949E+00_JPRB, 0.66509E+00_JPRB, 0.66219E+00_JPRB, 0.65805E+00_JPRB, 0.65213E+00_JPRB, 0.64365E+00_JPRB, 0.63170E+00_JPRB ,&
&  0.61784E+00_JPRB, 0.60591E+00_JPRB, 0.61636E+00_JPRB, 0.61775E+00_JPRB, 0.61621E+00_JPRB, 0.60975E+00_JPRB, 0.60046E+00_JPRB ,&
&  0.60420E+00_JPRB, 0.66714E+00_JPRB, 0.68096E+00_JPRB, 0.74146E+00_JPRB, 0.80984E+00_JPRB, 0.80361E+00_JPRB, 0.74223E+00_JPRB ,&
&  0.69376E+00_JPRB, 0.65184E+00_JPRB, 0.61872E+00_JPRB, 0.60526E+00_JPRB, 0.62204E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.62247E+00_JPRB, 0.60311E+00_JPRB, 0.58351E+00_JPRB, 0.56162E+00_JPRB, 0.54280E+00_JPRB, 0.52964E+00_JPRB, 0.52009E+00_JPRB ,&
&  0.51400E+00_JPRB, 0.51042E+00_JPRB, 0.50784E+00_JPRB, 0.50550E+00_JPRB, 0.50263E+00_JPRB, 0.49923E+00_JPRB, 0.49162E+00_JPRB ,&
&  0.48199E+00_JPRB, 0.47060E+00_JPRB, 0.46155E+00_JPRB, 0.45379E+00_JPRB, 0.43998E+00_JPRB, 0.43141E+00_JPRB, 0.43229E+00_JPRB ,&
&  0.43666E+00_JPRB, 0.44209E+00_JPRB, 0.44664E+00_JPRB, 0.45723E+00_JPRB, 0.47910E+00_JPRB, 0.50938E+00_JPRB, 0.54567E+00_JPRB ,&
&  0.61845E+00_JPRB, 0.67635E+00_JPRB, 0.74759E+00_JPRB, 0.76447E+00_JPRB, 0.77187E+00_JPRB, 0.81148E+00_JPRB, 0.86987E+00_JPRB ,&
&  0.90637E+00_JPRB, 0.97434E+00_JPRB, 0.96501E+00_JPRB, 0.88239E+00_JPRB, 0.79836E+00_JPRB, 0.73868E+00_JPRB, 0.70114E+00_JPRB ,&
&  0.67278E+00_JPRB, 0.64775E+00_JPRB, 0.62657E+00_JPRB, 0.61218E+00_JPRB, 0.59840E+00_JPRB, 0.58461E+00_JPRB, 0.57109E+00_JPRB ,&
&  0.56280E+00_JPRB, 0.55777E+00_JPRB, 0.55818E+00_JPRB, 0.57218E+00_JPRB, 0.62475E+00_JPRB, 0.71651E+00_JPRB, 0.84146E+00_JPRB ,&
&  0.94508E+00_JPRB, 0.10052E+01_JPRB, 0.10577E+01_JPRB, 0.10550E+01_JPRB, 0.10142E+01_JPRB, 0.96577E+00_JPRB, 0.91327E+00_JPRB ,&
&  0.84505E+00_JPRB, 0.76822E+00_JPRB, 0.71127E+00_JPRB, 0.67173E+00_JPRB, 0.64572E+00_JPRB, 0.63057E+00_JPRB, 0.62237E+00_JPRB ,&
&  0.61625E+00_JPRB, 0.61563E+00_JPRB, 0.62595E+00_JPRB, 0.63715E+00_JPRB, 0.63643E+00_JPRB, 0.62184E+00_JPRB, 0.60059E+00_JPRB ,&
&  0.58176E+00_JPRB, 0.56695E+00_JPRB, 0.56265E+00_JPRB, 0.55966E+00_JPRB, 0.55746E+00_JPRB, 0.55683E+00_JPRB, 0.55407E+00_JPRB ,&
&  0.55445E+00_JPRB, 0.59797E+00_JPRB, 0.64212E+00_JPRB, 0.76493E+00_JPRB, 0.82052E+00_JPRB, 0.76947E+00_JPRB, 0.71112E+00_JPRB ,&
&  0.66547E+00_JPRB, 0.63430E+00_JPRB, 0.60440E+00_JPRB, 0.59073E+00_JPRB, 0.64208E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.55235E+00_JPRB, 0.54719E+00_JPRB, 0.53434E+00_JPRB, 0.52063E+00_JPRB, 0.50850E+00_JPRB, 0.50088E+00_JPRB, 0.49452E+00_JPRB ,&
&  0.49022E+00_JPRB, 0.48710E+00_JPRB, 0.48494E+00_JPRB, 0.48192E+00_JPRB, 0.47738E+00_JPRB, 0.47059E+00_JPRB, 0.46187E+00_JPRB ,&
&  0.45379E+00_JPRB, 0.44583E+00_JPRB, 0.43909E+00_JPRB, 0.43166E+00_JPRB, 0.42242E+00_JPRB, 0.41738E+00_JPRB, 0.41952E+00_JPRB ,&
&  0.42513E+00_JPRB, 0.43199E+00_JPRB, 0.43841E+00_JPRB, 0.44834E+00_JPRB, 0.47338E+00_JPRB, 0.50282E+00_JPRB, 0.53010E+00_JPRB ,&
&  0.59229E+00_JPRB, 0.58180E+00_JPRB, 0.61430E+00_JPRB, 0.65481E+00_JPRB, 0.69275E+00_JPRB, 0.71017E+00_JPRB, 0.73078E+00_JPRB ,&
&  0.75384E+00_JPRB, 0.77630E+00_JPRB, 0.77631E+00_JPRB, 0.74239E+00_JPRB, 0.67887E+00_JPRB, 0.62028E+00_JPRB, 0.58599E+00_JPRB ,&
&  0.56889E+00_JPRB, 0.56124E+00_JPRB, 0.55434E+00_JPRB, 0.54811E+00_JPRB, 0.54358E+00_JPRB, 0.53926E+00_JPRB, 0.53462E+00_JPRB ,&
&  0.53026E+00_JPRB, 0.52689E+00_JPRB, 0.52522E+00_JPRB, 0.52636E+00_JPRB, 0.53734E+00_JPRB, 0.57053E+00_JPRB, 0.63697E+00_JPRB ,&
&  0.73150E+00_JPRB, 0.81939E+00_JPRB, 0.86398E+00_JPRB, 0.89027E+00_JPRB, 0.90941E+00_JPRB, 0.88023E+00_JPRB, 0.79778E+00_JPRB ,&
&  0.72164E+00_JPRB, 0.65926E+00_JPRB, 0.61554E+00_JPRB, 0.58518E+00_JPRB, 0.56510E+00_JPRB, 0.55371E+00_JPRB, 0.54863E+00_JPRB ,&
&  0.54688E+00_JPRB, 0.54682E+00_JPRB, 0.54994E+00_JPRB, 0.55249E+00_JPRB, 0.55383E+00_JPRB, 0.55120E+00_JPRB, 0.54368E+00_JPRB ,&
&  0.53298E+00_JPRB, 0.52550E+00_JPRB, 0.52105E+00_JPRB, 0.51690E+00_JPRB, 0.51247E+00_JPRB, 0.50743E+00_JPRB, 0.50353E+00_JPRB ,&
&  0.50242E+00_JPRB, 0.52330E+00_JPRB, 0.55788E+00_JPRB, 0.64626E+00_JPRB, 0.64189E+00_JPRB, 0.62672E+00_JPRB, 0.60380E+00_JPRB ,&
&  0.58771E+00_JPRB, 0.57800E+00_JPRB, 0.56129E+00_JPRB, 0.55294E+00_JPRB, 0.55418E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.51943E+00_JPRB, 0.51231E+00_JPRB, 0.50017E+00_JPRB, 0.48755E+00_JPRB, 0.47990E+00_JPRB, 0.47543E+00_JPRB, 0.47238E+00_JPRB ,&
&  0.46835E+00_JPRB, 0.46320E+00_JPRB, 0.45770E+00_JPRB, 0.45419E+00_JPRB, 0.44989E+00_JPRB, 0.44384E+00_JPRB, 0.43646E+00_JPRB ,&
&  0.42956E+00_JPRB, 0.42311E+00_JPRB, 0.41808E+00_JPRB, 0.41256E+00_JPRB, 0.40821E+00_JPRB, 0.40788E+00_JPRB, 0.40923E+00_JPRB ,&
&  0.41488E+00_JPRB, 0.42250E+00_JPRB, 0.42934E+00_JPRB, 0.44407E+00_JPRB, 0.46180E+00_JPRB, 0.48789E+00_JPRB, 0.51446E+00_JPRB ,&
&  0.56741E+00_JPRB, 0.50486E+00_JPRB, 0.53123E+00_JPRB, 0.59131E+00_JPRB, 0.62812E+00_JPRB, 0.64727E+00_JPRB, 0.63907E+00_JPRB ,&
&  0.63350E+00_JPRB, 0.63514E+00_JPRB, 0.63424E+00_JPRB, 0.62668E+00_JPRB, 0.60388E+00_JPRB, 0.56134E+00_JPRB, 0.52008E+00_JPRB ,&
&  0.49201E+00_JPRB, 0.48225E+00_JPRB, 0.47795E+00_JPRB, 0.47182E+00_JPRB, 0.46566E+00_JPRB, 0.46284E+00_JPRB, 0.46335E+00_JPRB ,&
&  0.46369E+00_JPRB, 0.46342E+00_JPRB, 0.46232E+00_JPRB, 0.46119E+00_JPRB, 0.46269E+00_JPRB, 0.47404E+00_JPRB, 0.50499E+00_JPRB ,&
&  0.56206E+00_JPRB, 0.62563E+00_JPRB, 0.68193E+00_JPRB, 0.72497E+00_JPRB, 0.73231E+00_JPRB, 0.71265E+00_JPRB, 0.67570E+00_JPRB ,&
&  0.63302E+00_JPRB, 0.59498E+00_JPRB, 0.56640E+00_JPRB, 0.54416E+00_JPRB, 0.52634E+00_JPRB, 0.51232E+00_JPRB, 0.50241E+00_JPRB ,&
&  0.49700E+00_JPRB, 0.49361E+00_JPRB, 0.49093E+00_JPRB, 0.48709E+00_JPRB, 0.48377E+00_JPRB, 0.48116E+00_JPRB, 0.47992E+00_JPRB ,&
&  0.47796E+00_JPRB, 0.47563E+00_JPRB, 0.47517E+00_JPRB, 0.47653E+00_JPRB, 0.47693E+00_JPRB, 0.47548E+00_JPRB, 0.47404E+00_JPRB ,&
&  0.47501E+00_JPRB, 0.48424E+00_JPRB, 0.49409E+00_JPRB, 0.56784E+00_JPRB, 0.58584E+00_JPRB, 0.57141E+00_JPRB, 0.55533E+00_JPRB ,&
&  0.54615E+00_JPRB, 0.53415E+00_JPRB, 0.52902E+00_JPRB, 0.49102E+00_JPRB, 0.51958E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.47559E+00_JPRB, 0.47477E+00_JPRB, 0.46948E+00_JPRB, 0.45948E+00_JPRB, 0.45137E+00_JPRB, 0.44529E+00_JPRB, 0.44137E+00_JPRB ,&
&  0.43927E+00_JPRB, 0.43724E+00_JPRB, 0.43423E+00_JPRB, 0.42898E+00_JPRB, 0.42448E+00_JPRB, 0.41900E+00_JPRB, 0.41334E+00_JPRB ,&
&  0.40853E+00_JPRB, 0.40409E+00_JPRB, 0.40143E+00_JPRB, 0.39807E+00_JPRB, 0.39658E+00_JPRB, 0.39772E+00_JPRB, 0.39944E+00_JPRB ,&
&  0.40210E+00_JPRB, 0.40466E+00_JPRB, 0.41325E+00_JPRB, 0.42491E+00_JPRB, 0.44584E+00_JPRB, 0.46016E+00_JPRB, 0.48248E+00_JPRB ,&
&  0.53269E+00_JPRB, 0.45598E+00_JPRB, 0.49136E+00_JPRB, 0.53632E+00_JPRB, 0.56587E+00_JPRB, 0.58733E+00_JPRB, 0.56517E+00_JPRB ,&
&  0.54778E+00_JPRB, 0.53663E+00_JPRB, 0.52840E+00_JPRB, 0.52256E+00_JPRB, 0.51839E+00_JPRB, 0.50586E+00_JPRB, 0.48976E+00_JPRB ,&
&  0.47198E+00_JPRB, 0.45852E+00_JPRB, 0.44761E+00_JPRB, 0.43688E+00_JPRB, 0.42753E+00_JPRB, 0.42162E+00_JPRB, 0.41888E+00_JPRB ,&
&  0.41855E+00_JPRB, 0.41890E+00_JPRB, 0.41894E+00_JPRB, 0.41973E+00_JPRB, 0.42211E+00_JPRB, 0.42672E+00_JPRB, 0.43483E+00_JPRB ,&
&  0.45433E+00_JPRB, 0.48556E+00_JPRB, 0.51867E+00_JPRB, 0.54926E+00_JPRB, 0.57187E+00_JPRB, 0.57909E+00_JPRB, 0.57736E+00_JPRB ,&
&  0.56527E+00_JPRB, 0.54714E+00_JPRB, 0.52786E+00_JPRB, 0.50998E+00_JPRB, 0.49429E+00_JPRB, 0.48085E+00_JPRB, 0.46836E+00_JPRB ,&
&  0.45788E+00_JPRB, 0.44875E+00_JPRB, 0.44212E+00_JPRB, 0.43748E+00_JPRB, 0.43307E+00_JPRB, 0.42914E+00_JPRB, 0.42704E+00_JPRB ,&
&  0.42562E+00_JPRB, 0.42478E+00_JPRB, 0.42478E+00_JPRB, 0.42733E+00_JPRB, 0.43145E+00_JPRB, 0.43483E+00_JPRB, 0.43667E+00_JPRB ,&
&  0.43700E+00_JPRB, 0.44311E+00_JPRB, 0.44862E+00_JPRB, 0.47959E+00_JPRB, 0.49417E+00_JPRB, 0.49708E+00_JPRB, 0.49578E+00_JPRB ,&
&  0.49442E+00_JPRB, 0.48628E+00_JPRB, 0.43161E+00_JPRB, 0.45911E+00_JPRB, 0.47001E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.44062E+00_JPRB, 0.43784E+00_JPRB, 0.43657E+00_JPRB, 0.43413E+00_JPRB, 0.43089E+00_JPRB, 0.42723E+00_JPRB, 0.42294E+00_JPRB ,&
&  0.41812E+00_JPRB, 0.41510E+00_JPRB, 0.41235E+00_JPRB, 0.40900E+00_JPRB, 0.40557E+00_JPRB, 0.40279E+00_JPRB, 0.40007E+00_JPRB ,&
&  0.39706E+00_JPRB, 0.39364E+00_JPRB, 0.39118E+00_JPRB, 0.38942E+00_JPRB, 0.38750E+00_JPRB, 0.38670E+00_JPRB, 0.38755E+00_JPRB ,&
&  0.38742E+00_JPRB, 0.38978E+00_JPRB, 0.39993E+00_JPRB, 0.41242E+00_JPRB, 0.42813E+00_JPRB, 0.44221E+00_JPRB, 0.45850E+00_JPRB ,&
&  0.44784E+00_JPRB, 0.40397E+00_JPRB, 0.43276E+00_JPRB, 0.46782E+00_JPRB, 0.50354E+00_JPRB, 0.53008E+00_JPRB, 0.51461E+00_JPRB ,&
&  0.49375E+00_JPRB, 0.47416E+00_JPRB, 0.45994E+00_JPRB, 0.45049E+00_JPRB, 0.44881E+00_JPRB, 0.44728E+00_JPRB, 0.44493E+00_JPRB ,&
&  0.44034E+00_JPRB, 0.43192E+00_JPRB, 0.42287E+00_JPRB, 0.41450E+00_JPRB, 0.40855E+00_JPRB, 0.40381E+00_JPRB, 0.39982E+00_JPRB ,&
&  0.39797E+00_JPRB, 0.39766E+00_JPRB, 0.39771E+00_JPRB, 0.39810E+00_JPRB, 0.39923E+00_JPRB, 0.40246E+00_JPRB, 0.40647E+00_JPRB ,&
&  0.41107E+00_JPRB, 0.41922E+00_JPRB, 0.43356E+00_JPRB, 0.45089E+00_JPRB, 0.46800E+00_JPRB, 0.48013E+00_JPRB, 0.48638E+00_JPRB ,&
&  0.48861E+00_JPRB, 0.48669E+00_JPRB, 0.48038E+00_JPRB, 0.47012E+00_JPRB, 0.45755E+00_JPRB, 0.44579E+00_JPRB, 0.43502E+00_JPRB ,&
&  0.42502E+00_JPRB, 0.41472E+00_JPRB, 0.40534E+00_JPRB, 0.39992E+00_JPRB, 0.39738E+00_JPRB, 0.39488E+00_JPRB, 0.39270E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.39097E+00_JPRB, 0.39088E+00_JPRB, 0.39170E+00_JPRB, 0.39380E+00_JPRB, 0.39732E+00_JPRB, 0.40086E+00_JPRB ,&
&  0.40356E+00_JPRB, 0.41040E+00_JPRB, 0.41926E+00_JPRB, 0.44184E+00_JPRB, 0.45632E+00_JPRB, 0.46163E+00_JPRB, 0.46123E+00_JPRB ,&
&  0.45695E+00_JPRB, 0.45140E+00_JPRB, 0.44644E+00_JPRB, 0.44159E+00_JPRB, 0.44094E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.41398E+00_JPRB, 0.41218E+00_JPRB, 0.41064E+00_JPRB, 0.40845E+00_JPRB, 0.40641E+00_JPRB, 0.40462E+00_JPRB, 0.40109E+00_JPRB ,&
&  0.39972E+00_JPRB, 0.39600E+00_JPRB, 0.39255E+00_JPRB, 0.39119E+00_JPRB, 0.38955E+00_JPRB, 0.38717E+00_JPRB, 0.38527E+00_JPRB ,&
&  0.38299E+00_JPRB, 0.38030E+00_JPRB, 0.37833E+00_JPRB, 0.37575E+00_JPRB, 0.37201E+00_JPRB, 0.36931E+00_JPRB, 0.36964E+00_JPRB ,&
&  0.37039E+00_JPRB, 0.37288E+00_JPRB, 0.38124E+00_JPRB, 0.38976E+00_JPRB, 0.39951E+00_JPRB, 0.40914E+00_JPRB, 0.41800E+00_JPRB ,&
&  0.43238E+00_JPRB, 0.38650E+00_JPRB, 0.41272E+00_JPRB, 0.43259E+00_JPRB, 0.46261E+00_JPRB, 0.48578E+00_JPRB, 0.47718E+00_JPRB ,&
&  0.45638E+00_JPRB, 0.43348E+00_JPRB, 0.41747E+00_JPRB, 0.40833E+00_JPRB, 0.40881E+00_JPRB, 0.41086E+00_JPRB, 0.40955E+00_JPRB ,&
&  0.40613E+00_JPRB, 0.39860E+00_JPRB, 0.39011E+00_JPRB, 0.38458E+00_JPRB, 0.38158E+00_JPRB, 0.37926E+00_JPRB, 0.37686E+00_JPRB ,&
&  0.37538E+00_JPRB, 0.37502E+00_JPRB, 0.37488E+00_JPRB, 0.37481E+00_JPRB, 0.37552E+00_JPRB, 0.37736E+00_JPRB, 0.37984E+00_JPRB ,&
&  0.38185E+00_JPRB, 0.38272E+00_JPRB, 0.38466E+00_JPRB, 0.38916E+00_JPRB, 0.39570E+00_JPRB, 0.40242E+00_JPRB, 0.40733E+00_JPRB ,&
&  0.40992E+00_JPRB, 0.41148E+00_JPRB, 0.41169E+00_JPRB, 0.40947E+00_JPRB, 0.40466E+00_JPRB, 0.39800E+00_JPRB, 0.39150E+00_JPRB ,&
&  0.38534E+00_JPRB, 0.37950E+00_JPRB, 0.37240E+00_JPRB, 0.36884E+00_JPRB, 0.36780E+00_JPRB, 0.36740E+00_JPRB, 0.36731E+00_JPRB ,&
&  0.36739E+00_JPRB, 0.36701E+00_JPRB, 0.36665E+00_JPRB, 0.36647E+00_JPRB, 0.36671E+00_JPRB, 0.36780E+00_JPRB, 0.37027E+00_JPRB ,&
&  0.37407E+00_JPRB, 0.38109E+00_JPRB, 0.39052E+00_JPRB, 0.40765E+00_JPRB, 0.42274E+00_JPRB, 0.43012E+00_JPRB, 0.43243E+00_JPRB ,&
&  0.42934E+00_JPRB, 0.42465E+00_JPRB, 0.41990E+00_JPRB, 0.41551E+00_JPRB, 0.41522E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.39577E+00_JPRB, 0.39213E+00_JPRB, 0.38949E+00_JPRB, 0.38557E+00_JPRB, 0.38284E+00_JPRB, 0.38089E+00_JPRB, 0.37871E+00_JPRB ,&
&  0.37710E+00_JPRB, 0.37745E+00_JPRB, 0.37635E+00_JPRB, 0.37578E+00_JPRB, 0.37482E+00_JPRB, 0.37200E+00_JPRB, 0.36845E+00_JPRB ,&
&  0.36642E+00_JPRB, 0.36457E+00_JPRB, 0.36288E+00_JPRB, 0.36091E+00_JPRB, 0.35889E+00_JPRB, 0.35621E+00_JPRB, 0.35610E+00_JPRB ,&
&  0.35700E+00_JPRB, 0.35962E+00_JPRB, 0.36305E+00_JPRB, 0.36706E+00_JPRB, 0.37339E+00_JPRB, 0.38214E+00_JPRB, 0.39054E+00_JPRB ,&
&  0.40020E+00_JPRB, 0.38862E+00_JPRB, 0.40483E+00_JPRB, 0.41427E+00_JPRB, 0.43126E+00_JPRB, 0.44527E+00_JPRB, 0.44474E+00_JPRB ,&
&  0.43602E+00_JPRB, 0.41971E+00_JPRB, 0.40567E+00_JPRB, 0.39705E+00_JPRB, 0.39515E+00_JPRB, 0.39353E+00_JPRB, 0.38775E+00_JPRB ,&
&  0.37990E+00_JPRB, 0.37061E+00_JPRB, 0.36346E+00_JPRB, 0.35953E+00_JPRB, 0.35804E+00_JPRB, 0.35759E+00_JPRB, 0.35602E+00_JPRB ,&
&  0.35436E+00_JPRB, 0.35270E+00_JPRB, 0.35187E+00_JPRB, 0.35146E+00_JPRB, 0.35103E+00_JPRB, 0.35125E+00_JPRB, 0.35213E+00_JPRB ,&
&  0.35313E+00_JPRB, 0.35382E+00_JPRB, 0.35420E+00_JPRB, 0.35519E+00_JPRB, 0.35661E+00_JPRB, 0.35813E+00_JPRB, 0.35955E+00_JPRB ,&
&  0.35990E+00_JPRB, 0.36018E+00_JPRB, 0.36000E+00_JPRB, 0.35981E+00_JPRB, 0.35814E+00_JPRB, 0.35480E+00_JPRB, 0.35093E+00_JPRB ,&
&  0.34800E+00_JPRB, 0.34596E+00_JPRB, 0.34432E+00_JPRB, 0.34319E+00_JPRB, 0.34292E+00_JPRB, 0.34359E+00_JPRB, 0.34462E+00_JPRB ,&
&  0.34571E+00_JPRB, 0.34672E+00_JPRB, 0.34727E+00_JPRB, 0.34762E+00_JPRB, 0.34742E+00_JPRB, 0.34793E+00_JPRB, 0.34953E+00_JPRB ,&
&  0.35345E+00_JPRB, 0.35864E+00_JPRB, 0.36617E+00_JPRB, 0.37909E+00_JPRB, 0.39230E+00_JPRB, 0.40104E+00_JPRB, 0.40679E+00_JPRB ,&
&  0.40773E+00_JPRB, 0.40576E+00_JPRB, 0.40299E+00_JPRB, 0.40126E+00_JPRB, 0.39962E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.37039E+00_JPRB, 0.36586E+00_JPRB, 0.36249E+00_JPRB, 0.35993E+00_JPRB, 0.35983E+00_JPRB, 0.35862E+00_JPRB, 0.35572E+00_JPRB ,&
&  0.35364E+00_JPRB, 0.35498E+00_JPRB, 0.35766E+00_JPRB, 0.35869E+00_JPRB, 0.35780E+00_JPRB, 0.35615E+00_JPRB, 0.35310E+00_JPRB ,&
&  0.34904E+00_JPRB, 0.34702E+00_JPRB, 0.34670E+00_JPRB, 0.34723E+00_JPRB, 0.34678E+00_JPRB, 0.34477E+00_JPRB, 0.34319E+00_JPRB ,&
&  0.34383E+00_JPRB, 0.34520E+00_JPRB, 0.34566E+00_JPRB, 0.34781E+00_JPRB, 0.35239E+00_JPRB, 0.36083E+00_JPRB, 0.36852E+00_JPRB ,&
&  0.37225E+00_JPRB, 0.37427E+00_JPRB, 0.36846E+00_JPRB, 0.38169E+00_JPRB, 0.39082E+00_JPRB, 0.40290E+00_JPRB, 0.41046E+00_JPRB ,&
&  0.41187E+00_JPRB, 0.40654E+00_JPRB, 0.39453E+00_JPRB, 0.38503E+00_JPRB, 0.37887E+00_JPRB, 0.37203E+00_JPRB, 0.36174E+00_JPRB ,&
&  0.35125E+00_JPRB, 0.34247E+00_JPRB, 0.33723E+00_JPRB, 0.33474E+00_JPRB, 0.33469E+00_JPRB, 0.33513E+00_JPRB, 0.33335E+00_JPRB ,&
&  0.33126E+00_JPRB, 0.32925E+00_JPRB, 0.32842E+00_JPRB, 0.32760E+00_JPRB, 0.32642E+00_JPRB, 0.32547E+00_JPRB, 0.32519E+00_JPRB ,&
&  0.32534E+00_JPRB, 0.32553E+00_JPRB, 0.32604E+00_JPRB, 0.32693E+00_JPRB, 0.32744E+00_JPRB, 0.32712E+00_JPRB, 0.32631E+00_JPRB ,&
&  0.32536E+00_JPRB, 0.32457E+00_JPRB, 0.32391E+00_JPRB, 0.32315E+00_JPRB, 0.32241E+00_JPRB, 0.32162E+00_JPRB, 0.32032E+00_JPRB ,&
&  0.31866E+00_JPRB, 0.31833E+00_JPRB, 0.31894E+00_JPRB, 0.31893E+00_JPRB, 0.31868E+00_JPRB, 0.31937E+00_JPRB, 0.32107E+00_JPRB ,&
&  0.32279E+00_JPRB, 0.32483E+00_JPRB, 0.32642E+00_JPRB, 0.32728E+00_JPRB, 0.32695E+00_JPRB, 0.32718E+00_JPRB, 0.32810E+00_JPRB ,&
&  0.33019E+00_JPRB, 0.33353E+00_JPRB, 0.33840E+00_JPRB, 0.34543E+00_JPRB, 0.35405E+00_JPRB, 0.36112E+00_JPRB, 0.36724E+00_JPRB ,&
&  0.37198E+00_JPRB, 0.37423E+00_JPRB, 0.37457E+00_JPRB, 0.37499E+00_JPRB, 0.37401E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.34282E+00_JPRB, 0.33899E+00_JPRB, 0.33625E+00_JPRB, 0.33587E+00_JPRB, 0.33767E+00_JPRB, 0.33788E+00_JPRB, 0.33606E+00_JPRB ,&
&  0.33440E+00_JPRB, 0.33579E+00_JPRB, 0.33910E+00_JPRB, 0.34099E+00_JPRB, 0.34169E+00_JPRB, 0.34237E+00_JPRB, 0.34106E+00_JPRB ,&
&  0.33739E+00_JPRB, 0.33376E+00_JPRB, 0.33400E+00_JPRB, 0.33528E+00_JPRB, 0.33609E+00_JPRB, 0.33443E+00_JPRB, 0.33248E+00_JPRB ,&
&  0.33273E+00_JPRB, 0.33272E+00_JPRB, 0.33445E+00_JPRB, 0.33631E+00_JPRB, 0.34038E+00_JPRB, 0.34732E+00_JPRB, 0.35266E+00_JPRB ,&
&  0.35496E+00_JPRB, 0.35353E+00_JPRB, 0.35278E+00_JPRB, 0.35604E+00_JPRB, 0.35992E+00_JPRB, 0.36862E+00_JPRB, 0.37804E+00_JPRB ,&
&  0.38196E+00_JPRB, 0.37912E+00_JPRB, 0.36794E+00_JPRB, 0.35768E+00_JPRB, 0.35029E+00_JPRB, 0.34255E+00_JPRB, 0.33496E+00_JPRB ,&
&  0.32741E+00_JPRB, 0.32197E+00_JPRB, 0.31879E+00_JPRB, 0.31734E+00_JPRB, 0.31684E+00_JPRB, 0.31610E+00_JPRB, 0.31377E+00_JPRB ,&
&  0.31076E+00_JPRB, 0.30841E+00_JPRB, 0.30665E+00_JPRB, 0.30532E+00_JPRB, 0.30412E+00_JPRB, 0.30355E+00_JPRB, 0.30287E+00_JPRB ,&
&  0.30256E+00_JPRB, 0.30300E+00_JPRB, 0.30384E+00_JPRB, 0.30386E+00_JPRB, 0.30366E+00_JPRB, 0.30292E+00_JPRB, 0.30226E+00_JPRB ,&
&  0.30136E+00_JPRB, 0.29933E+00_JPRB, 0.29769E+00_JPRB, 0.29753E+00_JPRB, 0.29757E+00_JPRB, 0.29813E+00_JPRB, 0.29850E+00_JPRB ,&
&  0.29820E+00_JPRB, 0.29817E+00_JPRB, 0.29880E+00_JPRB, 0.29957E+00_JPRB, 0.30011E+00_JPRB, 0.30044E+00_JPRB, 0.30105E+00_JPRB ,&
&  0.30273E+00_JPRB, 0.30471E+00_JPRB, 0.30615E+00_JPRB, 0.30733E+00_JPRB, 0.30821E+00_JPRB, 0.30962E+00_JPRB, 0.30987E+00_JPRB ,&
&  0.31022E+00_JPRB, 0.31166E+00_JPRB, 0.31495E+00_JPRB, 0.31967E+00_JPRB, 0.32482E+00_JPRB, 0.32912E+00_JPRB, 0.33336E+00_JPRB ,&
&  0.33710E+00_JPRB, 0.34019E+00_JPRB, 0.34215E+00_JPRB, 0.34284E+00_JPRB, 0.34343E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.31714E+00_JPRB, 0.31559E+00_JPRB, 0.31450E+00_JPRB, 0.31485E+00_JPRB, 0.31702E+00_JPRB, 0.31744E+00_JPRB, 0.31726E+00_JPRB ,&
&  0.31703E+00_JPRB, 0.31911E+00_JPRB, 0.32293E+00_JPRB, 0.32536E+00_JPRB, 0.32582E+00_JPRB, 0.32571E+00_JPRB, 0.32514E+00_JPRB ,&
&  0.32462E+00_JPRB, 0.32415E+00_JPRB, 0.32293E+00_JPRB, 0.32306E+00_JPRB, 0.32292E+00_JPRB, 0.32178E+00_JPRB, 0.32242E+00_JPRB ,&
&  0.32236E+00_JPRB, 0.32274E+00_JPRB, 0.32500E+00_JPRB, 0.32688E+00_JPRB, 0.33103E+00_JPRB, 0.33796E+00_JPRB, 0.34206E+00_JPRB ,&
&  0.34241E+00_JPRB, 0.33805E+00_JPRB, 0.33415E+00_JPRB, 0.33477E+00_JPRB, 0.33697E+00_JPRB, 0.34155E+00_JPRB, 0.34675E+00_JPRB ,&
&  0.34862E+00_JPRB, 0.34276E+00_JPRB, 0.33280E+00_JPRB, 0.32507E+00_JPRB, 0.31960E+00_JPRB, 0.31551E+00_JPRB, 0.31190E+00_JPRB ,&
&  0.30814E+00_JPRB, 0.30550E+00_JPRB, 0.30433E+00_JPRB, 0.30417E+00_JPRB, 0.30377E+00_JPRB, 0.30086E+00_JPRB, 0.29747E+00_JPRB ,&
&  0.29312E+00_JPRB, 0.29008E+00_JPRB, 0.28758E+00_JPRB, 0.28671E+00_JPRB, 0.28679E+00_JPRB, 0.28649E+00_JPRB, 0.28540E+00_JPRB ,&
&  0.28405E+00_JPRB, 0.28362E+00_JPRB, 0.28320E+00_JPRB, 0.28195E+00_JPRB, 0.28041E+00_JPRB, 0.27990E+00_JPRB, 0.27882E+00_JPRB ,&
&  0.27678E+00_JPRB, 0.27571E+00_JPRB, 0.27597E+00_JPRB, 0.27717E+00_JPRB, 0.27725E+00_JPRB, 0.27818E+00_JPRB, 0.27985E+00_JPRB ,&
&  0.27902E+00_JPRB, 0.27655E+00_JPRB, 0.27519E+00_JPRB, 0.27563E+00_JPRB, 0.27742E+00_JPRB, 0.27984E+00_JPRB, 0.28106E+00_JPRB ,&
&  0.28215E+00_JPRB, 0.28225E+00_JPRB, 0.28325E+00_JPRB, 0.28472E+00_JPRB, 0.28654E+00_JPRB, 0.28775E+00_JPRB, 0.28832E+00_JPRB ,&
&  0.28887E+00_JPRB, 0.28955E+00_JPRB, 0.29186E+00_JPRB, 0.29543E+00_JPRB, 0.29872E+00_JPRB, 0.30244E+00_JPRB, 0.30548E+00_JPRB ,&
&  0.30781E+00_JPRB, 0.31036E+00_JPRB, 0.31234E+00_JPRB, 0.31375E+00_JPRB, 0.31547E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.30043E+00_JPRB, 0.30129E+00_JPRB, 0.30210E+00_JPRB, 0.30211E+00_JPRB, 0.30322E+00_JPRB, 0.30408E+00_JPRB, 0.30560E+00_JPRB ,&
&  0.30642E+00_JPRB, 0.30814E+00_JPRB, 0.31046E+00_JPRB, 0.31049E+00_JPRB, 0.31014E+00_JPRB, 0.30991E+00_JPRB, 0.31047E+00_JPRB ,&
&  0.31266E+00_JPRB, 0.31368E+00_JPRB, 0.31306E+00_JPRB, 0.31282E+00_JPRB, 0.31179E+00_JPRB, 0.31084E+00_JPRB, 0.31264E+00_JPRB ,&
&  0.31296E+00_JPRB, 0.31359E+00_JPRB, 0.31554E+00_JPRB, 0.31728E+00_JPRB, 0.32134E+00_JPRB, 0.32650E+00_JPRB, 0.33070E+00_JPRB ,&
&  0.33164E+00_JPRB, 0.32856E+00_JPRB, 0.32278E+00_JPRB, 0.31846E+00_JPRB, 0.31766E+00_JPRB, 0.31874E+00_JPRB, 0.32122E+00_JPRB ,&
&  0.32050E+00_JPRB, 0.31472E+00_JPRB, 0.30715E+00_JPRB, 0.30284E+00_JPRB, 0.30097E+00_JPRB, 0.29842E+00_JPRB, 0.29458E+00_JPRB ,&
&  0.29152E+00_JPRB, 0.29059E+00_JPRB, 0.29068E+00_JPRB, 0.28990E+00_JPRB, 0.28812E+00_JPRB, 0.28503E+00_JPRB, 0.28106E+00_JPRB ,&
&  0.27753E+00_JPRB, 0.27405E+00_JPRB, 0.27185E+00_JPRB, 0.27036E+00_JPRB, 0.27018E+00_JPRB, 0.27029E+00_JPRB, 0.26935E+00_JPRB ,&
&  0.26734E+00_JPRB, 0.26602E+00_JPRB, 0.26425E+00_JPRB, 0.26099E+00_JPRB, 0.25865E+00_JPRB, 0.25788E+00_JPRB, 0.25675E+00_JPRB ,&
&  0.25518E+00_JPRB, 0.25466E+00_JPRB, 0.25450E+00_JPRB, 0.25406E+00_JPRB, 0.25499E+00_JPRB, 0.25676E+00_JPRB, 0.25441E+00_JPRB ,&
&  0.25228E+00_JPRB, 0.25086E+00_JPRB, 0.25097E+00_JPRB, 0.25260E+00_JPRB, 0.25530E+00_JPRB, 0.25826E+00_JPRB, 0.26068E+00_JPRB ,&
&  0.26170E+00_JPRB, 0.26200E+00_JPRB, 0.26236E+00_JPRB, 0.26256E+00_JPRB, 0.26378E+00_JPRB, 0.26625E+00_JPRB, 0.26916E+00_JPRB ,&
&  0.27064E+00_JPRB, 0.27320E+00_JPRB, 0.27603E+00_JPRB, 0.27958E+00_JPRB, 0.28173E+00_JPRB, 0.28384E+00_JPRB, 0.28597E+00_JPRB ,&
&  0.28771E+00_JPRB, 0.28991E+00_JPRB, 0.29297E+00_JPRB, 0.29544E+00_JPRB, 0.29800E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.28768E+00_JPRB, 0.28970E+00_JPRB, 0.29069E+00_JPRB, 0.29188E+00_JPRB, 0.29432E+00_JPRB, 0.29594E+00_JPRB, 0.29615E+00_JPRB ,&
&  0.29646E+00_JPRB, 0.29789E+00_JPRB, 0.29896E+00_JPRB, 0.29943E+00_JPRB, 0.29710E+00_JPRB, 0.29670E+00_JPRB, 0.29835E+00_JPRB ,&
&  0.30055E+00_JPRB, 0.30261E+00_JPRB, 0.30264E+00_JPRB, 0.30246E+00_JPRB, 0.30165E+00_JPRB, 0.30029E+00_JPRB, 0.30163E+00_JPRB ,&
&  0.30274E+00_JPRB, 0.30318E+00_JPRB, 0.30453E+00_JPRB, 0.30602E+00_JPRB, 0.30925E+00_JPRB, 0.31308E+00_JPRB, 0.31583E+00_JPRB ,&
&  0.31690E+00_JPRB, 0.31550E+00_JPRB, 0.30857E+00_JPRB, 0.30138E+00_JPRB, 0.30155E+00_JPRB, 0.30255E+00_JPRB, 0.30244E+00_JPRB ,&
&  0.30033E+00_JPRB, 0.29743E+00_JPRB, 0.29546E+00_JPRB, 0.29386E+00_JPRB, 0.29182E+00_JPRB, 0.28813E+00_JPRB, 0.28409E+00_JPRB ,&
&  0.28061E+00_JPRB, 0.27936E+00_JPRB, 0.27795E+00_JPRB, 0.27554E+00_JPRB, 0.27195E+00_JPRB, 0.26898E+00_JPRB, 0.26590E+00_JPRB ,&
&  0.26062E+00_JPRB, 0.25551E+00_JPRB, 0.25272E+00_JPRB, 0.25199E+00_JPRB, 0.25229E+00_JPRB, 0.25178E+00_JPRB, 0.24993E+00_JPRB ,&
&  0.24779E+00_JPRB, 0.24515E+00_JPRB, 0.24351E+00_JPRB, 0.24300E+00_JPRB, 0.24411E+00_JPRB, 0.24274E+00_JPRB, 0.23823E+00_JPRB ,&
&  0.23565E+00_JPRB, 0.23413E+00_JPRB, 0.23436E+00_JPRB, 0.23336E+00_JPRB, 0.23246E+00_JPRB, 0.23124E+00_JPRB, 0.23172E+00_JPRB ,&
&  0.23287E+00_JPRB, 0.23314E+00_JPRB, 0.23577E+00_JPRB, 0.24043E+00_JPRB, 0.24466E+00_JPRB, 0.24713E+00_JPRB, 0.24862E+00_JPRB ,&
&  0.24915E+00_JPRB, 0.24858E+00_JPRB, 0.24803E+00_JPRB, 0.24864E+00_JPRB, 0.25080E+00_JPRB, 0.25373E+00_JPRB, 0.25656E+00_JPRB ,&
&  0.25734E+00_JPRB, 0.25854E+00_JPRB, 0.26134E+00_JPRB, 0.26554E+00_JPRB, 0.26804E+00_JPRB, 0.27004E+00_JPRB, 0.27180E+00_JPRB ,&
&  0.27277E+00_JPRB, 0.27448E+00_JPRB, 0.27854E+00_JPRB, 0.28136E+00_JPRB, 0.28418E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.27457E+00_JPRB, 0.27608E+00_JPRB, 0.27836E+00_JPRB, 0.28065E+00_JPRB, 0.28165E+00_JPRB, 0.28118E+00_JPRB, 0.28042E+00_JPRB ,&
&  0.28085E+00_JPRB, 0.28299E+00_JPRB, 0.28528E+00_JPRB, 0.28590E+00_JPRB, 0.28621E+00_JPRB, 0.28658E+00_JPRB, 0.28830E+00_JPRB ,&
&  0.29086E+00_JPRB, 0.29379E+00_JPRB, 0.29361E+00_JPRB, 0.29316E+00_JPRB, 0.29254E+00_JPRB, 0.29135E+00_JPRB, 0.29239E+00_JPRB ,&
&  0.29529E+00_JPRB, 0.29537E+00_JPRB, 0.29512E+00_JPRB, 0.29440E+00_JPRB, 0.29617E+00_JPRB, 0.29957E+00_JPRB, 0.30174E+00_JPRB ,&
&  0.30084E+00_JPRB, 0.29422E+00_JPRB, 0.28594E+00_JPRB, 0.28555E+00_JPRB, 0.28726E+00_JPRB, 0.28896E+00_JPRB, 0.28913E+00_JPRB ,&
&  0.28869E+00_JPRB, 0.28783E+00_JPRB, 0.28674E+00_JPRB, 0.28558E+00_JPRB, 0.28407E+00_JPRB, 0.27997E+00_JPRB, 0.27520E+00_JPRB ,&
&  0.27136E+00_JPRB, 0.26792E+00_JPRB, 0.26457E+00_JPRB, 0.26002E+00_JPRB, 0.25537E+00_JPRB, 0.25199E+00_JPRB, 0.24811E+00_JPRB ,&
&  0.24337E+00_JPRB, 0.24002E+00_JPRB, 0.23896E+00_JPRB, 0.23885E+00_JPRB, 0.23891E+00_JPRB, 0.23806E+00_JPRB, 0.23699E+00_JPRB ,&
&  0.23510E+00_JPRB, 0.23366E+00_JPRB, 0.23209E+00_JPRB, 0.23117E+00_JPRB, 0.22566E+00_JPRB, 0.21761E+00_JPRB, 0.21185E+00_JPRB ,&
&  0.21121E+00_JPRB, 0.21380E+00_JPRB, 0.21713E+00_JPRB, 0.21797E+00_JPRB, 0.22012E+00_JPRB, 0.22112E+00_JPRB, 0.22068E+00_JPRB ,&
&  0.21782E+00_JPRB, 0.21447E+00_JPRB, 0.21496E+00_JPRB, 0.21612E+00_JPRB, 0.21668E+00_JPRB, 0.21805E+00_JPRB, 0.21962E+00_JPRB ,&
&  0.22255E+00_JPRB, 0.22478E+00_JPRB, 0.22701E+00_JPRB, 0.22938E+00_JPRB, 0.23090E+00_JPRB, 0.23081E+00_JPRB, 0.22884E+00_JPRB ,&
&  0.22889E+00_JPRB, 0.23454E+00_JPRB, 0.24446E+00_JPRB, 0.25466E+00_JPRB, 0.25728E+00_JPRB, 0.25968E+00_JPRB, 0.26181E+00_JPRB ,&
&  0.26300E+00_JPRB, 0.26655E+00_JPRB, 0.26970E+00_JPRB, 0.27160E+00_JPRB, 0.27293E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.26577E+00_JPRB, 0.26568E+00_JPRB, 0.26461E+00_JPRB, 0.26361E+00_JPRB, 0.26209E+00_JPRB, 0.26058E+00_JPRB, 0.26122E+00_JPRB ,&
&  0.26377E+00_JPRB, 0.26714E+00_JPRB, 0.27040E+00_JPRB, 0.27292E+00_JPRB, 0.27486E+00_JPRB, 0.27580E+00_JPRB, 0.27726E+00_JPRB ,&
&  0.27894E+00_JPRB, 0.28184E+00_JPRB, 0.28280E+00_JPRB, 0.28361E+00_JPRB, 0.28275E+00_JPRB, 0.28192E+00_JPRB, 0.28291E+00_JPRB ,&
&  0.28606E+00_JPRB, 0.28749E+00_JPRB, 0.28658E+00_JPRB, 0.28445E+00_JPRB, 0.28500E+00_JPRB, 0.28710E+00_JPRB, 0.28677E+00_JPRB ,&
&  0.28033E+00_JPRB, 0.26679E+00_JPRB, 0.25757E+00_JPRB, 0.26071E+00_JPRB, 0.26943E+00_JPRB, 0.27344E+00_JPRB, 0.27632E+00_JPRB ,&
&  0.27813E+00_JPRB, 0.27833E+00_JPRB, 0.27743E+00_JPRB, 0.27574E+00_JPRB, 0.27289E+00_JPRB, 0.26802E+00_JPRB, 0.26227E+00_JPRB ,&
&  0.25715E+00_JPRB, 0.25458E+00_JPRB, 0.25147E+00_JPRB, 0.24596E+00_JPRB, 0.24058E+00_JPRB, 0.23724E+00_JPRB, 0.23321E+00_JPRB ,&
&  0.22905E+00_JPRB, 0.22592E+00_JPRB, 0.22517E+00_JPRB, 0.22470E+00_JPRB, 0.22452E+00_JPRB, 0.22380E+00_JPRB, 0.22135E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21375E+00_JPRB, 0.20811E+00_JPRB, 0.20299E+00_JPRB, 0.19948E+00_JPRB, 0.19591E+00_JPRB, 0.19153E+00_JPRB ,&
&  0.18931E+00_JPRB, 0.19182E+00_JPRB, 0.20070E+00_JPRB, 0.21041E+00_JPRB, 0.21465E+00_JPRB, 0.20313E+00_JPRB, 0.19931E+00_JPRB ,&
&  0.19869E+00_JPRB, 0.19847E+00_JPRB, 0.19882E+00_JPRB, 0.20031E+00_JPRB, 0.20104E+00_JPRB, 0.20074E+00_JPRB, 0.20129E+00_JPRB ,&
&  0.20335E+00_JPRB, 0.20547E+00_JPRB, 0.20656E+00_JPRB, 0.20783E+00_JPRB, 0.20864E+00_JPRB, 0.20848E+00_JPRB, 0.20815E+00_JPRB ,&
&  0.20885E+00_JPRB, 0.21194E+00_JPRB, 0.21695E+00_JPRB, 0.22407E+00_JPRB, 0.22913E+00_JPRB, 0.23380E+00_JPRB, 0.23972E+00_JPRB ,&
&  0.24728E+00_JPRB, 0.25455E+00_JPRB, 0.25896E+00_JPRB, 0.26295E+00_JPRB, 0.26479E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.25763E+00_JPRB, 0.25834E+00_JPRB, 0.25459E+00_JPRB, 0.25133E+00_JPRB, 0.24970E+00_JPRB, 0.24924E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.25366E+00_JPRB, 0.25611E+00_JPRB, 0.25859E+00_JPRB, 0.26077E+00_JPRB, 0.26170E+00_JPRB, 0.26149E+00_JPRB, 0.26247E+00_JPRB ,&
&  0.26342E+00_JPRB, 0.26600E+00_JPRB, 0.26756E+00_JPRB, 0.27027E+00_JPRB, 0.27045E+00_JPRB, 0.27113E+00_JPRB, 0.27233E+00_JPRB ,&
&  0.27187E+00_JPRB, 0.27229E+00_JPRB, 0.27367E+00_JPRB, 0.27135E+00_JPRB, 0.26986E+00_JPRB, 0.26764E+00_JPRB, 0.26594E+00_JPRB ,&
&  0.25954E+00_JPRB, 0.25277E+00_JPRB, 0.24627E+00_JPRB, 0.25008E+00_JPRB, 0.25795E+00_JPRB, 0.25959E+00_JPRB, 0.26368E+00_JPRB ,&
&  0.26494E+00_JPRB, 0.26321E+00_JPRB, 0.26146E+00_JPRB, 0.25810E+00_JPRB, 0.25487E+00_JPRB, 0.25118E+00_JPRB, 0.24823E+00_JPRB ,&
&  0.24494E+00_JPRB, 0.24547E+00_JPRB, 0.24124E+00_JPRB, 0.22751E+00_JPRB, 0.22051E+00_JPRB, 0.21616E+00_JPRB, 0.20912E+00_JPRB ,&
&  0.20199E+00_JPRB, 0.20009E+00_JPRB, 0.19969E+00_JPRB, 0.19718E+00_JPRB, 0.19697E+00_JPRB, 0.19726E+00_JPRB, 0.19717E+00_JPRB ,&
&  0.19577E+00_JPRB, 0.19548E+00_JPRB, 0.19415E+00_JPRB, 0.19218E+00_JPRB, 0.19025E+00_JPRB, 0.18946E+00_JPRB, 0.18988E+00_JPRB ,&
&  0.19139E+00_JPRB, 0.19295E+00_JPRB, 0.19854E+00_JPRB, 0.20198E+00_JPRB, 0.19696E+00_JPRB, 0.19448E+00_JPRB, 0.19514E+00_JPRB ,&
&  0.19112E+00_JPRB, 0.19036E+00_JPRB, 0.19036E+00_JPRB, 0.19033E+00_JPRB, 0.19070E+00_JPRB, 0.19160E+00_JPRB, 0.19145E+00_JPRB ,&
&  0.19210E+00_JPRB, 0.19349E+00_JPRB, 0.19389E+00_JPRB, 0.19394E+00_JPRB, 0.19369E+00_JPRB, 0.19417E+00_JPRB, 0.19546E+00_JPRB ,&
&  0.19693E+00_JPRB, 0.19969E+00_JPRB, 0.20257E+00_JPRB, 0.20681E+00_JPRB, 0.21017E+00_JPRB, 0.21282E+00_JPRB, 0.21767E+00_JPRB ,&
&  0.22184E+00_JPRB, 0.23136E+00_JPRB, 0.24737E+00_JPRB, 0.25729E+00_JPRB, 0.25325E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.24765E+00_JPRB, 0.24643E+00_JPRB, 0.24501E+00_JPRB, 0.24365E+00_JPRB, 0.24456E+00_JPRB, 0.24576E+00_JPRB, 0.24737E+00_JPRB ,&
&  0.24919E+00_JPRB, 0.24971E+00_JPRB, 0.24965E+00_JPRB, 0.24736E+00_JPRB, 0.24609E+00_JPRB, 0.24009E+00_JPRB, 0.23799E+00_JPRB ,&
&  0.23641E+00_JPRB, 0.23842E+00_JPRB, 0.24111E+00_JPRB, 0.24521E+00_JPRB, 0.24689E+00_JPRB, 0.24866E+00_JPRB, 0.24767E+00_JPRB ,&
&  0.24546E+00_JPRB, 0.24280E+00_JPRB, 0.24021E+00_JPRB, 0.23818E+00_JPRB, 0.23758E+00_JPRB, 0.23585E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.23405E+00_JPRB, 0.23555E+00_JPRB, 0.23633E+00_JPRB, 0.23714E+00_JPRB, 0.24300E+00_JPRB, 0.24344E+00_JPRB, 0.24563E+00_JPRB ,&
&  0.24629E+00_JPRB, 0.24700E+00_JPRB, 0.24645E+00_JPRB, 0.24511E+00_JPRB, 0.24383E+00_JPRB, 0.23811E+00_JPRB, 0.23559E+00_JPRB ,&
&  0.22843E+00_JPRB, 0.22424E+00_JPRB, 0.21739E+00_JPRB, 0.21117E+00_JPRB, 0.20419E+00_JPRB, 0.19896E+00_JPRB, 0.19460E+00_JPRB ,&
&  0.19113E+00_JPRB, 0.18854E+00_JPRB, 0.18639E+00_JPRB, 0.18610E+00_JPRB, 0.18616E+00_JPRB, 0.18571E+00_JPRB, 0.18586E+00_JPRB ,&
&  0.18674E+00_JPRB, 0.18771E+00_JPRB, 0.18761E+00_JPRB, 0.18820E+00_JPRB, 0.18873E+00_JPRB, 0.18950E+00_JPRB, 0.19098E+00_JPRB ,&
&  0.19327E+00_JPRB, 0.19519E+00_JPRB, 0.19583E+00_JPRB, 0.19778E+00_JPRB, 0.19711E+00_JPRB, 0.19442E+00_JPRB, 0.19179E+00_JPRB ,&
&  0.18880E+00_JPRB, 0.18749E+00_JPRB, 0.18602E+00_JPRB, 0.18517E+00_JPRB, 0.18502E+00_JPRB, 0.18520E+00_JPRB, 0.18600E+00_JPRB ,&
&  0.18685E+00_JPRB, 0.18777E+00_JPRB, 0.18813E+00_JPRB, 0.18860E+00_JPRB, 0.18902E+00_JPRB, 0.18989E+00_JPRB, 0.19156E+00_JPRB ,&
&  0.19385E+00_JPRB, 0.19648E+00_JPRB, 0.19903E+00_JPRB, 0.20216E+00_JPRB, 0.20729E+00_JPRB, 0.21411E+00_JPRB, 0.22437E+00_JPRB ,&
&  0.23238E+00_JPRB, 0.24447E+00_JPRB, 0.24414E+00_JPRB, 0.24663E+00_JPRB, 0.24808E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.24024E+00_JPRB, 0.24017E+00_JPRB, 0.23934E+00_JPRB, 0.23884E+00_JPRB, 0.23932E+00_JPRB, 0.24014E+00_JPRB, 0.24123E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.23535E+00_JPRB, 0.23107E+00_JPRB, 0.22632E+00_JPRB, 0.22213E+00_JPRB, 0.21892E+00_JPRB, 0.21708E+00_JPRB ,&
&  0.21688E+00_JPRB, 0.21905E+00_JPRB, 0.22068E+00_JPRB, 0.22274E+00_JPRB, 0.22284E+00_JPRB, 0.22299E+00_JPRB, 0.22151E+00_JPRB ,&
&  0.21993E+00_JPRB, 0.21931E+00_JPRB, 0.21956E+00_JPRB, 0.22119E+00_JPRB, 0.22383E+00_JPRB, 0.22555E+00_JPRB, 0.22763E+00_JPRB ,&
&  0.23045E+00_JPRB, 0.22963E+00_JPRB, 0.23034E+00_JPRB, 0.22969E+00_JPRB, 0.23008E+00_JPRB, 0.23112E+00_JPRB, 0.23165E+00_JPRB ,&
&  0.23275E+00_JPRB, 0.23382E+00_JPRB, 0.23382E+00_JPRB, 0.23304E+00_JPRB, 0.22868E+00_JPRB, 0.22618E+00_JPRB, 0.22152E+00_JPRB ,&
&  0.21594E+00_JPRB, 0.21094E+00_JPRB, 0.20511E+00_JPRB, 0.20098E+00_JPRB, 0.19606E+00_JPRB, 0.19315E+00_JPRB, 0.19090E+00_JPRB ,&
&  0.18880E+00_JPRB, 0.18802E+00_JPRB, 0.18691E+00_JPRB, 0.18568E+00_JPRB, 0.18539E+00_JPRB, 0.18414E+00_JPRB, 0.18406E+00_JPRB ,&
&  0.18438E+00_JPRB, 0.18467E+00_JPRB, 0.18569E+00_JPRB, 0.18615E+00_JPRB, 0.18695E+00_JPRB, 0.18767E+00_JPRB, 0.18817E+00_JPRB ,&
&  0.18929E+00_JPRB, 0.19077E+00_JPRB, 0.19074E+00_JPRB, 0.19108E+00_JPRB, 0.19024E+00_JPRB, 0.18839E+00_JPRB, 0.18639E+00_JPRB ,&
&  0.18421E+00_JPRB, 0.18252E+00_JPRB, 0.18201E+00_JPRB, 0.18125E+00_JPRB, 0.18140E+00_JPRB, 0.18179E+00_JPRB, 0.18194E+00_JPRB ,&
&  0.18261E+00_JPRB, 0.18321E+00_JPRB, 0.18407E+00_JPRB, 0.18603E+00_JPRB, 0.18739E+00_JPRB, 0.18915E+00_JPRB, 0.19100E+00_JPRB ,&
&  0.19245E+00_JPRB, 0.19512E+00_JPRB, 0.19746E+00_JPRB, 0.20070E+00_JPRB, 0.20818E+00_JPRB, 0.21459E+00_JPRB, 0.22607E+00_JPRB ,&
&  0.23099E+00_JPRB, 0.24143E+00_JPRB, 0.23899E+00_JPRB, 0.23945E+00_JPRB, 0.24064E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.23474E+00_JPRB, 0.23565E+00_JPRB, 0.23555E+00_JPRB, 0.23654E+00_JPRB, 0.23732E+00_JPRB, 0.23841E+00_JPRB, 0.23846E+00_JPRB ,&
&  0.23796E+00_JPRB, 0.23394E+00_JPRB, 0.23176E+00_JPRB, 0.22843E+00_JPRB, 0.22532E+00_JPRB, 0.22201E+00_JPRB, 0.21930E+00_JPRB ,&
&  0.21756E+00_JPRB, 0.21620E+00_JPRB, 0.21495E+00_JPRB, 0.21323E+00_JPRB, 0.21108E+00_JPRB, 0.20891E+00_JPRB, 0.20693E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20521E+00_JPRB, 0.20650E+00_JPRB, 0.20874E+00_JPRB, 0.21284E+00_JPRB, 0.21587E+00_JPRB, 0.21978E+00_JPRB ,&
&  0.22577E+00_JPRB, 0.22222E+00_JPRB, 0.22240E+00_JPRB, 0.22275E+00_JPRB, 0.22254E+00_JPRB, 0.22241E+00_JPRB, 0.21977E+00_JPRB ,&
&  0.22379E+00_JPRB, 0.22403E+00_JPRB, 0.22762E+00_JPRB, 0.22655E+00_JPRB, 0.22177E+00_JPRB, 0.21817E+00_JPRB, 0.21313E+00_JPRB ,&
&  0.20965E+00_JPRB, 0.20510E+00_JPRB, 0.20203E+00_JPRB, 0.19918E+00_JPRB, 0.19572E+00_JPRB, 0.19430E+00_JPRB, 0.19190E+00_JPRB ,&
&  0.19047E+00_JPRB, 0.19018E+00_JPRB, 0.18917E+00_JPRB, 0.18916E+00_JPRB, 0.18830E+00_JPRB, 0.18707E+00_JPRB, 0.18628E+00_JPRB ,&
&  0.18557E+00_JPRB, 0.18482E+00_JPRB, 0.18456E+00_JPRB, 0.18427E+00_JPRB, 0.18439E+00_JPRB, 0.18437E+00_JPRB, 0.18438E+00_JPRB ,&
&  0.18456E+00_JPRB, 0.18469E+00_JPRB, 0.18425E+00_JPRB, 0.18411E+00_JPRB, 0.18317E+00_JPRB, 0.18235E+00_JPRB, 0.18106E+00_JPRB ,&
&  0.18026E+00_JPRB, 0.17977E+00_JPRB, 0.17901E+00_JPRB, 0.17851E+00_JPRB, 0.17836E+00_JPRB, 0.17885E+00_JPRB, 0.17949E+00_JPRB ,&
&  0.18022E+00_JPRB, 0.18160E+00_JPRB, 0.18272E+00_JPRB, 0.18423E+00_JPRB, 0.18591E+00_JPRB, 0.18721E+00_JPRB, 0.18910E+00_JPRB ,&
&  0.19096E+00_JPRB, 0.19289E+00_JPRB, 0.19620E+00_JPRB, 0.19983E+00_JPRB, 0.20464E+00_JPRB, 0.20980E+00_JPRB, 0.21238E+00_JPRB ,&
&  0.21520E+00_JPRB, 0.22166E+00_JPRB, 0.22431E+00_JPRB, 0.23251E+00_JPRB, 0.23386E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.22472E+00_JPRB, 0.22637E+00_JPRB, 0.22795E+00_JPRB, 0.22946E+00_JPRB, 0.23122E+00_JPRB, 0.23195E+00_JPRB, 0.23195E+00_JPRB ,&
&  0.22963E+00_JPRB, 0.22915E+00_JPRB, 0.22767E+00_JPRB, 0.22576E+00_JPRB, 0.22423E+00_JPRB, 0.22109E+00_JPRB, 0.21821E+00_JPRB ,&
&  0.21534E+00_JPRB, 0.21257E+00_JPRB, 0.21015E+00_JPRB, 0.20680E+00_JPRB, 0.20361E+00_JPRB, 0.20043E+00_JPRB, 0.19959E+00_JPRB ,&
&  0.19930E+00_JPRB, 0.19990E+00_JPRB, 0.20174E+00_JPRB, 0.20317E+00_JPRB, 0.20525E+00_JPRB, 0.20746E+00_JPRB, 0.21078E+00_JPRB ,&
&  0.21390E+00_JPRB, 0.21619E+00_JPRB, 0.21646E+00_JPRB, 0.21522E+00_JPRB, 0.21427E+00_JPRB, 0.21405E+00_JPRB, 0.21371E+00_JPRB ,&
&  0.21302E+00_JPRB, 0.21271E+00_JPRB, 0.21217E+00_JPRB, 0.21173E+00_JPRB, 0.21032E+00_JPRB, 0.20776E+00_JPRB, 0.20514E+00_JPRB ,&
&  0.20269E+00_JPRB, 0.20051E+00_JPRB, 0.19842E+00_JPRB, 0.19658E+00_JPRB, 0.19551E+00_JPRB, 0.19441E+00_JPRB, 0.19303E+00_JPRB ,&
&  0.19222E+00_JPRB, 0.19137E+00_JPRB, 0.19040E+00_JPRB, 0.18969E+00_JPRB, 0.18890E+00_JPRB, 0.18835E+00_JPRB, 0.18771E+00_JPRB ,&
&  0.18724E+00_JPRB, 0.18648E+00_JPRB, 0.18583E+00_JPRB, 0.18511E+00_JPRB, 0.18479E+00_JPRB, 0.18397E+00_JPRB, 0.18318E+00_JPRB ,&
&  0.18220E+00_JPRB, 0.18160E+00_JPRB, 0.18048E+00_JPRB, 0.17980E+00_JPRB, 0.17900E+00_JPRB, 0.17828E+00_JPRB, 0.17777E+00_JPRB ,&
&  0.17780E+00_JPRB, 0.17775E+00_JPRB, 0.17724E+00_JPRB, 0.17710E+00_JPRB, 0.17710E+00_JPRB, 0.17752E+00_JPRB, 0.17889E+00_JPRB ,&
&  0.17976E+00_JPRB, 0.18073E+00_JPRB, 0.18165E+00_JPRB, 0.18235E+00_JPRB, 0.18357E+00_JPRB, 0.18475E+00_JPRB, 0.18595E+00_JPRB ,&
&  0.18738E+00_JPRB, 0.18858E+00_JPRB, 0.19001E+00_JPRB, 0.19167E+00_JPRB, 0.19583E+00_JPRB, 0.19860E+00_JPRB, 0.20146E+00_JPRB ,&
&  0.20339E+00_JPRB, 0.20168E+00_JPRB, 0.21601E+00_JPRB, 0.21877E+00_JPRB, 0.22087E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.19977E+00_JPRB, 0.20270E+00_JPRB, 0.20448E+00_JPRB, 0.20591E+00_JPRB, 0.19919E+00_JPRB, 0.21055E+00_JPRB, 0.20723E+00_JPRB ,&
&  0.20670E+00_JPRB, 0.20576E+00_JPRB, 0.20767E+00_JPRB, 0.20787E+00_JPRB, 0.20735E+00_JPRB, 0.20653E+00_JPRB, 0.20541E+00_JPRB ,&
&  0.20434E+00_JPRB, 0.20294E+00_JPRB, 0.20207E+00_JPRB, 0.20112E+00_JPRB, 0.20037E+00_JPRB, 0.19962E+00_JPRB, 0.19910E+00_JPRB ,&
&  0.19913E+00_JPRB, 0.19998E+00_JPRB, 0.20150E+00_JPRB, 0.20083E+00_JPRB, 0.20078E+00_JPRB, 0.20114E+00_JPRB, 0.20172E+00_JPRB ,&
&  0.20194E+00_JPRB, 0.20200E+00_JPRB, 0.20167E+00_JPRB, 0.20053E+00_JPRB, 0.20059E+00_JPRB, 0.20071E+00_JPRB, 0.20091E+00_JPRB ,&
&  0.20091E+00_JPRB, 0.20066E+00_JPRB, 0.19979E+00_JPRB, 0.19898E+00_JPRB, 0.19815E+00_JPRB, 0.19714E+00_JPRB, 0.19591E+00_JPRB ,&
&  0.19490E+00_JPRB, 0.19412E+00_JPRB, 0.19316E+00_JPRB, 0.19220E+00_JPRB, 0.19138E+00_JPRB, 0.19058E+00_JPRB, 0.18983E+00_JPRB ,&
&  0.18935E+00_JPRB, 0.18899E+00_JPRB, 0.18860E+00_JPRB, 0.18803E+00_JPRB, 0.18738E+00_JPRB, 0.18677E+00_JPRB, 0.18623E+00_JPRB ,&
&  0.18587E+00_JPRB, 0.18527E+00_JPRB, 0.18467E+00_JPRB, 0.18402E+00_JPRB, 0.18352E+00_JPRB, 0.18270E+00_JPRB, 0.18198E+00_JPRB ,&
&  0.18127E+00_JPRB, 0.18063E+00_JPRB, 0.18014E+00_JPRB, 0.17977E+00_JPRB, 0.17944E+00_JPRB, 0.17920E+00_JPRB, 0.17903E+00_JPRB ,&
&  0.17891E+00_JPRB, 0.17881E+00_JPRB, 0.17881E+00_JPRB, 0.17873E+00_JPRB, 0.17862E+00_JPRB, 0.17858E+00_JPRB, 0.17887E+00_JPRB ,&
&  0.17910E+00_JPRB, 0.17926E+00_JPRB, 0.17935E+00_JPRB, 0.17901E+00_JPRB, 0.17942E+00_JPRB, 0.17978E+00_JPRB, 0.18021E+00_JPRB ,&
&  0.18058E+00_JPRB, 0.18139E+00_JPRB, 0.18238E+00_JPRB, 0.18359E+00_JPRB, 0.18591E+00_JPRB, 0.18796E+00_JPRB, 0.19014E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19614E+00_JPRB, 0.19758E+00_JPRB, 0.19850E+00_JPRB, 0.19919E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.18867E+00_JPRB, 0.18862E+00_JPRB, 0.18840E+00_JPRB, 0.18758E+00_JPRB, 0.18513E+00_JPRB, 0.18466E+00_JPRB, 0.18460E+00_JPRB ,&
&  0.18476E+00_JPRB, 0.18486E+00_JPRB, 0.18578E+00_JPRB, 0.18678E+00_JPRB, 0.18777E+00_JPRB, 0.18853E+00_JPRB, 0.18920E+00_JPRB ,&
&  0.18957E+00_JPRB, 0.18994E+00_JPRB, 0.18877E+00_JPRB, 0.18819E+00_JPRB, 0.18801E+00_JPRB, 0.18799E+00_JPRB, 0.18798E+00_JPRB ,&
&  0.18791E+00_JPRB, 0.18788E+00_JPRB, 0.18774E+00_JPRB, 0.18795E+00_JPRB, 0.18828E+00_JPRB, 0.18854E+00_JPRB, 0.18886E+00_JPRB ,&
&  0.18910E+00_JPRB, 0.18931E+00_JPRB, 0.18945E+00_JPRB, 0.18956E+00_JPRB, 0.18990E+00_JPRB, 0.19009E+00_JPRB, 0.19002E+00_JPRB ,&
&  0.18985E+00_JPRB, 0.18950E+00_JPRB, 0.18910E+00_JPRB, 0.18869E+00_JPRB, 0.18839E+00_JPRB, 0.18809E+00_JPRB, 0.18781E+00_JPRB ,&
&  0.18751E+00_JPRB, 0.18720E+00_JPRB, 0.18686E+00_JPRB, 0.18648E+00_JPRB, 0.18612E+00_JPRB, 0.18585E+00_JPRB, 0.18556E+00_JPRB ,&
&  0.18532E+00_JPRB, 0.18510E+00_JPRB, 0.18485E+00_JPRB, 0.18452E+00_JPRB, 0.18418E+00_JPRB, 0.18384E+00_JPRB, 0.18346E+00_JPRB ,&
&  0.18298E+00_JPRB, 0.18265E+00_JPRB, 0.18231E+00_JPRB, 0.18202E+00_JPRB, 0.18177E+00_JPRB, 0.18150E+00_JPRB, 0.18127E+00_JPRB ,&
&  0.18101E+00_JPRB, 0.18077E+00_JPRB, 0.18048E+00_JPRB, 0.18022E+00_JPRB, 0.17998E+00_JPRB, 0.17977E+00_JPRB, 0.17959E+00_JPRB ,&
&  0.17942E+00_JPRB, 0.17925E+00_JPRB, 0.17910E+00_JPRB, 0.17910E+00_JPRB, 0.17907E+00_JPRB, 0.17909E+00_JPRB, 0.17910E+00_JPRB ,&
&  0.17913E+00_JPRB, 0.17914E+00_JPRB, 0.17914E+00_JPRB, 0.17902E+00_JPRB, 0.17916E+00_JPRB, 0.17938E+00_JPRB, 0.17971E+00_JPRB ,&
&  0.18009E+00_JPRB, 0.18059E+00_JPRB, 0.18112E+00_JPRB, 0.18169E+00_JPRB, 0.18234E+00_JPRB, 0.18289E+00_JPRB, 0.18351E+00_JPRB ,&
&  0.18420E+00_JPRB, 0.18492E+00_JPRB, 0.18544E+00_JPRB, 0.18604E+00_JPRB, 0.18696E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB ,&
&  0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB, 0.18238E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB, 0.10685E+01_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.10925E+01_JPRB, 0.10906E+01_JPRB, 0.10890E+01_JPRB, 0.10880E+01_JPRB, 0.10874E+01_JPRB, 0.10867E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10816E+01_JPRB, 0.10778E+01_JPRB, 0.10740E+01_JPRB, 0.10695E+01_JPRB, 0.10637E+01_JPRB, 0.10567E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.10397E+01_JPRB, 0.10346E+01_JPRB, 0.10311E+01_JPRB, 0.10273E+01_JPRB, 0.10235E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.10161E+01_JPRB, 0.10130E+01_JPRB, 0.10099E+01_JPRB, 0.10072E+01_JPRB, 0.10060E+01_JPRB, 0.10044E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.10005E+01_JPRB, 0.99978E+00_JPRB, 0.99905E+00_JPRB, 0.99858E+00_JPRB, 0.99880E+00_JPRB, 0.99906E+00_JPRB ,&
&  0.99963E+00_JPRB, 0.10013E+01_JPRB, 0.10040E+01_JPRB, 0.10064E+01_JPRB, 0.10091E+01_JPRB, 0.10131E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10278E+01_JPRB, 0.10330E+01_JPRB, 0.10390E+01_JPRB, 0.10446E+01_JPRB, 0.10501E+01_JPRB, 0.10553E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10649E+01_JPRB, 0.10692E+01_JPRB, 0.10715E+01_JPRB, 0.10749E+01_JPRB, 0.10778E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.10825E+01_JPRB, 0.10832E+01_JPRB, 0.10846E+01_JPRB, 0.10857E+01_JPRB, 0.10869E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10916E+01_JPRB, 0.10933E+01_JPRB, 0.10945E+01_JPRB, 0.10960E+01_JPRB, 0.10964E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10995E+01_JPRB, 0.10994E+01_JPRB, 0.10995E+01_JPRB, 0.10995E+01_JPRB, 0.10997E+01_JPRB, 0.10998E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.11001E+01_JPRB, 0.11001E+01_JPRB, 0.10993E+01_JPRB, 0.10979E+01_JPRB, 0.10967E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.10947E+01_JPRB, 0.10950E+01_JPRB, 0.10957E+01_JPRB, 0.10975E+01_JPRB, 0.10980E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.10988E+01_JPRB, 0.10978E+01_JPRB, 0.10963E+01_JPRB, 0.10945E+01_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.10932E+01_JPRB, 0.10914E+01_JPRB, 0.10906E+01_JPRB, 0.10911E+01_JPRB, 0.10920E+01_JPRB, 0.10922E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.10794E+01_JPRB, 0.10698E+01_JPRB, 0.10592E+01_JPRB, 0.10466E+01_JPRB, 0.10347E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10124E+01_JPRB, 0.10049E+01_JPRB, 0.99897E+00_JPRB, 0.99214E+00_JPRB, 0.98504E+00_JPRB, 0.97786E+00_JPRB ,&
&  0.97259E+00_JPRB, 0.96670E+00_JPRB, 0.96012E+00_JPRB, 0.95201E+00_JPRB, 0.94636E+00_JPRB, 0.94036E+00_JPRB, 0.93483E+00_JPRB ,&
&  0.92771E+00_JPRB, 0.92389E+00_JPRB, 0.92162E+00_JPRB, 0.92098E+00_JPRB, 0.92557E+00_JPRB, 0.92949E+00_JPRB, 0.93003E+00_JPRB ,&
&  0.92861E+00_JPRB, 0.92225E+00_JPRB, 0.92271E+00_JPRB, 0.92418E+00_JPRB, 0.92790E+00_JPRB, 0.93566E+00_JPRB, 0.94628E+00_JPRB ,&
&  0.96223E+00_JPRB, 0.98205E+00_JPRB, 0.10037E+01_JPRB, 0.10217E+01_JPRB, 0.10397E+01_JPRB, 0.10558E+01_JPRB, 0.10676E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.10932E+01_JPRB, 0.11047E+01_JPRB, 0.11153E+01_JPRB, 0.11289E+01_JPRB, 0.11395E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11499E+01_JPRB, 0.11493E+01_JPRB, 0.11486E+01_JPRB, 0.11483E+01_JPRB, 0.11455E+01_JPRB, 0.11438E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11445E+01_JPRB, 0.11470E+01_JPRB, 0.11500E+01_JPRB, 0.11532E+01_JPRB, 0.11528E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.11520E+01_JPRB, 0.11487E+01_JPRB, 0.11427E+01_JPRB, 0.11375E+01_JPRB, 0.11343E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.11218E+01_JPRB, 0.11194E+01_JPRB, 0.11160E+01_JPRB, 0.11112E+01_JPRB, 0.11068E+01_JPRB, 0.11039E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10987E+01_JPRB, 0.10982E+01_JPRB, 0.10980E+01_JPRB, 0.11024E+01_JPRB, 0.11036E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11048E+01_JPRB, 0.11033E+01_JPRB, 0.11002E+01_JPRB, 0.10960E+01_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.11037E+01_JPRB, 0.11057E+01_JPRB, 0.11115E+01_JPRB, 0.11137E+01_JPRB, 0.11138E+01_JPRB, 0.11104E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10876E+01_JPRB, 0.10665E+01_JPRB, 0.10486E+01_JPRB, 0.10316E+01_JPRB, 0.10198E+01_JPRB, 0.10101E+01_JPRB ,&
&  0.10001E+01_JPRB, 0.99218E+00_JPRB, 0.98257E+00_JPRB, 0.97169E+00_JPRB, 0.95808E+00_JPRB, 0.94264E+00_JPRB, 0.92287E+00_JPRB ,&
&  0.90810E+00_JPRB, 0.89406E+00_JPRB, 0.88268E+00_JPRB, 0.87538E+00_JPRB, 0.87382E+00_JPRB, 0.83738E+00_JPRB, 0.83092E+00_JPRB ,&
&  0.84578E+00_JPRB, 0.82837E+00_JPRB, 0.81107E+00_JPRB, 0.78294E+00_JPRB, 0.77541E+00_JPRB, 0.78807E+00_JPRB, 0.79758E+00_JPRB ,&
&  0.78074E+00_JPRB, 0.82862E+00_JPRB, 0.83484E+00_JPRB, 0.83743E+00_JPRB, 0.83405E+00_JPRB, 0.82405E+00_JPRB, 0.83900E+00_JPRB ,&
&  0.87876E+00_JPRB, 0.89482E+00_JPRB, 0.93055E+00_JPRB, 0.97280E+00_JPRB, 0.10064E+01_JPRB, 0.10413E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.11369E+01_JPRB, 0.11553E+01_JPRB, 0.11725E+01_JPRB, 0.12002E+01_JPRB, 0.12274E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.12687E+01_JPRB, 0.12789E+01_JPRB, 0.12907E+01_JPRB, 0.13022E+01_JPRB, 0.13178E+01_JPRB, 0.13308E+01_JPRB, 0.13444E+01_JPRB ,&
&  0.13542E+01_JPRB, 0.13625E+01_JPRB, 0.13666E+01_JPRB, 0.13673E+01_JPRB, 0.13655E+01_JPRB, 0.13586E+01_JPRB, 0.13561E+01_JPRB ,&
&  0.13445E+01_JPRB, 0.13314E+01_JPRB, 0.13132E+01_JPRB, 0.12905E+01_JPRB, 0.12719E+01_JPRB, 0.12566E+01_JPRB, 0.12406E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.12089E+01_JPRB, 0.11942E+01_JPRB, 0.11776E+01_JPRB, 0.11619E+01_JPRB, 0.11458E+01_JPRB, 0.11341E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11192E+01_JPRB, 0.11166E+01_JPRB, 0.11130E+01_JPRB, 0.11141E+01_JPRB, 0.11134E+01_JPRB, 0.11168E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.11136E+01_JPRB, 0.11067E+01_JPRB, 0.11029E+01_JPRB, 0.11010E+01_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.10983E+01_JPRB, 0.11060E+01_JPRB, 0.11212E+01_JPRB, 0.11232E+01_JPRB, 0.11227E+01_JPRB, 0.11136E+01_JPRB, 0.11105E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.10837E+01_JPRB, 0.10520E+01_JPRB, 0.10312E+01_JPRB, 0.10133E+01_JPRB, 0.10027E+01_JPRB, 0.99523E+00_JPRB ,&
&  0.98911E+00_JPRB, 0.98062E+00_JPRB, 0.96916E+00_JPRB, 0.95206E+00_JPRB, 0.93021E+00_JPRB, 0.90533E+00_JPRB, 0.87123E+00_JPRB ,&
&  0.84225E+00_JPRB, 0.82139E+00_JPRB, 0.79426E+00_JPRB, 0.80227E+00_JPRB, 0.78851E+00_JPRB, 0.77838E+00_JPRB, 0.77172E+00_JPRB ,&
&  0.75190E+00_JPRB, 0.73207E+00_JPRB, 0.70070E+00_JPRB, 0.68324E+00_JPRB, 0.67056E+00_JPRB, 0.65541E+00_JPRB, 0.67577E+00_JPRB ,&
&  0.69239E+00_JPRB, 0.73897E+00_JPRB, 0.76108E+00_JPRB, 0.76937E+00_JPRB, 0.77776E+00_JPRB, 0.77591E+00_JPRB, 0.78701E+00_JPRB ,&
&  0.80071E+00_JPRB, 0.82080E+00_JPRB, 0.88917E+00_JPRB, 0.93594E+00_JPRB, 0.96953E+00_JPRB, 0.10028E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.10635E+01_JPRB, 0.10860E+01_JPRB, 0.11056E+01_JPRB, 0.11291E+01_JPRB, 0.11696E+01_JPRB, 0.12167E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.12949E+01_JPRB, 0.13187E+01_JPRB, 0.13489E+01_JPRB, 0.13834E+01_JPRB, 0.14297E+01_JPRB, 0.14811E+01_JPRB, 0.15355E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.16422E+01_JPRB, 0.16853E+01_JPRB, 0.17224E+01_JPRB, 0.17411E+01_JPRB, 0.17448E+01_JPRB, 0.17401E+01_JPRB ,&
&  0.17105E+01_JPRB, 0.16764E+01_JPRB, 0.16278E+01_JPRB, 0.15553E+01_JPRB, 0.14986E+01_JPRB, 0.14665E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.13644E+01_JPRB, 0.13192E+01_JPRB, 0.12772E+01_JPRB, 0.12342E+01_JPRB, 0.11960E+01_JPRB, 0.11616E+01_JPRB, 0.11385E+01_JPRB ,&
&  0.11204E+01_JPRB, 0.11086E+01_JPRB, 0.11021E+01_JPRB, 0.10977E+01_JPRB, 0.10986E+01_JPRB, 0.10971E+01_JPRB, 0.11036E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.11018E+01_JPRB, 0.10928E+01_JPRB, 0.10905E+01_JPRB, 0.10910E+01_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.10957E+01_JPRB, 0.11005E+01_JPRB, 0.11013E+01_JPRB, 0.10869E+01_JPRB, 0.10803E+01_JPRB, 0.10740E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.10717E+01_JPRB, 0.10626E+01_JPRB, 0.10337E+01_JPRB, 0.10142E+01_JPRB, 0.99418E+00_JPRB, 0.98392E+00_JPRB, 0.97454E+00_JPRB ,&
&  0.96736E+00_JPRB, 0.95723E+00_JPRB, 0.94948E+00_JPRB, 0.93172E+00_JPRB, 0.91386E+00_JPRB, 0.88688E+00_JPRB, 0.85682E+00_JPRB ,&
&  0.82547E+00_JPRB, 0.79378E+00_JPRB, 0.76732E+00_JPRB, 0.76775E+00_JPRB, 0.75645E+00_JPRB, 0.74920E+00_JPRB, 0.73884E+00_JPRB ,&
&  0.72408E+00_JPRB, 0.67090E+00_JPRB, 0.68918E+00_JPRB, 0.66970E+00_JPRB, 0.64816E+00_JPRB, 0.62221E+00_JPRB, 0.60076E+00_JPRB ,&
&  0.60515E+00_JPRB, 0.60892E+00_JPRB, 0.62674E+00_JPRB, 0.65730E+00_JPRB, 0.67257E+00_JPRB, 0.69896E+00_JPRB, 0.71536E+00_JPRB ,&
&  0.70610E+00_JPRB, 0.81861E+00_JPRB, 0.87717E+00_JPRB, 0.91503E+00_JPRB, 0.95130E+00_JPRB, 0.98021E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.10716E+01_JPRB, 0.11156E+01_JPRB, 0.11623E+01_JPRB, 0.12324E+01_JPRB, 0.12836E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.13896E+01_JPRB, 0.14225E+01_JPRB, 0.14750E+01_JPRB, 0.15403E+01_JPRB, 0.16121E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.17759E+01_JPRB, 0.18577E+01_JPRB, 0.19424E+01_JPRB, 0.20169E+01_JPRB, 0.20768E+01_JPRB, 0.21157E+01_JPRB, 0.21501E+01_JPRB ,&
&  0.21244E+01_JPRB, 0.20864E+01_JPRB, 0.20227E+01_JPRB, 0.19325E+01_JPRB, 0.18180E+01_JPRB, 0.16918E+01_JPRB, 0.15657E+01_JPRB ,&
&  0.14947E+01_JPRB, 0.14172E+01_JPRB, 0.13475E+01_JPRB, 0.12809E+01_JPRB, 0.12256E+01_JPRB, 0.11818E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.11336E+01_JPRB, 0.11190E+01_JPRB, 0.11096E+01_JPRB, 0.11021E+01_JPRB, 0.10990E+01_JPRB, 0.10900E+01_JPRB, 0.10880E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.10900E+01_JPRB, 0.10859E+01_JPRB, 0.10858E+01_JPRB, 0.10891E+01_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.11010E+01_JPRB, 0.11048E+01_JPRB, 0.10942E+01_JPRB, 0.10830E+01_JPRB, 0.10795E+01_JPRB, 0.10666E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10671E+01_JPRB, 0.10531E+01_JPRB, 0.10340E+01_JPRB, 0.10097E+01_JPRB, 0.99324E+00_JPRB, 0.97423E+00_JPRB ,&
&  0.95758E+00_JPRB, 0.93570E+00_JPRB, 0.91087E+00_JPRB, 0.88608E+00_JPRB, 0.86620E+00_JPRB, 0.84286E+00_JPRB, 0.82571E+00_JPRB ,&
&  0.80689E+00_JPRB, 0.78364E+00_JPRB, 0.76535E+00_JPRB, 0.76285E+00_JPRB, 0.75024E+00_JPRB, 0.73582E+00_JPRB, 0.73566E+00_JPRB ,&
&  0.73604E+00_JPRB, 0.72693E+00_JPRB, 0.71860E+00_JPRB, 0.69903E+00_JPRB, 0.65542E+00_JPRB, 0.57867E+00_JPRB, 0.59906E+00_JPRB ,&
&  0.59246E+00_JPRB, 0.57854E+00_JPRB, 0.58381E+00_JPRB, 0.59861E+00_JPRB, 0.61357E+00_JPRB, 0.65576E+00_JPRB, 0.66101E+00_JPRB ,&
&  0.64373E+00_JPRB, 0.80830E+00_JPRB, 0.84198E+00_JPRB, 0.88360E+00_JPRB, 0.92095E+00_JPRB, 0.94587E+00_JPRB, 0.96767E+00_JPRB ,&
&  0.99726E+00_JPRB, 0.10359E+01_JPRB, 0.10768E+01_JPRB, 0.11365E+01_JPRB, 0.12075E+01_JPRB, 0.12976E+01_JPRB, 0.13774E+01_JPRB ,&
&  0.14268E+01_JPRB, 0.14598E+01_JPRB, 0.15048E+01_JPRB, 0.15441E+01_JPRB, 0.15890E+01_JPRB, 0.16625E+01_JPRB, 0.17545E+01_JPRB ,&
&  0.18824E+01_JPRB, 0.19665E+01_JPRB, 0.21346E+01_JPRB, 0.22699E+01_JPRB, 0.23419E+01_JPRB, 0.24371E+01_JPRB, 0.25237E+01_JPRB ,&
&  0.24935E+01_JPRB, 0.24798E+01_JPRB, 0.24276E+01_JPRB, 0.22381E+01_JPRB, 0.20300E+01_JPRB, 0.18852E+01_JPRB, 0.17551E+01_JPRB ,&
&  0.16502E+01_JPRB, 0.15458E+01_JPRB, 0.14423E+01_JPRB, 0.13598E+01_JPRB, 0.12945E+01_JPRB, 0.12461E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.11905E+01_JPRB, 0.11672E+01_JPRB, 0.11446E+01_JPRB, 0.11335E+01_JPRB, 0.11130E+01_JPRB, 0.10956E+01_JPRB, 0.10785E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10706E+01_JPRB, 0.10763E+01_JPRB, 0.10822E+01_JPRB, 0.10857E+01_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.11127E+01_JPRB, 0.11349E+01_JPRB, 0.11353E+01_JPRB, 0.11348E+01_JPRB, 0.11327E+01_JPRB, 0.11114E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.10820E+01_JPRB, 0.10731E+01_JPRB, 0.10590E+01_JPRB, 0.10398E+01_JPRB, 0.10246E+01_JPRB, 0.10104E+01_JPRB ,&
&  0.99630E+00_JPRB, 0.93149E+00_JPRB, 0.89747E+00_JPRB, 0.86800E+00_JPRB, 0.83943E+00_JPRB, 0.82353E+00_JPRB, 0.81480E+00_JPRB ,&
&  0.80100E+00_JPRB, 0.79257E+00_JPRB, 0.78563E+00_JPRB, 0.77340E+00_JPRB, 0.75844E+00_JPRB, 0.73633E+00_JPRB, 0.72815E+00_JPRB ,&
&  0.73587E+00_JPRB, 0.71813E+00_JPRB, 0.70075E+00_JPRB, 0.67638E+00_JPRB, 0.63230E+00_JPRB, 0.61332E+00_JPRB, 0.61371E+00_JPRB ,&
&  0.60492E+00_JPRB, 0.57364E+00_JPRB, 0.57590E+00_JPRB, 0.57957E+00_JPRB, 0.58218E+00_JPRB, 0.60427E+00_JPRB, 0.64337E+00_JPRB ,&
&  0.68040E+00_JPRB, 0.80204E+00_JPRB, 0.85194E+00_JPRB, 0.90798E+00_JPRB, 0.94552E+00_JPRB, 0.97558E+00_JPRB, 0.10018E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.10856E+01_JPRB, 0.11419E+01_JPRB, 0.12180E+01_JPRB, 0.13063E+01_JPRB, 0.13817E+01_JPRB, 0.14516E+01_JPRB ,&
&  0.15423E+01_JPRB, 0.15326E+01_JPRB, 0.15715E+01_JPRB, 0.15942E+01_JPRB, 0.16384E+01_JPRB, 0.17347E+01_JPRB, 0.17843E+01_JPRB ,&
&  0.19013E+01_JPRB, 0.20729E+01_JPRB, 0.23492E+01_JPRB, 0.26145E+01_JPRB, 0.27401E+01_JPRB, 0.28683E+01_JPRB, 0.29041E+01_JPRB ,&
&  0.27561E+01_JPRB, 0.26699E+01_JPRB, 0.27329E+01_JPRB, 0.25074E+01_JPRB, 0.22585E+01_JPRB, 0.20467E+01_JPRB, 0.18955E+01_JPRB ,&
&  0.17497E+01_JPRB, 0.16094E+01_JPRB, 0.15184E+01_JPRB, 0.14437E+01_JPRB, 0.13675E+01_JPRB, 0.12997E+01_JPRB, 0.12861E+01_JPRB ,&
&  0.12745E+01_JPRB, 0.12592E+01_JPRB, 0.12354E+01_JPRB, 0.12073E+01_JPRB, 0.11598E+01_JPRB, 0.11130E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.10616E+01_JPRB, 0.10535E+01_JPRB, 0.10608E+01_JPRB, 0.10679E+01_JPRB, 0.10851E+01_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.11163E+01_JPRB, 0.11364E+01_JPRB, 0.11344E+01_JPRB, 0.11360E+01_JPRB, 0.11394E+01_JPRB, 0.10798E+01_JPRB, 0.10368E+01_JPRB ,&
&  0.10142E+01_JPRB, 0.10298E+01_JPRB, 0.10381E+01_JPRB, 0.10370E+01_JPRB, 0.10244E+01_JPRB, 0.10085E+01_JPRB, 0.99585E+00_JPRB ,&
&  0.98421E+00_JPRB, 0.95458E+00_JPRB, 0.89880E+00_JPRB, 0.82366E+00_JPRB, 0.79982E+00_JPRB, 0.79076E+00_JPRB, 0.79262E+00_JPRB ,&
&  0.79236E+00_JPRB, 0.78583E+00_JPRB, 0.78707E+00_JPRB, 0.77139E+00_JPRB, 0.75995E+00_JPRB, 0.72742E+00_JPRB, 0.70040E+00_JPRB ,&
&  0.69890E+00_JPRB, 0.69208E+00_JPRB, 0.70202E+00_JPRB, 0.68601E+00_JPRB, 0.66507E+00_JPRB, 0.65089E+00_JPRB, 0.59170E+00_JPRB ,&
&  0.58251E+00_JPRB, 0.56445E+00_JPRB, 0.55839E+00_JPRB, 0.55882E+00_JPRB, 0.58251E+00_JPRB, 0.60315E+00_JPRB, 0.63619E+00_JPRB ,&
&  0.66560E+00_JPRB, 0.79974E+00_JPRB, 0.84405E+00_JPRB, 0.87644E+00_JPRB, 0.90401E+00_JPRB, 0.94521E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.10717E+01_JPRB, 0.11384E+01_JPRB, 0.11963E+01_JPRB, 0.12903E+01_JPRB, 0.13862E+01_JPRB, 0.13989E+01_JPRB, 0.15389E+01_JPRB ,&
&  0.20045E+01_JPRB, 0.18968E+01_JPRB, 0.18054E+01_JPRB, 0.17699E+01_JPRB, 0.18029E+01_JPRB, 0.18592E+01_JPRB, 0.19518E+01_JPRB ,&
&  0.21559E+01_JPRB, 0.24747E+01_JPRB, 0.29380E+01_JPRB, 0.32507E+01_JPRB, 0.32603E+01_JPRB, 0.32897E+01_JPRB, 0.32282E+01_JPRB ,&
&  0.29926E+01_JPRB, 0.28289E+01_JPRB, 0.29227E+01_JPRB, 0.23984E+01_JPRB, 0.20309E+01_JPRB, 0.18325E+01_JPRB, 0.17397E+01_JPRB ,&
&  0.16475E+01_JPRB, 0.15539E+01_JPRB, 0.14989E+01_JPRB, 0.14275E+01_JPRB, 0.13345E+01_JPRB, 0.12802E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.12432E+01_JPRB, 0.12280E+01_JPRB, 0.12061E+01_JPRB, 0.11692E+01_JPRB, 0.11277E+01_JPRB, 0.11053E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.10827E+01_JPRB, 0.10742E+01_JPRB, 0.10741E+01_JPRB, 0.10871E+01_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.11071E+01_JPRB, 0.11232E+01_JPRB, 0.11202E+01_JPRB, 0.11004E+01_JPRB, 0.10747E+01_JPRB, 0.98863E+00_JPRB, 0.89770E+00_JPRB ,&
&  0.83266E+00_JPRB, 0.84065E+00_JPRB, 0.87832E+00_JPRB, 0.91076E+00_JPRB, 0.91495E+00_JPRB, 0.89330E+00_JPRB, 0.88185E+00_JPRB ,&
&  0.89033E+00_JPRB, 0.88937E+00_JPRB, 0.85326E+00_JPRB, 0.82453E+00_JPRB, 0.77845E+00_JPRB, 0.76677E+00_JPRB, 0.76759E+00_JPRB ,&
&  0.77104E+00_JPRB, 0.76950E+00_JPRB, 0.77117E+00_JPRB, 0.76970E+00_JPRB, 0.77026E+00_JPRB, 0.75799E+00_JPRB, 0.73611E+00_JPRB ,&
&  0.71085E+00_JPRB, 0.70529E+00_JPRB, 0.70657E+00_JPRB, 0.71031E+00_JPRB, 0.69999E+00_JPRB, 0.68326E+00_JPRB, 0.60571E+00_JPRB ,&
&  0.57631E+00_JPRB, 0.55618E+00_JPRB, 0.55560E+00_JPRB, 0.56959E+00_JPRB, 0.61184E+00_JPRB, 0.62753E+00_JPRB, 0.72202E+00_JPRB ,&
&  0.78896E+00_JPRB, 0.83604E+00_JPRB, 0.85045E+00_JPRB, 0.86084E+00_JPRB, 0.88892E+00_JPRB, 0.96012E+00_JPRB, 0.10567E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.11912E+01_JPRB, 0.12010E+01_JPRB, 0.10837E+01_JPRB, 0.11085E+01_JPRB, 0.12595E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.16915E+01_JPRB, 0.17481E+01_JPRB, 0.17634E+01_JPRB, 0.18681E+01_JPRB, 0.19101E+01_JPRB, 0.19455E+01_JPRB, 0.19775E+01_JPRB ,&
&  0.22568E+01_JPRB, 0.26529E+01_JPRB, 0.32523E+01_JPRB, 0.35591E+01_JPRB, 0.34664E+01_JPRB, 0.34821E+01_JPRB, 0.33254E+01_JPRB ,&
&  0.31982E+01_JPRB, 0.28298E+01_JPRB, 0.25004E+01_JPRB, 0.21233E+01_JPRB, 0.18218E+01_JPRB, 0.16483E+01_JPRB, 0.15456E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.13924E+01_JPRB, 0.13872E+01_JPRB, 0.13471E+01_JPRB, 0.12584E+01_JPRB, 0.11774E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10490E+01_JPRB, 0.10429E+01_JPRB, 0.10135E+01_JPRB, 0.10107E+01_JPRB, 0.10223E+01_JPRB, 0.10203E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.10104E+01_JPRB, 0.10249E+01_JPRB, 0.10476E+01_JPRB, 0.10704E+01_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.11417E+01_JPRB, 0.11142E+01_JPRB, 0.10738E+01_JPRB, 0.10627E+01_JPRB, 0.10290E+01_JPRB, 0.99730E+00_JPRB, 0.89407E+00_JPRB ,&
&  0.76898E+00_JPRB, 0.73929E+00_JPRB, 0.77737E+00_JPRB, 0.79757E+00_JPRB, 0.78221E+00_JPRB, 0.77897E+00_JPRB, 0.78943E+00_JPRB ,&
&  0.80025E+00_JPRB, 0.83478E+00_JPRB, 0.83232E+00_JPRB, 0.80831E+00_JPRB, 0.79012E+00_JPRB, 0.77850E+00_JPRB, 0.77845E+00_JPRB ,&
&  0.77652E+00_JPRB, 0.77425E+00_JPRB, 0.76440E+00_JPRB, 0.74903E+00_JPRB, 0.74709E+00_JPRB, 0.74488E+00_JPRB, 0.74036E+00_JPRB ,&
&  0.73651E+00_JPRB, 0.72700E+00_JPRB, 0.73149E+00_JPRB, 0.73232E+00_JPRB, 0.72017E+00_JPRB, 0.68442E+00_JPRB, 0.61069E+00_JPRB ,&
&  0.57604E+00_JPRB, 0.56075E+00_JPRB, 0.59204E+00_JPRB, 0.67580E+00_JPRB, 0.73443E+00_JPRB, 0.78485E+00_JPRB, 0.83290E+00_JPRB ,&
&  0.80993E+00_JPRB, 0.80631E+00_JPRB, 0.84571E+00_JPRB, 0.96732E+00_JPRB, 0.10310E+01_JPRB, 0.11152E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.13880E+01_JPRB, 0.14100E+01_JPRB, 0.12498E+01_JPRB, 0.13850E+01_JPRB, 0.17844E+01_JPRB, 0.17529E+01_JPRB ,&
&  0.18776E+01_JPRB, 0.20157E+01_JPRB, 0.21410E+01_JPRB, 0.21154E+01_JPRB, 0.20001E+01_JPRB, 0.20398E+01_JPRB, 0.22378E+01_JPRB ,&
&  0.25252E+01_JPRB, 0.29498E+01_JPRB, 0.37457E+01_JPRB, 0.41817E+01_JPRB, 0.40998E+01_JPRB, 0.40024E+01_JPRB, 0.38961E+01_JPRB ,&
&  0.38222E+01_JPRB, 0.34186E+01_JPRB, 0.29035E+01_JPRB, 0.24365E+01_JPRB, 0.20350E+01_JPRB, 0.17218E+01_JPRB, 0.15303E+01_JPRB ,&
&  0.14569E+01_JPRB, 0.14215E+01_JPRB, 0.14250E+01_JPRB, 0.13880E+01_JPRB, 0.12849E+01_JPRB, 0.11636E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.99883E+00_JPRB, 0.93690E+00_JPRB, 0.88115E+00_JPRB, 0.86342E+00_JPRB, 0.91581E+00_JPRB, 0.98406E+00_JPRB, 0.10042E+01_JPRB ,&
&  0.98367E+00_JPRB, 0.97536E+00_JPRB, 0.10263E+01_JPRB, 0.10822E+01_JPRB, 0.11046E+01_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.11926E+01_JPRB, 0.11496E+01_JPRB, 0.11080E+01_JPRB, 0.10674E+01_JPRB, 0.10265E+01_JPRB, 0.95732E+00_JPRB, 0.84805E+00_JPRB ,&
&  0.74481E+00_JPRB, 0.71513E+00_JPRB, 0.72338E+00_JPRB, 0.70564E+00_JPRB, 0.67961E+00_JPRB, 0.66606E+00_JPRB, 0.67346E+00_JPRB ,&
&  0.69316E+00_JPRB, 0.73888E+00_JPRB, 0.77368E+00_JPRB, 0.78605E+00_JPRB, 0.78208E+00_JPRB, 0.77428E+00_JPRB, 0.78118E+00_JPRB ,&
&  0.79257E+00_JPRB, 0.80393E+00_JPRB, 0.80802E+00_JPRB, 0.79794E+00_JPRB, 0.78708E+00_JPRB, 0.78173E+00_JPRB, 0.77586E+00_JPRB ,&
&  0.77202E+00_JPRB, 0.76555E+00_JPRB, 0.74598E+00_JPRB, 0.74317E+00_JPRB, 0.72010E+00_JPRB, 0.71051E+00_JPRB, 0.66373E+00_JPRB ,&
&  0.60798E+00_JPRB, 0.61654E+00_JPRB, 0.74977E+00_JPRB, 0.80428E+00_JPRB, 0.82232E+00_JPRB, 0.86287E+00_JPRB, 0.92056E+00_JPRB ,&
&  0.96988E+00_JPRB, 0.10119E+01_JPRB, 0.10542E+01_JPRB, 0.11075E+01_JPRB, 0.11795E+01_JPRB, 0.13129E+01_JPRB, 0.14723E+01_JPRB ,&
&  0.16718E+01_JPRB, 0.14603E+01_JPRB, 0.13990E+01_JPRB, 0.15724E+01_JPRB, 0.21361E+01_JPRB, 0.24564E+01_JPRB, 0.25368E+01_JPRB ,&
&  0.25854E+01_JPRB, 0.25112E+01_JPRB, 0.24427E+01_JPRB, 0.24553E+01_JPRB, 0.25780E+01_JPRB, 0.28017E+01_JPRB, 0.31603E+01_JPRB ,&
&  0.34632E+01_JPRB, 0.40008E+01_JPRB, 0.47879E+01_JPRB, 0.49693E+01_JPRB, 0.48625E+01_JPRB, 0.47697E+01_JPRB, 0.45808E+01_JPRB ,&
&  0.43651E+01_JPRB, 0.39435E+01_JPRB, 0.32714E+01_JPRB, 0.27080E+01_JPRB, 0.22342E+01_JPRB, 0.18770E+01_JPRB, 0.15943E+01_JPRB ,&
&  0.14610E+01_JPRB, 0.14348E+01_JPRB, 0.14305E+01_JPRB, 0.13983E+01_JPRB, 0.13289E+01_JPRB, 0.12310E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.98447E+00_JPRB, 0.91484E+00_JPRB, 0.85515E+00_JPRB, 0.88339E+00_JPRB, 0.96609E+00_JPRB, 0.10506E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.10673E+01_JPRB, 0.10716E+01_JPRB, 0.11184E+01_JPRB, 0.11742E+01_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.11963E+01_JPRB, 0.11484E+01_JPRB, 0.10862E+01_JPRB, 0.10117E+01_JPRB, 0.94285E+00_JPRB, 0.88543E+00_JPRB, 0.81734E+00_JPRB ,&
&  0.84793E+00_JPRB, 0.77100E+00_JPRB, 0.83835E+00_JPRB, 0.72132E+00_JPRB, 0.68877E+00_JPRB, 0.67892E+00_JPRB, 0.66091E+00_JPRB ,&
&  0.65455E+00_JPRB, 0.66991E+00_JPRB, 0.69217E+00_JPRB, 0.71581E+00_JPRB, 0.72099E+00_JPRB, 0.72372E+00_JPRB, 0.73799E+00_JPRB ,&
&  0.76144E+00_JPRB, 0.79490E+00_JPRB, 0.83158E+00_JPRB, 0.84596E+00_JPRB, 0.84830E+00_JPRB, 0.84000E+00_JPRB, 0.83587E+00_JPRB ,&
&  0.78710E+00_JPRB, 0.77007E+00_JPRB, 0.76820E+00_JPRB, 0.76166E+00_JPRB, 0.75465E+00_JPRB, 0.73568E+00_JPRB, 0.74173E+00_JPRB ,&
&  0.72014E+00_JPRB, 0.69673E+00_JPRB, 0.81384E+00_JPRB, 0.84145E+00_JPRB, 0.88134E+00_JPRB, 0.92519E+00_JPRB, 0.97013E+00_JPRB ,&
&  0.10136E+01_JPRB, 0.10588E+01_JPRB, 0.11101E+01_JPRB, 0.11635E+01_JPRB, 0.12884E+01_JPRB, 0.15171E+01_JPRB, 0.17467E+01_JPRB ,&
&  0.19889E+01_JPRB, 0.16472E+01_JPRB, 0.18765E+01_JPRB, 0.20804E+01_JPRB, 0.25467E+01_JPRB, 0.30161E+01_JPRB, 0.31974E+01_JPRB ,&
&  0.35148E+01_JPRB, 0.34674E+01_JPRB, 0.33797E+01_JPRB, 0.32982E+01_JPRB, 0.35293E+01_JPRB, 0.38697E+01_JPRB, 0.42485E+01_JPRB ,&
&  0.46736E+01_JPRB, 0.51732E+01_JPRB, 0.59987E+01_JPRB, 0.59479E+01_JPRB, 0.57627E+01_JPRB, 0.55516E+01_JPRB, 0.52063E+01_JPRB ,&
&  0.47757E+01_JPRB, 0.42556E+01_JPRB, 0.36121E+01_JPRB, 0.28926E+01_JPRB, 0.24033E+01_JPRB, 0.19949E+01_JPRB, 0.16634E+01_JPRB ,&
&  0.14426E+01_JPRB, 0.13286E+01_JPRB, 0.13229E+01_JPRB, 0.13001E+01_JPRB, 0.12690E+01_JPRB, 0.11948E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.94887E+00_JPRB, 0.96939E+00_JPRB, 0.10260E+01_JPRB, 0.11559E+01_JPRB, 0.12179E+01_JPRB, 0.12250E+01_JPRB, 0.12745E+01_JPRB ,&
&  0.11732E+01_JPRB, 0.11935E+01_JPRB, 0.12636E+01_JPRB, 0.12413E+01_JPRB, 0.12257E+01_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.12391E+01_JPRB, 0.11541E+01_JPRB, 0.10224E+01_JPRB, 0.94566E+00_JPRB, 0.93059E+00_JPRB, 0.91116E+00_JPRB, 0.87039E+00_JPRB ,&
&  0.89270E+00_JPRB, 0.87490E+00_JPRB, 0.89369E+00_JPRB, 0.92264E+00_JPRB, 0.94742E+00_JPRB, 0.85056E+00_JPRB, 0.72093E+00_JPRB ,&
&  0.68721E+00_JPRB, 0.71127E+00_JPRB, 0.74211E+00_JPRB, 0.75696E+00_JPRB, 0.78021E+00_JPRB, 0.78207E+00_JPRB, 0.78983E+00_JPRB ,&
&  0.79754E+00_JPRB, 0.80981E+00_JPRB, 0.84893E+00_JPRB, 0.86235E+00_JPRB, 0.85703E+00_JPRB, 0.85526E+00_JPRB, 0.86088E+00_JPRB ,&
&  0.80980E+00_JPRB, 0.78845E+00_JPRB, 0.77299E+00_JPRB, 0.74319E+00_JPRB, 0.78736E+00_JPRB, 0.79958E+00_JPRB, 0.80530E+00_JPRB ,&
&  0.81265E+00_JPRB, 0.83149E+00_JPRB, 0.86733E+00_JPRB, 0.90965E+00_JPRB, 0.95572E+00_JPRB, 0.10002E+01_JPRB, 0.10389E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.11102E+01_JPRB, 0.11532E+01_JPRB, 0.12372E+01_JPRB, 0.14823E+01_JPRB, 0.16982E+01_JPRB, 0.20749E+01_JPRB ,&
&  0.22576E+01_JPRB, 0.25024E+01_JPRB, 0.28540E+01_JPRB, 0.28397E+01_JPRB, 0.30751E+01_JPRB, 0.34059E+01_JPRB, 0.33666E+01_JPRB ,&
&  0.34734E+01_JPRB, 0.34633E+01_JPRB, 0.36903E+01_JPRB, 0.40418E+01_JPRB, 0.45204E+01_JPRB, 0.49279E+01_JPRB, 0.52576E+01_JPRB ,&
&  0.57665E+01_JPRB, 0.66571E+01_JPRB, 0.69795E+01_JPRB, 0.66644E+01_JPRB, 0.63121E+01_JPRB, 0.59774E+01_JPRB, 0.54796E+01_JPRB ,&
&  0.49716E+01_JPRB, 0.43938E+01_JPRB, 0.37157E+01_JPRB, 0.30244E+01_JPRB, 0.24064E+01_JPRB, 0.20665E+01_JPRB, 0.17325E+01_JPRB ,&
&  0.14207E+01_JPRB, 0.12150E+01_JPRB, 0.12262E+01_JPRB, 0.12542E+01_JPRB, 0.12392E+01_JPRB, 0.11354E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.11863E+01_JPRB, 0.15266E+01_JPRB, 0.16449E+01_JPRB, 0.16654E+01_JPRB, 0.15744E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.12929E+01_JPRB, 0.12885E+01_JPRB, 0.12978E+01_JPRB, 0.12851E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.12981E+01_JPRB, 0.11671E+01_JPRB, 0.98900E+00_JPRB, 0.92937E+00_JPRB, 0.94348E+00_JPRB, 0.93353E+00_JPRB, 0.91993E+00_JPRB ,&
&  0.92869E+00_JPRB, 0.93215E+00_JPRB, 0.93670E+00_JPRB, 0.94841E+00_JPRB, 0.96064E+00_JPRB, 0.99233E+00_JPRB, 0.90340E+00_JPRB ,&
&  0.81451E+00_JPRB, 0.82588E+00_JPRB, 0.80967E+00_JPRB, 0.85092E+00_JPRB, 0.98787E+00_JPRB, 0.99978E+00_JPRB, 0.10023E+01_JPRB ,&
&  0.10248E+01_JPRB, 0.98834E+00_JPRB, 0.98185E+00_JPRB, 0.95204E+00_JPRB, 0.90930E+00_JPRB, 0.90687E+00_JPRB, 0.90472E+00_JPRB ,&
&  0.82599E+00_JPRB, 0.77412E+00_JPRB, 0.74624E+00_JPRB, 0.73518E+00_JPRB, 0.78065E+00_JPRB, 0.84237E+00_JPRB, 0.86904E+00_JPRB ,&
&  0.88085E+00_JPRB, 0.90555E+00_JPRB, 0.94561E+00_JPRB, 0.99634E+00_JPRB, 0.10494E+01_JPRB, 0.10920E+01_JPRB, 0.11149E+01_JPRB ,&
&  0.11366E+01_JPRB, 0.11647E+01_JPRB, 0.12070E+01_JPRB, 0.13234E+01_JPRB, 0.15027E+01_JPRB, 0.18990E+01_JPRB, 0.24926E+01_JPRB ,&
&  0.28060E+01_JPRB, 0.30310E+01_JPRB, 0.34573E+01_JPRB, 0.40208E+01_JPRB, 0.43627E+01_JPRB, 0.42181E+01_JPRB, 0.40415E+01_JPRB ,&
&  0.40870E+01_JPRB, 0.41423E+01_JPRB, 0.48144E+01_JPRB, 0.53167E+01_JPRB, 0.56591E+01_JPRB, 0.60581E+01_JPRB, 0.62440E+01_JPRB ,&
&  0.66312E+01_JPRB, 0.70140E+01_JPRB, 0.70171E+01_JPRB, 0.66799E+01_JPRB, 0.63507E+01_JPRB, 0.60525E+01_JPRB, 0.55208E+01_JPRB ,&
&  0.49796E+01_JPRB, 0.42950E+01_JPRB, 0.34565E+01_JPRB, 0.27771E+01_JPRB, 0.21501E+01_JPRB, 0.19932E+01_JPRB, 0.17968E+01_JPRB ,&
&  0.14229E+01_JPRB, 0.12458E+01_JPRB, 0.12790E+01_JPRB, 0.14092E+01_JPRB, 0.14031E+01_JPRB, 0.12705E+01_JPRB, 0.11612E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.14369E+01_JPRB, 0.16875E+01_JPRB, 0.18737E+01_JPRB, 0.18878E+01_JPRB, 0.17269E+01_JPRB, 0.12873E+01_JPRB ,&
&  0.12282E+01_JPRB, 0.13552E+01_JPRB, 0.13649E+01_JPRB, 0.13618E+01_JPRB, 0.13486E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.13858E+01_JPRB, 0.12157E+01_JPRB, 0.10537E+01_JPRB, 0.96082E+00_JPRB, 0.96418E+00_JPRB, 0.96745E+00_JPRB, 0.96516E+00_JPRB ,&
&  0.98143E+00_JPRB, 0.99631E+00_JPRB, 0.10058E+01_JPRB, 0.10118E+01_JPRB, 0.10140E+01_JPRB, 0.10155E+01_JPRB, 0.10547E+01_JPRB ,&
&  0.91466E+00_JPRB, 0.93478E+00_JPRB, 0.90315E+00_JPRB, 0.99241E+00_JPRB, 0.13861E+01_JPRB, 0.13758E+01_JPRB, 0.14265E+01_JPRB ,&
&  0.13768E+01_JPRB, 0.13518E+01_JPRB, 0.12869E+01_JPRB, 0.12086E+01_JPRB, 0.11456E+01_JPRB, 0.11290E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.93229E+00_JPRB, 0.85264E+00_JPRB, 0.83574E+00_JPRB, 0.83909E+00_JPRB, 0.88389E+00_JPRB, 0.91273E+00_JPRB, 0.98409E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.10607E+01_JPRB, 0.11159E+01_JPRB, 0.11664E+01_JPRB, 0.11955E+01_JPRB, 0.12174E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.13228E+01_JPRB, 0.13962E+01_JPRB, 0.15500E+01_JPRB, 0.16404E+01_JPRB, 0.21381E+01_JPRB, 0.32468E+01_JPRB ,&
&  0.40160E+01_JPRB, 0.41405E+01_JPRB, 0.51839E+01_JPRB, 0.69591E+01_JPRB, 0.70645E+01_JPRB, 0.65562E+01_JPRB, 0.60211E+01_JPRB ,&
&  0.57653E+01_JPRB, 0.56243E+01_JPRB, 0.58723E+01_JPRB, 0.62415E+01_JPRB, 0.65005E+01_JPRB, 0.69069E+01_JPRB, 0.70181E+01_JPRB ,&
&  0.68899E+01_JPRB, 0.67574E+01_JPRB, 0.65329E+01_JPRB, 0.62542E+01_JPRB, 0.58194E+01_JPRB, 0.55283E+01_JPRB, 0.51538E+01_JPRB ,&
&  0.46854E+01_JPRB, 0.40588E+01_JPRB, 0.31010E+01_JPRB, 0.25289E+01_JPRB, 0.18474E+01_JPRB, 0.16843E+01_JPRB, 0.20859E+01_JPRB ,&
&  0.15662E+01_JPRB, 0.13932E+01_JPRB, 0.14985E+01_JPRB, 0.17146E+01_JPRB, 0.17683E+01_JPRB, 0.17496E+01_JPRB, 0.15125E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.14218E+01_JPRB, 0.19427E+01_JPRB, 0.21022E+01_JPRB, 0.20705E+01_JPRB, 0.18867E+01_JPRB, 0.15312E+01_JPRB ,&
&  0.15587E+01_JPRB, 0.14888E+01_JPRB, 0.14768E+01_JPRB, 0.14712E+01_JPRB, 0.14544E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.14487E+01_JPRB, 0.12729E+01_JPRB, 0.11283E+01_JPRB, 0.10229E+01_JPRB, 0.10070E+01_JPRB, 0.10242E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10654E+01_JPRB, 0.10679E+01_JPRB, 0.10718E+01_JPRB, 0.10705E+01_JPRB, 0.10437E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.10221E+01_JPRB, 0.10712E+01_JPRB, 0.10569E+01_JPRB, 0.10508E+01_JPRB, 0.13443E+01_JPRB, 0.14933E+01_JPRB, 0.16810E+01_JPRB ,&
&  0.17946E+01_JPRB, 0.19440E+01_JPRB, 0.18993E+01_JPRB, 0.17956E+01_JPRB, 0.16981E+01_JPRB, 0.16273E+01_JPRB, 0.14545E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.11790E+01_JPRB, 0.11397E+01_JPRB, 0.11971E+01_JPRB, 0.12782E+01_JPRB, 0.11392E+01_JPRB, 0.12373E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.13461E+01_JPRB, 0.13836E+01_JPRB, 0.14057E+01_JPRB, 0.14525E+01_JPRB, 0.15123E+01_JPRB, 0.15746E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.16582E+01_JPRB, 0.16567E+01_JPRB, 0.17562E+01_JPRB, 0.18875E+01_JPRB, 0.22166E+01_JPRB, 0.31147E+01_JPRB ,&
&  0.43082E+01_JPRB, 0.48639E+01_JPRB, 0.63310E+01_JPRB, 0.85358E+01_JPRB, 0.85803E+01_JPRB, 0.80295E+01_JPRB, 0.78334E+01_JPRB ,&
&  0.76623E+01_JPRB, 0.73933E+01_JPRB, 0.72141E+01_JPRB, 0.70171E+01_JPRB, 0.71856E+01_JPRB, 0.74686E+01_JPRB, 0.73490E+01_JPRB ,&
&  0.67877E+01_JPRB, 0.61981E+01_JPRB, 0.58826E+01_JPRB, 0.54606E+01_JPRB, 0.50865E+01_JPRB, 0.46209E+01_JPRB, 0.46355E+01_JPRB ,&
&  0.46748E+01_JPRB, 0.32089E+01_JPRB, 0.25572E+01_JPRB, 0.24163E+01_JPRB, 0.17717E+01_JPRB, 0.15445E+01_JPRB, 0.17516E+01_JPRB ,&
&  0.14512E+01_JPRB, 0.13471E+01_JPRB, 0.16375E+01_JPRB, 0.19302E+01_JPRB, 0.22672E+01_JPRB, 0.24682E+01_JPRB, 0.23927E+01_JPRB ,&
&  0.20440E+01_JPRB, 0.19941E+01_JPRB, 0.25277E+01_JPRB, 0.25483E+01_JPRB, 0.23278E+01_JPRB, 0.20765E+01_JPRB, 0.18478E+01_JPRB ,&
&  0.17327E+01_JPRB, 0.17166E+01_JPRB, 0.16722E+01_JPRB, 0.16341E+01_JPRB, 0.15900E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.15902E+01_JPRB, 0.14116E+01_JPRB, 0.12392E+01_JPRB, 0.11386E+01_JPRB, 0.11236E+01_JPRB, 0.11362E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.11327E+01_JPRB, 0.11050E+01_JPRB, 0.11065E+01_JPRB, 0.11036E+01_JPRB, 0.10952E+01_JPRB, 0.10634E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.99644E+00_JPRB, 0.11295E+01_JPRB, 0.10658E+01_JPRB, 0.11072E+01_JPRB, 0.12005E+01_JPRB, 0.15167E+01_JPRB, 0.17954E+01_JPRB ,&
&  0.20845E+01_JPRB, 0.22739E+01_JPRB, 0.24294E+01_JPRB, 0.26211E+01_JPRB, 0.26804E+01_JPRB, 0.25512E+01_JPRB, 0.23887E+01_JPRB ,&
&  0.22539E+01_JPRB, 0.22258E+01_JPRB, 0.19395E+01_JPRB, 0.20461E+01_JPRB, 0.19801E+01_JPRB, 0.17370E+01_JPRB, 0.19023E+01_JPRB ,&
&  0.18737E+01_JPRB, 0.18968E+01_JPRB, 0.18904E+01_JPRB, 0.19042E+01_JPRB, 0.19302E+01_JPRB, 0.19264E+01_JPRB, 0.19016E+01_JPRB ,&
&  0.18823E+01_JPRB, 0.18831E+01_JPRB, 0.19215E+01_JPRB, 0.19221E+01_JPRB, 0.20418E+01_JPRB, 0.25206E+01_JPRB, 0.34232E+01_JPRB ,&
&  0.46194E+01_JPRB, 0.48499E+01_JPRB, 0.52735E+01_JPRB, 0.77979E+01_JPRB, 0.96836E+01_JPRB, 0.93701E+01_JPRB, 0.89905E+01_JPRB ,&
&  0.89615E+01_JPRB, 0.83521E+01_JPRB, 0.77076E+01_JPRB, 0.73799E+01_JPRB, 0.77875E+01_JPRB, 0.77361E+01_JPRB, 0.73294E+01_JPRB ,&
&  0.66613E+01_JPRB, 0.60102E+01_JPRB, 0.56827E+01_JPRB, 0.53292E+01_JPRB, 0.47303E+01_JPRB, 0.44748E+01_JPRB, 0.45350E+01_JPRB ,&
&  0.44515E+01_JPRB, 0.38340E+01_JPRB, 0.28624E+01_JPRB, 0.25538E+01_JPRB, 0.20522E+01_JPRB, 0.18529E+01_JPRB, 0.20399E+01_JPRB ,&
&  0.18028E+01_JPRB, 0.18403E+01_JPRB, 0.20121E+01_JPRB, 0.21637E+01_JPRB, 0.34979E+01_JPRB, 0.38896E+01_JPRB, 0.41938E+01_JPRB ,&
&  0.37416E+01_JPRB, 0.32169E+01_JPRB, 0.36964E+01_JPRB, 0.34664E+01_JPRB, 0.29411E+01_JPRB, 0.24379E+01_JPRB, 0.20769E+01_JPRB ,&
&  0.19507E+01_JPRB, 0.19544E+01_JPRB, 0.18826E+01_JPRB, 0.18129E+01_JPRB, 0.17559E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.17215E+01_JPRB, 0.15308E+01_JPRB, 0.13559E+01_JPRB, 0.12441E+01_JPRB, 0.11984E+01_JPRB, 0.11969E+01_JPRB, 0.11871E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.11185E+01_JPRB, 0.11066E+01_JPRB, 0.10831E+01_JPRB, 0.10573E+01_JPRB, 0.10311E+01_JPRB, 0.99585E+00_JPRB ,&
&  0.98150E+00_JPRB, 0.10927E+01_JPRB, 0.10510E+01_JPRB, 0.10746E+01_JPRB, 0.12245E+01_JPRB, 0.16612E+01_JPRB, 0.21182E+01_JPRB ,&
&  0.24546E+01_JPRB, 0.27046E+01_JPRB, 0.30356E+01_JPRB, 0.34710E+01_JPRB, 0.38888E+01_JPRB, 0.40776E+01_JPRB, 0.40700E+01_JPRB ,&
&  0.44569E+01_JPRB, 0.37413E+01_JPRB, 0.37851E+01_JPRB, 0.34332E+01_JPRB, 0.32207E+01_JPRB, 0.30052E+01_JPRB, 0.28245E+01_JPRB ,&
&  0.26707E+01_JPRB, 0.25271E+01_JPRB, 0.23794E+01_JPRB, 0.22832E+01_JPRB, 0.22254E+01_JPRB, 0.21898E+01_JPRB, 0.21947E+01_JPRB ,&
&  0.22214E+01_JPRB, 0.21741E+01_JPRB, 0.21097E+01_JPRB, 0.20994E+01_JPRB, 0.26173E+01_JPRB, 0.37154E+01_JPRB, 0.46552E+01_JPRB ,&
&  0.46629E+01_JPRB, 0.46804E+01_JPRB, 0.84216E+01_JPRB, 0.93803E+01_JPRB, 0.91152E+01_JPRB, 0.99341E+01_JPRB, 0.10040E+02_JPRB ,&
&  0.91712E+01_JPRB, 0.79019E+01_JPRB, 0.68287E+01_JPRB, 0.66536E+01_JPRB, 0.80355E+01_JPRB, 0.78063E+01_JPRB, 0.72769E+01_JPRB ,&
&  0.65532E+01_JPRB, 0.60607E+01_JPRB, 0.54770E+01_JPRB, 0.50233E+01_JPRB, 0.47532E+01_JPRB, 0.49271E+01_JPRB, 0.45839E+01_JPRB ,&
&  0.43150E+01_JPRB, 0.46818E+01_JPRB, 0.38205E+01_JPRB, 0.29689E+01_JPRB, 0.25765E+01_JPRB, 0.25025E+01_JPRB, 0.25176E+01_JPRB ,&
&  0.24469E+01_JPRB, 0.25323E+01_JPRB, 0.26772E+01_JPRB, 0.34859E+01_JPRB, 0.54643E+01_JPRB, 0.55925E+01_JPRB, 0.52913E+01_JPRB ,&
&  0.47932E+01_JPRB, 0.53560E+01_JPRB, 0.47563E+01_JPRB, 0.40942E+01_JPRB, 0.33937E+01_JPRB, 0.28150E+01_JPRB, 0.24008E+01_JPRB ,&
&  0.21661E+01_JPRB, 0.21347E+01_JPRB, 0.20777E+01_JPRB, 0.19806E+01_JPRB, 0.19022E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.18937E+01_JPRB, 0.16609E+01_JPRB, 0.15300E+01_JPRB, 0.14087E+01_JPRB, 0.13386E+01_JPRB, 0.12779E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.10825E+01_JPRB, 0.10591E+01_JPRB, 0.10409E+01_JPRB, 0.10231E+01_JPRB, 0.99375E+00_JPRB, 0.96821E+00_JPRB ,&
&  0.98711E+00_JPRB, 0.10672E+01_JPRB, 0.10695E+01_JPRB, 0.12161E+01_JPRB, 0.14278E+01_JPRB, 0.17769E+01_JPRB, 0.22416E+01_JPRB ,&
&  0.25778E+01_JPRB, 0.28899E+01_JPRB, 0.32498E+01_JPRB, 0.41804E+01_JPRB, 0.52895E+01_JPRB, 0.62687E+01_JPRB, 0.64830E+01_JPRB ,&
&  0.60497E+01_JPRB, 0.60934E+01_JPRB, 0.57501E+01_JPRB, 0.51750E+01_JPRB, 0.46784E+01_JPRB, 0.42939E+01_JPRB, 0.38662E+01_JPRB ,&
&  0.34147E+01_JPRB, 0.29858E+01_JPRB, 0.26561E+01_JPRB, 0.24629E+01_JPRB, 0.23409E+01_JPRB, 0.23304E+01_JPRB, 0.23086E+01_JPRB ,&
&  0.22584E+01_JPRB, 0.22566E+01_JPRB, 0.22190E+01_JPRB, 0.26267E+01_JPRB, 0.36809E+01_JPRB, 0.51209E+01_JPRB, 0.53904E+01_JPRB ,&
&  0.60055E+01_JPRB, 0.60898E+01_JPRB, 0.76104E+01_JPRB, 0.87973E+01_JPRB, 0.73673E+01_JPRB, 0.84499E+01_JPRB, 0.90050E+01_JPRB ,&
&  0.80429E+01_JPRB, 0.73503E+01_JPRB, 0.77702E+01_JPRB, 0.78337E+01_JPRB, 0.74427E+01_JPRB, 0.86434E+01_JPRB, 0.79565E+01_JPRB ,&
&  0.68335E+01_JPRB, 0.60875E+01_JPRB, 0.52646E+01_JPRB, 0.51431E+01_JPRB, 0.44562E+01_JPRB, 0.43693E+01_JPRB, 0.35926E+01_JPRB ,&
&  0.38595E+01_JPRB, 0.33198E+01_JPRB, 0.35452E+01_JPRB, 0.31473E+01_JPRB, 0.31249E+01_JPRB, 0.29580E+01_JPRB, 0.28464E+01_JPRB ,&
&  0.29814E+01_JPRB, 0.32187E+01_JPRB, 0.38105E+01_JPRB, 0.56078E+01_JPRB, 0.76329E+01_JPRB, 0.62203E+01_JPRB, 0.67678E+01_JPRB ,&
&  0.76730E+01_JPRB, 0.72574E+01_JPRB, 0.53380E+01_JPRB, 0.45430E+01_JPRB, 0.37079E+01_JPRB, 0.31223E+01_JPRB, 0.27876E+01_JPRB ,&
&  0.24973E+01_JPRB, 0.23598E+01_JPRB, 0.22756E+01_JPRB, 0.21861E+01_JPRB, 0.20899E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.19634E+01_JPRB, 0.17353E+01_JPRB, 0.16208E+01_JPRB, 0.15654E+01_JPRB, 0.14611E+01_JPRB, 0.12830E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.10841E+01_JPRB, 0.10182E+01_JPRB, 0.98074E+00_JPRB, 0.97714E+00_JPRB, 0.98454E+00_JPRB, 0.97517E+00_JPRB ,&
&  0.10274E+01_JPRB, 0.11848E+01_JPRB, 0.12285E+01_JPRB, 0.13187E+01_JPRB, 0.15784E+01_JPRB, 0.19173E+01_JPRB, 0.21406E+01_JPRB ,&
&  0.28426E+01_JPRB, 0.31254E+01_JPRB, 0.34169E+01_JPRB, 0.46001E+01_JPRB, 0.63324E+01_JPRB, 0.79863E+01_JPRB, 0.85379E+01_JPRB ,&
&  0.91345E+01_JPRB, 0.86754E+01_JPRB, 0.69414E+01_JPRB, 0.56887E+01_JPRB, 0.48217E+01_JPRB, 0.42907E+01_JPRB, 0.37404E+01_JPRB ,&
&  0.31130E+01_JPRB, 0.25789E+01_JPRB, 0.23091E+01_JPRB, 0.21470E+01_JPRB, 0.20705E+01_JPRB, 0.19487E+01_JPRB, 0.19662E+01_JPRB ,&
&  0.20736E+01_JPRB, 0.22234E+01_JPRB, 0.23625E+01_JPRB, 0.30313E+01_JPRB, 0.45376E+01_JPRB, 0.57930E+01_JPRB, 0.61211E+01_JPRB ,&
&  0.58674E+01_JPRB, 0.57748E+01_JPRB, 0.71985E+01_JPRB, 0.78977E+01_JPRB, 0.83976E+01_JPRB, 0.81121E+01_JPRB, 0.74603E+01_JPRB ,&
&  0.58576E+01_JPRB, 0.55272E+01_JPRB, 0.59882E+01_JPRB, 0.65347E+01_JPRB, 0.73127E+01_JPRB, 0.89997E+01_JPRB, 0.86284E+01_JPRB ,&
&  0.72833E+01_JPRB, 0.63395E+01_JPRB, 0.55650E+01_JPRB, 0.47312E+01_JPRB, 0.49077E+01_JPRB, 0.36787E+01_JPRB, 0.43760E+01_JPRB ,&
&  0.45756E+01_JPRB, 0.43668E+01_JPRB, 0.36010E+01_JPRB, 0.30005E+01_JPRB, 0.26721E+01_JPRB, 0.32039E+01_JPRB, 0.33088E+01_JPRB ,&
&  0.38519E+01_JPRB, 0.43694E+01_JPRB, 0.77165E+01_JPRB, 0.90611E+01_JPRB, 0.87901E+01_JPRB, 0.74112E+01_JPRB, 0.88072E+01_JPRB ,&
&  0.81109E+01_JPRB, 0.70684E+01_JPRB, 0.56803E+01_JPRB, 0.46832E+01_JPRB, 0.38605E+01_JPRB, 0.33612E+01_JPRB, 0.30470E+01_JPRB ,&
&  0.27227E+01_JPRB, 0.25592E+01_JPRB, 0.24412E+01_JPRB, 0.23264E+01_JPRB, 0.21872E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.18759E+01_JPRB, 0.17455E+01_JPRB, 0.16903E+01_JPRB, 0.15510E+01_JPRB, 0.13439E+01_JPRB, 0.11997E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.10976E+01_JPRB, 0.10425E+01_JPRB, 0.98898E+00_JPRB, 0.96184E+00_JPRB, 0.97770E+00_JPRB, 0.10021E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.11381E+01_JPRB, 0.14446E+01_JPRB, 0.14265E+01_JPRB, 0.15448E+01_JPRB, 0.17642E+01_JPRB, 0.21032E+01_JPRB, 0.23832E+01_JPRB ,&
&  0.30783E+01_JPRB, 0.29948E+01_JPRB, 0.29726E+01_JPRB, 0.40247E+01_JPRB, 0.68383E+01_JPRB, 0.80708E+01_JPRB, 0.87669E+01_JPRB ,&
&  0.85449E+01_JPRB, 0.71217E+01_JPRB, 0.57113E+01_JPRB, 0.45487E+01_JPRB, 0.37922E+01_JPRB, 0.32949E+01_JPRB, 0.26655E+01_JPRB ,&
&  0.22317E+01_JPRB, 0.20190E+01_JPRB, 0.19651E+01_JPRB, 0.19177E+01_JPRB, 0.18625E+01_JPRB, 0.18597E+01_JPRB, 0.19452E+01_JPRB ,&
&  0.20719E+01_JPRB, 0.22149E+01_JPRB, 0.24137E+01_JPRB, 0.32016E+01_JPRB, 0.46436E+01_JPRB, 0.56895E+01_JPRB, 0.66656E+01_JPRB ,&
&  0.58792E+01_JPRB, 0.60876E+01_JPRB, 0.71595E+01_JPRB, 0.88461E+01_JPRB, 0.78873E+01_JPRB, 0.75191E+01_JPRB, 0.67762E+01_JPRB ,&
&  0.56481E+01_JPRB, 0.58510E+01_JPRB, 0.69674E+01_JPRB, 0.74153E+01_JPRB, 0.63964E+01_JPRB, 0.72231E+01_JPRB, 0.89775E+01_JPRB ,&
&  0.67592E+01_JPRB, 0.60416E+01_JPRB, 0.57057E+01_JPRB, 0.58611E+01_JPRB, 0.37795E+01_JPRB, 0.25574E+01_JPRB, 0.40091E+01_JPRB ,&
&  0.41149E+01_JPRB, 0.22659E+01_JPRB, 0.16382E+01_JPRB, 0.20350E+01_JPRB, 0.15444E+01_JPRB, 0.20259E+01_JPRB, 0.35502E+01_JPRB ,&
&  0.47945E+01_JPRB, 0.59541E+01_JPRB, 0.12301E+02_JPRB, 0.11065E+02_JPRB, 0.10526E+02_JPRB, 0.92617E+01_JPRB, 0.98452E+01_JPRB ,&
&  0.87603E+01_JPRB, 0.57502E+01_JPRB, 0.52404E+01_JPRB, 0.45660E+01_JPRB, 0.36981E+01_JPRB, 0.30747E+01_JPRB, 0.26434E+01_JPRB ,&
&  0.23120E+01_JPRB, 0.21274E+01_JPRB, 0.20317E+01_JPRB, 0.19946E+01_JPRB, 0.19951E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.15835E+01_JPRB, 0.15300E+01_JPRB, 0.14548E+01_JPRB, 0.12997E+01_JPRB, 0.11839E+01_JPRB, 0.11247E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.98718E+00_JPRB, 0.96601E+00_JPRB, 0.98678E+00_JPRB, 0.10133E+01_JPRB, 0.10434E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.13618E+01_JPRB, 0.16941E+01_JPRB, 0.19906E+01_JPRB, 0.22407E+01_JPRB, 0.21874E+01_JPRB, 0.27399E+01_JPRB ,&
&  0.29617E+01_JPRB, 0.26598E+01_JPRB, 0.28488E+01_JPRB, 0.38847E+01_JPRB, 0.55502E+01_JPRB, 0.65143E+01_JPRB, 0.67773E+01_JPRB ,&
&  0.59048E+01_JPRB, 0.49228E+01_JPRB, 0.42452E+01_JPRB, 0.38017E+01_JPRB, 0.31981E+01_JPRB, 0.24579E+01_JPRB, 0.20557E+01_JPRB ,&
&  0.18620E+01_JPRB, 0.18730E+01_JPRB, 0.19107E+01_JPRB, 0.18613E+01_JPRB, 0.18869E+01_JPRB, 0.19210E+01_JPRB, 0.20247E+01_JPRB ,&
&  0.21236E+01_JPRB, 0.22428E+01_JPRB, 0.25511E+01_JPRB, 0.36480E+01_JPRB, 0.55794E+01_JPRB, 0.68456E+01_JPRB, 0.59404E+01_JPRB ,&
&  0.50201E+01_JPRB, 0.50731E+01_JPRB, 0.73897E+01_JPRB, 0.80718E+01_JPRB, 0.71730E+01_JPRB, 0.72813E+01_JPRB, 0.65583E+01_JPRB ,&
&  0.66920E+01_JPRB, 0.70585E+01_JPRB, 0.65682E+01_JPRB, 0.69746E+01_JPRB, 0.69639E+01_JPRB, 0.65478E+01_JPRB, 0.70708E+01_JPRB ,&
&  0.66350E+01_JPRB, 0.55983E+01_JPRB, 0.46001E+01_JPRB, 0.40360E+01_JPRB, 0.44879E+01_JPRB, 0.31874E+01_JPRB, 0.20366E+01_JPRB ,&
&  0.16328E+01_JPRB, 0.12771E+01_JPRB, 0.10263E+01_JPRB, 0.10600E+01_JPRB, 0.10321E+01_JPRB, 0.18027E+01_JPRB, 0.34234E+01_JPRB ,&
&  0.70401E+01_JPRB, 0.98612E+01_JPRB, 0.13176E+02_JPRB, 0.13238E+02_JPRB, 0.11468E+02_JPRB, 0.10265E+02_JPRB, 0.11024E+02_JPRB ,&
&  0.68713E+01_JPRB, 0.51123E+01_JPRB, 0.48021E+01_JPRB, 0.38760E+01_JPRB, 0.28626E+01_JPRB, 0.22865E+01_JPRB, 0.19598E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.16889E+01_JPRB, 0.16222E+01_JPRB, 0.16194E+01_JPRB, 0.16379E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.13340E+01_JPRB, 0.12923E+01_JPRB, 0.12561E+01_JPRB, 0.11705E+01_JPRB, 0.10872E+01_JPRB, 0.10703E+01_JPRB, 0.10478E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.99078E+00_JPRB, 0.10009E+01_JPRB, 0.10335E+01_JPRB, 0.10708E+01_JPRB, 0.11268E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.12517E+01_JPRB, 0.14432E+01_JPRB, 0.18921E+01_JPRB, 0.23911E+01_JPRB, 0.28329E+01_JPRB, 0.25748E+01_JPRB, 0.29330E+01_JPRB ,&
&  0.26710E+01_JPRB, 0.25181E+01_JPRB, 0.29242E+01_JPRB, 0.38028E+01_JPRB, 0.50279E+01_JPRB, 0.49346E+01_JPRB, 0.43502E+01_JPRB ,&
&  0.38614E+01_JPRB, 0.34970E+01_JPRB, 0.31961E+01_JPRB, 0.28840E+01_JPRB, 0.23571E+01_JPRB, 0.21374E+01_JPRB, 0.20879E+01_JPRB ,&
&  0.20426E+01_JPRB, 0.20786E+01_JPRB, 0.20329E+01_JPRB, 0.19966E+01_JPRB, 0.20081E+01_JPRB, 0.20502E+01_JPRB, 0.21388E+01_JPRB ,&
&  0.22410E+01_JPRB, 0.24696E+01_JPRB, 0.31706E+01_JPRB, 0.48960E+01_JPRB, 0.59548E+01_JPRB, 0.58777E+01_JPRB, 0.57660E+01_JPRB ,&
&  0.57069E+01_JPRB, 0.63408E+01_JPRB, 0.68911E+01_JPRB, 0.81221E+01_JPRB, 0.69624E+01_JPRB, 0.67099E+01_JPRB, 0.62649E+01_JPRB ,&
&  0.62161E+01_JPRB, 0.68016E+01_JPRB, 0.61503E+01_JPRB, 0.65526E+01_JPRB, 0.81125E+01_JPRB, 0.76327E+01_JPRB, 0.64497E+01_JPRB ,&
&  0.63627E+01_JPRB, 0.53577E+01_JPRB, 0.48853E+01_JPRB, 0.44491E+01_JPRB, 0.59432E+01_JPRB, 0.57003E+01_JPRB, 0.21392E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.10269E+01_JPRB, 0.78021E+00_JPRB, 0.77391E+00_JPRB, 0.10092E+01_JPRB, 0.18478E+01_JPRB, 0.54892E+01_JPRB ,&
&  0.64700E+01_JPRB, 0.10554E+02_JPRB, 0.11901E+02_JPRB, 0.12584E+02_JPRB, 0.11133E+02_JPRB, 0.10341E+02_JPRB, 0.77015E+01_JPRB ,&
&  0.59688E+01_JPRB, 0.45815E+01_JPRB, 0.34915E+01_JPRB, 0.26970E+01_JPRB, 0.21016E+01_JPRB, 0.17845E+01_JPRB, 0.16343E+01_JPRB ,&
&  0.15669E+01_JPRB, 0.15326E+01_JPRB, 0.14773E+01_JPRB, 0.14192E+01_JPRB, 0.13494E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.11092E+01_JPRB, 0.11202E+01_JPRB, 0.11420E+01_JPRB, 0.11322E+01_JPRB, 0.11123E+01_JPRB, 0.11064E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10653E+01_JPRB, 0.10500E+01_JPRB, 0.10680E+01_JPRB, 0.11022E+01_JPRB, 0.11538E+01_JPRB, 0.12146E+01_JPRB, 0.12738E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.16858E+01_JPRB, 0.21798E+01_JPRB, 0.28913E+01_JPRB, 0.32020E+01_JPRB, 0.29618E+01_JPRB, 0.29634E+01_JPRB ,&
&  0.25456E+01_JPRB, 0.25381E+01_JPRB, 0.28156E+01_JPRB, 0.34437E+01_JPRB, 0.35777E+01_JPRB, 0.33424E+01_JPRB, 0.27630E+01_JPRB ,&
&  0.27495E+01_JPRB, 0.26944E+01_JPRB, 0.25801E+01_JPRB, 0.24750E+01_JPRB, 0.24279E+01_JPRB, 0.24245E+01_JPRB, 0.24397E+01_JPRB ,&
&  0.24397E+01_JPRB, 0.23662E+01_JPRB, 0.22768E+01_JPRB, 0.22176E+01_JPRB, 0.22078E+01_JPRB, 0.22616E+01_JPRB, 0.23782E+01_JPRB ,&
&  0.25900E+01_JPRB, 0.31231E+01_JPRB, 0.43613E+01_JPRB, 0.58969E+01_JPRB, 0.55611E+01_JPRB, 0.62656E+01_JPRB, 0.59885E+01_JPRB ,&
&  0.58391E+01_JPRB, 0.62145E+01_JPRB, 0.62304E+01_JPRB, 0.70339E+01_JPRB, 0.68847E+01_JPRB, 0.67060E+01_JPRB, 0.60425E+01_JPRB ,&
&  0.64186E+01_JPRB, 0.64115E+01_JPRB, 0.68056E+01_JPRB, 0.72035E+01_JPRB, 0.80576E+01_JPRB, 0.93047E+01_JPRB, 0.63616E+01_JPRB ,&
&  0.54142E+01_JPRB, 0.56041E+01_JPRB, 0.51843E+01_JPRB, 0.47249E+01_JPRB, 0.52345E+01_JPRB, 0.64606E+01_JPRB, 0.47275E+01_JPRB ,&
&  0.16105E+01_JPRB, 0.10503E+01_JPRB, 0.95226E+00_JPRB, 0.93424E+00_JPRB, 0.87881E+00_JPRB, 0.18821E+01_JPRB, 0.82066E+01_JPRB ,&
&  0.67609E+01_JPRB, 0.90125E+01_JPRB, 0.10201E+02_JPRB, 0.10461E+02_JPRB, 0.98158E+01_JPRB, 0.70323E+01_JPRB, 0.46227E+01_JPRB ,&
&  0.29264E+01_JPRB, 0.21613E+01_JPRB, 0.18858E+01_JPRB, 0.16638E+01_JPRB, 0.13943E+01_JPRB, 0.12621E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11797E+01_JPRB, 0.11483E+01_JPRB, 0.11063E+01_JPRB, 0.11015E+01_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.10430E+01_JPRB, 0.11141E+01_JPRB, 0.11799E+01_JPRB, 0.12215E+01_JPRB, 0.12219E+01_JPRB, 0.11928E+01_JPRB, 0.11657E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11467E+01_JPRB, 0.11642E+01_JPRB, 0.11946E+01_JPRB, 0.12678E+01_JPRB, 0.13597E+01_JPRB, 0.15125E+01_JPRB ,&
&  0.17579E+01_JPRB, 0.21180E+01_JPRB, 0.26280E+01_JPRB, 0.31698E+01_JPRB, 0.33001E+01_JPRB, 0.28858E+01_JPRB, 0.27214E+01_JPRB ,&
&  0.24867E+01_JPRB, 0.25235E+01_JPRB, 0.24098E+01_JPRB, 0.27496E+01_JPRB, 0.30731E+01_JPRB, 0.31731E+01_JPRB, 0.27675E+01_JPRB ,&
&  0.28053E+01_JPRB, 0.28076E+01_JPRB, 0.27872E+01_JPRB, 0.27861E+01_JPRB, 0.27932E+01_JPRB, 0.28048E+01_JPRB, 0.27630E+01_JPRB ,&
&  0.26472E+01_JPRB, 0.25309E+01_JPRB, 0.24773E+01_JPRB, 0.24866E+01_JPRB, 0.25579E+01_JPRB, 0.26643E+01_JPRB, 0.28847E+01_JPRB ,&
&  0.33443E+01_JPRB, 0.41911E+01_JPRB, 0.53235E+01_JPRB, 0.58142E+01_JPRB, 0.58282E+01_JPRB, 0.58067E+01_JPRB, 0.57325E+01_JPRB ,&
&  0.57352E+01_JPRB, 0.60881E+01_JPRB, 0.62572E+01_JPRB, 0.63248E+01_JPRB, 0.64140E+01_JPRB, 0.66333E+01_JPRB, 0.61609E+01_JPRB ,&
&  0.66803E+01_JPRB, 0.70623E+01_JPRB, 0.67802E+01_JPRB, 0.67894E+01_JPRB, 0.70700E+01_JPRB, 0.78123E+01_JPRB, 0.72285E+01_JPRB ,&
&  0.59891E+01_JPRB, 0.49848E+01_JPRB, 0.46017E+01_JPRB, 0.46539E+01_JPRB, 0.45222E+01_JPRB, 0.49056E+01_JPRB, 0.56933E+01_JPRB ,&
&  0.48897E+01_JPRB, 0.19785E+01_JPRB, 0.12908E+01_JPRB, 0.17516E+01_JPRB, 0.12519E+01_JPRB, 0.18644E+01_JPRB, 0.37689E+01_JPRB ,&
&  0.46662E+01_JPRB, 0.56589E+01_JPRB, 0.66187E+01_JPRB, 0.69027E+01_JPRB, 0.57006E+01_JPRB, 0.38784E+01_JPRB, 0.25498E+01_JPRB ,&
&  0.17842E+01_JPRB, 0.15017E+01_JPRB, 0.14362E+01_JPRB, 0.12431E+01_JPRB, 0.11031E+01_JPRB, 0.10693E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.97438E+00_JPRB, 0.97818E+00_JPRB, 0.99885E+00_JPRB, 0.10002E+01_JPRB, 0.99285E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.11253E+01_JPRB, 0.11902E+01_JPRB, 0.12732E+01_JPRB, 0.13051E+01_JPRB, 0.12890E+01_JPRB, 0.12725E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.13075E+01_JPRB, 0.13281E+01_JPRB, 0.13714E+01_JPRB, 0.14477E+01_JPRB, 0.15836E+01_JPRB, 0.17365E+01_JPRB, 0.19455E+01_JPRB ,&
&  0.22171E+01_JPRB, 0.25422E+01_JPRB, 0.29268E+01_JPRB, 0.33179E+01_JPRB, 0.35731E+01_JPRB, 0.36719E+01_JPRB, 0.24282E+01_JPRB ,&
&  0.21019E+01_JPRB, 0.26593E+01_JPRB, 0.24692E+01_JPRB, 0.27148E+01_JPRB, 0.29407E+01_JPRB, 0.30858E+01_JPRB, 0.29949E+01_JPRB ,&
&  0.30082E+01_JPRB, 0.29912E+01_JPRB, 0.29825E+01_JPRB, 0.29643E+01_JPRB, 0.29501E+01_JPRB, 0.29281E+01_JPRB, 0.28468E+01_JPRB ,&
&  0.27629E+01_JPRB, 0.27373E+01_JPRB, 0.27911E+01_JPRB, 0.28962E+01_JPRB, 0.30531E+01_JPRB, 0.32840E+01_JPRB, 0.36651E+01_JPRB ,&
&  0.42094E+01_JPRB, 0.48705E+01_JPRB, 0.54850E+01_JPRB, 0.54054E+01_JPRB, 0.54443E+01_JPRB, 0.54289E+01_JPRB, 0.56533E+01_JPRB ,&
&  0.55620E+01_JPRB, 0.52572E+01_JPRB, 0.57347E+01_JPRB, 0.62608E+01_JPRB, 0.63485E+01_JPRB, 0.62478E+01_JPRB, 0.60068E+01_JPRB ,&
&  0.67649E+01_JPRB, 0.70636E+01_JPRB, 0.70438E+01_JPRB, 0.61905E+01_JPRB, 0.60135E+01_JPRB, 0.65346E+01_JPRB, 0.67509E+01_JPRB ,&
&  0.59961E+01_JPRB, 0.52018E+01_JPRB, 0.46114E+01_JPRB, 0.40994E+01_JPRB, 0.37164E+01_JPRB, 0.37843E+01_JPRB, 0.40469E+01_JPRB ,&
&  0.41917E+01_JPRB, 0.46343E+01_JPRB, 0.36607E+01_JPRB, 0.22601E+01_JPRB, 0.18018E+01_JPRB, 0.24079E+01_JPRB, 0.31379E+01_JPRB ,&
&  0.33664E+01_JPRB, 0.36245E+01_JPRB, 0.36254E+01_JPRB, 0.42847E+01_JPRB, 0.27178E+01_JPRB, 0.19516E+01_JPRB, 0.14512E+01_JPRB ,&
&  0.12533E+01_JPRB, 0.12200E+01_JPRB, 0.12119E+01_JPRB, 0.11054E+01_JPRB, 0.10161E+01_JPRB, 0.10333E+01_JPRB, 0.97091E+00_JPRB ,&
&  0.98263E+00_JPRB, 0.10234E+01_JPRB, 0.10427E+01_JPRB, 0.10596E+01_JPRB, 0.10942E+01_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.12615E+01_JPRB, 0.13016E+01_JPRB, 0.13521E+01_JPRB, 0.13663E+01_JPRB, 0.13936E+01_JPRB, 0.14205E+01_JPRB, 0.14791E+01_JPRB ,&
&  0.15473E+01_JPRB, 0.16300E+01_JPRB, 0.17251E+01_JPRB, 0.18491E+01_JPRB, 0.20141E+01_JPRB, 0.21760E+01_JPRB, 0.23759E+01_JPRB ,&
&  0.25859E+01_JPRB, 0.27926E+01_JPRB, 0.30043E+01_JPRB, 0.32314E+01_JPRB, 0.33806E+01_JPRB, 0.34991E+01_JPRB, 0.29174E+01_JPRB ,&
&  0.18988E+01_JPRB, 0.29203E+01_JPRB, 0.24809E+01_JPRB, 0.27269E+01_JPRB, 0.28443E+01_JPRB, 0.30543E+01_JPRB, 0.29907E+01_JPRB ,&
&  0.29605E+01_JPRB, 0.29393E+01_JPRB, 0.29114E+01_JPRB, 0.29313E+01_JPRB, 0.29447E+01_JPRB, 0.29396E+01_JPRB, 0.29196E+01_JPRB ,&
&  0.29462E+01_JPRB, 0.30350E+01_JPRB, 0.31857E+01_JPRB, 0.33672E+01_JPRB, 0.36182E+01_JPRB, 0.39420E+01_JPRB, 0.42573E+01_JPRB ,&
&  0.45986E+01_JPRB, 0.49561E+01_JPRB, 0.51101E+01_JPRB, 0.50422E+01_JPRB, 0.50365E+01_JPRB, 0.52372E+01_JPRB, 0.56419E+01_JPRB ,&
&  0.56237E+01_JPRB, 0.54624E+01_JPRB, 0.57535E+01_JPRB, 0.58625E+01_JPRB, 0.57385E+01_JPRB, 0.60655E+01_JPRB, 0.58072E+01_JPRB ,&
&  0.65373E+01_JPRB, 0.63665E+01_JPRB, 0.69102E+01_JPRB, 0.57313E+01_JPRB, 0.51972E+01_JPRB, 0.54399E+01_JPRB, 0.51673E+01_JPRB ,&
&  0.45170E+01_JPRB, 0.43201E+01_JPRB, 0.38451E+01_JPRB, 0.32051E+01_JPRB, 0.30847E+01_JPRB, 0.30587E+01_JPRB, 0.33058E+01_JPRB ,&
&  0.35407E+01_JPRB, 0.35992E+01_JPRB, 0.27710E+01_JPRB, 0.19063E+01_JPRB, 0.17884E+01_JPRB, 0.21905E+01_JPRB, 0.24673E+01_JPRB ,&
&  0.25190E+01_JPRB, 0.26286E+01_JPRB, 0.27281E+01_JPRB, 0.23176E+01_JPRB, 0.15111E+01_JPRB, 0.12185E+01_JPRB, 0.10835E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.11235E+01_JPRB, 0.11215E+01_JPRB, 0.10859E+01_JPRB, 0.10025E+01_JPRB, 0.10432E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11399E+01_JPRB, 0.11898E+01_JPRB, 0.12267E+01_JPRB, 0.12439E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.14093E+01_JPRB, 0.14736E+01_JPRB, 0.15818E+01_JPRB, 0.16809E+01_JPRB, 0.17470E+01_JPRB, 0.17950E+01_JPRB, 0.18252E+01_JPRB ,&
&  0.18242E+01_JPRB, 0.18346E+01_JPRB, 0.19570E+01_JPRB, 0.20820E+01_JPRB, 0.22425E+01_JPRB, 0.24122E+01_JPRB, 0.25862E+01_JPRB ,&
&  0.27451E+01_JPRB, 0.29049E+01_JPRB, 0.30672E+01_JPRB, 0.32695E+01_JPRB, 0.34551E+01_JPRB, 0.37446E+01_JPRB, 0.34865E+01_JPRB ,&
&  0.25362E+01_JPRB, 0.22114E+01_JPRB, 0.25284E+01_JPRB, 0.25781E+01_JPRB, 0.25411E+01_JPRB, 0.27833E+01_JPRB, 0.29394E+01_JPRB ,&
&  0.28664E+01_JPRB, 0.28611E+01_JPRB, 0.28718E+01_JPRB, 0.29087E+01_JPRB, 0.29040E+01_JPRB, 0.29429E+01_JPRB, 0.29971E+01_JPRB ,&
&  0.31214E+01_JPRB, 0.32971E+01_JPRB, 0.34850E+01_JPRB, 0.37110E+01_JPRB, 0.39532E+01_JPRB, 0.42180E+01_JPRB, 0.44530E+01_JPRB ,&
&  0.46458E+01_JPRB, 0.48268E+01_JPRB, 0.48591E+01_JPRB, 0.47293E+01_JPRB, 0.48173E+01_JPRB, 0.50213E+01_JPRB, 0.51919E+01_JPRB ,&
&  0.53220E+01_JPRB, 0.56486E+01_JPRB, 0.58787E+01_JPRB, 0.58366E+01_JPRB, 0.54821E+01_JPRB, 0.58738E+01_JPRB, 0.56986E+01_JPRB ,&
&  0.59663E+01_JPRB, 0.58167E+01_JPRB, 0.64177E+01_JPRB, 0.52951E+01_JPRB, 0.47429E+01_JPRB, 0.45334E+01_JPRB, 0.41382E+01_JPRB ,&
&  0.37045E+01_JPRB, 0.35475E+01_JPRB, 0.32353E+01_JPRB, 0.25156E+01_JPRB, 0.25740E+01_JPRB, 0.23782E+01_JPRB, 0.29728E+01_JPRB ,&
&  0.33513E+01_JPRB, 0.36549E+01_JPRB, 0.23883E+01_JPRB, 0.19091E+01_JPRB, 0.17525E+01_JPRB, 0.19163E+01_JPRB, 0.19996E+01_JPRB ,&
&  0.19814E+01_JPRB, 0.18136E+01_JPRB, 0.16791E+01_JPRB, 0.14987E+01_JPRB, 0.11012E+01_JPRB, 0.98448E+00_JPRB, 0.95708E+00_JPRB ,&
&  0.10132E+01_JPRB, 0.10691E+01_JPRB, 0.10733E+01_JPRB, 0.10502E+01_JPRB, 0.10544E+01_JPRB, 0.11282E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.12805E+01_JPRB, 0.13279E+01_JPRB, 0.13414E+01_JPRB, 0.13572E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.13765E+01_JPRB, 0.14419E+01_JPRB, 0.15594E+01_JPRB, 0.16570E+01_JPRB, 0.17262E+01_JPRB, 0.17540E+01_JPRB, 0.17483E+01_JPRB ,&
&  0.17680E+01_JPRB, 0.18268E+01_JPRB, 0.19387E+01_JPRB, 0.20450E+01_JPRB, 0.21668E+01_JPRB, 0.22966E+01_JPRB, 0.24293E+01_JPRB ,&
&  0.25555E+01_JPRB, 0.26664E+01_JPRB, 0.27945E+01_JPRB, 0.29463E+01_JPRB, 0.31076E+01_JPRB, 0.33757E+01_JPRB, 0.34060E+01_JPRB ,&
&  0.27223E+01_JPRB, 0.18613E+01_JPRB, 0.21561E+01_JPRB, 0.22187E+01_JPRB, 0.22233E+01_JPRB, 0.25025E+01_JPRB, 0.26473E+01_JPRB ,&
&  0.25346E+01_JPRB, 0.25785E+01_JPRB, 0.26929E+01_JPRB, 0.27687E+01_JPRB, 0.28221E+01_JPRB, 0.29571E+01_JPRB, 0.30770E+01_JPRB ,&
&  0.32316E+01_JPRB, 0.33851E+01_JPRB, 0.35392E+01_JPRB, 0.37103E+01_JPRB, 0.38685E+01_JPRB, 0.40507E+01_JPRB, 0.42516E+01_JPRB ,&
&  0.44596E+01_JPRB, 0.46197E+01_JPRB, 0.46030E+01_JPRB, 0.45712E+01_JPRB, 0.45889E+01_JPRB, 0.46682E+01_JPRB, 0.47420E+01_JPRB ,&
&  0.48849E+01_JPRB, 0.50662E+01_JPRB, 0.54659E+01_JPRB, 0.57667E+01_JPRB, 0.58784E+01_JPRB, 0.55202E+01_JPRB, 0.55039E+01_JPRB ,&
&  0.53883E+01_JPRB, 0.50460E+01_JPRB, 0.52421E+01_JPRB, 0.56116E+01_JPRB, 0.40002E+01_JPRB, 0.38816E+01_JPRB, 0.32953E+01_JPRB ,&
&  0.33792E+01_JPRB, 0.31529E+01_JPRB, 0.28227E+01_JPRB, 0.24467E+01_JPRB, 0.21409E+01_JPRB, 0.20006E+01_JPRB, 0.24598E+01_JPRB ,&
&  0.30124E+01_JPRB, 0.27255E+01_JPRB, 0.20112E+01_JPRB, 0.17352E+01_JPRB, 0.16910E+01_JPRB, 0.20029E+01_JPRB, 0.20034E+01_JPRB ,&
&  0.17689E+01_JPRB, 0.15672E+01_JPRB, 0.14699E+01_JPRB, 0.17092E+01_JPRB, 0.12197E+01_JPRB, 0.91976E+00_JPRB, 0.87001E+00_JPRB ,&
&  0.94006E+00_JPRB, 0.10036E+01_JPRB, 0.10558E+01_JPRB, 0.10898E+01_JPRB, 0.11199E+01_JPRB, 0.11730E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.12526E+01_JPRB, 0.12537E+01_JPRB, 0.13050E+01_JPRB, 0.13197E+01_JPRB, 0.13464E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.13044E+01_JPRB, 0.13521E+01_JPRB, 0.14010E+01_JPRB, 0.14604E+01_JPRB, 0.15251E+01_JPRB, 0.15608E+01_JPRB, 0.15740E+01_JPRB ,&
&  0.16041E+01_JPRB, 0.16638E+01_JPRB, 0.17373E+01_JPRB, 0.18026E+01_JPRB, 0.18977E+01_JPRB, 0.19856E+01_JPRB, 0.20792E+01_JPRB ,&
&  0.21734E+01_JPRB, 0.22774E+01_JPRB, 0.24116E+01_JPRB, 0.25398E+01_JPRB, 0.26270E+01_JPRB, 0.27519E+01_JPRB, 0.28972E+01_JPRB ,&
&  0.30343E+01_JPRB, 0.27078E+01_JPRB, 0.24964E+01_JPRB, 0.19825E+01_JPRB, 0.15641E+01_JPRB, 0.19900E+01_JPRB, 0.20525E+01_JPRB ,&
&  0.21263E+01_JPRB, 0.21902E+01_JPRB, 0.22703E+01_JPRB, 0.23945E+01_JPRB, 0.25890E+01_JPRB, 0.27600E+01_JPRB, 0.28894E+01_JPRB ,&
&  0.30067E+01_JPRB, 0.31051E+01_JPRB, 0.31913E+01_JPRB, 0.32919E+01_JPRB, 0.34111E+01_JPRB, 0.35199E+01_JPRB, 0.36283E+01_JPRB ,&
&  0.37736E+01_JPRB, 0.39129E+01_JPRB, 0.40081E+01_JPRB, 0.41260E+01_JPRB, 0.41256E+01_JPRB, 0.42271E+01_JPRB, 0.43432E+01_JPRB ,&
&  0.44522E+01_JPRB, 0.45495E+01_JPRB, 0.48194E+01_JPRB, 0.50935E+01_JPRB, 0.51276E+01_JPRB, 0.47204E+01_JPRB, 0.46292E+01_JPRB ,&
&  0.45447E+01_JPRB, 0.41900E+01_JPRB, 0.41053E+01_JPRB, 0.51847E+01_JPRB, 0.32823E+01_JPRB, 0.34991E+01_JPRB, 0.34238E+01_JPRB ,&
&  0.30114E+01_JPRB, 0.28492E+01_JPRB, 0.25132E+01_JPRB, 0.20292E+01_JPRB, 0.18084E+01_JPRB, 0.16921E+01_JPRB, 0.20762E+01_JPRB ,&
&  0.23566E+01_JPRB, 0.19802E+01_JPRB, 0.15300E+01_JPRB, 0.13556E+01_JPRB, 0.14669E+01_JPRB, 0.19263E+01_JPRB, 0.19889E+01_JPRB ,&
&  0.16168E+01_JPRB, 0.15205E+01_JPRB, 0.13647E+01_JPRB, 0.20017E+01_JPRB, 0.16817E+01_JPRB, 0.87110E+00_JPRB, 0.75150E+00_JPRB ,&
&  0.78399E+00_JPRB, 0.84959E+00_JPRB, 0.94720E+00_JPRB, 0.10077E+01_JPRB, 0.10252E+01_JPRB, 0.10541E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.11013E+01_JPRB, 0.11503E+01_JPRB, 0.12104E+01_JPRB, 0.12424E+01_JPRB, 0.12733E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.10326E+01_JPRB, 0.10759E+01_JPRB, 0.11080E+01_JPRB, 0.11222E+01_JPRB, 0.11269E+01_JPRB, 0.11200E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10781E+01_JPRB, 0.11474E+01_JPRB, 0.13028E+01_JPRB, 0.14277E+01_JPRB, 0.15060E+01_JPRB, 0.15951E+01_JPRB ,&
&  0.16816E+01_JPRB, 0.17106E+01_JPRB, 0.16764E+01_JPRB, 0.16803E+01_JPRB, 0.16681E+01_JPRB, 0.16545E+01_JPRB, 0.18479E+01_JPRB ,&
&  0.21534E+01_JPRB, 0.25108E+01_JPRB, 0.28144E+01_JPRB, 0.30487E+01_JPRB, 0.25086E+01_JPRB, 0.19171E+01_JPRB, 0.20747E+01_JPRB ,&
&  0.23266E+01_JPRB, 0.21933E+01_JPRB, 0.19405E+01_JPRB, 0.19662E+01_JPRB, 0.20227E+01_JPRB, 0.21447E+01_JPRB, 0.23112E+01_JPRB ,&
&  0.24278E+01_JPRB, 0.25094E+01_JPRB, 0.25857E+01_JPRB, 0.26476E+01_JPRB, 0.27109E+01_JPRB, 0.27683E+01_JPRB, 0.28348E+01_JPRB ,&
&  0.28643E+01_JPRB, 0.28601E+01_JPRB, 0.29341E+01_JPRB, 0.31189E+01_JPRB, 0.32837E+01_JPRB, 0.34745E+01_JPRB, 0.36144E+01_JPRB ,&
&  0.36952E+01_JPRB, 0.37918E+01_JPRB, 0.42056E+01_JPRB, 0.44501E+01_JPRB, 0.42109E+01_JPRB, 0.38788E+01_JPRB, 0.38454E+01_JPRB ,&
&  0.36920E+01_JPRB, 0.33244E+01_JPRB, 0.27213E+01_JPRB, 0.39568E+01_JPRB, 0.39799E+01_JPRB, 0.34424E+01_JPRB, 0.29150E+01_JPRB ,&
&  0.26151E+01_JPRB, 0.24777E+01_JPRB, 0.22382E+01_JPRB, 0.19387E+01_JPRB, 0.16723E+01_JPRB, 0.19190E+01_JPRB, 0.20024E+01_JPRB ,&
&  0.20138E+01_JPRB, 0.16186E+01_JPRB, 0.12427E+01_JPRB, 0.11189E+01_JPRB, 0.11570E+01_JPRB, 0.16228E+01_JPRB, 0.16470E+01_JPRB ,&
&  0.13478E+01_JPRB, 0.12593E+01_JPRB, 0.11620E+01_JPRB, 0.17175E+01_JPRB, 0.28311E+01_JPRB, 0.87905E+00_JPRB, 0.64739E+00_JPRB ,&
&  0.65610E+00_JPRB, 0.68624E+00_JPRB, 0.71787E+00_JPRB, 0.72125E+00_JPRB, 0.71515E+00_JPRB, 0.74174E+00_JPRB, 0.78831E+00_JPRB ,&
&  0.84438E+00_JPRB, 0.89174E+00_JPRB, 0.91449E+00_JPRB, 0.96039E+00_JPRB, 0.99553E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.74944E+00_JPRB, 0.77161E+00_JPRB, 0.77294E+00_JPRB, 0.78183E+00_JPRB, 0.77782E+00_JPRB, 0.78062E+00_JPRB, 0.75811E+00_JPRB ,&
&  0.74124E+00_JPRB, 0.75685E+00_JPRB, 0.80790E+00_JPRB, 0.90267E+00_JPRB, 0.93179E+00_JPRB, 0.95733E+00_JPRB, 0.96929E+00_JPRB ,&
&  0.99625E+00_JPRB, 0.96245E+00_JPRB, 0.94071E+00_JPRB, 0.89911E+00_JPRB, 0.89358E+00_JPRB, 0.92834E+00_JPRB, 0.10243E+01_JPRB ,&
&  0.11680E+01_JPRB, 0.13482E+01_JPRB, 0.15856E+01_JPRB, 0.18576E+01_JPRB, 0.24593E+01_JPRB, 0.21640E+01_JPRB, 0.19788E+01_JPRB ,&
&  0.22345E+01_JPRB, 0.18510E+01_JPRB, 0.16710E+01_JPRB, 0.15587E+01_JPRB, 0.14802E+01_JPRB, 0.14317E+01_JPRB, 0.14889E+01_JPRB ,&
&  0.15250E+01_JPRB, 0.15820E+01_JPRB, 0.16501E+01_JPRB, 0.16726E+01_JPRB, 0.16869E+01_JPRB, 0.16524E+01_JPRB, 0.15888E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.15403E+01_JPRB, 0.16353E+01_JPRB, 0.17957E+01_JPRB, 0.20947E+01_JPRB, 0.22823E+01_JPRB, 0.24182E+01_JPRB ,&
&  0.24750E+01_JPRB, 0.26349E+01_JPRB, 0.29768E+01_JPRB, 0.30932E+01_JPRB, 0.31484E+01_JPRB, 0.29662E+01_JPRB, 0.27767E+01_JPRB ,&
&  0.25016E+01_JPRB, 0.20167E+01_JPRB, 0.17605E+01_JPRB, 0.26999E+01_JPRB, 0.28562E+01_JPRB, 0.23444E+01_JPRB, 0.24641E+01_JPRB ,&
&  0.22811E+01_JPRB, 0.21565E+01_JPRB, 0.20843E+01_JPRB, 0.18243E+01_JPRB, 0.15197E+01_JPRB, 0.16294E+01_JPRB, 0.18732E+01_JPRB ,&
&  0.16634E+01_JPRB, 0.12541E+01_JPRB, 0.10024E+01_JPRB, 0.95454E+00_JPRB, 0.10091E+01_JPRB, 0.13077E+01_JPRB, 0.14503E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.10443E+01_JPRB, 0.11253E+01_JPRB, 0.16861E+01_JPRB, 0.13156E+01_JPRB, 0.72488E+00_JPRB, 0.53527E+00_JPRB ,&
&  0.52721E+00_JPRB, 0.53778E+00_JPRB, 0.55344E+00_JPRB, 0.55787E+00_JPRB, 0.55932E+00_JPRB, 0.56687E+00_JPRB, 0.60134E+00_JPRB ,&
&  0.63073E+00_JPRB, 0.66049E+00_JPRB, 0.68344E+00_JPRB, 0.71653E+00_JPRB, 0.73633E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.72040E+00_JPRB, 0.73498E+00_JPRB, 0.75488E+00_JPRB, 0.76768E+00_JPRB, 0.78418E+00_JPRB, 0.79852E+00_JPRB, 0.83419E+00_JPRB ,&
&  0.86433E+00_JPRB, 0.89196E+00_JPRB, 0.92847E+00_JPRB, 0.96706E+00_JPRB, 0.98026E+00_JPRB, 0.99226E+00_JPRB, 0.99898E+00_JPRB ,&
&  0.10171E+01_JPRB, 0.10318E+01_JPRB, 0.10238E+01_JPRB, 0.10246E+01_JPRB, 0.10256E+01_JPRB, 0.10950E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.10974E+01_JPRB, 0.11115E+01_JPRB, 0.11714E+01_JPRB, 0.13527E+01_JPRB, 0.15133E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.17262E+01_JPRB, 0.14567E+01_JPRB, 0.13998E+01_JPRB, 0.11055E+01_JPRB, 0.10392E+01_JPRB, 0.10407E+01_JPRB, 0.98702E+00_JPRB ,&
&  0.97000E+00_JPRB, 0.98828E+00_JPRB, 0.10360E+01_JPRB, 0.10204E+01_JPRB, 0.10396E+01_JPRB, 0.10386E+01_JPRB, 0.10137E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.10453E+01_JPRB, 0.10921E+01_JPRB, 0.12287E+01_JPRB, 0.13283E+01_JPRB, 0.14939E+01_JPRB, 0.15509E+01_JPRB ,&
&  0.15533E+01_JPRB, 0.16022E+01_JPRB, 0.15376E+01_JPRB, 0.16013E+01_JPRB, 0.17827E+01_JPRB, 0.17966E+01_JPRB, 0.17095E+01_JPRB ,&
&  0.16859E+01_JPRB, 0.15521E+01_JPRB, 0.12902E+01_JPRB, 0.19582E+01_JPRB, 0.22345E+01_JPRB, 0.21087E+01_JPRB, 0.20857E+01_JPRB ,&
&  0.19517E+01_JPRB, 0.17644E+01_JPRB, 0.16311E+01_JPRB, 0.15765E+01_JPRB, 0.14153E+01_JPRB, 0.13349E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.12403E+01_JPRB, 0.10371E+01_JPRB, 0.92609E+00_JPRB, 0.97318E+00_JPRB, 0.12141E+01_JPRB, 0.18375E+01_JPRB, 0.16575E+01_JPRB ,&
&  0.12081E+01_JPRB, 0.97768E+00_JPRB, 0.13626E+01_JPRB, 0.20199E+01_JPRB, 0.11095E+01_JPRB, 0.71717E+00_JPRB, 0.50578E+00_JPRB ,&
&  0.48587E+00_JPRB, 0.50663E+00_JPRB, 0.53047E+00_JPRB, 0.54985E+00_JPRB, 0.57472E+00_JPRB, 0.59682E+00_JPRB, 0.61384E+00_JPRB ,&
&  0.63036E+00_JPRB, 0.65215E+00_JPRB, 0.67101E+00_JPRB, 0.69407E+00_JPRB, 0.71411E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.83956E+00_JPRB, 0.87598E+00_JPRB, 0.90230E+00_JPRB, 0.92377E+00_JPRB, 0.93956E+00_JPRB, 0.95757E+00_JPRB, 0.98291E+00_JPRB ,&
&  0.10054E+01_JPRB, 0.10276E+01_JPRB, 0.10494E+01_JPRB, 0.10782E+01_JPRB, 0.11141E+01_JPRB, 0.11442E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.11992E+01_JPRB, 0.12204E+01_JPRB, 0.12666E+01_JPRB, 0.12838E+01_JPRB, 0.13156E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.13148E+01_JPRB, 0.13146E+01_JPRB, 0.13200E+01_JPRB, 0.13384E+01_JPRB, 0.13542E+01_JPRB, 0.14731E+01_JPRB, 0.16495E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.13939E+01_JPRB, 0.10912E+01_JPRB, 0.89592E+00_JPRB, 0.88947E+00_JPRB, 0.89504E+00_JPRB, 0.88210E+00_JPRB ,&
&  0.88729E+00_JPRB, 0.90208E+00_JPRB, 0.91820E+00_JPRB, 0.90570E+00_JPRB, 0.94072E+00_JPRB, 0.10184E+01_JPRB, 0.10242E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10913E+01_JPRB, 0.11083E+01_JPRB, 0.12188E+01_JPRB, 0.12151E+01_JPRB, 0.13870E+01_JPRB, 0.14650E+01_JPRB ,&
&  0.14717E+01_JPRB, 0.13920E+01_JPRB, 0.11200E+01_JPRB, 0.11388E+01_JPRB, 0.12478E+01_JPRB, 0.12303E+01_JPRB, 0.12010E+01_JPRB ,&
&  0.13844E+01_JPRB, 0.14727E+01_JPRB, 0.13881E+01_JPRB, 0.17988E+01_JPRB, 0.19483E+01_JPRB, 0.19210E+01_JPRB, 0.17176E+01_JPRB ,&
&  0.15196E+01_JPRB, 0.12244E+01_JPRB, 0.12223E+01_JPRB, 0.12427E+01_JPRB, 0.11831E+01_JPRB, 0.11246E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.98591E+00_JPRB, 0.92124E+00_JPRB, 0.92207E+00_JPRB, 0.10245E+01_JPRB, 0.15342E+01_JPRB, 0.17449E+01_JPRB, 0.21169E+01_JPRB ,&
&  0.10028E+01_JPRB, 0.96118E+00_JPRB, 0.78595E+00_JPRB, 0.99169E+00_JPRB, 0.81078E+00_JPRB, 0.56364E+00_JPRB, 0.50698E+00_JPRB ,&
&  0.54562E+00_JPRB, 0.59044E+00_JPRB, 0.62575E+00_JPRB, 0.65147E+00_JPRB, 0.67496E+00_JPRB, 0.70071E+00_JPRB, 0.71318E+00_JPRB ,&
&  0.73035E+00_JPRB, 0.75012E+00_JPRB, 0.76919E+00_JPRB, 0.79784E+00_JPRB, 0.81469E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.89193E+00_JPRB, 0.92104E+00_JPRB, 0.94672E+00_JPRB, 0.97062E+00_JPRB, 0.99661E+00_JPRB, 0.10243E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.10935E+01_JPRB, 0.11135E+01_JPRB, 0.11334E+01_JPRB, 0.11616E+01_JPRB, 0.11881E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.12591E+01_JPRB, 0.12946E+01_JPRB, 0.13269E+01_JPRB, 0.13433E+01_JPRB, 0.13532E+01_JPRB, 0.13505E+01_JPRB ,&
&  0.13360E+01_JPRB, 0.13281E+01_JPRB, 0.13343E+01_JPRB, 0.13523E+01_JPRB, 0.13868E+01_JPRB, 0.13750E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.12460E+01_JPRB, 0.14078E+01_JPRB, 0.11349E+01_JPRB, 0.99688E+00_JPRB, 0.10015E+01_JPRB, 0.94495E+00_JPRB, 0.94008E+00_JPRB ,&
&  0.96631E+00_JPRB, 0.96413E+00_JPRB, 0.96389E+00_JPRB, 0.96138E+00_JPRB, 0.98827E+00_JPRB, 0.10414E+01_JPRB, 0.10693E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.12027E+01_JPRB, 0.12575E+01_JPRB, 0.13328E+01_JPRB, 0.14779E+01_JPRB, 0.15907E+01_JPRB, 0.16394E+01_JPRB ,&
&  0.16163E+01_JPRB, 0.15504E+01_JPRB, 0.12253E+01_JPRB, 0.12711E+01_JPRB, 0.13510E+01_JPRB, 0.12918E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.12761E+01_JPRB, 0.12136E+01_JPRB, 0.14324E+01_JPRB, 0.15379E+01_JPRB, 0.15052E+01_JPRB, 0.13620E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.96527E+00_JPRB, 0.82365E+00_JPRB, 0.84724E+00_JPRB, 0.96603E+00_JPRB, 0.94133E+00_JPRB, 0.81479E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.76563E+00_JPRB, 0.72957E+00_JPRB, 0.75561E+00_JPRB, 0.91576E+00_JPRB, 0.12563E+01_JPRB, 0.21271E+01_JPRB, 0.20634E+01_JPRB ,&
&  0.93428E+00_JPRB, 0.84171E+00_JPRB, 0.55069E+00_JPRB, 0.59949E+00_JPRB, 0.61455E+00_JPRB, 0.53890E+00_JPRB, 0.56411E+00_JPRB ,&
&  0.67248E+00_JPRB, 0.77202E+00_JPRB, 0.78652E+00_JPRB, 0.72978E+00_JPRB, 0.71314E+00_JPRB, 0.72476E+00_JPRB, 0.74691E+00_JPRB ,&
&  0.77000E+00_JPRB, 0.79160E+00_JPRB, 0.81382E+00_JPRB, 0.83814E+00_JPRB, 0.86358E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.91648E+00_JPRB, 0.94575E+00_JPRB, 0.97050E+00_JPRB, 0.99195E+00_JPRB, 0.10183E+01_JPRB, 0.10472E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.11194E+01_JPRB, 0.11436E+01_JPRB, 0.11667E+01_JPRB, 0.11922E+01_JPRB, 0.12210E+01_JPRB, 0.12524E+01_JPRB ,&
&  0.12819E+01_JPRB, 0.13018E+01_JPRB, 0.13402E+01_JPRB, 0.13864E+01_JPRB, 0.14177E+01_JPRB, 0.14316E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.14687E+01_JPRB, 0.14866E+01_JPRB, 0.15052E+01_JPRB, 0.15372E+01_JPRB, 0.16377E+01_JPRB, 0.14324E+01_JPRB ,&
&  0.15438E+01_JPRB, 0.13788E+01_JPRB, 0.11688E+01_JPRB, 0.10988E+01_JPRB, 0.10806E+01_JPRB, 0.10666E+01_JPRB, 0.10605E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10339E+01_JPRB, 0.10198E+01_JPRB, 0.10121E+01_JPRB, 0.10353E+01_JPRB, 0.10762E+01_JPRB, 0.11246E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.13166E+01_JPRB, 0.14441E+01_JPRB, 0.15821E+01_JPRB, 0.17458E+01_JPRB, 0.18732E+01_JPRB, 0.19499E+01_JPRB ,&
&  0.19602E+01_JPRB, 0.19363E+01_JPRB, 0.16771E+01_JPRB, 0.17561E+01_JPRB, 0.17681E+01_JPRB, 0.15442E+01_JPRB, 0.13352E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.10182E+01_JPRB, 0.10943E+01_JPRB, 0.11988E+01_JPRB, 0.11214E+01_JPRB, 0.97027E+00_JPRB, 0.82818E+00_JPRB ,&
&  0.66643E+00_JPRB, 0.61543E+00_JPRB, 0.61501E+00_JPRB, 0.71111E+00_JPRB, 0.70925E+00_JPRB, 0.64252E+00_JPRB, 0.65175E+00_JPRB ,&
&  0.64500E+00_JPRB, 0.63188E+00_JPRB, 0.67202E+00_JPRB, 0.82327E+00_JPRB, 0.10668E+01_JPRB, 0.10753E+01_JPRB, 0.11304E+01_JPRB ,&
&  0.80958E+00_JPRB, 0.69839E+00_JPRB, 0.67144E+00_JPRB, 0.62061E+00_JPRB, 0.65785E+00_JPRB, 0.67583E+00_JPRB, 0.77473E+00_JPRB ,&
&  0.93503E+00_JPRB, 0.11469E+01_JPRB, 0.11178E+01_JPRB, 0.91050E+00_JPRB, 0.75679E+00_JPRB, 0.72971E+00_JPRB, 0.74992E+00_JPRB ,&
&  0.77515E+00_JPRB, 0.79673E+00_JPRB, 0.82316E+00_JPRB, 0.85241E+00_JPRB, 0.88363E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.92038E+00_JPRB, 0.94767E+00_JPRB, 0.96896E+00_JPRB, 0.98607E+00_JPRB, 0.10109E+01_JPRB, 0.10374E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.11096E+01_JPRB, 0.11355E+01_JPRB, 0.11591E+01_JPRB, 0.11870E+01_JPRB, 0.12177E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.12821E+01_JPRB, 0.13142E+01_JPRB, 0.13371E+01_JPRB, 0.13907E+01_JPRB, 0.14550E+01_JPRB, 0.14958E+01_JPRB, 0.15205E+01_JPRB ,&
&  0.15254E+01_JPRB, 0.15550E+01_JPRB, 0.15892E+01_JPRB, 0.16244E+01_JPRB, 0.16336E+01_JPRB, 0.17164E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.15249E+01_JPRB, 0.13864E+01_JPRB, 0.12369E+01_JPRB, 0.12016E+01_JPRB, 0.11752E+01_JPRB, 0.11646E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.11198E+01_JPRB, 0.10887E+01_JPRB, 0.10707E+01_JPRB, 0.10769E+01_JPRB, 0.11095E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.13826E+01_JPRB, 0.15259E+01_JPRB, 0.16819E+01_JPRB, 0.18582E+01_JPRB, 0.20107E+01_JPRB, 0.21518E+01_JPRB ,&
&  0.22597E+01_JPRB, 0.23256E+01_JPRB, 0.21571E+01_JPRB, 0.21301E+01_JPRB, 0.20576E+01_JPRB, 0.15599E+01_JPRB, 0.12362E+01_JPRB ,&
&  0.96833E+00_JPRB, 0.95760E+00_JPRB, 0.10084E+01_JPRB, 0.10374E+01_JPRB, 0.87525E+00_JPRB, 0.77202E+00_JPRB, 0.64923E+00_JPRB ,&
&  0.55464E+00_JPRB, 0.51727E+00_JPRB, 0.52181E+00_JPRB, 0.57571E+00_JPRB, 0.56182E+00_JPRB, 0.53597E+00_JPRB, 0.56319E+00_JPRB ,&
&  0.57062E+00_JPRB, 0.60777E+00_JPRB, 0.67182E+00_JPRB, 0.77833E+00_JPRB, 0.88667E+00_JPRB, 0.88771E+00_JPRB, 0.93317E+00_JPRB ,&
&  0.82175E+00_JPRB, 0.80220E+00_JPRB, 0.74279E+00_JPRB, 0.58634E+00_JPRB, 0.75585E+00_JPRB, 0.85873E+00_JPRB, 0.99142E+00_JPRB ,&
&  0.11262E+01_JPRB, 0.15236E+01_JPRB, 0.17589E+01_JPRB, 0.13767E+01_JPRB, 0.10697E+01_JPRB, 0.78544E+00_JPRB, 0.75622E+00_JPRB ,&
&  0.78114E+00_JPRB, 0.79085E+00_JPRB, 0.81689E+00_JPRB, 0.85026E+00_JPRB, 0.88667E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.89467E+00_JPRB, 0.92568E+00_JPRB, 0.94273E+00_JPRB, 0.94584E+00_JPRB, 0.96728E+00_JPRB, 0.10032E+01_JPRB, 0.10231E+01_JPRB ,&
&  0.10441E+01_JPRB, 0.10556E+01_JPRB, 0.10799E+01_JPRB, 0.11032E+01_JPRB, 0.11272E+01_JPRB, 0.11536E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.12076E+01_JPRB, 0.12587E+01_JPRB, 0.12900E+01_JPRB, 0.13478E+01_JPRB, 0.14101E+01_JPRB, 0.14914E+01_JPRB, 0.15376E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.15881E+01_JPRB, 0.16360E+01_JPRB, 0.16936E+01_JPRB, 0.16870E+01_JPRB, 0.16993E+01_JPRB, 0.13345E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.13803E+01_JPRB, 0.13121E+01_JPRB, 0.12804E+01_JPRB, 0.12402E+01_JPRB, 0.12019E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.11223E+01_JPRB, 0.10659E+01_JPRB, 0.10890E+01_JPRB, 0.11012E+01_JPRB, 0.11165E+01_JPRB, 0.11852E+01_JPRB ,&
&  0.12548E+01_JPRB, 0.13619E+01_JPRB, 0.14963E+01_JPRB, 0.16586E+01_JPRB, 0.18317E+01_JPRB, 0.20021E+01_JPRB, 0.21696E+01_JPRB ,&
&  0.23258E+01_JPRB, 0.24525E+01_JPRB, 0.24769E+01_JPRB, 0.20857E+01_JPRB, 0.18194E+01_JPRB, 0.14180E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.10288E+01_JPRB, 0.96645E+00_JPRB, 0.96740E+00_JPRB, 0.96385E+00_JPRB, 0.78996E+00_JPRB, 0.67531E+00_JPRB, 0.56911E+00_JPRB ,&
&  0.52823E+00_JPRB, 0.49462E+00_JPRB, 0.49310E+00_JPRB, 0.49329E+00_JPRB, 0.49347E+00_JPRB, 0.49501E+00_JPRB, 0.51718E+00_JPRB ,&
&  0.53833E+00_JPRB, 0.58800E+00_JPRB, 0.68475E+00_JPRB, 0.81396E+00_JPRB, 0.93260E+00_JPRB, 0.99160E+00_JPRB, 0.88423E+00_JPRB ,&
&  0.84963E+00_JPRB, 0.76444E+00_JPRB, 0.73608E+00_JPRB, 0.77236E+00_JPRB, 0.83463E+00_JPRB, 0.10087E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.15121E+01_JPRB, 0.18033E+01_JPRB, 0.23553E+01_JPRB, 0.21223E+01_JPRB, 0.18178E+01_JPRB, 0.70706E+00_JPRB ,&
&  0.74842E+00_JPRB, 0.76116E+00_JPRB, 0.79589E+00_JPRB, 0.83044E+00_JPRB, 0.87436E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.73236E+00_JPRB, 0.76331E+00_JPRB, 0.78780E+00_JPRB, 0.79349E+00_JPRB, 0.82789E+00_JPRB, 0.86014E+00_JPRB, 0.88142E+00_JPRB ,&
&  0.90057E+00_JPRB, 0.93026E+00_JPRB, 0.95175E+00_JPRB, 0.98497E+00_JPRB, 0.10105E+01_JPRB, 0.10356E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.11242E+01_JPRB, 0.11684E+01_JPRB, 0.12312E+01_JPRB, 0.12950E+01_JPRB, 0.13750E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.14998E+01_JPRB, 0.15588E+01_JPRB, 0.16290E+01_JPRB, 0.16730E+01_JPRB, 0.16531E+01_JPRB, 0.16213E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.13612E+01_JPRB, 0.13813E+01_JPRB, 0.13675E+01_JPRB, 0.13324E+01_JPRB, 0.12831E+01_JPRB, 0.12230E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11116E+01_JPRB, 0.10593E+01_JPRB, 0.10200E+01_JPRB, 0.10933E+01_JPRB, 0.11084E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.13226E+01_JPRB, 0.14323E+01_JPRB, 0.15888E+01_JPRB, 0.17626E+01_JPRB, 0.19327E+01_JPRB, 0.21066E+01_JPRB ,&
&  0.22344E+01_JPRB, 0.23002E+01_JPRB, 0.23268E+01_JPRB, 0.19083E+01_JPRB, 0.16456E+01_JPRB, 0.13865E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.99784E+00_JPRB, 0.95351E+00_JPRB, 0.96808E+00_JPRB, 0.91770E+00_JPRB, 0.76968E+00_JPRB, 0.63643E+00_JPRB, 0.54441E+00_JPRB ,&
&  0.50769E+00_JPRB, 0.47169E+00_JPRB, 0.46884E+00_JPRB, 0.47349E+00_JPRB, 0.48666E+00_JPRB, 0.49842E+00_JPRB, 0.51946E+00_JPRB ,&
&  0.54870E+00_JPRB, 0.59446E+00_JPRB, 0.67293E+00_JPRB, 0.79658E+00_JPRB, 0.96587E+00_JPRB, 0.11500E+01_JPRB, 0.11087E+01_JPRB ,&
&  0.88471E+00_JPRB, 0.75678E+00_JPRB, 0.71745E+00_JPRB, 0.70304E+00_JPRB, 0.72298E+00_JPRB, 0.77953E+00_JPRB, 0.85196E+00_JPRB ,&
&  0.97996E+00_JPRB, 0.10540E+01_JPRB, 0.94997E+00_JPRB, 0.79436E+00_JPRB, 0.10572E+01_JPRB, 0.10443E+01_JPRB, 0.57716E+00_JPRB ,&
&  0.62888E+00_JPRB, 0.61677E+00_JPRB, 0.64312E+00_JPRB, 0.67481E+00_JPRB, 0.71441E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.56480E+00_JPRB, 0.61476E+00_JPRB, 0.63656E+00_JPRB, 0.66456E+00_JPRB, 0.68649E+00_JPRB, 0.71863E+00_JPRB, 0.74898E+00_JPRB ,&
&  0.77733E+00_JPRB, 0.80994E+00_JPRB, 0.83114E+00_JPRB, 0.86408E+00_JPRB, 0.89097E+00_JPRB, 0.91421E+00_JPRB, 0.93495E+00_JPRB ,&
&  0.95570E+00_JPRB, 0.99199E+00_JPRB, 0.10304E+01_JPRB, 0.10752E+01_JPRB, 0.11215E+01_JPRB, 0.11834E+01_JPRB, 0.12705E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14770E+01_JPRB, 0.15648E+01_JPRB, 0.16359E+01_JPRB, 0.16695E+01_JPRB, 0.17298E+01_JPRB, 0.17235E+01_JPRB ,&
&  0.12196E+01_JPRB, 0.14544E+01_JPRB, 0.14094E+01_JPRB, 0.13412E+01_JPRB, 0.12885E+01_JPRB, 0.12397E+01_JPRB, 0.12155E+01_JPRB ,&
&  0.11844E+01_JPRB, 0.10963E+01_JPRB, 0.10638E+01_JPRB, 0.10267E+01_JPRB, 0.10470E+01_JPRB, 0.10705E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.12742E+01_JPRB, 0.13717E+01_JPRB, 0.14767E+01_JPRB, 0.16230E+01_JPRB, 0.18081E+01_JPRB, 0.19720E+01_JPRB ,&
&  0.21412E+01_JPRB, 0.23154E+01_JPRB, 0.24152E+01_JPRB, 0.20669E+01_JPRB, 0.17570E+01_JPRB, 0.16076E+01_JPRB, 0.12841E+01_JPRB ,&
&  0.99710E+00_JPRB, 0.88809E+00_JPRB, 0.90611E+00_JPRB, 0.88811E+00_JPRB, 0.75620E+00_JPRB, 0.64202E+00_JPRB, 0.56138E+00_JPRB ,&
&  0.52788E+00_JPRB, 0.49247E+00_JPRB, 0.48087E+00_JPRB, 0.48544E+00_JPRB, 0.49602E+00_JPRB, 0.51067E+00_JPRB, 0.54460E+00_JPRB ,&
&  0.57021E+00_JPRB, 0.59493E+00_JPRB, 0.62766E+00_JPRB, 0.66288E+00_JPRB, 0.72621E+00_JPRB, 0.82285E+00_JPRB, 0.83341E+00_JPRB ,&
&  0.79507E+00_JPRB, 0.75721E+00_JPRB, 0.71891E+00_JPRB, 0.70912E+00_JPRB, 0.68951E+00_JPRB, 0.68128E+00_JPRB, 0.69324E+00_JPRB ,&
&  0.71335E+00_JPRB, 0.72757E+00_JPRB, 0.68747E+00_JPRB, 0.65451E+00_JPRB, 0.64660E+00_JPRB, 0.61422E+00_JPRB, 0.55934E+00_JPRB ,&
&  0.53852E+00_JPRB, 0.52034E+00_JPRB, 0.50179E+00_JPRB, 0.50954E+00_JPRB, 0.53382E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.48264E+00_JPRB, 0.50120E+00_JPRB, 0.52332E+00_JPRB, 0.54582E+00_JPRB, 0.56691E+00_JPRB, 0.58724E+00_JPRB, 0.61895E+00_JPRB ,&
&  0.65406E+00_JPRB, 0.67885E+00_JPRB, 0.69102E+00_JPRB, 0.71437E+00_JPRB, 0.74125E+00_JPRB, 0.77460E+00_JPRB, 0.80291E+00_JPRB ,&
&  0.83438E+00_JPRB, 0.87673E+00_JPRB, 0.91272E+00_JPRB, 0.95078E+00_JPRB, 0.98935E+00_JPRB, 0.10305E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.12740E+01_JPRB, 0.13986E+01_JPRB, 0.15167E+01_JPRB, 0.16207E+01_JPRB, 0.17273E+01_JPRB, 0.17509E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.13822E+01_JPRB, 0.15624E+01_JPRB, 0.14177E+01_JPRB, 0.12972E+01_JPRB, 0.12284E+01_JPRB, 0.11947E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.10525E+01_JPRB, 0.10215E+01_JPRB, 0.10638E+01_JPRB, 0.10036E+01_JPRB, 0.10138E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.11519E+01_JPRB, 0.12169E+01_JPRB, 0.12974E+01_JPRB, 0.14143E+01_JPRB, 0.15589E+01_JPRB, 0.17434E+01_JPRB ,&
&  0.19746E+01_JPRB, 0.22286E+01_JPRB, 0.24607E+01_JPRB, 0.20167E+01_JPRB, 0.18624E+01_JPRB, 0.17724E+01_JPRB, 0.15655E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.88965E+00_JPRB, 0.88846E+00_JPRB, 0.87372E+00_JPRB, 0.77667E+00_JPRB, 0.69141E+00_JPRB, 0.62732E+00_JPRB ,&
&  0.58612E+00_JPRB, 0.55354E+00_JPRB, 0.53808E+00_JPRB, 0.53288E+00_JPRB, 0.53591E+00_JPRB, 0.55582E+00_JPRB, 0.58260E+00_JPRB ,&
&  0.60761E+00_JPRB, 0.61503E+00_JPRB, 0.64477E+00_JPRB, 0.66120E+00_JPRB, 0.70678E+00_JPRB, 0.74783E+00_JPRB, 0.77386E+00_JPRB ,&
&  0.78499E+00_JPRB, 0.77792E+00_JPRB, 0.77201E+00_JPRB, 0.76601E+00_JPRB, 0.76133E+00_JPRB, 0.75504E+00_JPRB, 0.74526E+00_JPRB ,&
&  0.73003E+00_JPRB, 0.73484E+00_JPRB, 0.71674E+00_JPRB, 0.70240E+00_JPRB, 0.70056E+00_JPRB, 0.66589E+00_JPRB, 0.64619E+00_JPRB ,&
&  0.62400E+00_JPRB, 0.57289E+00_JPRB, 0.53677E+00_JPRB, 0.50975E+00_JPRB, 0.48950E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.51091E+00_JPRB, 0.48220E+00_JPRB, 0.48459E+00_JPRB, 0.48893E+00_JPRB, 0.50567E+00_JPRB, 0.52348E+00_JPRB, 0.51955E+00_JPRB ,&
&  0.52809E+00_JPRB, 0.53973E+00_JPRB, 0.55611E+00_JPRB, 0.57498E+00_JPRB, 0.59862E+00_JPRB, 0.62387E+00_JPRB, 0.64601E+00_JPRB ,&
&  0.67760E+00_JPRB, 0.71583E+00_JPRB, 0.75889E+00_JPRB, 0.80052E+00_JPRB, 0.84410E+00_JPRB, 0.87912E+00_JPRB, 0.91323E+00_JPRB ,&
&  0.95685E+00_JPRB, 0.10273E+01_JPRB, 0.11283E+01_JPRB, 0.12523E+01_JPRB, 0.14113E+01_JPRB, 0.15987E+01_JPRB, 0.17213E+01_JPRB ,&
&  0.12559E+01_JPRB, 0.70980E+00_JPRB, 0.14411E+01_JPRB, 0.15958E+01_JPRB, 0.13935E+01_JPRB, 0.12981E+01_JPRB, 0.11919E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11054E+01_JPRB, 0.10406E+01_JPRB, 0.10391E+01_JPRB, 0.98353E+00_JPRB, 0.97607E+00_JPRB, 0.97749E+00_JPRB ,&
&  0.98090E+00_JPRB, 0.10053E+01_JPRB, 0.10692E+01_JPRB, 0.11101E+01_JPRB, 0.11714E+01_JPRB, 0.12814E+01_JPRB, 0.14486E+01_JPRB ,&
&  0.16853E+01_JPRB, 0.19669E+01_JPRB, 0.22447E+01_JPRB, 0.18009E+01_JPRB, 0.14927E+01_JPRB, 0.12671E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.10191E+01_JPRB, 0.89398E+00_JPRB, 0.86878E+00_JPRB, 0.85660E+00_JPRB, 0.80089E+00_JPRB, 0.71503E+00_JPRB, 0.69459E+00_JPRB ,&
&  0.65462E+00_JPRB, 0.62712E+00_JPRB, 0.61167E+00_JPRB, 0.60121E+00_JPRB, 0.60044E+00_JPRB, 0.61282E+00_JPRB, 0.62797E+00_JPRB ,&
&  0.64927E+00_JPRB, 0.66465E+00_JPRB, 0.68873E+00_JPRB, 0.71662E+00_JPRB, 0.73358E+00_JPRB, 0.74987E+00_JPRB, 0.77432E+00_JPRB ,&
&  0.78374E+00_JPRB, 0.80092E+00_JPRB, 0.81691E+00_JPRB, 0.82288E+00_JPRB, 0.82731E+00_JPRB, 0.81457E+00_JPRB, 0.81756E+00_JPRB ,&
&  0.81494E+00_JPRB, 0.80677E+00_JPRB, 0.78525E+00_JPRB, 0.78639E+00_JPRB, 0.78292E+00_JPRB, 0.77308E+00_JPRB, 0.78258E+00_JPRB ,&
&  0.76975E+00_JPRB, 0.73497E+00_JPRB, 0.66763E+00_JPRB, 0.59453E+00_JPRB, 0.55546E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.61683E+00_JPRB, 0.57173E+00_JPRB, 0.53561E+00_JPRB, 0.51626E+00_JPRB, 0.50349E+00_JPRB, 0.49868E+00_JPRB, 0.49772E+00_JPRB ,&
&  0.50234E+00_JPRB, 0.50716E+00_JPRB, 0.49539E+00_JPRB, 0.50083E+00_JPRB, 0.51489E+00_JPRB, 0.53148E+00_JPRB, 0.55314E+00_JPRB ,&
&  0.56775E+00_JPRB, 0.58161E+00_JPRB, 0.59792E+00_JPRB, 0.62425E+00_JPRB, 0.65912E+00_JPRB, 0.69822E+00_JPRB, 0.73568E+00_JPRB ,&
&  0.77378E+00_JPRB, 0.81654E+00_JPRB, 0.87671E+00_JPRB, 0.96063E+00_JPRB, 0.10789E+01_JPRB, 0.12512E+01_JPRB, 0.14445E+01_JPRB ,&
&  0.16975E+01_JPRB, 0.12592E+01_JPRB, 0.90079E+00_JPRB, 0.14509E+01_JPRB, 0.15335E+01_JPRB, 0.14500E+01_JPRB, 0.12387E+01_JPRB ,&
&  0.12262E+01_JPRB, 0.12283E+01_JPRB, 0.11328E+01_JPRB, 0.10673E+01_JPRB, 0.96764E+00_JPRB, 0.94846E+00_JPRB, 0.93086E+00_JPRB ,&
&  0.91105E+00_JPRB, 0.89514E+00_JPRB, 0.92399E+00_JPRB, 0.95008E+00_JPRB, 0.97119E+00_JPRB, 0.10406E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.13928E+01_JPRB, 0.16599E+01_JPRB, 0.19564E+01_JPRB, 0.16800E+01_JPRB, 0.13557E+01_JPRB, 0.11127E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.96638E+00_JPRB, 0.94776E+00_JPRB, 0.90735E+00_JPRB, 0.85707E+00_JPRB, 0.79905E+00_JPRB, 0.74488E+00_JPRB, 0.73284E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.68035E+00_JPRB, 0.65751E+00_JPRB, 0.63632E+00_JPRB, 0.63437E+00_JPRB, 0.64382E+00_JPRB, 0.65526E+00_JPRB ,&
&  0.66812E+00_JPRB, 0.68461E+00_JPRB, 0.70966E+00_JPRB, 0.73897E+00_JPRB, 0.75289E+00_JPRB, 0.75915E+00_JPRB, 0.77206E+00_JPRB ,&
&  0.78573E+00_JPRB, 0.79882E+00_JPRB, 0.82660E+00_JPRB, 0.84475E+00_JPRB, 0.85062E+00_JPRB, 0.83634E+00_JPRB, 0.85465E+00_JPRB ,&
&  0.85562E+00_JPRB, 0.85804E+00_JPRB, 0.83373E+00_JPRB, 0.84357E+00_JPRB, 0.83494E+00_JPRB, 0.84603E+00_JPRB, 0.86000E+00_JPRB ,&
&  0.87249E+00_JPRB, 0.86048E+00_JPRB, 0.81710E+00_JPRB, 0.70122E+00_JPRB, 0.65490E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.66047E+00_JPRB, 0.63149E+00_JPRB, 0.61000E+00_JPRB, 0.58973E+00_JPRB, 0.56895E+00_JPRB, 0.54834E+00_JPRB, 0.53248E+00_JPRB ,&
&  0.51405E+00_JPRB, 0.51157E+00_JPRB, 0.50780E+00_JPRB, 0.50209E+00_JPRB, 0.49509E+00_JPRB, 0.49612E+00_JPRB, 0.50410E+00_JPRB ,&
&  0.51079E+00_JPRB, 0.51494E+00_JPRB, 0.52080E+00_JPRB, 0.52411E+00_JPRB, 0.53033E+00_JPRB, 0.54516E+00_JPRB, 0.56863E+00_JPRB ,&
&  0.60114E+00_JPRB, 0.63629E+00_JPRB, 0.68166E+00_JPRB, 0.73672E+00_JPRB, 0.79244E+00_JPRB, 0.87966E+00_JPRB, 0.99042E+00_JPRB ,&
&  0.12425E+01_JPRB, 0.12661E+01_JPRB, 0.77654E+00_JPRB, 0.12974E+01_JPRB, 0.16242E+01_JPRB, 0.15410E+01_JPRB, 0.14376E+01_JPRB ,&
&  0.14289E+01_JPRB, 0.13476E+01_JPRB, 0.12457E+01_JPRB, 0.10975E+01_JPRB, 0.95627E+00_JPRB, 0.91764E+00_JPRB, 0.89379E+00_JPRB ,&
&  0.85732E+00_JPRB, 0.82981E+00_JPRB, 0.82382E+00_JPRB, 0.82554E+00_JPRB, 0.82692E+00_JPRB, 0.85285E+00_JPRB, 0.95662E+00_JPRB ,&
&  0.11195E+01_JPRB, 0.13735E+01_JPRB, 0.16544E+01_JPRB, 0.15635E+01_JPRB, 0.12621E+01_JPRB, 0.11278E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10432E+01_JPRB, 0.10208E+01_JPRB, 0.92560E+00_JPRB, 0.85179E+00_JPRB, 0.79016E+00_JPRB, 0.75069E+00_JPRB, 0.75284E+00_JPRB ,&
&  0.73796E+00_JPRB, 0.71328E+00_JPRB, 0.69365E+00_JPRB, 0.66728E+00_JPRB, 0.65467E+00_JPRB, 0.65004E+00_JPRB, 0.65710E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.68129E+00_JPRB, 0.69722E+00_JPRB, 0.71663E+00_JPRB, 0.73046E+00_JPRB, 0.74240E+00_JPRB, 0.75195E+00_JPRB ,&
&  0.76796E+00_JPRB, 0.79144E+00_JPRB, 0.81807E+00_JPRB, 0.83591E+00_JPRB, 0.85178E+00_JPRB, 0.86299E+00_JPRB, 0.86603E+00_JPRB ,&
&  0.87395E+00_JPRB, 0.87301E+00_JPRB, 0.81513E+00_JPRB, 0.80879E+00_JPRB, 0.83739E+00_JPRB, 0.83244E+00_JPRB, 0.82166E+00_JPRB ,&
&  0.84961E+00_JPRB, 0.87183E+00_JPRB, 0.85937E+00_JPRB, 0.74274E+00_JPRB, 0.69501E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.67086E+00_JPRB, 0.64837E+00_JPRB, 0.63054E+00_JPRB, 0.61701E+00_JPRB, 0.60065E+00_JPRB, 0.58410E+00_JPRB, 0.57081E+00_JPRB ,&
&  0.55262E+00_JPRB, 0.54163E+00_JPRB, 0.53385E+00_JPRB, 0.52532E+00_JPRB, 0.51673E+00_JPRB, 0.50815E+00_JPRB, 0.50064E+00_JPRB ,&
&  0.49442E+00_JPRB, 0.48959E+00_JPRB, 0.48670E+00_JPRB, 0.48108E+00_JPRB, 0.47624E+00_JPRB, 0.47548E+00_JPRB, 0.48081E+00_JPRB ,&
&  0.49981E+00_JPRB, 0.52711E+00_JPRB, 0.55966E+00_JPRB, 0.59416E+00_JPRB, 0.62414E+00_JPRB, 0.66506E+00_JPRB, 0.71573E+00_JPRB ,&
&  0.91761E+00_JPRB, 0.10306E+01_JPRB, 0.12223E+01_JPRB, 0.19213E+01_JPRB, 0.19591E+01_JPRB, 0.17544E+01_JPRB, 0.17275E+01_JPRB ,&
&  0.17268E+01_JPRB, 0.16047E+01_JPRB, 0.14916E+01_JPRB, 0.12205E+01_JPRB, 0.10156E+01_JPRB, 0.90973E+00_JPRB, 0.84778E+00_JPRB ,&
&  0.79196E+00_JPRB, 0.75675E+00_JPRB, 0.74821E+00_JPRB, 0.74061E+00_JPRB, 0.73746E+00_JPRB, 0.77161E+00_JPRB, 0.82389E+00_JPRB ,&
&  0.92874E+00_JPRB, 0.11071E+01_JPRB, 0.13666E+01_JPRB, 0.15174E+01_JPRB, 0.12479E+01_JPRB, 0.11903E+01_JPRB, 0.12083E+01_JPRB ,&
&  0.13903E+01_JPRB, 0.11993E+01_JPRB, 0.98706E+00_JPRB, 0.86062E+00_JPRB, 0.78259E+00_JPRB, 0.76367E+00_JPRB, 0.73950E+00_JPRB ,&
&  0.72876E+00_JPRB, 0.71561E+00_JPRB, 0.70181E+00_JPRB, 0.68393E+00_JPRB, 0.67188E+00_JPRB, 0.66101E+00_JPRB, 0.65799E+00_JPRB ,&
&  0.66186E+00_JPRB, 0.66872E+00_JPRB, 0.67606E+00_JPRB, 0.68731E+00_JPRB, 0.70284E+00_JPRB, 0.71714E+00_JPRB, 0.73015E+00_JPRB ,&
&  0.74382E+00_JPRB, 0.76960E+00_JPRB, 0.76712E+00_JPRB, 0.77950E+00_JPRB, 0.80615E+00_JPRB, 0.82071E+00_JPRB, 0.79903E+00_JPRB ,&
&  0.78810E+00_JPRB, 0.80906E+00_JPRB, 0.80704E+00_JPRB, 0.80662E+00_JPRB, 0.81409E+00_JPRB, 0.81321E+00_JPRB, 0.76045E+00_JPRB ,&
&  0.74705E+00_JPRB, 0.74348E+00_JPRB, 0.73803E+00_JPRB, 0.71642E+00_JPRB, 0.69501E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.67634E+00_JPRB, 0.65891E+00_JPRB, 0.64040E+00_JPRB, 0.62178E+00_JPRB, 0.60132E+00_JPRB, 0.58633E+00_JPRB, 0.57836E+00_JPRB ,&
&  0.56593E+00_JPRB, 0.55504E+00_JPRB, 0.54689E+00_JPRB, 0.54305E+00_JPRB, 0.53973E+00_JPRB, 0.52988E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.49392E+00_JPRB, 0.48166E+00_JPRB, 0.47472E+00_JPRB, 0.46769E+00_JPRB, 0.46031E+00_JPRB, 0.45629E+00_JPRB, 0.45784E+00_JPRB ,&
&  0.46555E+00_JPRB, 0.47803E+00_JPRB, 0.50205E+00_JPRB, 0.52891E+00_JPRB, 0.55110E+00_JPRB, 0.57114E+00_JPRB, 0.58659E+00_JPRB ,&
&  0.70325E+00_JPRB, 0.90166E+00_JPRB, 0.96325E+00_JPRB, 0.16569E+01_JPRB, 0.17597E+01_JPRB, 0.17176E+01_JPRB, 0.17176E+01_JPRB ,&
&  0.17104E+01_JPRB, 0.18346E+01_JPRB, 0.15958E+01_JPRB, 0.13427E+01_JPRB, 0.11200E+01_JPRB, 0.97089E+00_JPRB, 0.86407E+00_JPRB ,&
&  0.78483E+00_JPRB, 0.73181E+00_JPRB, 0.70425E+00_JPRB, 0.69306E+00_JPRB, 0.68986E+00_JPRB, 0.70677E+00_JPRB, 0.75551E+00_JPRB ,&
&  0.82815E+00_JPRB, 0.95018E+00_JPRB, 0.11378E+01_JPRB, 0.12540E+01_JPRB, 0.12046E+01_JPRB, 0.12246E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.16451E+01_JPRB, 0.14992E+01_JPRB, 0.11080E+01_JPRB, 0.89244E+00_JPRB, 0.78822E+00_JPRB, 0.75133E+00_JPRB, 0.72060E+00_JPRB ,&
&  0.70835E+00_JPRB, 0.69709E+00_JPRB, 0.68741E+00_JPRB, 0.67792E+00_JPRB, 0.66811E+00_JPRB, 0.66340E+00_JPRB, 0.66068E+00_JPRB ,&
&  0.65866E+00_JPRB, 0.66039E+00_JPRB, 0.66217E+00_JPRB, 0.66487E+00_JPRB, 0.67028E+00_JPRB, 0.68225E+00_JPRB, 0.69958E+00_JPRB ,&
&  0.71654E+00_JPRB, 0.73940E+00_JPRB, 0.73851E+00_JPRB, 0.73226E+00_JPRB, 0.74737E+00_JPRB, 0.75602E+00_JPRB, 0.74273E+00_JPRB ,&
&  0.75257E+00_JPRB, 0.77601E+00_JPRB, 0.78301E+00_JPRB, 0.77006E+00_JPRB, 0.79392E+00_JPRB, 0.82487E+00_JPRB, 0.74862E+00_JPRB ,&
&  0.71992E+00_JPRB, 0.70666E+00_JPRB, 0.70428E+00_JPRB, 0.69468E+00_JPRB, 0.68572E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.66434E+00_JPRB, 0.65574E+00_JPRB, 0.63987E+00_JPRB, 0.62083E+00_JPRB, 0.60123E+00_JPRB, 0.58492E+00_JPRB, 0.57657E+00_JPRB ,&
&  0.56714E+00_JPRB, 0.55728E+00_JPRB, 0.54801E+00_JPRB, 0.54258E+00_JPRB, 0.53959E+00_JPRB, 0.52990E+00_JPRB, 0.51357E+00_JPRB ,&
&  0.49631E+00_JPRB, 0.48039E+00_JPRB, 0.46738E+00_JPRB, 0.45979E+00_JPRB, 0.45183E+00_JPRB, 0.44809E+00_JPRB, 0.44802E+00_JPRB ,&
&  0.44986E+00_JPRB, 0.45721E+00_JPRB, 0.47265E+00_JPRB, 0.49686E+00_JPRB, 0.51801E+00_JPRB, 0.53169E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.61043E+00_JPRB, 0.10034E+01_JPRB, 0.89246E+00_JPRB, 0.12938E+01_JPRB, 0.13884E+01_JPRB, 0.14572E+01_JPRB, 0.15498E+01_JPRB ,&
&  0.15902E+01_JPRB, 0.16419E+01_JPRB, 0.15481E+01_JPRB, 0.13823E+01_JPRB, 0.11888E+01_JPRB, 0.10490E+01_JPRB, 0.93702E+00_JPRB ,&
&  0.85215E+00_JPRB, 0.79088E+00_JPRB, 0.74634E+00_JPRB, 0.71234E+00_JPRB, 0.69068E+00_JPRB, 0.69027E+00_JPRB, 0.73011E+00_JPRB ,&
&  0.81223E+00_JPRB, 0.90573E+00_JPRB, 0.10172E+01_JPRB, 0.11472E+01_JPRB, 0.10917E+01_JPRB, 0.11959E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.17747E+01_JPRB, 0.18147E+01_JPRB, 0.13149E+01_JPRB, 0.96756E+00_JPRB, 0.81385E+00_JPRB, 0.74168E+00_JPRB, 0.69832E+00_JPRB ,&
&  0.67473E+00_JPRB, 0.66330E+00_JPRB, 0.65684E+00_JPRB, 0.65424E+00_JPRB, 0.65403E+00_JPRB, 0.65588E+00_JPRB, 0.65503E+00_JPRB ,&
&  0.65426E+00_JPRB, 0.65436E+00_JPRB, 0.65505E+00_JPRB, 0.65504E+00_JPRB, 0.65240E+00_JPRB, 0.65407E+00_JPRB, 0.66453E+00_JPRB ,&
&  0.68186E+00_JPRB, 0.70146E+00_JPRB, 0.70718E+00_JPRB, 0.70310E+00_JPRB, 0.70858E+00_JPRB, 0.71386E+00_JPRB, 0.70894E+00_JPRB ,&
&  0.72552E+00_JPRB, 0.73636E+00_JPRB, 0.74274E+00_JPRB, 0.75803E+00_JPRB, 0.77064E+00_JPRB, 0.84383E+00_JPRB, 0.74832E+00_JPRB ,&
&  0.71121E+00_JPRB, 0.69181E+00_JPRB, 0.68236E+00_JPRB, 0.67082E+00_JPRB, 0.66660E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.64370E+00_JPRB, 0.63609E+00_JPRB, 0.62084E+00_JPRB, 0.60304E+00_JPRB, 0.58991E+00_JPRB, 0.57633E+00_JPRB, 0.56599E+00_JPRB ,&
&  0.55754E+00_JPRB, 0.55003E+00_JPRB, 0.54396E+00_JPRB, 0.53681E+00_JPRB, 0.52764E+00_JPRB, 0.51538E+00_JPRB, 0.50377E+00_JPRB ,&
&  0.49148E+00_JPRB, 0.47948E+00_JPRB, 0.46753E+00_JPRB, 0.45535E+00_JPRB, 0.44580E+00_JPRB, 0.43952E+00_JPRB, 0.43547E+00_JPRB ,&
&  0.43525E+00_JPRB, 0.43879E+00_JPRB, 0.44838E+00_JPRB, 0.46751E+00_JPRB, 0.49025E+00_JPRB, 0.50806E+00_JPRB, 0.52575E+00_JPRB ,&
&  0.60398E+00_JPRB, 0.54903E+00_JPRB, 0.98737E+00_JPRB, 0.11191E+01_JPRB, 0.10896E+01_JPRB, 0.11093E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.14641E+01_JPRB, 0.14622E+01_JPRB, 0.13839E+01_JPRB, 0.12360E+01_JPRB, 0.11244E+01_JPRB, 0.10383E+01_JPRB ,&
&  0.97320E+00_JPRB, 0.91866E+00_JPRB, 0.85534E+00_JPRB, 0.79262E+00_JPRB, 0.74543E+00_JPRB, 0.73426E+00_JPRB, 0.75464E+00_JPRB ,&
&  0.84007E+00_JPRB, 0.90751E+00_JPRB, 0.95620E+00_JPRB, 0.10302E+01_JPRB, 0.10264E+01_JPRB, 0.11444E+01_JPRB, 0.13094E+01_JPRB ,&
&  0.17476E+01_JPRB, 0.18332E+01_JPRB, 0.14452E+01_JPRB, 0.11054E+01_JPRB, 0.88946E+00_JPRB, 0.77498E+00_JPRB, 0.69680E+00_JPRB ,&
&  0.65653E+00_JPRB, 0.63542E+00_JPRB, 0.62368E+00_JPRB, 0.62258E+00_JPRB, 0.63150E+00_JPRB, 0.64455E+00_JPRB, 0.65350E+00_JPRB ,&
&  0.66276E+00_JPRB, 0.66898E+00_JPRB, 0.66848E+00_JPRB, 0.66454E+00_JPRB, 0.65319E+00_JPRB, 0.63776E+00_JPRB, 0.63145E+00_JPRB ,&
&  0.63877E+00_JPRB, 0.65743E+00_JPRB, 0.67490E+00_JPRB, 0.67054E+00_JPRB, 0.67484E+00_JPRB, 0.68128E+00_JPRB, 0.68585E+00_JPRB ,&
&  0.70011E+00_JPRB, 0.70531E+00_JPRB, 0.70814E+00_JPRB, 0.73157E+00_JPRB, 0.73542E+00_JPRB, 0.78978E+00_JPRB, 0.74238E+00_JPRB ,&
&  0.70641E+00_JPRB, 0.68142E+00_JPRB, 0.66499E+00_JPRB, 0.65175E+00_JPRB, 0.64448E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.62725E+00_JPRB, 0.61696E+00_JPRB, 0.59856E+00_JPRB, 0.57961E+00_JPRB, 0.56715E+00_JPRB, 0.55770E+00_JPRB, 0.55111E+00_JPRB ,&
&  0.54659E+00_JPRB, 0.53754E+00_JPRB, 0.52880E+00_JPRB, 0.51734E+00_JPRB, 0.50338E+00_JPRB, 0.49174E+00_JPRB, 0.48422E+00_JPRB ,&
&  0.47470E+00_JPRB, 0.46932E+00_JPRB, 0.46330E+00_JPRB, 0.45248E+00_JPRB, 0.44076E+00_JPRB, 0.43125E+00_JPRB, 0.42529E+00_JPRB ,&
&  0.42274E+00_JPRB, 0.42546E+00_JPRB, 0.43222E+00_JPRB, 0.44673E+00_JPRB, 0.46076E+00_JPRB, 0.47640E+00_JPRB, 0.50293E+00_JPRB ,&
&  0.60150E+00_JPRB, 0.92417E+00_JPRB, 0.11433E+01_JPRB, 0.10158E+01_JPRB, 0.93547E+00_JPRB, 0.88062E+00_JPRB, 0.99974E+00_JPRB ,&
&  0.11072E+01_JPRB, 0.12181E+01_JPRB, 0.12801E+01_JPRB, 0.12980E+01_JPRB, 0.12227E+01_JPRB, 0.11661E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.98140E+00_JPRB, 0.89254E+00_JPRB, 0.80618E+00_JPRB, 0.75652E+00_JPRB, 0.75141E+00_JPRB, 0.78216E+00_JPRB ,&
&  0.84606E+00_JPRB, 0.90322E+00_JPRB, 0.91769E+00_JPRB, 0.97371E+00_JPRB, 0.10622E+01_JPRB, 0.10633E+01_JPRB, 0.13187E+01_JPRB ,&
&  0.16735E+01_JPRB, 0.15323E+01_JPRB, 0.13241E+01_JPRB, 0.11005E+01_JPRB, 0.92708E+00_JPRB, 0.78983E+00_JPRB, 0.71127E+00_JPRB ,&
&  0.65860E+00_JPRB, 0.62507E+00_JPRB, 0.60409E+00_JPRB, 0.59610E+00_JPRB, 0.60850E+00_JPRB, 0.63260E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.67715E+00_JPRB, 0.70035E+00_JPRB, 0.71133E+00_JPRB, 0.70654E+00_JPRB, 0.68686E+00_JPRB, 0.65796E+00_JPRB, 0.63195E+00_JPRB ,&
&  0.61966E+00_JPRB, 0.62167E+00_JPRB, 0.65541E+00_JPRB, 0.63931E+00_JPRB, 0.64147E+00_JPRB, 0.65426E+00_JPRB, 0.65712E+00_JPRB ,&
&  0.65807E+00_JPRB, 0.67722E+00_JPRB, 0.69235E+00_JPRB, 0.70027E+00_JPRB, 0.75797E+00_JPRB, 0.80884E+00_JPRB, 0.76076E+00_JPRB ,&
&  0.71563E+00_JPRB, 0.68009E+00_JPRB, 0.65973E+00_JPRB, 0.63636E+00_JPRB, 0.62830E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.62589E+00_JPRB, 0.60762E+00_JPRB, 0.57882E+00_JPRB, 0.55388E+00_JPRB, 0.53817E+00_JPRB, 0.52754E+00_JPRB, 0.52231E+00_JPRB ,&
&  0.51951E+00_JPRB, 0.51078E+00_JPRB, 0.50128E+00_JPRB, 0.48798E+00_JPRB, 0.47650E+00_JPRB, 0.46649E+00_JPRB, 0.45723E+00_JPRB ,&
&  0.44930E+00_JPRB, 0.44460E+00_JPRB, 0.44041E+00_JPRB, 0.43604E+00_JPRB, 0.42953E+00_JPRB, 0.42027E+00_JPRB, 0.41451E+00_JPRB ,&
&  0.41167E+00_JPRB, 0.41226E+00_JPRB, 0.41661E+00_JPRB, 0.42622E+00_JPRB, 0.43721E+00_JPRB, 0.45105E+00_JPRB, 0.47286E+00_JPRB ,&
&  0.53762E+00_JPRB, 0.78294E+00_JPRB, 0.82612E+00_JPRB, 0.80920E+00_JPRB, 0.80743E+00_JPRB, 0.79377E+00_JPRB, 0.83655E+00_JPRB ,&
&  0.91979E+00_JPRB, 0.99585E+00_JPRB, 0.10595E+01_JPRB, 0.10793E+01_JPRB, 0.10416E+01_JPRB, 0.10026E+01_JPRB, 0.95253E+00_JPRB ,&
&  0.89982E+00_JPRB, 0.84334E+00_JPRB, 0.78659E+00_JPRB, 0.73433E+00_JPRB, 0.69268E+00_JPRB, 0.68741E+00_JPRB, 0.70852E+00_JPRB ,&
&  0.75132E+00_JPRB, 0.78616E+00_JPRB, 0.80666E+00_JPRB, 0.85825E+00_JPRB, 0.98060E+00_JPRB, 0.10578E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11555E+01_JPRB, 0.10969E+01_JPRB, 0.10204E+01_JPRB, 0.91016E+00_JPRB, 0.80221E+00_JPRB, 0.71703E+00_JPRB ,&
&  0.65661E+00_JPRB, 0.61690E+00_JPRB, 0.59334E+00_JPRB, 0.58499E+00_JPRB, 0.59318E+00_JPRB, 0.61619E+00_JPRB, 0.64497E+00_JPRB ,&
&  0.67808E+00_JPRB, 0.71017E+00_JPRB, 0.72918E+00_JPRB, 0.73552E+00_JPRB, 0.72627E+00_JPRB, 0.70494E+00_JPRB, 0.67237E+00_JPRB ,&
&  0.64260E+00_JPRB, 0.62164E+00_JPRB, 0.62539E+00_JPRB, 0.62695E+00_JPRB, 0.62872E+00_JPRB, 0.63166E+00_JPRB, 0.63594E+00_JPRB ,&
&  0.64655E+00_JPRB, 0.68519E+00_JPRB, 0.68587E+00_JPRB, 0.69652E+00_JPRB, 0.73595E+00_JPRB, 0.75971E+00_JPRB, 0.72888E+00_JPRB ,&
&  0.70028E+00_JPRB, 0.67046E+00_JPRB, 0.64754E+00_JPRB, 0.62420E+00_JPRB, 0.63012E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.66607E+00_JPRB, 0.62848E+00_JPRB, 0.58202E+00_JPRB, 0.54612E+00_JPRB, 0.52090E+00_JPRB, 0.50287E+00_JPRB, 0.49292E+00_JPRB ,&
&  0.48921E+00_JPRB, 0.47817E+00_JPRB, 0.46216E+00_JPRB, 0.44882E+00_JPRB, 0.44194E+00_JPRB, 0.43680E+00_JPRB, 0.42975E+00_JPRB ,&
&  0.42389E+00_JPRB, 0.41863E+00_JPRB, 0.41488E+00_JPRB, 0.41333E+00_JPRB, 0.41246E+00_JPRB, 0.41079E+00_JPRB, 0.40640E+00_JPRB ,&
&  0.39994E+00_JPRB, 0.39806E+00_JPRB, 0.39936E+00_JPRB, 0.40642E+00_JPRB, 0.41659E+00_JPRB, 0.42908E+00_JPRB, 0.45008E+00_JPRB ,&
&  0.52128E+00_JPRB, 0.55276E+00_JPRB, 0.60189E+00_JPRB, 0.61314E+00_JPRB, 0.63047E+00_JPRB, 0.67602E+00_JPRB, 0.71840E+00_JPRB ,&
&  0.76053E+00_JPRB, 0.81607E+00_JPRB, 0.88209E+00_JPRB, 0.90829E+00_JPRB, 0.87978E+00_JPRB, 0.83057E+00_JPRB, 0.78734E+00_JPRB ,&
&  0.74709E+00_JPRB, 0.71881E+00_JPRB, 0.69201E+00_JPRB, 0.66724E+00_JPRB, 0.65172E+00_JPRB, 0.65166E+00_JPRB, 0.66219E+00_JPRB ,&
&  0.67033E+00_JPRB, 0.66504E+00_JPRB, 0.69612E+00_JPRB, 0.73862E+00_JPRB, 0.80629E+00_JPRB, 0.85670E+00_JPRB, 0.90686E+00_JPRB ,&
&  0.94684E+00_JPRB, 0.96332E+00_JPRB, 0.96992E+00_JPRB, 0.95364E+00_JPRB, 0.89663E+00_JPRB, 0.82726E+00_JPRB, 0.75585E+00_JPRB ,&
&  0.68930E+00_JPRB, 0.64593E+00_JPRB, 0.62412E+00_JPRB, 0.61437E+00_JPRB, 0.61387E+00_JPRB, 0.61960E+00_JPRB, 0.63255E+00_JPRB ,&
&  0.65229E+00_JPRB, 0.67106E+00_JPRB, 0.68632E+00_JPRB, 0.69333E+00_JPRB, 0.69127E+00_JPRB, 0.68668E+00_JPRB, 0.67939E+00_JPRB ,&
&  0.66993E+00_JPRB, 0.65758E+00_JPRB, 0.64838E+00_JPRB, 0.63945E+00_JPRB, 0.63217E+00_JPRB, 0.62756E+00_JPRB, 0.62468E+00_JPRB ,&
&  0.62274E+00_JPRB, 0.63967E+00_JPRB, 0.63979E+00_JPRB, 0.71336E+00_JPRB, 0.74640E+00_JPRB, 0.72879E+00_JPRB, 0.70378E+00_JPRB ,&
&  0.67285E+00_JPRB, 0.64444E+00_JPRB, 0.61911E+00_JPRB, 0.59762E+00_JPRB, 0.65410E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.56990E+00_JPRB, 0.56121E+00_JPRB, 0.53958E+00_JPRB, 0.51327E+00_JPRB, 0.49373E+00_JPRB, 0.47719E+00_JPRB, 0.46376E+00_JPRB ,&
&  0.45494E+00_JPRB, 0.44427E+00_JPRB, 0.43119E+00_JPRB, 0.42175E+00_JPRB, 0.42006E+00_JPRB, 0.41724E+00_JPRB, 0.41267E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.39715E+00_JPRB, 0.39203E+00_JPRB, 0.39021E+00_JPRB, 0.39078E+00_JPRB, 0.39266E+00_JPRB, 0.39334E+00_JPRB ,&
&  0.39094E+00_JPRB, 0.38657E+00_JPRB, 0.38615E+00_JPRB, 0.38839E+00_JPRB, 0.39398E+00_JPRB, 0.40311E+00_JPRB, 0.42362E+00_JPRB ,&
&  0.47837E+00_JPRB, 0.46720E+00_JPRB, 0.49090E+00_JPRB, 0.50611E+00_JPRB, 0.52835E+00_JPRB, 0.57192E+00_JPRB, 0.62020E+00_JPRB ,&
&  0.64923E+00_JPRB, 0.67264E+00_JPRB, 0.71624E+00_JPRB, 0.75207E+00_JPRB, 0.75001E+00_JPRB, 0.72297E+00_JPRB, 0.68609E+00_JPRB ,&
&  0.64928E+00_JPRB, 0.62193E+00_JPRB, 0.59923E+00_JPRB, 0.58115E+00_JPRB, 0.57012E+00_JPRB, 0.56644E+00_JPRB, 0.56624E+00_JPRB ,&
&  0.55885E+00_JPRB, 0.56068E+00_JPRB, 0.58228E+00_JPRB, 0.61830E+00_JPRB, 0.66150E+00_JPRB, 0.70519E+00_JPRB, 0.75254E+00_JPRB ,&
&  0.79587E+00_JPRB, 0.82654E+00_JPRB, 0.84851E+00_JPRB, 0.86271E+00_JPRB, 0.85170E+00_JPRB, 0.83131E+00_JPRB, 0.78451E+00_JPRB ,&
&  0.73181E+00_JPRB, 0.68864E+00_JPRB, 0.66100E+00_JPRB, 0.63962E+00_JPRB, 0.62455E+00_JPRB, 0.61545E+00_JPRB, 0.61061E+00_JPRB ,&
&  0.61030E+00_JPRB, 0.61343E+00_JPRB, 0.61478E+00_JPRB, 0.61372E+00_JPRB, 0.61175E+00_JPRB, 0.61038E+00_JPRB, 0.60846E+00_JPRB ,&
&  0.60684E+00_JPRB, 0.60527E+00_JPRB, 0.60352E+00_JPRB, 0.59831E+00_JPRB, 0.59082E+00_JPRB, 0.58318E+00_JPRB, 0.57680E+00_JPRB ,&
&  0.57157E+00_JPRB, 0.58027E+00_JPRB, 0.58418E+00_JPRB, 0.62417E+00_JPRB, 0.63816E+00_JPRB, 0.64096E+00_JPRB, 0.63570E+00_JPRB ,&
&  0.61897E+00_JPRB, 0.59983E+00_JPRB, 0.57260E+00_JPRB, 0.54952E+00_JPRB, 0.54980E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.51100E+00_JPRB, 0.50998E+00_JPRB, 0.50300E+00_JPRB, 0.48476E+00_JPRB, 0.46635E+00_JPRB, 0.45523E+00_JPRB, 0.44334E+00_JPRB ,&
&  0.43101E+00_JPRB, 0.42106E+00_JPRB, 0.40828E+00_JPRB, 0.40242E+00_JPRB, 0.39977E+00_JPRB, 0.39866E+00_JPRB, 0.39627E+00_JPRB ,&
&  0.39085E+00_JPRB, 0.38278E+00_JPRB, 0.37671E+00_JPRB, 0.37417E+00_JPRB, 0.37472E+00_JPRB, 0.37632E+00_JPRB, 0.37776E+00_JPRB ,&
&  0.37651E+00_JPRB, 0.37550E+00_JPRB, 0.37410E+00_JPRB, 0.37329E+00_JPRB, 0.37449E+00_JPRB, 0.38245E+00_JPRB, 0.39856E+00_JPRB ,&
&  0.44283E+00_JPRB, 0.40960E+00_JPRB, 0.42230E+00_JPRB, 0.44810E+00_JPRB, 0.47595E+00_JPRB, 0.51515E+00_JPRB, 0.55511E+00_JPRB ,&
&  0.57825E+00_JPRB, 0.57977E+00_JPRB, 0.59363E+00_JPRB, 0.62029E+00_JPRB, 0.62208E+00_JPRB, 0.61535E+00_JPRB, 0.59916E+00_JPRB ,&
&  0.57420E+00_JPRB, 0.54490E+00_JPRB, 0.51638E+00_JPRB, 0.49703E+00_JPRB, 0.48933E+00_JPRB, 0.48637E+00_JPRB, 0.48386E+00_JPRB ,&
&  0.47420E+00_JPRB, 0.47069E+00_JPRB, 0.48117E+00_JPRB, 0.49672E+00_JPRB, 0.51689E+00_JPRB, 0.55077E+00_JPRB, 0.59823E+00_JPRB ,&
&  0.63449E+00_JPRB, 0.67355E+00_JPRB, 0.71095E+00_JPRB, 0.73955E+00_JPRB, 0.76285E+00_JPRB, 0.76571E+00_JPRB, 0.74288E+00_JPRB ,&
&  0.71098E+00_JPRB, 0.67557E+00_JPRB, 0.64276E+00_JPRB, 0.61353E+00_JPRB, 0.59041E+00_JPRB, 0.57678E+00_JPRB, 0.56879E+00_JPRB ,&
&  0.56318E+00_JPRB, 0.55887E+00_JPRB, 0.55096E+00_JPRB, 0.53986E+00_JPRB, 0.52875E+00_JPRB, 0.51974E+00_JPRB, 0.51164E+00_JPRB ,&
&  0.50440E+00_JPRB, 0.49698E+00_JPRB, 0.49275E+00_JPRB, 0.49312E+00_JPRB, 0.49311E+00_JPRB, 0.49474E+00_JPRB, 0.49779E+00_JPRB ,&
&  0.49938E+00_JPRB, 0.50873E+00_JPRB, 0.50933E+00_JPRB, 0.58429E+00_JPRB, 0.59833E+00_JPRB, 0.59267E+00_JPRB, 0.58299E+00_JPRB ,&
&  0.56659E+00_JPRB, 0.55006E+00_JPRB, 0.53475E+00_JPRB, 0.48929E+00_JPRB, 0.51156E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.46515E+00_JPRB, 0.46523E+00_JPRB, 0.45989E+00_JPRB, 0.44947E+00_JPRB, 0.43669E+00_JPRB, 0.42790E+00_JPRB, 0.41639E+00_JPRB ,&
&  0.40446E+00_JPRB, 0.39770E+00_JPRB, 0.39004E+00_JPRB, 0.38592E+00_JPRB, 0.38547E+00_JPRB, 0.38147E+00_JPRB, 0.37795E+00_JPRB ,&
&  0.37475E+00_JPRB, 0.36922E+00_JPRB, 0.36567E+00_JPRB, 0.36266E+00_JPRB, 0.36229E+00_JPRB, 0.36248E+00_JPRB, 0.36216E+00_JPRB ,&
&  0.36025E+00_JPRB, 0.35709E+00_JPRB, 0.35435E+00_JPRB, 0.35250E+00_JPRB, 0.35419E+00_JPRB, 0.36089E+00_JPRB, 0.37425E+00_JPRB ,&
&  0.40627E+00_JPRB, 0.36801E+00_JPRB, 0.38354E+00_JPRB, 0.40211E+00_JPRB, 0.43195E+00_JPRB, 0.45640E+00_JPRB, 0.46849E+00_JPRB ,&
&  0.48254E+00_JPRB, 0.49720E+00_JPRB, 0.50979E+00_JPRB, 0.52177E+00_JPRB, 0.51924E+00_JPRB, 0.51105E+00_JPRB, 0.50362E+00_JPRB ,&
&  0.49277E+00_JPRB, 0.47908E+00_JPRB, 0.46502E+00_JPRB, 0.45046E+00_JPRB, 0.44173E+00_JPRB, 0.43698E+00_JPRB, 0.43067E+00_JPRB ,&
&  0.42118E+00_JPRB, 0.41219E+00_JPRB, 0.40663E+00_JPRB, 0.40692E+00_JPRB, 0.41451E+00_JPRB, 0.42977E+00_JPRB, 0.45917E+00_JPRB ,&
&  0.48990E+00_JPRB, 0.52398E+00_JPRB, 0.56085E+00_JPRB, 0.59083E+00_JPRB, 0.61314E+00_JPRB, 0.62158E+00_JPRB, 0.62070E+00_JPRB ,&
&  0.61021E+00_JPRB, 0.59639E+00_JPRB, 0.58350E+00_JPRB, 0.56721E+00_JPRB, 0.54694E+00_JPRB, 0.52859E+00_JPRB, 0.51415E+00_JPRB ,&
&  0.50314E+00_JPRB, 0.49164E+00_JPRB, 0.47865E+00_JPRB, 0.46471E+00_JPRB, 0.45029E+00_JPRB, 0.43755E+00_JPRB, 0.42683E+00_JPRB ,&
&  0.41769E+00_JPRB, 0.40961E+00_JPRB, 0.40523E+00_JPRB, 0.40646E+00_JPRB, 0.41118E+00_JPRB, 0.41686E+00_JPRB, 0.42206E+00_JPRB ,&
&  0.42853E+00_JPRB, 0.43961E+00_JPRB, 0.44614E+00_JPRB, 0.47652E+00_JPRB, 0.49142E+00_JPRB, 0.49774E+00_JPRB, 0.50177E+00_JPRB ,&
&  0.50105E+00_JPRB, 0.49456E+00_JPRB, 0.43400E+00_JPRB, 0.46228E+00_JPRB, 0.46570E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.43018E+00_JPRB, 0.42196E+00_JPRB, 0.41389E+00_JPRB, 0.41230E+00_JPRB, 0.40832E+00_JPRB, 0.40243E+00_JPRB, 0.39453E+00_JPRB ,&
&  0.38298E+00_JPRB, 0.37481E+00_JPRB, 0.36882E+00_JPRB, 0.36686E+00_JPRB, 0.36906E+00_JPRB, 0.36685E+00_JPRB, 0.36207E+00_JPRB ,&
&  0.35851E+00_JPRB, 0.35446E+00_JPRB, 0.35157E+00_JPRB, 0.34817E+00_JPRB, 0.34541E+00_JPRB, 0.34417E+00_JPRB, 0.34339E+00_JPRB ,&
&  0.34144E+00_JPRB, 0.33813E+00_JPRB, 0.33527E+00_JPRB, 0.33377E+00_JPRB, 0.33601E+00_JPRB, 0.34202E+00_JPRB, 0.35153E+00_JPRB ,&
&  0.34971E+00_JPRB, 0.33251E+00_JPRB, 0.35090E+00_JPRB, 0.37392E+00_JPRB, 0.38937E+00_JPRB, 0.40484E+00_JPRB, 0.41700E+00_JPRB ,&
&  0.42804E+00_JPRB, 0.43192E+00_JPRB, 0.43637E+00_JPRB, 0.44030E+00_JPRB, 0.44155E+00_JPRB, 0.43809E+00_JPRB, 0.43376E+00_JPRB ,&
&  0.43014E+00_JPRB, 0.42687E+00_JPRB, 0.42297E+00_JPRB, 0.41602E+00_JPRB, 0.40848E+00_JPRB, 0.40037E+00_JPRB, 0.39247E+00_JPRB ,&
&  0.38563E+00_JPRB, 0.37909E+00_JPRB, 0.37433E+00_JPRB, 0.37239E+00_JPRB, 0.37393E+00_JPRB, 0.37768E+00_JPRB, 0.38488E+00_JPRB ,&
&  0.39381E+00_JPRB, 0.40843E+00_JPRB, 0.43333E+00_JPRB, 0.45727E+00_JPRB, 0.47584E+00_JPRB, 0.48820E+00_JPRB, 0.49759E+00_JPRB ,&
&  0.49921E+00_JPRB, 0.49693E+00_JPRB, 0.49389E+00_JPRB, 0.48821E+00_JPRB, 0.47934E+00_JPRB, 0.46739E+00_JPRB, 0.45314E+00_JPRB ,&
&  0.44066E+00_JPRB, 0.42991E+00_JPRB, 0.41966E+00_JPRB, 0.40988E+00_JPRB, 0.40021E+00_JPRB, 0.39098E+00_JPRB, 0.38301E+00_JPRB ,&
&  0.37583E+00_JPRB, 0.36902E+00_JPRB, 0.36430E+00_JPRB, 0.36326E+00_JPRB, 0.36631E+00_JPRB, 0.37104E+00_JPRB, 0.37682E+00_JPRB ,&
&  0.38277E+00_JPRB, 0.39266E+00_JPRB, 0.40148E+00_JPRB, 0.41772E+00_JPRB, 0.43362E+00_JPRB, 0.44014E+00_JPRB, 0.44550E+00_JPRB ,&
&  0.44987E+00_JPRB, 0.44779E+00_JPRB, 0.44700E+00_JPRB, 0.44144E+00_JPRB, 0.43424E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.39172E+00_JPRB, 0.38232E+00_JPRB, 0.37862E+00_JPRB, 0.38023E+00_JPRB, 0.38293E+00_JPRB, 0.38114E+00_JPRB, 0.37586E+00_JPRB ,&
&  0.36548E+00_JPRB, 0.35617E+00_JPRB, 0.34865E+00_JPRB, 0.34503E+00_JPRB, 0.34635E+00_JPRB, 0.34630E+00_JPRB, 0.34340E+00_JPRB ,&
&  0.34036E+00_JPRB, 0.33506E+00_JPRB, 0.33057E+00_JPRB, 0.32606E+00_JPRB, 0.32263E+00_JPRB, 0.32103E+00_JPRB, 0.32027E+00_JPRB ,&
&  0.31856E+00_JPRB, 0.31643E+00_JPRB, 0.31572E+00_JPRB, 0.31544E+00_JPRB, 0.31701E+00_JPRB, 0.32112E+00_JPRB, 0.32645E+00_JPRB ,&
&  0.33611E+00_JPRB, 0.32170E+00_JPRB, 0.34002E+00_JPRB, 0.34506E+00_JPRB, 0.35051E+00_JPRB, 0.35917E+00_JPRB, 0.36984E+00_JPRB ,&
&  0.37948E+00_JPRB, 0.38446E+00_JPRB, 0.38708E+00_JPRB, 0.38809E+00_JPRB, 0.38707E+00_JPRB, 0.38677E+00_JPRB, 0.38392E+00_JPRB ,&
&  0.37929E+00_JPRB, 0.37725E+00_JPRB, 0.37626E+00_JPRB, 0.37406E+00_JPRB, 0.37170E+00_JPRB, 0.36791E+00_JPRB, 0.36356E+00_JPRB ,&
&  0.35937E+00_JPRB, 0.35509E+00_JPRB, 0.35146E+00_JPRB, 0.34795E+00_JPRB, 0.34530E+00_JPRB, 0.34425E+00_JPRB, 0.34503E+00_JPRB ,&
&  0.34649E+00_JPRB, 0.34984E+00_JPRB, 0.35828E+00_JPRB, 0.36977E+00_JPRB, 0.38157E+00_JPRB, 0.39176E+00_JPRB, 0.40012E+00_JPRB ,&
&  0.40514E+00_JPRB, 0.40760E+00_JPRB, 0.40739E+00_JPRB, 0.40558E+00_JPRB, 0.40174E+00_JPRB, 0.39707E+00_JPRB, 0.39034E+00_JPRB ,&
&  0.38405E+00_JPRB, 0.37842E+00_JPRB, 0.37242E+00_JPRB, 0.36691E+00_JPRB, 0.36081E+00_JPRB, 0.35407E+00_JPRB, 0.34790E+00_JPRB ,&
&  0.34309E+00_JPRB, 0.33968E+00_JPRB, 0.33720E+00_JPRB, 0.33603E+00_JPRB, 0.33624E+00_JPRB, 0.33859E+00_JPRB, 0.34141E+00_JPRB ,&
&  0.34507E+00_JPRB, 0.35114E+00_JPRB, 0.35851E+00_JPRB, 0.36773E+00_JPRB, 0.38015E+00_JPRB, 0.38915E+00_JPRB, 0.39656E+00_JPRB ,&
&  0.40166E+00_JPRB, 0.40256E+00_JPRB, 0.40389E+00_JPRB, 0.40203E+00_JPRB, 0.39836E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.35398E+00_JPRB, 0.34875E+00_JPRB, 0.34973E+00_JPRB, 0.35406E+00_JPRB, 0.36050E+00_JPRB, 0.35909E+00_JPRB, 0.35627E+00_JPRB ,&
&  0.35022E+00_JPRB, 0.34244E+00_JPRB, 0.33482E+00_JPRB, 0.32776E+00_JPRB, 0.32451E+00_JPRB, 0.32466E+00_JPRB, 0.32425E+00_JPRB ,&
&  0.32270E+00_JPRB, 0.31890E+00_JPRB, 0.31360E+00_JPRB, 0.30842E+00_JPRB, 0.30461E+00_JPRB, 0.30238E+00_JPRB, 0.30045E+00_JPRB ,&
&  0.29913E+00_JPRB, 0.29730E+00_JPRB, 0.29781E+00_JPRB, 0.29755E+00_JPRB, 0.29811E+00_JPRB, 0.30168E+00_JPRB, 0.30490E+00_JPRB ,&
&  0.30676E+00_JPRB, 0.31403E+00_JPRB, 0.31221E+00_JPRB, 0.31186E+00_JPRB, 0.31531E+00_JPRB, 0.32339E+00_JPRB, 0.33411E+00_JPRB ,&
&  0.34390E+00_JPRB, 0.34979E+00_JPRB, 0.35205E+00_JPRB, 0.35051E+00_JPRB, 0.34784E+00_JPRB, 0.34592E+00_JPRB, 0.34515E+00_JPRB ,&
&  0.34386E+00_JPRB, 0.34032E+00_JPRB, 0.33612E+00_JPRB, 0.33253E+00_JPRB, 0.33101E+00_JPRB, 0.32991E+00_JPRB, 0.32830E+00_JPRB ,&
&  0.32613E+00_JPRB, 0.32366E+00_JPRB, 0.32005E+00_JPRB, 0.31600E+00_JPRB, 0.31227E+00_JPRB, 0.31023E+00_JPRB, 0.31021E+00_JPRB ,&
&  0.31189E+00_JPRB, 0.31481E+00_JPRB, 0.31906E+00_JPRB, 0.32454E+00_JPRB, 0.32974E+00_JPRB, 0.33529E+00_JPRB, 0.33948E+00_JPRB ,&
&  0.34232E+00_JPRB, 0.34366E+00_JPRB, 0.34388E+00_JPRB, 0.34342E+00_JPRB, 0.34217E+00_JPRB, 0.34019E+00_JPRB, 0.33790E+00_JPRB ,&
&  0.33570E+00_JPRB, 0.33352E+00_JPRB, 0.33127E+00_JPRB, 0.32882E+00_JPRB, 0.32517E+00_JPRB, 0.32116E+00_JPRB, 0.31817E+00_JPRB ,&
&  0.31638E+00_JPRB, 0.31534E+00_JPRB, 0.31459E+00_JPRB, 0.31369E+00_JPRB, 0.31339E+00_JPRB, 0.31427E+00_JPRB, 0.31625E+00_JPRB ,&
&  0.31926E+00_JPRB, 0.32343E+00_JPRB, 0.32871E+00_JPRB, 0.33450E+00_JPRB, 0.34097E+00_JPRB, 0.34801E+00_JPRB, 0.35500E+00_JPRB ,&
&  0.35997E+00_JPRB, 0.36119E+00_JPRB, 0.36213E+00_JPRB, 0.36263E+00_JPRB, 0.35972E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.32208E+00_JPRB, 0.32157E+00_JPRB, 0.32590E+00_JPRB, 0.33102E+00_JPRB, 0.33693E+00_JPRB, 0.33854E+00_JPRB, 0.33604E+00_JPRB ,&
&  0.33294E+00_JPRB, 0.32883E+00_JPRB, 0.32276E+00_JPRB, 0.31524E+00_JPRB, 0.30844E+00_JPRB, 0.30643E+00_JPRB, 0.30642E+00_JPRB ,&
&  0.30658E+00_JPRB, 0.30466E+00_JPRB, 0.30087E+00_JPRB, 0.29658E+00_JPRB, 0.29161E+00_JPRB, 0.28708E+00_JPRB, 0.28350E+00_JPRB ,&
&  0.28207E+00_JPRB, 0.28121E+00_JPRB, 0.28149E+00_JPRB, 0.28170E+00_JPRB, 0.28243E+00_JPRB, 0.28502E+00_JPRB, 0.29046E+00_JPRB ,&
&  0.28845E+00_JPRB, 0.29108E+00_JPRB, 0.28334E+00_JPRB, 0.28591E+00_JPRB, 0.28863E+00_JPRB, 0.29357E+00_JPRB, 0.29965E+00_JPRB ,&
&  0.30617E+00_JPRB, 0.31146E+00_JPRB, 0.31469E+00_JPRB, 0.31488E+00_JPRB, 0.31339E+00_JPRB, 0.31343E+00_JPRB, 0.31367E+00_JPRB ,&
&  0.31329E+00_JPRB, 0.31250E+00_JPRB, 0.31056E+00_JPRB, 0.30654E+00_JPRB, 0.30096E+00_JPRB, 0.29613E+00_JPRB, 0.29269E+00_JPRB ,&
&  0.29183E+00_JPRB, 0.29129E+00_JPRB, 0.28964E+00_JPRB, 0.28783E+00_JPRB, 0.28593E+00_JPRB, 0.28482E+00_JPRB, 0.28532E+00_JPRB ,&
&  0.28665E+00_JPRB, 0.28785E+00_JPRB, 0.28874E+00_JPRB, 0.29056E+00_JPRB, 0.29365E+00_JPRB, 0.29675E+00_JPRB, 0.29942E+00_JPRB ,&
&  0.30109E+00_JPRB, 0.30122E+00_JPRB, 0.30081E+00_JPRB, 0.30006E+00_JPRB, 0.29956E+00_JPRB, 0.29902E+00_JPRB, 0.29903E+00_JPRB ,&
&  0.29900E+00_JPRB, 0.29853E+00_JPRB, 0.29728E+00_JPRB, 0.29532E+00_JPRB, 0.29347E+00_JPRB, 0.29189E+00_JPRB, 0.29112E+00_JPRB ,&
&  0.29110E+00_JPRB, 0.29129E+00_JPRB, 0.29103E+00_JPRB, 0.29050E+00_JPRB, 0.29079E+00_JPRB, 0.29150E+00_JPRB, 0.29302E+00_JPRB ,&
&  0.29516E+00_JPRB, 0.29783E+00_JPRB, 0.30144E+00_JPRB, 0.30515E+00_JPRB, 0.30670E+00_JPRB, 0.30929E+00_JPRB, 0.31425E+00_JPRB ,&
&  0.31898E+00_JPRB, 0.32153E+00_JPRB, 0.32299E+00_JPRB, 0.32395E+00_JPRB, 0.32416E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.29765E+00_JPRB, 0.30016E+00_JPRB, 0.30430E+00_JPRB, 0.31018E+00_JPRB, 0.31414E+00_JPRB, 0.31815E+00_JPRB, 0.31816E+00_JPRB ,&
&  0.31569E+00_JPRB, 0.31208E+00_JPRB, 0.30764E+00_JPRB, 0.30210E+00_JPRB, 0.29631E+00_JPRB, 0.29196E+00_JPRB, 0.29035E+00_JPRB ,&
&  0.29012E+00_JPRB, 0.28891E+00_JPRB, 0.28595E+00_JPRB, 0.28191E+00_JPRB, 0.27732E+00_JPRB, 0.27338E+00_JPRB, 0.27014E+00_JPRB ,&
&  0.26884E+00_JPRB, 0.26893E+00_JPRB, 0.26975E+00_JPRB, 0.27014E+00_JPRB, 0.27018E+00_JPRB, 0.27234E+00_JPRB, 0.27627E+00_JPRB ,&
&  0.27426E+00_JPRB, 0.27199E+00_JPRB, 0.26605E+00_JPRB, 0.26548E+00_JPRB, 0.26698E+00_JPRB, 0.26943E+00_JPRB, 0.27212E+00_JPRB ,&
&  0.27481E+00_JPRB, 0.27795E+00_JPRB, 0.28130E+00_JPRB, 0.28392E+00_JPRB, 0.28565E+00_JPRB, 0.28720E+00_JPRB, 0.28820E+00_JPRB ,&
&  0.28859E+00_JPRB, 0.28763E+00_JPRB, 0.28567E+00_JPRB, 0.28209E+00_JPRB, 0.27749E+00_JPRB, 0.27430E+00_JPRB, 0.27276E+00_JPRB ,&
&  0.27122E+00_JPRB, 0.27094E+00_JPRB, 0.27076E+00_JPRB, 0.26997E+00_JPRB, 0.26885E+00_JPRB, 0.26851E+00_JPRB, 0.26907E+00_JPRB ,&
&  0.26946E+00_JPRB, 0.26897E+00_JPRB, 0.26784E+00_JPRB, 0.26755E+00_JPRB, 0.26838E+00_JPRB, 0.27005E+00_JPRB, 0.27157E+00_JPRB ,&
&  0.27270E+00_JPRB, 0.27252E+00_JPRB, 0.27178E+00_JPRB, 0.27005E+00_JPRB, 0.26882E+00_JPRB, 0.26902E+00_JPRB, 0.27030E+00_JPRB ,&
&  0.27115E+00_JPRB, 0.27116E+00_JPRB, 0.27127E+00_JPRB, 0.27007E+00_JPRB, 0.26892E+00_JPRB, 0.26880E+00_JPRB, 0.26908E+00_JPRB ,&
&  0.26903E+00_JPRB, 0.26872E+00_JPRB, 0.26832E+00_JPRB, 0.26817E+00_JPRB, 0.26866E+00_JPRB, 0.26883E+00_JPRB, 0.26984E+00_JPRB ,&
&  0.27244E+00_JPRB, 0.27540E+00_JPRB, 0.27820E+00_JPRB, 0.28011E+00_JPRB, 0.28119E+00_JPRB, 0.28201E+00_JPRB, 0.28541E+00_JPRB ,&
&  0.28994E+00_JPRB, 0.29334E+00_JPRB, 0.29410E+00_JPRB, 0.29414E+00_JPRB, 0.29572E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.27794E+00_JPRB, 0.28099E+00_JPRB, 0.28581E+00_JPRB, 0.29064E+00_JPRB, 0.29370E+00_JPRB, 0.29847E+00_JPRB, 0.29961E+00_JPRB ,&
&  0.29907E+00_JPRB, 0.29730E+00_JPRB, 0.29310E+00_JPRB, 0.28793E+00_JPRB, 0.28301E+00_JPRB, 0.27890E+00_JPRB, 0.27600E+00_JPRB ,&
&  0.27428E+00_JPRB, 0.27275E+00_JPRB, 0.27090E+00_JPRB, 0.26805E+00_JPRB, 0.26454E+00_JPRB, 0.26124E+00_JPRB, 0.25857E+00_JPRB ,&
&  0.25744E+00_JPRB, 0.25721E+00_JPRB, 0.25744E+00_JPRB, 0.25719E+00_JPRB, 0.25681E+00_JPRB, 0.25785E+00_JPRB, 0.25943E+00_JPRB ,&
&  0.25833E+00_JPRB, 0.25406E+00_JPRB, 0.25120E+00_JPRB, 0.25091E+00_JPRB, 0.25106E+00_JPRB, 0.25164E+00_JPRB, 0.25249E+00_JPRB ,&
&  0.25295E+00_JPRB, 0.25378E+00_JPRB, 0.25542E+00_JPRB, 0.25729E+00_JPRB, 0.25881E+00_JPRB, 0.26151E+00_JPRB, 0.26291E+00_JPRB ,&
&  0.26312E+00_JPRB, 0.26240E+00_JPRB, 0.26102E+00_JPRB, 0.25846E+00_JPRB, 0.25664E+00_JPRB, 0.25609E+00_JPRB, 0.25539E+00_JPRB ,&
&  0.25602E+00_JPRB, 0.25666E+00_JPRB, 0.25725E+00_JPRB, 0.25630E+00_JPRB, 0.25447E+00_JPRB, 0.25419E+00_JPRB, 0.25466E+00_JPRB ,&
&  0.25462E+00_JPRB, 0.25342E+00_JPRB, 0.25142E+00_JPRB, 0.25013E+00_JPRB, 0.24906E+00_JPRB, 0.24885E+00_JPRB, 0.24874E+00_JPRB ,&
&  0.24848E+00_JPRB, 0.24742E+00_JPRB, 0.24676E+00_JPRB, 0.24655E+00_JPRB, 0.24650E+00_JPRB, 0.24698E+00_JPRB, 0.24680E+00_JPRB ,&
&  0.24707E+00_JPRB, 0.24740E+00_JPRB, 0.24643E+00_JPRB, 0.24599E+00_JPRB, 0.24575E+00_JPRB, 0.24592E+00_JPRB, 0.24589E+00_JPRB ,&
&  0.24549E+00_JPRB, 0.24498E+00_JPRB, 0.24470E+00_JPRB, 0.24487E+00_JPRB, 0.24451E+00_JPRB, 0.24379E+00_JPRB, 0.24461E+00_JPRB ,&
&  0.24784E+00_JPRB, 0.25127E+00_JPRB, 0.25420E+00_JPRB, 0.25609E+00_JPRB, 0.25675E+00_JPRB, 0.25830E+00_JPRB, 0.26151E+00_JPRB ,&
&  0.26460E+00_JPRB, 0.26653E+00_JPRB, 0.26950E+00_JPRB, 0.27183E+00_JPRB, 0.27425E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.26072E+00_JPRB, 0.26479E+00_JPRB, 0.26818E+00_JPRB, 0.27204E+00_JPRB, 0.27472E+00_JPRB, 0.27949E+00_JPRB, 0.28171E+00_JPRB ,&
&  0.28184E+00_JPRB, 0.28159E+00_JPRB, 0.28002E+00_JPRB, 0.27731E+00_JPRB, 0.27317E+00_JPRB, 0.26891E+00_JPRB, 0.26488E+00_JPRB ,&
&  0.26172E+00_JPRB, 0.25909E+00_JPRB, 0.25694E+00_JPRB, 0.25495E+00_JPRB, 0.25226E+00_JPRB, 0.24974E+00_JPRB, 0.24776E+00_JPRB ,&
&  0.24684E+00_JPRB, 0.24613E+00_JPRB, 0.24575E+00_JPRB, 0.24552E+00_JPRB, 0.24506E+00_JPRB, 0.24454E+00_JPRB, 0.24440E+00_JPRB ,&
&  0.24314E+00_JPRB, 0.23958E+00_JPRB, 0.23754E+00_JPRB, 0.23687E+00_JPRB, 0.23631E+00_JPRB, 0.23680E+00_JPRB, 0.23654E+00_JPRB ,&
&  0.23638E+00_JPRB, 0.23693E+00_JPRB, 0.23851E+00_JPRB, 0.24041E+00_JPRB, 0.24076E+00_JPRB, 0.24157E+00_JPRB, 0.24298E+00_JPRB ,&
&  0.24355E+00_JPRB, 0.24288E+00_JPRB, 0.24111E+00_JPRB, 0.24061E+00_JPRB, 0.24135E+00_JPRB, 0.24142E+00_JPRB, 0.24179E+00_JPRB ,&
&  0.24240E+00_JPRB, 0.24349E+00_JPRB, 0.24405E+00_JPRB, 0.24239E+00_JPRB, 0.24072E+00_JPRB, 0.24011E+00_JPRB, 0.24009E+00_JPRB ,&
&  0.23977E+00_JPRB, 0.23873E+00_JPRB, 0.23654E+00_JPRB, 0.23449E+00_JPRB, 0.23156E+00_JPRB, 0.22918E+00_JPRB, 0.22794E+00_JPRB ,&
&  0.22835E+00_JPRB, 0.22881E+00_JPRB, 0.23013E+00_JPRB, 0.23132E+00_JPRB, 0.23207E+00_JPRB, 0.23215E+00_JPRB, 0.23060E+00_JPRB ,&
&  0.22856E+00_JPRB, 0.22761E+00_JPRB, 0.22657E+00_JPRB, 0.22556E+00_JPRB, 0.22574E+00_JPRB, 0.22559E+00_JPRB, 0.22519E+00_JPRB ,&
&  0.22475E+00_JPRB, 0.22524E+00_JPRB, 0.22578E+00_JPRB, 0.22568E+00_JPRB, 0.22514E+00_JPRB, 0.22433E+00_JPRB, 0.22491E+00_JPRB ,&
&  0.22643E+00_JPRB, 0.22857E+00_JPRB, 0.23101E+00_JPRB, 0.23362E+00_JPRB, 0.23637E+00_JPRB, 0.23893E+00_JPRB, 0.24164E+00_JPRB ,&
&  0.24504E+00_JPRB, 0.24658E+00_JPRB, 0.24907E+00_JPRB, 0.25263E+00_JPRB, 0.25607E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.24132E+00_JPRB, 0.24663E+00_JPRB, 0.24999E+00_JPRB, 0.25309E+00_JPRB, 0.25592E+00_JPRB, 0.26051E+00_JPRB, 0.26371E+00_JPRB ,&
&  0.26439E+00_JPRB, 0.26449E+00_JPRB, 0.26411E+00_JPRB, 0.26297E+00_JPRB, 0.26109E+00_JPRB, 0.25927E+00_JPRB, 0.25629E+00_JPRB ,&
&  0.25260E+00_JPRB, 0.24918E+00_JPRB, 0.24609E+00_JPRB, 0.24320E+00_JPRB, 0.24053E+00_JPRB, 0.23810E+00_JPRB, 0.23622E+00_JPRB ,&
&  0.23567E+00_JPRB, 0.23548E+00_JPRB, 0.23456E+00_JPRB, 0.23426E+00_JPRB, 0.23418E+00_JPRB, 0.23330E+00_JPRB, 0.23176E+00_JPRB ,&
&  0.22990E+00_JPRB, 0.22678E+00_JPRB, 0.22360E+00_JPRB, 0.22262E+00_JPRB, 0.22259E+00_JPRB, 0.22181E+00_JPRB, 0.22069E+00_JPRB ,&
&  0.22194E+00_JPRB, 0.22406E+00_JPRB, 0.22643E+00_JPRB, 0.22766E+00_JPRB, 0.22849E+00_JPRB, 0.22981E+00_JPRB, 0.23143E+00_JPRB ,&
&  0.23172E+00_JPRB, 0.22921E+00_JPRB, 0.22695E+00_JPRB, 0.22654E+00_JPRB, 0.22652E+00_JPRB, 0.22618E+00_JPRB, 0.22654E+00_JPRB ,&
&  0.22718E+00_JPRB, 0.22864E+00_JPRB, 0.22961E+00_JPRB, 0.22858E+00_JPRB, 0.22671E+00_JPRB, 0.22534E+00_JPRB, 0.22448E+00_JPRB ,&
&  0.22403E+00_JPRB, 0.22348E+00_JPRB, 0.22249E+00_JPRB, 0.22091E+00_JPRB, 0.21912E+00_JPRB, 0.21737E+00_JPRB, 0.21556E+00_JPRB ,&
&  0.21402E+00_JPRB, 0.21412E+00_JPRB, 0.21643E+00_JPRB, 0.21797E+00_JPRB, 0.21771E+00_JPRB, 0.21638E+00_JPRB, 0.21455E+00_JPRB ,&
&  0.21330E+00_JPRB, 0.21087E+00_JPRB, 0.20886E+00_JPRB, 0.20731E+00_JPRB, 0.20619E+00_JPRB, 0.20468E+00_JPRB, 0.20415E+00_JPRB ,&
&  0.20458E+00_JPRB, 0.20640E+00_JPRB, 0.20709E+00_JPRB, 0.20723E+00_JPRB, 0.20688E+00_JPRB, 0.20659E+00_JPRB, 0.20756E+00_JPRB ,&
&  0.20889E+00_JPRB, 0.21098E+00_JPRB, 0.21329E+00_JPRB, 0.21634E+00_JPRB, 0.21917E+00_JPRB, 0.22270E+00_JPRB, 0.22567E+00_JPRB ,&
&  0.22758E+00_JPRB, 0.22830E+00_JPRB, 0.22981E+00_JPRB, 0.23204E+00_JPRB, 0.23561E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.22118E+00_JPRB, 0.22692E+00_JPRB, 0.23200E+00_JPRB, 0.23495E+00_JPRB, 0.23703E+00_JPRB, 0.23976E+00_JPRB, 0.24229E+00_JPRB ,&
&  0.24409E+00_JPRB, 0.24516E+00_JPRB, 0.24567E+00_JPRB, 0.24556E+00_JPRB, 0.24460E+00_JPRB, 0.24301E+00_JPRB, 0.24112E+00_JPRB ,&
&  0.23902E+00_JPRB, 0.23613E+00_JPRB, 0.23345E+00_JPRB, 0.23142E+00_JPRB, 0.22973E+00_JPRB, 0.22780E+00_JPRB, 0.22625E+00_JPRB ,&
&  0.22468E+00_JPRB, 0.22420E+00_JPRB, 0.22356E+00_JPRB, 0.22286E+00_JPRB, 0.22222E+00_JPRB, 0.22115E+00_JPRB, 0.22023E+00_JPRB ,&
&  0.21783E+00_JPRB, 0.21389E+00_JPRB, 0.21222E+00_JPRB, 0.21124E+00_JPRB, 0.21138E+00_JPRB, 0.21077E+00_JPRB, 0.20995E+00_JPRB ,&
&  0.21159E+00_JPRB, 0.21453E+00_JPRB, 0.21674E+00_JPRB, 0.21794E+00_JPRB, 0.21876E+00_JPRB, 0.21997E+00_JPRB, 0.22133E+00_JPRB ,&
&  0.21912E+00_JPRB, 0.21634E+00_JPRB, 0.21431E+00_JPRB, 0.21331E+00_JPRB, 0.21321E+00_JPRB, 0.21403E+00_JPRB, 0.21568E+00_JPRB ,&
&  0.21718E+00_JPRB, 0.21806E+00_JPRB, 0.21774E+00_JPRB, 0.21636E+00_JPRB, 0.21465E+00_JPRB, 0.21384E+00_JPRB, 0.21337E+00_JPRB ,&
&  0.21311E+00_JPRB, 0.21272E+00_JPRB, 0.21224E+00_JPRB, 0.21103E+00_JPRB, 0.20516E+00_JPRB, 0.20159E+00_JPRB, 0.19896E+00_JPRB ,&
&  0.19960E+00_JPRB, 0.19934E+00_JPRB, 0.19750E+00_JPRB, 0.20310E+00_JPRB, 0.20910E+00_JPRB, 0.20429E+00_JPRB, 0.19858E+00_JPRB ,&
&  0.19078E+00_JPRB, 0.19188E+00_JPRB, 0.18926E+00_JPRB, 0.18440E+00_JPRB, 0.18307E+00_JPRB, 0.18330E+00_JPRB, 0.18300E+00_JPRB ,&
&  0.18389E+00_JPRB, 0.18404E+00_JPRB, 0.18527E+00_JPRB, 0.18586E+00_JPRB, 0.18621E+00_JPRB, 0.18714E+00_JPRB, 0.18806E+00_JPRB ,&
&  0.19028E+00_JPRB, 0.19417E+00_JPRB, 0.19861E+00_JPRB, 0.19992E+00_JPRB, 0.20285E+00_JPRB, 0.20545E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20852E+00_JPRB, 0.21048E+00_JPRB, 0.21234E+00_JPRB, 0.21393E+00_JPRB, 0.21663E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.20732E+00_JPRB, 0.20983E+00_JPRB, 0.21289E+00_JPRB, 0.21601E+00_JPRB, 0.21836E+00_JPRB, 0.22071E+00_JPRB, 0.22278E+00_JPRB ,&
&  0.22451E+00_JPRB, 0.22565E+00_JPRB, 0.22640E+00_JPRB, 0.22682E+00_JPRB, 0.22709E+00_JPRB, 0.22679E+00_JPRB, 0.22604E+00_JPRB ,&
&  0.22525E+00_JPRB, 0.22390E+00_JPRB, 0.22211E+00_JPRB, 0.22063E+00_JPRB, 0.21961E+00_JPRB, 0.21834E+00_JPRB, 0.21657E+00_JPRB ,&
&  0.21476E+00_JPRB, 0.21321E+00_JPRB, 0.21201E+00_JPRB, 0.21079E+00_JPRB, 0.21069E+00_JPRB, 0.21125E+00_JPRB, 0.21171E+00_JPRB ,&
&  0.20716E+00_JPRB, 0.20188E+00_JPRB, 0.20305E+00_JPRB, 0.20407E+00_JPRB, 0.20172E+00_JPRB, 0.20302E+00_JPRB, 0.20379E+00_JPRB ,&
&  0.20487E+00_JPRB, 0.20631E+00_JPRB, 0.20776E+00_JPRB, 0.20848E+00_JPRB, 0.20897E+00_JPRB, 0.20973E+00_JPRB, 0.20916E+00_JPRB ,&
&  0.20712E+00_JPRB, 0.20535E+00_JPRB, 0.20377E+00_JPRB, 0.20215E+00_JPRB, 0.20172E+00_JPRB, 0.20334E+00_JPRB, 0.20433E+00_JPRB ,&
&  0.20337E+00_JPRB, 0.20191E+00_JPRB, 0.20052E+00_JPRB, 0.20083E+00_JPRB, 0.20162E+00_JPRB, 0.20199E+00_JPRB, 0.19981E+00_JPRB ,&
&  0.19871E+00_JPRB, 0.19590E+00_JPRB, 0.19330E+00_JPRB, 0.19075E+00_JPRB, 0.18956E+00_JPRB, 0.18601E+00_JPRB, 0.18218E+00_JPRB ,&
&  0.18138E+00_JPRB, 0.18545E+00_JPRB, 0.19161E+00_JPRB, 0.19347E+00_JPRB, 0.18195E+00_JPRB, 0.18292E+00_JPRB, 0.18331E+00_JPRB ,&
&  0.17874E+00_JPRB, 0.17344E+00_JPRB, 0.16963E+00_JPRB, 0.16801E+00_JPRB, 0.16743E+00_JPRB, 0.16695E+00_JPRB, 0.16754E+00_JPRB ,&
&  0.16861E+00_JPRB, 0.16921E+00_JPRB, 0.16986E+00_JPRB, 0.17114E+00_JPRB, 0.17238E+00_JPRB, 0.17365E+00_JPRB, 0.17477E+00_JPRB ,&
&  0.17721E+00_JPRB, 0.18013E+00_JPRB, 0.18240E+00_JPRB, 0.18538E+00_JPRB, 0.18856E+00_JPRB, 0.19171E+00_JPRB, 0.19474E+00_JPRB ,&
&  0.19806E+00_JPRB, 0.19985E+00_JPRB, 0.20151E+00_JPRB, 0.20443E+00_JPRB, 0.20586E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.20079E+00_JPRB, 0.20355E+00_JPRB, 0.20545E+00_JPRB, 0.20727E+00_JPRB, 0.20868E+00_JPRB, 0.21022E+00_JPRB, 0.21101E+00_JPRB ,&
&  0.21176E+00_JPRB, 0.21184E+00_JPRB, 0.21236E+00_JPRB, 0.21238E+00_JPRB, 0.21262E+00_JPRB, 0.21150E+00_JPRB, 0.21097E+00_JPRB ,&
&  0.21051E+00_JPRB, 0.21085E+00_JPRB, 0.21079E+00_JPRB, 0.21058E+00_JPRB, 0.20973E+00_JPRB, 0.20865E+00_JPRB, 0.20705E+00_JPRB ,&
&  0.20387E+00_JPRB, 0.20167E+00_JPRB, 0.20195E+00_JPRB, 0.20142E+00_JPRB, 0.20180E+00_JPRB, 0.20175E+00_JPRB, 0.20218E+00_JPRB ,&
&  0.19770E+00_JPRB, 0.19620E+00_JPRB, 0.19672E+00_JPRB, 0.19561E+00_JPRB, 0.19739E+00_JPRB, 0.19820E+00_JPRB, 0.20007E+00_JPRB ,&
&  0.20056E+00_JPRB, 0.20090E+00_JPRB, 0.20173E+00_JPRB, 0.20128E+00_JPRB, 0.20161E+00_JPRB, 0.20213E+00_JPRB, 0.20098E+00_JPRB ,&
&  0.19782E+00_JPRB, 0.19539E+00_JPRB, 0.19195E+00_JPRB, 0.18373E+00_JPRB, 0.18201E+00_JPRB, 0.18252E+00_JPRB, 0.18121E+00_JPRB ,&
&  0.17845E+00_JPRB, 0.17903E+00_JPRB, 0.17861E+00_JPRB, 0.17695E+00_JPRB, 0.17886E+00_JPRB, 0.18167E+00_JPRB, 0.18180E+00_JPRB ,&
&  0.18047E+00_JPRB, 0.18012E+00_JPRB, 0.17945E+00_JPRB, 0.17814E+00_JPRB, 0.17655E+00_JPRB, 0.17588E+00_JPRB, 0.17528E+00_JPRB ,&
&  0.17496E+00_JPRB, 0.17348E+00_JPRB, 0.17678E+00_JPRB, 0.17737E+00_JPRB, 0.17218E+00_JPRB, 0.17451E+00_JPRB, 0.17228E+00_JPRB ,&
&  0.16524E+00_JPRB, 0.16209E+00_JPRB, 0.16034E+00_JPRB, 0.15886E+00_JPRB, 0.15860E+00_JPRB, 0.15871E+00_JPRB, 0.15856E+00_JPRB ,&
&  0.15952E+00_JPRB, 0.16071E+00_JPRB, 0.16129E+00_JPRB, 0.16202E+00_JPRB, 0.16304E+00_JPRB, 0.16452E+00_JPRB, 0.16646E+00_JPRB ,&
&  0.16815E+00_JPRB, 0.17048E+00_JPRB, 0.17165E+00_JPRB, 0.17357E+00_JPRB, 0.17500E+00_JPRB, 0.17669E+00_JPRB, 0.18071E+00_JPRB ,&
&  0.18394E+00_JPRB, 0.19022E+00_JPRB, 0.19867E+00_JPRB, 0.20102E+00_JPRB, 0.19814E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.19535E+00_JPRB, 0.19742E+00_JPRB, 0.19946E+00_JPRB, 0.20109E+00_JPRB, 0.20275E+00_JPRB, 0.20416E+00_JPRB, 0.20490E+00_JPRB ,&
&  0.20511E+00_JPRB, 0.20452E+00_JPRB, 0.20463E+00_JPRB, 0.20327E+00_JPRB, 0.20200E+00_JPRB, 0.19778E+00_JPRB, 0.19736E+00_JPRB ,&
&  0.19638E+00_JPRB, 0.19706E+00_JPRB, 0.19657E+00_JPRB, 0.19679E+00_JPRB, 0.19597E+00_JPRB, 0.19552E+00_JPRB, 0.19380E+00_JPRB ,&
&  0.19132E+00_JPRB, 0.18951E+00_JPRB, 0.18775E+00_JPRB, 0.18728E+00_JPRB, 0.18811E+00_JPRB, 0.18869E+00_JPRB, 0.18833E+00_JPRB ,&
&  0.18837E+00_JPRB, 0.18862E+00_JPRB, 0.18900E+00_JPRB, 0.18892E+00_JPRB, 0.19413E+00_JPRB, 0.19634E+00_JPRB, 0.19752E+00_JPRB ,&
&  0.19737E+00_JPRB, 0.19770E+00_JPRB, 0.19782E+00_JPRB, 0.19688E+00_JPRB, 0.19593E+00_JPRB, 0.19153E+00_JPRB, 0.18872E+00_JPRB ,&
&  0.18087E+00_JPRB, 0.17520E+00_JPRB, 0.17093E+00_JPRB, 0.16831E+00_JPRB, 0.16724E+00_JPRB, 0.16724E+00_JPRB, 0.16678E+00_JPRB ,&
&  0.16519E+00_JPRB, 0.16371E+00_JPRB, 0.16278E+00_JPRB, 0.16351E+00_JPRB, 0.16468E+00_JPRB, 0.16598E+00_JPRB, 0.16747E+00_JPRB ,&
&  0.16877E+00_JPRB, 0.16909E+00_JPRB, 0.16903E+00_JPRB, 0.16946E+00_JPRB, 0.16978E+00_JPRB, 0.17029E+00_JPRB, 0.17065E+00_JPRB ,&
&  0.17163E+00_JPRB, 0.17071E+00_JPRB, 0.17132E+00_JPRB, 0.17255E+00_JPRB, 0.17010E+00_JPRB, 0.16619E+00_JPRB, 0.16233E+00_JPRB ,&
&  0.15872E+00_JPRB, 0.15667E+00_JPRB, 0.15501E+00_JPRB, 0.15393E+00_JPRB, 0.15368E+00_JPRB, 0.15377E+00_JPRB, 0.15433E+00_JPRB ,&
&  0.15506E+00_JPRB, 0.15609E+00_JPRB, 0.15675E+00_JPRB, 0.15762E+00_JPRB, 0.15869E+00_JPRB, 0.15985E+00_JPRB, 0.16123E+00_JPRB ,&
&  0.16296E+00_JPRB, 0.16460E+00_JPRB, 0.16575E+00_JPRB, 0.16737E+00_JPRB, 0.17044E+00_JPRB, 0.17448E+00_JPRB, 0.18056E+00_JPRB ,&
&  0.18727E+00_JPRB, 0.19239E+00_JPRB, 0.18941E+00_JPRB, 0.19217E+00_JPRB, 0.19435E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.19513E+00_JPRB, 0.19680E+00_JPRB, 0.19832E+00_JPRB, 0.19987E+00_JPRB, 0.20117E+00_JPRB, 0.20242E+00_JPRB, 0.20324E+00_JPRB ,&
&  0.20067E+00_JPRB, 0.19923E+00_JPRB, 0.19655E+00_JPRB, 0.19423E+00_JPRB, 0.19148E+00_JPRB, 0.18837E+00_JPRB, 0.18738E+00_JPRB ,&
&  0.18577E+00_JPRB, 0.18545E+00_JPRB, 0.18406E+00_JPRB, 0.18345E+00_JPRB, 0.18228E+00_JPRB, 0.18157E+00_JPRB, 0.18038E+00_JPRB ,&
&  0.17810E+00_JPRB, 0.17697E+00_JPRB, 0.17589E+00_JPRB, 0.17622E+00_JPRB, 0.17773E+00_JPRB, 0.17935E+00_JPRB, 0.18107E+00_JPRB ,&
&  0.18355E+00_JPRB, 0.18459E+00_JPRB, 0.18697E+00_JPRB, 0.18876E+00_JPRB, 0.19189E+00_JPRB, 0.19451E+00_JPRB, 0.19509E+00_JPRB ,&
&  0.19511E+00_JPRB, 0.19313E+00_JPRB, 0.19247E+00_JPRB, 0.19015E+00_JPRB, 0.18398E+00_JPRB, 0.18060E+00_JPRB, 0.17573E+00_JPRB ,&
&  0.17031E+00_JPRB, 0.16626E+00_JPRB, 0.16345E+00_JPRB, 0.16259E+00_JPRB, 0.16166E+00_JPRB, 0.16138E+00_JPRB, 0.16121E+00_JPRB ,&
&  0.16042E+00_JPRB, 0.15950E+00_JPRB, 0.15839E+00_JPRB, 0.15777E+00_JPRB, 0.15817E+00_JPRB, 0.15897E+00_JPRB, 0.16069E+00_JPRB ,&
&  0.16242E+00_JPRB, 0.16339E+00_JPRB, 0.16445E+00_JPRB, 0.16480E+00_JPRB, 0.16492E+00_JPRB, 0.16517E+00_JPRB, 0.16521E+00_JPRB ,&
&  0.16590E+00_JPRB, 0.16604E+00_JPRB, 0.16550E+00_JPRB, 0.16460E+00_JPRB, 0.16229E+00_JPRB, 0.15925E+00_JPRB, 0.15688E+00_JPRB ,&
&  0.15486E+00_JPRB, 0.15355E+00_JPRB, 0.15311E+00_JPRB, 0.15263E+00_JPRB, 0.15289E+00_JPRB, 0.15328E+00_JPRB, 0.15363E+00_JPRB ,&
&  0.15425E+00_JPRB, 0.15479E+00_JPRB, 0.15532E+00_JPRB, 0.15655E+00_JPRB, 0.15756E+00_JPRB, 0.15906E+00_JPRB, 0.16063E+00_JPRB ,&
&  0.16172E+00_JPRB, 0.16338E+00_JPRB, 0.16479E+00_JPRB, 0.16635E+00_JPRB, 0.17104E+00_JPRB, 0.17471E+00_JPRB, 0.18217E+00_JPRB ,&
&  0.18685E+00_JPRB, 0.18802E+00_JPRB, 0.18812E+00_JPRB, 0.19388E+00_JPRB, 0.19441E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.19568E+00_JPRB, 0.19626E+00_JPRB, 0.19625E+00_JPRB, 0.19723E+00_JPRB, 0.19800E+00_JPRB, 0.19885E+00_JPRB, 0.19812E+00_JPRB ,&
&  0.19747E+00_JPRB, 0.19605E+00_JPRB, 0.19493E+00_JPRB, 0.19188E+00_JPRB, 0.18947E+00_JPRB, 0.18652E+00_JPRB, 0.18396E+00_JPRB ,&
&  0.18167E+00_JPRB, 0.17949E+00_JPRB, 0.17743E+00_JPRB, 0.17569E+00_JPRB, 0.17405E+00_JPRB, 0.17261E+00_JPRB, 0.17121E+00_JPRB ,&
&  0.16908E+00_JPRB, 0.16849E+00_JPRB, 0.16848E+00_JPRB, 0.16977E+00_JPRB, 0.17244E+00_JPRB, 0.17412E+00_JPRB, 0.17618E+00_JPRB ,&
&  0.17839E+00_JPRB, 0.17760E+00_JPRB, 0.18074E+00_JPRB, 0.18374E+00_JPRB, 0.18618E+00_JPRB, 0.18821E+00_JPRB, 0.18956E+00_JPRB ,&
&  0.19019E+00_JPRB, 0.18493E+00_JPRB, 0.18603E+00_JPRB, 0.18153E+00_JPRB, 0.17674E+00_JPRB, 0.17471E+00_JPRB, 0.17071E+00_JPRB ,&
&  0.16905E+00_JPRB, 0.16593E+00_JPRB, 0.16392E+00_JPRB, 0.16218E+00_JPRB, 0.16006E+00_JPRB, 0.15955E+00_JPRB, 0.15870E+00_JPRB ,&
&  0.15823E+00_JPRB, 0.15835E+00_JPRB, 0.15759E+00_JPRB, 0.15737E+00_JPRB, 0.15690E+00_JPRB, 0.15646E+00_JPRB, 0.15687E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15831E+00_JPRB, 0.15908E+00_JPRB, 0.15920E+00_JPRB, 0.15914E+00_JPRB, 0.15912E+00_JPRB, 0.15902E+00_JPRB ,&
&  0.15930E+00_JPRB, 0.15931E+00_JPRB, 0.15888E+00_JPRB, 0.15840E+00_JPRB, 0.15727E+00_JPRB, 0.15615E+00_JPRB, 0.15505E+00_JPRB ,&
&  0.15409E+00_JPRB, 0.15356E+00_JPRB, 0.15311E+00_JPRB, 0.15298E+00_JPRB, 0.15301E+00_JPRB, 0.15334E+00_JPRB, 0.15369E+00_JPRB ,&
&  0.15401E+00_JPRB, 0.15477E+00_JPRB, 0.15534E+00_JPRB, 0.15627E+00_JPRB, 0.15739E+00_JPRB, 0.15839E+00_JPRB, 0.15992E+00_JPRB ,&
&  0.16158E+00_JPRB, 0.16339E+00_JPRB, 0.16599E+00_JPRB, 0.16824E+00_JPRB, 0.17107E+00_JPRB, 0.17470E+00_JPRB, 0.17732E+00_JPRB ,&
&  0.18015E+00_JPRB, 0.18277E+00_JPRB, 0.18288E+00_JPRB, 0.19026E+00_JPRB, 0.19438E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.18771E+00_JPRB, 0.18856E+00_JPRB, 0.18899E+00_JPRB, 0.18941E+00_JPRB, 0.19049E+00_JPRB, 0.19048E+00_JPRB, 0.19027E+00_JPRB ,&
&  0.18814E+00_JPRB, 0.18734E+00_JPRB, 0.18603E+00_JPRB, 0.18417E+00_JPRB, 0.18227E+00_JPRB, 0.17906E+00_JPRB, 0.17708E+00_JPRB ,&
&  0.17516E+00_JPRB, 0.17314E+00_JPRB, 0.17183E+00_JPRB, 0.17011E+00_JPRB, 0.16862E+00_JPRB, 0.16693E+00_JPRB, 0.16638E+00_JPRB ,&
&  0.16552E+00_JPRB, 0.16545E+00_JPRB, 0.16627E+00_JPRB, 0.16690E+00_JPRB, 0.16810E+00_JPRB, 0.16936E+00_JPRB, 0.17101E+00_JPRB ,&
&  0.17137E+00_JPRB, 0.17212E+00_JPRB, 0.17286E+00_JPRB, 0.17335E+00_JPRB, 0.17514E+00_JPRB, 0.17641E+00_JPRB, 0.17687E+00_JPRB ,&
&  0.17639E+00_JPRB, 0.17549E+00_JPRB, 0.17373E+00_JPRB, 0.17171E+00_JPRB, 0.16941E+00_JPRB, 0.16691E+00_JPRB, 0.16453E+00_JPRB ,&
&  0.16269E+00_JPRB, 0.16098E+00_JPRB, 0.15969E+00_JPRB, 0.15863E+00_JPRB, 0.15799E+00_JPRB, 0.15748E+00_JPRB, 0.15696E+00_JPRB ,&
&  0.15667E+00_JPRB, 0.15644E+00_JPRB, 0.15608E+00_JPRB, 0.15610E+00_JPRB, 0.15601E+00_JPRB, 0.15608E+00_JPRB, 0.15615E+00_JPRB ,&
&  0.15628E+00_JPRB, 0.15628E+00_JPRB, 0.15619E+00_JPRB, 0.15590E+00_JPRB, 0.15560E+00_JPRB, 0.15527E+00_JPRB, 0.15493E+00_JPRB ,&
&  0.15455E+00_JPRB, 0.15451E+00_JPRB, 0.15420E+00_JPRB, 0.15398E+00_JPRB, 0.15368E+00_JPRB, 0.15319E+00_JPRB, 0.15303E+00_JPRB ,&
&  0.15310E+00_JPRB, 0.15322E+00_JPRB, 0.15306E+00_JPRB, 0.15319E+00_JPRB, 0.15339E+00_JPRB, 0.15369E+00_JPRB, 0.15453E+00_JPRB ,&
&  0.15505E+00_JPRB, 0.15559E+00_JPRB, 0.15609E+00_JPRB, 0.15680E+00_JPRB, 0.15802E+00_JPRB, 0.15924E+00_JPRB, 0.16061E+00_JPRB ,&
&  0.16225E+00_JPRB, 0.16379E+00_JPRB, 0.16533E+00_JPRB, 0.16685E+00_JPRB, 0.16937E+00_JPRB, 0.17112E+00_JPRB, 0.17342E+00_JPRB ,&
&  0.17540E+00_JPRB, 0.17412E+00_JPRB, 0.18238E+00_JPRB, 0.18315E+00_JPRB, 0.18444E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.17189E+00_JPRB, 0.17198E+00_JPRB, 0.17117E+00_JPRB, 0.17029E+00_JPRB, 0.16352E+00_JPRB, 0.16973E+00_JPRB, 0.16904E+00_JPRB ,&
&  0.16956E+00_JPRB, 0.16976E+00_JPRB, 0.17119E+00_JPRB, 0.17142E+00_JPRB, 0.17117E+00_JPRB, 0.17107E+00_JPRB, 0.17084E+00_JPRB ,&
&  0.17056E+00_JPRB, 0.16982E+00_JPRB, 0.16947E+00_JPRB, 0.16892E+00_JPRB, 0.16836E+00_JPRB, 0.16742E+00_JPRB, 0.16640E+00_JPRB ,&
&  0.16594E+00_JPRB, 0.16601E+00_JPRB, 0.16639E+00_JPRB, 0.16471E+00_JPRB, 0.16394E+00_JPRB, 0.16344E+00_JPRB, 0.16317E+00_JPRB ,&
&  0.16274E+00_JPRB, 0.16261E+00_JPRB, 0.16261E+00_JPRB, 0.16255E+00_JPRB, 0.16388E+00_JPRB, 0.16429E+00_JPRB, 0.16460E+00_JPRB ,&
&  0.16466E+00_JPRB, 0.16450E+00_JPRB, 0.16382E+00_JPRB, 0.16319E+00_JPRB, 0.16256E+00_JPRB, 0.16146E+00_JPRB, 0.16030E+00_JPRB ,&
&  0.15931E+00_JPRB, 0.15856E+00_JPRB, 0.15770E+00_JPRB, 0.15704E+00_JPRB, 0.15649E+00_JPRB, 0.15598E+00_JPRB, 0.15564E+00_JPRB ,&
&  0.15555E+00_JPRB, 0.15555E+00_JPRB, 0.15553E+00_JPRB, 0.15551E+00_JPRB, 0.15547E+00_JPRB, 0.15544E+00_JPRB, 0.15542E+00_JPRB ,&
&  0.15573E+00_JPRB, 0.15570E+00_JPRB, 0.15554E+00_JPRB, 0.15526E+00_JPRB, 0.15514E+00_JPRB, 0.15481E+00_JPRB, 0.15458E+00_JPRB ,&
&  0.15440E+00_JPRB, 0.15417E+00_JPRB, 0.15406E+00_JPRB, 0.15403E+00_JPRB, 0.15402E+00_JPRB, 0.15400E+00_JPRB, 0.15409E+00_JPRB ,&
&  0.15423E+00_JPRB, 0.15443E+00_JPRB, 0.15481E+00_JPRB, 0.15511E+00_JPRB, 0.15539E+00_JPRB, 0.15564E+00_JPRB, 0.15615E+00_JPRB ,&
&  0.15669E+00_JPRB, 0.15722E+00_JPRB, 0.15774E+00_JPRB, 0.15809E+00_JPRB, 0.15885E+00_JPRB, 0.15962E+00_JPRB, 0.16046E+00_JPRB ,&
&  0.16136E+00_JPRB, 0.16249E+00_JPRB, 0.16366E+00_JPRB, 0.16483E+00_JPRB, 0.16638E+00_JPRB, 0.16762E+00_JPRB, 0.16882E+00_JPRB ,&
&  0.17016E+00_JPRB, 0.17265E+00_JPRB, 0.17330E+00_JPRB, 0.17344E+00_JPRB, 0.17291E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.16702E+00_JPRB, 0.16699E+00_JPRB, 0.16686E+00_JPRB, 0.16648E+00_JPRB, 0.16575E+00_JPRB, 0.16558E+00_JPRB, 0.16534E+00_JPRB ,&
&  0.16502E+00_JPRB, 0.16433E+00_JPRB, 0.16431E+00_JPRB, 0.16422E+00_JPRB, 0.16411E+00_JPRB, 0.16398E+00_JPRB, 0.16373E+00_JPRB ,&
&  0.16322E+00_JPRB, 0.16245E+00_JPRB, 0.16013E+00_JPRB, 0.15902E+00_JPRB, 0.15831E+00_JPRB, 0.15793E+00_JPRB, 0.15761E+00_JPRB ,&
&  0.15742E+00_JPRB, 0.15732E+00_JPRB, 0.15723E+00_JPRB, 0.15767E+00_JPRB, 0.15787E+00_JPRB, 0.15795E+00_JPRB, 0.15800E+00_JPRB ,&
&  0.15800E+00_JPRB, 0.15809E+00_JPRB, 0.15810E+00_JPRB, 0.15810E+00_JPRB, 0.15822E+00_JPRB, 0.15823E+00_JPRB, 0.15804E+00_JPRB ,&
&  0.15782E+00_JPRB, 0.15756E+00_JPRB, 0.15731E+00_JPRB, 0.15700E+00_JPRB, 0.15677E+00_JPRB, 0.15659E+00_JPRB, 0.15652E+00_JPRB ,&
&  0.15636E+00_JPRB, 0.15624E+00_JPRB, 0.15612E+00_JPRB, 0.15606E+00_JPRB, 0.15596E+00_JPRB, 0.15591E+00_JPRB, 0.15579E+00_JPRB ,&
&  0.15588E+00_JPRB, 0.15591E+00_JPRB, 0.15597E+00_JPRB, 0.15600E+00_JPRB, 0.15602E+00_JPRB, 0.15598E+00_JPRB, 0.15590E+00_JPRB ,&
&  0.15573E+00_JPRB, 0.15563E+00_JPRB, 0.15550E+00_JPRB, 0.15541E+00_JPRB, 0.15539E+00_JPRB, 0.15537E+00_JPRB, 0.15537E+00_JPRB ,&
&  0.15538E+00_JPRB, 0.15548E+00_JPRB, 0.15551E+00_JPRB, 0.15557E+00_JPRB, 0.15564E+00_JPRB, 0.15572E+00_JPRB, 0.15577E+00_JPRB ,&
&  0.15586E+00_JPRB, 0.15595E+00_JPRB, 0.15605E+00_JPRB, 0.15620E+00_JPRB, 0.15637E+00_JPRB, 0.15661E+00_JPRB, 0.15690E+00_JPRB ,&
&  0.15721E+00_JPRB, 0.15752E+00_JPRB, 0.15783E+00_JPRB, 0.15814E+00_JPRB, 0.15862E+00_JPRB, 0.15910E+00_JPRB, 0.15967E+00_JPRB ,&
&  0.16023E+00_JPRB, 0.16080E+00_JPRB, 0.16131E+00_JPRB, 0.16187E+00_JPRB, 0.16256E+00_JPRB, 0.16292E+00_JPRB, 0.16322E+00_JPRB ,&
&  0.16370E+00_JPRB, 0.16419E+00_JPRB, 0.16468E+00_JPRB, 0.16511E+00_JPRB, 0.16580E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB, 0.15753E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB, 0.94364E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.95322E+00_JPRB, 0.94926E+00_JPRB, 0.94569E+00_JPRB, 0.94198E+00_JPRB, 0.93815E+00_JPRB, 0.93500E+00_JPRB, 0.93212E+00_JPRB ,&
&  0.92947E+00_JPRB, 0.92777E+00_JPRB, 0.92677E+00_JPRB, 0.92618E+00_JPRB, 0.92615E+00_JPRB, 0.92586E+00_JPRB, 0.92573E+00_JPRB ,&
&  0.92562E+00_JPRB, 0.92572E+00_JPRB, 0.92576E+00_JPRB, 0.92517E+00_JPRB, 0.92411E+00_JPRB, 0.92271E+00_JPRB, 0.92099E+00_JPRB ,&
&  0.91853E+00_JPRB, 0.91568E+00_JPRB, 0.91277E+00_JPRB, 0.90966E+00_JPRB, 0.90661E+00_JPRB, 0.90312E+00_JPRB, 0.89939E+00_JPRB ,&
&  0.89600E+00_JPRB, 0.89374E+00_JPRB, 0.89179E+00_JPRB, 0.88973E+00_JPRB, 0.88844E+00_JPRB, 0.88910E+00_JPRB, 0.88960E+00_JPRB ,&
&  0.89010E+00_JPRB, 0.89088E+00_JPRB, 0.89322E+00_JPRB, 0.89557E+00_JPRB, 0.89774E+00_JPRB, 0.90021E+00_JPRB, 0.90321E+00_JPRB ,&
&  0.90510E+00_JPRB, 0.90643E+00_JPRB, 0.90746E+00_JPRB, 0.90862E+00_JPRB, 0.90925E+00_JPRB, 0.91014E+00_JPRB, 0.91073E+00_JPRB ,&
&  0.91109E+00_JPRB, 0.91141E+00_JPRB, 0.91110E+00_JPRB, 0.90959E+00_JPRB, 0.90531E+00_JPRB, 0.90209E+00_JPRB, 0.90056E+00_JPRB ,&
&  0.90070E+00_JPRB, 0.90142E+00_JPRB, 0.90271E+00_JPRB, 0.90477E+00_JPRB, 0.90812E+00_JPRB, 0.91207E+00_JPRB, 0.91667E+00_JPRB ,&
&  0.92168E+00_JPRB, 0.92774E+00_JPRB, 0.93520E+00_JPRB, 0.94291E+00_JPRB, 0.95132E+00_JPRB, 0.96110E+00_JPRB, 0.97017E+00_JPRB ,&
&  0.97958E+00_JPRB, 0.98941E+00_JPRB, 0.99769E+00_JPRB, 0.10054E+01_JPRB, 0.10127E+01_JPRB, 0.10198E+01_JPRB, 0.10251E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.10290E+01_JPRB, 0.10301E+01_JPRB, 0.10289E+01_JPRB, 0.10261E+01_JPRB, 0.10238E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.10144E+01_JPRB, 0.10106E+01_JPRB, 0.10065E+01_JPRB, 0.10007E+01_JPRB, 0.99438E+00_JPRB, 0.98840E+00_JPRB ,&
&  0.98246E+00_JPRB, 0.97592E+00_JPRB, 0.96983E+00_JPRB, 0.96400E+00_JPRB, 0.95857E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.99751E+00_JPRB, 0.98365E+00_JPRB, 0.97096E+00_JPRB, 0.95925E+00_JPRB, 0.94949E+00_JPRB, 0.94163E+00_JPRB, 0.93463E+00_JPRB ,&
&  0.92833E+00_JPRB, 0.92455E+00_JPRB, 0.92125E+00_JPRB, 0.91817E+00_JPRB, 0.91526E+00_JPRB, 0.91221E+00_JPRB, 0.90879E+00_JPRB ,&
&  0.90549E+00_JPRB, 0.90315E+00_JPRB, 0.90108E+00_JPRB, 0.89828E+00_JPRB, 0.89295E+00_JPRB, 0.88503E+00_JPRB, 0.87569E+00_JPRB ,&
&  0.86530E+00_JPRB, 0.85278E+00_JPRB, 0.83831E+00_JPRB, 0.82276E+00_JPRB, 0.81418E+00_JPRB, 0.80439E+00_JPRB, 0.79500E+00_JPRB ,&
&  0.78673E+00_JPRB, 0.78362E+00_JPRB, 0.78086E+00_JPRB, 0.77876E+00_JPRB, 0.77819E+00_JPRB, 0.78007E+00_JPRB, 0.78090E+00_JPRB ,&
&  0.78131E+00_JPRB, 0.78337E+00_JPRB, 0.79114E+00_JPRB, 0.79977E+00_JPRB, 0.80999E+00_JPRB, 0.82278E+00_JPRB, 0.83745E+00_JPRB ,&
&  0.84886E+00_JPRB, 0.85679E+00_JPRB, 0.86607E+00_JPRB, 0.87497E+00_JPRB, 0.88189E+00_JPRB, 0.88759E+00_JPRB, 0.89250E+00_JPRB ,&
&  0.89866E+00_JPRB, 0.90298E+00_JPRB, 0.90671E+00_JPRB, 0.90877E+00_JPRB, 0.90771E+00_JPRB, 0.90591E+00_JPRB, 0.90271E+00_JPRB ,&
&  0.89836E+00_JPRB, 0.89797E+00_JPRB, 0.90106E+00_JPRB, 0.90519E+00_JPRB, 0.91300E+00_JPRB, 0.92391E+00_JPRB, 0.93607E+00_JPRB ,&
&  0.95180E+00_JPRB, 0.97258E+00_JPRB, 0.99774E+00_JPRB, 0.10255E+01_JPRB, 0.10574E+01_JPRB, 0.10970E+01_JPRB, 0.11293E+01_JPRB ,&
&  0.11585E+01_JPRB, 0.11832E+01_JPRB, 0.12048E+01_JPRB, 0.12196E+01_JPRB, 0.12294E+01_JPRB, 0.12381E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.12292E+01_JPRB, 0.12257E+01_JPRB, 0.12180E+01_JPRB, 0.12086E+01_JPRB, 0.11989E+01_JPRB, 0.11894E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.11616E+01_JPRB, 0.11478E+01_JPRB, 0.11345E+01_JPRB, 0.11192E+01_JPRB, 0.11045E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.10752E+01_JPRB, 0.10591E+01_JPRB, 0.10437E+01_JPRB, 0.10285E+01_JPRB, 0.10134E+01_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.10506E+01_JPRB, 0.10280E+01_JPRB, 0.10073E+01_JPRB, 0.98896E+00_JPRB, 0.97262E+00_JPRB, 0.95913E+00_JPRB, 0.94748E+00_JPRB ,&
&  0.93720E+00_JPRB, 0.93038E+00_JPRB, 0.92364E+00_JPRB, 0.91656E+00_JPRB, 0.90951E+00_JPRB, 0.90684E+00_JPRB, 0.90201E+00_JPRB ,&
&  0.89550E+00_JPRB, 0.88742E+00_JPRB, 0.87995E+00_JPRB, 0.86978E+00_JPRB, 0.85447E+00_JPRB, 0.83594E+00_JPRB, 0.81293E+00_JPRB ,&
&  0.79812E+00_JPRB, 0.77958E+00_JPRB, 0.76014E+00_JPRB, 0.74024E+00_JPRB, 0.72619E+00_JPRB, 0.69684E+00_JPRB, 0.68596E+00_JPRB ,&
&  0.67882E+00_JPRB, 0.66037E+00_JPRB, 0.65003E+00_JPRB, 0.64538E+00_JPRB, 0.66223E+00_JPRB, 0.68437E+00_JPRB, 0.69654E+00_JPRB ,&
&  0.68277E+00_JPRB, 0.69353E+00_JPRB, 0.69049E+00_JPRB, 0.68766E+00_JPRB, 0.68971E+00_JPRB, 0.69578E+00_JPRB, 0.72407E+00_JPRB ,&
&  0.77483E+00_JPRB, 0.79618E+00_JPRB, 0.81067E+00_JPRB, 0.82608E+00_JPRB, 0.83807E+00_JPRB, 0.84960E+00_JPRB, 0.86001E+00_JPRB ,&
&  0.87176E+00_JPRB, 0.88263E+00_JPRB, 0.89409E+00_JPRB, 0.89920E+00_JPRB, 0.90642E+00_JPRB, 0.91341E+00_JPRB, 0.91902E+00_JPRB ,&
&  0.91357E+00_JPRB, 0.91185E+00_JPRB, 0.91570E+00_JPRB, 0.92537E+00_JPRB, 0.94060E+00_JPRB, 0.96507E+00_JPRB, 0.99151E+00_JPRB ,&
&  0.10197E+01_JPRB, 0.10426E+01_JPRB, 0.10787E+01_JPRB, 0.11273E+01_JPRB, 0.11859E+01_JPRB, 0.12580E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.14599E+01_JPRB, 0.15144E+01_JPRB, 0.15399E+01_JPRB, 0.15519E+01_JPRB, 0.15595E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.15544E+01_JPRB, 0.15439E+01_JPRB, 0.15331E+01_JPRB, 0.15177E+01_JPRB, 0.14967E+01_JPRB, 0.14693E+01_JPRB, 0.14384E+01_JPRB ,&
&  0.14019E+01_JPRB, 0.13665E+01_JPRB, 0.13318E+01_JPRB, 0.12980E+01_JPRB, 0.12641E+01_JPRB, 0.12330E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.11788E+01_JPRB, 0.11515E+01_JPRB, 0.11261E+01_JPRB, 0.11013E+01_JPRB, 0.10763E+01_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.10982E+01_JPRB, 0.10736E+01_JPRB, 0.10476E+01_JPRB, 0.10238E+01_JPRB, 0.10037E+01_JPRB, 0.98806E+00_JPRB, 0.97602E+00_JPRB ,&
&  0.96486E+00_JPRB, 0.95178E+00_JPRB, 0.93792E+00_JPRB, 0.92318E+00_JPRB, 0.90959E+00_JPRB, 0.90117E+00_JPRB, 0.89358E+00_JPRB ,&
&  0.88740E+00_JPRB, 0.87593E+00_JPRB, 0.86189E+00_JPRB, 0.84330E+00_JPRB, 0.82298E+00_JPRB, 0.80464E+00_JPRB, 0.78101E+00_JPRB ,&
&  0.76325E+00_JPRB, 0.74456E+00_JPRB, 0.71764E+00_JPRB, 0.70236E+00_JPRB, 0.66859E+00_JPRB, 0.64566E+00_JPRB, 0.63108E+00_JPRB ,&
&  0.62037E+00_JPRB, 0.62174E+00_JPRB, 0.62717E+00_JPRB, 0.63707E+00_JPRB, 0.64231E+00_JPRB, 0.63920E+00_JPRB, 0.64031E+00_JPRB ,&
&  0.63254E+00_JPRB, 0.62726E+00_JPRB, 0.62311E+00_JPRB, 0.62229E+00_JPRB, 0.62561E+00_JPRB, 0.63447E+00_JPRB, 0.65317E+00_JPRB ,&
&  0.69237E+00_JPRB, 0.74483E+00_JPRB, 0.79235E+00_JPRB, 0.81989E+00_JPRB, 0.82681E+00_JPRB, 0.83874E+00_JPRB, 0.84360E+00_JPRB ,&
&  0.85251E+00_JPRB, 0.85941E+00_JPRB, 0.87230E+00_JPRB, 0.88617E+00_JPRB, 0.90059E+00_JPRB, 0.91305E+00_JPRB, 0.92723E+00_JPRB ,&
&  0.93192E+00_JPRB, 0.93668E+00_JPRB, 0.93854E+00_JPRB, 0.95454E+00_JPRB, 0.98406E+00_JPRB, 0.10140E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.10857E+01_JPRB, 0.11450E+01_JPRB, 0.12338E+01_JPRB, 0.13302E+01_JPRB, 0.14197E+01_JPRB, 0.15057E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.16846E+01_JPRB, 0.17760E+01_JPRB, 0.18115E+01_JPRB, 0.18092E+01_JPRB, 0.18163E+01_JPRB, 0.18000E+01_JPRB ,&
&  0.17885E+01_JPRB, 0.17700E+01_JPRB, 0.17595E+01_JPRB, 0.17488E+01_JPRB, 0.17194E+01_JPRB, 0.16750E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.15770E+01_JPRB, 0.15221E+01_JPRB, 0.14692E+01_JPRB, 0.14157E+01_JPRB, 0.13640E+01_JPRB, 0.13137E+01_JPRB, 0.12712E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.11978E+01_JPRB, 0.11680E+01_JPRB, 0.11438E+01_JPRB, 0.11212E+01_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.11463E+01_JPRB, 0.11297E+01_JPRB, 0.11129E+01_JPRB, 0.10905E+01_JPRB, 0.10676E+01_JPRB, 0.10464E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.10147E+01_JPRB, 0.99855E+00_JPRB, 0.98245E+00_JPRB, 0.96313E+00_JPRB, 0.94454E+00_JPRB, 0.92668E+00_JPRB, 0.91086E+00_JPRB ,&
&  0.89578E+00_JPRB, 0.87586E+00_JPRB, 0.85661E+00_JPRB, 0.83531E+00_JPRB, 0.81615E+00_JPRB, 0.79628E+00_JPRB, 0.77284E+00_JPRB ,&
&  0.75498E+00_JPRB, 0.72843E+00_JPRB, 0.70461E+00_JPRB, 0.69196E+00_JPRB, 0.66186E+00_JPRB, 0.64568E+00_JPRB, 0.63576E+00_JPRB ,&
&  0.65147E+00_JPRB, 0.64600E+00_JPRB, 0.66365E+00_JPRB, 0.64342E+00_JPRB, 0.63027E+00_JPRB, 0.61829E+00_JPRB, 0.60510E+00_JPRB ,&
&  0.59539E+00_JPRB, 0.58408E+00_JPRB, 0.58504E+00_JPRB, 0.59114E+00_JPRB, 0.59532E+00_JPRB, 0.60971E+00_JPRB, 0.62782E+00_JPRB ,&
&  0.66793E+00_JPRB, 0.77949E+00_JPRB, 0.80514E+00_JPRB, 0.83183E+00_JPRB, 0.83981E+00_JPRB, 0.84944E+00_JPRB, 0.85689E+00_JPRB ,&
&  0.86558E+00_JPRB, 0.87175E+00_JPRB, 0.88006E+00_JPRB, 0.88871E+00_JPRB, 0.91368E+00_JPRB, 0.93153E+00_JPRB, 0.95204E+00_JPRB ,&
&  0.95939E+00_JPRB, 0.96866E+00_JPRB, 0.97686E+00_JPRB, 0.99828E+00_JPRB, 0.10387E+01_JPRB, 0.10586E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.11057E+01_JPRB, 0.11587E+01_JPRB, 0.12398E+01_JPRB, 0.13373E+01_JPRB, 0.14110E+01_JPRB, 0.14731E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.16327E+01_JPRB, 0.17412E+01_JPRB, 0.18548E+01_JPRB, 0.19067E+01_JPRB, 0.19136E+01_JPRB, 0.18964E+01_JPRB, 0.18746E+01_JPRB ,&
&  0.18673E+01_JPRB, 0.18348E+01_JPRB, 0.18144E+01_JPRB, 0.18030E+01_JPRB, 0.17669E+01_JPRB, 0.17244E+01_JPRB, 0.16801E+01_JPRB ,&
&  0.16237E+01_JPRB, 0.15731E+01_JPRB, 0.15302E+01_JPRB, 0.14804E+01_JPRB, 0.14272E+01_JPRB, 0.13755E+01_JPRB, 0.13250E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.12390E+01_JPRB, 0.12030E+01_JPRB, 0.11798E+01_JPRB, 0.11589E+01_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.11833E+01_JPRB, 0.11663E+01_JPRB, 0.11502E+01_JPRB, 0.11350E+01_JPRB, 0.11179E+01_JPRB, 0.11006E+01_JPRB, 0.10828E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10523E+01_JPRB, 0.10360E+01_JPRB, 0.10158E+01_JPRB, 0.99432E+00_JPRB, 0.97006E+00_JPRB, 0.94861E+00_JPRB ,&
&  0.92518E+00_JPRB, 0.90040E+00_JPRB, 0.86767E+00_JPRB, 0.83758E+00_JPRB, 0.81578E+00_JPRB, 0.79676E+00_JPRB, 0.77494E+00_JPRB ,&
&  0.75660E+00_JPRB, 0.73225E+00_JPRB, 0.70339E+00_JPRB, 0.68102E+00_JPRB, 0.65235E+00_JPRB, 0.64620E+00_JPRB, 0.68320E+00_JPRB ,&
&  0.69963E+00_JPRB, 0.69849E+00_JPRB, 0.69043E+00_JPRB, 0.67675E+00_JPRB, 0.66431E+00_JPRB, 0.61288E+00_JPRB, 0.62310E+00_JPRB ,&
&  0.60014E+00_JPRB, 0.58063E+00_JPRB, 0.57948E+00_JPRB, 0.58189E+00_JPRB, 0.59084E+00_JPRB, 0.61190E+00_JPRB, 0.63341E+00_JPRB ,&
&  0.67327E+00_JPRB, 0.80257E+00_JPRB, 0.82979E+00_JPRB, 0.85543E+00_JPRB, 0.86505E+00_JPRB, 0.87412E+00_JPRB, 0.88246E+00_JPRB ,&
&  0.88948E+00_JPRB, 0.90108E+00_JPRB, 0.91300E+00_JPRB, 0.93057E+00_JPRB, 0.95086E+00_JPRB, 0.96452E+00_JPRB, 0.99261E+00_JPRB ,&
&  0.10127E+01_JPRB, 0.10304E+01_JPRB, 0.10460E+01_JPRB, 0.10549E+01_JPRB, 0.10769E+01_JPRB, 0.10733E+01_JPRB, 0.10786E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.12585E+01_JPRB, 0.13514E+01_JPRB, 0.14372E+01_JPRB, 0.15111E+01_JPRB, 0.15929E+01_JPRB, 0.16936E+01_JPRB ,&
&  0.17667E+01_JPRB, 0.18735E+01_JPRB, 0.19943E+01_JPRB, 0.20348E+01_JPRB, 0.20324E+01_JPRB, 0.20228E+01_JPRB, 0.20253E+01_JPRB ,&
&  0.20276E+01_JPRB, 0.20365E+01_JPRB, 0.19737E+01_JPRB, 0.19341E+01_JPRB, 0.18505E+01_JPRB, 0.17465E+01_JPRB, 0.16624E+01_JPRB ,&
&  0.15612E+01_JPRB, 0.14712E+01_JPRB, 0.14149E+01_JPRB, 0.13735E+01_JPRB, 0.13434E+01_JPRB, 0.13240E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.12910E+01_JPRB, 0.12698E+01_JPRB, 0.12464E+01_JPRB, 0.12226E+01_JPRB, 0.12011E+01_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.11831E+01_JPRB, 0.11690E+01_JPRB, 0.11510E+01_JPRB, 0.11361E+01_JPRB, 0.11181E+01_JPRB, 0.11052E+01_JPRB, 0.10924E+01_JPRB ,&
&  0.10849E+01_JPRB, 0.10812E+01_JPRB, 0.10755E+01_JPRB, 0.10700E+01_JPRB, 0.10589E+01_JPRB, 0.10441E+01_JPRB, 0.10239E+01_JPRB ,&
&  0.99511E+00_JPRB, 0.93206E+00_JPRB, 0.88480E+00_JPRB, 0.85967E+00_JPRB, 0.83521E+00_JPRB, 0.81565E+00_JPRB, 0.79617E+00_JPRB ,&
&  0.76990E+00_JPRB, 0.74755E+00_JPRB, 0.72136E+00_JPRB, 0.68829E+00_JPRB, 0.67118E+00_JPRB, 0.68087E+00_JPRB, 0.68701E+00_JPRB ,&
&  0.69570E+00_JPRB, 0.71578E+00_JPRB, 0.71234E+00_JPRB, 0.69651E+00_JPRB, 0.69190E+00_JPRB, 0.69314E+00_JPRB, 0.64908E+00_JPRB ,&
&  0.61216E+00_JPRB, 0.58319E+00_JPRB, 0.57914E+00_JPRB, 0.57935E+00_JPRB, 0.58943E+00_JPRB, 0.61176E+00_JPRB, 0.66575E+00_JPRB ,&
&  0.75392E+00_JPRB, 0.86561E+00_JPRB, 0.87142E+00_JPRB, 0.87877E+00_JPRB, 0.89157E+00_JPRB, 0.90425E+00_JPRB, 0.91816E+00_JPRB ,&
&  0.93472E+00_JPRB, 0.95927E+00_JPRB, 0.98973E+00_JPRB, 0.10218E+01_JPRB, 0.10558E+01_JPRB, 0.10748E+01_JPRB, 0.10927E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.11200E+01_JPRB, 0.11240E+01_JPRB, 0.11190E+01_JPRB, 0.11142E+01_JPRB, 0.10939E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.13339E+01_JPRB, 0.14337E+01_JPRB, 0.15110E+01_JPRB, 0.15958E+01_JPRB, 0.17121E+01_JPRB, 0.18410E+01_JPRB ,&
&  0.19582E+01_JPRB, 0.21122E+01_JPRB, 0.23204E+01_JPRB, 0.22999E+01_JPRB, 0.22594E+01_JPRB, 0.22345E+01_JPRB, 0.22389E+01_JPRB ,&
&  0.22332E+01_JPRB, 0.22527E+01_JPRB, 0.21339E+01_JPRB, 0.19607E+01_JPRB, 0.18063E+01_JPRB, 0.16371E+01_JPRB, 0.15396E+01_JPRB ,&
&  0.14692E+01_JPRB, 0.13636E+01_JPRB, 0.13092E+01_JPRB, 0.12775E+01_JPRB, 0.12576E+01_JPRB, 0.12366E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.12097E+01_JPRB, 0.12094E+01_JPRB, 0.12026E+01_JPRB, 0.11943E+01_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.11386E+01_JPRB, 0.11387E+01_JPRB, 0.11310E+01_JPRB, 0.11154E+01_JPRB, 0.10806E+01_JPRB, 0.10359E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.98727E+00_JPRB, 0.98602E+00_JPRB, 0.98136E+00_JPRB, 0.98067E+00_JPRB, 0.98697E+00_JPRB, 0.99896E+00_JPRB, 0.10091E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.99218E+00_JPRB, 0.95892E+00_JPRB, 0.90873E+00_JPRB, 0.86805E+00_JPRB, 0.83868E+00_JPRB, 0.82097E+00_JPRB ,&
&  0.80724E+00_JPRB, 0.78808E+00_JPRB, 0.76459E+00_JPRB, 0.72755E+00_JPRB, 0.70190E+00_JPRB, 0.68319E+00_JPRB, 0.69063E+00_JPRB ,&
&  0.71391E+00_JPRB, 0.72282E+00_JPRB, 0.71618E+00_JPRB, 0.73160E+00_JPRB, 0.73067E+00_JPRB, 0.73128E+00_JPRB, 0.67470E+00_JPRB ,&
&  0.63264E+00_JPRB, 0.61258E+00_JPRB, 0.59827E+00_JPRB, 0.59531E+00_JPRB, 0.61154E+00_JPRB, 0.64328E+00_JPRB, 0.70735E+00_JPRB ,&
&  0.83387E+00_JPRB, 0.87766E+00_JPRB, 0.89424E+00_JPRB, 0.91542E+00_JPRB, 0.93323E+00_JPRB, 0.94372E+00_JPRB, 0.96703E+00_JPRB ,&
&  0.10161E+01_JPRB, 0.10900E+01_JPRB, 0.11604E+01_JPRB, 0.11783E+01_JPRB, 0.11841E+01_JPRB, 0.11946E+01_JPRB, 0.13451E+01_JPRB ,&
&  0.17557E+01_JPRB, 0.15739E+01_JPRB, 0.13603E+01_JPRB, 0.12359E+01_JPRB, 0.11785E+01_JPRB, 0.11585E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.13167E+01_JPRB, 0.14123E+01_JPRB, 0.15369E+01_JPRB, 0.16339E+01_JPRB, 0.17420E+01_JPRB, 0.18865E+01_JPRB, 0.20891E+01_JPRB ,&
&  0.22975E+01_JPRB, 0.25533E+01_JPRB, 0.30426E+01_JPRB, 0.27212E+01_JPRB, 0.25320E+01_JPRB, 0.24618E+01_JPRB, 0.24293E+01_JPRB ,&
&  0.23344E+01_JPRB, 0.23195E+01_JPRB, 0.21326E+01_JPRB, 0.18897E+01_JPRB, 0.17652E+01_JPRB, 0.16307E+01_JPRB, 0.15311E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.13463E+01_JPRB, 0.12941E+01_JPRB, 0.12545E+01_JPRB, 0.12397E+01_JPRB, 0.12237E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.11841E+01_JPRB, 0.11647E+01_JPRB, 0.11508E+01_JPRB, 0.11400E+01_JPRB, 0.11356E+01_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.10878E+01_JPRB, 0.11065E+01_JPRB, 0.11163E+01_JPRB, 0.10842E+01_JPRB, 0.10345E+01_JPRB, 0.95878E+00_JPRB, 0.89916E+00_JPRB ,&
&  0.85437E+00_JPRB, 0.84014E+00_JPRB, 0.83497E+00_JPRB, 0.83957E+00_JPRB, 0.85390E+00_JPRB, 0.87659E+00_JPRB, 0.89381E+00_JPRB ,&
&  0.91106E+00_JPRB, 0.91444E+00_JPRB, 0.91447E+00_JPRB, 0.89916E+00_JPRB, 0.87047E+00_JPRB, 0.83748E+00_JPRB, 0.81422E+00_JPRB ,&
&  0.79999E+00_JPRB, 0.79038E+00_JPRB, 0.77676E+00_JPRB, 0.75511E+00_JPRB, 0.73886E+00_JPRB, 0.71762E+00_JPRB, 0.71807E+00_JPRB ,&
&  0.70748E+00_JPRB, 0.71443E+00_JPRB, 0.73402E+00_JPRB, 0.74120E+00_JPRB, 0.75725E+00_JPRB, 0.76201E+00_JPRB, 0.71691E+00_JPRB ,&
&  0.66947E+00_JPRB, 0.64587E+00_JPRB, 0.63824E+00_JPRB, 0.64122E+00_JPRB, 0.69406E+00_JPRB, 0.81353E+00_JPRB, 0.87754E+00_JPRB ,&
&  0.91334E+00_JPRB, 0.93392E+00_JPRB, 0.95809E+00_JPRB, 0.98630E+00_JPRB, 0.10006E+01_JPRB, 0.10203E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.13206E+01_JPRB, 0.14167E+01_JPRB, 0.12613E+01_JPRB, 0.11877E+01_JPRB, 0.12504E+01_JPRB, 0.13409E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.14013E+01_JPRB, 0.13482E+01_JPRB, 0.13856E+01_JPRB, 0.14026E+01_JPRB, 0.14220E+01_JPRB, 0.14411E+01_JPRB ,&
&  0.14728E+01_JPRB, 0.15688E+01_JPRB, 0.17705E+01_JPRB, 0.18796E+01_JPRB, 0.20503E+01_JPRB, 0.22957E+01_JPRB, 0.25116E+01_JPRB ,&
&  0.27659E+01_JPRB, 0.29497E+01_JPRB, 0.30285E+01_JPRB, 0.29018E+01_JPRB, 0.28059E+01_JPRB, 0.27928E+01_JPRB, 0.27215E+01_JPRB ,&
&  0.24864E+01_JPRB, 0.23132E+01_JPRB, 0.21679E+01_JPRB, 0.20308E+01_JPRB, 0.18447E+01_JPRB, 0.16888E+01_JPRB, 0.15300E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.12672E+01_JPRB, 0.12083E+01_JPRB, 0.11666E+01_JPRB, 0.11506E+01_JPRB, 0.11418E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10631E+01_JPRB, 0.10475E+01_JPRB, 0.10526E+01_JPRB, 0.10808E+01_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.11331E+01_JPRB, 0.11201E+01_JPRB, 0.10717E+01_JPRB, 0.10645E+01_JPRB, 0.10231E+01_JPRB, 0.97687E+00_JPRB, 0.91336E+00_JPRB ,&
&  0.86567E+00_JPRB, 0.81949E+00_JPRB, 0.78568E+00_JPRB, 0.74536E+00_JPRB, 0.72421E+00_JPRB, 0.71961E+00_JPRB, 0.74810E+00_JPRB ,&
&  0.78689E+00_JPRB, 0.81625E+00_JPRB, 0.83542E+00_JPRB, 0.84123E+00_JPRB, 0.83734E+00_JPRB, 0.82276E+00_JPRB, 0.80644E+00_JPRB ,&
&  0.79185E+00_JPRB, 0.78797E+00_JPRB, 0.77847E+00_JPRB, 0.76687E+00_JPRB, 0.75344E+00_JPRB, 0.73831E+00_JPRB, 0.72958E+00_JPRB ,&
&  0.71597E+00_JPRB, 0.71210E+00_JPRB, 0.72125E+00_JPRB, 0.74218E+00_JPRB, 0.76885E+00_JPRB, 0.78011E+00_JPRB, 0.74195E+00_JPRB ,&
&  0.70475E+00_JPRB, 0.69527E+00_JPRB, 0.70922E+00_JPRB, 0.81510E+00_JPRB, 0.86475E+00_JPRB, 0.89340E+00_JPRB, 0.93035E+00_JPRB ,&
&  0.92152E+00_JPRB, 0.92125E+00_JPRB, 0.94761E+00_JPRB, 0.10319E+01_JPRB, 0.10645E+01_JPRB, 0.10996E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.13063E+01_JPRB, 0.14806E+01_JPRB, 0.16123E+01_JPRB, 0.14735E+01_JPRB, 0.15956E+01_JPRB, 0.19257E+01_JPRB, 0.17537E+01_JPRB ,&
&  0.16542E+01_JPRB, 0.15722E+01_JPRB, 0.16085E+01_JPRB, 0.16596E+01_JPRB, 0.16627E+01_JPRB, 0.17208E+01_JPRB, 0.17775E+01_JPRB ,&
&  0.17609E+01_JPRB, 0.18748E+01_JPRB, 0.21168E+01_JPRB, 0.23474E+01_JPRB, 0.25664E+01_JPRB, 0.28516E+01_JPRB, 0.31496E+01_JPRB ,&
&  0.35146E+01_JPRB, 0.38284E+01_JPRB, 0.40311E+01_JPRB, 0.39615E+01_JPRB, 0.38341E+01_JPRB, 0.36169E+01_JPRB, 0.33077E+01_JPRB ,&
&  0.28952E+01_JPRB, 0.26037E+01_JPRB, 0.24129E+01_JPRB, 0.22645E+01_JPRB, 0.20819E+01_JPRB, 0.18399E+01_JPRB, 0.15262E+01_JPRB ,&
&  0.12906E+01_JPRB, 0.11401E+01_JPRB, 0.10785E+01_JPRB, 0.10606E+01_JPRB, 0.10661E+01_JPRB, 0.10717E+01_JPRB, 0.10610E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10301E+01_JPRB, 0.10206E+01_JPRB, 0.10329E+01_JPRB, 0.11056E+01_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.11809E+01_JPRB, 0.11428E+01_JPRB, 0.11048E+01_JPRB, 0.10888E+01_JPRB, 0.10615E+01_JPRB, 0.99440E+00_JPRB, 0.89902E+00_JPRB ,&
&  0.89510E+00_JPRB, 0.85316E+00_JPRB, 0.83545E+00_JPRB, 0.81703E+00_JPRB, 0.79177E+00_JPRB, 0.74339E+00_JPRB, 0.70898E+00_JPRB ,&
&  0.71370E+00_JPRB, 0.74371E+00_JPRB, 0.77786E+00_JPRB, 0.80488E+00_JPRB, 0.81621E+00_JPRB, 0.81223E+00_JPRB, 0.80238E+00_JPRB ,&
&  0.79298E+00_JPRB, 0.78934E+00_JPRB, 0.78905E+00_JPRB, 0.78878E+00_JPRB, 0.78248E+00_JPRB, 0.77419E+00_JPRB, 0.77046E+00_JPRB ,&
&  0.76309E+00_JPRB, 0.75481E+00_JPRB, 0.74827E+00_JPRB, 0.75836E+00_JPRB, 0.77064E+00_JPRB, 0.79701E+00_JPRB, 0.81170E+00_JPRB ,&
&  0.74868E+00_JPRB, 0.76507E+00_JPRB, 0.86558E+00_JPRB, 0.89987E+00_JPRB, 0.93532E+00_JPRB, 0.96581E+00_JPRB, 0.99180E+00_JPRB ,&
&  0.10175E+01_JPRB, 0.10534E+01_JPRB, 0.10944E+01_JPRB, 0.11348E+01_JPRB, 0.11636E+01_JPRB, 0.12200E+01_JPRB, 0.13079E+01_JPRB ,&
&  0.15042E+01_JPRB, 0.14692E+01_JPRB, 0.14421E+01_JPRB, 0.15873E+01_JPRB, 0.21509E+01_JPRB, 0.22688E+01_JPRB, 0.22334E+01_JPRB ,&
&  0.21980E+01_JPRB, 0.20845E+01_JPRB, 0.20363E+01_JPRB, 0.20862E+01_JPRB, 0.21290E+01_JPRB, 0.22612E+01_JPRB, 0.24541E+01_JPRB ,&
&  0.24718E+01_JPRB, 0.26500E+01_JPRB, 0.28712E+01_JPRB, 0.30527E+01_JPRB, 0.32557E+01_JPRB, 0.35724E+01_JPRB, 0.39467E+01_JPRB ,&
&  0.43387E+01_JPRB, 0.46630E+01_JPRB, 0.47774E+01_JPRB, 0.46582E+01_JPRB, 0.44056E+01_JPRB, 0.41058E+01_JPRB, 0.37216E+01_JPRB ,&
&  0.32230E+01_JPRB, 0.28833E+01_JPRB, 0.26673E+01_JPRB, 0.24651E+01_JPRB, 0.22803E+01_JPRB, 0.20307E+01_JPRB, 0.16568E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.10984E+01_JPRB, 0.10620E+01_JPRB, 0.10572E+01_JPRB, 0.10703E+01_JPRB, 0.10957E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.10993E+01_JPRB, 0.10902E+01_JPRB, 0.11286E+01_JPRB, 0.11753E+01_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.11264E+01_JPRB, 0.11088E+01_JPRB, 0.11082E+01_JPRB, 0.11151E+01_JPRB, 0.10804E+01_JPRB, 0.10132E+01_JPRB, 0.95027E+00_JPRB ,&
&  0.10622E+01_JPRB, 0.10578E+01_JPRB, 0.11028E+01_JPRB, 0.93596E+00_JPRB, 0.88140E+00_JPRB, 0.85210E+00_JPRB, 0.80678E+00_JPRB ,&
&  0.77377E+00_JPRB, 0.75945E+00_JPRB, 0.75890E+00_JPRB, 0.77923E+00_JPRB, 0.79358E+00_JPRB, 0.79408E+00_JPRB, 0.79332E+00_JPRB ,&
&  0.78993E+00_JPRB, 0.79138E+00_JPRB, 0.80181E+00_JPRB, 0.80169E+00_JPRB, 0.79912E+00_JPRB, 0.79879E+00_JPRB, 0.80185E+00_JPRB ,&
&  0.77623E+00_JPRB, 0.76807E+00_JPRB, 0.77077E+00_JPRB, 0.77097E+00_JPRB, 0.78622E+00_JPRB, 0.80132E+00_JPRB, 0.83164E+00_JPRB ,&
&  0.88766E+00_JPRB, 0.86680E+00_JPRB, 0.97922E+00_JPRB, 0.10124E+01_JPRB, 0.10470E+01_JPRB, 0.10774E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.11623E+01_JPRB, 0.12024E+01_JPRB, 0.12348E+01_JPRB, 0.12678E+01_JPRB, 0.13700E+01_JPRB, 0.15148E+01_JPRB ,&
&  0.17949E+01_JPRB, 0.16426E+01_JPRB, 0.18096E+01_JPRB, 0.19174E+01_JPRB, 0.22770E+01_JPRB, 0.26247E+01_JPRB, 0.26448E+01_JPRB ,&
&  0.30059E+01_JPRB, 0.28279E+01_JPRB, 0.26662E+01_JPRB, 0.25254E+01_JPRB, 0.26220E+01_JPRB, 0.28260E+01_JPRB, 0.30478E+01_JPRB ,&
&  0.32300E+01_JPRB, 0.33592E+01_JPRB, 0.36263E+01_JPRB, 0.38210E+01_JPRB, 0.40404E+01_JPRB, 0.43201E+01_JPRB, 0.45910E+01_JPRB ,&
&  0.48364E+01_JPRB, 0.49880E+01_JPRB, 0.50603E+01_JPRB, 0.47448E+01_JPRB, 0.44787E+01_JPRB, 0.42059E+01_JPRB, 0.36496E+01_JPRB ,&
&  0.32099E+01_JPRB, 0.28986E+01_JPRB, 0.26484E+01_JPRB, 0.24727E+01_JPRB, 0.21531E+01_JPRB, 0.18865E+01_JPRB, 0.16223E+01_JPRB ,&
&  0.12783E+01_JPRB, 0.11473E+01_JPRB, 0.11519E+01_JPRB, 0.11748E+01_JPRB, 0.11777E+01_JPRB, 0.11873E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.11626E+01_JPRB, 0.12015E+01_JPRB, 0.11850E+01_JPRB, 0.11556E+01_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.11931E+01_JPRB, 0.12026E+01_JPRB, 0.12000E+01_JPRB, 0.11706E+01_JPRB, 0.11285E+01_JPRB, 0.11294E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.12160E+01_JPRB, 0.12148E+01_JPRB, 0.12140E+01_JPRB, 0.12201E+01_JPRB, 0.11166E+01_JPRB, 0.94730E+00_JPRB ,&
&  0.86777E+00_JPRB, 0.85897E+00_JPRB, 0.87034E+00_JPRB, 0.88092E+00_JPRB, 0.89078E+00_JPRB, 0.89442E+00_JPRB, 0.90505E+00_JPRB ,&
&  0.90968E+00_JPRB, 0.90007E+00_JPRB, 0.89992E+00_JPRB, 0.89121E+00_JPRB, 0.87566E+00_JPRB, 0.86673E+00_JPRB, 0.86428E+00_JPRB ,&
&  0.81960E+00_JPRB, 0.79445E+00_JPRB, 0.77913E+00_JPRB, 0.76074E+00_JPRB, 0.79745E+00_JPRB, 0.83260E+00_JPRB, 0.88506E+00_JPRB ,&
&  0.95753E+00_JPRB, 0.10308E+01_JPRB, 0.10845E+01_JPRB, 0.11345E+01_JPRB, 0.11751E+01_JPRB, 0.12113E+01_JPRB, 0.12449E+01_JPRB ,&
&  0.12820E+01_JPRB, 0.13190E+01_JPRB, 0.13610E+01_JPRB, 0.13923E+01_JPRB, 0.14619E+01_JPRB, 0.16892E+01_JPRB, 0.21249E+01_JPRB ,&
&  0.23014E+01_JPRB, 0.25085E+01_JPRB, 0.26618E+01_JPRB, 0.24258E+01_JPRB, 0.26463E+01_JPRB, 0.29646E+01_JPRB, 0.29879E+01_JPRB ,&
&  0.30851E+01_JPRB, 0.30778E+01_JPRB, 0.31889E+01_JPRB, 0.33593E+01_JPRB, 0.37940E+01_JPRB, 0.41097E+01_JPRB, 0.42425E+01_JPRB ,&
&  0.43286E+01_JPRB, 0.46552E+01_JPRB, 0.49128E+01_JPRB, 0.50205E+01_JPRB, 0.50243E+01_JPRB, 0.50664E+01_JPRB, 0.51567E+01_JPRB ,&
&  0.52346E+01_JPRB, 0.53121E+01_JPRB, 0.52781E+01_JPRB, 0.50856E+01_JPRB, 0.47860E+01_JPRB, 0.43930E+01_JPRB, 0.38560E+01_JPRB ,&
&  0.33899E+01_JPRB, 0.28378E+01_JPRB, 0.23913E+01_JPRB, 0.21288E+01_JPRB, 0.18862E+01_JPRB, 0.16418E+01_JPRB, 0.14726E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.12779E+01_JPRB, 0.13822E+01_JPRB, 0.13937E+01_JPRB, 0.13919E+01_JPRB, 0.13500E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.12193E+01_JPRB, 0.12498E+01_JPRB, 0.12677E+01_JPRB, 0.12425E+01_JPRB, 0.12091E+01_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.13869E+01_JPRB, 0.13633E+01_JPRB, 0.13288E+01_JPRB, 0.12876E+01_JPRB, 0.12706E+01_JPRB, 0.12707E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12918E+01_JPRB, 0.13179E+01_JPRB, 0.13173E+01_JPRB, 0.12909E+01_JPRB, 0.12572E+01_JPRB, 0.12313E+01_JPRB, 0.11206E+01_JPRB ,&
&  0.99627E+00_JPRB, 0.99685E+00_JPRB, 0.10003E+01_JPRB, 0.10764E+01_JPRB, 0.11666E+01_JPRB, 0.11808E+01_JPRB, 0.11867E+01_JPRB ,&
&  0.11987E+01_JPRB, 0.11518E+01_JPRB, 0.11378E+01_JPRB, 0.11239E+01_JPRB, 0.10970E+01_JPRB, 0.10693E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.94443E+00_JPRB, 0.88574E+00_JPRB, 0.85895E+00_JPRB, 0.84627E+00_JPRB, 0.88458E+00_JPRB, 0.95511E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.11167E+01_JPRB, 0.11859E+01_JPRB, 0.12536E+01_JPRB, 0.13120E+01_JPRB, 0.13512E+01_JPRB, 0.13866E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.15191E+01_JPRB, 0.15506E+01_JPRB, 0.16067E+01_JPRB, 0.17306E+01_JPRB, 0.24173E+01_JPRB, 0.32279E+01_JPRB ,&
&  0.31211E+01_JPRB, 0.31326E+01_JPRB, 0.33605E+01_JPRB, 0.36511E+01_JPRB, 0.37557E+01_JPRB, 0.36465E+01_JPRB, 0.36947E+01_JPRB ,&
&  0.40223E+01_JPRB, 0.44218E+01_JPRB, 0.52127E+01_JPRB, 0.52646E+01_JPRB, 0.53711E+01_JPRB, 0.55223E+01_JPRB, 0.54380E+01_JPRB ,&
&  0.55161E+01_JPRB, 0.55301E+01_JPRB, 0.56868E+01_JPRB, 0.55963E+01_JPRB, 0.54567E+01_JPRB, 0.54787E+01_JPRB, 0.54495E+01_JPRB ,&
&  0.55160E+01_JPRB, 0.55280E+01_JPRB, 0.52086E+01_JPRB, 0.49349E+01_JPRB, 0.44506E+01_JPRB, 0.44528E+01_JPRB, 0.40873E+01_JPRB ,&
&  0.33953E+01_JPRB, 0.24851E+01_JPRB, 0.22536E+01_JPRB, 0.19971E+01_JPRB, 0.18316E+01_JPRB, 0.16565E+01_JPRB, 0.15147E+01_JPRB ,&
&  0.14529E+01_JPRB, 0.15588E+01_JPRB, 0.15727E+01_JPRB, 0.15770E+01_JPRB, 0.15848E+01_JPRB, 0.15404E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.14192E+01_JPRB, 0.14402E+01_JPRB, 0.14238E+01_JPRB, 0.14035E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.16498E+01_JPRB, 0.15825E+01_JPRB, 0.15275E+01_JPRB, 0.14925E+01_JPRB, 0.14598E+01_JPRB, 0.14192E+01_JPRB, 0.13779E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.14112E+01_JPRB, 0.14121E+01_JPRB, 0.13637E+01_JPRB, 0.13072E+01_JPRB, 0.12678E+01_JPRB, 0.12651E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.11931E+01_JPRB, 0.12539E+01_JPRB, 0.14426E+01_JPRB, 0.18537E+01_JPRB, 0.17908E+01_JPRB, 0.18157E+01_JPRB ,&
&  0.17505E+01_JPRB, 0.16748E+01_JPRB, 0.16244E+01_JPRB, 0.15957E+01_JPRB, 0.15341E+01_JPRB, 0.14879E+01_JPRB, 0.14140E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.11152E+01_JPRB, 0.10747E+01_JPRB, 0.10595E+01_JPRB, 0.11130E+01_JPRB, 0.11807E+01_JPRB, 0.12826E+01_JPRB ,&
&  0.13632E+01_JPRB, 0.14436E+01_JPRB, 0.15161E+01_JPRB, 0.15774E+01_JPRB, 0.16141E+01_JPRB, 0.16464E+01_JPRB, 0.16785E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.17622E+01_JPRB, 0.18072E+01_JPRB, 0.18826E+01_JPRB, 0.22397E+01_JPRB, 0.35271E+01_JPRB, 0.48590E+01_JPRB ,&
&  0.46846E+01_JPRB, 0.47220E+01_JPRB, 0.61358E+01_JPRB, 0.73191E+01_JPRB, 0.65012E+01_JPRB, 0.56885E+01_JPRB, 0.53963E+01_JPRB ,&
&  0.57409E+01_JPRB, 0.63126E+01_JPRB, 0.69157E+01_JPRB, 0.70481E+01_JPRB, 0.68563E+01_JPRB, 0.68375E+01_JPRB, 0.67113E+01_JPRB ,&
&  0.64796E+01_JPRB, 0.63140E+01_JPRB, 0.60948E+01_JPRB, 0.58904E+01_JPRB, 0.57066E+01_JPRB, 0.58072E+01_JPRB, 0.57526E+01_JPRB ,&
&  0.56197E+01_JPRB, 0.53167E+01_JPRB, 0.48539E+01_JPRB, 0.45963E+01_JPRB, 0.37950E+01_JPRB, 0.36812E+01_JPRB, 0.41703E+01_JPRB ,&
&  0.34251E+01_JPRB, 0.26887E+01_JPRB, 0.22961E+01_JPRB, 0.20851E+01_JPRB, 0.20286E+01_JPRB, 0.21338E+01_JPRB, 0.21049E+01_JPRB ,&
&  0.18359E+01_JPRB, 0.18848E+01_JPRB, 0.20284E+01_JPRB, 0.20577E+01_JPRB, 0.20419E+01_JPRB, 0.19985E+01_JPRB, 0.18315E+01_JPRB ,&
&  0.18133E+01_JPRB, 0.17560E+01_JPRB, 0.17532E+01_JPRB, 0.17387E+01_JPRB, 0.17076E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.19141E+01_JPRB, 0.18023E+01_JPRB, 0.17355E+01_JPRB, 0.16575E+01_JPRB, 0.15699E+01_JPRB, 0.14887E+01_JPRB, 0.14370E+01_JPRB ,&
&  0.14642E+01_JPRB, 0.14790E+01_JPRB, 0.14569E+01_JPRB, 0.13992E+01_JPRB, 0.13488E+01_JPRB, 0.13184E+01_JPRB, 0.13321E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.14519E+01_JPRB, 0.14990E+01_JPRB, 0.15428E+01_JPRB, 0.19659E+01_JPRB, 0.22246E+01_JPRB, 0.24448E+01_JPRB ,&
&  0.24879E+01_JPRB, 0.25223E+01_JPRB, 0.24975E+01_JPRB, 0.24522E+01_JPRB, 0.23757E+01_JPRB, 0.22850E+01_JPRB, 0.21450E+01_JPRB ,&
&  0.19555E+01_JPRB, 0.17455E+01_JPRB, 0.16311E+01_JPRB, 0.16129E+01_JPRB, 0.16721E+01_JPRB, 0.15356E+01_JPRB, 0.16383E+01_JPRB ,&
&  0.17320E+01_JPRB, 0.18277E+01_JPRB, 0.18744E+01_JPRB, 0.18947E+01_JPRB, 0.18856E+01_JPRB, 0.18822E+01_JPRB, 0.18882E+01_JPRB ,&
&  0.19361E+01_JPRB, 0.19312E+01_JPRB, 0.18703E+01_JPRB, 0.20514E+01_JPRB, 0.27341E+01_JPRB, 0.43440E+01_JPRB, 0.59757E+01_JPRB ,&
&  0.65522E+01_JPRB, 0.79669E+01_JPRB, 0.99942E+01_JPRB, 0.10435E+02_JPRB, 0.88669E+01_JPRB, 0.78112E+01_JPRB, 0.76811E+01_JPRB ,&
&  0.81060E+01_JPRB, 0.86575E+01_JPRB, 0.88539E+01_JPRB, 0.83316E+01_JPRB, 0.82116E+01_JPRB, 0.80203E+01_JPRB, 0.76889E+01_JPRB ,&
&  0.71794E+01_JPRB, 0.66415E+01_JPRB, 0.63095E+01_JPRB, 0.60400E+01_JPRB, 0.58333E+01_JPRB, 0.53599E+01_JPRB, 0.52005E+01_JPRB ,&
&  0.52969E+01_JPRB, 0.43968E+01_JPRB, 0.39350E+01_JPRB, 0.41405E+01_JPRB, 0.33768E+01_JPRB, 0.33167E+01_JPRB, 0.36095E+01_JPRB ,&
&  0.30662E+01_JPRB, 0.25671E+01_JPRB, 0.23626E+01_JPRB, 0.22088E+01_JPRB, 0.22751E+01_JPRB, 0.27659E+01_JPRB, 0.30763E+01_JPRB ,&
&  0.28473E+01_JPRB, 0.26444E+01_JPRB, 0.29260E+01_JPRB, 0.30273E+01_JPRB, 0.30571E+01_JPRB, 0.28997E+01_JPRB, 0.26673E+01_JPRB ,&
&  0.25023E+01_JPRB, 0.23924E+01_JPRB, 0.22980E+01_JPRB, 0.21882E+01_JPRB, 0.20670E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.20923E+01_JPRB, 0.20301E+01_JPRB, 0.19291E+01_JPRB, 0.17921E+01_JPRB, 0.16718E+01_JPRB, 0.15789E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.14696E+01_JPRB, 0.14317E+01_JPRB, 0.14010E+01_JPRB, 0.13623E+01_JPRB, 0.13462E+01_JPRB, 0.13471E+01_JPRB, 0.13547E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.15134E+01_JPRB, 0.14590E+01_JPRB, 0.15018E+01_JPRB, 0.17588E+01_JPRB, 0.25834E+01_JPRB, 0.31786E+01_JPRB ,&
&  0.34392E+01_JPRB, 0.34553E+01_JPRB, 0.35144E+01_JPRB, 0.37355E+01_JPRB, 0.38229E+01_JPRB, 0.36641E+01_JPRB, 0.35636E+01_JPRB ,&
&  0.34190E+01_JPRB, 0.31651E+01_JPRB, 0.26489E+01_JPRB, 0.25979E+01_JPRB, 0.24351E+01_JPRB, 0.21677E+01_JPRB, 0.23389E+01_JPRB ,&
&  0.23642E+01_JPRB, 0.23146E+01_JPRB, 0.21955E+01_JPRB, 0.20843E+01_JPRB, 0.20054E+01_JPRB, 0.18861E+01_JPRB, 0.18197E+01_JPRB ,&
&  0.18314E+01_JPRB, 0.18993E+01_JPRB, 0.20720E+01_JPRB, 0.26474E+01_JPRB, 0.40407E+01_JPRB, 0.60299E+01_JPRB, 0.75395E+01_JPRB ,&
&  0.89086E+01_JPRB, 0.96720E+01_JPRB, 0.91885E+01_JPRB, 0.98942E+01_JPRB, 0.10935E+02_JPRB, 0.10286E+02_JPRB, 0.95947E+01_JPRB ,&
&  0.95027E+01_JPRB, 0.93514E+01_JPRB, 0.89805E+01_JPRB, 0.86449E+01_JPRB, 0.86695E+01_JPRB, 0.83823E+01_JPRB, 0.78162E+01_JPRB ,&
&  0.66975E+01_JPRB, 0.62888E+01_JPRB, 0.62115E+01_JPRB, 0.60921E+01_JPRB, 0.54256E+01_JPRB, 0.51256E+01_JPRB, 0.51756E+01_JPRB ,&
&  0.50513E+01_JPRB, 0.49605E+01_JPRB, 0.39234E+01_JPRB, 0.37267E+01_JPRB, 0.28610E+01_JPRB, 0.27818E+01_JPRB, 0.31941E+01_JPRB ,&
&  0.29192E+01_JPRB, 0.27045E+01_JPRB, 0.24612E+01_JPRB, 0.24964E+01_JPRB, 0.33809E+01_JPRB, 0.42876E+01_JPRB, 0.52007E+01_JPRB ,&
&  0.47991E+01_JPRB, 0.38764E+01_JPRB, 0.40608E+01_JPRB, 0.40925E+01_JPRB, 0.41533E+01_JPRB, 0.38836E+01_JPRB, 0.34554E+01_JPRB ,&
&  0.31219E+01_JPRB, 0.28630E+01_JPRB, 0.25981E+01_JPRB, 0.23496E+01_JPRB, 0.21880E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.21207E+01_JPRB, 0.20634E+01_JPRB, 0.19476E+01_JPRB, 0.17984E+01_JPRB, 0.16804E+01_JPRB, 0.15876E+01_JPRB, 0.15097E+01_JPRB ,&
&  0.14335E+01_JPRB, 0.13721E+01_JPRB, 0.13348E+01_JPRB, 0.13194E+01_JPRB, 0.13201E+01_JPRB, 0.13245E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.14968E+01_JPRB, 0.14626E+01_JPRB, 0.14739E+01_JPRB, 0.18684E+01_JPRB, 0.28010E+01_JPRB, 0.36646E+01_JPRB ,&
&  0.40102E+01_JPRB, 0.40273E+01_JPRB, 0.42712E+01_JPRB, 0.47051E+01_JPRB, 0.52585E+01_JPRB, 0.55924E+01_JPRB, 0.57244E+01_JPRB ,&
&  0.56123E+01_JPRB, 0.43749E+01_JPRB, 0.39303E+01_JPRB, 0.35355E+01_JPRB, 0.33224E+01_JPRB, 0.31583E+01_JPRB, 0.29807E+01_JPRB ,&
&  0.27141E+01_JPRB, 0.23756E+01_JPRB, 0.21171E+01_JPRB, 0.19436E+01_JPRB, 0.18527E+01_JPRB, 0.17611E+01_JPRB, 0.18433E+01_JPRB ,&
&  0.20492E+01_JPRB, 0.23285E+01_JPRB, 0.32031E+01_JPRB, 0.46845E+01_JPRB, 0.71337E+01_JPRB, 0.91465E+01_JPRB, 0.93754E+01_JPRB ,&
&  0.80909E+01_JPRB, 0.85390E+01_JPRB, 0.12516E+02_JPRB, 0.11240E+02_JPRB, 0.11150E+02_JPRB, 0.12020E+02_JPRB, 0.11216E+02_JPRB ,&
&  0.99936E+01_JPRB, 0.90020E+01_JPRB, 0.83777E+01_JPRB, 0.81084E+01_JPRB, 0.93108E+01_JPRB, 0.89626E+01_JPRB, 0.78879E+01_JPRB ,&
&  0.68141E+01_JPRB, 0.66125E+01_JPRB, 0.62280E+01_JPRB, 0.57913E+01_JPRB, 0.54424E+01_JPRB, 0.56789E+01_JPRB, 0.53219E+01_JPRB ,&
&  0.49932E+01_JPRB, 0.55501E+01_JPRB, 0.44393E+01_JPRB, 0.34204E+01_JPRB, 0.28395E+01_JPRB, 0.29538E+01_JPRB, 0.31999E+01_JPRB ,&
&  0.32149E+01_JPRB, 0.31652E+01_JPRB, 0.32580E+01_JPRB, 0.42875E+01_JPRB, 0.63992E+01_JPRB, 0.78039E+01_JPRB, 0.71077E+01_JPRB ,&
&  0.62223E+01_JPRB, 0.62147E+01_JPRB, 0.58224E+01_JPRB, 0.56663E+01_JPRB, 0.54612E+01_JPRB, 0.47193E+01_JPRB, 0.38261E+01_JPRB ,&
&  0.30742E+01_JPRB, 0.26074E+01_JPRB, 0.23669E+01_JPRB, 0.22345E+01_JPRB, 0.21632E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.21896E+01_JPRB, 0.20722E+01_JPRB, 0.19363E+01_JPRB, 0.17939E+01_JPRB, 0.16601E+01_JPRB, 0.15485E+01_JPRB, 0.14563E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.13038E+01_JPRB, 0.12889E+01_JPRB, 0.13005E+01_JPRB, 0.13199E+01_JPRB, 0.13086E+01_JPRB, 0.13134E+01_JPRB ,&
&  0.13711E+01_JPRB, 0.14996E+01_JPRB, 0.14549E+01_JPRB, 0.15961E+01_JPRB, 0.19986E+01_JPRB, 0.26381E+01_JPRB, 0.34988E+01_JPRB ,&
&  0.39302E+01_JPRB, 0.38562E+01_JPRB, 0.41258E+01_JPRB, 0.54054E+01_JPRB, 0.71617E+01_JPRB, 0.83126E+01_JPRB, 0.85842E+01_JPRB ,&
&  0.75080E+01_JPRB, 0.66340E+01_JPRB, 0.56681E+01_JPRB, 0.48239E+01_JPRB, 0.42060E+01_JPRB, 0.36497E+01_JPRB, 0.30071E+01_JPRB ,&
&  0.24366E+01_JPRB, 0.20644E+01_JPRB, 0.18894E+01_JPRB, 0.18214E+01_JPRB, 0.18335E+01_JPRB, 0.19055E+01_JPRB, 0.21683E+01_JPRB ,&
&  0.26341E+01_JPRB, 0.34430E+01_JPRB, 0.49196E+01_JPRB, 0.67478E+01_JPRB, 0.73056E+01_JPRB, 0.92526E+01_JPRB, 0.90660E+01_JPRB ,&
&  0.98752E+01_JPRB, 0.10540E+02_JPRB, 0.10490E+02_JPRB, 0.11066E+02_JPRB, 0.94815E+01_JPRB, 0.10263E+02_JPRB, 0.10625E+02_JPRB ,&
&  0.10019E+02_JPRB, 0.94042E+01_JPRB, 0.94299E+01_JPRB, 0.84689E+01_JPRB, 0.81827E+01_JPRB, 0.97518E+01_JPRB, 0.85266E+01_JPRB ,&
&  0.73183E+01_JPRB, 0.68286E+01_JPRB, 0.61526E+01_JPRB, 0.59676E+01_JPRB, 0.49979E+01_JPRB, 0.49641E+01_JPRB, 0.43848E+01_JPRB ,&
&  0.42366E+01_JPRB, 0.35215E+01_JPRB, 0.38967E+01_JPRB, 0.33151E+01_JPRB, 0.31602E+01_JPRB, 0.31676E+01_JPRB, 0.32964E+01_JPRB ,&
&  0.36812E+01_JPRB, 0.41731E+01_JPRB, 0.54441E+01_JPRB, 0.78336E+01_JPRB, 0.10575E+02_JPRB, 0.87892E+01_JPRB, 0.83343E+01_JPRB ,&
&  0.75947E+01_JPRB, 0.72543E+01_JPRB, 0.60973E+01_JPRB, 0.56376E+01_JPRB, 0.49738E+01_JPRB, 0.44217E+01_JPRB, 0.38585E+01_JPRB ,&
&  0.32847E+01_JPRB, 0.28102E+01_JPRB, 0.25495E+01_JPRB, 0.24019E+01_JPRB, 0.22938E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.20988E+01_JPRB, 0.19488E+01_JPRB, 0.18213E+01_JPRB, 0.16958E+01_JPRB, 0.15754E+01_JPRB, 0.14516E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.11973E+01_JPRB, 0.12157E+01_JPRB, 0.12479E+01_JPRB, 0.12767E+01_JPRB, 0.13144E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.15299E+01_JPRB, 0.14907E+01_JPRB, 0.15625E+01_JPRB, 0.19553E+01_JPRB, 0.25724E+01_JPRB, 0.30373E+01_JPRB ,&
&  0.36908E+01_JPRB, 0.34243E+01_JPRB, 0.37846E+01_JPRB, 0.55229E+01_JPRB, 0.92839E+01_JPRB, 0.11909E+02_JPRB, 0.11802E+02_JPRB ,&
&  0.11564E+02_JPRB, 0.95369E+01_JPRB, 0.72222E+01_JPRB, 0.54457E+01_JPRB, 0.40026E+01_JPRB, 0.29637E+01_JPRB, 0.22343E+01_JPRB ,&
&  0.18542E+01_JPRB, 0.16906E+01_JPRB, 0.16631E+01_JPRB, 0.16878E+01_JPRB, 0.18030E+01_JPRB, 0.19561E+01_JPRB, 0.25512E+01_JPRB ,&
&  0.32605E+01_JPRB, 0.42248E+01_JPRB, 0.53890E+01_JPRB, 0.61476E+01_JPRB, 0.62024E+01_JPRB, 0.76980E+01_JPRB, 0.91741E+01_JPRB ,&
&  0.88931E+01_JPRB, 0.91074E+01_JPRB, 0.10039E+02_JPRB, 0.10379E+02_JPRB, 0.11005E+02_JPRB, 0.10048E+02_JPRB, 0.96693E+01_JPRB ,&
&  0.73371E+01_JPRB, 0.61108E+01_JPRB, 0.55999E+01_JPRB, 0.57806E+01_JPRB, 0.74134E+01_JPRB, 0.96222E+01_JPRB, 0.89913E+01_JPRB ,&
&  0.78321E+01_JPRB, 0.72350E+01_JPRB, 0.65482E+01_JPRB, 0.52768E+01_JPRB, 0.50164E+01_JPRB, 0.41997E+01_JPRB, 0.50849E+01_JPRB ,&
&  0.50817E+01_JPRB, 0.47537E+01_JPRB, 0.39971E+01_JPRB, 0.31107E+01_JPRB, 0.26241E+01_JPRB, 0.33479E+01_JPRB, 0.38046E+01_JPRB ,&
&  0.46126E+01_JPRB, 0.56676E+01_JPRB, 0.10033E+02_JPRB, 0.11431E+02_JPRB, 0.11401E+02_JPRB, 0.87180E+01_JPRB, 0.88246E+01_JPRB ,&
&  0.77489E+01_JPRB, 0.74049E+01_JPRB, 0.62819E+01_JPRB, 0.48360E+01_JPRB, 0.39049E+01_JPRB, 0.33745E+01_JPRB, 0.31056E+01_JPRB ,&
&  0.29984E+01_JPRB, 0.28852E+01_JPRB, 0.26897E+01_JPRB, 0.24789E+01_JPRB, 0.22992E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.17705E+01_JPRB, 0.16212E+01_JPRB, 0.15195E+01_JPRB, 0.14353E+01_JPRB, 0.13408E+01_JPRB, 0.12494E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11156E+01_JPRB, 0.11516E+01_JPRB, 0.12140E+01_JPRB, 0.12732E+01_JPRB, 0.13266E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.14249E+01_JPRB, 0.16273E+01_JPRB, 0.15645E+01_JPRB, 0.16564E+01_JPRB, 0.20081E+01_JPRB, 0.25072E+01_JPRB, 0.28930E+01_JPRB ,&
&  0.34046E+01_JPRB, 0.32701E+01_JPRB, 0.39285E+01_JPRB, 0.57200E+01_JPRB, 0.11086E+02_JPRB, 0.12929E+02_JPRB, 0.12135E+02_JPRB ,&
&  0.11648E+02_JPRB, 0.86102E+01_JPRB, 0.55690E+01_JPRB, 0.39193E+01_JPRB, 0.28792E+01_JPRB, 0.21537E+01_JPRB, 0.18213E+01_JPRB ,&
&  0.17094E+01_JPRB, 0.16883E+01_JPRB, 0.17077E+01_JPRB, 0.17685E+01_JPRB, 0.19036E+01_JPRB, 0.22818E+01_JPRB, 0.30391E+01_JPRB ,&
&  0.38401E+01_JPRB, 0.46192E+01_JPRB, 0.51468E+01_JPRB, 0.52540E+01_JPRB, 0.53543E+01_JPRB, 0.66891E+01_JPRB, 0.92026E+01_JPRB ,&
&  0.86438E+01_JPRB, 0.91662E+01_JPRB, 0.96661E+01_JPRB, 0.10409E+02_JPRB, 0.99740E+01_JPRB, 0.93275E+01_JPRB, 0.86277E+01_JPRB ,&
&  0.63442E+01_JPRB, 0.56085E+01_JPRB, 0.61513E+01_JPRB, 0.67706E+01_JPRB, 0.65780E+01_JPRB, 0.74795E+01_JPRB, 0.93877E+01_JPRB ,&
&  0.72081E+01_JPRB, 0.68544E+01_JPRB, 0.66270E+01_JPRB, 0.63308E+01_JPRB, 0.37047E+01_JPRB, 0.26137E+01_JPRB, 0.38865E+01_JPRB ,&
&  0.37900E+01_JPRB, 0.21649E+01_JPRB, 0.17123E+01_JPRB, 0.23512E+01_JPRB, 0.17032E+01_JPRB, 0.23105E+01_JPRB, 0.43306E+01_JPRB ,&
&  0.57783E+01_JPRB, 0.75501E+01_JPRB, 0.14122E+02_JPRB, 0.11975E+02_JPRB, 0.11141E+02_JPRB, 0.10116E+02_JPRB, 0.97063E+01_JPRB ,&
&  0.88081E+01_JPRB, 0.57382E+01_JPRB, 0.42789E+01_JPRB, 0.32752E+01_JPRB, 0.27194E+01_JPRB, 0.26292E+01_JPRB, 0.25666E+01_JPRB ,&
&  0.25114E+01_JPRB, 0.23823E+01_JPRB, 0.22590E+01_JPRB, 0.20810E+01_JPRB, 0.19394E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.13442E+01_JPRB, 0.12447E+01_JPRB, 0.12165E+01_JPRB, 0.11917E+01_JPRB, 0.11267E+01_JPRB, 0.10799E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.10949E+01_JPRB, 0.11563E+01_JPRB, 0.12331E+01_JPRB, 0.13117E+01_JPRB, 0.13720E+01_JPRB, 0.13863E+01_JPRB ,&
&  0.14317E+01_JPRB, 0.15964E+01_JPRB, 0.18134E+01_JPRB, 0.20499E+01_JPRB, 0.24892E+01_JPRB, 0.25651E+01_JPRB, 0.32701E+01_JPRB ,&
&  0.33043E+01_JPRB, 0.32024E+01_JPRB, 0.41961E+01_JPRB, 0.61639E+01_JPRB, 0.91090E+01_JPRB, 0.99069E+01_JPRB, 0.89436E+01_JPRB ,&
&  0.70190E+01_JPRB, 0.48645E+01_JPRB, 0.35290E+01_JPRB, 0.27003E+01_JPRB, 0.21905E+01_JPRB, 0.19141E+01_JPRB, 0.18958E+01_JPRB ,&
&  0.18249E+01_JPRB, 0.18200E+01_JPRB, 0.18682E+01_JPRB, 0.18792E+01_JPRB, 0.21357E+01_JPRB, 0.26175E+01_JPRB, 0.33661E+01_JPRB ,&
&  0.40324E+01_JPRB, 0.45358E+01_JPRB, 0.47457E+01_JPRB, 0.46217E+01_JPRB, 0.59224E+01_JPRB, 0.69590E+01_JPRB, 0.69286E+01_JPRB ,&
&  0.71012E+01_JPRB, 0.75888E+01_JPRB, 0.97895E+01_JPRB, 0.98360E+01_JPRB, 0.92000E+01_JPRB, 0.94644E+01_JPRB, 0.83110E+01_JPRB ,&
&  0.76083E+01_JPRB, 0.73882E+01_JPRB, 0.66757E+01_JPRB, 0.71411E+01_JPRB, 0.74379E+01_JPRB, 0.68610E+01_JPRB, 0.72783E+01_JPRB ,&
&  0.72818E+01_JPRB, 0.63224E+01_JPRB, 0.49848E+01_JPRB, 0.39924E+01_JPRB, 0.39185E+01_JPRB, 0.31009E+01_JPRB, 0.20770E+01_JPRB ,&
&  0.17249E+01_JPRB, 0.14629E+01_JPRB, 0.11181E+01_JPRB, 0.13249E+01_JPRB, 0.12843E+01_JPRB, 0.20239E+01_JPRB, 0.42308E+01_JPRB ,&
&  0.82571E+01_JPRB, 0.11159E+02_JPRB, 0.14147E+02_JPRB, 0.11725E+02_JPRB, 0.10430E+02_JPRB, 0.10171E+02_JPRB, 0.10210E+02_JPRB ,&
&  0.50954E+01_JPRB, 0.35201E+01_JPRB, 0.27742E+01_JPRB, 0.24331E+01_JPRB, 0.23169E+01_JPRB, 0.22347E+01_JPRB, 0.21456E+01_JPRB ,&
&  0.20041E+01_JPRB, 0.18768E+01_JPRB, 0.18157E+01_JPRB, 0.17214E+01_JPRB, 0.15308E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.11309E+01_JPRB, 0.11110E+01_JPRB, 0.11157E+01_JPRB, 0.10866E+01_JPRB, 0.10241E+01_JPRB, 0.10330E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11721E+01_JPRB, 0.12493E+01_JPRB, 0.13435E+01_JPRB, 0.14503E+01_JPRB, 0.14930E+01_JPRB, 0.14848E+01_JPRB ,&
&  0.15131E+01_JPRB, 0.16900E+01_JPRB, 0.20583E+01_JPRB, 0.25401E+01_JPRB, 0.31241E+01_JPRB, 0.29549E+01_JPRB, 0.35889E+01_JPRB ,&
&  0.30194E+01_JPRB, 0.30161E+01_JPRB, 0.39803E+01_JPRB, 0.54006E+01_JPRB, 0.69677E+01_JPRB, 0.65518E+01_JPRB, 0.48614E+01_JPRB ,&
&  0.34467E+01_JPRB, 0.28559E+01_JPRB, 0.25602E+01_JPRB, 0.22420E+01_JPRB, 0.19884E+01_JPRB, 0.19026E+01_JPRB, 0.19200E+01_JPRB ,&
&  0.18771E+01_JPRB, 0.19076E+01_JPRB, 0.19291E+01_JPRB, 0.21298E+01_JPRB, 0.25418E+01_JPRB, 0.30408E+01_JPRB, 0.36284E+01_JPRB ,&
&  0.40132E+01_JPRB, 0.42916E+01_JPRB, 0.44433E+01_JPRB, 0.49613E+01_JPRB, 0.51813E+01_JPRB, 0.53048E+01_JPRB, 0.63201E+01_JPRB ,&
&  0.78399E+01_JPRB, 0.90077E+01_JPRB, 0.86592E+01_JPRB, 0.10012E+02_JPRB, 0.94116E+01_JPRB, 0.87221E+01_JPRB, 0.79590E+01_JPRB ,&
&  0.73126E+01_JPRB, 0.77441E+01_JPRB, 0.68818E+01_JPRB, 0.71637E+01_JPRB, 0.86740E+01_JPRB, 0.82781E+01_JPRB, 0.70485E+01_JPRB ,&
&  0.71693E+01_JPRB, 0.58786E+01_JPRB, 0.49353E+01_JPRB, 0.41474E+01_JPRB, 0.53595E+01_JPRB, 0.54634E+01_JPRB, 0.27501E+01_JPRB ,&
&  0.18236E+01_JPRB, 0.13939E+01_JPRB, 0.10634E+01_JPRB, 0.10895E+01_JPRB, 0.12743E+01_JPRB, 0.20253E+01_JPRB, 0.60156E+01_JPRB ,&
&  0.76691E+01_JPRB, 0.12913E+02_JPRB, 0.14820E+02_JPRB, 0.12950E+02_JPRB, 0.10881E+02_JPRB, 0.88678E+01_JPRB, 0.53741E+01_JPRB ,&
&  0.29936E+01_JPRB, 0.20145E+01_JPRB, 0.19519E+01_JPRB, 0.21390E+01_JPRB, 0.21940E+01_JPRB, 0.20597E+01_JPRB, 0.18620E+01_JPRB ,&
&  0.17266E+01_JPRB, 0.16721E+01_JPRB, 0.15777E+01_JPRB, 0.14784E+01_JPRB, 0.12985E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.11140E+01_JPRB, 0.11468E+01_JPRB, 0.11803E+01_JPRB, 0.11063E+01_JPRB, 0.10906E+01_JPRB, 0.11190E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.13038E+01_JPRB, 0.13949E+01_JPRB, 0.15237E+01_JPRB, 0.16327E+01_JPRB, 0.16252E+01_JPRB, 0.16005E+01_JPRB ,&
&  0.16602E+01_JPRB, 0.19104E+01_JPRB, 0.24166E+01_JPRB, 0.31242E+01_JPRB, 0.34660E+01_JPRB, 0.33133E+01_JPRB, 0.35007E+01_JPRB ,&
&  0.27985E+01_JPRB, 0.28643E+01_JPRB, 0.34181E+01_JPRB, 0.42508E+01_JPRB, 0.44719E+01_JPRB, 0.41532E+01_JPRB, 0.28470E+01_JPRB ,&
&  0.23535E+01_JPRB, 0.23183E+01_JPRB, 0.21107E+01_JPRB, 0.21486E+01_JPRB, 0.19536E+01_JPRB, 0.19276E+01_JPRB, 0.18861E+01_JPRB ,&
&  0.19308E+01_JPRB, 0.20364E+01_JPRB, 0.21692E+01_JPRB, 0.24595E+01_JPRB, 0.29228E+01_JPRB, 0.33716E+01_JPRB, 0.37824E+01_JPRB ,&
&  0.39140E+01_JPRB, 0.41904E+01_JPRB, 0.45554E+01_JPRB, 0.51292E+01_JPRB, 0.42902E+01_JPRB, 0.56325E+01_JPRB, 0.63449E+01_JPRB ,&
&  0.74740E+01_JPRB, 0.81099E+01_JPRB, 0.77857E+01_JPRB, 0.85432E+01_JPRB, 0.89500E+01_JPRB, 0.83476E+01_JPRB, 0.75039E+01_JPRB ,&
&  0.76585E+01_JPRB, 0.74122E+01_JPRB, 0.75235E+01_JPRB, 0.78396E+01_JPRB, 0.89567E+01_JPRB, 0.10795E+02_JPRB, 0.69773E+01_JPRB ,&
&  0.58293E+01_JPRB, 0.59949E+01_JPRB, 0.52019E+01_JPRB, 0.46231E+01_JPRB, 0.56272E+01_JPRB, 0.64316E+01_JPRB, 0.56099E+01_JPRB ,&
&  0.23838E+01_JPRB, 0.13928E+01_JPRB, 0.12975E+01_JPRB, 0.12959E+01_JPRB, 0.11936E+01_JPRB, 0.21773E+01_JPRB, 0.10295E+02_JPRB ,&
&  0.80903E+01_JPRB, 0.12933E+02_JPRB, 0.13597E+02_JPRB, 0.11777E+02_JPRB, 0.10214E+02_JPRB, 0.62943E+01_JPRB, 0.34073E+01_JPRB ,&
&  0.16755E+01_JPRB, 0.16511E+01_JPRB, 0.18678E+01_JPRB, 0.18257E+01_JPRB, 0.18036E+01_JPRB, 0.16854E+01_JPRB, 0.14711E+01_JPRB ,&
&  0.14325E+01_JPRB, 0.14147E+01_JPRB, 0.13113E+01_JPRB, 0.12074E+01_JPRB, 0.11383E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.11698E+01_JPRB, 0.12150E+01_JPRB, 0.12139E+01_JPRB, 0.11991E+01_JPRB, 0.12170E+01_JPRB, 0.12442E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.13359E+01_JPRB, 0.14762E+01_JPRB, 0.16144E+01_JPRB, 0.17806E+01_JPRB, 0.18528E+01_JPRB, 0.18166E+01_JPRB, 0.18260E+01_JPRB ,&
&  0.19922E+01_JPRB, 0.23647E+01_JPRB, 0.29236E+01_JPRB, 0.34351E+01_JPRB, 0.34829E+01_JPRB, 0.31830E+01_JPRB, 0.30774E+01_JPRB ,&
&  0.26365E+01_JPRB, 0.25945E+01_JPRB, 0.23420E+01_JPRB, 0.27083E+01_JPRB, 0.30870E+01_JPRB, 0.33804E+01_JPRB, 0.25977E+01_JPRB ,&
&  0.24304E+01_JPRB, 0.23819E+01_JPRB, 0.22599E+01_JPRB, 0.22464E+01_JPRB, 0.21736E+01_JPRB, 0.21557E+01_JPRB, 0.21371E+01_JPRB ,&
&  0.21316E+01_JPRB, 0.22911E+01_JPRB, 0.25313E+01_JPRB, 0.28665E+01_JPRB, 0.32710E+01_JPRB, 0.36037E+01_JPRB, 0.38920E+01_JPRB ,&
&  0.41431E+01_JPRB, 0.44757E+01_JPRB, 0.48165E+01_JPRB, 0.46517E+01_JPRB, 0.46717E+01_JPRB, 0.52381E+01_JPRB, 0.60343E+01_JPRB ,&
&  0.69386E+01_JPRB, 0.76030E+01_JPRB, 0.78162E+01_JPRB, 0.78229E+01_JPRB, 0.79685E+01_JPRB, 0.79591E+01_JPRB, 0.76040E+01_JPRB ,&
&  0.79592E+01_JPRB, 0.81050E+01_JPRB, 0.76464E+01_JPRB, 0.77984E+01_JPRB, 0.81948E+01_JPRB, 0.85691E+01_JPRB, 0.73836E+01_JPRB ,&
&  0.61851E+01_JPRB, 0.52323E+01_JPRB, 0.48667E+01_JPRB, 0.53072E+01_JPRB, 0.60940E+01_JPRB, 0.62673E+01_JPRB, 0.77594E+01_JPRB ,&
&  0.72044E+01_JPRB, 0.28918E+01_JPRB, 0.20371E+01_JPRB, 0.26899E+01_JPRB, 0.17382E+01_JPRB, 0.21354E+01_JPRB, 0.48082E+01_JPRB ,&
&  0.80589E+01_JPRB, 0.10902E+02_JPRB, 0.10876E+02_JPRB, 0.88074E+01_JPRB, 0.75906E+01_JPRB, 0.40639E+01_JPRB, 0.19768E+01_JPRB ,&
&  0.14889E+01_JPRB, 0.15803E+01_JPRB, 0.15850E+01_JPRB, 0.14221E+01_JPRB, 0.13634E+01_JPRB, 0.14136E+01_JPRB, 0.12498E+01_JPRB ,&
&  0.12554E+01_JPRB, 0.12566E+01_JPRB, 0.12602E+01_JPRB, 0.12153E+01_JPRB, 0.11550E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.12887E+01_JPRB, 0.13178E+01_JPRB, 0.13728E+01_JPRB, 0.14027E+01_JPRB, 0.14082E+01_JPRB, 0.14362E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.15944E+01_JPRB, 0.17568E+01_JPRB, 0.19240E+01_JPRB, 0.20720E+01_JPRB, 0.20891E+01_JPRB, 0.20761E+01_JPRB, 0.22040E+01_JPRB ,&
&  0.24603E+01_JPRB, 0.28395E+01_JPRB, 0.32460E+01_JPRB, 0.35981E+01_JPRB, 0.36753E+01_JPRB, 0.36320E+01_JPRB, 0.27408E+01_JPRB ,&
&  0.22637E+01_JPRB, 0.25249E+01_JPRB, 0.22493E+01_JPRB, 0.24035E+01_JPRB, 0.26623E+01_JPRB, 0.28592E+01_JPRB, 0.27856E+01_JPRB ,&
&  0.26977E+01_JPRB, 0.26168E+01_JPRB, 0.24894E+01_JPRB, 0.24423E+01_JPRB, 0.24523E+01_JPRB, 0.24796E+01_JPRB, 0.25365E+01_JPRB ,&
&  0.26563E+01_JPRB, 0.27881E+01_JPRB, 0.29939E+01_JPRB, 0.32844E+01_JPRB, 0.35869E+01_JPRB, 0.38867E+01_JPRB, 0.42258E+01_JPRB ,&
&  0.44611E+01_JPRB, 0.45870E+01_JPRB, 0.47434E+01_JPRB, 0.44406E+01_JPRB, 0.45927E+01_JPRB, 0.50509E+01_JPRB, 0.56499E+01_JPRB ,&
&  0.59151E+01_JPRB, 0.60717E+01_JPRB, 0.68655E+01_JPRB, 0.76744E+01_JPRB, 0.78844E+01_JPRB, 0.75980E+01_JPRB, 0.73405E+01_JPRB ,&
&  0.78924E+01_JPRB, 0.79889E+01_JPRB, 0.76680E+01_JPRB, 0.69111E+01_JPRB, 0.65574E+01_JPRB, 0.65482E+01_JPRB, 0.66963E+01_JPRB ,&
&  0.61320E+01_JPRB, 0.55599E+01_JPRB, 0.53483E+01_JPRB, 0.55800E+01_JPRB, 0.59038E+01_JPRB, 0.60683E+01_JPRB, 0.68611E+01_JPRB ,&
&  0.73041E+01_JPRB, 0.63555E+01_JPRB, 0.49251E+01_JPRB, 0.32711E+01_JPRB, 0.27996E+01_JPRB, 0.31166E+01_JPRB, 0.44486E+01_JPRB ,&
&  0.70246E+01_JPRB, 0.75118E+01_JPRB, 0.70193E+01_JPRB, 0.72458E+01_JPRB, 0.47340E+01_JPRB, 0.25012E+01_JPRB, 0.14822E+01_JPRB ,&
&  0.12997E+01_JPRB, 0.13367E+01_JPRB, 0.13299E+01_JPRB, 0.12505E+01_JPRB, 0.12676E+01_JPRB, 0.13051E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.12845E+01_JPRB, 0.13354E+01_JPRB, 0.12852E+01_JPRB, 0.12376E+01_JPRB, 0.12566E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.14910E+01_JPRB, 0.15495E+01_JPRB, 0.15941E+01_JPRB, 0.16100E+01_JPRB, 0.16467E+01_JPRB, 0.16499E+01_JPRB, 0.17146E+01_JPRB ,&
&  0.18653E+01_JPRB, 0.20374E+01_JPRB, 0.21863E+01_JPRB, 0.22596E+01_JPRB, 0.22783E+01_JPRB, 0.23431E+01_JPRB, 0.25267E+01_JPRB ,&
&  0.27281E+01_JPRB, 0.29642E+01_JPRB, 0.31739E+01_JPRB, 0.33971E+01_JPRB, 0.34955E+01_JPRB, 0.36582E+01_JPRB, 0.30522E+01_JPRB ,&
&  0.20502E+01_JPRB, 0.28587E+01_JPRB, 0.22762E+01_JPRB, 0.24768E+01_JPRB, 0.26630E+01_JPRB, 0.28834E+01_JPRB, 0.28564E+01_JPRB ,&
&  0.28257E+01_JPRB, 0.27978E+01_JPRB, 0.27270E+01_JPRB, 0.26479E+01_JPRB, 0.26158E+01_JPRB, 0.27317E+01_JPRB, 0.28997E+01_JPRB ,&
&  0.30530E+01_JPRB, 0.31704E+01_JPRB, 0.33204E+01_JPRB, 0.35462E+01_JPRB, 0.37876E+01_JPRB, 0.40528E+01_JPRB, 0.43226E+01_JPRB ,&
&  0.44292E+01_JPRB, 0.45412E+01_JPRB, 0.44741E+01_JPRB, 0.43445E+01_JPRB, 0.43976E+01_JPRB, 0.46982E+01_JPRB, 0.50961E+01_JPRB ,&
&  0.53383E+01_JPRB, 0.55566E+01_JPRB, 0.62529E+01_JPRB, 0.66935E+01_JPRB, 0.67063E+01_JPRB, 0.68328E+01_JPRB, 0.64495E+01_JPRB ,&
&  0.70692E+01_JPRB, 0.67269E+01_JPRB, 0.66631E+01_JPRB, 0.57145E+01_JPRB, 0.51457E+01_JPRB, 0.50550E+01_JPRB, 0.50842E+01_JPRB ,&
&  0.48246E+01_JPRB, 0.49629E+01_JPRB, 0.48932E+01_JPRB, 0.50077E+01_JPRB, 0.54561E+01_JPRB, 0.59407E+01_JPRB, 0.57956E+01_JPRB ,&
&  0.54329E+01_JPRB, 0.39011E+01_JPRB, 0.29911E+01_JPRB, 0.26990E+01_JPRB, 0.24498E+01_JPRB, 0.26862E+01_JPRB, 0.39935E+01_JPRB ,&
&  0.56007E+01_JPRB, 0.56645E+01_JPRB, 0.56322E+01_JPRB, 0.46052E+01_JPRB, 0.26414E+01_JPRB, 0.15967E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.11604E+01_JPRB, 0.12051E+01_JPRB, 0.12045E+01_JPRB, 0.11951E+01_JPRB, 0.12459E+01_JPRB, 0.11891E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.13202E+01_JPRB, 0.13476E+01_JPRB, 0.13511E+01_JPRB, 0.14055E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.16988E+01_JPRB, 0.18373E+01_JPRB, 0.19732E+01_JPRB, 0.20604E+01_JPRB, 0.20158E+01_JPRB, 0.19798E+01_JPRB, 0.19959E+01_JPRB ,&
&  0.20387E+01_JPRB, 0.21178E+01_JPRB, 0.22173E+01_JPRB, 0.23013E+01_JPRB, 0.23658E+01_JPRB, 0.24783E+01_JPRB, 0.26634E+01_JPRB ,&
&  0.28582E+01_JPRB, 0.30410E+01_JPRB, 0.32137E+01_JPRB, 0.34236E+01_JPRB, 0.36518E+01_JPRB, 0.40010E+01_JPRB, 0.37743E+01_JPRB ,&
&  0.26883E+01_JPRB, 0.22868E+01_JPRB, 0.25202E+01_JPRB, 0.24779E+01_JPRB, 0.24883E+01_JPRB, 0.27014E+01_JPRB, 0.28334E+01_JPRB ,&
&  0.28592E+01_JPRB, 0.28120E+01_JPRB, 0.27584E+01_JPRB, 0.27007E+01_JPRB, 0.26850E+01_JPRB, 0.28565E+01_JPRB, 0.30407E+01_JPRB ,&
&  0.32009E+01_JPRB, 0.33069E+01_JPRB, 0.34274E+01_JPRB, 0.35987E+01_JPRB, 0.37889E+01_JPRB, 0.39620E+01_JPRB, 0.41743E+01_JPRB ,&
&  0.42046E+01_JPRB, 0.42556E+01_JPRB, 0.42378E+01_JPRB, 0.41356E+01_JPRB, 0.41738E+01_JPRB, 0.42478E+01_JPRB, 0.43532E+01_JPRB ,&
&  0.45943E+01_JPRB, 0.50114E+01_JPRB, 0.54211E+01_JPRB, 0.56246E+01_JPRB, 0.55092E+01_JPRB, 0.57373E+01_JPRB, 0.55828E+01_JPRB ,&
&  0.56565E+01_JPRB, 0.52221E+01_JPRB, 0.53025E+01_JPRB, 0.46508E+01_JPRB, 0.43453E+01_JPRB, 0.40880E+01_JPRB, 0.40001E+01_JPRB ,&
&  0.38551E+01_JPRB, 0.37380E+01_JPRB, 0.35257E+01_JPRB, 0.36628E+01_JPRB, 0.42825E+01_JPRB, 0.46454E+01_JPRB, 0.47729E+01_JPRB ,&
&  0.39764E+01_JPRB, 0.30494E+01_JPRB, 0.23027E+01_JPRB, 0.23769E+01_JPRB, 0.21449E+01_JPRB, 0.24340E+01_JPRB, 0.31920E+01_JPRB ,&
&  0.40694E+01_JPRB, 0.33934E+01_JPRB, 0.33400E+01_JPRB, 0.27851E+01_JPRB, 0.17137E+01_JPRB, 0.11948E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.95235E+00_JPRB, 0.10010E+01_JPRB, 0.10259E+01_JPRB, 0.10202E+01_JPRB, 0.10852E+01_JPRB, 0.11335E+01_JPRB, 0.12188E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.13019E+01_JPRB, 0.13681E+01_JPRB, 0.14331E+01_JPRB, 0.15501E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.16308E+01_JPRB, 0.17285E+01_JPRB, 0.18250E+01_JPRB, 0.18115E+01_JPRB, 0.18061E+01_JPRB, 0.18051E+01_JPRB, 0.18283E+01_JPRB ,&
&  0.19314E+01_JPRB, 0.20268E+01_JPRB, 0.21307E+01_JPRB, 0.22134E+01_JPRB, 0.22773E+01_JPRB, 0.23962E+01_JPRB, 0.25473E+01_JPRB ,&
&  0.26501E+01_JPRB, 0.27362E+01_JPRB, 0.28303E+01_JPRB, 0.29661E+01_JPRB, 0.30912E+01_JPRB, 0.32713E+01_JPRB, 0.33599E+01_JPRB ,&
&  0.26382E+01_JPRB, 0.17844E+01_JPRB, 0.20042E+01_JPRB, 0.21710E+01_JPRB, 0.22247E+01_JPRB, 0.24883E+01_JPRB, 0.26414E+01_JPRB ,&
&  0.25932E+01_JPRB, 0.25762E+01_JPRB, 0.25977E+01_JPRB, 0.25794E+01_JPRB, 0.25409E+01_JPRB, 0.27325E+01_JPRB, 0.29205E+01_JPRB ,&
&  0.30937E+01_JPRB, 0.32415E+01_JPRB, 0.33779E+01_JPRB, 0.35188E+01_JPRB, 0.36356E+01_JPRB, 0.37403E+01_JPRB, 0.38433E+01_JPRB ,&
&  0.39158E+01_JPRB, 0.38684E+01_JPRB, 0.38517E+01_JPRB, 0.38816E+01_JPRB, 0.38390E+01_JPRB, 0.37498E+01_JPRB, 0.36567E+01_JPRB ,&
&  0.37504E+01_JPRB, 0.39685E+01_JPRB, 0.43094E+01_JPRB, 0.45733E+01_JPRB, 0.47251E+01_JPRB, 0.45893E+01_JPRB, 0.44783E+01_JPRB ,&
&  0.41743E+01_JPRB, 0.37997E+01_JPRB, 0.39912E+01_JPRB, 0.41690E+01_JPRB, 0.33077E+01_JPRB, 0.31650E+01_JPRB, 0.29444E+01_JPRB ,&
&  0.29395E+01_JPRB, 0.26180E+01_JPRB, 0.23898E+01_JPRB, 0.25015E+01_JPRB, 0.31640E+01_JPRB, 0.33132E+01_JPRB, 0.34601E+01_JPRB ,&
&  0.29856E+01_JPRB, 0.21376E+01_JPRB, 0.19783E+01_JPRB, 0.22751E+01_JPRB, 0.23213E+01_JPRB, 0.30980E+01_JPRB, 0.28724E+01_JPRB ,&
&  0.24537E+01_JPRB, 0.20012E+01_JPRB, 0.21259E+01_JPRB, 0.20477E+01_JPRB, 0.14090E+01_JPRB, 0.10416E+01_JPRB, 0.92404E+00_JPRB ,&
&  0.87144E+00_JPRB, 0.85523E+00_JPRB, 0.89895E+00_JPRB, 0.88217E+00_JPRB, 0.93977E+00_JPRB, 0.98441E+00_JPRB, 0.10932E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.12510E+01_JPRB, 0.13105E+01_JPRB, 0.13987E+01_JPRB, 0.14996E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.11496E+01_JPRB, 0.12037E+01_JPRB, 0.12046E+01_JPRB, 0.13133E+01_JPRB, 0.14426E+01_JPRB, 0.15336E+01_JPRB, 0.16159E+01_JPRB ,&
&  0.17355E+01_JPRB, 0.18855E+01_JPRB, 0.19986E+01_JPRB, 0.20703E+01_JPRB, 0.21435E+01_JPRB, 0.22514E+01_JPRB, 0.22819E+01_JPRB ,&
&  0.22593E+01_JPRB, 0.22373E+01_JPRB, 0.22335E+01_JPRB, 0.22293E+01_JPRB, 0.22487E+01_JPRB, 0.22750E+01_JPRB, 0.24703E+01_JPRB ,&
&  0.25300E+01_JPRB, 0.24289E+01_JPRB, 0.22971E+01_JPRB, 0.18650E+01_JPRB, 0.14598E+01_JPRB, 0.18566E+01_JPRB, 0.19021E+01_JPRB ,&
&  0.20996E+01_JPRB, 0.21703E+01_JPRB, 0.21533E+01_JPRB, 0.22716E+01_JPRB, 0.23907E+01_JPRB, 0.25363E+01_JPRB, 0.27092E+01_JPRB ,&
&  0.28396E+01_JPRB, 0.29786E+01_JPRB, 0.31119E+01_JPRB, 0.32186E+01_JPRB, 0.33102E+01_JPRB, 0.34082E+01_JPRB, 0.34199E+01_JPRB ,&
&  0.34143E+01_JPRB, 0.33508E+01_JPRB, 0.33161E+01_JPRB, 0.34155E+01_JPRB, 0.33866E+01_JPRB, 0.33340E+01_JPRB, 0.31696E+01_JPRB ,&
&  0.31332E+01_JPRB, 0.31947E+01_JPRB, 0.33916E+01_JPRB, 0.35160E+01_JPRB, 0.34688E+01_JPRB, 0.32721E+01_JPRB, 0.31031E+01_JPRB ,&
&  0.29207E+01_JPRB, 0.27293E+01_JPRB, 0.28360E+01_JPRB, 0.31299E+01_JPRB, 0.23713E+01_JPRB, 0.24816E+01_JPRB, 0.25478E+01_JPRB ,&
&  0.22311E+01_JPRB, 0.19890E+01_JPRB, 0.18410E+01_JPRB, 0.17427E+01_JPRB, 0.20908E+01_JPRB, 0.23953E+01_JPRB, 0.27969E+01_JPRB ,&
&  0.23629E+01_JPRB, 0.17301E+01_JPRB, 0.16995E+01_JPRB, 0.20873E+01_JPRB, 0.23862E+01_JPRB, 0.30254E+01_JPRB, 0.23568E+01_JPRB ,&
&  0.17029E+01_JPRB, 0.17084E+01_JPRB, 0.18743E+01_JPRB, 0.21358E+01_JPRB, 0.16330E+01_JPRB, 0.98614E+00_JPRB, 0.84042E+00_JPRB ,&
&  0.80542E+00_JPRB, 0.80281E+00_JPRB, 0.80771E+00_JPRB, 0.79933E+00_JPRB, 0.83553E+00_JPRB, 0.87242E+00_JPRB, 0.87724E+00_JPRB ,&
&  0.88496E+00_JPRB, 0.88741E+00_JPRB, 0.91966E+00_JPRB, 0.98129E+00_JPRB, 0.10781E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.76299E+00_JPRB, 0.76419E+00_JPRB, 0.74701E+00_JPRB, 0.77293E+00_JPRB, 0.82646E+00_JPRB, 0.93886E+00_JPRB, 0.10491E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.13183E+01_JPRB, 0.15174E+01_JPRB, 0.16915E+01_JPRB, 0.17502E+01_JPRB, 0.17236E+01_JPRB, 0.15709E+01_JPRB ,&
&  0.13671E+01_JPRB, 0.11545E+01_JPRB, 0.10276E+01_JPRB, 0.98160E+00_JPRB, 0.10314E+01_JPRB, 0.10566E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.12641E+01_JPRB, 0.14804E+01_JPRB, 0.16993E+01_JPRB, 0.21990E+01_JPRB, 0.24631E+01_JPRB, 0.18215E+01_JPRB, 0.17317E+01_JPRB ,&
&  0.20596E+01_JPRB, 0.21245E+01_JPRB, 0.17118E+01_JPRB, 0.17093E+01_JPRB, 0.18768E+01_JPRB, 0.20143E+01_JPRB, 0.21979E+01_JPRB ,&
&  0.22961E+01_JPRB, 0.23954E+01_JPRB, 0.24700E+01_JPRB, 0.25061E+01_JPRB, 0.25242E+01_JPRB, 0.24933E+01_JPRB, 0.24383E+01_JPRB ,&
&  0.23684E+01_JPRB, 0.22839E+01_JPRB, 0.23197E+01_JPRB, 0.24268E+01_JPRB, 0.25577E+01_JPRB, 0.26021E+01_JPRB, 0.25366E+01_JPRB ,&
&  0.24762E+01_JPRB, 0.24655E+01_JPRB, 0.26507E+01_JPRB, 0.26413E+01_JPRB, 0.24280E+01_JPRB, 0.22873E+01_JPRB, 0.20526E+01_JPRB ,&
&  0.18269E+01_JPRB, 0.15450E+01_JPRB, 0.13185E+01_JPRB, 0.20490E+01_JPRB, 0.23503E+01_JPRB, 0.20911E+01_JPRB, 0.18402E+01_JPRB ,&
&  0.17206E+01_JPRB, 0.16040E+01_JPRB, 0.14947E+01_JPRB, 0.14723E+01_JPRB, 0.16379E+01_JPRB, 0.22923E+01_JPRB, 0.23556E+01_JPRB ,&
&  0.20056E+01_JPRB, 0.14153E+01_JPRB, 0.12886E+01_JPRB, 0.14734E+01_JPRB, 0.17064E+01_JPRB, 0.20692E+01_JPRB, 0.16441E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.15815E+01_JPRB, 0.17585E+01_JPRB, 0.21993E+01_JPRB, 0.23897E+01_JPRB, 0.81608E+00_JPRB, 0.76428E+00_JPRB ,&
&  0.74040E+00_JPRB, 0.72594E+00_JPRB, 0.73983E+00_JPRB, 0.72786E+00_JPRB, 0.73033E+00_JPRB, 0.71141E+00_JPRB, 0.68585E+00_JPRB ,&
&  0.67767E+00_JPRB, 0.68751E+00_JPRB, 0.70297E+00_JPRB, 0.71427E+00_JPRB, 0.74373E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.69837E+00_JPRB, 0.69604E+00_JPRB, 0.69839E+00_JPRB, 0.70645E+00_JPRB, 0.71153E+00_JPRB, 0.71610E+00_JPRB, 0.74069E+00_JPRB ,&
&  0.78207E+00_JPRB, 0.84655E+00_JPRB, 0.93570E+00_JPRB, 0.99365E+00_JPRB, 0.97477E+00_JPRB, 0.90486E+00_JPRB, 0.86888E+00_JPRB ,&
&  0.85493E+00_JPRB, 0.84495E+00_JPRB, 0.87547E+00_JPRB, 0.92861E+00_JPRB, 0.99594E+00_JPRB, 0.10158E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.11122E+01_JPRB, 0.12140E+01_JPRB, 0.13078E+01_JPRB, 0.19424E+01_JPRB, 0.18595E+01_JPRB, 0.15447E+01_JPRB ,&
&  0.16018E+01_JPRB, 0.16086E+01_JPRB, 0.15361E+01_JPRB, 0.13559E+01_JPRB, 0.13296E+01_JPRB, 0.12946E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.13116E+01_JPRB, 0.13951E+01_JPRB, 0.14604E+01_JPRB, 0.15354E+01_JPRB, 0.15826E+01_JPRB, 0.15861E+01_JPRB, 0.16190E+01_JPRB ,&
&  0.16375E+01_JPRB, 0.16574E+01_JPRB, 0.16797E+01_JPRB, 0.15239E+01_JPRB, 0.16264E+01_JPRB, 0.17160E+01_JPRB, 0.17496E+01_JPRB ,&
&  0.17282E+01_JPRB, 0.17150E+01_JPRB, 0.18115E+01_JPRB, 0.17198E+01_JPRB, 0.16096E+01_JPRB, 0.14516E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.11941E+01_JPRB, 0.92778E+00_JPRB, 0.72165E+00_JPRB, 0.10693E+01_JPRB, 0.13515E+01_JPRB, 0.12430E+01_JPRB, 0.13549E+01_JPRB ,&
&  0.13279E+01_JPRB, 0.12821E+01_JPRB, 0.12329E+01_JPRB, 0.12396E+01_JPRB, 0.12540E+01_JPRB, 0.15926E+01_JPRB, 0.18060E+01_JPRB ,&
&  0.15308E+01_JPRB, 0.11486E+01_JPRB, 0.10064E+01_JPRB, 0.10363E+01_JPRB, 0.11562E+01_JPRB, 0.13375E+01_JPRB, 0.15295E+01_JPRB ,&
&  0.14413E+01_JPRB, 0.11913E+01_JPRB, 0.13854E+01_JPRB, 0.16920E+01_JPRB, 0.10219E+01_JPRB, 0.63722E+00_JPRB, 0.60501E+00_JPRB ,&
&  0.59788E+00_JPRB, 0.60450E+00_JPRB, 0.62427E+00_JPRB, 0.63589E+00_JPRB, 0.64212E+00_JPRB, 0.64344E+00_JPRB, 0.65442E+00_JPRB ,&
&  0.66186E+00_JPRB, 0.67234E+00_JPRB, 0.67666E+00_JPRB, 0.68479E+00_JPRB, 0.69419E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.87814E+00_JPRB, 0.89401E+00_JPRB, 0.91738E+00_JPRB, 0.94262E+00_JPRB, 0.95942E+00_JPRB, 0.98488E+00_JPRB, 0.10129E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10975E+01_JPRB, 0.11288E+01_JPRB, 0.11409E+01_JPRB, 0.11555E+01_JPRB, 0.11724E+01_JPRB, 0.11768E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.12129E+01_JPRB, 0.12752E+01_JPRB, 0.13043E+01_JPRB, 0.13436E+01_JPRB, 0.13948E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.13926E+01_JPRB, 0.13817E+01_JPRB, 0.13957E+01_JPRB, 0.13926E+01_JPRB, 0.14781E+01_JPRB, 0.14935E+01_JPRB, 0.14295E+01_JPRB ,&
&  0.13085E+01_JPRB, 0.12465E+01_JPRB, 0.14081E+01_JPRB, 0.11602E+01_JPRB, 0.11407E+01_JPRB, 0.11362E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.10278E+01_JPRB, 0.10750E+01_JPRB, 0.11336E+01_JPRB, 0.11932E+01_JPRB, 0.12706E+01_JPRB, 0.12931E+01_JPRB, 0.12958E+01_JPRB ,&
&  0.13077E+01_JPRB, 0.13375E+01_JPRB, 0.13608E+01_JPRB, 0.13099E+01_JPRB, 0.12728E+01_JPRB, 0.14069E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.14217E+01_JPRB, 0.14036E+01_JPRB, 0.11712E+01_JPRB, 0.10860E+01_JPRB, 0.11045E+01_JPRB, 0.99011E+00_JPRB, 0.94440E+00_JPRB ,&
&  0.10031E+01_JPRB, 0.89989E+00_JPRB, 0.62058E+00_JPRB, 0.82599E+00_JPRB, 0.10091E+01_JPRB, 0.10975E+01_JPRB, 0.11383E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.99621E+00_JPRB, 0.97239E+00_JPRB, 0.10385E+01_JPRB, 0.11032E+01_JPRB, 0.11002E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.95974E+00_JPRB, 0.89179E+00_JPRB, 0.10112E+01_JPRB, 0.13555E+01_JPRB, 0.20056E+01_JPRB, 0.18192E+01_JPRB ,&
&  0.13625E+01_JPRB, 0.11862E+01_JPRB, 0.18059E+01_JPRB, 0.24601E+01_JPRB, 0.99907E+00_JPRB, 0.69263E+00_JPRB, 0.57486E+00_JPRB ,&
&  0.60302E+00_JPRB, 0.61638E+00_JPRB, 0.62489E+00_JPRB, 0.64832E+00_JPRB, 0.68492E+00_JPRB, 0.70150E+00_JPRB, 0.70664E+00_JPRB ,&
&  0.72797E+00_JPRB, 0.76181E+00_JPRB, 0.78743E+00_JPRB, 0.82035E+00_JPRB, 0.84989E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.10289E+01_JPRB, 0.10537E+01_JPRB, 0.10722E+01_JPRB, 0.10922E+01_JPRB, 0.11181E+01_JPRB, 0.11444E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.11874E+01_JPRB, 0.12079E+01_JPRB, 0.12171E+01_JPRB, 0.12372E+01_JPRB, 0.12441E+01_JPRB, 0.13012E+01_JPRB, 0.13239E+01_JPRB ,&
&  0.13730E+01_JPRB, 0.14019E+01_JPRB, 0.14123E+01_JPRB, 0.14154E+01_JPRB, 0.14293E+01_JPRB, 0.14245E+01_JPRB, 0.14182E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.14916E+01_JPRB, 0.15456E+01_JPRB, 0.15960E+01_JPRB, 0.16220E+01_JPRB, 0.17140E+01_JPRB, 0.18368E+01_JPRB ,&
&  0.13963E+01_JPRB, 0.13815E+01_JPRB, 0.13814E+01_JPRB, 0.11990E+01_JPRB, 0.12080E+01_JPRB, 0.12192E+01_JPRB, 0.11730E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.11757E+01_JPRB, 0.12008E+01_JPRB, 0.12192E+01_JPRB, 0.12419E+01_JPRB, 0.12700E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.13214E+01_JPRB, 0.13532E+01_JPRB, 0.14027E+01_JPRB, 0.13431E+01_JPRB, 0.14864E+01_JPRB, 0.15005E+01_JPRB ,&
&  0.14796E+01_JPRB, 0.13684E+01_JPRB, 0.10531E+01_JPRB, 0.99332E+00_JPRB, 0.10111E+01_JPRB, 0.96488E+00_JPRB, 0.91642E+00_JPRB ,&
&  0.98256E+00_JPRB, 0.98484E+00_JPRB, 0.81834E+00_JPRB, 0.94097E+00_JPRB, 0.10137E+01_JPRB, 0.10461E+01_JPRB, 0.99663E+00_JPRB ,&
&  0.93194E+00_JPRB, 0.83221E+00_JPRB, 0.80551E+00_JPRB, 0.83332E+00_JPRB, 0.85265E+00_JPRB, 0.85261E+00_JPRB, 0.89252E+00_JPRB ,&
&  0.90081E+00_JPRB, 0.88205E+00_JPRB, 0.90556E+00_JPRB, 0.93647E+00_JPRB, 0.14142E+01_JPRB, 0.17043E+01_JPRB, 0.23764E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.10947E+01_JPRB, 0.99232E+00_JPRB, 0.13561E+01_JPRB, 0.87924E+00_JPRB, 0.60684E+00_JPRB, 0.60688E+00_JPRB ,&
&  0.67123E+00_JPRB, 0.70087E+00_JPRB, 0.70797E+00_JPRB, 0.72676E+00_JPRB, 0.77442E+00_JPRB, 0.80206E+00_JPRB, 0.82716E+00_JPRB ,&
&  0.83606E+00_JPRB, 0.87681E+00_JPRB, 0.93197E+00_JPRB, 0.97434E+00_JPRB, 0.99444E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.10921E+01_JPRB, 0.11230E+01_JPRB, 0.11499E+01_JPRB, 0.11748E+01_JPRB, 0.11986E+01_JPRB, 0.12247E+01_JPRB, 0.12481E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.12837E+01_JPRB, 0.12918E+01_JPRB, 0.13068E+01_JPRB, 0.13203E+01_JPRB, 0.13636E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.14250E+01_JPRB, 0.14425E+01_JPRB, 0.14404E+01_JPRB, 0.14645E+01_JPRB, 0.14517E+01_JPRB, 0.14271E+01_JPRB, 0.14227E+01_JPRB ,&
&  0.14313E+01_JPRB, 0.14303E+01_JPRB, 0.14305E+01_JPRB, 0.14354E+01_JPRB, 0.14401E+01_JPRB, 0.14104E+01_JPRB, 0.15034E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.15355E+01_JPRB, 0.15740E+01_JPRB, 0.14074E+01_JPRB, 0.13351E+01_JPRB, 0.12314E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.12189E+01_JPRB, 0.12231E+01_JPRB, 0.12441E+01_JPRB, 0.12729E+01_JPRB, 0.13075E+01_JPRB, 0.13237E+01_JPRB ,&
&  0.13535E+01_JPRB, 0.13703E+01_JPRB, 0.13869E+01_JPRB, 0.14061E+01_JPRB, 0.14529E+01_JPRB, 0.14568E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.14197E+01_JPRB, 0.14101E+01_JPRB, 0.11271E+01_JPRB, 0.10859E+01_JPRB, 0.11435E+01_JPRB, 0.10982E+01_JPRB, 0.99280E+00_JPRB ,&
&  0.99661E+00_JPRB, 0.99970E+00_JPRB, 0.10188E+01_JPRB, 0.10030E+01_JPRB, 0.99580E+00_JPRB, 0.96399E+00_JPRB, 0.90547E+00_JPRB ,&
&  0.78336E+00_JPRB, 0.68429E+00_JPRB, 0.66142E+00_JPRB, 0.70218E+00_JPRB, 0.72311E+00_JPRB, 0.73121E+00_JPRB, 0.71304E+00_JPRB ,&
&  0.71937E+00_JPRB, 0.74567E+00_JPRB, 0.80950E+00_JPRB, 0.95153E+00_JPRB, 0.11487E+01_JPRB, 0.21344E+01_JPRB, 0.23386E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.99372E+00_JPRB, 0.68113E+00_JPRB, 0.81440E+00_JPRB, 0.81914E+00_JPRB, 0.70966E+00_JPRB, 0.72912E+00_JPRB ,&
&  0.83517E+00_JPRB, 0.89247E+00_JPRB, 0.91371E+00_JPRB, 0.85241E+00_JPRB, 0.87115E+00_JPRB, 0.85885E+00_JPRB, 0.86582E+00_JPRB ,&
&  0.89034E+00_JPRB, 0.92812E+00_JPRB, 0.97471E+00_JPRB, 0.10189E+01_JPRB, 0.10568E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.11136E+01_JPRB, 0.11441E+01_JPRB, 0.11751E+01_JPRB, 0.12097E+01_JPRB, 0.12447E+01_JPRB, 0.12705E+01_JPRB, 0.12978E+01_JPRB ,&
&  0.13258E+01_JPRB, 0.13471E+01_JPRB, 0.13545E+01_JPRB, 0.13615E+01_JPRB, 0.13745E+01_JPRB, 0.14155E+01_JPRB, 0.14501E+01_JPRB ,&
&  0.14416E+01_JPRB, 0.14610E+01_JPRB, 0.14729E+01_JPRB, 0.14838E+01_JPRB, 0.14727E+01_JPRB, 0.14386E+01_JPRB, 0.14497E+01_JPRB ,&
&  0.14449E+01_JPRB, 0.14493E+01_JPRB, 0.14526E+01_JPRB, 0.14814E+01_JPRB, 0.15244E+01_JPRB, 0.16337E+01_JPRB, 0.14021E+01_JPRB ,&
&  0.15285E+01_JPRB, 0.16421E+01_JPRB, 0.16274E+01_JPRB, 0.14983E+01_JPRB, 0.13541E+01_JPRB, 0.13156E+01_JPRB, 0.13047E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.12794E+01_JPRB, 0.12712E+01_JPRB, 0.12670E+01_JPRB, 0.12755E+01_JPRB, 0.13068E+01_JPRB, 0.13192E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13772E+01_JPRB, 0.14173E+01_JPRB, 0.14638E+01_JPRB, 0.15222E+01_JPRB, 0.15702E+01_JPRB, 0.15958E+01_JPRB ,&
&  0.16120E+01_JPRB, 0.16228E+01_JPRB, 0.14784E+01_JPRB, 0.14632E+01_JPRB, 0.15131E+01_JPRB, 0.13859E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.99452E+00_JPRB, 0.92582E+00_JPRB, 0.10570E+01_JPRB, 0.99768E+00_JPRB, 0.91505E+00_JPRB, 0.80996E+00_JPRB ,&
&  0.68593E+00_JPRB, 0.62040E+00_JPRB, 0.62340E+00_JPRB, 0.66765E+00_JPRB, 0.66516E+00_JPRB, 0.67642E+00_JPRB, 0.70600E+00_JPRB ,&
&  0.75289E+00_JPRB, 0.80482E+00_JPRB, 0.88392E+00_JPRB, 0.10283E+01_JPRB, 0.12169E+01_JPRB, 0.12259E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.10256E+01_JPRB, 0.85087E+00_JPRB, 0.88557E+00_JPRB, 0.90604E+00_JPRB, 0.93108E+00_JPRB, 0.90514E+00_JPRB, 0.10032E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.12852E+01_JPRB, 0.12943E+01_JPRB, 0.10789E+01_JPRB, 0.97051E+00_JPRB, 0.92547E+00_JPRB, 0.92276E+00_JPRB ,&
&  0.93712E+00_JPRB, 0.96104E+00_JPRB, 0.99958E+00_JPRB, 0.10437E+01_JPRB, 0.10817E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.11093E+01_JPRB, 0.11414E+01_JPRB, 0.11742E+01_JPRB, 0.12062E+01_JPRB, 0.12313E+01_JPRB, 0.12511E+01_JPRB, 0.12656E+01_JPRB ,&
&  0.12896E+01_JPRB, 0.13140E+01_JPRB, 0.13315E+01_JPRB, 0.13405E+01_JPRB, 0.13499E+01_JPRB, 0.13755E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.14733E+01_JPRB, 0.15186E+01_JPRB, 0.15096E+01_JPRB, 0.14993E+01_JPRB, 0.15108E+01_JPRB, 0.15496E+01_JPRB ,&
&  0.15647E+01_JPRB, 0.15611E+01_JPRB, 0.15604E+01_JPRB, 0.15999E+01_JPRB, 0.16422E+01_JPRB, 0.17146E+01_JPRB, 0.15126E+01_JPRB ,&
&  0.16575E+01_JPRB, 0.17527E+01_JPRB, 0.16967E+01_JPRB, 0.16119E+01_JPRB, 0.14695E+01_JPRB, 0.14217E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.13835E+01_JPRB, 0.13640E+01_JPRB, 0.13320E+01_JPRB, 0.13111E+01_JPRB, 0.13023E+01_JPRB, 0.12945E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.13351E+01_JPRB, 0.13623E+01_JPRB, 0.14170E+01_JPRB, 0.14816E+01_JPRB, 0.15763E+01_JPRB, 0.16677E+01_JPRB, 0.17239E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.18595E+01_JPRB, 0.17497E+01_JPRB, 0.17547E+01_JPRB, 0.18397E+01_JPRB, 0.16677E+01_JPRB, 0.13505E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.98005E+00_JPRB, 0.10065E+01_JPRB, 0.10878E+01_JPRB, 0.98274E+00_JPRB, 0.87579E+00_JPRB, 0.80972E+00_JPRB ,&
&  0.71596E+00_JPRB, 0.64013E+00_JPRB, 0.65555E+00_JPRB, 0.65303E+00_JPRB, 0.64680E+00_JPRB, 0.68163E+00_JPRB, 0.72851E+00_JPRB ,&
&  0.78736E+00_JPRB, 0.88460E+00_JPRB, 0.98880E+00_JPRB, 0.11050E+01_JPRB, 0.11859E+01_JPRB, 0.11461E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10742E+01_JPRB, 0.10763E+01_JPRB, 0.85623E+00_JPRB, 0.11190E+01_JPRB, 0.11881E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.16888E+01_JPRB, 0.20541E+01_JPRB, 0.16093E+01_JPRB, 0.12868E+01_JPRB, 0.97476E+00_JPRB, 0.92639E+00_JPRB ,&
&  0.93829E+00_JPRB, 0.95805E+00_JPRB, 0.99178E+00_JPRB, 0.10341E+01_JPRB, 0.10729E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.10874E+01_JPRB, 0.11246E+01_JPRB, 0.11646E+01_JPRB, 0.11936E+01_JPRB, 0.12124E+01_JPRB, 0.12037E+01_JPRB, 0.11949E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.12130E+01_JPRB, 0.12338E+01_JPRB, 0.12559E+01_JPRB, 0.12712E+01_JPRB, 0.12870E+01_JPRB, 0.13138E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.13885E+01_JPRB, 0.14417E+01_JPRB, 0.14862E+01_JPRB, 0.15357E+01_JPRB, 0.15787E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.16803E+01_JPRB, 0.17437E+01_JPRB, 0.17615E+01_JPRB, 0.18005E+01_JPRB, 0.18219E+01_JPRB, 0.18838E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.17712E+01_JPRB, 0.17505E+01_JPRB, 0.16581E+01_JPRB, 0.15211E+01_JPRB, 0.14537E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.13450E+01_JPRB, 0.12710E+01_JPRB, 0.13050E+01_JPRB, 0.13294E+01_JPRB, 0.12725E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.13291E+01_JPRB, 0.13668E+01_JPRB, 0.14135E+01_JPRB, 0.14942E+01_JPRB, 0.16057E+01_JPRB, 0.17053E+01_JPRB, 0.18026E+01_JPRB ,&
&  0.19043E+01_JPRB, 0.20132E+01_JPRB, 0.21010E+01_JPRB, 0.19036E+01_JPRB, 0.18947E+01_JPRB, 0.17316E+01_JPRB, 0.13859E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.96667E+00_JPRB, 0.10139E+01_JPRB, 0.10946E+01_JPRB, 0.98550E+00_JPRB, 0.84688E+00_JPRB, 0.78704E+00_JPRB ,&
&  0.71786E+00_JPRB, 0.67953E+00_JPRB, 0.67473E+00_JPRB, 0.65233E+00_JPRB, 0.64684E+00_JPRB, 0.68599E+00_JPRB, 0.73687E+00_JPRB ,&
&  0.80120E+00_JPRB, 0.89088E+00_JPRB, 0.10004E+01_JPRB, 0.11188E+01_JPRB, 0.12395E+01_JPRB, 0.12802E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.10982E+01_JPRB, 0.11407E+01_JPRB, 0.12401E+01_JPRB, 0.12784E+01_JPRB, 0.15054E+01_JPRB, 0.17426E+01_JPRB ,&
&  0.19649E+01_JPRB, 0.18229E+01_JPRB, 0.21369E+01_JPRB, 0.29468E+01_JPRB, 0.27082E+01_JPRB, 0.22147E+01_JPRB, 0.89240E+00_JPRB ,&
&  0.89205E+00_JPRB, 0.89569E+00_JPRB, 0.93606E+00_JPRB, 0.98830E+00_JPRB, 0.10488E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.89989E+00_JPRB, 0.95727E+00_JPRB, 0.10154E+01_JPRB, 0.10907E+01_JPRB, 0.11552E+01_JPRB, 0.11563E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.11180E+01_JPRB, 0.11310E+01_JPRB, 0.11528E+01_JPRB, 0.11710E+01_JPRB, 0.11842E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.12646E+01_JPRB, 0.13043E+01_JPRB, 0.13493E+01_JPRB, 0.13915E+01_JPRB, 0.14525E+01_JPRB, 0.15240E+01_JPRB ,&
&  0.16009E+01_JPRB, 0.16841E+01_JPRB, 0.17520E+01_JPRB, 0.18195E+01_JPRB, 0.18794E+01_JPRB, 0.18743E+01_JPRB, 0.13772E+01_JPRB ,&
&  0.16846E+01_JPRB, 0.17261E+01_JPRB, 0.17547E+01_JPRB, 0.16683E+01_JPRB, 0.15211E+01_JPRB, 0.14317E+01_JPRB, 0.13963E+01_JPRB ,&
&  0.13618E+01_JPRB, 0.13105E+01_JPRB, 0.12483E+01_JPRB, 0.12041E+01_JPRB, 0.12820E+01_JPRB, 0.12296E+01_JPRB, 0.12703E+01_JPRB ,&
&  0.13137E+01_JPRB, 0.13568E+01_JPRB, 0.13986E+01_JPRB, 0.14680E+01_JPRB, 0.15817E+01_JPRB, 0.16940E+01_JPRB, 0.18208E+01_JPRB ,&
&  0.19390E+01_JPRB, 0.20394E+01_JPRB, 0.20960E+01_JPRB, 0.18421E+01_JPRB, 0.17580E+01_JPRB, 0.16544E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.10286E+01_JPRB, 0.11041E+01_JPRB, 0.11116E+01_JPRB, 0.10055E+01_JPRB, 0.88318E+00_JPRB, 0.80371E+00_JPRB ,&
&  0.71517E+00_JPRB, 0.67539E+00_JPRB, 0.67098E+00_JPRB, 0.65786E+00_JPRB, 0.65922E+00_JPRB, 0.69787E+00_JPRB, 0.74857E+00_JPRB ,&
&  0.80419E+00_JPRB, 0.87619E+00_JPRB, 0.93942E+00_JPRB, 0.99836E+00_JPRB, 0.10697E+01_JPRB, 0.11882E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.10338E+01_JPRB, 0.10490E+01_JPRB, 0.10767E+01_JPRB, 0.11134E+01_JPRB, 0.11866E+01_JPRB, 0.12639E+01_JPRB ,&
&  0.13931E+01_JPRB, 0.14643E+01_JPRB, 0.12396E+01_JPRB, 0.10375E+01_JPRB, 0.13402E+01_JPRB, 0.14323E+01_JPRB, 0.77181E+00_JPRB ,&
&  0.74134E+00_JPRB, 0.76043E+00_JPRB, 0.77685E+00_JPRB, 0.81550E+00_JPRB, 0.85625E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.72211E+00_JPRB, 0.74102E+00_JPRB, 0.82021E+00_JPRB, 0.84599E+00_JPRB, 0.92497E+00_JPRB, 0.95989E+00_JPRB, 0.99827E+00_JPRB ,&
&  0.10097E+01_JPRB, 0.10161E+01_JPRB, 0.10208E+01_JPRB, 0.10296E+01_JPRB, 0.10403E+01_JPRB, 0.10567E+01_JPRB, 0.10714E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11170E+01_JPRB, 0.11483E+01_JPRB, 0.11820E+01_JPRB, 0.12266E+01_JPRB, 0.12804E+01_JPRB, 0.13521E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.15484E+01_JPRB, 0.16619E+01_JPRB, 0.17826E+01_JPRB, 0.19097E+01_JPRB, 0.20055E+01_JPRB, 0.20342E+01_JPRB ,&
&  0.14234E+01_JPRB, 0.17294E+01_JPRB, 0.17107E+01_JPRB, 0.16282E+01_JPRB, 0.15189E+01_JPRB, 0.14472E+01_JPRB, 0.14195E+01_JPRB ,&
&  0.13905E+01_JPRB, 0.13017E+01_JPRB, 0.12698E+01_JPRB, 0.12304E+01_JPRB, 0.12104E+01_JPRB, 0.11755E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.12429E+01_JPRB, 0.12970E+01_JPRB, 0.13592E+01_JPRB, 0.14174E+01_JPRB, 0.15207E+01_JPRB, 0.16554E+01_JPRB, 0.18001E+01_JPRB ,&
&  0.19502E+01_JPRB, 0.20901E+01_JPRB, 0.21824E+01_JPRB, 0.19371E+01_JPRB, 0.17209E+01_JPRB, 0.16437E+01_JPRB, 0.13092E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.10480E+01_JPRB, 0.11332E+01_JPRB, 0.11632E+01_JPRB, 0.10508E+01_JPRB, 0.94752E+00_JPRB, 0.86443E+00_JPRB ,&
&  0.77806E+00_JPRB, 0.71058E+00_JPRB, 0.69109E+00_JPRB, 0.68626E+00_JPRB, 0.69375E+00_JPRB, 0.72305E+00_JPRB, 0.74930E+00_JPRB ,&
&  0.78140E+00_JPRB, 0.82006E+00_JPRB, 0.85436E+00_JPRB, 0.89343E+00_JPRB, 0.91511E+00_JPRB, 0.93650E+00_JPRB, 0.94214E+00_JPRB ,&
&  0.95110E+00_JPRB, 0.98657E+00_JPRB, 0.98360E+00_JPRB, 0.99909E+00_JPRB, 0.99045E+00_JPRB, 0.10046E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.10514E+01_JPRB, 0.10729E+01_JPRB, 0.10650E+01_JPRB, 0.10295E+01_JPRB, 0.95865E+00_JPRB, 0.87806E+00_JPRB, 0.71935E+00_JPRB ,&
&  0.69473E+00_JPRB, 0.67550E+00_JPRB, 0.68537E+00_JPRB, 0.69342E+00_JPRB, 0.71259E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.62640E+00_JPRB, 0.64469E+00_JPRB, 0.66436E+00_JPRB, 0.68807E+00_JPRB, 0.72429E+00_JPRB, 0.76693E+00_JPRB, 0.82201E+00_JPRB ,&
&  0.87877E+00_JPRB, 0.89739E+00_JPRB, 0.90267E+00_JPRB, 0.90572E+00_JPRB, 0.91186E+00_JPRB, 0.92726E+00_JPRB, 0.94701E+00_JPRB ,&
&  0.96606E+00_JPRB, 0.98415E+00_JPRB, 0.10018E+01_JPRB, 0.10227E+01_JPRB, 0.10491E+01_JPRB, 0.10895E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.13139E+01_JPRB, 0.14579E+01_JPRB, 0.16220E+01_JPRB, 0.17982E+01_JPRB, 0.19399E+01_JPRB, 0.19894E+01_JPRB ,&
&  0.11673E+01_JPRB, 0.14805E+01_JPRB, 0.17617E+01_JPRB, 0.16711E+01_JPRB, 0.15567E+01_JPRB, 0.14957E+01_JPRB, 0.14734E+01_JPRB ,&
&  0.14045E+01_JPRB, 0.13049E+01_JPRB, 0.12661E+01_JPRB, 0.12957E+01_JPRB, 0.11772E+01_JPRB, 0.11406E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.11916E+01_JPRB, 0.12408E+01_JPRB, 0.13054E+01_JPRB, 0.14007E+01_JPRB, 0.15457E+01_JPRB, 0.17099E+01_JPRB ,&
&  0.18940E+01_JPRB, 0.20880E+01_JPRB, 0.22747E+01_JPRB, 0.20283E+01_JPRB, 0.19887E+01_JPRB, 0.20248E+01_JPRB, 0.18646E+01_JPRB ,&
&  0.13912E+01_JPRB, 0.11816E+01_JPRB, 0.12255E+01_JPRB, 0.12447E+01_JPRB, 0.11125E+01_JPRB, 0.10267E+01_JPRB, 0.95062E+00_JPRB ,&
&  0.88135E+00_JPRB, 0.81319E+00_JPRB, 0.74351E+00_JPRB, 0.72737E+00_JPRB, 0.73355E+00_JPRB, 0.75493E+00_JPRB, 0.77937E+00_JPRB ,&
&  0.79404E+00_JPRB, 0.80043E+00_JPRB, 0.81415E+00_JPRB, 0.83289E+00_JPRB, 0.86924E+00_JPRB, 0.88846E+00_JPRB, 0.89500E+00_JPRB ,&
&  0.92198E+00_JPRB, 0.93564E+00_JPRB, 0.95515E+00_JPRB, 0.97814E+00_JPRB, 0.99984E+00_JPRB, 0.10125E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.99752E+00_JPRB, 0.10154E+01_JPRB, 0.10398E+01_JPRB, 0.10359E+01_JPRB, 0.99582E+00_JPRB, 0.90085E+00_JPRB, 0.81985E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.71508E+00_JPRB, 0.68249E+00_JPRB, 0.65609E+00_JPRB, 0.63456E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.61380E+00_JPRB, 0.60879E+00_JPRB, 0.61491E+00_JPRB, 0.62111E+00_JPRB, 0.63466E+00_JPRB, 0.66251E+00_JPRB, 0.69499E+00_JPRB ,&
&  0.72205E+00_JPRB, 0.76837E+00_JPRB, 0.80129E+00_JPRB, 0.80325E+00_JPRB, 0.79818E+00_JPRB, 0.80541E+00_JPRB, 0.82151E+00_JPRB ,&
&  0.84438E+00_JPRB, 0.86304E+00_JPRB, 0.87516E+00_JPRB, 0.88624E+00_JPRB, 0.90275E+00_JPRB, 0.92130E+00_JPRB, 0.94676E+00_JPRB ,&
&  0.98685E+00_JPRB, 0.10549E+01_JPRB, 0.11597E+01_JPRB, 0.13203E+01_JPRB, 0.15386E+01_JPRB, 0.17849E+01_JPRB, 0.19359E+01_JPRB ,&
&  0.14195E+01_JPRB, 0.76523E+00_JPRB, 0.15263E+01_JPRB, 0.18108E+01_JPRB, 0.16542E+01_JPRB, 0.16039E+01_JPRB, 0.15283E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.14171E+01_JPRB, 0.13489E+01_JPRB, 0.13132E+01_JPRB, 0.11691E+01_JPRB, 0.11133E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.10943E+01_JPRB, 0.11231E+01_JPRB, 0.11695E+01_JPRB, 0.12450E+01_JPRB, 0.13650E+01_JPRB, 0.15494E+01_JPRB ,&
&  0.17504E+01_JPRB, 0.19810E+01_JPRB, 0.22980E+01_JPRB, 0.21675E+01_JPRB, 0.22357E+01_JPRB, 0.20754E+01_JPRB, 0.18065E+01_JPRB ,&
&  0.15278E+01_JPRB, 0.13309E+01_JPRB, 0.13417E+01_JPRB, 0.13667E+01_JPRB, 0.12064E+01_JPRB, 0.10517E+01_JPRB, 0.10532E+01_JPRB ,&
&  0.95584E+00_JPRB, 0.91151E+00_JPRB, 0.83919E+00_JPRB, 0.78825E+00_JPRB, 0.77619E+00_JPRB, 0.79278E+00_JPRB, 0.81216E+00_JPRB ,&
&  0.81792E+00_JPRB, 0.81812E+00_JPRB, 0.82398E+00_JPRB, 0.83674E+00_JPRB, 0.86002E+00_JPRB, 0.87794E+00_JPRB, 0.88749E+00_JPRB ,&
&  0.91617E+00_JPRB, 0.94737E+00_JPRB, 0.97643E+00_JPRB, 0.10019E+01_JPRB, 0.10186E+01_JPRB, 0.10004E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.10139E+01_JPRB, 0.10141E+01_JPRB, 0.10271E+01_JPRB, 0.10121E+01_JPRB, 0.95987E+00_JPRB, 0.93873E+00_JPRB ,&
&  0.87128E+00_JPRB, 0.79954E+00_JPRB, 0.71327E+00_JPRB, 0.65997E+00_JPRB, 0.63995E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.69804E+00_JPRB, 0.66784E+00_JPRB, 0.63116E+00_JPRB, 0.62219E+00_JPRB, 0.62160E+00_JPRB, 0.61954E+00_JPRB, 0.63596E+00_JPRB ,&
&  0.65171E+00_JPRB, 0.67537E+00_JPRB, 0.68110E+00_JPRB, 0.70341E+00_JPRB, 0.71501E+00_JPRB, 0.71695E+00_JPRB, 0.72789E+00_JPRB ,&
&  0.73596E+00_JPRB, 0.74489E+00_JPRB, 0.75423E+00_JPRB, 0.76704E+00_JPRB, 0.78163E+00_JPRB, 0.78726E+00_JPRB, 0.79683E+00_JPRB ,&
&  0.81935E+00_JPRB, 0.84851E+00_JPRB, 0.89309E+00_JPRB, 0.99168E+00_JPRB, 0.11616E+01_JPRB, 0.13886E+01_JPRB, 0.16460E+01_JPRB ,&
&  0.18681E+01_JPRB, 0.13571E+01_JPRB, 0.97816E+00_JPRB, 0.16547E+01_JPRB, 0.18344E+01_JPRB, 0.18044E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.15902E+01_JPRB, 0.15411E+01_JPRB, 0.14789E+01_JPRB, 0.13921E+01_JPRB, 0.12158E+01_JPRB, 0.11225E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.10124E+01_JPRB, 0.99510E+00_JPRB, 0.10095E+01_JPRB, 0.10585E+01_JPRB, 0.11147E+01_JPRB, 0.11909E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.15440E+01_JPRB, 0.17906E+01_JPRB, 0.21669E+01_JPRB, 0.22872E+01_JPRB, 0.25551E+01_JPRB, 0.22452E+01_JPRB, 0.19147E+01_JPRB ,&
&  0.15977E+01_JPRB, 0.15763E+01_JPRB, 0.16335E+01_JPRB, 0.15057E+01_JPRB, 0.12776E+01_JPRB, 0.11632E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.99159E+00_JPRB, 0.94279E+00_JPRB, 0.88020E+00_JPRB, 0.84527E+00_JPRB, 0.84097E+00_JPRB, 0.84585E+00_JPRB ,&
&  0.84723E+00_JPRB, 0.83883E+00_JPRB, 0.83382E+00_JPRB, 0.84118E+00_JPRB, 0.85586E+00_JPRB, 0.86761E+00_JPRB, 0.88572E+00_JPRB ,&
&  0.90886E+00_JPRB, 0.93943E+00_JPRB, 0.97673E+00_JPRB, 0.10042E+01_JPRB, 0.10014E+01_JPRB, 0.98171E+00_JPRB, 0.10063E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.10246E+01_JPRB, 0.10093E+01_JPRB, 0.10192E+01_JPRB, 0.10091E+01_JPRB, 0.10072E+01_JPRB, 0.10002E+01_JPRB ,&
&  0.99164E+00_JPRB, 0.95975E+00_JPRB, 0.87125E+00_JPRB, 0.75990E+00_JPRB, 0.73104E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.76426E+00_JPRB, 0.73984E+00_JPRB, 0.70930E+00_JPRB, 0.67876E+00_JPRB, 0.66271E+00_JPRB, 0.64656E+00_JPRB, 0.62619E+00_JPRB ,&
&  0.61584E+00_JPRB, 0.61134E+00_JPRB, 0.60138E+00_JPRB, 0.61216E+00_JPRB, 0.63666E+00_JPRB, 0.64919E+00_JPRB, 0.65729E+00_JPRB ,&
&  0.66664E+00_JPRB, 0.67493E+00_JPRB, 0.66720E+00_JPRB, 0.66605E+00_JPRB, 0.67524E+00_JPRB, 0.68248E+00_JPRB, 0.68410E+00_JPRB ,&
&  0.69032E+00_JPRB, 0.70562E+00_JPRB, 0.72666E+00_JPRB, 0.76769E+00_JPRB, 0.85527E+00_JPRB, 0.99183E+00_JPRB, 0.11794E+01_JPRB ,&
&  0.14553E+01_JPRB, 0.14564E+01_JPRB, 0.83104E+00_JPRB, 0.14083E+01_JPRB, 0.19154E+01_JPRB, 0.19351E+01_JPRB, 0.18092E+01_JPRB ,&
&  0.17967E+01_JPRB, 0.17448E+01_JPRB, 0.16899E+01_JPRB, 0.14953E+01_JPRB, 0.13295E+01_JPRB, 0.11979E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.99550E+00_JPRB, 0.93417E+00_JPRB, 0.91027E+00_JPRB, 0.93618E+00_JPRB, 0.98747E+00_JPRB, 0.10303E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.14875E+01_JPRB, 0.18289E+01_JPRB, 0.21189E+01_JPRB, 0.26322E+01_JPRB, 0.26329E+01_JPRB, 0.22159E+01_JPRB ,&
&  0.18414E+01_JPRB, 0.18628E+01_JPRB, 0.18668E+01_JPRB, 0.16779E+01_JPRB, 0.13726E+01_JPRB, 0.12758E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.10627E+01_JPRB, 0.10217E+01_JPRB, 0.98196E+00_JPRB, 0.92836E+00_JPRB, 0.88256E+00_JPRB, 0.86138E+00_JPRB ,&
&  0.85883E+00_JPRB, 0.85563E+00_JPRB, 0.85180E+00_JPRB, 0.84831E+00_JPRB, 0.85121E+00_JPRB, 0.85708E+00_JPRB, 0.86856E+00_JPRB ,&
&  0.88972E+00_JPRB, 0.91685E+00_JPRB, 0.95903E+00_JPRB, 0.98497E+00_JPRB, 0.10017E+01_JPRB, 0.10151E+01_JPRB, 0.10325E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10447E+01_JPRB, 0.97012E+00_JPRB, 0.94746E+00_JPRB, 0.98445E+00_JPRB, 0.97772E+00_JPRB, 0.99335E+00_JPRB ,&
&  0.10201E+01_JPRB, 0.10182E+01_JPRB, 0.95316E+00_JPRB, 0.81911E+00_JPRB, 0.79333E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.77445E+00_JPRB, 0.75394E+00_JPRB, 0.73309E+00_JPRB, 0.71693E+00_JPRB, 0.70595E+00_JPRB, 0.69443E+00_JPRB, 0.67472E+00_JPRB ,&
&  0.64901E+00_JPRB, 0.63363E+00_JPRB, 0.60703E+00_JPRB, 0.59985E+00_JPRB, 0.60515E+00_JPRB, 0.60978E+00_JPRB, 0.61950E+00_JPRB ,&
&  0.62513E+00_JPRB, 0.62994E+00_JPRB, 0.63114E+00_JPRB, 0.62930E+00_JPRB, 0.62896E+00_JPRB, 0.62812E+00_JPRB, 0.62767E+00_JPRB ,&
&  0.62440E+00_JPRB, 0.62815E+00_JPRB, 0.63900E+00_JPRB, 0.65504E+00_JPRB, 0.69343E+00_JPRB, 0.76300E+00_JPRB, 0.86196E+00_JPRB ,&
&  0.10920E+01_JPRB, 0.12154E+01_JPRB, 0.13330E+01_JPRB, 0.19348E+01_JPRB, 0.22864E+01_JPRB, 0.21912E+01_JPRB, 0.21678E+01_JPRB ,&
&  0.22068E+01_JPRB, 0.21769E+01_JPRB, 0.20318E+01_JPRB, 0.16264E+01_JPRB, 0.13498E+01_JPRB, 0.12049E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.99729E+00_JPRB, 0.92254E+00_JPRB, 0.86533E+00_JPRB, 0.84407E+00_JPRB, 0.86282E+00_JPRB, 0.90778E+00_JPRB, 0.95544E+00_JPRB ,&
&  0.10536E+01_JPRB, 0.12174E+01_JPRB, 0.14561E+01_JPRB, 0.17180E+01_JPRB, 0.22173E+01_JPRB, 0.26051E+01_JPRB, 0.25097E+01_JPRB ,&
&  0.23996E+01_JPRB, 0.22647E+01_JPRB, 0.20573E+01_JPRB, 0.18668E+01_JPRB, 0.14904E+01_JPRB, 0.14637E+01_JPRB, 0.13283E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.11073E+01_JPRB, 0.10547E+01_JPRB, 0.10254E+01_JPRB, 0.98695E+00_JPRB, 0.93410E+00_JPRB, 0.89165E+00_JPRB ,&
&  0.86977E+00_JPRB, 0.86618E+00_JPRB, 0.86526E+00_JPRB, 0.85792E+00_JPRB, 0.85538E+00_JPRB, 0.85965E+00_JPRB, 0.86538E+00_JPRB ,&
&  0.86896E+00_JPRB, 0.87808E+00_JPRB, 0.88401E+00_JPRB, 0.92365E+00_JPRB, 0.95658E+00_JPRB, 0.97155E+00_JPRB, 0.96450E+00_JPRB ,&
&  0.97377E+00_JPRB, 0.99357E+00_JPRB, 0.97688E+00_JPRB, 0.97306E+00_JPRB, 0.96207E+00_JPRB, 0.94536E+00_JPRB, 0.90622E+00_JPRB ,&
&  0.90058E+00_JPRB, 0.86973E+00_JPRB, 0.82420E+00_JPRB, 0.80490E+00_JPRB, 0.79010E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.77151E+00_JPRB, 0.75207E+00_JPRB, 0.73806E+00_JPRB, 0.72303E+00_JPRB, 0.70992E+00_JPRB, 0.70378E+00_JPRB, 0.69844E+00_JPRB ,&
&  0.68771E+00_JPRB, 0.66097E+00_JPRB, 0.63298E+00_JPRB, 0.61983E+00_JPRB, 0.61548E+00_JPRB, 0.60449E+00_JPRB, 0.60537E+00_JPRB ,&
&  0.60680E+00_JPRB, 0.61181E+00_JPRB, 0.60768E+00_JPRB, 0.60385E+00_JPRB, 0.60163E+00_JPRB, 0.60065E+00_JPRB, 0.59839E+00_JPRB ,&
&  0.59280E+00_JPRB, 0.59056E+00_JPRB, 0.59587E+00_JPRB, 0.60377E+00_JPRB, 0.61920E+00_JPRB, 0.65269E+00_JPRB, 0.69483E+00_JPRB ,&
&  0.83850E+00_JPRB, 0.10079E+01_JPRB, 0.10997E+01_JPRB, 0.19206E+01_JPRB, 0.22472E+01_JPRB, 0.23775E+01_JPRB, 0.22505E+01_JPRB ,&
&  0.22853E+01_JPRB, 0.25344E+01_JPRB, 0.21573E+01_JPRB, 0.16388E+01_JPRB, 0.12831E+01_JPRB, 0.11199E+01_JPRB, 0.10217E+01_JPRB ,&
&  0.94050E+00_JPRB, 0.86867E+00_JPRB, 0.82100E+00_JPRB, 0.79365E+00_JPRB, 0.79566E+00_JPRB, 0.80748E+00_JPRB, 0.83643E+00_JPRB ,&
&  0.89115E+00_JPRB, 0.10072E+01_JPRB, 0.11833E+01_JPRB, 0.13098E+01_JPRB, 0.16840E+01_JPRB, 0.21331E+01_JPRB, 0.23877E+01_JPRB ,&
&  0.25267E+01_JPRB, 0.25553E+01_JPRB, 0.21808E+01_JPRB, 0.19327E+01_JPRB, 0.16446E+01_JPRB, 0.14988E+01_JPRB, 0.13409E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.11374E+01_JPRB, 0.10713E+01_JPRB, 0.10335E+01_JPRB, 0.99795E+00_JPRB, 0.95888E+00_JPRB, 0.91322E+00_JPRB ,&
&  0.87438E+00_JPRB, 0.86511E+00_JPRB, 0.86238E+00_JPRB, 0.85474E+00_JPRB, 0.85197E+00_JPRB, 0.85029E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84878E+00_JPRB, 0.84861E+00_JPRB, 0.84051E+00_JPRB, 0.86404E+00_JPRB, 0.88632E+00_JPRB, 0.88393E+00_JPRB, 0.89765E+00_JPRB ,&
&  0.94405E+00_JPRB, 0.97960E+00_JPRB, 0.97289E+00_JPRB, 0.94732E+00_JPRB, 0.97462E+00_JPRB, 0.99696E+00_JPRB, 0.89240E+00_JPRB ,&
&  0.86752E+00_JPRB, 0.84480E+00_JPRB, 0.81540E+00_JPRB, 0.79624E+00_JPRB, 0.78498E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.77223E+00_JPRB, 0.74375E+00_JPRB, 0.72348E+00_JPRB, 0.70794E+00_JPRB, 0.69506E+00_JPRB, 0.68402E+00_JPRB, 0.68128E+00_JPRB ,&
&  0.67872E+00_JPRB, 0.67219E+00_JPRB, 0.65442E+00_JPRB, 0.63318E+00_JPRB, 0.62346E+00_JPRB, 0.61966E+00_JPRB, 0.60717E+00_JPRB ,&
&  0.60121E+00_JPRB, 0.60078E+00_JPRB, 0.59797E+00_JPRB, 0.58953E+00_JPRB, 0.58201E+00_JPRB, 0.57729E+00_JPRB, 0.57488E+00_JPRB ,&
&  0.56478E+00_JPRB, 0.56028E+00_JPRB, 0.56167E+00_JPRB, 0.56312E+00_JPRB, 0.56795E+00_JPRB, 0.59543E+00_JPRB, 0.63961E+00_JPRB ,&
&  0.72583E+00_JPRB, 0.10752E+01_JPRB, 0.10819E+01_JPRB, 0.15833E+01_JPRB, 0.17936E+01_JPRB, 0.20333E+01_JPRB, 0.21630E+01_JPRB ,&
&  0.23192E+01_JPRB, 0.24930E+01_JPRB, 0.20455E+01_JPRB, 0.15703E+01_JPRB, 0.11788E+01_JPRB, 0.97156E+00_JPRB, 0.87725E+00_JPRB ,&
&  0.82972E+00_JPRB, 0.78771E+00_JPRB, 0.75647E+00_JPRB, 0.74653E+00_JPRB, 0.74982E+00_JPRB, 0.75478E+00_JPRB, 0.76338E+00_JPRB ,&
&  0.79664E+00_JPRB, 0.86868E+00_JPRB, 0.10349E+01_JPRB, 0.12279E+01_JPRB, 0.13037E+01_JPRB, 0.16130E+01_JPRB, 0.20622E+01_JPRB ,&
&  0.24842E+01_JPRB, 0.26700E+01_JPRB, 0.22721E+01_JPRB, 0.19442E+01_JPRB, 0.17019E+01_JPRB, 0.15182E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.12379E+01_JPRB, 0.11456E+01_JPRB, 0.10782E+01_JPRB, 0.10298E+01_JPRB, 0.99635E+00_JPRB, 0.97241E+00_JPRB, 0.93949E+00_JPRB ,&
&  0.90238E+00_JPRB, 0.88006E+00_JPRB, 0.86964E+00_JPRB, 0.86294E+00_JPRB, 0.85660E+00_JPRB, 0.84494E+00_JPRB, 0.82836E+00_JPRB ,&
&  0.81733E+00_JPRB, 0.81784E+00_JPRB, 0.81126E+00_JPRB, 0.82579E+00_JPRB, 0.83716E+00_JPRB, 0.82810E+00_JPRB, 0.83519E+00_JPRB ,&
&  0.89033E+00_JPRB, 0.92648E+00_JPRB, 0.94051E+00_JPRB, 0.95461E+00_JPRB, 0.10168E+01_JPRB, 0.11354E+01_JPRB, 0.94803E+00_JPRB ,&
&  0.87646E+00_JPRB, 0.84355E+00_JPRB, 0.81853E+00_JPRB, 0.80431E+00_JPRB, 0.79055E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.77417E+00_JPRB, 0.74408E+00_JPRB, 0.71918E+00_JPRB, 0.69521E+00_JPRB, 0.67777E+00_JPRB, 0.66012E+00_JPRB, 0.64864E+00_JPRB ,&
&  0.64445E+00_JPRB, 0.64524E+00_JPRB, 0.64407E+00_JPRB, 0.63603E+00_JPRB, 0.62137E+00_JPRB, 0.60907E+00_JPRB, 0.59976E+00_JPRB ,&
&  0.59037E+00_JPRB, 0.58752E+00_JPRB, 0.58583E+00_JPRB, 0.57669E+00_JPRB, 0.56649E+00_JPRB, 0.55836E+00_JPRB, 0.55047E+00_JPRB ,&
&  0.54395E+00_JPRB, 0.54065E+00_JPRB, 0.53787E+00_JPRB, 0.53529E+00_JPRB, 0.53399E+00_JPRB, 0.54831E+00_JPRB, 0.61680E+00_JPRB ,&
&  0.76126E+00_JPRB, 0.66283E+00_JPRB, 0.12976E+01_JPRB, 0.14554E+01_JPRB, 0.15121E+01_JPRB, 0.16148E+01_JPRB, 0.18777E+01_JPRB ,&
&  0.21536E+01_JPRB, 0.21252E+01_JPRB, 0.18441E+01_JPRB, 0.14796E+01_JPRB, 0.11584E+01_JPRB, 0.94740E+00_JPRB, 0.83743E+00_JPRB ,&
&  0.76208E+00_JPRB, 0.71594E+00_JPRB, 0.69514E+00_JPRB, 0.68385E+00_JPRB, 0.68589E+00_JPRB, 0.69581E+00_JPRB, 0.70700E+00_JPRB ,&
&  0.73074E+00_JPRB, 0.77711E+00_JPRB, 0.89566E+00_JPRB, 0.11132E+01_JPRB, 0.12835E+01_JPRB, 0.14264E+01_JPRB, 0.17273E+01_JPRB ,&
&  0.23057E+01_JPRB, 0.24114E+01_JPRB, 0.20401E+01_JPRB, 0.17719E+01_JPRB, 0.15983E+01_JPRB, 0.14719E+01_JPRB, 0.13487E+01_JPRB ,&
&  0.12343E+01_JPRB, 0.11258E+01_JPRB, 0.10481E+01_JPRB, 0.99428E+00_JPRB, 0.96253E+00_JPRB, 0.94889E+00_JPRB, 0.93849E+00_JPRB ,&
&  0.92610E+00_JPRB, 0.90977E+00_JPRB, 0.89552E+00_JPRB, 0.88003E+00_JPRB, 0.86384E+00_JPRB, 0.84132E+00_JPRB, 0.81160E+00_JPRB ,&
&  0.79132E+00_JPRB, 0.78335E+00_JPRB, 0.77926E+00_JPRB, 0.77655E+00_JPRB, 0.78454E+00_JPRB, 0.79012E+00_JPRB, 0.80487E+00_JPRB ,&
&  0.84754E+00_JPRB, 0.86791E+00_JPRB, 0.88573E+00_JPRB, 0.92098E+00_JPRB, 0.95593E+00_JPRB, 0.10527E+01_JPRB, 0.98506E+00_JPRB ,&
&  0.93390E+00_JPRB, 0.87992E+00_JPRB, 0.83399E+00_JPRB, 0.80951E+00_JPRB, 0.79519E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.77132E+00_JPRB, 0.74716E+00_JPRB, 0.71491E+00_JPRB, 0.68613E+00_JPRB, 0.66279E+00_JPRB, 0.63944E+00_JPRB, 0.62336E+00_JPRB ,&
&  0.61145E+00_JPRB, 0.61178E+00_JPRB, 0.61627E+00_JPRB, 0.61894E+00_JPRB, 0.61412E+00_JPRB, 0.59987E+00_JPRB, 0.58510E+00_JPRB ,&
&  0.57461E+00_JPRB, 0.57132E+00_JPRB, 0.57023E+00_JPRB, 0.56535E+00_JPRB, 0.55774E+00_JPRB, 0.54833E+00_JPRB, 0.54003E+00_JPRB ,&
&  0.53284E+00_JPRB, 0.52729E+00_JPRB, 0.52149E+00_JPRB, 0.51724E+00_JPRB, 0.51210E+00_JPRB, 0.51919E+00_JPRB, 0.58748E+00_JPRB ,&
&  0.78938E+00_JPRB, 0.10122E+01_JPRB, 0.12416E+01_JPRB, 0.11761E+01_JPRB, 0.11769E+01_JPRB, 0.12386E+01_JPRB, 0.14385E+01_JPRB ,&
&  0.15548E+01_JPRB, 0.15959E+01_JPRB, 0.15105E+01_JPRB, 0.13232E+01_JPRB, 0.11023E+01_JPRB, 0.94383E+00_JPRB, 0.82787E+00_JPRB ,&
&  0.73327E+00_JPRB, 0.67422E+00_JPRB, 0.64129E+00_JPRB, 0.63188E+00_JPRB, 0.63634E+00_JPRB, 0.64754E+00_JPRB, 0.65640E+00_JPRB ,&
&  0.67091E+00_JPRB, 0.69861E+00_JPRB, 0.78911E+00_JPRB, 0.99156E+00_JPRB, 0.13084E+01_JPRB, 0.13437E+01_JPRB, 0.17189E+01_JPRB ,&
&  0.21766E+01_JPRB, 0.19447E+01_JPRB, 0.17618E+01_JPRB, 0.16073E+01_JPRB, 0.14880E+01_JPRB, 0.13878E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.10304E+01_JPRB, 0.95764E+00_JPRB, 0.91935E+00_JPRB, 0.89621E+00_JPRB, 0.88704E+00_JPRB, 0.88450E+00_JPRB ,&
&  0.88205E+00_JPRB, 0.87811E+00_JPRB, 0.87713E+00_JPRB, 0.86741E+00_JPRB, 0.84844E+00_JPRB, 0.82313E+00_JPRB, 0.79439E+00_JPRB ,&
&  0.77226E+00_JPRB, 0.75331E+00_JPRB, 0.75916E+00_JPRB, 0.73998E+00_JPRB, 0.74236E+00_JPRB, 0.75340E+00_JPRB, 0.75953E+00_JPRB ,&
&  0.76604E+00_JPRB, 0.79630E+00_JPRB, 0.82319E+00_JPRB, 0.84405E+00_JPRB, 0.95402E+00_JPRB, 0.10344E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.97654E+00_JPRB, 0.90563E+00_JPRB, 0.84522E+00_JPRB, 0.80537E+00_JPRB, 0.78895E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.77071E+00_JPRB, 0.73709E+00_JPRB, 0.70229E+00_JPRB, 0.67149E+00_JPRB, 0.64360E+00_JPRB, 0.62239E+00_JPRB, 0.60922E+00_JPRB ,&
&  0.59786E+00_JPRB, 0.59067E+00_JPRB, 0.59205E+00_JPRB, 0.59713E+00_JPRB, 0.59672E+00_JPRB, 0.58785E+00_JPRB, 0.57486E+00_JPRB ,&
&  0.56140E+00_JPRB, 0.55109E+00_JPRB, 0.54349E+00_JPRB, 0.54011E+00_JPRB, 0.53485E+00_JPRB, 0.52953E+00_JPRB, 0.52139E+00_JPRB ,&
&  0.51532E+00_JPRB, 0.51017E+00_JPRB, 0.50226E+00_JPRB, 0.49385E+00_JPRB, 0.48780E+00_JPRB, 0.49044E+00_JPRB, 0.54410E+00_JPRB ,&
&  0.70336E+00_JPRB, 0.89427E+00_JPRB, 0.90011E+00_JPRB, 0.92859E+00_JPRB, 0.99562E+00_JPRB, 0.10235E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11753E+01_JPRB, 0.11836E+01_JPRB, 0.11201E+01_JPRB, 0.99831E+00_JPRB, 0.90314E+00_JPRB, 0.81257E+00_JPRB ,&
&  0.73086E+00_JPRB, 0.66703E+00_JPRB, 0.62671E+00_JPRB, 0.61033E+00_JPRB, 0.61166E+00_JPRB, 0.61953E+00_JPRB, 0.62402E+00_JPRB ,&
&  0.62750E+00_JPRB, 0.63198E+00_JPRB, 0.66603E+00_JPRB, 0.74997E+00_JPRB, 0.96179E+00_JPRB, 0.10953E+01_JPRB, 0.13202E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.13529E+01_JPRB, 0.12995E+01_JPRB, 0.12389E+01_JPRB, 0.11791E+01_JPRB, 0.11042E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.97495E+00_JPRB, 0.90954E+00_JPRB, 0.85951E+00_JPRB, 0.82714E+00_JPRB, 0.80910E+00_JPRB, 0.79993E+00_JPRB, 0.79881E+00_JPRB ,&
&  0.79153E+00_JPRB, 0.78883E+00_JPRB, 0.78247E+00_JPRB, 0.78034E+00_JPRB, 0.78014E+00_JPRB, 0.76601E+00_JPRB, 0.74202E+00_JPRB ,&
&  0.71650E+00_JPRB, 0.69415E+00_JPRB, 0.69028E+00_JPRB, 0.69206E+00_JPRB, 0.69426E+00_JPRB, 0.69355E+00_JPRB, 0.69331E+00_JPRB ,&
&  0.70575E+00_JPRB, 0.75312E+00_JPRB, 0.77053E+00_JPRB, 0.78859E+00_JPRB, 0.86484E+00_JPRB, 0.92269E+00_JPRB, 0.92446E+00_JPRB ,&
&  0.90932E+00_JPRB, 0.87540E+00_JPRB, 0.83137E+00_JPRB, 0.79275E+00_JPRB, 0.79146E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.78409E+00_JPRB, 0.75237E+00_JPRB, 0.71381E+00_JPRB, 0.67740E+00_JPRB, 0.64436E+00_JPRB, 0.61841E+00_JPRB, 0.60243E+00_JPRB ,&
&  0.59175E+00_JPRB, 0.58424E+00_JPRB, 0.57981E+00_JPRB, 0.57688E+00_JPRB, 0.57295E+00_JPRB, 0.56896E+00_JPRB, 0.56271E+00_JPRB ,&
&  0.55483E+00_JPRB, 0.54509E+00_JPRB, 0.53602E+00_JPRB, 0.52941E+00_JPRB, 0.52396E+00_JPRB, 0.52076E+00_JPRB, 0.51667E+00_JPRB ,&
&  0.50856E+00_JPRB, 0.49890E+00_JPRB, 0.48676E+00_JPRB, 0.47439E+00_JPRB, 0.46666E+00_JPRB, 0.46686E+00_JPRB, 0.50147E+00_JPRB ,&
&  0.61728E+00_JPRB, 0.65975E+00_JPRB, 0.71039E+00_JPRB, 0.75388E+00_JPRB, 0.81014E+00_JPRB, 0.85086E+00_JPRB, 0.87943E+00_JPRB ,&
&  0.89850E+00_JPRB, 0.91960E+00_JPRB, 0.95506E+00_JPRB, 0.94813E+00_JPRB, 0.88256E+00_JPRB, 0.82068E+00_JPRB, 0.76247E+00_JPRB ,&
&  0.70894E+00_JPRB, 0.66190E+00_JPRB, 0.62301E+00_JPRB, 0.60624E+00_JPRB, 0.60188E+00_JPRB, 0.60424E+00_JPRB, 0.60425E+00_JPRB ,&
&  0.60232E+00_JPRB, 0.59488E+00_JPRB, 0.59409E+00_JPRB, 0.60870E+00_JPRB, 0.64349E+00_JPRB, 0.70693E+00_JPRB, 0.81576E+00_JPRB ,&
&  0.86866E+00_JPRB, 0.86667E+00_JPRB, 0.87543E+00_JPRB, 0.87942E+00_JPRB, 0.85539E+00_JPRB, 0.82869E+00_JPRB, 0.80503E+00_JPRB ,&
&  0.78363E+00_JPRB, 0.76405E+00_JPRB, 0.74903E+00_JPRB, 0.73103E+00_JPRB, 0.71683E+00_JPRB, 0.70807E+00_JPRB, 0.70302E+00_JPRB ,&
&  0.69866E+00_JPRB, 0.69681E+00_JPRB, 0.69121E+00_JPRB, 0.67832E+00_JPRB, 0.66916E+00_JPRB, 0.66007E+00_JPRB, 0.64516E+00_JPRB ,&
&  0.62611E+00_JPRB, 0.61542E+00_JPRB, 0.61518E+00_JPRB, 0.62073E+00_JPRB, 0.62944E+00_JPRB, 0.63813E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64893E+00_JPRB, 0.67521E+00_JPRB, 0.68661E+00_JPRB, 0.75187E+00_JPRB, 0.81479E+00_JPRB, 0.82724E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84425E+00_JPRB, 0.82859E+00_JPRB, 0.79551E+00_JPRB, 0.75704E+00_JPRB, 0.79009E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.68528E+00_JPRB, 0.68296E+00_JPRB, 0.66253E+00_JPRB, 0.63837E+00_JPRB, 0.61682E+00_JPRB, 0.60008E+00_JPRB, 0.58585E+00_JPRB ,&
&  0.57569E+00_JPRB, 0.56803E+00_JPRB, 0.56260E+00_JPRB, 0.55888E+00_JPRB, 0.55417E+00_JPRB, 0.54745E+00_JPRB, 0.54028E+00_JPRB ,&
&  0.53443E+00_JPRB, 0.52768E+00_JPRB, 0.51747E+00_JPRB, 0.50998E+00_JPRB, 0.50531E+00_JPRB, 0.49992E+00_JPRB, 0.49465E+00_JPRB ,&
&  0.48813E+00_JPRB, 0.47533E+00_JPRB, 0.46246E+00_JPRB, 0.45384E+00_JPRB, 0.44722E+00_JPRB, 0.44493E+00_JPRB, 0.46718E+00_JPRB ,&
&  0.54179E+00_JPRB, 0.52889E+00_JPRB, 0.57572E+00_JPRB, 0.62572E+00_JPRB, 0.68502E+00_JPRB, 0.72088E+00_JPRB, 0.74041E+00_JPRB ,&
&  0.75248E+00_JPRB, 0.77440E+00_JPRB, 0.79248E+00_JPRB, 0.81652E+00_JPRB, 0.78804E+00_JPRB, 0.74154E+00_JPRB, 0.70236E+00_JPRB ,&
&  0.66609E+00_JPRB, 0.63445E+00_JPRB, 0.60845E+00_JPRB, 0.59078E+00_JPRB, 0.58452E+00_JPRB, 0.58194E+00_JPRB, 0.57623E+00_JPRB ,&
&  0.57032E+00_JPRB, 0.56016E+00_JPRB, 0.55096E+00_JPRB, 0.55075E+00_JPRB, 0.54808E+00_JPRB, 0.56672E+00_JPRB, 0.59377E+00_JPRB ,&
&  0.61685E+00_JPRB, 0.64784E+00_JPRB, 0.66258E+00_JPRB, 0.67143E+00_JPRB, 0.66468E+00_JPRB, 0.65630E+00_JPRB, 0.64721E+00_JPRB ,&
&  0.63946E+00_JPRB, 0.63251E+00_JPRB, 0.62702E+00_JPRB, 0.62003E+00_JPRB, 0.61194E+00_JPRB, 0.60628E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.59799E+00_JPRB, 0.59394E+00_JPRB, 0.59050E+00_JPRB, 0.58034E+00_JPRB, 0.56565E+00_JPRB, 0.55374E+00_JPRB, 0.54456E+00_JPRB ,&
&  0.53771E+00_JPRB, 0.53668E+00_JPRB, 0.53994E+00_JPRB, 0.54962E+00_JPRB, 0.55957E+00_JPRB, 0.57080E+00_JPRB, 0.57890E+00_JPRB ,&
&  0.58457E+00_JPRB, 0.60082E+00_JPRB, 0.62595E+00_JPRB, 0.67793E+00_JPRB, 0.71222E+00_JPRB, 0.73718E+00_JPRB, 0.75745E+00_JPRB ,&
&  0.76588E+00_JPRB, 0.76431E+00_JPRB, 0.74160E+00_JPRB, 0.71587E+00_JPRB, 0.67929E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.63189E+00_JPRB, 0.62951E+00_JPRB, 0.61640E+00_JPRB, 0.59809E+00_JPRB, 0.58236E+00_JPRB, 0.56880E+00_JPRB, 0.55796E+00_JPRB ,&
&  0.54408E+00_JPRB, 0.53225E+00_JPRB, 0.52708E+00_JPRB, 0.52531E+00_JPRB, 0.52323E+00_JPRB, 0.51810E+00_JPRB, 0.51148E+00_JPRB ,&
&  0.50431E+00_JPRB, 0.49627E+00_JPRB, 0.48697E+00_JPRB, 0.47753E+00_JPRB, 0.46955E+00_JPRB, 0.46096E+00_JPRB, 0.45440E+00_JPRB ,&
&  0.44831E+00_JPRB, 0.44299E+00_JPRB, 0.43461E+00_JPRB, 0.42435E+00_JPRB, 0.41976E+00_JPRB, 0.41466E+00_JPRB, 0.43400E+00_JPRB ,&
&  0.49827E+00_JPRB, 0.43528E+00_JPRB, 0.46862E+00_JPRB, 0.52555E+00_JPRB, 0.58202E+00_JPRB, 0.64074E+00_JPRB, 0.66520E+00_JPRB ,&
&  0.67900E+00_JPRB, 0.68905E+00_JPRB, 0.69533E+00_JPRB, 0.70711E+00_JPRB, 0.70768E+00_JPRB, 0.68119E+00_JPRB, 0.64534E+00_JPRB ,&
&  0.61792E+00_JPRB, 0.59265E+00_JPRB, 0.57636E+00_JPRB, 0.56056E+00_JPRB, 0.55286E+00_JPRB, 0.54962E+00_JPRB, 0.54076E+00_JPRB ,&
&  0.52834E+00_JPRB, 0.52173E+00_JPRB, 0.51639E+00_JPRB, 0.51656E+00_JPRB, 0.51851E+00_JPRB, 0.52160E+00_JPRB, 0.51514E+00_JPRB ,&
&  0.50911E+00_JPRB, 0.51869E+00_JPRB, 0.53535E+00_JPRB, 0.54331E+00_JPRB, 0.54488E+00_JPRB, 0.54081E+00_JPRB, 0.53330E+00_JPRB ,&
&  0.52744E+00_JPRB, 0.52293E+00_JPRB, 0.52181E+00_JPRB, 0.51874E+00_JPRB, 0.51465E+00_JPRB, 0.50913E+00_JPRB, 0.50368E+00_JPRB ,&
&  0.49924E+00_JPRB, 0.49573E+00_JPRB, 0.49260E+00_JPRB, 0.48921E+00_JPRB, 0.48449E+00_JPRB, 0.47741E+00_JPRB, 0.47372E+00_JPRB ,&
&  0.47239E+00_JPRB, 0.47381E+00_JPRB, 0.47820E+00_JPRB, 0.48583E+00_JPRB, 0.49656E+00_JPRB, 0.50682E+00_JPRB, 0.51886E+00_JPRB ,&
&  0.53031E+00_JPRB, 0.54399E+00_JPRB, 0.55705E+00_JPRB, 0.64419E+00_JPRB, 0.68749E+00_JPRB, 0.70225E+00_JPRB, 0.71366E+00_JPRB ,&
&  0.71685E+00_JPRB, 0.71304E+00_JPRB, 0.70305E+00_JPRB, 0.61041E+00_JPRB, 0.63145E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.58169E+00_JPRB, 0.57704E+00_JPRB, 0.56597E+00_JPRB, 0.55345E+00_JPRB, 0.54282E+00_JPRB, 0.53252E+00_JPRB, 0.52425E+00_JPRB ,&
&  0.51353E+00_JPRB, 0.50096E+00_JPRB, 0.49142E+00_JPRB, 0.48648E+00_JPRB, 0.48370E+00_JPRB, 0.47972E+00_JPRB, 0.47358E+00_JPRB ,&
&  0.46589E+00_JPRB, 0.45774E+00_JPRB, 0.44993E+00_JPRB, 0.44055E+00_JPRB, 0.43162E+00_JPRB, 0.42364E+00_JPRB, 0.41728E+00_JPRB ,&
&  0.41101E+00_JPRB, 0.40430E+00_JPRB, 0.39770E+00_JPRB, 0.39317E+00_JPRB, 0.38960E+00_JPRB, 0.38812E+00_JPRB, 0.40376E+00_JPRB ,&
&  0.45352E+00_JPRB, 0.39010E+00_JPRB, 0.41475E+00_JPRB, 0.44992E+00_JPRB, 0.50249E+00_JPRB, 0.56769E+00_JPRB, 0.60146E+00_JPRB ,&
&  0.61737E+00_JPRB, 0.62589E+00_JPRB, 0.62640E+00_JPRB, 0.62640E+00_JPRB, 0.63075E+00_JPRB, 0.62416E+00_JPRB, 0.60398E+00_JPRB ,&
&  0.58171E+00_JPRB, 0.56024E+00_JPRB, 0.54427E+00_JPRB, 0.53237E+00_JPRB, 0.52194E+00_JPRB, 0.51385E+00_JPRB, 0.50267E+00_JPRB ,&
&  0.48642E+00_JPRB, 0.47721E+00_JPRB, 0.47194E+00_JPRB, 0.46780E+00_JPRB, 0.46596E+00_JPRB, 0.46225E+00_JPRB, 0.45136E+00_JPRB ,&
&  0.44153E+00_JPRB, 0.43557E+00_JPRB, 0.43401E+00_JPRB, 0.43592E+00_JPRB, 0.43847E+00_JPRB, 0.43907E+00_JPRB, 0.43613E+00_JPRB ,&
&  0.43216E+00_JPRB, 0.42676E+00_JPRB, 0.42371E+00_JPRB, 0.42194E+00_JPRB, 0.42044E+00_JPRB, 0.42115E+00_JPRB, 0.42168E+00_JPRB ,&
&  0.42087E+00_JPRB, 0.42131E+00_JPRB, 0.42368E+00_JPRB, 0.42586E+00_JPRB, 0.42613E+00_JPRB, 0.42485E+00_JPRB, 0.42296E+00_JPRB ,&
&  0.42057E+00_JPRB, 0.41918E+00_JPRB, 0.42044E+00_JPRB, 0.42578E+00_JPRB, 0.43376E+00_JPRB, 0.44313E+00_JPRB, 0.45367E+00_JPRB ,&
&  0.46552E+00_JPRB, 0.47855E+00_JPRB, 0.48988E+00_JPRB, 0.53367E+00_JPRB, 0.56494E+00_JPRB, 0.59296E+00_JPRB, 0.61773E+00_JPRB ,&
&  0.62786E+00_JPRB, 0.62852E+00_JPRB, 0.52749E+00_JPRB, 0.56802E+00_JPRB, 0.57696E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.54352E+00_JPRB, 0.53105E+00_JPRB, 0.51615E+00_JPRB, 0.50560E+00_JPRB, 0.49970E+00_JPRB, 0.49333E+00_JPRB, 0.48703E+00_JPRB ,&
&  0.47856E+00_JPRB, 0.46942E+00_JPRB, 0.46054E+00_JPRB, 0.45260E+00_JPRB, 0.44634E+00_JPRB, 0.44200E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.43049E+00_JPRB, 0.42312E+00_JPRB, 0.41562E+00_JPRB, 0.40720E+00_JPRB, 0.39635E+00_JPRB, 0.38761E+00_JPRB, 0.38067E+00_JPRB ,&
&  0.37522E+00_JPRB, 0.37093E+00_JPRB, 0.36921E+00_JPRB, 0.36714E+00_JPRB, 0.36493E+00_JPRB, 0.36494E+00_JPRB, 0.37757E+00_JPRB ,&
&  0.38037E+00_JPRB, 0.35471E+00_JPRB, 0.37520E+00_JPRB, 0.40334E+00_JPRB, 0.44646E+00_JPRB, 0.51248E+00_JPRB, 0.55578E+00_JPRB ,&
&  0.57880E+00_JPRB, 0.58941E+00_JPRB, 0.59197E+00_JPRB, 0.58656E+00_JPRB, 0.58053E+00_JPRB, 0.58053E+00_JPRB, 0.57501E+00_JPRB ,&
&  0.55975E+00_JPRB, 0.54012E+00_JPRB, 0.52135E+00_JPRB, 0.50732E+00_JPRB, 0.49193E+00_JPRB, 0.47929E+00_JPRB, 0.46617E+00_JPRB ,&
&  0.45372E+00_JPRB, 0.44420E+00_JPRB, 0.43852E+00_JPRB, 0.43416E+00_JPRB, 0.42895E+00_JPRB, 0.42036E+00_JPRB, 0.41084E+00_JPRB ,&
&  0.40340E+00_JPRB, 0.39698E+00_JPRB, 0.39089E+00_JPRB, 0.38547E+00_JPRB, 0.38218E+00_JPRB, 0.38006E+00_JPRB, 0.37785E+00_JPRB ,&
&  0.37721E+00_JPRB, 0.37738E+00_JPRB, 0.37805E+00_JPRB, 0.37846E+00_JPRB, 0.37799E+00_JPRB, 0.37657E+00_JPRB, 0.37656E+00_JPRB ,&
&  0.37753E+00_JPRB, 0.37594E+00_JPRB, 0.37553E+00_JPRB, 0.37847E+00_JPRB, 0.38203E+00_JPRB, 0.38426E+00_JPRB, 0.38486E+00_JPRB ,&
&  0.38380E+00_JPRB, 0.38337E+00_JPRB, 0.38442E+00_JPRB, 0.38755E+00_JPRB, 0.39175E+00_JPRB, 0.39796E+00_JPRB, 0.40560E+00_JPRB ,&
&  0.41365E+00_JPRB, 0.42386E+00_JPRB, 0.43681E+00_JPRB, 0.45981E+00_JPRB, 0.48159E+00_JPRB, 0.50238E+00_JPRB, 0.52289E+00_JPRB ,&
&  0.53362E+00_JPRB, 0.54125E+00_JPRB, 0.55013E+00_JPRB, 0.55005E+00_JPRB, 0.54882E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.47179E+00_JPRB, 0.46525E+00_JPRB, 0.45686E+00_JPRB, 0.44947E+00_JPRB, 0.44547E+00_JPRB, 0.44164E+00_JPRB, 0.43540E+00_JPRB ,&
&  0.42960E+00_JPRB, 0.42428E+00_JPRB, 0.41907E+00_JPRB, 0.41442E+00_JPRB, 0.41044E+00_JPRB, 0.40735E+00_JPRB, 0.40299E+00_JPRB ,&
&  0.39678E+00_JPRB, 0.38929E+00_JPRB, 0.38106E+00_JPRB, 0.37427E+00_JPRB, 0.36605E+00_JPRB, 0.35755E+00_JPRB, 0.35115E+00_JPRB ,&
&  0.34577E+00_JPRB, 0.34201E+00_JPRB, 0.34048E+00_JPRB, 0.33843E+00_JPRB, 0.33901E+00_JPRB, 0.34197E+00_JPRB, 0.35134E+00_JPRB ,&
&  0.36725E+00_JPRB, 0.34916E+00_JPRB, 0.36870E+00_JPRB, 0.38674E+00_JPRB, 0.42254E+00_JPRB, 0.47400E+00_JPRB, 0.50874E+00_JPRB ,&
&  0.53053E+00_JPRB, 0.54786E+00_JPRB, 0.56011E+00_JPRB, 0.56203E+00_JPRB, 0.55366E+00_JPRB, 0.54879E+00_JPRB, 0.54568E+00_JPRB ,&
&  0.53637E+00_JPRB, 0.52144E+00_JPRB, 0.50102E+00_JPRB, 0.47940E+00_JPRB, 0.45983E+00_JPRB, 0.44523E+00_JPRB, 0.43384E+00_JPRB ,&
&  0.42486E+00_JPRB, 0.41697E+00_JPRB, 0.41032E+00_JPRB, 0.40130E+00_JPRB, 0.39096E+00_JPRB, 0.38039E+00_JPRB, 0.37058E+00_JPRB ,&
&  0.36091E+00_JPRB, 0.35156E+00_JPRB, 0.34342E+00_JPRB, 0.33673E+00_JPRB, 0.33295E+00_JPRB, 0.33106E+00_JPRB, 0.33025E+00_JPRB ,&
&  0.33049E+00_JPRB, 0.33234E+00_JPRB, 0.33451E+00_JPRB, 0.33624E+00_JPRB, 0.33732E+00_JPRB, 0.33677E+00_JPRB, 0.33424E+00_JPRB ,&
&  0.33357E+00_JPRB, 0.33332E+00_JPRB, 0.33348E+00_JPRB, 0.33478E+00_JPRB, 0.33532E+00_JPRB, 0.33626E+00_JPRB, 0.33897E+00_JPRB ,&
&  0.34155E+00_JPRB, 0.34308E+00_JPRB, 0.34455E+00_JPRB, 0.34638E+00_JPRB, 0.34873E+00_JPRB, 0.35247E+00_JPRB, 0.35782E+00_JPRB ,&
&  0.36394E+00_JPRB, 0.37086E+00_JPRB, 0.38090E+00_JPRB, 0.39409E+00_JPRB, 0.40791E+00_JPRB, 0.42188E+00_JPRB, 0.43576E+00_JPRB ,&
&  0.44462E+00_JPRB, 0.45351E+00_JPRB, 0.46448E+00_JPRB, 0.47001E+00_JPRB, 0.47361E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.41311E+00_JPRB, 0.41074E+00_JPRB, 0.40748E+00_JPRB, 0.40444E+00_JPRB, 0.40220E+00_JPRB, 0.39768E+00_JPRB, 0.39205E+00_JPRB ,&
&  0.38813E+00_JPRB, 0.38454E+00_JPRB, 0.38180E+00_JPRB, 0.37784E+00_JPRB, 0.37537E+00_JPRB, 0.37363E+00_JPRB, 0.36908E+00_JPRB ,&
&  0.36389E+00_JPRB, 0.35678E+00_JPRB, 0.34986E+00_JPRB, 0.34363E+00_JPRB, 0.33769E+00_JPRB, 0.33212E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.31999E+00_JPRB, 0.31551E+00_JPRB, 0.31294E+00_JPRB, 0.31075E+00_JPRB, 0.31192E+00_JPRB, 0.31776E+00_JPRB, 0.32752E+00_JPRB ,&
&  0.33551E+00_JPRB, 0.34284E+00_JPRB, 0.35636E+00_JPRB, 0.37063E+00_JPRB, 0.39766E+00_JPRB, 0.43561E+00_JPRB, 0.46235E+00_JPRB ,&
&  0.47830E+00_JPRB, 0.49582E+00_JPRB, 0.51655E+00_JPRB, 0.52959E+00_JPRB, 0.53067E+00_JPRB, 0.52358E+00_JPRB, 0.51807E+00_JPRB ,&
&  0.51089E+00_JPRB, 0.49729E+00_JPRB, 0.48250E+00_JPRB, 0.46469E+00_JPRB, 0.44611E+00_JPRB, 0.43074E+00_JPRB, 0.41858E+00_JPRB ,&
&  0.40769E+00_JPRB, 0.39736E+00_JPRB, 0.38753E+00_JPRB, 0.37724E+00_JPRB, 0.36609E+00_JPRB, 0.35410E+00_JPRB, 0.34140E+00_JPRB ,&
&  0.32916E+00_JPRB, 0.31883E+00_JPRB, 0.31089E+00_JPRB, 0.30657E+00_JPRB, 0.30407E+00_JPRB, 0.30194E+00_JPRB, 0.30043E+00_JPRB ,&
&  0.30047E+00_JPRB, 0.30167E+00_JPRB, 0.30365E+00_JPRB, 0.30570E+00_JPRB, 0.30747E+00_JPRB, 0.30777E+00_JPRB, 0.30557E+00_JPRB ,&
&  0.30325E+00_JPRB, 0.30264E+00_JPRB, 0.30269E+00_JPRB, 0.30025E+00_JPRB, 0.29762E+00_JPRB, 0.29827E+00_JPRB, 0.30077E+00_JPRB ,&
&  0.30517E+00_JPRB, 0.30922E+00_JPRB, 0.31256E+00_JPRB, 0.31530E+00_JPRB, 0.31734E+00_JPRB, 0.32084E+00_JPRB, 0.32518E+00_JPRB ,&
&  0.33059E+00_JPRB, 0.33728E+00_JPRB, 0.34661E+00_JPRB, 0.35808E+00_JPRB, 0.36885E+00_JPRB, 0.37843E+00_JPRB, 0.38545E+00_JPRB ,&
&  0.39055E+00_JPRB, 0.39525E+00_JPRB, 0.40044E+00_JPRB, 0.40624E+00_JPRB, 0.41114E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.36149E+00_JPRB, 0.36191E+00_JPRB, 0.36278E+00_JPRB, 0.36296E+00_JPRB, 0.35998E+00_JPRB, 0.35647E+00_JPRB, 0.35256E+00_JPRB ,&
&  0.35063E+00_JPRB, 0.34922E+00_JPRB, 0.34855E+00_JPRB, 0.34524E+00_JPRB, 0.34258E+00_JPRB, 0.34076E+00_JPRB, 0.33672E+00_JPRB ,&
&  0.33291E+00_JPRB, 0.32503E+00_JPRB, 0.31674E+00_JPRB, 0.31064E+00_JPRB, 0.30679E+00_JPRB, 0.30408E+00_JPRB, 0.29867E+00_JPRB ,&
&  0.29471E+00_JPRB, 0.29152E+00_JPRB, 0.28987E+00_JPRB, 0.28984E+00_JPRB, 0.29190E+00_JPRB, 0.29775E+00_JPRB, 0.30756E+00_JPRB ,&
&  0.31494E+00_JPRB, 0.32665E+00_JPRB, 0.32813E+00_JPRB, 0.34435E+00_JPRB, 0.36161E+00_JPRB, 0.38673E+00_JPRB, 0.40924E+00_JPRB ,&
&  0.42350E+00_JPRB, 0.43383E+00_JPRB, 0.44871E+00_JPRB, 0.46497E+00_JPRB, 0.47270E+00_JPRB, 0.47488E+00_JPRB, 0.47277E+00_JPRB ,&
&  0.46825E+00_JPRB, 0.46195E+00_JPRB, 0.45459E+00_JPRB, 0.44522E+00_JPRB, 0.43234E+00_JPRB, 0.41769E+00_JPRB, 0.40325E+00_JPRB ,&
&  0.39013E+00_JPRB, 0.37728E+00_JPRB, 0.36453E+00_JPRB, 0.35066E+00_JPRB, 0.33662E+00_JPRB, 0.32347E+00_JPRB, 0.31047E+00_JPRB ,&
&  0.29972E+00_JPRB, 0.29145E+00_JPRB, 0.28533E+00_JPRB, 0.28088E+00_JPRB, 0.27752E+00_JPRB, 0.27433E+00_JPRB, 0.27272E+00_JPRB ,&
&  0.27264E+00_JPRB, 0.27416E+00_JPRB, 0.27653E+00_JPRB, 0.27849E+00_JPRB, 0.28087E+00_JPRB, 0.28124E+00_JPRB, 0.27820E+00_JPRB ,&
&  0.27512E+00_JPRB, 0.27384E+00_JPRB, 0.26946E+00_JPRB, 0.26689E+00_JPRB, 0.26637E+00_JPRB, 0.26698E+00_JPRB, 0.26911E+00_JPRB ,&
&  0.27324E+00_JPRB, 0.27888E+00_JPRB, 0.28402E+00_JPRB, 0.28847E+00_JPRB, 0.29233E+00_JPRB, 0.29629E+00_JPRB, 0.30088E+00_JPRB ,&
&  0.30574E+00_JPRB, 0.31046E+00_JPRB, 0.31771E+00_JPRB, 0.32573E+00_JPRB, 0.33266E+00_JPRB, 0.34024E+00_JPRB, 0.34539E+00_JPRB ,&
&  0.34787E+00_JPRB, 0.34928E+00_JPRB, 0.35238E+00_JPRB, 0.35682E+00_JPRB, 0.36021E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.32138E+00_JPRB, 0.32342E+00_JPRB, 0.32564E+00_JPRB, 0.32584E+00_JPRB, 0.32501E+00_JPRB, 0.32473E+00_JPRB, 0.32345E+00_JPRB ,&
&  0.32098E+00_JPRB, 0.31953E+00_JPRB, 0.31963E+00_JPRB, 0.31806E+00_JPRB, 0.31497E+00_JPRB, 0.31115E+00_JPRB, 0.30695E+00_JPRB ,&
&  0.30359E+00_JPRB, 0.30021E+00_JPRB, 0.29479E+00_JPRB, 0.28778E+00_JPRB, 0.28175E+00_JPRB, 0.27782E+00_JPRB, 0.27568E+00_JPRB ,&
&  0.27469E+00_JPRB, 0.27452E+00_JPRB, 0.27401E+00_JPRB, 0.27455E+00_JPRB, 0.27665E+00_JPRB, 0.28153E+00_JPRB, 0.28879E+00_JPRB ,&
&  0.29604E+00_JPRB, 0.30334E+00_JPRB, 0.31085E+00_JPRB, 0.31922E+00_JPRB, 0.32934E+00_JPRB, 0.34255E+00_JPRB, 0.35576E+00_JPRB ,&
&  0.36865E+00_JPRB, 0.37337E+00_JPRB, 0.37534E+00_JPRB, 0.38150E+00_JPRB, 0.39245E+00_JPRB, 0.40069E+00_JPRB, 0.40607E+00_JPRB ,&
&  0.40895E+00_JPRB, 0.40814E+00_JPRB, 0.40572E+00_JPRB, 0.39986E+00_JPRB, 0.39106E+00_JPRB, 0.37968E+00_JPRB, 0.36735E+00_JPRB ,&
&  0.35521E+00_JPRB, 0.34266E+00_JPRB, 0.32921E+00_JPRB, 0.31623E+00_JPRB, 0.30477E+00_JPRB, 0.29442E+00_JPRB, 0.28439E+00_JPRB ,&
&  0.27539E+00_JPRB, 0.26829E+00_JPRB, 0.26269E+00_JPRB, 0.25878E+00_JPRB, 0.25557E+00_JPRB, 0.25304E+00_JPRB, 0.25219E+00_JPRB ,&
&  0.25236E+00_JPRB, 0.25362E+00_JPRB, 0.25427E+00_JPRB, 0.25573E+00_JPRB, 0.25823E+00_JPRB, 0.25916E+00_JPRB, 0.25775E+00_JPRB ,&
&  0.25362E+00_JPRB, 0.24947E+00_JPRB, 0.24546E+00_JPRB, 0.24473E+00_JPRB, 0.24468E+00_JPRB, 0.24602E+00_JPRB, 0.24810E+00_JPRB ,&
&  0.25208E+00_JPRB, 0.25775E+00_JPRB, 0.26282E+00_JPRB, 0.26652E+00_JPRB, 0.27061E+00_JPRB, 0.27422E+00_JPRB, 0.27778E+00_JPRB ,&
&  0.28170E+00_JPRB, 0.28590E+00_JPRB, 0.29148E+00_JPRB, 0.29703E+00_JPRB, 0.30208E+00_JPRB, 0.30619E+00_JPRB, 0.30938E+00_JPRB ,&
&  0.31173E+00_JPRB, 0.31272E+00_JPRB, 0.31349E+00_JPRB, 0.31563E+00_JPRB, 0.31880E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.28699E+00_JPRB, 0.28945E+00_JPRB, 0.29248E+00_JPRB, 0.29485E+00_JPRB, 0.29566E+00_JPRB, 0.29510E+00_JPRB, 0.29427E+00_JPRB ,&
&  0.29403E+00_JPRB, 0.29383E+00_JPRB, 0.29227E+00_JPRB, 0.28960E+00_JPRB, 0.28682E+00_JPRB, 0.28389E+00_JPRB, 0.28167E+00_JPRB ,&
&  0.27955E+00_JPRB, 0.27731E+00_JPRB, 0.27340E+00_JPRB, 0.26766E+00_JPRB, 0.26173E+00_JPRB, 0.25829E+00_JPRB, 0.25666E+00_JPRB ,&
&  0.25657E+00_JPRB, 0.25713E+00_JPRB, 0.25965E+00_JPRB, 0.26142E+00_JPRB, 0.26386E+00_JPRB, 0.26798E+00_JPRB, 0.27372E+00_JPRB ,&
&  0.27909E+00_JPRB, 0.28424E+00_JPRB, 0.28965E+00_JPRB, 0.29443E+00_JPRB, 0.29920E+00_JPRB, 0.30536E+00_JPRB, 0.31248E+00_JPRB ,&
&  0.32124E+00_JPRB, 0.32750E+00_JPRB, 0.32812E+00_JPRB, 0.32895E+00_JPRB, 0.33254E+00_JPRB, 0.33720E+00_JPRB, 0.34074E+00_JPRB ,&
&  0.34349E+00_JPRB, 0.34271E+00_JPRB, 0.34051E+00_JPRB, 0.33659E+00_JPRB, 0.33196E+00_JPRB, 0.32648E+00_JPRB, 0.31909E+00_JPRB ,&
&  0.31021E+00_JPRB, 0.30055E+00_JPRB, 0.29120E+00_JPRB, 0.28221E+00_JPRB, 0.27349E+00_JPRB, 0.26548E+00_JPRB, 0.25835E+00_JPRB ,&
&  0.25171E+00_JPRB, 0.24657E+00_JPRB, 0.24166E+00_JPRB, 0.23858E+00_JPRB, 0.23659E+00_JPRB, 0.23441E+00_JPRB, 0.23387E+00_JPRB ,&
&  0.23284E+00_JPRB, 0.23223E+00_JPRB, 0.23206E+00_JPRB, 0.23360E+00_JPRB, 0.23663E+00_JPRB, 0.23844E+00_JPRB, 0.23715E+00_JPRB ,&
&  0.23382E+00_JPRB, 0.23174E+00_JPRB, 0.23048E+00_JPRB, 0.22957E+00_JPRB, 0.22731E+00_JPRB, 0.22717E+00_JPRB, 0.22933E+00_JPRB ,&
&  0.23379E+00_JPRB, 0.23870E+00_JPRB, 0.24288E+00_JPRB, 0.24547E+00_JPRB, 0.24812E+00_JPRB, 0.25035E+00_JPRB, 0.25232E+00_JPRB ,&
&  0.25437E+00_JPRB, 0.25584E+00_JPRB, 0.25695E+00_JPRB, 0.25845E+00_JPRB, 0.26093E+00_JPRB, 0.26339E+00_JPRB, 0.26587E+00_JPRB ,&
&  0.26866E+00_JPRB, 0.27289E+00_JPRB, 0.27720E+00_JPRB, 0.28049E+00_JPRB, 0.28447E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.26363E+00_JPRB, 0.26632E+00_JPRB, 0.26895E+00_JPRB, 0.26907E+00_JPRB, 0.26727E+00_JPRB, 0.26632E+00_JPRB, 0.26647E+00_JPRB ,&
&  0.26687E+00_JPRB, 0.26687E+00_JPRB, 0.26546E+00_JPRB, 0.26523E+00_JPRB, 0.26519E+00_JPRB, 0.26423E+00_JPRB, 0.26351E+00_JPRB ,&
&  0.26214E+00_JPRB, 0.25903E+00_JPRB, 0.25460E+00_JPRB, 0.24920E+00_JPRB, 0.24520E+00_JPRB, 0.24316E+00_JPRB, 0.24293E+00_JPRB ,&
&  0.24376E+00_JPRB, 0.24440E+00_JPRB, 0.24697E+00_JPRB, 0.24914E+00_JPRB, 0.25180E+00_JPRB, 0.25538E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.26198E+00_JPRB, 0.26492E+00_JPRB, 0.26792E+00_JPRB, 0.27078E+00_JPRB, 0.27456E+00_JPRB, 0.28022E+00_JPRB, 0.28515E+00_JPRB ,&
&  0.29128E+00_JPRB, 0.29506E+00_JPRB, 0.29234E+00_JPRB, 0.28891E+00_JPRB, 0.28879E+00_JPRB, 0.29093E+00_JPRB, 0.29348E+00_JPRB ,&
&  0.29591E+00_JPRB, 0.29834E+00_JPRB, 0.30028E+00_JPRB, 0.30025E+00_JPRB, 0.29764E+00_JPRB, 0.29271E+00_JPRB, 0.28594E+00_JPRB ,&
&  0.27839E+00_JPRB, 0.27069E+00_JPRB, 0.26319E+00_JPRB, 0.25562E+00_JPRB, 0.24910E+00_JPRB, 0.24293E+00_JPRB, 0.23736E+00_JPRB ,&
&  0.23216E+00_JPRB, 0.22722E+00_JPRB, 0.22343E+00_JPRB, 0.22097E+00_JPRB, 0.22002E+00_JPRB, 0.21864E+00_JPRB, 0.21845E+00_JPRB ,&
&  0.21776E+00_JPRB, 0.21749E+00_JPRB, 0.21751E+00_JPRB, 0.21824E+00_JPRB, 0.21967E+00_JPRB, 0.22020E+00_JPRB, 0.21933E+00_JPRB ,&
&  0.21883E+00_JPRB, 0.22008E+00_JPRB, 0.21986E+00_JPRB, 0.21747E+00_JPRB, 0.21453E+00_JPRB, 0.21390E+00_JPRB, 0.21545E+00_JPRB ,&
&  0.21834E+00_JPRB, 0.22130E+00_JPRB, 0.22388E+00_JPRB, 0.22614E+00_JPRB, 0.22802E+00_JPRB, 0.23032E+00_JPRB, 0.23243E+00_JPRB ,&
&  0.23422E+00_JPRB, 0.23519E+00_JPRB, 0.23681E+00_JPRB, 0.23853E+00_JPRB, 0.24031E+00_JPRB, 0.24160E+00_JPRB, 0.24155E+00_JPRB ,&
&  0.24120E+00_JPRB, 0.24296E+00_JPRB, 0.24803E+00_JPRB, 0.25558E+00_JPRB, 0.26052E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.24411E+00_JPRB, 0.24651E+00_JPRB, 0.24778E+00_JPRB, 0.24754E+00_JPRB, 0.24685E+00_JPRB, 0.24747E+00_JPRB, 0.24798E+00_JPRB ,&
&  0.24866E+00_JPRB, 0.24877E+00_JPRB, 0.24742E+00_JPRB, 0.24623E+00_JPRB, 0.24783E+00_JPRB, 0.24744E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.24529E+00_JPRB, 0.24320E+00_JPRB, 0.24021E+00_JPRB, 0.23682E+00_JPRB, 0.23278E+00_JPRB, 0.23043E+00_JPRB, 0.23090E+00_JPRB ,&
&  0.23258E+00_JPRB, 0.23324E+00_JPRB, 0.23404E+00_JPRB, 0.23505E+00_JPRB, 0.23719E+00_JPRB, 0.23997E+00_JPRB, 0.24204E+00_JPRB ,&
&  0.24368E+00_JPRB, 0.24678E+00_JPRB, 0.24873E+00_JPRB, 0.24790E+00_JPRB, 0.25214E+00_JPRB, 0.25640E+00_JPRB, 0.26039E+00_JPRB ,&
&  0.26429E+00_JPRB, 0.26663E+00_JPRB, 0.26671E+00_JPRB, 0.26556E+00_JPRB, 0.26469E+00_JPRB, 0.26574E+00_JPRB, 0.26640E+00_JPRB ,&
&  0.26727E+00_JPRB, 0.27016E+00_JPRB, 0.27353E+00_JPRB, 0.27495E+00_JPRB, 0.27368E+00_JPRB, 0.26948E+00_JPRB, 0.26259E+00_JPRB ,&
&  0.25350E+00_JPRB, 0.24379E+00_JPRB, 0.23577E+00_JPRB, 0.22934E+00_JPRB, 0.22441E+00_JPRB, 0.22049E+00_JPRB, 0.21729E+00_JPRB ,&
&  0.21372E+00_JPRB, 0.21114E+00_JPRB, 0.21091E+00_JPRB, 0.21139E+00_JPRB, 0.21129E+00_JPRB, 0.20938E+00_JPRB, 0.20803E+00_JPRB ,&
&  0.20683E+00_JPRB, 0.20726E+00_JPRB, 0.20821E+00_JPRB, 0.20853E+00_JPRB, 0.20868E+00_JPRB, 0.20848E+00_JPRB, 0.20829E+00_JPRB ,&
&  0.20920E+00_JPRB, 0.20921E+00_JPRB, 0.20811E+00_JPRB, 0.20624E+00_JPRB, 0.20463E+00_JPRB, 0.20439E+00_JPRB, 0.20481E+00_JPRB ,&
&  0.20563E+00_JPRB, 0.20710E+00_JPRB, 0.20909E+00_JPRB, 0.21111E+00_JPRB, 0.21269E+00_JPRB, 0.21343E+00_JPRB, 0.21377E+00_JPRB ,&
&  0.21407E+00_JPRB, 0.21409E+00_JPRB, 0.21531E+00_JPRB, 0.21871E+00_JPRB, 0.22211E+00_JPRB, 0.22473E+00_JPRB, 0.22612E+00_JPRB ,&
&  0.22569E+00_JPRB, 0.22642E+00_JPRB, 0.22974E+00_JPRB, 0.23590E+00_JPRB, 0.24048E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.22936E+00_JPRB, 0.23156E+00_JPRB, 0.23227E+00_JPRB, 0.23183E+00_JPRB, 0.23125E+00_JPRB, 0.23213E+00_JPRB, 0.23340E+00_JPRB ,&
&  0.23409E+00_JPRB, 0.23420E+00_JPRB, 0.23324E+00_JPRB, 0.23134E+00_JPRB, 0.22993E+00_JPRB, 0.23018E+00_JPRB, 0.22865E+00_JPRB ,&
&  0.22721E+00_JPRB, 0.22740E+00_JPRB, 0.22608E+00_JPRB, 0.22463E+00_JPRB, 0.22244E+00_JPRB, 0.22132E+00_JPRB, 0.22210E+00_JPRB ,&
&  0.22431E+00_JPRB, 0.22492E+00_JPRB, 0.22556E+00_JPRB, 0.22643E+00_JPRB, 0.22809E+00_JPRB, 0.23012E+00_JPRB, 0.23140E+00_JPRB ,&
&  0.23227E+00_JPRB, 0.23126E+00_JPRB, 0.22980E+00_JPRB, 0.23240E+00_JPRB, 0.23582E+00_JPRB, 0.24028E+00_JPRB, 0.24336E+00_JPRB ,&
&  0.24604E+00_JPRB, 0.24794E+00_JPRB, 0.24877E+00_JPRB, 0.24866E+00_JPRB, 0.24899E+00_JPRB, 0.25091E+00_JPRB, 0.25240E+00_JPRB ,&
&  0.25299E+00_JPRB, 0.25286E+00_JPRB, 0.25163E+00_JPRB, 0.25048E+00_JPRB, 0.24729E+00_JPRB, 0.24222E+00_JPRB, 0.23437E+00_JPRB ,&
&  0.22606E+00_JPRB, 0.21926E+00_JPRB, 0.21409E+00_JPRB, 0.20997E+00_JPRB, 0.20712E+00_JPRB, 0.20683E+00_JPRB, 0.20671E+00_JPRB ,&
&  0.20614E+00_JPRB, 0.20558E+00_JPRB, 0.20685E+00_JPRB, 0.20738E+00_JPRB, 0.20285E+00_JPRB, 0.19788E+00_JPRB, 0.19321E+00_JPRB ,&
&  0.19204E+00_JPRB, 0.19272E+00_JPRB, 0.19375E+00_JPRB, 0.19614E+00_JPRB, 0.19990E+00_JPRB, 0.20074E+00_JPRB, 0.19958E+00_JPRB ,&
&  0.19384E+00_JPRB, 0.18877E+00_JPRB, 0.18573E+00_JPRB, 0.18321E+00_JPRB, 0.18172E+00_JPRB, 0.18183E+00_JPRB, 0.18264E+00_JPRB ,&
&  0.18494E+00_JPRB, 0.18673E+00_JPRB, 0.18921E+00_JPRB, 0.19054E+00_JPRB, 0.19037E+00_JPRB, 0.19019E+00_JPRB, 0.18951E+00_JPRB ,&
&  0.19104E+00_JPRB, 0.19565E+00_JPRB, 0.20253E+00_JPRB, 0.20554E+00_JPRB, 0.20983E+00_JPRB, 0.21275E+00_JPRB, 0.21409E+00_JPRB ,&
&  0.21495E+00_JPRB, 0.21540E+00_JPRB, 0.21577E+00_JPRB, 0.21995E+00_JPRB, 0.22572E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.21722E+00_JPRB, 0.21893E+00_JPRB, 0.21949E+00_JPRB, 0.21941E+00_JPRB, 0.21917E+00_JPRB, 0.21957E+00_JPRB, 0.22003E+00_JPRB ,&
&  0.21996E+00_JPRB, 0.21922E+00_JPRB, 0.21802E+00_JPRB, 0.21600E+00_JPRB, 0.21350E+00_JPRB, 0.21203E+00_JPRB, 0.21307E+00_JPRB ,&
&  0.21425E+00_JPRB, 0.21451E+00_JPRB, 0.21492E+00_JPRB, 0.21505E+00_JPRB, 0.21487E+00_JPRB, 0.21531E+00_JPRB, 0.21661E+00_JPRB ,&
&  0.21793E+00_JPRB, 0.21868E+00_JPRB, 0.21999E+00_JPRB, 0.22105E+00_JPRB, 0.22310E+00_JPRB, 0.22495E+00_JPRB, 0.22551E+00_JPRB ,&
&  0.22255E+00_JPRB, 0.21481E+00_JPRB, 0.20973E+00_JPRB, 0.21233E+00_JPRB, 0.22013E+00_JPRB, 0.22553E+00_JPRB, 0.22897E+00_JPRB ,&
&  0.23121E+00_JPRB, 0.23227E+00_JPRB, 0.23248E+00_JPRB, 0.23377E+00_JPRB, 0.23583E+00_JPRB, 0.23788E+00_JPRB, 0.23758E+00_JPRB ,&
&  0.23565E+00_JPRB, 0.23401E+00_JPRB, 0.23159E+00_JPRB, 0.22731E+00_JPRB, 0.22140E+00_JPRB, 0.21555E+00_JPRB, 0.21047E+00_JPRB ,&
&  0.20584E+00_JPRB, 0.20136E+00_JPRB, 0.19767E+00_JPRB, 0.19530E+00_JPRB, 0.19463E+00_JPRB, 0.19601E+00_JPRB, 0.19562E+00_JPRB ,&
&  0.19578E+00_JPRB, 0.19391E+00_JPRB, 0.19171E+00_JPRB, 0.18901E+00_JPRB, 0.18666E+00_JPRB, 0.18238E+00_JPRB, 0.17700E+00_JPRB ,&
&  0.17426E+00_JPRB, 0.17682E+00_JPRB, 0.18380E+00_JPRB, 0.18916E+00_JPRB, 0.18774E+00_JPRB, 0.18116E+00_JPRB, 0.17769E+00_JPRB ,&
&  0.17377E+00_JPRB, 0.16962E+00_JPRB, 0.16693E+00_JPRB, 0.16630E+00_JPRB, 0.16648E+00_JPRB, 0.16669E+00_JPRB, 0.16771E+00_JPRB ,&
&  0.16939E+00_JPRB, 0.17088E+00_JPRB, 0.17170E+00_JPRB, 0.17267E+00_JPRB, 0.17310E+00_JPRB, 0.17321E+00_JPRB, 0.17335E+00_JPRB ,&
&  0.17507E+00_JPRB, 0.17862E+00_JPRB, 0.18331E+00_JPRB, 0.18852E+00_JPRB, 0.19248E+00_JPRB, 0.19609E+00_JPRB, 0.19920E+00_JPRB ,&
&  0.20226E+00_JPRB, 0.20371E+00_JPRB, 0.20678E+00_JPRB, 0.21098E+00_JPRB, 0.21440E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.20367E+00_JPRB, 0.20745E+00_JPRB, 0.20887E+00_JPRB, 0.20880E+00_JPRB, 0.20768E+00_JPRB, 0.20668E+00_JPRB, 0.20642E+00_JPRB ,&
&  0.20621E+00_JPRB, 0.20506E+00_JPRB, 0.20420E+00_JPRB, 0.20360E+00_JPRB, 0.20292E+00_JPRB, 0.20210E+00_JPRB, 0.20243E+00_JPRB ,&
&  0.20377E+00_JPRB, 0.20554E+00_JPRB, 0.20692E+00_JPRB, 0.20765E+00_JPRB, 0.20736E+00_JPRB, 0.20779E+00_JPRB, 0.20867E+00_JPRB ,&
&  0.20868E+00_JPRB, 0.20879E+00_JPRB, 0.21198E+00_JPRB, 0.21222E+00_JPRB, 0.21311E+00_JPRB, 0.21309E+00_JPRB, 0.21314E+00_JPRB ,&
&  0.20818E+00_JPRB, 0.20288E+00_JPRB, 0.20037E+00_JPRB, 0.20266E+00_JPRB, 0.21062E+00_JPRB, 0.21207E+00_JPRB, 0.21568E+00_JPRB ,&
&  0.21782E+00_JPRB, 0.21882E+00_JPRB, 0.22033E+00_JPRB, 0.22162E+00_JPRB, 0.22314E+00_JPRB, 0.22243E+00_JPRB, 0.22028E+00_JPRB ,&
&  0.21740E+00_JPRB, 0.21640E+00_JPRB, 0.21310E+00_JPRB, 0.19881E+00_JPRB, 0.19019E+00_JPRB, 0.18529E+00_JPRB, 0.18012E+00_JPRB ,&
&  0.17536E+00_JPRB, 0.17285E+00_JPRB, 0.17229E+00_JPRB, 0.17088E+00_JPRB, 0.17177E+00_JPRB, 0.17398E+00_JPRB, 0.17489E+00_JPRB ,&
&  0.17456E+00_JPRB, 0.17500E+00_JPRB, 0.17470E+00_JPRB, 0.17364E+00_JPRB, 0.17171E+00_JPRB, 0.17048E+00_JPRB, 0.16977E+00_JPRB ,&
&  0.17027E+00_JPRB, 0.16991E+00_JPRB, 0.17390E+00_JPRB, 0.17501E+00_JPRB, 0.16962E+00_JPRB, 0.16852E+00_JPRB, 0.16694E+00_JPRB ,&
&  0.16168E+00_JPRB, 0.15874E+00_JPRB, 0.15712E+00_JPRB, 0.15650E+00_JPRB, 0.15755E+00_JPRB, 0.15901E+00_JPRB, 0.15971E+00_JPRB ,&
&  0.15993E+00_JPRB, 0.16104E+00_JPRB, 0.16161E+00_JPRB, 0.16224E+00_JPRB, 0.16266E+00_JPRB, 0.16357E+00_JPRB, 0.16488E+00_JPRB ,&
&  0.16651E+00_JPRB, 0.16946E+00_JPRB, 0.17236E+00_JPRB, 0.17560E+00_JPRB, 0.17750E+00_JPRB, 0.17898E+00_JPRB, 0.18244E+00_JPRB ,&
&  0.18437E+00_JPRB, 0.19156E+00_JPRB, 0.20196E+00_JPRB, 0.20440E+00_JPRB, 0.19900E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.19608E+00_JPRB, 0.19790E+00_JPRB, 0.19937E+00_JPRB, 0.19838E+00_JPRB, 0.19853E+00_JPRB, 0.19844E+00_JPRB, 0.19848E+00_JPRB ,&
&  0.19838E+00_JPRB, 0.19773E+00_JPRB, 0.19799E+00_JPRB, 0.19606E+00_JPRB, 0.19495E+00_JPRB, 0.19154E+00_JPRB, 0.19054E+00_JPRB ,&
&  0.18884E+00_JPRB, 0.18862E+00_JPRB, 0.18904E+00_JPRB, 0.19041E+00_JPRB, 0.19090E+00_JPRB, 0.19229E+00_JPRB, 0.19147E+00_JPRB ,&
&  0.19174E+00_JPRB, 0.19188E+00_JPRB, 0.19060E+00_JPRB, 0.19023E+00_JPRB, 0.19060E+00_JPRB, 0.19094E+00_JPRB, 0.19126E+00_JPRB ,&
&  0.19335E+00_JPRB, 0.19566E+00_JPRB, 0.19787E+00_JPRB, 0.19913E+00_JPRB, 0.20398E+00_JPRB, 0.20538E+00_JPRB, 0.20752E+00_JPRB ,&
&  0.20861E+00_JPRB, 0.20998E+00_JPRB, 0.21106E+00_JPRB, 0.21041E+00_JPRB, 0.20918E+00_JPRB, 0.20533E+00_JPRB, 0.20394E+00_JPRB ,&
&  0.19396E+00_JPRB, 0.18709E+00_JPRB, 0.18032E+00_JPRB, 0.17421E+00_JPRB, 0.16965E+00_JPRB, 0.16652E+00_JPRB, 0.16399E+00_JPRB ,&
&  0.16190E+00_JPRB, 0.15986E+00_JPRB, 0.15762E+00_JPRB, 0.15774E+00_JPRB, 0.15854E+00_JPRB, 0.15978E+00_JPRB, 0.16103E+00_JPRB ,&
&  0.16250E+00_JPRB, 0.16335E+00_JPRB, 0.16372E+00_JPRB, 0.16447E+00_JPRB, 0.16460E+00_JPRB, 0.16492E+00_JPRB, 0.16511E+00_JPRB ,&
&  0.16640E+00_JPRB, 0.16676E+00_JPRB, 0.16673E+00_JPRB, 0.16825E+00_JPRB, 0.16643E+00_JPRB, 0.16366E+00_JPRB, 0.16069E+00_JPRB ,&
&  0.15767E+00_JPRB, 0.15579E+00_JPRB, 0.15419E+00_JPRB, 0.15357E+00_JPRB, 0.15398E+00_JPRB, 0.15475E+00_JPRB, 0.15575E+00_JPRB ,&
&  0.15595E+00_JPRB, 0.15659E+00_JPRB, 0.15709E+00_JPRB, 0.15804E+00_JPRB, 0.15904E+00_JPRB, 0.16019E+00_JPRB, 0.16143E+00_JPRB ,&
&  0.16329E+00_JPRB, 0.16547E+00_JPRB, 0.16761E+00_JPRB, 0.16980E+00_JPRB, 0.17328E+00_JPRB, 0.17738E+00_JPRB, 0.18387E+00_JPRB ,&
&  0.18940E+00_JPRB, 0.19607E+00_JPRB, 0.19413E+00_JPRB, 0.19716E+00_JPRB, 0.19748E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.19098E+00_JPRB, 0.19178E+00_JPRB, 0.18980E+00_JPRB, 0.19011E+00_JPRB, 0.19082E+00_JPRB, 0.19119E+00_JPRB, 0.19167E+00_JPRB ,&
&  0.18917E+00_JPRB, 0.18837E+00_JPRB, 0.18606E+00_JPRB, 0.18409E+00_JPRB, 0.18164E+00_JPRB, 0.17911E+00_JPRB, 0.17720E+00_JPRB ,&
&  0.17529E+00_JPRB, 0.17462E+00_JPRB, 0.17372E+00_JPRB, 0.17461E+00_JPRB, 0.17467E+00_JPRB, 0.17643E+00_JPRB, 0.17786E+00_JPRB ,&
&  0.17862E+00_JPRB, 0.17972E+00_JPRB, 0.17834E+00_JPRB, 0.17797E+00_JPRB, 0.17966E+00_JPRB, 0.18118E+00_JPRB, 0.18379E+00_JPRB ,&
&  0.18650E+00_JPRB, 0.18970E+00_JPRB, 0.19491E+00_JPRB, 0.19680E+00_JPRB, 0.19915E+00_JPRB, 0.20058E+00_JPRB, 0.20052E+00_JPRB ,&
&  0.20024E+00_JPRB, 0.20009E+00_JPRB, 0.20094E+00_JPRB, 0.20068E+00_JPRB, 0.19378E+00_JPRB, 0.18854E+00_JPRB, 0.18297E+00_JPRB ,&
&  0.17786E+00_JPRB, 0.17294E+00_JPRB, 0.16853E+00_JPRB, 0.16556E+00_JPRB, 0.16238E+00_JPRB, 0.16011E+00_JPRB, 0.15833E+00_JPRB ,&
&  0.15654E+00_JPRB, 0.15554E+00_JPRB, 0.15420E+00_JPRB, 0.15380E+00_JPRB, 0.15411E+00_JPRB, 0.15493E+00_JPRB, 0.15608E+00_JPRB ,&
&  0.15745E+00_JPRB, 0.15841E+00_JPRB, 0.15961E+00_JPRB, 0.16071E+00_JPRB, 0.16150E+00_JPRB, 0.16198E+00_JPRB, 0.16167E+00_JPRB ,&
&  0.16187E+00_JPRB, 0.16213E+00_JPRB, 0.16167E+00_JPRB, 0.16163E+00_JPRB, 0.16047E+00_JPRB, 0.15911E+00_JPRB, 0.15771E+00_JPRB ,&
&  0.15582E+00_JPRB, 0.15437E+00_JPRB, 0.15386E+00_JPRB, 0.15358E+00_JPRB, 0.15432E+00_JPRB, 0.15542E+00_JPRB, 0.15615E+00_JPRB ,&
&  0.15650E+00_JPRB, 0.15639E+00_JPRB, 0.15641E+00_JPRB, 0.15763E+00_JPRB, 0.15889E+00_JPRB, 0.16050E+00_JPRB, 0.16218E+00_JPRB ,&
&  0.16271E+00_JPRB, 0.16408E+00_JPRB, 0.16526E+00_JPRB, 0.16674E+00_JPRB, 0.17175E+00_JPRB, 0.17563E+00_JPRB, 0.18303E+00_JPRB ,&
&  0.18640E+00_JPRB, 0.18671E+00_JPRB, 0.18551E+00_JPRB, 0.19007E+00_JPRB, 0.19059E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.18910E+00_JPRB, 0.19163E+00_JPRB, 0.19025E+00_JPRB, 0.19005E+00_JPRB, 0.18939E+00_JPRB, 0.18920E+00_JPRB, 0.18804E+00_JPRB ,&
&  0.18780E+00_JPRB, 0.18592E+00_JPRB, 0.18557E+00_JPRB, 0.18388E+00_JPRB, 0.18213E+00_JPRB, 0.17969E+00_JPRB, 0.17727E+00_JPRB ,&
&  0.17485E+00_JPRB, 0.17274E+00_JPRB, 0.17132E+00_JPRB, 0.17072E+00_JPRB, 0.16954E+00_JPRB, 0.16959E+00_JPRB, 0.17169E+00_JPRB ,&
&  0.17217E+00_JPRB, 0.17331E+00_JPRB, 0.17339E+00_JPRB, 0.17464E+00_JPRB, 0.17623E+00_JPRB, 0.17679E+00_JPRB, 0.17905E+00_JPRB ,&
&  0.18012E+00_JPRB, 0.18170E+00_JPRB, 0.18779E+00_JPRB, 0.19109E+00_JPRB, 0.19291E+00_JPRB, 0.19571E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19451E+00_JPRB, 0.19008E+00_JPRB, 0.19319E+00_JPRB, 0.19062E+00_JPRB, 0.18411E+00_JPRB, 0.18132E+00_JPRB, 0.17602E+00_JPRB ,&
&  0.17305E+00_JPRB, 0.16938E+00_JPRB, 0.16663E+00_JPRB, 0.16451E+00_JPRB, 0.16144E+00_JPRB, 0.15948E+00_JPRB, 0.15727E+00_JPRB ,&
&  0.15523E+00_JPRB, 0.15438E+00_JPRB, 0.15299E+00_JPRB, 0.15285E+00_JPRB, 0.15302E+00_JPRB, 0.15347E+00_JPRB, 0.15422E+00_JPRB ,&
&  0.15519E+00_JPRB, 0.15611E+00_JPRB, 0.15696E+00_JPRB, 0.15768E+00_JPRB, 0.15829E+00_JPRB, 0.15866E+00_JPRB, 0.15852E+00_JPRB ,&
&  0.15861E+00_JPRB, 0.15850E+00_JPRB, 0.15801E+00_JPRB, 0.15777E+00_JPRB, 0.15681E+00_JPRB, 0.15639E+00_JPRB, 0.15596E+00_JPRB ,&
&  0.15577E+00_JPRB, 0.15579E+00_JPRB, 0.15571E+00_JPRB, 0.15591E+00_JPRB, 0.15620E+00_JPRB, 0.15691E+00_JPRB, 0.15749E+00_JPRB ,&
&  0.15795E+00_JPRB, 0.15885E+00_JPRB, 0.15939E+00_JPRB, 0.16013E+00_JPRB, 0.16119E+00_JPRB, 0.16165E+00_JPRB, 0.16289E+00_JPRB ,&
&  0.16395E+00_JPRB, 0.16519E+00_JPRB, 0.16750E+00_JPRB, 0.16980E+00_JPRB, 0.17311E+00_JPRB, 0.17721E+00_JPRB, 0.17873E+00_JPRB ,&
&  0.18002E+00_JPRB, 0.18083E+00_JPRB, 0.17915E+00_JPRB, 0.18423E+00_JPRB, 0.18663E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.18254E+00_JPRB, 0.18445E+00_JPRB, 0.18568E+00_JPRB, 0.18652E+00_JPRB, 0.18783E+00_JPRB, 0.18843E+00_JPRB, 0.18897E+00_JPRB ,&
&  0.18710E+00_JPRB, 0.18652E+00_JPRB, 0.18549E+00_JPRB, 0.18420E+00_JPRB, 0.18328E+00_JPRB, 0.18089E+00_JPRB, 0.17853E+00_JPRB ,&
&  0.17593E+00_JPRB, 0.17362E+00_JPRB, 0.17226E+00_JPRB, 0.17093E+00_JPRB, 0.16958E+00_JPRB, 0.16846E+00_JPRB, 0.17045E+00_JPRB ,&
&  0.17164E+00_JPRB, 0.17237E+00_JPRB, 0.17366E+00_JPRB, 0.17347E+00_JPRB, 0.17477E+00_JPRB, 0.17684E+00_JPRB, 0.17962E+00_JPRB ,&
&  0.18134E+00_JPRB, 0.18423E+00_JPRB, 0.18593E+00_JPRB, 0.18679E+00_JPRB, 0.18800E+00_JPRB, 0.18884E+00_JPRB, 0.18856E+00_JPRB ,&
&  0.18725E+00_JPRB, 0.18550E+00_JPRB, 0.18343E+00_JPRB, 0.18076E+00_JPRB, 0.17784E+00_JPRB, 0.17504E+00_JPRB, 0.17225E+00_JPRB ,&
&  0.17021E+00_JPRB, 0.16836E+00_JPRB, 0.16639E+00_JPRB, 0.16459E+00_JPRB, 0.16288E+00_JPRB, 0.16132E+00_JPRB, 0.15976E+00_JPRB ,&
&  0.15853E+00_JPRB, 0.15756E+00_JPRB, 0.15636E+00_JPRB, 0.15574E+00_JPRB, 0.15512E+00_JPRB, 0.15486E+00_JPRB, 0.15473E+00_JPRB ,&
&  0.15516E+00_JPRB, 0.15569E+00_JPRB, 0.15622E+00_JPRB, 0.15653E+00_JPRB, 0.15682E+00_JPRB, 0.15697E+00_JPRB, 0.15725E+00_JPRB ,&
&  0.15750E+00_JPRB, 0.15779E+00_JPRB, 0.15738E+00_JPRB, 0.15701E+00_JPRB, 0.15650E+00_JPRB, 0.15593E+00_JPRB, 0.15576E+00_JPRB ,&
&  0.15605E+00_JPRB, 0.15630E+00_JPRB, 0.15613E+00_JPRB, 0.15642E+00_JPRB, 0.15693E+00_JPRB, 0.15745E+00_JPRB, 0.15878E+00_JPRB ,&
&  0.15949E+00_JPRB, 0.16017E+00_JPRB, 0.16081E+00_JPRB, 0.16131E+00_JPRB, 0.16208E+00_JPRB, 0.16269E+00_JPRB, 0.16347E+00_JPRB ,&
&  0.16441E+00_JPRB, 0.16533E+00_JPRB, 0.16661E+00_JPRB, 0.16824E+00_JPRB, 0.17217E+00_JPRB, 0.17472E+00_JPRB, 0.17689E+00_JPRB ,&
&  0.17784E+00_JPRB, 0.17417E+00_JPRB, 0.18130E+00_JPRB, 0.18034E+00_JPRB, 0.18005E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.17433E+00_JPRB, 0.17612E+00_JPRB, 0.17751E+00_JPRB, 0.17940E+00_JPRB, 0.17492E+00_JPRB, 0.18177E+00_JPRB, 0.17886E+00_JPRB ,&
&  0.17585E+00_JPRB, 0.17106E+00_JPRB, 0.17126E+00_JPRB, 0.17049E+00_JPRB, 0.16939E+00_JPRB, 0.16887E+00_JPRB, 0.16881E+00_JPRB ,&
&  0.16893E+00_JPRB, 0.16879E+00_JPRB, 0.16931E+00_JPRB, 0.16971E+00_JPRB, 0.17002E+00_JPRB, 0.17026E+00_JPRB, 0.17061E+00_JPRB ,&
&  0.17119E+00_JPRB, 0.17215E+00_JPRB, 0.17314E+00_JPRB, 0.17125E+00_JPRB, 0.17104E+00_JPRB, 0.17131E+00_JPRB, 0.17193E+00_JPRB ,&
&  0.17236E+00_JPRB, 0.17346E+00_JPRB, 0.17431E+00_JPRB, 0.17493E+00_JPRB, 0.17669E+00_JPRB, 0.17680E+00_JPRB, 0.17666E+00_JPRB ,&
&  0.17627E+00_JPRB, 0.17545E+00_JPRB, 0.17427E+00_JPRB, 0.17317E+00_JPRB, 0.17204E+00_JPRB, 0.17055E+00_JPRB, 0.16912E+00_JPRB ,&
&  0.16794E+00_JPRB, 0.16699E+00_JPRB, 0.16574E+00_JPRB, 0.16454E+00_JPRB, 0.16345E+00_JPRB, 0.16240E+00_JPRB, 0.16150E+00_JPRB ,&
&  0.16084E+00_JPRB, 0.16029E+00_JPRB, 0.15972E+00_JPRB, 0.15927E+00_JPRB, 0.15881E+00_JPRB, 0.15838E+00_JPRB, 0.15797E+00_JPRB ,&
&  0.15813E+00_JPRB, 0.15826E+00_JPRB, 0.15839E+00_JPRB, 0.15852E+00_JPRB, 0.15895E+00_JPRB, 0.15904E+00_JPRB, 0.15915E+00_JPRB ,&
&  0.15920E+00_JPRB, 0.15911E+00_JPRB, 0.15899E+00_JPRB, 0.15877E+00_JPRB, 0.15852E+00_JPRB, 0.15821E+00_JPRB, 0.15807E+00_JPRB ,&
&  0.15806E+00_JPRB, 0.15817E+00_JPRB, 0.15851E+00_JPRB, 0.15874E+00_JPRB, 0.15897E+00_JPRB, 0.15917E+00_JPRB, 0.15963E+00_JPRB ,&
&  0.16009E+00_JPRB, 0.16055E+00_JPRB, 0.16099E+00_JPRB, 0.16087E+00_JPRB, 0.16111E+00_JPRB, 0.16133E+00_JPRB, 0.16153E+00_JPRB ,&
&  0.16165E+00_JPRB, 0.16211E+00_JPRB, 0.16285E+00_JPRB, 0.16388E+00_JPRB, 0.16577E+00_JPRB, 0.16737E+00_JPRB, 0.16886E+00_JPRB ,&
&  0.17046E+00_JPRB, 0.17368E+00_JPRB, 0.17493E+00_JPRB, 0.17532E+00_JPRB, 0.17527E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.16486E+00_JPRB, 0.16568E+00_JPRB, 0.16625E+00_JPRB, 0.16671E+00_JPRB, 0.16651E+00_JPRB, 0.16656E+00_JPRB, 0.16660E+00_JPRB ,&
&  0.16656E+00_JPRB, 0.16650E+00_JPRB, 0.16677E+00_JPRB, 0.16701E+00_JPRB, 0.16728E+00_JPRB, 0.16757E+00_JPRB, 0.16792E+00_JPRB ,&
&  0.16806E+00_JPRB, 0.16838E+00_JPRB, 0.16717E+00_JPRB, 0.16617E+00_JPRB, 0.16552E+00_JPRB, 0.16513E+00_JPRB, 0.16491E+00_JPRB ,&
&  0.16473E+00_JPRB, 0.16463E+00_JPRB, 0.16450E+00_JPRB, 0.16539E+00_JPRB, 0.16596E+00_JPRB, 0.16648E+00_JPRB, 0.16688E+00_JPRB ,&
&  0.16703E+00_JPRB, 0.16727E+00_JPRB, 0.16747E+00_JPRB, 0.16768E+00_JPRB, 0.16793E+00_JPRB, 0.16791E+00_JPRB, 0.16780E+00_JPRB ,&
&  0.16761E+00_JPRB, 0.16730E+00_JPRB, 0.16693E+00_JPRB, 0.16658E+00_JPRB, 0.16630E+00_JPRB, 0.16597E+00_JPRB, 0.16560E+00_JPRB ,&
&  0.16525E+00_JPRB, 0.16483E+00_JPRB, 0.16444E+00_JPRB, 0.16407E+00_JPRB, 0.16371E+00_JPRB, 0.16340E+00_JPRB, 0.16306E+00_JPRB ,&
&  0.16283E+00_JPRB, 0.16262E+00_JPRB, 0.16240E+00_JPRB, 0.16225E+00_JPRB, 0.16200E+00_JPRB, 0.16191E+00_JPRB, 0.16169E+00_JPRB ,&
&  0.16162E+00_JPRB, 0.16154E+00_JPRB, 0.16148E+00_JPRB, 0.16141E+00_JPRB, 0.16141E+00_JPRB, 0.16139E+00_JPRB, 0.16138E+00_JPRB ,&
&  0.16135E+00_JPRB, 0.16148E+00_JPRB, 0.16136E+00_JPRB, 0.16120E+00_JPRB, 0.16102E+00_JPRB, 0.16084E+00_JPRB, 0.16067E+00_JPRB ,&
&  0.16053E+00_JPRB, 0.16037E+00_JPRB, 0.16028E+00_JPRB, 0.16029E+00_JPRB, 0.16033E+00_JPRB, 0.16038E+00_JPRB, 0.16046E+00_JPRB ,&
&  0.16054E+00_JPRB, 0.16064E+00_JPRB, 0.16075E+00_JPRB, 0.16070E+00_JPRB, 0.16072E+00_JPRB, 0.16074E+00_JPRB, 0.16076E+00_JPRB ,&
&  0.16077E+00_JPRB, 0.16082E+00_JPRB, 0.16090E+00_JPRB, 0.16102E+00_JPRB, 0.16128E+00_JPRB, 0.16138E+00_JPRB, 0.16155E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16217E+00_JPRB, 0.16252E+00_JPRB, 0.16290E+00_JPRB, 0.16335E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB, 0.16238E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB, 0.87096E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.87817E+00_JPRB, 0.87681E+00_JPRB, 0.87564E+00_JPRB, 0.87448E+00_JPRB, 0.87338E+00_JPRB, 0.87253E+00_JPRB, 0.87183E+00_JPRB ,&
&  0.87104E+00_JPRB, 0.87023E+00_JPRB, 0.86924E+00_JPRB, 0.86837E+00_JPRB, 0.86764E+00_JPRB, 0.86685E+00_JPRB, 0.86624E+00_JPRB ,&
&  0.86542E+00_JPRB, 0.86469E+00_JPRB, 0.86407E+00_JPRB, 0.86299E+00_JPRB, 0.86209E+00_JPRB, 0.86119E+00_JPRB, 0.85954E+00_JPRB ,&
&  0.85737E+00_JPRB, 0.85521E+00_JPRB, 0.85272E+00_JPRB, 0.85041E+00_JPRB, 0.84851E+00_JPRB, 0.84612E+00_JPRB, 0.84337E+00_JPRB ,&
&  0.84085E+00_JPRB, 0.83897E+00_JPRB, 0.83750E+00_JPRB, 0.83633E+00_JPRB, 0.83547E+00_JPRB, 0.83477E+00_JPRB, 0.83401E+00_JPRB ,&
&  0.83345E+00_JPRB, 0.83344E+00_JPRB, 0.83396E+00_JPRB, 0.83481E+00_JPRB, 0.83601E+00_JPRB, 0.83762E+00_JPRB, 0.84031E+00_JPRB ,&
&  0.84323E+00_JPRB, 0.84623E+00_JPRB, 0.84937E+00_JPRB, 0.85221E+00_JPRB, 0.85481E+00_JPRB, 0.85710E+00_JPRB, 0.85894E+00_JPRB ,&
&  0.86096E+00_JPRB, 0.86298E+00_JPRB, 0.86511E+00_JPRB, 0.86677E+00_JPRB, 0.86817E+00_JPRB, 0.86941E+00_JPRB, 0.87076E+00_JPRB ,&
&  0.87145E+00_JPRB, 0.87195E+00_JPRB, 0.87262E+00_JPRB, 0.87354E+00_JPRB, 0.87425E+00_JPRB, 0.87482E+00_JPRB, 0.87550E+00_JPRB ,&
&  0.87646E+00_JPRB, 0.87731E+00_JPRB, 0.87795E+00_JPRB, 0.87858E+00_JPRB, 0.87946E+00_JPRB, 0.88066E+00_JPRB, 0.88206E+00_JPRB ,&
&  0.88337E+00_JPRB, 0.88469E+00_JPRB, 0.88586E+00_JPRB, 0.88677E+00_JPRB, 0.88719E+00_JPRB, 0.88711E+00_JPRB, 0.88610E+00_JPRB ,&
&  0.88482E+00_JPRB, 0.88333E+00_JPRB, 0.88230E+00_JPRB, 0.88196E+00_JPRB, 0.88156E+00_JPRB, 0.88141E+00_JPRB, 0.88168E+00_JPRB ,&
&  0.88239E+00_JPRB, 0.88317E+00_JPRB, 0.88403E+00_JPRB, 0.88509E+00_JPRB, 0.88585E+00_JPRB, 0.88609E+00_JPRB, 0.88614E+00_JPRB ,&
&  0.88584E+00_JPRB, 0.88496E+00_JPRB, 0.88349E+00_JPRB, 0.88173E+00_JPRB, 0.87982E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.87245E+00_JPRB, 0.86841E+00_JPRB, 0.86551E+00_JPRB, 0.86343E+00_JPRB, 0.86204E+00_JPRB, 0.86064E+00_JPRB, 0.85934E+00_JPRB ,&
&  0.85803E+00_JPRB, 0.85835E+00_JPRB, 0.85823E+00_JPRB, 0.85815E+00_JPRB, 0.85780E+00_JPRB, 0.85860E+00_JPRB, 0.85891E+00_JPRB ,&
&  0.85830E+00_JPRB, 0.85614E+00_JPRB, 0.85424E+00_JPRB, 0.85184E+00_JPRB, 0.85156E+00_JPRB, 0.85107E+00_JPRB, 0.85157E+00_JPRB ,&
&  0.84982E+00_JPRB, 0.84757E+00_JPRB, 0.84429E+00_JPRB, 0.83619E+00_JPRB, 0.82817E+00_JPRB, 0.82049E+00_JPRB, 0.81377E+00_JPRB ,&
&  0.80759E+00_JPRB, 0.80283E+00_JPRB, 0.80005E+00_JPRB, 0.79859E+00_JPRB, 0.80189E+00_JPRB, 0.80264E+00_JPRB, 0.80260E+00_JPRB ,&
&  0.80114E+00_JPRB, 0.79604E+00_JPRB, 0.79682E+00_JPRB, 0.79812E+00_JPRB, 0.80026E+00_JPRB, 0.80594E+00_JPRB, 0.81123E+00_JPRB ,&
&  0.81748E+00_JPRB, 0.82455E+00_JPRB, 0.83580E+00_JPRB, 0.84391E+00_JPRB, 0.85173E+00_JPRB, 0.85808E+00_JPRB, 0.86151E+00_JPRB ,&
&  0.86483E+00_JPRB, 0.86775E+00_JPRB, 0.87072E+00_JPRB, 0.87312E+00_JPRB, 0.87483E+00_JPRB, 0.87675E+00_JPRB, 0.87902E+00_JPRB ,&
&  0.88022E+00_JPRB, 0.88090E+00_JPRB, 0.88164E+00_JPRB, 0.88247E+00_JPRB, 0.88293E+00_JPRB, 0.88364E+00_JPRB, 0.88477E+00_JPRB ,&
&  0.88591E+00_JPRB, 0.88566E+00_JPRB, 0.88536E+00_JPRB, 0.88617E+00_JPRB, 0.88718E+00_JPRB, 0.88840E+00_JPRB, 0.88904E+00_JPRB ,&
&  0.89026E+00_JPRB, 0.89132E+00_JPRB, 0.89295E+00_JPRB, 0.89541E+00_JPRB, 0.89706E+00_JPRB, 0.89867E+00_JPRB, 0.89888E+00_JPRB ,&
&  0.89899E+00_JPRB, 0.89794E+00_JPRB, 0.89702E+00_JPRB, 0.89738E+00_JPRB, 0.89774E+00_JPRB, 0.89946E+00_JPRB, 0.90170E+00_JPRB ,&
&  0.90493E+00_JPRB, 0.90547E+00_JPRB, 0.90527E+00_JPRB, 0.90389E+00_JPRB, 0.90232E+00_JPRB, 0.90003E+00_JPRB, 0.89774E+00_JPRB ,&
&  0.89521E+00_JPRB, 0.89201E+00_JPRB, 0.88746E+00_JPRB, 0.88233E+00_JPRB, 0.87714E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.87436E+00_JPRB, 0.86791E+00_JPRB, 0.86314E+00_JPRB, 0.85981E+00_JPRB, 0.85668E+00_JPRB, 0.85406E+00_JPRB, 0.85186E+00_JPRB ,&
&  0.84997E+00_JPRB, 0.84706E+00_JPRB, 0.84579E+00_JPRB, 0.84488E+00_JPRB, 0.84344E+00_JPRB, 0.84426E+00_JPRB, 0.84460E+00_JPRB ,&
&  0.84571E+00_JPRB, 0.84558E+00_JPRB, 0.84894E+00_JPRB, 0.84795E+00_JPRB, 0.84735E+00_JPRB, 0.84653E+00_JPRB, 0.84152E+00_JPRB ,&
&  0.83865E+00_JPRB, 0.83472E+00_JPRB, 0.82597E+00_JPRB, 0.81633E+00_JPRB, 0.80932E+00_JPRB, 0.78208E+00_JPRB, 0.78107E+00_JPRB ,&
&  0.78924E+00_JPRB, 0.77330E+00_JPRB, 0.76485E+00_JPRB, 0.75684E+00_JPRB, 0.74723E+00_JPRB, 0.74613E+00_JPRB, 0.75055E+00_JPRB ,&
&  0.74128E+00_JPRB, 0.76126E+00_JPRB, 0.76351E+00_JPRB, 0.76488E+00_JPRB, 0.76748E+00_JPRB, 0.76481E+00_JPRB, 0.77243E+00_JPRB ,&
&  0.79930E+00_JPRB, 0.80520E+00_JPRB, 0.81810E+00_JPRB, 0.82936E+00_JPRB, 0.84253E+00_JPRB, 0.85401E+00_JPRB, 0.86588E+00_JPRB ,&
&  0.87484E+00_JPRB, 0.88119E+00_JPRB, 0.88567E+00_JPRB, 0.88633E+00_JPRB, 0.88678E+00_JPRB, 0.88633E+00_JPRB, 0.88766E+00_JPRB ,&
&  0.88784E+00_JPRB, 0.88921E+00_JPRB, 0.89166E+00_JPRB, 0.89398E+00_JPRB, 0.89713E+00_JPRB, 0.89844E+00_JPRB, 0.90006E+00_JPRB ,&
&  0.90094E+00_JPRB, 0.90066E+00_JPRB, 0.89999E+00_JPRB, 0.89960E+00_JPRB, 0.89812E+00_JPRB, 0.89742E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89969E+00_JPRB, 0.90118E+00_JPRB, 0.90383E+00_JPRB, 0.90669E+00_JPRB, 0.91082E+00_JPRB, 0.91699E+00_JPRB, 0.92160E+00_JPRB ,&
&  0.92849E+00_JPRB, 0.93263E+00_JPRB, 0.93558E+00_JPRB, 0.93760E+00_JPRB, 0.93927E+00_JPRB, 0.94238E+00_JPRB, 0.94474E+00_JPRB ,&
&  0.94820E+00_JPRB, 0.94464E+00_JPRB, 0.93945E+00_JPRB, 0.93342E+00_JPRB, 0.92811E+00_JPRB, 0.92329E+00_JPRB, 0.91813E+00_JPRB ,&
&  0.91350E+00_JPRB, 0.90726E+00_JPRB, 0.89983E+00_JPRB, 0.89091E+00_JPRB, 0.88202E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.89335E+00_JPRB, 0.88344E+00_JPRB, 0.87466E+00_JPRB, 0.86868E+00_JPRB, 0.86275E+00_JPRB, 0.85810E+00_JPRB, 0.85393E+00_JPRB ,&
&  0.85040E+00_JPRB, 0.84699E+00_JPRB, 0.84484E+00_JPRB, 0.84285E+00_JPRB, 0.84161E+00_JPRB, 0.84040E+00_JPRB, 0.83964E+00_JPRB ,&
&  0.84208E+00_JPRB, 0.84223E+00_JPRB, 0.84609E+00_JPRB, 0.84519E+00_JPRB, 0.84457E+00_JPRB, 0.84091E+00_JPRB, 0.82885E+00_JPRB ,&
&  0.82083E+00_JPRB, 0.81261E+00_JPRB, 0.78872E+00_JPRB, 0.80168E+00_JPRB, 0.78974E+00_JPRB, 0.78196E+00_JPRB, 0.77249E+00_JPRB ,&
&  0.75394E+00_JPRB, 0.74531E+00_JPRB, 0.73497E+00_JPRB, 0.73344E+00_JPRB, 0.73861E+00_JPRB, 0.73425E+00_JPRB, 0.73828E+00_JPRB ,&
&  0.73259E+00_JPRB, 0.73100E+00_JPRB, 0.73220E+00_JPRB, 0.73287E+00_JPRB, 0.73993E+00_JPRB, 0.74169E+00_JPRB, 0.74768E+00_JPRB ,&
&  0.75952E+00_JPRB, 0.77781E+00_JPRB, 0.80644E+00_JPRB, 0.81577E+00_JPRB, 0.83553E+00_JPRB, 0.85143E+00_JPRB, 0.86607E+00_JPRB ,&
&  0.87568E+00_JPRB, 0.88172E+00_JPRB, 0.88640E+00_JPRB, 0.88810E+00_JPRB, 0.88786E+00_JPRB, 0.88355E+00_JPRB, 0.88373E+00_JPRB ,&
&  0.88555E+00_JPRB, 0.88813E+00_JPRB, 0.89146E+00_JPRB, 0.89314E+00_JPRB, 0.89632E+00_JPRB, 0.89819E+00_JPRB, 0.90219E+00_JPRB ,&
&  0.90537E+00_JPRB, 0.90864E+00_JPRB, 0.90977E+00_JPRB, 0.91185E+00_JPRB, 0.91456E+00_JPRB, 0.91719E+00_JPRB, 0.92007E+00_JPRB ,&
&  0.92349E+00_JPRB, 0.92876E+00_JPRB, 0.93605E+00_JPRB, 0.93504E+00_JPRB, 0.94148E+00_JPRB, 0.96109E+00_JPRB, 0.96913E+00_JPRB ,&
&  0.97837E+00_JPRB, 0.98212E+00_JPRB, 0.98206E+00_JPRB, 0.98158E+00_JPRB, 0.98050E+00_JPRB, 0.98204E+00_JPRB, 0.98312E+00_JPRB ,&
&  0.98399E+00_JPRB, 0.97689E+00_JPRB, 0.96882E+00_JPRB, 0.96180E+00_JPRB, 0.95433E+00_JPRB, 0.94836E+00_JPRB, 0.94200E+00_JPRB ,&
&  0.93741E+00_JPRB, 0.93329E+00_JPRB, 0.92796E+00_JPRB, 0.91836E+00_JPRB, 0.90658E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.91639E+00_JPRB, 0.90390E+00_JPRB, 0.89401E+00_JPRB, 0.88578E+00_JPRB, 0.87926E+00_JPRB, 0.87385E+00_JPRB, 0.86879E+00_JPRB ,&
&  0.86445E+00_JPRB, 0.85954E+00_JPRB, 0.85486E+00_JPRB, 0.85127E+00_JPRB, 0.84832E+00_JPRB, 0.84595E+00_JPRB, 0.84312E+00_JPRB ,&
&  0.84239E+00_JPRB, 0.84085E+00_JPRB, 0.84339E+00_JPRB, 0.84148E+00_JPRB, 0.84458E+00_JPRB, 0.84140E+00_JPRB, 0.82850E+00_JPRB ,&
&  0.81413E+00_JPRB, 0.79604E+00_JPRB, 0.77766E+00_JPRB, 0.78846E+00_JPRB, 0.78174E+00_JPRB, 0.77839E+00_JPRB, 0.75818E+00_JPRB ,&
&  0.74302E+00_JPRB, 0.72580E+00_JPRB, 0.75002E+00_JPRB, 0.73799E+00_JPRB, 0.73097E+00_JPRB, 0.72169E+00_JPRB, 0.70086E+00_JPRB ,&
&  0.69692E+00_JPRB, 0.68343E+00_JPRB, 0.68235E+00_JPRB, 0.68710E+00_JPRB, 0.69456E+00_JPRB, 0.70642E+00_JPRB, 0.71783E+00_JPRB ,&
&  0.71879E+00_JPRB, 0.76430E+00_JPRB, 0.79463E+00_JPRB, 0.81663E+00_JPRB, 0.83830E+00_JPRB, 0.85519E+00_JPRB, 0.86695E+00_JPRB ,&
&  0.87855E+00_JPRB, 0.88770E+00_JPRB, 0.89789E+00_JPRB, 0.90134E+00_JPRB, 0.91597E+00_JPRB, 0.92074E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.93547E+00_JPRB, 0.93552E+00_JPRB, 0.93246E+00_JPRB, 0.92549E+00_JPRB, 0.92049E+00_JPRB, 0.91480E+00_JPRB, 0.91411E+00_JPRB ,&
&  0.91378E+00_JPRB, 0.91516E+00_JPRB, 0.92255E+00_JPRB, 0.92924E+00_JPRB, 0.93546E+00_JPRB, 0.94094E+00_JPRB, 0.94715E+00_JPRB ,&
&  0.95368E+00_JPRB, 0.96633E+00_JPRB, 0.97983E+00_JPRB, 0.10034E+01_JPRB, 0.10254E+01_JPRB, 0.10393E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10568E+01_JPRB, 0.10515E+01_JPRB, 0.10491E+01_JPRB, 0.10482E+01_JPRB, 0.10493E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10505E+01_JPRB, 0.10431E+01_JPRB, 0.10338E+01_JPRB, 0.10227E+01_JPRB, 0.10118E+01_JPRB, 0.10020E+01_JPRB, 0.99326E+00_JPRB ,&
&  0.98452E+00_JPRB, 0.97484E+00_JPRB, 0.96085E+00_JPRB, 0.94521E+00_JPRB, 0.93009E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.95541E+00_JPRB, 0.93961E+00_JPRB, 0.92737E+00_JPRB, 0.91721E+00_JPRB, 0.91023E+00_JPRB, 0.90464E+00_JPRB, 0.90029E+00_JPRB ,&
&  0.89687E+00_JPRB, 0.89082E+00_JPRB, 0.88398E+00_JPRB, 0.87552E+00_JPRB, 0.86915E+00_JPRB, 0.86339E+00_JPRB, 0.85710E+00_JPRB ,&
&  0.84846E+00_JPRB, 0.83945E+00_JPRB, 0.83016E+00_JPRB, 0.82292E+00_JPRB, 0.82664E+00_JPRB, 0.82507E+00_JPRB, 0.82150E+00_JPRB ,&
&  0.80964E+00_JPRB, 0.79125E+00_JPRB, 0.77540E+00_JPRB, 0.77832E+00_JPRB, 0.77677E+00_JPRB, 0.76040E+00_JPRB, 0.75577E+00_JPRB ,&
&  0.76756E+00_JPRB, 0.76839E+00_JPRB, 0.76020E+00_JPRB, 0.75479E+00_JPRB, 0.72168E+00_JPRB, 0.67473E+00_JPRB, 0.70179E+00_JPRB ,&
&  0.69959E+00_JPRB, 0.67627E+00_JPRB, 0.67244E+00_JPRB, 0.67015E+00_JPRB, 0.67987E+00_JPRB, 0.69960E+00_JPRB, 0.71806E+00_JPRB ,&
&  0.72416E+00_JPRB, 0.79982E+00_JPRB, 0.83429E+00_JPRB, 0.85866E+00_JPRB, 0.87529E+00_JPRB, 0.88839E+00_JPRB, 0.89952E+00_JPRB ,&
&  0.91272E+00_JPRB, 0.93287E+00_JPRB, 0.96075E+00_JPRB, 0.98768E+00_JPRB, 0.10170E+01_JPRB, 0.10374E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.10854E+01_JPRB, 0.10620E+01_JPRB, 0.10368E+01_JPRB, 0.10124E+01_JPRB, 0.99068E+00_JPRB, 0.97192E+00_JPRB ,&
&  0.96389E+00_JPRB, 0.95754E+00_JPRB, 0.96423E+00_JPRB, 0.96499E+00_JPRB, 0.96890E+00_JPRB, 0.97374E+00_JPRB, 0.99031E+00_JPRB ,&
&  0.10022E+01_JPRB, 0.10222E+01_JPRB, 0.10461E+01_JPRB, 0.10794E+01_JPRB, 0.11098E+01_JPRB, 0.11296E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.11515E+01_JPRB, 0.11660E+01_JPRB, 0.11728E+01_JPRB, 0.11846E+01_JPRB, 0.12014E+01_JPRB, 0.12101E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.12236E+01_JPRB, 0.12222E+01_JPRB, 0.12134E+01_JPRB, 0.11976E+01_JPRB, 0.11748E+01_JPRB, 0.11485E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.10433E+01_JPRB, 0.10118E+01_JPRB, 0.98966E+00_JPRB, 0.97092E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.10338E+01_JPRB, 0.10025E+01_JPRB, 0.97956E+00_JPRB, 0.96492E+00_JPRB, 0.95362E+00_JPRB, 0.94565E+00_JPRB, 0.93866E+00_JPRB ,&
&  0.93177E+00_JPRB, 0.92515E+00_JPRB, 0.91578E+00_JPRB, 0.90687E+00_JPRB, 0.89980E+00_JPRB, 0.89233E+00_JPRB, 0.88609E+00_JPRB ,&
&  0.87560E+00_JPRB, 0.85121E+00_JPRB, 0.84617E+00_JPRB, 0.83547E+00_JPRB, 0.81698E+00_JPRB, 0.80687E+00_JPRB, 0.79943E+00_JPRB ,&
&  0.79019E+00_JPRB, 0.78625E+00_JPRB, 0.78635E+00_JPRB, 0.77921E+00_JPRB, 0.77348E+00_JPRB, 0.78082E+00_JPRB, 0.77881E+00_JPRB ,&
&  0.74725E+00_JPRB, 0.74737E+00_JPRB, 0.78933E+00_JPRB, 0.78895E+00_JPRB, 0.74144E+00_JPRB, 0.72919E+00_JPRB, 0.73473E+00_JPRB ,&
&  0.72831E+00_JPRB, 0.69179E+00_JPRB, 0.68539E+00_JPRB, 0.68280E+00_JPRB, 0.69744E+00_JPRB, 0.71297E+00_JPRB, 0.75140E+00_JPRB ,&
&  0.78394E+00_JPRB, 0.85391E+00_JPRB, 0.88954E+00_JPRB, 0.91712E+00_JPRB, 0.93773E+00_JPRB, 0.95887E+00_JPRB, 0.98673E+00_JPRB ,&
&  0.10287E+01_JPRB, 0.10893E+01_JPRB, 0.11641E+01_JPRB, 0.12302E+01_JPRB, 0.12914E+01_JPRB, 0.13223E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.13612E+01_JPRB, 0.13547E+01_JPRB, 0.12987E+01_JPRB, 0.12499E+01_JPRB, 0.11946E+01_JPRB, 0.11406E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10768E+01_JPRB, 0.10607E+01_JPRB, 0.10445E+01_JPRB, 0.10353E+01_JPRB, 0.10343E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.11019E+01_JPRB, 0.12612E+01_JPRB, 0.13130E+01_JPRB, 0.13455E+01_JPRB, 0.13634E+01_JPRB, 0.13736E+01_JPRB ,&
&  0.13917E+01_JPRB, 0.14167E+01_JPRB, 0.14452E+01_JPRB, 0.14602E+01_JPRB, 0.14743E+01_JPRB, 0.14637E+01_JPRB, 0.14763E+01_JPRB ,&
&  0.14657E+01_JPRB, 0.14506E+01_JPRB, 0.14314E+01_JPRB, 0.14149E+01_JPRB, 0.13945E+01_JPRB, 0.13734E+01_JPRB, 0.13396E+01_JPRB ,&
&  0.12948E+01_JPRB, 0.12429E+01_JPRB, 0.11898E+01_JPRB, 0.11330E+01_JPRB, 0.10788E+01_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.11572E+01_JPRB, 0.10879E+01_JPRB, 0.10392E+01_JPRB, 0.10152E+01_JPRB, 0.99892E+00_JPRB, 0.97496E+00_JPRB, 0.95649E+00_JPRB ,&
&  0.94303E+00_JPRB, 0.94150E+00_JPRB, 0.92801E+00_JPRB, 0.91383E+00_JPRB, 0.90481E+00_JPRB, 0.89719E+00_JPRB, 0.89323E+00_JPRB ,&
&  0.88862E+00_JPRB, 0.87771E+00_JPRB, 0.85525E+00_JPRB, 0.83147E+00_JPRB, 0.81473E+00_JPRB, 0.80064E+00_JPRB, 0.78832E+00_JPRB ,&
&  0.77888E+00_JPRB, 0.77830E+00_JPRB, 0.79164E+00_JPRB, 0.79256E+00_JPRB, 0.79023E+00_JPRB, 0.78080E+00_JPRB, 0.77052E+00_JPRB ,&
&  0.77048E+00_JPRB, 0.78623E+00_JPRB, 0.76495E+00_JPRB, 0.77411E+00_JPRB, 0.79268E+00_JPRB, 0.76896E+00_JPRB, 0.75110E+00_JPRB ,&
&  0.73533E+00_JPRB, 0.72088E+00_JPRB, 0.71183E+00_JPRB, 0.71089E+00_JPRB, 0.73613E+00_JPRB, 0.76622E+00_JPRB, 0.81732E+00_JPRB ,&
&  0.87759E+00_JPRB, 0.91927E+00_JPRB, 0.94890E+00_JPRB, 0.97943E+00_JPRB, 0.10082E+01_JPRB, 0.10454E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.13608E+01_JPRB, 0.14770E+01_JPRB, 0.15612E+01_JPRB, 0.16430E+01_JPRB, 0.15982E+01_JPRB, 0.15981E+01_JPRB ,&
&  0.17820E+01_JPRB, 0.18444E+01_JPRB, 0.17372E+01_JPRB, 0.16374E+01_JPRB, 0.15215E+01_JPRB, 0.14179E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.12563E+01_JPRB, 0.12259E+01_JPRB, 0.11802E+01_JPRB, 0.11471E+01_JPRB, 0.11515E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.12762E+01_JPRB, 0.15183E+01_JPRB, 0.19562E+01_JPRB, 0.17338E+01_JPRB, 0.16624E+01_JPRB, 0.16629E+01_JPRB, 0.16784E+01_JPRB ,&
&  0.17241E+01_JPRB, 0.17599E+01_JPRB, 0.17796E+01_JPRB, 0.17474E+01_JPRB, 0.16837E+01_JPRB, 0.16069E+01_JPRB, 0.15544E+01_JPRB ,&
&  0.15106E+01_JPRB, 0.14670E+01_JPRB, 0.14384E+01_JPRB, 0.14285E+01_JPRB, 0.14062E+01_JPRB, 0.13809E+01_JPRB, 0.13552E+01_JPRB ,&
&  0.13264E+01_JPRB, 0.13054E+01_JPRB, 0.12927E+01_JPRB, 0.12687E+01_JPRB, 0.12302E+01_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.11970E+01_JPRB, 0.11957E+01_JPRB, 0.11644E+01_JPRB, 0.11254E+01_JPRB, 0.10869E+01_JPRB, 0.10232E+01_JPRB, 0.97453E+00_JPRB ,&
&  0.94780E+00_JPRB, 0.93736E+00_JPRB, 0.93443E+00_JPRB, 0.92858E+00_JPRB, 0.92265E+00_JPRB, 0.91470E+00_JPRB, 0.90154E+00_JPRB ,&
&  0.88468E+00_JPRB, 0.87303E+00_JPRB, 0.86424E+00_JPRB, 0.84990E+00_JPRB, 0.82978E+00_JPRB, 0.80722E+00_JPRB, 0.78791E+00_JPRB ,&
&  0.77378E+00_JPRB, 0.76861E+00_JPRB, 0.78058E+00_JPRB, 0.79425E+00_JPRB, 0.80553E+00_JPRB, 0.80661E+00_JPRB, 0.79903E+00_JPRB ,&
&  0.77806E+00_JPRB, 0.77302E+00_JPRB, 0.78627E+00_JPRB, 0.79012E+00_JPRB, 0.79975E+00_JPRB, 0.79966E+00_JPRB, 0.78179E+00_JPRB ,&
&  0.76660E+00_JPRB, 0.75511E+00_JPRB, 0.75744E+00_JPRB, 0.77472E+00_JPRB, 0.83196E+00_JPRB, 0.89548E+00_JPRB, 0.92734E+00_JPRB ,&
&  0.95926E+00_JPRB, 0.98280E+00_JPRB, 0.10099E+01_JPRB, 0.10459E+01_JPRB, 0.11074E+01_JPRB, 0.12024E+01_JPRB, 0.13863E+01_JPRB ,&
&  0.16018E+01_JPRB, 0.17541E+01_JPRB, 0.18987E+01_JPRB, 0.17863E+01_JPRB, 0.17488E+01_JPRB, 0.18103E+01_JPRB, 0.19109E+01_JPRB ,&
&  0.20073E+01_JPRB, 0.20402E+01_JPRB, 0.20231E+01_JPRB, 0.20386E+01_JPRB, 0.19224E+01_JPRB, 0.17774E+01_JPRB, 0.16008E+01_JPRB ,&
&  0.14696E+01_JPRB, 0.13817E+01_JPRB, 0.13463E+01_JPRB, 0.13094E+01_JPRB, 0.13188E+01_JPRB, 0.13709E+01_JPRB, 0.14672E+01_JPRB ,&
&  0.16207E+01_JPRB, 0.18061E+01_JPRB, 0.19019E+01_JPRB, 0.18499E+01_JPRB, 0.18940E+01_JPRB, 0.19833E+01_JPRB, 0.20514E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.20407E+01_JPRB, 0.20515E+01_JPRB, 0.20275E+01_JPRB, 0.18764E+01_JPRB, 0.17372E+01_JPRB, 0.15793E+01_JPRB ,&
&  0.14268E+01_JPRB, 0.12918E+01_JPRB, 0.12212E+01_JPRB, 0.12004E+01_JPRB, 0.11855E+01_JPRB, 0.11650E+01_JPRB, 0.11294E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.10890E+01_JPRB, 0.11100E+01_JPRB, 0.11400E+01_JPRB, 0.11772E+01_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.12437E+01_JPRB, 0.12738E+01_JPRB, 0.12845E+01_JPRB, 0.12745E+01_JPRB, 0.12038E+01_JPRB, 0.11336E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10235E+01_JPRB, 0.98380E+00_JPRB, 0.95658E+00_JPRB, 0.92785E+00_JPRB, 0.90380E+00_JPRB, 0.89140E+00_JPRB, 0.88878E+00_JPRB ,&
&  0.88945E+00_JPRB, 0.87821E+00_JPRB, 0.86388E+00_JPRB, 0.84628E+00_JPRB, 0.83295E+00_JPRB, 0.82189E+00_JPRB, 0.81035E+00_JPRB ,&
&  0.79692E+00_JPRB, 0.79449E+00_JPRB, 0.80052E+00_JPRB, 0.81675E+00_JPRB, 0.82714E+00_JPRB, 0.83193E+00_JPRB, 0.83250E+00_JPRB ,&
&  0.82201E+00_JPRB, 0.80683E+00_JPRB, 0.81101E+00_JPRB, 0.82537E+00_JPRB, 0.83547E+00_JPRB, 0.84325E+00_JPRB, 0.81762E+00_JPRB ,&
&  0.80328E+00_JPRB, 0.80847E+00_JPRB, 0.83700E+00_JPRB, 0.91395E+00_JPRB, 0.95943E+00_JPRB, 0.98584E+00_JPRB, 0.10033E+01_JPRB ,&
&  0.99556E+00_JPRB, 0.10015E+01_JPRB, 0.10428E+01_JPRB, 0.11257E+01_JPRB, 0.12687E+01_JPRB, 0.15299E+01_JPRB, 0.18510E+01_JPRB ,&
&  0.20448E+01_JPRB, 0.21530E+01_JPRB, 0.22134E+01_JPRB, 0.20782E+01_JPRB, 0.21050E+01_JPRB, 0.24079E+01_JPRB, 0.25124E+01_JPRB ,&
&  0.25793E+01_JPRB, 0.25816E+01_JPRB, 0.26200E+01_JPRB, 0.24903E+01_JPRB, 0.22065E+01_JPRB, 0.19981E+01_JPRB, 0.18455E+01_JPRB ,&
&  0.16872E+01_JPRB, 0.15945E+01_JPRB, 0.15735E+01_JPRB, 0.16159E+01_JPRB, 0.17179E+01_JPRB, 0.19347E+01_JPRB, 0.22144E+01_JPRB ,&
&  0.24878E+01_JPRB, 0.26525E+01_JPRB, 0.27464E+01_JPRB, 0.27174E+01_JPRB, 0.26644E+01_JPRB, 0.25886E+01_JPRB, 0.25308E+01_JPRB ,&
&  0.24677E+01_JPRB, 0.24464E+01_JPRB, 0.24196E+01_JPRB, 0.23843E+01_JPRB, 0.21798E+01_JPRB, 0.18587E+01_JPRB, 0.15561E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.12271E+01_JPRB, 0.11447E+01_JPRB, 0.11246E+01_JPRB, 0.11246E+01_JPRB, 0.11233E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.10637E+01_JPRB, 0.10775E+01_JPRB, 0.11056E+01_JPRB, 0.11367E+01_JPRB, 0.11885E+01_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.13495E+01_JPRB, 0.13842E+01_JPRB, 0.13988E+01_JPRB, 0.13803E+01_JPRB, 0.13210E+01_JPRB, 0.12302E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.10786E+01_JPRB, 0.10337E+01_JPRB, 0.10051E+01_JPRB, 0.97568E+00_JPRB, 0.95075E+00_JPRB, 0.91700E+00_JPRB, 0.88450E+00_JPRB ,&
&  0.87488E+00_JPRB, 0.88028E+00_JPRB, 0.88318E+00_JPRB, 0.87540E+00_JPRB, 0.86681E+00_JPRB, 0.85919E+00_JPRB, 0.85221E+00_JPRB ,&
&  0.84491E+00_JPRB, 0.84687E+00_JPRB, 0.86364E+00_JPRB, 0.88969E+00_JPRB, 0.90251E+00_JPRB, 0.90629E+00_JPRB, 0.91298E+00_JPRB ,&
&  0.90660E+00_JPRB, 0.89019E+00_JPRB, 0.88112E+00_JPRB, 0.88549E+00_JPRB, 0.88873E+00_JPRB, 0.91125E+00_JPRB, 0.91124E+00_JPRB ,&
&  0.86150E+00_JPRB, 0.89051E+00_JPRB, 0.96930E+00_JPRB, 0.10113E+01_JPRB, 0.10396E+01_JPRB, 0.10591E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.11240E+01_JPRB, 0.11675E+01_JPRB, 0.12268E+01_JPRB, 0.15035E+01_JPRB, 0.21014E+01_JPRB, 0.23523E+01_JPRB ,&
&  0.24806E+01_JPRB, 0.22224E+01_JPRB, 0.20962E+01_JPRB, 0.21424E+01_JPRB, 0.24646E+01_JPRB, 0.26730E+01_JPRB, 0.28450E+01_JPRB ,&
&  0.30339E+01_JPRB, 0.30259E+01_JPRB, 0.29489E+01_JPRB, 0.27902E+01_JPRB, 0.25143E+01_JPRB, 0.23074E+01_JPRB, 0.22131E+01_JPRB ,&
&  0.21111E+01_JPRB, 0.20842E+01_JPRB, 0.21854E+01_JPRB, 0.24416E+01_JPRB, 0.27470E+01_JPRB, 0.30081E+01_JPRB, 0.32092E+01_JPRB ,&
&  0.34105E+01_JPRB, 0.35047E+01_JPRB, 0.34512E+01_JPRB, 0.33479E+01_JPRB, 0.31768E+01_JPRB, 0.30704E+01_JPRB, 0.30219E+01_JPRB ,&
&  0.29642E+01_JPRB, 0.29127E+01_JPRB, 0.28086E+01_JPRB, 0.26682E+01_JPRB, 0.24425E+01_JPRB, 0.22355E+01_JPRB, 0.19843E+01_JPRB ,&
&  0.16013E+01_JPRB, 0.13897E+01_JPRB, 0.12206E+01_JPRB, 0.11695E+01_JPRB, 0.11660E+01_JPRB, 0.11810E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.12068E+01_JPRB, 0.12141E+01_JPRB, 0.12556E+01_JPRB, 0.13086E+01_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.14659E+01_JPRB, 0.14794E+01_JPRB, 0.14712E+01_JPRB, 0.14536E+01_JPRB, 0.14105E+01_JPRB, 0.13304E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.12236E+01_JPRB, 0.12143E+01_JPRB, 0.10712E+01_JPRB, 0.10209E+01_JPRB, 0.99881E+00_JPRB, 0.95691E+00_JPRB ,&
&  0.93325E+00_JPRB, 0.92565E+00_JPRB, 0.91737E+00_JPRB, 0.91792E+00_JPRB, 0.91891E+00_JPRB, 0.91257E+00_JPRB, 0.91123E+00_JPRB ,&
&  0.92116E+00_JPRB, 0.94128E+00_JPRB, 0.97843E+00_JPRB, 0.10034E+01_JPRB, 0.10254E+01_JPRB, 0.10377E+01_JPRB, 0.10457E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.98676E+00_JPRB, 0.98462E+00_JPRB, 0.96491E+00_JPRB, 0.96722E+00_JPRB, 0.98764E+00_JPRB, 0.10194E+01_JPRB ,&
&  0.10413E+01_JPRB, 0.10134E+01_JPRB, 0.10853E+01_JPRB, 0.11230E+01_JPRB, 0.11454E+01_JPRB, 0.11592E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.11830E+01_JPRB, 0.12164E+01_JPRB, 0.12680E+01_JPRB, 0.13606E+01_JPRB, 0.18961E+01_JPRB, 0.28551E+01_JPRB, 0.28991E+01_JPRB ,&
&  0.28385E+01_JPRB, 0.23707E+01_JPRB, 0.26215E+01_JPRB, 0.27330E+01_JPRB, 0.29384E+01_JPRB, 0.31357E+01_JPRB, 0.32142E+01_JPRB ,&
&  0.35623E+01_JPRB, 0.36824E+01_JPRB, 0.35329E+01_JPRB, 0.32089E+01_JPRB, 0.29250E+01_JPRB, 0.27818E+01_JPRB, 0.27206E+01_JPRB ,&
&  0.27284E+01_JPRB, 0.26420E+01_JPRB, 0.29596E+01_JPRB, 0.32429E+01_JPRB, 0.34117E+01_JPRB, 0.35846E+01_JPRB, 0.37844E+01_JPRB ,&
&  0.39068E+01_JPRB, 0.38949E+01_JPRB, 0.38478E+01_JPRB, 0.35935E+01_JPRB, 0.34550E+01_JPRB, 0.33473E+01_JPRB, 0.31701E+01_JPRB ,&
&  0.30530E+01_JPRB, 0.29572E+01_JPRB, 0.28244E+01_JPRB, 0.28179E+01_JPRB, 0.25202E+01_JPRB, 0.23201E+01_JPRB, 0.21996E+01_JPRB ,&
&  0.19051E+01_JPRB, 0.15952E+01_JPRB, 0.14614E+01_JPRB, 0.14072E+01_JPRB, 0.13640E+01_JPRB, 0.13529E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.12993E+01_JPRB, 0.13164E+01_JPRB, 0.13865E+01_JPRB, 0.14168E+01_JPRB, 0.14376E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.15728E+01_JPRB, 0.15815E+01_JPRB, 0.15703E+01_JPRB, 0.15532E+01_JPRB, 0.15325E+01_JPRB, 0.14923E+01_JPRB, 0.14154E+01_JPRB ,&
&  0.14003E+01_JPRB, 0.13526E+01_JPRB, 0.12988E+01_JPRB, 0.12571E+01_JPRB, 0.12406E+01_JPRB, 0.11497E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.97290E+00_JPRB, 0.97677E+00_JPRB, 0.99359E+00_JPRB, 0.99944E+00_JPRB, 0.10059E+01_JPRB, 0.10110E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.11076E+01_JPRB, 0.11582E+01_JPRB, 0.12126E+01_JPRB, 0.12270E+01_JPRB, 0.12290E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.11954E+01_JPRB, 0.11321E+01_JPRB, 0.10958E+01_JPRB, 0.10316E+01_JPRB, 0.10885E+01_JPRB, 0.11352E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.12592E+01_JPRB, 0.12922E+01_JPRB, 0.13091E+01_JPRB, 0.13073E+01_JPRB, 0.13087E+01_JPRB, 0.13142E+01_JPRB ,&
&  0.13297E+01_JPRB, 0.13698E+01_JPRB, 0.14358E+01_JPRB, 0.15580E+01_JPRB, 0.26240E+01_JPRB, 0.37150E+01_JPRB, 0.38787E+01_JPRB ,&
&  0.35735E+01_JPRB, 0.37716E+01_JPRB, 0.39556E+01_JPRB, 0.36665E+01_JPRB, 0.37931E+01_JPRB, 0.38951E+01_JPRB, 0.37814E+01_JPRB ,&
&  0.39486E+01_JPRB, 0.39581E+01_JPRB, 0.38897E+01_JPRB, 0.37848E+01_JPRB, 0.36923E+01_JPRB, 0.35652E+01_JPRB, 0.34688E+01_JPRB ,&
&  0.34969E+01_JPRB, 0.36918E+01_JPRB, 0.37626E+01_JPRB, 0.38822E+01_JPRB, 0.39238E+01_JPRB, 0.40144E+01_JPRB, 0.40962E+01_JPRB ,&
&  0.41128E+01_JPRB, 0.41253E+01_JPRB, 0.40775E+01_JPRB, 0.39149E+01_JPRB, 0.37244E+01_JPRB, 0.35265E+01_JPRB, 0.32969E+01_JPRB ,&
&  0.31004E+01_JPRB, 0.27553E+01_JPRB, 0.24919E+01_JPRB, 0.23935E+01_JPRB, 0.23000E+01_JPRB, 0.20999E+01_JPRB, 0.20400E+01_JPRB ,&
&  0.19833E+01_JPRB, 0.18748E+01_JPRB, 0.19658E+01_JPRB, 0.18363E+01_JPRB, 0.17102E+01_JPRB, 0.15583E+01_JPRB, 0.13907E+01_JPRB ,&
&  0.13489E+01_JPRB, 0.14338E+01_JPRB, 0.15272E+01_JPRB, 0.15758E+01_JPRB, 0.15761E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.16507E+01_JPRB, 0.16630E+01_JPRB, 0.16648E+01_JPRB, 0.16630E+01_JPRB, 0.16466E+01_JPRB, 0.16089E+01_JPRB, 0.15564E+01_JPRB ,&
&  0.15049E+01_JPRB, 0.14441E+01_JPRB, 0.13894E+01_JPRB, 0.13454E+01_JPRB, 0.13237E+01_JPRB, 0.12909E+01_JPRB, 0.11940E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.10803E+01_JPRB, 0.10617E+01_JPRB, 0.10754E+01_JPRB, 0.11383E+01_JPRB, 0.11683E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.12958E+01_JPRB, 0.13660E+01_JPRB, 0.14128E+01_JPRB, 0.14612E+01_JPRB, 0.14943E+01_JPRB, 0.15311E+01_JPRB, 0.15755E+01_JPRB ,&
&  0.15067E+01_JPRB, 0.13814E+01_JPRB, 0.13023E+01_JPRB, 0.12585E+01_JPRB, 0.13219E+01_JPRB, 0.14238E+01_JPRB, 0.15083E+01_JPRB ,&
&  0.15511E+01_JPRB, 0.15617E+01_JPRB, 0.15678E+01_JPRB, 0.15651E+01_JPRB, 0.15471E+01_JPRB, 0.15370E+01_JPRB, 0.15354E+01_JPRB ,&
&  0.15639E+01_JPRB, 0.15963E+01_JPRB, 0.16423E+01_JPRB, 0.18656E+01_JPRB, 0.35473E+01_JPRB, 0.52966E+01_JPRB, 0.57886E+01_JPRB ,&
&  0.52737E+01_JPRB, 0.51765E+01_JPRB, 0.53036E+01_JPRB, 0.54822E+01_JPRB, 0.53517E+01_JPRB, 0.48758E+01_JPRB, 0.44868E+01_JPRB ,&
&  0.44775E+01_JPRB, 0.42774E+01_JPRB, 0.45658E+01_JPRB, 0.45132E+01_JPRB, 0.45296E+01_JPRB, 0.45960E+01_JPRB, 0.44961E+01_JPRB ,&
&  0.45088E+01_JPRB, 0.44252E+01_JPRB, 0.43997E+01_JPRB, 0.42759E+01_JPRB, 0.42471E+01_JPRB, 0.43257E+01_JPRB, 0.43348E+01_JPRB ,&
&  0.43809E+01_JPRB, 0.43979E+01_JPRB, 0.41170E+01_JPRB, 0.39056E+01_JPRB, 0.36661E+01_JPRB, 0.38276E+01_JPRB, 0.35094E+01_JPRB ,&
&  0.29947E+01_JPRB, 0.23818E+01_JPRB, 0.23719E+01_JPRB, 0.23453E+01_JPRB, 0.23652E+01_JPRB, 0.23107E+01_JPRB, 0.21510E+01_JPRB ,&
&  0.21460E+01_JPRB, 0.23731E+01_JPRB, 0.24281E+01_JPRB, 0.21443E+01_JPRB, 0.19679E+01_JPRB, 0.17658E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.16254E+01_JPRB, 0.16698E+01_JPRB, 0.16552E+01_JPRB, 0.16352E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.18213E+01_JPRB, 0.17665E+01_JPRB, 0.17395E+01_JPRB, 0.17416E+01_JPRB, 0.17237E+01_JPRB, 0.16933E+01_JPRB, 0.16527E+01_JPRB ,&
&  0.16153E+01_JPRB, 0.15736E+01_JPRB, 0.15270E+01_JPRB, 0.14833E+01_JPRB, 0.14519E+01_JPRB, 0.14103E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.12223E+01_JPRB, 0.12093E+01_JPRB, 0.12478E+01_JPRB, 0.14364E+01_JPRB, 0.14156E+01_JPRB, 0.14456E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.15468E+01_JPRB, 0.15542E+01_JPRB, 0.15797E+01_JPRB, 0.16537E+01_JPRB, 0.18243E+01_JPRB, 0.20406E+01_JPRB ,&
&  0.20003E+01_JPRB, 0.18188E+01_JPRB, 0.17701E+01_JPRB, 0.17535E+01_JPRB, 0.18316E+01_JPRB, 0.19038E+01_JPRB, 0.19964E+01_JPRB ,&
&  0.19581E+01_JPRB, 0.18909E+01_JPRB, 0.18359E+01_JPRB, 0.17973E+01_JPRB, 0.17701E+01_JPRB, 0.17705E+01_JPRB, 0.18040E+01_JPRB ,&
&  0.18465E+01_JPRB, 0.18441E+01_JPRB, 0.18804E+01_JPRB, 0.23434E+01_JPRB, 0.45873E+01_JPRB, 0.68483E+01_JPRB, 0.79128E+01_JPRB ,&
&  0.78865E+01_JPRB, 0.72891E+01_JPRB, 0.75633E+01_JPRB, 0.83711E+01_JPRB, 0.77348E+01_JPRB, 0.66924E+01_JPRB, 0.58614E+01_JPRB ,&
&  0.55038E+01_JPRB, 0.52117E+01_JPRB, 0.53448E+01_JPRB, 0.54638E+01_JPRB, 0.52331E+01_JPRB, 0.51680E+01_JPRB, 0.50376E+01_JPRB ,&
&  0.49175E+01_JPRB, 0.49216E+01_JPRB, 0.47603E+01_JPRB, 0.45849E+01_JPRB, 0.44619E+01_JPRB, 0.45657E+01_JPRB, 0.45160E+01_JPRB ,&
&  0.46255E+01_JPRB, 0.44565E+01_JPRB, 0.39064E+01_JPRB, 0.38450E+01_JPRB, 0.30754E+01_JPRB, 0.29287E+01_JPRB, 0.33949E+01_JPRB ,&
&  0.28945E+01_JPRB, 0.25119E+01_JPRB, 0.24542E+01_JPRB, 0.25529E+01_JPRB, 0.26336E+01_JPRB, 0.28596E+01_JPRB, 0.27947E+01_JPRB ,&
&  0.24054E+01_JPRB, 0.25260E+01_JPRB, 0.28807E+01_JPRB, 0.25406E+01_JPRB, 0.22790E+01_JPRB, 0.21090E+01_JPRB, 0.19007E+01_JPRB ,&
&  0.19218E+01_JPRB, 0.19010E+01_JPRB, 0.18751E+01_JPRB, 0.18391E+01_JPRB, 0.18277E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.19507E+01_JPRB, 0.17802E+01_JPRB, 0.17686E+01_JPRB, 0.17591E+01_JPRB, 0.17387E+01_JPRB, 0.17114E+01_JPRB, 0.16960E+01_JPRB ,&
&  0.16898E+01_JPRB, 0.16747E+01_JPRB, 0.16424E+01_JPRB, 0.16004E+01_JPRB, 0.15634E+01_JPRB, 0.15169E+01_JPRB, 0.14637E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.15225E+01_JPRB, 0.15355E+01_JPRB, 0.14695E+01_JPRB, 0.16018E+01_JPRB, 0.15922E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.17004E+01_JPRB, 0.18132E+01_JPRB, 0.18437E+01_JPRB, 0.18963E+01_JPRB, 0.20153E+01_JPRB, 0.22936E+01_JPRB, 0.26501E+01_JPRB ,&
&  0.27728E+01_JPRB, 0.26617E+01_JPRB, 0.26381E+01_JPRB, 0.27592E+01_JPRB, 0.29216E+01_JPRB, 0.26020E+01_JPRB, 0.24983E+01_JPRB ,&
&  0.23028E+01_JPRB, 0.21453E+01_JPRB, 0.20342E+01_JPRB, 0.19699E+01_JPRB, 0.19680E+01_JPRB, 0.20029E+01_JPRB, 0.20468E+01_JPRB ,&
&  0.20495E+01_JPRB, 0.19811E+01_JPRB, 0.20889E+01_JPRB, 0.29358E+01_JPRB, 0.52145E+01_JPRB, 0.72959E+01_JPRB, 0.82252E+01_JPRB ,&
&  0.83566E+01_JPRB, 0.82223E+01_JPRB, 0.84621E+01_JPRB, 0.92917E+01_JPRB, 0.86407E+01_JPRB, 0.76630E+01_JPRB, 0.70943E+01_JPRB ,&
&  0.67154E+01_JPRB, 0.64910E+01_JPRB, 0.64962E+01_JPRB, 0.60251E+01_JPRB, 0.54129E+01_JPRB, 0.50453E+01_JPRB, 0.47053E+01_JPRB ,&
&  0.44361E+01_JPRB, 0.44343E+01_JPRB, 0.44549E+01_JPRB, 0.42219E+01_JPRB, 0.40852E+01_JPRB, 0.38687E+01_JPRB, 0.40900E+01_JPRB ,&
&  0.45156E+01_JPRB, 0.34265E+01_JPRB, 0.28370E+01_JPRB, 0.31589E+01_JPRB, 0.24196E+01_JPRB, 0.23545E+01_JPRB, 0.26781E+01_JPRB ,&
&  0.25191E+01_JPRB, 0.25486E+01_JPRB, 0.28855E+01_JPRB, 0.28503E+01_JPRB, 0.30181E+01_JPRB, 0.33760E+01_JPRB, 0.34208E+01_JPRB ,&
&  0.31734E+01_JPRB, 0.29686E+01_JPRB, 0.35431E+01_JPRB, 0.31279E+01_JPRB, 0.27635E+01_JPRB, 0.25928E+01_JPRB, 0.24475E+01_JPRB ,&
&  0.23525E+01_JPRB, 0.22634E+01_JPRB, 0.21596E+01_JPRB, 0.20828E+01_JPRB, 0.20497E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.19163E+01_JPRB, 0.17578E+01_JPRB, 0.18183E+01_JPRB, 0.18291E+01_JPRB, 0.18174E+01_JPRB, 0.18052E+01_JPRB, 0.18067E+01_JPRB ,&
&  0.17901E+01_JPRB, 0.17698E+01_JPRB, 0.17238E+01_JPRB, 0.16495E+01_JPRB, 0.15950E+01_JPRB, 0.15492E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.14843E+01_JPRB, 0.16449E+01_JPRB, 0.15865E+01_JPRB, 0.15853E+01_JPRB, 0.15411E+01_JPRB, 0.16418E+01_JPRB, 0.17434E+01_JPRB ,&
&  0.19122E+01_JPRB, 0.20484E+01_JPRB, 0.22533E+01_JPRB, 0.25819E+01_JPRB, 0.29456E+01_JPRB, 0.33190E+01_JPRB, 0.38734E+01_JPRB ,&
&  0.40647E+01_JPRB, 0.39761E+01_JPRB, 0.37493E+01_JPRB, 0.37962E+01_JPRB, 0.36344E+01_JPRB, 0.31545E+01_JPRB, 0.29628E+01_JPRB ,&
&  0.26702E+01_JPRB, 0.24726E+01_JPRB, 0.23234E+01_JPRB, 0.22470E+01_JPRB, 0.21741E+01_JPRB, 0.20677E+01_JPRB, 0.19986E+01_JPRB ,&
&  0.19413E+01_JPRB, 0.20456E+01_JPRB, 0.25757E+01_JPRB, 0.38062E+01_JPRB, 0.58307E+01_JPRB, 0.76207E+01_JPRB, 0.79908E+01_JPRB ,&
&  0.83999E+01_JPRB, 0.80651E+01_JPRB, 0.65205E+01_JPRB, 0.74190E+01_JPRB, 0.93674E+01_JPRB, 0.88460E+01_JPRB, 0.78490E+01_JPRB ,&
&  0.76781E+01_JPRB, 0.70418E+01_JPRB, 0.66664E+01_JPRB, 0.58361E+01_JPRB, 0.50477E+01_JPRB, 0.45719E+01_JPRB, 0.41848E+01_JPRB ,&
&  0.39002E+01_JPRB, 0.38474E+01_JPRB, 0.38648E+01_JPRB, 0.38936E+01_JPRB, 0.37511E+01_JPRB, 0.38477E+01_JPRB, 0.42394E+01_JPRB ,&
&  0.42351E+01_JPRB, 0.39318E+01_JPRB, 0.29975E+01_JPRB, 0.29329E+01_JPRB, 0.22377E+01_JPRB, 0.21774E+01_JPRB, 0.24658E+01_JPRB ,&
&  0.24877E+01_JPRB, 0.30444E+01_JPRB, 0.37718E+01_JPRB, 0.38362E+01_JPRB, 0.44428E+01_JPRB, 0.45419E+01_JPRB, 0.45316E+01_JPRB ,&
&  0.42962E+01_JPRB, 0.38315E+01_JPRB, 0.43674E+01_JPRB, 0.37235E+01_JPRB, 0.31789E+01_JPRB, 0.29437E+01_JPRB, 0.27849E+01_JPRB ,&
&  0.26649E+01_JPRB, 0.25012E+01_JPRB, 0.23211E+01_JPRB, 0.21932E+01_JPRB, 0.20789E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.18945E+01_JPRB, 0.17996E+01_JPRB, 0.18612E+01_JPRB, 0.18392E+01_JPRB, 0.18596E+01_JPRB, 0.18421E+01_JPRB, 0.18624E+01_JPRB ,&
&  0.18457E+01_JPRB, 0.17831E+01_JPRB, 0.17021E+01_JPRB, 0.16312E+01_JPRB, 0.15794E+01_JPRB, 0.15327E+01_JPRB, 0.15093E+01_JPRB ,&
&  0.14961E+01_JPRB, 0.15816E+01_JPRB, 0.14612E+01_JPRB, 0.13883E+01_JPRB, 0.13659E+01_JPRB, 0.15253E+01_JPRB, 0.17899E+01_JPRB ,&
&  0.20657E+01_JPRB, 0.23188E+01_JPRB, 0.28084E+01_JPRB, 0.33152E+01_JPRB, 0.38640E+01_JPRB, 0.44169E+01_JPRB, 0.51329E+01_JPRB ,&
&  0.57039E+01_JPRB, 0.48626E+01_JPRB, 0.45385E+01_JPRB, 0.39934E+01_JPRB, 0.35672E+01_JPRB, 0.31996E+01_JPRB, 0.29432E+01_JPRB ,&
&  0.27414E+01_JPRB, 0.26138E+01_JPRB, 0.24430E+01_JPRB, 0.22310E+01_JPRB, 0.20309E+01_JPRB, 0.19333E+01_JPRB, 0.19268E+01_JPRB ,&
&  0.20329E+01_JPRB, 0.24584E+01_JPRB, 0.35549E+01_JPRB, 0.51344E+01_JPRB, 0.75343E+01_JPRB, 0.88045E+01_JPRB, 0.84840E+01_JPRB ,&
&  0.72645E+01_JPRB, 0.72747E+01_JPRB, 0.10581E+02_JPRB, 0.89967E+01_JPRB, 0.91859E+01_JPRB, 0.97480E+01_JPRB, 0.88449E+01_JPRB ,&
&  0.75951E+01_JPRB, 0.62089E+01_JPRB, 0.58629E+01_JPRB, 0.52428E+01_JPRB, 0.59700E+01_JPRB, 0.51461E+01_JPRB, 0.44539E+01_JPRB ,&
&  0.40465E+01_JPRB, 0.40225E+01_JPRB, 0.38878E+01_JPRB, 0.38891E+01_JPRB, 0.41178E+01_JPRB, 0.47478E+01_JPRB, 0.48242E+01_JPRB ,&
&  0.43665E+01_JPRB, 0.45188E+01_JPRB, 0.37650E+01_JPRB, 0.30823E+01_JPRB, 0.26487E+01_JPRB, 0.26685E+01_JPRB, 0.27929E+01_JPRB ,&
&  0.31512E+01_JPRB, 0.36891E+01_JPRB, 0.47786E+01_JPRB, 0.57933E+01_JPRB, 0.65778E+01_JPRB, 0.63142E+01_JPRB, 0.55890E+01_JPRB ,&
&  0.51391E+01_JPRB, 0.56791E+01_JPRB, 0.55232E+01_JPRB, 0.42748E+01_JPRB, 0.35162E+01_JPRB, 0.32275E+01_JPRB, 0.30124E+01_JPRB ,&
&  0.27323E+01_JPRB, 0.24425E+01_JPRB, 0.22313E+01_JPRB, 0.21209E+01_JPRB, 0.20513E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.19601E+01_JPRB, 0.18808E+01_JPRB, 0.18013E+01_JPRB, 0.17796E+01_JPRB, 0.18267E+01_JPRB, 0.18068E+01_JPRB, 0.17787E+01_JPRB ,&
&  0.17156E+01_JPRB, 0.16413E+01_JPRB, 0.16126E+01_JPRB, 0.15969E+01_JPRB, 0.15682E+01_JPRB, 0.15280E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.14979E+01_JPRB, 0.14741E+01_JPRB, 0.13532E+01_JPRB, 0.13767E+01_JPRB, 0.13917E+01_JPRB, 0.14367E+01_JPRB, 0.17675E+01_JPRB ,&
&  0.21577E+01_JPRB, 0.26271E+01_JPRB, 0.32432E+01_JPRB, 0.39311E+01_JPRB, 0.44829E+01_JPRB, 0.51228E+01_JPRB, 0.58434E+01_JPRB ,&
&  0.55280E+01_JPRB, 0.54114E+01_JPRB, 0.47575E+01_JPRB, 0.40821E+01_JPRB, 0.35253E+01_JPRB, 0.31026E+01_JPRB, 0.28880E+01_JPRB ,&
&  0.27206E+01_JPRB, 0.26217E+01_JPRB, 0.23869E+01_JPRB, 0.21090E+01_JPRB, 0.19964E+01_JPRB, 0.19778E+01_JPRB, 0.20244E+01_JPRB ,&
&  0.22306E+01_JPRB, 0.30751E+01_JPRB, 0.44034E+01_JPRB, 0.60433E+01_JPRB, 0.59710E+01_JPRB, 0.67864E+01_JPRB, 0.70127E+01_JPRB ,&
&  0.93781E+01_JPRB, 0.97256E+01_JPRB, 0.95504E+01_JPRB, 0.10198E+02_JPRB, 0.84725E+01_JPRB, 0.87156E+01_JPRB, 0.85369E+01_JPRB ,&
&  0.74326E+01_JPRB, 0.68381E+01_JPRB, 0.73117E+01_JPRB, 0.61028E+01_JPRB, 0.55918E+01_JPRB, 0.64032E+01_JPRB, 0.55047E+01_JPRB ,&
&  0.50364E+01_JPRB, 0.47728E+01_JPRB, 0.44109E+01_JPRB, 0.46675E+01_JPRB, 0.44365E+01_JPRB, 0.45052E+01_JPRB, 0.38505E+01_JPRB ,&
&  0.37865E+01_JPRB, 0.32256E+01_JPRB, 0.35852E+01_JPRB, 0.32733E+01_JPRB, 0.32723E+01_JPRB, 0.32614E+01_JPRB, 0.33705E+01_JPRB ,&
&  0.40742E+01_JPRB, 0.47366E+01_JPRB, 0.63547E+01_JPRB, 0.79568E+01_JPRB, 0.90286E+01_JPRB, 0.73947E+01_JPRB, 0.67692E+01_JPRB ,&
&  0.69953E+01_JPRB, 0.69923E+01_JPRB, 0.56798E+01_JPRB, 0.44383E+01_JPRB, 0.36691E+01_JPRB, 0.33141E+01_JPRB, 0.30834E+01_JPRB ,&
&  0.28379E+01_JPRB, 0.25963E+01_JPRB, 0.24024E+01_JPRB, 0.22642E+01_JPRB, 0.21454E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.18027E+01_JPRB, 0.17674E+01_JPRB, 0.16602E+01_JPRB, 0.16763E+01_JPRB, 0.17483E+01_JPRB, 0.17215E+01_JPRB, 0.16795E+01_JPRB ,&
&  0.16109E+01_JPRB, 0.15643E+01_JPRB, 0.15768E+01_JPRB, 0.15756E+01_JPRB, 0.15506E+01_JPRB, 0.15207E+01_JPRB, 0.14782E+01_JPRB ,&
&  0.14653E+01_JPRB, 0.14556E+01_JPRB, 0.13495E+01_JPRB, 0.13172E+01_JPRB, 0.14113E+01_JPRB, 0.14596E+01_JPRB, 0.16569E+01_JPRB ,&
&  0.22701E+01_JPRB, 0.28996E+01_JPRB, 0.35148E+01_JPRB, 0.42463E+01_JPRB, 0.47976E+01_JPRB, 0.52766E+01_JPRB, 0.54811E+01_JPRB ,&
&  0.52806E+01_JPRB, 0.44684E+01_JPRB, 0.39773E+01_JPRB, 0.36142E+01_JPRB, 0.33003E+01_JPRB, 0.30509E+01_JPRB, 0.28356E+01_JPRB ,&
&  0.26321E+01_JPRB, 0.22925E+01_JPRB, 0.19431E+01_JPRB, 0.17578E+01_JPRB, 0.17702E+01_JPRB, 0.17882E+01_JPRB, 0.19473E+01_JPRB ,&
&  0.25825E+01_JPRB, 0.34196E+01_JPRB, 0.44075E+01_JPRB, 0.54663E+01_JPRB, 0.54633E+01_JPRB, 0.59926E+01_JPRB, 0.73867E+01_JPRB ,&
&  0.79741E+01_JPRB, 0.79635E+01_JPRB, 0.92635E+01_JPRB, 0.10366E+02_JPRB, 0.10448E+02_JPRB, 0.89693E+01_JPRB, 0.86024E+01_JPRB ,&
&  0.63292E+01_JPRB, 0.50475E+01_JPRB, 0.45588E+01_JPRB, 0.44383E+01_JPRB, 0.53364E+01_JPRB, 0.70830E+01_JPRB, 0.66654E+01_JPRB ,&
&  0.63606E+01_JPRB, 0.59963E+01_JPRB, 0.55120E+01_JPRB, 0.47949E+01_JPRB, 0.44812E+01_JPRB, 0.41075E+01_JPRB, 0.52165E+01_JPRB ,&
&  0.52271E+01_JPRB, 0.51726E+01_JPRB, 0.44266E+01_JPRB, 0.36745E+01_JPRB, 0.31960E+01_JPRB, 0.38219E+01_JPRB, 0.43481E+01_JPRB ,&
&  0.51526E+01_JPRB, 0.61453E+01_JPRB, 0.10825E+02_JPRB, 0.11617E+02_JPRB, 0.10360E+02_JPRB, 0.89301E+01_JPRB, 0.91230E+01_JPRB ,&
&  0.80307E+01_JPRB, 0.76399E+01_JPRB, 0.61553E+01_JPRB, 0.44490E+01_JPRB, 0.34178E+01_JPRB, 0.28636E+01_JPRB, 0.25766E+01_JPRB ,&
&  0.24876E+01_JPRB, 0.24043E+01_JPRB, 0.22641E+01_JPRB, 0.20894E+01_JPRB, 0.19499E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.16705E+01_JPRB, 0.16405E+01_JPRB, 0.15192E+01_JPRB, 0.15119E+01_JPRB, 0.15735E+01_JPRB, 0.16193E+01_JPRB, 0.16105E+01_JPRB ,&
&  0.15846E+01_JPRB, 0.15843E+01_JPRB, 0.15973E+01_JPRB, 0.15679E+01_JPRB, 0.15267E+01_JPRB, 0.15001E+01_JPRB, 0.14621E+01_JPRB ,&
&  0.14664E+01_JPRB, 0.15994E+01_JPRB, 0.14190E+01_JPRB, 0.13968E+01_JPRB, 0.14936E+01_JPRB, 0.16206E+01_JPRB, 0.18235E+01_JPRB ,&
&  0.25564E+01_JPRB, 0.30102E+01_JPRB, 0.32828E+01_JPRB, 0.39598E+01_JPRB, 0.49857E+01_JPRB, 0.47459E+01_JPRB, 0.44774E+01_JPRB ,&
&  0.39217E+01_JPRB, 0.33233E+01_JPRB, 0.31058E+01_JPRB, 0.29552E+01_JPRB, 0.29303E+01_JPRB, 0.29161E+01_JPRB, 0.27604E+01_JPRB ,&
&  0.24247E+01_JPRB, 0.19941E+01_JPRB, 0.17459E+01_JPRB, 0.17285E+01_JPRB, 0.16662E+01_JPRB, 0.17196E+01_JPRB, 0.19207E+01_JPRB ,&
&  0.26375E+01_JPRB, 0.35056E+01_JPRB, 0.40410E+01_JPRB, 0.45407E+01_JPRB, 0.47754E+01_JPRB, 0.53568E+01_JPRB, 0.78280E+01_JPRB ,&
&  0.77045E+01_JPRB, 0.80852E+01_JPRB, 0.88119E+01_JPRB, 0.10494E+02_JPRB, 0.96722E+01_JPRB, 0.83615E+01_JPRB, 0.78566E+01_JPRB ,&
&  0.57158E+01_JPRB, 0.47833E+01_JPRB, 0.50940E+01_JPRB, 0.52997E+01_JPRB, 0.46822E+01_JPRB, 0.50702E+01_JPRB, 0.69004E+01_JPRB ,&
&  0.57785E+01_JPRB, 0.59109E+01_JPRB, 0.58763E+01_JPRB, 0.57500E+01_JPRB, 0.34140E+01_JPRB, 0.27349E+01_JPRB, 0.41216E+01_JPRB ,&
&  0.44670E+01_JPRB, 0.28545E+01_JPRB, 0.18838E+01_JPRB, 0.25747E+01_JPRB, 0.19563E+01_JPRB, 0.25188E+01_JPRB, 0.43234E+01_JPRB ,&
&  0.62797E+01_JPRB, 0.88448E+01_JPRB, 0.15501E+02_JPRB, 0.12969E+02_JPRB, 0.11062E+02_JPRB, 0.10882E+02_JPRB, 0.98409E+01_JPRB ,&
&  0.94850E+01_JPRB, 0.68799E+01_JPRB, 0.53930E+01_JPRB, 0.36628E+01_JPRB, 0.25887E+01_JPRB, 0.21302E+01_JPRB, 0.19780E+01_JPRB ,&
&  0.19206E+01_JPRB, 0.19350E+01_JPRB, 0.19364E+01_JPRB, 0.18544E+01_JPRB, 0.17474E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.17202E+01_JPRB, 0.15933E+01_JPRB, 0.13999E+01_JPRB, 0.13715E+01_JPRB, 0.14156E+01_JPRB, 0.14799E+01_JPRB, 0.14879E+01_JPRB ,&
&  0.15131E+01_JPRB, 0.15588E+01_JPRB, 0.16200E+01_JPRB, 0.15991E+01_JPRB, 0.15361E+01_JPRB, 0.15103E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.14528E+01_JPRB, 0.15574E+01_JPRB, 0.16848E+01_JPRB, 0.17439E+01_JPRB, 0.18656E+01_JPRB, 0.19534E+01_JPRB, 0.24065E+01_JPRB ,&
&  0.28610E+01_JPRB, 0.30050E+01_JPRB, 0.32762E+01_JPRB, 0.36838E+01_JPRB, 0.40606E+01_JPRB, 0.35934E+01_JPRB, 0.35533E+01_JPRB ,&
&  0.30638E+01_JPRB, 0.26535E+01_JPRB, 0.25190E+01_JPRB, 0.24791E+01_JPRB, 0.26047E+01_JPRB, 0.26874E+01_JPRB, 0.26033E+01_JPRB ,&
&  0.22761E+01_JPRB, 0.19507E+01_JPRB, 0.17847E+01_JPRB, 0.16617E+01_JPRB, 0.15908E+01_JPRB, 0.16511E+01_JPRB, 0.19315E+01_JPRB ,&
&  0.25736E+01_JPRB, 0.32951E+01_JPRB, 0.35938E+01_JPRB, 0.37187E+01_JPRB, 0.48573E+01_JPRB, 0.51063E+01_JPRB, 0.54681E+01_JPRB ,&
&  0.58590E+01_JPRB, 0.62462E+01_JPRB, 0.86323E+01_JPRB, 0.94682E+01_JPRB, 0.91332E+01_JPRB, 0.87486E+01_JPRB, 0.77415E+01_JPRB ,&
&  0.68822E+01_JPRB, 0.62130E+01_JPRB, 0.53283E+01_JPRB, 0.54535E+01_JPRB, 0.50353E+01_JPRB, 0.44446E+01_JPRB, 0.52311E+01_JPRB ,&
&  0.56329E+01_JPRB, 0.54214E+01_JPRB, 0.44984E+01_JPRB, 0.36521E+01_JPRB, 0.34710E+01_JPRB, 0.26172E+01_JPRB, 0.18836E+01_JPRB ,&
&  0.17065E+01_JPRB, 0.16512E+01_JPRB, 0.14807E+01_JPRB, 0.15728E+01_JPRB, 0.14977E+01_JPRB, 0.21718E+01_JPRB, 0.38117E+01_JPRB ,&
&  0.83054E+01_JPRB, 0.13247E+02_JPRB, 0.15545E+02_JPRB, 0.12354E+02_JPRB, 0.10420E+02_JPRB, 0.10435E+02_JPRB, 0.11067E+02_JPRB ,&
&  0.72430E+01_JPRB, 0.55924E+01_JPRB, 0.37215E+01_JPRB, 0.25481E+01_JPRB, 0.20298E+01_JPRB, 0.17704E+01_JPRB, 0.16871E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.16377E+01_JPRB, 0.16606E+01_JPRB, 0.17010E+01_JPRB, 0.16861E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.16854E+01_JPRB, 0.14839E+01_JPRB, 0.12721E+01_JPRB, 0.12189E+01_JPRB, 0.12998E+01_JPRB, 0.13317E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.14430E+01_JPRB, 0.15450E+01_JPRB, 0.15979E+01_JPRB, 0.15777E+01_JPRB, 0.15380E+01_JPRB, 0.14995E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.14739E+01_JPRB, 0.15932E+01_JPRB, 0.19117E+01_JPRB, 0.21833E+01_JPRB, 0.25268E+01_JPRB, 0.25030E+01_JPRB, 0.29274E+01_JPRB ,&
&  0.28469E+01_JPRB, 0.29777E+01_JPRB, 0.34879E+01_JPRB, 0.35794E+01_JPRB, 0.37008E+01_JPRB, 0.30761E+01_JPRB, 0.27761E+01_JPRB ,&
&  0.23932E+01_JPRB, 0.22762E+01_JPRB, 0.22454E+01_JPRB, 0.23579E+01_JPRB, 0.24828E+01_JPRB, 0.25029E+01_JPRB, 0.23786E+01_JPRB ,&
&  0.20777E+01_JPRB, 0.19349E+01_JPRB, 0.17418E+01_JPRB, 0.17216E+01_JPRB, 0.17098E+01_JPRB, 0.18350E+01_JPRB, 0.21145E+01_JPRB ,&
&  0.26090E+01_JPRB, 0.30994E+01_JPRB, 0.32857E+01_JPRB, 0.36504E+01_JPRB, 0.37054E+01_JPRB, 0.36617E+01_JPRB, 0.49575E+01_JPRB ,&
&  0.62067E+01_JPRB, 0.71817E+01_JPRB, 0.75012E+01_JPRB, 0.90909E+01_JPRB, 0.88894E+01_JPRB, 0.80975E+01_JPRB, 0.75909E+01_JPRB ,&
&  0.65878E+01_JPRB, 0.60588E+01_JPRB, 0.54087E+01_JPRB, 0.55228E+01_JPRB, 0.59611E+01_JPRB, 0.51454E+01_JPRB, 0.50672E+01_JPRB ,&
&  0.56439E+01_JPRB, 0.53740E+01_JPRB, 0.46852E+01_JPRB, 0.39299E+01_JPRB, 0.46229E+01_JPRB, 0.42947E+01_JPRB, 0.24532E+01_JPRB ,&
&  0.20911E+01_JPRB, 0.18739E+01_JPRB, 0.15785E+01_JPRB, 0.14069E+01_JPRB, 0.15287E+01_JPRB, 0.23135E+01_JPRB, 0.51714E+01_JPRB ,&
&  0.74059E+01_JPRB, 0.13794E+02_JPRB, 0.16163E+02_JPRB, 0.13184E+02_JPRB, 0.10187E+02_JPRB, 0.10428E+02_JPRB, 0.77709E+01_JPRB ,&
&  0.63319E+01_JPRB, 0.45622E+01_JPRB, 0.25643E+01_JPRB, 0.19174E+01_JPRB, 0.17922E+01_JPRB, 0.16916E+01_JPRB, 0.15893E+01_JPRB ,&
&  0.14843E+01_JPRB, 0.14560E+01_JPRB, 0.14984E+01_JPRB, 0.15466E+01_JPRB, 0.15676E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.14536E+01_JPRB, 0.12845E+01_JPRB, 0.11528E+01_JPRB, 0.10765E+01_JPRB, 0.11605E+01_JPRB, 0.12685E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.14231E+01_JPRB, 0.15466E+01_JPRB, 0.15970E+01_JPRB, 0.15907E+01_JPRB, 0.15317E+01_JPRB, 0.15058E+01_JPRB, 0.15242E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.18509E+01_JPRB, 0.22786E+01_JPRB, 0.27552E+01_JPRB, 0.29803E+01_JPRB, 0.29774E+01_JPRB, 0.29772E+01_JPRB ,&
&  0.26320E+01_JPRB, 0.28545E+01_JPRB, 0.34743E+01_JPRB, 0.36793E+01_JPRB, 0.29869E+01_JPRB, 0.26843E+01_JPRB, 0.21551E+01_JPRB ,&
&  0.21160E+01_JPRB, 0.21702E+01_JPRB, 0.21876E+01_JPRB, 0.22339E+01_JPRB, 0.22437E+01_JPRB, 0.22136E+01_JPRB, 0.21590E+01_JPRB ,&
&  0.19804E+01_JPRB, 0.18516E+01_JPRB, 0.17789E+01_JPRB, 0.17730E+01_JPRB, 0.19220E+01_JPRB, 0.21674E+01_JPRB, 0.24383E+01_JPRB ,&
&  0.27708E+01_JPRB, 0.30363E+01_JPRB, 0.32479E+01_JPRB, 0.36637E+01_JPRB, 0.30261E+01_JPRB, 0.40982E+01_JPRB, 0.49567E+01_JPRB ,&
&  0.55435E+01_JPRB, 0.61421E+01_JPRB, 0.62661E+01_JPRB, 0.74132E+01_JPRB, 0.80658E+01_JPRB, 0.76006E+01_JPRB, 0.67706E+01_JPRB ,&
&  0.61697E+01_JPRB, 0.53467E+01_JPRB, 0.56891E+01_JPRB, 0.60966E+01_JPRB, 0.64174E+01_JPRB, 0.74641E+01_JPRB, 0.55137E+01_JPRB ,&
&  0.49606E+01_JPRB, 0.55672E+01_JPRB, 0.50333E+01_JPRB, 0.44191E+01_JPRB, 0.48652E+01_JPRB, 0.56222E+01_JPRB, 0.59617E+01_JPRB ,&
&  0.28935E+01_JPRB, 0.19162E+01_JPRB, 0.19198E+01_JPRB, 0.17586E+01_JPRB, 0.15761E+01_JPRB, 0.24940E+01_JPRB, 0.90930E+01_JPRB ,&
&  0.75742E+01_JPRB, 0.12670E+02_JPRB, 0.14086E+02_JPRB, 0.11682E+02_JPRB, 0.96254E+01_JPRB, 0.81869E+01_JPRB, 0.67528E+01_JPRB ,&
&  0.46868E+01_JPRB, 0.32439E+01_JPRB, 0.21590E+01_JPRB, 0.17692E+01_JPRB, 0.16777E+01_JPRB, 0.16103E+01_JPRB, 0.15277E+01_JPRB ,&
&  0.14537E+01_JPRB, 0.13861E+01_JPRB, 0.14051E+01_JPRB, 0.13712E+01_JPRB, 0.14615E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.12976E+01_JPRB, 0.11906E+01_JPRB, 0.11671E+01_JPRB, 0.11555E+01_JPRB, 0.12084E+01_JPRB, 0.12543E+01_JPRB, 0.13373E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.15968E+01_JPRB, 0.16738E+01_JPRB, 0.16719E+01_JPRB, 0.16228E+01_JPRB, 0.16612E+01_JPRB, 0.17804E+01_JPRB ,&
&  0.19806E+01_JPRB, 0.22868E+01_JPRB, 0.27003E+01_JPRB, 0.30135E+01_JPRB, 0.31565E+01_JPRB, 0.28567E+01_JPRB, 0.25691E+01_JPRB ,&
&  0.23215E+01_JPRB, 0.24553E+01_JPRB, 0.24925E+01_JPRB, 0.25818E+01_JPRB, 0.25615E+01_JPRB, 0.24456E+01_JPRB, 0.19014E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.18863E+01_JPRB, 0.18579E+01_JPRB, 0.18936E+01_JPRB, 0.19276E+01_JPRB, 0.19883E+01_JPRB, 0.19677E+01_JPRB ,&
&  0.18393E+01_JPRB, 0.18186E+01_JPRB, 0.18275E+01_JPRB, 0.19351E+01_JPRB, 0.21933E+01_JPRB, 0.24427E+01_JPRB, 0.26936E+01_JPRB ,&
&  0.29301E+01_JPRB, 0.30762E+01_JPRB, 0.32356E+01_JPRB, 0.32798E+01_JPRB, 0.36283E+01_JPRB, 0.41867E+01_JPRB, 0.46786E+01_JPRB ,&
&  0.50380E+01_JPRB, 0.56872E+01_JPRB, 0.62350E+01_JPRB, 0.66330E+01_JPRB, 0.70647E+01_JPRB, 0.69343E+01_JPRB, 0.62521E+01_JPRB ,&
&  0.58057E+01_JPRB, 0.57432E+01_JPRB, 0.58806E+01_JPRB, 0.61361E+01_JPRB, 0.64837E+01_JPRB, 0.74074E+01_JPRB, 0.71349E+01_JPRB ,&
&  0.59475E+01_JPRB, 0.48950E+01_JPRB, 0.46625E+01_JPRB, 0.51045E+01_JPRB, 0.56056E+01_JPRB, 0.64559E+01_JPRB, 0.93261E+01_JPRB ,&
&  0.93427E+01_JPRB, 0.44128E+01_JPRB, 0.29696E+01_JPRB, 0.34193E+01_JPRB, 0.23855E+01_JPRB, 0.27300E+01_JPRB, 0.53958E+01_JPRB ,&
&  0.76355E+01_JPRB, 0.11083E+02_JPRB, 0.12292E+02_JPRB, 0.94582E+01_JPRB, 0.79313E+01_JPRB, 0.57668E+01_JPRB, 0.44658E+01_JPRB ,&
&  0.32937E+01_JPRB, 0.23666E+01_JPRB, 0.17811E+01_JPRB, 0.16005E+01_JPRB, 0.14717E+01_JPRB, 0.14700E+01_JPRB, 0.15166E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.13221E+01_JPRB, 0.13207E+01_JPRB, 0.13237E+01_JPRB, 0.13041E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.12963E+01_JPRB, 0.12557E+01_JPRB, 0.12809E+01_JPRB, 0.13151E+01_JPRB, 0.13656E+01_JPRB, 0.14201E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.16030E+01_JPRB, 0.17265E+01_JPRB, 0.18020E+01_JPRB, 0.18395E+01_JPRB, 0.18846E+01_JPRB, 0.20016E+01_JPRB, 0.21691E+01_JPRB ,&
&  0.23811E+01_JPRB, 0.26184E+01_JPRB, 0.28769E+01_JPRB, 0.31439E+01_JPRB, 0.34271E+01_JPRB, 0.34515E+01_JPRB, 0.21100E+01_JPRB ,&
&  0.18538E+01_JPRB, 0.22300E+01_JPRB, 0.21010E+01_JPRB, 0.21317E+01_JPRB, 0.21158E+01_JPRB, 0.21166E+01_JPRB, 0.18120E+01_JPRB ,&
&  0.16729E+01_JPRB, 0.16298E+01_JPRB, 0.16389E+01_JPRB, 0.16928E+01_JPRB, 0.17601E+01_JPRB, 0.18848E+01_JPRB, 0.19232E+01_JPRB ,&
&  0.19493E+01_JPRB, 0.20485E+01_JPRB, 0.21627E+01_JPRB, 0.23875E+01_JPRB, 0.26462E+01_JPRB, 0.28488E+01_JPRB, 0.29782E+01_JPRB ,&
&  0.30608E+01_JPRB, 0.31417E+01_JPRB, 0.32932E+01_JPRB, 0.34193E+01_JPRB, 0.38778E+01_JPRB, 0.41962E+01_JPRB, 0.46012E+01_JPRB ,&
&  0.47890E+01_JPRB, 0.48125E+01_JPRB, 0.55166E+01_JPRB, 0.60862E+01_JPRB, 0.63336E+01_JPRB, 0.59831E+01_JPRB, 0.55119E+01_JPRB ,&
&  0.58614E+01_JPRB, 0.63271E+01_JPRB, 0.68951E+01_JPRB, 0.62230E+01_JPRB, 0.60954E+01_JPRB, 0.66868E+01_JPRB, 0.72506E+01_JPRB ,&
&  0.65027E+01_JPRB, 0.57717E+01_JPRB, 0.57690E+01_JPRB, 0.58880E+01_JPRB, 0.62745E+01_JPRB, 0.66971E+01_JPRB, 0.83308E+01_JPRB ,&
&  0.95021E+01_JPRB, 0.74149E+01_JPRB, 0.44674E+01_JPRB, 0.28058E+01_JPRB, 0.28981E+01_JPRB, 0.35851E+01_JPRB, 0.48193E+01_JPRB ,&
&  0.78580E+01_JPRB, 0.90509E+01_JPRB, 0.94682E+01_JPRB, 0.88467E+01_JPRB, 0.49495E+01_JPRB, 0.33793E+01_JPRB, 0.26982E+01_JPRB ,&
&  0.18716E+01_JPRB, 0.14750E+01_JPRB, 0.14206E+01_JPRB, 0.13582E+01_JPRB, 0.12899E+01_JPRB, 0.13599E+01_JPRB, 0.15136E+01_JPRB ,&
&  0.14356E+01_JPRB, 0.12583E+01_JPRB, 0.12658E+01_JPRB, 0.12597E+01_JPRB, 0.12685E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.13160E+01_JPRB, 0.13221E+01_JPRB, 0.13637E+01_JPRB, 0.14501E+01_JPRB, 0.15687E+01_JPRB, 0.16917E+01_JPRB, 0.17048E+01_JPRB ,&
&  0.17246E+01_JPRB, 0.18088E+01_JPRB, 0.19005E+01_JPRB, 0.19927E+01_JPRB, 0.20888E+01_JPRB, 0.22148E+01_JPRB, 0.23539E+01_JPRB ,&
&  0.24980E+01_JPRB, 0.26471E+01_JPRB, 0.27958E+01_JPRB, 0.29711E+01_JPRB, 0.31211E+01_JPRB, 0.31228E+01_JPRB, 0.23039E+01_JPRB ,&
&  0.16949E+01_JPRB, 0.25851E+01_JPRB, 0.20078E+01_JPRB, 0.21026E+01_JPRB, 0.21480E+01_JPRB, 0.22784E+01_JPRB, 0.19452E+01_JPRB ,&
&  0.16256E+01_JPRB, 0.15958E+01_JPRB, 0.16356E+01_JPRB, 0.16180E+01_JPRB, 0.16482E+01_JPRB, 0.18119E+01_JPRB, 0.20156E+01_JPRB ,&
&  0.21549E+01_JPRB, 0.23333E+01_JPRB, 0.25505E+01_JPRB, 0.27883E+01_JPRB, 0.29307E+01_JPRB, 0.30537E+01_JPRB, 0.31514E+01_JPRB ,&
&  0.32505E+01_JPRB, 0.33720E+01_JPRB, 0.34492E+01_JPRB, 0.36901E+01_JPRB, 0.39098E+01_JPRB, 0.42548E+01_JPRB, 0.46255E+01_JPRB ,&
&  0.47097E+01_JPRB, 0.46793E+01_JPRB, 0.50483E+01_JPRB, 0.51116E+01_JPRB, 0.52066E+01_JPRB, 0.56782E+01_JPRB, 0.54125E+01_JPRB ,&
&  0.61482E+01_JPRB, 0.61828E+01_JPRB, 0.69923E+01_JPRB, 0.57603E+01_JPRB, 0.55084E+01_JPRB, 0.59118E+01_JPRB, 0.58833E+01_JPRB ,&
&  0.55015E+01_JPRB, 0.58562E+01_JPRB, 0.59956E+01_JPRB, 0.62371E+01_JPRB, 0.65428E+01_JPRB, 0.68234E+01_JPRB, 0.73859E+01_JPRB ,&
&  0.63516E+01_JPRB, 0.36071E+01_JPRB, 0.25507E+01_JPRB, 0.21811E+01_JPRB, 0.23264E+01_JPRB, 0.32214E+01_JPRB, 0.54123E+01_JPRB ,&
&  0.81504E+01_JPRB, 0.80402E+01_JPRB, 0.76516E+01_JPRB, 0.50640E+01_JPRB, 0.26910E+01_JPRB, 0.19566E+01_JPRB, 0.14689E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.10055E+01_JPRB, 0.11033E+01_JPRB, 0.11457E+01_JPRB, 0.11239E+01_JPRB, 0.11882E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.12150E+01_JPRB, 0.12858E+01_JPRB, 0.12702E+01_JPRB, 0.12893E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.13008E+01_JPRB, 0.13282E+01_JPRB, 0.14308E+01_JPRB, 0.15801E+01_JPRB, 0.18122E+01_JPRB, 0.20641E+01_JPRB, 0.21681E+01_JPRB ,&
&  0.19687E+01_JPRB, 0.17590E+01_JPRB, 0.19004E+01_JPRB, 0.20343E+01_JPRB, 0.21407E+01_JPRB, 0.22766E+01_JPRB, 0.24159E+01_JPRB ,&
&  0.25754E+01_JPRB, 0.27334E+01_JPRB, 0.28831E+01_JPRB, 0.30582E+01_JPRB, 0.32405E+01_JPRB, 0.34138E+01_JPRB, 0.32377E+01_JPRB ,&
&  0.23953E+01_JPRB, 0.19773E+01_JPRB, 0.22392E+01_JPRB, 0.20975E+01_JPRB, 0.20278E+01_JPRB, 0.21530E+01_JPRB, 0.21435E+01_JPRB ,&
&  0.18828E+01_JPRB, 0.16862E+01_JPRB, 0.17189E+01_JPRB, 0.16735E+01_JPRB, 0.14403E+01_JPRB, 0.16027E+01_JPRB, 0.19121E+01_JPRB ,&
&  0.21148E+01_JPRB, 0.23216E+01_JPRB, 0.25614E+01_JPRB, 0.27804E+01_JPRB, 0.29258E+01_JPRB, 0.31196E+01_JPRB, 0.32961E+01_JPRB ,&
&  0.34423E+01_JPRB, 0.35988E+01_JPRB, 0.36631E+01_JPRB, 0.37126E+01_JPRB, 0.39436E+01_JPRB, 0.42222E+01_JPRB, 0.44294E+01_JPRB ,&
&  0.45184E+01_JPRB, 0.47994E+01_JPRB, 0.49812E+01_JPRB, 0.50616E+01_JPRB, 0.49235E+01_JPRB, 0.54981E+01_JPRB, 0.55260E+01_JPRB ,&
&  0.58089E+01_JPRB, 0.56782E+01_JPRB, 0.61377E+01_JPRB, 0.50738E+01_JPRB, 0.49405E+01_JPRB, 0.50471E+01_JPRB, 0.51245E+01_JPRB ,&
&  0.50812E+01_JPRB, 0.54161E+01_JPRB, 0.55279E+01_JPRB, 0.59179E+01_JPRB, 0.64276E+01_JPRB, 0.62703E+01_JPRB, 0.64018E+01_JPRB ,&
&  0.40398E+01_JPRB, 0.25884E+01_JPRB, 0.18514E+01_JPRB, 0.18958E+01_JPRB, 0.21477E+01_JPRB, 0.29770E+01_JPRB, 0.45013E+01_JPRB ,&
&  0.60129E+01_JPRB, 0.43849E+01_JPRB, 0.35872E+01_JPRB, 0.29196E+01_JPRB, 0.16348E+01_JPRB, 0.11741E+01_JPRB, 0.96543E+00_JPRB ,&
&  0.85989E+00_JPRB, 0.87913E+00_JPRB, 0.95684E+00_JPRB, 0.97661E+00_JPRB, 0.95907E+00_JPRB, 0.10165E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.11418E+01_JPRB, 0.11664E+01_JPRB, 0.12408E+01_JPRB, 0.12428E+01_JPRB, 0.12925E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.11901E+01_JPRB, 0.12704E+01_JPRB, 0.13373E+01_JPRB, 0.14219E+01_JPRB, 0.14983E+01_JPRB, 0.16085E+01_JPRB, 0.16368E+01_JPRB ,&
&  0.15810E+01_JPRB, 0.15336E+01_JPRB, 0.16330E+01_JPRB, 0.17615E+01_JPRB, 0.18783E+01_JPRB, 0.19724E+01_JPRB, 0.20639E+01_JPRB ,&
&  0.21480E+01_JPRB, 0.22200E+01_JPRB, 0.22954E+01_JPRB, 0.23597E+01_JPRB, 0.24758E+01_JPRB, 0.26292E+01_JPRB, 0.27273E+01_JPRB ,&
&  0.23456E+01_JPRB, 0.16748E+01_JPRB, 0.19275E+01_JPRB, 0.19833E+01_JPRB, 0.18642E+01_JPRB, 0.19531E+01_JPRB, 0.20703E+01_JPRB ,&
&  0.18132E+01_JPRB, 0.16691E+01_JPRB, 0.16827E+01_JPRB, 0.16834E+01_JPRB, 0.13297E+01_JPRB, 0.14299E+01_JPRB, 0.16796E+01_JPRB ,&
&  0.19299E+01_JPRB, 0.21728E+01_JPRB, 0.23964E+01_JPRB, 0.25654E+01_JPRB, 0.27397E+01_JPRB, 0.29829E+01_JPRB, 0.32251E+01_JPRB ,&
&  0.34473E+01_JPRB, 0.36031E+01_JPRB, 0.36877E+01_JPRB, 0.37726E+01_JPRB, 0.39015E+01_JPRB, 0.40933E+01_JPRB, 0.41965E+01_JPRB ,&
&  0.43318E+01_JPRB, 0.44801E+01_JPRB, 0.47868E+01_JPRB, 0.51416E+01_JPRB, 0.53543E+01_JPRB, 0.50194E+01_JPRB, 0.50373E+01_JPRB ,&
&  0.49750E+01_JPRB, 0.47372E+01_JPRB, 0.48813E+01_JPRB, 0.50565E+01_JPRB, 0.38537E+01_JPRB, 0.39189E+01_JPRB, 0.41153E+01_JPRB ,&
&  0.46602E+01_JPRB, 0.46786E+01_JPRB, 0.48170E+01_JPRB, 0.50861E+01_JPRB, 0.52149E+01_JPRB, 0.51204E+01_JPRB, 0.46747E+01_JPRB ,&
&  0.32379E+01_JPRB, 0.20578E+01_JPRB, 0.16639E+01_JPRB, 0.17836E+01_JPRB, 0.21053E+01_JPRB, 0.30444E+01_JPRB, 0.32686E+01_JPRB ,&
&  0.39652E+01_JPRB, 0.23792E+01_JPRB, 0.20760E+01_JPRB, 0.23051E+01_JPRB, 0.14448E+01_JPRB, 0.10336E+01_JPRB, 0.85451E+00_JPRB ,&
&  0.79745E+00_JPRB, 0.83743E+00_JPRB, 0.85186E+00_JPRB, 0.84607E+00_JPRB, 0.84517E+00_JPRB, 0.90103E+00_JPRB, 0.97326E+00_JPRB ,&
&  0.10041E+01_JPRB, 0.10385E+01_JPRB, 0.10860E+01_JPRB, 0.11365E+01_JPRB, 0.11970E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.93133E+00_JPRB, 0.93876E+00_JPRB, 0.97898E+00_JPRB, 0.10632E+01_JPRB, 0.11305E+01_JPRB, 0.11893E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.12715E+01_JPRB, 0.13200E+01_JPRB, 0.13885E+01_JPRB, 0.14406E+01_JPRB, 0.14848E+01_JPRB, 0.15824E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.16555E+01_JPRB, 0.16334E+01_JPRB, 0.16608E+01_JPRB, 0.17220E+01_JPRB, 0.18779E+01_JPRB, 0.21360E+01_JPRB, 0.24045E+01_JPRB ,&
&  0.26752E+01_JPRB, 0.25997E+01_JPRB, 0.24304E+01_JPRB, 0.16313E+01_JPRB, 0.12593E+01_JPRB, 0.14462E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.16303E+01_JPRB, 0.16540E+01_JPRB, 0.15010E+01_JPRB, 0.13937E+01_JPRB, 0.13225E+01_JPRB, 0.13313E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.16390E+01_JPRB, 0.18519E+01_JPRB, 0.20714E+01_JPRB, 0.22252E+01_JPRB, 0.23932E+01_JPRB, 0.26341E+01_JPRB, 0.29138E+01_JPRB ,&
&  0.31573E+01_JPRB, 0.33224E+01_JPRB, 0.34133E+01_JPRB, 0.35915E+01_JPRB, 0.37222E+01_JPRB, 0.39211E+01_JPRB, 0.40838E+01_JPRB ,&
&  0.42432E+01_JPRB, 0.43488E+01_JPRB, 0.45385E+01_JPRB, 0.47464E+01_JPRB, 0.47116E+01_JPRB, 0.42788E+01_JPRB, 0.40115E+01_JPRB ,&
&  0.39110E+01_JPRB, 0.36779E+01_JPRB, 0.37670E+01_JPRB, 0.41271E+01_JPRB, 0.30577E+01_JPRB, 0.33090E+01_JPRB, 0.38216E+01_JPRB ,&
&  0.38545E+01_JPRB, 0.39365E+01_JPRB, 0.37861E+01_JPRB, 0.33611E+01_JPRB, 0.32393E+01_JPRB, 0.33004E+01_JPRB, 0.31929E+01_JPRB ,&
&  0.26190E+01_JPRB, 0.18849E+01_JPRB, 0.15354E+01_JPRB, 0.14000E+01_JPRB, 0.17167E+01_JPRB, 0.26645E+01_JPRB, 0.22810E+01_JPRB ,&
&  0.22430E+01_JPRB, 0.19135E+01_JPRB, 0.16502E+01_JPRB, 0.22840E+01_JPRB, 0.18140E+01_JPRB, 0.11812E+01_JPRB, 0.90020E+00_JPRB ,&
&  0.79259E+00_JPRB, 0.79389E+00_JPRB, 0.80393E+00_JPRB, 0.78965E+00_JPRB, 0.81057E+00_JPRB, 0.85657E+00_JPRB, 0.86264E+00_JPRB ,&
&  0.92124E+00_JPRB, 0.93141E+00_JPRB, 0.95933E+00_JPRB, 0.98276E+00_JPRB, 0.94308E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.64834E+00_JPRB, 0.63588E+00_JPRB, 0.65474E+00_JPRB, 0.72952E+00_JPRB, 0.77960E+00_JPRB, 0.81536E+00_JPRB, 0.87291E+00_JPRB ,&
&  0.94305E+00_JPRB, 0.10527E+01_JPRB, 0.11406E+01_JPRB, 0.11695E+01_JPRB, 0.11313E+01_JPRB, 0.10915E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.97550E+00_JPRB, 0.90812E+00_JPRB, 0.90172E+00_JPRB, 0.95618E+00_JPRB, 0.10758E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.16895E+01_JPRB, 0.19249E+01_JPRB, 0.25051E+01_JPRB, 0.26080E+01_JPRB, 0.14368E+01_JPRB, 0.15449E+01_JPRB ,&
&  0.20139E+01_JPRB, 0.20626E+01_JPRB, 0.15555E+01_JPRB, 0.13723E+01_JPRB, 0.12788E+01_JPRB, 0.12202E+01_JPRB, 0.12486E+01_JPRB ,&
&  0.13717E+01_JPRB, 0.14999E+01_JPRB, 0.16478E+01_JPRB, 0.18369E+01_JPRB, 0.20428E+01_JPRB, 0.22336E+01_JPRB, 0.24726E+01_JPRB ,&
&  0.26783E+01_JPRB, 0.27540E+01_JPRB, 0.28202E+01_JPRB, 0.29435E+01_JPRB, 0.31816E+01_JPRB, 0.34704E+01_JPRB, 0.37320E+01_JPRB ,&
&  0.39185E+01_JPRB, 0.40129E+01_JPRB, 0.42176E+01_JPRB, 0.42883E+01_JPRB, 0.39171E+01_JPRB, 0.35781E+01_JPRB, 0.31641E+01_JPRB ,&
&  0.28485E+01_JPRB, 0.24902E+01_JPRB, 0.23445E+01_JPRB, 0.30970E+01_JPRB, 0.33167E+01_JPRB, 0.30777E+01_JPRB, 0.31124E+01_JPRB ,&
&  0.32041E+01_JPRB, 0.31850E+01_JPRB, 0.28842E+01_JPRB, 0.23175E+01_JPRB, 0.19371E+01_JPRB, 0.21991E+01_JPRB, 0.23188E+01_JPRB ,&
&  0.20379E+01_JPRB, 0.15544E+01_JPRB, 0.13162E+01_JPRB, 0.12324E+01_JPRB, 0.12660E+01_JPRB, 0.18516E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.12855E+01_JPRB, 0.14015E+01_JPRB, 0.13035E+01_JPRB, 0.18688E+01_JPRB, 0.26765E+01_JPRB, 0.14662E+01_JPRB, 0.95500E+00_JPRB ,&
&  0.77094E+00_JPRB, 0.76361E+00_JPRB, 0.76510E+00_JPRB, 0.76337E+00_JPRB, 0.76597E+00_JPRB, 0.78475E+00_JPRB, 0.78573E+00_JPRB ,&
&  0.79732E+00_JPRB, 0.77816E+00_JPRB, 0.77513E+00_JPRB, 0.72949E+00_JPRB, 0.66380E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.66269E+00_JPRB, 0.65702E+00_JPRB, 0.64314E+00_JPRB, 0.63718E+00_JPRB, 0.64442E+00_JPRB, 0.65692E+00_JPRB, 0.67751E+00_JPRB ,&
&  0.70962E+00_JPRB, 0.74824E+00_JPRB, 0.78572E+00_JPRB, 0.82761E+00_JPRB, 0.83410E+00_JPRB, 0.78101E+00_JPRB, 0.77575E+00_JPRB ,&
&  0.76808E+00_JPRB, 0.77925E+00_JPRB, 0.77325E+00_JPRB, 0.85782E+00_JPRB, 0.88382E+00_JPRB, 0.89527E+00_JPRB, 0.10003E+01_JPRB ,&
&  0.99715E+00_JPRB, 0.11080E+01_JPRB, 0.11909E+01_JPRB, 0.13206E+01_JPRB, 0.21040E+01_JPRB, 0.19565E+01_JPRB, 0.14470E+01_JPRB ,&
&  0.17381E+01_JPRB, 0.17700E+01_JPRB, 0.14511E+01_JPRB, 0.12183E+01_JPRB, 0.11469E+01_JPRB, 0.10452E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.11575E+01_JPRB, 0.13171E+01_JPRB, 0.14726E+01_JPRB, 0.15954E+01_JPRB, 0.17143E+01_JPRB, 0.18609E+01_JPRB ,&
&  0.19525E+01_JPRB, 0.20331E+01_JPRB, 0.20070E+01_JPRB, 0.20017E+01_JPRB, 0.22798E+01_JPRB, 0.25261E+01_JPRB, 0.27009E+01_JPRB ,&
&  0.28381E+01_JPRB, 0.29983E+01_JPRB, 0.32183E+01_JPRB, 0.30845E+01_JPRB, 0.27980E+01_JPRB, 0.25962E+01_JPRB, 0.21598E+01_JPRB ,&
&  0.17745E+01_JPRB, 0.14664E+01_JPRB, 0.13991E+01_JPRB, 0.18807E+01_JPRB, 0.20965E+01_JPRB, 0.19604E+01_JPRB, 0.23234E+01_JPRB ,&
&  0.24424E+01_JPRB, 0.25087E+01_JPRB, 0.21666E+01_JPRB, 0.19220E+01_JPRB, 0.16337E+01_JPRB, 0.16594E+01_JPRB, 0.17334E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.11875E+01_JPRB, 0.10569E+01_JPRB, 0.99891E+00_JPRB, 0.99410E+00_JPRB, 0.12585E+01_JPRB, 0.14798E+01_JPRB ,&
&  0.14292E+01_JPRB, 0.12267E+01_JPRB, 0.12100E+01_JPRB, 0.17685E+01_JPRB, 0.15566E+01_JPRB, 0.98003E+00_JPRB, 0.72706E+00_JPRB ,&
&  0.67675E+00_JPRB, 0.68244E+00_JPRB, 0.67469E+00_JPRB, 0.67029E+00_JPRB, 0.68018E+00_JPRB, 0.69775E+00_JPRB, 0.68275E+00_JPRB ,&
&  0.67743E+00_JPRB, 0.65882E+00_JPRB, 0.64612E+00_JPRB, 0.63592E+00_JPRB, 0.63919E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.79690E+00_JPRB, 0.80186E+00_JPRB, 0.80879E+00_JPRB, 0.81850E+00_JPRB, 0.83970E+00_JPRB, 0.86364E+00_JPRB, 0.88364E+00_JPRB ,&
&  0.92042E+00_JPRB, 0.94850E+00_JPRB, 0.97608E+00_JPRB, 0.98908E+00_JPRB, 0.10040E+01_JPRB, 0.10146E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.10672E+01_JPRB, 0.10676E+01_JPRB, 0.10766E+01_JPRB, 0.11041E+01_JPRB, 0.11230E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.10843E+01_JPRB, 0.10587E+01_JPRB, 0.10816E+01_JPRB, 0.11220E+01_JPRB, 0.13235E+01_JPRB, 0.14569E+01_JPRB, 0.14437E+01_JPRB ,&
&  0.14276E+01_JPRB, 0.14678E+01_JPRB, 0.15161E+01_JPRB, 0.11897E+01_JPRB, 0.10877E+01_JPRB, 0.10249E+01_JPRB, 0.96069E+00_JPRB ,&
&  0.96571E+00_JPRB, 0.99263E+00_JPRB, 0.10320E+01_JPRB, 0.10732E+01_JPRB, 0.11274E+01_JPRB, 0.11507E+01_JPRB, 0.12045E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.12303E+01_JPRB, 0.12654E+01_JPRB, 0.13320E+01_JPRB, 0.14751E+01_JPRB, 0.16947E+01_JPRB, 0.16941E+01_JPRB ,&
&  0.16921E+01_JPRB, 0.18004E+01_JPRB, 0.16762E+01_JPRB, 0.16371E+01_JPRB, 0.15587E+01_JPRB, 0.13498E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.11014E+01_JPRB, 0.10612E+01_JPRB, 0.95010E+00_JPRB, 0.12527E+01_JPRB, 0.15055E+01_JPRB, 0.16114E+01_JPRB, 0.18351E+01_JPRB ,&
&  0.19484E+01_JPRB, 0.17407E+01_JPRB, 0.14595E+01_JPRB, 0.14577E+01_JPRB, 0.14558E+01_JPRB, 0.12610E+01_JPRB, 0.12460E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.10060E+01_JPRB, 0.96178E+00_JPRB, 0.97066E+00_JPRB, 0.11225E+01_JPRB, 0.16950E+01_JPRB, 0.18017E+01_JPRB ,&
&  0.14545E+01_JPRB, 0.12346E+01_JPRB, 0.14292E+01_JPRB, 0.22237E+01_JPRB, 0.12494E+01_JPRB, 0.88564E+00_JPRB, 0.69542E+00_JPRB ,&
&  0.68944E+00_JPRB, 0.69106E+00_JPRB, 0.69345E+00_JPRB, 0.69737E+00_JPRB, 0.70407E+00_JPRB, 0.72142E+00_JPRB, 0.71138E+00_JPRB ,&
&  0.70088E+00_JPRB, 0.69069E+00_JPRB, 0.70199E+00_JPRB, 0.72176E+00_JPRB, 0.76363E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.90054E+00_JPRB, 0.89778E+00_JPRB, 0.91125E+00_JPRB, 0.93013E+00_JPRB, 0.94849E+00_JPRB, 0.97009E+00_JPRB, 0.10044E+01_JPRB ,&
&  0.10445E+01_JPRB, 0.10759E+01_JPRB, 0.11055E+01_JPRB, 0.11258E+01_JPRB, 0.11314E+01_JPRB, 0.11442E+01_JPRB, 0.11573E+01_JPRB ,&
&  0.11724E+01_JPRB, 0.11670E+01_JPRB, 0.11849E+01_JPRB, 0.12157E+01_JPRB, 0.12531E+01_JPRB, 0.12675E+01_JPRB, 0.12818E+01_JPRB ,&
&  0.13001E+01_JPRB, 0.13434E+01_JPRB, 0.13928E+01_JPRB, 0.14524E+01_JPRB, 0.15521E+01_JPRB, 0.17257E+01_JPRB, 0.18961E+01_JPRB ,&
&  0.14673E+01_JPRB, 0.15625E+01_JPRB, 0.14463E+01_JPRB, 0.11870E+01_JPRB, 0.11076E+01_JPRB, 0.11213E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10513E+01_JPRB, 0.10605E+01_JPRB, 0.10698E+01_JPRB, 0.10889E+01_JPRB, 0.11250E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.11573E+01_JPRB, 0.11716E+01_JPRB, 0.12147E+01_JPRB, 0.11718E+01_JPRB, 0.13756E+01_JPRB, 0.13176E+01_JPRB ,&
&  0.12384E+01_JPRB, 0.11878E+01_JPRB, 0.92758E+00_JPRB, 0.93294E+00_JPRB, 0.93203E+00_JPRB, 0.82346E+00_JPRB, 0.76379E+00_JPRB ,&
&  0.93499E+00_JPRB, 0.10736E+01_JPRB, 0.97540E+00_JPRB, 0.12195E+01_JPRB, 0.13684E+01_JPRB, 0.14360E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.14522E+01_JPRB, 0.12844E+01_JPRB, 0.11422E+01_JPRB, 0.10781E+01_JPRB, 0.10604E+01_JPRB, 0.10299E+01_JPRB, 0.98707E+00_JPRB ,&
&  0.93930E+00_JPRB, 0.91582E+00_JPRB, 0.88989E+00_JPRB, 0.89239E+00_JPRB, 0.12678E+01_JPRB, 0.15781E+01_JPRB, 0.23431E+01_JPRB ,&
&  0.13383E+01_JPRB, 0.11669E+01_JPRB, 0.95943E+00_JPRB, 0.13912E+01_JPRB, 0.10357E+01_JPRB, 0.73558E+00_JPRB, 0.75914E+00_JPRB ,&
&  0.81153E+00_JPRB, 0.84610E+00_JPRB, 0.83913E+00_JPRB, 0.81845E+00_JPRB, 0.81288E+00_JPRB, 0.81308E+00_JPRB, 0.78683E+00_JPRB ,&
&  0.76997E+00_JPRB, 0.78183E+00_JPRB, 0.81468E+00_JPRB, 0.82840E+00_JPRB, 0.86515E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.95128E+00_JPRB, 0.97325E+00_JPRB, 0.99686E+00_JPRB, 0.10233E+01_JPRB, 0.10487E+01_JPRB, 0.10725E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.11316E+01_JPRB, 0.11557E+01_JPRB, 0.11795E+01_JPRB, 0.11925E+01_JPRB, 0.11980E+01_JPRB, 0.12130E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.12189E+01_JPRB, 0.12204E+01_JPRB, 0.12384E+01_JPRB, 0.12501E+01_JPRB, 0.12536E+01_JPRB, 0.12529E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12756E+01_JPRB, 0.13133E+01_JPRB, 0.13434E+01_JPRB, 0.13826E+01_JPRB, 0.14147E+01_JPRB, 0.14274E+01_JPRB, 0.15214E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.17636E+01_JPRB, 0.16851E+01_JPRB, 0.13357E+01_JPRB, 0.12628E+01_JPRB, 0.11858E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11458E+01_JPRB, 0.11578E+01_JPRB, 0.11870E+01_JPRB, 0.12168E+01_JPRB, 0.12511E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.12829E+01_JPRB, 0.12990E+01_JPRB, 0.13156E+01_JPRB, 0.13277E+01_JPRB, 0.13444E+01_JPRB, 0.13398E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.11501E+01_JPRB, 0.86510E+00_JPRB, 0.81489E+00_JPRB, 0.83860E+00_JPRB, 0.79697E+00_JPRB, 0.72471E+00_JPRB ,&
&  0.88344E+00_JPRB, 0.10412E+01_JPRB, 0.11103E+01_JPRB, 0.12037E+01_JPRB, 0.12317E+01_JPRB, 0.11731E+01_JPRB, 0.11345E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.92015E+00_JPRB, 0.85307E+00_JPRB, 0.81708E+00_JPRB, 0.83426E+00_JPRB, 0.83707E+00_JPRB, 0.82299E+00_JPRB ,&
&  0.79025E+00_JPRB, 0.80147E+00_JPRB, 0.84757E+00_JPRB, 0.91537E+00_JPRB, 0.10784E+01_JPRB, 0.20715E+01_JPRB, 0.23395E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.99221E+00_JPRB, 0.72753E+00_JPRB, 0.88263E+00_JPRB, 0.90248E+00_JPRB, 0.87788E+00_JPRB, 0.98930E+00_JPRB ,&
&  0.10894E+01_JPRB, 0.11069E+01_JPRB, 0.10272E+01_JPRB, 0.93011E+00_JPRB, 0.88859E+00_JPRB, 0.87142E+00_JPRB, 0.84220E+00_JPRB ,&
&  0.83510E+00_JPRB, 0.85928E+00_JPRB, 0.88383E+00_JPRB, 0.90493E+00_JPRB, 0.92809E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.10003E+01_JPRB, 0.10198E+01_JPRB, 0.10393E+01_JPRB, 0.10703E+01_JPRB, 0.11039E+01_JPRB, 0.11356E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.12031E+01_JPRB, 0.12249E+01_JPRB, 0.12427E+01_JPRB, 0.12550E+01_JPRB, 0.12631E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.12653E+01_JPRB, 0.12701E+01_JPRB, 0.12863E+01_JPRB, 0.13005E+01_JPRB, 0.13039E+01_JPRB, 0.13166E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.13539E+01_JPRB, 0.13677E+01_JPRB, 0.14024E+01_JPRB, 0.14533E+01_JPRB, 0.15231E+01_JPRB, 0.16751E+01_JPRB, 0.15458E+01_JPRB ,&
&  0.16856E+01_JPRB, 0.18643E+01_JPRB, 0.18291E+01_JPRB, 0.14543E+01_JPRB, 0.13182E+01_JPRB, 0.12678E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.12411E+01_JPRB, 0.12256E+01_JPRB, 0.12260E+01_JPRB, 0.12506E+01_JPRB, 0.12922E+01_JPRB, 0.13379E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.13846E+01_JPRB, 0.14158E+01_JPRB, 0.14524E+01_JPRB, 0.14860E+01_JPRB, 0.15475E+01_JPRB, 0.15633E+01_JPRB, 0.15298E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.14248E+01_JPRB, 0.12350E+01_JPRB, 0.11452E+01_JPRB, 0.11542E+01_JPRB, 0.10334E+01_JPRB, 0.93102E+00_JPRB ,&
&  0.99174E+00_JPRB, 0.96681E+00_JPRB, 0.94479E+00_JPRB, 0.11216E+01_JPRB, 0.11080E+01_JPRB, 0.99383E+00_JPRB, 0.94992E+00_JPRB ,&
&  0.83512E+00_JPRB, 0.73576E+00_JPRB, 0.67894E+00_JPRB, 0.67438E+00_JPRB, 0.69176E+00_JPRB, 0.73553E+00_JPRB, 0.74987E+00_JPRB ,&
&  0.78042E+00_JPRB, 0.83351E+00_JPRB, 0.89148E+00_JPRB, 0.98362E+00_JPRB, 0.12201E+01_JPRB, 0.12214E+01_JPRB, 0.14366E+01_JPRB ,&
&  0.10677E+01_JPRB, 0.89281E+00_JPRB, 0.95214E+00_JPRB, 0.99001E+00_JPRB, 0.10453E+01_JPRB, 0.10883E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.14208E+01_JPRB, 0.15700E+01_JPRB, 0.14337E+01_JPRB, 0.12315E+01_JPRB, 0.10266E+01_JPRB, 0.93764E+00_JPRB, 0.89713E+00_JPRB ,&
&  0.89254E+00_JPRB, 0.92148E+00_JPRB, 0.94483E+00_JPRB, 0.95940E+00_JPRB, 0.97924E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.10026E+01_JPRB, 0.10228E+01_JPRB, 0.10382E+01_JPRB, 0.10599E+01_JPRB, 0.10870E+01_JPRB, 0.11180E+01_JPRB, 0.11410E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.11853E+01_JPRB, 0.12037E+01_JPRB, 0.12254E+01_JPRB, 0.12413E+01_JPRB, 0.12638E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12953E+01_JPRB, 0.12940E+01_JPRB, 0.13076E+01_JPRB, 0.13239E+01_JPRB, 0.13403E+01_JPRB, 0.13583E+01_JPRB, 0.13844E+01_JPRB ,&
&  0.14004E+01_JPRB, 0.14004E+01_JPRB, 0.14118E+01_JPRB, 0.14309E+01_JPRB, 0.14365E+01_JPRB, 0.15359E+01_JPRB, 0.14275E+01_JPRB ,&
&  0.18726E+01_JPRB, 0.20598E+01_JPRB, 0.19446E+01_JPRB, 0.15737E+01_JPRB, 0.14343E+01_JPRB, 0.13786E+01_JPRB, 0.13539E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.13111E+01_JPRB, 0.12819E+01_JPRB, 0.12913E+01_JPRB, 0.13289E+01_JPRB, 0.13910E+01_JPRB, 0.14470E+01_JPRB ,&
&  0.14841E+01_JPRB, 0.15311E+01_JPRB, 0.15949E+01_JPRB, 0.16671E+01_JPRB, 0.17701E+01_JPRB, 0.18231E+01_JPRB, 0.18248E+01_JPRB ,&
&  0.18056E+01_JPRB, 0.17902E+01_JPRB, 0.16468E+01_JPRB, 0.16544E+01_JPRB, 0.16767E+01_JPRB, 0.13947E+01_JPRB, 0.11708E+01_JPRB ,&
&  0.10179E+01_JPRB, 0.96259E+00_JPRB, 0.99478E+00_JPRB, 0.10641E+01_JPRB, 0.99681E+00_JPRB, 0.92933E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.75297E+00_JPRB, 0.69079E+00_JPRB, 0.66335E+00_JPRB, 0.63406E+00_JPRB, 0.64019E+00_JPRB, 0.67222E+00_JPRB, 0.74638E+00_JPRB ,&
&  0.80314E+00_JPRB, 0.86604E+00_JPRB, 0.94975E+00_JPRB, 0.10312E+01_JPRB, 0.11624E+01_JPRB, 0.11613E+01_JPRB, 0.13178E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.11219E+01_JPRB, 0.11323E+01_JPRB, 0.89231E+00_JPRB, 0.11233E+01_JPRB, 0.12113E+01_JPRB, 0.13626E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.18776E+01_JPRB, 0.22372E+01_JPRB, 0.18838E+01_JPRB, 0.13891E+01_JPRB, 0.99367E+00_JPRB, 0.90134E+00_JPRB ,&
&  0.90292E+00_JPRB, 0.92880E+00_JPRB, 0.95436E+00_JPRB, 0.96904E+00_JPRB, 0.98589E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.95848E+00_JPRB, 0.97685E+00_JPRB, 0.99358E+00_JPRB, 0.10060E+01_JPRB, 0.10105E+01_JPRB, 0.10259E+01_JPRB, 0.10524E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.11011E+01_JPRB, 0.11248E+01_JPRB, 0.11458E+01_JPRB, 0.11676E+01_JPRB, 0.11849E+01_JPRB, 0.12167E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.13044E+01_JPRB, 0.13317E+01_JPRB, 0.13802E+01_JPRB, 0.14164E+01_JPRB, 0.14422E+01_JPRB, 0.15071E+01_JPRB ,&
&  0.15699E+01_JPRB, 0.16442E+01_JPRB, 0.16996E+01_JPRB, 0.17326E+01_JPRB, 0.17260E+01_JPRB, 0.18321E+01_JPRB, 0.16506E+01_JPRB ,&
&  0.20124E+01_JPRB, 0.22061E+01_JPRB, 0.20407E+01_JPRB, 0.16761E+01_JPRB, 0.15473E+01_JPRB, 0.14726E+01_JPRB, 0.14101E+01_JPRB ,&
&  0.13787E+01_JPRB, 0.13308E+01_JPRB, 0.12537E+01_JPRB, 0.12852E+01_JPRB, 0.13229E+01_JPRB, 0.13829E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.15411E+01_JPRB, 0.16159E+01_JPRB, 0.16953E+01_JPRB, 0.17742E+01_JPRB, 0.18845E+01_JPRB, 0.19758E+01_JPRB, 0.20345E+01_JPRB ,&
&  0.20763E+01_JPRB, 0.21117E+01_JPRB, 0.21455E+01_JPRB, 0.19954E+01_JPRB, 0.19738E+01_JPRB, 0.17826E+01_JPRB, 0.14213E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.94137E+00_JPRB, 0.98281E+00_JPRB, 0.10199E+01_JPRB, 0.97884E+00_JPRB, 0.86406E+00_JPRB, 0.82369E+00_JPRB ,&
&  0.71984E+00_JPRB, 0.65955E+00_JPRB, 0.65199E+00_JPRB, 0.64066E+00_JPRB, 0.63636E+00_JPRB, 0.68222E+00_JPRB, 0.72663E+00_JPRB ,&
&  0.79859E+00_JPRB, 0.89610E+00_JPRB, 0.98659E+00_JPRB, 0.10784E+01_JPRB, 0.12483E+01_JPRB, 0.14026E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.11556E+01_JPRB, 0.11570E+01_JPRB, 0.12691E+01_JPRB, 0.12104E+01_JPRB, 0.14152E+01_JPRB, 0.16368E+01_JPRB ,&
&  0.19531E+01_JPRB, 0.18753E+01_JPRB, 0.21628E+01_JPRB, 0.31836E+01_JPRB, 0.27137E+01_JPRB, 0.23027E+01_JPRB, 0.90175E+00_JPRB ,&
&  0.89900E+00_JPRB, 0.91888E+00_JPRB, 0.94886E+00_JPRB, 0.95526E+00_JPRB, 0.96133E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.85082E+00_JPRB, 0.88848E+00_JPRB, 0.89688E+00_JPRB, 0.92946E+00_JPRB, 0.93116E+00_JPRB, 0.92383E+00_JPRB, 0.94095E+00_JPRB ,&
&  0.96148E+00_JPRB, 0.97903E+00_JPRB, 0.10086E+01_JPRB, 0.10379E+01_JPRB, 0.10612E+01_JPRB, 0.10722E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.11772E+01_JPRB, 0.12345E+01_JPRB, 0.13108E+01_JPRB, 0.13689E+01_JPRB, 0.14199E+01_JPRB, 0.14954E+01_JPRB ,&
&  0.15603E+01_JPRB, 0.16718E+01_JPRB, 0.17889E+01_JPRB, 0.19017E+01_JPRB, 0.19740E+01_JPRB, 0.20011E+01_JPRB, 0.15828E+01_JPRB ,&
&  0.21170E+01_JPRB, 0.21323E+01_JPRB, 0.20230E+01_JPRB, 0.17432E+01_JPRB, 0.16103E+01_JPRB, 0.15189E+01_JPRB, 0.14490E+01_JPRB ,&
&  0.13982E+01_JPRB, 0.13298E+01_JPRB, 0.12538E+01_JPRB, 0.12046E+01_JPRB, 0.13006E+01_JPRB, 0.13240E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.14902E+01_JPRB, 0.15735E+01_JPRB, 0.16547E+01_JPRB, 0.17369E+01_JPRB, 0.18528E+01_JPRB, 0.19849E+01_JPRB, 0.21036E+01_JPRB ,&
&  0.22002E+01_JPRB, 0.22510E+01_JPRB, 0.23483E+01_JPRB, 0.20952E+01_JPRB, 0.20315E+01_JPRB, 0.19377E+01_JPRB, 0.13812E+01_JPRB ,&
&  0.99389E+00_JPRB, 0.95557E+00_JPRB, 0.10290E+01_JPRB, 0.10409E+01_JPRB, 0.96794E+00_JPRB, 0.85371E+00_JPRB, 0.77693E+00_JPRB ,&
&  0.69840E+00_JPRB, 0.68329E+00_JPRB, 0.67150E+00_JPRB, 0.66030E+00_JPRB, 0.65738E+00_JPRB, 0.69372E+00_JPRB, 0.72945E+00_JPRB ,&
&  0.80608E+00_JPRB, 0.86737E+00_JPRB, 0.94178E+00_JPRB, 0.10154E+01_JPRB, 0.11250E+01_JPRB, 0.12626E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.11841E+01_JPRB, 0.10580E+01_JPRB, 0.10532E+01_JPRB, 0.11006E+01_JPRB, 0.11293E+01_JPRB, 0.12154E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.14581E+01_JPRB, 0.15572E+01_JPRB, 0.14169E+01_JPRB, 0.11833E+01_JPRB, 0.15144E+01_JPRB, 0.15520E+01_JPRB, 0.75460E+00_JPRB ,&
&  0.74991E+00_JPRB, 0.78644E+00_JPRB, 0.81022E+00_JPRB, 0.83691E+00_JPRB, 0.86126E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.69907E+00_JPRB, 0.70811E+00_JPRB, 0.70990E+00_JPRB, 0.75215E+00_JPRB, 0.76383E+00_JPRB, 0.78563E+00_JPRB, 0.80339E+00_JPRB ,&
&  0.82269E+00_JPRB, 0.85263E+00_JPRB, 0.88703E+00_JPRB, 0.92180E+00_JPRB, 0.95055E+00_JPRB, 0.97244E+00_JPRB, 0.99078E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.10361E+01_JPRB, 0.10686E+01_JPRB, 0.11119E+01_JPRB, 0.11808E+01_JPRB, 0.12603E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.15449E+01_JPRB, 0.16779E+01_JPRB, 0.18314E+01_JPRB, 0.19877E+01_JPRB, 0.20977E+01_JPRB, 0.21663E+01_JPRB ,&
&  0.16778E+01_JPRB, 0.19920E+01_JPRB, 0.19418E+01_JPRB, 0.17724E+01_JPRB, 0.16444E+01_JPRB, 0.15768E+01_JPRB, 0.15092E+01_JPRB ,&
&  0.14517E+01_JPRB, 0.13491E+01_JPRB, 0.12942E+01_JPRB, 0.12451E+01_JPRB, 0.12566E+01_JPRB, 0.12646E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.14575E+01_JPRB, 0.15357E+01_JPRB, 0.16103E+01_JPRB, 0.17143E+01_JPRB, 0.18783E+01_JPRB, 0.20640E+01_JPRB ,&
&  0.22560E+01_JPRB, 0.24000E+01_JPRB, 0.25095E+01_JPRB, 0.22190E+01_JPRB, 0.20497E+01_JPRB, 0.19063E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.10233E+01_JPRB, 0.93866E+00_JPRB, 0.10107E+01_JPRB, 0.10608E+01_JPRB, 0.96436E+00_JPRB, 0.86482E+00_JPRB, 0.78101E+00_JPRB ,&
&  0.71698E+00_JPRB, 0.67847E+00_JPRB, 0.67741E+00_JPRB, 0.66720E+00_JPRB, 0.67279E+00_JPRB, 0.69873E+00_JPRB, 0.72492E+00_JPRB ,&
&  0.77667E+00_JPRB, 0.80180E+00_JPRB, 0.85656E+00_JPRB, 0.90158E+00_JPRB, 0.94899E+00_JPRB, 0.98698E+00_JPRB, 0.10184E+01_JPRB ,&
&  0.10346E+01_JPRB, 0.10424E+01_JPRB, 0.10419E+01_JPRB, 0.10648E+01_JPRB, 0.10682E+01_JPRB, 0.10804E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.11526E+01_JPRB, 0.11238E+01_JPRB, 0.10854E+01_JPRB, 0.10197E+01_JPRB, 0.96077E+00_JPRB, 0.74301E+00_JPRB ,&
&  0.70579E+00_JPRB, 0.66656E+00_JPRB, 0.68648E+00_JPRB, 0.71175E+00_JPRB, 0.70441E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.62225E+00_JPRB, 0.60995E+00_JPRB, 0.59613E+00_JPRB, 0.60271E+00_JPRB, 0.60979E+00_JPRB, 0.63508E+00_JPRB, 0.63793E+00_JPRB ,&
&  0.65510E+00_JPRB, 0.70707E+00_JPRB, 0.77164E+00_JPRB, 0.81074E+00_JPRB, 0.84268E+00_JPRB, 0.87311E+00_JPRB, 0.89912E+00_JPRB ,&
&  0.91459E+00_JPRB, 0.92715E+00_JPRB, 0.93977E+00_JPRB, 0.95616E+00_JPRB, 0.98566E+00_JPRB, 0.10328E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.11865E+01_JPRB, 0.12751E+01_JPRB, 0.13867E+01_JPRB, 0.15274E+01_JPRB, 0.16793E+01_JPRB, 0.17682E+01_JPRB, 0.18300E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.16636E+01_JPRB, 0.19754E+01_JPRB, 0.18546E+01_JPRB, 0.17042E+01_JPRB, 0.16237E+01_JPRB, 0.15450E+01_JPRB ,&
&  0.14262E+01_JPRB, 0.13072E+01_JPRB, 0.12553E+01_JPRB, 0.13106E+01_JPRB, 0.12195E+01_JPRB, 0.12136E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12787E+01_JPRB, 0.13191E+01_JPRB, 0.13357E+01_JPRB, 0.13672E+01_JPRB, 0.14252E+01_JPRB, 0.15639E+01_JPRB, 0.17959E+01_JPRB ,&
&  0.21081E+01_JPRB, 0.24262E+01_JPRB, 0.25935E+01_JPRB, 0.22166E+01_JPRB, 0.22628E+01_JPRB, 0.20897E+01_JPRB, 0.17537E+01_JPRB ,&
&  0.11916E+01_JPRB, 0.98280E+00_JPRB, 0.10165E+01_JPRB, 0.10457E+01_JPRB, 0.95861E+00_JPRB, 0.88162E+00_JPRB, 0.81351E+00_JPRB ,&
&  0.74292E+00_JPRB, 0.70417E+00_JPRB, 0.69245E+00_JPRB, 0.68708E+00_JPRB, 0.69505E+00_JPRB, 0.70756E+00_JPRB, 0.72866E+00_JPRB ,&
&  0.74374E+00_JPRB, 0.76615E+00_JPRB, 0.79916E+00_JPRB, 0.83512E+00_JPRB, 0.87752E+00_JPRB, 0.92003E+00_JPRB, 0.96464E+00_JPRB ,&
&  0.99387E+00_JPRB, 0.10192E+01_JPRB, 0.10429E+01_JPRB, 0.10614E+01_JPRB, 0.10709E+01_JPRB, 0.10780E+01_JPRB, 0.10617E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.10579E+01_JPRB, 0.10625E+01_JPRB, 0.10475E+01_JPRB, 0.99997E+00_JPRB, 0.95292E+00_JPRB, 0.91160E+00_JPRB ,&
&  0.84144E+00_JPRB, 0.74369E+00_JPRB, 0.65936E+00_JPRB, 0.63893E+00_JPRB, 0.62711E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.58830E+00_JPRB, 0.56204E+00_JPRB, 0.55379E+00_JPRB, 0.55546E+00_JPRB, 0.56769E+00_JPRB, 0.57559E+00_JPRB, 0.57620E+00_JPRB ,&
&  0.58021E+00_JPRB, 0.59438E+00_JPRB, 0.61059E+00_JPRB, 0.66238E+00_JPRB, 0.71960E+00_JPRB, 0.76442E+00_JPRB, 0.79329E+00_JPRB ,&
&  0.81589E+00_JPRB, 0.82326E+00_JPRB, 0.82097E+00_JPRB, 0.82342E+00_JPRB, 0.82720E+00_JPRB, 0.84220E+00_JPRB, 0.87822E+00_JPRB ,&
&  0.94186E+00_JPRB, 0.10144E+01_JPRB, 0.11069E+01_JPRB, 0.12321E+01_JPRB, 0.13983E+01_JPRB, 0.15684E+01_JPRB, 0.16943E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.69929E+00_JPRB, 0.17423E+01_JPRB, 0.20746E+01_JPRB, 0.18409E+01_JPRB, 0.17422E+01_JPRB, 0.15882E+01_JPRB ,&
&  0.13942E+01_JPRB, 0.13107E+01_JPRB, 0.12432E+01_JPRB, 0.12669E+01_JPRB, 0.11898E+01_JPRB, 0.11804E+01_JPRB, 0.11852E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.12415E+01_JPRB, 0.12427E+01_JPRB, 0.12243E+01_JPRB, 0.12191E+01_JPRB, 0.12640E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.17294E+01_JPRB, 0.21800E+01_JPRB, 0.25950E+01_JPRB, 0.23015E+01_JPRB, 0.23594E+01_JPRB, 0.19254E+01_JPRB, 0.14895E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.10471E+01_JPRB, 0.10281E+01_JPRB, 0.10438E+01_JPRB, 0.97349E+00_JPRB, 0.87950E+00_JPRB, 0.86497E+00_JPRB ,&
&  0.78646E+00_JPRB, 0.73358E+00_JPRB, 0.70554E+00_JPRB, 0.69773E+00_JPRB, 0.70312E+00_JPRB, 0.71675E+00_JPRB, 0.73208E+00_JPRB ,&
&  0.73868E+00_JPRB, 0.75293E+00_JPRB, 0.77192E+00_JPRB, 0.80569E+00_JPRB, 0.84684E+00_JPRB, 0.90142E+00_JPRB, 0.94614E+00_JPRB ,&
&  0.97214E+00_JPRB, 0.99436E+00_JPRB, 0.10168E+01_JPRB, 0.10315E+01_JPRB, 0.10449E+01_JPRB, 0.10229E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.10140E+01_JPRB, 0.10224E+01_JPRB, 0.10373E+01_JPRB, 0.10624E+01_JPRB, 0.10220E+01_JPRB, 0.98643E+00_JPRB, 0.99601E+00_JPRB ,&
&  0.97900E+00_JPRB, 0.90825E+00_JPRB, 0.78147E+00_JPRB, 0.66146E+00_JPRB, 0.61618E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.63995E+00_JPRB, 0.61004E+00_JPRB, 0.58098E+00_JPRB, 0.57243E+00_JPRB, 0.58177E+00_JPRB, 0.57684E+00_JPRB, 0.57376E+00_JPRB ,&
&  0.56938E+00_JPRB, 0.56562E+00_JPRB, 0.55147E+00_JPRB, 0.56013E+00_JPRB, 0.59229E+00_JPRB, 0.64236E+00_JPRB, 0.68344E+00_JPRB ,&
&  0.71536E+00_JPRB, 0.72719E+00_JPRB, 0.72759E+00_JPRB, 0.72669E+00_JPRB, 0.72569E+00_JPRB, 0.72640E+00_JPRB, 0.73227E+00_JPRB ,&
&  0.75592E+00_JPRB, 0.79638E+00_JPRB, 0.85071E+00_JPRB, 0.91632E+00_JPRB, 0.10098E+01_JPRB, 0.11723E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.18451E+01_JPRB, 0.13574E+01_JPRB, 0.88040E+00_JPRB, 0.18096E+01_JPRB, 0.20189E+01_JPRB, 0.19810E+01_JPRB, 0.16771E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.13852E+01_JPRB, 0.12636E+01_JPRB, 0.12789E+01_JPRB, 0.11701E+01_JPRB, 0.11482E+01_JPRB, 0.11625E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.11806E+01_JPRB, 0.11857E+01_JPRB, 0.11647E+01_JPRB, 0.11259E+01_JPRB, 0.11041E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.13432E+01_JPRB, 0.17365E+01_JPRB, 0.23617E+01_JPRB, 0.23498E+01_JPRB, 0.23979E+01_JPRB, 0.17875E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.12181E+01_JPRB, 0.11928E+01_JPRB, 0.11585E+01_JPRB, 0.10761E+01_JPRB, 0.97083E+00_JPRB, 0.90062E+00_JPRB, 0.91161E+00_JPRB ,&
&  0.84389E+00_JPRB, 0.79293E+00_JPRB, 0.75062E+00_JPRB, 0.72822E+00_JPRB, 0.72276E+00_JPRB, 0.72675E+00_JPRB, 0.73106E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.75370E+00_JPRB, 0.77346E+00_JPRB, 0.79772E+00_JPRB, 0.82526E+00_JPRB, 0.86152E+00_JPRB, 0.90887E+00_JPRB ,&
&  0.94600E+00_JPRB, 0.97079E+00_JPRB, 0.98588E+00_JPRB, 0.99133E+00_JPRB, 0.97352E+00_JPRB, 0.95776E+00_JPRB, 0.97941E+00_JPRB ,&
&  0.97518E+00_JPRB, 0.98129E+00_JPRB, 0.96409E+00_JPRB, 0.99008E+00_JPRB, 0.99902E+00_JPRB, 0.10007E+01_JPRB, 0.10210E+01_JPRB ,&
&  0.10513E+01_JPRB, 0.10309E+01_JPRB, 0.92409E+00_JPRB, 0.72316E+00_JPRB, 0.66884E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.70781E+00_JPRB, 0.68647E+00_JPRB, 0.67057E+00_JPRB, 0.66740E+00_JPRB, 0.66463E+00_JPRB, 0.64830E+00_JPRB, 0.62699E+00_JPRB ,&
&  0.59999E+00_JPRB, 0.57968E+00_JPRB, 0.55158E+00_JPRB, 0.52501E+00_JPRB, 0.52036E+00_JPRB, 0.53566E+00_JPRB, 0.56714E+00_JPRB ,&
&  0.59742E+00_JPRB, 0.61267E+00_JPRB, 0.62378E+00_JPRB, 0.63382E+00_JPRB, 0.64289E+00_JPRB, 0.64698E+00_JPRB, 0.64632E+00_JPRB ,&
&  0.65170E+00_JPRB, 0.66602E+00_JPRB, 0.68848E+00_JPRB, 0.71572E+00_JPRB, 0.75559E+00_JPRB, 0.82639E+00_JPRB, 0.96430E+00_JPRB ,&
&  0.12567E+01_JPRB, 0.14196E+01_JPRB, 0.74463E+00_JPRB, 0.14381E+01_JPRB, 0.20262E+01_JPRB, 0.20200E+01_JPRB, 0.19044E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.14892E+01_JPRB, 0.14045E+01_JPRB, 0.12974E+01_JPRB, 0.11735E+01_JPRB, 0.11263E+01_JPRB, 0.11123E+01_JPRB ,&
&  0.11082E+01_JPRB, 0.10802E+01_JPRB, 0.10605E+01_JPRB, 0.10555E+01_JPRB, 0.10285E+01_JPRB, 0.99489E+00_JPRB, 0.99579E+00_JPRB ,&
&  0.10843E+01_JPRB, 0.13585E+01_JPRB, 0.19525E+01_JPRB, 0.22643E+01_JPRB, 0.23645E+01_JPRB, 0.19219E+01_JPRB, 0.15252E+01_JPRB ,&
&  0.13721E+01_JPRB, 0.14103E+01_JPRB, 0.13337E+01_JPRB, 0.11612E+01_JPRB, 0.10081E+01_JPRB, 0.95359E+00_JPRB, 0.94866E+00_JPRB ,&
&  0.89213E+00_JPRB, 0.84327E+00_JPRB, 0.81129E+00_JPRB, 0.78679E+00_JPRB, 0.77075E+00_JPRB, 0.75973E+00_JPRB, 0.75526E+00_JPRB ,&
&  0.76519E+00_JPRB, 0.77164E+00_JPRB, 0.77917E+00_JPRB, 0.78981E+00_JPRB, 0.80120E+00_JPRB, 0.81972E+00_JPRB, 0.85339E+00_JPRB ,&
&  0.89671E+00_JPRB, 0.92818E+00_JPRB, 0.95343E+00_JPRB, 0.94404E+00_JPRB, 0.93575E+00_JPRB, 0.93927E+00_JPRB, 0.96331E+00_JPRB ,&
&  0.98045E+00_JPRB, 0.10204E+01_JPRB, 0.94054E+00_JPRB, 0.93840E+00_JPRB, 0.97523E+00_JPRB, 0.97396E+00_JPRB, 0.99839E+00_JPRB ,&
&  0.10403E+01_JPRB, 0.10983E+01_JPRB, 0.10835E+01_JPRB, 0.79982E+00_JPRB, 0.74660E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.75777E+00_JPRB, 0.73179E+00_JPRB, 0.72514E+00_JPRB, 0.72437E+00_JPRB, 0.72121E+00_JPRB, 0.71091E+00_JPRB, 0.69091E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.64704E+00_JPRB, 0.62544E+00_JPRB, 0.59740E+00_JPRB, 0.55723E+00_JPRB, 0.52171E+00_JPRB, 0.51866E+00_JPRB ,&
&  0.53905E+00_JPRB, 0.54773E+00_JPRB, 0.55535E+00_JPRB, 0.56491E+00_JPRB, 0.57577E+00_JPRB, 0.58832E+00_JPRB, 0.59665E+00_JPRB ,&
&  0.59997E+00_JPRB, 0.60608E+00_JPRB, 0.61670E+00_JPRB, 0.62841E+00_JPRB, 0.64632E+00_JPRB, 0.68206E+00_JPRB, 0.74887E+00_JPRB ,&
&  0.93451E+00_JPRB, 0.12053E+01_JPRB, 0.12734E+01_JPRB, 0.20759E+01_JPRB, 0.23791E+01_JPRB, 0.22444E+01_JPRB, 0.22266E+01_JPRB ,&
&  0.21600E+01_JPRB, 0.19182E+01_JPRB, 0.16815E+01_JPRB, 0.13735E+01_JPRB, 0.12073E+01_JPRB, 0.11285E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.10109E+01_JPRB, 0.95657E+00_JPRB, 0.92843E+00_JPRB, 0.90695E+00_JPRB, 0.88853E+00_JPRB, 0.88978E+00_JPRB ,&
&  0.92999E+00_JPRB, 0.10942E+01_JPRB, 0.15260E+01_JPRB, 0.21698E+01_JPRB, 0.23471E+01_JPRB, 0.21424E+01_JPRB, 0.18245E+01_JPRB ,&
&  0.18308E+01_JPRB, 0.17634E+01_JPRB, 0.16162E+01_JPRB, 0.13435E+01_JPRB, 0.11221E+01_JPRB, 0.11053E+01_JPRB, 0.99004E+00_JPRB ,&
&  0.92023E+00_JPRB, 0.87476E+00_JPRB, 0.84991E+00_JPRB, 0.83236E+00_JPRB, 0.81999E+00_JPRB, 0.80502E+00_JPRB, 0.78694E+00_JPRB ,&
&  0.77569E+00_JPRB, 0.77447E+00_JPRB, 0.76889E+00_JPRB, 0.76852E+00_JPRB, 0.76957E+00_JPRB, 0.77592E+00_JPRB, 0.79220E+00_JPRB ,&
&  0.83025E+00_JPRB, 0.86761E+00_JPRB, 0.86605E+00_JPRB, 0.87269E+00_JPRB, 0.87684E+00_JPRB, 0.87915E+00_JPRB, 0.88264E+00_JPRB ,&
&  0.89611E+00_JPRB, 0.93572E+00_JPRB, 0.94678E+00_JPRB, 0.94949E+00_JPRB, 0.95835E+00_JPRB, 0.96235E+00_JPRB, 0.94981E+00_JPRB ,&
&  0.95100E+00_JPRB, 0.93092E+00_JPRB, 0.89894E+00_JPRB, 0.82722E+00_JPRB, 0.79402E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.77731E+00_JPRB, 0.75214E+00_JPRB, 0.73621E+00_JPRB, 0.72843E+00_JPRB, 0.72290E+00_JPRB, 0.71651E+00_JPRB, 0.70023E+00_JPRB ,&
&  0.68269E+00_JPRB, 0.67167E+00_JPRB, 0.65455E+00_JPRB, 0.63675E+00_JPRB, 0.60228E+00_JPRB, 0.55930E+00_JPRB, 0.54027E+00_JPRB ,&
&  0.52985E+00_JPRB, 0.52914E+00_JPRB, 0.53880E+00_JPRB, 0.54411E+00_JPRB, 0.54830E+00_JPRB, 0.55688E+00_JPRB, 0.56432E+00_JPRB ,&
&  0.57152E+00_JPRB, 0.57419E+00_JPRB, 0.58112E+00_JPRB, 0.58814E+00_JPRB, 0.59518E+00_JPRB, 0.61917E+00_JPRB, 0.66062E+00_JPRB ,&
&  0.78306E+00_JPRB, 0.11291E+01_JPRB, 0.10005E+01_JPRB, 0.20202E+01_JPRB, 0.22866E+01_JPRB, 0.22842E+01_JPRB, 0.21334E+01_JPRB ,&
&  0.21287E+01_JPRB, 0.21208E+01_JPRB, 0.16845E+01_JPRB, 0.14057E+01_JPRB, 0.12171E+01_JPRB, 0.11202E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.97260E+00_JPRB, 0.91007E+00_JPRB, 0.85585E+00_JPRB, 0.82598E+00_JPRB, 0.80750E+00_JPRB, 0.80686E+00_JPRB ,&
&  0.82313E+00_JPRB, 0.91382E+00_JPRB, 0.12023E+01_JPRB, 0.17848E+01_JPRB, 0.22766E+01_JPRB, 0.23373E+01_JPRB, 0.21215E+01_JPRB ,&
&  0.21346E+01_JPRB, 0.21994E+01_JPRB, 0.20168E+01_JPRB, 0.16275E+01_JPRB, 0.13496E+01_JPRB, 0.12124E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.97950E+00_JPRB, 0.92235E+00_JPRB, 0.89100E+00_JPRB, 0.86502E+00_JPRB, 0.84261E+00_JPRB, 0.82189E+00_JPRB, 0.79745E+00_JPRB ,&
&  0.77547E+00_JPRB, 0.75621E+00_JPRB, 0.74866E+00_JPRB, 0.74315E+00_JPRB, 0.74599E+00_JPRB, 0.74955E+00_JPRB, 0.75521E+00_JPRB ,&
&  0.77648E+00_JPRB, 0.81382E+00_JPRB, 0.80512E+00_JPRB, 0.81488E+00_JPRB, 0.82249E+00_JPRB, 0.82646E+00_JPRB, 0.84610E+00_JPRB ,&
&  0.88729E+00_JPRB, 0.90004E+00_JPRB, 0.91761E+00_JPRB, 0.91682E+00_JPRB, 0.94756E+00_JPRB, 0.98430E+00_JPRB, 0.94121E+00_JPRB ,&
&  0.92698E+00_JPRB, 0.89559E+00_JPRB, 0.86310E+00_JPRB, 0.82779E+00_JPRB, 0.80219E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.77314E+00_JPRB, 0.75370E+00_JPRB, 0.73154E+00_JPRB, 0.71096E+00_JPRB, 0.70394E+00_JPRB, 0.69973E+00_JPRB, 0.68583E+00_JPRB ,&
&  0.67280E+00_JPRB, 0.66065E+00_JPRB, 0.64405E+00_JPRB, 0.62371E+00_JPRB, 0.59917E+00_JPRB, 0.56906E+00_JPRB, 0.54156E+00_JPRB ,&
&  0.52872E+00_JPRB, 0.51826E+00_JPRB, 0.52055E+00_JPRB, 0.52711E+00_JPRB, 0.53652E+00_JPRB, 0.54320E+00_JPRB, 0.55051E+00_JPRB ,&
&  0.55680E+00_JPRB, 0.56044E+00_JPRB, 0.56504E+00_JPRB, 0.56910E+00_JPRB, 0.57131E+00_JPRB, 0.58376E+00_JPRB, 0.62931E+00_JPRB ,&
&  0.72747E+00_JPRB, 0.12533E+01_JPRB, 0.10032E+01_JPRB, 0.16737E+01_JPRB, 0.19153E+01_JPRB, 0.20323E+01_JPRB, 0.19166E+01_JPRB ,&
&  0.19234E+01_JPRB, 0.18246E+01_JPRB, 0.15558E+01_JPRB, 0.13532E+01_JPRB, 0.12035E+01_JPRB, 0.11068E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.99089E+00_JPRB, 0.93538E+00_JPRB, 0.87268E+00_JPRB, 0.81261E+00_JPRB, 0.77588E+00_JPRB, 0.75588E+00_JPRB, 0.74924E+00_JPRB ,&
&  0.75252E+00_JPRB, 0.79944E+00_JPRB, 0.96845E+00_JPRB, 0.14690E+01_JPRB, 0.19927E+01_JPRB, 0.23199E+01_JPRB, 0.23809E+01_JPRB ,&
&  0.24313E+01_JPRB, 0.26041E+01_JPRB, 0.23985E+01_JPRB, 0.19605E+01_JPRB, 0.16196E+01_JPRB, 0.13683E+01_JPRB, 0.11808E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.99776E+00_JPRB, 0.95345E+00_JPRB, 0.91533E+00_JPRB, 0.87452E+00_JPRB, 0.82997E+00_JPRB, 0.78663E+00_JPRB ,&
&  0.75440E+00_JPRB, 0.73235E+00_JPRB, 0.71930E+00_JPRB, 0.72024E+00_JPRB, 0.72250E+00_JPRB, 0.72644E+00_JPRB, 0.73509E+00_JPRB ,&
&  0.74945E+00_JPRB, 0.77705E+00_JPRB, 0.77715E+00_JPRB, 0.78666E+00_JPRB, 0.79603E+00_JPRB, 0.81128E+00_JPRB, 0.82770E+00_JPRB ,&
&  0.87989E+00_JPRB, 0.88986E+00_JPRB, 0.88509E+00_JPRB, 0.90811E+00_JPRB, 0.96805E+00_JPRB, 0.10489E+01_JPRB, 0.96884E+00_JPRB ,&
&  0.93610E+00_JPRB, 0.88753E+00_JPRB, 0.84513E+00_JPRB, 0.81793E+00_JPRB, 0.79697E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.76547E+00_JPRB, 0.74138E+00_JPRB, 0.72549E+00_JPRB, 0.70873E+00_JPRB, 0.69804E+00_JPRB, 0.68973E+00_JPRB, 0.67492E+00_JPRB ,&
&  0.65737E+00_JPRB, 0.63861E+00_JPRB, 0.61903E+00_JPRB, 0.60124E+00_JPRB, 0.58689E+00_JPRB, 0.56958E+00_JPRB, 0.55548E+00_JPRB ,&
&  0.54275E+00_JPRB, 0.52781E+00_JPRB, 0.51827E+00_JPRB, 0.51507E+00_JPRB, 0.51880E+00_JPRB, 0.52415E+00_JPRB, 0.53146E+00_JPRB ,&
&  0.54013E+00_JPRB, 0.54589E+00_JPRB, 0.54702E+00_JPRB, 0.54955E+00_JPRB, 0.54844E+00_JPRB, 0.55901E+00_JPRB, 0.60733E+00_JPRB ,&
&  0.72289E+00_JPRB, 0.68780E+00_JPRB, 0.11247E+01_JPRB, 0.13444E+01_JPRB, 0.15136E+01_JPRB, 0.16781E+01_JPRB, 0.17733E+01_JPRB ,&
&  0.18190E+01_JPRB, 0.16356E+01_JPRB, 0.14091E+01_JPRB, 0.12591E+01_JPRB, 0.11399E+01_JPRB, 0.10523E+01_JPRB, 0.98400E+00_JPRB ,&
&  0.91703E+00_JPRB, 0.86648E+00_JPRB, 0.82594E+00_JPRB, 0.77409E+00_JPRB, 0.73726E+00_JPRB, 0.71951E+00_JPRB, 0.71197E+00_JPRB ,&
&  0.70583E+00_JPRB, 0.72269E+00_JPRB, 0.81027E+00_JPRB, 0.10892E+01_JPRB, 0.17151E+01_JPRB, 0.22022E+01_JPRB, 0.25542E+01_JPRB ,&
&  0.30224E+01_JPRB, 0.31028E+01_JPRB, 0.27779E+01_JPRB, 0.22885E+01_JPRB, 0.18225E+01_JPRB, 0.15079E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.11127E+01_JPRB, 0.10330E+01_JPRB, 0.98618E+00_JPRB, 0.94717E+00_JPRB, 0.90483E+00_JPRB, 0.84800E+00_JPRB, 0.78484E+00_JPRB ,&
&  0.73596E+00_JPRB, 0.70832E+00_JPRB, 0.69137E+00_JPRB, 0.68379E+00_JPRB, 0.68193E+00_JPRB, 0.68511E+00_JPRB, 0.69486E+00_JPRB ,&
&  0.71328E+00_JPRB, 0.74439E+00_JPRB, 0.76083E+00_JPRB, 0.76205E+00_JPRB, 0.77947E+00_JPRB, 0.80702E+00_JPRB, 0.83900E+00_JPRB ,&
&  0.86283E+00_JPRB, 0.85320E+00_JPRB, 0.83366E+00_JPRB, 0.87280E+00_JPRB, 0.95063E+00_JPRB, 0.10456E+01_JPRB, 0.99818E+00_JPRB ,&
&  0.95959E+00_JPRB, 0.90390E+00_JPRB, 0.84283E+00_JPRB, 0.81217E+00_JPRB, 0.79582E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.75410E+00_JPRB, 0.72973E+00_JPRB, 0.72234E+00_JPRB, 0.71960E+00_JPRB, 0.71594E+00_JPRB, 0.70115E+00_JPRB, 0.67233E+00_JPRB ,&
&  0.64786E+00_JPRB, 0.62711E+00_JPRB, 0.60558E+00_JPRB, 0.58736E+00_JPRB, 0.57521E+00_JPRB, 0.56626E+00_JPRB, 0.55585E+00_JPRB ,&
&  0.54834E+00_JPRB, 0.54190E+00_JPRB, 0.52829E+00_JPRB, 0.51562E+00_JPRB, 0.51011E+00_JPRB, 0.50920E+00_JPRB, 0.51316E+00_JPRB ,&
&  0.51776E+00_JPRB, 0.52321E+00_JPRB, 0.52702E+00_JPRB, 0.53001E+00_JPRB, 0.53028E+00_JPRB, 0.54089E+00_JPRB, 0.58260E+00_JPRB ,&
&  0.71498E+00_JPRB, 0.97472E+00_JPRB, 0.11329E+01_JPRB, 0.11425E+01_JPRB, 0.12394E+01_JPRB, 0.14172E+01_JPRB, 0.16118E+01_JPRB ,&
&  0.16021E+01_JPRB, 0.14620E+01_JPRB, 0.12659E+01_JPRB, 0.11312E+01_JPRB, 0.10294E+01_JPRB, 0.95745E+00_JPRB, 0.88632E+00_JPRB ,&
&  0.82800E+00_JPRB, 0.78858E+00_JPRB, 0.74957E+00_JPRB, 0.71478E+00_JPRB, 0.70019E+00_JPRB, 0.69598E+00_JPRB, 0.69318E+00_JPRB ,&
&  0.68911E+00_JPRB, 0.68723E+00_JPRB, 0.72480E+00_JPRB, 0.91826E+00_JPRB, 0.14751E+01_JPRB, 0.18484E+01_JPRB, 0.25447E+01_JPRB ,&
&  0.31430E+01_JPRB, 0.30440E+01_JPRB, 0.27330E+01_JPRB, 0.23170E+01_JPRB, 0.18736E+01_JPRB, 0.15043E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.10171E+01_JPRB, 0.94557E+00_JPRB, 0.89960E+00_JPRB, 0.86231E+00_JPRB, 0.82765E+00_JPRB, 0.78582E+00_JPRB ,&
&  0.74167E+00_JPRB, 0.70572E+00_JPRB, 0.68137E+00_JPRB, 0.66380E+00_JPRB, 0.65410E+00_JPRB, 0.65174E+00_JPRB, 0.65477E+00_JPRB ,&
&  0.66267E+00_JPRB, 0.68616E+00_JPRB, 0.73369E+00_JPRB, 0.72704E+00_JPRB, 0.74496E+00_JPRB, 0.77373E+00_JPRB, 0.79988E+00_JPRB ,&
&  0.79377E+00_JPRB, 0.80717E+00_JPRB, 0.78974E+00_JPRB, 0.82247E+00_JPRB, 0.10270E+01_JPRB, 0.11272E+01_JPRB, 0.10420E+01_JPRB ,&
&  0.97895E+00_JPRB, 0.91720E+00_JPRB, 0.86056E+00_JPRB, 0.81045E+00_JPRB, 0.79240E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.75678E+00_JPRB, 0.72132E+00_JPRB, 0.70341E+00_JPRB, 0.70498E+00_JPRB, 0.71154E+00_JPRB, 0.70137E+00_JPRB, 0.67383E+00_JPRB ,&
&  0.63995E+00_JPRB, 0.61165E+00_JPRB, 0.59169E+00_JPRB, 0.57717E+00_JPRB, 0.56605E+00_JPRB, 0.55637E+00_JPRB, 0.54561E+00_JPRB ,&
&  0.53569E+00_JPRB, 0.53023E+00_JPRB, 0.52630E+00_JPRB, 0.52072E+00_JPRB, 0.50978E+00_JPRB, 0.50120E+00_JPRB, 0.49836E+00_JPRB ,&
&  0.49674E+00_JPRB, 0.49731E+00_JPRB, 0.50278E+00_JPRB, 0.50922E+00_JPRB, 0.51220E+00_JPRB, 0.51853E+00_JPRB, 0.54599E+00_JPRB ,&
&  0.64307E+00_JPRB, 0.86744E+00_JPRB, 0.86756E+00_JPRB, 0.94164E+00_JPRB, 0.10847E+01_JPRB, 0.12404E+01_JPRB, 0.13767E+01_JPRB ,&
&  0.13805E+01_JPRB, 0.13074E+01_JPRB, 0.11700E+01_JPRB, 0.10325E+01_JPRB, 0.92213E+00_JPRB, 0.85659E+00_JPRB, 0.81008E+00_JPRB ,&
&  0.76816E+00_JPRB, 0.72657E+00_JPRB, 0.69048E+00_JPRB, 0.66352E+00_JPRB, 0.64528E+00_JPRB, 0.63788E+00_JPRB, 0.63819E+00_JPRB ,&
&  0.63949E+00_JPRB, 0.63907E+00_JPRB, 0.64623E+00_JPRB, 0.71577E+00_JPRB, 0.10329E+01_JPRB, 0.14393E+01_JPRB, 0.18919E+01_JPRB ,&
&  0.20110E+01_JPRB, 0.20595E+01_JPRB, 0.20201E+01_JPRB, 0.18975E+01_JPRB, 0.16966E+01_JPRB, 0.15000E+01_JPRB, 0.12895E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.10281E+01_JPRB, 0.95009E+00_JPRB, 0.88486E+00_JPRB, 0.82640E+00_JPRB, 0.77982E+00_JPRB, 0.74769E+00_JPRB ,&
&  0.72383E+00_JPRB, 0.70277E+00_JPRB, 0.68288E+00_JPRB, 0.66433E+00_JPRB, 0.65027E+00_JPRB, 0.64209E+00_JPRB, 0.64262E+00_JPRB ,&
&  0.64273E+00_JPRB, 0.64020E+00_JPRB, 0.65449E+00_JPRB, 0.66909E+00_JPRB, 0.68557E+00_JPRB, 0.69608E+00_JPRB, 0.70405E+00_JPRB ,&
&  0.71653E+00_JPRB, 0.76940E+00_JPRB, 0.76593E+00_JPRB, 0.81784E+00_JPRB, 0.98324E+00_JPRB, 0.10650E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.10121E+01_JPRB, 0.94554E+00_JPRB, 0.88389E+00_JPRB, 0.84029E+00_JPRB, 0.81038E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.89125E+00_JPRB, 0.78596E+00_JPRB, 0.71991E+00_JPRB, 0.69046E+00_JPRB, 0.68275E+00_JPRB, 0.66496E+00_JPRB, 0.64240E+00_JPRB ,&
&  0.61998E+00_JPRB, 0.60301E+00_JPRB, 0.59121E+00_JPRB, 0.57756E+00_JPRB, 0.56291E+00_JPRB, 0.55015E+00_JPRB, 0.53707E+00_JPRB ,&
&  0.52375E+00_JPRB, 0.51571E+00_JPRB, 0.50960E+00_JPRB, 0.50510E+00_JPRB, 0.50154E+00_JPRB, 0.49490E+00_JPRB, 0.48781E+00_JPRB ,&
&  0.48249E+00_JPRB, 0.47571E+00_JPRB, 0.47422E+00_JPRB, 0.48102E+00_JPRB, 0.48774E+00_JPRB, 0.49409E+00_JPRB, 0.51640E+00_JPRB ,&
&  0.59409E+00_JPRB, 0.67592E+00_JPRB, 0.75152E+00_JPRB, 0.81877E+00_JPRB, 0.88828E+00_JPRB, 0.97617E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.98241E+00_JPRB, 0.98980E+00_JPRB, 0.96828E+00_JPRB, 0.89621E+00_JPRB, 0.81607E+00_JPRB, 0.76247E+00_JPRB, 0.72482E+00_JPRB ,&
&  0.69347E+00_JPRB, 0.66017E+00_JPRB, 0.62647E+00_JPRB, 0.59983E+00_JPRB, 0.58420E+00_JPRB, 0.57866E+00_JPRB, 0.57860E+00_JPRB ,&
&  0.58090E+00_JPRB, 0.58609E+00_JPRB, 0.59095E+00_JPRB, 0.60345E+00_JPRB, 0.67710E+00_JPRB, 0.86378E+00_JPRB, 0.11564E+01_JPRB ,&
&  0.12885E+01_JPRB, 0.13220E+01_JPRB, 0.13453E+01_JPRB, 0.13418E+01_JPRB, 0.12652E+01_JPRB, 0.11824E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.10085E+01_JPRB, 0.91690E+00_JPRB, 0.83951E+00_JPRB, 0.77883E+00_JPRB, 0.73139E+00_JPRB, 0.69972E+00_JPRB, 0.68347E+00_JPRB ,&
&  0.67643E+00_JPRB, 0.67233E+00_JPRB, 0.66850E+00_JPRB, 0.66033E+00_JPRB, 0.64906E+00_JPRB, 0.64149E+00_JPRB, 0.63663E+00_JPRB ,&
&  0.63165E+00_JPRB, 0.62836E+00_JPRB, 0.62959E+00_JPRB, 0.63234E+00_JPRB, 0.63890E+00_JPRB, 0.64347E+00_JPRB, 0.65101E+00_JPRB ,&
&  0.66137E+00_JPRB, 0.68790E+00_JPRB, 0.70002E+00_JPRB, 0.80441E+00_JPRB, 0.90790E+00_JPRB, 0.93647E+00_JPRB, 0.97457E+00_JPRB ,&
&  0.10108E+01_JPRB, 0.98797E+00_JPRB, 0.92157E+00_JPRB, 0.88836E+00_JPRB, 0.97337E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.82822E+00_JPRB, 0.77846E+00_JPRB, 0.71647E+00_JPRB, 0.67569E+00_JPRB, 0.64777E+00_JPRB, 0.61708E+00_JPRB, 0.59400E+00_JPRB ,&
&  0.57904E+00_JPRB, 0.56932E+00_JPRB, 0.55950E+00_JPRB, 0.54732E+00_JPRB, 0.53517E+00_JPRB, 0.52600E+00_JPRB, 0.51695E+00_JPRB ,&
&  0.50782E+00_JPRB, 0.49906E+00_JPRB, 0.49095E+00_JPRB, 0.48476E+00_JPRB, 0.47735E+00_JPRB, 0.47118E+00_JPRB, 0.46558E+00_JPRB ,&
&  0.45918E+00_JPRB, 0.45310E+00_JPRB, 0.44606E+00_JPRB, 0.45003E+00_JPRB, 0.45963E+00_JPRB, 0.46970E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.53202E+00_JPRB, 0.56179E+00_JPRB, 0.62729E+00_JPRB, 0.69935E+00_JPRB, 0.75183E+00_JPRB, 0.79324E+00_JPRB, 0.79907E+00_JPRB ,&
&  0.77331E+00_JPRB, 0.78379E+00_JPRB, 0.77077E+00_JPRB, 0.73921E+00_JPRB, 0.71501E+00_JPRB, 0.66676E+00_JPRB, 0.62364E+00_JPRB ,&
&  0.59357E+00_JPRB, 0.57437E+00_JPRB, 0.55796E+00_JPRB, 0.54077E+00_JPRB, 0.53061E+00_JPRB, 0.52879E+00_JPRB, 0.53258E+00_JPRB ,&
&  0.53653E+00_JPRB, 0.54277E+00_JPRB, 0.54930E+00_JPRB, 0.55711E+00_JPRB, 0.57040E+00_JPRB, 0.62785E+00_JPRB, 0.76887E+00_JPRB ,&
&  0.87202E+00_JPRB, 0.93004E+00_JPRB, 0.96372E+00_JPRB, 0.97504E+00_JPRB, 0.94786E+00_JPRB, 0.90293E+00_JPRB, 0.85077E+00_JPRB ,&
&  0.79672E+00_JPRB, 0.73932E+00_JPRB, 0.68658E+00_JPRB, 0.64532E+00_JPRB, 0.61297E+00_JPRB, 0.59122E+00_JPRB, 0.57971E+00_JPRB ,&
&  0.57812E+00_JPRB, 0.58395E+00_JPRB, 0.59347E+00_JPRB, 0.60287E+00_JPRB, 0.60843E+00_JPRB, 0.60876E+00_JPRB, 0.60638E+00_JPRB ,&
&  0.60237E+00_JPRB, 0.59739E+00_JPRB, 0.59399E+00_JPRB, 0.59320E+00_JPRB, 0.59462E+00_JPRB, 0.59757E+00_JPRB, 0.60216E+00_JPRB ,&
&  0.60801E+00_JPRB, 0.62046E+00_JPRB, 0.63753E+00_JPRB, 0.71330E+00_JPRB, 0.74646E+00_JPRB, 0.78128E+00_JPRB, 0.81354E+00_JPRB ,&
&  0.85094E+00_JPRB, 0.86614E+00_JPRB, 0.84822E+00_JPRB, 0.81023E+00_JPRB, 0.80424E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.72451E+00_JPRB, 0.70698E+00_JPRB, 0.67500E+00_JPRB, 0.64061E+00_JPRB, 0.61133E+00_JPRB, 0.58507E+00_JPRB, 0.56445E+00_JPRB ,&
&  0.55067E+00_JPRB, 0.53953E+00_JPRB, 0.52805E+00_JPRB, 0.51719E+00_JPRB, 0.50539E+00_JPRB, 0.49474E+00_JPRB, 0.48399E+00_JPRB ,&
&  0.47638E+00_JPRB, 0.47119E+00_JPRB, 0.46648E+00_JPRB, 0.46098E+00_JPRB, 0.45520E+00_JPRB, 0.44924E+00_JPRB, 0.44207E+00_JPRB ,&
&  0.43584E+00_JPRB, 0.43241E+00_JPRB, 0.42637E+00_JPRB, 0.42480E+00_JPRB, 0.43197E+00_JPRB, 0.44194E+00_JPRB, 0.45263E+00_JPRB ,&
&  0.48838E+00_JPRB, 0.46804E+00_JPRB, 0.48233E+00_JPRB, 0.55461E+00_JPRB, 0.60942E+00_JPRB, 0.67233E+00_JPRB, 0.70398E+00_JPRB ,&
&  0.71260E+00_JPRB, 0.72158E+00_JPRB, 0.70438E+00_JPRB, 0.67864E+00_JPRB, 0.63743E+00_JPRB, 0.60212E+00_JPRB, 0.57552E+00_JPRB ,&
&  0.55233E+00_JPRB, 0.53175E+00_JPRB, 0.51553E+00_JPRB, 0.50270E+00_JPRB, 0.49237E+00_JPRB, 0.48833E+00_JPRB, 0.48995E+00_JPRB ,&
&  0.49271E+00_JPRB, 0.49608E+00_JPRB, 0.50114E+00_JPRB, 0.50979E+00_JPRB, 0.51986E+00_JPRB, 0.53518E+00_JPRB, 0.58439E+00_JPRB ,&
&  0.66117E+00_JPRB, 0.71450E+00_JPRB, 0.75777E+00_JPRB, 0.77847E+00_JPRB, 0.77036E+00_JPRB, 0.73975E+00_JPRB, 0.68970E+00_JPRB ,&
&  0.63808E+00_JPRB, 0.59808E+00_JPRB, 0.57276E+00_JPRB, 0.55669E+00_JPRB, 0.54723E+00_JPRB, 0.53953E+00_JPRB, 0.53290E+00_JPRB ,&
&  0.52670E+00_JPRB, 0.52253E+00_JPRB, 0.52265E+00_JPRB, 0.52723E+00_JPRB, 0.53436E+00_JPRB, 0.54058E+00_JPRB, 0.54347E+00_JPRB ,&
&  0.54377E+00_JPRB, 0.54296E+00_JPRB, 0.54410E+00_JPRB, 0.54603E+00_JPRB, 0.54805E+00_JPRB, 0.54844E+00_JPRB, 0.55137E+00_JPRB ,&
&  0.55692E+00_JPRB, 0.56709E+00_JPRB, 0.58006E+00_JPRB, 0.66694E+00_JPRB, 0.70282E+00_JPRB, 0.71328E+00_JPRB, 0.71910E+00_JPRB ,&
&  0.73219E+00_JPRB, 0.73351E+00_JPRB, 0.73636E+00_JPRB, 0.64302E+00_JPRB, 0.71383E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.62014E+00_JPRB, 0.61429E+00_JPRB, 0.59530E+00_JPRB, 0.57513E+00_JPRB, 0.55472E+00_JPRB, 0.53512E+00_JPRB, 0.52099E+00_JPRB ,&
&  0.51073E+00_JPRB, 0.49786E+00_JPRB, 0.48675E+00_JPRB, 0.47887E+00_JPRB, 0.47027E+00_JPRB, 0.46200E+00_JPRB, 0.45291E+00_JPRB ,&
&  0.44577E+00_JPRB, 0.44136E+00_JPRB, 0.43823E+00_JPRB, 0.43479E+00_JPRB, 0.43082E+00_JPRB, 0.42602E+00_JPRB, 0.42122E+00_JPRB ,&
&  0.41640E+00_JPRB, 0.41260E+00_JPRB, 0.40827E+00_JPRB, 0.40607E+00_JPRB, 0.40921E+00_JPRB, 0.41660E+00_JPRB, 0.42363E+00_JPRB ,&
&  0.45337E+00_JPRB, 0.40497E+00_JPRB, 0.41841E+00_JPRB, 0.45315E+00_JPRB, 0.49293E+00_JPRB, 0.54581E+00_JPRB, 0.58957E+00_JPRB ,&
&  0.61049E+00_JPRB, 0.63074E+00_JPRB, 0.62273E+00_JPRB, 0.60586E+00_JPRB, 0.57630E+00_JPRB, 0.54688E+00_JPRB, 0.52372E+00_JPRB ,&
&  0.50943E+00_JPRB, 0.49889E+00_JPRB, 0.48664E+00_JPRB, 0.47309E+00_JPRB, 0.46047E+00_JPRB, 0.45185E+00_JPRB, 0.44845E+00_JPRB ,&
&  0.44937E+00_JPRB, 0.45093E+00_JPRB, 0.45150E+00_JPRB, 0.45477E+00_JPRB, 0.46205E+00_JPRB, 0.47201E+00_JPRB, 0.48831E+00_JPRB ,&
&  0.51750E+00_JPRB, 0.55667E+00_JPRB, 0.58945E+00_JPRB, 0.61826E+00_JPRB, 0.63387E+00_JPRB, 0.63348E+00_JPRB, 0.60822E+00_JPRB ,&
&  0.57543E+00_JPRB, 0.54596E+00_JPRB, 0.52608E+00_JPRB, 0.51468E+00_JPRB, 0.50685E+00_JPRB, 0.50086E+00_JPRB, 0.49329E+00_JPRB ,&
&  0.48340E+00_JPRB, 0.47271E+00_JPRB, 0.46394E+00_JPRB, 0.45706E+00_JPRB, 0.45270E+00_JPRB, 0.45232E+00_JPRB, 0.45470E+00_JPRB ,&
&  0.45788E+00_JPRB, 0.46084E+00_JPRB, 0.46412E+00_JPRB, 0.46907E+00_JPRB, 0.47463E+00_JPRB, 0.48072E+00_JPRB, 0.48593E+00_JPRB ,&
&  0.49258E+00_JPRB, 0.50141E+00_JPRB, 0.51050E+00_JPRB, 0.54951E+00_JPRB, 0.58419E+00_JPRB, 0.60252E+00_JPRB, 0.61370E+00_JPRB ,&
&  0.61875E+00_JPRB, 0.62939E+00_JPRB, 0.54010E+00_JPRB, 0.58798E+00_JPRB, 0.61131E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.53569E+00_JPRB, 0.53233E+00_JPRB, 0.52482E+00_JPRB, 0.51083E+00_JPRB, 0.49555E+00_JPRB, 0.48239E+00_JPRB, 0.47436E+00_JPRB ,&
&  0.46866E+00_JPRB, 0.46059E+00_JPRB, 0.45351E+00_JPRB, 0.44810E+00_JPRB, 0.44147E+00_JPRB, 0.43403E+00_JPRB, 0.42489E+00_JPRB ,&
&  0.41767E+00_JPRB, 0.41213E+00_JPRB, 0.40806E+00_JPRB, 0.40545E+00_JPRB, 0.40364E+00_JPRB, 0.40193E+00_JPRB, 0.39954E+00_JPRB ,&
&  0.39714E+00_JPRB, 0.39537E+00_JPRB, 0.39361E+00_JPRB, 0.39309E+00_JPRB, 0.39377E+00_JPRB, 0.39815E+00_JPRB, 0.40472E+00_JPRB ,&
&  0.39530E+00_JPRB, 0.37074E+00_JPRB, 0.38290E+00_JPRB, 0.40154E+00_JPRB, 0.42471E+00_JPRB, 0.46281E+00_JPRB, 0.50282E+00_JPRB ,&
&  0.52638E+00_JPRB, 0.54066E+00_JPRB, 0.53175E+00_JPRB, 0.51796E+00_JPRB, 0.50297E+00_JPRB, 0.48684E+00_JPRB, 0.47391E+00_JPRB ,&
&  0.46249E+00_JPRB, 0.45470E+00_JPRB, 0.44813E+00_JPRB, 0.43875E+00_JPRB, 0.42749E+00_JPRB, 0.41833E+00_JPRB, 0.41292E+00_JPRB ,&
&  0.41120E+00_JPRB, 0.41229E+00_JPRB, 0.41394E+00_JPRB, 0.41582E+00_JPRB, 0.41921E+00_JPRB, 0.42576E+00_JPRB, 0.43447E+00_JPRB ,&
&  0.44312E+00_JPRB, 0.45654E+00_JPRB, 0.47462E+00_JPRB, 0.49131E+00_JPRB, 0.50637E+00_JPRB, 0.51495E+00_JPRB, 0.51787E+00_JPRB ,&
&  0.51337E+00_JPRB, 0.50192E+00_JPRB, 0.48828E+00_JPRB, 0.47462E+00_JPRB, 0.45985E+00_JPRB, 0.44337E+00_JPRB, 0.42667E+00_JPRB ,&
&  0.41292E+00_JPRB, 0.40213E+00_JPRB, 0.39468E+00_JPRB, 0.39157E+00_JPRB, 0.39104E+00_JPRB, 0.39325E+00_JPRB, 0.39674E+00_JPRB ,&
&  0.40062E+00_JPRB, 0.40469E+00_JPRB, 0.40774E+00_JPRB, 0.41214E+00_JPRB, 0.41684E+00_JPRB, 0.42251E+00_JPRB, 0.42872E+00_JPRB ,&
&  0.43586E+00_JPRB, 0.44319E+00_JPRB, 0.45127E+00_JPRB, 0.47341E+00_JPRB, 0.50106E+00_JPRB, 0.52486E+00_JPRB, 0.53771E+00_JPRB ,&
&  0.53862E+00_JPRB, 0.53649E+00_JPRB, 0.53666E+00_JPRB, 0.53422E+00_JPRB, 0.53783E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.47758E+00_JPRB, 0.47377E+00_JPRB, 0.46966E+00_JPRB, 0.46473E+00_JPRB, 0.45851E+00_JPRB, 0.45052E+00_JPRB, 0.44457E+00_JPRB ,&
&  0.44026E+00_JPRB, 0.43503E+00_JPRB, 0.43001E+00_JPRB, 0.42387E+00_JPRB, 0.41668E+00_JPRB, 0.40852E+00_JPRB, 0.39872E+00_JPRB ,&
&  0.39019E+00_JPRB, 0.38260E+00_JPRB, 0.37761E+00_JPRB, 0.37512E+00_JPRB, 0.37481E+00_JPRB, 0.37470E+00_JPRB, 0.37297E+00_JPRB ,&
&  0.37201E+00_JPRB, 0.37191E+00_JPRB, 0.37155E+00_JPRB, 0.37127E+00_JPRB, 0.37155E+00_JPRB, 0.37501E+00_JPRB, 0.37889E+00_JPRB ,&
&  0.38673E+00_JPRB, 0.36419E+00_JPRB, 0.37572E+00_JPRB, 0.38042E+00_JPRB, 0.39207E+00_JPRB, 0.41393E+00_JPRB, 0.44116E+00_JPRB ,&
&  0.46236E+00_JPRB, 0.46363E+00_JPRB, 0.45503E+00_JPRB, 0.44374E+00_JPRB, 0.43665E+00_JPRB, 0.42940E+00_JPRB, 0.42136E+00_JPRB ,&
&  0.41376E+00_JPRB, 0.40859E+00_JPRB, 0.40419E+00_JPRB, 0.39986E+00_JPRB, 0.39389E+00_JPRB, 0.38630E+00_JPRB, 0.37906E+00_JPRB ,&
&  0.37399E+00_JPRB, 0.37225E+00_JPRB, 0.37330E+00_JPRB, 0.37568E+00_JPRB, 0.37857E+00_JPRB, 0.38340E+00_JPRB, 0.38943E+00_JPRB ,&
&  0.39535E+00_JPRB, 0.39819E+00_JPRB, 0.40072E+00_JPRB, 0.40564E+00_JPRB, 0.41080E+00_JPRB, 0.41396E+00_JPRB, 0.41475E+00_JPRB ,&
&  0.41354E+00_JPRB, 0.40935E+00_JPRB, 0.40348E+00_JPRB, 0.39618E+00_JPRB, 0.38849E+00_JPRB, 0.38081E+00_JPRB, 0.37409E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.36363E+00_JPRB, 0.35904E+00_JPRB, 0.35575E+00_JPRB, 0.35462E+00_JPRB, 0.35504E+00_JPRB, 0.35664E+00_JPRB ,&
&  0.35998E+00_JPRB, 0.36381E+00_JPRB, 0.36677E+00_JPRB, 0.36977E+00_JPRB, 0.37266E+00_JPRB, 0.37546E+00_JPRB, 0.37961E+00_JPRB ,&
&  0.38562E+00_JPRB, 0.39061E+00_JPRB, 0.39544E+00_JPRB, 0.40461E+00_JPRB, 0.42220E+00_JPRB, 0.44467E+00_JPRB, 0.46325E+00_JPRB ,&
&  0.47268E+00_JPRB, 0.47282E+00_JPRB, 0.47430E+00_JPRB, 0.47521E+00_JPRB, 0.47883E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.43965E+00_JPRB, 0.43736E+00_JPRB, 0.43374E+00_JPRB, 0.43103E+00_JPRB, 0.42760E+00_JPRB, 0.42298E+00_JPRB, 0.41819E+00_JPRB ,&
&  0.41423E+00_JPRB, 0.41019E+00_JPRB, 0.40561E+00_JPRB, 0.39953E+00_JPRB, 0.39158E+00_JPRB, 0.38403E+00_JPRB, 0.37577E+00_JPRB ,&
&  0.36818E+00_JPRB, 0.36171E+00_JPRB, 0.35727E+00_JPRB, 0.35465E+00_JPRB, 0.35244E+00_JPRB, 0.35056E+00_JPRB, 0.34843E+00_JPRB ,&
&  0.34722E+00_JPRB, 0.34690E+00_JPRB, 0.34695E+00_JPRB, 0.34737E+00_JPRB, 0.34799E+00_JPRB, 0.35070E+00_JPRB, 0.35250E+00_JPRB ,&
&  0.35561E+00_JPRB, 0.35893E+00_JPRB, 0.36679E+00_JPRB, 0.37122E+00_JPRB, 0.37832E+00_JPRB, 0.38610E+00_JPRB, 0.39441E+00_JPRB ,&
&  0.40187E+00_JPRB, 0.40526E+00_JPRB, 0.40053E+00_JPRB, 0.39302E+00_JPRB, 0.38787E+00_JPRB, 0.38524E+00_JPRB, 0.37960E+00_JPRB ,&
&  0.37385E+00_JPRB, 0.36963E+00_JPRB, 0.36630E+00_JPRB, 0.36390E+00_JPRB, 0.36145E+00_JPRB, 0.35782E+00_JPRB, 0.35379E+00_JPRB ,&
&  0.34874E+00_JPRB, 0.34492E+00_JPRB, 0.34199E+00_JPRB, 0.34091E+00_JPRB, 0.34120E+00_JPRB, 0.34264E+00_JPRB, 0.34498E+00_JPRB ,&
&  0.34855E+00_JPRB, 0.35176E+00_JPRB, 0.35318E+00_JPRB, 0.35367E+00_JPRB, 0.35398E+00_JPRB, 0.35472E+00_JPRB, 0.35476E+00_JPRB ,&
&  0.35352E+00_JPRB, 0.35136E+00_JPRB, 0.34857E+00_JPRB, 0.34523E+00_JPRB, 0.34162E+00_JPRB, 0.33871E+00_JPRB, 0.33629E+00_JPRB ,&
&  0.33393E+00_JPRB, 0.33133E+00_JPRB, 0.32868E+00_JPRB, 0.32722E+00_JPRB, 0.32630E+00_JPRB, 0.32592E+00_JPRB, 0.32685E+00_JPRB ,&
&  0.32912E+00_JPRB, 0.33184E+00_JPRB, 0.33478E+00_JPRB, 0.33742E+00_JPRB, 0.33993E+00_JPRB, 0.34221E+00_JPRB, 0.34526E+00_JPRB ,&
&  0.35039E+00_JPRB, 0.35613E+00_JPRB, 0.36184E+00_JPRB, 0.36773E+00_JPRB, 0.37723E+00_JPRB, 0.39256E+00_JPRB, 0.40994E+00_JPRB ,&
&  0.42156E+00_JPRB, 0.42693E+00_JPRB, 0.43021E+00_JPRB, 0.43445E+00_JPRB, 0.43864E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.39491E+00_JPRB, 0.39658E+00_JPRB, 0.39666E+00_JPRB, 0.39717E+00_JPRB, 0.39674E+00_JPRB, 0.39466E+00_JPRB, 0.39064E+00_JPRB ,&
&  0.38623E+00_JPRB, 0.38126E+00_JPRB, 0.37601E+00_JPRB, 0.36934E+00_JPRB, 0.36252E+00_JPRB, 0.35698E+00_JPRB, 0.35110E+00_JPRB ,&
&  0.34522E+00_JPRB, 0.33984E+00_JPRB, 0.33607E+00_JPRB, 0.33301E+00_JPRB, 0.33004E+00_JPRB, 0.32718E+00_JPRB, 0.32360E+00_JPRB ,&
&  0.32052E+00_JPRB, 0.31777E+00_JPRB, 0.31812E+00_JPRB, 0.32066E+00_JPRB, 0.32277E+00_JPRB, 0.32426E+00_JPRB, 0.32555E+00_JPRB ,&
&  0.32597E+00_JPRB, 0.33610E+00_JPRB, 0.33061E+00_JPRB, 0.34130E+00_JPRB, 0.34455E+00_JPRB, 0.34528E+00_JPRB, 0.34592E+00_JPRB ,&
&  0.34867E+00_JPRB, 0.35199E+00_JPRB, 0.35442E+00_JPRB, 0.35101E+00_JPRB, 0.34597E+00_JPRB, 0.34202E+00_JPRB, 0.33771E+00_JPRB ,&
&  0.33186E+00_JPRB, 0.32940E+00_JPRB, 0.32929E+00_JPRB, 0.32957E+00_JPRB, 0.32968E+00_JPRB, 0.32922E+00_JPRB, 0.32799E+00_JPRB ,&
&  0.32618E+00_JPRB, 0.32354E+00_JPRB, 0.32045E+00_JPRB, 0.31738E+00_JPRB, 0.31543E+00_JPRB, 0.31328E+00_JPRB, 0.31147E+00_JPRB ,&
&  0.31094E+00_JPRB, 0.31174E+00_JPRB, 0.31239E+00_JPRB, 0.31247E+00_JPRB, 0.31173E+00_JPRB, 0.31116E+00_JPRB, 0.31091E+00_JPRB ,&
&  0.31008E+00_JPRB, 0.30891E+00_JPRB, 0.30740E+00_JPRB, 0.30545E+00_JPRB, 0.30336E+00_JPRB, 0.30209E+00_JPRB, 0.30173E+00_JPRB ,&
&  0.30117E+00_JPRB, 0.29951E+00_JPRB, 0.29712E+00_JPRB, 0.29653E+00_JPRB, 0.29642E+00_JPRB, 0.29670E+00_JPRB, 0.29723E+00_JPRB ,&
&  0.29840E+00_JPRB, 0.30056E+00_JPRB, 0.30298E+00_JPRB, 0.30535E+00_JPRB, 0.30775E+00_JPRB, 0.31123E+00_JPRB, 0.31544E+00_JPRB ,&
&  0.32024E+00_JPRB, 0.32544E+00_JPRB, 0.33051E+00_JPRB, 0.33524E+00_JPRB, 0.34023E+00_JPRB, 0.34788E+00_JPRB, 0.35925E+00_JPRB ,&
&  0.36955E+00_JPRB, 0.37703E+00_JPRB, 0.38145E+00_JPRB, 0.38586E+00_JPRB, 0.39088E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.35651E+00_JPRB, 0.36196E+00_JPRB, 0.36507E+00_JPRB, 0.36647E+00_JPRB, 0.36688E+00_JPRB, 0.36514E+00_JPRB, 0.36196E+00_JPRB ,&
&  0.35701E+00_JPRB, 0.35195E+00_JPRB, 0.34650E+00_JPRB, 0.34050E+00_JPRB, 0.33516E+00_JPRB, 0.33062E+00_JPRB, 0.32664E+00_JPRB ,&
&  0.32271E+00_JPRB, 0.31910E+00_JPRB, 0.31612E+00_JPRB, 0.31311E+00_JPRB, 0.30975E+00_JPRB, 0.30564E+00_JPRB, 0.30047E+00_JPRB ,&
&  0.29730E+00_JPRB, 0.29485E+00_JPRB, 0.29572E+00_JPRB, 0.29871E+00_JPRB, 0.30106E+00_JPRB, 0.30243E+00_JPRB, 0.30355E+00_JPRB ,&
&  0.30512E+00_JPRB, 0.30766E+00_JPRB, 0.30975E+00_JPRB, 0.31307E+00_JPRB, 0.31582E+00_JPRB, 0.31560E+00_JPRB, 0.31378E+00_JPRB ,&
&  0.31243E+00_JPRB, 0.31312E+00_JPRB, 0.31446E+00_JPRB, 0.31525E+00_JPRB, 0.31450E+00_JPRB, 0.31201E+00_JPRB, 0.30786E+00_JPRB ,&
&  0.30082E+00_JPRB, 0.29557E+00_JPRB, 0.29439E+00_JPRB, 0.29554E+00_JPRB, 0.29750E+00_JPRB, 0.29898E+00_JPRB, 0.29987E+00_JPRB ,&
&  0.30011E+00_JPRB, 0.29944E+00_JPRB, 0.29820E+00_JPRB, 0.29669E+00_JPRB, 0.29471E+00_JPRB, 0.29250E+00_JPRB, 0.29119E+00_JPRB ,&
&  0.28982E+00_JPRB, 0.28871E+00_JPRB, 0.28802E+00_JPRB, 0.28758E+00_JPRB, 0.28637E+00_JPRB, 0.28460E+00_JPRB, 0.28351E+00_JPRB ,&
&  0.28206E+00_JPRB, 0.28114E+00_JPRB, 0.27983E+00_JPRB, 0.27850E+00_JPRB, 0.27688E+00_JPRB, 0.27535E+00_JPRB, 0.27478E+00_JPRB ,&
&  0.27477E+00_JPRB, 0.27353E+00_JPRB, 0.27207E+00_JPRB, 0.27193E+00_JPRB, 0.27246E+00_JPRB, 0.27332E+00_JPRB, 0.27402E+00_JPRB ,&
&  0.27474E+00_JPRB, 0.27566E+00_JPRB, 0.27767E+00_JPRB, 0.28000E+00_JPRB, 0.28268E+00_JPRB, 0.28594E+00_JPRB, 0.28968E+00_JPRB ,&
&  0.29418E+00_JPRB, 0.29944E+00_JPRB, 0.30537E+00_JPRB, 0.31073E+00_JPRB, 0.31547E+00_JPRB, 0.31967E+00_JPRB, 0.32604E+00_JPRB ,&
&  0.33365E+00_JPRB, 0.33957E+00_JPRB, 0.34383E+00_JPRB, 0.34745E+00_JPRB, 0.35124E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.31937E+00_JPRB, 0.32493E+00_JPRB, 0.32934E+00_JPRB, 0.33185E+00_JPRB, 0.33205E+00_JPRB, 0.33075E+00_JPRB, 0.32772E+00_JPRB ,&
&  0.32325E+00_JPRB, 0.31865E+00_JPRB, 0.31402E+00_JPRB, 0.31020E+00_JPRB, 0.30762E+00_JPRB, 0.30541E+00_JPRB, 0.30350E+00_JPRB ,&
&  0.30112E+00_JPRB, 0.29837E+00_JPRB, 0.29513E+00_JPRB, 0.29230E+00_JPRB, 0.28901E+00_JPRB, 0.28457E+00_JPRB, 0.28129E+00_JPRB ,&
&  0.27898E+00_JPRB, 0.27687E+00_JPRB, 0.27623E+00_JPRB, 0.27754E+00_JPRB, 0.28002E+00_JPRB, 0.28239E+00_JPRB, 0.28502E+00_JPRB ,&
&  0.28688E+00_JPRB, 0.28813E+00_JPRB, 0.28914E+00_JPRB, 0.28908E+00_JPRB, 0.28898E+00_JPRB, 0.28863E+00_JPRB, 0.28794E+00_JPRB ,&
&  0.28721E+00_JPRB, 0.28658E+00_JPRB, 0.28541E+00_JPRB, 0.28530E+00_JPRB, 0.28518E+00_JPRB, 0.28399E+00_JPRB, 0.28179E+00_JPRB ,&
&  0.27819E+00_JPRB, 0.27321E+00_JPRB, 0.26929E+00_JPRB, 0.26735E+00_JPRB, 0.26676E+00_JPRB, 0.26706E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26911E+00_JPRB, 0.27076E+00_JPRB, 0.27231E+00_JPRB, 0.27306E+00_JPRB, 0.27289E+00_JPRB, 0.27269E+00_JPRB, 0.27142E+00_JPRB ,&
&  0.27018E+00_JPRB, 0.26937E+00_JPRB, 0.26882E+00_JPRB, 0.26817E+00_JPRB, 0.26597E+00_JPRB, 0.26324E+00_JPRB, 0.26252E+00_JPRB ,&
&  0.26091E+00_JPRB, 0.25983E+00_JPRB, 0.25831E+00_JPRB, 0.25747E+00_JPRB, 0.25651E+00_JPRB, 0.25513E+00_JPRB, 0.25366E+00_JPRB ,&
&  0.25249E+00_JPRB, 0.25184E+00_JPRB, 0.25143E+00_JPRB, 0.25054E+00_JPRB, 0.25083E+00_JPRB, 0.25140E+00_JPRB, 0.25280E+00_JPRB ,&
&  0.25405E+00_JPRB, 0.25527E+00_JPRB, 0.25690E+00_JPRB, 0.25851E+00_JPRB, 0.26096E+00_JPRB, 0.26387E+00_JPRB, 0.26721E+00_JPRB ,&
&  0.27010E+00_JPRB, 0.27406E+00_JPRB, 0.27859E+00_JPRB, 0.28284E+00_JPRB, 0.28626E+00_JPRB, 0.28905E+00_JPRB, 0.29324E+00_JPRB ,&
&  0.29986E+00_JPRB, 0.30557E+00_JPRB, 0.30892E+00_JPRB, 0.31211E+00_JPRB, 0.31484E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.29169E+00_JPRB, 0.29385E+00_JPRB, 0.29595E+00_JPRB, 0.29819E+00_JPRB, 0.29980E+00_JPRB, 0.30038E+00_JPRB, 0.29899E+00_JPRB ,&
&  0.29612E+00_JPRB, 0.29273E+00_JPRB, 0.28930E+00_JPRB, 0.28630E+00_JPRB, 0.28436E+00_JPRB, 0.28307E+00_JPRB, 0.28215E+00_JPRB ,&
&  0.28133E+00_JPRB, 0.28049E+00_JPRB, 0.27824E+00_JPRB, 0.27514E+00_JPRB, 0.27115E+00_JPRB, 0.26773E+00_JPRB, 0.26662E+00_JPRB ,&
&  0.26491E+00_JPRB, 0.26380E+00_JPRB, 0.26301E+00_JPRB, 0.26216E+00_JPRB, 0.26183E+00_JPRB, 0.26283E+00_JPRB, 0.26471E+00_JPRB ,&
&  0.26668E+00_JPRB, 0.26832E+00_JPRB, 0.26963E+00_JPRB, 0.27043E+00_JPRB, 0.26984E+00_JPRB, 0.26878E+00_JPRB, 0.26752E+00_JPRB ,&
&  0.26597E+00_JPRB, 0.26444E+00_JPRB, 0.26308E+00_JPRB, 0.26128E+00_JPRB, 0.25866E+00_JPRB, 0.25550E+00_JPRB, 0.25324E+00_JPRB ,&
&  0.25153E+00_JPRB, 0.24992E+00_JPRB, 0.24822E+00_JPRB, 0.24730E+00_JPRB, 0.24661E+00_JPRB, 0.24632E+00_JPRB, 0.24639E+00_JPRB ,&
&  0.24683E+00_JPRB, 0.24811E+00_JPRB, 0.24989E+00_JPRB, 0.25186E+00_JPRB, 0.25248E+00_JPRB, 0.25270E+00_JPRB, 0.25293E+00_JPRB ,&
&  0.25223E+00_JPRB, 0.25086E+00_JPRB, 0.24904E+00_JPRB, 0.24702E+00_JPRB, 0.24565E+00_JPRB, 0.24456E+00_JPRB, 0.24335E+00_JPRB ,&
&  0.24132E+00_JPRB, 0.24004E+00_JPRB, 0.23934E+00_JPRB, 0.23917E+00_JPRB, 0.23934E+00_JPRB, 0.23934E+00_JPRB, 0.23857E+00_JPRB ,&
&  0.23763E+00_JPRB, 0.23681E+00_JPRB, 0.23573E+00_JPRB, 0.23456E+00_JPRB, 0.23270E+00_JPRB, 0.23304E+00_JPRB, 0.23409E+00_JPRB ,&
&  0.23460E+00_JPRB, 0.23478E+00_JPRB, 0.23498E+00_JPRB, 0.23613E+00_JPRB, 0.23745E+00_JPRB, 0.23927E+00_JPRB, 0.24183E+00_JPRB ,&
&  0.24401E+00_JPRB, 0.24694E+00_JPRB, 0.25119E+00_JPRB, 0.25515E+00_JPRB, 0.25802E+00_JPRB, 0.25956E+00_JPRB, 0.26265E+00_JPRB ,&
&  0.26922E+00_JPRB, 0.27642E+00_JPRB, 0.28128E+00_JPRB, 0.28583E+00_JPRB, 0.28980E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.26616E+00_JPRB, 0.26741E+00_JPRB, 0.26859E+00_JPRB, 0.27000E+00_JPRB, 0.27179E+00_JPRB, 0.27285E+00_JPRB, 0.27365E+00_JPRB ,&
&  0.27296E+00_JPRB, 0.27100E+00_JPRB, 0.26867E+00_JPRB, 0.26622E+00_JPRB, 0.26505E+00_JPRB, 0.26476E+00_JPRB, 0.26472E+00_JPRB ,&
&  0.26408E+00_JPRB, 0.26371E+00_JPRB, 0.26176E+00_JPRB, 0.25859E+00_JPRB, 0.25490E+00_JPRB, 0.25235E+00_JPRB, 0.25156E+00_JPRB ,&
&  0.25094E+00_JPRB, 0.25004E+00_JPRB, 0.24960E+00_JPRB, 0.24902E+00_JPRB, 0.24675E+00_JPRB, 0.24560E+00_JPRB, 0.24599E+00_JPRB ,&
&  0.24681E+00_JPRB, 0.24832E+00_JPRB, 0.24852E+00_JPRB, 0.24616E+00_JPRB, 0.24839E+00_JPRB, 0.24819E+00_JPRB, 0.24634E+00_JPRB ,&
&  0.24394E+00_JPRB, 0.24144E+00_JPRB, 0.23916E+00_JPRB, 0.23689E+00_JPRB, 0.23456E+00_JPRB, 0.23279E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23132E+00_JPRB, 0.23150E+00_JPRB, 0.23195E+00_JPRB, 0.23284E+00_JPRB, 0.23347E+00_JPRB, 0.23383E+00_JPRB, 0.23388E+00_JPRB ,&
&  0.23401E+00_JPRB, 0.23423E+00_JPRB, 0.23465E+00_JPRB, 0.23491E+00_JPRB, 0.23521E+00_JPRB, 0.23545E+00_JPRB, 0.23529E+00_JPRB ,&
&  0.23413E+00_JPRB, 0.23224E+00_JPRB, 0.22977E+00_JPRB, 0.22882E+00_JPRB, 0.22871E+00_JPRB, 0.22707E+00_JPRB, 0.22454E+00_JPRB ,&
&  0.22218E+00_JPRB, 0.22171E+00_JPRB, 0.22294E+00_JPRB, 0.22422E+00_JPRB, 0.22524E+00_JPRB, 0.22561E+00_JPRB, 0.22536E+00_JPRB ,&
&  0.22437E+00_JPRB, 0.22199E+00_JPRB, 0.21911E+00_JPRB, 0.21680E+00_JPRB, 0.21544E+00_JPRB, 0.21532E+00_JPRB, 0.21628E+00_JPRB ,&
&  0.21647E+00_JPRB, 0.21656E+00_JPRB, 0.21764E+00_JPRB, 0.21858E+00_JPRB, 0.21929E+00_JPRB, 0.22015E+00_JPRB, 0.22166E+00_JPRB ,&
&  0.22372E+00_JPRB, 0.22705E+00_JPRB, 0.23120E+00_JPRB, 0.23578E+00_JPRB, 0.23891E+00_JPRB, 0.23820E+00_JPRB, 0.23909E+00_JPRB ,&
&  0.24418E+00_JPRB, 0.25025E+00_JPRB, 0.25532E+00_JPRB, 0.25941E+00_JPRB, 0.26322E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.24531E+00_JPRB, 0.24666E+00_JPRB, 0.24783E+00_JPRB, 0.24890E+00_JPRB, 0.25070E+00_JPRB, 0.25210E+00_JPRB, 0.25372E+00_JPRB ,&
&  0.25488E+00_JPRB, 0.25353E+00_JPRB, 0.25171E+00_JPRB, 0.25020E+00_JPRB, 0.24950E+00_JPRB, 0.24869E+00_JPRB, 0.24804E+00_JPRB ,&
&  0.24863E+00_JPRB, 0.24783E+00_JPRB, 0.24506E+00_JPRB, 0.24297E+00_JPRB, 0.23961E+00_JPRB, 0.23742E+00_JPRB, 0.23700E+00_JPRB ,&
&  0.23857E+00_JPRB, 0.23825E+00_JPRB, 0.23750E+00_JPRB, 0.23638E+00_JPRB, 0.23490E+00_JPRB, 0.23414E+00_JPRB, 0.23303E+00_JPRB ,&
&  0.23242E+00_JPRB, 0.23136E+00_JPRB, 0.22675E+00_JPRB, 0.22731E+00_JPRB, 0.22696E+00_JPRB, 0.22697E+00_JPRB, 0.22643E+00_JPRB ,&
&  0.22580E+00_JPRB, 0.22478E+00_JPRB, 0.22295E+00_JPRB, 0.22096E+00_JPRB, 0.21929E+00_JPRB, 0.21838E+00_JPRB, 0.21831E+00_JPRB ,&
&  0.21870E+00_JPRB, 0.21971E+00_JPRB, 0.22137E+00_JPRB, 0.22348E+00_JPRB, 0.22439E+00_JPRB, 0.22507E+00_JPRB, 0.22612E+00_JPRB ,&
&  0.22659E+00_JPRB, 0.22670E+00_JPRB, 0.22672E+00_JPRB, 0.22662E+00_JPRB, 0.22621E+00_JPRB, 0.22525E+00_JPRB, 0.22445E+00_JPRB ,&
&  0.22264E+00_JPRB, 0.22010E+00_JPRB, 0.21835E+00_JPRB, 0.21622E+00_JPRB, 0.21037E+00_JPRB, 0.20015E+00_JPRB, 0.19446E+00_JPRB ,&
&  0.19473E+00_JPRB, 0.19942E+00_JPRB, 0.20483E+00_JPRB, 0.20817E+00_JPRB, 0.21368E+00_JPRB, 0.21381E+00_JPRB, 0.20994E+00_JPRB ,&
&  0.20118E+00_JPRB, 0.19620E+00_JPRB, 0.19242E+00_JPRB, 0.18798E+00_JPRB, 0.18527E+00_JPRB, 0.18481E+00_JPRB, 0.18520E+00_JPRB ,&
&  0.18725E+00_JPRB, 0.19003E+00_JPRB, 0.19295E+00_JPRB, 0.19462E+00_JPRB, 0.19482E+00_JPRB, 0.19365E+00_JPRB, 0.19216E+00_JPRB ,&
&  0.19199E+00_JPRB, 0.19618E+00_JPRB, 0.20465E+00_JPRB, 0.21358E+00_JPRB, 0.21802E+00_JPRB, 0.21983E+00_JPRB, 0.22168E+00_JPRB ,&
&  0.22557E+00_JPRB, 0.23081E+00_JPRB, 0.23604E+00_JPRB, 0.23977E+00_JPRB, 0.24286E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.22519E+00_JPRB, 0.22633E+00_JPRB, 0.22751E+00_JPRB, 0.22879E+00_JPRB, 0.23025E+00_JPRB, 0.23182E+00_JPRB, 0.23315E+00_JPRB ,&
&  0.23464E+00_JPRB, 0.23558E+00_JPRB, 0.23579E+00_JPRB, 0.23469E+00_JPRB, 0.23383E+00_JPRB, 0.23378E+00_JPRB, 0.23447E+00_JPRB ,&
&  0.23247E+00_JPRB, 0.22949E+00_JPRB, 0.22819E+00_JPRB, 0.22792E+00_JPRB, 0.22725E+00_JPRB, 0.22691E+00_JPRB, 0.22768E+00_JPRB ,&
&  0.22859E+00_JPRB, 0.22815E+00_JPRB, 0.22773E+00_JPRB, 0.22647E+00_JPRB, 0.22547E+00_JPRB, 0.22471E+00_JPRB, 0.22365E+00_JPRB ,&
&  0.22007E+00_JPRB, 0.21008E+00_JPRB, 0.20379E+00_JPRB, 0.20751E+00_JPRB, 0.21414E+00_JPRB, 0.21338E+00_JPRB, 0.21345E+00_JPRB ,&
&  0.21303E+00_JPRB, 0.21171E+00_JPRB, 0.21024E+00_JPRB, 0.20889E+00_JPRB, 0.20839E+00_JPRB, 0.20878E+00_JPRB, 0.20975E+00_JPRB ,&
&  0.21084E+00_JPRB, 0.21318E+00_JPRB, 0.21577E+00_JPRB, 0.21689E+00_JPRB, 0.21803E+00_JPRB, 0.21862E+00_JPRB, 0.21883E+00_JPRB ,&
&  0.21835E+00_JPRB, 0.21721E+00_JPRB, 0.21632E+00_JPRB, 0.21540E+00_JPRB, 0.21449E+00_JPRB, 0.21311E+00_JPRB, 0.20902E+00_JPRB ,&
&  0.20372E+00_JPRB, 0.19836E+00_JPRB, 0.19188E+00_JPRB, 0.18538E+00_JPRB, 0.17964E+00_JPRB, 0.17547E+00_JPRB, 0.17194E+00_JPRB ,&
&  0.17149E+00_JPRB, 0.17550E+00_JPRB, 0.18358E+00_JPRB, 0.19316E+00_JPRB, 0.19789E+00_JPRB, 0.18831E+00_JPRB, 0.18236E+00_JPRB ,&
&  0.17657E+00_JPRB, 0.17181E+00_JPRB, 0.16887E+00_JPRB, 0.16729E+00_JPRB, 0.16617E+00_JPRB, 0.16611E+00_JPRB, 0.16671E+00_JPRB ,&
&  0.16810E+00_JPRB, 0.16961E+00_JPRB, 0.17102E+00_JPRB, 0.17195E+00_JPRB, 0.17232E+00_JPRB, 0.17219E+00_JPRB, 0.17182E+00_JPRB ,&
&  0.17239E+00_JPRB, 0.17486E+00_JPRB, 0.17917E+00_JPRB, 0.18423E+00_JPRB, 0.18853E+00_JPRB, 0.19263E+00_JPRB, 0.19821E+00_JPRB ,&
&  0.20525E+00_JPRB, 0.21072E+00_JPRB, 0.21547E+00_JPRB, 0.22058E+00_JPRB, 0.22341E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.20977E+00_JPRB, 0.21181E+00_JPRB, 0.21249E+00_JPRB, 0.21382E+00_JPRB, 0.21423E+00_JPRB, 0.21550E+00_JPRB, 0.21641E+00_JPRB ,&
&  0.21875E+00_JPRB, 0.22090E+00_JPRB, 0.22143E+00_JPRB, 0.21969E+00_JPRB, 0.21855E+00_JPRB, 0.21550E+00_JPRB, 0.21424E+00_JPRB ,&
&  0.21382E+00_JPRB, 0.21477E+00_JPRB, 0.21490E+00_JPRB, 0.21495E+00_JPRB, 0.21487E+00_JPRB, 0.21481E+00_JPRB, 0.21532E+00_JPRB ,&
&  0.21330E+00_JPRB, 0.21233E+00_JPRB, 0.21440E+00_JPRB, 0.21349E+00_JPRB, 0.21275E+00_JPRB, 0.21133E+00_JPRB, 0.21055E+00_JPRB ,&
&  0.20637E+00_JPRB, 0.20048E+00_JPRB, 0.19405E+00_JPRB, 0.19736E+00_JPRB, 0.20451E+00_JPRB, 0.20195E+00_JPRB, 0.20129E+00_JPRB ,&
&  0.20085E+00_JPRB, 0.19981E+00_JPRB, 0.19953E+00_JPRB, 0.19923E+00_JPRB, 0.20005E+00_JPRB, 0.20085E+00_JPRB, 0.20274E+00_JPRB ,&
&  0.20532E+00_JPRB, 0.20900E+00_JPRB, 0.20893E+00_JPRB, 0.20243E+00_JPRB, 0.19945E+00_JPRB, 0.19850E+00_JPRB, 0.19518E+00_JPRB ,&
&  0.19117E+00_JPRB, 0.18985E+00_JPRB, 0.18880E+00_JPRB, 0.18599E+00_JPRB, 0.18502E+00_JPRB, 0.18449E+00_JPRB, 0.18180E+00_JPRB ,&
&  0.17833E+00_JPRB, 0.17573E+00_JPRB, 0.17218E+00_JPRB, 0.16858E+00_JPRB, 0.16609E+00_JPRB, 0.16491E+00_JPRB, 0.16501E+00_JPRB ,&
&  0.16691E+00_JPRB, 0.16879E+00_JPRB, 0.17468E+00_JPRB, 0.17836E+00_JPRB, 0.17499E+00_JPRB, 0.17198E+00_JPRB, 0.16858E+00_JPRB ,&
&  0.16304E+00_JPRB, 0.16046E+00_JPRB, 0.15838E+00_JPRB, 0.15684E+00_JPRB, 0.15666E+00_JPRB, 0.15739E+00_JPRB, 0.15791E+00_JPRB ,&
&  0.15867E+00_JPRB, 0.15978E+00_JPRB, 0.16032E+00_JPRB, 0.16059E+00_JPRB, 0.16065E+00_JPRB, 0.16094E+00_JPRB, 0.16160E+00_JPRB ,&
&  0.16243E+00_JPRB, 0.16438E+00_JPRB, 0.16711E+00_JPRB, 0.17001E+00_JPRB, 0.17207E+00_JPRB, 0.17370E+00_JPRB, 0.17717E+00_JPRB ,&
&  0.18109E+00_JPRB, 0.19023E+00_JPRB, 0.20302E+00_JPRB, 0.21201E+00_JPRB, 0.20828E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.20655E+00_JPRB, 0.20784E+00_JPRB, 0.20753E+00_JPRB, 0.20770E+00_JPRB, 0.20722E+00_JPRB, 0.20815E+00_JPRB, 0.20719E+00_JPRB ,&
&  0.20699E+00_JPRB, 0.20574E+00_JPRB, 0.20459E+00_JPRB, 0.20177E+00_JPRB, 0.20025E+00_JPRB, 0.19416E+00_JPRB, 0.19243E+00_JPRB ,&
&  0.19091E+00_JPRB, 0.19165E+00_JPRB, 0.19183E+00_JPRB, 0.19228E+00_JPRB, 0.19310E+00_JPRB, 0.19468E+00_JPRB, 0.19456E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19136E+00_JPRB, 0.18898E+00_JPRB, 0.18682E+00_JPRB, 0.18722E+00_JPRB, 0.18683E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18487E+00_JPRB, 0.18550E+00_JPRB, 0.18598E+00_JPRB, 0.18735E+00_JPRB, 0.19226E+00_JPRB, 0.19053E+00_JPRB, 0.18945E+00_JPRB ,&
&  0.18948E+00_JPRB, 0.18962E+00_JPRB, 0.19041E+00_JPRB, 0.19105E+00_JPRB, 0.19341E+00_JPRB, 0.19291E+00_JPRB, 0.19574E+00_JPRB ,&
&  0.19362E+00_JPRB, 0.19234E+00_JPRB, 0.19036E+00_JPRB, 0.18746E+00_JPRB, 0.18344E+00_JPRB, 0.18090E+00_JPRB, 0.17851E+00_JPRB ,&
&  0.17583E+00_JPRB, 0.17358E+00_JPRB, 0.17067E+00_JPRB, 0.16965E+00_JPRB, 0.16810E+00_JPRB, 0.16667E+00_JPRB, 0.16539E+00_JPRB ,&
&  0.16444E+00_JPRB, 0.16312E+00_JPRB, 0.16199E+00_JPRB, 0.16124E+00_JPRB, 0.16102E+00_JPRB, 0.16114E+00_JPRB, 0.16223E+00_JPRB ,&
&  0.16427E+00_JPRB, 0.16589E+00_JPRB, 0.16717E+00_JPRB, 0.16840E+00_JPRB, 0.16692E+00_JPRB, 0.16426E+00_JPRB, 0.16163E+00_JPRB ,&
&  0.15795E+00_JPRB, 0.15593E+00_JPRB, 0.15401E+00_JPRB, 0.15286E+00_JPRB, 0.15252E+00_JPRB, 0.15242E+00_JPRB, 0.15267E+00_JPRB ,&
&  0.15294E+00_JPRB, 0.15340E+00_JPRB, 0.15373E+00_JPRB, 0.15419E+00_JPRB, 0.15479E+00_JPRB, 0.15538E+00_JPRB, 0.15627E+00_JPRB ,&
&  0.15769E+00_JPRB, 0.15951E+00_JPRB, 0.16195E+00_JPRB, 0.16478E+00_JPRB, 0.16814E+00_JPRB, 0.17248E+00_JPRB, 0.17877E+00_JPRB ,&
&  0.18511E+00_JPRB, 0.19521E+00_JPRB, 0.19610E+00_JPRB, 0.20223E+00_JPRB, 0.20566E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.20392E+00_JPRB, 0.20581E+00_JPRB, 0.20644E+00_JPRB, 0.20729E+00_JPRB, 0.20683E+00_JPRB, 0.20428E+00_JPRB, 0.20166E+00_JPRB ,&
&  0.19606E+00_JPRB, 0.19160E+00_JPRB, 0.18620E+00_JPRB, 0.18119E+00_JPRB, 0.17643E+00_JPRB, 0.17340E+00_JPRB, 0.17159E+00_JPRB ,&
&  0.17147E+00_JPRB, 0.17283E+00_JPRB, 0.17382E+00_JPRB, 0.17438E+00_JPRB, 0.17471E+00_JPRB, 0.17565E+00_JPRB, 0.17568E+00_JPRB ,&
&  0.17519E+00_JPRB, 0.17414E+00_JPRB, 0.17337E+00_JPRB, 0.17422E+00_JPRB, 0.17624E+00_JPRB, 0.17833E+00_JPRB, 0.18007E+00_JPRB ,&
&  0.18157E+00_JPRB, 0.18081E+00_JPRB, 0.18080E+00_JPRB, 0.18001E+00_JPRB, 0.17961E+00_JPRB, 0.17911E+00_JPRB, 0.17913E+00_JPRB ,&
&  0.18017E+00_JPRB, 0.18099E+00_JPRB, 0.18305E+00_JPRB, 0.18411E+00_JPRB, 0.18302E+00_JPRB, 0.18528E+00_JPRB, 0.18574E+00_JPRB ,&
&  0.18517E+00_JPRB, 0.18209E+00_JPRB, 0.17869E+00_JPRB, 0.17599E+00_JPRB, 0.17313E+00_JPRB, 0.17080E+00_JPRB, 0.16891E+00_JPRB ,&
&  0.16715E+00_JPRB, 0.16626E+00_JPRB, 0.16432E+00_JPRB, 0.16267E+00_JPRB, 0.16160E+00_JPRB, 0.16047E+00_JPRB, 0.15976E+00_JPRB ,&
&  0.15931E+00_JPRB, 0.15851E+00_JPRB, 0.15835E+00_JPRB, 0.15792E+00_JPRB, 0.15800E+00_JPRB, 0.15812E+00_JPRB, 0.15844E+00_JPRB ,&
&  0.15922E+00_JPRB, 0.16021E+00_JPRB, 0.16011E+00_JPRB, 0.15999E+00_JPRB, 0.15845E+00_JPRB, 0.15675E+00_JPRB, 0.15530E+00_JPRB ,&
&  0.15338E+00_JPRB, 0.15196E+00_JPRB, 0.15109E+00_JPRB, 0.15022E+00_JPRB, 0.15007E+00_JPRB, 0.15008E+00_JPRB, 0.15002E+00_JPRB ,&
&  0.15019E+00_JPRB, 0.15048E+00_JPRB, 0.15094E+00_JPRB, 0.15197E+00_JPRB, 0.15285E+00_JPRB, 0.15413E+00_JPRB, 0.15554E+00_JPRB ,&
&  0.15664E+00_JPRB, 0.15871E+00_JPRB, 0.16089E+00_JPRB, 0.16348E+00_JPRB, 0.16855E+00_JPRB, 0.17288E+00_JPRB, 0.18063E+00_JPRB ,&
&  0.18650E+00_JPRB, 0.19547E+00_JPRB, 0.19337E+00_JPRB, 0.19788E+00_JPRB, 0.20182E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.20315E+00_JPRB, 0.20433E+00_JPRB, 0.20397E+00_JPRB, 0.20414E+00_JPRB, 0.20287E+00_JPRB, 0.20100E+00_JPRB, 0.19832E+00_JPRB ,&
&  0.19551E+00_JPRB, 0.18976E+00_JPRB, 0.18567E+00_JPRB, 0.18112E+00_JPRB, 0.17729E+00_JPRB, 0.17391E+00_JPRB, 0.17151E+00_JPRB ,&
&  0.17040E+00_JPRB, 0.16999E+00_JPRB, 0.16974E+00_JPRB, 0.16883E+00_JPRB, 0.16719E+00_JPRB, 0.16610E+00_JPRB, 0.16474E+00_JPRB ,&
&  0.16373E+00_JPRB, 0.16369E+00_JPRB, 0.16456E+00_JPRB, 0.16642E+00_JPRB, 0.16972E+00_JPRB, 0.17206E+00_JPRB, 0.17419E+00_JPRB ,&
&  0.17622E+00_JPRB, 0.17255E+00_JPRB, 0.17242E+00_JPRB, 0.17284E+00_JPRB, 0.17284E+00_JPRB, 0.17226E+00_JPRB, 0.17122E+00_JPRB ,&
&  0.17445E+00_JPRB, 0.17531E+00_JPRB, 0.18051E+00_JPRB, 0.18029E+00_JPRB, 0.17931E+00_JPRB, 0.18064E+00_JPRB, 0.17931E+00_JPRB ,&
&  0.17911E+00_JPRB, 0.17600E+00_JPRB, 0.17370E+00_JPRB, 0.17038E+00_JPRB, 0.16705E+00_JPRB, 0.16537E+00_JPRB, 0.16346E+00_JPRB ,&
&  0.16287E+00_JPRB, 0.16278E+00_JPRB, 0.16139E+00_JPRB, 0.16054E+00_JPRB, 0.15889E+00_JPRB, 0.15702E+00_JPRB, 0.15586E+00_JPRB ,&
&  0.15467E+00_JPRB, 0.15372E+00_JPRB, 0.15322E+00_JPRB, 0.15272E+00_JPRB, 0.15250E+00_JPRB, 0.15238E+00_JPRB, 0.15225E+00_JPRB ,&
&  0.15226E+00_JPRB, 0.15218E+00_JPRB, 0.15162E+00_JPRB, 0.15133E+00_JPRB, 0.15030E+00_JPRB, 0.14960E+00_JPRB, 0.14902E+00_JPRB ,&
&  0.14850E+00_JPRB, 0.14831E+00_JPRB, 0.14820E+00_JPRB, 0.14823E+00_JPRB, 0.14834E+00_JPRB, 0.14878E+00_JPRB, 0.14929E+00_JPRB ,&
&  0.14980E+00_JPRB, 0.15074E+00_JPRB, 0.15146E+00_JPRB, 0.15250E+00_JPRB, 0.15373E+00_JPRB, 0.15477E+00_JPRB, 0.15645E+00_JPRB ,&
&  0.15830E+00_JPRB, 0.16048E+00_JPRB, 0.16368E+00_JPRB, 0.16658E+00_JPRB, 0.17000E+00_JPRB, 0.17463E+00_JPRB, 0.17962E+00_JPRB ,&
&  0.18507E+00_JPRB, 0.19119E+00_JPRB, 0.19175E+00_JPRB, 0.19752E+00_JPRB, 0.20118E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.19540E+00_JPRB, 0.19607E+00_JPRB, 0.19671E+00_JPRB, 0.19692E+00_JPRB, 0.19695E+00_JPRB, 0.19611E+00_JPRB, 0.19506E+00_JPRB ,&
&  0.19112E+00_JPRB, 0.18828E+00_JPRB, 0.18511E+00_JPRB, 0.18197E+00_JPRB, 0.17920E+00_JPRB, 0.17632E+00_JPRB, 0.17376E+00_JPRB ,&
&  0.17142E+00_JPRB, 0.16966E+00_JPRB, 0.16807E+00_JPRB, 0.16593E+00_JPRB, 0.16323E+00_JPRB, 0.16075E+00_JPRB, 0.15921E+00_JPRB ,&
&  0.15879E+00_JPRB, 0.15912E+00_JPRB, 0.16057E+00_JPRB, 0.16205E+00_JPRB, 0.16362E+00_JPRB, 0.16481E+00_JPRB, 0.16601E+00_JPRB ,&
&  0.16545E+00_JPRB, 0.16511E+00_JPRB, 0.16478E+00_JPRB, 0.16420E+00_JPRB, 0.16486E+00_JPRB, 0.16576E+00_JPRB, 0.16631E+00_JPRB ,&
&  0.16697E+00_JPRB, 0.16859E+00_JPRB, 0.16980E+00_JPRB, 0.17092E+00_JPRB, 0.17092E+00_JPRB, 0.17023E+00_JPRB, 0.16877E+00_JPRB ,&
&  0.16691E+00_JPRB, 0.16497E+00_JPRB, 0.16324E+00_JPRB, 0.16175E+00_JPRB, 0.16061E+00_JPRB, 0.15931E+00_JPRB, 0.15822E+00_JPRB ,&
&  0.15769E+00_JPRB, 0.15721E+00_JPRB, 0.15651E+00_JPRB, 0.15582E+00_JPRB, 0.15474E+00_JPRB, 0.15379E+00_JPRB, 0.15280E+00_JPRB ,&
&  0.15190E+00_JPRB, 0.15105E+00_JPRB, 0.15027E+00_JPRB, 0.14946E+00_JPRB, 0.14891E+00_JPRB, 0.14841E+00_JPRB, 0.14792E+00_JPRB ,&
&  0.14741E+00_JPRB, 0.14724E+00_JPRB, 0.14686E+00_JPRB, 0.14674E+00_JPRB, 0.14651E+00_JPRB, 0.14636E+00_JPRB, 0.14654E+00_JPRB ,&
&  0.14707E+00_JPRB, 0.14760E+00_JPRB, 0.14785E+00_JPRB, 0.14818E+00_JPRB, 0.14851E+00_JPRB, 0.14894E+00_JPRB, 0.14966E+00_JPRB ,&
&  0.15020E+00_JPRB, 0.15087E+00_JPRB, 0.15142E+00_JPRB, 0.15209E+00_JPRB, 0.15337E+00_JPRB, 0.15468E+00_JPRB, 0.15619E+00_JPRB ,&
&  0.15809E+00_JPRB, 0.16007E+00_JPRB, 0.16242E+00_JPRB, 0.16471E+00_JPRB, 0.16774E+00_JPRB, 0.16948E+00_JPRB, 0.17280E+00_JPRB ,&
&  0.17645E+00_JPRB, 0.17754E+00_JPRB, 0.18983E+00_JPRB, 0.19104E+00_JPRB, 0.19272E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.16916E+00_JPRB, 0.17112E+00_JPRB, 0.17244E+00_JPRB, 0.17402E+00_JPRB, 0.17265E+00_JPRB, 0.18444E+00_JPRB, 0.18234E+00_JPRB ,&
&  0.18111E+00_JPRB, 0.17648E+00_JPRB, 0.17619E+00_JPRB, 0.17455E+00_JPRB, 0.17252E+00_JPRB, 0.17026E+00_JPRB, 0.16817E+00_JPRB ,&
&  0.16623E+00_JPRB, 0.16408E+00_JPRB, 0.16255E+00_JPRB, 0.16132E+00_JPRB, 0.16020E+00_JPRB, 0.15918E+00_JPRB, 0.15839E+00_JPRB ,&
&  0.15838E+00_JPRB, 0.15896E+00_JPRB, 0.16005E+00_JPRB, 0.15992E+00_JPRB, 0.15961E+00_JPRB, 0.15919E+00_JPRB, 0.15897E+00_JPRB ,&
&  0.15839E+00_JPRB, 0.15797E+00_JPRB, 0.15759E+00_JPRB, 0.15703E+00_JPRB, 0.15773E+00_JPRB, 0.15843E+00_JPRB, 0.15914E+00_JPRB ,&
&  0.15981E+00_JPRB, 0.16049E+00_JPRB, 0.16054E+00_JPRB, 0.16045E+00_JPRB, 0.16009E+00_JPRB, 0.15956E+00_JPRB, 0.15871E+00_JPRB ,&
&  0.15788E+00_JPRB, 0.15700E+00_JPRB, 0.15606E+00_JPRB, 0.15541E+00_JPRB, 0.15485E+00_JPRB, 0.15432E+00_JPRB, 0.15386E+00_JPRB ,&
&  0.15362E+00_JPRB, 0.15347E+00_JPRB, 0.15328E+00_JPRB, 0.15298E+00_JPRB, 0.15261E+00_JPRB, 0.15223E+00_JPRB, 0.15190E+00_JPRB ,&
&  0.15164E+00_JPRB, 0.15115E+00_JPRB, 0.15070E+00_JPRB, 0.15023E+00_JPRB, 0.14988E+00_JPRB, 0.14941E+00_JPRB, 0.14902E+00_JPRB ,&
&  0.14865E+00_JPRB, 0.14837E+00_JPRB, 0.14828E+00_JPRB, 0.14823E+00_JPRB, 0.14818E+00_JPRB, 0.14822E+00_JPRB, 0.14834E+00_JPRB ,&
&  0.14852E+00_JPRB, 0.14871E+00_JPRB, 0.14898E+00_JPRB, 0.14918E+00_JPRB, 0.14936E+00_JPRB, 0.14951E+00_JPRB, 0.14991E+00_JPRB ,&
&  0.15038E+00_JPRB, 0.15089E+00_JPRB, 0.15136E+00_JPRB, 0.15155E+00_JPRB, 0.15237E+00_JPRB, 0.15322E+00_JPRB, 0.15416E+00_JPRB ,&
&  0.15515E+00_JPRB, 0.15645E+00_JPRB, 0.15785E+00_JPRB, 0.15936E+00_JPRB, 0.16111E+00_JPRB, 0.16222E+00_JPRB, 0.16348E+00_JPRB ,&
&  0.16518E+00_JPRB, 0.16800E+00_JPRB, 0.16895E+00_JPRB, 0.16927E+00_JPRB, 0.16938E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.15832E+00_JPRB, 0.15810E+00_JPRB, 0.15785E+00_JPRB, 0.15756E+00_JPRB, 0.15669E+00_JPRB, 0.15626E+00_JPRB, 0.15589E+00_JPRB ,&
&  0.15567E+00_JPRB, 0.15551E+00_JPRB, 0.15557E+00_JPRB, 0.15569E+00_JPRB, 0.15583E+00_JPRB, 0.15596E+00_JPRB, 0.15606E+00_JPRB ,&
&  0.15603E+00_JPRB, 0.15603E+00_JPRB, 0.15533E+00_JPRB, 0.15479E+00_JPRB, 0.15444E+00_JPRB, 0.15419E+00_JPRB, 0.15385E+00_JPRB ,&
&  0.15357E+00_JPRB, 0.15324E+00_JPRB, 0.15284E+00_JPRB, 0.15258E+00_JPRB, 0.15260E+00_JPRB, 0.15247E+00_JPRB, 0.15249E+00_JPRB ,&
&  0.15248E+00_JPRB, 0.15267E+00_JPRB, 0.15275E+00_JPRB, 0.15285E+00_JPRB, 0.15323E+00_JPRB, 0.15358E+00_JPRB, 0.15367E+00_JPRB ,&
&  0.15373E+00_JPRB, 0.15357E+00_JPRB, 0.15341E+00_JPRB, 0.15329E+00_JPRB, 0.15324E+00_JPRB, 0.15323E+00_JPRB, 0.15319E+00_JPRB ,&
&  0.15307E+00_JPRB, 0.15288E+00_JPRB, 0.15263E+00_JPRB, 0.15234E+00_JPRB, 0.15208E+00_JPRB, 0.15195E+00_JPRB, 0.15193E+00_JPRB ,&
&  0.15192E+00_JPRB, 0.15179E+00_JPRB, 0.15159E+00_JPRB, 0.15127E+00_JPRB, 0.15096E+00_JPRB, 0.15067E+00_JPRB, 0.15039E+00_JPRB ,&
&  0.15013E+00_JPRB, 0.15001E+00_JPRB, 0.14987E+00_JPRB, 0.14977E+00_JPRB, 0.14969E+00_JPRB, 0.14958E+00_JPRB, 0.14947E+00_JPRB ,&
&  0.14937E+00_JPRB, 0.14929E+00_JPRB, 0.14919E+00_JPRB, 0.14911E+00_JPRB, 0.14906E+00_JPRB, 0.14902E+00_JPRB, 0.14901E+00_JPRB ,&
&  0.14902E+00_JPRB, 0.14903E+00_JPRB, 0.14906E+00_JPRB, 0.14920E+00_JPRB, 0.14936E+00_JPRB, 0.14956E+00_JPRB, 0.14980E+00_JPRB ,&
&  0.15007E+00_JPRB, 0.15037E+00_JPRB, 0.15069E+00_JPRB, 0.15093E+00_JPRB, 0.15131E+00_JPRB, 0.15168E+00_JPRB, 0.15207E+00_JPRB ,&
&  0.15246E+00_JPRB, 0.15295E+00_JPRB, 0.15349E+00_JPRB, 0.15408E+00_JPRB, 0.15474E+00_JPRB, 0.15511E+00_JPRB, 0.15557E+00_JPRB ,&
&  0.15610E+00_JPRB, 0.15679E+00_JPRB, 0.15716E+00_JPRB, 0.15744E+00_JPRB, 0.15774E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB, 0.15154E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB ,&
&  0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB, 0.87819E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.89803E+00_JPRB, 0.89670E+00_JPRB, 0.89530E+00_JPRB, 0.89395E+00_JPRB, 0.89255E+00_JPRB, 0.89130E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.88900E+00_JPRB, 0.88795E+00_JPRB, 0.88705E+00_JPRB, 0.88609E+00_JPRB, 0.88543E+00_JPRB, 0.88466E+00_JPRB, 0.88397E+00_JPRB ,&
&  0.88314E+00_JPRB, 0.88244E+00_JPRB, 0.88126E+00_JPRB, 0.88020E+00_JPRB, 0.87893E+00_JPRB, 0.87781E+00_JPRB, 0.87623E+00_JPRB ,&
&  0.87512E+00_JPRB, 0.87360E+00_JPRB, 0.87217E+00_JPRB, 0.86992E+00_JPRB, 0.86819E+00_JPRB, 0.86638E+00_JPRB, 0.86482E+00_JPRB ,&
&  0.86346E+00_JPRB, 0.86252E+00_JPRB, 0.86164E+00_JPRB, 0.86058E+00_JPRB, 0.86003E+00_JPRB, 0.86021E+00_JPRB, 0.86051E+00_JPRB ,&
&  0.86074E+00_JPRB, 0.86118E+00_JPRB, 0.86190E+00_JPRB, 0.86221E+00_JPRB, 0.86188E+00_JPRB, 0.86172E+00_JPRB, 0.86150E+00_JPRB ,&
&  0.86108E+00_JPRB, 0.86033E+00_JPRB, 0.85948E+00_JPRB, 0.85893E+00_JPRB, 0.85822E+00_JPRB, 0.85722E+00_JPRB, 0.85631E+00_JPRB ,&
&  0.85583E+00_JPRB, 0.85537E+00_JPRB, 0.85480E+00_JPRB, 0.85411E+00_JPRB, 0.85381E+00_JPRB, 0.85353E+00_JPRB, 0.85315E+00_JPRB ,&
&  0.85277E+00_JPRB, 0.85248E+00_JPRB, 0.85206E+00_JPRB, 0.85155E+00_JPRB, 0.85111E+00_JPRB, 0.85122E+00_JPRB, 0.85160E+00_JPRB ,&
&  0.85218E+00_JPRB, 0.85318E+00_JPRB, 0.85451E+00_JPRB, 0.85584E+00_JPRB, 0.85738E+00_JPRB, 0.85919E+00_JPRB, 0.86126E+00_JPRB ,&
&  0.86344E+00_JPRB, 0.86576E+00_JPRB, 0.86849E+00_JPRB, 0.87145E+00_JPRB, 0.87426E+00_JPRB, 0.87707E+00_JPRB, 0.88013E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.88513E+00_JPRB, 0.88705E+00_JPRB, 0.88898E+00_JPRB, 0.89119E+00_JPRB, 0.89316E+00_JPRB, 0.89482E+00_JPRB ,&
&  0.89627E+00_JPRB, 0.89762E+00_JPRB, 0.89885E+00_JPRB, 0.90011E+00_JPRB, 0.90054E+00_JPRB, 0.90078E+00_JPRB, 0.90091E+00_JPRB ,&
&  0.90087E+00_JPRB, 0.90064E+00_JPRB, 0.90026E+00_JPRB, 0.89976E+00_JPRB, 0.89905E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.90543E+00_JPRB, 0.90181E+00_JPRB, 0.89853E+00_JPRB, 0.89586E+00_JPRB, 0.89322E+00_JPRB, 0.89137E+00_JPRB, 0.88980E+00_JPRB ,&
&  0.88855E+00_JPRB, 0.88695E+00_JPRB, 0.88527E+00_JPRB, 0.88429E+00_JPRB, 0.88416E+00_JPRB, 0.88435E+00_JPRB, 0.88346E+00_JPRB ,&
&  0.88269E+00_JPRB, 0.88147E+00_JPRB, 0.88051E+00_JPRB, 0.87931E+00_JPRB, 0.87740E+00_JPRB, 0.87493E+00_JPRB, 0.87167E+00_JPRB ,&
&  0.86918E+00_JPRB, 0.86630E+00_JPRB, 0.86326E+00_JPRB, 0.85809E+00_JPRB, 0.85377E+00_JPRB, 0.84956E+00_JPRB, 0.84538E+00_JPRB ,&
&  0.84031E+00_JPRB, 0.83599E+00_JPRB, 0.83245E+00_JPRB, 0.82934E+00_JPRB, 0.83259E+00_JPRB, 0.83462E+00_JPRB, 0.83575E+00_JPRB ,&
&  0.83549E+00_JPRB, 0.83158E+00_JPRB, 0.83029E+00_JPRB, 0.82963E+00_JPRB, 0.82929E+00_JPRB, 0.83310E+00_JPRB, 0.83574E+00_JPRB ,&
&  0.83770E+00_JPRB, 0.83894E+00_JPRB, 0.84208E+00_JPRB, 0.84357E+00_JPRB, 0.84422E+00_JPRB, 0.84376E+00_JPRB, 0.84256E+00_JPRB ,&
&  0.84272E+00_JPRB, 0.84287E+00_JPRB, 0.84280E+00_JPRB, 0.84194E+00_JPRB, 0.84173E+00_JPRB, 0.84159E+00_JPRB, 0.84076E+00_JPRB ,&
&  0.83924E+00_JPRB, 0.83794E+00_JPRB, 0.83660E+00_JPRB, 0.83522E+00_JPRB, 0.83374E+00_JPRB, 0.83318E+00_JPRB, 0.83287E+00_JPRB ,&
&  0.83287E+00_JPRB, 0.83339E+00_JPRB, 0.83461E+00_JPRB, 0.83612E+00_JPRB, 0.83809E+00_JPRB, 0.84122E+00_JPRB, 0.84585E+00_JPRB ,&
&  0.85163E+00_JPRB, 0.85835E+00_JPRB, 0.86559E+00_JPRB, 0.87236E+00_JPRB, 0.87790E+00_JPRB, 0.88270E+00_JPRB, 0.88795E+00_JPRB ,&
&  0.89332E+00_JPRB, 0.89781E+00_JPRB, 0.90207E+00_JPRB, 0.90623E+00_JPRB, 0.91065E+00_JPRB, 0.91375E+00_JPRB, 0.91588E+00_JPRB ,&
&  0.91751E+00_JPRB, 0.91938E+00_JPRB, 0.92116E+00_JPRB, 0.92302E+00_JPRB, 0.92405E+00_JPRB, 0.92382E+00_JPRB, 0.92281E+00_JPRB ,&
&  0.92126E+00_JPRB, 0.91955E+00_JPRB, 0.91707E+00_JPRB, 0.91350E+00_JPRB, 0.90965E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.89982E+00_JPRB, 0.89763E+00_JPRB, 0.89637E+00_JPRB, 0.89613E+00_JPRB, 0.89636E+00_JPRB, 0.89635E+00_JPRB, 0.89674E+00_JPRB ,&
&  0.89671E+00_JPRB, 0.89581E+00_JPRB, 0.89418E+00_JPRB, 0.89395E+00_JPRB, 0.89325E+00_JPRB, 0.89313E+00_JPRB, 0.89123E+00_JPRB ,&
&  0.89010E+00_JPRB, 0.88765E+00_JPRB, 0.88743E+00_JPRB, 0.88684E+00_JPRB, 0.88562E+00_JPRB, 0.88474E+00_JPRB, 0.87574E+00_JPRB ,&
&  0.86743E+00_JPRB, 0.86054E+00_JPRB, 0.85163E+00_JPRB, 0.84323E+00_JPRB, 0.83744E+00_JPRB, 0.80321E+00_JPRB, 0.79258E+00_JPRB ,&
&  0.80534E+00_JPRB, 0.81347E+00_JPRB, 0.82082E+00_JPRB, 0.82622E+00_JPRB, 0.81178E+00_JPRB, 0.79192E+00_JPRB, 0.77963E+00_JPRB ,&
&  0.74787E+00_JPRB, 0.75543E+00_JPRB, 0.76110E+00_JPRB, 0.76594E+00_JPRB, 0.77441E+00_JPRB, 0.77791E+00_JPRB, 0.78656E+00_JPRB ,&
&  0.80779E+00_JPRB, 0.81356E+00_JPRB, 0.82109E+00_JPRB, 0.82720E+00_JPRB, 0.83005E+00_JPRB, 0.83366E+00_JPRB, 0.84198E+00_JPRB ,&
&  0.84560E+00_JPRB, 0.84884E+00_JPRB, 0.85100E+00_JPRB, 0.85087E+00_JPRB, 0.85231E+00_JPRB, 0.85270E+00_JPRB, 0.85256E+00_JPRB ,&
&  0.85004E+00_JPRB, 0.84855E+00_JPRB, 0.84720E+00_JPRB, 0.84557E+00_JPRB, 0.84463E+00_JPRB, 0.84122E+00_JPRB, 0.83941E+00_JPRB ,&
&  0.83877E+00_JPRB, 0.83994E+00_JPRB, 0.84213E+00_JPRB, 0.84579E+00_JPRB, 0.85152E+00_JPRB, 0.85932E+00_JPRB, 0.86753E+00_JPRB ,&
&  0.87704E+00_JPRB, 0.88605E+00_JPRB, 0.89491E+00_JPRB, 0.90313E+00_JPRB, 0.91017E+00_JPRB, 0.91684E+00_JPRB, 0.92250E+00_JPRB ,&
&  0.92917E+00_JPRB, 0.93604E+00_JPRB, 0.94013E+00_JPRB, 0.94140E+00_JPRB, 0.94538E+00_JPRB, 0.94883E+00_JPRB, 0.95036E+00_JPRB ,&
&  0.95114E+00_JPRB, 0.95177E+00_JPRB, 0.95001E+00_JPRB, 0.94669E+00_JPRB, 0.94289E+00_JPRB, 0.93906E+00_JPRB, 0.93446E+00_JPRB ,&
&  0.92955E+00_JPRB, 0.92478E+00_JPRB, 0.91901E+00_JPRB, 0.91181E+00_JPRB, 0.90522E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.89668E+00_JPRB, 0.89758E+00_JPRB, 0.90035E+00_JPRB, 0.90121E+00_JPRB, 0.90356E+00_JPRB, 0.90342E+00_JPRB, 0.90457E+00_JPRB ,&
&  0.90391E+00_JPRB, 0.90418E+00_JPRB, 0.90408E+00_JPRB, 0.90347E+00_JPRB, 0.90246E+00_JPRB, 0.90049E+00_JPRB, 0.89872E+00_JPRB ,&
&  0.89747E+00_JPRB, 0.89580E+00_JPRB, 0.89574E+00_JPRB, 0.89511E+00_JPRB, 0.89336E+00_JPRB, 0.89164E+00_JPRB, 0.88117E+00_JPRB ,&
&  0.86981E+00_JPRB, 0.85852E+00_JPRB, 0.81694E+00_JPRB, 0.81517E+00_JPRB, 0.80934E+00_JPRB, 0.80615E+00_JPRB, 0.80875E+00_JPRB ,&
&  0.80296E+00_JPRB, 0.79635E+00_JPRB, 0.77514E+00_JPRB, 0.75456E+00_JPRB, 0.73946E+00_JPRB, 0.72247E+00_JPRB, 0.72080E+00_JPRB ,&
&  0.71835E+00_JPRB, 0.72349E+00_JPRB, 0.72895E+00_JPRB, 0.73314E+00_JPRB, 0.74488E+00_JPRB, 0.75190E+00_JPRB, 0.76482E+00_JPRB ,&
&  0.78081E+00_JPRB, 0.79257E+00_JPRB, 0.79348E+00_JPRB, 0.79052E+00_JPRB, 0.81323E+00_JPRB, 0.82843E+00_JPRB, 0.84207E+00_JPRB ,&
&  0.85008E+00_JPRB, 0.85149E+00_JPRB, 0.85604E+00_JPRB, 0.85688E+00_JPRB, 0.86011E+00_JPRB, 0.86626E+00_JPRB, 0.86959E+00_JPRB ,&
&  0.87337E+00_JPRB, 0.87334E+00_JPRB, 0.87127E+00_JPRB, 0.86762E+00_JPRB, 0.86120E+00_JPRB, 0.85746E+00_JPRB, 0.85683E+00_JPRB ,&
&  0.85993E+00_JPRB, 0.86752E+00_JPRB, 0.87787E+00_JPRB, 0.89033E+00_JPRB, 0.90304E+00_JPRB, 0.91413E+00_JPRB, 0.92289E+00_JPRB ,&
&  0.93030E+00_JPRB, 0.93578E+00_JPRB, 0.94605E+00_JPRB, 0.94812E+00_JPRB, 0.95452E+00_JPRB, 0.96915E+00_JPRB, 0.97151E+00_JPRB ,&
&  0.97632E+00_JPRB, 0.98472E+00_JPRB, 0.99029E+00_JPRB, 0.99480E+00_JPRB, 0.99796E+00_JPRB, 0.99945E+00_JPRB, 0.10004E+01_JPRB ,&
&  0.99965E+00_JPRB, 0.99551E+00_JPRB, 0.98852E+00_JPRB, 0.97954E+00_JPRB, 0.97083E+00_JPRB, 0.96205E+00_JPRB, 0.95103E+00_JPRB ,&
&  0.93959E+00_JPRB, 0.92744E+00_JPRB, 0.91524E+00_JPRB, 0.90344E+00_JPRB, 0.89874E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.90203E+00_JPRB, 0.90269E+00_JPRB, 0.90749E+00_JPRB, 0.90742E+00_JPRB, 0.90993E+00_JPRB, 0.90953E+00_JPRB, 0.90992E+00_JPRB ,&
&  0.90927E+00_JPRB, 0.90933E+00_JPRB, 0.90831E+00_JPRB, 0.90788E+00_JPRB, 0.90673E+00_JPRB, 0.90599E+00_JPRB, 0.90500E+00_JPRB ,&
&  0.90354E+00_JPRB, 0.90198E+00_JPRB, 0.90030E+00_JPRB, 0.89855E+00_JPRB, 0.89754E+00_JPRB, 0.89648E+00_JPRB, 0.89068E+00_JPRB ,&
&  0.88213E+00_JPRB, 0.86508E+00_JPRB, 0.83473E+00_JPRB, 0.83240E+00_JPRB, 0.82880E+00_JPRB, 0.83063E+00_JPRB, 0.82908E+00_JPRB ,&
&  0.81865E+00_JPRB, 0.76005E+00_JPRB, 0.76603E+00_JPRB, 0.75313E+00_JPRB, 0.74659E+00_JPRB, 0.73972E+00_JPRB, 0.72386E+00_JPRB ,&
&  0.71845E+00_JPRB, 0.71192E+00_JPRB, 0.71070E+00_JPRB, 0.71798E+00_JPRB, 0.72541E+00_JPRB, 0.73744E+00_JPRB, 0.75203E+00_JPRB ,&
&  0.75102E+00_JPRB, 0.77481E+00_JPRB, 0.78828E+00_JPRB, 0.79611E+00_JPRB, 0.81924E+00_JPRB, 0.83783E+00_JPRB, 0.84213E+00_JPRB ,&
&  0.84798E+00_JPRB, 0.84567E+00_JPRB, 0.85074E+00_JPRB, 0.86130E+00_JPRB, 0.87264E+00_JPRB, 0.87484E+00_JPRB, 0.87795E+00_JPRB ,&
&  0.87990E+00_JPRB, 0.88160E+00_JPRB, 0.88351E+00_JPRB, 0.88326E+00_JPRB, 0.88571E+00_JPRB, 0.89335E+00_JPRB, 0.90357E+00_JPRB ,&
&  0.91848E+00_JPRB, 0.93531E+00_JPRB, 0.95454E+00_JPRB, 0.97285E+00_JPRB, 0.98805E+00_JPRB, 0.10028E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.10279E+01_JPRB, 0.10376E+01_JPRB, 0.10424E+01_JPRB, 0.10459E+01_JPRB, 0.10458E+01_JPRB, 0.10411E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10514E+01_JPRB, 0.10524E+01_JPRB, 0.10531E+01_JPRB, 0.10551E+01_JPRB, 0.10552E+01_JPRB, 0.10533E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.10441E+01_JPRB, 0.10354E+01_JPRB, 0.10198E+01_JPRB, 0.10034E+01_JPRB, 0.99137E+00_JPRB, 0.97335E+00_JPRB ,&
&  0.95382E+00_JPRB, 0.93116E+00_JPRB, 0.91445E+00_JPRB, 0.90185E+00_JPRB, 0.90033E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.91511E+00_JPRB, 0.91869E+00_JPRB, 0.92502E+00_JPRB, 0.92730E+00_JPRB, 0.93042E+00_JPRB, 0.92991E+00_JPRB, 0.93095E+00_JPRB ,&
&  0.93118E+00_JPRB, 0.93243E+00_JPRB, 0.93285E+00_JPRB, 0.93273E+00_JPRB, 0.93148E+00_JPRB, 0.92989E+00_JPRB, 0.92734E+00_JPRB ,&
&  0.92333E+00_JPRB, 0.91937E+00_JPRB, 0.91333E+00_JPRB, 0.90801E+00_JPRB, 0.90489E+00_JPRB, 0.90520E+00_JPRB, 0.90183E+00_JPRB ,&
&  0.89466E+00_JPRB, 0.88047E+00_JPRB, 0.85844E+00_JPRB, 0.85202E+00_JPRB, 0.85354E+00_JPRB, 0.84194E+00_JPRB, 0.84134E+00_JPRB ,&
&  0.83649E+00_JPRB, 0.83511E+00_JPRB, 0.82416E+00_JPRB, 0.81813E+00_JPRB, 0.79538E+00_JPRB, 0.71652E+00_JPRB, 0.71129E+00_JPRB ,&
&  0.70653E+00_JPRB, 0.69621E+00_JPRB, 0.69506E+00_JPRB, 0.69741E+00_JPRB, 0.70578E+00_JPRB, 0.73289E+00_JPRB, 0.75644E+00_JPRB ,&
&  0.75471E+00_JPRB, 0.78261E+00_JPRB, 0.79401E+00_JPRB, 0.80443E+00_JPRB, 0.82400E+00_JPRB, 0.84064E+00_JPRB, 0.84109E+00_JPRB ,&
&  0.84514E+00_JPRB, 0.84805E+00_JPRB, 0.85720E+00_JPRB, 0.86720E+00_JPRB, 0.87624E+00_JPRB, 0.88381E+00_JPRB, 0.89393E+00_JPRB ,&
&  0.90702E+00_JPRB, 0.91894E+00_JPRB, 0.93382E+00_JPRB, 0.94347E+00_JPRB, 0.95171E+00_JPRB, 0.96163E+00_JPRB, 0.97507E+00_JPRB ,&
&  0.99475E+00_JPRB, 0.10153E+01_JPRB, 0.10426E+01_JPRB, 0.10533E+01_JPRB, 0.10690E+01_JPRB, 0.10834E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.11440E+01_JPRB, 0.11548E+01_JPRB, 0.11634E+01_JPRB, 0.11482E+01_JPRB, 0.11325E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11199E+01_JPRB, 0.11077E+01_JPRB, 0.11015E+01_JPRB, 0.10978E+01_JPRB, 0.10963E+01_JPRB, 0.10941E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.10795E+01_JPRB, 0.10661E+01_JPRB, 0.10472E+01_JPRB, 0.10300E+01_JPRB, 0.10132E+01_JPRB, 0.98340E+00_JPRB ,&
&  0.95335E+00_JPRB, 0.92272E+00_JPRB, 0.90845E+00_JPRB, 0.90227E+00_JPRB, 0.90677E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.93720E+00_JPRB, 0.94793E+00_JPRB, 0.95805E+00_JPRB, 0.96586E+00_JPRB, 0.96765E+00_JPRB, 0.96609E+00_JPRB, 0.96184E+00_JPRB ,&
&  0.95757E+00_JPRB, 0.95634E+00_JPRB, 0.95449E+00_JPRB, 0.95303E+00_JPRB, 0.95099E+00_JPRB, 0.94727E+00_JPRB, 0.94296E+00_JPRB ,&
&  0.93958E+00_JPRB, 0.92155E+00_JPRB, 0.91279E+00_JPRB, 0.91235E+00_JPRB, 0.90978E+00_JPRB, 0.90860E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.89886E+00_JPRB, 0.88755E+00_JPRB, 0.87988E+00_JPRB, 0.86952E+00_JPRB, 0.85867E+00_JPRB, 0.86432E+00_JPRB, 0.86308E+00_JPRB ,&
&  0.84185E+00_JPRB, 0.82604E+00_JPRB, 0.83317E+00_JPRB, 0.83404E+00_JPRB, 0.79374E+00_JPRB, 0.75287E+00_JPRB, 0.72570E+00_JPRB ,&
&  0.72615E+00_JPRB, 0.70688E+00_JPRB, 0.70366E+00_JPRB, 0.70510E+00_JPRB, 0.71723E+00_JPRB, 0.73147E+00_JPRB, 0.75795E+00_JPRB ,&
&  0.76616E+00_JPRB, 0.77217E+00_JPRB, 0.79383E+00_JPRB, 0.81444E+00_JPRB, 0.82645E+00_JPRB, 0.83749E+00_JPRB, 0.83993E+00_JPRB ,&
&  0.84919E+00_JPRB, 0.85838E+00_JPRB, 0.86795E+00_JPRB, 0.87896E+00_JPRB, 0.89523E+00_JPRB, 0.90595E+00_JPRB, 0.92146E+00_JPRB ,&
&  0.98719E+00_JPRB, 0.10431E+01_JPRB, 0.10826E+01_JPRB, 0.11135E+01_JPRB, 0.11314E+01_JPRB, 0.11364E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11147E+01_JPRB, 0.11340E+01_JPRB, 0.11533E+01_JPRB, 0.11772E+01_JPRB, 0.11979E+01_JPRB, 0.12132E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.11888E+01_JPRB, 0.11847E+01_JPRB, 0.12901E+01_JPRB, 0.13181E+01_JPRB, 0.13031E+01_JPRB, 0.12653E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.11593E+01_JPRB, 0.11440E+01_JPRB, 0.11311E+01_JPRB, 0.11194E+01_JPRB, 0.11058E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.10944E+01_JPRB, 0.10796E+01_JPRB, 0.10627E+01_JPRB, 0.10450E+01_JPRB, 0.10283E+01_JPRB, 0.10120E+01_JPRB, 0.98503E+00_JPRB ,&
&  0.95399E+00_JPRB, 0.92773E+00_JPRB, 0.91850E+00_JPRB, 0.91895E+00_JPRB, 0.92723E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.97179E+00_JPRB, 0.98680E+00_JPRB, 0.10003E+01_JPRB, 0.10088E+01_JPRB, 0.10045E+01_JPRB, 0.98116E+00_JPRB, 0.96148E+00_JPRB ,&
&  0.94843E+00_JPRB, 0.94556E+00_JPRB, 0.94290E+00_JPRB, 0.93810E+00_JPRB, 0.93469E+00_JPRB, 0.93139E+00_JPRB, 0.92932E+00_JPRB ,&
&  0.92871E+00_JPRB, 0.92474E+00_JPRB, 0.91484E+00_JPRB, 0.90361E+00_JPRB, 0.89855E+00_JPRB, 0.89886E+00_JPRB, 0.90089E+00_JPRB ,&
&  0.89799E+00_JPRB, 0.89381E+00_JPRB, 0.89538E+00_JPRB, 0.88259E+00_JPRB, 0.87264E+00_JPRB, 0.85385E+00_JPRB, 0.84322E+00_JPRB ,&
&  0.84500E+00_JPRB, 0.84802E+00_JPRB, 0.82932E+00_JPRB, 0.82780E+00_JPRB, 0.83142E+00_JPRB, 0.78749E+00_JPRB, 0.75470E+00_JPRB ,&
&  0.73479E+00_JPRB, 0.72020E+00_JPRB, 0.71359E+00_JPRB, 0.71617E+00_JPRB, 0.74626E+00_JPRB, 0.78741E+00_JPRB, 0.79934E+00_JPRB ,&
&  0.75951E+00_JPRB, 0.78080E+00_JPRB, 0.80862E+00_JPRB, 0.82279E+00_JPRB, 0.83319E+00_JPRB, 0.84758E+00_JPRB, 0.86151E+00_JPRB ,&
&  0.87486E+00_JPRB, 0.88757E+00_JPRB, 0.89935E+00_JPRB, 0.92882E+00_JPRB, 0.94843E+00_JPRB, 0.95023E+00_JPRB, 0.98255E+00_JPRB ,&
&  0.11481E+01_JPRB, 0.12783E+01_JPRB, 0.13551E+01_JPRB, 0.13850E+01_JPRB, 0.13893E+01_JPRB, 0.13651E+01_JPRB, 0.13477E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.13314E+01_JPRB, 0.13347E+01_JPRB, 0.13394E+01_JPRB, 0.13426E+01_JPRB, 0.13303E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.12801E+01_JPRB, 0.15713E+01_JPRB, 0.15212E+01_JPRB, 0.14486E+01_JPRB, 0.13980E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.12801E+01_JPRB, 0.12417E+01_JPRB, 0.12193E+01_JPRB, 0.11950E+01_JPRB, 0.11337E+01_JPRB, 0.10879E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.10358E+01_JPRB, 0.10224E+01_JPRB, 0.10101E+01_JPRB, 0.99787E+00_JPRB, 0.98728E+00_JPRB, 0.96894E+00_JPRB ,&
&  0.95200E+00_JPRB, 0.94623E+00_JPRB, 0.94685E+00_JPRB, 0.95460E+00_JPRB, 0.96252E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.99098E+00_JPRB, 0.10280E+01_JPRB, 0.10523E+01_JPRB, 0.10563E+01_JPRB, 0.10437E+01_JPRB, 0.98761E+00_JPRB, 0.93847E+00_JPRB ,&
&  0.90970E+00_JPRB, 0.90031E+00_JPRB, 0.90099E+00_JPRB, 0.90143E+00_JPRB, 0.90305E+00_JPRB, 0.90544E+00_JPRB, 0.90519E+00_JPRB ,&
&  0.90230E+00_JPRB, 0.90234E+00_JPRB, 0.90348E+00_JPRB, 0.89968E+00_JPRB, 0.89287E+00_JPRB, 0.88848E+00_JPRB, 0.88915E+00_JPRB ,&
&  0.88972E+00_JPRB, 0.88863E+00_JPRB, 0.88942E+00_JPRB, 0.88522E+00_JPRB, 0.88462E+00_JPRB, 0.87722E+00_JPRB, 0.86469E+00_JPRB ,&
&  0.84568E+00_JPRB, 0.84510E+00_JPRB, 0.85649E+00_JPRB, 0.85500E+00_JPRB, 0.86024E+00_JPRB, 0.82950E+00_JPRB, 0.78650E+00_JPRB ,&
&  0.75858E+00_JPRB, 0.74790E+00_JPRB, 0.76407E+00_JPRB, 0.80953E+00_JPRB, 0.83850E+00_JPRB, 0.79767E+00_JPRB, 0.81505E+00_JPRB ,&
&  0.81470E+00_JPRB, 0.81572E+00_JPRB, 0.82162E+00_JPRB, 0.84039E+00_JPRB, 0.86042E+00_JPRB, 0.87835E+00_JPRB, 0.89285E+00_JPRB ,&
&  0.90964E+00_JPRB, 0.93015E+00_JPRB, 0.95560E+00_JPRB, 0.97790E+00_JPRB, 0.99895E+00_JPRB, 0.10847E+01_JPRB, 0.11933E+01_JPRB ,&
&  0.13132E+01_JPRB, 0.13973E+01_JPRB, 0.14398E+01_JPRB, 0.15065E+01_JPRB, 0.15531E+01_JPRB, 0.15681E+01_JPRB, 0.15662E+01_JPRB ,&
&  0.15626E+01_JPRB, 0.15311E+01_JPRB, 0.15042E+01_JPRB, 0.15002E+01_JPRB, 0.15072E+01_JPRB, 0.15038E+01_JPRB, 0.14694E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.13528E+01_JPRB, 0.12804E+01_JPRB, 0.11906E+01_JPRB, 0.11798E+01_JPRB, 0.12097E+01_JPRB, 0.12401E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.12195E+01_JPRB, 0.12097E+01_JPRB, 0.11943E+01_JPRB, 0.11039E+01_JPRB, 0.10410E+01_JPRB, 0.10006E+01_JPRB ,&
&  0.98040E+00_JPRB, 0.95778E+00_JPRB, 0.95621E+00_JPRB, 0.95422E+00_JPRB, 0.95783E+00_JPRB, 0.96167E+00_JPRB, 0.94807E+00_JPRB ,&
&  0.93701E+00_JPRB, 0.93175E+00_JPRB, 0.93780E+00_JPRB, 0.94686E+00_JPRB, 0.96655E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.11001E+01_JPRB, 0.11143E+01_JPRB, 0.11066E+01_JPRB, 0.11042E+01_JPRB, 0.10769E+01_JPRB, 0.10422E+01_JPRB, 0.10019E+01_JPRB ,&
&  0.97179E+00_JPRB, 0.93664E+00_JPRB, 0.90292E+00_JPRB, 0.87343E+00_JPRB, 0.85915E+00_JPRB, 0.85544E+00_JPRB, 0.86122E+00_JPRB ,&
&  0.87194E+00_JPRB, 0.88257E+00_JPRB, 0.88939E+00_JPRB, 0.88694E+00_JPRB, 0.88107E+00_JPRB, 0.87958E+00_JPRB, 0.88284E+00_JPRB ,&
&  0.88528E+00_JPRB, 0.88862E+00_JPRB, 0.88976E+00_JPRB, 0.88704E+00_JPRB, 0.88313E+00_JPRB, 0.87956E+00_JPRB, 0.88181E+00_JPRB ,&
&  0.88254E+00_JPRB, 0.88039E+00_JPRB, 0.88689E+00_JPRB, 0.89006E+00_JPRB, 0.89591E+00_JPRB, 0.87199E+00_JPRB, 0.82832E+00_JPRB ,&
&  0.79541E+00_JPRB, 0.79509E+00_JPRB, 0.85303E+00_JPRB, 0.82915E+00_JPRB, 0.85428E+00_JPRB, 0.86759E+00_JPRB, 0.86438E+00_JPRB ,&
&  0.86063E+00_JPRB, 0.85949E+00_JPRB, 0.87612E+00_JPRB, 0.89208E+00_JPRB, 0.90670E+00_JPRB, 0.92507E+00_JPRB, 0.94421E+00_JPRB ,&
&  0.98054E+00_JPRB, 0.10309E+01_JPRB, 0.10783E+01_JPRB, 0.10960E+01_JPRB, 0.12234E+01_JPRB, 0.15659E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.17078E+01_JPRB, 0.17473E+01_JPRB, 0.18217E+01_JPRB, 0.18732E+01_JPRB, 0.18469E+01_JPRB, 0.18294E+01_JPRB, 0.18174E+01_JPRB ,&
&  0.17727E+01_JPRB, 0.17542E+01_JPRB, 0.17515E+01_JPRB, 0.17889E+01_JPRB, 0.17729E+01_JPRB, 0.17387E+01_JPRB, 0.17077E+01_JPRB ,&
&  0.16791E+01_JPRB, 0.16115E+01_JPRB, 0.15408E+01_JPRB, 0.14410E+01_JPRB, 0.13838E+01_JPRB, 0.13460E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.12989E+01_JPRB, 0.12873E+01_JPRB, 0.12761E+01_JPRB, 0.12514E+01_JPRB, 0.11617E+01_JPRB, 0.10631E+01_JPRB, 0.99638E+00_JPRB ,&
&  0.95381E+00_JPRB, 0.92454E+00_JPRB, 0.92105E+00_JPRB, 0.92967E+00_JPRB, 0.94657E+00_JPRB, 0.96253E+00_JPRB, 0.95006E+00_JPRB ,&
&  0.95282E+00_JPRB, 0.96283E+00_JPRB, 0.98911E+00_JPRB, 0.10268E+01_JPRB, 0.10664E+01_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.12147E+01_JPRB, 0.12173E+01_JPRB, 0.11941E+01_JPRB, 0.11778E+01_JPRB, 0.11503E+01_JPRB, 0.10981E+01_JPRB, 0.10231E+01_JPRB ,&
&  0.98263E+00_JPRB, 0.93675E+00_JPRB, 0.90418E+00_JPRB, 0.89114E+00_JPRB, 0.88304E+00_JPRB, 0.86913E+00_JPRB, 0.85283E+00_JPRB ,&
&  0.85344E+00_JPRB, 0.87062E+00_JPRB, 0.88997E+00_JPRB, 0.89895E+00_JPRB, 0.90464E+00_JPRB, 0.90509E+00_JPRB, 0.90832E+00_JPRB ,&
&  0.91140E+00_JPRB, 0.91398E+00_JPRB, 0.91755E+00_JPRB, 0.91028E+00_JPRB, 0.90451E+00_JPRB, 0.90591E+00_JPRB, 0.90876E+00_JPRB ,&
&  0.91585E+00_JPRB, 0.91558E+00_JPRB, 0.92051E+00_JPRB, 0.91868E+00_JPRB, 0.91195E+00_JPRB, 0.91012E+00_JPRB, 0.85573E+00_JPRB ,&
&  0.82365E+00_JPRB, 0.85222E+00_JPRB, 0.84735E+00_JPRB, 0.85663E+00_JPRB, 0.87898E+00_JPRB, 0.89107E+00_JPRB, 0.89966E+00_JPRB ,&
&  0.90492E+00_JPRB, 0.91133E+00_JPRB, 0.92300E+00_JPRB, 0.94171E+00_JPRB, 0.96931E+00_JPRB, 0.99992E+00_JPRB, 0.10312E+01_JPRB ,&
&  0.11138E+01_JPRB, 0.11183E+01_JPRB, 0.11684E+01_JPRB, 0.13137E+01_JPRB, 0.16472E+01_JPRB, 0.18346E+01_JPRB, 0.19374E+01_JPRB ,&
&  0.20421E+01_JPRB, 0.21135E+01_JPRB, 0.21414E+01_JPRB, 0.21827E+01_JPRB, 0.21723E+01_JPRB, 0.21571E+01_JPRB, 0.21824E+01_JPRB ,&
&  0.21715E+01_JPRB, 0.22048E+01_JPRB, 0.22120E+01_JPRB, 0.22332E+01_JPRB, 0.21860E+01_JPRB, 0.21589E+01_JPRB, 0.21365E+01_JPRB ,&
&  0.20970E+01_JPRB, 0.20245E+01_JPRB, 0.18907E+01_JPRB, 0.17568E+01_JPRB, 0.16355E+01_JPRB, 0.15642E+01_JPRB, 0.15065E+01_JPRB ,&
&  0.14452E+01_JPRB, 0.14008E+01_JPRB, 0.13551E+01_JPRB, 0.12973E+01_JPRB, 0.12211E+01_JPRB, 0.11552E+01_JPRB, 0.10980E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.98110E+00_JPRB, 0.97954E+00_JPRB, 0.98484E+00_JPRB, 0.10010E+01_JPRB, 0.10309E+01_JPRB, 0.10251E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10812E+01_JPRB, 0.10887E+01_JPRB, 0.11174E+01_JPRB, 0.11624E+01_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.13195E+01_JPRB, 0.13144E+01_JPRB, 0.12891E+01_JPRB, 0.12649E+01_JPRB, 0.12273E+01_JPRB, 0.11470E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.10409E+01_JPRB, 0.10190E+01_JPRB, 0.95696E+00_JPRB, 0.92468E+00_JPRB, 0.90253E+00_JPRB, 0.88212E+00_JPRB ,&
&  0.88135E+00_JPRB, 0.89830E+00_JPRB, 0.91612E+00_JPRB, 0.93361E+00_JPRB, 0.94586E+00_JPRB, 0.94642E+00_JPRB, 0.94575E+00_JPRB ,&
&  0.94326E+00_JPRB, 0.94701E+00_JPRB, 0.95834E+00_JPRB, 0.96418E+00_JPRB, 0.96831E+00_JPRB, 0.96547E+00_JPRB, 0.96196E+00_JPRB ,&
&  0.94942E+00_JPRB, 0.94789E+00_JPRB, 0.96523E+00_JPRB, 0.95591E+00_JPRB, 0.94928E+00_JPRB, 0.94131E+00_JPRB, 0.92600E+00_JPRB ,&
&  0.88890E+00_JPRB, 0.87692E+00_JPRB, 0.88461E+00_JPRB, 0.89628E+00_JPRB, 0.90917E+00_JPRB, 0.91829E+00_JPRB, 0.92610E+00_JPRB ,&
&  0.93739E+00_JPRB, 0.95345E+00_JPRB, 0.97514E+00_JPRB, 0.10022E+01_JPRB, 0.10386E+01_JPRB, 0.11082E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.13437E+01_JPRB, 0.13193E+01_JPRB, 0.15797E+01_JPRB, 0.18003E+01_JPRB, 0.21053E+01_JPRB, 0.23505E+01_JPRB, 0.24472E+01_JPRB ,&
&  0.26426E+01_JPRB, 0.26897E+01_JPRB, 0.27291E+01_JPRB, 0.27147E+01_JPRB, 0.26831E+01_JPRB, 0.26763E+01_JPRB, 0.27205E+01_JPRB ,&
&  0.27730E+01_JPRB, 0.27024E+01_JPRB, 0.28581E+01_JPRB, 0.27957E+01_JPRB, 0.27144E+01_JPRB, 0.26769E+01_JPRB, 0.26336E+01_JPRB ,&
&  0.25592E+01_JPRB, 0.24406E+01_JPRB, 0.22820E+01_JPRB, 0.20152E+01_JPRB, 0.19136E+01_JPRB, 0.18198E+01_JPRB, 0.16567E+01_JPRB ,&
&  0.15598E+01_JPRB, 0.14541E+01_JPRB, 0.13523E+01_JPRB, 0.13317E+01_JPRB, 0.12238E+01_JPRB, 0.11472E+01_JPRB, 0.11007E+01_JPRB ,&
&  0.10413E+01_JPRB, 0.10287E+01_JPRB, 0.10758E+01_JPRB, 0.11182E+01_JPRB, 0.11399E+01_JPRB, 0.11374E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11839E+01_JPRB, 0.12530E+01_JPRB, 0.12789E+01_JPRB, 0.13010E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.14771E+01_JPRB, 0.14582E+01_JPRB, 0.14247E+01_JPRB, 0.13833E+01_JPRB, 0.13044E+01_JPRB, 0.12056E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.11128E+01_JPRB, 0.10933E+01_JPRB, 0.10626E+01_JPRB, 0.10274E+01_JPRB, 0.97684E+00_JPRB, 0.90935E+00_JPRB ,&
&  0.90230E+00_JPRB, 0.93994E+00_JPRB, 0.99064E+00_JPRB, 0.10165E+01_JPRB, 0.10085E+01_JPRB, 0.10021E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10302E+01_JPRB, 0.10431E+01_JPRB, 0.10523E+01_JPRB, 0.10448E+01_JPRB, 0.10418E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.10149E+01_JPRB, 0.10256E+01_JPRB, 0.99918E+00_JPRB, 0.10498E+01_JPRB, 0.10616E+01_JPRB, 0.10587E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10059E+01_JPRB, 0.98467E+00_JPRB, 0.98485E+00_JPRB, 0.98754E+00_JPRB, 0.98689E+00_JPRB, 0.99383E+00_JPRB ,&
&  0.10115E+01_JPRB, 0.10388E+01_JPRB, 0.10786E+01_JPRB, 0.11410E+01_JPRB, 0.12086E+01_JPRB, 0.13157E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.16707E+01_JPRB, 0.19498E+01_JPRB, 0.22826E+01_JPRB, 0.23568E+01_JPRB, 0.26181E+01_JPRB, 0.28440E+01_JPRB, 0.29053E+01_JPRB ,&
&  0.31039E+01_JPRB, 0.31580E+01_JPRB, 0.33354E+01_JPRB, 0.33952E+01_JPRB, 0.34418E+01_JPRB, 0.34716E+01_JPRB, 0.34849E+01_JPRB ,&
&  0.34874E+01_JPRB, 0.34952E+01_JPRB, 0.35455E+01_JPRB, 0.34967E+01_JPRB, 0.33557E+01_JPRB, 0.32074E+01_JPRB, 0.30392E+01_JPRB ,&
&  0.28955E+01_JPRB, 0.27378E+01_JPRB, 0.25804E+01_JPRB, 0.24471E+01_JPRB, 0.22876E+01_JPRB, 0.20836E+01_JPRB, 0.18690E+01_JPRB ,&
&  0.16543E+01_JPRB, 0.14029E+01_JPRB, 0.12782E+01_JPRB, 0.12506E+01_JPRB, 0.12163E+01_JPRB, 0.11184E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.11357E+01_JPRB, 0.12331E+01_JPRB, 0.12825E+01_JPRB, 0.13626E+01_JPRB, 0.14039E+01_JPRB, 0.13038E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.13390E+01_JPRB, 0.14004E+01_JPRB, 0.14396E+01_JPRB, 0.14696E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.16185E+01_JPRB, 0.15890E+01_JPRB, 0.15564E+01_JPRB, 0.15140E+01_JPRB, 0.14266E+01_JPRB, 0.13109E+01_JPRB, 0.12276E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.11596E+01_JPRB, 0.11310E+01_JPRB, 0.11065E+01_JPRB, 0.10811E+01_JPRB, 0.10565E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.10998E+01_JPRB, 0.10861E+01_JPRB, 0.11119E+01_JPRB, 0.11343E+01_JPRB, 0.11069E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.11592E+01_JPRB, 0.11719E+01_JPRB, 0.11626E+01_JPRB, 0.11410E+01_JPRB, 0.11297E+01_JPRB, 0.11468E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.11582E+01_JPRB, 0.11359E+01_JPRB, 0.11055E+01_JPRB, 0.10972E+01_JPRB, 0.11409E+01_JPRB, 0.12007E+01_JPRB, 0.12087E+01_JPRB ,&
&  0.11891E+01_JPRB, 0.11683E+01_JPRB, 0.11437E+01_JPRB, 0.11254E+01_JPRB, 0.11063E+01_JPRB, 0.10896E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.11078E+01_JPRB, 0.11454E+01_JPRB, 0.12178E+01_JPRB, 0.13082E+01_JPRB, 0.15065E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.21562E+01_JPRB, 0.24665E+01_JPRB, 0.27898E+01_JPRB, 0.31349E+01_JPRB, 0.33836E+01_JPRB, 0.34790E+01_JPRB, 0.35165E+01_JPRB ,&
&  0.36669E+01_JPRB, 0.36484E+01_JPRB, 0.39142E+01_JPRB, 0.39125E+01_JPRB, 0.39484E+01_JPRB, 0.41211E+01_JPRB, 0.41093E+01_JPRB ,&
&  0.40183E+01_JPRB, 0.38448E+01_JPRB, 0.39563E+01_JPRB, 0.39016E+01_JPRB, 0.37740E+01_JPRB, 0.36308E+01_JPRB, 0.34233E+01_JPRB ,&
&  0.32156E+01_JPRB, 0.30054E+01_JPRB, 0.26044E+01_JPRB, 0.23208E+01_JPRB, 0.19917E+01_JPRB, 0.19802E+01_JPRB, 0.17905E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.13183E+01_JPRB, 0.12845E+01_JPRB, 0.12617E+01_JPRB, 0.12761E+01_JPRB, 0.12289E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.12967E+01_JPRB, 0.13557E+01_JPRB, 0.14417E+01_JPRB, 0.15662E+01_JPRB, 0.16542E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.14035E+01_JPRB, 0.15448E+01_JPRB, 0.16006E+01_JPRB, 0.16297E+01_JPRB, 0.16354E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.16525E+01_JPRB, 0.16055E+01_JPRB, 0.15596E+01_JPRB, 0.15156E+01_JPRB, 0.14598E+01_JPRB, 0.13910E+01_JPRB, 0.13060E+01_JPRB ,&
&  0.12319E+01_JPRB, 0.11780E+01_JPRB, 0.11512E+01_JPRB, 0.11421E+01_JPRB, 0.11336E+01_JPRB, 0.11154E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.12405E+01_JPRB, 0.11483E+01_JPRB, 0.12075E+01_JPRB, 0.14049E+01_JPRB, 0.12799E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.12764E+01_JPRB, 0.13143E+01_JPRB, 0.12951E+01_JPRB, 0.12650E+01_JPRB, 0.12541E+01_JPRB, 0.12806E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.13869E+01_JPRB, 0.12808E+01_JPRB, 0.12133E+01_JPRB, 0.11862E+01_JPRB, 0.12081E+01_JPRB, 0.12541E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.12663E+01_JPRB, 0.12444E+01_JPRB, 0.12294E+01_JPRB, 0.12148E+01_JPRB, 0.11967E+01_JPRB, 0.11846E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.11844E+01_JPRB, 0.12052E+01_JPRB, 0.12425E+01_JPRB, 0.13219E+01_JPRB, 0.13892E+01_JPRB, 0.16782E+01_JPRB, 0.23082E+01_JPRB ,&
&  0.30843E+01_JPRB, 0.32512E+01_JPRB, 0.36240E+01_JPRB, 0.43140E+01_JPRB, 0.45287E+01_JPRB, 0.45718E+01_JPRB, 0.45736E+01_JPRB ,&
&  0.46028E+01_JPRB, 0.45605E+01_JPRB, 0.45457E+01_JPRB, 0.47262E+01_JPRB, 0.47144E+01_JPRB, 0.47411E+01_JPRB, 0.45882E+01_JPRB ,&
&  0.42936E+01_JPRB, 0.41838E+01_JPRB, 0.41525E+01_JPRB, 0.39829E+01_JPRB, 0.37547E+01_JPRB, 0.36919E+01_JPRB, 0.35825E+01_JPRB ,&
&  0.33629E+01_JPRB, 0.29930E+01_JPRB, 0.22715E+01_JPRB, 0.20531E+01_JPRB, 0.15711E+01_JPRB, 0.15741E+01_JPRB, 0.18503E+01_JPRB ,&
&  0.17670E+01_JPRB, 0.15981E+01_JPRB, 0.14918E+01_JPRB, 0.14480E+01_JPRB, 0.14456E+01_JPRB, 0.15154E+01_JPRB, 0.14803E+01_JPRB ,&
&  0.13878E+01_JPRB, 0.14564E+01_JPRB, 0.15693E+01_JPRB, 0.17196E+01_JPRB, 0.18798E+01_JPRB, 0.19206E+01_JPRB, 0.17079E+01_JPRB ,&
&  0.17703E+01_JPRB, 0.17925E+01_JPRB, 0.17881E+01_JPRB, 0.17717E+01_JPRB, 0.17196E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.15607E+01_JPRB, 0.15163E+01_JPRB, 0.14653E+01_JPRB, 0.14480E+01_JPRB, 0.14514E+01_JPRB, 0.14308E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.12685E+01_JPRB, 0.11995E+01_JPRB, 0.11678E+01_JPRB, 0.11721E+01_JPRB, 0.11874E+01_JPRB, 0.11838E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.14651E+01_JPRB, 0.13282E+01_JPRB, 0.12006E+01_JPRB, 0.13467E+01_JPRB, 0.13057E+01_JPRB, 0.13492E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.14411E+01_JPRB, 0.14535E+01_JPRB, 0.14873E+01_JPRB, 0.15365E+01_JPRB, 0.16460E+01_JPRB, 0.18498E+01_JPRB ,&
&  0.18821E+01_JPRB, 0.16755E+01_JPRB, 0.15225E+01_JPRB, 0.14773E+01_JPRB, 0.15063E+01_JPRB, 0.13933E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.13311E+01_JPRB, 0.13171E+01_JPRB, 0.13182E+01_JPRB, 0.13194E+01_JPRB, 0.13204E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.13717E+01_JPRB, 0.13782E+01_JPRB, 0.14222E+01_JPRB, 0.15776E+01_JPRB, 0.19953E+01_JPRB, 0.26659E+01_JPRB ,&
&  0.31797E+01_JPRB, 0.33695E+01_JPRB, 0.36745E+01_JPRB, 0.43286E+01_JPRB, 0.47545E+01_JPRB, 0.50134E+01_JPRB, 0.53239E+01_JPRB ,&
&  0.54193E+01_JPRB, 0.54129E+01_JPRB, 0.53818E+01_JPRB, 0.50493E+01_JPRB, 0.45342E+01_JPRB, 0.41366E+01_JPRB, 0.39287E+01_JPRB ,&
&  0.37031E+01_JPRB, 0.36348E+01_JPRB, 0.37254E+01_JPRB, 0.36187E+01_JPRB, 0.35620E+01_JPRB, 0.33470E+01_JPRB, 0.34693E+01_JPRB ,&
&  0.33723E+01_JPRB, 0.20875E+01_JPRB, 0.16578E+01_JPRB, 0.18211E+01_JPRB, 0.14987E+01_JPRB, 0.15243E+01_JPRB, 0.17244E+01_JPRB ,&
&  0.16733E+01_JPRB, 0.16465E+01_JPRB, 0.17104E+01_JPRB, 0.17559E+01_JPRB, 0.17719E+01_JPRB, 0.19375E+01_JPRB, 0.19907E+01_JPRB ,&
&  0.19272E+01_JPRB, 0.18209E+01_JPRB, 0.19571E+01_JPRB, 0.21294E+01_JPRB, 0.22645E+01_JPRB, 0.22199E+01_JPRB, 0.20980E+01_JPRB ,&
&  0.20214E+01_JPRB, 0.19602E+01_JPRB, 0.18896E+01_JPRB, 0.17707E+01_JPRB, 0.16404E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.15326E+01_JPRB, 0.15069E+01_JPRB, 0.14528E+01_JPRB, 0.14606E+01_JPRB, 0.14940E+01_JPRB, 0.14811E+01_JPRB, 0.14054E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.12057E+01_JPRB, 0.11746E+01_JPRB, 0.11794E+01_JPRB, 0.12036E+01_JPRB, 0.12214E+01_JPRB, 0.12151E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.13930E+01_JPRB, 0.12605E+01_JPRB, 0.12068E+01_JPRB, 0.10990E+01_JPRB, 0.12062E+01_JPRB, 0.13555E+01_JPRB ,&
&  0.14625E+01_JPRB, 0.15335E+01_JPRB, 0.17015E+01_JPRB, 0.19348E+01_JPRB, 0.21689E+01_JPRB, 0.23726E+01_JPRB, 0.27162E+01_JPRB ,&
&  0.26641E+01_JPRB, 0.24173E+01_JPRB, 0.19972E+01_JPRB, 0.18975E+01_JPRB, 0.18002E+01_JPRB, 0.16028E+01_JPRB, 0.16200E+01_JPRB ,&
&  0.15612E+01_JPRB, 0.15365E+01_JPRB, 0.15163E+01_JPRB, 0.15052E+01_JPRB, 0.14892E+01_JPRB, 0.14496E+01_JPRB, 0.14347E+01_JPRB ,&
&  0.14413E+01_JPRB, 0.14622E+01_JPRB, 0.14859E+01_JPRB, 0.15568E+01_JPRB, 0.19378E+01_JPRB, 0.24325E+01_JPRB, 0.28213E+01_JPRB ,&
&  0.32773E+01_JPRB, 0.29769E+01_JPRB, 0.27963E+01_JPRB, 0.38065E+01_JPRB, 0.55996E+01_JPRB, 0.61721E+01_JPRB, 0.59188E+01_JPRB ,&
&  0.64517E+01_JPRB, 0.60235E+01_JPRB, 0.52353E+01_JPRB, 0.45788E+01_JPRB, 0.41587E+01_JPRB, 0.37815E+01_JPRB, 0.36009E+01_JPRB ,&
&  0.33284E+01_JPRB, 0.32607E+01_JPRB, 0.34084E+01_JPRB, 0.35051E+01_JPRB, 0.33599E+01_JPRB, 0.31674E+01_JPRB, 0.31708E+01_JPRB ,&
&  0.30286E+01_JPRB, 0.25614E+01_JPRB, 0.18253E+01_JPRB, 0.17671E+01_JPRB, 0.14333E+01_JPRB, 0.14262E+01_JPRB, 0.16643E+01_JPRB ,&
&  0.16554E+01_JPRB, 0.17873E+01_JPRB, 0.18829E+01_JPRB, 0.18958E+01_JPRB, 0.21890E+01_JPRB, 0.23855E+01_JPRB, 0.26439E+01_JPRB ,&
&  0.27655E+01_JPRB, 0.24003E+01_JPRB, 0.25540E+01_JPRB, 0.26866E+01_JPRB, 0.26657E+01_JPRB, 0.24537E+01_JPRB, 0.22413E+01_JPRB ,&
&  0.21081E+01_JPRB, 0.20022E+01_JPRB, 0.18707E+01_JPRB, 0.17014E+01_JPRB, 0.15848E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.15435E+01_JPRB, 0.15548E+01_JPRB, 0.15063E+01_JPRB, 0.15125E+01_JPRB, 0.15157E+01_JPRB, 0.14737E+01_JPRB, 0.14037E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.12770E+01_JPRB, 0.12397E+01_JPRB, 0.12236E+01_JPRB, 0.12298E+01_JPRB, 0.12267E+01_JPRB, 0.11940E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.12375E+01_JPRB, 0.11140E+01_JPRB, 0.10255E+01_JPRB, 0.97545E+00_JPRB, 0.10563E+01_JPRB, 0.12164E+01_JPRB ,&
&  0.14726E+01_JPRB, 0.17992E+01_JPRB, 0.22845E+01_JPRB, 0.26877E+01_JPRB, 0.32456E+01_JPRB, 0.37037E+01_JPRB, 0.39618E+01_JPRB ,&
&  0.40217E+01_JPRB, 0.30768E+01_JPRB, 0.27636E+01_JPRB, 0.23949E+01_JPRB, 0.21859E+01_JPRB, 0.19903E+01_JPRB, 0.18683E+01_JPRB ,&
&  0.17560E+01_JPRB, 0.16903E+01_JPRB, 0.16372E+01_JPRB, 0.15821E+01_JPRB, 0.15522E+01_JPRB, 0.15386E+01_JPRB, 0.15620E+01_JPRB ,&
&  0.15759E+01_JPRB, 0.15588E+01_JPRB, 0.16188E+01_JPRB, 0.19112E+01_JPRB, 0.25127E+01_JPRB, 0.30088E+01_JPRB, 0.32761E+01_JPRB ,&
&  0.30934E+01_JPRB, 0.30268E+01_JPRB, 0.48379E+01_JPRB, 0.54787E+01_JPRB, 0.54772E+01_JPRB, 0.65138E+01_JPRB, 0.69571E+01_JPRB ,&
&  0.64980E+01_JPRB, 0.52038E+01_JPRB, 0.43361E+01_JPRB, 0.37311E+01_JPRB, 0.41596E+01_JPRB, 0.37803E+01_JPRB, 0.34134E+01_JPRB ,&
&  0.31174E+01_JPRB, 0.32806E+01_JPRB, 0.32479E+01_JPRB, 0.32647E+01_JPRB, 0.32717E+01_JPRB, 0.32574E+01_JPRB, 0.28326E+01_JPRB ,&
&  0.25324E+01_JPRB, 0.29942E+01_JPRB, 0.24370E+01_JPRB, 0.19061E+01_JPRB, 0.16293E+01_JPRB, 0.16755E+01_JPRB, 0.18159E+01_JPRB ,&
&  0.19276E+01_JPRB, 0.20540E+01_JPRB, 0.21095E+01_JPRB, 0.22988E+01_JPRB, 0.27852E+01_JPRB, 0.30496E+01_JPRB, 0.31751E+01_JPRB ,&
&  0.31068E+01_JPRB, 0.33329E+01_JPRB, 0.32683E+01_JPRB, 0.30994E+01_JPRB, 0.27541E+01_JPRB, 0.23702E+01_JPRB, 0.21421E+01_JPRB ,&
&  0.20312E+01_JPRB, 0.19329E+01_JPRB, 0.17646E+01_JPRB, 0.16056E+01_JPRB, 0.15239E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.15395E+01_JPRB, 0.15676E+01_JPRB, 0.15658E+01_JPRB, 0.15558E+01_JPRB, 0.15322E+01_JPRB, 0.14870E+01_JPRB, 0.14443E+01_JPRB ,&
&  0.13820E+01_JPRB, 0.13072E+01_JPRB, 0.12818E+01_JPRB, 0.12742E+01_JPRB, 0.12701E+01_JPRB, 0.12476E+01_JPRB, 0.12072E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11340E+01_JPRB, 0.10018E+01_JPRB, 0.10163E+01_JPRB, 0.10504E+01_JPRB, 0.10690E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.15244E+01_JPRB, 0.21227E+01_JPRB, 0.27838E+01_JPRB, 0.36263E+01_JPRB, 0.43337E+01_JPRB, 0.45756E+01_JPRB, 0.45297E+01_JPRB ,&
&  0.40322E+01_JPRB, 0.36429E+01_JPRB, 0.31762E+01_JPRB, 0.28077E+01_JPRB, 0.25101E+01_JPRB, 0.23004E+01_JPRB, 0.20877E+01_JPRB ,&
&  0.19065E+01_JPRB, 0.17714E+01_JPRB, 0.16905E+01_JPRB, 0.16071E+01_JPRB, 0.15819E+01_JPRB, 0.15936E+01_JPRB, 0.16150E+01_JPRB ,&
&  0.15931E+01_JPRB, 0.16077E+01_JPRB, 0.18063E+01_JPRB, 0.22497E+01_JPRB, 0.24884E+01_JPRB, 0.28037E+01_JPRB, 0.30547E+01_JPRB ,&
&  0.41023E+01_JPRB, 0.44876E+01_JPRB, 0.45603E+01_JPRB, 0.59953E+01_JPRB, 0.43782E+01_JPRB, 0.53695E+01_JPRB, 0.64947E+01_JPRB ,&
&  0.59105E+01_JPRB, 0.51986E+01_JPRB, 0.50350E+01_JPRB, 0.40610E+01_JPRB, 0.33236E+01_JPRB, 0.43241E+01_JPRB, 0.36845E+01_JPRB ,&
&  0.32672E+01_JPRB, 0.33039E+01_JPRB, 0.32754E+01_JPRB, 0.35473E+01_JPRB, 0.28562E+01_JPRB, 0.25184E+01_JPRB, 0.20597E+01_JPRB ,&
&  0.21625E+01_JPRB, 0.20210E+01_JPRB, 0.23174E+01_JPRB, 0.20839E+01_JPRB, 0.21423E+01_JPRB, 0.20872E+01_JPRB, 0.20750E+01_JPRB ,&
&  0.22578E+01_JPRB, 0.23367E+01_JPRB, 0.23788E+01_JPRB, 0.28620E+01_JPRB, 0.36991E+01_JPRB, 0.33298E+01_JPRB, 0.36373E+01_JPRB ,&
&  0.37367E+01_JPRB, 0.36154E+01_JPRB, 0.32459E+01_JPRB, 0.29406E+01_JPRB, 0.24772E+01_JPRB, 0.21834E+01_JPRB, 0.20480E+01_JPRB ,&
&  0.19461E+01_JPRB, 0.17790E+01_JPRB, 0.16056E+01_JPRB, 0.15352E+01_JPRB, 0.14859E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.15086E+01_JPRB, 0.15048E+01_JPRB, 0.15118E+01_JPRB, 0.15056E+01_JPRB, 0.15136E+01_JPRB, 0.14821E+01_JPRB, 0.14404E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.13010E+01_JPRB, 0.12846E+01_JPRB, 0.12732E+01_JPRB, 0.12649E+01_JPRB, 0.12505E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.11663E+01_JPRB, 0.10250E+01_JPRB, 0.99494E+00_JPRB, 0.10796E+01_JPRB, 0.11021E+01_JPRB, 0.12318E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.25268E+01_JPRB, 0.31954E+01_JPRB, 0.40301E+01_JPRB, 0.45409E+01_JPRB, 0.44009E+01_JPRB, 0.40903E+01_JPRB ,&
&  0.40457E+01_JPRB, 0.33525E+01_JPRB, 0.29709E+01_JPRB, 0.27429E+01_JPRB, 0.24871E+01_JPRB, 0.22617E+01_JPRB, 0.20734E+01_JPRB ,&
&  0.18884E+01_JPRB, 0.17179E+01_JPRB, 0.16103E+01_JPRB, 0.15151E+01_JPRB, 0.14949E+01_JPRB, 0.15206E+01_JPRB, 0.15596E+01_JPRB ,&
&  0.16026E+01_JPRB, 0.17250E+01_JPRB, 0.20579E+01_JPRB, 0.24624E+01_JPRB, 0.27479E+01_JPRB, 0.30013E+01_JPRB, 0.36597E+01_JPRB ,&
&  0.40276E+01_JPRB, 0.40634E+01_JPRB, 0.47283E+01_JPRB, 0.55764E+01_JPRB, 0.60772E+01_JPRB, 0.55193E+01_JPRB, 0.59916E+01_JPRB ,&
&  0.43550E+01_JPRB, 0.35541E+01_JPRB, 0.31502E+01_JPRB, 0.28227E+01_JPRB, 0.32003E+01_JPRB, 0.47524E+01_JPRB, 0.43221E+01_JPRB ,&
&  0.39732E+01_JPRB, 0.36460E+01_JPRB, 0.36388E+01_JPRB, 0.33294E+01_JPRB, 0.29759E+01_JPRB, 0.21922E+01_JPRB, 0.28139E+01_JPRB ,&
&  0.30454E+01_JPRB, 0.31308E+01_JPRB, 0.28316E+01_JPRB, 0.23951E+01_JPRB, 0.21034E+01_JPRB, 0.25088E+01_JPRB, 0.26976E+01_JPRB ,&
&  0.28193E+01_JPRB, 0.28004E+01_JPRB, 0.37003E+01_JPRB, 0.42301E+01_JPRB, 0.43203E+01_JPRB, 0.39314E+01_JPRB, 0.42032E+01_JPRB ,&
&  0.37764E+01_JPRB, 0.35950E+01_JPRB, 0.32209E+01_JPRB, 0.26659E+01_JPRB, 0.22716E+01_JPRB, 0.20760E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.17859E+01_JPRB, 0.16012E+01_JPRB, 0.15019E+01_JPRB, 0.14464E+01_JPRB, 0.14403E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.14512E+01_JPRB, 0.14337E+01_JPRB, 0.14085E+01_JPRB, 0.14144E+01_JPRB, 0.14204E+01_JPRB, 0.13883E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.12669E+01_JPRB, 0.12313E+01_JPRB, 0.12111E+01_JPRB, 0.11970E+01_JPRB, 0.12057E+01_JPRB, 0.11972E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.12808E+01_JPRB, 0.11154E+01_JPRB, 0.10764E+01_JPRB, 0.11144E+01_JPRB, 0.11615E+01_JPRB, 0.12891E+01_JPRB ,&
&  0.19173E+01_JPRB, 0.26251E+01_JPRB, 0.30923E+01_JPRB, 0.37330E+01_JPRB, 0.42982E+01_JPRB, 0.40490E+01_JPRB, 0.38862E+01_JPRB ,&
&  0.37406E+01_JPRB, 0.32054E+01_JPRB, 0.28261E+01_JPRB, 0.25391E+01_JPRB, 0.23104E+01_JPRB, 0.21436E+01_JPRB, 0.19695E+01_JPRB ,&
&  0.17747E+01_JPRB, 0.16283E+01_JPRB, 0.15194E+01_JPRB, 0.14412E+01_JPRB, 0.13855E+01_JPRB, 0.14651E+01_JPRB, 0.15828E+01_JPRB ,&
&  0.17244E+01_JPRB, 0.19909E+01_JPRB, 0.22898E+01_JPRB, 0.26964E+01_JPRB, 0.31258E+01_JPRB, 0.32107E+01_JPRB, 0.42186E+01_JPRB ,&
&  0.41313E+01_JPRB, 0.42154E+01_JPRB, 0.46882E+01_JPRB, 0.62195E+01_JPRB, 0.55838E+01_JPRB, 0.51087E+01_JPRB, 0.52685E+01_JPRB ,&
&  0.37089E+01_JPRB, 0.31962E+01_JPRB, 0.33853E+01_JPRB, 0.35772E+01_JPRB, 0.32350E+01_JPRB, 0.35144E+01_JPRB, 0.49850E+01_JPRB ,&
&  0.40730E+01_JPRB, 0.38408E+01_JPRB, 0.38651E+01_JPRB, 0.41242E+01_JPRB, 0.21254E+01_JPRB, 0.16538E+01_JPRB, 0.25864E+01_JPRB ,&
&  0.27952E+01_JPRB, 0.18634E+01_JPRB, 0.14231E+01_JPRB, 0.18239E+01_JPRB, 0.14697E+01_JPRB, 0.19068E+01_JPRB, 0.29982E+01_JPRB ,&
&  0.33592E+01_JPRB, 0.36899E+01_JPRB, 0.57908E+01_JPRB, 0.54450E+01_JPRB, 0.47945E+01_JPRB, 0.49389E+01_JPRB, 0.44247E+01_JPRB ,&
&  0.39389E+01_JPRB, 0.31360E+01_JPRB, 0.28529E+01_JPRB, 0.23783E+01_JPRB, 0.20841E+01_JPRB, 0.18766E+01_JPRB, 0.17242E+01_JPRB ,&
&  0.15671E+01_JPRB, 0.14728E+01_JPRB, 0.14601E+01_JPRB, 0.14523E+01_JPRB, 0.14351E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.14130E+01_JPRB, 0.13598E+01_JPRB, 0.13087E+01_JPRB, 0.13211E+01_JPRB, 0.13114E+01_JPRB, 0.12726E+01_JPRB, 0.12897E+01_JPRB ,&
&  0.12830E+01_JPRB, 0.12194E+01_JPRB, 0.11670E+01_JPRB, 0.11451E+01_JPRB, 0.11395E+01_JPRB, 0.11929E+01_JPRB, 0.12237E+01_JPRB ,&
&  0.12493E+01_JPRB, 0.12976E+01_JPRB, 0.13598E+01_JPRB, 0.13139E+01_JPRB, 0.12867E+01_JPRB, 0.13047E+01_JPRB, 0.15368E+01_JPRB ,&
&  0.21803E+01_JPRB, 0.26504E+01_JPRB, 0.29302E+01_JPRB, 0.32162E+01_JPRB, 0.35175E+01_JPRB, 0.34992E+01_JPRB, 0.35999E+01_JPRB ,&
&  0.32809E+01_JPRB, 0.28531E+01_JPRB, 0.25194E+01_JPRB, 0.22380E+01_JPRB, 0.20913E+01_JPRB, 0.19564E+01_JPRB, 0.18104E+01_JPRB ,&
&  0.16215E+01_JPRB, 0.14885E+01_JPRB, 0.14287E+01_JPRB, 0.13692E+01_JPRB, 0.13773E+01_JPRB, 0.14802E+01_JPRB, 0.16356E+01_JPRB ,&
&  0.18925E+01_JPRB, 0.22382E+01_JPRB, 0.25032E+01_JPRB, 0.29660E+01_JPRB, 0.34191E+01_JPRB, 0.34629E+01_JPRB, 0.33105E+01_JPRB ,&
&  0.32454E+01_JPRB, 0.33020E+01_JPRB, 0.45447E+01_JPRB, 0.53733E+01_JPRB, 0.53160E+01_JPRB, 0.52651E+01_JPRB, 0.49795E+01_JPRB ,&
&  0.42831E+01_JPRB, 0.39604E+01_JPRB, 0.35335E+01_JPRB, 0.38500E+01_JPRB, 0.36942E+01_JPRB, 0.31698E+01_JPRB, 0.38384E+01_JPRB ,&
&  0.40801E+01_JPRB, 0.37305E+01_JPRB, 0.30112E+01_JPRB, 0.24818E+01_JPRB, 0.22520E+01_JPRB, 0.19187E+01_JPRB, 0.14266E+01_JPRB ,&
&  0.13107E+01_JPRB, 0.12928E+01_JPRB, 0.12148E+01_JPRB, 0.12530E+01_JPRB, 0.12749E+01_JPRB, 0.19722E+01_JPRB, 0.30005E+01_JPRB ,&
&  0.45199E+01_JPRB, 0.55316E+01_JPRB, 0.64772E+01_JPRB, 0.61220E+01_JPRB, 0.52113E+01_JPRB, 0.53834E+01_JPRB, 0.46089E+01_JPRB ,&
&  0.33334E+01_JPRB, 0.27982E+01_JPRB, 0.24729E+01_JPRB, 0.21322E+01_JPRB, 0.18863E+01_JPRB, 0.16695E+01_JPRB, 0.15069E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.14326E+01_JPRB, 0.14433E+01_JPRB, 0.14411E+01_JPRB, 0.13942E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.13775E+01_JPRB, 0.13249E+01_JPRB, 0.12726E+01_JPRB, 0.12369E+01_JPRB, 0.11906E+01_JPRB, 0.11834E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.11794E+01_JPRB, 0.11140E+01_JPRB, 0.10998E+01_JPRB, 0.11525E+01_JPRB, 0.12219E+01_JPRB, 0.12454E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.13263E+01_JPRB, 0.15199E+01_JPRB, 0.15369E+01_JPRB, 0.16324E+01_JPRB, 0.16324E+01_JPRB, 0.20007E+01_JPRB ,&
&  0.24201E+01_JPRB, 0.26518E+01_JPRB, 0.29024E+01_JPRB, 0.30008E+01_JPRB, 0.32746E+01_JPRB, 0.33412E+01_JPRB, 0.32577E+01_JPRB ,&
&  0.28431E+01_JPRB, 0.24951E+01_JPRB, 0.22090E+01_JPRB, 0.20022E+01_JPRB, 0.18416E+01_JPRB, 0.16577E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.14118E+01_JPRB, 0.13858E+01_JPRB, 0.14077E+01_JPRB, 0.15040E+01_JPRB, 0.16291E+01_JPRB, 0.18225E+01_JPRB ,&
&  0.20917E+01_JPRB, 0.24242E+01_JPRB, 0.27163E+01_JPRB, 0.31889E+01_JPRB, 0.29279E+01_JPRB, 0.26172E+01_JPRB, 0.30546E+01_JPRB ,&
&  0.34888E+01_JPRB, 0.38952E+01_JPRB, 0.40264E+01_JPRB, 0.52882E+01_JPRB, 0.53532E+01_JPRB, 0.48698E+01_JPRB, 0.46585E+01_JPRB ,&
&  0.40594E+01_JPRB, 0.40147E+01_JPRB, 0.34814E+01_JPRB, 0.36563E+01_JPRB, 0.42237E+01_JPRB, 0.36450E+01_JPRB, 0.36424E+01_JPRB ,&
&  0.40007E+01_JPRB, 0.36490E+01_JPRB, 0.31235E+01_JPRB, 0.25893E+01_JPRB, 0.31711E+01_JPRB, 0.34218E+01_JPRB, 0.16487E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.12858E+01_JPRB, 0.12603E+01_JPRB, 0.11939E+01_JPRB, 0.12804E+01_JPRB, 0.20463E+01_JPRB, 0.44862E+01_JPRB ,&
&  0.48014E+01_JPRB, 0.72850E+01_JPRB, 0.72331E+01_JPRB, 0.67565E+01_JPRB, 0.55350E+01_JPRB, 0.56649E+01_JPRB, 0.41185E+01_JPRB ,&
&  0.33320E+01_JPRB, 0.28180E+01_JPRB, 0.23543E+01_JPRB, 0.20709E+01_JPRB, 0.17914E+01_JPRB, 0.15489E+01_JPRB, 0.14510E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.13741E+01_JPRB, 0.13616E+01_JPRB, 0.13563E+01_JPRB, 0.13545E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.13625E+01_JPRB, 0.13031E+01_JPRB, 0.12495E+01_JPRB, 0.12126E+01_JPRB, 0.11682E+01_JPRB, 0.12081E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.11669E+01_JPRB, 0.11156E+01_JPRB, 0.11295E+01_JPRB, 0.11994E+01_JPRB, 0.12770E+01_JPRB, 0.13267E+01_JPRB ,&
&  0.13864E+01_JPRB, 0.14685E+01_JPRB, 0.16348E+01_JPRB, 0.18840E+01_JPRB, 0.20478E+01_JPRB, 0.20976E+01_JPRB, 0.24529E+01_JPRB ,&
&  0.25099E+01_JPRB, 0.25286E+01_JPRB, 0.28556E+01_JPRB, 0.29559E+01_JPRB, 0.28686E+01_JPRB, 0.28380E+01_JPRB, 0.25751E+01_JPRB ,&
&  0.23577E+01_JPRB, 0.21598E+01_JPRB, 0.19378E+01_JPRB, 0.17608E+01_JPRB, 0.16517E+01_JPRB, 0.15280E+01_JPRB, 0.14799E+01_JPRB ,&
&  0.14292E+01_JPRB, 0.14139E+01_JPRB, 0.14652E+01_JPRB, 0.15231E+01_JPRB, 0.16586E+01_JPRB, 0.17865E+01_JPRB, 0.19929E+01_JPRB ,&
&  0.22717E+01_JPRB, 0.25474E+01_JPRB, 0.28203E+01_JPRB, 0.32002E+01_JPRB, 0.27058E+01_JPRB, 0.32072E+01_JPRB, 0.33968E+01_JPRB ,&
&  0.36414E+01_JPRB, 0.37424E+01_JPRB, 0.36483E+01_JPRB, 0.44827E+01_JPRB, 0.49379E+01_JPRB, 0.45421E+01_JPRB, 0.40585E+01_JPRB ,&
&  0.38196E+01_JPRB, 0.34621E+01_JPRB, 0.34828E+01_JPRB, 0.38160E+01_JPRB, 0.41972E+01_JPRB, 0.50303E+01_JPRB, 0.36394E+01_JPRB ,&
&  0.32568E+01_JPRB, 0.35580E+01_JPRB, 0.33482E+01_JPRB, 0.29952E+01_JPRB, 0.35401E+01_JPRB, 0.50775E+01_JPRB, 0.37378E+01_JPRB ,&
&  0.15279E+01_JPRB, 0.12168E+01_JPRB, 0.13596E+01_JPRB, 0.13848E+01_JPRB, 0.12480E+01_JPRB, 0.19848E+01_JPRB, 0.78817E+01_JPRB ,&
&  0.62201E+01_JPRB, 0.86576E+01_JPRB, 0.74555E+01_JPRB, 0.65213E+01_JPRB, 0.56942E+01_JPRB, 0.54795E+01_JPRB, 0.44292E+01_JPRB ,&
&  0.33258E+01_JPRB, 0.27492E+01_JPRB, 0.23026E+01_JPRB, 0.19661E+01_JPRB, 0.16681E+01_JPRB, 0.14903E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.13616E+01_JPRB, 0.13436E+01_JPRB, 0.13357E+01_JPRB, 0.13270E+01_JPRB, 0.13452E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.13534E+01_JPRB, 0.12518E+01_JPRB, 0.11968E+01_JPRB, 0.11621E+01_JPRB, 0.12058E+01_JPRB, 0.12582E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.12391E+01_JPRB, 0.12358E+01_JPRB, 0.12319E+01_JPRB, 0.12912E+01_JPRB, 0.14089E+01_JPRB, 0.15025E+01_JPRB ,&
&  0.16220E+01_JPRB, 0.17588E+01_JPRB, 0.19482E+01_JPRB, 0.22316E+01_JPRB, 0.24985E+01_JPRB, 0.24462E+01_JPRB, 0.24395E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.24456E+01_JPRB, 0.24243E+01_JPRB, 0.26411E+01_JPRB, 0.26589E+01_JPRB, 0.27302E+01_JPRB, 0.23828E+01_JPRB ,&
&  0.22240E+01_JPRB, 0.19984E+01_JPRB, 0.17422E+01_JPRB, 0.15948E+01_JPRB, 0.14651E+01_JPRB, 0.14521E+01_JPRB, 0.15107E+01_JPRB ,&
&  0.14860E+01_JPRB, 0.15184E+01_JPRB, 0.16006E+01_JPRB, 0.16948E+01_JPRB, 0.18425E+01_JPRB, 0.19901E+01_JPRB, 0.21879E+01_JPRB ,&
&  0.24294E+01_JPRB, 0.26497E+01_JPRB, 0.28796E+01_JPRB, 0.29810E+01_JPRB, 0.30370E+01_JPRB, 0.31641E+01_JPRB, 0.34714E+01_JPRB ,&
&  0.36132E+01_JPRB, 0.37063E+01_JPRB, 0.37816E+01_JPRB, 0.40107E+01_JPRB, 0.42295E+01_JPRB, 0.41145E+01_JPRB, 0.37017E+01_JPRB ,&
&  0.35993E+01_JPRB, 0.36223E+01_JPRB, 0.36497E+01_JPRB, 0.39644E+01_JPRB, 0.43017E+01_JPRB, 0.50119E+01_JPRB, 0.46605E+01_JPRB ,&
&  0.38640E+01_JPRB, 0.32690E+01_JPRB, 0.33125E+01_JPRB, 0.38873E+01_JPRB, 0.43292E+01_JPRB, 0.49582E+01_JPRB, 0.57848E+01_JPRB ,&
&  0.56949E+01_JPRB, 0.30841E+01_JPRB, 0.23625E+01_JPRB, 0.35251E+01_JPRB, 0.24589E+01_JPRB, 0.22493E+01_JPRB, 0.42376E+01_JPRB ,&
&  0.64877E+01_JPRB, 0.79811E+01_JPRB, 0.72874E+01_JPRB, 0.60196E+01_JPRB, 0.55950E+01_JPRB, 0.49185E+01_JPRB, 0.38607E+01_JPRB ,&
&  0.29444E+01_JPRB, 0.23489E+01_JPRB, 0.19736E+01_JPRB, 0.17146E+01_JPRB, 0.14907E+01_JPRB, 0.14063E+01_JPRB, 0.13950E+01_JPRB ,&
&  0.13550E+01_JPRB, 0.13737E+01_JPRB, 0.13735E+01_JPRB, 0.13546E+01_JPRB, 0.13683E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.12825E+01_JPRB, 0.11963E+01_JPRB, 0.11461E+01_JPRB, 0.11433E+01_JPRB, 0.12277E+01_JPRB, 0.12913E+01_JPRB, 0.12949E+01_JPRB ,&
&  0.12981E+01_JPRB, 0.13074E+01_JPRB, 0.13468E+01_JPRB, 0.13449E+01_JPRB, 0.14518E+01_JPRB, 0.16606E+01_JPRB, 0.18254E+01_JPRB ,&
&  0.19648E+01_JPRB, 0.21102E+01_JPRB, 0.22955E+01_JPRB, 0.25954E+01_JPRB, 0.29981E+01_JPRB, 0.34949E+01_JPRB, 0.21845E+01_JPRB ,&
&  0.20239E+01_JPRB, 0.23807E+01_JPRB, 0.23554E+01_JPRB, 0.24796E+01_JPRB, 0.23478E+01_JPRB, 0.22615E+01_JPRB, 0.20485E+01_JPRB ,&
&  0.19712E+01_JPRB, 0.18130E+01_JPRB, 0.16906E+01_JPRB, 0.16569E+01_JPRB, 0.16192E+01_JPRB, 0.17064E+01_JPRB, 0.17458E+01_JPRB ,&
&  0.17352E+01_JPRB, 0.17919E+01_JPRB, 0.19236E+01_JPRB, 0.20500E+01_JPRB, 0.21733E+01_JPRB, 0.22860E+01_JPRB, 0.24227E+01_JPRB ,&
&  0.25877E+01_JPRB, 0.27487E+01_JPRB, 0.29631E+01_JPRB, 0.29079E+01_JPRB, 0.29396E+01_JPRB, 0.30349E+01_JPRB, 0.32544E+01_JPRB ,&
&  0.31756E+01_JPRB, 0.30119E+01_JPRB, 0.33406E+01_JPRB, 0.37053E+01_JPRB, 0.37681E+01_JPRB, 0.35226E+01_JPRB, 0.33775E+01_JPRB ,&
&  0.37129E+01_JPRB, 0.40341E+01_JPRB, 0.45230E+01_JPRB, 0.41559E+01_JPRB, 0.42755E+01_JPRB, 0.46108E+01_JPRB, 0.47008E+01_JPRB ,&
&  0.45472E+01_JPRB, 0.41516E+01_JPRB, 0.41207E+01_JPRB, 0.43681E+01_JPRB, 0.43829E+01_JPRB, 0.44274E+01_JPRB, 0.45882E+01_JPRB ,&
&  0.56355E+01_JPRB, 0.70910E+01_JPRB, 0.63074E+01_JPRB, 0.44068E+01_JPRB, 0.37521E+01_JPRB, 0.33703E+01_JPRB, 0.42641E+01_JPRB ,&
&  0.68556E+01_JPRB, 0.64166E+01_JPRB, 0.56291E+01_JPRB, 0.57633E+01_JPRB, 0.45725E+01_JPRB, 0.37445E+01_JPRB, 0.28829E+01_JPRB ,&
&  0.21922E+01_JPRB, 0.16925E+01_JPRB, 0.15073E+01_JPRB, 0.13445E+01_JPRB, 0.13093E+01_JPRB, 0.13278E+01_JPRB, 0.13685E+01_JPRB ,&
&  0.14145E+01_JPRB, 0.14219E+01_JPRB, 0.14006E+01_JPRB, 0.13669E+01_JPRB, 0.13505E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.12604E+01_JPRB, 0.11959E+01_JPRB, 0.11445E+01_JPRB, 0.11643E+01_JPRB, 0.12457E+01_JPRB, 0.12975E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.13190E+01_JPRB, 0.13291E+01_JPRB, 0.14108E+01_JPRB, 0.14828E+01_JPRB, 0.16774E+01_JPRB, 0.18932E+01_JPRB, 0.20834E+01_JPRB ,&
&  0.22464E+01_JPRB, 0.23759E+01_JPRB, 0.25516E+01_JPRB, 0.28060E+01_JPRB, 0.30686E+01_JPRB, 0.32893E+01_JPRB, 0.25618E+01_JPRB ,&
&  0.17603E+01_JPRB, 0.28007E+01_JPRB, 0.24060E+01_JPRB, 0.24568E+01_JPRB, 0.23143E+01_JPRB, 0.23350E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.20316E+01_JPRB, 0.19016E+01_JPRB, 0.19057E+01_JPRB, 0.19320E+01_JPRB, 0.18963E+01_JPRB, 0.19577E+01_JPRB, 0.20574E+01_JPRB ,&
&  0.21535E+01_JPRB, 0.22632E+01_JPRB, 0.23329E+01_JPRB, 0.23815E+01_JPRB, 0.24482E+01_JPRB, 0.25021E+01_JPRB, 0.25981E+01_JPRB ,&
&  0.27388E+01_JPRB, 0.28794E+01_JPRB, 0.29265E+01_JPRB, 0.29487E+01_JPRB, 0.28384E+01_JPRB, 0.29335E+01_JPRB, 0.30763E+01_JPRB ,&
&  0.29507E+01_JPRB, 0.27852E+01_JPRB, 0.30179E+01_JPRB, 0.30925E+01_JPRB, 0.31170E+01_JPRB, 0.34987E+01_JPRB, 0.35174E+01_JPRB ,&
&  0.40687E+01_JPRB, 0.41343E+01_JPRB, 0.49805E+01_JPRB, 0.39112E+01_JPRB, 0.38895E+01_JPRB, 0.41514E+01_JPRB, 0.39685E+01_JPRB ,&
&  0.37215E+01_JPRB, 0.39444E+01_JPRB, 0.40444E+01_JPRB, 0.44439E+01_JPRB, 0.45484E+01_JPRB, 0.43370E+01_JPRB, 0.39675E+01_JPRB ,&
&  0.42876E+01_JPRB, 0.47407E+01_JPRB, 0.45221E+01_JPRB, 0.34855E+01_JPRB, 0.34984E+01_JPRB, 0.38922E+01_JPRB, 0.53956E+01_JPRB ,&
&  0.60955E+01_JPRB, 0.54553E+01_JPRB, 0.48753E+01_JPRB, 0.42401E+01_JPRB, 0.31219E+01_JPRB, 0.24143E+01_JPRB, 0.17442E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.11109E+01_JPRB, 0.12259E+01_JPRB, 0.11881E+01_JPRB, 0.12335E+01_JPRB, 0.12964E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.13621E+01_JPRB, 0.13635E+01_JPRB, 0.13350E+01_JPRB, 0.13096E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.11578E+01_JPRB, 0.11238E+01_JPRB, 0.10890E+01_JPRB, 0.11590E+01_JPRB, 0.12922E+01_JPRB, 0.14107E+01_JPRB, 0.14239E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.12913E+01_JPRB, 0.14062E+01_JPRB, 0.15128E+01_JPRB, 0.17038E+01_JPRB, 0.19207E+01_JPRB, 0.21625E+01_JPRB ,&
&  0.24003E+01_JPRB, 0.26322E+01_JPRB, 0.28262E+01_JPRB, 0.30325E+01_JPRB, 0.32863E+01_JPRB, 0.35435E+01_JPRB, 0.31477E+01_JPRB ,&
&  0.23610E+01_JPRB, 0.20937E+01_JPRB, 0.25673E+01_JPRB, 0.23448E+01_JPRB, 0.21771E+01_JPRB, 0.22221E+01_JPRB, 0.22309E+01_JPRB ,&
&  0.22115E+01_JPRB, 0.20314E+01_JPRB, 0.20210E+01_JPRB, 0.20668E+01_JPRB, 0.19920E+01_JPRB, 0.20969E+01_JPRB, 0.22473E+01_JPRB ,&
&  0.23531E+01_JPRB, 0.24570E+01_JPRB, 0.25094E+01_JPRB, 0.25401E+01_JPRB, 0.25871E+01_JPRB, 0.26603E+01_JPRB, 0.27623E+01_JPRB ,&
&  0.28980E+01_JPRB, 0.30368E+01_JPRB, 0.31035E+01_JPRB, 0.29754E+01_JPRB, 0.29433E+01_JPRB, 0.30151E+01_JPRB, 0.30212E+01_JPRB ,&
&  0.29475E+01_JPRB, 0.30160E+01_JPRB, 0.31425E+01_JPRB, 0.32646E+01_JPRB, 0.32218E+01_JPRB, 0.37397E+01_JPRB, 0.38054E+01_JPRB ,&
&  0.40641E+01_JPRB, 0.41490E+01_JPRB, 0.45930E+01_JPRB, 0.35377E+01_JPRB, 0.36371E+01_JPRB, 0.37635E+01_JPRB, 0.37287E+01_JPRB ,&
&  0.37123E+01_JPRB, 0.39731E+01_JPRB, 0.41818E+01_JPRB, 0.43149E+01_JPRB, 0.43873E+01_JPRB, 0.38581E+01_JPRB, 0.37883E+01_JPRB ,&
&  0.37334E+01_JPRB, 0.42939E+01_JPRB, 0.37887E+01_JPRB, 0.36749E+01_JPRB, 0.34566E+01_JPRB, 0.38327E+01_JPRB, 0.43256E+01_JPRB ,&
&  0.47107E+01_JPRB, 0.40446E+01_JPRB, 0.33288E+01_JPRB, 0.24896E+01_JPRB, 0.16983E+01_JPRB, 0.12289E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.98627E+00_JPRB, 0.10235E+01_JPRB, 0.10855E+01_JPRB, 0.11114E+01_JPRB, 0.11514E+01_JPRB, 0.11937E+01_JPRB, 0.12072E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.11916E+01_JPRB, 0.11599E+01_JPRB, 0.11906E+01_JPRB, 0.11953E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.97731E+00_JPRB, 0.92504E+00_JPRB, 0.90306E+00_JPRB, 0.98936E+00_JPRB, 0.10958E+01_JPRB, 0.11578E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11071E+01_JPRB, 0.12046E+01_JPRB, 0.13283E+01_JPRB, 0.14865E+01_JPRB, 0.16929E+01_JPRB, 0.19117E+01_JPRB ,&
&  0.21556E+01_JPRB, 0.23497E+01_JPRB, 0.24898E+01_JPRB, 0.26449E+01_JPRB, 0.28433E+01_JPRB, 0.30654E+01_JPRB, 0.31778E+01_JPRB ,&
&  0.25874E+01_JPRB, 0.17794E+01_JPRB, 0.20584E+01_JPRB, 0.20737E+01_JPRB, 0.21336E+01_JPRB, 0.22042E+01_JPRB, 0.21722E+01_JPRB ,&
&  0.19600E+01_JPRB, 0.19139E+01_JPRB, 0.19857E+01_JPRB, 0.20419E+01_JPRB, 0.19680E+01_JPRB, 0.20753E+01_JPRB, 0.22091E+01_JPRB ,&
&  0.23661E+01_JPRB, 0.24889E+01_JPRB, 0.25698E+01_JPRB, 0.26509E+01_JPRB, 0.27097E+01_JPRB, 0.27905E+01_JPRB, 0.29308E+01_JPRB ,&
&  0.30866E+01_JPRB, 0.32312E+01_JPRB, 0.33174E+01_JPRB, 0.32906E+01_JPRB, 0.32868E+01_JPRB, 0.33543E+01_JPRB, 0.33115E+01_JPRB ,&
&  0.33021E+01_JPRB, 0.32845E+01_JPRB, 0.35353E+01_JPRB, 0.38540E+01_JPRB, 0.40057E+01_JPRB, 0.35968E+01_JPRB, 0.38069E+01_JPRB ,&
&  0.38574E+01_JPRB, 0.38047E+01_JPRB, 0.36883E+01_JPRB, 0.39664E+01_JPRB, 0.30054E+01_JPRB, 0.32809E+01_JPRB, 0.34217E+01_JPRB ,&
&  0.38772E+01_JPRB, 0.39601E+01_JPRB, 0.40214E+01_JPRB, 0.39436E+01_JPRB, 0.40815E+01_JPRB, 0.34646E+01_JPRB, 0.32094E+01_JPRB ,&
&  0.32595E+01_JPRB, 0.35308E+01_JPRB, 0.34495E+01_JPRB, 0.35391E+01_JPRB, 0.34101E+01_JPRB, 0.36511E+01_JPRB, 0.35272E+01_JPRB ,&
&  0.35286E+01_JPRB, 0.24917E+01_JPRB, 0.19680E+01_JPRB, 0.14972E+01_JPRB, 0.97275E+00_JPRB, 0.83186E+00_JPRB, 0.85893E+00_JPRB ,&
&  0.89409E+00_JPRB, 0.94125E+00_JPRB, 0.95361E+00_JPRB, 0.99741E+00_JPRB, 0.10351E+01_JPRB, 0.10271E+01_JPRB, 0.99835E+00_JPRB ,&
&  0.10011E+01_JPRB, 0.10361E+01_JPRB, 0.10269E+01_JPRB, 0.10179E+01_JPRB, 0.10026E+01_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.76717E+00_JPRB, 0.75979E+00_JPRB, 0.76824E+00_JPRB, 0.80884E+00_JPRB, 0.88741E+00_JPRB, 0.93620E+00_JPRB, 0.95751E+00_JPRB ,&
&  0.95430E+00_JPRB, 0.98428E+00_JPRB, 0.10479E+01_JPRB, 0.10861E+01_JPRB, 0.11753E+01_JPRB, 0.13188E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.18255E+01_JPRB, 0.19659E+01_JPRB, 0.21671E+01_JPRB, 0.23634E+01_JPRB, 0.26273E+01_JPRB, 0.28101E+01_JPRB ,&
&  0.27932E+01_JPRB, 0.24119E+01_JPRB, 0.22167E+01_JPRB, 0.17248E+01_JPRB, 0.15490E+01_JPRB, 0.18108E+01_JPRB, 0.17337E+01_JPRB ,&
&  0.17237E+01_JPRB, 0.17581E+01_JPRB, 0.17513E+01_JPRB, 0.17955E+01_JPRB, 0.18750E+01_JPRB, 0.19721E+01_JPRB, 0.21173E+01_JPRB ,&
&  0.22887E+01_JPRB, 0.24466E+01_JPRB, 0.25874E+01_JPRB, 0.27063E+01_JPRB, 0.27756E+01_JPRB, 0.28710E+01_JPRB, 0.30282E+01_JPRB ,&
&  0.32020E+01_JPRB, 0.33158E+01_JPRB, 0.34227E+01_JPRB, 0.36045E+01_JPRB, 0.35803E+01_JPRB, 0.36649E+01_JPRB, 0.36973E+01_JPRB ,&
&  0.37324E+01_JPRB, 0.36899E+01_JPRB, 0.36961E+01_JPRB, 0.37812E+01_JPRB, 0.37226E+01_JPRB, 0.33091E+01_JPRB, 0.32911E+01_JPRB ,&
&  0.34255E+01_JPRB, 0.33123E+01_JPRB, 0.29983E+01_JPRB, 0.34578E+01_JPRB, 0.25366E+01_JPRB, 0.30312E+01_JPRB, 0.34977E+01_JPRB ,&
&  0.35040E+01_JPRB, 0.34505E+01_JPRB, 0.32945E+01_JPRB, 0.32846E+01_JPRB, 0.32841E+01_JPRB, 0.30178E+01_JPRB, 0.29146E+01_JPRB ,&
&  0.28987E+01_JPRB, 0.25581E+01_JPRB, 0.25397E+01_JPRB, 0.28168E+01_JPRB, 0.30749E+01_JPRB, 0.28521E+01_JPRB, 0.24188E+01_JPRB ,&
&  0.19654E+01_JPRB, 0.17763E+01_JPRB, 0.15835E+01_JPRB, 0.16826E+01_JPRB, 0.12355E+01_JPRB, 0.78094E+00_JPRB, 0.78615E+00_JPRB ,&
&  0.80763E+00_JPRB, 0.83308E+00_JPRB, 0.83162E+00_JPRB, 0.87881E+00_JPRB, 0.89432E+00_JPRB, 0.87725E+00_JPRB, 0.88056E+00_JPRB ,&
&  0.90206E+00_JPRB, 0.89354E+00_JPRB, 0.87939E+00_JPRB, 0.84554E+00_JPRB, 0.81852E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.72986E+00_JPRB, 0.71177E+00_JPRB, 0.71104E+00_JPRB, 0.70672E+00_JPRB, 0.71610E+00_JPRB, 0.71632E+00_JPRB, 0.72988E+00_JPRB ,&
&  0.75665E+00_JPRB, 0.79057E+00_JPRB, 0.79039E+00_JPRB, 0.77485E+00_JPRB, 0.77642E+00_JPRB, 0.79346E+00_JPRB, 0.85075E+00_JPRB ,&
&  0.98092E+00_JPRB, 0.11204E+01_JPRB, 0.12165E+01_JPRB, 0.13359E+01_JPRB, 0.15702E+01_JPRB, 0.19909E+01_JPRB, 0.23948E+01_JPRB ,&
&  0.26947E+01_JPRB, 0.28840E+01_JPRB, 0.28142E+01_JPRB, 0.28484E+01_JPRB, 0.22419E+01_JPRB, 0.17111E+01_JPRB, 0.18944E+01_JPRB ,&
&  0.20465E+01_JPRB, 0.19921E+01_JPRB, 0.17209E+01_JPRB, 0.16677E+01_JPRB, 0.16587E+01_JPRB, 0.17369E+01_JPRB, 0.19207E+01_JPRB ,&
&  0.21083E+01_JPRB, 0.23136E+01_JPRB, 0.24767E+01_JPRB, 0.26029E+01_JPRB, 0.26963E+01_JPRB, 0.28074E+01_JPRB, 0.29144E+01_JPRB ,&
&  0.30341E+01_JPRB, 0.30963E+01_JPRB, 0.31561E+01_JPRB, 0.32999E+01_JPRB, 0.34587E+01_JPRB, 0.36276E+01_JPRB, 0.37310E+01_JPRB ,&
&  0.37660E+01_JPRB, 0.35895E+01_JPRB, 0.35860E+01_JPRB, 0.35813E+01_JPRB, 0.33749E+01_JPRB, 0.31222E+01_JPRB, 0.30711E+01_JPRB ,&
&  0.30297E+01_JPRB, 0.29631E+01_JPRB, 0.23901E+01_JPRB, 0.29544E+01_JPRB, 0.34465E+01_JPRB, 0.32403E+01_JPRB, 0.31428E+01_JPRB ,&
&  0.29584E+01_JPRB, 0.28249E+01_JPRB, 0.26663E+01_JPRB, 0.27175E+01_JPRB, 0.29235E+01_JPRB, 0.36635E+01_JPRB, 0.28989E+01_JPRB ,&
&  0.25275E+01_JPRB, 0.17948E+01_JPRB, 0.17117E+01_JPRB, 0.19558E+01_JPRB, 0.21524E+01_JPRB, 0.20144E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.14471E+01_JPRB, 0.15344E+01_JPRB, 0.20172E+01_JPRB, 0.22460E+01_JPRB, 0.79667E+00_JPRB, 0.71727E+00_JPRB ,&
&  0.72288E+00_JPRB, 0.72630E+00_JPRB, 0.72285E+00_JPRB, 0.74387E+00_JPRB, 0.74410E+00_JPRB, 0.75204E+00_JPRB, 0.77563E+00_JPRB ,&
&  0.78866E+00_JPRB, 0.78458E+00_JPRB, 0.77310E+00_JPRB, 0.74176E+00_JPRB, 0.74131E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.78288E+00_JPRB, 0.77722E+00_JPRB, 0.77864E+00_JPRB, 0.76974E+00_JPRB, 0.76137E+00_JPRB, 0.75933E+00_JPRB, 0.76294E+00_JPRB ,&
&  0.76533E+00_JPRB, 0.75591E+00_JPRB, 0.74415E+00_JPRB, 0.73549E+00_JPRB, 0.72417E+00_JPRB, 0.69164E+00_JPRB, 0.64780E+00_JPRB ,&
&  0.66119E+00_JPRB, 0.68002E+00_JPRB, 0.69563E+00_JPRB, 0.73299E+00_JPRB, 0.82974E+00_JPRB, 0.97786E+00_JPRB, 0.11818E+01_JPRB ,&
&  0.15692E+01_JPRB, 0.19919E+01_JPRB, 0.22779E+01_JPRB, 0.23307E+01_JPRB, 0.25577E+01_JPRB, 0.19450E+01_JPRB, 0.18791E+01_JPRB ,&
&  0.20899E+01_JPRB, 0.17697E+01_JPRB, 0.16058E+01_JPRB, 0.14978E+01_JPRB, 0.14840E+01_JPRB, 0.15290E+01_JPRB, 0.16442E+01_JPRB ,&
&  0.18279E+01_JPRB, 0.20181E+01_JPRB, 0.21794E+01_JPRB, 0.23360E+01_JPRB, 0.24792E+01_JPRB, 0.26261E+01_JPRB, 0.27314E+01_JPRB ,&
&  0.27627E+01_JPRB, 0.28090E+01_JPRB, 0.29087E+01_JPRB, 0.29346E+01_JPRB, 0.31893E+01_JPRB, 0.33423E+01_JPRB, 0.34012E+01_JPRB ,&
&  0.33295E+01_JPRB, 0.31958E+01_JPRB, 0.31946E+01_JPRB, 0.30611E+01_JPRB, 0.29333E+01_JPRB, 0.28268E+01_JPRB, 0.27105E+01_JPRB ,&
&  0.26004E+01_JPRB, 0.24352E+01_JPRB, 0.20388E+01_JPRB, 0.23572E+01_JPRB, 0.25851E+01_JPRB, 0.24120E+01_JPRB, 0.28121E+01_JPRB ,&
&  0.26251E+01_JPRB, 0.25318E+01_JPRB, 0.24312E+01_JPRB, 0.23342E+01_JPRB, 0.22643E+01_JPRB, 0.26319E+01_JPRB, 0.25778E+01_JPRB ,&
&  0.18382E+01_JPRB, 0.13215E+01_JPRB, 0.11591E+01_JPRB, 0.12259E+01_JPRB, 0.13805E+01_JPRB, 0.14856E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.13007E+01_JPRB, 0.12078E+01_JPRB, 0.13207E+01_JPRB, 0.16556E+01_JPRB, 0.11617E+01_JPRB, 0.74792E+00_JPRB, 0.65601E+00_JPRB ,&
&  0.63586E+00_JPRB, 0.63405E+00_JPRB, 0.64197E+00_JPRB, 0.67118E+00_JPRB, 0.69717E+00_JPRB, 0.72243E+00_JPRB, 0.74304E+00_JPRB ,&
&  0.75023E+00_JPRB, 0.76440E+00_JPRB, 0.77485E+00_JPRB, 0.78653E+00_JPRB, 0.79191E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.86842E+00_JPRB, 0.88074E+00_JPRB, 0.87230E+00_JPRB, 0.87828E+00_JPRB, 0.88399E+00_JPRB, 0.90068E+00_JPRB, 0.90090E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.90040E+00_JPRB, 0.87839E+00_JPRB, 0.88183E+00_JPRB, 0.87799E+00_JPRB, 0.88052E+00_JPRB, 0.86849E+00_JPRB ,&
&  0.87419E+00_JPRB, 0.89167E+00_JPRB, 0.89669E+00_JPRB, 0.89840E+00_JPRB, 0.88829E+00_JPRB, 0.86405E+00_JPRB, 0.84171E+00_JPRB ,&
&  0.88738E+00_JPRB, 0.10023E+01_JPRB, 0.11391E+01_JPRB, 0.12325E+01_JPRB, 0.14597E+01_JPRB, 0.14790E+01_JPRB, 0.16164E+01_JPRB ,&
&  0.17789E+01_JPRB, 0.15434E+01_JPRB, 0.15683E+01_JPRB, 0.13576E+01_JPRB, 0.13307E+01_JPRB, 0.12905E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.13727E+01_JPRB, 0.14567E+01_JPRB, 0.15537E+01_JPRB, 0.16353E+01_JPRB, 0.16610E+01_JPRB, 0.16685E+01_JPRB ,&
&  0.16544E+01_JPRB, 0.17613E+01_JPRB, 0.20964E+01_JPRB, 0.23307E+01_JPRB, 0.26062E+01_JPRB, 0.27433E+01_JPRB, 0.27541E+01_JPRB ,&
&  0.27102E+01_JPRB, 0.26922E+01_JPRB, 0.23681E+01_JPRB, 0.21363E+01_JPRB, 0.21024E+01_JPRB, 0.19461E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.20283E+01_JPRB, 0.20250E+01_JPRB, 0.16938E+01_JPRB, 0.18894E+01_JPRB, 0.21200E+01_JPRB, 0.20987E+01_JPRB, 0.24145E+01_JPRB ,&
&  0.21184E+01_JPRB, 0.19157E+01_JPRB, 0.18243E+01_JPRB, 0.17755E+01_JPRB, 0.17152E+01_JPRB, 0.16358E+01_JPRB, 0.16529E+01_JPRB ,&
&  0.13380E+01_JPRB, 0.11406E+01_JPRB, 0.10689E+01_JPRB, 0.10974E+01_JPRB, 0.13047E+01_JPRB, 0.16803E+01_JPRB, 0.13615E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.12224E+01_JPRB, 0.15923E+01_JPRB, 0.22537E+01_JPRB, 0.10872E+01_JPRB, 0.76573E+00_JPRB, 0.62358E+00_JPRB ,&
&  0.62804E+00_JPRB, 0.66025E+00_JPRB, 0.68908E+00_JPRB, 0.72685E+00_JPRB, 0.74840E+00_JPRB, 0.75656E+00_JPRB, 0.77097E+00_JPRB ,&
&  0.80387E+00_JPRB, 0.83192E+00_JPRB, 0.87129E+00_JPRB, 0.86996E+00_JPRB, 0.85531E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.91827E+00_JPRB, 0.93703E+00_JPRB, 0.93792E+00_JPRB, 0.95502E+00_JPRB, 0.97923E+00_JPRB, 0.98892E+00_JPRB, 0.99306E+00_JPRB ,&
&  0.98317E+00_JPRB, 0.98381E+00_JPRB, 0.97846E+00_JPRB, 0.99484E+00_JPRB, 0.99654E+00_JPRB, 0.99729E+00_JPRB, 0.99955E+00_JPRB ,&
&  0.10166E+01_JPRB, 0.10296E+01_JPRB, 0.10619E+01_JPRB, 0.10644E+01_JPRB, 0.10584E+01_JPRB, 0.10631E+01_JPRB, 0.10847E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11357E+01_JPRB, 0.11738E+01_JPRB, 0.12068E+01_JPRB, 0.13235E+01_JPRB, 0.14871E+01_JPRB, 0.17707E+01_JPRB ,&
&  0.16451E+01_JPRB, 0.15738E+01_JPRB, 0.14298E+01_JPRB, 0.12232E+01_JPRB, 0.11990E+01_JPRB, 0.11910E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.10242E+01_JPRB, 0.99225E+00_JPRB, 0.98009E+00_JPRB, 0.98392E+00_JPRB, 0.96896E+00_JPRB, 0.95588E+00_JPRB ,&
&  0.91703E+00_JPRB, 0.92846E+00_JPRB, 0.11262E+01_JPRB, 0.13823E+01_JPRB, 0.15915E+01_JPRB, 0.17804E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.15440E+01_JPRB, 0.15808E+01_JPRB, 0.14473E+01_JPRB, 0.13627E+01_JPRB, 0.12538E+01_JPRB, 0.11584E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.14137E+01_JPRB, 0.14281E+01_JPRB, 0.16895E+01_JPRB, 0.18510E+01_JPRB, 0.18930E+01_JPRB, 0.17229E+01_JPRB ,&
&  0.15011E+01_JPRB, 0.13863E+01_JPRB, 0.13589E+01_JPRB, 0.13448E+01_JPRB, 0.13596E+01_JPRB, 0.12960E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.11190E+01_JPRB, 0.10389E+01_JPRB, 0.10202E+01_JPRB, 0.98282E+00_JPRB, 0.12540E+01_JPRB, 0.14378E+01_JPRB, 0.18516E+01_JPRB ,&
&  0.13299E+01_JPRB, 0.12178E+01_JPRB, 0.10686E+01_JPRB, 0.16178E+01_JPRB, 0.94662E+00_JPRB, 0.63936E+00_JPRB, 0.63539E+00_JPRB ,&
&  0.70945E+00_JPRB, 0.75302E+00_JPRB, 0.77470E+00_JPRB, 0.80693E+00_JPRB, 0.81948E+00_JPRB, 0.81778E+00_JPRB, 0.82981E+00_JPRB ,&
&  0.86852E+00_JPRB, 0.87599E+00_JPRB, 0.88277E+00_JPRB, 0.88076E+00_JPRB, 0.89758E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.93818E+00_JPRB, 0.96710E+00_JPRB, 0.98334E+00_JPRB, 0.10003E+01_JPRB, 0.10236E+01_JPRB, 0.10376E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.10508E+01_JPRB, 0.10470E+01_JPRB, 0.10547E+01_JPRB, 0.10687E+01_JPRB, 0.10754E+01_JPRB, 0.10723E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.11222E+01_JPRB, 0.11513E+01_JPRB, 0.11447E+01_JPRB, 0.11397E+01_JPRB, 0.11563E+01_JPRB, 0.11796E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.11729E+01_JPRB, 0.11810E+01_JPRB, 0.11933E+01_JPRB, 0.12162E+01_JPRB, 0.12225E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.12569E+01_JPRB, 0.16798E+01_JPRB, 0.15424E+01_JPRB, 0.13183E+01_JPRB, 0.12848E+01_JPRB, 0.11784E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.10781E+01_JPRB, 0.10370E+01_JPRB, 0.10046E+01_JPRB, 0.10017E+01_JPRB, 0.10100E+01_JPRB, 0.98713E+00_JPRB ,&
&  0.99472E+00_JPRB, 0.10171E+01_JPRB, 0.10782E+01_JPRB, 0.11581E+01_JPRB, 0.12191E+01_JPRB, 0.12166E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.98354E+00_JPRB, 0.81134E+00_JPRB, 0.79644E+00_JPRB, 0.76852E+00_JPRB, 0.65958E+00_JPRB, 0.61654E+00_JPRB ,&
&  0.78609E+00_JPRB, 0.95357E+00_JPRB, 0.12782E+01_JPRB, 0.14804E+01_JPRB, 0.15192E+01_JPRB, 0.15826E+01_JPRB, 0.13712E+01_JPRB ,&
&  0.12153E+01_JPRB, 0.11190E+01_JPRB, 0.11144E+01_JPRB, 0.10766E+01_JPRB, 0.10895E+01_JPRB, 0.10785E+01_JPRB, 0.10052E+01_JPRB ,&
&  0.94882E+00_JPRB, 0.93546E+00_JPRB, 0.95235E+00_JPRB, 0.10088E+01_JPRB, 0.10852E+01_JPRB, 0.19100E+01_JPRB, 0.29887E+01_JPRB ,&
&  0.15081E+01_JPRB, 0.10895E+01_JPRB, 0.70448E+00_JPRB, 0.88136E+00_JPRB, 0.72984E+00_JPRB, 0.64270E+00_JPRB, 0.69289E+00_JPRB ,&
&  0.81271E+00_JPRB, 0.86592E+00_JPRB, 0.87639E+00_JPRB, 0.86968E+00_JPRB, 0.84794E+00_JPRB, 0.82857E+00_JPRB, 0.81937E+00_JPRB ,&
&  0.83678E+00_JPRB, 0.85409E+00_JPRB, 0.87491E+00_JPRB, 0.88811E+00_JPRB, 0.90824E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.94317E+00_JPRB, 0.96981E+00_JPRB, 0.99055E+00_JPRB, 0.10153E+01_JPRB, 0.10423E+01_JPRB, 0.10668E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.11216E+01_JPRB, 0.11355E+01_JPRB, 0.11529E+01_JPRB, 0.11738E+01_JPRB, 0.11906E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.12460E+01_JPRB, 0.12737E+01_JPRB, 0.13054E+01_JPRB, 0.13046E+01_JPRB, 0.13054E+01_JPRB, 0.13318E+01_JPRB, 0.13561E+01_JPRB ,&
&  0.13635E+01_JPRB, 0.13794E+01_JPRB, 0.14061E+01_JPRB, 0.14385E+01_JPRB, 0.14807E+01_JPRB, 0.15701E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.16235E+01_JPRB, 0.17513E+01_JPRB, 0.16368E+01_JPRB, 0.14390E+01_JPRB, 0.13474E+01_JPRB, 0.13027E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.12474E+01_JPRB, 0.12194E+01_JPRB, 0.11866E+01_JPRB, 0.11721E+01_JPRB, 0.11812E+01_JPRB, 0.11934E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.12470E+01_JPRB, 0.12933E+01_JPRB, 0.13540E+01_JPRB, 0.14126E+01_JPRB, 0.13820E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.12327E+01_JPRB, 0.11499E+01_JPRB, 0.92730E+00_JPRB, 0.85439E+00_JPRB, 0.79914E+00_JPRB, 0.70509E+00_JPRB, 0.62134E+00_JPRB ,&
&  0.73179E+00_JPRB, 0.83808E+00_JPRB, 0.10456E+01_JPRB, 0.13274E+01_JPRB, 0.13194E+01_JPRB, 0.12691E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.94974E+00_JPRB, 0.93134E+00_JPRB, 0.89877E+00_JPRB, 0.90143E+00_JPRB, 0.91908E+00_JPRB, 0.86264E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.86864E+00_JPRB, 0.89917E+00_JPRB, 0.95809E+00_JPRB, 0.11103E+01_JPRB, 0.12252E+01_JPRB, 0.18492E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.83897E+00_JPRB, 0.83312E+00_JPRB, 0.79514E+00_JPRB, 0.73434E+00_JPRB, 0.68993E+00_JPRB, 0.76082E+00_JPRB ,&
&  0.91437E+00_JPRB, 0.10377E+01_JPRB, 0.10658E+01_JPRB, 0.10119E+01_JPRB, 0.92298E+00_JPRB, 0.87282E+00_JPRB, 0.84630E+00_JPRB ,&
&  0.85983E+00_JPRB, 0.87224E+00_JPRB, 0.88827E+00_JPRB, 0.89634E+00_JPRB, 0.91374E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.92463E+00_JPRB, 0.95482E+00_JPRB, 0.97996E+00_JPRB, 0.10052E+01_JPRB, 0.10302E+01_JPRB, 0.10545E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10985E+01_JPRB, 0.11236E+01_JPRB, 0.11532E+01_JPRB, 0.11837E+01_JPRB, 0.12115E+01_JPRB, 0.12456E+01_JPRB, 0.12745E+01_JPRB ,&
&  0.13129E+01_JPRB, 0.13401E+01_JPRB, 0.13701E+01_JPRB, 0.13766E+01_JPRB, 0.13819E+01_JPRB, 0.14016E+01_JPRB, 0.14177E+01_JPRB ,&
&  0.14176E+01_JPRB, 0.14270E+01_JPRB, 0.14441E+01_JPRB, 0.14735E+01_JPRB, 0.14847E+01_JPRB, 0.15609E+01_JPRB, 0.14081E+01_JPRB ,&
&  0.17530E+01_JPRB, 0.19115E+01_JPRB, 0.17829E+01_JPRB, 0.15857E+01_JPRB, 0.14774E+01_JPRB, 0.14645E+01_JPRB, 0.14677E+01_JPRB ,&
&  0.14851E+01_JPRB, 0.15303E+01_JPRB, 0.14299E+01_JPRB, 0.13777E+01_JPRB, 0.13606E+01_JPRB, 0.13693E+01_JPRB, 0.14111E+01_JPRB ,&
&  0.14615E+01_JPRB, 0.15150E+01_JPRB, 0.15726E+01_JPRB, 0.16277E+01_JPRB, 0.16680E+01_JPRB, 0.16813E+01_JPRB, 0.16549E+01_JPRB ,&
&  0.15961E+01_JPRB, 0.15321E+01_JPRB, 0.13905E+01_JPRB, 0.13509E+01_JPRB, 0.12811E+01_JPRB, 0.10833E+01_JPRB, 0.88838E+00_JPRB ,&
&  0.84813E+00_JPRB, 0.91465E+00_JPRB, 0.99121E+00_JPRB, 0.11751E+01_JPRB, 0.11647E+01_JPRB, 0.10942E+01_JPRB, 0.99324E+00_JPRB ,&
&  0.90266E+00_JPRB, 0.83413E+00_JPRB, 0.83538E+00_JPRB, 0.80703E+00_JPRB, 0.78545E+00_JPRB, 0.77612E+00_JPRB, 0.76597E+00_JPRB ,&
&  0.78637E+00_JPRB, 0.81831E+00_JPRB, 0.86494E+00_JPRB, 0.91878E+00_JPRB, 0.97412E+00_JPRB, 0.98047E+00_JPRB, 0.15001E+01_JPRB ,&
&  0.11911E+01_JPRB, 0.96896E+00_JPRB, 0.11242E+01_JPRB, 0.66326E+00_JPRB, 0.73484E+00_JPRB, 0.79558E+00_JPRB, 0.92510E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.13889E+01_JPRB, 0.18670E+01_JPRB, 0.14966E+01_JPRB, 0.12201E+01_JPRB, 0.94800E+00_JPRB, 0.85937E+00_JPRB ,&
&  0.85950E+00_JPRB, 0.86421E+00_JPRB, 0.87549E+00_JPRB, 0.88152E+00_JPRB, 0.89734E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.89950E+00_JPRB, 0.92321E+00_JPRB, 0.95114E+00_JPRB, 0.97647E+00_JPRB, 0.99454E+00_JPRB, 0.10116E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.10516E+01_JPRB, 0.10695E+01_JPRB, 0.10959E+01_JPRB, 0.11274E+01_JPRB, 0.11627E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.12541E+01_JPRB, 0.13024E+01_JPRB, 0.13646E+01_JPRB, 0.14137E+01_JPRB, 0.14484E+01_JPRB, 0.14842E+01_JPRB, 0.15205E+01_JPRB ,&
&  0.15733E+01_JPRB, 0.16087E+01_JPRB, 0.16409E+01_JPRB, 0.17036E+01_JPRB, 0.17546E+01_JPRB, 0.18240E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.18989E+01_JPRB, 0.21637E+01_JPRB, 0.19771E+01_JPRB, 0.17206E+01_JPRB, 0.15935E+01_JPRB, 0.15755E+01_JPRB, 0.15689E+01_JPRB ,&
&  0.15730E+01_JPRB, 0.15559E+01_JPRB, 0.14462E+01_JPRB, 0.14828E+01_JPRB, 0.15148E+01_JPRB, 0.15088E+01_JPRB, 0.15735E+01_JPRB ,&
&  0.16433E+01_JPRB, 0.17045E+01_JPRB, 0.17674E+01_JPRB, 0.18285E+01_JPRB, 0.18713E+01_JPRB, 0.19223E+01_JPRB, 0.19153E+01_JPRB ,&
&  0.19037E+01_JPRB, 0.18978E+01_JPRB, 0.18887E+01_JPRB, 0.17171E+01_JPRB, 0.16443E+01_JPRB, 0.15528E+01_JPRB, 0.12895E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.97503E+00_JPRB, 0.94472E+00_JPRB, 0.10157E+01_JPRB, 0.98313E+00_JPRB, 0.90024E+00_JPRB, 0.83621E+00_JPRB ,&
&  0.78121E+00_JPRB, 0.74080E+00_JPRB, 0.76137E+00_JPRB, 0.74926E+00_JPRB, 0.68575E+00_JPRB, 0.70629E+00_JPRB, 0.73247E+00_JPRB ,&
&  0.73273E+00_JPRB, 0.76244E+00_JPRB, 0.80950E+00_JPRB, 0.84098E+00_JPRB, 0.91995E+00_JPRB, 0.10891E+01_JPRB, 0.11934E+01_JPRB ,&
&  0.12474E+01_JPRB, 0.95321E+00_JPRB, 0.90650E+00_JPRB, 0.91823E+00_JPRB, 0.82847E+00_JPRB, 0.10493E+01_JPRB, 0.14041E+01_JPRB ,&
&  0.20098E+01_JPRB, 0.21382E+01_JPRB, 0.27365E+01_JPRB, 0.42263E+01_JPRB, 0.32184E+01_JPRB, 0.21637E+01_JPRB, 0.87618E+00_JPRB ,&
&  0.87949E+00_JPRB, 0.87441E+00_JPRB, 0.89188E+00_JPRB, 0.89128E+00_JPRB, 0.89798E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.77503E+00_JPRB, 0.77565E+00_JPRB, 0.80520E+00_JPRB, 0.83753E+00_JPRB, 0.86656E+00_JPRB, 0.89496E+00_JPRB, 0.91741E+00_JPRB ,&
&  0.94880E+00_JPRB, 0.97617E+00_JPRB, 0.98649E+00_JPRB, 0.98777E+00_JPRB, 0.10044E+01_JPRB, 0.10147E+01_JPRB, 0.10450E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.11404E+01_JPRB, 0.12088E+01_JPRB, 0.12579E+01_JPRB, 0.13079E+01_JPRB, 0.13808E+01_JPRB, 0.14514E+01_JPRB ,&
&  0.15297E+01_JPRB, 0.16280E+01_JPRB, 0.17115E+01_JPRB, 0.17975E+01_JPRB, 0.18834E+01_JPRB, 0.18747E+01_JPRB, 0.14262E+01_JPRB ,&
&  0.19962E+01_JPRB, 0.22358E+01_JPRB, 0.21146E+01_JPRB, 0.18395E+01_JPRB, 0.16811E+01_JPRB, 0.16237E+01_JPRB, 0.16186E+01_JPRB ,&
&  0.16006E+01_JPRB, 0.15257E+01_JPRB, 0.14362E+01_JPRB, 0.13981E+01_JPRB, 0.15528E+01_JPRB, 0.15268E+01_JPRB, 0.15953E+01_JPRB ,&
&  0.16673E+01_JPRB, 0.17296E+01_JPRB, 0.17808E+01_JPRB, 0.18232E+01_JPRB, 0.18547E+01_JPRB, 0.18976E+01_JPRB, 0.19427E+01_JPRB ,&
&  0.19837E+01_JPRB, 0.20080E+01_JPRB, 0.20662E+01_JPRB, 0.18818E+01_JPRB, 0.18033E+01_JPRB, 0.19133E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.98845E+00_JPRB, 0.10019E+01_JPRB, 0.98829E+00_JPRB, 0.91625E+00_JPRB, 0.82465E+00_JPRB, 0.77326E+00_JPRB ,&
&  0.73705E+00_JPRB, 0.70744E+00_JPRB, 0.70078E+00_JPRB, 0.68827E+00_JPRB, 0.68803E+00_JPRB, 0.70339E+00_JPRB, 0.70894E+00_JPRB ,&
&  0.74462E+00_JPRB, 0.73705E+00_JPRB, 0.74447E+00_JPRB, 0.76913E+00_JPRB, 0.85637E+00_JPRB, 0.10478E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.15362E+01_JPRB, 0.92351E+00_JPRB, 0.89506E+00_JPRB, 0.89220E+00_JPRB, 0.93723E+00_JPRB, 0.10576E+01_JPRB, 0.12532E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.20919E+01_JPRB, 0.19971E+01_JPRB, 0.15770E+01_JPRB, 0.19570E+01_JPRB, 0.16554E+01_JPRB, 0.77617E+00_JPRB ,&
&  0.78061E+00_JPRB, 0.75724E+00_JPRB, 0.76533E+00_JPRB, 0.77419E+00_JPRB, 0.77789E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.63718E+00_JPRB, 0.62633E+00_JPRB, 0.63631E+00_JPRB, 0.65779E+00_JPRB, 0.67720E+00_JPRB, 0.71380E+00_JPRB, 0.75872E+00_JPRB ,&
&  0.81478E+00_JPRB, 0.85457E+00_JPRB, 0.87120E+00_JPRB, 0.87734E+00_JPRB, 0.88947E+00_JPRB, 0.89960E+00_JPRB, 0.91986E+00_JPRB ,&
&  0.95416E+00_JPRB, 0.98932E+00_JPRB, 0.10353E+01_JPRB, 0.10748E+01_JPRB, 0.11127E+01_JPRB, 0.11884E+01_JPRB, 0.12760E+01_JPRB ,&
&  0.13828E+01_JPRB, 0.14971E+01_JPRB, 0.16317E+01_JPRB, 0.17730E+01_JPRB, 0.19173E+01_JPRB, 0.19761E+01_JPRB, 0.19938E+01_JPRB ,&
&  0.14950E+01_JPRB, 0.20760E+01_JPRB, 0.21098E+01_JPRB, 0.19189E+01_JPRB, 0.17531E+01_JPRB, 0.17175E+01_JPRB, 0.16774E+01_JPRB ,&
&  0.16271E+01_JPRB, 0.14954E+01_JPRB, 0.14346E+01_JPRB, 0.13954E+01_JPRB, 0.14505E+01_JPRB, 0.14700E+01_JPRB, 0.15311E+01_JPRB ,&
&  0.15904E+01_JPRB, 0.16337E+01_JPRB, 0.16651E+01_JPRB, 0.16973E+01_JPRB, 0.17496E+01_JPRB, 0.18302E+01_JPRB, 0.19410E+01_JPRB ,&
&  0.20310E+01_JPRB, 0.21066E+01_JPRB, 0.21662E+01_JPRB, 0.20203E+01_JPRB, 0.18937E+01_JPRB, 0.20640E+01_JPRB, 0.19875E+01_JPRB ,&
&  0.13837E+01_JPRB, 0.10299E+01_JPRB, 0.10643E+01_JPRB, 0.10829E+01_JPRB, 0.96467E+00_JPRB, 0.87310E+00_JPRB, 0.80940E+00_JPRB ,&
&  0.73717E+00_JPRB, 0.68071E+00_JPRB, 0.68969E+00_JPRB, 0.69782E+00_JPRB, 0.70792E+00_JPRB, 0.71282E+00_JPRB, 0.73244E+00_JPRB ,&
&  0.76282E+00_JPRB, 0.75265E+00_JPRB, 0.75076E+00_JPRB, 0.75621E+00_JPRB, 0.78769E+00_JPRB, 0.83024E+00_JPRB, 0.87156E+00_JPRB ,&
&  0.93449E+00_JPRB, 0.87645E+00_JPRB, 0.87381E+00_JPRB, 0.90671E+00_JPRB, 0.93524E+00_JPRB, 0.97530E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10946E+01_JPRB, 0.11269E+01_JPRB, 0.11198E+01_JPRB, 0.10224E+01_JPRB, 0.93606E+00_JPRB, 0.69479E+00_JPRB ,&
&  0.67959E+00_JPRB, 0.66192E+00_JPRB, 0.66202E+00_JPRB, 0.65918E+00_JPRB, 0.65733E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.56208E+00_JPRB, 0.55626E+00_JPRB, 0.56354E+00_JPRB, 0.57460E+00_JPRB, 0.59348E+00_JPRB, 0.62576E+00_JPRB, 0.67377E+00_JPRB ,&
&  0.72858E+00_JPRB, 0.76083E+00_JPRB, 0.78874E+00_JPRB, 0.78192E+00_JPRB, 0.78603E+00_JPRB, 0.80614E+00_JPRB, 0.82134E+00_JPRB ,&
&  0.85756E+00_JPRB, 0.89511E+00_JPRB, 0.91779E+00_JPRB, 0.92604E+00_JPRB, 0.94193E+00_JPRB, 0.98264E+00_JPRB, 0.10407E+01_JPRB ,&
&  0.11143E+01_JPRB, 0.12182E+01_JPRB, 0.13612E+01_JPRB, 0.15542E+01_JPRB, 0.17664E+01_JPRB, 0.18507E+01_JPRB, 0.18514E+01_JPRB ,&
&  0.92782E+00_JPRB, 0.15501E+01_JPRB, 0.20459E+01_JPRB, 0.20113E+01_JPRB, 0.19113E+01_JPRB, 0.18433E+01_JPRB, 0.18028E+01_JPRB ,&
&  0.16160E+01_JPRB, 0.14247E+01_JPRB, 0.13459E+01_JPRB, 0.14412E+01_JPRB, 0.13509E+01_JPRB, 0.13826E+01_JPRB, 0.14272E+01_JPRB ,&
&  0.14817E+01_JPRB, 0.15262E+01_JPRB, 0.15464E+01_JPRB, 0.15588E+01_JPRB, 0.15988E+01_JPRB, 0.16748E+01_JPRB, 0.17769E+01_JPRB ,&
&  0.19093E+01_JPRB, 0.20765E+01_JPRB, 0.21988E+01_JPRB, 0.20605E+01_JPRB, 0.20169E+01_JPRB, 0.21396E+01_JPRB, 0.25463E+01_JPRB ,&
&  0.17244E+01_JPRB, 0.11728E+01_JPRB, 0.11770E+01_JPRB, 0.12070E+01_JPRB, 0.10520E+01_JPRB, 0.95300E+00_JPRB, 0.88296E+00_JPRB ,&
&  0.80278E+00_JPRB, 0.75453E+00_JPRB, 0.73218E+00_JPRB, 0.72152E+00_JPRB, 0.73896E+00_JPRB, 0.76584E+00_JPRB, 0.80868E+00_JPRB ,&
&  0.83395E+00_JPRB, 0.83719E+00_JPRB, 0.83159E+00_JPRB, 0.83538E+00_JPRB, 0.84328E+00_JPRB, 0.85529E+00_JPRB, 0.85733E+00_JPRB ,&
&  0.87946E+00_JPRB, 0.87442E+00_JPRB, 0.88591E+00_JPRB, 0.92652E+00_JPRB, 0.97862E+00_JPRB, 0.99519E+00_JPRB, 0.99104E+00_JPRB ,&
&  0.93952E+00_JPRB, 0.93667E+00_JPRB, 0.92266E+00_JPRB, 0.90035E+00_JPRB, 0.86295E+00_JPRB, 0.80376E+00_JPRB, 0.70334E+00_JPRB ,&
&  0.65485E+00_JPRB, 0.60466E+00_JPRB, 0.57688E+00_JPRB, 0.56401E+00_JPRB, 0.57322E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.54317E+00_JPRB, 0.53599E+00_JPRB, 0.53592E+00_JPRB, 0.55685E+00_JPRB, 0.55314E+00_JPRB, 0.56828E+00_JPRB, 0.58999E+00_JPRB ,&
&  0.64352E+00_JPRB, 0.69246E+00_JPRB, 0.73717E+00_JPRB, 0.73460E+00_JPRB, 0.73247E+00_JPRB, 0.71833E+00_JPRB, 0.72470E+00_JPRB ,&
&  0.74148E+00_JPRB, 0.76787E+00_JPRB, 0.79663E+00_JPRB, 0.81106E+00_JPRB, 0.82258E+00_JPRB, 0.83625E+00_JPRB, 0.85625E+00_JPRB ,&
&  0.88713E+00_JPRB, 0.93379E+00_JPRB, 0.10235E+01_JPRB, 0.11930E+01_JPRB, 0.14691E+01_JPRB, 0.17640E+01_JPRB, 0.19856E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.59982E+00_JPRB, 0.16019E+01_JPRB, 0.20780E+01_JPRB, 0.20291E+01_JPRB, 0.19398E+01_JPRB, 0.18450E+01_JPRB ,&
&  0.16085E+01_JPRB, 0.14619E+01_JPRB, 0.12951E+01_JPRB, 0.13746E+01_JPRB, 0.12456E+01_JPRB, 0.12390E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.13383E+01_JPRB, 0.13939E+01_JPRB, 0.14462E+01_JPRB, 0.14597E+01_JPRB, 0.14542E+01_JPRB, 0.14748E+01_JPRB, 0.15401E+01_JPRB ,&
&  0.16711E+01_JPRB, 0.18847E+01_JPRB, 0.21483E+01_JPRB, 0.22031E+01_JPRB, 0.24921E+01_JPRB, 0.23815E+01_JPRB, 0.25384E+01_JPRB ,&
&  0.18057E+01_JPRB, 0.13599E+01_JPRB, 0.13041E+01_JPRB, 0.13377E+01_JPRB, 0.11663E+01_JPRB, 0.98812E+00_JPRB, 0.98074E+00_JPRB ,&
&  0.86685E+00_JPRB, 0.81397E+00_JPRB, 0.80178E+00_JPRB, 0.78278E+00_JPRB, 0.76656E+00_JPRB, 0.79420E+00_JPRB, 0.83466E+00_JPRB ,&
&  0.87692E+00_JPRB, 0.89830E+00_JPRB, 0.89853E+00_JPRB, 0.90122E+00_JPRB, 0.90294E+00_JPRB, 0.89576E+00_JPRB, 0.88860E+00_JPRB ,&
&  0.90941E+00_JPRB, 0.93468E+00_JPRB, 0.96431E+00_JPRB, 0.99544E+00_JPRB, 0.10309E+01_JPRB, 0.10290E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.10076E+01_JPRB, 0.93784E+00_JPRB, 0.92358E+00_JPRB, 0.88746E+00_JPRB, 0.84076E+00_JPRB, 0.80232E+00_JPRB ,&
&  0.74118E+00_JPRB, 0.64901E+00_JPRB, 0.58889E+00_JPRB, 0.56959E+00_JPRB, 0.55085E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.57724E+00_JPRB, 0.55917E+00_JPRB, 0.55792E+00_JPRB, 0.56821E+00_JPRB, 0.57608E+00_JPRB, 0.58367E+00_JPRB, 0.56915E+00_JPRB ,&
&  0.58595E+00_JPRB, 0.62684E+00_JPRB, 0.65253E+00_JPRB, 0.67706E+00_JPRB, 0.67122E+00_JPRB, 0.65781E+00_JPRB, 0.65019E+00_JPRB ,&
&  0.64869E+00_JPRB, 0.65399E+00_JPRB, 0.67248E+00_JPRB, 0.68228E+00_JPRB, 0.69046E+00_JPRB, 0.70313E+00_JPRB, 0.71125E+00_JPRB ,&
&  0.72028E+00_JPRB, 0.73718E+00_JPRB, 0.77347E+00_JPRB, 0.84922E+00_JPRB, 0.99191E+00_JPRB, 0.12572E+01_JPRB, 0.16511E+01_JPRB ,&
&  0.23625E+01_JPRB, 0.17076E+01_JPRB, 0.78880E+00_JPRB, 0.17199E+01_JPRB, 0.21800E+01_JPRB, 0.21906E+01_JPRB, 0.18700E+01_JPRB ,&
&  0.17420E+01_JPRB, 0.15014E+01_JPRB, 0.13350E+01_JPRB, 0.13817E+01_JPRB, 0.11909E+01_JPRB, 0.11133E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.11977E+01_JPRB, 0.12719E+01_JPRB, 0.13258E+01_JPRB, 0.13488E+01_JPRB, 0.13481E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.14424E+01_JPRB, 0.15751E+01_JPRB, 0.18139E+01_JPRB, 0.21729E+01_JPRB, 0.29822E+01_JPRB, 0.28070E+01_JPRB, 0.27921E+01_JPRB ,&
&  0.18957E+01_JPRB, 0.16861E+01_JPRB, 0.16105E+01_JPRB, 0.15058E+01_JPRB, 0.12645E+01_JPRB, 0.11146E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.97112E+00_JPRB, 0.87527E+00_JPRB, 0.84358E+00_JPRB, 0.81304E+00_JPRB, 0.79371E+00_JPRB, 0.80323E+00_JPRB, 0.83434E+00_JPRB ,&
&  0.87482E+00_JPRB, 0.88733E+00_JPRB, 0.89187E+00_JPRB, 0.89593E+00_JPRB, 0.90060E+00_JPRB, 0.90528E+00_JPRB, 0.90577E+00_JPRB ,&
&  0.92032E+00_JPRB, 0.95183E+00_JPRB, 0.99026E+00_JPRB, 0.10013E+01_JPRB, 0.99322E+00_JPRB, 0.10043E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.11084E+01_JPRB, 0.10035E+01_JPRB, 0.97983E+00_JPRB, 0.92164E+00_JPRB, 0.88774E+00_JPRB, 0.85991E+00_JPRB ,&
&  0.82693E+00_JPRB, 0.74978E+00_JPRB, 0.66705E+00_JPRB, 0.58842E+00_JPRB, 0.57897E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.63169E+00_JPRB, 0.61251E+00_JPRB, 0.61460E+00_JPRB, 0.60825E+00_JPRB, 0.59672E+00_JPRB, 0.59187E+00_JPRB, 0.58487E+00_JPRB ,&
&  0.57691E+00_JPRB, 0.57533E+00_JPRB, 0.57360E+00_JPRB, 0.60537E+00_JPRB, 0.62095E+00_JPRB, 0.60976E+00_JPRB, 0.59800E+00_JPRB ,&
&  0.58913E+00_JPRB, 0.58965E+00_JPRB, 0.59557E+00_JPRB, 0.60078E+00_JPRB, 0.60463E+00_JPRB, 0.61398E+00_JPRB, 0.62184E+00_JPRB ,&
&  0.61885E+00_JPRB, 0.62095E+00_JPRB, 0.63372E+00_JPRB, 0.66333E+00_JPRB, 0.72271E+00_JPRB, 0.83206E+00_JPRB, 0.10374E+01_JPRB ,&
&  0.15453E+01_JPRB, 0.15995E+01_JPRB, 0.74838E+00_JPRB, 0.13985E+01_JPRB, 0.22463E+01_JPRB, 0.23571E+01_JPRB, 0.22319E+01_JPRB ,&
&  0.20308E+01_JPRB, 0.16146E+01_JPRB, 0.14534E+01_JPRB, 0.14370E+01_JPRB, 0.12406E+01_JPRB, 0.10945E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.96549E+00_JPRB, 0.97606E+00_JPRB, 0.10384E+01_JPRB, 0.11066E+01_JPRB, 0.11722E+01_JPRB, 0.12059E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.12380E+01_JPRB, 0.13028E+01_JPRB, 0.14521E+01_JPRB, 0.16185E+01_JPRB, 0.24922E+01_JPRB, 0.29737E+01_JPRB, 0.31136E+01_JPRB ,&
&  0.23984E+01_JPRB, 0.21629E+01_JPRB, 0.19477E+01_JPRB, 0.17093E+01_JPRB, 0.14084E+01_JPRB, 0.12998E+01_JPRB, 0.12929E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.97531E+00_JPRB, 0.91607E+00_JPRB, 0.86869E+00_JPRB, 0.82938E+00_JPRB, 0.81259E+00_JPRB, 0.80909E+00_JPRB ,&
&  0.82724E+00_JPRB, 0.84204E+00_JPRB, 0.84334E+00_JPRB, 0.84703E+00_JPRB, 0.85962E+00_JPRB, 0.87283E+00_JPRB, 0.89332E+00_JPRB ,&
&  0.90432E+00_JPRB, 0.93444E+00_JPRB, 0.97494E+00_JPRB, 0.98099E+00_JPRB, 0.10058E+01_JPRB, 0.10594E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.11743E+01_JPRB, 0.10490E+01_JPRB, 0.96969E+00_JPRB, 0.96964E+00_JPRB, 0.93653E+00_JPRB, 0.90478E+00_JPRB ,&
&  0.94278E+00_JPRB, 0.10544E+01_JPRB, 0.10080E+01_JPRB, 0.69972E+00_JPRB, 0.64765E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.66684E+00_JPRB, 0.63920E+00_JPRB, 0.63409E+00_JPRB, 0.62927E+00_JPRB, 0.61642E+00_JPRB, 0.60634E+00_JPRB, 0.59923E+00_JPRB ,&
&  0.58765E+00_JPRB, 0.56773E+00_JPRB, 0.55067E+00_JPRB, 0.55366E+00_JPRB, 0.56835E+00_JPRB, 0.57929E+00_JPRB, 0.56886E+00_JPRB ,&
&  0.55639E+00_JPRB, 0.55187E+00_JPRB, 0.56124E+00_JPRB, 0.56605E+00_JPRB, 0.56482E+00_JPRB, 0.56868E+00_JPRB, 0.56798E+00_JPRB ,&
&  0.56625E+00_JPRB, 0.56247E+00_JPRB, 0.56507E+00_JPRB, 0.57939E+00_JPRB, 0.60654E+00_JPRB, 0.65897E+00_JPRB, 0.74287E+00_JPRB ,&
&  0.10295E+01_JPRB, 0.12780E+01_JPRB, 0.16104E+01_JPRB, 0.24397E+01_JPRB, 0.29928E+01_JPRB, 0.28447E+01_JPRB, 0.26747E+01_JPRB ,&
&  0.25130E+01_JPRB, 0.21323E+01_JPRB, 0.18083E+01_JPRB, 0.15606E+01_JPRB, 0.13125E+01_JPRB, 0.11278E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.94583E+00_JPRB, 0.90704E+00_JPRB, 0.90358E+00_JPRB, 0.92579E+00_JPRB, 0.96332E+00_JPRB, 0.10063E+01_JPRB, 0.10316E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10979E+01_JPRB, 0.11986E+01_JPRB, 0.12996E+01_JPRB, 0.17665E+01_JPRB, 0.26038E+01_JPRB, 0.30352E+01_JPRB ,&
&  0.30474E+01_JPRB, 0.27734E+01_JPRB, 0.23628E+01_JPRB, 0.19725E+01_JPRB, 0.15429E+01_JPRB, 0.15692E+01_JPRB, 0.14281E+01_JPRB ,&
&  0.12566E+01_JPRB, 0.11191E+01_JPRB, 0.10211E+01_JPRB, 0.94754E+00_JPRB, 0.88130E+00_JPRB, 0.82674E+00_JPRB, 0.79860E+00_JPRB ,&
&  0.79451E+00_JPRB, 0.80122E+00_JPRB, 0.79866E+00_JPRB, 0.79232E+00_JPRB, 0.80940E+00_JPRB, 0.82673E+00_JPRB, 0.85277E+00_JPRB ,&
&  0.87667E+00_JPRB, 0.90441E+00_JPRB, 0.89739E+00_JPRB, 0.93165E+00_JPRB, 0.97991E+00_JPRB, 0.10133E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.98062E+00_JPRB, 0.10320E+01_JPRB, 0.10301E+01_JPRB, 0.10045E+01_JPRB, 0.98217E+00_JPRB, 0.96599E+00_JPRB, 0.91787E+00_JPRB ,&
&  0.92125E+00_JPRB, 0.89447E+00_JPRB, 0.83276E+00_JPRB, 0.73892E+00_JPRB, 0.69862E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.70487E+00_JPRB, 0.67582E+00_JPRB, 0.65685E+00_JPRB, 0.64437E+00_JPRB, 0.63817E+00_JPRB, 0.63676E+00_JPRB, 0.62951E+00_JPRB ,&
&  0.62006E+00_JPRB, 0.60060E+00_JPRB, 0.57575E+00_JPRB, 0.55396E+00_JPRB, 0.55574E+00_JPRB, 0.55477E+00_JPRB, 0.55311E+00_JPRB ,&
&  0.54269E+00_JPRB, 0.53911E+00_JPRB, 0.54207E+00_JPRB, 0.54204E+00_JPRB, 0.53924E+00_JPRB, 0.53633E+00_JPRB, 0.53584E+00_JPRB ,&
&  0.54046E+00_JPRB, 0.54228E+00_JPRB, 0.54179E+00_JPRB, 0.54499E+00_JPRB, 0.55663E+00_JPRB, 0.58878E+00_JPRB, 0.64616E+00_JPRB ,&
&  0.81267E+00_JPRB, 0.11734E+01_JPRB, 0.13859E+01_JPRB, 0.27999E+01_JPRB, 0.32401E+01_JPRB, 0.31587E+01_JPRB, 0.27994E+01_JPRB ,&
&  0.24648E+01_JPRB, 0.24146E+01_JPRB, 0.19821E+01_JPRB, 0.16013E+01_JPRB, 0.12936E+01_JPRB, 0.11204E+01_JPRB, 0.99581E+00_JPRB ,&
&  0.91194E+00_JPRB, 0.86725E+00_JPRB, 0.85255E+00_JPRB, 0.84751E+00_JPRB, 0.84202E+00_JPRB, 0.85336E+00_JPRB, 0.86785E+00_JPRB ,&
&  0.88674E+00_JPRB, 0.94430E+00_JPRB, 0.10535E+01_JPRB, 0.11320E+01_JPRB, 0.14025E+01_JPRB, 0.20546E+01_JPRB, 0.26177E+01_JPRB ,&
&  0.28959E+01_JPRB, 0.27806E+01_JPRB, 0.24911E+01_JPRB, 0.20967E+01_JPRB, 0.17634E+01_JPRB, 0.16362E+01_JPRB, 0.14915E+01_JPRB ,&
&  0.13432E+01_JPRB, 0.12405E+01_JPRB, 0.11037E+01_JPRB, 0.10093E+01_JPRB, 0.96329E+00_JPRB, 0.88947E+00_JPRB, 0.82849E+00_JPRB ,&
&  0.79474E+00_JPRB, 0.77965E+00_JPRB, 0.77155E+00_JPRB, 0.76411E+00_JPRB, 0.76870E+00_JPRB, 0.78470E+00_JPRB, 0.80727E+00_JPRB ,&
&  0.83298E+00_JPRB, 0.86710E+00_JPRB, 0.84835E+00_JPRB, 0.85757E+00_JPRB, 0.90236E+00_JPRB, 0.94328E+00_JPRB, 0.94717E+00_JPRB ,&
&  0.95250E+00_JPRB, 0.97239E+00_JPRB, 0.98829E+00_JPRB, 0.97215E+00_JPRB, 0.94534E+00_JPRB, 0.10095E+01_JPRB, 0.93939E+00_JPRB ,&
&  0.92444E+00_JPRB, 0.84249E+00_JPRB, 0.80092E+00_JPRB, 0.75822E+00_JPRB, 0.73318E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.72024E+00_JPRB, 0.70056E+00_JPRB, 0.68504E+00_JPRB, 0.66990E+00_JPRB, 0.65925E+00_JPRB, 0.65256E+00_JPRB, 0.64411E+00_JPRB ,&
&  0.63039E+00_JPRB, 0.61176E+00_JPRB, 0.59147E+00_JPRB, 0.56363E+00_JPRB, 0.54824E+00_JPRB, 0.54618E+00_JPRB, 0.55262E+00_JPRB ,&
&  0.54438E+00_JPRB, 0.53000E+00_JPRB, 0.53053E+00_JPRB, 0.53371E+00_JPRB, 0.53262E+00_JPRB, 0.52598E+00_JPRB, 0.51899E+00_JPRB ,&
&  0.52262E+00_JPRB, 0.52769E+00_JPRB, 0.53121E+00_JPRB, 0.53357E+00_JPRB, 0.53717E+00_JPRB, 0.55069E+00_JPRB, 0.59494E+00_JPRB ,&
&  0.73827E+00_JPRB, 0.14673E+01_JPRB, 0.14528E+01_JPRB, 0.26025E+01_JPRB, 0.28046E+01_JPRB, 0.27586E+01_JPRB, 0.25213E+01_JPRB ,&
&  0.22634E+01_JPRB, 0.20841E+01_JPRB, 0.17372E+01_JPRB, 0.14776E+01_JPRB, 0.12382E+01_JPRB, 0.10606E+01_JPRB, 0.94684E+00_JPRB ,&
&  0.87402E+00_JPRB, 0.82075E+00_JPRB, 0.78977E+00_JPRB, 0.77944E+00_JPRB, 0.77423E+00_JPRB, 0.77183E+00_JPRB, 0.77591E+00_JPRB ,&
&  0.78924E+00_JPRB, 0.83207E+00_JPRB, 0.93992E+00_JPRB, 0.11667E+01_JPRB, 0.13106E+01_JPRB, 0.17644E+01_JPRB, 0.24493E+01_JPRB ,&
&  0.28382E+01_JPRB, 0.27616E+01_JPRB, 0.23570E+01_JPRB, 0.20037E+01_JPRB, 0.17453E+01_JPRB, 0.15698E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.12413E+01_JPRB, 0.11591E+01_JPRB, 0.10579E+01_JPRB, 0.99109E+00_JPRB, 0.94166E+00_JPRB, 0.88516E+00_JPRB ,&
&  0.82688E+00_JPRB, 0.78280E+00_JPRB, 0.75831E+00_JPRB, 0.74099E+00_JPRB, 0.73397E+00_JPRB, 0.74444E+00_JPRB, 0.76164E+00_JPRB ,&
&  0.78014E+00_JPRB, 0.81117E+00_JPRB, 0.80633E+00_JPRB, 0.81571E+00_JPRB, 0.84780E+00_JPRB, 0.88539E+00_JPRB, 0.89170E+00_JPRB ,&
&  0.92070E+00_JPRB, 0.94645E+00_JPRB, 0.97271E+00_JPRB, 0.10151E+01_JPRB, 0.99705E+00_JPRB, 0.11010E+01_JPRB, 0.94624E+00_JPRB ,&
&  0.91863E+00_JPRB, 0.84847E+00_JPRB, 0.79317E+00_JPRB, 0.77200E+00_JPRB, 0.74872E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.72620E+00_JPRB, 0.70780E+00_JPRB, 0.69412E+00_JPRB, 0.68634E+00_JPRB, 0.67925E+00_JPRB, 0.66567E+00_JPRB, 0.65200E+00_JPRB ,&
&  0.63859E+00_JPRB, 0.62405E+00_JPRB, 0.60240E+00_JPRB, 0.57102E+00_JPRB, 0.54712E+00_JPRB, 0.53753E+00_JPRB, 0.54161E+00_JPRB ,&
&  0.54238E+00_JPRB, 0.53385E+00_JPRB, 0.52788E+00_JPRB, 0.52786E+00_JPRB, 0.52804E+00_JPRB, 0.52202E+00_JPRB, 0.51322E+00_JPRB ,&
&  0.50647E+00_JPRB, 0.50787E+00_JPRB, 0.51176E+00_JPRB, 0.51682E+00_JPRB, 0.52068E+00_JPRB, 0.52727E+00_JPRB, 0.55728E+00_JPRB ,&
&  0.69763E+00_JPRB, 0.73062E+00_JPRB, 0.16853E+01_JPRB, 0.22267E+01_JPRB, 0.22562E+01_JPRB, 0.21778E+01_JPRB, 0.21608E+01_JPRB ,&
&  0.19994E+01_JPRB, 0.17578E+01_JPRB, 0.14752E+01_JPRB, 0.12440E+01_JPRB, 0.10734E+01_JPRB, 0.96104E+00_JPRB, 0.87843E+00_JPRB ,&
&  0.81575E+00_JPRB, 0.76893E+00_JPRB, 0.73308E+00_JPRB, 0.71413E+00_JPRB, 0.71766E+00_JPRB, 0.72431E+00_JPRB, 0.73001E+00_JPRB ,&
&  0.73983E+00_JPRB, 0.75926E+00_JPRB, 0.81789E+00_JPRB, 0.99639E+00_JPRB, 0.13639E+01_JPRB, 0.17075E+01_JPRB, 0.22832E+01_JPRB ,&
&  0.28362E+01_JPRB, 0.25424E+01_JPRB, 0.20838E+01_JPRB, 0.17732E+01_JPRB, 0.15553E+01_JPRB, 0.14050E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.12061E+01_JPRB, 0.11448E+01_JPRB, 0.10959E+01_JPRB, 0.10358E+01_JPRB, 0.98860E+00_JPRB, 0.95396E+00_JPRB, 0.91501E+00_JPRB ,&
&  0.86949E+00_JPRB, 0.82475E+00_JPRB, 0.78185E+00_JPRB, 0.74372E+00_JPRB, 0.71329E+00_JPRB, 0.70907E+00_JPRB, 0.71674E+00_JPRB ,&
&  0.73015E+00_JPRB, 0.75828E+00_JPRB, 0.77956E+00_JPRB, 0.79565E+00_JPRB, 0.83254E+00_JPRB, 0.85768E+00_JPRB, 0.88387E+00_JPRB ,&
&  0.89471E+00_JPRB, 0.92266E+00_JPRB, 0.94299E+00_JPRB, 0.99458E+00_JPRB, 0.96524E+00_JPRB, 0.10383E+01_JPRB, 0.92997E+00_JPRB ,&
&  0.89668E+00_JPRB, 0.86736E+00_JPRB, 0.81242E+00_JPRB, 0.78427E+00_JPRB, 0.75648E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.73522E+00_JPRB, 0.70907E+00_JPRB, 0.69613E+00_JPRB, 0.68803E+00_JPRB, 0.68098E+00_JPRB, 0.67413E+00_JPRB, 0.65972E+00_JPRB ,&
&  0.64105E+00_JPRB, 0.62982E+00_JPRB, 0.61644E+00_JPRB, 0.59854E+00_JPRB, 0.57243E+00_JPRB, 0.55389E+00_JPRB, 0.55002E+00_JPRB ,&
&  0.54725E+00_JPRB, 0.54171E+00_JPRB, 0.53133E+00_JPRB, 0.52571E+00_JPRB, 0.51938E+00_JPRB, 0.51161E+00_JPRB, 0.50674E+00_JPRB ,&
&  0.49663E+00_JPRB, 0.49244E+00_JPRB, 0.49513E+00_JPRB, 0.50138E+00_JPRB, 0.50676E+00_JPRB, 0.51372E+00_JPRB, 0.52580E+00_JPRB ,&
&  0.63918E+00_JPRB, 0.11311E+01_JPRB, 0.15655E+01_JPRB, 0.18055E+01_JPRB, 0.18247E+01_JPRB, 0.17060E+01_JPRB, 0.17010E+01_JPRB ,&
&  0.15327E+01_JPRB, 0.13661E+01_JPRB, 0.11938E+01_JPRB, 0.10338E+01_JPRB, 0.90964E+00_JPRB, 0.84369E+00_JPRB, 0.80702E+00_JPRB ,&
&  0.76535E+00_JPRB, 0.72881E+00_JPRB, 0.69706E+00_JPRB, 0.67006E+00_JPRB, 0.66080E+00_JPRB, 0.67282E+00_JPRB, 0.68979E+00_JPRB ,&
&  0.69733E+00_JPRB, 0.70545E+00_JPRB, 0.73247E+00_JPRB, 0.84162E+00_JPRB, 0.11643E+01_JPRB, 0.13399E+01_JPRB, 0.18894E+01_JPRB ,&
&  0.21894E+01_JPRB, 0.19572E+01_JPRB, 0.16596E+01_JPRB, 0.14481E+01_JPRB, 0.12818E+01_JPRB, 0.11571E+01_JPRB, 0.10675E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.98135E+00_JPRB, 0.95296E+00_JPRB, 0.92456E+00_JPRB, 0.89874E+00_JPRB, 0.88006E+00_JPRB, 0.86462E+00_JPRB ,&
&  0.84585E+00_JPRB, 0.82182E+00_JPRB, 0.79282E+00_JPRB, 0.75604E+00_JPRB, 0.71753E+00_JPRB, 0.69506E+00_JPRB, 0.68519E+00_JPRB ,&
&  0.68421E+00_JPRB, 0.71002E+00_JPRB, 0.75665E+00_JPRB, 0.76190E+00_JPRB, 0.79999E+00_JPRB, 0.83011E+00_JPRB, 0.85801E+00_JPRB ,&
&  0.84734E+00_JPRB, 0.87426E+00_JPRB, 0.88550E+00_JPRB, 0.94355E+00_JPRB, 0.10729E+01_JPRB, 0.10215E+01_JPRB, 0.89580E+00_JPRB ,&
&  0.84927E+00_JPRB, 0.82626E+00_JPRB, 0.80232E+00_JPRB, 0.77428E+00_JPRB, 0.77044E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.76074E+00_JPRB, 0.72153E+00_JPRB, 0.70028E+00_JPRB, 0.68804E+00_JPRB, 0.68107E+00_JPRB, 0.67413E+00_JPRB, 0.66063E+00_JPRB ,&
&  0.64281E+00_JPRB, 0.62680E+00_JPRB, 0.61449E+00_JPRB, 0.60567E+00_JPRB, 0.58744E+00_JPRB, 0.55833E+00_JPRB, 0.53609E+00_JPRB ,&
&  0.53045E+00_JPRB, 0.52593E+00_JPRB, 0.52180E+00_JPRB, 0.51328E+00_JPRB, 0.50564E+00_JPRB, 0.50216E+00_JPRB, 0.49802E+00_JPRB ,&
&  0.49320E+00_JPRB, 0.48863E+00_JPRB, 0.48978E+00_JPRB, 0.49394E+00_JPRB, 0.49772E+00_JPRB, 0.50108E+00_JPRB, 0.50728E+00_JPRB ,&
&  0.56344E+00_JPRB, 0.86212E+00_JPRB, 0.10550E+01_JPRB, 0.13344E+01_JPRB, 0.14194E+01_JPRB, 0.13376E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.10981E+01_JPRB, 0.10044E+01_JPRB, 0.91538E+00_JPRB, 0.81756E+00_JPRB, 0.76058E+00_JPRB, 0.73388E+00_JPRB ,&
&  0.71474E+00_JPRB, 0.68954E+00_JPRB, 0.66374E+00_JPRB, 0.63741E+00_JPRB, 0.62008E+00_JPRB, 0.62427E+00_JPRB, 0.63691E+00_JPRB ,&
&  0.63737E+00_JPRB, 0.63860E+00_JPRB, 0.64364E+00_JPRB, 0.68708E+00_JPRB, 0.86373E+00_JPRB, 0.10331E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.13192E+01_JPRB, 0.12182E+01_JPRB, 0.11120E+01_JPRB, 0.10237E+01_JPRB, 0.95522E+00_JPRB, 0.90338E+00_JPRB ,&
&  0.86917E+00_JPRB, 0.84308E+00_JPRB, 0.82344E+00_JPRB, 0.80763E+00_JPRB, 0.79407E+00_JPRB, 0.77886E+00_JPRB, 0.77280E+00_JPRB ,&
&  0.77086E+00_JPRB, 0.75994E+00_JPRB, 0.74612E+00_JPRB, 0.72923E+00_JPRB, 0.70644E+00_JPRB, 0.68626E+00_JPRB, 0.67075E+00_JPRB ,&
&  0.65723E+00_JPRB, 0.65411E+00_JPRB, 0.68070E+00_JPRB, 0.71047E+00_JPRB, 0.73241E+00_JPRB, 0.75252E+00_JPRB, 0.76769E+00_JPRB ,&
&  0.78446E+00_JPRB, 0.83405E+00_JPRB, 0.82356E+00_JPRB, 0.87984E+00_JPRB, 0.98948E+00_JPRB, 0.93437E+00_JPRB, 0.84355E+00_JPRB ,&
&  0.80183E+00_JPRB, 0.77596E+00_JPRB, 0.76054E+00_JPRB, 0.75286E+00_JPRB, 0.79923E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.80362E+00_JPRB, 0.76124E+00_JPRB, 0.72166E+00_JPRB, 0.69679E+00_JPRB, 0.67940E+00_JPRB, 0.66515E+00_JPRB, 0.64753E+00_JPRB ,&
&  0.63077E+00_JPRB, 0.61433E+00_JPRB, 0.60100E+00_JPRB, 0.59135E+00_JPRB, 0.58163E+00_JPRB, 0.56079E+00_JPRB, 0.53249E+00_JPRB ,&
&  0.51358E+00_JPRB, 0.50863E+00_JPRB, 0.50519E+00_JPRB, 0.49804E+00_JPRB, 0.49126E+00_JPRB, 0.48869E+00_JPRB, 0.48569E+00_JPRB ,&
&  0.48544E+00_JPRB, 0.48573E+00_JPRB, 0.48867E+00_JPRB, 0.48982E+00_JPRB, 0.48714E+00_JPRB, 0.49307E+00_JPRB, 0.49308E+00_JPRB ,&
&  0.53471E+00_JPRB, 0.63193E+00_JPRB, 0.76884E+00_JPRB, 0.92470E+00_JPRB, 0.10361E+01_JPRB, 0.10730E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.10161E+01_JPRB, 0.97664E+00_JPRB, 0.91825E+00_JPRB, 0.86007E+00_JPRB, 0.79449E+00_JPRB, 0.72455E+00_JPRB, 0.68359E+00_JPRB ,&
&  0.66990E+00_JPRB, 0.65427E+00_JPRB, 0.63879E+00_JPRB, 0.62063E+00_JPRB, 0.60390E+00_JPRB, 0.59834E+00_JPRB, 0.59975E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.59145E+00_JPRB, 0.58604E+00_JPRB, 0.58955E+00_JPRB, 0.62224E+00_JPRB, 0.70963E+00_JPRB, 0.82042E+00_JPRB ,&
&  0.86273E+00_JPRB, 0.88971E+00_JPRB, 0.89817E+00_JPRB, 0.88496E+00_JPRB, 0.85177E+00_JPRB, 0.81134E+00_JPRB, 0.76961E+00_JPRB ,&
&  0.73937E+00_JPRB, 0.71935E+00_JPRB, 0.70967E+00_JPRB, 0.70506E+00_JPRB, 0.70226E+00_JPRB, 0.69736E+00_JPRB, 0.68952E+00_JPRB ,&
&  0.68405E+00_JPRB, 0.68172E+00_JPRB, 0.68170E+00_JPRB, 0.67741E+00_JPRB, 0.67131E+00_JPRB, 0.66338E+00_JPRB, 0.65665E+00_JPRB ,&
&  0.65090E+00_JPRB, 0.64657E+00_JPRB, 0.64973E+00_JPRB, 0.65907E+00_JPRB, 0.67450E+00_JPRB, 0.68682E+00_JPRB, 0.69639E+00_JPRB ,&
&  0.70563E+00_JPRB, 0.73716E+00_JPRB, 0.74722E+00_JPRB, 0.85786E+00_JPRB, 0.93110E+00_JPRB, 0.85455E+00_JPRB, 0.79171E+00_JPRB ,&
&  0.74060E+00_JPRB, 0.71510E+00_JPRB, 0.70986E+00_JPRB, 0.72894E+00_JPRB, 0.79751E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.68256E+00_JPRB, 0.67468E+00_JPRB, 0.66464E+00_JPRB, 0.65572E+00_JPRB, 0.64186E+00_JPRB, 0.62928E+00_JPRB, 0.61326E+00_JPRB ,&
&  0.59823E+00_JPRB, 0.58368E+00_JPRB, 0.57120E+00_JPRB, 0.56642E+00_JPRB, 0.55863E+00_JPRB, 0.54732E+00_JPRB, 0.52663E+00_JPRB ,&
&  0.50933E+00_JPRB, 0.49611E+00_JPRB, 0.48815E+00_JPRB, 0.48456E+00_JPRB, 0.48201E+00_JPRB, 0.47835E+00_JPRB, 0.47668E+00_JPRB ,&
&  0.47729E+00_JPRB, 0.47670E+00_JPRB, 0.47857E+00_JPRB, 0.47750E+00_JPRB, 0.47986E+00_JPRB, 0.47551E+00_JPRB, 0.47219E+00_JPRB ,&
&  0.49819E+00_JPRB, 0.52989E+00_JPRB, 0.60984E+00_JPRB, 0.69761E+00_JPRB, 0.79063E+00_JPRB, 0.85577E+00_JPRB, 0.90517E+00_JPRB ,&
&  0.93208E+00_JPRB, 0.90569E+00_JPRB, 0.85966E+00_JPRB, 0.80022E+00_JPRB, 0.75394E+00_JPRB, 0.70493E+00_JPRB, 0.65590E+00_JPRB ,&
&  0.63135E+00_JPRB, 0.61824E+00_JPRB, 0.60756E+00_JPRB, 0.59844E+00_JPRB, 0.58982E+00_JPRB, 0.58475E+00_JPRB, 0.58185E+00_JPRB ,&
&  0.57445E+00_JPRB, 0.56518E+00_JPRB, 0.55400E+00_JPRB, 0.54974E+00_JPRB, 0.55183E+00_JPRB, 0.57653E+00_JPRB, 0.63027E+00_JPRB ,&
&  0.65545E+00_JPRB, 0.66907E+00_JPRB, 0.67897E+00_JPRB, 0.69428E+00_JPRB, 0.69929E+00_JPRB, 0.69367E+00_JPRB, 0.66746E+00_JPRB ,&
&  0.64437E+00_JPRB, 0.62773E+00_JPRB, 0.61925E+00_JPRB, 0.61606E+00_JPRB, 0.61706E+00_JPRB, 0.61720E+00_JPRB, 0.61811E+00_JPRB ,&
&  0.61356E+00_JPRB, 0.60803E+00_JPRB, 0.60640E+00_JPRB, 0.60681E+00_JPRB, 0.60837E+00_JPRB, 0.60952E+00_JPRB, 0.61210E+00_JPRB ,&
&  0.61785E+00_JPRB, 0.62563E+00_JPRB, 0.63440E+00_JPRB, 0.63644E+00_JPRB, 0.63086E+00_JPRB, 0.62882E+00_JPRB, 0.63397E+00_JPRB ,&
&  0.64201E+00_JPRB, 0.66276E+00_JPRB, 0.68225E+00_JPRB, 0.75878E+00_JPRB, 0.78736E+00_JPRB, 0.76374E+00_JPRB, 0.71344E+00_JPRB ,&
&  0.67361E+00_JPRB, 0.65345E+00_JPRB, 0.65116E+00_JPRB, 0.66763E+00_JPRB, 0.65754E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.61247E+00_JPRB, 0.60953E+00_JPRB, 0.60481E+00_JPRB, 0.60143E+00_JPRB, 0.59336E+00_JPRB, 0.58457E+00_JPRB, 0.57546E+00_JPRB ,&
&  0.56373E+00_JPRB, 0.55169E+00_JPRB, 0.54124E+00_JPRB, 0.53707E+00_JPRB, 0.53397E+00_JPRB, 0.53102E+00_JPRB, 0.52326E+00_JPRB ,&
&  0.51189E+00_JPRB, 0.50112E+00_JPRB, 0.49164E+00_JPRB, 0.48544E+00_JPRB, 0.48314E+00_JPRB, 0.48024E+00_JPRB, 0.47641E+00_JPRB ,&
&  0.47365E+00_JPRB, 0.47352E+00_JPRB, 0.47209E+00_JPRB, 0.47045E+00_JPRB, 0.46821E+00_JPRB, 0.46243E+00_JPRB, 0.46167E+00_JPRB ,&
&  0.47399E+00_JPRB, 0.45119E+00_JPRB, 0.49441E+00_JPRB, 0.53613E+00_JPRB, 0.57760E+00_JPRB, 0.64473E+00_JPRB, 0.72792E+00_JPRB ,&
&  0.79824E+00_JPRB, 0.80967E+00_JPRB, 0.79079E+00_JPRB, 0.74817E+00_JPRB, 0.71639E+00_JPRB, 0.69298E+00_JPRB, 0.65912E+00_JPRB ,&
&  0.62557E+00_JPRB, 0.60289E+00_JPRB, 0.58634E+00_JPRB, 0.57299E+00_JPRB, 0.56354E+00_JPRB, 0.55554E+00_JPRB, 0.54824E+00_JPRB ,&
&  0.53859E+00_JPRB, 0.52898E+00_JPRB, 0.52078E+00_JPRB, 0.51696E+00_JPRB, 0.51896E+00_JPRB, 0.52593E+00_JPRB, 0.54525E+00_JPRB ,&
&  0.56045E+00_JPRB, 0.56831E+00_JPRB, 0.56873E+00_JPRB, 0.57470E+00_JPRB, 0.58487E+00_JPRB, 0.59309E+00_JPRB, 0.59101E+00_JPRB ,&
&  0.57721E+00_JPRB, 0.55967E+00_JPRB, 0.54592E+00_JPRB, 0.53674E+00_JPRB, 0.53257E+00_JPRB, 0.53328E+00_JPRB, 0.53784E+00_JPRB ,&
&  0.53906E+00_JPRB, 0.53785E+00_JPRB, 0.53700E+00_JPRB, 0.53868E+00_JPRB, 0.54180E+00_JPRB, 0.54810E+00_JPRB, 0.55676E+00_JPRB ,&
&  0.56691E+00_JPRB, 0.57819E+00_JPRB, 0.59128E+00_JPRB, 0.60278E+00_JPRB, 0.60735E+00_JPRB, 0.60941E+00_JPRB, 0.60931E+00_JPRB ,&
&  0.61041E+00_JPRB, 0.62006E+00_JPRB, 0.63098E+00_JPRB, 0.70291E+00_JPRB, 0.73395E+00_JPRB, 0.73077E+00_JPRB, 0.69214E+00_JPRB ,&
&  0.65704E+00_JPRB, 0.62459E+00_JPRB, 0.61486E+00_JPRB, 0.56637E+00_JPRB, 0.60442E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.55408E+00_JPRB, 0.55352E+00_JPRB, 0.54922E+00_JPRB, 0.54440E+00_JPRB, 0.54102E+00_JPRB, 0.53584E+00_JPRB, 0.53174E+00_JPRB ,&
&  0.52585E+00_JPRB, 0.52115E+00_JPRB, 0.51352E+00_JPRB, 0.50826E+00_JPRB, 0.50508E+00_JPRB, 0.50397E+00_JPRB, 0.50235E+00_JPRB ,&
&  0.49761E+00_JPRB, 0.48995E+00_JPRB, 0.48219E+00_JPRB, 0.47593E+00_JPRB, 0.47112E+00_JPRB, 0.46599E+00_JPRB, 0.46106E+00_JPRB ,&
&  0.45652E+00_JPRB, 0.45240E+00_JPRB, 0.44865E+00_JPRB, 0.44800E+00_JPRB, 0.44762E+00_JPRB, 0.44638E+00_JPRB, 0.44397E+00_JPRB ,&
&  0.45030E+00_JPRB, 0.40158E+00_JPRB, 0.41711E+00_JPRB, 0.43454E+00_JPRB, 0.45504E+00_JPRB, 0.50496E+00_JPRB, 0.57682E+00_JPRB ,&
&  0.64257E+00_JPRB, 0.68822E+00_JPRB, 0.70079E+00_JPRB, 0.68261E+00_JPRB, 0.66014E+00_JPRB, 0.64582E+00_JPRB, 0.63386E+00_JPRB ,&
&  0.61081E+00_JPRB, 0.59011E+00_JPRB, 0.57005E+00_JPRB, 0.55025E+00_JPRB, 0.53173E+00_JPRB, 0.51562E+00_JPRB, 0.50207E+00_JPRB ,&
&  0.49340E+00_JPRB, 0.48951E+00_JPRB, 0.48720E+00_JPRB, 0.48581E+00_JPRB, 0.48673E+00_JPRB, 0.48929E+00_JPRB, 0.49143E+00_JPRB ,&
&  0.49480E+00_JPRB, 0.50078E+00_JPRB, 0.50222E+00_JPRB, 0.50352E+00_JPRB, 0.50791E+00_JPRB, 0.51645E+00_JPRB, 0.51859E+00_JPRB ,&
&  0.51448E+00_JPRB, 0.50610E+00_JPRB, 0.49676E+00_JPRB, 0.48847E+00_JPRB, 0.48156E+00_JPRB, 0.47536E+00_JPRB, 0.47319E+00_JPRB ,&
&  0.47146E+00_JPRB, 0.47184E+00_JPRB, 0.47313E+00_JPRB, 0.47706E+00_JPRB, 0.48186E+00_JPRB, 0.48733E+00_JPRB, 0.49350E+00_JPRB ,&
&  0.50013E+00_JPRB, 0.50710E+00_JPRB, 0.51656E+00_JPRB, 0.52718E+00_JPRB, 0.53615E+00_JPRB, 0.54162E+00_JPRB, 0.54504E+00_JPRB ,&
&  0.54835E+00_JPRB, 0.55538E+00_JPRB, 0.56009E+00_JPRB, 0.59040E+00_JPRB, 0.61013E+00_JPRB, 0.62679E+00_JPRB, 0.62261E+00_JPRB ,&
&  0.60806E+00_JPRB, 0.59111E+00_JPRB, 0.50862E+00_JPRB, 0.54030E+00_JPRB, 0.54918E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.52078E+00_JPRB, 0.51561E+00_JPRB, 0.50966E+00_JPRB, 0.50257E+00_JPRB, 0.49670E+00_JPRB, 0.49247E+00_JPRB, 0.49033E+00_JPRB ,&
&  0.49060E+00_JPRB, 0.49154E+00_JPRB, 0.48980E+00_JPRB, 0.48405E+00_JPRB, 0.47838E+00_JPRB, 0.47464E+00_JPRB, 0.47271E+00_JPRB ,&
&  0.47007E+00_JPRB, 0.46557E+00_JPRB, 0.45967E+00_JPRB, 0.45393E+00_JPRB, 0.44750E+00_JPRB, 0.44209E+00_JPRB, 0.43750E+00_JPRB ,&
&  0.43365E+00_JPRB, 0.43160E+00_JPRB, 0.42998E+00_JPRB, 0.42886E+00_JPRB, 0.42902E+00_JPRB, 0.42842E+00_JPRB, 0.43252E+00_JPRB ,&
&  0.40631E+00_JPRB, 0.37524E+00_JPRB, 0.38835E+00_JPRB, 0.40098E+00_JPRB, 0.41333E+00_JPRB, 0.44243E+00_JPRB, 0.48543E+00_JPRB ,&
&  0.53751E+00_JPRB, 0.58274E+00_JPRB, 0.60983E+00_JPRB, 0.60791E+00_JPRB, 0.59375E+00_JPRB, 0.58455E+00_JPRB, 0.58004E+00_JPRB ,&
&  0.56827E+00_JPRB, 0.55383E+00_JPRB, 0.54133E+00_JPRB, 0.52647E+00_JPRB, 0.51051E+00_JPRB, 0.49608E+00_JPRB, 0.48454E+00_JPRB ,&
&  0.47569E+00_JPRB, 0.47037E+00_JPRB, 0.46865E+00_JPRB, 0.46678E+00_JPRB, 0.46505E+00_JPRB, 0.46490E+00_JPRB, 0.46376E+00_JPRB ,&
&  0.46345E+00_JPRB, 0.46308E+00_JPRB, 0.46384E+00_JPRB, 0.46393E+00_JPRB, 0.46477E+00_JPRB, 0.46422E+00_JPRB, 0.46224E+00_JPRB ,&
&  0.45892E+00_JPRB, 0.45606E+00_JPRB, 0.45436E+00_JPRB, 0.45053E+00_JPRB, 0.44236E+00_JPRB, 0.43457E+00_JPRB, 0.43049E+00_JPRB ,&
&  0.42949E+00_JPRB, 0.42944E+00_JPRB, 0.42951E+00_JPRB, 0.43117E+00_JPRB, 0.43406E+00_JPRB, 0.43780E+00_JPRB, 0.44252E+00_JPRB ,&
&  0.44822E+00_JPRB, 0.45384E+00_JPRB, 0.45866E+00_JPRB, 0.46368E+00_JPRB, 0.47132E+00_JPRB, 0.47858E+00_JPRB, 0.48567E+00_JPRB ,&
&  0.49286E+00_JPRB, 0.50134E+00_JPRB, 0.51052E+00_JPRB, 0.52581E+00_JPRB, 0.54546E+00_JPRB, 0.55968E+00_JPRB, 0.56361E+00_JPRB ,&
&  0.55878E+00_JPRB, 0.54451E+00_JPRB, 0.53876E+00_JPRB, 0.52956E+00_JPRB, 0.52352E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.47028E+00_JPRB, 0.46695E+00_JPRB, 0.46204E+00_JPRB, 0.45621E+00_JPRB, 0.45017E+00_JPRB, 0.44675E+00_JPRB, 0.44503E+00_JPRB ,&
&  0.44597E+00_JPRB, 0.44557E+00_JPRB, 0.44660E+00_JPRB, 0.44413E+00_JPRB, 0.44101E+00_JPRB, 0.43849E+00_JPRB, 0.43594E+00_JPRB ,&
&  0.43307E+00_JPRB, 0.42944E+00_JPRB, 0.42581E+00_JPRB, 0.42277E+00_JPRB, 0.41921E+00_JPRB, 0.41439E+00_JPRB, 0.40992E+00_JPRB ,&
&  0.40645E+00_JPRB, 0.40469E+00_JPRB, 0.40356E+00_JPRB, 0.40273E+00_JPRB, 0.40197E+00_JPRB, 0.40274E+00_JPRB, 0.40307E+00_JPRB ,&
&  0.40561E+00_JPRB, 0.37640E+00_JPRB, 0.38751E+00_JPRB, 0.39191E+00_JPRB, 0.39993E+00_JPRB, 0.41714E+00_JPRB, 0.43856E+00_JPRB ,&
&  0.46418E+00_JPRB, 0.49159E+00_JPRB, 0.51307E+00_JPRB, 0.52005E+00_JPRB, 0.51511E+00_JPRB, 0.51198E+00_JPRB, 0.51384E+00_JPRB ,&
&  0.51242E+00_JPRB, 0.50753E+00_JPRB, 0.50311E+00_JPRB, 0.49740E+00_JPRB, 0.48884E+00_JPRB, 0.47693E+00_JPRB, 0.46560E+00_JPRB ,&
&  0.45891E+00_JPRB, 0.45505E+00_JPRB, 0.45153E+00_JPRB, 0.44764E+00_JPRB, 0.44157E+00_JPRB, 0.43563E+00_JPRB, 0.43038E+00_JPRB ,&
&  0.42573E+00_JPRB, 0.41986E+00_JPRB, 0.41636E+00_JPRB, 0.41426E+00_JPRB, 0.41121E+00_JPRB, 0.40938E+00_JPRB, 0.40760E+00_JPRB ,&
&  0.40514E+00_JPRB, 0.40207E+00_JPRB, 0.40016E+00_JPRB, 0.39939E+00_JPRB, 0.39724E+00_JPRB, 0.39506E+00_JPRB, 0.39257E+00_JPRB ,&
&  0.39168E+00_JPRB, 0.39125E+00_JPRB, 0.39037E+00_JPRB, 0.39025E+00_JPRB, 0.39133E+00_JPRB, 0.39423E+00_JPRB, 0.39801E+00_JPRB ,&
&  0.40237E+00_JPRB, 0.40739E+00_JPRB, 0.41201E+00_JPRB, 0.41598E+00_JPRB, 0.42143E+00_JPRB, 0.42626E+00_JPRB, 0.43266E+00_JPRB ,&
&  0.44035E+00_JPRB, 0.44849E+00_JPRB, 0.45732E+00_JPRB, 0.46838E+00_JPRB, 0.48037E+00_JPRB, 0.48660E+00_JPRB, 0.49113E+00_JPRB ,&
&  0.49380E+00_JPRB, 0.48873E+00_JPRB, 0.48388E+00_JPRB, 0.47759E+00_JPRB, 0.47220E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.43056E+00_JPRB, 0.42580E+00_JPRB, 0.42035E+00_JPRB, 0.41596E+00_JPRB, 0.41185E+00_JPRB, 0.40732E+00_JPRB, 0.40461E+00_JPRB ,&
&  0.40356E+00_JPRB, 0.40374E+00_JPRB, 0.40519E+00_JPRB, 0.40579E+00_JPRB, 0.40586E+00_JPRB, 0.40643E+00_JPRB, 0.40591E+00_JPRB ,&
&  0.40487E+00_JPRB, 0.40321E+00_JPRB, 0.40102E+00_JPRB, 0.39800E+00_JPRB, 0.39434E+00_JPRB, 0.39088E+00_JPRB, 0.38640E+00_JPRB ,&
&  0.38319E+00_JPRB, 0.38123E+00_JPRB, 0.38010E+00_JPRB, 0.37967E+00_JPRB, 0.38062E+00_JPRB, 0.38197E+00_JPRB, 0.38317E+00_JPRB ,&
&  0.38234E+00_JPRB, 0.37566E+00_JPRB, 0.38171E+00_JPRB, 0.38413E+00_JPRB, 0.39181E+00_JPRB, 0.40556E+00_JPRB, 0.41806E+00_JPRB ,&
&  0.42676E+00_JPRB, 0.43874E+00_JPRB, 0.45201E+00_JPRB, 0.46091E+00_JPRB, 0.46107E+00_JPRB, 0.45975E+00_JPRB, 0.46287E+00_JPRB ,&
&  0.46491E+00_JPRB, 0.46412E+00_JPRB, 0.46361E+00_JPRB, 0.46210E+00_JPRB, 0.45889E+00_JPRB, 0.45497E+00_JPRB, 0.44873E+00_JPRB ,&
&  0.44223E+00_JPRB, 0.43710E+00_JPRB, 0.43171E+00_JPRB, 0.42616E+00_JPRB, 0.41847E+00_JPRB, 0.40971E+00_JPRB, 0.39986E+00_JPRB ,&
&  0.38968E+00_JPRB, 0.38239E+00_JPRB, 0.37770E+00_JPRB, 0.37342E+00_JPRB, 0.36925E+00_JPRB, 0.36656E+00_JPRB, 0.36347E+00_JPRB ,&
&  0.36198E+00_JPRB, 0.36217E+00_JPRB, 0.36113E+00_JPRB, 0.36058E+00_JPRB, 0.36135E+00_JPRB, 0.36157E+00_JPRB, 0.36260E+00_JPRB ,&
&  0.36214E+00_JPRB, 0.36136E+00_JPRB, 0.36098E+00_JPRB, 0.36056E+00_JPRB, 0.36000E+00_JPRB, 0.36138E+00_JPRB, 0.36405E+00_JPRB ,&
&  0.36702E+00_JPRB, 0.37133E+00_JPRB, 0.37603E+00_JPRB, 0.37978E+00_JPRB, 0.38317E+00_JPRB, 0.38838E+00_JPRB, 0.39500E+00_JPRB ,&
&  0.40271E+00_JPRB, 0.41193E+00_JPRB, 0.42118E+00_JPRB, 0.43055E+00_JPRB, 0.43698E+00_JPRB, 0.44042E+00_JPRB, 0.44320E+00_JPRB ,&
&  0.44479E+00_JPRB, 0.44115E+00_JPRB, 0.43951E+00_JPRB, 0.43812E+00_JPRB, 0.43468E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.37957E+00_JPRB, 0.37722E+00_JPRB, 0.37513E+00_JPRB, 0.37387E+00_JPRB, 0.37272E+00_JPRB, 0.37172E+00_JPRB, 0.37136E+00_JPRB ,&
&  0.37187E+00_JPRB, 0.37211E+00_JPRB, 0.37497E+00_JPRB, 0.37625E+00_JPRB, 0.37659E+00_JPRB, 0.37767E+00_JPRB, 0.37838E+00_JPRB ,&
&  0.37707E+00_JPRB, 0.37595E+00_JPRB, 0.37467E+00_JPRB, 0.37223E+00_JPRB, 0.36835E+00_JPRB, 0.36612E+00_JPRB, 0.36408E+00_JPRB ,&
&  0.36098E+00_JPRB, 0.35733E+00_JPRB, 0.35414E+00_JPRB, 0.35247E+00_JPRB, 0.35250E+00_JPRB, 0.35207E+00_JPRB, 0.35260E+00_JPRB ,&
&  0.35244E+00_JPRB, 0.35684E+00_JPRB, 0.34562E+00_JPRB, 0.35529E+00_JPRB, 0.36422E+00_JPRB, 0.37305E+00_JPRB, 0.38242E+00_JPRB ,&
&  0.38895E+00_JPRB, 0.39560E+00_JPRB, 0.40467E+00_JPRB, 0.40984E+00_JPRB, 0.41070E+00_JPRB, 0.40812E+00_JPRB, 0.40801E+00_JPRB ,&
&  0.40809E+00_JPRB, 0.40546E+00_JPRB, 0.40424E+00_JPRB, 0.40536E+00_JPRB, 0.40662E+00_JPRB, 0.40596E+00_JPRB, 0.40432E+00_JPRB ,&
&  0.40289E+00_JPRB, 0.39959E+00_JPRB, 0.39386E+00_JPRB, 0.38747E+00_JPRB, 0.38093E+00_JPRB, 0.37339E+00_JPRB, 0.36371E+00_JPRB ,&
&  0.35290E+00_JPRB, 0.34429E+00_JPRB, 0.33716E+00_JPRB, 0.33250E+00_JPRB, 0.33004E+00_JPRB, 0.32762E+00_JPRB, 0.32537E+00_JPRB ,&
&  0.32456E+00_JPRB, 0.32474E+00_JPRB, 0.32474E+00_JPRB, 0.32446E+00_JPRB, 0.32530E+00_JPRB, 0.32629E+00_JPRB, 0.32940E+00_JPRB ,&
&  0.33286E+00_JPRB, 0.33449E+00_JPRB, 0.33508E+00_JPRB, 0.33565E+00_JPRB, 0.33457E+00_JPRB, 0.33526E+00_JPRB, 0.33716E+00_JPRB ,&
&  0.33929E+00_JPRB, 0.34208E+00_JPRB, 0.34415E+00_JPRB, 0.34612E+00_JPRB, 0.34871E+00_JPRB, 0.35251E+00_JPRB, 0.35823E+00_JPRB ,&
&  0.36382E+00_JPRB, 0.37023E+00_JPRB, 0.37784E+00_JPRB, 0.38536E+00_JPRB, 0.39079E+00_JPRB, 0.39375E+00_JPRB, 0.39629E+00_JPRB ,&
&  0.39682E+00_JPRB, 0.39352E+00_JPRB, 0.38912E+00_JPRB, 0.38569E+00_JPRB, 0.38253E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.34000E+00_JPRB, 0.33978E+00_JPRB, 0.34047E+00_JPRB, 0.34061E+00_JPRB, 0.34023E+00_JPRB, 0.34072E+00_JPRB, 0.34335E+00_JPRB ,&
&  0.34561E+00_JPRB, 0.34700E+00_JPRB, 0.34827E+00_JPRB, 0.34897E+00_JPRB, 0.34935E+00_JPRB, 0.35113E+00_JPRB, 0.35252E+00_JPRB ,&
&  0.35277E+00_JPRB, 0.35168E+00_JPRB, 0.35035E+00_JPRB, 0.34865E+00_JPRB, 0.34632E+00_JPRB, 0.34379E+00_JPRB, 0.34168E+00_JPRB ,&
&  0.33891E+00_JPRB, 0.33579E+00_JPRB, 0.33275E+00_JPRB, 0.33135E+00_JPRB, 0.33102E+00_JPRB, 0.33092E+00_JPRB, 0.33161E+00_JPRB ,&
&  0.33225E+00_JPRB, 0.33482E+00_JPRB, 0.33786E+00_JPRB, 0.34151E+00_JPRB, 0.34510E+00_JPRB, 0.34978E+00_JPRB, 0.35490E+00_JPRB ,&
&  0.35969E+00_JPRB, 0.36222E+00_JPRB, 0.36461E+00_JPRB, 0.36380E+00_JPRB, 0.36538E+00_JPRB, 0.36641E+00_JPRB, 0.36690E+00_JPRB ,&
&  0.36598E+00_JPRB, 0.36053E+00_JPRB, 0.35630E+00_JPRB, 0.35621E+00_JPRB, 0.35760E+00_JPRB, 0.35883E+00_JPRB, 0.35975E+00_JPRB ,&
&  0.35932E+00_JPRB, 0.35786E+00_JPRB, 0.35427E+00_JPRB, 0.34932E+00_JPRB, 0.34398E+00_JPRB, 0.33777E+00_JPRB, 0.32926E+00_JPRB ,&
&  0.32136E+00_JPRB, 0.31495E+00_JPRB, 0.30994E+00_JPRB, 0.30556E+00_JPRB, 0.30254E+00_JPRB, 0.29930E+00_JPRB, 0.29808E+00_JPRB ,&
&  0.29820E+00_JPRB, 0.30005E+00_JPRB, 0.30112E+00_JPRB, 0.30046E+00_JPRB, 0.29937E+00_JPRB, 0.29962E+00_JPRB, 0.30218E+00_JPRB ,&
&  0.30677E+00_JPRB, 0.31033E+00_JPRB, 0.31169E+00_JPRB, 0.31162E+00_JPRB, 0.31106E+00_JPRB, 0.31217E+00_JPRB, 0.31426E+00_JPRB ,&
&  0.31544E+00_JPRB, 0.31616E+00_JPRB, 0.31741E+00_JPRB, 0.31740E+00_JPRB, 0.31941E+00_JPRB, 0.32275E+00_JPRB, 0.32647E+00_JPRB ,&
&  0.33007E+00_JPRB, 0.33460E+00_JPRB, 0.33943E+00_JPRB, 0.34388E+00_JPRB, 0.34815E+00_JPRB, 0.34950E+00_JPRB, 0.35074E+00_JPRB ,&
&  0.35133E+00_JPRB, 0.35008E+00_JPRB, 0.34819E+00_JPRB, 0.34548E+00_JPRB, 0.34222E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.30740E+00_JPRB, 0.31004E+00_JPRB, 0.31346E+00_JPRB, 0.31414E+00_JPRB, 0.31446E+00_JPRB, 0.31593E+00_JPRB, 0.31902E+00_JPRB ,&
&  0.32118E+00_JPRB, 0.32181E+00_JPRB, 0.32183E+00_JPRB, 0.32174E+00_JPRB, 0.32233E+00_JPRB, 0.32418E+00_JPRB, 0.32511E+00_JPRB ,&
&  0.32458E+00_JPRB, 0.32397E+00_JPRB, 0.32310E+00_JPRB, 0.32237E+00_JPRB, 0.32077E+00_JPRB, 0.31839E+00_JPRB, 0.31635E+00_JPRB ,&
&  0.31353E+00_JPRB, 0.31088E+00_JPRB, 0.30885E+00_JPRB, 0.30720E+00_JPRB, 0.30618E+00_JPRB, 0.30598E+00_JPRB, 0.30737E+00_JPRB ,&
&  0.30917E+00_JPRB, 0.31156E+00_JPRB, 0.31460E+00_JPRB, 0.31818E+00_JPRB, 0.32175E+00_JPRB, 0.32578E+00_JPRB, 0.32937E+00_JPRB ,&
&  0.33220E+00_JPRB, 0.33465E+00_JPRB, 0.33638E+00_JPRB, 0.33397E+00_JPRB, 0.33112E+00_JPRB, 0.33147E+00_JPRB, 0.33231E+00_JPRB ,&
&  0.33022E+00_JPRB, 0.32623E+00_JPRB, 0.32103E+00_JPRB, 0.31820E+00_JPRB, 0.31747E+00_JPRB, 0.31642E+00_JPRB, 0.31511E+00_JPRB ,&
&  0.31424E+00_JPRB, 0.31341E+00_JPRB, 0.31191E+00_JPRB, 0.30942E+00_JPRB, 0.30557E+00_JPRB, 0.30049E+00_JPRB, 0.29549E+00_JPRB ,&
&  0.29230E+00_JPRB, 0.29005E+00_JPRB, 0.28988E+00_JPRB, 0.28764E+00_JPRB, 0.28326E+00_JPRB, 0.27906E+00_JPRB, 0.27812E+00_JPRB ,&
&  0.27796E+00_JPRB, 0.27855E+00_JPRB, 0.27867E+00_JPRB, 0.27779E+00_JPRB, 0.27820E+00_JPRB, 0.27905E+00_JPRB, 0.28067E+00_JPRB ,&
&  0.28521E+00_JPRB, 0.28848E+00_JPRB, 0.28858E+00_JPRB, 0.28764E+00_JPRB, 0.28579E+00_JPRB, 0.28543E+00_JPRB, 0.28622E+00_JPRB ,&
&  0.28710E+00_JPRB, 0.28760E+00_JPRB, 0.28802E+00_JPRB, 0.28755E+00_JPRB, 0.28772E+00_JPRB, 0.28925E+00_JPRB, 0.29118E+00_JPRB ,&
&  0.29352E+00_JPRB, 0.29638E+00_JPRB, 0.29893E+00_JPRB, 0.30174E+00_JPRB, 0.30455E+00_JPRB, 0.30641E+00_JPRB, 0.30807E+00_JPRB ,&
&  0.30945E+00_JPRB, 0.30974E+00_JPRB, 0.30932E+00_JPRB, 0.30888E+00_JPRB, 0.30829E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.28277E+00_JPRB, 0.28695E+00_JPRB, 0.29225E+00_JPRB, 0.29339E+00_JPRB, 0.29568E+00_JPRB, 0.29819E+00_JPRB, 0.30166E+00_JPRB ,&
&  0.30396E+00_JPRB, 0.30426E+00_JPRB, 0.30281E+00_JPRB, 0.30132E+00_JPRB, 0.30062E+00_JPRB, 0.30055E+00_JPRB, 0.30020E+00_JPRB ,&
&  0.30012E+00_JPRB, 0.30034E+00_JPRB, 0.30029E+00_JPRB, 0.30004E+00_JPRB, 0.29874E+00_JPRB, 0.29680E+00_JPRB, 0.29462E+00_JPRB ,&
&  0.29105E+00_JPRB, 0.28863E+00_JPRB, 0.28773E+00_JPRB, 0.28727E+00_JPRB, 0.28634E+00_JPRB, 0.28671E+00_JPRB, 0.28839E+00_JPRB ,&
&  0.29212E+00_JPRB, 0.29508E+00_JPRB, 0.29892E+00_JPRB, 0.30284E+00_JPRB, 0.30516E+00_JPRB, 0.30689E+00_JPRB, 0.30892E+00_JPRB ,&
&  0.31020E+00_JPRB, 0.31056E+00_JPRB, 0.31068E+00_JPRB, 0.31078E+00_JPRB, 0.30831E+00_JPRB, 0.30815E+00_JPRB, 0.30879E+00_JPRB ,&
&  0.30836E+00_JPRB, 0.30565E+00_JPRB, 0.30168E+00_JPRB, 0.29835E+00_JPRB, 0.29505E+00_JPRB, 0.29259E+00_JPRB, 0.29052E+00_JPRB ,&
&  0.28893E+00_JPRB, 0.28805E+00_JPRB, 0.28778E+00_JPRB, 0.28601E+00_JPRB, 0.28052E+00_JPRB, 0.27518E+00_JPRB, 0.27217E+00_JPRB ,&
&  0.27077E+00_JPRB, 0.27040E+00_JPRB, 0.27173E+00_JPRB, 0.27037E+00_JPRB, 0.26624E+00_JPRB, 0.26275E+00_JPRB, 0.26125E+00_JPRB ,&
&  0.25958E+00_JPRB, 0.25888E+00_JPRB, 0.25971E+00_JPRB, 0.26041E+00_JPRB, 0.26105E+00_JPRB, 0.26127E+00_JPRB, 0.26270E+00_JPRB ,&
&  0.26425E+00_JPRB, 0.26608E+00_JPRB, 0.26605E+00_JPRB, 0.26335E+00_JPRB, 0.25957E+00_JPRB, 0.25689E+00_JPRB, 0.25658E+00_JPRB ,&
&  0.25833E+00_JPRB, 0.26007E+00_JPRB, 0.26074E+00_JPRB, 0.26111E+00_JPRB, 0.26233E+00_JPRB, 0.26492E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.26968E+00_JPRB, 0.27107E+00_JPRB, 0.27296E+00_JPRB, 0.27493E+00_JPRB, 0.27636E+00_JPRB, 0.27734E+00_JPRB, 0.27783E+00_JPRB ,&
&  0.27738E+00_JPRB, 0.27816E+00_JPRB, 0.27969E+00_JPRB, 0.28142E+00_JPRB, 0.28249E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.26628E+00_JPRB, 0.26934E+00_JPRB, 0.27226E+00_JPRB, 0.27382E+00_JPRB, 0.27556E+00_JPRB, 0.27748E+00_JPRB, 0.28090E+00_JPRB ,&
&  0.28495E+00_JPRB, 0.28635E+00_JPRB, 0.28464E+00_JPRB, 0.28177E+00_JPRB, 0.28000E+00_JPRB, 0.27932E+00_JPRB, 0.27925E+00_JPRB ,&
&  0.27923E+00_JPRB, 0.27919E+00_JPRB, 0.27844E+00_JPRB, 0.27661E+00_JPRB, 0.27397E+00_JPRB, 0.27127E+00_JPRB, 0.26918E+00_JPRB ,&
&  0.26872E+00_JPRB, 0.27003E+00_JPRB, 0.27215E+00_JPRB, 0.27222E+00_JPRB, 0.27227E+00_JPRB, 0.27351E+00_JPRB, 0.27563E+00_JPRB ,&
&  0.27962E+00_JPRB, 0.28153E+00_JPRB, 0.28339E+00_JPRB, 0.27949E+00_JPRB, 0.28379E+00_JPRB, 0.28537E+00_JPRB, 0.28583E+00_JPRB ,&
&  0.28705E+00_JPRB, 0.28806E+00_JPRB, 0.28914E+00_JPRB, 0.28856E+00_JPRB, 0.28447E+00_JPRB, 0.28316E+00_JPRB, 0.28471E+00_JPRB ,&
&  0.28600E+00_JPRB, 0.28445E+00_JPRB, 0.28143E+00_JPRB, 0.27858E+00_JPRB, 0.27618E+00_JPRB, 0.27442E+00_JPRB, 0.27291E+00_JPRB ,&
&  0.27210E+00_JPRB, 0.27115E+00_JPRB, 0.26896E+00_JPRB, 0.26349E+00_JPRB, 0.25735E+00_JPRB, 0.25324E+00_JPRB, 0.25030E+00_JPRB ,&
&  0.25016E+00_JPRB, 0.25202E+00_JPRB, 0.25368E+00_JPRB, 0.25368E+00_JPRB, 0.25211E+00_JPRB, 0.24935E+00_JPRB, 0.24558E+00_JPRB ,&
&  0.24053E+00_JPRB, 0.23859E+00_JPRB, 0.23821E+00_JPRB, 0.23744E+00_JPRB, 0.23795E+00_JPRB, 0.23989E+00_JPRB, 0.24199E+00_JPRB ,&
&  0.24454E+00_JPRB, 0.24459E+00_JPRB, 0.24311E+00_JPRB, 0.24066E+00_JPRB, 0.23807E+00_JPRB, 0.23786E+00_JPRB, 0.23860E+00_JPRB ,&
&  0.23934E+00_JPRB, 0.24008E+00_JPRB, 0.24123E+00_JPRB, 0.24270E+00_JPRB, 0.24455E+00_JPRB, 0.24705E+00_JPRB, 0.24909E+00_JPRB ,&
&  0.24908E+00_JPRB, 0.24796E+00_JPRB, 0.24907E+00_JPRB, 0.25098E+00_JPRB, 0.25202E+00_JPRB, 0.25270E+00_JPRB, 0.25319E+00_JPRB ,&
&  0.25283E+00_JPRB, 0.25279E+00_JPRB, 0.25466E+00_JPRB, 0.25931E+00_JPRB, 0.26349E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.25172E+00_JPRB, 0.25298E+00_JPRB, 0.25307E+00_JPRB, 0.25346E+00_JPRB, 0.25405E+00_JPRB, 0.25591E+00_JPRB, 0.26009E+00_JPRB ,&
&  0.26426E+00_JPRB, 0.26615E+00_JPRB, 0.26450E+00_JPRB, 0.26211E+00_JPRB, 0.26089E+00_JPRB, 0.26032E+00_JPRB, 0.26022E+00_JPRB ,&
&  0.26016E+00_JPRB, 0.25923E+00_JPRB, 0.25644E+00_JPRB, 0.25250E+00_JPRB, 0.24955E+00_JPRB, 0.24872E+00_JPRB, 0.24903E+00_JPRB ,&
&  0.25270E+00_JPRB, 0.25522E+00_JPRB, 0.25753E+00_JPRB, 0.25964E+00_JPRB, 0.26024E+00_JPRB, 0.26004E+00_JPRB, 0.26164E+00_JPRB ,&
&  0.26269E+00_JPRB, 0.26014E+00_JPRB, 0.25477E+00_JPRB, 0.25864E+00_JPRB, 0.26086E+00_JPRB, 0.26222E+00_JPRB, 0.26149E+00_JPRB ,&
&  0.26213E+00_JPRB, 0.26425E+00_JPRB, 0.26639E+00_JPRB, 0.26808E+00_JPRB, 0.26746E+00_JPRB, 0.26477E+00_JPRB, 0.26428E+00_JPRB ,&
&  0.26494E+00_JPRB, 0.26388E+00_JPRB, 0.26100E+00_JPRB, 0.25833E+00_JPRB, 0.25537E+00_JPRB, 0.25399E+00_JPRB, 0.25373E+00_JPRB ,&
&  0.25283E+00_JPRB, 0.25057E+00_JPRB, 0.24616E+00_JPRB, 0.24180E+00_JPRB, 0.23856E+00_JPRB, 0.23531E+00_JPRB, 0.23373E+00_JPRB ,&
&  0.23398E+00_JPRB, 0.23501E+00_JPRB, 0.23492E+00_JPRB, 0.23320E+00_JPRB, 0.22768E+00_JPRB, 0.21793E+00_JPRB, 0.21064E+00_JPRB ,&
&  0.20928E+00_JPRB, 0.21362E+00_JPRB, 0.22129E+00_JPRB, 0.22321E+00_JPRB, 0.22572E+00_JPRB, 0.22737E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22693E+00_JPRB, 0.21437E+00_JPRB, 0.20886E+00_JPRB, 0.20554E+00_JPRB, 0.20246E+00_JPRB, 0.20231E+00_JPRB, 0.20411E+00_JPRB ,&
&  0.20764E+00_JPRB, 0.21188E+00_JPRB, 0.21493E+00_JPRB, 0.21679E+00_JPRB, 0.21741E+00_JPRB, 0.21731E+00_JPRB, 0.21665E+00_JPRB ,&
&  0.21702E+00_JPRB, 0.22110E+00_JPRB, 0.22908E+00_JPRB, 0.23668E+00_JPRB, 0.23772E+00_JPRB, 0.23931E+00_JPRB, 0.23960E+00_JPRB ,&
&  0.23987E+00_JPRB, 0.24175E+00_JPRB, 0.24285E+00_JPRB, 0.24557E+00_JPRB, 0.24897E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.23955E+00_JPRB, 0.24066E+00_JPRB, 0.24072E+00_JPRB, 0.23973E+00_JPRB, 0.23840E+00_JPRB, 0.23853E+00_JPRB, 0.23992E+00_JPRB ,&
&  0.24285E+00_JPRB, 0.24494E+00_JPRB, 0.24539E+00_JPRB, 0.24539E+00_JPRB, 0.24452E+00_JPRB, 0.24256E+00_JPRB, 0.24097E+00_JPRB ,&
&  0.23973E+00_JPRB, 0.23903E+00_JPRB, 0.23847E+00_JPRB, 0.23834E+00_JPRB, 0.23833E+00_JPRB, 0.23928E+00_JPRB, 0.24012E+00_JPRB ,&
&  0.24368E+00_JPRB, 0.24359E+00_JPRB, 0.24291E+00_JPRB, 0.24271E+00_JPRB, 0.24257E+00_JPRB, 0.24267E+00_JPRB, 0.24264E+00_JPRB ,&
&  0.23973E+00_JPRB, 0.23211E+00_JPRB, 0.22560E+00_JPRB, 0.23149E+00_JPRB, 0.24289E+00_JPRB, 0.24322E+00_JPRB, 0.24343E+00_JPRB ,&
&  0.24480E+00_JPRB, 0.24763E+00_JPRB, 0.24973E+00_JPRB, 0.25076E+00_JPRB, 0.24968E+00_JPRB, 0.24813E+00_JPRB, 0.24781E+00_JPRB ,&
&  0.24818E+00_JPRB, 0.24640E+00_JPRB, 0.24100E+00_JPRB, 0.23472E+00_JPRB, 0.22844E+00_JPRB, 0.22599E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22185E+00_JPRB, 0.22030E+00_JPRB, 0.21974E+00_JPRB, 0.21926E+00_JPRB, 0.21968E+00_JPRB, 0.21745E+00_JPRB, 0.21404E+00_JPRB ,&
&  0.21076E+00_JPRB, 0.20653E+00_JPRB, 0.20158E+00_JPRB, 0.19633E+00_JPRB, 0.19192E+00_JPRB, 0.18692E+00_JPRB, 0.18346E+00_JPRB ,&
&  0.18285E+00_JPRB, 0.18700E+00_JPRB, 0.19688E+00_JPRB, 0.20683E+00_JPRB, 0.21296E+00_JPRB, 0.20313E+00_JPRB, 0.19681E+00_JPRB ,&
&  0.18958E+00_JPRB, 0.18347E+00_JPRB, 0.17999E+00_JPRB, 0.17931E+00_JPRB, 0.17960E+00_JPRB, 0.18052E+00_JPRB, 0.18231E+00_JPRB ,&
&  0.18465E+00_JPRB, 0.18755E+00_JPRB, 0.18971E+00_JPRB, 0.19134E+00_JPRB, 0.19252E+00_JPRB, 0.19287E+00_JPRB, 0.19265E+00_JPRB ,&
&  0.19240E+00_JPRB, 0.19370E+00_JPRB, 0.19611E+00_JPRB, 0.20048E+00_JPRB, 0.20335E+00_JPRB, 0.20716E+00_JPRB, 0.21220E+00_JPRB ,&
&  0.21964E+00_JPRB, 0.22817E+00_JPRB, 0.23171E+00_JPRB, 0.23418E+00_JPRB, 0.23717E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.22700E+00_JPRB, 0.23150E+00_JPRB, 0.23375E+00_JPRB, 0.23313E+00_JPRB, 0.23268E+00_JPRB, 0.23317E+00_JPRB, 0.23383E+00_JPRB ,&
&  0.23480E+00_JPRB, 0.23431E+00_JPRB, 0.23286E+00_JPRB, 0.23153E+00_JPRB, 0.23009E+00_JPRB, 0.22820E+00_JPRB, 0.22703E+00_JPRB ,&
&  0.22613E+00_JPRB, 0.22636E+00_JPRB, 0.22629E+00_JPRB, 0.22735E+00_JPRB, 0.22793E+00_JPRB, 0.23005E+00_JPRB, 0.22840E+00_JPRB ,&
&  0.22467E+00_JPRB, 0.22255E+00_JPRB, 0.22487E+00_JPRB, 0.22523E+00_JPRB, 0.22581E+00_JPRB, 0.22660E+00_JPRB, 0.22807E+00_JPRB ,&
&  0.22549E+00_JPRB, 0.22260E+00_JPRB, 0.21925E+00_JPRB, 0.22517E+00_JPRB, 0.23317E+00_JPRB, 0.23230E+00_JPRB, 0.23184E+00_JPRB ,&
&  0.23342E+00_JPRB, 0.23608E+00_JPRB, 0.23740E+00_JPRB, 0.23764E+00_JPRB, 0.23626E+00_JPRB, 0.23571E+00_JPRB, 0.23456E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.22875E+00_JPRB, 0.21962E+00_JPRB, 0.20309E+00_JPRB, 0.19687E+00_JPRB, 0.19412E+00_JPRB, 0.19021E+00_JPRB ,&
&  0.18502E+00_JPRB, 0.18293E+00_JPRB, 0.18288E+00_JPRB, 0.18103E+00_JPRB, 0.18115E+00_JPRB, 0.18075E+00_JPRB, 0.17958E+00_JPRB ,&
&  0.17808E+00_JPRB, 0.17704E+00_JPRB, 0.17550E+00_JPRB, 0.17296E+00_JPRB, 0.17102E+00_JPRB, 0.17058E+00_JPRB, 0.17142E+00_JPRB ,&
&  0.17339E+00_JPRB, 0.17629E+00_JPRB, 0.18485E+00_JPRB, 0.18983E+00_JPRB, 0.18795E+00_JPRB, 0.18440E+00_JPRB, 0.17818E+00_JPRB ,&
&  0.17231E+00_JPRB, 0.17054E+00_JPRB, 0.16931E+00_JPRB, 0.16824E+00_JPRB, 0.16862E+00_JPRB, 0.16971E+00_JPRB, 0.17080E+00_JPRB ,&
&  0.17213E+00_JPRB, 0.17373E+00_JPRB, 0.17475E+00_JPRB, 0.17532E+00_JPRB, 0.17588E+00_JPRB, 0.17617E+00_JPRB, 0.17667E+00_JPRB ,&
&  0.17673E+00_JPRB, 0.17810E+00_JPRB, 0.17924E+00_JPRB, 0.18165E+00_JPRB, 0.18325E+00_JPRB, 0.18537E+00_JPRB, 0.18998E+00_JPRB ,&
&  0.19362E+00_JPRB, 0.20470E+00_JPRB, 0.21961E+00_JPRB, 0.22624E+00_JPRB, 0.22294E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.21839E+00_JPRB, 0.22068E+00_JPRB, 0.22270E+00_JPRB, 0.22125E+00_JPRB, 0.22131E+00_JPRB, 0.22153E+00_JPRB, 0.22270E+00_JPRB ,&
&  0.22321E+00_JPRB, 0.22275E+00_JPRB, 0.22244E+00_JPRB, 0.22084E+00_JPRB, 0.21928E+00_JPRB, 0.21264E+00_JPRB, 0.20997E+00_JPRB ,&
&  0.20784E+00_JPRB, 0.20857E+00_JPRB, 0.20915E+00_JPRB, 0.21122E+00_JPRB, 0.21127E+00_JPRB, 0.21011E+00_JPRB, 0.20677E+00_JPRB ,&
&  0.20425E+00_JPRB, 0.20285E+00_JPRB, 0.20132E+00_JPRB, 0.20170E+00_JPRB, 0.20389E+00_JPRB, 0.20616E+00_JPRB, 0.20763E+00_JPRB ,&
&  0.20850E+00_JPRB, 0.21073E+00_JPRB, 0.21271E+00_JPRB, 0.21561E+00_JPRB, 0.22221E+00_JPRB, 0.22293E+00_JPRB, 0.22389E+00_JPRB ,&
&  0.22483E+00_JPRB, 0.22597E+00_JPRB, 0.22663E+00_JPRB, 0.22627E+00_JPRB, 0.22453E+00_JPRB, 0.21923E+00_JPRB, 0.21775E+00_JPRB ,&
&  0.20547E+00_JPRB, 0.19819E+00_JPRB, 0.19149E+00_JPRB, 0.18667E+00_JPRB, 0.18109E+00_JPRB, 0.17808E+00_JPRB, 0.17471E+00_JPRB ,&
&  0.17184E+00_JPRB, 0.16870E+00_JPRB, 0.16638E+00_JPRB, 0.16541E+00_JPRB, 0.16510E+00_JPRB, 0.16480E+00_JPRB, 0.16408E+00_JPRB ,&
&  0.16416E+00_JPRB, 0.16366E+00_JPRB, 0.16331E+00_JPRB, 0.16292E+00_JPRB, 0.16266E+00_JPRB, 0.16273E+00_JPRB, 0.16348E+00_JPRB ,&
&  0.16547E+00_JPRB, 0.16728E+00_JPRB, 0.16918E+00_JPRB, 0.17108E+00_JPRB, 0.17070E+00_JPRB, 0.16899E+00_JPRB, 0.16638E+00_JPRB ,&
&  0.16406E+00_JPRB, 0.16254E+00_JPRB, 0.16092E+00_JPRB, 0.16048E+00_JPRB, 0.16110E+00_JPRB, 0.16188E+00_JPRB, 0.16326E+00_JPRB ,&
&  0.16448E+00_JPRB, 0.16549E+00_JPRB, 0.16619E+00_JPRB, 0.16663E+00_JPRB, 0.16734E+00_JPRB, 0.16799E+00_JPRB, 0.16911E+00_JPRB ,&
&  0.17035E+00_JPRB, 0.17189E+00_JPRB, 0.17324E+00_JPRB, 0.17512E+00_JPRB, 0.17797E+00_JPRB, 0.18221E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.19648E+00_JPRB, 0.21469E+00_JPRB, 0.21684E+00_JPRB, 0.21689E+00_JPRB, 0.21844E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.20912E+00_JPRB, 0.21105E+00_JPRB, 0.21229E+00_JPRB, 0.21350E+00_JPRB, 0.21359E+00_JPRB, 0.21378E+00_JPRB, 0.21447E+00_JPRB ,&
&  0.21059E+00_JPRB, 0.20790E+00_JPRB, 0.20334E+00_JPRB, 0.19914E+00_JPRB, 0.19514E+00_JPRB, 0.19183E+00_JPRB, 0.18987E+00_JPRB ,&
&  0.18884E+00_JPRB, 0.18897E+00_JPRB, 0.18911E+00_JPRB, 0.18968E+00_JPRB, 0.18876E+00_JPRB, 0.18768E+00_JPRB, 0.18666E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18598E+00_JPRB, 0.18662E+00_JPRB, 0.19006E+00_JPRB, 0.19487E+00_JPRB, 0.20026E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20695E+00_JPRB, 0.20908E+00_JPRB, 0.21178E+00_JPRB, 0.21298E+00_JPRB, 0.21424E+00_JPRB, 0.21562E+00_JPRB, 0.21611E+00_JPRB ,&
&  0.21744E+00_JPRB, 0.21741E+00_JPRB, 0.21725E+00_JPRB, 0.21741E+00_JPRB, 0.20887E+00_JPRB, 0.20264E+00_JPRB, 0.19623E+00_JPRB ,&
&  0.19010E+00_JPRB, 0.18391E+00_JPRB, 0.17831E+00_JPRB, 0.17526E+00_JPRB, 0.17191E+00_JPRB, 0.16997E+00_JPRB, 0.16830E+00_JPRB ,&
&  0.16702E+00_JPRB, 0.16570E+00_JPRB, 0.16383E+00_JPRB, 0.16197E+00_JPRB, 0.16088E+00_JPRB, 0.15996E+00_JPRB, 0.15878E+00_JPRB ,&
&  0.15866E+00_JPRB, 0.15798E+00_JPRB, 0.15792E+00_JPRB, 0.15753E+00_JPRB, 0.15725E+00_JPRB, 0.15700E+00_JPRB, 0.15698E+00_JPRB ,&
&  0.15736E+00_JPRB, 0.15816E+00_JPRB, 0.15849E+00_JPRB, 0.15893E+00_JPRB, 0.15886E+00_JPRB, 0.15824E+00_JPRB, 0.15772E+00_JPRB ,&
&  0.15711E+00_JPRB, 0.15661E+00_JPRB, 0.15663E+00_JPRB, 0.15651E+00_JPRB, 0.15703E+00_JPRB, 0.15771E+00_JPRB, 0.15839E+00_JPRB ,&
&  0.15935E+00_JPRB, 0.16006E+00_JPRB, 0.16078E+00_JPRB, 0.16172E+00_JPRB, 0.16250E+00_JPRB, 0.16351E+00_JPRB, 0.16468E+00_JPRB ,&
&  0.16554E+00_JPRB, 0.16731E+00_JPRB, 0.16877E+00_JPRB, 0.17058E+00_JPRB, 0.17414E+00_JPRB, 0.17774E+00_JPRB, 0.18636E+00_JPRB ,&
&  0.19420E+00_JPRB, 0.21034E+00_JPRB, 0.21155E+00_JPRB, 0.20711E+00_JPRB, 0.20788E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.20368E+00_JPRB, 0.20510E+00_JPRB, 0.20571E+00_JPRB, 0.20666E+00_JPRB, 0.20701E+00_JPRB, 0.20846E+00_JPRB, 0.20853E+00_JPRB ,&
&  0.20863E+00_JPRB, 0.20488E+00_JPRB, 0.20261E+00_JPRB, 0.19995E+00_JPRB, 0.19764E+00_JPRB, 0.19490E+00_JPRB, 0.19223E+00_JPRB ,&
&  0.19017E+00_JPRB, 0.18834E+00_JPRB, 0.18636E+00_JPRB, 0.18364E+00_JPRB, 0.18088E+00_JPRB, 0.17890E+00_JPRB, 0.17753E+00_JPRB ,&
&  0.17619E+00_JPRB, 0.17708E+00_JPRB, 0.17877E+00_JPRB, 0.18211E+00_JPRB, 0.18657E+00_JPRB, 0.19022E+00_JPRB, 0.19513E+00_JPRB ,&
&  0.20110E+00_JPRB, 0.20297E+00_JPRB, 0.20556E+00_JPRB, 0.20672E+00_JPRB, 0.20644E+00_JPRB, 0.20707E+00_JPRB, 0.20500E+00_JPRB ,&
&  0.20515E+00_JPRB, 0.20227E+00_JPRB, 0.20675E+00_JPRB, 0.20349E+00_JPRB, 0.19664E+00_JPRB, 0.19363E+00_JPRB, 0.18747E+00_JPRB ,&
&  0.18312E+00_JPRB, 0.17839E+00_JPRB, 0.17528E+00_JPRB, 0.17355E+00_JPRB, 0.17037E+00_JPRB, 0.16866E+00_JPRB, 0.16601E+00_JPRB ,&
&  0.16356E+00_JPRB, 0.16279E+00_JPRB, 0.16130E+00_JPRB, 0.16073E+00_JPRB, 0.15995E+00_JPRB, 0.15881E+00_JPRB, 0.15757E+00_JPRB ,&
&  0.15673E+00_JPRB, 0.15600E+00_JPRB, 0.15563E+00_JPRB, 0.15510E+00_JPRB, 0.15434E+00_JPRB, 0.15359E+00_JPRB, 0.15272E+00_JPRB ,&
&  0.15215E+00_JPRB, 0.15187E+00_JPRB, 0.15166E+00_JPRB, 0.15165E+00_JPRB, 0.15162E+00_JPRB, 0.15142E+00_JPRB, 0.15157E+00_JPRB ,&
&  0.15185E+00_JPRB, 0.15248E+00_JPRB, 0.15299E+00_JPRB, 0.15368E+00_JPRB, 0.15428E+00_JPRB, 0.15509E+00_JPRB, 0.15566E+00_JPRB ,&
&  0.15632E+00_JPRB, 0.15727E+00_JPRB, 0.15810E+00_JPRB, 0.15908E+00_JPRB, 0.16015E+00_JPRB, 0.16105E+00_JPRB, 0.16248E+00_JPRB ,&
&  0.16394E+00_JPRB, 0.16538E+00_JPRB, 0.16743E+00_JPRB, 0.16950E+00_JPRB, 0.17216E+00_JPRB, 0.17659E+00_JPRB, 0.18161E+00_JPRB ,&
&  0.18793E+00_JPRB, 0.19723E+00_JPRB, 0.19751E+00_JPRB, 0.20064E+00_JPRB, 0.20233E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.20311E+00_JPRB, 0.20439E+00_JPRB, 0.20545E+00_JPRB, 0.20598E+00_JPRB, 0.20661E+00_JPRB, 0.20664E+00_JPRB, 0.20653E+00_JPRB ,&
&  0.20365E+00_JPRB, 0.20243E+00_JPRB, 0.20063E+00_JPRB, 0.19859E+00_JPRB, 0.19681E+00_JPRB, 0.19434E+00_JPRB, 0.19204E+00_JPRB ,&
&  0.18990E+00_JPRB, 0.18741E+00_JPRB, 0.18491E+00_JPRB, 0.18195E+00_JPRB, 0.17914E+00_JPRB, 0.17651E+00_JPRB, 0.17518E+00_JPRB ,&
&  0.17457E+00_JPRB, 0.17502E+00_JPRB, 0.17673E+00_JPRB, 0.17804E+00_JPRB, 0.17991E+00_JPRB, 0.18230E+00_JPRB, 0.18604E+00_JPRB ,&
&  0.18881E+00_JPRB, 0.19231E+00_JPRB, 0.19460E+00_JPRB, 0.19529E+00_JPRB, 0.19463E+00_JPRB, 0.19431E+00_JPRB, 0.19345E+00_JPRB ,&
&  0.19155E+00_JPRB, 0.18957E+00_JPRB, 0.18790E+00_JPRB, 0.18608E+00_JPRB, 0.18409E+00_JPRB, 0.18173E+00_JPRB, 0.17877E+00_JPRB ,&
&  0.17612E+00_JPRB, 0.17378E+00_JPRB, 0.17157E+00_JPRB, 0.16984E+00_JPRB, 0.16815E+00_JPRB, 0.16659E+00_JPRB, 0.16488E+00_JPRB ,&
&  0.16356E+00_JPRB, 0.16242E+00_JPRB, 0.16097E+00_JPRB, 0.15983E+00_JPRB, 0.15851E+00_JPRB, 0.15742E+00_JPRB, 0.15622E+00_JPRB ,&
&  0.15531E+00_JPRB, 0.15453E+00_JPRB, 0.15389E+00_JPRB, 0.15329E+00_JPRB, 0.15293E+00_JPRB, 0.15259E+00_JPRB, 0.15219E+00_JPRB ,&
&  0.15185E+00_JPRB, 0.15187E+00_JPRB, 0.15193E+00_JPRB, 0.15199E+00_JPRB, 0.15209E+00_JPRB, 0.15177E+00_JPRB, 0.15182E+00_JPRB ,&
&  0.15213E+00_JPRB, 0.15245E+00_JPRB, 0.15250E+00_JPRB, 0.15291E+00_JPRB, 0.15359E+00_JPRB, 0.15439E+00_JPRB, 0.15562E+00_JPRB ,&
&  0.15658E+00_JPRB, 0.15756E+00_JPRB, 0.15849E+00_JPRB, 0.15943E+00_JPRB, 0.16066E+00_JPRB, 0.16199E+00_JPRB, 0.16338E+00_JPRB ,&
&  0.16503E+00_JPRB, 0.16657E+00_JPRB, 0.16817E+00_JPRB, 0.16982E+00_JPRB, 0.17247E+00_JPRB, 0.17498E+00_JPRB, 0.17884E+00_JPRB ,&
&  0.18274E+00_JPRB, 0.18441E+00_JPRB, 0.19776E+00_JPRB, 0.19880E+00_JPRB, 0.20037E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.18208E+00_JPRB, 0.18432E+00_JPRB, 0.18580E+00_JPRB, 0.18737E+00_JPRB, 0.18503E+00_JPRB, 0.19659E+00_JPRB, 0.19447E+00_JPRB ,&
&  0.19407E+00_JPRB, 0.18917E+00_JPRB, 0.18967E+00_JPRB, 0.18901E+00_JPRB, 0.18763E+00_JPRB, 0.18607E+00_JPRB, 0.18430E+00_JPRB ,&
&  0.18256E+00_JPRB, 0.18034E+00_JPRB, 0.17852E+00_JPRB, 0.17700E+00_JPRB, 0.17560E+00_JPRB, 0.17439E+00_JPRB, 0.17317E+00_JPRB ,&
&  0.17305E+00_JPRB, 0.17375E+00_JPRB, 0.17484E+00_JPRB, 0.17408E+00_JPRB, 0.17444E+00_JPRB, 0.17528E+00_JPRB, 0.17645E+00_JPRB ,&
&  0.17742E+00_JPRB, 0.17881E+00_JPRB, 0.17973E+00_JPRB, 0.18001E+00_JPRB, 0.18051E+00_JPRB, 0.17983E+00_JPRB, 0.17936E+00_JPRB ,&
&  0.17867E+00_JPRB, 0.17768E+00_JPRB, 0.17644E+00_JPRB, 0.17537E+00_JPRB, 0.17418E+00_JPRB, 0.17264E+00_JPRB, 0.17106E+00_JPRB ,&
&  0.16963E+00_JPRB, 0.16838E+00_JPRB, 0.16703E+00_JPRB, 0.16592E+00_JPRB, 0.16485E+00_JPRB, 0.16379E+00_JPRB, 0.16281E+00_JPRB ,&
&  0.16202E+00_JPRB, 0.16132E+00_JPRB, 0.16057E+00_JPRB, 0.15979E+00_JPRB, 0.15893E+00_JPRB, 0.15811E+00_JPRB, 0.15731E+00_JPRB ,&
&  0.15696E+00_JPRB, 0.15664E+00_JPRB, 0.15632E+00_JPRB, 0.15598E+00_JPRB, 0.15588E+00_JPRB, 0.15578E+00_JPRB, 0.15564E+00_JPRB ,&
&  0.15547E+00_JPRB, 0.15521E+00_JPRB, 0.15512E+00_JPRB, 0.15503E+00_JPRB, 0.15495E+00_JPRB, 0.15488E+00_JPRB, 0.15498E+00_JPRB ,&
&  0.15515E+00_JPRB, 0.15539E+00_JPRB, 0.15590E+00_JPRB, 0.15643E+00_JPRB, 0.15700E+00_JPRB, 0.15753E+00_JPRB, 0.15831E+00_JPRB ,&
&  0.15917E+00_JPRB, 0.15999E+00_JPRB, 0.16069E+00_JPRB, 0.16126E+00_JPRB, 0.16222E+00_JPRB, 0.16311E+00_JPRB, 0.16400E+00_JPRB ,&
&  0.16496E+00_JPRB, 0.16620E+00_JPRB, 0.16749E+00_JPRB, 0.16881E+00_JPRB, 0.17036E+00_JPRB, 0.17165E+00_JPRB, 0.17294E+00_JPRB ,&
&  0.17447E+00_JPRB, 0.17734E+00_JPRB, 0.17923E+00_JPRB, 0.18046E+00_JPRB, 0.18176E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.16755E+00_JPRB, 0.16771E+00_JPRB, 0.16777E+00_JPRB, 0.16775E+00_JPRB, 0.16753E+00_JPRB, 0.16747E+00_JPRB, 0.16741E+00_JPRB ,&
&  0.16737E+00_JPRB, 0.16716E+00_JPRB, 0.16736E+00_JPRB, 0.16745E+00_JPRB, 0.16748E+00_JPRB, 0.16747E+00_JPRB, 0.16732E+00_JPRB ,&
&  0.16701E+00_JPRB, 0.16662E+00_JPRB, 0.16561E+00_JPRB, 0.16542E+00_JPRB, 0.16543E+00_JPRB, 0.16562E+00_JPRB, 0.16576E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16624E+00_JPRB, 0.16645E+00_JPRB, 0.16707E+00_JPRB, 0.16734E+00_JPRB, 0.16765E+00_JPRB, 0.16795E+00_JPRB ,&
&  0.16824E+00_JPRB, 0.16857E+00_JPRB, 0.16883E+00_JPRB, 0.16910E+00_JPRB, 0.16935E+00_JPRB, 0.16931E+00_JPRB, 0.16911E+00_JPRB ,&
&  0.16879E+00_JPRB, 0.16828E+00_JPRB, 0.16775E+00_JPRB, 0.16726E+00_JPRB, 0.16683E+00_JPRB, 0.16636E+00_JPRB, 0.16592E+00_JPRB ,&
&  0.16545E+00_JPRB, 0.16492E+00_JPRB, 0.16435E+00_JPRB, 0.16384E+00_JPRB, 0.16332E+00_JPRB, 0.16287E+00_JPRB, 0.16235E+00_JPRB ,&
&  0.16195E+00_JPRB, 0.16155E+00_JPRB, 0.16116E+00_JPRB, 0.16076E+00_JPRB, 0.16042E+00_JPRB, 0.16007E+00_JPRB, 0.15972E+00_JPRB ,&
&  0.15936E+00_JPRB, 0.15918E+00_JPRB, 0.15892E+00_JPRB, 0.15870E+00_JPRB, 0.15849E+00_JPRB, 0.15834E+00_JPRB, 0.15813E+00_JPRB ,&
&  0.15794E+00_JPRB, 0.15776E+00_JPRB, 0.15769E+00_JPRB, 0.15755E+00_JPRB, 0.15751E+00_JPRB, 0.15749E+00_JPRB, 0.15755E+00_JPRB ,&
&  0.15761E+00_JPRB, 0.15771E+00_JPRB, 0.15788E+00_JPRB, 0.15815E+00_JPRB, 0.15840E+00_JPRB, 0.15867E+00_JPRB, 0.15894E+00_JPRB ,&
&  0.15925E+00_JPRB, 0.15956E+00_JPRB, 0.15989E+00_JPRB, 0.16021E+00_JPRB, 0.16061E+00_JPRB, 0.16101E+00_JPRB, 0.16146E+00_JPRB ,&
&  0.16188E+00_JPRB, 0.16233E+00_JPRB, 0.16272E+00_JPRB, 0.16319E+00_JPRB, 0.16356E+00_JPRB, 0.16380E+00_JPRB, 0.16402E+00_JPRB ,&
&  0.16442E+00_JPRB, 0.16480E+00_JPRB, 0.16525E+00_JPRB, 0.16567E+00_JPRB, 0.16640E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB, 0.16087E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB, 0.87932E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.86627E+00_JPRB, 0.86648E+00_JPRB, 0.86660E+00_JPRB, 0.86674E+00_JPRB, 0.86677E+00_JPRB, 0.86682E+00_JPRB, 0.86687E+00_JPRB ,&
&  0.86680E+00_JPRB, 0.86693E+00_JPRB, 0.86723E+00_JPRB, 0.86766E+00_JPRB, 0.86808E+00_JPRB, 0.86849E+00_JPRB, 0.86896E+00_JPRB ,&
&  0.86943E+00_JPRB, 0.86964E+00_JPRB, 0.86978E+00_JPRB, 0.87014E+00_JPRB, 0.87074E+00_JPRB, 0.87131E+00_JPRB, 0.87227E+00_JPRB ,&
&  0.87254E+00_JPRB, 0.87188E+00_JPRB, 0.87112E+00_JPRB, 0.87130E+00_JPRB, 0.87102E+00_JPRB, 0.87089E+00_JPRB, 0.87078E+00_JPRB ,&
&  0.87061E+00_JPRB, 0.87051E+00_JPRB, 0.87022E+00_JPRB, 0.86996E+00_JPRB, 0.87018E+00_JPRB, 0.87066E+00_JPRB, 0.87119E+00_JPRB ,&
&  0.87176E+00_JPRB, 0.87236E+00_JPRB, 0.87351E+00_JPRB, 0.87469E+00_JPRB, 0.87597E+00_JPRB, 0.87704E+00_JPRB, 0.87830E+00_JPRB ,&
&  0.87918E+00_JPRB, 0.88008E+00_JPRB, 0.88155E+00_JPRB, 0.88344E+00_JPRB, 0.88532E+00_JPRB, 0.88734E+00_JPRB, 0.88901E+00_JPRB ,&
&  0.89149E+00_JPRB, 0.89457E+00_JPRB, 0.89808E+00_JPRB, 0.90169E+00_JPRB, 0.90471E+00_JPRB, 0.90776E+00_JPRB, 0.91112E+00_JPRB ,&
&  0.91409E+00_JPRB, 0.91604E+00_JPRB, 0.91819E+00_JPRB, 0.92047E+00_JPRB, 0.92227E+00_JPRB, 0.92203E+00_JPRB, 0.92196E+00_JPRB ,&
&  0.92199E+00_JPRB, 0.92158E+00_JPRB, 0.91908E+00_JPRB, 0.91702E+00_JPRB, 0.91510E+00_JPRB, 0.91294E+00_JPRB, 0.91042E+00_JPRB ,&
&  0.90808E+00_JPRB, 0.90593E+00_JPRB, 0.90334E+00_JPRB, 0.90052E+00_JPRB, 0.89801E+00_JPRB, 0.89560E+00_JPRB, 0.89317E+00_JPRB ,&
&  0.89077E+00_JPRB, 0.88871E+00_JPRB, 0.88673E+00_JPRB, 0.88477E+00_JPRB, 0.88261E+00_JPRB, 0.88060E+00_JPRB, 0.87856E+00_JPRB ,&
&  0.87666E+00_JPRB, 0.87471E+00_JPRB, 0.87291E+00_JPRB, 0.87113E+00_JPRB, 0.86981E+00_JPRB, 0.86882E+00_JPRB, 0.86800E+00_JPRB ,&
&  0.86731E+00_JPRB, 0.86685E+00_JPRB, 0.86666E+00_JPRB, 0.86644E+00_JPRB, 0.86628E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.86015E+00_JPRB, 0.86117E+00_JPRB, 0.86199E+00_JPRB, 0.86267E+00_JPRB, 0.86340E+00_JPRB, 0.86419E+00_JPRB, 0.86476E+00_JPRB ,&
&  0.86510E+00_JPRB, 0.86567E+00_JPRB, 0.86641E+00_JPRB, 0.86709E+00_JPRB, 0.86764E+00_JPRB, 0.86840E+00_JPRB, 0.86915E+00_JPRB ,&
&  0.86972E+00_JPRB, 0.87011E+00_JPRB, 0.87050E+00_JPRB, 0.87080E+00_JPRB, 0.87091E+00_JPRB, 0.87050E+00_JPRB, 0.87164E+00_JPRB ,&
&  0.87224E+00_JPRB, 0.87101E+00_JPRB, 0.86791E+00_JPRB, 0.85911E+00_JPRB, 0.85284E+00_JPRB, 0.84701E+00_JPRB, 0.84196E+00_JPRB ,&
&  0.83653E+00_JPRB, 0.83575E+00_JPRB, 0.83547E+00_JPRB, 0.83519E+00_JPRB, 0.83805E+00_JPRB, 0.83713E+00_JPRB, 0.83552E+00_JPRB ,&
&  0.83236E+00_JPRB, 0.82592E+00_JPRB, 0.82666E+00_JPRB, 0.82776E+00_JPRB, 0.82885E+00_JPRB, 0.83279E+00_JPRB, 0.83671E+00_JPRB ,&
&  0.84142E+00_JPRB, 0.84653E+00_JPRB, 0.85506E+00_JPRB, 0.86484E+00_JPRB, 0.87386E+00_JPRB, 0.88176E+00_JPRB, 0.88932E+00_JPRB ,&
&  0.89574E+00_JPRB, 0.90248E+00_JPRB, 0.91001E+00_JPRB, 0.91854E+00_JPRB, 0.92599E+00_JPRB, 0.93376E+00_JPRB, 0.94022E+00_JPRB ,&
&  0.94380E+00_JPRB, 0.94613E+00_JPRB, 0.95104E+00_JPRB, 0.95730E+00_JPRB, 0.96404E+00_JPRB, 0.96616E+00_JPRB, 0.96655E+00_JPRB ,&
&  0.96528E+00_JPRB, 0.96374E+00_JPRB, 0.95986E+00_JPRB, 0.95596E+00_JPRB, 0.95128E+00_JPRB, 0.94596E+00_JPRB, 0.93995E+00_JPRB ,&
&  0.93409E+00_JPRB, 0.92874E+00_JPRB, 0.92402E+00_JPRB, 0.91905E+00_JPRB, 0.91454E+00_JPRB, 0.91021E+00_JPRB, 0.90726E+00_JPRB ,&
&  0.90373E+00_JPRB, 0.90077E+00_JPRB, 0.89749E+00_JPRB, 0.89441E+00_JPRB, 0.88938E+00_JPRB, 0.88423E+00_JPRB, 0.87850E+00_JPRB ,&
&  0.87372E+00_JPRB, 0.86880E+00_JPRB, 0.86465E+00_JPRB, 0.86109E+00_JPRB, 0.85929E+00_JPRB, 0.85817E+00_JPRB, 0.85753E+00_JPRB ,&
&  0.85705E+00_JPRB, 0.85733E+00_JPRB, 0.85801E+00_JPRB, 0.85862E+00_JPRB, 0.85930E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.86718E+00_JPRB, 0.86862E+00_JPRB, 0.86983E+00_JPRB, 0.87034E+00_JPRB, 0.87133E+00_JPRB, 0.87211E+00_JPRB, 0.87284E+00_JPRB ,&
&  0.87342E+00_JPRB, 0.87349E+00_JPRB, 0.87348E+00_JPRB, 0.87361E+00_JPRB, 0.87375E+00_JPRB, 0.87476E+00_JPRB, 0.87533E+00_JPRB ,&
&  0.87531E+00_JPRB, 0.87447E+00_JPRB, 0.87498E+00_JPRB, 0.87506E+00_JPRB, 0.87401E+00_JPRB, 0.87252E+00_JPRB, 0.86444E+00_JPRB ,&
&  0.85908E+00_JPRB, 0.85442E+00_JPRB, 0.84851E+00_JPRB, 0.83898E+00_JPRB, 0.83395E+00_JPRB, 0.80759E+00_JPRB, 0.81119E+00_JPRB ,&
&  0.83239E+00_JPRB, 0.82902E+00_JPRB, 0.82283E+00_JPRB, 0.81358E+00_JPRB, 0.78374E+00_JPRB, 0.76803E+00_JPRB, 0.76374E+00_JPRB ,&
&  0.73970E+00_JPRB, 0.75591E+00_JPRB, 0.76005E+00_JPRB, 0.76194E+00_JPRB, 0.76661E+00_JPRB, 0.76442E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.78359E+00_JPRB, 0.78654E+00_JPRB, 0.79769E+00_JPRB, 0.81143E+00_JPRB, 0.82660E+00_JPRB, 0.84114E+00_JPRB, 0.85651E+00_JPRB ,&
&  0.86835E+00_JPRB, 0.87636E+00_JPRB, 0.88145E+00_JPRB, 0.88363E+00_JPRB, 0.88720E+00_JPRB, 0.89196E+00_JPRB, 0.90080E+00_JPRB ,&
&  0.91548E+00_JPRB, 0.93204E+00_JPRB, 0.94140E+00_JPRB, 0.94239E+00_JPRB, 0.94743E+00_JPRB, 0.95209E+00_JPRB, 0.95671E+00_JPRB ,&
&  0.95942E+00_JPRB, 0.96049E+00_JPRB, 0.95916E+00_JPRB, 0.95993E+00_JPRB, 0.96027E+00_JPRB, 0.95816E+00_JPRB, 0.95419E+00_JPRB ,&
&  0.94893E+00_JPRB, 0.94256E+00_JPRB, 0.93751E+00_JPRB, 0.93221E+00_JPRB, 0.92880E+00_JPRB, 0.92552E+00_JPRB, 0.92552E+00_JPRB ,&
&  0.92315E+00_JPRB, 0.92163E+00_JPRB, 0.91667E+00_JPRB, 0.91005E+00_JPRB, 0.90076E+00_JPRB, 0.89191E+00_JPRB, 0.88203E+00_JPRB ,&
&  0.87498E+00_JPRB, 0.86997E+00_JPRB, 0.86652E+00_JPRB, 0.86257E+00_JPRB, 0.86089E+00_JPRB, 0.85923E+00_JPRB, 0.85911E+00_JPRB ,&
&  0.85892E+00_JPRB, 0.86049E+00_JPRB, 0.86172E+00_JPRB, 0.86387E+00_JPRB, 0.86519E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.86263E+00_JPRB, 0.86220E+00_JPRB, 0.86227E+00_JPRB, 0.86204E+00_JPRB, 0.86278E+00_JPRB, 0.86305E+00_JPRB, 0.86583E+00_JPRB ,&
&  0.86828E+00_JPRB, 0.87065E+00_JPRB, 0.87278E+00_JPRB, 0.87462E+00_JPRB, 0.87638E+00_JPRB, 0.87689E+00_JPRB, 0.87655E+00_JPRB ,&
&  0.87588E+00_JPRB, 0.87444E+00_JPRB, 0.87409E+00_JPRB, 0.87373E+00_JPRB, 0.87203E+00_JPRB, 0.87110E+00_JPRB, 0.86236E+00_JPRB ,&
&  0.85567E+00_JPRB, 0.84988E+00_JPRB, 0.82240E+00_JPRB, 0.82890E+00_JPRB, 0.82180E+00_JPRB, 0.81490E+00_JPRB, 0.80647E+00_JPRB ,&
&  0.77981E+00_JPRB, 0.76782E+00_JPRB, 0.74261E+00_JPRB, 0.72203E+00_JPRB, 0.72208E+00_JPRB, 0.70654E+00_JPRB, 0.70650E+00_JPRB ,&
&  0.70205E+00_JPRB, 0.69617E+00_JPRB, 0.69352E+00_JPRB, 0.69272E+00_JPRB, 0.69743E+00_JPRB, 0.70214E+00_JPRB, 0.70742E+00_JPRB ,&
&  0.71863E+00_JPRB, 0.72907E+00_JPRB, 0.74467E+00_JPRB, 0.75024E+00_JPRB, 0.76866E+00_JPRB, 0.78956E+00_JPRB, 0.81187E+00_JPRB ,&
&  0.82635E+00_JPRB, 0.82424E+00_JPRB, 0.82965E+00_JPRB, 0.83961E+00_JPRB, 0.84841E+00_JPRB, 0.85842E+00_JPRB, 0.87455E+00_JPRB ,&
&  0.89622E+00_JPRB, 0.91706E+00_JPRB, 0.92653E+00_JPRB, 0.92613E+00_JPRB, 0.92577E+00_JPRB, 0.92527E+00_JPRB, 0.93028E+00_JPRB ,&
&  0.93492E+00_JPRB, 0.94320E+00_JPRB, 0.94977E+00_JPRB, 0.95903E+00_JPRB, 0.96483E+00_JPRB, 0.97408E+00_JPRB, 0.98026E+00_JPRB ,&
&  0.98531E+00_JPRB, 0.98712E+00_JPRB, 0.99523E+00_JPRB, 0.98700E+00_JPRB, 0.98668E+00_JPRB, 0.98307E+00_JPRB, 0.97366E+00_JPRB ,&
&  0.96295E+00_JPRB, 0.95366E+00_JPRB, 0.94293E+00_JPRB, 0.93136E+00_JPRB, 0.91873E+00_JPRB, 0.90832E+00_JPRB, 0.89947E+00_JPRB ,&
&  0.89291E+00_JPRB, 0.88551E+00_JPRB, 0.87951E+00_JPRB, 0.86962E+00_JPRB, 0.86528E+00_JPRB, 0.85832E+00_JPRB, 0.85550E+00_JPRB ,&
&  0.85399E+00_JPRB, 0.85569E+00_JPRB, 0.85647E+00_JPRB, 0.85948E+00_JPRB, 0.86181E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.83703E+00_JPRB, 0.84156E+00_JPRB, 0.84959E+00_JPRB, 0.85642E+00_JPRB, 0.86225E+00_JPRB, 0.86687E+00_JPRB, 0.86859E+00_JPRB ,&
&  0.87009E+00_JPRB, 0.87000E+00_JPRB, 0.87026E+00_JPRB, 0.87055E+00_JPRB, 0.87186E+00_JPRB, 0.87276E+00_JPRB, 0.87431E+00_JPRB ,&
&  0.87445E+00_JPRB, 0.87411E+00_JPRB, 0.87303E+00_JPRB, 0.87155E+00_JPRB, 0.87033E+00_JPRB, 0.87050E+00_JPRB, 0.86720E+00_JPRB ,&
&  0.86556E+00_JPRB, 0.85677E+00_JPRB, 0.83621E+00_JPRB, 0.83662E+00_JPRB, 0.83159E+00_JPRB, 0.82351E+00_JPRB, 0.80932E+00_JPRB ,&
&  0.78488E+00_JPRB, 0.74089E+00_JPRB, 0.76240E+00_JPRB, 0.74524E+00_JPRB, 0.72678E+00_JPRB, 0.70826E+00_JPRB, 0.68422E+00_JPRB ,&
&  0.66903E+00_JPRB, 0.64901E+00_JPRB, 0.63368E+00_JPRB, 0.62748E+00_JPRB, 0.62620E+00_JPRB, 0.63434E+00_JPRB, 0.64589E+00_JPRB ,&
&  0.65331E+00_JPRB, 0.67249E+00_JPRB, 0.68335E+00_JPRB, 0.69807E+00_JPRB, 0.71490E+00_JPRB, 0.73663E+00_JPRB, 0.74633E+00_JPRB ,&
&  0.76304E+00_JPRB, 0.76942E+00_JPRB, 0.78756E+00_JPRB, 0.81230E+00_JPRB, 0.84218E+00_JPRB, 0.86435E+00_JPRB, 0.88081E+00_JPRB ,&
&  0.90387E+00_JPRB, 0.91906E+00_JPRB, 0.92940E+00_JPRB, 0.93534E+00_JPRB, 0.94553E+00_JPRB, 0.95418E+00_JPRB, 0.96430E+00_JPRB ,&
&  0.97219E+00_JPRB, 0.98593E+00_JPRB, 0.10012E+01_JPRB, 0.10192E+01_JPRB, 0.10306E+01_JPRB, 0.10477E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.10825E+01_JPRB, 0.10906E+01_JPRB, 0.11072E+01_JPRB, 0.10957E+01_JPRB, 0.10894E+01_JPRB, 0.10619E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10233E+01_JPRB, 0.10035E+01_JPRB, 0.98488E+00_JPRB, 0.96737E+00_JPRB, 0.95171E+00_JPRB, 0.93855E+00_JPRB, 0.92558E+00_JPRB ,&
&  0.91064E+00_JPRB, 0.89114E+00_JPRB, 0.87584E+00_JPRB, 0.85990E+00_JPRB, 0.85234E+00_JPRB, 0.84314E+00_JPRB, 0.83911E+00_JPRB ,&
&  0.83669E+00_JPRB, 0.83575E+00_JPRB, 0.83477E+00_JPRB, 0.83690E+00_JPRB, 0.83737E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.82992E+00_JPRB, 0.83535E+00_JPRB, 0.84087E+00_JPRB, 0.84436E+00_JPRB, 0.84362E+00_JPRB, 0.84416E+00_JPRB, 0.84015E+00_JPRB ,&
&  0.83847E+00_JPRB, 0.83641E+00_JPRB, 0.83513E+00_JPRB, 0.83695E+00_JPRB, 0.83953E+00_JPRB, 0.84414E+00_JPRB, 0.84879E+00_JPRB ,&
&  0.85311E+00_JPRB, 0.85814E+00_JPRB, 0.86030E+00_JPRB, 0.86298E+00_JPRB, 0.86518E+00_JPRB, 0.87018E+00_JPRB, 0.87118E+00_JPRB ,&
&  0.87250E+00_JPRB, 0.86370E+00_JPRB, 0.84569E+00_JPRB, 0.83768E+00_JPRB, 0.83990E+00_JPRB, 0.81844E+00_JPRB, 0.81623E+00_JPRB ,&
&  0.82221E+00_JPRB, 0.82212E+00_JPRB, 0.80867E+00_JPRB, 0.80036E+00_JPRB, 0.77131E+00_JPRB, 0.68079E+00_JPRB, 0.67925E+00_JPRB ,&
&  0.65973E+00_JPRB, 0.62907E+00_JPRB, 0.61264E+00_JPRB, 0.59891E+00_JPRB, 0.59314E+00_JPRB, 0.60403E+00_JPRB, 0.62133E+00_JPRB ,&
&  0.62758E+00_JPRB, 0.65223E+00_JPRB, 0.66167E+00_JPRB, 0.67768E+00_JPRB, 0.69395E+00_JPRB, 0.71763E+00_JPRB, 0.73204E+00_JPRB ,&
&  0.74815E+00_JPRB, 0.76528E+00_JPRB, 0.79347E+00_JPRB, 0.83136E+00_JPRB, 0.86121E+00_JPRB, 0.88219E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.91317E+00_JPRB, 0.91509E+00_JPRB, 0.92993E+00_JPRB, 0.94340E+00_JPRB, 0.95522E+00_JPRB, 0.97130E+00_JPRB, 0.98237E+00_JPRB ,&
&  0.10056E+01_JPRB, 0.10451E+01_JPRB, 0.10823E+01_JPRB, 0.11053E+01_JPRB, 0.11211E+01_JPRB, 0.11383E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.11839E+01_JPRB, 0.12069E+01_JPRB, 0.12188E+01_JPRB, 0.11908E+01_JPRB, 0.11753E+01_JPRB, 0.11397E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10595E+01_JPRB, 0.10241E+01_JPRB, 0.10015E+01_JPRB, 0.97706E+00_JPRB, 0.95867E+00_JPRB, 0.93577E+00_JPRB ,&
&  0.91228E+00_JPRB, 0.88851E+00_JPRB, 0.87158E+00_JPRB, 0.86070E+00_JPRB, 0.85386E+00_JPRB, 0.84384E+00_JPRB, 0.83752E+00_JPRB ,&
&  0.82942E+00_JPRB, 0.82419E+00_JPRB, 0.81952E+00_JPRB, 0.82105E+00_JPRB, 0.82344E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.80302E+00_JPRB, 0.80708E+00_JPRB, 0.80695E+00_JPRB, 0.81047E+00_JPRB, 0.80624E+00_JPRB, 0.80640E+00_JPRB, 0.79692E+00_JPRB ,&
&  0.79050E+00_JPRB, 0.78645E+00_JPRB, 0.78462E+00_JPRB, 0.78687E+00_JPRB, 0.79149E+00_JPRB, 0.79907E+00_JPRB, 0.80850E+00_JPRB ,&
&  0.81922E+00_JPRB, 0.82238E+00_JPRB, 0.83098E+00_JPRB, 0.84337E+00_JPRB, 0.85025E+00_JPRB, 0.85916E+00_JPRB, 0.86437E+00_JPRB ,&
&  0.86542E+00_JPRB, 0.86091E+00_JPRB, 0.85654E+00_JPRB, 0.84433E+00_JPRB, 0.83750E+00_JPRB, 0.83566E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.81479E+00_JPRB, 0.80592E+00_JPRB, 0.82837E+00_JPRB, 0.83517E+00_JPRB, 0.78530E+00_JPRB, 0.73740E+00_JPRB, 0.70978E+00_JPRB ,&
&  0.67195E+00_JPRB, 0.62487E+00_JPRB, 0.60785E+00_JPRB, 0.59694E+00_JPRB, 0.60059E+00_JPRB, 0.60620E+00_JPRB, 0.62022E+00_JPRB ,&
&  0.63093E+00_JPRB, 0.64535E+00_JPRB, 0.66365E+00_JPRB, 0.68786E+00_JPRB, 0.70562E+00_JPRB, 0.72633E+00_JPRB, 0.74279E+00_JPRB ,&
&  0.76215E+00_JPRB, 0.78881E+00_JPRB, 0.82813E+00_JPRB, 0.87755E+00_JPRB, 0.91580E+00_JPRB, 0.94830E+00_JPRB, 0.96532E+00_JPRB ,&
&  0.10198E+01_JPRB, 0.10142E+01_JPRB, 0.10328E+01_JPRB, 0.10433E+01_JPRB, 0.10528E+01_JPRB, 0.10708E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11841E+01_JPRB, 0.12027E+01_JPRB, 0.12094E+01_JPRB, 0.12098E+01_JPRB, 0.12193E+01_JPRB, 0.12466E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.12950E+01_JPRB, 0.13885E+01_JPRB, 0.13346E+01_JPRB, 0.12910E+01_JPRB, 0.12434E+01_JPRB, 0.11908E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.10964E+01_JPRB, 0.10509E+01_JPRB, 0.10088E+01_JPRB, 0.97670E+00_JPRB, 0.94691E+00_JPRB, 0.92874E+00_JPRB ,&
&  0.91097E+00_JPRB, 0.89486E+00_JPRB, 0.88380E+00_JPRB, 0.87550E+00_JPRB, 0.86242E+00_JPRB, 0.84750E+00_JPRB, 0.83667E+00_JPRB ,&
&  0.82124E+00_JPRB, 0.81103E+00_JPRB, 0.80175E+00_JPRB, 0.79979E+00_JPRB, 0.79984E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.78139E+00_JPRB, 0.79128E+00_JPRB, 0.79669E+00_JPRB, 0.79803E+00_JPRB, 0.78955E+00_JPRB, 0.78876E+00_JPRB, 0.77957E+00_JPRB ,&
&  0.77434E+00_JPRB, 0.77472E+00_JPRB, 0.77078E+00_JPRB, 0.77142E+00_JPRB, 0.77291E+00_JPRB, 0.77430E+00_JPRB, 0.77776E+00_JPRB ,&
&  0.78398E+00_JPRB, 0.79325E+00_JPRB, 0.80376E+00_JPRB, 0.81598E+00_JPRB, 0.82954E+00_JPRB, 0.84402E+00_JPRB, 0.85582E+00_JPRB ,&
&  0.86117E+00_JPRB, 0.85937E+00_JPRB, 0.86315E+00_JPRB, 0.85834E+00_JPRB, 0.85135E+00_JPRB, 0.84187E+00_JPRB, 0.83402E+00_JPRB ,&
&  0.83926E+00_JPRB, 0.85180E+00_JPRB, 0.84383E+00_JPRB, 0.83819E+00_JPRB, 0.83175E+00_JPRB, 0.78656E+00_JPRB, 0.74009E+00_JPRB ,&
&  0.68981E+00_JPRB, 0.64388E+00_JPRB, 0.62344E+00_JPRB, 0.61447E+00_JPRB, 0.62233E+00_JPRB, 0.63948E+00_JPRB, 0.65701E+00_JPRB ,&
&  0.65841E+00_JPRB, 0.68441E+00_JPRB, 0.71237E+00_JPRB, 0.73187E+00_JPRB, 0.74816E+00_JPRB, 0.76511E+00_JPRB, 0.78316E+00_JPRB ,&
&  0.81226E+00_JPRB, 0.86180E+00_JPRB, 0.95156E+00_JPRB, 0.10590E+01_JPRB, 0.11483E+01_JPRB, 0.11705E+01_JPRB, 0.12492E+01_JPRB ,&
&  0.14369E+01_JPRB, 0.13495E+01_JPRB, 0.13116E+01_JPRB, 0.13006E+01_JPRB, 0.12828E+01_JPRB, 0.12810E+01_JPRB, 0.13054E+01_JPRB ,&
&  0.13475E+01_JPRB, 0.13805E+01_JPRB, 0.13795E+01_JPRB, 0.13622E+01_JPRB, 0.13522E+01_JPRB, 0.13538E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.13834E+01_JPRB, 0.14392E+01_JPRB, 0.15871E+01_JPRB, 0.13854E+01_JPRB, 0.12873E+01_JPRB, 0.12402E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.11428E+01_JPRB, 0.11027E+01_JPRB, 0.10611E+01_JPRB, 0.10130E+01_JPRB, 0.96681E+00_JPRB, 0.92197E+00_JPRB, 0.89633E+00_JPRB ,&
&  0.88473E+00_JPRB, 0.87696E+00_JPRB, 0.87483E+00_JPRB, 0.86781E+00_JPRB, 0.85523E+00_JPRB, 0.84454E+00_JPRB, 0.83190E+00_JPRB ,&
&  0.81295E+00_JPRB, 0.79366E+00_JPRB, 0.78031E+00_JPRB, 0.77707E+00_JPRB, 0.77531E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.78490E+00_JPRB, 0.79223E+00_JPRB, 0.79991E+00_JPRB, 0.81627E+00_JPRB, 0.81863E+00_JPRB, 0.80994E+00_JPRB, 0.78749E+00_JPRB ,&
&  0.77237E+00_JPRB, 0.76676E+00_JPRB, 0.76964E+00_JPRB, 0.77388E+00_JPRB, 0.77644E+00_JPRB, 0.77740E+00_JPRB, 0.77637E+00_JPRB ,&
&  0.77406E+00_JPRB, 0.77600E+00_JPRB, 0.78312E+00_JPRB, 0.79393E+00_JPRB, 0.80804E+00_JPRB, 0.82336E+00_JPRB, 0.83984E+00_JPRB ,&
&  0.85296E+00_JPRB, 0.85734E+00_JPRB, 0.86018E+00_JPRB, 0.85893E+00_JPRB, 0.86097E+00_JPRB, 0.86597E+00_JPRB, 0.86230E+00_JPRB ,&
&  0.85070E+00_JPRB, 0.85756E+00_JPRB, 0.86937E+00_JPRB, 0.85548E+00_JPRB, 0.84644E+00_JPRB, 0.81220E+00_JPRB, 0.76528E+00_JPRB ,&
&  0.72261E+00_JPRB, 0.68010E+00_JPRB, 0.66734E+00_JPRB, 0.66977E+00_JPRB, 0.68265E+00_JPRB, 0.69277E+00_JPRB, 0.71949E+00_JPRB ,&
&  0.74172E+00_JPRB, 0.76026E+00_JPRB, 0.77537E+00_JPRB, 0.78350E+00_JPRB, 0.79352E+00_JPRB, 0.82087E+00_JPRB, 0.88193E+00_JPRB ,&
&  0.99286E+00_JPRB, 0.11357E+01_JPRB, 0.13051E+01_JPRB, 0.12687E+01_JPRB, 0.13034E+01_JPRB, 0.13830E+01_JPRB, 0.15079E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.15794E+01_JPRB, 0.15960E+01_JPRB, 0.16814E+01_JPRB, 0.17402E+01_JPRB, 0.17905E+01_JPRB, 0.17654E+01_JPRB ,&
&  0.17162E+01_JPRB, 0.16519E+01_JPRB, 0.15967E+01_JPRB, 0.15785E+01_JPRB, 0.15894E+01_JPRB, 0.15925E+01_JPRB, 0.15626E+01_JPRB ,&
&  0.14959E+01_JPRB, 0.13901E+01_JPRB, 0.12723E+01_JPRB, 0.11713E+01_JPRB, 0.11429E+01_JPRB, 0.11453E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.10900E+01_JPRB, 0.10498E+01_JPRB, 0.10072E+01_JPRB, 0.94660E+00_JPRB, 0.88721E+00_JPRB, 0.85670E+00_JPRB ,&
&  0.84515E+00_JPRB, 0.84097E+00_JPRB, 0.84198E+00_JPRB, 0.82746E+00_JPRB, 0.82089E+00_JPRB, 0.81475E+00_JPRB, 0.79099E+00_JPRB ,&
&  0.77178E+00_JPRB, 0.76124E+00_JPRB, 0.74903E+00_JPRB, 0.75157E+00_JPRB, 0.76330E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.81466E+00_JPRB, 0.83996E+00_JPRB, 0.86439E+00_JPRB, 0.87252E+00_JPRB, 0.87528E+00_JPRB, 0.87118E+00_JPRB, 0.84848E+00_JPRB ,&
&  0.83772E+00_JPRB, 0.80884E+00_JPRB, 0.78529E+00_JPRB, 0.76305E+00_JPRB, 0.74795E+00_JPRB, 0.74044E+00_JPRB, 0.74489E+00_JPRB ,&
&  0.75434E+00_JPRB, 0.76427E+00_JPRB, 0.77379E+00_JPRB, 0.78175E+00_JPRB, 0.79167E+00_JPRB, 0.80521E+00_JPRB, 0.82270E+00_JPRB ,&
&  0.84125E+00_JPRB, 0.85450E+00_JPRB, 0.86568E+00_JPRB, 0.87331E+00_JPRB, 0.87900E+00_JPRB, 0.88019E+00_JPRB, 0.87033E+00_JPRB ,&
&  0.86984E+00_JPRB, 0.86726E+00_JPRB, 0.87385E+00_JPRB, 0.86813E+00_JPRB, 0.86158E+00_JPRB, 0.83377E+00_JPRB, 0.77932E+00_JPRB ,&
&  0.74846E+00_JPRB, 0.73030E+00_JPRB, 0.73569E+00_JPRB, 0.74775E+00_JPRB, 0.76557E+00_JPRB, 0.78585E+00_JPRB, 0.79964E+00_JPRB ,&
&  0.80318E+00_JPRB, 0.80683E+00_JPRB, 0.83031E+00_JPRB, 0.86489E+00_JPRB, 0.89102E+00_JPRB, 0.94093E+00_JPRB, 0.10064E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.12379E+01_JPRB, 0.14388E+01_JPRB, 0.14760E+01_JPRB, 0.15968E+01_JPRB, 0.19294E+01_JPRB, 0.18918E+01_JPRB ,&
&  0.19211E+01_JPRB, 0.19755E+01_JPRB, 0.20905E+01_JPRB, 0.22499E+01_JPRB, 0.22346E+01_JPRB, 0.22042E+01_JPRB, 0.21548E+01_JPRB ,&
&  0.20262E+01_JPRB, 0.19639E+01_JPRB, 0.19040E+01_JPRB, 0.19243E+01_JPRB, 0.18813E+01_JPRB, 0.18286E+01_JPRB, 0.17713E+01_JPRB ,&
&  0.16569E+01_JPRB, 0.15040E+01_JPRB, 0.13612E+01_JPRB, 0.12388E+01_JPRB, 0.11790E+01_JPRB, 0.11465E+01_JPRB, 0.11272E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.10823E+01_JPRB, 0.10605E+01_JPRB, 0.10320E+01_JPRB, 0.96772E+00_JPRB, 0.89380E+00_JPRB, 0.86235E+00_JPRB ,&
&  0.84240E+00_JPRB, 0.83035E+00_JPRB, 0.81722E+00_JPRB, 0.80763E+00_JPRB, 0.80438E+00_JPRB, 0.79152E+00_JPRB, 0.76990E+00_JPRB ,&
&  0.76655E+00_JPRB, 0.75850E+00_JPRB, 0.74955E+00_JPRB, 0.75586E+00_JPRB, 0.78586E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.86848E+00_JPRB, 0.90193E+00_JPRB, 0.91506E+00_JPRB, 0.92015E+00_JPRB, 0.92683E+00_JPRB, 0.92224E+00_JPRB, 0.87981E+00_JPRB ,&
&  0.86113E+00_JPRB, 0.80305E+00_JPRB, 0.76971E+00_JPRB, 0.74995E+00_JPRB, 0.74228E+00_JPRB, 0.73380E+00_JPRB, 0.72440E+00_JPRB ,&
&  0.73073E+00_JPRB, 0.74952E+00_JPRB, 0.77096E+00_JPRB, 0.78611E+00_JPRB, 0.79879E+00_JPRB, 0.80900E+00_JPRB, 0.81945E+00_JPRB ,&
&  0.83447E+00_JPRB, 0.85139E+00_JPRB, 0.87035E+00_JPRB, 0.88133E+00_JPRB, 0.88736E+00_JPRB, 0.89070E+00_JPRB, 0.88736E+00_JPRB ,&
&  0.88294E+00_JPRB, 0.87120E+00_JPRB, 0.88211E+00_JPRB, 0.88291E+00_JPRB, 0.87038E+00_JPRB, 0.85952E+00_JPRB, 0.82683E+00_JPRB ,&
&  0.78387E+00_JPRB, 0.79782E+00_JPRB, 0.81195E+00_JPRB, 0.82961E+00_JPRB, 0.84134E+00_JPRB, 0.85023E+00_JPRB, 0.85641E+00_JPRB ,&
&  0.86330E+00_JPRB, 0.87309E+00_JPRB, 0.89166E+00_JPRB, 0.91495E+00_JPRB, 0.94467E+00_JPRB, 0.98401E+00_JPRB, 0.10224E+01_JPRB ,&
&  0.10634E+01_JPRB, 0.10384E+01_JPRB, 0.11621E+01_JPRB, 0.14405E+01_JPRB, 0.19893E+01_JPRB, 0.21547E+01_JPRB, 0.22720E+01_JPRB ,&
&  0.24156E+01_JPRB, 0.24432E+01_JPRB, 0.25169E+01_JPRB, 0.25727E+01_JPRB, 0.25565E+01_JPRB, 0.25222E+01_JPRB, 0.25119E+01_JPRB ,&
&  0.24017E+01_JPRB, 0.23884E+01_JPRB, 0.23146E+01_JPRB, 0.22770E+01_JPRB, 0.21852E+01_JPRB, 0.21111E+01_JPRB, 0.20068E+01_JPRB ,&
&  0.18746E+01_JPRB, 0.17378E+01_JPRB, 0.15732E+01_JPRB, 0.14133E+01_JPRB, 0.12890E+01_JPRB, 0.12290E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.11221E+01_JPRB, 0.10933E+01_JPRB, 0.10694E+01_JPRB, 0.10245E+01_JPRB, 0.97390E+00_JPRB, 0.96308E+00_JPRB ,&
&  0.91208E+00_JPRB, 0.88143E+00_JPRB, 0.85122E+00_JPRB, 0.83805E+00_JPRB, 0.84302E+00_JPRB, 0.83961E+00_JPRB, 0.83107E+00_JPRB ,&
&  0.84410E+00_JPRB, 0.83187E+00_JPRB, 0.82384E+00_JPRB, 0.83165E+00_JPRB, 0.84436E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.91986E+00_JPRB, 0.94491E+00_JPRB, 0.95396E+00_JPRB, 0.96480E+00_JPRB, 0.98047E+00_JPRB, 0.97303E+00_JPRB, 0.95665E+00_JPRB ,&
&  0.96081E+00_JPRB, 0.92061E+00_JPRB, 0.95187E+00_JPRB, 0.81982E+00_JPRB, 0.76776E+00_JPRB, 0.75852E+00_JPRB, 0.74739E+00_JPRB ,&
&  0.75401E+00_JPRB, 0.77239E+00_JPRB, 0.79073E+00_JPRB, 0.80615E+00_JPRB, 0.82681E+00_JPRB, 0.83199E+00_JPRB, 0.83649E+00_JPRB ,&
&  0.84690E+00_JPRB, 0.86471E+00_JPRB, 0.89981E+00_JPRB, 0.92092E+00_JPRB, 0.94930E+00_JPRB, 0.97247E+00_JPRB, 0.96822E+00_JPRB ,&
&  0.93932E+00_JPRB, 0.93653E+00_JPRB, 0.94071E+00_JPRB, 0.91455E+00_JPRB, 0.88895E+00_JPRB, 0.87890E+00_JPRB, 0.89967E+00_JPRB ,&
&  0.90559E+00_JPRB, 0.88081E+00_JPRB, 0.90433E+00_JPRB, 0.91129E+00_JPRB, 0.90925E+00_JPRB, 0.90340E+00_JPRB, 0.89812E+00_JPRB ,&
&  0.89597E+00_JPRB, 0.89345E+00_JPRB, 0.90107E+00_JPRB, 0.91638E+00_JPRB, 0.94771E+00_JPRB, 0.99769E+00_JPRB, 0.10260E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.11000E+01_JPRB, 0.13638E+01_JPRB, 0.16556E+01_JPRB, 0.20943E+01_JPRB, 0.23660E+01_JPRB, 0.26015E+01_JPRB ,&
&  0.29349E+01_JPRB, 0.28820E+01_JPRB, 0.29976E+01_JPRB, 0.29648E+01_JPRB, 0.29145E+01_JPRB, 0.28481E+01_JPRB, 0.27690E+01_JPRB ,&
&  0.27208E+01_JPRB, 0.26131E+01_JPRB, 0.27105E+01_JPRB, 0.26225E+01_JPRB, 0.24633E+01_JPRB, 0.23573E+01_JPRB, 0.22542E+01_JPRB ,&
&  0.21587E+01_JPRB, 0.20564E+01_JPRB, 0.19227E+01_JPRB, 0.16395E+01_JPRB, 0.15107E+01_JPRB, 0.14136E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.11391E+01_JPRB, 0.10765E+01_JPRB, 0.10690E+01_JPRB, 0.10261E+01_JPRB, 0.98998E+00_JPRB, 0.98356E+00_JPRB ,&
&  0.95099E+00_JPRB, 0.91574E+00_JPRB, 0.91153E+00_JPRB, 0.91546E+00_JPRB, 0.91790E+00_JPRB, 0.91080E+00_JPRB, 0.92129E+00_JPRB ,&
&  0.89551E+00_JPRB, 0.87843E+00_JPRB, 0.89019E+00_JPRB, 0.88644E+00_JPRB, 0.89844E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.95195E+00_JPRB, 0.97925E+00_JPRB, 0.99924E+00_JPRB, 0.10177E+01_JPRB, 0.10397E+01_JPRB, 0.10585E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10364E+01_JPRB, 0.10329E+01_JPRB, 0.10135E+01_JPRB, 0.97799E+00_JPRB, 0.89168E+00_JPRB, 0.78455E+00_JPRB ,&
&  0.76266E+00_JPRB, 0.79202E+00_JPRB, 0.82729E+00_JPRB, 0.84483E+00_JPRB, 0.85033E+00_JPRB, 0.85901E+00_JPRB, 0.85916E+00_JPRB ,&
&  0.87016E+00_JPRB, 0.88954E+00_JPRB, 0.93245E+00_JPRB, 0.10036E+01_JPRB, 0.10704E+01_JPRB, 0.10725E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10431E+01_JPRB, 0.10333E+01_JPRB, 0.99693E+00_JPRB, 0.94345E+00_JPRB, 0.94630E+00_JPRB, 0.95409E+00_JPRB, 0.99542E+00_JPRB ,&
&  0.10375E+01_JPRB, 0.10466E+01_JPRB, 0.10349E+01_JPRB, 0.10235E+01_JPRB, 0.10086E+01_JPRB, 0.98764E+00_JPRB, 0.96385E+00_JPRB ,&
&  0.94359E+00_JPRB, 0.92924E+00_JPRB, 0.92173E+00_JPRB, 0.92773E+00_JPRB, 0.93670E+00_JPRB, 0.10444E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.14260E+01_JPRB, 0.17508E+01_JPRB, 0.18916E+01_JPRB, 0.22613E+01_JPRB, 0.25428E+01_JPRB, 0.28552E+01_JPRB ,&
&  0.31046E+01_JPRB, 0.30609E+01_JPRB, 0.33076E+01_JPRB, 0.33661E+01_JPRB, 0.33698E+01_JPRB, 0.32699E+01_JPRB, 0.30799E+01_JPRB ,&
&  0.29692E+01_JPRB, 0.30062E+01_JPRB, 0.30643E+01_JPRB, 0.29978E+01_JPRB, 0.28331E+01_JPRB, 0.27254E+01_JPRB, 0.26125E+01_JPRB ,&
&  0.25612E+01_JPRB, 0.25136E+01_JPRB, 0.23798E+01_JPRB, 0.21734E+01_JPRB, 0.19150E+01_JPRB, 0.16743E+01_JPRB, 0.14710E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.11311E+01_JPRB, 0.10597E+01_JPRB, 0.10418E+01_JPRB, 0.10343E+01_JPRB, 0.98864E+00_JPRB, 0.97986E+00_JPRB ,&
&  0.98449E+00_JPRB, 0.97322E+00_JPRB, 0.98454E+00_JPRB, 0.98996E+00_JPRB, 0.10229E+01_JPRB, 0.10297E+01_JPRB, 0.95776E+00_JPRB ,&
&  0.90847E+00_JPRB, 0.90646E+00_JPRB, 0.90201E+00_JPRB, 0.90278E+00_JPRB, 0.92346E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.10023E+01_JPRB, 0.10366E+01_JPRB, 0.10590E+01_JPRB, 0.10739E+01_JPRB, 0.10943E+01_JPRB, 0.11273E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.11131E+01_JPRB, 0.11003E+01_JPRB, 0.10935E+01_JPRB, 0.10799E+01_JPRB, 0.10513E+01_JPRB, 0.10182E+01_JPRB, 0.94845E+00_JPRB ,&
&  0.84988E+00_JPRB, 0.92404E+00_JPRB, 0.89504E+00_JPRB, 0.92470E+00_JPRB, 0.92963E+00_JPRB, 0.90502E+00_JPRB, 0.88030E+00_JPRB ,&
&  0.90327E+00_JPRB, 0.92756E+00_JPRB, 0.99000E+00_JPRB, 0.10561E+01_JPRB, 0.10321E+01_JPRB, 0.10330E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.11001E+01_JPRB, 0.10851E+01_JPRB, 0.10383E+01_JPRB, 0.10296E+01_JPRB, 0.10632E+01_JPRB, 0.11267E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11725E+01_JPRB, 0.11372E+01_JPRB, 0.11071E+01_JPRB, 0.10816E+01_JPRB, 0.10524E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.98445E+00_JPRB, 0.96084E+00_JPRB, 0.95031E+00_JPRB, 0.96281E+00_JPRB, 0.10163E+01_JPRB, 0.12089E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.15408E+01_JPRB, 0.17415E+01_JPRB, 0.20349E+01_JPRB, 0.23628E+01_JPRB, 0.26779E+01_JPRB, 0.29506E+01_JPRB, 0.31782E+01_JPRB ,&
&  0.34349E+01_JPRB, 0.34510E+01_JPRB, 0.38265E+01_JPRB, 0.38014E+01_JPRB, 0.36427E+01_JPRB, 0.35753E+01_JPRB, 0.33461E+01_JPRB ,&
&  0.31378E+01_JPRB, 0.29323E+01_JPRB, 0.30690E+01_JPRB, 0.30815E+01_JPRB, 0.30458E+01_JPRB, 0.30322E+01_JPRB, 0.29558E+01_JPRB ,&
&  0.28895E+01_JPRB, 0.27992E+01_JPRB, 0.24321E+01_JPRB, 0.20923E+01_JPRB, 0.17027E+01_JPRB, 0.16778E+01_JPRB, 0.15094E+01_JPRB ,&
&  0.13117E+01_JPRB, 0.10986E+01_JPRB, 0.10813E+01_JPRB, 0.10627E+01_JPRB, 0.10804E+01_JPRB, 0.10644E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.11004E+01_JPRB, 0.10951E+01_JPRB, 0.11226E+01_JPRB, 0.11658E+01_JPRB, 0.11522E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.99623E+00_JPRB, 0.97390E+00_JPRB, 0.93584E+00_JPRB, 0.93224E+00_JPRB, 0.95851E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.10712E+01_JPRB, 0.10982E+01_JPRB, 0.11153E+01_JPRB, 0.11317E+01_JPRB, 0.11564E+01_JPRB, 0.11849E+01_JPRB, 0.11803E+01_JPRB ,&
&  0.11624E+01_JPRB, 0.11490E+01_JPRB, 0.11378E+01_JPRB, 0.11181E+01_JPRB, 0.10937E+01_JPRB, 0.10550E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.93724E+00_JPRB, 0.10083E+01_JPRB, 0.90860E+00_JPRB, 0.93424E+00_JPRB, 0.11157E+01_JPRB, 0.10338E+01_JPRB, 0.99708E+00_JPRB ,&
&  0.96075E+00_JPRB, 0.99374E+00_JPRB, 0.10457E+01_JPRB, 0.10621E+01_JPRB, 0.99926E+00_JPRB, 0.10398E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.11812E+01_JPRB, 0.12101E+01_JPRB, 0.12216E+01_JPRB, 0.12539E+01_JPRB, 0.13447E+01_JPRB, 0.13840E+01_JPRB ,&
&  0.13391E+01_JPRB, 0.12877E+01_JPRB, 0.12320E+01_JPRB, 0.11883E+01_JPRB, 0.11502E+01_JPRB, 0.11107E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10042E+01_JPRB, 0.99617E+00_JPRB, 0.10343E+01_JPRB, 0.10868E+01_JPRB, 0.13416E+01_JPRB, 0.18174E+01_JPRB ,&
&  0.22711E+01_JPRB, 0.24517E+01_JPRB, 0.28101E+01_JPRB, 0.33418E+01_JPRB, 0.35718E+01_JPRB, 0.36998E+01_JPRB, 0.37470E+01_JPRB ,&
&  0.38259E+01_JPRB, 0.38457E+01_JPRB, 0.37832E+01_JPRB, 0.37970E+01_JPRB, 0.36505E+01_JPRB, 0.36713E+01_JPRB, 0.34821E+01_JPRB ,&
&  0.31417E+01_JPRB, 0.30444E+01_JPRB, 0.31125E+01_JPRB, 0.30784E+01_JPRB, 0.30056E+01_JPRB, 0.30281E+01_JPRB, 0.30319E+01_JPRB ,&
&  0.29111E+01_JPRB, 0.25479E+01_JPRB, 0.19336E+01_JPRB, 0.17458E+01_JPRB, 0.13189E+01_JPRB, 0.13123E+01_JPRB, 0.15189E+01_JPRB ,&
&  0.15144E+01_JPRB, 0.13543E+01_JPRB, 0.12250E+01_JPRB, 0.11622E+01_JPRB, 0.11622E+01_JPRB, 0.13380E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.12332E+01_JPRB, 0.12578E+01_JPRB, 0.13059E+01_JPRB, 0.13645E+01_JPRB, 0.13941E+01_JPRB, 0.13674E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.12114E+01_JPRB, 0.11294E+01_JPRB, 0.10577E+01_JPRB, 0.10313E+01_JPRB, 0.10413E+01_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.11997E+01_JPRB, 0.11979E+01_JPRB, 0.11954E+01_JPRB, 0.11976E+01_JPRB, 0.12231E+01_JPRB, 0.12397E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.12035E+01_JPRB, 0.11897E+01_JPRB, 0.11720E+01_JPRB, 0.11486E+01_JPRB, 0.11088E+01_JPRB, 0.10527E+01_JPRB, 0.10386E+01_JPRB ,&
&  0.11713E+01_JPRB, 0.11940E+01_JPRB, 0.10885E+01_JPRB, 0.96108E+00_JPRB, 0.10752E+01_JPRB, 0.10370E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.10342E+01_JPRB, 0.11163E+01_JPRB, 0.11539E+01_JPRB, 0.11031E+01_JPRB, 0.10540E+01_JPRB, 0.11743E+01_JPRB, 0.13337E+01_JPRB ,&
&  0.14743E+01_JPRB, 0.15685E+01_JPRB, 0.15814E+01_JPRB, 0.15912E+01_JPRB, 0.16471E+01_JPRB, 0.15191E+01_JPRB, 0.15132E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.13676E+01_JPRB, 0.12936E+01_JPRB, 0.12434E+01_JPRB, 0.11999E+01_JPRB, 0.11696E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.11391E+01_JPRB, 0.11226E+01_JPRB, 0.10852E+01_JPRB, 0.10881E+01_JPRB, 0.11247E+01_JPRB, 0.13216E+01_JPRB, 0.16595E+01_JPRB ,&
&  0.21409E+01_JPRB, 0.23919E+01_JPRB, 0.27289E+01_JPRB, 0.32660E+01_JPRB, 0.35195E+01_JPRB, 0.36349E+01_JPRB, 0.37894E+01_JPRB ,&
&  0.38745E+01_JPRB, 0.38940E+01_JPRB, 0.38702E+01_JPRB, 0.38046E+01_JPRB, 0.38091E+01_JPRB, 0.37765E+01_JPRB, 0.36336E+01_JPRB ,&
&  0.33180E+01_JPRB, 0.31480E+01_JPRB, 0.31373E+01_JPRB, 0.29374E+01_JPRB, 0.27977E+01_JPRB, 0.25976E+01_JPRB, 0.27024E+01_JPRB ,&
&  0.26732E+01_JPRB, 0.16866E+01_JPRB, 0.13613E+01_JPRB, 0.15667E+01_JPRB, 0.13156E+01_JPRB, 0.12839E+01_JPRB, 0.14202E+01_JPRB ,&
&  0.13930E+01_JPRB, 0.13398E+01_JPRB, 0.13374E+01_JPRB, 0.13002E+01_JPRB, 0.13368E+01_JPRB, 0.16963E+01_JPRB, 0.17168E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.16293E+01_JPRB, 0.17185E+01_JPRB, 0.16937E+01_JPRB, 0.16410E+01_JPRB, 0.16171E+01_JPRB, 0.15376E+01_JPRB ,&
&  0.14128E+01_JPRB, 0.13174E+01_JPRB, 0.12414E+01_JPRB, 0.12078E+01_JPRB, 0.12014E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.13710E+01_JPRB, 0.13260E+01_JPRB, 0.12880E+01_JPRB, 0.12790E+01_JPRB, 0.12969E+01_JPRB, 0.13025E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.12333E+01_JPRB, 0.12078E+01_JPRB, 0.11695E+01_JPRB, 0.11112E+01_JPRB, 0.10507E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.11955E+01_JPRB, 0.10390E+01_JPRB, 0.99702E+00_JPRB, 0.92922E+00_JPRB, 0.98857E+00_JPRB, 0.10601E+01_JPRB ,&
&  0.11358E+01_JPRB, 0.11721E+01_JPRB, 0.12189E+01_JPRB, 0.12131E+01_JPRB, 0.12221E+01_JPRB, 0.14184E+01_JPRB, 0.17191E+01_JPRB ,&
&  0.19100E+01_JPRB, 0.20288E+01_JPRB, 0.18909E+01_JPRB, 0.19806E+01_JPRB, 0.18514E+01_JPRB, 0.16595E+01_JPRB, 0.16597E+01_JPRB ,&
&  0.15338E+01_JPRB, 0.14268E+01_JPRB, 0.13255E+01_JPRB, 0.12769E+01_JPRB, 0.12498E+01_JPRB, 0.12138E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.11172E+01_JPRB, 0.11044E+01_JPRB, 0.10956E+01_JPRB, 0.11204E+01_JPRB, 0.12683E+01_JPRB, 0.14404E+01_JPRB ,&
&  0.17543E+01_JPRB, 0.17355E+01_JPRB, 0.17052E+01_JPRB, 0.23174E+01_JPRB, 0.31869E+01_JPRB, 0.33893E+01_JPRB, 0.32921E+01_JPRB ,&
&  0.35721E+01_JPRB, 0.37111E+01_JPRB, 0.38264E+01_JPRB, 0.38872E+01_JPRB, 0.39325E+01_JPRB, 0.38132E+01_JPRB, 0.35997E+01_JPRB ,&
&  0.33463E+01_JPRB, 0.31856E+01_JPRB, 0.30351E+01_JPRB, 0.28979E+01_JPRB, 0.27227E+01_JPRB, 0.26906E+01_JPRB, 0.27782E+01_JPRB ,&
&  0.25888E+01_JPRB, 0.22997E+01_JPRB, 0.16275E+01_JPRB, 0.16397E+01_JPRB, 0.13066E+01_JPRB, 0.12639E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.13687E+01_JPRB, 0.14282E+01_JPRB, 0.14447E+01_JPRB, 0.14222E+01_JPRB, 0.16689E+01_JPRB, 0.20488E+01_JPRB, 0.21444E+01_JPRB ,&
&  0.22319E+01_JPRB, 0.20682E+01_JPRB, 0.22292E+01_JPRB, 0.21933E+01_JPRB, 0.21152E+01_JPRB, 0.20417E+01_JPRB, 0.19379E+01_JPRB ,&
&  0.17863E+01_JPRB, 0.16253E+01_JPRB, 0.15046E+01_JPRB, 0.14411E+01_JPRB, 0.14006E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.14654E+01_JPRB, 0.13908E+01_JPRB, 0.13401E+01_JPRB, 0.13235E+01_JPRB, 0.13345E+01_JPRB, 0.13342E+01_JPRB, 0.13033E+01_JPRB ,&
&  0.12761E+01_JPRB, 0.12437E+01_JPRB, 0.12152E+01_JPRB, 0.11560E+01_JPRB, 0.11039E+01_JPRB, 0.10400E+01_JPRB, 0.10040E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.10664E+01_JPRB, 0.93785E+00_JPRB, 0.88588E+00_JPRB, 0.83689E+00_JPRB, 0.88804E+00_JPRB, 0.99545E+00_JPRB ,&
&  0.11141E+01_JPRB, 0.11992E+01_JPRB, 0.13888E+01_JPRB, 0.14398E+01_JPRB, 0.15214E+01_JPRB, 0.17986E+01_JPRB, 0.21811E+01_JPRB ,&
&  0.26466E+01_JPRB, 0.22354E+01_JPRB, 0.22360E+01_JPRB, 0.21010E+01_JPRB, 0.19691E+01_JPRB, 0.18042E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.14901E+01_JPRB, 0.13703E+01_JPRB, 0.13156E+01_JPRB, 0.12725E+01_JPRB, 0.12364E+01_JPRB, 0.12112E+01_JPRB, 0.11957E+01_JPRB ,&
&  0.11873E+01_JPRB, 0.11644E+01_JPRB, 0.11255E+01_JPRB, 0.10884E+01_JPRB, 0.12023E+01_JPRB, 0.13717E+01_JPRB, 0.14481E+01_JPRB ,&
&  0.14240E+01_JPRB, 0.15986E+01_JPRB, 0.22173E+01_JPRB, 0.25539E+01_JPRB, 0.28631E+01_JPRB, 0.32091E+01_JPRB, 0.34790E+01_JPRB ,&
&  0.36907E+01_JPRB, 0.37397E+01_JPRB, 0.37899E+01_JPRB, 0.33573E+01_JPRB, 0.37247E+01_JPRB, 0.36653E+01_JPRB, 0.34158E+01_JPRB ,&
&  0.31218E+01_JPRB, 0.30127E+01_JPRB, 0.27692E+01_JPRB, 0.27200E+01_JPRB, 0.27911E+01_JPRB, 0.29894E+01_JPRB, 0.26402E+01_JPRB ,&
&  0.22705E+01_JPRB, 0.24988E+01_JPRB, 0.22003E+01_JPRB, 0.18071E+01_JPRB, 0.14901E+01_JPRB, 0.14807E+01_JPRB, 0.15010E+01_JPRB ,&
&  0.14935E+01_JPRB, 0.15030E+01_JPRB, 0.15017E+01_JPRB, 0.16803E+01_JPRB, 0.21817E+01_JPRB, 0.25388E+01_JPRB, 0.25254E+01_JPRB ,&
&  0.24439E+01_JPRB, 0.26361E+01_JPRB, 0.26390E+01_JPRB, 0.25756E+01_JPRB, 0.25208E+01_JPRB, 0.24310E+01_JPRB, 0.22843E+01_JPRB ,&
&  0.20978E+01_JPRB, 0.19239E+01_JPRB, 0.17826E+01_JPRB, 0.16570E+01_JPRB, 0.15514E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.14986E+01_JPRB, 0.14198E+01_JPRB, 0.13767E+01_JPRB, 0.13682E+01_JPRB, 0.13830E+01_JPRB, 0.13675E+01_JPRB, 0.13292E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.12265E+01_JPRB, 0.11667E+01_JPRB, 0.11146E+01_JPRB, 0.10803E+01_JPRB, 0.10280E+01_JPRB, 0.10043E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.99238E+00_JPRB, 0.90882E+00_JPRB, 0.96924E+00_JPRB, 0.99198E+00_JPRB, 0.92431E+00_JPRB, 0.99457E+00_JPRB ,&
&  0.10951E+01_JPRB, 0.12741E+01_JPRB, 0.15456E+01_JPRB, 0.18433E+01_JPRB, 0.20877E+01_JPRB, 0.24533E+01_JPRB, 0.27604E+01_JPRB ,&
&  0.27085E+01_JPRB, 0.26252E+01_JPRB, 0.24544E+01_JPRB, 0.22514E+01_JPRB, 0.19672E+01_JPRB, 0.17344E+01_JPRB, 0.15683E+01_JPRB ,&
&  0.14286E+01_JPRB, 0.13482E+01_JPRB, 0.13058E+01_JPRB, 0.12731E+01_JPRB, 0.12610E+01_JPRB, 0.12552E+01_JPRB, 0.12644E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.11732E+01_JPRB, 0.11146E+01_JPRB, 0.11329E+01_JPRB, 0.12657E+01_JPRB, 0.13723E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.16358E+01_JPRB, 0.19169E+01_JPRB, 0.20451E+01_JPRB, 0.26123E+01_JPRB, 0.24180E+01_JPRB, 0.25954E+01_JPRB, 0.32308E+01_JPRB ,&
&  0.35230E+01_JPRB, 0.36149E+01_JPRB, 0.36339E+01_JPRB, 0.29527E+01_JPRB, 0.25505E+01_JPRB, 0.33984E+01_JPRB, 0.32460E+01_JPRB ,&
&  0.29221E+01_JPRB, 0.27915E+01_JPRB, 0.26501E+01_JPRB, 0.29036E+01_JPRB, 0.24240E+01_JPRB, 0.22065E+01_JPRB, 0.17943E+01_JPRB ,&
&  0.18664E+01_JPRB, 0.16629E+01_JPRB, 0.19094E+01_JPRB, 0.17338E+01_JPRB, 0.17229E+01_JPRB, 0.16552E+01_JPRB, 0.15566E+01_JPRB ,&
&  0.15488E+01_JPRB, 0.15767E+01_JPRB, 0.16978E+01_JPRB, 0.22615E+01_JPRB, 0.30628E+01_JPRB, 0.29031E+01_JPRB, 0.30493E+01_JPRB ,&
&  0.31419E+01_JPRB, 0.31259E+01_JPRB, 0.28855E+01_JPRB, 0.28586E+01_JPRB, 0.27430E+01_JPRB, 0.26033E+01_JPRB, 0.24420E+01_JPRB ,&
&  0.22718E+01_JPRB, 0.21086E+01_JPRB, 0.19361E+01_JPRB, 0.17634E+01_JPRB, 0.16131E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.15424E+01_JPRB, 0.14803E+01_JPRB, 0.14550E+01_JPRB, 0.14492E+01_JPRB, 0.14336E+01_JPRB, 0.13908E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.12644E+01_JPRB, 0.11821E+01_JPRB, 0.11118E+01_JPRB, 0.10646E+01_JPRB, 0.10454E+01_JPRB, 0.10220E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10738E+01_JPRB, 0.99108E+00_JPRB, 0.96750E+00_JPRB, 0.11117E+01_JPRB, 0.10240E+01_JPRB, 0.98440E+00_JPRB ,&
&  0.11508E+01_JPRB, 0.13702E+01_JPRB, 0.17479E+01_JPRB, 0.22454E+01_JPRB, 0.26230E+01_JPRB, 0.29745E+01_JPRB, 0.31357E+01_JPRB ,&
&  0.31442E+01_JPRB, 0.28076E+01_JPRB, 0.24560E+01_JPRB, 0.21435E+01_JPRB, 0.18254E+01_JPRB, 0.15902E+01_JPRB, 0.14094E+01_JPRB ,&
&  0.13066E+01_JPRB, 0.12656E+01_JPRB, 0.12386E+01_JPRB, 0.12014E+01_JPRB, 0.12006E+01_JPRB, 0.11594E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11246E+01_JPRB, 0.11095E+01_JPRB, 0.11848E+01_JPRB, 0.12550E+01_JPRB, 0.13572E+01_JPRB, 0.14412E+01_JPRB ,&
&  0.16112E+01_JPRB, 0.18158E+01_JPRB, 0.19418E+01_JPRB, 0.23170E+01_JPRB, 0.28781E+01_JPRB, 0.29822E+01_JPRB, 0.33261E+01_JPRB ,&
&  0.28932E+01_JPRB, 0.24793E+01_JPRB, 0.21421E+01_JPRB, 0.17852E+01_JPRB, 0.20706E+01_JPRB, 0.30825E+01_JPRB, 0.31802E+01_JPRB ,&
&  0.29078E+01_JPRB, 0.28441E+01_JPRB, 0.29267E+01_JPRB, 0.27209E+01_JPRB, 0.22079E+01_JPRB, 0.17259E+01_JPRB, 0.22095E+01_JPRB ,&
&  0.23215E+01_JPRB, 0.23176E+01_JPRB, 0.21392E+01_JPRB, 0.18049E+01_JPRB, 0.15888E+01_JPRB, 0.18254E+01_JPRB, 0.17605E+01_JPRB ,&
&  0.17142E+01_JPRB, 0.17571E+01_JPRB, 0.25317E+01_JPRB, 0.35901E+01_JPRB, 0.39120E+01_JPRB, 0.38584E+01_JPRB, 0.38714E+01_JPRB ,&
&  0.35498E+01_JPRB, 0.33538E+01_JPRB, 0.31342E+01_JPRB, 0.28862E+01_JPRB, 0.26713E+01_JPRB, 0.24598E+01_JPRB, 0.22890E+01_JPRB ,&
&  0.21427E+01_JPRB, 0.20103E+01_JPRB, 0.18724E+01_JPRB, 0.17408E+01_JPRB, 0.16282E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.15778E+01_JPRB, 0.15468E+01_JPRB, 0.15174E+01_JPRB, 0.14877E+01_JPRB, 0.14428E+01_JPRB, 0.13838E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.11375E+01_JPRB, 0.10714E+01_JPRB, 0.10443E+01_JPRB, 0.10232E+01_JPRB, 0.99580E+00_JPRB, 0.10040E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.12473E+01_JPRB, 0.10957E+01_JPRB, 0.10509E+01_JPRB, 0.10947E+01_JPRB, 0.11152E+01_JPRB, 0.97402E+00_JPRB ,&
&  0.12395E+01_JPRB, 0.14904E+01_JPRB, 0.18215E+01_JPRB, 0.24035E+01_JPRB, 0.32268E+01_JPRB, 0.30715E+01_JPRB, 0.31353E+01_JPRB ,&
&  0.31142E+01_JPRB, 0.27269E+01_JPRB, 0.24297E+01_JPRB, 0.20627E+01_JPRB, 0.17173E+01_JPRB, 0.14858E+01_JPRB, 0.12998E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.12282E+01_JPRB, 0.12192E+01_JPRB, 0.11951E+01_JPRB, 0.11659E+01_JPRB, 0.11369E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.11018E+01_JPRB, 0.11071E+01_JPRB, 0.11285E+01_JPRB, 0.12876E+01_JPRB, 0.14255E+01_JPRB, 0.13830E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.18184E+01_JPRB, 0.20300E+01_JPRB, 0.20509E+01_JPRB, 0.23701E+01_JPRB, 0.27446E+01_JPRB, 0.29295E+01_JPRB, 0.31070E+01_JPRB ,&
&  0.24828E+01_JPRB, 0.21710E+01_JPRB, 0.22759E+01_JPRB, 0.23204E+01_JPRB, 0.20309E+01_JPRB, 0.21769E+01_JPRB, 0.32922E+01_JPRB ,&
&  0.24625E+01_JPRB, 0.26653E+01_JPRB, 0.29934E+01_JPRB, 0.31096E+01_JPRB, 0.16420E+01_JPRB, 0.12319E+01_JPRB, 0.19211E+01_JPRB ,&
&  0.19798E+01_JPRB, 0.12900E+01_JPRB, 0.10380E+01_JPRB, 0.12883E+01_JPRB, 0.10809E+01_JPRB, 0.12926E+01_JPRB, 0.17275E+01_JPRB ,&
&  0.19903E+01_JPRB, 0.22033E+01_JPRB, 0.34458E+01_JPRB, 0.40608E+01_JPRB, 0.41514E+01_JPRB, 0.48135E+01_JPRB, 0.43677E+01_JPRB ,&
&  0.39631E+01_JPRB, 0.32821E+01_JPRB, 0.31002E+01_JPRB, 0.27507E+01_JPRB, 0.25025E+01_JPRB, 0.22743E+01_JPRB, 0.20820E+01_JPRB ,&
&  0.19346E+01_JPRB, 0.18112E+01_JPRB, 0.17224E+01_JPRB, 0.16535E+01_JPRB, 0.16145E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.16231E+01_JPRB, 0.15844E+01_JPRB, 0.15460E+01_JPRB, 0.14999E+01_JPRB, 0.14399E+01_JPRB, 0.13578E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.11579E+01_JPRB, 0.10859E+01_JPRB, 0.10386E+01_JPRB, 0.10196E+01_JPRB, 0.99674E+00_JPRB, 0.97511E+00_JPRB, 0.10019E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.12358E+01_JPRB, 0.13089E+01_JPRB, 0.12155E+01_JPRB, 0.11772E+01_JPRB, 0.11624E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.16196E+01_JPRB, 0.19136E+01_JPRB, 0.23895E+01_JPRB, 0.28206E+01_JPRB, 0.28274E+01_JPRB, 0.30079E+01_JPRB ,&
&  0.28600E+01_JPRB, 0.26101E+01_JPRB, 0.22576E+01_JPRB, 0.18889E+01_JPRB, 0.15913E+01_JPRB, 0.13582E+01_JPRB, 0.12500E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.12273E+01_JPRB, 0.12282E+01_JPRB, 0.11897E+01_JPRB, 0.11696E+01_JPRB, 0.11440E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.11514E+01_JPRB, 0.12021E+01_JPRB, 0.13386E+01_JPRB, 0.15720E+01_JPRB, 0.16159E+01_JPRB, 0.15387E+01_JPRB ,&
&  0.15176E+01_JPRB, 0.16950E+01_JPRB, 0.20367E+01_JPRB, 0.22140E+01_JPRB, 0.24608E+01_JPRB, 0.27295E+01_JPRB, 0.27296E+01_JPRB ,&
&  0.26601E+01_JPRB, 0.26809E+01_JPRB, 0.23731E+01_JPRB, 0.25395E+01_JPRB, 0.24421E+01_JPRB, 0.20064E+01_JPRB, 0.23842E+01_JPRB ,&
&  0.25615E+01_JPRB, 0.25667E+01_JPRB, 0.22350E+01_JPRB, 0.17909E+01_JPRB, 0.19693E+01_JPRB, 0.15875E+01_JPRB, 0.10627E+01_JPRB ,&
&  0.95728E+00_JPRB, 0.90760E+00_JPRB, 0.87187E+00_JPRB, 0.91210E+00_JPRB, 0.94156E+00_JPRB, 0.12718E+01_JPRB, 0.17258E+01_JPRB ,&
&  0.25884E+01_JPRB, 0.32398E+01_JPRB, 0.37968E+01_JPRB, 0.40909E+01_JPRB, 0.38875E+01_JPRB, 0.43367E+01_JPRB, 0.40636E+01_JPRB ,&
&  0.33697E+01_JPRB, 0.30826E+01_JPRB, 0.28418E+01_JPRB, 0.25741E+01_JPRB, 0.23774E+01_JPRB, 0.22131E+01_JPRB, 0.20680E+01_JPRB ,&
&  0.19537E+01_JPRB, 0.18567E+01_JPRB, 0.17838E+01_JPRB, 0.17243E+01_JPRB, 0.16633E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.16158E+01_JPRB, 0.15683E+01_JPRB, 0.15225E+01_JPRB, 0.14554E+01_JPRB, 0.13758E+01_JPRB, 0.12736E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.10129E+01_JPRB, 0.99369E+00_JPRB, 0.97372E+00_JPRB, 0.96118E+00_JPRB, 0.96127E+00_JPRB, 0.10052E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.12208E+01_JPRB, 0.14175E+01_JPRB, 0.13813E+01_JPRB, 0.14294E+01_JPRB, 0.13607E+01_JPRB, 0.14261E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.18246E+01_JPRB, 0.21647E+01_JPRB, 0.24670E+01_JPRB, 0.28268E+01_JPRB, 0.28525E+01_JPRB, 0.28206E+01_JPRB ,&
&  0.26511E+01_JPRB, 0.24395E+01_JPRB, 0.20656E+01_JPRB, 0.17235E+01_JPRB, 0.14508E+01_JPRB, 0.12857E+01_JPRB, 0.11935E+01_JPRB ,&
&  0.11917E+01_JPRB, 0.12308E+01_JPRB, 0.12445E+01_JPRB, 0.12127E+01_JPRB, 0.11894E+01_JPRB, 0.11832E+01_JPRB, 0.11911E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.12345E+01_JPRB, 0.12810E+01_JPRB, 0.13948E+01_JPRB, 0.13087E+01_JPRB, 0.12092E+01_JPRB, 0.14263E+01_JPRB ,&
&  0.15938E+01_JPRB, 0.18778E+01_JPRB, 0.19135E+01_JPRB, 0.21553E+01_JPRB, 0.22774E+01_JPRB, 0.23022E+01_JPRB, 0.24638E+01_JPRB ,&
&  0.24899E+01_JPRB, 0.28600E+01_JPRB, 0.22239E+01_JPRB, 0.22729E+01_JPRB, 0.29810E+01_JPRB, 0.25149E+01_JPRB, 0.23927E+01_JPRB ,&
&  0.27595E+01_JPRB, 0.26243E+01_JPRB, 0.28294E+01_JPRB, 0.25681E+01_JPRB, 0.41608E+01_JPRB, 0.47425E+01_JPRB, 0.15198E+01_JPRB ,&
&  0.10910E+01_JPRB, 0.97682E+00_JPRB, 0.91354E+00_JPRB, 0.86506E+00_JPRB, 0.10049E+01_JPRB, 0.14963E+01_JPRB, 0.24168E+01_JPRB ,&
&  0.29254E+01_JPRB, 0.38742E+01_JPRB, 0.41840E+01_JPRB, 0.45491E+01_JPRB, 0.38874E+01_JPRB, 0.38043E+01_JPRB, 0.31272E+01_JPRB ,&
&  0.29806E+01_JPRB, 0.28407E+01_JPRB, 0.25585E+01_JPRB, 0.23220E+01_JPRB, 0.21795E+01_JPRB, 0.20778E+01_JPRB, 0.19815E+01_JPRB ,&
&  0.18956E+01_JPRB, 0.18349E+01_JPRB, 0.17772E+01_JPRB, 0.17207E+01_JPRB, 0.16666E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.15461E+01_JPRB, 0.14984E+01_JPRB, 0.14387E+01_JPRB, 0.13502E+01_JPRB, 0.12653E+01_JPRB, 0.11715E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.99918E+00_JPRB, 0.97243E+00_JPRB, 0.94490E+00_JPRB, 0.94583E+00_JPRB, 0.96601E+00_JPRB, 0.10222E+01_JPRB ,&
&  0.11159E+01_JPRB, 0.12342E+01_JPRB, 0.13411E+01_JPRB, 0.14478E+01_JPRB, 0.15018E+01_JPRB, 0.14575E+01_JPRB, 0.17545E+01_JPRB ,&
&  0.19738E+01_JPRB, 0.20922E+01_JPRB, 0.24783E+01_JPRB, 0.26649E+01_JPRB, 0.27310E+01_JPRB, 0.27036E+01_JPRB, 0.25845E+01_JPRB ,&
&  0.24104E+01_JPRB, 0.21948E+01_JPRB, 0.18364E+01_JPRB, 0.15634E+01_JPRB, 0.13779E+01_JPRB, 0.12536E+01_JPRB, 0.11927E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.12623E+01_JPRB, 0.12805E+01_JPRB, 0.12561E+01_JPRB, 0.12366E+01_JPRB, 0.12286E+01_JPRB, 0.12396E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.13185E+01_JPRB, 0.13767E+01_JPRB, 0.14657E+01_JPRB, 0.12812E+01_JPRB, 0.15084E+01_JPRB, 0.15918E+01_JPRB ,&
&  0.17150E+01_JPRB, 0.18073E+01_JPRB, 0.17586E+01_JPRB, 0.18902E+01_JPRB, 0.20825E+01_JPRB, 0.21304E+01_JPRB, 0.22201E+01_JPRB ,&
&  0.24449E+01_JPRB, 0.22870E+01_JPRB, 0.20509E+01_JPRB, 0.22501E+01_JPRB, 0.27753E+01_JPRB, 0.36862E+01_JPRB, 0.23066E+01_JPRB ,&
&  0.20339E+01_JPRB, 0.30635E+01_JPRB, 0.35868E+01_JPRB, 0.36427E+01_JPRB, 0.52345E+01_JPRB, 0.72627E+01_JPRB, 0.46224E+01_JPRB ,&
&  0.13545E+01_JPRB, 0.99399E+00_JPRB, 0.11448E+01_JPRB, 0.10439E+01_JPRB, 0.10364E+01_JPRB, 0.15911E+01_JPRB, 0.48351E+01_JPRB ,&
&  0.38129E+01_JPRB, 0.46109E+01_JPRB, 0.47965E+01_JPRB, 0.45877E+01_JPRB, 0.37771E+01_JPRB, 0.32913E+01_JPRB, 0.28697E+01_JPRB ,&
&  0.24587E+01_JPRB, 0.22876E+01_JPRB, 0.20568E+01_JPRB, 0.19232E+01_JPRB, 0.18743E+01_JPRB, 0.18433E+01_JPRB, 0.17753E+01_JPRB ,&
&  0.17268E+01_JPRB, 0.16936E+01_JPRB, 0.16600E+01_JPRB, 0.16297E+01_JPRB, 0.15943E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.14341E+01_JPRB, 0.13893E+01_JPRB, 0.13090E+01_JPRB, 0.12304E+01_JPRB, 0.11847E+01_JPRB, 0.11140E+01_JPRB, 0.10690E+01_JPRB ,&
&  0.10292E+01_JPRB, 0.10131E+01_JPRB, 0.98428E+00_JPRB, 0.97918E+00_JPRB, 0.98249E+00_JPRB, 0.10202E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.12624E+01_JPRB, 0.13231E+01_JPRB, 0.14533E+01_JPRB, 0.16350E+01_JPRB, 0.16166E+01_JPRB, 0.18786E+01_JPRB ,&
&  0.23974E+01_JPRB, 0.25777E+01_JPRB, 0.25866E+01_JPRB, 0.26042E+01_JPRB, 0.26665E+01_JPRB, 0.27498E+01_JPRB, 0.24393E+01_JPRB ,&
&  0.21697E+01_JPRB, 0.19708E+01_JPRB, 0.16839E+01_JPRB, 0.14672E+01_JPRB, 0.13133E+01_JPRB, 0.12390E+01_JPRB, 0.12537E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.13146E+01_JPRB, 0.13228E+01_JPRB, 0.13040E+01_JPRB, 0.12867E+01_JPRB, 0.12732E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.13680E+01_JPRB, 0.14290E+01_JPRB, 0.14775E+01_JPRB, 0.14858E+01_JPRB, 0.15401E+01_JPRB, 0.15917E+01_JPRB, 0.16910E+01_JPRB ,&
&  0.17142E+01_JPRB, 0.17410E+01_JPRB, 0.17819E+01_JPRB, 0.18101E+01_JPRB, 0.19503E+01_JPRB, 0.20896E+01_JPRB, 0.21974E+01_JPRB ,&
&  0.23444E+01_JPRB, 0.23328E+01_JPRB, 0.21592E+01_JPRB, 0.23314E+01_JPRB, 0.27536E+01_JPRB, 0.33530E+01_JPRB, 0.32605E+01_JPRB ,&
&  0.30388E+01_JPRB, 0.31107E+01_JPRB, 0.37689E+01_JPRB, 0.48892E+01_JPRB, 0.55515E+01_JPRB, 0.58446E+01_JPRB, 0.70911E+01_JPRB ,&
&  0.59544E+01_JPRB, 0.26803E+01_JPRB, 0.21534E+01_JPRB, 0.29791E+01_JPRB, 0.22314E+01_JPRB, 0.24169E+01_JPRB, 0.37095E+01_JPRB ,&
&  0.41085E+01_JPRB, 0.49530E+01_JPRB, 0.51108E+01_JPRB, 0.39735E+01_JPRB, 0.33567E+01_JPRB, 0.29281E+01_JPRB, 0.24726E+01_JPRB ,&
&  0.21039E+01_JPRB, 0.18750E+01_JPRB, 0.17369E+01_JPRB, 0.16428E+01_JPRB, 0.15841E+01_JPRB, 0.15288E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.14669E+01_JPRB, 0.14527E+01_JPRB, 0.14526E+01_JPRB, 0.14534E+01_JPRB, 0.14539E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.13295E+01_JPRB, 0.12680E+01_JPRB, 0.12287E+01_JPRB, 0.12104E+01_JPRB, 0.11800E+01_JPRB, 0.11526E+01_JPRB, 0.11256E+01_JPRB ,&
&  0.10657E+01_JPRB, 0.10608E+01_JPRB, 0.10790E+01_JPRB, 0.10827E+01_JPRB, 0.10771E+01_JPRB, 0.11098E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.13444E+01_JPRB, 0.14548E+01_JPRB, 0.17192E+01_JPRB, 0.19950E+01_JPRB, 0.24637E+01_JPRB, 0.18759E+01_JPRB ,&
&  0.21580E+01_JPRB, 0.28058E+01_JPRB, 0.26469E+01_JPRB, 0.26455E+01_JPRB, 0.26097E+01_JPRB, 0.25451E+01_JPRB, 0.21555E+01_JPRB ,&
&  0.19512E+01_JPRB, 0.17965E+01_JPRB, 0.15866E+01_JPRB, 0.14078E+01_JPRB, 0.12951E+01_JPRB, 0.13093E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.13550E+01_JPRB, 0.14038E+01_JPRB, 0.13814E+01_JPRB, 0.13736E+01_JPRB, 0.13583E+01_JPRB, 0.13499E+01_JPRB, 0.14035E+01_JPRB ,&
&  0.14731E+01_JPRB, 0.15267E+01_JPRB, 0.15723E+01_JPRB, 0.15338E+01_JPRB, 0.16111E+01_JPRB, 0.16689E+01_JPRB, 0.17457E+01_JPRB ,&
&  0.16765E+01_JPRB, 0.15538E+01_JPRB, 0.16522E+01_JPRB, 0.18158E+01_JPRB, 0.19656E+01_JPRB, 0.20055E+01_JPRB, 0.20811E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.26880E+01_JPRB, 0.29048E+01_JPRB, 0.26170E+01_JPRB, 0.27424E+01_JPRB, 0.33390E+01_JPRB, 0.40835E+01_JPRB ,&
&  0.46227E+01_JPRB, 0.48551E+01_JPRB, 0.53095E+01_JPRB, 0.52986E+01_JPRB, 0.51892E+01_JPRB, 0.50848E+01_JPRB, 0.54810E+01_JPRB ,&
&  0.64173E+01_JPRB, 0.71002E+01_JPRB, 0.59311E+01_JPRB, 0.48420E+01_JPRB, 0.48473E+01_JPRB, 0.40914E+01_JPRB, 0.38403E+01_JPRB ,&
&  0.46250E+01_JPRB, 0.46947E+01_JPRB, 0.42911E+01_JPRB, 0.41329E+01_JPRB, 0.31225E+01_JPRB, 0.24885E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.15065E+01_JPRB, 0.13559E+01_JPRB, 0.12947E+01_JPRB, 0.12714E+01_JPRB, 0.13046E+01_JPRB, 0.13115E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.12935E+01_JPRB, 0.13106E+01_JPRB, 0.13429E+01_JPRB, 0.13720E+01_JPRB, 0.13690E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.12441E+01_JPRB, 0.12197E+01_JPRB, 0.12214E+01_JPRB, 0.12288E+01_JPRB, 0.12218E+01_JPRB, 0.12402E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.11452E+01_JPRB, 0.11043E+01_JPRB, 0.11395E+01_JPRB, 0.11359E+01_JPRB, 0.11383E+01_JPRB, 0.11686E+01_JPRB, 0.12316E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.14698E+01_JPRB, 0.17046E+01_JPRB, 0.20302E+01_JPRB, 0.22231E+01_JPRB, 0.24309E+01_JPRB, 0.24714E+01_JPRB ,&
&  0.19381E+01_JPRB, 0.28938E+01_JPRB, 0.24618E+01_JPRB, 0.25163E+01_JPRB, 0.24388E+01_JPRB, 0.24969E+01_JPRB, 0.21673E+01_JPRB ,&
&  0.18422E+01_JPRB, 0.16510E+01_JPRB, 0.15766E+01_JPRB, 0.14627E+01_JPRB, 0.13847E+01_JPRB, 0.13775E+01_JPRB, 0.14058E+01_JPRB ,&
&  0.14130E+01_JPRB, 0.14236E+01_JPRB, 0.14305E+01_JPRB, 0.14374E+01_JPRB, 0.14308E+01_JPRB, 0.14376E+01_JPRB, 0.14694E+01_JPRB ,&
&  0.15425E+01_JPRB, 0.16012E+01_JPRB, 0.16097E+01_JPRB, 0.16341E+01_JPRB, 0.16554E+01_JPRB, 0.17584E+01_JPRB, 0.18363E+01_JPRB ,&
&  0.17631E+01_JPRB, 0.16339E+01_JPRB, 0.16962E+01_JPRB, 0.17702E+01_JPRB, 0.18324E+01_JPRB, 0.20401E+01_JPRB, 0.21179E+01_JPRB ,&
&  0.25934E+01_JPRB, 0.27317E+01_JPRB, 0.32906E+01_JPRB, 0.27432E+01_JPRB, 0.28068E+01_JPRB, 0.33964E+01_JPRB, 0.37021E+01_JPRB ,&
&  0.38137E+01_JPRB, 0.47294E+01_JPRB, 0.51213E+01_JPRB, 0.54022E+01_JPRB, 0.53616E+01_JPRB, 0.50379E+01_JPRB, 0.47659E+01_JPRB ,&
&  0.50365E+01_JPRB, 0.55310E+01_JPRB, 0.59205E+01_JPRB, 0.49963E+01_JPRB, 0.47407E+01_JPRB, 0.42062E+01_JPRB, 0.45075E+01_JPRB ,&
&  0.51982E+01_JPRB, 0.50673E+01_JPRB, 0.43697E+01_JPRB, 0.35056E+01_JPRB, 0.22690E+01_JPRB, 0.16725E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.10915E+01_JPRB, 0.11077E+01_JPRB, 0.11652E+01_JPRB, 0.12192E+01_JPRB, 0.12440E+01_JPRB, 0.12394E+01_JPRB ,&
&  0.12445E+01_JPRB, 0.12636E+01_JPRB, 0.12776E+01_JPRB, 0.12792E+01_JPRB, 0.12692E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.12306E+01_JPRB, 0.12412E+01_JPRB, 0.12612E+01_JPRB, 0.13015E+01_JPRB, 0.13347E+01_JPRB, 0.14182E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.10460E+01_JPRB, 0.10836E+01_JPRB, 0.11215E+01_JPRB, 0.11656E+01_JPRB, 0.12219E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.14489E+01_JPRB, 0.16230E+01_JPRB, 0.18412E+01_JPRB, 0.20548E+01_JPRB, 0.22872E+01_JPRB, 0.24962E+01_JPRB, 0.24797E+01_JPRB ,&
&  0.20410E+01_JPRB, 0.19792E+01_JPRB, 0.25428E+01_JPRB, 0.24931E+01_JPRB, 0.23753E+01_JPRB, 0.22641E+01_JPRB, 0.21142E+01_JPRB ,&
&  0.18224E+01_JPRB, 0.16756E+01_JPRB, 0.16529E+01_JPRB, 0.16308E+01_JPRB, 0.14759E+01_JPRB, 0.15050E+01_JPRB, 0.15317E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.15278E+01_JPRB, 0.15360E+01_JPRB, 0.15531E+01_JPRB, 0.15349E+01_JPRB, 0.15380E+01_JPRB, 0.15771E+01_JPRB ,&
&  0.16604E+01_JPRB, 0.17189E+01_JPRB, 0.17463E+01_JPRB, 0.17327E+01_JPRB, 0.17927E+01_JPRB, 0.18934E+01_JPRB, 0.19055E+01_JPRB ,&
&  0.18342E+01_JPRB, 0.18294E+01_JPRB, 0.18720E+01_JPRB, 0.19217E+01_JPRB, 0.18822E+01_JPRB, 0.21834E+01_JPRB, 0.23510E+01_JPRB ,&
&  0.26630E+01_JPRB, 0.28775E+01_JPRB, 0.33874E+01_JPRB, 0.27375E+01_JPRB, 0.30316E+01_JPRB, 0.34217E+01_JPRB, 0.36540E+01_JPRB ,&
&  0.37523E+01_JPRB, 0.42210E+01_JPRB, 0.44256E+01_JPRB, 0.47002E+01_JPRB, 0.48379E+01_JPRB, 0.43880E+01_JPRB, 0.47933E+01_JPRB ,&
&  0.47447E+01_JPRB, 0.55129E+01_JPRB, 0.54245E+01_JPRB, 0.52951E+01_JPRB, 0.44594E+01_JPRB, 0.43017E+01_JPRB, 0.44809E+01_JPRB ,&
&  0.48947E+01_JPRB, 0.43985E+01_JPRB, 0.33356E+01_JPRB, 0.23715E+01_JPRB, 0.14705E+01_JPRB, 0.11188E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.10602E+01_JPRB, 0.11141E+01_JPRB, 0.11673E+01_JPRB, 0.11987E+01_JPRB, 0.12105E+01_JPRB, 0.12154E+01_JPRB ,&
&  0.12299E+01_JPRB, 0.12376E+01_JPRB, 0.12346E+01_JPRB, 0.12340E+01_JPRB, 0.12319E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.12182E+01_JPRB, 0.12122E+01_JPRB, 0.12093E+01_JPRB, 0.11834E+01_JPRB, 0.11489E+01_JPRB, 0.11051E+01_JPRB, 0.10313E+01_JPRB ,&
&  0.97480E+00_JPRB, 0.96016E+00_JPRB, 0.10152E+01_JPRB, 0.10849E+01_JPRB, 0.11541E+01_JPRB, 0.12307E+01_JPRB, 0.13440E+01_JPRB ,&
&  0.14774E+01_JPRB, 0.15799E+01_JPRB, 0.17026E+01_JPRB, 0.19186E+01_JPRB, 0.21731E+01_JPRB, 0.22549E+01_JPRB, 0.23237E+01_JPRB ,&
&  0.20848E+01_JPRB, 0.16568E+01_JPRB, 0.21614E+01_JPRB, 0.24306E+01_JPRB, 0.23632E+01_JPRB, 0.22281E+01_JPRB, 0.20598E+01_JPRB ,&
&  0.17759E+01_JPRB, 0.17758E+01_JPRB, 0.18356E+01_JPRB, 0.18424E+01_JPRB, 0.16392E+01_JPRB, 0.16526E+01_JPRB, 0.16598E+01_JPRB ,&
&  0.16749E+01_JPRB, 0.16727E+01_JPRB, 0.17052E+01_JPRB, 0.17233E+01_JPRB, 0.17148E+01_JPRB, 0.17478E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.18383E+01_JPRB, 0.18694E+01_JPRB, 0.19234E+01_JPRB, 0.19551E+01_JPRB, 0.19910E+01_JPRB, 0.20415E+01_JPRB, 0.20172E+01_JPRB ,&
&  0.19669E+01_JPRB, 0.19169E+01_JPRB, 0.20283E+01_JPRB, 0.22583E+01_JPRB, 0.24540E+01_JPRB, 0.22984E+01_JPRB, 0.25787E+01_JPRB ,&
&  0.27430E+01_JPRB, 0.29382E+01_JPRB, 0.30136E+01_JPRB, 0.35642E+01_JPRB, 0.26711E+01_JPRB, 0.31005E+01_JPRB, 0.31344E+01_JPRB ,&
&  0.35500E+01_JPRB, 0.35343E+01_JPRB, 0.35377E+01_JPRB, 0.36626E+01_JPRB, 0.38381E+01_JPRB, 0.36069E+01_JPRB, 0.40223E+01_JPRB ,&
&  0.44895E+01_JPRB, 0.48933E+01_JPRB, 0.50453E+01_JPRB, 0.52067E+01_JPRB, 0.48591E+01_JPRB, 0.48270E+01_JPRB, 0.46680E+01_JPRB ,&
&  0.47322E+01_JPRB, 0.32644E+01_JPRB, 0.22326E+01_JPRB, 0.15447E+01_JPRB, 0.11221E+01_JPRB, 0.93206E+00_JPRB, 0.96808E+00_JPRB ,&
&  0.10162E+01_JPRB, 0.10733E+01_JPRB, 0.11121E+01_JPRB, 0.11442E+01_JPRB, 0.11659E+01_JPRB, 0.11820E+01_JPRB, 0.12079E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.12237E+01_JPRB, 0.12210E+01_JPRB, 0.12185E+01_JPRB, 0.12179E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.11741E+01_JPRB, 0.11468E+01_JPRB, 0.10975E+01_JPRB, 0.10239E+01_JPRB, 0.94503E+00_JPRB, 0.87015E+00_JPRB, 0.82588E+00_JPRB ,&
&  0.79484E+00_JPRB, 0.82551E+00_JPRB, 0.89357E+00_JPRB, 0.98301E+00_JPRB, 0.10844E+01_JPRB, 0.11842E+01_JPRB, 0.12746E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.13665E+01_JPRB, 0.14588E+01_JPRB, 0.16833E+01_JPRB, 0.19370E+01_JPRB, 0.21291E+01_JPRB, 0.22423E+01_JPRB ,&
&  0.23347E+01_JPRB, 0.23651E+01_JPRB, 0.22520E+01_JPRB, 0.18220E+01_JPRB, 0.16557E+01_JPRB, 0.19956E+01_JPRB, 0.18042E+01_JPRB ,&
&  0.17761E+01_JPRB, 0.17471E+01_JPRB, 0.17634E+01_JPRB, 0.17855E+01_JPRB, 0.17521E+01_JPRB, 0.17253E+01_JPRB, 0.17458E+01_JPRB ,&
&  0.17640E+01_JPRB, 0.18009E+01_JPRB, 0.18370E+01_JPRB, 0.18539E+01_JPRB, 0.18909E+01_JPRB, 0.19474E+01_JPRB, 0.19938E+01_JPRB ,&
&  0.20194E+01_JPRB, 0.20723E+01_JPRB, 0.21270E+01_JPRB, 0.21985E+01_JPRB, 0.21775E+01_JPRB, 0.22080E+01_JPRB, 0.21952E+01_JPRB ,&
&  0.21822E+01_JPRB, 0.21760E+01_JPRB, 0.22320E+01_JPRB, 0.24132E+01_JPRB, 0.25206E+01_JPRB, 0.23151E+01_JPRB, 0.24156E+01_JPRB ,&
&  0.27272E+01_JPRB, 0.29095E+01_JPRB, 0.26946E+01_JPRB, 0.34920E+01_JPRB, 0.22958E+01_JPRB, 0.28874E+01_JPRB, 0.33028E+01_JPRB ,&
&  0.31437E+01_JPRB, 0.29343E+01_JPRB, 0.29241E+01_JPRB, 0.30094E+01_JPRB, 0.30482E+01_JPRB, 0.30642E+01_JPRB, 0.36137E+01_JPRB ,&
&  0.41010E+01_JPRB, 0.42965E+01_JPRB, 0.45729E+01_JPRB, 0.46367E+01_JPRB, 0.46341E+01_JPRB, 0.42772E+01_JPRB, 0.37590E+01_JPRB ,&
&  0.29856E+01_JPRB, 0.23864E+01_JPRB, 0.18370E+01_JPRB, 0.15774E+01_JPRB, 0.11613E+01_JPRB, 0.90278E+00_JPRB, 0.96144E+00_JPRB ,&
&  0.10070E+01_JPRB, 0.10389E+01_JPRB, 0.10839E+01_JPRB, 0.11178E+01_JPRB, 0.11292E+01_JPRB, 0.11456E+01_JPRB, 0.11720E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.11733E+01_JPRB, 0.11749E+01_JPRB, 0.11739E+01_JPRB, 0.11783E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.97225E+00_JPRB, 0.93918E+00_JPRB, 0.89126E+00_JPRB, 0.84403E+00_JPRB, 0.79798E+00_JPRB, 0.74776E+00_JPRB, 0.74022E+00_JPRB ,&
&  0.72230E+00_JPRB, 0.73275E+00_JPRB, 0.76534E+00_JPRB, 0.80030E+00_JPRB, 0.85349E+00_JPRB, 0.93062E+00_JPRB, 0.10006E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.10495E+01_JPRB, 0.11128E+01_JPRB, 0.12145E+01_JPRB, 0.14029E+01_JPRB, 0.17318E+01_JPRB, 0.22552E+01_JPRB ,&
&  0.25289E+01_JPRB, 0.26822E+01_JPRB, 0.26216E+01_JPRB, 0.24022E+01_JPRB, 0.20833E+01_JPRB, 0.20495E+01_JPRB, 0.21972E+01_JPRB ,&
&  0.22924E+01_JPRB, 0.21090E+01_JPRB, 0.18483E+01_JPRB, 0.18092E+01_JPRB, 0.17911E+01_JPRB, 0.17827E+01_JPRB, 0.18039E+01_JPRB ,&
&  0.18421E+01_JPRB, 0.18711E+01_JPRB, 0.18943E+01_JPRB, 0.19423E+01_JPRB, 0.19809E+01_JPRB, 0.20227E+01_JPRB, 0.20821E+01_JPRB ,&
&  0.21378E+01_JPRB, 0.21735E+01_JPRB, 0.22076E+01_JPRB, 0.22323E+01_JPRB, 0.23143E+01_JPRB, 0.24092E+01_JPRB, 0.24615E+01_JPRB ,&
&  0.24892E+01_JPRB, 0.24179E+01_JPRB, 0.25275E+01_JPRB, 0.26967E+01_JPRB, 0.26865E+01_JPRB, 0.25536E+01_JPRB, 0.25892E+01_JPRB ,&
&  0.27939E+01_JPRB, 0.29499E+01_JPRB, 0.22288E+01_JPRB, 0.29462E+01_JPRB, 0.35327E+01_JPRB, 0.32104E+01_JPRB, 0.29041E+01_JPRB ,&
&  0.26024E+01_JPRB, 0.24233E+01_JPRB, 0.25055E+01_JPRB, 0.25471E+01_JPRB, 0.27736E+01_JPRB, 0.28437E+01_JPRB, 0.26206E+01_JPRB ,&
&  0.30728E+01_JPRB, 0.32324E+01_JPRB, 0.36092E+01_JPRB, 0.37747E+01_JPRB, 0.36401E+01_JPRB, 0.32123E+01_JPRB, 0.26090E+01_JPRB ,&
&  0.19146E+01_JPRB, 0.18534E+01_JPRB, 0.15843E+01_JPRB, 0.15899E+01_JPRB, 0.13912E+01_JPRB, 0.87652E+00_JPRB, 0.93067E+00_JPRB ,&
&  0.95683E+00_JPRB, 0.98440E+00_JPRB, 0.10167E+01_JPRB, 0.10351E+01_JPRB, 0.10541E+01_JPRB, 0.10677E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.10624E+01_JPRB, 0.10459E+01_JPRB, 0.10331E+01_JPRB, 0.10092E+01_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.84876E+00_JPRB, 0.83409E+00_JPRB, 0.81863E+00_JPRB, 0.81045E+00_JPRB, 0.81128E+00_JPRB, 0.80377E+00_JPRB, 0.78736E+00_JPRB ,&
&  0.76263E+00_JPRB, 0.75135E+00_JPRB, 0.72678E+00_JPRB, 0.70205E+00_JPRB, 0.70555E+00_JPRB, 0.75807E+00_JPRB, 0.81123E+00_JPRB ,&
&  0.84297E+00_JPRB, 0.85597E+00_JPRB, 0.85353E+00_JPRB, 0.85012E+00_JPRB, 0.88711E+00_JPRB, 0.92851E+00_JPRB, 0.11788E+01_JPRB ,&
&  0.16129E+01_JPRB, 0.22056E+01_JPRB, 0.26244E+01_JPRB, 0.28185E+01_JPRB, 0.26766E+01_JPRB, 0.22739E+01_JPRB, 0.26057E+01_JPRB ,&
&  0.23374E+01_JPRB, 0.18349E+01_JPRB, 0.16834E+01_JPRB, 0.16375E+01_JPRB, 0.16423E+01_JPRB, 0.16489E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.17308E+01_JPRB, 0.17826E+01_JPRB, 0.18334E+01_JPRB, 0.18958E+01_JPRB, 0.19629E+01_JPRB, 0.20401E+01_JPRB, 0.21211E+01_JPRB ,&
&  0.22063E+01_JPRB, 0.22472E+01_JPRB, 0.23364E+01_JPRB, 0.22269E+01_JPRB, 0.23776E+01_JPRB, 0.25278E+01_JPRB, 0.26400E+01_JPRB ,&
&  0.26735E+01_JPRB, 0.26535E+01_JPRB, 0.28096E+01_JPRB, 0.28788E+01_JPRB, 0.29204E+01_JPRB, 0.29078E+01_JPRB, 0.28595E+01_JPRB ,&
&  0.29333E+01_JPRB, 0.27481E+01_JPRB, 0.21427E+01_JPRB, 0.24255E+01_JPRB, 0.27784E+01_JPRB, 0.23098E+01_JPRB, 0.25588E+01_JPRB ,&
&  0.22101E+01_JPRB, 0.19610E+01_JPRB, 0.17901E+01_JPRB, 0.18119E+01_JPRB, 0.20551E+01_JPRB, 0.23371E+01_JPRB, 0.21420E+01_JPRB ,&
&  0.20771E+01_JPRB, 0.21449E+01_JPRB, 0.24275E+01_JPRB, 0.25817E+01_JPRB, 0.25103E+01_JPRB, 0.22508E+01_JPRB, 0.19150E+01_JPRB ,&
&  0.16066E+01_JPRB, 0.14180E+01_JPRB, 0.12358E+01_JPRB, 0.11737E+01_JPRB, 0.90787E+00_JPRB, 0.80984E+00_JPRB, 0.84052E+00_JPRB ,&
&  0.85701E+00_JPRB, 0.85407E+00_JPRB, 0.86603E+00_JPRB, 0.87678E+00_JPRB, 0.89728E+00_JPRB, 0.89774E+00_JPRB, 0.91469E+00_JPRB ,&
&  0.92187E+00_JPRB, 0.91578E+00_JPRB, 0.90764E+00_JPRB, 0.90502E+00_JPRB, 0.87816E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.90955E+00_JPRB, 0.91187E+00_JPRB, 0.92297E+00_JPRB, 0.93437E+00_JPRB, 0.95355E+00_JPRB, 0.93973E+00_JPRB, 0.92166E+00_JPRB ,&
&  0.88452E+00_JPRB, 0.87524E+00_JPRB, 0.86494E+00_JPRB, 0.85972E+00_JPRB, 0.86668E+00_JPRB, 0.89105E+00_JPRB, 0.93854E+00_JPRB ,&
&  0.95270E+00_JPRB, 0.95649E+00_JPRB, 0.94683E+00_JPRB, 0.93278E+00_JPRB, 0.89358E+00_JPRB, 0.85821E+00_JPRB, 0.83342E+00_JPRB ,&
&  0.86456E+00_JPRB, 0.10918E+01_JPRB, 0.13782E+01_JPRB, 0.16413E+01_JPRB, 0.19620E+01_JPRB, 0.18604E+01_JPRB, 0.21671E+01_JPRB ,&
&  0.23598E+01_JPRB, 0.16612E+01_JPRB, 0.16346E+01_JPRB, 0.14590E+01_JPRB, 0.14592E+01_JPRB, 0.14527E+01_JPRB, 0.14863E+01_JPRB ,&
&  0.15514E+01_JPRB, 0.15896E+01_JPRB, 0.16117E+01_JPRB, 0.16443E+01_JPRB, 0.16993E+01_JPRB, 0.17587E+01_JPRB, 0.17931E+01_JPRB ,&
&  0.18123E+01_JPRB, 0.17514E+01_JPRB, 0.17697E+01_JPRB, 0.20131E+01_JPRB, 0.22481E+01_JPRB, 0.23953E+01_JPRB, 0.25192E+01_JPRB ,&
&  0.26820E+01_JPRB, 0.27963E+01_JPRB, 0.26487E+01_JPRB, 0.26684E+01_JPRB, 0.28247E+01_JPRB, 0.26618E+01_JPRB, 0.27438E+01_JPRB ,&
&  0.28274E+01_JPRB, 0.25279E+01_JPRB, 0.18725E+01_JPRB, 0.22398E+01_JPRB, 0.24648E+01_JPRB, 0.23179E+01_JPRB, 0.23400E+01_JPRB ,&
&  0.19712E+01_JPRB, 0.17002E+01_JPRB, 0.14122E+01_JPRB, 0.13253E+01_JPRB, 0.13903E+01_JPRB, 0.15983E+01_JPRB, 0.16672E+01_JPRB ,&
&  0.15260E+01_JPRB, 0.14913E+01_JPRB, 0.17158E+01_JPRB, 0.19170E+01_JPRB, 0.19824E+01_JPRB, 0.18317E+01_JPRB, 0.15250E+01_JPRB ,&
&  0.14462E+01_JPRB, 0.13863E+01_JPRB, 0.16034E+01_JPRB, 0.15649E+01_JPRB, 0.92684E+00_JPRB, 0.82057E+00_JPRB, 0.80449E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81731E+00_JPRB, 0.80979E+00_JPRB, 0.82095E+00_JPRB, 0.82561E+00_JPRB, 0.82475E+00_JPRB, 0.85578E+00_JPRB ,&
&  0.89326E+00_JPRB, 0.89368E+00_JPRB, 0.89692E+00_JPRB, 0.89726E+00_JPRB, 0.90176E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.10020E+01_JPRB, 0.10178E+01_JPRB, 0.10403E+01_JPRB, 0.10432E+01_JPRB, 0.10482E+01_JPRB, 0.10393E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.10132E+01_JPRB, 0.10060E+01_JPRB, 0.10063E+01_JPRB, 0.10094E+01_JPRB, 0.10203E+01_JPRB, 0.10334E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10737E+01_JPRB, 0.10823E+01_JPRB, 0.10993E+01_JPRB, 0.11140E+01_JPRB, 0.11091E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.10575E+01_JPRB, 0.10734E+01_JPRB, 0.11044E+01_JPRB, 0.12784E+01_JPRB, 0.15998E+01_JPRB, 0.21179E+01_JPRB ,&
&  0.20710E+01_JPRB, 0.16263E+01_JPRB, 0.14254E+01_JPRB, 0.12402E+01_JPRB, 0.12278E+01_JPRB, 0.12517E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.11408E+01_JPRB, 0.11394E+01_JPRB, 0.11685E+01_JPRB, 0.11959E+01_JPRB, 0.11832E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.10771E+01_JPRB, 0.10466E+01_JPRB, 0.10983E+01_JPRB, 0.12463E+01_JPRB, 0.15703E+01_JPRB, 0.17883E+01_JPRB, 0.17177E+01_JPRB ,&
&  0.18988E+01_JPRB, 0.21891E+01_JPRB, 0.22985E+01_JPRB, 0.23288E+01_JPRB, 0.22900E+01_JPRB, 0.22094E+01_JPRB, 0.21560E+01_JPRB ,&
&  0.21098E+01_JPRB, 0.19555E+01_JPRB, 0.17590E+01_JPRB, 0.20565E+01_JPRB, 0.21995E+01_JPRB, 0.22612E+01_JPRB, 0.20185E+01_JPRB ,&
&  0.17049E+01_JPRB, 0.14967E+01_JPRB, 0.13593E+01_JPRB, 0.12365E+01_JPRB, 0.11764E+01_JPRB, 0.12477E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.11931E+01_JPRB, 0.12369E+01_JPRB, 0.13174E+01_JPRB, 0.14267E+01_JPRB, 0.17270E+01_JPRB, 0.15505E+01_JPRB, 0.16657E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.13840E+01_JPRB, 0.11236E+01_JPRB, 0.16228E+01_JPRB, 0.92910E+00_JPRB, 0.75704E+00_JPRB, 0.77665E+00_JPRB ,&
&  0.81863E+00_JPRB, 0.83907E+00_JPRB, 0.86905E+00_JPRB, 0.89747E+00_JPRB, 0.90215E+00_JPRB, 0.90640E+00_JPRB, 0.92991E+00_JPRB ,&
&  0.95554E+00_JPRB, 0.96912E+00_JPRB, 0.98229E+00_JPRB, 0.98247E+00_JPRB, 0.99180E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.10564E+01_JPRB, 0.10716E+01_JPRB, 0.10851E+01_JPRB, 0.10953E+01_JPRB, 0.10990E+01_JPRB, 0.11004E+01_JPRB, 0.11022E+01_JPRB ,&
&  0.10999E+01_JPRB, 0.10948E+01_JPRB, 0.10920E+01_JPRB, 0.11065E+01_JPRB, 0.11288E+01_JPRB, 0.11403E+01_JPRB, 0.11664E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.11763E+01_JPRB, 0.11991E+01_JPRB, 0.12170E+01_JPRB, 0.12450E+01_JPRB, 0.12435E+01_JPRB, 0.12237E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.12039E+01_JPRB, 0.12057E+01_JPRB, 0.12089E+01_JPRB, 0.12210E+01_JPRB, 0.12418E+01_JPRB, 0.14293E+01_JPRB ,&
&  0.14836E+01_JPRB, 0.16833E+01_JPRB, 0.15069E+01_JPRB, 0.13386E+01_JPRB, 0.13227E+01_JPRB, 0.12158E+01_JPRB, 0.11587E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.10133E+01_JPRB, 0.97636E+00_JPRB, 0.95957E+00_JPRB, 0.99570E+00_JPRB, 0.10425E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.11727E+01_JPRB, 0.12224E+01_JPRB, 0.12610E+01_JPRB, 0.12920E+01_JPRB, 0.13886E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.13578E+01_JPRB, 0.14951E+01_JPRB, 0.15346E+01_JPRB, 0.15925E+01_JPRB, 0.15229E+01_JPRB, 0.14342E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.11322E+01_JPRB, 0.11364E+01_JPRB, 0.13784E+01_JPRB, 0.16294E+01_JPRB, 0.18141E+01_JPRB, 0.19000E+01_JPRB, 0.17238E+01_JPRB ,&
&  0.15406E+01_JPRB, 0.14131E+01_JPRB, 0.12839E+01_JPRB, 0.11673E+01_JPRB, 0.11112E+01_JPRB, 0.11117E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.10718E+01_JPRB, 0.11101E+01_JPRB, 0.11894E+01_JPRB, 0.12847E+01_JPRB, 0.20673E+01_JPRB, 0.25417E+01_JPRB ,&
&  0.15778E+01_JPRB, 0.12538E+01_JPRB, 0.80484E+00_JPRB, 0.11582E+01_JPRB, 0.80441E+00_JPRB, 0.72979E+00_JPRB, 0.79280E+00_JPRB ,&
&  0.89276E+00_JPRB, 0.97033E+00_JPRB, 0.10073E+01_JPRB, 0.10503E+01_JPRB, 0.96433E+00_JPRB, 0.95390E+00_JPRB, 0.95610E+00_JPRB ,&
&  0.97115E+00_JPRB, 0.99193E+00_JPRB, 0.10101E+01_JPRB, 0.10238E+01_JPRB, 0.10407E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.10713E+01_JPRB, 0.10841E+01_JPRB, 0.10973E+01_JPRB, 0.11146E+01_JPRB, 0.11295E+01_JPRB, 0.11485E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.11762E+01_JPRB, 0.11768E+01_JPRB, 0.11903E+01_JPRB, 0.12044E+01_JPRB, 0.12313E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12972E+01_JPRB, 0.13344E+01_JPRB, 0.13770E+01_JPRB, 0.14036E+01_JPRB, 0.14255E+01_JPRB, 0.14472E+01_JPRB, 0.14648E+01_JPRB ,&
&  0.14734E+01_JPRB, 0.14715E+01_JPRB, 0.14886E+01_JPRB, 0.15235E+01_JPRB, 0.15632E+01_JPRB, 0.16326E+01_JPRB, 0.14459E+01_JPRB ,&
&  0.16427E+01_JPRB, 0.17193E+01_JPRB, 0.16534E+01_JPRB, 0.15091E+01_JPRB, 0.14560E+01_JPRB, 0.14173E+01_JPRB, 0.14111E+01_JPRB ,&
&  0.14393E+01_JPRB, 0.13943E+01_JPRB, 0.13468E+01_JPRB, 0.12642E+01_JPRB, 0.12209E+01_JPRB, 0.12420E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.13595E+01_JPRB, 0.13682E+01_JPRB, 0.13596E+01_JPRB, 0.13571E+01_JPRB, 0.13374E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.11883E+01_JPRB, 0.10413E+01_JPRB, 0.10264E+01_JPRB, 0.10203E+01_JPRB, 0.94015E+00_JPRB, 0.81169E+00_JPRB ,&
&  0.74500E+00_JPRB, 0.74562E+00_JPRB, 0.86861E+00_JPRB, 0.12733E+01_JPRB, 0.14975E+01_JPRB, 0.15484E+01_JPRB, 0.14755E+01_JPRB ,&
&  0.13439E+01_JPRB, 0.12165E+01_JPRB, 0.10708E+01_JPRB, 0.98659E+00_JPRB, 0.96888E+00_JPRB, 0.10070E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.10253E+01_JPRB, 0.10154E+01_JPRB, 0.10232E+01_JPRB, 0.10497E+01_JPRB, 0.12121E+01_JPRB, 0.15137E+01_JPRB, 0.22747E+01_JPRB ,&
&  0.16208E+01_JPRB, 0.95806E+00_JPRB, 0.91453E+00_JPRB, 0.96791E+00_JPRB, 0.75363E+00_JPRB, 0.68203E+00_JPRB, 0.77823E+00_JPRB ,&
&  0.99051E+00_JPRB, 0.11193E+01_JPRB, 0.11162E+01_JPRB, 0.10986E+01_JPRB, 0.10504E+01_JPRB, 0.10553E+01_JPRB, 0.10393E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.10337E+01_JPRB, 0.10399E+01_JPRB, 0.10462E+01_JPRB, 0.10579E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.10684E+01_JPRB, 0.10861E+01_JPRB, 0.11034E+01_JPRB, 0.11218E+01_JPRB, 0.11356E+01_JPRB, 0.11579E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.11796E+01_JPRB, 0.11865E+01_JPRB, 0.12037E+01_JPRB, 0.12251E+01_JPRB, 0.12464E+01_JPRB, 0.12752E+01_JPRB, 0.13103E+01_JPRB ,&
&  0.13538E+01_JPRB, 0.13867E+01_JPRB, 0.14256E+01_JPRB, 0.14495E+01_JPRB, 0.14493E+01_JPRB, 0.14634E+01_JPRB, 0.14742E+01_JPRB ,&
&  0.14653E+01_JPRB, 0.14542E+01_JPRB, 0.14705E+01_JPRB, 0.14958E+01_JPRB, 0.14951E+01_JPRB, 0.15348E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.16753E+01_JPRB, 0.18264E+01_JPRB, 0.18530E+01_JPRB, 0.17095E+01_JPRB, 0.16223E+01_JPRB, 0.16263E+01_JPRB, 0.16594E+01_JPRB ,&
&  0.17226E+01_JPRB, 0.17858E+01_JPRB, 0.17266E+01_JPRB, 0.16233E+01_JPRB, 0.15463E+01_JPRB, 0.15449E+01_JPRB, 0.15586E+01_JPRB ,&
&  0.15757E+01_JPRB, 0.15924E+01_JPRB, 0.15868E+01_JPRB, 0.15661E+01_JPRB, 0.15403E+01_JPRB, 0.15080E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13460E+01_JPRB, 0.11991E+01_JPRB, 0.12447E+01_JPRB, 0.11640E+01_JPRB, 0.99442E+00_JPRB, 0.82969E+00_JPRB ,&
&  0.74018E+00_JPRB, 0.76692E+00_JPRB, 0.95314E+00_JPRB, 0.11677E+01_JPRB, 0.12271E+01_JPRB, 0.12240E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.97671E+00_JPRB, 0.91371E+00_JPRB, 0.89566E+00_JPRB, 0.90768E+00_JPRB, 0.94124E+00_JPRB, 0.96342E+00_JPRB ,&
&  0.96820E+00_JPRB, 0.97111E+00_JPRB, 0.96223E+00_JPRB, 0.95589E+00_JPRB, 0.10207E+01_JPRB, 0.11507E+01_JPRB, 0.18344E+01_JPRB ,&
&  0.14538E+01_JPRB, 0.10543E+01_JPRB, 0.13609E+01_JPRB, 0.60394E+00_JPRB, 0.66516E+00_JPRB, 0.62208E+00_JPRB, 0.70882E+00_JPRB ,&
&  0.84667E+00_JPRB, 0.11809E+01_JPRB, 0.15940E+01_JPRB, 0.13997E+01_JPRB, 0.12785E+01_JPRB, 0.11743E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10432E+01_JPRB, 0.10428E+01_JPRB, 0.10407E+01_JPRB, 0.10511E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.10875E+01_JPRB, 0.11095E+01_JPRB, 0.11149E+01_JPRB, 0.11353E+01_JPRB, 0.11189E+01_JPRB, 0.11393E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.11793E+01_JPRB, 0.12024E+01_JPRB, 0.12238E+01_JPRB, 0.12556E+01_JPRB, 0.12844E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.13839E+01_JPRB, 0.14171E+01_JPRB, 0.14524E+01_JPRB, 0.14514E+01_JPRB, 0.14753E+01_JPRB, 0.14931E+01_JPRB ,&
&  0.15351E+01_JPRB, 0.15286E+01_JPRB, 0.15410E+01_JPRB, 0.15525E+01_JPRB, 0.15386E+01_JPRB, 0.15919E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.17262E+01_JPRB, 0.19129E+01_JPRB, 0.20012E+01_JPRB, 0.18985E+01_JPRB, 0.17826E+01_JPRB, 0.17539E+01_JPRB, 0.17630E+01_JPRB ,&
&  0.18114E+01_JPRB, 0.18422E+01_JPRB, 0.17792E+01_JPRB, 0.18314E+01_JPRB, 0.18518E+01_JPRB, 0.18046E+01_JPRB, 0.18396E+01_JPRB ,&
&  0.18649E+01_JPRB, 0.18888E+01_JPRB, 0.19097E+01_JPRB, 0.19231E+01_JPRB, 0.19202E+01_JPRB, 0.19535E+01_JPRB, 0.19610E+01_JPRB ,&
&  0.19347E+01_JPRB, 0.19292E+01_JPRB, 0.19211E+01_JPRB, 0.18084E+01_JPRB, 0.16865E+01_JPRB, 0.14295E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.93338E+00_JPRB, 0.89564E+00_JPRB, 0.99115E+00_JPRB, 0.11229E+01_JPRB, 0.11109E+01_JPRB, 0.10645E+01_JPRB, 0.99332E+00_JPRB ,&
&  0.90539E+00_JPRB, 0.88667E+00_JPRB, 0.86921E+00_JPRB, 0.87730E+00_JPRB, 0.89766E+00_JPRB, 0.91924E+00_JPRB, 0.93773E+00_JPRB ,&
&  0.94372E+00_JPRB, 0.94841E+00_JPRB, 0.91990E+00_JPRB, 0.89594E+00_JPRB, 0.94424E+00_JPRB, 0.11140E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.10668E+01_JPRB, 0.94351E+00_JPRB, 0.85927E+00_JPRB, 0.68110E+00_JPRB, 0.66073E+00_JPRB, 0.81934E+00_JPRB ,&
&  0.13237E+01_JPRB, 0.16876E+01_JPRB, 0.23864E+01_JPRB, 0.40178E+01_JPRB, 0.28454E+01_JPRB, 0.21255E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10489E+01_JPRB, 0.10406E+01_JPRB, 0.10345E+01_JPRB, 0.10583E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.10023E+01_JPRB, 0.10319E+01_JPRB, 0.10634E+01_JPRB, 0.10929E+01_JPRB, 0.11068E+01_JPRB, 0.11124E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.11261E+01_JPRB, 0.11535E+01_JPRB, 0.11355E+01_JPRB, 0.11601E+01_JPRB, 0.11522E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.12344E+01_JPRB, 0.12827E+01_JPRB, 0.13137E+01_JPRB, 0.13358E+01_JPRB, 0.13954E+01_JPRB, 0.14752E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.16161E+01_JPRB, 0.16673E+01_JPRB, 0.16950E+01_JPRB, 0.16619E+01_JPRB, 0.16277E+01_JPRB, 0.11734E+01_JPRB ,&
&  0.16539E+01_JPRB, 0.19350E+01_JPRB, 0.21410E+01_JPRB, 0.21159E+01_JPRB, 0.19691E+01_JPRB, 0.18509E+01_JPRB, 0.18495E+01_JPRB ,&
&  0.18521E+01_JPRB, 0.18064E+01_JPRB, 0.17394E+01_JPRB, 0.17368E+01_JPRB, 0.19296E+01_JPRB, 0.18753E+01_JPRB, 0.19124E+01_JPRB ,&
&  0.19484E+01_JPRB, 0.19942E+01_JPRB, 0.20551E+01_JPRB, 0.21331E+01_JPRB, 0.22033E+01_JPRB, 0.22764E+01_JPRB, 0.23610E+01_JPRB ,&
&  0.24165E+01_JPRB, 0.24265E+01_JPRB, 0.24435E+01_JPRB, 0.21835E+01_JPRB, 0.20071E+01_JPRB, 0.19787E+01_JPRB, 0.15615E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.98700E+00_JPRB, 0.10685E+01_JPRB, 0.11349E+01_JPRB, 0.11059E+01_JPRB, 0.10020E+01_JPRB, 0.93858E+00_JPRB ,&
&  0.89710E+00_JPRB, 0.87841E+00_JPRB, 0.86666E+00_JPRB, 0.89427E+00_JPRB, 0.91332E+00_JPRB, 0.93690E+00_JPRB, 0.94644E+00_JPRB ,&
&  0.96273E+00_JPRB, 0.94437E+00_JPRB, 0.91166E+00_JPRB, 0.90226E+00_JPRB, 0.96517E+00_JPRB, 0.11621E+01_JPRB, 0.14744E+01_JPRB ,&
&  0.15985E+01_JPRB, 0.95930E+00_JPRB, 0.92166E+00_JPRB, 0.81889E+00_JPRB, 0.77156E+00_JPRB, 0.80518E+00_JPRB, 0.97649E+00_JPRB ,&
&  0.14437E+01_JPRB, 0.21962E+01_JPRB, 0.24342E+01_JPRB, 0.20814E+01_JPRB, 0.25343E+01_JPRB, 0.18347E+01_JPRB, 0.10482E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10030E+01_JPRB, 0.96757E+00_JPRB, 0.95879E+00_JPRB, 0.97049E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.84010E+00_JPRB, 0.86007E+00_JPRB, 0.88491E+00_JPRB, 0.92304E+00_JPRB, 0.96586E+00_JPRB, 0.99619E+00_JPRB, 0.10272E+01_JPRB ,&
&  0.10390E+01_JPRB, 0.10583E+01_JPRB, 0.10763E+01_JPRB, 0.10421E+01_JPRB, 0.10585E+01_JPRB, 0.10497E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10849E+01_JPRB, 0.11202E+01_JPRB, 0.11600E+01_JPRB, 0.12169E+01_JPRB, 0.12873E+01_JPRB, 0.13782E+01_JPRB ,&
&  0.14895E+01_JPRB, 0.15869E+01_JPRB, 0.16897E+01_JPRB, 0.17522E+01_JPRB, 0.18027E+01_JPRB, 0.18248E+01_JPRB, 0.18089E+01_JPRB ,&
&  0.10723E+01_JPRB, 0.17661E+01_JPRB, 0.19711E+01_JPRB, 0.20993E+01_JPRB, 0.21709E+01_JPRB, 0.20460E+01_JPRB, 0.20295E+01_JPRB ,&
&  0.19101E+01_JPRB, 0.17393E+01_JPRB, 0.16988E+01_JPRB, 0.17041E+01_JPRB, 0.17965E+01_JPRB, 0.17761E+01_JPRB, 0.17829E+01_JPRB ,&
&  0.17988E+01_JPRB, 0.18505E+01_JPRB, 0.19196E+01_JPRB, 0.20005E+01_JPRB, 0.21056E+01_JPRB, 0.22293E+01_JPRB, 0.23972E+01_JPRB ,&
&  0.25633E+01_JPRB, 0.27121E+01_JPRB, 0.28342E+01_JPRB, 0.25359E+01_JPRB, 0.22464E+01_JPRB, 0.22561E+01_JPRB, 0.19406E+01_JPRB ,&
&  0.12987E+01_JPRB, 0.10445E+01_JPRB, 0.11169E+01_JPRB, 0.12396E+01_JPRB, 0.11785E+01_JPRB, 0.10862E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.96431E+00_JPRB, 0.92553E+00_JPRB, 0.95497E+00_JPRB, 0.93480E+00_JPRB, 0.97381E+00_JPRB, 0.97821E+00_JPRB, 0.99293E+00_JPRB ,&
&  0.10141E+01_JPRB, 0.10005E+01_JPRB, 0.98705E+00_JPRB, 0.98515E+00_JPRB, 0.97749E+00_JPRB, 0.97140E+00_JPRB, 0.97372E+00_JPRB ,&
&  0.98670E+00_JPRB, 0.93767E+00_JPRB, 0.89792E+00_JPRB, 0.89913E+00_JPRB, 0.89833E+00_JPRB, 0.90564E+00_JPRB, 0.99302E+00_JPRB ,&
&  0.11475E+01_JPRB, 0.13218E+01_JPRB, 0.16087E+01_JPRB, 0.16844E+01_JPRB, 0.16059E+01_JPRB, 0.12229E+01_JPRB, 0.91396E+00_JPRB ,&
&  0.90973E+00_JPRB, 0.86687E+00_JPRB, 0.83390E+00_JPRB, 0.81622E+00_JPRB, 0.82253E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.76358E+00_JPRB, 0.77858E+00_JPRB, 0.77253E+00_JPRB, 0.79846E+00_JPRB, 0.81916E+00_JPRB, 0.84944E+00_JPRB, 0.88163E+00_JPRB ,&
&  0.90862E+00_JPRB, 0.93284E+00_JPRB, 0.96727E+00_JPRB, 0.95238E+00_JPRB, 0.96929E+00_JPRB, 0.96722E+00_JPRB, 0.95134E+00_JPRB ,&
&  0.95422E+00_JPRB, 0.97434E+00_JPRB, 0.99672E+00_JPRB, 0.10222E+01_JPRB, 0.10562E+01_JPRB, 0.11139E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.13786E+01_JPRB, 0.15179E+01_JPRB, 0.16474E+01_JPRB, 0.17686E+01_JPRB, 0.18162E+01_JPRB, 0.17648E+01_JPRB ,&
&  0.86168E+00_JPRB, 0.10680E+01_JPRB, 0.16830E+01_JPRB, 0.19183E+01_JPRB, 0.20600E+01_JPRB, 0.20975E+01_JPRB, 0.20567E+01_JPRB ,&
&  0.18803E+01_JPRB, 0.16239E+01_JPRB, 0.15494E+01_JPRB, 0.17036E+01_JPRB, 0.16419E+01_JPRB, 0.16398E+01_JPRB, 0.16280E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.16580E+01_JPRB, 0.17069E+01_JPRB, 0.17661E+01_JPRB, 0.18737E+01_JPRB, 0.20163E+01_JPRB, 0.22137E+01_JPRB ,&
&  0.24577E+01_JPRB, 0.27060E+01_JPRB, 0.29265E+01_JPRB, 0.25834E+01_JPRB, 0.25146E+01_JPRB, 0.24616E+01_JPRB, 0.25116E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.11543E+01_JPRB, 0.12194E+01_JPRB, 0.13344E+01_JPRB, 0.12445E+01_JPRB, 0.11809E+01_JPRB, 0.11398E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10469E+01_JPRB, 0.10386E+01_JPRB, 0.10201E+01_JPRB, 0.10186E+01_JPRB, 0.10383E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10931E+01_JPRB, 0.10951E+01_JPRB, 0.10973E+01_JPRB, 0.10737E+01_JPRB, 0.10484E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.10231E+01_JPRB, 0.10212E+01_JPRB, 0.10489E+01_JPRB, 0.10752E+01_JPRB, 0.10812E+01_JPRB, 0.10827E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10631E+01_JPRB, 0.11829E+01_JPRB, 0.12239E+01_JPRB, 0.12029E+01_JPRB, 0.10163E+01_JPRB, 0.84721E+00_JPRB ,&
&  0.82703E+00_JPRB, 0.79801E+00_JPRB, 0.77229E+00_JPRB, 0.76589E+00_JPRB, 0.75987E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.70430E+00_JPRB, 0.68679E+00_JPRB, 0.67193E+00_JPRB, 0.67880E+00_JPRB, 0.68918E+00_JPRB, 0.70748E+00_JPRB, 0.75882E+00_JPRB ,&
&  0.80179E+00_JPRB, 0.81869E+00_JPRB, 0.84100E+00_JPRB, 0.84498E+00_JPRB, 0.85920E+00_JPRB, 0.85121E+00_JPRB, 0.85720E+00_JPRB ,&
&  0.85231E+00_JPRB, 0.86634E+00_JPRB, 0.89071E+00_JPRB, 0.91573E+00_JPRB, 0.93670E+00_JPRB, 0.96022E+00_JPRB, 0.99620E+00_JPRB ,&
&  0.10381E+01_JPRB, 0.10955E+01_JPRB, 0.11868E+01_JPRB, 0.13340E+01_JPRB, 0.15644E+01_JPRB, 0.18419E+01_JPRB, 0.20425E+01_JPRB ,&
&  0.15666E+01_JPRB, 0.69389E+00_JPRB, 0.12902E+01_JPRB, 0.17793E+01_JPRB, 0.18543E+01_JPRB, 0.19320E+01_JPRB, 0.18547E+01_JPRB ,&
&  0.16867E+01_JPRB, 0.15331E+01_JPRB, 0.14291E+01_JPRB, 0.15551E+01_JPRB, 0.15316E+01_JPRB, 0.15236E+01_JPRB, 0.15161E+01_JPRB ,&
&  0.14983E+01_JPRB, 0.14979E+01_JPRB, 0.15085E+01_JPRB, 0.15486E+01_JPRB, 0.16320E+01_JPRB, 0.17688E+01_JPRB, 0.19525E+01_JPRB ,&
&  0.22051E+01_JPRB, 0.25492E+01_JPRB, 0.28685E+01_JPRB, 0.26374E+01_JPRB, 0.26849E+01_JPRB, 0.24063E+01_JPRB, 0.21229E+01_JPRB ,&
&  0.15014E+01_JPRB, 0.12281E+01_JPRB, 0.12651E+01_JPRB, 0.14021E+01_JPRB, 0.13106E+01_JPRB, 0.11530E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.11664E+01_JPRB, 0.11255E+01_JPRB, 0.11050E+01_JPRB, 0.10802E+01_JPRB, 0.10796E+01_JPRB, 0.10873E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.11086E+01_JPRB, 0.11280E+01_JPRB, 0.11424E+01_JPRB, 0.11395E+01_JPRB, 0.11196E+01_JPRB, 0.11151E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.11555E+01_JPRB, 0.11759E+01_JPRB, 0.11942E+01_JPRB, 0.12067E+01_JPRB, 0.11719E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.11266E+01_JPRB, 0.11047E+01_JPRB, 0.10825E+01_JPRB, 0.10553E+01_JPRB, 0.10302E+01_JPRB, 0.95138E+00_JPRB, 0.85744E+00_JPRB ,&
&  0.82566E+00_JPRB, 0.79040E+00_JPRB, 0.76477E+00_JPRB, 0.74657E+00_JPRB, 0.72742E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.75180E+00_JPRB, 0.73179E+00_JPRB, 0.70021E+00_JPRB, 0.65773E+00_JPRB, 0.65298E+00_JPRB, 0.64976E+00_JPRB, 0.66077E+00_JPRB ,&
&  0.70292E+00_JPRB, 0.75675E+00_JPRB, 0.76665E+00_JPRB, 0.76343E+00_JPRB, 0.74324E+00_JPRB, 0.74349E+00_JPRB, 0.74848E+00_JPRB ,&
&  0.76115E+00_JPRB, 0.77098E+00_JPRB, 0.79263E+00_JPRB, 0.82183E+00_JPRB, 0.85304E+00_JPRB, 0.87846E+00_JPRB, 0.89147E+00_JPRB ,&
&  0.90525E+00_JPRB, 0.91287E+00_JPRB, 0.93832E+00_JPRB, 0.98048E+00_JPRB, 0.10710E+01_JPRB, 0.12642E+01_JPRB, 0.15949E+01_JPRB ,&
&  0.23700E+01_JPRB, 0.19593E+01_JPRB, 0.93756E+00_JPRB, 0.16061E+01_JPRB, 0.17883E+01_JPRB, 0.19125E+01_JPRB, 0.16415E+01_JPRB ,&
&  0.15622E+01_JPRB, 0.13947E+01_JPRB, 0.12887E+01_JPRB, 0.13786E+01_JPRB, 0.13550E+01_JPRB, 0.13532E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.13128E+01_JPRB, 0.13160E+01_JPRB, 0.13228E+01_JPRB, 0.13450E+01_JPRB, 0.13882E+01_JPRB, 0.14887E+01_JPRB, 0.16766E+01_JPRB ,&
&  0.19143E+01_JPRB, 0.22240E+01_JPRB, 0.26655E+01_JPRB, 0.28120E+01_JPRB, 0.30117E+01_JPRB, 0.23428E+01_JPRB, 0.19105E+01_JPRB ,&
&  0.14217E+01_JPRB, 0.13708E+01_JPRB, 0.14557E+01_JPRB, 0.14718E+01_JPRB, 0.13632E+01_JPRB, 0.12418E+01_JPRB, 0.13016E+01_JPRB ,&
&  0.12411E+01_JPRB, 0.11941E+01_JPRB, 0.11725E+01_JPRB, 0.11432E+01_JPRB, 0.11254E+01_JPRB, 0.11210E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.11394E+01_JPRB, 0.11530E+01_JPRB, 0.11617E+01_JPRB, 0.11691E+01_JPRB, 0.11685E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.12360E+01_JPRB, 0.12631E+01_JPRB, 0.12859E+01_JPRB, 0.12590E+01_JPRB, 0.12069E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.11787E+01_JPRB, 0.11311E+01_JPRB, 0.11339E+01_JPRB, 0.10680E+01_JPRB, 0.98414E+00_JPRB, 0.89898E+00_JPRB ,&
&  0.88629E+00_JPRB, 0.87213E+00_JPRB, 0.82554E+00_JPRB, 0.77163E+00_JPRB, 0.75403E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.80811E+00_JPRB, 0.81281E+00_JPRB, 0.80360E+00_JPRB, 0.76066E+00_JPRB, 0.72076E+00_JPRB, 0.68227E+00_JPRB, 0.65460E+00_JPRB ,&
&  0.64948E+00_JPRB, 0.65668E+00_JPRB, 0.68538E+00_JPRB, 0.70700E+00_JPRB, 0.69816E+00_JPRB, 0.67659E+00_JPRB, 0.66957E+00_JPRB ,&
&  0.67635E+00_JPRB, 0.69657E+00_JPRB, 0.71306E+00_JPRB, 0.73956E+00_JPRB, 0.76678E+00_JPRB, 0.79037E+00_JPRB, 0.81647E+00_JPRB ,&
&  0.80883E+00_JPRB, 0.81171E+00_JPRB, 0.82398E+00_JPRB, 0.83722E+00_JPRB, 0.86279E+00_JPRB, 0.92484E+00_JPRB, 0.10034E+01_JPRB ,&
&  0.12798E+01_JPRB, 0.21393E+01_JPRB, 0.12571E+01_JPRB, 0.17212E+01_JPRB, 0.19366E+01_JPRB, 0.19204E+01_JPRB, 0.16879E+01_JPRB ,&
&  0.15184E+01_JPRB, 0.13271E+01_JPRB, 0.12631E+01_JPRB, 0.12612E+01_JPRB, 0.11614E+01_JPRB, 0.11493E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.11567E+01_JPRB, 0.11696E+01_JPRB, 0.11868E+01_JPRB, 0.11957E+01_JPRB, 0.12273E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.18244E+01_JPRB, 0.22996E+01_JPRB, 0.26656E+01_JPRB, 0.31446E+01_JPRB, 0.25319E+01_JPRB, 0.19834E+01_JPRB ,&
&  0.15688E+01_JPRB, 0.15610E+01_JPRB, 0.16011E+01_JPRB, 0.15921E+01_JPRB, 0.14496E+01_JPRB, 0.14109E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.12760E+01_JPRB, 0.12390E+01_JPRB, 0.12077E+01_JPRB, 0.11701E+01_JPRB, 0.11444E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.11273E+01_JPRB, 0.11453E+01_JPRB, 0.11584E+01_JPRB, 0.11770E+01_JPRB, 0.11879E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.12323E+01_JPRB, 0.12594E+01_JPRB, 0.12662E+01_JPRB, 0.12508E+01_JPRB, 0.12248E+01_JPRB, 0.12242E+01_JPRB ,&
&  0.12389E+01_JPRB, 0.12695E+01_JPRB, 0.11735E+01_JPRB, 0.11457E+01_JPRB, 0.11664E+01_JPRB, 0.10969E+01_JPRB, 0.10039E+01_JPRB ,&
&  0.10234E+01_JPRB, 0.11595E+01_JPRB, 0.11974E+01_JPRB, 0.84245E+00_JPRB, 0.81401E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.83401E+00_JPRB, 0.83095E+00_JPRB, 0.81494E+00_JPRB, 0.80858E+00_JPRB, 0.77302E+00_JPRB, 0.73546E+00_JPRB, 0.70225E+00_JPRB ,&
&  0.68076E+00_JPRB, 0.66948E+00_JPRB, 0.67260E+00_JPRB, 0.67144E+00_JPRB, 0.67135E+00_JPRB, 0.66280E+00_JPRB, 0.65724E+00_JPRB ,&
&  0.64437E+00_JPRB, 0.64945E+00_JPRB, 0.66580E+00_JPRB, 0.69270E+00_JPRB, 0.72389E+00_JPRB, 0.73495E+00_JPRB, 0.74672E+00_JPRB ,&
&  0.75060E+00_JPRB, 0.74830E+00_JPRB, 0.75040E+00_JPRB, 0.76067E+00_JPRB, 0.78124E+00_JPRB, 0.80743E+00_JPRB, 0.84880E+00_JPRB ,&
&  0.95003E+00_JPRB, 0.18241E+01_JPRB, 0.27713E+01_JPRB, 0.31036E+01_JPRB, 0.26240E+01_JPRB, 0.21517E+01_JPRB, 0.18798E+01_JPRB ,&
&  0.17772E+01_JPRB, 0.16242E+01_JPRB, 0.15164E+01_JPRB, 0.12905E+01_JPRB, 0.11131E+01_JPRB, 0.10444E+01_JPRB, 0.10330E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10828E+01_JPRB, 0.10894E+01_JPRB, 0.10916E+01_JPRB, 0.10763E+01_JPRB, 0.10725E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.11734E+01_JPRB, 0.13623E+01_JPRB, 0.17073E+01_JPRB, 0.22894E+01_JPRB, 0.29265E+01_JPRB, 0.26597E+01_JPRB, 0.22332E+01_JPRB ,&
&  0.21135E+01_JPRB, 0.19944E+01_JPRB, 0.18646E+01_JPRB, 0.17841E+01_JPRB, 0.15044E+01_JPRB, 0.17335E+01_JPRB, 0.16628E+01_JPRB ,&
&  0.15769E+01_JPRB, 0.15135E+01_JPRB, 0.14200E+01_JPRB, 0.13250E+01_JPRB, 0.12506E+01_JPRB, 0.11785E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.11148E+01_JPRB, 0.11264E+01_JPRB, 0.11471E+01_JPRB, 0.11742E+01_JPRB, 0.11962E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.11888E+01_JPRB, 0.11844E+01_JPRB, 0.11488E+01_JPRB, 0.11528E+01_JPRB, 0.11487E+01_JPRB, 0.11203E+01_JPRB, 0.11011E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.11779E+01_JPRB, 0.11667E+01_JPRB, 0.11598E+01_JPRB, 0.11493E+01_JPRB, 0.11427E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.99060E+00_JPRB, 0.10351E+01_JPRB, 0.97534E+00_JPRB, 0.89279E+00_JPRB, 0.85827E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.86399E+00_JPRB, 0.83512E+00_JPRB, 0.81612E+00_JPRB, 0.81502E+00_JPRB, 0.79116E+00_JPRB, 0.77429E+00_JPRB, 0.76641E+00_JPRB ,&
&  0.72682E+00_JPRB, 0.69864E+00_JPRB, 0.67610E+00_JPRB, 0.66892E+00_JPRB, 0.66940E+00_JPRB, 0.66227E+00_JPRB, 0.66337E+00_JPRB ,&
&  0.65427E+00_JPRB, 0.64394E+00_JPRB, 0.64555E+00_JPRB, 0.66667E+00_JPRB, 0.69539E+00_JPRB, 0.70291E+00_JPRB, 0.71897E+00_JPRB ,&
&  0.73557E+00_JPRB, 0.73309E+00_JPRB, 0.72409E+00_JPRB, 0.72338E+00_JPRB, 0.72884E+00_JPRB, 0.72867E+00_JPRB, 0.75322E+00_JPRB ,&
&  0.85463E+00_JPRB, 0.18784E+01_JPRB, 0.19987E+01_JPRB, 0.34734E+01_JPRB, 0.29751E+01_JPRB, 0.24950E+01_JPRB, 0.19344E+01_JPRB ,&
&  0.19094E+01_JPRB, 0.21060E+01_JPRB, 0.17011E+01_JPRB, 0.14204E+01_JPRB, 0.12042E+01_JPRB, 0.10560E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.10232E+01_JPRB, 0.10465E+01_JPRB, 0.10634E+01_JPRB, 0.10380E+01_JPRB, 0.10091E+01_JPRB, 0.99909E+00_JPRB, 0.10034E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.10705E+01_JPRB, 0.12401E+01_JPRB, 0.15786E+01_JPRB, 0.23756E+01_JPRB, 0.27666E+01_JPRB, 0.25496E+01_JPRB ,&
&  0.24731E+01_JPRB, 0.26000E+01_JPRB, 0.23868E+01_JPRB, 0.21493E+01_JPRB, 0.18666E+01_JPRB, 0.18593E+01_JPRB, 0.17851E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.16934E+01_JPRB, 0.16441E+01_JPRB, 0.15490E+01_JPRB, 0.14400E+01_JPRB, 0.13229E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.11361E+01_JPRB, 0.11017E+01_JPRB, 0.10850E+01_JPRB, 0.11009E+01_JPRB, 0.11397E+01_JPRB, 0.11719E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.11718E+01_JPRB, 0.11017E+01_JPRB, 0.10550E+01_JPRB, 0.10578E+01_JPRB, 0.10332E+01_JPRB, 0.10247E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.11172E+01_JPRB, 0.11289E+01_JPRB, 0.11252E+01_JPRB, 0.11342E+01_JPRB, 0.11754E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.10066E+01_JPRB, 0.10154E+01_JPRB, 0.10143E+01_JPRB, 0.94683E+00_JPRB, 0.90673E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.92367E+00_JPRB, 0.87630E+00_JPRB, 0.83839E+00_JPRB, 0.82074E+00_JPRB, 0.81016E+00_JPRB, 0.79658E+00_JPRB, 0.78390E+00_JPRB ,&
&  0.75873E+00_JPRB, 0.72848E+00_JPRB, 0.70409E+00_JPRB, 0.68311E+00_JPRB, 0.68065E+00_JPRB, 0.67393E+00_JPRB, 0.66538E+00_JPRB ,&
&  0.65401E+00_JPRB, 0.64746E+00_JPRB, 0.64457E+00_JPRB, 0.65267E+00_JPRB, 0.66915E+00_JPRB, 0.68044E+00_JPRB, 0.69646E+00_JPRB ,&
&  0.70536E+00_JPRB, 0.71573E+00_JPRB, 0.71823E+00_JPRB, 0.71400E+00_JPRB, 0.70581E+00_JPRB, 0.69463E+00_JPRB, 0.68687E+00_JPRB ,&
&  0.82950E+00_JPRB, 0.22120E+01_JPRB, 0.17624E+01_JPRB, 0.31551E+01_JPRB, 0.27304E+01_JPRB, 0.24450E+01_JPRB, 0.19422E+01_JPRB ,&
&  0.18306E+01_JPRB, 0.19788E+01_JPRB, 0.17390E+01_JPRB, 0.15111E+01_JPRB, 0.13442E+01_JPRB, 0.12023E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.10628E+01_JPRB, 0.10467E+01_JPRB, 0.10267E+01_JPRB, 0.99727E+00_JPRB, 0.98568E+00_JPRB, 0.97594E+00_JPRB, 0.97905E+00_JPRB ,&
&  0.98392E+00_JPRB, 0.99566E+00_JPRB, 0.10533E+01_JPRB, 0.12796E+01_JPRB, 0.17271E+01_JPRB, 0.24014E+01_JPRB, 0.28554E+01_JPRB ,&
&  0.29603E+01_JPRB, 0.32608E+01_JPRB, 0.29357E+01_JPRB, 0.25557E+01_JPRB, 0.22797E+01_JPRB, 0.21040E+01_JPRB, 0.19554E+01_JPRB ,&
&  0.18094E+01_JPRB, 0.17248E+01_JPRB, 0.17020E+01_JPRB, 0.16821E+01_JPRB, 0.16374E+01_JPRB, 0.15447E+01_JPRB, 0.14214E+01_JPRB ,&
&  0.12955E+01_JPRB, 0.12108E+01_JPRB, 0.11490E+01_JPRB, 0.11194E+01_JPRB, 0.11149E+01_JPRB, 0.11290E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.11266E+01_JPRB, 0.10885E+01_JPRB, 0.10292E+01_JPRB, 0.10287E+01_JPRB, 0.10142E+01_JPRB, 0.99855E+00_JPRB ,&
&  0.10479E+01_JPRB, 0.10792E+01_JPRB, 0.10850E+01_JPRB, 0.11229E+01_JPRB, 0.11253E+01_JPRB, 0.12207E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10044E+01_JPRB, 0.99759E+00_JPRB, 0.97622E+00_JPRB, 0.96029E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.97538E+00_JPRB, 0.94152E+00_JPRB, 0.90372E+00_JPRB, 0.86900E+00_JPRB, 0.84087E+00_JPRB, 0.82334E+00_JPRB, 0.80700E+00_JPRB ,&
&  0.77932E+00_JPRB, 0.75126E+00_JPRB, 0.73356E+00_JPRB, 0.71834E+00_JPRB, 0.70665E+00_JPRB, 0.69892E+00_JPRB, 0.68195E+00_JPRB ,&
&  0.66508E+00_JPRB, 0.64550E+00_JPRB, 0.65171E+00_JPRB, 0.66270E+00_JPRB, 0.67256E+00_JPRB, 0.68161E+00_JPRB, 0.70145E+00_JPRB ,&
&  0.69857E+00_JPRB, 0.69968E+00_JPRB, 0.70324E+00_JPRB, 0.70470E+00_JPRB, 0.69425E+00_JPRB, 0.69352E+00_JPRB, 0.67312E+00_JPRB ,&
&  0.79513E+00_JPRB, 0.95657E+00_JPRB, 0.19759E+01_JPRB, 0.27090E+01_JPRB, 0.24730E+01_JPRB, 0.22130E+01_JPRB, 0.19952E+01_JPRB ,&
&  0.19436E+01_JPRB, 0.18460E+01_JPRB, 0.16360E+01_JPRB, 0.14880E+01_JPRB, 0.13623E+01_JPRB, 0.12674E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.11402E+01_JPRB, 0.10794E+01_JPRB, 0.10328E+01_JPRB, 0.98951E+00_JPRB, 0.96633E+00_JPRB, 0.95456E+00_JPRB, 0.95576E+00_JPRB ,&
&  0.95835E+00_JPRB, 0.95782E+00_JPRB, 0.97267E+00_JPRB, 0.10853E+01_JPRB, 0.14652E+01_JPRB, 0.20282E+01_JPRB, 0.26925E+01_JPRB ,&
&  0.32794E+01_JPRB, 0.34890E+01_JPRB, 0.31868E+01_JPRB, 0.28355E+01_JPRB, 0.25150E+01_JPRB, 0.22737E+01_JPRB, 0.20904E+01_JPRB ,&
&  0.19213E+01_JPRB, 0.17815E+01_JPRB, 0.16770E+01_JPRB, 0.16215E+01_JPRB, 0.15854E+01_JPRB, 0.15520E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.14363E+01_JPRB, 0.13511E+01_JPRB, 0.12668E+01_JPRB, 0.11964E+01_JPRB, 0.11433E+01_JPRB, 0.11233E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.10826E+01_JPRB, 0.10690E+01_JPRB, 0.10607E+01_JPRB, 0.10289E+01_JPRB, 0.10461E+01_JPRB, 0.10491E+01_JPRB, 0.10512E+01_JPRB ,&
&  0.10780E+01_JPRB, 0.10913E+01_JPRB, 0.10802E+01_JPRB, 0.11334E+01_JPRB, 0.11321E+01_JPRB, 0.12010E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.10271E+01_JPRB, 0.10039E+01_JPRB, 0.99105E+00_JPRB, 0.98971E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.10136E+01_JPRB, 0.99602E+00_JPRB, 0.95381E+00_JPRB, 0.90976E+00_JPRB, 0.87584E+00_JPRB, 0.84928E+00_JPRB, 0.82574E+00_JPRB ,&
&  0.79957E+00_JPRB, 0.76892E+00_JPRB, 0.74613E+00_JPRB, 0.73504E+00_JPRB, 0.73111E+00_JPRB, 0.72593E+00_JPRB, 0.70807E+00_JPRB ,&
&  0.68652E+00_JPRB, 0.67049E+00_JPRB, 0.67160E+00_JPRB, 0.67828E+00_JPRB, 0.68930E+00_JPRB, 0.68469E+00_JPRB, 0.69687E+00_JPRB ,&
&  0.70387E+00_JPRB, 0.70227E+00_JPRB, 0.69989E+00_JPRB, 0.69711E+00_JPRB, 0.69160E+00_JPRB, 0.68716E+00_JPRB, 0.67069E+00_JPRB ,&
&  0.74569E+00_JPRB, 0.11971E+01_JPRB, 0.18768E+01_JPRB, 0.23286E+01_JPRB, 0.23019E+01_JPRB, 0.20297E+01_JPRB, 0.18657E+01_JPRB ,&
&  0.17248E+01_JPRB, 0.16391E+01_JPRB, 0.14864E+01_JPRB, 0.13805E+01_JPRB, 0.12846E+01_JPRB, 0.12088E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.11070E+01_JPRB, 0.10697E+01_JPRB, 0.10282E+01_JPRB, 0.99127E+00_JPRB, 0.96538E+00_JPRB, 0.94473E+00_JPRB, 0.93612E+00_JPRB ,&
&  0.93527E+00_JPRB, 0.93125E+00_JPRB, 0.93378E+00_JPRB, 0.97938E+00_JPRB, 0.12661E+01_JPRB, 0.14782E+01_JPRB, 0.22744E+01_JPRB ,&
&  0.29617E+01_JPRB, 0.27877E+01_JPRB, 0.26568E+01_JPRB, 0.24610E+01_JPRB, 0.22304E+01_JPRB, 0.20458E+01_JPRB, 0.18568E+01_JPRB ,&
&  0.16918E+01_JPRB, 0.15592E+01_JPRB, 0.14526E+01_JPRB, 0.13815E+01_JPRB, 0.13359E+01_JPRB, 0.13082E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.12695E+01_JPRB, 0.12488E+01_JPRB, 0.12166E+01_JPRB, 0.11673E+01_JPRB, 0.11166E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.10284E+01_JPRB, 0.10128E+01_JPRB, 0.10269E+01_JPRB, 0.10053E+01_JPRB, 0.10434E+01_JPRB, 0.10833E+01_JPRB, 0.10864E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.11120E+01_JPRB, 0.11060E+01_JPRB, 0.11093E+01_JPRB, 0.11818E+01_JPRB, 0.12569E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.10650E+01_JPRB, 0.10380E+01_JPRB, 0.10113E+01_JPRB, 0.10198E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.10582E+01_JPRB, 0.10110E+01_JPRB, 0.96675E+00_JPRB, 0.92206E+00_JPRB, 0.89036E+00_JPRB, 0.85632E+00_JPRB, 0.82387E+00_JPRB ,&
&  0.79590E+00_JPRB, 0.77808E+00_JPRB, 0.76306E+00_JPRB, 0.74705E+00_JPRB, 0.74454E+00_JPRB, 0.73665E+00_JPRB, 0.72070E+00_JPRB ,&
&  0.69907E+00_JPRB, 0.68429E+00_JPRB, 0.67962E+00_JPRB, 0.68188E+00_JPRB, 0.69327E+00_JPRB, 0.70010E+00_JPRB, 0.70864E+00_JPRB ,&
&  0.71441E+00_JPRB, 0.69602E+00_JPRB, 0.68571E+00_JPRB, 0.68653E+00_JPRB, 0.68635E+00_JPRB, 0.68838E+00_JPRB, 0.67728E+00_JPRB ,&
&  0.72070E+00_JPRB, 0.97173E+00_JPRB, 0.13670E+01_JPRB, 0.18798E+01_JPRB, 0.19499E+01_JPRB, 0.18111E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.15544E+01_JPRB, 0.14588E+01_JPRB, 0.13390E+01_JPRB, 0.12062E+01_JPRB, 0.11040E+01_JPRB, 0.10381E+01_JPRB, 0.99627E+00_JPRB ,&
&  0.97699E+00_JPRB, 0.96291E+00_JPRB, 0.94491E+00_JPRB, 0.92997E+00_JPRB, 0.91528E+00_JPRB, 0.90261E+00_JPRB, 0.89657E+00_JPRB ,&
&  0.89105E+00_JPRB, 0.88468E+00_JPRB, 0.88135E+00_JPRB, 0.89308E+00_JPRB, 0.10427E+01_JPRB, 0.11962E+01_JPRB, 0.15410E+01_JPRB ,&
&  0.16857E+01_JPRB, 0.17331E+01_JPRB, 0.17784E+01_JPRB, 0.17548E+01_JPRB, 0.16740E+01_JPRB, 0.15948E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.14242E+01_JPRB, 0.13249E+01_JPRB, 0.12575E+01_JPRB, 0.12086E+01_JPRB, 0.11630E+01_JPRB, 0.11284E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.11131E+01_JPRB, 0.11253E+01_JPRB, 0.11295E+01_JPRB, 0.11128E+01_JPRB, 0.10850E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.96972E+00_JPRB, 0.97076E+00_JPRB, 0.98694E+00_JPRB, 0.10149E+01_JPRB, 0.10267E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10702E+01_JPRB, 0.11335E+01_JPRB, 0.11251E+01_JPRB, 0.11490E+01_JPRB, 0.12146E+01_JPRB, 0.12836E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.11154E+01_JPRB, 0.10807E+01_JPRB, 0.10502E+01_JPRB, 0.10895E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.11473E+01_JPRB, 0.10681E+01_JPRB, 0.10043E+01_JPRB, 0.95719E+00_JPRB, 0.92430E+00_JPRB, 0.88517E+00_JPRB, 0.84096E+00_JPRB ,&
&  0.80821E+00_JPRB, 0.79430E+00_JPRB, 0.78029E+00_JPRB, 0.75905E+00_JPRB, 0.74339E+00_JPRB, 0.72741E+00_JPRB, 0.71373E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.69012E+00_JPRB, 0.68536E+00_JPRB, 0.68895E+00_JPRB, 0.68947E+00_JPRB, 0.69362E+00_JPRB, 0.70469E+00_JPRB ,&
&  0.70789E+00_JPRB, 0.68884E+00_JPRB, 0.68540E+00_JPRB, 0.68509E+00_JPRB, 0.68372E+00_JPRB, 0.68308E+00_JPRB, 0.67485E+00_JPRB ,&
&  0.72417E+00_JPRB, 0.81154E+00_JPRB, 0.10515E+01_JPRB, 0.12984E+01_JPRB, 0.14157E+01_JPRB, 0.14732E+01_JPRB, 0.14350E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.12767E+01_JPRB, 0.11787E+01_JPRB, 0.10825E+01_JPRB, 0.99395E+00_JPRB, 0.93341E+00_JPRB, 0.89756E+00_JPRB ,&
&  0.87614E+00_JPRB, 0.86631E+00_JPRB, 0.85719E+00_JPRB, 0.85098E+00_JPRB, 0.84464E+00_JPRB, 0.83557E+00_JPRB, 0.83383E+00_JPRB ,&
&  0.83771E+00_JPRB, 0.82704E+00_JPRB, 0.82302E+00_JPRB, 0.83661E+00_JPRB, 0.86921E+00_JPRB, 0.92917E+00_JPRB, 0.10889E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.12536E+01_JPRB, 0.13232E+01_JPRB, 0.13425E+01_JPRB, 0.13125E+01_JPRB, 0.12705E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.11318E+01_JPRB, 0.10978E+01_JPRB, 0.10643E+01_JPRB, 0.10349E+01_JPRB, 0.10147E+01_JPRB, 0.10013E+01_JPRB ,&
&  0.99111E+00_JPRB, 0.99761E+00_JPRB, 0.10082E+01_JPRB, 0.10160E+01_JPRB, 0.10139E+01_JPRB, 0.10094E+01_JPRB, 0.10014E+01_JPRB ,&
&  0.97793E+00_JPRB, 0.94448E+00_JPRB, 0.93137E+00_JPRB, 0.94264E+00_JPRB, 0.97574E+00_JPRB, 0.99371E+00_JPRB, 0.10033E+01_JPRB ,&
&  0.10239E+01_JPRB, 0.10602E+01_JPRB, 0.10726E+01_JPRB, 0.11979E+01_JPRB, 0.12903E+01_JPRB, 0.12621E+01_JPRB, 0.12197E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.11381E+01_JPRB, 0.10990E+01_JPRB, 0.10635E+01_JPRB, 0.11394E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.97932E+00_JPRB, 0.97232E+00_JPRB, 0.94335E+00_JPRB, 0.91777E+00_JPRB, 0.90324E+00_JPRB, 0.88804E+00_JPRB, 0.86226E+00_JPRB ,&
&  0.83387E+00_JPRB, 0.80787E+00_JPRB, 0.79107E+00_JPRB, 0.76911E+00_JPRB, 0.74801E+00_JPRB, 0.73447E+00_JPRB, 0.72115E+00_JPRB ,&
&  0.70337E+00_JPRB, 0.69754E+00_JPRB, 0.69651E+00_JPRB, 0.69410E+00_JPRB, 0.68937E+00_JPRB, 0.68654E+00_JPRB, 0.69323E+00_JPRB ,&
&  0.69415E+00_JPRB, 0.69253E+00_JPRB, 0.69212E+00_JPRB, 0.68679E+00_JPRB, 0.67998E+00_JPRB, 0.67792E+00_JPRB, 0.66639E+00_JPRB ,&
&  0.68309E+00_JPRB, 0.68116E+00_JPRB, 0.79774E+00_JPRB, 0.94129E+00_JPRB, 0.10637E+01_JPRB, 0.11511E+01_JPRB, 0.12087E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11219E+01_JPRB, 0.10474E+01_JPRB, 0.99691E+00_JPRB, 0.94555E+00_JPRB, 0.90388E+00_JPRB, 0.86943E+00_JPRB ,&
&  0.84662E+00_JPRB, 0.83378E+00_JPRB, 0.82392E+00_JPRB, 0.81448E+00_JPRB, 0.80453E+00_JPRB, 0.79586E+00_JPRB, 0.79086E+00_JPRB ,&
&  0.78514E+00_JPRB, 0.78421E+00_JPRB, 0.78573E+00_JPRB, 0.78382E+00_JPRB, 0.79259E+00_JPRB, 0.83318E+00_JPRB, 0.90854E+00_JPRB ,&
&  0.95250E+00_JPRB, 0.10025E+01_JPRB, 0.10667E+01_JPRB, 0.10952E+01_JPRB, 0.10773E+01_JPRB, 0.10536E+01_JPRB, 0.10218E+01_JPRB ,&
&  0.99593E+00_JPRB, 0.96506E+00_JPRB, 0.94938E+00_JPRB, 0.93157E+00_JPRB, 0.91162E+00_JPRB, 0.89684E+00_JPRB, 0.89026E+00_JPRB ,&
&  0.88936E+00_JPRB, 0.89001E+00_JPRB, 0.89813E+00_JPRB, 0.90897E+00_JPRB, 0.91122E+00_JPRB, 0.91052E+00_JPRB, 0.91464E+00_JPRB ,&
&  0.92018E+00_JPRB, 0.91264E+00_JPRB, 0.89840E+00_JPRB, 0.90754E+00_JPRB, 0.93321E+00_JPRB, 0.95297E+00_JPRB, 0.96648E+00_JPRB ,&
&  0.98003E+00_JPRB, 0.10009E+01_JPRB, 0.10269E+01_JPRB, 0.11122E+01_JPRB, 0.11365E+01_JPRB, 0.11165E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.10816E+01_JPRB, 0.10605E+01_JPRB, 0.10351E+01_JPRB, 0.10111E+01_JPRB, 0.92822E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.91372E+00_JPRB, 0.90331E+00_JPRB, 0.88164E+00_JPRB, 0.85999E+00_JPRB, 0.85052E+00_JPRB, 0.84712E+00_JPRB, 0.84461E+00_JPRB ,&
&  0.83558E+00_JPRB, 0.82270E+00_JPRB, 0.80791E+00_JPRB, 0.79107E+00_JPRB, 0.77309E+00_JPRB, 0.75782E+00_JPRB, 0.74349E+00_JPRB ,&
&  0.72206E+00_JPRB, 0.70925E+00_JPRB, 0.70750E+00_JPRB, 0.70427E+00_JPRB, 0.70147E+00_JPRB, 0.69697E+00_JPRB, 0.69688E+00_JPRB ,&
&  0.69947E+00_JPRB, 0.70048E+00_JPRB, 0.70042E+00_JPRB, 0.69314E+00_JPRB, 0.68710E+00_JPRB, 0.68632E+00_JPRB, 0.68124E+00_JPRB ,&
&  0.68274E+00_JPRB, 0.59303E+00_JPRB, 0.67864E+00_JPRB, 0.78455E+00_JPRB, 0.84512E+00_JPRB, 0.92772E+00_JPRB, 0.10226E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10313E+01_JPRB, 0.98366E+00_JPRB, 0.94220E+00_JPRB, 0.90078E+00_JPRB, 0.86632E+00_JPRB, 0.83846E+00_JPRB ,&
&  0.82051E+00_JPRB, 0.80907E+00_JPRB, 0.79856E+00_JPRB, 0.78547E+00_JPRB, 0.77165E+00_JPRB, 0.76389E+00_JPRB, 0.75860E+00_JPRB ,&
&  0.75651E+00_JPRB, 0.75860E+00_JPRB, 0.75939E+00_JPRB, 0.76565E+00_JPRB, 0.77118E+00_JPRB, 0.78710E+00_JPRB, 0.82386E+00_JPRB ,&
&  0.84558E+00_JPRB, 0.87302E+00_JPRB, 0.91482E+00_JPRB, 0.92833E+00_JPRB, 0.91694E+00_JPRB, 0.90502E+00_JPRB, 0.88913E+00_JPRB ,&
&  0.86892E+00_JPRB, 0.83952E+00_JPRB, 0.82664E+00_JPRB, 0.81467E+00_JPRB, 0.80480E+00_JPRB, 0.80026E+00_JPRB, 0.80740E+00_JPRB ,&
&  0.81521E+00_JPRB, 0.81746E+00_JPRB, 0.82235E+00_JPRB, 0.83087E+00_JPRB, 0.83578E+00_JPRB, 0.83442E+00_JPRB, 0.83887E+00_JPRB ,&
&  0.85261E+00_JPRB, 0.87041E+00_JPRB, 0.87747E+00_JPRB, 0.87671E+00_JPRB, 0.88416E+00_JPRB, 0.90181E+00_JPRB, 0.91642E+00_JPRB ,&
&  0.92775E+00_JPRB, 0.93744E+00_JPRB, 0.95070E+00_JPRB, 0.99271E+00_JPRB, 0.10602E+01_JPRB, 0.10481E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.10021E+01_JPRB, 0.97981E+00_JPRB, 0.97030E+00_JPRB, 0.83496E+00_JPRB, 0.88600E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.83774E+00_JPRB, 0.82983E+00_JPRB, 0.81331E+00_JPRB, 0.79537E+00_JPRB, 0.78267E+00_JPRB, 0.77777E+00_JPRB, 0.77647E+00_JPRB ,&
&  0.78070E+00_JPRB, 0.78399E+00_JPRB, 0.78366E+00_JPRB, 0.77883E+00_JPRB, 0.77133E+00_JPRB, 0.75998E+00_JPRB, 0.74987E+00_JPRB ,&
&  0.73426E+00_JPRB, 0.71810E+00_JPRB, 0.71196E+00_JPRB, 0.71120E+00_JPRB, 0.70865E+00_JPRB, 0.70126E+00_JPRB, 0.69650E+00_JPRB ,&
&  0.69416E+00_JPRB, 0.69580E+00_JPRB, 0.68893E+00_JPRB, 0.68437E+00_JPRB, 0.67880E+00_JPRB, 0.68015E+00_JPRB, 0.68055E+00_JPRB ,&
&  0.67940E+00_JPRB, 0.55905E+00_JPRB, 0.64307E+00_JPRB, 0.71135E+00_JPRB, 0.74281E+00_JPRB, 0.80484E+00_JPRB, 0.86804E+00_JPRB ,&
&  0.91098E+00_JPRB, 0.92880E+00_JPRB, 0.91913E+00_JPRB, 0.89773E+00_JPRB, 0.86640E+00_JPRB, 0.83779E+00_JPRB, 0.81335E+00_JPRB ,&
&  0.79029E+00_JPRB, 0.77219E+00_JPRB, 0.75570E+00_JPRB, 0.74014E+00_JPRB, 0.73008E+00_JPRB, 0.71996E+00_JPRB, 0.71117E+00_JPRB ,&
&  0.70942E+00_JPRB, 0.71443E+00_JPRB, 0.71402E+00_JPRB, 0.71628E+00_JPRB, 0.72047E+00_JPRB, 0.72386E+00_JPRB, 0.73876E+00_JPRB ,&
&  0.75154E+00_JPRB, 0.76006E+00_JPRB, 0.76946E+00_JPRB, 0.77408E+00_JPRB, 0.76996E+00_JPRB, 0.76955E+00_JPRB, 0.76518E+00_JPRB ,&
&  0.75975E+00_JPRB, 0.74887E+00_JPRB, 0.73807E+00_JPRB, 0.73026E+00_JPRB, 0.72547E+00_JPRB, 0.72205E+00_JPRB, 0.72564E+00_JPRB ,&
&  0.73410E+00_JPRB, 0.73424E+00_JPRB, 0.73408E+00_JPRB, 0.73750E+00_JPRB, 0.74138E+00_JPRB, 0.74560E+00_JPRB, 0.75351E+00_JPRB ,&
&  0.76832E+00_JPRB, 0.78924E+00_JPRB, 0.80783E+00_JPRB, 0.81609E+00_JPRB, 0.81907E+00_JPRB, 0.82802E+00_JPRB, 0.83952E+00_JPRB ,&
&  0.85153E+00_JPRB, 0.85851E+00_JPRB, 0.86354E+00_JPRB, 0.91465E+00_JPRB, 0.93412E+00_JPRB, 0.93335E+00_JPRB, 0.92458E+00_JPRB ,&
&  0.90114E+00_JPRB, 0.89181E+00_JPRB, 0.72863E+00_JPRB, 0.79861E+00_JPRB, 0.81878E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.77921E+00_JPRB, 0.76854E+00_JPRB, 0.75130E+00_JPRB, 0.73632E+00_JPRB, 0.72348E+00_JPRB, 0.71783E+00_JPRB, 0.71453E+00_JPRB ,&
&  0.71903E+00_JPRB, 0.72775E+00_JPRB, 0.73619E+00_JPRB, 0.74073E+00_JPRB, 0.74389E+00_JPRB, 0.74325E+00_JPRB, 0.73843E+00_JPRB ,&
&  0.73406E+00_JPRB, 0.72130E+00_JPRB, 0.71108E+00_JPRB, 0.70974E+00_JPRB, 0.70549E+00_JPRB, 0.70059E+00_JPRB, 0.69464E+00_JPRB ,&
&  0.68885E+00_JPRB, 0.68393E+00_JPRB, 0.67364E+00_JPRB, 0.66856E+00_JPRB, 0.66719E+00_JPRB, 0.66936E+00_JPRB, 0.67887E+00_JPRB ,&
&  0.60348E+00_JPRB, 0.52602E+00_JPRB, 0.59355E+00_JPRB, 0.66639E+00_JPRB, 0.70168E+00_JPRB, 0.74350E+00_JPRB, 0.78084E+00_JPRB ,&
&  0.83024E+00_JPRB, 0.86189E+00_JPRB, 0.87982E+00_JPRB, 0.87570E+00_JPRB, 0.85763E+00_JPRB, 0.83508E+00_JPRB, 0.81083E+00_JPRB ,&
&  0.78069E+00_JPRB, 0.74620E+00_JPRB, 0.72305E+00_JPRB, 0.71076E+00_JPRB, 0.70065E+00_JPRB, 0.69003E+00_JPRB, 0.68270E+00_JPRB ,&
&  0.67900E+00_JPRB, 0.67650E+00_JPRB, 0.67391E+00_JPRB, 0.67226E+00_JPRB, 0.67403E+00_JPRB, 0.67676E+00_JPRB, 0.68123E+00_JPRB ,&
&  0.68530E+00_JPRB, 0.68863E+00_JPRB, 0.68994E+00_JPRB, 0.68767E+00_JPRB, 0.68646E+00_JPRB, 0.69088E+00_JPRB, 0.69227E+00_JPRB ,&
&  0.69371E+00_JPRB, 0.69221E+00_JPRB, 0.69014E+00_JPRB, 0.68774E+00_JPRB, 0.68351E+00_JPRB, 0.67964E+00_JPRB, 0.67864E+00_JPRB ,&
&  0.68094E+00_JPRB, 0.67900E+00_JPRB, 0.68041E+00_JPRB, 0.68513E+00_JPRB, 0.69092E+00_JPRB, 0.69689E+00_JPRB, 0.70172E+00_JPRB ,&
&  0.71032E+00_JPRB, 0.72594E+00_JPRB, 0.73807E+00_JPRB, 0.74620E+00_JPRB, 0.75216E+00_JPRB, 0.75846E+00_JPRB, 0.76765E+00_JPRB ,&
&  0.77592E+00_JPRB, 0.78401E+00_JPRB, 0.79340E+00_JPRB, 0.81516E+00_JPRB, 0.82606E+00_JPRB, 0.82762E+00_JPRB, 0.82457E+00_JPRB ,&
&  0.81362E+00_JPRB, 0.79554E+00_JPRB, 0.79338E+00_JPRB, 0.78730E+00_JPRB, 0.78230E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.68692E+00_JPRB, 0.67944E+00_JPRB, 0.66718E+00_JPRB, 0.65944E+00_JPRB, 0.65551E+00_JPRB, 0.65733E+00_JPRB, 0.66059E+00_JPRB ,&
&  0.66470E+00_JPRB, 0.66619E+00_JPRB, 0.67382E+00_JPRB, 0.68209E+00_JPRB, 0.68738E+00_JPRB, 0.69113E+00_JPRB, 0.69035E+00_JPRB ,&
&  0.68989E+00_JPRB, 0.68924E+00_JPRB, 0.68384E+00_JPRB, 0.67884E+00_JPRB, 0.67240E+00_JPRB, 0.67221E+00_JPRB, 0.67126E+00_JPRB ,&
&  0.66889E+00_JPRB, 0.66464E+00_JPRB, 0.65721E+00_JPRB, 0.64773E+00_JPRB, 0.64326E+00_JPRB, 0.64217E+00_JPRB, 0.63948E+00_JPRB ,&
&  0.64565E+00_JPRB, 0.56571E+00_JPRB, 0.60543E+00_JPRB, 0.62858E+00_JPRB, 0.65912E+00_JPRB, 0.69087E+00_JPRB, 0.72356E+00_JPRB ,&
&  0.75691E+00_JPRB, 0.78422E+00_JPRB, 0.79522E+00_JPRB, 0.79659E+00_JPRB, 0.78228E+00_JPRB, 0.75719E+00_JPRB, 0.72870E+00_JPRB ,&
&  0.70464E+00_JPRB, 0.68654E+00_JPRB, 0.66654E+00_JPRB, 0.65451E+00_JPRB, 0.64474E+00_JPRB, 0.63241E+00_JPRB, 0.62157E+00_JPRB ,&
&  0.61601E+00_JPRB, 0.61239E+00_JPRB, 0.61146E+00_JPRB, 0.61294E+00_JPRB, 0.61577E+00_JPRB, 0.61800E+00_JPRB, 0.61817E+00_JPRB ,&
&  0.61813E+00_JPRB, 0.61784E+00_JPRB, 0.61600E+00_JPRB, 0.61630E+00_JPRB, 0.61790E+00_JPRB, 0.62192E+00_JPRB, 0.62012E+00_JPRB ,&
&  0.61435E+00_JPRB, 0.61576E+00_JPRB, 0.61846E+00_JPRB, 0.61936E+00_JPRB, 0.62077E+00_JPRB, 0.62217E+00_JPRB, 0.62110E+00_JPRB ,&
&  0.62021E+00_JPRB, 0.62326E+00_JPRB, 0.62682E+00_JPRB, 0.63342E+00_JPRB, 0.63828E+00_JPRB, 0.64781E+00_JPRB, 0.65620E+00_JPRB ,&
&  0.66472E+00_JPRB, 0.67772E+00_JPRB, 0.69036E+00_JPRB, 0.69957E+00_JPRB, 0.70539E+00_JPRB, 0.71060E+00_JPRB, 0.71609E+00_JPRB ,&
&  0.72053E+00_JPRB, 0.72475E+00_JPRB, 0.72878E+00_JPRB, 0.73220E+00_JPRB, 0.73073E+00_JPRB, 0.72810E+00_JPRB, 0.72647E+00_JPRB ,&
&  0.71838E+00_JPRB, 0.70562E+00_JPRB, 0.70039E+00_JPRB, 0.69531E+00_JPRB, 0.69008E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.62475E+00_JPRB, 0.61926E+00_JPRB, 0.61355E+00_JPRB, 0.61234E+00_JPRB, 0.61078E+00_JPRB, 0.61216E+00_JPRB, 0.61444E+00_JPRB ,&
&  0.62081E+00_JPRB, 0.62488E+00_JPRB, 0.63404E+00_JPRB, 0.64171E+00_JPRB, 0.64561E+00_JPRB, 0.65031E+00_JPRB, 0.65328E+00_JPRB ,&
&  0.65438E+00_JPRB, 0.65576E+00_JPRB, 0.65625E+00_JPRB, 0.66028E+00_JPRB, 0.66069E+00_JPRB, 0.65970E+00_JPRB, 0.65838E+00_JPRB ,&
&  0.65596E+00_JPRB, 0.65308E+00_JPRB, 0.64712E+00_JPRB, 0.63920E+00_JPRB, 0.62944E+00_JPRB, 0.62479E+00_JPRB, 0.61810E+00_JPRB ,&
&  0.61866E+00_JPRB, 0.59115E+00_JPRB, 0.60348E+00_JPRB, 0.60197E+00_JPRB, 0.62012E+00_JPRB, 0.64854E+00_JPRB, 0.68390E+00_JPRB ,&
&  0.70720E+00_JPRB, 0.71836E+00_JPRB, 0.71837E+00_JPRB, 0.71395E+00_JPRB, 0.69790E+00_JPRB, 0.68165E+00_JPRB, 0.66333E+00_JPRB ,&
&  0.64262E+00_JPRB, 0.63070E+00_JPRB, 0.61632E+00_JPRB, 0.60576E+00_JPRB, 0.60125E+00_JPRB, 0.59795E+00_JPRB, 0.59245E+00_JPRB ,&
&  0.58410E+00_JPRB, 0.57683E+00_JPRB, 0.57697E+00_JPRB, 0.58032E+00_JPRB, 0.58403E+00_JPRB, 0.58639E+00_JPRB, 0.58536E+00_JPRB ,&
&  0.57918E+00_JPRB, 0.57412E+00_JPRB, 0.57225E+00_JPRB, 0.57242E+00_JPRB, 0.56882E+00_JPRB, 0.56195E+00_JPRB, 0.55479E+00_JPRB ,&
&  0.55324E+00_JPRB, 0.55607E+00_JPRB, 0.55988E+00_JPRB, 0.56469E+00_JPRB, 0.56812E+00_JPRB, 0.57225E+00_JPRB, 0.57419E+00_JPRB ,&
&  0.57889E+00_JPRB, 0.58345E+00_JPRB, 0.59271E+00_JPRB, 0.60168E+00_JPRB, 0.60410E+00_JPRB, 0.61307E+00_JPRB, 0.62243E+00_JPRB ,&
&  0.63192E+00_JPRB, 0.64303E+00_JPRB, 0.65189E+00_JPRB, 0.65828E+00_JPRB, 0.66489E+00_JPRB, 0.66891E+00_JPRB, 0.67499E+00_JPRB ,&
&  0.68290E+00_JPRB, 0.68503E+00_JPRB, 0.69082E+00_JPRB, 0.69204E+00_JPRB, 0.68783E+00_JPRB, 0.68068E+00_JPRB, 0.67120E+00_JPRB ,&
&  0.66005E+00_JPRB, 0.64396E+00_JPRB, 0.63513E+00_JPRB, 0.62975E+00_JPRB, 0.62667E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.55183E+00_JPRB, 0.54976E+00_JPRB, 0.55008E+00_JPRB, 0.55317E+00_JPRB, 0.55657E+00_JPRB, 0.55920E+00_JPRB, 0.56479E+00_JPRB ,&
&  0.57460E+00_JPRB, 0.58228E+00_JPRB, 0.59178E+00_JPRB, 0.60152E+00_JPRB, 0.60907E+00_JPRB, 0.61419E+00_JPRB, 0.61698E+00_JPRB ,&
&  0.61532E+00_JPRB, 0.61271E+00_JPRB, 0.61569E+00_JPRB, 0.62035E+00_JPRB, 0.62172E+00_JPRB, 0.62120E+00_JPRB, 0.62105E+00_JPRB ,&
&  0.62025E+00_JPRB, 0.61756E+00_JPRB, 0.61403E+00_JPRB, 0.60896E+00_JPRB, 0.60310E+00_JPRB, 0.59997E+00_JPRB, 0.59715E+00_JPRB ,&
&  0.58984E+00_JPRB, 0.59073E+00_JPRB, 0.54029E+00_JPRB, 0.55493E+00_JPRB, 0.56725E+00_JPRB, 0.58859E+00_JPRB, 0.60987E+00_JPRB ,&
&  0.62459E+00_JPRB, 0.63312E+00_JPRB, 0.63695E+00_JPRB, 0.63451E+00_JPRB, 0.62147E+00_JPRB, 0.61358E+00_JPRB, 0.60493E+00_JPRB ,&
&  0.59374E+00_JPRB, 0.58570E+00_JPRB, 0.57219E+00_JPRB, 0.55767E+00_JPRB, 0.55269E+00_JPRB, 0.55313E+00_JPRB, 0.55304E+00_JPRB ,&
&  0.54894E+00_JPRB, 0.54493E+00_JPRB, 0.54598E+00_JPRB, 0.54687E+00_JPRB, 0.54670E+00_JPRB, 0.54149E+00_JPRB, 0.53490E+00_JPRB ,&
&  0.52451E+00_JPRB, 0.51430E+00_JPRB, 0.50723E+00_JPRB, 0.50208E+00_JPRB, 0.49759E+00_JPRB, 0.49258E+00_JPRB, 0.48777E+00_JPRB ,&
&  0.49168E+00_JPRB, 0.50335E+00_JPRB, 0.51430E+00_JPRB, 0.52333E+00_JPRB, 0.52916E+00_JPRB, 0.53304E+00_JPRB, 0.53825E+00_JPRB ,&
&  0.54433E+00_JPRB, 0.55468E+00_JPRB, 0.56207E+00_JPRB, 0.56873E+00_JPRB, 0.57378E+00_JPRB, 0.57857E+00_JPRB, 0.58403E+00_JPRB ,&
&  0.58865E+00_JPRB, 0.59611E+00_JPRB, 0.60188E+00_JPRB, 0.60866E+00_JPRB, 0.61283E+00_JPRB, 0.61836E+00_JPRB, 0.62472E+00_JPRB ,&
&  0.63172E+00_JPRB, 0.63788E+00_JPRB, 0.64048E+00_JPRB, 0.63945E+00_JPRB, 0.63238E+00_JPRB, 0.62295E+00_JPRB, 0.61376E+00_JPRB ,&
&  0.60170E+00_JPRB, 0.58701E+00_JPRB, 0.57196E+00_JPRB, 0.56234E+00_JPRB, 0.55436E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.49731E+00_JPRB, 0.49826E+00_JPRB, 0.50313E+00_JPRB, 0.50784E+00_JPRB, 0.51125E+00_JPRB, 0.51611E+00_JPRB, 0.52572E+00_JPRB ,&
&  0.53543E+00_JPRB, 0.54472E+00_JPRB, 0.55109E+00_JPRB, 0.55885E+00_JPRB, 0.56994E+00_JPRB, 0.57748E+00_JPRB, 0.58347E+00_JPRB ,&
&  0.58599E+00_JPRB, 0.58643E+00_JPRB, 0.58751E+00_JPRB, 0.59065E+00_JPRB, 0.59018E+00_JPRB, 0.58650E+00_JPRB, 0.58512E+00_JPRB ,&
&  0.58485E+00_JPRB, 0.58294E+00_JPRB, 0.58241E+00_JPRB, 0.58310E+00_JPRB, 0.58031E+00_JPRB, 0.57602E+00_JPRB, 0.57505E+00_JPRB ,&
&  0.57465E+00_JPRB, 0.56076E+00_JPRB, 0.55286E+00_JPRB, 0.54552E+00_JPRB, 0.54875E+00_JPRB, 0.55976E+00_JPRB, 0.57201E+00_JPRB ,&
&  0.57934E+00_JPRB, 0.58426E+00_JPRB, 0.58404E+00_JPRB, 0.58314E+00_JPRB, 0.57838E+00_JPRB, 0.56697E+00_JPRB, 0.55755E+00_JPRB ,&
&  0.55123E+00_JPRB, 0.54195E+00_JPRB, 0.51925E+00_JPRB, 0.50589E+00_JPRB, 0.50224E+00_JPRB, 0.50498E+00_JPRB, 0.50928E+00_JPRB ,&
&  0.51103E+00_JPRB, 0.50838E+00_JPRB, 0.50207E+00_JPRB, 0.49475E+00_JPRB, 0.48500E+00_JPRB, 0.47380E+00_JPRB, 0.46397E+00_JPRB ,&
&  0.45843E+00_JPRB, 0.45432E+00_JPRB, 0.45090E+00_JPRB, 0.45354E+00_JPRB, 0.45284E+00_JPRB, 0.44895E+00_JPRB, 0.45100E+00_JPRB ,&
&  0.46270E+00_JPRB, 0.47757E+00_JPRB, 0.49126E+00_JPRB, 0.49878E+00_JPRB, 0.50188E+00_JPRB, 0.50539E+00_JPRB, 0.50946E+00_JPRB ,&
&  0.51533E+00_JPRB, 0.52104E+00_JPRB, 0.52630E+00_JPRB, 0.53319E+00_JPRB, 0.54177E+00_JPRB, 0.54958E+00_JPRB, 0.55308E+00_JPRB ,&
&  0.55605E+00_JPRB, 0.55797E+00_JPRB, 0.55965E+00_JPRB, 0.56529E+00_JPRB, 0.57356E+00_JPRB, 0.57944E+00_JPRB, 0.58424E+00_JPRB ,&
&  0.58913E+00_JPRB, 0.59147E+00_JPRB, 0.59020E+00_JPRB, 0.58637E+00_JPRB, 0.58084E+00_JPRB, 0.56992E+00_JPRB, 0.55625E+00_JPRB ,&
&  0.54039E+00_JPRB, 0.52571E+00_JPRB, 0.51469E+00_JPRB, 0.50528E+00_JPRB, 0.49887E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.45615E+00_JPRB, 0.45611E+00_JPRB, 0.45909E+00_JPRB, 0.46546E+00_JPRB, 0.46971E+00_JPRB, 0.47358E+00_JPRB, 0.48138E+00_JPRB ,&
&  0.48858E+00_JPRB, 0.49496E+00_JPRB, 0.50250E+00_JPRB, 0.51069E+00_JPRB, 0.51710E+00_JPRB, 0.52631E+00_JPRB, 0.53509E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.54316E+00_JPRB, 0.54456E+00_JPRB, 0.54575E+00_JPRB, 0.54565E+00_JPRB, 0.54312E+00_JPRB, 0.54260E+00_JPRB ,&
&  0.54066E+00_JPRB, 0.53847E+00_JPRB, 0.54044E+00_JPRB, 0.54057E+00_JPRB, 0.54194E+00_JPRB, 0.54432E+00_JPRB, 0.54399E+00_JPRB ,&
&  0.54137E+00_JPRB, 0.52788E+00_JPRB, 0.52272E+00_JPRB, 0.51945E+00_JPRB, 0.52340E+00_JPRB, 0.52420E+00_JPRB, 0.52251E+00_JPRB ,&
&  0.51585E+00_JPRB, 0.51170E+00_JPRB, 0.51342E+00_JPRB, 0.51569E+00_JPRB, 0.50988E+00_JPRB, 0.49683E+00_JPRB, 0.48671E+00_JPRB ,&
&  0.47889E+00_JPRB, 0.46679E+00_JPRB, 0.44889E+00_JPRB, 0.43808E+00_JPRB, 0.43343E+00_JPRB, 0.42858E+00_JPRB, 0.42429E+00_JPRB ,&
&  0.42209E+00_JPRB, 0.42134E+00_JPRB, 0.42267E+00_JPRB, 0.42188E+00_JPRB, 0.41836E+00_JPRB, 0.41411E+00_JPRB, 0.41255E+00_JPRB ,&
&  0.41538E+00_JPRB, 0.41788E+00_JPRB, 0.42149E+00_JPRB, 0.42426E+00_JPRB, 0.42379E+00_JPRB, 0.42415E+00_JPRB, 0.42794E+00_JPRB ,&
&  0.43393E+00_JPRB, 0.43966E+00_JPRB, 0.44538E+00_JPRB, 0.44784E+00_JPRB, 0.45116E+00_JPRB, 0.45808E+00_JPRB, 0.45871E+00_JPRB ,&
&  0.45723E+00_JPRB, 0.45913E+00_JPRB, 0.46332E+00_JPRB, 0.47148E+00_JPRB, 0.48407E+00_JPRB, 0.49255E+00_JPRB, 0.49417E+00_JPRB ,&
&  0.49461E+00_JPRB, 0.49716E+00_JPRB, 0.50128E+00_JPRB, 0.51096E+00_JPRB, 0.52149E+00_JPRB, 0.52906E+00_JPRB, 0.53227E+00_JPRB ,&
&  0.52925E+00_JPRB, 0.52256E+00_JPRB, 0.51045E+00_JPRB, 0.49637E+00_JPRB, 0.48372E+00_JPRB, 0.47382E+00_JPRB, 0.46568E+00_JPRB ,&
&  0.45754E+00_JPRB, 0.45376E+00_JPRB, 0.45313E+00_JPRB, 0.45483E+00_JPRB, 0.45582E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.42411E+00_JPRB, 0.42596E+00_JPRB, 0.42831E+00_JPRB, 0.43138E+00_JPRB, 0.43469E+00_JPRB, 0.43669E+00_JPRB, 0.44056E+00_JPRB ,&
&  0.44282E+00_JPRB, 0.44946E+00_JPRB, 0.46028E+00_JPRB, 0.47112E+00_JPRB, 0.47902E+00_JPRB, 0.48576E+00_JPRB, 0.49240E+00_JPRB ,&
&  0.49785E+00_JPRB, 0.50503E+00_JPRB, 0.50606E+00_JPRB, 0.50373E+00_JPRB, 0.50115E+00_JPRB, 0.49915E+00_JPRB, 0.49730E+00_JPRB ,&
&  0.49472E+00_JPRB, 0.49447E+00_JPRB, 0.49809E+00_JPRB, 0.50215E+00_JPRB, 0.50624E+00_JPRB, 0.50714E+00_JPRB, 0.50543E+00_JPRB ,&
&  0.50862E+00_JPRB, 0.49868E+00_JPRB, 0.49209E+00_JPRB, 0.48450E+00_JPRB, 0.47785E+00_JPRB, 0.46617E+00_JPRB, 0.45753E+00_JPRB ,&
&  0.44730E+00_JPRB, 0.43826E+00_JPRB, 0.43434E+00_JPRB, 0.43019E+00_JPRB, 0.42044E+00_JPRB, 0.40920E+00_JPRB, 0.40099E+00_JPRB ,&
&  0.39561E+00_JPRB, 0.39016E+00_JPRB, 0.38352E+00_JPRB, 0.37801E+00_JPRB, 0.37347E+00_JPRB, 0.37184E+00_JPRB, 0.37355E+00_JPRB ,&
&  0.37783E+00_JPRB, 0.38190E+00_JPRB, 0.38441E+00_JPRB, 0.38512E+00_JPRB, 0.38361E+00_JPRB, 0.38112E+00_JPRB, 0.37972E+00_JPRB ,&
&  0.38042E+00_JPRB, 0.38195E+00_JPRB, 0.38409E+00_JPRB, 0.38933E+00_JPRB, 0.39016E+00_JPRB, 0.38852E+00_JPRB, 0.38285E+00_JPRB ,&
&  0.38043E+00_JPRB, 0.37927E+00_JPRB, 0.38101E+00_JPRB, 0.38620E+00_JPRB, 0.39478E+00_JPRB, 0.40312E+00_JPRB, 0.40910E+00_JPRB ,&
&  0.41209E+00_JPRB, 0.41386E+00_JPRB, 0.41704E+00_JPRB, 0.42089E+00_JPRB, 0.42267E+00_JPRB, 0.42231E+00_JPRB, 0.42112E+00_JPRB ,&
&  0.42418E+00_JPRB, 0.43189E+00_JPRB, 0.43997E+00_JPRB, 0.44524E+00_JPRB, 0.44797E+00_JPRB, 0.44719E+00_JPRB, 0.44047E+00_JPRB ,&
&  0.42809E+00_JPRB, 0.41376E+00_JPRB, 0.39982E+00_JPRB, 0.38804E+00_JPRB, 0.37985E+00_JPRB, 0.37594E+00_JPRB, 0.37879E+00_JPRB ,&
&  0.38644E+00_JPRB, 0.39627E+00_JPRB, 0.40522E+00_JPRB, 0.41183E+00_JPRB, 0.41910E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.39332E+00_JPRB, 0.39318E+00_JPRB, 0.39479E+00_JPRB, 0.39385E+00_JPRB, 0.39127E+00_JPRB, 0.38927E+00_JPRB, 0.39274E+00_JPRB ,&
&  0.39956E+00_JPRB, 0.41127E+00_JPRB, 0.42686E+00_JPRB, 0.44058E+00_JPRB, 0.44890E+00_JPRB, 0.45400E+00_JPRB, 0.46176E+00_JPRB ,&
&  0.46901E+00_JPRB, 0.47710E+00_JPRB, 0.47743E+00_JPRB, 0.46872E+00_JPRB, 0.46133E+00_JPRB, 0.45745E+00_JPRB, 0.45696E+00_JPRB ,&
&  0.45627E+00_JPRB, 0.45796E+00_JPRB, 0.46101E+00_JPRB, 0.46422E+00_JPRB, 0.46331E+00_JPRB, 0.46116E+00_JPRB, 0.46440E+00_JPRB ,&
&  0.46201E+00_JPRB, 0.45193E+00_JPRB, 0.44163E+00_JPRB, 0.41644E+00_JPRB, 0.41546E+00_JPRB, 0.40487E+00_JPRB, 0.39677E+00_JPRB ,&
&  0.38593E+00_JPRB, 0.37254E+00_JPRB, 0.36278E+00_JPRB, 0.35650E+00_JPRB, 0.35366E+00_JPRB, 0.35113E+00_JPRB, 0.34913E+00_JPRB ,&
&  0.34787E+00_JPRB, 0.34776E+00_JPRB, 0.34925E+00_JPRB, 0.34700E+00_JPRB, 0.34587E+00_JPRB, 0.34668E+00_JPRB, 0.34849E+00_JPRB ,&
&  0.34854E+00_JPRB, 0.34861E+00_JPRB, 0.34873E+00_JPRB, 0.34800E+00_JPRB, 0.34642E+00_JPRB, 0.34403E+00_JPRB, 0.34181E+00_JPRB ,&
&  0.34097E+00_JPRB, 0.34213E+00_JPRB, 0.34261E+00_JPRB, 0.33882E+00_JPRB, 0.33124E+00_JPRB, 0.32176E+00_JPRB, 0.31558E+00_JPRB ,&
&  0.31299E+00_JPRB, 0.31516E+00_JPRB, 0.32316E+00_JPRB, 0.33445E+00_JPRB, 0.34491E+00_JPRB, 0.35123E+00_JPRB, 0.35525E+00_JPRB ,&
&  0.35986E+00_JPRB, 0.35802E+00_JPRB, 0.35432E+00_JPRB, 0.35113E+00_JPRB, 0.34824E+00_JPRB, 0.34422E+00_JPRB, 0.34422E+00_JPRB ,&
&  0.34702E+00_JPRB, 0.34985E+00_JPRB, 0.35082E+00_JPRB, 0.34977E+00_JPRB, 0.34775E+00_JPRB, 0.34351E+00_JPRB, 0.33756E+00_JPRB ,&
&  0.33014E+00_JPRB, 0.32148E+00_JPRB, 0.31951E+00_JPRB, 0.32021E+00_JPRB, 0.32403E+00_JPRB, 0.33182E+00_JPRB, 0.34250E+00_JPRB ,&
&  0.35367E+00_JPRB, 0.36277E+00_JPRB, 0.37259E+00_JPRB, 0.38167E+00_JPRB, 0.38982E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.37365E+00_JPRB, 0.37508E+00_JPRB, 0.37250E+00_JPRB, 0.36969E+00_JPRB, 0.36944E+00_JPRB, 0.37112E+00_JPRB, 0.37680E+00_JPRB ,&
&  0.38651E+00_JPRB, 0.39916E+00_JPRB, 0.41136E+00_JPRB, 0.42151E+00_JPRB, 0.43066E+00_JPRB, 0.43469E+00_JPRB, 0.44071E+00_JPRB ,&
&  0.44723E+00_JPRB, 0.45322E+00_JPRB, 0.45409E+00_JPRB, 0.44390E+00_JPRB, 0.43400E+00_JPRB, 0.42874E+00_JPRB, 0.42877E+00_JPRB ,&
&  0.43053E+00_JPRB, 0.43309E+00_JPRB, 0.43912E+00_JPRB, 0.44120E+00_JPRB, 0.43507E+00_JPRB, 0.43335E+00_JPRB, 0.43091E+00_JPRB ,&
&  0.42504E+00_JPRB, 0.41316E+00_JPRB, 0.38051E+00_JPRB, 0.37452E+00_JPRB, 0.36606E+00_JPRB, 0.35688E+00_JPRB, 0.34781E+00_JPRB ,&
&  0.33748E+00_JPRB, 0.32706E+00_JPRB, 0.31994E+00_JPRB, 0.31573E+00_JPRB, 0.31635E+00_JPRB, 0.31756E+00_JPRB, 0.31710E+00_JPRB ,&
&  0.31923E+00_JPRB, 0.32186E+00_JPRB, 0.32417E+00_JPRB, 0.32453E+00_JPRB, 0.32418E+00_JPRB, 0.32320E+00_JPRB, 0.31887E+00_JPRB ,&
&  0.31499E+00_JPRB, 0.31303E+00_JPRB, 0.31150E+00_JPRB, 0.31019E+00_JPRB, 0.31014E+00_JPRB, 0.31075E+00_JPRB, 0.31102E+00_JPRB ,&
&  0.30882E+00_JPRB, 0.30526E+00_JPRB, 0.30124E+00_JPRB, 0.29418E+00_JPRB, 0.28172E+00_JPRB, 0.26657E+00_JPRB, 0.25715E+00_JPRB ,&
&  0.25573E+00_JPRB, 0.26650E+00_JPRB, 0.28752E+00_JPRB, 0.29701E+00_JPRB, 0.30452E+00_JPRB, 0.30846E+00_JPRB, 0.31285E+00_JPRB ,&
&  0.30776E+00_JPRB, 0.28414E+00_JPRB, 0.27118E+00_JPRB, 0.26415E+00_JPRB, 0.25877E+00_JPRB, 0.25481E+00_JPRB, 0.25463E+00_JPRB ,&
&  0.25798E+00_JPRB, 0.26437E+00_JPRB, 0.27086E+00_JPRB, 0.27400E+00_JPRB, 0.27322E+00_JPRB, 0.27002E+00_JPRB, 0.26661E+00_JPRB ,&
&  0.26397E+00_JPRB, 0.26866E+00_JPRB, 0.28340E+00_JPRB, 0.30494E+00_JPRB, 0.31315E+00_JPRB, 0.31952E+00_JPRB, 0.32535E+00_JPRB ,&
&  0.33166E+00_JPRB, 0.34125E+00_JPRB, 0.35283E+00_JPRB, 0.36190E+00_JPRB, 0.36991E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.34574E+00_JPRB, 0.34977E+00_JPRB, 0.35230E+00_JPRB, 0.35568E+00_JPRB, 0.36330E+00_JPRB, 0.37396E+00_JPRB, 0.38604E+00_JPRB ,&
&  0.39730E+00_JPRB, 0.40655E+00_JPRB, 0.41117E+00_JPRB, 0.41641E+00_JPRB, 0.42139E+00_JPRB, 0.42376E+00_JPRB, 0.42894E+00_JPRB ,&
&  0.42972E+00_JPRB, 0.42633E+00_JPRB, 0.42025E+00_JPRB, 0.41225E+00_JPRB, 0.40804E+00_JPRB, 0.40931E+00_JPRB, 0.41216E+00_JPRB ,&
&  0.41395E+00_JPRB, 0.41531E+00_JPRB, 0.41524E+00_JPRB, 0.41165E+00_JPRB, 0.40822E+00_JPRB, 0.40617E+00_JPRB, 0.40197E+00_JPRB ,&
&  0.39226E+00_JPRB, 0.34248E+00_JPRB, 0.30492E+00_JPRB, 0.30086E+00_JPRB, 0.31691E+00_JPRB, 0.31904E+00_JPRB, 0.31403E+00_JPRB ,&
&  0.30837E+00_JPRB, 0.30356E+00_JPRB, 0.30076E+00_JPRB, 0.29855E+00_JPRB, 0.29580E+00_JPRB, 0.29335E+00_JPRB, 0.29401E+00_JPRB ,&
&  0.29526E+00_JPRB, 0.29306E+00_JPRB, 0.29007E+00_JPRB, 0.28378E+00_JPRB, 0.27925E+00_JPRB, 0.27753E+00_JPRB, 0.27612E+00_JPRB ,&
&  0.27197E+00_JPRB, 0.26783E+00_JPRB, 0.26778E+00_JPRB, 0.26909E+00_JPRB, 0.27112E+00_JPRB, 0.27250E+00_JPRB, 0.26980E+00_JPRB ,&
&  0.26433E+00_JPRB, 0.25771E+00_JPRB, 0.24955E+00_JPRB, 0.23971E+00_JPRB, 0.23046E+00_JPRB, 0.22370E+00_JPRB, 0.21827E+00_JPRB ,&
&  0.21740E+00_JPRB, 0.22332E+00_JPRB, 0.23669E+00_JPRB, 0.25209E+00_JPRB, 0.26954E+00_JPRB, 0.25963E+00_JPRB, 0.24911E+00_JPRB ,&
&  0.23858E+00_JPRB, 0.22984E+00_JPRB, 0.22348E+00_JPRB, 0.22070E+00_JPRB, 0.21968E+00_JPRB, 0.21975E+00_JPRB, 0.22101E+00_JPRB ,&
&  0.22324E+00_JPRB, 0.22605E+00_JPRB, 0.22846E+00_JPRB, 0.22960E+00_JPRB, 0.23016E+00_JPRB, 0.23061E+00_JPRB, 0.23035E+00_JPRB ,&
&  0.22964E+00_JPRB, 0.23202E+00_JPRB, 0.23879E+00_JPRB, 0.24742E+00_JPRB, 0.25569E+00_JPRB, 0.26351E+00_JPRB, 0.27644E+00_JPRB ,&
&  0.29229E+00_JPRB, 0.31188E+00_JPRB, 0.32441E+00_JPRB, 0.33400E+00_JPRB, 0.34218E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.33859E+00_JPRB, 0.34745E+00_JPRB, 0.35927E+00_JPRB, 0.36956E+00_JPRB, 0.37859E+00_JPRB, 0.38715E+00_JPRB, 0.39466E+00_JPRB ,&
&  0.39880E+00_JPRB, 0.40121E+00_JPRB, 0.40215E+00_JPRB, 0.40573E+00_JPRB, 0.40273E+00_JPRB, 0.39581E+00_JPRB, 0.39028E+00_JPRB ,&
&  0.38566E+00_JPRB, 0.38208E+00_JPRB, 0.37682E+00_JPRB, 0.37407E+00_JPRB, 0.37381E+00_JPRB, 0.37386E+00_JPRB, 0.36818E+00_JPRB ,&
&  0.35760E+00_JPRB, 0.35644E+00_JPRB, 0.36100E+00_JPRB, 0.35546E+00_JPRB, 0.34968E+00_JPRB, 0.34414E+00_JPRB, 0.33823E+00_JPRB ,&
&  0.32213E+00_JPRB, 0.30195E+00_JPRB, 0.27515E+00_JPRB, 0.27141E+00_JPRB, 0.28714E+00_JPRB, 0.28061E+00_JPRB, 0.28392E+00_JPRB ,&
&  0.28460E+00_JPRB, 0.28480E+00_JPRB, 0.28330E+00_JPRB, 0.28166E+00_JPRB, 0.27812E+00_JPRB, 0.27701E+00_JPRB, 0.27548E+00_JPRB ,&
&  0.26966E+00_JPRB, 0.26464E+00_JPRB, 0.25659E+00_JPRB, 0.23876E+00_JPRB, 0.23397E+00_JPRB, 0.23258E+00_JPRB, 0.22794E+00_JPRB ,&
&  0.22038E+00_JPRB, 0.21503E+00_JPRB, 0.21305E+00_JPRB, 0.20909E+00_JPRB, 0.21126E+00_JPRB, 0.21331E+00_JPRB, 0.21327E+00_JPRB ,&
&  0.21321E+00_JPRB, 0.21275E+00_JPRB, 0.21041E+00_JPRB, 0.20653E+00_JPRB, 0.20341E+00_JPRB, 0.20236E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.20564E+00_JPRB, 0.21009E+00_JPRB, 0.22101E+00_JPRB, 0.22946E+00_JPRB, 0.22838E+00_JPRB, 0.22411E+00_JPRB, 0.21518E+00_JPRB ,&
&  0.20722E+00_JPRB, 0.20430E+00_JPRB, 0.20178E+00_JPRB, 0.20046E+00_JPRB, 0.20081E+00_JPRB, 0.20229E+00_JPRB, 0.20306E+00_JPRB ,&
&  0.20418E+00_JPRB, 0.20534E+00_JPRB, 0.20624E+00_JPRB, 0.20677E+00_JPRB, 0.20759E+00_JPRB, 0.20855E+00_JPRB, 0.21033E+00_JPRB ,&
&  0.21292E+00_JPRB, 0.21731E+00_JPRB, 0.22307E+00_JPRB, 0.22927E+00_JPRB, 0.23375E+00_JPRB, 0.23595E+00_JPRB, 0.24009E+00_JPRB ,&
&  0.24600E+00_JPRB, 0.26122E+00_JPRB, 0.28981E+00_JPRB, 0.33443E+00_JPRB, 0.33794E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.33818E+00_JPRB, 0.35023E+00_JPRB, 0.36546E+00_JPRB, 0.37494E+00_JPRB, 0.38038E+00_JPRB, 0.38313E+00_JPRB, 0.38212E+00_JPRB ,&
&  0.37874E+00_JPRB, 0.37816E+00_JPRB, 0.38919E+00_JPRB, 0.37103E+00_JPRB, 0.35343E+00_JPRB, 0.32042E+00_JPRB, 0.30396E+00_JPRB ,&
&  0.29149E+00_JPRB, 0.28932E+00_JPRB, 0.28898E+00_JPRB, 0.29364E+00_JPRB, 0.29742E+00_JPRB, 0.30036E+00_JPRB, 0.29648E+00_JPRB ,&
&  0.29104E+00_JPRB, 0.28179E+00_JPRB, 0.27403E+00_JPRB, 0.26680E+00_JPRB, 0.26699E+00_JPRB, 0.26586E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26035E+00_JPRB, 0.25854E+00_JPRB, 0.25849E+00_JPRB, 0.25892E+00_JPRB, 0.27049E+00_JPRB, 0.26961E+00_JPRB, 0.26985E+00_JPRB ,&
&  0.26955E+00_JPRB, 0.26949E+00_JPRB, 0.26847E+00_JPRB, 0.26675E+00_JPRB, 0.26202E+00_JPRB, 0.24834E+00_JPRB, 0.24741E+00_JPRB ,&
&  0.23846E+00_JPRB, 0.22803E+00_JPRB, 0.21784E+00_JPRB, 0.21077E+00_JPRB, 0.20647E+00_JPRB, 0.20471E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.19520E+00_JPRB, 0.18945E+00_JPRB, 0.18685E+00_JPRB, 0.18671E+00_JPRB, 0.18734E+00_JPRB, 0.18793E+00_JPRB, 0.18841E+00_JPRB ,&
&  0.18971E+00_JPRB, 0.19031E+00_JPRB, 0.19067E+00_JPRB, 0.19137E+00_JPRB, 0.19208E+00_JPRB, 0.19313E+00_JPRB, 0.19497E+00_JPRB ,&
&  0.19819E+00_JPRB, 0.20166E+00_JPRB, 0.20489E+00_JPRB, 0.20774E+00_JPRB, 0.20681E+00_JPRB, 0.20337E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19553E+00_JPRB, 0.19442E+00_JPRB, 0.19337E+00_JPRB, 0.19327E+00_JPRB, 0.19341E+00_JPRB, 0.19382E+00_JPRB, 0.19443E+00_JPRB ,&
&  0.19528E+00_JPRB, 0.19598E+00_JPRB, 0.19684E+00_JPRB, 0.19797E+00_JPRB, 0.19955E+00_JPRB, 0.20098E+00_JPRB, 0.20298E+00_JPRB ,&
&  0.20598E+00_JPRB, 0.20926E+00_JPRB, 0.21387E+00_JPRB, 0.21799E+00_JPRB, 0.22374E+00_JPRB, 0.23192E+00_JPRB, 0.24541E+00_JPRB ,&
&  0.25984E+00_JPRB, 0.29364E+00_JPRB, 0.30900E+00_JPRB, 0.32040E+00_JPRB, 0.32803E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.33828E+00_JPRB, 0.34794E+00_JPRB, 0.36199E+00_JPRB, 0.37112E+00_JPRB, 0.37831E+00_JPRB, 0.37418E+00_JPRB, 0.37021E+00_JPRB ,&
&  0.36606E+00_JPRB, 0.35208E+00_JPRB, 0.33488E+00_JPRB, 0.30571E+00_JPRB, 0.28115E+00_JPRB, 0.26240E+00_JPRB, 0.25191E+00_JPRB ,&
&  0.24551E+00_JPRB, 0.24331E+00_JPRB, 0.24111E+00_JPRB, 0.24044E+00_JPRB, 0.23902E+00_JPRB, 0.24046E+00_JPRB, 0.24165E+00_JPRB ,&
&  0.24112E+00_JPRB, 0.24010E+00_JPRB, 0.23961E+00_JPRB, 0.24277E+00_JPRB, 0.24891E+00_JPRB, 0.25371E+00_JPRB, 0.25835E+00_JPRB ,&
&  0.26162E+00_JPRB, 0.25893E+00_JPRB, 0.26028E+00_JPRB, 0.25844E+00_JPRB, 0.25668E+00_JPRB, 0.25718E+00_JPRB, 0.25833E+00_JPRB ,&
&  0.25874E+00_JPRB, 0.25904E+00_JPRB, 0.25820E+00_JPRB, 0.25499E+00_JPRB, 0.24307E+00_JPRB, 0.23703E+00_JPRB, 0.22915E+00_JPRB ,&
&  0.22109E+00_JPRB, 0.21171E+00_JPRB, 0.20407E+00_JPRB, 0.20045E+00_JPRB, 0.19598E+00_JPRB, 0.19431E+00_JPRB, 0.19114E+00_JPRB ,&
&  0.18753E+00_JPRB, 0.18412E+00_JPRB, 0.18195E+00_JPRB, 0.18085E+00_JPRB, 0.18044E+00_JPRB, 0.17925E+00_JPRB, 0.17884E+00_JPRB ,&
&  0.17916E+00_JPRB, 0.17950E+00_JPRB, 0.18045E+00_JPRB, 0.18132E+00_JPRB, 0.18229E+00_JPRB, 0.18313E+00_JPRB, 0.18414E+00_JPRB ,&
&  0.18580E+00_JPRB, 0.18788E+00_JPRB, 0.18884E+00_JPRB, 0.18970E+00_JPRB, 0.18945E+00_JPRB, 0.18893E+00_JPRB, 0.18814E+00_JPRB ,&
&  0.18748E+00_JPRB, 0.18728E+00_JPRB, 0.18730E+00_JPRB, 0.18722E+00_JPRB, 0.18769E+00_JPRB, 0.18857E+00_JPRB, 0.18947E+00_JPRB ,&
&  0.19102E+00_JPRB, 0.19241E+00_JPRB, 0.19400E+00_JPRB, 0.19593E+00_JPRB, 0.19811E+00_JPRB, 0.20071E+00_JPRB, 0.20361E+00_JPRB ,&
&  0.20617E+00_JPRB, 0.20945E+00_JPRB, 0.21221E+00_JPRB, 0.21439E+00_JPRB, 0.21986E+00_JPRB, 0.22705E+00_JPRB, 0.24159E+00_JPRB ,&
&  0.25806E+00_JPRB, 0.29734E+00_JPRB, 0.33907E+00_JPRB, 0.32494E+00_JPRB, 0.33015E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.31119E+00_JPRB, 0.32218E+00_JPRB, 0.33088E+00_JPRB, 0.33635E+00_JPRB, 0.34184E+00_JPRB, 0.34391E+00_JPRB, 0.35188E+00_JPRB ,&
&  0.34636E+00_JPRB, 0.32840E+00_JPRB, 0.31724E+00_JPRB, 0.30073E+00_JPRB, 0.28043E+00_JPRB, 0.26377E+00_JPRB, 0.25532E+00_JPRB ,&
&  0.25029E+00_JPRB, 0.24624E+00_JPRB, 0.24138E+00_JPRB, 0.23607E+00_JPRB, 0.23072E+00_JPRB, 0.22675E+00_JPRB, 0.22423E+00_JPRB ,&
&  0.22193E+00_JPRB, 0.22267E+00_JPRB, 0.22543E+00_JPRB, 0.22992E+00_JPRB, 0.23632E+00_JPRB, 0.24058E+00_JPRB, 0.24397E+00_JPRB ,&
&  0.24585E+00_JPRB, 0.23659E+00_JPRB, 0.23858E+00_JPRB, 0.24084E+00_JPRB, 0.23960E+00_JPRB, 0.24293E+00_JPRB, 0.24847E+00_JPRB ,&
&  0.24597E+00_JPRB, 0.23842E+00_JPRB, 0.24088E+00_JPRB, 0.23545E+00_JPRB, 0.22697E+00_JPRB, 0.22540E+00_JPRB, 0.21962E+00_JPRB ,&
&  0.21706E+00_JPRB, 0.20995E+00_JPRB, 0.20488E+00_JPRB, 0.20154E+00_JPRB, 0.19571E+00_JPRB, 0.19334E+00_JPRB, 0.18980E+00_JPRB ,&
&  0.18622E+00_JPRB, 0.18381E+00_JPRB, 0.18092E+00_JPRB, 0.17957E+00_JPRB, 0.17841E+00_JPRB, 0.17711E+00_JPRB, 0.17608E+00_JPRB ,&
&  0.17550E+00_JPRB, 0.17495E+00_JPRB, 0.17499E+00_JPRB, 0.17502E+00_JPRB, 0.17501E+00_JPRB, 0.17533E+00_JPRB, 0.17551E+00_JPRB ,&
&  0.17615E+00_JPRB, 0.17694E+00_JPRB, 0.17758E+00_JPRB, 0.17863E+00_JPRB, 0.17907E+00_JPRB, 0.17966E+00_JPRB, 0.18009E+00_JPRB ,&
&  0.18053E+00_JPRB, 0.18121E+00_JPRB, 0.18151E+00_JPRB, 0.18217E+00_JPRB, 0.18290E+00_JPRB, 0.18439E+00_JPRB, 0.18586E+00_JPRB ,&
&  0.18765E+00_JPRB, 0.18978E+00_JPRB, 0.19202E+00_JPRB, 0.19441E+00_JPRB, 0.19736E+00_JPRB, 0.20004E+00_JPRB, 0.20359E+00_JPRB ,&
&  0.20745E+00_JPRB, 0.21114E+00_JPRB, 0.21556E+00_JPRB, 0.22108E+00_JPRB, 0.22843E+00_JPRB, 0.23816E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.25701E+00_JPRB, 0.28274E+00_JPRB, 0.28903E+00_JPRB, 0.29375E+00_JPRB, 0.30250E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.30266E+00_JPRB, 0.30764E+00_JPRB, 0.31145E+00_JPRB, 0.31390E+00_JPRB, 0.31626E+00_JPRB, 0.31790E+00_JPRB, 0.31849E+00_JPRB ,&
&  0.31200E+00_JPRB, 0.30703E+00_JPRB, 0.30002E+00_JPRB, 0.29156E+00_JPRB, 0.28383E+00_JPRB, 0.27646E+00_JPRB, 0.26826E+00_JPRB ,&
&  0.26155E+00_JPRB, 0.25521E+00_JPRB, 0.24870E+00_JPRB, 0.24183E+00_JPRB, 0.23465E+00_JPRB, 0.22742E+00_JPRB, 0.22412E+00_JPRB ,&
&  0.22329E+00_JPRB, 0.22458E+00_JPRB, 0.22681E+00_JPRB, 0.22837E+00_JPRB, 0.22931E+00_JPRB, 0.22987E+00_JPRB, 0.22869E+00_JPRB ,&
&  0.22489E+00_JPRB, 0.22560E+00_JPRB, 0.22664E+00_JPRB, 0.22732E+00_JPRB, 0.22895E+00_JPRB, 0.23039E+00_JPRB, 0.23026E+00_JPRB ,&
&  0.22843E+00_JPRB, 0.22598E+00_JPRB, 0.22260E+00_JPRB, 0.21943E+00_JPRB, 0.21831E+00_JPRB, 0.21618E+00_JPRB, 0.21378E+00_JPRB ,&
&  0.21196E+00_JPRB, 0.20810E+00_JPRB, 0.20407E+00_JPRB, 0.20122E+00_JPRB, 0.19740E+00_JPRB, 0.19480E+00_JPRB, 0.19138E+00_JPRB ,&
&  0.18865E+00_JPRB, 0.18621E+00_JPRB, 0.18353E+00_JPRB, 0.18149E+00_JPRB, 0.17958E+00_JPRB, 0.17818E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17595E+00_JPRB, 0.17516E+00_JPRB, 0.17467E+00_JPRB, 0.17410E+00_JPRB, 0.17377E+00_JPRB, 0.17342E+00_JPRB, 0.17316E+00_JPRB ,&
&  0.17301E+00_JPRB, 0.17336E+00_JPRB, 0.17360E+00_JPRB, 0.17403E+00_JPRB, 0.17452E+00_JPRB, 0.17480E+00_JPRB, 0.17534E+00_JPRB ,&
&  0.17639E+00_JPRB, 0.17752E+00_JPRB, 0.17823E+00_JPRB, 0.17917E+00_JPRB, 0.18023E+00_JPRB, 0.18152E+00_JPRB, 0.18330E+00_JPRB ,&
&  0.18516E+00_JPRB, 0.18725E+00_JPRB, 0.18961E+00_JPRB, 0.19193E+00_JPRB, 0.19495E+00_JPRB, 0.19804E+00_JPRB, 0.20131E+00_JPRB ,&
&  0.20524E+00_JPRB, 0.20942E+00_JPRB, 0.21376E+00_JPRB, 0.21854E+00_JPRB, 0.22549E+00_JPRB, 0.23150E+00_JPRB, 0.23889E+00_JPRB ,&
&  0.24599E+00_JPRB, 0.25105E+00_JPRB, 0.28330E+00_JPRB, 0.28807E+00_JPRB, 0.29483E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.22892E+00_JPRB, 0.23554E+00_JPRB, 0.24211E+00_JPRB, 0.25146E+00_JPRB, 0.25728E+00_JPRB, 0.29285E+00_JPRB, 0.28773E+00_JPRB ,&
&  0.28446E+00_JPRB, 0.27130E+00_JPRB, 0.26998E+00_JPRB, 0.26648E+00_JPRB, 0.26202E+00_JPRB, 0.25721E+00_JPRB, 0.25230E+00_JPRB ,&
&  0.24771E+00_JPRB, 0.24229E+00_JPRB, 0.23770E+00_JPRB, 0.23408E+00_JPRB, 0.23084E+00_JPRB, 0.22766E+00_JPRB, 0.22489E+00_JPRB ,&
&  0.22373E+00_JPRB, 0.22363E+00_JPRB, 0.22287E+00_JPRB, 0.21858E+00_JPRB, 0.21506E+00_JPRB, 0.21290E+00_JPRB, 0.21225E+00_JPRB ,&
&  0.21219E+00_JPRB, 0.21396E+00_JPRB, 0.21512E+00_JPRB, 0.21612E+00_JPRB, 0.21760E+00_JPRB, 0.21679E+00_JPRB, 0.21595E+00_JPRB ,&
&  0.21474E+00_JPRB, 0.21340E+00_JPRB, 0.21201E+00_JPRB, 0.21079E+00_JPRB, 0.20957E+00_JPRB, 0.20774E+00_JPRB, 0.20620E+00_JPRB ,&
&  0.20468E+00_JPRB, 0.20270E+00_JPRB, 0.19997E+00_JPRB, 0.19817E+00_JPRB, 0.19606E+00_JPRB, 0.19426E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.19108E+00_JPRB, 0.18985E+00_JPRB, 0.18842E+00_JPRB, 0.18669E+00_JPRB, 0.18498E+00_JPRB, 0.18347E+00_JPRB, 0.18204E+00_JPRB ,&
&  0.18127E+00_JPRB, 0.18047E+00_JPRB, 0.17968E+00_JPRB, 0.17883E+00_JPRB, 0.17842E+00_JPRB, 0.17799E+00_JPRB, 0.17759E+00_JPRB ,&
&  0.17726E+00_JPRB, 0.17706E+00_JPRB, 0.17707E+00_JPRB, 0.17715E+00_JPRB, 0.17725E+00_JPRB, 0.17762E+00_JPRB, 0.17806E+00_JPRB ,&
&  0.17866E+00_JPRB, 0.17940E+00_JPRB, 0.18044E+00_JPRB, 0.18151E+00_JPRB, 0.18247E+00_JPRB, 0.18323E+00_JPRB, 0.18421E+00_JPRB ,&
&  0.18579E+00_JPRB, 0.18761E+00_JPRB, 0.18961E+00_JPRB, 0.19111E+00_JPRB, 0.19329E+00_JPRB, 0.19527E+00_JPRB, 0.19724E+00_JPRB ,&
&  0.19919E+00_JPRB, 0.20155E+00_JPRB, 0.20421E+00_JPRB, 0.20710E+00_JPRB, 0.21048E+00_JPRB, 0.21330E+00_JPRB, 0.21595E+00_JPRB ,&
&  0.21865E+00_JPRB, 0.22254E+00_JPRB, 0.22456E+00_JPRB, 0.22566E+00_JPRB, 0.22705E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.20779E+00_JPRB, 0.20790E+00_JPRB, 0.20831E+00_JPRB, 0.20877E+00_JPRB, 0.20920E+00_JPRB, 0.20988E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.21001E+00_JPRB, 0.20985E+00_JPRB, 0.21017E+00_JPRB, 0.21055E+00_JPRB, 0.21077E+00_JPRB, 0.21076E+00_JPRB, 0.21036E+00_JPRB ,&
&  0.20966E+00_JPRB, 0.20881E+00_JPRB, 0.20747E+00_JPRB, 0.20688E+00_JPRB, 0.20644E+00_JPRB, 0.20603E+00_JPRB, 0.20523E+00_JPRB ,&
&  0.20453E+00_JPRB, 0.20331E+00_JPRB, 0.20163E+00_JPRB, 0.20040E+00_JPRB, 0.20021E+00_JPRB, 0.20053E+00_JPRB, 0.20119E+00_JPRB ,&
&  0.20171E+00_JPRB, 0.20205E+00_JPRB, 0.20218E+00_JPRB, 0.20238E+00_JPRB, 0.20267E+00_JPRB, 0.20254E+00_JPRB, 0.20230E+00_JPRB ,&
&  0.20198E+00_JPRB, 0.20145E+00_JPRB, 0.20094E+00_JPRB, 0.20047E+00_JPRB, 0.20005E+00_JPRB, 0.19955E+00_JPRB, 0.19895E+00_JPRB ,&
&  0.19826E+00_JPRB, 0.19741E+00_JPRB, 0.19638E+00_JPRB, 0.19556E+00_JPRB, 0.19470E+00_JPRB, 0.19403E+00_JPRB, 0.19319E+00_JPRB ,&
&  0.19249E+00_JPRB, 0.19185E+00_JPRB, 0.19120E+00_JPRB, 0.19060E+00_JPRB, 0.19003E+00_JPRB, 0.18948E+00_JPRB, 0.18895E+00_JPRB ,&
&  0.18844E+00_JPRB, 0.18801E+00_JPRB, 0.18754E+00_JPRB, 0.18710E+00_JPRB, 0.18680E+00_JPRB, 0.18655E+00_JPRB, 0.18634E+00_JPRB ,&
&  0.18616E+00_JPRB, 0.18611E+00_JPRB, 0.18609E+00_JPRB, 0.18607E+00_JPRB, 0.18607E+00_JPRB, 0.18618E+00_JPRB, 0.18630E+00_JPRB ,&
&  0.18646E+00_JPRB, 0.18657E+00_JPRB, 0.18669E+00_JPRB, 0.18687E+00_JPRB, 0.18707E+00_JPRB, 0.18730E+00_JPRB, 0.18761E+00_JPRB ,&
&  0.18799E+00_JPRB, 0.18846E+00_JPRB, 0.18894E+00_JPRB, 0.18923E+00_JPRB, 0.18990E+00_JPRB, 0.19065E+00_JPRB, 0.19154E+00_JPRB ,&
&  0.19253E+00_JPRB, 0.19369E+00_JPRB, 0.19497E+00_JPRB, 0.19648E+00_JPRB, 0.19804E+00_JPRB, 0.19903E+00_JPRB, 0.20003E+00_JPRB ,&
&  0.20107E+00_JPRB, 0.20231E+00_JPRB, 0.20347E+00_JPRB, 0.20467E+00_JPRB, 0.20610E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB, 0.19258E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB, 0.77288E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.79445E+00_JPRB, 0.79366E+00_JPRB, 0.79289E+00_JPRB, 0.79200E+00_JPRB, 0.79088E+00_JPRB, 0.78962E+00_JPRB, 0.78856E+00_JPRB ,&
&  0.78740E+00_JPRB, 0.78657E+00_JPRB, 0.78516E+00_JPRB, 0.78369E+00_JPRB, 0.78212E+00_JPRB, 0.78072E+00_JPRB, 0.77920E+00_JPRB ,&
&  0.77757E+00_JPRB, 0.77592E+00_JPRB, 0.77440E+00_JPRB, 0.77270E+00_JPRB, 0.77086E+00_JPRB, 0.76910E+00_JPRB, 0.76734E+00_JPRB ,&
&  0.76582E+00_JPRB, 0.76413E+00_JPRB, 0.76246E+00_JPRB, 0.76055E+00_JPRB, 0.75926E+00_JPRB, 0.75749E+00_JPRB, 0.75602E+00_JPRB ,&
&  0.75436E+00_JPRB, 0.75319E+00_JPRB, 0.75185E+00_JPRB, 0.75071E+00_JPRB, 0.74942E+00_JPRB, 0.74906E+00_JPRB, 0.74832E+00_JPRB ,&
&  0.74784E+00_JPRB, 0.74741E+00_JPRB, 0.74736E+00_JPRB, 0.74721E+00_JPRB, 0.74714E+00_JPRB, 0.74730E+00_JPRB, 0.74800E+00_JPRB ,&
&  0.74863E+00_JPRB, 0.74933E+00_JPRB, 0.75042E+00_JPRB, 0.75151E+00_JPRB, 0.75251E+00_JPRB, 0.75348E+00_JPRB, 0.75455E+00_JPRB ,&
&  0.75620E+00_JPRB, 0.75796E+00_JPRB, 0.75970E+00_JPRB, 0.76156E+00_JPRB, 0.76335E+00_JPRB, 0.76472E+00_JPRB, 0.76591E+00_JPRB ,&
&  0.76676E+00_JPRB, 0.76776E+00_JPRB, 0.76864E+00_JPRB, 0.76948E+00_JPRB, 0.76971E+00_JPRB, 0.77030E+00_JPRB, 0.77081E+00_JPRB ,&
&  0.77118E+00_JPRB, 0.77085E+00_JPRB, 0.77109E+00_JPRB, 0.77122E+00_JPRB, 0.77149E+00_JPRB, 0.77159E+00_JPRB, 0.77234E+00_JPRB ,&
&  0.77305E+00_JPRB, 0.77416E+00_JPRB, 0.77537E+00_JPRB, 0.77691E+00_JPRB, 0.77810E+00_JPRB, 0.77935E+00_JPRB, 0.78051E+00_JPRB ,&
&  0.78183E+00_JPRB, 0.78327E+00_JPRB, 0.78483E+00_JPRB, 0.78615E+00_JPRB, 0.78744E+00_JPRB, 0.78885E+00_JPRB, 0.79019E+00_JPRB ,&
&  0.79175E+00_JPRB, 0.79277E+00_JPRB, 0.79373E+00_JPRB, 0.79452E+00_JPRB, 0.79502E+00_JPRB, 0.79536E+00_JPRB, 0.79552E+00_JPRB ,&
&  0.79553E+00_JPRB, 0.79543E+00_JPRB, 0.79524E+00_JPRB, 0.79513E+00_JPRB, 0.79498E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.80685E+00_JPRB, 0.80609E+00_JPRB, 0.80501E+00_JPRB, 0.80338E+00_JPRB, 0.80152E+00_JPRB, 0.79924E+00_JPRB, 0.79700E+00_JPRB ,&
&  0.79433E+00_JPRB, 0.79198E+00_JPRB, 0.78918E+00_JPRB, 0.78626E+00_JPRB, 0.78312E+00_JPRB, 0.78026E+00_JPRB, 0.77742E+00_JPRB ,&
&  0.77456E+00_JPRB, 0.77161E+00_JPRB, 0.76938E+00_JPRB, 0.76718E+00_JPRB, 0.76459E+00_JPRB, 0.76138E+00_JPRB, 0.75812E+00_JPRB ,&
&  0.75514E+00_JPRB, 0.75164E+00_JPRB, 0.74787E+00_JPRB, 0.74225E+00_JPRB, 0.73752E+00_JPRB, 0.73302E+00_JPRB, 0.72910E+00_JPRB ,&
&  0.72525E+00_JPRB, 0.72201E+00_JPRB, 0.71953E+00_JPRB, 0.71763E+00_JPRB, 0.71808E+00_JPRB, 0.71803E+00_JPRB, 0.71763E+00_JPRB ,&
&  0.71701E+00_JPRB, 0.71506E+00_JPRB, 0.71444E+00_JPRB, 0.71427E+00_JPRB, 0.71429E+00_JPRB, 0.71612E+00_JPRB, 0.71783E+00_JPRB ,&
&  0.71963E+00_JPRB, 0.72123E+00_JPRB, 0.72405E+00_JPRB, 0.72686E+00_JPRB, 0.72953E+00_JPRB, 0.73238E+00_JPRB, 0.73448E+00_JPRB ,&
&  0.73686E+00_JPRB, 0.74030E+00_JPRB, 0.74368E+00_JPRB, 0.74737E+00_JPRB, 0.74981E+00_JPRB, 0.75199E+00_JPRB, 0.75376E+00_JPRB ,&
&  0.75517E+00_JPRB, 0.75668E+00_JPRB, 0.75827E+00_JPRB, 0.75978E+00_JPRB, 0.76066E+00_JPRB, 0.76170E+00_JPRB, 0.76248E+00_JPRB ,&
&  0.76299E+00_JPRB, 0.76244E+00_JPRB, 0.76246E+00_JPRB, 0.76236E+00_JPRB, 0.76195E+00_JPRB, 0.76106E+00_JPRB, 0.76110E+00_JPRB ,&
&  0.76107E+00_JPRB, 0.76077E+00_JPRB, 0.76007E+00_JPRB, 0.76090E+00_JPRB, 0.76171E+00_JPRB, 0.76220E+00_JPRB, 0.76214E+00_JPRB ,&
&  0.76554E+00_JPRB, 0.77069E+00_JPRB, 0.77726E+00_JPRB, 0.78622E+00_JPRB, 0.79313E+00_JPRB, 0.79784E+00_JPRB, 0.80148E+00_JPRB ,&
&  0.80343E+00_JPRB, 0.80479E+00_JPRB, 0.80554E+00_JPRB, 0.80628E+00_JPRB, 0.80599E+00_JPRB, 0.80641E+00_JPRB, 0.80762E+00_JPRB ,&
&  0.80848E+00_JPRB, 0.80864E+00_JPRB, 0.80854E+00_JPRB, 0.80822E+00_JPRB, 0.80763E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.81254E+00_JPRB, 0.80922E+00_JPRB, 0.80583E+00_JPRB, 0.80143E+00_JPRB, 0.79833E+00_JPRB, 0.79433E+00_JPRB, 0.79096E+00_JPRB ,&
&  0.78715E+00_JPRB, 0.78332E+00_JPRB, 0.78029E+00_JPRB, 0.77770E+00_JPRB, 0.77502E+00_JPRB, 0.77233E+00_JPRB, 0.76985E+00_JPRB ,&
&  0.76722E+00_JPRB, 0.76441E+00_JPRB, 0.76271E+00_JPRB, 0.76090E+00_JPRB, 0.75883E+00_JPRB, 0.75637E+00_JPRB, 0.75163E+00_JPRB ,&
&  0.74574E+00_JPRB, 0.73814E+00_JPRB, 0.72982E+00_JPRB, 0.72185E+00_JPRB, 0.71763E+00_JPRB, 0.68951E+00_JPRB, 0.68457E+00_JPRB ,&
&  0.69018E+00_JPRB, 0.69370E+00_JPRB, 0.69965E+00_JPRB, 0.70579E+00_JPRB, 0.70134E+00_JPRB, 0.68902E+00_JPRB, 0.68130E+00_JPRB ,&
&  0.65831E+00_JPRB, 0.66371E+00_JPRB, 0.66523E+00_JPRB, 0.66747E+00_JPRB, 0.67257E+00_JPRB, 0.67359E+00_JPRB, 0.67854E+00_JPRB ,&
&  0.69525E+00_JPRB, 0.69645E+00_JPRB, 0.69933E+00_JPRB, 0.70200E+00_JPRB, 0.70454E+00_JPRB, 0.70716E+00_JPRB, 0.71133E+00_JPRB ,&
&  0.71488E+00_JPRB, 0.71928E+00_JPRB, 0.72432E+00_JPRB, 0.72756E+00_JPRB, 0.73070E+00_JPRB, 0.73450E+00_JPRB, 0.73800E+00_JPRB ,&
&  0.74021E+00_JPRB, 0.74234E+00_JPRB, 0.74459E+00_JPRB, 0.74705E+00_JPRB, 0.74937E+00_JPRB, 0.75120E+00_JPRB, 0.75280E+00_JPRB ,&
&  0.75429E+00_JPRB, 0.75417E+00_JPRB, 0.75439E+00_JPRB, 0.75493E+00_JPRB, 0.75501E+00_JPRB, 0.75492E+00_JPRB, 0.75562E+00_JPRB ,&
&  0.75718E+00_JPRB, 0.75876E+00_JPRB, 0.76089E+00_JPRB, 0.76406E+00_JPRB, 0.77045E+00_JPRB, 0.77903E+00_JPRB, 0.79499E+00_JPRB ,&
&  0.80819E+00_JPRB, 0.81972E+00_JPRB, 0.82479E+00_JPRB, 0.82976E+00_JPRB, 0.83472E+00_JPRB, 0.83709E+00_JPRB, 0.83881E+00_JPRB ,&
&  0.83826E+00_JPRB, 0.83644E+00_JPRB, 0.83321E+00_JPRB, 0.82950E+00_JPRB, 0.82612E+00_JPRB, 0.82304E+00_JPRB, 0.82107E+00_JPRB ,&
&  0.82012E+00_JPRB, 0.82051E+00_JPRB, 0.82019E+00_JPRB, 0.81804E+00_JPRB, 0.81519E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.82350E+00_JPRB, 0.81636E+00_JPRB, 0.81177E+00_JPRB, 0.80538E+00_JPRB, 0.79984E+00_JPRB, 0.79306E+00_JPRB, 0.78730E+00_JPRB ,&
&  0.78123E+00_JPRB, 0.77577E+00_JPRB, 0.77084E+00_JPRB, 0.76633E+00_JPRB, 0.76364E+00_JPRB, 0.76112E+00_JPRB, 0.75946E+00_JPRB ,&
&  0.75761E+00_JPRB, 0.75557E+00_JPRB, 0.75406E+00_JPRB, 0.75258E+00_JPRB, 0.75127E+00_JPRB, 0.74999E+00_JPRB, 0.74706E+00_JPRB ,&
&  0.73886E+00_JPRB, 0.72681E+00_JPRB, 0.70140E+00_JPRB, 0.70029E+00_JPRB, 0.69432E+00_JPRB, 0.69078E+00_JPRB, 0.69287E+00_JPRB ,&
&  0.68664E+00_JPRB, 0.68520E+00_JPRB, 0.67580E+00_JPRB, 0.66427E+00_JPRB, 0.65460E+00_JPRB, 0.63798E+00_JPRB, 0.63059E+00_JPRB ,&
&  0.62287E+00_JPRB, 0.61852E+00_JPRB, 0.61837E+00_JPRB, 0.61843E+00_JPRB, 0.62525E+00_JPRB, 0.63094E+00_JPRB, 0.63825E+00_JPRB ,&
&  0.64987E+00_JPRB, 0.65978E+00_JPRB, 0.66960E+00_JPRB, 0.66634E+00_JPRB, 0.67049E+00_JPRB, 0.67469E+00_JPRB, 0.68012E+00_JPRB ,&
&  0.68534E+00_JPRB, 0.68997E+00_JPRB, 0.69492E+00_JPRB, 0.69948E+00_JPRB, 0.70444E+00_JPRB, 0.71013E+00_JPRB, 0.71594E+00_JPRB ,&
&  0.72226E+00_JPRB, 0.72674E+00_JPRB, 0.72963E+00_JPRB, 0.73291E+00_JPRB, 0.73645E+00_JPRB, 0.74012E+00_JPRB, 0.74492E+00_JPRB ,&
&  0.75062E+00_JPRB, 0.75743E+00_JPRB, 0.76495E+00_JPRB, 0.77294E+00_JPRB, 0.78078E+00_JPRB, 0.79092E+00_JPRB, 0.80239E+00_JPRB ,&
&  0.81848E+00_JPRB, 0.83323E+00_JPRB, 0.85470E+00_JPRB, 0.86765E+00_JPRB, 0.89357E+00_JPRB, 0.91743E+00_JPRB, 0.93043E+00_JPRB ,&
&  0.92776E+00_JPRB, 0.92041E+00_JPRB, 0.91343E+00_JPRB, 0.90527E+00_JPRB, 0.89864E+00_JPRB, 0.88991E+00_JPRB, 0.88436E+00_JPRB ,&
&  0.87741E+00_JPRB, 0.87172E+00_JPRB, 0.86534E+00_JPRB, 0.85973E+00_JPRB, 0.85462E+00_JPRB, 0.84940E+00_JPRB, 0.84495E+00_JPRB ,&
&  0.84163E+00_JPRB, 0.83978E+00_JPRB, 0.83758E+00_JPRB, 0.83442E+00_JPRB, 0.82956E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.83719E+00_JPRB, 0.83267E+00_JPRB, 0.82962E+00_JPRB, 0.82370E+00_JPRB, 0.81650E+00_JPRB, 0.80934E+00_JPRB, 0.79815E+00_JPRB ,&
&  0.78752E+00_JPRB, 0.77705E+00_JPRB, 0.76782E+00_JPRB, 0.76043E+00_JPRB, 0.75436E+00_JPRB, 0.74977E+00_JPRB, 0.74779E+00_JPRB ,&
&  0.74508E+00_JPRB, 0.74445E+00_JPRB, 0.74400E+00_JPRB, 0.74436E+00_JPRB, 0.74551E+00_JPRB, 0.74629E+00_JPRB, 0.74720E+00_JPRB ,&
&  0.74232E+00_JPRB, 0.72931E+00_JPRB, 0.71177E+00_JPRB, 0.71203E+00_JPRB, 0.71232E+00_JPRB, 0.71756E+00_JPRB, 0.72231E+00_JPRB ,&
&  0.73051E+00_JPRB, 0.69719E+00_JPRB, 0.70107E+00_JPRB, 0.67871E+00_JPRB, 0.66203E+00_JPRB, 0.64493E+00_JPRB, 0.62154E+00_JPRB ,&
&  0.61316E+00_JPRB, 0.60178E+00_JPRB, 0.59566E+00_JPRB, 0.59618E+00_JPRB, 0.59978E+00_JPRB, 0.60957E+00_JPRB, 0.62150E+00_JPRB ,&
&  0.62651E+00_JPRB, 0.64904E+00_JPRB, 0.65166E+00_JPRB, 0.65169E+00_JPRB, 0.65467E+00_JPRB, 0.65816E+00_JPRB, 0.65508E+00_JPRB ,&
&  0.65678E+00_JPRB, 0.65727E+00_JPRB, 0.65947E+00_JPRB, 0.66741E+00_JPRB, 0.67981E+00_JPRB, 0.68742E+00_JPRB, 0.69460E+00_JPRB ,&
&  0.70363E+00_JPRB, 0.70908E+00_JPRB, 0.71243E+00_JPRB, 0.71641E+00_JPRB, 0.72200E+00_JPRB, 0.72818E+00_JPRB, 0.73834E+00_JPRB ,&
&  0.75064E+00_JPRB, 0.77270E+00_JPRB, 0.79309E+00_JPRB, 0.82305E+00_JPRB, 0.85101E+00_JPRB, 0.88639E+00_JPRB, 0.91638E+00_JPRB ,&
&  0.94635E+00_JPRB, 0.96927E+00_JPRB, 0.99271E+00_JPRB, 0.10140E+01_JPRB, 0.10301E+01_JPRB, 0.10210E+01_JPRB, 0.99433E+00_JPRB ,&
&  0.97733E+00_JPRB, 0.95218E+00_JPRB, 0.93558E+00_JPRB, 0.92147E+00_JPRB, 0.91080E+00_JPRB, 0.90468E+00_JPRB, 0.89877E+00_JPRB ,&
&  0.89430E+00_JPRB, 0.88882E+00_JPRB, 0.88288E+00_JPRB, 0.87534E+00_JPRB, 0.86913E+00_JPRB, 0.86309E+00_JPRB, 0.85877E+00_JPRB ,&
&  0.85379E+00_JPRB, 0.85173E+00_JPRB, 0.84720E+00_JPRB, 0.84284E+00_JPRB, 0.83951E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.85174E+00_JPRB, 0.85120E+00_JPRB, 0.84245E+00_JPRB, 0.83447E+00_JPRB, 0.82065E+00_JPRB, 0.80845E+00_JPRB, 0.79154E+00_JPRB ,&
&  0.77588E+00_JPRB, 0.76063E+00_JPRB, 0.74732E+00_JPRB, 0.73731E+00_JPRB, 0.72912E+00_JPRB, 0.72329E+00_JPRB, 0.72084E+00_JPRB ,&
&  0.72008E+00_JPRB, 0.72084E+00_JPRB, 0.72235E+00_JPRB, 0.72474E+00_JPRB, 0.72894E+00_JPRB, 0.73383E+00_JPRB, 0.73926E+00_JPRB ,&
&  0.74085E+00_JPRB, 0.73555E+00_JPRB, 0.71994E+00_JPRB, 0.71743E+00_JPRB, 0.72590E+00_JPRB, 0.72698E+00_JPRB, 0.74893E+00_JPRB ,&
&  0.76101E+00_JPRB, 0.77202E+00_JPRB, 0.76636E+00_JPRB, 0.76182E+00_JPRB, 0.75259E+00_JPRB, 0.66597E+00_JPRB, 0.65451E+00_JPRB ,&
&  0.63495E+00_JPRB, 0.61338E+00_JPRB, 0.60272E+00_JPRB, 0.59504E+00_JPRB, 0.59329E+00_JPRB, 0.60312E+00_JPRB, 0.61802E+00_JPRB ,&
&  0.62504E+00_JPRB, 0.65544E+00_JPRB, 0.65469E+00_JPRB, 0.65203E+00_JPRB, 0.65044E+00_JPRB, 0.65197E+00_JPRB, 0.64765E+00_JPRB ,&
&  0.65000E+00_JPRB, 0.65254E+00_JPRB, 0.65735E+00_JPRB, 0.66973E+00_JPRB, 0.68274E+00_JPRB, 0.69421E+00_JPRB, 0.70900E+00_JPRB ,&
&  0.73259E+00_JPRB, 0.75051E+00_JPRB, 0.76103E+00_JPRB, 0.77074E+00_JPRB, 0.78661E+00_JPRB, 0.80622E+00_JPRB, 0.83366E+00_JPRB ,&
&  0.85300E+00_JPRB, 0.87544E+00_JPRB, 0.89854E+00_JPRB, 0.92377E+00_JPRB, 0.94746E+00_JPRB, 0.97187E+00_JPRB, 0.99646E+00_JPRB ,&
&  0.10086E+01_JPRB, 0.10238E+01_JPRB, 0.10320E+01_JPRB, 0.10557E+01_JPRB, 0.10477E+01_JPRB, 0.10208E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.98587E+00_JPRB, 0.98561E+00_JPRB, 0.97099E+00_JPRB, 0.95888E+00_JPRB, 0.94697E+00_JPRB, 0.94037E+00_JPRB, 0.93191E+00_JPRB ,&
&  0.92553E+00_JPRB, 0.91774E+00_JPRB, 0.90841E+00_JPRB, 0.89701E+00_JPRB, 0.88874E+00_JPRB, 0.88004E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.86558E+00_JPRB, 0.86049E+00_JPRB, 0.85639E+00_JPRB, 0.85368E+00_JPRB, 0.85034E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.85780E+00_JPRB, 0.85201E+00_JPRB, 0.83587E+00_JPRB, 0.82527E+00_JPRB, 0.80556E+00_JPRB, 0.78427E+00_JPRB, 0.75960E+00_JPRB ,&
&  0.74030E+00_JPRB, 0.72553E+00_JPRB, 0.71300E+00_JPRB, 0.70572E+00_JPRB, 0.70180E+00_JPRB, 0.69974E+00_JPRB, 0.70123E+00_JPRB ,&
&  0.70378E+00_JPRB, 0.70130E+00_JPRB, 0.70437E+00_JPRB, 0.71081E+00_JPRB, 0.71763E+00_JPRB, 0.72398E+00_JPRB, 0.73114E+00_JPRB ,&
&  0.73289E+00_JPRB, 0.73156E+00_JPRB, 0.72812E+00_JPRB, 0.72288E+00_JPRB, 0.72239E+00_JPRB, 0.73409E+00_JPRB, 0.75429E+00_JPRB ,&
&  0.76210E+00_JPRB, 0.77220E+00_JPRB, 0.79015E+00_JPRB, 0.80753E+00_JPRB, 0.80386E+00_JPRB, 0.76427E+00_JPRB, 0.69705E+00_JPRB ,&
&  0.64967E+00_JPRB, 0.60785E+00_JPRB, 0.58792E+00_JPRB, 0.57885E+00_JPRB, 0.58446E+00_JPRB, 0.59924E+00_JPRB, 0.62562E+00_JPRB ,&
&  0.64419E+00_JPRB, 0.65334E+00_JPRB, 0.65724E+00_JPRB, 0.65987E+00_JPRB, 0.65856E+00_JPRB, 0.66066E+00_JPRB, 0.66236E+00_JPRB ,&
&  0.67186E+00_JPRB, 0.68104E+00_JPRB, 0.69319E+00_JPRB, 0.71314E+00_JPRB, 0.74106E+00_JPRB, 0.77210E+00_JPRB, 0.81475E+00_JPRB ,&
&  0.89747E+00_JPRB, 0.93868E+00_JPRB, 0.97209E+00_JPRB, 0.99234E+00_JPRB, 0.10193E+01_JPRB, 0.10319E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.10459E+01_JPRB, 0.10522E+01_JPRB, 0.10571E+01_JPRB, 0.10608E+01_JPRB, 0.10666E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.10541E+01_JPRB, 0.10635E+01_JPRB, 0.11493E+01_JPRB, 0.11696E+01_JPRB, 0.11466E+01_JPRB, 0.11157E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.10274E+01_JPRB, 0.10058E+01_JPRB, 0.98704E+00_JPRB, 0.97315E+00_JPRB, 0.96607E+00_JPRB, 0.96529E+00_JPRB ,&
&  0.95664E+00_JPRB, 0.94489E+00_JPRB, 0.92969E+00_JPRB, 0.91842E+00_JPRB, 0.90926E+00_JPRB, 0.89748E+00_JPRB, 0.88851E+00_JPRB ,&
&  0.87999E+00_JPRB, 0.87617E+00_JPRB, 0.87303E+00_JPRB, 0.87017E+00_JPRB, 0.86529E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.85584E+00_JPRB, 0.85107E+00_JPRB, 0.83739E+00_JPRB, 0.81233E+00_JPRB, 0.77457E+00_JPRB, 0.73814E+00_JPRB, 0.71312E+00_JPRB ,&
&  0.70123E+00_JPRB, 0.69482E+00_JPRB, 0.68584E+00_JPRB, 0.68155E+00_JPRB, 0.67985E+00_JPRB, 0.68049E+00_JPRB, 0.68351E+00_JPRB ,&
&  0.68866E+00_JPRB, 0.69434E+00_JPRB, 0.69890E+00_JPRB, 0.70171E+00_JPRB, 0.70772E+00_JPRB, 0.71753E+00_JPRB, 0.72902E+00_JPRB ,&
&  0.73966E+00_JPRB, 0.74379E+00_JPRB, 0.74652E+00_JPRB, 0.75031E+00_JPRB, 0.75292E+00_JPRB, 0.76174E+00_JPRB, 0.76383E+00_JPRB ,&
&  0.77025E+00_JPRB, 0.78615E+00_JPRB, 0.81106E+00_JPRB, 0.84308E+00_JPRB, 0.86264E+00_JPRB, 0.82028E+00_JPRB, 0.73645E+00_JPRB ,&
&  0.65804E+00_JPRB, 0.60366E+00_JPRB, 0.58028E+00_JPRB, 0.57472E+00_JPRB, 0.58545E+00_JPRB, 0.61076E+00_JPRB, 0.64366E+00_JPRB ,&
&  0.66009E+00_JPRB, 0.66661E+00_JPRB, 0.67534E+00_JPRB, 0.67967E+00_JPRB, 0.67925E+00_JPRB, 0.67782E+00_JPRB, 0.68090E+00_JPRB ,&
&  0.69306E+00_JPRB, 0.72294E+00_JPRB, 0.77146E+00_JPRB, 0.84056E+00_JPRB, 0.93428E+00_JPRB, 0.97056E+00_JPRB, 0.10425E+01_JPRB ,&
&  0.11594E+01_JPRB, 0.11871E+01_JPRB, 0.12204E+01_JPRB, 0.12314E+01_JPRB, 0.12363E+01_JPRB, 0.12292E+01_JPRB, 0.12290E+01_JPRB ,&
&  0.12295E+01_JPRB, 0.12224E+01_JPRB, 0.12021E+01_JPRB, 0.11766E+01_JPRB, 0.11668E+01_JPRB, 0.11722E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.12831E+01_JPRB, 0.14286E+01_JPRB, 0.12822E+01_JPRB, 0.12207E+01_JPRB, 0.12027E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.11054E+01_JPRB, 0.10900E+01_JPRB, 0.10728E+01_JPRB, 0.10326E+01_JPRB, 0.10051E+01_JPRB, 0.98436E+00_JPRB ,&
&  0.97180E+00_JPRB, 0.95711E+00_JPRB, 0.94542E+00_JPRB, 0.93651E+00_JPRB, 0.92582E+00_JPRB, 0.91461E+00_JPRB, 0.90782E+00_JPRB ,&
&  0.89967E+00_JPRB, 0.89193E+00_JPRB, 0.88433E+00_JPRB, 0.87333E+00_JPRB, 0.86377E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.83966E+00_JPRB, 0.83552E+00_JPRB, 0.81608E+00_JPRB, 0.78715E+00_JPRB, 0.74324E+00_JPRB, 0.70977E+00_JPRB, 0.68823E+00_JPRB ,&
&  0.67693E+00_JPRB, 0.66792E+00_JPRB, 0.66169E+00_JPRB, 0.65872E+00_JPRB, 0.65913E+00_JPRB, 0.66326E+00_JPRB, 0.66742E+00_JPRB ,&
&  0.67134E+00_JPRB, 0.67733E+00_JPRB, 0.68515E+00_JPRB, 0.69295E+00_JPRB, 0.70159E+00_JPRB, 0.71198E+00_JPRB, 0.72423E+00_JPRB ,&
&  0.73958E+00_JPRB, 0.75142E+00_JPRB, 0.75960E+00_JPRB, 0.76953E+00_JPRB, 0.78717E+00_JPRB, 0.81501E+00_JPRB, 0.83779E+00_JPRB ,&
&  0.84408E+00_JPRB, 0.84014E+00_JPRB, 0.85887E+00_JPRB, 0.89347E+00_JPRB, 0.91609E+00_JPRB, 0.88410E+00_JPRB, 0.76672E+00_JPRB ,&
&  0.67010E+00_JPRB, 0.61609E+00_JPRB, 0.60362E+00_JPRB, 0.61629E+00_JPRB, 0.64468E+00_JPRB, 0.67086E+00_JPRB, 0.68413E+00_JPRB ,&
&  0.69425E+00_JPRB, 0.69765E+00_JPRB, 0.70002E+00_JPRB, 0.70546E+00_JPRB, 0.70775E+00_JPRB, 0.70983E+00_JPRB, 0.72184E+00_JPRB ,&
&  0.75292E+00_JPRB, 0.80414E+00_JPRB, 0.85869E+00_JPRB, 0.88467E+00_JPRB, 0.95569E+00_JPRB, 0.11118E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.13171E+01_JPRB, 0.13396E+01_JPRB, 0.13736E+01_JPRB, 0.14381E+01_JPRB, 0.14643E+01_JPRB, 0.14793E+01_JPRB, 0.14715E+01_JPRB ,&
&  0.14461E+01_JPRB, 0.14016E+01_JPRB, 0.13580E+01_JPRB, 0.13457E+01_JPRB, 0.13764E+01_JPRB, 0.14394E+01_JPRB, 0.14755E+01_JPRB ,&
&  0.14854E+01_JPRB, 0.14173E+01_JPRB, 0.12736E+01_JPRB, 0.11459E+01_JPRB, 0.11023E+01_JPRB, 0.11076E+01_JPRB, 0.11264E+01_JPRB ,&
&  0.11194E+01_JPRB, 0.11065E+01_JPRB, 0.10915E+01_JPRB, 0.10852E+01_JPRB, 0.10237E+01_JPRB, 0.98475E+00_JPRB, 0.95268E+00_JPRB ,&
&  0.93424E+00_JPRB, 0.92111E+00_JPRB, 0.92124E+00_JPRB, 0.92062E+00_JPRB, 0.91503E+00_JPRB, 0.90482E+00_JPRB, 0.89206E+00_JPRB ,&
&  0.88289E+00_JPRB, 0.87113E+00_JPRB, 0.85849E+00_JPRB, 0.84632E+00_JPRB, 0.83998E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.85626E+00_JPRB, 0.83474E+00_JPRB, 0.80981E+00_JPRB, 0.77765E+00_JPRB, 0.75058E+00_JPRB, 0.73559E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.72140E+00_JPRB, 0.69669E+00_JPRB, 0.67031E+00_JPRB, 0.65267E+00_JPRB, 0.64419E+00_JPRB, 0.64248E+00_JPRB, 0.64896E+00_JPRB ,&
&  0.65980E+00_JPRB, 0.67055E+00_JPRB, 0.68189E+00_JPRB, 0.69524E+00_JPRB, 0.70888E+00_JPRB, 0.72031E+00_JPRB, 0.73286E+00_JPRB ,&
&  0.75080E+00_JPRB, 0.77038E+00_JPRB, 0.79202E+00_JPRB, 0.82374E+00_JPRB, 0.85700E+00_JPRB, 0.90025E+00_JPRB, 0.92425E+00_JPRB ,&
&  0.92981E+00_JPRB, 0.92098E+00_JPRB, 0.92208E+00_JPRB, 0.95034E+00_JPRB, 0.96626E+00_JPRB, 0.93719E+00_JPRB, 0.79683E+00_JPRB ,&
&  0.68448E+00_JPRB, 0.65331E+00_JPRB, 0.67385E+00_JPRB, 0.71865E+00_JPRB, 0.72489E+00_JPRB, 0.73105E+00_JPRB, 0.73278E+00_JPRB ,&
&  0.72426E+00_JPRB, 0.71815E+00_JPRB, 0.72745E+00_JPRB, 0.74325E+00_JPRB, 0.74653E+00_JPRB, 0.75384E+00_JPRB, 0.77606E+00_JPRB ,&
&  0.82692E+00_JPRB, 0.89342E+00_JPRB, 0.96201E+00_JPRB, 0.96134E+00_JPRB, 0.10835E+01_JPRB, 0.14916E+01_JPRB, 0.15990E+01_JPRB ,&
&  0.16525E+01_JPRB, 0.16822E+01_JPRB, 0.17361E+01_JPRB, 0.17645E+01_JPRB, 0.17288E+01_JPRB, 0.17100E+01_JPRB, 0.17030E+01_JPRB ,&
&  0.16549E+01_JPRB, 0.16285E+01_JPRB, 0.16130E+01_JPRB, 0.16640E+01_JPRB, 0.16761E+01_JPRB, 0.16602E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.16054E+01_JPRB, 0.14959E+01_JPRB, 0.13737E+01_JPRB, 0.12607E+01_JPRB, 0.12049E+01_JPRB, 0.11690E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.11225E+01_JPRB, 0.11127E+01_JPRB, 0.11116E+01_JPRB, 0.11115E+01_JPRB, 0.10648E+01_JPRB, 0.99745E+00_JPRB, 0.95106E+00_JPRB ,&
&  0.92329E+00_JPRB, 0.90339E+00_JPRB, 0.87959E+00_JPRB, 0.87179E+00_JPRB, 0.88191E+00_JPRB, 0.89225E+00_JPRB, 0.88417E+00_JPRB ,&
&  0.88028E+00_JPRB, 0.87054E+00_JPRB, 0.86620E+00_JPRB, 0.86513E+00_JPRB, 0.86763E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.84777E+00_JPRB, 0.82850E+00_JPRB, 0.80622E+00_JPRB, 0.78336E+00_JPRB, 0.77180E+00_JPRB, 0.76109E+00_JPRB, 0.73874E+00_JPRB ,&
&  0.73886E+00_JPRB, 0.69755E+00_JPRB, 0.66890E+00_JPRB, 0.65828E+00_JPRB, 0.65864E+00_JPRB, 0.65430E+00_JPRB, 0.64727E+00_JPRB ,&
&  0.65079E+00_JPRB, 0.66546E+00_JPRB, 0.68550E+00_JPRB, 0.70251E+00_JPRB, 0.71932E+00_JPRB, 0.73410E+00_JPRB, 0.74796E+00_JPRB ,&
&  0.76831E+00_JPRB, 0.79333E+00_JPRB, 0.82451E+00_JPRB, 0.86491E+00_JPRB, 0.90507E+00_JPRB, 0.95572E+00_JPRB, 0.98627E+00_JPRB ,&
&  0.10088E+01_JPRB, 0.10123E+01_JPRB, 0.99105E+00_JPRB, 0.98943E+00_JPRB, 0.98415E+00_JPRB, 0.96863E+00_JPRB, 0.92057E+00_JPRB ,&
&  0.75833E+00_JPRB, 0.73132E+00_JPRB, 0.78641E+00_JPRB, 0.78484E+00_JPRB, 0.77842E+00_JPRB, 0.76684E+00_JPRB, 0.76221E+00_JPRB ,&
&  0.76213E+00_JPRB, 0.76156E+00_JPRB, 0.76632E+00_JPRB, 0.76751E+00_JPRB, 0.77471E+00_JPRB, 0.79053E+00_JPRB, 0.83700E+00_JPRB ,&
&  0.93365E+00_JPRB, 0.92296E+00_JPRB, 0.96378E+00_JPRB, 0.10620E+01_JPRB, 0.13855E+01_JPRB, 0.16126E+01_JPRB, 0.17660E+01_JPRB ,&
&  0.19709E+01_JPRB, 0.20366E+01_JPRB, 0.20453E+01_JPRB, 0.20509E+01_JPRB, 0.20515E+01_JPRB, 0.20559E+01_JPRB, 0.20688E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.19785E+01_JPRB, 0.19546E+01_JPRB, 0.19643E+01_JPRB, 0.19210E+01_JPRB, 0.18676E+01_JPRB, 0.18061E+01_JPRB ,&
&  0.17358E+01_JPRB, 0.16482E+01_JPRB, 0.15260E+01_JPRB, 0.14103E+01_JPRB, 0.13135E+01_JPRB, 0.12679E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.11948E+01_JPRB, 0.11777E+01_JPRB, 0.11582E+01_JPRB, 0.11365E+01_JPRB, 0.11074E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.97481E+00_JPRB, 0.91479E+00_JPRB, 0.89750E+00_JPRB, 0.90386E+00_JPRB, 0.91127E+00_JPRB, 0.91487E+00_JPRB ,&
&  0.92314E+00_JPRB, 0.90701E+00_JPRB, 0.89512E+00_JPRB, 0.88249E+00_JPRB, 0.86957E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.84188E+00_JPRB, 0.83168E+00_JPRB, 0.81372E+00_JPRB, 0.79826E+00_JPRB, 0.79718E+00_JPRB, 0.78881E+00_JPRB, 0.77765E+00_JPRB ,&
&  0.80763E+00_JPRB, 0.78232E+00_JPRB, 0.78916E+00_JPRB, 0.71694E+00_JPRB, 0.68630E+00_JPRB, 0.67217E+00_JPRB, 0.65580E+00_JPRB ,&
&  0.65810E+00_JPRB, 0.67307E+00_JPRB, 0.68780E+00_JPRB, 0.70241E+00_JPRB, 0.72513E+00_JPRB, 0.74258E+00_JPRB, 0.75529E+00_JPRB ,&
&  0.77241E+00_JPRB, 0.79611E+00_JPRB, 0.82988E+00_JPRB, 0.86226E+00_JPRB, 0.90438E+00_JPRB, 0.95734E+00_JPRB, 0.99231E+00_JPRB ,&
&  0.96797E+00_JPRB, 0.95585E+00_JPRB, 0.97265E+00_JPRB, 0.96981E+00_JPRB, 0.98111E+00_JPRB, 0.97295E+00_JPRB, 0.96597E+00_JPRB ,&
&  0.93850E+00_JPRB, 0.85971E+00_JPRB, 0.86863E+00_JPRB, 0.84285E+00_JPRB, 0.82278E+00_JPRB, 0.80479E+00_JPRB, 0.79068E+00_JPRB ,&
&  0.78381E+00_JPRB, 0.78584E+00_JPRB, 0.79594E+00_JPRB, 0.80629E+00_JPRB, 0.82124E+00_JPRB, 0.86916E+00_JPRB, 0.94807E+00_JPRB ,&
&  0.10773E+01_JPRB, 0.10091E+01_JPRB, 0.11870E+01_JPRB, 0.12884E+01_JPRB, 0.15998E+01_JPRB, 0.19144E+01_JPRB, 0.20910E+01_JPRB ,&
&  0.24340E+01_JPRB, 0.24802E+01_JPRB, 0.24483E+01_JPRB, 0.24090E+01_JPRB, 0.24149E+01_JPRB, 0.23643E+01_JPRB, 0.22731E+01_JPRB ,&
&  0.21979E+01_JPRB, 0.20660E+01_JPRB, 0.21450E+01_JPRB, 0.21064E+01_JPRB, 0.20264E+01_JPRB, 0.19697E+01_JPRB, 0.18896E+01_JPRB ,&
&  0.18241E+01_JPRB, 0.17548E+01_JPRB, 0.16888E+01_JPRB, 0.15326E+01_JPRB, 0.14847E+01_JPRB, 0.14369E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.12243E+01_JPRB, 0.11621E+01_JPRB, 0.11827E+01_JPRB, 0.11413E+01_JPRB, 0.11195E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.10635E+01_JPRB, 0.99776E+00_JPRB, 0.99028E+00_JPRB, 0.98769E+00_JPRB, 0.96880E+00_JPRB, 0.94963E+00_JPRB, 0.96786E+00_JPRB ,&
&  0.93353E+00_JPRB, 0.92178E+00_JPRB, 0.93045E+00_JPRB, 0.89983E+00_JPRB, 0.87009E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.86215E+00_JPRB, 0.85105E+00_JPRB, 0.83067E+00_JPRB, 0.82203E+00_JPRB, 0.82510E+00_JPRB, 0.82701E+00_JPRB, 0.82626E+00_JPRB ,&
&  0.84505E+00_JPRB, 0.84936E+00_JPRB, 0.84475E+00_JPRB, 0.82644E+00_JPRB, 0.79822E+00_JPRB, 0.73562E+00_JPRB, 0.66352E+00_JPRB ,&
&  0.65637E+00_JPRB, 0.68609E+00_JPRB, 0.71798E+00_JPRB, 0.73453E+00_JPRB, 0.75648E+00_JPRB, 0.77821E+00_JPRB, 0.79630E+00_JPRB ,&
&  0.81062E+00_JPRB, 0.82259E+00_JPRB, 0.84808E+00_JPRB, 0.88552E+00_JPRB, 0.92396E+00_JPRB, 0.96531E+00_JPRB, 0.10052E+01_JPRB ,&
&  0.96407E+00_JPRB, 0.94578E+00_JPRB, 0.95141E+00_JPRB, 0.92780E+00_JPRB, 0.10035E+01_JPRB, 0.10153E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.99961E+00_JPRB, 0.97130E+00_JPRB, 0.93245E+00_JPRB, 0.90179E+00_JPRB, 0.87638E+00_JPRB, 0.85974E+00_JPRB, 0.84006E+00_JPRB ,&
&  0.82991E+00_JPRB, 0.83193E+00_JPRB, 0.85019E+00_JPRB, 0.87279E+00_JPRB, 0.91236E+00_JPRB, 0.10222E+01_JPRB, 0.11746E+01_JPRB ,&
&  0.13163E+01_JPRB, 0.14382E+01_JPRB, 0.15177E+01_JPRB, 0.15507E+01_JPRB, 0.18134E+01_JPRB, 0.20994E+01_JPRB, 0.22580E+01_JPRB ,&
&  0.24878E+01_JPRB, 0.24666E+01_JPRB, 0.25737E+01_JPRB, 0.26752E+01_JPRB, 0.27702E+01_JPRB, 0.27075E+01_JPRB, 0.25082E+01_JPRB ,&
&  0.23927E+01_JPRB, 0.24034E+01_JPRB, 0.23767E+01_JPRB, 0.23449E+01_JPRB, 0.22452E+01_JPRB, 0.21798E+01_JPRB, 0.20958E+01_JPRB ,&
&  0.20800E+01_JPRB, 0.20703E+01_JPRB, 0.19990E+01_JPRB, 0.19072E+01_JPRB, 0.17925E+01_JPRB, 0.16508E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.11921E+01_JPRB, 0.11254E+01_JPRB, 0.11228E+01_JPRB, 0.11418E+01_JPRB, 0.11050E+01_JPRB, 0.10946E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.10644E+01_JPRB, 0.10911E+01_JPRB, 0.10596E+01_JPRB, 0.10215E+01_JPRB, 0.98445E+00_JPRB, 0.95913E+00_JPRB ,&
&  0.90959E+00_JPRB, 0.92329E+00_JPRB, 0.92060E+00_JPRB, 0.90173E+00_JPRB, 0.87719E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.88053E+00_JPRB, 0.85558E+00_JPRB, 0.83754E+00_JPRB, 0.83777E+00_JPRB, 0.84559E+00_JPRB, 0.85677E+00_JPRB, 0.87551E+00_JPRB ,&
&  0.89529E+00_JPRB, 0.90174E+00_JPRB, 0.88398E+00_JPRB, 0.85541E+00_JPRB, 0.83048E+00_JPRB, 0.81255E+00_JPRB, 0.77428E+00_JPRB ,&
&  0.72790E+00_JPRB, 0.77289E+00_JPRB, 0.76507E+00_JPRB, 0.78147E+00_JPRB, 0.82707E+00_JPRB, 0.84644E+00_JPRB, 0.86833E+00_JPRB ,&
&  0.90270E+00_JPRB, 0.91306E+00_JPRB, 0.92842E+00_JPRB, 0.94946E+00_JPRB, 0.98173E+00_JPRB, 0.10564E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.10584E+01_JPRB, 0.10570E+01_JPRB, 0.10909E+01_JPRB, 0.11396E+01_JPRB, 0.11666E+01_JPRB, 0.11499E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.10754E+01_JPRB, 0.10359E+01_JPRB, 0.10020E+01_JPRB, 0.96195E+00_JPRB, 0.92124E+00_JPRB, 0.89453E+00_JPRB ,&
&  0.88034E+00_JPRB, 0.88351E+00_JPRB, 0.90638E+00_JPRB, 0.92828E+00_JPRB, 0.97633E+00_JPRB, 0.11856E+01_JPRB, 0.15500E+01_JPRB ,&
&  0.18109E+01_JPRB, 0.20391E+01_JPRB, 0.21521E+01_JPRB, 0.22703E+01_JPRB, 0.23919E+01_JPRB, 0.25139E+01_JPRB, 0.26223E+01_JPRB ,&
&  0.27288E+01_JPRB, 0.26274E+01_JPRB, 0.28645E+01_JPRB, 0.29508E+01_JPRB, 0.29480E+01_JPRB, 0.29598E+01_JPRB, 0.27409E+01_JPRB ,&
&  0.25621E+01_JPRB, 0.24029E+01_JPRB, 0.24688E+01_JPRB, 0.24664E+01_JPRB, 0.24128E+01_JPRB, 0.24371E+01_JPRB, 0.23722E+01_JPRB ,&
&  0.23284E+01_JPRB, 0.22609E+01_JPRB, 0.19996E+01_JPRB, 0.17985E+01_JPRB, 0.15509E+01_JPRB, 0.16223E+01_JPRB, 0.15056E+01_JPRB ,&
&  0.13536E+01_JPRB, 0.11586E+01_JPRB, 0.11625E+01_JPRB, 0.11633E+01_JPRB, 0.12081E+01_JPRB, 0.12482E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.12308E+01_JPRB, 0.11802E+01_JPRB, 0.11404E+01_JPRB, 0.11002E+01_JPRB, 0.10454E+01_JPRB, 0.99795E+00_JPRB ,&
&  0.99416E+00_JPRB, 0.99159E+00_JPRB, 0.96236E+00_JPRB, 0.92688E+00_JPRB, 0.90279E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.92289E+00_JPRB, 0.89790E+00_JPRB, 0.88136E+00_JPRB, 0.88053E+00_JPRB, 0.88689E+00_JPRB, 0.90371E+00_JPRB, 0.92418E+00_JPRB ,&
&  0.93869E+00_JPRB, 0.93403E+00_JPRB, 0.90340E+00_JPRB, 0.87620E+00_JPRB, 0.85763E+00_JPRB, 0.84078E+00_JPRB, 0.84487E+00_JPRB ,&
&  0.78128E+00_JPRB, 0.83516E+00_JPRB, 0.78533E+00_JPRB, 0.81482E+00_JPRB, 0.89488E+00_JPRB, 0.92203E+00_JPRB, 0.94750E+00_JPRB ,&
&  0.96149E+00_JPRB, 0.99473E+00_JPRB, 0.10049E+01_JPRB, 0.10256E+01_JPRB, 0.10714E+01_JPRB, 0.11779E+01_JPRB, 0.13013E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.12239E+01_JPRB, 0.12730E+01_JPRB, 0.13405E+01_JPRB, 0.13306E+01_JPRB, 0.13754E+01_JPRB, 0.14021E+01_JPRB ,&
&  0.13429E+01_JPRB, 0.12571E+01_JPRB, 0.11642E+01_JPRB, 0.10804E+01_JPRB, 0.10169E+01_JPRB, 0.96806E+00_JPRB, 0.94336E+00_JPRB ,&
&  0.93442E+00_JPRB, 0.93791E+00_JPRB, 0.94863E+00_JPRB, 0.10007E+01_JPRB, 0.10546E+01_JPRB, 0.13313E+01_JPRB, 0.18169E+01_JPRB ,&
&  0.23505E+01_JPRB, 0.24817E+01_JPRB, 0.27470E+01_JPRB, 0.32527E+01_JPRB, 0.33326E+01_JPRB, 0.33152E+01_JPRB, 0.32246E+01_JPRB ,&
&  0.31335E+01_JPRB, 0.29539E+01_JPRB, 0.28797E+01_JPRB, 0.28999E+01_JPRB, 0.28299E+01_JPRB, 0.29047E+01_JPRB, 0.27690E+01_JPRB ,&
&  0.25343E+01_JPRB, 0.24586E+01_JPRB, 0.24546E+01_JPRB, 0.24169E+01_JPRB, 0.23477E+01_JPRB, 0.23826E+01_JPRB, 0.23373E+01_JPRB ,&
&  0.22278E+01_JPRB, 0.19731E+01_JPRB, 0.15622E+01_JPRB, 0.14932E+01_JPRB, 0.12316E+01_JPRB, 0.12941E+01_JPRB, 0.15529E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.14649E+01_JPRB, 0.13601E+01_JPRB, 0.13261E+01_JPRB, 0.13485E+01_JPRB, 0.15047E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.13659E+01_JPRB, 0.13765E+01_JPRB, 0.13587E+01_JPRB, 0.13133E+01_JPRB, 0.12485E+01_JPRB, 0.11890E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11143E+01_JPRB, 0.10935E+01_JPRB, 0.10578E+01_JPRB, 0.99254E+00_JPRB, 0.94651E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.10019E+01_JPRB, 0.97326E+00_JPRB, 0.94763E+00_JPRB, 0.94293E+00_JPRB, 0.94890E+00_JPRB, 0.96285E+00_JPRB, 0.97648E+00_JPRB ,&
&  0.97708E+00_JPRB, 0.95644E+00_JPRB, 0.91845E+00_JPRB, 0.89801E+00_JPRB, 0.88090E+00_JPRB, 0.85964E+00_JPRB, 0.85695E+00_JPRB ,&
&  0.88447E+00_JPRB, 0.97703E+00_JPRB, 0.91431E+00_JPRB, 0.84067E+00_JPRB, 0.86927E+00_JPRB, 0.88997E+00_JPRB, 0.95077E+00_JPRB ,&
&  0.98159E+00_JPRB, 0.10575E+01_JPRB, 0.10954E+01_JPRB, 0.11278E+01_JPRB, 0.12002E+01_JPRB, 0.13433E+01_JPRB, 0.14722E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.15635E+01_JPRB, 0.16668E+01_JPRB, 0.17708E+01_JPRB, 0.17773E+01_JPRB, 0.16216E+01_JPRB, 0.16138E+01_JPRB ,&
&  0.15061E+01_JPRB, 0.13702E+01_JPRB, 0.12299E+01_JPRB, 0.11337E+01_JPRB, 0.10772E+01_JPRB, 0.10472E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10364E+01_JPRB, 0.10139E+01_JPRB, 0.10214E+01_JPRB, 0.10787E+01_JPRB, 0.13328E+01_JPRB, 0.17114E+01_JPRB ,&
&  0.22725E+01_JPRB, 0.24063E+01_JPRB, 0.25680E+01_JPRB, 0.30036E+01_JPRB, 0.32333E+01_JPRB, 0.33614E+01_JPRB, 0.34227E+01_JPRB ,&
&  0.33536E+01_JPRB, 0.32260E+01_JPRB, 0.30671E+01_JPRB, 0.28679E+01_JPRB, 0.28177E+01_JPRB, 0.28248E+01_JPRB, 0.27339E+01_JPRB ,&
&  0.25180E+01_JPRB, 0.23799E+01_JPRB, 0.23506E+01_JPRB, 0.22365E+01_JPRB, 0.21467E+01_JPRB, 0.19735E+01_JPRB, 0.19831E+01_JPRB ,&
&  0.20102E+01_JPRB, 0.13268E+01_JPRB, 0.11359E+01_JPRB, 0.13221E+01_JPRB, 0.11672E+01_JPRB, 0.12192E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.14192E+01_JPRB, 0.14061E+01_JPRB, 0.14632E+01_JPRB, 0.14398E+01_JPRB, 0.15242E+01_JPRB, 0.16937E+01_JPRB, 0.16773E+01_JPRB ,&
&  0.16172E+01_JPRB, 0.15257E+01_JPRB, 0.15400E+01_JPRB, 0.14944E+01_JPRB, 0.14212E+01_JPRB, 0.13534E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.12306E+01_JPRB, 0.11867E+01_JPRB, 0.11597E+01_JPRB, 0.10938E+01_JPRB, 0.10351E+01_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.11129E+01_JPRB, 0.10781E+01_JPRB, 0.10525E+01_JPRB, 0.10410E+01_JPRB, 0.10372E+01_JPRB, 0.10424E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.97437E+00_JPRB, 0.93084E+00_JPRB, 0.91998E+00_JPRB, 0.89886E+00_JPRB, 0.87644E+00_JPRB, 0.87649E+00_JPRB ,&
&  0.89872E+00_JPRB, 0.98628E+00_JPRB, 0.87441E+00_JPRB, 0.82378E+00_JPRB, 0.77927E+00_JPRB, 0.82110E+00_JPRB, 0.87299E+00_JPRB ,&
&  0.97915E+00_JPRB, 0.10386E+01_JPRB, 0.10789E+01_JPRB, 0.11481E+01_JPRB, 0.12966E+01_JPRB, 0.14693E+01_JPRB, 0.16638E+01_JPRB ,&
&  0.17604E+01_JPRB, 0.19333E+01_JPRB, 0.19512E+01_JPRB, 0.21661E+01_JPRB, 0.20811E+01_JPRB, 0.18425E+01_JPRB, 0.18164E+01_JPRB ,&
&  0.16495E+01_JPRB, 0.14909E+01_JPRB, 0.13495E+01_JPRB, 0.12668E+01_JPRB, 0.12066E+01_JPRB, 0.11514E+01_JPRB, 0.11002E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.10615E+01_JPRB, 0.10539E+01_JPRB, 0.10459E+01_JPRB, 0.11220E+01_JPRB, 0.13384E+01_JPRB, 0.15890E+01_JPRB ,&
&  0.20245E+01_JPRB, 0.18334E+01_JPRB, 0.16460E+01_JPRB, 0.21767E+01_JPRB, 0.28131E+01_JPRB, 0.28813E+01_JPRB, 0.27988E+01_JPRB ,&
&  0.30378E+01_JPRB, 0.30586E+01_JPRB, 0.30225E+01_JPRB, 0.30361E+01_JPRB, 0.29728E+01_JPRB, 0.28704E+01_JPRB, 0.27191E+01_JPRB ,&
&  0.24378E+01_JPRB, 0.22860E+01_JPRB, 0.21839E+01_JPRB, 0.21666E+01_JPRB, 0.20383E+01_JPRB, 0.19569E+01_JPRB, 0.19195E+01_JPRB ,&
&  0.17411E+01_JPRB, 0.16232E+01_JPRB, 0.12536E+01_JPRB, 0.12983E+01_JPRB, 0.10940E+01_JPRB, 0.11144E+01_JPRB, 0.12903E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.13814E+01_JPRB, 0.14194E+01_JPRB, 0.14063E+01_JPRB, 0.17214E+01_JPRB, 0.19259E+01_JPRB, 0.19470E+01_JPRB ,&
&  0.18798E+01_JPRB, 0.16518E+01_JPRB, 0.16986E+01_JPRB, 0.17362E+01_JPRB, 0.17009E+01_JPRB, 0.16373E+01_JPRB, 0.15596E+01_JPRB ,&
&  0.14660E+01_JPRB, 0.13810E+01_JPRB, 0.13203E+01_JPRB, 0.12378E+01_JPRB, 0.11643E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.12373E+01_JPRB, 0.11827E+01_JPRB, 0.11456E+01_JPRB, 0.11254E+01_JPRB, 0.11153E+01_JPRB, 0.11040E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.98812E+00_JPRB, 0.95521E+00_JPRB, 0.94109E+00_JPRB, 0.91263E+00_JPRB, 0.89231E+00_JPRB, 0.90038E+00_JPRB ,&
&  0.92844E+00_JPRB, 0.93113E+00_JPRB, 0.80564E+00_JPRB, 0.73397E+00_JPRB, 0.70253E+00_JPRB, 0.74912E+00_JPRB, 0.81433E+00_JPRB ,&
&  0.90503E+00_JPRB, 0.98029E+00_JPRB, 0.10791E+01_JPRB, 0.12030E+01_JPRB, 0.13611E+01_JPRB, 0.15445E+01_JPRB, 0.18581E+01_JPRB ,&
&  0.23455E+01_JPRB, 0.21817E+01_JPRB, 0.23169E+01_JPRB, 0.22786E+01_JPRB, 0.22254E+01_JPRB, 0.20351E+01_JPRB, 0.18415E+01_JPRB ,&
&  0.16593E+01_JPRB, 0.15295E+01_JPRB, 0.13976E+01_JPRB, 0.12876E+01_JPRB, 0.11958E+01_JPRB, 0.11244E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.11130E+01_JPRB, 0.11251E+01_JPRB, 0.11763E+01_JPRB, 0.13561E+01_JPRB, 0.15228E+01_JPRB, 0.16781E+01_JPRB ,&
&  0.16871E+01_JPRB, 0.17350E+01_JPRB, 0.23086E+01_JPRB, 0.24581E+01_JPRB, 0.25899E+01_JPRB, 0.27322E+01_JPRB, 0.28008E+01_JPRB ,&
&  0.29252E+01_JPRB, 0.27901E+01_JPRB, 0.27390E+01_JPRB, 0.24266E+01_JPRB, 0.27286E+01_JPRB, 0.26488E+01_JPRB, 0.24900E+01_JPRB ,&
&  0.22913E+01_JPRB, 0.22325E+01_JPRB, 0.20869E+01_JPRB, 0.20696E+01_JPRB, 0.20518E+01_JPRB, 0.20812E+01_JPRB, 0.16920E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.16174E+01_JPRB, 0.14757E+01_JPRB, 0.13190E+01_JPRB, 0.11876E+01_JPRB, 0.12356E+01_JPRB, 0.12867E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.13398E+01_JPRB, 0.13252E+01_JPRB, 0.14253E+01_JPRB, 0.18271E+01_JPRB, 0.22255E+01_JPRB, 0.22431E+01_JPRB ,&
&  0.20091E+01_JPRB, 0.20592E+01_JPRB, 0.21001E+01_JPRB, 0.20932E+01_JPRB, 0.20921E+01_JPRB, 0.20166E+01_JPRB, 0.19072E+01_JPRB ,&
&  0.17794E+01_JPRB, 0.16570E+01_JPRB, 0.15460E+01_JPRB, 0.14307E+01_JPRB, 0.13161E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.13711E+01_JPRB, 0.13114E+01_JPRB, 0.12593E+01_JPRB, 0.12222E+01_JPRB, 0.11892E+01_JPRB, 0.11630E+01_JPRB, 0.11304E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.10044E+01_JPRB, 0.98728E+00_JPRB, 0.96837E+00_JPRB, 0.94299E+00_JPRB, 0.92590E+00_JPRB, 0.93653E+00_JPRB ,&
&  0.96643E+00_JPRB, 0.91031E+00_JPRB, 0.77860E+00_JPRB, 0.78479E+00_JPRB, 0.81306E+00_JPRB, 0.77751E+00_JPRB, 0.81360E+00_JPRB ,&
&  0.87853E+00_JPRB, 0.96198E+00_JPRB, 0.10539E+01_JPRB, 0.12707E+01_JPRB, 0.15271E+01_JPRB, 0.18554E+01_JPRB, 0.21977E+01_JPRB ,&
&  0.23067E+01_JPRB, 0.24743E+01_JPRB, 0.25996E+01_JPRB, 0.24476E+01_JPRB, 0.22125E+01_JPRB, 0.19597E+01_JPRB, 0.17557E+01_JPRB ,&
&  0.16079E+01_JPRB, 0.14641E+01_JPRB, 0.13195E+01_JPRB, 0.12079E+01_JPRB, 0.11320E+01_JPRB, 0.10861E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.10900E+01_JPRB, 0.11156E+01_JPRB, 0.12517E+01_JPRB, 0.13095E+01_JPRB, 0.14219E+01_JPRB, 0.14973E+01_JPRB ,&
&  0.19349E+01_JPRB, 0.21196E+01_JPRB, 0.22149E+01_JPRB, 0.25612E+01_JPRB, 0.20984E+01_JPRB, 0.22456E+01_JPRB, 0.27582E+01_JPRB ,&
&  0.28612E+01_JPRB, 0.27865E+01_JPRB, 0.28342E+01_JPRB, 0.22733E+01_JPRB, 0.20109E+01_JPRB, 0.26511E+01_JPRB, 0.25226E+01_JPRB ,&
&  0.22975E+01_JPRB, 0.22238E+01_JPRB, 0.21256E+01_JPRB, 0.23036E+01_JPRB, 0.16941E+01_JPRB, 0.14783E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.11964E+01_JPRB, 0.13756E+01_JPRB, 0.13250E+01_JPRB, 0.13683E+01_JPRB, 0.13576E+01_JPRB, 0.12788E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.12985E+01_JPRB, 0.13505E+01_JPRB, 0.16917E+01_JPRB, 0.24518E+01_JPRB, 0.23977E+01_JPRB, 0.25975E+01_JPRB ,&
&  0.26072E+01_JPRB, 0.27337E+01_JPRB, 0.25485E+01_JPRB, 0.25465E+01_JPRB, 0.24669E+01_JPRB, 0.23334E+01_JPRB, 0.21459E+01_JPRB ,&
&  0.19959E+01_JPRB, 0.18566E+01_JPRB, 0.17063E+01_JPRB, 0.15699E+01_JPRB, 0.14573E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.14675E+01_JPRB, 0.13931E+01_JPRB, 0.13296E+01_JPRB, 0.12733E+01_JPRB, 0.12373E+01_JPRB, 0.12011E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.99885E+00_JPRB, 0.99145E+00_JPRB, 0.95314E+00_JPRB, 0.93091E+00_JPRB, 0.93223E+00_JPRB, 0.96709E+00_JPRB ,&
&  0.10008E+01_JPRB, 0.10184E+01_JPRB, 0.88300E+00_JPRB, 0.81861E+00_JPRB, 0.89600E+00_JPRB, 0.84238E+00_JPRB, 0.83341E+00_JPRB ,&
&  0.91269E+00_JPRB, 0.10029E+01_JPRB, 0.11749E+01_JPRB, 0.15571E+01_JPRB, 0.19349E+01_JPRB, 0.23366E+01_JPRB, 0.26532E+01_JPRB ,&
&  0.27857E+01_JPRB, 0.27464E+01_JPRB, 0.25952E+01_JPRB, 0.23857E+01_JPRB, 0.20422E+01_JPRB, 0.17616E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.14072E+01_JPRB, 0.12853E+01_JPRB, 0.12029E+01_JPRB, 0.11216E+01_JPRB, 0.10693E+01_JPRB, 0.10093E+01_JPRB, 0.99451E+00_JPRB ,&
&  0.10128E+01_JPRB, 0.10556E+01_JPRB, 0.11176E+01_JPRB, 0.12637E+01_JPRB, 0.13073E+01_JPRB, 0.13456E+01_JPRB, 0.14763E+01_JPRB ,&
&  0.17362E+01_JPRB, 0.18437E+01_JPRB, 0.20176E+01_JPRB, 0.22640E+01_JPRB, 0.26458E+01_JPRB, 0.26868E+01_JPRB, 0.29148E+01_JPRB ,&
&  0.23193E+01_JPRB, 0.20199E+01_JPRB, 0.18785E+01_JPRB, 0.16569E+01_JPRB, 0.20290E+01_JPRB, 0.28824E+01_JPRB, 0.28247E+01_JPRB ,&
&  0.25770E+01_JPRB, 0.24800E+01_JPRB, 0.24484E+01_JPRB, 0.20017E+01_JPRB, 0.15736E+01_JPRB, 0.12764E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.16319E+01_JPRB, 0.17074E+01_JPRB, 0.15683E+01_JPRB, 0.13783E+01_JPRB, 0.13218E+01_JPRB, 0.16370E+01_JPRB, 0.16566E+01_JPRB ,&
&  0.15330E+01_JPRB, 0.14909E+01_JPRB, 0.21039E+01_JPRB, 0.29299E+01_JPRB, 0.33003E+01_JPRB, 0.32007E+01_JPRB, 0.36105E+01_JPRB ,&
&  0.36146E+01_JPRB, 0.36551E+01_JPRB, 0.34815E+01_JPRB, 0.32275E+01_JPRB, 0.29278E+01_JPRB, 0.26066E+01_JPRB, 0.23541E+01_JPRB ,&
&  0.21560E+01_JPRB, 0.19726E+01_JPRB, 0.18035E+01_JPRB, 0.16588E+01_JPRB, 0.15522E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.15482E+01_JPRB, 0.14581E+01_JPRB, 0.13796E+01_JPRB, 0.13259E+01_JPRB, 0.12725E+01_JPRB, 0.12076E+01_JPRB, 0.11282E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.99472E+00_JPRB, 0.98130E+00_JPRB, 0.94230E+00_JPRB, 0.93201E+00_JPRB, 0.94476E+00_JPRB, 0.98042E+00_JPRB ,&
&  0.10286E+01_JPRB, 0.11662E+01_JPRB, 0.10066E+01_JPRB, 0.92240E+00_JPRB, 0.93479E+00_JPRB, 0.92261E+00_JPRB, 0.85465E+00_JPRB ,&
&  0.95217E+00_JPRB, 0.10750E+01_JPRB, 0.12787E+01_JPRB, 0.17079E+01_JPRB, 0.23838E+01_JPRB, 0.25314E+01_JPRB, 0.26863E+01_JPRB ,&
&  0.27810E+01_JPRB, 0.25681E+01_JPRB, 0.23612E+01_JPRB, 0.21410E+01_JPRB, 0.18285E+01_JPRB, 0.15931E+01_JPRB, 0.14296E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.12475E+01_JPRB, 0.11965E+01_JPRB, 0.11600E+01_JPRB, 0.10839E+01_JPRB, 0.10306E+01_JPRB, 0.10126E+01_JPRB ,&
&  0.10144E+01_JPRB, 0.10545E+01_JPRB, 0.11108E+01_JPRB, 0.12438E+01_JPRB, 0.14201E+01_JPRB, 0.14079E+01_JPRB, 0.16911E+01_JPRB ,&
&  0.17461E+01_JPRB, 0.18754E+01_JPRB, 0.19280E+01_JPRB, 0.22296E+01_JPRB, 0.24512E+01_JPRB, 0.24741E+01_JPRB, 0.26132E+01_JPRB ,&
&  0.22755E+01_JPRB, 0.23116E+01_JPRB, 0.26087E+01_JPRB, 0.28511E+01_JPRB, 0.23641E+01_JPRB, 0.23590E+01_JPRB, 0.32218E+01_JPRB ,&
&  0.23577E+01_JPRB, 0.24161E+01_JPRB, 0.25464E+01_JPRB, 0.25253E+01_JPRB, 0.11951E+01_JPRB, 0.10012E+01_JPRB, 0.14481E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.10727E+01_JPRB, 0.89165E+00_JPRB, 0.10509E+01_JPRB, 0.93098E+00_JPRB, 0.12107E+01_JPRB, 0.18973E+01_JPRB ,&
&  0.20737E+01_JPRB, 0.22157E+01_JPRB, 0.34890E+01_JPRB, 0.40210E+01_JPRB, 0.38349E+01_JPRB, 0.42511E+01_JPRB, 0.43495E+01_JPRB ,&
&  0.42895E+01_JPRB, 0.38547E+01_JPRB, 0.38668E+01_JPRB, 0.35380E+01_JPRB, 0.31545E+01_JPRB, 0.27951E+01_JPRB, 0.24925E+01_JPRB ,&
&  0.22147E+01_JPRB, 0.20155E+01_JPRB, 0.18779E+01_JPRB, 0.17392E+01_JPRB, 0.16293E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.14929E+01_JPRB, 0.13989E+01_JPRB, 0.12986E+01_JPRB, 0.12314E+01_JPRB, 0.11891E+01_JPRB, 0.11349E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.98758E+00_JPRB, 0.99707E+00_JPRB, 0.96426E+00_JPRB, 0.93802E+00_JPRB, 0.92163E+00_JPRB, 0.93672E+00_JPRB, 0.98554E+00_JPRB ,&
&  0.10285E+01_JPRB, 0.11000E+01_JPRB, 0.11304E+01_JPRB, 0.10318E+01_JPRB, 0.99740E+00_JPRB, 0.95562E+00_JPRB, 0.94688E+00_JPRB ,&
&  0.10666E+01_JPRB, 0.12227E+01_JPRB, 0.14665E+01_JPRB, 0.18364E+01_JPRB, 0.22187E+01_JPRB, 0.23639E+01_JPRB, 0.25814E+01_JPRB ,&
&  0.24624E+01_JPRB, 0.22516E+01_JPRB, 0.20979E+01_JPRB, 0.18852E+01_JPRB, 0.16193E+01_JPRB, 0.14415E+01_JPRB, 0.13144E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.11953E+01_JPRB, 0.11756E+01_JPRB, 0.10941E+01_JPRB, 0.10262E+01_JPRB, 0.99825E+00_JPRB, 0.97937E+00_JPRB ,&
&  0.98966E+00_JPRB, 0.10461E+01_JPRB, 0.11092E+01_JPRB, 0.11926E+01_JPRB, 0.13254E+01_JPRB, 0.13850E+01_JPRB, 0.13534E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.14901E+01_JPRB, 0.18793E+01_JPRB, 0.19975E+01_JPRB, 0.20183E+01_JPRB, 0.20726E+01_JPRB, 0.20418E+01_JPRB ,&
&  0.21655E+01_JPRB, 0.24956E+01_JPRB, 0.23175E+01_JPRB, 0.26475E+01_JPRB, 0.26341E+01_JPRB, 0.20551E+01_JPRB, 0.22645E+01_JPRB ,&
&  0.23202E+01_JPRB, 0.20913E+01_JPRB, 0.17270E+01_JPRB, 0.13728E+01_JPRB, 0.14818E+01_JPRB, 0.13826E+01_JPRB, 0.95695E+00_JPRB ,&
&  0.87963E+00_JPRB, 0.84181E+00_JPRB, 0.81296E+00_JPRB, 0.82494E+00_JPRB, 0.84973E+00_JPRB, 0.12133E+01_JPRB, 0.17968E+01_JPRB ,&
&  0.28010E+01_JPRB, 0.37128E+01_JPRB, 0.45415E+01_JPRB, 0.46649E+01_JPRB, 0.40699E+01_JPRB, 0.42826E+01_JPRB, 0.45624E+01_JPRB ,&
&  0.38685E+01_JPRB, 0.35949E+01_JPRB, 0.35343E+01_JPRB, 0.32127E+01_JPRB, 0.28932E+01_JPRB, 0.26197E+01_JPRB, 0.23564E+01_JPRB ,&
&  0.21299E+01_JPRB, 0.19671E+01_JPRB, 0.18402E+01_JPRB, 0.17154E+01_JPRB, 0.15968E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.13581E+01_JPRB, 0.12912E+01_JPRB, 0.12445E+01_JPRB, 0.12052E+01_JPRB, 0.11541E+01_JPRB, 0.11075E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.99455E+00_JPRB, 0.98941E+00_JPRB, 0.94266E+00_JPRB, 0.91121E+00_JPRB, 0.89727E+00_JPRB, 0.91776E+00_JPRB, 0.96899E+00_JPRB ,&
&  0.10106E+01_JPRB, 0.10575E+01_JPRB, 0.11948E+01_JPRB, 0.11578E+01_JPRB, 0.12216E+01_JPRB, 0.11221E+01_JPRB, 0.11789E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.14679E+01_JPRB, 0.18161E+01_JPRB, 0.20331E+01_JPRB, 0.22819E+01_JPRB, 0.24101E+01_JPRB, 0.23866E+01_JPRB ,&
&  0.22417E+01_JPRB, 0.21031E+01_JPRB, 0.18622E+01_JPRB, 0.15866E+01_JPRB, 0.13883E+01_JPRB, 0.12542E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11399E+01_JPRB, 0.11436E+01_JPRB, 0.10786E+01_JPRB, 0.10335E+01_JPRB, 0.10175E+01_JPRB, 0.10081E+01_JPRB, 0.99565E+00_JPRB ,&
&  0.10172E+01_JPRB, 0.10654E+01_JPRB, 0.11286E+01_JPRB, 0.11859E+01_JPRB, 0.11182E+01_JPRB, 0.10455E+01_JPRB, 0.12287E+01_JPRB ,&
&  0.14106E+01_JPRB, 0.16373E+01_JPRB, 0.16930E+01_JPRB, 0.19702E+01_JPRB, 0.18998E+01_JPRB, 0.18038E+01_JPRB, 0.19138E+01_JPRB ,&
&  0.20812E+01_JPRB, 0.24396E+01_JPRB, 0.20963E+01_JPRB, 0.23195E+01_JPRB, 0.28139E+01_JPRB, 0.22193E+01_JPRB, 0.19250E+01_JPRB ,&
&  0.21093E+01_JPRB, 0.18608E+01_JPRB, 0.16920E+01_JPRB, 0.15725E+01_JPRB, 0.22352E+01_JPRB, 0.27857E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.98532E+00_JPRB, 0.95018E+00_JPRB, 0.89480E+00_JPRB, 0.95423E+00_JPRB, 0.14019E+01_JPRB, 0.25139E+01_JPRB ,&
&  0.32694E+01_JPRB, 0.48047E+01_JPRB, 0.53948E+01_JPRB, 0.52600E+01_JPRB, 0.41515E+01_JPRB, 0.40668E+01_JPRB, 0.37046E+01_JPRB ,&
&  0.38301E+01_JPRB, 0.36809E+01_JPRB, 0.33773E+01_JPRB, 0.30509E+01_JPRB, 0.27335E+01_JPRB, 0.24494E+01_JPRB, 0.21957E+01_JPRB ,&
&  0.19770E+01_JPRB, 0.18078E+01_JPRB, 0.16700E+01_JPRB, 0.15570E+01_JPRB, 0.14462E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.13321E+01_JPRB, 0.12719E+01_JPRB, 0.12200E+01_JPRB, 0.11653E+01_JPRB, 0.11273E+01_JPRB, 0.10755E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.10130E+01_JPRB, 0.97101E+00_JPRB, 0.92391E+00_JPRB, 0.89911E+00_JPRB, 0.89559E+00_JPRB, 0.91226E+00_JPRB, 0.94696E+00_JPRB ,&
&  0.99915E+00_JPRB, 0.10269E+01_JPRB, 0.11251E+01_JPRB, 0.13049E+01_JPRB, 0.13805E+01_JPRB, 0.12934E+01_JPRB, 0.14409E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.17575E+01_JPRB, 0.21235E+01_JPRB, 0.22468E+01_JPRB, 0.22712E+01_JPRB, 0.23760E+01_JPRB, 0.22142E+01_JPRB ,&
&  0.20443E+01_JPRB, 0.18245E+01_JPRB, 0.15427E+01_JPRB, 0.13437E+01_JPRB, 0.12122E+01_JPRB, 0.11674E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.11157E+01_JPRB, 0.10759E+01_JPRB, 0.10411E+01_JPRB, 0.10381E+01_JPRB, 0.10441E+01_JPRB, 0.10451E+01_JPRB, 0.10496E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.11147E+01_JPRB, 0.11783E+01_JPRB, 0.12247E+01_JPRB, 0.10633E+01_JPRB, 0.12302E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.14001E+01_JPRB, 0.14842E+01_JPRB, 0.14683E+01_JPRB, 0.16530E+01_JPRB, 0.17641E+01_JPRB, 0.17235E+01_JPRB, 0.17823E+01_JPRB ,&
&  0.20477E+01_JPRB, 0.19875E+01_JPRB, 0.19537E+01_JPRB, 0.22411E+01_JPRB, 0.24590E+01_JPRB, 0.29021E+01_JPRB, 0.17391E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.17760E+01_JPRB, 0.18525E+01_JPRB, 0.20118E+01_JPRB, 0.26275E+01_JPRB, 0.40770E+01_JPRB, 0.35320E+01_JPRB ,&
&  0.13409E+01_JPRB, 0.10530E+01_JPRB, 0.12075E+01_JPRB, 0.12244E+01_JPRB, 0.11503E+01_JPRB, 0.17382E+01_JPRB, 0.51570E+01_JPRB ,&
&  0.47610E+01_JPRB, 0.57426E+01_JPRB, 0.55261E+01_JPRB, 0.46147E+01_JPRB, 0.37795E+01_JPRB, 0.36132E+01_JPRB, 0.37413E+01_JPRB ,&
&  0.35508E+01_JPRB, 0.32446E+01_JPRB, 0.29678E+01_JPRB, 0.27283E+01_JPRB, 0.24325E+01_JPRB, 0.21288E+01_JPRB, 0.18611E+01_JPRB ,&
&  0.16701E+01_JPRB, 0.15606E+01_JPRB, 0.14998E+01_JPRB, 0.14457E+01_JPRB, 0.13890E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.12520E+01_JPRB, 0.11983E+01_JPRB, 0.11587E+01_JPRB, 0.11277E+01_JPRB, 0.11127E+01_JPRB, 0.10649E+01_JPRB, 0.10138E+01_JPRB ,&
&  0.10431E+01_JPRB, 0.97100E+00_JPRB, 0.93052E+00_JPRB, 0.90025E+00_JPRB, 0.90202E+00_JPRB, 0.92620E+00_JPRB, 0.95046E+00_JPRB ,&
&  0.97366E+00_JPRB, 0.10160E+01_JPRB, 0.11327E+01_JPRB, 0.13731E+01_JPRB, 0.15168E+01_JPRB, 0.14512E+01_JPRB, 0.15327E+01_JPRB ,&
&  0.18332E+01_JPRB, 0.20276E+01_JPRB, 0.20288E+01_JPRB, 0.20878E+01_JPRB, 0.22778E+01_JPRB, 0.23777E+01_JPRB, 0.20660E+01_JPRB ,&
&  0.17976E+01_JPRB, 0.15561E+01_JPRB, 0.13493E+01_JPRB, 0.12296E+01_JPRB, 0.11929E+01_JPRB, 0.11684E+01_JPRB, 0.11471E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.10862E+01_JPRB, 0.10871E+01_JPRB, 0.10912E+01_JPRB, 0.11066E+01_JPRB, 0.10953E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11938E+01_JPRB, 0.12397E+01_JPRB, 0.12107E+01_JPRB, 0.12428E+01_JPRB, 0.12804E+01_JPRB, 0.13534E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.14300E+01_JPRB, 0.15037E+01_JPRB, 0.15702E+01_JPRB, 0.16954E+01_JPRB, 0.17654E+01_JPRB, 0.18171E+01_JPRB ,&
&  0.19634E+01_JPRB, 0.20089E+01_JPRB, 0.19054E+01_JPRB, 0.20698E+01_JPRB, 0.23035E+01_JPRB, 0.25870E+01_JPRB, 0.21496E+01_JPRB ,&
&  0.18483E+01_JPRB, 0.17276E+01_JPRB, 0.21125E+01_JPRB, 0.29456E+01_JPRB, 0.35621E+01_JPRB, 0.47593E+01_JPRB, 0.63549E+01_JPRB ,&
&  0.70233E+01_JPRB, 0.36876E+01_JPRB, 0.28986E+01_JPRB, 0.38321E+01_JPRB, 0.30227E+01_JPRB, 0.28108E+01_JPRB, 0.43583E+01_JPRB ,&
&  0.51962E+01_JPRB, 0.56188E+01_JPRB, 0.50941E+01_JPRB, 0.38271E+01_JPRB, 0.34730E+01_JPRB, 0.35437E+01_JPRB, 0.34288E+01_JPRB ,&
&  0.30685E+01_JPRB, 0.27810E+01_JPRB, 0.24796E+01_JPRB, 0.21530E+01_JPRB, 0.18747E+01_JPRB, 0.16702E+01_JPRB, 0.15451E+01_JPRB ,&
&  0.14774E+01_JPRB, 0.14416E+01_JPRB, 0.14157E+01_JPRB, 0.13698E+01_JPRB, 0.13118E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.11689E+01_JPRB, 0.11461E+01_JPRB, 0.11441E+01_JPRB, 0.11379E+01_JPRB, 0.11162E+01_JPRB, 0.10677E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.10044E+01_JPRB, 0.94141E+00_JPRB, 0.91436E+00_JPRB, 0.92322E+00_JPRB, 0.95382E+00_JPRB, 0.97106E+00_JPRB ,&
&  0.99248E+00_JPRB, 0.10560E+01_JPRB, 0.12624E+01_JPRB, 0.14926E+01_JPRB, 0.17161E+01_JPRB, 0.21163E+01_JPRB, 0.15469E+01_JPRB ,&
&  0.16819E+01_JPRB, 0.20028E+01_JPRB, 0.19661E+01_JPRB, 0.21163E+01_JPRB, 0.22211E+01_JPRB, 0.21674E+01_JPRB, 0.18629E+01_JPRB ,&
&  0.15653E+01_JPRB, 0.14030E+01_JPRB, 0.13093E+01_JPRB, 0.12486E+01_JPRB, 0.12322E+01_JPRB, 0.12260E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.11795E+01_JPRB, 0.11789E+01_JPRB, 0.11588E+01_JPRB, 0.11692E+01_JPRB, 0.11769E+01_JPRB, 0.11698E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.12371E+01_JPRB, 0.12633E+01_JPRB, 0.12804E+01_JPRB, 0.12451E+01_JPRB, 0.12911E+01_JPRB, 0.13295E+01_JPRB, 0.13754E+01_JPRB ,&
&  0.13198E+01_JPRB, 0.12562E+01_JPRB, 0.14218E+01_JPRB, 0.16084E+01_JPRB, 0.17279E+01_JPRB, 0.17163E+01_JPRB, 0.17790E+01_JPRB ,&
&  0.20184E+01_JPRB, 0.21721E+01_JPRB, 0.22470E+01_JPRB, 0.20222E+01_JPRB, 0.21503E+01_JPRB, 0.24003E+01_JPRB, 0.26558E+01_JPRB ,&
&  0.28987E+01_JPRB, 0.29299E+01_JPRB, 0.33806E+01_JPRB, 0.38036E+01_JPRB, 0.42055E+01_JPRB, 0.53047E+01_JPRB, 0.55879E+01_JPRB ,&
&  0.66589E+01_JPRB, 0.80949E+01_JPRB, 0.71803E+01_JPRB, 0.58459E+01_JPRB, 0.53552E+01_JPRB, 0.48473E+01_JPRB, 0.49994E+01_JPRB ,&
&  0.56441E+01_JPRB, 0.50501E+01_JPRB, 0.41398E+01_JPRB, 0.39420E+01_JPRB, 0.33186E+01_JPRB, 0.31428E+01_JPRB, 0.28191E+01_JPRB ,&
&  0.23418E+01_JPRB, 0.19903E+01_JPRB, 0.16613E+01_JPRB, 0.14559E+01_JPRB, 0.13506E+01_JPRB, 0.13228E+01_JPRB, 0.13148E+01_JPRB ,&
&  0.13161E+01_JPRB, 0.13074E+01_JPRB, 0.12909E+01_JPRB, 0.12617E+01_JPRB, 0.12145E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.11481E+01_JPRB, 0.11441E+01_JPRB, 0.11621E+01_JPRB, 0.11673E+01_JPRB, 0.11442E+01_JPRB, 0.11273E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.11488E+01_JPRB, 0.10079E+01_JPRB, 0.98505E+00_JPRB, 0.98846E+00_JPRB, 0.10072E+01_JPRB, 0.10138E+01_JPRB, 0.10193E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.11987E+01_JPRB, 0.14312E+01_JPRB, 0.16090E+01_JPRB, 0.17805E+01_JPRB, 0.20424E+01_JPRB, 0.19761E+01_JPRB ,&
&  0.15719E+01_JPRB, 0.22197E+01_JPRB, 0.19891E+01_JPRB, 0.21779E+01_JPRB, 0.21627E+01_JPRB, 0.21144E+01_JPRB, 0.18100E+01_JPRB ,&
&  0.15307E+01_JPRB, 0.14288E+01_JPRB, 0.14399E+01_JPRB, 0.13789E+01_JPRB, 0.13227E+01_JPRB, 0.12632E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.12597E+01_JPRB, 0.12495E+01_JPRB, 0.12643E+01_JPRB, 0.12716E+01_JPRB, 0.12655E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.12851E+01_JPRB, 0.12958E+01_JPRB, 0.12816E+01_JPRB, 0.13002E+01_JPRB, 0.13239E+01_JPRB, 0.13800E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.13899E+01_JPRB, 0.13483E+01_JPRB, 0.14545E+01_JPRB, 0.15461E+01_JPRB, 0.16210E+01_JPRB, 0.17878E+01_JPRB, 0.18332E+01_JPRB ,&
&  0.21022E+01_JPRB, 0.21248E+01_JPRB, 0.25055E+01_JPRB, 0.20197E+01_JPRB, 0.21138E+01_JPRB, 0.23660E+01_JPRB, 0.25374E+01_JPRB ,&
&  0.24981E+01_JPRB, 0.30550E+01_JPRB, 0.35289E+01_JPRB, 0.41645E+01_JPRB, 0.46515E+01_JPRB, 0.52258E+01_JPRB, 0.49832E+01_JPRB ,&
&  0.51455E+01_JPRB, 0.55681E+01_JPRB, 0.58428E+01_JPRB, 0.50288E+01_JPRB, 0.49003E+01_JPRB, 0.44759E+01_JPRB, 0.50850E+01_JPRB ,&
&  0.53638E+01_JPRB, 0.47033E+01_JPRB, 0.42457E+01_JPRB, 0.37114E+01_JPRB, 0.27961E+01_JPRB, 0.23889E+01_JPRB, 0.19674E+01_JPRB ,&
&  0.15784E+01_JPRB, 0.13396E+01_JPRB, 0.11494E+01_JPRB, 0.11145E+01_JPRB, 0.11297E+01_JPRB, 0.11509E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.11987E+01_JPRB, 0.11988E+01_JPRB, 0.11996E+01_JPRB, 0.11879E+01_JPRB, 0.11651E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.11581E+01_JPRB, 0.11596E+01_JPRB, 0.11838E+01_JPRB, 0.12106E+01_JPRB, 0.12298E+01_JPRB, 0.12635E+01_JPRB, 0.12278E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.10384E+01_JPRB, 0.10607E+01_JPRB, 0.10744E+01_JPRB, 0.10866E+01_JPRB, 0.10743E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.13674E+01_JPRB, 0.15638E+01_JPRB, 0.17642E+01_JPRB, 0.19740E+01_JPRB, 0.22413E+01_JPRB, 0.22299E+01_JPRB ,&
&  0.17608E+01_JPRB, 0.17898E+01_JPRB, 0.22017E+01_JPRB, 0.22129E+01_JPRB, 0.21074E+01_JPRB, 0.20447E+01_JPRB, 0.19250E+01_JPRB ,&
&  0.17410E+01_JPRB, 0.16390E+01_JPRB, 0.16843E+01_JPRB, 0.15246E+01_JPRB, 0.13235E+01_JPRB, 0.13273E+01_JPRB, 0.13517E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.13566E+01_JPRB, 0.13583E+01_JPRB, 0.13758E+01_JPRB, 0.13869E+01_JPRB, 0.13719E+01_JPRB, 0.13501E+01_JPRB ,&
&  0.13486E+01_JPRB, 0.13590E+01_JPRB, 0.13569E+01_JPRB, 0.13440E+01_JPRB, 0.13866E+01_JPRB, 0.14379E+01_JPRB, 0.14710E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.15482E+01_JPRB, 0.16075E+01_JPRB, 0.16726E+01_JPRB, 0.16653E+01_JPRB, 0.18902E+01_JPRB, 0.19716E+01_JPRB ,&
&  0.21344E+01_JPRB, 0.22641E+01_JPRB, 0.25751E+01_JPRB, 0.20167E+01_JPRB, 0.22448E+01_JPRB, 0.24514E+01_JPRB, 0.27200E+01_JPRB ,&
&  0.28766E+01_JPRB, 0.33294E+01_JPRB, 0.36719E+01_JPRB, 0.40629E+01_JPRB, 0.44179E+01_JPRB, 0.43125E+01_JPRB, 0.45716E+01_JPRB ,&
&  0.43879E+01_JPRB, 0.50785E+01_JPRB, 0.49575E+01_JPRB, 0.47519E+01_JPRB, 0.41607E+01_JPRB, 0.40420E+01_JPRB, 0.43662E+01_JPRB ,&
&  0.46306E+01_JPRB, 0.39630E+01_JPRB, 0.34026E+01_JPRB, 0.27130E+01_JPRB, 0.19955E+01_JPRB, 0.16259E+01_JPRB, 0.13137E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.10473E+01_JPRB, 0.10102E+01_JPRB, 0.10522E+01_JPRB, 0.10759E+01_JPRB, 0.11356E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11399E+01_JPRB, 0.11562E+01_JPRB, 0.11743E+01_JPRB, 0.11814E+01_JPRB, 0.11736E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.11219E+01_JPRB, 0.11119E+01_JPRB, 0.11026E+01_JPRB, 0.11033E+01_JPRB, 0.11189E+01_JPRB, 0.11254E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10892E+01_JPRB, 0.11315E+01_JPRB, 0.11474E+01_JPRB, 0.11536E+01_JPRB, 0.11616E+01_JPRB, 0.12186E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.14979E+01_JPRB, 0.16896E+01_JPRB, 0.18348E+01_JPRB, 0.19843E+01_JPRB, 0.21627E+01_JPRB, 0.23368E+01_JPRB ,&
&  0.20479E+01_JPRB, 0.16434E+01_JPRB, 0.20532E+01_JPRB, 0.21494E+01_JPRB, 0.20572E+01_JPRB, 0.20785E+01_JPRB, 0.21951E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.16679E+01_JPRB, 0.16761E+01_JPRB, 0.15739E+01_JPRB, 0.13898E+01_JPRB, 0.14082E+01_JPRB, 0.14351E+01_JPRB ,&
&  0.14484E+01_JPRB, 0.14470E+01_JPRB, 0.14701E+01_JPRB, 0.14958E+01_JPRB, 0.15099E+01_JPRB, 0.14933E+01_JPRB, 0.14637E+01_JPRB ,&
&  0.14707E+01_JPRB, 0.14861E+01_JPRB, 0.15077E+01_JPRB, 0.15083E+01_JPRB, 0.15346E+01_JPRB, 0.15844E+01_JPRB, 0.16263E+01_JPRB ,&
&  0.16656E+01_JPRB, 0.16752E+01_JPRB, 0.17608E+01_JPRB, 0.19194E+01_JPRB, 0.20723E+01_JPRB, 0.19120E+01_JPRB, 0.20892E+01_JPRB ,&
&  0.21606E+01_JPRB, 0.22700E+01_JPRB, 0.22832E+01_JPRB, 0.26401E+01_JPRB, 0.19087E+01_JPRB, 0.22865E+01_JPRB, 0.25040E+01_JPRB ,&
&  0.32135E+01_JPRB, 0.33494E+01_JPRB, 0.35739E+01_JPRB, 0.38515E+01_JPRB, 0.40818E+01_JPRB, 0.38088E+01_JPRB, 0.38925E+01_JPRB ,&
&  0.41436E+01_JPRB, 0.42963E+01_JPRB, 0.43348E+01_JPRB, 0.43484E+01_JPRB, 0.40181E+01_JPRB, 0.40196E+01_JPRB, 0.39469E+01_JPRB ,&
&  0.38705E+01_JPRB, 0.29977E+01_JPRB, 0.23686E+01_JPRB, 0.18085E+01_JPRB, 0.13559E+01_JPRB, 0.11627E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10842E+01_JPRB, 0.10661E+01_JPRB, 0.10656E+01_JPRB, 0.10906E+01_JPRB, 0.11377E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11498E+01_JPRB, 0.11649E+01_JPRB, 0.11657E+01_JPRB, 0.11513E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.11006E+01_JPRB, 0.10842E+01_JPRB, 0.10573E+01_JPRB, 0.10615E+01_JPRB, 0.10632E+01_JPRB, 0.10576E+01_JPRB, 0.10492E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.11114E+01_JPRB, 0.11603E+01_JPRB, 0.11897E+01_JPRB, 0.11968E+01_JPRB, 0.12122E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.13717E+01_JPRB, 0.15210E+01_JPRB, 0.17145E+01_JPRB, 0.18706E+01_JPRB, 0.20057E+01_JPRB, 0.20952E+01_JPRB, 0.22264E+01_JPRB ,&
&  0.22234E+01_JPRB, 0.22360E+01_JPRB, 0.21234E+01_JPRB, 0.16871E+01_JPRB, 0.15915E+01_JPRB, 0.19102E+01_JPRB, 0.17681E+01_JPRB ,&
&  0.16389E+01_JPRB, 0.15779E+01_JPRB, 0.15392E+01_JPRB, 0.14986E+01_JPRB, 0.14700E+01_JPRB, 0.14990E+01_JPRB, 0.15300E+01_JPRB ,&
&  0.15323E+01_JPRB, 0.15561E+01_JPRB, 0.15825E+01_JPRB, 0.16049E+01_JPRB, 0.16209E+01_JPRB, 0.16497E+01_JPRB, 0.16534E+01_JPRB ,&
&  0.16947E+01_JPRB, 0.17274E+01_JPRB, 0.17712E+01_JPRB, 0.17976E+01_JPRB, 0.17799E+01_JPRB, 0.18245E+01_JPRB, 0.18591E+01_JPRB ,&
&  0.19067E+01_JPRB, 0.19161E+01_JPRB, 0.19220E+01_JPRB, 0.20199E+01_JPRB, 0.20816E+01_JPRB, 0.18705E+01_JPRB, 0.19307E+01_JPRB ,&
&  0.21612E+01_JPRB, 0.22922E+01_JPRB, 0.21293E+01_JPRB, 0.26399E+01_JPRB, 0.16991E+01_JPRB, 0.23009E+01_JPRB, 0.29719E+01_JPRB ,&
&  0.30997E+01_JPRB, 0.32219E+01_JPRB, 0.33677E+01_JPRB, 0.34957E+01_JPRB, 0.35565E+01_JPRB, 0.33518E+01_JPRB, 0.36373E+01_JPRB ,&
&  0.39196E+01_JPRB, 0.38779E+01_JPRB, 0.38388E+01_JPRB, 0.38069E+01_JPRB, 0.36983E+01_JPRB, 0.33680E+01_JPRB, 0.29176E+01_JPRB ,&
&  0.22961E+01_JPRB, 0.22554E+01_JPRB, 0.17784E+01_JPRB, 0.14927E+01_JPRB, 0.11701E+01_JPRB, 0.10066E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10699E+01_JPRB, 0.10665E+01_JPRB, 0.10878E+01_JPRB, 0.11036E+01_JPRB, 0.11203E+01_JPRB, 0.11153E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11364E+01_JPRB, 0.11429E+01_JPRB, 0.11343E+01_JPRB, 0.11189E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.10379E+01_JPRB, 0.10194E+01_JPRB, 0.99941E+00_JPRB, 0.10299E+01_JPRB, 0.10200E+01_JPRB, 0.10051E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.10515E+01_JPRB, 0.10866E+01_JPRB, 0.11253E+01_JPRB, 0.11747E+01_JPRB, 0.12462E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.13965E+01_JPRB, 0.14862E+01_JPRB, 0.16372E+01_JPRB, 0.18309E+01_JPRB, 0.20738E+01_JPRB, 0.22050E+01_JPRB, 0.22260E+01_JPRB ,&
&  0.21119E+01_JPRB, 0.21462E+01_JPRB, 0.21863E+01_JPRB, 0.21639E+01_JPRB, 0.18661E+01_JPRB, 0.19584E+01_JPRB, 0.20175E+01_JPRB ,&
&  0.19701E+01_JPRB, 0.17491E+01_JPRB, 0.15879E+01_JPRB, 0.15535E+01_JPRB, 0.15367E+01_JPRB, 0.15697E+01_JPRB, 0.16082E+01_JPRB ,&
&  0.16279E+01_JPRB, 0.16598E+01_JPRB, 0.16805E+01_JPRB, 0.17066E+01_JPRB, 0.17562E+01_JPRB, 0.18425E+01_JPRB, 0.18836E+01_JPRB ,&
&  0.19075E+01_JPRB, 0.19635E+01_JPRB, 0.19732E+01_JPRB, 0.19406E+01_JPRB, 0.19902E+01_JPRB, 0.20317E+01_JPRB, 0.20727E+01_JPRB ,&
&  0.21501E+01_JPRB, 0.20905E+01_JPRB, 0.21528E+01_JPRB, 0.22375E+01_JPRB, 0.21934E+01_JPRB, 0.20390E+01_JPRB, 0.20670E+01_JPRB ,&
&  0.22228E+01_JPRB, 0.23415E+01_JPRB, 0.18040E+01_JPRB, 0.25451E+01_JPRB, 0.31347E+01_JPRB, 0.28719E+01_JPRB, 0.29413E+01_JPRB ,&
&  0.29673E+01_JPRB, 0.30713E+01_JPRB, 0.32316E+01_JPRB, 0.33758E+01_JPRB, 0.34546E+01_JPRB, 0.35150E+01_JPRB, 0.32831E+01_JPRB ,&
&  0.37986E+01_JPRB, 0.36793E+01_JPRB, 0.34807E+01_JPRB, 0.32883E+01_JPRB, 0.30195E+01_JPRB, 0.25446E+01_JPRB, 0.20729E+01_JPRB ,&
&  0.16655E+01_JPRB, 0.18600E+01_JPRB, 0.15526E+01_JPRB, 0.15052E+01_JPRB, 0.13845E+01_JPRB, 0.96984E+00_JPRB, 0.10211E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10573E+01_JPRB, 0.10617E+01_JPRB, 0.10499E+01_JPRB, 0.10585E+01_JPRB, 0.10700E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.10765E+01_JPRB, 0.10761E+01_JPRB, 0.10621E+01_JPRB, 0.10529E+01_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.10056E+01_JPRB, 0.99920E+00_JPRB, 0.98490E+00_JPRB, 0.99560E+00_JPRB, 0.98163E+00_JPRB, 0.99834E+00_JPRB, 0.97693E+00_JPRB ,&
&  0.95760E+00_JPRB, 0.95421E+00_JPRB, 0.96180E+00_JPRB, 0.97211E+00_JPRB, 0.10030E+01_JPRB, 0.10650E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.12156E+01_JPRB, 0.12321E+01_JPRB, 0.12765E+01_JPRB, 0.13998E+01_JPRB, 0.16748E+01_JPRB, 0.19549E+01_JPRB, 0.20494E+01_JPRB ,&
&  0.21586E+01_JPRB, 0.23045E+01_JPRB, 0.23988E+01_JPRB, 0.24785E+01_JPRB, 0.24629E+01_JPRB, 0.24400E+01_JPRB, 0.24903E+01_JPRB ,&
&  0.21103E+01_JPRB, 0.16942E+01_JPRB, 0.15990E+01_JPRB, 0.15744E+01_JPRB, 0.15990E+01_JPRB, 0.16184E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.16592E+01_JPRB, 0.16965E+01_JPRB, 0.17258E+01_JPRB, 0.17819E+01_JPRB, 0.18717E+01_JPRB, 0.19808E+01_JPRB, 0.20385E+01_JPRB ,&
&  0.20544E+01_JPRB, 0.20815E+01_JPRB, 0.21198E+01_JPRB, 0.20217E+01_JPRB, 0.22024E+01_JPRB, 0.22958E+01_JPRB, 0.23503E+01_JPRB ,&
&  0.23539E+01_JPRB, 0.23119E+01_JPRB, 0.23997E+01_JPRB, 0.24297E+01_JPRB, 0.24438E+01_JPRB, 0.23334E+01_JPRB, 0.22695E+01_JPRB ,&
&  0.23040E+01_JPRB, 0.20811E+01_JPRB, 0.16566E+01_JPRB, 0.19306E+01_JPRB, 0.22387E+01_JPRB, 0.21293E+01_JPRB, 0.28680E+01_JPRB ,&
&  0.28924E+01_JPRB, 0.28675E+01_JPRB, 0.29448E+01_JPRB, 0.30503E+01_JPRB, 0.32237E+01_JPRB, 0.32422E+01_JPRB, 0.32040E+01_JPRB ,&
&  0.33703E+01_JPRB, 0.31924E+01_JPRB, 0.29228E+01_JPRB, 0.25804E+01_JPRB, 0.22069E+01_JPRB, 0.18768E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.14841E+01_JPRB, 0.14204E+01_JPRB, 0.12673E+01_JPRB, 0.11524E+01_JPRB, 0.97204E+00_JPRB, 0.90695E+00_JPRB, 0.92911E+00_JPRB ,&
&  0.98195E+00_JPRB, 0.99085E+00_JPRB, 0.99654E+00_JPRB, 0.98755E+00_JPRB, 0.98572E+00_JPRB, 0.98331E+00_JPRB, 0.98578E+00_JPRB ,&
&  0.10022E+01_JPRB, 0.10262E+01_JPRB, 0.10259E+01_JPRB, 0.10224E+01_JPRB, 0.10176E+01_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.95807E+00_JPRB, 0.95090E+00_JPRB, 0.94689E+00_JPRB, 0.96031E+00_JPRB, 0.96997E+00_JPRB, 0.95367E+00_JPRB, 0.96117E+00_JPRB ,&
&  0.93632E+00_JPRB, 0.91590E+00_JPRB, 0.92543E+00_JPRB, 0.92970E+00_JPRB, 0.94741E+00_JPRB, 0.96685E+00_JPRB, 0.97826E+00_JPRB ,&
&  0.97696E+00_JPRB, 0.96617E+00_JPRB, 0.92093E+00_JPRB, 0.92487E+00_JPRB, 0.10237E+01_JPRB, 0.11991E+01_JPRB, 0.15281E+01_JPRB ,&
&  0.18854E+01_JPRB, 0.20490E+01_JPRB, 0.21389E+01_JPRB, 0.21410E+01_JPRB, 0.20592E+01_JPRB, 0.19292E+01_JPRB, 0.24945E+01_JPRB ,&
&  0.24617E+01_JPRB, 0.16399E+01_JPRB, 0.16217E+01_JPRB, 0.15159E+01_JPRB, 0.15415E+01_JPRB, 0.15408E+01_JPRB, 0.15578E+01_JPRB ,&
&  0.15601E+01_JPRB, 0.15972E+01_JPRB, 0.16578E+01_JPRB, 0.17548E+01_JPRB, 0.18527E+01_JPRB, 0.19585E+01_JPRB, 0.20325E+01_JPRB ,&
&  0.20621E+01_JPRB, 0.20890E+01_JPRB, 0.21727E+01_JPRB, 0.21605E+01_JPRB, 0.21708E+01_JPRB, 0.22811E+01_JPRB, 0.23821E+01_JPRB ,&
&  0.24658E+01_JPRB, 0.26233E+01_JPRB, 0.24213E+01_JPRB, 0.24383E+01_JPRB, 0.25325E+01_JPRB, 0.23114E+01_JPRB, 0.22964E+01_JPRB ,&
&  0.23132E+01_JPRB, 0.20083E+01_JPRB, 0.15322E+01_JPRB, 0.17227E+01_JPRB, 0.19609E+01_JPRB, 0.21926E+01_JPRB, 0.25129E+01_JPRB ,&
&  0.24750E+01_JPRB, 0.23711E+01_JPRB, 0.24092E+01_JPRB, 0.25800E+01_JPRB, 0.26516E+01_JPRB, 0.26393E+01_JPRB, 0.25794E+01_JPRB ,&
&  0.25377E+01_JPRB, 0.23160E+01_JPRB, 0.21514E+01_JPRB, 0.19123E+01_JPRB, 0.17124E+01_JPRB, 0.15564E+01_JPRB, 0.13642E+01_JPRB ,&
&  0.13594E+01_JPRB, 0.14472E+01_JPRB, 0.16798E+01_JPRB, 0.14568E+01_JPRB, 0.97160E+00_JPRB, 0.90314E+00_JPRB, 0.88725E+00_JPRB ,&
&  0.90404E+00_JPRB, 0.91495E+00_JPRB, 0.94097E+00_JPRB, 0.94311E+00_JPRB, 0.93236E+00_JPRB, 0.92057E+00_JPRB, 0.91197E+00_JPRB ,&
&  0.93399E+00_JPRB, 0.96534E+00_JPRB, 0.97903E+00_JPRB, 0.96933E+00_JPRB, 0.95912E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.99937E+00_JPRB, 0.10016E+01_JPRB, 0.10217E+01_JPRB, 0.10258E+01_JPRB, 0.10221E+01_JPRB, 0.10270E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.10171E+01_JPRB, 0.10218E+01_JPRB, 0.10292E+01_JPRB, 0.10389E+01_JPRB, 0.10539E+01_JPRB, 0.10683E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10577E+01_JPRB, 0.10801E+01_JPRB, 0.10959E+01_JPRB, 0.10918E+01_JPRB, 0.10892E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.12325E+01_JPRB, 0.12337E+01_JPRB, 0.12100E+01_JPRB, 0.12462E+01_JPRB, 0.14558E+01_JPRB, 0.22080E+01_JPRB ,&
&  0.22922E+01_JPRB, 0.16183E+01_JPRB, 0.14465E+01_JPRB, 0.13205E+01_JPRB, 0.13376E+01_JPRB, 0.13893E+01_JPRB, 0.13766E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.13560E+01_JPRB, 0.13724E+01_JPRB, 0.13906E+01_JPRB, 0.14310E+01_JPRB, 0.14654E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.14153E+01_JPRB, 0.14397E+01_JPRB, 0.14884E+01_JPRB, 0.16602E+01_JPRB, 0.19367E+01_JPRB, 0.18742E+01_JPRB ,&
&  0.18844E+01_JPRB, 0.20295E+01_JPRB, 0.21805E+01_JPRB, 0.22645E+01_JPRB, 0.22205E+01_JPRB, 0.21033E+01_JPRB, 0.20193E+01_JPRB ,&
&  0.19511E+01_JPRB, 0.18110E+01_JPRB, 0.16395E+01_JPRB, 0.19421E+01_JPRB, 0.20915E+01_JPRB, 0.22446E+01_JPRB, 0.21349E+01_JPRB ,&
&  0.20354E+01_JPRB, 0.18350E+01_JPRB, 0.18218E+01_JPRB, 0.19513E+01_JPRB, 0.20632E+01_JPRB, 0.20439E+01_JPRB, 0.19431E+01_JPRB ,&
&  0.17924E+01_JPRB, 0.16827E+01_JPRB, 0.16761E+01_JPRB, 0.15787E+01_JPRB, 0.15335E+01_JPRB, 0.12731E+01_JPRB, 0.12384E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.13753E+01_JPRB, 0.11434E+01_JPRB, 0.14439E+01_JPRB, 0.91516E+00_JPRB, 0.81593E+00_JPRB, 0.82242E+00_JPRB ,&
&  0.84601E+00_JPRB, 0.87968E+00_JPRB, 0.89063E+00_JPRB, 0.98418E+00_JPRB, 0.97402E+00_JPRB, 0.97064E+00_JPRB, 0.98581E+00_JPRB ,&
&  0.96550E+00_JPRB, 0.96709E+00_JPRB, 0.97083E+00_JPRB, 0.98594E+00_JPRB, 0.99411E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.10519E+01_JPRB, 0.10533E+01_JPRB, 0.10592E+01_JPRB, 0.10707E+01_JPRB, 0.10779E+01_JPRB, 0.10846E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.10772E+01_JPRB, 0.10879E+01_JPRB, 0.11045E+01_JPRB, 0.11212E+01_JPRB, 0.11358E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11571E+01_JPRB, 0.11782E+01_JPRB, 0.12012E+01_JPRB, 0.12127E+01_JPRB, 0.12122E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.12211E+01_JPRB, 0.12505E+01_JPRB, 0.12696E+01_JPRB, 0.12967E+01_JPRB, 0.13726E+01_JPRB, 0.16184E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.17366E+01_JPRB, 0.14882E+01_JPRB, 0.13299E+01_JPRB, 0.13158E+01_JPRB, 0.12585E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11316E+01_JPRB, 0.10798E+01_JPRB, 0.10826E+01_JPRB, 0.10713E+01_JPRB, 0.11019E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.11685E+01_JPRB, 0.12415E+01_JPRB, 0.13061E+01_JPRB, 0.13997E+01_JPRB, 0.14405E+01_JPRB, 0.13315E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.14401E+01_JPRB, 0.15861E+01_JPRB, 0.17230E+01_JPRB, 0.16663E+01_JPRB, 0.15675E+01_JPRB, 0.15115E+01_JPRB ,&
&  0.14701E+01_JPRB, 0.15140E+01_JPRB, 0.18863E+01_JPRB, 0.19928E+01_JPRB, 0.20699E+01_JPRB, 0.19818E+01_JPRB, 0.17912E+01_JPRB ,&
&  0.15661E+01_JPRB, 0.13151E+01_JPRB, 0.13006E+01_JPRB, 0.13207E+01_JPRB, 0.13588E+01_JPRB, 0.13325E+01_JPRB, 0.13019E+01_JPRB ,&
&  0.12038E+01_JPRB, 0.11272E+01_JPRB, 0.12618E+01_JPRB, 0.14173E+01_JPRB, 0.15349E+01_JPRB, 0.19091E+01_JPRB, 0.17542E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.11021E+01_JPRB, 0.72276E+00_JPRB, 0.98759E+00_JPRB, 0.75904E+00_JPRB, 0.75787E+00_JPRB, 0.83621E+00_JPRB ,&
&  0.94813E+00_JPRB, 0.10473E+01_JPRB, 0.11191E+01_JPRB, 0.10888E+01_JPRB, 0.10221E+01_JPRB, 0.10154E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10345E+01_JPRB, 0.10385E+01_JPRB, 0.10419E+01_JPRB, 0.10412E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.10650E+01_JPRB, 0.10737E+01_JPRB, 0.10869E+01_JPRB, 0.11041E+01_JPRB, 0.11195E+01_JPRB, 0.11341E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.11699E+01_JPRB, 0.11969E+01_JPRB, 0.12286E+01_JPRB, 0.12439E+01_JPRB, 0.12629E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.12961E+01_JPRB, 0.13274E+01_JPRB, 0.13436E+01_JPRB, 0.13625E+01_JPRB, 0.13750E+01_JPRB, 0.13917E+01_JPRB, 0.14131E+01_JPRB ,&
&  0.14301E+01_JPRB, 0.14727E+01_JPRB, 0.15385E+01_JPRB, 0.15903E+01_JPRB, 0.16654E+01_JPRB, 0.18256E+01_JPRB, 0.16183E+01_JPRB ,&
&  0.17919E+01_JPRB, 0.17329E+01_JPRB, 0.15749E+01_JPRB, 0.13808E+01_JPRB, 0.13174E+01_JPRB, 0.12627E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.11423E+01_JPRB, 0.11545E+01_JPRB, 0.11453E+01_JPRB, 0.11391E+01_JPRB, 0.11933E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.12592E+01_JPRB, 0.12919E+01_JPRB, 0.13087E+01_JPRB, 0.13433E+01_JPRB, 0.13472E+01_JPRB, 0.12885E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.11700E+01_JPRB, 0.11163E+01_JPRB, 0.11244E+01_JPRB, 0.11164E+01_JPRB, 0.10139E+01_JPRB, 0.90047E+00_JPRB ,&
&  0.10279E+01_JPRB, 0.11856E+01_JPRB, 0.13822E+01_JPRB, 0.20553E+01_JPRB, 0.20040E+01_JPRB, 0.18195E+01_JPRB, 0.15154E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.10131E+01_JPRB, 0.94676E+00_JPRB, 0.97335E+00_JPRB, 0.95403E+00_JPRB, 0.93990E+00_JPRB, 0.90315E+00_JPRB ,&
&  0.83782E+00_JPRB, 0.84067E+00_JPRB, 0.98451E+00_JPRB, 0.12022E+01_JPRB, 0.14875E+01_JPRB, 0.16021E+01_JPRB, 0.20848E+01_JPRB ,&
&  0.13418E+01_JPRB, 0.79289E+00_JPRB, 0.86964E+00_JPRB, 0.82800E+00_JPRB, 0.75482E+00_JPRB, 0.70755E+00_JPRB, 0.83086E+00_JPRB ,&
&  0.10417E+01_JPRB, 0.11089E+01_JPRB, 0.11338E+01_JPRB, 0.10887E+01_JPRB, 0.10835E+01_JPRB, 0.10883E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10756E+01_JPRB, 0.10743E+01_JPRB, 0.10743E+01_JPRB, 0.10663E+01_JPRB, 0.10616E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.10840E+01_JPRB, 0.10979E+01_JPRB, 0.11200E+01_JPRB, 0.11491E+01_JPRB, 0.11657E+01_JPRB, 0.11914E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.12528E+01_JPRB, 0.12863E+01_JPRB, 0.13233E+01_JPRB, 0.13579E+01_JPRB, 0.13899E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.14470E+01_JPRB, 0.14743E+01_JPRB, 0.14886E+01_JPRB, 0.14922E+01_JPRB, 0.14800E+01_JPRB, 0.14740E+01_JPRB, 0.14559E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.14420E+01_JPRB, 0.14568E+01_JPRB, 0.14677E+01_JPRB, 0.14925E+01_JPRB, 0.16194E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.16856E+01_JPRB, 0.16886E+01_JPRB, 0.16250E+01_JPRB, 0.14750E+01_JPRB, 0.14013E+01_JPRB, 0.13997E+01_JPRB, 0.14295E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.15684E+01_JPRB, 0.16341E+01_JPRB, 0.15285E+01_JPRB, 0.14091E+01_JPRB, 0.14260E+01_JPRB, 0.14429E+01_JPRB ,&
&  0.14617E+01_JPRB, 0.14646E+01_JPRB, 0.14661E+01_JPRB, 0.14491E+01_JPRB, 0.14345E+01_JPRB, 0.14126E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.12145E+01_JPRB, 0.10988E+01_JPRB, 0.11212E+01_JPRB, 0.11127E+01_JPRB, 0.96427E+00_JPRB, 0.73649E+00_JPRB ,&
&  0.67930E+00_JPRB, 0.96813E+00_JPRB, 0.13278E+01_JPRB, 0.19909E+01_JPRB, 0.18400E+01_JPRB, 0.16076E+01_JPRB, 0.14121E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.90525E+00_JPRB, 0.81427E+00_JPRB, 0.82424E+00_JPRB, 0.81773E+00_JPRB, 0.81567E+00_JPRB, 0.79290E+00_JPRB ,&
&  0.76697E+00_JPRB, 0.77307E+00_JPRB, 0.86457E+00_JPRB, 0.99811E+00_JPRB, 0.12018E+01_JPRB, 0.12503E+01_JPRB, 0.19345E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.10210E+01_JPRB, 0.13590E+01_JPRB, 0.66124E+00_JPRB, 0.77746E+00_JPRB, 0.85744E+00_JPRB, 0.97019E+00_JPRB ,&
&  0.99700E+00_JPRB, 0.13006E+01_JPRB, 0.15918E+01_JPRB, 0.14156E+01_JPRB, 0.12836E+01_JPRB, 0.11751E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.11216E+01_JPRB, 0.11116E+01_JPRB, 0.11089E+01_JPRB, 0.11000E+01_JPRB, 0.10866E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.11444E+01_JPRB, 0.11492E+01_JPRB, 0.11445E+01_JPRB, 0.11782E+01_JPRB, 0.11795E+01_JPRB, 0.11986E+01_JPRB, 0.12032E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.12543E+01_JPRB, 0.12844E+01_JPRB, 0.13308E+01_JPRB, 0.13522E+01_JPRB, 0.13756E+01_JPRB, 0.14130E+01_JPRB ,&
&  0.14506E+01_JPRB, 0.14965E+01_JPRB, 0.15022E+01_JPRB, 0.14958E+01_JPRB, 0.14705E+01_JPRB, 0.14575E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.14229E+01_JPRB, 0.14211E+01_JPRB, 0.14356E+01_JPRB, 0.14610E+01_JPRB, 0.14912E+01_JPRB, 0.15736E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.17304E+01_JPRB, 0.16493E+01_JPRB, 0.16035E+01_JPRB, 0.15157E+01_JPRB, 0.15418E+01_JPRB, 0.16040E+01_JPRB ,&
&  0.16988E+01_JPRB, 0.17898E+01_JPRB, 0.17920E+01_JPRB, 0.17251E+01_JPRB, 0.16825E+01_JPRB, 0.17122E+01_JPRB, 0.17361E+01_JPRB ,&
&  0.17524E+01_JPRB, 0.17611E+01_JPRB, 0.17694E+01_JPRB, 0.17670E+01_JPRB, 0.17698E+01_JPRB, 0.17849E+01_JPRB, 0.17622E+01_JPRB ,&
&  0.17106E+01_JPRB, 0.16579E+01_JPRB, 0.16496E+01_JPRB, 0.16952E+01_JPRB, 0.16580E+01_JPRB, 0.14288E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.74861E+00_JPRB, 0.75809E+00_JPRB, 0.11817E+01_JPRB, 0.17899E+01_JPRB, 0.16494E+01_JPRB, 0.14364E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.88618E+00_JPRB, 0.77598E+00_JPRB, 0.78266E+00_JPRB, 0.80077E+00_JPRB, 0.85144E+00_JPRB, 0.84030E+00_JPRB ,&
&  0.86501E+00_JPRB, 0.88881E+00_JPRB, 0.90977E+00_JPRB, 0.98549E+00_JPRB, 0.11667E+01_JPRB, 0.13035E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.13962E+01_JPRB, 0.10861E+01_JPRB, 0.10307E+01_JPRB, 0.11116E+01_JPRB, 0.10557E+01_JPRB, 0.12440E+01_JPRB, 0.15392E+01_JPRB ,&
&  0.19978E+01_JPRB, 0.19267E+01_JPRB, 0.25250E+01_JPRB, 0.38248E+01_JPRB, 0.30079E+01_JPRB, 0.21685E+01_JPRB, 0.12046E+01_JPRB ,&
&  0.11938E+01_JPRB, 0.11688E+01_JPRB, 0.11800E+01_JPRB, 0.11641E+01_JPRB, 0.11512E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.10909E+01_JPRB, 0.10769E+01_JPRB, 0.10785E+01_JPRB, 0.10794E+01_JPRB, 0.10780E+01_JPRB, 0.10891E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.11698E+01_JPRB, 0.12332E+01_JPRB, 0.12785E+01_JPRB, 0.13368E+01_JPRB, 0.13516E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.13323E+01_JPRB, 0.13773E+01_JPRB, 0.14188E+01_JPRB, 0.14400E+01_JPRB, 0.14115E+01_JPRB, 0.14033E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.14944E+01_JPRB, 0.15412E+01_JPRB, 0.15857E+01_JPRB, 0.16505E+01_JPRB, 0.16683E+01_JPRB, 0.15964E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.15080E+01_JPRB, 0.17014E+01_JPRB, 0.17358E+01_JPRB, 0.16962E+01_JPRB, 0.16828E+01_JPRB, 0.16204E+01_JPRB, 0.16755E+01_JPRB ,&
&  0.17814E+01_JPRB, 0.18727E+01_JPRB, 0.18261E+01_JPRB, 0.16996E+01_JPRB, 0.18673E+01_JPRB, 0.18883E+01_JPRB, 0.19377E+01_JPRB ,&
&  0.19707E+01_JPRB, 0.20056E+01_JPRB, 0.20449E+01_JPRB, 0.20933E+01_JPRB, 0.21261E+01_JPRB, 0.21639E+01_JPRB, 0.21869E+01_JPRB ,&
&  0.21674E+01_JPRB, 0.21333E+01_JPRB, 0.21538E+01_JPRB, 0.20426E+01_JPRB, 0.19781E+01_JPRB, 0.19979E+01_JPRB, 0.16867E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.78230E+00_JPRB, 0.11634E+01_JPRB, 0.15706E+01_JPRB, 0.14047E+01_JPRB, 0.12462E+01_JPRB, 0.11284E+01_JPRB ,&
&  0.98051E+00_JPRB, 0.85541E+00_JPRB, 0.82322E+00_JPRB, 0.88237E+00_JPRB, 0.96307E+00_JPRB, 0.10338E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.10238E+01_JPRB, 0.10460E+01_JPRB, 0.11036E+01_JPRB, 0.12232E+01_JPRB, 0.14599E+01_JPRB, 0.17899E+01_JPRB ,&
&  0.16985E+01_JPRB, 0.11631E+01_JPRB, 0.11956E+01_JPRB, 0.12288E+01_JPRB, 0.13095E+01_JPRB, 0.14565E+01_JPRB, 0.17597E+01_JPRB ,&
&  0.22363E+01_JPRB, 0.27340E+01_JPRB, 0.26438E+01_JPRB, 0.22096E+01_JPRB, 0.27591E+01_JPRB, 0.18718E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10833E+01_JPRB, 0.11052E+01_JPRB, 0.11036E+01_JPRB, 0.11003E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.94237E+00_JPRB, 0.92546E+00_JPRB, 0.91431E+00_JPRB, 0.90457E+00_JPRB, 0.91654E+00_JPRB, 0.95285E+00_JPRB, 0.99622E+00_JPRB ,&
&  0.10425E+01_JPRB, 0.10955E+01_JPRB, 0.11653E+01_JPRB, 0.11981E+01_JPRB, 0.12464E+01_JPRB, 0.12537E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.12635E+01_JPRB, 0.13015E+01_JPRB, 0.13409E+01_JPRB, 0.13656E+01_JPRB, 0.14133E+01_JPRB, 0.14747E+01_JPRB ,&
&  0.15526E+01_JPRB, 0.16257E+01_JPRB, 0.17137E+01_JPRB, 0.18494E+01_JPRB, 0.19065E+01_JPRB, 0.19316E+01_JPRB, 0.19772E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.15551E+01_JPRB, 0.16587E+01_JPRB, 0.16502E+01_JPRB, 0.17687E+01_JPRB, 0.18043E+01_JPRB, 0.18353E+01_JPRB ,&
&  0.19198E+01_JPRB, 0.18902E+01_JPRB, 0.18340E+01_JPRB, 0.17922E+01_JPRB, 0.18957E+01_JPRB, 0.19408E+01_JPRB, 0.19779E+01_JPRB ,&
&  0.19796E+01_JPRB, 0.19780E+01_JPRB, 0.20196E+01_JPRB, 0.20927E+01_JPRB, 0.21548E+01_JPRB, 0.22183E+01_JPRB, 0.23063E+01_JPRB ,&
&  0.23428E+01_JPRB, 0.23668E+01_JPRB, 0.24630E+01_JPRB, 0.22415E+01_JPRB, 0.20809E+01_JPRB, 0.21678E+01_JPRB, 0.19706E+01_JPRB ,&
&  0.14048E+01_JPRB, 0.91675E+00_JPRB, 0.11068E+01_JPRB, 0.14597E+01_JPRB, 0.12818E+01_JPRB, 0.11504E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.94137E+00_JPRB, 0.87345E+00_JPRB, 0.94893E+00_JPRB, 0.10327E+01_JPRB, 0.11111E+01_JPRB, 0.11766E+01_JPRB, 0.11819E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.10932E+01_JPRB, 0.10588E+01_JPRB, 0.10618E+01_JPRB, 0.10919E+01_JPRB, 0.11236E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.11121E+01_JPRB, 0.11342E+01_JPRB, 0.11674E+01_JPRB, 0.11942E+01_JPRB, 0.12099E+01_JPRB, 0.13485E+01_JPRB ,&
&  0.15817E+01_JPRB, 0.17951E+01_JPRB, 0.20447E+01_JPRB, 0.19814E+01_JPRB, 0.17830E+01_JPRB, 0.12981E+01_JPRB, 0.89578E+00_JPRB ,&
&  0.91928E+00_JPRB, 0.94197E+00_JPRB, 0.94669E+00_JPRB, 0.96047E+00_JPRB, 0.95209E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.84737E+00_JPRB, 0.83938E+00_JPRB, 0.83186E+00_JPRB, 0.83702E+00_JPRB, 0.84678E+00_JPRB, 0.89722E+00_JPRB, 0.98028E+00_JPRB ,&
&  0.10628E+01_JPRB, 0.11157E+01_JPRB, 0.11632E+01_JPRB, 0.11916E+01_JPRB, 0.12024E+01_JPRB, 0.11727E+01_JPRB, 0.11764E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11885E+01_JPRB, 0.12236E+01_JPRB, 0.12742E+01_JPRB, 0.13229E+01_JPRB, 0.13778E+01_JPRB, 0.14415E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.16295E+01_JPRB, 0.17467E+01_JPRB, 0.19155E+01_JPRB, 0.20598E+01_JPRB, 0.21685E+01_JPRB, 0.21711E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.11095E+01_JPRB, 0.15832E+01_JPRB, 0.15523E+01_JPRB, 0.16654E+01_JPRB, 0.18285E+01_JPRB, 0.20248E+01_JPRB ,&
&  0.19466E+01_JPRB, 0.17190E+01_JPRB, 0.16903E+01_JPRB, 0.18714E+01_JPRB, 0.18221E+01_JPRB, 0.18659E+01_JPRB, 0.18942E+01_JPRB ,&
&  0.18844E+01_JPRB, 0.18638E+01_JPRB, 0.18623E+01_JPRB, 0.19041E+01_JPRB, 0.19745E+01_JPRB, 0.20510E+01_JPRB, 0.21471E+01_JPRB ,&
&  0.22358E+01_JPRB, 0.23419E+01_JPRB, 0.24570E+01_JPRB, 0.20693E+01_JPRB, 0.21524E+01_JPRB, 0.23209E+01_JPRB, 0.25625E+01_JPRB ,&
&  0.20344E+01_JPRB, 0.12585E+01_JPRB, 0.12110E+01_JPRB, 0.15106E+01_JPRB, 0.13645E+01_JPRB, 0.13159E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.10395E+01_JPRB, 0.10728E+01_JPRB, 0.11448E+01_JPRB, 0.12123E+01_JPRB, 0.12608E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.11618E+01_JPRB, 0.11474E+01_JPRB, 0.11580E+01_JPRB, 0.11818E+01_JPRB, 0.11987E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11962E+01_JPRB, 0.12348E+01_JPRB, 0.12585E+01_JPRB, 0.12809E+01_JPRB, 0.13033E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.12240E+01_JPRB, 0.13020E+01_JPRB, 0.11976E+01_JPRB, 0.10621E+01_JPRB, 0.91625E+00_JPRB, 0.78286E+00_JPRB ,&
&  0.80556E+00_JPRB, 0.81698E+00_JPRB, 0.81598E+00_JPRB, 0.82526E+00_JPRB, 0.84314E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.75808E+00_JPRB, 0.76313E+00_JPRB, 0.75623E+00_JPRB, 0.77417E+00_JPRB, 0.77652E+00_JPRB, 0.84645E+00_JPRB, 0.91918E+00_JPRB ,&
&  0.10333E+01_JPRB, 0.11192E+01_JPRB, 0.11754E+01_JPRB, 0.11937E+01_JPRB, 0.11790E+01_JPRB, 0.11325E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11077E+01_JPRB, 0.11270E+01_JPRB, 0.11569E+01_JPRB, 0.11904E+01_JPRB, 0.12394E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.13628E+01_JPRB, 0.14685E+01_JPRB, 0.16052E+01_JPRB, 0.18017E+01_JPRB, 0.20599E+01_JPRB, 0.24140E+01_JPRB, 0.27818E+01_JPRB ,&
&  0.22124E+01_JPRB, 0.11095E+01_JPRB, 0.15748E+01_JPRB, 0.14948E+01_JPRB, 0.13753E+01_JPRB, 0.16151E+01_JPRB, 0.19036E+01_JPRB ,&
&  0.18060E+01_JPRB, 0.16758E+01_JPRB, 0.16044E+01_JPRB, 0.17759E+01_JPRB, 0.17554E+01_JPRB, 0.17744E+01_JPRB, 0.17880E+01_JPRB ,&
&  0.17861E+01_JPRB, 0.17659E+01_JPRB, 0.17510E+01_JPRB, 0.17876E+01_JPRB, 0.18510E+01_JPRB, 0.19414E+01_JPRB, 0.20341E+01_JPRB ,&
&  0.21504E+01_JPRB, 0.22742E+01_JPRB, 0.24439E+01_JPRB, 0.20747E+01_JPRB, 0.22394E+01_JPRB, 0.22675E+01_JPRB, 0.20403E+01_JPRB ,&
&  0.19849E+01_JPRB, 0.14285E+01_JPRB, 0.12998E+01_JPRB, 0.15715E+01_JPRB, 0.14590E+01_JPRB, 0.11520E+01_JPRB, 0.11749E+01_JPRB ,&
&  0.11266E+01_JPRB, 0.11459E+01_JPRB, 0.11361E+01_JPRB, 0.12122E+01_JPRB, 0.12553E+01_JPRB, 0.12864E+01_JPRB, 0.12677E+01_JPRB ,&
&  0.12491E+01_JPRB, 0.12176E+01_JPRB, 0.12109E+01_JPRB, 0.12243E+01_JPRB, 0.12392E+01_JPRB, 0.12762E+01_JPRB, 0.12895E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.13340E+01_JPRB, 0.13969E+01_JPRB, 0.14564E+01_JPRB, 0.14782E+01_JPRB, 0.14008E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.12112E+01_JPRB, 0.10850E+01_JPRB, 0.95707E+00_JPRB, 0.90721E+00_JPRB, 0.83095E+00_JPRB, 0.76560E+00_JPRB ,&
&  0.77631E+00_JPRB, 0.77367E+00_JPRB, 0.75921E+00_JPRB, 0.75807E+00_JPRB, 0.76076E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.80850E+00_JPRB, 0.79588E+00_JPRB, 0.76558E+00_JPRB, 0.74747E+00_JPRB, 0.74356E+00_JPRB, 0.77134E+00_JPRB, 0.81644E+00_JPRB ,&
&  0.88767E+00_JPRB, 0.98257E+00_JPRB, 0.10602E+01_JPRB, 0.10979E+01_JPRB, 0.10917E+01_JPRB, 0.10999E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.10263E+01_JPRB, 0.10427E+01_JPRB, 0.10532E+01_JPRB, 0.10713E+01_JPRB, 0.10981E+01_JPRB, 0.11288E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.11647E+01_JPRB, 0.11928E+01_JPRB, 0.13051E+01_JPRB, 0.14959E+01_JPRB, 0.17940E+01_JPRB, 0.23377E+01_JPRB ,&
&  0.34931E+01_JPRB, 0.35389E+01_JPRB, 0.18041E+01_JPRB, 0.18830E+01_JPRB, 0.15913E+01_JPRB, 0.16662E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.17029E+01_JPRB, 0.16067E+01_JPRB, 0.15626E+01_JPRB, 0.16691E+01_JPRB, 0.16468E+01_JPRB, 0.16602E+01_JPRB, 0.16645E+01_JPRB ,&
&  0.16905E+01_JPRB, 0.16881E+01_JPRB, 0.16626E+01_JPRB, 0.16549E+01_JPRB, 0.17038E+01_JPRB, 0.17876E+01_JPRB, 0.18836E+01_JPRB ,&
&  0.20198E+01_JPRB, 0.21285E+01_JPRB, 0.23012E+01_JPRB, 0.20934E+01_JPRB, 0.22367E+01_JPRB, 0.20869E+01_JPRB, 0.17308E+01_JPRB ,&
&  0.15142E+01_JPRB, 0.14467E+01_JPRB, 0.15384E+01_JPRB, 0.16326E+01_JPRB, 0.15091E+01_JPRB, 0.12037E+01_JPRB, 0.12273E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.11627E+01_JPRB, 0.11647E+01_JPRB, 0.11934E+01_JPRB, 0.12415E+01_JPRB, 0.12562E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.12301E+01_JPRB, 0.12275E+01_JPRB, 0.12328E+01_JPRB, 0.12485E+01_JPRB, 0.12918E+01_JPRB, 0.13212E+01_JPRB ,&
&  0.13561E+01_JPRB, 0.14094E+01_JPRB, 0.14784E+01_JPRB, 0.15471E+01_JPRB, 0.15362E+01_JPRB, 0.14590E+01_JPRB, 0.13611E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.13111E+01_JPRB, 0.11900E+01_JPRB, 0.11185E+01_JPRB, 0.10304E+01_JPRB, 0.10047E+01_JPRB, 0.97053E+00_JPRB ,&
&  0.96236E+00_JPRB, 0.95629E+00_JPRB, 0.91362E+00_JPRB, 0.80675E+00_JPRB, 0.80791E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.89842E+00_JPRB, 0.85496E+00_JPRB, 0.84950E+00_JPRB, 0.84515E+00_JPRB, 0.83193E+00_JPRB, 0.82854E+00_JPRB, 0.79200E+00_JPRB ,&
&  0.79281E+00_JPRB, 0.84447E+00_JPRB, 0.91097E+00_JPRB, 0.94707E+00_JPRB, 0.97004E+00_JPRB, 0.97292E+00_JPRB, 0.95320E+00_JPRB ,&
&  0.93248E+00_JPRB, 0.94912E+00_JPRB, 0.96961E+00_JPRB, 0.98250E+00_JPRB, 0.10034E+01_JPRB, 0.10217E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.10473E+01_JPRB, 0.10152E+01_JPRB, 0.10107E+01_JPRB, 0.10850E+01_JPRB, 0.12063E+01_JPRB, 0.13868E+01_JPRB ,&
&  0.21557E+01_JPRB, 0.45619E+01_JPRB, 0.23002E+01_JPRB, 0.21395E+01_JPRB, 0.19883E+01_JPRB, 0.18265E+01_JPRB, 0.16818E+01_JPRB ,&
&  0.17072E+01_JPRB, 0.15112E+01_JPRB, 0.14446E+01_JPRB, 0.15370E+01_JPRB, 0.14995E+01_JPRB, 0.14807E+01_JPRB, 0.14991E+01_JPRB ,&
&  0.15194E+01_JPRB, 0.15330E+01_JPRB, 0.15379E+01_JPRB, 0.15055E+01_JPRB, 0.15366E+01_JPRB, 0.15939E+01_JPRB, 0.16829E+01_JPRB ,&
&  0.17900E+01_JPRB, 0.19315E+01_JPRB, 0.21233E+01_JPRB, 0.20460E+01_JPRB, 0.21374E+01_JPRB, 0.20270E+01_JPRB, 0.17132E+01_JPRB ,&
&  0.15275E+01_JPRB, 0.15923E+01_JPRB, 0.16126E+01_JPRB, 0.16261E+01_JPRB, 0.15151E+01_JPRB, 0.13028E+01_JPRB, 0.13421E+01_JPRB ,&
&  0.12321E+01_JPRB, 0.12017E+01_JPRB, 0.12068E+01_JPRB, 0.12084E+01_JPRB, 0.12314E+01_JPRB, 0.12333E+01_JPRB, 0.12388E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.12107E+01_JPRB, 0.12059E+01_JPRB, 0.12096E+01_JPRB, 0.12161E+01_JPRB, 0.12386E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.13930E+01_JPRB, 0.14752E+01_JPRB, 0.15090E+01_JPRB, 0.15041E+01_JPRB, 0.14284E+01_JPRB, 0.13493E+01_JPRB ,&
&  0.13599E+01_JPRB, 0.14061E+01_JPRB, 0.12516E+01_JPRB, 0.11844E+01_JPRB, 0.11530E+01_JPRB, 0.11034E+01_JPRB, 0.11206E+01_JPRB ,&
&  0.11948E+01_JPRB, 0.13029E+01_JPRB, 0.13814E+01_JPRB, 0.93452E+00_JPRB, 0.92991E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.94664E+00_JPRB, 0.90267E+00_JPRB, 0.90173E+00_JPRB, 0.92681E+00_JPRB, 0.92452E+00_JPRB, 0.90199E+00_JPRB, 0.86158E+00_JPRB ,&
&  0.81415E+00_JPRB, 0.81729E+00_JPRB, 0.83821E+00_JPRB, 0.85797E+00_JPRB, 0.87284E+00_JPRB, 0.88740E+00_JPRB, 0.87766E+00_JPRB ,&
&  0.85734E+00_JPRB, 0.86686E+00_JPRB, 0.88422E+00_JPRB, 0.91395E+00_JPRB, 0.93780E+00_JPRB, 0.96255E+00_JPRB, 0.99519E+00_JPRB ,&
&  0.10051E+01_JPRB, 0.99786E+00_JPRB, 0.98092E+00_JPRB, 0.96989E+00_JPRB, 0.97418E+00_JPRB, 0.10092E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.14741E+01_JPRB, 0.39376E+01_JPRB, 0.37711E+01_JPRB, 0.27177E+01_JPRB, 0.21083E+01_JPRB, 0.18025E+01_JPRB, 0.16711E+01_JPRB ,&
&  0.18168E+01_JPRB, 0.18621E+01_JPRB, 0.17919E+01_JPRB, 0.15605E+01_JPRB, 0.14333E+01_JPRB, 0.13896E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.13781E+01_JPRB, 0.13678E+01_JPRB, 0.13529E+01_JPRB, 0.13481E+01_JPRB, 0.13981E+01_JPRB, 0.14651E+01_JPRB ,&
&  0.15303E+01_JPRB, 0.16622E+01_JPRB, 0.19005E+01_JPRB, 0.21004E+01_JPRB, 0.20555E+01_JPRB, 0.20929E+01_JPRB, 0.20069E+01_JPRB ,&
&  0.20907E+01_JPRB, 0.19912E+01_JPRB, 0.17690E+01_JPRB, 0.15811E+01_JPRB, 0.14133E+01_JPRB, 0.14595E+01_JPRB, 0.13980E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.12267E+01_JPRB, 0.12062E+01_JPRB, 0.12090E+01_JPRB, 0.12154E+01_JPRB, 0.12143E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11667E+01_JPRB, 0.11700E+01_JPRB, 0.11784E+01_JPRB, 0.11824E+01_JPRB, 0.11905E+01_JPRB, 0.12273E+01_JPRB ,&
&  0.12578E+01_JPRB, 0.13094E+01_JPRB, 0.13374E+01_JPRB, 0.14097E+01_JPRB, 0.14087E+01_JPRB, 0.13363E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.13366E+01_JPRB, 0.12956E+01_JPRB, 0.12612E+01_JPRB, 0.12111E+01_JPRB, 0.11432E+01_JPRB, 0.10849E+01_JPRB ,&
&  0.10890E+01_JPRB, 0.10952E+01_JPRB, 0.10442E+01_JPRB, 0.98255E+00_JPRB, 0.99016E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.99778E+00_JPRB, 0.96959E+00_JPRB, 0.95307E+00_JPRB, 0.96329E+00_JPRB, 0.96914E+00_JPRB, 0.95120E+00_JPRB, 0.92450E+00_JPRB ,&
&  0.90057E+00_JPRB, 0.87482E+00_JPRB, 0.88020E+00_JPRB, 0.86736E+00_JPRB, 0.86023E+00_JPRB, 0.85416E+00_JPRB, 0.85170E+00_JPRB ,&
&  0.85628E+00_JPRB, 0.86564E+00_JPRB, 0.85408E+00_JPRB, 0.85704E+00_JPRB, 0.88194E+00_JPRB, 0.90949E+00_JPRB, 0.92271E+00_JPRB ,&
&  0.93452E+00_JPRB, 0.94556E+00_JPRB, 0.94396E+00_JPRB, 0.94340E+00_JPRB, 0.93373E+00_JPRB, 0.94517E+00_JPRB, 0.98328E+00_JPRB ,&
&  0.12170E+01_JPRB, 0.31219E+01_JPRB, 0.22168E+01_JPRB, 0.27361E+01_JPRB, 0.20792E+01_JPRB, 0.17853E+01_JPRB, 0.14973E+01_JPRB ,&
&  0.16197E+01_JPRB, 0.19828E+01_JPRB, 0.17136E+01_JPRB, 0.15377E+01_JPRB, 0.14149E+01_JPRB, 0.13520E+01_JPRB, 0.13077E+01_JPRB ,&
&  0.12802E+01_JPRB, 0.12806E+01_JPRB, 0.12690E+01_JPRB, 0.12682E+01_JPRB, 0.12494E+01_JPRB, 0.12709E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.13340E+01_JPRB, 0.13941E+01_JPRB, 0.15455E+01_JPRB, 0.17165E+01_JPRB, 0.21019E+01_JPRB, 0.23175E+01_JPRB, 0.24020E+01_JPRB ,&
&  0.24747E+01_JPRB, 0.23183E+01_JPRB, 0.19968E+01_JPRB, 0.16852E+01_JPRB, 0.15249E+01_JPRB, 0.14721E+01_JPRB, 0.14556E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.12900E+01_JPRB, 0.12435E+01_JPRB, 0.12190E+01_JPRB, 0.12239E+01_JPRB, 0.12183E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.11438E+01_JPRB, 0.11582E+01_JPRB, 0.11702E+01_JPRB, 0.11620E+01_JPRB, 0.11449E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.12277E+01_JPRB, 0.12571E+01_JPRB, 0.13491E+01_JPRB, 0.13377E+01_JPRB, 0.12972E+01_JPRB, 0.12464E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.12910E+01_JPRB, 0.12775E+01_JPRB, 0.12753E+01_JPRB, 0.12835E+01_JPRB, 0.12531E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10577E+01_JPRB, 0.10423E+01_JPRB, 0.10103E+01_JPRB, 0.10198E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.10020E+01_JPRB, 0.98460E+00_JPRB, 0.99829E+00_JPRB, 0.10075E+01_JPRB, 0.99962E+00_JPRB, 0.96992E+00_JPRB, 0.95071E+00_JPRB ,&
&  0.95773E+00_JPRB, 0.95021E+00_JPRB, 0.93912E+00_JPRB, 0.91161E+00_JPRB, 0.89578E+00_JPRB, 0.87619E+00_JPRB, 0.86526E+00_JPRB ,&
&  0.86870E+00_JPRB, 0.87093E+00_JPRB, 0.86062E+00_JPRB, 0.85857E+00_JPRB, 0.85520E+00_JPRB, 0.86881E+00_JPRB, 0.88331E+00_JPRB ,&
&  0.89715E+00_JPRB, 0.90718E+00_JPRB, 0.92219E+00_JPRB, 0.92919E+00_JPRB, 0.92361E+00_JPRB, 0.91512E+00_JPRB, 0.93502E+00_JPRB ,&
&  0.11263E+01_JPRB, 0.31508E+01_JPRB, 0.18918E+01_JPRB, 0.27236E+01_JPRB, 0.20509E+01_JPRB, 0.17926E+01_JPRB, 0.14880E+01_JPRB ,&
&  0.15873E+01_JPRB, 0.18851E+01_JPRB, 0.16849E+01_JPRB, 0.14734E+01_JPRB, 0.13995E+01_JPRB, 0.13421E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.12398E+01_JPRB, 0.12244E+01_JPRB, 0.12261E+01_JPRB, 0.12230E+01_JPRB, 0.12090E+01_JPRB, 0.12137E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.12306E+01_JPRB, 0.13132E+01_JPRB, 0.16547E+01_JPRB, 0.19177E+01_JPRB, 0.24332E+01_JPRB, 0.29700E+01_JPRB ,&
&  0.29118E+01_JPRB, 0.26641E+01_JPRB, 0.20884E+01_JPRB, 0.18077E+01_JPRB, 0.16393E+01_JPRB, 0.15475E+01_JPRB, 0.14929E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.13931E+01_JPRB, 0.13119E+01_JPRB, 0.12718E+01_JPRB, 0.12625E+01_JPRB, 0.12475E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.11735E+01_JPRB, 0.11732E+01_JPRB, 0.11847E+01_JPRB, 0.11835E+01_JPRB, 0.11702E+01_JPRB, 0.11400E+01_JPRB, 0.11362E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.11935E+01_JPRB, 0.12442E+01_JPRB, 0.13670E+01_JPRB, 0.13476E+01_JPRB, 0.12841E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.12688E+01_JPRB, 0.12699E+01_JPRB, 0.13321E+01_JPRB, 0.13867E+01_JPRB, 0.14428E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.10612E+01_JPRB, 0.10295E+01_JPRB, 0.10123E+01_JPRB, 0.10115E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.10447E+01_JPRB, 0.10349E+01_JPRB, 0.10465E+01_JPRB, 0.10676E+01_JPRB, 0.10486E+01_JPRB, 0.10130E+01_JPRB, 0.98054E+00_JPRB ,&
&  0.99219E+00_JPRB, 0.99105E+00_JPRB, 0.98266E+00_JPRB, 0.94861E+00_JPRB, 0.93107E+00_JPRB, 0.91418E+00_JPRB, 0.90604E+00_JPRB ,&
&  0.90778E+00_JPRB, 0.89644E+00_JPRB, 0.87720E+00_JPRB, 0.86633E+00_JPRB, 0.86559E+00_JPRB, 0.86637E+00_JPRB, 0.88191E+00_JPRB ,&
&  0.87496E+00_JPRB, 0.88141E+00_JPRB, 0.90354E+00_JPRB, 0.91230E+00_JPRB, 0.92017E+00_JPRB, 0.90404E+00_JPRB, 0.91116E+00_JPRB ,&
&  0.10242E+01_JPRB, 0.12583E+01_JPRB, 0.21209E+01_JPRB, 0.24896E+01_JPRB, 0.20535E+01_JPRB, 0.18290E+01_JPRB, 0.16907E+01_JPRB ,&
&  0.17143E+01_JPRB, 0.17412E+01_JPRB, 0.16113E+01_JPRB, 0.14763E+01_JPRB, 0.14001E+01_JPRB, 0.13196E+01_JPRB, 0.12540E+01_JPRB ,&
&  0.11986E+01_JPRB, 0.11705E+01_JPRB, 0.11682E+01_JPRB, 0.11847E+01_JPRB, 0.11904E+01_JPRB, 0.11813E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.11187E+01_JPRB, 0.11463E+01_JPRB, 0.13649E+01_JPRB, 0.18656E+01_JPRB, 0.24901E+01_JPRB, 0.29461E+01_JPRB ,&
&  0.30515E+01_JPRB, 0.26728E+01_JPRB, 0.21003E+01_JPRB, 0.18728E+01_JPRB, 0.17098E+01_JPRB, 0.15935E+01_JPRB, 0.14840E+01_JPRB ,&
&  0.14219E+01_JPRB, 0.14197E+01_JPRB, 0.13909E+01_JPRB, 0.13352E+01_JPRB, 0.12792E+01_JPRB, 0.12487E+01_JPRB, 0.12176E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.12086E+01_JPRB, 0.12139E+01_JPRB, 0.11911E+01_JPRB, 0.11664E+01_JPRB, 0.11390E+01_JPRB, 0.11339E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.12011E+01_JPRB, 0.12839E+01_JPRB, 0.13598E+01_JPRB, 0.13477E+01_JPRB, 0.13094E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.12709E+01_JPRB, 0.12506E+01_JPRB, 0.12465E+01_JPRB, 0.13695E+01_JPRB, 0.14455E+01_JPRB, 0.15267E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.11304E+01_JPRB, 0.10827E+01_JPRB, 0.10612E+01_JPRB, 0.10594E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.11150E+01_JPRB, 0.11097E+01_JPRB, 0.11268E+01_JPRB, 0.11410E+01_JPRB, 0.11225E+01_JPRB, 0.10803E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.10013E+01_JPRB, 0.98216E+00_JPRB, 0.95226E+00_JPRB, 0.94010E+00_JPRB, 0.94114E+00_JPRB, 0.94354E+00_JPRB ,&
&  0.94523E+00_JPRB, 0.93055E+00_JPRB, 0.90900E+00_JPRB, 0.88727E+00_JPRB, 0.88289E+00_JPRB, 0.88504E+00_JPRB, 0.89604E+00_JPRB ,&
&  0.89056E+00_JPRB, 0.88844E+00_JPRB, 0.89391E+00_JPRB, 0.90024E+00_JPRB, 0.90803E+00_JPRB, 0.89896E+00_JPRB, 0.89155E+00_JPRB ,&
&  0.95144E+00_JPRB, 0.15427E+01_JPRB, 0.19061E+01_JPRB, 0.20402E+01_JPRB, 0.19629E+01_JPRB, 0.17997E+01_JPRB, 0.17241E+01_JPRB ,&
&  0.16095E+01_JPRB, 0.15887E+01_JPRB, 0.15596E+01_JPRB, 0.14915E+01_JPRB, 0.14085E+01_JPRB, 0.13299E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.11716E+01_JPRB, 0.11437E+01_JPRB, 0.11400E+01_JPRB, 0.11435E+01_JPRB, 0.11517E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.10682E+01_JPRB, 0.10570E+01_JPRB, 0.11619E+01_JPRB, 0.15808E+01_JPRB, 0.18210E+01_JPRB, 0.24841E+01_JPRB ,&
&  0.27890E+01_JPRB, 0.22102E+01_JPRB, 0.18678E+01_JPRB, 0.17124E+01_JPRB, 0.16027E+01_JPRB, 0.14912E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.13730E+01_JPRB, 0.13605E+01_JPRB, 0.13748E+01_JPRB, 0.13571E+01_JPRB, 0.12933E+01_JPRB, 0.12217E+01_JPRB, 0.11871E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11501E+01_JPRB, 0.11438E+01_JPRB, 0.11394E+01_JPRB, 0.11291E+01_JPRB, 0.10998E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.11501E+01_JPRB, 0.12590E+01_JPRB, 0.12824E+01_JPRB, 0.13126E+01_JPRB, 0.13267E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.12258E+01_JPRB, 0.12291E+01_JPRB, 0.13683E+01_JPRB, 0.16062E+01_JPRB, 0.16742E+01_JPRB, 0.15071E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.12156E+01_JPRB, 0.11514E+01_JPRB, 0.11181E+01_JPRB, 0.11319E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.11784E+01_JPRB, 0.11612E+01_JPRB, 0.11673E+01_JPRB, 0.11724E+01_JPRB, 0.11573E+01_JPRB, 0.11164E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.10585E+01_JPRB, 0.10177E+01_JPRB, 0.97934E+00_JPRB, 0.95313E+00_JPRB, 0.95546E+00_JPRB, 0.96186E+00_JPRB, 0.96243E+00_JPRB ,&
&  0.95096E+00_JPRB, 0.94570E+00_JPRB, 0.92854E+00_JPRB, 0.91360E+00_JPRB, 0.90521E+00_JPRB, 0.89889E+00_JPRB, 0.88508E+00_JPRB ,&
&  0.88533E+00_JPRB, 0.88386E+00_JPRB, 0.88747E+00_JPRB, 0.89524E+00_JPRB, 0.89244E+00_JPRB, 0.89567E+00_JPRB, 0.89195E+00_JPRB ,&
&  0.89915E+00_JPRB, 0.11365E+01_JPRB, 0.13197E+01_JPRB, 0.16199E+01_JPRB, 0.17329E+01_JPRB, 0.16883E+01_JPRB, 0.16168E+01_JPRB ,&
&  0.15256E+01_JPRB, 0.14738E+01_JPRB, 0.14586E+01_JPRB, 0.14191E+01_JPRB, 0.13568E+01_JPRB, 0.12977E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11732E+01_JPRB, 0.11639E+01_JPRB, 0.11465E+01_JPRB, 0.11255E+01_JPRB, 0.11211E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10441E+01_JPRB, 0.10183E+01_JPRB, 0.10266E+01_JPRB, 0.12448E+01_JPRB, 0.14981E+01_JPRB, 0.18478E+01_JPRB ,&
&  0.17741E+01_JPRB, 0.16309E+01_JPRB, 0.15337E+01_JPRB, 0.14578E+01_JPRB, 0.14138E+01_JPRB, 0.13377E+01_JPRB, 0.12793E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.12033E+01_JPRB, 0.11878E+01_JPRB, 0.11835E+01_JPRB, 0.11699E+01_JPRB, 0.11597E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.10776E+01_JPRB, 0.10701E+01_JPRB, 0.10729E+01_JPRB, 0.10831E+01_JPRB, 0.10763E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.11037E+01_JPRB, 0.11652E+01_JPRB, 0.12408E+01_JPRB, 0.12409E+01_JPRB, 0.12153E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.11968E+01_JPRB, 0.12244E+01_JPRB, 0.11918E+01_JPRB, 0.13820E+01_JPRB, 0.15532E+01_JPRB, 0.16342E+01_JPRB, 0.15298E+01_JPRB ,&
&  0.14122E+01_JPRB, 0.12856E+01_JPRB, 0.12122E+01_JPRB, 0.11728E+01_JPRB, 0.11869E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.12347E+01_JPRB, 0.12260E+01_JPRB, 0.12117E+01_JPRB, 0.12021E+01_JPRB, 0.11849E+01_JPRB, 0.11501E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10444E+01_JPRB, 0.99959E+00_JPRB, 0.97495E+00_JPRB, 0.96671E+00_JPRB, 0.96427E+00_JPRB, 0.95775E+00_JPRB ,&
&  0.95080E+00_JPRB, 0.95241E+00_JPRB, 0.94408E+00_JPRB, 0.94187E+00_JPRB, 0.93205E+00_JPRB, 0.92431E+00_JPRB, 0.90158E+00_JPRB ,&
&  0.88575E+00_JPRB, 0.87459E+00_JPRB, 0.87123E+00_JPRB, 0.87776E+00_JPRB, 0.88388E+00_JPRB, 0.88809E+00_JPRB, 0.89510E+00_JPRB ,&
&  0.90975E+00_JPRB, 0.88062E+00_JPRB, 0.99864E+00_JPRB, 0.12012E+01_JPRB, 0.13482E+01_JPRB, 0.14466E+01_JPRB, 0.14225E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.13393E+01_JPRB, 0.13260E+01_JPRB, 0.12923E+01_JPRB, 0.12416E+01_JPRB, 0.11925E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11334E+01_JPRB, 0.11236E+01_JPRB, 0.11137E+01_JPRB, 0.11037E+01_JPRB, 0.10949E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.10845E+01_JPRB, 0.10475E+01_JPRB, 0.10096E+01_JPRB, 0.98440E+00_JPRB, 0.10148E+01_JPRB, 0.11086E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.13687E+01_JPRB, 0.13268E+01_JPRB, 0.12831E+01_JPRB, 0.12448E+01_JPRB, 0.12156E+01_JPRB, 0.12043E+01_JPRB, 0.11885E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.11665E+01_JPRB, 0.11587E+01_JPRB, 0.11467E+01_JPRB, 0.11350E+01_JPRB, 0.11258E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.10921E+01_JPRB, 0.10805E+01_JPRB, 0.10819E+01_JPRB, 0.10792E+01_JPRB, 0.10725E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.10793E+01_JPRB, 0.11131E+01_JPRB, 0.11404E+01_JPRB, 0.11730E+01_JPRB, 0.11769E+01_JPRB, 0.11626E+01_JPRB, 0.11615E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.11467E+01_JPRB, 0.11196E+01_JPRB, 0.12977E+01_JPRB, 0.14356E+01_JPRB, 0.14293E+01_JPRB, 0.14101E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.13579E+01_JPRB, 0.12768E+01_JPRB, 0.11956E+01_JPRB, 0.12083E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.11372E+01_JPRB, 0.11621E+01_JPRB, 0.11501E+01_JPRB, 0.11387E+01_JPRB, 0.11286E+01_JPRB, 0.11099E+01_JPRB, 0.10848E+01_JPRB ,&
&  0.10585E+01_JPRB, 0.10417E+01_JPRB, 0.10205E+01_JPRB, 0.99367E+00_JPRB, 0.97474E+00_JPRB, 0.96209E+00_JPRB, 0.95013E+00_JPRB ,&
&  0.94317E+00_JPRB, 0.93596E+00_JPRB, 0.92990E+00_JPRB, 0.92313E+00_JPRB, 0.91096E+00_JPRB, 0.90017E+00_JPRB, 0.88458E+00_JPRB ,&
&  0.87386E+00_JPRB, 0.87154E+00_JPRB, 0.87113E+00_JPRB, 0.87478E+00_JPRB, 0.88238E+00_JPRB, 0.89127E+00_JPRB, 0.89755E+00_JPRB ,&
&  0.90751E+00_JPRB, 0.81807E+00_JPRB, 0.85825E+00_JPRB, 0.96852E+00_JPRB, 0.11038E+01_JPRB, 0.12238E+01_JPRB, 0.12628E+01_JPRB ,&
&  0.12589E+01_JPRB, 0.12417E+01_JPRB, 0.12138E+01_JPRB, 0.11834E+01_JPRB, 0.11539E+01_JPRB, 0.11328E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.10626E+01_JPRB, 0.10641E+01_JPRB, 0.10615E+01_JPRB, 0.10543E+01_JPRB, 0.10463E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.10385E+01_JPRB, 0.10146E+01_JPRB, 0.98701E+00_JPRB, 0.96737E+00_JPRB, 0.96829E+00_JPRB, 0.10163E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.11545E+01_JPRB, 0.11262E+01_JPRB, 0.10998E+01_JPRB, 0.10890E+01_JPRB, 0.10922E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.10996E+01_JPRB, 0.11109E+01_JPRB, 0.11131E+01_JPRB, 0.11152E+01_JPRB, 0.11128E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.10871E+01_JPRB, 0.10801E+01_JPRB, 0.10756E+01_JPRB, 0.10779E+01_JPRB, 0.10745E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.10661E+01_JPRB, 0.11056E+01_JPRB, 0.11403E+01_JPRB, 0.11390E+01_JPRB, 0.11280E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.11193E+01_JPRB, 0.11148E+01_JPRB, 0.12116E+01_JPRB, 0.12300E+01_JPRB, 0.12412E+01_JPRB, 0.12532E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.12517E+01_JPRB, 0.12323E+01_JPRB, 0.11900E+01_JPRB, 0.10818E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.10360E+01_JPRB, 0.10635E+01_JPRB, 0.10722E+01_JPRB, 0.10678E+01_JPRB, 0.10683E+01_JPRB, 0.10677E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10539E+01_JPRB, 0.10388E+01_JPRB, 0.10150E+01_JPRB, 0.99036E+00_JPRB, 0.96352E+00_JPRB, 0.94155E+00_JPRB ,&
&  0.93224E+00_JPRB, 0.92529E+00_JPRB, 0.90855E+00_JPRB, 0.89321E+00_JPRB, 0.87924E+00_JPRB, 0.86821E+00_JPRB, 0.86342E+00_JPRB ,&
&  0.85799E+00_JPRB, 0.85553E+00_JPRB, 0.86061E+00_JPRB, 0.86761E+00_JPRB, 0.87714E+00_JPRB, 0.88793E+00_JPRB, 0.90237E+00_JPRB ,&
&  0.90833E+00_JPRB, 0.73142E+00_JPRB, 0.76307E+00_JPRB, 0.85651E+00_JPRB, 0.93590E+00_JPRB, 0.10297E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.11634E+01_JPRB, 0.11489E+01_JPRB, 0.11269E+01_JPRB, 0.11181E+01_JPRB, 0.10944E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10150E+01_JPRB, 0.10095E+01_JPRB, 0.10192E+01_JPRB, 0.10250E+01_JPRB, 0.10205E+01_JPRB, 0.10117E+01_JPRB, 0.10075E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.99907E+00_JPRB, 0.99342E+00_JPRB, 0.98508E+00_JPRB, 0.99252E+00_JPRB, 0.10156E+01_JPRB, 0.10414E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10452E+01_JPRB, 0.10368E+01_JPRB, 0.10251E+01_JPRB, 0.10215E+01_JPRB, 0.10339E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10505E+01_JPRB, 0.10579E+01_JPRB, 0.10610E+01_JPRB, 0.10693E+01_JPRB, 0.10781E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.10968E+01_JPRB, 0.10892E+01_JPRB, 0.10755E+01_JPRB, 0.10554E+01_JPRB, 0.10425E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.10343E+01_JPRB, 0.10411E+01_JPRB, 0.10626E+01_JPRB, 0.10877E+01_JPRB, 0.10961E+01_JPRB, 0.10988E+01_JPRB, 0.11022E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.11158E+01_JPRB, 0.11183E+01_JPRB, 0.11306E+01_JPRB, 0.11857E+01_JPRB, 0.11890E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.11159E+01_JPRB, 0.11114E+01_JPRB, 0.93168E+00_JPRB, 0.10105E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.95300E+00_JPRB, 0.97199E+00_JPRB, 0.97313E+00_JPRB, 0.98376E+00_JPRB, 0.99055E+00_JPRB, 0.99562E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.10053E+01_JPRB, 0.99637E+00_JPRB, 0.97941E+00_JPRB, 0.95310E+00_JPRB, 0.92504E+00_JPRB, 0.90170E+00_JPRB ,&
&  0.88419E+00_JPRB, 0.87905E+00_JPRB, 0.86875E+00_JPRB, 0.85647E+00_JPRB, 0.84895E+00_JPRB, 0.84484E+00_JPRB, 0.84654E+00_JPRB ,&
&  0.85410E+00_JPRB, 0.85805E+00_JPRB, 0.85291E+00_JPRB, 0.85154E+00_JPRB, 0.86029E+00_JPRB, 0.87630E+00_JPRB, 0.88377E+00_JPRB ,&
&  0.88578E+00_JPRB, 0.69177E+00_JPRB, 0.73741E+00_JPRB, 0.79307E+00_JPRB, 0.84540E+00_JPRB, 0.91960E+00_JPRB, 0.99706E+00_JPRB ,&
&  0.10472E+01_JPRB, 0.10765E+01_JPRB, 0.10754E+01_JPRB, 0.10622E+01_JPRB, 0.10411E+01_JPRB, 0.10220E+01_JPRB, 0.99856E+00_JPRB ,&
&  0.97730E+00_JPRB, 0.96899E+00_JPRB, 0.97150E+00_JPRB, 0.97536E+00_JPRB, 0.97696E+00_JPRB, 0.98091E+00_JPRB, 0.97531E+00_JPRB ,&
&  0.97355E+00_JPRB, 0.97068E+00_JPRB, 0.96918E+00_JPRB, 0.96950E+00_JPRB, 0.96846E+00_JPRB, 0.96790E+00_JPRB, 0.96623E+00_JPRB ,&
&  0.95544E+00_JPRB, 0.94906E+00_JPRB, 0.94470E+00_JPRB, 0.93987E+00_JPRB, 0.94374E+00_JPRB, 0.95490E+00_JPRB, 0.97195E+00_JPRB ,&
&  0.99168E+00_JPRB, 0.99516E+00_JPRB, 0.10015E+01_JPRB, 0.10097E+01_JPRB, 0.10226E+01_JPRB, 0.10366E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10579E+01_JPRB, 0.10540E+01_JPRB, 0.10435E+01_JPRB, 0.10413E+01_JPRB, 0.10245E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.10016E+01_JPRB, 0.10022E+01_JPRB, 0.10052E+01_JPRB, 0.10185E+01_JPRB, 0.10270E+01_JPRB, 0.10278E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10474E+01_JPRB, 0.10550E+01_JPRB, 0.10497E+01_JPRB, 0.10581E+01_JPRB, 0.10571E+01_JPRB, 0.10592E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10055E+01_JPRB, 0.79747E+00_JPRB, 0.89621E+00_JPRB, 0.92322E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.92502E+00_JPRB, 0.93085E+00_JPRB, 0.93361E+00_JPRB, 0.93757E+00_JPRB, 0.93747E+00_JPRB, 0.93513E+00_JPRB, 0.93866E+00_JPRB ,&
&  0.94256E+00_JPRB, 0.93690E+00_JPRB, 0.92743E+00_JPRB, 0.92348E+00_JPRB, 0.91292E+00_JPRB, 0.89756E+00_JPRB, 0.88456E+00_JPRB ,&
&  0.86856E+00_JPRB, 0.86123E+00_JPRB, 0.85651E+00_JPRB, 0.85187E+00_JPRB, 0.84461E+00_JPRB, 0.83593E+00_JPRB, 0.83919E+00_JPRB ,&
&  0.84588E+00_JPRB, 0.85315E+00_JPRB, 0.85517E+00_JPRB, 0.85918E+00_JPRB, 0.86517E+00_JPRB, 0.87194E+00_JPRB, 0.88408E+00_JPRB ,&
&  0.77240E+00_JPRB, 0.64733E+00_JPRB, 0.70072E+00_JPRB, 0.75611E+00_JPRB, 0.80679E+00_JPRB, 0.88459E+00_JPRB, 0.94328E+00_JPRB ,&
&  0.98803E+00_JPRB, 0.10225E+01_JPRB, 0.10300E+01_JPRB, 0.10067E+01_JPRB, 0.98230E+00_JPRB, 0.97484E+00_JPRB, 0.97449E+00_JPRB ,&
&  0.97041E+00_JPRB, 0.96794E+00_JPRB, 0.97051E+00_JPRB, 0.97402E+00_JPRB, 0.97816E+00_JPRB, 0.98433E+00_JPRB, 0.98678E+00_JPRB ,&
&  0.98489E+00_JPRB, 0.97203E+00_JPRB, 0.95838E+00_JPRB, 0.93744E+00_JPRB, 0.91892E+00_JPRB, 0.91187E+00_JPRB, 0.90289E+00_JPRB ,&
&  0.88802E+00_JPRB, 0.88058E+00_JPRB, 0.88103E+00_JPRB, 0.88524E+00_JPRB, 0.89557E+00_JPRB, 0.90789E+00_JPRB, 0.92678E+00_JPRB ,&
&  0.94855E+00_JPRB, 0.96536E+00_JPRB, 0.97766E+00_JPRB, 0.99144E+00_JPRB, 0.99860E+00_JPRB, 0.10078E+01_JPRB, 0.10250E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.10296E+01_JPRB, 0.10204E+01_JPRB, 0.10112E+01_JPRB, 0.10083E+01_JPRB, 0.10063E+01_JPRB, 0.99683E+00_JPRB ,&
&  0.98783E+00_JPRB, 0.98214E+00_JPRB, 0.97977E+00_JPRB, 0.98520E+00_JPRB, 0.98458E+00_JPRB, 0.97910E+00_JPRB, 0.97678E+00_JPRB ,&
&  0.97501E+00_JPRB, 0.96582E+00_JPRB, 0.95417E+00_JPRB, 0.94952E+00_JPRB, 0.94385E+00_JPRB, 0.94320E+00_JPRB, 0.94017E+00_JPRB ,&
&  0.93704E+00_JPRB, 0.92359E+00_JPRB, 0.92979E+00_JPRB, 0.92353E+00_JPRB, 0.91996E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.89016E+00_JPRB, 0.89402E+00_JPRB, 0.89758E+00_JPRB, 0.89819E+00_JPRB, 0.89454E+00_JPRB, 0.89183E+00_JPRB, 0.89312E+00_JPRB ,&
&  0.89374E+00_JPRB, 0.88532E+00_JPRB, 0.87701E+00_JPRB, 0.87057E+00_JPRB, 0.86762E+00_JPRB, 0.85801E+00_JPRB, 0.84347E+00_JPRB ,&
&  0.83279E+00_JPRB, 0.83075E+00_JPRB, 0.83546E+00_JPRB, 0.83800E+00_JPRB, 0.82909E+00_JPRB, 0.81972E+00_JPRB, 0.81642E+00_JPRB ,&
&  0.81740E+00_JPRB, 0.82354E+00_JPRB, 0.83382E+00_JPRB, 0.83979E+00_JPRB, 0.84847E+00_JPRB, 0.86259E+00_JPRB, 0.86362E+00_JPRB ,&
&  0.86176E+00_JPRB, 0.72471E+00_JPRB, 0.76344E+00_JPRB, 0.77918E+00_JPRB, 0.81060E+00_JPRB, 0.85763E+00_JPRB, 0.89797E+00_JPRB ,&
&  0.93717E+00_JPRB, 0.95943E+00_JPRB, 0.96943E+00_JPRB, 0.96278E+00_JPRB, 0.94683E+00_JPRB, 0.93189E+00_JPRB, 0.92861E+00_JPRB ,&
&  0.92909E+00_JPRB, 0.93705E+00_JPRB, 0.95098E+00_JPRB, 0.96037E+00_JPRB, 0.96363E+00_JPRB, 0.96455E+00_JPRB, 0.96724E+00_JPRB ,&
&  0.96418E+00_JPRB, 0.94762E+00_JPRB, 0.92365E+00_JPRB, 0.89407E+00_JPRB, 0.87021E+00_JPRB, 0.85813E+00_JPRB, 0.84424E+00_JPRB ,&
&  0.82570E+00_JPRB, 0.81493E+00_JPRB, 0.81383E+00_JPRB, 0.81771E+00_JPRB, 0.82803E+00_JPRB, 0.83967E+00_JPRB, 0.84944E+00_JPRB ,&
&  0.86021E+00_JPRB, 0.87694E+00_JPRB, 0.89512E+00_JPRB, 0.91268E+00_JPRB, 0.92994E+00_JPRB, 0.94543E+00_JPRB, 0.95703E+00_JPRB ,&
&  0.96448E+00_JPRB, 0.96235E+00_JPRB, 0.95073E+00_JPRB, 0.94198E+00_JPRB, 0.93892E+00_JPRB, 0.94384E+00_JPRB, 0.94525E+00_JPRB ,&
&  0.93748E+00_JPRB, 0.93095E+00_JPRB, 0.92772E+00_JPRB, 0.91711E+00_JPRB, 0.90533E+00_JPRB, 0.89438E+00_JPRB, 0.88270E+00_JPRB ,&
&  0.87381E+00_JPRB, 0.87035E+00_JPRB, 0.86707E+00_JPRB, 0.86125E+00_JPRB, 0.85319E+00_JPRB, 0.85060E+00_JPRB, 0.84856E+00_JPRB ,&
&  0.84658E+00_JPRB, 0.84413E+00_JPRB, 0.84783E+00_JPRB, 0.86063E+00_JPRB, 0.87844E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.84195E+00_JPRB, 0.84877E+00_JPRB, 0.85861E+00_JPRB, 0.86959E+00_JPRB, 0.87360E+00_JPRB, 0.87323E+00_JPRB, 0.87047E+00_JPRB ,&
&  0.87069E+00_JPRB, 0.86201E+00_JPRB, 0.85547E+00_JPRB, 0.85121E+00_JPRB, 0.84984E+00_JPRB, 0.84561E+00_JPRB, 0.83542E+00_JPRB ,&
&  0.82685E+00_JPRB, 0.82317E+00_JPRB, 0.82719E+00_JPRB, 0.82827E+00_JPRB, 0.81260E+00_JPRB, 0.80733E+00_JPRB, 0.79907E+00_JPRB ,&
&  0.79757E+00_JPRB, 0.79743E+00_JPRB, 0.80015E+00_JPRB, 0.80608E+00_JPRB, 0.82222E+00_JPRB, 0.83364E+00_JPRB, 0.83966E+00_JPRB ,&
&  0.83755E+00_JPRB, 0.76113E+00_JPRB, 0.78910E+00_JPRB, 0.80067E+00_JPRB, 0.82282E+00_JPRB, 0.84572E+00_JPRB, 0.86694E+00_JPRB ,&
&  0.89483E+00_JPRB, 0.91080E+00_JPRB, 0.91792E+00_JPRB, 0.91972E+00_JPRB, 0.92149E+00_JPRB, 0.92107E+00_JPRB, 0.91376E+00_JPRB ,&
&  0.90959E+00_JPRB, 0.91750E+00_JPRB, 0.93533E+00_JPRB, 0.94478E+00_JPRB, 0.94961E+00_JPRB, 0.95226E+00_JPRB, 0.94924E+00_JPRB ,&
&  0.94215E+00_JPRB, 0.92726E+00_JPRB, 0.90210E+00_JPRB, 0.87160E+00_JPRB, 0.83697E+00_JPRB, 0.81240E+00_JPRB, 0.79832E+00_JPRB ,&
&  0.78932E+00_JPRB, 0.78884E+00_JPRB, 0.78967E+00_JPRB, 0.79142E+00_JPRB, 0.79525E+00_JPRB, 0.80447E+00_JPRB, 0.80873E+00_JPRB ,&
&  0.81676E+00_JPRB, 0.82448E+00_JPRB, 0.83669E+00_JPRB, 0.85659E+00_JPRB, 0.87656E+00_JPRB, 0.88794E+00_JPRB, 0.88894E+00_JPRB ,&
&  0.89499E+00_JPRB, 0.89821E+00_JPRB, 0.89488E+00_JPRB, 0.89500E+00_JPRB, 0.89152E+00_JPRB, 0.89324E+00_JPRB, 0.89098E+00_JPRB ,&
&  0.88256E+00_JPRB, 0.87630E+00_JPRB, 0.86843E+00_JPRB, 0.85310E+00_JPRB, 0.83957E+00_JPRB, 0.82960E+00_JPRB, 0.82123E+00_JPRB ,&
&  0.81995E+00_JPRB, 0.82368E+00_JPRB, 0.82909E+00_JPRB, 0.82411E+00_JPRB, 0.81044E+00_JPRB, 0.80251E+00_JPRB, 0.79276E+00_JPRB ,&
&  0.78644E+00_JPRB, 0.78949E+00_JPRB, 0.79103E+00_JPRB, 0.80095E+00_JPRB, 0.82298E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.79087E+00_JPRB, 0.80460E+00_JPRB, 0.82173E+00_JPRB, 0.83898E+00_JPRB, 0.84995E+00_JPRB, 0.85666E+00_JPRB, 0.85452E+00_JPRB ,&
&  0.84330E+00_JPRB, 0.83219E+00_JPRB, 0.83343E+00_JPRB, 0.83293E+00_JPRB, 0.83138E+00_JPRB, 0.82904E+00_JPRB, 0.82185E+00_JPRB ,&
&  0.81177E+00_JPRB, 0.80313E+00_JPRB, 0.79937E+00_JPRB, 0.79609E+00_JPRB, 0.78579E+00_JPRB, 0.77778E+00_JPRB, 0.76913E+00_JPRB ,&
&  0.76856E+00_JPRB, 0.77078E+00_JPRB, 0.77334E+00_JPRB, 0.77325E+00_JPRB, 0.78090E+00_JPRB, 0.79076E+00_JPRB, 0.79487E+00_JPRB ,&
&  0.78898E+00_JPRB, 0.79708E+00_JPRB, 0.71366E+00_JPRB, 0.78303E+00_JPRB, 0.80762E+00_JPRB, 0.82224E+00_JPRB, 0.83808E+00_JPRB ,&
&  0.84915E+00_JPRB, 0.86121E+00_JPRB, 0.87001E+00_JPRB, 0.87540E+00_JPRB, 0.89354E+00_JPRB, 0.90774E+00_JPRB, 0.91085E+00_JPRB ,&
&  0.91237E+00_JPRB, 0.91129E+00_JPRB, 0.91279E+00_JPRB, 0.91369E+00_JPRB, 0.91897E+00_JPRB, 0.91974E+00_JPRB, 0.91434E+00_JPRB ,&
&  0.90660E+00_JPRB, 0.89476E+00_JPRB, 0.87684E+00_JPRB, 0.85578E+00_JPRB, 0.82270E+00_JPRB, 0.79378E+00_JPRB, 0.77521E+00_JPRB ,&
&  0.76006E+00_JPRB, 0.75540E+00_JPRB, 0.74980E+00_JPRB, 0.74421E+00_JPRB, 0.74224E+00_JPRB, 0.74460E+00_JPRB, 0.74376E+00_JPRB ,&
&  0.75046E+00_JPRB, 0.75932E+00_JPRB, 0.76368E+00_JPRB, 0.77134E+00_JPRB, 0.78131E+00_JPRB, 0.79578E+00_JPRB, 0.80950E+00_JPRB ,&
&  0.82245E+00_JPRB, 0.83148E+00_JPRB, 0.83326E+00_JPRB, 0.83373E+00_JPRB, 0.83179E+00_JPRB, 0.83103E+00_JPRB, 0.82422E+00_JPRB ,&
&  0.81876E+00_JPRB, 0.81589E+00_JPRB, 0.80956E+00_JPRB, 0.79753E+00_JPRB, 0.78998E+00_JPRB, 0.78354E+00_JPRB, 0.78179E+00_JPRB ,&
&  0.78248E+00_JPRB, 0.78378E+00_JPRB, 0.78139E+00_JPRB, 0.78225E+00_JPRB, 0.77626E+00_JPRB, 0.75647E+00_JPRB, 0.74116E+00_JPRB ,&
&  0.74096E+00_JPRB, 0.74664E+00_JPRB, 0.75184E+00_JPRB, 0.76138E+00_JPRB, 0.77746E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.80470E+00_JPRB, 0.81162E+00_JPRB, 0.82187E+00_JPRB, 0.83655E+00_JPRB, 0.85160E+00_JPRB, 0.86136E+00_JPRB, 0.86117E+00_JPRB ,&
&  0.85011E+00_JPRB, 0.83656E+00_JPRB, 0.83191E+00_JPRB, 0.83097E+00_JPRB, 0.82774E+00_JPRB, 0.82615E+00_JPRB, 0.82133E+00_JPRB ,&
&  0.80989E+00_JPRB, 0.79264E+00_JPRB, 0.78429E+00_JPRB, 0.77012E+00_JPRB, 0.75661E+00_JPRB, 0.74548E+00_JPRB, 0.74514E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.76334E+00_JPRB, 0.77547E+00_JPRB, 0.78918E+00_JPRB, 0.79629E+00_JPRB, 0.79438E+00_JPRB, 0.79069E+00_JPRB ,&
&  0.78758E+00_JPRB, 0.78460E+00_JPRB, 0.79499E+00_JPRB, 0.80361E+00_JPRB, 0.80930E+00_JPRB, 0.82021E+00_JPRB, 0.82948E+00_JPRB ,&
&  0.83751E+00_JPRB, 0.84906E+00_JPRB, 0.86799E+00_JPRB, 0.88208E+00_JPRB, 0.89255E+00_JPRB, 0.90354E+00_JPRB, 0.90677E+00_JPRB ,&
&  0.91264E+00_JPRB, 0.92382E+00_JPRB, 0.92616E+00_JPRB, 0.91730E+00_JPRB, 0.90326E+00_JPRB, 0.89455E+00_JPRB, 0.88994E+00_JPRB ,&
&  0.88114E+00_JPRB, 0.86195E+00_JPRB, 0.84728E+00_JPRB, 0.83532E+00_JPRB, 0.81586E+00_JPRB, 0.79503E+00_JPRB, 0.77082E+00_JPRB ,&
&  0.74331E+00_JPRB, 0.72276E+00_JPRB, 0.70667E+00_JPRB, 0.69216E+00_JPRB, 0.68841E+00_JPRB, 0.67962E+00_JPRB, 0.67307E+00_JPRB ,&
&  0.67774E+00_JPRB, 0.69629E+00_JPRB, 0.71026E+00_JPRB, 0.71528E+00_JPRB, 0.71807E+00_JPRB, 0.73419E+00_JPRB, 0.75379E+00_JPRB ,&
&  0.76925E+00_JPRB, 0.77598E+00_JPRB, 0.77245E+00_JPRB, 0.77517E+00_JPRB, 0.78060E+00_JPRB, 0.78512E+00_JPRB, 0.77931E+00_JPRB ,&
&  0.77089E+00_JPRB, 0.76544E+00_JPRB, 0.76428E+00_JPRB, 0.76061E+00_JPRB, 0.76199E+00_JPRB, 0.76177E+00_JPRB, 0.75894E+00_JPRB ,&
&  0.75321E+00_JPRB, 0.74398E+00_JPRB, 0.73214E+00_JPRB, 0.72501E+00_JPRB, 0.71455E+00_JPRB, 0.69875E+00_JPRB, 0.69988E+00_JPRB ,&
&  0.71168E+00_JPRB, 0.73029E+00_JPRB, 0.74784E+00_JPRB, 0.76860E+00_JPRB, 0.79297E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.82827E+00_JPRB, 0.83683E+00_JPRB, 0.84321E+00_JPRB, 0.84806E+00_JPRB, 0.85746E+00_JPRB, 0.86905E+00_JPRB, 0.86929E+00_JPRB ,&
&  0.85473E+00_JPRB, 0.84095E+00_JPRB, 0.83121E+00_JPRB, 0.82834E+00_JPRB, 0.82987E+00_JPRB, 0.83200E+00_JPRB, 0.82599E+00_JPRB ,&
&  0.80459E+00_JPRB, 0.78165E+00_JPRB, 0.76738E+00_JPRB, 0.75322E+00_JPRB, 0.74426E+00_JPRB, 0.73843E+00_JPRB, 0.74490E+00_JPRB ,&
&  0.75469E+00_JPRB, 0.77036E+00_JPRB, 0.78305E+00_JPRB, 0.79208E+00_JPRB, 0.79548E+00_JPRB, 0.78358E+00_JPRB, 0.78443E+00_JPRB ,&
&  0.79103E+00_JPRB, 0.79280E+00_JPRB, 0.80744E+00_JPRB, 0.80654E+00_JPRB, 0.79291E+00_JPRB, 0.78183E+00_JPRB, 0.77100E+00_JPRB ,&
&  0.76812E+00_JPRB, 0.77931E+00_JPRB, 0.80461E+00_JPRB, 0.84128E+00_JPRB, 0.86278E+00_JPRB, 0.87106E+00_JPRB, 0.86773E+00_JPRB ,&
&  0.87009E+00_JPRB, 0.88191E+00_JPRB, 0.88865E+00_JPRB, 0.88048E+00_JPRB, 0.86240E+00_JPRB, 0.84777E+00_JPRB, 0.83494E+00_JPRB ,&
&  0.82567E+00_JPRB, 0.81435E+00_JPRB, 0.80656E+00_JPRB, 0.79662E+00_JPRB, 0.77453E+00_JPRB, 0.74829E+00_JPRB, 0.72538E+00_JPRB ,&
&  0.70827E+00_JPRB, 0.69148E+00_JPRB, 0.67104E+00_JPRB, 0.66444E+00_JPRB, 0.66823E+00_JPRB, 0.64522E+00_JPRB, 0.62044E+00_JPRB ,&
&  0.62406E+00_JPRB, 0.65548E+00_JPRB, 0.68015E+00_JPRB, 0.69028E+00_JPRB, 0.69134E+00_JPRB, 0.69591E+00_JPRB, 0.69965E+00_JPRB ,&
&  0.70656E+00_JPRB, 0.71561E+00_JPRB, 0.72262E+00_JPRB, 0.73407E+00_JPRB, 0.74590E+00_JPRB, 0.74452E+00_JPRB, 0.73047E+00_JPRB ,&
&  0.71510E+00_JPRB, 0.70638E+00_JPRB, 0.70277E+00_JPRB, 0.70504E+00_JPRB, 0.70840E+00_JPRB, 0.70636E+00_JPRB, 0.70062E+00_JPRB ,&
&  0.68901E+00_JPRB, 0.67071E+00_JPRB, 0.65512E+00_JPRB, 0.64634E+00_JPRB, 0.64584E+00_JPRB, 0.65248E+00_JPRB, 0.66897E+00_JPRB ,&
&  0.70062E+00_JPRB, 0.74864E+00_JPRB, 0.78429E+00_JPRB, 0.80459E+00_JPRB, 0.81042E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.85495E+00_JPRB, 0.86363E+00_JPRB, 0.87378E+00_JPRB, 0.88740E+00_JPRB, 0.89993E+00_JPRB, 0.89733E+00_JPRB, 0.88498E+00_JPRB ,&
&  0.86135E+00_JPRB, 0.85233E+00_JPRB, 0.85685E+00_JPRB, 0.85915E+00_JPRB, 0.86076E+00_JPRB, 0.85842E+00_JPRB, 0.84212E+00_JPRB ,&
&  0.80431E+00_JPRB, 0.77387E+00_JPRB, 0.75906E+00_JPRB, 0.75118E+00_JPRB, 0.74837E+00_JPRB, 0.75493E+00_JPRB, 0.76282E+00_JPRB ,&
&  0.77916E+00_JPRB, 0.78877E+00_JPRB, 0.80050E+00_JPRB, 0.80965E+00_JPRB, 0.81202E+00_JPRB, 0.81554E+00_JPRB, 0.81620E+00_JPRB ,&
&  0.80165E+00_JPRB, 0.80145E+00_JPRB, 0.80244E+00_JPRB, 0.79231E+00_JPRB, 0.77031E+00_JPRB, 0.74818E+00_JPRB, 0.72120E+00_JPRB ,&
&  0.70318E+00_JPRB, 0.69404E+00_JPRB, 0.71935E+00_JPRB, 0.78110E+00_JPRB, 0.82470E+00_JPRB, 0.83277E+00_JPRB, 0.82454E+00_JPRB ,&
&  0.82616E+00_JPRB, 0.83347E+00_JPRB, 0.83704E+00_JPRB, 0.82567E+00_JPRB, 0.80607E+00_JPRB, 0.78892E+00_JPRB, 0.77264E+00_JPRB ,&
&  0.76742E+00_JPRB, 0.76701E+00_JPRB, 0.74999E+00_JPRB, 0.71797E+00_JPRB, 0.69024E+00_JPRB, 0.67055E+00_JPRB, 0.66012E+00_JPRB ,&
&  0.65369E+00_JPRB, 0.64391E+00_JPRB, 0.63773E+00_JPRB, 0.64236E+00_JPRB, 0.62235E+00_JPRB, 0.60444E+00_JPRB, 0.60832E+00_JPRB ,&
&  0.62481E+00_JPRB, 0.63047E+00_JPRB, 0.63987E+00_JPRB, 0.64317E+00_JPRB, 0.64036E+00_JPRB, 0.63686E+00_JPRB, 0.63832E+00_JPRB ,&
&  0.64732E+00_JPRB, 0.65757E+00_JPRB, 0.66518E+00_JPRB, 0.67033E+00_JPRB, 0.67558E+00_JPRB, 0.66398E+00_JPRB, 0.64300E+00_JPRB ,&
&  0.62564E+00_JPRB, 0.62260E+00_JPRB, 0.62735E+00_JPRB, 0.62687E+00_JPRB, 0.62544E+00_JPRB, 0.62571E+00_JPRB, 0.62457E+00_JPRB ,&
&  0.62196E+00_JPRB, 0.61478E+00_JPRB, 0.60609E+00_JPRB, 0.60626E+00_JPRB, 0.61763E+00_JPRB, 0.64051E+00_JPRB, 0.67598E+00_JPRB ,&
&  0.72846E+00_JPRB, 0.76809E+00_JPRB, 0.80647E+00_JPRB, 0.82256E+00_JPRB, 0.82681E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.88898E+00_JPRB, 0.91124E+00_JPRB, 0.93038E+00_JPRB, 0.94348E+00_JPRB, 0.94559E+00_JPRB, 0.93898E+00_JPRB, 0.92300E+00_JPRB ,&
&  0.89292E+00_JPRB, 0.87576E+00_JPRB, 0.88464E+00_JPRB, 0.89922E+00_JPRB, 0.90803E+00_JPRB, 0.90789E+00_JPRB, 0.87607E+00_JPRB ,&
&  0.82112E+00_JPRB, 0.80002E+00_JPRB, 0.79657E+00_JPRB, 0.80581E+00_JPRB, 0.82363E+00_JPRB, 0.83013E+00_JPRB, 0.81751E+00_JPRB ,&
&  0.80471E+00_JPRB, 0.80584E+00_JPRB, 0.80821E+00_JPRB, 0.81473E+00_JPRB, 0.80380E+00_JPRB, 0.78250E+00_JPRB, 0.77311E+00_JPRB ,&
&  0.77002E+00_JPRB, 0.76182E+00_JPRB, 0.75319E+00_JPRB, 0.70399E+00_JPRB, 0.72662E+00_JPRB, 0.71441E+00_JPRB, 0.69544E+00_JPRB ,&
&  0.67789E+00_JPRB, 0.67997E+00_JPRB, 0.72156E+00_JPRB, 0.76807E+00_JPRB, 0.77955E+00_JPRB, 0.78206E+00_JPRB, 0.77669E+00_JPRB ,&
&  0.77292E+00_JPRB, 0.77330E+00_JPRB, 0.77471E+00_JPRB, 0.76660E+00_JPRB, 0.74630E+00_JPRB, 0.72009E+00_JPRB, 0.69144E+00_JPRB ,&
&  0.67427E+00_JPRB, 0.64970E+00_JPRB, 0.60650E+00_JPRB, 0.58643E+00_JPRB, 0.58470E+00_JPRB, 0.59276E+00_JPRB, 0.59659E+00_JPRB ,&
&  0.59349E+00_JPRB, 0.58595E+00_JPRB, 0.58946E+00_JPRB, 0.58622E+00_JPRB, 0.56438E+00_JPRB, 0.55102E+00_JPRB, 0.54492E+00_JPRB ,&
&  0.53358E+00_JPRB, 0.53282E+00_JPRB, 0.53800E+00_JPRB, 0.55000E+00_JPRB, 0.56053E+00_JPRB, 0.57479E+00_JPRB, 0.58097E+00_JPRB ,&
&  0.58345E+00_JPRB, 0.57990E+00_JPRB, 0.57398E+00_JPRB, 0.56210E+00_JPRB, 0.54377E+00_JPRB, 0.53681E+00_JPRB, 0.53855E+00_JPRB ,&
&  0.54780E+00_JPRB, 0.55420E+00_JPRB, 0.55407E+00_JPRB, 0.55329E+00_JPRB, 0.54199E+00_JPRB, 0.52956E+00_JPRB, 0.52016E+00_JPRB ,&
&  0.51588E+00_JPRB, 0.51902E+00_JPRB, 0.53480E+00_JPRB, 0.56029E+00_JPRB, 0.59272E+00_JPRB, 0.63188E+00_JPRB, 0.67700E+00_JPRB ,&
&  0.70893E+00_JPRB, 0.72770E+00_JPRB, 0.75241E+00_JPRB, 0.80065E+00_JPRB, 0.84746E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.88995E+00_JPRB, 0.91724E+00_JPRB, 0.94694E+00_JPRB, 0.96408E+00_JPRB, 0.96511E+00_JPRB, 0.96203E+00_JPRB, 0.94700E+00_JPRB ,&
&  0.91611E+00_JPRB, 0.89073E+00_JPRB, 0.88647E+00_JPRB, 0.89027E+00_JPRB, 0.90190E+00_JPRB, 0.92038E+00_JPRB, 0.92153E+00_JPRB ,&
&  0.89441E+00_JPRB, 0.87459E+00_JPRB, 0.86464E+00_JPRB, 0.85733E+00_JPRB, 0.84007E+00_JPRB, 0.81511E+00_JPRB, 0.79863E+00_JPRB ,&
&  0.79362E+00_JPRB, 0.79303E+00_JPRB, 0.78367E+00_JPRB, 0.76943E+00_JPRB, 0.73765E+00_JPRB, 0.69854E+00_JPRB, 0.66769E+00_JPRB ,&
&  0.65660E+00_JPRB, 0.64653E+00_JPRB, 0.60580E+00_JPRB, 0.62080E+00_JPRB, 0.62435E+00_JPRB, 0.62423E+00_JPRB, 0.62107E+00_JPRB ,&
&  0.63255E+00_JPRB, 0.66559E+00_JPRB, 0.69857E+00_JPRB, 0.70815E+00_JPRB, 0.71733E+00_JPRB, 0.71885E+00_JPRB, 0.72009E+00_JPRB ,&
&  0.72448E+00_JPRB, 0.71648E+00_JPRB, 0.68689E+00_JPRB, 0.64298E+00_JPRB, 0.59968E+00_JPRB, 0.56602E+00_JPRB, 0.53741E+00_JPRB ,&
&  0.51790E+00_JPRB, 0.50576E+00_JPRB, 0.49623E+00_JPRB, 0.49299E+00_JPRB, 0.50079E+00_JPRB, 0.51594E+00_JPRB, 0.51082E+00_JPRB ,&
&  0.49234E+00_JPRB, 0.47525E+00_JPRB, 0.45286E+00_JPRB, 0.42253E+00_JPRB, 0.39118E+00_JPRB, 0.35312E+00_JPRB, 0.33349E+00_JPRB ,&
&  0.33829E+00_JPRB, 0.37767E+00_JPRB, 0.42925E+00_JPRB, 0.44933E+00_JPRB, 0.47704E+00_JPRB, 0.50409E+00_JPRB, 0.49363E+00_JPRB ,&
&  0.44543E+00_JPRB, 0.37787E+00_JPRB, 0.35529E+00_JPRB, 0.34252E+00_JPRB, 0.33575E+00_JPRB, 0.33122E+00_JPRB, 0.33031E+00_JPRB ,&
&  0.33490E+00_JPRB, 0.34627E+00_JPRB, 0.35618E+00_JPRB, 0.35353E+00_JPRB, 0.34360E+00_JPRB, 0.33803E+00_JPRB, 0.33926E+00_JPRB ,&
&  0.34959E+00_JPRB, 0.37852E+00_JPRB, 0.44206E+00_JPRB, 0.55659E+00_JPRB, 0.59386E+00_JPRB, 0.62766E+00_JPRB, 0.66646E+00_JPRB ,&
&  0.70375E+00_JPRB, 0.73782E+00_JPRB, 0.76683E+00_JPRB, 0.80236E+00_JPRB, 0.84990E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.82631E+00_JPRB, 0.84531E+00_JPRB, 0.86719E+00_JPRB, 0.87886E+00_JPRB, 0.88022E+00_JPRB, 0.87750E+00_JPRB, 0.86900E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.83630E+00_JPRB, 0.82305E+00_JPRB, 0.81399E+00_JPRB, 0.80980E+00_JPRB, 0.81241E+00_JPRB, 0.81734E+00_JPRB ,&
&  0.81607E+00_JPRB, 0.81163E+00_JPRB, 0.79918E+00_JPRB, 0.78308E+00_JPRB, 0.76603E+00_JPRB, 0.75272E+00_JPRB, 0.74235E+00_JPRB ,&
&  0.73215E+00_JPRB, 0.71707E+00_JPRB, 0.69385E+00_JPRB, 0.65819E+00_JPRB, 0.61615E+00_JPRB, 0.57786E+00_JPRB, 0.54785E+00_JPRB ,&
&  0.52103E+00_JPRB, 0.45441E+00_JPRB, 0.41753E+00_JPRB, 0.42806E+00_JPRB, 0.51779E+00_JPRB, 0.54955E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.61003E+00_JPRB, 0.64866E+00_JPRB, 0.66715E+00_JPRB, 0.66736E+00_JPRB, 0.66162E+00_JPRB, 0.64880E+00_JPRB, 0.62894E+00_JPRB ,&
&  0.60382E+00_JPRB, 0.57326E+00_JPRB, 0.54721E+00_JPRB, 0.51792E+00_JPRB, 0.48792E+00_JPRB, 0.46589E+00_JPRB, 0.45060E+00_JPRB ,&
&  0.44006E+00_JPRB, 0.43036E+00_JPRB, 0.42259E+00_JPRB, 0.41277E+00_JPRB, 0.40362E+00_JPRB, 0.38247E+00_JPRB, 0.35995E+00_JPRB ,&
&  0.33729E+00_JPRB, 0.31944E+00_JPRB, 0.30086E+00_JPRB, 0.28258E+00_JPRB, 0.26765E+00_JPRB, 0.25788E+00_JPRB, 0.25060E+00_JPRB ,&
&  0.24855E+00_JPRB, 0.25517E+00_JPRB, 0.27473E+00_JPRB, 0.30768E+00_JPRB, 0.34387E+00_JPRB, 0.31730E+00_JPRB, 0.29760E+00_JPRB ,&
&  0.27640E+00_JPRB, 0.26402E+00_JPRB, 0.25946E+00_JPRB, 0.25898E+00_JPRB, 0.26031E+00_JPRB, 0.26265E+00_JPRB, 0.26515E+00_JPRB ,&
&  0.26872E+00_JPRB, 0.27213E+00_JPRB, 0.27568E+00_JPRB, 0.27842E+00_JPRB, 0.28030E+00_JPRB, 0.28334E+00_JPRB, 0.28834E+00_JPRB ,&
&  0.29614E+00_JPRB, 0.30879E+00_JPRB, 0.32905E+00_JPRB, 0.35525E+00_JPRB, 0.38265E+00_JPRB, 0.40757E+00_JPRB, 0.46086E+00_JPRB ,&
&  0.54843E+00_JPRB, 0.66000E+00_JPRB, 0.71829E+00_JPRB, 0.77553E+00_JPRB, 0.80408E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.76069E+00_JPRB, 0.77360E+00_JPRB, 0.78425E+00_JPRB, 0.78678E+00_JPRB, 0.78632E+00_JPRB, 0.78149E+00_JPRB, 0.76954E+00_JPRB ,&
&  0.74996E+00_JPRB, 0.72374E+00_JPRB, 0.69818E+00_JPRB, 0.67782E+00_JPRB, 0.66500E+00_JPRB, 0.66777E+00_JPRB, 0.67613E+00_JPRB ,&
&  0.69323E+00_JPRB, 0.71846E+00_JPRB, 0.72344E+00_JPRB, 0.71650E+00_JPRB, 0.68048E+00_JPRB, 0.64286E+00_JPRB, 0.60804E+00_JPRB ,&
&  0.57704E+00_JPRB, 0.54752E+00_JPRB, 0.53628E+00_JPRB, 0.51705E+00_JPRB, 0.50359E+00_JPRB, 0.48673E+00_JPRB, 0.46261E+00_JPRB ,&
&  0.42309E+00_JPRB, 0.39257E+00_JPRB, 0.37626E+00_JPRB, 0.38763E+00_JPRB, 0.47414E+00_JPRB, 0.53481E+00_JPRB, 0.54524E+00_JPRB ,&
&  0.57920E+00_JPRB, 0.60317E+00_JPRB, 0.59792E+00_JPRB, 0.57767E+00_JPRB, 0.55588E+00_JPRB, 0.53235E+00_JPRB, 0.51269E+00_JPRB ,&
&  0.49315E+00_JPRB, 0.46674E+00_JPRB, 0.41697E+00_JPRB, 0.34643E+00_JPRB, 0.31117E+00_JPRB, 0.29786E+00_JPRB, 0.28345E+00_JPRB ,&
&  0.27266E+00_JPRB, 0.26432E+00_JPRB, 0.26100E+00_JPRB, 0.25730E+00_JPRB, 0.25664E+00_JPRB, 0.25277E+00_JPRB, 0.24928E+00_JPRB ,&
&  0.24455E+00_JPRB, 0.24105E+00_JPRB, 0.23619E+00_JPRB, 0.23140E+00_JPRB, 0.22769E+00_JPRB, 0.22707E+00_JPRB, 0.22692E+00_JPRB ,&
&  0.22806E+00_JPRB, 0.23225E+00_JPRB, 0.24435E+00_JPRB, 0.25101E+00_JPRB, 0.25157E+00_JPRB, 0.24749E+00_JPRB, 0.24038E+00_JPRB ,&
&  0.23354E+00_JPRB, 0.23188E+00_JPRB, 0.23059E+00_JPRB, 0.23131E+00_JPRB, 0.23432E+00_JPRB, 0.23818E+00_JPRB, 0.24126E+00_JPRB ,&
&  0.24374E+00_JPRB, 0.24641E+00_JPRB, 0.24893E+00_JPRB, 0.25165E+00_JPRB, 0.25498E+00_JPRB, 0.25887E+00_JPRB, 0.26411E+00_JPRB ,&
&  0.26955E+00_JPRB, 0.27716E+00_JPRB, 0.28774E+00_JPRB, 0.30301E+00_JPRB, 0.31673E+00_JPRB, 0.32573E+00_JPRB, 0.33814E+00_JPRB ,&
&  0.35460E+00_JPRB, 0.40255E+00_JPRB, 0.53773E+00_JPRB, 0.76207E+00_JPRB, 0.79428E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.68150E+00_JPRB, 0.69372E+00_JPRB, 0.70831E+00_JPRB, 0.71185E+00_JPRB, 0.72224E+00_JPRB, 0.72126E+00_JPRB, 0.70404E+00_JPRB ,&
&  0.67599E+00_JPRB, 0.64942E+00_JPRB, 0.63498E+00_JPRB, 0.61653E+00_JPRB, 0.58846E+00_JPRB, 0.52846E+00_JPRB, 0.50557E+00_JPRB ,&
&  0.48688E+00_JPRB, 0.48639E+00_JPRB, 0.47691E+00_JPRB, 0.47684E+00_JPRB, 0.46547E+00_JPRB, 0.45382E+00_JPRB, 0.43475E+00_JPRB ,&
&  0.41717E+00_JPRB, 0.40003E+00_JPRB, 0.38806E+00_JPRB, 0.38248E+00_JPRB, 0.38012E+00_JPRB, 0.37517E+00_JPRB, 0.36247E+00_JPRB ,&
&  0.35340E+00_JPRB, 0.35134E+00_JPRB, 0.35353E+00_JPRB, 0.36190E+00_JPRB, 0.40415E+00_JPRB, 0.46052E+00_JPRB, 0.46122E+00_JPRB ,&
&  0.48328E+00_JPRB, 0.49621E+00_JPRB, 0.50891E+00_JPRB, 0.49761E+00_JPRB, 0.46458E+00_JPRB, 0.42487E+00_JPRB, 0.37123E+00_JPRB ,&
&  0.32039E+00_JPRB, 0.29138E+00_JPRB, 0.26783E+00_JPRB, 0.25123E+00_JPRB, 0.23976E+00_JPRB, 0.23520E+00_JPRB, 0.22934E+00_JPRB ,&
&  0.22555E+00_JPRB, 0.22263E+00_JPRB, 0.22148E+00_JPRB, 0.22094E+00_JPRB, 0.22020E+00_JPRB, 0.21805E+00_JPRB, 0.21746E+00_JPRB ,&
&  0.21803E+00_JPRB, 0.21816E+00_JPRB, 0.21790E+00_JPRB, 0.21875E+00_JPRB, 0.21975E+00_JPRB, 0.22123E+00_JPRB, 0.22262E+00_JPRB ,&
&  0.22475E+00_JPRB, 0.22597E+00_JPRB, 0.22786E+00_JPRB, 0.22998E+00_JPRB, 0.22905E+00_JPRB, 0.22800E+00_JPRB, 0.22532E+00_JPRB ,&
&  0.22223E+00_JPRB, 0.22209E+00_JPRB, 0.22170E+00_JPRB, 0.22243E+00_JPRB, 0.22423E+00_JPRB, 0.22660E+00_JPRB, 0.22966E+00_JPRB ,&
&  0.23258E+00_JPRB, 0.23561E+00_JPRB, 0.23799E+00_JPRB, 0.24036E+00_JPRB, 0.24429E+00_JPRB, 0.24842E+00_JPRB, 0.25365E+00_JPRB ,&
&  0.25904E+00_JPRB, 0.26517E+00_JPRB, 0.27351E+00_JPRB, 0.28426E+00_JPRB, 0.29980E+00_JPRB, 0.31717E+00_JPRB, 0.34406E+00_JPRB ,&
&  0.36831E+00_JPRB, 0.46713E+00_JPRB, 0.70281E+00_JPRB, 0.63272E+00_JPRB, 0.65293E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.64323E+00_JPRB, 0.66046E+00_JPRB, 0.68553E+00_JPRB, 0.67858E+00_JPRB, 0.69881E+00_JPRB, 0.68554E+00_JPRB, 0.66261E+00_JPRB ,&
&  0.61541E+00_JPRB, 0.55904E+00_JPRB, 0.51242E+00_JPRB, 0.46044E+00_JPRB, 0.41969E+00_JPRB, 0.39010E+00_JPRB, 0.37551E+00_JPRB ,&
&  0.36803E+00_JPRB, 0.36560E+00_JPRB, 0.36376E+00_JPRB, 0.36261E+00_JPRB, 0.35737E+00_JPRB, 0.35368E+00_JPRB, 0.34759E+00_JPRB ,&
&  0.34153E+00_JPRB, 0.33739E+00_JPRB, 0.33406E+00_JPRB, 0.33892E+00_JPRB, 0.34490E+00_JPRB, 0.35108E+00_JPRB, 0.35412E+00_JPRB ,&
&  0.35651E+00_JPRB, 0.35772E+00_JPRB, 0.36040E+00_JPRB, 0.35826E+00_JPRB, 0.36938E+00_JPRB, 0.39029E+00_JPRB, 0.40781E+00_JPRB ,&
&  0.42487E+00_JPRB, 0.43890E+00_JPRB, 0.43448E+00_JPRB, 0.41031E+00_JPRB, 0.34427E+00_JPRB, 0.31014E+00_JPRB, 0.27990E+00_JPRB ,&
&  0.26072E+00_JPRB, 0.24650E+00_JPRB, 0.23690E+00_JPRB, 0.23144E+00_JPRB, 0.22698E+00_JPRB, 0.22461E+00_JPRB, 0.22209E+00_JPRB ,&
&  0.21968E+00_JPRB, 0.21828E+00_JPRB, 0.21662E+00_JPRB, 0.21579E+00_JPRB, 0.21478E+00_JPRB, 0.21342E+00_JPRB, 0.21249E+00_JPRB ,&
&  0.21274E+00_JPRB, 0.21259E+00_JPRB, 0.21380E+00_JPRB, 0.21482E+00_JPRB, 0.21679E+00_JPRB, 0.21843E+00_JPRB, 0.22074E+00_JPRB ,&
&  0.22271E+00_JPRB, 0.22439E+00_JPRB, 0.22430E+00_JPRB, 0.22393E+00_JPRB, 0.22310E+00_JPRB, 0.22269E+00_JPRB, 0.22192E+00_JPRB ,&
&  0.22085E+00_JPRB, 0.22113E+00_JPRB, 0.22155E+00_JPRB, 0.22203E+00_JPRB, 0.22333E+00_JPRB, 0.22509E+00_JPRB, 0.22767E+00_JPRB ,&
&  0.23106E+00_JPRB, 0.23458E+00_JPRB, 0.23758E+00_JPRB, 0.24081E+00_JPRB, 0.24491E+00_JPRB, 0.24952E+00_JPRB, 0.25520E+00_JPRB ,&
&  0.26008E+00_JPRB, 0.26859E+00_JPRB, 0.28042E+00_JPRB, 0.29280E+00_JPRB, 0.31218E+00_JPRB, 0.33028E+00_JPRB, 0.36049E+00_JPRB ,&
&  0.40878E+00_JPRB, 0.57821E+00_JPRB, 0.86114E+00_JPRB, 0.63776E+00_JPRB, 0.62793E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.56263E+00_JPRB, 0.57219E+00_JPRB, 0.58655E+00_JPRB, 0.59826E+00_JPRB, 0.62238E+00_JPRB, 0.63602E+00_JPRB, 0.68152E+00_JPRB ,&
&  0.63590E+00_JPRB, 0.55730E+00_JPRB, 0.52530E+00_JPRB, 0.49035E+00_JPRB, 0.45063E+00_JPRB, 0.42190E+00_JPRB, 0.40169E+00_JPRB ,&
&  0.38946E+00_JPRB, 0.37898E+00_JPRB, 0.36881E+00_JPRB, 0.35622E+00_JPRB, 0.34330E+00_JPRB, 0.32904E+00_JPRB, 0.31842E+00_JPRB ,&
&  0.31211E+00_JPRB, 0.31052E+00_JPRB, 0.31001E+00_JPRB, 0.31402E+00_JPRB, 0.31953E+00_JPRB, 0.32673E+00_JPRB, 0.33517E+00_JPRB ,&
&  0.35489E+00_JPRB, 0.36172E+00_JPRB, 0.36287E+00_JPRB, 0.36263E+00_JPRB, 0.36347E+00_JPRB, 0.36594E+00_JPRB, 0.35650E+00_JPRB ,&
&  0.36393E+00_JPRB, 0.38410E+00_JPRB, 0.38967E+00_JPRB, 0.37000E+00_JPRB, 0.31537E+00_JPRB, 0.28291E+00_JPRB, 0.26022E+00_JPRB ,&
&  0.25197E+00_JPRB, 0.24200E+00_JPRB, 0.23579E+00_JPRB, 0.22979E+00_JPRB, 0.22572E+00_JPRB, 0.22384E+00_JPRB, 0.22195E+00_JPRB ,&
&  0.22066E+00_JPRB, 0.21983E+00_JPRB, 0.21787E+00_JPRB, 0.21703E+00_JPRB, 0.21588E+00_JPRB, 0.21523E+00_JPRB, 0.21490E+00_JPRB ,&
&  0.21473E+00_JPRB, 0.21441E+00_JPRB, 0.21463E+00_JPRB, 0.21515E+00_JPRB, 0.21608E+00_JPRB, 0.21682E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21845E+00_JPRB, 0.21931E+00_JPRB, 0.21956E+00_JPRB, 0.22013E+00_JPRB, 0.22062E+00_JPRB, 0.22188E+00_JPRB, 0.22250E+00_JPRB ,&
&  0.22314E+00_JPRB, 0.22369E+00_JPRB, 0.22375E+00_JPRB, 0.22445E+00_JPRB, 0.22547E+00_JPRB, 0.22745E+00_JPRB, 0.22969E+00_JPRB ,&
&  0.23255E+00_JPRB, 0.23591E+00_JPRB, 0.23958E+00_JPRB, 0.24325E+00_JPRB, 0.24798E+00_JPRB, 0.25308E+00_JPRB, 0.25978E+00_JPRB ,&
&  0.26704E+00_JPRB, 0.27639E+00_JPRB, 0.28920E+00_JPRB, 0.30463E+00_JPRB, 0.32359E+00_JPRB, 0.34201E+00_JPRB, 0.36227E+00_JPRB ,&
&  0.39672E+00_JPRB, 0.49509E+00_JPRB, 0.55511E+00_JPRB, 0.54454E+00_JPRB, 0.55596E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.54423E+00_JPRB, 0.54696E+00_JPRB, 0.54986E+00_JPRB, 0.55420E+00_JPRB, 0.55888E+00_JPRB, 0.56339E+00_JPRB, 0.56720E+00_JPRB ,&
&  0.54888E+00_JPRB, 0.52578E+00_JPRB, 0.50678E+00_JPRB, 0.48413E+00_JPRB, 0.46346E+00_JPRB, 0.44616E+00_JPRB, 0.42596E+00_JPRB ,&
&  0.40577E+00_JPRB, 0.38560E+00_JPRB, 0.36289E+00_JPRB, 0.34422E+00_JPRB, 0.32585E+00_JPRB, 0.31101E+00_JPRB, 0.30207E+00_JPRB ,&
&  0.29709E+00_JPRB, 0.29566E+00_JPRB, 0.29701E+00_JPRB, 0.29972E+00_JPRB, 0.30231E+00_JPRB, 0.30679E+00_JPRB, 0.31312E+00_JPRB ,&
&  0.32188E+00_JPRB, 0.33665E+00_JPRB, 0.34992E+00_JPRB, 0.35461E+00_JPRB, 0.35081E+00_JPRB, 0.34213E+00_JPRB, 0.33466E+00_JPRB ,&
&  0.32567E+00_JPRB, 0.31351E+00_JPRB, 0.29858E+00_JPRB, 0.28378E+00_JPRB, 0.26998E+00_JPRB, 0.25993E+00_JPRB, 0.25149E+00_JPRB ,&
&  0.24432E+00_JPRB, 0.23799E+00_JPRB, 0.23310E+00_JPRB, 0.22981E+00_JPRB, 0.22700E+00_JPRB, 0.22476E+00_JPRB, 0.22261E+00_JPRB ,&
&  0.22158E+00_JPRB, 0.22053E+00_JPRB, 0.21989E+00_JPRB, 0.21954E+00_JPRB, 0.21904E+00_JPRB, 0.21887E+00_JPRB, 0.21875E+00_JPRB ,&
&  0.21839E+00_JPRB, 0.21775E+00_JPRB, 0.21739E+00_JPRB, 0.21716E+00_JPRB, 0.21762E+00_JPRB, 0.21800E+00_JPRB, 0.21860E+00_JPRB ,&
&  0.21952E+00_JPRB, 0.22139E+00_JPRB, 0.22255E+00_JPRB, 0.22366E+00_JPRB, 0.22425E+00_JPRB, 0.22477E+00_JPRB, 0.22547E+00_JPRB ,&
&  0.22659E+00_JPRB, 0.22792E+00_JPRB, 0.22912E+00_JPRB, 0.23071E+00_JPRB, 0.23235E+00_JPRB, 0.23459E+00_JPRB, 0.23700E+00_JPRB ,&
&  0.23974E+00_JPRB, 0.24313E+00_JPRB, 0.24730E+00_JPRB, 0.25106E+00_JPRB, 0.25693E+00_JPRB, 0.26284E+00_JPRB, 0.26948E+00_JPRB ,&
&  0.27635E+00_JPRB, 0.28379E+00_JPRB, 0.29361E+00_JPRB, 0.30526E+00_JPRB, 0.31997E+00_JPRB, 0.32826E+00_JPRB, 0.33928E+00_JPRB ,&
&  0.35621E+00_JPRB, 0.39156E+00_JPRB, 0.52727E+00_JPRB, 0.54076E+00_JPRB, 0.54556E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.32498E+00_JPRB, 0.33072E+00_JPRB, 0.33469E+00_JPRB, 0.34372E+00_JPRB, 0.35618E+00_JPRB, 0.45346E+00_JPRB, 0.44850E+00_JPRB ,&
&  0.44769E+00_JPRB, 0.40480E+00_JPRB, 0.39687E+00_JPRB, 0.38477E+00_JPRB, 0.36947E+00_JPRB, 0.35055E+00_JPRB, 0.33883E+00_JPRB ,&
&  0.32908E+00_JPRB, 0.31771E+00_JPRB, 0.30706E+00_JPRB, 0.30079E+00_JPRB, 0.29580E+00_JPRB, 0.29137E+00_JPRB, 0.28874E+00_JPRB ,&
&  0.28764E+00_JPRB, 0.28794E+00_JPRB, 0.28857E+00_JPRB, 0.28943E+00_JPRB, 0.29265E+00_JPRB, 0.29621E+00_JPRB, 0.29981E+00_JPRB ,&
&  0.30146E+00_JPRB, 0.30569E+00_JPRB, 0.30654E+00_JPRB, 0.30338E+00_JPRB, 0.29830E+00_JPRB, 0.29103E+00_JPRB, 0.28370E+00_JPRB ,&
&  0.27616E+00_JPRB, 0.26918E+00_JPRB, 0.26340E+00_JPRB, 0.25816E+00_JPRB, 0.25218E+00_JPRB, 0.24650E+00_JPRB, 0.24283E+00_JPRB ,&
&  0.23948E+00_JPRB, 0.23607E+00_JPRB, 0.23336E+00_JPRB, 0.23201E+00_JPRB, 0.23050E+00_JPRB, 0.22949E+00_JPRB, 0.22873E+00_JPRB ,&
&  0.22853E+00_JPRB, 0.22842E+00_JPRB, 0.22850E+00_JPRB, 0.22879E+00_JPRB, 0.22878E+00_JPRB, 0.22871E+00_JPRB, 0.22868E+00_JPRB ,&
&  0.22926E+00_JPRB, 0.22930E+00_JPRB, 0.22924E+00_JPRB, 0.22903E+00_JPRB, 0.22972E+00_JPRB, 0.22996E+00_JPRB, 0.23035E+00_JPRB ,&
&  0.23097E+00_JPRB, 0.23183E+00_JPRB, 0.23249E+00_JPRB, 0.23325E+00_JPRB, 0.23398E+00_JPRB, 0.23487E+00_JPRB, 0.23591E+00_JPRB ,&
&  0.23697E+00_JPRB, 0.23851E+00_JPRB, 0.24010E+00_JPRB, 0.24150E+00_JPRB, 0.24274E+00_JPRB, 0.24443E+00_JPRB, 0.24632E+00_JPRB ,&
&  0.24912E+00_JPRB, 0.25184E+00_JPRB, 0.25462E+00_JPRB, 0.25625E+00_JPRB, 0.25894E+00_JPRB, 0.26119E+00_JPRB, 0.26361E+00_JPRB ,&
&  0.26644E+00_JPRB, 0.27106E+00_JPRB, 0.27781E+00_JPRB, 0.28522E+00_JPRB, 0.29354E+00_JPRB, 0.30041E+00_JPRB, 0.30496E+00_JPRB ,&
&  0.30883E+00_JPRB, 0.31305E+00_JPRB, 0.31853E+00_JPRB, 0.32140E+00_JPRB, 0.32366E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.28983E+00_JPRB, 0.28982E+00_JPRB, 0.28989E+00_JPRB, 0.28954E+00_JPRB, 0.28796E+00_JPRB, 0.28672E+00_JPRB, 0.28560E+00_JPRB ,&
&  0.28499E+00_JPRB, 0.28407E+00_JPRB, 0.28361E+00_JPRB, 0.28338E+00_JPRB, 0.28294E+00_JPRB, 0.28233E+00_JPRB, 0.28151E+00_JPRB ,&
&  0.28044E+00_JPRB, 0.27906E+00_JPRB, 0.27759E+00_JPRB, 0.27724E+00_JPRB, 0.27724E+00_JPRB, 0.27726E+00_JPRB, 0.27721E+00_JPRB ,&
&  0.27704E+00_JPRB, 0.27688E+00_JPRB, 0.27629E+00_JPRB, 0.27635E+00_JPRB, 0.27633E+00_JPRB, 0.27631E+00_JPRB, 0.27640E+00_JPRB ,&
&  0.27659E+00_JPRB, 0.27678E+00_JPRB, 0.27656E+00_JPRB, 0.27610E+00_JPRB, 0.27522E+00_JPRB, 0.27353E+00_JPRB, 0.27087E+00_JPRB ,&
&  0.26785E+00_JPRB, 0.26484E+00_JPRB, 0.26242E+00_JPRB, 0.26019E+00_JPRB, 0.25816E+00_JPRB, 0.25631E+00_JPRB, 0.25477E+00_JPRB ,&
&  0.25293E+00_JPRB, 0.25111E+00_JPRB, 0.24944E+00_JPRB, 0.24824E+00_JPRB, 0.24692E+00_JPRB, 0.24577E+00_JPRB, 0.24471E+00_JPRB ,&
&  0.24398E+00_JPRB, 0.24335E+00_JPRB, 0.24279E+00_JPRB, 0.24251E+00_JPRB, 0.24229E+00_JPRB, 0.24210E+00_JPRB, 0.24187E+00_JPRB ,&
&  0.24167E+00_JPRB, 0.24153E+00_JPRB, 0.24142E+00_JPRB, 0.24141E+00_JPRB, 0.24152E+00_JPRB, 0.24142E+00_JPRB, 0.24130E+00_JPRB ,&
&  0.24127E+00_JPRB, 0.24156E+00_JPRB, 0.24175E+00_JPRB, 0.24207E+00_JPRB, 0.24255E+00_JPRB, 0.24323E+00_JPRB, 0.24397E+00_JPRB ,&
&  0.24471E+00_JPRB, 0.24557E+00_JPRB, 0.24651E+00_JPRB, 0.24765E+00_JPRB, 0.24887E+00_JPRB, 0.25009E+00_JPRB, 0.25126E+00_JPRB ,&
&  0.25251E+00_JPRB, 0.25366E+00_JPRB, 0.25475E+00_JPRB, 0.25551E+00_JPRB, 0.25660E+00_JPRB, 0.25772E+00_JPRB, 0.25903E+00_JPRB ,&
&  0.26052E+00_JPRB, 0.26251E+00_JPRB, 0.26530E+00_JPRB, 0.26863E+00_JPRB, 0.27284E+00_JPRB, 0.27593E+00_JPRB, 0.27907E+00_JPRB ,&
&  0.28131E+00_JPRB, 0.28309E+00_JPRB, 0.28441E+00_JPRB, 0.28608E+00_JPRB, 0.28791E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB, 0.25898E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2000:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2000
